#ifndef __MATRIX__H__
#define __MATRIX__H__

#include <stdio.h>
#include "neunetze.h"

/*                                                                          */
/* Mit der Definition der Direkktive __MTRACE__ kann der Auf- und Abbau     */
/* der Matrizen in der Datei "Matrix.c" verfolgt werden                     */
/*                                                                          */

/*
#ifndef __MTRACE__
#define __MTRACE__
#undef __MTRACE__ 
#endif
*/

/*****************************************************************************/
/*                                                                           */
/*  DATENSTRUKTUR MATRIX (ohne Null-Eintraege)                               */
/*                                                                           */
/*****************************************************************************/

typedef struct matrix_head
{
  struct matrix_elem *HeadPtr;
  struct matrix_head *next;
} MAT_HEAD, *MATRIX_HEAD_PTR;

typedef struct matrix_elem
{
  int    zeile,
         spalte;
  float  element;
  struct matrix_elem   *next;
} MAT_ELEM, *MATRIX_ELEMENT_PTR;


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

MATRIX_HEAD_PTR  Net_To_Matrix (LMPNet);

#else

MATRIX_HEAD_PTR  Net_To_Matrix ();

#endif

/*                                                                           */
/* Parameter                                                                 */
/*                                                                           */
/*  LMPNet               net;                      Das in eine Matrix zu     */
/*                                                 konvertierende Netz       */
/*                                                                           */
/* Konvertieren eines Netzes in die obige Datenstruktur                      */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

void Show_Row (MATRIX_ELEMENT_PTR   ,     /* Zeiger auf Zeile         */
                      int                  ,     /* und Anzahl der Zeilenele-*/
                                                 /* elemnte der Matrix       */
                      FILE *               );    /* Zeiger auf Ausgabedatei  */

#else

void Show_Row ();

#endif

/*                                                                           */
/* Parameter                                                                 */
/*                                                                           */
/*  MATRIX_ELEMENT_PTR   R;                        Zeiger auf Zeile          */
/*  int                  n;                        und Anzahl der Zeilenele- */
/*                                                 elemente der Matrix       */
/*  FILE                 * Datei;                  Zeiger auff Aussgbedatei  */
/*                                                                           */
/*  Ausgabe einer Zeile einer Matrix oder Ausgabe eines Vektors in Zeilenform*/
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

float Element (MATRIX_ELEMENT_PTR R,      /* Zeiger auf Matrixzeile   */
                      int                i);     /* i-te Zeile der Matrix    */

#else

float Element ( );

#endif

/*                                                                           */
/* Parameter                                                                 */
/*                                                                           */
/*  MATRIX_ELEMENT_PTR R;                          Zeiger auf Matrixzeile    */
/*  int                i;                          i-te Zeile der Matrix     */
/*                                                                           */
/*  gibt den Wert des i-ten Elements einer Zeile zurueck                     */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

float element (MATRIX_HEAD_PTR ,          /* Zeiger auf Matrix        */
                      int             ,          /* i-te Zeile               */
                      int             );         /* j-te Spalte              */
#else

float element ( );

#endif

/*                                                                           */
/* Parameter                                                                 */
/*                                                                           */
/*  MATRIX_HEAD_PTR   matrix;                      Zeiger auf Matrix         */
/*  int               i,                           i-te Zeile                */
/*                    j;                           j-te Spalte               */
/*                                                                           */
/*  liefert den Wert des Elements der 'i`-ten Zeile und 'j`-ten Spalte       */
/*                                                                           */
/*****************************************************************************/





/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

MATRIX_HEAD_PTR Change_Value (MATRIX_HEAD_PTR,   /* Matrix            */
                                     int            ,   /* Zeile i           */
                                     int            ,   /* Spalte j          */
                                     float          );  /* neuer Wert        */

#else

MATRIX_HEAD_PTR Change_Value ( );

#endif

/*                                                                           */
/* Parameter                                                                 */
/*                                                                           */
/*  MATRIX_HEAD_PTR    M;                          zu bearbeitende Matrix    */
/*  int                i,                          Zeile i                   */
/*                     j;                          Spalte j                  */
/*  float              a;                          neuer Wert                */
/*                                                                           */
/*  ersetzt den Wert des Elements (i,j) in der Matrix M durch a              */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

MATRIX_ELEMENT_PTR Multiplication (MATRIX_ELEMENT_PTR, /* Zeiger Zeile*/
                                          float             );/*Multiplikator*/
#else

MATRIX_ELEMENT_PTR Multiplication ();

#endif

/*                                                                           */
/* Parameter                                                                 */
/*                                                                           */
/*  MATRIX_ELEMENT_PTR   R;                        Zeiger auf Zeile R        */
/*  float                a;                        Multiplikator             */
/*                                                                           */
/*  Multiplikation einer Zeile R` mit einer Zahl a`                        */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

MATRIX_HEAD_PTR Transponation (MATRIX_HEAD_PTR, /*transponierte Matrix*/
                                      int            );/* Anzahl Spalten     */
#else

MATRIX_HEAD_PTR Transponation ();

#endif

/*                                                                           */
/* Parameter                                                                 */
/*                                                                           */
/*  MATRIX_HEAD_PTR   matrix;                      zu transponierende Matrix */
/*  int               n;                           Anzahl Spalten            */
/*                                                                           */
/*  Bildet die Transponierte einer Matrix          	                     */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

MATRIX_ELEMENT_PTR Add_Rows (MATRIX_ELEMENT_PTR,
                                    MATRIX_ELEMENT_PTR);

#else

MATRIX_ELEMENT_PTR Add_Rows ();

#endif

/*                                                                           */
/* Parameter                                                                 */
/*                                                                           */
/*  MATRIX_ELEMENT_PTR   R1,                         neue, addierte Zeile    */
/*                       R2;                         zu addierende Zeile     */
/*                                                                           */
/*  liefert die Addition zweier Zeilen einer Matrix       	             */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

void Change_Rows (MATRIX_HEAD_PTR,
                         MATRIX_HEAD_PTR);

#else

void Change_Rows ();

#endif

/*                                                                           */
/* Parameter                                                                 */
/*                                                                           */
/*  MATRIX_HEAD_PTR R1,                                                      */
/*                  R2;                                                      */
/*                                                                           */
/*  vertauschen zweier Zeilen einer Matrix                                   */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)
 
MATRIX_ELEMENT_PTR Del_Row (MATRIX_ELEMENT_PTR Row);  /* zu loeschende Zeile */
 
#else
 
MATRIX_ELEMENT_PTR Del_Row ();
  
#endif
 
/*                                                                           */
/*  Loeschen einer im Speicher abgelegten Zeile "Row"                        */
/*                                                                           */
/*****************************************************************************/





/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

MATRIX_ELEMENT_PTR Change_ij (MATRIX_ELEMENT_PTR,  /* Zeile           */
                                     int               ,  /* Zeile i         */
                                     int               ,  /* Zeile j         */
                                     int               ); /* #Zeilenelemente */

#else

MATRIX_ELEMENT_PTR Change_ij ();

#endif

/*                                                                           */
/* Parameter                                                                 */
/*                                                                           */
/*  MATRIX_ELEMENT_PTR   H;                         zu bearbeitende Zeile    */
/*  int                  i,                         Zeile i                  */
/*                       j,                         Zeile j                  */
/*                       n;                         Anzahl Zeilenelemente    */
/*                                                                           */
/*  vertauschen der Spalten i` und j` in einer n`-spaltigen Zeile H`     */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

MATRIX_HEAD_PTR Change_Columns (MATRIX_HEAD_PTR M, /* Zeile          */
                                       int i             , /* Spalte i       */
                                       int j             , /* Spalte j       */
                                       int n             );/* #Zeilenelemente*/

#else

MATRIX_HEAD_PTR Change_Columns ();

#endif

/*                                                                           */
/* Parameter                                                                 */
/*                                                                           */
/*  MATRIX_ELEMENT_PTR   H;                         zu bearbeitende Zeile    */
/*  int                  i,                         Spalte i                 */
/*                       j,                         Spalte j                 */
/*                       n;                         Anzahl Zeilenelemente    */
/*                                                                           */
/*  vertauschen der Spalten 'i` und 'j` in 'n`-spaltiger Matrix 'M`          */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

MATRIX_HEAD_PTR Insert_Column (
                         MATRIX_HEAD_PTR   ,     /* Zu bearbeitende Matrix   */
                         MATRIX_ELEMENT_PTR,     /* Zeiger auf einzufuegenden*/
                                                 /* Spaltenvektor            */
                         int                     /* Spalte,nach der einzufue-*/
                                                 /* gender Vektor eingetragen*/
                                                 /* werden soll              */
    #ifdef __MTRACE__
                                            ,
                         FILE *            
    #endif
                                            );

#else

MATRIX_HEAD_PTR Insert_Column ();

#endif

/*                                                                           */
/* Parameter                                                                 */
/*                                                                           */
/*  MATRIX_HEAD_PTR      M;                        Zu bearbeitende Matrix    */
/*  MATRIX_ELEMENT_PTR   C;                        Zeiger auf einzufuegenden */
/*                                                 Spaltenvektor             */
/*  int                  n;                        Spalte, nach der einzufue-*/
/*                                                 gender Vektor eingetragen */
/*                                                 werden soll               */
/*  FILE               * Datei;                                              */
/*                                                                           */
/*  Einfuegen einer Spalte 'C` in die Matrix 'M` nach Spalte 'n`             */
/*                                                                           */
/*****************************************************************************/


/******************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

MATRIX_HEAD_PTR Del_Matrix (MATRIX_HEAD_PTR);/* zu loeschende Matrix  */

#else

MATRIX_HEAD_PTR Del_Matrix ( );

#endif

/*                                                                           */
/* Parameter                                                                 */
/*                                                                           */
/*  MATRIX_HEAD_PTR     Matrix;                    zu loeschende Matrix      */
/*                                                                           */
/*  Loeschen einer im Speicher abgelegten Matrix                             */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

MATRIX_ELEMENT_PTR Convert_Marking (MARKING);/*Zeiger Markierungsliste*/

#else

MATRIX_ELEMENT_PTR Convert_Marking ( );

#endif

/*                                                                           */
/* Parameter                                                                 */
/*                                                                           */
/*  MARKING            Marke;                      Zeiger Markierungsliste   */
/*                                                                           */
/*  Konvertiert eine Markierungsliste Marke` in einen Vektor                */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

void Print_Matrix_Ptr (MATRIX_HEAD_PTR,
                              int            ,
                              FILE *         );

#else

void Print_Matrix_Ptr ();

#endif

/*                                                                           */
/* Parameter                                                                 */
/*                                                                           */
/*   MATRIX_HEAD_PTR   M;                                                    */
/*   int               n;                                                    */
/*   FILE              *Datei;                                               */
/*                                                                           */
/*  Gibt die in Zeigerform gespeicherte Matrix auf dem Bildschirm aus        */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

MATRIX_HEAD_PTR Make_Zerro (
                            MATRIX_HEAD_PTR   ,   /* zu bearbeitende Matrix  */
                            MATRIX_ELEMENT_PTR,   /* i-te Zeile              */
                            int               );  /* j-te Spalte             */
#else

MATRIX_HEAD_PTR Make_Zerro ();

#endif

/*                                                                           */
/* Parameter                                                                 */
/*                                                                           */
/*   MATRIX_HEAD_PTR       M;                                                */
/*   MATRIX_ELEMENT_PTR    R;                       i-te Zeile               */
/*   int                   j;                       j-te Spalte              */
/*                                                                           */
/*  Aendert eine Matrix in eine obere Dreiecks-Matrix   	             */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

MATRIX_HEAD_PTR Gauss_Elimination (MATRIX_HEAD_PTR, /* Matrix         */
                                          int            );/* #Spalten Matrix*/

#else

MATRIX_HEAD_PTR Gauss_Elimination ();

#endif

/*                                                                           */
/* Parameter                                                                 */
/*                                                                           */
/* MATRIX_HEAD_PTR M;                               bearbeitende Matrix      */
/* int             n;                               #Spalten der Matrix      */
/*                                                                           */
/*  Wendet Gauss-Seidel Algorithmus auf n`-spaltige Inzidenz-Matrix M` an  */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

int Rang (MATRIX_HEAD_PTR   ,            /* Matrix, von der der Rang  */ 
                                                /* berechnet werden soll     */
                 int               ,            /* Anzahl der Spalten        */
                 bool              );           /* Gauss_elimination noetig? */
      
#else

int Rang ();

#endif

/*                                                                           */
/* Parameter                                                                 */
/*                                                                           */
/* MATRIX_HEAD_PTR    mat;                        Matrix, von der der Rang   */
/*                                                berechnet werden soll      */
/* int                tn;                         Anzahl der Spalten         */
/* bool               gauss;                      Gauss_elimination noetig?  */
/*                                                                           */
/*                                                                           */
/*  berechnet den Rang einer Matrix             	                     */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
/*                                                                           */
/*           Funktionen zur Bestimmung, ob eine Markierung M von             */
/*          der Startmarkierung M0 aus erreichbarr ist oder nicht            */
/*                                                                           */
/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

int Decide (LMPNet       net,                  /* zu bearbeitendes netz     */
            MARKING      Marke,                 /* erreichbare Markierung    */
            MARKING      Mark0                  /* Anfangsmarkierung         */

   #if defined(__MTRACE__)                      /* TRACE Matrixbibliothek    */
                               ,
            FILE       * Datei
   #endif                                       /* END __MTRACE__ DEFINITION */
	    );
#else

int Decide (net, Mark0, Marke

   #if defined(__MTRACE__)                      /* TRACEn Matrixbibliothek   */
                               ,
                               Datei
   #endif                                       /* END __MTRACE__ DEFINITION */
	    );

LMPNet       net;                              /* zu bearbeitendes netz     */
MARKING      Marke,                             /* erreichbare Markierung    */
             Mark0;                             /* Anfangsmarkierung         */

   #if defined(__MTRACE__)                      /* TRACEn Matrixbibliothek   */

   FILE    * Datei;

   #endif                                       /* END __MTRACE__ DEFINITION */

#endif                                          /* END STDC & C++ DEFINITION */

/*****************************************************************************/

#endif
