/******************************************************************************/
/***       Praktikum Werkzeuge zur Analyse von Petri-Netzen, SS 1992        ***/
/***       C-Quelldatei fuer Programmteil 'Test auf Erreichbarkeit'         ***/
/***                                        Juli 1992, Stefan Roemer        ***/
/******************************************************************************/
/***    Version 2.0 ( neue Datenstruktur )    Stefan Roemer, 10.05.1993     ***/
/******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <limits.h>
#include "erreich.h"
#include "vector.h"
#include "nbound.h"

/******************************************************************************/
/*                       T Y P D E F I N I T I O N E N                        */
/******************************************************************************/

/* Datentyp fuer die Darstellung der Liste Markierung Transition Markierung   */

typedef struct Reachelem_
{
  struct Reachelem_      *succ,
                  *prev;
  Vektor          altmark;
  char            *Transition;
  Vektor          neumark;
  bool            reach;
} Reach, *ReachList;



/*****************************************************************************/
/******************************************************************************/
/*                                                                            */
void search_reach_mark (Datei, UDGbegin, UDGend, Initmark, Reachmark, Sfeld, n)
/*                                                                            */
/******************************************************************************/

FILE            * Datei;                         /* Zeiger auf Zieldatei      */
Reach           * UDGbegin,                      /* Zeiger auf Beginn und Ende*/
                * UDGend;                        /* der Erreichbarkeitsliste  */
Vektor          Initmark,                        /* Anfangsmarkierung         */
                Reachmark;                       /* Erreichbare Markierung    */
Place           * Sfeld;                         /* Feld von Zeigern auf die  */
                                                 /* Stellen                   */
VCardinal       n;                               /* Anzahl Stellen            */

{
  ReachList       AktPtr,
                  reachPtr = NULL,
                  loeschPtr = NULL,
                  LastPtr,
                  FirstPtr;

  Vektor          InitVek,
                  ReachVek;
  bool            reachmarkreached = True,
                  loesch           = True;

  InitVek  = NeuerVektor(n);
  ReachVek = NeuerVektor(n);

  FirstPtr = UDGbegin;
  LastPtr  = UDGend;

  InitVek  = Initmark;
  ReachVek = Reachmark;

  /* Zeiger aufs Listenende */
  AktPtr = LastPtr;

  while (AktPtr)
  {
    if (!VektorGleich (ReachVek, AktPtr->neumark))
    {
      if (loesch) 
        loeschPtr = AktPtr;
      /* Erreichbare Markierung in der Liste bei Folgemarkierungen suchen */
      AktPtr = AktPtr->prev;
      if (loesch && AktPtr)
      {
        AktPtr->succ = NULL;
        LastPtr = AktPtr;
        Free (loeschPtr->neumark);
        Free (loeschPtr->altmark);
        Free (loeschPtr);
      } 
    }
    else
    {
      AktPtr->reach = reachmarkreached;
      /* Wenn Folgemarkierung gefunden */
      if (AktPtr->reach)
      {
        if (AktPtr->prev)
        { /* neuen Endezeiger setzen */
          reachPtr = AktPtr->prev;
        }
        reachmarkreached = !reachmarkreached;
        loesch           = !loesch;
      }
      /* Graphen ausgeben */
      /* SchreibeMarkierung (Datei, ReachVek, Sfeld, True); */
      if (!Datei)
        printf ("%s, ", AktPtr->Transition);
      else
        fprintf (Datei, "%s, ", AktPtr->Transition);
      ReachVek = AktPtr->altmark;

      /* Anfangsmarkierung erreicht */
      if (VektorGleich (InitVek, AktPtr->altmark))
      {
        /* SchreibeMarkierung (Datei, InitVek, Sfeld, True); */
        if (!Datei)
          printf ("\nStartmarkierung\n\n\n");
        else
          /* fprintf (Datei, "\nStartmarkierung\n\n\n"); */
        /* naechsten Pfad ausgeben   */
        if (reachPtr)
        {
          search_reach_mark(Datei, FirstPtr, reachPtr,
                            InitVek, Reachmark, Sfeld, n);
        }
      }
    }
  } /* while (AktPtr) */
  Free (InitVek);
  Free (ReachVek);
} /*** search_reach_mark() ***/


/******************************************************************************/
/*                                                                            */
    void Erreichbarkeitspfad (Net, mark, DateiName)
/*                                                                            */
/*  Prozedur zur Berechnung des Ueberdeckungsgraphen eines Systems fuer die   */
/*  Berechnung der Erreichbarkeitspfade einer Markierung 'mark' eines System  */
/*  'net'.                                                                    */
/*                                                                            */
/*  Es wird eine Datei mit dem Dateinamen und der Endung '.EP' angelegt, in   */
/*  die die errechneten Erreichbarkeitspfade geschrieben werden.              */
/*  Entspricht ComputeReach (teilw.)                                          */
/******************************************************************************/

LMPNet          Net;                             /* zu untersuchendes Netz    */
MARKING         mark;                            /* zu erreichende Markierung */
char            * DateiName;                     /* Datei fuer Erreichbar-    */
                                                 /* keitspfad                 */
{
KnotenMenge     VE,                              /* Markierungsknotenliste    */
                v,                               /* Markierungsknoten         */
                vQuer,                           /* aktuelle Markierung       */
                vStrichPtr;
ReachList       Neu,
                AktPtr,
                LastPtr,
                FirstPtr;
Place           Stelle,
                * Stellenfeld;
Transition      tr;
VCardinal       Stellenzahl,
                Stellennummer;
Vektor          NV,
                vStrich,
                InitVektor,
                HilfsVektor,
                HilfsPtr,
                SchaltVektor,
                ReachVektor;                     /* Vektor fuer zu erreichen- */
                                                 /* de Markierung             */
VorgaengerListe VorgPtr;                         /* Vorgaengerliste einer     */
                                                 /* Markierung                */

MARKING         hilfmark;                        /* Zeiger auf zu erreichende */
                                                 /* Markierung                */
bool            EPschreiben = False;
FILE            * Datei=NULL;

/* RR */
FILE *reverse_fire_file,
     *firing_file;
char enabled_transition[512],
     so_far[512],
     c;
/* end of RR */

if (*DateiName) /* Datei fuer Erreichbarkeitsgraphen oeffnen */
{
   if ((Datei = fopen (DateiName, "w")))
   {
      /*fprintf (Datei, "* ERREICHBARKEITSPFAD(e)");
      fprintf (Datei, "  des SYSTEMS '%s':\n\n", DateiName); */
      EPschreiben = True;
   }
   else
   {
      printf ("Datei '%s' wird nicht angelegt !\n", DateiName);
   }
}

Stellenzahl = Net->NumPlaces; /* Ermittle Anzahl der Stellen im Netz */
Stellenfeld = (Place *) Malloc (Stellenzahl * sizeof (Place));
StellenBenennen (Net, Stellenfeld);

/* Initialisiere Vektoren    */
NV           = NeuerVektor (Stellenzahl);
vStrich      = NeuerVektor (Stellenzahl);
InitVektor   = NeuerVektor (Stellenzahl);
SchaltVektor = NeuerVektor (Stellenzahl);
ReachVektor  = NeuerVektor (Stellenzahl);

/* Kopiere die Anfangsmarkierung in Hilfsvektor  */
for(Stelle = Net->Places, HilfsPtr = InitVektor;
     Stelle; Stelle = Stelle->nextplace)
{
    if (Stelle->marks)
    {
        *(HilfsPtr++) = Stelle->numname;
        *(HilfsPtr++) = Stelle->marks;
    }
}
*HilfsPtr = 0;

VE = (KnotenMenge) Malloc (sizeof (Knoten));     /* Initialisiere Knotenmenge */
VE->Markierung = ErzeugeVektor (InitVektor);     /* Einrichten der Liste der  */
VE->next = NULL;                                 /* Markierungen mit Start-   */
VE->PfadVorg = NULL;                             /* markierung als Wurzel     */

HilfsVektor = InitVektor;

/* Kopiere die zu erreichende Markierung in EPvektor */
hilfmark = mark;
HilfsPtr = ReachVektor;
Stellennummer = 1;
while (hilfmark)
   {
   if (hilfmark->value > 0)
      {
      *(HilfsPtr++) = Stellennummer;
      *(HilfsPtr++) = hilfmark->value;
      }
   hilfmark = hilfmark->next;
   Stellennummer++;
   }
*HilfsPtr = 0;


FirstPtr = LastPtr = AktPtr = NULL;
                                                 /* Initialiseirungen fuer    */
tr = Net->Transitions;                           /* Zeiger  Transitionsliste  */
v = VE;                                          /* Zeiger auf akt. Markierung*/
                                                 /* Hauptschleife des Algo-   */
while (v)                                        /* rithmus                   */
{
/* ab hier gleich */
   VektorAusSTListe (SchaltVektor, tr->preset);

   if (VektorKleiner(SchaltVektor, v->Markierung) || /* WENN ==> v aktiviert t*/
        VektorGleich(SchaltVektor, v->Markierung))
   {                                          /* DANN bilde Folgemar-      */
      Kopiere (vStrich, v->Markierung);          /* kierung: v' := v + N(t)   */
      Subtrahiere (vStrich, SchaltVektor);
      VektorAusTSListe (SchaltVektor, tr->postset);
      Addiere (vStrich, SchaltVektor, HilfsVektor);

      *NV = 0;                                   /* NV := (0,0,...,0)         */
                                                 /* Teste, ob und an welchen  */
      vQuer = VE;                                /* Stellen v'  v oder einen  */
      while (vQuer)                              /* Vorgaenger von v echt     */
      {                                       /* ueberdeckt.               */
         vQuer->besucht = False;
         vQuer = vQuer->next;
      }
      UntersuchePfad (VE, v, vStrich, NV, HilfsVektor);
      vStrichPtr = VE;
      /* v' in VE suchen; falls nicht da: neu einfuegen */
      if (! VektorGleich(vStrichPtr->Markierung, vStrich))
      {
         while (vStrichPtr->next &&              /* Falls v<> v              */
               ! VektorGleich(vStrichPtr->next->Markierung, vStrich))
         {
            vStrichPtr = vStrichPtr->next;       /* naechstes v               */
         }
         if (vStrichPtr->next)
         {
            vStrichPtr = vStrichPtr->next;
         }
         else                                    /* ANDERNFALLS neuen Knoten  */
         {                                    /* fuer v' einrichten        */
            vStrichPtr = vStrichPtr->next
        	        = (KnotenMenge) Malloc (sizeof (Knoten));
            vStrichPtr->Markierung = ErzeugeVektor (vStrich);
            vStrichPtr->PfadVorg = NULL;
            vStrichPtr->next = NULL;
         }
      }
/* bis hier gleich */

      if (EPschreiben)                            /* neue Kante v --t--> v'   */
      {                                        /* in Liste einfuegen       */
         Neu =(ReachList) Malloc (sizeof (Reach));
         Neu->succ = NULL;
         Neu->altmark = v->Markierung;
         Neu->Transition = tr->name;
         Neu->neumark = vStrichPtr->Markierung;
         Neu->reach = False;
         if (AktPtr == NULL)
         {
            AktPtr = Neu;
            AktPtr->prev = NULL;
            FirstPtr = AktPtr;
         }
         else
         {
            AktPtr->succ = Neu;
            Neu -> prev = AktPtr;
            AktPtr = Neu;
         }
         LastPtr = AktPtr;
      }
/* Ab hier gleich */
      if ((VorgPtr = vStrichPtr->PfadVorg))       /* Pfad-Vorgaengerliste von */
      {                                        /* v' aktualisieren         */
         if (! VektorGleich(VorgPtr->KnotenPtr->Markierung, v->Markierung))
         {
            while (VorgPtr->next &&
                   !VektorGleich(VorgPtr->next->KnotenPtr->Markierung,
				 v->Markierung)       )
            {
               VorgPtr = VorgPtr->next;
            }
         }
         if (! VorgPtr->next)
         {
            VorgPtr = VorgPtr->next
                    = (VorgaengerListe) Malloc(sizeof (Vorgaenger));
            VorgPtr->next = NULL;
            VorgPtr->KnotenPtr = v;
         }
      }
      else
      {
         VorgPtr = vStrichPtr->PfadVorg
                 = (VorgaengerListe) Malloc(sizeof (Vorgaenger));
         VorgPtr->next = NULL;
         VorgPtr->KnotenPtr = v;
      }
   } /* IF VektorKleiner || VektorGleich  */

/* Naechste Transition bzw. naechste Markierung untersuchen */
   if ( ! (tr = tr->nexttrans))
   {
      tr = Net->Transitions;
      v = v->next;
   }
} /*    END grosse while       */
/* bis hier gleich */

search_reach_mark (Datei, FirstPtr, LastPtr, InitVektor, ReachVektor,
                   Stellenfeld, Stellenzahl);

if (EPschreiben)                                 /* ... und  aufraeumen:      */
  fclose (Datei);

/* Markierungsvektoren aus dem Speicher entfernen */
Free (NV);
Free (vStrich);
Free (SchaltVektor);
Free (InitVektor);
Free (ReachVektor);
Free (Stellenfeld);
Free (HilfsVektor);

/* UDG aus Speicher loeschen */
RemoveUDG(VE);

/* RR: Drehe die berechneten Schaltfolgen in firing.res um. */
if ( (reverse_fire_file = fopen( DateiName, "r") ) == NULL) 
  {
    /* keine Schaltfolge berechnet */ 
    fprintf(stderr, "ERROR: firing sequence-file %s cannot be opened for reading\n", DateiName);
    exit( 16 );
  } /* end of if */
else
  {
    if (  (firing_file = fopen("erreichbar.res", "w") ) == NULL )
      {
       fprintf(stderr, "ERROR: Outputfile can not be opened for writing\n");
       exit( 15 );
      } /* end of if */
    else
       {
        strcpy(enabled_transition,"");
        while ( ( c = fgetc(reverse_fire_file) ) != EOF )
	     {
	       if ( (c!=',') && (c!=' ') )
	         {
	         strncat(enabled_transition, &c, 1);
	         } /* end of if */
               else
	         {
		   fscanf(firing_file, "%s", so_far);
		   strcpy(enabled_transition, so_far);
		   fprintf(firing_file, "%s", enabled_transition);
	         } /* end of else */
	     } /* end of while */
	     fprintf(firing_file, "\n");
        } /* end of else */
  } /* end of else */
  fclose(reverse_fire_file);
  fclose(firing_file);
/* end of RR */

} /*** Ueberdeckungsgraph() ***/


/******************************************************************************/
/* END OF FILE 'erreich.c'                                                    */
/******************************************************************************/
