/*****************************************************************************/
/*                                                                           */
/*   PRAKTIKUM ANALYSE DER PETRI-NETZE [ ERREICHBARKEITS-ANALYSE ]   SS 1993 */
/*   VERBESSERTE VERSION MIT DER NEUEN DATENSTRUKTUR FUER MATRIXOPERATIONEN  */
/*                                                                           */
/* FARZAD EGHTESSADI                                                         */
/* BEHJAT SAMANGHOON                                                         */
/*****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include "convertPEP.h"
#include "readlib.h"
#include "Matrix.h"
#include "rev.h"
#include "erreich.h"

/*******************************************/
/* Get places not marked by given marking. */
/*******************************************/
static LIST_POINTER Unmarked_Places (LMPNet net, MARKING mark)
{
  Place        pl;
  LIST_POINTER UnmPlace = NULL, /* Liste unmarkierter Stellen */
               Begin = NULL;

  for(pl=net->Places; pl; pl=pl->nextplace, mark=mark->next)
  {
    if (!mark->value)
    {
      UnmPlace = (LIST_POINTER) MYmalloc(sizeof (LIST_ELEMENT));
      strcpy (UnmPlace->name, pl->name);
      UnmPlace->next = Begin;
      Begin = UnmPlace;
    }
  }
  return (Begin);
}

/*************************/
/* Erreichbarkeitsanalye */
/*************************/
int Reachability (net, mark, mark0)

LMPNet       net;
MARKING      mark,  /* zu erreichende Markierung */
             mark0; /* Startmarkierung           */

{
  LIST_POINTER UnmPlace= NULL;                  /* Liste unmarkierter Stellen*/
  LIST_POINTER MaxTrap = NULL;                  /* Liste der maximalen Falle */
  int          Reach   = 0;                     /* Rueckgabewert Erreichbar? */

  if ((Reach = Decide (net, mark, mark0)) > 0)
    {
#ifdef DEBUG1
      printf("\nREACHY: Das LGS  'M = Mo + C*X' ist loesbar.....\n");
#endif
      UnmPlace = Unmarked_Places (net, mark);
      MaxTrap  = Berechne_maximale_Falle (net, UnmPlace);

      if (MaxTrap == NULL)
	Reach = 1;
      else
	Reach = 0;
    }
  return (Reach);
}                                          /***   ENDE von Reachability   ***/

/****************************************************************************/

int main( int argc, char **argv )
{

  FILE    * outfile,
          * markfile;
  LMPNet  net;
  Place   pl;
  MARKING marke,
          marke0,
          hilf;
  int     num,
          reachable;
  char    ch,
          name[101],
          filename[200];

  if ( argc < 4 )
  {
    fprintf(stderr,
             "ERROR: Usage: %s <Netfile> <Outputfile> <Markingfile>\n", *argv);
    exit ( 13 );
  }

  strcpy(filename, argv[1]);

  if ( !( markfile = fopen( argv[3], "r" ) ) ) 
  {
    fprintf( stderr,
	     "ERROR: Markingfile %s cannot be opened for reading!\n", argv[3]);
    exit ( 14 );
  } /* if */

  if (( net = read_PEP( filename ) ) )
  {
#ifdef DEBUG1
        show_matrix( net );
#endif
    NNinput_file = argv[3];
    NNinput_line = 1;

    for(pl = net->Places; pl; pl = pl->nextplace)
    {
      pl->numname = 0;
    }

    num = 1;
    if ( isdigit( ch = ReadWhiteSpace( markfile ) ) )
    {
      ungetc( ch, markfile );
      num = ReadNumber( markfile );
    }
    else
    {
      ungetc( ch, markfile );
    }
    *name = '\0';
    ReadName( markfile, name, 100 );
    if ( ! ( pl = find_P( net, name ) ) )
    {
      fprintf( stderr, "ERROR: place '%s' is not in the net", name );
      exit( 11 );
    }
    pl->numname += num;

    while ( ! feof( markfile ) )
    {
      if ( TestSymbol( markfile, ',' ) )
      {
        num = 1;
        if ( isdigit( ch = ReadWhiteSpace( markfile ) ) )
        {
          ungetc( ch, markfile );
          num = ReadNumber( markfile );
        }
        else
        {
          ungetc( ch, markfile );
        }
        *name = '\0';
        ReadName( markfile, name, 100 );
        if ( ! ( pl = find_P( net, name ) ) )
        {
          fprintf( stdout, "ERROR: place '%s' is not in the net", name );
          exit( 11 );
        }
        pl->numname += (Cardinal32) num;
      } /* if Testsymbol */
      else if ( ! feof( markfile ) )
      {
        fprintf( stdout, "ERROR: missing comma");
        exit( 12 );
      }
    } /* while !feof */

    marke0 = copy_start_marking( net );   
    marke  = copy_start_marking( net );
    for(hilf = marke, pl = net->Places; hilf;
        hilf = hilf->next, pl = pl->nextplace)
    {
      hilf->value = (int) pl->numname;
    }

    if ( ! ( outfile = fopen( argv[2], "w" ) ) )
    {
      fprintf( stderr,
	       "Warning: Outputfile %s could not be opened for writing!\n", 
               argv[2] );
      outfile=stdout;
    }

    reachable = Reachability( net, marke, marke0 ); 
    if ( reachable == 1 ) 
    {
      fprintf( outfile, "MARKING IS REACHABLE\n");  
      /* Ergebnisschaltfolge(n) werden in rev_reach.res geschrieben */
      Erreichbarkeitspfad( net, marke, "rev_reach.res" );
    }
    else if ( reachable == -1 )
    {
      fprintf( outfile, "REACHABILITY IS NOT EFFICIENTLY DECIDABLE\n" );
    }
    else
    {
      fprintf( outfile, "MARKING IS NOT REACHABLE\n");  
    }
    fclose( outfile );
  } /* if */

  remove_net( &net );
  fclose( markfile );
  exit ( 0 );
} /* main() */
 
