/**********************************************************************/
/* PRAKTIKUM ANALYSE DER PETRI-NETZE [ LEBENDIGKEITS-ANALYSE ] SS 93  */
/* VERBESSERTE VERSION MIT DER NEUEN DATENSTRUKTUR FUER               */
/* MATRIXOPERATIONEN                                                  */
/*                                                                    */
/* FARZAD EGHTESSADI                                                  */
/* BEHJAT SAMANGHOON                                                  */   
/**********************************************************************/
/* Aenderungen: Stefan Roemer, 10.03.1994                             */
/**********************************************************************/

#include <stdio.h>
#include <string.h>
#include "leb.h"
#include "rw_inf.h"
#include "Matrix.h" 
#include "mat_cplex.h"

extern void free_matrix();
extern void convert_matrix();

/**********************************************************************/
/* FUNKTION Delete_Marked_Places()                                    */
/* Loeschen der anfangs markierten Stellen   FARZAD E.                */
/**********************************************************************/

void Delete_Marked_Places(LMPNet net)
  {

    Place PlacePtr,
          Help;
    int   deleted;

    PlacePtr = net->Places; 
    while( PlacePtr )
      {
        if ( PlacePtr->marks > 0 ) 
          {
            Help = PlacePtr;
            PlacePtr = PlacePtr->nextplace;
            delete_place( net, Help->name ); 
          } 
        else
          {
            PlacePtr = PlacePtr->nextplace; 
          }
      } 
    return( net ); 
  }


/*********************************************************************/
/* FUNKTION InOut_Shared()                                           */
/* Erstellt eine Liste der IS bzw OS stellen im Netz  FARZAD E.      */
/*********************************************************************/

LIST_POINTER InOut_Shared(LMPNet net)
  {
    LIST_POINTER IO_List,ListPtr,Help;
    Place        pl;
    int ok=0;

    IO_List = (LIST_POINTER) MYmalloc(sizeof(LIST_ELEMENT));
    ListPtr = IO_List;
    Help=ListPtr;
    ListPtr->next = NULL;

    pl = net->Places;
    while ( pl )
      {
        if (    ( pl->preset  && pl->preset->nexttrans )
             || ( pl->postset && pl->postset->nexttrans ) )
          {
            strcpy(ListPtr->name, pl->name);
            /*printf("InOut_Shared_Name >  %s\n",ListPtr->name);*/
            ListPtr->next = (LIST_POINTER) malloc(sizeof(LIST_ELEMENT));
            Help= ListPtr;
            ListPtr=ListPtr->next;
            ListPtr->next = NULL;
            ok=1;
          }
        pl = pl->nextplace;
      }
      
    free(ListPtr);
    if ( ok == 1 ) 
      { 
        Help->next = NULL;
        return(IO_List);
      }
    else 
      {
        return(NULL);
      }
  }


/****************************************************************************/
/* FUNKTION Out_Shared()                                                    */
/* Erstellt eine Liste der OutP-Shared Stellen im Netz  FARZAD E.           */
/****************************************************************************/

LIST_POINTER Out_Shared(LMPNet net) 
  {

    LIST_POINTER  OS_List,ListPtr,Help;
    Place         PlacePtr;
    int  ok=0; 

    OS_List = (LIST_POINTER) malloc(sizeof(LIST_ELEMENT));
    ListPtr = OS_List;
    Help=ListPtr;
    ListPtr->next = NULL;

    PlacePtr = net->Places;

    while (PlacePtr != NULL)
      {
        if (place_successor_number( net, PlacePtr->name) > 1)
          {
            strcpy(ListPtr->name, PlacePtr->name);

            /*printf("Out_Shared_Name >  %s\n",ListPtr->name); */

            ListPtr->next = (LIST_POINTER) 
                     MYmalloc(sizeof(LIST_ELEMENT));
            Help= ListPtr;
            ListPtr=ListPtr->next;
            ListPtr->next = NULL;

            ok=1;
         }

         PlacePtr = PlacePtr->nextplace;
      }
      
      free(ListPtr);
      if (ok==1) 
      { 
         Help->next = NULL;  
         return(OS_List);
      }
      else 
      {
         return(NULL);
      }

}


/**********************************************************************/
/* FUNKTION Expand_Net() Expansion des Netzes         FARZAD E.       */
/**********************************************************************/

LMPNet Expand_Net(LMPNet net) 
  {
    LIST_POINTER ListPtr,List;
    char         TPname[50], TDname[50];     
    char         Pname[50]; 
    char         Pname1[50], Pname2[50];  
    Place        DelPlace; 
    STEdge       PostPtr;
    TSEdge       PrevPtr; 

    ListPtr = List = InOut_Shared(net); 
    while ( ListPtr )      
      {              
        strcpy(TPname,"tp");  
        strcat(TPname,ListPtr->name);
        insert_transition(net, TPname);
        DelPlace = find_P(net,ListPtr->name);
        PostPtr = DelPlace->postset;
        while ( PostPtr )       
        /* Die neuen Verbindungen im Nachbereich */
          {   
            strcpy(Pname, DelPlace->name);               
            strcat(Pname, PostPtr->trans->name);
            insert_place(net, Pname, 0);
            insert_TP_con(net, Pname, TPname, 1);  
            insert_PT_con(net, Pname, PostPtr->trans->name, 1);  
            PostPtr = PostPtr->nexttrans;
          } 
        PrevPtr = DelPlace->preset;
        while ( PrevPtr )       
        /* Die neuen Verbindungen im Vorbereich */ 
          {
            strcpy(Pname1,PrevPtr->trans->name);
            strcat(Pname1, DelPlace->name);
            insert_place(net, Pname1, 0);
            insert_TP_con(net, Pname1, PrevPtr->trans->name, 1);  
            insert_PT_con(net, Pname1, TPname, 1);
            PrevPtr = PrevPtr->nexttrans; 
            if ( PrevPtr )
              {
                strcpy(Pname2,PrevPtr->trans->name);
                strcat(Pname2, DelPlace->name);
                insert_place(net, Pname2,0);
                insert_PT_con(net, Pname2,TPname, 1);
                insert_TP_con(net, Pname2, PrevPtr->trans->name, 1); 
                /* Die Transitionen zwischen den Vorbereichsstellen */
                strcpy(TDname,"td");         
                strcat(TDname,Pname1);            
                insert_transition(net,TDname);
                insert_PT_con(net, Pname1, TDname, 1);
                insert_TP_con(net, Pname2, TDname, 1);
                PrevPtr = PrevPtr->nexttrans;
              }
          }    
        ListPtr = ListPtr->next;
      }
         
    ListPtr = List;
    while ( ListPtr )
      {
        delete_place(net,ListPtr->name);
        ListPtr=ListPtr->next;
      }

    if ( List )
      remove_list(&List);

    return(net); 
  }


/***********************************************************************/
/* FUNKTION Is_SC_Deadlock_Marked()                                    */
/* Ob jeder stark zusammenh. Deadlock markiert ist ?                   */
/* FARZAD E.                                                           */
/***********************************************************************/

int Is_SC_Deadlock_Marked(net)

LMPNet net;

{

LMPNet newnet;   
int stat, sol_stat;
double sol_solu;

      write_matrix(net,"lebendig.netcopy");
      write_netmarking(net,"lebendig.netcopy");
      newnet = praktikum_ini("lebendig.netcopy");
#ifdef DEBUG1
printf("Eingang von Teil 1:\n");
show_matrix( newnet );
#endif

      Delete_Marked_Places(newnet);
      Expand_Net(newnet);
      
      convert_matrix( newnet );
      stat = calc_simplex( &sol_stat, &sol_solu );
      free_matrix();
#ifdef DEBUG1
printf("Ausgang von Teil 1:\n");
show_matrix( newnet );
#endif
      remove_net(&newnet);
#ifdef DEBUG1
printf("Teil 1: stat = %d, sol_stat = %d, sol_solu = %6.2f\n", stat, sol_stat, sol_solu);
#endif
      if ( stat == 0 && sol_stat == 1 && sol_solu == 0.0 )
        {
#ifdef DEBUG1
printf("Teil 1: 1 (kann lebendig sein)\n");
#endif
          return ( 1 );
        } /* if */
      else
      if ( stat == 0 && sol_stat == 1 && sol_solu > 0.0 )
        {
#ifdef DEBUG1
printf("Teil 1: 1 (kann nicht lebendig sein)\n");
#endif
          return ( 0 );
        } /* if */
      else
      if ( stat == 0 && sol_stat == 2 )
        {
#ifdef DEBUG1
printf("Teil 1: 1 (kann lebendig sein)\n");
#endif
          /* return ( 1 ); */
        } /* if */
      else
      if ( stat == 0 && sol_stat == 3 )
        {
#ifdef DEBUG1
printf("Teil 1: 1 (kann nicht lebendig sein)\n");
#endif
          return ( 0 );
        } /* if */
      else
      if ( stat == 0 &&  sol_stat > 3 )
        {
printf("ERROR: sol_stat > 3 Teil 1: 1 (kann nicht lebendig sein)\n");
          return ( 0 );
        } /* if */
      else
      if ( stat == 1 )
        {
printf("ERROR: keine Loesung Teil 1: 1 (kann nicht lebendig sein)\n");
          return ( 0 );
        } /* if */
      else
        {
printf("ERROR: komischer Fall Teil 1: 0 (ist nicht lebendig)\n");
          return ( 0 );
        } /* else */
}

/**************************************************************************/
/* FUNKTION Is_SC_Deadlock_Trap()                                         */
/* Ob jeder stark zusammenh. Deadlock eine Falle ist ?                    */
/* FARZAD E.                                                              */
/**************************************************************************/

int Is_SC_Deadlock_Trap( net )

  LMPNet net;

  {

    LMPNet       newnet;      
    LIST_POINTER ListPtr,OutPList;
    Transition   TransPtr,NewTransPtr;
    Place        OSPlace;
    STEdge       OSPostPtr;
    TSEdge       NewTPostPtr,Help;
    char         t_p_Name[50];
    int          stat,
                 sol_stat;
    double       sol_solu;

    ListPtr = OutPList = Out_Shared( net );
    while ( ListPtr )
      {
        OSPlace = find_P( net, ListPtr->name );
        OSPostPtr = OSPlace->postset;
        strcpy( t_p_Name, OSPlace->preset->trans->name );
        strcat( t_p_Name, OSPlace->name );
        /*printf( "\n(t',p):= %s\n\n", t_p_Name );*/
        while( OSPostPtr )
          {
            TransPtr = OSPostPtr->trans;
#ifdef DEBUG1
printf("Transition Teil2 = %s.\n", TransPtr->name);
#endif
            /*printf("t_name := %s\n",TransPtr->name);*/
            write_matrix( net, "lebendig.netcopy" );
            write_netmarking( net, "lebendig.netcopy" );
            newnet = praktikum_ini("lebendig.netcopy" );
            /*show_matrix( newnet );*/
#ifdef DEBUG1
show_matrix( newnet );
#endif
            NewTransPtr = find_T( newnet, TransPtr->name );
            NewTPostPtr = NewTransPtr->postset;
            /* Alle Stellen aus dem Nachbereich von NewTransPtr loeschen */
            while( NewTPostPtr )
              {
                NewTPostPtr = ( Help = NewTPostPtr ) -> nextplace;
                delete_place( newnet, Help->place->name );
              }
            Expand_Net( newnet );
            /* printf("\nDas Netz wird jetzt expandiert..........\n\n"); */
            /* show_matrix( newnet ); */
            convert_matrix( newnet );

#ifdef DEBUG1
printf( "Stelle '%s': lower bound := 1.0\n",
        NewTransPtr->preset->place->preset->trans->preset->place->name );
printf("Das expandierte Netz:\n");
show_matrix( newnet );
#endif

            objx[NewTransPtr->preset->place->preset->trans->preset->place
                                                               ->numname] = 0.0;
            bdl[NewTransPtr->preset->place->preset->trans->preset->place
                                                               ->numname] = 1.0;
            stat = calc_simplex( &sol_stat, &sol_solu );
            free_matrix();
            remove_net( &newnet );


printf("Teil 2: stat = %d, sol_stat = %d, sol_solu = %6.2f\n", stat, sol_stat, sol_solu);
      if ( stat == 0 && sol_stat == 1 && sol_solu == 0.0 )
        {
#ifdef DEBUG1
printf("Teil 2: 1 (kann lebendig sein)\n");
#endif
          /* return ( 1 ); */
        } /* if */
      else
      if ( stat == 0 && sol_stat == 1 && sol_solu > 0.0 )
        {
#ifdef DEBUG1
printf("Teil 2: 1 (kann nicht lebendig sein)\n");
#endif
          return ( 0 );
        } /* if */
      else
      if ( stat == 0 &&  sol_stat == 2 )
        {
#ifdef DEBUG1
printf("Teil 2: 1 (kann lebendig sein)\n");
#endif
          /* return ( 1 ); */
        } /* if */
      else
      if ( stat == 0 && sol_stat == 3 )
        {
#ifdef DEBUG1
printf("Teil 2: 1 (kann nicht lebendig sein)\n");
#endif
          return ( 0 );
        } /* if */
      else
      if ( stat == 0 &&  sol_stat > 3 )
        {
printf("ERROR: sol_stat > 3 Teil 2: 1 (kann nicht lebendig sein)\n");
          return ( 0 );
        } /* if */
      else
      if ( stat == 1 )
        {
printf("ERROR: keine Loesung Teil 2: 1 (kann nicht lebendig sein)\n");
          return ( 0 );
        } /* if */
      else
        {
printf("ERROR: komischer Fall Teil 2: 0 (ist nicht lebendig)\n");
          return ( 0 );
        } /* else */



            OSPostPtr = OSPostPtr->nexttrans;
          }
        ListPtr = ListPtr->next;
      }
    if ( OutPList ) 
      {
        /*printf("\nNO OUTPUT-SHARED .... [1]\n\n");*/
#ifdef DEBUG1
printf("Teil 2: kann lebendig sein (no output_shared left)\n");
#endif
        return( 1 );
      }
             
#ifdef DEBUG1
printf("Teil 2: kann lebendig sein\n");
#endif
    return( 1 ); 

  }

/***********************************************************************/
/* FUNKTION Liveness()                                                 */
/* Entscheidet ob das eingegebene Netz lebendig ist   FARZAD E.        */
/***********************************************************************/

bool lebendig(net)

  LMPNet net;   

  {

    Place PlacePtr;

    if ( True /* beschraenkt(net) */)
      {

        PlacePtr = net->Places;
   
        while(PlacePtr != NULL)
          {
            if (PlacePtr->preset == NULL)
              {
#ifdef DEBUG1
                printf("Die Stelle [ %s ] besitzt keine Transitionen im",
                                                                PlacePtr->name);
                printf(" Vorbereich\n\n");
                printf("---------------------------------------------------\n");
#endif
                printf("ERROR: KEIN STARK ZUSAMMENHAENGENDES NETZ .......!!!\n\n");
                return(False);
              }
            PlacePtr=PlacePtr->nextplace;   
          }

        if ((Is_SC_Deadlock_Marked(net) == 1) && (Is_SC_Deadlock_Trap(net)== 1))
          {
/*
            printf("Das untersuchte System IST LEBENDIG!!!\n");
*/
            return( True );
          }
        else 
          {
/*
            printf("Das untersuchte System ist NICHT LEBENDIG!!!\n");
*/
            return( False );
          }

      }

  }

/* ENDE DER LEBENDIGKEITSANALYSE */

