/*****************************************************************************/
/***      Praktikum Werkzeuge zur Analyse von Petri-Netzen, SS 1992        ***/
/***      Header-Datei fuer Programmteil 'Test auf Beschraenktheit'        ***/
/*****************************************************************************/
/***   Version 2.0 ( neue Datenstruktur )    Stefan Roemer, 10.05.1993     ***/
/*****************************************************************************/


#ifndef __BOUND_H__
#define __BOUND_H__


#include "neunetze.h"
#include "vector.h"

/******************************************************************************/
/*                       T Y P D E F I N I T I O N E N                        */
/******************************************************************************/

/* Datentyp fuer die Darstellung der Liste der Pfadvorgaenger einer Markierung*/

typedef struct KnotenZelle     * KnotenMenge;

typedef struct VorgaengerZelle * VorgaengerListe;

typedef struct VorgaengerZelle
  {
    KnotenMenge     KnotenPtr;
    VorgaengerListe next;
  } Vorgaenger;


/* Datentyp 'Lineare Liste' fuer die Darstellung der Knotenmenge              */
/*                          ( Menge der Markierungsvektoren )                 */

typedef struct KnotenZelle
  { 
    Vektor          Markierung;         /* Knoten = Markierung.               */
    KnotenMenge     next;               /* Verweis auf naechsten Knoten.      */
    bool            besucht;            /* Flag; zeigt an, ob dieser Knoten   */
        				/* bei der Suche nach den Pfadvor-    */
        				/* gaengern bereits untersucht wurde. */
    VorgaengerListe PfadVorg;           /* Liste der Knoten, die eine Kante   */
        			        /* zu dieser Markierung besitzen.     */
  } Knoten;



/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++                            Globale Variablen                          ++*/
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

extern bool BOUND_statistics;
/* BG */
extern bool BOUND_progress;
extern Cardinal32 AnzahlKanten;
extern Cardinal32 AnzahlMarkierungen;
/* BG */


/******************************************************************************/
/*       Freigabe des vom Ueberdeckungsgraphen besetzten Speicherplatzes      */
/******************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void RemoveUDG( KnotenMenge KnotenPtr );

#else

void RemoveUDG( KnotenPtr );

#endif 

#if defined(__STDC__) || defined(__cplusplus)

void StellenBenennen( LMPNet System, Place * Stellenfeld );

#else

void StellenBenennen( System, Stellenfeld );

#endif
 
#if defined(__STDC__) || defined(__cplusplus)

void UntersuchePfad( KnotenMenge VE, KnotenMenge vQuer, Vektor vStrich, Vektor NV, Vektor HilfsVektor );

#else

void UntersuchePfad( VE, vQuer, vStrich, NV, HilfsVektor );

#endif

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++ Unterprogramm zur Generierung des Ueberdeckungsgraphens eines Systems ++*/
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#if defined(__STDC__) || defined(__cplusplus)

extern bool Ueberdeckungsgraph_n( LMPNet, char *, Cardinal32 );

#else

extern bool Ueberdeckungsgraph_n();

#endif 

/* bool Ueberdeckungsgraph_n( System, DateiName, Schranke )                  */
/*   LMPNet System;                                                          */
/*   char   * DateiName;                                                     */
/*   Cardinal32     Schranke;                                                */
/* Diese Funktion liefert True, falls das zu untersuchende System beschraenkt*/
/* ist, und False, falls nicht. Ausserdem wird eine Datei mit dem Dateinamen */
/* `DateiName` ( falls dieser ungleich "" ist ) angelegt, in die der zugehoe-*/
/* rige Ueberdeckungsgraph geschrieben wird.                                 */


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*+++      Hauptprogramm fuer Programmteil 'Test auf Beschraenktheit'     +++*/
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#if defined(__STDC__) || defined(__cplusplus)

extern bool n_bounded( LMPNet, Cardinal32, char * );

#else

extern bool n_bounded();

#endif

/* bool beschraenkt( System, Schranke )                                      */
/*   LMPNet         System;                                                  */
/*   Cardinal32     Schranke;                                                */
/* Diese Funktion liefert True, falls das zu untersuchende System beschraenkt*/
/* ist, und False, falls nicht. Ausserdem wird eine Datei 'TEST.UDG'angelegt,*/
/* in der der zugehoerige Ueberdeckungsgraph enthalten ist.                  */

/*****************************************************************************/


#endif  /* __BOUND_H__ */


/*****************************************************************************/
/* END OF FILE bound.h                                                       */
/*****************************************************************************/

