/*****************************************************************************/
/***       Praktikum Werkzeuge zur Analyse von Petri-Netzen, SS 1992       ***/
/***       C-Quelldatei fuer Programmteil 'Test auf Reversibilitaet'       ***/
/***           Konstantin Lazaridis, Lutz Lindau, Stefan Roemer            ***/
/***                               Version 1.3                             ***/
/*****************************************************************************/
/***   Version 2.0 ( neue Datenstruktur )        Stefan Roemer, 10.05.1993 ***/
/*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "rev.h"


/*****************************************************************************/
/***                     T Y P D E F I N I T I O N E N                     ***/
/***       Version 1.0           Stefan Roemer        Fr.,22.05.1992       ***/
/*****************************************************************************/
/*                                                                           */
/*    Datentyp fuer die interne Repraesentation einer Transition             */
/*    Diese Definition ist maschinenabhaengig !                              */
/*    Auf der SUN ist sizeof(unsigned short int) == 2                        */
/*    ( also sind bis zu 65535 Transitionen moeglich )                       */
/*    Wer mehr benoetigt, benutze z.B. 'unsigned int' => 32 Bits             */
/*    Bei einer Aenderung muessen dann aber auch die '%hu'-                  */
/*    Formatstrings in den (f/s)printf/sscanf-Anweisungen im                 */
/*    Programm an den neuen Typ angepasst werden !                           */
/*                                                                           */
/*****************************************************************************/

typedef unsigned short int TCardinal;

/* Datentyp 'Dynamisches Feld' zur Darstellung einer Menge von Transitionen */

typedef TCardinal * TransitionenMenge;

/* Datentyp 'Lineare Liste' fuer die interne Darstellung einer Stellenmenge */

struct SIntZelle
  { 
    char              * name;         /* Name der Stelle.                    */
    struct SIntZelle  * next;         /* Verweis auf naechstes Listenelement.*/
    TransitionenMenge PreSet,         /* Dynamische Liste der Vorgaenger der */
                                      /* der Stelle namens SIname.           */
                                      /* PreSet[0] =: m enthaelt die Anzahl  */
                                      /* der Feldelemente;                   */
                                      /* PreSet[1]...PreSet[m] enthalten     */
                                      /* die Ordnungszahlen der Transitionen.*/
        	      PostSet;        /* Dynamische Liste der Nachfolger     */
                                      /* PostSet[0] enthaelt Anzahl n;       */
                                      /* PostSet[1]...PostSet[n] sind wieder */
                                      /* die Ordnungszahlen von Transitionen.*/
  };

typedef struct SIntZelle  SIntElement;

typedef SIntElement       * IntStellenMenge;


/*****************************************************************************/
/***           P R O Z E D U R E N   u n d   F U N K T I O N E N           ***/
/*****************************************************************************/


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*+++            Makro zur Vereinfachung von Initialisierungen            +++*/
/*+++                         dynamischer Felder                          +++*/
/*+++        ( im folgenden insbes. fuer Mengen von Transitionen )        +++*/
/*+++      Version 1.0           Stefan Roemer        Mo.,25.05.1992      +++*/
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#define NEW_DYN_ARRAY(n) (TransitionenMenge) Malloc( ((n)+1)*sizeof(TCardinal) )

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*+++       Funktion zur Umwandlung einer Stellenliste in eine            +++*/
/*+++                    Algorithmus-interne Struktur                     +++*/
/*+++                                                                     +++*/
/*+++ Funktion zur Umwandlung einer linearen Liste von Stellennamen in    +++*/
/*+++ eine algorithmusinterne Darstellung.                                +++*/
/*+++ In dieser Darstellung wird jedem Listenelement zusaetzlich je ein   +++*/
/*+++ dynamischer Array fuer den Post- und den Preset der Stelle im Netz  +++*/
/*+++ zugeordnet, in dem jeweils die Nummern der Transitionen enthalten   +++*/
/*+++ sind, die in der entsprechenden Menge liegen. Zur besseren Handha-  +++*/
/*+++ bung werden  die  Namen  der  Transitionen  in  einem  Array   ge-  +++*/
/*+++ speichert und im Originalnetz so veraendert, dass sie fortlaufend   +++*/
/*+++ mit "T1","T2","T3",...,"Tn" durchnumeriert sind. Fuer diese interne +++*/
/*+++ Darstellung wird der Typ "IntStellenMenge" verwendet.               +++*/
/*+++ Weiterhin erzeugt diese Funktion ein dynamisches Feld, dessen Indi- +++*/
/*+++ ces die Nummern der im Netz vorhandenen Transitionen im obigen Sinne+++*/
/*+++ repraesentieren. Dieses Feld stellt die Menge PunktQ dar, wobei der +++*/
/*+++ Eintrag an der Stelle i gleich der Anzahl der Mengen PunktS ist, in +++*/
/*+++ denen die Transition i vorkommt.                                    +++*/
/*+++                                                                     +++*/
/*+++      Version 1.0            Lutz Lindau         Fr.,29.05.1992      +++*/
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#if defined(__STDC__) || defined(__cplusplus)

static IntStellenMenge Initialisiere_Q( LMPNet net, LIST_POINTER Qein, TCardinal * PunktQaus )

#else

static IntStellenMenge Initialisiere_Q( net, Qein, PunktQaus )

  LMPNet            net;             /* Das zu untersuchende Netz (wegen der */
        			     /* Transitionenliste) muss uebergeben   */
        			     /* werden */

  LIST_POINTER      Qein;            /* Zeiger fuer die eingegebene Stellen- */
        		             /* menge */

  TCardinal         * PunktQaus;     /* Ausgabe: Die Menge PunktQ in Form    */
        		             /* eines dynamischen Arrays mit Cardinal*/
        		             /* Eintraegen */
#endif 

  {        
    
    IntStellenMenge    anfneu,aktneu,hilfneu;     /* Hilfszeiger auf Element */
                                                  /* der internen Stellen-   */
                                                  /* liste */
    LIST_POINTER       aktalt;                    /* Hilfszeiger */
    Place              pl;
    STEdge             st;
    TSEdge             ts;
    Transition         t;                          /* Hilfszeiger auf Liste  */
        					   /* von Transitionen in    */
        					   /* Matrixform             */
    TCardinal          anzahl,i,AnzTran;           /* Hilfsvariablen         */

    /* Initialisieren des dynamischen Arrays fuer die Menge PunktQ */
    /* In der nullten Stelle des Feldes steht die Anzahl der Transitionen, */
    /* die weiteren Eintraege werden mit Null initialisiert. */
    *PunktQaus = AnzTran = net->NumTransitions;
    for( i = 1 ; i <= AnzTran ; ++i )         
      {			
        PunktQaus[i] = 0;
      }

    for(t = net->Transitions, anzahl = 0; t; t = t->nexttrans)
      {
        t->numname = ++anzahl;
      } /* while */

    /* Jetzt: Umwandeln der Eingabe-Stellenmenge in die Algorithmus-interne   */
    /*        Stellenmenge mit Post- und Presets                              */
    if ( Qein )  /* falls Stellen vorhanden... */
      {
        anfneu = aktneu = (IntStellenMenge) Malloc( sizeof( SIntElement ) );
        aktalt = Qein;         /* Anfang der externen Stellenliste */

        while( aktalt )        /* Bis alte Stellenliste abgearbeitet */
          {
            if ( ! ( pl = find_P( net, aktalt->name ) ) )
              {
		fprintf (stderr, "Error in call of 'Initialisiere_Q");
		fprintf (stderr, "()' : '%s' ", aktalt->name);
		fprintf (stderr, "is not a place name!\n");
              } /* if */
            aktneu->name = aktalt->name;	/* Namen kopieren */

            /* Nachfolger bestimmen und Nummern in Feld eintragen */ 
            for(st = pl->postset, anzahl = 0; st;
                st = st->nexttrans)
              {
                anzahl++;
              }
            aktneu->PostSet = NEW_DYN_ARRAY( anzahl );
            aktneu->PostSet[0] = anzahl;
            st = pl->postset;
            for ( i = 1 ; i <= anzahl ; i++ )
              {
                aktneu->PostSet[i] = st->trans->numname;
                st = st->nexttrans;
              }

            /* Vorgaenger bestimmen und Nummern in Feld eintragen */
            /* sowie Vermerk in Feld PunktQ vornehmen */ 
            for(ts = pl->preset, anzahl = 0; ts;
                ts = ts->nexttrans)
              {
                anzahl++;
              }
            aktneu->PreSet = NEW_DYN_ARRAY( anzahl );
            aktneu->PreSet[0] = anzahl;
            ts = pl->preset;
            for ( i = 1 ; i <= anzahl ; i++ )
              {
                PunktQaus[ aktneu->PreSet[i] = ts->trans->numname ]++;
                ts = ts->nexttrans;
              } /* while */
             
            if (( aktalt = aktalt->next ))
              {
        	hilfneu = (IntStellenMenge) Malloc( sizeof(SIntElement) );
        	aktneu->next = hilfneu;
        	aktneu = hilfneu;
              } /* if */
            else
              {
                aktneu->next = NULL;  /* Ende der alten Stellenliste */      
              } /* else */

          } /* while */

        /* Zum Schluss IntQaus die erstellte Liste zuweisen */

        return( anfneu );

      } /* if */

    return( NULL );

  }/* Ende Initialisiere_Q */


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*+++     Funktion zur Umwandlung der Stellenmenge Q aus der internen     +++*/
/*+++    Darstellung fuer den Algorithmus unten zurueck in die externe    +++*/
/*+++                    fuer das aufrufende Programm                     +++*/
/*+++      Version 1.0           Stefan Roemer        Mo.,25.05.1992      +++*/
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#if defined(__STDC__) || defined(__cplusplus)

static LIST_POINTER Umwandlung( IntStellenMenge Pein )
  
#else

static LIST_POINTER Umwandlung( Pein )
  
  IntStellenMenge Pein;              /* Stellenmenge in interner Darstellung */

#endif

  {
    
    IntStellenMenge Phelp,     /* Zeiger auf naechste Stelle in Stellenmenge */
                    Pptr;      /* Zeiger auf zu entfernende Stelle           */
    LIST_POINTER    Qptr,      /* Laufender Zeiger fuer die neuen Eintraege  */
                    Qaus;      /* Ergebnismenge, externe Listenschreibweise  */

    if ( Pein )
      {                                         /* Stellenmenge nicht leer : */
        Phelp = ( Pptr = Pein ) -> next;                 /* Menge umwandeln. */
        Qptr = Qaus = (LIST_POINTER) Malloc( sizeof( LIST_ELEMENT ) );
        strcpy( Qptr->name, Pptr->name );
        while ( Phelp )
          {
            Free( Pptr->PreSet );
            Free( Pptr->PostSet );
            Free( Pptr );
            Phelp = ( Pptr = Phelp ) -> next;
            Qptr = Qptr->next = 
	      (LIST_POINTER) Malloc( sizeof( LIST_ELEMENT ) );
            strcpy( Qptr->name, Pptr->name );
          } /* while */
        Qptr->next = NULL;
      } /* if */
    else                                              /* Stellenmenge leer : */
      {
        Qaus = NULL;                            /* Leere Menge zurueckgeben. */
      } /* else */

    return ( Qaus );

  } /* FUNCTION Umwandlung */


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*+++       Funktion zur Berechnung der maximalen Falle einer Menge       +++*/
/*+++            von Stellen nach dem vorgegebenen Algorithmus            +++*/
/*+++      Version 1.3           Stefan Roemer        Mi.,24.06.1992      +++*/
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#if defined(__STDC__) || defined(__cplusplus)

LIST_POINTER Berechne_maximale_Falle( LMPNet System, LIST_POINTER X )

#else

LIST_POINTER Berechne_maximale_Falle( System, X )

  LMPNet       System;                 /* Zu bearbeitendes, markiertes Netz. */
  LIST_POINTER X;                      /* Zu bearbeitende Menge von Stellen. */

#endif

  {

    IntStellenMenge            Q,          /* Interne, d.h. in dieser Funk-  */
                                           /* tion benutzte Form der Stellen-*/
                                           /* menge X.                       */
                               s,          /* Zeiger auf ein Element von Q.  */
                               s_davor;    /* Zeiger auf das Element von Q,  */
                                           /* das in der Listenschreibweise  */
                                           /* von Q unmittelbar vor s liegt. */
    TransitionenMenge          PunktQ;     /* PunktQ[0] =: k enthaelt die    */
                                           /* Kardinalitaet der Transitionen-*/
                                           /* menge T des zu untersuchenden  */
                                           /* Netzes. Die Feldindizes 1..k   */
                                           /* dienen der eindeutigen Zuord-  */
                                           /* nung einer Transition zu einem */
                                           /* Feldelement. Jedes Feldelement */
                                           /* PunktQ[i], i=1,...,k, enthaelt */
                                           /* die Anzahl der Mengen *s,      */
                                           /* s aus Q, die die Transition    */
                                           /* Nr. i enthalten.               */
                                           /* Transitionen t, die nicht in   */
                                           /* *Q enthalten sind, sind        */
                                           /* 'geloescht', d.h. PunktQ[t]==0.*/ 
    register TransitionenMenge tptr;       /* Zeiger auf eine Transition in  */
                                           /* einem PreSet bzw. PostSet.     */
    register TCardinal         i;          /* Schleifenzaehler.              */

                                               /* Initialisierungen          */
    PunktQ = NEW_DYN_ARRAY( System->NumTransitions );
    Q = Initialisiere_Q( System, X, PunktQ );

    s = (IntStellenMenge) Malloc( sizeof( SIntElement ) );   /* Hilfselement */
    s->next = Q;                               /* vor dem ersten Element ein-*/
    Q = s;                                     /* fuegen, damit auch die er- */
                                               /* ste Stelle einen Vorgaenger*/
                                               /* hat, auf den dann 's_davor'*/
                                               /* weisen kann.               */
    
    while (( s = ( s_davor = s ) -> next ))
      {
        tptr = s->PostSet;                     /* Testen, ob s* Teilmenge    */
        for ( i = *(tptr++) ; i ; i-- )        /* von *Q ist.                */
          {
            if ( ! PunktQ[ *(tptr++) ] )       /* falls keine Teilmenge:     */
              {
                tptr = s->PreSet;
                for ( i = *(tptr++) ; i ; i-- )/* *Q aktualisieren;          */
                  {
                    --PunktQ[ *(tptr++) ];
                  } /* for */
                s_davor->next = s->next;       /* Stelle aus Liste entfernen;*/
                Free( s->PreSet ); 
                Free( s->PostSet );
                Free( s );
                s = Q;                         /* an den Anfang der Stellen- */
                break; /* for */               /* liste zurueckgehen.        */
              } /* if */
          } /* for */
      } /* while */

    Q = ( s = Q ) -> next;                     /* Hilfselement wieder vom    */
    Free( s );                                 /* Anfang der Liste entfernen.*/

    Free( PunktQ );                            /* Von *Q belegten Speicher-  */
        				       /* bereich freigeben.         */

    return ( Umwandlung( Q ) );                /* Max. Trap Q zurueckgeben.  */

  } /* FUNCTION Berechne_maximale_Falle */


/*****************************************************************************/
/***       Praktikum Werkzeuge zur Analyse von Petri-Netzen, SS 1992       ***/
/***       Hauptprogramm fuer Programmteil `Test auf Reversibilitaet`      ***/
/***       Version 1.0      Konstantin Lazaridis      Fr.,29.05.1992       ***/
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

bool reversibel( LMPNet net )

#else

bool reversibel( net )
  /* Liefert : - `True` , falls das Netz reversibel ist.       */
  /*           - `False`, falls das Netz nicht reversibel ist. */

  LMPNet net;

#endif

  {

    Place        place;
    LIST_POINTER zeiger,   /* Hilfszeiger auf Menge der unmarkierten Stellen */
                 Qein = NULL,   /* Zeiger auf Menge der unmarkierten Stellen */
                 Qaus=NULL;     /* Zeiger auf Stellenmenge, die die groesste */
                                /* Falle enthaelt                            */

    if ( ! net || ! net->Places )
      {
/*
        puts( "Fehler bei Aufruf von reversibel(): Kein System uebergeben!" );
*/
        return ( False );
      } /* if */

    if ( 1 /* lebendig( net ) */ )          
                   /* Voraussetzung fuer Reversibilitaet :                   */
                   /* Lebendigkeit, Beschraenktheit und FC-Eigenschaft des   */
                   /* Netzes. Beschraenktheit und FC-Eigenschaft werden      */
                   /* bereits in lebendig(net) ueberprueft. Deshalb reicht   */
                   /* der Test auf Lebendigkeit.                             */
      { 
                                /* Liste der unmarkierten Stellen berechnen  */
        place = net->Places;
        while ( place && place->marks ) 
          {
            place = place->nextplace;
          } /* while */

        if ( place )             /* d.h. unmarkierte Stelle im Netz gefunden */
          {
            Qein = zeiger = (LIST_POINTER) Malloc( sizeof( LIST_ELEMENT ) );
            strcpy( zeiger->name, place->name );
            while (( place = place->nextplace )) 
              {
                if ( ! place->marks )    /* d.h. unmarkierte Stelle gefunden */
                  {
                    zeiger = zeiger->next
                           = (LIST_POINTER) Malloc( sizeof( LIST_ELEMENT ) ); 
                    strcpy( zeiger->name, place->name ); 
                  } /* if */
              } /* while ( place ... ) */
            zeiger->next = NULL;
          } /* if */

        if (( Qaus = Berechne_maximale_Falle( net, Qein ) ))
                                      /* Falls das Netz reversibel ist, darf */
                                      /* es keine unmarkierte Falle geben    */
          {
/*
            puts( "Das untersuchte System ist NICHT REVERSIBEL !!!\n" );
*/
            remove_list( &Qein );
            remove_list( &Qaus );
            return ( False );
            /* System nicht reversibel, da eine unmarkierte Falle existiert, */ 
            /* weil Ergebnis ungleich leerer Menge ist.                      */
          } /* if */
        else
          {
/*
            puts( "Das untersuchte System IST REVERSIBEL !!!\n" );
*/
            remove_list( &Qein );
            remove_list( &Qaus );
            return ( True );
            /* System ist reversibel, da keine unmarkierte Falle existiert,  */
            /* weil Ergebnis die leere Menge ist.                            */ 
          } /* else */ 
      } /* if */
    else
      {
/*
     puts( "System ist nicht lebendig, Reversibilitaet wird nicht getestet." );
*/
        return ( False );
        /* System ist nicht reversibel, da es entweder nicht die FC-Eigen-   */
        /* schaft besitzt, nicht beschraenkt oder nicht lebendig ist.        */
      } /* else */

  } /* reversibel() */  


/*****************************************************************************/
/* END OF FILE rev.c                                                         */
/*****************************************************************************/

