#ifndef __VECTOR_H__
#define __VECTOR_H__

#include "neunetze.h"

/* Datentyp fuer die Darstellung einer Teilmenge der natuerlichen Zahlen */

typedef unsigned int VCardinal;

/* Datentyp zur Darstellung eines Markierungsvektors */

typedef VCardinal * Vektor;

/******************************************************************************/
/*   Makro zum Einrichten eines neuen Markierungsvektors; nur ein mit dieser  */
/*   Funktion initialisierter Vektor ist physikalisch im Speicher vorhanden;  */
/*           auf alle anderen wird mit Hilfe von Zeigern verwiesen.           */
/******************************************************************************/

#define NeuerVektor(n) (Vektor) Malloc( ( ((n)<<1) + 1 ) * sizeof( VCardinal ) )


/******************************************************************************/
/*                  Funktion zum Kopieren  einer Markierung                   */
/******************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)
Vektor Kopiere( Vektor v1, Vektor v2 );
#else
Vektor Kopiere( v1, v2 );
/* Es wird v1 := v2 ausgefuehrt und ein Zeiger */
/* auf v1 als Funktionswert zurueckgegeben.    */
#endif


/******************************************************************************/
/*        Funktion zur Initialisierung eines neuen Markierungsvektors         */
/*           aus einem gegebenen ( geordneten ! ) Markierungsvektor           */
/******************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)
Vektor ErzeugeVektor( Vektor vek );
#else
Vektor ErzeugeVektor( vek );
#endif


/******************************************************************************/
/*    Funktion, die aus dem Vorbereich einer Transition den entsprechenden    */
/* Markierungsvektor generiert; der Zielvektor dieser Funktion ( vek ) muss   */
/*             gross genug sein, um das Ergebnis aufzunehmen.                 */
/******************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)
Vektor VektorAusTSListe( Vektor vek, TSEdge tsListe );
#else
Vektor VektorAusTSListe( vek, tsListe );
#endif

  
/******************************************************************************/
/*    Funktion, die aus dem Nachbereich einer Transition den entsprechenden   */
/* Markierungsvektor generiert; der Zielvektor dieser Funktion ( vek ) muss   */
/*             gross genug sein, um das Ergebnis aufzunehmen.                 */
/******************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)
Vektor VektorAusSTListe( Vektor vek, STEdge stListe );
#else
Vektor VektorAusSTListe( vek, stListe );
#endif

  
/*****************************************************************************/
/*                                                                           */
/*        Funktion zur Feststellung , ob eine Markierung Teil einer          */
/*                          anderen Markierung  ist                          */
/*                                                                           */
/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

bool VEnthalten (Vektor  v1,                    /* Die zuvergleichenden Vek- */
                Vektor  v2);                    /* toren, Test auf v1 == v2  */

#else

bool VEnthalten (v1, v2);

#endif

/******************************************************************************/
/*        Funktion zur Feststellung , ob eine Markierung gleich               */
/*                          anderen Markierung  ist                           */
/******************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)
bool VektorGleich( Vektor v1, Vektor v2 );
#else
bool VektorGleich( v1, v2 );
#endif


/******************************************************************************/
/*        Funktion zur Feststellung, ob eine Markierung 'kleiner' ist         */
/*                         als eine andere Markierung                         */
/******************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)
bool VektorKleiner( Vektor v1, Vektor v2 );
#else
bool VektorKleiner( v1, v2 );
#endif


/******************************************************************************/
/*              Funktion zur Addition zweier Markierungsvektoren              */
/******************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)
Vektor Addiere( Vektor v1, Vektor v2, Vektor HilfsVektor );
#else
Vektor Addiere( v1, v2, HilfsVektor );
#endif


/******************************************************************************/
/*            Funktion zur Subtraktion zweier Markierungsvektoren             */
/******************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)
Vektor Subtrahiere( Vektor v1, Vektor v2 );
#else
Vektor Subtrahiere( v1, v2 );
#endif


/******************************************************************************/
/*      Funktion zur Multiplikation eines Markierungsvektoren mit Omega       */
/******************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)
Vektor MultipliziereMitOmega( Vektor vek );
#else
Vektor MultipliziereMitOmega( vek );
#endif

#endif
