/*****************************************************************************/
/***       Praktikum Werkzeuge zur Analyse von Petri-Netzen, SS 1992       ***/
/***       Header-Datei fuer Programmteil 'Test auf Beschraenktheit'       ***/
/*****************************************************************************/
/***    Version 2.0 ( neue Datenstruktur )    Stefan Roemer, 10.05.1993    ***/
/*****************************************************************************/

#ifndef __BOUND_H__
#define __BOUND_H__

#include "neunetze.h"
#include "vector.h"

/* Wenn die Direktive __TRACE__ definiert ist, kann der Auf- und Abbau  */
/* der zur Berechnung der Schaltfolgen erforderlichen Listen verfolgt   */
/* werden. 								*/

/*
#ifndef __TRACE__
#define __TRACE__
#undef __TRACE__ 
#endif
*/

/*****************************************************************************/
/*                       T Y P D E F I N I T I O N E N                       */
/*****************************************************************************/

/* Datentyp zur Darstellung einer Liste von Pfadvorgaengern einer Markierung */
typedef struct KnotenZelle     * KnotenMenge;

typedef struct VorgaengerZelle * VorgaengerListe;

typedef struct VorgaengerZelle
{
  KnotenMenge     KnotenPtr;
  VorgaengerListe next;
} Vorgaenger;

/* Datentyp 'Lineare Liste' fuer die Darstellung der Knotenmenge */
/* (Menge der Markierungsvektoren) */
typedef struct KnotenZelle
{
  Vektor            Markierung;   /* Knoten = Markierung.      */
  KnotenMenge       next;         /* Verweis auf naechsten Knoten. */
  bool              besucht;      /* Flag; zeigt an, ob dieser Knoten   */
                                  /* bei der Suche nach den Pfadvor-    */
				  /* gaengern bereits untersucht wurde. */
  VorgaengerListe   PfadVorg;     /* Liste der Knoten, die eine Kante   */
                                  /* zu dieser Markierung besitzen.     */
} Knoten;


/*****************************************************************************/
/*       Freigabe des vom Ueberdeckungsgraphen besetzten Speicherplatzes     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void RemoveUDG (KnotenMenge KnotenPtr);

#else

void RemoveUDG ( KnotenPtr );

#endif

/*                                                                            */
/* Parameter                                                                  */
/*                                                                            */
/* KnotenMenge    KnotenPtr;                        zu entfernender UD-graph  */
/*                                                                            */
/*   Freigabe des vom Ueberdeckungsgraphen besetzten Speicherplatzes          */
/*                                                                            */
/******************************************************************************/



/******************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

extern void StellenBenennen (LMPNet, Place*);

#else

extern void StellenBenennen ( );
                   
#endif

/*                                                                            */
/* Parameter:                                                                 */
/*                                                                            */
/* Place       Places;                              Zeiger auf die Stellen-   */
/*                                                  liste eines Netzes        */
/* Place     * Stellenfeld;                         Feld von Zeigern auf die  */
/*                                                  Stellen.                  */
/*                                                                            */
/*        Die im Netz gegebenen Stellennamen speichern und durch neue,        */
/*                       numerisch geordnete ersetzen                         */
/*                                                                            */
/******************************************************************************/


/******************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

extern void SchreibeMarkierung (FILE  * ,
                                Vektor  ,
                                Place * ,
                                bool    );

#else

extern void SchreibeMarkierung ( );

#endif                                           /* END STDC & C++ DEFINITION */

/*                                                                            */
/* Parameter:                                                                 */
/*                                                                            */
/* FILE    * Datei;                                 Zeiger auf die Zieldatei. */
/* Vektor  vek;                                     Auszugebende Markierung   */
/* Place   * Stellenfeld;                           Stellenliste des Netzes.  */
/* bool    Newline;                                 Flag: Falls True, wird    */
/*                                                        nach dem Vektor ein */
/*                                                        '\n' geschrieben.   */
/*                                                                            */
/*        Prozedur, die einen Markierungsvektor in eine Datei schreibt        */
/*                                                                            */
/******************************************************************************/


/******************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

extern Vektor MarktoVek (MARKING    ,            /* Zeiger auf Markierung,    */
                         VCardinal  );           /* Groesse des Vektors 'Vek` */

#else

extern Vektor MarktoVek ( );

#endif                                           /* END STDC & C++ DEFINITION */

/*                                                                            */
/* Parameter:                                                                 */
/*                                                                            */
/*  MARKING   Mark;                                 Zeiger auf Markierung,    */
/*  VCardinal Stellen;                              Groesse des Vektors 'Vek` */
/*                                                                            */
/*  transferiert eine Markierung in einen Vektor                              */
/*                                                                            */
/******************************************************************************/



/******************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

extern void PrintUDG (KnotenMenge,               /* auszugebender UD-graph    */
                      Place  *   ,               /* Zeigerfeld auf Stellen    */
                                                 /* eines Netzes              */
                      FILE   *   );              /* zeigt auf geoeffnete Datei*/
#else

extern void PrintUDG ( );

#endif                                           /* END STDC & C++ DEFINITION */

/*                                                                            */
/* Parameter:                                                                 */
/*                                                                            */
/* KnotenMenge     KnotenPtr;                       zu entfernender UD-graph  */
/* FILE            *Datei;                          zeigt auf geoeffnete Datei*/
/* Place           *SFeld;                          Zeigerfeld auf Stellen    */
/*                                                  eines Netzes              */
/*                                                                            */
/*   Ausgabe eines Ueberdeckungsgraphen in `Datei`                            */
/*                                                                            */
/******************************************************************************/


/******************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

extern void UntersuchePfad  (KnotenMenge ,
                             KnotenMenge ,
                             Vektor      ,
                             Vektor      ,
                             Vektor      );

#else

extern void UntersuchePfad  ( );

#endif                                           /* END STDC & C++ DEFINITION */

/*                                                                            */
/* Parameter:                                                                 */
/*                                                                            */
/* KnotenMenge     VE,                              Die Menge der Markie-     */
/*                                                  rungsvektoren.            */
/*                 vQuer;                           Aktuell zu bearbeitende   */
/*                                                  Markierung.               */
/* Vektor          vStrich,                         Zu vergleichender Vektor  */
/*                 NV,                              Vektor der unbeschraenk-  */
/*                                                  ten Stellen.              */
/*                 HilfsVektor;                     Hilfsvektor zur Aufnahme  */
/*                                                  des Zwischenergebnisses   */
/*                                                  bei der Addition.         */
/*                                                                            */
/*     Funktion zur Untersuchung der Vorgaengermarkierungen des  bereits      */
/*     erstellten Ueberdeckungsgraphen, ob diese die aktuelle Markierung      */
/*        jeweils echt ueberdecken; dies ist eine REKURSIVE Funktion.         */
/*                                                                            */
/******************************************************************************/


/******************************************************************************/
/*                                                                            */
/*    Unterprogramm zur Generierung des Ueberdeckungsgraphens eines Systems   */
/*                                                                            */
/******************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

extern bool Ueberdeckungsgraph(LMPNet,
                               char * 
#if defined(__TRACE__)
                              ,FILE * 
#endif
                                      );
#else

extern bool Ueberdeckungsgraph();
                   
#endif

/*                                                                            */
/* Parameter:                                                                 */
/*                                                                            */
/* LMPNet          Net;                             zu untersuchendes Netz    */
/* char          * netname;                         Name des Netzes           */
/*                                                                            */
/*   #if defined(__TRACE__)                                                   */
/*   FILE         * Datei;                          Ausgabedatei              */
/*   #endif                                                                   */
/*                                                                            */
/*       Funktion zur Berechnung des Ueberdeckungsgraphen eines Systems       */
/*                                                                            */
/*  Liefert 1, falls das System beschraenkt ist, und 0, falls unbeschraenkt.  */
/*    Falls der uebergebene Dateiname leer ( "" ) ist, wird keine Datei be-   */
/*    schrieben, ansonsten wird eine Datei mit dem Dateinamen und der Endung  */
/*  '.UDG' angelegt, in die der erzeugte Ueberdeckungsgraph geschrieben wird. */
/*                                                                            */
/******************************************************************************/


/******************************************************************************/
/*                                                                            */
/*         Hauptprogramm fuer Programmteil 'Test auf Beschraenktheit'         */
/*                                                                            */
/******************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern bool beschraenkt(LMPNet
#if defined(__TRACE__)
                       ,FILE   * 
#endif
                                   );
#else

extern bool beschraenkt();

#endif

/*                                                                            */
/* Parameter:                                                                 */
/*                                                                            */
/*  LMPNet          Net;                            zu untersuchendes Netz    */
/*                                                                            */
/*   #if defined(__TRACE__)                         TRACE ausgabe             */
/*   FILE         * Datei;                          Ausgabedatei              */
/*   #endif                                         END TRACE DEFINITION      */
/*                                                                            */
/*         Hauptprogramm fuer Programmteil 'Test auf Beschraenktheit'         */
/*                                                                            */
/* Diese Funktion liefert True, falls das zu untersuchende System beschraenkt */
/* ist, und False, falls nicht. Ausserdem wird eine Datei 'TEST.UDG' angelegt,*/
/* in der der zugehoerige Ueberdeckungsgraph enthalten ist.                   */
/*                                                                            */
/******************************************************************************/

#endif /* __BOUND_H__  */

/******************************************************************************/
/*                            END OF FILE bound.h                             */
/******************************************************************************/

