/*****************************************************************************/
/* Programm zum Testen der Erreichbarkeit          Eberhard Meisel  1994     */
/*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "neunetze.h"
#include "convertPEP.h"
#include "reachy.h"

/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

void ShowHelp (char * Filename)

#else

void ShowHelp (Filename)

char *  Filename;

#endif

{
  printf ("\n\nUsage:\n\n");
  printf ("%s -I infile  < -M markfile | -m marking > [-O outfile]\n\n", Filename);
  printf ("  -I|i Name of PEP file to read\n");
  printf ("  -M   Name of file with marking to reach\n");
  printf ("  -m   Marking (e.g. 011000100001)\n");
  printf ("  -O|o Name of result file\n\n");
}


/*****************************************************************************/

int main( int argc, char **argv )
{
  FILE       * infile;
  FILE       * outfile=NULL;

  LMPNet       Netz;
  MARKING      EMark = NULL,
               Mark0 = NULL;

  int          reachable;

  char         endung [5] = ".res";

  char         pepname [50] = "";
  char         outname  [50] = "";
  char         netname  [50] = "";
  char         markname [50] = "";

  int          i = 0;

  bool         marking = False;

if (argc >= 2)
   {
   for (i = 0; i < argc; i++)                   /* Luecken mit Leerzeichen   */
      {                                         /* auffuellen                */
      if ((argv[i][0] == '-') || (argv[i][0] == '/'))
         {
         if (argv[i][1] == 'I' ||               /* WENN das 2. Zeichen ein   */
             argv[i][1] == 'i'    )             /* 'I' oder ein 'i' ist      */
            {                                   /* DANN                      */
            strcpy (pepname, argv[i]+2);       /* Name PEPdatei einlesen   */
            if (!*outname)                      /* WENN noch keine Ausgabe-  */
               {                                /* datei angeben wurde, DANN */
               strcpy (outname, argv[i]+2);     /* an den Namen der Eingabe- */
               strcat (outname, endung);        /* datei eine Endung haengen */
               strcpy (netname, outname);
               }                                /* RND IF (outname == "");   */
            }                                   /* END IF (2. Zeichen = I,i) */
         if (argv[i][1] == 'O' ||               /* WENN das 2. Zeichen ein   */
             argv[i][1] == 'o'    )             /* 'O' oder ein 'o' ist      */
            {                                   /* DANN                      */
            strcpy (outname, argv[i]+2);        /* Ausgabedateinamen kopieren*/
            if (!(strchr (outname, '.')))       /* WENN der Name keinen Punkt*/
               {                                /* enhaelt, DANN an den Namen*/
               strcat (outname, endung);        /* der Ausgabedatei eine En- */
               }                                /* dung haengen              */
            strcpy (netname, outname);
            }                                   /* END IF (2. Zeichen = I,i) */

         if (argv[i][1] == 'M')
           {
            strcpy (markname, argv[i]+2);
            marking = False;
            }
         if (argv[i][1] == 'm')
            {
            strcpy (markname, argv[i]+2);
            marking = True;
            }
         }
      }
   }
else
   {
   ShowHelp (*argv);
   exit (1);
   }

if (*pepname)
   {
   if ((infile = fopen (pepname, "r")))        /* EXISTIERT PEPdatei???    */
      {
      fclose (infile);
      Netz = read_PEP (pepname);               /* Netz aus PEP.datei lesen */
      }                                         /* END IF (Datei existiert)  */
   else                                         /* ANDERNFALLS               */
      {                                         /* Fehlermeldung ausgeben    */
      fprintf (stderr, "Could not read file '%s'!\n\n", pepname);
      ShowHelp (argv[0]);
      exit (1);
      }
   }                                            /* END IF (filename)         */
else
   {
   fprintf (stderr, "No input file given!!!\n");
   ShowHelp (argv[0]);
   exit (1);
   }
if (*outname)
   {
   outfile = fopen (outname, "w");              /* Oeffnen der Ausgabedatei  */
   if (!outfile)
      {
      fclose (outfile);
      fprintf (stderr, "Could not write file '%s'!!!\n\n", netname);
      ShowHelp (argv[0]);
      exit (1);
      }
   }                                             /* END IF (*outname)         */
else
   {
   fprintf (stderr, "No result file given!!!\n");
   ShowHelp (argv[0]);
   }


Mark0 = copy_start_marking (Netz);               /* Anfangsmarkierung des     */
                                                 /* Netzes einlesen           */
if (*markname)
   {
   if (marking)
      {
      EMark = ReadArgMark (Netz, markname);
      }
   else
      {
      EMark = ReadMarkFile (Netz, markname);
      }
   }
else
   {
   fprintf (stderr, "No marking or marking file given!!!\n");
   ShowPlaces (Mark0, Netz->Places, stdout);
   ShowHelp (argv[0]);
   exit (1);
   }


  if (Netz)
  {
    #if defined(__TRACE__)
      if ((reachable = Reachability (Netz, Mark0, EMark, outfile))== 1)
    #else
      if ((reachable = Reachability (Netz, Mark0, EMark))== 1)
    #endif
      {
//        Erreichbarkeitspfad( Netz, EMark, "rev_reach.res" );
	fprintf (outfile, "Marking is reachable.\n");
      }
      else if ( reachable == -1 )
      {
        fprintf (outfile, "Reachability could not be ");
        fprintf (outfile, "decided efficiently.\n");
      }
      else
      {
        fprintf (outfile, "Marking is not reachable.\n");
      }
  }
  fclose( outfile );
  remove_marklist (&Mark0);
  remove_marklist (&EMark);
  remove_net (&Netz);
  exit (0);
}

