/*****************************************************************************/
/*                                                                           */
/* Diese Testprogramm generiert aus einem Ueberdeckungsgraphen eines Petri-  */
/* netzes Markierungsdateien. Zudem wird eine Stapeldatei generiert, mit der */
/* diese Markierungen auf Erreichbarkeit getestet und die Schaltfolgen be-   */
/* rechnet werden koennen                                                    */
/*                                          Eberhard Meisel   1994           */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "neunetze.h"
#include "convertPEP.h"
#include "bound.h"
#include "reachy.h"

/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

void ShowHelp (char * Filename)

#else

void ShowHelp (Filename)

char *  Filename;

#endif
/*****************************************************************************/

{
  printf ("\n\n");
  printf ("Generate Marking files from covergraph of a Petri net.\n");
  printf ("Also generate batch files (suffix .test) to check file for\n");
  printf ("reachability or generate sequences to this marking.\n");
  printf ("Usage:\n\n");
  printf ("%s -I infile [-a -e -O outfile]\n\n", Filename);
  printf ("  -I|i Name of PEP file to read\n");
  printf ("  -O|o Name of result file\n");
  printf ("  -A|a Calculate all paths\n");
  printf ("  -E|e write batch file for reachability test\n");
  printf ("       (default is sequences)\n\n");
}

/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

void CreateMarkFiles (LMPNet      Netz    ,     /* zu untersuchendes Netz    */
                      ReachList   RList   ,     /* Erreichbarkeitsliste      */
                      bool        Minpfad ,     /* Option -1 setzen || nicht */
                      bool        Errtest ,     /* Option -E setzen || nicht */
                      char      * Pepname,      /* Name der PEPdatei         */
                      char      * Netzname)     /* Name der Ausgabedatei     */
#else

void CreateMarkFiles (Netz, RList, Minpfad, Errtest, Pepname, Netzname)

LMPNet        Netz;                             /* zu untersuchendes Netz    */
ReachList     RList;                            /* Erreichbarkeitsliste      */
bool          Minpfad;                          /* soll -1 angehaengt werden?*/
bool          Errtest;                          /* soll -E angehaengt werden?*/
char        * Pepname;                          /* Name der PEPdatei        */
char        * Netzname;                         /* Name der Ausgabedatei     */

#endif

/*                                                                           */
/*   Diese Prozedur generiert aus jeder neuen erreichbaren Markierung der    */
/*   Erreichbarkeitsliste eine Markierungsdatei mit fortlaufender Numerierung*/
/*   sowie eine Stapeldatei, benannt "Pepname.TEST", mit der diese Markie-   */
/*   rungen dann getestet werden koennen, sowie einer Stapeldatei, benannt   */
/*   "Pepname.REMOVE", die alle Testausgaben, Markierungsdateien und die     */
/*   Testdatei loescht.                                                      */
/*                                                                           */
/*****************************************************************************/

{
ReachList  AReach = RList;                      /* Laufzeiger ReachListe     */

Vektor     vekPtr;                              /* Laufzeiger fuer Markie-   */
                                                /* rungsvektor               */
Place    * Stellenfeld = (Place *) Malloc       /* Allokiere Speicher fuer   */
             (Netz->NumPlaces * sizeof (Place));/* Zeigerfeld auf die Stel-  */
                                                /* len des Netzes "Netz"     */
char       markname [50]  = "",                 /* Name der gerade geschrie- */
                                                /* benen Markierungsdatei    */
         * markfname      = NULL,               /* Zeiger auf Namen der zu   */
                                                /* schreibenden Markierungs- */
                                                /* datei                     */
         * testname       = NULL,               /* Zeiger auf Namen der zu   */
                                                /* schreibenden TEST-Datei   */
         * rmvname        = NULL,               /* Zeiger auf Namen der zu   */
                                                /* schreibenden remove-Datei */
           batchname [50] = "",                 /* Name Batch-Datei TEST     */
           removename[50] = "",                 /* Name Batch-Datei REMOVE   */
           outname   [50] = "",                 /* Name der Ausgabedatei zu  */
                                                /* obiger Markierungsdatei   */
           endung [10]    = "";                 /* Endung einer Markierungs- */
                                                /* datei                     */
FILE     * markfile;                            /* Zeiger Markierungsdatei   */
FILE     * batch;                               /* Zeiger Testbatchdatei     */
FILE     * remove;                              /* Zeiger Removebatchdatei   */
int        mfnum     = 1;                       /* Nummer des Markfiles      */


StellenBenennen (Netz, Stellenfeld);    /* Stellen temporaer umbenen.*/

if (AReach)                                     /* WENN Erreichbarkeitsliste */
   {                                            /* vorhanden                 */
   while (AReach->next)
      {                                         /* Zeiger auf das Listenende */
      AReach = AReach->next;                    /* setzen                    */
      }                                         /* END WHILE (AReach->next)  */

   strcpy (batchname, Netzname);                /* Batchdateinamen bilden    */
   strcat (batchname, ".TEST");                 /*    "beispiel.ll_net.TEST" */
   testname = batchname;

   if ((batch = fopen (testname, "w")))         /* WENN sich Ausgabedatei    */
      {                                         /* oeffnen laesst            */
      printf ("Testfile > %s", batchname);
      printf (" < is being generated!!\n");
      }                                         /* END IF (Datei oeffnen)    */
   else                                         /* ANDERNFALLS               */
      {
      printf ( "Could not write file '%s'!\n", batchname);
      exit (1);                                 /* und abbrechen             */
      }                                         /* END IF (Dateioeffnen)     */

   strcpy (removename, Netzname);               /* Batchdateinamen bilden    */
   strcat (removename, ".REMOVE");              /*  "beispiel.ll_net.REMOVE" */
   rmvname = removename;

   if ((remove = fopen (rmvname, "w")))         /* WENN sich Ausgabedatei    */
      {                                         /* oeffnen laesst            */
      printf ("Testfile > %s", removename);
      printf (" < is being generated!!\n");
      }                                         /* END IF (Datei oeffnen)    */
   else                                         /* ANDERNFALLS               */
      {
      printf ( "Could not write file '%s'!\n", removename);
      exit (1);                                 /* und abbrechen             */
      }                                         /* END IF (Dateioeffnen)     */

   while (AReach->prev)                         /* WEIL ReachListenelement   */
      {                                         /* noch vorhanden DO         */
      if (AReach->alt == 0)                     /* WENN neue Markierung      */
         {                                      /* DANN                      */
         if (AReach->mark)
            {
            sprintf (endung, "%d", mfnum++);
            strcpy (markname, Netzname);        /* Namen einer Markierungs-  */
            strcat (markname, ".M");            /* datei bilden, z. B.:      */
            strcat (markname, endung);          /* beispiel.PEP.M1           */
            strcpy (outname, markname);         /* Ausgabedateinamen bilden: */
            strcat (outname, ".res");           /* z.B. beispiel.PEP.M1.res  */
            markfname = markname;

            vekPtr = AReach->mark;              /* Vektorlaufzeiger auf Mar- */
                                                /* kierung setzen            */
            if ((markfile = fopen (markfname,"w")))
               {                                /* Markierungsdatei oeffnen  */
               while (*(vekPtr++))              /* Mit jedem Vektorelement DO*/
                  {
                  if (*vekPtr > 1)              /* WENN Markenzahl > 1       */
                     {                          /* DANN Ausgabe der Marken-  */
                     fprintf (markfile, " %hu", /* anzahl der Stelle         */
                                        *vekPtr);
                     }                          /* END IF (*vekPtr > 1)      */
                  fprintf (markfile, " %s", Stellenfeld
                        [(*(vekPtr-1))-1]->name);/* Stellennamen schreiben   */
                  if (*(++vekPtr))              /* FALLS noch eine Stelle da */
                     {                          /* DANN Komma zum separieren */
                     fprintf (markfile, ",");   /* der Stellen schreiben     */
                     }                          /* END IF (*(++vekPtr))      */
                  }                             /* END WHILE (*(vekPtr++))   */
               fprintf (markfile, "\n");        /* schliesse Markierung ab   */
               fclose (markfile);               /* schliesse Markierungsdatei*/

               if (Errtest)                     /* WENN Stapeldatei mit Test */
                  {                             /* auf Erreichbarkeit gene-  */
                  fprintf(batch,"ERREICHBARKEIT ");/* riert werden soll,     */
                  fprintf(batch,"-I%s -O%s -M%s\n",/* "Progammname -Optionen"*/
                                Pepname,        /* Eingabe: -IPepDatei       */
                                outname,        /* Ausgabe:-OPepDatei.Mx.res */
                                markname);      /* Markierung: -MMarkfile    */
                  }                             /* END IF (Errtest  == TRUE) */
               else                             /* ANDERNFALLS Stapeldatei   */
                  {                             /* mit Schaltfolgenberech-   */
                  fprintf(batch,"SCHALTFOLGEN ");/* nung generieren,         */
                  fprintf(batch,"-I%s -O%s -M%s",/* "Progammname -Optionen"  */
                                Pepname,        /* Eingabe: -IPepDatei       */
                                outname,        /* Ausgabe:-OPepDatei.Mx.res */
                                markname);      /* Markierung: -MMarkfile    */
                  fprintf (batch, Minpfad ?     /* sollen alle Pfade oder nur*/
                              " -1\n" : " -A\n");/* ein Pfad generiert werden*/
                  }                             /* ENDIF (Errtest = FALSE)   */
               fprintf (remove, "rm %s %s\n",   /* schreibe Loeschstapeldatei*/
                                markname,       /* loesche Markierungsdatei  */
                                outname);       /* loesche Ausgabedatei      */
               }                                /* ENDIF (Mark'datei oeffnen)*/
            else                                /* ANDERNFALLS Markierungs-  */
               {                                /* datei kann nicht geoeff-  */
               fprintf (stderr, "Could not open file %s ",markname);
               }                                /* END ELSE (Datei oeffnen)  */
            }                                   /* END IF (mfnum < 100)      */
         }                                      /* END IF (AReach->alt == 1) */
      AReach = AReach->prev;                    /* Laufzeiger auf vorheriges */
                                                /* Listenelement setzen      */
      }                                         /* END WHILE (AReach)        */
   fprintf (remove, "rm %s \n", batchname);
   fprintf (remove, "rm %s \n", removename);
   fclose (remove);                             /* schliessen der beiden     */
   fclose (batch);                              /* Stapeldateien             */
   }                                            /* END IF (AReach)           */
} /*** CreateMarkfiles() ***/


/*****************************************************************************/
/*                                                                           */
/*    Hauptprogramm                                                          */
/*                                                                           */

int main( int argc, char **argv )

{
FILE       * infile;                            /* Zeiger auf einzulesende   */
                                                /* PEPdatei                 */
FILE       * outfile;                           /* Zeiger auf Ausgabedatei   */

LMPNet       Netz = NULL;                       /* Zeiger auf das zu unter-  */
                                                /* suchende Netz             */
MARKING      Mark0 = NULL;                      /* Anfangsmarkierung in Mar- */
                                                /* kierungslistenform        */
Vektor       StartVek = NULL,                   /* Anfangsmarkierung in Vek- */
                                                /* tordarstellung            */
             ReachVek = NULL;                   /* zu erreichende Markierung */
                                                /* in Vektordarstellung      */

PfadKopf     PLKopf = (PfadKopf) Malloc         /* Zeiger auf Schaltfolgen   */
                         (sizeof (PfadKopfElem));

char         pepname [50] = "";                 /* Name der Pepdatei         */
char         netname  [50] = "";                /* Name der Netzes           */
char         outname  [50] = "";                /* Name der Ausgabedatei     */


int          i = 0;                             /* Argumentlistenzaehler     */

bool         MinPath = True,                    /* F: es werden alle Pfade   */
                                                /*    von der Anfangs- zu    */
                                                /*    der zu erreichenden    */
                                                /*    Markierung berechnet   */
                                                /* T: es wird nur der kuer-  */
                                                /*    zeste Pfad berechnet   */
             ErrTest = False;                   /* F: es wird eine Testdatei */
                                                /*    zur Berechnung von     */
                                                /*    Schaltfolgen generiert */
                                                /* T: es wird eine Testdatei */
                                                /*    zum Testen der Erreich-*/
                                                /*    barkeit generiert      */


{
TestVergleich = VektorGleich;                   /* Vergleichsfunktion fuer   */
                                                /* in reachy.c auf Gleich-   */
                                                /* heitstest setzen          */ 
if (argc > 0)                                   /* WENN mind. 1 Argument     */
   {                                            /* ueberrgeben wurden, DANN  */
   for (i = 0; i < argc; i++)                   /* MIT jedes Argument DO     */
      {
      if (argv[i][0] == '-')                    /* WENN das 1. Zeichen ein   */
        {                                       /* '-' ist, DANN             */
         if (argv[i][1] == 'I' ||               /* WENN das 2. Zeichen ein   */
             argv[i][1] == 'i'    )             /* 'I' oder ein 'i' ist      */
            {                                   /* DANN                      */
            strcpy (pepname, argv[i]+2);        /* Name PEPdatei einlesen   */
            if (!*outname)                      /* WENN noch keine Ausgabe-  */
               {                                /* datei angeben wurde, DANN */
               strcpy (outname, argv[i]+2);     /* an den Namen der Eingabe- */
               strcpy (netname, outname);
               }                                /* RND IF (outname == "");   */
            }                                   /* END IF (2. Zeichen = I,i) */
         if (argv[i][1] == 'O' ||               /* WENN das 2. Zeichen ein   */
             argv[i][1] == 'o'    )             /* 'O' oder ein 'o' ist      */
            {                                   /* DANN                      */
            strcpy (outname, argv[i]+2);        /* Ausgabedateinamen kopieren*/
            strcpy (netname, outname);
            }                                   /* END IF (2. Zeichen = I,i) */

         if (argv[i][1] == 'A' ||               /* WENN das 2. Zeichen eine  */
             argv[i][1] == 'a'    )             /* 'A' oder ein 'a' ist      */
            {                                   /* DANN                      */
            MinPath = False;                    /* Schalter auf True setzen  */
            }                                   /* END IF (2. Zeichen = 1)   */

         if (argv[i][1] == 'E' ||               /* WENN das 2. Zeichen eine  */
             argv[i][1] == 'e'    )             /* 'E' oder ein 'e' ist      */
            {                                   /* DANN                      */
            ErrTest = True;                     /* Schalter auf True setzen  */
            }                                   /* END IF (2. Zeichen = 1)   */
         }                                      /* END IF (1. zeichen = -)   */
      }                                         /* END FOR (jedes Argument)  */
   }                                            /* END IF (argc >= 3)        */
else                                            /* ANDERNFALLS               */
   {                                            /* meckern, da zuwenig Argu- */
   ShowHelp (*argv);                            /* mente uebergeben          */
   exit (1);                                    /* Programmabbruch           */
   }                                            /* END ELSE (argc >= 3)      */

if (*pepname)                                   /* WENN PEPdateiname ange-   */
   {                                            /* geben wurde DANN          */
   if (!(infile = fopen (pepname, "r")))        /* WENN Pepdatei !existiert  */
      {                                         /* DANN                      */
      fclose (infile);
      fprintf (stderr, "Could not read file '%s'!\n", pepname);
      ShowHelp (argv[0]);
      exit (1);                                 /* Programm abbrechen        */
      }                                         /* END ELSE (Pepdatei exist) */
   else                                         /* ANDERNFALLS               */
      {                                         /* Pepdatei existiert        */
      fclose (infile);                          /* Datei schliessen und      */
      Netz = read_PEP (pepname);                /* Netz aus Pep.datei lesen  */
      }                                         /* END ELSE (Datei existiert)*/
   }                                            /* END IF (pepname)          */
else
   {
   fprintf (stderr, "No PEP file given\n");
   ShowHelp (argv[0]);
   exit (1);
   }



Mark0 = copy_start_marking (Netz);              /* Anfangsmarkierung des     */
                                                /* Netzes einlesen           */

StartVek = MarktoVek (Mark0, Netz->NumPlaces);  /* Markierungslisten in Vek- */
ReachVek = MarktoVek (Mark0, Netz->NumPlaces);  /* toren umwandeln           */

PLKopf->init = StartVek;                        /* Pfadlistenkopf initiali-  */
PLKopf->rlist = NULL;                           /* sieren                    */
PLKopf->pfade = NULL;
PLKopf->minpfad = MinPath;

#if defined (__TRACE__)                         /* Ausgabe in Datei          */

}
remove_marklist (&Mark0);                       /* Markierungslisten aus dem */
                                                /* Speicher entfernen        */


if (Netz)                                       /* Generierung von Markie-   */
   {                                            /* rungsdateien              */
   PLKopf = ComputeReach (Netz, PLKopf,         /* Erreichbarkeitsliste      */
                      ReachVek, False, False, outfile);/* aufbauen           */
   if (PLKopf->rlist)                           /* WENN sich eine Erreichbar-*/
                                                /* keitsliste aufbauen liess */
      {                                         /* DANN                      */
      CreateMarkFiles (Netz, PLKopf->rlist,     /* generiere Markierungs-    */
                       MinPath, ErrTest,        /* dateien                   */
                       pepname, netname);  
      }                                         /* END IF (RList = ...(....) */
   else                                         /* ANDERNFALLS               */
      {                                         /* Rlist konnte nicht aufge- */
      fprintf (outfile, "Net '%s is unbounded\n", netname);
/* baut werden, da das Netz nicht beschraenkt ist     */
      }                                         /* END IF (RList = ...(....) */
   RemovePathHead (&PLKopf, Netz, outfile);     /* Speicher aufrauemen       */

   fclose (outfile);                            /* Ausgabedatei schliessen   */
   remove_net (&Netz);
   exit (0);                                    /* Programm beenden          */
   }                                            /* ENDe Generierung der      */
                                                /* Markierungsdateien        */

#else                                           /* NORMALE AUSGABE           */

remove_marklist (&Mark0);                       /* Markierungsliste aus dem  */
                                                /* Speicher entfernen        */


if (Netz)                                       /* Generierung von Markie-   */
   {                                            /* rungsdateien              */
   if ((PLKopf = ComputeReach (Netz, PLKopf,    /* WENN eine Erreichbarkeits-*/
                       ReachVek, False, False)))/* liste sich aufbauen laesst*/
      {                                         /* DANN                      */
      CreateMarkFiles (Netz, PLKopf->rlist,     /* generiere Testdatei sowie */
                       MinPath, ErrTest,        /* die Markierungsdateien    */
                       pepname, netname);
      }                                         /* END IF (RList = ...(....) */
   else                                         /* ANDERNFALLS               */
      {                                         /* Rlist konnte nicht aufge- */
      fprintf (outfile, "Net '%s is unbounded\n", netname);
/* baut werden, da das Netz nicht beschraenkt ist     */
      }                                         /* END IF (RList = ...(....) */
   RemovePathHead (&PLKopf);
   fclose (outfile);                            /* Ausgabedatei schliessen   */
   Free (PLKopf);
   remove_net (&Netz);
   exit (0);                                    /* Programm beenden          */
   }                                            /* ENDe Generierung der      */
                                                /* Markierungsdateien        */
}
#endif

fclose (outfile);
Free (StartVek);
Free (ReachVek);
remove_net (&Netz);
return (0);                                     /* ENDe ComputeSchaltfolgen  */
}                                               /* END   main()              */

