/*****************************************************************************/
/*                                                                           */
/*   PRAKTIKUM ANALYSE DER PETRI-NETZE [ ERREICHBARKEITS-ANALYSE ]   SS 1993 */
/*   VERBESSERTE VERSION MIT DER NEUEN DATENSTRUKTUR FUER MATRIXOPERATIONEN  */
/*                                                                           */
/*   BEHJAT SAMANGHOON   FARZAD EGHTESSADI                                   */
/*                                                                           */
/*   SCHALTFOLGEN          by EBERHARD MEISEL          WS 1993/94  & SS 94   */
/*                                                                           */
/*****************************************************************************/
/* 22.5. 2000, CS: Changed some error messages to english */

#include <ctype.h>
#include <string.h>
#include "neunetze.h"
#include "readlib.h"
#include "bound.h"
#include "Matrix.h"
#include "rev.h"

/* BG hier hinzugefuegt: */
bool  Beschraenkt = 1;
                                                /* 2-fachverkettete Liste zur*/
typedef struct reachelement     * ReachList;    /* Darstellung einer Liste   */
                                                /* Transition\Markierung mit */
typedef struct reachelement                     /* Liste direkter Vorgaenger */
{                                               /* von Markierung            */
   ReachList       prev,                        /* Zeiger auf vorhergehendes */
                   next,                        /* bzw. naechstes Listenelem.*/
                   alter;                       /* Zeiger auf Liste von      */
                                                /* Pfadgabelungen            */
   Vektor          mark;                        /* Von Transition "trans"    */
   Vektor          trans;                       /* aktivierte Markierung     */
   int             alt;                         /* wie oft kommt "mark" vor  */
#if defined(__TRACE__)
   int             num;                         /* Nummer Reachlistenelement */
#endif
} ReachElement;

                                                /* Datenstruktur fuer Liste  */
                                                /* von Erreichbarkeitspfaden */
typedef struct pfadelement      * Pfad;         /* Darstellung der Liste ei- */
                                                /* nes Erreichbarkeitspfades */
                                                /* 2-fachverkette Liste zur  */
                                                /* Darstellung einer Liste   */
typedef struct pfadelement                      /* Marking Transition        */
{
   Pfad            prev,                        /* Zeiger auf vorhergehendes */
                   succ;                        /* bzw. auf nachfolgendes    */
                                                /* Listenelement des Pfades  */
   Vektor          tran;                        /* zu schaltende Transition  */

#if defined(__TRACE__)
   Vektor          mark;                        /* Zeiger auf Markierung,    */
   int             pnum;                        /* die "tran" aktiviert      */
#endif
} PfadElement;


typedef struct pfadlistelement  * PfadList;

typedef struct pfadlistelement
{
   Pfad            pfad;                        /* Zeiger auf einen Pfad     */
   PfadList        next;                        /* naechste Pfadliste        */
} PfadListElem;


typedef struct pfadkopfelem     * PfadKopf;     /* Zeiger auf den Kopf einer */
                                                /* Liste von Erreichbarkeits-*/
typedef struct pfadkopfelem                     /* pfaden                    */
{
   Vektor          init;                        /* enthaelt die Markierung   */
                                                /* von der aus die Erreich-  */
                                                /* barkeitsliste aufgebaut   */
                                                /* wurde                     */
   ReachList       rlist;                       /* Zeiger Reachliste; diese  */
                                                /* enthaelt die im Speicher  */
                                                /* abgelegten Markierungs- & */
                                                /* Transitionsvektoren, auf  */
   PfadList        pfade;                       /* die die in den Pfadlisten */
                                                /* abgelegten Zeiger zeigen  */
   bool            minpfad;                     /* FLAG: True, wenn kuerze-  */
                                                /*       ster Pfad berechnet */
                                                /*       werden soll         */
   bool            reached; /* erreichbare Markierung erreicht */
} PfadKopfElem;

typedef struct alternateelement * AlternateList;/* Stack alternativer Mar-   */
                                                /* kierungen                 */
typedef struct alternateelement
{
   AlternateList   prev;                        /* Zeiger auf vorhergehendes */
                                                /* Listenelement             */
   ReachList       altptr;                      /* zeigt auf reachlistenele- */
                                                /* ment, wo ein Pfad sich ga-*/
   Pfad            bpfad;                       /* belt; Beginn und Ende des */
   Pfad            epfad;                       /* gleichen Pfadteils merken */
} AlternateElem;


/******************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

bool (*TestVergleich)(Vektor, Vektor);

#else

bool (*TestVergleich)();

#endif
/******************************************************************************/


/******************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

static void TransBenennen(Transition Transitions, Transition * Transfeld)

#else

static void TransBenennen (Transitions, Transfeld)
Transition   Transitions, /* Zeiger auf Liste der Transitionen eines Netzes   */
           * Transfeld; /* Feld von Zeigern auf die Transitionen eines Netzes */

#endif

/* Die in der Transitionsliste "Transitions" gegebenen Transitionen          */
/* speichern und mit neuen Nummern versehen.                                 */
/*****************************************************************************/
{
  VCardinal    num = 1;
  Transition   transPtr, /* Laufzeiger fuer Transitionsliste setzen */
              *feldPtr = Transfeld - 1;
  /* Zeiger auf Anfang des Transitionenfeldes setzen */

  for (transPtr=Transitions; transPtr; transPtr=transPtr->nexttrans)
  {
    (*(++feldPtr) = transPtr)->numname = num++;
  }
}


/******************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

void SchreibeTransition (FILE *Datei, Vektor vek,
                         Transition *Transfeld, int fill)

#else

void SchreibeTransition (Datei, vek, Transfeld, fill)

FILE          * Datei;       /* Zeiger auf die Zieldatei. */
Vektor          vek;         /* Auszugebende Markierung   */
Transition    * Transfeld;   /* Transitionenfeldliste     */
int             fill;        /* Transitionsname wird um   */
                             /* "fill" Leerzeichen aufgefuellt. */
#endif

/*                                                                           */
/*        Prozedur, die einen Transitionsvektor in eine Datei ausgibt        */
/*                                                                           */
/*****************************************************************************/

{
  fprintf (Datei, "%-*s",  fill, Transfeld [(*vek)-1]->name);
}


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

void RemovePathList (PfadList * PList

#if defined(__TRACE__)
                    ,LMPNet     Netz,
                     FILE     * Datei
#endif
                                         )
#else

void RemovePathList (PList                      /* call-by-reference !       */

#if defined(__TRACE__)
                           , Netz, Datei
#endif
                                         )

PfadList                      * PList;      /* zu loeschende PfadListe   */

#if defined(__TRACE__)

   LMPNet                       Netz;       /* Zeiger auf markiertes Netz*/
   FILE                       * Datei;      /* Zeiger auf Ausgabedatei   */

#endif

#endif

/*                                                                           */
/*  Die Funktion "RemovePathList" entfernt eine Pathliste aus dem Speicher   */
/*                                                                           */
/*****************************************************************************/

{
  PfadList    AList = *PList,
              DList = NULL;
  Pfad        APath = NULL,
              DPath = NULL;

#if defined(__TRACE__)
  int         maxlen = 1,
              len    = 1,
              num    = 1,
              Stellenzahl = Netz->NumPlaces,
              Transzahl = Netz->NumTransitions;

  Place      *Stellenfeld = (Place *) Malloc(Stellenzahl * sizeof (Place));
  Transition *Transfeld = (Transition *)Malloc(Transzahl * sizeof (Transition)),
              Trans = Netz->Transitions;

  StellenBenennen(Netz->Places, Stellenfeld);
  TransBenennen(Netz->Transitions, Transfeld);

  maxlen=max_transitionname_length(Netz);
  fprintf (Datei, "\n\n IN REMOVEPATHLIST \n\n");
#endif

  while (AList)
  {
    APath = AList->pfad; /* Laufzeiger auf 1. Pfadelement setzen */
#if defined(__TRACE__)
    fprintf (Datei, "Remove path > %2d <\n", num++);
#endif
    while (APath)
    {
      DPath = APath;                         /* Merke Element zum Loeschen*/
#if defined(__TRACE__)
      fprintf (Datei, "                   ");
      SchreibeMarkierung (Datei,APath->mark, Stellenfeld, False);
      if (APath->tran)
      {
        fprintf (Datei, "  -- ");
        SchreibeTransition (Datei,APath->tran, Transfeld,maxlen);
        fprintf (Datei, " --> ");
      }
      fprintf (Datei, "\n");
#endif
      APath = APath->prev;
      Free (DPath);
    }
    DList = AList;
    AList = AList->next;
    Free (DList);
  }
  Free (*PList);
  *PList = NULL;

#if defined(__TRACE__)
  fprintf (Datei, "\n\n IN REMOVEREACHLIST\n\n");
#endif
} /*** END RemovePathList () ***/


/******************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

void RemoveReachList (ReachList * RList

#if defined(__TRACE__)
                     ,FILE      * Datei         /* Zeiger Ausgabedatei       */
#endif
                                       )

#else

void RemoveReachList (RList         /* call-by-reference !       */

#if defined(__TRACE__)
                     ,Datei
#endif
                           )

ReachList                       * RList;      /* zu loeschende ReachListe  */

#if defined(__TRACE__)

   FILE                         * Datei;      /* Zeiger auf Ausgabedatei   */

#endif

#endif

/*                                                                           */
/*  Die Funktion "RemoveReachList" entfernt eine Reachliste aus dem Speicher */
/*                                                                           */
/*****************************************************************************/

{
  ReachList  AReach = *RList,            /* Laufzeiger ReachListe     */
             DReach,                     /* Loeschzeiger ReachListe   */
             MReach;                     /* Merkzeiger Alternateliste */

#if defined(__TRACE__)
  fprintf (Datei, "\n\nIN REMOVE_REACHLIST\n\n");
#endif

  if (AReach)
  {
    /* Zeiger auf das Listenende setzen */
    while (AReach->next)
    {
      AReach = AReach->next;
    }

    while (AReach)
    {

#if defined(__TRACE__)
      fprintf (Datei, "Remove RL-Element >%3d <  ", AReach->num);
#endif

      if (AReach->alter)
      {
        MReach = (ReachList) Malloc(sizeof (ReachElement));
        MReach = AReach->alter;
        while (MReach)
        {
#if defined(__TRACE__)
          fprintf (Datei, "Del age  > %3d <   ", MReach->num);
#endif
          DReach = MReach; /* Das zu loeschende Element merken */
          MReach = MReach->next;
          Free (DReach);
        }
      }
      DReach = AReach;
      AReach = AReach->prev;
      Free (DReach->mark);
      Free (DReach->trans);
      Free (DReach);
#if defined(__TRACE__)
      fprintf (Datei, "RL deleted\n");
#endif
    } /* while (AReach) */
    *RList = NULL;
  }
#if defined(__TRACE__)
  fprintf (Datei, "\n\n OUT REMOVEREACHLIST\n\n");
#endif
} /*** RemoveReachList() ***/


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

void RemovePathHead (PfadKopf * PLKopf          /* zu loeschende PfadListe   */

#if defined(__TRACE__)
                    ,LMPNet     Netz,
                     FILE     * Datei
#endif
                                         )
#else

void RemovePathHead (PLKopf                     /* call-by-reference !       */

   #if defined(__TRACE__)
                               ,Netz,Datei
   #endif
                                         )

PfadKopf                      * PLKopf  ;       /* zu loeschende PfadListe   */

#if defined(__TRACE__)

   LMPNet                       Netz     ;      /* Zeiger auf markiertes Netz*/
   FILE                       * Datei    ;      /* Zeiger auf Ausgabedatei   */

#endif

#endif

/*                                                                           */
/*  Die Funktion "RemovePathHead" entfernt einen PathHead aus dem Speicher   */
/*                                                                           */
/*****************************************************************************/

{
  ReachList   AReach = (*PLKopf)->rlist;
  PfadList    AList  = (*PLKopf)->pfade;

#if defined(__TRACE__)
  RemovePathList (&AList, Netz, Datei); /* Entferne Pathliste        */
  RemoveReachList (&AReach, Datei);     /* Entferne Reachliste       */
#else
  RemovePathList (&AList);              /* Entferne Pathliste        */
  RemoveReachList (&AReach);            /* Entferne Reachliste       */
#endif

  Free (*PLKopf); /* Zeiger auf PathList entfernen und auf NULL setzen */
  *PLKopf = NULL;
} /*** RemovePathHead() ***/


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

PfadKopf ComputePath  (PfadKopf   PKopf  ,      /* enthaelt Zeiger auf Er-   */
                                                /* reichbarkeitsliste        */
                       Vektor     ReachVek,     /* zu erreichende Markierung */
                       bool       Minpfad ,     /* Flag: gibt an ob kuerze-  */
                                                /* Pfad oder alle moeglichen */
                                                /* berechnet werden soll     */
                       bool       TeilMark      /* Flag: Wenn True, werden   */
                                                /* Pfade zu Teilmarkierungen */
                                                /* berechnet                 */

#if defined(__TRACE__)
                      ,LMPNet     Netz    ,     /* Zeiger markiertes Netz    */
                       FILE     * Datei         /* Zeiger auf Ausgabedatei   */
#endif
                                           )
#else

PfadKopf ComputePath  (PKopf, ReachVek, Minpfad, TeilMark

#if defined(__TRACE__)
                                 ,Netz    ,
                                  Datei
#endif
                                           )

PfadKopf                          PKopf   ;     /* Zeiger Reachliste         */
Vektor                            ReachVek;     /* zu erreichende Markierung */
bool                              Minpfad ;     /* Flag: gibt an ob kuerze-  */
bool                              TeilMark;     /* Flag: Wenn True, werden   */
                                                /* Pfade zu Teilmarkierungen */
                                                /* berechnet                 */

                                                /* Pfad oder alle moeglichen */
                                                /* berechnet werden soll     */
#if defined(__TRACE__)

   LMPNet                         Netz    ;     /* Zeiger markiertes Netz    */
   FILE                         * Datei   ;     /* Zeiger auf Ausgabedatei   */

#endif

#endif

/*                                                                           */
/*  Die Funktion "ComputePath" erzeugt eine Liste aller moeglichen Erreich-  */
/*  barkeitspfade, die von der Anfangsmarkierung "StartVek" zu einer zu er-  */
/*  reichenden Markierung "ReachVek" fuehren, von folgender Gestalt.         */
/*                                                                           */
/*   +-- Rueckgabewert der Funktion "ComputePath"                            */
/*   |                                                                       */
/*   |    Die Kopfelemente, die jeder auf einen der n Pfade zeigen           */
/*   |             |                                                         */
/*   |      +------+------------+------....------+----....----+              */
/*   |      |                   |                |            |              */
/*   |    Pfad 1              Pfad 2           Pfad i       Pfad n           */
/*   |                                                                       */
/*   |  +------+             +------+                      +------+          */
/*   +->| next |------------>| next |--------> ..... ----->| next | --> NULL */
/*      +------+             +------+                      +------+          */
/*      | Pfad |-----+       | Pfad |-----+                | Pfad |-----+    */
/*      +------+     |       +------+     |                +------+     |    */
/*                   |                    |                             |    */
/*                   |                    |                             |    */
/* erreichbare    N  |  erreichbare    N  |           erreichbare    N  |    */
/* Markierung     U  |  Markierung     U  |    .....  Markierung     U  |    */
/*                L  |                 L  |                          L  |    */
/*      +------+  L  |       +------+  L  |                +------+  L  |    */
/*      |      |--+  |       |      |--+  |                |      |--+  |    */
/*      | prev |     |       | prev |     |                | prev |     |    */
/*      |      |<-+  |       |      |<-+  |                |      |<-+  |    */
/*      +------+  |  |       +------+  |  |                +------+  |  |    */
/*      | mark |  |  |       | mark |  |  |                | mark |  |  |    */
/*      +------+  |  |       +------+  |  |    .....       +------+  |  |    */
/*      | tran |  |  |       | tran |  |  |                | tran |  |  |    */
/*      +------+  |  |       +------+  |  |                +------+  |  |    */
/*      |      |  |  |       |      |  |  |                |      |  |  |    */
/*   +--| succ |  |  |    +--| succ |  |  |             +--| succ |  |  |    */
/*   |  |      |  |  |    |  |      |  |  |             |  |      |  |  |    */
/*   |  +------+  |  |    |  +------+  |  |             |  +------+  |  |    */
/*   |            |  |    |            |  |             |            |  |    */
/*   :            :  :    :            :  :             :            :  :    */
/*   :            :  :    :            :  :    .....    :            :  :    */
/*   :            :  :    :            :  :             :            :  :    */
/*   |            |  |    |            |  |             |            |  |    */
/*   |  +------+  |  |    |  +------+  |  |             |  +------+  |  |    */
/*   |  |      |--+  |    |  |      |--+  |             |  |      |--+  |    */
/*   |  | prev |     |    |  | prev |     |             |  | prev |     |    */
/*   |  |      |<-+  |    |  |      |<-+  |             |  |      |<-+  |    */
/*   |  +------+  |  |    |  +------+  |  |             |  +------+  |  |    */
/*   |  | mark |  |  |    |  | mark |  |  |             |  | mark |  |  |    */
/*   |  +------+  |  |    |  +------+  |  |    .....    |  +------+  |  |    */
/*   |  | tran |  |  |    |  | tran |  |  |             |  | tran |  |  |    */
/*   |  +------+  |  |    |  +------+  |  |             |  +------+  |  |    */
/*   +->|      |  |  |    +->|      |  |  |             +->|      |  |  |    */
/*      | succ |  |  |       | succ |  |  |                | succ |  |  |    */
/*   +--|      |  |  |    +--|      |  |  |             +--|      |  |  |    */
/*   |  +------+  |  |    |  +------+  |  |             |  +------+  |  |    */
/*   |            |  |    |            |  |             |            |  |    */
/*   |            |  |    |            |  |             |            |  |    */
/*   |  +------+  |  |    |  +------+  |  |             |  +------+  |  |    */
/*   |  |      |--+  |    |  |      |--+  |             |  |      |--+  |    */
/*   |  | prev |     |    |  | prev |     |             |  | prev |     |    */
/*   |  |      |     |    |  |      |     |             |  |      |     |    */
/*   |  +------+     |    |  +------+     |             |  +------+     |    */
/*   |  | mark |     |    |  | mark |     |             |  | mark |     |    */
/*   |  +------+<----+    |  +------+<----+    .....    |  +------+<----+    */
/*   |  | tran |----+     |  | tran |----+              |  | tran |----+     */
/*   |  +------+    |     |  +------+    |              |  +------+    |     */
/*   |  |      |    |     |  |      |    |              |  |      |    |     */
/*   +->| succ |-+  |     +->| succ |-+  |              +->| succ |-+  |     */
/*      |      | |  |        |      | |  |                 |      | |  |     */
/*      +------+ |  |        +------+ |  |                 +------+ |  |     */
/*               |  |                 |  |                          |  |     */
/* Anfangs-      v  v   Anfangs-      v  v            Anfangs-      v  v     */
/* markierung    NULL   markierung    NULL     .....  markierung    NULL     */
/*                                                                           */
/*                                                                           */
/* Die Pfadlisten werden rueckwaerts verzeigert, so dass die Startmarkierung */
/* eines Erreichbarkeitspfades, die zuletzt in diesen Pfad eingetragen wird, */
/* am Anfang des Liste steht, und die erreichbare Markierung, die zuerst in  */
/* diesen Pfad eingetragen wird, am Ende der Liste steht.                    */
/*                                                                           */
/* Die Vorwaertsverzeigerung ist noetig zum Kopieren von den gleichen Pfad-  */
/* teilen zweier Pfade von der erreichbaren Markierung bis zur Pfadgabelung  */
/*                                                                           */
/*                                                                           */
/* Die uebergebenen Parameter                                                */
/*                                                                           */
/*  PKopf    enthaelt Zeiger auf Erreichbarkeitsliste                        */
/*  Reachvek enthaelt die zu erreichende Markierung in Vektorform            */
/*                                                                           */
/*  Minpfad  FLAG, wenn True, wird nur der kuerzeste Pfad berechnet          */
/*  TeilMark FLAG, wenn True, werden Pfade zu Teilmarkierungen berechnet     */
/*                                                                           */
/* Fuer Ausgabzwecke werden mit der Compilerdefinition (__TRACE__) aktiviert:*/
/*                                                                           */
/*  Netz     Zeiger vom Typ "LMPNet" zeigt auf das zu bearbeitende Netz      */
/*                                                                           */
/*  Datei    Zeiger auf eine Ausgabedatei                                    */
/*                                                                           */
/*****************************************************************************/

{
PfadList      NList  = NULL,                    /* Erreichbarkeitspfadliste  */
              AList  = NULL;                    /* Zeiger auf dito           */

Pfad          NPath  = NULL,                    /* (N)eues Pfadelement       */
              FPath  = NULL,                    /* (E)rstes Pfadelement      */
              APath  = NULL,                    /* (A)ktuelles Pfadelement   */
              HPath  = NULL;                    /* (H)ilfslaufzeiger beim    */
                                                /* kopieren des Pfades       */

AlternateList NAlt   = NULL,                    /* (N)eues AL'element        */
              AAlt   = NULL,                    /* (A)ktuelles AL'element    */
              DAlt   = NULL;                    /* zu loeschendes AL'elemnent*/

ReachList     AReach = PKopf->rlist,            /* (A)ktuelles RL'element    */
              LReach = PKopf->rlist,            /* Zeiger Reachlistenende    */
              MReach = (ReachList) Malloc       /* zum Merken des Anfangs ei-*/
                        (sizeof (ReachElement));/* ner Reachlistenalternativ-*/
                                                /* liste                     */
#if defined(__TRACE__)

int           PathNo = 1,                       /* Zaehlt Anzahl der Pfade   */
              maxlen = 1,
              len    = 1;

int           Stellenzahl = Netz->NumPlaces,    /* Ermittle Anzahl Stellen   */
              Transzahl = Netz->NumTransitions; /* Ermittle # Transitionen   */

Place       * Stellenfeld = (Place *) Malloc    /* Allokiere Speicher fuer   */
                 (Stellenzahl * sizeof (Place));/* Zeigerfeld auf die Stel-  */
                                                /* len des Netzes "Netz"     */

Transition  * Transfeld = (Transition *) Malloc /* Allokiere Speicher fuer   */
              (Transzahl * sizeof (Transition)),/* Zeigerfeld auf die Tran-  */
                                                /* sitionen des Netzes "Netz"*/
              Trans = Netz->Transitions;        /* Zeiger auf Transitionen   */
                                                /* des Netzes                */

                                                 
StellenBenennen (Netz->Places, Stellenfeld);    /* Stellen und Transitionen  */
TransBenennen (Netz->Transitions, Transfeld);   /* temporaer umbenennen      */

maxlen=max_transitionname_length(Netz);
fprintf (Datei, "\n\n  IN COMPUTEPATH\n\n");

#endif

if (TeilMark)                                   /* Wenn Teilmarkierungen     */
   {
   TestVergleich = VEnthalten;                  /* dann Markierungen auf     */
   }                                            /* Enthaltensein testen      */
else                                            /* ANDERNFALLS               */
   {                                            /* Markierungen auf Gleich-  */
   TestVergleich = VektorGleich;                /* heit testen               */
   } 
MReach->prev = MReach->next = NULL;             /* !!! Programmbeginn !!!    */
MReach->alter = NULL;                           /* Initialisierungen         */

while (!TestVergleich (ReachVek, AReach->mark)  /* letzte erreichbare Markie-*/
           &&    AReach->prev                )
   {                                            /* rung in der Erreichbar-   */
   LReach = AReach = AReach->prev;              /* keitsliste suchen         */
   }                                            /* END WHILE (!AReach->reach)*/
                                                /* Algorithmus Hauptschleife */
while (LReach)                                  /* SOLANGE es noch eine er-  */
   {                                            /* reichbare Markierung gibt */
   while (AReach)                               /* DO SOLANGE es noch einen  */
      {                                         /* Alternativen Pfad gibt DO */
      NPath = (Pfad) Malloc                     /* Pfadlistenelement eintra- */
                     (sizeof (PfadElement));    /* gen Speicher allozieren & */
#if defined (__TRACE__)
      NPath->mark = AReach->mark;               /* die neuen Werte samt ih-  */
#endif
                                                /* rer Transition ins neue   */
      NPath->tran = AReach->trans;              /* Listenelement eintragen   */
                                                /* und wie folgt verzeigern  */
      if (APath == NULL)                        /* WENN Laufzeiger leer ist, */
         {                                      /* DANN diesen auf das neue  */
         APath       = NPath;                   /* ELement zeigen lassen und */
         APath->succ = NULL;                    /* Nachfolger und Vorgaenger */
         APath->prev = NULL;                    /* auf NULL setzen sowie das */
         FPath       = APath;                   /* 1. Pfadelement merken     */

#if defined(__TRACE__)
         fprintf (Datei, "  reachable marking");
         fprintf (Datei, " path %d :\n", PathNo);
#endif

         }                                      /* END IF (PPath == NULL)    */
      else                                      /* ANDERNFALLS die Liste     */
         {                                      /* nicht leer ist, den Nach- */
         APath->succ = NPath;                   /* folger des akt. Elem. auf */
         NPath->prev = APath;                   /* das neue, den Vorgaenger  */
         NPath->succ = NULL;                    /* des neuen auf akt. Elem.  */
         APath       = NPath;                   /* akt. EL. = Neu El. setzen */
         }                                      /* END ELSE (PPath == NULL)  */

#if defined(__TRACE__)
      fprintf (Datei, "\n  ");
      SchreibeMarkierung (Datei, APath->mark,   /* Makierung ausgeben        */
                          Stellenfeld, False);
      if (APath->tran)
         {
         fprintf (Datei, "  -- ");              /* Transition ausgeben       */
         SchreibeTransition (Datei, APath->tran,
                             Transfeld, maxlen);
         fprintf (Datei, " -->   ");
         }
#endif

      if (AReach->alter && !Minpfad)            /* WENN Alternativliste vor- */
         {                                      /* handen && und alle Pfade  */
         MReach->alter = AReach->alter;         /* berechnet werden sollen   */
                                                /* DANN alten Vorg. merken   */
         while (AReach->alter->next)            /* WENN alternativer Pfad    */
            {                                   /* DANN alternatelistenele-  */
            NAlt = (AlternateList) Malloc       /* ment erzeugen und Zeiger  */
                    (sizeof (AlternateElem));   /* auf die naechste Alterna- */
            AReach->alter = AReach->alter->next;/* tive setzen, deren Adresse*/
            NAlt->altptr  = AReach->alter->alter;/* sowie Ende und Anfang des*/
            NAlt->epfad   = APath;              /* bisherigen Pfades merken  */
            NAlt->bpfad   = FPath;              /* und mit A-liste verzeigern*/
            if (AAlt == NULL)                   /* WENN Laufzeiger leer ist, */
               {                                /* DANN diesen auf das neue  */
               AAlt       = NAlt;               /* ELement zeigen lassen und */
               AAlt->prev = NULL;               /* Vorgaenger auf NULL setzen*/
               }                                /* END IF (PPath == NULL)    */
            else                                /* ANDERNFALLS Laufzeiger.   */
               {                                /* Vorgaenger auf neue Kante */
               NAlt->prev = AAlt;               /* es Element setzen         */
               AAlt       = NAlt;
               }                                /* END ELSE (PAlt == NULL)   */

#if defined(__TRACE__)
            if (AReach->alter && 
                AReach->alter->alter)
               {                                /* Nummer der Vorgaengermar- */
               fprintf (Datei, "AP -> %d <-  ", /* kierung ausgeben , auf    */
                      AReach->alter->alter->num);/* die gezeigt wird         */
               }                                /* END IF (Vorg.Nr. ausgeben)*/
#endif

            }
         AReach->alter = MReach->alter;         /* alten Vorgaenger wieder   */
         }                                      /* restaurieren              */
      if (AReach->trans)                        /* RL-element mit Startmark. */
         {                                      /* hat keine Transition      */
         AReach = AReach->alter->alter;         /* direkten Vorgaenger setzen*/
         }                                      /* END IF (AReach->trans)    */
      else                                      /* ANDERNFALLS SM erreicht!!!*/
         {                                      /* Pfad ist nun vollstaendig */
         AReach = AReach->prev;                 /* AReach auf NULL setzen    */
         NList = (PfadList) Malloc              /* darum neues Pfadlisten-   */
                        (sizeof (PfadListElem));/* element anlegen, um fer-  */
         NList->pfad = APath;                   /* tigen Pfad in die Pfadli- */
         NList->next = NULL;                    /* ste einhaengen. Anschlies-*/
                                                /* send Pfadliste ordnungs-  */
         if (AList == NULL)                     /* gemaess verzeigern        */
            {                                   /* sowie Anfang der Pfad-    */
            PKopf->pfade = AList = NList;       /* liste merken              */
            if (Minpfad)                        /* WENN nur ein Pfad berech- */
               {                                /* net werden soll, DANN     */
               break;                           /* BREAK (WHILE (LReach))    */
               }                                /* END IF (Minpfad)          */
            }                                   /* END IF (Alist == NULL)    */
         else                                   /* ANDERNFALLS               */
            {                                   /* Pfad an das Pfadlisten-   */
            AList->next = NList;                /* ende anfuegen und als neu-*/
            AList       = NList;                /* es Listenende nehmen      */
            }                                   /* END ELSE (AList == NULL)  */

#if defined(__TRACE__)
         fprintf (Datei, "\n\n  Inital Marking\n\n");
#endif

         APath = NULL;                          /* Zeiger fuer neuen Pfad    */
                                                /* auf NULL setzen           */

         if (AAlt != NULL)                      /* WENN eine alternativer    */
            {                                   /* Pfad zu basteln ist DANN  */
            HPath = AAlt->bpfad;                /* Kopiere Pfad vom Anfang   */
            while (HPath->prev != AAlt->epfad)  /* bis zur in AAlt->EPfad    */
               {                                /* gemerkten Stelle in APath */
               NPath = (Pfad) Malloc            /* also Speicher allokieren  */
                         (sizeof (PfadElement));/* kieren und alle Werte von */

#if defined (__TRACE__)
               NPath->mark = HPath->mark;       /* HPath in NPath eintragen  */
#endif

               NPath->tran = HPath->tran;       /* und verzeigern            */
               if (APath == NULL)               /* WENN Laufzeiger leer ist, */
                  {                             /* DANN diesen auf das neue  */
                  APath       = NPath;          /* ELement zeigen lassen und */
                  APath->succ = NULL;           /* Nachfolger und Vorgaenger */
                  APath->prev = NULL;           /* auf NULL setzen           */
                  FPath       = APath;

#if defined(__TRACE__)
                  fprintf (Datei, "  reachable Marking");
                  fprintf (Datei, " path %d :\n\n", PathNo);
#endif
               }
               else                             /* ANDERNFALLS die Liste     */
                  {                             /* nicht leer ist, den Nach- */
                  APath->succ = NPath;          /* folger des akt. Elem. auf */
                  NPath->prev = APath;          /* das neue, den Vorgaenger  */
                  NPath->succ = NULL;           /* des neuen auf akt. Elem.  */
                  APath       = NPath;          /* akt. EL. = Neu El. setzen */
                  }                             /* END ELSE (PPath == NULL)  */

#if defined(__TRACE__)
               fprintf (Datei, "  ");
               SchreibeMarkierung (Datei, APath->
                       mark, Stellenfeld, False);
               if (APath->tran)
                  {
                  fprintf (Datei, "  -- ");
                  SchreibeTransition (Datei, APath
                       ->tran, Transfeld, maxlen);
                  fprintf (Datei, " --> \n");
                  }
#endif

               HPath = HPath->succ;             /* naechstes Pfadelement     */
               }                                /* END WHILE (Pfadkopieren)  */
                                                /* neues Areach setzen, um   */
            AReach = AAlt->altptr;              /* alternativen Pfad zu      */
                                                /* vervollstaendigen.        */
            DAlt = AAlt;                        /* aktuelles letztes Alter-  */
            AAlt = AAlt->prev;                  /* nativlistenelement loe-   */
            Free (DAlt);                        /* schen                     */
            }                                   /* END IF (AAlt != NULL)     */
         }                                      /* END IF (AReach == NULL)   */
      }                                         /* END WHILE (AReach)        */

   LReach = LReach->prev;                       /* suche alternative er-     */
   if (LReach                               &&  /* reichbare Markierung,     */
       TestVergleich (ReachVek, LReach->mark)  )/* WENN vorhanden            */
      {                                         /* DANN setze aktuellen Zei- */
      AReach = LReach;                          /* ger auf dieses Reach-     */
      }                                         /* listenelement             */
   }                                            /* END WHILE (LReach)        */
Free (MReach);                                  /* Speicher freigeben        */

#if defined(__TRACE__)
  Free (Stellenfeld);
  Free (Transfeld);
  fprintf (Datei, "\n  OUT COMPUTEPATH\n");
#endif

return (PKopf);                                 /* Ergebnisliste zurueck     */
} /***  ComputePath()  ***/



#if defined(__TRACE__)

/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

void WriteReachList (LMPNet      Netz    ,      /* Zeiger markiertes Netz    */
                     ReachList   RList   ,      /* auszugebende ReachListe   */
                     Vektor      ReachVek,      /* zu erreichende Markierung */
                     FILE      * Datei    )
#else

void WriteReachList (Netz, RList, ReachVek, Datei)

LMPNet                          Netz;           /* Zeiger markiertes Netz    */

ReachList                       RList;          /* Zeiger auf Erreichbar-    */
                                                /* keitsliste                */
Vektor                          ReachVek;       /* zu erreichende Markierung */

FILE                           * Datei;         /* Zeiger auf Ausgabedatei   */

#endif                                          /* END STDC & C++ DEFINITION */

/*                                                                           */
/*  Die Funktion "WriteReachList" gibt eine Reachliste aus                   */
/*                                                                           */
/*****************************************************************************/

{
ReachList    AReach = RList,                    /* Laufzeiger ReachListe     */
             MReach = (ReachList) Malloc        /* Laufzeiger fuer die Liste */
                        (sizeof (ReachElement));/* der alternativen Vorgaen- */
                                                /* germarkierungen           */
int          maxlen = 1,
             len    = 1;

VCardinal    Stellenzahl = Netz->NumPlaces,     /* Ermittle Anzahl Stellen   */
             Transzahl = Netz->NumTransitions;  /* Ermittle # Transitionen   */

Place      * Stellenfeld = (Place *) Malloc     /* Allokiere Speicher fuer   */
                (Stellenzahl * sizeof (Place)); /* Zeigerfeld auf die Stel-  */
                                                /* len des Netzes "Netz"     */

Transition * Transfeld = (Transition *) Malloc  /* Allokiere Speicher fuer   */
              (Transzahl * sizeof (Transition)),/* Zeigerfeld auf die Tran-  */
                                                /* sitionen des Netzes "Netz"*/
             Trans = Netz->Transitions;         /* Zeiger auf Transitionen   */
                                                /* des Netzes                */


StellenBenennen (Netz->Places, Stellenfeld);    /* Stellen und Transitionen  */
TransBenennen (Netz->Transitions, Transfeld);   /* temporaer umbenennen      */

maxlen=max_transitionname_length(Netz);
fprintf (Datei, "\n\n\n\nReachability list\n\n");

MReach->prev = MReach->next = NULL;
MReach->alter = NULL;

while (AReach->prev)
   {
   AReach = AReach->prev;                       /* Laufzeiger auf vorheriges */
   }

while (AReach)                                  /* WEIL ReachListenelement   */
   {                                            /* noch vorhanden DO         */
   if (VektorGleich (AReach->mark, ReachVek))
      {
      fprintf (Datei, "Reachable Marking ");
      }
   else
      {
      fprintf (Datei, "                  ");
      }

   fprintf (Datei, "Nr: > %3d <  ", AReach->num);
   SchreibeMarkierung (Datei, AReach->mark,
                       Stellenfeld, False);
   if (AReach->trans)
      {
      fprintf (Datei, "   <- ");
      SchreibeTransition (Datei, AReach->trans,
                          Transfeld, maxlen);
      fprintf (Datei, " -  ");
      }
   else
      {
      fprintf (Datei, "\nInitial marking\n");
      }

   MReach->alter = AReach->alter;
   while (MReach->alter)                        /* WEIL Allternativliste     */
      {                                         /* noch vorhanden DO         */
      fprintf (Datei, "%2d",
               MReach->alter->alter->num);
      MReach->alter = MReach->alter->next;      /* Laufzeiger auf naechstes  */
                                                /* Listenelement setzen      */
      if (MReach->alter)
         {
         fprintf (Datei, "  ,  ");
         }
      else
         {
         fprintf (Datei, "\n\n");
         }
      }                                         /* END WHILE (AReach->alter) */
   AReach = AReach->next;                       /* Laufzeiger auf vorheriges */
                                                /* Listenelement setzen      */
   }                                            /* END WHILE (AReach)        */
fprintf (Datei, "\n\n\n");
Free (Transfeld);                               /* und nicht mehr benoetig-  */
Free (Stellenfeld);                             /* ten Speicher freigeben    */
Free (MReach);                                  /* gemerktes Element loeschen*/
} /*** WriteReachList() ***/

#endif



/******************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

PfadKopf ComputeReach (LMPNet   Netz      ,     /* zu untersuchendes Netz    */
                       PfadKopf PKopf     ,     /* Enthaelt Zeiger Erreich-  */
                                                /* barkeitsliste und StartM. */
                       Vektor   ReachVek  ,     /* zu erreichende Markierung */
                       bool     Minpfad   ,     /* Flag: gibt an ob kuerze-  */
                                                /* Pfad oder alle moeglichen */
                                                /* berechnet werden soll     */
                       bool     TeilMark        /* Flag: gibt an ob Pfade zu */
                                                /* Teilmarkierungen          */
                                                /* berechnet werden sollen   */


#if defined(__TRACE__)
                                           ,
                       FILE   * Datei           /* Zeiger auf Ausgabedatei   */
#endif
                                           )
#else

PfadKopf ComputeReach (Netz, PKopf, ReachVek, Minpfad, TeilMark

#if defined(__TRACE__)
                    ,Datei
#endif
                                          )

LMPNet               Netz      ;                /* zu untersuchendes Netz    */
PfadKopf             PKopf     ;                /* Enthaelt Zeiger Erreich-  */
                                                /* barkeitsliste und StartM. */
Vektor               ReachVek  ;                /* zu erreichende Markierung */
bool                 Minpfad   ;                /* Flag: gibt an ob kuerze-  */
                                                /* Pfad oder alle moeglichen */
                                                /* berechnet werden soll     */
bool                 TeilMark  ;                /* Flag: gibt an ob Pfade zu */
                                                /* Teilmarkierungen          */
                                                /* berechnet werden sollen   */

#if defined(__TRACE__)
   FILE            * Datei;                     /* Zeiger auf Ausgabedatei   */
#endif

#endif

/*                                                                           */
/*  Funktion zur Berechnung der Erreichbarkeitspfade von der Startmarkierung */
/*  eines Systems "Netz" zu einer erreichbaren Markierung "EMark".           */
/*                                                                           */
/*   Die Funktion "Erreichbarkeitsliste" legt nach dem in der Funktion       */
/*   "Ueberdeckungsgraph" beschriebenen Algorithmus eine Liste vom Typ       */
/*   "ReachList" an.                                                         */
/*                                                                           */
/*      +---------+                                                          */
/*      |         |--->NULL                                                  */
/*      |  prev   |                                                          */
/*      |         |<--+                                                      */
/*      +---------+   |                                                      */
/*      |  mark   |---|-------->  Startmarkierung                            */
/*      +---------+   |                                                      */
/*      |  alte   |===|========> NULL                                        */
/*      +---------+   |                                                      */
/*      |  tran   |   |                                                      */
/*      +---------+   |                                                      */
/*      |         |   |                                                      */
/*  +---|  next   |   |                                                      */
/*  |   |         |   |                                                      */
/*  |   +---------+   |                                                      */
/*  |                 |         Liste vom Typ "reachlist", enthaelt die      */
/*  |                 |         Adressen auf die direkten Vorgaenger-        */
/*  |   +---------+   |         markierungen der Markierung "mark"           */
/*  |   |         |---+                                                      */
/*  |   |  prev   |             +---------+                                  */
/*  |   |         |<--+  NULL<--|  prev   |                                  */
/*  |   +---------+   |         +---------+                                  */
/*  |   |  mark   |   |         |  mark   |-->NULL                           */
/*  |   +---------+   |         +---------+                                  */
/*  |   |  alter  |===|========>|  alter  |---> Startmarkierung              */
/*  |   +---------+   |         +---------+                                  */
/*  |   |  tran   |   |         |  tran   |-->NULL                           */
/*  |   +---------+   |         +---------+                                  */
/*  +-->|         |   |         |  next   |-->NULL                           */
/*      |  succ   |   |         +---------+                                  */
/*  +---|         |   |                                                      */
/*  |   +---------+   |                                                      */
/*  |                 |                                                      */
/*                                                                           */
/*  :                 :                                                      */
/*  :                 :                                                      */
/*  :                 :                                                      */
/*                                                                           */
/*  |                 |                                                      */
/*  |   +---------+   |                                                      */
/*  |   |         |---+                                                      */
/*  |   |  prev   |             +---------+              +---------+         */
/*  |   |         |     NULL<---|  prev   |<--- .... <---|  prev   |         */
/*  |   +---------+             +---------+              +---------+         */
/*  |   |  mark   |             |  mark   |--->NULL      |  mark   |--->NULL */
/*  |   +---------+             +---------+              +---------+         */
/*  |   |  alter  |============>|  alter  |              |  alter  |         */
/*  |   +---------+             +---------+              +---------+         */
/*  |   |  tran   |             |  tran   |--->NULL      |  tran   |--->NULL */
/*  |   +---------+             +---------+              +---------+         */
/*  +-->|         |             |  next   |---> .... --->|  next   |--->NULL */
/*      |  succ   |             +---------+              +---------+         */
/*      |         |--->NULL                                                  */
/*      +---------+                                                          */
/*                                                                           */
/*    Der Rueckgabewert der Funktion "ComputeReach" "AReach" ist der Zeiger  */
/*    auf das letzte Element der Erreichbarkeitsliste                        */
/*                                                                           */
/*****************************************************************************/

{
KnotenMenge     v          = NULL,              /* Markierungsknoten         */
                vQuer      = NULL,              /* aktuelle Markierung       */
                vStrichPtr = NULL,              /* Folgemarkierung           */
                VE         = NULL;              /* Zeiger auf Anfang des     */
                                                /* Ueberdeckungsgraphen      */
PfadKopf        pkopf      = PKopf;             /* Rueckgabewert             */

ReachList       NReach     = NULL,              /* Zeiger auf (N)eues,       */
                AReach     = NULL,              /* (A)ktuelles und           */
                VReach     = NULL,              /* (V)erzweigungsliste       */
                BReach     = NULL,              /* (B)eginn Verzweigungsliste*/
                ZReach     = NULL;              /* zum (Z)aehlen der Vorkom- */
                                                /* men einer Markierung      */
VorgaengerListe VorgPtr    = NULL;              /* Vorgaengerliste einer     */
                                                /* Markierung                */
VCardinal       Szahl     = Netz->NumPlaces,    /* Ermittle Anzahl der Stel- */
                Tzahl     = Netz->NumTransitions;/* len und die Anzahl der   */
                                                /* Transitionen              */
Vektor          NV        = NeuerVektor (Szahl),/* Neuer MarkierungsVektor   */
                vStrich   = NeuerVektor (Szahl),/* aus v" := v + N(t) ge-    */
                                                /* bildete Folgemarkierung   */
                SchaltVek = NeuerVektor (Szahl),/* Markierung, die aus dem   */
                                                /* Schalten von Transition   */
                                                /* "t" folgt                 */
                HilfVek   = PKopf->init,        /* Hilfsvektor fuer Berech-  */
                                                /* nungen, zuerst Startmark. */
                HilfPtr   = NULL;               /* Hilfszeiger zum Erzeugen  */
                                                /* eines Transitionsvektors  */
Place         * Stellenfeld = (Place *) Malloc  /* Allokiere Speicher fuer   */
                   (Szahl * sizeof (Place));    /* Zeigerfeld auf die Stel-  */
                                                /* len des Netzes "Netz"     */
Transition      Trans,
              * Transfeld = (Transition *) Malloc/* Allokiere Speicher fuer  */
                   (Tzahl * sizeof(Transition));/* Zeigerfeld auf die Tran-  */
                                                /* sitionen des Netzes "Netz"*/
  bool          reachmarkreached = False;       /* Flag: ist zu erreichende  */
                                                /* Markierung erreicht?      */

#if defined(__TRACE__)
  int           rlnum     = 0,
                len       = 1,
                maxlen    = 1;
#endif
  if (TeilMark)
  {
    TestVergleich = VEnthalten;
  }
  else
  {
    TestVergleich = VektorGleich;
  } 

#if defined(__TRACE__)
  fprintf (Datei, "\n\n IN COMPUTEREACH \n");
  fprintf (Datei, "\n Generate");
  fprintf (Datei, Minpfad ? "short" : "complete");
  fprintf (Datei, " Reachability list\n\n");

  maxlen=max_transitionname_length(Netz);
#endif

  Trans = Netz->Transitions;
  StellenBenennen (Netz, Stellenfeld);  /* Stellen und Transitionen  */
  TransBenennen (Netz->Transitions, Transfeld); /* temporaer umbenennen      */

NReach = (ReachList)Malloc(sizeof(ReachElement));
NReach->mark = ErzeugeVektor(HilfVek);         /* Startmarkierung kopieren  */
AReach = NReach;
AReach->next = AReach->prev = NULL;             /* und Zeiger auf NULL setzen*/
AReach->alter = NULL;
AReach->trans = NULL;
AReach->alt = 0;

#if defined(__TRACE__)
AReach->num = ++rlnum;
fprintf (Datei, " RL-No: %3d |", AReach->num);
SchreibeMarkierung (Datei, AReach->mark,
                    Stellenfeld, True);
#endif

VE = (KnotenMenge) Malloc (sizeof (Knoten));    /* Initialisiere UDG         */
VE->Markierung = ErzeugeVektor(HilfVek);        /* Einrichten der Wurzel des */
VE->next = NULL;                                /* UDG mit der Startmarkie-  */
VE->PfadVorg = NULL;                            /* rung                      */
v = VE;                                         /* Zeiger auf Wurzel des UDG */

/* ssch: */
reachmarkreached = TestVergleich(ReachVek, AReach->mark);
  if (!Minpfad && reachmarkreached)
    reachmarkreached = !reachmarkreached;

  while (v && !reachmarkreached) /* Hauptschleife des Algorithmus */
  {
    VektorAusSTListe (SchaltVek, Trans->preset);

    if (VektorGleich (SchaltVek, v->Markierung) ||
        VektorKleiner( SchaltVek, v->Markierung )  )
    {                                         /* DANN bilde Folgemar-      */
      Kopiere (vStrich, v->Markierung);        /* kierung: v" := v + N(t)   */
      Subtrahiere (vStrich, SchaltVek);
      VektorAusTSListe (SchaltVek, Trans->postset);
      Addiere (vStrich, SchaltVek, HilfVek);

      *NV = 0;                                  /* NV := (0,0,...,0)         */
                                                /* Teste, ob und an welchen  */
      vQuer = VE;                               /* Stellen v"  v oder einen  */
      while (vQuer)                             /* Vorgaenger von v echt     */
      {                                      /* ueberdeckt.               */
        vQuer->besucht = False;
        vQuer = vQuer->next;
      }

      UntersuchePfad (VE, v, vStrich, NV, HilfVek);

      if (* MultipliziereMitOmega (NV))         /*WENN v' := v' + omega * NV */
      {                                         /* DANN Berechnung abbrechen */
  
        /* BG hier muss vernuenftige Rueckgabe hin!!! */
        Beschraenkt = 0;

#if defined (__TRACE__)
        fprintf (Datei, "\n\nNet is NOT BOUNDED. ");
        fprintf (Datei, "Computation of path is not possible.\n\n");
        RemoveReachList (&AReach, Datei);      /* und Reachliste entfernen  */
#else
        RemoveReachList (&AReach);             /* und Reachliste entfernen  */
#endif
        break;                                 /* BREAK in WHILE (v &&      */
                                               /*        !reachmarkreached) */
      }                                      /* END IF (MultmitOmega ()) */

      vStrichPtr = VE;
      if (! VektorGleich                        /* v" in VE suchen; falls    */
	  (vStrichPtr->Markierung,vStrich))     /* nicht da: neu einfuegen   */
      {
        while (vStrichPtr->next &&             /* Falls v".next <> v        */
               ! VektorGleich (vStrichPtr->next->Markierung, vStrich))
        {
          vStrichPtr = vStrichPtr->next;      /* naechstes v               */
        }
        if (vStrichPtr->next)                  /* FALLS es weitere v" gibt, */
        {                                   /* DANN das naechste v" holen*/
          vStrichPtr = vStrichPtr->next;
        }                                   /* END IF (vStrichPtr->next) */
        else                                   /* ANDERNFALLS neuen Knoten  */
        {                                   /* fuer v" einrichten        */
          vStrichPtr = vStrichPtr->next = (KnotenMenge) Malloc(sizeof (Knoten));
          vStrichPtr->Markierung = ErzeugeVektor(vStrich);
          vStrichPtr->PfadVorg = NULL;
          vStrichPtr->next = NULL;
        }
      }

      /* Neue Kante v --t--> v" in ReachListe einfuegen */
      NReach = (ReachList) Malloc(sizeof (ReachElement));
      HilfPtr = NReach->trans = NeuerVektor (1);/* Speicher fuer neue Kante  */
      *(HilfPtr++) = Trans->numname;            /* anfordern,                */
      *HilfPtr = 0;                             /* Transitionsnummer in den  */
      NReach->mark = ErzeugeVektor (            /* pieren; sowie Markierung  */
                        vStrichPtr->Markierung);/* v' umkopieren             */
      NReach->alter = NULL;
      NReach->next  = NULL;
      NReach->prev  = AReach;                   /* neueKante.Vorgaenger auf  */
      AReach->next  = NReach;
      AReach        = NReach;                   /* Laufzeiger weiter setzen  */
      AReach->alt   = 0;

      if ((reachmarkreached = TestVergleich  /* Anfangsmarkierung == zu   */
                   (ReachVek, AReach->mark)))/* erreichende Markierung?   */
      {                                      /* DANN merken, dass erreich-*/
        pkopf->reached = reachmarkreached;   /* bare Markierung erreicht  */
      }                                      /* END IF (reachmarkreached) */
      if (!Minpfad && reachmarkreached)      /* WENN alle Pfade berechnet */
      {                                      /* werden sollen             */
        reachmarkreached = !reachmarkreached;/* DANN Flag notten          */
      }                                      /* END IF (!MinPfad)         */
      ZReach = AReach->prev;                 /* Zaehlzeiger auf vorletz-  */
                                             /* tes RL-element setzen     */

#if defined(__TRACE__)
      AReach->num = ++rlnum;
      fprintf (Datei, " RL-No: %3d |", AReach->num);
#endif
      while (ZReach)                            /* SOLANGE nicht am Listen-  */
      {                                      /* anfang DO                 */
        if (VektorGleich (ZReach->mark,        /* WENN "ZReach->mark" =     */
                          v->Markierung))      /*      "v->Markierung"      */
        {
          VReach = AReach->alter;             /* Alternativlistenelement   */
          NReach = (ReachList) Malloc         /* erzeugen und in die       */
                      (sizeof (ReachElement));/* AlternativListe einfuegen */
          NReach->trans = NULL;               /* Transition nicht gebraucht*/
          NReach->mark  = NULL;               /* pieren; sowie Markierung  */
          NReach->alter = ZReach;
          if (VReach == NULL)                 /* WENN Laufzeiger leer ist  */
          {                                /* DANN diesen auf das neue  */
            VReach        = NReach;          /* Element zeigen lassen     */
            VReach->prev  = NULL;            /* Dessen Vorgaenger auf NULL*/
            VReach->next  = NULL;            /* Dessen Vorgaenger auf NULL*/
            BReach        = VReach;
          }
          else                                /* ANDERNFALLS (AReach)      */
          {                                /* Laufzeiger.Nachfolger auf */
            VReach->next  = NReach;          /* Dessen Vorgaenger auf NULL*/
            NReach->prev  = VReach;          /* neueKante.Vorgaenger auf  */
            NReach->next  = NULL;            /* Dessen Vorgaenger auf NULL*/
            VReach        = NReach;          /* Laufzeiger weiter setzen  */
          }
#if defined(__TRACE__)
          VReach->num   = ZReach->num;
          fprintf (Datei, " age: %3d | ", VReach->num);
#endif
          AReach->alter = VReach;
        }                                   /* END IF ((VGleich(x1,x2))  */
        if (VektorGleich(ZReach->mark, vStrichPtr->Markierung))
        {
          AReach->alt++;
        }
        ZReach  = ZReach->prev;                /* vorheriges Listenelement  */
      }                                      /* END WHILE (ZReach)        */
      AReach->alter = BReach; /* Anfang der Alternativliste eintragen */
#if defined(__TRACE__)
      SchreibeMarkierung (Datei, v->Markierung, Stellenfeld, False);
      if (AReach->trans)
      {
        fprintf (Datei, "   => ");
        SchreibeTransition (Datei, AReach->trans, Transfeld, maxlen);
        fprintf (Datei, "=> ");
      }
      SchreibeMarkierung (Datei, AReach->mark, Stellenfeld, True);
#endif
      if ((VorgPtr = vStrichPtr->PfadVorg))     /* Pfad-Vorgaengerliste von  */
      {                                      /* v" aktualisieren          */
        if (! VektorGleich (VorgPtr->          /* WENN die Markierung nicht */
	  KnotenPtr->Markierung, v->Markierung)) /* gleich der Vorgangermar-  */
        {                                   /* rung ist, DANN suche      */
          while (VorgPtr->next &&             /* entsprechende Stelle zum  */
                 ! VektorGleich (VorgPtr->    /* Einfuegen in die Liste    */
		   next->KnotenPtr->Markierung, /* der Vorgaenger            */
                         v->Markierung)       )
          {
            VorgPtr = VorgPtr->next;
          }
        }
        if (! VorgPtr->next)
        {                                   /* FALLS keinen Vorgaenger   */
          VorgPtr = VorgPtr->next =           /* gefunden, der gleich der  */
                      (VorgaengerListe) Malloc  /* v-Markierung ist,         */
                      (sizeof (Vorgaenger));    /* DANN Knoten v in die Vor- */
          VorgPtr->next = NULL;               /* gaengerliste einfuegen    */
          VorgPtr->KnotenPtr = v;
        }
      } /* IF (VorgPtr = vStrichPtr->PfadVorg) */
      else
      { /* sofort Knoten v in Vorgaengerliste einfuegen */
         VorgPtr = vStrichPtr->PfadVorg =
                      (VorgaengerListe) Malloc(sizeof (Vorgaenger));
         VorgPtr->next = NULL;
         VorgPtr->KnotenPtr = v;
      }
    } /* IF v aktiviert t */

/* WENN die letzte Transition untersucht wurde, */
/* DANN den naechsten Knoten mit allen Transitionen untersuchen */
   if ( !(Trans = Trans->nexttrans))
   {
     Trans = Netz->Transitions;
     v = v->next;
   }
 }                                            /* END grosse WHILE (v)      */

#if defined(__TRACE__)
PrintUDG (VE, Stellenfeld, Datei);
fprintf (Datei, "\n OUT COMPUTEREACH \n\n");
#endif

RemoveUDG (VE);                                 /* UDG aus Speicher loeschen */
Free (NV);                                      /* nicht mehr benoetigte     */
Free (vStrich);                                 /* Markierungsvektoren aus   */
Free (HilfVek);                                 /* dem Speicher entfernen    */
Free (SchaltVek);                               /* ebenso die Platzhalter    */
Free (Stellenfeld);                             /* zum temporaeren umbenen-  */
Free (Transfeld);                               /* nen der Stellen u. Trans. */

pkopf->rlist = AReach;                          /* Ergebnis zuweisen         */
return (pkopf);                                 /* Ergebnis zurueckgeben     */
} /* Erreichbarkeitspfade */


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

PfadKopf ComputeSchaltfolgen (LMPNet    Netz  , /* zu untersuchendes Netz    */
                              PfadKopf  PlKopf, /* Schaltfolgenliste         */
                              Vektor    StartVek,/* Anfangsmarkierung        */
                              Vektor    ReachVek,/* zu erreichende Markierung*/
                              bool      MinPfad,/* Flag: gibt an ob kuerze-  */
                                                /* Pfad oder alle moeglichen */
                                                /* berechnet werden soll     */
                              bool      TeilMark/* Flag: Wenn True, werden   */
                                                /* Pfade zu Teilmarkierungen */
                                                /* berechnet                 */


#if defined(__TRACE__)
                                               ,
                              FILE *    Datei   /* Zeiger auf Ausgabedatei   */
#endif
                                                )
#else

PfadKopf ComputeSchaltfolgen (Netz, PlKopf, StartVek, 
                              ReachVek, MinPfad, TeilMark

#if defined(__TRACE__)
                             ,Datei             /* Zeiger auf Ausgabedatei   */
#endif
                                      )

LMPNet                   Netz    ;              /* zu untersuchendes Netz    */
PfadKopf                 PlKopf  ;              /* Schaltfolgenliste         */
Vektor                   StartVek;              /* Anfangsmarkierung         */
Vektor                   ReachVek;              /* zu erreichende Markierung */
bool                     MinPfad ;              /* Flag: gibt an ob kuerze-  */
                                                /* Pfad oder alle moeglichen */
                                                /* berechnet werden soll     */
bool                     TeilMark;              /* Flag: Wenn True, werden   */
                                                /* Pfade zu Teilmarkierungen */
                                                /* berechnet                 */

#if defined(__TRACE__)
   FILE               * Datei   ;               /* Zeiger auf Ausgabedatei   */
#endif

#endif

/*                                                                           */
/*  Funktion zur Berechnung der Erreichbarkeitspfade von der Startmarkierung */
/*  "PLkopf->init" eines Systems "Netz" zu einer erreichbaren Markierung     */
/*  "ReachVek des System "Netz".                                             */
/*                                                                           */
/*****************************************************************************/

{
#if defined (__TRACE__)

VCardinal      Stellenzahl = Netz->NumPlaces;
Place        * Stellenfeld = (Place *) Malloc(Stellenzahl * sizeof (Place));

StellenBenennen (Netz->Places, Stellenfeld);     /* Stellen und Transitionen */

fprintf (Datei, "\n\nIN COMPUTESCHALTFOLGEN\n\n");
#endif

/* TEST ob die Startmarkierung gleich der erreichbaren Markierung ist      */
if ((TeilMark? VEnthalten:VektorGleich)(ReachVek, StartVek))
{
#if defined (__TRACE__)
   fprintf (Datei, "\n Initial marking ");
   SchreibeMarkierung (Datei, StartVek, Stellenfeld, False);
   fprintf (Datei, "  equals ");
   fprintf (Datei, "\n Reachmark ");
   SchreibeMarkierung (Datei, ReachVek, Stellenfeld, True);
#endif

   PlKopf->reached = True;                       
}

#if defined(__TRACE__)
  fprintf (Datei, MinPfad ? " Minpath is True\n\n"
                          : " Minpath is False\n\n");
#endif

                                                /* Reachliste berechnen      */
if ( !(PlKopf->rlist)                  ||       /* WENN keine Reachliste ODER*/
     !VektorGleich (PlKopf->init, StartVek) ||  /* neue Startsmarkierung ODER*/
     !PlKopf->minpfad                    )      /* bisher nur kuerzester     */
   {                                            /* Pfad berechnet            */

#if defined(__TRACE__)

   if (!(PlKopf->rlist))
      {
      fprintf (Datei, " No Reachlists yet\n");
      }
   if (!VektorGleich (PlKopf->init, StartVek))
      {
      fprintf (Datei, " new Inital marking\n");
      }
   if (!PlKopf->minpfad)
      {
      fprintf (Datei, " calculate all paths\n");
      }
#endif

   PlKopf->init = StartVek;

/* DANN Reachliste neu berechnen */
#if defined(__TRACE__)
   PlKopf = ComputeReach (Netz, PlKopf, ReachVek, MinPfad, TeilMark, Datei);
#else
   PlKopf = ComputeReach (Netz, PlKopf, ReachVek, MinPfad, TeilMark);
#endif

   PlKopf->minpfad = MinPfad;
   }                                            /* END IF (! || ! || !)      */

if (PlKopf->rlist)                              /* WENN rlist <> leer        */
   {
#if defined(__TRACE__)
   PlKopf = ComputePath (PlKopf, ReachVek,      /* koennen Erreichbarkeits-  */
                         MinPfad, TeilMark,     /* pfade berechnet werden    */
                         Netz, Datei);
#else
   PlKopf = ComputePath (PlKopf,  ReachVek,     /* koennen Erreichbarkeits-  */
                         MinPfad, TeilMark);    /* pfade berechnet werden    */
#endif
   }
else                                            /* ANDERNFALLS keine Reach-  */
   {                                            /* liste vorhanden           */
#if defined(__TRACE__)
   fprintf (Datei, "\n!! no reachable Marking!!\n");
   RemovePathList (&PlKopf->pfade, Netz, Datei);/* Alte Pathlist entfernen   */
#else
   RemovePathList (&PlKopf->pfade);             /* Alte Pathlist entfernen   */
#endif
   }
return (PlKopf);
} /* ComputeSchaltfolgen  */


#ifdef __WPL__

/******************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

void WritePathList (LMPNet     Netz,            /* zu bearbeitendes Netz     */
                    PfadKopf   PLKopf,          /* auszugebende PfadListe    */
                    char     * Netzname         /* Name der Netzdatei        */

#if defined(__TRACE__)
                   ,FILE     * Datei            /* Zeiger auf Ausgabedatei   */
#endif
                                    )

#else

void WritePathList (Netz, PLKopf, Netzname

#if defined(__TRACE__)
                              ,Datei            /* Zeiger auf Ausgabedatei   */
#endif
                                    )

LMPNet      Netz;                               /* zu bearbeitendes Netz     */
PfadKopf    PLKopf;                             /* auszugebende PfadListe    */
char      * Netzname;

#if defined(__TRACE__)
   FILE     * Datei;                            /* Zeiger auf Ausgabedatei   */
#endif

#endif

/*                                                                           */
/*  Wenn die TRACE-Definition eingeschaltet ist, erfolgt die Ausgabe der     */
/*  Pfade in der Datei, auf die der Dateizeiger von "outfile" zeigt.         */
/*                                                                           */
/*  Wenn die TRACE-Definition ausgeschaltet ist, erfolgt die Ausgabe der Pfa-*/
/*  de in eine Datei, der an den Netznamen die Endung ".Pfa" angehaengt wird.*/
/*                                                                           */
/*****************************************************************************/

{
PfadList       AList = PLKopf->pfade;
Pfad           APath;
VCardinal      Stellenzahl = Netz->NumPlaces,
               Transzahl = Netz->NumTransitions;
Place         *Stellenfeld = (Place *) Malloc(Stellenzahl * sizeof (Place));
Transition    *Transfeld = (Transition *) Malloc
               (Transzahl * sizeof (Transition));
int            PNum = 0;

#if !defined (__TRACE__)
                                                /* Ausgabedatei einrichten   */
FILE         * Datei;
char           DateiName[50];

if (*Netzname)
{
  strcat (strcpy (DateiName, Netzname), ".res");/* Dateinamen mit der Endung*/
  if (!(Datei = fopen (DateiName, "w")))       /* ".res" versehen           */
  {
    printf ("Could not write file '%s'!\n", Netzname);
  }
}

#endif

fprintf (Datei, "\n* Reach path(s) for %s are:\n\n\n", Netzname);

StellenBenennen (Netz->Places, Stellenfeld);    /* Stellen und Transitionen  */
TransBenennen (Netz->Transitions, Transfeld);   /* temporaer umbenennen      */

while (AList)                                   /* WEIL noch ein Pfad vor-   */
   {                                            /* handen ist, DO            */
   APath = AList->pfad;                         /* Laufzeiger Pfad setzen    */

#if defined(__TRACE__)
   fprintf (Datei, "\npath %2d", AList->pnum);  /* Pfadnummer ausgeben       */
#else
   fprintf (Datei, "\npath %2d", ++PNum);       /* Pfadnummer ausgeben       */
#endif

   fprintf (Datei, "  Initial marking\n");      /* Ausgabe Startmarkierung   */
   while (APath)                                /* WEIL noch Pfadelemente    */
      {                                         /* vorhanden sind, DO        */
      if (APath->tran)                          /* WENN Transition vorhanden */
         {                                      /* DANN (alle Markierungen   */
         fprintf (Datei, "           ---- ");   /* ausser der Startmarkie-   */
         SchreibeTransition (Datei,APath->tran, /* rung haben eine zugehoe-  */
                             Transfeld, 0);     /* rige Transition)          */
         fprintf (Datei, " --->\n");
         }                                      /* END IF (APath-tran)       */
      SchreibeMarkierung (Datei, APath->mark,   /* Folgemarkierung ausgeben  */
                          Stellenfeld, False);
      APath = APath->prev;                      /* Pfadlaufzeiger auf naech- */
                                                /* stes Pfadelement setzen   */
      }                                         /* END WHILE (APath)         */
   fprintf (Datei, "\nmarking to reach");

#if defined(__TRACE__)
   fprintf (Datei, ": path %2d\n\n", AList->pnum);
#else
   fprintf (Datei, ": path %2d\n\n", PNum);
#endif

   AList = AList->next;                         /* Listenlaufzeiger auf      */
                                                /* naechste Pfadliste setzen */
   }                                            /* END WHILE (AList)         */
Free (Transfeld);                               /* und nicht mehr benoetig-  */
Free (Stellenfeld);                             /* ten Speicher freigeben    */

#if !defined(__TRACE__)
fclose (Datei);
#endif
} /*** WritePathList() ***/

#endif /* __WPL__ DEFINITION    */


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

void WriteSchaltfolgen (LMPNet     Netz    ,    /* zu bearbeitendes Netz     */
                        PfadKopf   PLKopf  ,    /* auszugebende PfadListe    */
                        bool       Short   ,    /* Kurze Ausgabe der Transi- */
                                                /* tionen                    */
                        char     * Netzname     /* Name der Netzdatei        */

#if defined(__TRACE__)
                       ,FILE     * Datei        /* Zeiger auf Ausgabedatei   */
#endif
                                           )

#else

void WriteSchaltfolgen (Netz, PLKopf, Short,  Netzname

#if defined(__TRACE__)
                        ,Datei       /* Zeiger auf Ausgabedatei   */
#endif
                                         )

LMPNet      Netz;                               /* zu bearbeitendes Netz     */
PfadKopf    PLKopf;                             /* auszugebende PfadListe    */
char      * Netzname;                           /* Name der Ausgabedatei     */
                                                /* ohne Endung               */
bool        Short;                              /* Kurze Ausgabe der Transi- */
                                                /* tionen                    */
#if defined(__TRACE__)
   FILE     * Datei;                            /* Zeiger auf Ausgabedatei   */
#endif

#endif

/*                                                                           */
/*  Wenn die TRACE-Definition eingeschaltet ist, erfolgt die Ausgabe der     */
/*  Schaltfolgen der Transitionen in der Datei, auf die der Dateizeiger      */
/*  "outfile" zeigt.                                                         */
/*                                                                           */
/*  Wenn die TRACE-Definition ausgeschaltet ist, erfolgt die Ausgabe der     */
/*  Schaltfolgen der Transitionen in einer Datei, die uber netname angespro- */
/*  chen wird.                                                               */
/*                                                                           */
/*****************************************************************************/

{
PfadList     AList = PLKopf->pfade;             /* Laufzeiger fuer PathList  */

Pfad         APath = NULL;                      /* Laufzeiger fuer Pfad      */

#if defined (__TRACE__)
  Pfad       MPath = NULL;                      /* Merkzeiger fuer Pfad      */
  int        num = 1;
#endif

int          fill  = 1;

VCardinal    Stellenzahl = Netz->NumPlaces,
             Transzahl   = Netz->NumTransitions;

Place        * Stellenfeld = (Place *) Malloc(Stellenzahl * sizeof (Place));

Transition * Transfeld = (Transition *) Malloc(Transzahl * sizeof (Transition));

#if !defined(__TRACE__)
FILE         * Datei=stdout;
char           DateiName[50];

if (*Netzname)                                  /* Datei fuer die Pfadlisten */
   {                                            /* oeffnen                   */
   strcpy (DateiName, Netzname);
   if (!(Datei = fopen (DateiName, "w")))
      {
      fprintf (stderr, "Could not write file '%s'!\n", Netzname);
      }
   }
#endif

StellenBenennen (Netz, Stellenfeld);    /* Stellen und Transitionen  */
TransBenennen (Netz->Transitions, Transfeld);   /* temporaer umbenennen      */

#if defined (__TRACE__)
  fprintf (Datei, "\n\n   IN WRITE SCHALTFOLGEN \n\n");
  fprintf (Datei, "   * The sequence in net");
  fprintf (Datei, " '%s` with initial marking \n", Netzname);
  SchreibeMarkierung (Datei, AList->pfad->mark, Stellenfeld, False );
  fprintf (Datei, "\n\n");
#endif

if (!Short)                                     /* WENN keine Kurzausgabe    */
 {
   fill=max_transitionname_length(Netz);
 }

   /* BG Signifikante Anfangszeile eingefuegt. */
   fprintf( Datei, "_SEQUENCE:\n" );

while (AList)                                   /* WEIL noch ein Pfad da ist */
   {
   APath = AList->pfad;                         /* Laufzeiger Pfad setzen    */

#if defined(__TRACE__)                       /* TRACE ausgabe             */
   fprintf (Datei, "   Sequence %2d : ", num++);
#endif

   while (APath)                                /* WEIL noch Pfadelemente da */
      {
      if (APath->tran)                          /* WENN Transition vorhanden */
         {                                      /* DANN (alle Markierungen   */
         SchreibeTransition (Datei, APath->tran,/* ausser der Startmarkie-   */
                             Transfeld, fill);  /* rung haben eine zugehoe-  */
                                                /* rige Transition)          */
         if (APath->prev && APath->prev->tran)  /* WENN nicht Pfadende       */
            {                                   /* DANN trenne Transitionen  */
            fprintf (Datei, ",");               /* durch Komma Leerzeichen   */
            }                                   /* END IF (nicht Pfadende)   */
         else                                   /* ANDERNFALLS (Pfadende)    */
            {                                   /* DANN                      */
            fprintf (Datei, "\n");              /* Zeilenumbruch             */
            }                                   /* END ELSE (nicht Pfadende) */
         }                                      /* END IF (APath->tran)      */

#if defined (__TRACE__)
      MPath = APath; /* gerade abgearbeitete Pfadmarkierung merken     */
#endif

      APath = APath->prev;                      /* naechstes Pfadelement     */
      }                                         /* END WHILE (APath)         */
   AList = AList->next;                         /* naechste Pfadliste        */
   }                                            /* END WHILE (AList)         */

#if defined (__TRACE__)
  fprintf (Datei, "\n\n   for reachable marking");
  SchreibeMarkierung (Datei, MPath->mark, Stellenfeld, True );
#endif

Free (Transfeld);                               /* Speicher leerfegen        */
Free (Stellenfeld);

#if !defined(__TRACE__)

/* BG hier schon Ausgabe, ob das Netz evtl. nicht beschraenkt war */
if (Beschraenkt == 0)
    fprintf (Datei, "Net is not bounded\n");

fclose (Datei);

#else

fprintf (Datei, "\n\n OUT WRITE SCHALT FOLGEN");

#endif

} /* WritePathList */


/*******************************************/
/* Get places not marked by given marking. */
/*******************************************/
static LIST_POINTER Unmarked_Places (LMPNet net, MARKING mark)
{
  Place        pl;
  LIST_POINTER UnmPlace = NULL, /* Liste unmarkierter Stellen */
               Begin = NULL;

  for(pl=net->Places; pl; pl=pl->nextplace, mark=mark->next)
  {
    if (!mark->value)
    {
      UnmPlace = (LIST_POINTER) MYmalloc(sizeof (LIST_ELEMENT));
      strcpy (UnmPlace->name, pl->name);
      UnmPlace->next = Begin;
      Begin = UnmPlace;
      }                                         /* END IF (mark->value==0.0) */
   }                                            /* END while (mark != NULL)  */
return (Begin);                                 /* gebe die Liste der unmar- */
                                                /* kierten Stellen zurueck   */
}                                               /*  ENDE von Unmarked_Places */

/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

int Reachability (LMPNet   Netz, 
                  MARKING  Mark0, /* zu erreichende Markeirung */
                  MARKING  Marke  /* Startmarkierung           */

#if defined(__TRACE__)
                 ,FILE   * Datei
#endif
                                )
#else

int Reachability (Netz, Mark0, Marke

#if defined(__TRACE__)
                  ,Datei
#endif
                  )

LMPNet       Netz;
MARKING      Mark0, /* Startmarkierung           */
             Marke; /* zu erreichende Markierung */

#if defined(__TRACE__)
    FILE   * Datei;
#endif

#endif

/*                                                                           */
/*  Erreichbarkeitsanalye                                                    */
/*                                                                           */
/*  Problem: Gegeben ist ein Petrinetz "Netz", eine Startmarkierung "Mark0"  */
/*           und eine weiter Markierung "Marke". Die Funktion soll entschei- */
/*           den, ob die gegebene Markierung "Marke" aus der Startmarkierung */
/*           "Mark0" erreichbar ist oder nicht.                              */
/*                                                                           */
/*  Loesung: Fuer ein gebenes Petrinetz ("Netz", "Mark0") ist die Markierung */
/*           "Marke" genau dann erreichbar, wenn folgende zwei Bedingungen   */
/*           erfuellt sind:                                                  */
/*                                                                           */
/*             1. das lineare Gleichungssystem "Marke" = "Mark0" + C * X     */
/*                besitzt eine reelle Loesung.                               */
/*                                                                           */
/*             2. jede Falle von "Netz" wird durch "Marke" markiert.         */
/*                                                                           */
/*           In der ersten Bedingung ist C die zu Netz "Netz" gehoerende In- */
/*           Inzidenzmatrix und der Vektor X ist eine S-Invariante von Netz  */
/*           "Netz". Die Erfuellung von Bedingung 1 ist durch die Loesung des*/
/*           linearen Gleichungssystems nachweisbar. Die Loesung ist nicht   */
/*           explizit gefordert, sie muss nur Element der rationalen Zahlen  */
/*           sein. Das Problem, ob das lineare Gleichungssystem eine rationa-*/
/*           le Loesung hat, wird in der Funktion "decide" beschrieben.      */
/*                                                                           */
/*           Zu Bedingung 2 ist zu sagen, dass jede Falle des Netzes "Netz"  */
/*           durch "Marke" markiert wird, genau dann wenn die maximale Falle */
/*           aller unmarkierten Stellen die leere Menge ist. Die zur Berech- */
/*           nung noetige Funktion "berechne_maximale_Falle" findet schon bei*/
/*           der Bestimmung der Reversiblitaet Anwendung und wird von dort   */
/*           uebernommen.                                                    */
/*                                                                           */
/*****************************************************************************/

{
  LIST_POINTER unmplace = NULL, /* Liste unmarkierter Stellen*/
               maxtrap  = NULL; /* Liste der maximalen Falle */

#if defined(__TRACE__)
  int reached = Decide (Netz, Mark0, Marke, Datei); /* Ist Marke Erreichbar? */
#else
  int reached = Decide (Netz, Mark0, Marke);
#endif

  if (reached > 0) /* Marke in Netz erreichbar? */
  {
    /* Menge der unmarkierten Stellen unter Markierung Marke */
    unmplace = Unmarked_Places (Netz, Marke);
    maxtrap  = Berechne_maximale_Falle (Netz, unmplace);
    /* WENN maximale Falle leer ist, dann ist Markierung "Marke" erreichbar. */
    if (!maxtrap)
    {
      reached = 1;
    }
    else /* ANDERNFALLS ist die Markierung nicht erreichbar. */
    {
#if defined(__TRACE__)
      fprintf (Datei, "\n MAXTRAP <> empty !!!\n");
#endif
      reached = 0;
      remove_list (&maxtrap);
    }
  }

  remove_list (&unmplace);
  return (reached);
} /*** Reachability() ***/

/*****************************************************************************/
/*******************    ENDE der  Erreichbarkeitsanalye    *******************/
/*****************************************************************************/




/*****************************************************************************/
/***                                                                       ***/
/***   Nun einige Funktionen zum einfacheren     Autor: Eberhard Meisel    ***/
/***   Testen von Netzen auf Erreichbarkeit             Juni 94            ***/
/***                                                                       ***/
/*****************************************************************************/



/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

void ShowPlaces  (MARKING    Marking,           /* Zeiger auf Markierung     */
                  Place      Stellen,           /* Zeiger auf Stellenliste   */
                  FILE     * Datei  )           /* Zeiger auf Ausgabedatei   */

#else

void ShowPlaces (Marking, Stellen, Datei)

MARKING     Marking;                            /* Zeiger auf Markierung     */
Place       Stellen;                            /* Zeiger auf Stellenliste   */
FILE      * Datei  ;                            /* Zeiger auf Ausgabedatei   */

#endif                                          /* END STDC & C++ DEFINITION */

/*                                                                           */
/* Gibt die Stellen eines Netzes mit ihrer Markierung auf dem Bildschirm aus */
/*                                                                           */
/*****************************************************************************/

{                                               /* Deklarationen             */
int      pllen = 0,
         fill = 1;

Place    stelle = Stellen;                      /* Laufzeiger Stellenliste   */

MARKING  markPtr = Marking;                     /* Laufzeiger Markierungs    */

if (Marking)
   {
   while (stelle)
      {                                         /* Berechne die Laenge des   */
      pllen = (int) (strlen (stelle->name) - 2);/* laengsten Stellennamens   */
      if (pllen > fill)
         {
         fill = pllen;
         }
      stelle = stelle->nextplace;
      }                                         /* END WHILE Stelle          */
   stelle = Stellen;
   while (stelle)
      {                                         /* Gebe Stellennamen aus     */
      fprintf (Datei, " %-*s ", fill, stelle->name);
      stelle = stelle->nextplace;
      }                                         /* END WHILE (Stelle)        */
   fprintf (Datei,"\n");                        /* Zeilentrenner             */
   stelle = Stellen;
   fill++;
   while (stelle)                               /* Gebe die Markierung der   */
      {                                         /* Stellen aus               */
      fprintf (Datei, " %-*d ", fill, markPtr->value);
      markPtr = markPtr->next;
      stelle = stelle->nextplace;
      }                                         /* END WHILE (Stelle)        */
   fprintf (Datei, "\n");                       /* Zeilentrenner             */
   }                                            /* ENDIF (Marhierung)        */
else
   {                                            /* Fehler : kein netz        */
   fprintf (stderr,"Error in ShowMarking: No Marking\n");
   }                                            /* else                      */
}                                               /* ENDE von ShowMarking      */


/*****************************************************************************/
/* Liest eine Markierung aus einer Datei ein                                 */
/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)
MARKING ReadMarkFile (LMPNet Net, char *Markfilename)
#else
MARKING ReadMarkFile (Netz, Markfilename)
LMPNet      Net;
char      * Markfilename;
#endif

{
  FILE      * markfile;
  Place       placeptr;
  MARKING     marke    = copy_start_marking (Net),
              markptr  = marke;
  int         num      = 1;
  char        ch,
              err_str[200],
              name[101];

  if (!(markfile = fopen (Markfilename, "r")))
    {
      fprintf (stderr, "ERROR: Marking file '%s`", Markfilename);
      fprintf (stderr, " could not be opened for reading!\n");
      exit (1);
    }

  for(placeptr=Net->Places; placeptr; placeptr=placeptr->nextplace)
    {
      placeptr->numname = 0;
    }

  if (isdigit (ch = ReadWhiteSpace (markfile)))
    {
      ungetc (ch, markfile);
      num = ReadNumber (markfile);
    }
  else
    ungetc (ch, markfile);

  if (!feof(markfile))
    {
      *name = '\0';
      ReadName (markfile, name, 100);
      if (!(placeptr = find_P (Net, name)))
	{
	  sprintf (err_str, "Place '%s' not found in net", name);
	  NNError (err_str);
	}                                      /* END IF (!(name im Net))   */
      placeptr->numname = (Cardinal32) num;
    }

  while (!feof(markfile))
    {
      if (TestSymbol (markfile, ','))
	{
	  num = 1;
	  if (isdigit (ch = ReadWhiteSpace (markfile)))
	    {
	      ungetc (ch, markfile);
	      num = ReadNumber (markfile);
	    }
	  else
	    {
	      ungetc (ch, markfile);
	    }
	  *name = '\0';
	  if (!feof(markfile))
	    ReadName (markfile, name, 100);
	  if (!(placeptr = find_P (Net, name)))
	    {
	      sprintf (err_str, "Place '%s' not found in net", name);
	      NNError (err_str);
	    }
	  placeptr->numname += (Cardinal32) num;
	} /* END IF (Testsymbol (...)   */
      else if (!feof (markfile))
	{
	  NNError ("',' expected");
	} /* END ELSE IF (!feof (...))  */
    } /* END WHILE (!feof (...))    */

  placeptr = Net->Places;

  while (markptr)
    {
      markptr->value = (int) placeptr->numname;
      markptr        = markptr->next;
      placeptr       = placeptr->nextplace;
    }
  return (marke);
} /*** END ReadMarkFile (...) ***/


/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

MARKING ReadArgMark (LMPNet    Netz   ,         /* Zeiger auf Netz           */
                     char    * ArgMark)         /* Zeiger auf Datei mit Mar- */
                                                /* kierungsliste             */
#else

MARKING ReadArgMark (Netz, ArgMark)

LMPNet      Netz;                               /* Zeiger auf Stellenliste   */
char     *  ArgMark;                            /* Zeiger auf Datei mit Mar- */
                                                /* kierungsliste             */

#endif                                          /* END STDC & C++ DEFINITION */

/*                                                                           */
/* Liest eine Markierungsfolge der Art "001100100001100" ein                 */
/*                                                                           */
/*****************************************************************************/

{
MARKING  marke   = copy_start_marking (Netz),
         markptr = marke;

int      j       = 0;

while (markptr)
   {
   markptr->value = ArgMark[j++] - '0';
   markptr        = markptr->next;
   }                                            /* END WHILE (markptr)       */

return (marke);
}                                               /*** END ReadMarking (...) ***/
