/*****************************************************************************/
/* Testprogramm zur Ausgabe der Schaltfolgen      Eberhard Meisel  1994      */
/*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "neunetze.h"
#include "convertPEP.h"
#include "reachy.h"

/*****************************************************************************/
/*                                                                           */
/* fordert den Benutzer auf, eine Markierungsfolge von Hand einzugeben       */
/*                                                                           */
/*****************************************************************************/
#if defined(__STDC__) || defined(__cplusplus)

void ShowHelp (char * Filename)

#else

void ShowHelp (Filename)

char *  Filename;

#endif

{
  printf ("\n\nUsage:\n\n");
  printf ("%s -I infile  <-M markfile | -m marking> [-O outfile -A -T]\n\n",
          Filename);
  printf ("  -I,i Name of PEP net file to read\n");
  printf ("  -M   Name of marking file\n");
  printf ("  -m   Marking, e.g. 011000100001\n");
  printf ("  -O,o Name of result file (w/o suffix)\n");
  printf ("  -A,a Calculate all paths\n");
  printf ("  -T,t Assume submarking\n\n");
}

/*****************************************************************************/
/*****************************************************************************/
int main( int argc, char **argv )
{
FILE       * infile;
FILE       * outfile=NULL;

LMPNet       Netz = NULL;

MARKING      EMark = NULL, /* Anfangs- und zu erreichende Markierung */
             Mark0 = NULL; /* als Markierungslistenform              */
Vektor       StartVek = NULL, /* Anfangsmarkierung in Vektordarstellung */
             ReachVek = NULL; /* zu erreichende Markierung */
			      /* in Vektordarstellung      */

PfadKopf     PLKopf = (PfadKopf) Malloc (sizeof (PfadKopfElem));

char         pepname [50] = "";
char         netname  [50] = "";
char         outname  [50] = "";
char         markname [50] = "";

char         *endung = ".res";

int          i = 1;

bool         marking = False,
             MinPath = True,
             TeilMark = False;

/* TestVergleich = VGleich; */

  if (argc < 2)
  {
    ShowHelp (*argv);
    exit (1);
  }
  for (i = 0; i < argc; i++)
  {
    if ((argv[i][0] == '-'))
    {
      if ((argv[i][1] == 'I') || (argv[i][1] == 'i'))
      {
        strcpy (pepname, argv[i]+2); /* Name PEP_datei einlesen   */
        if (!(*outname))             /* WENN noch keine Ausgabe-  */
        {                            /* datei angeben wurde, DANN */
          strcpy (outname, pepname); /* an den Namen der Eingabe- */
          strcat (outname, endung);  /* datei eine Endung haengen */
          strcpy (netname, outname);
        }
      }
      if ((argv[i][1] == 'O') || (argv[i][1] == 'o'))
      {
        strcpy (outname, argv[i]+2);  /* Ausgabedateinamen kopieren*/
        if (!(strchr (outname, '.'))) /* WENN der Name keinen Punkt*/
        {                             /* enhaelt, DANN an den Namen*/
          strcat (outname, endung);   /* der Ausgabedatei eine En- */
        }                             /* dung haengen              */
        strcpy (netname, outname);
      }
      if (argv[i][1] == 'M')          /* Name einer Datei, die eine*/
      {                               /* Markierung enthaelt       */
        strcpy (markname, argv[i]+2);
        marking = False;
      }
      if (argv[i][1] == 'm')          /* Markierung der Form */
      {                               /* 010...10 uebergeben */
        strcpy (markname, argv[i]+2);
        marking = True;
      }
      if ((argv[i][1] == 'T') || (argv[i][1] == 't'))
      {                                         /* Teilfolge erzeugen    */
        TeilMark = True;
/*        TestVergleich = VEnthalten;*/
      }
      if ((argv[i][1] == 'a') || (argv[i][1] == 'A'))
      {                                    /* alle Schaltfolgen erzeugen */
        MinPath = False;
      }
    }
  }

if (*pepname)
   {
   if ((infile = fopen (pepname, "r")))        /* EXISTIERT pepdatei???    */
      {
      fclose (infile);
      Netz = read_PEP (pepname);               /* Netz aus pepdatei lesen */
      }                                         /* END IF (Datei existiert)  */
   else                                         /* ANDERNFALLS               */
      {                                         /* Fehlermeldung ausgeben    */
      fclose (infile);
      fprintf (stderr, "No file named '%s'!\n\n", pepname);
      ShowHelp (argv[0]);
      exit (1);
      }
   }                                            /* END IF (pepname)         */
else
   {
   fprintf (stderr, "No net file given!!!\n");
   ShowHelp (argv[0]);
   exit (1);
   }

Mark0 = copy_start_marking (Netz);               /* Anfangsmarkierung des     */
                                                 /* Netzes einlesen           */



if (*outname)
   {
   outfile = fopen (outname, "w");
   if (!outfile)
      {
      fclose (outfile);
      fprintf (stderr, "Could not write '%s'!!!\n\n", netname);
      ShowHelp (argv[0]);
      exit (1);
      }
   }

else
   {
   fprintf (stderr, "No result file name given!!!\n");
   ShowHelp (argv[0]);
   }


if (*markname)
   {
   if (marking)
      {
      EMark = ReadArgMark (Netz, markname);
      }
   else
      {
      EMark = ReadMarkFile (Netz, markname);
      }
   }
else
   {
   fprintf (stderr, "No marking or marking file given!!!\n");
   ShowPlaces (Mark0, Netz->Places, stderr);
   ShowHelp (argv[0]);
   exit (1);
   }


if (Netz)
   {
   StartVek = MarktoVek (Mark0, Netz->NumPlaces);
   ReachVek = MarktoVek (EMark, Netz->NumPlaces);

#if defined (__TRACE__)
   fprintf (outfile, " Inital marking \n");
   ShowPlaces (Mark0, Netz->Places, outfile);

   fprintf (outfile, " desired marking \n");
   ShowPlaces (EMark, Netz->Places, outfile);
   fprintf (outfile, "\n\n");
#endif

   remove_marklist (&Mark0);
   remove_marklist (&EMark);

   PLKopf->init = StartVek;
   PLKopf->rlist = NULL;
   PLKopf->pfade = NULL;
   PLKopf->minpfad = MinPath;

#if defined(__TRACE__)
   PLKopf = ComputeSchaltfolgen (Netz, PLKopf, StartVek,
                                 ReachVek, MinPath, TeilMark,outfile);
   if (PLKopf->reached)
      {
      WriteSchaltfolgen (Netz, PLKopf, MinPath, netname, outfile );
      }
#else
   PLKopf = ComputeSchaltfolgen (Netz, PLKopf, StartVek, 
                                 ReachVek, MinPath, TeilMark);
   
   /* BG hier Testen, ob es beschraenkt war */
   if (Beschraenkt == 0)
   {
      printf ("Net is not bounded \n");
      fprintf (outfile, "Net is not bounded\n");
   }


   if (PLKopf->pfade)
      {
 
        /* BG Signifikante Anfangszeile eingefuegt. */
        if (PLKopf->reached)
          WriteSchaltfolgen (Netz, PLKopf, MinPath, netname);
      }

#endif

   }

#if defined(__TRACE__)
RemovePathHead (&PLKopf, Netz, outfile);
#else
RemovePathHead (&PLKopf);
#endif


/* ssch: ueberfluessig, da bereits durch ComputeReach geschehen */
Free (StartVek);
Free (ReachVek);
remove_net (&Netz);
exit (0);
}

