%{
#define YYSTYPE void*
#include <stdlib.h>
#include "ExprInterface.h"
#include <string.h>
void yyerror(char *s);
int yylex(void);
extern void Error(const char*);
#define YYDEBUG 1
%}


%token IDENT
%token NUMBER
%token NOTEQ
%token LEQ
%token GEQ
%token BLACK_TOKEN
%token EQUAL
%token LESS
%token GREATER
%token PLUS
%token MINUS
%token TIMES
%token DIVIDE
%token BRACE_OPEN
%token BRACE_CLOSE
%token BRACE_OPEN1
%token BRACE_CLOSE1
%token BRACE_OPEN2
%token BRACE_CLOSE2
%token LAND
%token LOR
%token LIMP
%token BOOL_TRUE
%token BOOL_FALSE
%token MODULO
%token DP
%token KOM
%token PP
%token CROSS
%token PROJ0
%token PROJ1
%token LNOT

%left      LOR
%right     LAND
%right     LNOT
%nonassoc  EQUAL NOTEQ GEQ LEQ GREATER LESS
%left      PLUS MINUS
%left      TIMES DIVIDE MODULO
%%


START   :       EQUANTORLIST DP S | S
        ;

EQUANTORLIST : EQUANTOR 
             | EQUANTOR LAND EQUANTORLIST
             ;

EQUANTOR : EQ1 
	 | EQ1 { store_last(); } CROSS SET { make_cross(); }
	 ;

EQ1      : IDENT { new_quantor($1); free($1); }  SET 
	 ;


SET : BRACE_OPEN1 VALUELIST BRACE_CLOSE1
    ;

VALUELIST : ELEMENT KOM VALUELIST
          | ELEMENT
          ;

ELEMENT : NUMBER           { new_value(*((int*)$1)); free($1); }
	| BOOL_TRUE	   { new_value_bool(True); }
	| BOOL_FALSE	   { new_value_bool(False); }
	| BLACK_TOKEN	   { new_value_dot();    }
        | NUMBER PP NUMBER 
                { new_values(*((int*)$1), *((int*)$3)); 
		  free($1); free($3);
	}	
        ;
          

S	: 	E2
	|	S LOR  E2        { new_operator(OR);  }
	|       S LIMP E2        { new_operator(IMP); }
	;

E2	:	E2 LAND E	{ new_operator(AND); }
	|	E
	;

E	:	R 
	| 	E EQUAL R        { new_operator(EQ); }
	|	E NOTEQ R        { new_operator(NOT_EQ); }
	;

R	: 	A        
	|	R LEQ A            { new_operator(LE_EQ); }
	|	R GEQ A            { new_operator(GT_EQ); }    
	|	R GREATER A        { new_operator(GT); }
	|	R LESS    A        { new_operator(LE); }
	;

A	: 	M
	|	A PLUS  M        { new_operator(ADD); }
	|	A MINUS M        { new_operator(SUB); }
	;

M 	: 	F
	|	M TIMES  F        { new_operator(MULT); }
	|	M DIVIDE F        { new_operator(DIV);  }
	|	M MODULO F	  { new_operator(MOD);  }
	; 

F	:	BRACE_OPEN  S  BRACE_CLOSE
	|       LNOT F          { new_operator(NOT); } 
	|	IDENT          { new_ident((char*)$$); free($$);   }
	|	NUMBER         { new_number(*(int*)$$); free($$); }
	|       BOOL_TRUE      { new_number_bool(True);  }
	|	BOOL_FALSE     { new_number_bool(False); }
	|	BLACK_TOKEN    { new_number_dot(); } 
	|       PROJ0 BRACE_OPEN IDENT BRACE_CLOSE { 
	                new_ident((char*)$3); free($3);
			new_operator(PR0); }
	|       PROJ1 BRACE_OPEN IDENT BRACE_CLOSE { 
	                new_ident((char*)$3); free($3);
			new_operator(PR1); }
	|	error
	;

%%

#include "ExprRegex.c"

void yyerror(char *s)  /* Called by yyparse on error */
{
    char m[512];
    sprintf(m, "%s while reading \"%s\"",s,yytext);
    Error(m);
    exit(1);
}

