// HLClipboard.cc
// Ulf Fildebrandt 1997/98
// Cecile Bui Thanh: added asynchronous Links (2001)
// last-update: Dec. 2001

  //**********
  // includes
  //**********

#include <unistd.h>
#include <time.h>
#include "../include/SLStack.h"
#include "HLNet.h"
#include "HLBox.h"
#include "../include/String.h"

extern "C" {
#include "frontend.h"
#include "../../backend/evaltree.h"
#include "../../backend/box_sub.h"
#include "../../backend/mnet.h"
#include "../../backend/tables.h"
#include "../../backend/block.h"
#include "../../backend/f_parse.h"
#include "../../backend/main.h"
}

extern VHMap<String, HLNet *> hlnets;

void
HLNet::paste() {
  if(clipboard == (HLNet *) NULL)
    return;
  String text;
  VHSet<String> BL;
  VHSet<String> TEM;
  VHSet<String> Schnitt;
  VHSet<String> P;
  const VHMap<String, Block *> &T = clipboard->Blocks();
  for(Pix u = T.first(); u; T.next(u)) {
    TEM.add(T.key(u));
  }
  VHMap<String, Transition *> Trans((Transition *) NULL, 10);
  VHMap<String, Place *> Stellen((Place *) NULL, 10);
  VHMap<String, PTransition *> PTrans((PTransition *) NULL, 10);
  VHMap<String, Block *> Blocks((Block *) NULL, 10);
  VHMap<String, Box *>   Boxes((Box *) NULL, 10);
  HLEdge *e;
  const VHMap<String, Box *> &bo = clipboard->Boxes();
  for(Pix i = bo.first(); i; bo.next(i)) {
    text = NewBox();
    Boxes[text] = clipboard->GetBoxFromName(bo.key(i));
    *(boxes[text]) = *(clipboard->GetBoxFromName(bo.key(i)));
    SEL += text + ",";
  }
  clipboard->RecalcRootBlocks();
  VHSet<String> bl = clipboard->RootBlocks();
  for(Pix i = bl.first(); i; bl.next(i)) {
    text = NewBlock();
    Blocks[text] = clipboard->GetBlockFromName(bl(i));
    SEL += text + ",";
  }
  VHMap<String, Block *> TE = Blocks;
  for(Pix i = TE.first(); i; TE.next(i)) {
    *blocks[TE.key(i)] = *TE.contents(i);
    pasteBlock(&Trans, &Stellen, &Blocks, &PTrans, &Boxes, TE.contents(i)->Name(), TE.key(i));
    for(Pix k = Boxes.first(); k; Boxes.next(k)) {
      if(Boxes.contents(k)->Name() == clipboard->blocks[TE.contents(i)->Name()]->BoxName()) {
        blocks[TE.key(i)]->setBox(Boxes.key(k), GetBoxFromName(Boxes.key(k)));
      }
    }
  }
  for(Pix j = Boxes.first(); j; Boxes.next(j)) {
    VHSet<String> BL = boxes[Boxes.key(j)]->BlockKeySet();
    for(Pix m = BL.first(); m; BL.next(m)) {
      boxes[Boxes.key(j)]->RemoveBlock(BL(m));
      for(Pix v = Blocks.first(); v; Blocks.next(v)) {
        if(Blocks.contents(v)->Name() == BL(m)) {
          boxes[Boxes.key(j)]->AddBlock(Blocks.key(v), GetBlockFromName(Blocks.key(v)));
          break;
        }
      }
    }
  }

  // Copying Links Set (we should only enter this loop when loading a module)
  VHMap<String,Link> DL = clipboard->getDeclaredLinks();
  for (Pix i = DL.first();i;DL.next(i))
  {
    String l_name = DL.key(i);
    if (!declared_links.contains(l_name))
        declared_links[l_name] = DL[l_name];
    else 
      {
        // if the type is the same, no problem (just don't add it),
        // otherwise, delete every link annotations with this link name
        if ( declared_links[l_name].get_type()
              != DL[l_name].get_type())
          {
            HLTransition * trans;
			for (Pix i=clipboard->transitions.first(); i; clipboard->transitions.next(i))
              {
                 trans = clipboard->transitions.contents(i)->asHLTransition();
                 trans->deleteLinkTerms(l_name);
              }
          }
      } // end if (!declared_links.contains(name))
  };

  for(Pix i = clipboard->transitions.first(); i; clipboard->transitions.next(i)) {
    BL = clipboard->transitions.contents(i)->BlockKeySet();
    Schnitt = TEM;
    Schnitt &= BL;
    if(Schnitt.empty()) {
      boolean schon = true;
      for(Pix no = Trans.first(); no; Trans.next(no)) {
        if(Trans.contents(no)->Name() == clipboard->transitions.contents(i)->Name()) {
          schon = false;
          break;
        }
      }
      if(schon) {
        text = NewTransition();
        Trans[text] = clipboard->transitions.contents(i)->asHLTransition();
        *transitions[text]->asHLTransition() = *clipboard->transitions.contents(i)->asHLTransition();
        SEL += text + ",";
        for(Pix k = Boxes.first(); k; Boxes.next(k)) {
          if(Boxes.contents(k)->Name() == clipboard->transitions.contents(i)->BoxName()) {
            transitions[text]->setBox(Boxes.key(k), GetBoxFromName(Boxes.key(k)));
          }
        }
      }
    }
  }
  for(Pix i = clipboard->ptransitions.first(); i; clipboard->ptransitions.next(i)) {
    BL = clipboard->ptransitions.contents(i)->BlockKeySet();
    Schnitt = TEM;
    Schnitt &= BL;
    if(Schnitt.empty()) {
      boolean schon = true;
      for(Pix no = PTrans.first(); no; PTrans.next(no)) {
        if(PTrans.contents(no)->Name() == clipboard->ptransitions.contents(i)->Name()) {
          schon = false;
          break;
        }
      }
      if(schon) {
        text = NewPTransition();
        PTrans[text] = clipboard->ptransitions.contents(i)->asHLPTransition();
        *ptransitions[text]->asHLPTransition() = *clipboard->ptransitions.contents(i)->asHLPTransition();
        SEL += text + ",";
        for(Pix k = Boxes.first(); k; Boxes.next(k)) {
          if(Boxes.contents(k)->Name() == clipboard->ptransitions.contents(i)->BoxName()) {
            ptransitions[text]->setBox(Boxes.key(k), GetBoxFromName(Boxes.key(k)));
          }
        }
      }
    }
  }
  for(Pix i = clipboard->places.first(); i; clipboard->places.next(i)) {
    BL = clipboard->places.contents(i)->BlockKeySet();
    Schnitt = TEM;
    Schnitt &= BL;
    if(Schnitt.empty()) {
      boolean schon = true;
      for(Pix no = Stellen.first(); no; Stellen.next(no)) {
        if(Stellen.contents(no)->Name() == clipboard->places.contents(i)->Name()) {
          schon = false;
          break;
        }
      }
      if(schon) {
        text = NewPlace();
        Stellen[text] = clipboard->places.contents(i);
        *places[text]->asHLPlace() = *clipboard->places.contents(i)->asHLPlace();
        SEL += text + ",";
        for(Pix k = Boxes.first(); k; Boxes.next(k)) {
          if(Boxes.contents(k)->Name() == clipboard->places.contents(i)->BoxName()) {
            places[text]->setBox(Boxes.key(k), GetBoxFromName(Boxes.key(k)));
          }
        }
      }
    }
  }
  for(Pix i = Trans.first(); i; Trans.next(i)) {
    if(transitions[Trans.key(i)]->BlockKeySet().empty()) 
      *transitions[Trans.key(i)]->asHLTransition() = *Trans.contents(i)->asHLTransition();
    P = transitions[Trans.key(i)]->Phantoms();
    for(Pix j = P.first(); j; P.next(j)) {
      for(Pix l = PTrans.first(); l; PTrans.next(l)) {
        if(PTrans.contents(l)->Name() == P(j)) {
          transitions[Trans.key(i)]->RemovePTrans(P(j));
          transitions[Trans.key(i)]->AddPTransition(PTrans.key(l), ptransitions[PTrans.key(l)]);
        }
      }
    }
    transitions[Trans.key(i)]->RemoveInEdges();
    transitions[Trans.key(i)]->RemoveOutEdges();
  }
  for(Pix i = PTrans.first(); i; PTrans.next(i)) {
    if(ptransitions[PTrans.key(i)]->BlockKeySet().empty()) 
      *ptransitions[PTrans.key(i)]->asHLPTransition() = *PTrans.contents(i)->asHLPTransition();
    P = ptransitions[PTrans.key(i)]->SyncTrans();
    for(Pix j = P.first(); j; P.next(j)) {
      for(Pix l = Trans.first(); l; Trans.next(l)) {
        if(Trans.contents(l)->Name() == P(j)) {
          ptransitions[PTrans.key(i)]->RemoveSyncTrans(P(j));
          ptransitions[PTrans.key(i)]->AddSyncTransition(Trans.key(l), transitions[Trans.key(l)]);
        }
      }
    }
    ptransitions[PTrans.key(i)]->RemoveInPEdges();
    ptransitions[PTrans.key(i)]->RemoveOutPEdges();
  }
  for(Pix i = Stellen.first(); i; Stellen.next(i)) {
    if(places[Stellen.key(i)]->BlockKeySet().empty())
      *(places[Stellen.key(i)]->asHLPlace()) = *(Stellen.contents(i)->asHLPlace());
    places[Stellen.key(i)]->RemoveInEdges();
    places[Stellen.key(i)]->RemoveOutEdges();
    places[Stellen.key(i)]->RemoveInPEdges();
    places[Stellen.key(i)]->RemoveOutPEdges();
  }
  for(Pix i = clipboard->edges.first(); i; clipboard->edges.next(i)) {
    e = clipboard->edges.contents(i)->asHLEdge();
    for (Pix j = Stellen.first(); j; Stellen.next(j)) {
      for (Pix k = Trans.first(); k; Trans.next(k)) {
        if(Stellen.contents(j)->Name() == e->Src().Name()) {
          if (Trans.contents(k)->Name() == e->Dest().Name()) {
            String text = NewEdge(Stellen.key(j), Trans.key(k));
            *edges[text] = *(clipboard->FindEdge(Stellen.contents(j)->Name(), Trans.contents(k)->Name()));
            edges[text]->asHLEdge()->clone(*(clipboard->FindEdge(Stellen.contents(j)->Name(), Trans.contents(k)->Name())->asHLEdge()));
          }
        } else if (Stellen.contents(j)->Name() == e->Dest().Name()) {
          if (Trans.contents(k)->Name() == e->Src().Name()) {
            String text = NewEdge(Trans.key(k), Stellen.key(j));
            *edges[text] = *(clipboard->FindEdge(Trans.contents(k)->Name(), Stellen.contents(j)->Name()));
            edges[text]->asHLEdge()->clone(*(clipboard->FindEdge(Trans.contents(k)->Name(), Stellen.contents(j)->Name())->asHLEdge()));
          }
        }
      }
    }
  }
  for(Pix i = clipboard->pedges.first(); i; clipboard->pedges.next(i)) {
    e = clipboard->pedges.contents(i)->asHLEdge();
    for (Pix j = Stellen.first(); j; Stellen.next(j)) {
      for (Pix k = PTrans.first(); k; PTrans.next(k)) {
        if(Stellen.contents(j)->Name() == e->Src().Name()) {
          if (PTrans.contents(k)->Name() == e->Dest().Name()) {
            String text = NewPEdge(Stellen.key(j), PTrans.key(k));
            //*pedges[text] = *(clipboard->FindPEdge(Stellen.contents(j)->Name(), PTrans.contents(k)->Name()));
          }
        } else if (Stellen.contents(j)->Name() == e->Dest().Name()) {
          if (PTrans.contents(k)->Name() == e->Src().Name()) {
            String text = NewPEdge(PTrans.key(k), Stellen.key(j));
            //*pedges[text] = *(clipboard->FindPEdge(PTrans.contents(k)->Name(), Stellen.contents(j)->Name()));
          }
        }
      }
    }
  }
}  // HLNet::paste

void
HLNet::cut(int c, String kette[]) {
  String n;
  copy(c, kette);
  for(int i = 0; i < c; i++) {
    if (transitions.contains(kette[i])) {
      n = transitions[kette[i]]->BoxName();
      DeleteBox(n);
      RemoveTransition(kette[i]);
    } else if (places.contains(kette[i])) {
      n = places[kette[i]]->BoxName();
      DeleteBox(n);
      RemovePlace(kette[i]);
    } else if (ptransitions.contains(kette[i])) {
      n = ptransitions[kette[i]]->BoxName();
      DeleteBox(n);
      RemovePTransition(kette[i]);
    } else if (blocks.contains(kette[i]))
      RemoveBlock(kette[i]);
    else if (boxes.contains(kette[i])) {
      for(Pix j = transitions.first(); j; transitions.next(j)) {
        if(transitions.contents(j)->BoxName() == kette[i])
          RemoveTransition(transitions.key(j));
      }
      for(Pix j = places.first(); j; places.next(j)) {
        if(places.contents(j)->BoxName() == kette[i])
          RemovePlace(places.key(j));
      }
      for(Pix j = ptransitions.first(); j; ptransitions.next(j)) {
        if(ptransitions.contents(j)->BoxName() == kette[i])
          RemovePTransition(ptransitions.key(j));
      }
      for(Pix j = blocks.first(); j; blocks.next(j)) {
        if(blocks.contents(j)->BoxName() == kette[i])
          RemoveBlock(blocks.key(j));
      }
      DeleteBox(kette[i]);
    }
  }
}  // HLNet::cut

void
HLNet::copyBlock(VHMap<String, Transition *> *Trans,
                VHMap<String, Place *> *Stellen,
                VHMap<String, Block *> *Blocks,
                VHMap<String, PTransition *> *PTrans,
                VHMap<String, Box *> *Boxes,
                String fromBlock, String toBlock) {
  VHSet<String> BL = GetBlockFromName(fromBlock)->BlockKeySet();
  for(Pix i = BL.first(); i;BL.next(i)) {
    boolean schon = true;
    for(Pix no = Blocks->first(); no; Blocks->next(no)) {
      if(Blocks->contents(no)->Name() == BL(i)) {
        schon = false;
        break;
      }
    }
    if(schon) {
      String text = clipboard->NewBlock();
      (*Blocks)[text] = blocks[BL(i)];
      *(clipboard->blocks[text]->asHLBlock()) = *((*Blocks)[text]->asHLBlock());
      String bo = blocks[BL(i)]->BoxName();
      for(Pix in = Boxes->first(); in; Boxes->next(in)) {
        if(Boxes->contents(in)->Name() == bo) {
          clipboard->blocks[text]->setBox(Boxes->key(in), clipboard->GetBoxFromName(Boxes->key(in)));
          break;
        }
      }
      clipboard->blocks[toBlock]->RemoveBlock(BL(i));
      clipboard->blocks[toBlock]->AddBlock(text, clipboard->GetBlockFromName(text));
      copyBlock(Trans, Stellen, Blocks, PTrans, Boxes, BL(i), text);
    }
  }
  for(Pix i = transitions.first(); i;transitions.next(i)) {
    VHSet<String> bl = transitions.contents(i)->BlockKeySet();
    VHSet<String> temp = GetBlockFromName(fromBlock)->BlockKeySet();
    temp &= bl;
    if (bl.contains(fromBlock) && temp.empty()) {
      boolean schon = true;
      for(Pix no = Trans->first(); no; Trans->next(no)) {
        if(Trans->contents(no)->Name() == transitions.contents(i)->Name()) {
          schon = false;
          break;
        }
      }
      if(schon) {
        String text = clipboard->NewTransition();
        (*Trans)[text] = transitions.contents(i);
        *(clipboard->transitions[text]->asHLTransition()) = *((*Trans)[text]->asHLTransition());
        String bo = transitions.contents(i)->BoxName();
        for(Pix in = Boxes->first(); in; Boxes->next(in)) {
          if(Boxes->contents(in)->Name() == bo) {
            clipboard->transitions[text]->setBox(Boxes->key(in), clipboard->GetBoxFromName(Boxes->key(in)));
            break;
          }
        }
        for(Pix t = bl.first(); t; bl.next(t)) {
          clipboard->transitions[text]->RemoveBlock(bl(t));
          for(Pix c = Blocks->first(); c; Blocks->next(c)) {
            if(Blocks->contents(c)->Name() == bl(t)) {
              clipboard->transitions[text]->AddBlock(Blocks->key(c), clipboard->blocks[Blocks->key(c)]);
            }
          }
        }
        clipboard->transitions[text]->RemoveInEdges();
        clipboard->transitions[text]->RemoveOutEdges();
      }
    }
  }
  for(Pix i = places.first(); i;places.next(i)) {
    VHSet<String> bl = places.contents(i)->BlockKeySet();
    VHSet<String> temp = GetBlockFromName(fromBlock)->BlockKeySet();
    temp &= bl;
    if (bl.contains(fromBlock) && temp.empty()) {
      boolean schon = true;
      for(Pix no = Stellen->first(); no; Stellen->next(no)) {
        if(Stellen->contents(no)->Name() == places.contents(i)->Name()) {
          schon = false;
          break;
        }
      }
      if(schon) {
        String text = clipboard->NewPlace();
        (*Stellen)[text] = places.contents(i);
        *(clipboard->places[text]->asHLPlace()) = *((*Stellen)[text]->asHLPlace());
        String bo = places.contents(i)->BoxName();
        for(Pix in = Boxes->first(); in; Boxes->next(in)) {
          if(Boxes->contents(in)->Name() == bo) {
            clipboard->places[text]->setBox(Boxes->key(in), clipboard->GetBoxFromName(Boxes->key(in)));
            break;
          }
        }
        for(Pix t = bl.first(); t;bl.next(t)) {
          clipboard->places[text]->RemoveBlock(bl(t));
          for(Pix c = Blocks->first(); c; Blocks->next(c)) {
            if(Blocks->contents(c)->Name() == bl(t)) {
              clipboard->places[text]->AddBlock(Blocks->key(c), clipboard->blocks[Blocks->key(c)]);
            }
          }
        }
        clipboard->places[text]->RemoveInEdges();
        clipboard->places[text]->RemoveOutEdges();
        clipboard->places[text]->RemoveInPEdges();
        clipboard->places[text]->RemoveOutPEdges();
      }
    }
  }
  for(Pix i = ptransitions.first(); i;ptransitions.next(i)) {
    VHSet<String> bl = ptransitions.contents(i)->BlockKeySet();
    VHSet<String> temp = GetBlockFromName(fromBlock)->BlockKeySet();
    temp &= bl;
    if (bl.contains(fromBlock) && temp.empty()) {
      boolean schon = true;
      for(Pix no = PTrans->first(); no; PTrans->next(no)) {
        if(PTrans->contents(no)->Name() == ptransitions.contents(i)->Name()) {
          schon = false;
          break;
        }
      }
      if(schon) {
        String text = clipboard->NewPTransition();
        (*PTrans)[text] = ptransitions.contents(i);
        *(clipboard->ptransitions[text]->asHLPTransition()) = *((*PTrans)[text]->asHLPTransition());
        String bo = ptransitions.contents(i)->BoxName();
        for(Pix in = Boxes->first(); in; Boxes->next(in)) {
          if(Boxes->contents(in)->Name() == bo) {
            clipboard->ptransitions[text]->setBox(Boxes->key(in), clipboard->GetBoxFromName(Boxes->key(in)));
            break;
          }
        }
        for(Pix t = bl.first(); t;bl.next(t)) {
          clipboard->ptransitions[text]->RemoveBlock(bl(t));
          for(Pix c = Blocks->first(); c; Blocks->next(c)) {
            if(Blocks->contents(c)->Name() == bl(t)) {
              clipboard->ptransitions[text]->AddBlock(Blocks->key(c), clipboard->blocks[Blocks->key(c)]);
            }
          }
        }
        clipboard->ptransitions[text]->RemoveInPEdges();
        clipboard->ptransitions[text]->RemoveOutPEdges();
      }
    }
  }
}  // HLNet::copyBlock

void
HLNet::pasteBlock(VHMap<String, Transition *> *Trans,
                  VHMap<String, Place *> *Stellen,
                  VHMap<String, Block *> *Blocks,
                  VHMap<String, PTransition *> *PTrans,
                  VHMap<String, Box *> *Boxes,
                  String fromBlock, String toBlock) {
  VHSet<String> bl = clipboard->GetBlockFromName(fromBlock)->BlockKeySet();
  for(Pix j = bl.first(); j;bl.next(j)) {
    String text = NewBlock();
    SEL += text + ",";
    (*Blocks)[text] = clipboard->GetBlockFromName(bl(j));
    *(blocks[text]->asHLBlock()) = *((*Blocks)[text]->asHLBlock());
    pasteBlock(Trans, Stellen, Blocks, PTrans, Boxes, bl(j), text);
    blocks[toBlock]->RemoveBlock(bl(j));
    blocks[toBlock]->AddBlock(text, GetBlockFromName(text));
    String bo = clipboard->blocks[bl(j)]->BoxName();
    for(Pix in = Boxes->first(); in; Boxes->next(in)) {
      if(Boxes->contents(in)->Name() == bo) {
        blocks[text]->setBox(Boxes->key(in), GetBoxFromName(Boxes->key(in)));
        break;
      }
    }
  }
  for(Pix i = clipboard->transitions.first(); i;clipboard->transitions.next(i)) {
    VHSet<String> bl = clipboard->transitions.contents(i)->BlockKeySet();
    VHSet<String> temp = clipboard->GetBlockFromName(fromBlock)->BlockKeySet();
    VHSet<String> t = bl;
    t &= temp;
    if((t.contains(fromBlock) || t.empty()) && bl.contains(fromBlock)) {
      String text = NewTransition();
      (*Trans)[text] = clipboard->transitions.contents(i);
      SEL += text + ",";
      *(transitions[text]->asHLTransition()) = *((*Trans)[text]->asHLTransition());
      String bo = clipboard->transitions.contents(i)->BoxName();
      for(Pix in = Boxes->first(); in; Boxes->next(in)) {
        if(Boxes->contents(in)->Name() == bo) {
          transitions[text]->setBox(Boxes->key(in), GetBoxFromName(Boxes->key(in)));
          break;
        }
      }
      for(Pix l = bl.first(); l;bl.next(l)) {
        transitions[text]->RemoveBlock(bl(l));
        for(Pix c = Blocks->first(); c; Blocks->next(c)) {
          if(Blocks->contents(c)->Name() == bl(l)) {
            transitions[text]->AddBlock(Blocks->key(c), blocks[Blocks->key(c)]);
          }
        }
      }
      transitions[text]->RemoveInEdges();
      transitions[text]->RemoveOutEdges();
    }
  }
  for(Pix i = clipboard->places.first(); i;clipboard->places.next(i)) {
    VHSet<String> bl = clipboard->places.contents(i)->BlockKeySet();
    VHSet<String> temp = clipboard->GetBlockFromName(fromBlock)->BlockKeySet();
    VHSet<String> t = bl;
    t &= temp;
    if(bl.contains(fromBlock) && (t.contains(fromBlock) || t.empty())) {
      String text = NewPlace();
      SEL += text + ",";
      (*Stellen)[text] = clipboard->places.contents(i);
      *(places[text]->asHLPlace()) = *((*Stellen)[text]->asHLPlace());
      String bo = clipboard->places.contents(i)->BoxName();
      for(Pix in = Boxes->first(); in; Boxes->next(in)) {
        if(Boxes->contents(in)->Name() == bo) {
          places[text]->setBox(Boxes->key(in), GetBoxFromName(Boxes->key(in)));
          break;
        }
      }
      for(Pix l = bl.first(); l;bl.next(l)) {
        places[text]->RemoveBlock(bl(l));
        for(Pix c = Blocks->first(); c; Blocks->next(c)) {
          if(Blocks->contents(c)->Name() == bl(l)) {
            places[text]->AddBlock(Blocks->key(c), blocks[Blocks->key(c)]);
            break;
          }
        }
      }
      places[text]->RemoveInEdges();
      places[text]->RemoveOutEdges();
      places[text]->RemoveInPEdges();
      places[text]->RemoveOutPEdges();
    }
  }
  for(Pix i = clipboard->ptransitions.first(); i;clipboard->ptransitions.next(i)) {
    VHSet<String> bl = clipboard->ptransitions.contents(i)->BlockKeySet();
    VHSet<String> temp = clipboard->GetBlockFromName(fromBlock)->BlockKeySet();
    VHSet<String> t = bl;
    t &= temp;
    if((t.contains(fromBlock) || t.empty()) && bl.contains(fromBlock)) {
      String text = NewPTransition();
      SEL += text + ",";
      (*PTrans)[text] = clipboard->ptransitions.contents(i);
      *(ptransitions[text]->asHLPTransition()) = *((*PTrans)[text]->asHLPTransition());
      String bo = clipboard->ptransitions.contents(i)->BoxName();
      for(Pix in = Boxes->first(); in; Boxes->next(in)) {
        if(Boxes->contents(in)->Name() == bo) {
          ptransitions[text]->setBox(Boxes->key(in), GetBoxFromName(Boxes->key(in)));
          break;
        }
      }
      for(Pix l = bl.first(); l;bl.next(l)) {
        ptransitions[text]->RemoveBlock(bl(l));
        for(Pix c = Blocks->first(); c; Blocks->next(c)) {
          if(Blocks->contents(c)->Name() == bl(l)) {
            ptransitions[text]->AddBlock(Blocks->key(c), blocks[Blocks->key(c)]);
          }
        }
      }
      ptransitions[text]->RemoveInPEdges();
      ptransitions[text]->RemoveOutPEdges();
    }
  }
  RecalcBlockVis();
}  // HLNet::pasteBlock

void
HLNet::copy(int c, String kette[]) {
  String zw;
  if (clipboard != (HLNet *) NULL) {
    // clear the clipborad link_table
    clipboard -> clearDeclaredLinks();

    int blocksc = 1;
    int placesc = 1;
    int transc  = 1;
    int ptransc = 1;
    int boxesc  = 1;
    if(!blocks.empty()) {
      for(Pix i = blocks.first(); i;blocks.next(i)) {
        if(blocks.contents(i)->Id() > blocksc)
          blocksc = blocks.contents(i)->Id();
      }
    }
    if(!boxes.empty()) {
      for(Pix i = boxes.first(); i; boxes.next(i)) {
        if(boxes.contents(i)->Id() > boxesc)
          boxesc = boxes.contents(i)->Id();
      }
    }
    if(!places.empty()) {
      for(Pix i = places.first(); i;places.next(i)) {
        if(places.contents(i)->Id() > placesc)
          placesc = places.contents(i)->Id();
      }
    }
    if(!transitions.empty()) {
      for(Pix i = transitions.first(); i;transitions.next(i)) {
        if(transitions.contents(i)->Id() > transc)
          transc = transitions.contents(i)->Id();
      }
    }
    if(!ptransitions.empty()) {
      for(Pix i = ptransitions.first(); i;ptransitions.next(i)) {
        if(ptransitions.contents(i)->Id() > ptransc)
          ptransc = ptransitions.contents(i)->Id();
      }
    }
    if(!blocks.empty())
      blocks.contents(blocks.first())->ResetMaxIdTo(blocksc);
    if(!boxes.empty())
      boxes.contents(boxes.first())->ResetMaxIdTo(boxesc);
    if(!places.empty())
      places.contents(places.first())->ResetMaxIdTo(placesc);
    if(!transitions.empty())
      transitions.contents(transitions.first())->ResetMaxIdTo(transc);
    if(!ptransitions.empty())
      ptransitions.contents(ptransitions.first())->ResetMaxIdTo(ptransc);
  }
  clipboard = new HLNet();
  VHSet<String> BL;
  VHMap<String, Transition *> Trans((Transition *) NULL, 10);
  VHMap<String, Place *> Stellen((Place *) NULL, 10);
  VHMap<String, Block *> Blocks((Block *) NULL, 10);
  VHMap<String, Box *>   Boxes((Box *) NULL, 10);
  VHMap<String, PTransition *> PTrans((PTransition *) NULL, 10);
  HLTransition *t;
  HLPTransition *pt;
  String text;
  for(int i = 0; i < c; i++) {
    if (transitions.contains(kette[i])) {
      text = clipboard->NewTransition();
      Trans[text] = transitions[kette[i]]->asHLTransition();
      *clipboard->transitions[text]->asHLTransition() = *Trans[text]->asHLTransition();
      BL = clipboard->transitions[text]->BlockKeySet();
      for(Pix j = BL.first(); j; BL.next(j)) {
        clipboard->transitions[text]->RemoveBlock(BL(j));
      }
    }
    else if (places.contains(kette[i])) {
      text = clipboard->NewPlace();
      Stellen[text] = places[kette[i]]->asHLPlace();
      *clipboard->places[text]->asHLPlace() = *Stellen[text]->asHLPlace();
      BL = clipboard->places[text]->BlockKeySet();
      for(Pix j = BL.first(); j; BL.next(j)) {
        clipboard->places[text]->RemoveBlock(BL(j));
      }
    }
    else if (ptransitions.contains(kette[i])) {
      text = clipboard->NewPTransition();
      PTrans[text] = ptransitions[kette[i]]->asHLPTransition();
      *clipboard->ptransitions[text]->asHLPTransition() = *PTrans[text]->asHLPTransition();
      BL = clipboard->ptransitions[text]->BlockKeySet();
      for(Pix j = BL.first(); j; BL.next(j)) {
        clipboard->ptransitions[text]->RemoveBlock(BL(j));
      }
    }
  }
  for(int i = 0; i < c; i++) {
    if(boxes.contains(kette[i])) {
      String text = clipboard->NewBox();
      Boxes[text] = boxes[kette[i]];
      *(clipboard->GetBoxFromName(text)) = *boxes[kette[i]];
      VHSet<String> roots = RootBlocks();
      for(Pix j = roots.first(); j; roots.next(j)) {
        if(blocks[roots(j)]->BoxName() == kette[i]) {
          boolean schon = true;
          for(Pix no = Blocks.first(); no; Blocks.next(no)) {
            if(Blocks.contents(no)->Name() == blocks[roots(j)]->Name()) {
              schon = false;
              break;
            }
          }
          if(schon) {
            zw = clipboard->NewBlock();
            Blocks[zw] = blocks[roots(j)];
            *(clipboard->GetBlockFromName(zw)->asHLBlock()) = *(blocks[roots(j)]->asHLBlock());
            copyBlock(&Trans, &Stellen, &Blocks, &PTrans, &Boxes, blocks[roots(j)]->Name(), zw);
            VHSet<String> BL = blocks[roots(j)]->BlockKeySet();
            for(Pix m = BL.first(); m; BL.next(m)) {
              clipboard->GetBlockFromName(zw)->asHLBlock()->RemoveBlock(BL(m));
              for(Pix v = Blocks.first(); v; Blocks.next(v)) {
                if(Blocks.contents(v)->Name() == BL(m)) {
                  clipboard->GetBlockFromName(zw)->asHLBlock()->AddBlock(Blocks.key(v), clipboard->GetBlockFromName(Blocks.key(v)));
                  break;
                }
              }
            }
            for(Pix in = Boxes.first(); in; Boxes.next(in)) {
              if(Boxes.contents(in)->Name() == kette[i]) {
                clipboard->blocks[zw]->setBox(Boxes.key(in), clipboard->GetBoxFromName(Boxes.key(in)));
                break;
              }
            }
          }
        }
      }
      for(Pix j = clipboard->boxes.first(); j; clipboard->boxes.next(j)) {
        VHSet<String> BL = clipboard->boxes.contents(j)->BlockKeySet();
        for(Pix m = BL.first(); m; BL.next(m)) {
          clipboard->boxes.contents(j)->RemoveBlock(BL(m));
          for(Pix v = Blocks.first(); v; Blocks.next(v)) {
            if(Blocks.contents(v)->Name() == BL(m)) {
              clipboard->boxes.contents(j)->AddBlock(Blocks.key(v), clipboard->GetBlockFromName(Blocks.key(v)));
              break;
            }
          }
        }
      }
      for(Pix j = transitions.first(); j; transitions.next(j)) {
        if(transitions.contents(j)->BoxName() == kette[i]) {
          boolean schon = true;
          for(Pix no = Trans.first(); no; Trans.next(no)) {
            if(Trans.contents(no)->Name() == transitions.contents(j)->Name()) {
              schon = false;
              break;
            }
          }
          if(schon) {
            zw = clipboard->NewTransition();
            Trans[zw] = transitions.contents(j);
            *clipboard->transitions[zw] = *transitions.contents(j);
            clipboard->transitions[zw]->setBox(text, clipboard->GetBoxFromName(text));
            BL = clipboard->transitions[zw]->BlockKeySet();
            for(Pix x = BL.first(); x; BL.next(x)) {
              clipboard->transitions[zw]->RemoveBlock(BL(x));
            }
          }
        }
      }
      for(Pix j = places.first(); j; places.next(j)) {
        if(places.contents(j)->BoxName() == kette[i]) {
          boolean schon = true;
          for(Pix no = Stellen.first(); no; Stellen.next(no)) {
            if(Stellen.contents(no)->Name() == places.contents(j)->Name()) {
              schon = false;
              break;
            }
          }
          if(schon) {
            zw = clipboard->NewPlace();
            Stellen[zw] = places.contents(j);
            *clipboard->places[zw] = *places.contents(j);
            clipboard->places[zw]->setBox(text, clipboard->GetBoxFromName(text));
            BL = clipboard->places[zw]->BlockKeySet();
            for(Pix x = BL.first(); x; BL.next(x)) {
              clipboard->places[zw]->RemoveBlock(BL(x));
            }
          }
        }
      }
      for(Pix j = ptransitions.first(); j; ptransitions.next(j)) {
        if(ptransitions.contents(j)->BoxName() == kette[i]) {
          boolean schon = true;
          for(Pix no = PTrans.first(); no; PTrans.next(no)) {
            if(PTrans.contents(no)->Name() == ptransitions.contents(j)->Name()) {
              schon = false;
              break;
            }
          }
          if(schon) {
            zw = clipboard->NewPTransition();
            PTrans[zw] = ptransitions.contents(j);
            *clipboard->places[zw] = *places.contents(j);
            clipboard->ptransitions[zw]->setBox(text, clipboard->GetBoxFromName(text));
            BL = clipboard->ptransitions[zw]->BlockKeySet();
            for(Pix x = BL.first(); x; BL.next(x)) {
              clipboard->ptransitions[zw]->RemoveBlock(BL(x));
            }
          }
        }
      }   
    }
  }
  for(int i = 0; i < c; i++) {
    if(blocks.contains(kette[i])) {
      String text = clipboard->NewBlock();
      Blocks[text] = blocks[kette[i]];
      *(clipboard->GetBlockFromName(text)->asHLBlock()) = *(blocks[kette[i]]->asHLBlock());
      copyBlock(&Trans, &Stellen, &Blocks, &PTrans, &Boxes, kette[i], text);
      VHSet<String> BL = blocks[kette[i]]->BlockKeySet();
      for(Pix m = BL.first(); m; BL.next(m)) {
        clipboard->GetBlockFromName(text)->asHLBlock()->RemoveBlock(BL(m));
        for(Pix v = Blocks.first(); v; Blocks.next(v)) {
          if(Blocks.contents(v)->Name() == BL(m)) {
            clipboard->GetBlockFromName(text)->asHLBlock()->AddBlock(Blocks.key(v), clipboard->GetBlockFromName(Blocks.key(v)));
            break;
          }
        }
      }
    }
  }
  VHSet<String> P;
  for(Pix i = clipboard->transitions.first(); i; clipboard->transitions.next(i)) {
//    if(clipboard->transitions.contents(i)->BlockKeySet().empty())
//      *(clipboard->transitions.contents(i))->asHLTransition() = *Trans[(clipboard->transitions.contents(i))->Name()]->asHLTransition();
    P = clipboard->transitions.contents(i)->Phantoms();
    for(Pix j = P.first(); j; P.next(j)) {
      for(Pix l = PTrans.first(); l; PTrans.next(l)) {
        if(PTrans.contents(l)->Name() == P(j)) {
          clipboard->transitions.contents(i)->RemovePTrans(P(j));
          clipboard->transitions.contents(i)->AddPTransition(PTrans.key(l), clipboard->ptransitions[PTrans.key(l)]);
        }
      }
    }
    clipboard->transitions.contents(i)->RemoveInEdges();
    clipboard->transitions.contents(i)->RemoveOutEdges();
  }
  for(Pix i = clipboard->places.first(); i; clipboard->places.next(i)) {
//    if(clipboard->places.contents(i)->BlockKeySet().empty())
//      *(clipboard->places.contents(i)->asHLPlace()) = *(Stellen[(clipboard->places.contents(i))->Name()]->asHLPlace());
    clipboard->places.contents(i)->RemoveInEdges();
    clipboard->places.contents(i)->RemoveOutEdges();
    clipboard->places.contents(i)->RemoveInPEdges();
    clipboard->places.contents(i)->RemoveOutPEdges();
  }
  for(Pix i = clipboard->ptransitions.first(); i;clipboard->ptransitions.next(i)) {
//    if(clipboard->ptransitions.contents(i)->BlockKeySet().empty())
//      *(clipboard->ptransitions.contents(i))->asHLPTransition() = *PTrans[(clipboard->ptransitions.contents(i))->Name()]->asHLPTransition();
    P = clipboard->ptransitions.contents(i)->SyncTrans();
    for(Pix j = P.first(); j; P.next(j)) {
      for(Pix l = Trans.first(); l; Trans.next(l)) {
        if(Trans.contents(l)->Name() == P(j)) {
          clipboard->ptransitions.contents(i)->RemoveSyncTrans(P(j));
          clipboard->ptransitions.contents(i)->AddSyncTransition(Trans.key(l), clipboard->transitions[Trans.key(l)]);
        }
      }
    }
    clipboard->ptransitions.contents(i)->RemoveInPEdges();
    clipboard->ptransitions.contents(i)->RemoveOutPEdges();
  }
  for(Pix i = transitions.first(); i; transitions.next(i)) {
    for(Pix v = Trans.first(); v; Trans.next(v)) {
      if (Trans.contents(v)->Name() == transitions.contents(i)->Name()) {
        t = transitions.contents(i)->asHLTransition();
        const Map<String, Edge *> &out_edges = t->OutputEdges();
        const Map<String, Edge *> &in_edges = t->InputEdges();
        for(Pix j = out_edges.first(); j; out_edges.next(j)) {
          HLEdge *e = out_edges.contents(j)->asHLEdge();
          for (Pix b = Stellen.first(); b; Stellen.next(b)) {
            if(Stellen.contents(b)->Name() == e->Dest().Name()) {
              String text = clipboard->NewEdge(Trans.key(v), Stellen.key(b));
               *clipboard->edges[text] = *FindEdge(Trans.contents(v)->Name(), Stellen.contents(b)->Name());
              clipboard->edges[text]->asHLEdge()->clone(*(FindEdge(Trans.contents(v)->Name(), Stellen.contents(b)->Name())->asHLEdge()));
            }
          }
        }
        for (Pix j = in_edges.first(); j; in_edges.next(j)) {
          HLEdge *e = in_edges.contents(j)->asHLEdge();
          for (Pix b = Stellen.first(); b; Stellen.next(b)) {
            if (Stellen.contents(b)->Name() == e->Src().Name()) {
              String text = clipboard->NewEdge(Stellen.key(b), Trans.key(v));
              *clipboard->edges[text] = *FindEdge(Stellen.contents(b)->Name(), Trans.contents(v)->Name());
              clipboard->edges[text]->asHLEdge()->clone(*(FindEdge(Stellen.contents(b)->Name(), Trans.contents(v)->Name())->asHLEdge()));
            }
          }
        }
      }
    }
  }
  for(Pix i = ptransitions.first(); i; ptransitions.next(i)) {
    for(Pix v = PTrans.first(); v; PTrans.next(v)) {
      if (PTrans.contents(v)->Name() == ptransitions.contents(i)->Name()) {
        pt = ptransitions.contents(i)->asHLPTransition();
        const Map<String, Edge *> &out_edges = pt->OutputPEdges();
        const Map<String, Edge *> &in_edges = pt->InputPEdges();
        for(Pix j = out_edges.first(); j; out_edges.next(j)) {
          HLEdge *e = out_edges.contents(j)->asHLEdge();
          for (Pix b = Stellen.first(); b; Stellen.next(b)) {
            if(Stellen.contents(b)->Name() == e->Dest().Name()) {
              String text = clipboard->NewPEdge(PTrans.key(v), Stellen.key(b));
              //*clipboard->pedges[text] = *FindPEdge(PTrans.contents(v)->Name(), Stellen.contents(b)->Name());
            }
          }
        }
        for (Pix j = in_edges.first(); j; in_edges.next(j)) {
          HLEdge *e = in_edges.contents(j)->asHLEdge();
          for (Pix b = Stellen.first(); b; Stellen.next(b)) {
            if (Stellen.contents(b)->Name() == e->Src().Name()) {
              String text = clipboard->NewPEdge(Stellen.key(b), PTrans.key(v));
              //*clipboard->pedges[text] = *FindPEdge(Stellen.contents(b)->Name(), PTrans.contents(v)->Name());
            }
          }
        }
      }
    }
  }
  const int Wert = int(DefaultHLSize / 2);
  for(Pix i = clipboard->transitions.first(); i; clipboard->transitions.next(i)) {
    clipboard->transitions.contents(i)->SetX(clipboard->transitions.contents(i)->GetX() - Wert);
    clipboard->transitions.contents(i)->SetY(clipboard->transitions.contents(i)->GetY() - Wert);
  }
  for(Pix i = clipboard->places.first(); i; clipboard->places.next(i)) {
    clipboard->places.contents(i)->SetX(clipboard->places.contents(i)->GetX() - Wert);
    clipboard->places.contents(i)->SetY(clipboard->places.contents(i)->GetY() - Wert);
  }
  for(Pix i = clipboard->ptransitions.first(); i; clipboard->ptransitions.next(i)) {
    clipboard->ptransitions.contents(i)->SetX(clipboard->ptransitions.contents(i)->GetX() - Wert);
    clipboard->ptransitions.contents(i)->SetY(clipboard->ptransitions.contents(i)->GetY() - Wert);
  }
}  // HLNet::copy

  //*** Dateiende HLClipboard.cc ***
