// HLComposition.cc
// Ulf Fildebrandt 1997/98
// Cecile Bui Thanh: added Tie operator (2001)
// last-update: June 2002

  //**********
  // includes
  //**********

#include <time.h>
#include "../include/SLStack.h"
#include "HLNet.h"
#include "HLBox.h"
#include "../include/String.h"

extern "C" {
#include "../../frontend/frontend.h"
#include "../../backend/evaltree.h"
#include "../../backend/readHL.h"
#include "../../backend/box_sub.h"
#include "../../backend/mnet.h"
#include "../../backend/tables.h"
#include "../../backend/block.h"
#include "../../backend/f_parse.h"
#include "../../backend/main.h"

  void InitBlockTable(void);

// from decls.h
  extern TreeNode *ExprTree;
  int exprparse();

  extern char *pep_file;
}

extern VHMap<String, HLNet *> hlnets;

#define _null_ ((((SymTabEntry *)node->op1)->s_init_or_cap==0) ? CHAN_NULL : 0)
#define _entry_ ((SymTabEntry *)node->op1)

Integer Handle(TreeNode *node)
{
  Integer n = -1;

  switch ( node->type )
    {
      case N_CONST  : {
                        n = add_node( N_CONST, 
                                      (int) node->op1, (int) node->op2 );
                        break;
                      }
      case N_VAR    : { 
                        switch ( (int)node->op2 ) {
                          case  _PREPRIMED:
                            n = Add_preprimed(  _entry_ );
                            break;
                          case  _UNPRIMED:
                            n = Add_unprimed(   _entry_ );
                            break;
                          case  _POSTPRIMED: 
                            n = Add_postprimed( _entry_ );
                            break;
                          case _READ:
                            n = Add_Channel( _entry_, _null_ );
                            break;
                          case _WRITE:
                            n = Add_Channel( _entry_, _null_ | CHAN_WRITE  );
                            break;
                          case _EMPTY:
                            n = Add_Channel( _entry_, _null_ | CHAN_CHECK );
                            break;
                          case _FULL:
                            n = Add_Channel( _entry_, 
                                             _null_ | CHAN_CHECK | CHAN_WRITE );
                            break;
                        }
                        break;
                      }
      case N_NOT    :
      case N_SIGN   :
      case N_KLAMM  : {
                        n = add_node( node->type, Handle( node->op1 ), 0 );
                        break;
                      }
      case N_OR     :
      case N_AND    :
      case N_EQ     :
      case N_UNEQ   :
      case N_LT     :
      case N_GT     :
      case N_LE     : 
      case N_GE     : 
      case N_PLUS   : 
      case N_MIN    : 
      case N_MULT   : 
      case N_DIV    : 
      case N_MOD    : {
#ifdef UNPRIMED_LOCAL
/* Wenn man UNPRIMED_LOCAL im Makefile setzt, so werden die Implikationen
   bei Verwendung von ungestrichenen Variablen LOKAL im Guard eingefuegt.
   Die Pros und Kontras gegenueber GLOBAL sind in [ACK97] aufgezeigt. */
                        Integer n1, n2;
                        
                        n1 = Handle( node->op1 );
                        if ( node->type == N_OR )
                          n1 = add_UnprimedGuard();

                        n2 = Handle( node->op2 );
                        if ( node->type == N_OR )
                          n2 = add_UnprimedGuard();

                        n = add_node( node->type, n1, n2 );
#else
                        n = add_node( node->type, Handle( node->op1 ), 
                                      Handle( node->op2 ) );
#endif
                        break;
                      }
      default: Msg( ERR, "Internal Error: Unknown node type in expr !" );
    }

  return n;
}  // HLNet::Handle

void
HLNet::MakeExpression(String term) {

  InitMultiSets();
  InitVarMSets();
  InitAllSymbolTables();
  InitBlockTable();

  extern int exprleng;
  exprleng = 0;

  extern bool GL_Frags;
  GL_Frags = True;

  init_ExpressionParser(NULL);

  ofstream expr("expression");
  expr << "~" << term << "~" << "@" << endl;
  expr.close();

  extern FILE *exprin;
  exprin = fopen("expression", "r");
  exprparse();

  if(ExprTree != NULL) {
    Handle(ExprTree->op1);
    exit_ExpressionParser();

    M_Net net = M_CreateBasicBox( action_set, expr_tree, expr_def, &expr_vars, 
                            StructToRef((ShortEntryPtr)ExprTree->op2) );
    GL_AddActions = False;
    scale = 1;
    Modify_Transition(net->Transitions);
    pep_file = new char[clipboardfile.length()];
    strcpy(pep_file, clipboardfile.data());
    HandleNet(net);
    fclose(exprin);
  } else {
    fclose(exprin);
    exit_ExpressionParser();
    return;
  }
  loadmodul(clipboardfile);
  String objekte = SEL;
  int zaehler = 1;
  while(objekte != "") {
    objekte = objekte.after(",");
    zaehler++;
  }
  String kette[zaehler];
  objekte = SEL;
  for(int i = 0;i < zaehler;i++) {
    kette[i] = (i == zaehler-1) ? objekte : objekte.before(",");
    objekte = objekte.after(",");
  }
  for(int i = 0; i < zaehler;i++) {
    if(ContainTransition(kette[i])) {
      FindTransition(kette[i])->SetX(60);
      FindTransition(kette[i])->SetY(150);
    }
    if(ContainPlace(kette[i])) {
      FindPlace(kette[i])->SetX(60);
      if(FindPlace(kette[i])->State() == 0) {
        FindPlace(kette[i])->SetY(90);
      } else {
        FindPlace(kette[i])->SetY(210);
      }
    }
  }
  SEL = "";
}  // HLNet::MakeExpression

void
HLNet::loadmodul(CONST char *argv, boolean inbox, String b, boolean clear_links)
{
  /*if(clipboard != NULL) {
    int blocksc = 1;
    int placesc = 1;
    int transc  = 1;
    int ptransc = 1;
    if(!blocks.empty()) {
      for(Pix i = blocks.first(); i;blocks.next(i)) {
        if(blocks.contents(i)->Id() > blocksc)
          blocksc = blocks.contents(i)->Id();
      }
    }
    if(!places.empty()) {
      for(Pix i = places.first(); i;places.next(i)) {
        if(places.contents(i)->Id() > placesc)
          placesc = places.contents(i)->Id();
      }
    }
    if(!transitions.empty()) {
      for(Pix i = transitions.first(); i;transitions.next(i)) {
        if(transitions.contents(i)->Id() > transc)
          transc = transitions.contents(i)->Id();
      }
    }
    if(!ptransitions.empty()) {
      for(Pix i = ptransitions.first(); i;ptransitions.next(i)) {
        if(ptransitions.contents(i)->Id() > ptransc)
          ptransc = ptransitions.contents(i)->Id();
      }
    }
    clipboard->DeleteAll();
    if(!blocks.empty())
      blocks.contents(blocks.first())->ResetMaxIdTo(blocksc);
    if(!places.empty())
      places.contents(places.first())->ResetMaxIdTo(placesc);
    if(!transitions.empty())
      transitions.contents(transitions.first())->ResetMaxIdTo(transc);
    if(!ptransitions.empty())
      ptransitions.contents(ptransitions.first())->ResetMaxIdTo(ptransc);
  } else {*/
    clipboard = new HLNet();
  //}
  ifstream in(argv);
  if (!in) {
    return;
  }  // if Dateifehler
  in >> *clipboard;
  if (clear_links)
      clipboard -> clearDeclaredLinks();
  paste();
  String objekte = SEL;
  int zaehler = 1;
  while(objekte != "") {
    objekte = objekte.after(",");
    zaehler++;
  }
  String kette[zaehler];
  objekte = SEL;
  for(int i = 0;i < zaehler;i++) {
    kette[i] = (i == zaehler-1) ? objekte : objekte.before(",");
    objekte = objekte.after(",");
  }
  if(inbox)
    TakeInBox(zaehler, kette);
  else
    AddToBox(zaehler, kette, b);
}  // HLNet::loadmodul

void
HLNet::savemodul(CONST char *argv)
{
  if(clipboard == (HLNet *) NULL)
    return;
  ofstream out(argv);
  if(out) {
    out << *clipboard;
  }
}  // HLNet::savemodul

void
HLNet::Parallel(String Block1, String Block2, String Name)
{
  Box *ptr = GetBoxFromName(Name);
  Box *ptr2 = GetBoxFromName(Block1);
  Box *ptr3 = GetBoxFromName(Block2);
  ptr->SetX(int((ptr2->GetX() + ptr3->GetX()) / 2));
  ptr->SetY(int((ptr2->GetY() + ptr3->GetY()) / 2));
  for (Pix i = places.first(); i;places.next(i)) {
    if (places.contents(i)->BoxName() == Block1) {
      places.contents(i)->setBox(Name, ptr);
    }
    if (places.contents(i)->BoxName() == Block2) {
      places.contents(i)->setBox(Name, ptr);
    }
  }
  for (Pix i = transitions.first(); i; transitions.next(i)) {
    if (transitions.contents(i)->BoxName() == Block1) {
      transitions.contents(i)->setBox(Name, ptr);
    }
    if (transitions.contents(i)->BoxName() == Block2) {
      transitions.contents(i)->setBox(Name, ptr);
    }
  }
  for (Pix i = ptransitions.first(); i;ptransitions.next(i)) {
    if (ptransitions.contents(i)->BoxName() == Block1) {
      ptransitions.contents(i)->setBox(Name, ptr);
    }
    if (ptransitions.contents(i)->BoxName() == Block2) {
      ptransitions.contents(i)->setBox(Name, ptr);
    }
  }
  DeleteBox(Block1);
  DeleteBox(Block2);
}  // HLNet::Parallel

void
HLNet::Sequence(String Block1, String Block2, String Name)
{
  VHSet<Val> Typ, schnitt;
  VHMap<String, VHSet<Val> > Typen(VHSet<Val>(4), 5);
  String typstring;
  int same;
  Val counter = 0;
  Box *ptr = GetBoxFromName(Name);
  Box *ptr2 = GetBoxFromName(Block1);
  Box *ptr3 = GetBoxFromName(Block2);
  ptr->SetX(int((ptr2->GetX() + ptr3->GetX()) / 2));
  ptr->SetY(int((ptr2->GetY() + ptr3->GetY()) / 2));
  VHMap<String, Place *> First((Place *) NULL, 10);
  VHMap<String, Place *> Second((Place *) NULL, 10);
  for(Pix i = places.first(); i; places.next(i)) {
    if (places.contents(i)->BoxName() == Block1 &&
        places.contents(i)->State() == 1) {
      First[places.contents(i)->Name()] = places.contents(i);
      for(Pix j = Typen.first(); j; Typen.next(j)) {
        same = (Typen.contents(j) == places.contents(i)->asHLPlace()->Type());
        schnitt = places.contents(i)->asHLPlace()->Type();
        schnitt &= Typen.contents(j);
        if(schnitt.length() != 0 & !same) {
          DeleteBox(Name);
          return;
        }
      }
      Typen[counter.asString()] = places.contents(i)->asHLPlace()->Type();
      counter = counter + 1;
    }
  }
  for(Pix i = places.first(); i; places.next(i)) {
    if (places.contents(i)->BoxName() == Block2 &&
        places.contents(i)->State() == 0) {
      Second[places.contents(i)->Name()] = places.contents(i);
      for(Pix j = Typen.first(); j; Typen.next(j)) {
        same = (Typen.contents(j) == places.contents(i)->asHLPlace()->Type());
        schnitt = places.contents(i)->asHLPlace()->Type();
        schnitt &= Typen.contents(j);
        if(schnitt.length() != 0 & !same) {
          DeleteBox(Name);
          return;
        }
      }
      Typen[counter.asString()] = places.contents(i)->asHLPlace()->Type();
      counter = counter + 1;
    }
  }
  if (First.length() == 0 || Second.length() == 0) {
    DeleteBox(Name);
    return;
  }
  String PlaceName;
  String temp, n, anno;
  for(Pix i = First.first(); i; First.next(i)) {
    for(Pix j = Second.first(); j; Second.next(j)) {
      if(First.contents(i)->asHLPlace()->Type() == Second.contents(j)->asHLPlace()->Type()) {
        temp = First.contents(i)->Name() + "@" + Second.contents(j)->Name();
        NewPlace(temp);
        places[temp]->SetX(int(First.contents(i)->GetX() +
                               Second.contents(j)->GetX()) / 2);
        places[temp]->SetY(int(First.contents(i)->GetY() +
                               Second.contents(j)->GetY()) / 2);
        places[temp]->setBox(Name, ptr);
        Typ = First.contents(i)->asHLPlace()->Type();
        typstring = First.contents(i)->asHLPlace()->GetTypString();
        places[temp]->asHLPlace()->SetTyp(Typ, typstring);
        const Map<String, Edge *> &in = First.contents(i)->InputEdges();
        for(Pix k = in.first(); k; in.next(k)) {
          n = NewEdge(in.contents(k)->Src().Name(), temp);
          anno = FindEdge(in.contents(k)->Src().Name(), First.contents(i)->Name())->asHLEdge()->VarsToString();
          edges[n]->asHLEdge()->ChangeVars(anno);
        }
        const Map<String, Edge *> &out = Second.contents(j)->OutputEdges();
        for(Pix k = out.first(); k; out.next(k)) {
          n = NewEdge(temp, out.contents(k)->Dest().Name());
          anno = FindEdge(Second.contents(j)->Name(), out.contents(k)->Dest().Name())->asHLEdge()->VarsToString();
          edges[n]->asHLEdge()->ChangeVars(anno);
        }
      }
    } 
  }
  for(Pix i = First.first(); i; First.next(i)) {
    const Map<String, Edge *> &in = FindPlace(First.key(i))->InputEdges();
    VHSet<String> in2;
    for(Pix j = in.first(); j; in.next(j)) {
      char *h = new char[5];
      sprintf(h, "A%d", in.contents(j)->Id());
      String he(h);
      in2.add(he);
      delete [] h;
    }
    RemovePlace(First.key(i));
    for(Pix j = in2.first(); j; in2.next(j)) {
      RemoveEintrag(in2(j));
    }
  }
  for(Pix i = Second.first(); i; Second.next(i)) {
    const Map<String, Edge *> &out = FindPlace(Second.key(i))->OutputEdges();
    VHSet<String> out2;
    for(Pix j = out.first(); j; out.next(j)) {
      char *h = new char[5];
      sprintf(h, "A%d", out.contents(j)->Id());
      String he(h);
      out2.add(he);
      delete [] h;
    }
    RemovePlace(Second.key(i));
    for(Pix j = out2.first(); j; out2.next(j)) {
      RemoveEintrag(out2(j));
    }
  }
  for (Pix i = places.first(); i;places.next(i)) {
    if (places.contents(i)->BoxName() == Block1) {
      places.contents(i)->setBox(Name, ptr);
    }
    if (places.contents(i)->BoxName() == Block2) {
      places.contents(i)->setBox(Name, ptr);
    }
  }
  for (Pix i = transitions.first(); i; transitions.next(i)) {
    if (transitions.contents(i)->BoxName() == Block1) {
      transitions.contents(i)->setBox(Name, ptr);
    }
    if (transitions.contents(i)->BoxName() == Block2) {
      transitions.contents(i)->setBox(Name, ptr);
    }
  }
  for (Pix i = ptransitions.first(); i;ptransitions.next(i)) {
    if (ptransitions.contents(i)->BoxName() == Block1) {
      ptransitions.contents(i)->setBox(Name, ptr);
    }
    if (ptransitions.contents(i)->BoxName() == Block2) {
      ptransitions.contents(i)->setBox(Name, ptr);
    }
  }
  DeleteBox(Block1);
  DeleteBox(Block2);
}  // HLNet::Sequence

void
HLNet::Substitution(String Box, String Transition)
{
  VHSet<Val> Typ, schnitt;
  VHMap<String, VHSet<Val> > Typen(VHSet<Val>(4), 5);
  String typstring;
  int same;
  Val counter = 0;
  VHMap<String, Place *> InputBox((Place *) NULL, 10);
  const VHSet<String> &InputTrans = transitions[Transition]->InputPlaces();
  for(Pix i = places.first(); i; places.next(i)) {
    if (places.contents(i)->BoxName() == Box &&
        places.contents(i)->State() == 0) {
      InputBox[places.contents(i)->Name()] = places.contents(i);
      for(Pix j = Typen.first(); j; Typen.next(j)) {
        same = (Typen.contents(j) == places.contents(i)->asHLPlace()->Type());
        schnitt = places.contents(i)->asHLPlace()->Type();
        schnitt &= Typen.contents(j);
        if(schnitt.length() != 0 & !same) {
          return;
        }
      }
      Typen[counter.asString()] = places.contents(i)->asHLPlace()->Type();
      counter = counter + 1;
    }
  }
  VHMap<String, Place *> OutputBox((Place *) NULL, 10);
  const VHSet<String> &OutputTrans = transitions[Transition]->OutputPlaces();
  for(Pix i = places.first(); i; places.next(i)) {
    if (places.contents(i)->BoxName() == Box &&
        places.contents(i)->State() == 1) {
      OutputBox[places.contents(i)->Name()] = places.contents(i);
      for(Pix j = Typen.first(); j; Typen.next(j)) {
        same = (Typen.contents(j) == places.contents(i)->asHLPlace()->Type());
        schnitt = places.contents(i)->asHLPlace()->Type();
        schnitt &= Typen.contents(j);
        if(schnitt.length() != 0 & !same) {
          return;
        }
      }
      Typen[counter.asString()] = places.contents(i)->asHLPlace()->Type();
      counter = counter + 1;
    }
  }
  // Test, ob wirklich Eingangs- und Ausgangsstellen vorhanden sind.
  if (OutputBox.length() == 0 || InputBox.length() == 0 || InputTrans.length() == 0 || OutputTrans.length() == 0)
    return;

  String PlaceName;
  String temp, n, anno;
  for(Pix i = InputBox.first(); i; InputBox.next(i)) {
    for(Pix j = InputTrans.first(); j; InputTrans.next(j)) {
      if(InputBox.contents(i)->asHLPlace()->Type() == places[InputTrans(j)]->asHLPlace()->Type()) {
        temp = InputBox.contents(i)->Name() + "@" + InputTrans(j);
        NewPlace(temp);
        places[temp]->SetX(int(InputBox.contents(i)->GetX() +
                               FindTransition(Transition)->GetX()) / 2);
        places[temp]->SetY(int(InputBox.contents(i)->GetY() +
                               FindTransition(Transition)->GetY()) / 2);
        Typ = InputBox.contents(i)->asHLPlace()->Type();
        typstring = InputBox.contents(i)->asHLPlace()->GetTypString();
        places[temp]->asHLPlace()->SetTyp(Typ, typstring);
        const Map<String, Edge *> &in = InputBox.contents(i)->OutputEdges();
        for(Pix k = in.first(); k; in.next(k)) {
          n = NewEdge(temp, in.contents(k)->Dest().Name());
          anno = FindEdge(InputBox.contents(i)->Name(), in.contents(k)->Dest().Name())->asHLEdge()->VarsToString();
          edges[n]->asHLEdge()->ChangeVars(anno);
        }
        const Map<String, Edge *> &out = places[InputTrans(j)]->OutputEdges();
        for(Pix k = out.first(); k; out.next(k)) {
          n = NewEdge(temp, out.contents(k)->Dest().Name());
          anno = FindEdge(InputTrans(j), out.contents(k)->Dest().Name())->asHLEdge()->VarsToString();
          edges[n]->asHLEdge()->ChangeVars(anno);
        }
        const Map<String, Edge *> &aus = places[InputTrans(j)]->InputEdges();
        for(Pix k = aus.first(); k; aus.next(k)) {
          n = NewEdge(aus.contents(k)->Src().Name(), temp);
          anno = FindEdge(aus.contents(k)->Src().Name(), InputTrans(j))->asHLEdge()->VarsToString();
          edges[n]->asHLEdge()->ChangeVars(anno);
        }
      }
    } 
  }

  for(Pix i = InputBox.first(); i; InputBox.next(i)) {
    const Map<String, Edge *> &in = FindPlace(InputBox.key(i))->OutputEdges();
    VHSet<String> in2;
    for(Pix j = in.first(); j; in.next(j)) {
      char *h = new char[5];
      sprintf(h, "A%d", in.contents(j)->Id());
      String he(h);
      in2.add(he);
      delete [] h;
    }
    RemovePlace(InputBox.key(i));
    for(Pix j = in2.first(); j; in2.next(j)) {
      RemoveEintrag(in2(j));
    }
  }
  for(Pix i = InputTrans.first(); i; InputTrans.next(i)) {
    const Map<String, Edge *> &out = FindPlace(InputTrans(i))->OutputEdges();
    VHSet<String> out2;
    for(Pix j = out.first(); j; out.next(j)) {
      char *h = new char[5];
      sprintf(h, "A%d", out.contents(j)->Id());
      String he(h);
      out2.add(he);
      delete [] h;
    }
    RemovePlace(InputTrans(i));
    for(Pix j = out2.first(); j; out2.next(j)) {
      RemoveEintrag(out2(j));
    }
  }

  for(Pix i = OutputBox.first(); i; OutputBox.next(i)) {
    for(Pix j = OutputTrans.first(); j; OutputTrans.next(j)) {
      if(OutputBox.contents(i)->asHLPlace()->Type() == places[OutputTrans(j)]->asHLPlace()->Type()) {
        temp = OutputBox.contents(i)->Name() + "@" + OutputTrans(j);
        NewPlace(temp);
        places[temp]->SetX(int(OutputBox.contents(i)->GetX() +
                               FindTransition(Transition)->GetX()) / 2);
        places[temp]->SetY(int(OutputBox.contents(i)->GetY() +
                               FindTransition(Transition)->GetY()) / 2);
        Typ = OutputBox.contents(i)->asHLPlace()->Type();
        typstring = OutputBox.contents(i)->asHLPlace()->GetTypString();
        places[temp]->asHLPlace()->SetTyp(Typ, typstring);
        const Map<String, Edge *> &in = OutputBox.contents(i)->InputEdges();
        for(Pix k = in.first(); k; in.next(k)) {
          n = NewEdge(in.contents(k)->Src().Name(), temp);
          anno = FindEdge(in.contents(k)->Src().Name(), OutputBox.contents(i)->Name())->asHLEdge()->VarsToString();
          edges[n]->asHLEdge()->ChangeVars(anno);
        }
        const Map<String, Edge *> &out = places[OutputTrans(j)]->InputEdges();
        for(Pix k = out.first(); k; out.next(k)) {
          n = NewEdge(out.contents(k)->Src().Name(), temp);
          anno = FindEdge(out.contents(k)->Src().Name(), OutputTrans(j))->asHLEdge()->VarsToString();
          edges[n]->asHLEdge()->ChangeVars(anno);
        }
        const Map<String, Edge *> &ein = places[InputTrans(j)]->OutputEdges();
        for(Pix k = ein.first(); k; ein.next(k)) {
          n = NewEdge(temp, ein.contents(k)->Dest().Name());
          anno = FindEdge(InputTrans(j), ein.contents(k)->Dest().Name())->asHLEdge()->VarsToString();
          edges[n]->asHLEdge()->ChangeVars(anno);
        }
      }
    }
  }

  for(Pix i = OutputBox.first(); i; OutputBox.next(i)) {
    const Map<String, Edge *> &in = FindPlace(OutputBox.key(i))->InputEdges();
    VHSet<String> in2;
    for(Pix j = in.first(); j; in.next(j)) {
      char *h = new char[5];
      sprintf(h, "A%d", in.contents(j)->Id());
      String he(h);
      in2.add(he);
      delete [] h;
    }
    RemovePlace(OutputBox.key(i));
    for(Pix j = in2.first(); j; in2.next(j)) {
      RemoveEintrag(in2(j));
    }
  }
  for(Pix i = OutputTrans.first(); i; OutputTrans.next(i)) {
    const Map<String, Edge *> &out = FindPlace(OutputTrans(i))->InputEdges();
    VHSet<String> out2;
    for(Pix j = out.first(); j; out.next(j)) {
      char *h = new char[5];
      sprintf(h, "A%d", out.contents(j)->Id());
      String he(h);
      out2.add(he);
      delete [] h;
    }
    RemovePlace(OutputTrans(i));
    for(Pix j = out2.first(); j; out2.next(j)) {
      RemoveEintrag(out2(j));
    }
  }

  const Map<String, Edge *> &out = FindTransition(Transition)->OutputEdges();
  VHSet<String> out2;
  for(Pix j = out.first(); j; out.next(j)) {
    char *h = new char[5];
    sprintf(h, "A%d", out.contents(j)->Id());
    String he(h);
    out2.add(he);
    delete [] h;
  }
  for(Pix j = out2.first(); j; out2.next(j)) {
    RemoveEintrag(out2(j));
  }
  const Map<String, Edge *> &in = FindTransition(Transition)->InputEdges();
  VHSet<String> in2;
  for(Pix j = in.first(); j; in.next(j)) {
    char *h = new char[5];
    sprintf(h, "A%d", in.contents(j)->Id());
    String he(h);
    in2.add(he);
    delete [] h;
  }
  for(Pix j = in2.first(); j; in2.next(j)) {
    RemoveEintrag(in2(j));
  }

  RemoveTransition(Transition);
  DeleteBox(Box);
}  // HLNet::Substitution

M_Net
HLNet::Synchronize()
  // Synchronisiert das "ubergebene Netz
  // mit Hilfe des Synchronisierers
{
  String actions = "";
  int c;
  Pix pointer = NULL;
  // Collect all terms available
  VHSet<String> ACT = clipboard->ActionTerms();
  VHSet<String> in2, ex2;
  synchro_error = false;
  synchro_error_msg = "";
  // Check arity of included actions and store in in2
  // If include_string is empty ACT is already set up
  if(include_string != "")
  {
    VHSet<String> in = ParseString(include_string);
    for(Pix i = in.first(); i; in.next(i)) {
      if(!in(i).contains("(")) {
        c = 0;
        for(Pix j = ACT.first(); j; ACT.next(j)) {
          if(ACT(j).contains("(")) {
            if(ACT(j).before("(") == in(i)) {
              c++;
              pointer = j;
            }
          }
          else if (ACT(j)==in(i))
            {
              c++;
              pointer = j;
            }
        } // for all actions in ACT
        if(c == 1)
          in2.add(ACT(pointer));
        else if (c>1)
          {
            synchro_error = true;
            synchro_error_msg += ", "+in(i);
          }
       }
      else // in(i) contains "("
          in2.add(in(i));
      // end if(!in(i).contains("("))

      if (synchro_error)
       {
         synchro_error_msg = synchro_error_msg.after(", ");
         synchro_error_msg = "You must give arity for action(s):\n" + synchro_error_msg;
       }
    }// end for all actions included

    ACT &= in2;
  }


  // Same for exclude string and ex2
  if(exclude_string != "" && ! synchro_error) {
    VHSet<String> ex = ParseString(exclude_string);
    for(Pix i = ex.first(); i; ex.next(i)) {
      if(!ex(i).contains("(")) {
        c = 0;
        for(Pix j = ACT.first(); j; ACT.next(j)) {
          if(ACT(j).contains("(")) {
            if(ACT(j).before("(") == ex(i)) {
              c++;
              pointer = j;
            }
          }
          else if (ACT(j)==ex(i))
            {
              c++;
              pointer = j;
            }
        }
        if(c == 1)
          ex2.add(ACT(pointer));
        else if (c>1)
          {
            synchro_error = true;
            synchro_error_msg += ", "+ex(i);
          }
      }
      else // ex(i) contains "("
           ex2.add(ex(i));

      if (synchro_error)
       {
         synchro_error_msg = synchro_error_msg.after(", ");
         synchro_error_msg = "You must give arity for action(s):\n" + synchro_error_msg;
       }
    } // end for all actions excluded
    ACT -= ex2;
    if (ACT.length()==0)
      {
        synchro_error = true;
        synchro_error_msg = "No action left... no synchronization";
      }
  }
  for(Pix k = ACT.first(); k; ACT.next(k)) {
    if(actions != "")
      actions += "," + ACT(k);
    else
      actions = ACT(k);
  }
  M_Net mnet = NULL;

  // if the string is empty, we can just paste (since there's a cut before
  // calling Synchronize() ), set the error flags, and exit the function
  if (actions == "")
   {
     synchro_error = true;     
/*     if (synchro_error_msg=="") // the actions are not in the selection
        synchro_error_msg = "Warning, your selection didn't contain the specified action(s)";
*/
     synchro_error_msg = "Warning, your selection didn't contain the specified action(s)";
     paste();

     return mnet;
   };

  clipboard->DeleteBoxes();

  // the link declarations are needed, in order to keep link terms
  // on transition, when we'll reload the file after synchronization
  clipboard->declared_links = declared_links;
  savemodul(clipboardfile);

  pep_file = new char[clipboardfile.length()];
  strcpy(pep_file, clipboardfile.data());

  GL_AddActions = False;
  scale = 1;

  InitMultiSets();
  InitVarMSets();
  InitAllSymbolTables();
  InitBlockTable();

  mnet = read_HLnet(clipboardfile.data(), (char *)NULL);
  MultiSet spec_actions = MScreate_empty_set();
  spec_actions = StrToMultiSet(actions.data(), NULL);

  if(MSlength(spec_actions) == 0)
    return mnet;

  if (flag&MNET_SYNCHRONIZE) {
    M_Synchronisation(mnet, spec_actions, max_arc_vars);
  }

  if (flag&MNET_RESTRICTION) {
    M_Restriction(mnet, spec_actions);
  }

  MSremove(spec_actions);
 
  if (mnet)
  {
    PrintNet_PEP(mnet);
    String b = NewBox();
    loadmodul(clipboardfile, false, b,true);
    DeleteBox(b);
  }
  else
  { // if we enter here, then we have caught a status from somewhere else...
    paste();
    synchro_error = true;
    synchro_error_msg = "An error occured while launching the synchronizer. Please try again";
  };
  return mnet;
}  // HLNet::Synchronize


/* newImportLink --------------------------------------------------------
 * trans         : transition from where the importation link term comes
 * added_places  : set of places added by the tie operation.
 * lname         : name of the link term (and name of the link) imported
 *                 to the transition
 * ltargs        : args of this link term
 * nof_lterm     : number of occurences of this importation link term,
 *                 in trans
 *-----------------------------------------------------------------------
 * This function add args (nof_lterm times) as output to trans, to the
 * tie place of lname. If the place and the arc doesn't exist, it creates
 * them (and update added_places).
 *----------------------------------------------------------------------*/
void
HLNet::newImportLink ( HLTransition * trans,
                       VHMap<String, HLPlace *> &added_places,
                       String lname, String ltargs, int nof_lterm)
{
  Link *l = &(declared_links[lname]);
  String new_name = "";
  HLPlace * tie_place = (HLPlace *) NULL;

  if (l->get_places().empty())
      { // if the tie place doesn't exist yet, create it
        new_name = NewPlace();
        tie_place = places[new_name]->asHLPlace();
        l -> add_place(tie_place);
        VHSet<Val> ltype = l -> get_type();
        String lstype = l -> get_string_type();
        tie_place -> SetTyp( ltype, lstype );
        added_places[new_name] = tie_place;
      }
  else // else just remember the pointer to this place
      {
        // NB: l -> get_places().contents(l -> get_places().first())
        // don't give the same value as l -> get_places().contents(k)...
	VHMap<String,Place*> k = l -> get_places();
        Pix j = k.first();
        tie_place = (k.contents(j)) ->asHLPlace();
      };

  // if HLEdge from tie_place* to trans* doesn't exist, add it, and set the visibility to 5
  if (!trans->IsOutput(tie_place->Name()))
      {
        new_name = NewEdge(tie_place->Name(),trans->Name());
        trans->EdgeFrom(tie_place->Name())->SetVisibility (5);
        // set the label of this new arc with lterm args as annotation
        if (ltargs == "") ltargs = "dot";
        if (ltargs.contains(","))
             trans -> EdgeFrom(tie_place->Name()) -> asHLEdge()
                    -> changeAnno (("("+ltargs+")"),nof_lterm);
        else
             trans -> EdgeFrom(tie_place->Name()) -> asHLEdge()
                    -> changeAnno (ltargs,nof_lterm);
      }
  else // add lterm args to its annotations
      {
        if (ltargs == "") ltargs = "dot";
        if (ltargs.contains(","))
             trans -> EdgeFrom(tie_place->Name()) -> asHLEdge()
                    -> addAnno (("("+ltargs+")"),nof_lterm);
        else
             trans -> EdgeFrom(tie_place->Name()) -> asHLEdge()
                    -> addAnno (ltargs,nof_lterm);
      };
} // HLNet::newImportLink

/* newExportLink --------------------------------------------------------
 * trans         : transition from where the exportation link term comes
 * added_places  : set of places added by the tie operation.
 * lname         : name of the link term (and name of the link) imported
 *                 to the transition
 * ltargs        : args of this link term
 * nof_lterm     : number of occurences of this exportation link term,
 *                 in trans
 *-----------------------------------------------------------------------
 * This function add args (nof_lterm times) as input of trans, from the
 * tie place of lname. If the place and the arc doesn't exist, it creates
 * them (and update added_places).
 *----------------------------------------------------------------------*/
void
HLNet::newExportLink ( HLTransition * trans,
                       VHMap<String, HLPlace *> &added_places,
                       String lname, String ltargs, int nof_lterm)
{
  Link *l = &(declared_links[lname]);
  String new_name = "";
  HLPlace * tie_place = (HLPlace *) NULL;

  if (l->get_places().empty())
      {
        new_name = NewPlace();
        tie_place = places[new_name]->asHLPlace();
        l -> add_place(tie_place);
        VHSet<Val> ltype = l -> get_type();
        String lstype = l -> get_string_type();
        tie_place -> SetTyp( ltype, lstype );
        added_places[new_name] = tie_place;
      }
  else
      {
        // NB: l -> get_places().contents(l -> get_places().first())
        // don't give the same value as l -> get_places().contents(k)...
        VHMap<String,Place*> k = l->get_places();
        Pix j = k.first();
        tie_place = (k.contents(j)) ->asHLPlace();
      };

  // if HLEdge from trans* to tie_place* doesn't exist, add it
  if (!trans->IsInput(tie_place->Name()))
      {
        new_name = NewEdge(trans->Name(),tie_place->Name());
        trans -> EdgeTo(tie_place->Name())
               -> SetVisibility (5);
        // set the label of this new arc with lterm args
        if (ltargs == "") ltargs = "dot";

        if (ltargs.contains(","))
             trans -> EdgeTo(tie_place->Name()) -> asHLEdge()
                    -> changeAnno (("("+ltargs+")"),nof_lterm);
        else
             trans -> EdgeTo(tie_place->Name()) -> asHLEdge()
                    -> changeAnno (ltargs,nof_lterm);
      }
  else // add lterm args to its annotations
      {
        if (ltargs.contains(","))
             trans -> EdgeTo(tie_place->Name()) -> asHLEdge()
                    -> addAnno (("("+ltargs+")"),nof_lterm);
        else
             trans -> EdgeTo(tie_place->Name()) -> asHLEdge()
                    -> addAnno (ltargs,nof_lterm);
      }; // if (!trans->IsInput(tie_place->Name()))
} //HLNet::newExportLink

void
HLNet::Tie(String links_to_tie)
{
  //build a set of names
  VHSet<String> names_set = ParseLinkString(links_to_tie);

  //build a VHMap from this set
  VHMap<String,int> tie_set(0);
  
  for (Pix i=names_set.first();i;names_set.next(i))
  {
    String n = names_set(i);
    tie_set[n] = 1;
  }
  
  // remember which places were added, in order to calculate
  // their graphical positions, after tiing
  VHMap<String, HLPlace *> added_places((HLPlace *) NULL);
  
  HLTransition *trans = (HLTransition *) NULL;
  
  // for all transitions of the net, tie links which are in tie_set
  for (Pix i=transitions.first();i;transitions.next(i))
  {
    trans = transitions.contents(i)->asHLTransition();
    LinkTerm lterm;
    String lname;
    MultiSET<LinkTerm> ltset = *(new MultiSET<LinkTerm>());

    Pix t;

    // NB: we need to do this initialization, otherwise, it only
    // look at the first element of the Import Multiset...
    ltset += trans->getImport();

    // for each link_importation of trans
    for (t=ltset.first();t;ltset.next(t))
    {
      lterm = ltset.element(t);
      lname = lterm.get_name();
      int nof_lterm = ltset.nof(t);
      String ltargs = lterm.get_args_string();

      if (tie_set[lname])
      { // if this LinkTerm name is in tie_set,
        // add a tie place if it doesn't exist.
        // we assume the link has been already declared

        newImportLink(trans, added_places, lname, ltargs, nof_lterm);

      }; //if (tie_set[lterm.get_name()])

    }; //for (Pix j=trans->getImport().first();j;trans->getImport().next(j))

    // same remark as for the Import Multiset
    ltset.clear();
    ltset += trans->getExport();
     
    // for each link_exportation of trans
    for (t=ltset.first();t;ltset.next(t))
    {
      lterm = ltset.element(t);
      lname = lterm.get_name();
      int nof_lterm = ltset.nof(t);
      String ltargs = lterm.get_args_string();

      if (tie_set[lname])
      { // if this LinkTerm name is in tie_set
        // add a tie place if it doesn't exist
        // we assume the link has been already declared

        newExportLink(trans, added_places, lname, ltargs, nof_lterm);
	
      }; //if (tie_set[lterm.get_name()])
      
    }; //for (Pix j=trans->getExport().first();j;trans->getExport().next(j))

    // delete the tied links in this transition
    trans -> deleteLinkTerms(names_set);

  }; //for (Pix i=transitions->first();i;transitions->next(i))
  
  // for each added place, calculate its position
  for (Pix i=added_places.first();i;added_places.next(i))
        added_places.contents(i)->asTiePlace();

  // for each link tied, clear the place, in order to enable reusing
  // of these link names
  String l_name;
  for (Pix i=names_set.first();i;names_set.next(i))
  {
    l_name = names_set(i);
    if (declared_links.contains(l_name))
       declared_links[l_name].clear_places();
  }  
} // HLNet::Tie

void
HLNet::Restrict()
  // entfernt aus dem clipboard alle komplement"aren
  // Transitionen und f"ugt das Netz wieder ein
{
  String actions;
  int c;
  Pix pointer = NULL;
  VHSet<String> ACT = clipboard->ActionTerms();
  VHSet<String> in2, ex2;
  if(include_string != "") {
    VHSet<String> in = ParseString(include_string);
    for(Pix i = in.first(); i; in.next(i)) {
      if(!in(i).contains("(")) {
        c = 0;
        for(Pix j = ACT.first(); j; ACT.next(j)) {
          if(ACT(j).contains("(")) {
            if(ACT(j).before("(") == in(i)) {
              c++;
              pointer = j;
            }
          }
        }
        if(c == 1)
          in2.add(ACT(pointer));
        else
          in2.add(in(i));
      }
    }
    ACT &= in2;
  }
  if(exclude_string != "") {
    VHSet<String> ex = ParseString(exclude_string);
    for(Pix i = ex.first(); i; ex.next(i)) {
      if(!ex(i).contains("(")) {
        c = 0;
        for(Pix j = ACT.first(); j; ACT.next(j)) {
          if(ACT(j).contains("(")) {
            if(ACT(j).before("(") == ex(i)) {
              c++;
              pointer = j;
            }
          }
        }
        if(c == 1)
          ex2.add(ACT(pointer));
        else
          ex2.add(ex(i));
      }
    }
    ACT -= ex2;
  }
  for(Pix k = ACT.first(); k; ACT.next(k)) {
    if(actions != "")
      actions += "," + ACT(k);
    else
      actions = ACT(k);
  }

  savemodul(clipboardfile);
  M_Net mnet;
  pep_file = new char[clipboardfile.length()];
  strcpy(pep_file, clipboardfile.data());
  GL_AddActions = False;

  InitMultiSets();
  InitVarMSets();
  InitAllSymbolTables();
  InitBlockTable();

  mnet = read_HLnet(clipboardfile.data(), (char *)NULL);
  MultiSet spec_actions = MScreate_empty_set();
  spec_actions = StrToMultiSet(actions.data(), NULL);

  if(MSlength(spec_actions) == 0)
    return;

  M_Restriction(mnet, spec_actions);

  MSremove(spec_actions);
  if (mnet)
  {
    PrintNet_PEP(mnet);
    String b = NewBox();
    loadmodul(clipboardfile, false, b);
  }
}  // HLNet::Restrict


VHSet<String>
HLNet::ParseLinkString(String s)
  // zerlegt einen "ubergebenen String in eine Menge von
  // Linkstermen.
{
  VHSet<String> terms;
  String temp = s;
  String action, colon;
  boolean error_found = false;
  // delete spaces
  if (temp.contains (" "))
     temp = temp.gsub(" ","");
  if (temp.contains ("\t"))
  temp = temp.gsub("\t","");

  while(temp != "") {
    if (!temp.contains("(")){ //only links without arguments
         while (temp.contains(","))
         {
           action = temp.before(",");
           if (action == "")
              { error_found = true; break; }
           temp = temp.after(",");
           terms.add (action);
         }; // while (temp.contains(","))
         if (temp == "")
              { error_found = true; break; }
         terms.add (temp);
		 temp = "";
       }
    else if(temp.contains(",")) { // here, temp.contains("(")
      // link name is the part before '('
      action = temp.before("(");
      colon = temp.before(",");

      if (action == "")
              { error_found = true; break; };

      // if link name and colon in this order: l+(...,
      if((action.length()>0)&&(action.length() < colon.length())) {
        temp = temp.after("(");
        colon = temp.before(")");

        if (colon.contains(",")) {
        // re-add arguments to the link term
          action += "(";
          action += colon;
          action += ")";
        }

        terms.add(action);
        temp = temp.after(")");
        if (temp != "")
         {
           if (temp.First()==',')
               temp = temp.after(",");
           else
             { error_found = true; break; };
         }
      }
	  else {
        // else more than one link term before first '('
        terms.add(colon);
        // continue after colon
        temp = temp.after(",");
      }
    } else { // no '(' and no ',' = only one link term without args
        terms.add(temp);
        temp = "";
      }
  }
  if (error_found==true) // this should not happen
   {
     synchro_error_msg = "\""+s+"\" is not a correct link term list";
     fprintf (stderr, "ERROR: the link term list is not correct\n");
     terms.clear();
   }
  return terms;
}  // HLNet::ParseLinkString


VHSet<String>
HLNet::ParseString(String s)
  // zerlegt einen "ubergebenen String in eine Menge von
  // Aktionstermen.
{
  VHSet<String> terms;
  String temp = s;
  String action, colon;
  boolean error_found = false;
  // delete spaces
  if (temp.contains (" "))
     temp = temp.gsub(" ","");
  if (temp.contains ("\t"))
  temp = temp.gsub("\t","");

  while(temp != "") {
    if (!temp.contains("(")){
         while (temp.contains(","))
         {
           action = temp.before(",");
           if (action == "")
              { error_found = true; break; }
           temp = temp.after(",");
           terms.add (action);
         }; // while (temp.contains(","))
         if (temp == "")
              { error_found = true; break; }
         terms.add (temp);
		 temp = "";
       }
    else if(temp.contains(",")) { // here, temp.contains("(")
      // action is the part before '('
      action = temp.before("(");
      colon = temp.before(",");

      if (action == "")
              { error_found = true; break; };

      // if action and colon in this order: a(...,
      if((action.length()>0)&&(action.length() < colon.length())) {
        temp = temp.after("(");
        colon = temp.before(")");
        int c = 0;
        // count arity
        while(colon.contains(",")) {
          if (colon.before(",")=="")
              { colon = ""; break; };
		  colon = colon.after(",");
          c++;
        };
		if (colon == "")
              { error_found = true; break; }
        else c++;
        // add arity times x as (x,x,x,x)
        if(c > 0) {
          action += "(";
          for(int j = 0; j < c; j++) {
            action += "x";
            if(j < c-1) {
              action += ",";
            }
          }
          action += ")";
        }
        if(action[0] == '^') {
          action = action.after("^");
        }
        terms.add(action);
        temp = temp.after(")");
        if (temp != "")
         {
           if (temp.First()==',')
               temp = temp.after(",");
           else
             { error_found = true; break; };
         }
      }
	  else {
        // else more than one action before first '('
        if(colon[0] == '^') {
          colon = colon.after("^");
        }
        terms.add(colon);
        // continue after colon
        temp = temp.after(",");
      }
    } else { // only one term
      if(temp[0] == '^') {
        temp = temp.after("^");
      }
      // if '(' it must be arity 1 as no colon contained
      if(temp.contains("(")) {
        temp = temp.before("(");
        if (temp == "")
             { error_found = true; break; };
        terms.add(temp+"(x)");
      } else {
        terms.add(temp);
      }
      temp = "";
    }
  }
  if (error_found==true)
   {
     synchro_error_msg = "\""+s+"\" is not a correct action list";
     fprintf (stderr, "ERROR: the term list is not correct\n");
     terms.clear();
   }
  return terms;
}  // HLNet::ParseString


VHSet<String>
HLNet::ActionTerms(void)
  // liefert die Menge der Aktionterme zur"uck, erweitert mit
  // ihrer Arit"at, z.B. A(x,x),A(x,x,x)
{
  VHSet<String> terms;
  VHSet<String> tmp_terms;
  String temp;

  for(Pix i = transitions.first(); i; transitions.next(i)) {
    temp = transitions.contents(i)->asHLTransition()->ActionTerms();
    tmp_terms = ParseString(temp);
    terms |= tmp_terms;
  }
  if (transitions.length() == 0)
     synchro_error_msg = "Your selection doesn't contain any transitions!";

  return terms;
}  // HLNet::ActionTerms

void
HLNet::Choice(String Block1, String Block2, String Name)
{
  VHSet<Val> Typ, schnitt;
  VHMap<Val, VHSet<Val> > Typen(VHSet<Val>(4), 5);
  String typstring;
  int same;
  Val counter = 0;
  Box *ptr = GetBoxFromName(Name);
  Box *ptr2 = GetBoxFromName(Block1);
  Box *ptr3 = GetBoxFromName(Block2);
  ptr->SetX(int((ptr2->GetX() + ptr3->GetX()) / 2));
  ptr->SetY(int((ptr2->GetY() + ptr3->GetY()) / 2));
  VHMap<String, Place *> First((Place *) NULL, 10);
  VHMap<String, Place *> Second((Place *) NULL, 10);
  VHMap<String, Place *> First2((Place *) NULL, 10);
  VHMap<String, Place *> Second2((Place *) NULL, 10);
  for(Pix i = places.first(); i; places.next(i)) {
    if (places.contents(i)->BoxName() == Block1 &&
        places.contents(i)->State() == 0) {
      First[places.contents(i)->Name()] = places.contents(i);
      for(Pix j = Typen.first(); j; Typen.next(j)) {
        same = (Typen.contents(j) == places.contents(i)->asHLPlace()->Type());
        schnitt = places.contents(i)->asHLPlace()->Type();
        schnitt &= Typen.contents(j);
        if(schnitt.length() != 0 & !same) {
          DeleteBox(Name);
          return;
        }
      }
      Typen[counter.asString()] = places.contents(i)->asHLPlace()->Type();
      counter = counter + 1;
    }
  }
  for(Pix i = places.first(); i; places.next(i)) {
    if (places.contents(i)->BoxName() == Block2 &&
        places.contents(i)->State() == 0) {
      Second[places.contents(i)->Name()] = places.contents(i);
      for(Pix j = Typen.first(); j; Typen.next(j)) {
        same = (Typen.contents(j) == places.contents(i)->asHLPlace()->Type());
        schnitt = places.contents(i)->asHLPlace()->Type();
        schnitt &= Typen.contents(j);
        if(schnitt.length() != 0 & !same) {
          DeleteBox(Name);
          return;
        }
      }
      Typen[counter.asString()] = places.contents(i)->asHLPlace()->Type();
      counter = counter + 1;
    }
  }
  for(Pix i = places.first(); i; places.next(i)) {
    if (places.contents(i)->BoxName() == Block1 &&
        places.contents(i)->State() == 1) {
      First2[places.contents(i)->Name()] = places.contents(i);
      for(Pix j = Typen.first(); j; Typen.next(j)) {
        same = (Typen.contents(j) == places.contents(i)->asHLPlace()->Type());
        schnitt = places.contents(i)->asHLPlace()->Type();
        schnitt &= Typen.contents(j);
        if(schnitt.length() != 0 & !same) {
          DeleteBox(Name);
          return;
        }
      }
      Typen[counter.asString()] = places.contents(i)->asHLPlace()->Type();
      counter = counter + 1;
    }
  }
  for(Pix i = places.first(); i; places.next(i)) {
    if (places.contents(i)->BoxName() == Block2 &&
        places.contents(i)->State() == 1) {
      Second2[places.contents(i)->Name()] = places.contents(i);
      for(Pix j = Typen.first(); j; Typen.next(j)) {
        same = (Typen.contents(j) == places.contents(i)->asHLPlace()->Type());
        schnitt = places.contents(i)->asHLPlace()->Type();
        schnitt &= Typen.contents(j);
        if(schnitt.length() != 0 & !same) {
          DeleteBox(Name);
          return;
        }
      }
      Typen[counter.asString()] = places.contents(i)->asHLPlace()->Type();
      counter = counter + 1;
    }
  }
  if (First.length() == 0 || Second.length() == 0 || First2.length() == 0 || Second2.length() == 0) {
    DeleteBox(Name);
    return;
  }
  String PlaceName;
  String temp, n, anno;
  for(Pix i = First.first(); i; First.next(i)) {
    for(Pix j = Second.first(); j; Second.next(j)) {
      if(First.contents(i)->asHLPlace()->Type() == Second.contents(j)->asHLPlace()->Type()) {
        temp = First.contents(i)->Name() + "@" + Second.contents(j)->Name();
        NewPlace(temp);
        places[temp]->SetX(int(First.contents(i)->GetX() +
                               Second.contents(j)->GetX()) / 2);
        places[temp]->SetY(int(First.contents(i)->GetY() +
                               Second.contents(j)->GetY()) / 2);
        places[temp]->setBox(Name, ptr);
        places[temp]->asHLPlace()->SetEntry();
        Typ = First.contents(i)->asHLPlace()->Type();
        typstring = First.contents(i)->asHLPlace()->GetTypString();
        places[temp]->asHLPlace()->SetTyp(Typ, typstring);
        places[temp]->asHLPlace()->DeleteInitMarking();
        const Map<String, Edge *> &in = First.contents(i)->OutputEdges();
        for(Pix k = in.first(); k; in.next(k)) {
          n = NewEdge(temp, in.contents(k)->Dest().Name());
          anno = FindEdge(First.contents(i)->Name(), in.contents(k)->Dest().Name())->asHLEdge()->VarsToString();
          edges[n]->asHLEdge()->ChangeVars(anno);
        }
        const Map<String, Edge *> &out = Second.contents(j)->OutputEdges();
        for(Pix k = out.first(); k; out.next(k)) {
          n = NewEdge(temp, out.contents(k)->Dest().Name());
          anno = FindEdge(Second.contents(j)->Name(), out.contents(k)->Dest().Name())->asHLEdge()->VarsToString();
          edges[n]->asHLEdge()->ChangeVars(anno);
        }
      }
    } 
  }

  for(Pix i = First.first(); i; First.next(i)) {
    const Map<String, Edge *> &in = FindPlace(First.key(i))->OutputEdges();
    VHSet<String> in2;
    for(Pix j = in.first(); j; in.next(j)) {
      char *h = new char[5];
      sprintf(h, "A%d", in.contents(j)->Id());
      String he(h);
      in2.add(he);
      delete [] h;
    }
    RemovePlace(First.key(i));
    for(Pix j = in2.first(); j; in2.next(j)) {
      RemoveEintrag(in2(j));
    }
  }
  for(Pix i = Second.first(); i; Second.next(i)) {
    const Map<String, Edge *> &out = FindPlace(Second.key(i))->OutputEdges();
    VHSet<String> out2;
    for(Pix j = out.first(); j; out.next(j)) {
      char *h = new char[5];
      sprintf(h, "A%d", out.contents(j)->Id());
      String he(h);
      out2.add(he);
      delete [] h;
    }
    RemovePlace(Second.key(i));
    for(Pix j = out2.first(); j; out2.next(j)) {
      RemoveEintrag(out2(j));
    }
  }

  /*for(Pix i = First.first(); i; First.next(i)) {
    RemovePlace(First.key(i));
  }
  for(Pix i = Second.first(); i; Second.next(i)) {
    RemovePlace(Second.key(i));
  }*/

  for(Pix i = First2.first(); i; First2.next(i)) {
    for(Pix j = Second2.first(); j; Second2.next(j)) {
      if(First2.contents(i)->asHLPlace()->Type() == Second2.contents(j)->asHLPlace()->Type()) {
        temp = First2.contents(i)->Name() + "@" + Second2.contents(j)->Name();
        NewPlace(temp);
        places[temp]->SetX(int(First2.contents(i)->GetX() +
                               Second2.contents(j)->GetX()) / 2);
        places[temp]->SetY(int(First2.contents(i)->GetY() +
                               Second2.contents(j)->GetY()) / 2);
        places[temp]->setBox(Name, ptr);
        places[temp]->asHLPlace()->SetExit();
        Typ = First2.contents(i)->asHLPlace()->Type();
        typstring = First2.contents(i)->asHLPlace()->GetTypString();
        places[temp]->asHLPlace()->SetTyp(Typ, typstring);
        const Map<String, Edge *> &in = First2.contents(i)->InputEdges();
        for(Pix k = in.first(); k; in.next(k)) {
          n = NewEdge(in.contents(k)->Src().Name(), temp);
          anno = FindEdge(in.contents(k)->Src().Name(), First2.contents(i)->Name())->asHLEdge()->VarsToString();
          edges[n]->asHLEdge()->ChangeVars(anno);
        }
        const Map<String, Edge *> &out = Second2.contents(j)->InputEdges();
        for(Pix k = out.first(); k; out.next(k)) {
          n = NewEdge(out.contents(k)->Src().Name(), temp);
          anno = FindEdge(out.contents(k)->Src().Name(), Second2.contents(j)->Name())->asHLEdge()->VarsToString();
          edges[n]->asHLEdge()->ChangeVars(anno);
        }
      }
    } 
  }

  for(Pix i = First2.first(); i; First2.next(i)) {
    const Map<String, Edge *> &in = FindPlace(First2.key(i))->InputEdges();
    VHSet<String> in2;
    for(Pix j = in.first(); j; in.next(j)) {
      char *h = new char[5];
      sprintf(h, "A%d", in.contents(j)->Id());
      String he(h);
      in2.add(he);
      delete [] h;
    }
    RemovePlace(First2.key(i));
    for(Pix j = in2.first(); j; in2.next(j)) {
      RemoveEintrag(in2(j));
    }
  }
  for(Pix i = Second2.first(); i; Second2.next(i)) {
    const Map<String, Edge *> &out = FindPlace(Second2.key(i))->InputEdges();
    VHSet<String> out2;
    for(Pix j = out.first(); j; out.next(j)) {
      char *h = new char[5];
      sprintf(h, "A%d", out.contents(j)->Id());
      String he(h);
      out2.add(he);
      delete [] h;
    }
    RemovePlace(Second2.key(i));
    for(Pix j = out2.first(); j; out2.next(j)) {
      RemoveEintrag(out2(j));
    }
  }

  /*for(Pix i = First2.first(); i; First2.next(i)) {
    RemovePlace(First2.key(i));
  }
  for(Pix i = Second2.first(); i; Second2.next(i)) {
    RemovePlace(Second2.key(i));
  }*/

  for (Pix i = places.first(); i;places.next(i)) {
    if (places.contents(i)->BoxName() == Block1) {
      places.contents(i)->setBox(Name, ptr);
    }
    if (places.contents(i)->BoxName() == Block2) {
      places.contents(i)->setBox(Name, ptr);
    }
  }
  for (Pix i = transitions.first(); i; transitions.next(i)) {
    if (transitions.contents(i)->BoxName() == Block1) {
      transitions.contents(i)->setBox(Name, ptr);
    }
    if (transitions.contents(i)->BoxName() == Block2) {
      transitions.contents(i)->setBox(Name, ptr);
    }
  }
  for (Pix i = ptransitions.first(); i;ptransitions.next(i)) {
    if (ptransitions.contents(i)->BoxName() == Block1) {
      ptransitions.contents(i)->setBox(Name, ptr);
    }
    if (ptransitions.contents(i)->BoxName() == Block2) {
      ptransitions.contents(i)->setBox(Name, ptr);
    }
  }
  DeleteBox(Block1);
  DeleteBox(Block2);
}  // HLNet::Choice

int
HLNet::IsAbstract(String Box)
  // enth"alt die Box Macros?
{
  if (!boxes.contains(Box)) {
    return 0;
  }
  VHSet<String> nodes;
  VHSet<String> macros;
  for(Pix i = transitions.first(); i; transitions.next(i)) {
    if(transitions.contents(i)->BoxName() == Box) {
      nodes.add(transitions.key(i));
      macros |= transitions.contents(i)->asHLTransition()->GetMacros();
    }
  }
  for(Pix i = places.first(); i; places.next(i)) {
    if(places.contents(i)->BoxName() == Box) {
      nodes.add(places.key(i));
      macros |= places.contents(i)->asHLPlace()->GetMacros();
    }
  }
  for(Pix i = edges.first(); i; edges.next(i)) {
    if(nodes.contains(edges.contents(i)->Src().Name()) && nodes.contains(edges.contents(i)->Dest().Name())) {
      macros |= edges.contents(i)->asHLEdge()->GetMacros();
    }
  }
  return (!macros.empty());
} // HLNet::IsAbstract

  //*** Dateiende HLComposition.cc ***
