// HLEdge.cc
// Tobias Himstedt 1995
// Burkhard Bieber 1995/96
// last-update: 07.03.96
// Cecile Bui Thanh: added some functions for the Tie operator (2001)
// last-update: Dec. 2001

  //**********
  // includes
  //**********

#include <ctype.h>
#include <string.h>
#include "HLEdge.h"
#include "HLNet.h"
#include "Val.h"

  //*********************
  // Methoden fr HLEdge
  //*********************

HLEdge::HLEdge(const Net &n,const String &Type) : Edge(n, Type), abstract(false)
  // Standard Konstruktor
{
  base_classes.add("Edge");
  net->asHLNet()->DefaultHLEdgeVars(rx_vars, ry_vars);
  Anno = "a";
  vars.add(Val(Anno));
}  // HLEdge::HLEdge

HLEdge::HLEdge(Vertex &f, Vertex &t, const Net &n, const String &Type) :
  Edge(f, t, n, Type), abstract(false) 
  // Standard Konstruktor
{
  base_classes.add("Edge");
  net->asHLNet()->DefaultHLEdgeVars(rx_vars, ry_vars);
  Anno = "a";
  vars.add(Val(Anno));
}  // HLEdge::HLEdge

HLEdge::HLEdge(const HLEdge &e) : Edge(e), Anno(e.Anno), abstract(e.abstract),
   rx_vars(e.rx_vars), ry_vars(e.ry_vars)
  // Copy Konstruktor
{
  base_classes.add("Edge");
  if(!abstract) {
    for(Pix i = e.Vars().first(); i; e.Vars().next(i)) {
      vars.add(e.Vars().element(i));
    }
  }
  Anno = e.Anno;
}  // HLEdge::HLEdge

void
HLEdge::Write(ostream &f) const
  // Standard Ausgabe auf Stream f
{
  Edge::Write(f);
  if (abstract)
    f << "p\"" << Anno << '"';
  else
    f << "p\"" << VarsToString() << '"';
  if (rx_vars != net->asHLNet()->DefaultHLEdgeVarsX() ||
    ry_vars != net->asHLNet()->DefaultHLEdgeVarsY())
    f << 'q' << rx_vars << '@' << ry_vars;
}  // HLEdge::Write

void 
HLEdge::ReadOptional(istream &f)
  // Angaben fr Kante von Stream f einlesen
{
  String new_vars;
  char c;
  char dummy;

  Edge::ReadOptional(f);
  dimension = weight;
  f >> c;
  switch (c) {
    case 'p':
      f >> c;  // Anfhrungszeichen berlesen
      readline(f, new_vars, '"');
      if (new_vars.contains("$")) {
        abstract = true;
        Anno = new_vars;
      } else {
        abstract = false;
        ChangeVars(new_vars);
        Anno = new_vars;
      }
      break;
    case 'q':
      f >> rx_vars >> dummy >> ry_vars;
      break;
    default:
      f.putback(c);
  }  // switch ber c
}  // HLEdge::ReadOptional

VHSet<String>
HLEdge::GetMacros(void)
  // Macros herausfiltern
{
  VHSet<String> macros;
  if(!Anno.contains("$"))
    return macros;
  String davor, dahinter;
  davor = Anno.after("$");
  dahinter = davor.before("$");
  while(davor != "") {
    macros.add(dahinter);
    davor = davor.after("$");
    davor = davor.after("$");
    dahinter = davor.before("$");
  }
  return macros;
}

void
HLEdge::InsertMacro(String macro, String ersetzung)
  // Macros ersetzen
{
  String davor, dahinter;
  macro = "$" + macro + "$";
  while(Anno.contains(macro)) {
    davor = Anno.before(macro);
    dahinter = Anno.after(macro);
    Anno = davor + ersetzung + dahinter;
  }
}

void
HLEdge::Instantiate()
  // Annotationen in Kante eintragen
{
  ChangeVars(Anno);
  abstract = false;
}

void
HLEdge::addAnno(String new_Anno, int nb_occ)
{
  if (nb_occ > 0)
  {
    for (int i=nb_occ;i>0;i--)
    {
      Anno = Anno+","+new_Anno;
    };
    ChangeVars(Anno);
  }; // end if (nb_occ>0)
} // HLEdge::addAnno

void
HLEdge::changeAnno(String new_Anno, int nb_occ)
{
  if (nb_occ > 0)
  {
    Anno = new_Anno;
    for (int i=nb_occ-1;i>0;i--)
    {
      Anno = Anno+","+new_Anno;
    };
    ChangeVars(Anno);
  }; // end if (nb_occ>0)
} // HLEdge::changeAnno

void
HLEdge::ChangeVars(const String &new_vars)
  // Variablen werden gendert.
  // Neue Variablen sind durch String new_vars gegeben.
{
  vars.clear();
  
  // Server communication
  String from_node=from->Name();
  String to_node=to->Name();
  String tmp_vars=new_vars;
  net->asHLNet()->ChangeVars(from_node.data(), to_node.data(), tmp_vars.data());

  int co = 0;
  int weight = 0;
  char *s = strdup(new_vars), *s2 = s;
  
  String together = "";
  s = strtok(s,",");
  while (s) {
    String t = String(s);
    if(t.First() == '(') {
      if(together == "") {
        t = t.after("(");
        together += "(";
        together += t;
      } else {
        together += ",";
        together += t;
      }
      co++;
    }
    else {
      if(t.Last() == ')') {
        together += ",";
        // t = t.before(")");
        together += t;
        co++;
        dimension = co;
        Tupel tupel = StringToTupel(together);
        vars.add(tupel);
        weight++;
        together = "";
        co = 0;
      }
      else {
        if(together != "") {
          together += ",";
          together += t;
          co++;
        }
        else {
	  String temp = String(s).gsub(" ","");
          // temp.setType(temp.digit());
          Tupel tupel = StringToTupel(temp);
          vars.add(tupel);
          weight++;
          dimension = 1;
        }
      }
    }
    String(s).gsub(" ","");
    s = strtok(NULL, ",");
  }  // while noch etwas von s zu bearbeiten
  free(s2);
  SetWeight(weight/*vars.length()*/);
}  // HLEdge::ChangeVars

String
HLEdge::VarsToString(void) const
  // Multimenge der Variablen wird in String umgewandelt
  // Hufigkeiten der Elemente werden noch nicht bercksichtigt.
  // Elemente werden einfach sooft ausgegben, wie ntig
  // also a,a,a,a   statt 4*x o..
{
  int j, z;
  String l;
  Pix i, n;

  for (i = vars.first(); i; vars.next(i)) {
    n = i;
    z = 1;
    vars.next(n);
    for (j = 0; j < vars.nof(i); j++) {
      l += vars.element(i).asString();
      if (j+1 < vars.nof(i)) 
        l += ",";
    }  // for j
    if (n)
      l += ",";
    z++;
  }  // for i ber die Variablen
  return l;
}  // HLEdge::VarsToString

void
HLEdge::ChangeForeach()
  // Foreach in Anno aufl"osen
{
  int j;
  String vor, nach, danach, davor, ersetzung, rest;
  String set, Tupel, neu;
  while(Anno.contains("$FOREACH$")) {
    danach = Anno.after("$FOREACH$");
    davor  = Anno.before("$FOREACH$");
    ersetzung = danach.before(")");
    rest = danach.after(")");
    ersetzung = ersetzung + ")";
    set = ersetzung.before("}");
    set = set + "}";
    VHSet<Val> menge;
    StringToSet(set, menge);
    Tupel = ersetzung.after(":");
    neu = "";
    j = 0;
    for(Pix i = menge.first(); i; menge.next(i), j++) {
      String newTupel = Tupel;
      while(newTupel.contains("$ELEMENT$")) {
        vor =  newTupel.before("$ELEMENT$");
        nach = newTupel.after("$ELEMENT$");
        newTupel = vor + menge(i).asString() + nach;
      }
      while(newTupel.contains("$INDEX$")) {
        vor =  newTupel.before("$INDEX$");
        nach = newTupel.after("$INDEX$");
        newTupel = vor + IntToString(j) + nach;
      }
      if(neu != "")
        neu = neu + "),(" + newTupel;
      else
        neu = newTupel + ")";
    }
    Anno = davor + neu + rest;
  }
}  // HLEdge::ChangeForeach

void
HLEdge::StringToSet(String s, SET<Val> &set) const
  // Konvertiert String nach Set
{
  set.clear();
  s = s.gsub("{", "");
  s = s.gsub("}", "");
  String sv = s.before(",");
  while (sv != "") {
    StringRangeAppendSet(sv, set);
    s = s.after(",");
    sv = s.before(",");
  }  // while
  StringRangeAppendSet(s, set);
}  // HLEdge::StringToSet

void
HLEdge::StringRangeAppendSet(String range, SET<Val> &set) const
  // Fgt an die Menge set einen Wert an, der aus dem String range
  // geewonnen wird. range kann aber auch eine Bereich start..ende sein
  // In diesem Fall werden alle Werte von start bis end zur Menge
  // zugefgt.
{
  String lower = range.before("..");
  if (lower != "") {
    Val anfang = StringToVal(lower);
    String upper = range.after("..");
    Val ende = StringToVal(upper);
    if (anfang.Type() != INTEGER || ende.Type() != INTEGER) {
      Error("Lower and upper bounds must be Integers");
      return ;
    }  // if
    if (ende < anfang) { 
      Val tmp = anfang;
      anfang = ende;
      ende = tmp;
    }  // if
    for (int p = anfang.asInt(); p <= ende.asInt(); p++) {
      set.add(p);
    }
  }  // if
  else {
    set.add(StringToVal(range));
  }
}  // HLEdge::StringRangeAppendSet

int
HLEdge::ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[])
  // TCL-Schnittstelle
{
  if (argc != 1) {
    Tcl_AppendResult(ip, "wrong #args for Edge get", 
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl fr Edge get

  if (!strcmp(argv[0], "Vars")) {
    if(abstract)
      Result(ip, Anno);
    else
      Result(ip, VarsToString());
  }

  else if (!strcmp(argv[0], "VarsX"))
    sprintf(ip->result, "%d", rx_vars);

  else if (!strcmp(argv[0], "VarsY"))
    sprintf(ip->result, "%d", ry_vars);

  else
    return Edge::ObjectGet(ip, argc, argv);
  return TCL_OK;
}  // HLEdge::ObjectGet

int
HLEdge::ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[])
  // TCL-Schnittstelle
{
  char *v = NULL;

  if (argc < 2) {
    Tcl_AppendResult(ip, "wrong #args for Edge set",
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl fr Edge set
  Tk_ArgvInfo parse_table[] = {
    {"Vars", TK_ARGV_STRING, (char *) NULL, (char *) &v,
     "Variables of  arc"},
    {"VarsX", TK_ARGV_INT, (char *) NULL, (char *) &rx_vars,
     "Relativ x-position of variables annotation"},
    {"VarsY", TK_ARGV_INT, (char *) NULL, (char *) &ry_vars,
     "Relativ y-position of variables annotation"},
    {NULL, TK_ARGV_END, NULL, NULL, NULL}};
  if (Tk_ParseArgv(ip, NULL, &argc, argv, parse_table, 
    TK_ARGV_DONT_SKIP_FIRST_ARG | TK_ARGV_NO_ABBREV) != TCL_OK)
    return TCL_ERROR;
  if (v) {
    String var(v);
    if(var.contains("$")) {
      abstract = true;
      Anno = var;
    } else {
      ChangeVars(v);
      abstract = false;
    }
  }
  if (argc > 0)
    return Edge::ObjectSet(ip, argc, argv);
  return TCL_OK;
}  // HLEdge::ObjectSet

  //*** Dateiende HLEdge.cc ***

