// HLEdge.h
// Tobias Himstedt 1995
// Burkhard Bieber 1995/96
// Cecile Bui Thanh: added some functions for the Tie operator (2001)
// last-update: Dec. 2001

#ifndef __HLEDGE_H__
#define __HLEDGE_H__

  //**********
  // includes
  //**********

#include "../Low/Edge.h"
#include "../include/MultiSet.h"
#include "Val.h"

  //*********
  // Klassen
  //*********

class HLEdge : public Edge {
  private:
      // Datenelemente
    MultiSET<Val> vars;  // Multimenge der Variablen an der HL Kante
    String Anno;
    bool abstract;
    int rx_vars, ry_vars;   // rel. Pos. der Multimenge
    int dimension;
      // I/O-Methoden
    virtual void Write(ostream &f) const;
    virtual void ReadOptional(istream &f);
  public:
      // Konstruktoren
    HLEdge(Vertex &f, Vertex &t, const Net &n, const String &Type = "HLEdge");
    HLEdge(const Net &n, const String &Type = "HLEdge");
    HLEdge(const HLEdge &e);
      // Cast
    virtual HLEdge *asHLEdge() const;
      // liefert die Multimenge der Variablen
    const MultiSET<Val> &Vars(void) const;
      // Macros aus Annotation
    VHSet<String> GetMacros(void);
	String getAnno(void) const;
      //add new_Anno to Anno, nb_occ times and update vars
	void addAnno(String new_Anno, int nb_occ=1);
     //change Anno and update vars
	void changeAnno(String new_Anno, int nb_occ=1);
    void InsertMacro(String macro, String ersetzung);
    void Instantiate(void);
    void ChangeForeach(void);
    void StringToSet(String s, SET<Val> &set) const;
    void StringRangeAppendSet(String range, SET<Val> &set) const;
      // Konvertierung von String zu MultiSet
    String VarsToString(void) const;
      // Multimenge ndern
    void ChangeVars(const String &new_vars);
      // wichtige Daten der Kante kopieren
    void clone(const HLEdge& e);
      // Tcl-Schnittstelle
    int ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[]);
    int ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[]);
};  // class HLEdge

  //********************
  // Inlines fr HLEdge
  //********************

inline void
HLEdge::clone(const HLEdge &e)
{
  if (Id() == e.Id())
    return;
  vars = e.vars;
  abstract = e.abstract;
  Anno = e.Anno;
  rx_vars = e.rx_vars;
  ry_vars = e.ry_vars;
  dimension = e.dimension;
}

inline const MultiSET<Val> &
HLEdge::Vars(void) const
{
  return vars;
}  // HLEdge::Vars

inline HLEdge *
HLEdge::asHLEdge() const
{
  return (HLEdge *) this;
}  // HLEdge::asHLEdge

inline String
HLEdge::getAnno(void) const
{
  return Anno;
} // HLEdge::getAnno

#endif  // #ifndef __HLEDGE_H__

  //*** Dateiende HLEdge.h ***

