// HLNet.cc
// Tobias Himstedt 1995
// Burkhard Bieber 1995/96
// Ulf Fildebrandt 1997/98
// Christian Stehno: changed time.h to sys/time.h (2000)
// Cecile Bui Thanh: added asynchronous links (2001)
// last-update: Dec. 2001

  //**********
  // includes
  //**********

#include <unistd.h>
#include <sys/time.h>
#include "../include/SLStack.h"
#include "HLNet.h"
#include "HLBox.h"
#include "../include/String.h"

extern "C" {
#include "frontend.h"
#include "../../backend/evaltree.h"
#include "../../backend/box_sub.h"
#include "../../backend/mnet.h"
#include "../../backend/tables.h"
#include "../../backend/block.h"
#include "../../backend/f_parse.h"
#include "../../backend/main.h"
}

extern VHMap<String, HLNet *> hlnets;

  //*******************
  // Methods for HLNet
  //*******************

//HLNet::HLNet() : rx_place_type(20), ry_place_type(-5),
//  rx_trans_at(20), ry_trans_at(1),
//  rx_trans_vt(20), ry_trans_vt(-12),
//  rx_edge_var(10), ry_edge_var(0),
HLNet::HLNet() : rx_place_type(20), ry_place_type(-3),
  rx_trans_at(20), ry_trans_at(-12),
  rx_trans_vt(20), ry_trans_vt(12),
  rx_trans_lt(20), ry_trans_lt(0),
  rx_edge_var(10), ry_edge_var(0),
  possible_value_set(EmptySet, 10), alle(true),
  cache(true), preAnalyse(true), inAnalyse(true),
  timeout(true), clipboard((HLNet *) NULL)
  // Construktor
{
  srand((unsigned int) getpid);
  object_size = DefaultHLSize;  // overwrite LL-value
  declared_links = *(new VHMap<String,Link>(EmptyLink,10));
  rx_trans_mean = 20;
  ry_trans_mean = -24;
}  // HLNet::HLNet

HLNet *
HLNet::asHLNet() const
  // Cast
{
  return (HLNet *) this;
}  // HLNet::asHLNet

Block *
HLNet::AllocBlock() const
{
  return new HLBlock(*this);
}  // HLNet::AllocBlock

Box *
HLNet::AllocBox() const
{
  return new HLBox(*this);
}  // HLNet::AllocBlock

Place *
HLNet::AllocPlace() const
{
  return new HLPlace(*this);
}  // HLNet::AllocPlace

Transition *
HLNet::AllocTransition() const
{
  return new HLTransition(*this);
}  // HLNet::AllocTransition

PTransition *
HLNet::AllocPTransition() const
{
  return new HLPTransition(*this);
}  // HLNet::AllocPTransition

Edge *
HLNet::AllocEdge() const
{
  return new HLEdge(*this);
}  // HLNet::AllocEdge

Edge *
HLNet::AllocEdgeFT(Vertex &f, Vertex &t) const
{
  return new HLEdge(f, t, *this);
}  // HLNet::AllocEdgeFT

Edge *
HLNet::AllocPEdge() const
{
  return new HLEdge(*this);
}  // HLNet::AllocPEdge

Edge *
HLNet::AllocPEdgeFT(Vertex &f, Vertex &t) const
{
  return new HLEdge(f, t, *this);
}  // HLNet::AllocPEdgeFT

void
HLNet::removeLink (String l_name)
{
  if (declared_links.contains(l_name))
  {
    declared_links.del(l_name);
    for (Pix i= transitions.first();i;transitions.next(i))
         transitions.contents(i) -> asHLTransition()
                -> deleteLinkTerms(l_name);
  } // end if (declared_links.contains(l_name))
}  // HLNet::removeLink

void
HLNet::changeLinkType(String l_name, String l_type)
{
  if (declared_links.contains(l_name)) // else do nothing
    {
      VHSet<Val> new_type = *(new VHSet<Val>(10));
      stringToType(l_type,new_type);
      if (!new_type.empty()) // else don't change
      {
        int old_arity = declared_links[l_name].get_arity();
        declared_links[l_name].set_type(l_type, new_type);
        int new_arity = declared_links[l_name].get_arity();
        if (old_arity != new_arity)
        // then delete every link terms with old type
        {
          HLTransition * trans = (HLTransition *) NULL;
          for (Pix k=transitions.first(); k ; transitions.next(k))
          {
            trans = transitions.contents(k) -> asHLTransition();
	    trans -> deleteLinkTerms(l_name);
          }; // for each transition
        }; // if (old_arity != new_arity)
      }; // if (!new_type.empty())
    }; // if (declared_links.contains(l_name))
} // HLNet::changeLinkType

VHSet<String> 
HLNet::ActiveTransitions()
  // returns set of active transitions
{
  if(activeTransitions.empty()) {
    VHSet<String> ac = Net::ActiveTransitions();
    activeTransitions.clear();
    char buff[32];
    char buff2[256];
    String answer;

/*    int akt = 0;
    for (Pix i = ac.first(); i; ac.next(i)) {
      if(count != 0) {
        write(Verbindungen[akt]->pipe1[1], "IsActive", 32);
        read(Verbindungen[akt]->pipe2[0], buff, 32);
        write(Verbindungen[akt]->pipe1[1], ac(i).data(), 32);
        read(Verbindungen[akt]->pipe2[0], buff, 32);
        answer = buff;
        if(answer == "Yes")
          activeTransitions.add(ac(i));
        //do {
          (++akt) %= count; 
        //} while (notsave[akt] != 0);
      } else {
        if (!SearchBinding(*transitions[ac(i)]).empty()) {
          activeTransitions.add(ac(i));
        }
      }
    }  // for Pix on active transitions
*/

    VHMap<String, int> IPtoT(0, 2);

    if(count != 0) {
      int akt = 0;
      Pix b = ac.first();
      while(akt < count && b) {
        write(Verbindungen[akt]->pipe1[1], "IsActive", 32);
        read(Verbindungen[akt]->pipe2[0], buff, 32);
        write(Verbindungen[akt]->pipe1[1], ac(b).data(), 32);
        IPtoT[ac(b)] = Verbindungen[akt]->pipe2[0];
        akt++;
        ac.next(b);
      }
      boolean Ende = true;
      int last = akt;
      while(b) {
        for(int f = 0;f < akt;f++) {
          read(Verbindungen[f]->pipe2[0], buff, 32);
          answer = buff;
          if(answer == "Yes") {
            for(Pix d = IPtoT.first(); d; IPtoT.next(d)) {
              if(IPtoT.contents(d) == Verbindungen[f]->pipe2[0]) {
                activeTransitions.add(IPtoT.key(d));
                const Map<String, Edge *> &input_edges = transitions[IPtoT.key(d)]->InputEdges();
                Pix i;
                HLEdge *hledge;
                HLPlace *inp;

                String Markierung = "";

                for(i = input_edges.first();i;input_edges.next(i)) {
                  hledge = input_edges.contents(i)->asHLEdge();
                  inp = hledge->Src().asHLPlace();
                  MultiSET<Val> tmp_vals = inp->Marking();
                  for(Pix mar = tmp_vals.first();mar;tmp_vals.next(mar)) {
                    if(Markierung == "")
                      Markierung = tmp_vals.element(mar).asString();
                    else
                      Markierung += "," + tmp_vals.element(mar).asString();
                  }
                }
                write(Verbindungen[f]->pipe1[1], "OK", 32);
                read(Verbindungen[f]->pipe2[0], buff2, 256);
                answer = buff2;
                transitions[IPtoT.key(d)]->asHLTransition()->AddMarking(Markierung, 1);
                transitions[IPtoT.key(d)]->asHLTransition()->AddMarking(Markierung, answer);
                IPtoT.del(IPtoT.key(d));
              }
            }
          } else {
            for(Pix d = IPtoT.first(); d; IPtoT.next(d)) {
              if(IPtoT.contents(d) == Verbindungen[f]->pipe2[0]) {
                activeTransitions.del(IPtoT.key(d));
                const Map<String, Edge *> &input_edges = transitions[IPtoT.key(d)]->InputEdges();
                Pix i;
                HLEdge *hledge;
                HLPlace *inp;

                String Markierung = "";

                for(i = input_edges.first();i;input_edges.next(i)) {
                  hledge = input_edges.contents(i)->asHLEdge();
                  inp = hledge->Src().asHLPlace();
                  MultiSET<Val> tmp_vals = inp->Marking();
                  for(Pix mar = tmp_vals.first();mar;tmp_vals.next(mar)) {
                    if(Markierung == "")
                      Markierung = tmp_vals.element(mar).asString();
                    else
                      Markierung += "," + tmp_vals.element(mar).asString();
                  }
                }
                transitions[IPtoT.key(d)]->asHLTransition()->AddMarking(Markierung, 0);
                transitions[IPtoT.key(d)]->asHLTransition()->AddMarking(Markierung, "");
                IPtoT.del(IPtoT.key(d));
              }
            }
          }
          boolean neu = false;
          while(!neu) {
            if(b) {
              write(Verbindungen[f]->pipe1[1], "IsActive", 32);
              read(Verbindungen[f]->pipe2[0], buff, 32);
              write(Verbindungen[f]->pipe1[1], ac(b).data(), 32);
              IPtoT[ac(b)] = Verbindungen[f]->pipe2[0];
              neu = true;
              ac.next(b);
            } else if(Ende) {
              last = f;
              Ende = false;
              neu = true;
            }
          }
        }
      }

      for(int f = 0;f < last;f++) {
        read(Verbindungen[f]->pipe2[0], buff, 32);
        answer = buff;
        if(answer == "Yes") {
          for(Pix d = IPtoT.first(); d; IPtoT.next(d)) {
            if(IPtoT.contents(d) == Verbindungen[f]->pipe2[0]) {
              activeTransitions.add(IPtoT.key(d));
              const Map<String, Edge *> &input_edges = transitions[IPtoT.key(d)]->InputEdges();
              Pix i;
              HLEdge *hledge;
              HLPlace *inp;

              String Markierung = "";

              for(i = input_edges.first();i;input_edges.next(i)) {
                hledge = input_edges.contents(i)->asHLEdge();
                inp = hledge->Src().asHLPlace();
                MultiSET<Val> tmp_vals = inp->Marking();
                for(Pix mar = tmp_vals.first();mar;tmp_vals.next(mar)) {
                  if(Markierung == "")
                    Markierung = tmp_vals.element(mar).asString();
                  else
                    Markierung += "," + tmp_vals.element(mar).asString();
                }
              }
              write(Verbindungen[f]->pipe1[1], "OK", 32);
              read(Verbindungen[f]->pipe2[0], buff2, 256);
              answer = buff2;
              transitions[IPtoT.key(d)]->asHLTransition()->AddMarking(Markierung, 1);
              transitions[IPtoT.key(d)]->asHLTransition()->AddMarking(Markierung, answer);
              IPtoT.del(IPtoT.key(d));
            }
          }
        } else {
          for(Pix d = IPtoT.first(); d; IPtoT.next(d)) {
            if(IPtoT.contents(d) == Verbindungen[f]->pipe2[0]) {
              activeTransitions.del(IPtoT.key(d));
              const Map<String, Edge *> &input_edges = transitions[IPtoT.key(d)]->InputEdges();
              Pix i;
              HLEdge *hledge;
              HLPlace *inp;

              String Markierung = "";

              for(i = input_edges.first();i;input_edges.next(i)) {
                hledge = input_edges.contents(i)->asHLEdge();
                inp = hledge->Src().asHLPlace();
                MultiSET<Val> tmp_vals = inp->Marking();
                for(Pix mar = tmp_vals.first();mar;tmp_vals.next(mar)) {
                  if(Markierung == "")
                    Markierung = tmp_vals.element(mar).asString();
                  else
                    Markierung += "," + tmp_vals.element(mar).asString();
                }
              }
              transitions[IPtoT.key(d)]->asHLTransition()->AddMarking(Markierung, 0);
              transitions[IPtoT.key(d)]->asHLTransition()->AddMarking(Markierung, "");
              IPtoT.del(IPtoT.key(d));
            }
          }
        }
      }
    } else { // count == 0, i.e. no server connection
      VHSet<String> tr;
      for(Pix b = ac.first(); b;ac.next(b)) {
        tr = SearchBinding(*transitions[ac(b)]);
        if((!tr.empty()))
          activeTransitions.add(ac(b));
        else
          activeTransitions.del(ac(b));
      }
    }
  }
  return activeTransitions;
}  // HLNet::ActiveTransitions

void
HLNet::EvaluateAutoBinding(Pix i, SET<String> &binding_strings, int all)
{
  String help;

  for(Pix j = i; j;bindungen.next(j)) {
    if(bindungen(j) == "") {
      binding_strings.add("");
      continue;
    }
    bind.clear();
    if(BelegteString != "")
      ParseAsignment(BelegteString+","+bindungen(j));
    else
      ParseAsignment(bindungen(j));
    if((actual_trans->EvaluateVT(bind) == true) && CheckBinding(*actual_trans)) {
      Pix k = bind.first();
      while(k && BelegteVars.contains(bind.key(k))) {
        bind.next(k); 
      }
      if(k) {
        help = bind.key(k) + "=" + bind.contents(k).asString();
        bind.next(k);
        for(;k;bind.next(k))
          help += "," + bind.key(k) + "=" + bind.contents(k).asString();
        binding_strings.add(help);
      }
    } // if valid binding, add
  } // for j on all possible bindings
  BelegteVars.clear();
}  // HLNet::EvaluateAutoBinding

VHSet<String>
HLNet::SearchBindings(const Transition &t, int all)
  // Find set of possible Bindings
{
  VHSet<String> bs;

  actual_trans = t.asHLTransition();
  CheckPossibleValues();
  EvaluateAutoBinding(bindungen.first(), bs, all);
  return bs;
}  // HLNet::SearchBindings

void
HLNet::PreAnalyse(HLTransition *actual_trans)
  // evaluates ValueTerm before construction of binding
  // just use constant assignments
{
  String id,va;
  String ValueTerm = actual_trans->ValueTerm().Text();
  String Sets;
  VHSet<String> Bindungen2;

  BelegteString = "";
  BelegteVars.clear();

  if(ValueTerm.contains(":")) {
    Sets = ValueTerm.before(":");
    ValueTerm = ValueTerm.after(":");
  }

  // Make list from ands
  Sets = Sets.gsub("&",",");
  Sets = Sets.gsub(" ","");

  while(Sets != "") {
    id = Sets.before("{");
    va = Sets.after("{");
    va = va.before("}");
    if(va.contains("..") | va.contains(",")) {
      Sets = Sets.after(",");
      continue;
    }

    Bindungen2 = bindungen;
    if(!BelegteVars.contains(id)) {
      BelegteVars[id] = StringToVal(va);
      for(Pix i = Bindungen2.first(); i; Bindungen2.next(i)) {
        bindungen.del(Bindungen2(i));
        bindungen.add(Bindungen2(i) + "," + id + "=" + va);
        if(BelegteString != "")
          BelegteString += "," + id + "=" + va;
        else
          BelegteString = id + "=" + va;
      }
      if(bindungen.empty()) {
        bindungen.add(id + "=" + va);
        if(BelegteString != "")
          BelegteString += "," + id + "=" + va;
        else
          BelegteString = id + "=" + va;
      }
    }
    Sets = Sets.after(",");
  }

  ValueTerm = ValueTerm.gsub("&",",");
  ValueTerm = ValueTerm.gsub(" ","");

  if(!ValueTerm.contains("=") | ValueTerm.contains("|"))
    return;

  while(ValueTerm != "") {
    id = ValueTerm.before("=");
    if(id == "")
      break;
    if(id.contains("<") | id.contains(">") | id.contains("!"))
      break;
    while(id.contains("("))
      id = id.after("(");
    while(id.contains(")"))
      id = id.before(")");

    if(id.contains("#")) {
      ValueTerm = ValueTerm.after(",");
      continue;
    }

    va = ValueTerm.after("=");
    while(va.contains(")"))
      va = va.before(")");
    while(va.contains("("))
      va = va.after("(");

    if(va == "")
      break;

    ValueTerm = va.after(",");

    bind.clear();

    if (ValueTerm == "") {
      VT Wahr(va);
      Val wert = Wahr.Evaluate(bind);
      if((wert == true) & !BelegteVars.contains(id)) {
        BelegteVars[id] = wert;
        if(BelegteString != "")
          BelegteString += "," + id + "=" + wert.asString();
        else
          BelegteString = id + "=" + wert.asString();
      }
    }
    else {
      va = va.before(",");
      if(va.contains("<") | va.contains(">"))
        break;
      VT Wahr(va);
      Val wert = Wahr.Evaluate(bind);
      if((wert == true) & !BelegteVars.contains(id)) {
        BelegteVars[id] = wert;
        if(BelegteString != "")
          BelegteString += "," + id + "=" + wert.asString();
        else
          BelegteString = id + "=" + wert.asString();
      }
    }  // else
  }  // while ValueTerm not empty
}  // HLNet::PreAnalyse

void
HLNet::InAnalyse(HLTransition *actual_trans,VHMap<String,VHSet<Val> > &Mengen)
  // evaluates Valueterm after construction of
  // bindings for input-arcs
  // evaluates more complex terms than PreAnalyse
{
  String id,va,H;
  String ValueTerm = actual_trans->ValueTerm().Text();
  String Sets;
  VHSet<String> Bindungen2;

  if(ValueTerm.contains(":")) {
    Sets = ValueTerm.before(":");
    ValueTerm = ValueTerm.after(":");
  }
 
  ValueTerm = ValueTerm.gsub("&",",");
  ValueTerm = ValueTerm.gsub(" ","");

  if(!ValueTerm.contains("=") | ValueTerm.contains("|"))
    return;

  while (ValueTerm != "") {
    id = ValueTerm.before("=");

    if(id.contains(">") | id.contains("<") |
       id.contains("#") | id.contains("!") |
       id.contains("+") | id.contains("-") |
       id.contains("*") | id.contains("/") |
       id.contains("%")) {
      ValueTerm = ValueTerm.after(",");
      continue;
    }
    // Parantheses may be misaligned due to splitting
    while(id.contains("("))
      id = id.after("(");
    while(id.contains(")"))
      id = id.before(")");

    va = ValueTerm.after("=");
    if(va.contains(","))
      va = va.before(",");

    // Be careful with valueterm. Correct nesting has
    // to be ensured. So count the parantheses!
    char* string=va.data();
    int nesting=0;
    while (string++)
    {
      string=strchr(string,')');
      ++nesting;
    }
    string=va.data();
    while (string++)
    {
      string=strchr(string,'(');
      --nesting;
    }
    --nesting;
    // This case may be bad?!
    if (nesting<0)
      return;
    for(;nesting>0;--nesting)
      va = va.before(")");
    ValueTerm = ValueTerm.after(",");
    if(BelegteVars.contains(id)) {
      continue;
    }

    if(va == "")
      break;

    VT vt(va);
    Bindungen2 = bindungen;
    for(Pix u = Bindungen2.first(); u; Bindungen2.next(u)) {
      ParseAsignment(Bindungen2(u));
      Val v = vt.Evaluate(bind);
      if(v != false) {
        VHSet<Val> tmp_vals=Mengen[id];
        for(Pix m = tmp_vals.first(); m; tmp_vals.next(m)) {
          if (tmp_vals(m)==v) {
        if(!BelegteVars.contains(id))
          BelegteVars[id] = v;
        H = Bindungen2(u);
        bindungen.del(H);
        bindungen.add(H + "," + id + "=" + v.asString());
          }
        }
      }
    }
  }  // while ValueTerm not empty

  ValueTerm = actual_trans->ValueTerm().Text();

  if(ValueTerm.contains(":")) {
    Sets = ValueTerm.before(":");
    ValueTerm = ValueTerm.after(":");
  }
 
  ValueTerm = ValueTerm.gsub("&",",");
  ValueTerm = ValueTerm.gsub(" ","");

  while (ValueTerm != "") {
    id = ValueTerm.after("=");
    if(id == "")
      break;

    if(id.contains(","))
      id = id.before(",");

    while(id.contains(")"))
      id = id.before(")");
    while(id.contains("("))
      id = id.after("(");

    if(id.contains("#")) {
      break;
    }
    if(id.contains(">") | id.contains("<") |
       id.contains("+") | id.contains("-") |
       id.contains("*") | id.contains("/") |
       id.contains("%")) {
      ValueTerm = ValueTerm.after(",");
      continue;
    }
    va = ValueTerm.before("=");

    if(va.contains(","))
      va = va.after(",");
    while(va.contains("("))
      va = va.after("(");
    while(va.contains(")"))
      va = va.before(")");
    if(va.contains("!"))
      break;

    ValueTerm = ValueTerm.after(",");

    if(BelegteVars.contains(id)) {
      continue;
    }
    if(id == va)
      continue;
    if(va.contains("<") | va.contains(">"))
      break;

    if(va == "")
      break;

    VT vt(va);
    Bindungen2 = bindungen;
    for(Pix u = Bindungen2.first(); u; Bindungen2.next(u)) {
      ParseAsignment(Bindungen2(u));
      Val v = vt.Evaluate(bind);
      if(v != false) {
        if(!BelegteVars.contains(id))
          BelegteVars[id] = v;
        H = Bindungen2(u);
        bindungen.del(H);
        bindungen.add(H + "," + id + "=" + v.asString());
      }
    }
  }  // while ValueTerm not empty
} // HLNet::InAnalyse

VHSet<String>
HLNet::SearchBinding(const Transition &tra)
  // is there a Binding, where this Transition can fire?
{
  VHSet<String> bs;

  actual_trans = tra.asHLTransition();

  VHMap<String, VHSet<Val> > Mengen(VHSet<Val>(4),10);
  VHSet<Val> Temp;
  const Map<String, Edge *> &input_edges = actual_trans->InputEdges();
  const Map<String, Edge *> &output_edges = actual_trans->OutputEdges();
  Pix i;
  String H, T, bindung, t;
  int position;
  VHSet<String> Bindungen3, Bindungen2;
  Val wert;
  MultiSET<String> variablen;

  String ValueTerm = actual_trans->ValueTerm().Text();
  String id,va;
  String Sets;
  String Markierung = "";

  HLEdge *hledge;
  HLPlace *inp;
  if(cache) {
    for(i = input_edges.first();i;input_edges.next(i)) {
      hledge = input_edges.contents(i)->asHLEdge();
      inp = hledge->Src().asHLPlace();
      MultiSET<Val> tmp_vals = inp->Marking();
      for(Pix mar = tmp_vals.first();mar;tmp_vals.next(mar)) {
        if(Markierung == "")
          Markierung = tmp_vals.element(mar).asString();
        else
          Markierung += "," + tmp_vals.element(mar).asString();
      }
    }
    if(actual_trans->InCache(Markierung)) {
      if(actual_trans->ReturnCache1(Markierung))
        bs.add(actual_trans->ReturnCache2(Markierung));
      else
        bs.clear();
      return bs;
    }
  }

  if(preAnalyse)
    PreAnalyse(actual_trans);

  bindungen.clear();
  if(BelegteString != "")
    bindungen.add(BelegteString);

// Create list of variables with possible values (marking)
// both for input and output variables
  for(i = input_edges.first();i;input_edges.next(i)) {
    hledge = input_edges.contents(i)->asHLEdge();
    inp = hledge->Src().asHLPlace();
    MultiSET<Val> vars_in_edge = hledge->Vars();
    MultiSET<Val> tmp_vals = inp->Marking();
    for(Pix k = vars_in_edge.first(); k; vars_in_edge.next(k)) {
      variablen = vars_in_edge.element(k).Strings();
      if(!variablen.empty()) {
        for(Pix v = variablen.first();v;variablen.next(v)) {
          if(BelegteVars.contains(variablen.element(v)))
            continue;
          for(Pix u = tmp_vals.first();u;tmp_vals.next(u)) {
            position = vars_in_edge.element(k).find(variablen.element(v));
            wert = tmp_vals.element(u).get(position).asString();
            Temp.add(wert);
          }
          if(!Mengen.contains(variablen.element(v))) {
            Mengen[variablen.element(v)] = Temp;
          } else {
            VHSet<Val> neu = Mengen[variablen.element(v)];
            neu &= Temp;
            Mengen[variablen.element(v)] = neu;
          }
          Temp.clear();
        }
      }
    }
  }
  for(i = output_edges.first();i;output_edges.next(i)) {
    hledge = output_edges.contents(i)->asHLEdge();
    inp = hledge->Dest().asHLPlace();
    MultiSET<Val> vars_in_edge = hledge->Vars();
    VHSet<Val> tmp_vals = inp->Type();
    for(Pix k = vars_in_edge.first(); k; vars_in_edge.next(k)) {
      variablen = vars_in_edge.element(k).Strings();
      if(!variablen.empty()) {
        for(Pix v = variablen.first();v;variablen.next(v)) {
          if(BelegteVars.contains(variablen.element(v)))
            continue;
          Temp.clear();
          for(Pix u = tmp_vals.first();u;tmp_vals.next(u)) {
            position = vars_in_edge.element(k).find(variablen.element(v));
            wert = tmp_vals(u).get(position).asString();
            Temp.add(wert);
          }
          if(!Mengen.contains(variablen.element(v))) {
            Mengen[variablen.element(v)] = Temp;
            Temp.clear();
          } else {
            VHSet<Val> neu = Mengen[variablen.element(v)];
            neu &= Temp;
            Mengen[variablen.element(v)] = neu;
          }
        }
      }
    }
  }
  // if one variable has no value, there is no binding, return
  for(Pix b = Mengen.first();b;Mengen.next(b)) {
    if(Mengen.contents(b).length() == 0) {
      bs.clear();
      bindungen.clear();
      return bs;
    }
    /* cerr << "Var:" << Mengen.key(b) << "-";
    for(Pix s = Mengen.contents(b).first();s;Mengen.contents(b).next(s)) {
      cerr << " " << Mengen.contents(b)(s).asString();
    }
    cerr << endl; */
  }

    // for all input arcs create possible bindings
  for (i = input_edges.first(); i; input_edges.next(i)) {
    HLEdge *e = input_edges.contents(i)->asHLEdge(); 
    HLPlace *p = e->Src().asHLPlace();
      // get tokens from the current input place
    const MultiSET<Val> &marking = p->Marking();
    MultiSET<Val> vars_in_edge = e->Vars();
    for (Pix k = vars_in_edge.first(); k; vars_in_edge.next(k)) {
        // Die mglichen Werte bedingt durch die Werte der auf der Stelle
        // liegenden Tokens.
        // Zwischenspeicher fr Werte
      VHSet<Val> tmp_vals(20);
      int gefunden = 0;
      Bindungen3 = bindungen;
      for (Pix n = marking.first(); n; marking.next(n)) {
        if(marking.element(n).EQUAL(vars_in_edge.element(k))) {
         // nur wenn die Markierung = der Kantenannotation
          gefunden = 1;
          // Bindungen3 = bindungen;
          if(!Bindungen3.empty()) {
            for(Pix nb = Bindungen3.first(); nb; Bindungen3.next(nb)) {
              t = "";
              bindung = Bindungen3(nb);
              bindungen.del(bindung);
              variablen = vars_in_edge.element(k).Strings();
              for(Pix v = variablen.first();v;variablen.next(v)) {
                if(!BelegteVars.contains(variablen.element(v))) {
                  position = vars_in_edge.element(k).find(variablen.element(v));
                  wert = marking.element(n).get(position);
                  if(t != "")
                    t += ",";
                  t += variablen.element(v) + "=" + wert.asString();
                }
              } // for v ueber alle Variablen in der Kantenannotation
              MultiSET<Val> t_vars2 = vars_in_edge;
              MultiSET<Val> t_marking2 = marking;
              t_vars2.del(vars_in_edge.element(k));
              t_marking2.del(marking.element(n));
              if(t != "" && bindung != "")
                possible(t_vars2,t_marking2,bindung+","+t);
              else if(t != "")
                possible(t_vars2,t_marking2,t);
              else
                possible(t_vars2,t_marking2,bindung);
            } // for nb ueber alle m"oglichen Bindungen
          } // for n over all markings of the input place
          else {
            t = "";
            variablen = vars_in_edge.element(k).Strings();
            for(Pix v = variablen.first();v;variablen.next(v)) {
              if(!BelegteVars.contains(variablen.element(v))) {
                position = vars_in_edge.element(k).find(variablen.element(v));
                wert = marking.element(n).get(position);
                if(/*v != variablen.first()*/ t != "")
                  t += ",";
                t += variablen.element(v) + "=" + wert.asString();
              }
            } // for v ueber alle m"oglichen Variablen in der Kantenannotation
            MultiSET<Val> t_vars2 = vars_in_edge;
            MultiSET<Val> t_marking2 = marking;
            t_vars2.del(vars_in_edge.element(k));
            t_marking2.del(marking.element(n));
            if(t != "" && bindung != "")
                possible(t_vars2,t_marking2,bindung+","+t);
              else if(t != "")
                possible(t_vars2,t_marking2,t);
              else
                possible(t_vars2,t_marking2,bindung);
          }  // wenn Bindungen noch nicht belegt war 
        }
      }  // for Pix n
      if(!gefunden) {
        bs.clear();
        bindungen.clear();
        return bs;
      } else {
        if(bindungen.empty())
          bindungen.add("");
      }
      // gefunden = 0;
    }  // for Pix
  }  // for i ueber alle Eingangskanten

  String s = "";
  if (bindungen.first())
    s = bindungen(bindungen.first());
  MultiSET<String> vari;
  s = s.gsub(" ", "");
  while (s != "") {
    id = s.before("=");
    s = s.after(",");
    vari.add(id);
  }

  if(inAnalyse) {
    int before, after;
    do {
      before = BelegteVars.length();
      InAnalyse(actual_trans,Mengen);
      after = BelegteVars.length();
    } while(before != after);
  }

  for(Pix u = bindungen.first();u;bindungen.next(u)) {
    String help = bindungen(u);
    if(help[0] == ',') {
      bindungen.del(help);
      help = help.after(",");
      bindungen.add(help);
      u = bindungen.first();
    }
  }

  if (!input_edges.first())
    bindungen.add("");
    // for all output arcs
  Bindungen2 = bindungen;
  for (i = output_edges.first(); i; output_edges.next(i)) {
  //  HLEdge *e = output_edges.contents(i)->asHLEdge(); 
  //  HLPlace *p = e->Dest().asHLPlace();
    MultiSET<Val> vars_in_edge = output_edges.contents(i)->asHLEdge()->Vars();
    VHSet<Val> tmp_vals;
    for (Pix k = vars_in_edge.first(); k; vars_in_edge.next(k)) {
      variablen = vars_in_edge.element(k).Strings();
      VHSet<Val> belegungen;
      for(Pix v = variablen.first();v;variablen.next(v)) {
        if(BelegteVars.contains(variablen.element(v)) | vari.contains(variablen.element(v)))
          continue;

        tmp_vals = Mengen[variablen.element(v)];
        Bindungen2 = bindungen;
        for(Pix h = bindungen.first(); h;bindungen.next(h)) {
          T = bindungen(h);
          bindungen.del(T);
          Bindungen2.del(T);
          for(Pix m = tmp_vals.first(); m; tmp_vals.next(m)) {
            H = T;
            if(H != "")
              H += ",";
            H += variablen.element(v) + "=" + tmp_vals(m).asString();
            vari.add(variablen.element(v));
            bind.clear();
            ParseAsignment(H);
            if((actual_trans->EvaluateVT(bind) == true) && CheckBinding(*actual_trans)) {
              bs.add(H);
              bindungen.add(H);
              actual_trans->AddMarking(Markierung, 1);
              actual_trans->AddMarking(Markierung, H);
              return bs;
            }
            Bindungen2.add(H);
          } // for m over all possible values of a variable
        } 
        bindungen = Bindungen2;
        belegungen.clear();
      } // for v over all variables of an arc annotation
    }  // for k over all arc annotations
  }  // for i over all output arcs

  BelegteVars.clear();
  BelegteString = "";

  for(Pix o = bindungen.first();o;bindungen.next(o)) {
    bind.clear();
    ParseAsignment(bindungen(o));
    if((actual_trans->EvaluateVT(bind) == true) && CheckBinding(*actual_trans)) {
      bs.add(bindungen(o));
      actual_trans->AddMarking(Markierung, 1);
      actual_trans->AddMarking(Markierung, bindungen(o));
      return bs;
    }
  }

  if(cache) {
    actual_trans->AddMarking(Markierung, 0);
    actual_trans->AddMarking(Markierung, "");
  }

  return bs;
}  // HLNet::SearchBinding

void
HLNet::CheckPossibleValues()
{
  VHMap<String, VHSet<Val> > Mengen(VHSet<Val>(4),10);
  VHSet<Val> Temp;
  const Map<String, Edge *> &input_edges = actual_trans->InputEdges();
  const Map<String, Edge *> &output_edges = actual_trans->OutputEdges();
  Pix i;
  String H, T, bindung, t;
  int position;
  VHSet<String> Bindungen3, Bindungen2;
  Val wert;
  MultiSET<String> variablen;
  String ValueTerm = actual_trans->ValueTerm().Text();
  String id,va;

  if(preAnalyse)
    PreAnalyse(actual_trans);

  bindungen.clear();  
  if(BelegteString != "") {
    bindungen.add(BelegteString);
  }

  HLEdge *hledge;
  HLPlace *inp;
  for(i = input_edges.first();i;input_edges.next(i)) {
    hledge = input_edges.contents(i)->asHLEdge();
    inp = hledge->Src().asHLPlace();
    MultiSET<Val> vars_in_edge = hledge->Vars();
    MultiSET<Val> tmp_vals = inp->Marking();
    for(Pix k = vars_in_edge.first(); k; vars_in_edge.next(k)) {
      variablen = vars_in_edge.element(k).Strings();
      if(!variablen.empty()) {
        for(Pix v = variablen.first();v;variablen.next(v)) {
          if(BelegteVars.contains(variablen.element(v)))
            continue;
          for(Pix u = tmp_vals.first();u;tmp_vals.next(u)) {
            position = vars_in_edge.element(k).find(variablen.element(v));
            wert = tmp_vals.element(u).get(position).asString();
            Temp.add(wert);
          }
          if(!Mengen.contains(variablen.element(v))) {
            Mengen[variablen.element(v)] = Temp;
          } else {
            VHSet<Val> neu = Mengen[variablen.element(v)];
            neu &= Temp;
            Mengen[variablen.element(v)] = neu;
          }
          Temp.clear();
        }
      }
    }
  }
  for(i = output_edges.first();i;output_edges.next(i)) {
    hledge = output_edges.contents(i)->asHLEdge();
    inp = hledge->Dest().asHLPlace();
    MultiSET<Val> vars_in_edge = hledge->Vars();
    VHSet<Val> tmp_vals = inp->Type();
    for(Pix k = vars_in_edge.first(); k; vars_in_edge.next(k)) {
      variablen = vars_in_edge.element(k).Strings();
      if(!variablen.empty()) {
        for(Pix v = variablen.first();v;variablen.next(v)) {
          if(BelegteVars.contains(variablen.element(v)))
            continue;
          for(Pix u = tmp_vals.first();u;tmp_vals.next(u)) {
            position = vars_in_edge.element(k).find(variablen.element(v));
            wert = tmp_vals(u).get(position).asString();
            Temp.add(wert);
          }
          if(!Mengen.contains(variablen.element(v))) {
            Mengen[variablen.element(v)] = Temp;
            Temp.clear();
          } else {
            VHSet<Val> neu = Mengen[variablen.element(v)];
            neu &= Temp;
            Mengen[variablen.element(v)] = neu;
          }
        }
      }
    }
  }
 /* for(Pix b = Mengen.first();b;Mengen.next(b)) {
    cerr << "Var:" << Mengen.key(b) << "-";
    for(Pix s = Mengen.contents(b).first();s;Mengen.contents(b).next(s)) {
      cerr << " " << Mengen.contents(b)(s);
    }
    cerr << endl;
  }*/

    // Alle Eingangskanten durchlaufen
  for (i = input_edges.first(); i; input_edges.next(i)) {
    HLEdge *e = input_edges.contents(i)->asHLEdge(); 
    HLPlace *p = e->Src().asHLPlace();
      // Tokens holen, die auf aktueller Eingangstelle sind
    const MultiSET<Val> &marking = p->Marking();
    MultiSET<Val> vars_in_edge = e->Vars();
    for (Pix k = vars_in_edge.first(); k; vars_in_edge.next(k)) {
        // Die mglichen Werte bedingt durch die Werte der auf der Stelle
        // liegenden Tokens.
        // Zwischenspeicher fr Werte
      VHSet<Val> tmp_vals(20);
      int gefunden = 0;
      Bindungen3 = bindungen;
      for (Pix n = marking.first(); n; marking.next(n)) {
        if(marking.element(n).EQUAL(vars_in_edge.element(k))) {
         // nur wenn die Markierung = der Kantenannotation
          gefunden = 1;
          // Bindungen3 = bindungen;
          if(!Bindungen3.empty()) {
            //t = "";
            for(Pix nb = Bindungen3.first(); nb; Bindungen3.next(nb)) {
              t = "";
              bindung = Bindungen3(nb);
              bindungen.del(bindung);
              variablen = vars_in_edge.element(k).Strings();
              for(Pix v = variablen.first();v;variablen.next(v)) {
                if(!BelegteVars.contains(variablen.element(v))) {
                  position = vars_in_edge.element(k).find(variablen.element(v));
                  wert = marking.element(n).get(position);
                  if(t != "")
                    t += ",";
                  t += variablen.element(v) + "=" + wert.asString();
                }
              } // for v ueber alle Variablen in der Kantenannotation
              MultiSET<Val> t_vars2 = vars_in_edge;
              MultiSET<Val> t_marking2 = marking;
              t_vars2.del(vars_in_edge.element(k));
              t_marking2.del(marking.element(n));
              if(t != "" && bindung != "")
                possible(t_vars2,t_marking2,bindung+","+t);
              else if(t != "")
                possible(t_vars2,t_marking2,t);
              else
                possible(t_vars2,t_marking2,bindung);
            } // for nb ueber alle m"oglichen Bindungen
          } // for n ueber alle Markierungen der Vorbereichsstelle
          else {
            t = "";
            variablen = vars_in_edge.element(k).Strings();
            for(Pix v = variablen.first();v;variablen.next(v)) {
              if(!BelegteVars.contains(variablen.element(v))) {
                position = vars_in_edge.element(k).find(variablen.element(v));
                wert = marking.element(n).get(position);
                if(/*v != variablen.first()*/t != "")
                  t += ",";
                t += variablen.element(v) + "=" + wert.asString();
              }
            } // for v ueber alle m"oglichen Variablen in der Kantenannotation
            MultiSET<Val> t_vars2 = vars_in_edge;
            MultiSET<Val> t_marking2 = marking;
            t_vars2.del(vars_in_edge.element(k));
            t_marking2.del(marking.element(n));
            possible(t_vars2,t_marking2,t);
          }  // wenn bindungen noch nicht belegt war 
        }
      }  // for Pix n
      if(!gefunden) {
        bindungen.clear();
        return;
      } else {
        if(bindungen.empty())
          bindungen.add("");
      }
    }  // for Pix
  }  // for i ueber alle Eingangskanten
  for(Pix u = bindungen.first();u;bindungen.next(u)) {
    String help = bindungen(u);
    if(help[0] == ',') {
      bindungen.del(help);
      help = help.after(",");
      bindungen.add(help);
      u = bindungen.first();
    }
  }

  String s = "";
  if (bindungen.first())
    s = bindungen(bindungen.first());
  MultiSET<String> vari;
  s = s.gsub(" ", "");
  while (s != "") {
    id = s.before("=");
    s = s.after(",");
    vari.add(id);
  }

  if(inAnalyse) {
    int zahl1 = 0;
    int zahl2 = BelegteVars.length();
    while(zahl1 != zahl2) {
      zahl1 = BelegteVars.length();
      InAnalyse(actual_trans,Mengen);
      zahl2 = BelegteVars.length();
    }
  }

  if (!input_edges.first())
    bindungen.add("");
    // Alle Ausgangskanten durchlaufen
  Bindungen2 = bindungen;
  for (i = output_edges.first(); i; output_edges.next(i)) {
    //  HLEdge *e = output_edges.contents(i)->asHLEdge(); 
    //  HLPlace *p = e->Dest().asHLPlace();
    MultiSET<Val> vars_in_edge = output_edges.contents(i)->asHLEdge()->Vars();
    VHSet<Val> tmp_vals;
    for (Pix k = vars_in_edge.first(); k; vars_in_edge.next(k)) {
      variablen = vars_in_edge.element(k).Strings();
      VHSet<Val> belegungen;
      for(Pix v = variablen.first();v;variablen.next(v)) {
        if(BelegteVars.contains(variablen.element(v)) || vari.contains(variablen.element(v))) {
          continue;
        }
        position = vars_in_edge.element(k).find(variablen.element(v));
        tmp_vals = Mengen[variablen.element(v)];
        for(Pix l = tmp_vals.first(); l;tmp_vals.next(l)) {
          wert = tmp_vals(l).get(position);
          if(!belegungen.contains(wert)) {
            belegungen.add(wert);
          }
        } // for l ueber alle m"oglichen Belegungen der Nachbereichsstelle
        Bindungen2 = bindungen;
        for(Pix h = bindungen.first(); h;bindungen.next(h)) {
          T = bindungen(h);
          bindungen.del(T);
          Bindungen2.del(T);
          for(Pix m = belegungen.first(); m;belegungen.next(m)) {
            H = T;
            if(H != "")
              H += ",";
            H += variablen.element(v) + "=" + belegungen(m).asString();
            vari.add(variablen.element(v));
            Bindungen2.add(H);
          } // for m ueber alle m"oglichen Belegungen einer Variable
        } 
        bindungen = Bindungen2;
        belegungen.clear();
      } // for v ueber alle Variablen in einer Kantenannotation
    }  // for k ueber alle Kantenannotationen
  }  // for i ueber alle Ausgangskanten
  BelegteVars.clear();
}  // HLNet::CheckPossibleValues


void
HLNet::possible(MultiSET<Val> &t_vars, MultiSET<Val> &t_marking, String bindung)
{
  String t;
  MultiSET<String> variablen;
  Val wert;
  int position;
  if(t_vars.length() == 0) {
    bind.clear();
    ParseAsignment(bindung);
    Pix i = bind.first();
    if(i) {
      String help = bind.key(i) + "=" + bind.contents(i).asString();
      bind.next(i);
      for(;i;bind.next(i)) {
        help += "," + bind.key(i) + "=" + bind.contents(i).asString();
      }
      bindungen.add(help);
    }
  }
  else {
    for(Pix i = t_vars.first(); i;t_vars.next(i)) {
      VHSet<Val> tmp_vals(20);
      int gefunden = 0;
      for(Pix j = t_marking.first(); j; t_marking.next(j)) {
        if(t_vars.element(i).EQUAL(t_marking.element(j))) {
          gefunden = 1;
          variablen = t_vars.element(i).Strings();
          t = "";
          for(Pix v = variablen.first();v;variablen.next(v)) {
            if(BelegteVars.contains(variablen.element(v)))
              continue;
            position = t_vars.element(i).find(variablen.element(v));
            wert = t_marking.element(j).get(position);
            if(t != "")
              t += ",";
            t += variablen.element(v) + "=" + wert.asString();
          }
          MultiSET<Val> t_vars2 = t_vars;
          MultiSET<Val> t_marking2 = t_marking;
          t_vars2.del(t_vars.element(i));
          t_marking2.del(t_marking.element(j));
          if(t != "" && bindung != "")
            possible(t_vars2,t_marking2,bindung+","+t);
          else if(t != "")
            possible(t_vars2,t_marking2,t);
          else
            possible(t_vars2,t_marking2,bindung);
        }
      }
      if(!gefunden) {
        return;
      }
    }
  }     
} // HLNet::possible


String
HLNet::RandomBinding(const Transition &t)
{
  VHSet<String> binding_strings;
  Pix binding;

  binding_strings = SearchBindings(t, alle);

  if (binding_strings.length() == 0) { // no binding found
    return "no binding";
  } 
  binding = binding_strings.first();
  {
    int k = 0;
    int rnd_num = rand() % binding_strings.length();
    while(binding && (k < rnd_num)) {
      binding_strings.next(binding);
      k++;
    }
  }
  return binding_strings(binding);
}  // HLNet::RandomBinding

void
HLNet::RandomFire(const String &t_name)
{
  String t = RandomBinding(*transitions[t_name]);
  if (t != "no binding")
    FireTransition(t_name, t);
}  // HLNet::RandomFire

bool
HLNet::CheckBinding(const HLTransition &t)
  // Check, if binding matches the Marking of the input places.
  // This is needed since possibly too many (colored) tokens
  // are used in the binding. This problem is raised in parts by
  // arcs annotated with real multi sets.
  // NOTE: before this you have to call 
  // EvaluateVT AND ParseAsignment !!
{
  Pix i, j, m;

    // Markierung auf den Eingangsstellen zusammen mit den
    // Variablen der Eingangskanten kopieren, um das Schalten
    // zu simulieren.


  const Map<String, Edge *> &in_edges = t.InputEdges();
  const Map<String, Edge *> &out_edges = t.OutputEdges();
  MultiSET<Val> cmark;
  for  (i = in_edges.first(); i; in_edges.next(i)) {
    MultiSET<Val> smark;
    const MultiSET<Val> &vars = in_edges.contents(i)->asHLEdge()->Vars();
    for (j = vars.first(); j; vars.next(j)) {
      if(vars.element(j).hasStrings()) {
        MultiSET<String> variablen = vars.element(j).Strings();
        Val wert = vars.element(j);
        for(Pix b = variablen.first(); b;variablen.next(b)) {
          int position = vars.element(j).find(variablen.element(b));
          Val w = bind[variablen.element(b)];
          wert.set(w,position);
        } // for b ueber alle Variablen in einer Kantenannotation
        smark.add(wert, vars.nof(j));
      } 
      else {
        smark.add(vars.element(j), vars.nof(j));
      }
    } // for j ueber alle Kantenannotationen
    cmark = places[in_edges.key(i)]->asHLPlace()->Marking();
    for (m = smark.first(); m; smark.next(m)) {
      if(cmark.nof(smark.element(m)) >= smark.nof(m))
        cmark.del(smark.element(m), smark.nof(m));
      else
        return false;
    }
  } // for i ueber alle Eingangskanten

  for (i = out_edges.first();i ;out_edges.next(i)) {
    HLEdge *e = out_edges.contents(i)->asHLEdge(); 
    HLPlace *p = e->Dest().asHLPlace();
    MultiSET<Val> vars_in_edge = out_edges.contents(i)->asHLEdge()->Vars();
    VHSet<Val> tmp_vals = p->Type();
    for (Pix k = vars_in_edge.first(); k; vars_in_edge.next(k)) {
      MultiSET<String> variablen = vars_in_edge.element(k).Strings();
      for (Pix g = variablen.first();g ;variablen.next(g)) {
        if(!bind.contains(variablen.element(g)))
          return false;
      }
    }
  }
  return true;
}  // HLNet::CheckBinding

void
HLNet::FireTransition(const String &t_name, const String &bind_string)
  // Eine Transition schalten.
{
  Pix i, j;
  HLTransition *t = transitions[t_name]->asHLTransition();
  VHSet<String> active;
  VHSet<String> never;
  VHSet<String> ever;
  MultiSET<Val> ma;
 
  timeval start;

  if(timeout)
    gettimeofday(&start, NULL);

    // Zuweisungs-String parsen und in Bindung bind eintragen
  ParseAsignment(bind_string);
    // Jetzt testen, ob Transition unter der Bindung bind schalten kann
  if ((t->EvaluateVT(bind)!=true) || !CheckBinding(*t)) {
    Error("The transition can not fire under this binding");
  }
  else {  // Ja -> dann schalten
    const Map<String, Edge *> &in_edges = t->InputEdges();
    for  (i = in_edges.first(); i; in_edges.next(i)) {
      MultiSET<Val> smark;
      const MultiSET<Val> &vars = in_edges.contents(i)->asHLEdge()->Vars();
      for (j = vars.first(); j; vars.next(j)) {
        if(vars.element(j).hasStrings()) {
          MultiSET<String> variablen = vars.element(j).Strings();
          Val wert = vars.element(j);
          for(Pix b = variablen.first(); b;variablen.next(b)) {
            /*int position = vars.element(j).find(variablen.element(b));
            Val w = bind[variablen.element(b)];
            wert.set(w,position);*/
            wert.replaceAll(variablen.element(b), bind[variablen.element(b)]);
          } // for b ueber alle Variablen in einer Kantenannotation
          smark.add(wert, vars.nof(j));
        } 
        else {
          smark.add(vars.element(j), vars.nof(j));
        }
      } // for j ueber alle Kantenannotationen
      places[in_edges.key(i)]->asHLPlace()->SubtractMarking(smark);
    }  // for i ueber alle Eingangskanten
    const Map<String, Edge *> &out_edges = t->OutputEdges();
    for  (i = out_edges.first(); i; out_edges.next(i)) {
      MultiSET<Val> smark;
      const MultiSET<Val> &vars = out_edges.contents(i)->asHLEdge()->Vars();
      Val wert;
      for (j = vars.first(); j; vars.next(j)) {
        if(vars.element(j).hasStrings()) {
          MultiSET<String> variablen = vars.element(j).Strings();
          wert = vars.element(j);
          for(Pix b = variablen.first(); b;variablen.next(b)) {
            /*int position = vars.element(j).find(variablen.element(b));
            Val w = bind[variablen.element(b)];
            wert.set(w,position);*/
            wert.replaceAll(variablen.element(b), bind[variablen.element(b)]);
          } // for b ueber alle Variablen in einer Kantenannotation
          smark.add(wert,vars.nof(j));
        }
        else {
          smark.add(vars.element(j), vars.nof(j));
        }
      } // for j ueber alle Kantenannotationen
      places[out_edges.key(i)]->asHLPlace()->AddMarking(smark);
    }  // for i ueber die Ausgangskanten
    //active.add(t_name);
    VHSet<String> in = t->InputPlaces();
    VHSet<String> out = t->OutputPlaces();
    for (i = in.first(); i; in.next(i)) {
      const Map<String, Edge *> &OutEdges = places[in(i)]->OutputEdges();
      for (j = OutEdges.first(); j; OutEdges.next(j)) {
        Edge *e = OutEdges.contents(j);
        active.add(e->Dest().asTransition()->Name());
      }  // for j ueber alle Ausgangskanten
    }  // for i ueber alle Eingangsstellen
    for (i = out.first(); i; out.next(i)) {
      const Map<String, Edge *> &OutEdges = places[out(i)]->OutputEdges();
      for (j = OutEdges.first(); j; OutEdges.next(j)) {
        Edge *e = OutEdges.contents(j);
        active.add(e->Dest().asTransition()->Name());
      }  // for j ueber alle Ausgangskanten
    }  // for i ueber alle Ausgangsstellen

    // Die Schaltungsserver f"uhren ebenfalls das Feuern der
    // Transition aus.
    String t1 = t_name;
    String b1 = bind_string;
    char buff[32];
    char buff2[256];
    for(int n = 0; n < count; n++) {
      write(Verbindungen[n]->pipe1[1], "Fire", 32);
      read(Verbindungen[n]->pipe2[0], buff, 32);
      write(Verbindungen[n]->pipe1[1], t1.data(), 32);
      read(Verbindungen[n]->pipe2[0], buff, 32);
      write(Verbindungen[n]->pipe1[1], b1.data(), 256);
    }

    // Hier wird nach dem Schalten berechnet, welche Transitionen schalten
    // k"onnen. Nur die Transitionen, die ver"anderte Eingangsstellen
    // besitzen, werden betrachtet.
    VHSet<String> tr;
    String answer;
    VHMap<String, int> IPtoT(0, 2);

    VHSet<String> ac;

    for(Pix g = active.first(); g; active.next(g)) {
      if(transitions[active(g)]->asHLTransition()->CanFire())
        ac.add(active(g));
      else
        activeTransitions.del(active(g));
    }

    if(count != 0) {
      int akt = 0;
      Pix b = ac.first();
      while(akt < count && b) {
        write(Verbindungen[akt]->pipe1[1], "IsActive", 32);
        read(Verbindungen[akt]->pipe2[0], buff, 32);
        write(Verbindungen[akt]->pipe1[1], ac(b).data(), 32);
        IPtoT[ac(b)] = Verbindungen[akt]->pipe2[0];
        akt++;
        ac.next(b);
      }
      boolean Ende = true;
      int last = akt;
      while(b) {
        for(int f = 0;f < akt;f++) {
          read(Verbindungen[f]->pipe2[0], buff, 32);
          answer = buff;
          if(answer == "Yes") {
            for(Pix d = IPtoT.first(); d; IPtoT.next(d)) {
              if(IPtoT.contents(d) == Verbindungen[f]->pipe2[0]) {
                activeTransitions.add(IPtoT.key(d));
                const Map<String, Edge *> &input_edges = transitions[IPtoT.key(d)]->InputEdges();
                Pix i;
                HLEdge *hledge;
                HLPlace *inp;

                String Markierung = "";

                for(i = input_edges.first();i;input_edges.next(i)) {
                  hledge = input_edges.contents(i)->asHLEdge();
                  inp = hledge->Src().asHLPlace();
                  MultiSET<Val> tmp_vals = inp->Marking();
                  for(Pix mar = tmp_vals.first();mar;tmp_vals.next(mar)) {
                    if(Markierung == "")
                      Markierung = tmp_vals.element(mar).asString();
                    else
                      Markierung += "," + tmp_vals.element(mar).asString();
                  }
                }
                write(Verbindungen[f]->pipe1[1], "OK", 32);
                read(Verbindungen[f]->pipe2[0], buff2, 256);
                answer = buff2;
                transitions[IPtoT.key(d)]->asHLTransition()->AddMarking(Markierung, 1);
                transitions[IPtoT.key(d)]->asHLTransition()->AddMarking(Markierung, answer);
                IPtoT.del(IPtoT.key(d));
              }
            }
          } else {
            for(Pix d = IPtoT.first(); d; IPtoT.next(d)) {
              if(IPtoT.contents(d) == Verbindungen[f]->pipe2[0]) {
                activeTransitions.del(IPtoT.key(d));
                const Map<String, Edge *> &input_edges = transitions[IPtoT.key(d)]->InputEdges();
                Pix i;
                HLEdge *hledge;
                HLPlace *inp;

                String Markierung = "";

                for(i = input_edges.first();i;input_edges.next(i)) {
                  hledge = input_edges.contents(i)->asHLEdge();
                  inp = hledge->Src().asHLPlace();
                  MultiSET<Val> tmp_vals = inp->Marking();
                  for(Pix mar = tmp_vals.first();mar;tmp_vals.next(mar)) {
                    if(Markierung == "")
                      Markierung = tmp_vals.element(mar).asString();
                    else
                      Markierung += "," + tmp_vals.element(mar).asString();
                  }
                }
                transitions[IPtoT.key(d)]->asHLTransition()->AddMarking(Markierung, 0);
                transitions[IPtoT.key(d)]->asHLTransition()->AddMarking(Markierung, "");
                IPtoT.del(IPtoT.key(d));
              }
            }
          }
          boolean neu = false;
          while(!neu) {
            if(b) {
              write(Verbindungen[f]->pipe1[1], "IsActive", 32);
              read(Verbindungen[f]->pipe2[0], buff, 32);
              write(Verbindungen[f]->pipe1[1], ac(b).data(), 32);
              IPtoT[ac(b)] = Verbindungen[f]->pipe2[0];
              neu = true;
              ac.next(b);
            } else if(Ende) {
              last = f;
              Ende = false;
              neu = true;
            }
          }
        }
      }

      for(int f = 0;f < last;f++) {
        read(Verbindungen[f]->pipe2[0], buff, 32);
        answer = buff;
        if(answer == "Yes") {
          for(Pix d = IPtoT.first(); d; IPtoT.next(d)) {
            if(IPtoT.contents(d) == Verbindungen[f]->pipe2[0]) {
              activeTransitions.add(IPtoT.key(d));
              const Map<String, Edge *> &input_edges = transitions[IPtoT.key(d)]->InputEdges();
              Pix i;
              HLEdge *hledge;
              HLPlace *inp;

              String Markierung = "";

              for(i = input_edges.first();i;input_edges.next(i)) {
                hledge = input_edges.contents(i)->asHLEdge();
                inp = hledge->Src().asHLPlace();
                MultiSET<Val> tmp_vals = inp->Marking();
                for(Pix mar = tmp_vals.first();mar;tmp_vals.next(mar)) {
                  if(Markierung == "")
                    Markierung = tmp_vals.element(mar).asString();
                  else
                    Markierung += "," + tmp_vals.element(mar).asString();
                }
              }
              write(Verbindungen[f]->pipe1[1], "OK", 32);
              read(Verbindungen[f]->pipe2[0], buff2, 256);
              answer = buff2;
              transitions[IPtoT.key(d)]->asHLTransition()->AddMarking(Markierung, 1);
              transitions[IPtoT.key(d)]->asHLTransition()->AddMarking(Markierung, answer);
              IPtoT.del(IPtoT.key(d));
            }
          }
        } else {
          for(Pix d = IPtoT.first(); d; IPtoT.next(d)) {
            if(IPtoT.contents(d) == Verbindungen[f]->pipe2[0]) {
              activeTransitions.del(IPtoT.key(d));
              const Map<String, Edge *> &input_edges = transitions[IPtoT.key(d)]->InputEdges();
              Pix i;
              HLEdge *hledge;
              HLPlace *inp;

              String Markierung = "";

              for(i = input_edges.first();i;input_edges.next(i)) {
                hledge = input_edges.contents(i)->asHLEdge();
                inp = hledge->Src().asHLPlace();
                MultiSET<Val> tmp_vals = inp->Marking();
                for(Pix mar = tmp_vals.first();mar;tmp_vals.next(mar)) {
                  if(Markierung == "")
                    Markierung = tmp_vals.element(mar).asString();
                  else
                    Markierung += "," + tmp_vals.element(mar).asString();
                }
              }
              transitions[IPtoT.key(d)]->asHLTransition()->AddMarking(Markierung, 0);
              transitions[IPtoT.key(d)]->asHLTransition()->AddMarking(Markierung, "");
              IPtoT.del(IPtoT.key(d));
            }
          }
        }
      }
    } else {
      for(Pix b = ac.first(); b;ac.next(b)) {
        tr = SearchBinding(*transitions[ac(b)]);
        if((!tr.empty()))
          activeTransitions.add(ac(b));
        else
          activeTransitions.del(ac(b));
      }
    }

/*
    int akt = 0;
    for(Pix b = active.first(); b;active.next(b)) {
      if(transitions[active(b)]->asHLTransition()->CanFire()) {
        if(count != 0) {
          write(Verbindungen[akt]->pipe1[1], "IsActive", 32);
          read(Verbindungen[akt]->pipe2[0], buff, 32);
          write(Verbindungen[akt]->pipe1[1], active(b).data(), 32);
          read(Verbindungen[akt]->pipe2[0], buff, 32);
          answer = buff;
          if(answer == "Yes")
            activeTransitions.add(active(b));
          else
            activeTransitions.del(active(b));

          (++akt) %= count;
        } else {
          tr = SearchBinding(*transitions[active(b)]);
          if((!tr.empty())) {
            activeTransitions.add(active(b));
          }
          else
            activeTransitions.del(active(b));
        }
      }
      else
        activeTransitions.del(active(b));
    }

    if(count == 0) {
      for(Pix b = active.first(); b;active.next(b)) {
        if(transitions[active(b)]->asHLTransition()->CanFire()) {
          tr = SearchBinding(*transitions[active(b)]);
          if((!tr.empty())) {
            activeTransitions.add(active(b));
          }
          else
            activeTransitions.del(active(b));
        } else {
          activeTransitions.del(active(b));
        }
      }
    } else {
      fd_set fdvar;
      fd_set aktuell;
      VHMap<String, int> IPtoT(0, 2);
      FD_ZERO(&fdvar);
      int erg;

      Pix b = active.first();
      while(akt < count && b) {
        if(transitions[active(b)]->asHLTransition()->CanFire()) {
          write(Verbindungen[akt]->pipe1[1], "IsActive", 32);
          read(Verbindungen[akt]->pipe2[0], buff, 32);
          write(Verbindungen[akt]->pipe1[1], active(b).data(), 32);
          IPtoT[active(b)] = Verbindungen[akt]->pipe2[0];
          akt++;
        } else {
          activeTransitions.del(active(b));
        }
        active.next(b);
      }
      int max = 0;
      for(int z = 0;z < akt;z++) {
        FD_SET(Verbindungen[z]->pipe2[0], &fdvar);
        if(Verbindungen[z]->pipe2[0] > max)
          max = Verbindungen[z]->pipe2[0];
      }
//aktuell = fdvar;
      FD_ZERO(&aktuell);
      for(int v = 0;v < max+1;v++) {
        if(FD_ISSET(v, &fdvar))
          FD_SET(v, &aktuell);
      }
      
      do {
        select(max+1, &fdvar, (fd_set *)0, (fd_set *)0, (timeval *)0);
        for(int z = 0;z < akt;z++) {
          if(FD_ISSET(Verbindungen[z]->pipe2[0], &fdvar)) {
            read(Verbindungen[z]->pipe2[0], buff, 32);
            answer = buff;
            if(answer == "Yes") {
              for(Pix d = IPtoT.first(); d; IPtoT.next(d)) {
                if(IPtoT.contents(d) == Verbindungen[z]->pipe2[0]) {
                  activeTransitions.add(IPtoT.key(d));
                  IPtoT.del(IPtoT.key(d));
                }
              }
            } else {
              for(Pix d = IPtoT.first(); d; IPtoT.next(d)) {
                if(IPtoT.contents(d) == Verbindungen[z]->pipe2[0]) {
                  activeTransitions.del(IPtoT.key(d));
                  IPtoT.del(IPtoT.key(d));
                }
              }
            }
            if(b)
              active.next(b);
            while(1) {
              if(b) {
                if(transitions[active(b)]->asHLTransition()->CanFire()) {
                  write(Verbindungen[z]->pipe1[1], "IsActive", 32);
                  read(Verbindungen[z]->pipe2[0], buff, 32);
                  write(Verbindungen[z]->pipe1[1], active(b).data(), 32);
                  IPtoT[active(b)] = Verbindungen[z]->pipe2[0];
                  break;
                } else {
                  activeTransitions.del(active(b));
                  if(b)
                    active.next(b);
                }
              } else {
                FD_CLR(Verbindungen[z]->pipe2[0], &aktuell);
                break;
              }
            }
            FD_ZERO(&fdvar);
            for(int v = 0;v < max+1;v++) {
              if(FD_ISSET(v, &aktuell))
                FD_SET(v, &fdvar);
            }
            //break;
          }
        }
        max = 0;
        for(int u = 0;u < akt;u++) {
          if(FD_ISSET(Verbindungen[u]->pipe2[0], &aktuell)) {
            if(max < Verbindungen[u]->pipe2[0])
              max = Verbindungen[u]->pipe2[0];
          }
        }
      } while(max != 0);
    }
*/
  }  // else, Schalten geht klar
  if(timeout) {
    timeval result,end;
    gettimeofday(&end, NULL);
    if(end.tv_usec < start.tv_usec) {
      int nsec = (start.tv_usec - end.tv_usec) / CLOCKS_PER_SEC + 1;
      start.tv_usec -= CLOCKS_PER_SEC * nsec;
      start.tv_sec += nsec;
    }
    if(end.tv_usec - start.tv_usec > CLOCKS_PER_SEC) {
      int nsec = (start.tv_usec - end.tv_usec) / CLOCKS_PER_SEC;
      start.tv_usec += CLOCKS_PER_SEC * nsec;
      start.tv_sec -= nsec;
    }
    result.tv_sec = end.tv_sec - start.tv_sec;
    result.tv_usec = end.tv_usec - start.tv_usec;

    summe.tv_usec += result.tv_usec;
    if(summe.tv_usec > CLOCKS_PER_SEC) {
      summe.tv_sec += 1;
      summe.tv_usec -= CLOCKS_PER_SEC;
    }
    summe.tv_sec += result.tv_sec;
    double erg1 = (double)result.tv_sec + (double)result.tv_usec / CLOCKS_PER_SEC;
    double erg2 = (double)summe.tv_sec + (double)summe.tv_usec / CLOCKS_PER_SEC;
    char *za = new char[100];
    String NAME = t_name;
    sprintf(za, "%-7s\t%.6f sec\t%.6f sec\n", NAME.data(), erg1, erg2);
    protokoll = protokoll + za;
  }
}  // HLNet::FireTransition

void
HLNet::ParseAsignment(const String &asignment_string)
  // Einen String in ein Binding umwandeln.
{
  String s = asignment_string.gsub(" ", "");
  bind.clear();
  while (s != "") {
    String id = s.before("=");
    String v = s.after("=");
    if (v[0] == '(') {
      v = v.after("(");
      String str = v.before(")");
      String tok = str;
      tok = tok.before(")");
      int count = 0;
      while(tok != "") {
        tok = tok.after(",");
        count++;
      }
      Tupel tup(count);
      while(count) {
        tup.add(Val(str.before(",")));
        str.after(",");
        count--;
      }
      // Val v1 = StringToVal(v.after("(").before(","));
      // Val v2 = StringToVal(v.after(",").before(")"));
      bind[id] = tup;
      s = v.after("),");
    }  // if
    else {
      s = v.after(",");
      if((bind.contains(id)) && (StringToVal(v) != bind[id])) {
        // bind.clear();
        //return;
      }
      if (s == "") {
        if(!bind.contains(id))
          bind[id] = StringToVal(v);
      }
      else {
        v = v.before(",");
        if(!bind.contains(id))
          bind[id] = StringToVal(v);
      }  // else
    }  // else
  }  // while s nicht alle
/*
  while(tok) {
    String ident = String(tok).gsub(" ", "");
    tok = strtok(NULL, ",");
    bind[ident] = StringToVal(tok);
    tok = strtok(NULL, "=");
  }
  free(s1);
*/
}  // HLNet::ParseAsignment

void
HLNet::ChangeVars(const char *t1, const char *t2, const char *t3)
{
  char buff[32];
  for(int n = 0; n < count; n++) {
    write(Verbindungen[n]->pipe1[1], "ChangeVars", 32);
    read(Verbindungen[n]->pipe2[0], buff, 32);
    write(Verbindungen[n]->pipe1[1], t1, 32);
    read(Verbindungen[n]->pipe2[0], buff, 32);
    write(Verbindungen[n]->pipe1[1], t2, 32);
    read(Verbindungen[n]->pipe2[0], buff, 32);
    write(Verbindungen[n]->pipe1[1], t3, 256);
  }
}  // Net::ChangeVars



void HLNet::Convert(void)
  // Konvertiert ein Netz mit eindimensionaler Kantenannotation und
  // entsprechenden Valuetermen der Transitionen in das normale
  // Format mit mehrdimensionaler Kantenannotation um
{
  MultiSET<Val> vars_in_edge;
  Val wert, type;
  String temp;
  int pr;
  Transition *t;
  String Value, help;
  HLEdge *e;
 
  for(Pix i = transitions.first();i;transitions.next(i)) {
    t = transitions.contents(i);
    const Map<String, Edge *> &income = t->InputEdges();
    const Map<String, Edge *> &outgo = t->OutputEdges();
    Value = t->asHLTransition()->ValueTerm().Text();
    Value = Value.gsub("(","");
    Value = Value.gsub(")","");
    t->asHLTransition()->ChangeValueTerm(Value);
    for(Pix m = income.first(); m; income.next(m)) {
      e = income.contents(m)->asHLEdge();
      type = e->Src().asHLPlace()->Type()(e->Src().asHLPlace()->Type().first());
      temp = type.asString();
      pr = 0;
      while(temp != "") {
        temp = temp.after(",");
        pr++;
      }
      if(pr > 1) {
        vars_in_edge = e->Vars();
        help = "";
        for (Pix k = vars_in_edge.first(); k; vars_in_edge.next(k)) {
          wert = vars_in_edge.element(k);
          Tupel tu(pr);
          for(int z = 0; z < pr; z++) {
            char *h = new char[5];
            sprintf(h, "%d", z);
            String he(h);
            tu.add(Val("pr"+he+wert.asString()));
            delete [] h;
          }
          if(k != vars_in_edge.first())
            help += ",";
          help += tu.asString();
        }
        e->ChangeVars(help);
      }
    }
    for(Pix aus = outgo.first(); aus; outgo.next(aus)) {
      e = outgo.contents(aus)->asHLEdge();
      type = e->Dest().asHLPlace()->Type()(e->Dest().asHLPlace()->Type().first());
      temp = type.asString();
      pr = 0;
      while(temp != "") {
        temp = temp.after(",");
        pr++;
      }
      if(pr > 1) {
        vars_in_edge = e->Vars();
        help = "";
        for (Pix k = vars_in_edge.first(); k; vars_in_edge.next(k)) {
          wert = vars_in_edge.element(k);
          Tupel tu(pr);
          for(int z = 0; z < pr; z++) {
            char *h = new char[5];
            sprintf(h, "%d", z);
            String he(h);
            tu.add(Val("pr"+he+wert.asString()));
            delete [] h;
          }
          if(k != vars_in_edge.first())
            help += ",";
          help += tu.asString();
        }
        e->ChangeVars(help);
      }
    }
  }
}  // HLNet::Convert

int
HLNet::CreateHigh(ClientData cd, Tcl_Interp *ip, int argc, CONST char *argv[])
  // TCL-Schnittstelle
{
  if (argc != 1) {
    ip->result = "wrong # args";
    return TCL_ERROR;
  }  // if Argumentanzahl falsch
  Net *new_net = new HLNet();
  sprintf(ip->result, "Net%d", net_counter);
  Tcl_CreateCommand(ip, ip->result, Net::CommandBase, (ClientData) new_net, 
    Net::Destroy);
  return TCL_OK;
}  // HLNet::CreateHigh

int 
HLNet::ObjectCommand(Tcl_Interp *ip, int argc, CONST char *argv[])
  // TCL-Schnittstelle
{
  if (argc < 2) {
    Tcl_AppendResult(ip, "wrong # args: should be \"", argv[0],
      " option ?arg arg ...?\"",
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl

  if (EQ(1, "FireTransition")) {
    if (argc != 4) {
      Tcl_AppendResult(ip, "command FireTransition has two arguments", NULL);
      return TCL_ERROR;
    }  // if falsche Argumentzahl fr FireTransition
    if (!transitions.contains(argv[2])) {
      Tcl_AppendResult(ip, "Transition ", argv[2], " does not exist", NULL);
      return TCL_ERROR;
    }  // if Transition unbekannt
    FireTransition(argv[2], argv[3]);
  }  // if Transition schalten

  else if (EQ(1, "Convert")) {
    Convert();
  }

  else if (EQ(1, "read")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command read has argument filename", NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl
    ifstream in(argv[2]);
    if (!in) {
      Tcl_AppendResult(ip, "can't open file ", argv[2], NULL);
      return TCL_ERROR;
    }
    DeleteAll();
    in >> *this;
    in.close();
    String boxname(argv[2]);
    if(boxname.contains(".hl_net"))
      boxname = boxname.before(".hl_net");
    else
      boxname = boxname.before(".ll_net");
    boxname = boxname + ".box";
    ifstream boxes(boxname.data());
    if (boxes) {
      ReadBoxes(boxes);
      boxes.close();
    }
    // Drop out if no connection exists
    if (!count)
      return TCL_OK;
    EstablishConnection();
    char buff[32];
    for(int n = 0; n < count; n++) {
      write(Verbindungen[n]->pipe1[1], "Load", 32);
      read(Verbindungen[n]->pipe2[0], buff, 32);
      write(Verbindungen[n]->pipe1[1], argv[2], 256);
    }
  }  // if read

  else if (EQ(1, "Statistic")) {
    if (argc != 2) {
      Tcl_AppendResult(ip, "command Statistic has no argument", NULL);
      return TCL_ERROR;
    }
    Tcl_AppendResult(ip, protokoll.data(), NULL);
  }
  
  else if (EQ(1, "IsValidBinding")) {
    if (argc !=4) {
      Tcl_AppendResult(ip, "command IsValidBinding has two args", NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl fr IsValidBinding
    if (!transitions.contains(argv[2])) {
      Tcl_AppendResult(ip, "Transition ", argv[2], " does not exist", NULL);
      return TCL_ERROR;
    }  // if Transition unbekannt
    bind.clear();
    ParseAsignment(argv[3]);
      // Jetzt testen, ob Transition unter der Bindung bind schalten kann.
    INTRES(
      (transitions[argv[2]]->asHLTransition()->EvaluateVT(bind) == true )
      && CheckBinding(*(transitions[argv[2]]->asHLTransition())) );
  }  // if Binding validieren

  else if (EQ(1, "SearchBindings")) {
    if (argc != 4) {
      Tcl_AppendResult(ip, "command SearchBindings has two arguments",
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl fr SearchBindings
    if (!transitions.contains(argv[2])) {
      Tcl_AppendResult(ip, "Transition ", argv[2], " does not exist",
        NULL);
      return TCL_ERROR;
    }  // if Transition unbekannt
    if(argv[3] != "nix") {
      bind.clear();
      ParseAsignment(argv[3]);
      BelegteVars = bind;
      BelegteString = argv[3];
    }
    VHSet<String> bindings = SearchBindings(*transitions[argv[2]], 1);
    for (Pix i = bindings.first(); i; bindings.next(i))
      Tcl_AppendElement(ip, bindings(i));
  }  // if Binding suchen

  else if (EQ(1, "SearchMacros")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command SearchMacros has two arguments", NULL);
      return TCL_ERROR;
    }
    if (!boxes.contains(argv[2])) {
      Tcl_AppendResult(ip, "Box ", argv[2], " does not exist", NULL);
      return TCL_ERROR;
    }
    String block(argv[2]);
    VHSet<String> nodes;
    VHSet<String> macros;
    for(Pix i = transitions.first(); i; transitions.next(i)) {
      if(transitions.contents(i)->BoxName() == block) {
        nodes.add(transitions.key(i));
        macros |= transitions.contents(i)->asHLTransition()->GetMacros();
      }
    }
    for(Pix i = places.first(); i; places.next(i)) {
      if(places.contents(i)->BoxName() == block) {
        nodes.add(places.key(i));
        macros |= places.contents(i)->asHLPlace()->GetMacros();
      }
    }
    for(Pix i = edges.first(); i; edges.next(i)) {
      if(nodes.contains(edges.contents(i)->Src().Name()) && nodes.contains(edges.contents(i)->Dest().Name())) {
        macros |= edges.contents(i)->asHLEdge()->GetMacros();
      }
    }
    for(Pix i = macros.first(); i; macros.next(i)) {
      Tcl_AppendElement(ip, macros(i).data());
    }
  }

  else if (EQ(1, "InsertMacros")) {
    if (argc != 4) {
      Tcl_AppendResult(ip, "command InsertMacros has two arguments", NULL);
      return TCL_ERROR;
    }
    if (!boxes.contains(argv[2])) {
      Tcl_AppendResult(ip, "Box ", argv[2], " does not exist", NULL);
      return TCL_ERROR;
    }
    String block(argv[2]);
    String werte(argv[3]);
    boolean foreach;
    werte += "~";
    VHSet<String> nodes;
    VHSet<String> macros;
    VHMap<String, String> ersetzungen("", 5);
    if (werte != "") {
      String davor   = werte.before("~");
      String dahinter = werte.after("~");
      String key     = davor.before("=");
      String content = davor.after("=");
      ersetzungen[key] = content;
      davor = werte.after("~");
      while(dahinter != "") {
        davor = dahinter.before("~");
        key = davor.before("=");
        content = davor.after("=");
        ersetzungen[key] = content;
        dahinter = dahinter.after("~");
      }
      for(Pix i = transitions.first(); i; transitions.next(i)) {
        if(transitions.contents(i)->BoxName() == block) {
          nodes.add(transitions.key(i));
          foreach = false;
          macros = transitions.contents(i)->asHLTransition()->GetMacros();
          for(Pix j = macros.first(); j; macros.next(j)) {
            if(macros(j) != "FOREACH") {
              if(macros(j) != "ELEMENT" && macros(j) != "INDEX") {
                if(ersetzungen.contains(macros(j))) {
                  transitions.contents(i)->asHLTransition()->InsertMacro(macros(j), ersetzungen[macros(j)]);
                }
              }
            } else {
              foreach = true;
            }
          }
          if(foreach) {
            transitions.contents(i)->asHLTransition()->ChangeForeach();
          }
          transitions.contents(i)->asHLTransition()->Instantiate();
        }
      }
      for(Pix i = places.first(); i; places.next(i)) {
        if(places.contents(i)->BoxName() == block) {
          nodes.add(places.key(i));
          macros = places.contents(i)->asHLPlace()->GetMacros();
          for(Pix j = macros.first(); j; macros.next(j)) {
            if(ersetzungen.contains(macros(j))) {
              places.contents(i)->asHLPlace()->InsertMacro(macros(j), ersetzungen[macros(j)]);
            }
          }
          places.contents(i)->asHLPlace()->Instantiate();
        }
      }
      for(Pix i = edges.first(); i; edges.next(i)) {
        if(nodes.contains(edges.contents(i)->Src().Name()) && nodes.contains(edges.contents(i)->Dest().Name())) {
          foreach = false;
          macros = edges.contents(i)->asHLEdge()->GetMacros();
          for(Pix j = macros.first(); j; macros.next(j)) {
            if(macros(j) != "FOREACH") {
              if(macros(j) != "ELEMENT" && macros(j) != "INDEX") {
                if(ersetzungen.contains(macros(j))) {
                  edges.contents(i)->asHLEdge()->InsertMacro(macros(j), ersetzungen[macros(j)]);
                }
              }
            } else {
              foreach = true;
            }
          }
          if(foreach) {
            edges.contents(i)->asHLEdge()->ChangeForeach();
          }
          edges.contents(i)->asHLEdge()->Instantiate();
        }
      }
    }
  }

  else if (EQ(1, "IsAbstract")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command IsAbstract has 1 Argument", NULL);
      return TCL_ERROR;
    }
    String b = argv[2];
    if(IsAbstract(b))
      Tcl_AppendResult(ip, "1", NULL);
    else
      Tcl_AppendResult(ip, "0", NULL);
  }

  else if (EQ(1, "Parallel")) {
    if (argc != 4) {
      Tcl_AppendResult(ip, "command Parallel has 2 Arguments", NULL);
      return TCL_ERROR;
    }
    String Box1 = argv[2];
    String Box2 = argv[3];
    if(!boxes.contains(Box1)) {
      Tcl_AppendResult(ip, "Box ", argv[2], " doesn't exist", NULL);
      return TCL_ERROR;
    }
    if(!boxes.contains(Box2)) {
      Tcl_AppendResult(ip, "Box ", argv[3], " doesn't exist", NULL);
      return TCL_ERROR;
    }
    String name = NewBox();
    Parallel(Box1, Box2, name);
    Tcl_AppendResult(ip, name.data(), NULL);
  }

  else if (EQ(1, "Sequence")) {
    if (argc != 4) {
      Tcl_AppendResult(ip, "command Sequence has 2 Arguments", NULL);
      return TCL_ERROR;
    }
    String Box1 = argv[2];
    String Box2 = argv[3];
    if(!boxes.contains(Box1)) {
      Tcl_AppendResult(ip, "Box ", argv[2], " doesn't exist", NULL);
      return TCL_ERROR;
    }
    if(!boxes.contains(Box2)) {
      Tcl_AppendResult(ip, "Box ", argv[3], " doesn't exist", NULL);
      return TCL_ERROR;
    }
    String name = NewBox();
    Sequence(Box1, Box2, name);
    Tcl_AppendResult(ip, name.data(), NULL);
  }

  else if (EQ(1, "Choice")) {
    if (argc != 4) {
      Tcl_AppendResult(ip, "command Choice has 2 Arguments", NULL);
      return TCL_ERROR;
    }
    String Box1 = argv[2];
    String Box2 = argv[3];
    if(!boxes.contains(Box1)) {
      Tcl_AppendResult(ip, "Box ", argv[2], " doesn't exist", NULL);
      return TCL_ERROR;
    }
    if(!boxes.contains(Box2)) {
      Tcl_AppendResult(ip, "Box ", argv[3], " doesn't exist", NULL);
      return TCL_ERROR;
    }
    String name = NewBox();
    Choice(Box1, Box2, name);
    Tcl_AppendResult(ip, name.data(), NULL);
  }

  else if (EQ(1, "Substitution")) {
    if (argc != 4) {
      Tcl_AppendResult(ip, "command Substitution has 2 Arguments",
        NULL);
      return TCL_ERROR;
    }
    String Box = argv[2];
    String Transition = argv[3];
    if(!boxes.contains(Box)) {
      Tcl_AppendResult(ip, "Box ", argv[2], " doesn't exist", NULL);
      return TCL_ERROR;
    }
    if(!transitions.contains(Transition)) {
      Tcl_AppendResult(ip, "Transition ", argv[3], " doesn't exist", NULL);
      return TCL_ERROR;
    }
    Substitution(Box, Transition);
  }

  else if (EQ(1, "ToNets")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command ToNets has argument filename", NULL);
      return TCL_ERROR;
    }
    if(!hlnets.contains(argv[2]))
      hlnets[argv[2]] = clipboard;
  }

  else if (EQ(1, "InsertNet")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command InsertNet has argument filename", NULL);
      return TCL_ERROR;
    }
    clipboard_temp = clipboard;
    if(hlnets.contains(argv[2]))
      clipboard = hlnets[argv[2]];
  }


  else if (EQ(1, "Restrict")) {
    if (argc > 2) {
      SEL = "";
      String para = argv[2];
      int zaehler = 0;
      while(para != "") {
        para = para.after(" ");
        zaehler++;
      }
      String kette[zaehler];
      para = argv[2];
      for(int i = 0;i < zaehler;i++) {
        kette[i] = (i == zaehler-1) ? para : para.before(" ");
        para = para.after(" ");
      }
      cut(zaehler, kette);
      Restrict();
    }
    else {
      Tcl_AppendResult(ip, "command Restrict needs some nodes", NULL);
      return TCL_ERROR;
    }
  }

  else if (EQ(1, "Expression")) {
    if (argc > 1) {
      if(argv[2] != "")
        MakeExpression(argv[2]);
    } else {
      Tcl_AppendResult(ip, "command Expression needs an expression", NULL);
      return TCL_ERROR;
    }
  }

  else if (EQ(1, "Synchronize")) {
    if (argc != 4) {
      Tcl_AppendResult(ip, "command Synchronize has two arguments", NULL);
      return TCL_ERROR;
    } else {
      SEL = "";
      String para = argv[3];
      int zaehler = atoi(argv[2]);
      String kette[zaehler];
      para = argv[3];
      for(int i = 0;i < zaehler;i++) {
        kette[i] = (i == zaehler-1) ? para : para.before(" ");
        para = para.after(" ");
      }
      cut(zaehler, kette);
      Synchronize();
      if (synchro_error) {
         Tcl_AppendResult(ip, synchro_error_msg.data(), NULL);
         return TCL_ERROR;
       }
    }
  }

  else if (EQ(1, "Tie")) {
    if (argc > 2) {
      String para = argv[2];
      Tie(para);
    }
    else {
      Tcl_AppendResult(ip, "command Tie need some link names", NULL);
      return TCL_ERROR;
    }
  }

  else if (EQ(1, "Link")) {
    if (argc > 3) {
      String l_name = argv[3];
      if (EQ(2,"delete"))
          removeLink(l_name);
      else
        {
          String l_type = "";
          if (argc > 4)
               l_type = argv[4];
          if (EQ(2,"add"))
             { // how do you send an error message to say the link has not been added?
               boolean changed = addLink(l_name,l_type);
               if (!changed)
                {
                  Tcl_AppendResult(ip, "This link may already exist or is uncorrect",
                     NULL);
                  return TCL_ERROR;
                };
             }
          else if (EQ(2,"change"))
             {
               changeLinkType(l_name,l_type);
             }
          else
             {
               Tcl_AppendResult(ip, "Unknown Link option", NULL);
               return TCL_ERROR;
             };
        }
    }
    else {
      Tcl_AppendResult(ip, "command Link need some arguments", NULL);
      return TCL_ERROR;
    }
  }

  else if (EQ(1, "SynchronOption")) {
    int argument;
    if (argc != 4) {
      Tcl_AppendResult(ip, "command SynchronOption has two arguments",
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl fr SynchronOption
    else {
      if (EQ(2, "Max_Arc_Var")) {
        if(Tcl_GetInt(ip, argv[3], &argument) != TCL_OK) {
          return TCL_ERROR;
        }
        max_arc_vars = argument;
      }
      else if (EQ(2, "Max_Trans")) {
        if (Tcl_GetInt(ip, argv[3], &argument) != TCL_OK) {
          return TCL_ERROR;
        }
        max_trans_anz = argument;
      }
      else if (EQ(2, "EliminateDead")) {
        if(Tcl_GetInt(ip, argv[3], &argument) != TCL_OK) {
          return TCL_ERROR;
        }
        if(argument)
          flag |= ELIMINATE_DEAD;
        else
          flag &= (~ELIMINATE_DEAD);
      }
      else if (EQ(2, "RemoveDuplicate")) {
        if(Tcl_GetInt(ip, argv[3], &argument) != TCL_OK) {
          return TCL_ERROR;
        }
        if(argument)
          flag |= REMOVE_DUPL;
        else
          flag &= (~REMOVE_DUPL);
      }
      else if (EQ(2, "RealName")) {
        if(Tcl_GetInt(ip, argv[3], &argument) != TCL_OK) {
          return TCL_ERROR;
        }
        if(argument)
          var_style = VAR_REAL;
        else
          var_style = VAR_COM;
      }
      else if (EQ(2, "BlockOutput")) {
        if(Tcl_GetInt(ip, argv[3], &argument) != TCL_OK) {
          return TCL_ERROR;
        }
        if(!argument)
          flag |= BLOCK_OUTPUT;
        else
          flag &= (~BLOCK_OUTPUT);
      }
      else if (EQ(2, "PhantomOutput")) {
        if(Tcl_GetInt(ip, argv[3], &argument) != TCL_OK) {
          return TCL_ERROR;
        }
        if(!argument)
          flag |= PHANTOM_TRANS_OUTPUT;
        else
          flag &= (~PHANTOM_TRANS_OUTPUT);
      }
      else if (EQ(2, "Restrict")) {
        if(Tcl_GetInt(ip, argv[3], &argument) != TCL_OK) {
          return TCL_ERROR;
        }
        if(!argument)
          flag |= MNET_RESTRICTION;
        else
          flag &= (~MNET_RESTRICTION);
      }
      else if (EQ(2, "Synchronize")) {
        if(Tcl_GetInt(ip, argv[3], &argument) != TCL_OK) {
          return TCL_ERROR;
        }
        if(!argument)
          flag |= MNET_SYNCHRONIZE;
        else
          flag &= (~MNET_SYNCHRONIZE);
      }
      else if (EQ(2, "CheckNet")) {
        if(Tcl_GetInt(ip, argv[3], &argument) != TCL_OK) {
          return TCL_ERROR;
        }
        if(!argument)
          flag |= CHECK_NET;
        else
          flag &= (~CHECK_NET);
      }
      else if (EQ(2, "includestring")) {
        if (!strcmp(argv[3],",leer"))
          include_string = "";
        else
          include_string = argv[3];
      }
      else if (EQ(2, "excludestring")) {
        if (!strcmp(argv[3],",leer"))
          exclude_string = "";
        else
          exclude_string = argv[3];
      }
      else if (EQ(2, "filename")) {
        clipboardfile = argv[3];
      }
    }
  }

  else if (EQ(1, "RandomFire")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command RandomFire has one argument",
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl fr RandomFire
    if (!transitions.contains(argv[2])) {
      Tcl_AppendResult(ip, "Transition ", argv[2], " does not exist",
        NULL);
      return TCL_ERROR;
    }  // if Transition unbekannt
    RandomFire(argv[2]);
  }  // if zuflliges Schalten

  else if (EQ(1, "DeleteStatistic")) {
    if (argc != 2) {
      Tcl_AppendResult(ip, "command DeleteStatistic has no arguments",
        NULL);
      return TCL_ERROR;
    }
    summe.tv_sec = 0;
    summe.tv_usec = 0;
    protokoll = "";
  }

  else if (EQ(1, "get")) {
    if (EQ(2, "VarsToBind")) {
      if (argc != 4) {
        Tcl_AppendResult(ip, "command VarsToBind has one argument",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl fr VarsToBind
      if (!transitions.contains(argv[3])) {  
        Tcl_AppendResult(ip, "Transition ", argv[3], " does not exist",
          NULL);
        return TCL_ERROR;
      }  // if Transition unbekannt
      actual_trans = transitions[argv[3]]->asHLTransition();
      CheckPossibleValues();
      String bindung;
      if(BelegteString != "")
        bindung = BelegteString+","+bindungen(bindungen.first());
      else
        bindung = bindungen(bindungen.first());
      bind.clear();
      ParseAsignment(bindung);
      BelegteString = "";
      for(Pix i = bind.first(); i; bind.next(i)) {
        Tcl_AppendElement(ip, bind.key(i));
      }
    }  // Variablen zu Bindings

    else if (EQ(2, "Nets")) {
      if(argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      for(Pix i = hlnets.first(); i; hlnets.next(i)) {
        Tcl_AppendElement(ip, hlnets.key(i));
      }
    }

    else if (EQ(2, "PossVals")) {
      if (argc != 4) {
        Tcl_AppendResult(ip, "command PossVals has one argument",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl fr PossVals
      CheckPossibleValues();
      String bindung;
      VHSet<String> schon;
      for(Pix i = bindungen.first(); i;bindungen.next(i)) {
        if(BelegteString != "")
          bindung = BelegteString+","+bindungen(i);
        else
          bindung = bindungen(i);
        bind.clear();
        ParseAsignment(bindung);
        if(!schon.contains(bind[argv[3]].asString())) {
          schon.add(bind[argv[3]].asString());
          Tcl_AppendElement(ip, bind[argv[3]].asString());
        }
      }
      BelegteString = ""; 
    }

    else if (EQ(2, "Links")) {
      if(argc != 3) {
        Tcl_AppendResult(ip, "command get Links has no argument",
          NULL);
        return TCL_ERROR;
      }
      for(Pix i = declared_links.first(); i; declared_links.next(i)) {
        Tcl_AppendElement(ip, declared_links.key(i));
        Tcl_AppendElement(ip, declared_links.contents(i).get_string_type());
      }

   }  // if mgliche Werte
    else return Net::ObjectCommand(ip, argc, argv);
  }  // if get irgendwas

  else if (EQ(1, "Copy")) {
    if (argc > 2) {
      SEL = "";
      String para = argv[2];
      int zaehler = 0;
      while(para != "") {
        para = para.after(" ");
        zaehler++;
      }
      String kette[zaehler];
      para = argv[2];
      for(int i = 0;i < zaehler;i++) {
        kette[i] = (i == zaehler-1) ? para : para.before(" ");
        para = para.after(" ");
      }
      copy(zaehler, kette);
    }
    else {
      Tcl_AppendResult(ip, "command Copy needs some nodes",
        NULL);
      return TCL_ERROR;
    }
  }

  else if (EQ(1, "Cut")) {
   if (argc > 2) {
      SEL = "";
      String para = argv[2];
      int zaehler = 0;
      while(para != "") {
        para = para.after(" ");
        zaehler++;
      }
      String kette[zaehler];
      para = argv[2];
      for(int i = 0;i < zaehler;i++) {
        kette[i] = (i == zaehler-1) ? para : para.before(" ");
        para = para.after(" ");
      }
      cut(zaehler, kette);
    }
    else {
      Tcl_AppendResult(ip, "command Cut needs some nodes",
        NULL);
      return TCL_ERROR;
    }
  }

  else if (EQ(1, "Paste")) {
    if (argc != 2) {
      Tcl_AppendResult(ip, "command Paste has no argument",
        NULL);
      return TCL_ERROR;
    }
    paste();
  }

  else if (EQ(1, "LoadModul")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command LoadModul has one argument",
        NULL);
      return TCL_ERROR;
    }
    loadmodul(argv[2]);
  }

  else if (EQ(1, "SaveModul")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command SaveModul has one argument",
        NULL);
      return TCL_ERROR;
    }
    savemodul(argv[2]);
  }

  else if (EQ(1, "Cache")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command Cache has one argument",
        NULL);
      return TCL_ERROR;
    }
    String erg(argv[2]);
    if(erg == "1") {
      cache = true;
    } else {
      cache = false;
    }
  }

  else if (EQ(1, "PreAnalyse")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command PreAnalyse has one argument",
        NULL);
      return TCL_ERROR;
    }
    String erg(argv[2]);
    if(erg == "1") {
      preAnalyse = true;
    } else {
      preAnalyse = false;
    }
  }

  else if (EQ(1, "InAnalyse")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command InAnalyse has one argument",
        NULL);
      return TCL_ERROR;
    }
    String erg(argv[2]);
    if(erg == "1") {
      inAnalyse = true;
    } else {
      inAnalyse = false;
    } 
  }

  else if (EQ(1, "ProcSemantic")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command ProcSemantic has one argument",
        NULL);
      return TCL_ERROR;
    }
    String erg(argv[2]);
    if(erg == "1")
      GL_ProcDecl = True;
    else
      GL_ProcDecl = False;
  }

  else if (EQ(1, "AllBindings")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command AllBindings has one argument",
        NULL);
      return TCL_ERROR;
    }
    String erg(argv[2]);
    if(erg == "1") {
      alle = true;
    } else {
      alle = false;
    } 
  }

  else if (EQ(1, "TimeOutput")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command TimeOutput has one argument",
        NULL);
      return TCL_ERROR;
    }
    String erg(argv[2]);
    if(erg == "1") {
      timeout = true;
    } else {
      timeout = false;
    } 
  }

  else
    return Net::ObjectCommand(ip, argc, argv);
  return TCL_OK;
}  // HLNet::ObjectCommand

  //*** Dateiende HLNet.cc ***
    
