// HLNet.h
// Tobias Himstedt 1995
// Burkhard Bieber 1995/96
// Cecile Bui Thanh: added asynchronous links (2001)
// last-update: Dec. 2001

#ifndef __HLNET_H__
#define __HLNET_H__

  //**********
  // includes
  //**********

#include "../include/OSLSet.h"
#include "../include/VHSet.h"
#include "HLBlock.h"
#include "HLPlace.h"
#include "HLTransition.h"
#include "HLPTransition.h"
#include "HLEdge.h"
#include "../Low/Net.h"
#include "../Low/Link.h"
extern "C" {
#include "../../backend/mnet.h"
}

  //************
  // Konstanten
  //************

const int DefaultHLSize = 30;

#define MNET_SYNCHRONIZE 8192
#define MNET_RESTRICTION 16384


  //*********
  // Klassen
  //*********

class HLNet : public Net {
  private:
      // Datenelemente
    int rx_place_type, ry_place_type;
    int rx_trans_at, ry_trans_at;
    int rx_trans_vt, ry_trans_vt;
	int rx_trans_lt, ry_trans_lt;
    int rx_edge_var, ry_edge_var;
    VHMap<Val, VHSet<Val> > possible_value_set;
    VHSet<String> bindungen;
    Binding bind;
    Binding BelegteVars;
    String BelegteString;
    String include_string;
    String exclude_string;
    String clipboardfile;
    boolean alle; // consider all bindings
    boolean cache;
    boolean preAnalyse;
    boolean inAnalyse;
    boolean timeout;
    HLTransition *actual_trans;
      // Methoden
    virtual int FormatIsOk(const String &format) const;
    virtual const String &Format() const;
    HLNet *clipboard;
      // variables for error-handling, when there's an error
      // while thrying to synchronize
    boolean synchro_error;
    String synchro_error_msg;
      // Methoden fuer Tie()
    void newImportLink ( HLTransition * trans,
                         VHMap<String, HLPlace *> &added_places,
						 String lname,
						 String ltargs,
						 int nof_lterm);
    void newExportLink ( HLTransition * trans,
                         VHMap<String, HLPlace *> &added_places,
						 String lname,
						 String ltargs,
						 int nof_lterm);
  protected:
    virtual Block *AllocBlock() const;
    virtual Box *AllocBox() const;
    virtual Place *AllocPlace() const;
    virtual Transition *AllocTransition() const;
    virtual PTransition *AllocPTransition() const;
    virtual Edge *AllocEdge() const;
    virtual Edge *AllocEdgeFT(Vertex &f, Vertex &t) const;
    virtual Edge *AllocPEdge() const;
    virtual Edge *AllocPEdgeFT(Vertex &f, Vertex &t) const;
    void ParseAsignment(const String &asignment_string);
    void PreAnalyse(HLTransition *actual_trans);
    void InAnalyse(HLTransition *actual_trans,VHMap<String,VHSet<Val> > &Mengen);
    VHSet<String> SearchBindings(const Transition &t, int all_flag);
    VHSet<String> SearchBinding(const Transition &tra);
    bool CheckBinding(const HLTransition &t);
    void EvaluateAutoBinding(Pix i, SET<String> &binding_strings, int);
    void CheckPossibleValues();
    void possible(MultiSET<Val> &t_vars, MultiSET<Val> &t_marking, String bindung);
    virtual int ObjectCommand(Tcl_Interp *ip, int argc, CONST char *argv[]);
  public:
      // Defaults abholen
    void DefaultHLPlaceType(int &px, int &py) const;
    int DefaultHLPlaceTypeX() const;
    int DefaultHLPlaceTypeY() const;
    void DefaultHLTransAT(int &px, int &py) const;
    int DefaultHLTransATX() const;
    int DefaultHLTransATY() const;
    void DefaultHLTransVT(int &px, int &py) const;
    int DefaultHLTransVTX() const;
    int DefaultHLTransVTY() const;
    void DefaultHLTransLT(int &px, int &py) const;
    int DefaultHLTransLTX() const;
    int DefaultHLTransLTY() const;
    void DefaultHLTransMean(int &px, int &py) const;
    void DefaultHLEdgeVars(int& px, int& py) const;
    int DefaultHLEdgeVarsX() const;
    int DefaultHLEdgeVarsY() const;
      // Casts
    virtual HLNet *asHLNet(void) const;

	  // do the tie operation with the links in names_set
    void Tie(String links_to_tie);

//    void SetLinksToTie(String s){ links_to_tie = s; };

      // remove a link, and every link terms with l_name as name.
      // if this link wasn't declared, do nothing
    void removeLink (String l_name);

	// change the type of a link
	void changeLinkType(String link_name, String new_type);

      // Simulation betreffend
    virtual VHSet<String> ActiveTransitions();
    void RandomFire(const String &t_name);
    String RandomBinding(const Transition &t);
      // Transition t wird geschaltet 
    void FireTransition(const String &t, const String &bind_string);
      // Umwandlung von altem in neues Format
    void Convert(void);
    virtual void copy(int c, String kette[]);
    virtual void cut(int c, String kette[]);
    virtual void paste(void);
    virtual void loadmodul(CONST char *argv, boolean inbox = true, String b = "", boolean clear_links = false);
    virtual void savemodul(CONST char *argv);
    virtual void copyBlock(VHMap<String, Transition *> *Trans,
              VHMap<String, Place *> *Stellen,
              VHMap<String, Block *> *Blocks,
              VHMap<String, PTransition *> *PTrans,
              VHMap<String, Box *> *Boxes,
              String fromBlock, String toBlock);
    virtual void pasteBlock(VHMap<String, Transition *> *Trans,
              VHMap<String, Place *> *Stellen,
              VHMap<String, Block *> *Blocks,
              VHMap<String, PTransition *> *PTrans,
              VHMap<String, Box *> *Boxes,
              String fromBlock, String toBlock);
    virtual void Parallel(String Block1, String Block2, String name);
    virtual void Sequence(String Block1, String Block2, String name);
    virtual void Choice(String Block1, String Block2, String name);
    virtual void Substitution(String Box, String Transition);
    M_Net Synchronize(void);
    void Restrict(void);
    void MakeExpression(String term);
    int IsAbstract(String Box);
    VHSet<String> ActionTerms(void);
    VHSet<String> ParseLinkString(String s);
    VHSet<String> ParseString(String s);
    void ChangeVars(const char *t1, const char *t2, const char *t3);
      // Konstruktor
    HLNet();
    ~HLNet(){};
      // Tcl-Schnittstellen
    static int CreateHigh(ClientData cd, Tcl_Interp *ip, int argc,
      CONST char *argv[]);
};  // class HLNet

  //************
  // Konstanten
  //************

const String HLNetString = "HLNet";

  //*******************
  // Inlines fr HLNet
  //*******************

inline int 
HLNet::FormatIsOk(const String &f) const
{
  return (f == HLNetString);
}  // HLNet::FormatIsOk

inline const String & 
HLNet::Format() const
{
  return HLNetString;
}  // HLNet::Format

inline void
HLNet::DefaultHLPlaceType(int &x, int &y) const
{
  x = rx_place_type;
  y = ry_place_type;
}  // HLNet::DefaultHLPlaceType

inline int
HLNet::DefaultHLPlaceTypeX() const
{
  return rx_place_type;
}  // HLNet::DefaultHLPlaceTypeX

inline int
HLNet::DefaultHLPlaceTypeY() const
{
  return ry_place_type;
}  // HLNet::DefaultHLPlaceTypeY

inline void
HLNet::DefaultHLTransAT(int &x, int &y) const
{
  x = rx_trans_at;
  y = ry_trans_at;
}  // HLNet::DefaultHLTransAT

inline int
HLNet::DefaultHLTransATX() const
{
  return rx_trans_at;
}  // HLNet::DefaultHLTransATX

inline int
HLNet::DefaultHLTransATY() const
{
  return ry_trans_at;
}  // HLNet::DefaultHLTransATY

inline void
HLNet::DefaultHLTransVT(int &x, int &y) const
{
  x = rx_trans_vt;
  y = ry_trans_vt;
}  // HLNet::DefaultHLTransVT

inline int
HLNet::DefaultHLTransVTX() const
{
  return rx_trans_vt;
}  // HLNet::DefaultHLTransVTX

inline int
HLNet::DefaultHLTransVTY() const
{
  return ry_trans_vt;
}  // HLNet::DefaultHLTransVTY

inline void
HLNet::DefaultHLTransLT(int &x, int &y) const
{
  x = rx_trans_lt;
  y = ry_trans_lt;
}  // HLNet::DefaultHLTransLT

inline int
HLNet::DefaultHLTransLTX() const
{
  return rx_trans_lt;
}  // HLNet::DefaultHLTransLTX

inline int
HLNet::DefaultHLTransLTY() const
{
  return ry_trans_lt;
}  // HLNet::DefaultHLTransLTY

inline void
HLNet::DefaultHLTransMean(int &x, int &y) const
{
  x = this->rx_trans_mean;
  y = this->ry_trans_mean;
}  // HLNet::DefaultHLTransMean

inline void
HLNet::DefaultHLEdgeVars(int &x, int &y) const
{
  x = rx_edge_var;
  y = ry_edge_var;
}  // HLNet::DefaultHLEdgeVars

inline int
HLNet::DefaultHLEdgeVarsX() const
{
  return rx_edge_var;
}  // HLNet::DefaultHLEdgeVarsX

inline int
HLNet::DefaultHLEdgeVarsY() const
{
  return ry_edge_var;
}  // HLNet::DefaultHLEdgeVars

#endif  // #ifndef __HLNET_H__

  //*** Dateiende HLNet.h ***

