// HLPTransition.cc
// Tobias Himstedt 1995
// Burkhard Bieber 1995/96
// last-update: 12.01.96

  //**********
  // includes
  //**********

#include "HLPTransition.h"
#include "HLNet.h"

  //****************************
  // Methoden fr HLPTransition
  //****************************

HLPTransition::HLPTransition(const Net &n, const String &Type) :
  PTransition(n, Type), action_terms(""), value_term("tt") 
{
  base_classes.add("PTransition");
  net->asHLNet()->DefaultHLTransAT(rx_at, ry_at);
  net->asHLNet()->DefaultHLTransVT(rx_vt, ry_vt);
}  // HLPTransition::HLPTransition

HLPTransition *
HLPTransition::asHLPTransition() const
{
  return (HLPTransition *) this;
}  // HLPTransition::asHLPTransition

void
HLPTransition::Write(ostream &f) const
{
  PTransition::Write(f);
  f << "g\"" << value_term.Text() << '"';
  if (rx_vt != net->asHLNet()->DefaultHLTransVTX() || 
    ry_vt != net->asHLNet()->DefaultHLTransVTY())
    f << 'h' << rx_vt << '@' << ry_vt;
  if (action_terms != "")
    f << "i\"" << action_terms << '"';
  if (rx_at != net->asHLNet()->DefaultHLTransATX() || 
    ry_at != net->asHLNet()->DefaultHLTransATY())
    f << 'j' << rx_at << '@' << ry_at;
}  // HLPTransition::Write

void
HLPTransition::ReadOptional(istream &f)
{
  PTransition::ReadOptional(f);
  char c;
  f.get(c);
  String g;

  switch (c) {
    case 'g':
      f >> c;
      readline(f, g, '"');
      value_term.ChangeTerm(g);
      break;
    case 'h':
      f >> rx_vt >> c >> ry_vt;
      break;
    case 'i':
      f >> c;
      readline(f, action_terms, '"');
      break;
    case 'j':
      f >> rx_at >> c >> ry_at;
      break;
    default:
      f.putback(c);
  }  // switch ber c
}  // HLPTransition::ReadOptional

int
HLPTransition::ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[])
{
  if (argc != 1) {
    Tcl_AppendResult(ip, "wrong #args for PTransition get",
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl fr PTransition get
  if (EQ(0, "AT"))
    Result(ip, action_terms);
  else if (EQ(0, "VT"))
    Result(ip, value_term.Text());
  else if (EQ(0, "ATX"))
    INTRES(rx_at);
  else if (EQ(0, "ATY"))
    INTRES(ry_at);
  else if (EQ(0, "VTX"))
    INTRES(rx_vt);
  else if (EQ(0, "VTY"))
    INTRES(ry_vt);
  else
    return PTransition::ObjectGet(ip, argc, argv);
  return TCL_OK;
}  // HLPTransition::ObjectGet
	
int
HLPTransition::ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[])
{
  if (argc < 2) {
    Tcl_AppendResult(ip, "wrong #args for PTransition set",
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl fr PTransition set
  char *at = NULL;
  char *vt = NULL;
  Tk_ArgvInfo parse_table[] = {
    {"AT", TK_ARGV_STRING, NULL, (char *) &at,
     "Actionterms of PTransition"},
    {"VT", TK_ARGV_STRING, NULL, (char *) &vt,
     "Valueterm of PTransition"},
    {"ATX", TK_ARGV_INT, NULL, (char *) &rx_at,
     "Relativ x - position of Actionterms"},
    {"ATY", TK_ARGV_INT, NULL, (char *) &ry_at,
     "Relativ y - position of Actionterms"},
    {"VTX", TK_ARGV_INT, NULL, (char *) &rx_vt,
     "Relativ x - position of Valueterm"},
    {"VTY", TK_ARGV_INT, NULL, (char *) &ry_vt,
     "Relativ y - position of Valueterm"},
    {NULL, TK_ARGV_END, NULL, NULL, NULL}};
  if (Tk_ParseArgv(ip, NULL, &argc, argv, parse_table, 
    TK_ARGV_DONT_SKIP_FIRST_ARG | TK_ARGV_NO_ABBREV) != TCL_OK)
    return TCL_ERROR;
  if (vt) 
    value_term.ChangeTerm(vt);
  if (at)
    action_terms = at;
  if (argc > 0)
    return PTransition::ObjectSet(ip, argc, argv);
  return TCL_OK;
}  // HLPTransition::ObjectSet

  //*** Dateiende HLPTransition.cc ***

