// HLPlace.cc
// Tobias Himstedt 1995
// Burkhard Bieber 1995/96
// Cecile Bui Thanh: added some functions for the Tie operator (2001)
// last-update: Dec. 2001

  //**********
  // includes
  //**********

#include "HLPlace.h"
#include "HLNet.h"
#include "HLBox.h"
#include "../include/MultiSet.h"
#include "../include/VHSet.h"

  //**********************
  // Methoden fr HLPlace
  //**********************

HLPlace::HLPlace(const Net &n, const String &t) : Place(n, t),
  type(10), type_string("{dot}"), abstract(false)
  // Konstruktor
{
  net->asHLNet()->DefaultHLPlaceType(rx_type, ry_type);
  type.add(dot);
}  // HLPlace::HLPlace

void
HLPlace::AddToken(const Val &new_value)
  // Token mit wert new_value zufgen
{
  Place::AddTokens(1);
  current_marking.add(new_value);
}  // HLPlace::AddToken

void
HLPlace::RemoveToken(const Val &v)
  // Token mit Wert v (sofern vorhanden) entfernen
{
  if (current_marking.contains(v)) {
    Place::RemoveTokens(1);
    current_marking.del(v);
  }  // if
}  // HLPlace::RemoveToken

void
HLPlace::DeleteInitMarking(void)
  // Initial Marking wird gelscht
{
  initial_marking.clear();
}  // HLPlace::DeleteInitMarking

void
HLPlace::DeleteAllTokens()
  // Markieung entfernen
{
  Place::DeleteAllTokens();
  net->ChangeMarking(name.data(), "");
  current_marking.clear();
}  // HLPlace::DeleteAllTokens

void
HLPlace::SetTyp(VHSet<Val> &Typ, String &t)
  // Typ "andern
{
  HLTransition *tr;

  DeleteAllTokens();
  type = Typ;
  String na = Name();
  String ne = t;
  for(Pix i = incoming_edges.first();i;incoming_edges.next(i)) {
    tr = incoming_edges.contents(i)->Src().asHLTransition();
    tr->CacheFlush();
  }
  net->ChangeTyp(na.data(), ne.data());
  type_string = t;
}  // HLPlace::SetTyp

void
HLPlace::WriteMarking(ostream &f) const
  // Markierung (Init & Current) und evtl. Entry-Flag schreiben
{
  if (state == entry) {
    f << 'e';
  }  // if Entry-Stelle
  if (!initial_marking.empty())
    f << "z\"{" << MarkingToString(initial_marking) << "}\"";
  if (!current_marking.empty())
    f << "y\"{" << MarkingToString(current_marking) << "}\"";
  f << "Z\"" << type_string << "\"";
}  // HLPlace::WriteMarking

void
HLPlace::Write(ostream &f) const
  // Werte fr HL Stellen schreiben
{
  Place::Write(f);
  if (ry_type != net->asHLNet()->DefaultHLPlaceTypeX() || 
    ry_type != net->asHLNet()->DefaultHLPlaceTypeY())
    f << 'B' << rx_type << '@' << ry_type;
}  // HLPlace::Write

void
HLPlace::ReadOptional(istream &f)
  // Optionale Parameter von stream f einlesen
{
  char c;
  String mark, init_mark, type_s;

  Place::ReadOptional(f);
  f >> c;
  switch (c) {
    case 'y':
      f >> c;
      readline(f, mark, '"');
      StringToMultiSet(mark, current_marking);
      tokens = current_marking.elements();
      break;
    case 'z':
      f >> c;
      readline(f, init_mark, '"');
      StringToMultiSet(init_mark, initial_marking);
      initial_tokens = initial_marking.elements();
      break;
    case 'Z':
      f >> c;
      readline(f, type_s, '"');
      if (type_s.contains("$")) {
        abstract = true;
        type_string = type_s;
      } else {
        abstract = false;
        StringToType(type_s);
      }
      break;
    case 'B':
      f >> rx_type >> c >> ry_type;
      break;
    default:
      f.putback(c);
  }  // switch ber c
}  // HLPlace::ReadOptional

void 
HLPlace::SetEntry(int read_flag)
  // Stelle als Entry-Stelle festlegen, evtl. Markierung anpassen. type=dot.
{
  Place::SetEntry(read_flag);
  if (!read_flag) {
    initial_marking.add(dot);
    current_marking.add(dot);
  }  // if nicht aus der Datei gelesen
  //type.add(dot);
}  // HLPlace::SetEntry

void 
HLPlace::SetExit()
  // Stelle als Exit-Stelle festlegen. Also type=dot.
{
  Place::SetExit();
  //type.add(dot);
}  // HLPlace::SetExit

VHSet<String>
HLPlace::GetMacros(void)
  // Macros herausfiltern
{
  VHSet<String> macros;
  if(!type_string.contains("$"))
    return macros;
  String davor, dahinter;
  davor = type_string.after("$");
  dahinter = davor.before("$");
  while(davor != "") {
    macros.add(dahinter);
    davor = davor.after("$");
    davor = davor.after("$");
    dahinter = davor.before("$");
  }
  davor = reference.after("$");
  dahinter = davor.before("$");
  while(davor != "") {
    macros.add(dahinter);
    davor = davor.after("$");
    davor = davor.after("$");
    dahinter = davor.before("$");
  }
  return macros;
}

void
HLPlace::InsertMacro(String macro, String ersetzung)
  // Macros ersetzen
{
  String davor, dahinter;
  macro = "$" + macro + "$";
  while(type_string.contains(macro)) {
    davor = type_string.before(macro);
    dahinter = type_string.after(macro);
    type_string = davor + ersetzung + dahinter;
  }
  while(reference.contains(macro)) {
    davor = reference.before(macro);
    dahinter = reference.after(macro);
    reference = davor + ersetzung + dahinter;
  }
}

void
HLPlace::Instantiate()
  // Typ umwandeln
{
  String vor, nach;
  while(type_string.contains("U ")) {
    vor = type_string.before("U ");
    nach = type_string.after("U ");
    type_string = vor + "U" + nach;
  }
  while(type_string.contains(" U")) {
    vor = type_string.before(" U");
    nach = type_string.after(" U");
    type_string = vor + "U" + nach;
  }
  while(type_string.contains("}U{")) {
    vor = type_string.before("}U{");
    nach = type_string.after("}U{");
    type_string = vor + "," + nach;
  }
  StringToType(type_string);
  abstract = false;
}

String
HLPlace::MarkingToString(const MultiSET<Val> &mark) const
  // Konvertiert Markierung bzw Multimenge ber Val in einen String
{
  String l;
  Pix n = mark.first();

  for (Pix i = mark.first(); i; mark.next(i)) {
    mark.next(n);
      // die Umkehrung der Schreibweise kann evtl. Probleme bringen!
    if (mark.nof(i) > 1)
      l += IntToString(mark.nof(i)) + "*" + mark.element(i).asString();
    else
      l += mark.element(i).asString();
    if (n)
      l += ",";
  }  // for i ber Markierungen
  return l;
}  // HLPlace::MarkingToString


void
AppendRange(int start, int end, String &l)
  // Hngt an String l den String start..end an, falls start-end>3
  // sonst start, start+1, .. end
{
  if (end - start > 3)  // falls es sich lohnt
    l += IntToString(start) + ".." + IntToString(end);
  else {  // Falls nicht -> normal eintragen
    for (int j = start; j < end; j++) 
      l += IntToString(j) + ",";
    l += IntToString(end);
  }  // else
}  // AppendRange

void
HLPlace::StringRangeAppendMultiSet(String range, MultiSET<Val> &set) const
  // Fgt an die Multimenge set einen Wert an, der aus dem String range
  // gewonnen wird. range kann aber auch eine Bereich start..ende sein
  // In diesem Fall werden alle Werte von start bis end zur Multimenge#
  // zugefgt.
{
  String lower = range.before("..");
  if (lower != "") {
    Val anfang = StringToVal(lower);
    String upper = range.after("..");
    Val ende = StringToVal(upper);
    if (anfang.Type() != INTEGER || ende.Type() != INTEGER) {
      Error("Lower and upper bounds must be Integers");
      return ;
    }  // if
    if (ende < anfang) { 
      Val tmp = anfang;
      anfang = ende;
      ende = tmp;
    }  // if
    for (int p = anfang.asInt(); p <= ende.asInt(); p++)
      set.add(p);
  } else {
    String v = range.before("*");
    if (v != "") { 
      String h = range.after("*");
      set.add(StringToVal(h), atoi(v));
    }  // if
    else {
      set.add(StringToVal(range));
    }
  }  // else,
}  // HLPlace::StringRangeAppendMultiSet
	
void
HLPlace::StringToMultiSet(String s, MultiSET<Val> &set) const
  // Konvertiert String in eine Multimenge ber Val
{
  set.clear();
  String sv;

  if (s == "")
    return ;
  s = s.gsub("{", "");
  s = s.gsub("}", "");
  s = s.gsub(" ", "");
  do {
    if (s[0] == '(') {
      String t1 = s.after("(");
        // Tupel gefunden
      String ta = t1.before(")");
      String te = ta;
      int help = 0;
      while(te != "") {
        te = te.after(",");
        help++;
      }
      Tupel temp(help);
      while(ta != "") {
        if(ta.before(",") != "")
          temp.add(StringToVal(ta.before(",")));
        else
          temp.add(StringToVal(ta));
        ta = ta.after(",");
      }
      set.add(temp);
      /*Val v1 = StringToVal(ta.before(","));
      Val v2 = StringToVal(ta.after(","));
      set.add(Tupel(v1, v2));*/
      s = t1.after("),");
    } else {
      sv = s.before(",");
      if (sv == "")
        StringRangeAppendMultiSet(s, set);
      else
        StringRangeAppendMultiSet(sv, set);
      s = s.after(",");
    }  // else
  } while(s != "");
}  // HLPlace::StringToMultiSet

void
HLPlace::StringRangeAppendSet(String range, SET<Val> &set) const
  // Fgt an die Menge set einen Wert an, der aus dem String range
  // geewonnen wird. range kann aber auch eine Bereich start..ende sein
  // In diesem Fall werden alle Werte von start bis end zur Menge
  // zugefgt.
{
  String lower = range.before("..");
  if (lower != "") {
    Val anfang = StringToVal(lower);
    String upper = range.after("..");
    Val ende = StringToVal(upper);
    if (anfang.Type() != INTEGER || ende.Type() != INTEGER) {
      Error("Lower and upper bounds must be Integers");
      return ;
    }  // if
    if (ende < anfang) { 
      Val tmp = anfang;
      anfang = ende;
      ende = tmp;
    }  // if
    for (int p = anfang.asInt(); p <= ende.asInt(); p++) {
      set.add(p);
    }
  }  // if
  else {
    set.add(StringToVal(range));
  }
}  // HLPlace::StringRangeAppendSet

void
HLPlace::StringToSet(String s, SET<Val> &set) const
  // Konvertiert String nach Set
{
  set.clear();
  s = s.gsub("{", "");
  s = s.gsub("}", "");
  String sv = s.before(",");
  while (sv != "") {
    StringRangeAppendSet(sv, set);
    s = s.after(",");
    sv = s.before(",");
  }  // while
  StringRangeAppendSet(s, set);
}  // HLPlace::StringToSet

void 
HLPlace::StringToType(String s)
  // Konvertiert einen String s in den Typ der Stelle (this->type)
{
  type.clear();
  net->ChangeTyp(name.data(), s.data());
  String sv = s.before("X");
  if (sv != "") {
    type_string = s;
    VHSet<Val> s1(20), s2(20), temp(20);
    StringToSet(sv, s1);
    /*String av = s.after("X");
    StringToSet(av, s2);
    for (Pix i = s1.first(); i; s1.next(i)) {
      for (Pix j = s2.first(); j; s2.next(j)) {
        type.add(Tupel(s1(i), s2(j)));
      }
    }*/
    String av = s.after("X");
    for(Pix i = s1.first(); i; s1.next(i)) {
      String te = s;
      int help = 0;
      while(te != "") {
        te = te.after("X");
        help++;
      }
      Tupel temp(help);
      temp.add(s1(i));
      add(temp,av);
    }
  }  // if
  else {
    if (s.contains("{")) 
      type_string = s;
    else
      type_string = "{" + s + "}";
    StringToSet(s, type);
  }  // else
}  // HLPlace::StringToType


void
HLPlace::add(Tupel va, String s)
 // rekursive Methode, um alle Elemente der n-dimensionalen Menge zu bilden
{
  Tupel temp = va;
  String sv = s.before("X");
  VHSet<Val> s1(20);
  if(sv != "") {
    StringToSet(sv, s1);
    String av = s.after("X");
    for(Pix i = s1.first(); i; s1.next(i)) {
      Tupel temp2(temp);
      temp2.add(s1(i));
      add(temp2,av);
    }
  }
  else {
    StringToSet(s, s1);
    for(Pix j = s1.first(); j; s1.next(j)) {
      Tupel temp2(temp);
      temp2.add(s1(j));
      type.add(temp2);
    } // for j "uber alle Elemente der letzten Menge
  }  // else
}  // HLPlace::add
             
String
SetToString(const SET<Val> &val_set)
  // Konvertiert eine Menge ber Val in eine String	
{
  String l;
  int start = 0, end = 0, c_flag = 0;
  Pix n = val_set.first();

  for (Pix i = val_set.first(); i; val_set.next(i)) {
    val_set.next(n);
    if (val_set(i).Type() == INTEGER) {
        // Werden int's schon gezhlt?
      if (!c_flag) {  // nein
        end = start = val_set(i).asInt();
        c_flag = 1;
      }  // if !c_flag
        // Ist Integer!
      else {
          // Und ist der vorige Wert einen kleiner?
        if (val_set(i).asInt() -1 == end)
          end++;
        else {  // falls nicht -> Zhlen abbrechen
          AppendRange(start, end, l);
          c_flag = 0;
          l += ",";
          l += val_set(i).asString();
          if (n) 
            l += ",";
        }  // else
      }  // else
    }  // if INTEGER
    else {  // Kein Integer
      if (c_flag) {
        AppendRange(start, end, l);
        l += ",";
        c_flag = 0;
      }  // if c_flag
      l += val_set(i).asString();
      if (n) 
        l += ",";
    }  // else
  }  // for i
  if (c_flag)
    AppendRange(start, end, l);
  return l;
}  // SetToString

void
HLPlace::asTiePlace()
{
  int x_sum(0), y_sum(0);
  
  VHMap<String, HLTransition *> every_trans((HLTransition *) NULL);
  HLTransition * trans = (HLTransition *) NULL;
  
  for (Pix i= incoming_edges.first(); i ; incoming_edges.next(i))
  {
    trans = incoming_edges.contents(i)->Src().asHLTransition();
	every_trans[trans->Name()] = trans;
	x_sum += trans -> GetX();
	y_sum += trans -> GetY();
  }; //for (Pix i= incoming_edges.first(); i ; incoming_edges.next(i))

  for (Pix i= outgoing_edges.first(); i ; outgoing_edges.next(i))
  {
    trans = outgoing_edges.contents(i)->Dest().asHLTransition();
    String t_name = trans -> Name();
    // please don't replace by (!IsInput(trans -> Name()))
    if (!every_trans.contains(t_name))
		{
          every_trans[t_name] = trans;
          x_sum += trans -> GetX();
          y_sum += trans -> GetY();
		};
  }; //for (Pix i= outgoing_edges.first(); i ; outgoing_edges.next(i))
  
  // set the new position of this place
  if (every_trans.length() > 1 )
       {
         px = x_sum / (every_trans.length());
         py = y_sum / (every_trans.length());
       }
  else {
         px = x_sum + 80;
         py = y_sum;
       }  
  boolean same(true);

  // if every transitions are in a same block, add this place to this block
  VHSet<String> block_names;
  VHMap<String, Block*> block_ptrs ((HLBlock*)NULL);
  String b_name;

  trans = every_trans.contents(every_trans.first());
  block_names = trans -> BlockKeySet();
  block_ptrs = trans -> Blocks();
  for (Pix i=every_trans.first();i;every_trans.next(i))
  {
    trans = every_trans.contents(i);
    if (block_names != trans -> BlockKeySet())
        {
          same = false;
          break;
        };
  }; //  for (Pix i=every_trans.first();i;every_trans.next(i))

  if(same)
    for (Pix i=block_names.first();i;block_names.next(i))
    {
      b_name = block_names(i);
      AddBlock(b_name,block_ptrs[b_name]);
    }; // for (Pix i=block_names.first();i;block_names.next(i))

  // if every transitions are in a same box, add this HLPlace to this box
  
  trans = every_trans.contents(every_trans.first());
  b_name = trans -> BoxName();
  
  if (b_name !="")
  {
    same = true;
  
    for (Pix i=every_trans.first();i;every_trans.next(i))
    {
      trans = every_trans.contents(i);
      if (b_name != trans -> BoxName())
          {
            same = false;
            break;
          };
    }; //  for (Pix i=every_trans.first();i;every_trans.next(i))
  
    if (same)
    {
      HLBox * box_ptr = (HLBox *) NULL;
	  if (trans -> BoxPointer())
            box_ptr = new HLBox( *(trans -> BoxPointer() -> asHLBox()) );
      // else, there is something strange... since the name is not empty
      setBox(b_name, box_ptr);
    };
  } // end if (b_name !="")

} // HLPlace::asTiePlace()

int
HLPlace::ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[])
  // Informationen fr Tcl-Teil bereitstellen
{
  if (argc != 1) {
    Tcl_AppendResult(ip, "wrong #args for Place get",
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl fr Place get

  else if (EQ(0, "Marking"))
    Result(ip, MarkingToString(current_marking));

  else if (EQ(0, "InitMarking"))
    Result(ip, MarkingToString(initial_marking));

  else if (EQ(0, "Type"))
    Result(ip, type_string);

  else if (EQ(0, "TypeX"))
    INTRES(rx_type);

  else if (EQ(0, "TypeY"))
    INTRES(ry_type);

  else
    return Place::ObjectGet(ip, argc, argv);
  return TCL_OK;
}  // HLPlace::ObjectGet

int
HLPlace::ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[])
  // Informationen aus Tcl-Teil bernehmen
{
  if (argc < 2) {
    Tcl_AppendResult(ip, "wrong #args for Place set",
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl fr Place set
  char *cm = NULL;
  char *im = NULL;
  char *dt = NULL;
  char *t  = NULL;
  Tk_ArgvInfo parse_table[] = {
    {"Marking", TK_ARGV_STRING, (char *) NULL, (char *) &cm,
     "New Marking"},
    {"InitMarking", TK_ARGV_STRING, (char *) NULL, (char *) &im,
     "Initial Marking"},
    {"DelToken", TK_ARGV_STRING, (char *) NULL, (char *) &dt,
     "Token to delete"},
    {"Type", TK_ARGV_STRING, (char *) NULL, (char *) &t,
     "Type of Place"},
    {"TypeX", TK_ARGV_INT, NULL, (char *) &rx_type,
     "Relativ x - position of type"},
    {"TypeY", TK_ARGV_INT, NULL, (char *) &ry_type,
     "Relativ y - position of type"},
    {NULL, TK_ARGV_END, NULL, NULL, NULL}};
  if (Tk_ParseArgv(ip, NULL, &argc, argv, parse_table, 
    TK_ARGV_DONT_SKIP_FIRST_ARG) != TCL_OK)
  return TCL_ERROR;
  if (cm) {
    StringToMultiSet(cm, current_marking);
    net->ChangeMarking(name.data(), cm);
    tokens = current_marking.elements();
  }  // if cm
  if (im) {
    StringToMultiSet(im, initial_marking);
    initial_tokens = initial_marking.elements();
  }  // if im
  if (dt)
    RemoveToken(StringToVal(dt));
  if (t) {
    String test(t);
    if (test.contains("$")) {
      type_string = test;
      abstract = true;
    } else {
      StringToType(t);
      abstract = false;
    }
  }
  if (argc > 0)
    return Place::ObjectSet(ip, argc, argv);
  return TCL_OK;
}  // HLPlace::ObjectSet

  //*******************************************************
  // Hilfsfunktionen fr die Datenkonvertierung C++ -> Tcl
  //*******************************************************

  // Multimenge in Tcl Liste verwandlen
void
MultiSetToTclList(Tcl_Interp *ip, const MultiSET<Val> &v)
{
  for (Pix i = v.first(); i; v.next(i)) {
    if (v.nof(i) > 1)
      Tcl_AppendElement(ip, IntToString(v.nof(i)) + "*" + 
        v.element(i).asString());
    else
      Tcl_AppendElement(ip, v.element(i).asString());
  }  // for i
}  // MultiSetToTclList

void
SetToTclList(Tcl_Interp *ip, const SET<Val> &v)
{
  Tcl_SetResult(ip, SetToString(v), TCL_VOLATILE);
/*
  for (Pix i = v.first(); i; v.next(i)) {
    Tcl_AppendElement(ip, v(i).asString());
  }  // for i
*/
}  // SetToTclList

  //*** Dateiende HLPlace.cc ***

