// HLPlace.h
// Tobias Himstedt 1995
// Burkhard Bieber 1995
// Cecile Bui Thanh: added some functions for the Tie operator (2001)
// last-update: Dec. 2001

#ifndef __HLPLACE_H__
#define __HLPLACE_H__

  //**********
  // includes
  //**********

#include "../include/MultiSet.h"
#include "Val.h"
#include "../Low/Place.h"
#include "../include/VHSet.h"

  //*********
  // Klassen
  //*********

class HLPlace : public Place {
  protected:
    VHSet<Val> type;
    String type_string;
    boolean abstract;
    MultiSET<Val> current_marking;
    MultiSET<Val> initial_marking;
    int rx_type, ry_type;
    virtual void Write(ostream &f) const;
    virtual void WriteMarking(ostream &f) const;
    virtual void ReadOptional(istream &f);
    String MarkingToString(const MultiSET<Val> &s) const;
    void StringToSet(String s, SET<Val> &v) const;
    void StringRangeAppendSet(String range, SET<Val> &b) const;
    void StringToMultiSet(String s, MultiSET<Val> &v) const;
    void StringRangeAppendMultiSet(String range, MultiSET<Val> &b) const;
    void StringToType(String s);
      // void StringToTupelSet(String s, SET<Tupel>& set) const;
  public:
    virtual void AddTokens(int k = 1);
    virtual HLPlace *asHLPlace() const;
      // calculate of the place as barycenter of the transition to which
      // the place is linked
      // if all transitions are in the same(s) block(s) then this place
      // is added to these blocks. Idem with transistions' box.
	void asTiePlace(void);
      // const Set<Val>& Type() const;
    VHSet<Val> Type() const;
    void SetTyp(VHSet<Val> &Typ, String &t);
    void SetTypString(String typ) {type_string = typ;};
    String GetTypString(void) {return type_string;};
    const MultiSET<Val> &Marking() const;
    const MultiSET<Val> &InitMarking() const;
    void AddToken(const Val &new_val);
    void RemoveToken(const Val &v);
    void DeleteInitMarking(void);
    void SubtractMarking(const MultiSET<Val> &v);
    void AddMarking(const MultiSET<Val> &v);
    void add(Tupel va, String s);
    VHSet<String> GetMacros(void);
    void InsertMacro(String macro, String ersetzung);
    void Instantiate(void);
    virtual void DeleteAllTokens();
    virtual void SetEntry(int read_flag = 0);
    virtual void SetExit();
    const HLPlace &operator =(const HLPlace &p);
    HLPlace(const Net &n,const String &Type = "HLPlace");
    HLPlace(const HLPlace &p);
    virtual int ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[]);
    virtual int ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[]); 
};  // class HLPlace

  //************
  // Prototypen
  //************

String SetToString(const SET<Val> &val_set);
void MultiSetToTclList(Tcl_Interp *ip, const MultiSET<Val> &v);
void SetToTclList(Tcl_Interp *ip, const SET<Val> &v);

  //*********************
  // Inlines fr HLPlace
  //*********************

inline
HLPlace::HLPlace(const HLPlace &p) : Place(p),
  type(p.type), abstract(false),
  current_marking(p.current_marking), initial_marking(p.initial_marking),
  rx_type(p.rx_type), ry_type(p.ry_type)
{
  // Nix
}  // HLPlace::HLPlace

inline const HLPlace &
HLPlace::operator =(const HLPlace &p)
{
  if (p.Id() == Id())
    return *this;
  Place::operator=(p);
  type = p.type;
  type_string = p.type_string;
  abstract = p.abstract;
  current_marking = p.current_marking;
  initial_marking = p.initial_marking;
  rx_type = p.rx_type;
  ry_type = p.ry_type;
  return *this;
}

inline void
HLPlace::SubtractMarking(const MultiSET<Val> &v)
{
  current_marking -= v;
  tokens = current_marking.elements();
}  // HLPlace::SubtractMarking

inline void
HLPlace::AddMarking(const MultiSET<Val> &v)
{
  current_marking += v;
  tokens = current_marking.elements();
}  // HLPlace::AddMarking

inline const MultiSET<Val> &
HLPlace::Marking() const
{
  return current_marking;
}  // HLPlace::Marking

inline const MultiSET<Val> &
HLPlace::InitMarking() const
{
  return initial_marking;
}  // HLPlace::InitMarking

inline HLPlace *
HLPlace::asHLPlace() const
{
  return (HLPlace *) this;
}  // HLPlace::asHLPlace

inline VHSet<Val>
HLPlace::Type() const
{
  return type;
}  // HLPlace::Type

inline void
HLPlace::AddTokens(int k)
{
  Place::AddTokens(k);
  current_marking.add(type(type.first()), k);
}  // HLPlace::AddTokens

#endif  // #ifndef __HLPLACE_H__

  //*** Dateiende HLPlace.h ***
    
