// HLTransition.cc
// Tobias Himstedt 1995
// Burkhard Bieber 1995/96
// Christian Stehno: Added wait.h (2000)
// Cecile Bui Thanh: added asynchronous links (2001)
// last-update: Sept. 2002

  //**********
  // includes
  //**********

#include "HLTransition.h"
#include "HLNet.h"
#include <signal.h>
#include <sys/wait.h>

  //**********
  // Templates
  //**********

//template class VHMap<String,int>;
//template class MultiSET<String>;

  //***************************
  // Methoden fr HLTransition
  //***************************

HLTransition::HLTransition(const Net &n, const String &Type) :
  Transition(n, Type), action_terms(""), Value("tt"),
  links(""), value_term("tt"), abstract(false),
  Link_import(), Link_export(),
  Cache1(0), Cache2("")
  // Konstruktor
{
  base_classes.add("Transition");
  net->asHLNet()->DefaultHLTransAT(rx_at, ry_at);
  net->asHLNet()->DefaultHLTransVT(rx_vt, ry_vt);
  net->asHLNet()->DefaultHLTransLT(rx_links, ry_links);
  net->asHLNet()->DefaultHLTransMean(rx_mean, ry_mean);
}  // HLTransition::HLTransition

HLTransition *
HLTransition::asHLTransition() const
  // Destruktor
{
  return (HLTransition *) this;
}  // HLTransition::asHLTransition


/* CheckSimpleLink *************************************************
* String checked_name   : the name of the link checked
* int export_or_import  : say if it's an import(2) or an export(1)
* exportset             : set where to add the new link if it'an export
* importset             : set where to add the new link if it'an import
*
* This function check if checked_name(args) matches with any declared link
* and if yes add it to the right (multi)set of links.
*
* The returned value is true if the link has been added, false
* otherwise
********************************************************************/
boolean
HLTransition::CheckSimpleLink( String checked_name,
                               MultiSET<LinkTerm> &exportset,
                               MultiSET<LinkTerm> &importset,
                               int export_or_import)
{
  if (!net->asHLNet()->getDeclaredLinks().contains(checked_name))
    { return false;};
  Link p = net->asHLNet()->getDeclaredLinks()[checked_name];
			
  // if checked_name is already declared in the net,
  //  check if it matches the type
  if (p.get_arity()==0)
	{ // add this new LinkTerm in the right set
	  LinkTerm *lterm = new LinkTerm(checked_name,"");
	  switch (export_or_import)
	  {
		case 1:	// link exportation
			exportset.add (*lterm); break;
		case 2:	// link importation
			importset.add (*lterm); break;
		default: // impossible case
			break;
	  }//switch (export_or_import)
	  return true;
	}
  else { return false;}
}

/* CheckLinkArgs *************************************************
* String checked_name   : the name of the link checked
* String &args          : the String to analyse
* int export_or_import  : say if it's an import(2) or an export(1)
* boolean &correct      : say if args is or not correct
* exportset             : set where to add the new link if it'an export
* importset             : set where to add the new link if it'an import
*
* This function check if checked_name(args) matches with any declared link
* and if yes add it to the right (multi)set of links.
*
* The returned value is true if the link has been added, false
* otherwise
******************************************************************/
boolean
HLTransition::CheckLinkArgs(String checked_name,
                            String &args,
                            int export_or_import,
                            boolean &correct,
                            MultiSET<LinkTerm> &exportset,
                            MultiSET<LinkTerm> &importset)
{
  // we suppose here that args is not ""
  // and doesn't contain white spaces
  
  int nb_args = 0;	// number of arguments
  String analysed_arg;
  String work = args;
  
  // count the number of arguments
  // Note: this doesn't work if the tupel contains tupels...
  // for instance: "5,(2,3),4,3"
  while (work.contains(","))
	 {
	   analysed_arg = work.before(",");
	   work = work.after(",");
	   if (analysed_arg =="")
		  {
			correct = false;
			return false;
		  }
	   else
		  nb_args++;
	 }; // while (work.contains(","))

  if (work=="")
	{
	   correct = false;
	   return false;
	}
  else if (net->asHLNet()->getDeclaredLinks().contains(checked_name))
	{
	  nb_args++;
	  Link p = net->asHLNet()->getDeclaredLinks()[checked_name];
	  if ( p.get_arity() != nb_args)
          return false;

	  else
		 { // add this new LinkTerm in the right set
		   LinkTerm *lterm = new LinkTerm(checked_name,args);
		   
		   switch (export_or_import)
		   {
			 case 1:	// link exportation
				exportset.add (*lterm); break;
			 case 2:	// link importation
				importset.add (*lterm); break;
			 default: // impossible case
				break;
		   }//switch (export_or_import)

		   return true;
		 }
	};

  return false;

}

VHSet<String>
HLTransition::ParseLinks(String &new_links)
{
  VHSet<String> undeclared(5);
  //delete every white spaces in the new link label
  new_links = new_links.gsub(" ","");
  new_links = new_links.gsub("\t","");
  new_links = new_links.gsub("\n","");

  if (new_links == "")
    {
      links = "";
      // clear actual multisets of Link export/import
      Link_import.clear();
      Link_export.clear();
    }
  else	// Parse the new label and, if it is correct, update the 
    {	// MultiSETs Link_import and Link_export 
      MultiSET<LinkTerm> tmp_export = *(new MultiSET<LinkTerm>());
      MultiSET<LinkTerm> tmp_import = *(new MultiSET<LinkTerm>());
      String *tmp_name = new String();
      String new_link_label = "";
	  String mark = "";

      boolean correct = true;	// is the label correct
      char t;
      unsigned int i=0;
      int export_or_import;	// 0: not initialized
                              // 1: export
                            // 2: import
      
      while (correct && new_links != "")
      {
        //check link name
        *tmp_name = "";
        i=0;
        export_or_import = 0;
        
        while (*tmp_name=="" && new_links != "" )
          {
            t = new_links[i];
            if (t == '+')
                {
                  export_or_import = 1;
                  mark = "+";
                  *tmp_name = new_links.before("+");
                  new_links = new_links.after("+");
                }
            else if (t=='-')
                {
                  export_or_import = 2;
                  mark = "-";
                  *tmp_name = new_links.before("-");
                  new_links = new_links.after("-");
                }
            else if( !(    ((t <= int('9')) && (t >= int('0')))
                        || ((t <= int('Z')) && (t >= int('A')))
                        || ((t <= int('z')) && (t >= int('a')))
                        || (t == int('_'))))
                {
                    correct = false; break;
                }
            i++;
           };// end while (*tmp_name=="" && new_links != "" )
        
        // if export_or_import has not been changed, then no '+'
        // or '-' were found
        if (export_or_import == 0)
            correct = false;
        
        if (!correct) // exit while (correct && new_links != "")
            break;

        if (new_links == "" ) // nothing left to be analysed
          { 
            // add the link term if it matches with a declared link
            if (!CheckSimpleLink(*tmp_name,tmp_export,
                                 tmp_import,export_or_import))
                  undeclared.add(*tmp_name);  // don't change the previous label
              else new_link_label += ","+*tmp_name + mark;
          }
        else
          {
            t = new_links[0];
            if (t == ',') // i.e. if there are no args
                {
                  new_links = new_links.after(",");
                  // keep the link only if it matches with a declared link
                  if (!CheckSimpleLink(*tmp_name, tmp_export,
                                       tmp_import, export_or_import))
                        undeclared.add(*tmp_name);
                  else new_link_label += "," + *tmp_name + mark;
                }
            else if (t != '(' || ! (new_links.contains(")")))
                {
                  correct = false;
                }
            else  //t == '(' and new_links.contains(")")
                { // same remark as for CheckLinkArgs:
                  // it doesn't work for tupels of tupels
                  String args = new_links.before (")");
                  new_links = new_links.after(")");
                  args = args.after ("(");
                  
                  if (args == "")
                     {
                       if (!CheckSimpleLink(*tmp_name, tmp_export,
                                       tmp_import, export_or_import))
                            undeclared.add(*tmp_name);
                       else new_link_label += "," + *tmp_name + mark + "()";
                     }
                  else
                     {
                       if (!CheckLinkArgs(*tmp_name, args, export_or_import,
                                     correct, tmp_export,tmp_import))
                        {
                          if (correct)
                              undeclared.add(*tmp_name+"("+args+")");
                        }
                       else new_link_label += "," + *tmp_name+mark+"("+args+")";
                     }; // end if (args == "")
                  
                  if (new_links == "") break;

                  if (new_links.First()!=',')
					  correct = false;
				  else
					{
				       new_links = new_links.after(",");
					   if (new_links == "") correct = false;
					};
                }
          }; //if (new_links == "")
        
      }// end while (correct && new_links != "")
      
      if (!correct)
          undeclared.clear();

      else
          {
            //update import, export
            Link_import.clear();
            Link_export.clear();
            
            Link_import += tmp_import;
            Link_export += tmp_export;
			links = new_link_label.after(",");  // the first is a comma
          }// end if(correct)

    }; // end if new_links == ""

  return (undeclared);
}// End of ParseLinks


// deleteLinkTermsInLabel : delete every linkterms in the link label
// which have l_name as name
void
HLTransition::deleteLinkTermsInLabel(String l_name)
{
  VHSet<String> every_terms = net -> asHLNet() -> ParseLinkString(links);
  VHSet<String> new_terms;
  String new_label = "";
  String term = "";
  String term_name = "";

  for (Pix i=every_terms.first();i;every_terms.next(i))
   {
     term = every_terms(i);
     if (term.contains("+"))
       {
         term_name = term.before("+");
         if (term_name != l_name) new_terms.add(term);         
       }
     else
       {
         term_name = term.before("-");
         if (term_name != l_name) new_terms.add(term);         
       };
   } // end for all terms in every_terms

  if (new_terms.length()!=0)
   {
     for (Pix i=new_terms.first();i;new_terms.next(i))
       {
         term = new_terms(i);
         new_label += "," + term;
       };
     new_label = new_label.after(",");
   };
  links = new_label;
}

void
HLTransition::deleteLinkTerms(String l_name)
{
  if (links.contains(l_name.data())) // else nothing to do
  {
    // delete in the import/export multisets
    Pix i;
    LinkTerm l;
    for (i=Link_export.first();i;Link_export.next(i))
    {
      l = Link_export.element(i);
      if (l.get_name() == l_name)
          Link_export.del(l, Link_export.nof(i));
    }; //for each link in Link_export
  
    for (i=Link_import.first();i;Link_import.next(i))
    {
      l = Link_import.element(i);
      if (l.get_name() == l_name)
          Link_import.del(l, Link_import.nof(i));
    }; //for each link in Link_import
  
    deleteLinkTermsInLabel(l_name);

  }
} // end of HLTransition::deleteLinkTerms(String l_name)



void
HLTransition::deleteLinkTerms(VHSet<String> l_names)
{
  for (Pix i= l_names.first();i; l_names.next(i))
      deleteLinkTerms(l_names(i));
} // end of HLTransition::deleteLinkTerms(VHSet<String> l_names)

void
HLTransition::CacheFlush(void)
  // L"osche den Cache
{
  Cache1.clear();
  Cache2.clear();
}  // HLTransition::CacheFlush

int
HLTransition::InCache(String Marking)
  // Existiert eine Markierung?
{
  return Cache1.contains(Marking);
}  // HLTransition::InCache

void
HLTransition::AddMarking(String Marking, int erg)
  // Trage einen neuen Eintrag in den Cache ein
{
  Cache1[Marking] = erg;
}  // HLTransition::AddMarking

void
HLTransition::AddMarking(String Marking, String erg)
  // Trage einen neuen Eintrag in den Cache ein
{
  Cache2[Marking] = erg;
}  // HLTransition::AddMarking

int
HLTransition::ReturnCache1(String Marking)
  // Liefere das Ergebnis der Markierung zur"uck
{
  if(InCache(Marking))
    return Cache1[Marking];
  return 0;
}  // HLTransition::ReturnCache

String
HLTransition::ReturnCache2(String Marking)
  // Liefere das Ergebnis der Markierung zur"uck
{
  if(InCache(Marking))
    return Cache2[Marking];
  return "";
}  // HLTransition::ReturnCache

VHSet<String>
HLTransition::GetMacros(void)
  // Macros herausfiltern
{
  VHSet<String> macros;
  if(!Value.contains("$") && !action_terms.contains("$"))
    return macros;
  String davor, dahinter;
  davor = Value.after("$");
  dahinter = davor.before("$");
  while(davor != "") {
    macros.add(dahinter);
    davor = davor.after("$");
    davor = davor.after("$");
    dahinter = davor.before("$");
  }
  davor = action_terms.after("$");
  dahinter = davor.before("$");
  while(davor != "") {
    macros.add(dahinter);
    davor = davor.after("$");
    davor = davor.after("$");
    dahinter = davor.before("$");
  }
  davor = reference.after("$");
  dahinter = davor.before("$");
  while(davor != "") {
    macros.add(dahinter);
    davor = davor.after("$");
    davor = davor.after("$");
    dahinter = davor.before("$");
  }
  return macros;
}

void
HLTransition::InsertMacro(String macro, String ersetzung)
  // Macros ersetzen
{
  String davor, dahinter;
  macro = "$" + macro + "$";
  while(Value.contains(macro)) {
    davor = Value.before(macro);
    dahinter = Value.after(macro);
    Value = davor + ersetzung + dahinter;
  }
  while(action_terms.contains(macro)) {
    davor = action_terms.before(macro);
    dahinter = action_terms.after(macro);
    action_terms = davor + ersetzung + dahinter;
  }
  while(reference.contains(macro)) {
    davor = reference.before(macro);
    dahinter = reference.after(macro);
    reference = davor + ersetzung + dahinter;
  }
}

void
HLTransition::Instantiate()
  // Valueterm "ubernehmen
{
  ChangeValueTerm(Value);
  abstract = false;
}  // HLTransition::Instantiate

void
HLTransition::ChangeForeach()
  // Foreach im Valueterm aufl"osen
{
  int j;
  String vor, nach, danach, davor, ersetzung, rest;
  String set, Tupel, neu;
  while(Value.contains("$FOREACH$")) {
    danach = Value.after("$FOREACH$");
    davor  = Value.before("$FOREACH$");
    ersetzung = danach.before(")");
    rest = danach.after(")");
    ersetzung = ersetzung + ")";
    set = ersetzung.before("}");
    set = set + "}";
    VHSet<Val> menge;
    StringToSet(set, menge);
    Tupel = ersetzung.after(":");
    neu = "";
    j = 0;
    for(Pix i = menge.first(); i; menge.next(i), j++) {
      String newTupel = Tupel;
      while(newTupel.contains("$ELEMENT$")) {
        vor =  newTupel.before("$ELEMENT$");
        nach = newTupel.after("$ELEMENT$");
        newTupel = vor + menge(i).asString() + nach;
      }
      while(newTupel.contains("$INDEX$")) {
        vor =  newTupel.before("$INDEX$");
        nach = newTupel.after("$INDEX$");
        newTupel = vor + IntToString(j) + nach;
      }
      if(neu != "")
        neu = neu + " & " + newTupel;
      else
        neu = newTupel;
    }
    Value = davor + neu + rest;
  }
}

void
HLTransition::ChangeValueTerm(const String &vt)
{
  String v = vt;
  CacheFlush();
  
  if(!value_term.ChangeTerm(vt)) {
    net->ChangeValueTerm(name.data(), v.data());
    value_term.ChangeTerm(vt);
  }
}  // HLTransition::ChangeValueTerm

void
HLTransition::StringToSet(String s, SET<Val> &set) const
  // Konvertiert String nach Set
{
  set.clear();
  s = s.gsub("{", "");
  s = s.gsub("}", "");
  String sv = s.before(",");
  while (sv != "") {
    StringRangeAppendSet(sv, set);
    s = s.after(",");
    sv = s.before(",");
  }  // while
  StringRangeAppendSet(s, set);
}  // HLTransition::StringToSet

String
HLTransition::SetToString(VHSet<String> set) const
{
  String result = "";
  if ( !set.empty())
    {
      for (Pix i=set.first();i;set.next(i))
          result += "," + set(i);
      result = result.after(","); // delete the first char which is a comma
    }
  return result;
}  // HLTransition::SetToString


void
HLTransition::StringRangeAppendSet(String range, SET<Val> &set) const
  // Fgt an die Menge set einen Wert an, der aus dem String range
  // geewonnen wird. range kann aber auch eine Bereich start..ende sein
  // In diesem Fall werden alle Werte von start bis end zur Menge
  // zugefgt.
{
  String lower = range.before("..");
  if (lower != "") {
    Val anfang = StringToVal(lower);
    String upper = range.after("..");
    Val ende = StringToVal(upper);
    if (anfang.Type() != INTEGER || ende.Type() != INTEGER) {
      Error("Lower and upper bounds must be Integers");
      return ;
    }  // if
    if (ende < anfang) { 
      Val tmp = anfang;
      anfang = ende;
      ende = tmp;
    }  // if
    for (int p = anfang.asInt(); p <= ende.asInt(); p++) {
      set.add(p);
    }
  }  // if
  else {
    set.add(StringToVal(range));
  }
}  // HLTransition::StringRangeAppendSet


void
HLTransition::Write(ostream &f) const
  // Ausgabe der HL Werte zur Transition
{
  Transition::Write(f);
  if(abstract) {
    f << "g\"" << Value << "\"";
  } else {
    f << "g\"" << value_term.Text() << "\"";
  }
  if (rx_vt != net->asHLNet()->DefaultHLTransVTX() || 
    ry_vt != net->asHLNet()->DefaultHLTransVTY())
    f << 'h' << rx_vt << '@' << ry_vt;
  if (action_terms != "")
    f << "i\"" << action_terms << "\"";
  if (rx_at != net->asHLNet()->DefaultHLTransATX() || 
    ry_at != net->asHLNet()->DefaultHLTransATY())
    f << 'j' << rx_at << '@' << ry_at;
  if (links != "")
    f << "l\"" << links << "\"";
  if (rx_links != net->asHLNet()->DefaultHLTransLTX() ||
       ry_links != net->asHLNet()->DefaultHLTransLTY())
    f << 'k' << rx_links << '@' << ry_links;
}  // HLTransition::Write

void
HLTransition::ReadOptional(istream &f)
  // optionale Werte einlesen
{
  Transition::ReadOptional(f);
  char c;
  f.get(c);
  String g;
  String L;

  switch (c) {
    case 'g':
      f >> c;
      readline(f, g, '"');
      if (g.contains("$")) {
        Value = g;
        abstract = true;
      } else {
        Value = g;
        value_term.ChangeTerm(g);
        abstract = false;
      }
      break;
    case 'h':
      f >> rx_vt >> c >> ry_vt;
      break;
    case 'i':
      f >> c;
      readline(f, action_terms, '"');
      break;
    case 'j':
      f >> rx_at >> c >> ry_at;
      break;
    case 'l':
      f >> c;
      readline(f, L, '"');
      ParseLinks(L);
      break;
    case 'k':
      f >> rx_links >> c >> ry_links;
      break;
    default:
      f.putback(c);
  }  // switch ber c
}  // HLTransition::ReadOptional

int
HLTransition::ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[])
  // TCL-Schnittstelle
{
  if (argc != 1) {
    Tcl_AppendResult(ip, "wrong #args for Transition get",
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl fr Transition get

  if (EQ(0, "AT"))
    Result(ip, action_terms);

  else if (EQ(0, "VT")) {
    if (abstract == true)
      Result(ip, Value);
    else
      Result(ip, value_term.Text());
  }
  else if (EQ(0, "LT"))
    Result(ip, links);

  else if (EQ(0, "ATX"))
    INTRES(rx_at);

  else if (EQ(0, "ATY"))
    INTRES(ry_at);

  else if (EQ(0, "VTX"))
    INTRES(rx_vt);

  else if (EQ(0, "VTY"))
    INTRES(ry_vt);

  else if (EQ(0, "LTX"))
    INTRES(rx_links);

  else if (EQ(0, "LTY"))
    INTRES(ry_links);

  else
    return Transition::ObjectGet(ip, argc, argv);
  return TCL_OK;
}  // HLTransition::ObjectGet

int
HLTransition::ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[])
  // TCL-Schnittstelle
{
  if (argc < 2) {
    Tcl_AppendResult(ip, "wrong #args for Transition set",
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl fr Transition set
  char *at = NULL;
  char *vt = NULL;
  char *lt = NULL;	// Link terms
  Tk_ArgvInfo parse_table[] = {
    {"AT", TK_ARGV_STRING, NULL, (char *) &at,
     "Actionterms of Transition"},
    {"VT", TK_ARGV_STRING, NULL, (char *) &vt,
     "Valueterm of Transition"},
    {"LT", TK_ARGV_STRING, NULL, (char *) &lt,
     "Link terms of Transition"},
    {"ATX", TK_ARGV_INT, NULL, (char *) &rx_at,
     "Relativ x - position of Actionterms"},
    {"ATY", TK_ARGV_INT, NULL, (char *) &ry_at,
     "Relativ y - position of Actionterms"},
    {"VTX", TK_ARGV_INT, NULL, (char *) &rx_vt,
     "Relativ x - position of Valueterm"},
    {"VTY", TK_ARGV_INT, NULL, (char *) &ry_vt,
     "Relativ y - position of Valueterm"},
    {"LTX", TK_ARGV_INT, NULL, (char *) &rx_links,
     "Relativ x - position of Link terms"},
    {"LTY", TK_ARGV_INT, NULL, (char *) &ry_links,
     "Relativ y - position of Link terms"},
    {NULL, TK_ARGV_END, NULL, NULL, NULL}};
  if (Tk_ParseArgv(ip, NULL, &argc, argv, parse_table, 
    TK_ARGV_DONT_SKIP_FIRST_ARG | TK_ARGV_NO_ABBREV) != TCL_OK)
    return TCL_ERROR;
  if (vt) {
    String v(vt);
    if(v.contains("$")) {
      abstract = true;
      Value = v;
    } else {
      abstract = false;
      ChangeValueTerm(vt);
    }
  }
  if (at)
    action_terms = at;
  if (lt)
    {
      String li(lt);
      VHSet<String> undeclared = ParseLinks(li);
      if (!undeclared.empty())
        {
          String undecl_string =  SetToString(undeclared);
          Tcl_AppendResult(ip,
          "These links are not declared or don't match their type:", undecl_string.data(),
              NULL);
          return TCL_ERROR;
        }
    };
  if (argc > 0)
    return Transition::ObjectSet(ip, argc, argv);
  return TCL_OK;
}  // HLTransition::ObjectSet




  //*** Dateiende HLTransition.cc ***

