// HLTransition.h
// Tobias Himstedt 1995
// Burkhard Bieber 1995
// Cecile Bui Thanh: added asynchronous links (2001)
// last-update: Nov. 2001

#ifndef __HLTRANSITION_H__
#define __HLTRANSITION_H__

  //**********
  // includes
  //**********

#include "Val.h"
#include "../Low/Transition.h"
#include "../include/MultiSet.h"
#include "../Low/Link.h"

  //*********
  // Klassen
  //*********

class HLTransition : public Transition {
  private:
	boolean CheckSimpleLink( String name,
                             MultiSET<LinkTerm> &exportset,
                             MultiSET<LinkTerm> &importset,
                             int export_or_import);
	boolean CheckLinkArgs( String tmp_name,
                           String &args,
                           int export_or_import,
                           boolean &correct,
                           MultiSET<LinkTerm> &exportset,
                           MultiSET<LinkTerm> &importset);
    void deleteLinkTermsInLabel(String l_name);
  protected:
    String action_terms;
    String Value;
	String links;	// link label (link importations and exportations)
    VT value_term;
    boolean abstract;
	  // link importations, link exportations
	MultiSET<LinkTerm> Link_import, Link_export;
    VHMap<String, int> Cache1;
    VHMap<String, String> Cache2;
    int rx_at, ry_at;
    int rx_vt, ry_vt;
	int rx_links, ry_links; // relative positions of links label
    virtual void ReadOptional(istream &f);
    virtual void Write(ostream &f) const;
  public:
    /* ---------- Asynchronous Links ---------- */
	MultiSET<LinkTerm> getImport() const;
	MultiSET<LinkTerm> getExport() const;

	  // (re)build Link_import and Link_export if the new label
	  // is correct, otherwise no changes are done
	  // The returned set is a set of undeclared links, which have
	  // not been added in Link_import/export
	VHSet<String> ParseLinks(String &new_links);
      // remove every LinkTerms with l_name as name
    void deleteLinkTerms(String l_name);
      // remove every LinkTerms with a name in l_names
	void deleteLinkTerms(VHSet<String> l_names);

      // Kann diese Transition schalten
    virtual int CanFire(void) const;
    void ChangeValueTerm(const String &new_vt);
    void ChangeActionTerms(const String &new_at);
    VHSet<String> GetMacros(void);
    void InsertMacro(String macro, String ersetzung);
    void Instantiate(void);
    void ChangeForeach(void);
    void StringToSet(String s, SET<Val> &set) const;
    String SetToString(VHSet<String> set) const;
    void StringRangeAppendSet(String range, SET<Val> &set) const;
    virtual HLTransition *asHLTransition() const;
    const String &ActionTerms(void) const;
    const VT &ValueTerm(void) const;
    Val EvaluateVT(Binding &b);
    void CacheFlush(void);
    void AddMarking(String Marking, int erg);
    void AddMarking(String Marking, String erg);
    int InCache(String Marking);
    int ReturnCache1(String Marking);
    String ReturnCache2(String Marking);
    const HLTransition &operator =(const HLTransition &t);
    HLTransition(const Net &n, const String &Type = "HLTransition");
    ~HLTransition(){};
    int ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[]);
    int ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[]);
};

inline const HLTransition &
HLTransition::operator =(const HLTransition &t)
{
  if (Name() == t.Name())
    return *this;
  Transition::operator =(t);
  action_terms = t.action_terms;
  value_term = t.value_term;
  Value = t.Value;
  abstract = t.abstract;
  rx_at = t.rx_at;
  ry_at = t.ry_at;
  rx_vt = t.rx_vt;
  ry_vt = t.ry_vt;

  // copy link terms
  rx_links = t.rx_links;
  ry_links = t.ry_links;
  links = t.links;
  Link_import.clear();
  Link_export.clear();
  Link_import += t.getImport();
  Link_export += t.getExport();
  return *this;
}

inline const String &
HLTransition::ActionTerms(void) const
{
  return action_terms;
}  // HLTransition::ActionTerms

inline void
HLTransition::ChangeActionTerms(const String &at)
{
  action_terms = at;
}  // HLTransition::ChangeActionTerms

inline const VT &
HLTransition::ValueTerm(void) const
{
  return value_term;
}  // HLTransition::ValueTerm

inline Val
HLTransition::EvaluateVT(Binding &b)
{
  return value_term.Evaluate(b);
}  // HLTransition::EvaluateVT

inline int
HLTransition::CanFire(void) const {
  if(!(this->Transition::CanFire()))
    return 0;
  return 1;
}

inline MultiSET<LinkTerm>
HLTransition::getImport() const
{
  return Link_import;
}

inline MultiSET<LinkTerm>
HLTransition::getExport() const
{
  return Link_export;
}

#endif  // #ifndef __HLTRANSITION_H__

  //*** Dateiende HLTransition.h ***

