#include "Val.h"
#include "../include/SLStack.h"
#include <ctype.h>
#include <stdio.h>

int err;

Val::Val(const Val& v1, const Val& v2) : vtype(TUPEL), 
    tupel(new Tupel(v1,v2))
{
    // Nix
}

Val::Val(const String& str) : vtype(STRING), string(new String(str))
{
    // Nix
}

Val::~Val() {
  if (vtype == STRING) {
    delete string;
  }
  else if (vtype == TUPEL) {
    delete tupel;
  }
}

const Val& 
Val::operator =(const Val& v) 
{
    vtype = v.vtype;
    if (vtype == TUPEL) {
	tupel = new Tupel(*v.tupel);
    }
    else if(vtype == STRING) {
        string = new String(*v.string);
    }
    else
	value = v.value;
    return *this;
}

Val::Val(const Val& v):vtype(v.vtype)
{
    if (vtype == TUPEL) {
	tupel = new Tupel(*v.tupel);
    }
    else if(vtype == STRING) {
           string = new String(*v.string);
         }
    else
	value = v.value;
}

Val::Val(const Tupel& t) : vtype(TUPEL), tupel(new Tupel(t))
{
    
    // Nix
}


String
Binding::asString(void) const
{
    String l;
    Pix i,n;

    for (i=first(), n=i; i; next(i)) {
	l += key(i) + "=" + contents(i).asString();
	next(n);
	if (n)
	    l += ",";
    }
    return l;
}


Val
Val::get(int i) const
{
    if(vtype == TUPEL)
      return tupel->get(i);
    if(vtype == INTEGER)
      return value;
    if(vtype == DOT)
      return *this;
    if(vtype == STRING)
      return *string;
    if(vtype == BOOLEAN)
      return (value ? Val("tt") : Val("ff"));
    return Val(0);
}

void
Val::set(Val a, int i)
{
    if(a.vtype == TUPEL)
      set(a.get(0), i);
    if(vtype == TUPEL)
      tupel->set(a,i);
    else {
        vtype = a.vtype;
        if(vtype == STRING) {
          string = new String(*a.string);
        }
        else if(vtype == TUPEL) {
          tupel = new Tupel(*a.tupel);
        }
        else
          value = a.value;
    }
}

void
Val::replaceAll(String s, Val a)
{
  if(vtype == STRING)
    *this = a;
  if(vtype != TUPEL)
    return;
  tupel->replaceAll(s, a);
}

int
Val::find(String s) const
{
    if(vtype == STRING) {
      return (s == *string);
    }
    if(vtype == TUPEL) {
      return tupel->find(s);
    }
    if(vtype == DOT) {
      return ((s == "dot") || (s == "(dot)"));
    }
    return 0;
}


int
Val::hasStrings(void) const
{
    if(vtype == TUPEL)
      return tupel->hasStrings();
    return (vtype == STRING);
}


MultiSET<String>
Val::Strings() const
{
    MultiSET<String> var;
    if(vtype == TUPEL)
      return tupel->Strings();
    if(vtype == STRING)
      var.add(*string);
    return var;
}


String
Val::asString(void) const
{
    switch (vtype) {
    case INTEGER:
	return IntToString(value);
    case DOT:
	return "dot";
    case BOOLEAN:
	return (value ? "tt" : "ff");
    case TUPEL:
	return tupel->asString();
    case STRING:
        return *string;
    }
    return "";
}
	

Tupel
StringToTupel(String val_string)
{
    val_string = val_string.gsub("(","");
    val_string = val_string.gsub(")","");
    
    String te = val_string;
    String ta = te;
    int help = 0;
    while(te != "") {
      te = te.after(",");
      help++;
    }
    Tupel temp(help);
    while(ta != "") {
      if(ta.before(",") != "")
        temp.add(StringToVal(ta.before(",")));
      else
        temp.add(StringToVal(ta));
      ta = ta.after(",");
    }
    return temp;
}



Val
StringToVal(const String& val_string)
{
    int v;
    Val value;

    if (sscanf(val_string, "%d", &v))
	value = v;
    else {
	String num = val_string.gsub(" ","");
	if (num == "dot")
	    value = dot;
	else if (num == "ff")
	    value = Val(false);
	else if (num == "tt")
	    value = Val(true);
	else if (num.contains("("))
	    value = StringToTupel(val_string);
	else {
/*	    Error("This is no valid Value\n"
		  "Value must be Integer, tt (=true), ff (=false) or dot");*/
            value = val_string;
	    // value = dot;
	}
    }
    return value;
}

int
IsVal(const String& val_string)
{
    int v;

    if (sscanf(val_string, "%d", &v))
      return 1;
    else {
      String num = val_string.gsub(" ","");
      if (num == "dot")
	return 1;
      else if (num == "ff")
	return 1;
      else if (num == "tt")
	return 1;
    }
    return 0;
}

ostream&
operator <<(ostream& s, const Val& v)
{
    switch (v.vtype) {
    case INTEGER:
	return s << v.value;
    case DOT:
	return s << "dot";
    case BOOLEAN:
	return s << (v.value ? "tt" : "ff");
    case TUPEL:
	return s << v.tupel->asString();
    case STRING:
        return s << v.string;
    }
    return s;  // Damit der Compiler Ruhe gibt
}

Val
Val::pr0() const
{
    if (vtype != TUPEL) {
	Error("Use of tuple without a tuple!!");
	return 0;
    }
    return tupel->pr0();
}

Val
Val::pr1() const
{
    if (vtype != TUPEL) {
	Error("Use of tuple without a tuple!!");
	return 0;
    }
    return tupel->pr1();
}

	
Val
Val::operator +(const Val& a) const
{
    if (vtype != INTEGER || a.vtype != INTEGER) {
	//Error("Addition is only defined for INTEGER");
        //err = -1;
	//return Val(0);
        return Val(false);
    }
    return Val(value + a.value);
}

Val
Val::operator -(const Val& a) const
{
    if (vtype != INTEGER || a.vtype != INTEGER) {
	//Error("Subtraction is only defined for INTEGER");
        //err = -1;
	//return Val(0);
        return Val(false);
    }
    return Val(value - a.value);
}

Val
Val::operator *(const Val& a) const
{
    if (vtype != INTEGER || a.vtype != INTEGER) {
	//Error("Multiplication is only defined for INTEGER");
        //err = -1;
	//return Val(0);
        return Val(false);
    }
    return Val(value * a.value);
}

Val
Val::operator /(const Val& a) const
{
    if (a.vtype != INTEGER || a.vtype != INTEGER) {
	//Error("Division is only defined for INTEGER");
        //err = -1;
	//return Val(0);
        return Val(false);
    }
    if (a.value == 0) {
	Error("Division by zero");
	return 0;
    }
    return Val(value / a.value);
}

Val
Val::operator %(const Val& a) const
{
    if (a.vtype != INTEGER || a.vtype != INTEGER) {
	//Error("Modulo is only defined for INTEGER");
        //err = -1;
	//return Val(0);
        return Val(false);
    }
    if (a.value == 0) {
	Error("Modulo by zero");
	return 0;
    }
    return Val(value % a.value);
}

bool
Val::operator &(const Val& a) const
{
    if ((vtype != BOOLEAN && vtype != INTEGER) || (a.vtype != BOOLEAN && a.vtype != INTEGER)) {
	Error("& is only defined for BOOLEAN");
        //err = -1;
	return false;
    }
    // return ((value && a.value) ? true : false);
    return ((*this == true) && (a == true)) ? true : false;
}

bool
Val::operator |(const Val& a) const
{
    if ((vtype != BOOLEAN && vtype != INTEGER) || (a.vtype != BOOLEAN && a.vtype != INTEGER)) {
	//Error("| is only defined for BOOLEAN");
        //err = -1;
	return false;
    }
    return ((value || a.value) ? true : false);
}

bool
Val::imp(const Val& a) const
{
    if ((vtype != BOOLEAN && vtype != INTEGER) || (a.vtype != BOOLEAN && a.vtype != INTEGER)) {
	//Error("-> is only defined for BOOLEAN");
        //err = -1;
	return false;
    }
    return ((!value || a.value) ? true : false);
}

bool
Val::operator !=(const int i) const
{
  if (vtype == BOOLEAN || vtype == INTEGER)
    return value != i;
  else
    return true;
}

bool
Val::operator ==(const int i) const
{
    return value==i;
}

bool
Val::operator ==(const Val& a) const
{
    if (vtype == a.vtype) {
        if(vtype == DOT)
           return 1;
	if (vtype == TUPEL)
            return *tupel == *a.tupel;
	else if(vtype == STRING) {
               return (*string == *a.string);
             }
        else
	    return value == a.value;
    }
    if(a.vtype == TUPEL) {
      return (*this == a.get(0));
    }
    if(vtype == TUPEL) {
      return (get(0) == a);
    }
    return false;
}

bool
Val::EQUAL(const Val& a) const
{
    if (vtype == a.vtype) {
        if(vtype == DOT)
           return true;
	else if (vtype == TUPEL)
            return tupel->EQUAL(*a.tupel);
	else if(vtype == STRING) {
               return (*string == *a.string);
             }
        else
	    return value == a.value;
    }
    if(a.vtype == TUPEL) {
      return (this->EQUAL(a.get(0)));
    }
    if(vtype == TUPEL) {
      return (get(0).EQUAL(a));
    }
    return (((vtype == STRING) && (a.vtype == INTEGER)) || ((vtype == INTEGER) && (a.vtype == STRING)) || ((vtype == STRING) && (a.vtype == DOT)) || ((vtype == DOT) && (a.vtype == STRING)) || ((vtype == BOOLEAN) && (a.vtype == STRING)) || ((vtype == STRING) && (a.vtype == BOOLEAN)));
}

bool
Val::operator !=(const Val& a) const
{
    return !(*this == a);
}

bool
Val::operator <(const Val& a) const
{
    if ((vtype == DOT && a.vtype == INTEGER) || (vtype == DOT && a.vtype == INTEGER))
        return false;
    if (vtype != INTEGER || a.vtype != INTEGER) {
	Error("Relation < is only defined for INTEGER (and DOT)");
	return false;
    }
    return (value < a.value); // ? true : false;
}

bool
Val::operator >(const Val& a) const
{
    if ((vtype == DOT && a.vtype == INTEGER) || (vtype == DOT && a.vtype == INTEGER))
        return false;
    if (vtype != INTEGER || a.vtype != INTEGER) {
	Error("Relation > is only defined for INTEGER (and DOT)");
	return 0;
    }
    return (value > a.value); // ? true : false;
}

bool
Val::operator <=(const Val& a) const
{
    if ((vtype == DOT && a.vtype == INTEGER) || (vtype == DOT && a.vtype == INTEGER))
        return false;
    if (vtype != INTEGER || a.vtype != INTEGER) {
	Error("Relation <= is only defined for INTEGER");
	return false;
    }
    return (value <= a.value); // ? true : false;
}

bool
Val::operator >=(const Val& a) const
{
    if ((vtype == DOT && a.vtype == INTEGER) || (vtype == DOT && a.vtype == INTEGER))
        return false;
    if (vtype != INTEGER || a.vtype != INTEGER) {
	Error("Relation >= is only defined for INTEGER");
	return false;
    }
    return (value >= a.value); // ? true : false;
}

bool
Val::operator !() const
{
    if (vtype != BOOLEAN && vtype != INTEGER) {
	//Error("Operator ! is only defined for BOOLEAN");
	return false;
    }  
    return (value == 1) ? false : true;
}

int
VTNode::operator ==(const VTNode& a) const
{
    if (node_type == a.node_type) 
	
	switch (node_type) {
	    
	case OPERATOR:
	    return (a.otype == otype);

	case VALUE:
	    return (*a.v == *v);

	case VARIABLE:
	    return *ident == *a.ident;
	    break;
	default:
	    Error("Something is wrong with VTNode::operator==");
	    return false;
	}
    else
	return false;

}
void
VTNode::Delete(void)
{
    if (node_type == VALUE)
	delete v;
    else if (node_type == VARIABLE)
	delete ident;
    
}   
 
VTNode&
VTNode::operator =(const VTNode& right)
{
    if (this == &right)
	return *this;
    
    Delete();

    node_type = right.node_type;

    switch (node_type) {
    case OPERATOR:
	otype = right.otype;
	break;
    case VALUE:
	v = new Val(*right.v);
	break;
    case VARIABLE:
	ident = new String(*right.ident);
	break;

    default:
	Error("Something is wrong with VtNode::operator=");

    }

    return *this;
};

VTNode::VTNode(const VTNode& a)
{
    node_type = a.node_type;
    switch (node_type) {

    case VALUE:
	v = new Val(*a.v);
	break;

    case VARIABLE:
	ident = new String(*a.ident);
	break;

    case OPERATOR:
	otype = a.otype;
	break;
    /* case PROJECTION:
	ptype = a.ptype;
	break; */
    }
}

Val
VT::EvaluateDict(Binding& binding, const Pix& i)
{
    if (!i)    // Ende der Rekursion, letzte Variable erreicht
	return FixedEvaluate(binding);

    
    const String& var = intern_set.key(i);
    
    const SET<Val>& values = intern_set.contents(i);
    for (Pix k=values.first(); k; values.next(k)) {
	bind[var] = values(k);
	Pix l=i;
	intern_set.next(l);
	if (EvaluateDict(binding,l) == true)
	    return true;
    }
cerr << "Evaluation false" << endl;
    return false;
}

Val
VT::Evaluate(Binding& bindings)
{
    bind.clear();
    return EvaluateDict(bindings, intern_set.first());
}

Val
VT::FixedEvaluate(Binding& binding)
{
    if (upn_list.empty()) {
	Error("Nothing to evaluate");
	return false;
    }

    Val ret;

    Val a, b;
    int cont = 1;

    SLStack<Val> upn_stack;

    for (Pix i=upn_list.first(); i!=0 && cont; upn_list.next(i)) {
	
	switch (upn_list(i).Type()) {      // Was fr ein Typ

	case OPERATOR:   
	    // Falls Operator -> die beiden obersten Werte vom Stack holen
	    // und auf diesen den Operator anwenden

	    b = upn_stack.pop();
		
	    switch (upn_list(i).Operator()) {

	    case AND:
		a = upn_stack.pop();
                //err = 0;
		upn_stack.push(a & b);
                /*if(err == -1) {
                  upn_stack.clear();
                  return false;
                }*/
		break;

	    case OR:
		a = upn_stack.pop();
                //err = 0;
		upn_stack.push(a | b);
                /*if(err == -1) {
                  upn_stack.clear();
                  return false;
                }*/
		break;

	    case IMP:
		a = upn_stack.pop();
                //err = 0;
		upn_stack.push(a.imp(b));
                /*if(err == -1) {
                  upn_stack.clear();
                  return false;
                }*/
		break;

	    case ADD:
		a = upn_stack.pop();
                //err = 0;
		upn_stack.push(a + b);
                /*if(err == -1) {
                  upn_stack.clear();
                  return false;
                }*/
		break;

	    case SUB:
		a = upn_stack.pop();
                //err = 0;
		upn_stack.push(a - b);
                /*if(err == -1) {
                  upn_stack.clear();
                  return false;
                }*/
		break;
		
	    case DIV:
		a = upn_stack.pop();
                //err = 0;
		upn_stack.push(a / b);
                /*if(err == -1) {
                  upn_stack.clear();
                  return false;
                }*/
		break;
		
	    case MULT:
		a = upn_stack.pop();
                //err = 0;
		upn_stack.push(a * b);
                /*if(err == -1) {
                  upn_stack.clear();
                  return false;
                }*/
		break;

	    case MOD:
		a = upn_stack.pop();
                //err = 0;
		upn_stack.push(a % b);
                /*if(err == -1) {
                  upn_stack.clear();
                  return false;
                }*/
		break;

	    case EQ:
		a = upn_stack.pop();
		upn_stack.push((a == b) ? true : false);
		break;

	    case NOT_EQ:
		a = upn_stack.pop();
		upn_stack.push((a != b) ? true : false); 
		break;

	    case GT:
		a = upn_stack.pop();
		upn_stack.push((a > b) ? true : false);
		break;

	    case LE:
		a = upn_stack.pop();
		upn_stack.push((a < b) ? true : false);
		break;

	    case GT_EQ:
		a = upn_stack.pop();
		upn_stack.push((a >= b) ? true : false);
		break;

	    case LE_EQ:
		a = upn_stack.pop();
		upn_stack.push((a <= b) ? true : false);
		break;

	    case NOT:
		upn_stack.push(!b);
		break;
	    case PR0:
		upn_stack.push(b.pr0());
		break;
	    case PR1:
		upn_stack.push(b.pr1());
		break;
	    }
	    break;
	case VALUE:
	    // Fall Zahl -> auf Stack bringen
	    upn_stack.push(upn_list(i).Value());
	    break;
	    
	case VARIABLE:
	    // Falls Variable -> den Wert der Variable auf Stack
	    if (binding.contains(upn_list(i).Identifier())) {
		upn_stack.push(binding[upn_list(i).Identifier()]);
	    }
	    else if (bind.contains(upn_list(i).Identifier())) {
		upn_stack.push(bind[upn_list(i).Identifier()]);
		//cout << "Var :" << upn_list(i).Identifier()
		//    << "Wert :" << bind[upn_list(i).Identifier()] << endl;
	    }
	    else { // In diesem Fall -> ueberhaupt keine Bindung gefunden
		// Liste als abgearbeitet erklaeren
		// Stack loeschen und false drauf
		cont = 0;
		upn_stack.clear();
		upn_stack.push(false);
	    }
	    /* case PROJECTION:
	       if (upn_list(i).Projection() == PR0) {
	       upn_list.next(i);
	       upn_stack.push(binding[upn_list(i).Identifier()].pr0());
	       }
	       else {
	       upn_list.next(i);
	       upn_stack.push(binding[upn_list(i).Identifier()].pr1());
	       }
	       break; */
	    
	}
	
    }

    // Alles ausgewertet -> dann Wert zurck
    ret = upn_stack.pop();

    if (!upn_stack.empty())
	Error("Something is wrong with VT::Evaluate.\n"
	      "The stack is not empty as it should be");

    return ret;
}

int
VT::ChangeTerm(const String& new_vt) 
{
    string_term = new_vt;
    vars_in_VT.clear();
    upn_list.clear();
    return parse_VT(*this);
}



// Fuer Martin

extern "C" {

    int CheckVT(const char* vt_string) {
	VT vt(vt_string);
	Binding b;
	return (vt.Evaluate(b) == true) ? 1 : 0;
    }
}

    
