#ifndef VAL_H
#define VAL_H


#include <iostream>
#include "../include/SLList.h"
#include "../include/VHMap.h"
#include "../include/VHSet.h"
#include "../include/Utils.h"
#include "../include/MultiSet.h"
#include "Enums.h"
#include "ExprInterface.h"


class VT;


extern "C" { int  parse_VT(VT& a); }


    
class Tupel;

    
class Val {               // Wert -> Integer, Dot, Bool, String
private:

    friend ostream& operator <<(ostream& s, const Val& v);

    ValueType vtype;      // hierdurch angezeigt
    
    // int value;            // und der dazugehrige Wert

    union {
	int    value;
	Tupel* tupel;
        String* string;
    };

public:

    Val          pr0() const;
    Val          pr1() const;

    bool operator  &(const Val& a) const;
    bool operator  |(const Val& a) const;
    bool        imp(const Val& a) const;
    Val operator  +(const Val& a) const;
    Val operator  -(const Val& a) const;
    Val operator  *(const Val& a) const;
    Val operator  /(const Val& a) const;
    Val operator  %(const Val& a) const;
    bool operator ==(const Val& a) const;
    bool operator ==(const int i) const;
    bool operator !=(const Val& a) const;
    bool operator !=(const int i) const;
    bool operator  <(const Val& a) const;
    bool operator  >(const Val& a) const;
    bool operator <=(const Val& a) const;
    bool operator >=(const Val& a) const;
    bool operator  !() const;
    bool EQUAL(const Val& a) const;

    const Val& operator =(const Val& v);

    String asString(void) const;

    int hasStrings(void) const;

    MultiSET<String> Strings(void) const;

    int find(String s) const;

    Val get(int i) const;
    void set(Val v, int i);
    void replaceAll(String s, Val a);

    Tupel *getTupel(void)  {
        return (tupel);
    }

    ValueType getType(void) const {
	return vtype;
    }

    ValueType Type(void) const {
	return vtype;
    }

    int asInt(void) const {
	return value;
    }

    Val() : vtype(INTEGER), value(0) {};
   
    Val(const int a) : vtype(INTEGER), value(a) {};

    Val(const bool a) : vtype(BOOLEAN), value(a) {};

    Val(const DotType a) : vtype(DOT), value(-12345) {};
    
    Val(const Val& v1, const Val& v2);

    Val(const String& st);

    Val(const Tupel& t);

    Val(const Val& v);

    ~Val();
};


inline unsigned int 
Hash(const Val& v)
{
    return hash_string(v.asString());
}

inline int
LessEq(const Val& a, const Val& b)
{
    return a.asInt()<=b.asInt();
}


class Tupel {
private:

    Val* v;
    int count;
    int dimension;

public:

    Tupel(int capacity) {
      v = new Val[capacity];
      dimension = capacity;
      count = 0;
    };

    Tupel(Val a, Val b) /*: v0(a), v1(b) */ {
      v = new Val[2];
      v[0] = a;
      v[1] = b;
      dimension = count = 2;
    };

    Tupel() /*: v0(0), v1(0) */ {
      v = new Val[2];
      v[0] = Val(0);
      v[1] = Val(0);
      dimension = count = 2;
    };

    Tupel(const Tupel& a) {
      v = new Val[a.dimension];
      for(int i = 0;i < a.dimension;i++) {
        v[i] = a.v[i];
      }
      dimension = a.dimension;
      count = a.count;
    };

    ~Tupel() {
      delete [] v;
    };

    Val pr0() const {
        return v[0];
        // return v0;
    };

    Val pr1() const {
        return v[1];
        // return v1;
    };

    Val get(int i) const {
        if(i < dimension) {
          return v[i];
        }
        return Val(0); // damit der Compiler Ruhe gibt
    };

    void set(Val a, int i) {
        if(i < dimension) {
          v[i] = a;
        }
    };

    int find(String s) const {
        for(int i = 0;i < dimension;i++) {
           if(v[i].find(s)) {
             return i;
           }
        }
        return -1;
    };

    void replaceAll(String s, Val a) {
        for(int i = 0;i < dimension;i++) {
          if(v[i] == s)
            set(a,i);
        }
    };

    void add(Val a) {
        v[count++] = a;
    };
    
    int operator ==(const Tupel& a) const {
        if(dimension != a.dimension)
          return 0;
        for(int i = 0; i < dimension; i++) {
          if(v[i] != a.v[i])
            return 0;
        }
        return 1;
        // return v0 == a.v0 && v1 == a.v1;
    };

    int EQUAL(const Tupel& a) const {
        if(dimension != a.dimension)
          return 0;
        for(int i = 0; i < dimension; i++) {
          if(!v[i].EQUAL(a.v[i]))
            return 0;
        }
        return 1;
    };

    String asString() const {
        String temp;
        if(dimension > 1)
          temp = "(";
        for(int i = 0;i < dimension;i++) {          
           temp += v[i].asString();
           if(i < dimension-1)
             temp += ",";
        }
        if(dimension > 1)
          temp += ")";
        return temp;
	// return "(" + v0.asString() + "," + v1.asString() + ")";
    };

    int hasStrings() const {
       for(int i = 0;i < dimension;i++) {
         if(v[i].getType() == STRING)
           return 1;
       }
       return 0;
    };

    MultiSET<String> Strings() const {
       MultiSET<String> var;
       for(int i = 0;i < dimension;i++) {
         if(v[i].getType() == STRING)
           var.add(v[i].asString());
       }
       return var;
    };
	
	const int getDimension(void) const {
	  return dimension;
	};
	
}; // class Tupel


inline unsigned int 
Hash(const Tupel& a)
{
    return hash_int(a.pr0().asInt() * a.pr1().asInt());
}

class VTNode {
private:
    
   
    NodeType node_type;

   
    union {                       // Knoten kann sein
	
	Val* v;                   // Wert (hier Pointer da in unions keine
				  // Klassen mit Konstruktoren zugelassen)
	    
	String* ident;          // oder aber Variable

	OperatorType otype;       // oder Operator.

	// Proj ptype;

    };
    
    void Delete(void);

public:

    NodeType Type(void) {
	return node_type;
    };

    OperatorType Operator() {
	return otype;
    };

    const Val& Value() {
	return *v;
    };

    /* Proj Projection() const {
	return ptype;
    }; */

    const String& Identifier() {
	return *ident;
    };

    int operator ==(const VTNode& v) const;

    VTNode& operator =(const VTNode& v);

    VTNode(const VTNode& a);
    
    VTNode(const int a)          : node_type(VALUE), v(new Val(a)) {};
    
    VTNode(const bool a)     : node_type(VALUE), v(new Val(a)) {};

    VTNode(const DotType a)      : node_type(VALUE), v(new Val(a)) {};

    VTNode(const String& s)    : node_type(VARIABLE), 
	ident(new String(s)) {};

    VTNode(const char* s)        : node_type(VARIABLE), 
	ident(new String(s)) {};

    VTNode(const OperatorType t) : node_type(OPERATOR), otype(t) {};


    // VTNode(Proj a) : node_type(PROJECTION) , ptype(a) {};

    VTNode() : node_type(VALUE), v(new Val(true)) {};


    ~VTNode() {
	Delete();
    }; 
};


// class TupelBinding : public VHMap<String, Tupel> {
// public:

//     String asString(void) const;

//     TupelBinding() : VHMap<String, Tupel>(Tupel(0,0), 10) {};
// };

class Binding : public VHMap<String, Val> {
public:

    String asString(void) const;

    Binding() : VHMap<String,Val>(0,10) {};
};
    
class VT {

    friend void new_ident(char *s);
    friend void new_number(int a);
    friend void new_number_dot(void);
    friend void new_number_bool(bool);
    friend void new_operator(OperatorType t);
    friend void new_value(int);
    friend void new_values(int,int);
    friend void store_last(void);
    friend void new_value_bool(bool);
    friend void new_value_dot(void);
    friend void make_cross(void);
    friend int  parse_VT(VT& a);

protected:
    
    Binding bind;

    String string_term;
    
    VHSet<String> vars_in_VT;

    SLList<VTNode> upn_list;

    VHMap<String, VHSet<Val> > intern_set;

    Val EvaluateDict(Binding& b, const Pix& i);

    Val FixedEvaluate(Binding& bb);
    
public:

    int ChangeTerm(const String& new_term);

    const String& Text(void) const {
	return string_term;
    };

    Val Evaluate(Binding& b);
    
    VT(const String& ct) : string_term(ct), intern_set(5) {
	parse_VT(*this);
    };


};

Val StringToVal(const String& val_string);
int IsVal(const String& val_string);
Tupel StringToTupel(String t);

#endif
