// Block.cc
// Blcke fr Lowlevel-Netze
// Burkhard Bieber 1995/96
// last-update: 07.03.96

  //**********
  // includes
  //**********

#include <ctype.h>
#include "Block.h"
#include "Net.h"

  //**************************
  // Inits fr static members
  //**************************

int Block::block_counter = 1;
int Block::active_objects = 0;

  //********************
  // Methoden fr Block
  //********************

Block::~Block()
{
  if (--active_objects == 0)
    block_counter = 1;
  VHMap<String, Place *> Places = net->Places();
  VHMap<String, Transition *> Trans = net->Transitions();
  VHMap<String, PTransition *> PTrans = net->PTransitions();
  for (Pix i = Places.first(); i;Places.next(i)) {
    if (Places.contents(i)->BlockKeySet().contains(name)) {
      net->RemovePlace(Places.key(i));
    }
  }
  for (Pix i = Trans.first(); i;Trans.next(i)) {
    if (Trans.contents(i)->BlockKeySet().contains(name)) {
      net->RemoveTransition(Trans.key(i));
    } 
  }
  for (Pix i = PTrans.first(); i;PTrans.next(i)) {
    if (PTrans.contents(i)->BlockKeySet().contains(name)) {
      net->RemovePTransition(PTrans.key(i));
    }
  }
  const VHMap<String, Block *> bl = net->Blocks();
  for(Pix i = bl.first(); i; bl.next(i)) {
    if(bl.key(i) != name)
      bl.contents(i)->RemoveBlock(name);
  }
}  // Block::~Block

void
Block::del(String name, Block *b)
{
  if( b == NULL)
    return;
  for(Pix i = b->blocks.first(); i; b->blocks.next(i)) {
    if(b->blocks.contents(i)->Name() == name) {
      b->RemoveBlock(name);
      return;
    } else {
      del(name, (b->blocks).contents(i));
    }
  }
}  // Block::delete

Block::Block(const Block &b) : Vertex(b),
  rootblock_flag(1)
  // Konstruktor
{
  id = block_counter++;
  name = PreFix() + IntToString(id);
  active_objects++;
}  // Block::Block

Block::Block(const Net &n, const String &Type) : Vertex(n, Type),
  rootblock_flag(1)
  // Konstruktor
{
  net->DefaultBlockName(rx_name, ry_name);
  net->DefaultBlockMean(rx_mean, ry_mean);
  base_classes.add("Vertex");
  active_objects++;
  id = block_counter++;
  name = PreFix() + IntToString(id);
}  // Block::Block

void
Block::Write(ostream &f) const
  // Ausgabe blockspezifischer Dinge
{
  Vertex::Write(f);
  if (rx_name != net->DefaultBlockNameX() ||
    ry_name != net->DefaultBlockNameY()) 
    f << 'n' << rx_name << '@' << ry_name;
  if (rx_mean != net->DefaultBlockMeanX() ||
    ry_mean != net->DefaultBlockMeanY()) 
    f << 'a' << rx_mean << '@' << ry_mean;
}  // Block::Write

void
Block::ReadOptional(istream &f)
  // optionale Werte einlesen
{
  char c;
  Vertex::ReadOptional(f);
  f.get(c);
  switch (c) {
    case 'P':
      f.get(c);
      break;
    default:
      f.putback(c);
  }  // switch ber c
}  // Block::ReadOptional

void
Block::ChangeMyName(const String &new_name) 
  // Namensnderung durchfhren
{
  net->ChangeBlockName(name, new_name);
  Vertex::ChangeMyName(new_name);
}  // Block::ChangeMyName

int
Block::ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[])
  // TCL-Schnittstelle
{
  if (argc != 1) {
    Tcl_AppendResult(ip, "wrong #args for Block get\n",
       NULL);
    return TCL_ERROR;
  }  // if mehr als ein Argument
  else
    return Vertex::ObjectGet(ip, argc, argv);
  return TCL_OK;
}  // Block::ObjectGet

int 
Block::ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[])
  // TCL-Schnittstelle
{
  if (argc < 2) {
    Tcl_AppendResult(ip, "wrong #args for Block set\n",
      NULL);
    return TCL_ERROR;
  }  // if weniger als 2 Argumente
  if (EQ(0, "Blocks")) {
    String para = argv[1];
    int zaehler = 0;
    while(para != "") {
      para = para.after(" ");
      zaehler++;
    }
    String kette[zaehler];
    para = argv[1];
    blocks.clear();
    for(int i = 0;i < zaehler;i++) {
      kette[i] = (i == zaehler-1) ? para : para.before(" ");
      para = para.after(" ");
      if (net->GetBlockFromName(kette[i]) != NULL) {
        AddBlock(kette[i], net->GetBlockFromName(kette[i]));
        blocks[kette[i]]->UnsetRootBlock();
      }
    }
    if (zaehler != 0)
      net->RecalcBlockVis();
  }
  if (EQ(0, "Box")) {
    box = "";
    boxPointer = (Box *) NULL;
    String para(argv[1]);
    if(net->ContainBox(para)) {
      setBox(para, net->GetBoxFromName(para));
    }
  }
  if (argc > 0)
    return Vertex::ObjectSet(ip, argc,argv);
  return TCL_OK;
}  // Block::ObjectSet

  //*** Dateiende Block.cc ***

