// Block.h
// Blcke fr Lowlevel-Netze
// Burkhard Bieber 1995
// last-update: 16.11.95

#ifndef __BLOCK_H__
#define __BLOCK_H__

  //**********
  // includes
  //**********

#include "Edge.h"
#include "Vertex.h"
#include "../include/String.h"

  //************
  // Konstanten
  //************

const String BlockPrefix = "B";

  //*********
  // Klassen
  //*********

class Block : public Vertex {
  protected:
      // Datei I/O-Funktionen
    virtual void Write(ostream &f) const;
    virtual void ReadOptional(istream &f);
      // Sonstige Funktionen
    virtual const String &PreFix() const;
    virtual void ChangeMyName(const String &new_name);
      //*************************
      // statische Datenelemente
      //*************************
    static int block_counter;
    static int active_objects;
      //***********************
      // normale Datenelemente
      //***********************
      // Boole'sch. Ansage, ob Wurzel
    int rootblock_flag;
  public:
      //**********************
      // Konstante Funktionen
      //**********************
    int operator ==(const Block &b) const;
    int GetRootBlock(void) const;
      //***************************
      // Modifizierende Funktionen
      //***************************
    static void ResetMaxIdTo(int nr);
    virtual void SetMaxId(int id);
    const Block &operator =(const Block &b);
    void SetRootBlock(void);
    void UnsetRootBlock(void);
    void del(String name, Block *b);
      //************************
      // Konvertierungsmethoden
      //************************
    virtual Block *asBlock() const;
      //****************************
      // Konstruktoren / Destruktor
      //****************************
    Block(const Net &n, const String &Type = "Block");
    Block(const Block &b);
    virtual ~Block();
       // Tcl-Schnittstelle
    int ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[]);
    int ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[]);
};  // class Block

  //*********
  // inlines
  //*********

inline const Block &
Block::operator =(const Block &b)
{
  if (b.Id() == Id()) 
    return *this;
  Vertex::operator =(b);
  rootblock_flag = b.rootblock_flag;
  return *this;
}  // Block::operator =

inline int
Block::operator ==(const Block &b) const
{
  return (name == b.name) ? 0 : 1;
}  // Block::operator ==

inline int
Block::GetRootBlock(void) const
{
  return rootblock_flag;
}  // Block::GetRootBlock

inline void
Block::SetRootBlock(void)
{
  rootblock_flag = 1;
}  // Block::SetRootBlock

inline void
Block::UnsetRootBlock(void)
{
  rootblock_flag = 0;
}  // Block::UnsetRootBlock

inline const String &
Block::PreFix() const
{
  return BlockPrefix;
}  // Block::PreFix

inline void
Block::SetMaxId(int nr)
{
  block_counter = (nr>block_counter) ? nr+1 : block_counter;
}  // Block::SetMaxId

inline void
Block::ResetMaxIdTo(int nr)
{
  block_counter = nr;
}  // Block::ResetMaxId

inline Block *
Block::asBlock() const
{
  return (Block *) this;
}  // Block::asBlock

#endif  // #ifndef __BLOCK_H__

  //*** Dateiende Block.h ***

