// Box.cc
// Boxen fr Lowlevel-Netze
// Ulf Fildebrandt 1997
// last-update: 18.11.97

  //**********
  // includes
  //**********

#include "Box.h"

  //**************************
  // Inits fr static members
  //**************************

int Box::box_counter = 1;
int Box::active_objects = 0;

  //**********
  // Methoden
  //**********

Box::~Box()
{
  if (--active_objects == 0)
    box_counter = 1;
}

Box::Box(const Net &n, const String &t) : Vertex(n, t)
{
  net->DefaultBlockName(rx_name, ry_name);
  net->DefaultBlockMean(rx_mean, ry_mean);
  base_classes.add("Vertex");
  active_objects++;
  id = box_counter++;
  name = BoxPrefix + IntToString(id);
}  // Box::Box

void
Box::Write(ostream &f) const
  // keine neuen Dinge im HL Bereich zu schreiben
{
  Vertex::Write(f);
  int c = 0;
  const VHMap<String, Block *> blocks = net->Blocks();
  const VHMap<String, Place *> places = net->Places();
  const VHMap<String, Transition *> transitions = net->Transitions();
  const VHMap<String, PTransition *> ptransitions = net->PTransitions();
  VHSet<String> Menge;
  for(Pix i = places.first(); i; places.next(i)) {
    if(places.contents(i)->BoxName() == name)
      Menge.add(places.key(i));
  }
  if(Menge.length() != 0)
    f << "p\"";
  for(Pix j = Menge.first(); j; Menge.next(j)) {
    f << Menge(j);
    c++;
    if(c != Menge.length())
      f << ",";
  }
  if(Menge.length() != 0)
    f << "\"";
  Menge.clear();
  c = 0;
  for(Pix i = transitions.first(); i; transitions.next(i)) {
    if(transitions.contents(i)->BoxName() == name)
      Menge.add(transitions.key(i));
  }
  if(Menge.length() != 0)
    f << "t\"";
  for(Pix j = Menge.first(); j; Menge.next(j)) {
    f << Menge(j);
    c++;
    if(c != Menge.length())
      f << ",";
  }
  if(Menge.length() != 0)
    f << "\"";
  Menge.clear();
  c = 0;
  for(Pix i = ptransitions.first(); i; ptransitions.next(i)) {
    if(ptransitions.contents(i)->BoxName() == name)
      Menge.add(ptransitions.key(i));
  }
  if(Menge.length() != 0)
    f << "H\"";
  for(Pix j = Menge.first(); j; Menge.next(j)) {
    f << Menge(j);
    c++;
    if(c != Menge.length())
      f << ",";
  }
  if(Menge.length() != 0)
    f << "\"";
  Menge.clear();
  c = 0;
  for(Pix i = blocks.first(); i; blocks.next(i)) {
    if(blocks.contents(i)->BoxName() == name)
      Menge.add(blocks.key(i));
  }
  if(Menge.length() != 0)
    f << "K\"";
  for(Pix j = Menge.first(); j; Menge.next(j)) {
    f << Menge(j);
    c++;
    if(c != Menge.length())
      f << ",";
  }
  if(Menge.length() != 0)
    f << "\"";   
}  // Box::Write

void
Box::ReadOptional(istream &f)
  // Optionale Parameter von stream f einlesen
{
  char c, dummy;
  String s;
  char *s1;

  Vertex::ReadOptional(f);
  f.get(c);
  switch (c) {
    case 't':
      {
        f >> dummy;
        readline(f, s, '"');
        s1 = strdup(s);
        s1 = strtok(s1, ",");
        while(s1) {
          net->FindTransition(s1)->setBox(name, this);
          String(s1).gsub(" ","");
          s1 = strtok(NULL, ",");
        }
        break;
      }
    case 'H':
      {
        f >> dummy;
        readline(f, s, '"');
        s1 = strdup(s);
        s1 = strtok(s1, ",");
        while(s1) {
          net->FindPTransition(s1)->setBox(name, this);
          String(s1).gsub(" ","");
          s1 = strtok(NULL, ",");
        }
        break;
      }
    case 'K': // f"ur Bl"ocke
      {
        f >> dummy;
        readline(f, s, '"');
        s1 = strdup(s);
        s1 = strtok(s1, ",");
        while(s1) {
          net->GetBlockFromName(s1)->setBox(name, this);
          AddBlock(s1, net->GetBlockFromName(s1));
          String(s1).gsub(" ","");
          s1 = strtok(NULL, ",");
        }
        break;
      }
    case 'p':
      {
        f >> dummy;
        readline(f, s, '"');
        s1 = strdup(s);
        s1 = strtok(s1, ",");
        while(s1) {
          net->FindPlace(s1)->setBox(name, this);
          String(s1).gsub(" ","");
          s1 = strtok(NULL, ",");
        }
        break;
      }
    case 'F':
      {
        f >> dummy;
        readline(f, s, '"');
        net->loadmodul(s, false, name);
        if(s.contains(".hl_net"))
          s = s.before(".hl_net");
        else
          s = s.before(".ll_net");
        s = s + ".box";
        ifstream boxes(s.data());
        if (boxes) {
          net->ReadBoxes(boxes);
          boxes.close();
        }
        break;
      }
    default:
      f.putback(c);
  }
}  // Box::ReadOptional

  // Informationen fr Tcl-Teil bereitstellen
int
Box::ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[])
{
  if (argc != 1) {
    Tcl_AppendResult(ip, "wrong #args for Box get",
      NULL);
    return TCL_ERROR;
  }  // if Argumentanzahl ist Kse
  else
    return Vertex::ObjectGet(ip, argc, argv);
  return TCL_OK;
}  // Box::ObjectGet

  // Informationen aus Tcl-Teil bernehmen
int
Box::ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[])
{
  if (argc < 2) {
    Tcl_AppendResult(ip, "wrong #args for Box set",
      NULL);
    return TCL_ERROR;
  }  // if Argumentanzahl falsch
  if (EQ(0, "Blocks")) {
    String para = argv[1];
    int zaehler = 0;
    while(para != "") {
      para = para.after(" ");
      zaehler++;
    }
    String kette;
    para = argv[1];
    blocks.clear();
    for(int i = 0;i < zaehler;i++) {
      kette = (i == zaehler-1) ? para : para.before(" ");
      para = para.after(" ");
      if (net->ContainBlock(kette))
        AddBlock(kette, net->GetBlockFromName(kette));
    }
  }

  if (argc > 0)
    return Vertex::ObjectSet(ip, argc, argv);
  return TCL_OK;
}  // Box::ObjectSet


