// Box.h
// Highlevel-Boxs fr den Netzeditor
// Ulf Fildebrandt 1997
// last-update: 18.11.97

#ifndef __BOX_H__
#define __BOX_H__

  //**********
  // Includes
  //**********

#include "Edge.h"
#include "Vertex.h"
#include "Net.h"
#include "../include/String.h"

  //************
  // Konstanten
  //************

const String BoxPrefix = "Box";

  //**********
  //  Klassen 
  //**********

class Box : public Vertex {
  protected:
    virtual void Write(ostream &f) const;
    virtual void ReadOptional(istream &f);
      // Sonstige Funktionen
    virtual const String &PreFix() const;
      //*************************
      // statische Datenelemente
      //*************************
    static int box_counter;
    static int active_objects;

  public:
      //***************************
      // Modifizierende Funktionen
      //***************************
    static void ResetMaxIdTo(int nr);
    virtual void SetMaxId(int id);

    virtual Box *asBox(void) const;
    Box(const Net &n, const String &Type="Box");
    Box(const Box &b);
    virtual ~Box();
    virtual int ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[]);
    virtual int ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[]);
};

inline
Box::Box(const Box &b) : Vertex(b)
{
  id = box_counter++;
  name = BoxPrefix + IntToString(id);
  active_objects++;
}  // Box::Box

inline Box *
Box::asBox() const
{
  return (Box *) this;
}  // Box::asBox

inline const String &
Box::PreFix() const
{
  return BoxPrefix;
}  // Box::PreFix

inline void
Box::SetMaxId(int nr)
{
  box_counter = (nr>box_counter) ? nr+1 : box_counter;
}  // Box::SetMaxId

inline void
Box::ResetMaxIdTo(int nr)
{
  box_counter = nr;
}  // Box::ResetMaxId

#endif

