// Edge.cc
// Tobias Himstedt 1995
// Burkhard Bieber 1995/96
// last-update: 24.06.96

  //**********
  // includes
  //**********

#include "Edge.h"
#include "Vertex.h"
#include "Block.h"
#include "Net.h"

  //*******************************
  // Inits fr statische Variablen
  //*******************************

int Edge::edge_counter = 1;
int Edge::active_objects = 0;

  //*******************
  // Methoden fr Edge
  //*******************

Edge::Edge(const Net &n, const String &Type) : NetObject(Type),
  id(edge_counter++), from(NULL), to(NULL),
  weight(1), splinex(0), spliney(0), net((Net *) &n), phantom_flag(0),
  blocks_from((Block *) NULL), blocks_to((Block *) NULL)
{
  net->DefaultEdgeWeight(rx_weight, ry_weight);
  active_objects++;
}  // Edge::Edge

Edge::Edge(Vertex &f, Vertex &t, const Net &n, const String &Type) : 
  NetObject(Type), id(edge_counter++),
  from(&f), to(&t),
  weight(1), splinex(0), spliney(0), net((Net *) &n), phantom_flag(0),
  blocks_from((Block *) NULL), blocks_to((Block *) NULL)
{
  net->DefaultEdgeWeight(rx_weight, ry_weight);
  active_objects++;   
  base_classes.add("NetObject");
}  // Edge::Edge

Edge::~Edge()
{
  Pix i;
  String temp;
  char buff[100];

  if(!phantom_flag) {
    from->RemoveOutEdge(*this);
    to->RemoveInEdge(*this);
    for (i=blocks_from.first(); i; blocks_from.next(i))
      blocks_from.contents(i)->RemoveOutEdge(*this);
    for (i=blocks_to.first(); i; blocks_to.next(i))
      blocks_to.contents(i)->RemoveInEdge(*this);
    sprintf(buff, "A%d", id);
    temp = buff;
    net->RemovePEintrag(temp);
  }
  else {
    from->RemoveOutPEdge(*this);
    to->RemoveInPEdge(*this);
    for (i=blocks_from.first(); i; blocks_from.next(i))
      blocks_from.contents(i)->RemoveOutPEdge(*this);
    for (i=blocks_to.first(); i; blocks_to.next(i))
      blocks_to.contents(i)->RemoveInPEdge(*this);
    sprintf(buff, "A%d", id);
    temp = buff;
    net->RemoveEintrag(temp);
  }
  if (--active_objects == 0)
    edge_counter = 1;
}  // Edge::~Edge

Edge *
Edge::asEdge() const 
{
  return (Edge *) this;
}  // Edge::asEdge

HLEdge *
Edge::asHLEdge() const
{
  cerr << "brutal cast error! Attempt Edge -> HLEdge\n";
  exit(1);
  return NULL;
}  // Edge::asHLEdge
	
int
Edge::Id() const
{
  return id;
}  // Edge::Id

void
Edge::ReadStandard(istream &f)
{
  int id_from, id_to;
  char c;

  f >> id_from >> c >> id_to;
  if (phantom_flag) {
    from = net->GetPFirstFromId(id_from, c);
    to = net->GetPSecondFromId(id_to, c);
  }  // if Phantomkante
  else {  // normale Kante
    from = net->GetFirstFromId(id_from, c);
    to = net->GetSecondFromId(id_to, c);
  }  // else, normale Kante
  if (!to || !from) {  // Fehler eins von beiden existiert nicht
    if (!from) 
      cerr << "Vertex with Id " << id_from << " does not exist" << endl;
    if (!to) 
      cerr << "Vertex with Id " << id_to << " does not exist" << endl;
  }  // if einer oder beide Knoten unbekannt
  else {
    if (phantom_flag) {
      from->SetOutPEdge(*this);
      to->SetInPEdge(*this);
    }  // if Phantomkante
    else {  // normale Kante
      from->SetOutEdge(*this);
      to->SetInEdge(*this);
    }  // else, normale Kante
  }  // else, alles Roger
}  // Edge::ReadStandard

void
Edge::ReadOptional(istream &f)
{
  NetObject::ReadOptional(f);
  char c;
  f >> c;
  char dummy;
  switch (c) {
    case 'w':
      f >> weight;
      break;
    case 'n':
      f >> rx_weight >> dummy >> ry_weight;
      break;
    case 'J':
      f >> splinex >> dummy >> spliney;
      break;
    default:
      f.putback(c);
  }  // switch ber c
}  // Edge::ReadOptional

void
Edge::Write(ostream &f) const
{
  if (from->IsA("Transition") || from->IsA("PTransition"))
    f << from->Id() << '<' << to->Id();
  else
    f << from->Id() << '>' << to->Id();
  if (weight != 1) 
    f << 'w' << weight;
  if (rx_weight != net->DefaultEdgeWX() || 
    ry_weight != net->DefaultEdgeWY()) 
    f << 'n' << rx_weight << '@' << ry_weight;
  if(splinex != 0 || spliney != 0)
    f << 'J' << splinex << '@' << spliney;
  NetObject::Write(f);
}  // Edge::Write

void
Edge::AddBlockSrc(Block &b)
{
  blocks_from[b.Name()] = &b;  // von dieser Kante zum Block
  b.SetOutEdge(*this);  // vom Block zu dieser Kante
}  // Edge::AddBlockSrc

void
Edge::AddBlockDest(Block &b)
{
  blocks_to[b.Name()] = &b;  // vom Block zu dieser Kante
  b.SetInEdge(*this);  // von dieser Kante zum Block
}  // Edge::AddBlockDest

void
Edge::ResetBlockSrc(void)
  // Quellblockverweis lschen
{
  for (Pix i=blocks_from.first(); i; blocks_from.next(i))
    blocks_from.contents(i)->RemoveOutEdge(*this);
  blocks_from.clear();
}  // Edge::ResetBlockSrc

void
Edge::ResetBlockDest(void)
  // Zielblockverweis lschen
{
  for (Pix i=blocks_to.first(); i; blocks_to.next(i))
    blocks_to.contents(i)->RemoveInEdge(*this);
  blocks_to.clear();
}  // Edge::ResetBlockDest

int
Edge::ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[])
{
  if (argc != 1) {
    Tcl_AppendResult(ip, "wrong #args for Edge get", 
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl fr Edge
  if (!strcmp(argv[0], "Src"))
    Result(ip, from->Name());
  else if (!strcmp(argv[0], "Dest"))
    Result(ip, to->Name());
  else if (!strcmp(argv[0], "BlockSrc")) {
    char buff[200];
    for (Pix i = blocks_from.first(); i; blocks_from.next(i))  {
      //sprintf(buff, "B%d", blocks_from.contents(i)->Id());
      strcpy(buff, blocks_from.contents(i)->Name());
      Tcl_AppendElement(ip, buff);
    }  // for i ber die Blcke
  }  // if BlockSrc
  else if (!strcmp(argv[0], "BlockDest")) {
    char buff[200];
    for (Pix i = blocks_to.first(); i; blocks_to.next(i))  {
      //sprintf(buff, "B%d", blocks_to.contents(i)->Id());
      strcpy(buff, blocks_to.contents(i)->Name());
      Tcl_AppendElement(ip, buff);
    }  // for i ber die Blcke
  }  // if BlockDest
  else if (!strcmp(argv[0], "Weight"))
    sprintf(ip->result, "%d", weight);
  else if (!strcmp(argv[0], "WeightX"))
    sprintf(ip->result, "%d", rx_weight );
  else if (!strcmp(argv[0], "WeightY"))
    sprintf(ip->result, "%d", ry_weight);
  else if (!strcmp(argv[0], "SplineX"))
    sprintf(ip->result, "%d", splinex);
  else if (!strcmp(argv[0], "SplineY"))
    sprintf(ip->result, "%d", spliney);
  else
    return NetObject::ObjectGet(ip, argc, argv);
  return TCL_OK;
}  // Edge::ObjectGet

int
Edge::ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[])
{
  int w = -1;

  if (argc < 2) {
    Tcl_AppendResult(ip, "wrong #args for Edge set",
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl fr Edge set
  Tk_ArgvInfo parse_table[] = {
    {"Weight", TK_ARGV_INT, (char *) NULL, (char *) &w,
     "Weight of arc"},
    {"WeightX", TK_ARGV_INT, (char *) NULL, (char *) &rx_weight,
     "Relativ x-position of weight annotation"},
    {"WeightY", TK_ARGV_INT, (char *) NULL, (char *) &ry_weight,
     "Relativ y-position of weight annotation"},
    {"SplineX", TK_ARGV_INT, (char *) NULL, (char *) &splinex,
     "Absolut x-position of splinemiddle"},
    {"SplineY", TK_ARGV_INT, (char *) NULL, (char *) &spliney,
     "Absolut y-position of splinemiddle"},
    {NULL, TK_ARGV_END, NULL, NULL, NULL}};
  if (Tk_ParseArgv(ip, NULL, &argc, argv, parse_table, 
    TK_ARGV_DONT_SKIP_FIRST_ARG | TK_ARGV_NO_ABBREV) != TCL_OK)
    return TCL_ERROR;
  if (w != -1)
    SetWeight(w);
  return NetObject::ObjectSet(ip, argc, argv);
}  // Edge::ObjectSet

  //*** Dateiende Edge.cc ***

