// -------------------------------------------------------------------------
// Programm : HIPS Highlevel Petrinetz Simulator/Editor 
// ----------------------------------------------------
// Datei    : Net.h
// Funktion : Defintion fuer einfache Stellen/Transitionssysteme
// Author   : Tobias Himstedt
//            Diplomarbeit 1994 
//            Universitaet Hildesheim
//            Institut fuer Theoretische Informatik
//            Prof. Dr. Best
// -------------------------------------------------------------------------
// Burkhard Bieber 1995/96
// Cecile Bui Thanh: added asynchronous links (2001)
// last-update: Dec. 2001

#ifndef __NET_H__
#define __NET_H__

  //**********
  // includes
  //**********

#include <sys/time.h>
#include <iostream>
#include <math.h>
#include <tcl.h>
#include <tk.h>
#include "Block.h"
#include "Box.h"
#include "Place.h"
#include "Transition.h"
#include "PTransition.h"
#include "NetFileOrder.h"
#include "../include/Socket.h"
#include "Link.h"
extern "C" {
#include <string.h>
}

  //************
  // Konstanten
  //************
const int DefaultLLSize = 26;

  //*********
  // Klassen
  //*********

class HLNet;  // Forward fuer Cast-Warnung

class Net {
    // Kommentare und Titel zum Netz
  struct Title {
      // Datenelemente
    int x, y;  // Position
    String s;  // der Text
    String font;  // der Font
    static int counter;  // Zaehler aller Titel
      // Konstruktoren
    Title(const String &g, int xp, int yp) : x(xp), y(yp), s(g), font("NULL") {};
    Title() : x(0), y(0), s(), font("NULL") {};
  };  // struct Title
  private:
      // Methoden fuer den Netzdateikopf
    virtual int FormatIsOk(const String &format) const;
    virtual const String &Format() const;
      // einen neuen Netzdateiabschnitt aufnehmen
    void NewFileOrder(const String &sn, const String &ln,
      int of, int s, int e, int l);
      // Netzdateiabschnitt suchen
    int ExistsFileOrder(const String &sn);
  protected:
    static int net_counter;
      // TCL-Schnittstelle
    static void Destroy(ClientData cd);
    static int CommandBase(ClientData cd, Tcl_Interp *ip, int argc,
      CONST char *argv[]);
    virtual int ObjectCommand(Tcl_Interp *ip, int argc, CONST char *argv[]);
    int CalcOffset(Tcl_Interp *ip, const String &t1, const String &t2,
      int x1, int y1, int x2, int y2) const;
    int CalcOffset(Tcl_Interp *ip, const String &t1, const String &t2,
      int x1, int y1, int xm, int ym, int x2, int y2);
    int TranslateNorm(Tcl_Interp *ip,
      int x1, int y1, int x2, int y2, int up) const;
      // Lesemethoden
    virtual void ReadBlockDefaults(istream &f);
    virtual void ReadPlaceDefaults(istream &f);
    virtual void ReadTransitionDefaults(istream &f);
    virtual void ReadEdgeDefaults(istream &f);
      // ---------------------------------------------------------------
      //  Friends
      // ---------------------------------------------------------------
      // Stream Ausgabe/Eingabe
    friend ostream &operator <<(ostream &f, const Net &n);
    friend istream &operator >>(istream &f, Net &n);
      // ---------------------------------------------------------------
      //  Datenelemente
      // ---------------------------------------------------------------
    double object_size;
    VHMap<String, Block *> blocks;              // Menge der Bloecke
    VHMap<String, Box *> boxes;
    VHMap<String, Place *> places;              // Menge der Stellen
    VHMap<String, Transition *> transitions;    // Menge der Transitionen
    VHMap<String, PTransition *> ptransitions;  // Menge der Phantomtransitionen
    VHMap<String, Edge *> edges;                // Menge der Kanten
    VHMap<String, Edge *> pedges;               // Menge der Phantomkanten
    VHMap<int, Title> titles;                   // Menge der Titel
    VHMap<String, NetFileOrder *> fileorder;    // Menge der Dateikomponenten
    VHSet<String> activeTransitions;            // Menge der aktive Transitionen
    VHMap<String,Link> declared_links;          // set of declared asynchronous links
    VHMap<int, int> Blockkonvert;
    VHMap<int, int> Placekonvert;
    VHMap<int, int> Transkonvert;
    VHMap<int, int> PTranskonvert;

    Socket *Verbindungen[50];
    int notsave[50];
    int count;
    int stillsave;

      // Netztyp, anzuzeigende Knoten
    int net_type_switch;
    int sync_trans_switch;
    int phantom_trans_switch;
      // Defaults fuer das Netz
      // 1. Bloecke
    int rx_block_name, ry_block_name;
    int rx_block_mean, ry_block_mean;
    int block_thickness;
      // 2. Stellen
    int rx_place_name, ry_place_name;
    int rx_place_mean, ry_place_mean;
    int place_thickness;
      // 3. Transitionen / Phantomtransitionen
    int rx_trans_name, ry_trans_name;
    int rx_trans_mean, ry_trans_mean;
    int transition_thickness;
      // 4. Kanten / Phantomkanten
    int rx_edge_weight, ry_edge_weight;
    int edge_thickness;
      // 5. Fonts
    String NameP;
    String NameT;
    String MeaningP;
    String MeaningT;
    String Comments;
    String AnnotationsA;
    String ActionTermT;
    String ValueTermT;
    String TypeP;
    Net *clipboard;
    Net *clipboard_temp;
    String SEL;
    int with_time;
    String protokoll;
    timeval summe;
    int defaultTrans;
    double def_earliest;
    double def_latest;
      // ---------------------------------------------------------------
      //  Modifizierende Funktionen
      // ---------------------------------------------------------------
    void Read(istream &f);
    void EatComment(istream &f);
    int  ReadHeader(istream &f);
    void ReadBlock(istream &f);
    void ReadBox(istream &f);
    void ReadPlace(istream &f);
    void ReadTransition(istream &f);
    void ReadPTransition(istream &f);
    void ReadEdge(istream &f);
    void ReadPEdge(istream &f);
	void ReadLink(istream &f);
    void ReadTitle(istream &f);
    int  ReadBoxHeader(istream &f);
    void RecalcBlockVisRec(Block *bl);
    virtual void copyBlock(VHMap<String, Transition *> *Trans,
              VHMap<String, Place *> *Stellen,
              VHMap<String, Block *> *Blocks,
              VHMap<String, PTransition *> *PTrans,
              VHMap<String, Box *> *Boxes,
              String fromBlock, String toBlock);
    virtual void pasteBlock(VHMap<String, Transition *> *Trans,
              VHMap<String, Place *> *Stellen,
              VHMap<String, Block *> *Blocks,
              VHMap<String, PTransition *> *PTrans,
              VHMap<String, Box *> *Boxes,
              String fromBlock, String toBlock);
    void moveBlock(String block, VHSet<String> &moved, int x, int y, int all);
    void moveBox(String box, VHSet<String> &moved, int x, int y, int all);
    virtual void Parallel(String Block1, String Block2, String name);
    virtual void Sequence(String Block1, String Block2, String name);
    virtual void Choice(String Block1, String Block2, String name);
    virtual void Substitution(String Box, String Transition);
    String TakeInBox(int c, String kette[]);
    void   AddToBox(int c, String kette[], String b);
    void   TakeInBoxRek(String box, String block, Box *ptr);
      // ---------------------------------------------------------------
      //  Konstante Funktionen
      // ---------------------------------------------------------------
    void Write(ostream &f) const;
    void WriteBoxes(ostream &f) const;
    virtual Block *AllocBlock() const;
    virtual Box *AllocBox() const;
    virtual Place *AllocPlace() const;
    virtual Transition *AllocTransition() const;
    virtual PTransition *AllocPTransition() const;
    virtual Edge *AllocEdge() const;
    virtual Edge *AllocEdgeFT(Vertex &f, Vertex &t) const;
    virtual Edge *AllocPEdge() const;
    virtual Edge *AllocPEdgeFT(Vertex &f, Vertex &t) const;
  public:

    /* ------------ asynchronous links ------------ */
    
	// get set of declared asynchronous links
    VHMap<String,Link> getDeclaredLinks() const;

	// clear set of declared asynchronous links
    void clearDeclaredLinks();
    
	// add a new link. The return is true iff the link has
	// been added (every link names are different)
	boolean addLink(String lname, String l_type);

	// convert a String into a set of Val.
	// this function uses StringToSet(String,Set<Val>), add(Tupel,String)
	// which have been copied from HLPlace
	void stringToType (String string_type, VHSet<Val> &type);
	
    void StringToSet(String s, SET<Val> &v) const;
    void StringRangeAppendSet(String range, SET<Val> &b) const;
    void add(Tupel va, String s, SET<Val> &type);
		
	boolean correct_link_name(String s);
	
      // ---------------------------------------------------------------
      //  Konstante Funktionen
      // ---------------------------------------------------------------
    virtual HLNet *asHLNet() const;
      // Liefert den Typ des Netzes, alt=0, neu=1
    int GetNetTypeSwitch(void);
      // Die jeweilige Menge der Netzobjekte zurueckliefern.
    const VHMap<String, Block *> &Blocks() const;
    const VHMap<String, Box *> &Boxes() const;
    const VHMap<String, Place *> &Places() const;
    const VHMap<String, Transition *> &Transitions() const;
    const VHMap<String, PTransition *> &PTransitions() const;
      // Liefert die Menge der Wurzelbloecke zurueck
    const VHSet<String> RootBlocks();
      // Liefert die Menge der aktiven Transitionen zurueck
    virtual VHSet<String> ActiveTransitions();
    virtual VHSet<String> GetCurrent();
      // fuer Vertex (also Stellen und Transitionen) zur Blocksuche
    Block *GetBlockFromId(int id);
    Block *GetBlockFromName(String name);
    String GetOldestFather(String &n);
    Box   *GetBoxFromName(String name);
      // fuer Phantomtransitionen zur Transitionssuche
    Transition *GetTransitionFromId(int id);
    Transition *FindTransition(String name);
    int ContainTransition(String name) {return transitions.contains(name);};
    int ContainPlace(String name) {return places.contains(name);};
    int ContainBlock(String name) {return blocks.contains(name);};
    int ContainBox(String name)   {return boxes.contains(name);};
    int ContainPTransition(String name) {return ptransitions.contains(name);};
    PTransition *FindPTransition(String name);
    Place *FindPlace(String name);
      // fuer Kanten
    Vertex *GetFirstFromId(int id, char c);
    Vertex *GetSecondFromId(int id, char c);
    Edge *FindEdge(const String &from, const String &to);
      // fuer Phantomkanten
    Vertex *GetPFirstFromId(int id, char c);
    Vertex *GetPSecondFromId(int id, char c);
    Edge *FindPEdge(const String &from, const String &to);
      // Defaults abholen
    double ObjectSize() const;
    int WithTime() const {return with_time;};
    int DefaultTrans() const {return defaultTrans;};
    double Earliest() const {return def_earliest;};
    double Latest() const {return def_latest;};
    void DefaultBlockName(int &px, int &py) const;
    int DefaultBlockNameX() const;
    int DefaultBlockNameY() const;
    void DefaultBlockMean(int &px, int &py) const;
    int DefaultBlockMeanX() const;
    int DefaultBlockMeanY() const;
    void DefaultPlaceName(int &px, int &py) const;
    int DefaultPlaceNameX() const;
    int DefaultPlaceNameY() const;
    void DefaultPlaceMean(int &px, int &py) const;
    int DefaultPlaceMeanX() const;
    int DefaultPlaceMeanY() const;
    void DefaultTransName(int &px, int &py) const;
    int DefaultTransNameX() const;
    int DefaultTransNameY() const;
    void DefaultTransMean(int &px, int &py) const;
    int DefaultTransMeanX() const;
    int DefaultTransMeanY() const;
    void DefaultEdgeWeight(int &px, int &py) const;
    int DefaultEdgeWX() const;
    int DefaultEdgeWY() const;
    int BlockKonvert(int id);
    int PlaceKonvert(int id);
    int TransKonvert(int id);
    int PTransKonvert(int id);
    void BlockKonvert(int key, int value);
    void PlaceKonvert(int key, int value);
    void TransKonvert(int key, int value);
    void PTransKonvert(int key, int value);
      // ---------------------------------------------------------------
      //  Modifizierende Funktionen
      // ---------------------------------------------------------------
    void ChangeBlockName(const String &old_name, const String &new_name);
    void ChangePlaceName(const String &old_name, const String &new_name);
    void ChangeTransName(const String &old_name, const String &new_name);
    void ChangePTransName(const String &old_name, const String &new_name);
      // Neubestimmung der Wurzelbloecke im Netz
    void RecalcRootBlocks(void);
      // Neubestimmung der Sichbarkeiten fuer die Netzobjekte
    void RecalcBlockVis(void);
      // Generiert einen neuen Block
    virtual const String &NewBlock(void);
      // Generiert eine neue Box
    virtual const String &NewBox(void);
      // Generiert eine neue Stelle
    virtual const String &NewPlace(void);
      // Generiert eine neue Stelle
    virtual void NewPlace(String name);
      // Generiert eine neue Transition
    virtual const String &NewTransition(void);
      // Generiert eine neue Phantomtransition
    virtual const String &NewPTransition(void);
      // Generiert eine neue Kante
    virtual String NewEdge(const String &from, const String &to);
      // Generiert eine neue Phantomkante
    virtual String NewPEdge(const String &from, const String &to);
      // Entfernt Block p und alle Verweise auf ihn
    virtual void RemoveBlock(const String &b_name);
      // Enfernt Stelle p und alle zu und wegfuehrenden Kanten
    virtual void RemovePlace(const String &p_name);
      // Entfernt Transition t und alle zu und wegfuehrenden Kanten
    virtual void RemoveTransition(const String &t_name);
      // Entfernt Phantomtransition t und alle zu und wegfuehrenden Kanten
    virtual void RemovePTransition(const String &t_name);
      // Entfernt die Kante
    virtual void RemoveEdge(Edge &e);
      // Entfernt die Phantomkante
    virtual void RemovePEdge(Edge &e);
      // Loescht alle Bloecke und die Verweise auf diese Bloecke
    void DeleteBlocks(void);
      // Loescht all Boxen und die Verweise auf die Boxen
    void DeleteBoxes(void);
    virtual void DeleteBox(String &name);
    void ReadBoxes(istream &f);
      // Loescht alle Stellen und alle zu und wegfuehrenden Kanten und alle Tokens
    void DeletePlaces(void);
    void RemoveEintrag(String &n);
    void RemovePEintrag(String &n);
      // Loescht alle Transitionen und alle zu und wegfuehrenden Kanten
    void DeleteTransitions(void);
      // Loescht alle Phantomtransitionen und alle zu und wegfuehrenden Kanten
    void DeletePTransitions(void);
      // Loescht die Informationen ueber die Dateikomponentenreihenfolge
    void DeleteFileOrder(void);
      // Schmeisst alles weg
    void DeleteAll(void);
      // Transition t wird geschaltet 
    void FireTransition(const String &t);
    virtual void loadmodul(CONST char *argv, boolean inbox = true, String b = "", boolean clear_links = false);
    virtual void savemodul(CONST char *argv);
      // Netzfunktionen f"ur die Schaltungsserver
    void ChangeTyp(char *t1, char *t2);
    void ChangeMarking(char *t1, char *t2);
    void ChangeValueTerm(char *t1, char *t2);
    void EstablishConnection(void);
    virtual void copy(int c, String kette[]);
    virtual void cut(int c, String kette[]);
    virtual void paste(void);
    void AppSuffix(String Suffix, String Box);
      // ---------------------------------------------------------------
      //  Konstruktor/Destruktor
      // ---------------------------------------------------------------
    Net();
    virtual ~Net();
    static int CreateLow(ClientData cd, Tcl_Interp *ip, int objc, 
      Tcl_Obj *CONST objv[]);
};  // class Net

  //************
  // Konstanten
  //************

const String PTNet = "PetriBox";

  //******************************
  // inlines fuer die Klasse "Net"
  //******************************

inline VHMap<String,Link>
Net::getDeclaredLinks() const
{
  return declared_links;
}// HLNet::getDeclaredLinks

  // Formatgedoens
inline int 
Net::FormatIsOk(const String &f) const
{
  return (f == "PetriBox") || (f == "PTNet");
}  // Net::FormatIsOk

inline const String &
Net::Format() const
{
  return PTNet;
}  // Net::Format

inline int
Net::GetNetTypeSwitch(void)
  // Liefert den Typ zurueck, alt=0, neu=1
{
  return net_type_switch;
}  // Net::GetNetTypeSwitch

  // Mengen von Netzobjekten abliefern

inline const VHMap<String, Block *> &
Net::Blocks() const
{
  return blocks;
}  // Net::Blocks

inline const VHMap<String, Box *> &
Net::Boxes() const
{
  return boxes;
}  // Net::Boxes

inline const VHMap<String, Place *> &
Net::Places() const
{
  return places;
}  // Net::Places

inline const VHMap<String, Transition *> &
Net::Transitions() const
{
  return transitions;
}  // Net::Transitions

inline const VHMap<String, PTransition *> &
Net::PTransitions() const
{
  return ptransitions;
}  // Net::PTransitions

  // allgemeines

inline double
Net::ObjectSize() const
{
  return object_size;
}  // Net::ObjectSize

  // Defaults fuer Bloecke

inline void 
Net::DefaultBlockName(int &px, int &py) const
{
  px = rx_block_name;
  py = ry_block_name;
}  // Net::DefaultBlockName

inline int
Net::DefaultBlockNameX() const
{
  return rx_block_name;
}  // Net::DefaultBlockNameX

inline int
Net::DefaultBlockNameY() const
{
  return ry_block_name;
}  // Net::DefaultBlockNameY

inline void 
Net::DefaultBlockMean(int &px, int &py) const
{
  px = rx_block_mean;
  py = ry_block_mean;
}  // Net::DefaultBlockMean

inline int
Net::DefaultBlockMeanX() const
{
  return rx_block_mean;
}  // Net::DefaultBlockMeanX

inline int
Net::DefaultBlockMeanY() const
{
  return ry_block_mean;
}  // Net::DefaultBlockMeanY

  // Defaults fuer Stellen

inline void 
Net::DefaultPlaceName(int &px, int &py) const
{
  px = rx_place_name;
  py = ry_place_name;
}  // Net::DefaultPlaceName

inline int
Net::DefaultPlaceNameX() const
{
  return rx_place_name;
}  // Net::DefaultPlaceNameX

inline int
Net::DefaultPlaceNameY() const
{
  return ry_place_name;
}  // Net::DefaultPlaceNameY

inline void 
Net::DefaultPlaceMean(int &px, int &py) const
{
  px = rx_place_mean;
  py = ry_place_mean;
}  // Net::DefaultPlaceMean

inline int
Net::DefaultPlaceMeanX() const
{
  return rx_place_mean;
}  // Net::DefaultPlaceMeanX

inline int
Net::DefaultPlaceMeanY() const
{
  return ry_place_mean;
}  // Net::DefaultPlaceMeanY

  // Defaults fuer Transitionen / Phantomtransitionen

inline void 
Net::DefaultTransName(int &px, int &py) const
{
  px = rx_trans_name;
  py = ry_trans_name;
}  // Net::DefaultTransName

inline int
Net::DefaultTransNameX() const
{
  return rx_trans_name;
}  // Net::DefaultTransNameX

inline int
Net::DefaultTransNameY() const
{
  return ry_trans_name;
}  // Net::DefaultTransNameY

inline void 
Net::DefaultTransMean(int &px, int &py) const
{
  px = rx_trans_mean;
  py = ry_trans_mean;
}  // Net::DefaultTransMean

inline int
Net::DefaultTransMeanX() const
{
  return rx_trans_mean;
}  // Net::DefaultTransMeanY

inline int
Net::DefaultTransMeanY() const
{
  return ry_trans_mean;
}  // Net::DefaultTransMeanY

  // Defaults fuer Kanten / Phantomkanten

inline void 
Net::DefaultEdgeWeight(int &px, int &py) const
{
  px = rx_edge_weight;
  py = ry_edge_weight;
}  // Net::DefaultEdgeWeight

inline int 
Net::DefaultEdgeWX() const
{
  return rx_edge_weight;
}  // Net::DefaultEdgeWX

inline int 
Net::DefaultEdgeWY() const
{
  return ry_edge_weight;
}  // Net::DefaultEdgeWY

  //******************
  // Sonstige Inlines
  //******************

inline void  
GetNormal(const int x1, const int y1, const int x2, const int y2,
  double &nx, double &ny, double &sx, double &sy, double &r)
{
  r = sqrt((double)(x2-x1) * (x2-x1) + (y2-y1) * (y2-y1));
  nx= (x2-x1)/r, ny= (y2-y1)/r;
  sx = -ny / 2.0;
  sy = nx / 2.0;
}  // GetNormal

#endif  // #ifndef __NET_H__

  //*** Dateiende Net.h ***

