// NetObject.h
// Tobias Himstedt 1995
// Burkhard Bieber 1995/96
// last-update: 12.03.96

#ifndef __NETOBJECT_H__
#define __NETOBJECT_H__

  //**********
  // includes
  //**********

#include <fstream>
#include <tcl.h>
#include <tk.h>
#include "../include/VHSet.h"
#include "../include/Utils.h"

  // verschiedene Zustnde fr Netzobjekte auf dem Schirm
enum BlockVisType {EXPOSED, HIDDEN, EXPOSED_CONTENTS};

  //********************
  // Forwards fr Casts
  //********************

class Vertex;
class Edge;
class Transition;
class Place;
class Token;

  //*********
  // Klassen
  //*********

class NetObject {
  protected:
      // ---------------------------------------------------------------
      //  Datenelemente
      // ---------------------------------------------------------------
    String type;
    VHSet<String> base_classes;
    int visibility;  // fr Kanten
    BlockVisType block_vis;  // Sichtbarkeit fr Block auf/zu
      // ---------------------------------------------------------------
      //  Modifizierende Funktionen
      // ---------------------------------------------------------------
    void SetType(const String &new_type);
      // ---------------------------------------------------------------
      //  Konstante Funktionen
      // ---------------------------------------------------------------
    virtual void Write(ostream &f) const;
    virtual void ReadStandard(istream &f) = 0;
    virtual void ReadOptional(istream &f);
    NetObject(const String &Type);
  public:
      // ---------------------------------------------------------------
      //  Konstante Funktionen
      // ---------------------------------------------------------------
    const String &ClassType(void) const;
    virtual int Id() const  = 0;
    int IsA(const String &Type) const;
    BlockVisType GetBlockVis(void) const;
    void SetVisibility(int i) {visibility = i;};
    int GetVisibility() const;
      // ---------------------------------------------------------------
      //  Modifizierende Funktionen
      // ---------------------------------------------------------------
    void SetBlockVis(BlockVisType new_block_vis);
      // ---------------------------------------------------------------
      //  Operatoren
      // ---------------------------------------------------------------
    void operator =(const NetObject &k);
    friend ostream &operator <<(ostream &f, const NetObject &t);
    friend istream &operator >>(istream &f, NetObject &t);
      // ---------------------------------------------------------------
      //  Konstruktor/Destruktor
      // ---------------------------------------------------------------
    NetObject();
    NetObject(const NetObject &n);
    virtual ~NetObject() {};
    int ObjectCommand(Tcl_Interp *ip, int argc, CONST char *argv[]);
    virtual int ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[]);
    virtual int ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[]);
};  // class NetObject

  //***********************
  // Inlines fr NetObject
  //***********************

inline
NetObject::NetObject() : type("NetObject"),
  base_classes(), visibility(1), block_vis(EXPOSED)
{
  // Nix
}  // NetObject::NetObject

inline
NetObject::NetObject(const String &Type) : type(Type),
  base_classes(5), visibility(1), block_vis(EXPOSED)
{
  // Nix
}  // NetObject::NetObject

inline
NetObject::NetObject(const NetObject &n) : type(n.type), 
  base_classes(n.base_classes), visibility(n.visibility),
  block_vis(n.block_vis)
{
  // Nix
}  // NetObject::NetObject

inline void
NetObject::SetType(const String &new_type) 
{
  type = new_type;
}  // NetObject::SetType

inline int
NetObject::GetVisibility() const
{
  return visibility;
}  // NetObject::GetVisibility

inline const String &
NetObject::ClassType(void) const  
{
  return type;
}  // NetObject::ClassType

inline BlockVisType
NetObject::GetBlockVis(void) const
{
  return block_vis;
}  // NetObject::GetBlockVis

inline void
NetObject::SetBlockVis(BlockVisType new_block_vis)
{
  block_vis = new_block_vis;
}  // NetObject::SetBlockVis

inline void
NetObject::operator =(const NetObject &a)
{
  if (this == &a)
    return; 
  type = a.type;
  visibility = a.visibility;
  block_vis = a.block_vis;
}  // NetObject::operator =

inline int
NetObject::IsA(const String &Type) const
{
  return type == Type || base_classes.contains(Type);
}  // NetObject::IsA

#endif  // #ifndef __NETOBJECT_H__
 	
  //*** Dateiende NetObject.h ***

