// PTransition.cc
// Tobias Himstedt 1995
// Burkhard Bieber 1995/96
// Christian Stehno: changed ip->result to use strdup
// last-update: 30.05.2000

  //**********
  // includes
  //**********

#pragma implementation 

#include <strstream>
#include "PTransition.h"
#include "Net.h"

  //*******************************
  // Inits fr statische Variablen
  //*******************************

int PTransition::ptrans_counter = 1;
int PTransition::active_objects = 0;

  //**************************
  // Methoden fr PTransition
  //**************************

PTransition::~PTransition()
  // Destruktor
{
  if (--active_objects == 0)
    ptrans_counter = 1;
  for(Pix i = synctrans.first(); i; synctrans.next(i)) {
    synctrans.contents(i)->RemovePTrans(name);
    // Delete(this,i,name);
  }
}  // PTransition::~PTransition

PTransition::PTransition(const PTransition &t) : Vertex(t),
  refined(t.refined), synctrans(t.synctrans)
  // Konstruktor
{
  net->DefaultTransName(rx_name, ry_name);
  net->DefaultTransMean(rx_mean, ry_mean);
  id = ptrans_counter++;
  active_objects++;
  name = PreFix() + IntToString(id);
}  // PTransition::PTransition

PTransition::PTransition(const Net &n, const String &Type) : Vertex(n, Type),
  refined(no), synctrans((Transition *) NULL, 5)
  // Konstruktor
{
  net->DefaultTransName(rx_name, ry_name);
  net->DefaultTransMean(rx_mean, ry_mean);
  id = ptrans_counter++;
  base_classes.add("Vertex");
  active_objects++;
  name = PreFix() + IntToString(id);
}  // PTransition::PTransition

void
PTransition::RemoveSyncTrans(String& name)
  // Die Synchrontransition name wird aus der Liste der
  // Synchrontransitionen gel"oscht
{
  synctrans.del(name);
}  // PTransition::RemoveSyncTrans

const PTransition &
PTransition::operator =(PTransition const& t)
{
  if (Name() == t.Name())
    return *this;
  Vertex::operator =(t);
  synctrans = t.synctrans;
  return *this;
}

void
PTransition::Write(ostream &f) const
  // Ausgabe der Angaben
{
  Vertex::Write(f);
  if (rx_name != net->DefaultTransNameX() || 
    ry_name != net->DefaultTransNameY()) 
    f << 'n' << rx_name << '@' << ry_name;
  if (rx_mean != net->DefaultTransMeanX() || 
    ry_mean != net->DefaultTransMeanY())
    f << 'a' << rx_mean << '@' << ry_mean;
  if ((net->GetNetTypeSwitch() == 1) && !synctrans.empty()) {
    f << "P\"(";
    int tr_count = 1;
    for (Pix i = synctrans.first(); i; synctrans.next(i)) {
      if (tr_count != 1)
        f << ",";
      f << synctrans.contents(i)->Id();
      tr_count++;
    }  // for i ber die SyncTrans-Verweise
    f << ")\"";
  }  // if Angaben vorhanden
}  // PTransition::Write

void
PTransition::ReadOptional(istream &f)
  // optionale Angaben lesen
{
  char c;
  String syncs_str;
  int sync_id;
  Transition *tr;

  Vertex::ReadOptional(f);
  f.get(c);
  if (c == 'r')
    refined = yes;
  else if (c == 'P') {
    f >> c;
    readline(f, syncs_str, '"');
    istrstream s((const char*)syncs_str);
    s >> c;  // Klammer auf
    while (s && (c != ')')) {
      s >> sync_id >> c;
      tr = net->GetTransitionFromId(sync_id);
      if (sync_id != 0) {
        synctrans[tr->Name()] = tr;
        tr->AddPTransition(Name(), this);
      }  // if Transition gefunden
      else {
        cerr << "error: PTransition::ReadOptional (Id=" << id 
          << ") : transition-id not found (" << sync_id << ")" << endl;
      }  // else, Transition nicht gefunden
    }  // while nicht am Ende
  }  // else if Phantomverweis
  else
    f.putback(c);
}  // PTransition::ReadOptional

VHSet<String>
PTransition::InputPlaces(void) const
  // Menge der Eingangsstellen liefern
{
  VHSet<String> ret;

  for (Pix i = incoming_edges.first(); i; incoming_edges.next(i))
    ret.add(incoming_edges.key(i));
  return ret;
}  // PTransition::InputPlaces

VHSet<String>
PTransition::OutputPlaces(void) const
  // Menge der Ausgangsstellen liefern
{
  VHSet<String> ret;

  for (Pix i = outgoing_edges.first(); i; outgoing_edges.next(i))
    ret.add(outgoing_edges.key(i));
  return ret;
}  // PTransition::OutputPlaces

void
PTransition::ChangeMyName(const String &new_name) 
  // Umbennungen haben interne Konsequenzen bei der Verwaltung von Verweisen
{
  net->ChangePTransName(name, new_name);
  Vertex::ChangeMyName(new_name);
}  // PTransition::ChangeMyName

void
PTransition::AddSyncTransition(String name, Transition *ptr)
{
  synctrans[name] = ptr;
} // PTransition::AddSyncTransition

VHSet<String>
PTransition::SyncTrans(void)
{
  VHSet<String> SY;
  for(Pix i = synctrans.first(); i; synctrans.next(i)) {
    SY.add(synctrans.key(i));
  }
  return SY;
}  // PTransition::SyncTrans

int
PTransition::ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[])
  // TCL-Schnittstelle
{
  if (argc != 1) {
    Tcl_AppendResult(ip, "wrong #args for PTransition get",
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl fr PTransition get

  if (EQ(0, "InPlaces"))
    for (Pix i = incoming_edges.first(); i; incoming_edges.next(i))
      Tcl_AppendElement(ip, incoming_edges.key(i));

  else if (EQ(0, "OutPlaces"))
    for (Pix i = outgoing_edges.first(); i; outgoing_edges.next(i))
      Tcl_AppendElement(ip, outgoing_edges.key(i));

  else if (EQ(0, "Synchrons"))
    for (Pix i = synctrans.first(); i; synctrans.next(i))
      Tcl_AppendElement(ip, synctrans.key(i));

  else if (EQ(0, "Refined"))
    ip->result = strdup((refined == yes) ? "yes" : "no");
  else
    return Vertex::ObjectGet(ip, argc, argv);
  return TCL_OK;
}  // PTransition::ObjectGet

int 
PTransition::ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[])
  // TCL-Schnittstelle
{
  if (argc < 2) {
    Tcl_AppendResult(ip, "wrong #args for PTransition set",
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl fr Transition set
  if (EQ(0, "Synchrons")) {
    String para = argv[1];
    int zaehler = 0;
    while(para != "") {
      para = para.after(" ");
      zaehler++;
    }
    String kette;
    para = argv[1];
    synctrans.clear();
    for(int i = 0;i < zaehler;i++) {
      kette = (i == zaehler-1) ? para : para.before(" ");
      para = para.after(" ");
      if (net->ContainTransition(kette))
        AddSyncTransition(kette, net->FindTransition(kette));
    }
  }
  if (EQ(0, "Blocks")) {
    String para = argv[1];
    int zaehler = 0;
    while(para != "") {
      para = para.after(" ");
      zaehler++;
    }
    String kette;
    para = argv[1];
    blocks.clear();
    for(int i = 0;i < zaehler;i++) {
      kette = (i == zaehler-1) ? para : para.before(" ");
      para = para.after(" ");
      if (net->ContainBlock(kette))
        AddBlock(kette, net->GetBlockFromName(kette));
    }
  }
  if (EQ(0, "Box")) {
    box = "";
    boxPointer = (Box *) NULL;
    String para(argv[1]);
    if(net->ContainBox(para)) {
      setBox(para, net->GetBoxFromName(para));
    }
  }
  if (argc > 0)
    return Vertex::ObjectSet(ip, argc, argv);
  return TCL_OK;
}  // PTransition:ObjectSet
  //*** Dateiende PTransition.cc ***

