// PTransition.h
// Tobias Himstedt 1995
// Burkhard Bieber 1995/96
// last-update: 07.03.96

#ifndef __PTRANSITION_H__
#define __PTRANSITION_H__

#pragma interface

  //**********
  // includes
  //**********

class PTransition;

#include "Place.h"
#include "Transition.h"

  //************
  // Konstanten
  //************
const String PTransitionPrefix = "PT";

  //*********
  // Klassen
  //*********

  // Definition der Grundfunktionalitt einer Phantomtransition
class PTransition : public Vertex {
  protected:
    static int ptrans_counter;
    static int active_objects;
    yes_no_type refined;
    VHMap<String, Transition *> synctrans;
    virtual void ReadOptional(istream &f);
    virtual void Write(ostream &f) const;
    virtual const String &PreFix() const;
    virtual void ChangeMyName(const String &new_name);
  public:
      // ---------------------------------------------------------------
      //  Konstante Funktionen
      // ---------------------------------------------------------------
      // Menge der Eingangsstellen zurckgeben
    VHSet<String> InputPlaces(void) const;
      // Menge der Ausgangsstellen zurckgeben
    VHSet<String> OutputPlaces(void) const;
      // ---------------------------------------------------------------
      //  Konvertierungsfunktionen
      // ---------------------------------------------------------------
    virtual PTransition *asPTransition() const;
      // ---------------------------------------------------------------
      //  Modifizierende Funktionen
      // ---------------------------------------------------------------
    static void ResetMaxIdTo(int nr);
    virtual void SetMaxId(int id);
    void RemoveSyncTrans(String& name);
    const PTransition &operator =(PTransition const& t);
    void AddSyncTransition(String name, Transition *ptr);
    VHSet<String> SyncTrans(void);
      // ---------------------------------------------------------------
      //  Konstruktor/Destruktor
      // ---------------------------------------------------------------
    PTransition(const Net &n, const String &Type = "PTransition");
    PTransition(const PTransition &t);
    virtual ~PTransition();
      // TCL-Schnittstelle
    int ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[]);
    int ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[]);
};  // class PTransition

  //*************************
  // Inlines fr PTransition
  //*************************

inline const String &
PTransition::PreFix() const
  // Prefix des Namens
{
  return PTransitionPrefix;
}  // PTransition::PreFix

inline void
PTransition::SetMaxId(int nr)
  // Id-Verwaltung
{
  ptrans_counter = (nr>ptrans_counter) ? nr+1 : ptrans_counter;
}  // PTransition::SetMaxId

inline void
PTransition::ResetMaxIdTo(int nr)
  // Id-Verwaltung
{
  ptrans_counter = nr;
}  // PTransition::ResetMaxId

inline PTransition *
PTransition::asPTransition() const
  // Cast
{
  return (PTransition *) this;
}  // PTransition::asPTransition

#endif  // #ifndef __PTRANSITION_H__

  //*** Dateiende PTransition.h ***

