// Place.cc
// Tobias Himstedt 1995
// Burkhard Bieber 1995/96
// Christian Stehno: changed ip->result to use strdup (2000)
// Cecile Bui Thanh: updates for using VHSet<Place>
// last-update: Nov. 2001

  //**********
  // includes
  //**********
 
#pragma implementation

#include <ctype.h>
#include "Place.h"
#include "Net.h"

  //*******************************
  // Inits fr statische Variablen
  //*******************************

int Place::place_counter = 1;
int Place::active_objects = 0;

  //********************
  // Methoden fr Place
  //********************


Place::~Place()
{
  if (--active_objects == 0)
    place_counter = 1;
}  // Place::~Place

Place::Place(const Place &p) : Vertex(p),
  has_current(0), tokens(p.tokens),
  initial_tokens(p.initial_tokens), capacity(p.capacity),
  state(internal)
{
  id = place_counter++;
  String temp = PreFix() + IntToString(id);
  const VHMap<String, Place *> &Stellen = net->Places();
  while(Stellen.contains(temp)) {
    id++;
    ResetMaxIdTo(id);
    temp = PreFix() + IntToString(id);
  }
  name = temp;
//  name = PreFix()+IntToString(id);
  active_objects++;
}  // Place::Place

Place::Place(const Net &n, const String &Type) : Vertex(n, Type), 
  has_current(0), tokens(0),
  initial_tokens(0), capacity(-1),
  state(internal)
{
  net->DefaultPlaceName(rx_name, ry_name);
  net->DefaultPlaceMean(rx_mean, ry_mean);
  base_classes.add("Vertex");
  active_objects++;
  id = place_counter++;
  String temp = PreFix() + IntToString(id);
  const VHMap<String, Place *> &Stellen = net->Places();
  while(Stellen.contains(temp)) {
    id++;
    ResetMaxIdTo(id);
    temp = PreFix() + IntToString(id);
  }
  name = temp;
//  name = PreFix()+IntToString(id);
}  // Place::Place

void
Place::WriteMarking(ostream &f) const
{
  if (state == entry) {
    f << 'e';
  }  // if Entry-Stelle
  if (initial_tokens > 0)
    f << 'M' << initial_tokens;
  if (tokens > 0)
    f << 'm' << tokens;
}  // Place::WriteMarking

void
Place::Write(ostream &f) const
{
  Vertex::Write(f);
  if (rx_name != net->DefaultPlaceNameX() || 
    ry_name != net->DefaultPlaceNameY()) 
    f << 'n' << rx_name << '@' << ry_name;
  if (rx_mean != net->DefaultPlaceMeanX() || 
    ry_mean != net->DefaultPlaceMeanY())
    f << 'a' << rx_mean << '@' << ry_mean;
  WriteMarking(f);
  if (capacity != -1)
    f << 'k' << capacity;
  if (state == exit)
    f << 'x';
}  // Place::Write

void
Place::ReadOptional(istream &f)
{
  char c;

  Vertex::ReadOptional(f);
  f.get(c);
  switch (c) {
    case 'M':
      f >> initial_tokens;
      // if (!has_current)
      // tokens = initial_tokens;
      break;
    case 'm':
      f >> tokens;
      has_current = 1;
      break;
    case 'k':
      f >> capacity;
      break;
    case 'x':
      SetExit();
      break;
    case 'e':
      SetEntry(1);
      break;
    default:
      f.putback(c);
  }  // switch ber c
}  // Place::ReadOptional

void
Place::SetEntry(int read_flag)
{
  state = entry;
  if (!read_flag) {
    initial_tokens = 1;
    tokens = 1;
  }  // if not read from the file
}  // Place::SetEntry

void
Place::SetExit()
{
  state = exit;
  initial_tokens = 0;
}  // Place::SetExit

void
Place::ChangeMyName(const String &new_name)
{
  net->ChangePlaceName(name, new_name);
  //Vertex::ChangeMyName(new_name);
}  // Place::ChangeMyName

void  
Place::AddTokens(int num)  
{
  if (capacity == -1)
    tokens += num; 
  else
    if (tokens + num > capacity) {
      tokens = capacity;
      cerr << "Error: Place " << name << "capacity (=" << capacity 
        << ") exceeded\n";
    }  // if zuviele Tokens
    else
      tokens += num; 
}  // Place::AddTokens

int 
Place::ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[])
  // TCL-Schnittstelle
{
  if (argc != 1) {
    Tcl_AppendResult(ip, "wrong #args for Place get",
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl fr Place get

  else if (EQ(0, "Marking"))
    INTRES(tokens);

  else if (EQ(0, "InitMarking"))
    INTRES(initial_tokens);

  else if (EQ(0, "Capacity"))
    INTRES(capacity);

  else if (EQ(0, "State"))
    ip->result = strdup((state == entry ) ? "entry" : 
      (state == exit) ? "exit" : "internal");

  else
    return Vertex::ObjectGet(ip, argc, argv);
  return TCL_OK;
}  // Place::ObjectGet

int 
Place::ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[])
  // TCL-Schnittstelle
{
  if (argc < 2) {
    Tcl_AppendResult(ip, "wrong #args for Place set",
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl fr Place set
  if (EQ(0, "Blocks")) {
    String para = argv[1];
    int zaehler = 0;
    while(para != "") {
      para = para.after(" ");
      zaehler++;
    }
    String kette;
    para = argv[1];
    blocks.clear();
    for(int i = 0;i < zaehler;i++) {
      kette = (i == zaehler-1) ? para : para.before(" ");
      para = para.after(" ");
      if (net->ContainBlock(kette))
        AddBlock(kette, net->GetBlockFromName(kette));
    }
  }
  if (EQ(0, "Box")) {
    box = "";
    boxPointer = (Box *) NULL;
    String para(argv[1]);
    if(net->ContainBox(para)) {
      setBox(para, net->GetBoxFromName(para));
    }
  }
  int nt = 0;
  int c = -1;
  char *st = NULL;
  int dt = 0;
  Tk_ArgvInfo parse_table[] = {
    {"NewToken", TK_ARGV_INT, (char *) NULL, (char *) &nt,
     "Number of new tokens"},
    {"Marking", TK_ARGV_INT, (char *) NULL, (char *) &tokens,
     "Current Marking"},
    {"DelToken", TK_ARGV_INT, (char *) NULL, (char *) &dt,
     "Number of tokens to delete"},
    {"InitMarking", TK_ARGV_INT, (char *) NULL, (char *) &initial_tokens,
     "Initial Marking"},
    {"Capacity", TK_ARGV_INT, NULL, (char *) &c,
     "Capacity of Place"},
    {"State", TK_ARGV_STRING, NULL, (char *) &st,
     "State of Place"},
    {NULL, TK_ARGV_END, NULL, NULL, NULL}};
  if (Tk_ParseArgv(ip, NULL, &argc, argv, parse_table, 
    TK_ARGV_DONT_SKIP_FIRST_ARG) != TCL_OK)
    return TCL_ERROR;
  if (nt > 0)
    AddTokens(nt);
  if (c > -1)
    capacity = c;
  if (dt > 0)
    RemoveTokens(dt);
  if (st) 
    if (!strcmp(st, "entry"))
      state = entry;
    else if (!strcmp(st, "exit"))
      state = exit;
    else 
      state = internal;
  if (argc > 0)
    return Vertex::ObjectSet(ip, argc, argv);
  return TCL_OK;
}  // Place::ObjectSet

  //*** Dateiende Place.cc ***
	
