// Place.h
// Tobias Himstedt 1995
// Burkhard Bieber 1995/96
// Cecile Bui Thanh: updates for using VHSet<Place>
// last-update: Nov. 2001

#ifndef __PLACE_H__
#define __PLACE_H__

  //**********
  // includes
  //**********

#pragma interface

#include "Edge.h"
#include "Vertex.h"

  //************
  // Konstanten
  //************

const String PlacePrefix = "P";

  // -------------------------------------------------------------------------
  // Klasse      : Place
  // -------------------------------------------------------------------------
  // Beschreibung: Definition der Grundfunktionalitt einer Stelle
  // -------------------------------------------------------------------------
class Place : public Vertex {
  protected:
    short int has_current;
    virtual void Write(ostream &f) const;
    virtual void ReadOptional(istream &f);
    virtual const String& PreFix() const;
    virtual void WriteMarking(ostream &f) const;
      // ---------------------------------------------------------------
      //  Datenelemente
      // ---------------------------------------------------------------
    static int place_counter;
    static int active_objects;
    int tokens;
    int initial_tokens;
    int capacity;  // -1 -> unendlich
    enum {entry, exit, internal} state;
  public:
      // ---------------------------------------------------------------
      //  Konstante Funktionen
      // ---------------------------------------------------------------
      // Anzahl der auf dieser Stelle liegenden Marken zurck
    int NumTokens(void) const;
    int operator ==(const Place &p) const;
    int Capacity() const;
    int State(void) {return state;};
      // ---------------------------------------------------------------
      //  Modifizierende Funktionen
      // ---------------------------------------------------------------
    static void ResetMaxIdTo(int nr);
    virtual void SetMaxId(int id);
    virtual void ChangeMyName(const String &new_name);
    void setInit() {state = internal;}
    virtual void SetEntry(int read_flag = 0);
    virtual void SetExit();
    void SetCapacity(int c);
      // Marke auf Stelle zufgen
    virtual void AddTokens(int num = 1);
      // k Token entfernen
    void RemoveTokens(int k = 1);    
      // Alle Token auf dieser Stelle lschen
    virtual void DeleteAllTokens();
    const Place &operator =(const Place &p);
      // Cast
    virtual Place *asPlace() const;
      //  Konstruktoren
    Place(const Net &n,const String &Type = "Place");
    Place(const Place &p);
      // Destruktor
    virtual ~Place();
      // Tcl-Schnittstelle
    int ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[]);
    int ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[]);
      //Hash function needed by VHSet<Place>
    unsigned int Hash() const;
};  // class Place

  //*******************
  // Inlines fr Place
  //*******************

inline const Place &
Place::operator =(const Place &p)
{
  if (p.Id() == Id()) 
    return *this;
  Vertex::operator =(p);
  tokens = p.tokens;
  initial_tokens = p.initial_tokens;
  capacity = p.capacity;
  state = p.state;
  return *this;
}  // Place::operator =

inline int
Place::NumTokens(void) const 
{
  return tokens;
}  // Place::NumTokens

inline void
Place::DeleteAllTokens(void)
{
  tokens = 0;
}  // Place::DeleteAllTokens

inline void
Place::RemoveTokens(int k)
{
  if (k <= tokens) 
    tokens -= k;
  else
    cerr << "Place::RemoveTokens: Too much tokens to remove\n";
}  // Place::RemoveTokens

inline int
Place::operator ==(const Place &p) const
{
  return (name == p.name) ? 0 : 1;
}  // Place::operator ==

inline int
Place::Capacity() const
{
  return capacity;
}  // Place::Capacity

inline const String &
Place::PreFix() const
{
  return PlacePrefix;
}  // Place::PreFix

inline void
Place::SetMaxId(int nr)
{
  place_counter = (nr>place_counter) ? nr+1 : place_counter;
}  // Place::SetMaxId

inline void
Place::ResetMaxIdTo(int nr)
{
  place_counter = nr;
}  // Place::ResetMaxId

inline Place *
Place::asPlace() const
{
  return (Place *) this;
}  // Place::asPlace

inline unsigned int
Place::Hash() const
{
	return  hash_string(name);
}


#endif  // #ifndef __PLACE_H__

  //*** Dateiende Place.h ***

