// Transition.cc
// Tobias Himstedt 1995
// Burkhard Bieber 1995/96
// Christian Stehno: changed ip->result to use strdup
// last-update: 30.05.2000

  //**********
  // includes
  //**********

#include "Transition.h"
#include "Net.h"

  //*******************************
  // Inits fr statische Variablen
  //*******************************

int Transition::trans_counter = 1;
int Transition::active_objects = 0;

  //*************************
  // Methoden fr Transition
  //*************************

Transition::~Transition()
{
  if (--active_objects == 0)
    trans_counter = 1;
  for(Pix i = phantomtrans.first(); i; phantomtrans.next(i)) {
    phantomtrans.contents(i)->RemoveSyncTrans(name);
  }
}  // Transition::~Transition

Transition::Transition(const Transition &t) : Vertex(t),
  earliest(0), latest(0),
  refined(t.refined), channel_0_sync(t.channel_0_sync),
  phantomtrans(t.phantomtrans)
{
  net->DefaultTransName(rx_name, ry_name);
  net->DefaultTransMean(rx_mean, ry_mean);
  if(net->DefaultTrans()) {
    earliest = net->Earliest();
    latest   = net->Latest();
  }
  id = trans_counter++;
  active_objects++;
  String temp = PreFix() + IntToString(id);
  const VHMap<String, Transition *> &Trans = net->Transitions();
  while(Trans.contains(temp)) {
    id++;
    ResetMaxIdTo(id);
    temp = PreFix() + IntToString(id);
  }
  name = temp;
  //name = PreFix() + IntToString(id);
}  // Transition::Transition

Transition::Transition(const Net &n, const String &Type) : Vertex(n, Type),
  earliest(0), latest(0),
  refined(no), channel_0_sync(no),
  phantomtrans((PTransition *) NULL, 5)
{
  net->DefaultTransName(rx_name, ry_name);
  net->DefaultTransMean(rx_mean, ry_mean);
  if(net->DefaultTrans()) {
    earliest = net->Earliest();
    latest   = net->Latest();
  }
  id = trans_counter++;
  base_classes.add("Vertex");
  active_objects++;
  String temp = PreFix() + IntToString(id);
  const VHMap<String, Transition *> &Trans = net->Transitions();
  while(Trans.contains(temp)) {
    id++;
    ResetMaxIdTo(id);
    temp = PreFix() + IntToString(id);
  }
  name = temp;
//  name = PreFix() + IntToString(id);
}  // Transition::Transition

void
Transition::Write(ostream &f) const
  // Ausgabe der Werte fr eine Transition
{
  Vertex::Write(f);
  if (rx_name != net->DefaultTransNameX() || 
    ry_name != net->DefaultTransNameY()) 
    f << 'n' << rx_name << '@' << ry_name;
  if (rx_mean != net->DefaultTransMeanX() || 
    ry_mean != net->DefaultTransMeanY())
    f << 'a' << rx_mean << '@' << ry_mean;
  if ((net->GetNetTypeSwitch() == 1) && (channel_0_sync == 1))
    f << "S";
  if ((net->GetNetTypeSwitch() == 1) && !phantomtrans.empty()) {
    f << "P\"(";
    int ptr_count = 1;
    for (Pix i = phantomtrans.first(); i; phantomtrans.next(i)) {
      if (ptr_count != 1)
        f << ",";
      f << phantomtrans.contents(i)->Id();
      ptr_count++;
    }  // for i ber die Phantomverweise
    f << ")\"";
  }  // if Angaben vorhanden
  if(net->WithTime()) {
    f << "E" << earliest;
    f << "L" << latest;
  }
}  // Transition::Write

void
Transition::ReadOptional(istream &f)
{
  String line;
  char c;

  Vertex::ReadOptional(f);
  f.get(c);
  if (c == 'r')
    refined = yes;
  else if (c == 'S')
    channel_0_sync = yes;
  else if (c == 'P') {
    f.get(c);
    readline(f, line, '"');
  }  // else if Phantomverweis
  else if (c == 'E') {
   f >> earliest;
  }
  else if (c == 'L') {
   f >> latest;
  }
  else
    f.putback(c);
}  // Transition::ReadOptional

void Transition::AddPTransition(String name, PTransition *ptr)
  // Die Phantomtransition 'ptr' wird unter dem Namen 'name'
  // in die Liste der Phantomtransitionen eingetragen.
{
  phantomtrans[name] = ptr;
}  // Transition::AddPTransition

void Transition::RemovePTrans(String& name)
  // Die Phantomtransition name wird aus der Liste der
  // Phantomtransition gel"oscht
{
  phantomtrans.del(name);
}  // Transition::RemovePTransition

int
Transition::CanFire(void) const
{
  for (Pix i = incoming_edges.first(); i; incoming_edges.next(i)) {
    const Edge *e = incoming_edges.contents(i);
    if ((unsigned int)e->Src().asPlace()->NumTokens() < e->Weight())
      return false;
  }  // for i over all incoming edges
  return true;
}  // Transition::CanFire

VHSet<String>
Transition::InputPlaces(void) const
{
  VHSet<String> ret;

  for (Pix i = incoming_edges.first(); i; incoming_edges.next(i))
    ret.add(incoming_edges.key(i));
  return ret;
}  // Transition::InputPlaces

VHSet<String>
Transition::OutputPlaces(void) const
{
  VHSet<String> ret;

  for (Pix i = outgoing_edges.first(); i; outgoing_edges.next(i))
    ret.add(outgoing_edges.key(i));
  return ret;
}  // Transition::OutputPlaces

VHSet<String>
Transition::Phantoms(void) const
{
  VHSet<String> ret;

  for (Pix i = phantomtrans.first(); i; phantomtrans.next(i))
    ret.add(phantomtrans.key(i));
  return ret;
}  // Transition::Phantoms

void
Transition::ChangeMyName(const String &new_name) 
{
  net->ChangeTransName(name, new_name);
  //Vertex::ChangeMyName(new_name);
}  // Transition::ChangeMyName

int
Transition::ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[])
  // TCL-Schnittstelle
{
  if (argc != 1) {
    Tcl_AppendResult(ip, "wrong #args for Transition get",
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl fr Transition get
  if (EQ(0, "InPlaces"))
    for (Pix i = incoming_edges.first(); i; incoming_edges.next(i))
      Tcl_AppendElement(ip, incoming_edges.key(i));

  else if (EQ(0, "OutPlaces"))
    for (Pix i = outgoing_edges.first(); i; outgoing_edges.next(i))
      Tcl_AppendElement(ip, outgoing_edges.key(i));

  else if (EQ(0, "Phantoms"))
    for (Pix i = phantomtrans.first(); i; phantomtrans.next(i))
      Tcl_AppendElement(ip, phantomtrans.key(i));

  else if (EQ(0, "IsActive")) {
    INTRES(CanFire());
  }  // if IsActive

  else if (EQ(0, "Refined"))
    ip->result = strdup((refined == yes) ? "yes" : "no");

  else if (EQ(0, "Channel0Sync"))
    ip->result = strdup((channel_0_sync == yes) ? "yes" : "no");

  else if (EQ(0, "Earliest"))
    Tcl_AppendElement(ip, DoubleToString(earliest));

  else if (EQ(0, "Latest"))
    Tcl_AppendElement(ip, DoubleToString(latest));

  else
    return Vertex::ObjectGet(ip, argc, argv);
  return TCL_OK;
}  // Transition::ObjectGet

int 
Transition::ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[])
  // TCL-Schnittstelle
{
  if (argc < 2) {
    Tcl_AppendResult(ip, "wrong #args for Transition set",
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl fr Transition set
  if (EQ(0, "Phantoms")) {
    String para = argv[1];
    int zaehler = 0;
    while(para != "") {
      para = para.after(" ");
      zaehler++;
    }
    String kette;
    para = argv[1];
    channel_0_sync = no;
    phantomtrans.clear();
    for(int i = 0;i < zaehler;i++) {
      kette = (i == zaehler-1) ? para : para.before(" ");
      para = para.after(" ");
      if (net->ContainPTransition(kette)) {
        AddPTransition(kette, net->FindPTransition(kette));
        channel_0_sync = yes;
      }
    }
  }
  if (EQ(0, "Blocks")) {
    String para = argv[1];
    int zaehler = 0;
    while(para != "") {
      para = para.after(" ");
      zaehler++;
    }
    String kette;
    para = argv[1];
    blocks.clear();
    for(int i = 0;i < zaehler;i++) {
      kette = (i == zaehler-1) ? para : para.before(" ");
      para = para.after(" ");
      if (net->ContainBlock(kette)) {
        AddBlock(kette, net->GetBlockFromName(kette));
      }
    }
  }
  if (EQ(0, "Box")) {
    box = "";
    boxPointer = (Box *) NULL;
    String para(argv[1]);
    if(net->ContainBox(para)) {
      setBox(para, net->GetBoxFromName(para));
    }
  }
  Tk_ArgvInfo parse_table[] = {
    {"Earliest", TK_ARGV_FLOAT, (char *) NULL, (char *) &earliest,
     "Earliest Time to fire"},
    {"Latest", TK_ARGV_FLOAT, (char *) NULL, (char *) &latest,
     "Latest Time to fire"},
    {NULL, TK_ARGV_END, NULL, NULL, NULL}};
  if (Tk_ParseArgv(ip, NULL, &argc, argv, parse_table, 
    TK_ARGV_DONT_SKIP_FIRST_ARG) != TCL_OK)
    return TCL_ERROR;
  if (argc > 0)
    return Vertex::ObjectSet(ip, argc, argv);
  return TCL_OK;
}  // Place::ObjectSet

  //*** Dateiende Transition.cc ***

