// Transition.h
// Tobias Himstedt 1995
// Burkhard Bieber 1995/96
// last-update: 01.03.96

#ifndef __TRANSITION_H__
#define __TRANSITION_H__

  //**********
  // includes
  //**********

  // Ja / Nein
enum yes_no_type {no, yes};

#include "Place.h"
#include "PTransition.h"

  //************
  // Konstanten
  //************

const String TransitionPrefix = "T";

  //*********
  // Klassen
  //*********

  // Definition der Grundfunktionalitt einer Transition
class Transition : public Vertex {
  protected:
    static int trans_counter;
    static int active_objects;
    double earliest;
    double latest;
    yes_no_type refined;
    yes_no_type channel_0_sync;
    VHMap<String, PTransition *> phantomtrans;
    virtual void ReadOptional(istream &f);
    virtual void Write(ostream &f) const;
    virtual const String &PreFix() const;
      // neuen Status setzen (Sync)
    void SetChannel0Sync(yes_no_type new_channel_0_sync);
  public:
      // ---------------------------------------------------------------
      //  Konstante Funktionen
      // ---------------------------------------------------------------
      // Kann diese Transition schalten
    virtual int CanFire(void) const;
      // Menge der Eingangsstellen zurckgeben
    VHSet<String> InputPlaces(void) const;
      // Menge der Ausgangsstellen zurckgeben
    VHSet<String> OutputPlaces(void) const;
      // Menge der Phantomtransitionen zurckgeben
    VHSet<String> Phantoms(void) const;
      // Status der Transition (Sync) liefern
    yes_no_type GetChannel0Sync(void);
      // ---------------------------------------------------------------
      //  Konvertierungsfunktionen
      // ---------------------------------------------------------------
    virtual Transition *asTransition() const;
      // ---------------------------------------------------------------
      //  Modifizierende Funktionen
      // ---------------------------------------------------------------
    void AddPTransition(String name, PTransition *ptr);
    void RemovePTrans(String& name);
    const Transition &operator =(const Transition &t);
    static void ResetMaxIdTo(int nr);
    virtual void SetMaxId(int id);
    virtual void ChangeMyName(const String &new_name);
      // ---------------------------------------------------------------
      //  Konstruktor/Destruktor
      // ---------------------------------------------------------------
    Transition(const Net &n,const String &Type = "Transition");
    Transition(const Transition &t);
    virtual ~Transition();
    int ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[]);
    int ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[]);
};  // class Transition

  //************************
  // Inlines fr Transition
  //************************

inline const Transition &
Transition::operator =(const Transition &t)
{
  if (Name() == t.Name())
    return *this;
  Vertex::operator =(t);
  phantomtrans = t.phantomtrans;
/*  phantomtrans.clear();
  for(Pix i = t.phantomtrans.first(); i;t.phantomtrans.next(i)) {
    AddPTransition(t.phantomtrans.key(i), t.phantomtrans.contents(i));
  }*/
//  phantomtrans = t.phantomtrans;
  earliest = t.earliest;
  latest = t.latest;
  channel_0_sync = t.channel_0_sync;
  refined = t.refined;
  return *this;
}

inline const String &
Transition::PreFix() const
{
  return TransitionPrefix;
}  // Transition::PreFix

inline void
Transition::SetMaxId(int nr)
{
  trans_counter = (nr>trans_counter) ? nr+1 : trans_counter;
}  // Transition::SetMaxId

inline void
Transition::ResetMaxIdTo(int nr)
{
  trans_counter = nr;
}  // Transition::ResetMaxId

inline Transition *
Transition::asTransition() const
{
  return (Transition *) this;
}  // Transition::asTransition

inline yes_no_type
Transition::GetChannel0Sync(void)
  // Status der Transition (Sync) liefern
{
  return channel_0_sync;
}  // Transition::GetChannel0Sync

inline void
Transition::SetChannel0Sync(yes_no_type new_channel_0_sync)
  // neuen Status setzen (Sync)
{
  channel_0_sync = new_channel_0_sync;
}  // Transition::SetChannel0Sync

#endif  // #ifndef __TRANSITION_H__

  //*** Dateiende Transition.h ***

