// Vertex.cc
// Tobias Himstedt 1995
// Burkhard Bieber 1995/96:
//   - Block/HLBlock ergnzt
//   - Blockverweise
//   - Phantomkanten
// Christian Stehno: changed string compare operator to correct order
// last-update: 30.05.2000

  //**********
  // includes
  //**********

#include <ctype.h>
#include <iomanip.h>
#include <strstream>
#include "Edge.h"
#include "Vertex.h"
#include "Net.h"

  //*********************
  // Methoden fr Vertex
  //*********************

Vertex::Vertex():
  NetObject(), 
  px(0), py(0),
  net((Net *) NULL), box(""), boxPointer((Box *) NULL),
  incoming_edges((Edge *) NULL, 5), outgoing_edges((Edge *) NULL, 5),
  incoming_pedges((Edge *) NULL, 5), outgoing_pedges((Edge *) NULL, 5),
  blocks((Block *) NULL, 5)
{}


Vertex::Vertex(const Net &n, const String &Type) : NetObject(Type), 
  px(0), py(0),
  net((Net *) &n), box(""), boxPointer((Box *) NULL),
  incoming_edges((Edge *) NULL, 5), outgoing_edges((Edge *) NULL, 5),
  incoming_pedges((Edge *) NULL, 5), outgoing_pedges((Edge *) NULL, 5),
  blocks((Block *) NULL, 5)
  // Konstruktor
{
  base_classes.add("NetObject");
}  // Vertex::Vertex

Vertex::~Vertex()
  // Destruktor
{
  Pix i;
  String temp;
  char buff[100];

  if(type != "HLBlock" & type != "Block" & type != "Box" & type != "HLBox") {
    for (i = incoming_pedges.first(); i; incoming_pedges.next(i)) {
      sprintf(buff, "PA%d", incoming_pedges.contents(i)->Id());
      temp = buff;
      net->RemovePEintrag(temp);
      delete incoming_pedges.contents(i);
    }
    for (i = incoming_edges.first(); i; incoming_edges.next(i)) {
      sprintf(buff, "A%d", incoming_edges.contents(i)->Id());
      temp = buff;
      net->RemoveEintrag(temp);
      delete incoming_edges.contents(i);
    }
    for (i = outgoing_edges.first(); i; outgoing_edges.next(i)) {
      sprintf(buff, "A%d", outgoing_edges.contents(i)->Id());
      temp = buff;
      net->RemoveEintrag(temp);
      delete outgoing_edges.contents(i);
    }
    for (i = outgoing_pedges.first(); i; outgoing_pedges.next(i)) {
      sprintf(buff, "PA%d", outgoing_pedges.contents(i)->Id());
      temp = buff;
      net->RemovePEintrag(temp);
      delete outgoing_pedges.contents(i);
    }
  }
  incoming_edges.clear();
  outgoing_edges.clear();
  incoming_pedges.clear();
  outgoing_pedges.clear();
    // Spter: evtl. wird hier Speicher nicht wieder freigegeben.
    // Aber die Zeilen sollten nicht in dieser Form verwendet werden,
    // da es sonst zur Verwendung von delete auf einen NULL-Pointer
    // kommt. Das Problem sollte sich in dieser Form allerdings nicht
    // allzusehr negativ bemerkbar machen.
//  for (i = blocks.first(); i; blocks.next(i))
//    delete blocks.contents(i);
//  blocks.clear();
}  // Vertex::~Vertex

void
Vertex::DeleteBox(String &n)
{
  if(n == box) {
    box = "";
    boxPointer = (Box *) NULL;
  }
}  // Vertex::DeleteBox

const Vertex &
Vertex::operator =(const Vertex &v)
  // Zuweisungsoperator
{
  if (this == &v)
    return *this;
  NetObject::operator =(v);
  meaning = v.meaning;
  reference = v.reference;
  incoming_edges = v.incoming_edges;
  outgoing_edges = v.outgoing_edges;
  incoming_pedges = v.incoming_pedges;
  outgoing_pedges = v.outgoing_pedges;
  blocks = v.blocks;
  px = v.px;
  py = v.py;
  rx_name = v.rx_name;
  ry_name = v.ry_name;
  rx_mean = v.rx_mean;
  ry_mean = v.ry_mean;
  net = v.net;
  return *this;
}  // Vertex::operator =

Transition *
Vertex::asTransition() const
  // Cast
{
  cerr << "brutal cast error! Attempt Vertex -> Transition" << endl;
  exit(1);
  return NULL;
}  // Vertex::asTransition

PTransition *
Vertex::asPTransition() const 
  // Cast
{
  cerr << "brutal cast error! Attempt Vertex -> PTransition" << endl;
  exit(1);
  return NULL;
}  // Vertex::asPTransition

HLTransition *
Vertex::asHLTransition() const 
  // Cast
{
  cerr << "brutal cast error! Attempt Vertex -> HLTransition" << endl;
  exit(1);
  return NULL;
}  // Vertex::asHLTransition

HLPTransition *
Vertex::asHLPTransition() const 
  // Cast
{
  cerr << "brutal cast error! Attempt Vertex -> HLPTransition" << endl;
  exit(1);
  return NULL;
}  // Vertex::asHLPTransition

Place *
Vertex::asPlace() const 
  // Cast
{
  cerr << "brutal cast error! Attempt Vertex -> Place" << endl;
  exit(1);
  return NULL;
}  // Vertex::asPlace

HLPlace *
Vertex::asHLPlace() const 
  // Cast
{
  cerr << "brutal cast error! Attempt Vertex -> HLPlace" << endl;
  exit(1);
  return NULL;
}  // Vertex::asHLPlace

Block *
Vertex::asBlock() const
  // Cast
{
  cerr << "brutal cast error! Attempt Vertex -> Block\n";
  exit(1);
  return NULL;
}  // Vertex::asBlock

HLBlock * 
Vertex::asHLBlock() const 
  // Cast
{
  cerr << "brutal cast error! Attempt Vertex -> HLBlock\n";
  exit(1);
  return NULL;
}  // Vertex::asHLBlock

Box *
Vertex::asBox() const
  // Cast
{
  cerr << "brutal cast error! Attempt Vertex -> Box\n";
  exit(1);
  return NULL;
}  // Vertex::asBox

HLBox * 
Vertex::asHLBox() const 
  // Cast
{
  cerr << "brutal cast error! Attempt Vertex -> HLBox\n";
  exit(1);
  return NULL;
}  // Vertex::asHLBox

void
Vertex::Write(ostream &f) const
  // Ausgabe der knotenrelevanten Angaben in den Ausgabe-Stream
{
  f << Id() << "\"" << Name() << "\"" << px << "@" << py;
  if (meaning != "") 
    f << "b\"" << meaning << '"';
  if (reference != "")
    f << "R\"" << reference << '"';
  if ((net->GetNetTypeSwitch() == 1) && !blocks.empty()) {
    f << "u\"(";
    int bl_count = 1;
    for (Pix i = blocks.first(); i; blocks.next(i)) {
      if (bl_count != 1)
        f << ",";
      f << blocks.contents(i)->Id();
      bl_count++;
    }  // for i ber die Blockverweise
    f << ")\"";
  }  // if Angaben vorhanden
  NetObject::Write(f);
}  // Vertex::Write

void
Vertex::ReadStandard(istream &f)
  // Die wichtigsten Angaben lesen oder erstellen.
{
  int c;

  f >> ws;
  c = f.peek();
  if (isdigit(c)) {
    int i;
    f >> i;
    SetMaxId(i+1);
    id = Id();
    if(PreFix() == "B") {
      net->BlockKonvert(i, id);
    }
    else if(PreFix() == "P") {
      net->PlaceKonvert(i ,id);
    }
    else if(PreFix() == "T") {
      net->TransKonvert(i, id);
    }
    else if(PreFix() == "PT") {
      net->PTransKonvert(i, id);
    }
    //f >> id;
    //SetMaxId(id);
  } else {
    SetMaxId(id+1);
    id = Id();
    if(PreFix() == "B") {
      net->BlockKonvert(id, id);
    }
    else if(PreFix() == "P") {
      net->PlaceKonvert(id ,id);
    }
    else if(PreFix() == "T") {
      net->TransKonvert(id, id);
    }
    else if(PreFix() == "PT") {
      net->PTransKonvert(id, id);
    }
  }
  f >> ws;
  c = f.peek();
  if (c == '"') {
    char c;
    f >> c;
    readline(f,name,'"');
  }
  else {
    name = PreFix()+IntToString(id);
  }
  char term;
  f >> px >> term >> py;
}  // Vertex::ReadStandard

void
Vertex::ReadOptional(istream &f)
  // optionale Werte lesen
{
  char c, dummy;
  String blocks_str;
  int block_id;
  Block *block;

  NetObject::ReadOptional(f);
  f.get(c);
  switch (c) {
    case 'n':
      f >> rx_name >> dummy >> ry_name;
      break;
    case 'b':
      f >> c;
      readline(f, meaning, '"');
      break;
    case 'a':
      f >> rx_mean >> dummy >> ry_mean;
      break;
    case 'T':
      {
        f >> c;
        String temp;
        readline(f, temp, '"');
        break;
      }
    case 'R':
      f >> dummy;
      readline(f, reference, '"');
      break;
    case 'u':
      {  // <- Das sind sehr wichtige Klammern!
        f >> c;
        readline(f, blocks_str, '"');
        istrstream bs((const char*) blocks_str);
        bs >> c;  // Klammer auf
        while (bs && (c != ')')) {
          bs >> block_id >> c;
          block = net->GetBlockFromId(block_id);
          if (block != 0) {
            blocks[block->Name()] = block;
          }  // if Block gefunden
          else {
            cerr << "error: Vertex::ReadOptional (Id=" << id 
              << ") : block-id not found ("
              << block_id << ")" << endl;
          }  // else, Block nicht gefunden
        }  // while nicht am Ende
        break;
      }  // <- Die auch!
//    case 't':
    case 'c':
	// Noch zu implementieren!
      break;
    default:
      f.putback(c);	    
  }  // switch ber das nchste Zeichen
}  // Vertex::ReadOptional

void
Vertex::ChangeInName(const String &old_name, const String &new_name)
  // Name der Eingangskante ndern
{
#ifdef WARN
  if (incoming_edges.contains(new_name)) {
    cerr << "warning: name " << new_name << " exists" << endl;
  }  // if bereits vorhanden
  else {
#endif
    Edge *e = incoming_edges[old_name];

    incoming_edges.del(old_name);
    incoming_edges[new_name] = e;
#ifdef WARN
  }  // else, kein Problem
#endif
}  // Vertex::ChangeInName

void
Vertex::ChangeOutName(const String &old_name, const String &new_name)
  // Name der Ausgangskante ndern
{
#ifdef WARN
  if (outgoing_edges.contains(new_name)) {
    cerr << "warning: name " << new_name << " exists" << endl;
  }  // if bereits bekannt
  else {
#endif
    Edge *e = outgoing_edges[old_name];

    outgoing_edges.del(old_name);
    outgoing_edges[new_name] = e;
#ifdef WARN
  }  // else, kein Problem
#endif
}  // Vertex::ChangeOutName

void
Vertex::ChangeInPName(const String &old_name, const String &new_name)
  // Name der Eingangsphantomkante ndern
{
#ifdef WARN
  if (incoming_pedges.contains(new_name)) {
    cerr << "warning: name " << new_name << " exists" << endl;
  }  // if bereits vorhanden
  else {
#endif
    Edge *e = incoming_pedges[old_name];

    incoming_pedges.del(old_name);
    incoming_pedges[new_name] = e;
#ifdef WARN
  }  // else, kein Problem
#endif
}  // Vertex::ChangeInPName

void
Vertex::ChangeOutPName(const String &old_name, const String &new_name)
  // Name der Ausgangsphantomkante ndern
{
#ifdef WARN
  if (outgoing_pedges.contains(new_name)) {
    cerr << "warning: name " << new_name << " existis" << endl;
  }  // if bereits bekannt
  else {
#endif
    Edge *e = outgoing_pedges[old_name];

    outgoing_pedges.del(old_name);
    outgoing_pedges[new_name] = e;
#ifdef WARN
  }  // else, kein Problem
#endif
}  // Vertex::ChangeOutPName

Edge *
Vertex::EdgeFrom(const String &n) const
  // Eingangskante
{
  Pix i = incoming_edges.seek(n);
  return i ? (Edge *) incoming_edges.contents(i) : (Edge *) NULL;
}  // Vertex::EdgeFrom

Edge *
Vertex::EdgeTo(const String &n) const
  // Ausgangskante
{
  Pix i = outgoing_edges.seek(n);
  return i ? (Edge *) outgoing_edges.contents(i) : (Edge *) NULL;
}  // Vertex::EdgeTo

Edge *
Vertex::PEdgeTo(const String &n) const
  // Ausgangsphantomkante
{
  Pix i = outgoing_pedges.seek(n);
  return i ? (Edge *) outgoing_pedges.contents(i) : (Edge *) NULL;
}  // Vertex::PEdgeTo

Edge *
Vertex::PEdgeFrom(const String &n) const
  // Eingangsphantomkante
{
  Pix i = incoming_pedges.seek(n);
  return i ? (Edge *) incoming_pedges.contents(i) : (Edge *) NULL;
}  // Vertex::PEdgeFrom

VHSet<String>
Vertex::BlockKeySet(void)
  // Menge der Blcke als Set zurckgeben, nur Keys
{
  VHSet<String> ret;

  for (Pix i = blocks.first(); i; blocks.next(i))
    ret.add(blocks.contents(i)->Name());
// ret.add(blocks.key(i))
  return ret;
}  // Vertex::BlockKeySet

int
Vertex::ObjectGet(Tcl_Interp *ip, int argc, CONST char *argv[])
  // TCL-Schnittstelle
{
  if (argc != 1) {
    Tcl_AppendResult(ip, "wrong #args for Vertex get",
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl fr Vertex get

  if (!strcmp(argv[0], "Mean"))
    Result(ip, meaning);

  else if (!strcmp(argv[0], "x"))
    INTRES(px);

  else if (!strcmp(argv[0], "y"))
    INTRES(py);

  else if (EQ(0, "NameX"))
    INTRES(rx_name);

  else if (EQ(0, "NameY"))
    INTRES(ry_name);

  else if (EQ(0, "MeanX"))
    INTRES(rx_mean);

  else if (EQ(0, "MeanY"))
    INTRES(ry_mean);

  else if (EQ(0, "OutEdges")) {
    char buff[200];
    for (Pix i = outgoing_edges.first(); i; outgoing_edges.next(i)) {
      sprintf(buff, "A%d", outgoing_edges.contents(i)->Id());
      Tcl_AppendElement(ip, buff);
    }
  }  // if ausgehende Kanten

  else if (EQ(0, "InEdges")) {
    char buff[200];
    for (Pix i = incoming_edges.first(); i; incoming_edges.next(i))  {
      sprintf(buff, "A%d", incoming_edges.contents(i)->Id());
      Tcl_AppendElement(ip, buff);
    }
  }  // if eingehende Kanten

  else if (EQ(0, "OutPEdges")) {
    char buff[200];
    for (Pix i = outgoing_pedges.first(); i; outgoing_pedges.next(i)) {
      sprintf(buff, "PA%d", outgoing_pedges.contents(i)->Id());
      Tcl_AppendElement(ip, buff);
    }
  }  // if ausgehende Phantomkanten

  else if (EQ(0, "InPEdges")) {
    char buff[200];
    for (Pix i = incoming_pedges.first(); i; incoming_pedges.next(i))  {
      sprintf(buff, "PA%d", incoming_pedges.contents(i)->Id());
      Tcl_AppendElement(ip, buff);
    }
  }  // if eingehende Phantomkanten

  else if (EQ(0, "Box")) {
    Tcl_SetResult(ip, box, TCL_STATIC);
  }

  else if (EQ(0, "Blocks")) {
    char buff[2000];
    for (Pix i = blocks.first(); i; blocks.next(i)) {
      strcpy(buff, blocks.contents(i)->Name());
      //sprintf(buff, "B%d", blocks.contents(i)->Id());
      Tcl_AppendElement(ip, buff);
    }  // for i ber die Blcke
  }  // if Blcke

  else if  (EQ(0, "Reference")) 
    Tcl_SetResult(ip, reference, TCL_STATIC);

  else 
    return NetObject::ObjectGet(ip, argc, argv);
  return TCL_OK;
}  // Vertex::ObjectGet

const char* forbidden[] = {
 "newer", "older", ""
};

int
Vertex::ObjectSet(Tcl_Interp *ip, int argc, CONST char *argv[])
  // TCL-Schnittstelle
{
  if (argc < 2) {
    Tcl_AppendResult(ip, "wrong #args for Vertex set",
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl fr Vertex set
  char *m = NULL;
  char *n = NULL;
  char *o = NULL;
  Tk_ArgvInfo parse_table[] = {
    {"x", TK_ARGV_INT, (char *) NULL, (char *) &px,
     "x - position of object"},
    {"y", TK_ARGV_INT, (char *) NULL, (char *) &py,
     "y - position of object"},
    {"Mean", TK_ARGV_STRING, NULL, (char *) &m,
     "Meaning of vertex"},
    {"Name", TK_ARGV_STRING, NULL, (char *) &n,
     "Name of vertex"},
    {"Reference", TK_ARGV_STRING, NULL, (char *) &o,
     "Reference of vertex"},
    {"NameX", TK_ARGV_INT, NULL, (char *) &rx_name,
     "Relativ x - position of name"},
    {"NameY", TK_ARGV_INT, NULL, (char *) &ry_name,
     "Relativ y - position of name"},
    {"MeanX", TK_ARGV_INT, NULL, (char *) &rx_mean,
     "Relativ x - position of meaning"},
    {"MeanY", TK_ARGV_INT, NULL, (char *) &ry_mean,
     "Relativ y - position of meaning"},
    {NULL, TK_ARGV_END, NULL, NULL, NULL}};
  if (Tk_ParseArgv(ip, NULL, &argc, argv, parse_table, 
    TK_ARGV_DONT_SKIP_FIRST_ARG | TK_ARGV_NO_ABBREV) != TCL_OK)
    return TCL_ERROR;
  if (o)
    reference = o;
  if (m) 
    meaning = m;
  if (n) {
    const char** k = forbidden;
    while(*k != NULL) {
      if(!strcmp(*k,n))
        return NetObject::ObjectSet(ip, argc, argv);
      k++;
    }
    int boo = 1;
    char t = n[0];
    int i = 0;
    while(t) {
      if(!(((t <= int('9')) && (t >= int('0'))) || ((t <= int('Z')) && (t >= int('A'))) || ((t <= int('z')) && (t >= int('a'))) || (t == int('_'))))
        boo = 0;
      i++; t = n[i];
    }
    if(boo == 1)
      ChangeMyName(n);
  }
  return NetObject::ObjectSet(ip, argc, argv);
}  // Vertex::ObjectSet

  //*** Dateiende Vertex.cc ***

