#include "net.h"

string BINDUNG;

net::net(istrstream &os) {
  if(os)
    read(os);
}

void
net::read(istrstream &os) {
  string line;
  string first;
  string last;
  char c;

  EatComment(os);
  if(!ReadHeader(os))
    return;
  while(os) {
    os >> line;
    os.get(c);
    if(line == "PL") {
      c = os.peek();
      while(isdigit(c) || c == '"') {
        place pl(os);
        Places.insert(StringPlaceMap::value_type(pl.Name(), pl));
        os.get(c);
        c = os.peek();
      }
    }
    if(line == "TR") {
      c = os.peek();
      while(isdigit(c) || c == '"') {
        transition tr(os);
        Transitions.insert(StringTransMap::value_type(tr.Name(), tr));
        os.get(c);
        c = os.peek();
      }
    }
    if(line == "TP") {
      c = os.peek();
      while(isdigit(c)) {
        edge e(this, os);
        Edges.insert(StringEdgeMap::value_type(e.Name(), e));
        os.get(c);
        c = os.peek();
      }
    }
    if(line == "PT") {
      c = os.peek();
      while(isdigit(c)) {
        edge e(this, os);
        Edges.insert(StringEdgeMap::value_type(e.Name(), e));
        os.get(c);
        c = os.peek();
      }
    }
  }
}

int
net::ReadHeader(istrstream &os) {
  string header;

  os >> header;
  if(!(header == "PEP"))
    return 0;
  EatComment(os);
  os >> header;
  if(!(header == "HLNet"))
    return 0;
  EatComment(os);
  os >> header;
  if(!(header == "FORMAT_N" || header == "FORMAT_N2"))
    return 0;
  return 1;
}


void
net::EatComment(istrstream &os) {
  char c;
  while (os.get(c) && c == '%') {
    while (os.get(c) && c != '\n') {
      ;
    }
  }
  os.putback(c);
}

string
net::GetTransitionFromId(int id) {
  StringTransMap::iterator i = Transitions.begin();

  while(i != Transitions.end()) {
    if((*i).second.Id() == id)
      return (*i).second.Name();
    ++i;
  }
  return "";
}

string
net::GetPlaceFromId(int id) {
  StringPlaceMap::iterator i = Places.begin();

  while(i != Places.end()) {
    if((*i).second.Id() == id)
      return (*i).second.Name();
    ++i;
  }
  return "";
}

int
net::IsActive(string &tr) {
  set<string, less<string> > bs = SearchBinding((*Transitions.find(tr)).second);
  if(bs.empty())
    return 0;
  BINDUNG = *(bs.begin());
  return 1;
  //return !SearchBinding((*Transitions.find(tr)).second).empty();
}

void
net::FireWithout(string &t_name) {
  string tr = t_name;
  set<string, less<string> > bs = SearchBinding((*Transitions.find(tr)).second);
  if(bs.empty())
    return;
  string bind = (*bs.begin());
  FireTransition(tr, bind);
}

string
net::Marking(void) {
  string Marking = "";
  string m;
  for(StringPlaceMap::iterator i = Places.begin(); i != Places.end(); ++i) {
    m = (*i).second.Markierung();
    if(m != "") {
      if(Marking == "")
        Marking = (*i).first + ":" + (*i).second.Markierung();
      else
        Marking += "_" + (*i).first + ":" + (*i).second.Markierung();
    }
  }
  return Marking;
}

string
net::ActiveTransitions(void) {
  string active = "";
  string tr;
  for(StringTransMap::iterator i = Transitions.begin(); i != Transitions.end(); ++i) {
    tr = (*i).first;
    if(IsActive(tr)) {
      if(active == "")
        active = tr;
      else
        active += "," + tr;
    }
  }
  return active;
}

set<string, less<string> >
net::InputEdges(string &t) {
  set<string, less<string> > ret;

  for(StringEdgeMap::iterator i = Edges.begin(); i != Edges.end(); ++i) {
    if((*i).second.Dest() == t) {
      ret.insert((*i).first);
    }
  }
  return ret;
}

set<string, less<string> >
net::OutputEdges(string &t) {
  set<string, less<string> > ret;

  for(StringEdgeMap::iterator i = Edges.begin(); i != Edges.end(); ++i) {
    if((*i).second.Src() == t) {
      ret.insert((*i).first);
    }
  }
  return ret;
}

void
net::PreAnalyse(transition *actual_trans)
  // wertet den ValueTerm vor dem Zusammenstellen der Bindungen aus
{
  string id, va;
  set<string, less<string> > Bindungen2;

  BelegteString = "";
  BelegteVars.erase(BelegteVars.begin(), BelegteVars.end());

  // Valueterm auswerten !!
  string ValueTerm = actual_trans->ValueTerm().Text();
  string Sets;
  unsigned int co = ValueTerm.find(":");
  if(co != string::npos) {
    Sets = ValueTerm.substr(0,co);
    ValueTerm = ValueTerm.substr(co+1, 1000);
  }

  while(Sets.find(" ") != string::npos)
    Sets.serase(Sets.find(" "), 1);
  while(Sets.find("&") != string::npos)
    Sets = Sets.replace(Sets.find("&"), 1, ",");

  while(Sets != "") {
    co = Sets.find("{");
    if(co != string::npos) {
      id = Sets.substr(0,co);
      va = Sets.substr(co+1, 1000);
      co = va.find("}");
      va = va.substr(0,co);
      if(va.find("..") != string::npos | va.find(",") != string::npos) {
        co = Sets.find(",");
        if(co != string::npos)
          Sets = Sets.substr(co+1, 1000);
        else
          Sets = "";
        continue;
      }
      Bindungen2 = bindungen;
      if(!BelegteVars.contains(id)) {
        BelegteVars.insert(multimap<string, val, less<string> >::value_type(id, StringToVal(va)));
        for(set<string, less<string> >::iterator i = Bindungen2.begin(); i != Bindungen2.end(); ++i) {
          bindungen.erase((*i));
          bindungen.insert((*i) + "," + id + "=" + va);
        }
        if(bindungen.empty())
          bindungen.insert(id + "=" + va);
      }
    }
    co = Sets.find(",");
    if(co != string::npos)
      Sets = Sets.substr(0,co);
    else
      Sets = "";
  }

  while(ValueTerm.find(" ") != string::npos)
    ValueTerm.serase(ValueTerm.find(" "), 1);
  while(ValueTerm.find("&") != string::npos)
    ValueTerm = ValueTerm.replace(ValueTerm.find("&"), 1, ",");

  if(ValueTerm.find("=") == string::npos | ValueTerm.find("|") != string::npos)
    return;

  while (ValueTerm != "") {
    co = ValueTerm.find("=");
    if(co == string::npos)
      break;

    id = ValueTerm.substr(0,co);

    if(id.find("<") != string::npos | id.find(">") != string::npos)
      break;
    co = id.find("(");
    while(co != string::npos) {
      id = id.substr(co+1, 1000);
      co = id.find("(");
    }
    co = id.find(")");
    while(co != string::npos) {
      id = id.substr(0,co);
      co = id.find(")");
    }

    if(id.find("#") != string::npos) {
      co = ValueTerm.find(",");
      if(co != string::npos)
        ValueTerm = ValueTerm.substr(co+1, 1000);
      else
        ValueTerm = "";
      continue;
    }

    co = ValueTerm.find("=");
    if(co != string::npos)
      va = ValueTerm.substr(co+1,1000);
    else
      va = "";
    co = va.find("(");
    while(co != string::npos) {
      va = va.substr(co+1, 1000);
      co = va.find("(");
    }
    co = va.find(")");
    while(co != string::npos) {
      va = va.substr(0,co);
      co = va.find(")");
    }

    co = va.find(",");
    if(co != string::npos)
      ValueTerm = va.substr(co+1,1000);
    else
      ValueTerm = "";

    bind.erase(bind.begin(), bind.end());

    if (ValueTerm == "") {
      if(!IsVal(va)) {
        VT Wahr(va);
        val wert = Wahr.Evaluate(bind);
        if(wert != false & !BelegteVars.contains(id)) {
          BelegteVars.insert(multimap<string, val, less<string> >::value_type(id, wert));
          if(BelegteString != "")
            BelegteString += "," + id + "=" + wert.asString();
          else
            BelegteString = id + "=" + wert.asString();
        }
      } else {
        val wert = StringToVal(va);
        if(!BelegteVars.contains(id)) {
          BelegteVars.insert(multimap<string, val, less<string> >::value_type(id, wert));
          if(BelegteString != "")
            BelegteString += "," + id + "=" + wert.asString();
          else
            BelegteString = id + "=" + wert.asString();
        }
      }
    }
    else {
      co = va.find(",");
      va = va.substr(0,co);
      if(va.find("<") != string::npos | va.find(">") != string::npos)
        break;
      if(!IsVal(va)) {
        VT Wahr(va);
        val wert = Wahr.Evaluate(bind);
        if(wert != false & !BelegteVars.contains(id)) {
          BelegteVars.insert(multimap<string, val, less<string> >::value_type(id, wert));
          if(BelegteString != "")
            BelegteString += "," + id + "=" + wert.asString();
          else
            BelegteString = id + "=" + wert.asString();
        }
      } else {
        val wert = StringToVal(va);
        if(!BelegteVars.contains(id)) {
          BelegteVars.insert(multimap<string, val, less<string> >::value_type(id, wert));
          if(BelegteString != "")
            BelegteString += "," + id + "=" + wert.asString();
          else
            BelegteString = id + "=" + wert.asString();
        }
      }
    }  // else
  }  // while s nicht alle
}  // net::PreAnalyse

void
net::InAnalyse(transition *actual_trans)
  // unter den Bindungen der Eingangskanten werden Analysen durchgef"uhrt
{
  string id, va, H;
  string ValueTerm = actual_trans->ValueTerm().Text();
  string Sets;
  set<string, less<string> > Bindungen2;

  unsigned int co = ValueTerm.find(":");
  if(co != string::npos) {
    Sets = ValueTerm.substr(0,co);
    ValueTerm = ValueTerm.substr(co+1, 1000);
  }

  while(ValueTerm.find(" ") != string::npos)
    ValueTerm.serase(ValueTerm.find(" "), 1);
  while(ValueTerm.find("&") != string::npos)
    ValueTerm = ValueTerm.replace(ValueTerm.find("&"), 1, ",");

  while(ValueTerm != "") {
    if(ValueTerm.find("=") == string::npos | ValueTerm.find("|") != string::npos)
      break;

    co = ValueTerm.find("=");
    if(co != string::npos)
      id = ValueTerm.substr(0,co);
    else
      id = "";

    co = id.find("(");
    while(co != string::npos) {
      id = id.substr(co+1, 1000);
      co = id.find("(");
    }
    co = id.find(")");
    while(co != string::npos) {
      id = id.substr(0,co);
      co = id.find("(");
    }
    
    if(id.find("#") != string::npos)
      break;

    if(id.find(">") != string::npos | id.find("<") != string::npos) {
      co = ValueTerm.find(",");
      if(co != string::npos)
        ValueTerm = ValueTerm.substr(co+1, 1000);
      else
        ValueTerm = "";
      continue;
    } else {
      co = ValueTerm.find("=");
      if(co != string::npos)
        va = ValueTerm.substr(co+1, 1000);
      else
        va = "";
      co = va.find(",");
      if(co != string::npos)
        va = va.substr(0,co);
      co = va.find(")");
      if(co != string::npos)
        va = va.substr(0,co);
      co = ValueTerm.find(",");
      if(co != string::npos)
        ValueTerm = ValueTerm.substr(co+1, 1000);
      else
        ValueTerm = "";
    }
    if(BelegteVars.contains(id))
      continue;
    if(id == va)
      continue;
    if(!IsVal(va)) {
      VT vt(va);
      Bindungen2 = bindungen;
      for(set<string, less<string> >::iterator u = Bindungen2.begin(); u != Bindungen2.end(); ++u) {
        H = (*u);
        ParseAsignment(H);
        val v = vt.Evaluate(bind);
        if(v != false) {
          BelegteVars.insert(multimap<string, val, less<string> >::value_type(id, v));
          H = (*u);
          bindungen.erase(H);
          bindungen.insert(H + "," + id + "=" + v.asString());
        }
      }
    } else {
      Bindungen2 = bindungen;
      for(set<string, less<string> >::iterator u = Bindungen2.begin(); u != Bindungen2.end(); ++u) {
        H = (*u);
        BelegteVars.insert(multimap<string, val, less<string> >::value_type(id, StringToVal(va)));
        bindungen.erase(H);
        bindungen.insert(H + "," + id + "=" + va);
      }
    }
  } // while ValueTerm nicht leer

  ValueTerm = actual_trans->ValueTerm().Text();

  co = ValueTerm.find(":");
  if(co != string::npos) {
    Sets = ValueTerm.substr(0,co);
    ValueTerm = ValueTerm.substr(co+1, 1000);
  }

  while(ValueTerm.find(" ") != string::npos)
    ValueTerm.serase(ValueTerm.find(" "), 1);
  while(ValueTerm.find("&") != string::npos)
    ValueTerm = ValueTerm.replace(ValueTerm.find("&"), 1, ",");

  while(ValueTerm != "") {
    if(ValueTerm.find("=") == string::npos | ValueTerm.find("|") != string::npos)
      break;

    co = ValueTerm.find("=");
    if(co != string::npos)
      id = ValueTerm.substr(co+1, 1000);
    else
      id = "";
    co = id.find(",");
    if(co != string::npos)
      id = id.substr(0,co);

    co = id.find(")");
    while(co != string::npos) {
      id = id.substr(0,co);
      co = id.find(")");
    }
    co = id.find("(");
    while(co != string::npos) {
      id = id.substr(co+1, 1000);
      co = id.find("(");
    }

    if(id.find("#") != string::npos)
      break;

    if(id.find(">") != string::npos | id.find("<") != string::npos |
       id.find("+") != string::npos | id.find("-") != string::npos |
       id.find("*") != string::npos | id.find("/") != string::npos |
       id.find("%") != string::npos) {
      co = ValueTerm.find(",");
      if(co != string::npos)
        ValueTerm = ValueTerm.substr(co+1, 1000);
      else
        ValueTerm = "";
      continue;
    } else {
      co = ValueTerm.find("=");
      va = ValueTerm.substr(0,co);
      co = va.find(",");
      if(co != string::npos)
        va = va.substr(co+1,1000);
      co = va.find("(");
      while(co != string::npos) {
        va = va.substr(co+1,1000);
        co = va.find("(");
      }
      co = va.find(")");
      while(co != string::npos) {
        va = va.substr(0,co);
        co = va.find(")");
      }
      co = ValueTerm.find(",");
      if(co != string::npos)
        ValueTerm = ValueTerm.substr(co+1,1000);
      else
        ValueTerm = "";
    }
    if(BelegteVars.contains(id))
      continue;
    if(id == va)
      continue;
    if(va.find("<") != string::npos | va.find(">") != string::npos)
      break;

    if(!IsVal(va)) {
      VT vt(va);
      Bindungen2 = bindungen;
      for(set<string, less<string> >::iterator u = Bindungen2.begin(); u != Bindungen2.end(); ++u) {
        H = (*u);
        ParseAsignment(H);
        val v = vt.Evaluate(bind);
        if(v != false & !IsVal(va)) {
          BelegteVars.insert(multimap<string, val, less<string> >::value_type(id, v));
          H = (*u);
          bindungen.erase(H);
          bindungen.insert(H + "," + id + "=" + v.asString());
        }
      }
    } else {
      Bindungen2 = bindungen;
      for(set<string, less<string> >::iterator u = Bindungen2.begin(); u != Bindungen2.end(); ++u) {
        H = (*u);
        BelegteVars.insert(multimap<string, val, less<string> >::value_type(id, StringToVal(va)));
        bindungen.erase(H);
        bindungen.insert(H + "," + id + "=" + va);
      }
    }
  } // while ValueTerm nicht leer
}  // net::InAnalyse

set<string, less<string> >
net::SearchBinding(const transition &tra)
  // gibt es ein Binding, unter dem diese Transition schalten kann?
{
  set<string, less<string> > bs;

  transition actual_trans = tra;

  map<string, set<val, less<val> >, less<string> > Mengen;
  set<val, less<val> > Temp;
  string hj = actual_trans.Name();
  set<string, less<string> > input_edges = InputEdges(hj);
  set<string, less<string> > output_edges = OutputEdges(hj);
  string H, T, bindung, t, te;
  int position;
  unsigned int co;
  string id, va;
  set<string, less<string> > Bindungen3, Bindungen2;
  val wert(0);
  multiset<string, less<string> > variablen;
  edge *hledge;
  place *inp;

  PreAnalyse(&actual_trans);

  bindungen.erase(bindungen.begin(), bindungen.end());
  if(BelegteString != "")
    bindungen.insert(BelegteString);

  for(set<string, less<string> >::iterator i = input_edges.begin(); i != input_edges.end(); ++i) {
    te = *i;
    hledge = &Edges[te];
    inp = &Places[hledge->Src()];
    multiset<val, less<val> > vars_in_edge = hledge->Vars();
    multiset<val, less<val> > tmp_vals = inp->Marking();
    for(multiset<val, less<val> >::iterator k = vars_in_edge.begin(); k != vars_in_edge.end(); ++k) {
      variablen = (*k).Strings();
      if(!variablen.empty()) {
        for(multiset<string, less<string> >::iterator v = variablen.begin(); v != variablen.end(); ++v) {
          te = *v;
          if(BelegteVars.contains(te))
            continue;
          position = (*k).find(te);
          for(multiset<val, less<val> >::iterator u = tmp_vals.begin(); u != tmp_vals.end(); ++u) {
            wert = (*u).get(position).asString();
            Temp.insert(wert);
          }
          if(Mengen.find(te) == Mengen.end()) {
            Mengen[te] = Temp;
          } else {
            set<val, less<val> > neu;// = Mengen[te];
            val te2;
            for(set<val, less<val> >::iterator r = Temp.begin(); r != Temp.end(); ++r) {
              te2 = *r;
              if((*Mengen.find(te)).second.find(te2) != (*Mengen.find(te)).second.end())
                neu.insert(te2);
            }
            Mengen[te] = neu;
          }
          Temp.erase(Temp.begin(), Temp.end());
        }
      }
    }
  }
  for(set<string, less<string> >::iterator i = output_edges.begin(); i != output_edges.end(); ++i) {
    te = *i;
    hledge = &Edges[te];
    inp = &Places[hledge->Dest()];
    multiset<val, less<val> > vars_in_edge = hledge->Vars();
    set<val, less<val> > tmp_vals = inp->Type();
    for(multiset<val, less<val> >::iterator k = vars_in_edge.begin(); k != vars_in_edge.end(); ++k) {
      variablen = (*k).Strings();
      if(!variablen.empty()) {
        for(multiset<string, less<string> >::iterator v = variablen.begin(); v != variablen.end(); ++v) {
          te = *v;
          if(BelegteVars.contains(te))
            continue;
          position = (*k).find(te);
          for(set<val, less<val> >::iterator u = tmp_vals.begin(); u != tmp_vals.end(); ++u) {
            wert = (*u).get(position).asString();
            Temp.insert(wert);
          }
          if(Mengen.find(te) == Mengen.end()) {
            Mengen[te] = Temp;
          } else {
            set<val, less<val> > neu;// = Mengen[te];
            val te2;
            for(set<val, less<val> >::iterator r = Temp.begin(); r != Temp.end(); ++r) {
              te2 = *r;
              if((*Mengen.find(te)).second.find(te2) != (*Mengen.find(te)).second.end())
                neu.insert(te2);
            }
            // neu &= Temp;
            Mengen[te] = neu;
          }
          Temp.erase(Temp.begin(), Temp.end());
        }
      }
    }
  }

  for(map<string, set<val, less<val> >, less<string> >::iterator i = Mengen.begin(); i != Mengen.end(); ++i) {
    if((*i).second.size() == 0) {
      bs.erase(bs.begin(), bs.end());
      bindungen.erase(bindungen.begin(), bindungen.end());
      return bs;
    }
    /*cerr << "Var:" << (*i).first << "-" << (*i).second.size();
    for(set<val, less<val> >::iterator s = (*i).second.begin(); s != (*i).second.end(); ++s) {
      cerr << " 1" << (*s).asString();
    }
    cerr << endl;*/ 
  }

    // Alle Eingangskanten durchlaufen
  for (set<string, less<string> >::iterator i = input_edges.begin(); i != input_edges.end(); ++i) {
    edge *e = &Edges[(*i)]; 
    place *p = &Places[e->Src()];
      // Tokens holen, die auf aktueller Eingangstelle sind
    multiset<val, less<val> > marking = p->Marking();
    multiset<val, less<val> > vars_in_edge = e->Vars();
    for (multiset<val, less<val> >::iterator k = vars_in_edge.begin(); k != vars_in_edge.end(); ++k) {
        // Die mglichen Werte bedingt durch die Werte der auf der Stelle
        // liegenden Tokens.
        // Zwischenspeicher fr Werte
      set<val, less<val> > tmp_vals;
      int gefunden = 0;
      Bindungen3 = bindungen;
      for (multiset<val, less<val> >::iterator n = marking.begin(); n != marking.end(); ++n) {
        if((*n).EQUAL((*k))) {
         // nur wenn die Markierung = der Kantenannotation
          gefunden = 1;
          // Bindungen3 = bindungen;
          if(!Bindungen3.empty()) {
            for(set<string, less<string> >::iterator nb = Bindungen3.begin(); nb != Bindungen3.end(); ++nb) {
              t = "";
              bindung = (*nb);
              bindungen.erase(bindung);
              variablen = (*k).Strings();
              for(set<string, less<string> >::iterator v = variablen.begin(); v != variablen.end(); ++v) {
                if(!BelegteVars.contains(*v)) {
                  position = (*k).find((*v));
                  wert = (*n).get(position);
                  if(t != "")
                    t += ",";
                  t += (*v) + "=" + wert.asString();
                }
              } // for v "uber alle Variablen in der Kantenannotation
              multiset<val, less<val> > t_vars2 = vars_in_edge;
              multiset<val, less<val> > t_marking2 = marking;
              t_vars2.erase((*k));
              t_marking2.erase((*n));
              if(t != "" && bindung != "")
                possible(t_vars2,t_marking2,bindung+","+t);
              else if(t != "")
                possible(t_vars2,t_marking2,t);
              else
                possible(t_vars2,t_marking2,bindung);
            } // for nb "uber alle m"oglichen Bindungen
          } // for n "uber alle Markierungen der Vorbereichsstelle
          else {
            t = "";
            variablen = (*k).Strings();
            for(set<string, less<string> >::iterator v = variablen.begin(); v != variablen.end();++v) {
              if(!BelegteVars.contains((*v))) {
                position = (*k).find((*v));
                wert = (*n).get(position);
                if(t != "")
                  t += ",";
                t += (*v) + "=" + wert.asString();
              }
            } // for v "uber alle m"oglichen Variablen in der Kantenannotation
            multiset<val, less<val> > t_vars2 = vars_in_edge;
            multiset<val, less<val> > t_marking2 = marking;
            for(multiset<val, less<val> >::iterator v = t_vars2.begin(); v != t_vars2.end(); ++v) {
              if((*v) == (*k)) {
                t_vars2.erase(v);
                break;
              }
            }
            for(multiset<val, less<val> >::iterator v = t_marking2.begin(); v != t_marking2.end(); ++v) {
              if((*v) == (*n)) {
                t_marking2.erase(v);
                break;
              }
            }
            if(t != "" && bindung != "")
                possible(t_vars2,t_marking2,bindung+","+t);
              else if(t != "")
                possible(t_vars2,t_marking2,t);
              else
                possible(t_vars2,t_marking2,bindung);
          }  // wenn Bindungen noch nicht belegt war 
        }
      }  // for Pix n
      if(!gefunden) {
        bs.erase(bs.begin(), bs.end());
        bindungen.erase(bindungen.begin(), bindungen.end());
        return bs;
      } else {
        if(bindungen.empty())
          bindungen.insert("");
      }
      // gefunden = 0;
    }  // for Pix
  }  // for i ber alle Eingangskanten

  string s = "";
  if (bindungen.begin() != bindungen.end())
    s = *(bindungen.begin());
  multiset<string, less<string> > vari;
  while(s.find(" ") != string::npos)
    s.serase(s.find(" "), 1);
  while (s != "") {
    co = s.find("=");
    id = s.substr(0,co);
    co = s.find(",");
    if(co != string::npos)
      s = s.substr(co+1,1000);
    else
      s = "";
    vari.insert(id);
  }

  int zahl1 = 0;
  int zahl2 = BelegteVars.length();
  int drin = 0;
  while(zahl1 != zahl2 | drin == 0) {
    drin = 1;
    zahl1 = BelegteVars.length();
    InAnalyse(&actual_trans);
    zahl2 = BelegteVars.length();
  }

  /*for(set<string, less<string> >::iterator u = bindungen.begin(); u != bindungen.end(); ++u) {
    String help = bindungen(u);
    if(help[0] == ',') {
      bindungen.del(help);
      help = help.after(",");
      bindungen.add(help);
      u = bindungen.first();
    }
    cerr << (*u) << endl;
  }*/

    // Alle Ausgangskanten durchlaufen
  Bindungen2 = bindungen;
  for (set<string, less<string> >::iterator i = output_edges.begin(); i != output_edges.end(); ++i) {
    multiset<val, less<val> > vars_in_edge = Edges[(*i)].Vars();
    set<val, less<val> > tmp_vals;
    for (multiset<val, less<val> >::iterator k = vars_in_edge.begin(); k != vars_in_edge.end(); ++k) {
      variablen = (*k).Strings();
      set<val, less<val> > belegungen;
      for(set<string, less<string> >::iterator v = variablen.begin(); v != variablen.end(); ++v) {
        te = *v;
        if(BelegteVars.contains(te) | vari.find(te) != vari.end()) {
          continue;
        }

        tmp_vals = Mengen[te];
        Bindungen2 = bindungen;
        for(set<string, less<string> >::iterator h = bindungen.begin(); h != bindungen.end(); ++h) {
          T = (*h);
          string tem = T;
          Bindungen2.erase(tem);
          for(set<val, less<val> >::iterator m = tmp_vals.begin(); m != tmp_vals.end(); ++m) {
            H = T;
            if(H != "")
              H += ",";
            H += (*v) + "=" + (*m).asString();
            vari.insert(te);
            bind.erase(bind.begin(), bind.end());
            ParseAsignment(H);
            if(actual_trans.EvaluateVT(bind) == true & CheckBinding(actual_trans)) {
              bs.insert(H);
              return bs;
            }
            Bindungen2.insert(H);
          } // for m "uber alle m"oglichen Belegungen einer Variable 
        }
        bindungen = Bindungen2;
      } // for v "uber alle Variablen in einer Kantenannotation
    }  // for k "uber alle Kantenannotationen
  }  // for i ber alle Ausgangskanten

  BelegteVars.erase(BelegteVars.begin(), BelegteVars.end());
  BelegteString = "";

  for(set<string, less<string> >::iterator o = bindungen.begin(); o != bindungen.end(); ++o) {
    bind.erase(bind.begin(), bind.end());
    ParseAsignment((*o));
    if(actual_trans.EvaluateVT(bind) == true & CheckBinding(actual_trans)) {
      bs.insert((*o));
      return bs;
    }
  }
  return bs;
}  // net::SearchBinding

void
net::possible(multiset<val, less<val> > &t_vars, multiset<val, less<val> > &t_marking, string bindung)
{
  string t;
  multiset<string, less<string> > variablen;
  val wert;
  int position;
  if(t_vars.size() == 0) {
    bind.erase(bind.begin(), bind.end());
    ParseAsignment(bindung);
    multimap<string, val, less<string> >::iterator i = bind.begin();
    if(i != bind.end()) {
      string help = bind.key(i) + "=" + (*i).second.asString();
      ++i;
      for(;i != bind.end();++i) {
        help += "," + bind.key(i) + "=" + (*i).second.asString();
      }
      bindungen.insert(help);
    }
  }
  else {
    for(multiset<val, less<val> >::iterator i = t_vars.begin(); i != t_vars.end();++i) {
      set<val, less<val> > tmp_vals;
      int gefunden = 0;
      for(multiset<val, less<val> >::iterator j = t_marking.begin(); j != t_marking.end(); ++j) {
        if((*i).EQUAL(*j)) {
          gefunden = 1;
          variablen = (*i).Strings();
          t = "";
          for(multiset<string, less<string> >::iterator v = variablen.begin(); v != variablen.end(); ++v) {
            if(BelegteVars.contains((*v)))
              continue;
            position = (*i).find(*v);
            wert = (*j).get(position);
            if(t != "")
              t += ",";
            t += (*v) + "=" + wert.asString();
          }
          multiset<val, less<val> > t_vars2 = t_vars;
          multiset<val, less<val> > t_marking2 = t_marking;
          for(multiset<val, less<val> >::iterator v = t_vars2.begin(); v != t_vars2.end(); ++v) {
            if((*i) == (*v)) {
              t_vars2.erase(v);
              break;
            }
          }
          for(multiset<val, less<val> >::iterator v = t_marking2.begin(); v != t_marking2.end(); ++v) {
            if((*j) == (*v)) {
              t_marking2.erase(v);
              break;
            }
          }
          if(t != "" && bindung != "")
            possible(t_vars2,t_marking2,bindung+","+t);
          else if(t != "")
            possible(t_vars2,t_marking2,t);
          else
            possible(t_vars2,t_marking2,bindung);
        }
      }
      if(!gefunden) {
        return;
      }
    }
  }
} // net::possible

void
net::ParseAsignment(const string &asignment_string)
  // Einen String in ein Binding umwandeln.
{
  string s = asignment_string;
  while(s.find(" ") != string::npos)
    s.serase(s.find(" "), 1);
  bind.erase(bind.begin(), bind.end());
  unsigned int co;
  while (s != "") {
    co = s.find("=");
    string id = s.substr(0,co);
    string v;
    if(co != string::npos)
      v = s.substr(co+1,1000);
    else
      v = "";
    if (v.at(0) == '(') {
      co = v.find("(");
      if(co != string::npos)
        v = v.substr(co+1,1000);
      else
        v = "";
      string str = v.substr(0,co);
      string tok = str;
      co = tok.find(")");
      tok = tok.substr(0,co);
      int count = 0;
      while(tok != "") {
        co = tok.find(",");
        if(co != string::npos)
          tok = tok.substr(co+1,1000);
        else
          tok = "";
        count++;
      }
      Tupel tup(count);
      string tu;
      while(count) {
        co = str.find(",");
        tu = str.substr(0,co);
        tup.add(val(tu));
        if(co != string::npos)
          str = str.substr(co+1,1000);
        else
          str = "";
        count--;
      }
      // Val v1 = StringToVal(v.after("(").before(","));
      // Val v2 = StringToVal(v.after(",").before(")"));
      bind.add(id, tup);
      co = v.find("),");
      if(co != string::npos)
        s = v.substr(co+2,1000);
      else
        s = "";
    }  // if
    else {
      co = v.find(",");
      if(co != string::npos)
        s = v.substr(co+1,1000);
      else
        s = "";
      if((bind.contains(id)) && (StringToVal(v) != bind[id])) {
        return;
      }
      if (s == "") {
        if(!bind.contains(id))
          bind.add(id, StringToVal(v));
      }
      else {
        co = v.find(",");
        v = v.substr(0,co);
        if(!bind.contains(id))
          bind.add(id, StringToVal(v));
      }  // else
    }  // else
  }  // while s nicht alle
}  // net::ParseAsignment


int
net::CheckBinding(transition &t)
  // Es wird geprft, ob das Binding mit den Markierungen auf
  // den Eingangsstellen zusammenpat, da vorher evtl. zuviele
  // Marken (farbige) im Binding verwendet werden. Das Problem
  // wird teilweise auch durch mit echten Multimengen beschriftete
  // Kanten erzeugt.
  // VORHER EvaluateVT aufgerufen haben!!
  // VORHER ParseAsignment augerufen haben!!
{
    // Markierung auf den Eingangsstellen zusammen mit den
    // Variablen der Eingangskanten kopieren, um das Schalten
    // zu simulieren.

  string hj = t.Name();
  set<string, less<string> > in_edges = InputEdges(hj);
  set<string, less<string> > out_edges = OutputEdges(hj);
  multiset<val, less<val> > cmark;
  for (set<string, less<string> >::iterator i = in_edges.begin(); i != in_edges.end(); ++i) {
    multiset<val, less<val> > smark;
    multiset<val, less<val> > vars = Edges[(*i)].Vars();
    for (multiset<val, less<val> >::iterator j = vars.begin(); j != vars.end(); ++j) {
      if((*j).hasStrings()) {
        multiset<string, less<string> > variablen = (*j).Strings();
        val wert = (*j);
        for(multiset<string, less<string> >::iterator b = variablen.begin(); b != variablen.end(); ++b) {
          int position = (*j).find((*b));
          val w = bind.wert((*b));
          wert.set(w, position);
        } // for b "uber alle Variablen in einer Kantenannotation
        smark.insert(wert);
      } 
      else {
        smark.insert((*j));
      }
    } // for j "uber alle Kantenannotationen
    cmark = Places[Edges[(*i)].Src()].Marking();

    for(multiset<val, less<val> >::iterator m = smark.begin(); m != smark.end(); ++m) {
      if(cmark.find((*m)) != cmark.end())
        cmark.erase((*m));
      else
        return 0;
    } // for i "uber alle Eingangskanten
  } // for i "uber alle Eingangskanten

  for (multiset<string, less<string> >::iterator i = out_edges.begin(); i != out_edges.end(); ++i) {
    edge *e = &Edges[(*i)]; 
    place *p = &Places[e->Dest()];
    multiset<val, less<val> > vars_in_edge = e->Vars();
    set<val, less<val> > tmp_vals = p->Type();
    for (multiset<val, less<val> >::iterator k = vars_in_edge.begin(); k != vars_in_edge.end(); ++k) {
      multiset<string, less<string> > variablen = (*k).Strings();
      for (multiset<string, less<string> >::iterator g = variablen.begin(); g != variablen.end(); ++g) {
        if(!bind.contains(*g)) {
          return 0;
        }
      }
    }
  }
  return 1;
}  // net::CheckBinding

void
net::FireTransition(string &t_name, string &bind_string)
  // Eine Transition schalten.
{
  string te, te2;
  transition *t = &Transitions[t_name];
    // Zuweisungs-String parsen und in Bindung bind eintragen
  ParseAsignment(bind_string);
    // Jetzt testen, ob Transition unter der Bindung bind schalten kann
  if ((t->EvaluateVT(bind) != true) /*|| !CheckBinding(*t)*/) {
        // Nein -> dann Fehlermeldung
    cerr << "The transition cannot fire under this binding" << endl;
  }
  else {  // Ja -> dann schalten
    set<string, less<string> > in_edges = InputEdges(t_name);
    set<string, less<string> > out_edges = OutputEdges(t_name);
    for (set<string, less<string> >::iterator i = in_edges.begin(); i != in_edges.end(); ++i) {
      te = *i;
      multiset<val, less<val> > smark;
      multiset<val, less<val> > vars = Edges[te].Vars();
      for (multiset<val, less<val> >::iterator j = vars.begin(); j != vars.end(); ++j) {
        if((*j).hasStrings()) {
          multiset<string, less<string> > variablen = (*j).Strings();
          val wert = (*j);
          for(multiset<string, less<string> >::iterator b = variablen.begin(); b != variablen.end(); ++b) {
            te2 = *b;
            int position = (*j).find(te2);
            val w = bind.wert(te2);
            wert.set(w, position);
          } // for b "uber alle Variablen in einer Kantenannotation
          smark.insert(wert);
        } 
        else {
          val wert = *j;
          smark.insert(wert);
        }
        Places[Edges[te].Src()].SubtractMarking(smark);
      } // for j "uber alle Kantenannotationen
    } // for i "uber alle Eingangskanten

    for (multiset<string, less<string> >::iterator i = out_edges.begin(); i != out_edges.end(); ++i) {
      te = *i;
      multiset<val, less<val> > smark;
      multiset<val, less<val> > vars = Edges[te].Vars();
      val wert;
      for (multiset<val, less<val> >::iterator j = vars.begin(); j != vars.end(); ++j) {
        if((*j).hasStrings()) {
          multiset<string, less<string> > variablen = (*j).Strings();
          wert = (*j);
          for(multiset<string, less<string> >::iterator b = variablen.begin(); b != variablen.end(); ++b) {
            /*int position = (*j).find((*b));
            val w = bind[(*b)];
            wert.set(w,position);*/
            wert.replaceAll((*b), bind[(*b)]);
          } // for b "uber alle Variablen in einer Kantenannotation
          smark.insert(wert);
        }
        else {
          val wert = *j;
          smark.insert(wert);
        }
      } // for j "uber alle Kantenannotationen
      Places[Edges[te].Dest()].AddMarking(smark);
    }  // for i ber die Ausgangskanten
  }  // else, Schalten geht klar
}  // net::FireTransition


void
net::RenameTransition(string &old, string &neu)
  // eine Transition wird umbenannt
{
  Transitions.insert(StringTransMap::value_type(neu, Transitions[old]));
  Transitions.erase(old);
  for(StringEdgeMap::iterator i = Edges.begin(); i != Edges.end(); ++i) {
    if((*i).second.Src() == old) {
      (*i).second.ChangeSrc(neu);
    }
    if((*i).second.Dest() == neu) {
      (*i).second.ChangeDest(neu);
    }
  }
}  // net::RenameTransition


void
net::RenamePlace(string &old, string &neu)
  // eine Stelle wird umbenannt
{
  string type = Places[old].TypeString();
  string marking = Places[old].MarkingString();
  Places.erase(old);
  place p(neu);
  p.ChangeMarking(marking);
  p.ChangeType(type);
  Places.insert(StringPlaceMap::value_type(p.Name(), p));
  for(StringEdgeMap::iterator i = Edges.begin(); i != Edges.end(); ++i) {
    if((*i).second.Src() == old) {
      (*i).second.ChangeSrc(neu);
    }
    if((*i).second.Dest() == neu) {
      (*i).second.ChangeDest(neu);
    }
  }
}  // net::RenamePlace


void
net::ChangeValueTerm(string &t, string &vt)
  // den Valueterm einer Transition "andern
{
  Transitions[t].ChangeValueTerm(vt);
}  // net::ChangeValueTerm

void
net::ChangeTyp(string &p, string &typ)
  // den Typ einer Stelle "andern
{
  Places[p].StringToType(typ);
}  // net::ChangeTyp

void
net::ChangeMarking(string &p, string &mark)
  // die Markierung einer Stelle "andern
{
  Places[p].ChangeMarking(mark);
}  // net::ChangeMarking

void
net::ChangeVars(string &b1, string &b2, string &vars)
  // die Kantenannotation "andern
{
  for(StringEdgeMap::iterator i = Edges.begin(); i != Edges.end(); ++i) {
    if((*i).second.Src() == b1) {
      for(StringEdgeMap::iterator j = Edges.begin(); j != Edges.end(); ++j) {
        if((*j).second.Dest() == b2) {
          if(i == j) {
            (*i).second.ChangeVars(vars);
            break;
          }
        }
      }
    }
  }
}  // net::ChangeVars

void
net::NewPlace(string &p)
  // eine neue Stelle im Netz erstellen
{
  place pl(p);
  Places.insert(StringPlaceMap::value_type(pl.Name(), pl));
  string PL = pl.Name();
  string le("");
  string DOT("{dot}");
  ChangeMarking(PL, le);
  ChangeTyp(PL, DOT);
}  // net::NewPlace


void
net::NewTransition(string &t)
  // eine neue Transition im Netz erstellen
{
  transition tr(t);
  Transitions.insert(StringTransMap::value_type(tr.Name(), tr));
}  // net::NewTransition

void
net::NewEdge(string &from, string &to)
  // eine neue Kante von from nach to erzeugen
{
  edge e(this, from, to);
  string A("a");
  e.ChangeVars(A);
  Edges.insert(StringEdgeMap::value_type(e.Name(), e));
}  // net::NewEdge

void
net::DeletePlace(string &p)
  // eine Stelle wird gel"oscht
{
  for(StringEdgeMap::iterator i = Edges.begin(); i != Edges.end(); ++i) {
    if((*i).second.Src() == p || (*i).second.Dest() == p)
      Edges.erase(i);
  }
  Places.erase(p);
}

void
net::DeleteTransition(string &t)
  // eine Transition wird gel"oscht
{
  for(StringEdgeMap::iterator i = Edges.begin(); i != Edges.end(); ++i) {
    if((*i).second.Src() == t || (*i).second.Dest() == t)
      Edges.erase(i);
  }
  Transitions.erase(t);
}

void
net::DeleteEdge(string &from, string &to)
  // eine Kante von from nach to wird gel"oscht
{
  for(StringEdgeMap::iterator i = Edges.begin(); i != Edges.end(); ++i) {
    if((*i).second.Src() == from && (*i).second.Dest() == to) {
      Edges.erase(i);
      break;
    }
  }
}

