#ifndef VAL
#define VAL

using namespace std;


#include <iostream.h>
#include <string>
#include <map>
#include <multimap.h>
#include <set>
#include <list>
#include <stack>
#include "Enums.h"
#include "ExprInterface.h"

#if defined (__GNUC__) && __GNUC__ <= 2 && __GNUC_MINOR__ < 8
#define serase remove
#else
#define serase erase
#endif

class VT;


extern "C" { int  parse_VT(VT& a); }


    
class Tupel;

    
class val {               // Wert -> Integer, Dot, Bool, String
private:

    friend ostream& operator <<(ostream& s, const val& v);

    ValueType vtype;      // hierdurch angezeigt
    
    // int value;            // und der dazugehrige Wert

    union {
	int    value;
	Tupel* tupel;
        string* String;
    };

public:

    val          pr0() const;
    val          pr1() const;

    val operator  &(const val& a) const;
    val operator  |(const val& a) const;
    val         imp(const val& a) const;
    val operator  +(const val& a) const;
    val operator  -(const val& a) const;
    val operator  *(const val& a) const;
    val operator  /(const val& a) const;
    val operator  %(const val& a) const;
    int operator ==(const val& a) const;
    int operator !=(const val& a) const;
    int operator  <(const val& a) const;
    int operator  >(const val& a) const;
    int operator <=(const val& a) const;
    int operator >=(const val& a) const;
    val operator  !() const;

    const val& operator =(const val& v);

    int EQUAL(const val& a) const;

    string asString(void) const;

    int hasStrings(void) const;

    multiset<string, less<string> > Strings(void) const;

    int find(string s) const;

    val get(int i) const;
    void set(val v, int i);
    void replaceAll(string s, val a);

    ValueType getType(void) const {
        return vtype;
    }

    ValueType Type(void) const {
	return vtype;
    }

    int asInt(void) const {
	return value;
    }

    val() : vtype(INTEGER), value(0) {};
   
    val(const int a) : vtype(INTEGER), value(a) {};

    val(const bool a) : vtype(BOOLEAN), value(int(a)) {};

    val(const DotType a) : vtype(DOT), value(-12345) {};
    
    val(const val& v1, const val& v2);

    val(const string& st);

    val(const Tupel& t);

    val(const val& v);

    ~val();
};


/*inline unsigned int 
Hash(const val& v)
{
    return 1;
}*/

inline int
LessEq(const val& a, const val& b)
{
    return a.asInt()<=b.asInt();
}


class Tupel {
private:

    val* v;
    int count;
    int dimension;

public:

    Tupel(int capacity) {
      v = new val[capacity];
      dimension = capacity;
      count = 0;
    };

    Tupel(val a, val b) {
      v = new val[2];
      v[0] = a;
      v[1] = b;
      dimension = count = 2;
    };

    Tupel() {
      v = new val[2];
      v[0] = val(0);
      v[1] = val(0);
      dimension = count = 2;
    };

    Tupel(const Tupel& a) {
      v = new val[a.dimension];
      for(int i = 0;i < a.dimension;i++) {
        v[i] = a.v[i];
      }
      dimension = a.dimension;
      count = a.count;
    };

    int Dim() {return dimension;};

    ~Tupel() {
      delete [] v;
    };

    val pr0() const {
        return v[0];
    };

    val pr1() const {
        return v[1];
    };

    val get(int i) const {
        if(i < dimension) {
          return v[i];
        }
        return val(0); // damit der Compiler Ruhe gibt
    };

    void set(val a, int i) {
        if(i < dimension) {
          v[i] = a;
        }
    };

    int find(string s) const {
        for(int i = 0;i < dimension;i++) {
           if(v[i].find(s))
             return i;
        }
        return -1;
    };

    void replaceAll(string s, val a) {
        for(int i = 0;i < dimension;i++) {
          if(v[i] == s)
            set(a,i);
        }
    };

    void add(val a) {
        v[count++] = a;
    };
    
    int operator ==(const Tupel& a) const {
        if(dimension != a.dimension)
          return 0;
        for(int i = 0; i < dimension; i++) {
          if(v[i] != a.v[i])
            return 0;
        }
        return 1;
    };

    int EQUAL(const Tupel& a) const {
        if(dimension != a.dimension)
          return 0;
        for(int i = 0; i < dimension; i++) {
          if(!v[i].EQUAL(a.v[i]))
            return 0;
        }
        return 1;
    };

    string asString() const {
        string temp;
        if(dimension > 1)
          temp = "(";
        for(int i = 0;i < dimension;i++) {          
           temp += v[i].asString();
           if(i < dimension-1)
             temp += ",";
        }
        if(dimension > 1)
          temp += ")";
        return temp;
    };

    int hasStrings() const {
       for(int i = 0;i < dimension;i++) {
         if(v[i].getType() == STRING)
           return 1;
       }
       return 0;
    };

    multiset<string, less<string> > Strings() const {
       multiset<string, less<string> > var;
       for(int i = 0;i < dimension;i++) {
         if(v[i].getType() == STRING)
           var.insert(v[i].asString());
       }
       return var;
    };
};


/*inline unsigned int 
Hash(const Tupel& a)
{
    return 1;
}*/

class VTNode {
private:
    
   
    NodeType node_type;

   
    union {                       // Knoten kann sein
	
	val* v;                   // Wert (hier Pointer da in unions keine
				  // Klassen mit Konstruktoren zugelassen)
	    
	string* ident;          // oder aber Variable

	OperatorType otype;       // oder Operator.

	// Proj ptype;

    };
    
    void Delete(void);

public:

    NodeType Type(void) {
	return node_type;
    };

    OperatorType Operator() {
	return otype;
    };

    const val& Value() {
	return *v;
    };

    /* Proj Projection() const {
	return ptype;
    }; */

    const string& Identifier() {
	return *ident;
    };

    int operator ==(const VTNode& v) const;

    VTNode& operator =(const VTNode& v);

    VTNode(const VTNode& a);
    
    VTNode(const int a)          : node_type(VALUE), v(new val(a)) {};
    
    VTNode(const bool a)     : node_type(VALUE), v(new val(a)) {};

    VTNode(const DotType a)      : node_type(VALUE), v(new val(a)) {};

    VTNode(const string& s)    : node_type(VARIABLE), 
	ident(new string(s)) {};

    VTNode(const char* s)        : node_type(VARIABLE), 
	ident(new string(s)) {};

    VTNode(const OperatorType t) : node_type(OPERATOR), otype(t) {};


    // VTNode(Proj a) : node_type(PROJECTION) , ptype(a) {};

    VTNode() : node_type(VALUE), v(new val(true)) {};


    ~VTNode() {
	Delete();
    }; 
};


// class TupelBinding : public VHMap<String, Tupel> {
// public:

//     String asString(void) const;

//     TupelBinding() : VHMap<String, Tupel>(Tupel(0,0), 10) {};
// };

/*class Binding : public multimap<string,val,less<string> > {
public:
    string asString(void) const;
    Binding() : multimap<string,val,less<string> >() {};
};*/

class Binding {
protected:
  multimap<string, val, less<string> > b;
public:
  string asString(void);
  Binding() {};
  void add(string s, val v) {b.insert(multimap<string,val, less<string> >::value_type(s,v));};
  void insert(multimap<string, val, less<string> >::value_type vt) {b.insert(vt);};
  val wert(string s) {return (*b.find(s)).second;};
  multimap<string, val, less<string> >::iterator find(string s) {
    return b.find(s);
  }
  multimap<string, val, less<string> >::iterator begin() {
    return b.begin();
  };
  multimap<string, val, less<string> >::iterator end() {
    return b.end();
  };
  string key(multimap<string, val, less<string> >::iterator i) {
    return (*i).first;
  };
  void erase(multimap<string, val, less<string> >::iterator i, multimap<string, val, less<string> >::iterator j) {
    b.erase(i,j);
  };
  int contains(string s) {
    return (b.find(s) != b.end());
  };
  val operator[] (const string s) {
    return wert(s);
  };
  int length() {
    return b.size();
  };
};
    
class VT {

    friend void new_ident(char *s);
    friend void new_number(int a);
    friend void new_number_dot(void);
    friend void new_number_bool(bool);
    friend void new_operator(OperatorType t);
    friend void new_value(int);
    friend void new_values(int,int);
    friend void store_last(void);
    friend void new_value_bool(bool);
    friend void new_value_dot(void);
    friend void make_cross(void);
    friend int  parse_VT(VT& a);

protected:
    
    Binding bind;

    string string_term;
    
    set<string, less<string> > vars_in_VT;

    list<VTNode> upn_list;

    map<string, set<val, less<val> >, less<string> > intern_set;

    val EvaluateDict(Binding& b, const map<string, set<val, less<val> >, less<string> >::iterator& i);

    val FixedEvaluate(Binding& bb);
    
public:

    void ChangeTerm(const string& new_term);

    const string& Text(void) const {
	return string_term;
    };

    val Evaluate(Binding& b);
    
    VT(const string& ct) : string_term(ct), intern_set() {
	parse_VT(*this);
    };


};

string IntToString(int i);
val StringToVal(const string& val_string);
int IsVal(const string& val_string);
Tupel StringToTupel(string t);


#endif
