# BlockView.Tcl, graphische Darstellung der Blockstruktur
# Dialoge und Prozeduren fr den Netzeditor
# last-update: 10.1.98

proc UpdateBlockTreeDialog {} {
  global PT Dialog Id2Name

  catch {
    $PT(block_tree.canvas) delete all
    BlockTreeInit $PT(block_tree.canvas)
  }
}
  

proc BlockTreeDialog {} {
  # Darstellung der Blockstruktur als Baum.
  # Der Dialog ist nicht-modal!
  global PT Dialog Id2Name

  destroy .btd

  set t [toplevel .btd]
  wm title $t {Hierarchy Tree Dialog}

  set Dialog(block_tree.name) {}
  set Dialog(block_tree.mean) {}
  set Dialog(block_tree.blockvis) {}

    # Canvas
  frame $t.cf
  pack $t.cf -side top -fill both -expand 1 -padx 2 -pady 2
  set c [canvas $t.cf.canv -width 300 -height 200 -background white \
    -xscrollcommand "$t.cf.pad.xscroll set" \
    -yscrollcommand "$t.cf.yscroll set" \
    -scrollregion {0 0 1000 1000} \
    -relief sunken -borderwidth 3]
  set PT(block_tree.canvas) $c
  scrollbar $t.cf.yscroll -command "$t.cf.canv yview" \
    -relief sunken -orient vertical -width 12
  frame $t.cf.pad
  scrollbar $t.cf.pad.xscroll -command "$t.cf.canv xview" \
    -relief sunken -orient horizontal -width 12
  set pad [expr {[$t.cf.yscroll cget -width] + 2 * \
    ([$t.cf.yscroll cget -bd] + \
     [$t.cf.yscroll cget -highlightthickness])}]
  frame $t.cf.pad.it -width $pad -height $pad
  pack $t.cf.pad -side bottom -fill x
  pack $t.cf.pad.it -side right
  pack $t.cf.pad.xscroll -side bottom -fill x
  pack $t.cf.yscroll -side right -fill y
  pack $t.cf.canv -side left -expand yes -fill both

    # Entry Area
  frame $t.top -relief raised -bd 2
  pack $t.top -side top -fill x -expand 0 -padx 2m -pady 2m

  frame $t.top.name
  label $t.top.name.label -text "Name" -width 10 -anchor w
  label $t.top.name.label2 -textvariable Dialog(block_tree.name) \
    -anchor w
  pack $t.top.name.label -side left -fill x
  pack $t.top.name.label2 -side left -fill x -expand 1
  pack $t.top.name -padx 3m -pady 2m -side top -fill x -expand 1

  frame $t.top.mean
  label $t.top.mean.label -text "Meaning" -width 10 -anchor w
  entry $t.top.mean.label2 -textvariable Dialog(block_tree.mean) \
    -relief sunken
  pack $t.top.mean.label -side left -fill x
  pack $t.top.mean.label2 -side left -fill x -expand 1
  pack $t.top.mean -padx 3m -pady 2m -side top -fill x -expand 1

  frame $t.top.blockvis -relief groove -bd 2
  label $t.top.blockvis.label -text "Block Visibility"
  set Dialog(block_tree.blockvis_widget) [frame $t.top.blockvis.radio]
  radiobutton $t.top.blockvis.radio.exposed -text "Exposed" -value exposed
  radiobutton $t.top.blockvis.radio.hidden -text "Hidden" -value hidden
  radiobutton $t.top.blockvis.radio.exposed_contents -text "Exposed Contents" \
    -value exposed_contents
  foreach rb {exposed hidden exposed_contents} {
    $Dialog(block_tree.blockvis_widget).$rb configure -anchor w
    $Dialog(block_tree.blockvis_widget).$rb configure -state disabled
    $Dialog(block_tree.blockvis_widget).$rb configure \
      -variable Dialog(block_tree.blockvis)
    $Dialog(block_tree.blockvis_widget).$rb configure \
      -command {BlockTree_BlockVis_Change}
  }  ;# foreach RadioButton
  pack $t.top.blockvis -fill x -expand 1
  pack $t.top.blockvis.label \
    $t.top.blockvis.radio \
    -fill x -expand 1 -side left
  pack $t.top.blockvis.radio.exposed \
    $t.top.blockvis.radio.hidden \
    $t.top.blockvis.radio.exposed_contents \
    -fill x -expand 1 -side top -anchor w

  BlockTreeInit $c

    # Action Area
  frame $t.action -relief raised -bd 2
  pack $t.action -side top -fill both

  frame $t.action.default -relief sunken -bd 1
    # Ok-Button
  button $t.action.default.ok -text "Ok" \
    -command {set Dialog(block_tree.button) Ok}
  pack $t.action.default -side left -expand 1 -padx 3m -pady 2m
  pack $t.action.default.ok 
    # Expose all-Button
  button $t.action.exposed_contents_all -text "Exposed Contents All" \
    -command {BlockTree_BlockVis_Change expose_all}
  pack $t.action.exposed_contents_all \
    -side left -expand 1 -padx 3m -pady 2m
    # Cancel-Button
  button $t.action.cancel -text "Cancel" \
    -command {
      global PT

      set blocks [$PT(canvas) find withtag Block]
      foreach b $blocks {
        set n $Id2Name($b)
        catch {
          block$n configure -background white
          $PT(canvas) itemconfigure $b -image block$n
        }
      }
      destroy .btd
      return 
    }

  pack $t.action.cancel -side left -expand 1 -padx 3m -pady 2m

  set old_focus [focus]
  tkwait visibility $t
  focus $t

  tkwait variable Dialog(block_tree.button)
  if {$Dialog(block_tree.button) == "Cancel"} {
    set blocks [$PT(canvas) find withtag Block]
    foreach b $blocks {
      set n $Id2Name($b)
      catch {
        block$n configure -background white
        $PT(canvas) itemconfigure $b -image block$n
      }
    }
    destroy $t
    return 
  }  ;# if Abbruch
  focus $old_focus

  if {$Dialog(block_tree.selected) == {}} {
    return
  }  ;# if kein Block selektiert

  set old_name $Dialog(block_tree.selected)
  set new_name $Dialog(block_tree.selected)

  if {$Dialog(block_tree.name) != $old_name} {
    if {[$PT(net) Exists Block $Dialog(block_tree.name)]} {
      tk_dialog .d {Error} \
        {Block "$Dialog(block_tree.name)" exists already. \
        The new name is not accepted} error 0 {Ok}
    } else {
      $PT(net) Block $block set Name $Dialog(block_tree.name)
      set new_name $Dialog(block_tree.name)
      set c [$PT(canvas) find withtag $old_name]
      $PT(canvas) dtag $old_name
      $PT(canvas) addtag $new_name withtag $c
      set Id2Name($c) $new_name
      if {$PT(NameSwitch)} {
        set n [$PT(canvas) find withtag Name@$old_name]
        unset IsAnnoFrom($n);
        $PT(canvas) delete $n
        CreateName $new_name Block
      }  ;# if Name anzeigen
      if {$PT(MeanSwitch)} {
        set n [$PT(canvas) find withtag Mean@$old_name]
        $PT(canvas) dtag Mean@$old_name
        set IsAnnoFrom($n) $new_name
        $PT(canvas) addtag Mean@$new_name withtag $n
      }  ;# if Meaning angezeigt
    }  ;# else, noch nicht vergeben
  }  ;# if Name des Blocks gendert
  catch {
    if {$Dialog(block_tree.mean) != [$PT(net) Block $new_name get Mean]} {
      $PT(net) Block $new_name set Mean $Dialog(block_tree.mean)
      if {$PT(MeanSwitch)} {
        $PT(canvas) delete Mean@$new_name
        CreateAnno $new_name Block Mean
      }  ;# if Meaning anzeigen
    }  ;# if Meaning des Blocks gendert
  }
  set blocks [$PT(canvas) find withtag Block]
  set boxes  [$PT(canvas) find withtag Box]
  foreach b $blocks {
    set n $Id2Name($b)
    catch {
      block$n configure -background white
      $PT(canvas) itemconfigure $b -image block$n
    }
  }
  foreach b $boxes {
    set n $Id2Name($b)
    catch {
      box$n configure -background grey
      $PT(canvas) itemconfigure $b -image box$n
    }
  }
}  ;# BlockTreeDialog

proc BlockTree_BlockVis_Display {} {
  # Anzeigen des aktuellen Zustandes der Blocksichtbarkeit fr den
  # gewhlten Block.
  # Aus der globale Variable Dialog:
  #   Dialog(block_tree.blockvis_widget) - Widget woran sich die
  #     RadioButtons anschlieen
  #   Dialog(block_tree.selected) - der Name des Blockes
  #   Dialog(block_tree.blockvis) - wird anhand des gewhlten Blockes
  #     neu mit Inhalt gefllt
  global PT Dialog Id2Name

  set blocks [$PT(canvas) find withtag Block]
  foreach b $blocks {
    set n $Id2Name($b)
    catch {
      block$n configure -background white
      $PT(canvas) itemconfigure $b -image block$n
    }
  }
  #$PT(canvas) itemconfigure BlockMark -fill white
  $PT(canvas) dtag BlockMark BlockMark
  if {$Dialog(block_tree.selected) != {}} {
    set Dialog(block_tree.blockvis) \
      [$PT(net) Block $Dialog(block_tree.selected) get BlockVis]
    foreach rb {exposed exposed_contents} {
      $Dialog(block_tree.blockvis_widget).$rb configure -state normal
    }  ;# foreach RadioButton
    $PT(canvas) addtag BlockMark withtag $Dialog(block_tree.selected)
    set n $Dialog(block_tree.selected)
    catch {
      block$n configure -background orange
      $PT(canvas) itemconfigure $n -image block$n
    }
    #$PT(canvas) itemconfigure $Dialog(block_tree.selected) -fill orange
  } else {
    set Dialog(block_tree.blockvis) 0
    foreach rb {exposed exposed_contents} {
      $Dialog(block_tree.blockvis_widget).$rb configure -state disabled
    }  ;# foreach RadioButton
  }  ;# else, keine Ausgabe mglich
}  ;# BlockTree_BlockVis_Display

proc BlockTree_BlockVis_Change {{all_flag one}} {
  # Bei nderung der Blocksichtbarkeit mu eine Neuberechnung der
  # Sichtbarkeiten der anderen Blcke erfolgen und dementsprechend
  # die Anzeige der Netzobjekte.
  # all_flag - falls gleich "expose_all", dann alle exposed_contents, 
  #            falls gleich "one", dann nur den selektierten ndern
  # Aus der globale Variable Dialog:
  #   Dialog(block_tree.selected) - der Name des Blockes
  #   Dialog(block_tree.blockvis) - neuer Zustand (gewnschter)
  global PT Dialog

  if {($Dialog(block_tree.selected) != {}) || ($all_flag == "expose_all")} {
    if {![$PT(net) Exists Block $Dialog(block_tree.selected)]} {
      if {![$PT(net) Exists Box $Dialog(block_tree.selected)]} {
        foreach b [$PT(net) get Boxes] {
          $PT(net) Box $b set Visibility 1
        }
        foreach b [$PT(net) get Blocks] {
          $PT(net) Block $b set BlockVis exposed_contents
        }  ;# foreach Block im Netz
        $PT(net) RecalcBlockVis
        Repaint
        return
      }
      set old_blockvis [$PT(net) Box $Dialog(block_tree.selected) get Visibility]
      if {($old_blockvis != $Dialog(block_tree.blockvis))
        || ($all_flag == "expose_all")} {
         # 1. neuen Status in den Kernel bringen
        if {$all_flag == "one"} {
          if {$Dialog(block_tree.blockvis) == "exposed"} {
            set Dialog(block_tree.blockvis) 0
          }
          if {$Dialog(block_tree.blockvis) == "exposed_contents"} {
            set Dialog(block_tree.blockvis) 4
          }
          if {$Dialog(block_tree.blockvis) != "hidden"} {
            $PT(net) Box $Dialog(block_tree.selected) set \
              Visibility $Dialog(block_tree.blockvis)
          }
        } else {
          foreach b [$PT(net) get Boxes] {
            $PT(net) Box $b set Visibility 4
          }
          foreach b [$PT(net) get Blocks] {
            $PT(net) Block $b set BlockVis exposed_contents
          }  ;# foreach Block im Netz
        }  ;# else, alle
        $PT(net) RecalcBlockVis
        #BlockTree_BlockVis_Display  ;# sicherheitshalber und berhaupt
          # 3. Bild neu aufbauen
        Repaint
        BlockTreeShowBlockViss
      }  ;# if Aktionen sind ntig
      return
    }  ;# if Block oder Box gibt's ja garnicht
    set old_blockvis [$PT(net) Block $Dialog(block_tree.selected) get BlockVis]
    if {($old_blockvis != $Dialog(block_tree.blockvis))
      || ($all_flag == "expose_all")} {
        # 1. neuen Status in den Kernel bringen
      if {$all_flag == "one"} {
        $PT(net) Block $Dialog(block_tree.selected) set \
          BlockVis $Dialog(block_tree.blockvis)
      } else {
        foreach b [$PT(net) get Boxes] {
          $PT(net) Box $b set Visibility 1
        }
        foreach b [$PT(net) get Blocks] {
          $PT(net) Block $b set BlockVis exposed_contents
        }  ;# foreach Block im Netz
      }  ;# else, alle
      $PT(net) RecalcBlockVis
      BlockTree_BlockVis_Display  ;# sicherheitshalber und berhaupt
        # 3. Bild neu aufbauen
      Repaint
      BlockTreeShowBlockViss
    }  ;# if Aktionen sind ntig
  }  ;# if berhaupt ein Block gewhlt oder alle ffnen
}  ;# BlockTree_BlockVis_Change

proc BlockTreeSelectBlock {taglist} {
  # Button-1 ist ber dem Blocksymbol gedrckt worden.
  global PT Dialog Id2Name

    # den Block im Netz highlighten
  set blocks [$PT(canvas) find withtag Block]
  foreach b $blocks {
    set n $Id2Name($b)
    catch {
      block$n configure -background white
      $PT(canvas) itemconfigure $b -image block$n
    }
  }
  set boxes [$PT(canvas) find withtag Box]
  foreach b $boxes {
    set n $Id2Name($b)
    catch {
      box$n configure -background grey
      $PT(canvas) itemconfigure $b -image box$n
    }
  }
  #$PT(canvas) itemconfigure BlockMark -fill white
  $PT(canvas) dtag BlockMark BlockMark
    # einen evtl. vorhandene vorherige Selektion aufheben
  set TAGS [$PT(block_tree.canvas) gettags Selected]
  if {[lsearch $TAGS BoxNode] > -1} {
    $PT(block_tree.canvas) itemconfigure Selected -fill grey
  } else {
    $PT(block_tree.canvas) itemconfigure Selected -fill white
  }
 # $PT(block_tree.canvas) itemconfigure Selected -fill white
  $PT(block_tree.canvas) dtag Selected
    # den Block identifizieren
  set index [lsearch -regexp $taglist ".*@Name|.*@Box"]
  set name [lindex [split [lindex $taglist $index] "@"] 0]
  if {([string length $name] != 0) && ([$PT(net) Exists Block $name] != 0)} {
      # Daten holen und ausgeben
    set Dialog(block_tree.selected) $name
    set Dialog(block_tree.name) $name
    set Dialog(block_tree.mean) [$PT(net) Block $name get Mean]
    set Dialog(block_tree.blockvis) [$PT(net) Block $name get BlockVis]
    foreach rw {exposed exposed_contents} {
      "$Dialog(block_tree.blockvis_widget).$rw" configure -state normal
    }  ;# foreach rel. Widget-Pfad
      # den Block im Netz highlighten
    $PT(canvas) addtag BlockMark withtag $name
    catch {
      block$name configure -background orange
      $PT(canvas) itemconfigure $n -image block$n
    }
    #$PT(canvas) itemconfigure $name -fill orange
      # den Block als selektiert markieren
    set boxtag "$name@Box"
    $PT(block_tree.canvas) itemconfigure $boxtag -fill orange
    $PT(block_tree.canvas) addtag Selected withtag $boxtag
  } else {
    # Keine Meldung ist besser, da Netz auch ohne Blcke
    # im Editor landen knnen, dann strt eine Meldung. 
    # puts "error: BlockTreeSelectBlock: unknown block ($name)"
  }  ;# else, Fehlerfall
}  ;# BlockTreeSelectBlock

proc BoxTreeSelectBlock {taglist} {
  # Button-1 ist ber dem Blocksymbol gedrckt worden.
  global PT Dialog Id2Name

    # den Block im Netz highlighten
  set blocks [$PT(canvas) find withtag Block]
  foreach b $blocks {
    set n $Id2Name($b)
    catch {
      block$n configure -background white
      $PT(canvas) itemconfigure $b -image block$n
    }
  }
  set boxes [$PT(canvas) find withtag Box]
  foreach b $boxes {
    set n $Id2Name($b)
    catch {
      box$n configure -background grey
      $PT(canvas) itemconfigure $b -image box$n
    }
  }
  $PT(canvas) dtag BlockMark BlockMark
    # einen evtl. vorhandene vorherige Selektion aufheben
  set TAGS [$PT(block_tree.canvas) gettags Selected]
  if {[lsearch $TAGS BoxNode] > -1} {
    $PT(block_tree.canvas) itemconfigure Selected -fill grey
  } else {
    $PT(block_tree.canvas) itemconfigure Selected -fill white
  }
  $PT(block_tree.canvas) dtag Selected
    # den Block identifizieren
  set index [lsearch -regexp $taglist ".*@Name|.*@Box"]
  set name [lindex [split [lindex $taglist $index] "@"] 0]
  if {([string length $name] != 0) && ([$PT(net) Exists Box $name] != 0)} {
      # Daten holen und ausgeben
    set Dialog(block_tree.selected) $name
    set Dialog(block_tree.name) $name
    set Dialog(block_tree.mean) [$PT(net) Box $name get Mean]
    set Dialog(block_tree.blockvis) [$PT(net) Box $name get Visibility]
    foreach rw {exposed exposed_contents} {
      "$Dialog(block_tree.blockvis_widget).$rw" configure -state normal
    }  ;# foreach rel. Widget-Pfad
      # den Block im Netz highlighten
    $PT(canvas) addtag BlockMark withtag $name
    catch {
      box$name configure -background orange
      $PT(canvas) itemconfigure $n -image box$n
    }
   # $PT(canvas) itemconfigure $name -fill orange
      # den Block als selektiert markieren
    set boxtag "$name@Box"
    $PT(block_tree.canvas) itemconfigure $boxtag -fill orange
    $PT(block_tree.canvas) addtag Selected withtag $boxtag
  } else {
    # Keine Meldung ist besser, da Netz auch ohne Blcke
    # im Editor landen knnen, dann strt eine Meldung. 
    # puts "error: BlockTreeSelectBlock: unknown block ($name)"
  }  ;# else, Fehlerfall
}  ;# BoxTreeSelectBlock

proc BlockTreeSwitchBlock {taglist} {
  # Mouse-Click im Blocksymbol.
  global PT Dialog

  BlockTreeSelectBlock $taglist
  set index [lsearch -regexp $taglist ".*@Name|.*@Box"]
  if {$index > -1} {
    set name [lindex [split [lindex $taglist $index] "@"] 0]
    set blockvis $Dialog(block_tree.blockvis)
    switch $blockvis {
      "exposed" {
        set new_blockvis "exposed_contents"
      }
      "hidden"  {
        set new_blockvis "hidden"
      }
      "exposed_contents" {
        set new_blockvis "exposed"
      }
      default {
        set new_blockvis {}
        puts "error: BlockTreeSwitchBlock: unkown blockvis"
      }
    }  ;# switch ber die Sichtbarkeit
    if {($new_blockvis == "exposed") || ($new_blockvis == "exposed_contents")} {
      $PT(net) Block $name set BlockVis $new_blockvis
      Repaint
      set Dialog(block_tree.blockvis) $new_blockvis
    }  ;# if sinnvolle nderung
    set Dialog(block_tree.blockvis) [$PT(net) Block $name get BlockVis]
    BlockTreeShowBlockViss
  } else {
    puts "error: BlockTreeSwitchBlock: unknown block"
  }  ;# else, Fehlerfall
}  ;# BlockTreeSwitchBlock

proc BoxTreeSwitchBlock {taglist} {
  # Mouse-Click im Blocksymbol.
  global PT Dialog

  BoxTreeSelectBlock $taglist
  set index [lsearch -regexp $taglist ".*@Name|.*@Box"]
  if {$index > -1} {
    set name [lindex [split [lindex $taglist $index] "@"] 0]
    set blockvis $Dialog(block_tree.blockvis)
    switch $blockvis {
      "exposed" {
        set new_blockvis "exposed_contents"
      }
      "hidden"  {
        set new_blockvis "hidden"
      }
      "exposed_contents" {
        set new_blockvis "exposed"
      }
      0 {
        set new_blockvis "exposed"
      }
      4 {
        set new_blockvis "exposed_contents"
      }
      default {
        set new_blockvis {}
        puts "error: BlockTreeSwitchBlock: unkown blockvis"
      }
    }  ;# switch ber die Sichtbarkeit

    if {$new_blockvis == "exposed"} {
      $PT(net) Box $name set Visibility 4
      Repaint
      set Dialog(block_tree.blockvis) $new_blockvis
    }
    if {$new_blockvis == "exposed_contents"} {
      $PT(net) Box $name set Visibility 0
      Repaint
      set Dialog(block_tree.blockvis) $new_blockvis
    }  ;# if sinnvolle nderung
    #set Dialog(block_tree.blockvis) [$PT(net) Block $name get BlockVis]
    BlockTreeShowBlockViss
  } else {
    puts "error: BlockTreeSwitchBlock: unknown block"
  }  ;# else, Fehlerfall
}  ;# BoxTreeSwitchBlock

proc BlockTreeShowBlockViss {} {
  # Stellt die Blcke im BlockTreeDialog ihrer BlockVis nach dar.
  global PT

  set blocks [$PT(net) get Blocks]
  foreach b $blocks {
    switch [$PT(net) Block $b get BlockVis] {
      exposed {
        set co $PT(block_tree.color.exposed)
      }
      hidden {
        set co $PT(block_tree.color.hidden)
      }
      exposed_contents {
        set co $PT(block_tree.color.exposed_contents)
      }
      default {
        puts "BlockTreeShowBlockVis: unknown BlockVis for block $b"
        return
      }
    }  ;# switch ber die Blocksichtbarkeit
      # Gemeiner kleiner Trick fr die Texte, die auch per Tag angesprochen
      # werdem, aber kein outline kennen.
    catch {$PT(block_tree.canvas) itemconfigure $b -outline $co}
  }  ;# foreach Block im Netz
}  ;# BlockTreeShowBlockViss

proc BlockTreeDrawBlock {b x y} {
  # Zeichne einen Block fr die Baumdarstellung.
  global PT

  $PT(block_tree.canvas) create rectangle \
    [expr {$x - $PT(block_tree.block_halfsize)}] \
    [expr {$y - $PT(block_tree.block_halfsize)}] \
    [expr {$x + $PT(block_tree.block_halfsize)}] \
    [expr {$y + $PT(block_tree.block_halfsize)}] \
    -width 2 -fill white -outline gray \
    -tags "$b BlockNode $b@Box"
  $PT(block_tree.canvas) create text $x $y \
    -text $b -anchor c -tags "$b BlockNode $b@Name"
}  ;# BlockTreeDrawBlock

proc BlockTreeDrawBox {b x y} {
  # Zeichne eine Box fr die Baumdarstellung.
  global PT

  $PT(block_tree.canvas) create rectangle \
    [expr {$x - $PT(block_tree.block_halfsize)}] \
    [expr {$y - $PT(block_tree.block_halfsize)}] \
    [expr {$x + $PT(block_tree.block_halfsize)}] \
    [expr {$y + $PT(block_tree.block_halfsize)}] \
    -width 2 -fill grey -outline black \
    -tags "$b BoxNode $b@Box"
  $PT(block_tree.canvas) create text $x $y \
    -text $b -anchor c -tags "$b BoxNode $b@Name"
}  ;# BlockTreeDrawBox

proc BlockTreeVisit {b xo yo x y new_x} {
  # Einzeichnen der Blcke, Rekursionsprozedur ber die Blcke.
  global PT
  upvar $new_x nx

  set lx $x
  set next_x $lx
  BlockTreeDrawBlock $b $x $y
  set children [lsort [$PT(net) Block $b get Blocks]]
  foreach c $children {
    set li [$PT(block_tree.canvas) create line $x $y $lx \
      [expr {$y + $PT(block_tree.dy)}]]
    $PT(block_tree.canvas) lower $li
    BlockTreeVisit $c $x $y $lx [expr $y + $PT(block_tree.dy)] next_x
    set lx [expr $next_x + $PT(block_tree.dx)]
  }  ;# foreach seiner Kinderchen
  if {[llength $children] == 0} {
    set nx $lx
  } else {
    set nx [expr $lx - $PT(block_tree.dx)]
  }  ;# else, Kinder vorhanden
}  ;# BlockTreeVisit

proc BoxTreeVisit {b xo yo x y new_x} {
  # Einzeichnen der Boxen, Rekursionsprozedur ber die Blcke.
  global PT
  upvar $new_x nx

  set lx $x
  set next_x $lx
  BlockTreeDrawBox $b $x $y
  set children [lsort [$PT(net) Box $b get Blocks]]
  foreach c $children {
    set li [$PT(block_tree.canvas) create line $x $y $lx \
      [expr $y + $PT(block_tree.dy)]]
    $PT(block_tree.canvas) lower $li
    BlockTreeVisit $c $x $y $lx [expr $y + $PT(block_tree.dy)] next_x
    set lx [expr $next_x + $PT(block_tree.dx)]
  }  ;# foreach seiner Kinderchen
  if {[llength $children] == 0} {
    set nx $lx
  } else {
    set nx [expr $lx - $PT(block_tree.dx)]
  }  ;# else, Kinder vorhanden
}  ;# BoxTreeVisit

proc BlockTreeInit {c} {
  # Die Grafikdarstellung initialisieren.
  global PT Dialog

    # die blichen Werte eintragen
  set PT(block_tree.dx) 40
  set PT(block_tree.dy) 40
  set PT(block_tree.block_halfsize) 15
  set PT(block_tree.color.exposed) green
  set PT(block_tree.color.hidden) lightgray
  set PT(block_tree.color.exposed_contents) blue
    # Schleife ber die Wurzelblcke vorbereiten
  set lx $PT(block_tree.dx)
  set next_x 0
  set boxes [$PT(net) get Boxes]
  set blocks [$PT(net) get Blocks]
  
  set rootblocks [$PT(net) get RootBlocks]
  foreach box $boxes {
    BoxTreeVisit $box 0 0 $lx $PT(block_tree.dy) next_x
    set childblocks [$PT(net) Box $box get Blocks]
    foreach ch $childblocks {
      set index [lsearch $rootblocks $ch]
      if {$index > -1} {
        set rootblocks [lreplace $rootblocks $index $index]
      }
    }
    set lx [expr $next_x + $PT(block_tree.dx)]
  }  ;# foreach box in den Boxen
  foreach rb [lsort $rootblocks] {
    BlockTreeVisit $rb 0 0 $lx $PT(block_tree.dy) next_x
    set lx [expr $next_x + $PT(block_tree.dx)]
  }  ;# foreach rb ber die RootBlocks
  #$PT(block_tree.canvas)
  $c bind BlockNode <Button-1> {
    BlockTreeSelectBlock [%W gettags [%W find withtag current]]
  }
  #$PT(block_tree.canvas)
  $c bind BlockNode <Button-2> {
    BlockTreeSwitchBlock [%W gettags [%W find withtag current]]
  }
  $c bind BoxNode <Button-1> {
    BoxTreeSelectBlock [%W gettags [%W find withtag current]]
  }
  $c bind BoxNode <Button-2> {
    BoxTreeSwitchBlock [%W gettags [%W find withtag current]]
  }
  BlockTreeShowBlockViss
  set Dialog(block_tree.selected) [lindex $rootblocks 0]
  set Dialog(box_tree.selected) [lindex $boxes 0]
  if {[string length $Dialog(block_tree.selected)] == 0 && \
      [string length $Dialog(box_tree.selected)] == 0} {
    set $Dialog(block_tree.selected) "no block"
    #$PT(block_tree.canvas)
    $c create text 20 20 \
      -text "There are no blocks in the net!" -anchor w
  }  ;# if kein Block vorhanden
    # einen Aufruf vortuschen, um eine sinnvolle Startanzeige zu erlangen
  BlockTreeSelectBlock "$Dialog(block_tree.selected)@Name"
}  ;# BlockTreeInit

  #*** Dateiende BlockView.tcl ***

