# ----------------------------------------------------------------
# NetBase.tcl
# ----------------------------------------------------------------
# Diese Datei enth"alt das Basisgelumpe, das fuer HL und LL Editor
# notwendig ist.
#
# Tobias Himstedt 3/95
# Burkhard Bieber 1995/96
# Ulf Fildebrandt 1997/98
# last-update: 20.5.98
# -------------------------------------------------------------
# Generelles:
# In diesem Modul sind Prozeduren, die sowohl vom Lowlevel, als
# auch vom Highlevel-Editor benutzt werden.
#
# Der Kern beider Editoren ist in C++ implementiert. Dieser Kern
# stellt verschiedene Funktionen zur Verfuegung. Es finden sich hier
# Methoden zum erstellen von Transitionen, Stellen und Kanten
# wieder. Weiterhin k"onnen diese Elemente modifiziert werden
# (Position, Aktionsterme, Werterme ...). Aufgrund der m"oglichen
# Vielzahl der Objekte ist der Ansatz dabei funktionsorientiert
# (vgl. Tcl und Tk, Kap. 28.3 Ousterhout). Wenn ein neues Netz
# erstellt werden soll, wird die im Kern enthaltene Funktion MakeNet
# aufgerufen, die je nach Editor-Typ ein HL-Netz oder LL-Netz
# erstellt. Diese liefert einen Namen zurueck, unter dem das Netz dann
# angesprochen werden kann. 
# Bsp.: MakeNet -> liefert Net0
# Net0 create Place -> liefert s1
# Net0 Place s1 set x 10 y 20 -> liefert nix und teilt dem Kern die
# Position der Stelle s1 mit.
#
# Die Trennung ist also offensichtlich. Der C++-Kern uebernimmt die
# Funktionalit"at, Tcl/Tk ist hingegen fuer die Darstellung der Objekte,
# sowie fuer das gesamte restliche Benutzerschnittstelle zust"andig. 
# Fuer weitere Informationen zum C++-Teil betrachte man die Sourcen und
# lese meine Dimplomarbeit.
#
# Im Tcl-Teil existieren mehrere globale Variablen. 
# In dem Array PT alle einfachen Variablen gespeichert, z.B. ob
# bestimmte Annotationen sichtbar oder nicht sein sollen.
# Das Array IsAnnoFrom bildet die Canvas-ID's der Annotationen auf den
# Name des Knotens oder die Kante ab, zu dem die Annotation geh"ohrt
# (wichtig fuer das Verschieben). 
# In dem Array Dialog sind etwaige Variablen gesichert, die fuer die
# Dialog wichtig sind.

# -------------------------------------------------------------
# Name
#   UpdateAnno
# ---------
# Parameter 
#   node_type -> Transition oder Place
#   annotype -> VT, AT, Type, Name, Mean, Time
# ---------
# Description
#   Je nach Flag PT(annotype) wird die Annotation neu kreiert
#   oder ausgeschaltet.
proc UpdateAnno {node_type annotype} {
  # Sp"ater: mu"s fuer Var_ab evtl. ge"andert werden - oder in CreateAnno?!?
  global PT IsAnnoFrom

  switch $annotype {
    VT   {set flag $PT(VTSwitch)}
    AT   {set flag $PT(ATSwitch)}
    Type {set flag $PT(TypeSwitch)}
    Name {set flag $PT(NameSwitch)}
    Mean {set flag $PT(MeanSwitch)}
    Time {set flag $PT(Time)}
  }  ;# switch type of annotation
  if {$flag} {
    foreach p [$PT(net) get [format "%ss" $node_type]] {
      if {[$PT(net) $node_type $p get BlockVis] == "exposed"} {
        switch $annotype {
          VT    { set transition [$PT(net) get Transitions]
                  foreach trans $transition {
                    if {[llength [$PT(canvas) find withtag VT@$trans]] > 0} {
                      $PT(canvas) itemconfigure [$PT(canvas) find withtag VT@$trans] -font $PT(valueTerms)
                    } else {
                      CreateAnno $trans Transition VT
                    }
                  }
                  return
                }
          AT    { set transition [$PT(net) get Transitions]
                  foreach trans $transition {
                    if {[llength [$PT(canvas) find withtag AT@$trans]] > 0} {
                      $PT(canvas) itemconfigure [$PT(canvas) find withtag AT@$trans] -font $PT(actionTerms)
                    } else {
                      CreateAnno $trans Transition AT
                    }
                    if {[llength [$PT(canvas) find withtag LT@$trans]] > 0} {
                      $PT(canvas) itemconfigure [$PT(canvas) find withtag LT@$trans] -font $PT(actionTerms)
                    } else {
                      CreateAnno $trans Transition LT
                    }
                  }
                  return
                }
          Type  { set places [$PT(net) get Places]
                  foreach place $places {
                    if {[llength [$PT(canvas) find withtag Type@$place]] > 0} {
                      $PT(canvas) itemconfigure [$PT(canvas) find withtag Type@$place] -font $PT(types)
                    } else {
                      CreateAnno $place Place Type
                    }
                  }
                  return
                }
          Time  { set transition [$PT(net) get Transitions]
                  foreach trans $transition {
                    if {[llength [$PT(canvas) find withtag Time@$trans]] > 0} {
		      $PT(canvas) itemconfigure [$PT(canvas) find withtag Time@$trans] -font $PT(nameTrans) -text [ format {[%s;%s]} [$PT(net) Transition $trans get Earliest] [$PT(net) Transition $trans get Latest] ]
                    } else {
                      if {[$PT(net) Transition $trans get BlockVis] == "exposed"} {
                        CreateAnno $trans Transition Time
                      }
                    }
                  }
                  return
                }
        }
        CreateAnno $p $node_type $annotype
      }  ;# if visible
    }  ;# foreach vertex
  } else {
    foreach p [$PT(net) get [format "%ss" $node_type]] {
      if {[$PT(net) $node_type $p get BlockVis] == "exposed"} {
          set c [$PT(canvas) find withtag $annotype@$p]
          $PT(canvas) delete $c
          unset IsAnnoFrom($c)
      }  ;# if visible
    }  ;# foreach vertex
  }  ;# else turn off
}  ;# UpdateAnno

# -------------------------------------------------------------
# Name
#   UpdateVar_ab
# ---------
# Parameter 
#   nothing
# ---------
# Description
#   Je nach Flag PT(Var_ab_Switch) wird die Annotation neu kreiert
#   oder ausgeschaltet.
proc UpdateVar_ab {} {
  # Spaeter s.o. und BlockVis
  # vorerst garnichts machen
  return

  global PT

  puts "------------"
  set places [$PT(net) get Places]
  foreach p $places {
    foreach ie [$PT(net) Place $p get InEdges] {
      if {[$PT(net) Edge $ie get Visibility] != 1} {
        if {$PT(Var_ab_Switch)} {
          MkEdge [ $PT(net) Edge $ie get Src ] $p $ie
        } else {
          $PT(canvas) delete Vars@$ie
        }
      }
    }  ;# foreach ie
    foreach oe [$PT(net) Place $p get OutEdges] {
      if {[$PT(net) Edge $oe get Visibility] != 1} {
        if {$PT(Var_ab_Switch)} {
          MkEdge $p [ $PT(net) Edge $oe get Dest ] $oe
        } else {
          $PT(canvas) delete Vars@$oe
        }
      }
    }  ;# foreach oe
  }  ;# foreach p
}  ;# UpdateVar_ab

# -------------------------------------------------------------
# Name
#   UpdateInvisibleArcs
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Je nach Flag werden die unsichbaren Kanten ein oder ausgeschaltet
#   Eine Kante gilt als sichtbar, wenn ihr visibility flags != 1 ist.
proc UpdateInvisibleArcs {} {
  # Todo: BlockVis und Blockkanten
  # Phantomkanten sind schon da
  global PT

  set places [$PT(net) get Places]
  foreach p $places {
      # usual edges
    foreach ie [$PT(net) Place $p get InEdges] {
      if {[$PT(net) Edge $ie get Visibility] != 1} {
        if {$PT(InvisibleArcsOn)} {
          if {[$PT(net) Edge $ie get BlockVis] == "exposed"} {
            MkEdge [$PT(net) Edge $ie get Src] $p $ie
          } else {
            MkBlockEdge [$PT(net) Edge $ie get BlockSrc] $p $ie
          }  ;# else block edge
        } else {
          $PT(canvas) delete $ie
          $PT(canvas) delete Weight@$ie
          $PT(canvas) delete Vars@$ie
        }  ;# else turn off edge
      }  ;# if edge might need change
    }  ;# foreach incoming edge
    foreach oe  [$PT(net) Place $p get OutEdges] {
      if {[$PT(net) Edge $oe get Visibility] != 1} {
        if {$PT(InvisibleArcsOn)} {
          if {[$PT(net) Edge $oe get BlockVis] == "exposed"} {
            MkEdge $p [$PT(net) Edge $oe get Dest] $oe
          } else {
            MkBlockEdge [$PT(net) Edge $oe get BlockDest] $p $oe
          }  ;# else block edge
        } else {
          $PT(canvas) delete $oe
          $PT(canvas) delete Weight@$oe
          $PT(canvas) delete Vars@$oe
        }  ;# else turn off edge
      }  ;# if edge might need change
    }  ;# foreach outgoing edge
      # Phantom edge
    foreach ie [$PT(net) Place $p get InPEdges] {
      if {[$PT(net) PEdge $ie get Visibility] != 1} {
        if {$PT(InvisibleArcsOn)} {
          if {[$PT(net) PEdge $ie get BlockVis] == "exposed"} {
            MkPEdge [$PT(net) PEdge $ie get Src] $p $ie
          } else {
            MkBlockPEdge [$PT(net) PEdge $ie get BlockSrc] $p $ie
          }  ;# else block edge
        } else {
          $PT(canvas) delete $ie
          $PT(canvas) delete Weight@$ie
          $PT(canvas) delete Vars@$ie
        }  ;# else turn off edge
      }  ;# if phantom edge might need change
    }  ;# foreach incoming phantom edge
    foreach oe  [$PT(net) Place $p get OutPEdges] {
      if {[$PT(net) PEdge $oe get Visibility] != 1} {
        if {$PT(InvisibleArcsOn)} {
          if {[$PT(net) PEdge $oe get BlockVis] == "exposed"} {
            MkPEdge $p [$PT(net) PEdge $oe get Dest] $oe
          } else {
            MkBlockPEdge [$PT(net) PEdge $oe get BlockDest] $p $oe
          }  ;# else block edge
        } else {
          $PT(canvas) delete $oe
          $PT(canvas) delete Weight@$oe
          $PT(canvas) delete Vars@$oe
        }  ;# else turn off phantom edge
      }  ;# if phantom edge might need change
    }  ;# foreach outgoing phanom edge
  }  ;# foreach place
}  ;# UpdateInvisibleArcs
	    
# -------------------------------------------------------------
# Name
#   UpdatePlaceState
# ---------
# Parameter 
#   place -> Stellenname
# ---------
# Description
#   Je nach Status wird die Stelle rot (entry), gruen (exit) oder
#   weis (intern) eingefaerbt.
proc UpdatePlaceState {place} {
  global PT

  set state [$PT(net) Place $place get State]
  switch $state {
    entry {
      $PT(canvas) itemconfigure $place -fill green} 
    exit {
      $PT(canvas) itemconfigure $place -fill red} 
    internal {
      $PT(canvas) itemconfigure $place -fill white} 
  }  ;# switch on state
}  ;# UpdatePlaceState

# -------------------------------------------------------------
# Name
#   UpdateTransitionState
# ---------
# Parameter 
#   transition -> Transitionsname
# ---------
# Description
#   Synchronisationstransitionen die durch Channels mit der Kapazitaet 0
#   enstanden sind werden hier passend markiert.
#
proc UpdateTransitionState {transition} {
  global PT

  set channel0sync [$PT(net) Transition $transition get Channel0Sync]
  switch $channel0sync {
    yes {
      $PT(canvas) itemconfigure $transition -fill $PT(color.sync_transition)} 
    no {
      $PT(canvas) itemconfigure $transition -fill $PT(color.transition)} 
  }  ;# switch on state
}  ;# UpdateTransitionState

# -------------------------------------------------------------
# Name
#   UpdatePTransitionState
# ---------
# Parameter 
#   transition -> Phantomtransitionsname
# ---------
# Description
#   Phantomtransitionen werden hier passend markiert.
#
proc UpdatePTransitionState {transition} {
  global PT

  $PT(canvas) itemconfigure $transition -fill $PT(color.phantom_transition)
}  ;# UpdatePTransitionState

proc SwitchSyncTransDisplay {} {
  # Tell C++-core what has changed and redraw screen
  global PT

  $PT(net) set SyncTransSwitch $PT(SyncTransSwitch)
  Repaint
}  ;# SwitchSyncTransDisplay

proc SwitchPhantomTransDisplay {} {
  # Tell C++-core what has changed and redraw screen
  global PT

  $PT(net) set PhantomTransSwitch $PT(PhantomTransSwitch)
  Repaint
}  ;# SwitchPhantomTransDisplay

proc AlignNodes {} {
#TODO: Comment, Grid does not work here!
  global PT

  set places [$PT(net) get Places]
  set transitions [$PT(net) get Transitions]
    # turn on gridding for this temporarily
  set old_status $PT(Gridding)
  set PT(Gridding) 1
  foreach p $places {
    set coords [$PT(canvas) coords $p]
    set x [expr [lindex $coords 0] + $PT(OSizeHalf)]
    set y [expr [lindex $coords 1] + $PT(OSizeHalf)]
    Grid $x $y
    $PT(canvas) coords $p \
      [expr $x - $PT(OSizeHalf)] [expr $y - $PT(OSizeHalf)] \
      [expr $x + $PT(OSizeHalf)] [expr $y + $PT(OSizeHalf)]
    LayoutTokens $p
    LayoutAnno $p Place Name
  }  ;# foreach place
  foreach t $transitions {
    set coords [$PT(canvas) coords $t]
    set x [expr [lindex $coords 0] + $PT(OSizeHalf)]
    set y [expr [lindex $coords 1] + $PT(OSizeHalf)]
    Grid $x $y
    $PT(canvas) coords $t \
      [expr $x - $PT(OSizeHalf)] [expr $y - $PT(OSizeHalf)] \
      [expr $x + $PT(OSizeHalf)] [expr $y + $PT(OSizeHalf)]
    set pre [$PT(net) Transition $t get InEdges]
    set post [$PT(net) Transition $t get OutEdges]
    foreach e $pre {
      SetupEdge [$PT(net) Edge $e get Src] $t $e
      SetupEdgeAnno $e
    }  ;# foreach incoming edge
    foreach e $post {
      SetupEdge $t [$PT(net) Edge $e  get Dest] $e
      SetupEdgeAnno $e
    }  ;# foreach outgoing edge
    LayoutAnno $t Transition Name
  }  ;# foreach transition
  set PT(Gridding) $old_status
}  ;# LayoutNodes

# -------------------------------------------------------------
# Name
#   DeleteNet
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Netz wird geloescht, Zeichenflaeche wird geloescht (nach Nachfrage)
proc DeleteNet {} {
  global PT Id2Name IsAnnoFrom Frames

  if {$PT(NetChanged)} {
    if {[tk_dialog .d {Delete Net} {Discard changes?} question \
             0 {Yes, beam it away} {Cancel}]} {
      return
    }  ;# return if canceled or dialog closed
  }
  set places [$PT(net) get Places]
  foreach p $places {
    catch {destroy $Frames($p); unset $Frames($p)}
  }
  $PT(net) DeleteAll
  $PT(canvas) delete all
  $PT(net) set ObjectSize [expr int($PT(ObjectSize) / $PT(Zoom))]
  set PT(ObjectSize) [$PT(net) get ObjectSize]
  set PT(OSizeHalf) [expr $PT(ObjectSize) / 2]
  set PT(Zoom) 1
  set PT(OldZoom) 1
  unset Id2Name
  unset IsAnnoFrom
}  ;# DeleteNet

# -------------------------------------------------------------
# Name
#   Quit
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Netz wird geloescht, Zeichenflaeche wird geloescht (nach Nachfrage)
#   und dann Ende im Gelaende.
proc Quit {} {
  global PT Frames

  AnimateEnd
  set places [$PT(net) get Places]
  if {$PT(NetChanged)} {
    switch [tk_dialog .d {Quit?} \
        {The current Net is not saved! Save it now?} \
        question 0 {No, discard it} {Cancel} {Yes, save} {Yes, save as ...}] {
      0 {
        }
      1 {
          return
        }
      2 {
          SaveNet $PT(Filename)
        }
      3 {
          FSBox SaveNet "Save Net" $PT(Filter) . save
        }
    }  ;# switch on dialog result
  }  ;# if net was changed
  $PT(net) DeleteAll
  destroy .td
  destroy .pd
  destroy .icon
  foreach p $places {
    catch {destroy $Frames($p); unset $Frames($p)}
  }
  end_session
  exit
}  ;# Quit

# -------------------------------------------------------------
# Name
#   UpdateNames
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Die Namen der Knoten werden je nach Flag ein- oder ausgeschaltet.
proc UpdateNames {} {
  global PT

  if {$PT(NameSwitch)} {
    foreach block [$PT(net) get Blocks] {
      if {[$PT(net) Block $block get BlockVis] == "exposed"} {
        if {[llength [$PT(canvas) find withtag Name@$block]] == 0} {
          CreateName $block Block
        }
      }  ;# if visible
    }  ;# foreach block
    foreach box [$PT(net) get Boxes] {
      if {[$PT(net) Box $box get Visibility] == 0} {
        if {[llength [$PT(canvas) find withtag Name@$box]] == 0} {
          CreateName $box Box
        }
      }  ;# if visible
    }  ;# foreach box
    foreach place [$PT(net) get Places] {
      if {[$PT(net) Place $place get BlockVis] == "exposed"} {
        if {[llength [$PT(canvas) find withtag Name@$place]] > 0} {
          $PT(canvas) itemconfigure [$PT(canvas) find withtag Name@$place] -font $PT(namePlace)
        } else {
          CreateName $place Place
        }
      }  ;# if visible
    }  ;# foreach place
    foreach transition [$PT(net) get Transitions] {
      if {[$PT(net) Transition $transition get BlockVis] == "exposed"} {
        if {[llength [$PT(canvas) find withtag Name@$transition]] > 0} {
          $PT(canvas) itemconfigure [$PT(canvas) find withtag Name@$transition] -font $PT(nameTrans)
        } else {
          CreateName $transition Transition
        }
      }  ;# if visible
    }  ;# foreach transition
    foreach transition [$PT(net) get PTransitions] {
      if {[$PT(net) PTransition $transition get BlockVis] == "exposed"} {
        if {[llength [$PT(canvas) find withtag Name@$transition]] > 0} {
          $PT(canvas) itemconfigure [$PT(canvas) find withtag Name@$transition] -font $PT(nameTrans)
        } else {
          CreateName $transition PTransition
        }
      }  ;# if visible
    }  ;# foreach phantom transition
  } else {
    foreach block [$PT(net) get Blocks] {
      $PT(canvas) delete Name@$block
    }  ;# foreach block
    foreach box [$PT(net) get Boxes] {
      $PT(canvas) delete Name@$box
    }  ;# foreach box
    foreach place [$PT(net) get Places] {
      $PT(canvas) delete Name@$place
    }  ;# foreach place
    foreach transition [$PT(net) get Transitions] {
      $PT(canvas) delete Name@$transition
    }  ;# foreach transition
    foreach transition [$PT(net) get PTransitions] {
      $PT(canvas) delete Name@$transition
    }  ;# foreach phantom transition
  }  ;# else delete name
}  ;# UpdateNames

# -------------------------------------------------------------
# Name
#   CreateName
# ---------
# Parameter 
#   name -> Knoten-Name
#   type -> Knoten-Typ (Block, Place, Transition, PTransition)
# ---------
# Description
#   Name eines Knotens wird auf dem Canvas kreiert. Der Text wird
#   mit den Tags Name, Annotation, NodeAnnotation versehen
#   In globalem Array IsAnnoFrom wird wird zur Canvas-ID des Textes
#   der Name des Knotens abgelegt
proc CreateName {name type} {
  global PT IsAnnoFrom

  set bo 0
  switch $type {
    Place {set c [$PT(canvas) create text 0 0 -text $name -anchor nw -tags Name@$name -font $PT(namePlace)];set bo 1}
    Transition {set c [$PT(canvas) create text 0 0 -text $name -anchor nw -tags Name@$name -font $PT(nameTrans)];set bo 1}
  }
  if {$bo == 0} {
    set c [$PT(canvas) create text 0 0 -text $name -anchor nw \
      -tags Name@$name -font $PT(AnnoFont)]
  }
  $PT(canvas) addtag Annotation withtag $c
  $PT(canvas) addtag NodeAnnotation withtag $c
  set IsAnnoFrom($c) $name
  LayoutAnno $name $type Name
}  ;# CreateName

# -------------------------------------------------------------
# Name
#   CreateAnno
# ---------
# Parameter 
#   name -> Knoten-Name
#   type -> Knoten-Typ (Block, Place, Transition oder PTransition)
#   kind -> Annotations-Typ (AT, VT, Mean, Name ...)
# ---------
# Description
#   Eine Annotation eines Knoten wird auf dem Canvas kreiert.
#   In globalem Array IsAnnoFrom wird wird zur Canvas-ID des Textes
#   der Name des Knotens abgelegt
proc CreateAnno {name type kind} {
  global PT IsAnnoFrom

  if {$kind != "Time"} {
    set at [$PT(net) $type $name get $kind]
  }
  if {![$PT(net) $type $name get Visibility]} {
    return
  }
  set bo 0
  switch $type {
    Place { switch $kind {
        Type {set c [$PT(canvas) create text 0 0 -text $at -anchor nw \
		-tags $kind@$name -font $PT(types) -width 210];set bo 1}
        Mean {set c [$PT(canvas) create text 0 0 -text $at -anchor nw \
		-tags $kind@$name -font $PT(meanPlace) -width 210];set bo 1}
      }
    }
    Transition {
      if {[$PT(net) Transition $name get Channel0Sync] == "yes" && $PT(ATSwitch)} {
        return
      }
      switch $kind {
        Mean {set c [$PT(canvas) create text 0 0 -text $at -anchor nw \
		-tags $kind@$name -font $PT(meanTrans) -width 210];set bo 1}
        VT {set c [$PT(canvas) create text 0 0 -text $at -anchor nw \
		-tags $kind@$name -font $PT(valueTerms) -width 210];set bo 1}
        AT {set c [$PT(canvas) create text 0 0 -text $at -anchor nw \
		-tags $kind@$name -font $PT(actionTerms) -width 210];set bo 1}
        LT {set c [$PT(canvas) create text 0 0 -text $at -anchor nw \
		-tags $kind@$name -font $PT(actionTerms) -width 210];set bo 1}
        Time {set c [$PT(canvas) create text 0 0 -text [ format {[%s;%s]} [$PT(net) Transition $name get Earliest] [$PT(net) Transition $name get Latest] ] -anchor nw -tags $kind@$name -font $PT(nameTrans) -width 210]
              set bo 1}
    
      }
    }
  }  ;# switch type of annotation
  if {$bo == 0} {
    set c [$PT(canvas) create text 0 0 -text $at -anchor nw \
      -tags $kind@$name -font $PT(AnnoFont) -width 210]
  }
  $PT(canvas) addtag Annotation withtag $c
  $PT(canvas) addtag NodeAnnotation withtag $c
  set IsAnnoFrom($c) $name
  LayoutAnno $name $type $kind
}  ;# CreateAnno

# -------------------------------------------------------------
# Name
#   UpdateMeanings
# ---------
# Parameter
#   nothing
# ---------
# Description
proc UpdateMeanings {} {
  global PT

  if {$PT(MeanSwitch)} {
    foreach block [$PT(net) get Blocks] {
      if {[$PT(net) Block $block get BlockVis] == "exposed"} {
        set mean [$PT(net) Block $block get Mean]
        if {$mean != ""} {
          if {[llength [$PT(canvas) find withtag Mean@$block]] == 0} {
            CreateAnno $block Block Mean
          }            
        }  ;# if Meaning is present
      }  ;# if visible
    }  ;# foreach block
    update idletasks
    foreach box [$PT(net) get Boxes] {
      if {[$PT(net) Box $box get Visibility] == 0} {
        set mean [$PT(net) Box $box get Mean]
        if {$mean != ""} {
          if {[llength [$PT(canvas) find withtag Mean@$box]] == 0} {
            CreateAnno $box Box Mean
          }            
        }  ;# if Meaning is present
      }  ;# if visible
    }  ;# foreach Box
    update idletask
    foreach place [$PT(net) get Places] {
      if {[$PT(net) Place $place get BlockVis] == "exposed"} {
        set mean [$PT(net) Place $place get Mean]
        if {$mean != ""} {
          if {[llength [$PT(canvas) find withtag Mean@$place]] > 0} {
            $PT(canvas) itemconfigure [$PT(canvas) find withtag Mean@$place] -font $PT(meanPlace)
          } else {
            CreateAnno $place Place Mean
          }
        }  ;# if Meaning is present
      }  ;# if visible
    }  ;# foreach place
    update idletasks
    foreach transition [$PT(net) get Transitions] {
      if {[$PT(net) Transition $transition get BlockVis] == "exposed"} {
        set mean [$PT(net) Transition $transition get Mean]
        if {$mean != ""} {
          if {[llength [$PT(canvas) find withtag Mean@$transition]] > 0} {
            $PT(canvas) itemconfigure [$PT(canvas) find withtag Mean@$transition] -font $PT(meanTrans)
          } else {
            CreateAnno $transition Transition Mean
          }
        }  ;# if Meaning is present
      }  ;# if visible
    }  ;# foreach transition
    update idletasks
    foreach transition [$PT(net) get PTransitions] {
      if {[$PT(net) PTransition $transition get BlockVis] == "exposed"} {
        set mean [$PT(net) PTransition $transition get Mean]
        if {$mean != ""} {
          if {[llength [$PT(canvas) find withtag Mean@$transition]] > 0} {
            $PT(canvas) itemconfigure [$PT(canvas) find withtag Mean@$transition] -font $PT(meanTrans)
          } else {
            CreateAnno $transition PTransition Mean
          }
        }  ;# if Meaning is present
      }  ;# if visible
    }  ;# foreach phantom transition
    update idletasks
  } else {
    foreach block [$PT(net) get Blocks] {
      $PT(canvas) delete Mean@$block
    }  ;# foreach block
    update idletasks
    foreach box [$PT(net) get Boxes] {
      $PT(canvas) delete Mean@$box
    }  ;# foreach Box
    update idletask
    foreach place [$PT(net) get Places] {
      $PT(canvas) delete Mean@$place
    }  ;# foreach place
    update idletasks
    foreach transition [$PT(net) get Transitions] {
      $PT(canvas) delete Mean@$transition
    }  ;# foreach transition
    update idletasks
    foreach transition [$PT(net) get PTransitions] {
      $PT(canvas) delete Mean@$transition
    }  ;# foreach phantom transition
    update idletasks
  }  ;# else delete Meaning
}  ;# UpdateMeanings


proc UpdateEdgeAnno {} {
  global PT Id2Name

  if {$PT(EdgeVars)} {
    set edges [$PT(canvas) find withtag Edge]
    foreach edge $edges {
      if {[llength [$PT(canvas) find withtag Vars@$Id2Name($edge)]] > 0} {
        if {[lsearch -regexp $edge PA] > -1} {
          $PT(canvas) itemconfigure [$PT(canvas) find withtag Vars@$Id2Name($edge)] -font $PT(edge)
        }
      } else {
        if {[lsearch -regexp $edge PA] > -1} {
          CreateEdgeVars $Id2Name($edge)
        }
      }
    }
  } else {
    set edges [$PT(canvas) find withtag Edge]
    foreach edge $edges {
      $PT(canvas) delete [$PT(canvas) find withtag Vars@$Id2Name($edge)]
    }
  }
}

# -------------------------------------------------------------
# Name
#   LayoutAnno
# ---------
# Parameter 
#   name -> Knoten-Name
#   type -> Knoten-Typ (Block, Place, Transition oder PTransition)
#   anno_type -> Annotations-Typ (AT, VT, Mean, Name ...)
# ---------
# Description
#   Eine Annotation eines Knoten wird auf dem Canvas an die entsprechende
#   Position gelegt. Die Position wird aus dem C++-Kern gewonnen.
#TODO: Zeitlabel besser platzieren, unabhaengig von Mean!
proc LayoutAnno {name type anno_type} {
  global PT

  set coords [$PT(canvas) coords $name]
  if {$anno_type != "Time"} {
    set nx [expr [$PT(net) $type $name get [format "%sX" $anno_type]] * $PT(Zoom) + \
      [lindex $coords 0] + $PT(OSizeHalf)]
    set ny [expr [$PT(net) $type $name get [format "%sY" $anno_type]] * $PT(Zoom) + \
      [lindex $coords 1] + $PT(OSizeHalf)]
  } else {
    set nx [expr [$PT(net) $type $name get [format "%sX" Mean]] * $PT(Zoom) + \
      [lindex $coords 0] + $PT(OSizeHalf)]
    set ny [expr [$PT(net) $type $name get [format "%sY" Mean]] * $PT(Zoom) + \
      [lindex $coords 1] + 3*$PT(OSizeHalf)]
  }
  $PT(canvas) coords $anno_type@$name $nx $ny
}  ;# LayoutAnno

# -------------------------------------------------------------
# Name
#   UnsetAllBindings
# ---------
# Parameter 
#   Nothing 
# ---------
# Description
#   Alle Bindings des Canvas werden geloescht
proc UnsetAllBindings {} {
  # Todo: Bloecke ergaenzen
  global PT

  foreach type {all Node Block Place Transition PTransition Edge PEdge \
    Deletable Active PActive Annotation Area NodeAnnotation EdgeAnnotation Title} {
    $PT(canvas) bind $type <Enter> {}
    $PT(canvas) bind $type <Leave> {}
    $PT(canvas) bind $type <1> {}
    $PT(canvas) bind $type <2> {}
    $PT(canvas) bind $type <B1-Motion> {}
    $PT(canvas) bind $type <ButtonRelease-1> {}
    $PT(canvas) bind $type <Double-1> {}
    $PT(canvas) bind $type <Escape> {}
    $PT(canvas) bind $type <Delete> {}
    $PT(canvas) bind $type m {}
    $PT(canvas) bind $type n {}
  }  ;# foreach graphical element
    # delete common Canvas-Bindings
  bind $PT(canvas) p {}
  bind $PT(canvas) t {}
  bind $PT(canvas) a {}
  bind $PT(canvas) o {}
  bind $PT(canvas) c {}
  bind $PT(canvas) <2> {}
  bind $PT(canvas) <B2-Motion> {}
  bind $PT(canvas) <Motion> {}
  bind $PT(canvas) <1> {}
  bind $PT(canvas) <Delete> {}
}  ;# UnsetAllBindings

# -------------------------------------------------------------
# Name
#   EditBindings
# ---------
# Parameter 
#   Nothing 
# ---------
# Description
#   Die Event-Bindings zum Editieren werden festgelegt.
proc EditBindings {} {
  global PT Id2Name

   # bind $PT(canvas) <1> DeselectAll
  bind $PT(canvas) <Enter> {set PT(canvas) %W}
  $PT(canvas) bind Deletable <1> {Select %x %y}
  $PT(canvas) bind Edge <1> {Select %x %y}
  $PT(canvas) bind PActive <1> {Select %x %y}
  $PT(canvas) bind Token <1> {Select %x %y}
  $PT(canvas) bind Deletable <Shift-Button-1> {AddSelection %x %y}
  $PT(canvas) bind Node <B1-Motion> {Move %x %y}
  $PT(canvas) bind Node <ButtonRelease-1> {StopMove %x %y}
  $PT(canvas) bind Edge <B1-Motion> {MoveEdge %x %y}
  $PT(canvas) bind Area <1> {AreaMoveStart %x %y}
  $PT(canvas) bind Area <B1-Motion> {AreaMove %x %y}
  $PT(canvas) bind Area <ButtonRelease-1> {StopMove %x %y}
  $PT(canvas) bind Annotation <1> {AnnotationMoveStart %x %y}
  $PT(canvas) bind Annotation <B1-Motion> {AnnotationMove %x %y}
  $PT(canvas) bind NodeAnnotation <ButtonRelease-1> {AnnotationStopMove %x %y}
  $PT(canvas) bind EdgeAnnotation <ButtonRelease-1> {EdgeAnnoStopMove %x %y}
  $PT(canvas) bind Block <Double-1> {
    BlockDialog $Id2Name([$PT(canvas) find withtag current])
  }
  $PT(canvas) bind Box <Double-1> {
    BoxDialog $Id2Name([$PT(canvas) find withtag current])
  }
  $PT(canvas) bind Place <Double-1> {
    PlaceDialog $Id2Name([$PT(canvas) find withtag current])
  }
#TODO: bind keys to places and skip check for place (faster?)
  bind $PT(canvas) m {
    set tags [$PT(canvas) gettags current]
    if {[lsearch $tags Place] > -1} {
      set place $Id2Name([$PT(canvas) find withtag current])
      DelAllTokens $place
      $PT(net) Place $place set Marking \
        [expr 1+[$PT(net) Place $place get Marking]]
      LayoutTokens $place
    }
    $PT(net) clearActive
  }
  bind $PT(canvas) <Shift-Key-M> {
    set tags [$PT(canvas) gettags current]
    if {[lsearch $tags Place] > -1} {
      set place $Id2Name([$PT(canvas) find withtag current])
      set num [expr [$PT(net) Place $place get Marking]-1]
      if {$num>=0} {
        DelAllTokens $place
        $PT(net) Place $place set Marking $num
        LayoutTokens $place
      }
    }
    $PT(net) clearActive
  }
  $PT(canvas) bind Transition <Double-1> {
    TransDialog Transition $Id2Name([$PT(canvas) find withtag current])
  }
  $PT(canvas) bind PTransition <Double-1> {
    TransDialog PTransition $Id2Name([$PT(canvas) find withtag current])
  }
  $PT(canvas) bind Edge <Button-3> {
    set edge [$PT(canvas) find withtag current]
    set tags [$PT(canvas) gettags $edge]
    if {[lsearch $tags Deletable] > -1} {
      EdgeDialog $Id2Name([$PT(canvas) find withtag current])
    }
  }
  $PT(canvas) bind Transition <Button-2> {
    ShowPhantomRelation Transition $Id2Name([$PT(canvas) find withtag current])
  }
  $PT(canvas) bind PTransition <Button-2> {
    ShowPhantomRelation PTransition $Id2Name([$PT(canvas) find withtag current])
  }
  $PT(canvas) bind Title <Double-1> {CreateTitle %x %y}
  $PT(canvas) bind Title <1> {AnnotationMoveStart %x %y}
  $PT(canvas) bind Title <B1-Motion> {AnnotationMove %x %y}
  $PT(canvas) bind Title <ButtonRelease-1> {TitleStopMove %x %y}
  bind $PT(canvas) p {NewPlace %x %y}
  bind $PT(canvas) t {NewTransition %x %y}
# Todo: Ist die naechste Zeile ausreichend fuer das Verschiebungsproblem?
# NEIN:dann neues Problem bind $PT(canvas) a {UnsetAllBindings; EdgeBindings}
  bind $PT(canvas) a {EdgeBindings}
  bind $PT(canvas) r {Repaint}
  bind $PT(canvas) + {Bigger}
  bind $PT(canvas) - {Smaller}
  bind $PT(canvas) <Delete> DeleteElements
  bind . <Control-q> Quit
  bind $PT(canvas) <Control-a> SelectAll
  bind $PT(canvas) <<Cut>> Cut
  bind $PT(canvas) <<Copy>> Copy
  bind $PT(canvas) <<Paste>> Paste
  bind $PT(canvas) <Escape> ClearAreas
  bind $PT(canvas) <2> {if {$PT(stroke) == 1} {StrokeStop} else {AreaStart %x %y} }
  bind $PT(canvas) <B2-Motion> {AreaStroke %x %y}
  bind $PT(canvas) <c> {CreateTitle %x %y}

    # We are so far.
  $PT(ModeW) configure -text "Edit Mode"
  bind .g.fr2b.edge \
    <Any-Enter> {set PT(messageText) "Make Edge"}
  bind .g.fr2b.edge \
    <Any-Leave> {set PT(messageText) ""}
  bind .g.fr3b.reset \
    <Any-Enter> {set PT(messageText) "Reset to initial marking"}
  bind .g.fr3b.reset \
    <Any-Leave> {set PT(messageText) ""}
  bind .g.fr3b.execute \
    <Any-Enter> {set PT(messageText) "Execute transition"}
  bind .g.fr3b.execute \
    <Any-Leave> {set PT(messageText) ""}
  bind .g.fr3b.animate \
    <Any-Enter> {set PT(messageText) "Animate"}
  bind .g.fr3b.animate \
    <Any-Leave> {set PT(messageText) ""}
  bind .g.fr3b.ref \
    <Any-Enter> {set PT(messageText) "Show Reference"}
  bind .g.fr3b.ref \
    <Any-Leave> {set PT(messageText) ""}
  bind .g.fr3b.expose \
    <Any-Enter> {set PT(messageText) "Exposed Contents All"}
  bind .g.fr3b.expose \
    <Any-Leave> {set PT(messageText) ""}
  bind .g.fr4b.new \
    <Any-Enter> {set PT(messageText) "Delete Net"}
  bind .g.fr4b.new \
    <Any-Leave> {set PT(messageText) ""}
  bind .g.fr4b.open \
    <Any-Enter> {set PT(messageText) "Open File"}
  bind .g.fr4b.open \
    <Any-Leave> {set PT(messageText) ""}
  bind .g.fr4b.save \
    <Any-Enter> {set PT(messageText) "Save Net"}
  bind .g.fr4b.save \
    <Any-Leave> {set PT(messageText) ""}
  bind .g.fr4c.cut \
    <Any-Enter> {set PT(messageText) "Cut Selection"}
  bind .g.fr4c.cut \
    <Any-Leave> {set PT(messageText) ""}
  bind .g.fr4c.copy \
    <Any-Enter> {set PT(messageText) "Copy Selection"}
  bind .g.fr4c.copy \
    <Any-Leave> {set PT(messageText) ""}
  bind .g.fr4c.paste \
    <Any-Enter> {set PT(messageText) "Paste Clipboard"}
  bind .g.fr4c.paste \
    <Any-Leave> {set PT(messageText) ""}
  bind .g.fr4d.print \
    <Any-Enter> {set PT(messageText) "Print Net"}
  bind .g.fr4d.print \
    <Any-Leave> {set PT(messageText) ""}
  bind .g.fr2b.transition \
    <Any-Enter> {set PT(messageText) "Make Transition"}
  bind .g.fr2b.transition \
    <Any-Leave> {set PT(messageText) ""}
  bind .g.fr2b.place \
    <Any-Enter> {set PT(messageText) "Make Place"}
  bind .g.fr2b.place \
    <Any-Leave> {set PT(messageText) ""}
  bind .g.fr2b.comment \
    <Any-Enter> {set PT(messageText) "Make Comment"}
  bind .g.fr2b.comment \
    <Any-Leave> {set PT(messageText) ""}

  send_to_simulator "stateLLEdit"
}  ;# EditBindings

# -------------------------------------------------------------
# Name
#   AnnotationMoveStart
# ---------
# Parameter 
#   x, y -> wo war der Event
# ---------
# Description
#   Eine Annotation soll verschoben werden. Dazu werden die Anfangs- bzw. 
#   Startwerte in PT(x1), PT(y1) abgelegt.
proc AnnotationMoveStart {x y} {
  global PT

  DeselectAll
  set coords [$PT(canvas) coords current]
  $PT(canvas) itemconfigure current -fill blue
#  set title [$PT(canvas) find withtag current]
  $PT(canvas) addtag selected withtag current
    # Differenz zwischen Startpunkt (Cursor) und linker, oberen Ecke des Textes
  set PT(xv) [expr int(($x - [lindex $coords 0]) / $PT(Zoom))]
  set PT(yv) [expr int(($y - [lindex $coords 1]) / $PT(Zoom))]
    # Startpunkt
  set PT(x1) $x 
  set PT(y1) $y
}  ;# AnnotationMoveStart

# -------------------------------------------------------------
# Name
#   AnnotationMove
# ---------
# Parameter 
#   x, y -> wo war der Event
# ---------
# Description
#   Eine Annotation wird verschoben. Sie wird um die Differnz
#   (x-PT(x1), y-PT(y1) verschoben. PT(x1) und PT(y1) werden aktualisiert.
proc AnnotationMove {x y} {
  global PT

  set c [$PT(canvas) find withtag current]
  $PT(canvas) move $c [expr ($x-$PT(x1))] [expr ($y-$PT(y1))]
  set PT(x1) $x
  set PT(y1) $y
}  ;# AnnotationMove

# -------------------------------------------------------------
# Name
#   GetType
# ---------
# Parameter 
#   c -> Canvas-ID des Elements
# ---------
# Description
#   Ermittelt den zu der ID c gehoerenden Typ des Knotens
#   (Block, Place, Transition oder PTransition).
# ---------
# Rueckgabe
#   Der Typ (Block, Place, Transition, PTransition) 
proc GetType {c} {
  global PT

  set tags [$PT(canvas) gettags $c]
  return [lindex $tags \
    [lsearch -regexp $tags {Block|Place|Transition|PTransition|Box}]]
}  ;# GetType

# -------------------------------------------------------------
# Name
#   EdgeAnnoStopMove
# ---------
# Parameter 
#   x, y -> Event-Position
# ---------
# Description
#   Eine Annotation einer Kante wurde an (x, y) verschoben. Dem C++-Kern
#   wird die neue Position der Annotation mitgeteilt.
proc EdgeAnnoStopMove {x y} {
  global PT IsAnnoFrom

  set c [$PT(canvas) find withtag current]
  set tags [$PT(canvas) gettags current]
  set element $IsAnnoFrom($c)
  set coords [$PT(canvas) coords $element]
  set nx [expr [lindex $coords 0] + \
    ([lindex $coords 2] - [lindex $coords 0])/2]
  set ny [expr [lindex $coords 1] + \
    ([lindex $coords 3] - [lindex $coords 1])/2]
  Grid $x $y
  if {[lsearch $tags Weight*] > -1} {
    $PT(net) Edge $element set WeightX [expr int($x-$nx-$PT(xv))] \
      WeightY [expr int($y-$ny-$PT(yv))]
  }  ;# if Kantengewicht wurde verschoben
  if {[lsearch $tags Vars*] > -1} {
    $PT(net) Edge $element set VarsX [expr int($x-$nx-$PT(xv))] \
      VarsY [expr int($y-$ny-$PT(yv))]
  }  ;# if Variablen wurden verschoben
}  ;# EdgeAnnoStopMove
    
# -------------------------------------------------------------
# Name
#   AnnotationStopMove
# ---------
# Parameter 
#   x, y -> Event-Position
# ---------
# Description
#   Eine Annotation eines Knotens wurde an (x, y) verschoben. Dem C++-Kern
#   wird die neue Position der Annotation mitgeteilt.
proc AnnotationStopMove {x y} {
  global PT IsAnnoFrom

  set c [$PT(canvas) find withtag current]
  $PT(canvas) itemconfigure current -fill black
  $PT(canvas) dtag selected
  set tags [$PT(canvas) gettags current]
  set element $IsAnnoFrom($c)
  set nx [expr [lindex [$PT(canvas) coords $element] 0] + $PT(OSizeHalf) ]
  set ny [expr [lindex [$PT(canvas) coords $element] 1] + $PT(OSizeHalf) ]
  Grid $x $y
  set anno [lindex $tags [lsearch -regexp $tags Name|Mean|Type|AT|LT|VT|Time]]
  set ann_type [lindex [split $anno @] 0]
  $PT(net) [GetType $element] $element set \
    [format "%sX" $ann_type] [expr int(($x-$nx) / $PT(Zoom)-$PT(xv))] \
    [format "%sY" $ann_type] [expr int(($y-$ny) / $PT(Zoom)-$PT(yv))]
}  ;# AnnotationStopMove

# -------------------------------------------------------------
# Name
#   EdgeBindings
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Eine neue Kante soll kreiert werden. Binding wird auf alle
#   Knoten gelegt. Falls Button auf Knoten -> SetFirstEdge.
# Todo: Farben
# TODO: Probleme mit dem Binding
proc EdgeBindings {} {
  global PT

  $PT(canvas) bind Node <Enter> {
    set node $Id2Name([%W find withtag current])
    if {![$PT(net) Exists Block $node]} {
      set PT(OldColor) [lindex [%W itemconfigure current -fill] 4]
      %W itemconfigure current -fill blue
    }
  }  ;# Canvas-Binding <Enter> fuer den Knoten
  $PT(canvas) bind Node <Leave>  {
    set node $Id2Name([%W find withtag current])
    if {![$PT(net) Exists Block $node]} {
      %W itemconfigure current -fill $PT(OldColor)
    }
  }  ;# Canvas-Binding <Leave> fuer den Knoten
  $PT(canvas) bind Node <1> SetFirstEdge
  $PT(ModeW) configure -text "Select First Node"
}  ;# EdgeBinding

# -------------------------------------------------------------
# Name
#   PEdgeBindings
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Eine neue Phantomkante soll kreiert werden. Binding wird auf alle
#   Knoten gelegt. Falls Button auf Knoten -> SetFirstEdge.
# Todo: Farben
# TODO: Probleme mit dem Binding
proc PEdgeBindings {} {
  global PT

  $PT(canvas) bind Place <Enter> { 
    set PT(OldColor) [lindex [%W itemconfigure current -fill] 4] 
    %W itemconfigure current -fill blue 
  }  ;# Canvas-Binding <Enter> fuer den Knoten
  $PT(canvas) bind Place <Leave>  { 
    %W itemconfigure current -fill $PT(OldColor) 
  }  ;# Canvas-Binding <Leave> fuer den Knoten
  $PT(canvas) bind Place <1> SetFirstPEdge
  $PT(canvas) bind PTransition <Enter> { 
    set PT(OldColor) [lindex [%W itemconfigure current -fill] 4] 
    %W itemconfigure current -fill blue 
  }  ;# Canvas-Binding <Enter> fuer den Knoten
  $PT(canvas) bind PTransition <Leave>  { 
    %W itemconfigure current -fill $PT(OldColor) 
  }  ;# Canvas-Binding <Leave> fuer den Knoten
  $PT(canvas) bind PTransition <1> SetFirstPEdge
  $PT(ModeW) configure -text "Select First Node"
}  ;# PEdgeBinding

# -------------------------------------------------------------
# Name
#   DeleteEdge
# ---------
# Parameter 
#   name -> Name der Kante
# ---------
# Description
#   Eine Kante soll geloescht werden. Die zugehoerigen Annotationen werden
#   ermittelt und geloescht. 
proc DeleteEdge {name} {
  global PT IsAnnoFrom

  set v [$PT(canvas) find withtag Vars@$name]
  set w [$PT(canvas) find withtag Weight@$name]
  if { $v != ""} { 
    $PT(canvas) delete $v
    unset IsAnnoFrom($v)
  }  ;# if Variablen vorhanden
  if {$w != ""} {
    $PT(canvas) delete $w
    unset IsAnnoFrom($w)
  }  ;# if Kantengewicht vorhanden
  $PT(canvas) delete point
  $PT(canvas) delete $name
}  ;# DeleteEdge

# -------------------------------------------------------------
# Name
#   DeleteElements
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Alle Canvas-Objekte mit dem Tag selected sollen entfernt werden.
#   Bloecke, Transition oder Stellen werden rausgesucht und geloescht. 
#   Alles was zu dem jeweiligen Knoten gehoert, wird ebenfalls geloescht.
# Todo: Blockkante, Phantome
proc DeleteElements {} {
  global PT Id2Name IsAnnoFrom Frames

  set elements [$PT(canvas) find withtag selected]
  if {[llength $elements] > 4} {
    if {[tk_dialog .d {Delete Elements} {Really delete the selection?} \
      warning 0 {Sure} {Njet}] == 1} {
      return ;
    }  ;# if Dialogabfrage ergab Abbruch
  }  ;# if mehr als 4 Elemente loeschen
  foreach c $elements {
    set tags [$PT(canvas) gettags $c]
    if {[lsearch $tags Block] > -1} {
        # Block loeschen, vorerst nur den Block, kein Inhalt
      set name $Id2Name($c)
      DeleteNodeAnno $name
      set bedges [$PT(canvas) find withtag BEdge]
      foreach b $bedges {
        set elements $Id2Name($b)
        set liste [split $elements "@"]
        foreach e $liste {
          if {$e == $name} {
            $PT(canvas) delete $b
          }
        }
      }
      $PT(net) delete Block $name
      $PT(canvas) delete $c
    } elseif {[lsearch $tags Box] > -1} {
        # Box loeschen, ohne Inhalt
      set name $Id2Name($c)
      DeleteNodeAnno $name
      set bedges [$PT(canvas) find withtag BEdge]
      foreach b $bedges {
        set elements $Id2Name($b)
        set liste [split $elements "@"]
        foreach e $liste {
          if {$e == $name} {
            $PT(canvas) delete $b
          }
        }
      }
      $PT(net) delete Box $name
      $PT(canvas) delete $c
    } elseif {[lsearch $tags Place] > -1} {
        # Stelle loeschen
      set name $Id2Name($c)
      catch {destroy $Frames($name); unset $Frames($name)}
      foreach ie [$PT(net) Place $name get InEdges] {
        DeleteEdge $ie
      }  ;# foreach eingehende Kante
      foreach oe [$PT(net) Place $name get OutEdges] {
        DeleteEdge $oe
      }  ;# foreach ausgehende Kante
        # dann noch die Tokens loeschen
      DelAllTokens $name
      set bedges [$PT(canvas) find withtag BEdge]
      foreach b $bedges {
        set elements $Id2Name($b)
        set liste [split $elements "@"]
        foreach e $liste {
          if {$e == $name} {
            $PT(canvas) delete $b
          }
        }
      }
      DeleteNodeAnno $name
      $PT(net) delete Place $name
      $PT(canvas) delete $c
    } elseif {[lsearch $tags Transition] > -1} {
        # Transition loeschen
      set name $Id2Name($c)
      foreach ie [$PT(net) Transition $name get InEdges] {
        DeleteEdge $ie
      }  ;# foreach eingehende Kante
      foreach oe [$PT(net) Transition $name get OutEdges] {
        DeleteEdge $oe
      }  ;# foreach rausgehende Kante
      set bedges [$PT(canvas) find withtag BEdge]
      foreach b $bedges {
        set elements $Id2Name($b)
        set liste [split $elements "@"]
        foreach e $liste {
          if {$e == $name} {
            $PT(canvas) delete $b
          }
        }
      }
      DeleteNodeAnno $name
      $PT(net) delete Transition $name
      $PT(canvas) delete $c
    } elseif { [lsearch $tags Edge] > -1} {
        # Kante entfernen
      set name $Id2Name($c)
      DeleteEdge $name
      $PT(net) delete Edge $name
    } elseif { [lsearch $tags PTransition] > -1} {
      set name $Id2Name($c)
      foreach ie [$PT(net) PTransition $name get InPEdges] {
        DeleteEdge $ie
      }  ;# foreach eingehende Kante
      foreach oe [$PT(net) PTransition $name get OutPEdges] {
        DeleteEdge $oe
      }  ;# foreach rausgehende Kante
      DeleteNodeAnno $name
      $PT(net) delete PTransition $name
      $PT(canvas) delete $c
    } elseif { [lsearch $tags Title] > -1} {
      set title [$PT(canvas) find withtag selected]
      $PT(canvas) delete $title
      $PT(net) delete Title $title
    }
  }  ;# foreach zu loeschendes Graphikelement
  ClearAreas
  NetChanged
  UnsetAllBindings
  EditBindings
}  ;# DeleteElements

# -------------------------------------------------------------
# Name
#   Center
# ---------
# Parameter 
#   which -> bigrectangle (Block), oval (Stelle) oder rectangle (Transition)
#   x, y  -> Position
# ---------
# Description
#   Ein Rechteck oder Kreis wird um den Punkt (x, y) zentriert.
#   Falls es nocht nicht exisiert wird das Objekt kreiert und mit dem
#   Tag TempNode versehen.
# Todo: Farben, speziell PT(color.node)
proc Center {which x y {block ""}} {
  global PT

  if {[$PT(canvas) find withtag TempNode] == {}} {
    if [string compare $which "bigrectangle"] {
      $PT(canvas) create $which \
        [expr $x - $PT(OSizeHalf)] [expr $y - $PT(OSizeHalf)] \
        [expr $x + $PT(OSizeHalf)] [expr $y + $PT(OSizeHalf)] \
        -fill $PT(color.node)
    } else {
      if {[lsearch -regexp $block Box] > -1} {
        if {$PT(Filter) == "*.hl_net"} {
          if {[$PT(net) IsAbstract $block]} {
            image create bitmap box$block -file "$PT(bitmaps)/boxab$PT(Zoom)" \
                          -maskfile "$PT(bitmaps)/boxmask$PT(Zoom)" \
                          -foreground black -background grey
          } else {
            image create bitmap box$block -file "$PT(bitmaps)/box$PT(Zoom)" \
                          -maskfile "$PT(bitmaps)/boxmask$PT(Zoom)" \
                          -foreground black -background grey
          }
        } else {
          image create bitmap box$block -file "$PT(bitmaps)/box$PT(Zoom)" \
                          -maskfile "$PT(bitmaps)/boxmask$PT(Zoom)" \
                          -foreground black -background grey
        }
        set block [$PT(canvas) create image $x $y -image box$block]
      } else {
        image create bitmap block$block -file "$PT(bitmaps)/block$PT(Zoom)" \
                          -maskfile "$PT(bitmaps)/mask$PT(Zoom)" \
                          -foreground black -background white
        set block [$PT(canvas) create image $x $y -image block$block]
      }
      return $block
#      return [$PT(canvas) create image $x $y -image block]
#      return [$PT(canvas) create bitmap -file \
#      "@$PT(bitmaps)/block" -maskfile "@$PT(bitmaps)/mask" \
#      -foreground black -background white]
#      $PT(canvas) create polygon \
#        [expr $x - $PT(ObjectSize)] [expr $y - $PT(ObjectSize) + 6] \
#        [expr $x - $PT(ObjectSize) + 6] [expr $y - $PT(ObjectSize)] \
#        [expr $x + $PT(ObjectSize) - 6] [expr $y - $PT(ObjectSize)] \
#        [expr $x + $PT(ObjectSize)] [expr $y - $PT(ObjectSize) + 6] \
#        [expr $x + $PT(ObjectSize)] [expr $y + $PT(ObjectSize) - 6] \
#        [expr $x + $PT(ObjectSize) - 6] [expr $y + $PT(ObjectSize)] \
#        [expr $x - $PT(ObjectSize) + 6] [expr $y + $PT(ObjectSize)] \
#        [expr $x - $PT(ObjectSize)] [expr $y + $PT(ObjectSize) - 6] \
#        -fill $PT(color.block) -outline black -width 2
    }  ;# else, es war ein Block gemeint
  } else {
    if [string compare $which "bigrectangle"] {
      $PT(canvas) coords TempNode \
        [expr $x - $PT(OSizeHalf)] [expr $y - $PT(OSizeHalf)] \
        [expr $x + $PT(OSizeHalf)] [expr $y + $PT(OSizeHalf)]
    } else {
       $PT(canvas) coords TempNode $x $y
#      $PT(canvas) coords TempNode \
#        [expr $x - $PT(ObjectSize)] [expr $y - $PT(ObjectSize) + 6] \
#        [expr $x - $PT(ObjectSize) + 6] [expr $y - $PT(ObjectSize)] \
#        [expr $x + $PT(ObjectSize) - 6] [expr $y - $PT(ObjectSize)] \
#        [expr $x + $PT(ObjectSize)] [expr $y - $PT(ObjectSize) + 6] \
#        [expr $x + $PT(ObjectSize)] [expr $y + $PT(ObjectSize) - 6] \
#        [expr $x + $PT(ObjectSize) - 6] [expr $y + $PT(ObjectSize)] \
#        [expr $x - $PT(ObjectSize) + 6] [expr $y + $PT(ObjectSize)] \
#        [expr $x - $PT(ObjectSize)] [expr $y + $PT(ObjectSize) - 6]
    }  ;# else, es war ein Block gemeint
  }  ;# else, Objekt bereits vorhanden
}  ;# Center

# -------------------------------------------------------------
# Name
#   DrawBlock
# ---------
# Parameter 
#   x, y   -> Position
# ---------
# Description
#   Ein Rechteck wird an (x, y) erstellt und mit Tag TempNode versehen.
proc DrawBlock {x y} {
  global PT

  Grid $x $y
  set item [Center bigrectangle $x $y B]
  $PT(canvas) addtag TempNode withtag $item
}  ;# DrawBlock

# -------------------------------------------------------------
# Name
#   DrawBox
# ---------
# Parameter 
#   x, y   -> Position
# ---------
# Description
#   Ein Rechteck wird an (x, y) erstellt und mit Tag TempNode versehen.
proc DrawBox {x y} {
  global PT

  Grid $x $y
  set item [Center bigrectangle $x $y Box]
  $PT(canvas) addtag TempNode withtag $item
}  ;# DrawBlock

# -------------------------------------------------------------
# Name
#   DrawPlace
# ---------
# Parameter 
#   x, y   -> Position
# ---------
# Description
#   Eine Kreis wird an (x, y) erstellt und mit Tag TempNode versehen.
proc DrawPlace {x y} {
  global PT

  Grid $x $y
  set item [Center oval $x $y]
  $PT(canvas) addtag TempNode withtag $item
}  ;# DrawPlace

# -------------------------------------------------------------
# Name
#   DrawTransition
# ---------
# Parameter 
#   x, y   -> Position
# ---------
# Description
#   Eine Rechteck wird an (x,y) erstellt und mit Tag TempNode versehen.
proc DrawTransition {x y} {
  global PT

  Grid $x $y
  set item [Center rectangle $x $y]
  $PT(canvas) addtag TempNode withtag $item
}  ;# DrawTransition

# -------------------------------------------------------------
# Name
#   DrawPTransition
# ---------
# Parameter 
#   x, y   -> Position
# ---------
# Description
#   Eine Rechteck wird an (x,y) erstellt und mit Tag TempNode versehen.
proc DrawPTransition {x y} {
  global PT

  Grid $x $y
  set item [Center rectangle $x $y]
  $PT(canvas) addtag TempNode withtag $item
}  ;# DrawTransition

# -------------------------------------------------------------
# Name
#   DrawEdge
# ---------
# Parameter 
#   x, y   -> Position
# ---------
# Description
#   Eine Kante wird von der Position des Objektes mit dem Tag FirstNode
#   zu (x, y) kreiert und wieder mit mit Tag TempEdge versehen.
proc DrawEdge {x y} {
  global PT

  $PT(canvas) delete TempEdge
  Grid $x $y
  set x1 [lindex [$PT(canvas) coords FirstNode] 0]
  set y1 [lindex [$PT(canvas) coords FirstNode] 1]
  set x0 [expr $x1+$PT(OSizeHalf)]
  set y0 [expr $y1+$PT(OSizeHalf)]
  set xm [expr int(($x1+$x0) / 2)]
  set ym [expr int(($y1+$y0) / 2)]

  set li  [$PT(canvas) create line \
		   $x0 $y0 \
		   $xm $ym \
		   $x $y \
		   -smooth true -splinesteps 20 \
		   -arrow last \
		   -width 1 -tags TempEdge]

  $PT(canvas) raise TempEdge grid
}  ;# DrawEdge

proc DrawPEdge {x y} {
  global PT

  $PT(canvas) delete TempEdge
  Grid $x $y
  set x1 [lindex [$PT(canvas) coords FirstNode] 0]
  set y1 [lindex [$PT(canvas) coords FirstNode] 1]
  set x0 [expr $x1+$PT(OSizeHalf)]
  set y0 [expr $y1+$PT(OSizeHalf)]
  set xm [expr int(($x1+$x0) / 2)]
  set ym [expr int(($y1+$y0) / 2)]

  set li  [$PT(canvas) create line \
		   $x0 $y0 \
		   $xm $ym \
		   $x $y \
		   -smooth true -splinesteps 20 \
		   -arrow last \
		   -width 1 -tags TempEdge]

  $PT(canvas) raise TempEdge grid
}  ;# DrawPEdge

# -------------------------------------------------------------
# Name
#   SetFirstEdge
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Eine neue Kante soll erstellt werden. Der erste Knoten der Kante
#   wurde ausgewaehlt (Tag current). Es wird festgestellt, was es fuer ein
#   Knoten ist und eine entprechende Binding auf das Gegenstueck gelegt,
#   so das bei zuerst gewaehlter Transition danach nur eine Stelle
#   ausgewaehlt werden kann. Die Gegenstuecke werden dann noch bei
#   Leave oder Enter-Event eingefaerbt.
# Todo: Farben
proc SetFirstEdge {} {
  global PT

  UnsetAllBindings
  set type {}
  bind $PT(canvas) <Escape> {
        $PT(canvas) itemconfigure FirstNode -fill white
        $PT(canvas) dtag FirstNode
        if {[lsearch [$PT(canvas) gettags current] Block|Box] == -1} {
          $PT(canvas) itemconfigure current -fill white
        }
        UnsetAllBindings
        $PT(canvas) delete TempEdge
        EditBindings
  }
  bind $PT(canvas) <Delete> {
        $PT(canvas) itemconfigure FirstNode -fill white
        $PT(canvas) dtag FirstNode
        if {[lsearch [$PT(canvas) gettags current] Block|Box] == -1} {
          $PT(canvas) itemconfigure current -fill white
        }
        UnsetAllBindings
        $PT(canvas) delete TempEdge
        EditBindings
  }
  if {[lsearch [$PT(canvas) gettags current] Place] > -1} {
      # bind $PT(canvas) <1> {}
    $PT(canvas) bind Transition <1> {NewEdge}
      # $PT(canvas) itemconfigure current -fill green
    set type Transition
      # spiegelbildlich, da FirstNode Place war
    set PT(type) Place
    $PT(canvas) addtag FirstNode withtag current
    bind $PT(canvas) <Motion> "DrawEdge %x %y"
    set PT(oldx) $PT(x1)
    set PT(oldy) $PT(y1)
    $PT(ModeW) configure -text "Select Transition"
  } elseif {[lsearch [$PT(canvas) gettags current] Transition] > -1} {
      # bind $PT(canvas) <1> {}
    $PT(canvas) bind Place <1> {NewEdge}
    set type Place
      # spiegelbildlich, da FirstNode Transition war
    set PT(type) Transition
      # PT(canvas) itemconfigure current -fill green
    $PT(canvas) addtag FirstNode withtag current
    bind $PT(canvas) <Motion> "DrawEdge %x %y"
    set PT(oldx) $PT(x1)
    set PT(oldy) $PT(y1)
    $PT(ModeW) configure -text "Select Place"
  }
  $PT(canvas) bind $type <Enter> {
    set PT(OldColor) [lindex [$PT(canvas) itemconfigure current -fill] 4]
    %W itemconfigure current -fill blue
  }
  $PT(canvas) bind $type <Leave> {
    %W itemconfigure current -fill $PT(OldColor)
  }
}  ;# SetFirstEdge


proc SetFirstPEdge {} {
  global PT

  UnsetAllBindings
  set type {}
  bind $PT(canvas) <Escape> {
        $PT(canvas) itemconfigure FirstNode -fill white
        $PT(canvas) dtag FirstNode
        $PT(canvas) itemconfigure current -fill white
        UnsetAllBindings
        $PT(canvas) delete TempEdge
        EditBindings
  }
  bind $PT(canvas) <Delete> {
        $PT(canvas) itemconfigure FirstNode -fill white
        $PT(canvas) dtag FirstNode
        $PT(canvas) itemconfigure current -fill white
        UnsetAllBindings
        $PT(canvas) delete TempEdge
        EditBindings
  }
  if {[lsearch [$PT(canvas) gettags current] Place] > -1} {
      # bind $PT(canvas) <1> {}
    $PT(canvas) bind PTransition <1> {NewPEdge}
      # $PT(canvas) itemconfigure current -fill green
    set type PTransition
      # spiegelbildlich, da FirstNode Place war
    set PT(type) Place
    $PT(canvas) addtag FirstNode withtag current
    bind $PT(canvas) <Motion> {DrawPEdge %x %y}
    set PT(oldx) $PT(x1)
    set PT(oldy) $PT(y1)
    $PT(ModeW) configure -text "Select Transition"
  } elseif {[lsearch [$PT(canvas) gettags current] PTransition] > -1} {
      # bind $PT(canvas) <1> {}
    $PT(canvas) bind Place <1> {NewPEdge}
    set type Place
      # spiegelbildlich, da FirstNode Transition war
    set PT(type) PTransition
      # PT(canvas) itemconfigure current -fill green
    $PT(canvas) addtag FirstNode withtag current
    bind $PT(canvas) <Motion> {DrawPEdge %x %y}
    set PT(oldx) $PT(x1)
    set PT(oldy) $PT(y1)
    $PT(ModeW) configure -text "Select Place"
  }
  $PT(canvas) bind $type <Enter> {
    set PT(OldColor) [lindex [$PT(canvas) itemconfigure current -fill] 4]
    %W itemconfigure current -fill blue
  }
  $PT(canvas) bind $type <Leave> {
    %W itemconfigure current -fill $PT(OldColor)
  }
}  ;# SetFirstEdge

# -------------------------------------------------------------
# Name
#   SetupEdge
# ---------
# Parameter 
#   n1 -> Name des ersten Knotens
#   n2 -> Name des zweiten Knotens
#   edge -> Canvas ID der Kante, falls die schon existiert, sonst -12345
# ---------
# Description
#   Fuer eine Kante von n1 nach n2 werden die Anfangs- und Endpositionen
#   bestimmt. Diese Berechnung erfolgt im C++-Kern (Geschwindigkeit)
#   Falls die Kante noch nicht existiert wird sie erzeugt, sonst die
#   werden die Koordinaten der Kante entprechend veraendert.
proc SetupEdge {n1 n2 {edge "-12345"}} {
  global PT Id2Name

  NetChanged
  set tags1 [$PT(canvas) gettags $n1]
  set type1 [lindex $tags1 [lsearch -regexp $tags1 Transition|Place]]
  if {[lsearch -exact $tags1 $n1]==-1} {
    set n1 $Id2Name($n1)
  }

  set tags2 [$PT(canvas) gettags $n2]
  set type2 [lindex $tags2 [lsearch -regexp $tags2 Transition|Place]]
  if {[lsearch -exact $tags2 $n2]==-1} {
    set n2 $Id2Name($n2)
  }

  set x1 [lindex [$PT(canvas) coords $n1] 0]
  set y1 [lindex [$PT(canvas) coords $n1] 1]
  set x2 [lindex [$PT(canvas) coords $n2] 0]
  set y2 [lindex [$PT(canvas) coords $n2] 1]

 
  if {$edge == -12345} {
    set typea [lindex $tags1 [lsearch -regexp $tags1 Block|Place|Transition|PTransition]]
    set ea [$PT(net) $typea $n1 get OutEdges]
    set typeb [lindex $tags2 [lsearch -regexp $tags2 Block|Place|Transition|PTransition]]
    set eb [$PT(net) $typeb $n2 get InEdges]
    set found -1
    foreach a $ea {
        if {[lsearch $eb $a]!=-1} {
          set found $a
	  break
        }
    }
    if {$found == -1} {
	return TCL_ERROR
    }
    set xm [$PT(net) Edge $found get SplineX]
    set ym [$PT(net) Edge $found get SplineY]
    if {($xm == 0) && ($ym == 0)} {
      set xm [expr int(($x1 + $x2) / 2 + $PT(OSizeHalf))]
      set ym [expr int(($y1 + $y2) / 2 + $PT(OSizeHalf))]
    } else {
      set xm [expr int($xm * $PT(Zoom))]
      set ym [expr int($ym * $PT(Zoom))]
    }
    set offset [$PT(net) CalcOffset $type1 $type2 [expr int($x1)] \
      [expr int($y1)] $xm $ym [expr int($x2)] [expr int($y2)]]
    $PT(canvas) create line \
            [lindex $offset 0] [lindex $offset 1] \
            $xm $ym \
            [lindex $offset 2] [lindex $offset 3] \
            -smooth true -splinesteps 20 -arrow last -width 1 -tags Edge
  } else {
    set xm [$PT(net) Edge $edge get SplineX]
    set ym [$PT(net) Edge $edge get SplineY]
    if {($xm == 0) && ($ym == 0)} {
      set xm [expr int(($x1 + $x2) / 2 + $PT(OSizeHalf))]
      set ym [expr int(($y1 + $y2) / 2 + $PT(OSizeHalf))]
    } else {
      set xm [expr int($xm * $PT(Zoom))]
      set ym [expr int($ym * $PT(Zoom))]
    }
    set offset [$PT(net) CalcOffset $type1 $type2 [expr int($x1)] \
      [expr int($y1)] $xm $ym [expr int($x2)] [expr int($y2)]]
#    if {[expr [$PT(net) Edge $edge get Visibility ] != 1 && \
#        $PT(InvisibleArcsOn) == 0]} {
#      return
#    }
    $PT(canvas) coords $edge \
       [lindex $offset 0] [lindex $offset 1] \
       $xm $ym \
       [lindex $offset 2] [lindex $offset 3]
  }  ;# else
}  ;# SetupEdge

# -------------------------------------------------------------
# Name
#   SetupPEdge
# ---------
# Parameter 
#   n1 -> Name des ersten Knotens
#   n2 -> Name des zweiten Knotens
#   edge -> Canvas ID der Kante, falls die schon existiert, sonst -12345
# ---------
# Description
#   Fuer eine Phantomkante von n1 nach n2 werden die Anfangs- und Endpositionen
#   bestimmt. Diese Berechnung erfolgt im C++-Kern (Geschwindigkeit)
#   Falls die Phantomkante noch nicht existiert wird sie erzeugt, sonst die
#   werden die Koordinaten der Phantomkante entprechend veraendert.
proc SetupPEdge {n1 n2 {edge "-12345"}} {
  global PT Id2Name

  NetChanged
  set x1 [lindex [$PT(canvas) coords $n1] 0]
  set y1 [lindex [$PT(canvas) coords $n1] 1]
  set x2 [lindex [$PT(canvas) coords $n2] 0]
  set y2 [lindex [$PT(canvas) coords $n2] 1]
  set tags1 [$PT(canvas) gettags $n1]
  set type1 [lindex $tags1 [lsearch -regexp $tags1 PTransition|Place]]
  set tags2 [$PT(canvas) gettags $n2]
  set type2 [lindex $tags2 [lsearch -regexp $tags2 PTransition|Place]]

  set gefunden 0
  foreach r1 $tags1 {
    if {$r1 == $n1} {
      set gefunden 1
    }
  }
  if {$gefunden == 0} {
    set n1 $Id2Name($n1)
  }

  foreach r2 $tags2 {
    if {$r2 == $n2} {
      set gefunden 1
    }
  }
  if {$gefunden == 0} {
    set n2 $Id2Name($n2)
  }

  if {$edge == -12345} {
    set tagsa [$PT(canvas) gettags $n1]
    set indexa [lsearch -regexp $tagsa Block|Place|Transition|PTransition]
    set typea [lindex $tagsa $indexa]
    set ea [$PT(net) $typea $n1 get OutPEdges]
    set tagsb [$PT(canvas) gettags $n2]
    set indexb [lsearch -regexp $tagsb Block|Place|Transition|PTransition]
    set typeb [lindex $tagsb $indexb]
    set eb [$PT(net) $typeb $n2 get InPEdges]
    set cx 0
    foreach a $ea {
      set cy 0
      foreach b $eb {
        if {[lindex $eb $cy] == [lindex $ea $cx]} {
          set found [lindex $eb $cy]
        }
        set cy [expr $cy + 1]
      }
      set cx [expr $cx + 1]
    }
    set xm [expr int([$PT(net) PEdge $found get SplineX])]
    set ym [expr int([$PT(net) PEdge $found get SplineY])]
    if {($xm == 0) && ($ym == 0)} {
      set xm [expr int(($x1 + $x2) / 2) + int($PT(OSizeHalf))]
      set ym [expr int(($y1 + $y2) / 2) + int($PT(OSizeHalf))]
    } else {
      set xm [expr int($xm * $PT(Zoom))]
      set ym [expr int($ym * $PT(Zoom))]
    }
    set offset [$PT(net) CalcOffset $type1 $type2 [expr int($x1)] \
      [expr int($y1)] $xm $ym [expr int($x2)] [expr int($y2)]]

    $PT(canvas) create line \
        [lindex $offset 0] [lindex $offset 1] \
        $xm $ym \
        [lindex $offset 2] [lindex $offset 3] -width 1 \
        -smooth true -splinesteps 20 -arrow last \
        -tags Edge -fill $PT(color.phantom_edge)
  } else {
    set xm [$PT(net) PEdge $edge get SplineX]
    set ym [$PT(net) PEdge $edge get SplineY]
    if {($xm == 0) && ($ym == 0)} {
      set xm [expr int(($x1 + $x2) / 2) + int($PT(OSizeHalf))]
      set ym [expr int(($y1 + $y2) / 2) + int($PT(OSizeHalf))]
    } else {
      set xm [expr int($xm * $PT(Zoom))]
      set ym [expr int($ym * $PT(Zoom))]
    }
    set offset [$PT(net) CalcOffset $type1 $type2 [expr int($x1)] \
      [expr int($y1)] $xm $ym [expr int($x2)] [expr int($y2)]]
    if {[expr [$PT(net) PEdge $edge get Visibility ] != 1 && \
      $PT(InvisibleArcsOn) == 0]} {
      return
    }
    $PT(canvas) coords $edge \
         [lindex $offset 0] [lindex $offset 1] \
         $xm $ym \
         [lindex $offset 2] [lindex $offset 3]
    $PT(canvas) itemconfigure $edge -fill $PT(color.phantom_transition)
  }  ;# else
}  ;# SetupPEdge

# -------------------------------------------------------------
# Name
#   SetupBlockEdge
# ---------
# Parameter 
#   n1 -> Name des ersten Knotens
#   n2 -> Name des zweiten Knotens
#   edge -> Canvas ID der Kante, falls die schon existiert, sonst -12345
# ---------
# Description
#   Fuer eine Kante von n1 nach n2 werden die Anfangs- und Endpositionen
#   bestimmt. Diese Berechnung erfolgt im C++-Kern (Geschwindigkeit)
#   Falls die Kante noch nicht existiert wird sie erzeugt, sonst die
#   werden die Koordinaten der Kante entprechend veraendert.
# Todo: Kommentar
proc SetupBlockEdge {n1 n2 {edge "-12345"}} {
  global PT

  set x1 [lindex [$PT(canvas) coords $n1] 0]
  set y1 [lindex [$PT(canvas) coords $n1] 1]
  set x2 [lindex [$PT(canvas) coords $n2] 0]
  set y2 [lindex [$PT(canvas) coords $n2] 1]
  set tags1 [$PT(canvas) gettags $n1]
  set tags2 [$PT(canvas) gettags $n2]
  if {([string length $tags1] == 0) || ([string length $tags2] == 0)} {
    return
  }  ;# if unbekannte Beteiligte ;-)
  set type1 [lindex $tags1 [lsearch -regexp $tags1 Place|Transition|Block|Box]]
  set type2 [lindex $tags2 [lsearch -regexp $tags2 Place|Transition|Block|Box]]
  if {$type2 == "Block" || $type2 == "Box"} {
    set type2 Block
    set offset [$PT(net) CalcOffset $type1 $type2 [expr int($x1)] \
      [expr int($y1)] [expr int($x2 - $PT(ObjectSize))] [expr int($y2 - $PT(ObjectSize) + 6)]]
  } else {
    set type1 Block
    set offset [$PT(net) CalcOffset $type1 $type2 [expr int($x1 - $PT(ObjectSize))] \
      [expr int($y1 - $PT(ObjectSize) + 6)] [expr int($x2)] [expr int($y2)]]
  }
  if {$edge == -12345} {
    eval {$PT(canvas) create line} $offset \
      {-arrow last -tags BEdge -fill $PT(color.block_edge)}
  } else {
    set netedge [string range $edge 0 [expr [string first "@" $edge] - 1]]
 #   set offset [$PT(net) CalcOffset $type1 $type2 [expr int($x1)] \
 #     [expr int($y1)] [expr int($x2)] [expr int($y2)]]
    if {[expr [$PT(net) Edge $netedge get Visibility ] != 1 && \
      $PT(InvisibleArcsOn) == 0]} {
      return
    }
    eval {$PT(canvas) coords $edge} $offset 
  }  ;# else
}  ;# SetupBlockEdge

# -------------------------------------------------------------
# Name
#   SetupBlockEdges
# ---------
# Parameter 
#   n1 -> Name des ersten Knotens
#   n2 -> Name des zweiten Knotens
#   edge -> Canvas ID der Kante, falls die schon existiert, sonst -12345
# ---------
# Description
#   Fuer eine Kante von n1 nach n2 werden die Anfangs- und Endpositionen
#   bestimmt. Diese Berechnung erfolgt im C++-Kern (Geschwindigkeit)
#   Falls die Kante noch nicht existiert wird sie erzeugt, sonst die
#   werden die Koordinaten der Kante entprechend veraendert.
# Todo: Kommentar
proc SetupBlockEdges {ns1 ns2 {edge "-12345"}} {
  global Id2Name

  foreach n1 $ns1 {
    foreach n2 $ns2 {
      SetupBlockEdge $n1 $n2 "$edge@$n1@$n2"
    }  ;# foreach Knoten in ns2
  }  ;# foreach Knoten in ns1
}  ;# SetupBlockEdges

# -------------------------------------------------------------
# Name
#   SetupBlockPEdge
# ---------
# Parameter 
#   n1 -> Name des ersten Knotens
#   n2 -> Name des zweiten Knotens
#   edge -> Canvas ID der Kante, falls die schon existiert, sonst -12345
# ---------
# Description
#   Fuer eine Phantomkante von n1 nach n2 werden die Anfangs- und Endpositionen
#   bestimmt. Diese Berechnung erfolgt im C++-Kern (Geschwindigkeit)
#   Falls die Phantomkante noch nicht existiert wird sie erzeugt, sonst die
#   werden die Koordinaten der Phantomkante entprechend veraendert.
proc SetupBlockPEdge {n1 n2 {edge "-12345"}} {
  global PT

  set x1 [lindex [$PT(canvas) coords $n1] 0]
  set y1 [lindex [$PT(canvas) coords $n1] 1]
  set x2 [lindex [$PT(canvas) coords $n2] 0]
  set y2 [lindex [$PT(canvas) coords $n2] 1]
  set tags1 [$PT(canvas) gettags $n1]
  set type1 [lindex $tags1 [lsearch -regexp $tags1 Place|PTransition|Block]]
  set tags2 [$PT(canvas) gettags $n2]
  set type2 [lindex $tags2 [lsearch -regexp $tags2 Place|PTransition|Block]]
  set offset [$PT(net) CalcOffset $type1 $type2 [expr int($x1)] \
    [expr int($y1)] [expr int($x2)] [expr int($y2)]] 
  if {$edge == -12345} {
    eval {$PT(canvas) create line} $offset \
      {-arrow last -tags Edge -fill $PT(color.block_edge)}
  } else {
    if {[expr [$PT(net) PEdge $edge get Visibility ] != 1 && \
      $PT(InvisibleArcsOn) == 0]} {
      return
    }
    eval {$PT(canvas) coords $edge} $offset 
  }  ;# else
}  ;# SetupBlockPEdge

# -------------------------------------------------------------
# Name
#   CreateEdgeWeight
# ---------
# Parameter 
#   edge -> Name der Kante
# ---------
# Description
#   Fuer die Kante edge wird ein Kantengewicht auf dem Canvas erzeugt,
#   jedoch nur dann wenn das Gewicht != 1 ist.
proc CreateEdgeWeight {edge} {
  global PT IsAnnoFrom

  set w [$PT(net) Edge $edge get Weight]
  if {$w != 1} {
    set c [$PT(canvas) create text 0 0 -text $w \
      -anchor center -tags Weight@$edge]
    $PT(canvas) addtag EdgeAnnotation withtag $c
    $PT(canvas) addtag Annotation withtag $c
    set IsAnnoFrom($c) $edge
    SetupEdgeAnno $edge
  }  ;# if Kantengewicht ungleich 1
}  ;# CreateEdgeWeight

# -------------------------------------------------------------
# Name
#   NewNode
# ---------
# Parameter 
#   kind -> Knoten-Typ (e.g. Block, Place, Transition)
# ---------
# Description
#   Ein neuer Knoten vom Typ kind soll erzeugt werden. Die Position
#   ist noch unbekannt. Motion Event zeichnet den Knoten an der jeweiligen
#   Position, Button1 erzeugt ihn schliesslich. 
proc NewNode {kind} {
  global PT

  bind $PT(canvas) <Motion> "Draw$kind %x %y"
  bind $PT(canvas) <1> "New$kind %x %y"
}  ;# NewNode

# -------------------------------------------------------------
# Name
#   NewBlock
# ---------
# Parameter 
#   x, y -> Position
# ---------
# Description
#   Ein neuer Block wird an (x, y) erzeugt. Ruft MkBlock mit dem Namen 
#   aus dem C++-Kern auf.
proc NewBlock {x y} {
  global PT Blocks

  Grid $x $y
  set id [$PT(net) create Block]

  set Blocks($id) 0
  MkBlock $x $y $id
  set coords [$PT(canvas) coords $id]
  $PT(net) Block $id set \
    x [expr int($x / $PT(Zoom))] \
    y [expr int($y / $PT(Zoom))]
 
  UnsetAllBindings 
  EditBindings
  NetChanged
}  ;# NewBlock

# -------------------------------------------------------------
# Name
#   NewBox
# ---------
# Parameter 
#   x, y -> Position
# ---------
# Description
#   Eine neue Box wird an (x, y) erzeugt. Ruft MkBox mit dem Namen 
#   aus dem C++-Kern auf.
proc NewBox {x y} {
  global PT

  Grid $x $y
  set id [$PT(net) create Box]

  MkBox $x $y $id
  set coords [$PT(canvas) coords $id]
  $PT(net) Box $id set \
    x [expr int($x / $PT(Zoom))] \
    y [expr int($y / $PT(Zoom))]
 
  UnsetAllBindings 
  EditBindings
  UpdateBlockTreeDialog
  NetChanged
}  ;# NewBox

# -------------------------------------------------------------
# Name
#   NewPlace
# ---------
# Parameter 
#   x, y -> Position
# ---------
# Description
#   Eine neue Stelle wird an (x, y) erzeugt. Ruft MkPlace mit dem Namen 
#   aus dem C++-Kern auf.
proc NewPlace {x y} {
  global PT Id2Name

  Grid $x $y
  set id [$PT(net) create Place]
  set frame [string trim $PT(canvas) ".boxframe"]
  set frame [string trim $frame ".fenster.canvas"]
  if {$frame != "drawing"} {
    $PT(net) Place $id set Box $frame
  }
  MkPlace $x $y $id
  set coords [$PT(canvas) coords $id]
  $PT(net) Place $id set \
    x [expr int(($x + $PT(OSizeHalf)) / $PT(Zoom))] \
    y [expr int(($y + $PT(OSizeHalf)) / $PT(Zoom))]
  
  UnsetAllBindings 
  EditBindings
  NetChanged
}  ;# NewPlace

# -------------------------------------------------------------
# Name
#   NewTransition
# ---------
# Parameter 
#   x, y -> Position
# ---------
# Description
#   Eine neue Transition wird an (x, y) erzeugt. Ruft MkTransition mit 
#   dem Namen aus dem C++-Kern auf.
proc NewTransition {x y} {
  global PT

  Grid $x $y
  set id [$PT(net) create Transition]
  set frame [string trim $PT(canvas) ".boxframe"]
  set frame [string trim $frame ".fenster.canvas"]
  if {$frame != "drawing"} {
    $PT(net) Transition $id set Box $frame
  }
  MkTransition $x $y $id
  set coords [$PT(canvas) coords $id]
  $PT(net) Transition $id set \
    x [expr int(($x + $PT(OSizeHalf)) / $PT(Zoom))] \
    y [expr int(($y + $PT(OSizeHalf)) / $PT(Zoom))]
  UnsetAllBindings 
  EditBindings
  NetChanged
}  ;# NewTransition

# -------------------------------------------------------------
# Name
#   NewPTransition
# ---------
# Parameter 
#   x, y -> Position
# ---------
# Description
#   Eine neue PTransition wird an (x, y) erzeugt. Ruft MkTransition mit 
#   dem Namen aus dem C++-Kern auf.
proc NewPTransition {x y} {
  global PT

  Grid $x $y
  set id [$PT(net) create PTransition]
  set frame [string trim $PT(canvas) ".blockframe"]
  set frame [string trim $frame ".fenster.canvas"]
  if {$frame != "drawing"} {
    $PT(net) PTransition $id set Blocks $frame
  }
  MkPTransition $x $y $id
  set coords [$PT(canvas) coords $id]
  $PT(net) PTransition $id set \
    x [expr int(($x + $PT(OSizeHalf)) * $PT(Zoom))] \
    y [expr int(($y + $PT(OSizeHalf)) * $PT(Zoom))]
  UnsetAllBindings 
  EditBindings
  NetChanged
}  ;# NewPTransition

# -------------------------------------------------------------
# Name
#   NewEdge
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Eine neue Kante wird erzeugt. Der erste Knoten ist derjenige, dessen
#   Canvas-Objekt das Tag FirstNode hat, der Zielknoten ist derjenige mit
#   Tag current. 
# Todo: Farben
proc NewEdge {} {
  global PT Id2Name

  UnsetAllBindings
  set coords [$PT(canvas) coords TempEdge]
  set xm 0
  set ym 0
  $PT(canvas) delete TempEdge
  set n1 [$PT(canvas) find withtag FirstNode]
  set n2 [$PT(canvas) find withtag current]
  set id [$PT(net) create Edge $Id2Name($n1) $Id2Name($n2)]
  if {[llength [$PT(canvas) find withtag $id]] == 0} {
    MkEdge $n1 $n2 $id
    #set edge [$PT(canvas) find withtag $id]
    #set coords [$PT(canvas) coords $edge]
    #$PT(net) Edge $id set SplineX [expr int([lindex $coords 2] * $PT(Zoom))]
    #$PT(net) Edge $id set SplineY [expr int([lindex $coords 3] * $PT(Zoom))]
  }  ;# if Kante mit dieser Id noch nicht vorhanden
  $PT(canvas) itemconfigure FirstNode -fill white
  $PT(canvas) dtag FirstNode
  $PT(canvas) itemconfigure current -fill white
  UnsetAllBindings
  EditBindings
  $PT(canvas) addtag LastNew withtag $n1
  NetChanged
}  ;# NewEdge

proc NewPEdge {} {
  global PT Id2Name

  UnsetAllBindings
  set coords [$PT(canvas) coords TempEdge]
  set xm 0
  set ym 0
  $PT(canvas) delete TempEdge
  set n1 [$PT(canvas) find withtag FirstNode]
  set n2 [$PT(canvas) find withtag current]
  set id [$PT(net) create PEdge $Id2Name($n1) $Id2Name($n2)]
  if {[llength [$PT(canvas) find withtag $id]] == 0} {
    MkPEdge $n1 $n2 $id
    #set edge [$PT(canvas) find withtag $id]
    #set coords [$PT(canvas) coords $edge]
    #$PT(net) PEdge $id set SplineX [expr int([lindex $coords 2] * $PT(Zoom))]
    #$PT(net) PEdge $id set SplineY [expr int([lindex $coords 3] * $PT(Zoom))]
  }  ;# if Kante mit dieser Id noch nicht vorhanden
  $PT(canvas) itemconfigure FirstNode -fill white
  $PT(canvas) dtag FirstNode
  $PT(canvas) itemconfigure current -fill white
  UnsetAllBindings
  EditBindings
  $PT(canvas) addtag LastNew withtag $n1
  NetChanged
}  ;# NewPEdge

# -------------------------------------------------------------
# Name
#   AddSelection
# ---------
# Parameter 
#   x, y -> Position
# ---------
# Description
#   Ein Objekt wird zur Selektion addiert, d.h. es bekommt Tag selected.
#   Nur moeglich bei Knoten und Kanten.
#   Wird z.B. bei Shift-B1 Event gerufen.
# Todo: Farben
proc AddSelection {x y} {
  # Todo: hat evtl. mit dem Moven zu tun
  global PT

  set tags [$PT(canvas) gettags current]
  if {[lsearch $tags Deletable] > -1} {
      # Markieren
    $PT(canvas) addtag selected withtag current
    $PT(canvas) itemconfigure selected -fill blue
  }  ;# if selektierbares Objekt
  set PT(x1) $x
  set PT(y1) $y
}  ;# AddSelection
   
# -------------------------------------------------------------
# Name
#   Select
# ---------
# Parameter 
#   x, y -> Position
# ---------
# Description
#   Button wurde auf Knoten oder Kante gedrueckt. Alles andere wird 
#   deselektiert und das aktuelle Objekt selektiert.
# Todo: Farben
proc Select {x y} {
  # Todo: hat evtl. mit dem moven was zu tun
  global PT Id2Name

  DeselectAll
  $PT(canvas) dtag LastNew
  set tags [$PT(canvas) gettags current]
#TODO: select place instead of token
#  if {[lsearch $tags Token] > -1} {
#    $PT(canvas) addtag current below current
#    set tags [$PT(canvas) gettags current]
#  }
  set coords [$PT(canvas) coords current]
  if {[lsearch $tags Edge] > -1} {
    if {[llength $coords] > 4} {
      $PT(canvas) addtag point withtag \
      [$PT(canvas) create rect [expr $x - 3] [expr $y - 3] \
                               [expr $x + 3] [expr $y + 3] -outline black -fill red]
#      $PT(canvas) addtag point withtag \
#      [$PT(canvas) create rect [expr int([lindex $coords 2] - 3)] \
#                               [expr int([lindex $coords 3] - 3)] \
#                               [expr int([lindex $coords 2] + 3)] \
#                               [expr int([lindex $coords 3] + 3)] -outline black -fill red]
#      $PT(canvas) bind point <B1-Motion> {$PT(canvas) coords point [expr [$PT(canvas) canvasx %x] - 3] \
#                                                                   [expr [$PT(canvas) canvasy %y] - 3] \
#                                                                   [expr [$PT(canvas) canvasx %x] + 3] \
#                                                                   [expr [$PT(canvas) canvasy %y] + 3]
#                                                                   MoveEdge %x %y}
#      $PT(canvas) bind point <ButtonRelease-1> {
#        set current [$PT(canvas) find withtag selected]
#        set edge $Id2Name($current)
#        if {[lsearch -regexp $edge P] > -1} {
#          StopMovePEdge
#        } else {
#          StopMoveEdge
#        }
#      }
      $PT(canvas) bind point <Button-2> {
        set current [$PT(canvas) find withtag selected]
        set edge $Id2Name($current)
        if {[lsearch -regexp $edge P] > -1} {
          $PT(net) PEdge $edge set SplineX 0
          $PT(net) PEdge $edge set SplineY 0
          SetupPEdge [$PT(net) PEdge $edge get Src] [$PT(net) PEdge $edge get Dest] $edge
        } else {
          $PT(net) Edge $edge set SplineX 0
          $PT(net) Edge $edge set SplineY 0
          SetupEdge [$PT(net) Edge $edge get Src] [$PT(net) Edge $edge get Dest] $edge
          SetupEdgeAnno $edge
        }
      }
    }
  }
  #if {[lsearch $tags Deletable] > -1} {
      # Markieren
    $PT(canvas) addtag selected withtag current
    if {[lsearch -regexp [$PT(canvas) gettags current] Block|Box]} {
      set PT(OldColor) [lindex [$PT(canvas) itemconfigure current -fill] 4]
      $PT(canvas) itemconfigure selected -fill blue
    } else {
      if {[lsearch -regexp [$PT(canvas) gettags current] Block] > -1} {
        block$Id2Name([$PT(canvas) find withtag current]) configure -background blue
        $PT(canvas) itemconfigure current -image block$Id2Name([$PT(canvas) find withtag current])
      } else {
        catch {
          box$Id2Name([$PT(canvas) find withtag current]) configure -background blue
          $PT(canvas) itemconfigure current -image box$Id2Name([$PT(canvas) find withtag current])
        }
      }
    }
  #}  ;# if selektierbares Objekt
  Grid $x $y
  set PT(x1) $x
  set PT(y1) $y
  set PT(BeginX) $x
  set PT(BeginY) $y
}  ;# Select

# -------------------------------------------------------------
# Name
#   DeselectAll
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Alle selektierten Elemente werden deselektiert. Dabei werden
#   Exit und Entry Stellen wieder richtig eingefaerbt. Ein evtl.
#   bestehender Bereich wird wieder geloescht.
# Todo: Farben
proc DeselectAll {} {
  global PT Id2Name

  #$PT(canvas) delete point
  $PT(canvas) delete [$PT(canvas) find withtag point]
  $PT(canvas) itemconfigure Title -fill black
  foreach item [$PT(canvas) find withtag selected] {
    set tags [$PT(canvas) gettags $item]
    if {[lsearch $tags Node] > -1} {
      if {[lsearch $tags Place] > -1} {
        UpdatePlaceState $Id2Name($item)
      } else {
        if {[lsearch $tags Transition] > -1} {
          UpdateTransitionState $Id2Name($item)
        } elseif {[lsearch $tags PTransition] > -1} {
          UpdatePTransitionState $Id2Name($item)
        } else {
          if {[lsearch $tags Block] == -1} {
            if {[lsearch $tags Box] == -1} {
              $PT(canvas) itemconfigure $item -fill white
            } else {
              catch {
                box$Id2Name($item) configure -background grey
                $PT(canvas) itemconfigure $item -image box$Id2Name($item)
              }
            }
          } else {
            block$Id2Name($item) configure -background white
            $PT(canvas) itemconfigure $item -image block$Id2Name($item)
          }
        }  ;# else, keine (P)Transition
      }  ;# else, keine Stelle
    } else {  ;# kein Knoten
      set tags [$PT(canvas) gettags $item]
      if {[lsearch $tags Deletable] > -1} {
        $PT(canvas) itemconfigure $item -fill black
      } else {
        if {[lsearch $tags EdgeAnnotation] > -1 || [lsearch $tags Annotation] > -1 || [lsearch $tags Token] > -1} {
          $PT(canvas) itemconfigure $item -fill black
        } else {
          if {[lsearch -regexp $tags "Area*"] < 0} {
            $PT(canvas) itemconfigure $item -fill $PT(color.phantom_transition)
          }
        }
      }
    }  ;# else, kein Knoten
  }  ;# foreach selektiertes Objekt

  $PT(canvas) dtag selected
  $PT(canvas) dtag Area$PT(areas)
  $PT(canvas) delete Area$PT(areas)
}  ;# DeselectAll
    
# -------------------------------------------------------------
# Name
#   Move
# ---------
# Parameter 
#   x, y -> Position
# ---------
# Description
#   Alle Elemente mit dem Tag selected werden nach (x, y) bewegt,
#   solange B1-event gilt. Wenn B1-Release dann StopMove.
proc Move {x y} {
  global Id2Name PT

    # Alle Items finden die selektiert sind.
  set item_new [$PT(canvas) find withtag LastNew]
  set items_to_move [$PT(canvas) find withtag selected]
  foreach one $items_to_move {
    if {[expr $item_new - 0] != [expr $one - 0] } {
      Grid $x $y
      $PT(canvas) move selected [expr int($x - $PT(x1))] [expr int($y - $PT(y1))]
      set coords [$PT(canvas) coords $Id2Name($one)]
      set ID [GetType $Id2Name($one)]
    
      if {$ID == "Block"} {
        set xtemp [expr int([lindex $coords 0]) + int($PT(ObjectSize)) + int($x) - int($PT(x1))]
        set ytemp [expr int([lindex $coords 1]) + int($PT(ObjectSize)) - 6 + int($y) - int($PT(y1))]
      } else {
        set xtemp [expr int([lindex $coords 0]) + int($PT(OSizeHalf)) + int($x) - $PT(x1)]
        set ytemp [expr int([lindex $coords 1]) + int($PT(OSizeHalf)) + int($y) - $PT(y1)]
      }
      set xtemp [expr int($xtemp - 0)]
      set ytemp [expr int($ytemp - 0)]     
      $PT(net) $ID $Id2Name($one) set \
          x $xtemp \
          y $ytemp
      set PT(x1) int($x)
      set PT(y1) int($y)
      set ie [$PT(net) $ID $Id2Name($one) get InEdges]
      foreach e $ie {
        if {[$PT(net) Edge $e get Visibility]} {
          if {$PT(MoveEdgesWith)} {
            if {[$PT(canvas) find withtag [$PT(net) Edge $e get Src]] != "" && \
              [$PT(canvas) find withtag [$PT(net) Edge $e get Dest]] != ""} {
              SetupEdge [$PT(net) Edge $e get Src] [$PT(net) Edge $e get Dest] $e
            }
          }
        }
      }
      set oe [$PT(net) $ID $Id2Name($one) get OutEdges]
      foreach o $oe {
        if {[$PT(net) Edge $o get Visibility]} {
          if {$PT(MoveEdgesWith)} {
            if {[$PT(canvas) find withtag [$PT(net) Edge $o get Src]] != "" && \
              [$PT(canvas) find withtag [$PT(net) Edge $o get Dest]] != ""} {
              SetupEdge [$PT(net) Edge $o get Src] [$PT(net) Edge $o get Dest] $o
            }
          }
        }
      }
      #$PT(canvas) bind Node <ButtonRelease-1> {StopMove}
      NetChanged
    }
  }
}  ;# Move

# -------------------------------------------------------------
# Name
#   MoveEdge
# ---------
# Parameter 
#   x, y -> Position
# ---------
# Description
#   Der Punkt (x, y) ist der Mittelpunkt des Splines.
#   Die mit selected markierte Kante wird veraendert
proc MoveEdge {x y} {
   global PT Id2Name

  Grid $x $y
  set edge [$PT(canvas) find withtag selected]
  set coords [$PT(canvas) coords $edge]
  set xm [expr int(([lindex $coords 0] + [lindex $coords 4]) / 2)]
  set ym [expr int(([lindex $coords 1] + [lindex $coords 5]) / 2)]
  set xdist [expr int($xm - $x)]
  set ydist [expr int($ym - $y)]
  set xneu [expr int($xm - ($xdist * 2))]
  set yneu [expr int($ym - ($ydist * 2))]
  set xneu2 [expr int(($xm - ($xdist * 2)) / $PT(Zoom))]
  set yneu2 [expr int(($ym - ($ydist * 2)) / $PT(Zoom))]
  $PT(canvas) coords $edge \
        [lindex $coords 0] [lindex $coords 1] \
        $xneu2 $yneu2 \
        [lindex $coords 4] [lindex $coords 5]
  set tags [$PT(canvas) gettags $edge]
  if {[lsearch $tags Deletable] > -1} {
    $PT(canvas) bind Edge <ButtonRelease-1> {StopMoveEdge}
    $PT(net) Edge $Id2Name($edge) set SplineX $xneu2
    $PT(net) Edge $Id2Name($edge) set SplineY $yneu2
    set Src [$PT(net) Edge $Id2Name($edge) get Src]
    set Dest [$PT(net) Edge $Id2Name($edge) get Dest]
    SetupEdge $Src $Dest $Id2Name($edge)
  } else {
    $PT(canvas) bind Edge <ButtonRelease-1> {StopMovePEdge}
    $PT(net) PEdge $Id2Name($edge) set SplineX $x
    $PT(net) PEdge $Id2Name($edge) set SplineY $y
    set Src [$PT(net) PEdge $Id2Name($edge) get Src]
    set Dest [$PT(net) PEdge $Id2Name($edge) get Dest]
    SetupPEdge $Src $Dest $Id2Name($edge)
  }
}

# -------------------------------------------------------------
# Name
#   MovePEdge
# ---------
# Parameter 
#   x, y -> Position
# ---------
# Description
#   Der Punkt (x, y) ist der Mittelpunkt des Splines.
#   Die mit selected markierte Kante wird veraendert
proc MovePEdge {x y} {
   global PT Id2Name

  Grid $x $y
  set edge [$PT(canvas) find withtag selected]
  set coords [$PT(canvas) coords $edge]
  $PT(canvas) bind Edge <ButtonRelease-1> {StopMovePEdge}
  $PT(canvas) coords $edge \
        [lindex $coords 0] [lindex $coords 1] \
        $x $y \
        [lindex $coords 4] [lindex $coords 5]
  $PT(net) PEdge $Id2Name($edge) set SplineX $x
  $PT(net) PEdge $Id2Name($edge) set SplineY $y
}


# -------------------------------------------------------------
# Name
#   StopMoveEdge
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#  Die mit selected markierte Kante wird in ihre endg"ultige Form
#  gebracht. Anschlie"send werden die Koordinaten des Splinemittelpunktes
#  an den C++-Kern "ubermittelt
proc StopMoveEdge {} {
   global PT Id2Name
 
   UnsetAllBindings
   EditBindings
   set edges [$PT(canvas) find withtag selected]
   foreach edge $edges {
     set edge $Id2Name($edge)
     set coords [$PT(canvas) coords $edge]
     #set x [expr [lindex $coords 2] / $PT(Zoom)]
     #set y [expr [lindex $coords 3] / $PT(Zoom)]
     set x [lindex $coords 2]
     set y [lindex $coords 3]
     #$PT(net) Edge $edge set SplineX [expr int($x)]
     #$PT(net) Edge $edge set SplineY [expr int($y)]
     #SetupEdge [$PT(net) Edge $edge get Src] [$PT(net) Edge $edge get Dest] $edge
     SetupEdgeAnno $edge
   }
   #DeselectAll
   #$PT(canvas) bind Node <B1-Release> {}
   NetChanged
}

# -------------------------------------------------------------
# Name
#   StopMovePEdge
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#  Die mit selected markierte Kante wird in ihre endg"ultige Form
#  gebracht. Anschlie"send werden die Koordinaten des Splinemittelpunktes
#  an den C++-Kern "ubermittelt
proc StopMovePEdge {} {
   global PT Id2Name

   UnsetAllBindings
   EditBindings
   set edges [$PT(canvas) find withtag selected]
   foreach edge $edges {
     set coords [$PT(canvas) coords $edge]
     set x [lindex $coords 2]
     set y [lindex $coords 3]
     $PT(net) PEdge $Id2Name($edge) set SplineX [expr int($x / $PT(Zoom))]
     $PT(net) PEdge $Id2Name($edge) set SplineY [expr int($y / $PT(Zoom))]
     SetupPEdge [$PT(net) PEdge $Id2Name($edge) get Src] [$PT(net) PEdge $Id2Name($edge) get Dest] $Id2Name($edge)
   }
   DeselectAll
   #$PT(canvas) bind Node <B1-Release> {}
   NetChanged
}


# -------------------------------------------------------------
# Name
#   StopMove
# ---------
# Parameter 
#   x, y -> Position
# ---------
# Description
#   Der Punkt (x, y) ist der Endpunkt der Verschiebung. Alle
#   Elemente mit Tag selected werden relativ dazu verschoben.
#   Fuer die Knoten werden Ein- und Ausgangskanten wieder in Form
#   gebracht und die Annotationen mitverschoben. Fuer Stellen
#   werden die Tokens ebenfalls verschoben.
# Todo: Blockkanten
proc StopMove {x y} {
  global PT Id2Name

  set items_to_move [$PT(canvas) find withtag selected]
  foreach item $items_to_move {
    set tags [$PT(canvas) gettags $item]
    set index [lsearch -regexp $tags Block|Box|Place|Transition|PTransition]
    if {$index > -1} {
      set type [lindex $tags $index]
      set coords [$PT(canvas) coords $item]
      set area [$PT(canvas) find withtag Area]
      if {$type != "Block"} {
        if {$type != "Box"} {
          $PT(net) $type $Id2Name($item) set x [expr int(([lindex $coords 0] + $PT(ObjectSize)) / $PT(Zoom))]
          $PT(net) $type $Id2Name($item) set y [expr int(([lindex $coords 1] + $PT(ObjectSize)) / $PT(Zoom))]
        } else {
          $PT(net) $type $Id2Name($item) set x [expr int(([lindex $coords 0]) / $PT(Zoom))]
          $PT(net) $type $Id2Name($item) set y [expr int(([lindex $coords 1]) / $PT(Zoom))]
          if {$PT(NodesWithBox)} {
            $PT(net) MoveBox $Id2Name($item) [expr int($x - $PT(BeginX))] [expr int($y - $PT(BeginY))]
          }
        }
      } else {
        $PT(net) $type $Id2Name($item) set x [expr int(([lindex $coords 0]) / $PT(Zoom))]
        $PT(net) $type $Id2Name($item) set y [expr int(([lindex $coords 1]) / $PT(Zoom))]
        if {$PT(NodesWithBlock)} {
          $PT(net) MoveBlock $Id2Name($item) [expr int($x - $PT(BeginX))] [expr int($y - $PT(BeginY))]
        }
      }
      foreach edge [$PT(net) $type $Id2Name($item) get InEdges] {
        if {[$PT(net) Edge $edge get BlockVis] == "exposed"} {
          SetupEdge [$PT(net) Edge $edge get Src] \
            [$PT(net) Edge $edge get Dest] $edge
          SetupEdgeAnno $edge
        } else {
          if {$type != "Block"} {
            SetupBlockEdges [$PT(net) Edge $edge get BlockSrc] \
              $Id2Name($item) $edge
          } else {
            SetupBlockEdges [$PT(net) Edge $edge get Src] $Id2Name($item) $edge
          }  ;# else, Blockkante, bewegtes Objekt ist ein Block
        }  ;# else, evtl. Blockkante
      }  ;# foreach Kante
      foreach edge [$PT(net) $type $Id2Name($item) get OutEdges] {
        if {[$PT(net) Edge $edge get BlockVis] == "exposed"} {
          SetupEdge [$PT(net) Edge $edge get Src] \
            [$PT(net) Edge $edge get Dest] $edge
          SetupEdgeAnno $edge
        } else {
          if {$type != "Block"} {
            SetupBlockEdges $Id2Name($item) \
              [$PT(net) Edge $edge get BlockDest] $edge
          } else {
            SetupBlockEdges $Id2Name($item) [$PT(net) Edge $edge get Dest] $edge
          }  ;# else, Blockkante, bewegtes Objekt ist ein Block
        }  ;# else, evtl. Blockkante
      }  ;# foreach Kante
      foreach edge [$PT(net) $type $Id2Name($item) get InPEdges] {
        if {[$PT(net) PEdge $edge get BlockVis] == "exposed"} {
          SetupPEdge [$PT(net) PEdge $edge get Src] \
            [$PT(net) PEdge $edge get Dest] $edge
        }  ;# if Phantomkante sichtbar
      }  ;# foreach Phantomkante
      foreach edge [$PT(net) $type $Id2Name($item) get OutPEdges] {
        if {[$PT(net) PEdge $edge get BlockVis] == "exposed"} {
          SetupPEdge [$PT(net) PEdge $edge get Src] \
            [$PT(net) PEdge $edge get Dest] $edge
        }  ;# if Phantomkante sichtbar
      }  ;# foreach Phantomkante
      if {$type == "Place"} {
        LayoutTokens $Id2Name($item)
      }
      LayoutAnnotations $Id2Name($item) $type
    } else { #no node has been moved
      set index  [lsearch -regexp $tags Edge]
      set index2 [lsearch -regexp $tags EdgeAnnotation]
      if {$index > -1 && $index2 == -1} {
        if {[lsearch -regexp $tags PA] > -1} {
          $PT(net) PEdge $Id2Name($item) set SplineX [expr int([$PT(net) PEdge $Id2Name($item) get SplineX] / $PT(Zoom))]
          $PT(net) PEdge $Id2Name($item) set SplineY [expr int([$PT(net) PEdge $Id2Name($item) get SplineY] / $PT(Zoom))]
        } else {
          $PT(net) Edge $Id2Name($item) set SplineX [expr int([$PT(net) Edge $Id2Name($item) get SplineX] / $PT(Zoom))]
          $PT(net) Edge $Id2Name($item) set SplineY [expr int([$PT(net) Edge $Id2Name($item) get SplineY] / $PT(Zoom))]
        }
      }
    }
  }  ;# foreach zu bewegendes Objekt
  NetChanged

}  ;# StopMove

# -------------------------------------------------------------
# Name
#   ShowPhantomRelation
# ------------------
# Parameter
#   trans_type - Typ der Transition (Transition, PTransition)
#   name - Name der Transition
# ------------------
# Description
#   Highlight phantom transitions of name, if available.
proc ShowPhantomRelation {trans_type name} {
  global PT

    # die anderen, zugehoerigen Transitionen holen
  switch $trans_type {
    Transition {
      if {[$PT(net) Transition $name get Channel0Sync] == "no"} {
        return
      }  ;# if keine Synchronisationstransition
      set other_phantoms [$PT(net) Transition $name get Phantoms]
      set trs $name
    }
    PTransition {
      set trs [$PT(net) PTransition $name get Synchrons]
      foreach t $trs {
        set other_phantoms [$PT(net) Transition $t get Phantoms]
      }  ;# foreach SyncTrans
      lappend trs $name
    }
  }  ;# switch
    # die ganzen Transitionen raustellen
  foreach t $trs {
    if {[$PT(net) Transition $t get BlockVis] == "exposed"} {
      $PT(canvas) itemconfigure $t -width 3
    }
  }  ;# foreach Transition
  foreach t $other_phantoms {
    if {[$PT(net) PTransition $t get BlockVis] == "exposed"} {
      $PT(canvas) itemconfigure $t -width 3
    }
  }  ;# foreach Transition
  for {set i 0} {$i < 3} {incr i 1} {
    foreach c {green cyan blue yellow orange red black} {
      foreach t $trs {
        if {[$PT(net) Transition $t get BlockVis] == "exposed"} {
          $PT(canvas) itemconfigure $t -outline $c
	}
      }  ;# foreach Transition
      foreach t $other_phantoms {
        if {[$PT(net) PTransition $t get BlockVis] == "exposed"} {
          $PT(canvas) itemconfigure $t -outline $c
	}
      }  ;# foreach Transition
      update
      after [expr int($PT(Speed) / 10)]
    }  ;# foreach Farbe
  }  ;# for i
  foreach t $trs {
    if {[$PT(net) Transition $t get BlockVis] == "exposed"} {
      $PT(canvas) itemconfigure $t -width 1
    }
  }  ;# foreach Transition
  foreach t $other_phantoms {
    if {[$PT(net) PTransition $t get BlockVis] == "exposed"} {
      $PT(canvas) itemconfigure $t -width 1
    }
  }  ;# foreach Transition
}  ;# ShowPhantomRelation

# -------------------------------------------------------------
# Name
#   AdjustCanvasScrollRegion
# ------------------
# Parameter
#   Nothing
# ------------------
# Description
#   Die Groesse der Zeichenflaeche (Canvas) anpassen.
proc AdjustCanvasScrollRegion {} {
  global PT

  set gridsize 50.0
  set box [$PT(canvas) bbox all]
  set x1 0.0
  set y1 0.0
  set x2 [lindex $box 2]
  set y2 [lindex $box 3]
  if {"$x2" != ""} {
    set x2 [expr (round($x2 / $gridsize) + 1.0) * $gridsize]
    if {$x2 < 6000.0} {
      set x2 6000.0
    }  ;# if x2 zu klein
  } else {
    set x2 6000.0
  }  ;# else, x2 nicht definiert
  if {"$y2" != ""} {
    set y2 [expr (round($y2 / $gridsize) + 1) * $gridsize]
    if {$y2 < 6000.0} {
      set y2 6000.0
    }  ;# if y2 zu klein
  } else {
    set y2 6000.0
  }  ;# else, y2 nicht definiert
  $PT(canvas) configure -scrollregion "$x1 $y1 $x2 $y2"
}  ;# AdjustCanvasScrollRegion

# -------------------------------------------------------------
# Name
#   Repaint
# ---------
# Parameter 
#   nothing
# ---------
# Description
#   Die Hierarchie der Bloecke wird neu berechnet und die sich
#   daraus ergebenden Sichtbarkeiten bzgl. der Bloecke auf der
#   Canvas gezeichnet. Das kann dann schon mal ein bisschen dauern.
proc Repaint {} {
  global PT Id2Name

  if {$PT(canvas) != ".drawing.canvas"} {
    return
  }

    # Blockhierarchie aufbauen
  $PT(net) RecalcRootBlocks
  $PT(net) RecalcBlockVis

  set visblocks [$PT(net) get VisBlocks]
  set visboxes  [$PT(net) get VisBoxes]
  set visplaces [$PT(net) get VisPlaces]
  set vistransitions [$PT(net) get VisTransitions]
  set visptransitions [$PT(net) get VisPTransitions]
  $PT(canvas) delete all
  DrawGrid
  $PT(canvas) xview moveto 0.0
  $PT(canvas) yview moveto 0.0

  $PT(ModeW) configure -text "Create Transitions ..."
  update
  foreach t $vistransitions {
    MkTransition [expr [$PT(net) Transition $t get x] * $PT(Zoom) - $PT(OSizeHalf)] \
      [expr [$PT(net) Transition $t get y] * $PT(Zoom) - $PT(OSizeHalf)] $t
  }  ;# foreach Transition im Netz

  $PT(ModeW) configure -text "Create Phantom Transitions ..."
  update
  foreach t $visptransitions {
    MkPTransition [expr [$PT(net) PTransition $t get x] * $PT(Zoom) - $PT(OSizeHalf)] \
      [expr [$PT(net) PTransition $t get y] * $PT(Zoom) - $PT(OSizeHalf)] $t
  }  ;# foreach Phantomtransition im Netz

  $PT(ModeW) configure -text "Create Places ..."
  update
  foreach p $visplaces {
    MkPlace [expr [$PT(net) Place $p get x] * $PT(Zoom) - $PT(OSizeHalf)] \
      [expr [$PT(net) Place $p get y] * $PT(Zoom) - $PT(OSizeHalf)] $p
  }  ;# foreach Stelle im Netz

  $PT(ModeW) configure -text "Create Blocks ..."
  update
  foreach b $visblocks {
    MkBlock [expr [$PT(net) Block $b get x] * $PT(Zoom)] \
      [expr [$PT(net) Block $b get y] * $PT(Zoom)] $b
  }  ;# foreach Block im Netz

  $PT(ModeW) configure -text "Create Boxes ..."
  update
  foreach b $visboxes {
    MkBox [expr [$PT(net) Box $b get x] * $PT(Zoom)] \
      [expr [$PT(net) Box $b get y] * $PT(Zoom)] $b
  }  ;# foreach Box im Netz

  $PT(ModeW) configure -text "Create Arcs & Tokens ...(P)"
  update
  set factor [expr $PT(Zoom) / $PT(OldZoom)]
    # weiter mit Kanten und Markierungen
  foreach p $visplaces {
      # Kanten erstellen
    set in [$PT(net) Place $p get InEdges]
    foreach ie $in {
      if {[$PT(net) Edge $ie get BlockVis] == "exposed"} {
        if {[$PT(net) Edge $ie get Visibility] != 1} {
          if $PT(InvisibleArcsOn) {
            MkEdge [$PT(net) Edge $ie get Src] $p $ie
          }  ;# if unsichtbare Kanten doch sichtbar
        } else {
          MkEdge [$PT(net) Edge $ie get Src] $p $ie
        }  ;# else, auf jeden Fall anzeigen
      } else {
        if {[$PT(net) Edge $ie get Visibility] != 1} {
          if $PT(InvisibleArcsOn) {
            MkBlockEdge [$PT(net) Edge $ie get BlockSrc] $p $ie
          }  ;# if unsichtbare Kanten doch sichtbar
        } else {
          MkBlockEdge [$PT(net) Edge $ie get BlockSrc] $p $ie
        }  ;# else, auf jeden Fall anzeigen
      }  ;# else nicht sichtbar oder Blockkante
    }  ;# foreach eingehende Kante
    set out [$PT(net) Place $p get OutEdges]
    foreach oe $out {
      if {[$PT(net) Edge $oe get BlockVis] == "exposed"} {
        if {[$PT(net) Edge $oe get Visibility] != 1} {
          if $PT(InvisibleArcsOn) {
            MkEdge $p [$PT(net) Edge $oe get Dest] $oe
          }  ;# if unsichtbare Kanten doch sichtbar
        } else { 
          MkEdge $p [$PT(net) Edge $oe get Dest] $oe
        }  ;# else, auf jeden Fall anzeigen
      } else {
        if {[$PT(net) Edge $oe get Visibility] != 1} {
          if $PT(InvisibleArcsOn) {
            MkBlockEdge $p [$PT(net) Edge $oe get BlockDest] $oe
          }  ;# if unsichtbare Kanten doch sichtbar
        } else { 
          MkBlockEdge $p [$PT(net) Edge $oe get BlockDest] $oe
        }  ;# else, auf jeden Fall anzeigen
      }  ;# else nicht sichtbar oder Blockkante
    }  ;# foreach ausgehende Kante
      # Markierung der Stelle
    LayoutTokens $p
  }  ;# foreach sichtbare Stelle im Netz
  $PT(ModeW) configure -text "Create Arcs & Tokens ...(T)"
  update
  foreach t $vistransitions {
    set inedges [$PT(net) Transition $t get InEdges]
    foreach ie $inedges {
      if {[$PT(net) Edge $ie get BlockVis] == "hidden"} {
        if {[$PT(net) Edge $ie get Visibility] != 1} {
          if $PT(InvisibleArcsOn) {
            MkBlockEdge [$PT(net) Place [$PT(net) Edge $ie get Src] get Blocks] $t $ie
          }
        } else {
          MkBlockEdge [$PT(net) Place [$PT(net) Edge $ie get Src] get Blocks] $t $ie
        }
      }
    }
    set outedges [$PT(net) Transition $t get OutEdges]
    foreach oe $outedges {
      if {[$PT(net) Edge $oe get BlockVis] == "hidden"} {
        if {[$PT(net) Edge $oe get Visibility] != 1} {
          if $PT(InvisibleArcsOn) {
            MkBlockEdge $t [$PT(net) Place [$PT(net) Edge $oe get Dest] get Blocks] $oe
          }
        } else {
          MkBlockEdge $t [$PT(net) Place [$PT(net) Edge $oe get Dest] get Blocks] $oe
        }
      }
    }
  }
    # Kanten von SyncTrans zu Bloecken
  $PT(ModeW) configure -text "Create Arcs & Tokens ...(ST)"
  update
  if {$PT(SyncTransSwitch) == 1} {
    foreach t $vistransitions {
      if {[$PT(net) Transition $t get Channel0Sync] == "yes"} {
        set inedges [$PT(net) Transition $t get InEdges]
        foreach ie $inedges {
          if {[$PT(net) Edge $ie get BlockVis] == "hidden"} {
            MkBlockEdge [$PT(net) Edge $ie get BlockSrc] $t $ie
          }  ;# if eigentlich nicht sichtbar
        }  ;# foreach Eingangskante
        set outedges [$PT(net) Transition $t get OutEdges]
        foreach oe $outedges {
          if {[$PT(net) Edge $oe get BlockVis] == "hidden"} {
            MkBlockEdge $t [$PT(net) Edge $oe get BlockDest] $oe
          }  ;# if eigentlich nicht sichtbar
        }  ;# foreach Ausgangskante
      }  ;# if eine SyncTrans liegt vor
    }  ;# foreach t ueber die sichtbaren Transitionen
  }  ;# if SyncTrans anzeigen

  $PT(ModeW) configure -text "Create Phantom Arcs ..."
  update
    # weiter mit Phantomkanten
  set places [$PT(net) get Places]
  foreach p $places {
    if {[$PT(net) Place $p get BlockVis] == "exposed"} {
        # Phantomkanten
      set in [$PT(net) Place $p get InPEdges]
      foreach ie $in {
        if {[$PT(net) PEdge $ie get BlockVis] == "exposed"} {
          if {[$PT(net) PEdge $ie get Visibility] != 1} {
            if $PT(InvisibleArcsOn) {
              MkPEdge [$PT(net) PEdge $ie get Src] $p $ie
            }  ;# if unsichtbare Phantomkanten doch sichtbar
          } else {
            MkPEdge [$PT(net) PEdge $ie get Src] $p $ie
          }  ;# else, auf jeden Fall anzeigen
        }  ;# sichtbare Phantomkante
      }  ;# foreach eingehende Phantomkante
      set out [$PT(net) Place $p get OutPEdges]
      foreach oe $out {
        if {[$PT(net) PEdge $oe get BlockVis] == "exposed"} {
          if {[$PT(net) PEdge $oe get Visibility] != 1} {
            if $PT(InvisibleArcsOn) {
              MkPEdge $p [$PT(net) PEdge $oe get Dest] $oe
            }  ;# if unsichtbare Phantomkanten doch sichtbar
          } else { 
            MkPEdge $p [$PT(net) PEdge $oe get Dest] $oe
          }  ;# else, auf jeden Fall anzeigen
        }  ;# if sichtbare Phantomkante
      }  ;# foreach ausgehende Phantomkante
    }  ;# if Stelle sichtbar
  }  ;# foreach Stelle im Netz

  UpdateBlockTreeDialog

  AdjustCanvasScrollRegion

  $PT(ModeW) configure -text "Edit Mode"
  DisplayTitles
}  ;# Repaint


# ------------------------------------------------------------
# Name
#   SaveModule
# ------------
# Parameter
#   filename
# ------------
# Description
#   Eine Netzdatei soll aus dem bestehenden Netz abgespeichert
#   werden.
# ------------------------------------------------------------
proc SaveModule {filename} {
  global PT Id2Name

  if {$filename != ""} {
    Copy
    $PT(net) SaveModul $filename
  }
}  ;# SaveModule


# ------------------------------------------------------------
# Name
#   LoadModule
# ------------
# Parameter
#   filename
# ------------
# Description
#   Eine Netzdatei soll geladen und in das bestehende Netz
#   eingefuegt werden.
# ------------------------------------------------------------
proc LoadModule {filename} {
  global PT Id2Name Blocks

  if {[file exists $filename]} {
    $PT(net) LoadModul $filename
    set selected [$PT(net) Selected]
    set node_list [split $selected ","]

    set l [llength $node_list]
    set node_list [lrange $node_list 0 [expr $l - 2]]
    foreach t $node_list {
      if {[$PT(net) Exists Transition $t] > 0} {
        if {[$PT(net) Transition $t get BlockVis] == "exposed"} {
          MkTransition [expr [$PT(net) Transition $t get x] * $PT(Zoom)] \
            [expr [$PT(net) Transition $t get y] * $PT(Zoom)] $t
        }
        $PT(net) Transition $t set \
          x [expr [$PT(net) Transition $t get x] + int($PT(OSizeHalf))] \
          y [expr [$PT(net) Transition $t get y] + int($PT(OSizeHalf))]
      }
      if {[$PT(net) Exists PTransition $t] > 0} {
        if {[$PT(net) PTransition $t get BlockVis] == "exposed"} {
          MkPTransition [expr [$PT(net) PTransition $t get x] * $PT(Zoom)] \
            [expr [$PT(net) PTransition $t get y] * $PT(Zoom)] $t
        }
        $PT(net) PTransition $t set \
          x [expr [$PT(net) PTransition $t get x] + int($PT(OSizeHalf))] \
          y [expr [$PT(net) PTransition $t get y] + int($PT(OSizeHalf))]
      }
      if {[$PT(net) Exists Place $t] > 0} {
        if {[$PT(net) Place $t get BlockVis] == "exposed"} {
          MkPlace [expr [$PT(net) Place $t get x] * $PT(Zoom)] \
            [expr [$PT(net) Place $t get y] * $PT(Zoom)] $t
          LayoutTokens $t
        }
        $PT(net) Place $t set \
          x [expr [$PT(net) Place $t get x] + int($PT(OSizeHalf))] \
          y [expr [$PT(net) Place $t get y] + int($PT(OSizeHalf))]
      }
      if {[$PT(net) Exists Block $t] > 0} {
        if {[$PT(net) Block $t get BlockVis] == "exposed"} {
          MkBlock [expr [$PT(net) Block $t get x] * $PT(Zoom)] \
            [expr [$PT(net) Block $t get y] * $PT(Zoom)] $t
          set Blocks($t) 0
        }
      }
    }
    DeselectAll
    $PT(net) RecalcBlockVis
    Repaint
    set MinX 10000
    set MinY 10000
    set MaxX 0
    set MaxY 0
    set box ""
    foreach n $node_list {
      set box [$PT(net) [GetType $n] $n get Box]
      set node [$PT(canvas) find withtag $n]
      if {$node != ""} {
        set cor [$PT(canvas) coords $node]
        if {[lindex $cor 0] < $MinX} {
          set MinX [lindex $cor 0]
        }
        if {[lindex $cor 1] < $MinY} {
          set MinY [lindex $cor 1]
        }
        if {[lindex $cor 0] > $MaxX} {
          set MaxX [lindex $cor 0]
        }
        if {[lindex $cor 1] > $MaxY} {
          set MaxY [lindex $cor 1]
        }
        $PT(canvas) addtag selected withtag $node
      }
    }
    if {$box != ""} {
      $PT(net) Box $box set x [expr int(($MaxX + $MinX) / 2 + $PT(OSizeHalf))]
      $PT(net) Box $box set y [expr int(($MaxY + $MinY) / 2 + $PT(OSizeHalf))]
    }
#    $PT(net) RecalcBlockVis

#    foreach p $node_list {
        # Kanten erstellen
#      if {[$PT(net) Exists Transition $p] > 0} {
#        set in [$PT(net) Transition $p get InEdges]
#        foreach ie $in {
#          if {[$PT(net) Edge $ie get BlockVis] == "exposed"} {
#            if {[$PT(net) Edge $ie get Visibility] != 1} {
#              if $PT(InvisibleArcsOn) {
#                MkEdge [$PT(net) Edge $ie get Src] $p $ie
#              }  ;# if unsichtbare Kanten doch sichtbar
#            } else {
#              MkEdge [$PT(net) Edge $ie get Src] $p $ie
#            }  ;# else, auf jeden Fall anzeigen
#          } else {
#            if {[$PT(net) Edge $ie get Visibility] != 1} {
#              if $PT(InvisibleArcsOn) {
#                MkBlockEdge [$PT(net) Edge $ie get BlockSrc] $p $ie
#              }  ;# if unsichtbare Kanten doch sichtbar
#            } else {
#              MkBlockEdge [$PT(net) Edge $ie get BlockSrc] $p $ie
#            }  ;# else, auf jeden Fall anzeigen
#          }  ;# else nicht sichtbar oder Blockkante
#        }  ;# foreach eingehende Kante
#        set out [$PT(net) Transition $p get OutEdges]
#        foreach oe $out {
#          if {[$PT(net) Edge $oe get BlockVis] == "exposed"} {
#            if {[$PT(net) Edge $oe get Visibility] != 1} {
#              if $PT(InvisibleArcsOn) {
#                MkEdge $p [$PT(net) Edge $oe get Dest] $oe
#              }  ;# if unsichtbare Kanten doch sichtbar
#            } else { 
#              MkEdge $p [$PT(net) Edge $oe get Dest] $oe
#            }  ;# else, auf jeden Fall anzeigen
#          } else {
#           if {[$PT(net) Edge $oe get Visibility] != 1} {
#             if $PT(InvisibleArcsOn) {
#                MkBlockEdge $p [$PT(net) Edge $oe get BlockDest] $oe
#              }  ;# if unsichtbare Kanten doch sichtbar
#            } else { 
#              MkBlockEdge $p [$PT(net) Edge $oe get BlockDest] $oe
#            }  ;# else, auf jeden Fall anzeigen
#          }  ;# else nicht sichtbar oder Blockkante
#        }  ;# foreach ausgehende Kante
#        # Markierung der Stelle
#      } else {
#        if {[$PT(net) Exists PTransition $p] > 0} {
#          set in [$PT(net) PTransition $p get InPEdges]
#          foreach ie $in {
#            if {[$PT(net) PEdge $ie get BlockVis] == "exposed"} {
#              if {[$PT(net) PEdge $ie get Visibility] != 1} {
#                if $PT(InvisibleArcsOn) {
#                  MkPEdge [$PT(net) PEdge $ie get Src] $p $ie
#                }  ;# if unsichtbare Kanten doch sichtbar
#              } else {
#                MkPEdge [$PT(net) PEdge $ie get Src] $p $ie
#              }  ;# else, auf jeden Fall anzeigen
#            } else {
#              if {[$PT(net) PEdge $ie get Visibility] != 1} {
#                if $PT(InvisibleArcsOn) {
#                  MkBlockPEdge [$PT(net) PEdge $ie get BlockSrc] $p $ie
#                }  ;# if unsichtbare Kanten doch sichtbar
#              } else {
#                MkBlockPEdge [$PT(net) PEdge $ie get BlockSrc] $p $ie
#              }  ;# else, auf jeden Fall anzeigen
#            }  ;# else nicht sichtbar oder Blockkante
#          }  ;# foreach eingehende Kante
#          set out [$PT(net) PTransition $p get OutPEdges]
#          foreach oe $out {
#            if {[$PT(net) PEdge $oe get BlockVis] == "exposed"} {
#              if {[$PT(net) PEdge $oe get Visibility] != 1} {
#                if $PT(InvisibleArcsOn) {
#                  MkPEdge $p [$PT(net) PEdge $oe get Dest] $oe
#                }  ;# if unsichtbare Kanten doch sichtbar
#              } else {
#                MkPEdge $p [$PT(net) PEdge $oe get Dest] $oe
#             }  ;# else, auf jeden Fall anzeigen
#            } else {
#              if {[$PT(net) PEdge $oe get Visibility] != 1} {
#                if $PT(InvisibleArcsOn) {
#                  MkBlockPEdge $p [$PT(net) PEdge $oe get BlockDest] $oe
#                }  ;# if unsichtbare Kanten doch sichtbar
#              } else { 
#                MkBlockPEdge $p [$PT(net) PEdge $oe get BlockDest] $oe
#              }  ;# else, auf jeden Fall anzeigen
#            }  ;# else nicht sichtbar oder Blockkante
#          }  ;# foreach ausgehende Kante
#        }
#      }
#    }  ;# foreach sichtbare Stelle im Netz

    foreach e $node_list {
      if {[$PT(net) Exists Transition $e] > 0} {
        $PT(canvas) addtag selected withtag $e
        set oe [$PT(net) Transition $e get OutEdges]
        foreach o $oe {
          $PT(canvas) addtag selected withtag $o
        }
        set ie [$PT(net) Transition $e get InEdges]
        foreach i $ie {
          $PT(canvas) addtag selected withtag $i
        }
      }
      if {[$PT(net) Exists PTransition $e] > 0} {
        $PT(canvas) addtag selected withtag $e
        set oe [$PT(net) PTransition $e get OutPEdges]
        foreach o $oe {
          $PT(canvas) addtag selected withtag $o
        }
        set ie [$PT(net) PTransition $e get InPEdges]
        foreach i $ie {
          $PT(canvas) addtag selected withtag $i
        }
      }
      if {[$PT(net) Exists Place $e] > 0} {
        $PT(canvas) addtag selected withtag $e
      }
      if {[$PT(net) Exists Block $e] > 0} {
        $PT(canvas) addtag selected withtag $e
      }
    }

    if {($MinX != 10000) && ($MinY != 10000) && ($MaxX != 0) && ($MaxY != 0)} {
      if {[llength [$PT(canvas) find withtag selected]] > 0} {
        $PT(canvas) addtag Area withtag \
          [$PT(canvas) create rect $MinX $MinY $MaxX $MaxY -outline black]
        set box [$PT(canvas) bbox selected]
        $PT(canvas) itemconfigure Area -fill green -stipple gray25
        $PT(canvas) coords Area \
          [lindex $box 0] [lindex $box 1] [lindex $box 2] [lindex $box 3]
        $PT(canvas) raise Area
      }
      update idletask
    }

    foreach e $node_list {
      if {[$PT(net) Exists Transition $e] > 0} {
        $PT(canvas) addtag Area withtag $e
        set oe [$PT(net) Transition $e get OutEdges]
        foreach o $oe {
          $PT(canvas) addtag Area withtag $o
        }
        set ie [$PT(net) Transition $e get InEdges]
        foreach i $ie {
          $PT(canvas) addtag Area withtag $i
        }
      }
      if {[$PT(net) Exists PTransition $e] > 0} {
        $PT(canvas) addtag Area withtag $e
        set oe [$PT(net) PTransition $e get OutPEdges]
        foreach o $oe {
          $PT(canvas) addtag Area withtag $o
        }
        set ie [$PT(net) PTransition $e get InPEdges]
        foreach i $ie {
          $PT(canvas) addtag Area withtag $i
        }
      }
      if {[$PT(net) Exists Place $e] > 0} {
        $PT(canvas) addtag Area withtag $e
      }
      if {[$PT(net) Exists Block $e] > 0} {
        $PT(canvas) addtag Area withtag $e
      }
    }

    NetChanged
  }
}  ;# LoadModule



# -------------------------------------------------------------
# Name
#   LoadNet
# ---------
# Parameter 
#   filename -> klar
# ---------
# Description
#   Eine Netzdatei soll geladen werden. 
#   Initialisierung wird aufgerufen, alles geloescht, falls die
#   Datei existiert, wird sie geladen (vom C++-Kern). Danach werden
#   die graphischen Objekte erstellt (das ist zeitkritisch).
proc LoadNet {filename} {
  global PT Id2Name Blocks

  set PT(OldZoom) $PT(Zoom)
  set PT(Zoom) 1
  set factor [expr double($PT(Zoom)) / double($PT(OldZoom))]
  set PT(ObjectSize) [expr $PT(ObjectSize) * $factor]
  set PT(OSizeHalf) [expr $PT(ObjectSize) / 2]
  $PT(net) set ObjectSize $PT(ObjectSize)

  Init
  $PT(net) DeleteAll
  set PT(Zoom) 1
  set factor [expr double($PT(Zoom)) / double($PT(OldZoom))]
  set PT(ObjectSize) [expr $PT(ObjectSize) / $factor]
  set PT(OSizeHalf) [expr $PT(ObjectSize) / 2]
  set PT(ObjectSize) [$PT(net) get ObjectSize]
  set PT(OSizeHalf) [expr int($PT(ObjectSize) / 2)]
  $PT(net) set ObjectSize $PT(ObjectSize)
  set PT(OldZoom) 1

  if {[file exists $filename]} {
    $PT(net) read $filename
  }  ;# if Netzdatei ist vorhanden

  set visblocks [$PT(net) get VisBlocks]    
  set visplaces [$PT(net) get VisPlaces]    
  set vistransitions [$PT(net) get VisTransitions]
  set visptransitions [$PT(net) get VisPTransitions]
  foreach b $visblocks {
    set Blocks($b) 0
    $PT(net) Block $b set \
      x [expr int([$PT(net) Block $b get x] * $PT(Zoom))] \
      y [expr int([$PT(net) Block $b get y] * $PT(Zoom))]
  }  ;# foreach sichtbaren Block im Netz
  foreach p $visplaces {
    $PT(net) Place $p set \
      x [expr int(([$PT(net) Place $p get x] + $PT(OSizeHalf)) * $PT(Zoom))] \
      y [expr int(([$PT(net) Place $p get y] + $PT(OSizeHalf)) * $PT(Zoom))]
  }  ;# foreach sichtbare Stelle im Netz
  foreach t $vistransitions {
    $PT(net) Transition $t set \
      x [expr int(([$PT(net) Transition $t get x] + $PT(OSizeHalf)) * $PT(Zoom))] \
      y [expr int(([$PT(net) Transition $t get y] + $PT(OSizeHalf)) * $PT(Zoom))]
  }  ;# foreach sichtbare Transition im Netz
  foreach t $visptransitions {
    $PT(net) PTransition $t set \
      x [expr int([$PT(net) PTransition $t get x] + $PT(OSizeHalf))] \
      y [expr int([$PT(net) PTransition $t get y] + $PT(OSizeHalf))]
  }  ;# foreach sichtbare PTransition im Netz

  set PT(Filename) $filename
  $PT(FilenameW) configure -text $filename
    # Blockhierarchie aufbauen
  $PT(net) RecalcRootBlocks
    # Die obersten Bloecke aufklappen. 
    # Bei Netzen aus dem Uebersetzer ist das nur genau einer.
  set rootblocks [$PT(net) get RootBlocks]
  foreach rb $rootblocks {
    $PT(net) Block $rb set BlockVis exposed_contents
  }  ;# foreach RootBlock im Netz
  set font [$PT(net) get NameP]
  if {$font == "NULL"} {
    set PT(namePlace) "-*-Helvetica-Bold-R-Normal--*-100-*"
  } else {
    set PT(namePlace) $font
  }
  set font [$PT(net) get NameT]
  if {$font == "NULL"} {
    set PT(nameTrans) "-*-Helvetica-Bold-R-Normal--*-100-*"
  } else {
    set PT(nameTrans) $font
  }
  set font [$PT(net) get MeaningP]
  if {$font == "NULL"} {
    set PT(meanPlace) $PT(AnnoFont)
  } else {
    set PT(meanPlace) $font
  }
  set font [$PT(net) get MeaningT]
  if {$font == "NULL"} {
    set PT(meanTrans) $PT(AnnoFont)
  } else {
    set PT(meanTrans) $font
  }
  set font [$PT(net) get AnnotationsA]
  if {$font == "NULL"} {
    set PT(edge) $PT(AnnoFont)
  } else {
    set PT(edge) $font
  }
  set font [$PT(net) get ActionTermT]
  if {$font == "NULL"} {
    set PT(actionTerms) $PT(AnnoFont)
  } else {
    set PT(actionTerms) $font
  }
  set font [$PT(net) get ValueTermT]
  if {$font == "NULL"} {
    set PT(valueTerms) $PT(AnnoFont)
  } else {
    set PT(valueTerms) $font
  }
  set font [$PT(net) get TypeP]
  if {$font == "NULL"} {
    set PT(types) $PT(AnnoFont)
  } else {
    set PT(types) $font
  }
  set font [$PT(net) get Comments]
  if {$font == "NULL"} {
    set PT(comments) $PT(AnnoFont)
  } else {
    set PT(comments) $font
  }

  Repaint  

  NetChanged 0 
}  ;# LoadNet

# -------------------------------------------------------------
# Name
#   SaveNet
# ---------
# Parameter 
#   f -> Dateiname
# ---------
# Description
#   Das Netz wird gespeichert. Vorher wird der C++-Kern hinsichtlich 
#   der Positionen der Knoten auf den neusten Stand gebracht.
proc SaveNet {f} {
  global PT Id2Name

    # Fuer den kleinen Burkhard. Sehr witzig!
  if {$PT(Filename) != $f} {
    new_filename_server $f
  }  ;# if eine wichtig Mitteilung an das PEPTool war faellig
    # irgendsoeinduselhattedienaechstezeileauskommentiert BB
  file_changed_server
  set visblocks [$PT(net) get VisBlocks]    
  set visplaces [$PT(net) get VisPlaces]    
  set vistransitions [$PT(net) get VisTransitions]
  set visptransitions [$PT(net) get VisPTransitions]
  set edges [$PT(canvas) find withtag Edge]

  foreach b $visblocks {
    set coords [$PT(canvas) coords $b]
 #   $PT(net) Block $b set \
 #     x [expr int([lindex $coords 0]) - int($PT(ObjectSize))] \
 #     y [expr int([lindex $coords 1]) - int($PT(ObjectSize) + 6)]
  }  ;# foreach sichtbaren Block im Netz
  foreach p $visplaces {
    set coords [$PT(canvas) coords $p]
    $PT(net) Place $p set \
      x [expr int([lindex $coords 0]) + int($PT(OSizeHalf))] \
      y [expr int([lindex $coords 1]) + int($PT(OSizeHalf))]
  }  ;# foreach sichtbare Stelle im Netz
  foreach t $vistransitions {
    set coords [$PT(canvas) coords $t]
    $PT(net) Transition $t set \
      x [expr int([lindex $coords 0]) + int($PT(OSizeHalf))] \
      y [expr int([lindex $coords 1]) + int($PT(OSizeHalf))]
  }  ;# foreach sichtbare Transition im Netz
  foreach t $visptransitions {
    set coords [$PT(canvas) coords $t]
    $PT(net) PTransition $t set \
      x [expr int([lindex $coords 0]) + int($PT(OSizeHalf))] \
      y [expr int([lindex $coords 1]) + int($PT(OSizeHalf))]
  }  ;# foreach sichtbare PTransition im Netz
  set $PT(Filename) $f
  $PT(net) WithTime $PT(Time)
  $PT(net) save $f
  $PT(FilenameW) configure -text $f
  NetChanged 0
  foreach b $visblocks {
    set coords [$PT(canvas) coords $b]
  #  $PT(net) Block $b set \
  #    x [expr int([lindex $coords 0]) + int($PT(OSizeHalf))] \
  #    y [expr int([lindex $coords 1]) + int($PT(OSizeHalf) - 6)]
  }  ;# foreach sichtbaren Block im Netz
  foreach p $visplaces {
    set coords [$PT(canvas) coords $p]
    $PT(net) Place $p set \
      x [expr int([lindex $coords 0]) + int($PT(ObjectSize))] \
      y [expr int([lindex $coords 1]) + int($PT(ObjectSize))]
  }  ;# foreach sichtbare Stelle im Netz
  foreach t $vistransitions {
    set coords [$PT(canvas) coords $t]
    $PT(net) Transition $t set \
      x [expr int([lindex $coords 0]) + int($PT(ObjectSize))] \
      y [expr int([lindex $coords 1]) + int($PT(ObjectSize))]
  }  ;# foreach sichtbare Transition im Netz
  foreach t $visptransitions {
    set coords [$PT(canvas) coords $t]
    $PT(net) PTransition $t set \
      x [expr int([lindex $coords 0]) + int($PT(ObjectSize))] \
      y [expr int([lindex $coords 1]) + int($PT(ObjectSize))]
  }  ;# foreach sichtbare PTransition im Netz
}  ;# SaveNet

# -------------------------------------------------------------
# Name
#   AreaStart
# ---------
# Parameter 
#   x, y -> Position
# ---------
# Description
#   Ein Bereich soll selektiert werden. (x, y) ist der Startpunkt
proc AreaStart {x y} {
  global PT

  # No Gridding for Area!
  set PT(areaX1) [$PT(canvas) canvasx $x]
  set PT(areaY1) [$PT(canvas) canvasy $y]
  #$PT(canvas) delete Area
  set PT(select) [$PT(canvas) find withtag selected]
  DeselectAll
}  ;# AreaStart

# -------------------------------------------------------------
# Name
#   ClearAreas
# ------------
# Parameter
#   Nothing
# ------------
# Description
#   Alle Areas werden gel"oscht und der Zaehler zur"uckgesetzt
proc ClearAreas {} {
  global PT

  set counter 0
  while {$counter < $PT(areas)} {
    set temp [$PT(canvas) find withtag Area$counter]
    foreach t $temp {
      set tags [$PT(canvas) gettags $t]
      if {[lsearch -regexp $tags "Node"] < 0} {
        if {[lsearch -regexp $tags "Edge"] < 0} {
          if {[lsearch -regexp $tags "Token"] < 0} {
            $PT(canvas) delete $t
          }
        }
      }
    }
    $PT(canvas) dtag Area$counter
    set counter [expr $counter + 1]
  }
  set temp [$PT(canvas) find withtag Area]
  foreach t $temp {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags "Node"] < 0} {
      if {[lsearch -regexp $tags "Edge"] < 0} {
        if {[lsearch -regexp $tags "Token"] < 0} {
          $PT(canvas) delete $t
        }
      }
    }
  }
  $PT(canvas) dtag Area
  set PT(areas) 0
}  ;# ClearAreas

# -------------------------------------------------------------
# Name
#   AreaStroke
# ---------
# Parameter 
#   x, y -> Position
# ---------
# Description
#   Die Bereichswahl ist noch nicht fertig. (x, y) ist die vorlaeufige 
#   Endposition. Wenn B2 losgelassen wird, wird StrokeStop aufgerufen.
# Todo: Farben
proc AreaStroke {x y} {
  global PT

  # No Gridding for Area!
  set x [$PT(canvas) canvasx $x]
  set y [$PT(canvas) canvasy $y]
  $PT(canvas) delete Area$PT(areas)
  $PT(canvas) addtag Area$PT(areas) withtag \
    [$PT(canvas) create rect $PT(areaX1) $PT(areaY1) $x $y -outline black]
  set PT(areaX2) $x
  set PT(areaY2) $y
  set PT(stroke) 1
  bind $PT(canvas) <B2-Motion> {AreaStroke %x %y}
}  ;# AreaStroke

# -------------------------------------------------------------
# Name
#   StrokeStop
# ---------
# Parameter 
#   x, y -> Position
# ---------
# Description
#   Ein Bereich wurde selektiert. (x, y) ist die Endposition. Alle
#   Objekte innerhalb des Bereich werden selektiert. Der Bereich 
#   wird gruenlich unterlegt und ist ebenfalls verschiebbar (Tag Area)
# Todo: Farben
proc StrokeStop {} {
  global PT Id2Name

  $PT(canvas) addtag selected enclosed \
    $PT(areaX1) $PT(areaY1) $PT(areaX2) $PT(areaY2)  
  if {[llength [$PT(canvas) find withtag selected]] > 0} {
    set box [$PT(canvas) bbox selected]
    $PT(canvas) itemconfigure Area$PT(areas) -fill green -stipple gray25
    $PT(canvas) coords Area$PT(areas) \
      [lindex $box 0] [lindex $box 1] [lindex $box 2] [lindex $box 3]
    $PT(canvas) raise Area$PT(areas)
    $PT(canvas) addtag Area$PT(areas) withtag selected
    foreach s $PT(select) {
      $PT(canvas) addtag selected withtag $s
    }
  } else {
    $PT(canvas) delete Area$PT(areas) 
  }  ;# else, nichts selektiert innerhalb
  $PT(canvas) bind Area$PT(areas) <1> {AreaMoveStart %x %y}
  $PT(canvas) bind Area$PT(areas) <B1-Motion> {AreaMove %x %y}
  $PT(canvas) bind Area$PT(areas) <ButtonRelease-1> {StopMove %x %y}
  set PT(stroke) 0
  set PT(areas) [expr $PT(areas) + 1]
}  ;# StrokeStop

# -------------------------------------------------------------
# Name
#   AreaMoveStart
# ---------
# Parameter 
#   x, y -> Position
# ---------
# Description
#   Ein Bereich soll verschoben werden. (x, y) ist der Startpunkt
#   des Verschiebungs-Vektors.
proc AreaMoveStart {x y} {
  global PT

  set PT(x1) $x
  set PT(y1) $y
  set PT(BeginX) $x
  set PT(BeginY) $y
  set temp [$PT(canvas) find withtag current]
  foreach t $temp {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags "Area*"] > -1} {
      set PT(aktuellArea) [lindex $tags [lsearch -regexp $tags "Area*"]]
    }
  }
}  ;# AreaMoveStart

# -------------------------------------------------------------
# Name
#   AreaMove
# ---------
# Parameter 
#   x, y -> Position
# ---------
# Description
#   Der Bereich wird um x - PT(x1), y - PT(y1) verschoben. PT(x1) und 
#   PT(y1) werden aktualisiert.
proc AreaMove {x y} {
  global PT Id2Name

  NetChanged
  set counter 0
  if {$PT(AreaTogether)} {
    $PT(canvas) move Area [expr $x - $PT(x1)] [expr $y - $PT(y1)]
    while {$counter < $PT(areas)} {
      # prevent multiple moves in overlapping areas
      $PT(canvas) dtag AreaMove
      $PT(canvas) addtag AreaMove withtag Area$counter
      for {set i 0} {$i < $counter} {incr i} {
        $PT(canvas) dtag Area$i AreaMove
      }
      $PT(canvas) dtag Area AreaMove
      $PT(canvas) move AreaMove [expr $x - $PT(x1)] [expr $y - $PT(y1)]
      set visblocks [$PT(net) get VisBlocks]    
      set visplaces [$PT(net) get VisPlaces]    
      set vistransitions [$PT(net) get VisTransitions]
      set visptransitions [$PT(net) get VisPTransitions]
      set edges [$PT(canvas) find withtag Edge]
      foreach edge $edges {
        set coords [$PT(canvas) coords $edge]
        set tags [$PT(canvas) gettags $edge]
        if {[lsearch $tags AreaMove] > 0} {
          if {[lsearch -regexp $tags PA] > 0} {
            set X [$PT(net) PEdge $Id2Name($edge) get SplineX]
            set Y [$PT(net) PEdge $Id2Name($edge) get SplineY]
            if {$X != 0 || $Y != 0} {
              $PT(net) PEdge $Id2Name($edge) set SplineX [expr int([lindex $coords 2])]
              $PT(net) PEdge $Id2Name($edge) set SplineY [expr int([lindex $coords 3])]
            }
          } else {
            set X [expr int([$PT(net) Edge $Id2Name($edge) get SplineX] / $PT(Zoom))]
            set Y [expr int([$PT(net) Edge $Id2Name($edge) get SplineY] / $PT(Zoom))]
            if {$X != 0 || $Y != 0} {
              $PT(net) Edge $Id2Name($edge) set SplineX [expr int([lindex $coords 2])]
              $PT(net) Edge $Id2Name($edge) set SplineY [expr int([lindex $coords 3])]
            }
          }
        } 
      }
      set counter [expr $counter + 1]
    }
  } else {
    $PT(canvas) move $PT(aktuellArea) [expr $x - $PT(x1)] [expr $y - $PT(y1)]
    set visblocks [$PT(net) get VisBlocks]    
    set visplaces [$PT(net) get VisPlaces]    
    set vistransitions [$PT(net) get VisTransitions]
    set visptransitions [$PT(net) get VisPTransitions]
    set edges [$PT(canvas) find withtag Edge]
    foreach edge $edges {
      set coords [$PT(canvas) coords $edge]
      set tags [$PT(canvas) gettags $edge]
      if {[lsearch $tags $PT(aktuellArea)] > 0} {
        if {[lsearch -regexp $tags PA] > 0} {
          set X [$PT(net) PEdge $Id2Name($edge) get SplineX]
          set Y [$PT(net) PEdge $Id2Name($edge) get SplineY]
          if {$X != 0 || $Y != 0} {
            $PT(net) PEdge $Id2Name($edge) set SplineX [expr int([lindex $coords 2])]
            $PT(net) PEdge $Id2Name($edge) set SplineY [expr int([lindex $coords 3])]
          }
        } else {
          set X [expr int([$PT(net) Edge $Id2Name($edge) get SplineX] / $PT(Zoom))]
          set Y [expr int([$PT(net) Edge $Id2Name($edge) get SplineY] / $PT(Zoom))]
          if {$X != 0 || $Y != 0} {
            $PT(net) Edge $Id2Name($edge) set SplineX [expr int([lindex $coords 2])]
            $PT(net) Edge $Id2Name($edge) set SplineY [expr int([lindex $coords 3])]
          }
        }
      } 
    }
  }
  set PT(x1) $x
  set PT(y1) $y
}  ;# AreaMove

# -------------------------------------------------------------
# Name
#   MoveOptions
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Die Optionen f"ur die verschiedenen Moveoptionen werden
#   gesetzt.
proc MoveOptions {} {
  global PT

  if [catch {set d [toplevel .moveoptions]}] {
   raise .moveoptions
   return
  }  ;# if doppelt
  wm title $d "Move Options"
  wm protocol $d WM_DELETE_WINDOW "destroy $d"

    # Container fr die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
  checkbutton $d.options.nodeswithblock \
    -text "Move Nodes with Block" -anchor w \
    -variable PT(NodesWithBlock)
  checkbutton $d.options.nodeswithbox \
    -text "Move Nodes with Box" -anchor w \
    -variable PT(NodesWithBox)
  checkbutton $d.options.edgeswith \
    -text "Show Arcs while moving" -anchor w \
    -variable PT(MoveEdgesWith)
  checkbutton $d.options.areatogether \
    -text "Move Selections together" -anchor w \
    -variable PT(AreaTogether)
  pack $d.options.nodeswithblock \
    $d.options.nodeswithbox \
    $d.options.edgeswith \
    $d.options.areatogether \
     -side top -anchor w -fill x

    # Container fr die Buttons der Kommandos
  frame $d.commands -relief raised -bd 1
  pack $d.commands -side bottom -fill both
    # Kommando-Buttons
  button $d.commands.ok -text "Ok" \
    -command "destroy $d"
  button $d.commands.default -text "Default" \
    -command "MoveOptionsDefault"
  button $d.commands.cancel -text "Cancel" \
    -command "destroy $d"
  pack $d.commands.ok \
    $d.commands.default \
    $d.commands.cancel \
    -side left -expand 1
}  ;# MoveOptions

# -------------------------------------------------------------
# Name
#   MoveOptionsDefault
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Die Defaults f"ur die MoveOptions werden hergestellt.
proc MoveOptionsDefault {} {
  global PT

  set PT(NodesWithBlock) 1
  set PT(NodesWithBox) 1
  set PT(AreaTogether) 1
}  ;# MoveOptionsDefault

# -------------------------------------------------------------
# Name
#   Cut
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Die selektierten Objekte werden in das clipboard "ubertragen
#   und anschlie"send aus canvas gel"oscht
proc Cut {} {
  global PT Id2Name

  set visblocks [$PT(net) get VisBlocks]    
  set visplaces [$PT(net) get VisPlaces]    
  set vistransitions [$PT(net) get VisTransitions]
  set visptransitions [$PT(net) get VisPTransitions]
  set edges [$PT(canvas) find withtag Edge]
  set SELECTED ""
  foreach t [$PT(canvas) find withtag Transition] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag Place] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag PTransition] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag Block] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  $PT(net) Cut $SELECTED
  $PT(canvas) delete withtag selected
  $PT(canvas) delete withtag Area
  ClearAreas
  NetChanged
} ;# Cut

# -------------------------------------------------------------
# Name
#   Copy
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Eine Kopie der Objekte wird in das clipboard "ubertragen
proc Copy {} {
  global PT Id2Name

  set visblocks [$PT(net) get VisBlocks]    
  set visplaces [$PT(net) get VisPlaces]    
  set vistransitions [$PT(net) get VisTransitions]
  set visptransitions [$PT(net) get VisPTransitions]
  set edges [$PT(canvas) find withtag Edge]
  set SELECTED ""
  foreach t [$PT(canvas) find withtag Transition] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag Place] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag PTransition] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag Block] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag Box] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  $PT(net) Copy $SELECTED
} ;# Copy

# -------------------------------------------------------------
# Name
#   Paste
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Die Objekte aus dem clipboard werden wieder in das canvas kopiert
#   und zwar an die Position der alten Objekte. Mit Hilfe eines gr"unen
#   Rahmens lassen sie sich verschieben.
proc Paste {} {
  global PT Id2Name Blocks

  ClearAreas
  $PT(net) Paste
  $PT(net) RecalcRootBlocks
  $PT(net) RecalcBlockVis
  set selected [$PT(net) Selected]
  set node_list [split $selected ","]

  set l [llength $node_list]
  set node_list [lrange $node_list 0 [expr $l - 2]]
  foreach t $node_list {
    if {[$PT(net) Exists Transition $t] > 0} {
      if {[$PT(net) Transition $t get BlockVis] == "exposed"} {
        MkTransition [expr [$PT(net) Transition $t get x] * $PT(Zoom)] \
          [expr [$PT(net) Transition $t get y] * $PT(Zoom)] $t
      }
    }
    if {[$PT(net) Exists PTransition $t] > 0} {
      if {[$PT(net) PTransition $t get BlockVis] == "exposed"} {
        MkPTransition [expr [$PT(net) PTransition $t get x] * $PT(Zoom)] \
          [expr [$PT(net) PTransition $t get y] * $PT(Zoom)] $t
      }
    }
    if {[$PT(net) Exists Place $t] > 0} {
      if {[$PT(net) Place $t get BlockVis] == "exposed"} {
        MkPlace [expr [$PT(net) Place $t get x] * $PT(Zoom)] \
          [expr [$PT(net) Place $t get y] * $PT(Zoom)] $t
        LayoutTokens $t
        foreach i {1 2 3 4 5} {
          $PT(canvas) addtag selected withtag t$i@$t
        }
        $PT(canvas) addtag selected withtag NumTokens@$t
      }
    }
    if {[$PT(net) Exists Block $t] > 0} {
      if {[$PT(net) Block $t get BlockVis] == "exposed"} {
        MkBlock [expr [$PT(net) Block $t get x] * $PT(Zoom)] \
          [expr [$PT(net) Block $t get y] * $PT(Zoom)] $t
      }
      set Blocks($t) 0
    }
    if {[$PT(net) Exists Box $t] > 0} {
      if {[$PT(net) Box $t get Visibility]} {
        MkBox [expr [$PT(net) Box $t get x] * $PT(Zoom)] \
          [expr [$PT(net) Box $t get y] * $PT(Zoom)] $t
      }
    }
  }
  DeselectAll
  set MinX 10000
  set MinY 10000
  set MaxX 0
  set MaxY 0
  foreach n $node_list {
    set node [$PT(canvas) find withtag $n]
    set cor [$PT(canvas) coords $node]
    if {[lindex $cor 0] < $MinX} {
      set MinX [lindex $cor 0]
    }
    if {[lindex $cor 1] < $MinY} {
      set MinY [lindex $cor 1]
    }
    if {[lindex $cor 0] > $MaxX} {
      set MaxX [lindex $cor 0]
    }
    if {[lindex $cor 1] > $MaxY} {
      set MaxY [lindex $cor 1]
    }
    $PT(canvas) addtag selected withtag $node
  }
  foreach p $node_list {
      # Kanten erstellen
    if {[$PT(net) Exists Transition $p] > 0} {
      set in [$PT(net) Transition $p get InEdges]
      foreach ie $in {
        if {[$PT(net) Edge $ie get BlockVis] == "exposed"} {
          if {[$PT(net) Edge $ie get Visibility] != 1} {
            if $PT(InvisibleArcsOn) {
              MkEdge [$PT(net) Edge $ie get Src] $p $ie
            }  ;# if unsichtbare Kanten doch sichtbar
          } else {
            MkEdge [$PT(net) Edge $ie get Src] $p $ie
          }  ;# else, auf jeden Fall anzeigen
        } else {
          if {[$PT(net) Edge $ie get Visibility] != 1} {
            if $PT(InvisibleArcsOn) {
              MkBlockEdge [$PT(net) Edge $ie get BlockSrc] $p $ie
            }  ;# if unsichtbare Kanten doch sichtbar
          } else {
            MkBlockEdge [$PT(net) Edge $ie get BlockSrc] $p $ie
          }  ;# else, auf jeden Fall anzeigen
        }  ;# else nicht sichtbar oder Blockkante
      }  ;# foreach eingehende Kante
      set out [$PT(net) Transition $p get OutEdges]
      foreach oe $out {
        if {[$PT(net) Edge $oe get BlockVis] == "exposed"} {
          if {[$PT(net) Edge $oe get Visibility] != 1} {
            if $PT(InvisibleArcsOn) {
              MkEdge $p [$PT(net) Edge $oe get Dest] $oe
            }  ;# if unsichtbare Kanten doch sichtbar
          } else { 
            MkEdge $p [$PT(net) Edge $oe get Dest] $oe
          }  ;# else, auf jeden Fall anzeigen
        } else {
          if {[$PT(net) Edge $oe get Visibility] != 1} {
            if $PT(InvisibleArcsOn) {
              MkBlockEdge $p [$PT(net) Edge $oe get BlockDest] $oe
            }  ;# if unsichtbare Kanten doch sichtbar
          } else { 
            MkBlockEdge $p [$PT(net) Edge $oe get BlockDest] $oe
          }  ;# else, auf jeden Fall anzeigen
        }  ;# else nicht sichtbar oder Blockkante
      }  ;# foreach ausgehende Kante
    } else {
      if {[$PT(net) Exists PTransition $p] > 0} {
        set in [$PT(net) PTransition $p get InPEdges]
        foreach ie $in {
          if {[$PT(net) PEdge $ie get BlockVis] == "exposed"} {
            if {[$PT(net) PEdge $ie get Visibility] != 1} {
              if $PT(InvisibleArcsOn) {
                MkPEdge [$PT(net) PEdge $ie get Src] $p $ie
              }  ;# if unsichtbare Kanten doch sichtbar
            } else {
              MkPEdge [$PT(net) PEdge $ie get Src] $p $ie
            }  ;# else, auf jeden Fall anzeigen
          } else {
            if {[$PT(net) PEdge $ie get Visibility] != 1} {
              if $PT(InvisibleArcsOn) {
                MkBlockPEdge [$PT(net) PEdge $ie get BlockSrc] $p $ie
              }  ;# if unsichtbare Kanten doch sichtbar
            } else {
              MkBlockPEdge [$PT(net) PEdge $ie get BlockSrc] $p $ie
            }  ;# else, auf jeden Fall anzeigen
          }  ;# else nicht sichtbar oder Blockkante
        }  ;# foreach eingehende Kante
        set out [$PT(net) PTransition $p get OutPEdges]
        foreach oe $out {
          if {[$PT(net) PEdge $oe get BlockVis] == "exposed"} {
            if {[$PT(net) PEdge $oe get Visibility] != 1} {
              if $PT(InvisibleArcsOn) {
                MkPEdge $p [$PT(net) PEdge $oe get Dest] $oe
              }  ;# if unsichtbare Kanten doch sichtbar
            } else {
              MkPEdge $p [$PT(net) PEdge $oe get Dest] $oe
            }  ;# else, auf jeden Fall anzeigen
          } else {
            if {[$PT(net) PEdge $oe get Visibility] != 1} {
              if $PT(InvisibleArcsOn) {
                MkBlockPEdge $p [$PT(net) PEdge $oe get BlockDest] $oe
              }  ;# if unsichtbare Kanten doch sichtbar
            } else { 
              MkBlockPEdge $p [$PT(net) PEdge $oe get BlockDest] $oe
            }  ;# else, auf jeden Fall anzeigen
          }  ;# else nicht sichtbar oder Blockkante
        }  ;# foreach ausgehende Kante
      }
    }
  }  ;# foreach sichtbare Stelle im Netz

  foreach e $node_list {
    if {[$PT(net) Exists Transition $e] > 0} {
      set oe [$PT(net) Transition $e get OutEdges]
      foreach o $oe {
        $PT(canvas) addtag selected withtag $o
      }
      set ie [$PT(net) Transition $e get InEdges]
      foreach i $ie {
        $PT(canvas) addtag selected withtag $i
      }
    }
    if {[$PT(net) Exists PTransition $e] > 0} {
      set oe [$PT(net) PTransition $e get OutPEdges]
      foreach o $oe {
        $PT(canvas) addtag selected withtag $o
      }
      set ie [$PT(net) PTransition $e get InPEdges]
      foreach i $ie {
        $PT(canvas) addtag selected withtag $i
      }
    }
  }

  if {[llength [$PT(canvas) find withtag selected]] > 0} {
    set box [$PT(canvas) bbox selected]
    $PT(canvas) create rect [lindex $box 0] [lindex $box 1] \
      [lindex $box 2] [lindex $box 3] -fill green -stipple gray25 \
      -tags Area
    $PT(canvas) raise Area
    $PT(canvas) addtag Area withtag selected
  }
  set PT(aktuellArea) Area
  update idletask
} ;# Paste

# -------------------------------------------------------------
# Name
#   NetChanged
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Das Netz wurde veraendert.
proc NetChanged {{yesno 1}} {
  global PT

  if {$yesno} {
    if {$yesno != $PT(NetChanged)} {
      catch {.g.fr4b.save configure -state normal;
      .mbar.file.menu entryconfigure 2 -state normal}
    }
    set PT(NetChanged) 1
  } else {
    catch {.g.fr4b.save configure -state disabled;
    .mbar.file.menu entryconfigure 2 -state disabled}
    set PT(NetChanged) 0
  }
}  ;# NetChanged

# -------------------------------------------------------------
# Name
#   Grid
# ---------
# Parameter 
#   x, y -> Position (CALL BY REFERENCE)
# ---------
# Description
#   Eine Position wird gerastert, falls PT(Gridding)
proc Grid {x y} {
  global PT
  upvar x gx y gy 

  if $PT(Gridding) {
    set gx [$PT(canvas) canvasx $x $PT(GridSize)]
    set gy [$PT(canvas) canvasy $y $PT(GridSize)]
  } else {
    set gx [$PT(canvas) canvasx $x]
    set gy [$PT(canvas) canvasy $y]
  }
}  ;# Grid

proc DrawGrid { } {
  global PT

  $PT(canvas) delete withtag grid
  if $PT(Gridding) {
    set canvascoords [$PT(canvas) cget -scrollregion]
    set width [lindex $canvascoords 2]
    set height [lindex $canvascoords 3]
    for {set i 0} {$i<=$width} {incr i $PT(GridSize)} {
      $PT(canvas) create line $i 0 $i $height -tag grid -fill gray80
    }
    for {set i 0} {$i<=$height} {incr i $PT(GridSize)} {
      $PT(canvas) create line 0 $i $width $i -tag grid -fill gray80
    }
  } else {
    # We need an element for correct stacking
    $PT(canvas) create rectangle 0 0 1 1 -tag grid -outline ""
  }
  $PT(canvas) lower grid all
}

# -------------------------------------------------------------
# Name
#   ResetToInitMarking
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Das Netz wird in Anfangsmarkierung gebracht.
proc ResetToInitMarking {} {
  global PT

  foreach place [$PT(net) get Places] {
    set init_marking [$PT(net) Place $place get InitMarking]
    if {$init_marking != [$PT(net) Place $place get Marking]} {
      DelAllTokens $place
      $PT(net) Place $place set Marking $init_marking
      LayoutTokens $place
      NetChanged
    }
  }  ;# foreach Stelle im Netz
  $PT(net) clearActive
}  ;# ResetToInitMarking

# -------------------------------------------------------------
# Name
#   Zoom
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Alle Knoten werden gezoomt. der Zoomfactor ergibt sich dabei aus
#   altem und neuem Zoomfaktor.
#   Diese Aktion ist EXTREM zeitintensiv.
#   Nach dem Zoomen werden alle Kanten und Annotationen wieder
#   in Form gebracht.
proc Zoom {} {
  global PT

  DeselectAll
  $PT(ModeW) configure -text "Zooming ...."
  update
  set factor [expr double($PT(Zoom)) / double($PT(OldZoom))]
  set PT(ObjectSize) [expr $PT(ObjectSize) * $factor]
  set PT(OSizeHalf) [expr $PT(ObjectSize) / 2]
  $PT(net) set ObjectSize $PT(ObjectSize)
  $PT(canvas) scale Node 0 0  $factor $factor
  $PT(canvas) scale Token 0 0 $factor $factor
  $PT(canvas) addtag selected withtag Node
  Repaint
  #StopMove
  set PT(OldZoom) $PT(Zoom)
  $PT(ModeW) configure -text "Edit Mode"
}  ;# Zoom

# Name
#   Bigger
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Das Netz wird in der naechsth"oheren Vergr"o"serungsstufe
#   dargestellt.
proc Bigger {} {
  global PT

  if {$PT(Zoom) == 0.25} {
    set PT(OldZoom) $PT(Zoom)
    set PT(Zoom) 0.50
    Zoom
  } elseif {$PT(Zoom) == 0.50} {
    set PT(OldZoom) $PT(Zoom)
    set PT(Zoom) 0.75
    Zoom
  } elseif {$PT(Zoom) == 0.75} {
    set PT(OldZoom) $PT(Zoom)
    set PT(Zoom) 1
    Zoom
  } elseif {$PT(Zoom) == 1} {
    set PT(OldZoom) $PT(Zoom)
    set PT(Zoom) 1.5
    Zoom
  } elseif {$PT(Zoom) == 1.5} {
    set PT(OldZoom) $PT(Zoom)
    set PT(Zoom) 2
    Zoom
  }  
} ;# Bigger

# Name
#   Smaller
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Das Netz wird in der naechstkleineren Vergr"o"serungsstufe
#   dargestellt.
proc Smaller {} {
  global PT

  if {$PT(Zoom) == 0.50} {
    set PT(OldZoom) $PT(Zoom)
    set PT(Zoom) 0.25
    Zoom
  } elseif {$PT(Zoom) == 0.75} {
    set PT(OldZoom) $PT(Zoom)
    set PT(Zoom) 0.50
    Zoom
  } elseif {$PT(Zoom) == 1} {
    set PT(OldZoom) $PT(Zoom)
    set PT(Zoom) 0.75
    Zoom
  } elseif {$PT(Zoom) == 1.5} {
    set PT(OldZoom) $PT(Zoom)
    set PT(Zoom) 1
    Zoom
  } elseif {$PT(Zoom) == 2} {
    set PT(OldZoom) $PT(Zoom)
    set PT(Zoom) 1.5
    Zoom
  }
} ;# Smaller

# -------------------------------------------------------------
# Name
#   DisplayTitles
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Alle im Kern gespeicherten Title werden auf den Canvas gebracht 
#   Wird nur von von LoadNet aufgerufen.
proc DisplayTitles {} {
  global PT

  foreach t [$PT(net) get Titles] {
    set ti [$PT(net) get Title $t]
    if {[lindex $ti 3] == "NULL"} {
      set c [$PT(canvas) create text [lindex $ti 1] [lindex $ti 2]\
        -tags Title -text [lindex $ti 0] -font $PT(TitleFont)]
    } else {
      set c [$PT(canvas) create text [lindex $ti 1] [lindex $ti 2]\
        -tags Title -text [lindex $ti 0] -font [lindex $ti 3]]
    }
    $PT(canvas) addtag _title@$t withtag $c
  }  ;# foreach Titel
}  ;# DisplayTitles

# -------------------------------------------------------------
# Name
#   TitleStopMove
# ---------
# Parameter 
#   x, y -> Position
# ---------
# Description
#   Ein Titel wurde an die Stelle (x, y) verschoben. Dem C++-Kern  wird
#   die neue Position mitgeteilt
proc TitleStopMove {x y} {
  global PT

  set c [$PT(canvas) find withtag current]
#  $PT(canvas) dtag selected
 # $PT(canvas) itemconfigure current -fill black
  set tags [$PT(canvas) gettags $c]
  set t [lindex $tags [lsearch -glob $tags _title*]]
  set nr [lindex [split $t @] 1]
  Grid $x $y
  $PT(net) set Title $nr \
    [lindex [$PT(canvas) itemconfigure $c -text] 4] \
    [expr int($x)] [expr int($y)]
}  ;# TitleStopMove

# -------------------------------------------------------------
# Name
#   SelectAll
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Es werden alle Objekte im Canvas mit den Tags Area und
#   selected versehen, so da"s sie als selektiert
#   gekennzeichnet sind.
proc SelectAll {} {
  global PT

  AnimateEnd
  $PT(canvas) addtag selected withtag Node
  $PT(canvas) addtag selected withtag Edge 
  $PT(canvas) addtag selected withtag Annotation
  $PT(canvas) addtag selected withtag Token
  $PT(canvas) addtag Area withtag Node
  $PT(canvas) addtag Area withtag Edge
  $PT(canvas) addtag Area withtag Annotation
  $PT(canvas) addtag Area withtag Token
  set box [$PT(canvas) bbox selected]
  $PT(canvas) create rect [lindex $box 0] [lindex $box 1]\
    [lindex $box 2] [lindex $box 3] -fill green -stipple gray25\
     -tags Area
  $PT(canvas) raise Area
}  ;# SelectAll

# -------------------------------------------------------------
# Name
#   CommonInit
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Alle wichtigen globalen Variablen werden initialisiert.
proc CommonInit {} {
  global PT IsAnnoFrom

  set PT(wait) 1
  set PT(FireWait) 1
  set PT(MaxList) 0

  MoveOptionsDefault
  set PT(MoveEdgesWith) 0

    # Fuer Auswahl eines Bereichs
  set PT(areaX1) 0
  set PT(areaY1) 0
  set PT(areaX2) 0
  set PT(areaY2) 0
  set PT(areas)  0
  set PT(selected) ""
  set PT(aktuellArea) ""

    # Objektgroessen
  set PT(ObjectSize) [$PT(net) get ObjectSize]
  set PT(OSizeHalf) [expr $PT(ObjectSize) / 2]

    # Welche Annotation sollen an, welche aus sein
  set PT(NameSwitch) 0
  set PT(MeanSwitch) 1
  set PT(ATSwitch) 0
  set PT(VTSwitch) 0
  set PT(Time) 0

    # Welche Transitionen sind sichtbar?
  set PT(SyncTransSwitch) 0
  $PT(net) set SyncTransSwitch $PT(SyncTransSwitch)

  set PT(PhantomTransSwitch) 1
  $PT(net) set PhantomTransSwitch $PT(PhantomTransSwitch)

    # Welches Referenzformat
  set PT(ReferenceFormat) 1

    # Netz wurde noch nicht veraendert
  NetChanged 0
  set PT(GridSize) 20
    # Gridding nicht an
  set PT(Gridding) 0
    # Unsichtbare Kanten sind ausgeschaltet
  set PT(InvisibleArcsOn) 0

    # Kantenbeschriftungen sichtbar
  set PT(EdgeVars) 1

    # Todo: Move-inits oder was
    #     set PT(gx) 0
    #     set PT(gy) 0

    # Annoationsarray instanziieren
  set IsAnnoFrom(1) {}

    # Default-Mode -> Edit
  set PT(Mode) Edit

    # Default-Liste leer
  set PT(node_list) ""
  set PT(loop_list) ""
  set PT(counters) 0
  set PT(counterl) 0

    # Default-Delay
  set PT(Speed) 100

    # Fonts
    # set PT(AnnoFont) "-*-helvetica-medium-r-narrow-*-*-140-*"
  set PT(AnnoFont) "-*-Helvetica-Medium-R-Normal--*-100-*"
  set PT(TitleFont) "-Adobe-Times-bold-r-normal--*-180-*"
  set font [$PT(net) get NameP]
  if {$font == "NULL"} {
    set PT(namePlace) "-*-Helvetica-Bold-R-Normal--*-100-*"
  } else {
    set PT(namePlace) $font
  }
  set font [$PT(net) get NameT]
  if {$font == "NULL"} {
    set PT(nameTrans) "-*-Helvetica-Bold-R-Normal--*-100-*"
  } else {
    set PT(nameTrans) $font
  }
  set font [$PT(net) get MeaningP]
  if {$font == "NULL"} {
    set PT(meanPlace) $PT(AnnoFont)
  } else {
    set PT(meanPlace) $font
  }
  set font [$PT(net) get MeaningT]
  if {$font == "NULL"} {
    set PT(meanTrans) $PT(AnnoFont)
  } else {
    set PT(meanTrans) $font
  }
  set font [$PT(net) get AnnotationsA]
  if {$font == "NULL"} {
    set PT(edge) $PT(AnnoFont)
  } else {
    set PT(edge) $font
  }
  set font [$PT(net) get ActionTermT]
  if {$font == "NULL"} {
    set PT(actionTerms) $PT(AnnoFont)
  } else {
    set PT(actionTerms) $font
  }
  set font [$PT(net) get ValueTermT]
  if {$font == "NULL"} {
    set PT(valueTerms) $PT(AnnoFont)
  } else {
    set PT(valueTerms) $font
  }
  set font [$PT(net) get TypeP]
  if {$font == "NULL"} {
    set PT(types) $PT(AnnoFont)
  } else {
    set PT(types) $font
  }
  set font [$PT(net) get Comments]
  if {$font == "NULL"} {
    set PT(comments) $PT(AnnoFont)
  } else {
    set PT(comments) $font
  }
  set PT(scomments) $PT(AnnoFont)

    # Zoomfaktoren
  set PT(Zoom) 1
  set PT(OldZoom) 1

    # Fast simulation with fewer graphical overhead
  set PT(FastSimulation) 0

    # Highlighting beim Feueren an oder aus
  set PT(HighlightWhileFire) 0

    # Farben
    # die naechste Zeile ist leicht provisorisch
  set PT(color.node) "white"
  set PT(color.place) "white"
  set PT(color.transition) "white"
  set PT(color.sync_transition) "cyan"
  set PT(color.active_transition) "yellow"
  set PT(color.phantom_transition) "gray"
  set PT(color.edge) "black"
  set PT(color.phantom_edge) "gray"
  set PT(color.block_edge) "blue"
  set PT(color.block) "white"
  set PT(color.active_block) "yellow"
  set PT(OldColor) "white"

  set PT(stroke) 0
    # Animation
  set PT(InAnimationLoop) 0
  set PT(AnimationNum) 0
  set PT(AnimationTime) [clock seconds]

    # Springen der Kanten verhindern
  set PT(x1) 0
  set PT(y1) 0
  set PT(Oldx) 0
  set PT(Oldy) 0
  set PT(type) {}
  set PT(Output) 0
}  ;# CommonInit

  #**************************
  # Kommunikationsfunktionen
  #**************************

proc end_session {{flag "Nein"}} {
  global PT

  if  {($flag == "JA") && ($PT(NetChanged))} {
    SaveNet $PT(Filename)
  }  ;# if Speichern ist gefordert
  goodbye_server
  destroy .
}  ;# end_session

  # Kommunikation zum PEPTool aufbauen, so denn vorhanden
init_client_com

  #*** Dateiende NetBase.tcl ***

