# NetDialog.tcl
# Allgemeine Dialoge fuer beide Netzeditoren.
# Burkhard Bieber 1995/96
# Ulf Fildebrandt 1997/1998
# last-update: 13.1.98

proc Help {} {
  tk_dialog .d {Help} \
  {Ask "Mr. Online Help" pep_help@informatik.uni-oldenburg.de} question 0 {Ok}
}  ;# Help

# -------------------------------------------------------------
# Name
#   InfoDialog
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Der fantastische Info Dialog.
proc InfoDialog {} {
  global my_auto_path
  global my_sounds_path
  global my_bitmaps_path
    
  destroy .info
  toplevel .info
  wm title .info "Information"
  frame .info.mess -bd 2 -relief sunken
  frame .info.action -bd 2 -relief sunken
  pack .info.mess .info.action -side left -fill both -expand yes
  message .info.mess.titel -text \
    "The PEP High- and Lowlevel Petrinet Editor" \
    -font "-Adobe-Times-*-r-*-*-24-*-*-*-*-*-*-*"\
    -justify center
  message .info.mess.message -aspect 500 -justify left  \
    -text "\
      Tobias Himstedt, 94/95\n\
      Burkhard Bieber, 95/96\n\n\
      Universitt Hildesheim\n\
      Institut fr Informatik\n\
      Dipl.-Inform. Bernd Grahlmann\n\
      Prof. Dr. Eike Best"
  pack .info.mess.titel .info.mess.message -fill both -expand yes -side top
  set spock [image create photo -file \
	[file join $my_bitmaps_path .. pep_edit spock_b.gif] -gamma 0.1]
  label .info.action.b -image $spock
  button .info.action.l -text "Fascinating" -command {destroy .info} \
    -state disabled
  pack .info.action.b .info.action.l -side top -expand yes -anchor center
  set old_focus [focus]
  tkwait visibility .info
  grab .info
  focus .info
  foreach i {0.4 0.7 1.0} {
    $spock configure -gamma $i
    $spock redither 
    update
  }  ;# foreach Gammafaktor
  catch {.info.action.l configure -state normal}
  catch {tkwait window .info}
}  ;# InfoDialog

# -------------------------------------------------------------
# Name
#   CreateTitle
# ---------
# Parameter 
#   x, y -> Position
# ---------
# Description
#   Ein neuer Titel soll an (x, y) erstellet werden. Es erscheint eine 
#   Dialogbox, in der der Titel eingegeben werden kann.
proc CreateTitle {x y} {
  global PT Dialog

  destroy .title
  set t [toplevel .title]

  wm title $t "Edit Comment"
  message $t.mess -text "Edit Comment" \
    -font "-Adobe-Times-*-r-*-*-16-*-*-*-*-*-*-*" \
    -justify center -width 150
  text $t.text -width 40 -height 10 -relief sunken -setgrid true -wrap word
  set c [$PT(canvas) find withtag current]
  set nr -1
  if {$c != ""} {
    $t.text insert end [lindex [$PT(canvas) itemconfigure current -text] 4]
    set tags [$PT(canvas) gettags current]
    set te [lindex $tags [lsearch -glob $tags _title*]]
    set nr [lindex [split $te @] 1]
  }  ;# if nicht leer
    # ------ Action Area
  frame $t.action -bd 1
  frame $t.action.default -relief sunken -bd 1
  button $t.action.default.ok -text "Ok" \
    -command {set Dialog(Button) Ok}
  pack $t.action.default.ok 
  pack $t.action.default -side left -fill x -padx 3m -pady 2m
  button $t.action.cancel -text Cancel \
   -command {set Dialog(Button) Cancel}
  pack $t.action.cancel -side right -padx 3m -pady 2m -fill x
  pack $t.mess $t.text $t.action -side top
  set old_focus [focus]
  tkwait visibility $t
  grab $t
  focus $t.text
  tkwait variable Dialog(Button)
  set text [$t.text get 1.0 "end -1 chars"]
  destroy $t
  focus $old_focus
  if {$Dialog(Button) == "Cancel"} {
    return 
  }  ;# if Abbruch des ganzen Unsinns
  Grid $x $y
  if {$c != ""} {
    $PT(net) set Title $nr $text [expr int($x)] [expr int($y)]
    $PT(canvas) itemconfigure $c -tags Title -text $text
  } else {
    set a [$PT(canvas) create text $x $y \
      -justify left \
      -font $PT(TitleFont) \
      -tags Title -text $text]
    $PT(canvas) addtag _title@[$PT(net) create Title $text \
      [expr int($x)] [expr int($y)]] withtag $a
  }  ;# else, neuer Text
}  ;# CreateTitle

# -------------------------------------------------------------
# Name
#   BlockDialog
# ---------
# Parameter 
#   block - Name des Blocks
# ---------
# Description
#   Die Informationen "uber einen Block werden angezeigt, bzw.
#   lassen sich auch "andern.
proc BlockDialog {block} {
  global PT Dialog Id2Name IsAnnoFrom Blocks

  destroy .bd
  set Dialog(block.name) $block
  set Dialog(block.mean) [$PT(net) Block $block get Mean]
  set Dialog(block.blockvis) [$PT(net) Block $block get BlockVis]
  set Dialog(block.blocks) [$PT(net) Block $block get Blocks]
  set Dialog(block.box)    [$PT(net) Block $block get Box]
  set list ""

  set counter 0
  while {$counter < [llength $Dialog(block.blocks)]} {
    if {$counter == 0} {
      set list [lindex $Dialog(block.blocks) 0]
    } else {
      set list [concat $list,]
      set list [concat $list[lindex $Dialog(block.blocks) $counter]]
    }
    set counter [expr $counter + 1]
  }
  set Dialog(block.blocks) $list

  set t [toplevel .bd]
  wm title $t {Block Dialog}

  frame $t.top -relief raised -bd 2
  pack $t.top -side top -fill both -expand 1 -padx 2m -pady 2m

  message $t.top.msg -text "Block Dialog for\nBlock $block" \
    -aspect 1000 -font -Adobe-Times-Medium-R-Normal--*-180-*-*-*-*-*-* \
    -justify center
  pack $t.top.msg -side top -fill x

  frame $t.top.name
  label $t.top.name.label -text "Name" -width 10 -anchor w
  label $t.top.name.entry -textvariable Dialog(block.name) -anchor w
  pack $t.top.name.label -side left -fill x
  pack $t.top.name.entry -side left -fill x -expand 1
  pack $t.top.name -padx 3m -pady 2m -side top -fill x -expand 1

  frame $t.top.mean
  label $t.top.mean.label -text "Meaning" -width 10 -anchor w
  entry $t.top.mean.entry -textvariable Dialog(block.mean) -relief sunken
  pack $t.top.mean.label -side left -fill x
  pack $t.top.mean.entry -side left -fill x -expand 1
  pack $t.top.mean -padx 3m -pady 2m -side top -fill x -expand 1

  frame $t.top.blocks
  label $t.top.blocks.label -text "Blocks" -width 10 -anchor w
  entry $t.top.blocks.entry -textvariable Dialog(block.blocks) -relief sunken
  pack $t.top.blocks.label -side left -fill x
  pack $t.top.blocks.entry -side left -fill x -expand 1
  pack $t.top.blocks -padx 3m -pady 2m -side top -fill x -expand 1

  frame $t.top.box
  label $t.top.box.label -text "Box" -width 10 -anchor w
  entry $t.top.box.entry -textvariable Dialog(block.box) -relief sunken
  pack $t.top.box.label -side left -fill x
  pack $t.top.box.entry -side left -fill x -expand 1
  pack $t.top.box -padx 3m -pady 2m -side top -fill x -expand 1

  frame $t.top.blockvis -relief groove -bd 2
  label $t.top.blockvis.label -text "Block Visibility"
  frame $t.top.blockvis.radio
  radiobutton $t.top.blockvis.radio.exposed -text "Exposed" \
    -variable Dialog(block.blockvis) -value exposed -anchor w
  radiobutton $t.top.blockvis.radio.hidden -text "Hidden" \
    -variable Dialog(block.blockvis) -value hidden -anchor w -state disabled
  radiobutton $t.top.blockvis.radio.exposed_contents -text "Exposed Contents" \
    -variable Dialog(block.blockvis) -value exposed_contents -anchor w
  pack $t.top.blockvis -fill x -expand 1
  pack $t.top.blockvis.label \
    $t.top.blockvis.radio \
    -fill x -expand 1 -side left
  pack $t.top.blockvis.radio.exposed \
    $t.top.blockvis.radio.hidden \
    $t.top.blockvis.radio.exposed_contents \
    -fill x -expand 1 -side top -anchor w

    # Action Area
  frame $t.action -relief raised -bd 2
  pack $t.action -side bottom -fill both

  frame $t.action.default -relief sunken -bd 1
  button $t.action.default.ok -text "Ok" \
    -command {set Dialog(block.button) Ok}
  pack $t.action.default -side left -expand 1 -padx 3m -pady 2m
  pack $t.action.default.ok 

  button $t.action.cancel -text "Cancel" \
    -command {set Dialog(block.button) Cancel}
  pack $t.action.cancel -side left -expand 1 -padx 3m -pady 2m

  foreach i {name mean} {
    bind $t.top.$i.entry <Return> {set Dialog(Button) Ok}
  }  ;# for i ueber die Eingabefelder
  set old_focus [focus]
  tkwait visibility $t
  grab $t
  focus $t

  tkwait variable Dialog(block.button)
  destroy $t

  if {$Dialog(block.button) == "Cancel"} {
    return 
  }  ;# if Abbruch

  set old_name $block
  set new_name $block

  focus $old_focus
  if {$Dialog(block.name) != $old_name} {
    if {[$PT(net) Exists Block $Dialog(block.name)]} {
      tk_dialog .d {Error} \
        {Block "$Dialog(block.name)" exists already. \
        The new name is not accepted} error 0 {Ok}
    } else {
      $PT(net) Block $block set Name $Dialog(block.name)
      set new_name $Dialog(block.name)
      set c [$PT(canvas) find withtag $old_name]
      $PT(canvas) dtag $old_name
      $PT(canvas) addtag $new_name withtag $c
      set Id2Name($c) $new_name
      if {$PT(NameSwitch)} {
        set n [$PT(canvas) find withtag Name@$old_name]
        unset IsAnnoFrom($n);
        $PT(canvas) delete $n
        CreateName $new_name Block
      }  ;# if Name anzeigen
      if {$PT(MeanSwitch)} {
        set n [$PT(canvas) find withtag Mean@$old_name]
        $PT(canvas) dtag Mean@$old_name
        set IsAnnoFrom($n) $new_name
        $PT(canvas) addtag Mean@$new_name withtag $n
      }  ;# if Meaning angezeigt
    }  ;# else, noch nicht vergeben
  }  ;# if Name des Blocks geaendert
  if {$Dialog(block.mean) != [$PT(net) Block $new_name get Mean]} {
    $PT(net) Block $new_name set Mean $Dialog(block.mean)
    if $PT(MeanSwitch) {
      $PT(canvas) delete Mean@$new_name
      CreateAnno $new_name Block Mean
    }  ;# if Meaning anzeigen
  }  ;# if Meaning des Blocks geaendert
    # Todo: nur bei Aenderung neuberechnen
  if {$Dialog(block.blockvis) != "hidden"} {
    if {$Blocks($new_name) != 1} {
      $PT(net) Block $new_name set BlockVis $Dialog(block.blockvis)
      Repaint
    }
  }  ;# if nicht Hidden gewaehlt

  set Dialog(block.blocks) [split $Dialog(block.blocks) ","]
  $PT(net) Block $new_name set Blocks $Dialog(block.blocks)
  $PT(net) Block $new_name set Box $Dialog(block.box)
}  ;# BlockDialog

# -------------------------------------------------------------
# Name
#   BoxDialog
# ---------
# Parameter 
#   box - Name der Box
# ---------
# Description
#   Die Informationen "uber eine Box werden angezeigt, bzw. wie
#   Meaning auch "andern.
proc BoxDialog {box} {
  global PT Dialog Id2Name IsAnnoFrom

  destroy .bd
  set Dialog(box.name) $box
  set Dialog(box.mean) [$PT(net) Box $box get Mean]
  set Dialog(box.blockvis) [$PT(net) Box $box get Visibility]
  set Dialog(box.blocks) [$PT(net) Box $box get Blocks]
  set list ""

  set counter 0
  while {$counter < [llength $Dialog(box.blocks)]} {
    if {$counter == 0} {
      set list [lindex $Dialog(box.blocks) 0]
    } else {
      set list [concat $list,]
      set list [concat $list[lindex $Dialog(box.blocks) $counter]]
    }
    set counter [expr $counter + 1]
  }
  set Dialog(box.blocks) $list

  set t [toplevel .bd]
  wm title $t {Box Dialog}

  frame $t.top -relief raised -bd 2
  pack $t.top -side top -fill both -expand 1 -padx 2m -pady 2m

  message $t.top.msg -text "Box Dialog for\nBox $box" \
    -aspect 1000 -font -Adobe-Times-Medium-R-Normal--*-180-*-*-*-*-*-* \
    -justify center
  pack $t.top.msg -side top -fill x

  frame $t.top.name
  label $t.top.name.label -text "Name" -width 10 -anchor w
  label $t.top.name.entry -textvariable Dialog(box.name) -anchor w
  pack $t.top.name.label -side left -fill x
  pack $t.top.name.entry -side left -fill x -expand 1
  pack $t.top.name -padx 3m -pady 2m -side top -fill x -expand 1

  frame $t.top.mean
  label $t.top.mean.label -text "Meaning" -width 10 -anchor w
  entry $t.top.mean.entry -textvariable Dialog(box.mean) -relief sunken
  pack $t.top.mean.label -side left -fill x
  pack $t.top.mean.entry -side left -fill x -expand 1
  pack $t.top.mean -padx 3m -pady 2m -side top -fill x -expand 1

  frame $t.top.blocks
  label $t.top.blocks.label -text "Blocks" -width 10 -anchor w
  entry $t.top.blocks.entry -textvariable Dialog(box.blocks) -relief sunken
  pack $t.top.blocks.label -side left -fill x
  pack $t.top.blocks.entry -side left -fill x -expand 1
  pack $t.top.blocks -padx 3m -pady 2m -side top -fill x -expand 1

  frame $t.top.blockvis -relief groove -bd 2
  label $t.top.blockvis.label -text "Block Visibility"
  frame $t.top.blockvis.radio
  radiobutton $t.top.blockvis.radio.exposed -text "Visible" \
    -variable Dialog(box.blockvis) -value 0 -anchor w
  radiobutton $t.top.blockvis.radio.exposed_contents -text "Invisible" \
    -variable Dialog(box.blockvis) -value 4 -anchor w
  pack $t.top.blockvis -fill x -expand 1
  pack $t.top.blockvis.label \
    $t.top.blockvis.radio \
    -fill x -expand 1 -side left
  pack $t.top.blockvis.radio.exposed \
    $t.top.blockvis.radio.exposed_contents \
    -fill x -expand 1 -side top -anchor w

    # Action Area
  frame $t.action -relief raised -bd 2
  pack $t.action -side bottom -fill both

  frame $t.action.default -relief sunken -bd 1
  button $t.action.default.ok -text "Ok" \
    -command {set Dialog(box.button) Ok}
  pack $t.action.default -side left -expand 1 -padx 3m -pady 2m
  pack $t.action.default.ok 

  button $t.action.frame -text "Open Frame" \
    -command "OpenFrame $box"
  pack $t.action.frame -side left -expand 1 -padx 3m -pady 2m
  button $t.action.cancel -text "Cancel" \
    -command {set Dialog(box.button) Cancel}
  pack $t.action.cancel -side left -expand 1 -padx 3m -pady 2m

  foreach i {name mean} {
    bind $t.top.$i.entry <Return> {set Dialog(Button) Ok}
  }  ;# for i ueber die Eingabefelder
  set old_focus [focus]
  tkwait visibility $t
  grab $t
  focus $t

  tkwait variable Dialog(box.button)
  destroy $t

  if {$Dialog(box.button) == "Cancel"} {
    return 
  }  ;# if Abbruch

  set old_name $box
  set new_name $box

  focus $old_focus
  if {$Dialog(box.name) != $old_name} {
    if {[$PT(net) Exists Box $Dialog(box.name)]} {
      tk_dialog .d {Error} \
        {Box "$Dialog(box.name)" exists already. \
        The new name is not accepted} error 0 {Ok}
    } else {
      $PT(net) Box $box set Name $Dialog(box.name)
      set new_name $Dialog(box.name)
      set c [$PT(canvas) find withtag $old_name]
      $PT(canvas) dtag $old_name
      $PT(canvas) addtag $new_name withtag $c
      set Id2Name($c) $new_name
      if {$PT(NameSwitch)} {
        set n [$PT(canvas) find withtag Name@$old_name]
        unset IsAnnoFrom($n);
        $PT(canvas) delete $n
        CreateName $new_name Block
      }  ;# if Name anzeigen
      if {$PT(MeanSwitch)} {
        set n [$PT(canvas) find withtag Mean@$old_name]
        $PT(canvas) dtag Mean@$old_name
        set IsAnnoFrom($n) $new_name
        $PT(canvas) addtag Mean@$new_name withtag $n
      }  ;# if Meaning angezeigt
    }  ;# else, noch nicht vergeben
  }  ;# if Name des Blocks geaendert
  if {$Dialog(box.mean) != [$PT(net) Box $new_name get Mean]} {
    $PT(net) Box $new_name set Mean $Dialog(box.mean)
    if $PT(MeanSwitch) {
      $PT(canvas) delete Mean@$new_name
      CreateAnno $new_name Box Mean
    }  ;# if Meaning anzeigen
  }  ;# if Meaning des Blocks geaendert
    # Todo: nur bei Aenderung neuberechnen
  $PT(net) Box $new_name set Visibility $Dialog(box.blockvis)
  Repaint

  set Dialog(box.blocks) [split $Dialog(box.blocks) ","]
  $PT(net) Box $new_name set Blocks $Dialog(box.blocks)
}  ;# BoxDialog


  # Anzeigen des aktuellen Zustandes der Blocksichtbarkeit fuer den
  # gewaehlten Block.
  # Aus der globale Variable Dialog:
  #   Dialog(blocklist.radiow) - Widget woran sich die RadioButtons anschliessen
  #   Dialog(blocklist.selected) - der Name des Blockes
  #   Dialog(blocklist.blockvis) - wird anhand des gewaehlten Blockes
  #     neu mit Inhalt gefuellt
proc BlockList_BlockVis_Display {} {
  global PT Dialog Id2Name

  #$PT(canvas) itemconfigure BlockMark -fill white
  set blocks [$PT(canvas) find withtag Block]
  foreach b $blocks {
    set n $Id2Name($b)
    catch {
      block$n configure -background white
      $PT(canvas) itemconfigure $b -image block$n
    }
  }
  $PT(canvas) dtag BlockMark BlockMark
  if {$Dialog(blocklist.selected) != {}} {
    set Dialog(blocklist.blockvis) \
      [$PT(net) Block $Dialog(blocklist.selected) get BlockVis]
    foreach rb {exposed exposed_contents} {
      $Dialog(blocklist.radiow).$rb configure -state normal
    }  ;# foreach RadioButton
    $PT(canvas) addtag BlockMark withtag $Dialog(blocklist.selected)
    set n $Dialog(blocklist.selected)
    catch {
      block$n configure -background orange
      $PT(canvas) itemconfigure $n -image block$n
    }
    #$PT(canvas) itemconfigure $Dialog(blocklist.selected) -fill orange
  } else {
    set Dialog(blocklist.blockvis) 0
    foreach rb {exposed exposed_contents} {
      $Dialog(blocklist.radiow).$rb configure -state disabled
    }  ;# foreach RadioButton
  }  ;# else, keine Ausgabe moeglich
}  ;# BlockList_BlockVis_Display

  # Bei Aenderung der Blocksichtbarkeit muss eine Neuberechnung der
  # Sichtbarkeiten der anderen Bloecke erfolgen und dementsprechend
  # die Anzeige der Netzobjekte.
  # all_flag - falls gleich "expose_all", dann alle exposed_contents, 
  #            falls gleich "one", dann nur den selektierten aendern
  # Aus der globale Variable Dialog:
  #   Dialog(blocklist.listw) - Widget der Blockliste
  #   Dialog(blocklist.blocks) - Liste der Blocknamen
  #   Dialog(blocklist.selected) - der Name des Blockes
  #   Dialog(blocklist.blockvis) - neuer Zustand (gewuenschter)
proc BlockList_BlockVis_Change {{all_flag one}} {
  global PT Dialog

  if {($Dialog(blocklist.selected) != {}) || ($all_flag == "expose_all")} {
    if {[$PT(net) Exists Block $Dialog(blocklist.selected)] == 0} {
      return
    }  ;# if Block gibt's ja garnicht
    set old_blockvis [$PT(net) Block $Dialog(blocklist.selected) get BlockVis]
    if {($old_blockvis != $Dialog(blocklist.blockvis))
      || ($all_flag == "expose_all")} {
        # 1. neuen Status in den Kernel bringen
      if {$all_flag == "one"} {
        $PT(net) Block $Dialog(blocklist.selected) set \
          BlockVis $Dialog(blocklist.blockvis)
      } else {
        foreach b $Dialog(blocklist.blocks) {
          $PT(net) Block $b set BlockVis exposed_contents
        }  ;# foreach Block im Netz
      }  ;# else, alle
        # 2. Liste im Fenster updaten
      set line [lsearch $Dialog(blocklist.blocks) $Dialog(blocklist.selected)]
      $Dialog(blocklist.listw) delete 0 end
        # Todo: in Repaint auch nochmal aufgerufen
      $PT(net) RecalcBlockVis
      foreach b $Dialog(blocklist.blocks) {
        set m [$PT(net) Block $b get Mean]
        set bv [$PT(net) Block $b get BlockVis]
        $Dialog(blocklist.listw) insert end "$b - $m ($bv)"
      }  ;# foreach Block im Netz
      $Dialog(blocklist.listw) select set $line $line
      BlockList_BlockVis_Display  ;# sicherheitshalber und ueberhaupt
        # 3. Bild neu aufbauen
      Repaint
    }  ;# if Aktionen sind noetig
  }  ;# if ueberhaupt ein Block gewaehlt oder alle oeffnen
}  ;# BlockList_BlockVis_Change

proc BlockList {} {
  global PT Dialog

    # Menueeintrag abschalten
  .mbar.hierarchy.menu entryconfigure "Block List" -state disabled

    # Toplevel erzeugen
  set t [toplevel .blocklist]
  wm title $t "Block List"

    # Eingabebereich
  frame $t.input -relief raised -bd 2
  pack $t.input -side top -fill both -expand 1 -padx 2m -pady 2m

  frame $t.input.head
  label $t.input.head.label -text "Name - Meaning (Block Visibility)" \
    -anchor w
  set Dialog(blocklist.listw) [listbox $t.input.list -width 40 \
    -yscrollcommand "$t.input.sy set" \
    -xscrollcommand "$t.input.pad.sx set"]
  scrollbar $t.input.sy -orient vertical -command "$t.input.list yview"
  frame $t.input.pad
  scrollbar $t.input.pad.sx -orient horizontal \
    -command "$t.input.pad.sx xview"
  set pad [expr [$t.input.sy cget -width] + 2 * \
    ([$t.input.sy cget -bd] + [$t.input.sy cget -highlightthickness])]
  frame $t.input.pad.it -width $pad -height $pad
  frame $t.input.head.padit -width $pad -height $pad
  pack $t.input.head -side top -fill x
  pack $t.input.head.label -side left -fill x
  pack $t.input.head.padit -side right
  pack $t.input.pad -side bottom -fill x
  pack $t.input.pad.it -side right
  pack $t.input.pad.sx -side bottom -fill x
  pack $t.input.sy -side right -fill y
  pack $t.input.list -side left -fill both -expand 1

    # vorhandene Bloecke anzeigen
  set Dialog(blocklist.blocks) [lsort [$PT(net) get Blocks]]
  foreach b $Dialog(blocklist.blocks) {
    set m [$PT(net) Block $b get Mean]
    set bv [$PT(net) Block $b get BlockVis]
    $t.input.list insert end "$b - $m ($bv)"
  }  ;# foreach Block im Netz
    # Falls mindestens ein Block vorhanden, den ersten anwaehlen.
  if {[llength $Dialog(blocklist.blocks)] != 0} {
    $t.input.list select set 0 0
    set Dialog(blocklist.selected) [lindex $Dialog(blocklist.blocks) 0]
  } else {
    set Dialog(blocklist.selected) {}
  }  ;# else, zur Zeit keine Bloecke
    # Informationen zum selektierten Block anzeigen
  frame $t.disp -relief raised -bd 2
  pack $t.disp -side top -fill both -expand 1 -padx 2m -pady 2m
    # BlockVis-Stuff
  frame $t.disp.blockvis -relief groove -bd 2
  label $t.disp.blockvis.label -text "Block Visibility"
  set Dialog(blocklist.radiow) [frame $t.disp.blockvis.radio]
  radiobutton $t.disp.blockvis.radio.exposed -text "Exposed" \
    -value exposed
  radiobutton $t.disp.blockvis.radio.hidden -text "Hidden" \
   -value hidden -state disabled
  radiobutton $t.disp.blockvis.radio.exposed_contents \
    -text "Exposed Contents" -value exposed_contents
  foreach rb {exposed hidden exposed_contents} {
    $Dialog(blocklist.radiow).$rb configure -anchor w
    $Dialog(blocklist.radiow).$rb configure \
      -variable Dialog(blocklist.blockvis)
    $Dialog(blocklist.radiow).$rb configure \
      -command {BlockList_BlockVis_Change}
  }  ;# foreach RadioButton
  pack $t.disp.blockvis -fill x -expand 1
  pack $t.disp.blockvis.label \
    $t.disp.blockvis.radio \
    -fill x -expand 1 -side left
  pack $t.disp.blockvis.radio.exposed \
    $t.disp.blockvis.radio.hidden \
    $t.disp.blockvis.radio.exposed_contents \
    -fill x -expand 1 -side top -anchor w
  BlockList_BlockVis_Display
    # Binding fuer die ListBox
  bind $t.input.list <Button-1> {
    set line [%W nearest %y]
    set Dialog(blocklist.selected) [lindex $Dialog(blocklist.blocks) $line]
    BlockList_BlockVis_Display
  }  ;# Binding

    # Button-Bereich
  frame $t.action -relief raised -bd 2
  pack $t.action -side bottom -fill both
    # Ok-Button
  button $t.action.ok -text "Ok" \
    -command "
      set Dialog(blocklist.selected) {}
      BlockList_BlockVis_Display
      .mbar.hierarchy.menu entryconfigure {Block List} -state active
      destroy $t
    "
    # Expose all-Button
  button $t.action.exposed_contents_all -text "Exposed Contents All" \
    -command {BlockList_BlockVis_Change expose_all}
    # Cancel-Button
  button $t.action.cancel -text "Cancel" \
    -command "
      set Dialog(blocklist.selected) {}
      BlockList_BlockVis_Display
      .mbar.hierarchy.menu entryconfigure {Block List} -state active
      destroy $t
    "
  pack $t.action.ok \
    $t.action.exposed_contents_all \
    $t.action.cancel \
    -side left -expand 1 -padx 3m -pady 2m
}  ;# BlockList

proc MarkingDialog {place} {
  global PT Dialog Id2Name IsAnnoFrom Place2Show Frames

  destroy .td

  set old_variables [$PT(net) Place $place get Marking]
  set Dialog(Vars) $old_variables

  set t [toplevel .place$place -bd 0]
  wm title $t "$place"
  wm protocol $t WM_DELETE_WINDOW "destroy $t"

  frame $t.top -bd 1 -relief raised
  frame $t.action -bd 1 -relief raised

  pack $t.top -side top -fill both -expand 1
  pack $t.action -side top -fill both -expand 1

  frame $t.top.variables
  label $t.top.variables.entry -width 40 -height 1 -text $Dialog(Vars)
  pack $t.top.variables.entry -side left
  pack $t.top.variables -padx 3m -pady 2m -anchor w
  set Place2Show($place) $t.top.variables.entry
  set Frames($place) $t
}

proc MarkingBox {} {
  global PT Dialog Id2Name IsAnnoFrom

  destroy .td

  set Dialog(Vars) {}

  set t [toplevel .td -bd 0]
  wm title $t {Marking Nodes}

  frame $t.top -bd 1 -relief raised
  frame $t.action -bd 1 -relief raised

  message $t.top.msg -text "Dialog for Marking\nNodes" \
    -aspect 1000 -font -Adobe-Times-Medium-R-Normal--*-180-*-*-*-*-*-* \
    -justify center
  pack $t.top.msg -side top -fill x

  pack $t.top -side top -fill both -expand 1
  pack $t.action -side top -fill both -expand 1

  frame $t.top.variables
  label $t.top.variables.label -text "Nodes" -width 10
  entry $t.top.variables.entry -textvariable Dialog(Vars)
  pack $t.top.variables.entry -side right -expand yes -fill x
  pack $t.top.variables.label -side left
  pack $t.top.variables -padx 3m -pady 2m -anchor w -fill x

  frame $t.action.default -relief sunken -bd 1
  button $t.action.default.ok -text "Ok" \
    -command {set Dialog(Button) Ok}
  pack $t.action.default.ok 
  pack $t.action.default -side left -fill x -padx 3m -pady 2m

  button $t.action.cancel -text Cancel \
    -command {set Dialog(Button) Cancel}
  pack $t.action.cancel -side right -padx 3m -pady 2m -fill x

  bind $t.top.variables.entry <Return> {set Dialog(Button) Ok}

  set old_focus [focus]
  tkwait visibility $t
  grab $t
  focus $t

  tkwait variable Dialog(Button)
  destroy $t

  focus $old_focus

  Highlight $Dialog(Vars)
}  ;# MarkingBox

# -------------------------------------------------------------
# Name
#   FiringBox
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Die beiden Listen (Sequence und Loop) werden editiert
#   und "ubernommen, so da"s bei der n"achsten Ausf"uhrung
#   von Execute oder Animate nur diese Listen genommen werden.
proc FiringBox {} {
  global PT Dialog Id2Name IsAnnoFrom

  destroy .td

  set list ""

  set counter 0
  while {$counter < [llength $PT(node_list)]} {
    if {$counter == 0} {
      set list [lindex $PT(node_list) 0]
    } else {
      set list [concat $list,]
      set list [concat $list[lindex $PT(node_list) $counter]]
    }
    set counter [expr $counter + 1]
  }
  set Dialog(Vars) $list

  set list ""

  set counter 0
  while {$counter < [llength $PT(loop_list)]} {
    if {$counter == 0} {
      set list [lindex $PT(loop_list) 0]
    } else {
      set list [concat $list,]
      set list [concat $list[lindex $PT(loop_list) $counter]]
    }
    set counter [expr $counter + 1]
  }

  set Dialog(Loop) $list

  set t [toplevel .td -bd 0]
  wm title $t {Firing Transition List}

  frame $t.top -bd 1 -relief raised
  frame $t.action -bd 1 -relief raised

  message $t.top.msg -text "Dialog for Firing\nTransition List" \
    -aspect 1000 -font -Adobe-Times-Medium-R-Normal--*-180-*-*-*-*-*-* \
    -justify center
  pack $t.top.msg -side top -fill x

  pack $t.top -side top -fill both -expand 1
  pack $t.action -side top -fill both -expand 1

  frame $t.top.variables
  label $t.top.variables.label -text "Sequence" -width 10
  entry $t.top.variables.entry -textvariable Dialog(Vars)
  frame $t.top.variables2
  label $t.top.variables2.label -text "Loop" -width 10
  entry $t.top.variables2.entry -textvariable Dialog(Loop)
  pack $t.top.variables.entry -side right -expand yes -fill x
  pack $t.top.variables.label -side left
  pack $t.top.variables2.entry -side right -expand yes -fill x
  pack $t.top.variables2.label -side left
  pack $t.top.variables $t.top.variables2 -padx 3m -pady 2m -anchor w -fill x

  frame $t.action.default -relief sunken -bd 1
  button $t.action.default.ok -text "Ok" \
    -command {set Dialog(Button) Ok}
  pack $t.action.default.ok 
  pack $t.action.default -side left -fill x -padx 3m -pady 2m

  button $t.action.cancel -text Cancel \
    -command {set Dialog(Button) Cancel}
  pack $t.action.cancel -side right -padx 3m -pady 2m -fill x

  bind $t.top.variables.entry <Return> {set Dialog(Button) Ok}

  set old_focus [focus]
  tkwait visibility $t
  grab $t
  focus $t

  tkwait variable Dialog(Button)
  destroy $t

  focus $old_focus

  FireTransitionLists -sequence=$Dialog(Vars) -loop=$Dialog(Loop)
}  ;# Execute Transitionlist

# -------------------------------------------------------------
# Name
#   OpenFrame
# ---------
# Parameter 
#   box - Name der Box
# ---------
# Description
#   Es wird ein neues Fenster ge"offnet, in dem alle Objekte, die
#   zu dieser Box geh"oren, dargestellt werden.
proc OpenFrame {box} {
  global PT Id2Name Dialog Frames Blocks

  set blocks [$PT(net) get Boxes]
  set places [$PT(net) get Places]
  set transitions [$PT(net) get Transitions]
  set ptransitions [$PT(net) get PTransitions]
  set visblocks ""
  set visplaces ""
  set vistransitions ""
  set visptransitions ""

  foreach p $places {
    set BL [$PT(net) Place $p get Box]
    if {[lsearch -regexp $BL $box] > -1} {
      lappend visplaces $p
    }
  }
  foreach t $transitions {
    set BL [$PT(net) Transition $t get Box]
    if {[lsearch -regexp $BL $box] > -1} {
      lappend vistransitions $t
    }
  }
  destroy .boxframe$box
  toplevel .boxframe$box
  wm title .boxframe$box "Box $box"
  wm protocol .boxframe$box WM_DELETE_WINDOW "destroy .boxframe$box;set Blocks($box) 0"
  frame .boxframe$box.mbar -relief raised -bd 2 
  frame .boxframe$box.fenster
  frame .boxframe$box.status

  menubutton .boxframe$box.mbar.edit -text Edit -menu .boxframe$box.mbar.edit.menu
  menu .boxframe$box.mbar.edit.menu -tearoff 0
  .boxframe$box.mbar.edit.menu add command -label "Cut"
  .boxframe$box.mbar.edit.menu add command -label "Copy"
  .boxframe$box.mbar.edit.menu add command -label "Paste"
  .boxframe$box.mbar.edit.menu add separator
  .boxframe$box.mbar.edit.menu add command -label "Repaint Frame" \
    -command RepaintFrame

  pack .boxframe$box.mbar .boxframe$box.mbar.edit -side left

  scrollbar .boxframe$box.fenster.yscroll -command ".boxframe$box.fenster.canvas yview" \
    -relief sunken -orient vertical -width 12
  frame .boxframe$box.fenster.pad
  scrollbar .boxframe$box.fenster.pad.xscroll -command ".boxframe$box.fenster.canvas xview" \
    -relief sunken -orient horizontal -width 12
  set pad [expr [.boxframe$box.fenster.yscroll cget -width] + 2 * \
    ([.boxframe$box.fenster.yscroll cget -bd] + \
     [.boxframe$box.fenster.yscroll cget -highlightthickness])]
  frame .boxframe$box.fenster.pad.it -width $pad -height $pad
  canvas .boxframe$box.fenster.canvas -width 500 -height 500 \
    -scrollregion {0 0 6000 6000} \
    -xscrollcommand ".boxframe$box.fenster.pad.xscroll set" \
    -yscrollcommand ".boxframe$box.fenster.yscroll set" \
    -relief sunken -borderwidth 3
  button .boxframe$box.status.quit -text "Close Frame" -command {
                                                    if {$PT(canvas) == ".drawing.canvas"} {
                                                      return
                                                    }                                                    
                                                    set frame [string trim $PT(canvas) ".fenster.canvas"]
                                                    destroy .$frame
                                                    set box [string trim $frame ".boxframe"]
                                                    set Blocks($box) 0}
  pack .boxframe$box.status.quit -side right
  pack .boxframe$box.status -side bottom
  pack .boxframe$box.fenster.pad -side bottom -fill x
  pack .boxframe$box.fenster.pad.it -side right
  pack .boxframe$box.fenster.pad.xscroll -side bottom -fill x
  pack .boxframe$box.fenster.yscroll -side right -fill y
  pack .boxframe$box.fenster.canvas -side left -expand yes -fill both
  pack .boxframe$box.mbar .boxframe$box.fenster -expand yes -fill both -side top
  pack .boxframe$box.status -expand yes -fill both -side bottom

  set old $PT(canvas)
  set PT(canvas) .boxframe$box.fenster.canvas
  set Frames($PT(canvas)) $box
  set Blocks($box) 1
  RepaintFrame
  set PT(canvas) $old
}  ;# OpenFrame

# -------------------------------------------------------------
# Name
#   Find
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Wie in jedem Tool existiert nun ein Fenster, in dem mit Hilfe
#   des Namens nach Objekten gesucht werden kann.
proc Find {} {
  global PT Dialog

  set t [toplevel .td -bd 0]
  wm title $t {Find Dialog}

  frame $t.top -bd 1 -relief raised
  frame $t.action -bd 1 -relief raised

  message $t.top.msg -text "Find Item" \
    -aspect 1000 -font -Adobe-Times-Medium-R-Normal--*-180-*-*-*-*-*-* \
    -justify center
  pack $t.top.msg -side top -fill x

  pack $t.top -side top -fill both -expand 1
  pack $t.action -side top -fill both -expand 1

  frame $t.top.name 
  label $t.top.name.label -text "Name"
  entry $t.top.name.entry -textvariable Dialog(Name) -relief sunken
  pack $t.top.name.entry -side right -fill x -expand 1
  pack $t.top.name.label -side left -padx 3m
  pack $t.top.name -padx 3m -pady 2m -side top -anchor w -fill x

#  frame $t.top.mean
#  label $t.top.mean.label -text "Meaning" -width 15
#  entry $t.top.mean.entry -textvariable Dialog(Mean) -relief sunken
#  pack $t.top.mean.entry -side right -fill x -expand yes
#  pack $t.top.mean.label -side left
#  pack $t.top.mean -padx 3m -pady 2m -side top -anchor w -fill x

#  frame $t.top.ref
#  label $t.top.ref.label -text "Reference" -width 15
#  entry $t.top.ref.entry -textvariable Dialog(Ref) -relief sunken
#  pack $t.top.ref.entry -side right -expand yes -fill x
#  pack $t.top.ref.label -side right 
#  pack $t.top.ref -padx 3m -pady 2m -side top -anchor e -fill x

#  frame $t.top.blocks
#  label $t.top.blocks.label -text "Blocks" -width 15
#  entry $t.top.blocks.entry -textvariable Dialog(blocks) -relief sunken
#  pack $t.top.blocks.entry -side right -fill x -expand yes
#  pack $t.top.blocks.label -side left
#  pack $t.top.blocks -padx 3m -pady 2m -side top -anchor w -fill x

#  frame $t.top.box
#  label $t.top.box.label -text "Box" -width 15
#  entry $t.top.box.entry -textvariable Dialog(box) -relief sunken
#  pack $t.top.box.entry -side right -expand yes -fill x
#  pack $t.top.box.label -side right
#  pack $t.top.box -padx 3m -pady 2m -side top -anchor e -fill x

#  frame $t.top.init_mark 
#  label $t.top.init_mark.label -text "Initial Marking" -width 15
#  entry $t.top.init_mark.entry -textvariable Dialog(InitMarking) \
#   -relief sunken
#  pack $t.top.init_mark.entry -side right -fill x -expand yes
#  pack $t.top.init_mark.label -side left
#  pack $t.top.init_mark -padx 3m -pady 2m -side top -anchor w -fill x

#  frame $t.top.mark 
#  label $t.top.mark.label -text "Current Marking" -width 15
#  entry $t.top.mark.entry -textvariable Dialog(Marking) -relief sunken
#  pack $t.top.mark.entry -side right -fill x -expand yes
#  pack $t.top.mark.label -side left
#  pack $t.top.mark -padx 3m -pady 2m -side top -anchor w -fill x
# 
#  frame $t.top.state -bd 1 -relief sunken
#  label $t.top.state.label -text "Status" -anchor center

#  frame $t.top.state.buttons
#  radiobutton $t.top.state.buttons.entry -text Entry \
#    -variable Dialog(State) -value entry -anchor w
#  radiobutton $t.top.state.buttons.internal -text Internal \
#    -variable Dialog(State) -value internal -anchor w
#  radiobutton $t.top.state.buttons.exit -text Exit \
#    -variable Dialog(State) -value exit -anchor w
#  pack $t.top.state.buttons.entry \
#    $t.top.state.buttons.internal \
#    $t.top.state.buttons.exit \
#    -side top -anchor w
#  pack $t.top.state.buttons -side right -fill x
#  pack $t.top.state.label -side left
#  pack $t.top.state -padx 3m -pady 2m -side top

    # Action Area
  frame $t.action.default -relief sunken -bd 1
  button $t.action.default.ok -text "Go To" \
    -command FindAnd
  pack $t.action.default.ok 
  pack $t.action.default -side left -fill x -padx 3m -pady 2m

  button $t.action.cancel -text Cancel \
    -command "destroy $t;return"
  pack $t.action.cancel -side right -padx 3m -pady 2m -fill x

}  ;

proc FindAnd {} {
  global Dialog PT

  set Dialog(Button) Ok
  set block 1
  if {[$PT(net) Exists Transition $Dialog(Name)]} {
    set x [$PT(net) Transition $Dialog(Name) get x]
    set y [$PT(net) Transition $Dialog(Name) get y]
  } elseif {[$PT(net) Exists Place $Dialog(Name)]} {
    set x [$PT(net) Place $Dialog(Name) get x]
    set y [$PT(net) Place $Dialog(Name) get y]
  } elseif {[$PT(net) Exists Block $Dialog(Name)]} {
    set x [$PT(net) Block $Dialog(Name) get x]
    set y [$PT(net) Block $Dialog(Name) get y]
    set block 0
  } elseif {[$PT(net) Exists PTransition $Dialog(Name)]} {
    set x [$PT(net) PTransition $Dialog(Name) get x]
    set y [$PT(net) PTransition $Dialog(Name) get y]
  } else {
    return
  }
  set x [expr $x * $PT(Zoom)]
  set y [expr $y * $PT(Zoom)]
  set unitsX [.drawing.pad.xscroll get]
  set unitsY [.drawing.yscroll get]
  set relX [expr $x / 6000.0]
  set relY [expr $y / 6000.0]
  set MidX [expr ([lindex $unitsX 1] - [lindex $unitsX 0]) / 2]
  set MidY [expr ([lindex $unitsY 1] - [lindex $unitsY 0]) / 2]
  set Rx [expr [lindex $unitsX 0] + $relX - $MidX]
  set Ry [expr [lindex $unitsY 0] + $relY - $MidY]
  set AbsX [expr $Rx - [lindex $unitsX 0]]
  set AbsY [expr $Ry - [lindex $unitsY 0]]
  .drawing.canvas xview moveto $AbsX
  .drawing.canvas yview moveto $AbsY
  set obj [$PT(canvas) find withtag $Dialog(Name)]
  if {$block} {
    $PT(canvas) itemconfigure $obj -fill orange
    update
    after 1000
    $PT(canvas) itemconfigure $obj -fill white
  } else {
    block$Dialog(Name) configure -background orange
    $PT(canvas) itemconfigure $obj -image block$Dialog(Name)
    update
    after 1000
    block$Dialog(Name) configure -background white
    $PT(canvas) itemconfigure $obj -image block$Dialog(Name)
  }
} ;# FindAnd

proc TemplateDirectory {} {
  global PT Dialog

  set t [toplevel .td -bd 0]
  wm title $t {Template Directory}

  frame $t.top -bd 1 -relief raised
  frame $t.action -bd 1 -relief raised

  message $t.top.msg -text "Template Directory" \
    -aspect 1000 -font -Adobe-Times-Medium-R-Normal--*-180-*-*-*-*-*-* \
    -justify center
  pack $t.top.msg -side top -fill x

  pack $t.top -side top -fill both -expand 1
  pack $t.action -side top -fill both -expand 1

  frame $t.top.name 
  label $t.top.name.label -text "Template Directory" -width 17
  entry $t.top.name.entry -textvariable PT(my_macros) -relief sunken -width 40
  set old $PT(my_macros)
  pack $t.top.name.entry -side right -fill x -expand yes
  pack $t.top.name.label -side left 
  pack $t.top.name -padx 3m -pady 2m -side top -anchor w -fill x

    # Action Area
  frame $t.action.default -relief sunken -bd 1
  button $t.action.default.ok -text "Ok" \
    -command "destroy $t;return"
  pack $t.action.default.ok 
  pack $t.action.default -side left -fill x -padx 3m -pady 2m

  button $t.action.cancel -text Cancel \
    -command "destroy $t;set PT(my_macros) $old;return"
  pack $t.action.cancel -side right -padx 3m -pady 2m -fill x
}

# -------------------------------------------------------------
# Name
#   FrameBindings
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Dies sind die EditBindings innerhalb eines Frames.
proc FrameBindings {} {
  global PT Id2Name

   # bind $PT(canvas) <1> DeselectAll
  bind $PT(canvas) <Enter> {set PT(canvas) %W;FrameBindings}
  $PT(canvas) bind Deletable <1> {Select %x %y;FrameBindings}
  $PT(canvas) bind Edge <1> {Select %x %y;FrameBindings}
  $PT(canvas) bind PActive <1> {Select %x %y;FrameBindings}
  $PT(canvas) bind Deletable <Shift-Button-1> {AddSelection %x %y;FrameBindings}
  $PT(canvas) bind Node <B1-Motion> {Move %x %y;FrameBindings}
  $PT(canvas) bind Node <ButtonRelease-1> {FrameStopMove;FrameBindings}
  $PT(canvas) bind Edge <B1-Motion> {MoveEdge %x %y;FrameBindings}
  $PT(canvas) bind Area <1> {AreaMoveStart %x %y;FrameBindings}
  $PT(canvas) bind Area <B1-Motion> {AreaMove %x %y;FrameBindings}
  $PT(canvas) bind Area <ButtonRelease-1> {StopMove;FrameBindings}
  $PT(canvas) bind Annotation <1> {AnnotationMoveStart %x %y;FrameBindings}
  $PT(canvas) bind Annotation <B1-Motion> {AnnotationMove %x %y;FrameBindings}
  $PT(canvas) bind NodeAnnotation <ButtonRelease-1> {AnnotationStopMove %x %y;FrameBindings}
  $PT(canvas) bind EdgeAnnotation <ButtonRelease-1> {EdgeAnnoStopMove %x %y;FrameBindings}
  $PT(canvas) bind Block <Double-1> {
    BlockDialog $Id2Name([$PT(canvas) find withtag current])
    FrameBindings
  }
  $PT(canvas) bind Place <Double-1> {
    PlaceDialog $Id2Name([$PT(canvas) find withtag current])
    FrameBindings
  }
  $PT(canvas) bind Place m {
    set place $Id2Name([$PT(canvas) find withtag current])
    DelAllTokens $place
    $PT(net) Place $place set Marking \
      [expr 1+[$PT(net) Place $place get Marking]]
    LayoutTokens $place
    $PT(net) clearActive
  }
  $PT(canvas) bind Place n {
    set place $Id2Name([$PT(canvas) find withtag current])
    if {[set num [expr [$PT(net) Place $place get Marking]-1]]>=0} {
      DelAllTokens $place
      $PT(net) Place $place set Marking $num
      LayoutTokens $place
    }
    $PT(net) clearActive
  }
  $PT(canvas) bind Transition <Double-1> {
    TransDialog Transition $Id2Name([$PT(canvas) find withtag current])
    FrameBindings
  }
  $PT(canvas) bind PTransition <Double-1> {
    TransDialog PTransition $Id2Name([$PT(canvas) find withtag current])
    FrameBindings
  }
  $PT(canvas) bind Edge <Double-1> {
    set edge [$PT(canvas) find withtag current]
    set tags [$PT(canvas) gettags $edge]
    if {[lsearch $tags Deletable] > -1} {
      EdgeDialog $Id2Name([$PT(canvas) find withtag current])
    }
    FrameBindings
  }
  $PT(canvas) bind Transition <Button-2> {
    ShowPhantomRelation Transition $Id2Name([$PT(canvas) find withtag current])
    FrameBindings
  }
  $PT(canvas) bind PTransition <Button-2> {
    ShowPhantomRelation PTransition $Id2Name([$PT(canvas) find withtag current])
    FrameBindings
  }
  bind $PT(canvas) p {NewPlace %x %y;FrameBindings}
  bind $PT(canvas) t {NewTransition %x %y;FrameBindings}
# Todo: Ist die naechste Zeile ausreichend fuer das Verschiebungsproblem?
# NEIN:dann neues Problem bind $PT(canvas) a {UnsetAllBindings; EdgeBindings}
  bind $PT(canvas) a {EdgeBindings}
  bind $PT(canvas) r {RepaintFrame;FrameBindings}
  bind $PT(canvas) <Delete> {DeleteElements;FrameBindings}
  bind $PT(canvas) <2> {if {$PT(stroke) == 1} {StrokeStop} else {AreaStart %x %y};FrameBindings }
  bind $PT(canvas) <B2-Motion> {AreaStroke %x %y;FrameBindings}
  bind $PT(canvas) <c> {CreateTitle %x %y;FrameBindings}
}  ;# FrameBindings


proc FrameFireBindings {c} {
  global PT

  #UnsetAllBindings
  $c bind Active <1> FireTransition
  $c bind PActive <1> FirePTransition
  bind $c <Escape> { EndFire }
}

# -------------------------------------------------------------
# Name
#   RepaintFrame
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Die Bildinformationen innerhalb eines Frames werden neu
#   gezeichnet.
proc RepaintFrame {} {
  global PT Id2Name Frames

    # Blockhierarchie aufbauen
  $PT(net) RecalcRootBlocks
  $PT(net) RecalcBlockVis

  set visptransitions ""
  set visblocks [$PT(net) Box $Frames($PT(canvas)) get Blocks]
      
  set visplaces [$PT(net) get Places]
  set visp ""
  foreach vis $visplaces {
    if {[$PT(net) Place $vis get Box] == $Frames($PT(canvas))} {
      lappend visp $vis
    }
  }
  set visplaces $visp
  
  set vistransitions [$PT(net) get Transitions]
  set vist ""
  foreach vis $vistransitions {
    if {[$PT(net) Transition $vis get Box] == $Frames($PT(canvas))} {
      lappend vist $vis
    }
  }
  set vistransitions $vist

  $PT(canvas) delete all
  DrawGrid
  $PT(canvas) xview moveto 0.0
  $PT(canvas) yview moveto 0.0

  #$PT(ModeW) configure -text "Create Transitions ..."
  update
  foreach t $vistransitions {
    MkTransition [expr [$PT(net) Transition $t get x] * $PT(Zoom) - $PT(OSizeHalf)] \
      [expr [$PT(net) Transition $t get y] * $PT(Zoom) - $PT(OSizeHalf)] $t
  }  ;# foreach Transition im Netz

  #$PT(ModeW) configure -text "Create Phantom Transitions ..."
  update
  foreach t $visptransitions {
    MkPTransition [expr [$PT(net) PTransition $t get x] * $PT(Zoom) - $PT(OSizeHalf)] \
      [expr [$PT(net) PTransition $t get y] * $PT(Zoom) - $PT(OSizeHalf)] $t
  }  ;# foreach Phantomtransition im Netz

  #$PT(ModeW) configure -text "Create Places ..."
  update
  foreach p $visplaces {
    MkPlace [expr [$PT(net) Place $p get x] * $PT(Zoom) - $PT(OSizeHalf)] \
      [expr [$PT(net) Place $p get y] * $PT(Zoom) - $PT(OSizeHalf)] $p
  }  ;# foreach Stelle im Netz

  #$PT(ModeW) configure -text "Create Blocks ..."
  update
  foreach b $visblocks {
    MkBlock [expr [$PT(net) Block $b get x] * $PT(Zoom)] \
      [expr [$PT(net) Block $b get y] * $PT(Zoom)] $b
  }  ;# foreach Block im Netz

  #$PT(ModeW) configure -text "Create Arcs & Tokens ..."
  update
  set factor [expr $PT(Zoom) / $PT(OldZoom)]
    # weiter mit Kanten und Markierungen
  foreach p $visplaces {
      # Kanten erstellen
    set in [$PT(net) Place $p get InEdges]
    foreach ie $in {
      if {[lsearch -regexp $visplaces $p] > -1 && \
          [lsearch -regexp $vistransitions [$PT(net) Edge $ie get Src]] > -1} {
        if {![$PT(net) Edge $ie get Visibility]} {    
          if $PT(InvisibleArcsOn) {
            MkEdge [$PT(net) Edge $ie get Src] $p $ie
          }
        } else {
          MkEdge [$PT(net) Edge $ie get Src] $p $ie
        }  ;# else, auf jeden Fall anzeigen
      } else {
        if {[lsearch -regexp $visplaces $p] > -1 && \
            [lsearch -regexp $visblocks [$PT(net) Edge $ie get BlockSrc]] > -1} {
          if $PT(InvisibleArcsOn) {
            MkBlockEdge [$PT(net) Edge $ie get BlockSrc] $p $ie
          }  ;# if unsichtbare Kanten doch sichtbar
        } else {
          MkBlockEdge [$PT(net) Edge $ie get BlockSrc] $p $ie
        }  ;# else, auf jeden Fall anzeigen
      }  ;# else nicht sichtbar oder Blockkante
    }  ;# foreach eingehende Kante
    set out [$PT(net) Place $p get OutEdges]
    foreach oe $out {
      if {[lsearch -regexp $visplaces $p] > -1 && \
          [lsearch -regexp $vistransitions [$PT(net) Edge $oe get Dest]] > -1} {
        if {![$PT(net) Edge $oe get Visibility]} {  
          if $PT(InvisibleArcsOn) {
            MkEdge $p [$PT(net) Edge $oe get Dest] $oe
          }  ;# if unsichtbare Kanten doch sichtbar
        } else { 
          MkEdge $p [$PT(net) Edge $oe get Dest] $oe
        }  ;# else, auf jeden Fall anzeigen
      } else {
        if {[lsearch -regexp $visplaces $p] > -1 && \
            [lsearch -regexp $visblocks [$PT(net) Edge $oe get BlockDest]] > -1} {
          if $PT(InvisibleArcsOn) {
            MkBlockEdge $p [$PT(net) Edge $oe get BlockDest] $oe
          }  ;# if unsichtbare Kanten doch sichtbar
        } else { 
          MkBlockEdge $p [$PT(net) Edge $oe get BlockDest] $oe
        }  ;# else, auf jeden Fall anzeigen
      }  ;# else nicht sichtbar oder Blockkante
    }  ;# foreach ausgehende Kante
      # Markierung der Stelle
    LayoutTokens $p
  }  ;# foreach sichtbare Stelle im Netz
    # Kanten von SyncTrans zu Bloecken
  if {$PT(SyncTransSwitch) == 1} {
    foreach t $vistransitions {
      if {[$PT(net) Transition $t get Channel0Sync] == "yes"} {
        set inedges [$PT(net) Transition $t get InEdges]
        foreach ie $inedges {
          if {[$PT(net) Edge $ie get BlockVis] == "hidden"} {
            MkBlockEdge [$PT(net) Edge $ie get BlockSrc] $t $ie
          }  ;# if eigentlich nicht sichtbar
        }  ;# foreach Eingangskante
        set outedges [$PT(net) Transition $t get OutEdges]
        foreach oe $outedges {
          if {[$PT(net) Edge $oe get BlockVis] == "hidden"} {
            MkBlockEdge $t [$PT(net) Edge $oe get BlockDest] $oe
          }  ;# if eigentlich nicht sichtbar
        }  ;# foreach Ausgangskante
      }  ;# if eine SyncTrans liegt vor
    }  ;# foreach t ueber die sichtbaren Transitionen
  }  ;# if SyncTrans anzeigen

  #$PT(ModeW) configure -text "Create Phantom Arcs ..."
  update
    # weiter mit Phantomkanten
  set places [$PT(net) get Places]
  foreach p $places {
    if {[$PT(net) Place $p get BlockVis] == "exposed"} {
        # Phantomkanten
      set in [$PT(net) Place $p get InPEdges]
      foreach ie $in {
        if {[$PT(net) PEdge $ie get BlockVis] == "exposed"} {
          if {[lsearch -regexp $visplaces $p] > -1 && \
            [lsearch -regexp $vistransitions [$PT(net) Edge $ie get Dest]] > -1} {
            if $PT(InvisibleArcsOn) {
              MkPEdge [$PT(net) PEdge $ie get Src] $p $ie
            }  ;# if unsichtbare Phantomkanten doch sichtbar
          } else {
            MkPEdge [$PT(net) PEdge $ie get Src] $p $ie
          }  ;# else, auf jeden Fall anzeigen
        }  ;# sichtbare Phantomkante
      }  ;# foreach eingehende Phantomkante
      set out [$PT(net) Place $p get OutPEdges]
      foreach oe $out {
        if {[$PT(net) PEdge $oe get BlockVis] == "exposed"} {
          if {[lsearch -regexp $visplaces $p] > -1 && \
            [lsearch -regexp $vistransitions [$PT(net) Edge $ie get Dest]] > -1} {
            if $PT(InvisibleArcsOn) {
              MkPEdge $p [$PT(net) PEdge $oe get Dest] $oe
            }  ;# if unsichtbare Phantomkanten doch sichtbar
          } else { 
            MkPEdge $p [$PT(net) PEdge $oe get Dest] $oe
          }  ;# else, auf jeden Fall anzeigen
        }  ;# if sichtbare Phantomkante
      }  ;# foreach ausgehende Phantomkante
    }  ;# if Stelle sichtbar
  }  ;# foreach Stelle im Netz

  AdjustCanvasScrollRegion

  DisplayTitles
  FrameBindings
}  ;# RepaintFrame

# -------------------------------------------------------------
# Name
#   FrameStopMove
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Der Mousebutton wurde losgelassen, so da"s nun die die Kanten
#   und alle anderen Informationen upgedatet werden.
proc FrameStopMove {} {
  global PT Id2Name Frames

  set visptransitions ""
  set visblocks [$PT(net) Box $Frames($PT(canvas)) get Blocks]
      
  set visplaces [$PT(net) get Places]
  set visp ""
  foreach vis $visplaces {
    if {[$PT(net) Place $vis get Box] == $Frames($PT(canvas))} {
      lappend visp $vis
    }
  }
  set visplaces $visp
  
  set vistransitions [$PT(net) get Transitions]
  set vist ""
  foreach vis $vistransitions {
    if {[$PT(net) Transition $vis get Box] == $Frames($PT(canvas))} {
      lappend vist $vis
    }
  }
  set vistransitions $vist

  set items_to_move [$PT(canvas) find withtag selected]
  foreach item $items_to_move {
    set tags [$PT(canvas) gettags $item]
    set index [lsearch -regexp $tags Block|Place|Transition|PTransition]
    if {$index > -1} {
      set type [lindex $tags $index]
      set coords [$PT(canvas) coords $item]
      set area [$PT(canvas) find withtag Area]
      $PT(net) $type $Id2Name($item) set x [expr int(([lindex $coords 0] + $PT(ObjectSize)) / $PT(Zoom))]
      $PT(net) $type $Id2Name($item) set y [expr int(([lindex $coords 1] + $PT(ObjectSize)) / $PT(Zoom))]
      foreach edge [$PT(net) $type $Id2Name($item) get InEdges] {
        if {$type == "Place"} {
          if {[lsearch -regexp $visplaces [$PT(net) Edge $edge get Dest]] > -1 && \
              [lsearch -regexp $vistransitions [$PT(net) Edge $edge get Src]] > -1} {
            SetupEdge [$PT(net) Edge $edge get Src] \
              [$PT(net) Edge $edge get Dest] $edge
            SetupEdgeAnno $edge
          }
        } elseif {$type == "Transition"} {
          if {[lsearch -regexp $visplaces [$PT(net) Edge $edge get Src]] > -1 && \
              [lsearch -regexp $vistransitions [$PT(net) Edge $edge get Dest]] > -1} {
            SetupEdge [$PT(net) Edge $edge get Src] \
              [$PT(net) Edge $edge get Dest] $edge
            SetupEdgeAnno $edge
          }
        } elseif {$type != "Block"} {
          SetupBlockEdges [$PT(net) Edge $edge get BlockSrc] \
            $Id2Name($item) $edge
        } else {
          SetupBlockEdges [$PT(net) Edge $edge get Src] $Id2Name($item) $edge
        }  ;# else, Blockkante, bewegtes Objekt ist ein Block
      }  ;# foreach Kante
      foreach edge [$PT(net) $type $Id2Name($item) get OutEdges] {
        if {$type == "Place"} {
          if {[lsearch -regexp $visplaces [$PT(net) Edge $edge get Src]] > -1 && \
              [lsearch -regexp $vistransitions [$PT(net) Edge $edge get Dest]] > -1} {
            SetupEdge [$PT(net) Edge $edge get Src] \
              [$PT(net) Edge $edge get Dest] $edge
            SetupEdgeAnno $edge
          }
        } elseif {$type == "Transition"} {
          if {[lsearch -regexp $visplaces [$PT(net) Edge $edge get Dest]] > -1 && \
              [lsearch -regexp $vistransitions [$PT(net) Edge $edge get Src]] > -1} {
            SetupEdge [$PT(net) Edge $edge get Src] \
              [$PT(net) Edge $edge get Dest] $edge
            SetupEdgeAnno $edge
          }
        } elseif {$type != "Block"} {
          SetupBlockEdges $Id2Name($item) \
            [$PT(net) Edge $edge get BlockDest] $edge
        } else {
          SetupBlockEdges $Id2Name($item) [$PT(net) Edge $edge get Dest] $edge
        }  ;# else, Blockkante, bewegtes Objekt ist ein Block
      }  ;# foreach Kante
      foreach edge [$PT(net) $type $Id2Name($item) get InPEdges] {
        if {[$PT(net) PEdge $edge get BlockVis] == "exposed"} {
          SetupPEdge [$PT(net) PEdge $edge get Src] \
            [$PT(net) PEdge $edge get Dest] $edge
        }  ;# if Phantomkante sichtbar
      }  ;# foreach Phantomkante
      foreach edge [$PT(net) $type $Id2Name($item) get OutPEdges] {
        if {[$PT(net) PEdge $edge get BlockVis] == "exposed"} {
          SetupPEdge [$PT(net) PEdge $edge get Src] \
            [$PT(net) PEdge $edge get Dest] $edge
        }  ;# if Phantomkante sichtbar
      }  ;# foreach Phantomkante
      if {$type == "Place"} {
          # evtl. Token mitverschieben
        LayoutTokens $Id2Name($item)
      }  ;# if Stellen brauchen natuerlich wieder eine Sonderbehandlung
      LayoutAnnotations $Id2Name($item) $type
    }  ;# if passendes Element aus Block|Place|Transition|PTransition
  }  ;# foreach zu bewegendes Objekt
  $PT(canvas) bind Node <B1-Release> {}
  NetChanged
}  ;# FrameStopMove


##  AskForFont
##-----------------------------------------------------------
##  testet direkt, ob ein Font existiert. Diese Prozedur ist
##  nicht in 'ExistFont' integriert, weil sie auch von C aus,
##  beim Laden eines FA aufgerufen wird.
##-----------------------------------------------------------
##
proc AskForFont { newFont } {
   global PT

   if {[catch {.fr2bo.entry1 configure \
		   -font $newFont}] == 0} {
       .fr2bo.entry1 configure\
	   -font $PT(AnnoFont)
       return 1
   } else {
       .fr2bo.entry1 configure\
	   -font $PT(AnnoFont)
       return 0
   }
}

proc ChangeAnnotation {type} {
    global PT fontType
    
    append font "-*-$fontType(family)-$fontType(weight)-"
    append font "$fontType(slant)-Normal--*-$fontType(size)-*"

    set PT($type) $font
    switch $type {
         namePlace   {UpdateNames;$PT(net) set NameP $font}
         nameTrans   {UpdateNames;$PT(net) set NameT $font}
         meanPlace   {UpdateMeanings;$PT(net) set MeaningP $font}
         meanTrans   {UpdateMeanings;$PT(net) set MeaningT $font}
         edge        {UpdateEdgeAnno;$PT(net) set AnnotationsA $font}
         actionTerms {UpdateAnno Transition AT;$PT(net) set ActionTermT $font}
         valueTerms  {UpdateAnno Transition VT;$PT(net) set ValueTermT $font}
         types       {UpdateAnno Place Type;$PT(net) set TypeP $font}
         comments    {set PT(TitleFont) $font;$PT(net) set Comments $font}
    }
}  ;# ChangeAnnotation

# -------------------------------------------------------------
# Name
#   TransitionDialog
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Es wird ein Fenster aufgebaut, in dem alle aktiven Transitionen
#   dargestellt werden, so dass sie geschaltet werden k"onnen.
#   Zus"atzlich besteht die M"oglichkeit, das canvas auf diese Knoten
#   zu orientieren. Der Editor wechselt in den Executemodus.
proc TransitionDialog {} {
  global PT Dialog Id2Name IsAnnoFrom

  destroy .td

  set t [toplevel .td -bd 0]
  wm title .td {Transition Dialog}

  frame $t.top -bd 1 -relief raised
  frame $t.action -bd 1 -relief raised
  pack $t.top $t.action -side top -fill both -expand yes

  message $t.top.msg -text "Execution Dialog for Transitions" \
    -aspect 1000 -font -Adobe-Times-Medium-R-Normal--*-180-*-*-*-*-*-* \
    -justify center

  pack $t.top.msg -side top -fill both -expand yes

  frame $t.top.list 
  frame $t.top.list.vars -bd 1 -relief sunken
  pack $t.top.list.vars -side left -expand yes -fill x
  label $t.top.list.vars.label -text "Active Transitions"

  frame $t.top.list.vars.f
  pack $t.top.list.vars.f -side bottom

  listbox $t.top.list.vars.f.l \
    -yscrollcommand "$t.top.list.vars.f.scroll set" -height 10
  bind $t.top.list.vars.f.l <1> {global Dialog
                                 set Dialog(Trans) [%W get @%x,%y]
                                 catch {
                                   set Dialog(Bind) [$PT(net) Transition $Dialog(Trans) get VT]
                                   $PT(canvas) itemconfigure [$PT(canvas) find withtag $Dialog(Trans)] -fill orange
                                 }
                                 catch {
				   if {$Dialog(Trans)!=$Dialog(Last)} {
                                   if {[$PT(net) Transition $Dialog(Last) get IsActive]} {
                                     $PT(canvas) itemconfigure [$PT(canvas) find withtag $Dialog(Last)] -fill yellow
                                   } else {
                                     $PT(canvas) itemconfigure [$PT(canvas) find withtag $Dialog(Last)] -fill white
                                   }
				   }
                                 }
                                 set Dialog(Last) $Dialog(Trans)
                                }
  scrollbar $t.top.list.vars.f.scroll -command "$t.top.list.vars.f.l yview"
  pack $t.top.list.vars.label -side top
  pack $t.top.list.vars.f.l -side left
  pack $t.top.list.vars.f.scroll -side left -fill y

  pack $t.top.list -side top -expand yes -fill x
  entry $t.top.bind -relief sunken -textvariable Dialog(Bind)
  pack $t.top.bind -side bottom -expand yes -fill x
  set liste [$PT(net) get ActiveTransitions]
  set counter [llength $liste]
  if {$counter > $PT(MaxList)} {
    set PT(MaxList) $counter
  }
  foreach i [lsort $liste] {
    $t.top.list.vars.f.l insert end $i
  }

    # Action Area
  frame $t.action.end
  frame $t.action.option

  frame $t.action.end.default -relief sunken -bd 1
  button $t.action.end.default.ok -text "Go To" \
    -command {global Dialog PT
              set Dialog(Button) Ok
              set x [$PT(net) Transition $Dialog(Trans) get x]
              set y [$PT(net) Transition $Dialog(Trans) get y]
              set x [expr $x * $PT(Zoom)]
              set y [expr $y * $PT(Zoom)]
              set unitsX [.drawing.pad.xscroll get]
              set unitsY [.drawing.yscroll get]
              set relX [expr $x / 6000.0]
              set relY [expr $y / 6000.0]
              set MidX [expr ([lindex $unitsX 1] - [lindex $unitsX 0]) / 2]
              set MidY [expr ([lindex $unitsY 1] - [lindex $unitsY 0]) / 2]
              set Rx [expr [lindex $unitsX 0] + $relX - $MidX]
              set Ry [expr [lindex $unitsY 0] + $relY - $MidY]
              set AbsX [expr $Rx - [lindex $unitsX 0]]
              set AbsY [expr $Ry - [lindex $unitsY 0]]
              .drawing.canvas xview moveto $AbsX
              .drawing.canvas yview moveto $AbsY
              }
  pack $t.action.end.default.ok 
  pack $t.action.end.default -side left -fill x -padx 3m -pady 2m
  pack $t.action.end -side bottom
  pack $t.action.option -side top

  button $t.action.end.info -text Info \
    -command {global Dialog;InfoTransition $Dialog(Trans)}
  pack $t.action.end.info -side right -padx 3m -pady 2m -fill x

  button $t.action.end.cancel -text Cancel \
    -command {destroy .td;set Dialog(Button) Cancel;catch {
                                   if {[$PT(net) Transition $Dialog(Last) get IsActive]} {
                                     $PT(canvas) itemconfigure [$PT(canvas) find withtag $Dialog(Last)] -fill yellow
                                   } else {
                                     $PT(canvas) itemconfigure [$PT(canvas) find withtag $Dialog(Last)] -fill white
                                   }
                                 }}
  pack $t.action.end.cancel -side right -padx 3m -pady 2m -fill x

  FireBindings
  set old_focus [focus]
  focus $t
}  ;# TransitionDialog

# -------------------------------------------------------------
# Name
#   AllTransitionDialog
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Es werden alle Transitionen in die Liste eingetragen, nicht nur
#   die aktiven. Au"serdem befindet man sich weiter im Editmode.
proc AllTransitionDialog {} {
  global PT Dialog Id2Name IsAnnoFrom

  destroy .td

  set t [toplevel .td -bd 0]
  wm title .td {Transition Dialog}

  frame $t.top -bd 1 -relief raised
  frame $t.action -bd 1 -relief raised
  pack $t.top $t.action -side top -fill both -expand yes

  message $t.top.msg -text "Dialog for Transitions" \
    -aspect 1000 -font -Adobe-Times-Medium-R-Normal--*-180-*-*-*-*-*-* \
    -justify center

  pack $t.top.msg -side top -fill both -expand yes

  frame $t.top.list 
  frame $t.top.list.vars -bd 1 -relief sunken
  pack $t.top.list.vars -side left -expand yes -fill x
  label $t.top.list.vars.label -text "Transitions"

  frame $t.top.list.vars.f
  pack $t.top.list.vars.f -side bottom

  listbox $t.top.list.vars.f.l \
    -yscrollcommand "$t.top.list.vars.f.scroll set" -height 10
  bind $t.top.list.vars.f.l <1> {global Dialog
                                 set Dialog(Trans) [%W get @%x,%y]
                                 catch {
                                   set Dialog(Bind) [$PT(net) Transition $Dialog(Trans) get VT]
                                 }
                                 $PT(canvas) itemconfigure [$PT(canvas) find withtag $Dialog(Trans)] -fill orange
                                 catch {
				   if {$Dialog(Trans)!=$Dialog(Last)} {
                                     $PT(canvas) itemconfigure [$PT(canvas) find withtag $Dialog(Last)] -fill white
				   }
                                 }
                                 set Dialog(Last) $Dialog(Trans)
                                }
  scrollbar $t.top.list.vars.f.scroll -command "$t.top.list.vars.f.l yview"
  pack $t.top.list.vars.label -side top
  pack $t.top.list.vars.f.l -side left
  pack $t.top.list.vars.f.scroll -side left -fill y

  pack $t.top.list -side top -expand yes -fill x
  entry $t.top.bind -relief sunken -textvariable Dialog(Bind)
  pack $t.top.bind -side bottom -expand yes -fill x
  set liste [$PT(net) get Transitions]
  set counter [llength $liste]
  if {$counter > $PT(MaxList)} {
    set PT(MaxList) $counter
  }
  foreach i [lsort $liste] {
    $t.top.list.vars.f.l insert end $i
  }

    # Action Area
  frame $t.action.end
  frame $t.action.option

  frame $t.action.end.default -relief sunken -bd 1
  button $t.action.end.default.ok -text "Go To" \
    -command {global Dialog;set Dialog(Name) $Dialog(Trans);FindAnd}
  pack $t.action.end.default.ok 
  pack $t.action.end.default -side left -fill x -padx 3m -pady 2m
  pack $t.action.end -side bottom
  pack $t.action.option -side top

  button $t.action.end.info -text Info \
    -command {global Dialog;InfoTransition $Dialog(Trans)}
  pack $t.action.end.info -side right -padx 3m -pady 2m -fill x

  button $t.action.end.cancel -text Cancel \
    -command {global PT;destroy .td;set Dialog(Button) Cancel;catch {
                                   $PT(canvas) itemconfigure [$PT(canvas) find withtag $Dialog(Last)] -fill white
                                 }}
  pack $t.action.end.cancel -side right -padx 3m -pady 2m -fill x

  set old_focus [focus]
  focus $t
}  ;# AllTransitionDialog

# -------------------------------------------------------------
# Name
#   UpdateActiveList
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Die Liste der aktiven Transitionen in der Liste wird
#   upgedatet.
proc UpdateActiveList {} {
  global Dialog PT

  catch {
    .td.top.list.vars.f.l delete 0 $PT(MaxList)
    set counter 0
    foreach i [$PT(net) get ActiveTransitions] {
      .td.top.list.vars.f.l insert end $i
      set counter [expr $counter + 1]
    }
    if {$counter > $PT(MaxList)} {
      set PT(MaxList) $counter
    }
  }
}  ;# UpdateActiveList

# -------------------------------------------------------------
# Name
#   ZoomDialog
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Der ZoomFactor laesst sich stufenlos regeln.
proc ZoomDialog {} {
  global PT Dialog

  set t [toplevel .zoom -bd 0]
  wm title $t {Find Dialog}

  frame $t.top -bd 1 -relief raised
  frame $t.action -bd 1 -relief raised

  message $t.top.msg -text "Zoom Dialog" \
    -aspect 1000 -font -Adobe-Times-Medium-R-Normal--*-180-*-*-*-*-*-* \
    -justify center
  pack $t.top.msg -side top -fill x

  pack $t.top -side top -fill both -expand 1
  pack $t.action -side top -fill both -expand 1

  frame $t.top.name 
  label $t.top.name.label -text "Zoom Factor" -width 15
  entry $t.top.name.entry -textvariable Dialog(Zoom) -relief sunken
  set Dialog(Zoom) $PT(Zoom)
  pack $t.top.name.entry -side right -fill x -expand yes
  pack $t.top.name.label -side left 
  pack $t.top.name -padx 3m -pady 2m -side top -anchor w -fill x

    # Action Area
  frame $t.action.default -relief sunken -bd 1
  button $t.action.default.ok -text "Ok" \
   -command {set PT(Zoom) $Dialog(Zoom);Zoom}
  pack $t.action.default.ok 
  pack $t.action.default -side left -fill x -padx 3m -pady 2m

  button $t.action.cancel -text Cancel \
    -command "destroy $t;return"
  pack $t.action.cancel -side right -padx 3m -pady 2m -fill x

}  ;# ZoomDialog


##  ExitFontWindow
##-----------------------------------------------------------
##  Das Font-Window wird ueber den Cancel-Button verlassen.
##-----------------------------------------------------------
##
proc ExitFontWindow {} {
    EditBindings
    destroy .font
}

proc Print {} {
  global PT

  set t [toplevel .print -bd 0]
  wm title $t {Print Net to File}

  frame $t.top -bd 1 -relief raised
  set PT(print.region) all
  frame $t.top.region -relief raised -bd 2
  radiobutton $t.top.all -text {All} -variable PT(print.region) -value all 
  radiobutton $t.top.sel -text {Selected} -variable PT(print.region) -value selected 
  radiobutton $t.top.scr -text {Screen} -variable PT(print.region) -value {}
  pack $t.top.all $t.top.sel $t.top.scr -side left -in $t.top.region

  set PT(print.color) color
  frame $t.top.colormode -relief raised -bd 2
  radiobutton $t.top.color -text {Color} -variable PT(print.color) -value color
  radiobutton $t.top.gray -text {Gray} -variable PT(print.color) -value gray
  radiobutton $t.top.mono -text {Mono} -variable PT(print.color) -value mono
  pack $t.top.color $t.top.gray $t.top.mono -side left -in $t.top.colormode

  set PT(print.orientation) no
  frame $t.top.orientation -relief raised -bd 2
  radiobutton $t.top.port -text {Portrait} -variable PT(print.orientation) -value no
  radiobutton $t.top.land -text {Landscape} -variable PT(print.orientation) -value yes
  pack $t.top.port $t.top.land -side left -in $t.top.orientation

  pack $t.top.region $t.top.colormode $t.top.orientation -side top -fill both -expand yes
  pack $t.top -fill both -expand yes

  frame $t.button -bd 1 -relief raised
  button $t.button.cancel -text "Cancel" -command {destroy .print;return}
  button $t.button.print -text "Print" -command {FSBox PrintCommand "Save as PostScript" {Postscript .ps} . save;destroy .print}
    
  pack $t.button.cancel $t.button.print -side left -fill x -expand yes
  pack $t.button -fill x -expand yes
}

proc PrintCommand {filename} {
  global PT

  set gridding $PT(Gridding)
  if $gridding {
    set PT(Gridding) 0
    DrawGrid
  }
  if {$PT(print.region)!=""} {
    set box [$PT(canvas) bbox $PT(print.region)]
    set x1 [lindex $box 0]
    set y1 [lindex $box 1]
    set width [expr [lindex $box 2]-$x1]
    set height [expr [lindex $box 3]-$y1]
    $PT(canvas) postscript -x $x1 -y $y1 -width $width -height $height\
      -file $filename -colormode $PT(print.color) -rotate $PT(print.orientation)
  } else {
    $PT(canvas) postscript $options
      -file $filename -colormode $PT(print.color) -rotate $PT(print.orientation)
  }
  if $gridding {
    set PT(Gridding) 1
    DrawGrid
  }
}

  #*** Dateiende NetDialog.tcl ***
