# NetDialogLL.tcl
# Dialoge fuer Stellen, Transitionen und Kanten
# Tobias Himstedt 1995
# Burkhard Bieber 1995/96
# Ulf Fildebrandt 1997/98
# last-update: 14.1.98

  #*********
  # Stellen
  #*********

proc PlaceDialog {place} {
  global PT Dialog Id2Name IsAnnoFrom

  destroy .td

  set Dialog(Name) $place
  set Dialog(Mean) [$PT(net) Place $place get Mean]
  set Dialog(InitMarking) [$PT(net) Place $place get InitMarking]
  set Dialog(Marking) [$PT(net) Place $place get Marking]
  set Dialog(State) [$PT(net) Place $place get State]
  set Dialog(blocks) [$PT(net) Place $place get Blocks]
  set Dialog(box) [$PT(net) Place $place get Box]
  set Dialog(Ref) [$PT(net) Place $place get Reference]
  set list ""

  set counter 0
  while {$counter < [llength $Dialog(blocks)]} {
    if {$counter == 0} {
      set list [lindex $Dialog(blocks) 0]
    } else {
      set list [concat $list,]
      set list [concat $list[lindex $Dialog(blocks) $counter]]
    }
    set counter [expr $counter + 1]
  }
  set Dialog(blocks) $list

  set t [toplevel .td  -bd 0]
  wm title $t {Place Dialog}

  frame $t.top  -bd 1  -relief raised
  frame $t.action  -bd 1  -relief raised

  message $t.top.msg  -text "Place Dialog for\nPlace $place" \
    -aspect 1000  -font -Adobe-Times-Medium-R-Normal--*-180-*-*-*-*-*-* \
    -justify center
  pack $t.top.msg  -side top  -fill x

  pack $t.top  -side top  -fill both  -expand 1
  pack $t.action  -side top  -fill both  -expand 1

  frame $t.top.name 
  label $t.top.name.label  -text "Name" -width 15
  entry $t.top.name.entry  -textvariable Dialog(Name)  -relief sunken
  pack $t.top.name.entry -side right -fill x -expand yes
  pack $t.top.name.label -side right
  pack $t.top.name  -padx 3m  -pady 2m  -side top  -anchor w -fill x

  frame $t.top.mean
  label $t.top.mean.label  -text "Meaning" -width 15
  entry $t.top.mean.entry  -textvariable Dialog(Mean)  -relief sunken
  pack $t.top.mean.entry -side right -fill x -expand yes
  pack $t.top.mean.label -side right
  pack $t.top.mean  -padx 3m  -pady 2m  -side top  -anchor w -fill x

  frame $t.top.ref
  label $t.top.ref.label  -text "Reference" -width 15
  entry $t.top.ref.entry  -textvariable Dialog(Ref)  -relief sunken
  pack $t.top.ref.entry  -side right  -expand yes -fill x
  pack $t.top.ref.label  -side right 
  pack $t.top.ref  -padx 3m  -pady 2m  -side top  -anchor e -fill x

  frame $t.top.blocks
  label $t.top.blocks.label  -text "Blocks" -width 15
  entry $t.top.blocks.entry  -textvariable Dialog(blocks)  -relief sunken
  pack $t.top.blocks.entry -side right -fill x -expand yes
  pack $t.top.blocks.label -side right
  pack $t.top.blocks  -padx 3m  -pady 2m  -side top  -anchor w -fill x

  frame $t.top.box
  label $t.top.box.label  -text "Box" -width 15
  entry $t.top.box.entry  -textvariable Dialog(box)  -relief sunken
  pack $t.top.box.entry  -side right -expand yes -fill x
  pack $t.top.box.label  -side right
  pack $t.top.box  -padx 3m  -pady 2m  -side top  -anchor e -fill x

  frame $t.top.init_mark 
  label $t.top.init_mark.label  -text "Initial Marking" -width 15
  entry $t.top.init_mark.entry  -textvariable Dialog(InitMarking) \
    -relief sunken
  pack $t.top.init_mark.entry -side right -fill x -expand yes
  pack $t.top.init_mark.label -side right
  pack $t.top.init_mark  -padx 3m  -pady 2m  -side top  -anchor w -fill x

  frame $t.top.mark 
  label $t.top.mark.label  -text "Current Marking" -width 15
  entry $t.top.mark.entry  -textvariable Dialog(Marking)  -relief sunken
  pack $t.top.mark.entry -side right -fill x -expand yes
  pack $t.top.mark.label -side right
  pack $t.top.mark  -padx 3m  -pady 2m  -side top  -anchor w -fill x

  frame $t.top.state  -bd 1  -relief sunken
  label $t.top.state.label  -text "Status"  -anchor center

  frame $t.top.state.buttons
  radiobutton $t.top.state.buttons.entry  -text Entry \
    -variable Dialog(State)  -value entry  -anchor w
  radiobutton $t.top.state.buttons.internal  -text Internal \
    -variable Dialog(State)  -value internal  -anchor w
  radiobutton $t.top.state.buttons.exit  -text Exit \
    -variable Dialog(State)  -value exit  -anchor w
  pack $t.top.state.buttons.entry \
    $t.top.state.buttons.internal \
    $t.top.state.buttons.exit \
    -side top  -anchor w
  pack $t.top.state.buttons -side right
  pack $t.top.state.label  -side left
  pack $t.top.state  -padx 3m  -pady 2m  -side top

    # Action Area
  frame $t.action.default  -relief sunken  -bd 1
  button $t.action.default.ok  -text "Ok" \
    -command {set Dialog(Button) Ok}
  pack $t.action.default.ok 
  pack $t.action.default  -side left  -fill x  -padx 3m  -pady 2m

  button $t.action.cancel  -text Cancel \
    -command {set Dialog(Button) Cancel}
  pack $t.action.cancel  -side right  -padx 3m  -pady 2m  -fill x
  button $t.action.marking -text Marking \
    -command {MarkingDialog $Dialog(Name)}
  pack $t.action.marking -side right  -padx 3m  -pady 2m  -fill x

  foreach i {name mean mark init_mark} {
    bind $t.top.$i.entry <Return> {set Dialog(Button) Ok}
  }  ;# foreach Eingabefeld

  set old_focus [focus]
  tkwait visibility $t
  grab $t
  focus $t

  tkwait variable Dialog(Button)
  destroy $t

  if {$Dialog(Button) == "Cancel"} {
    return 
  }  ;# if Abbruch

  set old_name $place
  set new_name $place

  focus $old_focus
  if {$Dialog(Name) != $old_name} {
    if {[$PT(net) Exists Transition $Dialog(Name)] || [$PT(net) Exists PTransition $Dialog(Name)] || [$PT(net) Exists Place $Dialog(Name)] || [$PT(net) Exists Block $Dialog(Name)]} {
      tk_dialog .d {Error} \
        "A node named '$Dialog(Name)' already exists. The new name is not accepted" \
        error 0 {Ok}
    } else {
      $PT(net) Place $place set Name $Dialog(Name)
      if {[$PT(net) Exists Place $Dialog(Name)]} {
        set new_name $Dialog(Name)
        set c [$PT(canvas) find withtag $old_name]
        $PT(canvas) dtag $old_name
        $PT(canvas) addtag $new_name withtag $c
        set Id2Name($c) $new_name
        if {$PT(NameSwitch)} {
          set n [$PT(canvas) find withtag Name@$old_name]
          unset IsAnnoFrom($n)
          $PT(canvas) delete $n
          CreateName $new_name Place
        }  ;# if Name anzeigen
        if {$PT(MeanSwitch)} {
          set n [$PT(canvas) find withtag Mean@$old_name]
          $PT(canvas) dtag Mean@$old_name
          set IsAnnoFrom($n) $new_name
          $PT(canvas) addtag Mean@$new_name withtag $n
        }  ;# if Meaning angezeigt
        set num [$PT(net) Place $new_name get Marking]
        if {$num != 0} {
          for {set i 1} {$i <= $num} {incr i} {
            set n [$PT(canvas) find withtag t$i@$old_name]
            $PT(canvas) dtag t$i@$old_name
            $PT(canvas) addtag t$i@$new_name withtag $n
          }  ;# for i ueber die Tokens
          set n [$PT(canvas) find withtag NumTokens@$old_name]
          $PT(canvas) dtag NumTokens@$old_name
          $PT(canvas) addtag NumTokens@$new_name withtag $n
        }
      }  ;# if Markierung vorhanden
    }  ;# if neuer Name erlaubt
  }  ;# if Name gendert
  if {$Dialog(Mean) != [$PT(net) Place $new_name get Mean]} {
    $PT(net) Place $new_name set Mean $Dialog(Mean)
    if {$PT(MeanSwitch)} {
      $PT(canvas) delete Mean@$new_name
      CreateAnno $new_name Place Mean
    }  ;# if Meaning anzeigen
  }  ;# if Meaning gendert
  if {$Dialog(Ref) != [$PT(net) Place $new_name get Reference]} {
    $PT(net) Place $new_name set Reference $Dialog(Ref)
  }  ;# Reference ge"andert
  if {$Dialog(State) != [$PT(net) Place $new_name get State]} {
    $PT(net) Place $new_name set State $Dialog(State)
    UpdatePlaceState $place
  }  ;# if Status gendert
  regsub -all {[^0-9]} $Dialog(Marking) "" Dialog(Marking)
  if {$Dialog(Marking) != ""} {
    if {$Dialog(Marking) != [$PT(net) Place $new_name get Marking]} {
      DelAllTokens $new_name
      $PT(net) Place $new_name set Marking $Dialog(Marking)
      LayoutTokens $new_name
    }  ;# if Markierung gendert
  }
  regsub -all {[^0-9]} $Dialog(InitMarking) "" Dialog(InitMarking)
  if {$Dialog(InitMarking) != ""} {
    if {$Dialog(InitMarking) != [$PT(net) Place $new_name get InitMarking]} {
      $PT(net) Place $new_name set InitMarking $Dialog(InitMarking)
    }  ;# if Anfangsmarkierung gendert
  }
  $PT(net) clearActive

  set Dialog(blocks) [split $Dialog(blocks) ","]
  set bo 1
  foreach tr $Dialog(blocks) {
    if {![$PT(net) Exists Block $tr]} {
      set bo 0
    }
  }
  if {$bo} {
    $PT(net) Place $new_name set Blocks $Dialog(blocks)
  }
  $PT(net) Place $new_name set Box $Dialog(box)
}  ;# PlaceDialog

proc CreatePlace {t} {
  global PT Dialog

  frame $t.top  -bd 1  -relief raised
  frame $t.action  -bd 1  -relief raised

  pack $t.top  -side top  -fill both  -expand 1
  pack $t.action  -side top  -fill both  -expand 1

  frame $t.top.edge
  label $t.top.edge.label  -text "Edge Weight" -width 15
  entry $t.top.edge.entry  -textvariable Dialog(EdgeVars)  -relief sunken
  pack $t.top.edge.entry  -side right  -expand yes -fill x
  pack $t.top.edge.label  -side right 
  pack $t.top.edge  -padx 3m  -pady 2m  -side top  -anchor e -fill x

  frame $t.top.mean
  label $t.top.mean.label  -text "Meaning" -width 15
  entry $t.top.mean.entry  -textvariable Dialog(Mean)  -relief sunken
  pack $t.top.mean.entry  -side right  -expand yes -fill x
  pack $t.top.mean.label  -side right 
  pack $t.top.mean  -padx 3m  -pady 2m  -side top  -anchor e -fill x

  frame $t.top.init_mark 
  label $t.top.init_mark.label  -text "Initial Marking" -width 15
  entry $t.top.init_mark.entry  -textvariable Dialog(InitMarking) \
    -relief sunken
  pack $t.top.init_mark.entry  -side right  -expand yes -fill x 
  pack $t.top.init_mark.label  -side right 
  pack $t.top.init_mark  -padx 3m  -pady 2m  -side top  -anchor e -fill x

  frame $t.top.mark 
  label $t.top.mark.label  -text "Current Marking" -width 15
  entry $t.top.mark.entry  -textvariable Dialog(Marking)  -relief sunken
  pack $t.top.mark.entry  -side right  -expand yes -fill x
  pack $t.top.mark.label  -side right
  pack $t.top.mark  -padx 3m  -pady 2m  -side top  -anchor e -fill x
}

proc UpdatePlace {place} {
  global PT Dialog

  set Dialog(Mean) [$PT(net) Place $place get Mean]
  set Dialog(InitMarking) [$PT(net) Place $place get InitMarking]
  set Dialog(Marking) [$PT(net) Place $place get Marking]
}


  #**************
  # Transitionen
  #**************

proc TransDialog {transtype trans} {
  global PT Dialog Id2Name IsAnnoFrom

  destroy .td

  set Dialog(Name) $trans
  set Dialog(Mean) [$PT(net) $transtype $trans get Mean]
  set Dialog(Ref)  [$PT(net) $transtype $trans get Reference]
  set Dialog(box) [$PT(net) $transtype $trans get Box]
  if {$transtype == "Transition"} {
    set Dialog(Earliest) [$PT(net) $transtype $trans get Earliest]
    set Dialog(Latest) [$PT(net) $transtype $trans get Latest]
    set Dialog(Transitions) [$PT(net) $transtype $trans get Phantoms]
  } else {
    set Dialog(Earliest) ""
    set Dialog(Latest) ""
    set Dialog(Transitions) [$PT(net) $transtype $trans get Synchrons]
  }
  set list ""

  set counter 0
  while {$counter < [llength $Dialog(Transitions)]} {
    if {$counter == 0} {
      set list [lindex $Dialog(Transitions) 0]
    } else {
      set list [concat $list,]
      set list [concat $list[lindex $Dialog(Transitions) $counter]]
    }
    set counter [expr $counter + 1]
  }
  set Dialog(Transitions) $list
  set Dialog(Blocks) [$PT(net) $transtype $trans get Blocks]
  set list ""

  set counter 0
  while {$counter < [llength $Dialog(Blocks)]} {
    if {$counter == 0} {
      set list [lindex $Dialog(Blocks) 0]
    } else {
      set list [concat $list,]
      set list [concat $list[lindex $Dialog(Blocks) $counter]]
    }
    set counter [expr $counter + 1]
  }
  set Dialog(Blocks) $list

  set t [toplevel .td  -bd 0]
  wm title $t {Transition Dialog}

  frame $t.top  -bd 1  -relief raised
  frame $t.action  -bd 1  -relief raised

  message $t.top.msg  -text "Transition Dialog for\nTransition $trans" \
    -aspect 1000  -font -Adobe-Times-Medium-R-Normal--*-180-*-*-*-*-*-* \
    -justify center
  pack $t.top.msg  -side top  -fill x

  pack $t.top  -side top  -fill both  -expand 1
  pack $t.action  -side top  -fill both  -expand 1

  frame $t.top.name 
  label $t.top.name.label  -text "Name" -width 15
  entry $t.top.name.entry  -textvariable Dialog(Name)  -relief sunken
  pack $t.top.name.entry -side right -fill x -expand yes
  pack $t.top.name.label -side right
  pack $t.top.name  -padx 3m  -pady 2m  -side top  -anchor w -fill x

  frame $t.top.mean
  label $t.top.mean.label  -text "Meaning" -width 15
  entry $t.top.mean.entry  -textvariable Dialog(Mean)  -relief sunken
  pack $t.top.mean.entry -side right -fill x -expand yes
  pack $t.top.mean.label -side right
  pack $t.top.mean  -padx 3m  -pady 2m  -side top  -anchor w -fill x

  frame $t.top.ref
  label $t.top.ref.label  -text "Reference" -width 15
  entry $t.top.ref.entry  -textvariable Dialog(Ref)  -relief sunken
  pack $t.top.ref.entry  -side right  -expand yes -fill x
  pack $t.top.ref.label  -side right 
  pack $t.top.ref  -padx 3m  -pady 2m  -side top  -anchor e -fill x

  if {$PT(Time)} {
    frame $t.top.earliest
    label $t.top.earliest.label  -text "Earliest" -width 15
    entry $t.top.earliest.entry  -textvariable Dialog(Earliest)  -relief sunken
    pack  $t.top.earliest.entry -side right -fill x -expand yes
    pack  $t.top.earliest.label -side left
    pack  $t.top.earliest  -padx 3m  -pady 2m  -side top  -anchor w -fill x

    frame $t.top.latest
    label $t.top.latest.label  -text "Latest" -width 15
    entry $t.top.latest.entry  -textvariable Dialog(Latest)  -relief sunken
    pack  $t.top.latest.entry -side right -fill x -expand yes
    pack  $t.top.latest.label -side left
    pack  $t.top.latest  -padx 3m  -pady 2m  -side top  -anchor w -fill x
  }

  frame $t.top.transitions
  label $t.top.transitions.label  -text "Transitions" -width 15
  if {$trans == "Transition"} {
    entry $t.top.transitions.entry  -textvariable Dialog(Transitions)  -relief sunken
  } else {
    entry $t.top.transitions.entry  -textvariable Dialog(Transitions)  -relief sunken
  }
  pack  $t.top.transitions.entry -side right -expand yes -fill x
  pack  $t.top.transitions.label -side right
  pack  $t.top.transitions  -padx 3m  -pady 2m  -side top  -anchor w -fill x

  frame $t.top.blocks
  label $t.top.blocks.label  -text "Blocks" -width 15
  entry $t.top.blocks.entry  -textvariable Dialog(Blocks)  -relief sunken
  pack  $t.top.blocks.entry -side right -expand yes -fill x
  pack  $t.top.blocks.label -side right
  pack  $t.top.blocks  -padx 3m  -pady 2m  -side top  -anchor w -fill x

  frame $t.top.box
  label $t.top.box.label  -text "Box" -width 15
  entry $t.top.box.entry  -textvariable Dialog(box)  -relief sunken
  pack  $t.top.box.entry  -side right -expand yes -fill x
  pack  $t.top.box.label  -side right -fill x
  pack  $t.top.box  -padx 3m  -pady 2m  -side top  -anchor e -fill x

  frame $t.action.default  -relief sunken  -bd 1
  button $t.action.default.ok  -text "Ok" \
    -command {set Dialog(Button) Ok}
  pack $t.action.default.ok 
  pack $t.action.default  -side left  -fill x  -padx 3m  -pady 2m

  button $t.action.cancel  -text Cancel \
    -command {set Dialog(Button) Cancel}
  pack $t.action.cancel  -side right  -padx 3m  -pady 2m  -fill x

  foreach i {name mean} {
    bind $t.top.$i.entry <Return> {set Dialog(Button) Ok}
  }  ;# foreach Eingabefeld

  set old_focus [focus]
  tkwait visibility $t
  grab $t
  focus $t

  tkwait variable Dialog(Button)
  destroy $t

  if {$Dialog(Button) == "Cancel"} {
    return 
  }  ;# if Abbruch

  set old_name $trans
  set new_name $trans

  focus $old_focus
  if {$Dialog(Name) != $old_name} {
    if {[$PT(net) Exists Transition $Dialog(Name)] || [$PT(net) Exists PTransition $Dialog(Name)] || [$PT(net) Exists Place $Dialog(Name)] || [$PT(net) Exists Block $Dialog(Name)]} {
      tk_dialog .d {Error} \
        "A node named '$Dialog(Name)' already exists.\
        The new name is not accepted" \
        error 0 {Ok}
    } else {
      $PT(net) $transtype $trans set Name $Dialog(Name)
      if {[$PT(net) Exists $transtype $Dialog(Name)]} {
        set new_name $Dialog(Name)
        set c [$PT(canvas) find withtag $old_name]
        $PT(canvas) dtag $old_name
        $PT(canvas) addtag $new_name withtag $c
        set Id2Name($c) $new_name
        if {$PT(NameSwitch)} {
          set n [$PT(canvas) find withtag Name@$old_name]
          unset IsAnnoFrom($n)
          $PT(canvas) delete $n
          CreateName $new_name $transtype
        }  ;# if Name anzeigen
        if {$PT(MeanSwitch)} {
          set n [$PT(canvas) find withtag Mean@$old_name]
          $PT(canvas) dtag Mean@$old_name
          set IsAnnoFrom($n) $new_name
          $PT(canvas) addtag Mean@$new_name withtag $n
        }  ;# if Meaning angezeigt
      } ;# Name wurde uebernommen
    }  ;# if neuer Name erlaubt
  }  ;# if Name gendert
  if {$Dialog(Mean) != [$PT(net) $transtype $new_name get Mean]} {
    $PT(net) $transtype $new_name set Mean $Dialog(Mean)
    if {$PT(MeanSwitch)} {
      $PT(canvas) delete Mean@$new_name
      CreateAnno $new_name $transtype Mean
    }  ;# if Meaning anzeigen
  }  ;# if Meaning gendert
  if {$Dialog(Ref) != [$PT(net) $transtype $new_name get Reference]} {
    $PT(net) $transtype $new_name set Reference $Dialog(Ref)
  }  ;# if Reference ge"andert
  $PT(net) $transtype $trans set Earliest $Dialog(Earliest)
  $PT(net) $transtype $trans set Latest $Dialog(Latest)
  if {$PT(Time)} {
    UpdateAnno Transition Time
  }
  set Dialog(Transitions) [split $Dialog(Transitions) ","]
  set Dialog(Blocks)      [split $Dialog(Blocks) ","]

  if {$transtype == "Transition"} {
    set bo 1
    foreach tr $Dialog(Transitions) {
      if {![$PT(net) Exists PTransition $tr]} {
        set bo 0
      }
    }
    if {$bo} {
      $PT(net)  Transition $new_name set Phantoms $Dialog(Transitions)
    }
  } else {
    set bo 1
    foreach tr $Dialog(Transitions) {
      if {![$PT(net) Exists Transition $tr]} {
        set bo 0
      }
    }
    if {$bo} {
      $PT(net) PTransition $new_name set Synchrons $Dialog(Transitions)
    }
  }
  $PT(net) $transtype $new_name set Blocks $Dialog(Blocks)
  $PT(net) $transtype $new_name set Box $Dialog(box)
}  ;# TransDialog

  #********
  # Kanten
  #********

proc EdgeDialog {edge} {
  global PT Dialog Id2Name IsAnnoFrom

  destroy .td

  set old_weight [$PT(net) Edge $edge get Weight]
  set Dialog(Weight) $old_weight

  set t [toplevel .td  -bd 0]
  wm title $t {Arc Dialog}

  frame $t.top  -bd 1  -relief raised
  frame $t.action  -bd 1  -relief raised

  message $t.top.msg  -text "Arc Dialog for\nArc $edge" \
    -aspect 1000  -font -Adobe-Times-Medium-R-Normal--*-180-*-*-*-*-*-* \
    -justify center
  pack $t.top.msg  -side top  -fill x

  pack $t.top  -side top  -fill both  -expand 1
  pack $t.action  -side top  -fill both  -expand 1

  frame $t.top.weight
  label $t.top.weight.label  -text "Weight" -width 10
  entry $t.top.weight.entry  -textvariable Dialog(Weight)
  pack $t.top.weight.entry -side right -expand yes -fill x
  pack $t.top.weight.label -side left
  pack $t.top.weight  -padx 3m  -pady 2m  -anchor w -fill x

  frame $t.action.default  -relief sunken  -bd 1
  button $t.action.default.ok  -text "Ok" \
    -command {set Dialog(Button) Ok}
  pack $t.action.default.ok 
  pack $t.action.default  -side left  -fill x  -padx 3m  -pady 2m

  button $t.action.cancel  -text Cancel \
    -command {set Dialog(Button) Cancel}
  pack $t.action.cancel  -side right  -padx 3m  -pady 2m  -fill x

  bind $t.top.weight.entry <Return> {set Dialog(Button) Ok}

  set old_focus [focus]
  tkwait visibility $t
  grab $t
  focus $t

  tkwait variable Dialog(Button)
  destroy $t

  focus $old_focus
  if {$Dialog(Weight) != $old_weight} {
    $PT(net) Edge $edge set Weight $Dialog(Weight)
    if {$old_weight == 1} {
      CreateEdgeWeight $edge
    } else {
      if {$Dialog(Weight) == 1} {
        set c [$PT(canvas) find withtag  Weight@$edge]
        $PT(canvas) delete $c
        unset IsAnnoFrom($c)
      } else {
        $PT(canvas) itemconfigure Weight@$edge -text $Dialog(Weight)
      }  ;# else, neues Gewicht ungleich 1
    }  ;# else, altes Gewicht ungleich 1
  }  ;# if Gewicht gendert
}  ;# EdgeDialog

# -------------------------------------------------------------
# Name
#   InfoTransition
# ---------
# Eingabe 
#   trans - Name der Transition
# ---------
# Beschreibung
#   Es wird eine Dialogbox mit den Vor- und Nachbereichsinformationen
#   der Transition aufgebaut, als da w"aren: die Vor- und
#   Nachbereichsstellen mitsamt ihres Typs, ihrer Markierung und
#   der Annotation der Kante, die die Stelle mit der Transition
#   verbindet.
proc InfoTransition {trans} {
  global PT Dialog Id2Name IsAnnoFrom

  destroy .id

  set Dialog(Bind) ""
  set Dialog(ActualTrans) $trans

  set t [toplevel .id  -bd 0]
  set Dialog(Fire) $t
  set in  [$PT(net) Transition $trans get InPlaces]
  set out [$PT(net) Transition $trans get OutPlaces]
  wm title .id {Transition Areainfo Dialog}

  frame $t.top  -bd 1  -relief raised
  frame $t.info  -bd 1 -relief raised
  frame $t.action  -bd 1  -relief raised
  pack $t.top $t.info $t.action  -side top  -fill both  -expand yes

  message $t.top.msg  -text "Area of Transition $trans" \
    -aspect 1000  -font -Adobe-Times-Medium-R-Normal--*-180-*-*-*-*-*-* \
    -justify center

  pack $t.top.msg  -side top  -fill both  -expand yes

  frame $t.top.list 
  frame $t.top.list.vars  -bd 1  -relief sunken
  frame $t.top.list.vals  -bd 1  -relief sunken
  pack $t.top.list.vars $t.top.list.vals  -side left  -expand yes  -fill x
  label $t.top.list.vars.label  -text "Input places"

  label $t.top.list.vals.label  -text "Output places"
  frame $t.top.list.vars.f
  frame $t.top.list.vals.f
  pack $t.top.list.vars.f  -side bottom
  pack $t.top.list.vals.f  -side bottom

  listbox $t.top.list.vars.f.l \
    -yscrollcommand "$t.top.list.vars.f.scroll set"  -height 10 
  listbox $t.top.list.vals.f.l \
    -yscrollcommand "$t.top.list.vals.f.scroll set"  -height 10 
  bind $t.top.list.vars.f.l <1> {
	global PT Dialog
        set actPlace [%W get @%x,%y]
        set in  [$PT(net) Place $actPlace get OutEdges]
        foreach i $in {
          if {[$PT(net) Edge $i get Dest] == $Dialog(ActualTrans)} {
            set Dialog(EdgeVars) [$PT(net) Edge $i get Weight]
          }
        }
        UpdatePlace $actPlace
        set x [$PT(net) $actPlace Place get x]
        set y [$PT(net) $actPlace Place get y]
        set x [expr $x * $PT(Zoom)]
        set y [expr $y * $PT(Zoom)]
        set unitsX [.drawing.pad.xscroll get]
        set unitsY [.drawing.yscroll get]
        set relX [expr $x / 6000.0]
        set relY [expr $y / 6000.0]
        set MidX [expr ([lindex $unitsX 1] - [lindex $unitsX 0]) / 2]
        set MidY [expr ([lindex $unitsY 1] - [lindex $unitsY 0]) / 2]
        set Rx [expr [lindex $unitsX 0] + $relX - $MidX]
        set Ry [expr [lindex $unitsY 0] + $relY - $MidY]
        set AbsX [expr $Rx - [lindex $unitsX 0]]
        set AbsY [expr $Ry - [lindex $unitsY 0]]
        #$PT(canvas) addtag selected withtag $actPlace
        catch {
          $PT(canvas) itemconfigure $PT(LastPlace) -fill white
        }
        set PT(LastPlace) [$PT(canvas) find withtag $actPlace]
        $PT(canvas) itemconfigure [$PT(canvas) find withtag $actPlace] -fill orange
        .drawing.canvas xview moveto $AbsX
        .drawing.canvas yview moveto $AbsY
      }
  bind $t.top.list.vals.f.l <1> {
	global PT Dialog
        set actPlace [%W get @%x,%y]
        set out [$PT(net) Place $actPlace get InEdges]
        foreach o $out {
          if {[$PT(net) Edge $o get Src] == $Dialog(ActualTrans)} {
            set Dialog(EdgeVars) [$PT(net) Edge $o get Weight]
          }
        }
        UpdatePlace $actPlace
        set x [$PT(net) Place $actPlace get x]
        set y [$PT(net) Place $actPlace get y]
        set x [expr $x * $PT(Zoom)]
        set y [expr $y * $PT(Zoom)]
        set unitsX [.drawing.pad.xscroll get]
        set unitsY [.drawing.yscroll get]
        set relX [expr $x / 6000.0]
        set relY [expr $y / 6000.0]
        set MidX [expr ([lindex $unitsX 1] - [lindex $unitsX 0]) / 2]
        set MidY [expr ([lindex $unitsY 1] - [lindex $unitsY 0]) / 2]
        set Rx [expr [lindex $unitsX 0] + $relX - $MidX]
        set Ry [expr [lindex $unitsY 0] + $relY - $MidY]
        set AbsX [expr $Rx - [lindex $unitsX 0]]
        set AbsY [expr $Ry - [lindex $unitsY 0]]
        #$PT(canvas) addtag selected withtag $actPlace
        catch {
          $PT(canvas) itemconfigure $PT(LastPlace) -fill white
        }
        set PT(LastPlace) [$PT(canvas) find withtag $actPlace]
        $PT(canvas) itemconfigure [$PT(canvas) find withtag $actPlace] -fill orange
        .drawing.canvas xview moveto $AbsX
        .drawing.canvas yview moveto $AbsY
      }
  scrollbar $t.top.list.vars.f.scroll  -command ".id.top.list.vars.f.l yview"
  scrollbar $t.top.list.vals.f.scroll  -command ".id.top.list.vals.f.l yview"
  pack $t.top.list.vars.label  -side top
  pack $t.top.list.vars.f.l  -side left
  pack $t.top.list.vars.f.scroll  -side left  -fill y

  pack $t.top.list.vals.label  -side top
  pack $t.top.list.vals.f.l  -side left
  pack $t.top.list.vals.f.scroll  -side left  -fill y
  pack $t.top.list  -side top  -expand yes  -fill x
#  entry $t.top.bind  -relief sunken  -textvariable Dialog(EdgeVars)
#  pack $t.top.bind  -side bottom  -expand yes  -fill x
  foreach i $in {
    $t.top.list.vars.f.l insert end $i
  }
  foreach o $out {
    $t.top.list.vals.f.l insert end $o
  }
  CreatePlace $t.info

    # Action Area
  frame $t.action.end

  frame $t.action.end.default  -relief sunken  -bd 1
  button $t.action.end.default.ok  -text "Close Info" \
    -command {set Dialog(Button) Ok;catch {$PT(canvas) itemconfigure $PT(LastPlace) -fill white};destroy .id}
  pack $t.action.end.default.ok 
  pack $t.action.end.default  -side right  -fill x  -padx 3m  -pady 2m
  pack $t.action.end  -side bottom

  set old_focus [focus]
  focus $t
}  ;# InfoTransition


##  FontSelection
##-----------------------------------------------------------
##  Es wird ein Toplevel-Fenster bereitgestellt, mit dessen
##  Hilfe neue Fonts fuer Kanten- und Knotenbeschriftungen
##  und fuer Kommentare ausgewaehlt werden koennen.
##-----------------------------------------------------------
##
proc FontSelection {} {
    global PT fontType

    ## Das Toplevel-Fenster wird erstellt.
    ##
    destroy .font
    toplevel .font

    wm title .font  " "
    wm geometry .font 450x680+450+0
    wm minsize .font 450 680
    wm maxsize .font 450 680


    set labelFont \
	"-*-times-bold-I-Normal--*-140-*"

    ##  Die Canvas fuer die Darstellung des geaenderten
    ##  Fonts wird erstellt und gepackt
    ##
    canvas .font.inf_can\
	-height 65\
	-relief ridge\
	-borderwidth 4


    ##  Die gesamten Radiobutton zum Setzen 
    ##  des gewuenschten Font werden erstellt
    ##  und gepackt.
    ##
    frame .font.sel_font\
	-relief ridge\
	-borderwidth 4
    set family [frame .font.sel_font.fa]
    label $family.la\
	-text "Family:"\
	-width 7\
	-anchor w\
	-padx 5\
	-font $labelFont
    radiobutton $family.times\
	-text "Times" -value Times\
	-variable fontType(family) \
	-command {ExistFont} 
    radiobutton $family.courier\
	-text "Courier" -value Courier\
	-variable fontType(family) \
	-command {ExistFont} 
    radiobutton $family.helvetica\
	-text "Helvetica" -value Helvetica\
	-variable fontType(family)\
	-command {ExistFont} 
    pack $family.la $family.times $family.courier $family.helvetica\
	-side left 

    set weight [frame .font.sel_font.we]
    label $weight.la\
	-text "Weight:"\
	-width 7 -anchor w -padx 5\
	-font $labelFont
    radiobutton $weight.normal -text "Medium" -value Medium\
	-variable fontType(weight)\
	-command ExistFont 
    radiobutton $weight.bold -text "Bold" -value Bold\
	-variable fontType(weight)\
	-command ExistFont 
    pack $weight.la $weight.normal $weight.bold\
	-side left

    set slant [frame .font.sel_font.sla]
    label $slant.la -text "Slant:"  -width 7 -anchor w -padx 5\
	-font $labelFont
    radiobutton $slant.it -text "Italic" -value I\
	-variable fontType(slant)\
	-command ExistFont 
    radiobutton $slant.ro -text "Roman" -value R\
	-variable fontType(slant)\
	-command ExistFont 
    radiobutton $slant.ob -text "Oblique" -value O\
	-variable fontType(slant)\
	-command ExistFont 
    pack $slant.la $slant.it $slant.ro $slant.ob\
	-side left

    set size [frame .font.sel_font.si]
    label $size.la -text "Size:" -width 7 -anchor nw -pady 4 -padx 5\
	-font $labelFont
    set size1 [frame $size.si1]
    radiobutton $size1.8\
	-text " 8 Points"\
	-value 80\
	-variable fontType(size)\
	-command ExistFont 
    radiobutton $size1.10 -text "10 Points" -value 100\
	-variable fontType(size)\
	-command ExistFont 
    radiobutton $size1.12 -text "12 Points" -value 120\
	-variable fontType(size)\
	-command ExistFont 
    radiobutton $size1.14 -text "14 Points" -value 140\
	-variable fontType(size)\
	-command ExistFont 
    radiobutton $size1.18 -text "18 Points" -value 180\
	-variable fontType(size)\
	-command ExistFont 
    radiobutton $size1.24 -text "24 Points" -value 240\
	-variable fontType(size)\
	-command ExistFont 
    radiobutton $size1.36 -text "36 Points" -value 360\
	-variable fontType(size)\
	-command ExistFont 
    pack $size1.8 $size1.10 $size1.12 $size1.14\
	$size1.18 $size1.24 $size1.36\
	-side top -anchor w
    pack $size.la $size1 -side left -fill y

    pack $family $weight $slant $size\
	-side top\
	-pady 2m -fill x 


    ##  Erstellen und Packen des unteren Bereichs 
    ##  mit mehreren Button ueber die die Elemente 
    ##  auf der Zeichenflaeche des Editors geaendert 
    ##  werden koennen.
    ##
    frame .font.but\
	-relief ridge\
	-borderwidth 4
    set change [frame .font.but.ch -border 2]
    label $change.la -text "Change font for:" -anchor nw -pady 7 -width 17\
	-font $labelFont
    set change1 [frame $change.buts1 -border 2]
    set change2 [frame $change.buts2 -border 2]
    button $change1.namePlace\
	-text "Names of Places"\
	-command {ChangeAnnotation namePlace} 
    button $change1.nameTrans\
	-text "Names of Transitions"\
	-command {ChangeAnnotation nameTrans}
    button $change1.meanPlace\
	-text "Meaning of Places"\
	-command {ChangeAnnotation meanPlace}
    button $change1.meanTrans\
	-text "Meaning of Transitions"\
	-command {ChangeAnnotation meanTrans}
    button $change2.comments\
	-text "Comments"\
	-command {ChangeAnnotation comments}
    button $change2.scomments\
	-text "Selected Comments"\
	-command {ChangeAnnotation scomments}
    pack $change2.comments $change2.scomments\
        -side top -fill x
    pack $change1.namePlace $change1.nameTrans \
	$change1.meanPlace $change1.meanTrans \
	-side top -fill x
    pack $change.la -side top
    pack $change1 $change2 -side left -fill y
    pack $change\
	-side top -pady 1m -fill x



    ##  Erstellung des unteren Bereichs mit dem
    ##  einen Abschlussbutton
    ##
    frame .font.exit\
	-relief ridge\
	-borderwidth 4
    set sunk [frame .font.exit.ca\
		  -bd 1\
		  -relief sunken ]
    button $sunk.close\
	-highlightbackground Black\
	-text "Close" -command { ExitFontWindow}
    pack $sunk.close\
	-padx 2m -pady 2m
    pack $sunk \
	-side left\
	-expand 1\
	-padx 3m -pady 2m

    ##  Packen der drei Bereiche in das
    ##  Toplevel-Fenster.
    ##
    pack .font.inf_can .font.sel_font .font.but .font.exit\
	-side top\
	-fill both\
	-expand yes\
	-padx 1m\
	-pady 1m


    ##  Die Ueberschrift wird auf der Canvas  
    ##  gesetzt.
    ##
    set fontType(family) Times
    set fontType(weight) Bold
    set fontType(slant) I
    set fontType(size) 180
    append newFont "-*-$fontType(family)-$fontType(weight)"
    append newFont "-R-Normal--*-$fontType(size)-*"
	
    .font.inf_can create text 145 35\
	-anchor center\
	-text "Font - Selection"\
	-font $newFont\
	-tags fontText 

    ExistFont
}

##  ExistFont
##-----------------------------------------------------------
##  Es wird getestet, ob ein ueber die Radiobutton aus-
##  gewaehlter Font auf dem System existiert. Falls dies der
##  Fall ist, wird dieser Font oben auf der Canvas angezeigt
##  und man hat die Moeglichkeit die Elemente auf der 
##  Zeichenflaeche zu aendern. Sonst wird eine Meldung 
##  ausgegeben, dass dieser Font nicht existiert.
##-----------------------------------------------------------
##
proc ExistFont {} {
    global fontType 

    append newFont "-*-$fontType(family)-$fontType(weight)"
    append newFont "-$fontType(slant)-normal--*-$fontType(size)-*"

    if { [AskForFont $newFont] == 1} {

	.font.inf_can itemconfigure fontText\
	    -text "Font - Selection" -font $newFont
	.font.but.ch.buts1.namePlace configure -state normal
	.font.but.ch.buts1.nameTrans configure -state normal
	.font.but.ch.buts1.meanPlace configure -state normal
	.font.but.ch.buts1.meanTrans configure -state normal
        .font.but.ch.buts2.comments configure -state normal
	.font.but.ch.buts2.scomments configure -state normal
	return yes
    } else {
	.font.inf_can itemconfigure fontText\
	    -font -*-Courier-Medium-R-Normal--*-140-*\
	    -text "This Font Does Not Exist"
        .font.but.ch.buts1.namePlace configure -state disabled
	.font.but.ch.buts1.nameTrans configure -state disabled
	.font.but.ch.buts1.meanPlace configure -state disabled
	.font.but.ch.buts1.meanTrans configure -state disabled
        .font.but.ch.buts2.comments configure -state disabled
	.font.but.ch.buts2.scomments configure -state disabled
	return no
    }
}

  #*** Dateiende NetDialogLL.tcl ***   
