# --------------------------------------------------------------------
# NetHLEdit.tcl
# --------------------------------------------------------------------
# Diese Datei bildet die Oberflche fr den Highlevel-Petrinetzeditor
# Siehe auch NetBase.tcl
# Tobias Himstedt 3/95
# Burkhard Bieber 1995/96
# Ulf Fildebrandt 1997/1998
# last-update: 5.2.98
# --------------------------------------------------------------------

if [catch {set env(PEPHOME)}] {  
  set my_auto_path_common "/opt/pep/lib"
  set my_auto_path_net "/opt/pep/lib/net"
  set my_auto_path_hl "/opt/pep/lib/net/HL"
  set my_bitmaps_path "/opt/pep/bitmaps/editors"
  set my_macros "/opt/pep/lib/bpn2mnet/___procs.hl"
} else { 
  set my_auto_path_common "$env(PEPHOME)/lib" 
  set my_auto_path_net "$env(PEPHOME)/lib/net"
  set my_auto_path_hl "$env(PEPHOME)/lib/net/HL"
  set my_bitmaps_path "$env(PEPHOME)/bitmaps/editors"
  set my_macros "$env(PEPHOME)/lib/bpn2mnet/___procs.hl"
  set PT(my_macros) "$env(PEPHOME)/lib/bpn2mnet/___procs.hl"
}  ;# else, eine Path-Angabe war vorhanden
set PT(bitmaps) $my_bitmaps_path
set auto_path [linsert $auto_path 0 $my_auto_path_common]
set auto_path [linsert $auto_path 0 $my_auto_path_net]
set auto_path [linsert $auto_path 0 $my_auto_path_hl]

package require netedit

# -------------------------------------------------------------
# Name
#   SetupWindow
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Die Gestalt des Hauptfensters wird festgelegt, die Mens bestimmt usw.
proc SetupWindow {} {
  global PT

  wm minsize . 200 200
  set PT(canvas) .drawing.canvas

  bind all <1> {focus %W}

  frame .drawing
  frame .mbar -relief raised -bd 2 
  frame .status -relief flat -bd 2
  scrollbar .drawing.yscroll -command ".drawing.canvas yview" \
    -relief sunken -orient vertical -width 12
  frame .drawing.pad
  scrollbar .drawing.pad.xscroll -command ".drawing.canvas xview" \
    -relief sunken -orient horizontal -width 12
  set pad [expr [.drawing.yscroll cget -width] + 2 * \
    ([.drawing.yscroll cget -bd] + \
     [.drawing.yscroll cget -highlightthickness])]
  frame .drawing.pad.it -width $pad -height $pad
  canvas .drawing.canvas -width 500 -height 500 \
    -scrollregion {0 0 6000 6000} \
    -xscrollcommand ".drawing.pad.xscroll set" \
    -yscrollcommand ".drawing.yscroll set" \
    -relief sunken -borderwidth 3
  bind .drawing.pad.xscroll <4> {.drawing.canvas xview scroll -1 units}
  bind .drawing.yscroll <4> {.drawing.canvas yview scroll -1 units}
  bind .drawing.canvas <4> {.drawing.canvas yview scroll -1 units}
  bind .drawing.pad.xscroll <5> {.drawing.canvas xview scroll 1 units}
  bind .drawing.yscroll <5> {.drawing.canvas yview scroll 1 units}
  bind .drawing.canvas <5> {.drawing.canvas yview scroll 1 units}

    # Mens
  menubutton .mbar.file -text File -menu .mbar.file.menu -underline 0
  menubutton .mbar.edit -text Edit -menu .mbar.edit.menu -underline 0
  menubutton .mbar.hierarchy -text Hierarchy -menu .mbar.hierarchy.menu -underline 0
  menubutton .mbar.composition -text Composition -menu .mbar.composition.menu -underline 0
  menubutton .mbar.action -text Action -menu .mbar.action.menu -underline 0
  menubutton .mbar.network -text Network -menu .mbar.network.menu -underline 0
  menubutton .mbar.option -text Option -menu .mbar.option.menu -underline 0
  menubutton .mbar.help -text Help -menu .mbar.help.menu -underline 0

    # File-Men
  menu .mbar.file.menu -borderwidth 1 -tearoff 0
  .mbar.file.menu add command -label Open \
    -command {AnimateEnd;FSBox LoadNet "Open Net" $PT(Filter)}
  .mbar.file.menu add command -label Reload \
    -command {AnimateEnd;LoadNet $PT(Filename)}
  .mbar.file.menu add command -label "Save Net" -state disabled \
    -command {AnimateEnd
              if {$PT(Zoom) != 1} {
                  switch [tk_dialog .d {Zoom Warning} {The Zoomfactor is not equal 1} question 0 {Go on} {Cancel}] {
                      0 {
                        set PT(AnimationStop) 1
                        set PT(wait) 1
                      }
                      1 {return}
                  }
              }
              SaveNet $PT(Filename)}
  .mbar.file.menu add command -label "Save Net As..." \
    -command {AnimateEnd
              if {$PT(Zoom) != 1} {
                  switch [tk_dialog .d {Zoom Warning} {The Zoomfactor is not equal 1} question 0 {Go on} {Cancel}] {
                      0 {
                        set PT(AnimationStop) 1
                        set PT(wait) 1
                      }
                      1 {return}
                  }
              }
              FSBox SaveNet "Save Net" $PT(Filter) . save}
  .mbar.file.menu add command -label "Save Selection as" \
    -command {AnimateEnd;FSBox SaveModule "Save Module" $PT(Filter) . save}
  .mbar.file.menu add separator
  .mbar.file.menu add command -label "Print" -command {AnimateEnd;Print}
  .mbar.file.menu add separator
  .mbar.file.menu add command -label Quit \
    -command {Quit} -accelerator "Ctrl-Q"

    # Edit-Men
  menu .mbar.edit.menu -tearoff 0
  .mbar.edit.menu add command -label "New Place" \
    -command {AnimateEnd;NewNode Place} -accelerator P
  .mbar.edit.menu add command -label "New Transition" \
    -command {AnimateEnd;NewNode Transition} -accelerator T
  .mbar.edit.menu add command -label "New PTransition" \
    -command {AnimateEnd;NewNode PTransition}
  .mbar.edit.menu add command -label "New Block" \
    -command {AnimateEnd;NewNode Block}
  .mbar.edit.menu add command -label "New Box" \
    -command {AnimateEnd;NewNode Box}
  .mbar.edit.menu add command -label "New Arc" \
    -command {AnimateEnd;EdgeBindings} -accelerator A
  .mbar.edit.menu add command -label "New PArc" \
    -command {AnimateEnd;PEdgeBindings}
  .mbar.edit.menu add command -label "Create Comment" \
    -command {AnimateEnd;CreateTitle 100 100} -accelerator C
  .mbar.edit.menu add separator
  .mbar.edit.menu add command -label "Cut" \
    -command {AnimateEnd;Cut} -accelerator "Ctrl-x"
  .mbar.edit.menu add command -label "Copy" \
    -command {AnimateEnd;Copy} -accelerator "Ctrl-c"
  .mbar.edit.menu add command -label "Paste" \
    -command {AnimateEnd;Paste} -accelerator "Ctrl-v"
  .mbar.edit.menu add separator
  .mbar.edit.menu add command -label "Find" \
    -command {AnimateEnd;Find}
  .mbar.edit.menu add separator
  .mbar.edit.menu add command -label "Select some Nodes" \
    -command {AnimateEnd;MarkingBox}
  .mbar.edit.menu add command -label "Select All" \
    -command SelectAll -accelerator "Ctrl-a"
  .mbar.edit.menu add command -label "Delete Elements" \
    -command {AnimateEnd;DeleteElements} -accelerator Del
  .mbar.edit.menu add command -label "Delete Net" \
    -command {AnimateEnd;DeleteNet}
  .mbar.edit.menu add separator
  .mbar.edit.menu add command -label "Repaint" \
    -command {AnimateEnd;Repaint} -accelerator R

    # Block-Men
  menu .mbar.hierarchy.menu -tearoff 0
  .mbar.hierarchy.menu add command -label "Hierarchy Tree" \
    -command {AnimateEnd;BlockTreeDialog}
  .mbar.hierarchy.menu add command -label "Block List" \
    -command {AnimateEnd;BlockList}
  .mbar.hierarchy.menu add command -label "Other Nets List" \
    -command {AnimateEnd;NetsDialog}

    # Action-Men
  menu .mbar.action.menu -tearoff 0
  .mbar.action.menu add command -label "Mark Active Transitions" \
    -command {AnimateEnd;ActiveTransitions}
  .mbar.action.menu add command -label "Active Transition Dialog" \
    -command {AnimateEnd;TransitionDialog}
  .mbar.action.menu add command -label "All Transition Dialog" \
    -command {AnimateEnd;AllTransitionDialog}
  .mbar.action.menu add separator
  .mbar.action.menu add command -label "Execute Transition" \
    -command {AnimateEnd;FireBindings}
  .mbar.action.menu add command -label "Execute Transition List" \
    -command {AnimateEnd;FiringBox}
  .mbar.action.menu add command -label "Animate" \
    -command {AnimateEnd;Animate}
  .mbar.action.menu add checkbutton -label "Fast Simulation" \
    -variable PT(FastSimulation)
  .mbar.action.menu add checkbutton -label "Highlight Execution" \
    -variable PT(HighlightWhileFire)
#  .mbar.action.menu add checkbutton -label "Reference Format" \
#    -variable PT(ReferenceFormat)
  .mbar.action.menu add separator
  .mbar.action.menu add command -label "Reset to initial Marking" \
    -command {AnimateEnd;$PT(net) DeleteStatistic;ResetToInitMarking}
  .mbar.action.menu add command -label "Clear Transition Lists" \
    -command {AnimateEnd;set PT(node_list) "";set PT(counters) 0;set PT(counterl) 0;set PT(loop_list) ""}
  .mbar.action.menu add separator
  .mbar.action.menu add command -label "Show Reference" \
    -command {AnimateEnd;ShowReferenceBindings}
  .mbar.action.menu add command -label "Show Active Trans Ref" \
    -command {AnimateEnd;GetActiveTransitionList}
  .mbar.action.menu add command -label "Show Marking Ref" \
    -command {AnimateEnd;GetCurrentMarking}
  .mbar.action.menu add separator
  .mbar.action.menu add command -label "Convert to new Net Format" \
    -command {AnimateEnd;FastNet} 

    # Option-Men
  menu .mbar.option.menu -tearoff 0
  .mbar.option.menu add checkbutton -label "Show Names" \
    -command {AnimateEnd;UpdateNames} -variable PT(NameSwitch)
  .mbar.option.menu add checkbutton -label "Show Meanings" \
    -command {AnimateEnd; UpdateMeanings} -variable PT(MeanSwitch)
  .mbar.option.menu add checkbutton -label "Show Action Terms"\
    -command {AnimateEnd;UpdateAnno Transition AT} -variable PT(ATSwitch)
  .mbar.option.menu add checkbutton -label "Show Value Terms" \
    -command {AnimateEnd;UpdateAnno Transition VT} -variable PT(VTSwitch)
  .mbar.option.menu add checkbutton -label "Show Types" \
    -command {AnimateEnd;UpdateAnno Place Type} -variable PT(TypeSwitch)
  .mbar.option.menu add checkbutton -label "Show Arc Annotations" \
    -command {AnimateEnd;UpdateEdgeAnno} -variable PT(EdgeVars)
  .mbar.option.menu add checkbutton -label "Show Time" \
    -command {AnimateEnd;UpdateAnno Transition Time} -variable PT(Time)
  .mbar.option.menu add checkbutton -label "Show Sync Transitions" \
    -command {AnimateEnd;SwitchSyncTransDisplay} -variable PT(SyncTransSwitch)
  .mbar.option.menu add checkbutton -label "Show Phantom Transitions" \
    -command {AnimateEnd;SwitchPhantomTransDisplay} -variable PT(PhantomTransSwitch)
  .mbar.option.menu add checkbutton -label "Show Invisible Arcs" \
    -command {AnimateEnd;UpdateInvisibleArcs} -variable PT(InvisibleArcsOn)
  .mbar.option.menu add separator
  .mbar.option.menu add command -label "Synchronisation Options" \
    -command SynchronOptions
  .mbar.option.menu add command -label "Simulation Options" \
    -command {AnimateEnd;SimulationOptions}
  .mbar.option.menu add command -label "Simulation Statistics" \
    -command {SimulationStatistic}
  .mbar.option.menu add command -label "Template Directory" \
    -command {AnimateEnd;TemplateDirectory}
  .mbar.option.menu add checkbutton -label "Output Meaning" \
    -variable PT(Output)
  .mbar.option.menu add checkbutton -label "Random Bindings" \
    -variable PT(RandBind)
  .mbar.option.menu add separator
#  .mbar.option.menu add checkbutton -label "Display @a + @b" \
#    -command {AnimateEnd;UpdateVar_ab} -variable PT(Var_ab_Switch)
  .mbar.option.menu add command -label "Move Options" \
    -command {AnimateEnd;MoveOptions}
  .mbar.option.menu add checkbutton -label "Gridding" \
    -variable PT(Gridding) -command DrawGrid
  .mbar.option.menu add command -label "Align to Grid" \
    -command {AnimateEnd;AlignNodes}
  .mbar.option.menu add command -label "Change Font" \
    -command {FontSelection}
#  .mbar.option.menu add command -label "Zoom" -command ZoomDialog
  .mbar.option.menu add cascade -label "Zoom" -menu .mbar.option.menu.zoom

     # Zoom-Men
  menu .mbar.option.menu.zoom -tearoff 0
  .mbar.option.menu.zoom add radiobutton -label "25%" \
    -variable PT(Zoom) -value 0.25 -command Zoom
  .mbar.option.menu.zoom add radiobutton -label "50%" \
    -variable PT(Zoom) -value 0.50 -command Zoom
  .mbar.option.menu.zoom add radiobutton -label "75%" \
    -variable PT(Zoom) -value 0.75 -command Zoom
  .mbar.option.menu.zoom add radiobutton -label "100%" \
    -variable PT(Zoom) -value 1 -command Zoom
  .mbar.option.menu.zoom add radiobutton -label "150%" \
    -variable PT(Zoom) -value 1.5 -command Zoom
  .mbar.option.menu.zoom add radiobutton -label "200%" \
    -variable PT(Zoom) -value 2 -command Zoom

    # Composition-Men
  menu .mbar.composition.menu -tearoff 0
  .mbar.composition.menu add command -label "Nela" \
    -command {IconWindow}
  .mbar.composition.menu add separator
  .mbar.composition.menu add command -label "Take in Box" \
    -command {AnimateEnd;TakeInBox}
  .mbar.composition.menu add command -label "Add to Box" \
    -command {AnimateEnd;AddToBoxDialog}
  .mbar.composition.menu add command -label "Instantiate Box" \
    -command {AnimateEnd;Instantiate}
  .mbar.composition.menu add command -label "Append Suffix" \
    -command {AnimateEnd;AppendSuffix}
  .mbar.composition.menu add separator
  .mbar.composition.menu add command -label "Parallel" \
    -command {AnimateEnd;Parallel}
  .mbar.composition.menu add command -label "Sequence" \
    -command {AnimateEnd;Sequence}
  .mbar.composition.menu add command -label "Choice" \
    -command {AnimateEnd;Choice}
  .mbar.composition.menu add command -label "Transition Substitution" \
    -command {AnimateEnd;Substitution}
  .mbar.composition.menu add command -label "Synchronisation" \
    -command {AnimateEnd;Synchronize}
  .mbar.composition.menu add command -label "Restriction" \
    -command {AnimateEnd;Restrict}
  .mbar.composition.menu add command -label "Tie" \
    -command {AnimateEnd;Tie}
  .mbar.composition.menu add command -label "Expression" \
    -command {AnimateEnd;Expression}
  .mbar.composition.menu add separator
  .mbar.composition.menu add command -label "Load Module" \
    -command {AnimateEnd;FSBox LoadModule "Load Module" $PT(Filter)}
  .mbar.composition.menu add command -label "Load Templatenet" \
    -command {AnimateEnd;FSBox LoadModule "Load Template" $PT(Filter) $PT(my_macros)}

    # Network-Men
  menu .mbar.network.menu -tearoff 0
  .mbar.network.menu add command -label "Select Servers" \
    -command {ConnectionWindow}
  .mbar.network.menu add command -label "Establish Connections" \
    -command "$PT(net) Establish"

    # Help-Men
  menu .mbar.help.menu -tearoff 0
  .mbar.help.menu add command -label "Info" -command {AnimateEnd;InfoDialog}

    # Packer arbeiten lassen
  pack .mbar .mbar.file .mbar.edit .mbar.hierarchy .mbar.composition .mbar.action .mbar.network .mbar.option \
    -side left 
  pack .mbar.help \
    -side right

    frame .g -borderwidth 2

    frame .g.fr2b -borderwidth 2

    set toolbar_trans [image create photo -file [file join $PT(bitmaps) trans.gif]]
    button .g.fr2b.transition \
      -image $toolbar_trans \
      -command {AnimateEnd;NewNode Transition}

    set toolbar_place [image create photo -file [file join $PT(bitmaps) place.gif]]
    button .g.fr2b.place \
      -image $toolbar_place \
      -command {AnimateEnd;NewNode Place}

    button .g.fr2b.edge \
      -bitmap "@$PT(bitmaps)/arrow.xm"\
      -command {AnimateEnd;EdgeBindings}

    set toolbar_text [image create photo -file [file join $PT(bitmaps) comment.gif]]
    button .g.fr2b.comment \
      -image $toolbar_text \
      -command {AnimateEnd;CreateTitle 100 100}

    frame .g.fr3b \
      -borderwidth 2

    set toolbar_reset [image create photo -file [file join $PT(bitmaps) reset.gif]]
    button .g.fr3b.reset \
      -image $toolbar_reset \
      -command ResetToInitMarking

    set PT(toolbar.onestep) [image create photo -file [file join $PT(bitmaps) onestep.gif]]
    button .g.fr3b.execute \
      -image $PT(toolbar.onestep) \
      -command OneStep

    set PT(toolbar.multistep) [image create photo -file [file join $PT(bitmaps) multistep.gif]]
    set PT(toolbar.stop) [image create photo -file [file join $PT(bitmaps) stop.gif]]
    button .g.fr3b.animate \
      -image $PT(toolbar.multistep) \
      -command {AnimateEnd;Animate}

    button .g.fr3b.ref \
      -bitmap "@$PT(bitmaps)/reference.xm"\
      -command {AnimateEnd;ShowReferenceBindings}

    button .g.fr3b.expose \
      -bitmap "@$PT(bitmaps)/expose.xm"\
      -command {AnimateEnd;
                 set PT(canvas) .drawing.canvas
                 foreach b [$PT(net) get Boxes] {
                   $PT(net) Box $b set Visibility 1
                 }
                 foreach b [$PT(net) get Blocks] {
                   $PT(net) Block $b set BlockVis exposed_contents
                 }
                 $PT(net) RecalcBlockVis
                 Repaint
               }

    frame .g.fr4b -borderwidth 1

    set toolbar_new [image create photo -file [file join $PT(bitmaps) .. pep_edit new.gif]]
    button .g.fr4b.new \
      -image $toolbar_new \
      -command {AnimateEnd;DeleteNet}

    set toolbar_save [image create photo -file [file join $PT(bitmaps) .. pep_edit save.gif]]
    button .g.fr4b.save \
      -image $toolbar_save -state disabled \
      -command {AnimateEnd
                if {$PT(Zoom) != 1} {
                  switch [tk_dialog .d {Zoom Warning} {The Zoomfactor is not 1} question 0 {Go on} {Cancel}] {
                      0 {
                        set PT(AnimationStop) 1
                        set PT(wait) 1
                      }
                      1 {return}
                  }
                }
                SaveNet $PT(Filename)}

    set toolbar_open [image create photo -file [file join $PT(bitmaps) .. pep_edit open.gif]]
    button .g.fr4b.open \
      -image $toolbar_open \
      -command {AnimateEnd;FSBox LoadNet "Open Net" $PT(Filter)}

    frame .g.fr4c

    set toolbar_cut [image create photo -file [file join $PT(bitmaps) .. pep_edit cut.gif]]
    button .g.fr4c.cut \
      -image $toolbar_cut \
      -command {AnimateEnd;Cut}

    set toolbar_copy [image create photo -file [file join $PT(bitmaps) .. pep_edit copy.gif]]
    button .g.fr4c.copy \
      -image $toolbar_copy \
      -command {AnimateEnd;Copy}

    set toolbar_paste [image create photo -file [file join $PT(bitmaps) .. pep_edit paste.gif]]
    button .g.fr4c.paste \
      -image $toolbar_paste \
      -command {AnimateEnd;Paste}

    frame .g.fr4d

    set toolbar_print [image create photo -file [file join $PT(bitmaps) print.gif]]
    button .g.fr4d.print \
      -image $toolbar_print \
      -command {.mbar.file.menu invoke 6}

    pack .g.fr4b.new \
      .g.fr4b.open \
      .g.fr4b.save \
      -side left

    pack .g.fr4c.cut .g.fr4c.copy .g.fr4c.paste -side left

    pack .g.fr4d.print -side left

    pack .g.fr2b.transition\
      .g.fr2b.place\
      .g.fr2b.edge\
      .g.fr2b.comment\
      -side left

    pack .g.fr3b.reset\
      .g.fr3b.execute\
      .g.fr3b.animate\
      .g.fr3b.ref\
      .g.fr3b.expose\
      -side left

    pack .g.fr4b -side left
    frame .g.sep1 -width 3 -borderwidth 1 -relief raised
    pack .g.sep1 -padx 4 -side left -pady 8 -fill y
    pack .g.fr4c -side left
    frame .g.sep2 -width 3 -borderwidth 1 -relief raised
    pack .g.sep2 -padx 4 -side left -pady 8 -fill y
    pack .g.fr4d -side left

    pack .g.fr2b .g.fr3b\
      -side left\
      -padx 5

    frame .fr2bo \
      -borderwidth 2
    entry .fr2bo.entry1 \
      -relief sunken\
      -borderwidth 1\
      -textvariable PT(messageHeader)\
      -state disabled\
      -width 6
    entry .fr2bo.entry2 \
      -borderwidth 1\
      -width 30\
      -relief sunken\
      -justify center\
      -highlightcolor AntiqueWhite\
      -state disabled\
      -textvariable PT(messageText)
    button .fr2bo.quit \
      -text Quit \
      -command {Quit}

    pack .fr2bo.entry2 \
      -side left\
      -pady 1m\
      -fill x

    pack .fr2bo.quit \
      -side right \
      -pady 1m \
      -fill x

  label .status.file -text "File "
  label .status.filename -text $PT(Filename) -relief sunken
  set PT(FilenameW) .status.filename
  label  .status.mode -text $PT(Mode) -relief sunken -width 25
  scale .status.scale -from 0 -to 500 -length 200 -variable PT(Speed) \
    -orient horizontal -width 5
  label .status.animation -text "Delay "
  set PT(ModeW) .status.mode
  set PT(NEXT) ""
  label .status.next -text $PT(NEXT) -width 8
  set PT(NEXTW) .status.next
  pack .status.file -side left
  pack .status.filename -side left -ipadx 3 -padx 3
  pack .status.mode -side right -ipadx 3 -padx 3
  pack .status.scale -side right
  pack .status.animation -side right
  pack .status.next -side right

  pack .drawing.pad -side bottom -fill x
  pack .drawing.pad.it -side right
  pack .drawing.pad.xscroll -side bottom -fill x
  pack .drawing.yscroll -side right -fill y
  pack .drawing.canvas -side left -expand yes -fill both

  pack .mbar .g .status -side top -fill x
  pack .fr2bo -side bottom -fill x
  pack .drawing -expand yes -fill both -side top
  tk_menuBar .mbar .mbar.file
  focus .mbar
  wm protocol . WM_DELETE_WINDOW "Quit"
  EditBindings
}  ;# SetupWindow

# -------------------------------------------------------------
# Name
#   IconWindow
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Das Fenster mit den Icons zur Programmierung mit NELA wird
#   aufgebaut.
proc IconWindow {} {
  global PT

  set w .icon
  destroy $w
  toplevel $w
  wm title $w "Programming Icons"
  wm geometry .icon 210x340+500+0
  wm minsize .icon 210 340
  wm maxsize .icon 210 340
  
  image create photo icon1 -file "$PT(bitmaps)/parallel.gif"
  image create photo icon2 -file "$PT(bitmaps)/sequence.gif"
  image create photo icon3 -file "$PT(bitmaps)/choice.gif"
  image create photo icon4 -file "$PT(bitmaps)/object.gif"
  image create photo icon5 -file "$PT(bitmaps)/takeinbox.gif"
  image create photo icon6 -file "$PT(bitmaps)/instantiate.gif"
  image create photo icon7 -file "$PT(bitmaps)/substitution.gif"
  image create photo icon8 -file "$PT(bitmaps)/addtobox.gif"
  image create photo icon9 -file "$PT(bitmaps)/expr.gif"
  image create photo icon10 -file "$PT(bitmaps)/synchronize.gif"
  image create photo icon11 -file "$PT(bitmaps)/restrict.gif"
  image create photo icon12 -file "$PT(bitmaps)/exit.gif"
  image create photo nela -file "$PT(bitmaps)/nela.gif"

  frame $w.buttons1
  pack $w.buttons1 -fill y
  button $w.buttons1.icon1 -image icon1 -command "AnimateEnd;Parallel"
  button $w.buttons1.icon2 -image icon2 -command {AnimateEnd;Sequence}
  button $w.buttons1.icon3 -image icon3 -command {AnimateEnd;Choice}
  pack $w.buttons1.icon1 $w.buttons1.icon2 $w.buttons1.icon3 -side left
  frame $w.buttons2
  pack $w.buttons2 -fill y
  button $w.buttons2.icon4 -image icon4 -command {AnimateEnd;FSBox LoadModule "Load Module" $PT(Filter)}
  button $w.buttons2.icon5 -image icon5 -command {AnimateEnd;TakeInBox}
  button $w.buttons2.icon6 -image icon6 -command {AnimateEnd;Instantiate}
  pack $w.buttons2.icon4 $w.buttons2.icon5 $w.buttons2.icon6 -side left
  frame $w.buttons3
  pack $w.buttons3 -fill y
  button $w.buttons3.icon7 -image icon7 -command {AnimateEnd;Substitution}
  button $w.buttons3.icon8 -image icon8 -command {AnimateEnd;AddToBoxDialog}
  button $w.buttons3.icon9 -image icon9 -command {AnimateEnd;Expression}
  pack $w.buttons3.icon7 $w.buttons3.icon8 $w.buttons3.icon9 -side left
  frame $w.buttons4
  pack $w.buttons4 -fill y
  button $w.buttons4.icon10 -image icon10 -command {AnimateEnd;Synchronize}
  button $w.buttons4.icon11 -image icon11 -command {AnimateEnd;Restrict}
  button $w.buttons4.icon12 -image icon12 -command {AnimateEnd;destroy .icon}
  pack $w.buttons4.icon10 $w.buttons4.icon11 $w.buttons4.icon12 -side left
  frame $w.nela
  pack $w.nela -fill y
  label $w.nela.ne -image nela -relief sunken -bd 1
  pack $w.nela.ne
  pack $w.buttons1 $w.buttons2 $w.buttons3 $w.buttons4 $w.nela
}  ;# IconWindow

# -------------------------------------------------------------
# Name
#   LayoutAnnotatios
# ---------
# Eingabe 
#   name -> Name des Knotens
#   type -> Type des Knotens
# ---------
# Beschreibung
#   Je nach Flag werden die Annotationen (Name und Meaning) fr 
#   einen Knoten angezeigt bzw. aktualisiert.
proc LayoutAnnotations {name type} {
  global PT

  if $PT(NameSwitch) {
    LayoutAnno $name $type Name
  }
  if $PT(MeanSwitch) {
    LayoutAnno $name $type Mean
  }
  if {($type == "Transition") || ($type == "PTransition")} {
    if $PT(VTSwitch) {
      LayoutAnno $name $type VT
    }
    if $PT(ATSwitch) {
      LayoutAnno $name $type AT
      LayoutAnno $name $type LT
    }
    if $PT(Time) {
      LayoutAnno $name $type Time
    }
  } elseif {$type == "Place"} {
    if $PT(TypeSwitch) {
      LayoutAnno $name $type Type
    }
  }  ;# elseif Stelle
}  ;# LayoutAnnotations

# -------------------------------------------------------------
# Name
#   DelAllTokens
# ---------
# Eingabe 
#   place -> Stellenname
# ---------
# Beschreibung
#   Die graphischen Token einer Stelle werden vom Schirm gelscht.
proc DelAllTokens {place} {
  global PT

  set num_tokens [llength [split [$PT(net) Place $place get Marking] ,]]
  for {set i 1} {$i <= $num_tokens} {incr i} {
    $PT(canvas) delete t$i@$place
  }
  $PT(canvas) delete NumTokens@$place
}  ;# DelAllTokens

# -------------------------------------------------------------
# Name
#   FireTrans
# ---------
# Eingabe 
#   c -> Canvas-ID der Transition
# ---------
# Beschreibung
#   Die graphische Animation einer schaltenden Transition wird ausgefhrt. 
#   Die Transition ist dabei ber die Canvas-ID c gegeben.
proc FireTrans {c} {
  global PT Id2Name Place2Show

  while {$PT(FireWait) == 0} {
    after 200
  }
  set PT(FireWait) 0
  if {$c == 12345} {
    set name $PT(trans)
  } else {
    set name $Id2Name($c)
  }
  if {[$PT(net) Exists Transition $name] == 0} {
    if {[$PT(net) Exists Block $name] == 0} {
      return
    }
      # Es ist ein Block, also eine aktive Transition des Blockes schalten.
    set at [$PT(net) get ActiveTransitionsForBlock $name]
    set t_name [lindex $at 0]
    set edge_color $PT(color.block_edge)
  } else {
    set t_name $name
    set edge_color $PT(color.edge)
  }  ;# else, war eine Transition

  if {[$PT(net) Transition $t_name get IsActive] == 0} {
    set PT(FireWait) 1
    return
  }


  set pre [$PT(net) Transition $t_name get InPlaces]
  set post [$PT(net) Transition $t_name get OutPlaces]
  set ine [$PT(net) Transition $t_name get InEdges]
  set oute [$PT(net) Transition $t_name get OutEdges]

  if {!$PT(RandBind)} {
    set bind [FireDialog $t_name]
    if {$bind == ""} {
      set PT(AnimationStop) 1
      set PT(FireWait) 1
      return
    }
    if {![$PT(net) IsValidBinding $t_name $bind]} {
      switch [tk_dialog .d {Transition cannot fire} {The selected binding is not valid} error 0 {Ok}] {
                      0 {
                        set PT(FireWait) 1
                        return
                      }
                  }
    }
  }  ;# if nicht Random-Binding
  if {!$PT(FastSimulation)} {
    foreach e $ine {
      $PT(canvas) itemconfigure $e -fill green -width 3
    }
    update idletasks
    after $PT(Speed)
  }
  foreach p $pre { 
    DelAllTokens $p
  }
  if {!$PT(FastSimulation)} {
    foreach e $ine {
      $PT(canvas) itemconfigure $e -fill $edge_color -width 1
    }
  }
  if $PT(Output) {
    puts [$PT(net) Transition $t_name get Mean]
  }
  if $PT(HighlightWhileFire) {
    ShowRef $t_name
  }  ;# if Anzeige im B(PN)-Editor
  ShowPhantomRelation Transition $t_name
  if $PT(RandBind) {
    $PT(net) RandomFire $t_name
  } else {
    $PT(net) FireTransition $t_name $bind
  }  ;# else, kein zuflliges Binding
  incr PT(AnimationNum)

  if {!$PT(FastSimulation)} {
    foreach e $oute {
      $PT(canvas) itemconfigure $e -fill green -width 3
    }
  }
  foreach p $pre {
    catch {$Place2Show($p) configure -text [$PT(net) Place $p get Marking]}
    LayoutTokens $p
  }
  if {!$PT(FastSimulation)} {
    update idletasks
    after $PT(Speed)
  }

  foreach p $post {
    catch {$Place2Show($p) configure -text [$PT(net) Place $p get Marking]}
    LayoutTokens $p
  }  ;# foreach Stelle im Nachbereich
  if {!$PT(FastSimulation)} {
    foreach e $oute {
      $PT(canvas) itemconfigure $e -fill $edge_color -width 1
    }  ;# foreach Ausgangskante
  }
  ActiveTransitions
  set PT(FireWait) 1
}  ;# FireTrans

# -------------------------------------------------------------
# Name
#   OneStep
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Es wird in den Einzelschrittmodus umgeschaltet. Entweder eine
#   Transition wird geschaltet aus der Liste oder die aktiven
#   Transitionen werden farbig markiert.
proc OneStep {} {
  global PT

  AnimateEnd
  .g.fr3b.execute configure -image $PT(toolbar.stop) -command EndFire
  if {[llength $PT(node_list)] == $PT(counters)} {
    if {[llength $PT(loop_list)] == 0} {
      FireBindings
      return
    } else {
      set t [lindex $PT(loop_list) $PT(counterl)]
    }
  } else {
    set t [lindex $PT(node_list) $PT(counters)]
  }
  while {$PT(wait) == 0} {
    after 200
  }
  set PT(wait) 0
  ActiveTransitions
  if {$t == "tic"} {
    bell
  } else {
    if {$t == "" || ![$PT(net) Transition $t get IsActive]} {
      if {![tk_dialog .d {Active Transition Error} "The transition $t is not active!" error 0 {Ok}]} {
        set PT(AnimationStop) 1
        set PT(wait) 1
        return
      }
    }
    set PT(trans) $t
    FireTrans 12345
  }

  if {[llength $PT(node_list)] == $PT(counters)} {
    set PT(counterl) [expr $PT(counterl) + 1]
    if {$PT(counterl) == [llength $PT(loop_list)]} {
      set PT(counterl) 0
    }
    $PT(NEXTW) configure -text [lindex $PT(loop_list) $PT(counterl)]
    update
    set PT(wait) 1
  } else {
    if {[expr $PT(counters)] < [llength $PT(node_list)]} {
      set PT(counters) [expr $PT(counters) + 1]
      if {$PT(counters) == [llength $PT(node_list)]} {
        $PT(NEXTW) configure -text [lindex $PT(loop_list) 0]
      } else {
        $PT(NEXTW) configure -text [lindex $PT(node_list) $PT(counters)]
      }
      update
    }
    set PT(wait) 1
  }
}  ;# OneStep

# -------------------------------------------------------------
# Name
#   DeleteNodeAnno
# ---------
# Eingabe 
#   name -> Knotenname
# ---------
# Beschreibung
#   Die mglichen Annotationen eines Knoten werden vom Canvas entfernt
proc DeleteNodeAnno {name} {
  global PT IsAnnoFrom

  set n [$PT(canvas) find withtag Name@$name]
  if {[llength $n] > 0} {
    unset IsAnnoFrom($n)
    $PT(canvas) delete $n
  }
  set m [$PT(canvas) find withtag Mean@$name]
  if {[llength $m] > 0} {
    unset IsAnnoFrom($m)
    $PT(canvas) delete $m
  }
  set m [$PT(canvas) find withtag VT@$name]
  if {[llength $m] > 0} {
    unset IsAnnoFrom($m)
    $PT(canvas) delete $m
  }
  set m [$PT(canvas) find withtag AT@$name]
  if {[llength $m] > 0} {
    unset IsAnnoFrom($m)
    $PT(canvas) delete $m
  }
  set m [$PT(canvas) find withtag LT@$name]
  if {[llength $m] > 0} {
    unset IsAnnoFrom($m)
    $PT(canvas) delete $m
  }
  set m [$PT(canvas) find withtag Type@$name]
  if {[llength $m] > 0} {
    unset IsAnnoFrom($m)
    $PT(canvas) delete $m
  }
}  ;# DeleteNodeAnno

# -------------------------------------------------------------
# Name
#   CreateEdgeVars
# ---------
# Eingabe 
#   edge -> Kantenname
# ---------
# Beschreibung
#   Die Variablen der Kante edge werden graphisch erzeugt.
proc CreateEdgeVars {edge} {
  global PT IsAnnoFrom

  set vars [$PT(net) Edge $edge get Vars]
  if {$PT(EdgeVars)} {
    set c [$PT(canvas) create text 0 0 -text $vars \
      -anchor center -tags Vars@$edge -font $PT(edge) ]
    $PT(canvas) addtag EdgeAnnotation withtag $c
    $PT(canvas) addtag Annotation withtag $c
    set IsAnnoFrom($c) $edge
    SetupEdgeAnno $edge
  }
}  ;# CreateEdgeVars

# -------------------------------------------------------------
# Name
#   SetupEdgeAnno
# ---------
# Eingabe 
#   edge -> Kantenname
# ---------
# Beschreibung
#   Die Variablen der Kante edge (Tag Vars@edge) werden positioniert.
proc SetupEdgeAnno {edge} {
  global PT

  if {[expr [$PT(net) Edge $edge get Visibility ] != 1 && \
    $PT(InvisibleArcsOn) == 0] } {
    return
  }
  set coords [$PT(canvas) coords $edge]
  set nx [expr ([lindex $coords 2] - [lindex $coords 0])/2 + \
    [lindex $coords 0] + [$PT(net) Edge $edge get VarsX]]
  set ny [expr ([lindex $coords 3] - [lindex $coords 1])/2 + \
    [lindex $coords 1] + [$PT(net) Edge $edge get VarsY]]
  $PT(canvas) coords Vars@$edge $nx $ny
}  ;# SetupEdgeAnno

# -------------------------------------------------------------
# Name
#   MkBlock
# ---------
# Eingabe 
#   x, y -> Position
#   id   -> Blockname
# ---------
# Beschreibung
#   Eine vorhandenen Block mit Name id wird an (x, y) graphisch erstellt.
proc MkBlock {x y id} {
  global PT Id2Name

  $PT(canvas) delete TempNode
  set item [Center bigrectangle $x $y $id]
  $PT(canvas) addtag Block withtag $item
  $PT(canvas) addtag Node withtag $item
  $PT(canvas) addtag $id withtag $item
  $PT(canvas) addtag Deletable withtag $item
  set Id2Name($item) $id
  if {$PT(NameSwitch)} {
    CreateName $id Block
  }  ;# if Name anzeigen
  if {$PT(MeanSwitch)} {
    CreateAnno $id Block Mean
  }  ;# if Meaning soll angezeigt werden
  #block$id configure -background black
  #$PT(canvas) itemconfigure $item -image block$id
  NetChanged
}  ;# MkBlock

# -------------------------------------------------------------
# Name
#   MkBox
# ---------
# Eingabe 
#   x, y -> Position
#   id   -> Blockname
# ---------
# Beschreibung
#   Eine vorhandenen Box mit Name id wird an (x, y) graphisch erstellt.
proc MkBox {x y id} {
  global PT Id2Name

  $PT(canvas) delete TempNode
  set item [Center bigrectangle $x $y $id]
  $PT(canvas) addtag Box withtag $item
  $PT(canvas) addtag Node withtag $item
  $PT(canvas) addtag $id withtag $item
  $PT(canvas) addtag Deletable withtag $item
  set Id2Name($item) $id
  if {$PT(NameSwitch)} {
    CreateName $id Box
  }  ;# if Name anzeigen
  if {$PT(MeanSwitch)} {
    CreateAnno $id Box Mean
  }  ;# if Meaning soll angezeigt werden
  NetChanged
}  ;# MkBox

# -------------------------------------------------------------
# Name
#   MkPlace
# ---------
# Eingabe 
#   x, y -> Position
#   id   -> Stellenname
# ---------
# Beschreibung
#   Eine vorhandene Stelle mit Name id wird an (x, y) graphisch erstellt.
proc MkPlace {x y id} {
  global PT Id2Name

  $PT(canvas) delete TempNode
  set item [Center oval $x $y]
  $PT(canvas) addtag Place withtag $item
  $PT(canvas) addtag Node withtag $item
  $PT(canvas) addtag $id withtag $item
  $PT(canvas) addtag Deletable withtag $item
  set Id2Name($item) $id
  if {$PT(NameSwitch)} {
    CreateName $id Place
  }
  if {$PT(MeanSwitch)} {
    CreateAnno $id Place Mean
  }
  if $PT(TypeSwitch) {
    CreateAnno $id Place Type
  }
  UpdatePlaceState $id
  NetChanged
}  ;# MkPlace

# -------------------------------------------------------------
# Name
#   MkTransition
# ---------
# Eingabe 
#   x, y -> Position
#   id   -> Transitionsname
# ---------
# Beschreibung
#   Eine vorhandene Transition mit Name id wird an (x, y) graphisch erstellt.
proc MkTransition {x y id} {
  global PT Id2Name

  $PT(canvas) delete TempNode
  set item [Center rectangle $x $y]
  $PT(canvas) addtag Transition withtag $item
  $PT(canvas) addtag Node withtag $item
  $PT(canvas) addtag $id withtag $item
  $PT(canvas) addtag Deletable withtag $item
  set Id2Name($item) $id
  if {$PT(NameSwitch)} {
    CreateName $id Transition
  }
  if {$PT(MeanSwitch)} {
    CreateAnno $id Transition Mean
  }
  if $PT(VTSwitch) {
    CreateAnno $id Transition VT
  }
  if $PT(ATSwitch) {
    CreateAnno $id Transition AT
    CreateAnno $id Transition LT
  }
  if {$PT(Time)} {
    CreateAnno $id Transition Time
  }
  UpdateTransitionState $id
  NetChanged
}  ;# MkTransition

# -------------------------------------------------------------
# Name
#   MkPTransition
# ---------
# Eingabe 
#   x, y -> Position
#   id   -> Phantomtransitionsname
# ---------
# Beschreibung
#   Eine vorhandene Phantomtransition mit Name id wird an (x, y)
#   graphisch erstellt.
proc MkPTransition {x y id} {
  global PT Id2Name

  $PT(canvas) delete TempNode
  set item [Center rectangle $x $y]
  $PT(canvas) addtag PTransition withtag $item
  $PT(canvas) addtag Node withtag $item
  $PT(canvas) addtag $id withtag $item
  $PT(canvas) addtag Deletable withtag $item
  set Id2Name($item) $id
  if {$PT(NameSwitch)} {
    CreateName $id PTransition
  }
  if {$PT(MeanSwitch)} {
    CreateAnno $id PTransition Mean
  }
  if $PT(VTSwitch) {
    CreateAnno $id PTransition VT
  }
  if $PT(ATSwitch) {
    CreateAnno $id PTransition AT
  }
  UpdatePTransitionState $id
  NetChanged
}  ;# MkPTransition

# -------------------------------------------------------------
# Name
#   MkEdge
# ---------
# Eingabe 
#   n1 -> Knoten 1
#   n2 -> Knoten 2
#   id -> Kantenname
# ---------
# Beschreibung
#   Eine vorhandene Kante von n1 nach n2 mit Name id wird graphisch erstellt.
proc MkEdge {n1 n2 id} {
  global Id2Name PT

  set edge [SetupEdge $n1 $n2]
  set coords [$PT(canvas) coords $edge]
  $PT(canvas) addtag $id withtag $edge
  $PT(canvas) addtag Deletable withtag $edge
  $PT(canvas) raise $edge grid
  set Id2Name($edge) $id
  CreateEdgeVars $id
  NetChanged
}  ;# MkEdge

# -------------------------------------------------------------
# Name
#   MkPEdge
# ---------
# Eingabe 
#   n1 -> Knoten 1
#   n2 -> Knoten 2
#   id -> Phantomkantenname
# ---------
# Beschreibung
#   Eine vorhandene Phantomkante von n1 nach n2 mit Name id
#   wird graphisch erstellt.
proc MkPEdge {n1 n2 id} {
  global Id2Name PT

  set edge [SetupPEdge $n1 $n2]
  $PT(canvas) addtag $id withtag $edge
  $PT(canvas) raise $edge grid
  set Id2Name($edge) $id
  NetChanged
}  ;# MkPEdge

# -------------------------------------------------------------
# Name
#   MkBlockEdge
# ---------
# Eingabe 
#   n1 -> Knoten 1
#   n2 -> Knoten 2
#   id -> Kantenname
# ---------
# Beschreibung
#   Eine vorhandene Kante von n1 nach n2 mit Name id wird graphisch erstellt.
#proc MkBlockEdge {n1 n2 id} {
#  global Id2Name PT
#
#  set edge [SetupBlockEdge $n1 $n2]
#  $PT(canvas) addtag $id withtag $edge
#  $PT(canvas) raise $edge grid
#  set Id2Name($edge) $id
#  NetChanged
#}  ;# MkBlockEdge
# die neue Variante mit Mengen von Knoten
proc MkBlockEdge {ns1 ns2 id} {
  global Id2Name PT

  foreach n1 $ns1 {
    foreach n2 $ns2 {
      set edge [SetupBlockEdge $n1 $n2]
      $PT(canvas) addtag "$id@$n1@$n2" withtag $edge
      $PT(canvas) raise $edge grid
      set Id2Name($edge) "$id@$n1@$n2"
    }  ;# foreach Knoten in ns2
  }  ;# foreach Knoten in ns1
  NetChanged
}  ;# MkBlockEdge

# -------------------------------------------------------------
# Name
#   MkBlockPEdge
# ---------
# Eingabe 
#   n1 -> Knoten 1
#   n2 -> Knoten 2
#   id -> Phantomkantenname
# ---------
# Beschreibung
#   Eine vorhandene Phantomkante von n1 nach n2 mit Name id wird
#   graphisch erstellt.
proc MkBlockPEdge {n1 n2 id} {
  global Id2Name PT

  set edge [SetupBlockPEdge $n1 $n2]
  $PT(canvas) addtag $id withtag $edge
  $PT(canvas) raise $edge grid
  set Id2Name($edge) $id
  NetChanged
}  ;# MkBlockPEdge

# -------------------------------------------------------------
# Name
#   CenterToken
# ---------
# Eingabe 
#   id   -> Id des Tokens
#   x, y -> Position
#   val  -> Wert des Tokens
# ---------
# Beschreibung
#   Ein Token mit ID id wird an (x, y) zentriert dargestellt. Falls 
#   das Token (graphisch) noch nicht existiert, wird es erstellt.
proc CenterToken {id x y val} {
  global PT

  set o3 [ expr $PT(ObjectSize) / 6]
    # Existiert das Tokens schon
  if {[$PT(canvas) find withtag $id] == {}} {
      # Nein -> dann erstellen
      # Ist es ein BlackToken?
    if {$val=="dot"} {
        # Dann kleinen Kreis machen.
      set tid [$PT(canvas) create oval [expr $x - $o3] [expr $y - $o3] \
        [expr $x + $o3] [expr $y + $o3] -tags Token -fill black]
    } else {
        # sonst Text des Wertes
      set tid [$PT(canvas) create text $x $y \
        -text $val -tags Token -font $PT(AnnoFont) -anchor center]
    }
    $PT(canvas) addtag $id withtag $tid
  } else {
      # ja -> dann verschieben
    if {$val == "dot"} {
      $PT(canvas) coords $id [expr $x - $o3] [expr $y - $o3] \
        [expr $x + $o3] [expr $y + $o3]
      $PT(canvas) itemconfigure $id -fill black
    } else {
      $PT(canvas) coords $id $x $y
      $PT(canvas) itemconfigure $id -fill black
    }
  }
}  ;# CenterToken

# -------------------------------------------------------------
# Name
#   LayoutTokens
# ---------
# Eingabe 
#   place_name -> Der Name sagt's
# ---------
# Beschreibung
#   Die Tokens der Stelle place_name werden graphisch angeordnet.
#   Falls mehr als 5 Tokens darzustellen sind, wird nur noch die 
#   Anzahl dieser dagestellt.
proc LayoutTokens {place_name} {
  global PT

  if {[llength [$PT(canvas) find withtag $place_name]] == 0} {
    return
  }  ;# if Stelle ist nicht auf dem Schirm, also keine Aktion ntig
  set px [expr [lindex [$PT(canvas) coords $place_name] 0] + $PT(OSizeHalf)]
  set py [expr [lindex [$PT(canvas) coords $place_name] 1] + $PT(OSizeHalf)]
  set o3 [expr $PT(ObjectSize) / 6]
  set val [split [$PT(net) Place $place_name get Marking] ","]
  DelAllTokens $place_name
  set num [llength $val]
  switch $num {
    0  {
      return}
    1  {
      CenterToken t1@$place_name $px $py [lindex $val 0]
    }
    2  {
      CenterToken t1@$place_name [expr $px-2*$o3+2] $py [lindex $val 0]
      CenterToken t2@$place_name [expr $px+2*$o3-2] $py [lindex $val 1]
    }
      # Nicht schn, aber unheimlich selten!!!
    3  {
      CenterToken t1@$place_name [expr $px-2*$o3] $py [lindex $val 0]
      CenterToken t2@$place_name $px $py  [lindex $val 1]
      CenterToken t3@$place_name [expr $px+2*$o3] $py [lindex $val 2]
    }
    4  {
      CenterToken t1@$place_name [expr $px-2*$o3] $py [lindex $val 0]
      CenterToken t2@$place_name $px $py  [lindex $val 1]
      CenterToken t3@$place_name [expr $px+2*$o3] $py [lindex $val 2]
      CenterToken t4@$place_name $px [expr $py-2*$o3] [lindex $val 3]
    }
    5  {
      CenterToken t1@$place_name [expr $px-2*$o3] $py [lindex $val 0]
      CenterToken t2@$place_name $px $py  [lindex $val 1]
      CenterToken t3@$place_name [expr $px+2*$o3] $py [lindex $val 2]
      CenterToken t4@$place_name $px [expr $py-2*$o3] [lindex $val 3]
      CenterToken t5@$place_name $px [expr $py+2*$o3] [lindex $val 4]
    }
    default {
      foreach i {1 2 3 4 5} {
        $PT(canvas) delete t$i@$place_name
      }
        # Existiert der Text schon?
      if {[$PT(canvas) find withtag NumTokens@$place_name] == {}} {
          # Nein -> dann kreieren
        $PT(canvas) create text $px $py -text "\#$num" \
          -anchor center -tags NumTokens@$place_name -font $PT(AnnoFont) -tags Token
      } else {
          # Text nderen
        $PT(canvas) itemconfigure NumTokens@$place_name -text "\#$num"
        $PT(canvas) itemconfigure NumTokens@$place_name -fill black
        $PT(canvas) coords NumTokens@$place_name $px $py
      }  ;# else
    }  ;# default
  }  ;# switch
}  ;# LayoutTokens

# -------------------------------------------------------------
# Name
#   FastNet
# ---------
# Eingabe
#   Nix
# ---------
# Beschreibung
#   "Uberf"uhrt ein HL-Netz in alter Notation in das neue Format
# -------------------------------------------------------------
proc FastNet {} {
  global PT

  $PT(net) Convert
}

# -------------------------------------------------------------
# Name
#   SimulationStatistic
# ---------
# Eingabe
#   Nix
# ---------
# Beschreibung
#   Gibt die Simulationszeiten des aktuellen Netzes aus
# -------------------------------------------------------------
proc SimulationStatistic {} {
  global PT

  # if already open
  if [catch {set d [toplevel .simtime]}] {
   raise .simtime
   return
  }
  wm title $d "Simulation Statistic"
  wm protocol $d WM_DELETE_WINDOW "destroy $d"

    # Container fr die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both -expand true
  set t [text $d.options.t -setgrid true -wrap word \
    -font *-helvetica-medium-r-*-10-* \
    -width 42 -height 14 \
    -yscrollcommand "$d.options.sy set"]
  scrollbar $d.options.sy -orient vert -command "$d.options.t yview"
  pack $d.options.sy -side right -fill y
  pack $d.options.t -side left -fill both -expand true

  $d.options.t insert end [$PT(net) Statistic]

    # Container fr die Buttons der Kommandos
  frame $d.commands -relief raised -bd 1
  pack $d.commands -side bottom -fill both
    # Kommando-Button
  button $d.commands.clear -text "Clear" \
    -command "$d.options.t delete 0.1 end; $PT(net) DeleteStatistic"
  button $d.commands.update -text "Update" \
    -command {.simtime.options.t delete 0.1 end; .simtime.options.t insert end [$PT(net) Statistic]; .simtime.options.t see end}
  button $d.commands.close -text "Close" \
    -command "destroy $d"
  pack $d.commands.clear $d.commands.update $d.commands.close -side left -expand 1
} ; # SimulationStatistics

# -------------------------------------------------------------
# Name
#   SimulationOptions
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Die Simulationsoptionen werden gesetzt, d.h. die PreAnalyse, InAnalyse
#   und der Cache k"onnen abgeschaltet werden.
proc SimulationOptions {} {
  global PT oldvals

  set d [toplevel .simulationoptions]
  wm title $d "Simulation Options"
  wm protocol $d WM_DELETE_WINDOW "destroy $d"

  foreach i {PreAnalyse InAnalyse Cache AllBindings StatisticsOutput} {
    set oldvals($i) $PT($i)
  }
    # Container fr die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
  checkbutton $d.options.preanalyse \
    -text "PreAnalyse" -anchor w \
    -variable PT(PreAnalyse)
  checkbutton $d.options.inanalyse \
    -text "InAnalyse" -anchor w \
    -variable PT(InAnalyse)
  checkbutton $d.options.cache \
    -text "Cache" -anchor w \
    -variable PT(Cache)
  checkbutton $d.options.bindings \
    -text "AllBindings" -anchor w \
    -variable PT(AllBindings)
  checkbutton $d.options.output \
    -text "Time Protocol" -anchor w \
    -variable PT(StatisticsOutput)
  pack $d.options.preanalyse \
    $d.options.inanalyse \
    $d.options.cache \
    $d.options.bindings \
    $d.options.output \
     -side top -anchor w -fill x

    # Container fr die Buttons der Kommandos
  frame $d.commands -relief raised -bd 1
  pack $d.commands -side bottom -fill both
    # Kommando-Buttons
  button $d.commands.ok -text "Ok" \
    -command "SetSimOptions;unset oldvals;destroy $d"
  button $d.commands.default -text "Default" \
    -command "SimulationOptionsDefaults"
  button $d.commands.cancel -text "Cancel" \
    -command "global PT oldvals; \
	foreach i {PreAnalyse InAnalyse Cache AllBindings StatisticsOutput} \
	  {set PT(\$i) \$oldvals(\$i)};unset oldvals;destroy $d"
  pack $d.commands.ok \
    $d.commands.default \
    $d.commands.cancel \
    -side left -expand 1
} ; # SimulationOptions

proc SimulationOptionsDefaults {} {
  global PT

  set PT(Cache) 1
  set PT(PreAnalyse) 1
  set PT(InAnalyse) 1
  set PT(AllBindings) 0
  set PT(StatisticsOutput) 0
  set PT(Output) 0
} ; # SimulationOptionsDefaults

proc SetSimOptions {} {
  global PT

  $PT(net) Cache $PT(Cache)
  $PT(net) InAnalyse $PT(InAnalyse)
  $PT(net) PreAnalyse $PT(PreAnalyse)
  $PT(net) AllBindings $PT(AllBindings)
  $PT(net) TimeOutput $PT(StatisticsOutput)
} ; # SetSimOptions

# -------------------------------------------------------------
# Name
#   Init
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Highlevel-Netz spezifische Initialisierungen werden durchgefhrt.
proc Init {} {
  global PT

    # Highlevel Netz im Kern kreieren
  if [catch {set PT(net)}] {
    set PT(net) [MakeHLNet]
  }  ;# if bisher kein Netz vorhanden
    # Standard Initialisierung
  CommonInit
    # Stellen-Typ anzeigen?
  set PT(TypeSwitch) 1
    # Variablen @a, @b anzeigen?
  set PT(Var_ab_Switch) 0
  set PT(Filename) noname.hl_net
  set PT(Filter) {{HL Net} .hl_net}
  set PT(RandBind) 1

  SynchronizerOptionsDefaults
  SetSynchronizerOptions
  SimulationOptionsDefaults
  SetSimOptions
}  ;# Init

Init
SetupWindow

  # Falls ein Dateiname in der Kommandozeile bergeben wurde,
  # wird hier versucht diese als Netzdatei zu betrachten und zu laden.
if {$argc > 0} {
  LoadNet [lindex $argv 0]
}  ;# if Argumente vorhanden
DrawGrid

  #*** Dateiende NetHLEdit.tcl ***

