# ----------------------------------------------------------------
# NetSim.tcl
# ----------------------------------------------------------------

# -------------------------------------------------------------
# Name
#   ActiveTransitions
# ---------
# Parameter 
#   Nothing 
# ---------
# Description
#   Die aktiven Transitionen werden aus dem C++-Kern gewonnen.
#   Sie werden PT(color.active_transitions) eingefaerbt und mit
#   dem Tag Active versehen. Vorher werden alle Objekte mit diesem
#   Tag wieder PT(color.transition) gefaerbt.
proc ActiveTransitions {} {
  # Todo: das hier in Animate einsetzen?
  global PT Id2Name

#  UnsetAllBindings
#  EditBindings
  UpdateActiveList

  if {$PT(NetChanged)} {
    $PT(net) clearActive
  }
  set ac [$PT(net) get ActiveTransitions]
  set aktive [$PT(canvas) find withtag Active]
  set boxes [$PT(net) get Boxes]

  foreach a $aktive {
    catch {
      $PT(canvas) itemconfigure $a -fill $PT(color.transition)
    }
    catch {
      set id [$PT(canvas) find withtag Name@$Id2Name($a)]
      $PT(canvas) raise $id
    }
  }
  foreach a $aktive {
    catch {
      block$Id2Name($a) configure -background white
      $PT(canvas) itemconfigure block$Id2Name($a) -image block$Id2Name($a)
    }
  }
  $PT(canvas) dtag Active
  $PT(canvas) itemconfigure PActive -fill $PT(color.phantom_transition)
  $PT(canvas) dtag PActive
  foreach a $ac {
    if {[$PT(net) Transition $a get Channel0Sync] != "no"} {
      set trs [$PT(net) Transition $a get Phantoms]
      foreach name $trs {
        $PT(canvas) itemconfigure $name -fill $PT(color.active_transition)
        $PT(canvas) addtag PActive withtag $name
        $PT(canvas) raise $name
        $PT(canvas) raise [$PT(canvas) find withtag Name@$name]
      }
    }
    if {[$PT(net) Transition $a get BlockVis] == "exposed"} {
      $PT(canvas) itemconfigure $a -fill $PT(color.active_transition)
      $PT(canvas) addtag Active withtag $a
      $PT(canvas) raise $a
      $PT(canvas) raise [$PT(canvas) find withtag Name@$a]
    } else {
      foreach b $boxes {
        catch {
          set id [.boxframe$b.fenster.canvas find withtag $a]
          .boxframe$b.fenster.canvas itemconfigure $a -fill $PT(color.active_transition)
          .boxframe$b.fenster.canvas addtag Active withtag $a
          .boxframe$b.fenster.canvas raise $a
          .boxframe$b.fenster.canvas raise [.boxframe$b.fenster.canvas find withtag Name@$a]
        }
      }
      set tblocks [$PT(net) Transition $a get Blocks]
      foreach block $tblocks {
        if {[$PT(net) Block $block get BlockVis] == "exposed"} {
          block$block configure -background yellow
          $PT(canvas) itemconfigure block$block -image block$block
        }
        #$PT(canvas) itemconfigure $block -fill $PT(color.active_block)
        $PT(canvas) addtag Active withtag $block
        $PT(canvas) raise $block
        set act [$PT(net) get ActiveTransitionsForBlock $block]
        foreach t $act {
          #$PT(canvas) addtag Active withtag $t
        }
      }  ;# foreach Fatherblock of the transition
    }  ;# else, transition disappeard into block
  }  ;# foreach active transition
}  ;# ActiveTransitions

# -------------------------------------------------------------
# Name
#   GetActiveTransitionList
# ---------
# Parameter
#   Nothing
# ---------
# Description
#   Die Liste der aktiven Transition wird zurueckgeliefert mit
#   Hilfe von highlight_in_bpn2
proc GetActiveTransitionList {} {
  highlight_in_bpn2 [Active]
}

# -------------------------------------------------------------
# Name
#   GetCurrentMarking
# ---------
# Parameter
#   Nothing
# ---------
# Description
#   Die Liste der aktuellen Markierung wird mit Hilfe von
#   highlight_in_bpn2 zurueckgeliefert
proc GetCurrentMarking {} {
  global PT

  set ac [$PT(net) get CurrentMarking]
  set str ""
  foreach t $ac {
    if {$t != [lindex $ac 0]} {
      set str "$str,$t"
    } else {
      set str $t
    }
  }
  highlight_in_bpn2 $str
}

proc Active {} {
  global PT

  set ac [$PT(net) get ActiveTransitions]
  set str ""
  foreach t $ac {
    if {$t != [lindex $ac 0]} {
      set str "$str,$t"
    } else {
      set str $t
    }
  }
  return $str
}

# -------------------------------------------------------------
# Name
#   FireBindings
# ---------
# Parameter 
#   Nothing 
# ---------
# Description
#   Die Event-Bindings werden geloescht. Auf die aktiven Transitionen
#   (Tag Active) wird ein Binding gelegt
proc FireBindings {} {
  # Todo: Transitionen in Bloecken beachten
  global PT

  UnsetAllBindings
  ActiveTransitions
  $PT(canvas) bind Active <1> FireTransition
  $PT(canvas) bind PActive <1> FirePTransition
  bind . <Escape> { EndFire }
  set boxes [$PT(net) get Boxes]
  foreach b $boxes {
    catch {
      FrameFireBindings .boxframe$b.fenster.canvas
    }
  }
  $PT(ModeW) configure -text "Select Active Transition"

  send_to_simulator "stateLLSimulation"
}  ;# FireBindings


proc ExecuteTransList {nodes} {
  global PT
  set nodes [string trim $nodes]
  set PT(node_list) [split $nodes ","]
  set PT(loop_list) ""
  set PT(counters) 0
}


proc EndFire {} {
  global PT Id2Name

  DeselectAll
  UnsetAllBindings
  EditBindings
  .g.fr3b.execute configure -image $PT(toolbar.onestep) -command OneStep
  set act [$PT(canvas) find withtag Transition]
  foreach n $act {
    $PT(canvas) itemconfigure $n -fill $PT(color.transition)
  }
}

# -------------------------------------------------------------
# Name
#   Highlight
# ---------
# Parameter 
#   nodes -> ein Liste von Namen von Knoten
# ---------
# Description
#   Alle Elemente der Liste nodes werden hervorgehoben.
#   Genauer: Ihnen wird das Tag selected beigefuegt.
# Todo: Farben
proc Highlight {nodes} {
  global PT

  set node_list [split $nodes ","]
  DeselectAll
  foreach n $node_list {
    $PT(canvas) addtag selected withtag $n
  }  ;# foreach Knoten in der Liste
  $PT(canvas) itemconfigure selected -fill blue
}  ;# Highlight

# -------------------------------------------------------------
# Name
#   ShowRef
# ---------
# Parameter 
#   node -> Node name
# ---------
# Description
#   Given node shall be highlighted in B(PN)^2 editor. Therefore the
#   reference string is fetched from c++ core and sent to editor
# Todo: Color
proc ShowRef {node} {
  global PT

  #$PT(canvas) itemconfigure Node -fill white
  #$PT(canvas) itemconfigure $node -fill blue
  set type [GetType $node]
  if {$type == "Block"} {
    set ref ""
  } else {
    #don't know why commented out. Just to fix error set to something...
    set ref ""
    #set ref [$PT(net) $type $node get Reference]
  }
  if {$PT(ReferenceFormat)} {
    highlight_in_bpn2 $node
  } else {
    if {$ref != ""} {
      highlight_in_bpn2 $ref
    }
  }  ;# if Referenz ist vorhanden
}  ;# ShowRef

# -------------------------------------------------------------
# Name
#   ShowReferenceBindings
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Es werden die Bindungen festgelegt, mit denen man sich die 
#   Referenzen anzeigen lassen kann. Auf alle Knoten wird also ein
#   <1>-Event gelegt. Falls <Escape> -> dann wieder EditBindings.
proc ShowReferenceBindings {} {
  global PT Id2Name

  DeselectAll
  $PT(ModeW) configure -text "Click Transition or Place"
  UnsetAllBindings
  $PT(canvas) bind Node <1> {
    ShowRef $Id2Name([$PT(canvas) find withtag current])
  }  ;# Canvas-Binding <Button-1> fuer den Knoten
  bind . <Escape> { 
    UnsetAllBindings
    EditBindings
  }  ;# Binding fuer die ganze Canvas
}  ;# ShowReferenceBindings

# -------------------------------------------------------------
# Name
#   FireTransition
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Die Transition, die unter Cursor ist (<1> Event wurde
#   auf dieser ausgeloest) wird geschaltet. Wenn es ein Block
#   ist, dann wird eine beliebige, aktive Transition innerhalb
#   des Blockes geschaltet.
proc FireTransition {} {
  global PT

  FireTrans [$PT(canvas) find withtag current]
  FireBindings
}  ;# FireTransition


proc FireTransName {name} {
  global PT

  set PT(trans) $name
  FireTrans 12345
}
  
# -------------------------------------------------------------
# Name
#   FireTransitionLists
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Die Transition, die unter Cursor ist (<1> Event wurde
#   auf dieser ausgeloest) wird geschaltet. Wenn es ein Block
#   ist, dann wird eine beliebige, aktive Transition innerhalb
#   des Blockes geschaltet.
proc FireTransitionLists {List1 {List2 ""}} {
  global PT

  set list1 [string trim $List1 "-sequence="]
  set list2 [string trim $List2 "-loop="]
  set PT(node_list) [split $list1 ","]
  set PT(loop_list) [split $list2 ","]
  set PT(counters) 0
  set PT(counterl) 0
  if {[llength $PT(node_list)] == 0} {
    if {[llength $PT(loop_list)] == 0} {
      $PT(NEXTW) configure -text ""
    } else {
      $PT(NEXTW) configure -text [lindex $PT(loop_list) 0]
    }
  } else {
    $PT(NEXTW) configure -text [lindex $PT(node_list) 0]
  }
  update
  $PT(ModeW) configure -text "Simulation"
#  UnsetAllBindings
#  set PT(AnimationStop) 0
#  focus $PT(canvas)
#  bind . <Escape> {AnimateEnd}
#  $PT(ModeW) configure -text "Animating, Escape to Stop"
#  foreach l $list1 {
#    set PT(trans) $l
#    FireTrans 12345
#    update
#  }
#  while {!$PT(AnimationStop)} {
#    foreach l $list2 {
#      set PT(trans) $l
#      FireTrans 12345
#    }
#    update
#  }
}  ;# FireTransitionLists


# -------------------------------------------------------------
# Name
#   FirePTransition
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Die Transition, die unter Cursor ist (<1> Event wurde
#   auf dieser ausgeloest) wird geschaltet. Wenn es ein Block
#   ist, dann wird eine beliebige, aktive Transition innerhalb
#   des Blockes geschaltet.
proc FirePTransition {} {
  global PT Id2Name

  set ptrans $Id2Name([$PT(canvas) find withtag current])
  set synch [$PT(net) PTransition $ptrans get Synchrons]
  foreach a $synch {
    set PT(trans) $a
    FireTrans 12345
  }
  FireBindings
}  ;# FireTransition


# -------------------------------------------------------------
# Name
#   Animate
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Animation wird durchgefuehrt, solange bis Escape gedrueckt wird.
# Todo: Farben
proc Animate {} {
  global PT

  EndFire
  if {$PT(NetChanged) && $PT(HighlightWhileFire)} {
    tk_dialog .d {Warning!} "Net has been changed. Highlighting may fail." warning 0 {OK}
  }
  .g.fr3b.animate configure -image $PT(toolbar.stop) -command "AnimateEnd"
  set PT(messageText) "Stop Animation"
  bind .g.fr3b.animate \
    <Any-Enter> {set PT(messageText) "Stop Animation"}
  bind .g.fr3b.animate \
    <Any-Leave> {set PT(messageText) ""}
  set active [$PT(net) get ActiveTransitions]
  set all [$PT(net) get Transitions]
  set PT(AnimationStop) 0
  set PT(AnimationNum) 0
  set PT(AnimationTime) [clock seconds]
  UnsetAllBindings
  focus $PT(canvas)
  # herausgenommen, damit das Verschieben waehrend des Animierens klappt
  # grab set $PT(canvas)
  UnsetAllBindings
  bind . <Escape> {AnimateEnd}
  # while active transitions exist and animation is not stopped
  while {[llength $active]>0 && !$PT(AnimationStop)} {
    set PT(InAnimationLoop) 1
    set blocks [$PT(net) get VisBlocks]

    #clear old colors
    foreach b $blocks {
      catch {
        block$block configure -background white
        $PT(canvas) itemconfigure [$PT(canvas) find withtag $block] \
		-image block$block
      }
    }
    catch {
      $PT(canvas) itemconfigure Active -fill white
    }
    $PT(canvas) dtag Active

    #Mark active transitions and blocks
    foreach a $active {
      if {[$PT(net) Transition $a get BlockVis] == "exposed"} {
        $PT(canvas) itemconfigure $a -fill Yellow
        $PT(canvas) addtag Active withtag $a
        $PT(canvas) raise $a
        $PT(canvas) raise Name@$a
      } else {
        set boxes [$PT(net) get Boxes]
        foreach b $boxes {
          catch {
            #set id [.boxframe$b.fenster.canvas find withtag $a]
            .boxframe$b.fenster.canvas itemconfigure $a
            .boxframe$b.fenster.canvas addtag Active withtag $a
            .boxframe$b.fenster.canvas raise $a
            .boxframe$b.fenster.canvas raise Name@$a
          }
        }
        set tblocks [$PT(net) Transition $a get Blocks]
        foreach block $tblocks {
          catch {
            block$block configure -background yellow
            $PT(canvas) itemconfigure [$PT(canvas) find withtag $block] \
		-image block$block
            #$PT(canvas) itemconfigure $block -fill yellow
            #$PT(canvas) addtag Active withtag $block
          }
          $PT(canvas) raise $block
          $PT(canvas) addtag $a withtag $block
          set act [$PT(net) get ActiveTransitionsForBlock $block]
          foreach t $act {
            #$PT(canvas) addtag Active withtag $t
          }
        }  ;# foreach parent block of transition
      }  ;# else transition not shown
    }  ;# foreach active transition

    if {$PT(counters) == [llength $PT(node_list)]} {
      #if just animating
      if {[llength $PT(loop_list)] == 0} {
        $PT(ModeW) configure -text "Animating, Press Esc to Stop"
	#choose randomly active transition
        set random [expr {int(rand()*[llength $active])}]
        set t_index [lindex $active $random]
        if {[$PT(canvas) find withtag $t_index] == ""} {
          set PT(trans) $t_index
          FireTrans 12345
        } else {
          set t [lindex [$PT(canvas) find withtag $t_index] 0]
          FireTrans $t
        }
      } else {
	#else simulate given sequence
        focus $PT(canvas)
        while {!$PT(AnimationStop)} {
          $PT(ModeW) configure -text "Simulation"
	  #choose next transition
          set PT(trans) [lindex $PT(loop_list) $PT(counterl)]
#Added for tic animation, CS 12.6.2001
	  if {$PT(trans) == "tic"} {
	    bell
	  } else {
#
          if {![$PT(net) Transition $PT(trans) get IsActive]} {
            switch [tk_dialog .d {Active Transition Error} "The transition $PT(trans) is not active!" error 0 {Ok}] {
              0 {
                set PT(AnimationStop) 1
                return
              }
            }
          }
          FireTrans 12345
} ;#else real transition
          set PT(counterl) [expr $PT(counterl) + 1]
          if {$PT(counterl) == [llength $PT(loop_list)]} {
            set PT(counterl) 0
          }
          $PT(NEXTW) configure -text [lindex $PT(loop_list) $PT(counterl)]
          update
        }
        if {$PT(counterl) == [llength $PT(loop_list)]} {
          set PT(counterl) 0
        }
      }
    } else {
      while {!$PT(AnimationStop)} {
        $PT(ModeW) configure -text "Simulation"
        focus $PT(canvas)
        set t [lindex $PT(node_list) $PT(counters)]
        #if {![$PT(net) Transition $t get IsActive]} {
       #   return
       # }
        # set t [lindex [$PT(canvas) find withtag $T] 0]
#Added for tic animation, CS 12.6.2001
	  if {$t == "tic"} {
	    bell
	  } else {
#
        if {($t == "") || ([$PT(net) Transition $t get IsActive] == 0)} {
          switch [tk_dialog .d {Active Transition Error} "The transition $t is not active!" error 0 {Ok}] {
            0 {
              set PT(AnimationStop) 1
              return
            }
          }
        }
        set T [lindex [$PT(canvas) find withtag $t] 0]
        if {$T == ""} {
          set PT(trans) $t
          FireTrans 12345
        } else {
          FireTrans $T
        }
} ;#else real transition
        set PT(counters) [expr $PT(counters) + 1]
        if {$PT(counters) == [llength $PT(node_list)]} {
          $PT(NEXTW) configure -text [lindex $PT(loop_list) 0]
        } else {
          $PT(NEXTW) configure -text [lindex $PT(node_list) $PT(counters)]
        }
        update
        if {[llength $PT(node_list)] == $PT(counters)} {
          break
        }
        update
      }
      if {[llength $PT(loop_list)] > 0} {
        while {!$PT(AnimationStop)} {
          set PT(trans) [lindex $PT(loop_list) $PT(counterl)]
#Added for tic animation, CS 12.6.2001
	  if {$PT(trans) == "tic"} {
	    bell
	  } else {
#
          if {![$PT(net) Transition $PT(trans) get IsActive]} {
            switch [tk_dialog .d {Active Transition Error} "The transition $PT(trans) is not active!" error 0 {Ok}] {
              0 {
                set PT(AnimationStop) 1
                return
              }
            }
          }
          FireTrans 12345
} ;#else real transition
          set PT(counterl) [expr $PT(counterl) + 1]
          if {[llength $PT(loop_list)] == $PT(counterl)} {
            set PT(counterl) 0
          }
          $PT(ModeW) configure -text "Simulation"
          $PT(NEXTW) configure -text [lindex $PT(loop_list) $PT(counterl)]
          update
        }
      }
      set PT(AnimationStop) 1
    }

    #clean up for next transition animation
    foreach a $active {
      set tblocks [$PT(net) Transition $a get Blocks]
      foreach block $tblocks {
        $PT(canvas) dtag $block $a
      }
    }
    set active [$PT(net) get ActiveTransitions]
    set all [$PT(net) get Transitions]
    set PT(InAnimationLoop) 0
    ActiveTransitions
    update
  }  ; # while active transitions exist and animation is not stopped
  set PT(InAnimationLoop) 0
  grab release $PT(canvas)
  DeselectAll
  AnimateEnd
}  ;# Animate

# -------------------------------------------------------------
# Name
# AnimateEnd
# --------------
# Parameter
#   Nothing
# --------------
# Description
#   Die Animation wird beendet. Alle Einstellungen werden
#   zur"uckgesetzt
proc AnimateEnd {} {
   global PT Id2Name
   
   .g.fr3b.animate configure -image $PT(toolbar.multistep) -command "Animate"
  set PT(messageText) "Animate"
  bind .g.fr3b.animate \
    <Any-Enter> {set PT(messageText) "Animate"}
  bind .g.fr3b.animate \
    <Any-Leave> {set PT(messageText) ""}

   set PT(AnimationStop) 1
#   puts "$PT(AnimationNum) transitions simulated, [expr [clock seconds] - $PT(AnimationTime)] seconds used."
   set PT(AnimationNum) 0
   # while {$PT(InAnimationLoop) > 0} {after 200}
   set aktive [$PT(canvas) find withtag Active]
   foreach a $aktive {
     catch {
       $PT(canvas) itemconfigure $a -fill $PT(color.transition)
     }
   }
   $PT(canvas) dtag Active
   $PT(canvas) itemconfigure PActive -fill $PT(color.phantom_transition)
   $PT(canvas) dtag PActive
   foreach a [$PT(net) get Blocks] {
     catch {
       block$a configure -background white
       $PT(canvas) itemconfigure block$a -image block$a
     }
   }

   # grab release $PT(canvas)
   UnsetAllBindings
   EditBindings
}  ;# AnimateEnd

  #*** Dateiende NetSim.tcl ***

