# --------------------------------------------------------------------
# Program.tcl
# --------------------------------------------------------------------
# Diese Datei stellt die Dialogboxen zur Programmierung mit Nela 
# zur Verf"ugung
# Ulf Fildebrandt 12/97
# last-update: 11.3.98
# --------------------------------------------------------------------

# -------------------------------------------------------------
# Name
#   Parallel
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Es wird ein Dialogfenster mit zwei Bereichen aufgebaut, die die
#   vollst"andige Boxstruktur des Netzes enthalten. Jeweils eine Box
#   wird ausgew"ahlt und anschlie"send zu einer Box vereinigt.
proc Parallel {} {
  global PT Dialog

  destroy .sd

  set t [toplevel .sd]
  wm title $t {Parallel Dialog}

  set Dialog(block1.selected) ""
  set Dialog(block1.name) ""
  set Dialog(block1.mean) ""
  set Dialog(block1.blockvis) ""

  set Dialog(block2.selected) ""
  set Dialog(block2.name) ""
  set Dialog(block2.mean) ""
  set Dialog(block2.blockvis) ""
    # Canvas
  frame $t.two
  frame $t.two.ce
  frame $t.two.cf
  pack $t.two.cf $t.two.ce -side left -fill both -expand 1 -padx 2 -pady 2
  set c1 [canvas $t.two.cf.canv -width 300 -height 200 -background white \
    -xscrollcommand "$t.two.cf.pad.xscroll set" \
    -yscrollcommand "$t.two.cf.yscroll set" \
    -scrollregion {0 0 1000 1000} \
    -relief sunken -borderwidth 3]
  set c2 [canvas $t.two.ce.canv -width 300 -height 200 -background white \
    -xscrollcommand "$t.two.ce.pad.xscroll set" \
    -yscrollcommand "$t.two.ce.yscroll set" \
    -scrollregion {0 0 1000 1000} \
    -relief sunken -borderwidth 3]
  #set PT(block_tree.canvas) $c1
  scrollbar $t.two.cf.yscroll -command "$t.two.cf.canv yview" \
    -relief sunken -orient vertical -width 12
  scrollbar $t.two.ce.yscroll -command "$t.two.ce.canv yview" \
    -relief sunken -orient vertical -width 12
  frame $t.two.cf.pad
  frame $t.two.ce.pad
  scrollbar $t.two.cf.pad.xscroll -command "$t.two.cf.canv xview" \
    -relief sunken -orient horizontal -width 12
  scrollbar $t.two.ce.pad.xscroll -command "$t.two.ce.canv xview" \
    -relief sunken -orient horizontal -width 12
  set pad1 [expr [$t.two.cf.yscroll cget -width] + 2 * \
    ([$t.two.cf.yscroll cget -bd] + \
     [$t.two.cf.yscroll cget -highlightthickness])]
  set pad2 [expr [$t.two.ce.yscroll cget -width] + 2 * \
    ([$t.two.ce.yscroll cget -bd] + \
     [$t.two.ce.yscroll cget -highlightthickness])]
  frame $t.two.cf.pad.it -width $pad1 -height $pad1
  frame $t.two.ce.pad.it -width $pad2 -height $pad2
  pack $t.two.cf.pad -side bottom -fill x
  pack $t.two.ce.pad -side bottom -fill x
  pack $t.two.cf.pad.it -side right
  pack $t.two.ce.pad.it -side right
  pack $t.two.cf.pad.xscroll -side bottom -fill x
  pack $t.two.ce.pad.xscroll -side bottom -fill x
  pack $t.two.cf.yscroll -side right -fill y
  pack $t.two.ce.yscroll -side right -fill y
  pack $t.two.cf.canv -side left -expand yes -fill both
  pack $t.two.ce.canv -side left -expand yes -fill both

    # Entry Area
  BlockTreeInitTwo $c1
  BlockTreeInitTwo $c2

    # Action Area
  frame $t.action -relief raised -bd 2
  pack $t.action -side top -fill both

  frame $t.action.default -relief sunken -bd 1
    # Ok-Button
  button $t.action.default.ok -text "Parallel" \
    -command {set Dialog(block_tree.button) Parallel}
  pack $t.action.default -side left -expand 1 -padx 3m -pady 2m
  pack $t.action.default.ok 
    # Cancel-Button
  button $t.action.cancel -text "Cancel" \
    -command {set Dialog(block_tree.button) Cancel}
  pack $t.action.cancel -side left -expand 1 -padx 3m -pady 2m
  pack $t.two $t.action -side bottom

  set old_focus [focus]
  tkwait visibility $t
  focus $t

  tkwait variable Dialog(block_tree.button)
  destroy $t
  if {$Dialog(block_tree.button) == "Cancel"} {
    return
  }
  if {$Dialog(block1.name) != "" && $Dialog(block2.name) != ""} {
    $PT(net) Parallel $Dialog(block1.name) $Dialog(block2.name)
  }
  Repaint
}  ;# Parallel


# -------------------------------------------------------------
# Name
#   Sequence
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Die altbekannte Dialogbox wird aufgebaut und zwei Box m"ussen
#   selektiert werden, anschlie"send wird die Box des linken Fensters
#   hinter die Box des rechten Fensters geh"angt.
proc Sequence {} {
  global PT Dialog 

  destroy .sd

  set t [toplevel .sd]
  wm title $t {Sequence Dialog}

  set Dialog(block1.selected) ""
  set Dialog(block1.name) ""
  set Dialog(block1.mean) ""
  set Dialog(block1.blockvis) ""

  set Dialog(block2.selected) ""
  set Dialog(block2.name) ""
  set Dialog(block2.mean) ""
  set Dialog(block2.blockvis) ""

    # Canvas
  frame $t.labels
  label $t.labels.text1 -text "You must select one box in each window. The box in the left window" -anchor w
  label $t.labels.text2 -text "will be executed first. Then the box in the right window will be executed" -anchor w
  pack $t.labels.text2 $t.labels.text1 -side bottom
  frame $t.two
  frame $t.two.ce
  frame $t.two.cf
  pack $t.two.cf $t.two.ce -side left -fill both -expand 1 -padx 2 -pady 2
  set c1 [canvas $t.two.cf.canv -width 300 -height 200 -background white \
    -xscrollcommand "$t.two.cf.pad.xscroll set" \
    -yscrollcommand "$t.two.cf.yscroll set" \
    -scrollregion {0 0 1000 1000} \
    -relief sunken -borderwidth 3]
  set c2 [canvas $t.two.ce.canv -width 300 -height 200 -background white \
    -xscrollcommand "$t.two.ce.pad.xscroll set" \
    -yscrollcommand "$t.two.ce.yscroll set" \
    -scrollregion {0 0 1000 1000} \
    -relief sunken -borderwidth 3]
 # set PT(block_tree.canvas) $c1
  scrollbar $t.two.cf.yscroll -command "$t.two.cf.canv yview" \
    -relief sunken -orient vertical -width 12
  scrollbar $t.two.ce.yscroll -command "$t.two.ce.canv yview" \
    -relief sunken -orient vertical -width 12
  frame $t.two.cf.pad
  frame $t.two.ce.pad
  scrollbar $t.two.cf.pad.xscroll -command "$t.two.cf.canv xview" \
    -relief sunken -orient horizontal -width 12
  scrollbar $t.two.ce.pad.xscroll -command "$t.two.ce.canv xview" \
    -relief sunken -orient horizontal -width 12
  set pad1 [expr [$t.two.cf.yscroll cget -width] + 2 * \
    ([$t.two.cf.yscroll cget -bd] + \
     [$t.two.cf.yscroll cget -highlightthickness])]
  set pad2 [expr [$t.two.ce.yscroll cget -width] + 2 * \
    ([$t.two.ce.yscroll cget -bd] + \
     [$t.two.ce.yscroll cget -highlightthickness])]
  frame $t.two.cf.pad.it -width $pad1 -height $pad1
  frame $t.two.ce.pad.it -width $pad2 -height $pad2
  pack $t.two.cf.pad -side bottom -fill x
  pack $t.two.ce.pad -side bottom -fill x
  pack $t.two.cf.pad.it -side right
  pack $t.two.ce.pad.it -side right
  pack $t.two.cf.pad.xscroll -side bottom -fill x
  pack $t.two.ce.pad.xscroll -side bottom -fill x
  pack $t.two.cf.yscroll -side right -fill y
  pack $t.two.ce.yscroll -side right -fill y
  pack $t.two.cf.canv -side left -expand yes -fill both
  pack $t.two.ce.canv -side left -expand yes -fill both

  BlockTreeInitTwo $c1
  BlockTreeInitTwo $c2

    # Action Area
  frame $t.action -relief raised -bd 2
  pack $t.action -side top -fill both

  frame $t.action.default -relief sunken -bd 1
    # Ok-Button
  button $t.action.default.ok -text "Sequence" \
    -command {set Dialog(block_tree.button) Sequence}
  pack $t.action.default -side left -expand 1 -padx 3m -pady 2m
  pack $t.action.default.ok 
    # Cancel-Button
  button $t.action.cancel -text "Cancel" \
    -command {set Dialog(block_tree.button) Cancel}
  pack $t.action.cancel -side left -expand 1 -padx 3m -pady 2m
  pack $t.labels $t.two $t.action -side bottom

  set old_focus [focus]
  tkwait visibility $t
  focus $t

  tkwait variable Dialog(block_tree.button)
  destroy $t
  if {$Dialog(block_tree.button) == "Cancel"} {
    return
  }
  if {$Dialog(block1.name) != "" && $Dialog(block2.name) != ""} {
    $PT(net) Sequence $Dialog(block1.name) $Dialog(block2.name)
  }
  Repaint
}  ;# Sequence

# -------------------------------------------------------------
# Name
#   Choice
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   In der altbekannten Fensterdarstellung m"ussen wiederum zwei
#   Boxen ausgew"ahlt werden, die anschlie"send in Konflikt
#   gesetzt werden.
proc Choice {} {
  global PT Dialog 

  destroy .sd

  set t [toplevel .sd]
  wm title $t {Choice Dialog}

  set Dialog(block1.selected) ""
  set Dialog(block1.name) ""
  set Dialog(block1.mean) ""
  set Dialog(block1.blockvis) ""

  set Dialog(block2.selected) ""
  set Dialog(block2.name) ""
  set Dialog(block2.mean) ""
  set Dialog(block2.blockvis) ""

    # Canvas
  frame $t.two
  frame $t.two.ce
  frame $t.two.cf
  pack $t.two.cf $t.two.ce -side left -fill both -expand 1 -padx 2 -pady 2
  set c1 [canvas $t.two.cf.canv -width 300 -height 200 -background white \
    -xscrollcommand "$t.two.cf.pad.xscroll set" \
    -yscrollcommand "$t.two.cf.yscroll set" \
    -scrollregion {0 0 1000 1000} \
    -relief sunken -borderwidth 3]
  set c2 [canvas $t.two.ce.canv -width 300 -height 200 -background white \
    -xscrollcommand "$t.two.ce.pad.xscroll set" \
    -yscrollcommand "$t.two.ce.yscroll set" \
    -scrollregion {0 0 1000 1000} \
    -relief sunken -borderwidth 3]
  #set PT(block_tree.canvas) $c1
  scrollbar $t.two.cf.yscroll -command "$t.two.cf.canv yview" \
    -relief sunken -orient vertical -width 12
  scrollbar $t.two.ce.yscroll -command "$t.two.ce.canv yview" \
    -relief sunken -orient vertical -width 12
  frame $t.two.cf.pad
  frame $t.two.ce.pad
  scrollbar $t.two.cf.pad.xscroll -command "$t.two.cf.canv xview" \
    -relief sunken -orient horizontal -width 12
  scrollbar $t.two.ce.pad.xscroll -command "$t.two.ce.canv xview" \
    -relief sunken -orient horizontal -width 12
  set pad1 [expr [$t.two.cf.yscroll cget -width] + 2 * \
    ([$t.two.cf.yscroll cget -bd] + \
     [$t.two.cf.yscroll cget -highlightthickness])]
  set pad2 [expr [$t.two.ce.yscroll cget -width] + 2 * \
    ([$t.two.ce.yscroll cget -bd] + \
     [$t.two.ce.yscroll cget -highlightthickness])]
  frame $t.two.cf.pad.it -width $pad1 -height $pad1
  frame $t.two.ce.pad.it -width $pad2 -height $pad2
  pack $t.two.cf.pad -side bottom -fill x
  pack $t.two.ce.pad -side bottom -fill x
  pack $t.two.cf.pad.it -side right
  pack $t.two.ce.pad.it -side right
  pack $t.two.cf.pad.xscroll -side bottom -fill x
  pack $t.two.ce.pad.xscroll -side bottom -fill x
  pack $t.two.cf.yscroll -side right -fill y
  pack $t.two.ce.yscroll -side right -fill y
  pack $t.two.cf.canv -side left -expand yes -fill both
  pack $t.two.ce.canv -side left -expand yes -fill both

  BlockTreeInitTwo $c1
  BlockTreeInitTwo $c2

    # Action Area
  frame $t.action -relief raised -bd 2
  pack $t.action -side top -fill both

  frame $t.action.default -relief sunken -bd 1
    # Ok-Button
  button $t.action.default.ok -text "Choice" \
    -command {set Dialog(block_tree.button) Sequence}
  pack $t.action.default -side left -expand 1 -padx 3m -pady 2m
  pack $t.action.default.ok 
    # Cancel-Button
  button $t.action.cancel -text "Cancel" \
    -command {set Dialog(block_tree.button) Cancel}
  pack $t.action.cancel -side left -expand 1 -padx 3m -pady 2m
  pack $t.two $t.action -side bottom

  set old_focus [focus]
  tkwait visibility $t
  focus $t

  tkwait variable Dialog(block_tree.button)
  destroy $t
  if {$Dialog(block_tree.button) == "Cancel"} {
    return
  }
  if {$Dialog(block1.name) != "" && $Dialog(block2.name) != ""} {
    $PT(net) Choice $Dialog(block1.name) $Dialog(block2.name)
  }
  Repaint
}  ;# Choice

# -------------------------------------------------------------
# Name
#   Instantiate
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Es wird ein Fenster aufgebaut, in dem unten die Boxhierarchie
#   dargestellt wird. Nachdem eine Box selektiert wurde, erscheinen
#   die Makros im oberen Fenster und lassen sich "uber die 
#   Eingabezeile "andern.
proc Instantiate {} {
  global PT Dialog 

  destroy .ind

  set t [toplevel .ind]
  wm title $t {Instantiate Dialog}

  set Dialog(Bind) ""
  set Dialog(block1.selected) ""
  set Dialog(block1.name) ""
  set Dialog(block1.mean) ""
  set Dialog(block1.blockvis) ""

  set Dialog(block2.selected) ""
  set Dialog(block2.name) ""
  set Dialog(block2.mean) ""
  set Dialog(block2.blockvis) ""

    # Canvas
  frame $t.two
  frame $t.two.cf
  pack $t.two.cf -side left -fill both -expand 1 -padx 2 -pady 2
  set c1 [canvas $t.two.cf.canv -width 300 -height 200 -background white \
    -xscrollcommand "$t.two.cf.pad.xscroll set" \
    -yscrollcommand "$t.two.cf.yscroll set" \
    -scrollregion {0 0 1000 1000} \
    -relief sunken -borderwidth 3]
  #set PT(block_tree.canvas) $c1
  scrollbar $t.two.cf.yscroll -command "$t.two.cf.canv yview" \
    -relief sunken -orient vertical -width 12
  frame $t.two.cf.pad
  scrollbar $t.two.cf.pad.xscroll -command "$t.two.cf.canv xview" \
    -relief sunken -orient horizontal -width 12
  set pad1 [expr [$t.two.cf.yscroll cget -width] + 2 * \
    ([$t.two.cf.yscroll cget -bd] + \
     [$t.two.cf.yscroll cget -highlightthickness])]
  frame $t.two.cf.pad.it -width $pad1 -height $pad1
  pack $t.two.cf.pad -side bottom -fill x
  pack $t.two.cf.pad.it -side right
  pack $t.two.cf.pad.xscroll -side bottom -fill x
  pack $t.two.cf.yscroll -side right -fill y
  pack $t.two.cf.canv -side left -expand yes -fill both
  frame $t.top -bd 1 -relief raised
  pack $t.top -side top -fill both -expand yes

  frame $t.top.list 
  frame $t.top.list.vars -bd 1 -relief sunken
  pack $t.top.list.vars -side left -expand yes -fill x
  label $t.top.list.vars.label -text "Macros in Object Net"

  frame $t.top.list.vars.f
  pack $t.top.list.vars.f -side bottom

  listbox $t.top.list.vars.f.l \
    -yscrollcommand "$t.top.list.vars.f.scroll set" -height 10
  bind $t.top.list.vars.f.l <1> {global Dialog
                                   set Dialog(Variable) [.ind.top.list.vars.f.l get @%x,%y]
                                   set Dialog(Bind) $Dialog($Dialog(Variable))
                                }
  scrollbar $t.top.list.vars.f.scroll -command "$t.top.list.vars.f.l yview"
  pack $t.top.list.vars.label -side top
  pack $t.top.list.vars.f.l -side left
  pack $t.top.list.vars.f.scroll -side left -fill y
  pack $t.top.list -side top -expand yes -fill x

  entry $t.top.bind -relief sunken -textvariable Dialog(Bind)
  pack $t.top.bind -side bottom -expand yes -fill x
  label $t.top.macro -textvariable Dialog(Variable)
  pack $t.top.macro -side bottom -expand yes -fill x
  bind $t.top.bind <Return> {global Dialog;set Dialog($Dialog(Variable)) $Dialog(Bind)}

    # Entry Area
  BlockTreeInitTwo $c1

    # Action Area
  frame $t.action -relief raised -bd 2
  pack $t.action -side top -fill both

  frame $t.action.default -relief sunken -bd 1
    # Ok-Button
  button $t.action.default.ok -text "Instantiate" \
    -command {set Dialog(block_tree.button) Instantiate}
  pack $t.action.default -side left -expand 1 -padx 3m -pady 2m
  pack $t.action.default.ok 
    # Cancel-Button
  button $t.action.cancel -text "Cancel" \
    -command {set Dialog(block_tree.button) Cancel}
  pack $t.action.cancel -side left -expand 1 -padx 3m -pady 2m
  pack $t.two $t.action -side bottom

  set old_focus [focus]
  tkwait visibility $t
  focus $t

  tkwait variable Dialog(block_tree.button)
  destroy $t
  if {$Dialog(block_tree.button) == "Cancel"} {
    return
  }
  if {$Dialog(block2.name) == ""} {
    return
  }
  set Macros [$PT(net) SearchMacros $Dialog(block2.name)]
  set z ""
  foreach m $Macros {
    if {$z != ""} {
      set z [concat $z~$m=$Dialog($m)]
    } else {
      set z [concat $m=$Dialog($m)]
    }
  }
  $PT(net) InsertMacros $Dialog(block2.name) $z
  Repaint
}  ;# Instantiate

# -------------------------------------------------------------
# Name
#   Synchronize
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Die ausgewaehlten Knoten werden ins clipboard kopiert, dort
#   synchronisiert und anschlie"send wieder ins Netz eingef"ugt
proc Synchronize {} {
  global PT Id2Name Blocks

  if {[ActionFrame] == 0} {
    return
  }

  set SELECTED ""
  foreach t [$PT(canvas) find withtag Transition] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag Place] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag PTransition] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag Block] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag Box] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  $PT(net) Synchronize [llength [split $SELECTED]] $SELECTED
  NetChanged

  set selected [$PT(net) Selected]
  set node_list [split $selected ","]

  set l [llength $node_list]
  set node_list [lrange $node_list 0 [expr $l - 2]]
  foreach t $node_list {
    if {[$PT(net) Exists Transition $t] > 0} {
      $PT(net) Transition $t set \
        x [expr [$PT(net) Transition $t get x] + int($PT(OSizeHalf))] \
        y [expr [$PT(net) Transition $t get y] + int($PT(OSizeHalf))]
    }
    if {[$PT(net) Exists PTransition $t] > 0} {
      $PT(net) PTransition $t set \
        x [expr [$PT(net) PTransition $t get x] + int($PT(OSizeHalf))] \
        y [expr [$PT(net) PTransition $t get y] + int($PT(OSizeHalf))]
    }
    if {[$PT(net) Exists Place $t] > 0} {
      $PT(net) Place $t set \
        x [expr [$PT(net) Place $t get x] + int($PT(OSizeHalf))] \
        y [expr [$PT(net) Place $t get y] + int($PT(OSizeHalf))]
    }
    if {[$PT(net) Exists Block $t] > 0} {
      set Blocks($t) 0
    }
  } 
  NetChanged

  Repaint
} ;# Synchronize

# -------------------------------------------------------------
# Name
#   Restrict
# ---------
# Parameter 
#   Nothing
# ---------
# Description
#   Die ausgewaehlten Knoten werden ins clipboard kopiert, dort
#   werden die Transitionen mit komplementaeren Actiontermen entfernt
#   und anschlie"send wieder ins Netz eingef"ugt
proc Restrict {} {
  global PT Id2Name Blocks

  if {[ActionFrame] == 0} {
    return
  }

  set SELECTED ""
  foreach t [$PT(canvas) find withtag Transition] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag Place] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag PTransition] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag Block] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag Box] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  $PT(net) Restrict $SELECTED

  set selected [$PT(net) Selected]
  set node_list [split $selected ","]

  set l [llength $node_list]
  set node_list [lrange $node_list 0 [expr $l - 2]]
  foreach t $node_list {
    if {[$PT(net) Exists Transition $t] > 0} {
      $PT(net) Transition $t set \
        x [expr [$PT(net) Transition $t get x] + int($PT(OSizeHalf))] \
        y [expr [$PT(net) Transition $t get y] + int($PT(OSizeHalf))]
    }
    if {[$PT(net) Exists PTransition $t] > 0} {
      $PT(net) PTransition $t set \
        x [expr [$PT(net) PTransition $t get x] + int($PT(OSizeHalf))] \
        y [expr [$PT(net) PTransition $t get y] + int($PT(OSizeHalf))]
    }
    if {[$PT(net) Exists Place $t] > 0} {
      $PT(net) Place $t set \
        x [expr [$PT(net) Place $t get x] + int($PT(OSizeHalf))] \
        y [expr [$PT(net) Place $t get y] + int($PT(OSizeHalf))]
    }
    if {[$PT(net) Exists Block $t] > 0} {
      set Blocks($t) 0
    }
  }

  Repaint
} ;# Restrict

# -------------------------------------------------------------
# Name
#   Tie
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Es wird ein Fenster aufgebaut, in dem Tie-Links deklariert
#   und fuer die Tie-Operation ausgewaehlt werden koennen. 
#   Ein Druck auf den Tie-Button fuehrt die Operation durch
#   (alles im C++-Kern).
proc Tie {} {
  global PT Dialog

  if [catch {set tl [toplevel .exe_hl_tie_dlg]}] {
    raise .exe_hl_tie_dlg
    return
  }  ;# if already open
  wm title $tl "Tie - Asynchronous Links"
  wm protocol $tl WM_DELETE_WINDOW "destroy $tl"

    # Eingabebereich
  frame $tl.decls
  pack $tl.decls -side top -expand 1 -fill both
  label $tl.decls.label -text "Link declarations" -bd 2
  pack $tl.decls.label -side top -fill x
  frame $tl.decls.list
  pack $tl.decls.list -side top -fill both -expand 1
  listbox $tl.decls.list.lb -bd 2 -selectmode extended
  pack $tl.decls.list.lb -side left -fill both -expand 1
  frame $tl.decls.entry
  pack $tl.decls.entry -side top -fill x -expand 1
  label $tl.decls.entry.namelabel -text "Name"
  entry $tl.decls.entry.nameentry
  label $tl.decls.entry.typelabel -text "Type"
  entry $tl.decls.entry.typeentry
  pack $tl.decls.entry.namelabel\
    $tl.decls.entry.nameentry $tl.decls.entry.typelabel\
    $tl.decls.entry.typeentry\
    -side left -fill x -expand 1

  set links [$PT(net) get Links]
  while {[llength $links]} {
    $tl.decls.list.lb insert end "[lindex $links 0] [lindex $links 1]"
    set links [lrange $links 2 end]
  }

  bind $tl.decls.list.lb <ButtonRelease-1> {tie_listbox2entry %x %y}
# TODO: cursor is possibly wrong (without coordinates!)
  bind $tl.decls.list.lb <Up> {tie_listbox2entry %x %y}
  bind $tl.decls.list.lb <Down> {tie_listbox2entry %x %y}

  frame $tl.decls.button
  pack $tl.decls.button -side top -expand 1 -fill x
  button $tl.decls.button.add -text "Add"\
    -command "tie_link add $tl;\
		$tl.decls.entry.nameentry delete 0 end;\
		$tl.decls.entry.typeentry delete 0 end"
  button $tl.decls.button.edit -text "Change"\
    -command {tie_link change .exe_hl_tie_dlg}
  button $tl.decls.button.delete -text "Delete"\
    -command {tie_link delete .exe_hl_tie_dlg;\
		.exe_hl_tie_dlg.decls.entry.nameentry delete 0 end;\
		.exe_hl_tie_dlg.decls.entry.typeentry delete 0 end;Repaint}
  pack $tl.decls.button.add \
    $tl.decls.button.edit \
    $tl.decls.button.delete \
    -side left -expand 1

    # Button-Bereich
  frame $tl.commands -relief raised -bd 2
  pack $tl.commands -side bottom -fill x
  button $tl.commands.ok -text "Tie" \
    -command { set Dialog(block_tree.button) Tie }
  button $tl.commands.cancel -text "Close" \
    -command { set Dialog(block_tree.button) Cancel }
  pack $tl.commands.ok \
    $tl.commands.cancel \
    -side left -expand 1

  tkwait variable Dialog(block_tree.button)
  if {$Dialog(block_tree.button) == "Cancel"} {
    destroy $tl
    return
  }
  set selection ""
  foreach selnum "[$tl.decls.list.lb curselection]" {
    lappend selection [lindex [split [$tl.decls.list.lb get $selnum]] 0]
  }
  if {$selection != ""} {
    $PT(net) Tie [join $selection ,]
    NetChanged
  }
  destroy $tl
  Repaint
} ;# Tie

proc tie_listbox2entry {x y} {
  set active [split [.exe_hl_tie_dlg.decls.list.lb get @$x,$y]]
  .exe_hl_tie_dlg.decls.entry.nameentry delete 0 end
  .exe_hl_tie_dlg.decls.entry.typeentry delete 0 end
  .exe_hl_tie_dlg.decls.entry.nameentry insert 0 [lindex $active 0]
  .exe_hl_tie_dlg.decls.entry.typeentry insert 0 [lindex $active 1]
}

proc tie_link {command tl} {
  global PT

  set type [$tl.decls.entry.typeentry get]
  if {[string first \{ $type] == -1} {
    set type "\{$type\}"
  }
  set string [concat [$tl.decls.entry.nameentry get] $type]

  if {![catch {$PT(net) Link $command [$tl.decls.entry.nameentry get] $type}]} {
    switch $command {
      add {$tl.decls.list.lb insert end "$string"}
      delete {$tl.decls.list.lb delete active}
      change {set line [$tl.decls.list.lb index active]
		$tl.decls.list.lb delete $line
		$tl.decls.list.lb insert $line "$string";
		$tl.decls.list.lb activate $line
		$tl.decls.list.lb selection set $line}
    }
    NetChanged
  } else {
    puts {Error in Link call}
  }
}

proc TakeInBox {} {
  global PT Id2Name

  set visblocks [$PT(net) get VisBlocks]    
  set visplaces [$PT(net) get VisPlaces]    
  set vistransitions [$PT(net) get VisTransitions]
  set visptransitions [$PT(net) get VisPTransitions]
  set edges [$PT(canvas) find withtag Edge]
  set SELECTED ""
  foreach t [$PT(canvas) find withtag Transition] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag Place] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag PTransition] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag Block] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  if {$SELECTED != ""} {
    set b [$PT(net) TakeInBox $SELECTED]
    set GX 0
    set GY 0
    foreach t $SELECTED {
      set GX [expr $GX + [$PT(net) [GetType $t] $t get x]]
      set GY [expr $GY + [$PT(net) [GetType $t] $t get y]]
    }
    set x [expr int($GX / [llength $SELECTED])]
    set y [expr int($GY / [llength $SELECTED])]
    $PT(net) Box $b set x $x
    $PT(net) Box $b set y $y
    Repaint
  }
} ;# TakeInBox

proc AddToBox {box} {
  global PT Id2Name Dialog

  set visblocks [$PT(net) get VisBlocks]    
  set visplaces [$PT(net) get VisPlaces]    
  set vistransitions [$PT(net) get VisTransitions]
  set visptransitions [$PT(net) get VisPTransitions]
  set edges [$PT(canvas) find withtag Edge]
  set SELECTED ""
  foreach t [$PT(canvas) find withtag Transition] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag Place] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag PTransition] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  foreach t [$PT(canvas) find withtag Block] {
    set tags [$PT(canvas) gettags $t]
    if {[lsearch -regexp $tags selected] > -1} {
      set SELECTED [concat $SELECTED $Id2Name($t)]
    }
  }
  if {$SELECTED != "" && $box != ""} {
    $PT(net) AddToBox $SELECTED $box
    Repaint
  }
} ;# AddToBox

# -------------------------------------------------------------
# Name
#   AddToBoxDialog
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Im Netz m"ussen einige Knoten selektiert sein, so da"s man
#   im Dialogfenster die Box angeben kann, zu der diese Knoten
#   jetzt geh"oren sollen.
proc AddToBoxDialog {} {
  global PT Dialog 

  destroy .add

  set t [toplevel .add]
  wm title $t {AddToBox Dialog}

  set Dialog(block1.selected) ""
  set Dialog(block1.name) ""
  set Dialog(block1.mean) ""
  set Dialog(block1.blockvis) ""

  set Dialog(block2.selected) ""
  set Dialog(block2.name) ""
  set Dialog(block2.mean) ""
  set Dialog(block2.blockvis) ""

    # Canvas
  frame $t.two
  frame $t.two.cf
  pack $t.two.cf -side left -fill both -expand 1 -padx 2 -pady 2
  set c1 [canvas $t.two.cf.canv -width 300 -height 200 -background white \
    -xscrollcommand "$t.two.cf.pad.xscroll set" \
    -yscrollcommand "$t.two.cf.yscroll set" \
    -scrollregion {0 0 1000 1000} \
    -relief sunken -borderwidth 3]
  #set PT(block_tree.canvas) $c1
  scrollbar $t.two.cf.yscroll -command "$t.two.cf.canv yview" \
    -relief sunken -orient vertical -width 12
  frame $t.two.cf.pad
  scrollbar $t.two.cf.pad.xscroll -command "$t.two.cf.canv xview" \
    -relief sunken -orient horizontal -width 12
  set pad1 [expr [$t.two.cf.yscroll cget -width] + 2 * \
    ([$t.two.cf.yscroll cget -bd] + \
     [$t.two.cf.yscroll cget -highlightthickness])]
  frame $t.two.cf.pad.it -width $pad1 -height $pad1
  pack $t.two.cf.pad -side bottom -fill x
  pack $t.two.cf.pad.it -side right
  pack $t.two.cf.pad.xscroll -side bottom -fill x
  pack $t.two.cf.yscroll -side right -fill y
  pack $t.two.cf.canv -side left -expand yes -fill both
  frame $t.top -bd 1 -relief raised
  pack $t.top -side top -fill both -expand yes

  frame $t.top.list 
  frame $t.top.list.vars -bd 1 -relief sunken
  pack $t.top.list.vars -side left -expand yes -fill x
  label $t.top.list.vars.label -text "Transitions in Net"

  frame $t.top.list.vars.f
  pack $t.top.list.vars.f -side bottom
  scrollbar $t.top.list.vars.f.scroll -command "$t.top.list.vars.f.l yview"

  pack $t.top.list -side top -expand yes -fill x

    # Entry Area
  BlockTreeInitTwo $c1

    # Action Area
  frame $t.action -relief raised -bd 2
  pack $t.action -side top -fill both

  frame $t.action.default -relief sunken -bd 1
    # Ok-Button
  button $t.action.default.ok -text "AddToBox" \
    -command {set Dialog(block_tree.button) Instantiate}
  pack $t.action.default -side left -expand 1 -padx 3m -pady 2m
  pack $t.action.default.ok 
    # Cancel-Button
  button $t.action.cancel -text "Cancel" \
    -command {set Dialog(block_tree.button) Cancel}
  pack $t.action.cancel -side left -expand 1 -padx 3m -pady 2m
  pack $t.two $t.action -side bottom

  set old_focus [focus]
  tkwait visibility $t
  focus $t

  tkwait variable Dialog(block_tree.button)
  destroy $t
  if {$Dialog(block_tree.button) == "Cancel"} {
    return
  }
  if {$Dialog(block2.name) != ""} {
    AddToBox $Dialog(block2.name)
  }
}  ;# AddToBoxDialog

# -------------------------------------------------------------
# Name
#   AppendSuffix
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Es wird ein Suffix angegeben und eine Box. An alle Name der
#   Knoten dieser Box wird der Suffix angeh"angt.
proc AppendSuffix {} {
  global PT Dialog Id2Name

  destroy .app

  set t [toplevel .app]
  wm title $t {AddToBox Dialog}

  set Dialog(block1.selected) ""
  set Dialog(block1.name) ""
  set Dialog(block1.mean) ""
  set Dialog(block1.blockvis) ""

  set Dialog(block2.selected) ""
  set Dialog(block2.name) ""
  set Dialog(block2.mean) ""
  set Dialog(block2.blockvis) ""

    # Canvas
  frame $t.suff
  entry $t.suff.suf -textvariable Dialog(suffix)
  pack $t.suff.suf
  frame $t.two
  frame $t.two.cf
  pack $t.two.cf -side left -fill both -expand 1 -padx 2 -pady 2
  set c1 [canvas $t.two.cf.canv -width 300 -height 200 -background white \
    -xscrollcommand "$t.two.cf.pad.xscroll set" \
    -yscrollcommand "$t.two.cf.yscroll set" \
    -scrollregion {0 0 1000 1000} \
    -relief sunken -borderwidth 3]
  #set PT(block_tree.canvas) $c1
  scrollbar $t.two.cf.yscroll -command "$t.two.cf.canv yview" \
    -relief sunken -orient vertical -width 12
  frame $t.two.cf.pad
  scrollbar $t.two.cf.pad.xscroll -command "$t.two.cf.canv xview" \
    -relief sunken -orient horizontal -width 12
  set pad1 [expr [$t.two.cf.yscroll cget -width] + 2 * \
    ([$t.two.cf.yscroll cget -bd] + \
     [$t.two.cf.yscroll cget -highlightthickness])]
  frame $t.two.cf.pad.it -width $pad1 -height $pad1
  pack $t.two.cf.pad -side bottom -fill x
  pack $t.two.cf.pad.it -side right
  pack $t.two.cf.pad.xscroll -side bottom -fill x
  pack $t.two.cf.yscroll -side right -fill y
  pack $t.two.cf.canv -side left -expand yes -fill both
  frame $t.top -bd 1 -relief raised
  pack $t.suff $t.top -side top -fill both -expand yes

  frame $t.top.list 
  frame $t.top.list.vars -bd 1 -relief sunken
  pack $t.top.list.vars -side left -expand yes -fill x
  label $t.top.list.vars.label -text "Transitions in Net"

  frame $t.top.list.vars.f
  pack $t.top.list.vars.f -side bottom
  scrollbar $t.top.list.vars.f.scroll -command "$t.top.list.vars.f.l yview"

  pack $t.top.list -side top -expand yes -fill x

    # Entry Area
  BlockTreeInitTwo $c1

    # Action Area
  frame $t.action -relief raised -bd 2
  pack $t.action -side top -fill both

  frame $t.action.default -relief sunken -bd 1
    # Ok-Button
  button $t.action.default.ok -text "Append Suffix" \
    -command {set Dialog(block_tree.button) Instantiate}
  pack $t.action.default -side left -expand 1 -padx 3m -pady 2m
  pack $t.action.default.ok 
    # Cancel-Button
  button $t.action.cancel -text "Cancel" \
    -command {set Dialog(block_tree.button) Cancel}
  pack $t.action.cancel -side left -expand 1 -padx 3m -pady 2m
  pack $t.two $t.action -side bottom

  set old_focus [focus]
  tkwait visibility $t
  focus $t

  tkwait variable Dialog(block_tree.button)
  destroy $t
  if {$Dialog(block_tree.button) == "Cancel"} {
    return
  }
  if {$Dialog(block2.name) != ""} {
    $PT(net) AppendSuffix $Dialog(suffix) $Dialog(block2.name)
    Repaint
  }
}  ;# AppendSuffix

# -------------------------------------------------------------
# Name
#   Substitution
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Eine Dialogbox wird aufgebaut, in der oben alle Transitionen
#   des Netzes dargestellt werden, dadrunter befindet sich eine
#   Eingabezeile, die angibt, welche Transition aktuell ersetzt
#   wird. Im unteren Bereich l"a"st sich eine Box ausw"ahlen, die
#   f"ur die Transition substituiert wird.
proc Substitution {} {
  global PT Dialog 

  destroy .sub

  set t [toplevel .sub]
  wm title $t {Substitution Dialog}

  set Dialog(block1.selected) ""
  set Dialog(block1.name) ""
  set Dialog(block1.mean) ""
  set Dialog(block1.blockvis) ""

  set Dialog(block2.selected) ""
  set Dialog(block2.name) ""
  set Dialog(block2.mean) ""
  set Dialog(block2.blockvis) ""

    # Canvas
  frame $t.two
  frame $t.two.cf
  pack $t.two.cf -side left -fill both -expand 1 -padx 2 -pady 2
  set c1 [canvas $t.two.cf.canv -width 300 -height 200 -background white \
    -xscrollcommand "$t.two.cf.pad.xscroll set" \
    -yscrollcommand "$t.two.cf.yscroll set" \
    -scrollregion {0 0 1000 1000} \
    -relief sunken -borderwidth 3]
  #set PT(block_tree.canvas) $c1
  scrollbar $t.two.cf.yscroll -command "$t.two.cf.canv yview" \
    -relief sunken -orient vertical -width 12
  frame $t.two.cf.pad
  scrollbar $t.two.cf.pad.xscroll -command "$t.two.cf.canv xview" \
    -relief sunken -orient horizontal -width 12
  set pad1 [expr [$t.two.cf.yscroll cget -width] + 2 * \
    ([$t.two.cf.yscroll cget -bd] + \
     [$t.two.cf.yscroll cget -highlightthickness])]
  frame $t.two.cf.pad.it -width $pad1 -height $pad1
  pack $t.two.cf.pad -side bottom -fill x
  pack $t.two.cf.pad.it -side right
  pack $t.two.cf.pad.xscroll -side bottom -fill x
  pack $t.two.cf.yscroll -side right -fill y
  pack $t.two.cf.canv -side left -expand yes -fill both
  frame $t.top -bd 1 -relief raised
  pack $t.top -side top -fill both -expand yes

  frame $t.top.list 
  frame $t.top.list.vars -bd 1 -relief sunken
  pack $t.top.list.vars -side left -expand yes -fill x
  label $t.top.list.vars.label -text "Transitions in Net"

  frame $t.top.list.vars.f
  pack $t.top.list.vars.f -side bottom

  listbox $t.top.list.vars.f.l \
    -yscrollcommand "$t.top.list.vars.f.scroll set" -height 10
  set Trans [$PT(net) get Transitions]
  .sub.top.list.vars.f.l delete 0 1000
  foreach i $Trans {
    .sub.top.list.vars.f.l insert end $i
    set Dialog($i) $i
  }
  bind $t.top.list.vars.f.l <1> {global Dialog
                                   set Dialog(Bind) $Dialog([.sub.top.list.vars.f.l get @%x,%y])
                                }
  scrollbar $t.top.list.vars.f.scroll -command "$t.top.list.vars.f.l yview"
  pack $t.top.list.vars.label -side top
  pack $t.top.list.vars.f.l -side left
  pack $t.top.list.vars.f.scroll -side left -fill y

  pack $t.top.list -side top -expand yes -fill x
  entry $t.top.bind -relief sunken -textvariable Dialog(Bind)
  pack $t.top.bind -side bottom -expand yes -fill x
  bind $t.top.bind <Return> {global Dialog;set Dialog([.sub.list.vars.f.l get @%x,%y]) $Dialog(Bind)}

    # Entry Area
  BlockTreeInitTwo $c1

    # Action Area
  frame $t.action -relief raised -bd 2
  pack $t.action -side top -fill both

  frame $t.action.default -relief sunken -bd 1
    # Ok-Button
  button $t.action.default.ok -text "Substitute" \
    -command {set Dialog(block_tree.button) Instantiate}
  pack $t.action.default -side left -expand 1 -padx 3m -pady 2m
  pack $t.action.default.ok 
    # Cancel-Button
  button $t.action.cancel -text "Cancel" \
    -command {set Dialog(block_tree.button) Cancel}
  pack $t.action.cancel -side left -expand 1 -padx 3m -pady 2m
  pack $t.two $t.action -side bottom

  set old_focus [focus]
  tkwait visibility $t
  focus $t

  tkwait variable Dialog(block_tree.button)
  destroy $t
  if {$Dialog(block_tree.button) == "Cancel"} {
    return
  }
  if {$Dialog(Bind) != "" && $Dialog(block2.name) != ""} {
    $PT(net) Substitution $Dialog(block2.name) $Dialog(Bind)
  }
  Repaint
}  ;# Substitution

# -------------------------------------------------------------
# Name
#   BlockTreeInitTwo
# ---------
# Eingabe 
#   c - canvas, in das die Boxhierarchie gezeichnet wird
# ---------
# Beschreibung
#   Im Canvas c werden die Boxen des Netzs dargestellt. Da
#   es in den Dialogen mehr als eine Boxdarstellung geben kann
#   mu"s die Funktion abh"angig sein vom Canvas.
proc BlockTreeInitTwo {c} {
  global PT Dialog

    # die blichen Werte eintragen
  set PT(block_tree.dx) 40
  set PT(block_tree.dy) 40
  set PT(block_tree.block_halfsize) 15
  set PT(block_tree.color.exposed) green
  set PT(block_tree.color.hidden) lightgray
  set PT(block_tree.color.exposed_contents) blue
    # Schleife ber die Wurzelblcke vorbereiten
  set lx $PT(block_tree.dx)
  set next_x 0
  set boxes [$PT(net) get Boxes]
  foreach box $boxes {
    BoxTreeVisitTwo $box $c 0 0 $lx $PT(block_tree.dy) next_x
    set lx [expr $next_x + $PT(block_tree.dx)]
  }  ;# foreach box in den Boxen
  $c bind BoxNode <Button-1> {
    BlockTreeSelectBlockTwo [%W gettags [%W find withtag current]] %W
  }
  set Dialog(box_tree.selected) [lindex $boxes 0]
  if {[string length $Dialog(box_tree.selected)] == 0} {
    set $Dialog(box_tree.selected) "no block"
    $c create text 20 20 \
      -text "There are no boxes in the net!" -anchor w
  }
}  ;# BlockTreeInit

# -------------------------------------------------------------
# Name
#   BlockTreeDrawBoxTwo
# ---------
# Eingabe 
#   b - Name der Box
#   x - X-Koordinate des Boxnamens
#   y - Y-Koordinate des Boxnamens
#   c - canvas, in das die Box gemalt werden soll
# ---------
# Beschreibung
#   Es wird das Symbol f"ur eine Box in das "ubergebene canvas
#   gemalt, versehen mit dem Namen b.
proc BlockTreeDrawBoxTwo {b x y c} {
  global PT

  $c create rectangle \
    [expr $x - $PT(block_tree.block_halfsize)] \
    [expr $y - $PT(block_tree.block_halfsize)] \
    [expr $x + $PT(block_tree.block_halfsize)] \
    [expr $y + $PT(block_tree.block_halfsize)] \
    -width 2 -fill grey -outline black \
    -tags "$b BoxNode $b@Box"
  $c create text $x $y \
    -text $b -anchor c -tags "$b BoxNode $b@Name"
}  ;# BlockTreeDrawBox

# -------------------------------------------------------------
# Name
#   BlockTreeVisitTwo
# ---------
# Eingabe 
#   b   - Name der Box
#   can - canvas, in das die Box positioniert wird
#   xo  - 
#   yo  - 
#   x   - X-Koordinate des Boxnamens
#   y   - Y-Koordinate des Boxnamens
#   new_x - gibt an, um wieviele Pixel die n"achste
#           Box verschoben werden soll.
# ---------
# Beschreibung
#   Erstellt eine Box im Canvas can und "andert die
#   Positioniervariablen.
proc BoxTreeVisitTwo {b can xo yo x y new_x} {
  global PT
  upvar $new_x nx

  set lx $x
  set next_x $lx
  BlockTreeDrawBoxTwo $b $x $y $can
  set nx $lx
}  ;# BoxTreeVisitTwo

# -------------------------------------------------------------
# Name
#   BlockTreeSelectBlockTwo
# ---------
# Eingabe 
#   taglist - Die Tagliste des selektierten Objektes
#   c       - Das canvas, in dem selektiert wurde
# ---------
# Beschreibung
#   Es wurde ein Objekt im canvas c selektiert und die
#   Tagliste wurde "ubergeben.
proc BlockTreeSelectBlockTwo {taglist c} {
  global PT Dialog

    # den Block im Netz highlighten
  #$PT(canvas) itemconfigure BlockMark -fill white
  #$PT(canvas) dtag BlockMark BlockMark
    # einen evtl. vorhandene vorherige Selektion aufheben
  $c itemconfigure Selected -fill grey
  $c dtag Selected
    # den Block identifizieren
  set index [lsearch -regexp $taglist ".*@Name|.*@Box"]
  set name [lindex [split [lindex $taglist $index] "@"] 0]
  if {([string length $name] != 0) && ([$PT(net) Exists Box $name] != 0)} {
      # Daten holen und ausgeben
    if {$c == ".sd.two.cf.canv"} {
      set Dialog(block1.selected) $name
      set Dialog(block1.name) $name
      set Dialog(block1.mean) [$PT(net) Box $name get Mean]
      set Dialog(block1.blockvis) [$PT(net) Box $name get Visibility]
    } else {
      if {$c == ".ind.two.cf.canv"} {
        set Macros [$PT(net) SearchMacros $name]
        set Dialog(block2.name) $name
        set counter 0
        .ind.top.list.vars.f.l delete 0 1000
        foreach i $Macros {
          .ind.top.list.vars.f.l insert end $i
          set Dialog($i) $counter
          if {$i == "ID_SET"} {
            set Dialog($i) "{dot}"
          }
          if {$i == "NAME"} {
            set Dialog($i) "v"
          }
          if {$i == "DECL_SET"} {
            set Dialog($i) "{0..2}"
          }
          if {$i == "REF"} {
            set Dialog($i) ""
          }
          if {$i == "INIT"} {
            set Dialog($i) "0"
          }
          if {$i == "EXIT"} {
            set Dialog($i) "b"
          }
          if {$i == "CAPACITY"} {
            set Dialog($i) "2"
          }
          if {$i == "BODY"} {
            set Dialog($i) ""
          }
          if {$i == "PAR_POST"} {
            set Dialog($i) ""
          }
          if {$i == "PAR_PRE"} {
            set Dialog($i) ""
          }
          if {$i == "MAX"} {
            set Dialog($i) "4"
          }
          if {$i == "NEW_IDS"} {
            set Dialog($i) "{0..2}"
          }
          set counter [expr $counter + 1]
        }
      } else {
        set Dialog(block2.selected) $name
        set Dialog(block2.name) $name
        set Dialog(block2.mean) [$PT(net) Box $name get Mean]
        set Dialog(block2.blockvis) [$PT(net) Box $name get Visibility]
      }
    }
    #foreach rw {exposed exposed_contents} {
    #  "$Dialog(block_tree.blockvis_widget).$rw" configure -state normal
    #}  ;# foreach rel. Widget-Pfad
      # den Block im Netz highlighten
   # $PT(canvas) addtag BlockMark withtag $name
   # $PT(canvas) itemconfigure $name -fill orange
      # den Block als selektiert markieren
    set boxtag "$name@Box"
    $c itemconfigure $boxtag -fill orange
    $c addtag Selected withtag $boxtag
  } else {
    # Keine Meldung ist besser, da Netz auch ohne Blcke
    # im Editor landen knnen, dann strt eine Meldung. 
    # puts "error: BlockTreeSelectBlock: unknown block ($name)"
  }  ;# else, Fehlerfall
}  ;# BlockTreeSelectBlockTwo

# -------------------------------------------------------------
# Name
#   ConnectionWindow
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Es wird ein Fenster aufgebaut, in dem der Inhalt der Datei
#   $PEPHOME/.pep_server ge"andert werden kann. Mit den Add und Delete
#   Button kann diese Liste erweitert oder verkleinert werden.
proc ConnectionWindow {} {
  global PT Dialog

  if [winfo exists .con] {
    raise .con
    return
  }

  set t [toplevel .con]
  wm title $t {Connection Dialog}

    # Canvas
  frame $t.top -bd 1 -relief raised
  pack $t.top -side top -fill both -expand yes

  frame $t.top.vars -bd 1 -relief sunken
  pack $t.top.vars -side left -expand yes -fill both
  label $t.top.vars.label -text "Servers"
  pack $t.top.vars.label -side top

  frame $t.top.vars.f
  set Trans [GetServer]
  listbox $t.top.vars.f.l \
    -yscrollcommand "$t.top.vars.f.scroll set" -height 10
  $t.top.vars.f.l delete 0 end
  foreach i $Trans {
    .con.top.vars.f.l insert end $i
  }
  bind $t.top.vars.f.l <1> {global Dialog
                            set Dialog(XY) [.con.top.vars.f.l index @%x,%y]
                            set Dialog(Bind) [.con.top.vars.f.l get $Dialog(XY)]}
  scrollbar $t.top.vars.f.scroll -command "$t.top.vars.f.l yview"
  pack $t.top.vars.f.l -side left -expand yes -fill both
  pack $t.top.vars.f.scroll -side left -fill y

  pack $t.top.vars.f -side top -expand yes -fill both

  entry $t.top.vars.bind -relief sunken -textvariable Dialog(Bind)
  pack $t.top.vars.bind -side bottom -expand yes -fill x

    # Action Area
  frame $t.action -relief raised -bd 2
  pack $t.action -side top -fill both

  frame $t.action.default -relief sunken -bd 1
  button $t.action.default.ok1 -text "Save user defaults" \
    -command {SetServer user [.con.top.vars.f.l get 0 end]}
  button $t.action.default.ok2 -text "Save PEP defaults" \
    -command {SetServer global [.con.top.vars.f.l get 0 end]}
  pack $t.action.default -side top -expand 1 -padx 3m -pady 2m
  pack $t.action.default.ok1 $t.action.default.ok2 -side left -padx 2 -pady 2
  frame $t.action.addition -relief sunken -bd 1
    # Add-Button
  button $t.action.addition.add -text "Add" \
    -command {.con.top.vars.f.l insert end $Dialog(Bind);update}
  pack $t.action.addition -side left -expand 1 -padx 3m -pady 2m
  pack $t.action.addition.add -padx 2 -pady 2
    # Delete-Button
  button $t.action.delete -text "Delete" \
    -command {.con.top.vars.f.l delete $Dialog(XY);update}
  pack $t.action.delete -side left -expand 1 -padx 3m -pady 2m
    # Cancel-Button
  button $t.action.cancel -text "Close" \
    -command {set Dialog(Bind) "";destroy .con;return}
  pack $t.action.cancel -side left -expand 1 -padx 3m -pady 2m
  pack $t.action -side bottom
}  ;# ConnectionWindow

# -------------------------------------------------------------
# Name
#   SynchronOptions
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Die Optionen des Synchronisierers lassen sich "andern.
#   Es werden nur die Variablen ge"andert. An den C++-Kern
#   werden sie erst nach Dr"ucken des Ok-Buttons "ubermittelt.
proc SynchronOptions {} {
  global PT

  set d [toplevel .dlg_option_sync_hl_net]
  wm title $d "Sync HL Net"
  wm protocol $d WM_DELETE_WINDOW "destroy $d"

  set PT(syncoptions_temp) [array get PT option.dlg.sync_hl_net.*]
    # Container fr die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
  frame $d.options.filen
  label $d.options.filen.name1 -text "Filename"
  entry $d.options.filen.name2 -textvariable PT(option.dlg.sync_hl_net.filename)
  pack $d.options.filen.name1 $d.options.filen.name2 -side left
  checkbutton $d.options.check_net \
    -text "Check Net" -anchor w \
    -variable PT(option.dlg.sync_hl_net.check_net)
  checkbutton $d.options.rem_dup \
    -text "Remove Duplicates" -anchor w \
    -variable PT(option.dlg.sync_hl_net.rem_dup)
  checkbutton $d.options.dead_trans \
    -text "Eliminate Dead Transitions" -anchor w \
    -variable PT(option.dlg.sync_hl_net.dead_trans)
  checkbutton $d.options.bpn2_names \
    -text "Use B(PN) Names for Communication Variables" -anchor w \
    -variable PT(option.dlg.sync_hl_net.bpn2_names)
  checkbutton $d.options.disable_blocks \
    -text "No Block Information" -anchor w \
    -variable PT(option.dlg.sync_hl_net.disable_blocks)
  checkbutton $d.options.disable_phantoms \
    -text "No Phantom Transitions" -anchor w \
    -variable PT(option.dlg.sync_hl_net.disable_phantoms)
  checkbutton $d.options.no_sync \
    -text "No Synchronisation" -anchor w \
    -variable PT(option.dlg.sync_hl_net.no_sync)
  checkbutton $d.options.no_restriction \
    -text "No Restriction" -anchor w \
    -variable PT(option.dlg.sync_hl_net.no_restriction)
  checkbutton $d.options.transition_flag \
    -text "Restrict Number of Transitions" -anchor w \
    -variable PT(option.dlg.sync_hl_net.transition_flag)
  scale $d.options.transition_number \
    -label "Maximum Number of Transitions" -from 1 -to 1000 \
    -orient horizontal \
    -variable PT(option.dlg.sync_hl_net.transition_number)
  checkbutton $d.options.max_vars_arc_flag \
    -text "Restrict Number of Variables in an Arc Label" -anchor w \
    -variable PT(option.dlg.sync_hl_net.max_vars_arc_flag)
  scale $d.options.max_vars_arc_number \
    -label "Maximum Number of Variables" -from 1 -to 20 \
    -orient horizontal \
    -variable PT(option.dlg.sync_hl_net.max_vars_arc_number)
  pack $d.options.filen \
    $d.options.check_net \
    $d.options.rem_dup \
    $d.options.dead_trans \
    $d.options.bpn2_names \
    $d.options.no_sync \
    $d.options.disable_blocks \
    $d.options.disable_phantoms \
    $d.options.no_restriction \
    $d.options.transition_flag \
    $d.options.transition_number \
    $d.options.max_vars_arc_flag \
    $d.options.max_vars_arc_number \
    -side top -anchor w -fill x

    # Container fr die Buttons der Kommandos
  frame $d.commands -relief raised -bd 1
  pack $d.commands -side bottom -fill both
    # Kommando-Buttons
  button $d.commands.ok -text "Ok" \
    -command "SetSynchronizerOptions;global PT; unset PT(syncoptions_temp);destroy $d"
  button $d.commands.defaults -text "Defaults" \
    -command SynchronizerOptionsDefaults
  button $d.commands.cancel -text "Cancel" \
    -command "global PT;array set PT \$PT(syncoptions_temp);unset PT(syncoptions_temp);destroy $d"
  pack $d.commands.ok \
    $d.commands.defaults \
    $d.commands.cancel \
    -side left -expand 1
} ;# SynchronOptions

# -------------------------------------------------------------
# Name
#   SynchronizerOptionsDefaults
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Die Synchronizervariablen im Tcl-Teil werden auf die
#   Defaultwerte gesetzt.
proc SynchronizerOptionsDefaults {} {
  global PT

  set PT(option.dlg.sync_hl_net.filename) "_tmp.hl_net"
  set PT(option.dlg.sync_hl_net.rem_dup) 1
  set PT(option.dlg.sync_hl_net.dead_trans) 1
  set PT(option.dlg.sync_hl_net.bpn2_names) 1
  set PT(option.dlg.sync_hl_net.disable_blocks) 0
  set PT(option.dlg.sync_hl_net.disable_phantoms) 0
  set PT(option.dlg.sync_hl_net.check_net) 0

  set PT(option.dlg.sync_hl_net.no_sync) 0
  set PT(option.dlg.sync_hl_net.no_restriction) 0
  set PT(option.dlg.sync_hl_net.transition_flag) 1
  set PT(option.dlg.sync_hl_net.transition_number) 100
  set PT(option.dlg.sync_hl_net.max_vars_arc_flag) 0
  set PT(option.dlg.sync_hl_net.max_vars_arc_number) 10
} ;# SynchronizerOptionsDefaults

# -------------------------------------------------------------
# Name
#   SetSynchronizerOptions
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Die Synchronizeroptionen werden an den C++-Kern
#   "ubertragen.
proc SetSynchronizerOptions {} {
  global PT

  $PT(net) SynchronOption filename        $PT(option.dlg.sync_hl_net.filename)
  $PT(net) SynchronOption Max_Arc_Var     $PT(option.dlg.sync_hl_net.max_vars_arc_number)
  $PT(net) SynchronOption Max_Trans       $PT(option.dlg.sync_hl_net.transition_number)
  $PT(net) SynchronOption EliminateDead   $PT(option.dlg.sync_hl_net.dead_trans)
  $PT(net) SynchronOption RemoveDuplicate $PT(option.dlg.sync_hl_net.rem_dup)
  $PT(net) SynchronOption RealName        $PT(option.dlg.sync_hl_net.bpn2_names)
  $PT(net) SynchronOption BlockOutput     $PT(option.dlg.sync_hl_net.disable_blocks)
  $PT(net) SynchronOption PhantomOutput   $PT(option.dlg.sync_hl_net.disable_phantoms)
  $PT(net) SynchronOption CheckNet        $PT(option.dlg.sync_hl_net.check_net)
  $PT(net) SynchronOption Synchronize     $PT(option.dlg.sync_hl_net.no_sync)
  $PT(net) SynchronOption Restrict        $PT(option.dlg.sync_hl_net.no_restriction)
} ;# SetSynchronizerOptions

# -------------------------------------------------------------
# Name
#   ActionFrame
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Es wird ein Fenster aufgebaut, in dem spezielle include und
#   excludestrings eingegeben werden k"onnen. Sind diese Strings
#   leer, so werden immer alle Aktionen in der Selektion genommen
proc ActionFrame {} {
  global PT

  set tl [toplevel .exe_sync_hl_net_dlg]
  wm title $tl "Sync HL Net"
  wm protocol $tl WM_DELETE_WINDOW "destroy $tl"

    # Eingabebereich
  frame $tl.input -bd 2
  pack $tl.input -side top -fill both
  label $tl.input.general1 -text "Give the Actions with their Arities," -anchor w 
  label $tl.input.general2 -text "e.g.: sem(x,x),X(x,x,x),Y(x)" -anchor w
  label $tl.input.general3 -text "But if there is only one Action called X," -anchor w
  label $tl.input.general4 -text "then you can write X without its Arity." -anchor w

  pack $tl.input.general1 $tl.input.general2 $tl.input.general3 $tl.input.general4 \
    -side top -anchor w -fill x

  frame $tl.input.include_actions
  pack $tl.input.include_actions \
    -side top -expand 1 -fill x
  label $tl.input.include_actions.label -text "Include Actions" -bd 2
  entry $tl.input.include_actions.entry -bd 2 \
    -textvariable PT(option.exe_sync_hl_net.include_actions)
  pack $tl.input.include_actions.label \
    -side left
  pack $tl.input.include_actions.entry \
    -side left -fill x -expand 1

  frame $tl.input.exclude_actions
  pack $tl.input.exclude_actions \
    -side top -expand 1 -fill x
  label $tl.input.exclude_actions.label -text "Exclude Actions" -bd 2
  entry $tl.input.exclude_actions.entry -bd 2 \
    -textvariable PT(option.exe_sync_hl_net.exclude_actions)
  pack $tl.input.exclude_actions.label \
    -side left
  pack $tl.input.exclude_actions.entry \
    -side left -fill x -expand 1

    # Ausgabebereich ist leer

    # Button-Bereich
  frame $tl.commands -relief raised -bd 2
  pack $tl.commands -side bottom -fill x
  button $tl.commands.ok -text "Ok" \
    -command {
      global PT
      destroy .exe_sync_hl_net_dlg
      if {$PT(option.exe_sync_hl_net.include_actions) != ""} {
        $PT(net) SynchronOption includestring $PT(option.exe_sync_hl_net.include_actions)
      } else {
        $PT(net) SynchronOption includestring ,leer
      }
      if {$PT(option.exe_sync_hl_net.exclude_actions) != ""} {
        $PT(net) SynchronOption excludestring $PT(option.exe_sync_hl_net.exclude_actions)
      } else {
        $PT(net) SynchronOption excludestring ,leer
      }
      set PT(Abbruch) 1}

  button $tl.commands.cancel -text "Cancel" \
    -command "destroy $tl;set PT(Abbruch) 0"
  pack $tl.commands.ok \
    $tl.commands.cancel \
    -side left -expand 1
  tkwait variable PT(Abbruch)
  return $PT(Abbruch)
} ;# ActionFrame


# -------------------------------------------------------------
# Name
#   NetsDialog
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Eine Liste der zur Verf"ugung stehenden Netze wird erstellt,
#   so da"s daraus ein Netz ausgew"ahlt werden kann. Dieses Netz
#   kann anschlie"send in den Netzeditor geladen oder
#   gel"oscht werden
proc NetsDialog {} {
  global PT Dialog

  destroy .nets

  set t [toplevel .nets]
  wm title $t {Nets Dialog}

  set Dialog(block1.selected) ""
  set Dialog(block1.name) ""
  set Dialog(block1.mean) ""
  set Dialog(block1.blockvis) ""

  set Dialog(block2.selected) ""
  set Dialog(block2.name) ""
  set Dialog(block2.mean) ""
  set Dialog(block2.blockvis) ""

    # Canvas
  frame $t.two
  frame $t.two.cf
  pack $t.two.cf -side left -fill both -expand 1 -padx 2 -pady 2
  frame $t.top -bd 1 -relief raised
  pack $t.top -side top -fill both -expand yes

  frame $t.top.list
  pack $t.top.list -expand yes -fill x
  frame $t.top.list.vars -bd 1 -relief sunken
  pack $t.top.list.vars -side left -expand yes -fill x
  label $t.top.list.vars.label -text "Nets"

  frame $t.top.list.vars.f
  pack $t.top.list.vars.f -side bottom -fill x

  listbox $t.top.list.vars.f.l \
    -yscrollcommand "$t.top.list.vars.f.scroll set" -height 10
  set Trans [$PT(net) get Nets]
  .nets.top.list.vars.f.l delete 0 1000
  foreach i $Trans {
    .nets.top.list.vars.f.l insert end $i
  }
  bind $t.top.list.vars.f.l <1> {global Dialog
                                   set Dialog(Bind) [.nets.top.list.vars.f.l get @%x,%y]
                                   set Dialog(XY) [.nets.top.list.vars.f.l index @%x,%y]
                                }
  scrollbar $t.top.list.vars.f.scroll -command "$t.top.list.vars.f.l yview"
  pack $t.top.list.vars.label -side top
  pack $t.top.list.vars.f.l -side left -fill both -expand yes
  pack $t.top.list.vars.f.scroll -side left -fill both

  pack $t.top.list -side top -expand yes -fill x
  entry $t.top.bind -relief sunken -textvariable Dialog(Bind)
  pack $t.top.bind -side bottom -expand yes -fill x

    # Action Area
  frame $t.action -relief raised -bd 2
  pack $t.action -side top -fill both

  frame $t.action.default -relief sunken -bd 1
   # Ok-Button
  button $t.action.default.ok -text "Add current net to others" \
    -command {ToNets;.nets.top.list.vars.f.l insert end $PT(Filename);update}
  pack $t.action.default -side left -expand 1 -padx 3m -pady 2m
  pack $t.action.default.ok
  frame $t.action.addition -relief sunken -bd 1
    # Add-Button
  button $t.action.addition.add -text "Insert other Net" \
    -command {InsertNet $Dialog(Bind);update}
  pack $t.action.addition -side left -expand 1 -padx 3m -pady 2m
  pack $t.action.addition.add 
    # Cancel-Button
  button $t.action.cancel -text "Cancel" \
    -command {destroy .nets;return}
  pack $t.action.cancel -side left -expand 1 -padx 3m -pady 2m
  pack $t.two $t.action -side bottom

  set old_focus [focus]
  tkwait visibility $t
  focus $t
}  ;# NetsDialog

proc ToNets {} {
  global PT

  SelectAll
  Cut
  $PT(net) ToNets $PT(Filename)
  DeselectAll
}  ;# ToNets

proc InsertNet {name} {
  global PT

  $PT(net) InsertNet $name
  $PT(net) DeleteAll
  $PT(canvas) delete all
  DrawGrid
  Paste
  $PT(net) RestoreClipboard
  DeselectAll
  Repaint
  set PT(Filename) $name
}  ;# InsertNet

proc DeleteNets {name} {
  global PT

  $PT(net) DeleteNet $name
}  ;# DeleteNets

# -------------------------------------------------------------
# Name
#   Find
# ---------
# Eingabe 
#   Nix
# ---------
# Beschreibung
#   Ausgehend von einer Anweisung wird eine BasicBox
#   generiert.
proc Expression {} {
  global PT Dialog

  set Dialog(Sema) 1

  set t [toplevel .td -bd 0]
  wm title $t {Expression Dialog}

  frame $t.top -bd 1 -relief raised
  frame $t.action -bd 1 -relief raised

  message $t.top.msg -text "Expression Dialog" \
    -aspect 1000 -font -Adobe-Times-Medium-R-Normal--*-180-*-*-*-*-*-* \
    -justify center
  message $t.top.bsp -text "You can create a basic box based upon \n\
	an expression like \"a'='a+1 and c!=2\"." -aspect 1000 \
    -font -Adobe-Times-Medium-R-Normal--*-140-*-*-*-*-*-* -justify left
  pack $t.top.msg $t.top.bsp -side top -fill x

  pack $t.top -side top -fill both -expand 1
  pack $t.action -side top -fill both -expand 1

  frame $t.top.name
  frame $t.top.sema 
  label $t.top.name.label -text "Expression" -width 15
  set Dialog(Name) ""
  entry $t.top.name.entry -textvariable Dialog(Name) -relief sunken
  checkbutton $t.top.sema.s1 -text "Procedure semantic" \
    -variable Dialog(Sema) -anchor w
  pack $t.top.name.entry -side right -fill x -expand yes
  pack $t.top.name.label -side left
  pack $t.top.sema.s1 -side left -fill x -expand yes 
  pack $t.top.name $t.top.sema -padx 3m -pady 2m -side top -anchor w -fill x

    # Action Area
  frame $t.action.default -relief sunken -bd 1
  button $t.action.default.ok -text "Create Net" \
    -command "ExprAnd; global Dialog; set Dialog(Name) {}"
  pack $t.action.default.ok 
  pack $t.action.default -side left -fill x -padx 3m -pady 2m

  button $t.action.cancel -text Close \
    -command "destroy $t"
  pack $t.action.cancel -side right -padx 3m -pady 2m -fill x

}  ;


proc ExprAnd {} {
  global PT Dialog

  $PT(net) ProcSemantic $Dialog(Sema)
  $PT(net) Expression $Dialog(Name)
  Repaint
}

  #*** Dateiende Program.tcl ***

