// netAppInit.cc, main-file for the NetWish application
// Burkhard Bieber 1996
// started: 960920, last-update: 960920

//redefine to avoid problems with windows
#define BOOLEAN PEP_BOOL
#include "../Low/Net.h"
#include "../High/HLNet.h"
#undef BOOLEAN
#include <tcl.h>
#include <tk.h>
#include <sys/time.h>
#include <unistd.h>
#include <limits.h>

#if (TCL_MAJOR_VERSION == 8) && (TCL_MINOR_VERSION == 0) /* Tcl8.0 stuff */
#define Tcl_GetString(objPtr)	Tcl_GetStringFromObj(objPtr, (int *)NULL)
#endif

/* This EXTERN declaration is needed for Tcl < 8.0.3 */
#ifndef EXTERN
# ifdef __cplusplus
#  define EXTERN extern "C"
# else
#  define EXTERN extern
# endif
#endif

#ifdef TCL_STORAGE_CLASS
# undef TCL_STORAGE_CLASS
#endif
#ifdef BUILD_netedit
# define TCL_STORAGE_CLASS DLLEXPORT
#else
# define TCL_STORAGE_CLASS DLLIMPORT
#endif

#ifdef WIN32
#   define WIN32_LEAN_AND_MEAN
#   include <windows.h>
#   undef WIN32_LEAN_AND_MEAN
/* VC++ has an entry point called DllMain instead of DllEntryPoint */
#   if defined(_MSC_VER)
#	define DllEntryPoint DllMain
#   endif
#endif


extern "C" {
// Datenstrukturen f"ur den Synchronisierer

Cardinal var_style = VAR_BOTH;   /* Ausgabe-Stil f. Kommunikationsvariablen */

char *SrcBuffer = NULL;          /* Buffer fuer den Quelltext               */
char **SrcLine;                  /* Array von Zeigern auf die einzelnen 
                                    Zeilenanfaenge im Quelltext             */

Cardinal scale = 1,              /* Skalierungsfaktor fuer den Netz-Graphen */
         max_arc_vars = 1,       /* maximale Anzahl von Variablen in einem 
                                                                Kantenlabel */
         max_proc_inst = 2;      /* maximale Anzahl von Aktivierungen einer
                                                                   Prozedur */
Cardinal32 flag=0;                 /* globale Programm-Flags                  */

char    *source,                 /* Quelldatei b(pn)2 bzw. bei syncmnet
                                    das zu synchronisierende Netz           */
        *actions_string;         /* die beim Aufruf von syncmnet ggf. 
                                    spezifizierten Aktionssymbole           */
char    *exclude_string = NULL;  /* ggf. auszuscliessende Aktionssymbole    */
char    *pep_file = NULL;        /* Ausgabedatei im Standard-PEP-Format     */
char    *source_ref = NULL;      /* Referenzen fuer das Eingabe-Netz        */

VHMap<String, Net *> nets((Net *) NULL);
VHMap<String, HLNet *> hlnets((HLNet *) NULL);

/*
 *----------------------------------------------------------------------
 *
 * Tcl_AppInit --
 *
 *	This procedure performs application-specific initialization.
 *	Most applications, especially those that incorporate additional
 *	packages, will have their own version of this procedure.
 *
 * Results:
 *	Returns a standard Tcl completion code, and leaves an error
 *	message in interp->result if an error occurs.
 *
 * Side effects:
 *	Depends on the startup script.
 *
 *----------------------------------------------------------------------
 */

EXTERN int
Netedit_Init(Tcl_Interp *interp)
{
#ifdef USE_TCL_STUBS
    if (Tcl_InitStubs(interp, "8.0", 0) == NULL) {
#else
    if (Tcl_PkgRequire(interp, "Tcl", "8.0", 0) == NULL) {
#endif
	return TCL_ERROR;
    }
#ifdef USE_TK_STUBS
    if (Tk_InitStubs(interp, "8.0", 0) == NULL) {
#else
    if (Tcl_PkgRequire(interp, "Tk", "8.0", 0) == NULL) {
#endif
	return TCL_ERROR;
    }

    if (Tcl_PkgProvide(interp, "netedit", "1.0") != TCL_OK) {
	return TCL_ERROR;
    }

    /*
     * Call Tcl_CreateCommand for application-specific commands, if
     * they weren't already created by the init procedures called above.
     */

    Tcl_CreateObjCommand(interp, "MakeNet", Net::CreateLow, NULL, NULL);
    Tcl_CreateCommand(interp, "MakeHLNet", HLNet::CreateHigh, NULL, NULL);

    /*
     * Specify a user-specific startup file to invoke if the application
     * is run interactively.  Typically the startup file is "~/.apprc"
     * where "app" is the name of the application.  If this line is deleted
     * then no user-specific startup file will be run under any conditions.
     */

    Tcl_SetVar(interp, "tcl_rcFileName", "~/.wishrc", TCL_GLOBAL_ONLY);
    return TCL_OK;
}

}
#ifdef WIN32
/*
 *----------------------------------------------------------------------
 *
 * DllEntryPoint --
 *
 *	This wrapper function is used by Windows to invoke the
 *	initialization code for the DLL.  If we are compiling
 *	with Visual C++, this routine will be renamed to DllMain.
 *	routine.
 *
 * Results:
 *	Returns TRUE;
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

BOOL APIENTRY
DllEntryPoint(HINSTANCE hInst, DWORD reason, LPVOID reserved)
{
    return TRUE;
}
#endif

  // end of file 'netAppInit.cc'
