// netAppInit.cc, main-file for the NetWish application
// Burkhard Bieber 1996
// started: 960920, last-update: 960920

// following the (c) notice from the base file
/* 
 * tkAppInit.c --
 *
 *	Provides a default version of the Tcl_AppInit procedure for
 *	use in wish and similar Tk-based applications.
 *
 * Copyright (c) 1993 The Regents of the University of California.
 * Copyright (c) 1994 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * SCCS: @(#) tkAppInit.c 1.21 96/03/26 16:47:07
 */

#include <sys/time.h>
#include <unistd.h>
#include <limits.h>
#include "tk.h"
#include "../Low/Net.h"
#include "../High/HLNet.h"
#include "../../backend/mnet.h"

// Datenstrukturen f"ur den Synchronisierer

Cardinal var_style = VAR_BOTH;   /* Ausgabe-Stil f. Kommunikationsvariablen */

char *SrcBuffer = NULL;          /* Buffer fuer den Quelltext               */
char **SrcLine;                  /* Array von Zeigern auf die einzelnen 
                                    Zeilenanfaenge im Quelltext             */

Cardinal scale = 1,              /* Skalierungsfaktor fuer den Netz-Graphen */
         max_arc_vars = 1,       /* maximale Anzahl von Variablen in einem 
                                                                Kantenlabel */
         max_proc_inst = 2;      /* maximale Anzahl von Aktivierungen einer
                                                                   Prozedur */
Cardinal32 flag=0;                 /* globale Programm-Flags                  */

char    *source,                 /* Quelldatei b(pn)2 bzw. bei syncmnet
                                    das zu synchronisierende Netz           */
        *actions_string;         /* die beim Aufruf von syncmnet ggf. 
                                    spezifizierten Aktionssymbole           */
char    *exclude_string = NULL;  /* ggf. auszuscliessende Aktionssymbole    */
char    *pep_file = NULL;        /* Ausgabedatei im Standard-PEP-Format     */
char    *source_ref = NULL;      /* Referenzen fuer das Eingabe-Netz        */

VHMap<String, Net *> nets((Net *) NULL);
VHMap<String, HLNet *> hlnets((HLNet *) NULL);

/*
 *----------------------------------------------------------------------
 *
 * main --
 *
 *	This is the main program for the application.
 *
 * Results:
 *	None: Tk_Main never returns here, so this procedure never
 *	returns either.
 *
 * Side effects:
 *	Whatever the application does.
 *
 *----------------------------------------------------------------------
 */

int
main(int argc, char **argv)
{
    Tk_Main(argc, argv, Tcl_AppInit);
    return 0;			/* Needed only to prevent compiler warning. */
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_AppInit --
 *
 *	This procedure performs application-specific initialization.
 *	Most applications, especially those that incorporate additional
 *	packages, will have their own version of this procedure.
 *
 * Results:
 *	Returns a standard Tcl completion code, and leaves an error
 *	message in interp->result if an error occurs.
 *
 * Side effects:
 *	Depends on the startup script.
 *
 *----------------------------------------------------------------------
 */

int
Tcl_AppInit(Tcl_Interp *interp)
{
    if (Tcl_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    if (Tk_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
//    Tcl_StaticPackage(interp, "Tk", Tk_Init, (Tcl_PackageInitProc *) NULL);

    /*
     * Call the init procedures for included packages.  Each call should
     * look like this:
     *
     * if (Mod_Init(interp) == TCL_ERROR) {
     *     return TCL_ERROR;
     * }
     *
     * where "Mod" is the name of the module.
     */

    /*
     * Call Tcl_CreateCommand for application-specific commands, if
     * they weren't already created by the init procedures called above.
     */

    Tcl_CreateObjCommand(interp, "MakeNet", Net::CreateLow, NULL, NULL);
    Tcl_CreateCommand(interp, "MakeHLNet", HLNet::CreateHigh, NULL, NULL);

    /*
     * Specify a user-specific startup file to invoke if the application
     * is run interactively.  Typically the startup file is "~/.apprc"
     * where "app" is the name of the application.  If this line is deleted
     * then no user-specific startup file will be run under any conditions.
     */

    Tcl_SetVar(interp, "tcl_rcFileName", "~/.wishrc", TCL_GLOBAL_ONLY);
    return TCL_OK;
}

  // end of file 'netAppInit.cc'
