// This may look like C code, but it is really -*- C++ -*-
/* 
Copyright (C) 1988 Free Software Foundation
    written by Doug Lea (dl@rocky.oswego.edu)

This file is part of the GNU C++ Library.  This library is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This library is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/


// Modified for Templates by Tobias Himstedt
// Universitaet Hildesheim
// 1/95

#ifndef _Bag_h
#define _Bag_h 1

#include <Pix.h>

template<class Type>
class Bag {
protected:

    int                   count;
    
public:
    virtual              ~Bag();
    
    int                   length() const;                // current number of items
    int                   empty() const;
    
    virtual Pix           add(const Type& item) = 0;      // add item; return Pix
    virtual void          del(const Type& item) = 0;      // delete item
    int                   contains(const Type& item) const;     // is item in set?
    virtual void          nof(const Type& i) const;

    virtual void          clear();                 // delete all items
    
    virtual Pix           first() const = 0;          // Pix of first item or 0
    virtual void          next(Pix& i) const = 0;        // advance to next or 0
    virtual Type&         operator () (Pix i) const = 0 ; // access item at i
    
    virtual int           owns(Pix i) const;             // is i a valid Pix  ?
    virtual Pix           seek(const Type& item) const;         // Pix of item
    
    int                   operator == (const Bag<Type>& b) const;
    int                   operator != (const Bag<Type>& b) const;
    
    void                  error(const char* msg) const;
    virtual int           OK() const = 0;                // rep invariant
};



template<class Type> inline
Bag<Type>::~Bag()
{
    // Nix
}

template<class Type> inline int
Bag<Type>::length() const
{
    return count;
}

template<class Type> inline int 
Bag<Type>::empty() const
{
    return count == 0;
}

template<class Type> inline
Pix Bag<Type>::seek(const Type& item) const
{
    for (Pix i = first(); i != 0 && !(Equal((*this)(i), item)); next(i));
    return i;
}

template<class Type> inline
int Bag<Type>::owns(Pix idx) const
{
    if (idx == 0) return 0;
    for (Pix i = first(); i; next(i)) if (i == idx) return 1;
    return 0;
}

template<class Type> inline
void Bag<Type>::clear() 
{
    Pix i = first(); 
    while (i != 0)
	{
	    del((*this)(i));
	    i = first();
	}
}

template<class Type> inline
int Bag<Type>::contains (const Type& item) const
{
    return seek(item) != 0;
}

template<class Type> inline
int Bag<Type>::operator == (const Bag<Type>& b) const
{
    int n = count;
    if (n != b.count) return 0;
    if (n == 0) return 1;
    Pix i = first();
    Pix j = b.first();
    while (n-- > 0)
	{
	    if ((b.seek((*this)(i)) == 0) || (seek(b(j)) == 0)) return 0;
	    next(i);
	    b.next(j);
	}
    return 1;
}

template<class Type> inline
int Bag<Type>::operator != (const Bag<Type>& b) const
{
    return !(*this == b);
}

template<class Type> int
Bag<Type>::nof(const Type& item)
{
    int n = 0;
    for (Pix p = first(); p; next(p)) 
	if (Equal((*this)(p), item)) ++n;
    return n;
}

template<class Type> inline
void Bag<Type>::error(const char* msg) const
{
    cerr << "Bag :" << msg;
}

#endif
