// This may look like C code, but it is really -*- C++ -*-
/* 
Copyright (C) 1988 Free Software Foundation
    written by Doug Lea (dl@rocky.oswego.edu)

This file is part of the GNU C++ Library.  This library is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This library is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

// Modified for Templates by Tobias Himstedt
// Universitaet Hildesheim
// 1/95

// Modified for gcc 2.7.2 by Burkhard Bieber
// Changed <Pix.h> to "Pix.h" - Christian Stehno

// last-update: 30.05.2000

#ifndef Map_h
#define Map_h 1

#include "Pix.h"

template<class Key, class Value>
class Map {
protected:
    unsigned int                     count;
    Value                   def;
    
public:
    Map(const Value& dflt);
    virtual              ~Map();
    
    int                   length() const; // current number of items
    int                   empty() const;
    
    virtual int           contains(const Key& key) const;      // is key mapped?
    
    virtual void          clear();                 // delete all items
    
    virtual Value&        operator [] (const Key& key) = 0; // access contents by key
    
    virtual void          del(const Key& key) = 0;       // delete entry
    
    virtual Pix           first() const = 0;             // Pix of first item or 0
    virtual void          next(Pix& i) const = 0;        // advance to next or 0
    virtual Key&          key(Pix i) const = 0;          // access key at i
    virtual Value&        contents(Pix i) const = 0;     // access contents at i
    
    virtual int           owns(Pix i) const;             // is i a valid Pix  ?
    virtual Pix           seek(const Key& key) const;          // Pix of key
    
    const Value&          dflt() const;                  // access default val
    
    void                  error(const char* msg) const;
    virtual int           OK() const = 0;                // rep invariant
};



template<class Key, class Value> inline 
Map<Key, Value>::~Map() 
{}

template<class Key, class Value> inline int 
Map<Key, Value>::length() const
{
    return count;
}

template<class Key, class Value> inline int 
Map<Key, Value>::empty() const
{
    return count == 0;
}

template<class Key, class Value> inline const Value&
Map<Key, Value>::dflt() const
{
    return def;
}

template<class Key, class Value> inline 
Map<Key, Value>::Map(const Value& dflt) :def(dflt)
{
    count = 0;
}

template<class Key, class Value> Pix 
Map<Key, Value>::seek(const Key& item) const
{
    Pix i;
    for (i = first(); i != 0 && !(Equal(key(i), item)); next(i));
    return i;
}

template<class Key, class Value> int 
Map<Key, Value>::owns(Pix idx) const
{
    if (idx == 0) return 0;
    for (Pix i = first(); i; next(i)) if (i == idx) return 1;
    return 0;
}

template<class Key, class Value> void 
Map<Key, Value>::clear()
{
    Pix i = first(); 
    while (i != 0)
      {
	  del(key(i));
	  i = first();
      }
}

template<class Key, class Value> int 
Map<Key, Value>::contains (const Key& item) const
{
    return seek(item) != 0;
}


template<class Key, class Value> void 
Map<Key, Value>::error(const char* msg) const
{
    cerr << "Map :" << msg << endl;
}

#endif
