
#ifndef MULTISET_H
#define MULTISET_H

#include "VHMap.h"


inline int 
max(int a, int b)
{
    return (a>b) ? a : b;
}

inline int 
min(int a, int b)
{
    return (a<b) ? a : b;
}

template<class Type>
class MultiSET {
private:
    
    VHMap<Type, int> rep;

public:

    MultiSET();

    void add(const Type& a, int h=1);
    
    void del(const Type& a, int h=1);

    void clear();

    int contains(const Type& a) const;

    int length() const;

    int elements() const;

    int empty() const;
    Pix first() const;

    void next(Pix& i) const;

    int nof(const Type& a) const;

    int nof(Pix i) const;


    Type& element(Pix i) const;

    Pix seek(const Type& a) const;


    void operator |=(const MultiSET<Type>& b);

    void operator &=(const MultiSET<Type>& b);

    void operator +=(const MultiSET<Type>& b);

    void operator -=(const MultiSET<Type>& b);

    int operator ==(const MultiSET<Type>& b) const;

    int operator !=(const MultiSET<Type>& b) const;
};

template<class Type> inline
MultiSET<Type>::MultiSET() : rep(0,10)
{
    // Nix
}

template<class Type> inline void
MultiSET<Type>::add(const Type& a,int h) 
{
    rep[a] += h;
}

template<class Type> inline void
MultiSET<Type>::del(const Type& a, int h) 
{
    if (rep[a] == 0)
	rep.del(a);
    else
	if (h<rep[a])
	    rep[a] -= h;
	else
	    rep.del(a);
}

template<class Type> inline int
MultiSET<Type>::contains(const Type& a) const
{
    return rep.contains(a);
}

template<class Type> inline int
MultiSET<Type>::length() const
{
    return rep.length();
}

template<class Type> inline int
MultiSET<Type>::empty() const
{
    return rep.empty();
}

template<class Type> inline Pix
MultiSET<Type>::first() const
{
    return rep.first();
}

template<class Type> inline void
MultiSET<Type>::next(Pix& i) const
{   
    rep.next(i);
}

template<class Type> inline Type&
MultiSET<Type>::element(Pix i) const
{
    return rep.key(i);
}

template<class Type> inline int
MultiSET<Type>::nof(Pix i) const
{
    return rep.contents(i);
}

template<class Type> inline int 
MultiSET<Type>::nof(const Type& a) const
{
    Pix i=rep.seek(a);
    return i ? rep.contents(i) : 0;
}

template<class Type> inline Pix 
MultiSET<Type>::seek(const Type& a) const
{
    return rep.seek(a);
}

template<class Type> inline void 
MultiSET<Type>::clear() 
{
    rep.clear();
}


template<class T> void
MultiSET<T>::operator |=(const MultiSET<T>& b)
{
    Pix i;
    // Alle Elemente von b durchlaufen
    for (i=b.first(); i; b.next(i)) 
	rep[b.element(i)] = max( rep[b.element(i)], b.nof(b.element(i)));
}

template<class Type> int 
MultiSET<Type>::elements() const
{
    int ret = 0;

    for (Pix i=rep.first(); i; rep.next(i))
	ret += rep.contents(i);
    return ret;
}

template<class T> void
MultiSET<T>::operator &=(const MultiSET<T>& b)
{
    Pix i;
    // Alle Elemente von b durchlaufen
    for (i=b.first(); i; b.next(i)) 
	if (rep.contains(b.element(i)))
	    // Wenn b(i) auch in this, dann minimum beider	    
	    rep[b.element(i)] = min( rep[b.element(i)], b.nof(b.element(i)));

    // Alle Elemente von this, die nicht in b sind, aus this streichen
    
    i = first();
    Pix n = i;

    while (i) {
	next(n);
	if (!b.contains(element(i)))
	    rep.del(element(i));
	i = n;
    }
}

template<class T> void
MultiSET<T>::operator +=(const MultiSET<T>& b)
{
    Pix i;
    // Alle Elemente von b durchlaufen
    for (i=b.first(); i; b.next(i)) 
	rep[b.element(i)] += b.nof(b.element(i));

}

template<class T> void
MultiSET<T>::operator -=(const MultiSET<T>& b)
{
    for (Pix i=b.first(); i; b.next(i))
	if (nof(b.element(i)) >= b.nof(i))
	    del(b.element(i), b.nof(i));
	    // rep[b.element(i)] -= b.nof(i);
}

template<class T> int
MultiSET<T>::operator ==(const MultiSET<T>& b) const
{
    MultiSET<T> tmp = b;

    for (Pix i=first(); i; next(i)) {
	T e = element(i);
	if (nof(i) != tmp.nof(e))
	    return 0;
	tmp.del(e,nof(i));
    }
    return tmp.empty();
}

template<class T> inline int
MultiSET<T>::operator !=(const MultiSET<T>& b) const
{
    return !(*this == b);
}


#endif

 
