// This may look like C code, but it is really -*- C++ -*-
/* 
Copyright (C) 1988 Free Software Foundation
    written by Doug Lea (dl@rocky.oswego.edu)

This file is part of the GNU C++ Library.  This library is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This library is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/


// Modified for Templates by Tobias Himstedt
// Universitaet Hildesheim
// 1/95

#ifndef SLStack_h
#define SLStack_h 1

#include "Stack.h"
#include "SLList.h"

template<class Type>
class SLStack : public Stack<Type>
{
    SLList<Type>     p;
    
  public:
    SLStack();
    SLStack(const SLStack<Type>& s);
    ~SLStack();
    
    void          operator = (const SLStack<Type>& r);
    
    void          push(const Type& item);
    Type           pop();
    Type&          top() const;               
    void          del_top();
    
    int           empty() const;
    int           full() const;
    int           length() const;
    
    void          clear();
    
    int           OK() const;
    
};

template<class Type> inline 
SLStack<Type>::SLStack() : p() 
{

}

template<class Type> inline 
SLStack<Type>::SLStack(const SLStack<Type>& a) : p(a.p) 
{

}

template<class Type> inline 
SLStack<Type>::~SLStack() {}

template<class Type> inline void 
SLStack<Type>::push(const Type& item)
{
    p.prepend(item);
}

template<class Type> inline Type 
SLStack<Type>::pop()
{
    return p.remove_front();
}

template<class Type> inline Type& 
SLStack<Type>::top() const
{
  return p.front();
}

template<class Type> inline void 
SLStack<Type>::del_top()
{
    p.del_front();
}

template<class Type> inline void 
SLStack<Type>::operator =(const SLStack<Type>& s)
{
    p = s.p;
}

template<class Type> inline int 
SLStack<Type>::empty() const
{
    return p.empty();
}

template<class Type> inline int 
SLStack<Type>::full() const
{
    return 0;
}

template<class Type> inline int 
SLStack<Type>::length() const
{
    return p.length();
}

template<class Type> inline int 
SLStack<Type>::OK() const
{
    return p.OK();
}

template<class Type> inline void 
SLStack<Type>::clear() 
{
    p.clear();
}


#endif
