// This may look like C code, but it is really -*- C++ -*-
/* 
Copyright (C) 1988 Free Software Foundation
    written by Doug Lea (dl@rocky.oswego.edu)

This file is part of the GNU C++ Library.  This library is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This library is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/


// Modified for Templates by Tobias Himstedt
// Universitaet Hildesheim
// 1/95

// Modified for gcc 2.7.2 by Burkhard Bieber

// Changed <Pix.h> to "Pix.h" - Christian Stehno
// last-update: 30.05.2000

#ifndef _Set_h
#define _Set_h 1

#include "Pix.h"

template<class Type>
class SET {
protected:

    unsigned int          count;
    
public:
    virtual              ~SET();
    
    int                   length() const;                // current number of items
    int                   empty() const;
    
    virtual Pix           add(const Type& item) = 0;      // add item; return Pix
    virtual void          del(const Type& item) = 0;      // delete item
    int                   contains(const Type& item) const;     // is item in set?
    
    virtual void          clear();                 // delete all items
    
    virtual Pix           first() const = 0;          // Pix of first item or 0
    virtual void          next(Pix& i) const = 0;        // advance to next or 0
    virtual Type&         operator () (Pix i) const = 0 ; // access item at i
    
    virtual int           owns(Pix i) const;             // is i a valid Pix  ?
    virtual Pix           seek(const Type& item) const;         // Pix of item
    
    void                  operator |= (const SET<Type>& b); // add all items in b
    void                  operator -= (const SET<Type>& b); // delete items also in b
    void                  operator &= (const SET<Type>& b); // delete items not in b
    
    int                   operator == (const SET<Type>& b) const;
    int                   operator != (const SET<Type>& b) const;
    int                   operator <= (const SET<Type>& b) const; 
    
    void                  error(const char* msg) const;
    virtual int           OK() const = 0;                // rep invariant
};

template<class T> inline int 
Equal(const T& s1, const T& s2)
{
    return s1 == s2;
}

template<class T> inline unsigned int 
Hash(const T& s1)
{
    return s1.Hash();
}


template<class Type> inline
SET<Type>::~SET()
{
    // Nix
}

template<class Type> inline int
SET<Type>::length() const
{
    return count;
}

template<class Type> inline int 
SET<Type>::empty() const
{
    return count == 0;
}

template<class Type> inline
Pix SET<Type>::seek(const Type& item) const
{
    Pix i;
    for (i = first(); i != 0 && !(Equal((*this)(i), item)); next(i));
    return i;
}

template<class Type> inline
int SET<Type>::owns(Pix idx) const
{
    if (idx == 0) return 0;
    for (Pix i = first(); i; next(i)) if (i == idx) return 1;
    return 0;
}

template<class Type> inline
void SET<Type>::clear() 
{
    Pix i = first(); 
    while (i != 0)
	{
	    del((*this)(i));
	    i = first();
	}
}

template<class Type> inline
int SET<Type>::contains (const Type& item) const
{
    return seek(item) != 0;
}

template<class Type> inline
int SET<Type>::operator <= (const SET<Type>& b) const
{
    if (count > b.count) return 0;
    if (count == 0) return 1;
    for (Pix i = first(); i; next(i)) if (b.seek((*this)(i)) == 0) return 0;
    return 1;
}

template<class Type> inline
int SET<Type>::operator == (const SET<Type>& b) const
{
    unsigned int n = count;
    if (n != b.count) return 0;
    if (n == 0) return 1;
    Pix i = first();
    Pix j = b.first();
    while (n-- > 0)
	{
	    if ((b.seek((*this)(i)) == 0) || (seek(b(j)) == 0)) return 0;
	    next(i);
	    b.next(j);
	}
    return 1;
}

template<class Type> inline
int SET<Type>::operator != (const SET<Type>& b) const
{
    return !(*this == b);
}

template<class Type> inline
void SET<Type>::operator |= (const SET<Type>& b)
{
    if (&b != this)
	for (Pix i = b.first(); i; b.next(i)) add(b(i));
}

template<class Type> inline
void SET<Type>::operator -= (const SET<Type>& b)
{
    if (&b == this)
	clear();
    else
	for (Pix i = b.first(); i; b.next(i)) del(b(i));
}


template<class Type> inline
void SET<Type>::operator &= (const SET<Type>& b)
{
    if (&b != this)
	{
	    Pix i = first();
	    Pix n = i;
	    while (i != 0)
		{
		    next(n);
		    if (b.seek((*this)(i)) == 0) del((*this)(i));
		    i = n;
		}
	}
}

template<class Type> inline
void SET<Type>::error(const char* msg) const
{
    cerr << "Set :" << msg;
}

#endif
