// -------------------------------------------------------------------------
// Programm : HIPS Highlevel Petrinetz Simulator/Editor 
// ----------------------------------------------------
// Datei    : MyString.h
// Funktion : Defintion einer String-Klasse nach Stroustrup
// Author   : Tobias Himstedt
//            Diplomarbeit 1994 
//            Universitaet Hildesheim
//            Institut fuer Theoretische Informatik
//            Prof. Dr. Best
// -------------------------------------------------------------------------
// Burkhard Bieber 1996
// Christian Stehno: added some types for ANSI-C++ (30.05.2000)
// Cecile Bui Thanh: added some gards before calling strcmp
// last-update: Dec. 2001


#ifndef MYSTRING_H
#define MYSTRING_H

#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cctype>

using namespace std;
extern char NULL_STRING[1];

// -------------------------------------------------------------------------
// Klasse       : String
// -------------------------------------------------------------------------
// Beschreibung : String Klasse
// -------------------------------------------------------------------------
class String {
private:

    // ---------------------------------------------------------------
    //  Datenelemente
    // ---------------------------------------------------------------

    int type;
    struct srep {
	// Zeichenkette
	char* s;

	// Anzahl der Verweise auf *s

	unsigned int n;
	// Laenge von s
	unsigned int len;
	
	// Default Konstruktor
	srep() { n = 1; s=NULL_STRING, len=0; };

	// Konstruktor fuer char*
	srep(const char* x) {
	    len = strlen(x);
	    s = new char[len + 1];
	    strcpy(s, x);
	    n = 1;
	};
	
	// Destruktor
	~srep() {
	    if (s != NULL_STRING) { // !!UNSCHOEN!!
	      delete [] s;
            }
	};
    };
    
    // String 
    srep* p;

    void Resize(int len);

public:
    // ---------------------------------------------------------------
    //  Friends
    // ---------------------------------------------------------------
    
    // Ausgabe
    friend ostream &operator <<(ostream& f, const String& s);

    friend istream &operator >>(istream& f, String& s);

    friend int readline(istream& s, String& x, char delimiter='\n',
		    int discard=1);

    // Gleichheit
    friend int operator ==(const String& s1, const String& s2) {
	return ((s1.p->s && s2.p->s) ? (strcmp(s1.p->s, s2.p->s) == 0) : 0);
    };
    
    friend int operator ==(const String& s1, const char* s2) {
	return ((s1.p->s && s2) ? (strcmp(s1.p->s, s2) == 0) : 0);
    };

    friend int operator <(const String& s1, const String& s2) {
	return s1.p->s && s2.p->s ? strcmp(s1.p->s, s2.p->s) < 0 : 0;
    };
    
    friend int operator <(const String& s1, const char* s2) {
	return (s1.p->s && s2) ? strcmp(s1.p->s, s2) < 0 : 0;
    };

    // Ungleichheit
    friend int operator !=(const String& s1, const String& s2) {
	return s1.p->s && s2.p->s ? strcmp(s1.p->s, s2.p->s) != 0 : 1;
    };
    
    friend int operator !=(const String& s1, const char* s2) {
	return ((s1.p->s && s2) ? strcmp(s1.p->s, s2) != 0 : 1);
    };

    String gsub(const char* s1, const char* s2) const;

    // ---------------------------------------------------------------
    //  Konstante Funktionen
    // ---------------------------------------------------------------

    unsigned int length(void) const {
	return p->len;
    };

    String StrTok(const String& delimiter, int& start, 
		    char& found_delimiter) const;

    String before(const char* s) const;

    String after(const char* s) const;

    bool contains(const char* s) const;

    String FromTo(int a, int b) const;

    char First() {return *(p->s);}

//    char Last() {return *(p->s+(sizeof(char)*((p->len)-1)));}
    char Last() { return (p->s[(p->len)-1] );}

    int StringToInt(void) const;

    int digit(void) const;

    int getType(void) const {return type;};

    void setType(int w) { type = w; };

    char* data(void) { *((p->s)+(p->len)) = 0;return p->s;};


    // ---------------------------------------------------------------
    //  Operatoren
    // ---------------------------------------------------------------

    // Zuweisung von char*
    String& operator =(const char* s);
    
    // Zuweisung von anderem String
    String& operator =(const String& s);

    // Subscript operator fuer nicht konstante Strings
    char& operator [](int i);

    // Subscript operator fuer konstante Strings    
    const char& operator [](int i) const;

    void operator +=(const String& s1);

    operator const char* () const {
	return p->s;
    };


    // ----------------
    // Hiermit sehr vorsichtig sein
    // Noch umaenderen
    operator char* () const {
	return p->s;
    }; 


    // ---------------------------------------------------------------
    //  Konstruktor/Destruktor
    // ---------------------------------------------------------------

    // Default
    String();
    
    // Vom char*
    String(const char* s);
    
    // Copy
    String(const String& s); 	
        
    String(int len);

    ~String();

};

// String operator +(const String& s1, const String& s2);
// String IntToString(const long int c);

inline
String::String()
{
    p = new srep;
}

inline
String::String(int len)
{
    p = new srep;
    p->s = new char[len+1];
    p->len = len;
}

inline
String::String(const char* s)
{
    p = new srep(s);
}

inline
String::String(const String& s1)
{
    p = s1.p;
    p->n++;
}

inline
String::~String()
{
    p->n--;
    if (!p->n)
      delete p;
}

inline String&
String::operator =(const char* s)
{
    p->n--;
    if (!p->n) {
	delete p;
    };
    p = new srep(s);
    return *this;
}

inline String&
String::operator =(const String& s1)
{
    if (this == &s1)
	return *this;
    p->n--;
    if (!p->n) {
	delete p;
    };
    p = s1.p;
    p->n++;
    return *this;
}

inline char&
String::operator [](int i)
{
    if ((unsigned int) i>p->len || i<0) 
	cerr <<"String Index ausserhalb der Grenze\n";
    if (p->n > 1) {
	srep* np = new srep(p->s);
	p->n--;
	p = np;
    }
    return p->s[i];
}

inline const char&
String::operator [](int i) const
{
    if ((unsigned int) i>p->len || i<0)
	cerr << "String Index ausserhalb der Grenze";
    return p->s[i];
}

inline bool
String::contains(const char* s) const
{
    return strstr(p->s, s) != NULL;
}

inline String
String::before(const char* s) const
{
    char* adr = strstr(p->s, s);
    if (!adr)
	return "";

    return FromTo(0, adr-p->s);
}

inline String
String::after(const char* s) const
{
    char *adr = strstr(p->s, s);
    if (!adr)
	return "";
    int start = adr-p->s+strlen(s);

    return FromTo(start, length());
}

inline int
String::StringToInt() const
{
    return atoi(p->s);
}

inline int
String::digit() const
{
    char *t = p->s;
    if(!isdigit(*t)) 
      return 0;
    return 1;
}

inline String
DoubleToString(const double c)  
{
    char buffer[30];

    sprintf(buffer, "%.0f", c);

    return String(buffer);
}

inline String
IntToString(const long int c)  
{
    char buffer[30];

    sprintf(buffer, "%ld", c);

    return String(buffer);
}

inline String 
operator +(const String& s1,const String& s2)
{
    String s3(s1);
    s3 += s2;
    return s3;
}

inline String
String::gsub(const char* s, const char* sub) const
{
    if (!contains(s))
	return *this;

    String result = before(s);
    result += (sub + after(s).gsub(s,sub));
    return result;
}

#endif
