/*****************************************************************************/
/*  formula.y                                                                */
/*  YACC-Quelldatei: Parser fuer temporallogische Formeln mit Makros.        */
/*  Mi.,13.07.1994 - Mi.,10.08.1994                           Stefan Roemer  */
/*****************************************************************************/

%{

#include <stdlib.h>
#include "neunetze.h"
#include "scanner.h"
#include "set.h"
#include "output.h"
#include "transform.h"
#include "sdlform.h"
#include "memlib.h"

%}

/* Moegliche Datentypen der Nichtterminalsymbole */
%union {  
         Cardinal32 number;
         Place      place;
         Transition transition;
         Macro      macro;
         ASET       aset;
         SET        set;
	 BTree	    bnode;
	 T_variable var;
	 T_defitem  def;
	 T_list	    list;
	 T_ident    ident;
	 T_sigident sig;
       }


/* Token ( Terminalsymbole ) der Grammatik, ausser 'Interpunktion' */        
/* Diese Tabelle muss UNVERAENDERT bleiben; sie darf evtl.         */
/* VERLAENGERT werden.                                             */
/* Die Token-Liste in beiden Grammatiken muss gleich sein!	   */

%token  BOX
%token  DIAMOND
%token  FORALL
%token  EXISTS
%token  IMPLICATION
%token  EQUIVALENCE
%token  OR
%token  AND
%token  NOT
%token  FALSE
%token  TRUE
%token  UNION
%token  INTERSECTION
%token  DIFFERENCE
%token  PRESET
%token  POSTSET
%token  ALL_PLACES
%token  ENTRY_PLACES
%token  EXIT_PLACES
%token  ALL_TRANSITIONS
%token  YYDOT
%token  FORM_DELIM
%token  <place> PLACE
%token  <number> NUMBER
%token  <transition> TRANSITION
%token  <set> EXTPLABEL
%token  <macro> MACRO
%token  <macro> F_MACRO_CONC
%token  <macro> F_MACRO_DEAD
%token  <macro> F_MACRO_LIVE
%token  <macro> F_MACRO_TERM
%token  <macro> F_MACRO_REV
%token  <macro> F_MACRO_PREACH
%token  <macro> F_MACRO_REACH
%token  <macro> F_MACRO_EXCL

%token  SDL_FORALL SDL_EXISTS
%token  <var> SDL_VARIABLE
%token  <def> SDL_CHANNEL_IDENT SDL_VAR_IDENT
%token  <def> SDL_PRCS_IDENT SDL_PRCD_IDENT
%token	SDL_ALLPIDS SDL_IDSET
%token	SDL_VARSET SDL_QUEUEPOS
%token	SDL_PROPERTY
%token	SDLP_NOQUEUE
%token	SDLP_NOINSTANCES
%token	SDLP_QUEUE
%token	SDLP_PARENT
%token	SDLP_OFFSPRING
%token	SDLP_SENDER
%token	SDLP_SIGNAL
%token	SDLP_RECEIVER
%token	SDLP_STATE
%token	SDLP_CALLER

%token	LE GE
%token	SDL_NULL
%token	SDL_SYSTEM
%token	SDL_INTERNAL
%token	SDL_LEAVING
%token	SDL_EMPTY
%token	SDL_TRUE

/* LTL-Tokens */
%token	LTL_ALWAYS
%token	LTL_EVENTUALLY
%token	LTL_UNTIL
%token	LTL_DUAL

/* Rueckgabetypen der nichtterminalen Grammatiksymbole */
%type  <bnode>	formula
%type  <bnode>	macro
%type  <aset>	pl_set
%type  <aset>	pl_list
%type  <aset>	safe_pl_set
%type  <aset>	safe_pl_list
%type  <aset>	unsafe_pl_set
%type  <aset>	places
%type  <aset>	tr_set
%type  <aset>	tr_list
%type  <aset>	trans
%type  <bnode>	quant_formula
%type  <number>	quant_op
%type  <list>	quant_set
%type  <list>	explicit_set
%type  <list>	set_elem
%type  <def>	sdl_proto_process
%type  <def>	sdl_proto_procedure
%type  <def>	sdl_proto_variable
%type  <ident>	sdl_process_ident
%type  <ident>	sdl_qual_process
%type  <ident>	sdl_qual_caller
%type  <ident>	sdl_qual_prcs2
%type  <ident>	sdl_qual_prcd2
%type  <def>	sdl_unqual_process
%type  <def>	sdl_unqual_procedure
%type  <ident>	sdl_procedure_ident
%type  <ident>	sdl_qual_procedure
%type  <ident>	sdl_variable_ident
%type  <bnode>	sdl_queue
%type  <bnode>	sdl_property
%type  <bnode>	arith_op class_op
%type  <bnode>	any_op state_op
%type  <bnode>	caller_op
%type  <bnode>	signal_op
%type  <ident>	value
%type  <bnode>	condition
%type  <var>	state_name
%type  <ident>	sigval
%type  <list>	sigargs
%type  <sig>	signal_name


/* Startsymbol der Grammatik */
%start  multi


/* Prioritaeten und Assoziativitaeten der Operatoren */

%nonassoc ','
%nonassoc BOX LTL_ALWAYS
%nonassoc DIAMOND LTL_EVENTUALLY
%nonassoc ':'
%left     IMPLICATION
%left     EQUIVALENCE DIFFERENCE
%nonassoc FORALL EXISTS
%left	  LTL_UNTIL LTL_DUAL
%left     OR UNION
%left     AND INTERSECTION
%nonassoc NOT YYDOT



%%

multi	: formula
	  {
		if (sdl_variables) $1 = sdl_use_node($1);
		print_form($1);
		ESstr( "\n" );
	  }
	| multi FORM_DELIM formula
	  {
		ESstr( "\n.\n" );
		if (sdl_variables) $3 = sdl_use_node($3);
		print_form($3);
	  }
	;

formula	: '(' formula ')'
		{ $$ = NewNode('(', NULL, $2); }
	| LTL_ALWAYS formula
		{ $$ = NewNode(LTL_ALWAYS, NULL, $2); }
	| LTL_EVENTUALLY formula	
		{ $$ = NewNode(LTL_EVENTUALLY, NULL, $2); }
	| formula LTL_UNTIL formula
		{ $$ = NewNode(LTL_UNTIL, $1, $3); }
	| formula LTL_DUAL formula
		{ $$ = NewNode(LTL_DUAL, $1, $3); }
	| formula AND formula
		{ $$ = NewNode(AND, $1, $3); }
	| formula OR formula
		{ $$ = NewNode(OR, $1, $3); }
	| NOT formula
		{ $$ = NewNode(NOT, NULL, $2); }
	| formula IMPLICATION formula
		{ $$ = NewNode(IMPLICATION, $1, $3); }
	| formula EQUIVALENCE formula
		{ $$ = NewNode(EQUIVALENCE, $1, $3); }
	| safe_pl_list
		{ $$ = form_ext_pl_label($1->set, $1->or? OR : AND); }
	| places
		{ $$ = form_ext_pl_label($1->set, $1->or? OR : AND); }
	| macro
		{ $$ = $1; }
	| TRUE
		{ $$ = NewNode(TRUE, NULL, NULL); }
	| FALSE
		{ $$ = NewNode(FALSE, NULL, NULL); }
	| quant_formula
	| sdl_property
		{ $$ = NewNode(SDL_PROPERTY, $1, NULL); }
	| condition
	;

places	: PLACE
		{ set_insert(&(($$ = allocASET())->set), $1); }
	| EXTPLABEL
		{ ($$ = allocASET())->set = $1; }
	;

safe_pl_set
	: ALL_PLACES
	  {
		register Place pl = FT_net->LastPlace;
		$$ = allocASET();
		while (pl)
		{
			set_insert(&($$->set), pl);
			pl = pl->prevplace;
		} /* while */
	  }
	| ENTRY_PLACES
	  {
		register Place pl = FT_net->LastPlace;
		TestPetriBox();
		$$ = allocASET();
		while (pl)
		{
			if (pl->label == ENTRY) set_insert(&($$->set), pl);
			pl = pl->prevplace;
		} /* while */
	  }
	| EXIT_PLACES
          {
		register Place pl = FT_net->LastPlace;
		TestPetriBox();
		$$ = allocASET();
		while (pl)
		{
			if (pl->label == EXIT) set_insert(&($$->set), pl);
			pl = pl->prevplace;
		} /* while */
	  }
	| YYDOT tr_set
	  {
		($$ = allocASET())->set = calcTpreset($2->set);
		$$->or = $2->or;
	  }
	| PRESET '(' tr_list ')'
	  {
		($$ = allocASET())->set = calcTpreset($3->set);
		$$->or = $3->or;
	  }
	| POSTSET '(' tr_list ')'
	  {
		($$ = allocASET())->set = calcTpostset($3->set);
		$$->or = $3->or;
	  }
	;

safe_pl_list
	: safe_pl_set
		{ $$ = $1; }
	| AND safe_pl_set
		{ ($$ = $2)->or = False; }
	| OR safe_pl_set
		{ ($$ = $2)->or = True; }
	;

unsafe_pl_set
	: '(' pl_list ')'
	  {
		$$ = $2;
	  }
	| '{' pl_list '}'
	  {
		$$ = $2;
	  }
	| tr_set YYDOT
	  {
		($$ = allocASET())->set = calcTpostset($1->set);
		$$->or = $1->or;
	  }
	| pl_set UNION pl_set
	  {
		($$ = $1)->or |= $3->or;
		set_union(&($$->set), $3->set);
	  }
	| pl_set INTERSECTION pl_set
	  {
		($$ = $1)->or |= $3->or;
		set_intersection(&($$->set), $3->set);
	  }
	| pl_set DIFFERENCE pl_set
	  {
		($$ = $1)->or |= $3->or;
		set_difference(&($$->set), $3->set);
	  }
	;

pl_set  : safe_pl_set
		{ $$ = $1; }
	| unsafe_pl_set
		{ $$ = $1; }
	| AND pl_set
		{ ($$ = $2)->or = False; }
	| OR pl_set
		{ ($$ = $2)->or = True; }
	;

pl_list	: places
		{ $$ = $1; }
	| pl_set
		{ $$ = $1; }
	| pl_list ',' places
		{ ($$ = $1)->or |= $3->or;
		  set_union(&($$->set), $3->set); }
	| pl_list AND places
		{ ($$ = $1)->or |= $3->or;
		  set_union(&($$->set), $3->set); }
	| pl_list OR places
		{ ($$ = $1)->or = True;
		  set_union(&($$->set), $3->set); }
	;

macro	: F_MACRO_CONC '(' tr_list ')'	{ $$ = $1($3); }
	| F_MACRO_DEAD			{ $$ = $1(NULL); }
	| F_MACRO_EXCL '(' pl_list ')'	{ $$ = $1($3); }
	| F_MACRO_LIVE
	  {
		register ASET aset = allocASET();
		register Transition tr = FT_net->LastTransition;
		while (tr)
		{
			set_insert(&(aset->set), (Place) tr);
			tr = tr->prevtrans;
		} /* while */
		$$ = $1(aset);
	  }
	| F_MACRO_LIVE '(' tr_list ')'	{ $$ = $1($3); }
	| F_MACRO_PREACH '(' pl_list ')'{ $$ = $1($3); }
	| F_MACRO_REACH '(' pl_list ')' { $$ = $1($3); }
	| F_MACRO_REV			{ $$ = $1(NULL); }
	| F_MACRO_TERM			{ $$ = $1(NULL); }
	;

trans	: TRANSITION
		{ set_insert(&(($$ = allocASET())->set), (Place) $1); }
	| tr_set
		{ $$ = $1; }
	;

tr_set	: '{' tr_list '}'
	  {
		$$ = $2;
	  }
	| 'T'
	  {
		register Transition tr = FT_net->LastTransition;
		$$ = allocASET();
		while (tr)
		{
			set_insert(&($$->set), (Place) tr);
			tr = tr->prevtrans;
		} /* while */
	  }
	| YYDOT pl_set
	  {
		($$ = allocASET())->set = calcPpreset($2->set);
		$$->or = $2->or;
	  }
	| PRESET '(' pl_list ')'
	  {
		($$ = allocASET())->set = calcPpreset($3->set);
		$$->or = $3->or;
	  }
	| pl_set YYDOT
	  {
		($$ = allocASET())->set = calcPpostset($1->set);
		$$->or = $1->or;
	  }
	| POSTSET '(' pl_list ')'
	  {
		($$ = allocASET())->set = calcPpostset($3->set);
		$$->or = $3->or;
	  }
	| tr_set UNION tr_set
	  {
		($$ = $1)->or |= $3->or;
		set_union(&($$->set), $3->set);
	  }
	| tr_set INTERSECTION tr_set
	  {
		($$ = $1)->or |= $3->or;
		set_intersection(&($$->set), $3->set);
	  }
	| tr_set DIFFERENCE tr_set
	  {
		($$ = $1)->or |= $3->or;
		set_difference(&($$->set), $3->set);
	  }
	;

tr_list : trans
		{ $$ = $1; }
	| tr_list ',' trans
		{ ($$ = $1)->or |= $3->or;
		  set_union(&($$->set), $3->set); }
	| tr_list AND trans
		{ ($$ = $1)->or |= $3->or;
		  set_union(&($$->set), $3->set); }
	| tr_list OR trans
		{ ($$ = $1)->or = True;
		  set_union(&($$->set), $3->set); }
	| AND tr_list
		{ ($$ = $2)->or = False; }
	| OR tr_list
		{ ($$ = $2)->or = True; }
	;

/************ ab hier SDL-Formeln ************/

quant_formula
	: quant_op SDL_VARIABLE quant_set ':'
	  {
		if ($2->set) yyerror("variable name already in use");
		$2->set = $3;
		list_add(sdl_variables,$2);
	  }
	  formula
	  {
		list_remove_tail(sdl_variables);
		$$ = NewNode($1,(BTree)$2,$6);
	  }
	;

quant_op: SDL_FORALL { $$ = SDL_FORALL; }
	| SDL_EXISTS { $$ = SDL_EXISTS; }
	;

quant_set
	: '{' explicit_set '}'
	  {	$$ = $2;	}
	| SDL_IDSET '(' sdl_proto_process ')'
	  {	$$ = $3->set;	}
	| SDL_IDSET '(' sdl_proto_procedure ')'
	  {	$$ = $3->set;	}
	| SDL_ALLPIDS
	  {	$$ = sdl_pidset;}
	| SDL_VARSET '(' sdl_proto_variable ')'
	  {	$$ = $3->set;	}
	| SDL_QUEUEPOS '(' sdl_proto_process ')'
	  {
		int i;
		$$ = alloc_list();
		for (i = 1; i <= $3->size; i++) list_add($$,i);
	  }
	| SDL_QUEUEPOS '(' SDL_CHANNEL_IDENT ')'
	  {
		int i;
		$$ = alloc_list();
		for (i = 1; i <= $3->size; i++) list_add($$,i);
	  }
	;

explicit_set
	: set_elem
	| set_elem ',' explicit_set
	  {
		int i;
		for_list ($3,i,int) list_add($1,i);
		$$ = $1;
	  }
	;

set_elem: NUMBER
	  {
		list_add($$ = alloc_list(), $1);
	  }
	| NUMBER YYDOT YYDOT NUMBER
	  {
		int i;
		$$ = alloc_list();
		for (i = $1; i <= $4; i++) list_add($$,i);
	  }
	;

/************ allgemeine Identifier ************/

sdl_proto_process
	: SDL_PRCS_IDENT;

sdl_proto_procedure
	: SDL_PRCD_IDENT;

sdl_proto_variable
	: sdl_proto_process YYDOT SDL_VAR_IDENT
	  {	$$ = sdl_search_var_def($1,$3->name);		}
	| sdl_proto_procedure YYDOT SDL_VAR_IDENT
	  {	$$ = sdl_search_var_def($1,$3->name);		}
	;

/************ spezielle Identifier fuer Prozesse ************/

sdl_process_ident
	: sdl_proto_process
	  {
		$$ = alloc_ident();
		$$->ptr = $1;
		$$->value = 0;
		$$->var = NULL;
	  }
	| sdl_proto_process '[' NUMBER ']'
	  {
		$$ = alloc_ident();
		$$->ptr = $1;
		$$->value = $3;
		$$->var = NULL;
	  }
	| sdl_proto_process '[' SDL_VARIABLE ']'
	  {
		if (!$3->set) yyerror("undeclared variable");
		$$ = alloc_ident();
		$$->ptr = $1;
		$$->value = 0;
		$$->var = $3;
	  }
	;

/* diese Regeln hier moegen umstaendlich wirken, aber es war
   nicht ganz einfach, um shift/reduce-Konflikte herumzukommen
*/

sdl_qual_process
	: sdl_process_ident YYDOT
	  {
		if (!$1->value && !$1->var)
		{
			$1->value = 1;
			if ($1->ptr->set->count > 1)
				yyerror("need to specify process instance");
		}
		$$ = $1;
	  }
	;

sdl_qual_prcs2
	: sdl_process_ident
	  {
		if (!$1->value && !$1->var)
		{
			$1->value = 1;
			if ($1->ptr->set->count > 1)
				yyerror("need to specify process instance");
		}
		$$ = $1;
	  }
	| SDL_NULL
	  {	$$ = NULL;	}
	| SDL_SYSTEM
	  {	$$ = sdl_system;	}
	;

sdl_unqual_process
	: sdl_process_ident YYDOT
	  {
		if ($1->value || $1->var)
			yyerror("unexpected process instance specification");
		$$ = $1->ptr;
		free($1);
	  }
	;

/************ spezielle Identifier fuer Prozeduren ************/

sdl_procedure_ident
	: sdl_proto_procedure
	  {
		$$ = alloc_ident();
		$$->ptr = $1;
		$$->value = 0;
		$$->var = NULL;
	  }
	| sdl_proto_procedure '[' NUMBER ']'
	  {
		$$ = alloc_ident();
		$$->ptr = $1;
		$$->value = $3;
		$$->var = NULL;
	  }
	| sdl_proto_procedure '[' SDL_VARIABLE ']'
	  {
		if (!$3->set) yyerror("undeclared variable");
		$$ = alloc_ident();
		$$->ptr = $1;
		$$->value = 0;
		$$->var = $3;
	  }
	;

sdl_qual_procedure
	: sdl_procedure_ident YYDOT
	  {
		if (!$1->value && !$1->var)
		{
			$1->value = 1;
			if ($1->ptr->set->count > 1)
				yyerror("need to specify procedure instance");
		}
		$$ = $1;
	  }
	;

sdl_qual_prcd2
	: sdl_procedure_ident
	  {
		if (!$1->value && !$1->var)
		{
			$1->value = 1;
			if ($1->ptr->set->count > 1)
				yyerror("need to specify procedure instance");
		}
		$$ = $1;
	  }
	;

sdl_unqual_procedure
	: sdl_procedure_ident YYDOT
	  {
		if ($1->value || $1->var)
			yyerror("unexpected procedure instance specification");
		$$ = $1->ptr;
		free($1);
	  }
	;

/************ spezielle Identifier fuer Variablen ************/

sdl_variable_ident
	: sdl_qual_process SDL_VAR_IDENT
	  {	$$ = sdl_search_var_ident($1,$2->name);		}
	| sdl_qual_procedure SDL_VAR_IDENT
	  {	$$ = sdl_search_var_ident($1,$2->name);		}
	;

/************ Element in der Warteschlange ************/

sdl_queue
	: SDLP_QUEUE '[' value ']' YYDOT
	  {
		$$ = (BTree) $3;
	  }
	;
		
/************ Eigenschaften ************/

sdl_property
	: sdl_qual_process SDLP_NOQUEUE arith_op
	  {
		$$ = NewNode(SDLP_NOQUEUE,(BTree)$1,$3);
	  }
	| sdl_unqual_process SDLP_NOINSTANCES arith_op
	  {
		$$ = NewNode(SDLP_NOINSTANCES,(BTree)$1,$3);
	  }
	| sdl_qual_process SDLP_PARENT any_op
	  {
		$$ = NewNode(SDLP_PARENT,(BTree)$1,$3);
	  }
	| sdl_qual_process SDLP_OFFSPRING any_op
	  {
		$$ = NewNode(SDLP_OFFSPRING,(BTree)$1,$3);
	  }
	| sdl_qual_process SDLP_SENDER any_op
	  {
		$$ = NewNode(SDLP_SENDER,(BTree)$1,$3);
	  }
	| sdl_variable_ident any_op
	  {
		$$ = NewNode(SDL_VARIABLE,(BTree)$1,$2);
		if (($1->ptr->set == sdl_pidset) != ($2->op <= 1))
			yyerror("operand mismatch");
	  }
	| sdl_unqual_procedure YYDOT SDLP_NOINSTANCES arith_op
	  {
		$$ = NewNode(SDLP_NOINSTANCES,(BTree)$1,$4);
	  }
	| SDL_CHANNEL_IDENT YYDOT SDLP_NOQUEUE arith_op
	  {
		T_ident ident = alloc_ident();
		ident->ptr = $1;
		$$ = NewNode(SDLP_NOQUEUE,(BTree)ident,$4);
	  }
	| sdl_qual_process SDLP_STATE state_op
	  {
		$$ = NewNode(SDLP_STATE,(BTree)$1,$3);
	  }
	| sdl_qual_procedure SDLP_STATE state_op
	  {
		$$ = NewNode(SDLP_STATE,(BTree)$1,$3);
	  }
	| sdl_qual_process sdl_queue SDLP_SIGNAL signal_op
	  {
		$$ = NewNode(SDLP_SIGNAL,(BTree)$1,$4);
		$4->left = $2;
	  }
	| sdl_qual_process sdl_queue SDLP_SENDER any_op
	  {
		$$ = NewNode(SDLP_QUEUE,(BTree)$1,$4);
		$4->left = $2;
	  }
	| SDL_CHANNEL_IDENT YYDOT sdl_queue SDLP_SIGNAL signal_op
	  {
		T_ident ident = alloc_ident();
		ident->ptr = $1;
		$$ = NewNode(SDLP_SIGNAL,(BTree)ident,$5);
		$5->left = $3;
	  }
	| SDL_CHANNEL_IDENT YYDOT sdl_queue SDLP_SENDER any_op
	  {
		T_ident ident = alloc_ident();
		ident->ptr = $1;
		$$ = NewNode(SDLP_QUEUE,(BTree)ident,$5);
		$5->left = $3;
	  }
	| SDL_CHANNEL_IDENT YYDOT sdl_queue SDLP_RECEIVER any_op
	  {
		T_ident ident = alloc_ident();
		ident->ptr = $1;
		$$ = NewNode(SDLP_RECEIVER,(BTree)ident,$5);
		$5->left = $3;
	  }
	| sdl_qual_procedure SDLP_CALLER caller_op
	  {
		$$ = NewNode(SDLP_CALLER,(BTree)$1,$3);
	  }
	;

/************ Operatoren ************/

arith_op: '=' value	{ $$ = NewNode('=',NULL,(BTree)$2);	}
	| BOX value	{ $$ = NewNode('#',NULL,(BTree)$2);	}
	| '<' value	{ $$ = NewNode('<',NULL,(BTree)$2);	}
	| '>' value	{ $$ = NewNode('>',NULL,(BTree)$2);	}
	| LE value	{ $$ = NewNode(LE,NULL,(BTree)$2);	}
	| GE value	{ $$ = NewNode(GE,NULL,(BTree)$2);	}
	;

class_op: '=' sdl_qual_prcs2	{ $$ = NewNode(0,NULL,(BTree)$2);	}
	| BOX sdl_qual_prcs2	{ $$ = NewNode(1,NULL,(BTree)$2);	}
	;

any_op	: arith_op
	| class_op
	;

state_op: '=' state_name
	  { $$ = NewNode('=',NULL,(BTree)$2); }
	| BOX state_name
	  { $$ = NewNode('#',NULL,(BTree)$2); }
	;

signal_op
	: '=' signal_name
	  { $$ = NewNode('=',NULL,(BTree)$2); }
	| BOX signal_name
	  { $$ = NewNode('#',NULL,(BTree)$2); }
	;

caller_op
	: '=' sdl_qual_caller	{ $$ = NewNode(0,NULL,(BTree)$2);	}
	| BOX sdl_qual_caller	{ $$ = NewNode(1,NULL,(BTree)$2);	}
	;

sdl_qual_caller
	: sdl_qual_prcs2
	  {
		if (!$1 || $1 == sdl_system)
			yyerror("process identifier expected");
		$$ = $1;
	  }
	| sdl_qual_prcd2
	;

/************ Werte ************/

value	: NUMBER
	  {
		$$ = alloc_ident();
		$$->ptr = NULL; $$->value = $1; $$->var = NULL;
	  }
	| SDL_VARIABLE
	  {
		if (!$1->set) yyerror("undeclared variable");
		$$ = alloc_ident();
		$$->ptr = NULL; $$->value = 0; $$->var = $1;
	  }
	| TRUE
	  {
		$$ = alloc_ident();
		$$->ptr = NULL; $$->value = TT; $$->var = NULL;
	  }
	| FALSE
	  {
		$$ = alloc_ident();
		$$->ptr = NULL; $$->value = FF; $$->var = NULL;
	  }
	;

condition:value any_op
	  {
		$$ = NewNode(SDL_TRUE,(BTree)$1,$2);
	  }
	;

state_name
	: SDL_VARIABLE
	| SDL_INTERNAL
	  {
		$$ = alloc_variable();
		$$->name = NULL;
		$$->value = DOT;
	  }
	| SDL_LEAVING
	  {
		$$ = alloc_variable();
		$$->name = NULL;
		$$->value = 0;
	  }
	;

/************ Signale ************/

sigval	: value
	| sdl_qual_prcs2
	;

sigargs	: sigval
	  {	list_add($$ = alloc_list(), $1);	}
	| sigval ',' sigargs
	  {	list_add($$ = $3, $1);			}
	;

signal_name
	: SDL_VARIABLE
	  {
		T_sigident sig = alloc_sigident();
		sig->name = $1->name;
		sig->args = NULL;
		$$ = sig;
	  }
	| SDL_VARIABLE '(' sigargs ')'
	  {
		T_sigident sig = alloc_sigident();
		sig->name = $1->name;
		sig->args = $3;
		revert_list($3);
		$$ = sig;
	  }
	| SDL_EMPTY
	  {
		T_sigident sig = alloc_sigident();
		sig->name = NULL;
		sig->args = NULL;
		$$ = sig;
	  }
	;
%%

/*****************************************************************************/

/* no trailer, no further includes here */

/*****************************************************************************/
/* END OF FILE formula.y                                                     */
/*****************************************************************************/

