/*****************************************************************************/
/*  output.c                                                                 */
/*  C source file: output routines for the formula translator.               */
/*  Mo.,08.08.1994 - Mo.,08.08.1994                           Stefan Roemer  */
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "neunetze.h"
#include "output.h"
#include "transform.h"
#include "scanner.h"
#include "sdlform.h"
#include "formula.h"

/*****************************************************************************/

bool       FT_pretty     = True,
           FT_userlike   = False;
char       * FT_OutString,
           * FT_OutStringPtr,
           * FT_OutStringMax;
Cardinal32 FT_OutStringLen,
           FT_OutStringX=0,
           FT_OutXpos,
           FT_OutYpos;

/*****************************************************************************/

#define FONT_EMPTYSET     0xAA
#define FONT_AND          0xAC
#define FONT_NOT          0xAE
#define FONT_EQUIVALENCE  0xAF
#define FONT_IMPLICATION  0xB0
#define FONT_BOX          0xB6
#define FONT_DIAMOND      0xB7
#define FONT_OR           0xC7
#define FONT_FORALL       179
#define FONT_EXISTS       180
#define FONT_DOT          0xFD /*!?!*/
#define FONT_UNION        0xFE /*!?!*/
#define FONT_INTERSECTION 0xFF /*!?!*/

unsigned char *sym_and          = "*",
              *sym_or           = "+",
              sym_not           = '-',
              *sym_box           = "#",
              *sym_diamond       = "^",
              sym_forall        = FONT_FORALL,
              sym_exists        = FONT_EXISTS,
              sym_union         = FONT_UNION,
              sym_intersection  = FONT_INTERSECTION,
              *sym_false         = "0",
              *sym_true          = "1",
              sym_l_par         = '(',
              sym_r_par         = ')',
	      sym_until		= 'U',
	      sym_dual		= 'V',
              sym_comment       = '%';
char          *sym_implication	= "=>",
              *sym_equivalence	= "<=>",
	      *sym_always	= "[]",
	      *sym_eventually	= "<>";

/*****************************************************************************/
/* Tastaturumbelegungen einiger Symbole.                                     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void set_hartmut_keycodes( void )

#else

void set_hartmut_keycodes()

#endif

  {

    sym_and          = "*";
    sym_or           = "+";
    sym_not          = 'N';
    sym_box          = "B";
    sym_diamond      = "D";
    sym_forall       = '!';
    sym_exists       = '?';
    sym_union        = 'u';
    sym_intersection = 'n';
    sym_false        = "F";
    sym_true         = "T";
    sym_implication  = "->";
    sym_equivalence  = "<->";
    sym_comment      = ';';

  } /* set_hartmut_keycodes() */


/*****************************************************************************/
/* Tastaturumbelegungen einiger Symbole.                                     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void set_PEP_keycodes( void )

#else

void set_PEP_keycodes()

#endif

  {

    sym_and          = "*";
    sym_or           = "+";
    sym_not          = '-';
    sym_box          = "#";
    sym_diamond      = "^";
    sym_forall       = '!';
    sym_exists       = '?';
    sym_union        = 'u';
    sym_intersection = 'n';
    sym_false        = "0";
    sym_true         = "1";
    sym_implication  = "=>";
    sym_equivalence  = "<=>";
    sym_comment      = '%';

  } /* set_PEP_keycodes() */


/*****************************************************************************/
/* Bezeichnungen fuer LTL-Operatoren                                         */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)
void set_LTL_keycodes( void )
#else
void set_LTL_keycodes()
#endif
{
	sym_and		 = "&&";
	sym_or		 = "||";
	sym_not          = '!';
	sym_box          = "[]";
	sym_diamond      = "<>";
	sym_false        = "false";
	sym_true         = "true";
	sym_implication  = "->";
	sym_equivalence  = "<->";
	sym_comment      = '%';
	sym_always	 = "[]";
	sym_eventually	 = "<>";
	sym_until	 = 'U';
	sym_dual	 = 'V';

} /* set_LTL_keycodes() */

/*****************************************************************************/
/* Expand the output string.                                                 */

#if defined(__STDC__) || defined(__cplusplus)

static void expand( void )

#else

static void expand()

#endif

  {
    register int offset = ( (int) FT_OutStringPtr ) - ( (int) FT_OutString );

    FT_OutStringMax = FT_OutString = (char *) Realloc( (char *) FT_OutString,
                  ( FT_OutStringLen += OUTSTRING_LENGTH ) * sizeof( char ) );
    FT_OutStringMax += FT_OutStringLen;
    FT_OutStringPtr = FT_OutString + offset;
  } /* expand() */


/*****************************************************************************/

#define Xch(CH) if ((Cardinal32)FT_OutStringPtr > (Cardinal32)FT_OutStringMax)\
              expand(); \
	    *( FT_OutStringPtr++ ) = (CH); FT_OutStringX++

#define Xstr( STR ) { register char * ptr = (STR); \
                      while ( *ptr ) \
                        { Xch( *ptr ); ++ptr; }\
                    }


/*****************************************************************************/
/*****************************************************************************/
/* Print a single character and end this line.                               */

#if defined(__STDC__) || defined(__cplusplus)

void Ech( char ch )

#else

void Ech( ch )

  char ch;

#endif

  {

        Xch( ch );
        *FT_OutStringPtr = '\0';
        if ( FT_OutXpos + FT_OutStringX > LINEWIDTH )
          {
            putc( '\n', yyout );
            FT_OutXpos = 0;
          } /* if */
        FT_OutXpos += FT_OutStringX;
        fprintf( yyout, "%s", FT_OutString );
        Xinit();

  } /* Ech() */


/*****************************************************************************/
/* Print a string and end this line.                                         */

#if defined(__STDC__) || defined(__cplusplus)

void Estr( char * str )

#else

void Estr( str )

  char * str;

#endif

  {

        Xstr( str );
        *FT_OutStringPtr = '\0';
        if ( FT_OutXpos + FT_OutStringX > LINEWIDTH )
          {
            putc( '\n', yyout );
            FT_OutXpos = 0;
          } /* if */
        FT_OutXpos += FT_OutStringX;
        fprintf( yyout, "%s", FT_OutString );
        Xinit();

  } /* Estr() */


/*****************************************************************************/
/* Print a single character with leading spaces.                             */

#if defined(__STDC__) || defined(__cplusplus)

void PSch( char ch )

#else

void PSch( ch )

  char ch;

#endif

  {
    if ( FT_pretty )
    {
      Xch( ' ' );
    }
    else if ( FT_userlike )
    {
      Xstr( FT_space );
    }
    Xch( ch );
  } /* PSch() */


/*****************************************************************************/
/* Print a single character with leading spaces and end this line.           */

#if defined(__STDC__) || defined(__cplusplus)

void ESch( char ch )

#else

void ESch( ch )

  char ch;

#endif

  {
    if ( FT_pretty )
    {
      Xch( ' ' );
    }
    else if ( FT_userlike )
    {
      Xstr( FT_space );
    }
    Ech(ch);
  } /* ESch() */


/*****************************************************************************/
/* Print a string with leading spaces.                                       */

#if defined(__STDC__) || defined(__cplusplus)

void PSstr( char * str )

#else

void PSstr( str )

  char * str;

#endif

  {
    if ( FT_pretty )
    {
      Xch( ' ' );
    }
    else if ( FT_userlike )
    {
      Xstr( FT_space );
    }
    Xstr( str );

  } /* PSstr() */


/*****************************************************************************/
/* Print a string with leading spaces and end this line.                     */

#if defined(__STDC__) || defined(__cplusplus)

void ESstr( char * str )

#else

void ESstr( str )

  char * str;

#endif

  {
    if ( FT_pretty )
    {
      Xch( ' ' );
    }
    else if ( FT_userlike )
    {
      Xstr( FT_space );
    }
    Estr(str);
  } /* ESch() */


/*****************************************************************************/
/* Output the result of a parse tree.					     */

#if defined(__STDC__) || defined(__cplusplus)
void print_form(BTree node)
#else
void print_form(node)
  BTree node;
#endif
{
	if (node->left) print_form(node->left);
	switch (node->op)
	{
		case '(':
			PSl_par(); break;
		case DIAMOND:
			PSdiamond(); break;
		case BOX:
			PSbox(); break;
		case FORALL:
			PSforall(); break;
		case EXISTS:
			PSexists(); break;
		case AND:
			PSand(); break;
		case OR:
			PSor(); break;
		case NOT:
			PSnot(); break;
		case IMPLICATION:
			PSimplication(); break;
		case EQUIVALENCE:
			PSequivalence(); break;
		case TRUE:
			PStrue(); break;
		case FALSE:
			PSfalse(); break;
		case PLACE:
			PSstr(((Place) node->right)->name); return;
		case LTL_ALWAYS:
			PSalways(); break;
		case LTL_EVENTUALLY:
			PSeventually(); break;
		case LTL_UNTIL:
			PSuntil(); break;
		case LTL_DUAL:
			PSdual(); break;
	}

	if (node->right) print_form(node->right);

	if (node->op=='(')
	{
	  PSch( sym_r_par );
	}

} /* print_form */

/*****************************************************************************/
/* END OF FILE output.c                                                      */
/*****************************************************************************/

