/*****************************************************************************/
/*  output.h                                                                 */
/*  C header file: output routines for the formula translator.               */
/*  Mo.,08.08.1994 - Mo.,08.08.1994                           Stefan Roemer  */
/*****************************************************************************/

#ifndef __OUTPUT_H__
#define __OUTPUT_H__

/*****************************************************************************/

#include <stdio.h>
#include "neunetze.h"
#include "parser.h"

/*****************************************************************************/

extern bool       FT_pretty,
                  FT_userlike;
extern char       * FT_OutString,
                  * FT_OutStringPtr,
                  * FT_OutStringMax;
extern Cardinal32 FT_OutStringLen,
                  FT_OutStringX;
extern unsigned char *sym_and,
                     *sym_or,
                     sym_not,
                     *sym_box,
                     *sym_diamond,
                     sym_forall,
                     sym_exists,
                     sym_union,
                     sym_intersection,
                     *sym_false,
                     *sym_true,
                     sym_l_par,
                     sym_r_par,
                     sym_comment,
		     sym_until, sym_dual;
extern char          *sym_implication,
                     *sym_equivalence,
		     *sym_always, *sym_eventually;

/*****************************************************************************/

#define LINEWIDTH 75
#define OUTSTRING_LENGTH 3000

/*****************************************************************************/

#define Xinit() *( FT_OutStringPtr = FT_OutString ) = '\0', \
                FT_OutStringX = 0;

/*****************************************************************************/
/* Tastaturumbelegungen einiger Symbole.                                     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern void set_hartmut_keycodes( void );

#else

extern void set_hartmut_keycodes();

#endif


/*****************************************************************************/
/* Tastaturumbelegungen einiger Symbole.                                     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern void set_PEP_keycodes( void );

#else

extern void set_PEP_keycodes();

#endif


/*****************************************************************************/
/* Bezeichnungen fuer LTL-Symbole                                            */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern void set_LTL_keycodes( void );

#else

extern void set_LTL_keycodes();

#endif


/*****************************************************************************/
/* Print a single character and end line.                                    */
 
#if defined(__STDC__) || defined(__cplusplus)
 
void Ech( char );
 
#else
 
void Ech();
 
#endif
 

/*****************************************************************************/
/* Print a string and end line.                                              */
 
#if defined(__STDC__) || defined(__cplusplus)
 
void Estr( char * );
 
#else
 
void Estr();
 
#endif
 

/*****************************************************************************/
/* Print a single character with leading spaces.                             */
 
#if defined(__STDC__) || defined(__cplusplus)
 
void PSch( char );
 
#else
 
void PSch();
 
#endif
 

/*****************************************************************************/
/* Print a single character with leading spaces and end line.                */
 
#if defined(__STDC__) || defined(__cplusplus)
 
void ESch( char );
 
#else
 
void ESch();
 
#endif
 

/*****************************************************************************/
/* Print a string with leading spaces and end line.                          */
 
#if defined(__STDC__) || defined(__cplusplus)
 
void ESstr( char * );
 
#else
 
void ESstr();
 
#endif
 

/*****************************************************************************/
/* Output the result of a parse tree.                                        */
 
#if defined(__STDC__) || defined(__cplusplus)
 
void print_form( BTree );
 
#else
 
void print_form();
 
#endif
 

/*****************************************************************************/

#define Eand()          Estr( sym_and )
#define Eor()           Estr( sym_or )
#define Enot()          Ech( sym_not )
#define Eimplication()  Estr( sym_implication )
#define Eequivalence()  Estr( sym_equivalence )
#define Ebox()          Estr( sym_box )
#define Ediamond()      Estr( sym_diamond )
#define Etrue()         Estr( sym_true )
#define Efalse()        Estr( sym_false )
#define El_par()        Ech( sym_l_par )
#define Er_par()        Ech( sym_r_par )
#define Ealways()       Estr( sym_always )
#define Eeventually()   Estr( sym_eventually )
#define Euntil()        Ech( sym_until )
#define Edual()         Ech( sym_dual )

#define PSand()         PSstr( sym_and )
#define PSor()          PSstr( sym_or )
#define PSnot()         PSch( sym_not )
#define PSimplication() PSstr( sym_implication )
#define PSequivalence() PSstr( sym_equivalence )
#define PSbox()         PSstr( sym_box )
#define PSdiamond()     PSstr( sym_diamond )
#define PSexists()      PSch( sym_exists )
#define PSforall()      PSch( sym_forall )
#define PStrue()        PSstr( sym_true )
#define PSfalse()       PSstr( sym_false )
#define PSl_par()       PSch( sym_l_par )
#define PSr_par()       PSch( sym_r_par )
#define PSalways()      PSstr( sym_always )
#define PSeventually()  PSstr( sym_eventually )
#define PSuntil()       PSch( sym_until )
#define PSdual()        PSch( sym_dual )

#define ESand()         ESstr( sym_and )
#define ESor()          ESstr( sym_or )
#define ESimplication() ESstr( sym_implication )
#define ESequivalence() ESstr( sym_equivalence )
#define EStrue()        ESstr( sym_true )
#define ESfalse()       ESstr( sym_false )
#define ESalways()      ESstr( sym_always )
#define ESeventually()  ESstr( sym_eventually )

/*****************************************************************************/

#endif /* __OUTPUT_H__ */

/*****************************************************************************/
/* END OF FILE output.h                                                      */
/*****************************************************************************/

