/*****************************************************************************/
/* parser.h -- functions used while parsing formulae                         */
/*****************************************************************************/

#ifndef __PARSER_H__
#define __PARSER_H__

#include <stdio.h>
#include "set.h"

#if defined(__STDC__) || defined(__cplusplus)

typedef struct btree_
  {
    int           op;
    SET           (* function)( struct btree_ * );
    struct btree_ * left,
                  * right;
  } BTreeElem,
    * BTree;

typedef BTree (* Macro)( ASET );


#else

typedef BTree (* Macro)();

typedef struct btree_
  {
    int           op;
    Macro         function;
    struct btree_ * left,
                  * right;
  } BTreeElem,
    * BTree;

#endif


typedef struct
  {
    char  * name;
    int   mclass;
    Macro function;
  } MacroTableEntry,
    * MacroTable;


/*****************************************************************************/

extern MacroTable FT_MacroTable;

/*****************************************************************************/
/* Feststellen, ob uebergebenes Netz eine Petri-Box ist.                     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)
 
void TestPetriBox( void );
 
#else
 
void TestPetriBox();

#endif


/*****************************************************************************/
/* Initialize the Macro funtion                                              */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void initMacroTable( bool use_spin );

#else

void initMacroTable( use_spin );

#endif

/*****************************************************************************/
/* Generate a new node for the parse tree.                                   */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern BTree NewNode( int, BTree, BTree );

#else

extern BTree NewNode( /* int op, BTree left, BTree right */ );

#endif


/*****************************************************************************/
/* Parse through an extended place labelling.                                */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern BTree form_ext_pl_label( SET, int );

#else

extern BTree form_ext_pl_label( /* SET plset, int op */ );

#endif

/*****************************************************************************/

#endif /* __PARSER_H__ */

/*****************************************************************************/
/* END OF FILE parser.h                                                      */
/*****************************************************************************/

