/*****************************************************************************/
/* readSDL.c                                                                 */
/*****************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "readSDL.h"
#include "readlib.h"
#include "ref.h"
#include "list.h"
#include "sdlform.h"
#include "strbuf.h"
#include "mtupel.h"

/*****************************************************************************/

void ReadBPNRefObject(FILE *infile,char ch,int *index,char **name,char **ref)
{
	static char buffer[1024];
	char *c = buffer;
	*c = ch;

	*index = 0;
	*name = buffer;
	*ref = NULL;
	
	if (isdigit(*c))
	{
		ungetc(*c,infile);
		*index=ReadNumber(infile);
		*name = NULL;
	}
	else
	{
		while (isalnum(*++c = getc(infile)));
		*c++ = '\0';
	}

	if (ReadCharComment(infile) != 'T')
	  PEPerror("'T' expected");
	ReadCharComment(infile);

	*ref = c;
	while ((*c++ = getc(infile)) != '"' && !feof(infile));
	*--c = '\0';

	if (ReadCharComment(infile) != '\n')
	  PEPerror("garbage characters after reference");
}

/*****************************************************************************/

void parse_ref(Place pl, char *ref)
{
	MTupel value;
	T_defitem def;
	T_list list = alloc_list();
	int bits,block;

	char *c = strchr(ref,';');
	if (!c)
	{
	  fprintf(stderr,"Parse error in SDL references: ';' expected\n");
	  exit(2);
	}
	*c++ = '\0';
	value = read_MTupel(&c);

	c = ref;
	while (*c)
	{
		bits = 0;
		block = read_posint(&c);
		if (*c == ':' && c++) bits = read_posint(&c);

		for_list (sdl_defitems,def,T_defitem)
		    if (def->nr == block)
		    {
			T_plref p = alloc_plref();

			p->place = pl;
			p->nr = block;
			p->bits = bits;
			p->value = value;
			list_add(def->list,p);
			list_add(list,p);
		    }

		if (*c && *c != '*')
		{
			fprintf(stderr,"Parse error in SDL references:"
					" '*' expected\n");
			exit(2);
		}
		if (*c) c++;
	}
	pl->component = (Cardinal32) list;
}

/*****************************************************************************/
/***************************** read_SDLref ***********************************/
/*****************************************************************************/

T_list read_set(char *string)
{
	T_list set = alloc_list();
	char ch, *c;
	int i, i1, i2;

	#define LBRACE '{'
	#define RBRACE '}'

	if (*string++ != LBRACE)
	  PEPerror("error while reading set");
	for (;;)
	{
		c = string;
		if (*c == 't' && *(c+1) == 't')
		{
			i1 = TT;
			ch = *(c += 2);
			*c = 0;
		}
		else if (*c == 'f' && *(c+1) == 'f')
		{
			i1 = FF;
			ch = *(c += 2);
			*c = 0;
		}
		else
		{
			while (isdigit(*c)) c++;
			ch = *c;
			*c = 0;
			i1 = atoi(string);
		}

		if (ch == ',')
		{
			list_add(set,i1);
			string = c+1;
			continue;
		}
		else if (ch == '.')
		{
			string = c+2;
			for (c = string; isdigit(*c); c++);
			ch = *c;
			*c = 0;
			i2 = atoi(string);
			for (i = i1; i <= i2; i++) list_add(set,i);

			if (ch == ',') 
			{
				string = c+1;
				continue;
			}
			else if (ch == RBRACE)
				return set;	
			else
			  PEPerror("error while reading set");
		}
		else if (ch == RBRACE)
		{
			list_add(set,i1);
			return set;
		}
		else
		  PEPerror("error while reading set");
	}
}

char ref_type_chars[] = REF_TYPE_CHARS;

void read_SDLref(LMPNet net, char *PEPfilename)
{
	register char	ch;
	register FILE	*infile;
	Place		pl;
	T_defitem	def, pdef = NULL;
	T_list		idset;
	char		*c, *cmd,
			*name, *ref;
	int		ident;
	int		size, type, proc, ver;
	int		i, refread;

	NNinput_file = PEPfilename;
	NNinput_line = 1;

	/* Oeffne PEP-Datei */
	if (!(infile = fopen(PEPfilename, "r")))
	{
		fprintf(stderr, "ERROR: read_SDLref(): could not open file "
				"'%s' for reading.\n", PEPfilename);
		exit(2);
	} /* if */

	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "PEP"))
		PEPerror("keyword 'PEP' expected");

	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strncmp(cmd, "References_SDL_LL",17))
		PEPerror("format identifier 'References_SDL_LL' expected");

	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strncmp(cmd, "FORMAT_N", 8))
		PEPerror("keyword 'FORMAT_N' or 'FORMAT_N2' expected");

	/* Read in DEF settings */
	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "DEF")) PEPerror("keyword 'DEF' expected");
	ReadNewline(infile);

	sdl_defitems = alloc_list();
	sdl_pidset = alloc_list();
	sdl_variables = alloc_list();
	list_add(sdl_pidset,DOT);
	list_add(sdl_pidset,0);
	sdl_system = alloc_ident();
	sdl_system->ptr = alloc_defitem();
	sdl_system->value = 1;
	sdl_system->var = NULL;
	list_add(sdl_system->ptr->set = alloc_list(), 0);

	while ((ch = getc(infile)) != 'P')
	{
	    ungetc(ch,infile);
	    size = type = ident = proc = refread = ver = 0;
	    idset = NULL;
	    name = NULL;
	
	    while ((ch = ReadCharComment(infile)) != '\n')
	    {
		if (isdigit(ch))
		{
			ident = ch - '0';
			while (isdigit(ch = getc(infile)))
			{
				ident *= 10;
				ident += ch - '0';
			}
			ungetc(ch,infile);
		}
		else
		{
		    if (refread) for (c = ref_type_chars; *c; c++)
			if (ch == *c) type = c-ref_type_chars;

		    switch(ch)
		    {
			case 'R':
				refread++;
			case 'C':
			case 'X':
				cmd=RL_ReadEnclString(infile);
				break;
			case 'b':
				cmd=RL_ReadEnclString(infile);
				name = MYstrdup(cmd);
				break;
			case 'u':
			case 'c':
			case 'v':
				cmd=RL_ReadEnclString(infile);
				((ch == 'u')? proc : (ch == 'c')? size : ver)
					= atoi(cmd);
				break;
			case 'P':
			case 'U':
			case 'V':
			case 'I':
			case 'O':
				cmd=RL_ReadEnclString(infile);
				idset = (*cmd == 'P')? sdl_pidset
						      : read_set(cmd);
				break;
			default:
			        for (c = ref_type_chars; *c; c++)
					if (ch == *c) break;
				if (ch != *c)
					PEPerror("unknown token in definition");
		    }
		}
	    }
	
	    list_add(sdl_defitems,(def = alloc_defitem()));
	    def->type = type;
	    def->nr = ident;
	    def->name = name;
	    def->list = alloc_list();

	    switch (type)
	    {
		case REF_CHANNEL:
			def->size = size + ver;
			break;
		case REF_SIGNAL:
			break;
		case REF_VARIABLE:
		case REF_IN_PARM:
		case REF_IN_OUT_PARM:
			for_list (sdl_defitems,pdef,T_defitem)
				if (pdef->nr == proc) break;
			if (type == REF_IN_PARM) def->type = REF_VARIABLE;
			def->proc = pdef;
			def->set = idset;
			break;
		case REF_PROCESS:
			def->size = size + ver/2;
			def->set = idset;
			for_list (idset,i,int) list_add(sdl_pidset,i);
			break;
		case REF_PROCEDURE:
			def->set = idset;
			break;
		case REF_SYSTEM:
		case REF_BLOCK:
		case REF_SIGNALROUTE:
		case REF_CONNECT:
		case REF_ACTION:
		case REF_LABEL:
		case REF_ANSWER:
		case REF_TRIGGER:
			free_list(def->list);
			free(def);
			list_remove_tail(sdl_defitems);
			break;
		case REF_SYNTYPE:
		default:
			fprintf(stderr,"unknown reference type in file %s, "
				 "line %u\n", NNinput_file, NNinput_line);
			exit(2);
	    }
	    NNinput_line++;
	}

	ungetc(ch,infile);
	cmd=ReadCmdToken(infile);

	/* Read in place nodes ***********************************************/
	if (strcmp(cmd, "PL")) PEPerror("keyword 'PL' expected");
	ReadNewline(infile);
	pl = net->Places;
	while ((ch = ReadCharComment(infile)) && (ch != 'T'))
	{
		ReadBPNRefObject(infile,ch,&ident,&name,&ref);
		if (!pl || (ident && ident != pl->numname) ||
		    (name && strcmp(name,pl->name)))
		{
			for (pl = net->Places; pl; pl = pl->nextplace)
				if ((ident && ident == pl->numname) ||
				    (name && !strcmp(name,pl->name)))
					break;
			if (!pl)
				PEPerror("invalid place identifier");
		}

		if (ref) parse_ref(pl,ref);
		pl = pl->nextplace;
		if (!pl) pl = net->Places;
		NNinput_line++;
	}

	/* Der Rest interessiert hier nicht. */

	fclose(infile);

} /* read_SDLref() */

/*****************************************************************************/

void read_SDLinfo(char *PEPfilename)
{
	register char	ch;
	register FILE	*infile;
	T_list		list = NULL;
	T_key		key;
	char		c[2], *cmd;
	int		ident;

	NNinput_file = PEPfilename;
	NNinput_line = 1;

	/* Oeffne PEP-Datei */
	if (!(infile = fopen(PEPfilename, "r")))
	{
		fprintf(stderr, "ERROR: read_SDLinfo(): could not open file "
				"'%s' for reading.\n", PEPfilename);
		exit(2);
	} /* if */

	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "PEP"))
		PEPerror("keyword 'PEP' expected");

	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strncmp(cmd, "SDL_Information",15))
		PEPerror("format identifier 'SDL_Information' expected");

	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strncmp(cmd, "FORMAT_N", 8))
		PEPerror("keyword 'FORMAT_N' or 'FORMAT_N2' expected");

	ReadNewline(infile);
	sdl_states = alloc_list();
	sdl_pidlist = alloc_list();
	sdl_signals = alloc_list();
	
	key = alloc_key();
	key->name = "System";
	key->value = 0;
	list_add(sdl_pidlist,key);

	c[1] = 0;

	for (;;)
	{
		initbuf();
		for (;;)
		{
			ch = getc(infile);
			if (feof(infile) || ch == ' ' || ch == '\n') break;
			*c = ch;
			addbuf(c);
		}
		if (feof(infile)) break;
		if (ch == '\n')
		{
			if (!strcmp(strbuf,"STATE")) list = sdl_states;
			else if (!strcmp(strbuf,"PID")) list = sdl_pidlist;
			else if (!strcmp(strbuf,"SIGNAL")) list = sdl_signals;
			else PEPerror("unknown keyword");
		} else if (list)
		{
			if (ReadWhiteSpace(infile) != '=')
				PEPerror("'=' expected");
			ident=ReadNumber(infile);

			key = alloc_key();
			key->name = MYstrdup(strbuf);
			key->value = ident;
			list_add(list,key);

			ReadNewline(infile);
		}
		else ReadNewline(infile);
	}
	fclose(infile);
}

