/*****************************************************************************/
/*  scanner.c                                                                */
/*****************************************************************************/
/*  C source file: Scanner for the formula translator.                       */
/*****************************************************************************/
/*                            Stefan Roemer,  Mo.,25.07.1994-Mo.,01.08.1994  */
/*****************************************************************************/

%option noyywrap
%option nounput

%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "scanner.h"
#include "netarray.h"
#include "transform.h"
#include "sdlform.h"
#include "formula.h"
#include "output.h"

/*****************************************************************************/

Cardinal32        FT_line = 1,
                  FT_column = 0,
                  FT_spaceLen;
char              * FT_space,
                  * FT_spaceMax,
                  FT_msg[200];

int		mclass;
char              name[ NAMELENGTH + 1 ];

static int find_Macro( char * name, Macro * macroptr, bool errorMessages );

/****** The space preserving mode did not work, and I did not fix it yet
 ****** but these parts may help to activate it...

#define PutSpace( CH ) if ( (int) ptr > (int) FT_spaceMax ) \
                         { \
                           register int offset = (int) ( ptr - FT_space ); \
                           FT_spaceMax = FT_space \
                                     = (char *) Realloc( (char *) FT_space, \
                                          ( FT_spaceLen +=  SPACE_STR_LEN ) \
                                                        * sizeof( char ) ); \
                           FT_spaceMax += FT_spaceLen; \
                           ptr = FT_space + offset; \
                         } \
                       *( ptr++ ) = (CH);

[\n\r]		{resize(FT_space, yyleng);
		 strcat(FT_space, yytext);
		 FT_line++;
		 FT_column = 0;
		}
* handle comments *
(%.*)		{resize(FT_space, yyleng+1);
		 strcat(FT_space, yytext);
		 strcat(FT_space, "\n");
		}
* store whitespace *
[[:blank:]]+	{resize(FT_space, yyleng);
		 strcat(FT_space, yytext);
		 FT_column+=yyleng;
		}

*****************/

%}

/* Single character operators like D are never followed by one of the
   following signs */
SC_OP_DELIM	[^[:alnum:]_]

%x DOUBLEQUOTE QUOTE

/*****************************************************************************/
/* Der eigentliche Scanner.                                                  */
/*****************************************************************************/

%%

[\n\r]		{FT_line++;
		 FT_column = 0;
		}
(%.*)		{FT_column+=yyleng;
		}
[[:blank:]]+	{FT_column+=yyleng;
		}

^\.\n		{FT_line++;
		 FT_column = 0;
		 return FORM_DELIM;
		}
^\.		{FT_column++;
		 return YYDOT;
		}
0|([1-9][0-9]*)	{FT_column+=yyleng;
		 yylval.number=atoi(yytext);
		 return NUMBER;
		}
true  		{FT_column+=yyleng;
		 return TRUE;
		}
false 		{FT_column+=yyleng;
		 return FALSE;
		}
#		{FT_column++;
		 return ( BOX );}
\^		{FT_column++;
		 return ( DIAMOND );}
\*		{FT_column++;
		 return ( AND );}
\+		{FT_column++;
		 return ( OR );}
[\-=]>		{FT_column+=2;
		 return IMPLICATION;}
-		{FT_column++;
		 return NOT;}
!		{FT_column++;
		 return ( sym_not == '!'? NOT : FORALL );}
\?		{FT_column++;
		 return ( EXISTS );}
\.		{FT_column++;
		 return ( YYDOT );}
=		{FT_column++;
		 return '=';}
\<[=\-]>		{FT_column+=3;
                 return EQUIVALENCE;}
\<=		{FT_column+=2;
		 return LE;}
\<>		{FT_column+=2;
		 return LTL_EVENTUALLY;}
\<		{FT_column++;
		 return '<';}
>=		{FT_column+=2;
		 return GE;}
>		{FT_column++;
		 return '>';}
&&?		{FT_column+=yyleng;
		 return AND;}
\|\|?		{FT_column+=yyleng;
		 return OR;}
\[\]		{FT_column+=2;
		 return LTL_ALWAYS;}
\[		{FT_column++;
		 return '[';}
\/\\		{FT_column+=2;
		 return AND;}
\/		{FT_column++;
		 return '/';}
\\\/		{FT_column+=2;
		 return OR;}
\\		{FT_column++;
		 return '\\';}

[_[:alpha:]]([_[:alnum:]])*	{FT_column+=yyleng;
      if ((yylval.place = find_place( yytext, FT_PlArray, FT_net )))
      {
        return ( PLACE );
      }
      else if ((yylval.transition = find_transition( yytext, FT_TrArray, FT_net )))
      {
        return ( TRANSITION );
      }
      else if ((mclass = find_Macro( yytext, &(yylval.macro), False )))
      {
        return ( mclass );
      }
      else if ((mclass = sdl_find_keyword(yytext,&yylval)))
      {
        return mclass;
      }
      else if ((mclass = sdl_make_variable(yytext,&yylval)))
      {
        return mclass;
      }
      else
      {
        sprintf( FT_msg, "unknown identifier: '%s'", yytext);
        yyerror( FT_msg );
      }
		}

\$[_[:alpha:]][_[:alnum:]]*	{FT_column+=yyleng;
		return ( find_Macro( yytext,
                            &( yylval.macro ), True ) );}


\"		{FT_column++;
		 BEGIN(DOUBLEQUOTE);}
\'		{FT_column++;
		 BEGIN(QUOTE);}
<QUOTE,DOUBLEQUOTE>{
	[[:blank:]]+	{FT_column+=yyleng;} /* skip blanks, but keep on counting */
	[\n\r]		{FT_line++;
		 FT_column = 0;}
}
<QUOTE>{
	[^'[:space:]]*	{strncpy(name,yytext,NAMELENGTH);FT_column+=yyleng;}
	\'	{BEGIN(INITIAL); FT_column++;
                 if ( CM_countPMeanings
                      && ( yylval.set = (SET) find_extPLabel( name, FT_PlMArray ) ) )
                 {
                   return ( EXTPLABEL );
                 }
                 else
                 {
                   sprintf( FT_msg,
                   "extended place labelling '%s' not used in the net", name );
                   yyerror( FT_msg );
                 }}
}
<DOUBLEQUOTE>{
	[^"[:space:]]*	{strncpy(name,yytext,NAMELENGTH);FT_column+=yyleng;}
	\"	{BEGIN(INITIAL); FT_column++;
                 if ( CM_countPMeanings
                      && ( yylval.set = (SET) find_extPLabel( name, FT_PlMArray ) ) )
                 {
                   return ( EXTPLABEL );
                 }
                 else
                 {
                   sprintf( FT_msg,
                   "extended place labelling '%s' not used in the net", name );
                   yyerror( FT_msg );
                 }}
}

B/{SC_OP_DELIM}		{FT_column++;
		 return ( BOX );}
D/{SC_OP_DELIM}		{FT_column++;
		 return ( DIAMOND );}
F/{SC_OP_DELIM}		{FT_column++;
		 return ( FALSE );}
N/{SC_OP_DELIM}		{FT_column++;
		 return ( NOT );}
S/{SC_OP_DELIM}		{FT_column++;
		 return ( ALL_PLACES );}
T/{SC_OP_DELIM}		{FT_column++;
		 return ( 'T' );}
n/{SC_OP_DELIM}		{FT_column++;
		 return ( INTERSECTION );}
u/{SC_OP_DELIM}		{FT_column++;
		 return ( UNION );}
U/{SC_OP_DELIM}		{FT_column++;
		 return ( LTL_UNTIL );}
V/{SC_OP_DELIM}		{FT_column++;
		 return ( LTL_DUAL );}
.		{FT_column++;
		 return ( yytext[0] );}

%%

/*****************************************************************************/
/* Fehlerbehandlungsroutine, die von dem von "yacc" erzeugten Parser benoe-  */
/* tigt wird; diese wird bei Fehlern, die waehrend des Parsens des Ausdrucks */
/* auftreten, aufgerufen und gibt die Fehlermeldung `msg` auf dem Bildschirm */
/* aus.                                                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

int yyerror( char * msg )

#else

int yyerror( msg )

  char * msg;

#endif

  {
    fprintf( stderr, "Line %u, column %u: %s.\n", FT_line, FT_column, msg );
    exit ( 123 );
  } /* yyerror() */


/*****************************************************************************/
/* Search for a macro name in the macro table.                               */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static int find_Macro( char * name, Macro * macroptr, bool errorMessages )

#else

static int find_Macro( name, macroptr, errorMessages )

  char  * name;
  Macro * macroptr;
  bool  errorMessages;

#endif

  {

    register char       * ptr,
                        * mptr,
                        * upname;
    register MacroTable tab;

    upname = Strdup( name );
    ptr = upname - 1;
    while ((*ptr = toupper( *++ptr )))
      skip;
    if ( *upname )
      {
        tab = FT_MacroTable - 1;
        while ((mptr = (++tab)->name))
          {
            ptr = upname;
            while ( *ptr && ( *ptr == *mptr ) )
              {
                ptr++;
                mptr++;
              } /* while */
            if ( ! isupper( *mptr ) )
              {
                while ( *ptr && ( *ptr == toupper( *mptr ) ) )
                  {
                    ptr++;
                    mptr++;
                  } /* while */
                if ( *ptr && errorMessages )
                  {
                    sprintf( FT_msg, "misspelled macro identifier: '%s'",
                                                                       name );
                    yyerror( FT_msg );
                  } /* if */
                else if ( *ptr )
                  {
                    return ( 0 );
                  } /* else if */
                else
                  {
                    *macroptr = *ptr ? NULL : tab->function;
                    return ( tab->mclass );
                  } /* else */
              } /* if */
          } /* while */
      } /* if */
    else if ( errorMessages )
      {
        yyerror( "missing macro identifier" );
      } /* else if */
    
    *macroptr = NULL;
    Free( upname );
    return ( 0 );

  } /* find_Macro() */


/*****************************************************************************/
/* END OF FILE scanner.c                                                     */
/*****************************************************************************/

