#include <string.h>
#include "sdlform.h"
#include "formula.h"
#include "readSDL.h"
#include "ref.h"
#include "list.h"
#include "scanner.h"
#include "transform.h"
#include "strbuf.h"

/*****************************************************************************/

T_list sdl_defitems = NULL;
T_list sdl_variables = NULL;
T_list sdl_pidset = NULL;
T_ident sdl_system = NULL;
T_list sdl_states = NULL;
T_list sdl_pidlist = NULL;
T_list sdl_signals = NULL;

/*****************************************************************************/

char *reserved[]
	= {	"FORALL", "EXISTS",
		"PID", "IDSET", "VALUE", "QUEUEPOS",
		"noqueue", "noinstances", "queue",
		"parent", "offspring", "sender",
		"state", "signal", "receiver",
		"caller",
		"NULL", "SYSTEM",
		"TRUE", "FALSE",
		"INTERNAL", "LEAVING",
		"EMPTY"
	  };

int  _reserved[]
	= {	SDL_FORALL, SDL_EXISTS,
		SDL_ALLPIDS, SDL_IDSET, SDL_VARSET, SDL_QUEUEPOS,
		SDLP_NOQUEUE, SDLP_NOINSTANCES, SDLP_QUEUE,
		SDLP_PARENT, SDLP_OFFSPRING, SDLP_SENDER,
		SDLP_STATE, SDLP_SIGNAL, SDLP_RECEIVER,
		SDLP_CALLER,
		SDL_NULL, SDL_SYSTEM,
		TRUE, FALSE,
		SDL_INTERNAL, SDL_LEAVING,
		SDL_EMPTY
	  };

/*****************************************************************************/

int sdl_find_keyword(char *yytext, void *ptr)
{
	YYSTYPE *yylval = (YYSTYPE*) ptr;
	T_variable var;
	T_defitem def;
	int i;

	if (!sdl_defitems) return 0;

	for (i = 0; i < sizeof(reserved)/sizeof(*reserved); i++)
		if (!strcmp(yytext,reserved[i]))
			return _reserved[i];

	if (!strcmp(yytext,"dot")) { yylval->number = DOT; return NUMBER; }
	if (!strcmp(yytext,"tt")) { yylval->number = TT; return NUMBER; }
	if (!strcmp(yytext,"ff")) { yylval->number = FF; return NUMBER; }

	for_list (sdl_variables,var,T_variable)
		if (!strcmp(yytext,var->name))
		{
			yylval->var = var;
			return SDL_VARIABLE;
		}

	for_list (sdl_defitems,def,T_defitem)
		if (!strcmp(yytext,def->name))
		{
			yylval->def = def;
			switch (def->type)
			{
			    case REF_CHANNEL:
				return SDL_CHANNEL_IDENT;
			    case REF_VARIABLE:
			    case REF_IN_OUT_PARM:
				return SDL_VAR_IDENT;
			    case REF_PROCESS:
				return SDL_PRCS_IDENT;
			    case REF_PROCEDURE:
				return SDL_PRCD_IDENT;
			    default:
				continue;
			}
		}
		
	return 0;
}

int sdl_make_variable(char *yytext, void *ptr)
{
	YYSTYPE *yylval = (YYSTYPE*) ptr;
	if (!sdl_variables) return 0;
	yylval->var = alloc_variable();
	yylval->var->name = MYstrdup(yytext);
	yylval->var->set = NULL;
	return SDL_VARIABLE;
}

T_defitem sdl_search_var_def(T_defitem proc, char *varname)
{
	T_defitem def;

	for_list (sdl_defitems,def,T_defitem)
		if (!strcmp(def->name,varname) && def->proc == proc)
			return def;
	
	yyerror("not a variable");
	return NULL;	/* can't happen */
}

T_ident sdl_search_var_ident(T_ident proc, char *varname)
{
	T_defitem def;
	T_ident newident;

	for_list (sdl_defitems,def,T_defitem)
		if (!strcmp(def->name,varname) && def->proc == proc->ptr)
		{
			newident = alloc_ident();
			newident->ptr = def;
			newident->var = proc->var;
			newident->value = proc->value;
			return newident;
		}
	
	yyerror("not a variable");
	return NULL;	/* can't happen */
}

/************ Hilfsroutinen ************/

int get_val(T_ident ident)
{
	return ident->var? ident->var->value : ident->value;
}

int check_pid(T_ident ident)
{
	int pid = get_val(ident);
	if (ident->var) pid -= list_elem(ident->ptr->set,0,int)-1;
	if (pid < 0 || pid > ident->ptr->set->count)
		yyerror("invalid process id");
	return pid + list_elem(ident->ptr->set,0,int) - 1;
}

int check_op(int op, int value, int cmp)
{
	switch (op)
	{
	    case '=': return value == cmp;
	    case '#': return value != cmp;
	    case '<': return value < cmp;
	    case LE: return value <= cmp;
	    case '>': return value > cmp;
	    case GE: return value >= cmp;
	    case 0: return value == cmp;
	    case 1: return value != cmp;
	}
	return 0; /* can't happen */
}

/************ find_key ************/

T_key find_key(T_list list, char *name)
{
	/* fixme: lineare Suche vermeiden */
	T_key key;
	for_list (list,key,T_key)
		if (!strcmp(name,key->name)) return key;
	yyerror("unknown identifier");
	return NULL;	/* can't happen */
}

/************ Routinen fuer Inout-Parameter ************/

#define PlaceNode(pl) NewNode(PLACE,NULL,(BTree) pl)
#define dim(x) (MT_Place(ref->value,x))

static BTree ionode;

void gather_places(T_defitem def, int proc, int pid,
		   T_list list, int op, int val)
{
	T_plref ref;
	int dimen = 1;
	BTree node = NULL;
	Place pl;
	
	if (def->proc->type == REF_PROCESS) { dimen = 2; proc = pid; }

	for_list (def->list,ref,T_plref)
		if (ref->bits == 4 && dim(dimen) == proc
				&& check_op(op,dim(0),val))
			node = node? NewNode(OR,node,PlaceNode(ref->place))
				   : PlaceNode(ref->place);

	if (!node) return;
	node = NewNode('(',NULL,node);
	for_list (list,pl,Place) node = NewNode(AND,PlaceNode(pl),node);
	node = NewNode('(',NULL,node);
	ionode = ionode? NewNode(OR,ionode,node) : node;
}

void trace_parm(Place pl, int op, int val, int prcsid, T_list list)
{
	T_plref ref;
	T_defitem def, pdef = NULL;
	T_list plrefs = (T_list) pl->component;
	int refnr = 0, pid, procnr = 0;

	for_list (plrefs,ref,T_plref)
	{
		if (ref->bits == 32) refnr = ref->nr;
		if (ref->bits == 32768) procnr = ref->nr;
	}

	ref = list_elem(plrefs,0,T_plref);
	pid = dim(1);	/* in/out-Parameter kommen nur in Prozeduren vor */
	if (prcsid != DOT && dim(2) != prcsid) return;
	prcsid = dim(2);

	if (pid != DOT)
	    for_list (sdl_defitems,pdef,T_defitem)
		if (pdef->nr == procnr)
		{
			for_list (pdef->used,procnr,int)
				if (procnr == pid) return;
			list_add(pdef->used,pid);
			break;
		}

	list_add(list,pl);

	for_list (sdl_defitems,def,T_defitem)
		if (def->nr == refnr)
		{
			if (def->type == REF_VARIABLE)
				gather_places(def,dim(1),dim(2),list,op,val);
			else
			    for_list (def->list,ref,T_plref)
				if (ref->bits == 0 && dim(0) == pid)
				{
				    int i = list_pos(def->list);
				    trace_parm(ref->place,op,val,prcsid,list);
				    list_set_pos(def->list,i);
				}
			break;
		}

	if (pdef) list_remove_tail(pdef->used);
	list_remove_tail(list);
}

BTree trace_inout_parm(T_defitem var, int pid, int op, int val)
{
	T_plref ref;
	T_list newlist;
	T_defitem def;

	ionode = NULL;

	for_list (sdl_defitems,def,T_defitem)
		if (def->type == REF_PROCEDURE) def->used = alloc_list();
	list_add(var->proc->used,pid);

	for_list (var->list,ref,T_plref)
		if (ref->bits == 0 && dim(0) == pid)
		{
			void **save = var->list->run;
			trace_parm(ref->place,op,val,DOT,
					newlist = alloc_list());
			var->list->run = save;
			free_list(newlist);
		}

	for_list (sdl_defitems,def,T_defitem)
		if (def->type == REF_PROCEDURE) free_list(def->used);

	return ionode? NewNode('(',NULL,ionode) : NewNode(FALSE,NULL,NULL);
}

/************ Interpretieren von Signal-Ausdruecken ************/

int find_sigval(T_sigident sig)
{
	T_ident ident;
	char number[16];
	int val;

	if (!sig->name) return DOT;

	initbuf();
	addbuf(sig->name);
	if (!sig->args) return find_key(sdl_signals,strbuf)->value;

	addbuf("(");
	for_list (sig->args,ident,T_ident)
	{
	    if (!ident)
		addbuf("Null");
	    else if (ident->ptr)
		addbuf(list_elem(sdl_pidlist,check_pid(ident),T_key)->name);
	    else
	    {
		val = get_val(ident);
		if (val == TT) addbuf("True");
		else if (val == FF) addbuf("False");
		else
		{
			sprintf(number,"%d",val);
			addbuf(number);
		}
	    }

	    if (list_pos(sig->args) < sig->args->count - 1) 
		addbuf(",");
	}
	addbuf(")");
	return find_key(sdl_signals,strbuf)->value;
}

int get_val_or_pid(BTree node)
{
	if (node->op <= 1)
		return node->right? check_pid((T_ident) node->right) : DOT;
	else
		return get_val((T_ident) node->right);
}

/************ Expandieren von SDL-Formeln ************/

BTree sdl_translate_property(BTree node)
{
	T_ident ident = (T_ident) node->left;
	T_list places = alloc_list();
	T_plref ref;
	T_defitem def;

	BTree newnode = NULL;
	Place pl;
	char *name;
	int op = node->right->op;
	int pid = 0, val, dimen, bits = 0;
	int channel, index;

	switch (node->op)
	{
	    case SDLP_NOQUEUE:
		channel = (ident->ptr->type == REF_CHANNEL);
		if (!channel) pid = check_pid(ident);
		val = get_val((T_ident) node->right->right);
		for_list (ident->ptr->list,ref,T_plref)
		    if (ref->bits == 8192)
			if (channel || dim(1) == pid)
			    if (check_op(op,dim(0),val))
				list_add(places,ref->place);
		break;
	    case SDLP_NOINSTANCES:
		def = (T_defitem) node->left;
		val = get_val((T_ident) node->right->right);
		for_list (def->list,ref,T_plref)
		    if (ref->bits == 1 && check_op(op,dim(0),val))
			list_add(places,ref->place);
		break;
	    case SDLP_PARENT:
	    case SDLP_OFFSPRING:
	    case SDLP_SENDER:
		pid = check_pid(ident);
		val = get_val_or_pid(node->right);
		if (node->op == SDLP_PARENT) bits = 128;
		if (node->op == SDLP_OFFSPRING) bits = 256;
		if (node->op == SDLP_SENDER) bits = 512;
		for_list (ident->ptr->list,ref,T_plref)
		    if (ref->bits == bits &&
				dim(1) == pid && check_op(op,dim(0),val))
			list_add(places,ref->place);
		break;
	    case SDL_VARIABLE:
		pid = get_val(ident);
		if (ident->var) pid -= list_elem(ident->ptr->proc->set,0,int)-1;
		if (pid < 0 || pid > ident->ptr->proc->set->count)
			yyerror("invalid process/procedure id");
		pid += list_elem(ident->ptr->proc->set,0,int) - 1;

		if (ident->ptr->set == sdl_pidset)
			if (node->right->right)
				val = check_pid((T_ident) node->right->right);
			else
				val = DOT;
		else
			val = get_val((T_ident) node->right->right);

		if (ident->ptr->type == REF_IN_OUT_PARM)
		{
			free_list(places);
			return trace_inout_parm(ident->ptr,pid,op,val);
		}
		dimen = (ident->ptr->proc->type == REF_PROCESS)? 2 : 1;
		for_list (ident->ptr->list,ref,T_plref)
		    if (ref->bits == 4 &&
				dim(dimen) == pid && check_op(op,dim(0),val))
			list_add(places,ref->place);
		break;
	    case SDLP_STATE:
		pid = check_pid(ident);
		name = ((T_variable) node->right->right)->name;
		if (name)
		{
			initbuf();
			addbuf(ident->ptr->name);
			addbuf(".");
			addbuf(name);
			val = find_key(sdl_states,strbuf)->value;
		}
		else val = ((T_variable) node->right->right)->value;
		for_list (ident->ptr->list,ref,T_plref)
		    if (ref->bits == 1024 &&
				dim(1) == pid && check_op(op,dim(0),val))
			list_add(places,ref->place);
		break;
	    case SDLP_SIGNAL:
	    case SDLP_QUEUE:
	    case SDLP_RECEIVER:
		channel = (ident->ptr->type == REF_CHANNEL);
		dimen = (node->op == SDLP_QUEUE)? 2 : 3;
		if (!channel) pid = check_pid(ident);

		if (node->op == SDLP_SIGNAL)
		{
			val = find_sigval((T_sigident) node->right->right);
			dimen = 1;
		}
		else
			val = get_val_or_pid(node->right);

		index = get_val((T_ident) node->right->left);

		for_list (ident->ptr->list,ref,T_plref)
		    if (ref->bits == (channel? 4 : 16384))
			if (channel || dim(3) == pid)
			    if (dim(0) == index && check_op(op,dim(dimen),val))
				list_add(places,ref->place);
		break;
	    case SDLP_CALLER:
		pid = check_pid(ident);
		index = ident->ptr->nr;
		val = check_pid((T_ident)node->right->right);
		def = ((T_ident)node->right->right)->ptr;
		dimen = (def->type == REF_PROCEDURE)? 1 : 2;

		for_list (def->list,ref,T_plref)
		    if (ref->bits == 32768 &&
				dim(0) == pid && check_op(op,dim(dimen),val))
		    {
			T_list plrefs = (T_list) ref->place->component;
			T_plref ref2;

			for_list (plrefs,ref2,T_plref)
			    if (ref2->nr == index && ref2->bits == 64)
				list_add(places,ref->place);
		    }

		break;
	    default:
		break;
	}

	for_list (places,pl,Place)
		newnode = newnode? NewNode(OR,newnode,PlaceNode(pl))
				 : PlaceNode(pl);
		
	free_list(places);
	return newnode? NewNode('(',NULL,newnode) : NewNode(FALSE,NULL,NULL);
}

BTree sdl_condition(BTree node)
{
	int op = node->right->op;
	int val1 = get_val((T_ident) node->left);
	int val2 = get_val_or_pid(node->right);

	return NewNode(check_op(op,val1,val2)? TRUE : FALSE, NULL,NULL);
}

BTree sdl_quant_formula(BTree node)
{
	int i,j;
	BTree tree = NULL, newtree;
	T_variable var = (T_variable) node->left;

	for_list (var->set,i,int)
	{
		var->value = i;
		j = list_pos(var->set);
		newtree = NewNode('(',NULL,sdl_use_node(node->right));
		tree = tree? NewNode((node->op == SDL_FORALL)? AND : OR,
					tree, newtree) : newtree;
		list_set_pos(var->set,j);
	}
	return tree? NewNode('(',NULL,tree)
		   : NewNode((node->op == SDL_FORALL)? TRUE : FALSE, NULL,NULL);
}

BTree sdl_use_node(BTree node)
{
	BTree node1, node2;

	if (node->op == SDL_PROPERTY) return sdl_translate_property(node->left);
	if (node->op == SDL_TRUE) return sdl_condition(node);
	if (node->op == SDL_FORALL || node->op == SDL_EXISTS)
		return sdl_quant_formula(node);
	if (node->op == PLACE) return node;
	node1 = node->left? sdl_use_node(node->left) : NULL;
	node2 = node->right? sdl_use_node(node->right) : NULL;
	return NewNode(node->op,node1,node2);
}
