/*****************************************************************************/
/*  set.c                                                                    */
/*  C source file: operations on sets.                                       */
/*  Mo.,28.02.1994 - Mo.,01.08.1994                           Stefan Roemer  */
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "neunetze.h"
#include "transform.h"
#include "set.h"


/*****************************************************************************/
/* Global variables.                                                         */


Cardinal32 SET_numAllocations = 0;
SET        SET_freeList       = NULL;


/*****************************************************************************/
/* Allocate an element of the type 'SET'.                                    */

#if defined(__STDC__) || defined(__cplusplus)

SET allocSET( void )

#else

SET allocSET()

#endif

  {

    register SET newSet;

    if ((newSet = SET_freeList))
      {
        SET_freeList = newSet->next;
      } /* if */
    else if ((newSet = (SET) malloc( sizeof( SET_ELEM ) )))
      {
        SET_numAllocations++;
      } /* else if */
    else
      {
        fprintf( stderr, "ERROR: unfold.allocSET(): memory overflow.\n" );
        exit ( 3 );
      } /* else */

    return ( newSet );

  } /* allocSET() */


/*****************************************************************************/
/* Allocate an element of the type 'ASET'.                                   */

#if defined(__STDC__) || defined(__cplusplus)
ASET allocASET( void )
#else
ASET allocASET()
#endif
{
    register ASET newSet;

    newSet = (ASET) malloc(sizeof(ASET_ELEM));
    newSet->or = False;
    newSet->set = NULL;

    return (newSet);

} /* allocASET() */


/*****************************************************************************/
/* Delete a set ( giving the allocated memory free ).                        */



#if defined(__STDC__) || defined(__cplusplus)

void set_delete_mem( SET * set )

#else

void set_delete_mem( set )

  SET * set; /* call-by-reference ! */

#endif

  {

    register SET del = *set;

    while ( del )
      {
        del = ( *set = del )->next;
        Free( *set );
      } /* while */
    *set = NULL;

  } /* set_delete_mem() */


/*****************************************************************************/
/* Delete a set.                                                             */

#if defined(__STDC__) || defined(__cplusplus)

static void set_delete( SET * set )

#else

static void set_delete( set )

  SET * set; /* call-by-reference ! */

#endif

  {

    register SET li,
                 del;

    for ( li = *set ; li ; li = ( del = li )->next, freeSET( del ) )
      skip;
    *set = NULL;

  } /* set_delete() */


/*****************************************************************************/
/* Check the equality of two sets.                                           */

#if defined(__STDC__) || defined(__cplusplus)

bool sets_are_equal( SET set1, SET set2 )

#else
 
bool sets_are_equal( set1, set2 )

  SET set1;
  SET set2;
 
#endif

  {

    register SET s1,
                 s2;

    for ( s1 = set1, s2 = set2 ; s1 && s2 && ( s1->item == s2->item ) ;
                                                s1 = s1->next, s2 = s2->next )
      skip;

    return ( (bool) ! ( s1 || s2 ) );

  } /* sets_are_equal() */


/*****************************************************************************/
/* Compare two sets ( set1<set2 (-1), set1==set2 (0) or set1>set2 (1) )      */

#if defined(__STDC__) || defined(__cplusplus)

int set_compare( SET set1, SET set2 )

#else
 
int set_compare( set1, set2 )

  SET set1;
  SET set2;
 
#endif

  {

    register SET s1,
                 s2;

    for ( s1 = set1, s2 = set2 ; s1 && s2 && ( s1->item == s2->item ) ;
                                                s1 = s1->next, s2 = s2->next )
      skip;

    return ( ! ( s1 || s2 ) ? 0
                            : ! s1 || ( s2 && ( s1->item < s2->item ) ) ? -1
                                                                        : 1 );

  } /* set_compare() */


/*****************************************************************************/
/* Insert an element into a set.                                             */

#if defined(__STDC__) || defined(__cplusplus)

SET set_insert( SET * set, Place item )

#else

SET set_insert( set, item )

  SET   * set; /* call-by-reference !!! */
  Place item;

#endif

  {

    register SET        setptr,
                        newSet;
    register Cardinal32 numname = item->numname;

    if (    ( ! ( setptr = *set ) )
         || ( numname < setptr->item->numname ) )
      {
        ( newSet = allocSET() )->item = item;
        newSet->next = setptr;
        *set = newSet;
      } /* if */
    else if ( item != setptr->item )
      {
        while (    ( newSet = setptr->next )
                && ( newSet->item->numname < numname ) )
          {
            setptr = newSet;
          } /* while */
        if ( ! ( newSet && ( newSet->item == item ) ) )
          {
            ( newSet = allocSET() )->item = item;
            newSet->next = setptr->next;
            setptr->next = newSet;
          } /* if */
      } /* else if */

    return ( *set );

  } /* set_insert() */


/*****************************************************************************/
/* Compute the union of two sets.                                            */

#if defined(__STDC__) || defined(__cplusplus)

SET set_union( SET * set1, SET set2 )

#else

SET set_union( set1, set2 )

  SET  * set1, /* call-by-reference !!! */
       set2;

#endif

  {

    register SET s1,
                 s2,
                 s2del;

    if ((s2 = set2))
      {
        if ((s1 = *set1))
          {
            if ( s2->item->numname < s1->item->numname )
              {
                *set1 = s2;
                s2 = s1;
                s1 = *set1;
              } /* if */
            else if ( s1->item == s2->item )
              {
                s2 = ( s2del = s2 ) -> next;
                freeSET( s2del );
              } /* else if */
            while ( s1->next && s2 )
              {
                if ( s2->item->numname < s1->next->item->numname )
                  {
                    s2del = s1->next;
                    s1 = s1->next = s2;
                    s2 = s2del;
                  } /* if */
                else if ( s1->next->item < s2->item )
                  {
                    s1 = s1->next;
                  } /* else if */
                else
                  {
                    s2 = ( s2del = s2 ) -> next;
                    freeSET( s2del );
                  } /* else */
              } /* while */
            if ( s2 )
              {
                s1->next = s2;
              } /* if */
          } /* if */
        else
          {
            *set1 = s2;
          } /* else */
      } /* if */

    return ( *set1 );

  } /* set_union() */


/*****************************************************************************/
/* Compute the intersection of two sets.                                     */

#if defined(__STDC__) || defined(__cplusplus)

SET set_intersection( SET * set1, SET set2 )

#else

SET set_intersection( set1, set2 )

  SET  * set1, /* call-by-reference !!! */
       set2;

#endif

  {

    register SET s1,
                 s2,
                 s1del,
                 s2del;

    if ( ( s1 = *set1 ) && ( s2 = set2 ) )
      {
        do {
             while ( s2 && ( s2->item->numname < s1->item->numname ) )
               {
                 s2 = ( s2del = s2 )->next;
                 freeSET( s2del );
               } /* while */
             if ( s2 && ( s2->item == s1->item ) )
               {
                 s1del = NULL;
                 s2 = ( s2del = s2 )->next;
                 freeSET( s2del );
               } /* if */
             else
               {
                 s1 = ( s1del = s1 )->next;
                 freeSET( s1del );
               } /* else */
           } while ( s1del );
        s1 = ( *set1 = s1 )->next;
        while ( s1 && s2 )
          {
            while ( s2 && ( s2->item->numname < s1->item->numname ) )
              {
                s2 = ( s2del = s2 )->next;
                freeSET( s2del );
              } /* while */
            if ( s2 && ( s2->item == s1->item ) )
              {
                s1 = s1->next;
                s2 = ( s2del = s2 )->next;
                freeSET( s2del );
              } /* if */
            else
              {
                s1 = ( s1del = s1 )->next;
                freeSET( s1del );
              } /* else */
          } /* while */
        while ( s2 )
          {
            s2 = ( s2del = s2 )->next;
            freeSET( s2del );
          } /* while */
      } /* if */
    else
      {
        set_delete( set1 );
        set_delete( &set2 );
      } /* else */

    return ( *set1 );

  } /* set_intersection() */


/*****************************************************************************/
/* Compute the difference of two sets.                                       */

#if defined(__STDC__) || defined(__cplusplus)

SET set_difference( SET * set1, SET set2 )

#else

SET set_difference( set1, set2 )

  SET  * set1, /* call-by-reference !!! */
       set2;

#endif

  {

    register SET s1,
                 s2 = NULL,
                 s1del,
                 s2del;

    if ( ( s1 = *set1 ) && ( s2 = set2 ) )
      {
        do {
             while ( s2 && ( s2->item->numname < s1->item->numname ) )
               {
                 s2 = ( s2del = s2 )->next;
                 freeSET( s2del );
               } /* while */
             if ( s2 && ( s2->item == s1->item ) )
               {
                 s1 = ( s1del = s1 )->next;
                 freeSET( s1del );
               } /* if */
             else
               {
                 s1del = NULL;
               } /* else */
           } while ( s1del );
        s1 = ( *set1 = s1 )->next;
        while ( s1 && s2 )
          {
            while ( s2 && ( s2->item->numname < s1->item->numname ) )
              {
                s2 = ( s2del = s2 )->next;
                freeSET( s2del );
              } /* while */
            if ( s2 && ( s2->item == s1->item ) )
              {
                s1 = ( s1del = s1 )->next;
                freeSET( s1del );
              } /* if */
            else
              {
                s1 = s1->next;
              } /* else */
          } /* while */
        while ( s2 )
          {
            s2 = ( s2del = s2 )->next;
            freeSET( s2del );
          } /* while */
      } /* if */
    else if ( s2 )
      {
        set_delete( &set2 );
      } /* else */

    return ( *set1 );

  } /* set_difference() */


/*****************************************************************************/
/* Display a set.                                                            */

#if defined(__STDC__) || defined(__cplusplus)

void set_print( SET set )

#else

void set_print( set )

  SET set;

#endif

  {

    printf( "{" );
    if ( set )
      {
        printf( "%s", set->item->name );
        for ( skip ; (set = set->next); printf( ",%s", set->item->name ) )
          skip;
      } /* if */
    printf( "}\n" );

  } /* set_print() */


/****************************************************************************/
/* Calculate the pre-set of a set of places.                                */

#if defined(__STDC__) || defined(__cplusplus)

SET calcPpreset( SET p_set )

#else

SET calcPpreset( p_set )

  SET p_set;

#endif

  {

    register SET    set = p_set;
    register TSEdge ts;
    SET             t_set = NULL;

    while ( set )
      {
        ts = set->item->preset;
        while ( ts )
          {
            set_insert( &t_set, (Place) ts->trans );
            ts = ts->nexttrans;
          } /* while */
        set = set->next;
      } /* while */

    set_delete( &p_set );

    return ( t_set );

  } /* calcPpreset() */


/****************************************************************************/
/* Calculate the post-set of a set of places.                               */

#if defined(__STDC__) || defined(__cplusplus)

SET calcPpostset( SET p_set )

#else

SET calcPpostset( p_set )

  SET p_set;

#endif

  {

    register SET    set = p_set;
    register STEdge st;
    SET             t_set = NULL;

    while ( set )
      {
        st = set->item->postset;
        while ( st )
          {
            set_insert( &t_set, (Place) st->trans );
            st = st->nexttrans;
          } /* while */
        set = set->next;
      } /* while */

    set_delete( &p_set );

    return ( t_set );

  } /* calcPpostset() */


/****************************************************************************/
/* Calculate the pre-set of a set of transitions.                           */

#if defined(__STDC__) || defined(__cplusplus)

SET calcTpreset( SET t_set )

#else

SET calcTpreset( t_set )

  SET t_set;

#endif

  {

    register SET    set = t_set;
    register STEdge st;
    SET             p_set = NULL;

    while ( set )
      {
        st = ( (Transition) set->item )->preset;
        while ( st )
          {
            set_insert( &p_set, st->place );
            st = st->nextplace;
          } /* while */
        set = set->next;
      } /* while */

    set_delete( &t_set );

    return ( p_set );

  } /* calcTpreset() */


/****************************************************************************/
/* Calculate the post-set of a set of transitions.                          */

#if defined(__STDC__) || defined(__cplusplus)

SET calcTpostset( SET p_set )

#else

SET calcTpostset( p_set )

  SET p_set;

#endif

  {

    register SET    set = p_set;
    register TSEdge ts;
    SET             t_set = NULL;

    while ( set )
      {
        ts = ( (Transition) set->item )->postset;
        while ( ts )
          {
            set_insert( &t_set, ts->place );
            ts = ts->nextplace;
          } /* while */
        set = set->next;
      } /* while */

    set_delete( &p_set );

    return ( t_set );

  } /* calcTpostset() */


/*****************************************************************************/
/* END OF FILE set.c                                                         */
/*****************************************************************************/

