/*****************************************************************************/
/*  set.h                                                                    */
/*  C header file: operations on sets.                                       */
/*  Mo.,28.02.1994 - Mo.,01.08.1994                           Stefan Roemer  */
/*****************************************************************************/


#ifndef __SET_H__
#define __SET_H__

#include <stdio.h>
#include "neunetze.h"


/*****************************************************************************/
/* Data structure: set ( singly linked list ).                               */

typedef struct setelem_
  {
    Place           item;
    struct setelem_ * next;
  } SET_ELEM,
    * SET;

typedef struct asetelem_
{
	SET	set;
	int	or;
} ASET_ELEM, *ASET;

/*****************************************************************************/
/* External globals.                                                         */

extern Cardinal32 SET_numAllocations;
extern SET        SET_freeList;


/*****************************************************************************/
/* Allocate an element of the type 'SET'.                                    */

#if defined(__STDC__) || defined(__cplusplus)

extern SET allocSET( void );

#else

extern SET allocSET();

#endif


/*****************************************************************************/
/* Allocate an element of the type 'ASET'.                                   */

#if defined(__STDC__) || defined(__cplusplus)
extern ASET allocASET( void );
#else
extern ASET allocASET();
#endif


/*****************************************************************************/
/* Deallocate an element of the type 'SET'.                                  */

#define freeSET( SetElem ) (SetElem)->next = SET_freeList, \
                           SET_freeList = (SetElem)


/*****************************************************************************/
/* Delete a set ( giving the allocated memory free ).                        */

#if defined(__STDC__) || defined(__cplusplus)

extern void set_delete_mem( SET * );

#else

extern void set_delete_mem();

#endif


/*****************************************************************************/
/* Check the equality of two sets.                                           */

#if defined(__STDC__) || defined(__cplusplus)

extern bool sets_are_equal( SET, SET );

#else

extern bool sets_are_equal();

#endif


/*****************************************************************************/
/* Compare two sets ( set1<set2 (-1), set1==set2 (0) or set1>set2 (1) )      */
 
#if defined(__STDC__) || defined(__cplusplus)
 
extern int set_compare( SET, SET );
 
#else
 
extern int set_compare();
 
#endif


/*****************************************************************************/
/* Insert an element into a set.                                             */

#if defined(__STDC__) || defined(__cplusplus)
 
extern SET set_insert( SET *, Place );
 
#else
 
extern SET set_insert();
 
#endif


/*****************************************************************************/
/* Compute the union of two sets.                                            */

#if defined(__STDC__) || defined(__cplusplus)

extern SET set_union( SET *, SET );

#else

extern SET set_union();

#endif


/*****************************************************************************/
/* Compute the intersection of two sets.                                     */

#if defined(__STDC__) || defined(__cplusplus)

extern SET set_intersection( SET *, SET );

#else

extern SET set_intersection();

#endif


/*****************************************************************************/
/* Compute the difference of two sets.                                       */

#if defined(__STDC__) || defined(__cplusplus)

extern SET set_difference( SET *, SET );

#else

extern SET set_difference();

#endif


/*****************************************************************************/
/* Display a set.                                                            */

#if defined(__STDC__) || defined(__cplusplus)

extern void set_print( SET set );

#else

extern void set_print();

#endif


/*****************************************************************************/
/* Calculate the pre-set of a set of places.                                 */

#if defined(__STDC__) || defined(__cplusplus)

extern SET calcPpreset( SET );

#else

extern SET calcPpreset();

#endif


/*****************************************************************************/
/* Calculate the post-set of a set of places.                                */
 
#if defined(__STDC__) || defined(__cplusplus)
 
extern SET calcPpostset( SET );
 
#else
 
extern SET calcPpostset();
 
#endif
 

/*****************************************************************************/
/* Calculate the pre-set of a set of transitions.                            */
 
#if defined(__STDC__) || defined(__cplusplus)
 
extern SET calcTpreset( SET );
 
#else
 
extern SET calcTpreset();
 
#endif


/*****************************************************************************/
/* Calculate the post-set of a set of transitions.                           */

#if defined(__STDC__) || defined(__cplusplus)

extern SET calcTpostset( SET );

#else

extern SET calcTpostset();

#endif


/*****************************************************************************/


#endif /* __SET_H__ */


/*****************************************************************************/
/* END OF FILE set.h                                                         */
/*****************************************************************************/

