/*****************************************************************************/
/* transform.c                                                               */
/*****************************************************************************/
/* Formula translator for the model checker.                                 */
/*****************************************************************************/
/*                                    Stefan Roemer,  28.02.1994-10.08.1994  */
/*                                    Stefan Roemer,  28.02.1994-08.11.1994  */
/* Mehrere Formeln/Datei verwalten    Stefan Roemer,  28.02.1994-13.09.1996  */
/* Syntax umstrukturiert              Stefan Schwoon, 14.04.1997	     */
/* SDL-Formeln                        Stefan Schwoon, Juni 1998 	     */
/* SDL-Formeln fuer LTL               Stefan Schwoon, Oktober 1998 	     */
/*****************************************************************************/
#define VERSION "2.9.1 -- L1&L2-MC"
#define DATE "21.10.1998"
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "transform.h"
#include "convertPEP.h"
#include "readlib.h"
#include "netarray.h"
#include "scanner.h"
#include "output.h"
#include "sdlform.h"
#include "formula.h"
#include "parser.h"

/*****************************************************************************/
/* Global variables.                                                         */
/*****************************************************************************/

LMPNet          FT_net = NULL;
PlMArray   FT_PlMArray;
Place      * FT_PlArray;
Transition * FT_TrArray;

/*****************************************************************************/
/* Main program of the formula translator.                                   */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

int main( int argc, char ** argv )

#else

int main( argc, argv )

  int  argc;
  char ** argv;

#endif

  {

    register Cardinal32 count;
    register Place      pl;
    register Transition tr;
    register char       ch,
                        * ptr;
    char                netFilename[NAMELENGTH],
                        inFilename[NAMELENGTH],
                        outFilename[NAMELENGTH],
                        sdlFilename[NAMELENGTH],
                        sdlInfofilename[NAMELENGTH];
    bool                showVersion = False,
                        showUsage   = False,
                        useLTL      = False;
    static char         null[2] = "\0";

    FT_OutStringMax = FT_OutString = (char *) Malloc(
                    ( FT_OutStringLen = OUTSTRING_LENGTH ) * sizeof( char ) );
    FT_OutStringMax += FT_OutStringLen;
    FT_spaceMax = FT_space
        = (char *) Malloc( ( FT_spaceLen = SPACE_STR_LEN ) * sizeof( char ) );
    FT_spaceMax += FT_spaceLen;
    Xinit();
    CM_placeMList = CM_transMList = NULL;
    CM_storeMeaning = True;
    *CM_nettype = '\0';
    CM_countMPlaces = CM_countMTransitions = 0;
    CM_countPMeanings = CM_countTMeanings = 0;
    set_PEP_keycodes();
    SET_freeList = NULL;

    FT_userlike = FT_pretty = False;

    *netFilename = *inFilename = *outFilename
	= *sdlFilename = *sdlInfofilename = '\0';

#ifdef YYDEBUG
{
	extern int yydebug;
	extern int yy_flex_debug;
	yydebug = yy_flex_debug = 1;
}
#endif
    if ( argc > 1 )
      {
        count = 0;
        while ( ++count < argc )
          {
            if ( ( ch = *( ptr = argv[count] ) ) == '-' )
              {
                while ((ch = tolower( *++ptr )))
                  {
                    switch ( ch )
                      {
                        case 'f' : if ( *++ptr != '=' )
                                     {
                                       fprintf( stderr,
                       "ERROR: transform.main(): '=' expected after '-f'.\n" );
                                       exit ( 11 );
                                     } /* if */
                                   ch = *++ptr;
                                   switch ( ch )
                                     {
                                       case 'h' : set_hartmut_keycodes();
                                                  break;
                                       case 'b' :
                                       case 'p' : set_PEP_keycodes();
                                                  break;
                                       default  : fprintf( stderr,
          "ERROR: transform.main(): 'b', 'h' or 'p' expected after '-f='.\n" );
                                       exit ( 11 );
                                     } /* switch */
                                   break;
                        case 'i' : if ( *++ptr != '=' )
                                     {
                                       fprintf( stderr,
                       "ERROR: transform.main(): '=' expected after '-i'.\n" );
                                       exit ( 11 );
                                     } /* if */
                                   if ( *inFilename )
                                     {
                                       fprintf( stderr,
                  "WARNING: transform.main(): parameter '-i' appears twice;" );
                                       fprintf( stderr,
                                                 " %*s'%s' overwrites '%s'.\n",
                                                 24, "", ptr + 1, inFilename );
                                     }
                                   strcpy( inFilename, ++ptr );
                                   ptr = null;
                                   break;
                        case 'n' : if ( *++ptr != '=' )
                                     {
                                       fprintf( stderr,
                       "ERROR: transform.main(): '=' expected after '-n'.\n" );
                                       exit ( 11 );
                                     } /* if */
                                   if ( *outFilename )
                                     {
                                       fprintf( stderr,
                  "WARNING: transform.main(): parameter '-n' appears twice;" );
                                       fprintf( stderr,
                                                 " %*s'%s' overwrites '%s'.\n",
                                                24, "", ptr + 1, netFilename );
                                     } /* if */
                                   strcpy( netFilename, ++ptr );
                                   ptr = null;
                                   break;
                        case 'o' : if ( *++ptr != '=' )
                                     {
                                       fprintf( stderr,
                       "ERROR: transform.main(): '=' expected after '-o'.\n" );
                                       exit ( 11 );
                                     } /* if */
                                   if ( *outFilename )
                                     {
                                       fprintf( stderr,
                  "WARNING: transform.main(): parameter '-o' appears twice;" );
                                       fprintf( stderr,
                                                 " %*s'%s' overwrites '%s'.\n",
                                                24, "", ptr + 1, outFilename );
                                     } /* if */
                                   strcpy( outFilename, ++ptr );
                                   ptr = null;
                                   break;
                        case 's' : if ( *++ptr != '=' )
                                     {
                                       fprintf( stderr,
                       "ERROR: transform.main(): '=' expected after '-s'.\n" );
                                       exit ( 11 );
                                     } /* if */
                                   if ( *sdlFilename )
                                     {
                                       fprintf( stderr,
                  "WARNING: transform.main(): parameter '-s' appears twice;" );
                                       fprintf( stderr,
                                                 " %*s'%s' overwrites '%s'.\n",
                                                24, "", ptr + 1, sdlFilename );
                                     } /* if */
                                   strcpy( sdlFilename, ++ptr );
                                   ptr = null;
                                   break;
                        case 'x' : if ( *++ptr != '=' )
                                     {
                                       fprintf( stderr,
                       "ERROR: transform.main(): '=' expected after '-x'.\n" );
                                       exit ( 11 );
                                     } /* if */
                                   if ( *sdlInfofilename )
                                     {
                                       fprintf( stderr,
                  "WARNING: transform.main(): parameter '-x' appears twice;" );
                                       fprintf( stderr,
                                                 " %*s'%s' overwrites '%s'.\n",
                                             24, "", ptr + 1, sdlInfofilename );
                                     } /* if */
                                   strcpy( sdlInfofilename, ++ptr );
                                   ptr = null;
                                   break;
                        case 'p' : if ( *++ptr != '=' )
                                     {
                                       fprintf( stderr,
                       "ERROR: transform.main(): '=' expected after '-p'.\n" );
                                       exit ( 11 );
                                     } /* if */
                                   ch = *++ptr;
                                   switch ( ch )
                                     {
                                       case 'c' : FT_pretty     = False;
                                                  FT_userlike   = False;
                                                  break;
                                       case 'p' : FT_pretty     = True;
                                                  FT_userlike   = False;
                                                  break;
                                       case 'u' : FT_pretty     = False;
                                                  FT_userlike   = True;
                                                  break;
                                       default  : fprintf( stderr,
          "ERROR: transform.main(): 'c', 'p' or 'u' expected after '-p='.\n" );
                                       exit ( 11 );
                                     } /* switch */
                                   break;
                        case 'v' : showVersion = True;
                                   break;
                        case 'l' : useLTL = True;
                                   break;
                        case '$' : showUsage = True;
                                   break;
                        default  : fprintf( stderr,
                            "WARNING: transform.main(): bad parameter '%s'.\n",
                                                                 argv[count] );
                                   break;
                      } /* switch */
                  } /* while */
              } /* if */
            else
              {
                if ( *netFilename )
                  {
                    fprintf( stderr,
                     "WARNING: unfold.main(): parameter '-n' appears twice;" );
                    fprintf( stderr, " %*s'%s' overwrites '%s'.\n",
                                                    24, "", ptr, netFilename );
                  } /* if */
                strcpy( netFilename, ptr );
              } /* else */
          } /* while */
      } /* if */
    else
      {
        showUsage = True;
      } /* else */

    if ( showVersion )
      {
        printf( "Version %s (%s), Stefan Roemer, Stefan Schwoon\n",
                                                               VERSION, DATE );
      } /* if */

    if ( showUsage )
      {
      printf( "'%s' -- formula translator for the model checker.\n", *argv );
        printf( "parameters: ( in any order )\n" );
        printf( "  <PEP_netfile>      or\n" );
        printf( "  -n=<PEP_netfile>   specify net file\n" );
        printf( "  -i=<formula_file>  specify input file\n" );
        printf( "  -o=<formula_file>  formula output file\n" );
        printf( "  -s=<sdl_ref_file>  specify SDL reference file\n" );
        printf( "  -x=<sdl_info_file> specify SDL information file\n" );
        printf( "  -f=p|h             use font: (p)ep or (h)artmut\n" );
        printf( "  -p=c|p|u           output style: (c)ompressed," );
        printf( " (p)retty or (u)ser-like\n" );
        printf( "  -l                 use LTL syntax for formulae\n" );
        printf( "  -v                 display program version number\n" );
      } /* if */

    if ( ! ( showVersion || showUsage || ( *netFilename && *inFilename ) ) )
      {
        fprintf( stderr,
                   "ERROR: transform.main(): missing input file names.\n" );
        exit ( 10 );
      } /* else */

    if ( ! ( *netFilename && *inFilename ) )
      {
        exit ( 0 );
      } /* if */

    if ( ! ( FT_net = read_PEP( netFilename ) ) )
      {
        fprintf( stderr,
                 "ERROR: transform.main(): file '%s' could not be opened\n",
                 netFilename );
        exit( 3 );
      } /* if */

    if (*sdlFilename)
    {
	read_SDLref(FT_net,sdlFilename);

	if (!*sdlInfofilename)
	{
		char *c;

		strcpy(sdlInfofilename,sdlFilename);
		c = strrchr(sdlInfofilename,'.');
		if (!c) strcat(sdlInfofilename,".");
		c = strrchr(sdlInfofilename,'.');
		strcpy(c,".sdl_info");
	}
	read_SDLinfo(sdlInfofilename);
    }


    if (useLTL)
    {
	set_LTL_keycodes();
    }
    initMacroTable( useLTL );

    if ( ! ( yyin = fopen( inFilename, "r" ) ) )
      {
        fprintf( stderr,
                 "ERROR: transform.main(): file '%s' could not be opened\n",
                 inFilename );
        exit( 3 );
      } /* if */

    if ( ! *outFilename )
      {
        strcat( strcpy( outFilename, inFilename ), ".ft" );
      } /* if */

    if ( ! ( yyout = fopen( outFilename, "w" ) ) )
      {
        fprintf( stderr,
                 "ERROR: transform.main(): file '%s' could not be opened\n",
                 outFilename );
        exit( 3 );
      } /* if */
    
    for ( pl = FT_net->Places, count = 1 ; pl ; pl = pl->nextplace )
      {
        pl->inscript = (char *) count++;
      } /* for */
    for ( tr = FT_net->Transitions, count = 1 ; tr ; tr = tr->nexttrans )
      {
        tr->inscript = (char *) count++;
      } /* for */
    FT_PlArray=GeneratePlArray(FT_net);
    FT_TrArray=GenerateTrArray(FT_net);
    FT_PlMArray=GeneratePlMArray();

    if ( (useLTL? ltparse : yyparse)() )
      {
        PEPerror( "Parsing error" );
      } /* if */
    fprintf(yyout, "\n\n");
    fclose( yyout );
    fclose( yyin );
    set_delete_mem( &SET_freeList );
    delete_meaning_list( CM_transMList );
    RemovePLMArray(FT_PlMArray);
    Free( FT_PlArray );
    Free( FT_TrArray );

//    remove_net( &FT_net );

    exit( 0 );

  } /* main() */


/*****************************************************************************/
/* END OF FILE transform.c                                                   */
/*****************************************************************************/

