/****************************************************************************/
/*             O P E R A T I O N S   O N   B I T - A R R A Y S              */
/****************************************************************************/

#ifndef __BITARRAY_H__
#define __BITARRAY_H__

#include <sys/types.h>
#include "neunetze.h"

/* Data structure: bit array. u_char is type unsigned char */
typedef u_char *BIT_ARRAY;

/****************************************************************************/

#define Test_Bit( ARRAY, INDEX ) \
  ( *( (ARRAY) + ( (INDEX) >> 3 ) ) & ( 1 << ( (INDEX) & 7 ) ) )

#define Set_Bit( ARRAY, INDEX ) \
  ( *( (ARRAY) + ( (INDEX) >> 3 ) ) |= ( 1 << ( (INDEX) & 7 ) ) )

#define Reset_Bit( ARRAY, INDEX ) \
  ( *( (ARRAY) + ( (INDEX) >> 3 ) ) &= ~( 1 << ( (INDEX) & 7 ) ) )

#define mymalloc_bitarray(SIZE) \
  (BIT_ARRAY) MYmalloc (SIZE * sizeof (u_char))

#define myrealloc_bitarray(ARRAY,SIZE) \
  (BIT_ARRAY) MYrealloc (ARRAY,SIZE * sizeof (u_char))

/****************************************************************************/
/* Liefert Speicher fuer neues Bitarray zurueck, Groesse in GL_bitArrayLen  */

#if defined(__STDC__) || defined(__cplusplus)

BIT_ARRAY bit_alloc( void );

#else

BIT_ARRAY bit_alloc( );

#endif

/****************************************************************************/
/* Liefert den Wert der Arraygroesse zurueck                                */

#if defined(__STDC__) || defined(__cplusplus)

Cardinal32 bit_getsize( void );

#else

Cardinal32 bit_getsize( );

#endif

/****************************************************************************/
/* Setzt die Groesse der Bitarrays, muss vor dem ersten Erzeugen von Arrays */
/* aufgerufen werden, danach sollte keine weitere Aenderung gemacht werden  */

#if defined(__STDC__) || defined(__cplusplus)

void bit_setsize( Cardinal32 size );

#else

void bit_setsize( size );

#endif

/****************************************************************************/
/* Test, ob array1<array2 (-1), array1==array2 (0) oder array1>array2 (1).  */

#if defined(__STDC__) || defined(__cplusplus)

int bit_compare( BIT_ARRAY array1, BIT_ARRAY array2 );

#else

int bit_compare( array1, array2 );

#endif

/****************************************************************************/
/* Display a bit array as a place list.                                     */

#if defined(__STDC__) || defined(__cplusplus)

void bit_print_name( BIT_ARRAY arrayptr, LMPNet net );

#else

void bit_print_name( arrayptr, net );

#endif

/****************************************************************************/
/* Display a bit array as a place list.                                     */

#if defined(__STDC__) || defined(__cplusplus)

void bit_print_numname( BIT_ARRAY arrayptr, LMPNet net );

#else

void bit_print_numname( arrayptr, net );

#endif

/*****************************************************************************/
/* Kopieren eines Bit-Arrays, Rueckgabe ist die neu angelegte Kopie          */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

BIT_ARRAY bit_copy( BIT_ARRAY array );

#else

BIT_ARRAY bit_copy( array );

#endif

/*****************************************************************************/
/* Vereinigen zweier Bit-Arrays.                                             */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

BIT_ARRAY bit_union( BIT_ARRAY array1, BIT_ARRAY array2 );

#else

BIT_ARRAY bit_union( array1, array2 );

#endif

/*****************************************************************************/
/* Calculate the difference of two bit arrays.                               */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

BIT_ARRAY bit_difference( BIT_ARRAY array1, BIT_ARRAY array2 );

#else

BIT_ARRAY bit_difference( array1, array2 );

#endif

#endif /* __BITARRAY_H__ */

/**************************/
/* END OF FILE bitarray.h */
/**************************/
