/*****************************************************************************/
/* Check for the cut-off property.                                           */
/*****************************************************************************/

#include "cutoff_ev.h"
#include "hash.h"
#include "sub_unf.h"
#include "nets.h"
#include "queue.h"

/*****************************************************************************/
/* Insert a new event in the unfolding.                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Event Insert_Event_With_CE_Edges( LMPNet unfold_net, Transition tr_org,
                                  Condition * conds )

#else

Event Insert_Event_With_CE_Edges( unfold_net, tr_org, conds )

  LMPNet unfold_net;
  Transition tr_org;
  Condition  * conds;

#endif

  {

    register Event      event;
    register Condition  cond;
    register STEdge     ce,
                        ceold;
    register TSEdge     ec;
    register Cardinal32 size,
                        maxsize;
    register Condition  * coptr;

    event = insert_event(unfold_net, tr_org);
    ceold = ce = event->preset = (STEdge) MYmalloc( sizeof( STElem ) );
    ce->prevplace = NULL;
    ce->weight = 0;
    ce->nexttrans = NULL;
    ce->trans = event;
    for ( size = (Cardinal32) tr_org->inscript; --size; skip )
      {
	( ce->nextplace = (STEdge) MYmalloc( sizeof( STElem ) ) )
	  ->prevplace = ce;
        ce = ce->nextplace;
	ce->trans = event;
        ce->weight = 0;
        ce->nexttrans = NULL;
      } /* for */
    ce->nextplace = NULL;
    ce = ceold;
    ce->place = cond = *( coptr = conds + 1 );
    if ( (ceold = (STEdge) cond->marks ))
      {
        ( ceold->nexttrans = ce )->prevtrans = ceold;
      } /* if */
    else
      {
        ( cond->postset = ce )->prevtrans = NULL;
      } /* else */
    cond->marks = (Cardinal32) ce;
    while ( (ce = ce->nextplace) )
      {
        if ( (ceold = (STEdge) ( ( ce->place = cond = *++coptr )->marks ) ))
          {
            ( ceold->nexttrans = ce )->prevtrans = ceold;
          } /* if */
        else
          {
            ( cond->postset = ce )->prevtrans = NULL;
          } /* else */
        cond->marks = (Cardinal32) ce;
      } /* while */

    /* LocalConfigDepth() */
    maxsize = 0;
    for ( ce = event->preset ; ce ; ce = ce->nextplace )
      {
        if ( ( ec = ce->place->preset ) &&
             ( ( size = (Cardinal32) ec->trans->label ) > maxsize ) )
          maxsize = size;
      } /* for */
    event->label = (MultiSet) ++maxsize;

#ifdef _EV_DISPLAY_IT_
    printf( "Event E%u (T%u/%s) inserted with depth %u.\n", event->numname,
                                     ( (Transition) event->inscript )->numname,
           ( (Transition) event->inscript )->name, (Cardinal32) event->label );
#endif /* _EV_DISPLAY_IT_ */

    return ( event );

  } /* Insert_Event_With_CE_Edges() */

/*****************************************************************************/
/* Test if finalStateList contains an cutoff event                           */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

int IsNotCutoffEvent_LIST( LIST          finalStateList,
                                  Event         event )

#else

int IsNotCutoffEvent_LIST( finalStateList, event )

  LIST          finalStateList;
  Event         event;

#endif

  {

    register HASH hash,
                  newhash;
    register LIST newCutoff;
    register int  test = 2;

    if ( GL_flags & CHECK_CONFIGS )
      {
        if ( (int) event > 10 )
          {
            printf( "E%u: ", event->numname );
            printf( "(depth=%u) ", (Cardinal32) event->label );
            printf( "(size=%u) ", GL_configSize );
          } /* if */
        else
          {
            printf( "--  Start Marking  --: " );
          } /* else */
        list_print( finalStateList );
      } /* if */

    if ( ( hash = GL_hashTable[ GL_hashIndex ] )
           && ( ( test = list_compare( finalStateList, hash->list ) ) == 1 ) )
      {
        while ( ( newhash = hash->next ) && ( ( test = list_compare(
                                    finalStateList, newhash->list ) ) == 1 ) )
          {
            hash = newhash;
          } /* while */
        if ( newhash && ! test )
          {
            hash = newhash;
            list_delete( &finalStateList );
          } /* if */
        else
          {
            ( newhash = (HASH) MYmalloc( sizeof( HASH_ELEM ) ) )->event = event;
            newhash->list = finalStateList;
            newhash->next = hash->next;
            hash = hash->next = newhash;
          } /* else */
      } /* if */
    else if ( ! test )
      {
        list_delete( &finalStateList );
      } /* else if */
    else
      {
        ( hash = (HASH) MYmalloc( sizeof( HASH_ELEM ) ) )->list = finalStateList;
        hash->next = GL_hashTable[ GL_hashIndex ];
        ( GL_hashTable[ GL_hashIndex ] = hash )->event = event;
      } /* else */

    if ( ! test )
      {
        GL_numCutoffs++;
        ( ( newCutoff = allocLIST() )->next = GL_cutoffList );
        ( GL_cutoffList = newCutoff )->item = (Place) event;
        ( ( newCutoff = allocLIST() )->next = GL_cutoffCorrespondingList );
        ( GL_cutoffCorrespondingList = newCutoff )->item =
	  (GL_flags & LAST_CORR_EVENT) ? (Place) hash->minConfigSize
	    : (Place) hash->event;
      } /* if */

    if ( hash )
      hash->minConfigSize = (Cardinal32) event;

#ifdef _EV_DISPLAY_IT_
printf( "E%u is %s cutoff event\n", (int) event > 10 ? event->numname : 0,
                                                        test ? "NOT a" : "A" );
#endif /* _EV_DISPLAY_IT_ */
    return ( test );

  } /* IsNotCutoffEvent_LIST() */


/*****************************************************************************/
/* The function 'mark()' ( also known as 'final_state()' ).                  */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LIST Mark_LIST( Event event, LMPNet unfold_net )

#else

LIST Mark_LIST( event, unfold_net )

  Event event;
  LMPNet unfold_net;

#endif

  {

    register Place      cond,
                        * stackptr,
                        * stackptr2;
    register Event      ev;
    register STEdge     ce;
    register TSEdge     ec;
    LIST                placeList;
    register Cardinal32 sum;

    IncCMark();
    IncEMark();
    InitCStack();
    *( stackptr2 = GL_stackC2 ) = NULL;
    sum = 1;
    EMark( event );
    for ( ce = event->preset ; ce ; ce = ce->nextplace )
      {
        *++stackptr2 = cond = ce->place;
/*
        if ( ( cond = ce->place )->postset->nexttrans )
          *++stackptr2=cond;
*/
        Set_Bit( GL_conflictArray, cond->numname );
        CMark( PushCondition( cond ) );
      } /* for */
    while ( (cond = PopCondition()) )
      {
        if ( IsNotEMarked( ev = cond->preset->trans ) )
          {
            EMark( ev );
            sum++;
            for ( ce = ev->preset ; ce ; ce = ce->nextplace )
              {
                CMark( cond = ce->place );
                if ( ( ec = cond->preset ) && IsNotEMarked( ec->trans ))
                  PushCondition( cond );
                Set_Bit( GL_conflictArray, cond->numname );
/*
                if ( cond->postset->nexttrans )
                  *++stackptr2 = cond;
*/
                *++stackptr2 = cond;
              } /* for */
          } /* if */
      } /* while */
    GL_configSize = sum;

    IncEMark();
    InitCStack();
    placeList = NULL;
    for ( ce = event->preset ; ce ; ce = ce->nextplace )
      {
        PushCondition( ce->place );
      } /* for */
    while ( (cond = PopCondition()) )
      {
        for ( ec = (ev=cond->preset->trans)->postset; ec ; ec = ec->nextplace )
          {
            if ( !IsCMarked( cond = ec->place ))
              list_insert( &placeList, (Place) cond->inscript, cond->tokenage );
          } /* for */
        if ( IsNotEMarked( ev ) )
          {
            EMark( ev );
            for ( ce = ev->preset ; ce ; ce = ce->nextplace )
              {
                ec = ( cond = ce->place )->preset;
                if ( ec && IsNotEMarked( ec->trans ))
                  PushCondition( cond );
              } /* for */
          } /* if */
      } /* while */

    for ( ec = ( (Transition) event->inscript )->postset ; ec ;
                                                           ec = ec->nextplace )
      {
        list_insert( &placeList, ec->place, 0 );
      } /* for */

    for ( cond = unfold_net->Places ; cond && ! cond->preset ;
                                                       cond = cond->nextplace )
      {
        if ( ! IsCMarked( cond ))
          list_insert( &placeList, (Place) cond->inscript, 0 );
      } /* for */

    hash_index_list( placeList );

    GL_stackC2ptr = stackptr2;

    return ( placeList );

  } /* Mark_LIST() */


/*****************************************************************************/
/* Check for the cut-off property.                                           */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

int IsNotCutoffEvent_ARRAY( LIST          finalStateList,
                                   Event         event )

#else

int IsNotCutoffEvent_ARRAY( finalStateList, event )

  LIST          finalStateList;
  Event         event;

#endif

  {

    register HASH      hash,
                       newhash;
    register LIST      newCutoff;
    register BIT_ARRAY finalStateArray = (BIT_ARRAY) finalStateList;
    register int       test = 2;

    if ( GL_flags & CHECK_CONFIGS )
      {
        if ( (int) event > 10 )
          {
            printf( "E%u: ", event->numname );
            printf( "(depth=%u) ", (Cardinal32) event->label );
            printf( "(size=%u) ", GL_configSize );
          } /* if */
        else
          {
            printf( "--  Start Marking  --: " );
          } /* else */
        bit_print_name( (BIT_ARRAY) finalStateList, GL_net );
      } /* if */

    if ( ( hash = GL_hashTable[ GL_hashIndex ] )
           && ( ( test = bit_compare( finalStateArray,
                                            (BIT_ARRAY) hash->list ) ) == 1 ) )
      {
        while ( ( newhash = hash->next ) && ( ( test = bit_compare(
                        finalStateArray, (BIT_ARRAY) newhash->list ) ) == 1 ) )
          {
            hash = newhash;
          } /* while */
        if ( newhash && ! test )
          {
            hash = newhash;
            Free( finalStateArray );
if ( (int) event > 2 ) event->preset->weight = (Cardinal32) newhash->list;
          } /* if */
        else
          {
            ( newhash = (HASH) MYmalloc( sizeof( HASH_ELEM ) ) )->event = event;
            newhash->list = (LIST) finalStateArray;
            newhash->next = hash->next;
            hash = hash->next = newhash;
if ( (int) event > 2 ) event->preset->weight = (Cardinal32) finalStateArray;
          } /* else */
      } /* if */
    else if ( ! test )
      {
        Free( finalStateArray );
if ( (int) event > 2 ) event->preset->weight = (Cardinal32) hash->list;
      } /* else if */
    else
      {
        ( hash = (HASH) MYmalloc( sizeof( HASH_ELEM ) ) )->list
                                                      = (LIST) finalStateArray;
        hash->next = GL_hashTable[ GL_hashIndex ];
        ( GL_hashTable[ GL_hashIndex ] = hash )->event = event;
if ( (int) event > 2 ) event->preset->weight = (Cardinal32) finalStateArray;
      } /* else */

    if ( ! test )
      {
        GL_numCutoffs++;
        ( ( newCutoff = allocLIST() )->next = GL_cutoffList );
        ( GL_cutoffList = newCutoff )->item = (Place) event;
        ( ( newCutoff = allocLIST() )->next = GL_cutoffCorrespondingList );
	/* bei EV2 evtl. ohne GL_getLast... ??? (war vorher nicht) */
        ( GL_cutoffCorrespondingList = newCutoff )->item =
	  (GL_flags & LAST_CORR_EVENT ) ? (Place) hash->minConfigSize
	    : (Place) hash->event;
      } /* if */

    if ( hash )
      hash->minConfigSize = (Cardinal32) event;

/* DIE FOLGENDEN ZEILEN KOENNEN SPAETER WIEDER WEG !!! */
    if ( GL_flags & CHECK_CONFIGS )
      {
        if ( (int) event > 10 )
          {
            printf( "E%u: ", event->numname );
            printf( "(depth=%u) ", (Cardinal32) event->label );
            printf( "(size=%u) ", GL_configSize );
            bit_print_name( (BIT_ARRAY) event->preset->weight, GL_net );
          } /* if */
        else
          {
            printf( "--  Start Marking  --: " );
            bit_print_name( GL_BitStartMarking, GL_net );
          } /* else */
      } /* if */

#ifdef _EV_DISPLAY_IT_
printf( "E%u is %s cutoff event\n", (int) event > 10 ? event->numname : 0,
                                                        test ? "NOT a" : "A" );
#endif /* _EV_DISPLAY_IT_ */
    return ( test );

  } /* IsNotCutoffEvent_ARRAY() */


/*****************************************************************************/
/* The function 'mark()' ( also known as 'final_state()' ).                  */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LIST Mark_ARRAY( Event event, LMPNet unfold_net )

#else

LIST Mark_ARRAY( event, unfold_net )

  Event event;
  LMPNet unfold_net;

#endif

  {

    register Place      cond,
                        * stackptr,
                        * stackptr2;
    register Event      ev;
    register STEdge     ce;
    register TSEdge     ec;
    BIT_ARRAY           placeArray;
    register Cardinal32 sum;

    IncCMark();
    IncEMark();
    InitCStack();
    *( stackptr2 = GL_stackC2 ) = NULL;
    sum = 1;
    EMark( event );
    for ( ce = event->preset ; ce ; ce = ce->nextplace )
      {
        *++stackptr2 = cond = ce->place;
/*
        if ( ( cond = ce->place )->postset->nexttrans )
          *++stackptr2=cond;
*/
        Set_Bit( GL_conflictArray, cond->numname );
        CMark( PushCondition( cond ) );
      } /* for */
    while ( (cond = PopCondition()) )
      {
        if ( IsNotEMarked( ev = cond->preset->trans ) )
          {
            EMark( ev );
            sum++;
            for ( ce = ev->preset ; ce ; ce = ce->nextplace )
              {
                CMark( cond = ce->place );
                ec = cond->preset;
                if ( ec && IsNotEMarked( ec->trans ))
                  PushCondition( cond );
                Set_Bit( GL_conflictArray, cond->numname );
/*
                if ( cond->postset->nexttrans )
                  *++stackptr2 = cond;
*/
                *++stackptr2 = cond;
              } /* for */
          } /* if */
      } /* while */
    GL_configSize = sum;

    IncEMark();
    InitCStack();
    placeArray = bit_alloc();
    for ( ce = event->preset ; ce ; ce = ce->nextplace )
      {
        PushCondition( ce->place );
      } /* for */
    while ( (cond = PopCondition()) )
      {
        for ( ec = (ev=cond->preset->trans)->postset; ec ; ec = ec->nextplace )
          {
            if ( ! IsCMarked( cond = ec->place ))
              Set_Bit( placeArray, ( (Place) cond->inscript )->numname );
          } /* for */
        if ( IsNotEMarked( ev ) )
          {
            EMark( ev );
            for ( ce = ev->preset ; ce ; ce = ce->nextplace )
              {
                ec = ( cond = ce->place )->preset;
                if ( ec && IsNotEMarked( ec->trans ))
                  PushCondition( cond );
              } /* for */
          } /* if */
      } /* while */

    for ( ec = ( (Transition) event->inscript )->postset ; ec ;
                                                           ec = ec->nextplace )
      {
        Set_Bit( placeArray, ec->place->numname );
      } /* while */

    for ( cond = unfold_net->Places ; cond && ! cond->preset ;
                                                       cond = cond->nextplace )
      {
        if ( ! IsCMarked( cond ))
          Set_Bit( placeArray, ( (Place) cond->inscript )->numname );
      } /* for */

    hash_index_bit( placeArray );

    GL_stackC2ptr = stackptr2;

    return ( (LIST) placeArray );

  } /* Mark_ARRAY() */



