/* Funktionen zum Ueberpruefen auf cut_offs */

#include "cutoff_mm.h"
#include "hash.h"
#include "nets.h"
#include "queue.h"
#include "sub_unf.h"
#include "neunetze.h"


/****************************************************************************/
/* Fuegt neuen Event in die Entfaltung ein. Dieser entspricht der           */
/* uebergebenen Transition im originalen Netz. Gleichzeitig werden die      */
/* passenden Kanten cond->event eingefuegt.                                 */
/* Seiteneffekte: GL_UselessTransitions                                     */
/*                GL_LiConflictsFunction                                    */
/*                GL_listArray1 gibt cond an                                */
/****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Event insert_event_with_ce_edges( LMPNet unfold_net, Transition tr_org )

#else

Event insert_event_with_ce_edges( unfold_net, tr_org )

  LMPNet unfold_net;
  Transition tr_org;

#endif

  {

    register Event      event;
    register Condition  cond;
    register STEdge     ce,
                        ceold;
    register Cardinal32 size;
    register LIST       * listArrayPtr;

    /* Neues Event in Entfaltung einfuegen */
    event=insert_event(unfold_net, tr_org);

    /* InsertCE_Edges() */
    ceold = ce = event->preset = (STEdge) MYmalloc( sizeof( STElem ) );
    ce->prevplace = NULL;
    ce->weight = 0;
    ce->nexttrans = NULL;
    ce->trans = event;
    /* Alle Kanten einfuegen. Die Anzahl ist in inscript gesichert */
    for ( size = (Cardinal32) tr_org->inscript ; --size ; skip )
      {
        ( ce->nextplace = (STEdge) MYmalloc( sizeof( STElem ) ) )
	  ->prevplace = ce;
        ce = ce->nextplace;
        ce->trans = event;
	ce->weight = 0;
        ce->nexttrans = NULL;
      } /* for */
    ce->nextplace = NULL;
    ce = ceold;
    ce->place = cond = (Condition) *( listArrayPtr = GL_listArray1 );
    if ((ceold = (STEdge) cond->marks ))
      {
        ( ceold->nexttrans = ce )->prevtrans = ceold;
      } /* if */
    else
      {
        ( cond->postset = ce )->prevtrans = NULL;
      } /* else */
    cond->marks = (Cardinal32) ce;
    /* Die uebrigen Stellen an den Kanten einfuegen */
    for ( ce = ce->nextplace ; *++listArrayPtr ; 
	  ce = ce->nextplace, listArrayPtr++ )
      {
        ce->place = cond = (*listArrayPtr)->item;
        if ((ceold = (STEdge) cond->marks ))
          {
            ( ceold->nexttrans = ce )->prevtrans = ceold;
          } /* if */
        else
          {
            ( cond->postset = ce )->prevtrans = NULL;
          } /* else */
        cond->marks = (Cardinal32) ce;
      } /* for */

    insert_queue(event);

    return ( event );

  } /* insert_event_with_ce_edges() */

/* Einfuegen des Events in die Hashliste ?*/
/* Rueckgabe 0, falls Event Cutoff ist, durch Vergleich */
/* der Markierung(=finalStateList), sonst 1, bzw. 2     */
/* Seiteneffekte: GL_hashTable,                         */
/*                GL_numCutoffs, GL_cutoffList,         */
/*                GL_cutoffCorrespondingList            */
/* nur lesend:    GL_flags, GL_configSize, GL_hashIndex */

#if defined(__STDC__) || defined(__cplusplus)

int IsNotCutoffPoint_LIST( LIST finalStateList, Event event )

#else

int IsNotCutoffPoint_LIST( finalStateList, event )

  LIST  finalStateList;
  Event event;

#endif

  {

    HASH hash,
         newhash;
    LIST newCutoff, listptr, listptr2;
    int  test = 2;
    bool lastflag = False;

    /* Evtl. Informationen ausgeben */
    if ( GL_flags & CHECK_CONFIGS )
      {
        if ( (int) event > 10 )
          {
            printf( "E%u: ", event->numname );
            printf( "(depth=%u) ", (Cardinal32) event->label );
            printf( "(size=%u) ", GL_configSize );
          } /* if */
        else
          {
            printf( "--  Start Marking  --: " );
          } /* else */
        list_print( finalStateList );
      } /* if checkConfigs */

    /* Wenn Eintrag im Hash und finalStateList groesser als hash->list */
    if ( ( hash = GL_hashTable[ GL_hashIndex ] )
         && ( ( test = list_compare( finalStateList, hash->list ) ) == 1 ) )
      {
        /* Alle Eintraege ueberspringen, die kleiner sind */
        /* hash zeigt auf den letzten Eintrag             */
        while ( ( newhash = hash->next ) &&
                ( ( test = list_compare( finalStateList, newhash->list ) ) == 1 ) )
          {
            hash = newhash;
          } /* while */
        if ( GL_flags & REDUCED_UNF )
          {
            if ( newhash && ! test )
              {
                ( hash = newhash )->event = event;
                list_delete( &finalStateList );
              } /* if */
            else
              { /* minConfigSize wird nicht gesetzt, warum ?! */
                newhash = (HASH) MYmalloc( sizeof( HASH_ELEM ) );
                newhash->event = event;
                newhash->list = finalStateList;
                newhash->next = hash->next;
                hash = hash->next = newhash;
              } /* else */
          } /* if reducedUnfolding */
        /* Es folgen noch weitere Listen, aber die akt. Listen sind gleich gross */
        /* Dann wird die Konfiguration verworfen, evtl. die Groesse im Hash anpassen */
        else if ( newhash && ! test )
          {
            lastflag = True;
            if ( GL_configSize <= ( hash = newhash )->minConfigSize )
              {
                hash->minConfigSize = GL_configSize;
                test++;
              } /* if */
            list_delete( &finalStateList );
          } /* else if */
        /* Keine weiteren Listen oder aktuelle Liste ist kleiner */
        /* Konfiguration in der aktuellen Position einfuegen     */
        else
          {
            newhash = (HASH) MYmalloc( sizeof( HASH_ELEM ) );
            newhash->event = event;
            newhash->list = finalStateList;
            newhash->minConfigSize = GL_configSize;
            newhash->next = hash->next;
            hash = hash->next = newhash;
          } /* else */
      } /* if hash && test==1 */
    /* Sonst falls Eintrag im Hash und finalStateList==hash->list    */
    /* wie oben bei !test: Konfiguration verwerfen und Size anpassen */
    else if ( ! test )
      {
        lastflag = True;
        if ( GL_flags & REDUCED_UNF )
          {
            hash->event = event;
          } /* if */
        else if ( GL_configSize <= hash->minConfigSize )
          {
            hash->minConfigSize = GL_configSize;
            test++; /* test=1 */
          } /* else if */
        list_delete( &finalStateList );
      } /* else if */
    /* Sonst (kein Eintrag im Hash oder finalStateList kleiner als hash->list  */
    /* Neuen Eintrag vorne am Hasheintrag einfuegen, Liste und Event anhaengen */
    else
      {
        hash = (HASH) MYmalloc( sizeof( HASH_ELEM ) );
        hash->list = finalStateList;
        hash->minConfigSize = GL_configSize;
        hash->event = event;
        hash->next = GL_hashTable[ GL_hashIndex ];
        GL_hashTable[ GL_hashIndex ] = hash;
      } /* else */

    /* Wenn finalStateList==hash->list neuen Cutoff einfuegen */
    if ( ! test )
      {
        GL_numCutoffs++;
        newCutoff = allocLIST();
	newCutoff->item=(Place)event;
	if(!(listptr=GL_cutoffList)||
	   (((Event)listptr->item)->numname<event->numname))
	  {
	    newCutoff->next = listptr;
	    GL_cutoffList = newCutoff;
	    newCutoff = allocLIST();
	    newCutoff->item = (Place) hash->event;
	    newCutoff->next = GL_cutoffCorrespondingList;
	    GL_cutoffCorrespondingList = newCutoff;
	  }
	else
	  {
	    for(listptr2=GL_cutoffCorrespondingList;listptr->next&&
		  ((Event)listptr->next->item)->numname>event->numname;
		listptr=listptr->next,listptr2=listptr2->next)
	      skip;
	    /* Hinten anfuegen? */
	    if(!listptr->next)
	      {
		newCutoff->next=NULL;
		listptr->next=newCutoff;
		newCutoff = allocLIST();
		newCutoff->next=NULL;
		listptr2->next=newCutoff;
	      }
	    else
	      {
		newCutoff->next=listptr->next;
		listptr->next=newCutoff;
		newCutoff = allocLIST();
		newCutoff->next=listptr2->next;
		listptr2->next=newCutoff;
	      }
	    newCutoff->item = (Place) hash->event;
	  } /* else hinten anfuegen */
      } /* if neuer Cutoff */
    if ( (GL_flags & LAST_CORR_EVENT) && lastflag )
      hash->event = event;

    return ( test );

  } /* IsNotCutoffPoint_LIST() */


/****************************************************************************/
/* Berechnung des Endzustandes eines Ereignisses ( bzw. der damit verbunde- */
/* nen lokalen Konfiguration ).                                             */
/* Seiteneffekte: GL_ConflictArray, GL_configSize, GL_stackC2ptr            */
/* nur lesend:    GL_StackC2                                                */
/****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LIST FinalState_LIST( Event event, LMPNet unfold_net )

#else

LIST FinalState_LIST( event, unfold_net )

  Event event;
  LMPNet unfold_net;

#endif

  {
    register Place      cond,
                        * stackptr,
                        * stackptr2;
    register Event      ev;
    register STEdge     ce;
    register TSEdge     ec;
    LIST                placeList = NULL;
    register Cardinal32 sum;
    BIT_ARRAY           conflArrayPtr;

    if (event->inscript)
      { /* falls kein Tic-Event normale Berechnung */
	/* sum = (Anzahl Conditions + Anzahl Kanten im Nachbereich +2)/8 */
	sum = ( unfold_net->NumPlaces + (Cardinal32)
		( (Transition) event->inscript )->prevtrans + 2 ) >> 3;
	/* GL_conflictArray loeschen */
	for ( sum++,conflArrayPtr = GL_conflictArray ; sum-- ;
	      *( conflArrayPtr++ ) = '\0' )
	  skip;

	IncCMark();
	IncEMark();
	InitCStack();
	/* Stack stores visited conditions for conflict calculation */
	*( stackptr2 = GL_stackC2 ) = NULL;
	sum = 1;
	EMark( event );

	/* Im ersten Durchgang werden die dazugehoerigen Bed. und Ereignisse */
	/* markiert und bzgl. Konflikten gesichert, dann wird die Liste      */
	/* erstellt. Zuerst den Vorbereich auf den Stack legen und markieren */
	for ( ce = event->preset ; ce ; ce = ce->nextplace )
	  {
	    *++stackptr2 = cond = ce->place;
/*
  if ( ( cond = ce->place )->postset->nexttrans )
  *++stackptr2=cond;
*/
	    Set_Bit( GL_conflictArray, cond->numname );
	    CMark( PushCondition( cond ) );
	  } /* for ce */
	/* Alle Bedingungen bearbeiten: Transitionen im Vorbereich markieren,*/
	/* Bedingungen dessen Vorbereichs markieren und ggf. auf Stack legen */
	while ( (cond = PopCondition()) )
	  {
	    if ( IsNotEMarked( ev = cond->preset->trans ) )
	      {
		EMark( ev );
		sum++;
		for ( ce = ev->preset ; ce ; ce = ce->nextplace )
		  {
		    CMark( cond = ce->place );
		    if ( ( ec = cond->preset ) && IsNotEMarked( ec->trans ))
		      PushCondition( cond );
		    Set_Bit( GL_conflictArray, cond->numname );
/*
  if ( cond->postset->nexttrans )
  *++stackptr2 = cond;
*/
		    *++stackptr2 = cond;
		  } /* for */
	      } /* if */
	  } /* while cond */
	GL_configSize = sum;

	/* Nun richtige Liste erstellen, zuerst den Vorbereich des Events */
	IncEMark();
	InitCStack();
	for ( ce = event->preset ; ce ; ce = ce->nextplace )
	  {
	    PushCondition( ce->place );
	  } /* for */
	/* Nun alle auf dem Stack liegenden Ereignisse bearbeiten */
	while ( (cond = PopCondition()) )
	  {
	    ev=cond->preset->trans;
	    /* Dann von allen Vorgaengertransitionen die Nachfolgerstellen */
	    /* einfuegen, sofern noch nicht vorhanden                      */
	    for ( ec = ev->postset; ec ; ec = ec->nextplace )
	      {
		if ( !IsCMarked( cond = ec->place ))
		{
		  list_insert( &placeList, (Place) cond->inscript, 
			       cond->tokenage );
		  CMark(cond);
		}
	      } /* for */
	    /* Ggf. die Vorbereichststellen der Transitionen auf den Stack */
	    if ( IsNotEMarked( ev ) )
	      {
		EMark( ev );
		for ( ce = ev->preset ; ce ; ce = ce->nextplace )
		  {
		    cond = ce->place;
		    if ( ( ec = cond->preset ) && IsNotEMarked( ec->trans ))
		      PushCondition( cond );
		  } /* for */
	      } /* if */
	  } /* while cond */

	/* Nun noch den Nachbereich des aktuellen Events einfuegen */
	/* Ist eine Schalttransition, also Tokenage 0              */
	for ( ec = ( (Transition) event->inscript )->postset ; ec ;
	      ec = ec->nextplace )
	  {
	    list_insert( &placeList, ec->place, 0 );
	  } /* for */

	/* Startmarkierung einfuegen, ist vorne in der Entfaltung enthalten */
	for ( cond = unfold_net->Places ; cond && ! cond->preset ;
	      cond = cond->nextplace )
	  {
	    if ( !IsCMarked( cond ))
	      list_insert( &placeList, (Place) cond->inscript, 0 );
	  } /* for */
	GL_stackC2ptr = stackptr2;
      } /* if kein Tic-Event */
    else
      { /* Tic-Event hat nur Nachbereich als FinalState. Dazu Vorbereich */
	/* duplizieren, da Nachbereich evtl. noch nicht existiert */
	for(sum=0,ce=event->preset;ce;ce=ce->nextplace)
	  {
/* TODO: Stop aging if not needed */
	    STEdge tempst;
            if ((tempst=(((Place)ce->place->inscript)->postset)))
              {  
//                while(tempst&&(!tempst->trans->time_label||ce->place->tokenage>=tempst->trans->time_label->high))
                while(tempst && (
		  !tempst->trans->time_label || (ce->place->tokenage>=tempst->trans->time_label->high)|| is_inf_lft(tempst->trans)))
		  tempst=tempst->nexttrans;
	      }
            if (tempst)
	      list_insert(&placeList, (Place)ce->place->inscript, 
			  ce->place->tokenage+1);
	    else
	      list_insert(&placeList, (Place)ce->place->inscript,
                          ce->place->tokenage);
	    sum++;
	  }
	/* sum = (Anzahl Conditions + Anzahl Kanten im Nachbereich +2)/8 */
	sum = ( sum + unfold_net->NumPlaces + 2 ) >> 3;
	/* GL_conflictArray loeschen */
	for ( sum++,conflArrayPtr = GL_conflictArray ; sum-- ;
	      *( conflArrayPtr++ ) = '\0' )
	  skip;

	/* GL_configSize berechnen */
	IncCMark();
	IncEMark();
	InitCStack();
	/* Stack speichert die besuchten Bedingungen zwecks Konfliktberechnungen */
	sum = 1;
	EMark( event );

	/* Im ersten Durchgang werden die dazugehoerigen Bed. und Ereignisse */
	/* markiert und bzgl. Konflikten gesichert, dann wird die Liste      */
	/* erstellt. Zuerst den Vorbereich auf den Stack legen und markieren */
	for ( ce = event->preset ; ce ; ce = ce->nextplace )
	  {
	    cond = ce->place;
	    CMark( PushCondition( cond ) );
	  } /* for ce */
	/* Alle Bedingungen bearbeiten: Transitionen im Vorbereich markieren,*/
	/* Bedingungen dessen Vorbereichs markieren und ggf. auf Stack legen */
	while ( (cond = PopCondition()) )
	  {
	    if ( IsNotEMarked( ev = cond->preset->trans ) )
	      {
		EMark( ev );
		sum++;
		for ( ce = ev->preset ; ce ; ce = ce->nextplace )
		  {
		    CMark( cond = ce->place );
		    if ( ( ec = cond->preset ) && IsNotEMarked( ec->trans ))
		      PushCondition( cond );
		  } /* for */
	      } /* if */
	  } /* while cond */
	GL_configSize = sum;

      } /* else tic-Event */
    /* Hashindex berechnen */
    hash_index_list( placeList );

    return ( placeList );

  } /* FinalState_LIST() */


/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

int IsNotCutoffPoint_ARRAY( LIST finalStateList, Event event )

#else

int IsNotCutoffPoint_ARRAY( finalStateList, event )

  LIST  finalStateList;
  Event event;

#endif

  {

    register HASH      hash,
                       newhash;
    register LIST      newCutoff;
    register BIT_ARRAY finalStateArray = (BIT_ARRAY) finalStateList;
    register int       test = 2;
    bool               lastflag = False;

    if ( GL_flags & CHECK_CONFIGS )
      {
        if ( (int) event > 10 )
          {
            printf( "E%u: ", event->numname );
            printf( "(depth=%u) ", (Cardinal32) event->label );
            printf( "(size=%u) ", GL_configSize );
          } /* if */
        else
          {
            printf( "--  Start Marking  --: " );
          } /* else */
        bit_print_name( (BIT_ARRAY) finalStateList, GL_net );
      } /* if */

    if ( ( hash = GL_hashTable[ GL_hashIndex ] )
           && ( ( test = bit_compare( finalStateArray,
                                            (BIT_ARRAY) hash->list ) ) == 1 ) )
      {
        while ( ( newhash = hash->next ) && ( ( test = bit_compare(
                        finalStateArray, (BIT_ARRAY) newhash->list ) ) == 1 ) )
          {
            hash = newhash;
          } /* while */
        if ( GL_flags & REDUCED_UNF )
          {
            if ( newhash && ! test )
              {
                ( hash = newhash )->event = event;
                Free( finalStateArray );
              } /* if */
            else
              {
                ( newhash = (HASH) MYmalloc( sizeof( HASH_ELEM ) ) )->event
                                                                       = event;
                newhash->list = (LIST) finalStateArray;
                newhash->next = hash->next;
                hash = hash->next = newhash;
if ( (int) event > 2 ) event->preset->weight = (Cardinal32) finalStateArray;
              } /* else */
          } /* if */
        else if ( newhash && ! test )
          {
            lastflag = True;
            if ( GL_configSize <= ( hash = newhash )->minConfigSize )
              {
                hash->minConfigSize = GL_configSize;
                test++;
              } /* if */
            Free( finalStateArray );
if ( (int) event > 2 ) event->preset->weight = (Cardinal32) newhash->list;
          } /* else if */
        else
          {
            ( newhash = (HASH) MYmalloc( sizeof( HASH_ELEM ) ) )->event = event;
            newhash->list = (LIST) finalStateArray;
            newhash->minConfigSize = GL_configSize;
            newhash->next = hash->next;
            hash = hash->next = newhash;
if ( (int) event > 2 ) event->preset->weight = (Cardinal32) finalStateArray;
          } /* else */
      } /* if */
    else if ( ! test )
      {
        lastflag = True;
        if ( GL_flags & REDUCED_UNF )
          {
            hash->event = event;
          } /* if */
        else if ( GL_configSize <= hash->minConfigSize )
          {
            hash->minConfigSize = GL_configSize;
            test++;
          } /* else if */
        Free( finalStateArray );
if ( (int) event > 2 ) event->preset->weight = (Cardinal32) hash->list;
      } /* else if */
    else
      {
        ( hash = (HASH) MYmalloc( sizeof( HASH_ELEM ) ) )->list
                                                      = (LIST) finalStateArray;
        hash->minConfigSize = GL_configSize;
        hash->next = GL_hashTable[ GL_hashIndex ];
        ( GL_hashTable[ GL_hashIndex ] = hash )->event = event;
if ( (int) event > 2 ) event->preset->weight = (Cardinal32) finalStateArray;
      } /* else */

    if ( ! test )
      {
        GL_numCutoffs++;
        ( ( newCutoff = allocLIST() )->next = GL_cutoffList );
        ( GL_cutoffList = newCutoff )->item = (Place) event;
        ( ( newCutoff = allocLIST() )->next = GL_cutoffCorrespondingList );
        ( GL_cutoffCorrespondingList = newCutoff )->item = (Place) hash->event;
      } /* if */
    if ( (GL_flags & LAST_CORR_EVENT) && lastflag )
      hash->event = event;

/* DIE FOLGENDEN ZEILEN KOENNEN SPAETER WIEDER WEG !!! */
if ( GL_flags & CHECK_CONFIGS )
  {
    if ( (int) event > 10 )
      {
        printf( "E%u: ", event->numname );
        printf( "(depth=%u) ", (Cardinal32) event->label );
        printf( "(size=%u) ", GL_configSize );
        bit_print_name( (BIT_ARRAY) event->preset->weight, GL_net );
      } /* if */
    else
      {
        printf( "--  Start Marking  --: " );
        bit_print_name( GL_BitStartMarking, GL_net );
      } /* else */
  } /* if */

    return ( test );

  } /* IsNotCutoffPoint_ARRAY() */


/*****************************************************************************/
/* Berechnung des Endzustandes eines Ereignisses ( bzw. der damit verbunde-  */
/* nen lokalen Konfiguration ).                                              */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LIST FinalState_ARRAY( Event event, LMPNet unfold_net )

#else

LIST FinalState_ARRAY( event, unfold_net )

  Event event;
  LMPNet unfold_net;

#endif

  {

    register Place      cond,
                        * stackptr,
                        * stackptr2;
    register Event      ev;
    register STEdge     ce;
    register TSEdge     ec;
    BIT_ARRAY           placeArray;
    register Cardinal32 sum;

    /* sum = (Anzahl Conditions + Anzahl Kanten im Nachbereich +2)/8 */
    sum = ( unfold_net->NumPlaces + (Cardinal32)
	    ( (Transition) event->inscript )->prevtrans + 2 ) >> 3;
    /* GL_conflictArray loeschen */
    for ( placeArray = GL_conflictArray ; sum-- ;
	  *( placeArray++ ) = '\0' )
      skip;

    IncCMark();
    IncEMark();
    InitCStack();
    *( stackptr2 = GL_stackC2 ) = NULL;
    sum = 1;
    EMark( event );
    for ( ce = event->preset ; ce ; ce = ce->nextplace )
      {
        *++stackptr2 = cond = ce->place;
/*
        cond = ce->place;
        if ((cond->postset->nexttrans))
          *++stackptr2=cond;
*/
        Set_Bit( GL_conflictArray, cond->numname );
        CMark( PushCondition( cond ) );
      } /* for */
    while ( (cond = PopCondition()) )
      {
        if ( IsNotEMarked( ev = cond->preset->trans ) )
          {
            EMark( ev );
            sum++;
            for ( ce = ev->preset ; ce ; ce = ce->nextplace )
              {
                CMark( cond = ce->place );
                ec = cond->preset;
                if (( ec && IsNotEMarked( ec->trans ) ))
                  PushCondition( cond );
                Set_Bit( GL_conflictArray, cond->numname );
/*
                if (( cond->postset->nexttrans ))
                  *++stackptr2 = cond;
*/
                *++stackptr2 = cond;
              } /* for */
          } /* if */
      } /* while */
    GL_configSize = sum;

    IncEMark();
    InitCStack();
    placeArray = bit_alloc();
    for ( ce = event->preset ; ce ; ce = ce->nextplace )
      {
        PushCondition( ce->place );
      } /* for */
    while ( (cond = PopCondition()) )
      {
        for ( ec = (ev=cond->preset->trans)->postset; ec ; ec = ec->nextplace )
          {
            if ( !IsCMarked( cond = ec->place ))
              Set_Bit( placeArray, ( (Place) cond->inscript )->numname );
          } /* for */
        if ( IsNotEMarked( ev ) )
          {
            EMark( ev );
            for ( ce = ev->preset ; ce ; ce = ce->nextplace )
              {
                ec = ( cond = ce->place )->preset;
                if ( ec && IsNotEMarked( ec->trans ))
                  PushCondition( cond );
              } /* for */
          } /* if */
      } /* while */

    for ( ec = ( (Transition) event->inscript )->postset ; ec ;
                                                           ec = ec->nextplace )
      {
        Set_Bit( placeArray, ec->place->numname );
      } /* for */

    for ( cond = unfold_net->Places ; cond && ! cond->preset ;
                                                       cond = cond->nextplace )
      {
        if ( !IsCMarked( cond ))
          Set_Bit( placeArray, ( (Place) cond->inscript )->numname );
      } /* while */

    hash_index_bit( placeArray );

    GL_stackC2ptr = stackptr2;

    return ( (LIST) placeArray );

  } /* FinalState_ARRAY() */


/*****************************************************************************/
/* Berechnung des Endzustandes eines Ereignisses ( bzw. der damit verbunde-  */
/* nen lokalen Konfiguration ).                                              */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LIST FinalState_MARKING( Event event, LMPNet unfold_net )

#else

LIST FinalState_MARKING( event, unfold_net )

  Event event;
  LMPNet unfold_net;

#endif

  {

    register Place      cond,
                        * stackptr,
                        * stackptr2;
    register Event      ev;
    register STEdge     ce;
    register TSEdge     ec;
    BIT_ARRAY           placeArray,
                        bitMarking;
    register Cardinal32 sum;

    /* sum = (Anzahl Conditions + Anzahl Kanten im Nachbereich +2)/8 */
    sum = ( unfold_net->NumPlaces + (Cardinal32)
	    ( (Transition) event->inscript )->prevtrans + 2 ) >> 3;
    /* GL_conflictArray loeschen */
    for ( placeArray = GL_conflictArray ; sum-- ;
	  *( placeArray++ ) = '\0' )
      skip;

    IncCMark();
    IncEMark();
    InitCStack();
    *( stackptr2 = GL_stackC2 ) = NULL;
    sum = 1;
    EMark( event );
    for ( ce = event->preset ; ce ; ce = ce->nextplace )
      {
        *++stackptr2 = cond = ce->place;
/*
        cond = ce->place;
        if (( cond->postset->nexttrans ))
          *++stackptr2=cond;
*/
        Set_Bit( GL_conflictArray, cond->numname );
        CMark( PushCondition( cond ) );
      } /* for */
    while ( (cond = PopCondition()) )
      {
        if ( IsNotEMarked( ev = cond->preset->trans ) )
          {
            EMark( ev );
            sum++;
            for ( ce = ev->preset ; ce ; ce = ce->nextplace )
              {
                CMark( cond = ce->place );
                if ( ( ec = cond->preset ) && IsNotEMarked( ec->trans ))
                  PushCondition( cond );
                Set_Bit( GL_conflictArray, cond->numname );
/*
                if (( cond->postset->nexttrans ))
                  *++stackptr2 = cond;
*/
                *++stackptr2 = cond;
              } /* for */
          } /* if */
      } /* while cond */
    GL_configSize = sum;

    IncEMark();
    InitCStack();
    placeArray = bit_alloc();
    for ( ce = event->preset ; ce ; ce = ce->nextplace )
      {
        PushCondition( ce->place );
      } /* for */
    while ( (cond = PopCondition()) )
      {
        for ( ec = (ev=cond->preset->trans)->postset; ec ; ec = ec->nextplace )
          {
            if ( !IsCMarked( cond = ec->place ))
              Set_Bit( placeArray, ( (Place) cond->inscript )->numname );
          } /* for */
        if ( IsNotEMarked( ev ) )
          {
            EMark( ev );
            for ( ce = ev->preset ; ce ; ce = ce->nextplace )
              {
                ec = ( cond = ce->place )->preset;
                if ( ec && IsNotEMarked( ec->trans ))
                  PushCondition( cond );
              } /* for */
          } /* if */
      } /* while */

    for ( ec = ( (Transition) event->inscript )->postset ; ec ;
                                                           ec = ec->nextplace )
      {
        Set_Bit( placeArray, ec->place->numname );
      } /* for */

    for ( cond = unfold_net->Places ; cond && ! cond->preset ;
                                                       cond = cond->nextplace )
      {
        if ( !IsCMarked( cond ))
          Set_Bit( placeArray, ( (Place) cond->inscript )->numname );
      } /* for */

    hash_index_bit( placeArray );

    GL_stackC2ptr = stackptr2;

    if ( ( ce = event->preset ) && ( ec = ce->place->preset ) )
      {
        bitMarking = bit_copy( (BIT_ARRAY) ec->trans->preset->weight );
        while ( (ce = ce->nextplace) )
          {
            if ( (ec = ce->place->preset) )
              {
                bit_union( bitMarking,(BIT_ARRAY)ec->trans->preset->weight );
              } /* if */
            else
              {
                bit_union( bitMarking, GL_BitStartMarking );
              } /* else */
          } /* while */
      } /* if */
    else
      {
        bitMarking = bit_copy( GL_BitStartMarking );
      } /* else */
/*
    printf( "VORHER     : " );
    bit_print_name( bitMarking, GL_net );
*/
    event = (Transition) event->inscript;
    for ( ce = event->preset ; ce ; ce = ce->nextplace )
      {
        Reset_Bit( bitMarking, ce->place->numname );
      } /* for */
/*
    printf( "Nach Reset : " );
    bit_print_name( bitMarking, GL_net );
*/
    bit_difference( bitMarking, GL_BitStartMarking );
    for ( ec = event->postset ; ec ; ec = ec->nextplace )
      {
        Set_Bit( bitMarking, ec->place->numname );
      } /* for */
/*
    printf( "ueber Mark.: " );
    bit_print_name( bitMarking, GL_net );
*/
    printf( "ohne M0    : " );
    bit_print_name( bitMarking, GL_net );
    printf( "ueber Konf.: " );
    bit_print_name( (BIT_ARRAY) placeArray, GL_net );
    printf("\n");
Free( bitMarking );

    return ( (LIST) placeArray );

  } /* FinalState_MARKING() */

