/*****************************************************************************/
/*  global.h                                                                 */
/*  Mo.,17.10.1994 - Do.,20.10.1994                            Stefan Roemer */
/*****************************************************************************/

#ifndef __GLOBAL_H__
#define __GLOBAL_H__

#include <stdlib.h>
#include <time.h>
#include "neunetze.h"
#include "memlib.h"
#include "bitarray.h"
#include "list.h"

/*****************************************************************************/

#define PROGRAM "Unfold"
#define VERSION "5.1"
#define DATE "20.02.2002"

#define PAGEWIDTH 75

/*****************************************************************************/
/* Global type definitions.                                                  */

typedef Place Condition;
typedef Transition Event;

/*****************************************************************************/
/* Beschriftete, markierte Netze, geaenderte Bedeutungen gegenueber neunetze */
/*****************************************************************************/
/*
                                     Stelle bzw. Bedingung
typedef struct PlElem_               ---------------------

  {

    struct PlElem_ * prevplace,      Zeiger auf vor dieser liegende Stelle.
                   * nextplace;      Zeiger auf die nachfolgende Stelle.
    struct TSElem_ * preset;         Zeiger auf die Liste der eingehenden Kanten.
    struct STElem_ * postset;        Zeiger auf die Liste der ausgehenden Kanten.
    char           * name,          !conflArrayptr ?!
                                    !Cond: n.u.
                   * inscript;      !Liste von zugehoerigen Bedingungen
                                    !Cond: Zeiger auf Originalstelle
    Cardinal32     numname;          Fortlaufende Nummer bzgl. der Stellenliste
    PLabel         label;            Beschriftung dieser Stelle:
                                     ENTRY/EMPTY/EXIT/...
                                    !Cond: n.u.
    Cardinal32     component,        Angabe, in welcher Komponente sich die
                                     Stelle befindet.
                                    !Cond:Markierungs-Flag
                   marks,            Anzahl der Marken auf dieser Stelle.
                                    !Cond:Zeiger auf zuletzt eingefuegte C->E-Kante
    Cardinal32     tokenage;         Alter des Token auf dieser Stelle

  } PlElem,                          Verbundtyp 'Stelle'.
    * Place;                         Zeiger auf 'Stelle'.


                                     Transition bzw. Ereignis
typedef struct TrElem_               ------------------------

  {

    struct TrElem_ * prevtrans,     !Anzahl Kanten im Nachbereich
                                    !Even:ConflictArray
                   * nexttrans;      Zeiger auf die nachfolgende Transition.
    struct STElem_ * preset;         Zeiger auf die Liste der eingehenden Kanten.
    struct TSElem_ * postset;        Zeiger auf die Liste der ausgehenden Kanten.
    char           * name,           Transitionenname: Zeiger (!) auf String.
                                    !Even:Markierungs-Flag
                   * inscript;      !Anzahl Kanten im Vorbereich
                                    !Even:Zeiger auf Transition im Originalnetz
    Cardinal32     numname;          Fortlaufende Nummer bzgl. der Listenordnung
    MultiSet       label;            Beschriftung dieser Transition:
                                     Zeiger auf eine Multimenge ueber den
                                     Aktionsnamen.
                                    !Even:LocalConfigDepth
    Time_stamp     * time_label;     Zeitbeschriftung der Transition
                                    !Even:Vorbereich, falls TIC-Event

  } TrElem,                          Verbundtyp 'Transition'.
    * Transition;                    Zeiger auf 'Transition'.


                                     Kante Stelle-->Transition
typedef struct STElem_               -------------------------

  {

    struct STElem_ * prevplace,      Zeiger auf davorliegende S/T-Kante in
                                     bezug auf die Stellenliste.
                   * nextplace,      Zeiger auf die nachfolgende S/T-Kante
                                     bezueglich der Stellenliste.
                   * prevtrans,      Zeiger auf die vorangehende S/T-Kante
                                     bzgl. der Transitionenliste.
                   * nexttrans;      Zeiger auf die naechste S/T-Kante in
                                     Richtung der Transitionenliste.
    Place          place;            Zeiger auf Stelle, von der diese ausgeht.
    Transition     trans;            Zeiger auf Transition, in die diese fuehrt.
    Cardinal32     weight;           Kantengewicht S->T, >= 0.

  } STElem,                          Verbundtyp 'Stellen-Transitionen-Kante'.
    * STEdge;                        Zeiger auf 'Stellen-Transitionen-Kante'.


                                     Kante Transition-->Stelle
typedef struct TSElem_               -------------------------

  {

    struct TSElem_ * prevplace,      Zeiger auf die dieser Kante vorausge-
                                     hende T/S-Kante bzgl. der Stellenliste.
                   * nextplace,      Zeiger auf die nachfolgende T/S-Kante
                                     bzgl. der Stellenliste.
                   * prevtrans,      Zeiger auf die dieser Kante vorausge-
                                     hende T/S-Kante bzgl. der Trans.-Liste.
                   * nexttrans;      Zeiger auf die nachfolgende T/S-Kante
                                     bzgl. der Transitionenliste.
    Place          place;            Zeiger auf Stelle, in die diese fuehrt.
    Transition     trans;            Zeiger auf Transition, aus der diese kommt.
    Cardinal32     weight;           Kantengewicht T->S, >= 0.

  } TSElem,                          Verbundtyp 'Transitionen-Stellen-Kante'.
    * TSEdge;                        Zeiger auf 'Transitionen-Stellen-Kante'.


                                     Kopf eines Netzes
typedef struct                       -----------------

  {

    Place      Places,               Liste der Stellen (Bedingungen):
                                     Zeiger auf die erste Stelle der Liste.
               LastPlace;            Zeiger auf die letzte Stelle der Liste.
    Transition Transitions,          Liste der Transitionen (Ereignisse):
                                     Zeiger auf die erste Transition.
               LastTransition;       Zeiger auf die letzte Transition.
    Cardinal32 NumPlaces,            Anzahl der Stellen im Netz.
               NumTransitions,       Anzahl der Transitionen im Netz.
               NumComponents;        Anzahl der zusammenhaengenden Komponen-
                                     ten des Netzes.

  } LMPNetHead,                      Verbundtyp 'Kopf des Netzes'.
    * LMPNet;                        Zeiger auf 'Kopf eines Netzes'.
                                     LMPNet steht fuer
                                     Labelled and Marked Petri Net           */
/*****************************************************************************/
/*                                                                          */
/* ec->weight    : ( jeweils erster Kantenknoten bzgl. Bedingungs-Liste )   */
/*                 Zeiger auf zuletzt eingefuegte E->C-Kante                */
/*                                                                          */
/* ce->weight    : ( jeweils erster Kantenknoten bzgl. Bedingungs-Liste )   */
/*                 Zeiger auf Final-State-Markierung des Ereignisses.       */
/*                                                                          */
/****************************************************************************/

/****************************************************************************/
/* defines for user options, use with GL_flags. Don't use bits twice!       */
#define SORT_BEGIN          4
#define LAST_CORR_EVENT     8
#define REDUCED_UNF        16
#define USE_BITARRAYS      32
#define CHECK_CONFIGS     256
#define PRINT_PROGRESS    512
#define PROGRESS_NEWLINE 1024
#define USELESS_TRANS    2048
#define WITH_TIME        4096
#define BETTER_TIME      8192

/* Global variables ( prefix 'GL_' indicates 'global' ).                    */
extern Event       * GL_stackE;
extern Condition   * GL_stackC,
                   * GL_stackC2,
                   * GL_stackC2ptr;
                   /* Das entfaltete Netz (entspricht Fin)            */
extern LMPNet      GL_net,
                   GL_unfold_net;
                   /* Wird wohl cut-off entsprechen */
extern LIST        GL_cutoffList,
                   GL_cutoffCorrespondingList,
                   * GL_listArray,
                   /* Diese Zeiger zeigen i.A. auf das 1.,2.,4. Feld */
                   /* von GL_listArray */
                   * GL_listArray1,
                   * GL_listArray2,
                   * GL_listArray4;
extern Cardinal32  GL_maxTpreset,
                   GL_maxTpostset,
                   GL_maxEvents,
                   GL_numCutoffs,
                   GL_changeAlgo,
                   GL_configSize,
                   /* Anzahl Conditions in Entfaltung */
                   GL_numCFlag,
                   GL_EV_equalities;
extern Cardinal32  GL_flags;
extern BIT_ARRAY   GL_conflictArray,
                   GL_BitStartMarking;
extern char        * GL_UselessTransitions,
                   /* Anzahl Events in Entfaltung */
                   * GL_numEFlag;

/****************************************************************************/
/* Global defines/constants.                                                */

#define EVENTS_TO_CHANGE_ALGO 3000
#define MAX_NUMBER_OF_EVENTS 10000

/* The following number must be divisible by 8 ! */
#define DYN_STACKSIZE 2000

/* define if Cutoffs should be printed to stdout */
#undef LOG_THE_CUTOFFS

/*****************************************************************************/

#endif /* __GLOBAL_H__ */
