/*****************************************************************************/
/*                  O P E R A T I O N S   O N   L I S T S                    */
/* Aenderung 10.6.99: list.h erstellt und Anpassungen gemacht      -C. Stehno*/
/*****************************************************************************/

#include "list.h"
#include "global.h"

static Cardinal32 GL_LISTsize = 0;
  /* Um nicht Speicher fuer jedes Element anzufordern   */
  /* wird viel auf einmal reserviert und hier verwaltet */
static LIST       GL_freeList = NULL;

/****************************************************************************/
/* Ausgabe der Listengroesse                                                */

#if defined(__STDC__) || defined(__cplusplus)

Cardinal32 list_getsize( void )

#else

Cardinal32 list_getsize( void )

#endif

{
  return GL_LISTsize;
} 


/****************************************************************************/
/* Anlegen einer Liste vom Typ 'LIST' im Speicher.                          */

#if defined(__STDC__) || defined(__cplusplus)

LIST allocLIST( void )

#else

LIST allocLIST()

#endif

  {

    register LIST newList;

    if ( (newList = GL_freeList) )
      {
        GL_freeList = newList->next;
      } /* if */
    else
      {
	newList = (LIST) MYmalloc( sizeof(LIST_ELEM) );
        GL_LISTsize++;
      } /* else */

    newList->item=NULL;
    newList->tokenage=0;
    newList->next=NULL;

    return ( newList );

  } /* allocLIST() */


/****************************************************************************/
/* Entfernen der globalen Liste aus dem Speicher.                           */

#if defined(__STDC__) || defined(__cplusplus)

void freeLIST( void )

#else

void freeLIST()

#endif

{
  list_delete_mem( &GL_freeList );
}


/****************************************************************************/
/* Entfernen einer Liste vom Typ 'LIST' aus dem Speicher.                   */

#if defined(__STDC__) || defined(__cplusplus)

void list_delete_mem( LIST * list )

#else

void list_delete_mem( list )

  LIST * list; /* call-by-reference ! */

#endif

  {

    register LIST del;

    for ( del = *list ; *list ; del = *list )
      {
	*list = (*list)->next;
	Free( del );
      }

  } /* list_delete_mem() */


/****************************************************************************/
/* Entfernen einer Liste vom Typ 'LIST', freier Speicher wird vom Programm  */
/* verwaltet (via GL_freeList)                                              */
#if defined(__STDC__) || defined(__cplusplus)

void list_delete( LIST * list )

#else

void list_delete( list )

  LIST * list; /* call-by-reference ! */

#endif

  {

    register LIST li,del;

    for ( li = *list ; li ; skip )
      {
	del=li;
	li=li->next;
	del->next = GL_freeList;
	GL_freeList = del;
      }
    *list=NULL;

  } /* list_delete() */

/****************************************************************************/
/* Test auf Gleichheit zweier Listen.                                       */

#if defined(__STDC__) || defined(__cplusplus)

bool lists_are_equal( LIST list1, LIST list2 )

#else

bool lists_are_equal( list1, list2 )

  LIST list1;
  LIST list2;

#endif

  {

    register LIST l1,
                  l2;

    for ( l1 = list1, l2 = list2 ; 
          l1 && l2 && ( l1->item == l2->item ) ;
	  l1 = l1->next, l2 = l2->next )
      skip;

    return ( (bool) ! ( l1 || l2 ) );

  } /* lists_are_equal() */


/****************************************************************************/
/* Check whether a list contains an element (modulo place-equivalence).     */

#if defined(__STDC__) || defined(__cplusplus)

bool list_contains( LIST list, Place element )

#else

bool list_contains( list, element )

  LIST list;
  Place element;

#endif

  {
    while (list && ((Place)list->item->inscript)!=element)
      list=list->next;
    return ((bool)list);
  }

/****************************************************************************/
/* Check whether a list is included in another.                             */

#if defined(__STDC__) || defined(__cplusplus)

bool list_included( LIST list1, LIST list2 )

#else

bool list_included( list1, list2 )

  LIST list1;
  LIST list2;

#endif

  {

    register LIST l1,
                  l2;

    for ( l1 = list1, l2 = list2 ; 
	  l1 && l2 && ( l1->item->numname >= l2->item->numname ) ; 
	  l2 = l2->next )
      {
        if ( l1->item == l2->item )
          l1 = l1->next;
      } /* for */

    return ( (bool) ! l1 );

  } /* list_included() */


/****************************************************************************/
/* Check whether a reversed list is included in another.                    */

#if defined(__STDC__) || defined(__cplusplus)

bool rev_list_included( LIST list1, LIST list2 )

#else

bool rev_list_included( list1, list2 )

  LIST list1;
  LIST list2;

#endif

  {

    register LIST l1,
                  l2;

    for ( l1 = list1, l2 = list2 ; 
	  l1 && l2 && ( l1->item->numname <= l2->item->numname ) ; 
	  l2 = l2->next )
      {
        if ( l1->item == l2->item )
          l1 = l1->next;
      } /* for */
    return ( (bool) ! l1 );

  } /* rev_list_included() */

/****************************************************************************/
/* Check if all transitions not fireable but activated have same I(t) for   */
/* both marking lists                                                       */
/****************************************************************************/

static bool check_notfireable(LIST list1, LIST list2)
{
  register LIST l1, ptr1, l2, ptr2, l3;
  LIST translist_ena1=NULL,translist_ena2=NULL,translist_nf=NULL;
  register STEdge st1, st2;
/* wait_sure means marking will wait this long */
/* wait_need means marking has to stay this long for correctness */
  register Cardinal32 wait_needed=0;
  register Cardinal32 wait_sure1=MAX_INT, wait_sure2=MAX_INT;
  register bool difference=False;

  /* list1 and list2 are same markings, so same length */
  for (l1=list1,l2=list2;l1;l1=l1->next,l2=l2->next)
  { /* check all places for transitions */
    for (st1=l1->item->postset;st1;st1=st1->nexttrans)
      { /* check all transitions */
	register Cardinal32 clock1=MAX_INT, clock2=MAX_INT;
	for (st2=st1->trans->preset, ptr1=list1, ptr2=list2; st2; 
             st2=st2->nextplace)
	  { /* search for correct preset. skip already checked */
	    /* by initalizing ptr1,2 before. presets are ordered! */
	    while (ptr1 && (ptr1->item)!=st2->place)
	      ptr1=ptr1->next,ptr2=ptr2->next;
	    /* if found and transition may wait */
	    if (ptr1)
	      {
		clock1=Min(clock1,ptr1->tokenage);
		clock2=Min(clock2,ptr2->tokenage);
	      }
	  } /* for st2 */
	/* if transition activated */
	if (!st2)
	  {
	    if (st1->trans->time_label)
	      {
		if (clock1<st1->trans->time_label->low ||
		    clock2<st1->trans->time_label->low)
		  {
	            if (clock1!=clock2)
		      return 0;
		    wait_needed=Max(wait_needed,
                      st1->trans->time_label->low-clock1);
		    list_insert(&translist_nf,(Place)st1->trans,st1->trans->time_label->low-clock1);
		  }
		if (clock1!=clock2)
		  difference=True;
		wait_sure1=Min(wait_sure1,
				st1->trans->time_label->high-clock1);
                wait_sure2=Min(wait_sure2,
				st1->trans->time_label->high-clock2);
	      }
	    else
	      {
	      wait_sure1=wait_sure2=0;
	      }
	    list_insert(&translist_ena1,(Place)st1->trans,st1->trans->time_label->low-clock1);
	    list_insert(&translist_ena2,(Place)st1->trans,st1->trans->time_label->low-clock2);
	  } /* if activated */
      } /* for st1 */
  } /* for l1,l2 */
  for (l1=translist_nf; l1; l1=l1->next)
    {
#if 1
      for(l2=translist_ena1,l3=translist_ena2;l2;l2=l2->next,l3=l3->next)
	if ((l2->tokenage<l1->tokenage)!=(l3->tokenage<l1->tokenage))
	  {
	  return 0;
	  }
#else
      if ((wait_sure1<l1->tokenage)!=(wait_sure2<l1->tokenage))
	return 0;
#endif
    }
  return 1;

  /* set to 0 if only clock vector checking needed */
#if 0
  if ((wait_sure1<wait_needed)!=(wait_sure2<wait_needed))
#endif
#if 0
  if (wait_sure1!=wait_sure2)
#else
  if (difference&&wait_needed)
#endif
    return 0;
  else
    return 1;
}

/****************************************************************************/
/* Compare two lists: list1<list2 (-1), list1==list2 (0) or list1>list2 (1) */
/* Comparison is based on the numeric values of list->item                  */
/* list->item is sorted as the whole place list is allocated at once, so    */
/* places with small place number also have small mem pointers              */
/* Changes 23.6.1999: Made the result calculation more obvious           -CS*/

#if defined(__STDC__) || defined(__cplusplus)

int list_compare( LIST list1, LIST list2 )

#else

int list_compare( list1, list2 )

  LIST list1;
  LIST list2;

#endif

{

  register LIST l1,l2;
  register int compared;

  /* skip all entries which are equal */
  for ( l1 = list1, l2 = list2;
	l1 && l2 && ( l1->item == l2->item ) && 
	  (l1->tokenage == l2->tokenage);
	l1 = l1->next, l2 = l2->next )
    skip;

  /* if end is reached lists are equal */
  if ( ! (l1 || l2) ) /* l1==l2==NULL */
    return 0;
  else
    {
      if (!(GL_flags & BETTER_TIME))
	{
	  /* sonst wenn l1 zu ende ist l2 groesser */
	  if(!l1)
	    return -1;
	  else
	    /* sonst ist entweder l2 zu ende (return 1) */
	    /* oder das letzte Element entscheidet      */
	    return ( ( l2 && 
		       ( ( l1->item < l2->item )
			 || ((l1->item==l2->item)&&(l1->tokenage<l2->tokenage))))
		     ? -1 : 1 );
	}/* if !BETTER_TIME */
      else /* time unfolding optimised */
	{
	  /* if same places modulo timing */
          while(l1&&l2&&l1->item==l2->item)
	    {
	      l1=l1->next;
	      l2=l2->next;
	    }
	  if ((compared=!(l1||l2)))
	    {
	      /* check if the same transistions are activated */
	      compared=check_notfireable(list1, list2);
	    }
	  return !compared;
	} /* else BETTER_TIME */
    }
} /* list_compare() */


/****************************************************************************/
/* Insert an element in a list.                                             */

#if defined(__STDC__) || defined(__cplusplus)

void list_insert( LIST * list, Place item, Cardinal32 tokenage )

#else

void list_insert( list, item, tokenage )

  LIST  * list; /* call-by-reference !!! */
  Place item;
  Cardinal32 tokenage;

#endif

  {

    register LIST       listptr,
                        newlist=*list;
    register Cardinal32 numname = item->numname;

    /* Wenn neue Liste oder Stelle vorne anzufuegen */
    if ( !(listptr = *list)
         || ( numname < listptr->item->numname ) 
	 || ((numname==listptr->item->numname) &&
	     tokenage<listptr->tokenage))
      {
        newlist = allocLIST();
        newlist->next = listptr;
        *list = newlist;
      } /* if */
    /* Wenn Element nicht vorher einzufuegen testen, */
    /* ob nicht schon vorhanden als erstes Element   */
    else if ( (item != listptr->item) || (tokenage!=listptr->tokenage) )
      {
        /* Stelle zur Einfuegung suchen */
        while ( ( newlist = listptr->next ) &&
                (( newlist->item->numname < numname ) ||
		 (newlist->tokenage<tokenage)) )
          {
            listptr = newlist;
          } /* while */
        /* Wenn nicht vorhanden einfuegen */
        if ( ! ( newlist && ( newlist->item == item ) && 
		 (newlist->tokenage==tokenage) ) )
          {
            newlist = allocLIST();
            newlist->next = listptr->next;
            listptr->next = newlist;
          } /* if */
      } /* else */
    newlist->item = item;
    newlist->tokenage = tokenage;

  } /* list_insert() */


/****************************************************************************/
/* Insert an element in a reverse ordered list.                             */

#if defined(__STDC__) || defined(__cplusplus)

void rev_list_insert( LIST * list, Place item )

#else

void rev_list_insert( list, item )

  LIST  * list; /* call-by-reference !!! */
  Place item;

#endif

  {

    register LIST       listptr,
                        newlist;
    register Cardinal32 numname = item->numname;

    /* Wenn neue Liste oder Stelle vorne anzufuegen */
    if ( ( ! ( listptr = *list ) )
         || ( numname > listptr->item->numname ) )
      {
        ( newlist = allocLIST() )->item = item;
        newlist->next = listptr;
        *list = newlist;
      } /* if */
    /* Wenn Element nicht vorher einzufuegen testen, */
    /* ob nicht schon vorhanden als erstes Element   */
    else if ( item != listptr->item )
      {
        /* Wenn nicht, Stelle zur Einfuegung suchen */
        while ( ( newlist = listptr->next ) &&
                ( newlist->item->numname > numname ) )
          {
            listptr = newlist;
          } /* while */
        /* Wenn nicht vorhanden einfuegen */
        if ( ! ( newlist && ( newlist->item == item ) ) )
          {
            ( newlist = allocLIST() )->item = item;
            newlist->next = listptr->next;
            listptr->next = newlist;
          } /* if */
      } /* else if */

  } /* rev_list_insert() */


/****************************************************************************/
/* Copy a reverse ordered list without taking those elements included in    */
/* the list 'diffList'.                                                     */

#if defined(__STDC__) || defined(__cplusplus)

LIST rev_list_copy_diff( LIST copyList, LIST diffList )

#else

LIST rev_list_copy_diff( copyList, diffList )

  LIST copyList,
       diffList;

#endif

  {

    register LIST copyptr,
                  diffptr,
                  newList = NULL,
                  newptr  = NULL; /* to please C++ */

    for ( copyptr = copyList, diffptr = diffList ; copyptr && diffptr ;
                                                     copyptr = copyptr->next )
      {
        while ( diffptr && ( diffptr->item->numname>copyptr->item->numname ) )
          {
            diffptr = diffptr->next;
          } /* while */
        if ( diffptr && ( diffptr->item == copyptr->item ) )
          {
            diffptr = diffptr->next;
          } /* if */
        else
          {
            if ( newList )
              {
                ( newptr = newptr->next = allocLIST() )->next = NULL;
              } /* if */
            else
              {
                ( newList = newptr = allocLIST() )->next = NULL;
              } /* else */
            newptr->item = copyptr->item;
          } /* else */
      } /* for */
    if ( copyptr && ! newList )
      {
        ( newList = newptr = allocLIST() )->next = NULL;
        newptr->item = copyptr->item;
        copyptr = copyptr->next;
      } /* if */
    for ( skip ; copyptr ; copyptr = copyptr->next )
      {
        ( newptr = newptr->next = allocLIST() )->next = NULL;
        newptr->item = copyptr->item;
      } /* for */

    return ( newList );

  } /* rev_list_copy_diff() */


/****************************************************************************/
/* Reverse a list.                                                          */

#if defined(__STDC__) || defined(__cplusplus)

void list_reverse( LIST * list )

#else

void list_reverse( list )

  LIST * list;

#endif

  {

    register LIST listptr,
                  revlist;

    if ( (revlist = (listptr = *list)) )
      {
        listptr = listptr->next;
        revlist->next = NULL;
        while ( listptr )
          {
            *list = listptr->next;
            listptr->next = revlist;
            revlist = listptr;
            listptr = *list;
          } /* while */
      } /* if */

    *list = revlist;

  } /* list_reverse() */


/****************************************************************************/
/* Display a list as a set of conditions ( respectively events ) on the     */
/* screen.                                                                  */

#if defined(__STDC__) || defined(__cplusplus)

void list_print( LIST list )

#else

void list_print( list )

  LIST list;

#endif

  {

    printf( "{" );
    while ( list )
      {
        printf( "%u_%u", list->item->numname, list->tokenage );
        list = list->next;
        printf( "%s",list?",":"}\n" );
      } /* if */

  } /* list_print() */

