/*****************************************************************************/
/* EVunfold.c                                                                */
/* Improved McMillan unfoldings by Esparza/Vogler                            */
/*****************************************************************************/
/* IMPORTANT:                                                                */
/* This file requires 'neunetze' version 1.7.3 (09.01.1995) or higher !!!    */
/*****************************************************************************/
/*      Entfalten 1-sicherer Petrinetze nach dem McMillan-Algorithmus        */
/*****************************************************************************/
/* Version 4.0.4: Stefan Roemer, Mo.,11.04.1994 - Fr.,16.08.1996.            */
/* Korrekturen gegenueber 'mausi': Heuristik 2 eingefuehrt;                  */
/*                                 Kardinalitaet der Konfigurationen;        */
/*                                 Bedingungen im Nachbereich von cutoff-    */
/*                                         Ereignissen spaeter einfuegen;    */
/*                                 doppelte Konfigurationen entfernt.        */
/* Eigene Version von GenEvents().                                           */
/* Konflikt-Bit-Arrays.                                                      */
/* MCI-Dateiausgabe.                                                         */
/* Bit-Arrays nur bei GL_maxTpreset > 2                                      */
/* Bit-Arrays ausschaltbar fuer groessere Entfaltungen (Option -n).          */
/* Bit-Arrays fuer Markierungen in Hash-Liste moeglich (Option -b).          */
/* Statistik-Ausgabe (Option -s).                                            */
/* Konsistenztest fuer maximale Konfigurationen (Option -#). (25.10.1994)    */
/* "Externe" Berechnung maximaler Konfigurationen (Option -k).(20.10.-25.10.)*/
/* Entfernen von Transitionen, die an Kanten mit Gewicht > 1 haengen (02.11.)*/
/* Korrektur der fehlerhaften Berechnung max. Konfigurationen (08.11./30.01) */
/* Final States aus Markierungen berechnen (17.11.-17.11.1994).              */
/* Berechnen von 'reduced unfoldings' [Alex] (01.12.1994) (Option -r).       */
/* Jeweils die ersten korrespondierenden Ereignisse benutzen (Option -l).    */
/* Verbesserungen von Esparza/Vogler eingearbeitet (08.05.-22.06).           */
/* Beschraenkung der Zahl zu generierender Ereignisse (07.06.95) (Option -x).*/
/* Option -M                                                                 */
/* Option -T                                                                 */
/* Option -F: Einlesen von Communicatings FSAs                               */
/* 05.03.96: Entfernen isolierter Stellen nach Rausschmiss toter Trans.(T++) */
/* 23.06.1999: Aufteilung in mehrere Files                  -Christian Stehno*/
/*****************************************************************************/
/* TO DO: Seitenbegrenzung auch fuer Nachbereiche bei UNF-Format.            */
/* TO DO: Sortieren der Konfigurationen.                                     */
/* EVTL.: Final States aus Markierungen berechnen.                           */
/* EVTL.: Maximale Konfigurationen in externe Datei auslagern.               */
/* EVTL.: In der Hash-Tabelle nur Events ablegen (wie McMillan).             */
/*****************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include "convertPEP.h"
#include "rw_fsa.h"

#include "global.h"
#include "hash.h"
#include "sub_unf.h"
#include "unfold.h"
#include "cutoff_mm.h"
#include "cutoff_ev.h"
#include "cutoff_erv.h"
#include "cutoff_ev2.h"
#include "nets.h"
#include "poss_ext.h"
#include "unf_out.h"
#include "rw_inf.h"

/****************************************************************************/
/* Global variables ( prefix 'GL_' indicates 'global' ).                    */

Event             * GL_stackE;
LMPNet            GL_net,
                  GL_unfold_net;
LIST              GL_cutoffList,
                  GL_cutoffCorrespondingList,
                  * GL_listArray,
                  * GL_listArray1,
                  * GL_listArray2,
                  * GL_listArray4;
Cardinal32        GL_maxTpreset,
                  GL_maxTpostset,
                  GL_maxEvents,
                  GL_numCutoffs,
                  GL_changeAlgo,
                  GL_configSize,
                  GL_numCFlag,
                  GL_EV_equalities = 0;
bool              GL_ParikhInAdvance;
Cardinal32        GL_flags=0; /* set all values to false */
BIT_ARRAY         GL_conflictArray,
                  GL_BitStartMarking;
char              * GL_UselessTransitions = NULL,
                  * GL_numEFlag;
Condition         * GL_stackC;
Condition         * GL_stackC2,
                  * GL_stackC2ptr;

#define Error(funct,msg,code) {\
fprintf( stderr,\
         "ERROR: %s.%s(): %s\n", PROGRAM, funct, msg );\
exit ( code );}

/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static void clean_up( LMPNet unfold_net, LMPNet net )

#else

static void clean_up( unfold_net, net )

  LMPNet unfold_net;
  LMPNet net;

#endif

  {

    register Condition  cond;
    register Event      event;
    register Cardinal32 count;

    count = hash_size();
    if ( GL_flags & USE_BITARRAYS )
      {
/* !!!!!!!!!!!!!!!!!!!!!!!!!!!! NOCH ZU TUN !!!!!!!!!!!!!!!!!!!!!!!!!!!! */
      } /* if */
    else
      {
        while ( count )
          {
            hash_delete( GL_hashTable[ --count ] );
          } /* while */
      } /* else */
    Free( GL_hashTable );
    list_delete_mem( &GL_cutoffList );
    list_delete_mem( &GL_cutoffCorrespondingList );
    /* free globally allocated mem */
    freeLIST();
    for ( cond = unfold_net->Places ; cond ; cond = cond->nextplace )
      {
        cond->marks = 0;
        cond->label = ( (Place) cond->inscript )->label;
        cond->component = ( (Place) cond->inscript )->component;
        cond->inscript = ( (Place) cond->inscript )->name;
      } /* for */
    if ( (event = unfold_net->Transitions) )
      {
        event->prevtrans = NULL;
        for ( skip ; event ; event = event->nexttrans )
          {
            if (event->nexttrans)
              event->nexttrans->prevtrans = event;
            if (event->inscript)
              {
                event->label = ( (Transition) event->inscript )->label;
                event->inscript = ( (Transition) event->inscript )->name;
              }
          } /* for */
      } /* if */
    if ((event = net->Transitions))
      {
        event->prevtrans = NULL;
        for ( skip ; event ; event = event->nexttrans )
          {
            event->inscript = NULL;
            if (event->nexttrans)
              event->nexttrans->prevtrans = event;
          } /* for */
      } /* if */
    for ( cond = net->Places ; cond ; cond = cond->nextplace )
      {
        cond->inscript = NULL;
      } /* for */

  } /* clean_up() */


/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static void statistics( LMPNet unfold_net, LMPNet net )

#else

static void statistics( unfold_net, net )

  LMPNet unfold_net;
  LMPNet net;

#endif

  {

    register Place      pl;
    register Transition tr;
    register STEdge     st;
    register TSEdge     ts;
    register Cardinal32 num1,
                        num2,
                        num3,
                        count;
    register u_char     size;
    register LIST       list;
    register HASH       hash;
    register char       * ptr;
    Cardinal32          numST,
                        numTS,
                        numBE,
                        numEB,
                        memSTdiv,
                        memBEdiv;
    u_char              sizePl,
                        sizeTr,
                        sizeST,
                        sizeTS;

    if ( ( size = (u_char) sizeof( PlElem ) ) & 7 )
      {
        size &= (u_char) -8;
        size += (u_char) 8;
      }
    sizePl = size + (u_char) 8;
    if ( ( size = (u_char) sizeof( TrElem ) ) & 7 )
      {
        size &= (u_char) -8;
        size += (u_char) 8;
      }
    sizeTr = size + (u_char) 8;
    if ( ( size = (u_char) sizeof( STElem ) ) & 7 )
      {
        size &= (u_char) -8;
        size += (u_char) 8;
      }
    sizeST = size + (u_char) 8;
    if ( ( size = (u_char) sizeof( TSElem ) ) & 7 )
      {
        size &= (u_char) -8;
        size += (u_char) 8;
      }
    sizeTS = size + (u_char) 8;
    num1 = num2 = 0;
    for ( pl = net->Places ; pl ; pl = pl->nextplace )
      {
        for ( ts = pl->preset ; ts ; num2++, ts = ts->nexttrans )
          skip;
        for ( st = pl->postset ; st ; num1++, st = st->nexttrans )
          skip;
      } /* for */
    numST = num1;
    numTS = num2;
    num1 = num2 = 0;
    for ( pl = unfold_net->Places ; pl ; pl = pl->nextplace )
      {
        if (pl->preset)
          num2++;
        for ( st = pl->postset ; st ; num1++, st = st->nexttrans )
          skip;
      } /* for */
    numBE = num1;
    numEB = num2;
    num1 = 0;
    for ( pl = net->Places ; pl ; pl = pl->nextplace )
      {
        for ( num2 = 1, ptr = pl->name ; *( ptr++ ) ; num2++ )
          skip;
        if ( num2 & 7 )
          {
            num2 &= (Cardinal32) -8;
            num2 += (Cardinal32) 8;
          }
        num1 += num2 + (u_char) 8;
      } /* for */
    memSTdiv = num1;
    num1 = 0;
    for ( tr = net->Transitions ; tr ; tr = tr->nexttrans )
      {
        for ( num2 = 1, ptr = tr->name ; *( ptr++ ) ; num2++ )
          skip;
        if ( num2 & 7 )
          {
            num2 &= (Cardinal32) -8;
            num2 += (Cardinal32) 8;
          }
        num1 += num2 + (u_char) 8;
/* Hier evtl. noch den Speicherbedarf der Multimengen hinzuaddieren */
      } /* for */
    memSTdiv += num1;
    num1 = 0;
    for ( pl = unfold_net->Places ; pl ; pl = pl->nextplace )
      {
        for ( num2 = 1, ptr = pl->name ; *( ptr++ ) ; num2++ )
          skip;
        if ( num2 & 7 )
          {
            num2 &= (Cardinal32) -8;
            num2 += (Cardinal32) 8;
          }
        num1 += num2 + (u_char) 8;
      } /* for */
    memBEdiv = num1;
    num1 = 0;
    for ( tr = unfold_net->Transitions ; tr ; tr = tr->nexttrans )
      {
        for ( num2 = 1, ptr = tr->name ; *( ptr++ ) ; num2++ )
          skip;
        if ( num2 & 7 )
          {
            num2 &= (Cardinal32) -8;
            num2 += (Cardinal32) 8;
          }
        num1 += num2 + (u_char) 8;
      } /* for */
    memBEdiv += num1;
    printf( "GL_maxTpostset = %u, ", GL_maxTpostset );
    printf( "GL_maxTpreset = %u\n", GL_maxTpreset );
    printf( "List elements: %u\n", list_getsize() );
    printf( "* Original system:\n" );
    printf( "  |S|=%6u, |T|=%6u, |ST|=%6u, |TS|=%6u\n",
            net->NumPlaces, net->NumTransitions, numST, numTS );
    printf( "  --> allocated memory: %10u Bytes\n",
                  net->NumPlaces * sizePl + net->NumTransitions * sizeTr
                               + numST * sizeST + numTS * sizeTS + memSTdiv );
    printf( "* Unfolded net:\n" );
/* es gilt: |EB| = |B| - |M0| */
    printf( "  |B|=%6u, |E|=%6u, |BE|=%6u, |EB|=%6u\n",
                      unfold_net->NumPlaces, unfold_net->NumTransitions,
                                                               numBE, numEB );
    printf( "  --> allocated memory: %10u Bytes\n",
                           unfold_net->NumPlaces * sizePl
                         + unfold_net->NumTransitions * sizeTr
                         + numBE * sizeST + numEB * sizeTS + memBEdiv );
    printf( "* Hash table:\n" );
    printf( "  Number of buckets: %u.\n", count=hash_size() );
    if ( ( num1 = count * sizeof( HASH ) ) & 7 )
      {
        num1 &= (Cardinal32) -8;
        num1 += 8;
      }
    num1 += (u_char) 8;
    printf( "  --> allocated memory: %10u Bytes\n", num1 );
    printf( "  Hash table entries: " );
    if ( ( size = sizeof( HASH_ELEM ) ) & 7 )
      {
        size &= (u_char) -8;
        size += (u_char) 8;
      }
    size += (u_char) 8;
    if ( GL_flags & USE_BITARRAYS )
      {
        num1 = bit_getsize();
        if ( num1 & 7 )
          {
            num1 &= (Cardinal32) -8;
            num1 += 8;
          }
        num1 += 8 + size;
        num2 = num3 = 0;
        while ( count )
          {
            if ( (hash = GL_hashTable[ --count ]) )
              {
                num3++;
                num2 += num1;
                while ( (hash = hash->next) )
                  {
                    num3++;
                    num2 += num1;
                  } /* while */
              } /* if */
          } /* while */
        printf( "%u bit arrays ( %u (head) + %u (array) bytes each )\n",
                                            num3, size + 8, num1 - 8 - size );
        printf( "  --> allocated memory: %10u Bytes\n", num2 );
      } /* if */
    else
      {
        if ( ( num1 = sizeof( LIST_ELEM ) ) & 7 )
          {
            num1 &= (Cardinal32) -8;
            num1 += 8;
          }
        num1 += 8;
        num2 = num3 = 0;
        while ( count )
          {
            if ( (hash = GL_hashTable[ --count ]) )
              {
                num3++;
                if ( (list = hash->list) )
                  {
                    num2++;
                    while ( (list = list->next) )
                      {
                        num2++;
                      } /* while */
                  } /* if */
                while ( (hash = hash->next) )
                  {
                    num3++;
                    if ( (list = hash->list) )
                      {
                        num2++;
                        while ( (list = list->next) )
                          {
                            num2++;
                          } /* while */
                      } /* if */
                  } /* while */
              } /* if */
          } /* while */
        printf( "%u linear lists ( %u set list elements )\n",
                                                                 num3, num2 );
        printf( "  --> allocated memory: %10u Bytes\n",
                                                  num3 * size + num2 * num1 );
      } /* else */
    printf( "* Number of cutoff events: %u.\n",
                                                       num1 = GL_numCutoffs );
    if ( ( size = sizeof( LIST_ELEM ) ) & 7 )
      {
        size &= (u_char) -8;
        size += 8;
      }
    size += 8;
    printf( "  --> allocated memory: %10u Bytes\n", ( num1 * size ) << 1 );
    printf( "* Other bit arrays: <...yet missing...>\n" );

  } /* statistics() */

/*****************************************************************************/

static int show_usage(void)
{
  printf( "'%s' -- unfold 1-safe petri nets.\n", PROGRAM );
  printf( "parameters: ( in any order )\n" );
  printf( "  [-f=]<PEPnetfile>  specify input file (Petri net, PEP format)\n" );
  printf( "  -F=<FSAfile>       specify input file (Communicating FSAs)\n" );
  printf( "  -u=<unf_file>      output in '.unf' file format\n" );
  printf( "  -m=<mci_file>      output in '.mci' file format\n" );
  printf( "  -t                 display unfolding time\n" );
  printf( "  -p                 display progress status (one line)\n" );
  printf( "  -q                 display progress status (multiple lines)\n" );
  printf( "  -b                 use bit arrays for hash table entries\n" );
  printf( "  -n                 change B-conflict algorithm after\n" );
  printf( "                     %u events\n", EVENTS_TO_CHANGE_ALGO );
  printf( "  -n=<number>        change B-conflict algorithm after\n" );
  printf( "                     <number> events, number>0\n" );
  printf( "  -x                 restrict the number of maximal\n" );
  printf( "                     generated events to %u\n", MAX_NUMBER_OF_EVENTS );
  printf( "  -x=<number>        restrict the number of maximal\n" );
  printf( "                     generated events to <number>\n" );
  printf( "  -i                 show net ('.inf' format)\n" );
  printf( "  -o                 show unfolded net ('.inf' format)\n" );
  printf( "  -d                 show hash table\n" );
  printf( "  -h                 show help (this text)\n" );
#if 0
/* bislang undokumentierte Flags */
  printf( "  -a                 use bit arrays with Marking\n" );
  printf( "  -r                 Reduce unfolding\n" );
  printf( "  -w                 Parikh in advance\n" );
  printf( "  -y                 change Parikh settings??\n" );
  printf( "  -+                 sort in from beginning\n" );
  printf( "  -@@                ERV_unfold\n" );
  printf( "  -@@@               NERV_unfold\n" );
  printf( "  -@+                EV2_unfold\n" );
  printf( "  -X[+]              use Time expansion unfolding\n" );
  printf( "                     +: use optimised unfolding\n" );
#endif
  printf( "  -@                 switch to Esparza/Vogler unfolding\n" );
  printf( "                       (Default: produce McMillan unfolding)\n" );
  printf( "  -X                 use Time expansion unfolding\n" );
  printf( "  -#                 output processed cuts during calculation\n" );
  printf( "  -s                 display statistics\n" );
  printf( "  -v                 display program version number\n" );
  printf( "  -l                 use last corresponding events\n" );
  printf( "  -M                 use current marking instead of initial\n" );
  printf( "  -T[=<filename>]    print the list of unused transitions\n" );
  printf( "                     if the filename is given, the list will\n" );
  printf( "                     be written to that file\n" );
  exit(1);
} /* show_usage */


/*****************************************************************************/
/* main()                                                                    */

typedef enum { NotSpecified = 0, PN, FSA, STG } InFileType;


#if defined(__STDC__) || defined(__cplusplus)

int main( int argc, char ** argv )

#else

int main( argc, argv )
  int  argc;
  char ** argv;

#endif

  {

    register Cardinal32 count;
    register Place      pl;
    register char       ch,
                        * ptr;
    LMPNet              net,
                        unfold_net;
    Transition          tr;
    STEdge              st;
    TSEdge              ts;
    bool                notremoved,
                        output_ent             = False,
                        output_mci             = False,
                        output_unf             = False,
                        showHash               = False,
                        showSTNet              = False,
                        showUnfolding          = False,
                        showStatistics         = False,
                        showTime               = False,
                        showVersion            = False,
                        showUsage              = False;
    char                inFilename[NAMELENGTH],
                        trFilename[NAMELENGTH],
                        entFilename[NAMELENGTH],
                        mciFilename[NAMELENGTH],
                        unfFilename[NAMELENGTH];
    InFileType          inFileType = NotSpecified;
    FILE                * trfile = NULL;
    static char         null[2] = "\0";

    GL_flags = 0;
    GL_changeAlgo = ~(Cardinal32) 1; /* 0xFFFFFFFF */
    GL_maxEvents = ~(Cardinal32) 1; /* 0xFFFFFFFF */

    *inFilename = *trFilename = '\0';
    unfold = M_unfold;
    FinalState = FinalState_LIST;
    Mark = Mark_LIST;
    IsNotCutoffPoint = IsNotCutoffPoint_LIST;
    IsNotCutoffEvent = IsNotCutoffEvent_LIST;
    GL_ParikhInAdvance = True;
    GL_countQueueCompares = 0;

    GL_Display_Parikh = Display_Parikh_Vector;
    GL_Empty_Parikh = Empty_Parikh_Vector;
    GL_Calculate_Parikh = Calculate_Parikh_Vector;
    GL_Calculate_Parikh_T = Calculate_Parikh_Vector_T;
    GL_Calculate_PSize_T = Calculate_PSize_T;
    GL_Compare_Parikh_A = Compare_Parikh_Vectors_A;
    GL_Compare_Parikh_L = Compare_Parikh_Vectors_L;
    EV_insert_PE_queue_condlist  = insert_PE_queue_condlist;
    EV_insert_PE_queue_condarray = insert_PE_queue_condarray;

    if ( argc > 1 )
      {
        count = 0;
        while ( ++count < argc )
          {
            if ( ( ch = *( ptr = argv[ count ] ) ) == '-' )
              {
                while ( (ch = *++ptr) )
                  {
                    switch ( ch )
                      {
/* OPTION -a IST NOCH FEHLERHAFT !!! */
                        case 'a' : GL_flags |= USE_BITARRAYS;
                                   FinalState = FinalState_MARKING;
/*
                                   Mark = Mark_MARKING;
*/
                                   IsNotCutoffPoint = IsNotCutoffPoint_ARRAY;
                                   ERV_IsNotCutoffPoint = 
				     ((unfold!=NERV_unfold)?
				      ERV_IsNotCutoffPoint_ARRAY:
				      NERV_IsNotCutoffPoint_ARRAY);
				   IsNotCutoffEvent = IsNotCutoffEvent_ARRAY;
                                   break;
                        case 'b' : GL_flags |= USE_BITARRAYS;
                                   FinalState = FinalState_ARRAY;
                                   Mark = Mark_ARRAY;
                                   IsNotCutoffPoint = IsNotCutoffPoint_ARRAY;
                                   IsNotCutoffEvent = IsNotCutoffEvent_ARRAY;
                                   break;
                        case 'e' : if ( *++ptr != '=' )
                                     {
                                       Error("main","'=' expected after '-e'.",
					     11);
                                     } /* if */
                                   if ( output_ent )
                                     {
                                       fprintf( stderr,
                     "WARNING: unfold.main(): parameter '-e' appears twice;" );
                                       fprintf( stderr,
                                                " %*s'%s' overwrites '%s'.\n",
                                                24, "", ptr + 1, entFilename );
                                     } /* if */
                                   strcpy( entFilename, ++ptr );
                                   output_ent = True;
                                   ptr = null;
                                   break;
                        case 'f' : if ( *++ptr != '=' )
                                     {
                                       Error("main","'=' expected after '-f'.",
					     11);
                                     } /* if */
                                   if ( inFileType )
                                     {
                                       fprintf( stderr,
                     "WARNING: unfold.main(): input file already specified;" );
                                       fprintf( stderr,
                                                " %*s'%s' overwrites '%s'.\n",
                                                24, "", ptr + 1, inFilename );
                                     } /* if */
                                   inFileType = PN;
                                   strcpy( inFilename, ++ptr );
                                   ptr = null;
                                   break;
                        case 'h' : showHash = True;
                                   break;
                        case 'i' : showSTNet = True;
                                   break;
                        case 'l' : GL_flags |= LAST_CORR_EVENT;
                                   break;
                        case 'm' : if ( *++ptr != '=' )
                                     {
                                       Error("main","'=' expected after '-m'.",
					     11);
                                     } /* if */
                                   if ( output_mci )
                                     {
                                       fprintf( stderr,
                     "WARNING: unfold.main(): parameter '-m' appears twice;" );
                                       fprintf( stderr,
                                                " %*s'%s' overwrites '%s'.\n",
                                                24, "", ptr + 1, mciFilename );
                                     } /* if */
                                   strcpy( mciFilename, ++ptr );
                                   output_mci = True;
                                   ptr = null;
                                   break;
                        case 'n' : if ( *++ptr == '=' )
                                     {
                                       if ( isdigit( ch = *++ptr ) )
                                         {
                                           GL_changeAlgo = ch - '0';
                                           while ( isdigit( ch = *++ptr ) )
                                             {
                                               GL_changeAlgo *= 10;
                                               GL_changeAlgo += ch - '0';
                                             } /* while */
                                         } /* if */
                                       else
                                         {
                                           Error("main","number expected after '-n='.",11 );
                                         } /* else */
                                     } /* if */
                                   else
                                     {
                                       GL_changeAlgo = EVENTS_TO_CHANGE_ALGO;
                                     } /* else */
                                   ptr--;
                                   break;
                        case 'o' : showUnfolding = True;
                                   break;
                        case 'p' : GL_flags |= PRINT_PROGRESS;
                                   break;
                        case 'q' : GL_flags |= PRINT_PROGRESS | PROGRESS_NEWLINE;
                                   break;
                        case 'r' : GL_flags |= REDUCED_UNF;
                                   break;
/* Kein Break? CS*/     case 's' : showStatistics = True;
                        case 't' : showTime = True;
                                   break;
                        case 'u' : if ( *++ptr != '=' )
                                     {
                                       Error("main","'=' expected after '-u'.",
					     11 );
                                     } /* if */
                                   if ( output_unf )
                                     {
                                       fprintf( stderr,
                     "WARNING: unfold.main(): parameter '-u' appears twice;" );
                                       fprintf( stderr,
                                                " %*s'%s' overwrites '%s'.\n",
                                                24, "", ptr + 1, unfFilename );
                                     } /* if */
                                   strcpy( unfFilename, ++ptr );
                                   output_unf = True;
                                   ptr = null;
                                   break;
                        case 'v' : showVersion = True;
                                   break;
                        case 'w' : GL_ParikhInAdvance = False;
                                   break;
                        case 'x' : if ( *++ptr == '=' )
                                     {
                                       if ( isdigit( ch = *++ptr ) )
                                         {
                                           GL_maxEvents = ch - '0';
                                           while ( isdigit( ch = *++ptr ) )
                                             {
                                               GL_maxEvents *= 10;
                                               GL_maxEvents += ch - '0';
                                             } /* while */
                                         } /* if */
                                       else
                                         {
                                           Error("main","number expected after '-x='.", 11 );
                                         } /* else */
                                       if ( ! GL_maxEvents )
                                         {
                                           Error("main","number greater than 0 expected after '-x='.", 11 );
                                         } /* else */
                                     } /* if */
                                   else
                                     {
                                       GL_maxEvents = MAX_NUMBER_OF_EVENTS;
                                     } /* else */
                                   ptr--;
                                   break;
                        case 'y' : GL_Display_Parikh = Display_Parikh_Array;
                                   GL_Empty_Parikh = Empty_Parikh_Array;
                                   GL_Calculate_Parikh
                                                      = Calculate_Parikh_Array;
                                   GL_Calculate_Parikh_T
                                                    = Calculate_Parikh_Array_T;
                                   GL_Calculate_PSize_T = Calculate_PSize_T;
                                   GL_Compare_Parikh_A
                                                     = Compare_Parikh_Arrays_A;
                                   GL_Compare_Parikh_L
                                                     = Compare_Parikh_Arrays_L;
                                   break;
                        case 'F' : if ( *++ptr != '=' )
                                     {
                                       Error("main","'=' expected after '-F'.",
					     11 );
                                     } /* if */
                                   if ( *inFilename )
                                     {
                                       fprintf( stderr,
                     "WARNING: unfold.main(): input file already specified;" );
                                       fprintf( stderr,
                                                " %*s'%s' overwrites '%s'.\n",
                                                 24, "", ptr + 1, inFilename );
                                     } /* if */
                                   inFileType = FSA;
                                   strcpy( inFilename, ++ptr );
                                   ptr = null;
                                   break;
                        case 'M' : CM_use_imarks = False;
                                   break;
                        case 'T' : if ( *( ptr + 1 ) == '=' )
                                     {
                                       ptr++;
                                       if ( *trFilename )
                                         {
                                           fprintf( stderr,
                     "WARNING: unfold.main(): parameter '-T' appears twice;" );
                                           fprintf( stderr,
                                                " %*s'%s' overwrites '%s'.\n",
                                                 24, "", ptr + 1, trFilename );
                                         } /* if */
                                       strcpy( trFilename, ++ptr );
                                       ptr = null;
                                     } /* if */
                                   GL_flags |= USELESS_TRANS;
                                   break;
                        case '@' : if ( *( ptr + 1 ) == '@' )
                                     {
                                       ptr++;
                                       if ( *( ptr + 1 ) == '@' )
                                         {
                                           ptr++;
                                           unfold = NERV_unfold;
					   ERV_IsNotCutoffPoint=
					     NERV_IsNotCutoffPoint_LIST;
                                         } /* if */
                                       else
                                         {
                                           unfold = ERV_unfold;
					   ERV_IsNotCutoffPoint=
					     ERV_IsNotCutoffPoint_LIST;
                                         } /* else */
                                     } /* if */
                                   else if ( *( ptr + 1 ) == '+' )
                                     {
                                       ptr++;
                                       unfold = EV2_unfold;
                                       GL_ParikhInAdvance = False;
                                       EV_insert_PE_queue_condlist
                                               = insert_PE_queue_condlist_size;
                                       EV_insert_PE_queue_condarray
                                              = insert_PE_queue_condarray_size;
				       IsNotCutoffEvent 
					      = IsNotCutoffEvent_LIST_size;
                                     } /* else if */
                                   else
                                     {
                                       unfold = EV_unfold;
                                     } /* else */
                                   break;
		        case 'X' : GL_flags|=WITH_TIME;
                                   unfold = M_unfold;
                                   GL_flags &= ~USE_BITARRAYS;
    				   FinalState = FinalState_LIST;
    				   Mark = Mark_LIST;
    				   IsNotCutoffPoint = IsNotCutoffPoint_LIST;
    				   IsNotCutoffEvent = IsNotCutoffEvent_LIST;
			           if ( *( ptr + 1 ) == '+' )
                                     {
                                       ptr++;
				       GL_flags|=BETTER_TIME;
				     }
                                   break;
                        case '#' : GL_flags |= CHECK_CONFIGS;
                                   break;
                        case '+' : GL_flags |= SORT_BEGIN;
                                   break;
                        case '$' : showUsage = True;
                                   break;
                        default  : fprintf( stderr,
                               "WARNING: unfold.main(): bad parameter '%s'.\n",
                                                                 argv[count] );
                                   break;
                      } /* switch */
                  } /* while */
              } /* if */
            else
              {
                if ( inFileType )
                  {
                    fprintf( stderr,
                     "WARNING: unfold.main(): input file already specified;" );
                    fprintf( stderr, " %*s'%s' overwrites '%s'.\n",
                                                     24, "", ptr, inFilename );
                  } /* if */
                inFileType = PN;
                strcpy( inFilename, ptr );
              } /* else */
          } /* while */
      } /* if */
    else
      {
        showUsage = True;
      } /* else */

    if ( showVersion )
      {
        printf( "%s: version %s (%s)\n", PROGRAM, VERSION, DATE );
      } /* if */
    if ( showUsage )
      show_usage();
    if ( ! ( showVersion || showUsage || inFileType ) )
      {
        Error("main","missing parameter ( PEP net file ).", 10 );
      } /* if */

    if ( inFileType )
      {
        GL_net=net= (inFileType == PN) ? read_PEP( inFilename )
               : (inFileType == FSA) ? read_FSA( inFilename ) : NULL;
        if ( inFileType == FSA ) write_FSA( net, "fsa-test.ll_net" );
        /* Testen bestimmter Transitionseigenschaften */
        tr = net->Transitions;
        while ( tr!=NULL )
          {
            if ( ! ( tr->preset && tr->postset ) )
              {
                printf( "The net is not T-restricted in '%s'.\n", tr->name );
              } /* if */
            notremoved = True;
            for ( st = tr->preset ; st ; st = st->nextplace )
              {
                if ( st->weight > 1 )
                  {
printf(
"Transition '%s' is connected to an arc with weight > 1 and is removed.\n",
tr->name );
                    tr = delete_transition_direct( net, tr );
                    notremoved = False;
                    break;
                  } /* if */
              } /* for st */
            if ( notremoved )
              {
                for ( ts = tr->postset ; ts ; ts = ts->nextplace )
                  {
                    if ( ts->weight > 1 )
                      {
printf(
"Transition '%s' is connected to an arc with weight > 1 and is removed.\n",
tr->name );
                        tr = delete_transition_direct( net, tr );
                        notremoved = False;
                        break;
                      } /* if */
                  } /* for ts */
              } /* if notremoved */
            if (notremoved)
              tr = tr->nexttrans;
          } /* while tr */
        /* Alle Stellen ohne Markierung ueberspringen */
        for ( pl = net->Places ; pl && ! pl->marks ; pl = pl->nextplace )
          skip;
        if ( pl==NULL )
          {
            Error("main","Net without start marking.", 13 );
          } /* if */
        /* Test auf 1-sichere Startmarkierung */
        for ( pl = net->Places ; pl ; pl = pl->nextplace )
          {
            if ( pl->marks > 1 )
              printf( "Place '%s' has more than 1 token.\n", pl->name );
          } /* for */
        if ( showSTNet )
          {
            show_matrix( net );
          } /* if */
        start_clock();
        /* Entfaltung berechnen */
        unfold_net=unfold( net );
        /* Nur zu statistischen Zwecken */
        unfold_net->NumComponents = net->NumComponents;
        stop_clock();
        if ( ( unfold == EV2_unfold ) || ( unfold == ERV_unfold ) )
          {
            for ( pl = unfold_net->Places ; pl ; skip )
              {
                if ( pl->label == DELETE )
                  {
                    pl = delete_place_direct( unfold_net, pl );
                  } /* if */
                else
                  {
                    pl = pl->nextplace;
                  } /* else */
              } /* for */
            count = 1;
            for ( pl = unfold_net->Places ; pl ; pl = pl->nextplace )
              {
                pl->numname = count++;
              } /* for */
          } /* if unfold */
        if ( output_unf && *unfFilename )
          {
            write_unfolding_UNF( unfFilename, inFilename, unfold_net );
          } /* if */
        if ( output_mci && *mciFilename )
          {
            write_unfolding_MCI( mciFilename, unfold_net, net );
          } /* if */
        if ( output_ent && *entFilename )
          {
            write_unfolding_ent( entFilename, unfold_net );
          } /* if */
        if ( showUnfolding )
          {
            write_unfolding_ent( NULL, unfold_net );
          } /* if */
        if ( showHash )
          {
            hash_show( net );
          } /* if showHash */
        if ( showTime )
          {
            printf( "|S|=%4u, |T|=%4u, |B|=%5u, |E|=%5u, ",
                    net->NumPlaces, net->NumTransitions,
                    unfold_net->NumPlaces, unfold_net->NumTransitions );
            printf( "Cutoffs: %u, ", GL_numCutoffs);
	    print_time(stdout);
            if (unfold != M_unfold)
              {
                if ( GL_EV_equalities )
                  printf( "%u equality case%s detected.\n", GL_EV_equalities,
                                          ( GL_EV_equalities == 1 )?"":"s" );
                printf( "%u comparison%s", GL_countQueueCompares,
                        ( GL_countQueueCompares == 1 )?"":"s");
                printf( " for queue inserting (%s).\n",
                        (GL_flags & SORT_BEGIN) ? "beginning" : "end" );
              } /* if */
            printf( "Calculated %s%s unfolding.\n",
                       (unfold == M_unfold) ? "McMillan" :
                       "Esparza/Vogler",
                       (GL_flags&WITH_TIME)? " with time expansion" : "" );
          } /* if showTime */
        if ( showStatistics )
          {
            statistics( unfold_net, net );
          } /* if */
        clean_up( unfold_net, net );
        if ( GL_flags & USELESS_TRANS )
          {
            ptr = GL_UselessTransitions;
            notremoved = False;
            count = 1;
            if ( *trFilename )
              {
                if ( ! ( trfile = fopen( trFilename, "w" ) ) )
                  {  
                    Error("main","Could not open file for writing.", 4 );
                  } /* if */
                putc( ' ', trfile );
              } /* if */
            else
              { 
                printf(
             "Useless transitions for the construction of the unfolding:\n{" );
                trfile = stdout;
              }
            for ( tr = net->Transitions ; tr ; tr = tr->nexttrans )
              {
                if ( ! *++ptr )
                  {
                    if ( notremoved )
                      {
                        count++;
                        putc( ',', trfile );
                      }
                    if ( ( count += strlen( tr->name ) + 1 ) > PAGEWIDTH )
                      {
                        count = strlen( tr->name ) + 2;
                        fprintf( trfile, "\n  %s", tr->name );
                      } /* if */
                    else
                      {
                        fprintf( trfile, " %s", tr->name );
                      } /* else */
                    notremoved = True;
                  } /* if */
              } /* for tr */
            if ( !*trFilename )
              printf( " }" );
            fprintf( trfile, "\n" );
            if ( *trFilename )
              fclose( trfile );
 
            Free( GL_UselessTransitions );
          } /* if get_useless_transitions */
 
        remove_net( &net );
        for ( pl = unfold_net->Places ; pl ; pl = pl->nextplace )
          {
            pl->inscript = NULL;
          } /* for */
        for ( tr = unfold_net->Transitions ; tr ; tr = tr->nexttrans )
          {
            tr->inscript = NULL;
          } /* for */
        remove_net( &unfold_net );
      } /* if inFileType */

    exit( 0 );
  } /* main() */


/*****************************************************************************/
/* END OF FILE main.c                                                        */
/*****************************************************************************/

