/****************************************************************************/
/*                   O P E R A T I O N S   O N   N E T S                    */
/****************************************************************************/
/* (in addition to those defined in 'neunetze.[hc]' and 'convertPEP.[hc]'.) */
/****************************************************************************/

#include "nets.h"
#include "sub_unf.h"
#include "unfold.h"

/****************************************************************************/
/* Bedingung in das uebergebene entfaltete Netz hinten einfuegen            */
/* Ggf. Stacks anpassen                                                     */
/****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Place insert_condition( LMPNet unfold_net, Place pl_org, 
			BIT_ARRAY conflArrayPtr )

#else

Place insert_condition( unfold_net, pl_org, conflArrayPtr )

  LMPNet unfold_net;
  Place  pl_org;
  BIT_ARRAY conflArrayPtr;

#endif

  {

    register Place cond;

    /* Speicher reservieren und NumNamen setzen, ggf. Stackgroesse */
    /* anpassen, um alle Bedingungen aufnehmen zu koennen          */
    cond = (Place) MYmalloc( sizeof( PlElem ) );
    cond->numname = ++unfold_net->NumPlaces;
    ResizeCStack( cond->numname );
    /* Wenn noetig in die Liste einpassen */
    if ( unfold_net->Places )
      {
        cond->prevplace = unfold_net->LastPlace;
        unfold_net->LastPlace->nextplace = cond;
      } /* if */
    /* sonst als einziges Element einfuegen */
    else
      {
        unfold_net->Places = cond;
        cond->prevplace = NULL;
      } /* else */
    unfold_net->LastPlace = cond;
    cond->nextplace = NULL;
    cond->preset = NULL;
    cond->postset = NULL;
    /* Zeiger auf originale Stelle */
    cond->inscript = (char *) pl_org;
    cond->label = pl_org->label;
    cond->marks = 0;
    cond->tokenage=0;
    /* component wird als Markierungsflag benutzt !*/

    /* Wenn die Konflikte direkt im Netz bearbeitet werden einfuegen */
    /* sonst ist cond->name auf NULL gesetzt */
    if ( GL_changeAlgo > unfold_net->NumTransitions )
      cond->name = (char *) conflArrayPtr;
    else
      cond->name = NULL;

    /* Bedingungsname bei der Stelle eintragen */
    rev_list_insert( (LIST *) &( pl_org->inscript ), cond );

    return ( cond );

  } /* insert_condition() */

/****************************************************************************/
/* Event in das uebergebene entfaltete Netz hinten einfuegen                */
/* Ggf. Stacks anpassen                                                     */
/****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Event insert_event( LMPNet unfold_net, Transition tr_org )

#else

Event insert_event( unfold_net, tr_org )

  LMPNet     unfold_net;
  Transition tr_org;

#endif

{
  Condition cond;
  Event     event;

  event = (Event) MYmalloc( sizeof( TrElem ) );
  if ( ( event->numname = ++unfold_net->NumTransitions ) > GL_maxEvents )
    {
      fprintf( stderr, "Maximal number of events (%u) has been exceeded.\n",
	       GL_maxEvents );
      exit ( 3 );
    } /* if */
  /* Ggf. Stackgroesse anpassen */
  ResizeEStack( event->numname );
  /* Passend in die Liste einhaengen und Felder belegen */
  unfold_net->LastTransition = unfold_net->Transitions ?
    ( unfold_net->LastTransition->nexttrans = event ) :
    ( unfold_net->Transitions = event );
  event->prevtrans = event->nexttrans = NULL;
  event->preset = NULL;
  event->postset = NULL;
  if (tr_org)
    {
      /* Zeiger auf originale Transition sichern */
      event->inscript = (char *) tr_org;
      if ( GL_flags & USELESS_TRANS )
	GL_UselessTransitions[ tr_org->numname ] = 1;
    }
  else
    {
      /* Zeiger auf originale Transition = NULL, also Tic-Event */
      /* Muss nicht als useless gespeichert werden */
      event->inscript = NULL;
    }
  event->label = (MultiSet) 0;
  event->name = NULL;
  event->time_label=NULL;
  if ( GL_changeAlgo == event->numname )
    {
      GL_LiConflictsFunction = LiConflictsNet;
      for ( cond = unfold_net->Places ; cond ; cond = cond->nextplace )
	{
	  if ( ( GL_maxTpreset > 2 ) && cond->name && 
	       ( ! cond->nextplace || 
		 ( cond->name != cond->nextplace->name ) ) )
	    {
	      Free( cond->name );
	      cond->name = NULL;
	    } /* if */
	} /* for */
    } /* if GL_changeAlgo */
  return (event);
}

/****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void insert_ec_edge( Event event, Condition condition )

#else

void insert_ec_edge( event, condition )

  Event     event;
  Condition condition;

#endif

  {

    register TSEdge ec;

    if ( event->postset )
      {
        ec = (TSEdge) event->postset->weight;
        event->postset->weight = (Cardinal32) MYmalloc( sizeof(TSElem) );
	ec->nextplace = (TSEdge) event->postset->weight;
        ec->nextplace->prevplace = ec;
        ec=ec->nextplace;
        ec->weight = 1;
      } /* if */
    else
      {
        event->postset = ec = (TSEdge) MYmalloc( sizeof(TSElem) );
        ec->prevplace = NULL;
        ec->weight = (Cardinal32) ec;
      } /* else */
    ec->nextplace = NULL;
    ec->prevtrans = ec->nexttrans = NULL;
    ec->place = condition;
    ec->trans = event;
    condition->preset = ec;

  } /* insert_ec_edge() */

/****************************************************************************/
/* Nachbereich aller Events der Liste in das uebergebene entfaltete Netz    */
/* einfuegen.                                                               */
/****************************************************************************/

void insert_postset_list( LIST list, LMPNet unfold_net)
{
  Event     event;
  Condition cond;
  TSEdge    ts;
  STEdge    st;

  for ( skip ; list ; list = list->next )
    {
      event = (Event) list->item;
      if (event->inscript) /* kein TIC-Event */
	{
	  for ( ts = ( (Transition) event->inscript )->postset ; ts ; 
		ts = ts->nextplace )
	    {
	      cond = insert_condition( unfold_net, ts->place, NULL );
	      cond->label = INTERRUPT;
	      insert_ec_edge( event, cond );
	    } /* for ts */
	} /* if kein Tic-Event */
      else
	{
	  /* Vorbereich duplizieren */
	  for(st=event->preset;st;st=st->nextplace)
	    {
	      cond = insert_condition( unfold_net, (Place)st->place->inscript,
				       NULL );
	      cond->tokenage = st->place->tokenage+1;
	      cond->label = INTERRUPT;
	      insert_ec_edge( event, cond );
	    }
	}
    } /* for list */
}
