#include "parikh.h"
#include "sub_unf.h"

PARIKH_ARRAY         GL_parikh_table = NULL;
Cardinal32           GL_parikh_table_size = 0;

PARIKH_VECTOR        GL_parikh_vector = NULL;

PARIKH_ARRAY         GL_ParikhArray1 = NULL,
                     GL_ParikhArray2 = NULL;

Condition            * GL_CArray      = NULL;
Cardinal32           GL_countQueueCompares = 0;

/*****************************************************************************/
/* Some function pointer definitions.                                        */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void          (* GL_Display_Parikh)( PARIKH_VECTOR );
PARIKH_VECTOR (* GL_Empty_Parikh)( void );
PARIKH_VECTOR (* GL_Calculate_Parikh)( Event );
PARIKH_VECTOR (* GL_Calculate_Parikh_T)( Transition );
PARIKH_VECTOR (* GL_Calculate_PSize_T)( Transition );
int           (* GL_Compare_Parikh_A)( PARIKH_VECTOR, PARIKH_VECTOR,
                                                   Condition *, Condition * );
int           (* GL_Compare_Parikh_L)( PARIKH_VECTOR, PARIKH_VECTOR,
                                                               Event, Event );

#else

void          (* GL_Display_Parikh)();
PARIKH_VECTOR (* GL_Empty_Parikh)();
PARIKH_VECTOR (* GL_Calculate_Parikh)();
PARIKH_VECTOR (* GL_Calculate_Parikh_T)();
PARIKH_VECTOR (* GL_Calculate_PSize_T)();
int           (* GL_Compare_Parikh_A)();
int           (* GL_Compare_Parikh_L)();

#endif

/*****************************************************************************/
/* Print a T-Parikh vector array table.                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void Display_Parikh_Array_Table( PARIKH_ARRAY parikh_table,
                                        Cardinal32   vecsize,
                                        Cardinal32   maxlevel )

#else

void Display_Parikh_Array_Table( parikh_table, vecsize, maxlevel )

  PARIKH_ARRAY parikh_table;
  Cardinal32   vecsize,
               maxlevel;

#endif

  {

    register Cardinal32   count,
                          level;
    register PARIKH_ARRAY parikh = parikh_table,
                          ptr;

    printf( "PARIKH_TABLE:\n" );

    for ( level = 1 ; level <= maxlevel ; level++ )
      {
	ptr = parikh;
        printf( "%4ua <(%u)", level, *( (Cardinal32 *) ptr++ ) );
	count = GL_net->NumTransitions + 1;
        while ( --count && ! *++ptr )
          skip;
        if ( count )
          {
            if ( *ptr > 1 )
              printf( "%u*", *ptr );
            printf( "T%u", GL_net->NumTransitions + 1 - count );
            while ( count )
              {
                while ( --count && ! *++ptr )
                  skip;
                if ( count )
                  {
                    putchar ( ',' );
                    if ( *ptr > 1 )
                      printf( "%u*", *ptr );
                    printf( "T%u", GL_net->NumTransitions + 1 - count );
                  } /* if */
              } /* while count */
          } /* if count */
        else
          {
            printf( "empty" );
          } /* else */
        putchar( '>' );
        putchar( '\n' );

        ptr = ( parikh += vecsize );

        printf( "%4ua <(%u)", level, *( (Cardinal32 *) ptr++ ) );
        count = GL_net->NumTransitions + 1;
        while ( --count && ! *++ptr )
          skip;
        if ( count )
          {
            if ( *ptr > 1 )
              printf( "%u*", *ptr );
            printf( "T%u", GL_net->NumTransitions + 1 - count );
            while ( count )
              {
                while ( --count && ! *++ptr )
                  skip;
                if ( count )
                  {
                    putchar ( ',' );
                    if ( *ptr > 1 )
                      printf( "%u*", *ptr );
                    printf( "T%u", GL_net->NumTransitions + 1 - count );
                  } /* if */
              } /* while count */
          } /* if count */
        else
          {
            printf( "empty" );
          } /* else */
        putchar( '>' );
        putchar( '\n' );
        parikh += vecsize;
      } /* for level */

  } /* Display_Parikh_Array_Table() */


/*****************************************************************************/
/* Compare two T-Parikh vector arrays lexicographically.                     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static int Compare_Parikh_Arrays_LW( PARIKH_ARRAY parikh1,
                                     PARIKH_ARRAY parikh2 )

#else

static int Compare_Parikh_Arrays_LW( parikh1, parikh2 )

  PARIKH_ARRAY parikh1,
               parikh2;

#endif

  {

    register PARIKH_ARRAY ptr1,
                          ptr2;
    register Cardinal32   count;

    if ( *( (Cardinal32 *) parikh1 ) < *( (Cardinal32 *) parikh2 ) )
      {
        return ( -1 );
      } /* if */
    else if ( *( (Cardinal32 *) parikh1 ) > *( (Cardinal32 *) parikh2 ) )
      {
        return ( 1 );
      } /* if */

    ptr1 = parikh1 + 1;
    ptr2 = parikh2 + 1;
    count = GL_net->NumTransitions + 1;

    while ( --count && ( *ptr1 == *ptr2 ) )
      {
        ptr1++;
        ptr2++;
      } /* while */

    if ( count )
      {
        if ( *ptr1 < *ptr2 )
          {
            return ( *ptr1 ? -1 : 1 );
          } /* if */
        else /* *ptr1 > *ptr2 */
          {
            return ( *ptr2 ? 1 : -1 );
          } /* else */
      } /* if */
    else
      {
        return ( 0 );
      } /* else */

  } /* Compare_Parikh_Arrays_LW() */


/*****************************************************************************/
/* Compare two configurations level-wise.                                    */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static int Compare_Configs_LevelWise_A( Condition * condarray1,
                                        Condition * condarray2 )

#else

static int Compare_Configs_LevelWise_A( condarray1, condarray2 )

  Condition * condarray1,
            * condarray2;

#endif

  {

    register Event        ev,
                          event,
                          * stackEptr;
    register STEdge       ce;
    register TSEdge       ec;
    register PARIKH_ARRAY parikh;
    register Cardinal32   size,
                          maxsize,
                          memsize,
                          count;
    register int          cmp;
    register Condition    * coptr;
    register Cardinal32   vecsize = ( GL_net->NumTransitions + 3 )
                                                             & ~(Cardinal32) 1;

    IncEMark();
    InitEStack();

    /* LocalConfigDepths */
    memsize = maxsize = 0;
    count = (Cardinal32) ( ev = (Event) *( coptr = condarray1 ) )->inscript;
    while ( count-- )
      {
        if ( ( ec = (*++coptr)->preset ) && ( ( size = (Cardinal32)
                        ec->trans->label ) > memsize ))
          memsize = size;
        PushEvent( *coptr );
      } /* while */
    count = (Cardinal32) ( *( coptr = condarray2 ) )->inscript;
    while ( count-- )
      {
        if ( ( ec = (*++coptr)->preset ) && ( ( size = (Cardinal32)
                        ec->trans->label ) > maxsize ))
          maxsize = size;
      } /* while */
    if ( ++memsize > ( size = ++maxsize ) )
      maxsize = memsize;

    if ( ! GL_parikh_table )
      {
        if ( GL_net->NumTransitions > PV_Item_Max )
          {
            fprintf( stderr, "Change PV_Item for Parikh vectors.\n" );
            exit( 111 );
          } /* if */
        GL_parikh_table = (PARIKH_ARRAY) MYmalloc(
                              ( GL_parikh_table_size = GL_net->NumTransitions )
                                * 2 * vecsize * sizeof( Parikh_Array_Entry ) );
      } /* if */
    else if ( maxsize > GL_parikh_table_size )
      {
        parikh = GL_parikh_table = (PARIKH_ARRAY)
                                             MYrealloc( (char *) GL_parikh_table,
                              ( GL_parikh_table_size = maxsize + 12 ) * vecsize
                                          * 2 * sizeof( Parikh_Array_Entry ) );
        for ( count = maxsize * vecsize * 2 ; count-- ; *( parikh++ ) = 0 )
          skip;
      } /* else if */
    else
      {
        for ( count = ( maxsize * vecsize ) << 1, parikh = GL_parikh_table ;
                                                  count-- ; *( parikh++ ) = 0 )
          skip;
      } /* else */

    GL_parikh_table -= vecsize << 1;
    *( (Cardinal32 *) ( parikh = GL_parikh_table
                                      + ( ( memsize * vecsize ) << 1 ) ) ) = 1;
    ( ++parikh )[ ev->numname ]++;
    while ( (event = PopEvent()) )
      {
        ++*( (Cardinal32 *) ( parikh = GL_parikh_table
                        + ( ( vecsize * (Cardinal32) event->label ) << 1 ) ) );
        if ( ! ++( ( ++parikh )[ ( (Transition)event->inscript )->numname ] ) )
          {
            fprintf( stderr, "Parikh_Array_Entry size is too small!\n" );
            exit ( 111 );
          } /* if */
        for ( ce = event->preset ; ce ; ce = ce->nextplace )
          {
            PushEvent( ce->place );
          } /* for */
      } /* while */

    IncEMark();
    InitEStack();
    count = (Cardinal32) ( ev = (Event) *( coptr = condarray2 ) )->inscript;
    GL_parikh_table += vecsize;
    *( (Cardinal32 *) ( parikh = GL_parikh_table
                                         + ( ( size * vecsize ) << 1 ) ) ) = 1;
    ( ++parikh )[ ev->numname ]++;
    for ( coptr++ ; count-- ; coptr++ )
      {
        PushEvent( *coptr );
      } /* for */
    while ( (event = PopEvent()) )
      {
        ++*( (Cardinal32 *) ( parikh = GL_parikh_table
                        + ( ( vecsize * (Cardinal32) event->label ) << 1 ) ) );
        if ( ! ++( ( ++parikh )[ ( (Transition)event->inscript )->numname ] ) )
          {
            fprintf( stderr, "Parikh_Array_Entry size is too small!\n" );
            exit ( 111 );
          } /* if */
        for ( ce = event->preset ; ce ; ce = ce->nextplace )
          {
            PushEvent( ce->place );
          } /* for */
      } /* while */

    parikh = ( GL_parikh_table += vecsize );
#ifdef _SHOW_PARIKH_TABLE_
    Display_Parikh_Array_Table( GL_parikh_table, vecsize, maxsize );
#endif /* _SHOW_PARIKH_TABLE_ */
    maxsize++;
    while ( --maxsize && ! ( cmp = Compare_Parikh_Arrays_LW( parikh,
                                                         parikh + vecsize ) ) )
      {
        parikh += vecsize << 1;
      } /* while */

    if ( maxsize )
      {
#ifdef _SHOW_PARIKH_TABLE_
printf( "VERGLEICH ERGAB: %s.\n",
        cmp == 1 ? "GROESSER" : cmp == -1 ? "KLEINER" : "GLEICH" );
#endif /* _SHOW_PARIKH_TABLE_ */
        return ( cmp );
      } /* if */
    else
      {
/*
        printf( "EQUALITY CASE FOUND - SOMETHING GETS WRONG !!!\n" );
*/
        GL_EV_equalities++;
        return ( 0 );
      } /* else */

  } /* Compare_Configs_LevelWise_A() */


/*****************************************************************************/
/* Compare two configurations level-wise.                                    */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static int Compare_Configs_LevelWise_L( Event event1, Event event2 )

#else

static int Compare_Configs_LevelWise_L( event1, event2 )

  Event event1,
        event2;

#endif

  {

    register Event        event,
                          * stackEptr;
    register STEdge       ce;
    register PARIKH_ARRAY parikh;
    register Cardinal32   size,
                          maxsize,
                          memsize,
                          count;
    register int          cmp;
    register Cardinal32   vecsize = ( GL_net->NumTransitions + 3 )
                                                             & ~(Cardinal32) 1;

    memsize = (Cardinal32) event1->label;
    size = maxsize = (Cardinal32) event2->label;
    if ( memsize > maxsize )
      maxsize = memsize;

    if ( ! GL_parikh_table )
      {
        if ( GL_net->NumTransitions > PV_Item_Max )
          {
            fprintf( stderr, "Change PV_Item for Parikh vectors.\n" );
            exit( 111 );
          } /* if */
        GL_parikh_table = (PARIKH_ARRAY) MYmalloc(
                              ( GL_parikh_table_size = GL_net->NumTransitions )
                                * 2 * vecsize * sizeof( Parikh_Array_Entry ) );
      } /* if */
    else if ( maxsize > GL_parikh_table_size )
      {
        parikh = GL_parikh_table = (PARIKH_ARRAY)
                                             MYrealloc( (char *) GL_parikh_table,
                              ( GL_parikh_table_size = maxsize + 12 ) * vecsize
                                          * 2 * sizeof( Parikh_Array_Entry ) );
        for ( count = maxsize * vecsize * 2 ; count-- ; *( parikh++ ) = 0 )
          skip;
      } /* else if */
    else
      {
        for ( count = ( maxsize * vecsize ) << 1, parikh = GL_parikh_table ;
                                                  count-- ; *( parikh++ ) = 0 )
          skip;
      } /* else */

    IncEMark();
    InitEStack();

    EMark( event1 );
    for ( ce = event1->preset ; ce ; ce = ce->nextplace )
      {
        PushEvent( ce->place );
      } /* for */

    GL_parikh_table -= vecsize << 1;
    *( (Cardinal32 *) ( parikh = GL_parikh_table
                                      + ( ( memsize * vecsize ) << 1 ) ) ) = 1;
    ( ++parikh )[ ( (Transition) event1->inscript )->numname ]++;
    while ( (event = PopEvent()) )
      {
        ++*( (Cardinal32 *) ( parikh = GL_parikh_table
                        + ( ( vecsize * (Cardinal32) event->label ) << 1 ) ) );
        if ( ! ++( ( ++parikh )[ ( (Transition)event->inscript )->numname ] ) )
          {
            fprintf( stderr, "Parikh_Array_Entry size is too small!\n" );
            exit ( 111 );
          } /* if */
        for ( ce = event->preset ; ce ; ce = ce->nextplace )
          {
            PushEvent( ce->place );
          } /* for */
      } /* while */

    IncEMark();
    InitEStack();

    EMark( event2 );
    for ( ce = event2->preset ; ce ; ce = ce->nextplace )
      {
        PushEvent( ce->place );
      } /* for */

    GL_parikh_table += vecsize;
    *( (Cardinal32 *) ( parikh = GL_parikh_table
                                         + ( ( size * vecsize ) << 1 ) ) ) = 1;
    ( ++parikh )[ ( (Transition) event2->inscript )->numname ]++;
    while ( (event = PopEvent()) )
      {
        ++*( (Cardinal32 *) ( parikh = GL_parikh_table
                        + ( ( vecsize * (Cardinal32) event->label ) << 1 ) ) );
        if ( ! ++( ( ++parikh )[ ( (Transition)event->inscript )->numname ] ) )
          {
            fprintf( stderr, "Parikh_Array_Entry size is too small!\n" );
            exit ( 111 );
          } /* if */
        for ( ce = event->preset ; ce ; ce = ce->nextplace )
          {
            PushEvent( ce->place );
          } /* for */
      } /* while */

    parikh = ( GL_parikh_table += vecsize );
#ifdef _SHOW_PARIKH_TABLE_
    Display_Parikh_Array_Table( GL_parikh_table, vecsize, maxsize );
#endif /* _SHOW_PARIKH_TABLE_ */
    maxsize++;
    while ( --maxsize && ! ( cmp = Compare_Parikh_Arrays_LW( parikh,
                                                         parikh + vecsize ) ) )
      {
        parikh += vecsize << 1;
      } /* while */

    if ( maxsize )
      {
#ifdef _SHOW_PARIKH_TABLE_
printf( "VERGLEICH ERGAB: %s.\n",
        cmp == 1 ? "GROESSER" : cmp == -1 ? "KLEINER" : "GLEICH" );
#endif /* _SHOW_PARIKH_TABLE_ */
        return ( cmp );
      } /* if */
    else
      {
/*
        printf( "EQUALITY CASE FOUND - SOMETHING GETS WRONG !!!\n" );
*/
        GL_EV_equalities++;
        return ( 0 );
      } /* else */

  } /* Compare_Configs_LevelWise_L() */


/*****************************************************************************/
/* Print a T-Parikh vector array.                                            */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void Display_Parikh_Array( PARIKH_VECTOR parikh )

#else

void Display_Parikh_Array( parikh )

  PARIKH_VECTOR parikh;

#endif

  {

    register Cardinal32   count = GL_net->NumTransitions + 1;
    register PARIKH_ARRAY ptr   = (PARIKH_ARRAY) parikh;

    printf( "<(%u)", *( (Cardinal32 *) ptr++ ) );

    while ( --count && ! *++ptr )
      skip;
    if ( count )
      {
        if ( *ptr > 1 )
          printf( "%u*", *ptr );
        printf( "T%u", GL_net->NumTransitions + 1 - count );
        while ( count )
          {
            while ( --count && ! *++ptr )
              skip;
            if ( count )
              {
                putchar ( ',' );
                if ( *ptr > 1 )
                  printf( "%u*", *ptr );
                printf( "T%u", GL_net->NumTransitions + 1 - count );
              } /* if */
          } /* while */
      } /* if */
    else
      {
        printf( "empty" );
      } /* else */

    putchar( '>' );
    putchar( '\n' );

  } /* Display_Parikh_Array() */


/*****************************************************************************/
/* Allocate an empty T-Parikh vector array.                                  */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

PARIKH_VECTOR Empty_Parikh_Array( void )

#else

PARIKH_VECTOR Empty_Parikh_Array()

#endif

  {

    return ( (PARIKH_VECTOR) ( GL_ParikhInAdvance ? MYmalloc(
    ( GL_net->NumTransitions + 2 ) * sizeof( Parikh_Array_Entry ) ) : NULL ) );

  } /* Empty_Parikh_Array() */


/*****************************************************************************/
/* Calculate the T-Parikh vector array for an event of the unfolding.        */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

PARIKH_VECTOR Calculate_Parikh_Array( Event event )

#else

PARIKH_VECTOR Calculate_Parikh_Array( event )

  Event event;

#endif

  {

    register Event        ev,
                          * stackEptr;
    register STEdge       ce;
    register PARIKH_ARRAY parikh;
    register Cardinal32   count = 1;

    IncEMark();
    InitEStack();
    ( parikh = ( (PARIKH_ARRAY) MYmalloc( ( GL_net->NumTransitions + 2 )
                                    * sizeof( Parikh_Array_Entry ) ) ) + 1 )
                      [ ( (Transition) ( ev = event )->inscript )->numname ]++;
    EMark( ev );
    for ( ce = ev->preset ; ce ; ce = ce->nextplace )
      {
        PushEvent( ce->place );
      } /* for */

    while ( (ev = PopEvent()) )
      {
        if ( ! ++parikh[ ( (Transition) ev->inscript )->numname ] )
          {
            fprintf( stderr, "Parikh_Array_Entry size is too small!\n" );
            exit ( 111 );
          } /* if */
        count++;
        for ( ce = ev->preset ; ce ; ce = ce->nextplace )
          {
            PushEvent( ce->place );
          } /* for */
      } /* while */

    *( (Cardinal32 *) --parikh ) = count;

    return ( (PARIKH_VECTOR) parikh );

  } /* Calculate_Parikh_Array() */


/*****************************************************************************/
/* Calculate the T-Parikh vector array for an event of the unfolding.        */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

PARIKH_VECTOR Calculate_Parikh_Array_T( Transition trans )

#else

PARIKH_VECTOR Calculate_Parikh_Array_T( trans )

  Transition trans;

#endif

  {

    register Event        event,
                          * stackEptr;
    register STEdge       ce;
    register Cardinal32   count;
    register PARIKH_ARRAY parikh;
    register Condition    * coptr;

    IncEMark();
    InitEStack();
    ( parikh = ( (PARIKH_ARRAY) MYmalloc( ( GL_net->NumTransitions + 2 )
                 * sizeof( Parikh_Array_Entry ) ) ) + 1 ) [ trans->numname ]++;
    count = (Cardinal32) trans->inscript;
    for ( coptr = GL_CArray ; count-- ; coptr++ )
      {
        PushEvent( *coptr );
      } /* for */

    count = 1;
    while ( (event = PopEvent()) )
      {
        if ( ! ++parikh[ ( (Transition) event->inscript )->numname ] )
          {
            fprintf( stderr, "Parikh_Array_Entry size is too small!\n" );
            exit ( 111 );
          } /* if */
        count++;
        for ( ce = event->preset ; ce ; ce = ce->nextplace )
          {
            PushEvent( ce->place );
          } /* for */
      } /* while */

    *( (Cardinal32 *) --parikh ) = count;

    return ( (PARIKH_VECTOR) parikh );

  } /* Calculate_Parikh_Array_T() */


/*****************************************************************************/
/* Calculate the T-Parikh vector array for an event of the unfolding.        */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static PARIKH_ARRAY Calculate_Parikh_Array_T_static( PARIKH_ARRAY parikh,
                                                     Condition    * condarray )

#else

static PARIKH_ARRAY Calculate_Parikh_Array_T_static( parikh, condarray )

  PARIKH_ARRAY parikh;
  Condition    * condarray;

#endif

  {

    register Transition   trans;
    register Event        event,
                          * stackEptr;
    register STEdge       ce;
    register Cardinal32   count;
    register PARIKH_ARRAY par;
    register Condition    * coptr;

    for ( count = GL_net->NumTransitions + 2, par = parikh ; count-- ;
                                                              *( par++ ) = 0 )
      skip;
    IncEMark();
    InitEStack();
    count = (Cardinal32) ( trans = (Transition) *condarray )->inscript;
    ( par = parikh + 1 ) [ trans->numname ]++;
    for ( coptr = GL_CArray ; count-- ; coptr++ )
      {
        PushEvent( *coptr );
      } /* for */

    count = 1;
    while ( (event = PopEvent()) )
      {
        if ( ! ++par[ ( (Transition) event->inscript )->numname ] )
          {
            fprintf( stderr, "Parikh_Array_Entry size is too small!\n" );
            exit ( 111 );
          } /* if */
        count++;
        for ( ce = event->preset ; ce ; ce = ce->nextplace )
          {
            PushEvent( ce->place );
          } /* for */
      } /* while */

    *( (Cardinal32 *) parikh ) = count;

    return ( parikh );

  } /* Calculate_Parikh_Array_T_static() */


/*****************************************************************************/
/* Compare two T-Parikh vector arrays lexicographically.                     */
/*****************************************************************************/

/* &&& */
#if defined(__STDC__) || defined(__cplusplus)

int Compare_Parikh_Arrays_A( PARIKH_VECTOR parikh1,
                                    PARIKH_VECTOR parikh2,
                                    Condition     * condarray1,
                                    Condition     * condarray2 )

#else

int Compare_Parikh_Arrays_A( parikh1, parikh2, condarray1, condarray2 )

  PARIKH_VECTOR parikh1,
                parikh2;
  Condition     condarray1,
                condarray2;

#endif

  {

    register PARIKH_ARRAY ptr1,
                          ptr2;
    register Cardinal32   count;

    if ( ! GL_ParikhInAdvance )
      {
/* QQQ */
        if ( (Cardinal32) parikh1 < (Cardinal32) parikh2 )
          {
            return ( -1 );
          } /* if */
        else if ( (Cardinal32) parikh1 > (Cardinal32) parikh2 )
          {
            return ( 1 );
          } /* else if */
        Calculate_Parikh_Array_T_static( GL_ParikhArray1, condarray1 );
        Calculate_Parikh_Array_T_static( GL_ParikhArray2, condarray2 );
        parikh1 = (PARIKH_VECTOR) GL_ParikhArray1;
        parikh2 = (PARIKH_VECTOR) GL_ParikhArray2;
#ifdef _TRACE_EV_QUEUE_
printf( "Vector1: ");
Display_Parikh_Array( parikh1 );
printf( "Vector2: ");
Display_Parikh_Array( parikh2 );
#endif /* _TRACE_EV_QUEUE_ */
      } /* if */
    else
      {
GL_countQueueCompares++;
#ifdef _TRACE_EV_QUEUE_
printf( "Vector1: ");
Display_Parikh_Array( parikh1 );
printf( "Vector2: ");
Display_Parikh_Array( parikh2 );
#endif /* _TRACE_EV_QUEUE_ */
        if ( *( (Cardinal32 *) parikh1 ) < *( (Cardinal32 *) parikh2 ) )
          {
            return ( -1 );
          } /* if */
        else if ( *( (Cardinal32 *) parikh1 ) > *( (Cardinal32 *) parikh2 ) )
          {
            return ( 1 );
          } /* else if */
      } /* else */

    ptr1 = (PARIKH_ARRAY) parikh1 + 1;
    ptr2 = (PARIKH_ARRAY) parikh2 + 1;
    count = GL_net->NumTransitions + 1;

    while ( --count && ( *ptr1 == *ptr2 ) )
      {
        ptr1++;
        ptr2++;
      } /* while */

    if ( count )
      {
        if ( *ptr1 < *ptr2 )
          {
            return ( *ptr1 ? -1 : 1 );
          } /* if */
        else /* *ptr1 > *ptr2 */
          {
            return ( *ptr2 ? 1 : -1 );
          } /* else */
      } /* if */
    else
      {
/*
        printf( "EQUALITY CASE FOUND!\n" );
*/
        GL_EV_equalities++;
/*
printf("VEC1: ");Display_Parikh_Array( parikh1 );
printf("VEC2: ");Display_Parikh_Array( parikh2 );
*/
        return ( Compare_Configs_LevelWise_A( condarray1, condarray2 ) );
      } /* else */

  } /* Compare_Parikh_Arrays_A() */


/*****************************************************************************/
/* Compare two T-Parikh vector arrays lexicographically.                     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

int Compare_Parikh_Arrays_L( PARIKH_VECTOR parikh1,
                                    PARIKH_VECTOR parikh2,
                                    Event         event1,
                                    Event         event2 )

#else

int Compare_Parikh_Arrays_L( parikh1, parikh2, event1, event2 )

  PARIKH_VECTOR parikh1,
                parikh2;
  Event         event1,
                event2;

#endif

  {

    register PARIKH_ARRAY ptr1,
                          ptr2;
    register Cardinal32   count;

    if ( *( (Cardinal32 *) parikh1 ) < *( (Cardinal32 *) parikh2 ) )
      {
        return ( -1 );
      } /* if */
    else if ( *( (Cardinal32 *) parikh1 ) > *( (Cardinal32 *) parikh2 ) )
      {
        return ( 1 );
      } /* if */

    ptr1 = (PARIKH_ARRAY) parikh1 + 1;
    ptr2 = (PARIKH_ARRAY) parikh2 + 1;
    count = GL_net->NumTransitions + 1;

    while ( --count && ( *ptr1 == *ptr2 ) )
      {
        ptr1++;
        ptr2++;
      } /* while */

    if ( count )
      {
        if ( *ptr1 < *ptr2 )
          {
            return ( *ptr1 ? -1 : 1 );
          } /* if */
        else /* *ptr1 > *ptr2 */
          {
            return ( *ptr2 ? 1 : -1 );
          } /* else */
      } /* if */
    else
      {
/*
        printf( "EQUALITY CASE FOUND!\n" );
*/
        GL_EV_equalities++;
/*
printf("VEK1: ");Display_Parikh_Array( parikh1 );
printf("VEK2: ");Display_Parikh_Array( parikh2 );
*/
        return ( Compare_Configs_LevelWise_L( event1, event2 ) );
      } /* else */

  } /* Compare_Parikh_Arrays_L() */


/*****************************************************************************/
/* Compare two configurations level-wise.                                    */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static int P_Compare_Configs_LevelWise( Event event1, Event event2 )

#else

static int P_Compare_Configs_LevelWise( event1, event2 )

  Event event1,
        event2;

#endif

  {

    register Event        event,
                          * stackEptr;
    register STEdge       ce;
    register PARIKH_ARRAY parikh;
    register Cardinal32   size,
                          maxsize,
                          memsize,
                          count;
    register int          cmp;
    register Cardinal32   vecsize = ( GL_net->NumTransitions + 3 )
                                                             & ~(Cardinal32) 1;

    /* LocalConfigDepths */
    memsize = (Cardinal32) event1->label;
    size = maxsize = (Cardinal32) event2->label;
    if ( memsize > maxsize )
      maxsize = memsize;

    if ( ! GL_parikh_table )
      {
        if ( GL_net->NumTransitions > PV_Item_Max )
          {
            fprintf( stderr, "Change PV_Item for Parikh vectors.\n" );
            exit( 111 );
          } /* if */
        GL_parikh_table = (PARIKH_ARRAY) MYmalloc(
                              ( GL_parikh_table_size = GL_net->NumTransitions )
                                * 2 * vecsize * sizeof( Parikh_Array_Entry ) );
      } /* if */
    else if ( maxsize > GL_parikh_table_size )
      {
        parikh = GL_parikh_table = (PARIKH_ARRAY)
                                             MYrealloc( (char *) GL_parikh_table,
                              ( GL_parikh_table_size = maxsize + 12 ) * vecsize
                                          * 2 * sizeof( Parikh_Array_Entry ) );
        for ( count = maxsize * vecsize * 2 ; count-- ; *( parikh++ ) = 0 )
          skip;
      } /* else if */
    else
      {
        for ( count = ( maxsize * vecsize ) << 1, parikh = GL_parikh_table ;
                                                  count-- ; *( parikh++ ) = 0 )
          skip;
      } /* else */

    IncEMark();
    InitEStack();
    GL_parikh_table -= vecsize << 1;
    *( (Cardinal32 *) ( parikh = GL_parikh_table
                                      + ( ( memsize * vecsize ) << 1 ) ) ) = 1;
    ( ++parikh )[ ( (Transition) event1->inscript )->numname ]++;
    for ( ce = event1->preset ; ce ; ce = ce->nextplace )
      {
        PushEvent( ce->place );
      } /* for */
    while ( (event = PopEvent()) )
      {
        ++*( (Cardinal32 *) ( parikh = GL_parikh_table
                        + ( ( vecsize * (Cardinal32) event->label ) << 1 ) ) );
        if ( ! ++( ( ++parikh )[ ( (Transition)event->inscript )->numname ] ) )
          {
            fprintf( stderr, "Parikh_Array_Entry size is too small!\n" );
            exit ( 111 );
          } /* if */
        for ( ce = event->preset ; ce ; ce = ce->nextplace )
          {
            PushEvent( ce->place );
          } /* for */
      } /* while */

    IncEMark();
    InitEStack();
    GL_parikh_table += vecsize;
    *( (Cardinal32 *) ( parikh = GL_parikh_table
                                         + ( ( size * vecsize ) << 1 ) ) ) = 1;
    ( ++parikh )[ ( (Transition) event2->inscript )->numname ]++;
    for ( ce = event1->preset ; ce ; ce = ce->nextplace )
      {
        PushEvent( ce->place );
      } /* for */
    while ( (event = PopEvent()) )
      {
        ++*( (Cardinal32 *) ( parikh = GL_parikh_table
                        + ( ( vecsize * (Cardinal32) event->label ) << 1 ) ) );
        if ( ! ++( ( ++parikh )[ ( (Transition)event->inscript )->numname ] ) )
          {
            fprintf( stderr, "Parikh_Array_Entry size is too small!\n" );
            exit ( 111 );
          } /* if */
        for ( ce = event->preset ; ce ; ce = ce->nextplace )
          {
            PushEvent( ce->place );
          } /* for */
      } /* while */

    parikh = ( GL_parikh_table += vecsize );
#ifdef _SHOW_PARIKH_TABLE_
    printf( "Compare events e%u - e%u:\n", event1->numname, event2->numname );

    Display_Parikh_Array_Table( GL_parikh_table, vecsize, maxsize );
#endif /* _SHOW_PARIKH_TABLE_ */
    maxsize++;
    while ( --maxsize && ! ( cmp = Compare_Parikh_Arrays_LW( parikh,
                                                         parikh + vecsize ) ) )
      {
        parikh += vecsize << 1;
      } /* while */

    if ( maxsize )
      {
#ifdef _SHOW_PARIKH_TABLE_
printf( "VERGLEICH ERGAB: %s.\n",
        cmp == 1 ? "GROESSER" : cmp == -1 ? "KLEINER" : "GLEICH" );
#endif /* _SHOW_PARIKH_TABLE_ */
        return ( cmp );
      } /* if */
    else
      {
/*
        printf( "EQUALITY CASE FOUND - SOMETHING GETS WRONG !!!\n" );
*/
        GL_EV_equalities++;
        return ( 0 );
      } /* else */

  } /* P_Compare_Configs_LevelWise() */


/*****************************************************************************/
/* Calculate the T-Parikh vector array for an event of the unfolding.        */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static PARIKH_ARRAY P_Calculate_Parikh( PARIKH_ARRAY parikh,
                                 Event        thisevent )

#else

static PARIKH_ARRAY P_Calculate_Parikh( parikh, thisevent )

  PARIKH_ARRAY parikh;
  Event        thisevent;

#endif

  {

    register Event        event,
                          * stackEptr;
    register STEdge       ce;
    register Cardinal32   count;
    register PARIKH_ARRAY par;

    for ( count = GL_net->NumTransitions + 2, par = parikh ; count-- ;
                                                              *( par++ ) = 0 )
      skip;
    IncEMark();
    InitEStack();
    ( par = parikh + 1 ) [ ( (Transition) thisevent->inscript )->numname ]++;
    for ( ce = thisevent->preset ; ce ; ce = ce->nextplace )
      {
        PushEvent( ce->place );
      } /* for */
    count = 1;
    while ( (event = PopEvent()) )
      {
        if ( ! ++par[ ( (Transition) event->inscript )->numname ] )
          {
            fprintf( stderr, "Parikh_Array_Entry size is too small!\n" );
            exit ( 111 );
          } /* if */
        count++;
        for ( ce = event->preset ; ce ; ce = ce->nextplace )
          {
            PushEvent( ce->place );
          } /* for */
      } /* while */

    *( (Cardinal32 *) parikh ) = count;
/*
Display_Parikh_Array( parikh );
*/

    return ( parikh );

  } /* P_Calculate_Parikh() */


/*****************************************************************************/
/* Compare two T-Parikh vector arrays lexicographically.                     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

int P_compare( Event event1, Event event2 )

#else

int P_compare( event1, event2 )

  Event event1,
        event2;

#endif

  {

    register PARIKH_ARRAY ptr1,
                          ptr2;
    register Cardinal32   count;

/*
printf("COMPARE: E%u(%s) - E%u(%s)\n",event1->numname,((Transition)event1->inscript)->name,event2->numname,((Transition)event2->inscript)->name);
*/
    ptr1 = P_Calculate_Parikh( GL_ParikhArray1, event1 );
    ptr2 = P_Calculate_Parikh( GL_ParikhArray2, event2 );
/*
Display_Parikh_Array( (PARIKH_VECTOR) GL_ParikhArray1 );
Display_Parikh_Array( (PARIKH_VECTOR) GL_ParikhArray2 );
*/
    ptr1++;
    ptr2++;
    count = GL_net->NumTransitions + 1;

    while ( --count && ( *ptr1 == *ptr2 ) )
      {
        ptr1++;
        ptr2++;
      } /* while */

    if ( count )
      {
        if ( *ptr1 < *ptr2 )
          {
/*
printf("==> %s\n", *ptr1 ? "SMALLER": "GREATER" );
*/
            return ( *ptr1 ? -1 : 1 );
          } /* if */
        else /* *ptr1 > *ptr2 */
          {
/*
printf("==> %s\n", *ptr2 ? "GREATER" : "SMALLER" );
*/
            return ( *ptr2 ? 1 : -1 );
          } /* else */
      } /* if */
    else
      {
/*
        printf( "EQUALITY CASE FOUND!\n" );
*/
        GL_EV_equalities++;
/*
Display_Parikh_Array( (PARIKH_VECTOR) GL_ParikhArray1 );
Display_Parikh_Array( (PARIKH_VECTOR) GL_ParikhArray2 );
*/
        return ( P_Compare_Configs_LevelWise( event1, event2 ) );
      } /* else */

  } /* P_compare() */


/*****************************************************************************/
/* Print a T-Parikh vector.                                                  */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void Display_Parikh_Vector( PARIKH_VECTOR parikh )

#else

void Display_Parikh_Vector( parikh )

  PARIKH_VECTOR parikh;

#endif

  {

    register PARIKH_VECTOR ptr = parikh;

    printf( "<(%u)", *( (Cardinal32 *) ptr++ ) );

    if ( ptr && ptr->numname )
      {
        if ( ptr->count > 1 )
          printf( "%u*", ptr->count );
        printf( "T%u", ptr->numname );
        while ( (++ptr)->numname )
          {
            putchar( ',' );
            if ( ptr->count > 1 )
              printf( "%u*", ptr->count );
            printf( "T%u", ptr->numname );
          } /* while */
      } /* if */
    else
      {
        printf( "empty" );
      } /* else */

    putchar( '>' );
    putchar( '\n' );

  } /* Display_Parikh_Vector() */


/*****************************************************************************/
/* Allocate an empty T-Parikh vector.                                        */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

PARIKH_VECTOR Empty_Parikh_Vector( void )

#else

PARIKH_VECTOR Empty_Parikh_Vector()

#endif

  {

    return ( (PARIKH_VECTOR) ( GL_ParikhInAdvance ?  MYmalloc(
                               sizeof( Parikh_Vector_Entry ) << 1 ) : NULL ) );

  } /* Empty_Parikh_Vector() */


/*****************************************************************************/
/* Calculate the T-Parikh vector for an event of the unfolding.              */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

PARIKH_VECTOR Calculate_Parikh_Vector( Event event )

#else

PARIKH_VECTOR Calculate_Parikh_Vector( event )

  Event event;

#endif

  {

    register Event         ev,
                           * stackEptr;
    register STEdge        ce;
    register PARIKH_VECTOR parikh,
                           copy;
    register Cardinal32    size,
                           elem,
                           count;
    register PV_Item       item;
    PARIKH_VECTOR          destination;

    IncEMark();
    InitEStack();
    if ( ! GL_parikh_vector )
      {
        if ( GL_net->NumTransitions > PV_Item_Max )
          {
            fprintf( stderr, "Change PV_Item for Parikh vectors.\n" );
            exit( 111 );
          } /* if */
        GL_parikh_vector = (PARIKH_VECTOR) MYmalloc(
              ( GL_net->NumTransitions + 2 ) * sizeof( Parikh_Vector_Entry ) );
      } /* if */
    ( parikh = GL_parikh_vector )->numname = 0;
    (++parikh)->numname = ( (Transition) ( ev = event )->inscript )->numname;
    ( parikh + ( parikh->count = size = elem = 1 ) )->numname = 0;
    EMark( ev );
    for ( ce = ev->preset ; ce ; ce = ce->nextplace )
      {
        PushEvent( ce->place );
      } /* for */

    while ( (ev = PopEvent()) )
      {
        elem++;
        if ( ( item = ( (Transition) ev->inscript )->numname )
                                                            < parikh->numname )
          {
            if ( item < ( GL_parikh_vector + 1 )->numname )
              {
                for ( parikh = GL_parikh_vector + ( count = ++size ) + 1 ;
                                count-- ; *parikh = *( parikh - 1 ), parikh-- )
                  skip;
                parikh->numname = item;
                parikh->count = 1;
              } /* if */
            else
              {
                for ( skip ; item < parikh->numname ; parikh-- )
                  skip;
                if ( item == parikh->numname )
                  {
                    if ( ! ++( parikh->count ) )
                      {
                        fprintf( stderr,"Size of PV_Count is too small!\n" );
                        exit( 111 );
                      } /* if */
                  } /* if */
                else
                  {
                    count = GL_parikh_vector + ++size - parikh;
                    for ( parikh += ++count ; --count ;
                                          *parikh = *( parikh - 1 ), parikh-- )
                      skip;
                    parikh->numname = item;
                    parikh->count = 1;
                  } /* else */
              } /* else */
          } /* if */
        else if ( item > parikh->numname )
          {
            if ( item > ( GL_parikh_vector + size )->numname )
              {
                ( parikh = GL_parikh_vector + ++size )->numname = item;
                ( parikh + ( parikh->count = 1 ) )->numname = 0;
              } /* if */
            else
              {
                while ( parikh->numname && ( item > parikh->numname ) )
                  {
                    parikh++;
                  } /* while */
                if ( item == parikh->numname )
                  {
                    if ( ! ++(parikh->count) )
                      {
                        fprintf( stderr, "PV_Count size is too small!\n" );
                        exit( 111 );
                      } /* if */
                  } /* if */
                else
                  {
                    parikh += ( count=GL_parikh_vector + ++size - parikh + 1 );
                    for ( skip ; count-- ; *parikh = *(parikh - 1), parikh-- )
                      skip;
                    parikh->numname = item;
                    parikh->count = 1;
                  } /* else */
              } /* else */
          } /* else if */
        else if ( ! ++(parikh->count) )
          {
            fprintf( stderr, "PV_Count size is too small for this net!\n" );
            exit ( 111 );
          } /* else if */

        for ( ce = ev->preset ; ce ; ce = ce->nextplace )
          {
            PushEvent( ce->place );
          } /* for */
      } /* while */

    *( (Cardinal32 *) ( destination = copy = (PARIKH_VECTOR)
           MYmalloc( ( ++size + 1 ) * sizeof( Parikh_Vector_Entry ) ) ) ) = elem;
    for ( parikh = GL_parikh_vector ; size-- ; *++copy = *++parikh )
      skip;

    return ( destination );

  } /* Calculate_Parikh_Vector() */


/*****************************************************************************/
/* Calculate the T-Parikh vector for a transition.                           */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

PARIKH_VECTOR Calculate_Parikh_Vector_T( Transition trans )

#else

PARIKH_VECTOR Calculate_Parikh_Vector_T( trans )

  Transition trans;

#endif

  {

    register Event         ev,
                           event,
                           * stackEptr;
    register STEdge        ce;
    register PARIKH_VECTOR parikh,
                           copy;
    register Cardinal32    size,
                           elem,
                           count;
    register PV_Item       item;
    register Condition     * coptr;
    PARIKH_VECTOR          destination;

    IncEMark();
    InitEStack();
    if ( ! GL_parikh_vector )
      {
        if ( GL_net->NumTransitions > PV_Item_Max )
          {
            fprintf( stderr, "Change PV_Item for Parikh vectors.\n" );
            exit( 111 );
          } /* if */
        GL_parikh_vector = (PARIKH_VECTOR) MYmalloc(
              ( GL_net->NumTransitions + 2 ) * sizeof( Parikh_Vector_Entry ) );
      } /* if */
    ( parikh = GL_parikh_vector )->numname = 0;
    (++parikh)->numname = trans->numname;
    ( parikh + ( parikh->count = size = elem = 1 ) )->numname = 0;
    count = (Cardinal32) trans->inscript;
    for ( coptr = GL_CArray ; count-- ; coptr++ )
      {
        PushEvent( *coptr );
      } /* for */

    while ( (ev = PopEvent()) )
      {
        elem++;
        if ( ( item = ( (Transition) ev->inscript )->numname )
                                                            < parikh->numname )
          {
            if ( item < ( GL_parikh_vector + 1 )->numname )
              {
                for ( parikh = GL_parikh_vector + ( count = ++size ) + 1 ;
                                count-- ; *parikh = *( parikh - 1 ), parikh-- )
                  skip;
                parikh->numname = item;
                parikh->count = 1;
              } /* if */
            else
              {
                for ( skip ; item < parikh->numname ; parikh-- )
                  skip;
                if ( item == parikh->numname )
                  {
                    if ( ! ++(parikh->count) )
                      {
                        fprintf( stderr,"Size of PV_Count is too small!\n" );
                        exit( 111 );
                      } /* if */
                  } /* if */
                else
                  {
                    count = GL_parikh_vector + ++size - parikh;
                    for ( parikh += ++count ; --count ;
                                          *parikh = *( parikh - 1 ), parikh-- )
                      skip;
                    parikh->numname = item;
                    parikh->count = 1;
                  } /* else */
              } /* else */
          } /* if */
        else if ( item > parikh->numname )
          {
            if ( item > ( GL_parikh_vector + size )->numname )
              {
                ( parikh = GL_parikh_vector + ++size )->numname = item;
                ( parikh + ( parikh->count = 1 ) )->numname = 0;
              } /* if */
            else
              {
                while ( parikh->numname && ( item > parikh->numname ) )
                  {
                    parikh++;
                  } /* while */
                if ( item == parikh->numname )
                  {
                    if ( ! ++(parikh->count) )
                      {
                        fprintf( stderr, "PV_Count size is too small!\n" );
                        exit( 111 );
                      } /* if */
                  } /* if */
                else
                  {
                    parikh += ( count=GL_parikh_vector + ++size - parikh + 1 );
                    for ( skip ; count-- ; *parikh = *(parikh - 1), parikh-- )
                      skip;
                    parikh->numname = item;
                    parikh->count = 1;
                  } /* else */
              } /* else */
          } /* else if */
        else if ( ! ++(parikh->count) )
          {
            fprintf( stderr, "PV_Count size is too small for this net!\n" );
            exit ( 111 );
          } /* else if */

        for ( ce = ev->preset ; ce ; ce = ce->nextplace )
          {
            PushEvent( ce->place );
          } /* for */
      } /* while */

    *( (Cardinal32 *) ( destination = copy = (PARIKH_VECTOR)
           MYmalloc( ( ++size + 1 ) * sizeof( Parikh_Vector_Entry ) ) ) ) = elem;
    for ( parikh = GL_parikh_vector ; size-- ; *++copy = *++parikh )
      skip;

    return ( destination );

  } /* Calculate_Parikh_Vector_T() */


/*****************************************************************************/
/* Compare two T-Parikh vectors lexicographically.                           */
/*****************************************************************************/

/* &&& */
#if defined(__STDC__) || defined(__cplusplus)

int Compare_Parikh_Vectors_A( PARIKH_VECTOR parikh1,
                                     PARIKH_VECTOR parikh2,
                                     Condition     * condarray1,
                                     Condition     * condarray2 )

#else

int Compare_Parikh_Vectors_A( parikh1, parikh2, condarray1, condarray2 )

  PARIKH_VECTOR parikh1,
                parikh2;
  Condition     * condarray1,
                * condarray2;

#endif

  {

    register PARIKH_VECTOR ptr1 = parikh1,
                           ptr2 = parikh2;

GL_countQueueCompares++;
    if ( ! GL_ParikhInAdvance )
      {
        if ( (Cardinal32) parikh1 < (Cardinal32) parikh2 )
          {
            return ( -1 );
          } /* if */
        else if ( (Cardinal32) parikh1 > (Cardinal32) parikh2 )
          {
            return ( 1 );
          } /* if */
        return ( Compare_Parikh_Arrays_A( parikh1, parikh2, condarray1,
                                                                condarray2 ) );
      } /* if */
#ifdef _TRACE_EV_QUEUE_
printf( "Vector1: ");
Display_Parikh_Vector( parikh1 );
printf( "Vector2: ");
Display_Parikh_Vector( parikh2 );
#endif /* _TRACE_EV_QUEUE_ */
    if ( *( (Cardinal32 *) ptr1 ) < *( (Cardinal32 *) ptr2 ) )
      {
        return ( -1 );
      } /* if */
    else if ( *( (Cardinal32 *) ptr1 ) > *( (Cardinal32 *) ptr2 ) )
      {
        return ( 1 );
      } /* if */
    ptr1++;
    ptr2++;

    while (      ptr1->numname && ptr2->numname
            && ( ptr1->numname == ptr2->numname )
            && ( ptr1->count == ptr2->count ) )
      {
        ptr1++;
        ptr2++;
      } /* while */

    if ( ! ( ptr1->numname || ptr2->numname ) )
      {
/*
        printf( "EQUALITY CASE FOUND!\n" );
*/
        GL_EV_equalities++;
        return ( Compare_Configs_LevelWise_A( condarray1, condarray2 ) );
      } /* if */

    return ( ptr1->numname && ptr2->numname ?
                                     ptr1->numname == ptr2->numname ?
                                          ptr1->count < ptr2->count ? -1 : 1
                                    : ptr1->numname < ptr2->numname ? -1 : 1
                                : ptr1->numname ? 1 : ptr2->numname ? -1 : 0 );

  } /* Compare_Parikh_Vectors_A() */


/*****************************************************************************/
/* Compare two T-Parikh vectors lexicographically.                           */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

int Compare_Parikh_Vectors_L( PARIKH_VECTOR parikh1,
                                     PARIKH_VECTOR parikh2,
                                     Event         event1,
                                     Event         event2 )

#else

int Compare_Parikh_Vectors_L( parikh1, parikh2, event1, event2 )

  PARIKH_VECTOR parikh1,
                parikh2;
  Event         event1,
                event2;

#endif

  {

    register PARIKH_VECTOR ptr1 = parikh1,
                           ptr2 = parikh2;

    if ( *( (Cardinal32 *) ptr1 ) < *( (Cardinal32 *) ptr2 ) )
      {
        return ( -1 );
      } /* if */
    else if ( *( (Cardinal32 *) ptr1 ) > *( (Cardinal32 *) ptr2 ) )
      {
        return ( 1 );
      } /* if */
    ptr1++;
    ptr2++;

    while (      ptr1->numname && ptr2->numname
            && ( ptr1->numname == ptr2->numname )
            && ( ptr1->count == ptr2->count ) )
      {
        ptr1++;
        ptr2++;
      } /* while */

    if ( ! ( ptr1->numname || ptr2->numname ) )
      {
/*
        printf( "EQUALITY CASE FOUND!\n" );
*/
        GL_EV_equalities++;
        return ( Compare_Configs_LevelWise_L( event1, event2 ) );
      } /* if */

    return ( ptr1->numname && ptr2->numname ?
                                     ptr1->numname == ptr2->numname ?
                                          ptr1->count < ptr2->count ? -1 : 1
                                    : ptr1->numname < ptr2->numname ? -1 : 1
                                : ptr1->numname ? 1 : ptr2->numname ? -1 : 0 );

  } /* Compare_Parikh_Vectors_L() */


/*****************************************************************************/
/* Calculate the T-Parikh vector size for an event of the unfolding.         */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

PARIKH_VECTOR Calculate_PSize_T( Transition trans )

#else

PARIKH_VECTOR Calculate_PSize_T( trans )

  Transition trans;

#endif

  {

    register Event        event,
                          * stackEptr;
    register STEdge       ce;
    register Cardinal32   count;
    register Condition    * coptr;

    IncEMark();
    InitEStack();
    count = (Cardinal32) trans->inscript;
    for ( coptr = GL_CArray ; count-- ; coptr++ )
      {
        PushEvent( *coptr );
      } /* for */

    count = 1;
    while ( (event = PopEvent()) )
      {
        count++;
        for ( ce = event->preset ; ce ; ce = ce->nextplace )
          {
            PushEvent( ce->place );
          } /* for */
      } /* while */

    return ( (PARIKH_VECTOR) count );

  } /* Calculate_PSize_T() */


