/*###########################################################################*/
/*#              ROUTINES FOR THE IMPROVED UNFOLDING ALGORITHM              #*/
/*###########################################################################*/

#ifndef __PARIKH_H__
#define __PARIKH_H__

#include "global.h"


/*****************************************************************************/
/* Type definitions for Parikh vector arrays.                                */
/*****************************************************************************/

typedef unsigned short int Parikh_Array_Entry;

typedef Parikh_Array_Entry * PARIKH_ARRAY;

extern Condition * GL_CArray;
extern Cardinal32 GL_countQueueCompares;

/*****************************************************************************/
/* Type definitions for Parikh vectors.                                      */
/*****************************************************************************/

typedef Cardinal PV_Item;

#define PV_Item_Max 0xFFFE

typedef Cardinal PV_Count;

typedef struct Parikh_Vector_Entry_
  {
    PV_Item  numname;
    PV_Count count;
  } Parikh_Vector_Entry,
    * PARIKH_VECTOR;

extern bool      GL_ParikhInAdvance;

extern PARIKH_ARRAY GL_ParikhArray1,
                    GL_ParikhArray2;

/*****************************************************************************/
/* Some function pointer definitions.                                        */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern void          (* GL_Display_Parikh)( PARIKH_VECTOR );
extern PARIKH_VECTOR (* GL_Empty_Parikh)( void );
extern PARIKH_VECTOR (* GL_Calculate_Parikh)( Event );
extern PARIKH_VECTOR (* GL_Calculate_Parikh_T)( Transition );
extern PARIKH_VECTOR (* GL_Calculate_PSize_T)( Transition );
extern int           (* GL_Compare_Parikh_A)( PARIKH_VECTOR, PARIKH_VECTOR,
                                                   Condition *, Condition * );
extern int           (* GL_Compare_Parikh_L)( PARIKH_VECTOR, PARIKH_VECTOR,
                                                               Event, Event );

#else

extern void          (* GL_Display_Parikh)();
extern PARIKH_VECTOR (* GL_Empty_Parikh)();
extern PARIKH_VECTOR (* GL_Calculate_Parikh)();
extern PARIKH_VECTOR (* GL_Calculate_Parikh_T)();
extern PARIKH_VECTOR (* GL_Calculate_PSize_T)();
extern int           (* GL_Compare_Parikh_A)();
extern int           (* GL_Compare_Parikh_L)();

#endif


/*****************************************************************************/
/* Print a T-Parikh vector array table.                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void Display_Parikh_Array_Table( PARIKH_ARRAY parikh_table,
                                        Cardinal32   vecsize,
                                        Cardinal32   maxlevel );

#else

void Display_Parikh_Array_Table( parikh_table, vecsize, maxlevel );

#endif

/*****************************************************************************/
/* Print a T-Parikh vector array.                                            */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void Display_Parikh_Array( PARIKH_VECTOR parikh );

#else

void Display_Parikh_Array( parikh );

#endif

/*****************************************************************************/
/* Allocate an empty T-Parikh vector array.                                  */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

PARIKH_VECTOR Empty_Parikh_Array( void );

#else

PARIKH_VECTOR Empty_Parikh_Array();

#endif

/*****************************************************************************/
/* Calculate the T-Parikh vector array for an event of the unfolding.        */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

PARIKH_VECTOR Calculate_Parikh_Array( Event event );

#else

PARIKH_VECTOR Calculate_Parikh_Array( event );

#endif

/*****************************************************************************/
/* Calculate the T-Parikh vector array for an event of the unfolding.        */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

PARIKH_VECTOR Calculate_Parikh_Array_T( Transition trans );

#else

PARIKH_VECTOR Calculate_Parikh_Array_T( trans );

#endif

/*****************************************************************************/
/* Calculate the T-Parikh vector array for an event of the unfolding.        */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

PARIKH_ARRAY Calculate_Parikh_Array_T_extern( PARIKH_ARRAY parikh,
                                                     Condition    * condarray );

#else

PARIKH_ARRAY Calculate_Parikh_Array_T_extern( parikh, condarray );

#endif

/*****************************************************************************/
/* Compare two T-Parikh vector arrays lexicographically.                     */
/*****************************************************************************/

/* &&& */
#if defined(__STDC__) || defined(__cplusplus)

int Compare_Parikh_Arrays_A( PARIKH_VECTOR parikh1,
                                    PARIKH_VECTOR parikh2,
                                    Condition     * condarray1,
                                    Condition     * condarray2 );

#else

int Compare_Parikh_Arrays_A( parikh1, parikh2, condarray1, condarray2 );

#endif

/*****************************************************************************/
/* Compare two T-Parikh vector arrays lexicographically.                     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

int Compare_Parikh_Arrays_L( PARIKH_VECTOR parikh1,
                                    PARIKH_VECTOR parikh2,
                                    Event         event1,
                                    Event         event2 );

#else

int Compare_Parikh_Arrays_L( parikh1, parikh2, event1, event2 );

#endif

/*****************************************************************************/
/* Compare two T-Parikh vector arrays lexicographically.                     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

int P_compare( Event event1, Event event2 );

#else

int P_compare( event1, event2 );

#endif

/*****************************************************************************/
/* Print a T-Parikh vector.                                                  */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void Display_Parikh_Vector( PARIKH_VECTOR parikh );

#else

void Display_Parikh_Vector( parikh );

#endif

/*****************************************************************************/
/* Allocate an empty T-Parikh vector.                                        */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

PARIKH_VECTOR Empty_Parikh_Vector( void );

#else

PARIKH_VECTOR Empty_Parikh_Vector();

#endif

/*****************************************************************************/
/* Calculate the T-Parikh vector for an event of the unfolding.              */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

PARIKH_VECTOR Calculate_Parikh_Vector( Event event );

#else

PARIKH_VECTOR Calculate_Parikh_Vector( event );

#endif

/*****************************************************************************/
/* Calculate the T-Parikh vector for a transition.                           */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

PARIKH_VECTOR Calculate_Parikh_Vector_T( Transition trans );

#else

PARIKH_VECTOR Calculate_Parikh_Vector_T( trans );

#endif

/*****************************************************************************/
/* Compare two T-Parikh vectors lexicographically.                           */
/*****************************************************************************/

/* &&& */
#if defined(__STDC__) || defined(__cplusplus)

int Compare_Parikh_Vectors_A( PARIKH_VECTOR parikh1,
                                     PARIKH_VECTOR parikh2,
                                     Condition     * condarray1,
                                     Condition     * condarray2 );

#else

int Compare_Parikh_Vectors_A( parikh1, parikh2, condarray1, condarray2 );

#endif

/*****************************************************************************/
/* Compare two T-Parikh vectors lexicographically.                           */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

int Compare_Parikh_Vectors_L( PARIKH_VECTOR parikh1,
                                     PARIKH_VECTOR parikh2,
                                     Event         event1,
                                     Event         event2 );

#else

int Compare_Parikh_Vectors_L( parikh1, parikh2, event1, event2 );

#endif

/*****************************************************************************/
/* Calculate the T-Parikh vector size for an event of the unfolding.         */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

PARIKH_VECTOR Calculate_PSize_T( Transition trans );

#else

PARIKH_VECTOR Calculate_PSize_T( trans );

#endif

#endif /* __PARIKH_H__ */
