#include "queue.h"
#include "sub_unf.h"
#include "unfold.h"

/* Data structure: queue (holding events) ( doubly linked list ). */
typedef struct q_elem_
  {
    Event          event;
    Cardinal32     LocalConfigDepth;
    struct q_elem_ *prev, *next;
  } QUEUE_ELEM,
    *QUEUE;

typedef struct
{
  QUEUE first;
  QUEUE last;
} QUEUE_HEAD;

/* Ereignisqueue (entspricht pe) */
/* bei queueE->first entfernen, bei queueE->last anfuegen */
QUEUE_HEAD* GL_queueE;

/*****************************************************************************/
/* Operations on the event queue (pe)                                        */
/*****************************************************************************/

void init_queue(void)
{
  GL_queueE=(QUEUE_HEAD*)MYmalloc(sizeof(QUEUE_HEAD));
  GL_queueE->first = GL_queueE->last = NULL;
}

void delete_queue(void)
{
  Free(GL_queueE);
}

bool queue_not_empty(void)
{
  return ((bool)GL_queueE->first);
}

/****************************************************************************/
/* Remove first event from global queue and return it, NULL otherwise       */

Event pop_queue(void)
{
  Event event = NULL;
  QUEUE qudel=GL_queueE->first;

  if (qudel)
    {
      event=qudel->event;
      if ( (GL_queueE->first = GL_queueE->first->next) )
	{
	  GL_queueE->first->prev = NULL;
	} /* if */
      else
	{
	  GL_queueE->last = NULL;
	} /* else */
      /* Listeneintrag entfernen */
      Free( qudel );
    }
  return event;
}

/* LocalConfigDepth() */
static Cardinal32 localConfigDepth(Event event)
{
  register Cardinal32 maxsize=0,
                      size;
  register STEdge     ce;
  register TSEdge     ec;

  for ( ce = event->preset ; ce ; ce = ce->nextplace )
    {
      ec = ce->place->preset;
      if ( ec && ( ( size = (Cardinal32) ec->trans->label ) > maxsize ))
	maxsize = size;
    } /* for */
  return maxsize;
}

/* LocalConfigSize() */
static Cardinal32 localConfigSize(Event event)
{
  register Cardinal32 size = 1;
  register STEdge     ce;
  register Event      ev, *stackEptr;

  IncEMark();
  InitEStack();
  EMark( event );
  for ( ce = event->preset ; ce ; ce = ce->nextplace )
    {
      PushEvent( ce->place );
    } /* for */
  while ( (ev = PopEvent()) )
    {
      size++;
      for ( ce = ev->preset ; ce ; ce = ce->nextplace )
	{
	  PushEvent( ce->place );
	} /* for */
    } /* while */
  return size;
}

/****************************************************************************/
/* Insert event in queue */

#if defined(__STDC__) || defined(__cplusplus)

void insert_queue( Event event )

#else

void insert_queue( event )

  Event     event;

#endif

{
  register QUEUE      qunew,
                      qu;
  register Cardinal32 maxsize,
                      size;
#define _TAKE_CONFIG_SIZE_

  maxsize=localConfigDepth(event);

#ifndef _TAKE_CONFIG_SIZE_
  size = maxsize;
#else /* _TAKE_CONFIG_SIZE_ */
  size = localConfigSize(event);
#endif /* _TAKE_CONFIG_SIZE_ */

  qunew = (QUEUE) MYmalloc( sizeof( QUEUE_ELEM ) );
  qunew->event = event;
  event->label = (MultiSet) ++maxsize;
  qunew->LocalConfigDepth = size;
/*
  event->prevtrans = (Transition) size;
*/
  /* Von hinten durchsuchen und einhaengen */
  if ( ( qu = GL_queueE->last ) && (( size < qu->LocalConfigDepth ) ||
       ((unfold==NERV_unfold) && ( maxsize == qu->LocalConfigDepth )
	&& ( P_compare( event, qu->event ) < 0 )) ) )
    {
      while ( (( qu = qu->prev ) && ( size < qu->LocalConfigDepth )) ||
	      ((unfold==NERV_unfold) && ( maxsize == qu->LocalConfigDepth )
	       && ( P_compare( event, qu->event ) < 0 ) ))
	skip;
      if ( qu )
	{
	  ( qu->next = ( qunew->next = qu->next )->prev = qunew )->prev = qu;
	} /* if */
      else
	{
	  ( qunew->next = GL_queueE->first )->prev = qunew;
	  ( GL_queueE->first = qunew )->prev = NULL;
	} /* else */
    } /* if */
  /* sonst hinten einhaengen */
  else if ( qu )
    {
      ( GL_queueE->last = ( ( GL_queueE->last->next = qunew )->prev
			   = GL_queueE->last )->next )->next = NULL;
    } /* else if */
  /* sonst neu anlegen */
  else
    {
      qunew->prev = qunew->next = NULL;
      GL_queueE->first = GL_queueE->last = qunew;
    } /* else */
} /* insert_queue */
