/*****************************************************************************/
/*               AUXILIARY ROUTINES FOR THE UNFOLDING PROCEDURE              */
/*****************************************************************************/

#include <time.h>
#include "sub_unf.h"
#include "poss_ext.h"
#include "nets.h"
#include "queue.h"
#include "unfold.h"

/*****************************************************************************/
/* Some function pointer definitions.                                        */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

bool  (* GL_LiConflictsFunction)( LIST * ),
             LiConflictsNet( LIST * );

#else

bool  (* GL_LiConflictsFunction)(),
             LiConflictsNet();

#endif

/* Time management */

time_t   GL_time;
Cardinal GL_time_inc;

/* start time measuring */
void start_clock(void)
{
  GL_time = clock();
  GL_time_inc = 0;
}

void stop_clock(void)
{
  GL_time = clock() - GL_time;
}

/* since the clock measures only 36 minutes the value has to be updated 
   sometimes with this macro */
void update_clock(void)
{

  register time_t now;
  static time_t time_old=0;

  if ( (now = clock()) < time_old )
    GL_time_inc++;
  time_old = now;
}

void print_time(FILE* file)
{
  fprintf( file, "time: %9.2f s\n",
	  ( ( (double)(unsigned long) GL_time ) / CLOCKS_PER_SEC )
	  + 4295.0 * GL_time_inc );
}

/*****************************************************************************/
/* Stackfunctions                                                            */
/*****************************************************************************/

/* Globale Variablen */
Cardinal32        GL_stackCsize,
                  GL_stackEsize;

#if defined(__STDC__) || defined(__cplusplus)

void CreateStacks( void )

#else

void CreateStacks( )

#endif
{
  GL_stackCsize = GL_stackEsize = DYN_STACKSIZE;
  if ( GL_flags & CHECK_CONFIGS )
    {
      printf( "STACKSIZE COND'S = %u\n", GL_stackCsize );
      printf( "STACKSIZE EVENTS = %u\n", GL_stackEsize );
    } /* if */
  GL_stackC = (Place *) MYmalloc( GL_stackCsize * sizeof( Place ) );
  GL_stackC2 = (Place *) MYmalloc( GL_stackCsize * sizeof( Place ) );
  GL_stackE = (Event *) MYmalloc( GL_stackEsize * sizeof( Event ) );
  /* Groesse: (Anzahl Bed. auf dem Stack + Nachbereich der Events +1)/8 +1 */
  GL_conflictArray = 
    mymalloc_bitarray(( ( ( GL_stackCsize + GL_maxTpostset + 1 ) >> 3 ) + 1 ));
}

#if defined(__STDC__) || defined(__cplusplus)

void FreeStacks( void )

#else

void FreeStacks( )

#endif
{
  Free( GL_stackC );
  Free( GL_stackC2 );
  Free( GL_stackE );
  Free( GL_conflictArray );
}

#if defined(__STDC__) || defined(__cplusplus)

void ResizeCStack( Cardinal32 num )

#else

void ResizeCStack( num )

  Cardinal32 num;

#endif
{
  if ( num > GL_stackCsize )
    {
      GL_stackC = (Condition *) MYrealloc( GL_stackC,
                      ( GL_stackCsize += DYN_STACKSIZE ) * sizeof( Place ) );
      GL_stackC2 = (Condition *) MYrealloc( GL_stackC2,
                                           GL_stackCsize * sizeof( Place ) );
      GL_conflictArray = myrealloc_bitarray( GL_conflictArray,
        ( ( ( GL_stackCsize + GL_maxTpostset + 1 ) >> 3 ) + 1 ) );
      if ( GL_flags & CHECK_CONFIGS )
        {
          printf( "STACKSIZE COND'S = %u\n", GL_stackCsize );
        } /* if */
    }
}

#if defined(__STDC__) || defined(__cplusplus)

void ResizeEStack( Cardinal32 num )

#else

void ResizeEStack( num )

  Cardinal32 num;

#endif
{
  if ( num > GL_stackEsize )
    {
      GL_stackE = (Event *) MYrealloc( GL_stackE,
                      ( GL_stackEsize += DYN_STACKSIZE ) * sizeof( Event ) );
      if ( GL_flags & CHECK_CONFIGS )
        printf( "STACKSIZE EVENTS = %u\n",GL_stackEsize );
    } /* if */
}

/****************************************************************************/
/* Berechnet per globalem Stackpointer ein neues ConflictArray              */
/* Seiteneffekte: GL_stackC2ptr wird geleert                                */
/****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

BIT_ARRAY Calculate_Conflicts( BIT_ARRAY conflictArray, LMPNet unfold_net,
			       Transition trans)

#else

BIT_ARRAY Calculate_Conflicts( conflictArray, unfold_net, trans )

  BIT_ARRAY conflictArray;
  LMPNet unfold_net;
  Transition trans;

#endif

  {

    register Condition  cond,
                        * stackptr;
    register Event      ev;
    register TSEdge     ec;
    register STEdge     ce;
    register BIT_ARRAY  conflArrayPtr=NULL;

    /* Hier sind alle Bed. der lokalen Konfiguration des soeben */
    /* eingefuegten Ereignisses enthalten                       */
    stackptr = GL_stackC2ptr;
    IncEMark();
    /* Alle Bedingungen im Stack C2 durchlaufen und Markierungen erneuern */
    while (( cond = *(stackptr--) ))
      {
        /* Wenn es einen Vorbereich gibt diesen markieren */
        if ( cond->preset )
	  EMark( cond->preset->trans );
      } /* while */
    stackptr = GL_stackC2ptr;

    /* Nocheinmal alle durchlaufen und neue Bed. einfuegen */
    while ( (cond = PopCondition()) )
      {
        /* Diesmal den Nachbereich durchlaufen */
        for ( ce = cond->postset ; ce ; ce = ce->nexttrans )
          {
            /* Alle nicht markierten markieren */
            if ( IsNotEMarked( ev = ce->trans ) )
              {
                EMark( ev );
                /* Testen, ob der Nachbereich dieser Events in Konflikt */
                /* steht, wenn nicht auf den Stack legen zum Bearbeiten */
                /* und Bit setzen, da nun Konflikt moeglich             */
                for ( ec = ev->postset ; ec ; ec = ec->nextplace )
                  {
                    cond = ec->place;
                    if ( ! Test_Bit( conflictArray, cond->numname ))
                      {
                        PushCondition( cond );
                        Set_Bit( conflictArray, cond->numname );
                      } /* if !TestBit */
                  } /* for ec */
              } /* if IsNotEMarked */
          } /* for ce */
      } /* while ( cond = popCond() )  */

    if ( ( GL_maxTpreset > 2 ) 
	 && ( GL_changeAlgo > unfold_net->NumTransitions ) )
      {
	/* Hilfspointer zum Durchlaufen des Speicher */
	register BIT_ARRAY  conflArrayPtr2;
	/* This is the current size of conflictArray */
	register Cardinal32 size = ( unfold_net->NumPlaces + 
		 (Cardinal32) trans->prevtrans + 2 ) >> 3;
        size++;

	/* conflArray wird kopiert und zurueckgegeben */
	conflArrayPtr = mymalloc_bitarray(size) + size;
	conflArrayPtr2 = conflictArray + size;
	while ((size--))
	  *--conflArrayPtr = *--conflArrayPtr2;
      } /* if */
    else
      {
	/* sonst wird nur ein Array gebraucht, das unveraendert bleibt */
	conflArrayPtr = conflictArray;
      } /* else */

    /*
    printf("Konflikte: ");
    bit_print_numname(conflArrayPtr, unfold_net);
    */
    return conflArrayPtr;

  } /* Calculate_Conflicts() */


/*****************************************************************************/
/* Test whether there are any conflicts if the net would be expanded by      */
/* an event having the places stored in 'listArrayPtr' in its preset.        */
/* ( This routine uses bit arrays to check if there are conflicts. )         */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

bool LiConflictsBit( LIST * listArrayPtr )

#else

bool LiConflictsBit( listArrayPtr )

  LIST * listArrayPtr;

#endif

  {

    register LIST       * ptr;
    register BIT_ARRAY  array1 = (BIT_ARRAY) ( *listArrayPtr )->item->name;
    register Cardinal32 conum1 = ( *listArrayPtr )->item->numname,
                        conum2;

    for ( ptr = listArrayPtr - 1 ; *--ptr ; ptr-- )
      {
        if ( conum1 >= ( conum2 = ( *ptr )->item->numname ) )
          {
            if ( Test_Bit( array1, conum2 ) )
              {
                return ( True );
              } /* if */
          } /* if */
        else if ( Test_Bit( ( *ptr )->item->name, conum1 ) )
          {
            return ( True );
          } /* else if */
      } /* for ptr */

    return ( False );

  } /* LiConflictsBit() */


/*****************************************************************************/
/* Test whether there are any conflicts if the net would be expanded by      */
/* an event having the places stored in 'listArrayPtr' in its preset.        */
/* ( This routine checks the conflict property directly on the net. )        */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

bool LiConflictsNet( LIST * listArrayPtr )

#else

bool LiConflictsNet( listArrayPtr )

  LIST * listArrayPtr;

#endif

#ifdef CONDITION_STACK

  {

    register STEdge    ce;
    register LIST      * ptr = listArrayPtr + 1;
    register Condition cond,
                       * stackptr;
    register Event     event;

    IncCMark();
    IncEMark();
    InitCStack();

    /* Alle Bedingungen im Vorbereich auf den Stack und markieren */
    while ( *--ptr )
      {
        cond = ( *( ptr-- ) )->item;
        CMark( PushCondition( cond ) );
      } /* while */

    /* Alle erreichbaren Bedingungen im Vorbereich ueberpruefen */
    while ( (cond = PopCondition()) )
      {
        if ( IsNotEMarked( event = cond->preset->trans ) )
          {
            EMark( event );
            for ( ce = event->preset ; ce ; ce = ce->nextplace )
              {
                if ( IsCMarked( cond = ce->place ) )
                  {
                    return( True );
                  } /* if */
                CMark( PushCondition( cond ) );
              } /* for ce */
          } /* if IsNotMarked event */
      } /* while cond */

#else /* Event-Stack */

  {

    register STEdge    ce;
    register LIST      * ptr = listArrayPtr + 1;
    register Condition cond;
    register Event     event,
                       * stackEptr;

    IncCMark();
    IncEMark();
    InitEStack();

    /* Bedingungen markieren und Vorbereich ablegen */
    while ( *--ptr )
      {
        CMark( cond = ( *( ptr-- ) )->item );
        PushEvent( cond );
      } /* while */

    /* Alle erreichbaren Events im Vorbereich testen */
    while ( (event = PopEvent()) )
      {
        for ( ce = event->preset ; ce ; ce = ce->nextplace )
          {
            if ( IsCMarked( cond = ce->place ) )
              {
                return( True );
              } /* if */
            CMark( cond );
            PushEvent( cond );
          } /* for */
      } /* while event */

#endif /* CONDITION_STACK */

    return ( False );

  } /* LiConflictsNet() */


/****************************************************************************/
/* Berechnung des Endzustandes eines Ereignisses ( bzw. der damit verbunde- */
/* nen lokalen Konfiguration ). Entspricht einer einfachen Version von      */
/* FinalState                                                               */
/****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LIST CalculateCut( Event event, LMPNet unfold_net )

#else

LIST CalculateCut( event, unfold_net )

  Event event;
  LMPNet unfold_net;

#endif

  {

    register Place      cond,
                        * stackptr;
    register Event      ev;
    register STEdge     ce;
    register TSEdge     ec;
    LIST                placeList = NULL;

    if (event->inscript)
      {
	IncCMark();
	IncEMark();
	InitCStack();
	EMark( event );
	
	/* Im ersten Durchgang werden die dazugehoerigen Bed. und Ereignisse */
	/* markiert, dann wird die Liste erstellt. Zuerst den Vorbereich auf */
	/* den Stack legen und markieren */
	for ( ce = event->preset ; ce ; ce = ce->nextplace )
	  CMark( PushCondition( ce->place ) );
	/* Alle Bedingungen bearbeiten: Transitionen im Vorbereich markieren und */
	/* Bedingungen dessen Vorbereichs markieren und ggf. auf den Stack legen */
	while ( (cond = PopCondition()) )
	  {
	    if ( IsNotEMarked( ev = cond->preset->trans ) )
	      {
		EMark( ev );
		for ( ce = ev->preset ; ce ; ce = ce->nextplace )
		  {
		    CMark( cond = ce->place );
		    if ( ( ec = cond->preset ) && IsNotEMarked( ec->trans ))
		      PushCondition( cond );
		  } /* for */
	      } /* if */
	  } /* while cond */
	
	/* Nun die Liste erstellen, zuerst den Vorbereich des Events */
	IncEMark();
	InitCStack();
	for ( ce = event->preset ; ce ; ce = ce->nextplace )
	  PushCondition( ce->place );
	/* Nun alle auf dem Stack liegenden Ereignisse bearbeiten */
	while ( (cond = PopCondition()) )
	  {
	    ev=cond->preset->trans;
	    /* Dann von allen Vorgaengertransitionen die Nachfolgerstellen */
	    /* einfuegen, sofern noch nicht vorhanden                      */
	    for ( ec = ev->postset; ec ; ec = ec->nextplace )
	      {
		if ( !IsCMarked( cond = ec->place ))
		  list_insert( &placeList, cond, cond->tokenage );
	      } /* for */
	    /* Ggf. die Vorbereichststellen der Transitionen auf den Stack */
	    if ( IsNotEMarked( ev ) )
	      {
		EMark( ev );
		for ( ce = ev->preset ; ce ; ce = ce->nextplace )
		  {
		    cond = ce->place;
		    if ( ( ec = cond->preset ) && IsNotEMarked( ec->trans ))
		      PushCondition( cond );
		  } /* for */
	      } /* if */
	  } /* while cond */
	
	/* Nun noch den Nachbereich des aktuellen Events einfuegen   */
	/* Ist immer eine Schalttransition, da FinalState mit Events */
	/* aus der Queue aufgerufen wird                             */
	for ( ec = event->postset; ec; ec = ec->nextplace )
	  list_insert( &placeList, ec->place, 0 );
	
	/* Startmarkierung einfuegen, ist vorne in der Entfaltung enthalten */
	for ( cond = unfold_net->Places ; cond && ! cond->preset ;
	      cond = cond->nextplace )
	  {
	    if ( !IsCMarked( cond ))
	      list_insert( &placeList, cond, 0 );
	  } /* for */
      } /* if kein Tic */
    else
      {
	for ( ec = event->postset; ec; ec = ec->nextplace )
	  list_insert( &placeList, ec->place, ec->place->tokenage);
      }

    return ( placeList );

  } /* CalculateCut() */

#if defined(__STDC__) || defined(__cplusplus)

LIST CalculateCutFromPreset( LIST *preset_ptr, LMPNet unfold_net )

#else

LIST CalculateCutFromPreset( preset_ptr, unfold_net )

  LIST *preset_ptr;
  LMPNet unfold_net;

#endif

  {
    register Place      cond,
                        * stackptr;
    register Event      ev;
    register STEdge     ce;
    register TSEdge     ec;
    LIST                placeList = NULL,
                        *ptr;
    BIT_ARRAY           bitarray;
    Cardinal32          i;

    IncCMark();
    IncEMark();
    InitCStack();

    /* Im ersten Durchgang werden die dazugehoerigen Bed. und Ereignisse */
    /* markiert, dann wird die Liste erstellt. Zuerst den Vorbereich auf */
    /* den Stack legen und markieren */
    CMark( PushCondition( (Condition)*preset_ptr ) );
    for ( ptr=preset_ptr ; *++ptr ; ptr++ )
      CMark( PushCondition( (*ptr)->item ) );
    /* Alle Bedingungen bearbeiten: Transitionen im Vorbereich markieren und */
    /* Bedingungen dessen Vorbereichs markieren und ggf. auf den Stack legen */
    while ( (cond = PopCondition()) )
      {
        if ( IsNotEMarked( ev = cond->preset->trans ) )
          {
            EMark( ev );
            for ( ce = ev->preset ; ce ; ce = ce->nextplace )
              {
                CMark( cond = ce->place );
                if ( ( ec = cond->preset ) && IsNotEMarked( ec->trans ))
                  PushCondition( cond );
              } /* for */
          } /* if */
      } /* while cond */

    if (!bit_getsize())
      bit_setsize((unfold_net->NumPlaces>>3)+1);
    bitarray=bit_alloc();
    for(i=bit_getsize();i--;*(bitarray+i)='\0')
      skip;

    /* Nun die Liste erstellen, zuerst den Vorbereich des Events */
    IncEMark();
    InitCStack();
    PushCondition( (Condition)*preset_ptr );
    for ( ptr=preset_ptr ; *++ptr ; ptr++ )
        PushCondition( (*ptr)->item );
    /* Nun alle auf dem Stack liegenden Ereignisse bearbeiten */
    while ( (cond = PopCondition()) )
      {
        ev=cond->preset->trans;
        /* Dann von allen Vorgaengertransitionen die Nachfolgerstellen */
        /* einfuegen, sofern noch nicht vorhanden                      */
        for ( ec = ev->postset; ec ; ec = ec->nextplace )
          {
            if ( !IsCMarked( cond = ec->place ))
              {
                list_insert( &placeList, cond, cond->tokenage );
                Set_Bit(bitarray,cond->numname);
              }
          } /* for */
        /* Ggf. die Vorbereichststellen der Transitionen auf den Stack */
        if ( IsNotEMarked( ev ) )
          {
            EMark( ev );
            for ( ce = ev->preset ; ce ; ce = ce->nextplace )
              {
                cond = ce->place;
                if ( ( ec = cond->preset ) && IsNotEMarked( ec->trans ))
                  PushCondition( cond );
              } /* for */
          } /* if */
      } /* while cond */

    /* Nachbereich wird nicht eingefuegt, da Tic noch nicht enthalten   */
    /* aber den Preset sollte man einfuegen!                            */
    cond=(Condition)*preset_ptr;
    list_insert( &placeList, cond, cond->tokenage );
    Set_Bit(bitarray,cond->numname);
    for ( ptr=preset_ptr ; *++ptr ; ptr++ )
      {
        cond=(*ptr)->item;
        list_insert( &placeList, cond, cond->tokenage );
        Set_Bit(bitarray,cond->numname);
      }

    /* Startmarkierung einfuegen, ist vorne in der Entfaltung enthalten */
    for ( cond = unfold_net->Places ; cond && ! cond->preset ;
                                                       cond = cond->nextplace )
      {
        if ( !IsCMarked( cond ))
          {
            list_insert( &placeList, cond, 0 );
            Set_Bit(bitarray,cond->numname);
          }
      } /* for */
    return ( placeList );

  } /* CalculateCutFromPreset() */

/****************************************************************************/
/* Prueft, ob der gegebene Cut ein Tic-Event zulaesst.                      */
/* Dies ist nicht der Fall, wenn                                            */
/* a) Keine Transition mehr vorhanden ist                                   */
/* b) eine Transition die lft erreicht hat                                  */
/* c) bereits ein TIC eingefuegt wurde                                      */
/****************************************************************************/

static bool is_tic_cut (LIST cut)
  {
    register bool   do_tic=True, /* Tic may happen if do_tic is true */
                    some_postset=False, /* Tic not needed, if no postset with cut */
                    time_left=False; /* If current transition reached lft */
    register LIST   listptr, condlist;
    register STEdge ce, ceold;

    /* Check if already tic inserted */
    /* Just check for first condition since all have to be in cut */
    for(ce=cut->item->postset;ce;ce=ce->nexttrans)
      {
	if (!ce->trans->inscript)
	  {
	    /* Compare preset of tic with cut */
	    for(listptr=cut,ceold=ce->trans->preset;
		(ceold&&(ceold->place==listptr->item));
		ceold=ceold->nextplace, listptr=listptr->next)
	      skip;
	    if (!(ceold||listptr))
	      {
#ifdef _EVUnfold_LOG_TICS_
		printf("No TIC event\n\n");
#endif
		return False;
	      }
	  }
      }
		
    /* Check all conditions if Tic has to be inserted */
    for (listptr=cut;(listptr && do_tic);listptr=listptr->next)
      {
	/* Nachbereich weiter untersuchen */
        for(ce=((Place)listptr->item->inscript)->postset;do_tic && ce;
	    ce=ce->nexttrans)
          {
	    time_left=False;
	    /* Wenn kein Zeitlabel muss Transition sofort schalten.*/
	    /* Genau so, wenn lft erreicht.                   */
	    /* check if transition is activated */
	    /* ceold=NULL means trans is activated */
	    for(ceold=ce->trans->preset;ceold &&
	           (condlist=(LIST)list_contains(listptr,ceold->place));
		ceold=ceold->nextplace)
	      {
	        if (ce->trans->time_label&&
		    (ce->trans->time_label->high>condlist->tokenage))
		  time_left=True;
	      }
	    if (!ceold)
	      {
	        some_postset=True;
		if (!ce->trans->time_label)
		  do_tic=False;
		else
		  if (!time_left)
		    do_tic=False;
	      } /* if activated */
          } /* for ce */
      } /* for listptr */

    if (do_tic && some_postset)
      {
#ifdef _EVUnfold_LOG_TICS_
	printf("It's a TIC event\n\n");
#endif
	return True;
      }
    else
      {
#ifdef _EVUnfold_LOG_TICS_
	printf("No TIC event\n\n");
#endif
	return False;
      }
  }

/****************************************************************************/
/* Fuegt ggf. ein Tic-Event in die Entfaltung ein. Dazu wird ein Cut        */
/* uebergeben, an den ein Tic angehaengt werden soll. Auch der Nachbereich  */
/* des Tics und moegliche weitere Events werden in die Entfaltung bzw. in   */
/* die Event-Queue eingefuegt.                                              */
/* Achtung: cut enthaelt nun die Conditions!                                */
/****************************************************************************/
void InsertTic( LMPNet unfold_net, LIST cut )
  {
    LIST	newcut;
    register LIST       listptr, listptr2;
    register Condition  cond;
    register Event      event;
    register STEdge     ce, ceold;
    register TSEdge     ec;

#ifdef _EVUnfold_LOG_TICS_
printf("Start Cut:");
list_print(cut);
printf("\n");
#endif

/* check des direkten Nachbereichs auf lft=0 */
/* Zum Beschleunigen bei nicht-Zeitnetzen    */

    for (listptr=cut;listptr;listptr=listptr->next)
      {
	for (ce=((Place)listptr->item->inscript)->postset;
	     ce && (!(ce->trans->time_label) || 
		    (ce->trans->time_label->high<=listptr->tokenage));
	     ce=ce->nexttrans)
	  skip;
	if (ce)
	  break;
      }
    if (listptr)
      {

    if ( is_tic_cut(cut) )
      {
#ifdef _EVUnfold_LOG_TICS_
printf("TIC!\n");
#endif
        /* Neues Tic-Event in Entfaltung einfuegen */
	event=insert_tic(unfold_net);

        /* InsertCE_Edges() */
        ce = event->preset = (STEdge) MYmalloc( sizeof( STElem ) );
        ce->prevplace = NULL;
        ce->weight = 0;
        ce->nexttrans = NULL;
        ce->trans = event;
        /* Alle Kanten einfuegen. Die Bedingungen entsprechen dem Cut */
        for ( listptr=cut->next ; listptr ; listptr=listptr->next )
          {
            ( ce->nextplace = (STEdge) MYmalloc( sizeof( STElem ) ) )
                           ->prevplace = ce;
            ce = ce->nextplace;
            ce->trans = event;
            ce->weight=0;
            ce->nexttrans = NULL;
          } /* for */
        ce->nextplace = NULL;

        /* Die uebrigen Stellen an den Kanten einfuegen */
        for ( ce=event->preset, listptr=cut; listptr ;
              ce=ce->nextplace, listptr=listptr->next )
          {
            ce->place = cond = (Condition) listptr->item;
            if ((ceold = (STEdge) cond->marks ))
              {
                ( ceold->nexttrans = ce )->prevtrans = ceold;
              } /* if */
            else
              {
                ( cond->postset = ce )->prevtrans = NULL;
              } /* else */
            cond->marks = (Cardinal32) ce;
          } /* for ce=event */

	insert_queue(event);
      } /* if do_tic */

    	/* Hier weitere Cuts testen: Alle von diesem Cut aus erreichbaren! */
    	for (listptr=cut;listptr;listptr=listptr->next)
      	  {
	    /* Betrachte alle Transitionen in jedem Nachbereich */
            for (ce=listptr->item->postset;ce;ce=ce->nexttrans)
              {
		event=ce->trans;
		// only events with postset, all others are not yet processed!
                if (event->inscript && event->postset)
                  {
		    /* Test, ob aktiviert, dabei die breits vorher getesteten auslassen */
	            for(ceold=event->preset;ceold; ceold=ceold->nextplace)
//			  list_contains(listptr,(Place)ceold->place);
//			  list_contains(listptr,(Place)ceold->place->inscript);
		    {  LIST list=listptr;
		      while(list && (list->item!=(Place)ceold->place))
			list=list->next;
		      if (!list)
		        break;
		    }
                    /* Erzeuge neuen Cut */
		    if (!ceold)
		      {
			newcut=NULL;
			for (listptr2=cut;listptr2;listptr2=listptr2->next)
			  {
			    for(ceold=event->preset;ceold&&(listptr2->item!=ceold->place);
				ceold=ceold->nextplace)
			      skip;
			    if (!ceold)
			      list_insert(&newcut,listptr2->item,listptr2->item->tokenage);
			  }
			for(ec=event->postset;ec;ec=ec->nextplace)
			  list_insert(&newcut,ec->place,ec->place->tokenage);

#ifdef _EVUnfold_LOG_TICS_
printf("Rekursion!\n");
list_print(newcut);
#endif
			InsertTic(unfold_net, newcut);
		      } /* if active */
                  } /* if inscript && postset */
              } /* for ce */
      	  } /* for listptr */
      } /* if das alles noetig */
    list_delete(&cut);

  } /* InsertTic */
