/****************************************************************************/
/*     O P E R A T I O N S   F O R   U N F O L D I N G   O U T P U T        */
/****************************************************************************/

#include <string.h>
#include "unf_out.h"
#include "global.h"
#include "rw_mci.h"
#include "readlib.h"

/* Ausgabe des entfalteten Netzes mitsamt Cutoff-Events und maximalen       */
/* Prozessen in eine "*.ent"-Datei, die von Hartmut Truee innerhalb seiner  */
/* Diplomarbeit ( Model Checker ) gelesen werden kann.                      */

#if defined(__STDC__) || defined(__cplusplus)

void write_unfolding_ent( char * filename, LMPNet unfold_net )

#else

void write_unfolding_ent( filename, unfold_net )

  char * filename;
  LMPNet unfold_net;

#endif

  {

             FILE       * ENTfile = stdout;
             char       ENTfilename[NAMELENGTH];
    register Condition  cond;
    register Event      event;
    register STEdge     ce;
    register TSEdge     ec;
    register Cardinal32 len,
                        namelen,
                        orgnamelen,
                        translen;
    register char       * name;
    register LIST       list = NULL;


    if ( filename && *filename )
      {
        if (    ( ( len = strlen( strcpy( ENTfilename, filename ) ) ) < 4 )
             || strcmp( ENTfilename + len - 4, ".ent" ) )
          {
            strcat( ENTfilename, ".ent" );
          } /* if */
        if ( ! ( ENTfile = fopen( ENTfilename, "w" ) ) )
          {
            PEPerror("write_unfolding_ent: cannot open file for writing." );
          } /* if */
      } /* if filename */
    /* namelen ist max. Laenge des Eventnamen, orgnamelen die des Transitionsnamen */
    namelen = orgnamelen = 0;
    for ( event = unfold_net->Transitions; event; event = event->nexttrans )
      {
        if ( ( len = strlen( event->name ) ) > namelen )
          namelen = len;
        if ( event->inscript &&
             (( len = strlen( ((Transition) event->inscript)->name)) > orgnamelen ))
          orgnamelen = len;
      } /* for */
    /* translen ist die Feldgroesse fuer Eventname und Transitionsname hinter einander */
    if ( ( translen = ++namelen + orgnamelen ) < 8 )
      translen = 8;
    /* namelen ist max. Laenge des Bed.namen, orgnamelen die des Stellennamen */
    namelen = orgnamelen = 0;
    for ( cond = unfold_net->Places ; cond ; cond = cond->nextplace )
      {
        if ( ( len = strlen( cond->name ) ) > namelen )
          namelen = len;
        if ( ( len = strlen( ( (Place)cond->inscript )->name ) ) > orgnamelen )
          orgnamelen = len;
      } /* for */
    /* Zuerst die Anzahl Bedingungen und Ereignisse ausgeben */
    fprintf( ENTfile, "%u %u\n", unfold_net->NumPlaces,
                                   unfold_net->NumTransitions );
    /* Dann Ueberschrift und Einrueckung  */
    fprintf( ENTfile, "Inzidenzmatrix:\n%*s", namelen + orgnamelen + 3, "" );
    /* Danach alle Ereignisse und die dazu gehoerigen Transitionen ausgeben */
    for ( event = unfold_net->Transitions; event; event = event->nexttrans )
      {
        if (event->inscript)
          {
            fprintf( ENTfile, " %s %-*s", event->name,
                                          (int)(translen-strlen(event->name)-3),
                                          ((Transition) event->inscript)->name );
          }
        else
          {
            fprintf( ENTfile, " %s %-*s", event->name,
                                          (int)(translen-6),
                                          "tic" );
          }
      } /* for */
    fprintf( ENTfile, "\n" );
    translen -= 8;
    /* Nun die Bedingungen und zugehoerige Stellen ausgeben */
    /* Danach fuer jedes Ereignis e drucke (x,y)            */
    /* mit x==1 iff. e im Vorbereich und y==1 iff. e im Nachbereich */
    for ( cond = unfold_net->Places ; cond ; cond = cond->nextplace )
      {
        ce = cond->postset;
        ec = cond->preset;
        name = cond->name;
        fprintf( ENTfile, "%-*s %-*s |", namelen, name, orgnamelen,
                                          ( (Place) cond->inscript )->name );
        for ( event = unfold_net->Transitions ; event ;
                                                     event = event->nexttrans )
          {
            if ( ec && ( ec->trans == event ) )
              {
                fprintf( ENTfile, " (-%u,", !!ec->weight );
                ec = ec->nexttrans;
              } /* if */
            else
              {
                fprintf( ENTfile, " ( 0," );
              } /* else */
            if ( ce && ( ce->trans == event ) )
              {
                fprintf( ENTfile, "%u)%*s", ce->weight?(!!ce->weight):1, translen, "" );
                ce = ce->nexttrans;
              } /* if */
            else
              {
                fprintf( ENTfile, "0)%*s", translen, "" );
              } /* else */
          }  /* for event */
        fprintf( ENTfile, "\n" );
      } /* for cond */

    /* Ausgabe der Cutoff-Events */
    fprintf( ENTfile, "\nCutoff\n" );
    for ( list = GL_cutoffList ; list ; list = list->next )
      {
        fprintf( ENTfile, "%s;", ( (Event) list->item )->name );
      } /* for */
    fprintf( ENTfile, "\n\n" );

    if ( filename && *filename )
      {
        fclose( ENTfile );
      }
  } /* write_unfolding_ent() */


/*****************************************************************************/
/* Schreiben des entfalteten Netzes in eine ".unf"-Datei.                    */
/* ( unf = unfolding ASCII file format )                                     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void write_unfolding_UNF( char * filename, char * infilename, LMPNet unfold_net )

#else

void write_unfolding_UNF( filename, infilename, unfold_net )

  char * filename,
       * infilename;
  LMPNet unfold_net;

#endif

  {

    FILE                * UNFfile = stdout;
    register STEdge     st;
    register Condition  cond;
    register Event      event;
    register Cardinal32 count;
    register int        len,
                        namelen,
                        orgnamelen;
    register char       * name;
    char                UNFfilename[NAMELENGTH];
    register LIST       list,
                        listPred;
    static char         * PlaceLabel = "0exidn",
                        * dontknow = "start marking",
                        * asterisks
                           = "*********************************************\n";

    if ( filename && *filename )
      {
        if (    ( ( len = strlen( strcpy( UNFfilename, filename ) ) ) < 4 )
             || strcmp( UNFfilename + len - 4, ".unf" ) )
          {
            strcat( UNFfilename, ".unf" );
          } /* if */
        if ( ! ( UNFfile = fopen( UNFfilename, "w" ) ) )
          {
            PEPerror("write_unfolding_UNF: Cannot open file for writing." );
          } /* if */
        fputs( asterisks, UNFfile );
        fprintf( UNFfile, "* FILE: %-36s*\n", UNFfilename );
        fputs( asterisks, UNFfile );
      } /* if filename */
    /* namelen ist max. Laenge des Eventnamen, orgnamelen die des Transitionsnamen */
    namelen = orgnamelen = 0;
    for ( event = unfold_net->Transitions; event; event = event->nexttrans )
      {
        if ( ( len = strlen( event->name ) ) > namelen )
          namelen = len;
        if ( event->inscript && ( len = strlen( ( (Transition) event->inscript )->name ) )
                                      > orgnamelen )
          orgnamelen = len;
      } /* for */
    /* Mindestwerte setzen */
    if ( ++namelen < 5 )
      namelen = 5;
    if ( ++orgnamelen < 8 )
      orgnamelen = 8;
    /* Statistik ausgeben */
    fprintf( UNFfile, "\nUnfolding of the net described in file '%s'.\n",
                      infilename );
    fprintf( UNFfile, "=========\n\n" );
    fprintf( UNFfile, "%u condition%s.\n", unfold_net->NumPlaces,
                                    ( unfold_net->NumPlaces == 1 )?"":"s" );
    fprintf( UNFfile, "%u event%s.\n", unfold_net->NumTransitions,
                               ( unfold_net->NumTransitions == 1 )?"":"s" );
    fprintf( UNFfile, "%u component%s.\n\n", unfold_net->NumComponents,
                                ( unfold_net->NumComponents == 1 )?"":"s" );
    /* Numname und zugeh. Transitionen der Events ausgeben */
    fprintf( UNFfile, "Names and labellings of events:\n");
    fprintf( UNFfile, "numname  %-*s %-*s label (multiset or variable)\n",
                      namelen, "name", orgnamelen, "tr_name" );
    for ( event = unfold_net->Transitions; event; event = event->nexttrans )
      {
        fprintf( UNFfile, "%7d. ", event->numname );
        fprintf( UNFfile, "%-*s ", namelen, event->name );
        fprintf( UNFfile, "%-*s ", orgnamelen, event->inscript?
                                   ( (Transition) event->inscript )->name :
	                           "tic");
/* wenn einmal die Beschriftungen benoetigt werden:
        event->inscript?MSWriteMultiset( UNFfile, ( (Transition) event->inscript )->label ):"fprintf( UNFfile, "tic");
*/
        fprintf( UNFfile, "{}\n" );
      } /* for */
    /* Das gleiche fuer Bedingungen und Stellen */
    namelen = orgnamelen = 0;
    for ( cond = unfold_net->Places ; cond ; cond = cond->nextplace )
      {
        if ( ( len = strlen( cond->name ) ) > namelen )
          namelen = len;
        if ( ( len = strlen( ( (Place)cond->inscript )->name ) ) > orgnamelen )
          orgnamelen = len;
      } /* for */
    if ( ++namelen < 5 )
      namelen = 5;
    if ( ++orgnamelen < 8 )
      orgnamelen = 8;
    fprintf( UNFfile, "\nList of conditions and edges:\n" );
    fprintf( UNFfile,
             "preset     numname  label<comp>%-*s %-*s      postset\n",
             namelen, "name", orgnamelen, "pl_name" );
    /* Ggf. Vorbereichstransition ausgeben, dann Bedingungsname und Label  */
    /* danach Komponente der Stelle und die Namen von Bedingung und Stelle */
    /* Am Ende die Nachfolgertransition */
    for ( cond = unfold_net->Places ; cond ; cond = cond->nextplace )
      {
        if ( cond->preset )
          fprintf( UNFfile, "%7d -->", cond->preset->trans->numname );
        else
          fputs( "           ", UNFfile );
        fprintf( UNFfile, " %7d. (%c)", cond->numname,
                 PlaceLabel[ (int) cond->label ] );
        fprintf( UNFfile, " <%2u>  ", ( (Place)cond->inscript )->component );
        fprintf( UNFfile, "%-*s ", namelen, cond->name );
        fprintf( UNFfile, "%-*s ", orgnamelen,
                 ( (Place) cond->inscript )->name );
        if ( (st = cond->postset) )
          {
            fprintf( UNFfile, " --> %u", st->trans->numname );
            while ((st = st->nexttrans))
              fprintf( UNFfile, ",%u", st->trans->numname );
          } /* if */
        fputs( "\n", UNFfile );
      } /* for cond */

    /* Ausgabe der Cutoff-Events */
    count = GL_numCutoffs;
    fprintf( UNFfile, "\n%u cutoff event%s", count, (count == 1)?"":"s" );
    fputs( " (corresponding event):\n", UNFfile );
    if ( (list = GL_cutoffList) )
      {
        len = strlen( name = ( (Event) list->item )->name );
        fputs( name, UNFfile );
        len += strlen( name = ( (int) ( event = (Event)
            ( listPred = GL_cutoffCorrespondingList )->item ) == 1 ) ? dontknow
                                                           : event->name ) + 2;
        fprintf( UNFfile, "(%s)", name );
        while ( (list = list->next) )
          {
            putc( ',', UNFfile );
            if ( (int) ( len += strlen( name
                         = ( (Event) list->item )->name ) + 1 ) > PAGEWIDTH )
              {
                fputs( "\n  ", UNFfile );
                len = 0;
              } /* if */
            fputs( name, UNFfile );
            if ( (int) ( len += strlen( name = ( (int) ( event = (Event)
                      ( listPred = listPred->next )->item ) == 1 ) ? dontknow
                                            : event->name ) + 2 ) > PAGEWIDTH )
              {
                fputs( "\n  ", UNFfile );
                len = 0;
              } /* if */
            fprintf( UNFfile, "(%s)", name );
          } /* while */
      } /* if */
    fputs( ".\n\n", UNFfile );

    if ( filename && *filename )
      {
        putc( '\n', UNFfile );
        fputs( asterisks, UNFfile );
        fprintf( UNFfile, "* END OF FILE %-30s*\n", UNFfilename );
        fputs( asterisks, UNFfile );
        putc( '\n', UNFfile );
        fclose( UNFfile );
      } /* if */
    else
      {
        puts( "$$$$$$$$$$$$$$$$$$$ END OF UNFOLDING $$$$$$$$$$$$$$$$$$$" );
      } /* else */

  } /* write_unfolding_UNF() */


/************************************************************************/
/* Schreiben des entfalteten Netzes in eine ".mci"-Datei.               */
/* ( mci = model checker input file format )                            */
/************************************************************************/

#define WriteNum WriteNumMCI

#if defined(__STDC__) || defined(__cplusplus)

void write_unfolding_MCI( char * filename, LMPNet unfold_net, LMPNet net )

#else

void write_unfolding_MCI( filename, unfold_net, net )

  char * filename;
  LMPNet unfold_net;
  LMPNet net;

#endif

  {

    FILE                * MCIfile=stdout;
    register Condition  cond;
    register Event      event;
    register STEdge     ce;
    register char       * name;
    register int        len;
    char                MCIfilename[NAMELENGTH];
    register LIST       list,
                        corrlist;
    Cardinal32          num,
                        max;
    static Cardinal32   null = 0;

    if ( filename && *filename )
      {
        if (    ( ( len = strlen( strcpy( MCIfilename, filename ) ) ) < 4 )
             || strcmp( MCIfilename + len - 4, ".mci" ) )
          {
            strcat( MCIfilename, ".mci" );
          } /* if */
        if ( ! ( MCIfile = fopen( MCIfilename, "w" ) ) )
          {
            PEPerror("write_unfolding_MCI: Cannot open file for writing." );
          } /* if */

	/* Anzahl der Bedingungen */
	WriteNum( unfold_net->NumPlaces, MCIfile );
	/* Anzahl der Ereignisse */
	WriteNum( unfold_net->NumTransitions, MCIfile );
	/* Num. Name der Originalen Transitionen */
	for ( event=unfold_net->Transitions; event; event = event->nexttrans )
	  {
	    if (event->inscript)
	    {
	      WriteNum( ( (Transition) event->inscript )->numname, MCIfile );
	    }
	    else
	    {
	      WriteNum(null, MCIfile);
	    }
	  } /* for */
	/* Bedingungen: OriginalStelle, Vorbereichstransition, 
	   Nachbereichstransitionen */
	for ( cond = unfold_net->Places ; cond ; cond = cond->nextplace )
	  {
	    WriteNum( ( (Place) cond->inscript )->numname, MCIfile );
	    if ( cond->preset )
	      {
		WriteNum( cond->preset->trans->numname, MCIfile );
	      } /* if */
	    else
	      {
		WriteNum( null, MCIfile );
	      } /* else */
	    for ( ce = cond->postset ; ce ; ce = ce->nexttrans )
	      {
		WriteNum( ce->trans->numname, MCIfile );
	      } /* for */
	    WriteNum( null, MCIfile );
	  } /* for */
	/* Ausgabe der Cutoff-Events und der korrespondierenden Ereignisse */
	for ( list = GL_cutoffList, corrlist = GL_cutoffCorrespondingList; 
	      list; list = list->next, corrlist = corrlist->next )
	  {
	    WriteNum( ( (Event) list->item )->numname, MCIfile );
	    if ( corrlist->item == (Place) 1 )
	      {
		WriteNum( null, MCIfile );
	      } /* if */
	    else
	      {
		WriteNum( ( (Event) corrlist->item )->numname, MCIfile );
	      } /* else */
	  } /* for */
	WriteNum( null, MCIfile );
	
	/* Einfach-Null */
	WriteNum( null, MCIfile );
	/* Laenge der nachfolgenden Stellenliste */
	WriteNum( net->NumPlaces, MCIfile );
	/* Laenge der nachfolgenden Transitionsliste */
	WriteNum( net->NumTransitions, MCIfile );
	/* Laenge des laengsten Stellen-/Transitionsnamens */
	num = max_placename_length( net );
	max = max_transitionname_length( net );
	if ( max <= num )
	  max = num;
	WriteNum ( max, MCIfile );
	/* Liste der Stellennamen */
	for ( cond = net->Places ; cond ; cond = cond->nextplace )
	  {
#ifdef __NULLEN_AUFFUELLEN__
	    num = max;
	    for ( name = cond->name ; *name ; WriteChar( name++ ), num-- )
	      skip;
	    for ( WriteChar( name ) ; num-- ; WriteChar( name ) )
	      skip;
#else
	    for ( name = cond->name ; *name ; WriteChar( name++ ) )
	      skip;
	    WriteChar( name );
#endif
	  } /* for */
	/* Einfach-Null */
	WriteChar( "" );
	/* Liste der Transitionsnamen */
	for ( event = net->Transitions ; event ; event = event->nexttrans )
	  {
#ifdef __NULLEN_AUFFUELLEN__
	    num = max;
	    for ( name = event->name ; *name ; WriteChar( name++ ), num-- )
	      skip;
	    for ( WriteChar( name ) ; num-- ; WriteChar( name ) )
	      skip;
#else
	    for ( name = event->name ; *name ; WriteChar( name++ ), num-- )
	      skip;
	    WriteChar( name );
#endif
	  } /* for */
	/* Einfach-Null */
	WriteChar( "" );
	
	fclose( MCIfile );
      } /* if filename */

  } /* write_unfolding_MCI() */
