/* Alle Entfaltungsfunktionen. Es gibt insgesamt 5 verschiedene */

#include "unfold.h"
#include "hash.h"
#include "sub_unf.h"
#include "nets.h"
#include "queue.h"
#include "poss_ext.h"
#include "cutoff_mm.h"
#include "cutoff_ev.h"
#include "cutoff_erv.h"

/*****************************************************************************/
/* Some function pointer definitions.                                        */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LMPNet(* unfold)( LMPNet );
int   (* IsNotCutoffPoint)( LIST, Event );
int   (* ERV_IsNotCutoffPoint)( LIST, Event );
LIST  (* FinalState)( Event, LMPNet );

int   (* IsNotCutoffEvent)( LIST, Event );
LIST  (* Mark)( Event, LMPNet );

#else

LMPNet(* unfold)();
int   (* IsNotCutoffPoint)();
int   (* ERV_IsNotCutoffPoint)();
LIST  (* FinalState)();

int   (* IsNotCutoffEvent)();
LIST  (* Mark)();

#endif

/* configure some global data and other things for all unfolder */
/* in:   LMPNet net -- net to be unfolded                       */
/* out:  none                                                   */
/* side:                                                        */

static void init_data(LMPNet net)
{
  register Cardinal32 counter;
  register Transition tr;
  register TSEdge ts;
  register STEdge st;

  /* Transitionen durchnummerieren */
  for ( counter = 1, tr = net->Transitions ; tr ; tr = tr->nexttrans )
    {
      tr->numname = counter++;
    } /* for */
  
  /* Wenn flag gesetzt, Speicher reservieren */
  if ( GL_flags & USELESS_TRANS )
    {
      GL_UselessTransitions = (char *) MYmalloc( counter * sizeof( char ) );
    } /* if */
  
  /* Maximale Anzahl von Vor- und Nachbereichsstellen bestimmen */
  /* und aktuelle Zahlen speichern                              */
  GL_maxTpreset = GL_maxTpostset = 0;
  for ( tr = net->Transitions ; tr ; tr = tr->nexttrans )
    {
      for ( counter = 0, st = tr->preset; st; st = st->nextplace )
	counter++;
      tr->inscript = (char *) counter;
      if ( counter > GL_maxTpreset )
	GL_maxTpreset = counter;
      for ( counter = 0, ts = tr->postset; ts; ts = ts->nextplace )
	counter++;
      tr->prevtrans = (Transition) counter;
      if ( counter > GL_maxTpostset )
	GL_maxTpostset = counter;
    } /* for */
  
  /* Arrays bieten 8 Bit Platz */
  bit_setsize( ( ( net->NumPlaces + 1 ) >> 3 ) + 1);

  GL_configSize = 0;
  GL_numCFlag = 0;
  GL_numEFlag = (char *) 0;
  GL_numCutoffs = 0;
  GL_cutoffList = GL_cutoffCorrespondingList = NULL;
  init_queue();

  /* nur in EV */
  GL_EV_equalities = 0;
  GL_PE_queue = GL_PE_queue_last = NULL;
  
  CreateStacks();
  
  /* Fuer den Vorbereich von Events werden hier Stellenlisten abgelegt */
  ( *( GL_listArray = (LIST *) MYmalloc( ( ( GL_maxTpreset + 1 ) << 1 )
				       * sizeof( LIST ) ) ) ) = NULL;
  GL_listArray4 = ( GL_listArray2 = ( GL_listArray1 = GL_listArray+1 )+1 )+2;
  GL_LiConflictsFunction = LiConflictsBit;

  hash_alloc(net->NumPlaces);
} /* init_data() */


/* Some cleaning up operations. To be called after unfolding. */
/* in:   LMPNet net -- net to be unfolded                     */
/*       LMPNet unfold_net -- the unfolding                   */
/* out:  none                                                 */
/* side: */

static void cleanup_data(LMPNet net, LMPNet unfold_net)
{
  register Place pl;
  register Event event;
  char  name[ NAMELENGTH ];

  update_clock();

  FreeStacks();
  Free( GL_listArray );

  delete_queue();

  /* Liste zugehoeriger Bedingungen loeschen */
  for ( pl = net->Places ; pl ; pl = pl->nextplace )
    {
      list_delete_mem( (LIST *) &( pl->inscript ) );
    } /* for */
  /* Bedingungen passend benennen */
  for ( pl = unfold_net->Places ; pl ; pl = pl->nextplace )
      {
        if ( ( GL_maxTpreset > 2 ) && pl->name
             && ( GL_changeAlgo > unfold_net->NumTransitions )
             && ( ! pl->nextplace || ( pl->name != pl->nextplace->name ) ) )
          {
            Free( pl->name );
          } /* if */
        sprintf( name, "B%u", pl->numname );
        pl->name = MYstrdup( name );
      } /* for */
    /* Ebenso die Ereignisse */
    for ( event = unfold_net->Transitions; event; event = event->nexttrans )
      {
        sprintf( name, "E%u", event->numname );
        event->name = MYstrdup( name );
      } /* for */

    list_reverse( &GL_cutoffList );
    list_reverse( &GL_cutoffCorrespondingList );

    update_clock();
    /* End of cleaning up */
}

/* prints information on current prefix during unfolding      */
/* in:   LMPNet unfold_net -- current prefix                  */
/* out:  none                                                 */
/* side: GL_flags -- to check if print each line with newline */

static void print_progress(LMPNet unfold_net)
{
  register int counter; /* Gesamtzahl auszugebender Zeichen */
  
  counter=printf( "|B|=%6u, |E|=%6u, Cutoffs:%6u ",
	  unfold_net->NumPlaces, unfold_net->NumTransitions,
	  GL_numCutoffs);
  /* Entweder eine Ausgabe pro Zeile, oder jede Ausgabe in der */
  /* gleichen Zeile, dann ist die letzte zu loeschen           */
  if ( GL_flags & PROGRESS_NEWLINE )
    {
      putchar( '\n' );
    } /* if */
  else
    {
      while ((counter--))
	putchar( '\b' );
    } /* else */
}

/*****************************************************************************/
/* The unfolding procedure according to McMillan                             */
/* useable with time (Time expansion)                                        */
/* in:   LMPNET net -- net to be unfolded                                    */
/* out:  LMPNet     -- finite prefix                                         */
/* side: */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LMPNet M_unfold( LMPNet net )

#else

LMPNet M_unfold( net )

  LMPNet net;

#endif

  {

    register LMPNet     unfold_net;
    register Place      pl;
    register TSEdge     ts;
    register STEdge     st;
    register Condition  cond;
    register Event      event;
             LIST       list,
                        startmarking;
    register Cardinal32 counter,
                        numname,
                        marks;
    register BIT_ARRAY  conflArrayPtr=NULL;

    /* Initialization */
    init_data(net);
    /* GL_unfold_net kann weg, wenn alle Files angepasst */
    GL_unfold_net = unfold_net = empty_net();

    /* Startmarkierung des Originalnetzes betrachten */
    startmarking = NULL;
    counter = numname = 0;
    /* Bitarrays werden analog wie der Rest behandelt */
    if ( GL_flags & USE_BITARRAYS )
      {
        conflArrayPtr = bit_alloc();
        for ( pl = net->Places ; pl ; pl = pl->nextplace )
          {
            pl->inscript = NULL;
            pl->numname = ++numname;
            /* Wenn Stelle markiert, dann einfuegen und Bits setzen */
            /* (Markierung als Bitarray repraesentieren)            */
            if ( pl->marks )
              {
                list_insert( &startmarking, pl, 0 );
                Set_Bit( conflArrayPtr, pl->numname );
              } /* if pl->marks */
          } /* for pl */
        /* Startmarkierung speichern */
        GL_BitStartMarking = bit_copy( conflArrayPtr );
        /* Hashindex berechnen */
        hash_index_bit( conflArrayPtr );
        /* In conflArrayPtr steht aktuell die Startmarkierung */
        /* Diese wird entsprechend von *_ARRAY behandelt      */
        IsNotCutoffPoint( (LIST) conflArrayPtr, (Event) 1 );
      } /* if USE_BITARRAYS */
    else
      {
        /* Alle vorhandenen Stellen betrachten */
        for ( pl = net->Places ; pl ; pl = pl->nextplace )
          {
            /* inscript tr"agt eigentlich die Koordinaten der Stellen, */
            /* in diesem Fall aber die zugeh. Bedingungen              */
            pl->inscript = NULL;
            pl->numname = ++numname;
            /* Wenn Stelle markiert, dann einfuegen */
            if (( marks=pl->marks ))
              {
		/* Be aware that the token age is wrong! */
		/* This is done to stay with list even for unsafe nets */
		while (marks--)
		  list_insert( &startmarking, pl, marks );
                counter++;
              }
          } /* for */
        /* Hashberechnung */
        hash_index_list( startmarking );
        /* Startmarkierung im Hash eintragen , auf jeden Fall kein cut-off */
        /* Event 1 ist wohl das virtuelle Ereignis ?!                      */
        IsNotCutoffPoint( startmarking, (Event) 1 );
      } /* else !USE_BITARRAYS */
    /* Je nach LiConflictsFunction muessen die Konflikte */
    /* direkt eingetragen werden */
    /* Bei USE_BITARRAYS zeigt conflArrayPtr auf ein Byte, sonst auf */
    /* (Anzahl Stellen in Startmarkierung+1)/8+1 Byte, mit Nullen      */
    /* Wird ggf. in die Startbedingungen eingetragen                   */
    if ( GL_changeAlgo > unfold_net->NumTransitions )
      {
        counter++; counter >>= 3;
        conflArrayPtr = mymalloc_bitarray(++counter);
        for ( conflArrayPtr += counter ; counter-- ; *--conflArrayPtr = '\0' )
          skip;
      } /* if */


    /* ################################################### */
    /* Startmarkierung in die Entfaltung uebertragen.      */
    /* Gleichzeitig Nachbereich einfuegen und PE(Fin)      */
    /* berechnen. Fuer Time expansion wird Startmarkierung */
    /* durch aequivalenten Cut ersetzt (fuer InsertTic)    */
    for ( list = startmarking, startmarking=NULL ; list ; list = list->next )
      {
        cond = insert_condition( unfold_net, pl = list->item, conflArrayPtr );
	/* Stelle durch Bedingung ersetzen */
        list_insert(&startmarking,cond,0);
        /* Transitionen in Entfaltung und Ereignisqueue einfuegen */
        if ( pl->postset )
          {
            GenEvents( cond, unfold_net );
          } /* if */
      } /* for list */
    /* Neu bei Zeitnetzen: Tics einfuegen */
    if (GL_flags&WITH_TIME)
      InsertTic( unfold_net, startmarking );
    else
      list_delete(&startmarking);

    /* ########################################################## */
    /* Main loop of the unfolding procedure. (while pe!=empty do) */
        /* Waehle erstes Event (kleinste Konfigurationsgroesse) */
    while ( (event = pop_queue()) )
      {
        /* Zeit checken */
	update_clock();

	/* FinalState berechnet die Markierung von [e] und setzt das     */
	/* GL_conflictArray und	in GL_stackC2ptr alle besuchten          */
	/* Bedingungen aufzaehlen.  */
        /* IsNotCutoffPoint fuegt das Event in die Entfaltung ein und    */
	/* testet auf Cutoff-Eigenschaft.                                */
        /* Falls event kein cutoff ist, auch den Nachbereich eingefuegen.*/
        if ( IsNotCutoffPoint(FinalState( event, unfold_net ), event) )
          {
	    if (event->inscript)
	      {
            /* Falls die Transition einen Nachbereich hat (eigentlich immer) */
            /* werden die Stellen eingefuegt, vorher moegliche Konflikte     */
	    /* berechnen */
            if ( (ts = ( (Transition) event->inscript )->postset) )
              {
                conflArrayPtr=Calculate_Conflicts( GL_conflictArray, 
						   unfold_net, ts->trans ); 
		  /* Ausgehend von GL_stackC2ptr */

                /* Neue Bedingungen und Kanten einfuegen, */
                /* ggf. neue Events in die Queue stellen  */
                while (ts) 
                  {
                    cond = insert_condition( unfold_net, ts->place, 
					     conflArrayPtr );
                    insert_ec_edge( event, cond );

                    /* pe neu berechnen */
                    if ( ts->place->postset )
                      {
                        GenEvents( cond, unfold_net );
                      } /* if */
                    ts = ts->nextplace;
                  } /* while ts */
              } /* if ts->postset */
	      } /* kein Tic */
	    else
	      {
	    /* Groesse des benutzten GL_conflictArray berechnen */
	    for (counter = unfold_net->NumPlaces + 2, st=event->preset;
		 st; st=st->nextplace)
	      counter++;
	    /* counter ist (Anzahl Conditions + Anzahl Kanten im Nachbereich +2)/8 */
	    counter = counter >> 3;
	    numname = ++counter;
	    /* GL_conflictArray loeschen */
	    for ( conflArrayPtr = GL_conflictArray ; counter-- ;
		  *( conflArrayPtr++ ) = '\0' )
	      skip;

	    /* Tics are in conflict with everything before */
	    for(counter=unfold_net->NumPlaces; counter; --counter)
	    {
	      Set_Bit(GL_conflictArray, counter);
	    }

      { /* Hilfspointer zum Durchlaufen des Speicher */
        register BIT_ARRAY  conflArrayPtr2;
        /* This is the current size of conflictArray */
 
        /* conflArray wird kopiert und zurueckgegeben */
        conflArrayPtr = mymalloc_bitarray(numname) + numname;
        conflArrayPtr2 = GL_conflictArray + numname;
        while ((numname--))
          *--conflArrayPtr = *--conflArrayPtr2;
      }

	    /* Nachbereich einfuegen */
	    for (st=event->preset; st; st=st->nextplace)
	      {
		STEdge tempst;
		cond = insert_condition( unfold_net, 
					 (Place)st->place->inscript, 
					 conflArrayPtr );
		/* Dies war eine Tic-Transition, also Uhr weiter ruecken */
/* TODO: Stop aging if not needed */
		if ((tempst=(((Place)st->place->inscript)->postset)))
		  {  
//		    while (tempst&&((!tempst->trans->time_label)||(st->place->tokenage>=tempst->trans->time_label->high)||(st->place->tokenage>=temps->trans->time_label_low&&is_inf_lft(tempst->trans))))
		    while (tempst &&  (!tempst->trans->time_label ||
		    (st->place->tokenage>=tempst->trans->time_label->high)||
		    (st->place->tokenage>=tempst->trans->time_label->low&&is_inf_lft(tempst->trans))))
		      tempst=tempst->nexttrans;
		  }
		if (tempst)
		  cond->tokenage=st->place->tokenage+1;
		else
		  cond->tokenage=st->place->tokenage;
		insert_ec_edge( event, cond );
		if (((Place)st->place->inscript)->postset)
		  GenEvents( cond, unfold_net );
	      } /* for st */
  } /* else Tic */

            /* Neu bei Zeitnetzen: Tics einfuegen */
	    if (GL_flags&WITH_TIME)
	      InsertTic( unfold_net, CalculateCut(event, unfold_net));

            if ( GL_flags & PRINT_PROGRESS )
	      print_progress(unfold_net);
          } /* if IsNotCutoffPoint */
#ifdef LOG_THE_CUTOFFS
        else /* pe:=pe\e ist schon passiert */
          printf("CUTOFF found\n");
#endif /* LOG_THE_CUTOFFS */
      } /* while pe != leer */

    /* Am Ende der while-Schleife muessen alle Nachbereiche der cut-offs */
    /* ergaenzt werden, da nur die Events eingefuegt waren. Zum          */
    /* Praefix gehoeren aber auch die Nachbereiche der Events            */
    insert_postset_list(GL_cutoffList, unfold_net);

    /* Wenn immer die gleiche Zeile benutzt wurde wird hier endlich */
    /*  die Zeile gewechselt */
    if ( GL_flags & PRINT_PROGRESS )
      putchar( '\n' );

    cleanup_data(net, unfold_net);

    return unfold_net;

  } /* M_unfold() */


/*****************************************************************************/
/* The improved unfolding procedure.                                         */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LMPNet EV_unfold( LMPNet net )

#else

LMPNet EV_unfold( net )

  LMPNet net;

#endif

  {

    register LMPNet     unfold_net;
    register Place      pl;
    register Transition tr;
    register TSEdge     ts;
    register Condition  cond;
    register Event      event;
    register PE_QUEUE   qudel;
             LIST       list,
                        startmarking;
    register Cardinal32 counter,
                        numname;
    register BIT_ARRAY  conflArrayPtr=NULL;

    /* Initialization */
    init_data(net);
    GL_unfold_net = unfold_net = empty_net();

    /* neu EV */
    GL_CArray = (Condition *) MYmalloc( GL_maxTpreset * sizeof( Condition ) );
    if ( ! GL_ParikhInAdvance )
      {
/* QQQ */
        GL_ParikhArray1 = ( (PARIKH_ARRAY) MYmalloc(
             ( net->NumTransitions + 2 ) * sizeof( Parikh_Array_Entry ) ) );
        GL_ParikhArray2 = ( (PARIKH_ARRAY) MYmalloc(
             ( net->NumTransitions + 2 ) * sizeof( Parikh_Array_Entry ) ) );
      } /* if */

    /* Look at the initial marking of the original net */
    startmarking = NULL;
    counter = numname = 0;
    if ( GL_flags & USE_BITARRAYS )
      {
        conflArrayPtr = bit_alloc();
        for ( pl = net->Places ; pl ; pl = pl->nextplace )
          {
            pl->inscript = NULL;
            pl->numname = ++numname;
            if ( pl->marks )
              {
                list_insert( &startmarking, pl, 0);
                Set_Bit( conflArrayPtr, pl->numname );
              } /* if */
          } /* for */
        GL_BitStartMarking = bit_copy( conflArrayPtr );
        hash_index_bit( conflArrayPtr );
        /* neu EV */
        GL_configSize = 1;
        IsNotCutoffEvent( (LIST) conflArrayPtr, (Event) 1 );
      } /* if */
    else
      {
        for ( pl = net->Places ; pl ; pl = pl->nextplace )
          {
            pl->inscript = NULL;
            pl->numname = ++numname;
            if ( pl->marks )
              {
                list_insert( &startmarking, pl, 0);
                counter++;
              }
          } /* for */
        hash_index_list( startmarking );
        /* neu EV */
        GL_configSize = 1;
        IsNotCutoffEvent( startmarking, (Event) 1 );
      } /* else */
    if ( GL_changeAlgo > unfold_net->NumTransitions )
      {
        counter++; counter >>= 3;
        conflArrayPtr = mymalloc_bitarray(++counter);
        for ( conflArrayPtr += counter ; counter-- ; *--conflArrayPtr = '\0' )
          skip;
      } /* if */

    /* Transfer the initial marking in the unfolding */
    for ( list = startmarking ; list ; list = list->next )
      {
        insert_condition( unfold_net, list->item, conflArrayPtr );
      } /* for */

    /* neu in EV statt GenEvents */
    /* Calculate the possible extensions for the initial conditions */
    for ( cond = unfold_net->Places ; cond ; cond = cond->nextplace )
      {
        if ( ( (Place) cond->inscript )->postset )
          {
            Get_PE_start( cond );
          } /* if */
      } /* for */

    /* Main loop of the unfolding procedure. */
    /* ab hier anders in EV */
    while ( (qudel = GL_PE_queue) )
      {
	update_clock();

        if ( (GL_PE_queue = qudel->next) )
          {
            GL_PE_queue->prev = NULL;
          } /* if */
        else
          {
            GL_PE_queue_last = NULL;
          } /* else */
        tr = (Transition) *( qudel->conds );
        counter = ( unfold_net->NumPlaces + (Cardinal32)
                                                      tr->prevtrans + 2 ) >> 3;
        for ( conflArrayPtr = GL_conflictArray ; counter-- ;
                                                  *( conflArrayPtr++ ) = '\0' )
          skip;
        event = Insert_Event_With_CE_Edges( unfold_net, tr, qudel->conds );
        if ( IsNotCutoffEvent( Mark( event, unfold_net ), event ) )
          {
            if ( (ts = ( (Transition) event->inscript )->postset) )
              {
                conflArrayPtr=Calculate_Conflicts( GL_conflictArray, 
						  unfold_net, tr );

#ifdef _EV_DISPLAY_IT_
printf("INSERT CONDITIONS IN THE POSTSET:\n");
#endif /* _EV_DISPLAY_IT_ */
                while ( ts )
                  {
                    cond = insert_condition( unfold_net, ts->place, 
					     conflArrayPtr );
                    insert_ec_edge( event, cond );
#ifdef _EV_DISPLAY_IT_
printf("CONDITION C%u (%s) INSERTED\n",cond->numname,
                                                ((Place)cond->inscript)->name);
#endif /* _EV_DISPLAY_IT_ */
                    if ( ts->place->postset )
                      {
#ifdef _EV_DISPLAY_IT_
printf("GET_PE\n");
#endif /* _EV_DISPLAY_IT_ */
                        Get_PE( cond );
                      } /* if */
		    ts = ts->nextplace;
                  } /* while */
              } /* if */

            if ( GL_flags & PRINT_PROGRESS )
	      print_progress(unfold_net);
          } /* if */
#ifdef LOG_THE_CUTOFFS
	else printf("CUTOFF found\n");
#endif /* LOG_THE_CUTOFFS */

#ifdef _EV_DISPLAY_IT_
Display_PE_Queue();
#endif /* _EV_DISPLAY_IT_ */
        Free( qudel->conds );
        Free( qudel );

      } /* while */

    /* Append the post-sets of the cutoff events in the unfolding. */
    insert_postset_list(GL_cutoffList, unfold_net);
 
    if ( GL_flags & PRINT_PROGRESS )
      putchar( '\n' );

    if ( ! GL_ParikhInAdvance )
      {
/* QQQ */
        Free( GL_ParikhArray1 );
        Free( GL_ParikhArray2 );
      } /* if */
    /* Some cleaning up operations. */
    cleanup_data(net, unfold_net);

    return unfold_net;

  } /* EV_unfold() */


/*###########################################################################*/
/* EV2_unfold: Entfalten nach dem verbesserten Algorithmus.                  */
/*             Einsortieren in die Schlange erfolgt nur nach Groesse der Konf*/
/*             Einfuegen der Ereignisse erst nach Entfernen aus der Schlange.*/
/*###########################################################################*/

/*****************************************************************************/
/* The further improved unfolding procedure.                                 */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LMPNet EV2_unfold( LMPNet net )

#else

LMPNet EV2_unfold( net )

  LMPNet net;

#endif

  {

    register LMPNet     unfold_net;
    register Place      pl;
    register Transition tr;
    register TSEdge     ts;
    register Condition  cond;
    register Event      event;
    register PE_QUEUE   qudel;
             LIST       list,
                        startmarking;
    register Cardinal32 counter,
                        numname;
    register BIT_ARRAY  conflArrayPtr=NULL;
/* neu in Ver. 2 */
    register Condition  * condptr;
    bool                cont;
/* Ver. 2 */

    /* Initialization */
    init_data(net);
    GL_unfold_net = unfold_net = empty_net();

    GL_CArray = (Condition *) MYmalloc( GL_maxTpreset * sizeof( Condition ) );
    if ( ! GL_ParikhInAdvance )
      {
/* QQQ */
        GL_ParikhArray1 = ( (PARIKH_ARRAY) MYmalloc(
             ( net->NumTransitions + 2 ) * sizeof( Parikh_Array_Entry ) ) );
        GL_ParikhArray2 = ( (PARIKH_ARRAY) MYmalloc(
             ( net->NumTransitions + 2 ) * sizeof( Parikh_Array_Entry ) ) );
      } /* if */

    /* Look at the initial marking of the original net */
    startmarking = NULL;
    counter = numname = 0;
    if ( GL_flags & USE_BITARRAYS )
      {
        conflArrayPtr = bit_alloc();
        for ( pl = net->Places ; pl ; pl = pl->nextplace )
          {
            pl->inscript = NULL;
            pl->numname = ++numname;
            if ( pl->marks )
              {
                list_insert( &startmarking, pl, 0);
                Set_Bit( conflArrayPtr, pl->numname );
              } /* if */
          } /* for */
        GL_BitStartMarking = bit_copy( conflArrayPtr );
        hash_index_bit( conflArrayPtr );
        GL_configSize = 1;
        IsNotCutoffEvent( (LIST) conflArrayPtr, (Event) 1 );
      } /* if */
    else
      {
        for ( pl = net->Places ; pl ; pl = pl->nextplace )
          {
            pl->inscript = NULL;
            pl->numname = ++numname;
            if ( pl->marks )
              {
                list_insert( &startmarking, pl, 0);
                counter++;
              }
          } /* for */
        hash_index_list( startmarking );
        GL_configSize = 1;
        IsNotCutoffEvent( startmarking, (Event) 1 );
      } /* else */
    if ( GL_changeAlgo > unfold_net->NumTransitions )
      {
        counter++; counter >>= 3;
        conflArrayPtr = mymalloc_bitarray(++counter);
        for ( conflArrayPtr += counter ; counter-- ; *--conflArrayPtr = '\0' )
          skip;
      } /* if */

    /* Transfer the initial marking in the unfolding */
    for ( list = startmarking ; list ; list = list->next )
      {
        insert_condition( unfold_net, list->item, conflArrayPtr );
      } /* for */

    /* Calculate the possible extensions for the initial conditions */
    for ( cond = unfold_net->Places ; cond ; cond = cond->nextplace )
      {
        if ( ( (Place) cond->inscript )->postset )
          {
            Get_PE_start( cond );
          } /* if */
      } /* for */

    /* Main loop of the unfolding procedure. */
    while ( (qudel = GL_PE_queue) )
      {
	update_clock();
        if ( (GL_PE_queue = qudel->next) )
          {
            GL_PE_queue->prev = NULL;
          } /* if */
        else
          {
            GL_PE_queue_last = NULL;
          } /* else */
/*@+*/
        tr = (Transition) *( condptr = (Condition *) qudel->conds );
        cont = True;
        for ( counter = (Cardinal32) tr->prevtrans ; counter-- && cont ; skip )
          {
            if ( ( *++condptr )->label == 4 )
              cont = False;
          } /* for */
        if ( ! cont )
          {
            Free( qudel->conds );
            Free( qudel );
            continue;
          } /* if */
/*@+*/
        counter = ( unfold_net->NumPlaces + (Cardinal32)
                                                      tr->prevtrans + 2 ) >> 3;
        for ( conflArrayPtr = GL_conflictArray ; counter-- ;
                                                  *( conflArrayPtr++ ) = '\0' )
          skip;
        event = Insert_Event_With_CE_Edges( unfold_net, tr, qudel->conds );
        if ( IsNotCutoffEvent( Mark( event, unfold_net ), event ) )
          {
            if ( (ts = ( (Transition) event->inscript )->postset) )
              {
                conflArrayPtr=Calculate_Conflicts( GL_conflictArray, 
						  unfold_net, tr );

#ifdef _EV_DISPLAY_IT_
printf("INSERT CONDITIONS IN THE POSTSET:\n");
#endif /* _EV_DISPLAY_IT_ */
                for ( skip ; ts ; ts = ts->nextplace )
                  {
		    cond = insert_condition( unfold_net, ts->place, 
					     conflArrayPtr );
                    insert_ec_edge( event, cond);
#ifdef _EV_DISPLAY_IT_
printf("CONDITION C%u (%s) INSERTED\n",cond->numname,
                                                ((Place)cond->inscript)->name);
#endif /* _EV_DISPLAY_IT_ */
                    if ( ts->place->postset )
                      {
#ifdef _EV_DISPLAY_IT_
printf("GET_PE\n");
#endif /* _EV_DISPLAY_IT_ */
                        Get_PE( cond );
                      } /* if */
                  } /* for */
              } /* if */

            if ( GL_flags & PRINT_PROGRESS )
	      print_progress(unfold_net);
          } /* if */
#ifdef LOG_THE_CUTOFFS
else printf("CUTOFF found\n");
#endif /* LOG_THE_CUTOFFS */

#ifdef _EV_DISPLAY_IT_
Display_PE_Queue();
#endif /* _EV_DISPLAY_IT_ */
        Free( qudel->conds );
        Free( qudel );

      } /* while */

    /* Append the post-sets of the cutoff events in the unfolding. */
    insert_postset_list(GL_cutoffList, unfold_net);

    if ( GL_flags & PRINT_PROGRESS )
      putchar( '\n' );

    /* Some cleaning up operations. */
    if ( ! GL_ParikhInAdvance )
      {
/* QQQ */
        Free( GL_ParikhArray1 );
        Free( GL_ParikhArray2 );
      } /* if */
    cleanup_data(net, unfold_net);

    return unfold_net;
  } /* EV2_unfold() */


/*###########################################################################*/

/*****************************************************************************/
/* The further improved unfolding procedure.                                 */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LMPNet ERV_unfold( LMPNet net )

#else

LMPNet ERV_unfold( net )

  LMPNet net;

#endif

  {

    register LMPNet     unfold_net;
    register Place      pl;
    register TSEdge     ts;
    register Condition  cond;
    register Event      event;
             LIST       list,
                        startmarking;
    register Cardinal32 counter,
                        numname;
    register BIT_ARRAY  conflArrayPtr=NULL;

    /* Initialization */
    init_data(net);
    GL_unfold_net = unfold_net = empty_net();

    GL_CArray = (Condition *) MYmalloc( GL_maxTpreset * sizeof( Condition ) );
/* QQQ */
    GL_ParikhArray1 = ( (PARIKH_ARRAY) MYmalloc(
             ( net->NumTransitions + 2 ) * sizeof( Parikh_Array_Entry ) ) );
    GL_ParikhArray2 = ( (PARIKH_ARRAY) MYmalloc(
             ( net->NumTransitions + 2 ) * sizeof( Parikh_Array_Entry ) ) );

    /* Look at the initial marking of the original net */
    startmarking = NULL;
    counter = numname = 0;
    if ( GL_flags & USE_BITARRAYS )
      {
        conflArrayPtr = bit_alloc();
        for ( pl = net->Places ; pl ; pl = pl->nextplace )
          {
            pl->inscript = NULL;
            pl->numname = ++numname;
            if ( pl->marks )
              {
                list_insert( &startmarking, pl, 0);
                Set_Bit( conflArrayPtr, pl->numname );
              } /* if */
          } /* for */
        GL_BitStartMarking = bit_copy( conflArrayPtr );
        hash_index_bit( conflArrayPtr );
        ERV_IsNotCutoffPoint( (LIST) conflArrayPtr, (Event) 1 );
      } /* if */
    else
      {
        for ( pl = net->Places ; pl ; pl = pl->nextplace )
          {
            pl->inscript = NULL;
            pl->numname = ++numname;
            if ( pl->marks )
              {
                list_insert( &startmarking, pl, 0);
                counter++;
              }
          } /* for */
        hash_index_list( startmarking );
        ERV_IsNotCutoffPoint( startmarking, (Event) 1 );
      } /* else */
    if ( GL_changeAlgo > unfold_net->NumTransitions )
      {
        counter++; counter >>= 3;
        conflArrayPtr = mymalloc_bitarray(++counter);
        for ( conflArrayPtr += counter ; counter-- ; *--conflArrayPtr = '\0' )
          skip;
      } /* if */

    /* Transfer the initial marking in the unfolding */
    for ( list = startmarking ; list ; list = list->next )
      {
        cond = insert_condition( unfold_net, pl = list->item, conflArrayPtr );
        if ( pl->postset )
          {
            GenEvents( cond, unfold_net );
          } /* if */
      } /* for */

    /* Main loop of the unfolding procedure. */
        /* Waehle erstes Event (kleinste Konfigurationsgroesse) */
    while ( (event = pop_queue()) )
      {
	update_clock();


if ( ! event->numname ) continue;
        if ( ERV_IsNotCutoffPoint( FinalState( event, unfold_net ), event ) )
          {
            if ( (ts = ( (Transition) event->inscript )->postset) )
              {
                conflArrayPtr=Calculate_Conflicts( GL_conflictArray, 
						  unfold_net, ts->trans );

                while (ts)
                  {
		    cond = insert_condition( unfold_net, ts->place, 
					     conflArrayPtr );
                    insert_ec_edge( event, cond );
                    if ( ts->place->postset )
                      {
                        GenEvents( cond, unfold_net );
                      } /* if */
		    ts = ts->nextplace;
                  } /* while ts */
              } /* if */

            if ( GL_flags & PRINT_PROGRESS )
	      print_progress(unfold_net);
          } /* if */
      } /* while */

    /* Append the post-sets of the cutoff events in the unfolding. */
    insert_postset_list(GL_cutoffList, unfold_net);

/*
    for ( cond = unfold_net->Places ; cond ; skip )
      {
        cond = cond->numname ? cond->nextplace
                             : delete_place_direct( unfold_net, cond );
printf("%u\n", cond->numname);
      }
*/

    if ( GL_flags & PRINT_PROGRESS )
      putchar( '\n' );

    /* Some cleaning up operations. */
/* QQQ */
    Free( GL_ParikhArray1 );
    Free( GL_ParikhArray2 );
    cleanup_data(net, unfold_net);

    return unfold_net;
  } /* ERV_unfold() */


/*###########################################################################*/

/*****************************************************************************/
/* The further improved unfolding procedure.                                 */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LMPNet NERV_unfold( LMPNet net )

#else

LMPNet NERV_unfold( net )

  LMPNet net;

#endif

  {

    register LMPNet     unfold_net;
    register Place      pl;
    register TSEdge     ts;
    register Condition  cond;
    register Event      event;
             LIST       list,
                        startmarking;
    register Cardinal32 counter,
                        numname;
    register BIT_ARRAY  conflArrayPtr=NULL;

    /* Initialization */
    init_data(net);
    GL_unfold_net = unfold_net = empty_net();

    GL_CArray = (Condition *) MYmalloc( GL_maxTpreset * sizeof( Condition ) );
/* QQQ */
    GL_ParikhArray1 = ( (PARIKH_ARRAY) MYmalloc(
             ( net->NumTransitions + 2 ) * sizeof( Parikh_Array_Entry ) ) );
    GL_ParikhArray2 = ( (PARIKH_ARRAY) MYmalloc(
             ( net->NumTransitions + 2 ) * sizeof( Parikh_Array_Entry ) ) );

    /* Look at the initial marking of the original net */
    startmarking = NULL;
    counter = numname = 0;
    if ( GL_flags & USE_BITARRAYS )
      {
        conflArrayPtr = bit_alloc();
        for ( pl = net->Places ; pl ; pl = pl->nextplace )
          {
            pl->inscript = NULL;
            pl->numname = ++numname;
            if ( pl->marks )
              {
                list_insert( &startmarking, pl, 0);
                Set_Bit( conflArrayPtr, pl->numname );
              } /* if */
          } /* for */
        GL_BitStartMarking = bit_copy( conflArrayPtr );
        hash_index_bit( conflArrayPtr );
        ERV_IsNotCutoffPoint( (LIST) conflArrayPtr, (Event) 1 );
      } /* if */
    else
      {
        for ( pl = net->Places ; pl ; pl = pl->nextplace )
          {
            pl->inscript = NULL;
            pl->numname = ++numname;
            if ( pl->marks )
              {
                list_insert( &startmarking, pl, 0);
                counter++;
              }
          } /* for */
        hash_index_list( startmarking );
        ERV_IsNotCutoffPoint( startmarking, (Event) 1 );
      } /* else */
    if ( GL_changeAlgo > unfold_net->NumTransitions )
      {
        counter++; counter >>= 3;
        conflArrayPtr = mymalloc_bitarray(++counter);
        for ( conflArrayPtr += counter ; counter-- ; *--conflArrayPtr = '\0' )
          skip;
      } /* if */

    /* Transfer the initial marking in the unfolding */
    for ( list = startmarking ; list ; list = list->next )
      {
        cond = insert_condition( unfold_net, pl = list->item, conflArrayPtr );
        if ( pl->postset )
          {
            GenEvents( cond, unfold_net );
          } /* if */
      } /* for */

    /* Main loop of the unfolding procedure. */
        /* Waehle erstes Event (kleinste Konfigurationsgroesse) */
    while ( (event = pop_queue()) )
      {
	update_clock();

        if ( ERV_IsNotCutoffPoint( FinalState( event, unfold_net ), event ) )
          {
            if ( (ts = ( (Transition) event->inscript )->postset) )
              {
                conflArrayPtr=Calculate_Conflicts( GL_conflictArray, 
						  unfold_net, ts->trans ); 

                while (ts)
                  {
		    cond = insert_condition( unfold_net, ts->place, 
					     conflArrayPtr );
                    insert_ec_edge( event, cond );
                    if ( ts->place->postset )
                      {
                        GenEvents( cond, unfold_net );
                      } /* if */
		    ts = ts->nextplace;
                  } /* while ts */
              } /* if */

            if ( GL_flags & PRINT_PROGRESS )
	      print_progress(unfold_net);
          } /* if */
      } /* while */

    /* Append the post-sets of the cutoff events in the unfolding. */
    insert_postset_list(GL_cutoffList, unfold_net);

    if ( GL_flags & PRINT_PROGRESS )
      putchar( '\n' );

    /* Some cleaning up operations. */
/* QQQ */
    Free( GL_ParikhArray1 );
    Free( GL_ParikhArray2 );
    cleanup_data(net, unfold_net);

    return unfold_net;
  } /* NERV_unfold() */


/*###########################################################################*/
/*#           END OF ROUTINES FOR THE IMPROVED UNFOLDING ALGORITHM          #*/
/*###########################################################################*/

