/*************************************************************************/
/* bpn.y                                                                 */
/* YACC-source file: parser for b(pn)2                                   */
/* 1993 by Burkhard Graves                                               */
/* 1994 by Lars Jenner                                                   */
/* last changes January 1995 by Stephan Merkel                           */
/*************************************************************************/
/* Note: Rules containing square brackets ('[' and ']') are no longer    */
/* supported and provided only for historical continuity with older      */
/* versions.                                                             */
/*************************************************************************/

%{
#include <string.h>
#include "range.h"
#include "symtab.h"

extern int yylex(void);
#define YYDEBUG 1

void yyerror(char *msg)
{
    Msg(ERR,msg);
}
%}

%union {
    SymTabEntry  *sym;
    TreeNode     *node;
    ArgListEntry *arg;
    Range        *s_range;
    RefStruct     ref;
    int           wert;
}

%token  BEGIN_
%token  END
%token	CONST
%token  VAR
%token  CHAN
%token	STACK
%token  PROC
%token	VAL
%token  RES
%token  REF
%token  OF
%token  INIT
%token  DO
%token  OD
%token  REPEAT
%token  EXIT
%token	ENTER
%token  PARALLEL
%token  CHOICE
%token  NOT
%token  AND
%token  OR
%token  TRUE
%token  FALSE
%token	MAX
%token	PROCGRP
%token	ENDGRP
%token  <wert> NUMBER
%token	<wert> CONST_IDENT
%token  <sym>  IDENT
%token	CA
%token	EMPTY
%token	FULL
%token  GE
%token  LE
%token	MOD

%left       PARALLEL
%left       CHOICE
%left       ';' ','
%left       OR
%left       AND
%right      NOT
%nonassoc   '=' '#' GE LE '>' '<'
%left       '+' '-'
%left       '*' '/' MOD
%left       UNARYMINUS

%type <wert>     const init
%type <node>     program block scope com alt_set expr valexpr simple
%type <sym>      proc_ident par_list par_decl rpar_decl par_ident
%type <arg>      arg_list
%type <s_range>  menge elem_list elem

%start  program

%%

program
    : block
        {
            DoIt($1);
        }
    ;

block
    : begin scope END
        {
            $$=GenNode(_BLOCK,(TreeNode *)dec_blknum(),$2);
        }
    | begin IDENT scope END IDENT
        {
            $$=GenNode(_BLOCK,(TreeNode *)dec_blknum(),$3);
        }
    ;

begin
    : BEGIN_
        {
            inc_blknum();
        }
    ;

scope
    : xdecl com
        {
            $$=$2;
        }
    | com
        {
            $$=$1;
        }
    ;

xdecl
    : decl
    | xdecl decl
    ;

decl
    : VAR ident_list ':' menge init ';'
        {
            give_typ(_VAR,$4,$5);
        }
    | VAR ident_list ':' CHAN const OF menge ';'
        {
            give_typ(_CHANNEL,$7,$5);
        }
    | VAR ident_list ':' STACK const OF menge ';'
        {
            give_typ(_STACK,$7,$5);
        }
    | PROC proc_ident max_kram
        {
            give_typ(_PROC,NULL,0);
        }
      block ';'
        {
            LinkBody($2,$5);
        }
    | PROC proc_ident '('
        {
            give_typ(_PPROC,NULL,0);
        }
      par_list ')' max_kram
        {
            LinkParlist($2,$5);
        }
      block ';'
        {
            LinkBody($2,$9);
        }
    | CONST IDENT '=' const ';'
        {
            give_typ(MNET_CONST,NULL,$4);
        }
    ;

max_kram
    :
    | MAX const
    ;

menge              : '{' elem_list '}'     { $$=$2; }
    ;

elem_list          : elem                  { $$=insert_elem($1,NULL); }
                   | elem_list ',' elem    { $$=insert_elem($3,$1);   }
    ;

elem               : const                 { $$=GenRange($1,$1); }
                   | const '.' '.' const   { $$=GenRange($1,$4); }
    ;

proc_ident
    : IDENT
        {
            $$=make_ident($1);
        }
    ;

par_list
    : VAL par_decl
        {
            $$=GenParList(NULL,_VAL,$2);
        }
    | par_list ',' VAL par_decl
        {
            $$=GenParList($1,_VAL,$4);
        }
    | CONST par_decl
        {
            $$=GenParList(NULL,_CONSTPAR,$2);
        }
    | par_list ',' CONST par_decl
        {
            $$=GenParList($1,_CONSTPAR,$4);
        }
    | REF rpar_decl
        {
            $$=GenParList(NULL,_REF,$2);
        }
    | par_list ',' REF rpar_decl
        {
            $$=GenParList($1,_REF,$4);
        }
    ;

par_decl
    : par_ident ':' menge
        {
            give_typ(_VAR,$3,_NO_INIT);
            $$=$1;
        }
    ;

rpar_decl
    : par_ident ':' menge
        {
            give_typ(_VAR,$3,_NO_INIT);
            $$=$1;
        }
    | par_ident ':' CHAN const OF menge
        {
            give_typ(_CHANNEL,$6,$4);
            $$=$1;
        }
    | par_ident ':' STACK const OF menge
        {
            give_typ(_STACK,$6,$4);
            $$=$1;  
        }
    ;

par_ident
    : IDENT
        {
            $$=make_par($1);
        }
    ;

ident_list
    : IDENT
        {
            make_ident($1);
        }
    | IDENT ',' ident_list
        {
            make_ident($1);
        }
    ;

init
    : /* empty init */
        {
            $$=_NO_INIT;
        }
    | INIT const
        {
            $$=$2;
        }
    ;

com
    : '<' expr CA
        {
            $$=GenNode(_EXPR,$2,NULL);
        }
    | '<' IDENT EMPTY CA
        {
            $$=GenNode(_EMPTY,(TreeNode *)chk2_ident($2,_CHANNEL,_STACK)
                             ,(TreeNode *)'?');
        }
    | '<' IDENT FULL CA
        {
            $$=GenNode(_FULL,(TreeNode *)chk2_ident($2,_CHANNEL,_STACK)
                            ,(TreeNode *)'!');
        }

    | com ';' com
        {
            $$=GenNode(_SEQ,$1,$3);
        }
    | com PARALLEL com
        {
            $$=GenNode(_PAR,$1,$3);
        }
    | block
        {
            $$=$1;
        }
    | DO com ENTER alt_set OD
        {
            $$=GenNode(_DO,$4,$2);
        }
    | DO alt_set OD
        {
            $$=GenNode(_DO,$2,NULL);
        }
    | IDENT
        {
            $$=GenNode(_CALL,(TreeNode *)chk_ident($1,_PROC,0),NULL);
        }
    | IDENT '('
        {
            chk_ident($1,_PPROC,0);
        }
      arg_list ')'
        {
            if (ActPar)
                Msg(ERR,"Too few arguments.");
            $$=GenNode(_PCALL,(TreeNode *)$1,(TreeNode *)$4);
        }
    ;

arg_list
    : IDENT
        {
            $$=GenArgList(_IDENT,NULL,$1,NULL);
        }
    | arg_list ',' IDENT
        {
            $$=GenArgList(_IDENT,$1,$3,NULL);
        }
    | valexpr
        {
            $$=GenArgList(_VAL,NULL,NULL,$1);
        }
    | arg_list ',' valexpr
        {
            $$=GenArgList(_VAL,$1,NULL,$3);
        }
    ;

alt_set
    : com ';' REPEAT
        {
            $$=GenNode(_REPEAT,$1,NULL);
        }
    | com ';' EXIT
        {
            $$=GenNode(_EXIT,$1,NULL);
        }
    | alt_set CHOICE alt_set
        {
            $$=GenNode(_CHOICE,$1,$3);
        }
    ;

expr
    : '(' expr ')'      { $$=$2; }
    | IDENT             { $$=GenNode(_IDENT,(TreeNode *)$1,NULL); }
    | valexpr           { $$=$1; }
    ;

valexpr
    : simple            { $$=$1; }
    | expr OR expr      { $$=GenNode(_OR,$1,$3); }
    | expr AND expr     { $$=GenNode(_AND,$1,$3); }
    | NOT expr          { $$=GenNode(_NOT,$2,NULL); }
    | expr '=' expr     { $$=GenNode(_EQUAL,$1,$3); }
    | expr '#' expr     { $$=GenNode(_NEQUAL,$1,$3); }
    | expr '<' expr     { $$=GenNode(_LESS,$1,$3); }
    | expr '>' expr     { $$=GenNode(_GREATER,$1,$3); }
    | expr GE expr      { $$=GenNode(_GEQUAL,$1,$3); }
    | expr LE expr      { $$=GenNode(_LEQUAL,$1,$3); }
    | expr '+' expr     { $$=GenNode(_PLUS,$1,$3); }
    | expr '-' expr     { $$=GenNode(_MINUS,$1,$3); }
    | expr '*' expr     { $$=GenNode(_MULT,$1,$3); }
    | expr '/' expr     { $$=GenNode(_DIV,$1,$3); }
    | expr MOD expr     { $$=GenNode(_MOD,$1,$3); }
    | '-' expr %prec UNARYMINUS     { $$=GenNode(_UMIN,$2,NULL); }
    ;

simple
    : '\'' IDENT  { $$=GenNode(_PRE_IDENT
                               ,(TreeNode *)chk_ident($2,_VAR,_CONSTPAR)
                               ,NULL); }
    | IDENT '\''  { $$=GenNode(_POST_IDENT
                               ,(TreeNode *)chk_ident($1,_VAR,_CONSTPAR)
                               ,NULL); }
    | IDENT '?'   { $$=GenNode(_R_CHANNEL
                               ,(TreeNode *)chk2_ident($1,_CHANNEL,_STACK)
                               ,NULL); }
    | IDENT '!'   { $$=GenNode(_W_CHANNEL
                               ,(TreeNode *)chk2_ident($1,_CHANNEL,_STACK)
                               ,NULL); }
    | NUMBER      { $$=GenNode(_SIMPLE,(TreeNode *)$1,NULL); }
    | TRUE        { $$=GenNode(_SIMPLE,(TreeNode *)_TRUE,NULL); }
    | FALSE       { $$=GenNode(_SIMPLE,(TreeNode *)_FALSE,NULL); }
    | CONST_IDENT { $$=GenNode(_SIMPLE,(TreeNode *)$1,NULL); }
    ;

const
    : NUMBER        { $$=$1; }
    | '-' NUMBER    { $$=-$2; }
    | TRUE          { $$=_TRUE; }
    | FALSE         { $$=_FALSE; }
    | CONST_IDENT   { $$=$1; }
    ;

%%
