/**********************************/
/* main.c                         */
/* Burkhard Graves, 06.10.93      */
/* Lars Jenner, Juni 1994         */
/* Burkhard Graves, 25.02.95      */
/**********************************/

#include <string.h>
#include <stdarg.h>
#include <ctype.h>
#include "main.h"
#include "tree.h"
#include "symtab.h"

/*********************************/
/* Datenstrukturen               */
/*********************************/

extern FILE* yyin,*yyout;
int dinit,rec_depth,max_rec_depth;
bool GL_ParserMsg=True,GL_ExplFree=False;

char *prgname,*srcfile,*destfile;
const char *msg_type[]={"Info","Warning","Error","Fatal"};

/************************/
/* Aufruf demonstrieren */
/************************/

static void usage(void)
{
    fprintf(stderr,"usage: %s source [target] [-r#] [-i]\n\n",prgname);
    fprintf(stderr,"-r: set depth of recursion\n");
    fprintf(stderr,"-i: enable standard initialization\n");
    exit(1);
}

/****************/
/* File oeffnen */
/****************/

static FILE *open_file(char *fname,char *mode)
{
    FILE *f;

    if ((f=fopen(fname,mode))==NULL) {
        fprintf(stderr,"Could not open file %s !\n",fname);
        exit(1);
    }
    printf("%c: %s\n",*mode,fname);
    return f;
}

/******************/
/* Flags einlesen */
/******************/

static void read_flags(char *ptr)
{
    while (*++ptr) {
        switch(*ptr) {
        case 'r':
            max_rec_depth=atoi(++ptr);
            while (isdigit(*ptr))
                ++ptr;
            --ptr;
            break;
        case 'i':
            dinit=1;
            break;
        default:
            usage();
        }
    }
}

/*********************************/
/* Hauptprogramm                 */
/*********************************/

extern int yyparse(void);

int main(int argc,char *argv[])
{
    int i;

    printf("\nb2b, compiled %s, written by B. Graves\n\n",__DATE__);

    dinit=rec_depth=0;
    max_rec_depth=5;
    srcfile=destfile=NULL;
    prgname=argv[0];

    for (i=1; i<argc; ++i)
        if (*(argv[i])!='-')
            if (!srcfile)
                srcfile=argv[i];
            else if (!destfile)
                destfile=argv[i];
            else
                usage();
        else
            read_flags(argv[i]);
    if (!srcfile)
        usage();

    yyin=open_file(srcfile,"r");
    yyout=(destfile)?open_file(destfile,"w"):stdout;

    s_init();
    yyparse();

    fclose(yyin);
    if (yyout!=stdout)
        fclose(yyout);

    return 0;
}

/**********************************/
/* END OF FILE main.c             */
/**********************************/
