/**********************************/
/* range.c                        */
/* Burkhard Graves, 14.07.95      */
/**********************************/

#include <string.h>
#include "range.h"

/* Create Range. Similar to GenRange. */
Range *GenRange(int von,int bis)
{
    Range *ptr;

    if (is_bool(von) != is_bool(bis)) {
        Msg(ERR,"Malformed range (expected bool).");
        von=_FALSE;
        bis=_TRUE;
    }
    ptr=(Range *)MYmalloc(sizeof(Range));
    ptr->next=NULL;
    if (bis<von) {
        ptr->von=bis;
        ptr->bis=von;
    } else {
        ptr->von=von;
        ptr->bis=bis;
    }
    return ptr;
}

/* insert element insert into set head. */
Range *insert_elem(Range *insert,Range *head)
{
    Range *ptr1,*ptr2,*ptr3;

    if (!head)
    {
      ptr1=GenRange(insert->von,insert->bis);
      ptr1->next=insert;
      return ptr1;
    }
    ptr2=(ptr1=head)->next;
    for (;;) {
        if (insert->bis<ptr2->von) {                              /* ( ) [ ] */
            (ptr1->next=insert)->next=ptr2;
            if (insert->von<head->von)
                head->von=insert->von;
            break;
        } else if (insert->von>ptr2->bis) {                       /* [ ] ( ) */
            ptr2=(ptr1=ptr2)->next;
            if (!ptr2) {
                head->bis=(ptr1->next=insert)->bis;
                break;
            }
        } else if (insert->von<=ptr2->von) {
            if ((ptr2->von=insert->von)<head->von)
                head->von=insert->von;
            if (insert->bis<=ptr2->bis) {                         /* ( [ ) ] */
                MYfree(insert);
                break;
            } else {                                              /* ( [ ] ) */
                ptr3=(ptr1=ptr2)->next;
                while (ptr3 && insert->bis>=ptr3->von) {
                    if (insert->bis<ptr3->bis)
                        insert->bis=ptr3->bis;
                    ptr3=(ptr2=ptr3)->next;
                    MYfree(ptr2);
                }
                if ((ptr1->bis=insert->bis)>head->bis)
                    head->bis=insert->bis;
                ptr1->next=ptr3;
                MYfree(insert);
                break;
            }
        } else if (insert->bis<=ptr2->bis) {                      /* [ ( ) ] */
            MYfree(insert);
            break;
        } else {                                                  /* [ ( ] ) */
            ptr3=(ptr1=ptr2)->next;
            while (ptr3 && insert->bis>=ptr3->von) {
                if (ptr3->bis>insert->bis)
                    insert->bis=ptr3->bis;
                ptr3=(ptr2=ptr3)->next;
                MYfree(ptr2);
            }
            if ((ptr1->bis=insert->bis)>head->bis)
                head->bis=insert->bis;
            ptr1->next=ptr3;
            MYfree(insert);
            break;
        }
    }
    return RangeCompress(head);
}
 
void delete_set(Range *range)
{
    Range *ptr;
    while (range) {
	ptr=range;
        range=range->next;
        MYfree(ptr);
    }
}

Range *RangeDuplicate(Range *range)
{
    Range* ptr=NULL;

    if(range)
    {
      ptr=GenRange(range->von,range->bis);
      ptr->next=RangeDuplicate(range->next);
    }
    return ptr;
}

void sub_write_range(char *ptr,int x)
{
    if (is_bool(x))
        if (x==_TRUE)
            strcpy(ptr,"true");
        else
            strcpy(ptr,"false");
    else
        sprintf(ptr,"%i",x);
}
 
char *WriteRange(char *buffer,Range *range) 
{
    char *ptr;

    range=range->next;
    ptr=buffer;
    *ptr++='{';
    for (;;) {
        if (range->von==range->bis)
            sub_write_range(ptr,range->von);
        else {
            sub_write_range(ptr,range->von);
            ptr=(char *)strchr(ptr,0);
            *ptr++='.'; *ptr++='.';
            sub_write_range(ptr,range->bis);
        }
        ptr=(char *)strchr(ptr,0);
        if ((range=range->next)!=NULL)
            *ptr++=',';
        else
            break;
    }
    *ptr++='}';
    *ptr=0;
    return buffer;
}

void sub_display_range(FILE *file,int x)
{
    if (is_bool(x))
        if (x==_TRUE)
            fprintf(file,"true");
        else
            fprintf(file,"false");
    else
        fprintf(file,"%i",x);
}
 
void DisplayRange(FILE *file,Range *range)
{
    range=range->next;
    fputc('{',file);
    for (;;) {
        if ((range->von=range->bis))
            sub_display_range(file,range->von);
        else {
            sub_display_range(file,range->von);
            fprintf(file,"..");
            sub_display_range(file,range->bis);
        }
        if ((range=range->next)!=NULL)
            fputc(',',file);
        else
            break;
    }
    fputc('}',file);
}

int IsSubRange(Range *sub,Range *super)
{
    if (sub->von<super->von || sub->bis>super->bis)
        return 0;
    super=super->next;
    while ((sub=sub->next)!=NULL) {
        while (super && sub->von>super->bis)
            super=super->next;
        if (!super || sub->von<super->von || sub->bis>super->bis)
            return 0;
    }
    return 1;
}

int IsInRange(Range *range,int elem)
{
    assert(range);
    if (elem<range->von || elem>range->bis)
        return 0;
    while ((range=range->next)!=NULL && elem>range->bis)
        ;
    return (!range || elem<range->von)?0:1;
}

Range *RangeCompress(Range *range)
{
    Range *ptr1,*ptr2;

    ptr2=(ptr1=range->next)->next;
    while (ptr2) {
        if (ptr1->bis!=ptr2->von-1)
            ptr2=(ptr1=ptr2)->next;
        else {
            ptr1->bis=ptr2->bis;
            ptr1->next=ptr2->next;
            MYfree(ptr2);
            ptr2=ptr1->next;
        }
    }
    return range;
}

