/**********************************/
/* symtab.c                       */
/* Burkhard Graves, 06.10.93      */
/* Lars Jenner, Juni 1994         */
/* Burkhard Graves, 24.02.95      */
/**********************************/

#include <string.h>
#include "symtab.h"
#include "tree.h"

/**********************************/
/* Datenstrukturen                */
/**********************************/

int blknum;             /* Verschachtelungstiefe */

SymTabEntry dummy,      /* Dummy-Element der Symboltabelle   */
            *s_start;   /* Start der Symboltabelle ('lokal') */

/****************************************************************/
/* Initialisierung der Symboltabelle (Aufruf aus 'main' heraus) */
/****************************************************************/

void s_init(void)
{
    SymTabEntry *entry;

    (s_start=entry=&dummy)->s_next=NULL;
    entry->s_name="-";              /* kein gueltiger Bezeichner ! */
    entry->s_type=_VAR;             /* nicht _UDEC  !              */
    entry->s_blknum=blknum=0;       /* ungueltige Blocknummer !    */
    entry->s_body=NULL;
    entry->s_proc=NULL;
    entry->s_parlist=NULL;
    entry->s_partype=_NOPAR;

 /* entry->s_special=undefined;     */
 /* entry->s_range=undefined;       */
 /* entry->s_init_or_cap=undefined; */
}

/****************************************************************************/
/* Absuchen der Symboltabelle nach dem Bezeichner 'ptr'. Konnte er gefunden */
/* werden, so wird ein Zeiger auf ihn zurueckgegeben, ansonsten wird ein    */
/* NULL-Pointer zurueckgegeben. (Aufruf durch 's_lookup')                   */
/****************************************************************************/

static SymTabEntry *s_find(char *ptr)
{
    SymTabEntry *entry;

    entry=s_start;
    while (entry && (*entry->s_name!=*ptr || strcmp(entry->s_name,ptr))) {
        entry=entry->s_next;
    }
    return entry;
}

/**************************************************************/
/* Erzeugung eines neuen Eintrages in der Symboltabelle       */
/* (Aufruf durch 's_lookup' und 'make_ident' und GenParList)  */
/**************************************************************/

static SymTabEntry *s_create(char *name)
{
    SymTabEntry *new_entry;

    (new_entry=(SymTabEntry *)MYmalloc(sizeof(SymTabEntry)))->s_next=s_start;
    s_start=new_entry;

    new_entry->s_name=MYstrdup(name);
    new_entry->s_type=_UDEC;
    new_entry->s_blknum=blknum;

    new_entry->s_body=NULL;
    new_entry->s_proc=NULL;
    new_entry->s_parlist=NULL;
    new_entry->s_partype=_NOPAR;

 /* new_entry->s_special=undefined;      */
 /* new_entry->s_range=undefined;        */
 /* new->entry->s_init_or_cap=undefined; */

    return new_entry;
}

/***************************************************************************/
/* Feststellung, ob ein Bezeichner 'ptr' bereits in der Symboltabelle ist. */
/* Falls nicht, so wird ein entsprechender Eintrag erzeugt.                */
/* Rueckgabewert ist ein Zeiger auf den Eintrag. (Aufruf durch den Scanner)*/
/***************************************************************************/

SymTabEntry *s_lookup(char *ptr)
{
    SymTabEntry *entry;

    return ((entry=s_find(ptr))!=NULL)?entry:s_create(ptr);
}

/*************************************************************************/
/* Feststellung, ob eine Variable mehrfach definiert wurde, oder ob eine */
/* lokale Definition eine globale ueberdeckt.                            */
/* (Aufruf durch den Parser in der 'ident_list'-Regel)                   */
/*************************************************************************/

SymTabEntry *make_ident(SymTabEntry *entry)
{
    if (entry->s_type!=_UDEC) {
        if (entry->s_blknum==blknum) {
            Msg(ERR,"Multiple define of identifier %s.",entry->s_name);
        } else {
            entry=s_create(entry->s_name);
        }
    }
    entry->s_special=0;
    entry->s_body=NULL;
    entry->s_proc=NULL;
    entry->s_parlist=NULL;
    entry->s_partype=_NOPAR;

 /* entry->s_range=undefined;       */
 /* entry->s_init_or_cap=undefined; */

    return entry;
}

/*************************************************************************/

SymTabEntry *make_par(SymTabEntry *entry)
{

    if (entry->s_type!=_UDEC) {
        if (entry->s_blknum==blknum+1) {
            Msg(ERR,"Multiple define of parameter %s.",entry->s_name);
        } else {
            entry=s_create(entry->s_name);
        }
    }

    entry->s_blknum =blknum+1;    /* lokal zum folgenden Prozedurrumpf! */

    entry->s_parlist=NULL;
    entry->s_partype=_VAL;        /* default */
    entry->s_proc=ActProc;        /* doppeltgemoppelt, auch in GenParList */
    entry->s_body=NULL;

    entry->s_special=0;

 /* entry->s_range=undefined;       */
 /* entry->s_init_or_cap=undefined; */

    return entry;
}

/************************************************************************/
/* Feststellung, ob eine Variable den geforderten Typ besitzt.          */
/* (Aufruf durch '?-primed' und 'r_w_channel')                          */
/* Bei Prozeduren wird noch deren Verwendung in s_special vermerkt, und */
/* bei Parameter-Prozeduren wird ActPar gesetzt.                        */
/************************************************************************/

SymTabEntry *chk_ident(SymTabEntry *entry,int type,int partype)
{
    if (entry->s_type==_UDEC)
        Msg(ERR,"Identifier %s was not declared.",entry->s_name);
    if (entry->s_type!=type || (partype && entry->s_partype==partype))
        Msg(ERR,"Wrong usage of identifier %s.",entry->s_name);
    if (entry->s_type==_PROC || entry->s_type==_PPROC )
        entry->s_special=255;
    if (entry->s_type==_PPROC)
        ActPar=entry->s_parlist;
    return entry;
}

SymTabEntry *chk2_ident(SymTabEntry *entry,int type1,int type2)
{
    if (entry->s_type==_UDEC)
        Msg(ERR,"Identifier %s was not declared.",entry->s_name);
    if (entry->s_type!=type1 && entry->s_type!=type2)
        Msg(ERR,"Wrong usage of identifier %s.",entry->s_name);
    return entry;
}

/*********************************************************************/
/* Inkrementierung von 'blknum'.                                     */
/* (Aufruf durch den Parser am Beginn eines Blockes ('block'-Regel)) */
/*********************************************************************/

void inc_blknum(void)
{
    ++blknum;
}

/*******************************************************************/
/* Dekrementierung von 'blknum' und Entfernung von lokalen         */
/* Eintraegen aus der Symboltabelle. (Aber noch nicht loeschen,    */
/* sondern nur zurueckgehen !)                                     */
/* (Aufruf durch den Parser am Ende eines Blockes ('block'-Regel)) */
/*******************************************************************/

SymTabEntry *dec_blknum(void)
{
    SymTabEntry *ptr1,*ptr2,*memo;

    memo=ptr1=ptr2=s_start;

    while (ptr1->s_blknum==blknum) {
        if (ptr1->s_type!=_UDEC && !(ptr1->s_special & (_G_RW | _G_PRUPR))) {
            Msg(WAR,"Identifier %s declared, but not used.",ptr1->s_name);
#if 0
            ptr1->s_type=_UDEC; /* !!! */
#endif
        }
        ptr1->s_stack=NULL;
        ptr2=ptr1;
        ptr1=ptr1->s_next;
    }
    s_start=ptr1;
    --blknum;
    if (ptr1==ptr2) {
        memo=NULL;
    } else {
        ptr2->s_next=NULL;
    }
    return memo;
}

/************************************************************************/
/* Verleiht neuen Eintraegen in der Symboltabelle diverse Eigenschaften */
/* (Aufruf durch den Parser in der 'decl'-Regel)                        */
/* Bei Parameter-Prozedur-Deklarationen wird ActProc gesetzt.           */
/************************************************************************/

void give_typ(int type,Range *range,int init_or_cap)
{
    SymTabEntry *ptr;

    if ((ptr=s_start)->s_type!=_UDEC)
        Msg(FAT,"Parse error at give_typ !",NULL);

    while (ptr->s_type==_UDEC) {
        ptr->s_range=(ptr->s_next && ptr->s_next->s_type==_UDEC)?
                     RangeDuplicate(range):range;
        if (dinit && init_or_cap==_NO_INIT)
            init_or_cap = range->von;
        switch(ptr->s_type=type) {
        case MNET_CONST:
            ptr->s_init_or_cap=init_or_cap;
            break;
        case _VAR:
            if (init_or_cap!=_NO_INIT && !IsInRange(range,init_or_cap)) {
                Msg(ERR,"Wrong initialisation.");
            }
            ptr->s_init_or_cap=init_or_cap;
            break;
        case _CHANNEL:
        case _STACK:
            if (init_or_cap<0) {
                Msg(ERR,"Wrong capacity");
            }
            ptr->s_init_or_cap=init_or_cap;
            break;
        case _PROC:
            ptr->s_init_or_cap=0;
            break;
        case _PPROC:
            ptr->s_init_or_cap=0;
            ActProc=ptr;
            break;
        default:
            Msg(FAT,"default give_typ ?");
            break;
        }
        ptr=ptr->s_next;
    }
}

/**********************************/
/* END OF FILE symtab.c           */
/**********************************/
