/**********************************/
/* tree.c                         */
/* Burkhard Graves, 10.10.93      */
/* Lars Jenner, Juni 1994         */
/* Burkhard Graves, 24.02.95      */
/**********************************/

#include <string.h>
#include <ctype.h>
#include "tree.h"
#include "symtab.h"

/**************/
/* Prototypen */
/**************/

static void sub_translate PROTO ((TreeNode *node));
static void   delete_tree PROTO ((TreeNode *node));
static void         block PROTO ((TreeNode *node));
static void           com PROTO ((TreeNode *node));
static  int       alt_set PROTO ((TreeNode *node,unsigned mode,int flag));

/**********************************/
/* Datenstrukturen                */
/**********************************/

SymTabEntry* ActPar=NULL;
SymTabEntry* ActProc=NULL;

char buffer[1024];
char *txt_ptr=buffer;

/**********************************************************************/
/* Verwaltung eines Stacks fuer einen Symboltabelleneintrag, in dem   */
/* die Rekursionstiefen aller aktuellen Aktivierungen abgelegt werden */
/**********************************************************************/

static void push(SymTabEntry *ptr)
{
    Stack *new;

    ptr->s_blknum=rec_depth;
    (new=(Stack *)MYmalloc(sizeof(Stack)))->next=ptr->s_stack;
    if (ptr->s_stack) {
        if (ptr->s_partype==_REF) {
            new->ref=ptr->s_stack->ref;
            new->depth=ptr->s_stack->depth;
        } else {
            new->ref=NULL;
            new->depth=(ptr->s_partype==_CONSTPAR)?
                       ptr->s_stack->depth:ptr->s_blknum;
        }
    } else {
        new->ref=NULL;
        new->depth=ptr->s_blknum;
    }
    ptr->s_stack=new;
}

static void pop(SymTabEntry *ptr)
{
    Stack *old;

    ptr->s_blknum=rec_depth;
    ptr->s_stack=(old=ptr->s_stack)->next;
    MYfree(old);
}

/******************************/
/* Verwaltung des Textpuffers */
/******************************/

static char *inc_txt_ptr(void)
{
    char *ptr;

    txt_ptr+=strlen(ptr=txt_ptr)+1;
    if (txt_ptr-buffer>896)
        txt_ptr=buffer;
    return ptr;
}

/************************************************/
/* Drei Funktionen zur Formatierung der Ausgabe */
/************************************************/

static void tabulator(void)
{
    if (rec_depth && yycoluno==0)
        fprintf(yyout,"%*c",yycoluno=rec_depth*4,' ');
}

static void newline(void)
{
    if (yycoluno)
        fputc('\n',yyout);
    yycoluno=0;
}

static void new_tab(void)
{
    newline();
    tabulator();
}

/**********************************************************/
/* Invertierung der (lokalen) Symboltabelle (fuer's Auge) */
/**********************************************************/

static SymTabEntry *invert_list(SymTabEntry *ptr1)
{
    SymTabEntry *ptr2,*ptr3;

    ptr3=NULL;
    while (ptr1) {
        ptr1=(ptr2=ptr1)->s_next;
        ptr2->s_next=ptr3;
        ptr3=ptr2;
    }
    return ptr3;
}

/****************************************/
/* Loeschen der (lokalen) Symboltabelle */
/****************************************/

static void delete_sym(SymTabEntry *ptr)
{
    if (ptr) {
        delete_sym(ptr->s_next);
        if (ptr->s_body)
            delete_tree(ptr->s_body);
        MYfree(ptr->s_name);
        delete_set(ptr->s_range);
        MYfree(ptr);
    }
}

/*******************************/
/* Loeschen der Argument-Liste */
/*******************************/

static void delete_arg(ArgListEntry *ptr)
{
    if (ptr) {
        delete_arg(ptr->next);
        if (ptr->value)
            delete_tree(ptr->value);
        MYfree(ptr);
    }
}

/*****************************/
/* Loeschen des Syntaxbaumes */
/*****************************/

static void delete_tree(TreeNode *node)
{
    switch(node->type) {
    case _BLOCK:
        if (node->op2) {
            TreeNode *node2;

            node2=node->op2;
            node->op2=NULL;
            delete_tree(node2);
            delete_sym((SymTabEntry *)node->op1);
        } else
            node=NULL;
        break;
    case _CALL:
    case _PCALL:
        delete_arg((ArgListEntry *)node->op2);
        if (((SymTabEntry *)node->op1)->s_body) {
            delete_tree(((SymTabEntry *)node->op1)->s_body);
            ((SymTabEntry *)node->op1)->s_body=NULL;
        }
        break;
    case _IDENT:
    case _PRE_IDENT:
    case _POST_IDENT:
    case _R_CHANNEL:
    case _W_CHANNEL:
    case _SIMPLE:
    case _FULL:
    case _EMPTY:
        break;
    case _DO:
        delete_tree(node->op1);
        if (node->op2)
            delete_tree(node->op2);
        break;
    case _REPEAT:
    case _EXIT:
    case _EXPR:
    case _NOT:
    case _UMIN:
        delete_tree(node->op1);
        break;
    case _SEQ:
    case _PAR:
    case _CHOICE:
    case _OR:
    case _AND:
    case _EQUAL:
    case _NEQUAL:
    case _LESS:
    case _GREATER:
    case _GEQUAL:
    case _LEQUAL:
    case _PLUS:
    case _MINUS:
    case _MULT:
    case _DIV:
    case _MOD:
        delete_tree(node->op1);
        delete_tree(node->op2);
        break;
    default:
        Msg(FAT,"default delete_tree ?");
        break;
    }
    if (node)
        MYfree(node);
}

/******************************************************/
/* Einen numerischen Wert in einen String uebersetzen */
/******************************************************/

static char *transl(int x)
{
    if (x==_TRUE) {
        strcpy(txt_ptr,"true");
    } else if (x==_FALSE) {
        strcpy(txt_ptr,"false");
    } else {
        sprintf(txt_ptr,"%i",x);
    }
    return inc_txt_ptr();
}

/********************************************************************/
/* Den Namen eines Symbols um die Rekursionstiefe ergaenzen, in dem */
/* das Symbol auftaucht.                                            */
/********************************************************************/

static char *fn0(SymTabEntry *ptr,int x)
{
    char numtxt[10];

    strcpy(txt_ptr,ptr->s_name);
    sprintf(numtxt,"%d",x);
    strcat(txt_ptr,numtxt);
    return inc_txt_ptr();
}

static char *fn1(SymTabEntry *ptr)
{
    return fn0(ptr,ptr->s_stack->depth);
}

static char *fn2(SymTabEntry *ptr)
{
    return fn0(ptr,ptr->s_stack->next->depth);
}

/********************************************/
/* Syntaxbaum in einen Ausdruck uebersetzen */
/********************************************/

static void referenz(SymTabEntry *ptr)
{
    if (ptr->s_partype==_CONSTPAR)
        transl(ptr->s_stack->depth);
    else if (ptr->s_stack->ref) {
	fn0(ptr->s_stack->ref,ptr->s_stack->depth);
    } else {
        /* Achtung bei rek. Prozeduraufruf */
        if (yylineno && ptr->s_stack->depth==rec_depth)
            fn2(ptr);
        else
            fn1(ptr);
    }
    --txt_ptr;
}

static void un_translate(TreeNode *node, char *ptr)
{
    strcpy(txt_ptr,ptr);
    txt_ptr+=strlen(ptr);
    sub_translate(node->op1);
}

static void bin_translate(TreeNode *node, char *ptr)
{
    sub_translate(node->op1);
    strcpy(txt_ptr,ptr);
    txt_ptr+=strlen(ptr);
    sub_translate(node->op2);
}

static void sub_translate(TreeNode *node)
{
    *txt_ptr++='(';
    switch (node->type) {
    case _IDENT:
        referenz((SymTabEntry *)node->op1);
        break;
    case _OR:
        bin_translate(node," or ");
        break;
    case _AND:
        bin_translate(node," and ");
        break;
    case _NOT:
        un_translate(node," not ");
        break;
    case _EQUAL:
        bin_translate(node," = ");
        break;
    case _NEQUAL:
        bin_translate(node," # ");
        break;
    case _LESS:
        bin_translate(node," < ");
        break;
    case _GREATER:
        bin_translate(node," > ");
        break;
    case _GEQUAL:
        bin_translate(node," >= ");
        break;
    case _LEQUAL:
        bin_translate(node," <= ");
        break;
    case _PLUS:
        bin_translate(node," + ");
        break;
    case _MINUS:
        bin_translate(node," - ");
        break;
    case _MULT:
        bin_translate(node," * ");
        break;
    case _DIV:
        bin_translate(node," / ");
        break;
    case _MOD:
        bin_translate(node," % ");
        break;
    case _UMIN:
        un_translate(node," - ");
        break;
    case _PRE_IDENT:
        *txt_ptr++='\'';
        referenz((SymTabEntry *)node->op1);
        break;
    case _POST_IDENT:
        referenz((SymTabEntry *)node->op1);
        *txt_ptr++='\'';
        break;
    case _R_CHANNEL:
        referenz((SymTabEntry *)node->op1);
        *txt_ptr++='?';
        break;
    case _W_CHANNEL:
        referenz((SymTabEntry *)node->op1);
        *txt_ptr++='!';
        break;
    case _SIMPLE:
        transl((int)(node->op1));
        --txt_ptr;
        break;
    }
    *txt_ptr++=')';
}

static char *translate(TreeNode *node)
{
    txt_ptr=buffer;
    sub_translate(node);
    *txt_ptr='\0';
    return buffer;
}

/**********************************************************/
/* Symboltabelle bei Ausgang aus einem Block modifizieren */
/**********************************************************/

static void leave_block(SymTabEntry *ptr)
{
    --rec_depth;
    while (ptr) {
        pop(ptr);
        ptr=ptr->s_next;
    }
}

/*****************************/
/* Ausgabe der Deklarationen */
/*****************************/

static char *display_menge(Range *range)
{
    WriteRange(txt_ptr,range);
    return inc_txt_ptr();
}

static void declarations(SymTabEntry *ptr)
{
    FILE *stream=yyout;

    ++rec_depth;
    while (ptr) {
        push(ptr);
        if (ptr->s_partype==_NOPAR) {
            switch(ptr->s_type) {
            case _UDEC:
                break;
            case _VAR:
                tabulator();
                fprintf(stream,"var %s: %s"
                              ,fn1(ptr)
                              ,display_menge(ptr->s_range));
                if (ptr->s_init_or_cap!=_NO_INIT) {
                    fprintf(stream," init %s",transl(ptr->s_init_or_cap));
                }
                fputc(';',stream);
                newline();
                break;
            case _CHANNEL:
            case _STACK:
                tabulator();
                fprintf(stream,"var %s: %s %s of %s;"
                              ,fn1(ptr)
                              ,(ptr->s_type==_STACK)?"stack":"chan"
                              ,transl(ptr->s_init_or_cap)
                              ,display_menge(ptr->s_range));
                newline();
                break;
            case _PROC:
            case _PPROC:
            case MNET_CONST:
                break;
            default:
                Msg(FAT,"default declarations ?");
            }
        }
        ptr=ptr->s_next;
    }
}

/****************************************/
/* Aufruf einer Prozedur mit Parametern */
/****************************************/

static void pcall(SymTabEntry *proc,ArgListEntry *arglist)
{
             int flag;
    ArgListEntry *arg;
            FILE *stream=yyout;

    new_tab();
    fprintf(stream,"begin");
    newline();
    if (rec_depth<max_rec_depth) {
        declarations((SymTabEntry *)proc->s_body->op1);

        arg = arglist;
        while (arg) {
            if (arg->parameter->s_partype==_VAL) {
                assert(arg->parameter->s_type==_VAR);
                tabulator();
                fprintf(stream,"var %s: %s"
                              ,fn1(arg->parameter)
                              ,display_menge(arg->parameter->s_range));
                flag=0;
                if (arg->value && arg->value->type==_SIMPLE) {
                    fprintf(stream," init %s"
                                  ,transl((int)(arg->value->op1)));
                    flag=1;
                } else if (arg->argument
                           && arg->argument->s_partype==_CONSTPAR) {
                    fprintf(stream," init %s"
                                  ,transl(arg->argument->s_stack->depth));
                    flag=1;
                } else if (arg->parameter->s_init_or_cap!=_NO_INIT) {
                    fprintf(stream," init %s"
                                  ,transl(arg->parameter->s_init_or_cap));
                }
                fputc(';',stream);
                newline();

                if (!flag) {
                    tabulator();
                    fprintf(stream,"<%s'=",fn1(arg->parameter));
                    if (arg->value) {
                        yylineno=1; /* siehe referenz() */
                        fprintf(stream,translate(arg->value));
                        yylineno=0;
                    } else {
                        if (arg->argument->s_stack->ref)
                            fprintf(stream,fn0(arg->argument->s_stack->ref
                                               ,arg->argument->s_stack->depth));
                        else
                            fprintf(stream,(arg->argument->s_stack->depth
                                            ==rec_depth)?
                                            fn2(arg->argument)
                                           :fn1(arg->argument));
                        if (arg->argument->s_type==_CHANNEL
                            || arg->argument->s_type==_STACK)
                            fputc('?',stream);
                    }
                    fprintf(stream,">; ");
                }
            }
            arg = arg->next;
        }
        newline();

        arg = arglist;
        while (arg) {
            if (arg->parameter->s_partype==_CONSTPAR) {
                arg->parameter->s_stack->depth
                =(arg->value)?
                  (int)(arg->value->op1)
                 :(arg->argument->s_blknum==rec_depth)?
		   arg->argument->s_stack->next->depth 
                  :arg->argument->s_stack->depth;
            }
            arg=arg->next;
        }

        arg = arglist;
        while (arg) {
            if (arg->parameter->s_partype==_REF) {
                if (arg->argument->s_partype==_REF) {
                    if (arg->argument->s_blknum==rec_depth) {
		        arg->parameter->s_stack->ref=arg->argument
                                                     ->s_stack
		                                     ->next->ref; 
		        arg->parameter->s_stack->depth=arg->argument
                                                       ->s_stack
		                                       ->next->depth; 
                    } else {
		        arg->parameter->s_stack->ref=arg->argument
                                                     ->s_stack
		                                     ->ref; 
		        arg->parameter->s_stack->depth=arg->argument
                                                       ->s_stack
		                                       ->depth; 
                    }
                } else {
                    arg->parameter->s_stack->ref=arg->argument;
                    arg->parameter->s_stack->depth
                    =(arg->argument->s_blknum==rec_depth)?
                      arg->argument->s_stack->next->depth
                     :arg->argument->s_blknum;
                }
            }
            arg = arg->next;
        }

        com(proc->s_body->op2);
        leave_block((SymTabEntry *)proc->s_body->op1);
    } else {
        ++rec_depth;
        tabulator();
        fprintf(stream,"<false>");
        --rec_depth;
    }
    new_tab();
    fprintf(stream,"end");
}

/***************************************/
/* Abarbeitung eines com-Syntaxknotens */
/***************************************/

static void com(TreeNode *node)
{
    FILE *stream=yyout;

    switch(node->type) {
    case _BLOCK:
        block(node);
        break;
    case _EXPR:
        tabulator();
#if 1
        translate(node->op1);
        if ((int)(yycoluno+=strlen(buffer))>70)
            new_tab();
        fprintf(stream,"<%s>",buffer);
#else
        fprintf(stream,"<%s>",translate(node->op1));
#endif
        break;
    case _FULL:
    case _EMPTY:
        tabulator();
        txt_ptr=buffer;
        *txt_ptr++='<';
        referenz((SymTabEntry *)node->op1);
        *txt_ptr++=(char)(int)node->op2;
        *txt_ptr++=(char)(int)node->op2;
        *txt_ptr++='>';
        *txt_ptr='\0';
        fprintf(stream,buffer);
        break;
    case _SEQ:
        com(node->op1);
        tabulator();
        fprintf(stream,"; "); yycoluno+=2;
        com(node->op2);
        break;
    case _PAR:
        com(node->op1);
        new_tab();
        fprintf(stream,"||");
        newline();
        com(node->op2);
    break;
    case _DO:
        new_tab();
        fprintf(stream,"do ");
        if (node->op2) {
            com(node->op2);
            tabulator();
            fprintf(stream," enter");
            newline();
        }
        alt_set(node->op1,_EXIT,alt_set(node->op1,_REPEAT,0));
        new_tab();
        fprintf(stream,"od");
        newline();
        break;
    case _CALL:
        block(((SymTabEntry *)node->op1)->s_body);
        break;
    case _PCALL:
        pcall((SymTabEntry*)(node->op1),(ArgListEntry*)(node->op2));
        break;
    default:
        Msg(FAT,"default com ?");
    }
}

/*******************************************/
/* Abarbeitung eines alt_set-Syntaxknotens */
/*******************************************/

static int alt_set(TreeNode *node,unsigned mode,int flag)
{
    FILE *stream=yyout;

    switch(node->type) {
    case _REPEAT:
    case _EXIT:
        if (mode==node->type) {
            new_tab();
            fprintf(stream,(flag)?"[] ":"   ");
            com(node->op1);
            tabulator();
            fprintf(stream,(mode==_EXIT)?"; exit":"; repeat");
            newline();
            flag=1;
        }
        break;
    case _CHOICE:
        flag=alt_set(node->op1,mode,flag);
        flag=alt_set(node->op2,mode,flag);
        break;
    default:
        Msg(FAT,"default alt_set ?");
    }
    return flag;
}

/*****************************************/
/* Abarbeitung eines block-Syntaxknotens */
/*****************************************/

static void block(TreeNode *node)
{
    FILE *stream=yyout;

    node->op1=(TreeNode *)invert_list((SymTabEntry *)(node->op1));
    new_tab();
    fprintf(stream,"begin"); yycoluno+=5;
    newline();
    if (rec_depth<max_rec_depth) {
        declarations((SymTabEntry *)node->op1);
        com(node->op2);
        leave_block((SymTabEntry *)node->op1);
    } else {
        ++rec_depth;
        tabulator();
        fprintf(stream,"<false>");
        --rec_depth;
    }
    new_tab();
    fprintf(stream,"end"); yycoluno+=3;
    newline();
}

/*************************************************************************/
/* Verknuepft den Bezeicher ProcIdent in der Symboltabelle mit           */
/* dem Prozedurrumpf Body. Auerdem wird s_special zurueckgesetzt, fuer  */
/* den Fall, da es einen rekursiven Aufruf im Rumpf gab, der aber       */
/* nicht als Verwendung zaehlt. (siehe chk_ident)                        */
/*************************************************************************/

void LinkBody(SymTabEntry *ProcIdent,TreeNode *Body)
{
    ProcIdent->s_body    = Body;
    ProcIdent->s_special = 0;
}

/***************************************************************************/
/* Verknuepft den Bezeicher ProcIdent in der Symboltabelle mit             */
/* der erzeugten Parameterliste, und setzt ActProc zurueck.(s.GenParList)  */
/***************************************************************************/

void LinkParlist(SymTabEntry *ProcIdent,SymTabEntry *ParList)
{
    ProcIdent->s_parlist = ParList;
    ActProc = NULL;
}

/***************************************************************************/
/* Haengt an die bisherige ParList den Parameter Par an, und               */
/*   gibt ihm den jeweiligen Parametertyp, sowie den                       */
/*   ProcIdent ActProc der aktuellen Deklaration, der in give_typ gesetzt, */
/*   und in LinkParlist zurueckgesetzt wird.                               */
/***************************************************************************/

SymTabEntry *GenParList(SymTabEntry *ParList, int Type, SymTabEntry *Par)
{
    SymTabEntry *ParPtr;

    Par->s_parlist = NULL;
    Par->s_partype = Type;
    Par->s_proc    = ActProc;

    ParPtr = ParList;
    if (!ParPtr)
        return Par;
    else {
        while (ParPtr->s_parlist)
            ParPtr = ParPtr->s_parlist;
        ParPtr->s_parlist = Par;
        return ParList;
    }
}

/***************************************************************************/
/* Haengt an die bisherige ArgList das Argument Arg an, wofuer ein neues   */
/* Element NewArg geschaffen wird. Es wird die Vertraeglichkeit            */
/* mit dem tatsaechlich uebergebenen Ausdruck (_IDENT oder _VAL)           */
/* ueberprueft.                                                            */
/***************************************************************************/

ArgListEntry *GenArgList(int ArgType
                         ,ArgListEntry *ArgList
                         ,SymTabEntry *argument
                         ,TreeNode *value)
{
    ArgListEntry *NewArg,*ArgPtr;

    if (!ActPar)
        Msg(ERR,"Too many arguments.");
    (NewArg=(ArgListEntry*)MYmalloc(sizeof(ArgListEntry)))->next=NULL;
    NewArg->parameter=ActPar;
    NewArg->value=value;
    NewArg->argument=argument;

    switch (ArgType) {
    case _IDENT:
        switch (argument->s_type) {
        case _UDEC :
            Msg(ERR,"Identifier %s was not declared.",argument->s_name);
            break;
        case _VAR:
            argument->s_special|=_G_PRUPR;
            if (ActPar->s_partype==_CONSTPAR) {
                if (argument->s_partype!=_CONSTPAR)
                    Msg(ERR,"Expected constant for parameter %s."
                          ,ActPar->s_name);
                if (!IsSubRange(argument->s_range,ActPar->s_range))
                    Msg(WAR,"Value ranges for parameter and argument are incompatible.");
            } else if (ActPar->s_partype==_REF) {
                if (argument->s_partype==_CONSTPAR)
                    Msg(ERR,"Passing a reference parameter (%s) \na constant (%s) is forbidden.",argument->s_name,ActPar->s_name);
                if (ActPar->s_type!=_VAR)
                    Msg(ERR,"Expected channel/stack identifier for reference parameter %s.",ActPar->s_name);
                if (!IsSubRange(argument->s_range,ActPar->s_range))
                    Msg(WAR,"Value range for parameter and argument are incompatible.");
            }
            break;
        case _CHANNEL:
        case _STACK:
            argument->s_special|=_G_RW;
            if (ActPar->s_partype==_REF) {
                if (ActPar->s_type!=argument->s_type /* _CHANNEL */)
                    Msg(ERR,"Type conflict for reference parameter %s."
                          ,ActPar->s_name);
                if (!IsSubRange(argument->s_range,ActPar->s_range))
                    Msg(WAR,"Value range for parameter and argument are incompatible.");
            }
            break;
        case _PROC :
        case _PPROC :
            Msg(ERR,"Wrong usage of identifier %s as parameter.",argument->s_name);
            break;
        default:
            Msg(FAT,"default argidenttype ?");
            break;
        }
        break;
    case _VAL:
        if (ActPar->s_partype==_CONSTPAR) {
            if (value->type!=_SIMPLE)
                Msg(ERR,"Expected constant for parameter %s.",ActPar->s_name);
            if (!IsInRange(ActPar->s_range,(int)(value->op1)))
                Msg(ERR,"Constant not in range of parameter %s."
                      ,ActPar->s_name);
        }
        if (ActPar->s_partype==_REF)
            Msg(ERR,"Expected Pass-by-Reference for parameter %s.",ActPar->s_name);
        break;
    default:
        Msg(FAT,"default argtype ?");
        break;
    }
    ActPar=ActPar->s_parlist;

    ArgPtr=ArgList;
    if (!ArgPtr) {
        return NewArg;
    } else {
        while (ArgPtr->next)
            ArgPtr=ArgPtr->next;
        ArgPtr->next=NewArg;
        return ArgList;
    }
}

/**************************************/
/* Erzeugung eines Syntaxbaum-Knotens */
/**************************************/

TreeNode *GenNode(unsigned type,TreeNode *op1,TreeNode *op2)
{
    TreeNode *node;

    (node=(TreeNode *)MYmalloc(sizeof(TreeNode)))->type=type;
    node->op1=op1;
    node->op2=op2;
    switch (type) {
    case _IDENT:
        ((SymTabEntry *)op1)->s_special|=_LG_UNPRIMED;
        break;
    case _PRE_IDENT:
    case _POST_IDENT:
        ((SymTabEntry *)op1)->s_special|=_LG_PRIMED;
        break;
    case _FULL:
    case _R_CHANNEL:
        ((SymTabEntry *)op1)->s_special|=_LG_READING;
        break;
    case _EMPTY:
    case _W_CHANNEL:
        ((SymTabEntry *)op1)->s_special|=_LG_WRITING;
        break;
    default:
        break;
    }
    return node;
}

/************************************/
/* Start des Uebersetzungsvorganges */
/************************************/

void DoIt(TreeNode *node)
{
    yylineno=yycoluno=0;
    block(node);
    delete_tree(node);
}

/**********************/
/* END OF FILE tree.c */
/**********************/

