/**********************************/
/* tree.h                         */
/* Burkhard Graves, 08.10.93      */
/* Lars Jenner, Juni 1994         */
/* Burkhard Graves, 24.02.95      */
/**********************************/

#ifndef __TREE_H__
#define __TREE_H__

#include "main.h"
#include "range.h"

extern int yylineno,yycoluno;

/**********************************/
/* Globale Defines                */
/**********************************/

#define _BLOCK      1
#define _EXPR       2
#define _SEQ        3
#define _PAR        4
#define _DO         5
#define _REPEAT     6
#define _EXIT       7
#define _CHOICE     8
#define _CALL       9
#define _PCALL      10
#define _VAL        11
#define _CONSTPAR   12
#define _REF        13
#define _IDENT      14
#define _NOPAR      15
#define _OR         16
#define _AND        17
#define _NOT        18
#define _EQUAL      19
#define _NEQUAL     20
#define _LESS       21
#define _GREATER    22
#define _GEQUAL     23
#define _LEQUAL     24
#define _PLUS       25
#define _MINUS      26
#define _MULT       27
#define _DIV        28
#define _MOD        29
#define _UMIN       30
#define _PRE_IDENT  31
#define _POST_IDENT 32
#define _R_CHANNEL  33
#define _W_CHANNEL  34
#define _SIMPLE     35
#define _EMPTY      36
#define _FULL       37

/**********************************/
/* Globale Typdefinitionen        */
/**********************************/

typedef struct treenode {
    unsigned type;
    struct treenode *op1,*op2;
} TreeNode;

typedef struct stack {
    int                 depth;
    struct symtabentry *ref;   /* falls Referenzparameter       */
    struct stack       *next;
} Stack;

typedef struct RefStruct_ {
  int                    start_l, start_c; /* Startzeile & Startspalte */
  int                      end_l,   end_c; /*   Endzeile &   Endspalte */
} RefStruct;

typedef struct symtabentry {
    RefStruct		   s_ref;	  /* Definition im Quelltext       */
    int                    s_type;        /* Typ                           */
    char                  *s_name;        /* Name                          */
    int                    s_blknum;      /* Blocknummer                   */
    struct symtabentry    *s_next;        /* Verkettung                    */
/*     Falls Parameter :                                                   */
    int                    s_partype;     /* Parametertyp                  */
    struct symtabentry    *s_proc;        /* die zugehoerige Proz.         */
/*     Falls Prozedur :                                                    */
    struct symtabentry    *s_parlist;     /* die Parameterliste            */
    TreeNode              *s_body;        /* der Prozedurrumpf             */

    int                    s_special;     /* fuer diverse Zwecke           */
    Range                 *s_range;       /* Wertebereich                  */
    int                    s_init_or_cap; /* Initialisierungswert          */
                                          /*  oder Kapazitaet              */
    Stack                 *s_stack;       /* fuer Rekursion                */
} SymTabEntry;

typedef struct arglistentry {
    SymTabEntry         *parameter;
    SymTabEntry         *argument;
    TreeNode            *value;
    struct arglistentry *next;
} ArgListEntry;

/**********************************/
/* Globale Datenstrukturen        */
/**********************************/

extern SymTabEntry* ActPar;
extern SymTabEntry* ActProc;

/**********************************/
/* Globale Funktionen             */
/**********************************/

void     LinkBody PROTO ((SymTabEntry *ProcIdent,TreeNode *Body));
void  LinkParlist PROTO ((SymTabEntry *ProcIdent,SymTabEntry *Parlist));

SymTabEntry  *GenParList PROTO ((SymTabEntry *ParList
                                 ,int Type,SymTabEntry *Par));

ArgListEntry *GenArgList PROTO ((int ArgType
                                 ,ArgListEntry *ArgList
                                 ,SymTabEntry *argument
                                 ,TreeNode *value));

TreeNode   *GenNode PROTO ((unsigned type,TreeNode *op1,TreeNode *op2));
void           DoIt PROTO ((TreeNode *node));

#endif

/**********************************/
/* END OF FILE tree.h             */
/**********************************/
