/******************************************************************************/
/* ExprParse.y                                                                */
/*                                                                            */
/* BISON-Parser fuer Transitionenguards und Wertebereiche                     */
/*                                                                            */
/* Martin Ackermann, Sommer 1995                                              */
/* Martin Ackermann, Winter 1996/97                                           */
/*   - Makros hinzugefuegt                                                    */
/******************************************************************************/

%{
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "f_parse.h"
#include "compose.h"
#include "macros.h"


#define YYDEBUG 1


extern int   vtlex( void );
extern char *vttext;


Var      parsed_comnr;          /* com_nr der Variablen, deren Defs gerade 
                                   geparsed werden                           */
Cardinal foreach_counter;       /* numeriert die FOREACHs eines Guards       */

bool     add_with_cmd;          /* True: Werte werden nicht direkt eingefuegt*/

Cardinal CmdEntries;            /* zum Zwischenspeichern der Kommando-Anzahl */

bool     foreach_expr = False;  /* True: die expr eines foreach wird geparsed*/

Cardinal anz_nodes;             /* zum Zwischenspeichern der Knoten-Anzahl   */

typedef struct c_val_type_ { 
  bool     is_value;
  Integer *ptr;
} c_val_type;


/******************************************************************************/
/* Das Kreuzprodukt parsed_range besteht aus mehreren Einzelmengen.           */
/*                                                                            */
/* Diese Routine fuegt c1..c2 als zusaetzliche Werte in diejenige Einzelmenge */
/* ein, die durch parsed_index vorgegeben ist.                                */
/* c1 und/oder c2 koennen Konstanten oder CONST_MACROs sein.                  */
/*                                                                            */
/* Sofern c1 und c2 beides Konstanten sind und die aktuelle Einzelmenge kein  */
/* SET_MACRO enthaelt, werden die Werte sofort eingefuegt.                    */
/* Anderenfalls wird ein Kommando erzeugt, das die Werte spaeter einfuegt.    */
/******************************************************************************/

void add_range( c_val_type c1, c_val_type c2 )
  {
    if ( (!add_with_cmd) && c1.is_value && c2.is_value )
      add_values( &(parsed_range[parsed_index]), 
                  (int) c1.ptr, (int) c2.ptr, True, 0, NULL );
    else
      { Add_Cmd(   (parser_start==-'R') ? Place_AddValues :
                 ( (parser_start==-'G') ? Trans_AddValues : Foreach_AddValues ),
                 c1.ptr, (Cardinal32) c2.ptr, parsed_index )->p4 
          = parsed_comnr + 
          ( ( ((c2.is_value)?2:0) | ((c1.is_value)?1:0) ) * MaxAnzComVars );
      }
  }


void vterror(char *s) /* wird von vtparse bei einem Fehler aufgerufen */
  {
    Msg( FAT, "%s in Guard/Set while reading \"%s\"",s,vttext);
  }
%}

%union {
  char       sym[MAX_IDENT_LENGTH];
  Integer    wert;
  void      *ptr;
  c_val_type c_val;
}

%token  GUARD   /* leeres Token; signalisiert, dass ein        Guard folgt */
%token  RANGE   /* leeres Token; signalisiert, dass ein Kreuzprodukt folgt */
%token  FOREACH /* leeres Token; signalisiert, dass ein    Intervall folgt */

%token  <ptr>  SET_MACRO      /* Makro fuer einen Wertebereich          */ 
%token  <ptr>  CONST_MACRO    /* Makro fuer eine Konstante              */
%token         FOREACH_MACRO  /* Schluesselwort FOREACH                 */
%token  <ptr>  REPLACE_MACRO  /* Makro fuer  Ersetzung einer Transition */
%token  <wert> MARK_MACRO     /* Makro fuer Markierung einer Transition */

%token  PRJ
%token  CROSS
%token  UNION

%token  NOT
%token  AND
%token  OR
%token  TRUE
%token  FALSE
%token  TOKEN
%token  <wert> NUMBER
%token  <sym> IDENT
%token  GE
%token  LE


%left           OR
%right          AND /* don't change to left - TreeToGuard needs right-order */
%right          NOT
%nonassoc        '=' '#' '<' '>' Less Greater GE LE
%left           '+' '-'
%left           '*' '/' '%'
%left           Unaryminus

%type <wert>    expr valexpr simple
%type <c_val>   const

%start          start

%%

start      : GUARD   
               { 
                 foreach_counter = 0;
               } 
             full_expr
               {
                 /* Falls FOREACH-Makros enthalten sind, wird ein Kommando 
                    erzeugt, um diese spaeter zu expandieren. */
                 if ( foreach_counter )
                   Add_Cmd( Guard_Expand, NULL, 0, 0 );
               }
           | RANGE   mult_range
           | FOREACH range
	;

full_expr  : expr {}
           | mult_def ':' expr
           | REPLACE_MACRO expr
               {
                 Add_NetCmd( Trans_Replace, $1, 0, 0 );
               }
           | MARK_MACRO expr
               {
                 Add_Cmd( Trans_Mark, NULL, $1, 0 );
               }
	;

mult_def   : def
           | def AND mult_def
	;

def        : IDENT 
               {
                 parsed_comnr = StrToCom( $1, &(parsed_trans->vars), 
                                          parsed_trans );
                 parsed_range = global_range; /* Kreuzprodukt initialisieren */
                 parsed_index = 0; /* mit der ersten Einzelmenge beginnen */
               }
             mult_range
               {
                 AddRangeToDef( parsed_comnr );

                 /* Falls gerade eine eindimensionale Variable definiert wurde,
                    muss in allen Makros der Definition die Projektionsnummer 
                    korrigiert werden. */
                 if ( parsed_index == 1 )
                   CorrectIndices( parsed_comnr );
               }
	;

mult_range : range CROSS mult_range
           | range
	;

range      : '{' 
               { if ( parsed_index >= MAX_DIM )
                   Msg( FAT, "Range exceeds maximum dimension !" );
                 else
                   {
                     /* Einzelmenge wird initialisiert */
                     parsed_range[parsed_index] = MYmalloc(sizeof(Integer));
                     parsed_range[parsed_index][0] = 1;
                   }

                 /* In Einzelmengen ohne SET_MACRO koennen konstante Werte
                    direkt eingefuegt werden. */
                 add_with_cmd = False;
               }
             elements '}'
               { 
                 parsed_index++; /* naechste Einzelmenge parsen */
               }
           | SET_MACRO /* SET_MACROs muessen ganz vorne stehen. Dahinter 
                          koennen ueber Vereinigungsoperator noch Teilbereiche 
                          mit Konstanten und CONST_MACROs folgen */
               {
                 /* Markiert die aktuelle Einzelmenge als noch nicht gesetzt 
                    und erzeugt ein Kommando, um die Einzelmenge spaeter 
                    mit den Werten aus SET_MACRO zu initialisieren. */
                 parsed_range[parsed_index] = EMPTY_SET;
                 Add_Cmd( (parser_start==-'R')?Place_SetRange: 
                         ((parser_start==-'G')?Trans_SetRange:Foreach_SetRange),
                          $1, parsed_comnr, parsed_index );

                 /* In Einzelmengen mit SET_MACRO muessen zusaetzliche Teil-
                    bereiche spaeter mit einem Kommando hinzugefuegt werden. */
                 add_with_cmd = True;
               }
             extension
               {
                 parsed_index++; /* naechste Einzelmenge parsen */
               }
	;

elements   : values ',' elements
           | values
	;

extension  : 
           | UNION '{' elements '}' extension
	;

values     : const
               { add_range( $1,$1 );
               }
           | const '.' '.' const
               { add_range( $1,$4 );
               }
	;

const      :       NUMBER { $$.ptr = (Integer*)(int)  $1;  $$.is_value = True; }
           |   '-' NUMBER { if (-$2 < MIN_NEG)   
                              Msg( FAT, "Constant out of range !" );
                            $$.ptr = (Integer*)(int) -$2;  $$.is_value = True; }
           |       TRUE   { $$.ptr = (Integer*)(int)  TT;  $$.is_value = True; }
           |       FALSE  { $$.ptr = (Integer*)(int)  FF;  $$.is_value = True; }
           |       TOKEN  { $$.ptr = (Integer*)(int) DOT;  $$.is_value = True; }
           | CONST_MACRO  { $$.ptr =                  $1;  $$.is_value = False;}
	;

expr       : '(' expr ')'
               { $$ = add_node( N_KLAMM, $2, 0 );
               }
           | IDENT
               { $$ = EP_Add_ident( $1, -1 );
               }
           | valexpr
               { $$ = $1;
               }
           | FOREACH_MACRO 
               {
                 if ( foreach_expr )
                   Msg( FAT, "Nested FOREACH-Macros are not allowed !" );
                 else
                   {
                     /* Kommando erzeugen, das den Wertebereich des FOREACHs
                        initialisiert. */
                     Add_Cmd( Foreach_Init, NULL, 0, 0 );

                     /* Position des Foreach_Init-Kommandos speichern. */
                     CmdEntries = GetCmdEntries()-1;

                     /* Initialisierung zum Parsen eines Wertebereichs */
                     parsed_range = global_range;
                     parsed_index = 0;

                     /* Sorgt dafuer, dass add_values Kommandos fuer den
                        Wertebereich des FOREACHs erzeugt. */
                     parser_start = -'F';
                   }
               }
             range ':' 
               {
                 /* Parameter 1 des Foreach_Init-Kommandos setzen */
                 GetFirstCmd()[CmdEntries].p1 = *global_range;

                 /* bisherige Makroanzahl zwischenspeichern */
                 CmdEntries   = GetCmdEntries();

                 /* bisherige Knotenanzahl zwischenspeichern */ 
                 anz_nodes    = expr_tree->op1;

                 /* wieder auf den alten Wert zuruecksetzen */
                 parser_start = -'G';

                 /* Sorgt dafuer, dass geschachtelte FOREACH-Makros abgefangen
                    werden koennen. */
                 foreach_expr = True;
               }
             '(' expr ')'
               {
                 foreach_expr = False;

                 /* Anzahl der Makros in der FOREACH-expr ermitteln */
                 CmdEntries   = GetCmdEntries() - CmdEntries;

                 /* Knotenanzahl der FOREACH-expr ermitteln */
                 anz_nodes    = expr_tree->op1 - anz_nodes;

                 /* Foreach-Kommando erzeugen, das sich auf den N_MACRO-Knoten
                    mit der eindeutigen Nummer foreach_counter bezieht. */
                 Add_Cmd( Guard_AddForeach, ExtractLastCmds(CmdEntries), 
                          CmdEntries, foreach_counter )
                   ->p4 = anz_nodes;

                 /* Knoten in den Baum einfuegen, der mit der Nummer 
                    foreach_counter markiert ist. 
                    (An p1 des Knotens haengt die FOREACH-expr.)       */
                 $$ = add_node( N_MACRO, $7, foreach_counter++ );
                 if ( foreach_counter > MAX_FOREACHS )
                   Msg( FAT, "Too many FOREACH-macros in a single guard !" );
               }
	;

valexpr    : simple        { $$ = $1; }
           | expr OR  expr { $$ = add_node( N_OR  , $1, $3 ); }
           | expr AND expr { $$ = add_node( N_AND , $1, $3 ); }        
           |      NOT expr { $$ = add_node( N_NOT , $2,  0 ); }
           |      '!' expr %prec NOT        { $$ = add_node( N_NOT , $2,  0 ); }
           | expr '=' expr { $$ = add_node( N_EQ  , $1, $3 ); }
           | expr '#' expr { $$ = add_node( N_UNEQ, $1, $3 ); }
           | expr '<' expr %prec Less       { $$ = add_node( N_LT  , $1, $3 ); }
           | expr '>' expr %prec Greater    { $$ = add_node( N_GT  , $1, $3 ); }
           | expr GE  expr { $$ = add_node( N_GE  , $1, $3 ); }
           | expr LE  expr { $$ = add_node( N_LE  , $1, $3 ); }
           | expr '+' expr { $$ = add_node( N_PLUS, $1, $3 ); }
           | expr '-' expr { $$ = add_node( N_MIN , $1, $3 ); }
           | expr '*' expr { $$ = add_node( N_MULT, $1, $3 ); }
           | expr '/' expr { $$ = add_node( N_DIV , $1, $3 ); }
           | expr '%' expr { $$ = add_node( N_MOD , $1, $3 ); }
           |      '-' expr %prec Unaryminus { $$ = add_node( N_SIGN, $2,  0 ); }
	;

simple     : PRJ NUMBER '(' IDENT ')'
               { 
                 if ( $2 >= MAX_DIM )
                   Msg(FAT, "Index of projection exceeds maximum dimension !");
                 $$ = EP_Add_ident( $4, $2 );
               }
           | NUMBER
               { $$ = add_node( N_CONST , $1, 0 );
               }
           | TRUE
               { $$ = add_node( N_CONST , TT, 0 );
               }
           | FALSE
               { $$ = add_node( N_CONST , FF, 0 );
               }
           | TOKEN
               { $$ = add_node( N_CONST , DOT, 0 );
               }
           | CONST_MACRO
               { 
                 /* Knoten mit Markierung fuer ein SetConst-Kommando erzeugen */
                 $$ = add_node( N_CONST, 0, ++index_counter );

                 /* Zugehoeriges SetConst-Kommando erzeugen */
                 Add_Cmd( Guard_SetConst, $1, index_counter, $$ );
               }
	;
%%

#include "ExprRegex.c"

