/******************************************************************************/
/* ExprRegex.l                                                                */
/*                                                                            */
/* FLEX-Scanner fuer den Parser ExprParse                                     */
/*                                                                            */
/* Martin Ackermann, Sommer 1995                                              */
/* Martin Ackermann, Winter 1996/97                                           */
/*   - Makros hinzugefuegt                                                    */
/******************************************************************************/

%option noyywrap
%option nounput

%{
#include <string.h>
#include "f_parse.h"
#include "macros.h"

const char *vtstr;
%}

delim   [ \t\n]
ws      {delim}+
digit   [0-9]
letter  [A-Za-z_@]
macro   "$"{letter}({letter}|{digit})*"$"
name    ({letter}|{macro})({letter}|{digit}|{macro})*
ident   ("~"|"-")?"'"?{name}(""|"??"|"?"|"!!"|"!"|"'")
number  {digit}+

%%
                { /* Dieser Code wird vor jedem Aufruf von yylex ausgefuehrt.

                     Falls parser_start positiv ist, wurden noch keine Token
                     geliefert. In diesem Fall wird ein leeres Token geliefert,
                     um einen der drei in ExprParse enthaltenen Parser zu
                     starten.

                     Der Betrag von parser_start gibt den laufenden bzw. zu
                     startenden Parser an. Das Vorzeichen von parser_start
                     zeigt an, ob der Parser schon gestartet wurde.           */

                  if ( parser_start>0 ) {
                    parser_start = -parser_start;
                    if ( parser_start == -'R' )
                      return RANGE;
                    else if (parser_start == -'G' )
                      return GUARD;
                    else
                      return FOREACH;
                  }
                }

"|"             { return OR; }
"&"             { return AND; }
"X"             { return CROSS; }
"U"             { return UNION; }
dot             { return TOKEN; }
tt              { return TRUE; }
ff              { return FALSE; }
pr/{number}     { return PRJ; }
"<="            { return LE; }
">="            { return GE; }
{number}        { int py; 
                  sscanf(yytext,"%d",&py);
                  if (py > MAX_POS)
                    Msg( FAT, "Constant out of range !" );
                  yylval.wert = py;
                  return NUMBER;} 

"$DECL_SET$"/.* { yylval.ptr = &macro_decl_set; return SET_MACRO; }
"$ID_SET$"/.*   { yylval.ptr = &macro_id_set  ; return SET_MACRO; }
"$NEW_IDS$"/.*  { yylval.ptr = &macro_new_ids ; return SET_MACRO; }
"$SENDERS$"/.*	{ yylval.ptr = &macro_senders ; return SET_MACRO; }
"$SENDERS2$"/.*	{ yylval.ptr = &macro_senders2; return SET_MACRO; }
"$SIGNALS$"/.*	{ yylval.ptr = &macro_signals ; return SET_MACRO; }

"$MAX$"         { yylval.ptr = &macro_max     ; return CONST_MACRO; }
"$CAPACITY$"    { yylval.ptr = &macro_capacity; return CONST_MACRO; }
"$CAPACITPLUSEINS$"    { yylval.ptr = &macro_capacity_plus_eins; return CONST_MACRO; }
"$INIT$"        { yylval.ptr = &macro_init    ; return CONST_MACRO; }
"$INDEX$"       { yylval.ptr = &macro_index   ; return CONST_MACRO; }
"$ELEMENT$"     { yylval.ptr = &macro_element ; return CONST_MACRO; }
"$BASEID$"	{ yylval.ptr = &macro_baseid  ; return CONST_MACRO; }

"$FOREACH$"     { return FOREACH_MACRO; }
"$BODY$"        { yylval.ptr  = &macro_body    ; return REPLACE_MACRO; }

"$PAR_PRE$"     { yylval.wert = MARK_PAR_PRE   ; return MARK_MACRO; }
"$PAR_POST$"    { yylval.wert = MARK_PAR_POST  ; return MARK_MACRO; }
"$CALL_PAR$"    { yylval.wert = MARK_CALL_PAR  ; return MARK_MACRO; }
"$RETURN_PAR$"  { yylval.wert = MARK_RETURN_PAR; return MARK_MACRO; }

{ident}         { strcpy(yylval.sym, yytext); 
                  return IDENT;}
{ws}            { /* Nix */ }
[-=#<>+*/%:{},.()!?'] { return yytext[0]; }
%%
