/******************************************************************************/
/* HLSetRegex.l                                                               */
/*                                                                            */
/* FLEX-Scanner zum Einlesen von Multimengen von Aktionstermen                */
/*                                                                            */
/* Martin Ackermann, Sommer 1995                                              */
/* Martin Ackermann, Winter 1996/97                                           */
/*   - Makros hinzugefuegt                                                    */
/******************************************************************************/

%option noyywrap
%option nounput

%{
#include <unistd.h>
#include <string.h>
#include "f_parse.h"

#ifndef NO_FLEX_HACKS
void* (*hldummy)(void*,yy_size_t) = yy_flex_realloc;
#endif

const char *hlstr;

%}

delim   [ \t\n]
ws      {delim}*
digit   [0-9]
letter  [A-Za-z_@]
macro   "$"{letter}({letter}|{digit})*"$"
name    ({letter}|{macro})({letter}|{digit}|{macro})*
ident   ("~"|"-")?"'"?{name}(""|"??"|"?"|"!!"|"!"|"'")
number  "-"?[1-9]{digit}*|0|"dot"|"tt"|"ff"
single	{ident}|{number}
actname "^"?{name}("@"{name})?(""|"??"|"?"|"!!"|"!")
action  {actname}(""|"("{ws}{single}({ws},{ws}{single})*{ws}")")
set     ""|({ws}{action}({ws},{ws}{action})*{ws})

%%
{set}           { /* Wenn parsed_length den Wert -1 hat, war der Scanner
                     darauf eingestellt, zu testen, ob das Anfangsstueck der 
                     Eingabe einer gueltigen Multimenge von Aktionstermen 
                     entspricht.

                     Falls ein solches Anfangsstueck erkannt wurde, setzt
                     der Scanner parsed_length auf die Laenge des Anfangs-
                     stuecks und liefert 1. Falls kein Anfangsstueck erkannt
                     wurde, hat der Scanner gar nicht diese Regel betreten, 
                     sondern sofort einen Wert ungleich 1 geliefert. 

                     Wenn parsed_length ungleich -1 ist, so soll der Scanner
                     die Eingabe in ihre Bestandteile zerlegen. Nach dem REJECT
                     wird dann eine der nachfolgenden Regeln abgearbeitet. */

                     if ( parsed_length == -1 )
                      {
                           parsed_length = strlen( yytext );
                           yyless(0); 
                           return 1;
                      }
                  else
                      REJECT;
                }
{number}	{ return 7; }
{actname}       { return 2; }
{ident}         { return 3; }          
"("             { return 4; }
")"             { return 5; }
","             { return 6; }
{ws}            { /* Nix */ }
 
%%
