/******************************************************************************/
/* VMSetRegex.l                                                               */
/*                                                                            */
/* FLEX-Scanner zum Einlesen von Kantenbeschriftungen                         */
/*                                                                            */
/* Martin Ackermann, Sommer 1995                                              */
/* Martin Ackermann, Winter 1996/97                                           */
/*   - Makros, Tupel und Konstanten hinzugefuegt                              */
/******************************************************************************/

%option noyywrap
%option nounput

%{
#include <unistd.h>
#include <string.h>
#include "f_parse.h"
#include "macros.h"

#ifndef NO_FLEX_HACKS
void* (*vmdummy)(void*,yy_size_t) = yy_flex_realloc;
#endif

const char *vmstr;

union { Integer wert; void *ptr; } vmlval;

%}

ws      [[:space:]]*
letter  [[:alpha:]_@]
macro   "$"{letter}([[:alnum:]_@])*"$"
name    ({letter}|{macro})([[:alnum:]_@]|{macro})*
ident   [~-]?"'"?{name}("??"|"?"|"!!"|"!"|"'")?
number  "-"?[1-9][[:digit:]]*|0|"$MAX$"|"$CAPACITY$"|"$CAPACITPLUSEINS$"|"$INIT$"|"$INDEX$"|"$ELEMENT$"|"$BASEID$"
single  {ident}|{number}
tupel   "("{ws}{single}({ws},{ws}{single})+{ws}")"
simple  {single}|{tupel}
foreach "$FOREACH$"[^:]*":"{ws}{simple}
elem    {simple}|{foreach}
set     {ws}{elem}({ws},{ws}{elem})*{ws}

%x FE

%%
{set}           { /* Wenn parsed_length den Wert -1 hat, war der Scanner
                     darauf eingestellt, zu testen, ob das Anfangsstueck der 
                     Eingabe einer gueltigen Kantenbeschriftung entspricht.

                     Falls ein solches Anfangsstueck erkannt wurde, setzt
                     der Scanner parsed_length auf die Laenge des Anfangs-
                     stuecks und liefert 1. Falls kein Anfangsstueck erkannt
                     wurde, hat der Scanner gar nicht diese Regel betreten, 
                     sondern sofort einen Wert ungleich 1 geliefert. 

                     Wenn parsed_length ungleich -1 ist, so soll der Scanner
                     die Eingabe in ihre Bestandteile zerlegen. Nach dem REJECT
                     wird dann eine der nachfolgenden Regeln abgearbeitet. */

                  if ( parsed_length == -1 )
                      {
                           parsed_length = strlen( yytext );
                           yyless(0); 
                           return 1;
                      }
                  else
                      REJECT;
                }
"$MAX$"         { vmlval.ptr = &macro_max;      return 6; }
"$CAPACITY$"    { vmlval.ptr = &macro_capacity; return 6; }
"$CAPACITPLUSEINS$"    { vmlval.ptr = &macro_capacity_plus_eins; return 6; }
"$INIT$"        { vmlval.ptr = &macro_init;     return 6; }
"$INDEX$"       { vmlval.ptr = &macro_index;    return 6; }
"$ELEMENT$"     { vmlval.ptr = &macro_element;  return 6; }
"$BASEID$"	{ vmlval.ptr = &macro_baseid;   return 6; }
{number}        { vmlval.wert = atoi( vmtext );
                  if (vmlval.wert < MIN_NEG || vmlval.wert > MAX_POS)
                    Msg( FAT, "Constant in arc label out of range !" );
                                   vmlval.wert += V_CONST_NULL; return 2;}
dot             { vmlval.wert =            DOT +  V_CONST_NULL; return 2;}
tt              { vmlval.wert =             TT +  V_CONST_NULL; return 2;}
ff              { vmlval.wert =             FF +  V_CONST_NULL; return 2;}

  /* Nachdem das Schluesselwort $FOREACH$ erkannt ist, soll als naechstes der */
  /* komplette FOREACH-Wertebereich mit dem Token 8 abgedeckt werden.         */
  /* Der anschliessende Doppelpunkt wird ueberlesen.                          */

"$FOREACH$"     { BEGIN FE; return 7; }
<FE>[^:]*       { return 8; }
<FE>":"         { BEGIN INITIAL; }

{ident}         { return 3; }
"("             { return 4; }
")"             { return 5; }

","             { /* Nix */ }

{ws}            { /* Nix */ }
 
%%
