/*****************************************************************************/
/* ANNOTAT.C                                                                 */
/*                                                                           */
/* Martin Ackermann, Fruehjahr 95                                            */
/* Martin Ackermann, Winter 1996/97                                          */
/*   - Tupel, Konstanten und lokale Umbenennung hinzugefuegt                 */
/* Stefan Schwoon, Sommer 98						     */
/*   - var_safereal-Modus hinzugefuegt					     */
/*****************************************************************************/

#include <string.h>
#include "mnet.h"
#include "tables.h"


/****************************************************************************/
/* Haengt an string den Namen der Kommunikationsvariablen nr an.            */
/*                                                                          */
/* var_style == VAR_REAL: nur der real_name wird ausgegeben (s. Option -n)  */
/* var_style == VAR_COM : z.B. fuer Variable 17 wird c17 ausgegeben...      */
/* var_style == VAR_BOTH: an den real_name wird @nr angehaengt, z.B. @17    */
/*                       (Damit wird die Umbenennung bei allgemeinen Netzen */
/*                        beruecksichtigt.)                                 */
/* var_style == VAR_SAFEREAL: wie VAR_REAL, es wird jedoch ueberprueft,     */
/*			  ob Fehler auftreten koennen, weil zwei Variablen  */
/*			  im Bereich einer Transition denselben Namen	    */
/*			  haben, und nur dann ein Index angehaengt	    */
/*                                                                          */
/* Falls die Variable mit dem Zeichen LOCAL_RENAMING beginnt, so wird sie   */
/* auch im Falle VAR_REAL mit einer Nummer versehen. Genauer gesagt wird    */
/* fuer das i-te Vorkommen von string in *v_ptr der Postfix @i angehaengt.  */
/****************************************************************************/

void StrCatVar( ComVar nr, Variables v_ptr, char *string )
  { 
    if ( nr < V_CONST_OFFSET )
      { /* Variablenname */

        bool     add_prime = False;
        Cardinal str_len   = strlen( string );

#ifdef PRINT_FLAG_FOR_DEBUGGING_PURPOSES
        if ( v_ptr->ComArray[nr].flag )
          str_len += sprintf( string+str_len, "F%d", v_ptr->ComArray[nr].flag );
#endif

        if ( var_style != VAR_COM )
          {
            VariableToStr( v_ptr->ComArray[nr].var, string+str_len );
          }
        if ( var_style == VAR_BOTH || var_style == VAR_SAFEREAL ||
             ((v_ptr->ComArray[nr].flag & VAR_RENAME) && var_style==VAR_REAL) ) 
          {
            str_len = strlen(string);
            if (str_len>0 && string[str_len-1] == '\'')
              { /* der Name ist postprimed -> "@nr" vor dem prime eingefuegen */
                string[str_len-1] = 0;
                add_prime = True;
              }
          }
        if (var_style == VAR_COM)  strcat( string, "c" );
        if (var_style != VAR_REAL && var_style != VAR_SAFEREAL)
          sprintf( string+strlen(string), "@%d", nr );
        if ( var_style == VAR_SAFEREAL ||
	     ((v_ptr->ComArray[nr].flag & VAR_RENAME) && var_style==VAR_REAL) )
          {
            /* Nummer des Vorkommens von string in *v_ptr ermitteln */
            int i, counter = 0, allcntr = 0;

            for ( i=v_ptr->ComEntries-1; i>=0; --i )
              if ( v_ptr->ComArray[i].var == v_ptr->ComArray[nr].var )
              {
		++allcntr;
		counter+=(i>=nr);
	      }
            /* falls Nummer des Vorkommens > 0 => anhaengen */
            if ( allcntr > 1 )
              sprintf( string+strlen(string), "@%d", counter );
          }
        if (add_prime)
          strcat( string, "'" );
      }
    else if ( nr < V_TUPEL_OFFSET )
      { /* Konstante */
        Integer wert;
        wert = nr - V_CONST_NULL;
        if ( wert > MAX_POS )
          switch ( wert )
            {
              case  FF: strcat( string, "ff"  ); break;
              case  TT: strcat( string, "tt"  ); break;
              case DOT: strcat( string, "dot" ); break;
            }
        else
          sprintf( string+strlen(string), "%d", wert );
      }
    else
      { /* Tupel */

        Var *v = TplToArray( nr - V_TUPEL_OFFSET );
        Cardinal i;
        
        strcat( string, "(" );
        for ( i=1; i<*v; i++ )
          {
            StrCatVar( v[i], v_ptr, string );
            strcat( string, "," );
          }
        strcpy( string+strlen(string)-1, ")" );
      }      
  }
