/* block.h                                         */
/* Include-Datei fuer die Behandlung von Bloecken  */
/* im Uebersetzer von B(PN) nach Highlevel-Netzen */
/* Burkhard Bieber 1996                            */
/* last-update: 27.02.96                           */

#ifndef __BLOCK_H__
#define __BLOCK_H__

  /************/
  /* includes */
  /************/

#include "mnet.h"
#include "frontend.h"

  /***********/
  /* defines */
  /***********/

  /* Tags fuer Named-Blocks */
#define BLOCK_PARALLEL 1  /* Block enthaelt Paralleloperator */

#define BLOCK_CHUNK_CHILD_MAX  5

  /*********/
  /* Typen */
  /*********/

  /* Zeiger auf eine Liste von Zeigern */
typedef BlockPtr *BlockTablePtr;

  /***********/
  /* externs */
  /***********/

  /* Anzahl der maximal im BlockTable abzulegenden Bloecke; dieser Wert wird */
  /* dynamisch an die tatsaechlich benoetigte Groesse angepasst und sollte   */
  /* nicht geaendert werden.                                                 */
extern Cardinal32 MaxBlocks;

  /* Aktuelle, tatsaechliche Anzahl vorkommender Bloecke, die im BlockTable */
  /* gespeichert sind.                                                      */
extern Cardinal32 BlockCounter;

  /*********************************/
  /* Prototypen fuer BlockStruct's */
  /*********************************/

  /* Den BlockTable initialisieren */
void InitBlockTable(void);

  /* Den BlockTable wegschmeissen. */
void RemoveBlockTable(void);

  /* Den Block in den BlockTable aufnehmen. Aufruf aus dem Parser. */
void add_block(SymTabEntry *entry, BlockNr *bl, RefNr ref, Cardinal tags);

  /* Einen Block ohne eigenen Namen in den BlockTable aufnehmen. */
  /* Aufruf aus dem Parser heraus.                               */
void add_unnamed_block(Integer blknum, BlockNr *bl, RefNr ref, Cardinal tags);

  /* Stellen werden mit der Blocknummer markiert */
void M_AddAllPlacesBlock(M_Net net, BlockNr bl);

  /* Transitionen werden mit der Blocknummer markiert */
void M_AddAllTransBlock(M_Net net, BlockNr bl);

  /* Phantomtransitionen werden mit der Blocknummer markiert */
void M_AddAllPTransBlock(M_Net net, BlockNr bl);

  /* Eine Funktion zum Kopieren der Blockverweise aus der Stelle src    */
  /* in die Stelle dest. Dabei wird der Speicher fuer die Blockverweise */
  /* in der Zielstelle alloziert.                                       */
void M_CopyPlaceBlockEntries(M_Place dest, M_Place src);

  /* Eine Funktion zum Kopieren der Blockverweise aus der Transition src    */
  /* in die Transition dest. Dabei wird der Speicher fuer die Blockverweise */
  /* in der Zieltransition alloziert.                                       */
void M_CopyTransBlockEntries(M_Transition dest, M_Transition src);

  /***************************************/
  /* Prototypen fuer M_Block's und Netze */
  /***************************************/

  /* Blockspezifische Datenelemente eines Netzes initialisieren. */
void M_InitBlockInNet(M_Net net);

  /* Die Liste der Bloecke aus dem Netz wegschmeissen, NICHT BlockTable. */
void M_RemoveBlocksFromNet(M_Net net);

  /* Blockinformationen von net_src nach net_dest kopieren. */
void M_CopyBlockInNets(M_Net net_dest, M_Net net_src);

  /* Blockinformationen vereinigen; net1 = net1 vereinigt net2 */
void M_JointBlockUnion(M_Net net1, M_Net net2);

  /* Einen Block an das bisherige Netz anhaengen. */
void M_AppendBlock(M_Net net, BlockNr bn);

  /* Die Liste der Bloecke zu einem Netzelement in die Datei schreiben. */
void PrintBlocks(FILE *ofp, M_Net net, Cardinal numblocks, BlockNr *blocks);

  /* Falls gewuenscht (NO_BLOCK_OUTPUT-Flag auf False), dann werden die */
  /* im Netz verwendeten Bloecke in die Datei im PEP-Format ausgegeben. */
  /* Die Datei ofp muss bereits zum Schreiben offen sein.               */
void PrintNet_Blocks_PEP(M_Net net, FILE *ofp, bool    in_ref_file);

  /* Bloecke vervollstaendigen */
void M_CreateBlockElements(M_Net net);

  /* Funktion zum Umwandeln des Blockkodes `blk` in den entsprechenden. */
  /* Zeiger auf die Blockstruktur.                                      */
BlockPtr BlockToStruct(BlockNr blk);

#endif  /* #ifndef __BLOCK_H__ */

  /*** Dateiende Block.h ***/

