/*****************************************************************************/
/* bpn.y                                                                     */
/* BISON-source file: parser for b(pn)2                                      */
/*   (if possible, don't use yacc)                                           */
/* 1993 by Burkhard Graves                                                   */
/* 1994 by Lars Jenner                                                       */
/* 1995 by Martin Ackermann                                                  */
/* Burkhard Bieber 1995/96:                                                  */
/*   - Named-Blocks eingebaut, betrifft hauptschlich die "block"-Regel      */
/*   - Behandlung von Channels mit der Kapazitt 0 erweitert                 */
/* last-update: 02.04.96                                                     */
/*****************************************************************************/
%{
#include <string.h>
#include "function.h"
#include "symtab.h"
#include "scanner.h"
#include "compose.h"
#include "box_sub.h"
#include "parser.h"
#include "block.h"

int yyerror(char *msg)
{
  Msg(_E,msg);
  return 1;
}
%}

%union {
  SymTabEntry	*sym;
  Integer	wert;
  M_Net		net;
  decl_type	decl_;
  alt_set_type	alt_set_;
  RefStruct	ref;
}

%token	<ref> BEGIN
%token	<ref> END
%token	VAR
%token  CONST
%token	CHAN
%token  STACK
%token  PROC
%token  RES
%token  REF
%token	OF
%token	INIT
%token	<ref> DO
%token	OD
%token	<ref> REPEAT
%token	<ref> EXIT_terminal
%token  <ref> ENTER
%token	PARALLEL
%token	CHOICE
%token	NOT
%token	AND
%token	OR
%token	TRUE
%token	FALSE
%token	<wert> NUMBER
%token	<sym> IDENT
%token	GE
%token	LE

%token  <ref> ';'
%token  <ref> '<'
%token  <ref> '>'

%left		PARALLEL
%left		CHOICE

%left		';' ','
%nonassoc	'<' '>'
%left		OR
%right		AND /* don't change to left - TreeToGuard needs right-order */
%right		NOT
%nonassoc	'=' '#' Less Greater GE LE
%left		'+' '-'
%left		'*' '/' '%'
%left		Unaryminus

%type <wert>	        const, init
%type <net>		bpn2, block, scope, com
%type <decl_>		decl
%type <alt_set_>	alt_set
%type <wert>            expr, valexpr, simple

%start	start

%%

start	   :   { InitParser();
                 /* yydebug = 1; */
	       }
             bpn2
	       { ExitParser( $2 );
	       }

bpn2       : scope 
               { $$ = $1;
	       }
           | decl
	       { $$ = $1.var_net;
	         M_Remove_net( &$1.exit_net );
		 MSremove( $1.scope_actions );
	       } 

block	   : BEGIN
               {
                 blknum++;
               }
             scope END
               {
		 RefNr r_entry, r_exit;
                 BlockNr bl;

		 r_entry = StructToRef(&$1);
		 r_exit  = StructToRef(&$4);
		 M_SetAllPlaceRefs($3,    ENTRY, r_entry);
                 M_SetAllPlaceRefs($3,    EXIT,  r_exit);
		 M_AddAllTransRefs($3, TR_ENTRY, r_entry);
		 M_AddAllTransRefs($3, TR_EXIT,  r_exit);
		 $$ = $3;
                 add_unnamed_block(&$1, &$4, &bl);
                 M_AddAllPlacesBlock($3, bl);
                 M_AddAllTransBlock($3, bl);
                 M_AddAllPTransBlock($3, bl);
                 M_AppendBlock($3, bl);
                 blknum--;
	       }
           | BEGIN
               {
                 blknum++;
               }
             IDENT scope END IDENT
               {
		 RefNr r_entry, r_exit;
                 BlockNr bl;

		 r_entry = StructToRef(&$1);
		 r_exit  = StructToRef(&$5);
		 M_SetAllPlaceRefs($4,    ENTRY, r_entry);
                 M_SetAllPlaceRefs($4,    EXIT,  r_exit);
		 M_AddAllTransRefs($4, TR_ENTRY, r_entry);
		 M_AddAllTransRefs($4, TR_EXIT,  r_exit);
                 add_block(&$1, $3, $6, &bl);
                 M_AddAllPlacesBlock($4, bl);
                 M_AddAllTransBlock($4, bl);
                 M_AddAllPTransBlock($4, bl);
                 M_AppendBlock($4, bl);
		 $$ = $4;
                 blknum--;
	       }

scope	   : com		
               { $$ = $1;
	       }
	   | decl ';' com
	       { $$ = Scope( $3, $1 );
	       }
           | const_decl ';' com
               { $$ = $3;
	       } 

decl	   : decl ';' decl	
	       { $$ = Sum_Declerators( $1, $3 );
	       }
           | decl ';' const_decl
               { $$ = $1;
	       }
           | const_decl ';' decl
               { $$ = $3;
	       }
	   | VAR init_var ident_list ':' value_set init
	       { Create_IdentBoxes( &($$.var_net), &($$.exit_net), 0, $6, 
				    &($$.scope_actions), CreateVariableBox );
	       }
	   | VAR init_var ident_list ':' CHAN NUMBER OF value_set
	       { Create_IdentBoxes( &($$.var_net), &($$.exit_net), $6, NONE,
				    &($$.scope_actions), CreateChannelBox );
	       }
	   | VAR init_var ident_list ':' STACK NUMBER OF value_set
	       { Create_IdentBoxes( &($$.var_net), &($$.exit_net), $6, NONE,
				    &($$.scope_actions), CreateStackBox );
	       }

const_decl : CONST IDENT '=' const
               { 
		 $2->const_value = $4;
	       }

init_var   : /* empty rule */ 
               { list = NULL;
	       }

value_set  :   {  values = MYmalloc( sizeof( Integer ) );
                 *values = 1;
               }
             '{' range '}'   
               {
               }

range      : sub_range	
               {
	       }
	   | range ',' sub_range  
	       { 
	       }

sub_range  : const		
               { add_values( &values, $1, $1, True );
	       }
	   | const '.' '.' const 
	       { add_values( &values, $1, $4, True );
	       }

ident_list : IDENT	
               { add_ident( $1 );
	       }
	   | IDENT ',' ident_list
	       { add_ident( $1 );
	       }

init	   : /* empty rule */
	       { $$ = NONE;
	       }
	   | INIT const
	       { $$ = $2;
	       }

const      :       NUMBER { $$ =  $1; }   
	   |   '-' NUMBER { $$ = -$2; }
	   |       TRUE   { $$ =  TT; }	   
	   |	   FALSE  { $$ =  FF; }     

com        :   { init_ExpressionParser();
	       }
	     '<' expr '>'
	       { RefStruct ref;
                 ref.start_l = $2.start_l;
                 ref.start_c = $2.start_c;
                 ref.end_l   = $4.end_l;
                 ref.end_c   = $4.end_c;
                 $$ = Create_Expression( StructToRef( &ref ) ); 
	       }
	   | block
	       { $$ = $1;
	       }
	   | com ';' com
	       { $$ = M_Sequence( $1, $3, StructToRef( &$2 ) );
	       }
	   | com PARALLEL com
	       { $$ = M_ConcurrentComposition( $1, $3 );
	       }
	   | com CHOICE com
	       { $$ = M_Choice( $1, $3 );
	       }
	   | DO alt_set OD
	       { $$ = M_Iteration( $2, NULL, StructToRef( &$1 ), 0 );
	       }
	   | DO com ENTER alt_set OD
	       { $$ = M_Iteration( $4, $2, 
				   StructToRef( &$1 ), StructToRef ( &$3 ) );
	       }

alt_set    : com ';' REPEAT
	       { $$.repeat_net = $1;
	         $$.exit_net = M_OnlyPlaces(0);
                 $$.exit_net->Places->numname = 1;
		 M_CopyRef( &$$.last_repeat, &$3 );
		 M_CopyRef( &$$.last_colon,  &$2 );
	       }
	   | com ';' EXIT_terminal
	       { $$.exit_net = $1;
		 $$.repeat_net = M_OnlyPlaces(0);
                 $$.repeat_net->Places->numname = 1;
	       }
	   | alt_set CHOICE alt_set
	       { 
                 alt_set_type *last;
                 if ( $3.repeat_net->Transitions == NULL )
                    last = &$1;
                 else
                    last = &$3;
                 $$.repeat_net = M_Choice( $1.repeat_net, $3.repeat_net );
		 $$.exit_net = M_Choice( $1.exit_net, $3.exit_net );
		 M_CopyRef( &$$.last_repeat, &(last->last_repeat) );
		 M_CopyRef( &$$.last_colon,  &(last->last_colon)  );
	       }
			
expr       : '(' expr ')'	
               { $$ = add_node( N_KLAMM, $2, 0 );
	       }
	   | IDENT 			
	       { $$ = Add_unprimed( $1->s_name );
	       }
	   | valexpr
	       { $$ = $1;
	       }

valexpr    : simple        { $$ = $1; }
	   | expr OR  expr { $$ = add_node( N_OR  , $1, $3 ); }
	   | expr AND expr { $$ = add_node( N_AND , $1, $3 ); }	
	   |	  NOT expr { $$ = add_node( N_NOT , $2,  0 ); }
	   |      '!' expr %prec NOT        
             { $$ = add_node( N_NOT , $2,  0 ); 
             }
	   | expr '=' expr { $$ = add_node( N_EQ  , $1, $3 ); }
	   | expr '#' expr { $$ = add_node( N_UNEQ, $1, $3 ); }
           | expr '<' expr %prec Less       
             { $$ = add_node( N_LT  , $1, $3 ); 
             }
	   | expr '>' expr %prec Greater    
             { $$ = add_node( N_GT  , $1, $3 ); 
             }
	   | expr GE  expr { $$ = add_node( N_GE  , $1, $3 ); }
	   | expr LE  expr { $$ = add_node( N_LE  , $1, $3 ); }
	   | expr '+' expr { $$ = add_node( N_PLUS, $1, $3 ); }
	   | expr '-' expr { $$ = add_node( N_MIN , $1, $3 ); }
	   | expr '*' expr { $$ = add_node( N_MULT, $1, $3 ); }
	   | expr '/' expr { $$ = add_node( N_DIV , $1, $3 ); }
	   | expr '%' expr { $$ = add_node( N_MOD , $1, $3 ); }
	   |	  '-' expr %prec Unaryminus { $$ = add_node( N_SIGN, $2,  0 ); }

simple     : '\'' IDENT			
               { $$ = Add_preprimed( $2->s_name );
               }
	   | IDENT '\''	
               { $$ = Add_postprimed( $1->s_name );
	       }
	   | IDENT '?'  	
               { $$ = Add_Channel( $1->s_name, 0 );
	       }
	   | IDENT '?' '?' 	
               { $$ = Add_Channel( $1->s_name, CHAN_CHECK );
	       }
	   | IDENT '!'	
               { $$ = Add_Channel( $1->s_name, CHAN_WRITE );
	       }
	   | IDENT '!' '!'	
               { $$ = Add_Channel( $1->s_name, CHAN_WRITE | CHAN_CHECK);
	       }
	   | NUMBER		
               { $$ = add_node( N_CONST , $1, 0 );
               }
	   | TRUE
               { $$ = add_node( N_CONST , TT, 0 );
               }
	   | FALSE
               { $$ = add_node( N_CONST , FF, 0 );
               }
%%

