/****************************************************************************/
/* compose.h                                                                */
/*                                                                          */
/* Martin Ackermann, Fruehjahr 95                                           */
/* Burkhard Bieber 1995:                                                    */
/*   - Schoenheitskorrekturen                                               */
/* Martin Ackermann, Winter 1996/97                                         */
/*   - Prozeduren, andere Referenzen, mehr Effizienz, Fehler beseitigt      */
/****************************************************************************/

#ifndef __COMPOSE_H__
#define __COMPOSE_H__

#include "main.h"
#include "mnet.h"
#include "multiset.h"
#include "frontend.h"


/* ACHTUNG: bitweise Belegung !!!! */
#define CHAN_CHECK	1	/* Aktion fuer Full/Empty-Check */
#define CHAN_WRITE	2	/* Aktion fuer Write            */
#define CHAN_NULL       4       /* Aktion fuer Kanal mit Kap. 0 */

/**************************************/
/* Rueckgabestruktur fuer decl-Regeln */
/**************************************/

typedef struct decl_struct	{ M_Net         init_net;
                                  M_Net		var_net; 
				  M_Net		exit_net;
                                  MultiSet      scope_actions;
				} decl_type;

/*****************************************/
/* Rueckgabestruktur fuer alt_set-Regeln */
/*****************************************/

typedef struct alt_set_struct	{ M_Net		repeat_net;
				  M_Net		exit_net;
				} alt_set_type;

/****************************************************************************/
/* Globalen Variablen zum Parsen einer Expression                           */
/****************************************************************************/

extern MultiSet       action_set;	           /* auftretende HL-Aktionen */
extern VariableStruct expr_vars;            /* im Guard vorkommende Variablen */
extern node_ptr       expr_tree;       /* Guard als Baum in einem dynamischen 
					                          Knotenarray */
extern node_ptr       unprimed_guard;                   /* Guard-Ergaenzungen */
extern Integer       *expr_def;          /* Definitionen fuer Guard-Variablen */

/**************/
/* Prozeduren */
/**************/

Cardinal CopyTree( node_ptr old_tree, Cardinal i, node_ptr guard, Cardinal head,
		   bool    expand );
node_ptr TreeToGuard( node_ptr old_tree, bool    expand );

M_Net Scope( M_Net net, decl_type decl );

void HandleNet( M_Net net );

void add_values( Integer* *values, Integer first, Integer last, 
		 bool    do_realloc, Cardinal additional, Integer* *def );

Action *Add_VariableAction( SymTabEntry *s_entry, MultiSet *set, 
			    Variables v_ptr, node_ptr *ext_ptr );
Action *Add_ChannelAction(  SymTabEntry *s_entry, MultiSet *set, 
			    Variables v_ptr, node_ptr *ext_ptr, Cardinal flag );

Integer Add_preprimed(  SymTabEntry *s_entry );
Integer Add_unprimed(   SymTabEntry *s_entry );
Integer Add_postprimed( SymTabEntry *s_entry );
Integer Add_Channel( SymTabEntry *s_entry, Cardinal flag );

Variables Create_empty_Variables( Variables v_ptr, Cardinal ComEntries );

void AddSingleRangeToDef( Integer* *d_ptr, Var com_nr, 
			  Integer pr_nr, Integer *range, bool    resize );

decl_type Sum_Declarators( decl_type d1, decl_type d2 );

void init_ExpressionParser( Variables v_ptr );
void exit_ExpressionParser( void );
Integer add_node( Cardinal typ, Integer op1, Integer op2 );
Integer add_UnprimedGuard( void );
void Modify_Transition( M_Transition tr );

#endif  /* #ifndef __COMPOSE_H__ */

  /*** Dateiende compose.h ***/

