/****************************************************************************/
/* f_parse.c                                                                */
/*                                                                          */
/* Hilfsroutinen zum Einlesen der Beschriftungselemente an M-Netzen         */
/*                                                                          */
/* Martin Ackermann, Fruehjahr 95                                           */
/* Martin Ackermann, Winter 1996/97                                         */
/*   - Makros, Tupel und Konstanten hinzugefuegt                            */
/****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "compose.h"
#include "var_mset.h"
#include "multiset.h"
#include "tables.h"
#include "f_parse.h"
#include "macros.h"


bool         GL_AddActions = False;   /* gibt an, ob geparsete Aktionsterme 
                                 zu spec_actions hinzugefuegt werden sollen   */
Cardinal     index_counter=0; /* markiert die Zuordnung Makros <-> Knoten     */

Integer*     global_range[MAX_DIM];  /* darin werden Definitionen erzeugt     */

MultiSet     spec_actions;    /* hier werden alle Aktionsterme gesammelt, 
                                 die in einem Template vorkommen              */
int          parsed_length;   /* hier legen die Scanner die Laenge des
                                 gescannten Ausdrucks ab                      */
M_Transition parsed_trans;    /* Transition, die die 'Variables' fuer 
                                 'akt_component' enthaelt                     */
Integer*    *parsed_range;    /* hier traegt der ExprParser Kreuzprodukte 
                                 als Wertebereiche von Stellen ein            */
Cardinal     parsed_index;    /* Index der Einzelmenge in 'parsed_range',
                                 die gerade geparst wird                      */
int          parser_start;    /* <0 : es wurden schon Token geliefert
                                 'F': zu Beginn eines Foreach-Parsings
                                 'R': zu Beginn eines Range-Parsings
                                 'G': zu Beginn eines Guard-Parsings          */
                             

extern int   vtparse(void);   /* ExprParser fuer eine komplette Annotation    */

extern int   vmlex(void);     /* Scanner fuer eine Multimenge von Variablen   */
extern char *vmtext;          /* hier legt der Scanner den zuletzt 
                                 gescannten Ausdruck ab                       */

extern int   hllex(void);     /* Scanner f. eine Multimenge von Aktionstermen */
extern char *hltext;          /* hier legt der Scanner den zuletzt 
                                 gescannten Ausdruck ab                       */

extern const char *vtstr;
extern const char *vmstr;
extern const char *hlstr;

/*****************************************************************************/
/* Die Annotation 'string' wird geparst. Dabei werden auftretende Variablen  */
/* in 'tr->vars' gespeichert. Falls die Bedingung der Annotation nur 'tt'    */
/* ist (evtl. hinter einer Definition), so wird dieses 'tt' abgeschnitten.   */
/*                                                                           */
/* RUECKGABE: - die Annotation als Guard mit Teilbaeumen                     */
/*            - Geparste Definitionen werden in tr->def gespeichert.         */
/*****************************************************************************/

node_ptr ParseAnnotation( const char *string, M_Transition tr )
{
  Integer* *def = &(tr->def);

  /* Definitionen initialisieren */
  *def=MYmalloc( 2*sizeof( Integer ) );
  (*def)[0] = 2;
  (*def)[1] = -1;
  
  parsed_trans = tr;
  
  /* Guard initialisieren (Groesse wird in 100er-Schritten angepasst) */
  expr_tree = MYmalloc( 100 * sizeof( node ) );
  expr_tree->typ = N_HEAD;
  expr_tree->op1 = 1;
  expr_tree->op2 = 100;
  
  parser_start = 'G'; /* ExprParse auf Guard-Parsing einstellen */
  vtstr = string;

  vtparse();
 
  /* Nur 'tt' im Baum ? -> leeren Baum zurueckgeben ! */
  if( expr_tree->op1 == 2 && 
      expr_tree[1].typ == N_CONST && expr_tree[1].op1 == TT )
    { 
      expr_tree[0].typ = N_HEAD;
      expr_tree[0].op1 = 0;
      expr_tree[0].op2 = 1; 
      expr_tree = MYrealloc( expr_tree, expr_tree->op2 * sizeof( node ) );
    }
  else
    {
      /* Falls der Baum keine FOREACH-Knoten enthaelt, wird er sofort in
         einen Guard umgewandelt. */

     if ( !GetCmdEntries() || GetNextCmd()[-1].cmd_nr != Guard_Expand )
        expr_tree = TreeToGuard( expr_tree, False );
      else
        {
          /* Fuer Baeume mit FOREACH-Knoten wird die zuvor auf 100 
             aufgerundete Groesse zur tatsaechlichen Groesse korrigiert. 

             (Die Umwandlung in einen Guard wird spaeter nebenbei durch das
              Kommando Guard_Expand erledigt.) */
          expr_tree->op2 = expr_tree->op1;
          expr_tree = MYrealloc( expr_tree, expr_tree->op2 * sizeof( node ) );
        }
    }
  return expr_tree;
}


/*****************************************************************************/
/* Falls 'name' noch nicht in 'vptr' steht, wird 'name' in 'vptr' eingetragen*/
/* und die erzeugte 'com_nr' zurueckgegeben.                                 */
/* Falls 'name' schon      in 'vptr' steht, wird die zugehoerige 'com_nr'    */
/* zurueckgegeben.                                                           */
/*                                                                           */
/* Wenn 'name' Makros enthaelt, so wird zusaetzlich ein Kommando erzeugt,    */
/* das fuer die Transition 'tr' spaeter die Makros expandiert.               */
/*                                                                           */
/* Falls der Name mit dem Zeichen VAR_RENAME_CHAR beginnt, wird fuer die     */
/* Variable das lokale Renaming eingeschaltet.                               */
/* Falls der Name mit dem Zeichen VAR_MINOR_CHAR beginnt, wird die Variable  */
/* so markiert, dass sie bei Namenskonflikten umbenannt wird.                */
/*****************************************************************************/

ComVar StrToCom( const char *name, Variables vptr, M_Transition tr )
{
  Var       var_nr;
  ComVar    com_nr;
  
  var_nr = StrToVariable( (char*) name );
  
  /* ComArray nach var_nr durchsuchen */
  com_nr = 0; 
  while( com_nr < vptr->ComEntries && 
         vptr->ComArray[com_nr].var != var_nr )
    {
      com_nr++;
    }
  
  /* falls var_nr nicht im ComArray -> neu eintragen */
  if ( com_nr == vptr->ComEntries )
    {
      vptr->ComArray = MYrealloc( vptr->ComArray, ++(vptr->ComEntries) 
                                  * sizeof(ComArrayEntry) );
      vptr->ComArray[ com_nr ].var  = var_nr;
      vptr->ComArray[ com_nr ].flag = 0;

      if ( vptr->ComEntries > MaxAnzComVars )
        Msg( FAT, "Transition has too many ComVars !" );

      /* fuer Variablennamen, die Makros enthalten, ein Kommando erzeugen */
      if ( tr && strchr( name, '$' ) )
        {
          Add_Cmd( Trans_ChangeVar, NULL, com_nr, 0 )->component = tr;
        }
    }

  if ( name[0] == VAR_RENAME_CHAR )
    vptr->ComArray[ com_nr ].flag |= VAR_RENAME;
  else if ( name[0] == VAR_MINOR_CHAR )
    vptr->ComArray[ com_nr ].flag |= VAR_MINOR;
  
  return com_nr;
}


/****************************************************************************/
/* Liest die naechsten Scanner-Token ein und bildet daraus einen Tupel.     */
/*                                                                          */
/* Wenn 'direct_insert' schon beim Aufruf False ist, werden alle Tupel-     */
/* Komponenten (ausser den Konstanten) nicht direkt, sondern ueber          */
/* Kommandos eingefuegt.                                                    */
/*                                                                          */
/* Falls fuer 'direct_insert' zunaechst True uebergeben wird, kann der Wert */
/* waehrend des Einlesens noch auf False wechseln, naemlich wenn der Tupel  */
/* ein Makro enthaelt.                                                      */
/* Damit wird an das aufrufende ParseElement signalisiert, dass der Tupel   */
/* nicht konstant ist.                                                      */
/* (Als Nebeneffekt werden alle nachfolgenden Tupel-Komponenten vom Typ     */
/*  Bezeichnern indirekt eingefuegt. Das ist nicht notwendig, stoert aber   */
/*  auch nicht weiter.)                                                     */
/****************************************************************************/

extern union { Integer wert; void *ptr; } vmlval;

static Var *ParseTupel( bool    *direct_insert, M_Transition tr )
{
  int       token;
  Var      *tupel = MYmalloc( sizeof( Var ) ); 
                    /* 1 Element fuer die Laengenangabe */
  Cardinal  CmdEntries = GetCmdEntries(); /* Kommandozahl zwischenspeichern */
  
  *tupel = 1; /* Laengenangabe initialisieren */

  while ( (token = vmlex()) != 5 /* bis zur 'Klammer zu' scannen */ )
    {
      tupel = MYrealloc( tupel, (*tupel + 1) * sizeof( Var ) );
      switch (token)
        {
          case 2: /* Konstante */
              tupel[(*tupel)++] = vmlval.wert;
            break;
          case 3: /* Bezeichner */
              if ( *direct_insert )
                tupel[(*tupel)++] = StrToCom( vmtext, &(tr->vars), tr );
              else
                Add_Cmd( Tupel_SetVar, MYstrdup( vmtext ), (*tupel)++, 0 )
                  ->component = tr;
            break;
          case 6: /* Makro */
              Add_Cmd( Tupel_SetConst, vmlval.ptr, (*tupel)++, 0 );
              *direct_insert = False;
            break;
          default:
            Msg( FAT,"Internal Error: Unknown token type !");
        }
    }

  if ( (*tupel-1) > MAX_DIM )
    Msg( FAT,"Tupel exceeds maximum dimension !");
  
  CmdEntries = GetCmdEntries() - CmdEntries;
  /* Aufgrund der reallocs steht die Adresse des Tupels erst jetzt endgueltig 
     fest. Falls Kommandos fuer den Tupel erzeugt wurden, muss ihr Parameter
     mit der Tupel-Adresse jetzt gesetzt werden. */
  if ( CmdEntries )
    {
      Cmd *c, *last;

      last = GetNextCmd();
      for( c = last - CmdEntries; c < last; c++ )
        {
          if ( c->cmd_nr == Tupel_SetConst || c->cmd_nr == Tupel_SetVar )
            c->p3 = (Cardinal32)tupel;
        }
    }

  return tupel;
}


/****************************************************************************/
/* Fuegt das 'token' in die Kantenbeschriftung 'new' ein.                   */
/*                                                                          */
/* Variablen werden tr->vars eingetragen.                                   */
/*                                                                          */
/* Wenn 'direct_insert' schon beim Aufruf False ist, wird das 'token' nicht */
/* direkt, sondern ueber Kommandos eingefuegt.                              */
/*                                                                          */
/* Falls fuer 'direct_insert' zunaechst True uebergeben wird, kann der Wert */
/* waehrend des Einlesens noch auf False wechseln, naemlich wenn ein Makro, */
/* ein FOREACH-Konstrukt oder ein nicht konstanter Tupel vorliegt.          */
/*                                                                          */
/* Nur wenn nach Einlesen des Tokens 'direct_insert' noch True ist, wird    */
/* das 'token' direkt in 'new' eingefuegt.                                  */
/****************************************************************************/

static void ParseElement( int token, Variable *new, bool    direct_insert,
                          M_Transition tr )
{   
  Var      *tupel;
  Cardinal  CmdEntries;

  switch (token)
    {
      case 2: /* Konstante */
          if ( direct_insert )
            new[0].nr = vmlval.wert;
          else
            Add_Cmd( Arc_AddConst, NULL, vmlval.wert, 0 );
        break;
      case 3: /* Bezeichner */
          if ( direct_insert )
            new[0].nr = StrToCom( vmtext, &(tr->vars), tr );
          else
            Add_Cmd( Arc_AddVar, MYstrdup(vmtext), (Cardinal32)tr, 0 );
        break;
      case 4: /* Klammer auf -> Tupel folgt */
          tupel = ParseTupel( &direct_insert, tr );
          if ( direct_insert )
            new[0].nr = ArrayToTpl( &tupel ) + V_TUPEL_OFFSET;
          else
            Add_Cmd( Arc_AddTupel, tupel, 0, 0 );
        break;
      case 6: /* Makro */
          Add_Cmd( Arc_AddConst, vmlval.ptr, 0, 0 );
          direct_insert = False;
        break;
      case 7: /* FOREACH-Konstrukt */
        {
          Add_Cmd( Foreach_Init, NULL, 0, 0 );
          CmdEntries = GetCmdEntries()-1;
          vmlex(); /* Wertebereich einlesen... */
          StringToRange( vmtext, global_range, 'F' ); /* ...und parsen */

          /* Parameter 1 des Foreach_Init-Kommandos setzen */
          GetFirstCmd()[CmdEntries].p1 = *global_range;

          CmdEntries = GetCmdEntries();
          /* FOREACH-Element parsen und dabei ALLE token (ausser konstante
             Tupel-Komponenten) indirekt ueber Kommandos eintragen */
          ParseElement( vmlex(), new, False, tr );
          CmdEntries = GetCmdEntries() - CmdEntries;

          Add_Cmd( Arc_AddForeach, ExtractLastCmds(CmdEntries), CmdEntries, 0 );
          direct_insert = False;
        }
        break;
      default:
        Msg( FAT,"Internal Error: Unknown token type !");
    }
  if ( direct_insert )
    V_MSsum( V_mmLeaf, new );
}


/****************************************************************************/
/* Zunaechst wird ueberprueft, ob der komplette 'string' eine gueltige      */
/* Multimenge von Variablen ist.                                            */
/* Anschliessend werden die Variablen in ein VarMSet umgewandelt, wobei die */
/* Kommunikationsnummern aus 'tr->vars' ermittelt bzw. in 'tr->vars' neu    */
/* eingetragen werden.                                                      */
/****************************************************************************/

VarMSet StrToVarMSet( const char *string, M_Transition tr )
{
  Variable  new[2];
  int       token;
  
  /* V_mmLeaf initialisieren */
  V_mmLeaf->anzahl = 0;
  new[0].anzahl = 1;
  new[1].anzahl = 0;
  
  vmstr = string;
  parsed_length = -1;           /* Scanner auf Multimengentest einstellen */
  if ( vmlex() == 1 && parsed_length == strlen(string) )
    {   /* Der erste Scanneraufruf ergab, dass string eine gueltige 
           Multimenge ist.                                              */
      while ( (token = vmlex()) != 0 )
        /* Alle weiteren Scanneraufrufe liefern ein Element.  */
        ParseElement( token, new, True, tr );
    }
  else
    {
      Msg( FAT, "Syntax-Error in arclabel \"%s\" !", string );
    }
 
  return V_MScreate_copy( V_mmLeaf, NULL );
}

/* Hilfsroutine fuer StrToMultiSet */
ComVar scan_number(char *wert)
{
	return V_CONST_NULL +
	 (*wert == 'd'? DOT : *wert == 't'? TT : *wert == 'f'? FF : atoi(wert));
}

/****************************************************************************/
/* Zunaechst wird ueberprueft, ob der komplette 'string' eine gueltige      */
/* Multimenge von Aktionstermen ist.                                        */
/* Anschliessend werden die Aktionsterme in ein MultiSet umgewandelt, wobei */
/* die Kommunikationsnummern aus 'tr->vars' ermittelt bzw. in 'tr->vars'    */
/* neu eingetragen werden.                                                  */
/*                                                                          */
/* Falls 'tr' == NULL ist, werden keine Kommunikationsnummern ermittelt     */
/* bzw. erzeugt.                                                            */
/* VORSICHT: So geparste Aktionsterme koennen nicht von VerboseActToStr     */
/*           ausgegeben werden.                                             */
/*                                                                          */
/* Falls das globale Flag GL_AddActions gesetzt ist, werden die gescannten  */
/* Aktionsterme an die globale Multimenge 'spec_actions' angehaengt.        */
/****************************************************************************/

MultiSet StrToMultiSet( const char *string, M_Transition tr )
{
  Action       new[2];
  MultiSet     actions;
  int          pos;
 
  /* mmLeaf initialisieren */
  mmLeaf->anzahl = 0;
  new[0].anzahl = 1;
  new[1].anzahl = 0;
  
  hlstr = string;
  parsed_length = -1;           /* Scanner auf Multimengentest einstellen */
  if ( hllex() == 1 && parsed_length == strlen(string) )
    {   /* Der erste Scanneraufruf ergab, dass string eine gueltige 
           Multimenge ist.                                              */
      while ( hllex() == 2 ) /* Aktionssymbol geparst ? */
        {
          if ( hltext[0] == '^' )
            new[0].nr = StrToAction( hltext+1 ) | 1;
          else
            new[0].nr = StrToAction( hltext );
          pos = strchr( hltext, '$' ) - hltext;
          new[0].arity = 0;
          if ( hllex() == 4 ) /* Klammer oder Komma parsen */
            {   /* falls Klammer auf */
              do
                {   
                  if (hllex() == 7)
		  {
		    if (tr)	/* Konstante */
                      new[0].com[new[0].arity] = scan_number(hltext);
		  }
		  else
                    if ( tr )	/* Variablenname gescannt */
                      new[0].com[ new[0].arity ] =
                        StrToCom( hltext, &(tr->vars), tr );
                  new[0].arity++;
                } 
              while( hllex() == 6 ); /* solange bis Klammer zu */
              hllex(); /* Komma parsen */
            }
          /* Falls das Aktionssymbol ein Makro enthaelt, muss der Aktionsterm
             indirekt eingefuegt werden. */
          if ( pos >= 0 )
            {
              Add_Cmd( Trans_AddAction, MScreate_copy(new), 0, 0 );
            }
          else
            MSsum( mmLeaf, new );
        }
    }
  else
    {
      Msg( FAT, "Syntax-Error in actionset \"%s\" !", string );
    }
  
  actions = MScreate_copy( mmLeaf );
  
  /* GL_AddActions gesetzt -> alle Aktionsterme nach spec_actions kopieren */
  if ( GL_AddActions )
    {
      MSsum( mmLeaf, spec_actions );
      MSremove( spec_actions );
      spec_actions = MScreate_copy( mmLeaf );
    }
  return actions;
}


/****************************************************************************/
/* Falls 's1' != NULL oder 's2' != NULL, wird 's2' an 's1' gehaengt und     */
/* der entzeugte String zurueckgegeben, ansonsten wird NULL geliefert.      */
/****************************************************************************/

char* ConcatStrings( const char *s1, const char *s2 )
{ 
  char     *new;
  Cardinal length = 0;
  
  if ( s1 )
    length += strlen( s1 );
  if ( s2 )
    length += strlen( s2 );
  
  if ( length )
    {
      new = MYmalloc( (length + 2) * sizeof(char) );
      /* +2 fuer den '*' und das abschliessende '\0' */
      if ( s1 )
        strcpy( new, s1 );
      else
        *new = 0;
      if ( s2 )
        {
          if ( new[0] && s2[0] )
            strcat( new, "*" );
          strcat( new, s2 );
        }
      return new;
    }
  else
    return NULL;
}

/****************************************************************************/
/* Funktion, die vom Parser aufgerufen wird, um Identifier an den GuardBaum */
/* (global in expr_tree) anzuhaengen.                                       */
/*                                                                          */
/* Wenn str Makros enthaelt, die fuer ein Template einen konstanten Wert    */
/* haben, wird fuer str sofort eine Kommunikationsnummer in                 */
/* parsed_trans->vars (global) reserviert bzw. ermittelt. Beim Kopieren des */
/* Templates muss dann nur noch parsed_trans->vars mit einem Kommando       */
/* angepasst werden (siehe StrToCom).                                       */
/*                                                                          */
/* Falls str jedoch Makros mit veraenderlichen Werten enthaelt ($INDEX$     */
/* oder $ELEMENT$), so muessen bei der Expansion des zugehoerigen FOREACH-  */
/* Konstrukts MEHRERE verschiedene Eintraege in parsed_trans->vars          */
/* reserviert bzw. ermittelt werden.                                        */
/****************************************************************************/

Integer EP_Add_ident( char* str, Integer prj )
{
  Integer n;

  if ( strstr( str, "$INDEX$" ) || strstr( str, "$ELEMENT$" ) )
    { 
      /* Der Knoten wird mit der eindeutigen Nummer aus 'index_counter'
         versehen, auf die sich dann das zugehoerige Kommando bezieht. */
      n = add_node( N_VAR, -(++index_counter), prj );
      Add_Cmd( Guard_SetVar, MYstrdup( str ), index_counter, n );
    }
  else
    n =add_node( N_VAR, StrToCom( str, &(parsed_trans->vars), parsed_trans ), 
                 prj );
  return n;
}

/****************************************************************************/
/* Der String str wird als Kreuzprodukt geparst und unter *range abgelegt.  */
/*                                                                          */
/* kind_of_cmds kann 'R' oder 'F' sein und gibt an, ob die Makros in str    */
/* als Place-Commands oder als Foreach-Commands uebersetzt werden sollen.   */
/****************************************************************************/

void StringToRange( const char* str, Integer **range, int kind_of_cmds )
{
  parsed_range = range;
  parsed_index = 0;
  
  if (str && *str!=0)
    vtstr = str;
  else
    vtstr = "{dot}";
  parser_start = kind_of_cmds;
  vtparse();
  if ( parsed_index < MAX_DIM )
    parsed_range[ parsed_index ] = NULL;
}


/****************************************************************************/
/* Der Wertebereich in parsed_range (global) wird als Definition fuer die   */
/* Variable com_nr an das Array parsed_def (global) angehaengt.             */
/****************************************************************************/

void AddRangeToDef( Var com_nr )
{
  Cardinal i;

  for( i=0; i<parsed_index; i++ )
    {
      /* Leere Teilbereiche werden spaeter durch ein Kommando hinzugefuegt */
      if ( parsed_range[i] != EMPTY_SET )
        {
          AddSingleRangeToDef( &(parsed_trans->def), com_nr, 
                               (parsed_index > 1)?i:-1, parsed_range[i], True );
          MYfree( parsed_range[i] );
        }
    }
}
