/******************************************************************************/
/* iter.c                                                                     */
/* Burkhard Graves, 15.07.94                                                  */
/* Martin Ackermann, Fruehjahr 95                                             */
/* (modifiziert fuer M-Netze, enter-Netz eingefuegt)                          */
/* Burkhard Bieber:                                                           */
/*   - Schoenheitskorrekturen                                                 */
/* Martin Ackermann, Winter 1996/97                                           */
/*   - Prozeduren, andere Referenzen, mehr Effizienz, Fehler beseitigt        */
/******************************************************************************/

  /************/
  /* includes */
  /************/

#include <stdio.h>
#include "mnet.h"
#include "move.h"
#include "compose.h"
#include "box_sub.h"
#include "multiset.h"
#include "ptrans.h"

  /**************/
  /* Funktionen */
  /**************/

M_Net SmallIteration(alt_set_type alt, M_Net enter)
{
    register M_Place   pl1,pl2,pl3;
             M_Place   pl;
             M_Net     net_a,net_b,net_c;

             M_STEdge  stlast,pstlast;
             M_TSEdge  tslast,ptslast;

             int       pos;
             Cardinal  dim;
             
             VariableStruct vars;

    net_a=enter; /* wird beim Betreten der Schleife einmalig ausgefuehrt */
    net_b=alt.repeat_net;
    net_c=alt.exit_net;

    /*-----------------------------------------------------*/
    /* Koordinaten und Dimension des (zukuenftigen) Netzes */
    /*-----------------------------------------------------*/

    M_CreateTr(net_b,MScreate_empty_set(),NULL,
	       Create_empty_Variables( &vars, 1 ),(net_b->x_pos-=2*RASTER),
	       net_b->y_pos+(net_b->y_dim-RASTER)/2, 0)->flags = TR_REPEAT;
    net_b->x_dim+=2*RASTER;
    M_turn_left(net_b);

    /*--------------------------------------------*/
    /*  entry(net_b) * exit(net_a) * entry(net_c) */
    /*--------------------------------------------*/

    M_PreparePlaces(net_a,EXIT,EXIT);
    M_PreparePlaces(net_c,ENTRY,ENTRY);
    dim=0;
    pl1=net_b->LastPlace;
    while (pl1 && pl1->numname) {
        if (pl1->label==ENTRY) {
            pl2=net_a->LastPlace;
            while (pl2) {
                pl3=net_c->LastPlace;
                while (pl3) {
                    dim+=2*RASTER;

(pl=M_DuplicatePlace(net_b,pl1,&stlast,&tslast,&pstlast,&ptslast))->label=EMPTY;
M_SumPlaceRefs( pl, pl2 );
M_SumPlaceRefs( pl, pl3 );
(tslast=M_allocTSEdge())->label=V_MScreate_singleton( 0 );
tslast->flags = 0;
tslast->prevplace=NULL;
tslast->prevtrans=NULL;
tslast->nexttrans=NULL;
if ((tslast->nextplace=net_b->Transitions->postset)!=NULL)
    tslast->nextplace->prevplace=tslast;
(tslast->place=pl)->preset
=(tslast->trans=net_b->Transitions)->postset=tslast;
/*
if (tslast) {
*/
    if ((tslast->nexttrans=M_CopyPlacePreset(pl2,pl,&tsdummy,False))!=NULL) {
        tslast->nexttrans->prevtrans=tslast;
    }
/*
} else {
    pl->preset=M_CopyPlacePreset(pl2,pl,&tslast,False);
}
*/

if (stlast) {
    if ((stlast->nexttrans=M_CopyPlacePostset(pl3,pl,&stdummy,False))!=NULL) {
        stlast->nexttrans->prevtrans=stlast;
    }
} else {
    pl->postset=M_CopyPlacePostset(pl3,pl,&stlast,False);
}

if (ptslast) {
    if ((ptslast->nexttrans=M_CopyPlacePreset(pl2,pl,&tsdummy,True))!=NULL) {
        ptslast->nexttrans->prevtrans=ptslast;
    }
} else {
    pl->ppreset=M_CopyPlacePreset(pl2,pl,&ptslast,True);
}


if (pstlast) {
    if ((pstlast->nexttrans=M_CopyPlacePostset(pl3,pl,&stdummy,True))!=NULL) {
        pstlast->nexttrans->prevtrans=pstlast;
    }
} else {
    pl->ppostset=M_CopyPlacePostset(pl3,pl,&pstlast,True);
}

                    pl3=pl3->prevplace;
                }
                pl2=pl2->prevplace;
            }
            pl1=M_RemovePlace(net_b,pl1);
        } else {
            if (pl1->label==EXIT) {
                pl1->label=EMPTY;
                (stlast=M_allocSTEdge())->label = V_MScreate_singleton( 0 );
		stlast->flags = 0;
                stlast->prevplace=NULL;
                stlast->prevtrans=NULL;
                stlast->nexttrans=NULL;
                if ((stlast->nextplace=net_b->Transitions->preset)!=NULL)
                    stlast->nextplace->prevplace=stlast;
                (stlast->place=pl1)->postset
                =(stlast->trans=net_b->Transitions)->preset=stlast;
                pl1->y_pos +=RASTER;
            }
            pl1=pl1->prevplace;
        }
    }

    /*-----------------------------------------------*/
    /* Die neuen Stellen (links) zentriert anordnen. */
    /*-----------------------------------------------*/

    dim -= RASTER;
    pos=((int)(net_b->y_dim-(dim))>>1)+net_b->y_pos;
    if (dim>net_b->y_dim) {
        net_b->y_dim=dim;
        net_b->y_pos=pos;
    }
    dim -= RASTER;
    pos+=dim;
    while (pl1) {
        pos=(pl1->y_pos=pos)-2*RASTER;
        pl1->numname=1;
        pl1=pl1->prevplace;
    }

    M_move_hv(net_c,net_b->x_pos-net_c->x_pos
                    +((int)(net_b->x_dim-net_c->x_dim)>>1)
                   ,net_b->y_pos-net_c->y_pos+net_b->y_dim-RASTER);
    if (net_c->x_dim>net_b->x_dim) {
        net_b->x_dim=net_c->x_dim;
        net_b->x_pos=net_c->x_pos;
    }
    net_b->y_dim+=net_c->y_dim-RASTER;
    
    M_move_hv(net_a,net_b->x_pos-net_a->x_pos
                    +((int)(net_b->x_dim-net_a->x_dim)>>1)
                   ,net_b->y_pos-net_a->y_pos-net_a->y_dim+RASTER);
    if (net_a->x_dim>net_b->x_dim) {
        net_b->x_dim=net_a->x_dim;
        net_b->x_pos=net_a->x_pos;
    }
    net_b->y_dim+=net_a->y_dim-RASTER;
    net_b->y_pos=net_a->y_pos;

    return M_DisjointNetUnion(net_b,M_DisjointNetUnion(M_RestoreKillPlaces(net_a,EXIT),M_RestoreKillPlaces(net_c,ENTRY)));
}

/*******/
/* ??? */
/*******/

M_Net BigIteration(alt_set_type alt, M_Net enter)
{
    register M_Place  pl1,pl2,pl3,pl4;
             M_Place  pl;
             M_Place  merke;
             M_Place  entry_pl,exit_pl;
             M_Net    a_net,b_net,c_net,
                      net_a,net_b,net_c;
             M_STEdge stlast,pstlast;
             M_TSEdge tslast,ptslast;
             int      pos;
             int      net_x_pos,net_y_pos;
             int      x_offset,y_offset;
             Cardinal dim;
             Cardinal net_x_dim,net_y_dim;
             Cardinal entry_x_dim,exit_x_dim;

    a_net=enter; /* wird beim Betreten der Schleife einmalig ausgefuehrt */
    net_a=M_CreateNetCopy(a_net); 
    net_b=M_CreateNetCopy(b_net=alt.repeat_net);
    net_c=M_CreateNetCopy(c_net=alt.exit_net);

    /*-----------------------------------------*/
    /* Das nach rechts gedrehte 'net_b' unter   */
    /* das nach links   gedrehte 'b_net' legen. */
    /*-----------------------------------------*/

    M_turn_right(net_b);
    M_turn_left(b_net);
    M_move_hv(net_b,b_net->x_pos-net_b->x_pos,
                    b_net->y_pos-net_b->y_pos+b_net->y_dim+RASTER);

    /*----------------------------------------------------*/
    /* Koordinaten und Dimension des (zukuenftigen) Netzes */
    /*----------------------------------------------------*/

    net_x_pos=b_net->x_pos;
    net_y_pos=b_net->y_pos;
    net_x_dim=b_net->x_dim;
    net_y_dim=b_net->y_dim*2+RASTER;

    /*-------------------------------*/
    /*  entry(a_net) * entry(net_a)  */
    /*-------------------------------*/

    entry_x_dim=0;
    merke=M_PreparePlaces(net_a,EXIT,ENTRY);
    pl1=a_net->LastPlace;
    while (pl1 && pl1->numname) {
        if (pl1->label==ENTRY) {
            pl2=merke;
            while (pl2) {
                entry_x_dim+=2*RASTER;
                if (M_DuplicatePlace(a_net,pl1,&stlast,&tslast,
				     &pstlast,&ptslast)->postset) {
                    if ((stlast->nexttrans
                         =M_CopyPlacePostset(pl2,a_net->Places,
					     &stdummy,False))!=NULL)
                        stlast->nexttrans->prevtrans=stlast;
                } else
                    a_net->Places->postset
                    =M_CopyPlacePostset(pl2,a_net->Places,&stdummy,False);
		if (a_net->Places->ppostset!=NULL) {
		    if ((pstlast->nexttrans
			=M_CopyPlacePostset(pl2,a_net->Places,
					     &stdummy,True))!=NULL)
		        pstlast->nexttrans->prevtrans=pstlast;
		} else 
		    a_net->Places->ppostset
		      =M_CopyPlacePostset(pl2,a_net->Places,&stdummy,True);
                pl2=pl2->prevplace;
            }
            pl1=M_RemovePlace(a_net,pl1);
        } else
            pl1=pl1->prevplace;
    }
    entry_pl=pl1;

    /*-----------------------------*/
    /*  exit(c_net) * exit(net_c)   */
    /*-----------------------------*/

    exit_x_dim=0;
    merke=M_PreparePlaces(net_c,ENTRY,EXIT);
    pl1=c_net->LastPlace;
    while (pl1 && pl1->numname) {
        if (pl1->label==EXIT) {
            pl2=merke;
            while (pl2) {
                exit_x_dim+=2*RASTER;
                if (M_DuplicatePlace(c_net,pl1,&stlast,&tslast,
				     &pstlast,&ptslast)->preset) {
                    if ((tslast->nexttrans
                         =M_CopyPlacePreset(pl2,c_net->Places,
					    &tsdummy,False))!=NULL)
                        tslast->nexttrans->prevtrans=tslast;
                } else
                    c_net->Places->preset
                    =M_CopyPlacePreset(pl2,c_net->Places,&tsdummy,False);
                if (c_net->Places->ppreset!=NULL) {
                    if ((ptslast->nexttrans
                         =M_CopyPlacePreset(pl2,c_net->Places,
					     &tsdummy,True))!=NULL)
                        ptslast->nexttrans->prevtrans=ptslast;
                } else
		  c_net->Places->ppreset
		    =M_CopyPlacePreset(pl2,c_net->Places,&tsdummy,True);
                pl2=pl2->prevplace;
            }
            pl1=M_RemovePlace(c_net,pl1);
        } else
            pl1=pl1->prevplace;
    }
    exit_pl=pl1;

    M_PreparePlaces(a_net,EXIT,EXIT);
    M_PreparePlaces(c_net,ENTRY,ENTRY);
    merke=M_PreparePlaces(net_b,ENTRY,EXIT);

    /*-----------------------------------------------------------*/
    /*  entry(b_net) * exit(net_b) * entry(c_net) * exit(a_net)  */
    /*-----------------------------------------------------------*/

    dim=0;
    pl1=b_net->LastPlace;
    while (pl1 && pl1->numname) {
        if (pl1->label==ENTRY) {
            pl2=merke;
            while (pl2) {
                pl3=c_net->LastPlace;
                while (pl3) {
                    pl4=a_net->LastPlace;
                    while (pl4) {
                        dim+=2*RASTER;

(pl=M_DuplicatePlace(b_net,pl1,&stlast,&tslast,&pstlast,&ptslast))->label=EMPTY;
M_SumPlaceRefs( pl, pl2 );
M_SumPlaceRefs( pl, pl3 );
M_SumPlaceRefs( pl, pl4 );
pl->preset=M_CopyPlacePreset(pl2,pl,&tslast,False);
if (stlast) {
    if ((stlast->nexttrans=M_CopyPlacePostset(pl3,pl,&stdummy,False))!=NULL) {
        stlast->nexttrans->prevtrans=stlast;
    }
} else {
    pl->postset=M_CopyPlacePostset(pl3,pl,&stlast,False);
}
if (tslast) {
    if ((tslast->nexttrans=M_CopyPlacePreset(pl4,pl,&tsdummy,False))!=NULL) {
        tslast->nexttrans->prevtrans=tslast;
    }
} else {
    pl->preset=M_CopyPlacePreset(pl4,pl,&tslast,False);
}

pl->ppreset=M_CopyPlacePreset(pl2,pl,&ptslast,True);
if (pstlast) {
    if ((pstlast->nexttrans=M_CopyPlacePostset(pl3,pl,&stdummy,True))!=NULL) {
        pstlast->nexttrans->prevtrans=pstlast;
    }
} else {
    pl->ppostset=M_CopyPlacePostset(pl3,pl,&pstlast,True);
}

if (ptslast) {
    if ((ptslast->nexttrans=M_CopyPlacePreset(pl4,pl,&tsdummy,True))!=NULL) {
        ptslast->nexttrans->prevtrans=ptslast;
    }
} else {
    pl->ppreset=M_CopyPlacePreset(pl4,pl,&ptslast,True);
}


                        pl4=pl4->prevplace;
                    }
                    pl3=pl3->prevplace;
                }
                pl2=pl2->prevplace;
            }
            pl1=M_RemovePlace(b_net,pl1);
        } else
            pl1=pl1->prevplace;
    }

    /*-----------------------------------------------*/
    /* Die neuen Stellen (links) zentriert anordnen. */
    /*-----------------------------------------------*/

    dim -= RASTER;
    pos=((int)(net_y_dim-(dim))>>1)+net_y_pos;
    if (dim>net_y_dim) {
        net_y_dim=dim;
        net_y_pos=pos;
    }
    dim -=RASTER;
    pos+=dim;
    while (pl1) {
        pos=(pl1->y_pos=pos)-2*RASTER;
        pl1->numname=1;
        pl1=pl1->prevplace;
    }

    /*-----------------------------------------------------------*/
    /*  exit(b_net) * entry(net_b) * entry(net_c) * exit(net_a)  */
    /*-----------------------------------------------------------*/

    dim=0;
    pl1=b_net->LastPlace;
    while (pl1 && pl1->numname) {
        if (pl1->label==EXIT) {
            pl2=net_b->LastPlace;
            while (pl2) {
                pl3=net_c->LastPlace;
                while (pl3) {
                    pl4=net_a->LastPlace;
                    while (pl4) {
                        dim+=2*RASTER;

(pl=M_DuplicatePlace(b_net,pl1,&stlast,&tslast,&pstlast,&ptslast))->label=EMPTY;
M_SumPlaceRefs( pl, pl2 );
M_SumPlaceRefs( pl, pl3 );
M_SumPlaceRefs( pl, pl4 );

if ((pl->postset=M_CopyPlacePostset(pl2,pl,&stlast,False))!=0) {
    if ((stlast->nexttrans=M_CopyPlacePostset(pl3,pl,&stdummy,False))!=NULL) {
        stlast->nexttrans->prevtrans=stlast;
    }
} else {
    pl->postset=M_CopyPlacePostset(pl3,pl,&stlast,False);
}
if (tslast) {
    if ((tslast->nexttrans=M_CopyPlacePreset(pl4,pl,&tsdummy,False))!=NULL) {
        tslast->nexttrans->prevtrans=tslast;
    }
} else {
    pl->preset=M_CopyPlacePreset(pl4,pl,&tslast,False);
}

if ((pl->ppostset=M_CopyPlacePostset(pl2,pl,&pstlast,True))!=0) {
    if ((pstlast->nexttrans=M_CopyPlacePostset(pl3,pl,&stdummy,True))!=NULL) {
        pstlast->nexttrans->prevtrans=pstlast;
    }
} else {
    pl->ppostset=M_CopyPlacePostset(pl3,pl,&pstlast,True);
}
if (ptslast) {
    if ((ptslast->nexttrans=M_CopyPlacePreset(pl4,pl,&tsdummy,True))!=NULL) {
        ptslast->nexttrans->prevtrans=ptslast;
    }
} else {
    pl->ppreset=M_CopyPlacePreset(pl4,pl,&ptslast,True);
}

                        pl4=pl4->prevplace;
                    }
                    pl3=pl3->prevplace;
                }
                pl2=pl2->prevplace;
            }
            pl1=M_RemovePlace(b_net,pl1);
        } else
            pl1=pl1->prevplace;
    }

    /*------------------------------------------------*/
    /* Die neuen Stellen (rechts) zentriert anordnen. */
    /*------------------------------------------------*/

    dim -= RASTER;
    pos=((int)(net_y_dim-(dim))>>1)+net_y_pos;
    if (dim>net_y_dim) {
        net_y_dim=dim;
        net_y_pos=pos;
    }
    pos+=dim-RASTER;
    while (pl1) {
        pos=(pl1->y_pos=pos)-2*RASTER;
        pl1->numname=1;
        pl1=pl1->prevplace;
    }

    /*-------------------------------------------------------*/
    /* 'net_a' rechts neben 'a_net' legen. Dann diese beiden */
    /* oberhalb des bisher konstruierten Netzes zentrieren.  */
    /*-------------------------------------------------------*/

    M_move_hv(net_a,a_net->x_pos-net_a->x_pos+a_net->x_dim+RASTER,
                    a_net->y_pos-net_a->y_pos);
    x_offset=net_x_pos-a_net->x_pos+(net_x_dim-RASTER)/2-a_net->x_dim;
    y_offset=net_y_pos-a_net->y_pos-a_net->y_dim+RASTER;
    M_move_hv(net_a,x_offset,y_offset);
    M_move_hv(a_net,x_offset,y_offset);
    net_y_dim+=a_net->y_dim-RASTER;
    net_y_pos=a_net->y_pos;
    if (net_x_dim<(Cardinal)(RASTER+2*a_net->x_dim)) {
        net_x_dim=RASTER+2*a_net->x_dim;
        net_x_pos=a_net->x_pos;
    }

    entry_x_dim -= RASTER;
    pl1=entry_pl;
    pos=(int)(net_x_dim-(entry_x_dim))/2+net_x_pos;
    if (entry_x_dim>net_x_dim) {
        net_x_dim=entry_x_dim;
        net_x_pos=pos;
    }
    pos+=entry_x_dim-RASTER;
    while (pl1) {
        pos=(pl1->x_pos=pos)-2*RASTER;
        pl1->numname=1;
        pl1=pl1->prevplace;
    }

    /*-------------------------------------------------------*/
    /* 'net_c' rechts neben 'c_net' legen. Dann diese beiden */
    /* unterhalb des bisher konstruierten Netzes zentrieren. */
    /*-------------------------------------------------------*/

    M_move_hv(net_c,c_net->x_pos-net_c->x_pos+c_net->x_dim+RASTER,
                    c_net->y_pos-net_c->y_pos);
    x_offset=net_x_pos-c_net->x_pos+(net_x_dim-RASTER)/2-c_net->x_dim;
    y_offset=net_y_pos-c_net->y_pos+net_y_dim-RASTER;
    M_move_hv(net_c,x_offset,y_offset);
    M_move_hv(c_net,x_offset,y_offset);
    net_y_dim+=c_net->y_dim-RASTER;
    if (net_x_dim<(Cardinal)(2*c_net->x_dim+RASTER)) {
        net_x_dim=2*c_net->x_dim+RASTER;
        net_x_pos=c_net->x_pos;
    }

    exit_x_dim -= RASTER;
    pl1=exit_pl;
    pos=(int)(net_x_dim-(exit_x_dim))/2+net_x_pos;
    if (exit_x_dim>net_x_dim) {
        net_x_dim=exit_x_dim;
        net_x_pos=pos;
    }
    pos+=exit_x_dim-RASTER;
    while (pl1) {
        pos=(pl1->x_pos=pos)-2*RASTER;
        pl1->numname=1;
        pl1=pl1->prevplace;
    }

    /*---------*/
    /* Fertig. */
    /*---------*/

    b_net->x_pos=net_x_pos;
    b_net->y_pos=net_y_pos;
    b_net->x_dim=net_x_dim;
    b_net->y_dim=net_y_dim;

    return M_DisjointNetUnion(b_net,
            M_DisjointNetUnion(M_RestoreKillPlaces(net_b,ENTRY | EXIT),
             M_DisjointNetUnion(M_RestoreKillPlaces(c_net,ENTRY),
              M_DisjointNetUnion(M_RestoreKillPlaces(net_c,ENTRY | EXIT),
               M_DisjointNetUnion(M_RestoreKillPlaces(a_net,EXIT),
                                  M_RestoreKillPlaces(net_a,ENTRY | EXIT)
                               )
                              )
                             )
                            )
                             );
}

/*******/
/* ??? */
/*******/

M_Net MediumIteration(alt_set_type alt, M_Net enter)
{
    register M_Place  pl1,pl2,pl3,pl4;
             M_Place  pl;
             M_Place  merke;
             M_Place  exit_pl;
             M_Net    a_net,b_net,c_net,
                            net_b,net_c;
             M_STEdge stlast,pstlast;
             M_TSEdge tslast,ptslast;
             int    pos;
             int    x_offset,y_offset;
             Cardinal dim;
             Cardinal exit_x_dim;

    a_net=enter; /* wird beim Betreten der Schleife einmalig ausgefuehrt */
    net_b=M_CreateNetCopy(b_net=alt.repeat_net);
    net_c=M_CreateNetCopy(c_net=alt.exit_net);


    /*------------------------------------------*/
    /* Das nach rechts gedrehte 'net_b' unter   */
    /* das nach links   gedrehte 'b_net' legen. */
    /*------------------------------------------*/

    M_turn_right(net_b);
    M_turn_left(b_net);
    M_move_hv(net_b,b_net->x_pos-net_b->x_pos,
                    b_net->y_pos-net_b->y_pos+b_net->y_dim+RASTER);

    b_net->y_dim*=2; b_net->y_dim+=RASTER;

    /*---------------------------*/
    /* exit(c_net) * exit(net_c) */
    /*---------------------------*/

    exit_x_dim=0;
    merke=M_PreparePlaces(net_c,ENTRY,EXIT);
    pl1=c_net->LastPlace;
    while (pl1 && pl1->numname) {
        if (pl1->label==EXIT) {
            pl2=merke;
            while (pl2) {
                exit_x_dim+=2*RASTER;
                if (M_DuplicatePlace(c_net,pl1,&stlast,&tslast,
				     &pstlast,&ptslast)->preset) {
                    if ((tslast->nexttrans
                         =M_CopyPlacePreset(pl2,c_net->Places,
					    &tsdummy,False))!=NULL)
                        tslast->nexttrans->prevtrans=tslast;
                } else
                    c_net->Places->preset
                    =M_CopyPlacePreset(pl2,c_net->Places,&tsdummy,False);
                if (c_net->Places->ppreset!=NULL) {
                    if ((ptslast->nexttrans
                         =M_CopyPlacePreset(pl2,c_net->Places,
					    &tsdummy,True))!=NULL)
                        ptslast->nexttrans->prevtrans=ptslast;
                } else
		  c_net->Places->ppreset
		    =M_CopyPlacePreset(pl2,c_net->Places,&tsdummy,True);
                pl2=pl2->prevplace;
            }
            pl1=M_RemovePlace(c_net,pl1);
        } else
            pl1=pl1->prevplace;
    }
    exit_pl=pl1;

    M_PreparePlaces(a_net,EXIT,EXIT);
    M_PreparePlaces(c_net,ENTRY,ENTRY);
    merke=M_PreparePlaces(net_b,ENTRY,EXIT);

    /*---------------------------------------------------------*/
    /* entry(b_net) * exit(net_b) * entry(c_net) * exit(a_net) */
    /*---------------------------------------------------------*/

    dim=0;
    pl1=b_net->LastPlace;
    while (pl1 && pl1->numname) {
        if (pl1->label==ENTRY) {
            pl2=merke;
            while (pl2) {
                pl3=c_net->LastPlace;
                while (pl3) {
                    pl4=a_net->LastPlace;
                    while (pl4) {
                        dim+=2*RASTER;

(pl=M_DuplicatePlace(b_net,pl1,&stlast,&tslast,&pstlast,&ptslast))->label=EMPTY;
M_SumPlaceRefs( pl, pl2 );
M_SumPlaceRefs( pl, pl3 );
M_SumPlaceRefs( pl, pl4 );
pl->preset=M_CopyPlacePreset(pl2,pl,&tslast,False);
if (stlast) {
    if ((stlast->nexttrans=M_CopyPlacePostset(pl3,pl,&stdummy,False))!=NULL) {
        stlast->nexttrans->prevtrans=stlast;
    }
} else {
    pl->postset=M_CopyPlacePostset(pl3,pl,&stlast,False);
}
if (tslast) {
    if ((tslast->nexttrans=M_CopyPlacePreset(pl4,pl,&tsdummy,False))!=NULL) {
        tslast->nexttrans->prevtrans=tslast;
    }
} else {
    pl->preset=M_CopyPlacePreset(pl4,pl,&tslast,False);
}

pl->ppreset=M_CopyPlacePreset(pl2,pl,&ptslast,True);
if (pstlast) {
    if ((pstlast->nexttrans=M_CopyPlacePostset(pl3,pl,&stdummy,True))!=NULL) {
        pstlast->nexttrans->prevtrans=pstlast;
    }
} else {
    pl->ppostset=M_CopyPlacePostset(pl3,pl,&pstlast,True);
}
if (ptslast) {
    if ((ptslast->nexttrans=M_CopyPlacePreset(pl4,pl,&tsdummy,True))!=NULL) {
        ptslast->nexttrans->prevtrans=ptslast;
    }
} else {
    pl->ppreset=M_CopyPlacePreset(pl4,pl,&ptslast,True);
}

                        pl4=pl4->prevplace;
                    }
                    pl3=pl3->prevplace;
                }
                pl2=pl2->prevplace;
            }
            pl1=M_RemovePlace(b_net,pl1);
        } else
            pl1=pl1->prevplace;
    }

    /*-----------------------------------------------*/
    /* Die neuen Stellen (links) zentriert anordnen. */
    /*-----------------------------------------------*/

    dim -= RASTER;
    pos=((int)(b_net->y_dim-(dim))>>1)+b_net->y_pos;
    if (dim>b_net->y_dim) {
        b_net->y_dim=dim;
        b_net->y_pos=pos;
    }
    dim -= RASTER;
    pos+=dim;
    while (pl1) {
        pos=(pl1->y_pos=pos)-2*RASTER;
        pl1->numname=1;
        pl1=pl1->prevplace;
    }

    /*-------------------------------------------*/
    /* exit(b_net) * entry(net_b) * entry(net_c) */
    /*-------------------------------------------*/

    dim=0;
    pl1=b_net->LastPlace;
    while (pl1 && pl1->numname) {
        if (pl1->label==EXIT) {
            pl2=net_b->LastPlace;
            while (pl2) {
                pl3=net_c->LastPlace;
                while (pl3) {
                    dim+=2*RASTER;

(pl=M_DuplicatePlace(b_net,pl1,&stlast,&tslast,&pstlast,&ptslast))->label=EMPTY;
M_SumPlaceRefs( pl, pl2 );
M_SumPlaceRefs( pl, pl3 );
if ((pl->postset=M_CopyPlacePostset(pl2,pl,&stlast,False))!=0) {
    if ((stlast->nexttrans=M_CopyPlacePostset(pl3,pl,&stdummy,False))!=NULL) {
        stlast->nexttrans->prevtrans=stlast;
    }
} else {
    pl->postset=M_CopyPlacePostset(pl3,pl,&stlast,False);
}
if ((pl->ppostset=M_CopyPlacePostset(pl2,pl,&pstlast,True))!=0) {
    if ((pstlast->nexttrans=M_CopyPlacePostset(pl3,pl,&stdummy,True))!=NULL) {
        pstlast->nexttrans->prevtrans=pstlast;
    }
} else {
    pl->ppostset=M_CopyPlacePostset(pl3,pl,&pstlast,True);
}

                    pl3=pl3->prevplace;
                }
                pl2=pl2->prevplace;
            }
            pl1=M_RemovePlace(b_net,pl1);
        } else
            pl1=pl1->prevplace;
    }

    /*------------------------------------------------*/
    /* Die neuen Stellen (rechts) zentriert anordnen. */
    /*------------------------------------------------*/

    dim -= RASTER;
    pos=((int)(b_net->y_dim-(dim))>>1)+b_net->y_pos;
    if (dim>b_net->y_dim) {
        b_net->y_dim=dim;
        b_net->y_pos=pos;
    }
    pos+=dim-RASTER;
    while (pl1) {
        pos=(pl1->y_pos=pos)-2*RASTER;
        pl1->numname=1;
        pl1=pl1->prevplace;
    }

    /*--------------------------------------------------------------*/
    /* 'a_net' oberhalb des bisher konstruierten Netzes zentrieren. */
    /*--------------------------------------------------------------*/

    M_move_hv(a_net,b_net->x_pos-a_net->x_pos+
                    ((int)(b_net->x_dim-a_net->x_dim)>>1),
                    b_net->y_pos-a_net->y_pos-a_net->y_dim+RASTER);
    if (a_net->x_dim>b_net->x_dim) {
        b_net->x_dim=a_net->x_dim;
        b_net->x_pos=a_net->x_pos;
    }
    b_net->y_dim+=a_net->y_dim-RASTER;
    b_net->y_pos=a_net->y_pos;

    /*-------------------------------------------------------*/
    /* 'net_c' rechts neben 'c_net' legen. Dann diese beiden */
    /* unterhalb des bisher konstruierten Netzes zentrieren. */
    /*-------------------------------------------------------*/

    M_move_hv(net_c,c_net->x_pos-net_c->x_pos+c_net->x_dim+RASTER,
                    c_net->y_pos-net_c->y_pos);
    x_offset=b_net->x_pos-c_net->x_pos+(b_net->x_dim-RASTER)/2-c_net->x_dim;
    y_offset=b_net->y_pos-c_net->y_pos+b_net->y_dim-RASTER;
    M_move_hv(net_c,x_offset,y_offset);
    M_move_hv(c_net,x_offset,y_offset);
    b_net->y_dim+=c_net->y_dim-RASTER;
    if (b_net->x_dim<(Cardinal)(c_net->x_dim*2+RASTER)) {
        b_net->x_dim=c_net->x_dim*2+RASTER;
        b_net->x_pos=c_net->x_pos;
    }

    pl1=exit_pl;
    exit_x_dim -= RASTER;
    pos=((int)(b_net->x_dim-(exit_x_dim))>>1)+b_net->x_pos;
    if (exit_x_dim>b_net->x_dim) {
        b_net->x_dim=exit_x_dim;
        b_net->x_pos=pos;
    }
    pos+=exit_x_dim-RASTER;
    while (pl1) {
        pos=(pl1->x_pos=pos)-2*RASTER;
        pl1->numname=1;
        pl1=pl1->prevplace;
    }

    /*---------*/
    /* Fertig. */
    /*---------*/

    return M_DisjointNetUnion(b_net,
            M_DisjointNetUnion(M_RestoreKillPlaces(net_b,ENTRY | EXIT),
             M_DisjointNetUnion(M_RestoreKillPlaces(c_net,ENTRY),
              M_DisjointNetUnion(M_RestoreKillPlaces(net_c,ENTRY | EXIT),
                               M_RestoreKillPlaces(a_net,EXIT)
                              )
                             )
                            )
                           );
}

  /********************/
  /* Dateiende iter.c */
  /********************/

