/****************************************************************************/
/* macros.c                                                                 */
/*                                                                          */
/* - Routinen zum Einlesen von Konfigurationsdateien mit Makros,            */
/* - Routinen um diese Dateien vorzucompilieren und anschliessend bei       */
/*   Bedarf die erzeugte Zwischendarstellung zu interpretieren, so dass man */
/*   letztlich M-Netze erhaelt                                              */
/*                                                                          */
/* Martin Ackermann, Winter 96/97                                           */
/****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "mnet.h"
#include "macros.h"
#include "readHL.h"
#include "compose.h"
#include "tables.h"
#include "f_parse.h"
#include "box_sub.h"


/****************************************************************************/
/* globale Defines und Variablen                                            */
/****************************************************************************/

#define NOTMPL ((void*)-1) /* Markierung fuer optionale Templates, die nicht
     geladen wurden, weil die entsprechende Konfigurationsdatei gefehlt hat */

char path_to_templates[FILENAME_MAX]=""; /* Pfad, aus dem Konfigurations-
                                                     dateien gelesen werden */
int  p_t_t_len = 0;                 /* Laenge des Strings path_to_templates */

Template tmpl[]= { {         "var_box.hl_net", NULL, NULL, 0, NULL, 0 } , 
                   {        "var_init.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {        "var_term.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {     "initvar_box.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {    "initvar_init.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {    "initvar_term.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {       "chan0_box.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {      "chan0_init.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {      "chan0_term.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {       "chan1_box.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {      "chan1_init.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {      "chan1_term.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {      "chan_box_0.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {     "chan_init_0.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {     "chan_term_0.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {       "stack_box.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {      "stack_init.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {      "stack_term.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {      "proc_box_0.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {  "proc_box_set_0.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {     "proc_init_0.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {     "proc_term_0.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {     "proc_call_0.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {         "val_par.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {       "const_par.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {         "res_par.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {      "varref_par.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {  "varref_par_rec.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {    "chan0ref_par.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {"chan0ref_par_rec.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {     "chanref_par.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   { "chanref_par_rec.hl_net", NULL, NULL, 0, NULL, 0 } ,
		   /* ab hier SDL */
                   {	   "channel_0.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {     "signalroute.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {            "init.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {            "term.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {          "create.hl_net", NULL, NULL, 0, NULL, 0 } ,
		   {	     "connect.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {	   "process_0.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {	"process_exit.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {	    "variable.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {	    "var_init.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {	    "var_term.hl_net", NULL, NULL, 0, NULL, 0 } ,
                   {	   "procedure.hl_net", NULL, NULL, 0, NULL, 0 } ,
		   {	   "proc_call.hl_net", NULL, NULL, 0, NULL, 0 } ,
		   {	"prcd_in_parm.hl_net", NULL, NULL, 0, NULL, 0 } ,
		   { "prcd_inout_parm.hl_net", NULL, NULL, 0, NULL, 0 } ,
		   {     "proc_create.hl_net", NULL, NULL, 0, NULL, 0 } ,
		   {           "label.hl_net", NULL, NULL, 0, NULL, 0 } ,
		   {            "join.hl_net", NULL, NULL, 0, NULL, 0 }
		 };

bool bpn_templates;
 
Cardinal      akt_nr; 
       /* zeigt waehrend readHL.c auf das Template, das gerade gelesen wird */

                                           /* Array mit FOREACH-Konstrukten */
Foreach_data  foreach[ MAX_FOREACHS ] = { { NULL, 0, NULL, 0 } };
    /* das leeres Schlusselement ist durch Wertebereich NULL gekennzeichnet */

M_Transition  trans_mark[NUM_T_MARKS]; /* Array mit markierten Transitionen */
M_Place       place_mark[NUM_P_MARKS]; /* Array mit markierten Stellen      */

Integer       dot_set[]={ 2, DOT };     /* Wertebereich nur mit Element dot */


/****************************************************************************/
/* aktuelle Belegung der verschiedenen Makros                               */
/****************************************************************************/

Integer      *macro_decl_set,
             *macro_id_set = dot_set,
             *macro_new_ids,
	     *macro_senders,
	     *macro_senders2,
	     *macro_signals;

Integer       macro_max,
              macro_capacity,
              macro_capacity_plus_eins,
              macro_init,
              macro_index,
              macro_element,
	      macro_baseid;
 
char         *macro_name,
             *macro_arg,
             *macro_ext;

RefNr         macro_ref;
M_Net         macro_body;


/****************************************************************************/
/* Liefert einen Zeiger auf den nullten Eintrag im cmd-Array des Templates, */
/* das gerade gelesen wird.                                                 */
/****************************************************************************/

Cmd *GetFirstCmd( void )
  {
    Template *t = tmpl + akt_nr;
    
    return t->cmd;
  }


/****************************************************************************/
/* Liefert einen Zeiger auf den naechsten leeren Eintrag im cmd-Array des   */
/* Templates, das gerade gelesen wird.                                      */
/****************************************************************************/

Cmd *GetNextCmd( void )
  {
    Template *t = tmpl + akt_nr;
    
    return t->cmd + t->cmd_entries;
  }


/****************************************************************************/
/* Liefert die Anzahl der Eintraege im cmd-Array des Templates, das gerade  */
/* gelesen wird.                                                            */
/****************************************************************************/

Cardinal GetCmdEntries( void )
  {
    Template *t = tmpl + akt_nr;
    
    return t->cmd_entries;
  }


/****************************************************************************/
/* Schneidet die letzten anz Kommandos ab vom cmd-Array des Templates, das  */
/* gerade gelesen wird. Fuer die abgeschnittenen Kommandos wird ein passen- */
/* des neues Array angelegt, dessen Startadresse zurueckgeliefert wird.     */
/****************************************************************************/

Cmd *ExtractLastCmds( Cardinal anz )
  {
    Template *t = tmpl + akt_nr;
    Cmd      *LastCmds;
    size_t    anz_bytes = anz * sizeof( Cmd );

    LastCmds = MYmalloc( anz_bytes );
    t->cmd_entries -= anz;
    memcpy( LastCmds, t->cmd + t->cmd_entries, anz_bytes );
    t->cmd = MYrealloc( t->cmd, t->cmd_entries * sizeof( Cmd ) );
    
    return LastCmds;
  }


/****************************************************************************/
/* Eindimensionale Definitionen werden zunaechst wie mehrdimensionale       */
/* Definitionen geparst. Mit dieser Routine werden fuer eindimensionale     */
/* Definitionen anschliessend die notwendigen Anpassungen vorgenommen.      */
/*                                                                          */
/* Sie durchlaeuft die Kommandos des aktuellen Templates von hinten:        */
/* - Solange Kommandos gefunden werden, die den Wertebereich der Variable   */
/*   com_nr beeinflussen, wird die Projektionsnummer von mehrdimensional    */
/*   auf eindimensional korrigiert.                                         */
/* - Sobald ein anderes Kommando gefunden wird oder die Kommandoliste ganz  */
/*   durchlaufen wurde, bricht die Routine ab.                              */
/****************************************************************************/

void CorrectIndices( Var com_nr )
  {
    if ( flag & MNET_SYNCHRONIZE )
      {
        /* There are no Macros in the source net ! */
        return;
      }
    else
    {
      Template *t = tmpl + akt_nr;
      Cmd      *c = t->cmd + t->cmd_entries - 1;

      while ( c>=t->cmd && (    
               (c->cmd_nr==Trans_AddValues && (c->p4 % MaxAnzComVars)==com_nr)
            || (c->cmd_nr==Trans_SetRange  &&  c->p2                 ==com_nr)))
        (c--)->p3 = -1;
        /* funktioniert nur, weil sich keine anderen cmds dazwischen setzen */
    }
  }


/****************************************************************************/
/* Der Bezug zwischen VAR- / CONST-Knoten in einem Baum und Kommandos, wel- */
/* che sich auf diese Knoten beziehen, wird zunaechst durch eine eindeutige */
/* Numerierung (counter_index) hergestellt.                                 */
/* Erst wenn durch TreeToGuard der Tree in einen Guard wird, steht die      */
/* relative Adresse der Knoten fest. Dann kann anstelle des counter_index   */
/* diese relative Adresse des Knotens zur Identifizierung verwendet werden. */
/*                                                                          */
/* Diese Routine durchlaeuft die Kommandos des aktuellen Templates von      */
/* hinten. Wenn sie das Kommando vom Typ cmd_nr gefunden hat, das sich auf  */
/* den Knoten mit counter_index bezieht, ergaenzt sie das Kommando um die   */
/* relative Adresse des Knotens (node_index). Dadurch kann dieses Kommando  */
/* spaeter DIREKT auf den Knoten zugreifen.                                 */
/****************************************************************************/

void CounterIndex2NodeIndex ( Cardinal cmd_nr, Cardinal counter_index, 
                              Cardinal node_index )
  {
    Template *t = tmpl + akt_nr;
    Cmd      *c = t->cmd + t->cmd_entries - 1;

    while ( c>=t->cmd && (c->cmd_nr!=cmd_nr || c->p2!=counter_index) )
      c--;
    
    if ( c<t->cmd )
      Msg( FAT, "Internal Error: Command not found !" );
    c->p3 = node_index;
  }


/****************************************************************************/
/* Das cmd-Array des Templates, das gerade gelesen wird, wird um ein        */
/* Kommando erweitert, dessen Nummer und Parameter an die Prozedur ueber-   */
/* geben werden.                                                            */
/* Da Parameter 4 nur von wenigen Kommandos benutzt wird, wird er nicht     */
/* an Add_Cmd uebergeben, sondern muss anschliessend extra gesetzt werden.  */
/*                                                                          */
/* Um die Komponente zu ermitteln, auf die sich das Kommando bezieht,       */
/* greift die Prozedur auf die Variable akt_component aus readHL.c zu.      */
/*                                                                          */
/* Rueckgabe: ein Zeiger auf das neu erzeugte Kommando                      */
/****************************************************************************/

Cmd* Add_Cmd( Cardinal cmd_code, void *p1, Cardinal32 p2, Cardinal32 p3 )
  {
    Cmd      *c;

    if ( flag & MNET_SYNCHRONIZE )
      {
        Msg( FAT, "Macros are not allowed in the source net !" );
        return NULL;
      }
    else
      {
        Template *t = tmpl + akt_nr;
        
        t->cmd = MYrealloc( t->cmd, ++(t->cmd_entries) * sizeof( Cmd ) );
        c = t->cmd + t->cmd_entries - 1;
        
        c->component = akt_component;
        c->cmd_nr    = cmd_code;
        c->p1        = p1;
        c->p2        = p2;
        c->p3        = p3;

        return c;
      }
  }


/****************************************************************************/
/* Funktioniert analog zu Add_Cmd (siehe oben).                             */
/*                                                                          */
/* Allerdings wird das neue Kommando nicht in das Array cmd eingefuegt,     */
/* sondern in das Array netcmd.                                             */
/*                                                                          */
/* Die Kommandos in netcmd fuehren dazu, dass sich die Adresse von einzelnen*/
/* Netzkomponenten aendert. Da sich aber die Kommandos aus cmd auf feste    */
/* Adressen beziehen, muessen die Kommandos aus netcmd ganz zum Schluss     */
/* ausgefuehrt werden.                                                      */
/*                                                                          */
/* Bisher muessen in netcmd nur die Trans_Replace-Kommandos abgelegt werden.*/
/****************************************************************************/

Cmd* Add_NetCmd( Cardinal cmd_code, void *p1, Cardinal32 p2, Cardinal32 p3 )
  {
    Cmd      *c;

    if ( flag & MNET_SYNCHRONIZE )
      {
        Msg( FAT, "Macros are not allowed in the source net !" );
        return NULL;
      }
    else
      {
        Template *t = tmpl + akt_nr;

        t->netcmd = MYrealloc( t->netcmd, ++(t->netcmd_entries) * sizeof(Cmd) );
        c = t->netcmd + t->netcmd_entries - 1;

        c->component = akt_component;
        c->cmd_nr    = cmd_code;
        c->p1        = p1;
        c->p2        = p2;
        c->p3        = p3;
        return c;
      }
  }


/****************************************************************************/
/* Gibt ein Kommando-Array frei. Dazu muss das erste Kommando (first_cmd)   */
/* und die Anzahl der Kommandos (cmd_entries) uebergeben werden.            */
/*                                                                          */
/* Bei allen Kommandos, deren Parameter eigene Speicherbereiche belegen,    */
/* werden auch diese Speicherbereiche freigegeben.                          */
/****************************************************************************/

void RemoveCmds( Cmd *first_cmd, Cardinal cmd_entries  )
  {
    Cmd *c, *last_cmd = first_cmd + cmd_entries;

    for ( c=first_cmd; c<last_cmd; c++ )
      switch( c->cmd_nr ) {
        case Trans_AddAction:
        case Guard_SetVar:
        case Tupel_SetVar:
        case Arc_AddVar:
        case Arc_AddTupel:     MYfree( c->p1 ); 
                               break;
        case Foreach_Init:     if ( c->p1 != EMPTY_SET )
                                 MYfree( c->p1 );
                               break;
        case Guard_AddForeach:
        case Arc_AddForeach:   RemoveCmds( c->p1, c->p2 );
                               break;
      }
    MYfree( first_cmd );
  }


/****************************************************************************/
/* Gibt den Speicherbereich fuer alle Templates frei, die eingelesen worden */
/* sind.                                                                    */
/*                                                                          */
/* Templates, die nicht benoetigt wurden und somit auch nicht eingelesen    */
/* worden sind, erkennt die Prozedur daran, dass deren net-Komponente noch  */
/* ist.                                                                     */
/****************************************************************************/

void RemoveTemplates( void )
  {
    Cardinal i;
    
    for ( i=0; i < sizeof( tmpl ) / sizeof( Template ); i++ )
      if ( tmpl[i].net )
        {
          M_Remove_net( &(tmpl[i].net) );
          RemoveCmds( tmpl[i].cmd, tmpl[i].cmd_entries );
        }
          MYfree( tmpl[i].netcmd ); 
            /* Da die Parameter der Kommandos in netcmd bisher keinen eigenen
               Speicherbereich belegen, kann netcmd im Gegensatz zu cmd direkt
               freigegeben werden. */
  }


/****************************************************************************/
/* Liefert TRUE, gdw. das File name lesbar ist.                             */
/****************************************************************************/

static bool file_exists( char *name )
{
  FILE *file;
  file = fopen( name, "r");
  if ( file )
    {
      fclose( file );
      return True;
    } 
  return False;
}


/****************************************************************************/
/* Liest das Template mit der Nummer nr aus der entsprechenden Konfigura-   */
/* tionsdatei ein. Falls die Konfigurationsdatei nicht lesbar ist, bestimmt */
/* das Flag forced_load das Verhalten der Prozedur:                         */
/* - forced_load==TRUE : Fehlermeldung und Programmabbruch                  */
/* - forced_load==FALSE: Template als nicht vorhanden markieren             */
/*                                                                          */
/* Mit forced_load kann also zwischen obligatorischen Templates (z.B.       */
/* Datenbox) und optionalen Templates (z.B. Initnetz) unterschieden werden. */
/****************************************************************************/

M_Net GetTemplate( Cardinal nr, bool forced_load)
  { 
    char *pephome;

    if ( ! tmpl[nr].cmd )
      {
        /* aktuellen Stand des globalen Transitionenzaehlers speichern */
        Cardinal32 real_trans_anz = trans_anz;

        /* Wurde der Suchpfad noch nicht gesetzt (durch Kommandozeilenoption
           oder fruehere Aufrufe von GetTemplate) ? */
        if ( *path_to_templates == 0 )
          {
            /* Standardpfad ist $PEPHOME/lib/bpn2mnet/___procs.hl bzw.
                                $PEPHOME/lib/bpn2mnet/no_procs.hl,
                                $PEPHOME/lib/sdl2mnet,
               je nachdem, ob die Prozedursemantik, die vereinfachte oder
	       die SDL-Semantik verwendet wird. */
            pephome = getenv( "PEPHOME" );
            if ( !pephome )
              Msg( FAT, "Environment variable $PEPHOME must be set !\n" );
            p_t_t_len = sprintf( path_to_templates, "%s/lib/%s/", pephome, 
				(bpn_templates) ? 
				(( GL_ProcDecl ) ? "bpn2mnet/___procs.hl" :
						  "bpn2mnet/no_procs.hl"):
				"sdl2mnet"
									);
          }

        /* Name der Konfigurationsdatei anhaengen */
        strcpy( path_to_templates + p_t_t_len, tmpl[nr].file );

        if ( forced_load || file_exists( path_to_templates ) )
          { 
            akt_nr = nr;
            tmpl[nr].cmd    = NULL;
            tmpl[nr].netcmd = NULL;
            tmpl[nr].net    = read_HLnet( path_to_templates, NULL );
          }
        else
          tmpl[nr].cmd = NOTMPL;

        /* Name der Konfigurationsdatei wieder vom Pfad abschneiden */
        path_to_templates[p_t_t_len] = 0;

        /* Stand des globalen Transitionenzaehlers wieder herstellen,
           weil die Transitionen von Templates nicht zu den Transitionen
           des vom Backend erzeugten M-Netzes gerechnet werden duerfen. */
        trans_anz = real_trans_anz;
      }

    return tmpl[nr].net;
  }


/****************************************************************************/
/* Fuegt in den Kantenlabel von st die Variable nr ein.                     */
/****************************************************************************/

static void AddVarToLabel( Var nr, M_STEdge st )
{
  V_mmLeaf[0].anzahl = 1;
  V_mmLeaf[0].nr     = nr;
  V_mmLeaf[1].anzahl = 0;
  
  V_MSsum( V_mmLeaf, st->label );
  VSremove( st->label );
  st->label = V_MScreate_copy( V_mmLeaf, NULL );
}


/****************************************************************************/
/* Kopiert 'org' nach 'name' und expandiert dabei alle in 'org' enthaltenen */
/* Makros der Form $...$. 'name' muss genuegend gross dimensioniert sein !! */
/****************************************************************************/

static void ExpandMacros( char *org, char *name )
{
  Cardinal macro_length;

  do
    {
      if (*org == '$' )
        { 
          org++;
          macro_length = strchr( org, '$' ) - org;
          org[macro_length] = 0; /* Abschneiden, damit strcmp funktioniert */
          if ( !strcmp( org, "NAME") )
            name += sprintf( name, macro_name ) - 1;
          else if ( !strcmp( org, "INDEX" ) )
            name += sprintf( name, "%d", macro_index ) - 1;
          else if ( !strcmp( org, "ELEMENT" ) )
            name += sprintf( name, "%d", macro_element ) - 1;
          else if ( !strcmp( org, "MAX" ) )
            name += sprintf( name, "%d", macro_max ) - 1;
          else if ( !strcmp( org, "CAPACITY" ) )
            name += sprintf( name, "%d", macro_capacity ) - 1;
          else if ( !strcmp( org, "CAPACITPLUSEINS" ) )
            name += sprintf( name, "%d", macro_capacity_plus_eins ) - 1;
          else if ( !strcmp( org, "INIT" ) )
            name += sprintf( name, "%d", macro_init ) - 1;
          else if ( !strcmp( org, "BASEID" ) )
            name += sprintf( name, "%d", macro_baseid ) - 1;
          else if ( !strcmp( org, "ARG" ) )
            name += sprintf( name, "%s", macro_arg ) - 1;
          else if ( !strcmp( org, "EXT" ) )
            name += sprintf( name, "%s", macro_ext ) - 1;
          else
            Msg( FAT, "Unknown macro in hl_net-box !" );
          org += macro_length;
          *org = '$'; /* Original-String wiederherstellen */
          org++;
        }
      else
        *name = *(org++); /* ein Zeichen kopieren */
    }
  while( *(name++) );
}


/****************************************************************************/
/* Wandelt die Parameter eines AddValues-Kommandos so um, dass sie fuer den */
/* Aufruf der Prozedur add_values verwendet werden koennen.                 */
/****************************************************************************/

static void Macro_AddValues( Integer* *set, Cmd *c, bool    do_realloc, 
                             Cardinal additional, Integer* *def )
{
  Integer   lower, upper;
  /* p4 ist bei AddValues-Kommandos ein Kombi-Parameter: 
     - p4 MOD MaxAnzComVars liefert eine Variablennummer
     - p4 DIV MaxAnzComVars liefert ein Statuswort */
  bool      p1_is_val =  (c->p4 / MaxAnzComVars) & 1;
  bool      p2_is_val =  (c->p4 / MaxAnzComVars) & 2;

  if (p1_is_val)
    lower = (int) c->p1;         /* p1 ist ein Wert   */
  else
    lower = *(Integer *)(c->p1); /* p1 ist ein Zeiger */

  if (p2_is_val)
    upper = (int) c->p2;         /* p2 ist ein Wert   */
  else
    upper = *(Integer *)(c->p2); /* p2 ist ein Zeiger */
  
  add_values( set, lower, upper, do_realloc, additional, def );
}


/****************************************************************************/
/* Ein Kommandoarray wird abgearbeitet. c zeigt auf das erste Kommando und  */
/* cmd_entries gibt die Kommandoanzahl an.                                  */ 
/*                                                                          */
/* Als net_ptr kann man meistens NULL uebergeben. Er wird naemlich nur dann */
/* benoetigt, wenn folgende Kommandos abgearbeitet werden sollen:           */
/* - Trans_Replace                                                          */
/*                                                                          */
/* Fuer alle anderen Kommandos ist die Adresse der zu manipulierenden Netz- */
/* komponente als Kommandoparameter vorgegeben. Da die Kommandos jedoch auf */
/* eine Kopie des Templates angewendet werden, muss diese Adresse ueber     */
/* numname angepasst werden. Beim Erzeugen der Netzkopie hat die Prozedur   */
/* M_CreateNetCopy die numnames passend gesetzt.                            */
/****************************************************************************/

void InterpretCmds( Cmd *c, Cardinal cmd_entries, M_Net *net_ptr )
  {
    Integer     *foreach_set;
    M_Place      pl;
    M_Transition tr;
    M_STEdge     st;
    Cmd         *last = c + cmd_entries;

    for ( ; c<last; c++ )
      {
        switch( c->cmd_nr ) {
          case Place_SetRange:
            /* Setze eine leere Einzelmenge innerhalb des (moeglicherweise
               mehrdimensionalen) Wertebereichs einer Stelle. */
            {
              Integer  *set   = *(Integer **)(c->p1);
              Cardinal  index = c->p3;

              pl = (M_Place) ( ((M_Place)     (c->component)) -> numname ); 
              M_CopySingleRange( pl->range + index, set );
              break;
            }
          case Place_AddValues:
            /* Fuege ein Intervall zu einer Einzelmenge des (moeglicherweise
               mehrdimensionalen) Wertebereichs einer Stelle hinzu. */
            {
              Cardinal  index = c->p3;

              pl = (M_Place) ( ((M_Place)     (c->component)) -> numname ); 
              Macro_AddValues( &(pl->range[index]), c, True, 0, NULL );
              break;
            }
          case Place_AddRef:
            /* Fuege zu einer Stelle eine Referenz hinzu. */
            {
              pl = (M_Place) ( ((M_Place)     (c->component)) -> numname ); 
              
              if (pl->RefEntries)
                pl->ref = MYrealloc( pl->ref, 
                                      (pl->RefEntries + 1) * sizeof (Ref) );
              else
                pl->ref = MYmalloc( sizeof( Ref ) );

              pl->ref[pl->RefEntries].nr   = macro_ref;
              pl->ref[pl->RefEntries].flag = c->p2;
              pl->RefEntries++;
              break;
            }
          case Place_Mark:
            /* Trage eine Stelle als markiert in das Markierungsarray ein. */
            {
              pl = (M_Place) ( ((M_Place)     (c->component)) -> numname );
              place_mark[ c->p2 ] = pl;
              break;
            }
	  case Place_Meaning:
	    /* Aendere die Bedeutung (nur $NAME$ als Makro moeglich) */
	    {
              M_Place pl = (M_Place) (((M_Place) (c->component))->numname);
              char *name = MYmalloc(strlen(pl->meaning)+strlen(macro_name));

              ExpandMacros(pl->meaning, name);

	      free(pl->meaning);
	      pl->meaning = MYstrdup(name);
	      free(name);
              break;
            }
          case Trans_SetRange:
            /* Setze fuer einen Transitionenguard eine Einzelmenge einer
               (moeglicherweise mehrdimensionalen) Definition. */
            {
              Integer  *set   = *(Integer **)(c->p1);
              Var       com_nr= c->p2;
              Cardinal  index = c->p3;

              tr = (M_Transition) ( ((M_Transition)(c->component)) -> numname );
              AddSingleRangeToDef( &(tr->def), com_nr, index, set, True );
              break;
            }
          case Trans_AddValues:
            /* Fuege fuer einen Transitionenguard ein Intervall in eine Einzel-
               menge einer (moeglicherweise mehrdimensionalen) Definition ein.*/
            {
              Cardinal  index     = c->p3;
              Var       com_nr    =  c->p4 % MaxAnzComVars;
              Integer  *d;

              tr = (M_Transition) ( ((M_Transition)(c->component)) -> numname );

              d = tr->def + 1;
              while( *d != -1 && 
                     ( *d != (Integer)com_nr || *(d+1) != (Integer)index ) )
                {
                  d += 2;
                  d += *d;
                }
              d += 2;

              Macro_AddValues( &d, c, False, 
                               *(tr->def) - (d - tr->def) - *d, &(tr->def) );
              break;
            }
          case Trans_ChangeVar:
            /* Aendere im ComArray einer Transition den Namen einer Variable,
               indem die Makros im bisherigen Namen expandiert werden. */
            {
              Cardinal  com_nr = c->p2;
              char      name[ MAX_IDENT_LENGTH ];

              tr = (M_Transition) ( ((M_Transition)(c->component)) -> numname );

              ExpandMacros( VariableGetStr( tr->vars.ComArray[com_nr].var ), 
                            name );
              tr->vars.ComArray[ com_nr ].var  = StrToVariable( name );
              /* tr->vars.ComArray[ com_nr ].flag wird uebernommen */
              break;
            }
          case Trans_AddAction:
            /* Fuege zum Label einer Transition einen Aktionsterm hinzu. Makros
               im Namen des Aktionssymbols werden expandiert. */
            {
              MultiSet  act   = (MultiSet) c->p1;
              char      name[ MAX_IDENT_LENGTH ];
              Cardinal  old_act_nr;

              tr = (M_Transition) ( ((M_Transition)(c->component)) -> numname );
              ExpandMacros( ActionGetStr( act->nr ), name );

              old_act_nr= act->nr;
              act->nr   = StrToAction( name ) | (act->nr&1);
              MSresize( tr->label, MSlength( tr->label )+1 );
                /* Resizing waere bei doppelten Elementen ueberfluessig. */
              MSsum( tr->label, act );
              if ( GL_AddActions )
                {
                  MSresize( spec_actions, MSlength( spec_actions )+1 );
                    /* Resizing waere bei doppelten Elementen ueberfluessig. */
                  MSsum( spec_actions, act );
                }
              act->nr = old_act_nr;
              break;
            }
          case Trans_Replace:
            /* Ersetze eine Transition durch ein Netz. */
            {
              tr = (M_Transition) ( ((M_Transition)(c->component)) -> numname );
              *net_ptr = M_Replacement( *net_ptr, tr, macro_body );
              break;
            }
          case Trans_Mark:
            /* Trage eine Transition als markiert in das Markierungsarray ein.*/
            {
              tr = (M_Transition) ( ((M_Transition)(c->component)) -> numname );
              trans_mark[ c->p2 ] = tr;
              break;
            }
          case Trans_AddRef:
            {
            /* Fuege zu einer Transition eine Referenz hinzu. */
              tr = (M_Transition) ( ((M_Transition)(c->component)) -> numname );
              
              if (tr->RefEntries)
                tr->ref = MYrealloc( tr->ref, 
                                      (tr->RefEntries + 1) * sizeof (Ref) );
              else
                tr->ref = MYmalloc( sizeof( Ref ) );

              tr->ref[tr->RefEntries].nr   = macro_ref;
              tr->ref[tr->RefEntries].flag = c->p2;
              tr->RefEntries++;
              break;
            }
	  case Trans_Meaning:
	   /* Aendere die Bedeutung (nur $NAME$ als Makro moeglich) */
	   {
              M_Transition tr = (M_Transition)
			(((M_Transition) (c->component))->numname);
              char *name = MYmalloc(strlen(tr->meaning)+strlen(macro_name));

              ExpandMacros(tr->meaning, name);

	      free(tr->meaning);
	      tr->meaning = MYstrdup(name);
	      free(name);
              break;
            }
          case Guard_SetVar:
            /* Wandle einen Variablenknoten mit Makros durch Expansion um in
               einen Variabklenknoten mit Kommunikationsnummer. */
            {
              char     *str = c->p1;
              Cardinal  node_index = c->p3;

              char      name[ MAX_IDENT_LENGTH ];

              tr = (M_Transition) ( ((M_Transition)(c->component)) -> numname );
              ExpandMacros( str, name );
              tr->guard[node_index].op1 = StrToCom( name, &(tr->vars), tr );
              break;
            }
          case Guard_SetConst:
            /* Wandle einen Konstantenknoten mit Wertmakro durch Expansion um 
               in einen Konstantenknoten mit Konstante. */
            {
              Cardinal  value      = *(Integer *)(c->p1);
              Cardinal  node_index = c->p3;

              tr = (M_Transition) ( ((M_Transition)(c->component)) -> numname );
              tr->guard[node_index].op1 = value;
              break;
            }
          case Guard_AddForeach:
            /* Initialisiere das FOREACH-Konstrukt mit der Nummer c->p3.

               foreach_set ist vorher durch die Kommandos Foreach_init, 
               Foreach_SetRange und Foreach_AddValues gesetzt worden.

               Wenn bei der Abarbeitung von Guard_Expand die Prozedur CopyTree
               auf den zugehoerigen FOREACH-Knoten stoesst, greift sie auf
               die hier initialisierten Werte zurueck. */
            {
              foreach[ c->p3 ].set         = foreach_set;
              foreach[ c->p3 ].anz_nodes   = c->p4;
              foreach[ c->p3 ].cmd_entries = c->p2;
              foreach[ c->p3 ].cmd         = c->p1;
              break;
            }
          case Guard_Expand:
            /* Expandiere alle FOREACH-Makros in einem Tree und wandle ihn
               anschliessend in einen Guard um. */
            {
              node_ptr new_guard;
              Integer  anz_nodes;
              Cardinal i;

              tr = (M_Transition) ( ((M_Transition)(c->component)) -> numname );

              /* berechne die Knotenanzahl des expandierten Guards */
              anz_nodes = tr->guard->op2;
              for ( i=0; i<MAX_FOREACHS && foreach[i].set; i++ )
                {
                  if ( *(foreach[i].set) > 2 )
                    /* falls der Teilbaum mindestens einmal wiederholt wird */
                    anz_nodes +=   ( foreach[i].anz_nodes + 1 )
                                   /* Knoten-Anzahl + N_AND-Knoten */
                                 * ( *(foreach[i].set) - 2 ) 
                                   /* Anzahl der Wiederholungen - 1 */
                                 - 1;
                                   /* wegfallender N_MACRO-Knoten */
                }
              new_guard = MYmalloc( anz_nodes * sizeof( node ) );

              new_guard->typ = N_HEAD;
              new_guard->op1 = 1;
              new_guard->op2 = 0;
              CopyTree( tr->guard, tr->guard->op1 - 1, 
                        new_guard, new_guard->op2++, True );

              MYfree( tr->guard );
              tr->guard = TreeToGuard( new_guard, True );
              break;
            }
          case Arc_AddConst:
            /* Fuege eine Konstante (c->p1==NULL) oder den Wert eines Makros
               (c->p1==Makrozeiger) zu einer Kantenbeschriftung hinzu. */
            {
              st = (M_STEdge) ( ((M_STEdge)(c->component)) -> numname );
                /* ST-Kanten und TS-Kanten muessen identisch aufgebaut sein ! */

              if ( c->p1 )
                AddVarToLabel( *(Integer *)(c->p1) + V_CONST_NULL, st );
              else
                AddVarToLabel( c->p2, st );
              break;
            }
          case Arc_AddTupel:
            /* Fuege einen Tupel zu einer Kantenbeschriftung hinzu. */
            {
              Var      *tpl_org = (Var*) (c->p1), *tpl_neu;
              size_t    anz_bytes;

              st = (M_STEdge) ( ((M_STEdge)(c->component)) -> numname );
                /* ST-Kanten und TS-Kanten muessen identisch aufgebaut sein ! */

              anz_bytes = *tpl_org * sizeof( Var );
              tpl_neu = MYmalloc( anz_bytes );
              memcpy( tpl_neu, tpl_org, anz_bytes );

              AddVarToLabel( ArrayToTpl( &tpl_neu ) + V_TUPEL_OFFSET, st );

              break;
            }
          case Arc_AddForeach:
            /* Expandiere ein FOREACH-Konstrukt innerhalb einer Kanten-
               beschriftung. */
            {
              for( macro_index = 1; macro_index < *foreach_set; macro_index++ )
                {
                  macro_element = foreach_set[macro_index];
                  InterpretCmds( c->p1, c->p2, net_ptr );
                }
              MYfree( foreach_set );
              break;
            }
          case Arc_AddVar:
            /* Fuege ein Varible zu einer Kantenbeschriftung hinzu. Makros im
               Variablenname werden expandiert. */
            {
              char      name[ MAX_IDENT_LENGTH ];

              st = (M_STEdge) ( ((M_STEdge)(c->component)) -> numname );
                /* ST-Kanten und TS-Kanten muessen identisch aufgebaut sein ! */
              tr = (M_Transition) ( ((M_Transition)(c->p2)) -> numname);

              ExpandMacros( c->p1, name );
              AddVarToLabel( StrToCom( name, &(tr->vars), tr ), st );

              break;
            }
          case Tupel_SetConst:
            /* Setze eine Komponente eines Tupels auf einen konstanten Wert. */
            {
              Cardinal  value = *(Integer *)(c->p1);
              Cardinal  index = c->p2;
              Var      *tupel = (Var*) c->p3;

              tupel[index] = value + V_CONST_NULL;
              break;
            }
          case Tupel_SetVar:
            /* Setze eine Komponente eines Tupels auf eine Variable. Makros im
               Variablenname werden expandiert. */
            {
              char     *str   = c->p1;
              Cardinal  index = c->p2;
              Var      *tupel = (Var*) c->p3;

              char      name[ MAX_IDENT_LENGTH ];

              tr = (M_Transition) ( ((M_Transition)(c->component)) -> numname );
              ExpandMacros( str, name );
              tupel[index] = StrToCom( name, &(tr->vars), tr );
              break;
            }
          case Foreach_Init:
            /* Initialisiere foreach_set mit einem evtl. unvollstaendigen
               Wertebereich. */
            {
              M_CopySingleRange( &foreach_set, c->p1 );
              break;
            }
          case Foreach_SetRange:
            /* Setze eine Einzelmenge als eindimensionalen Wertebereich 
               foreach_set. */
            {
              Integer  *set   = *(Integer **)(c->p1);

              M_CopySingleRange( &foreach_set, set );
              break;
            }
          case Foreach_AddValues:
            /* Fuege ein Intervall in den eindimensionalen Wertebereich 
               foreach_set ein. */
            {
              Macro_AddValues( &foreach_set, c, True, 0, NULL );
              break;
            }

          default:
            Msg( FAT,"Internal Error: Unknown command type !");
        }
      }
}


/****************************************************************************/
/* Kopiere eine lueckenhafte Schablone und fuelle durch Abarbeiten der      */
/* zugehoerigen Kommandos alle Luecken.                                     */ 
/*                                                                          */
/* Mit forced_load kann zwischen obligatorischen Templates (z.B. Datenbox   */
/* und optionalen Templates (z.B. Initnetz) unterschieden werden.           */
/****************************************************************************/

M_Net GetBox( Cardinal nr, bool forced_load )
  { 
    M_Net        net;

    net = GetTemplate( nr, forced_load );
    if ( net )
      {
        net = M_CreateNetCopy( net );
        InterpretCmds( tmpl[nr].cmd   , tmpl[nr].cmd_entries   , &net );
        InterpretCmds( tmpl[nr].netcmd, tmpl[nr].netcmd_entries, &net );
      }
    return net;
  }
