/****************************************************************************/
/* macros.h                                                                 */
/*                                                                          */
/* Martin Ackermann, Winter 96/97                                           */
/****************************************************************************/

#ifndef __MACROS_H__
#define __MACROS_H__

/****************************************************************************/
/* Namen der Schablonen im tmpl-Arrray                                      */
/****************************************************************************/

/* these numbers MUST correspond with the tmpl-array */

enum {
	VAR_BOX,
	VAR_INIT,
	VAR_TERM,
	INITVAR_BOX,
	INITVAR_INIT,
	INITVAR_TERM,
	CHAN0_BOX,
	CHAN0_INIT,
	CHAN0_TERM,
	CHAN1_BOX,
	CHAN1_INIT,
	CHAN1_TERM,
	CHAN_BOX,
	CHAN_INIT,
	CHAN_TERM,
	STACK_BOX,
	STACK_INIT,
	STACK_TERM,
	PROC_BOX,
	PROC_BOX_SET,
	PROC_INIT,
	PROC_TERM,
	PROC_CALL,
	VAL_PAR,
	CONST_PAR,
	RES_PAR,
	VARREF_PAR,
	VARREF_PAR_REC,
	CHAN0REF_PAR,
	CHAN0REF_PAR_REC,
	CHANREF_PAR,
	CHANREF_PAR_REC,

	SDL_CHANNEL,
	SDL_SIGNALROUTE,
	SDL_INIT,
	SDL_TERM,
	SDL_CREATE,
	SDL_CONNECT,
	SDL_PROCESS,
	SDL_PROCESS_EXIT,
	SDL_VARIABLE,
	SDL_VAR_INIT,
	SDL_VAR_TERM,
	SDL_PROCEDURE,
	SDL_PROC_CALL,
	SDL_PRCD_IN_PARM,
	SDL_PRCD_INOUT_PARM,
	SDL_PROC_CREATE,
	SDL_LABEL,
	SDL_JOIN
};


/****************************************************************************/
/* Namen der verschiedenen Kommandos                                        */
/****************************************************************************/

#define Place_SetRange     1
#define Place_AddValues    2
#define Place_AddRef       3
#define Place_Mark         4
#define Place_Meaning      5

#define Trans_SetRange    11
#define Trans_AddValues   12
#define Trans_ChangeVar   13
#define Trans_AddAction   14
#define Trans_Replace     15
#define Trans_Mark        16
#define Trans_AddRef      17
#define Trans_Meaning	  18

#define Arc_AddConst      21
#define Arc_AddTupel      22
#define Arc_AddForeach    23
#define Arc_AddVar        24

#define Tupel_SetConst    31
#define Tupel_SetVar      32

#define Foreach_SetRange  41
#define Foreach_AddValues 42
#define Foreach_Init      43

#define Guard_SetConst    51
#define Guard_AddForeach  52
#define Guard_Expand      53
#define Guard_SetVar      54

/****************************************************************************/
/* Namen der Markierungen fuer Transitionen                                 */
/****************************************************************************/

#define MARK_PAR_PRE       0 /* these numbers must correspond with the... */
#define MARK_PAR_POST      1 /* ...trans_mark-array                       */
#define MARK_CALL_PAR      2
#define MARK_RETURN_PAR    3
#define NUM_T_MARKS	   4


/****************************************************************************/
/* Namen der Markierungen fuer Stellen                                      */
/****************************************************************************/

#define MARK_REF_PAR       0 /* these numbers must correspond with the... */
#define NUM_P_MARKS	   1 /* ...place_mark-array                       */


/****************************************************************************/
/* sonstige Defines                                                         */
/****************************************************************************/

#define MAX_FOREACHS      10  
                    /* maximal zulaessige Anzahl von FOREACH-Makros pro Guard */


/****************************************************************************/
/* Datenstrukturen fuer Kommandos                                           */
/****************************************************************************/

typedef struct Cmd_ {
  void      *component; /* Netzkomponente, auf die sich das Kommando bezieht  */
  Cardinal   cmd_nr;    /* Art des Kommandos                                  */
  void      *p1;        /* Parameter 1: Typ / Bedeutung haengen von cmd_nr ab */
  Cardinal32 p2;        /* Parameter 2: Typ / Bedeutung haengen von cmd_nr ab */
  Cardinal32 p3;        /* Parameter 3: Typ / Bedeutung haengen von cmd_nr ab */
  Cardinal32 p4;        /* Parameter 4: Typ / Bedeutung haengen von cmd_nr ab */
} Cmd;

typedef struct Foreach_data_ {
  Integer  *set;         /* Wertebereich des FOREACH-Konstrukts               */
  Cardinal  anz_nodes;   /* Anzahl der Knoten des FOREACH-Teilbaums           */
  Cmd      *cmd;         /* Kommandos, die sich auf den FOREACH-Teilbaum bez. */
  Cardinal  cmd_entries; /* Anzahl dieser Kommandos                           */
} Foreach_data;

typedef struct Template_ {
  char      *file;/* Name der Konfigurationsdatei, die das Template liefert */
  M_Net      net;                          /* lueckenhafte M-Netz-Schablone */
  Cmd       *cmd;        /* lokale Kommandos, welche die Luecken auffuellen */
  Cardinal   cmd_entries;                        /* Anzahl dieser Kommandos */
  Cmd       *netcmd;        /* Kommandos, welche die Adresse von Netzkompo-
                               nenten veraendern und deshalb nach allen 
                               anderen Kommandos ausgefuehrt werden muessen
                                                 (bisher nur Trans_Replace) */
  Cardinal   netcmd_entries;                     /* Anzahl dieser Kommandos */
} Template;


/****************************************************************************/
/* exportierte globale Variablen                                            */
/****************************************************************************/

extern char          path_to_templates[FILENAME_MAX];
                      /* Pfad, aus dem Konfigurationsdateien gelesen werden */
extern int           p_t_t_len;     /* Laenge des Strings path_to_templates */

extern Foreach_data  foreach[];            /* Array mit FOREACH-Konstrukten */

extern Integer       dot_set[];         /* Wertebereich nur mit Element dot */

extern M_Transition  trans_mark[];     /* Array mit markierten Transitionen */
extern M_Place       place_mark[];     /* Array mit markierten Stellen      */

extern Template	     tmpl[];

bool bpn_templates;

/****************************************************************************/
/* aktuelle Belegung der verschiedenen Makros                               */
/****************************************************************************/

extern Integer      *macro_decl_set,
                    *macro_id_set,
                    *macro_new_ids,
                    *macro_senders,
                    *macro_senders2,
		    *macro_signals;

extern Integer       macro_max,
                     macro_capacity,
                     macro_capacity_plus_eins,
                     macro_init,
                     macro_index,
                     macro_element,
		     macro_baseid;

extern char         *macro_name,
                    *macro_arg,
                    *macro_ext;

extern RefNr         macro_ref;
extern M_Net         macro_body;


/****************************************************************************/
/* exportierte Funktionen                                                   */
/****************************************************************************/

M_Net     GetBox( Cardinal nr, bool forced_load );
void      RemoveTemplates( void );
Cmd      *ExtractLastCmds( Cardinal anz );
void      CorrectIndices( Var com_nr );
void      CounterIndex2NodeIndex ( Cardinal cmd_nr, Cardinal counter_index, 
                                   Cardinal node_index );
Cmd      *Add_Cmd(   Cardinal cmd_code, void *p1, Cardinal32 p2, Cardinal32 p3);
Cmd      *Add_NetCmd(Cardinal cmd_code, void *p1, Cardinal32 p2, Cardinal32 p3);
Cmd      *GetFirstCmd( void );
Cmd      *GetNextCmd( void );
Cardinal  GetCmdEntries( void );
void      InterpretCmds( Cmd *c, Cardinal cmd_entries, M_Net *net_ptr );

#endif /* __MACROS_H__ */


/*****************************************************************************/
/* END OF FILE MACROS.H                                                      */
/*****************************************************************************/
