/****************************************************************************/
/* main.c                                                                   */
/*                                                                          */
/* Burkhard Graves, 10.10.93                                                */
/* Lars Jenner, Juni 1994                                                   */
/* Martin Ackermann, Fruehjahr 95                                           */
/* Burkhard Bieber 1995/96:                                                 */
/*   - Blockbehandlung ergaenzt                                             */
/*   - Phantomtransitionsbehandlung ergaenzt                                */
/*   - Schoenheitskorrekturen                                               */
/* Martin Ackermann, Winter 1996/97                                         */
/*   - Prozeduren, andere Referenzen, mehr Effizienz, Fehler beseitigt      */
/* Stefan Schwoon, November 1997: -C, -P				    */
/****************************************************************************/

  /************/
  /* includes */
  /************/

#include <string.h>
#include <time.h>
#include <ctype.h>
#include <limits.h>

#include "main.h"
#include "multiset.h"
#include "mnet.h"
#include "tables.h"
#include "macros.h"
#include "block.h"
#include "evaltree.h"
#include "compose.h"
#include "readHL.h"
#include "f_parse.h"
#include "box_sub.h"

  /*******************/
  /* Datenstrukturen */
  /*******************/

Cardinal var_style = VAR_BOTH;   /* Ausgabe-Stil f. Kommunikationsvariablen */

char *SrcBuffer = NULL;          /* Buffer fuer den Quelltext               */
char **SrcLine;                  /* Array von Zeigern auf die einzelnen 
                                    Zeilenanfaenge im Quelltext             */

Cardinal scale = 1,              /* Skalierungsfaktor fuer den Netz-Graphen */
         max_arc_vars = 2,       /* maximale Anzahl von Variablen in einem 
                                                                Kantenlabel */
         max_proc_inst = 2;      /* maximale Anzahl von Aktivierungen einer
                                                                   Prozedur */
Cardinal32 flag;                 /* globale Programm-Flags                  */

char    *source,                 /* Quelldatei b(pn)2 bzw. bei syncmnet
                                    das zu synchronisierende Netz           */
        *actions_string;         /* die beim Aufruf von syncmnet ggf. 
                                    spezifizierten Aktionssymbole           */
char    *exclude_string = NULL;  /* ggf. auszuscliessende Aktionssymbole    */
char    *pep_file = NULL;        /* Ausgabedatei im Standard-PEP-Format     */
char    *source_ref = NULL;      /* Referenzen fuer das Eingabe-Netz        */

extern int yyparse();

  /************************/
  /* Aufruf demonstrieren */
  /************************/

static void usage(char *myname)
{
  fprintf(stderr, 
    "\nUsage: %s source [-p=fname] [-z=fname] [-y=f_name] [-Z=fname]\n%*c",
    myname, ((int) strlen(myname))+15, ' ');
  fprintf(stderr, "[-j[=action[,action..]]] [-X[=action[,action..]]]\n%*c", 
	  ((int) strlen(myname))+15, ' ');
  fprintf(stderr, "[-obxsretlqnmhaigu!vfckw]\n\n");
  fprintf(stderr, " z: specify resulting reference file\n");
  fprintf(stderr, " p: specify resulting PEP-file\n");
  fprintf(stderr, " o: disable warnings at old syntax\n");
  fprintf(stderr, " b: named blocks in output\n");
  fprintf(stderr, " x: phantom transitions in output\n");
  fprintf(stderr, "s#: specify scale\n");
  fprintf(stderr, " r: remove duplicates\n");
  fprintf(stderr, " e: eliminate dead transitions\n");
  fprintf(stderr, " t: use parallel termination\n");
  fprintf(stderr, " l: big loop\n");
  fprintf(stderr, " q: medium loop\n");
  fprintf(stderr, " n: use bpn-names for communication variables\n");
  fprintf(stderr, "m#: specify maximum number of transitions\n");
  fprintf(stderr, " h: make PEP-file shorter\n");
  fprintf(stderr, " a: allow b(pn)2-fragments\n");
  fprintf(stderr, "i#: specify default number of procedure instances\n");
  fprintf(stderr, " g: generally use procedure semantic\n");
  fprintf(stderr, " u: use set for procedure identifiers instead of stack\n");
  fprintf(stderr, " !: no scoping\n");
  fprintf(stderr, " J: no disjunct idsets\n");
  fprintf(stderr, "C#: specify which channel box to use\n");
  fprintf(stderr, "P#: specify which procedure box to use\n\n");
  fprintf(stderr, " ---ONLY IN SYNCHRONIZE MODE---\n");
  fprintf(stderr, " j: enter synchronize mode with specified actions\n");
  fprintf(stderr, "    (use -j to specify ALL actions)\n");
  fprintf(stderr, " d: do restriction after synchronization\n");
  fprintf(stderr, " Z: specify reference file for source net\n");
  fprintf(stderr, " X: exclude specified actions from synchronization\n\n");
  fprintf(stderr, " ---ONLY FOR DEBUGGING PURPOSES---\n");
  fprintf(stderr, " y: specify directory with box templates\n");
  fprintf(stderr, "v#: specify maximum number of variables in an arc label\n");
  fprintf(stderr, " f: free memory explicitly\n");
  fprintf(stderr, " c: check resulting net\n");
  fprintf(stderr, " k: check for conflicting variable names\n");
  fprintf(stderr, " w: warn if different sets are intersected\n"); 
  fprintf(stderr, "version 2.0 (20.02.2002)\n\n");
  exit(1);
}

/*
-p=fname: Ausgabe des Netzes im PEP-Format in die Datei 'fname'
-z=fname: Ausgabe der Referenz-Tabelle     in die Datei 'fname'
-y=fname: Vorgabe eines Verzeichnisses mit zu benutzenden BOX-Templates
          (Standardmaessig werden die Templates aus 
           $PEPHOME/lib/bpn2mnet/no_procs.hl/  bzw.
           $PEPHOME/lib/bpn2mnet/___procs.hl/  gelesen.)
-o:       Keine Warnungen ausgeben bei alter Syntax (Wertebereichen und 
          Aktionen mit eckigen Klammern [...]) 
-a:       b(pn)2-Fragmente akzeptieren (nur Deklarationen bzw. nur Anweisungen)
-b:       Ausgabe der Named-Blocks nicht vornehmen
-x:       Ausgabe der Phantomtransitionen und zugehoerigen Infos nicht vornehmen
          (Dazu gehoeren auch Infos ueber Synchronisationstransitionen.)
-s#:      Vorgabe des Skalierungsfaktors (DEFAULT = 1)
          (Der Skalierungsfaktor vergroessert den Abstand zwischen Knoten 
           im Netz-Graphen.)
-r:       Entfernen von Transitionen bzw. Stellen mit identischer Beschriftung
          UND identischem Vor- & Nachbereich
          (Stellen werden nur betrachtet, wenn sie mit 'dot' beschriftet sind.)
-e:       Entfernen von Transitionen, die nie schalten koennen
          (Das kostet in der Regel Uebersetzungszeit, bringt jedoch 
           insbesondere bei Kanaelen mit Kapazitaet 0 erhebliche Einsparungen.)
-t:       Erzwingen der parallelen Anordnung der Terminierungsnetze von 
          Datenboxen gemaess theoretischer Vorlage
          (Standardmaessig wird die erheblich effektivere sequentielle 
           Terminierung verwendet.)
-l:       Erzwingen des komplizierten LOOP-Konstrukts gemaess theoretischer
          Vorlage
          (Standardmaessig wird eine stark vereinfachte Variante verwendet.)
-q:       Verwendung einer leicht vereinfachten LOOP-Variante (siehe -l)
-n:       Beibehalten der Originalnamen fuer Kommunikationsvariablen
          (Gemaess theoretischer Vorlage muessen bei der Synchronisation
           Kommunikationsvariablen umbenannt werden, was zu kryptischen
           Bezeichnungen fuehrt.
           Bei Netzen, die aus b(pn)2-Programmen entstanden sind, ist diese
           Umbenennung ueberfluessig und kann mit -n ausgeschaltet werden.
           Bei allgemeineren Netzen kann die Angabe die Angabe der Option -n
           jedoch zu Fehlern fuehren.)
-h:       Fuer das PEP-file wird ein verkuerztes Format verwendet
          (Referenzen und Quelltext-Meanings werden in einer Symboltabelle
           angehaengt. In den Transitionen- & Stellen-Definitionen steht dann
           nur noch der Index in die Symboltabelle.
           Bei den Transitionen- und Stellen-Definitionen wird die Numerierung 
           weggelassen.)
-i#:      Vorgabe der maximalen Anzahl von gleichzeitigen Aktivierungen EINER 
          Prozedur (DEFAULT = 2)
          (Diese Vorgabe kann fuer einzelne Prozeduren ueberschrieben werden
           durch Angabe von 'proc pname [(arg_list)] max const'.)
-m#:      Vorgabe der maximalen Anzahl von Transitionen
          (Um die Berechnung von sehr grossen Netzen zu verhindern, z.B. auf
           Rechnern mit weniger RAM, kann man die maximale Anzahl von 
           Transitionen begrenzen.)
-g:       Prozedur-Semantik auf jeden Fall verwenden
          (Ohne -g benutzt bpn2mnet bei Programmen ohne Prozeduren eine
           vereinfachte Semantik.)
-u:       Anstelle eines Stacks wird fuer die Verwaltung der Prozedurinstanzen
          eine ungeordnete Menge verwendet
-!:       Abschalten des Scoping zu Lern- & Demonstrationszwecken
          (Zwischen den Netzen fuer Datenboxen und Programmanweisungen werden
           keine Verknuepfungen hergestellt.)
-v#:      Vorgabe der maximalen Anzahl von Variablen in einem Kantenlabel
          (DEFAULT = 1)
          (Damit keine unendlich grossen Netze entstehen bzw. Speicherueber-
           laeufe auftreten, wird eine Transition geloescht, wenn beim
           Synchronisieren eine ihrer Kanten die vorgegebene Variablenanzahl 
           ueberschreiten wuerde.)
-f:       Einschalten einer internen Speicherverwaltung
          (So koennen Fehler bei der Speicherverwendung aufdeckt werden; der
           Uebersetzer wird allerdings extrem verlangsamt.)
-c:       Ueberpruefen der internen Verzeigerung des Ergebnis-Netzes
-k:       Warnung ausgeben bei moeglich Fehlern, die durch -n entstanden sind
          (Diese Option ist nuetzlich, um bei neuartigen MACRO-Boxen Konflikte
           bei der Benennung von Variablen zu entdecken.
           ACHTUNG: Die Uebersetzung wird SEHR ineffizient !)
-w        Warnung ausgeben, falls eine Variable an verschiedene Wertebereiche
          gebunden wird
          (Ist bisher nur bei Stack- / Prozedur-Boxen der Fall.)
-C#:      Vorgabe der Channel-Box (default 0). Es wird einfach der Name
	  in der Makro-Tabelle geaendert.
-P#:      Vorgabe der Procedure-Box (default 0). dito.
-J:       Keine disjunkten Identifier-Sets
*/

  /*****************/
  /* Zahl einlesen */
  /*****************/
static Cardinal read_card(char *ptr, char *myname)
{
  Cardinal zahl;

  if (!isdigit(*++ptr))
    usage(myname);
  zahl = 0;
  do {
    zahl *= 10;
    zahl += *ptr - '0';
  } while (isdigit(*++ptr));
  return zahl;
}

  /******************/
  /* Namen einlesen */
  /******************/
static char *read_name(char *ptr1, char *myname)
  /* Namen fangen immer mit "=" gleich an! */
{
  if (*++ptr1 != '=' || !*++ptr1)
    usage(myname);
  return ptr1;
}

void vchange(int num, char c)
{
	tmpl[num].file = strdup(tmpl[num].file);
	*(strchr(tmpl[num].file,'.')-1) = c;
}

  /*******************/
  /* "flag" einlesen */
  /*******************/
static void read_flags(char *ptr, char *myname)
{
  while (*++ptr) {
    switch(*ptr) {
      case 'v':   max_arc_vars=read_card(ptr,myname);
                  return;
      case 's':   scale = read_card(ptr, myname);
                  return;
      case 'i':   max_proc_inst = read_card(ptr, myname);
                  return;
      case 'm': { max_trans_anz = read_card(ptr, myname);
                  return;
                }
      case 'o':   GL_OldSyn = True; break;
      case 'p':   pep_file = read_name(ptr, myname);
                  return;
      case 'y':   strcpy( path_to_templates, read_name(ptr, myname) );
                  p_t_t_len = strlen( path_to_templates );
                  return;
      case 'z':   ref_file = read_name(ptr, myname);
                  return;
      case 'b':   flag |= BLOCK_OUTPUT; break;
      case 'x':   flag |= PHANTOM_TRANS_OUTPUT; break;
      case 'c':   flag |= CHECK_NET; break;
      case 'f':   GL_ExplFree = True; break;
      case 'r':   flag |= REMOVE_DUPL; break;
      case 't':   flag |= PAR_TERM; break;
      case 'e':   flag |= ELIMINATE_DEAD; break;
      case 'l':   flag |= BIG_LOOP; break;
      case 'q':   flag |= MEDIUM_LOOP; break;
/*    case 'N':   var_style = VAR_REAL; break;
*/    case 'n':   var_style = VAR_SAFEREAL; break;
      case 'k':   flag |= CHECK_NAMES; break;
      case 'h':   flag |= SHORT_FILE; break;
      case 'a':   GL_Frags = True; break;
      case '!':   flag |= NO_SCOPING; break;
      case 'g':   GL_ProcDecl = True; break;
      case 'u':   flag |= USE_PROC_SET; break;
      case 'w':   flag |= INTERSECT_WARNING; break;
      case 'C':   if (!*(ptr+1)) break;
		  ptr++;
		  vchange(CHAN_BOX,*ptr);
		  vchange(CHAN_INIT,*ptr);
		  vchange(CHAN_TERM,*ptr);
		  break;
      case 'P':   if (!*(ptr+1)) break;
		  ptr++;
		  vchange(PROC_BOX,*ptr);
		  vchange(PROC_BOX_SET,*ptr);
		  vchange(PROC_INIT,*ptr);
		  vchange(PROC_TERM,*ptr);
		  vchange(PROC_CALL,*ptr);
		  break;
      case 'J':	  flag |= NO_DISJUNCT_IDSETS; break;
      case 'd':   flag |= MNET_RESTRICTION; break;
      case 'Z':   source_ref = read_name(ptr, myname);
                  return;
      case 'j':   flag |= MNET_SYNCHRONIZE;
                  if ( ptr[1] == '=') 
                    {
                      actions_string=read_name(ptr,myname);
                      return;
                    }
                  else 
                    {
                      actions_string=NULL;
                      break;
                    }
      case 'X':   exclude_string=read_name(ptr,myname);
                  return;
      default: usage(myname);
    }
  }
}

  /*********************************/
  /* Hauptprogramm                 */
  /*********************************/

int main(int argc, char *argv[])
{
  clock_t time;
  int i, ret_value;

  bpn_templates=True;

  /* Aufrufoptionen der Reihe nach einlesen und behandeln */
  source=NULL;
  flag=0;
  for (i=1; i<argc; ++i)
    if (*(argv[i])!='-')
      if (!source)
        source=argv[i];
      else
        usage(argv[0]);
    else
      read_flags(argv[i],argv[0]);
  
  if (!source)
    usage(argv[0]);

  ret_value = 0;
  if ( !(flag & MNET_SYNCHRONIZE ))
    {
      if ((yyin=fopen(source,"r"))==NULL) {
        fprintf(stderr,"\nError: Could not open file %s !\n\n",
                source);
        exit(2);
      }

#ifdef DEBUG
      printf("\nParsing program %s ...\n",source);
  
      {
        extern int yydebug;
        yydebug=1;
      }
#endif

      GL_ParserMsg = True;
      ret_value = bpn2parse( yyin, yyout, stdout );
      GL_ParserMsg = False;
      fclose ( yyin );
    }

  if ( !ret_value ) {

    M_Net net;

    InitMultiSets();
    InitVarMSets();
    InitAllSymbolTables();
    InitBlockTable();

    if ( flag & MNET_SYNCHRONIZE )
      {
	int i;

        /* Falls keine Aktionssymbole spezifiziert wurden, laeuft die 
           Synchronisation und die Restriktion ueber alle Aktionssymbole */
        if ( !actions_string )
          {
            GL_AddActions = True;   
            spec_actions = MScreate_empty_set();
          }
        
	printf("\nReading source net %s ...\n",source);
	net = read_HLnet( source, source_ref );

      if (flag&CHECK_NET) {
        printf("Checking consistency of source net: ");
        if ((i=M_CheckNet(net))!=0) {
	  printf("inconsistent (%i)!\n\n",i);
	  exit(5);
        } else
	  printf("OK !\n\n");
      }

        if ( actions_string ) 
          spec_actions = StrToMultiSet( actions_string, NULL );
        
	if ( exclude_string ) {
	  MultiSet exclude_actions;
	  register MultiSet mmptr;

	  mmptr = spec_actions;
	  while ( mmptr->anzahl )
	    {
	      mmptr->anzahl=1;
	      mmptr++;
	    }
	  
	  GL_AddActions = False;
	  exclude_actions = StrToMultiSet( exclude_string, NULL );
	  
	  MSweakdifference( spec_actions, exclude_actions );
	  
	  mmptr = exclude_actions;
	  while ( mmptr->anzahl )
	    {
	      mmptr->nr^=1;
	      mmptr++;
	    }
	  MSweakdifference( spec_actions, exclude_actions );
	  
	  MSremove( exclude_actions );
	}

	printf("Creating resulting net ...\n");
	time=clock();

        if ( !(flag&NO_SCOPING) )
          M_Synchronisation( net, spec_actions, max_arc_vars );
        
	if ( flag&MNET_RESTRICTION )
          M_Restriction( net, spec_actions );
        
        MSremove( spec_actions );
      }
    else
      {
	printf("\nCreating resulting net ...\n");
	time=clock();

	net = Block2Net( tree );
      }

    HandleNet( net );

    printf("Creation complete (%f sec).\n\n",
           (float)(clock()-time)/CLOCKS_PER_SEC);

    /* bei eingeschalteter Speicherueberwachung alle angeforderten Seiten 
       explizit freigeben */
    if ( GL_ExplFree ) {

      if ( SrcBuffer ) {
        MYfree( SrcBuffer );
        MYfree( SrcLine   );
      }
      
      RemoveAllSymbolTables();
      RemoveBlockTable();

      if ( !(flag & MNET_SYNCHRONIZE) )
        {
          RemoveTemplates();
          delete_tree( tree );
        }

      M_Remove_net( &net );

      MYfree( mmDummy ); MYfree( mmLeaf ); MYfree( mmSingle ); MYfree( mmSub );
      MYfree( V_mmDummy ); MYfree( V_mmLeaf );
      
    }
  }

  return ret_value;
}
