/****************************************************************************/
/* mnet.h                                                                   */
/*                                                                          */
/* Defines und Datentypen fuer die interne Darstellung von M-Netzen         */
/*                                                                          */
/* Burkhard Graves, 26.6.'93                                                */
/* Martin Ackermann, Sommer 95                                              */
/* Burkhard Bieber 1995:                                                    */
/*   - Blocktypen ergaenzt                                                  */
/*   - Sync-/Phantom-Transitionen ergaenzt                                  */
/* Martin Ackermann, Winter 1996/97                                         */
/*   - Prozeduren, andere Referenzen, mehr Effizienz, Fehler beseitigt      */
/****************************************************************************/

#ifndef __MNET_H__
#define __MNET_H__

  /************/
  /* includes */
  /************/

#include "var_mset.h"

  /*********************/
  /* Typen und defines */
  /*********************/

  /* Platzhalter fuer uninitialisierte Wertebereiche in Kreuzprodukten */
#define EMPTY_SET ((Integer*) -1)

  /* Raster fuer Knoten im Netz */
#define RASTER 30

  /* Wenn Namen von Kommunikationsvariablen mit diesem Zeichen beginnen,
     werden sie unabhaengig von var_style umbanannt bei der Ausgabe */
#define VAR_RENAME_CHAR '~'

  /* Wenn Namen von Kommunikationsvariablen mit diesem Zeichen beginnen,
     gehen sie bei einer Substitution zugunsten des anderen Namens verloren */
#define VAR_MINOR_CHAR '-'

  /*****************************/
  /* Beschriftung einer Stelle */
  /*****************************/
typedef char PLabel;

  /* Achtung: Bits nicht doppelt belegen ! */
#define EMPTY    0
#define ENTRY    1
#define EXIT     2
#define INTERN1  4
#define INTERN2  8

  /**************************/
  /* Flags von Transitionen */
  /**************************/
/* Achtung: Bits nicht doppelt belegen ! */
#define TR_MOVE_HORIZ    1  /* wird ggf. nach rechts verschoben     */
#define TR_MOVE_VERT     2  /* wird ggf. nach oben   verschoben     */
#define TR_CHAN0_ACCESS  8  /* synchronisiert einen Channel Kap. 0  */
#define TR_REF_REC      16  /* reicht einen rekursiven RefPar durch */
#define TR_REPEAT       32  /* fuer das Repeat in einer do-od-Box   */
#define TR_SCOPE        64  /* steht im Kontrollfluss.              */

  /********************/
  /* Flags von Kanten */
  /********************/
/* Achtung: Bits nicht doppelt belegen ! */
#define EDGE_INV         4  /* wird im Netzeditor nur bedingt angezeigt */

  /*********************/
  /* Flags von Stellen */
  /*********************/

  /***********************************/
  /* Datentyp Binaerbaum fuer Guards */
  /***********************************/
#define N_HEAD   0
#ifndef N_AND
  #define N_AND    101
  #define N_OR     102
  #define N_NOT    103
  #define N_CONST  104
  #define N_VAR    105
  #define N_KLAMM  106
  #define N_EQ     107
  #define N_UNEQ   108
  #define N_LT     109
  #define N_GT     110
  #define N_LE     111
  #define N_GE     112
  #define N_PLUS   113
  #define N_MIN    114
  #define N_MULT   115
  #define N_DIV    116
  #define N_MOD    117
  #define N_REM    118
  #define N_SIGN   119
#endif

typedef struct node_struct {
  Cardinal typ;  /* Knotentyp                  */
  Integer  op1;  /* Bedeutung von op1 & op2... */
  Integer  op2;  /* ...ist abhaengig vom Typ   */
} node, *node_ptr;

  /******************************/
  /* Referenzen (auf Quelltext) */
  /******************************/
/* Achtung: Bits nicht doppelt belegen ! */
#define    PRE_REF     1
#define   POST_REF     2
#define    VAL_REF     4
#define  ENTRY_REF     8
#define   EXIT_REF    16
#define   INIT_REF    32
#define   TERM_REF    64
#define ACCESS_REF   128
#define   READ_REF   256
#define  WRITE_REF   512
#define   FULL_REF  1024
#define  EMPTY_REF  2048
#define  SHIFT_REF  4096
#define   CALL_REF  8192
#define RETURN_REF 16384

typedef Cardinal RefNr; /* Index einer Referenz in der ref-Symboltabelle */

typedef struct Ref_ {
  RefNr    nr;
  Cardinal flag;
} Ref;

  /******************/
  /* Typ der Bloecke */
  /******************/
typedef Cardinal BlockNr;

  /* Blocknamen fuer begin-end */
typedef struct BlockStruct_ {
  char    *blockname;         /* Name des Blockes                           */
  Integer  blockdeep;         /* Tiefe innerhalb der Blockhierarchie ab 0   */
  Cardinal blocktags;         /* Spezialitaeten fuer Bloecke                */
  Cardinal childcount;        /* Anzahl der direkt tiefer liegenden Bloecke */
  Cardinal childmax;          /* # derzeitig max. Speicherplaetze in childs */
  BlockNr *children;          /* die direkten Nachfolger als Blocknummern   */
  RefNr    ref;               /* Referenz auf das zugehoerige BEGIN         */
} BlockStruct, *BlockPtr;

  /**********************************************************************/
  /* Eintraege im ComArray                                              */
  /* (geben alle Variablen an, die von einer Transition benutzt werden) */
  /**********************************************************************/

  /* Achtung: Bits nicht doppelt belegen ! */
#define VAR_RENAME 1 /* Bedeutung siehe VAR_RENAME_CHAR */
#define VAR_MINOR  2 /* Bedeutung siehe  VAR_MINOR_CHAR */

typedef struct ComArrayEntry_ {
  Var     var;
  char    flag;
} ComArrayEntry;

  /**********************************************/
  /* Struktur zur uebersichtlicheren Gliederung */
  /**********************************************/
typedef struct VariableStruct_ {
  Cardinal       ComEntries;  /* Anzahl der Eintraege im ComArray  */
  ComArrayEntry *ComArray;    /* Liste  der auftretenden Variablen */
} *Variables, VariableStruct;

  /**********/
  /* Stelle */
  /**********/
  #define MAX_DIM 4  /* max. Dim. f. Vektoren im Wertebereich */
                     /* Muss mindestens 4 betragen !!!        */

typedef struct M_PlElem_ {
  struct M_PlElem_ *prevplace,    /* Zur (doppelten) Verkettung in der      */
                   *nextplace;    /*   Stellenliste.                        */
  struct M_TSElem_ *preset;       /* Zeiger auf erste Kante des Vorbereichs */
  struct M_STElem_ *postset;      /*   bzw. des Nachbereichs.               */
  struct M_TSElem_ *ppreset;      /* Erste Kante im Vorbereich (PhantomTr)  */
  struct M_STElem_ *ppostset;     /* Erste Kante im Nachbereich (PhantomTr) */
  PLabel            label;        /* Label der Stelle                       */
  Integer*          range[MAX_DIM];   /* Wertebereich der Stelle            */
  Cardinal32        numname;      /* Numerischer Name fuer div. Zwecke.     */
  Cardinal          component;    /* Zeigt Komponentenzugehoerigkeit an.    */
  Cardinal          RefEntries;   /* Anzahl der Eintraege in ref            */
  Ref              *ref;          /* Array mit Quelltext-Referenzen         */
  Cardinal          BlockEntries; /* Anzahl der Eintraege in blocks         */
  BlockNr          *blocks;       /* Array mit den Verweisen auf den Block  */
  int               x_pos,
                    y_pos,
                    flags;
  char             *init_marking,
                   *curr_marking,
                   *meaning;
} M_PlElem, *M_Place;

  /**************/
  /* Transition */
  /**************/

typedef struct {
  double low;
  double high;
} Time_stamp;

typedef struct M_TrElem_ {
  struct M_TrElem_ *prevtrans,    /* Zur (doppelten) Verkettung in der       */
                   *nexttrans;    /*   Transitionenliste                     */
  struct M_STElem_ *preset;       /* Zeiger auf erste Kante des Vorbereichs  */
  struct M_TSElem_ *postset;      /*   bzw. des Nachbereichs                 */
  MultiSet          label;        /* Beschriftung der Transition             */
  Integer          *def;          /* Array mit Wertebereichen von Variablen  */
  node_ptr          guard;        /* Array mit Knoten von mehreren Baeumen   */
  VariableStruct    vars;
  Cardinal32        numname;      /* Numerischer Name fuer div. Zwecke       */
  Cardinal          RefEntries;   /* Anzahl der Eintraege in ref             */
  Ref              *ref;          /* Array mit Quelltext-Referenzen          */
  Cardinal          BlockEntries; /* Anzahl der Eintraege in blocks          */
  BlockNr          *blocks;       /* Array mit den Verweisen auf den Block   */
  Cardinal          PhantomEntries;  /* Anzahl der Eintraege in phantoms     */
  struct M_TrElem_ *(*phantoms);  /* Array mit Phantom-/Sync-Transitionen    */
  int               x_pos,
                    y_pos,
                    flags;
  char             *meaning;
  char             *links;
  Time_stamp       *time_label;
} M_TrElem, *M_Transition;

  /********************/
  /* allgemeine Kante */
  /********************/
#define _KANTENTYP_( name )                                                   \
struct name {                                                                 \
    struct name    *prevplace, /* Zur (doppelten, vertikalen) Verkettung   */ \
                   *nextplace, /*   in Bezug auf die Stellenliste.         */ \
                   *prevtrans, /* Zur (doppelten, horizontalen) Verkettung */ \
                   *nexttrans; /*   in Bezug auf die Transitionenliste.    */ \
    M_Place         place;     /* Zeiger auf den Ursprung                  */ \
    M_Transition    trans;     /*   bzw. auf das Ziel der Kante.           */ \
    VarMSet         label;     /* Label der Kante.                         */ \
    Cardinal32      numname;   /* Numerischer Name fuer div. Zwecke.       */ \
    Cardinal        flags;                                                    \
}

  /*****************************/
  /* Kante Stelle-->Transition */
  /*****************************/
typedef _KANTENTYP_( M_STElem_ ) M_STElem, *M_STEdge;

  /*****************************/
  /* Kante Transition-->Stelle */
  /*****************************/
typedef _KANTENTYP_( M_TSElem_ ) M_TSElem, *M_TSEdge;

  /***********************/
  /* Block (Named-Block) */
  /***********************/
typedef struct M_BlockElem_ {
  struct M_BlockElem_ *prevblock,  /* Doppelt verkette Liste der            */
    *nextblock;                    /* Blockelemente                         */
  BlockNr blocknr;                 /* Index des BlockStruct's in dem        */
                                   /* BlockTable                            */
  Cardinal32 numname;              /* Numerischer Name fuer div. Zwecke.    */
  int x_min, y_min, x_max, y_max;  /* Hilfsvar. zur Positionsbestimmung     */
  int x_pos,                       /* Bildschirmposition, wird in           */
    y_pos;                         /* M_CreateBlockElements erzeugt         */
  int ever_used;                   /* Flag fuer Benutzung des Eintrages     */
}  M_BlockElem, *M_Block;

  /****************/
  /* Link symbols */
  /****************/
typedef char *linkdef;

  /*********************/
  /* Kopf eines Netzes */
  /*********************/
typedef struct {
  M_Place Places,            /* Zeiger auf erste bzw. auf letzte Stelle      */
    LastPlace;               /* in der Stellenliste.                         */
  M_Transition Transitions,  /* Zeiger auf erste bzw. auf letzte Transition  */
    LastTransition;          /* in der Transitionenliste.                    */
  M_Block Blocks,            /* Zeiger auf ersten bzw. auf letzten Block in  */
    LastBlock;               /* der Blockliste (fuer Netz, nicht BlockTable) */
  M_Transition PTransitions, /* Zeiger auf erste bzw. auf letzte Phantom-    */
    LastPTransition;         /* transition in der Liste.                     */
  linkdef *Links;            /* Zeiger auf erste Linksymbol in der           */
                             /* Linksymbolenliste                            */
  Cardinal32 NumPlaces,      /* Anzahl der Stellen,                          */
    NumTransitions,          /*        der Transitionen,                     */
    NumComponents;           /*        der Komponenten.                      */
  Cardinal32 NumLinks,       /*        der Linksymbolen                      */
          MaxNumLinks;       /*           and max of Link symbols            */
  Cardinal32 NumBlocks;      /*        der Bloecke                           */
                             /* NICHT in NumComponents enthalten             */
  Cardinal32 NumPTransitions;/* Anzahl der Phantomtransitionen               */
                             /* auch NICHT in NumComponents enthalten        */
  Cardinal x_dim,            /* Breite des Netzes                            */
    y_dim;                   /* Hoehe des Netzes                             */
  int x_pos,                 /* Startpos. (oben links)                       */
    y_pos;
} M_NetHead, *M_Net;

  /**********************************/
  /* extern's auf globale Variablen */
  /**********************************/

extern Cardinal32 trans_anz;      /* Gesamtanzahl der Transitionen im Netz */
extern M_STEdge   stdummy;
extern M_TSEdge   tsdummy;
extern char  entry_type[];        /* Typ der Entry-Stellen als String      */
extern Cardinal32 max_trans_anz;  /* Maximal-Anzahl von Transitionen */
extern char *ref_file;

  /*********************************************************************/
  /* Funktionen zur Allokation und zur Freigabe von Speicherplatz fuer */
  /* einzelne Elemente eines Netzes; realisiert als Makros (meistens). */
  /*********************************************************************/

  /* Funktionalitaet: M_Net   M_allocHead(void)   */
  /*                  M_Place M_allocPlace(void)  */
  /*             M_Transition M_allocTrans(void)  */
  /*                 M_STEdge M_allocSTEdge(void) */
  /*                 M_TSEdge M_allocTSEdge(void) */

  /*                    int M_removeHead(M_Net _Head)          */
  /*                    int M_removePlace(M_Net _Place)        */
  /*                    int M_removeTrans(M_Transition _Trans) */
  /*                    int M_removeSTEdge(M_STEdge _STEdge)   */
  /*                    int M_removeTSEdge(M_TSEdge _TSEdge)   */

#define M_allocHead()   ( (M_Net) MYmalloc( sizeof( M_NetHead ) ) )
#define M_allocPlace()  ( (M_Place) MYmalloc( sizeof( M_PlElem ) ) )
extern M_Transition M_allocTrans(void);
#define M_allocSTEdge() ( (M_STEdge) MYmalloc( sizeof( M_STElem ) ) )
#define M_allocTSEdge() ( (M_TSEdge) MYmalloc( sizeof( M_TSElem ) ) )

#define M_removeHead( _Head )     MYfree( _Head )
#define M_removePlace( _Place )   MYfree( _Place )
#define M_removeTrans( _Trans ) \
        { trans_anz--; MYfree( _Trans ); } skip
#define M_removeBlock(_Block)  MYfree(_Block)
#define M_removeSTEdge( _STEdge ) \
        { VSremove( _STEdge->label ); MYfree( _STEdge ); } skip
#define M_removeTSEdge( _TSEdge ) \
        { VSremove( _TSEdge->label ); MYfree( _TSEdge ); } skip

  /***********************/
  /* Funktionsprototypen */
  /***********************/

  /* Referenzen */
#define M_RefsAreEqual( r1, r2 )                \
( (r1)->s_ref.start_l == (r2)->s_ref.start_l && \
  (r1)->s_ref.start_c == (r2)->s_ref.start_c && \
  (r1)->s_ref.end_l   == (r2)->s_ref.end_l   && \
  (r1)->s_ref.end_c   == (r2)->s_ref.end_c      )

void M_SumPlaceRefs( M_Place dst, M_Place src );
void M_AddAllPlaceRefs( M_Net net, PLabel label, RefNr ref, int flag );

  /* Stellen, Transitionen, Kanten, Netze, Annotationen */
M_Transition          M_CreateTr(M_Net net, MultiSet mm, node_ptr tree,
                                 Variables vars, int x_pos, int y_pos,
                                 RefNr ref);
     M_Place       M_CreateEdges(Cardinal32  preset, VarMSet  pre_label,
                                 Cardinal32 postset, VarMSet post_label,
                                 M_Place pl, M_Net net, Cardinal flags);
M_Transition    M_DuplicateTrans(M_Net net, M_Transition tr, M_STEdge *stlast,
                                 M_TSEdge *tslast, ComVar *sub);
    M_STEdge   M_CopyTransPreset(M_Transition tr, M_Transition newtr,
                                 M_STEdge *stlast,ComVar *sub,bool is_phant);
    M_TSEdge  M_CopyTransPostset(M_Transition tr, M_Transition newtr,
                                 M_TSEdge *tslast,ComVar *sub,bool is_phant);
     M_Place    M_DuplicatePlace(M_Net net,M_Place pl,M_STEdge *stlast,M_TSEdge 
                                 *tslast,M_STEdge *pstlast,M_TSEdge *ptslast);
    M_TSEdge   M_CopyPlacePreset(M_Place pl, M_Place newpl, M_TSEdge *tslast,
                                 bool is_phant);
    M_STEdge  M_CopyPlacePostset(M_Place pl, M_Place newpl, M_STEdge *stlast,
                                 bool is_phant);
     M_Place       M_RemovePlace(M_Net net, M_Place pl);
M_Transition  M_RemoveTransition(M_Net net,M_Transition tr,bool tr_is_phant);
       M_Net     M_CreateNetCopy(M_Net net);
        void        M_Remove_net(M_Net *net);
       M_Net  M_DisjointNetUnion(M_Net net1, M_Net net2);
        void       M_numname_all(M_Net net);
         int          M_CheckNet(M_Net net);
        void          PrintRange(M_Place pl, FILE *ofp);
        void           PrintAnno(FILE *ofp, M_Transition tr, bool is_phant);
        void        PrintNet_PEP(M_Net net);
        void        PrintMeaning(RefNr,int,FILE*);
        void    PrintSingleRange(Integer*, FILE *ofp);
     M_Place     M_PreparePlaces(M_Net net, PLabel label1, PLabel label2);
       M_Net M_RestoreKillPlaces(M_Net net, PLabel label);
        void   M_CopySingleRange(Integer* *dest_ptr, Integer *src);
        void         M_CopyRange(M_Place dest, M_Place src);
     bool M_SingleRangesAreEqual(Integer *r1, Integer *r2);
        bool    M_RangesAreEqual(M_Place pl1, M_Place pl2 );
        void  M_RemoveDuplicates(M_Net net, Cardinal32 *c_places,
                                            Cardinal32 *c_transitions,
                                            Cardinal32 *c_places_phant);

M_Net M_empty_net();
M_Place M_fast_insert_place(M_Net net, Cardinal32 P_numname,
                            M_Place    *pl_last);
M_Transition M_fast_insert_transition(Cardinal32 *NumTransitionsPtr,
                                      M_Transition *TransitionsPtr,
                                      M_Transition *LastTransitionPtr,
                                      Cardinal32 T_numname,
                                      M_Transition *tr_last, bool do_phants);
M_TSEdge M_fast_insert_TP_edge(M_Place first_place, M_Transition first_trans,
                               Cardinal32 P_numname, Cardinal32 T_numname,
                               M_Place *pl_last, M_Transition *tr_last,
                               M_TSEdge *ts_last, bool do_phants);
M_STEdge M_fast_insert_PT_edge(M_Place first_place, M_Transition first_trans,
                               Cardinal32 P_numname, Cardinal32 T_numname,
                               M_Place *pl_last, M_Transition *tr_last,
                               M_STEdge *st_last, bool do_phants);
M_Block M_fast_insert_block( M_Net net, Cardinal32 B_numname,
                             M_Block *bl_last, char *name );
void M_insert_link_symbol(M_Net net, char *ls);

#endif  /* #ifndef __MNET_H__ */
