/****************************************************************************/
/* move.c                                                                   */
/* Burkhard Graves, 20.12.93                                                */
/* Martin Ackermann, Fruehjahr 95                                           */
/* (lediglich Umbenennung der Variablen- & Funktionsnamen auf M-Netze)      */
/* Martin Ackermann, Winter 1996/97                                         */
/*   - Prozeduren, andere Referenzen, mehr Effizienz, Fehler beseitigt      */
/****************************************************************************/

  /************/
  /* includes */
  /************/

#include "move.h"

  /**************/
  /* Funktionen */
  /**************/

  /**********************************************/
  /* Netz 'net2' neben Netz 'net1' legen, dabei */
  /* EXIT-Stellen auf gleiche Hoehe bringen.    */
  /**********************************************/
void M_juxtapose(M_Net net1, M_Net net2)
{
  register M_Place pl;
  register int pos;

  M_move_hv(net2, net1->x_pos - net2->x_pos + net1->x_dim + RASTER,
    net1->y_pos - net2->y_pos);
  net1->x_dim += net2->x_dim + RASTER;
  if (net2->y_dim > net1->y_dim) {
    pos = (net1->y_dim = net2->y_dim) + net1->y_pos - RASTER;
    pl = net1->Places;
    while (pl) {
      if (pl->label == EXIT)
        pl->y_pos = pos;
      pl = pl->nextplace;
    }  /* while Stellen */
  } else if (net1->y_dim > net2->y_dim) {
    pos = (net2->y_dim = net1->y_dim) + net2->y_pos - RASTER;
    pl = net2->Places;
    while (pl) {
      if (pl->label == EXIT)
        pl->y_pos = pos;
      pl=pl->nextplace;
    }  /* while Stellen */
  }  /* if Netz1 laenger als Netz2 */
}  /* M_juxtapose */

  /*********************************************/
  /* Netz horizontal und vertikal verschieben. */
  /*********************************************/
M_Net M_move_hv(M_Net net, int x_offset, int y_offset)
{
  register M_Transition tr;
  register M_Place pl;

  net->x_pos += x_offset;
  net->y_pos += y_offset;
  tr = net->Transitions;
  while (tr) {
    tr->x_pos += x_offset;
    tr->y_pos += y_offset;
    tr = tr->nexttrans;
  }  /* while Transitionen */
  tr = net->PTransitions;
  while (tr) {
    tr->x_pos += x_offset;
    tr->y_pos += y_offset;
    tr = tr->nexttrans;
  }  /* while Phantomtransitionen */
  pl = net->Places;
  while (pl) {
    pl->x_pos += x_offset;
    pl->y_pos += y_offset;
    pl = pl->nextplace;
  }  /* while Stellen */
  return net;
}  /* M_move_hv */

  /**************************************/
  /* Netz um 90 Grad nach links drehen. */
  /**************************************/
M_Net M_turn_left(M_Net net)
{
  register M_Transition tr;
  register M_Place pl;
  register int i;
  register Cardinal c;

  i = net->x_pos;
  net->x_pos = net->y_pos;
  net->y_pos = -i - net->x_dim + RASTER;
  c = net->x_dim;
  net->x_dim = net->y_dim;
  net->y_dim = c;
  tr = net->Transitions;
  while (tr) {
    i = tr->x_pos;
    tr->x_pos = tr->y_pos;
    tr->y_pos = -i;
    tr = tr->nexttrans;
  }  /* while Transitionen */
  tr = net->PTransitions;
  while (tr) {
    i = tr->x_pos;
    tr->x_pos = tr->y_pos;
    tr->y_pos = -i;
    tr = tr->nexttrans;
  }  /* while Phantomtransitionen */
  pl = net->Places;
  while (pl) {
    i = pl->x_pos;
    pl->x_pos = pl->y_pos;
    pl->y_pos = -i;
    pl = pl->nextplace;
  }  /* while Stellen */
  return net;
}  /* M_turn_left */

  /***************************************/
  /* Netz um 90 Grad nach rechts drehen. */
  /***************************************/
M_Net M_turn_right(M_Net net)
{
  register M_Transition tr;
  register M_Place pl;
  register int i;
  register Cardinal c;

  i = net->y_pos;
  net->y_pos = net->x_pos;
  net->x_pos = -i - net->y_dim + RASTER;
  c = net->x_dim;
  net->x_dim = net->y_dim;
  net->y_dim = c;
  tr = net->Transitions;
  while (tr) {
    i = tr->y_pos;
    tr->y_pos = tr->x_pos;
    tr->x_pos = -i;
    tr = tr->nexttrans;
  }  /* while Transitionen */
  tr = net->PTransitions;
  while (tr) {
    i = tr->y_pos;
    tr->y_pos = tr->x_pos;
    tr->x_pos = -i;
    tr = tr->nexttrans;
  }  /* while Phantomtransitionen */
  pl = net->Places;
  while (pl) {
    i = pl->y_pos;
    pl->y_pos = pl->x_pos;
    pl->x_pos = -i;
    pl = pl->nextplace;
  }  /* while Stellen */
  return net;
}  /* M_turn_right */

  /****************************************************************************/
  /* Verschiebt die Stellen und Transitionen, deren y-Position oberhalb von   */
  /* 'y1' bzw. unterhalb von 'y2' liegt, gleichmaessig nach oben bzw. unten,  */
  /* so dass der Abstand zwischen 'y1' und 'y2' um 'offset' vergroessert wird.*/
  /****************************************************************************/
M_Net M_up_and_down(M_Net net, int y1, int y2, int offset)
{
  register M_Transition tr;
  register M_Place pl;
  register int up, down;

  if (offset > 0) {
    net->y_dim += offset;
    down = offset / 2;
    up = offset - down;
    net->y_pos -= up;

    tr = net->Transitions;
    while (tr) {
      if (tr->y_pos < y1)
        tr->y_pos -= up;
      else if (tr->y_pos > y2)
        tr->y_pos += down;
      tr = tr->nexttrans;
    }  /* while Transitionen */

    tr = net->PTransitions;
    while (tr) {
      if (tr->y_pos < y1)
        tr->y_pos -= up;
      else if (tr->y_pos > y2)
        tr->y_pos += down;
      tr = tr->nexttrans;
    }  /* while Phantomtransitionen */

    pl = net->Places;
    while (pl) {
      if (pl->y_pos < y1)
        pl->y_pos -= up;
      else if (pl->y_pos > y2)
        pl->y_pos += down;
      pl = pl->nextplace;
    }  /* while Stellen */
  }  /* if offset > 0 */
  return net;
}  /* M_up_and_down */

  /****************************************************************************/
  /* Verschiebt die Stellen und Transitionen, deren x-Position links von      */
  /* 'x1' bzw. rechts von 'x2' liegt, gleichmaessig nach links bzw. rechts,   */
  /* so dass der Abstand zwischen 'x1' und 'x2' um 'offset' vergroessert wird.*/
  /****************************************************************************/
M_Net M_left_and_right(M_Net net, int x1, int x2, int offset)
{
  register M_Transition tr;
  register M_Place pl;
  register int left, right;

  if (offset > 0) {
    net->x_dim += offset;
    right = offset / 2;
    left = offset - right;
    net->x_pos -= left;

    tr = net->Transitions;
    while (tr) {
      if (tr->x_pos < x1)
        tr->x_pos -= left;
      else if (tr->x_pos > x2)
        tr->x_pos += right;
      tr = tr->nexttrans;
    }  /* while Transitionen */

    tr = net->PTransitions;
    while (tr) {
      if (tr->x_pos < x1)
        tr->x_pos -= left;
      else if (tr->x_pos > x2)
        tr->x_pos += right;
      tr = tr->nexttrans;
    }  /* while Phantomtransitionen */

    pl = net->Places;
    while (pl) {
      if (pl->x_pos < x1)
        pl->x_pos -= left;
      else if (pl->x_pos > x2)
        pl->x_pos += right;
      pl = pl->nextplace;
    }  /* while Stellen */
  }  /* if offset > 0 */
  return net;
}  /* M_left_and_right */

  /*** Dateiende move.c ***/

