/* ptrans.c, Behandlung der Phantomtransitionen */
/* Burkhard Bieber 1995/96                      */
/* last-update: 08.04.96                        */

  /************/
  /* includes */
  /************/

#include "ptrans.h"
#include "multiset.h"
#include "block.h"

  /**************/
  /* Funktionen */
  /**************/

  /* Alloziert Speicher fuer eine Phantomtransition. */
M_Transition M_allocPTrans(void)
{
  M_Transition ptr;

  ptr = MYmalloc(sizeof(M_TrElem));
  return ptr;
}  /* M_allocPTrans */

  /* In der Liste der Phantomtransitionen / SyncTrans von tr */
  /* wird die Transition add_tr aufgenommen.                 */
void M_AddPTransPhantom(M_Transition tr, M_Transition add_tr)
{
  if (tr->PhantomEntries++)
    tr->phantoms = MYrealloc(tr->phantoms,
      tr->PhantomEntries * sizeof(M_Transition));
  else
    tr->phantoms = MYmalloc(sizeof(M_Transition));
  tr->phantoms[tr->PhantomEntries - 1] = add_tr;
}  /* M_AddPTransPhantom */

  /* Eine Funktion zum Kopieren der Phantomverweise aus der Transition src */
  /* in die Transition dest. Dabei wird der Speicher fuer die Verweise in  */
  /* der Zieltransition alloziert.                                         */
void M_CopyTransPhantomEntries(M_Transition dest, M_Transition src)
{
  int phantoms_size;

  dest->PhantomEntries = src->PhantomEntries;
  if (src->PhantomEntries) {
    phantoms_size = src->PhantomEntries * sizeof(M_Transition);
    dest->phantoms = MYmalloc(phantoms_size);
    memcpy(dest->phantoms, src->phantoms, phantoms_size);
  }  /* if es gibt etwas zu kopieren */
  else {
    dest->phantoms = NULL;
  }  /* else, keine Verweise */
}  /* M_CopyTransPhantomEntries */

  /* Ausgabe der Phantomverweisliste einer Phantomtransition */
  /* in die bereits offene Datei ofp.                        */
  /* Der Kode fuer den Phantomverweis ist 'P' - Phantom      */
void PrintPhantoms(FILE *ofp, M_Net net, M_Transition tr)
{
  M_Transition *phantomid_ptr;
  Cardinal c, phantoms_written;

  if (!(flag & PHANTOM_TRANS_OUTPUT) || (tr->PhantomEntries == 0))
    return;  /* keine Ausgabe gewuenscht oder keine Phantomverweise vorhanden */
  phantomid_ptr = tr->phantoms;
  c = 0;
  phantoms_written = 0;
  while (c < tr->PhantomEntries) {
    if (phantoms_written == 0)  /* erster zu schreibender Verweis */
      fprintf(ofp, "P\"(%u", (*phantomid_ptr)->numname);
    else  /* weitere zu schreibende Verweise ( > 1) */
      fprintf(ofp, ",%u", (*phantomid_ptr)->numname);
    phantoms_written++;
    phantomid_ptr++;
    c++;
  }  /* while noch Verweise auszugeben */
  if (phantoms_written != 0)
    fprintf(ofp, ")\"");
}  /* PrintPhantoms */

  /* Die Transition tr wird als Phantomtransition an den Anfang der */
  /* Phantomtransitionsliste im Netz net kopiert. Die ein- und aus- */
  /* gehenden Kanten werden als Phantomkanten kopiert.              */
M_Transition M_Copy2PTransition(M_Net net, M_Transition tr)
{
  register M_Transition phtr;  /* die neue Phantomtransition */
  int array_length;

  if (net && tr) {
    ++(net->NumPTransitions);
      /* erzeugen */
    phtr = M_allocPTrans();
      /* und verzeigern */
    phtr->nexttrans = net->PTransitions;
    if (net->PTransitions != NULL)
      net->PTransitions->prevtrans = phtr;
    else
      net->LastPTransition = phtr;
    net->PTransitions = phtr;
    phtr->prevtrans = NULL;
      /* Werte setzen */
    phtr->numname = 0;
    phtr->label = (MultiSet) MScreate_copy(tr->label);
    phtr->vars.ComEntries = tr->vars.ComEntries;
    array_length = tr->vars.ComEntries * sizeof(ComArrayEntry);
    phtr->vars.ComArray = MYmalloc(array_length);
    memcpy(phtr->vars.ComArray, tr->vars.ComArray, array_length);
    phtr->guard = MYmalloc(tr->guard->op2 * sizeof(node));
    memcpy(phtr->guard, tr->guard, tr->guard->op2 * sizeof(node));
    phtr->def = MYmalloc(*(tr->def) * sizeof(Integer));
    memcpy(phtr->def, tr->def, *(tr->def) * sizeof(Integer));
    phtr->RefEntries = tr->RefEntries;
    if (tr->RefEntries) {
      array_length = tr->RefEntries * sizeof(Ref);
      phtr->ref = MYmalloc(array_length);
      memcpy(phtr->ref, tr->ref, array_length);
    }  /* if Referenzen vorhanden */
    if (tr->meaning)
      phtr->meaning=MYstrdup(tr->meaning);
    else
      phtr->meaning=NULL;
    M_CopyTransBlockEntries(phtr, tr);
      /* Phantomverweis behandeln, ist vorerst leer */
    phtr->PhantomEntries = 0;
    phtr->phantoms = NULL;
    phtr->x_pos = tr->x_pos;
    phtr->y_pos = tr->y_pos;
    phtr->flags = tr->flags & ~TR_CHAN0_ACCESS;
    phtr->preset = M_CopyTransPreset(tr, phtr, &stdummy, NULL, True);
    phtr->postset = M_CopyTransPostset(tr, phtr, &tsdummy, NULL, True);
  }  /* if keine Nullzeiger */
  else {
    Msg( FAT,"M_Copy2PTransition: net or transition is NULL-pointer.");
    phtr = NULL;
  }  /* else, Fehler */
  return phtr;
}  /* M_Copy2PTransition */

  /*** Dateiende ptrans.c ***/

