/*****************************************************************************/
/* readHL.c                                                                  */
/*                                                                           */
/* Liest ein M-Netz im Format des Netzeditors ein und wandelt es in eine     */
/* interne Darstellung um.                                                   */
/*                                                                           */
/* Stefan Roemer,    Winter    95                                            */
/* Martin Ackermann, Fruehjahr 95                                            */
/*                   Winter    96/97                                         */
/*                             (fuer Makros, Tupel und Konstanten erweitert) */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <ctype.h>
#include "readlib.h"
#include "readHL.h"
#include "multiset.h"
#include "f_parse.h"
#include "block.h"
#include "macros.h"
#include "ptrans.h"
#include "tables.h"
#include "mnet.h"

/*****************************************************************************/
char* trueguard="tt";
/*****************************************************************************/

void *akt_component; 
   /* Zeiger auf die Stelle / Transition / Kante, die gerade eingelesen wird */

/*****************************************************************************/
/* Einlesen einer Zeichenkette aus einer Textdatei; fuehrende Leerzeichen    */
/* werden ueberlesen und nicht dem String zugeordnet; Kommentarzeilen werden */
/* ueberlesen. Funktion fuer das neue Dateiformat.                           */
/*****************************************************************************/

static void ReadEnclString( FILE * file, char * string, int maxlen )
  {

    register char ch = 0,
                  * str = string;
    register int  len = 0;
    char          delimiter;

    if (    ( ( delimiter = ReadCharComment( file ) ) != '\'' )
         && ( delimiter != '"' ) )
      {
        PEPerror( "ReadEnclString(): string leading ' or \" expected" );
      } /* if */
    while ( ! feof( file ) && ( ( *str = getc( file ) ) != delimiter )
            && ( len < maxlen ) )
      {
        str++;
        len++;
      } /* while */
    if ( ( len == maxlen ) && ( *str != delimiter ) )
      {
        while ( ! feof( file ) && ( ( ch = getc( file ) ) != delimiter ) )
          skip;
      } /* if */
    else
      {
        ch = *str;
      } /* else */
    if ( ch != delimiter )
      {
        sprintf( string, "ReadEnclString(): closing %c is missing", delimiter );
        PEPerror( string );
      } /* if */
    *str = '\0';
    
  } /* ReadEnclString() */

/*****************************************************************************/
/***************************** read_HLnet ************************************/
/*****************************************************************************/
/* Neu eingefuegt (neues Format)       19.12.1994 - 09.01.1995 Stefan Roemer */
/*****************************************************************************/

M_Net read_HLnet( char * PEPfilename, char * REFfilename )
  {

    int                 cx, cy,
                        sx, sy,
                        px, py,
                        prx, pry,
                        gx, gy,
                        grx, gry,
                        mx, my,
                        mrx, mry,
                        arx, ary,
                        color, thickness,
                        cmarks, imarks, capacity,
                        num = 0, num2 = 0, weight,
                        def_pl_radius = 10,
                        def_pl_prx = 10,
                        def_pl_pry = 0,
                        def_pl_mrx = 10,
                        def_pl_mry = 10,
                        def_pl_imarks = 0,
                        def_pl_cmarks = 0,
                        def_pl_capacity = OMEGA,
                        def_pl_thickness = 2,
                        def_pl_color = 1,
                        def_tr_size_x = 10,
                        def_tr_size_y = 10,
                        def_tr_prx = 10,
                        def_tr_pry = 0,
                        def_tr_mrx = 10,
                        def_tr_mry = 10,
                        def_tr_arx = 10,
                        def_tr_ary = 30,
                        def_tr_thickness = 2,
                        def_tr_color = 1;
    char                ch,
                        * chPtr;
    M_Place             * PlArray;
    M_Transition        * TrArray;
    M_Transition        * PTArray;
    M_Block             * BlArray = NULL;
    char                name[NAMESIZE+1],
                        *cmd, *meaning,
                        *links, *guard,
                        *marking, *blocks,
                        *phantoms, *name2,
                        *reference;
    double		eft, lft;
    FILE                * infile;
    M_Net               net;
    register Cardinal32 AnzBlNamen,
                        MaxBlNamen;
    Cardinal32          AnzPlNamen,
                        MaxPlNamen,
                        AnzTrNamen,
                        MaxTrNamen,
                        AnzPTNamen,
                        MaxPTNamen,
                        ident,
                        count;
    Cardinal32          max_ident = 0,
                        invisible;
    M_Place             pl_last = NULL;
    M_Transition        tr_last = NULL;
    M_Block             bl_last = NULL;
    M_STEdge            st_last = NULL;
    M_TSEdge            ts_last = NULL;
    bool                entry_place,
                        exit_place,
                        refined,
                        sync,
                        autonumbering,
                        abs_meaning,
                        abs_name,
                        abs_guard,
                        num_marking,
                        specified;

void ScanPhantoms( char *phants, M_Transition* *phPtr, Cardinal *phEntriesPtr,
		   M_Transition * PhArray )
  {
    if ( *phants != '(' )
      PEPerror( "phantom list should begin with '('" );
    phants++;

    while ( isdigit( *phants ) )
      { 
	if ( sscanf( phants, "%d%n,%n", &sx, &num, &num ) < 1 )
	  PEPerror( "phantom list has wrong format" );
	phants += num;
	
	if ( *phEntriesPtr )
	  (*phPtr) = MYrealloc( (*phPtr), 
				 ( (*phEntriesPtr)+1) * sizeof(M_Transition) );
	else
	  (*phPtr) = MYmalloc( sizeof(M_Transition) );
	
	(*phPtr)[(*phEntriesPtr)++] = PhArray[ sx ];
      }
  }

void ScanBlocks( char *blcks, BlockNr* *blocksPtr, Cardinal *BlockEntriesPtr )
  {
        if ( blcks && *blcks )
          {
            if ( *blcks != '(' )
              PEPerror( "BlockList should begin with '('" );

            blcks++;
            while ( isdigit ( *blcks ) )
              {
                sscanf( blcks, "%d%n", &cx, &num );
		blcks += num + 1;
		BlArray[cx]->ever_used = True;
                if ((*BlockEntriesPtr)++)
                  *blocksPtr = MYrealloc(*blocksPtr, 
					 (*BlockEntriesPtr) * sizeof(BlockNr));
                else
                  *blocksPtr = MYmalloc(sizeof(BlockNr));
                (*blocksPtr)[(*BlockEntriesPtr)-1] = BlArray[cx]->blocknr;
              }
          }
  }

void read_TransTags ( void )
  {
        int lrx, lry;        //positions of link terms - ignored
        lrx=0, lry=0;
        count++; 
        ident = 0;
        cx = cy =  mx = my = px = py = 0;
        prx = def_tr_prx;
        pry = def_tr_pry;
        mrx = def_tr_mrx;
        mry = def_tr_mry;
        arx = def_tr_arx;
        ary = def_tr_ary;
        invisible = 0;
        num_marking = True;
        abs_name = abs_meaning = specified = sync = False;
        *name = '\0';
	blocks = links = meaning = name2 = phantoms = reference = NULL;
	eft=lft=0;
        guard=trueguard;
        color = def_tr_color;
        sx = def_tr_size_x;
        sy = def_tr_size_y;
        thickness = def_tr_thickness;
        while ( ch != '\n' )
          {
            refined = False;
            if ( isdigit( ch ) || ( ch == '-' ) )
              {
                ungetc( ch, infile );
                num = ReadNumber( infile );
                if ( TestSymbol( infile, '@' ) )
                  {
                    cx = num;
                    cy = ReadNumber( infile );
                  } /* if */
                else
                  {
                    if ( ( ident = num ) != count )
                      {
                        autonumbering = False;
                      } /* if */
                    if ( ident > max_ident )
                      {
                        max_ident = ident;
                      } /* if */
                    specified = True;
                  } /* else */
              } /* if */ 
            else
              {
                switch ( ch )
                  {
                    case '\'':
                    case '"' : ungetc( ch, infile );
                               ReadEnclString( infile, name, NAMESIZE );
                               break;
                    case 'n' : ReadCoordinates( infile, &prx, &pry );
                               break;
                    case 'N' : ReadCoordinates( infile, &px, &py );
                               break;
                    case 'b' : meaning=MYstrdup(RL_ReadEnclString( infile ));
                               break;
                    case 'a' : ReadCoordinates( infile, &mrx, &mry );
                               break;
                    case 'A' : ReadCoordinates( infile, &mx, &my );
                               break;
                    case 'v' : invisible = ReadNumber( infile );
                               break;
                    case 'g' : guard=MYstrdup(RL_ReadEnclString( infile ));
                               break;
                    case 'h' : ReadCoordinates( infile, &grx, &gry );
                               break;
                    case 'H' : ReadCoordinates( infile, &gx, &gy );
                               abs_guard = True;
                               break;
                    case 'R' : reference=MYstrdup(RL_ReadEnclString( infile ));
                               break;
                    case 'i' : name2=MYstrdup(RL_ReadEnclString( infile ));
                               break;
                    case 'j' : ReadCoordinates( infile, &arx, &ary );
                               break;
                    case 's' : sy = sx = ReadNumber( infile );
                               break;
                    case 'S' : ch = ReadCharComment( infile );
                               ungetc( ch, infile );
                               if ( isdigit( ch ) )
                                 {
                                   sx = ReadNumber( infile );
                                   if ( TestSymbol( infile, '@' ) )
                                     {
                                       sy = ReadNumber( infile );
                                     } /* if */
                                   else
                                     {
                                       PEPerror( "missing '@'" );
                                     } /* else */
                                 }
                               else
                                 {
                                   sync = True;
                                 }
                               break;
                    case 't' : thickness = ReadNumber( infile );
                               break;
                    case 'r' : if ( refined )
                                 {
                                   PEPerror( "refined flag doubly defined" );
                                 } /* else */
                               else
                                 {
                                   refined = True;
                                 } /* else */
                               break;
                    case 'c' : color = ReadNumber( infile );
                               break;
                    case 'u' : blocks=MYstrdup(RL_ReadEnclString( infile ));
                               break;
                    case 'P' : phantoms=MYstrdup(RL_ReadEnclString( infile ));
                               break;
                    case 'l' : links=MYstrdup(RL_ReadEnclString( infile ));
                               break;
                    case 'E' : eft=ReadDouble( infile );
                               break;
                    case 'L' : lft=ReadDouble( infile );
                               break;
                    case 'k' : ReadCoordinates( infile, &lrx, &lry );
                               break;
                    default  : Error((errbuf, "unknown token '%c'", ch));
                  } /* switch */
              } /* else */
            ch = ReadCharComment( infile );
          } /* while */
  }

void insert_Trans( M_Transition    **TrArrayPtr,
		   Cardinal32       *AnzTrNamenPtr,
		   Cardinal32       *MaxTrNamenPtr,
		   Cardinal32       *NumTransitionsPtr,
		   M_Transition     *TransitionsPtr,
                   M_Transition     *LastTransitionPtr,
		   bool              do_phants )
  {
        /* insert transition node in the net */
        if ( autonumbering )
          {
            ident = count;
          } /* if */
        else if ( ! specified )
          {
            PEPerror( "missing transition identifier" );
          } /* if */
        if ( ident > (*AnzTrNamenPtr) )
          {
            (*AnzTrNamenPtr) = ident;
          } /* if */
        else if ( (*TrArrayPtr)[ ident ] )
          {
            Error((errbuf, "transition identifier %d multiply used", ident));
          } /* else if */
        while ( (*AnzTrNamenPtr) >= (*MaxTrNamenPtr) )
          {
            (*TrArrayPtr) = (M_Transition *) MYrealloc( (char *) (*TrArrayPtr),
                            ( (*MaxTrNamenPtr)+=1000 ) * sizeof(M_Transition) );
            for ( count=(*MaxTrNamenPtr)-1000; count<(*MaxTrNamenPtr); count++ )
              {
                (*TrArrayPtr)[ count ] = (M_Transition) NULL;
              } /* for */
          } /* if */
        akt_component = (*TrArrayPtr)[ident] = M_fast_insert_transition( NumTransitionsPtr, 
							 TransitionsPtr,
							 LastTransitionPtr,
							 ident, &tr_last,
							 do_phants );
        tr_last->vars.ComArray = NULL;
        tr_last->flags = invisible | (sync ? TR_CHAN0_ACCESS : 0);

        /* Bei NO_SCOPING werden in den eingelesenen Templates bestimmte 
           markierte Komponenten verschoben, um eine bessere Darstellung zu 
           haben. */
        if ( flag & NO_SCOPING )
          {
            cx += ( (tr_last->flags & 1) ? 2*RASTER : 0 );
            cy -= ( (tr_last->flags & 2) ? 2*RASTER : 0 );
          }
        tr_last->x_pos = cx;
        tr_last->y_pos = cy;

	if ( phantoms && *phantoms )
	  {
	    if ( do_phants )
	    {
	      /* adresses of the normal transitions are already known
		 -> parse phantoms know */
	      ScanPhantoms( phantoms, &(tr_last->phantoms),
			    &(tr_last->PhantomEntries), TrArray );
	      Free(phantoms);
	    }
	    else
              /* adresses of the phantom transitions are not yet known 
		 -> parse phantoms later */
	      tr_last->phantoms = (M_Transition*) phantoms;
	  }

        /* Abgrenzungen des Netzes anpassen */
        if ( cx < net->x_pos )
          net->x_pos = cx;
        if ( cy < net->y_pos )
          net->y_pos = cy;
        if ( cx+RASTER > net->x_dim )
          net->x_dim = cx+RASTER; /* hier wird zunaechst x_max ermittelt */
        if ( cy+RASTER > net->y_dim )
          net->y_dim = cy+RASTER; /* hier wird zunaechst y_max ermittelt */

        if ( name2 && *name2 )
          {
            tr_last->label = StrToMultiSet( name2, tr_last );
	    Free(name2);
          } 
        else
            tr_last->label = MScreate_empty_set();
        if ( guard && *guard ) /* immer erfuellt */
          {
            tr_last->guard = ParseAnnotation( guard, tr_last );
	    if (guard!=trueguard)
	      Free(guard);
          }
        if ( reference && !strncmp( reference, "$REF$", 5 ) )
	{
          Add_Cmd( Trans_AddRef, NULL, atoi( reference + 5 ), 0 );
	  Free(reference);
	}
        if ( meaning )
          {
		char *c = strstr(meaning,"$NAME$");
		if (c) Add_Cmd(Trans_Meaning,NULL,0,0);
		tr_last->meaning = meaning;
          } /* if */
        tr_last->links = links;
	if (eft||lft)
        {
          tr_last->time_label=(Time_stamp*)MYmalloc(sizeof(Time_stamp));
          tr_last->time_label->low=eft;
          tr_last->time_label->high=lft;
          eft=lft=0;
        }

	ScanBlocks( blocks, &(tr_last->blocks), &(tr_last->BlockEntries) );
	if(blocks)
	  Free(blocks);
        NNinput_line++;
  } 

void read_ArcTags( char sym )
  {
        weight = 1;
        gx = gy = 0;
        invisible = 0;
        guard = name2 = meaning = NULL;
	*name = '\0';
        while ( ch != '\n' )
          {
            if ( isdigit( ch ) )
              {
                ungetc( ch, infile );
                num = ReadNumber( infile );
                if ( TestSymbol( infile, sym ) )
                  {
                    num2 = ReadNumber( infile );
                  } /* if */
                else
                  {
		    Error((errbuf, "'%c' expected", sym));
                  } /* else */
              } /* if */ 
            else
              {
                switch ( ch )
                  {
                    case 'J' :
                    case ',' : ReadCoordinates( infile, &mx, &my );
                               break;
                    case 'w' : weight = ReadNumber( infile );
                               break;
                    case 'n' : ReadCoordinates( infile, &prx, &pry );
                               break;
                    case 'N' : ReadCoordinates( infile, &px, &py );
                               break;
                    case 'p' : guard=MYstrdup(RL_ReadEnclString( infile ));
                               break;
                    case 'q' : ReadCoordinates( infile, &grx, &gry );
                               break;
                    case 'Q' : ReadCoordinates( infile, &gx, &gy );
                               break;
                    case 'v' : invisible = ReadNumber( infile );
                               break;
                    case 't' : thickness = ReadNumber( infile );
                               break;
                    case 'c' : color = ReadNumber( infile );
                               break;
                    default  : Error((errbuf, "unknown token '%c'", ch));
                  } /* switch */
              } /* else */
            ch = ReadCharComment( infile );
          } /* while */
  }

void insert_PTArc( Cardinal32 AnzTrNamen, M_Transition *TrArray, 
		   bool do_phants )
  {
        read_ArcTags( '>' );
        if ( ! num || ( num > AnzPlNamen ) || ! PlArray[ num ] )
          {
            PEPerror( "arc: incorrect place identifier (554)" );
          } /* if */
        if ( ! num2 || ( num2 > AnzTrNamen ) || ! TrArray[ num2 ]  )
          {
            PEPerror( "arc: incorrect transition identifier" );
          } /* if */
        pl_last = PlArray[ num ];
        tr_last = TrArray[ num2 ];
        akt_component = M_fast_insert_PT_edge( net->Places, net->Transitions, 
			       pl_last->numname, tr_last->numname,
			       &pl_last, &tr_last, &st_last, do_phants );
        st_last->flags = invisible;
        if ( guard && *guard )
          {
            st_last->label = StrToVarMSet( guard, tr_last );
	    Free(guard);
          } /* if */
        else
          {
            PEPerror( "arc: missing arc annotation" );
          } /* else */
        NNinput_line++;
  }

void insert_TPArc( Cardinal32 AnzTrNamen, M_Transition *TrArray, 
		   bool do_phants )
  {
        read_ArcTags( '<' );
        if ( ! num || ( num > AnzTrNamen ) || ! TrArray[ num ] )
          {
            PEPerror( "arc: incorrect transition identifier" );
          } /* if */
        if ( ! num2 || ( num2 > AnzPlNamen ) || ! PlArray[ num2 ]  )
          {
            PEPerror( "arc: incorrect place identifier(587)" );
          } /* if */
        tr_last = TrArray[ num ];
        pl_last = PlArray[ num2 ];
        akt_component = M_fast_insert_TP_edge( net->Places, net->Transitions, 
			       pl_last->numname, tr_last->numname,
			       &pl_last, &tr_last, &ts_last, do_phants );
        ts_last->flags = invisible;
        if ( guard && *guard )
          {
            ts_last->label = StrToVarMSet( guard, tr_last );
	    Free(guard);
          } /* if */
        else
          {
            PEPerror( "arc: missing arc annotation" );
          } /* else */
        NNinput_line++;
  }

#define __LOG_IT__
#undef __LOG_IT__
    NNinput_file = PEPfilename;
    NNinput_line = 1;
    count = MaxPlNamen = MaxTrNamen = MaxPTNamen = 1000;
    PlArray = (M_Place *) MYmalloc( count * sizeof( M_Place ) );
    TrArray = (M_Transition *) MYmalloc( count * sizeof( M_Transition ) );
    PTArray = (M_Transition *) MYmalloc( count * sizeof( M_Transition ) );
    AnzPlNamen = AnzTrNamen = AnzPTNamen = AnzBlNamen = 0;
    while ( --count )
      {
        PlArray[ count ] = (M_Place) NULL;
        TrArray[ count ] = (M_Transition) NULL;
        PTArray[ count ] = (M_Transition) NULL;
      } /* while */

    /* Oeffne PEP-Datei */
    if ( ! ( infile = fopen( PEPfilename, "r" ) ) )
      {
        fprintf( stderr,
               "ERROR: CM.read_PEP(): could not open file '%s' for reading.\n",
                 PEPfilename );
        exit ( 4 );
      } /* if */

    cmd=ReadCmdToken( infile );
    if ( strcmp( cmd, "PEP" ) )
      {
        PEPerror( "keyword 'PEP' expected" );
        return ( NULL );
      } /* if */
    ReadNewline( infile );
    cmd=ReadCmdToken( infile );
    if ( strcmp( cmd, "HLNet" ) )
      {
        PEPerror( "format identifier 'HLNet' expected" );
      } /* if */
    ReadNewline( infile );
    cmd=ReadCmdToken( infile );
    if ( strcmp( cmd, "FORMAT_N" ) && strcmp( cmd, "FORMAT_N2" ) )
      {
        PEPerror( "keyword 'FORMAT_N' or 'FORMAT_N2' expected" );
        return ( NULL );
      } /* if */

    /* Initialize net */
    net = M_empty_net();
    net->NumComponents = 1;

    /* Read in default settings. */
    ReadNewline( infile );
    cmd=ReadCmdToken( infile );
    while ( *cmd == 'D' )
      {
        if ( ! strcmp( cmd, "DPL" ) )
          {
            ReadNewline( infile );
          } /* if */
        else if ( ! strcmp( cmd, "DTR" ) )
          {
            ReadNewline( infile );
          } /* else if */
        else if ( ! strcmp( cmd, "DPT" ) )
          {
            ReadNewline( infile );
          } /* else if */
        else
          {
            Error((errbuf, "read_new_PEP(): unknown keyword '%s'", cmd));
            return ( NULL );
          } /* else */
        cmd=ReadCmdToken( infile );
      } /* while */

    /* Read in blocks */
    if ( !strcmp( cmd, "BL" ) )
      {
        BlArray = (M_Block *) MYmalloc( ( count = MaxBlNamen = 1000 ) 
                                          * sizeof( M_Block ) );
        while ( --count )
          {
            BlArray[ count ] = (M_Block) NULL;
          } /* while */

        ReadNewline( infile );

        count = 0;
        autonumbering = True;
        while ( ( ch = ReadCharComment( infile ) ) && ( isdigit( ch ) ) )
          {
            count++; 
            ident = 0;
            cx = cy = 0;
            *name = '\0';
	    blocks = meaning = reference = NULL;
            specified = False;
            while ( ch != '\n' )
              {
                if ( isdigit( ch ) || ( ch == '-' ) )
                  {
                    ungetc( ch, infile );
                    num = ReadNumber( infile );
                    if ( TestSymbol( infile, '@' ) )
                      {
                        cx = num;
                        cy = ReadNumber( infile );
                      } /* if */
                    else
                      {
                        if ( ( ident = num ) != count )
                          {
                            autonumbering = False;
                          } /* if */
                        if ( ident > max_ident )
                          {
                            max_ident = ident;
                          } /* if */
                        specified = True;
                      } /* else */
                  } /* if */ 
                else
                  {
                    switch ( ch )
                      {
                        case '\'':
                        case '"' : ungetc( ch, infile );
                                   ReadEnclString( infile, name, NAMESIZE );
                                   break;
                        case 'b' : meaning=MYstrdup(RL_ReadEnclString(infile));
                                   break;
                        case 'u' : blocks=MYstrdup(RL_ReadEnclString( infile ));
                                   break;
                        default  : Error((errbuf, "unknown token '%c'", ch));
                      } /* switch */
                  } /* else */
                ch = ReadCharComment( infile );
              } /* while */

            /* insert block in the net */
            if ( autonumbering )
              {
                ident = count;
              } /* if */
            else if ( ! specified )
              {
                PEPerror( "missing block identifier" );
              } /* if */
            if ( ident > AnzBlNamen )
              {
                AnzBlNamen = ident;
              } /* if */
            else if ( BlArray[ ident ] )
              {
                Error((errbuf, "block identifier %d multiply used", ident));
              } /* else if */
            while ( AnzBlNamen >= MaxBlNamen )
              {
                BlArray = (M_Block *) MYrealloc( (char *) BlArray,
                                      ( MaxBlNamen += 1000 ) * sizeof(M_Block));
                for ( count = MaxBlNamen - 1000 ; count < MaxBlNamen ; count++ )
                  {
                    BlArray[ count ] = (M_Block) NULL;
                  } /* for */
              } /* if */
            akt_component = BlArray[ ident ] = M_fast_insert_block( net, ident, 
                                                    &bl_last, meaning );
	    Free(meaning);
	    bl_last->numname = (Cardinal32) blocks;

            NNinput_line++;
          } /* while */
        ungetc( ch, infile );
        cmd=ReadCmdToken( infile );
      } /* if */

    /* now that all new blocknr are known -> scan and translate blocklists */
    bl_last = net->Blocks;
    while ( bl_last )
      {
	if ( bl_last->numname )
	  {
	    char    *str = (char*) bl_last->numname;
	    BlockNr  nr;
	    
	    BlockNr *bnr_ptr;
	    BlockPtr b_ptr;

	    if ( *str != '(' )
	      PEPerror( "BlockList should begin with '('" );
	    
	    str++;
	    while ( isdigit ( *str ) )
	      {
		sscanf( str, "%hu%n", &nr, &num );
		str += num + 1;
		
		b_ptr = BlockToStruct( bl_last->blocknr );
		
		b_ptr->childcount++;
		if (b_ptr->childcount+1 > b_ptr->childmax) 
		  { /* if kein Platz mehr */
		    /* dann eben neuen Platz schaffen */
		    b_ptr->childmax += BLOCK_CHUNK_CHILD_MAX;
		    b_ptr->children = (BlockNr *) MYrealloc(b_ptr->children,
				  BLOCK_CHUNK_CHILD_MAX * b_ptr->childmax);
		  }  /* if kein Platz mehr */
		bnr_ptr = b_ptr->children + b_ptr->childcount - 1;
		*bnr_ptr = BlArray[nr]->blocknr;
		*(bnr_ptr + 1) = 0;  /* Listenende */
		
	      } /* while */
	    MYfree( (char*) bl_last->numname );
	  } /* if */
	bl_last = bl_last->nextblock;
    }

    /* Read Link symbols *************************************************/
    if ( !strcmp( cmd, "LS" ))
      {
	char linkdefs[513];

        ReadNewline( infile );
        if (!feof(infile))
          {fgets(linkdefs,512,infile); NNinput_line++;}
        while (!feof(infile) && linkdefs && strchr( linkdefs, '"' )) {
          M_insert_link_symbol(net,linkdefs);
          *linkdefs = ReadCharComment(infile);
          fgets(linkdefs+1,511,infile);
          NNinput_line++;
        }
        /* put back the last '\n' because of the ReadNewLine after
           strcmp(cmd,"PL")*/
        ungetc ('\n',infile);
	strcpy(sbuf,linkdefs);
	cmd=sbuf;
        sbuf[strlen(sbuf)-1]='\0';
      }; /* if ( strcmp( cmd, "LS" ) ) */

    /* Read in place nodes ***************************************************/
    if ( strcmp( cmd, "PL" ) )
      {
        fprintf (stderr, "cmd: \"%s\"\n",cmd);
        PEPerror( "keyword 'PL' expected" );
        return ( NULL );
      } /* if */
    ReadNewline( infile );

    count = 0;
    autonumbering = True;
    while ( ( ch = ReadCharComment( infile ) ) && ( ch != 'T' ) )
      {
        count++; 
        ident = 0;
        cx = cy =  mx = my = px = py = 0;
        prx = def_pl_prx;
        pry = def_pl_pry;
        mrx = def_pl_mrx;
        mry = def_pl_mry;
        invisible = 0;
        cmarks = def_pl_cmarks;
        imarks = def_pl_imarks;
        num_marking = True;
        abs_name = abs_meaning = specified = False;
        blocks = guard = marking = meaning = name2 = reference = NULL;
	*name = '\0';
        capacity = def_pl_capacity;
        color = def_pl_color;
        sx = def_pl_radius;
        thickness = def_pl_thickness;
        entry_place = exit_place = False;
        while ( ch != '\n' )
          {
            if ( isdigit( ch ) || ( ch == '-' ) )
              {
                ungetc( ch, infile );
                num = ReadNumber( infile );
                if ( TestSymbol( infile, '@' ) )
                  {
                    cx = num;
                    cy = ReadNumber( infile );
                  } /* if */
                else
                  {
                    if ( ( ident = num ) != count )
                      {
                        autonumbering = False;
                      } /* if */
                    if ( ident > max_ident )
                      {
                        max_ident = ident;
                      } /* if */
                    specified = True;
                  } /* else */
              } /* if */ 
            else
              {
                switch ( ch )
                  {
                    case '\'':
                    case '"' : ungetc( ch, infile );
                               ReadEnclString( infile, name, NAMESIZE );
                               break;
                    case 'n' : ReadCoordinates( infile, &prx, &pry );
                               break;
                    case 'N' : ReadCoordinates( infile, &px, &py );
                               abs_name = True;
                               break;
                    case 'b' : meaning=MYstrdup(RL_ReadEnclString( infile ));
                               break;
                    case 'a' : ReadCoordinates( infile, &mrx, &mry );
                               break;
                    case 'A' : ReadCoordinates( infile, &mx, &my );
                               abs_meaning = True;
                               break;
                    case 'M' : imarks = ReadNumber( infile );
                               num_marking = True;
                               break;
                    case 'm' : cmarks = ReadNumber( infile );
                               num_marking = True;
                               break;
                    case 'Z' : guard=MYstrdup(RL_ReadEnclString( infile ));
                               num_marking = False;
                               break;
                    case 'B' : ReadCoordinates( infile, &mrx, &mry );
                               break;
                    case 'z' : name2=MYstrdup(RL_ReadEnclString( infile ));
                               num_marking = False;
                               break;
                    case 'y' : marking=MYstrdup(RL_ReadEnclString( infile ));
                               num_marking = False;
                               break;
                    case 'R' : reference=MYstrdup(RL_ReadEnclString( infile ));
                               break;
                    case 'k' : capacity = ReadNumber( infile );
                               break;
                    case 'e' : if ( exit_place )
                                 {
                                   PEPerror( "both exit and entry flag found" );
                                 } /* else */
                               else
                                 {
                                   entry_place = True;
                                 } /* else */
                               break;
                    case 'x' : if ( entry_place )
                                 {
                                   PEPerror( "both entry and exit flag found" );
                                 } /* else */
                               else
                                 {
                                   exit_place = True;
                                 } /* else */
                               break;
                    case 'v' : invisible = ReadNumber( infile );
                               break;
                    case 's' : sx = ReadNumber( infile );
                               break;
                    case 't' : thickness = ReadNumber( infile );
                               break;
                    case 'c' : color = ReadNumber( infile );
                               break;
                    case 'u' : blocks=MYstrdup(RL_ReadEnclString( infile ));
                               break;
                    default  : Error((errbuf, "unknown token '%c'", ch));
                  } /* switch */
              } /* else */
            ch = ReadCharComment( infile );
          } /* while */

        /* insert place node in the net */
        if ( autonumbering )
          {
            ident = count;
          } /* if */
        else if ( ! specified )
          {
            PEPerror( "missing place identifier" );
          } /* if */
        if ( ident > AnzPlNamen )
          {
            AnzPlNamen = ident;
          } /* if */
        else if ( PlArray[ ident ] )
          {
            Error((errbuf, "place identifier %d multiply used", ident));
          } /* else if */
        while ( AnzPlNamen >= MaxPlNamen )
          {
            PlArray = (M_Place *) MYrealloc( (char *) PlArray,
                                  ( MaxPlNamen += 1000 ) * sizeof( M_Place ) );
            for ( count = MaxPlNamen - 1000 ; count < MaxPlNamen ; count++ )
              {
                PlArray[ count ] = (M_Place) NULL;
              } /* for */
          } /* if */
        akt_component = PlArray[ ident ] = M_fast_insert_place( net, ident, &pl_last );
        pl_last->flags = invisible;

        /* Bei NO_SCOPING werden in den eingelesenen Templates bestimmte 
           markierte Komponenten verschoben, um eine bessere Darstellung zu 
           haben. */
        if ( flag & NO_SCOPING )
          {
            cx += ( (pl_last->flags & 1) ? 2*RASTER : 0 );
            cy -= ( (pl_last->flags & 2) ? 2*RASTER : 0 );
          }
        pl_last->x_pos = cx;
        pl_last->y_pos = cy;

        /* Abgrenzungen des Netzes anpassen */
        if ( cx < net->x_pos )
          net->x_pos = cx;
        if ( cy < net->y_pos )
          net->y_pos = cy;
        if ( cx+RASTER > net->x_dim )
          net->x_dim = cx+RASTER; /* hier wird zunaechst x_max ermittelt */
        if ( cy+RASTER > net->y_dim )
          net->y_dim = cy+RASTER; /* hier wird zunaechst y_max ermittelt */

        pl_last->label = entry_place ? ENTRY : exit_place ? EXIT : EMPTY;
        StringToRange( guard, pl_last->range, 'R' );
	if (guard)
	  Free(guard);
	if (reference)
	{
          if ( !strncmp( reference, "$REF$", 5 ) )
            Add_Cmd( Place_AddRef, NULL, atoi( reference + 5 ), 0 );
          else if ( !strcmp( reference, "$REF_PAR$" ) )
            Add_Cmd( Place_Mark, NULL, MARK_REF_PAR, 0 );
	  Free(reference);
	}
        if ( name2 )
          {
            pl_last->init_marking = name2;
          } /* if */
        if ( marking )
          {
            pl_last->curr_marking = marking;
          } /* if */
        if ( meaning )
          {
		char *c = strstr(meaning,"$NAME$");
		if (c) Add_Cmd(Place_Meaning,NULL,0,0);
		pl_last->meaning = meaning;
          }
        pl_last->component = 1;
	ScanBlocks( blocks, &(pl_last->blocks), &(pl_last->BlockEntries) );
	if(blocks)
	  Free(blocks);
        NNinput_line++;
      } /* while */

    /* Read in transition nodes **********************************************/
    ungetc( ch, infile );
    cmd=ReadCmdToken( infile );
    if ( strcmp( cmd, "TR" ) )
      {
        PEPerror( "keyword 'TR' expected" );
        return ( NULL );
      } /* if */
    ReadNewline( infile );

    count = 0;
    autonumbering = True;
    while ( ( ch = ReadCharComment( infile ) ) && ( isdigit(ch) ) )
      {
	read_TransTags();
	insert_Trans( &TrArray, &AnzTrNamen, &MaxTrNamen, 
		      &(net->NumTransitions),
		      &(net->Transitions), &(net->LastTransition), False );
      } /* while */

    /* Read in phantoms ******************************************************/
    ungetc( ch, infile );
    cmd=ReadCmdToken( infile );
    if ( !strcmp( cmd, "PTR" ) )
      {
        ReadNewline( infile );
	count = 0;
	autonumbering = True;

        while ( ( ch = ReadCharComment( infile ) ) && ( isdigit( ch ) ) )
          {
	    read_TransTags();
	    insert_Trans( &PTArray, &AnzPTNamen, &MaxPTNamen, 
			  &(net->NumPTransitions),
			  &(net->PTransitions), &(net->LastPTransition), True );
          } /* while */
        ungetc( ch, infile );
        cmd=ReadCmdToken( infile );
      } /* if */

    /* Read in T->P arc nodes ************************************************/
    if ( strcmp( cmd, "TP" ) )
      {
        PEPerror( "keyword 'TP' expected" );
        return ( NULL );
      } /* if */
    ReadNewline( infile );

    while ( ( ch = ReadCharComment( infile ) ) && ( ch != 'P' ) )
      {
	insert_TPArc( AnzTrNamen, TrArray, False );
      } /* while */

    /* Read in P->T arc nodes ************************************************/
    ungetc( ch, infile );
    cmd=ReadCmdToken( infile );
    if ( strcmp( cmd, "PT" ) )
      {
        PEPerror( "keyword 'PT' expected" );
        return ( NULL );
      } /* if */
    ReadNewline( infile );

    while ( ( ch = ReadCharComment( infile ) ) && isdigit( ch )
            && ! feof( infile ) )
      {
	insert_PTArc( AnzTrNamen, TrArray, False );
      } /* while */

    if ( ! feof( infile ) )
      {
        ungetc( ch, infile );
        cmd=ReadCmdToken( infile );

        /* Read in PT->P-Arcs ************************************************/
        if ( ! strcmp( cmd, "PTP" ) )
          {
            ReadNewline( infile );
            while ( ( ch = ReadCharComment( infile ) ) && ( isdigit( ch ) ) )
              {
		insert_TPArc( AnzPTNamen, PTArray, True );
              } /* while */
            ungetc( ch, infile );
            cmd=ReadCmdToken( infile );
          } /* if */

        /* Read in P->PT-Arcs ************************************************/
        if ( ! strcmp( cmd, "PPT" ) )
          {
            ReadNewline( infile );
            while ( ( ch = ReadCharComment( infile ) ) && ! feof( infile ) &&
                    ( isdigit( ch ) ) )
              {
		insert_PTArc( AnzPTNamen, PTArray, True );
              } /* while */

            if ( ! feof( infile ) )
              {
                ungetc( ch, infile );
                cmd=ReadCmdToken( infile );
              }
          } /* if */

        /* Read in additional text strings ***********************************/
        if ( ( ! feof(infile ) ) && ( ! strcmp( cmd, "TX" ) ) )
          {
            ReadNewline( infile );
            while ( ( ch = ReadCharComment( infile ) ) && ! feof( infile ) )
              {
                while ( ch != '\n' )
                  {
                    switch ( ch )
                      {
                        case '\'':
                        case '"' : ungetc( ch, infile );
                                   guard=MYstrdup(RL_ReadEnclString( infile ));
                                   break;
                        case 'N' : ReadCoordinates( infile, &px, &py );
                                   break;
                        default  : Error((errbuf, "unknown token '%c'", ch));
                      } /* switch */
                    ch = ReadCharComment( infile );
                  } /* while */
                NNinput_line++;
              } /* while */
          } /* if */
      } /* if */

    fclose( infile );

    net->x_dim -= net->x_pos; /* korrigiere x_max zu x_dim */
    net->y_dim -= net->y_pos; /* korrigiere y_max zu y_dim */

    /* now that the adresses of the phantom transitions are known
       -> parse phantom lists of the normal transitions */
    tr_last = net->Transitions;
    while ( tr_last )
      {
	if ( tr_last->phantoms )
	  {
	    chPtr = (char*) (tr_last->phantoms);
	    ScanPhantoms( chPtr, &(tr_last->phantoms), 
			  &(tr_last->PhantomEntries), PTArray ); 
	    MYfree( chPtr );
	  }
	tr_last = tr_last->nexttrans;
      }

  if ( (flag & MNET_SYNCHRONIZE) && REFfilename ) {
    register Cardinal32 AnzDefNamen,
                        MaxDefNamen;
    RefNr               * DefArray;
    SyncRefEntryPtr     ref;

void ScanRefs( char *refs, Ref* *refPtr, Cardinal *RefEntriesPtr )
  {
    while ( *refs )
      { 
	if ( sscanf( refs, "%d:%d%n*%n", &sx, &sy, &num, &num ) < 2 )
	  PEPerror( "reference specification has wrong format" );
	refs += num;
	
	if ( *RefEntriesPtr )
	  (*refPtr) = MYrealloc( (*refPtr), 
				 ( (*RefEntriesPtr) + 1) * sizeof( Ref ) );
	else
	  (*refPtr) = MYmalloc( sizeof( Ref ) );
	
	(*refPtr)[(*RefEntriesPtr)  ].nr   = sx;
	(*refPtr)[(*RefEntriesPtr)++].flag = sy;
      }
  }

    NNinput_file = REFfilename;
    NNinput_line = 1;
    count = MaxDefNamen = 1000;
    DefArray = (RefNr *) MYmalloc( count * sizeof( RefNr ) );
    AnzDefNamen = 0;
    while ( --count )
      {
        DefArray[ count ] = 0;
      } /* while */


    /* Oeffne REF-Datei */
    if ( ! ( infile = fopen( REFfilename, "r" ) ) )
      {
        fprintf( stderr,
               "ERROR: CM.read_REF(): could not open file '%s' for reading.\n",
                 REFfilename );
        exit ( 4 );
      } /* if */

    cmd=ReadCmdToken( infile );
    if ( strcmp( cmd, "PEP" ) )
      {
        PEPerror( "keyword 'PEP' expected" );
        return ( NULL );
      } /* if */
    ReadNewline( infile );
    cmd=ReadCmdToken( infile );
    if ( strncmp( cmd, "References", strlen("References") ) )
      {
        PEPerror( "format identifier 'References...' expected" );
      } /* if */
    ReadNewline( infile );
    cmd=ReadCmdToken( infile );
    if ( strcmp( cmd, "FORMAT_N" ) && strcmp( cmd, "FORMAT_N2" ) )
      {
        PEPerror( "keyword 'FORMAT_N' or 'FORMAT_N2' expected" );
        return ( NULL );
      } /* if */

    /* Read in Definitions */
    ReadNewline( infile );
    cmd=ReadCmdToken( infile );
    if ( strcmp( cmd, "DEF" ) )
      {
        PEPerror( "keyword 'DEF' expected" );
        return ( NULL );
      } /* if */
    ReadNewline( infile );

    count = 0;
    autonumbering = True;
    while ( ( ch = ReadCharComment( infile ) ) && ( isdigit(ch) ) )
      {
	count++; 
	ident = 0;
	invisible = 0;
	*name = '\0';
	meaning = reference = NULL;
	specified = False;
	while ( ch != '\n' )
	  {
	    if ( isdigit( ch ) )
	      {
		ungetc( ch, infile );
		num = ReadNumber( infile );
		if ( ( ident = num ) != count )
		  {
		    autonumbering = False;
		  } /* if */
		specified = True;
	      } /* if */ 
	    else
	      {
		switch ( ch )
		  {
                    case 'R' : reference=MYstrdup(RL_ReadEnclString( infile ));
			       break;
                    case 'C' : invisible = _CHANNEL;
		               ReadEnclString( infile, name, MEANINGSIZE );
                               break;
                    case 'S' : invisible = _STACK;
		               ReadEnclString( infile, name, MEANINGSIZE );
                               break;
                    case 'V' : invisible = _VAR;
		               ReadEnclString( infile, name, MEANINGSIZE );
                               break;
                    case 'U' : invisible = _PROC;
                               break;
                    case 'E' : invisible = _BEGIN;
		               ReadEnclString( infile, name, MEANINGSIZE );
                               break;
                    case 'A' : invisible = _ACTION;
                               break;
		    case 'b' : meaning=MYstrdup(RL_ReadEnclString( infile ));
                               break;
                    default  : Error((errbuf, "unknown token '%c'", ch));
		  } /* switch */
	      } /* else */
	    ch = ReadCharComment( infile );
	  } /* while */

	/* insert reference in the net */
	if ( autonumbering )
	  {
	    ident = count;
	  } /* if */
	else if ( ! specified )
	  {
	    PEPerror( "missing reference identifier" );
	  } /* if */
	if ( ident > AnzDefNamen )
	  {
	    AnzDefNamen = ident;
	  } /* if */
	else if ( DefArray[ ident ] != 0 )
	  {
	    Error((errbuf, "definition identifier %d multiply used", ident));
	  } /* else if */
	if ( AnzDefNamen >= MaxDefNamen )
	  {
	    DefArray = (RefNr *) MYrealloc( (char *) DefArray,
				   ( MaxDefNamen += 1000 ) * sizeof(RefNr));
	    for ( count = MaxDefNamen - 1000 ; count < MaxDefNamen ; count++ )
	      {
		DefArray[ count ] = 0;
	      } /* for */
	  } /* if */

	ref = MYmalloc( sizeof(SyncRefEntry) );
	if ( reference )
	  {
	    if ( sscanf( reference, "(%d,%d;%d,%d)", 
			 &(ref->s_ref.start_l), &(ref->s_ref.start_c),
			 &(ref->s_ref.end_l), &(ref->s_ref.end_c) ) != 4 )
	      PEPerror( "reference has wrong format" );
	    Free(reference);
	  }
	else
	  {
	    ref->s_ref.start_l = ref->s_ref.start_c =
	    ref->s_ref.end_l   = ref->s_ref.end_c   = 0;
	  }
	ref->s_type        = invisible;
	ref->s_description = (*name  ) ? MYstrdup( name )    : NULL;
	ref->s_meaning     = meaning;

	DefArray[ ident ] = StructToRef( (ShortEntryPtr) ref );
	NNinput_line++;
      } /* while */

    /* Read in block references */
    ungetc( ch, infile );
    cmd=ReadCmdToken( infile );

    if ( !strcmp( cmd, "BL" ) )
      {
        ReadNewline( infile );

        count = 0;
        autonumbering = True;
        while ( ( ch = ReadCharComment( infile ) ) && ch != 'P' )
          {
            count++; 
            ident = 0;
            *name = '\0';
            specified = False;
            while ( ch != '\n' )
              {
                if ( ch == 'B' )
                  {
                    num = ReadNumber( infile );

		    if ( ( ident = num ) != count )
		      {
			autonumbering = False;
		      } /* if */
		    specified = True;

                  } /* if */ 
                else
                  {
                    switch ( ch )
                      {
                        case 'T' : ReadEnclString( infile, name, NAMESIZE );
                                   break;
                        default  : Error((errbuf, "unknown token '%c'", ch));
                      } /* switch */
                  } /* else */
                ch = ReadCharComment( infile );
              } /* while */

            /* set block references */
            if ( autonumbering )
              {
                ident = count;
              } /* if */
            else if ( ! specified )
              {
                PEPerror( "missing block identifier" );
              } /* if */
            if ( ( ident > AnzBlNamen ) || ( BlArray[ ident ] == 0 ) )
              {
                Error((errbuf, "block identifier %d does not exist", ident));
              } /* if */

	    sscanf( name, "%d:%d", &cx, &cy );
	    BlockToStruct( BlArray[ident]->blocknr )->ref = cx;

	    NNinput_line++;
          } /* while */
        ungetc( ch, infile );
        cmd=ReadCmdToken( infile );
      } /* if */

    /* Read in place references **********************************************/
    if ( strcmp( cmd, "PL" ) )
      {
        PEPerror( "keyword 'PL' expected" );
           fprintf (stderr, "cmd: \"%s\"\n",cmd);
        return ( NULL );
      } /* if */

    count = 0;
    autonumbering = True;
    while ( ( ch = ReadCharComment( infile ) ) && ch != 'T' )
      {
	count++; 
	ident = 0;
	*name = '\0';
	specified = False;
	while ( ch != '\n' )
	  {
	    if ( ch == 'P' )
	      {
		num = ReadNumber( infile );
		
		if ( ( ident = num ) != count )
		  {
		    autonumbering = False;
		  } /* if */
		specified = True;
		
	      } /* if */ 
	    else
	      {
		switch ( ch )
		  {
		  case 'T' : ReadEnclString( infile, name, NAMESIZE );
		             break;
		  default  : Error((errbuf, "unknown token '%c'", ch));
		  } /* switch */
	      } /* else */
	    ch = ReadCharComment( infile );
	  } /* while */
	
	/* set place references */
	if ( autonumbering )
	  {
	    ident = count;
	  } /* if */
	else if ( ! specified )
	  {
	    PEPerror( "missing place identifier" );
	  } /* if */
	if ( ( ident > AnzPlNamen ) || ( PlArray[ ident ] == 0 ) )
	  {
	    Error((errbuf, "place identifier %d does not exist", ident));
	  } /* if */

	ScanRefs( name, &(PlArray[ident]->ref), &(PlArray[ident]->RefEntries) );

	NNinput_line++;
      } /* while */

    /* Read in transition references *****************************************/
    ungetc( ch, infile );
    cmd=ReadCmdToken( infile );
    if ( strcmp( cmd, "TR" ) )
      {
        PEPerror( "keyword 'TR' expected" );
        return ( NULL );
      } /* if */
    ReadNewline( infile );

    count = 0;
    autonumbering = True;
    while ( ( ch = ReadCharComment( infile ) ) && ch != 'P' && !feof( infile ) )
      {
	count++; 
	ident = 0;
	*name = '\0';
	specified = False;
	while ( ch != '\n' )
	  {
	    if ( ch == 'T' && TestDigit( infile ) )
	      {
		num = ReadNumber( infile );
		
		if ( ( ident = num ) != count )
		  {
		    autonumbering = False;
		  } /* if */
		specified = True;
		
	      } /* if */ 
	    else
	      {
		switch ( ch )
		  {
		  case 'T' : ReadEnclString( infile, name, NAMESIZE );
		             break;
		  default  : Error((errbuf, "unknown token '%c'", ch));
		  } /* switch */
	      } /* else */
	    ch = ReadCharComment( infile );
	  } /* while */
	
	/* set transition references */
	if ( autonumbering )
	  {
	    ident = count;
	  } /* if */
	else if ( ! specified )
	  {
	    PEPerror( "missing transition identifier" );
	  } /* if */
	if ( ( ident > AnzTrNamen ) || ( TrArray[ ident ] == 0 ) )
	  {
	    Error((errbuf, "transition identifier %d does not exist", ident));
	  } /* if */
	
	ScanRefs( name, &(TrArray[ident]->ref), &(TrArray[ident]->RefEntries) );

	NNinput_line++;
      } /* while */

    /* Read in phantom transition references *********************************/
    if ( !feof ( infile ) )
      {
	ungetc( ch, infile );
	cmd=ReadCmdToken( infile );
	if ( !strcmp( cmd, "PTR" ) )
	  {
	    ReadNewline( infile );

	    count = 0;
	    autonumbering = True;
	    while ( ( ch = ReadCharComment( infile ) ) && !feof( infile ) )
	      {
		count++; 
		ident = 0;
		*name = '\0';
		specified = False;
		while ( ch != '\n' )
		  {
		    if ( ch == 'P' )
		      {
			ch = ReadCharComment( infile ); /* T von PT */
			num = ReadNumber( infile );
		    
			if ( ( ident = num ) != count )
			  {
			    autonumbering = False;
			  } /* if */
			specified = True;
			
		      } /* if */ 
		    else
		      {
			switch ( ch )
			  {
			  case 'T' : ReadEnclString( infile, name, NAMESIZE );
			             break;
			  default  : Error((errbuf, "unknown token '%c'", ch));
			  } /* switch */
		      } /* else */
		    ch = ReadCharComment( infile );
		  } /* while */
	
		/* set phantom transition references */
		if ( autonumbering )
		  {
		    ident = count;
		  } /* if */
		else if ( ! specified )
		  {
		    PEPerror( "missing phantom transition identifier" );
		  } /* if */
		if ( ( ident > AnzPTNamen ) || ( PTArray[ ident ] == 0 ) )
		  {
		    Error((errbuf, 
			     "phantom transition identifier %d does not exist", 
			     ident));
		  } /* if */
		
		ScanRefs( name, &(PTArray[ident]->ref), 
                                &(PTArray[ident]->RefEntries) );

		NNinput_line++;
	      } /* while */
	  } /* if */
      }

    MYfree( DefArray );

    fclose( infile );
  }

    MYfree( PlArray );
    MYfree( TrArray );
    MYfree( PTArray );
    if ( BlArray )
      MYfree( BlArray );

    return( net );

  } /* read_HLnet() */
