/****************************************************************************/
/* RENAMING.C                                                               */
/*                                                                          */
/* Martin Ackermann, Fruehjahr 95                                           */
/****************************************************************************/

  /************/
  /* includes */
  /************/

#include "renaming.h"
#include "multiset.h"
#include "var_mset.h"
#include "tables.h"
#include <string.h>

  /**************/
  /* Funktionen */
  /**************/

/****************************************************************************/
/* Diese Routine versucht, in ren die v2 als Zuordnung fuer v1 einzutragen. */
/*                                                                          */
/* Wenn diese Zuordnung schon eingetragen ist oder neu eingetragen werden   */
/* konnte, wird TRUE zurueckgeliefert.                                      */
/*                                                                          */
/* Falls jedoch schon eine andere Zuordnung fuer v1 eingetragen ist oder    */
/* v2 schon als Zuordnung einer anderen Variablen eingetragen ist, ist      */
/* keine Umbenennung moeglich und es wird FALSE geliefert.                  */
/****************************************************************************/

bool    ren_entry( ComVar v1, ComVar v2, ComVar *ren, Cardinal renEntries )
{
  if ((v1 >= V_CONST_OFFSET  && v1 < V_TUPEL_OFFSET)
	|| (v2 >= V_CONST_OFFSET && v2 < V_TUPEL_OFFSET))
	return v1 == v2;
  if ( ren[v1] != UNUSED )  /* Ist schon eine Renaming fuer v1 eingetragen ? */
    return ( ren[v1]==v2 );    /* Stimmt altes Renaming mit neuem ueberein ? */ 
  else
    { 
      Cardinal i;

      for ( i = 0; i < renEntries; i++ )
        if ( ren[i] == v2 ) return False;/* Renaming mit v2 schon vergeben ? */

      ren[v1] = v2;                           /* Trage Renaming fuer v1 ein. */
      return True;
    }
}


/****************************************************************************/
/* Diese Routine versucht, fuer alle Kommunikationsvariablen der Aktionen   */
/* in mm1 eine Umbenennung zu finden, so dass schliesslich mm1 mit mm2      */
/* (bis auf die Umbenennung) uebereinstimmt.                                */
/* Dabei werden evtl. schon vorhandene Zuordnungen in ren beruecksichtigt.  */
/* Zusaetzliche Zuordnungen werden ebenfalls in ren eingetragen.            */
/*                                                                          */
/* Rueckgabe:  True, falls mm1 in mm2 umbenannt werden kann                 */
/*            False, falls mm1 unter Beruecksichtigung ven ren nicht in mm2 */
/*                   umbenannt werden kann.                                 */
/****************************************************************************/

bool    MSren_equal( MultiSet mm1, MultiSet mm2, 
                     ComVar *ren, Cardinal renEntries )
    { 
      Cardinal i;
                                  /* Die Mengen aktionsweise durchforsten:   */
      while ( mm1->anzahl )
      {                           /* Bei der ersten Nicht-Uebereinstimmung   */
                                  /* wird die weitere Suche abgebrochen und  */
                                  /* 'False' als Ergebnis geliefert.         */
        if ( mm1->anzahl != mm2->anzahl )
          return False;
        else
          { if ( mm1->nr != mm2->nr )
              return False;
            else
              { if ( mm1->arity != mm2->arity )
                  return False;
                else
                  for ( i = 0; i < mm1->arity; i++ )
                    if ( !ren_entry( mm1->com[i], mm2->com[i], 
                                    ren, renEntries ) )
                      return False;
              }
          }
        mm1++; mm2++;
      } /* while */

                                  /* An dieser Stelle wurde die erste Menge  */
                                  /* bis zum Ende abgearbeitet; das Ergebnis */
                                  /* des Vergleichs ist 'True', wenn jetzt   */
                                  /* auch die zweite Menge 'leer' ist, sonst */
                                  /* 'False'.                                */
      return ( !mm2->anzahl );
    }


/****************************************************************************/
/* Diese Routine versucht, fuer alle Kommunikationsvariablen in mm1 eine    */
/* Umbenennung zu finden, so dass schliesslich mm1 mit mm2 (bis auf die     */
/* Umbenennung) uebereinstimmt.                                             */
/* Dabei werden evtl. schon vorhandene Zuordnungen in ren beruecksichtigt.  */
/* Zusaetzliche Zuordnungen werden ebenfalls in ren eingetragen.            */
/*                                                                          */
/* Rueckgabe:  True, falls mm1 in mm2 umbenannt werden kann                 */
/*            False, falls mm1 unter Beruecksichtigung ven ren nicht in mm2 */
/*                   umbenannt werden kann.                                 */
/****************************************************************************/

bool    V_MSren_equal( VarMSet mm1, VarMSet mm2, 
                       ComVar *ren, Cardinal renEntries )
  {

                                  /* Die Mengen elementeweise durchforsten:  */
    while ( mm1->anzahl )
      {
                                  /* Bei der ersten Nicht-Uebereinstimmung   */
                                  /* wird die weitere Suche abgebrochen und  */
                                  /* 'False' als Ergebnis geliefert.         */
        if ( mm1->anzahl != mm2->anzahl )
          return False;
        else
	  {
	    if ( mm1->nr < V_CONST_OFFSET && mm2->nr < V_CONST_OFFSET )
	      {
		if ( !ren_entry( mm1->nr, mm2->nr, ren, renEntries ) )
		  return False;
	      }
	    else 
	      {
		if ( mm1->nr >= V_TUPEL_OFFSET && mm2->nr >= V_TUPEL_OFFSET )
		  {
		    Var *v1 = TplToArray( mm1->nr - V_TUPEL_OFFSET );
		    Var *v2 = TplToArray( mm2->nr - V_TUPEL_OFFSET );
		    Cardinal i;

		    if ( *v1 != *v2 )
		      return False;
	
		    for (i=1; i<*v1; i++)
		      if ( v1[i] < V_CONST_OFFSET && v2[i] < V_CONST_OFFSET )
			{
			  if ( !ren_entry( v1[i], v2[i], ren, renEntries ) )
			    return False;
			}
		      else
			{
			  if ( v1[i]>=V_CONST_OFFSET && v2[i]>=V_CONST_OFFSET )
			    {
			      if ( mm1->nr != mm2->nr )
				return False;
			    }
			  else /* unterschiedlicher Typ im Tupel */
			    return False;
			}
		  }
		else 
		  {
		    if (    mm1->nr<V_TUPEL_OFFSET && mm1->nr>=V_CONST_OFFSET
			 && mm2->nr<V_TUPEL_OFFSET && mm2->nr>=V_CONST_OFFSET )
		      {
			if ( mm1->nr != mm2->nr )
			  return False;
		      }
		    else /* unterschiedlicher Typ */
		      return False;
		  }
	      }
	  }
        mm1++; mm2++;
      } /* while */

                                  /* An dieser Stelle wurde die erste Menge  */
                                  /* bis zum Ende abgearbeitet; das Ergebnis */
                                  /* des Vergleichs ist 'True', wenn jetzt   */
                                  /* auch die zweite Menge 'leer' ist, sonst */
                                  /* 'False'.                                */
    return ( !mm2->anzahl );

  }


/****************************************************************************/
/* Liefert TRUE, falls fuer tr1 eine Umbenennung gefunden werden kann, so   */
/* dass tr1 und tr2 renaming equivalent sind.                               */
/****************************************************************************/

bool    Tr_are_equal( M_Transition tr1, M_Transition tr2 )
{ 
  M_STEdge	st1, st2;
  M_TSEdge	ts1, ts2;
  node_ptr      n1, n2;
  Integer       *d1, *d2;
  ComVar	ren[ MaxAnzComVars ];
  Cardinal	i, renEntries;

  renEntries = tr1->vars.ComEntries;
  for ( i = 0; i < renEntries; i++ )
    ren[i] = UNUSED;


  /* Renamings bzgl. der Aktionenmenge eintragen, sofern moeglich */

  if ( !MSren_equal( tr1->label, tr2->label, ren, renEntries ) )
    return False;


  /* Waehrend des Guard-Vergleichs Renamings eintragen, sofern moeglich */

  n1 = tr1->guard; n2 = tr2->guard;
  while( (n1->typ != N_HEAD || n1->op1 != 0) && 
	 (n2->typ != N_HEAD || n2->op1 != 0) )
    {
      if ( n1->typ != n2->typ ||
	   /* Typ vergleichen */

	   ( n1->typ != N_VAR && n1->op1 != n2->op1 ) ||
	   /* ausser bei Knoten vom Typ N_VAR auch op1 vergleichen */

	   ( n1->typ != N_HEAD  && n1->typ != N_CONST && 
	     n1->typ != N_KLAMM && n1->typ != N_NOT   &&
	     n1->typ != N_SIGN  && n1->op2 != n2->op2 ) ||
	   /* ausser bei den genannten Typen auch op2 vergleichen */
	   
	   ( n1->typ == N_VAR && !ren_entry(n1->op1, n2->op1, ren, renEntries) )
	   /* beim Typ N_VAR Renaming fuer op1 eintragen */

         ) return False;

      n1++; n2++;
    }

  if ( n1->typ != N_HEAD || n1->op1 != 0 || n2->typ != N_HEAD || n2->op1 != 0 )
    /* Ungleich viele Knoten vorhanden ? */
    return False;


  /* Stimmt Postset ueberein ? */

  ts1=tr1->postset; ts2=tr2->postset;
  while (ts1 && ts2 && ts1->place==ts2->place && 
	 V_MSren_equal( ts1->label, ts2->label, ren, renEntries ) )
    {
      ts1=ts1->nextplace;
      ts2=ts2->nextplace;
    }
  if (!ts1 && !ts2)
    {
      /* Der Preset etwa auch ? */
      
      st1=tr1->preset; st2=tr2->preset;
      while (st1 && st2 && st1->place==st2->place &&  
	     V_MSren_equal( st1->label, st2->label, ren, renEntries ) )
	{
	  st1=st1->nextplace;
	  st2=st2->nextplace;
	}
      if (!st1 && !st2) 
	{
	  /* Stimmen Bindungen fuer umbenannte Variablen ueberein ? */
	  
	  d1 = tr1->def + 1;
	  while( *d1 != -1 )
	    {
	      if ( ren[ *d1 ] != UNUSED )
		{
		  d2 = tr2->def + 1;
		  while( *d2 != -1 && *d2 != (Integer) ren[ *d1 ] )
		    d2 += *(d2+2) + 2;  /* suche Entsprechung in tr2->def */
		  if ( *d2 == -1 )
		    return False;
		  else
		    {
		      Cardinal i;
		      for( i=2; i<d1[2]+2; i++ )  /* vergleiche die Werte */
			if ( d1[i] != d2[i] )
			  return False;
		    }
		}
	      d1 += *(d1+2) + 2;
	    }
	  
	  return True;
	}
    }


  return False;
}

  /*** Dateiende renaming.c ***/
