/*****************************************************************************/
/*  tables.c                                                                 */
/*  Di.,27.10.1992 - Mi.,04.11.1992                           Stefan Roemer  */
/*                                                                           */
/*  Martin Ackermann, Sommer 95:                                             */
/*  - FunctionTable entfernt                                                 */
/*  - Groesse vom mmDummy & mmLeaf mit Faktor MaxComb vervielfacht           */
/*    (siehe multiset.c)                                                     */
/*  - RefTable hinzugefuegt                                                  */
/*                                                                           */
/*  Burkhard Bieber:                                                         */
/*    - Schoenheitskorrekturen                                               */
/*  last-update: 17.09.95                                                    */
/*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "multiset.h"
#include "tables.h"

/*****************************************************************************/
/*            Globale Variablen                                              */
/*****************************************************************************/

SymTab   ActTable = NULL;
SymTab   VarTable = NULL;
SymTab   RefTable = NULL;
SymTab   TplTable = NULL;

/*****************************************************************************/
/*             Hilfsfunktionen fuer den 'box expression'-Parser              */
/*****************************************************************************/
/*****************************************************************************/
/* Umwandeln eines Aktionsnamens ( String ) in entsprechenden Aktionscode    */
/* ( Cardinal ). Falls der Name noch nicht in der Symboltabelle enthalten    */
/* ist, wird er in derselben am Ende eingefuegt.                             */
/*****************************************************************************/

Action_nr StrToAction(char *Name)

  {

    register SymTab TblPtr = ActTable;

                                /* Den Namen in der Symboltabelle suchen     */
                                /* Lineare Suche ... eventuell schnelleres   */
                                /* Verfahren fuer die Symboltabellenverwal-  */
                                /* tung verwenden !!!                        */
    while ( *TblPtr && ( ( *Name != **TblPtr ) || strcmp( Name, *TblPtr ) ) )
      {
        TblPtr++;
      } /* while */
                                /* Falls Name nicht in Liste vorhanden:      */
                                /* am Ende der Tabelle einfuegen.            */
    if ( ! *TblPtr )
      {
        if ( ++ActCounter >= MaxActions )
          {
            int OffSet=(int)(TblPtr-ActTable);
            TblPtr = ( ActTable = (SymTab) MYrealloc( (char *) ActTable,
                              (Cardinal32) ( MaxActions += ACTION_BUF_SIZE )
                                                     * sizeof( SymTabEntry_Pars ) )
                     ) + OffSet;
            MSresize( mmDummy, MaxActions*MaxComb );
            MSresize( mmLeaf,  MaxActions*MaxComb );
            MSresize( mmSub,   MaxActions*MaxComb );
          } /* if */
        * TblPtr    = MYstrdup( Name );
        *(TblPtr+1) = NULL;
      } /* if */
                                /* Position des Namens in der Symboltabelle  */
                                /* in den Aktionscode umrechnen und zurueck- */
                                /* geben; in der Tabelle sind alle Namen     */
                                /* unkonjugiert!                             */
    return ( (Action_nr) ( ( TblPtr - ActTable + 1 ) << 1 ) );

  } /* StrToAction() */

/*****************************************************************************/
/* Umwandlung Aktion(scode) --> Aktionsname (String).                        */
/*****************************************************************************/

char * ActionToStr(Action_nr akt,char *str)

  {

    return ( strcat( strcpy( str, akt & 1 ? "^" : "" ),
                     ActionGetStr( akt ) ) );

  } /* ActionToStr() */

/*****************************************************************************/
/* Umwandeln eines Variablennamens ( String ) in entsprechenden Variablencode*/
/* ( Cardinal ). Falls der Name noch nicht in der Symboltabelle enthalten    */
/* ist, wird er in derselben am Ende eingefuegt.                             */
/*****************************************************************************/

Var StrToVariable(char *Name)

  {

    register SymTab TblPtr = VarTable;

    if ( Name[0] == VAR_RENAME_CHAR || Name[0] == VAR_MINOR_CHAR )
      Name++;

                                /* Den Namen in der Symboltabelle suchen     */
                                /* Lineare Suche ... eventuell schnelleres   */
                                /* Verfahren fuer die Symboltabellenverwal-  */
                                /* tung verwenden !!!                        */
    while ( *TblPtr && ( ( *Name != **TblPtr ) || strcmp( Name, *TblPtr ) ) )
      {
        TblPtr++;
      } /* while */
                                /* Falls Name nicht in Liste vorhanden:      */
                                /* am Ende der Tabelle einfuegen.            */
    if ( ! *TblPtr )
      {
        if ( ++VarCounter >= MaxVars )
          {
            int OffSet=(int)(TblPtr-VarTable);
            TblPtr = ( VarTable = 
		       (SymTab) MYrealloc( (char *) VarTable,
					   ( MaxVars += ACTION_BUF_SIZE )
					   * sizeof( SymTabEntry_Pars ) )
                     ) + OffSet;
          } /* if */
        * TblPtr    = MYstrdup( Name );
        *(TblPtr+1) = NULL;
      } /* if */
                                /* Position des Namens in der Symboltabelle  */
                                /* in den Variablencode umrechnen und        */
                                /* zurueckgeben.                             */
    return ( (Var) ( TblPtr - VarTable + 1 ) );

  } /* StrToVariable() */

/*****************************************************************************/
/* Umwandlung Variable(ncode) --> Variablenname (String).                    */
/*****************************************************************************/

char * VariableToStr(Var var,char *str)

  {
    char *org = VariableGetStr( var );

    if ( str )
      strcpy( str, org );

    return ( org );

  } /* VarToStr() */

/*****************************************************************************/
/* Umwandeln einer Referenz-Struktur in entsprechenden Referenzcode          */
/* ( Cardinal ). Falls die Struktur noch nicht in der Symboltabelle enthal-  */
/* ten ist, wird sie in derselben am Ende eingefuegt.                        */
/*****************************************************************************/

RefNr StructToRef( ShortEntryPtr r_ptr )

  {

    register SymTab TblPtr = RefTable;

    if ( flag & MNET_SYNCHRONIZE )
      {
	/* Im Synchronize-Modus soll jede Referenz aus dem Referenzfile
           aufgenommen werden, auch wenn die Quelltextstelle fehlt oder
           mit einer anderen Referenz uebereinstimmt. */
	TblPtr+=RefCounter;
      }
    else
      {
                                /* Die Struktur in der Symboltabelle suchen  */
                                /* Lineare Suche ... eventuell schnelleres   */
                                /* Verfahren fuer die Symboltabellenverwal-  */
                                /* tung verwenden !!!                        */
	while ( *TblPtr && !M_RefsAreEqual( (ShortEntryPtr) *TblPtr, r_ptr ) )
	  {
	    TblPtr++;
	  } /* while */
      }
                                /* Falls Struktur nicht in Liste vorhanden:  */
                                /* am Ende der Tabelle einfuegen.            */
    if ( ! *TblPtr )
      {
        if ( ++RefCounter >= MaxRefs )
          {
            int OffSet=(int)(TblPtr-RefTable);
            TblPtr = ( RefTable = 
		       (SymTab) MYrealloc( (char *) RefTable,
					   ( MaxRefs += ACTION_BUF_SIZE )
					   * sizeof( SymTabEntry_Pars ) )
                     ) + OffSet;
          } /* if */
        *TblPtr     = (SymTabEntry_Pars) r_ptr;
        *(TblPtr+1) = NULL;
      } /* if */
                                /* Position der Struktur in der Symboltabelle*/
                                /* in den Referenzcode umrechnen und         */
                                /* zurueckgeben.                             */
    return ( (Cardinal) ( TblPtr - RefTable + 1 ) );

  }

/*****************************************************************************/
/* Umwandlung Referenz(code) --> Referenz-Struktur.                          */
/*****************************************************************************/

ShortEntryPtr RefToStruct( RefNr ref )

  {
    return ( (ShortEntryPtr) (RefTable[ ref - 1 ]) );

  }

/*****************************************************************************/
/* Umwandeln eines Tupelarrays in den entsprechenden Tupelcode (Cardinal).   */
/* Falls das Array noch nicht in der Symboltabelle enthalten ist, wird es    */
/* in derselben am Ende eingefuegt.                                          */
/*                                                                           */
/* Um die Speicherverwaltung zu vereinfachen, geht der Speicher fuer das     */
/* Tupelarray stets in die Verwaltung der Symboltabellen ueber. Deshalb darf */
/* die aufrufende Prozedur spaeter nicht mehr auf den Tupel direkt zu-       */
/* greifen, sondern muss seine ggf. neue Adresse ueber TplToArray ermitteln. */
/*****************************************************************************/


static bool    M_TplsAreEqual( Var *tpl1, Var *tpl2 )

  {
    Cardinal i1 = 0;
    Cardinal i2 = 0;

    while ( (i1<*tpl1) && (tpl1[i1] == tpl2[i2]) )
      {
	i1++; 
	i2++;
      }
    return (i1 == *tpl1);
    
  }

Var ArrayToTpl( Var* *array_ptr )

  {

    register SymTab TblPtr = TplTable;

                                /* Den Tupel in der Symboltabelle suchen     */
                                /* Lineare Suche ... eventuell schnelleres   */
                                /* Verfahren fuer die Symboltabellenverwal-  */
                                /* tung verwenden !!!                        */
    while ( *TblPtr && !M_TplsAreEqual( (Var*) *TblPtr, *array_ptr ) )
      {
        TblPtr++;
      } /* while */
                                /* Falls Tupel nicht in Liste vorhanden:     */
                                /* am Ende der Tabelle einfuegen.            */
    if ( ! *TblPtr )
      {
        if ( ++TplCounter >= MaxTpls )
          {
            int OffSet=(int)(TblPtr-TplTable);
            TblPtr = ( TplTable = 
		       (SymTab) MYrealloc( (char *) TplTable,
					   ( MaxTpls += ACTION_BUF_SIZE )
					   * sizeof( SymTabEntry_Pars ) )
                     ) + OffSet;
          } /* if */
        * TblPtr    = (SymTabEntry_Pars) *array_ptr;
        *(TblPtr+1) = NULL;
      } /* if */
    else
      MYfree( *array_ptr );

    *array_ptr = NULL; /* Direktzugriffe durch die aufrufende Prozedur verhin-
                          dern, weil die Adresse evtl. ungueltig geworden ist.*/

                                /* Position des Tupels in der Symboltabelle  */
                                /* in den Tupelcode umrechnen und            */
                                /* zurueckgeben.                             */
    return ( (Cardinal) ( TblPtr - TplTable + 1 ) );

  }

/*****************************************************************************/
/* Umwandlung Tupel(code) --> Tupel-Array                                    */
/*****************************************************************************/

Var* TplToArray( Var tpl )

  {
    return ( (Var*) (TplTable[ tpl - 1 ]) );

  }

/*****************************************************************************/
/* Entfernen einer Symboltabelle aus dem Speicher.                           */
/*****************************************************************************/

void RemoveSymbolTable(SymTab *table)

  {

    register SymTab STPtr;

    if ((STPtr=*table)!=NULL)
      {
        while ( *STPtr )
          {
	    if ( *table!=RefTable )
              MYfree( *STPtr );
	    else
                 if ( flag & MNET_SYNCHRONIZE )
	      /* Im Synchronize-Modus gehoeren alle Eintraege der RefTable
                 zur RefTable und werden hier geloescht. */
		   {
		     if (      ((SyncRefEntryPtr)(*STPtr))->s_description )
		       MYfree( ((SyncRefEntryPtr)(*STPtr))->s_description );
		     if (      ((SyncRefEntryPtr)(*STPtr))->s_meaning     )
		       MYfree( ((SyncRefEntryPtr)(*STPtr))->s_meaning     );
		     MYfree( *STPtr );
		   }
	    else
		 if ( ((ShortEntryPtr)(*STPtr))->s_type==_ACTION   ||
		      ((ShortEntryPtr)(*STPtr))->s_type==_PROCCALL ||
		      ((ShortEntryPtr)(*STPtr))->s_type==_BEGIN     )
	      /* Die restlichen Eintraege der RefTable sind Verweise auf die
                 Symboltabelle und werden mit der Symboltabelle geloescht. */
	      MYfree( *STPtr );
	    STPtr++;
          } /* while */
        MYfree( *table );
        *table = NULL;
      } /* if */

  } /* RemoveSymbolTable() */

/*****************************************************************************/
/* Initialisiere globale Variablen: Symboltabellen.                          */
/*****************************************************************************/

void InitAllSymbolTables()

  /* GLOBAL: SymTab   ActTable,   VarTable,   RefTable;
             Cardinal MaxActions, MaxVars,    MaxRefs;
                      ActCounter, VarCounter, RefCounter; */

  {

                                /* Initialisieren der Aktionen-Symboltabelle.*/
    if ( ActTable )
      {
        RemoveSymbolTable( &ActTable );
      } /* if */
    ActTable = (SymTab) MYmalloc( MaxActions * sizeof( SymTabEntry_Pars ) );
    * ActTable = NULL;
    MaxActions = ACTION_BUF_SIZE;
    ActCounter = 0;

                                /* Initialisieren der Variablen-Symbol-      */
                                /* tabelle.                                  */
    if ( VarTable )
      {
        RemoveSymbolTable( &VarTable );
      } /* if */
    VarTable = (SymTab) MYmalloc( MaxVars * sizeof( SymTabEntry_Pars ) );
    * VarTable = NULL;
    MaxVars = ACTION_BUF_SIZE;
    VarCounter = 0;

                                /* Initialisieren der Referenzen-Symbol-     */
                                /* tabelle.                                  */
    if ( RefTable )
      {
        RemoveSymbolTable( &RefTable );
      } /* if */
    RefTable = (SymTab) MYmalloc( MaxRefs * sizeof( SymTabEntry_Pars ) );
    * RefTable = NULL;
    MaxRefs = ACTION_BUF_SIZE;
    RefCounter = 0;

                                /* Initialisieren der Tupel-Symboltabelle.   */
    if ( TplTable )
      {
        RemoveSymbolTable( &TplTable );
      } /* if */
    TplTable = (SymTab) MYmalloc( MaxTpls * sizeof( SymTabEntry_Pars ) );
    * TplTable = NULL;
    MaxTpls = ACTION_BUF_SIZE;
    TplCounter = 0;
  } /* InitAllSymbolTables() */

void RemoveAllSymbolTables()
{
      RemoveSymbolTable( &ActTable );  
      RemoveSymbolTable( &VarTable );
      RemoveSymbolTable( &RefTable );
      RemoveSymbolTable( &TplTable );
}

/*****************************************************************************/
/* Ausgabe der Aktionen-Symboltabelle auf dem Bildschirm.                    */
/* Nur zu Analysezwecken eingerichtet.                                       */
/*****************************************************************************/

void PrintActionTable()

  {

    register SymTab table;
    register Cardinal i = 2;

    printf( "table of action symbols:\n" );
    if ((table=ActTable)!=NULL)
      {
        while ( *table )
          {
            printf( "%6u. '%s'.\n", i++, *(table++) );
            i++;
          } /* while */
      } /* if */
    printf( "end of action symbols\n" );

  } /* PrintActionTable() */

/*****************************************************************************/
/* Ausgabe der Variablen-Symboltabelle auf dem Bildschirm.                   */
/* Nur zu Analysezwecken eingerichtet.                                       */
/*****************************************************************************/

void PrintVariableTable()

  {

    register SymTab table;
    register Cardinal i = 1;

    printf( "table of variable symbols:\n" );
    if ((table=VarTable)!=NULL)
      {
        while ( *table )
          {
            printf( "%6u. '%s'.\n", i++, *(table++) );
          } /* while */
      } /* if */
    printf( "end of variable symbols\n" );

  } /* PrintVariableTable() */

  /*** Dateiende tables.c ***/

