/*****************************************************************************/
/*  tables.h                                                                 */
/*  Di.,27.10.1992 - Mi.,04.11.1992                           Stefan Roemer  */
/*                                                                           */
/*  Martin Ackermann, Sommer 95:                                             */
/*  - FunctionTable entfernt                                                 */
/*  - Groesse vom mmDummy & mmLeaf mit Faktor MaxComb vervielfacht           */
/*    (siehe multiset.c)                                                     */
/*  - RefTable hinzugefuegt                                                  */
/*****************************************************************************/

#ifndef __TABLES_H__
#define __TABLES_H__

#include "main.h"
#include "mnet.h"

#define VariableGetStr( nr ) VarTable[ nr - 1 ]
#define   ActionGetStr( nr ) ActTable[ ( nr >> 1 ) - 1 ]

/*****************************************************************************/
/* Initialisierungsfunktion, die die vom Parser benutzten globalen Variablen */
/* ( insbesondere die Symboltabellen ) einrichtet. Diese Funktion ist unbe-  */
/* dingt zu Beginn des Hauptprogramms ( vor "Benutzung" des Parsers ) aufzu- */
/* rufen!                                                                    */
/*****************************************************************************/

extern void InitAllSymbolTables(void);
extern void RemoveAllSymbolTables(void);
  /* GLOBAL SymTab    ActTabelle,    (schreibend)
                      VarTable,      (schreibend)
                      RefTable,      (schreibend)
                      TplTable;      (schreibend)
            Cardinal  MaxActions,    (schreibend)
                      MaxVars,       (schreibend)
                      MaxRefs,       (schreibend)
		      MaxTpls,       (schreibend)
                      ActCounter,    (schreibend)
                      VarCounter,    (schreibend)
                      RefCounter,    (schreibend)
		      TplCounter     (schreibend) */

/*****************************************************************************/
/* Funktion zur Umwandlung eines Aktionsnamens `Name` (String) in den ent-   */
/* sprechenden Aktionscode (Cardinal bzw. Action), der als Ergebnis zurueck- */
/* geliefert wird. Falls der Name noch nicht in der Aktions-Symboltabelle    */
/* vorhanden ist, wird dieser hinzugefuegt und ein entsprechender Code fuer  */
/* ihn berechnet.                                                            */
/*****************************************************************************/

extern Action_nr StrToAction(char *Name);
  /* GLOBAL: SymTab   ActTable;    (schreibend)
             Cardinal ActCounter;  (schreibend)
             MultiSet mmDummy,     (schreibend)
                      mmLeaf       (schreibend) */

/*****************************************************************************/
/* Funktion zur Umwandlung eines Aktionscodes `akt` in den entsprechenden    */
/* Namen, der in der bereitgestellten Zeichenkette `str` abgelegt wird.      */
/* `str` sollte also ein Zeiger auf einen zur Aufnahme des Aktionsnamens ge- */
/* nuegend gross initialisierten Speicherbereich sein; `str` wird schliess-  */
/* lich als Funktionsergebnis geliefert.                                     */
/*****************************************************************************/

extern char *ActionToStr(Action_nr akt,char *str);
  /* GLOBAL: SymTab ActTable; (lesend)  */

/*****************************************************************************/
/* Funktion zum Umwandeln eines Variablennamens `Name` (String) in den ent-  */
/* sprechenden Variablencode (Var bzw. Cardinal), der als Funktionsergebnis  */
/* geliefert wird. Falls der Variablenname bisher noch nicht existiert, so   */
/* wird ein neuer Code fuer diesen ermittelt und der Name in die Variablen-  */
/* Symboltabelle aufgenommen.                                                */
/*****************************************************************************/

extern Var StrToVariable(char *Name);
  /* GLOBAL: SymTab   VarTable;   (schreibend)
             Cardinal VarCounter; (schreibend)  */

/*****************************************************************************/
/* Funktion zum Umwandeln des Variablencodes `var` in den entsprechenden     */
/* Variablennamen `str`, der in den dafuer zu initialisierenden Speicher-    */
/* bereich kopiert wird. Rueckgabe ist der Zeiger `str`.                     */
/*****************************************************************************/

extern char *VariableToStr(Var var,char *str);
  /* GLOBAL: SymTab VarTable; (lesend)  */

/*****************************************************************************/
/* Funktion zum Umwandeln einer Referenzstruktur in den entsprechenden       */
/* Referenzcode(Cardinal), der als Funktionsergebnis geliefert wird.         */
/* Falls die Referenzstruktur bisher noch nicht existiert, so                */
/* wird ein neuer Code fuer diese ermittelt und die Struktur in die          */
/* Referenzen-Symboltabelle aufgenommen.                                     */
/*****************************************************************************/

extern RefNr StructToRef( ShortEntryPtr r_ptr );
  /* GLOBAL: SymTab   RefTable;   (schreibend)
             Cardinal RefCounter; (schreibend)  */

/*****************************************************************************/
/* Funktion zum Umwandeln des Referenzcodes `ref` in die entsprechende       */
/* Referenzstruktur.                                                         */
/*****************************************************************************/

extern ShortEntryPtr RefToStruct( RefNr ref );
  /* GLOBAL: SymTab RefTable; (lesend)  */

/*****************************************************************************/
/* Funktion zum Umwandeln eines Tupelarrays in den entsprechenden            */
/* Tupelcode(Cardinal), der als Funktionsergebnis geliefert wird.            */
/*****************************************************************************/

extern Var ArrayToTpl( Var* *array );
  /* GLOBAL: SymTab   TplTable;   (schreibend)
             Cardinal TplCounter; (schreibend)  */

/*****************************************************************************/
/* Funktion zum Umwandeln des Tupelcodes `tpl` in das entsprechende          */
/* Tupelarray.                                                               */
/*****************************************************************************/

extern Var* TplToArray( Var tpl );
  /* GLOBAL: SymTab TplTable; (lesend)  */

/*****************************************************************************/
/* Funktion zur Freigabe des von der Symboltabelle `table` belegten Haupt-   */
/* speicherplatzes; `table` ist hiernach undefiniert!                        */
/*****************************************************************************/

extern void RemoveSymbolTable(SymTab *table);

/*****************************************************************************/
/* Prozeduren zur Darstellung der Symboltabelle `table` auf dem Bildschirm.  */
/* Nur zu Analysezwecken eingerichtet!                                       */
/*****************************************************************************/

extern void PrintActionTable(void);
extern void PrintVariableTable(void);

#endif  /* #ifndef __TABLES_H__ */

  /*** Dateiende tables.h ***/

