/*****************************************************************************/
/*  var_mset.h                                                               */
/*  C-Headerdatei: Multimengen ueber Aktionen -- Datenstrukturen und         */
/*                                               Funktionsprototypen/Makros  */
/*  Di., 03.11.1992 - Di.,02.02.1993                          Stefan Roemer  */
/*  Martin Ackermann, Fruehjahr 95                                           */
/*****************************************************************************/

#ifndef __V_MULTISET_H__
#define __V_MULTISET_H__


#include <stdio.h>
#include <stdlib.h>
#include "main.h"

#define  V_CONST_OFFSET  MaxAnzComVars
   /* ComVars aus [V_CONST_OFFSET..V_CONST_NULL  -1] sind neg. Konstanten.   */
#define  V_CONST_NULL (V_CONST_OFFSET - MIN_NEG)
   /* ComVars aus [V_CONST_NULL  ..V_TUPEL_OFFSET-1] sind pos. Konstanten.   */ 
#define  V_TUPEL_OFFSET  (V_CONST_NULL + NONE + 1)
   /* ComVars   >= V_TUPEL_OFFSET sind Tupel.                                */

/*****************************************************************************/
/* Globale Variablen fuer die Behandlung von Multimengen. Diese sind nur     */
/* deshalb hier global zusammengefasst worden, um die Aufrufe einiger        */
/* Funktionen einfacher zu halten. Sie duerfen auf keinen Fall veraendert    */
/* werden! Zu Beginn des Hauptprogramms sollte die Funktion `InitMultisets`  */
/* aufgerufen werden, um diese Variablen zu initialisieren!                  */
/*****************************************************************************/

extern VarMSet V_mmDummy;    /* Hilfsmenge bei der Berechnung von Summe und  */
                             /* Vereinigung von Multimengen zur Aufnahme der */
                             /* Ergebnismenge, die "laenger" sein kann als   */
                             /* jeder der "Summanden".                       */

extern VarMSet V_mmLeaf;     /* Hilfsmenge zur voruebergehenden Aufnahme von */
                             /* Multimengen beim Aufbau des Syntaxbaumes aus */
                             /* dem Box Expression.                          */


extern bool     MSdisplay_style;


/*****************************************************************************/
/* FUNKTIONEN AUF MULTIMENGEN:                                               */
/*****************************************************************************/

/*****************************************************************************/
/* Funktion zum Initialisieren/Allokieren der globalen Variablen des Multi-  */
/* mengen-Funktionspakets. Diese Funktion muss vom Hauptprogramm aufgerufen  */
/* werden, bevor mit Multimengen operiert wird!!!                            */
/*****************************************************************************/

extern void InitVarMSets( void );

/*****************************************************************************/
/* Funktion zum Einrichten einer neuen Multimenge, hier wegen der Kuerze als */
/* Makro realisiert. Nur eine mit dieser Funktion initialisierte Multimenge  */
/* ist physikalisch im Speicher vorhanden, und zwar mit der uebergebenen     */
/* "Laenge" `n`, d.h. sie kann spaeter hoechstens `n` verschiedene Aktionen  */
/* aufnehmen, wobei konjugierte Aktionen extra mitgezaehlt werden muessen,   */
/* Mehrfachvorkommen eines Aktionsnamens in der entsprechenden Menge jedoch  */
/* nicht. Spaeter wird auf die so initialisierten Multimengen mit Hilfe von  */
/* Zeigern verwiesen.                                                        */
/* Rueckgabe: Zeiger auf den so bereitgestellten Speicherbereich zur Aufnahme*/
/*            einer Multimenge mit hoechstens `n` verschiedenen Aktionen.    */
/*****************************************************************************/

                        /* Funktionalitaet:  MultiSet MSalloc( Cardinal n )  */
#define VSalloc( n ) (VarMSet) MYmalloc( ((n)+1) * sizeof( Variable ) )


/*****************************************************************************/
/* Funktion ( Makro ) zum Freigeben des von der Multimenge `mm` im Haupt-    */
/* speicher belegten Speicherplatzes.                                        */
/*****************************************************************************/

                           /* Funktionalitaet:  int MSremove( MultiSet mm )  */
                           /* Hier bewusst auf Indirektion `Multiset * mm`   */
                           /* verzichtet! Nach Ausfuehrung dieser Funktion   */
                           /* ist also `mm` undefiniert !!!                  */
#define VSremove( mm ) MYfree( mm )


/*****************************************************************************/
/* Funktion zum Kopieren einer Multimenge; kopiert `mm1 := mm2`.             */
/* Der Inhalt der Multimenge `mm2` wird in den Speicherbereich kopiert, den  */
/* die Multimenge `mm1` "belegt". Dabei muss darauf geachtet werden, dass    */
/* `mm1` genuegend gross zur Aufnahme der Multimenge initialisiert wurde.    */
/* Vorzugsweise wird es sich bei `mm1` also meistens um die globale Menge    */
/* `mmLeaf` handeln, die gross genug dimensioniert ist, da sie sogar alle    */
/* moeglichen vorkommenden Aktionsnamen aufnehmen kann.                      */
/* Zurueckgegeben wird der Zeiger `mm1`.                                     */
/*****************************************************************************/

extern VarMSet V_MScopy( VarMSet mm1, VarMSet mm2, ComVar *sub );

/*****************************************************************************/
/* Funktion zur Initialisierung einer neuen Multimenge, die als einzige      */
/* Aktion `akt` enthaelt, und zwar genau einmal: `{1 akt}` bzw. `{akt}`.     */
/* Rueckgabe: Zeiger auf den so initialisierten Speicherbereich der          */
/*            "Laenge" 1.                                                    */
/*****************************************************************************/

extern VarMSet V_MScreate_singleton( ComVar var );

/*****************************************************************************/
/* Funktion zur Initialisierung einer neuen Multimenge als Kopie einer       */
/* bereits vorhandenen Multimenge `mm`; der Speicherbedarf der neu allokier- */
/* ten Menge entspricht ihrer tatsaechlichen "Laenge".                       */
/* Ein Zeiger auf diese neu eingerichtete Menge wird zurueckgegeben.         */
/*****************************************************************************/

extern VarMSet V_MScreate_copy( VarMSet mm, ComVar *sub );

/*****************************************************************************/
/* Funktion zur Berechnung der Maechtigkeit einer Multimenge `mm`: `|mm|`.   */
/* Rueckgabe ist die so ermittelte Kardinalitaet ( als Cardinal ).           */
/*****************************************************************************/

extern Cardinal V_MScardinality( VarMSet mm );

/*****************************************************************************/
/* Funktion zur Berechnung, wie oft eine Aktion `akt` in einer Multimenge    */
/* `mm` enthalten ist: `#akt( mm )`.                                         */
/* Rueckgabe: ermittelte Anzahl Vorkommen ( als Cardinal ).                  */
/*****************************************************************************/

extern Cardinal V_MScount( ComVar var, VarMSet mm );

/*****************************************************************************/
/* Funktion zur Feststellung, ob zwei Multimengen `mm1`, `mm2` gleich sind.  */
/* Rueckgabe: True,  falls `mm1 == mm2`, d.h. `mm1` enthaelt genau die glei- */
/*                   chen Elemente wie `mm2` ( in der gleichen Anzahl ).     */
/*            False, falls `mm1 != mm2`.                                     */
/*****************************************************************************/

extern bool    V_MSis_equal( VarMSet mm1, VarMSet mm2 );

/*****************************************************************************/
/* Funktion zur Summenbildung zweier Multimengen `mm1` und `mm2`: es wird    */
/* `mm1 := mm1 + mm2` berechnet, d.h. `mm2` wird zu `mm1` "hinzuaddiert".    */
/* Rueckgabe: Zeiger auf die Ergebnismenge `mm1`, deren alter Inhalt ueber-  */
/*            schrieben wird.                                                */
/* Seiteneffekt: Benutzung der globalen Variablen `mmDummy` zur Aufnahme     */
/*               des Zwischenergebnisses.                                    */
/*****************************************************************************/

extern VarMSet V_MSsum( VarMSet mm1, VarMSet mm2 );

/*****************************************************************************/
/* Funktion zum Schreiben einer Multimenge `mm` in eine Datei `fileptr`.     */
/* Dabei beeinflusst die globale Variable `MSdisplay_style` das Ausgabe-     */
/* format fuer mehrfach vorkommende Namen:                                   */
/* MSdisplay_style == MS_WIDE  ==>  {a, a, ^b, ^b, ^b}                       */
/*                 == MS_SHORT ==>  {2 a, 3 ^b}                              */
/*****************************************************************************/

extern void V_MSWriteMultiset( FILE *fileptr, VarMSet mm, void *vars );

  /* GLOBAL: MSdisplay_style; (lesend) */


/*****************************************************************************/


#endif /* __V_MULTISET_H__ */


/*****************************************************************************/
/* END OF FILE multiset.h                                                    */
/*****************************************************************************/

