/**********************************************************************/
/* > box.c <                                                          */
/* Funktionen, die bei Abarbeitung des Syntaxbaumes aufgerufen werden */
/* Burkhard Graves, 17.12.'93                                         */
/**********************************************************************/

#include <stdio.h>
#include <string.h>
#include "box.h"
#include "boxtypes.h"
#include "box_sub.h"
#include "dat_cha.h"
#include "iter.h"
#include "move.h"
#include "main.h"

/***********************************/
/* Basic Box mit Multimengen-Label */
/***********************************/

BOX BOXMultiLabelAction(TTree Knoten)
{
    OUT("\tMultiLabelAction\n");
    return CreateBasicBox((MultiSet)Knoten->Operand1,
                          (Time_stamp *)Knoten->Operand2);
} /* BOXMultiLabelAction() */

/*********************************/
/* Basic Box mit Variablen-Label */
/*********************************/

BOX BOXVariable(TTree Knoten)
{
    LPNet net;
    StackElem *ptr;

    OUT("\tVariable\n");
    ptr=search((Var)*(((MultiSet)Knoten->Operand1)+1));
    if (ptr) {
        if (ptr->count) {                               /* Rekursion */
            if (--ptr->count)
                net=Eval(ptr->knoten);
            else
                (net=OnlyPlaces(0))->Places->numname=1;
            ++ptr->count;
        } else {                                        /* Verfeinerung */
            net=LPNCreateNetCopy((LPNet)ptr->knoten);
        }
    } else
        net=CreateBasicBox((MultiSet)Knoten->Operand1,
                           (Time_stamp *)Knoten->Operand2);
    return net;
} /* BOXVariable() */

/***********/
/* Sequenz */
/***********/

BOX BOXSequence(TTree Knoten)
{
    register LPNet    net1,net2;
    register Place    pl1,pl2;
             STEdge   stdummy;
             TSEdge   tsdummy;
             Cardinal x_dim;
             int      x_pos;

    net1=Eval(Knoten->Operand1);
    net2=Eval(Knoten->Operand2);
    OUT("\tSequence\n");

    move_hv(net2,net1->x_pos-net2->x_pos
                 +(int)(net1->x_dim-net2->x_dim)/2,
                 net1->y_pos-net2->y_pos+net1->y_dim-1);
    if (net2->x_dim>net1->x_dim) {
        net1->x_dim=net2->x_dim;
        net1->x_pos=net2->x_pos;
    }
    net1->y_dim+=net2->y_dim-1;

    x_dim=0;
    PreparePlaces(net2,ENTRY,ENTRY);
    pl1=net1->LastPlace;
    while (pl1 && pl1->numname) {
        if (pl1->label==EXIT) {   /* pl1 laeuft durch die net1-EXIT-Stellen  */
            pl2=net2->LastPlace;  /* pl2 laeuft durch die net2-ENTRY-Stellen */
            while (pl2) {
                x_dim+=2;
                DuplicatePlace(net1,pl1,&stdummy,&tsdummy)->label=EMPTY;
                net1->Places->postset
                =CopyPlacePostset(pl2,net1->Places,&stdummy);
                pl2=pl2->prevplace;
            }
            pl1=LPNRemovePlace(net1,pl1);
        } else
             pl1=pl1->prevplace;
    }

    x_pos=(int)(net1->x_dim-(--x_dim))/2+net1->x_pos;
    if (x_dim>net1->x_dim) {
        net1->x_dim=x_dim;
        net1->x_pos=x_pos;
    }
    while (pl1) {
        x_pos=(pl1->x_pos=x_pos)+2;
        pl1->numname=1;
        pl1=pl1->prevplace;
    }

    return DisjointNetUnion(net1,RestoreKillPlaces(net2,ENTRY));
} /* BOXSequence() */

/***********/
/* Auswahl */
/***********/

BOX BOXChoice(TTree Knoten)
{
    register LPNet    net1,net2;
    register Place    pl1,pl2;
             Place    merke;
             STEdge   stlast,stdummy;
             TSEdge   tslast,tsdummy;
             Cardinal x_dim;
             int      x_pos;

    net1=Eval(Knoten->Operand1);
    net2=Eval(Knoten->Operand2);
    OUT("\tChoice\n");

    juxtapose(net1,net2);

    merke=PreparePlaces(net2,ENTRY,EXIT);

    x_dim=0;
    pl1=net1->LastPlace;
    while (pl1 && pl1->numname) {
        if (pl1->label==ENTRY) {  /* pl1 laeuft durch ENTRY-Stellen von net1 */
            pl2=net2->LastPlace;  /* pl2 laeuft durch ENTRY-Stellen von net2 */
            while (pl2) {
                x_dim+=2;
                if (DuplicatePlace(net1,pl1,&stlast,&tslast)->postset!=NULL) {
                    if ((stlast->nexttrans
                         =CopyPlacePostset(pl2,net1->Places,&stdummy))!=NULL)
                        stlast->nexttrans->prevtrans=stlast;
                } else
                    net1->Places->postset
                    =CopyPlacePostset(pl2,net1->Places,&stdummy);
                pl2=pl2->prevplace;
            }
            pl1=LPNRemovePlace(net1,pl1);
        } else
            pl1=pl1->prevplace;
    }

    x_pos=(int)(net1->x_dim-(--x_dim))/2+net1->x_pos;
    if (x_dim>net1->x_dim) {
        net1->x_dim=x_dim;
        net1->x_pos=x_pos;
    }
    while (pl1) {
        x_pos=(pl1->x_pos=x_pos)+2;
        pl1->numname=1;
        pl1=pl1->prevplace;
    }

    x_dim=0;
    pl1=net1->LastPlace;
    while (pl1 && pl1->numname) {
        if (pl1->label==EXIT) {    /* pl1 laeuft durch EXIT-Stellen von net1 */
            pl2=merke;             /* pl2 laeuft durch EXIT-Stellen von net2 */
            while (pl2) {
                x_dim+=2;
                if (DuplicatePlace(net1,pl1,&stlast,&tslast)->preset!=NULL) {
                    if ((tslast->nexttrans
                         =CopyPlacePreset(pl2,net1->Places,&tsdummy))!=NULL)
                        tslast->nexttrans->prevtrans=tslast;
                } else
                    net1->Places->preset
                    =CopyPlacePreset(pl2,net1->Places,&tsdummy);
                pl2=pl2->prevplace;
            }
            pl1=LPNRemovePlace(net1,pl1);
        } else
            pl1=pl1->prevplace;
    }

    x_pos=(int)(net1->x_dim-(--x_dim))/2+net1->x_pos;
    if (x_dim>net1->x_dim) {
        net1->x_dim=x_dim;
        net1->x_pos=x_pos;
    }
    while (pl1) {
        x_pos=(pl1->x_pos=x_pos)+2;
        pl1->numname=1;
        pl1=pl1->prevplace;
    }

    return DisjointNetUnion(net1,RestoreKillPlaces(net2,ENTRY | EXIT));
} /* BOXChoice() */

/********************/
/* Nebenlaeufigkeit */
/********************/

BOX BOXParallel(TTree Knoten)
{
    LPNet net1,net2;

    net1=Eval(Knoten->Operand1);
    net2=Eval(Knoten->Operand2);
    OUT("\tConcurrentComposition\n");

    juxtapose(net1,net2);
/* sollte eigentlich berflssig sein
    net1->x_pos=net1->x_pos;
    net2->y_pos=net1->y_pos;
    net2->x_dim=net1->x_dim;
    net2->y_dim=net1->y_dim;
*/
    return DisjointNetUnion(net1,net2);
} /* BOXParallel() */

/*******************/
/* Synchronisation */
/*******************/

BOX BOXSynchronisation(TTree Knoten)
{
    LPNet net;
    Cardinal32 numTrans;

    net=Eval(Knoten->Operand1);
    numTrans=net->NumTransitions;
    OUT("\tSynchronisation\n");
    net=Synchronisation(net,(MultiSet)Knoten->Operand2,Knoten->Depth);
    if (numTrans==net->NumTransitions)
      WARNING("Synchronisation found no matching symbols!");

    return net;

} /* BOXSynchronisation() */

/*************************************************************************/
/* Anwendung der Restriktions-Operation auf das Netz 'net', welches auch */
/* den Rueckgabewert darstellt.                                          */
/*************************************************************************/

BOX BOXRestriction(TTree Knoten)
  {
    LPNet net;

    net=Eval(Knoten->Operand1);
    OUT("\tRestriction\n");
    return Restriction(net,(MultiSet)Knoten->Operand2);
  } /* BOXRestriction() */

/***********/
/* Scoping */
/***********/

BOX BOXScoping(TTree Knoten)
{
    LPNet net;

    net=Eval(Knoten->Operand1);
    OUT("\tScoping\n");

    return (bpn_modus)?
        Scoping(net,(MultiSet)Knoten->Operand2,Knoten->Depth)
        :
        Restriction(
            Synchronisation(net,(MultiSet)Knoten->Operand2,Knoten->Depth),
            (MultiSet)Knoten->Operand2);

} /* BOXScoping() */

BOX BOXTie(TTree Knoten)
{
  LPNet net;
  Place place;
  Transition tr;
  STEdge st;
  TSEdge ts;
  MultiSet mm;
  Cardinal32 weight, weight2;
  Action akt;
  char* str=(char*)MYmalloc(50*sizeof(char));

  net=Eval(Knoten->Operand1);
  OUT("\tTie-Operator\n");

  for (mm=(MultiSet)Knoten->Operand2;*mm;mm++)
    {
      ActionToStr(*mm,str);
      akt=*mm++|2; /* Search for link symbols */
      
      /* Neue Stelle einfuegen */
      place=LPNallocPlace();
      net->NumPlaces++;
      if ((place->nextplace=net->Places))
	{
	  place->nextplace->prevplace=place;
	}
      else
	{
	  net->LastPlace=place;
	}
      net->Places=place;
      place->prevplace=NULL;
      place->name=strdup(str);
      place->numname=1;
      place->component=1;
      place->label=EMPTY;
      place->preset=NULL;
      place->x_pos=place->y_pos=0;
      place->marks=0;
      place->tokenage=0;

      /* Fuer jede Transition ggf. Kante(n) einfuegen */
      for (tr=net->LastTransition;tr;tr=tr->prevtrans)
	{

	  if ((weight=MScount(akt,tr->label)))
	    { /* count exports */
	      ts=LPNallocTSEdge();
	      ts->nextplace=tr->postset;
	      ts->nexttrans=place->preset;
	      place->preset=tr->postset=ts;
	      ts->prevplace=ts->prevtrans=NULL;
	      ts->place=place;
	      ts->trans=tr;
	      ts->weight=weight;
	    }
	  if ((weight2=MScount(akt|1,tr->label)))
	    { /* count imports */
	      st=LPNallocSTEdge();
	      st->nextplace=tr->preset;
	      st->nexttrans=place->postset;
	      place->postset=tr->preset=st;
	      st->prevplace=st->prevtrans=NULL;
	      st->place=place;
	      st->trans=tr;
	      st->weight=weight2;
	    }
	  place->x_pos+=(tr->x_pos-place->x_pos)>>1;
	  place->y_pos+=(tr->y_pos-place->y_pos)>>1;
	  MSremove_action_pairs(tr->label,akt,(weight<weight2)?weight2:weight);
	}
      
      /* Wenn Stelle unverbunden, wieder loeschen */
      if (!(place->postset||place->preset))
	{
	  WARNING("Tie operator found no link symbols!");
	  net->Places=place->nextplace;
	  net->Places->prevplace=NULL;
	  LPNremovePlace(place);
	}
    }
  free(str);
  return net;

} /* BOXTie() */

/**************************************************************/
/* Anwendung einer Relabelling-Funktion 'f' auf das Netz.     */
/* Rueckgabewert ist das (moeglicherweise) manipulierte Netz. */
/**************************************************************/

BOX BOXRelabelling(TTree Knoten)
  {
    LPNet      net;
    Function   f;
    Transition tr;

    net=Eval(Knoten->Operand1);
    OUT("\tRelabelling: ");
    if ((Cardinal)(Cardinal32)Knoten->Operand2==FUNCTION)
      {
        OUT("\tFUNCTION NAME\n");
        f=FunctionTable[(Cardinal)(Cardinal32)Knoten->Operand3-1].FuncMappings;
      }
    else
      {    /* (Cardinal)Knoten->Operand2==MAP ! */
        OUT("\tMAPPING\n");
        f=(Function)Knoten->Operand3;
      }

    if (*f)
      {
        tr=net->Transitions;
        while (tr)
          {
            tr->label=MSrelabel(tr->label,f);
            tr=tr->nexttrans;
          }
      }
    return net;
  } /* BOXRelabelling() */

/****************/
/* Verfeinerung */
/****************/

BOX BOXRefinement(TTree Knoten)
{
    LPNet net,rnet;

    OUT("\tRefinement\n");
    rnet=Eval(Knoten->Operand3);
    push((Var)(Cardinal32)Knoten->Operand2,(Cardinal)0,(TTree)rnet);
    net=Eval(Knoten->Operand1);
    LPNRemove_net(&rnet);
    pop();
    return net;
} /* BOXRefinement() */

/*************/
/* Rekursion */
/*************/

BOX BOXRecursion(TTree Knoten)
{
    LPNet net;

    OUT("\tRecursion\n");
    push((Var)(Cardinal32)Knoten->Operand1,Knoten->Depth,Knoten->Operand2);
    net=Eval(Knoten->Operand2);
    pop();
    return net;
} /* BOXRecursion() */

/*************/
/* Iteration */
/*************/

BOX BOXIteration(TTree Knoten)
{
    OUT("\tIteration\n");
    return (flag&BIG_LOOP)?
        BigIteration(Knoten):(flag&MEDIUM_LOOP)?
            MediumIteration(Knoten):SmallIteration(Knoten);
} /* BOXIteration */

/************************/
/* Filename-Operationen */
/************************/

BOX BOXFilename(TTree Knoten)
{
    OUT("\tFilename-Operation\n");
    PEPerror("\tFilename operation not yet implemented !!!");
    /* um Warnungen zu vermeiden */
    Knoten=Knoten;
    return NULL;
}

/***************/
/* Channel-Box */
/***************/

BOX BOXChannel(TTree Knoten)
{
    OUT("\tChannel-Box\n");
    return ChannelBox(Knoten);
}

/*************/
/* Stack-Box */
/*************/

BOX BOXStack(TTree Knoten)
{
    OUT("\tStack-Box\n");
    return (Knoten->Depth)?StackBox(Knoten):ChannelBox(Knoten);
}

/************/
/* Data-Box */
/************/

BOX BOXData(TTree Knoten)
{
    OUT("\tData-Box\n");
    return DataBox(Knoten);
}

/************/
/* Stop-Box */
/************/

BOX BOXStop(TTree Knoten)
{
    LPNet net;

    OUT("\tStop-Box\n");
    /* Um Warnungen zu vermeiden */
    Knoten=Knoten;
    (net=OnlyPlaces(0))->Places->numname=1;
    return net;
}

/*******************/
/* Atomare Auswahl */
/*******************/

BOX BOXAtomChoice(TTree Knoten)
{
    register LPNet      net1;
    register Place      pl;
    register Transition tr,tr1;
    register STEdge     st;
    register TSEdge     ts;

    if ((Knoten->Operand1->Operator!=ATOM_CHOICE
         && Knoten->Operand1->Operator!=ACTION
        ) || Knoten->Operand2->Operator!=ACTION)
        PEPerror("\'#\' wrongly used !");

    net1=Eval(Knoten->Operand1);

    OUT("\tAtomChoice\n");

    tr1=net1->Transitions;

    (tr=LPNallocTrans())->numname=1;
    tr->label=MScreate_copy((MultiSet)Knoten->Operand2->Operand1);
    tr->bpn_label=NULL;
    tr->prevtrans=NULL;
    net1->Transitions=((tr->nexttrans=tr1)->prevtrans=tr);
    ++net1->NumTransitions;
    tr->y_pos=tr1->y_pos;
    if (flag&BUNDLE)
        tr->x_pos=tr1->x_pos;
    else {
        while (tr1) {
            --tr1->x_pos;
            tr1=tr1->nexttrans;
        }
        tr->x_pos=--net1->x_pos+(++net1->x_dim);
        ++net1->x_dim;
    }

    pl=net1->Places;
    (tr->preset=(st=LPNallocSTEdge()))->weight=1;
    st->nextplace=st->prevplace=st->prevtrans=NULL;
    pl->postset=((st->nexttrans=(st->place=pl)->postset)->prevtrans=st);
    st->trans=tr;

    pl=net1->LastPlace;
    (tr->postset=(ts=LPNallocTSEdge()))->weight=1;
    ts->nextplace=ts->prevplace=ts->prevtrans=NULL;
    pl->preset=((ts->nexttrans=(ts->place=pl)->preset)->prevtrans=ts);
    ts->trans=tr;

    return net1;
} /* BOXAtomChoice() */

/****************************************/
/* Initialisierung der Funktionstabelle */
/****************************************/

BOX (* Operation[ S_BOX - ACTION + 1 ])( TTree );

void BOXInitOpArray()
{
    unsigned char OpZaehler = 0;

    Operation[OpZaehler++] = BOXMultiLabelAction;
    Operation[OpZaehler++] = BOXVariable;
    Operation[OpZaehler++] = BOXSequence;
    Operation[OpZaehler++] = BOXChoice;
    Operation[OpZaehler++] = BOXParallel;
    Operation[OpZaehler++] = BOXSynchronisation;
    Operation[OpZaehler++] = BOXRestriction;
    Operation[OpZaehler++] = BOXScoping;
    Operation[OpZaehler++] = BOXTie;
    Operation[OpZaehler++] = BOXRelabelling;
    Operation[OpZaehler++] = BOXRefinement;
    Operation[OpZaehler++] = BOXRecursion;
    Operation[OpZaehler++] = BOXIteration;
    Operation[OpZaehler++] = BOXFilename;
    Operation[OpZaehler++] = BOXChannel;
    Operation[OpZaehler++] = BOXData;
    Operation[OpZaehler++] = BOXStop;
    Operation[OpZaehler++] = BOXAtomChoice;
    Operation[OpZaehler  ] = BOXStack;

} /* BOXInitOpArray() */

/*********************/
/* END OF FILE box.c */
/*********************/
