/**********************************************************************/
/* > box_sub.c <                                                      */
/* Unterroutinen von Funktionen in box.c                              */
/* Burkhard Graves, 26.6.'93                                          */
/* Aenderung 8.6.1999: CreateTr aus iter.c/dat_cha.c -Christian Stehno*/
/**********************************************************************/

#include "box_sub.h"
#include "boxtypes.h"
#include "multiset.h"
#include "move.h"

BOX (* Operation[ S_BOX - ACTION + 1 ])( TTree );


/*******************/
/* Stack-Klamotten */
/*******************/

StackElem *top=NULL;

void push(Var var,Cardinal count,TTree knoten)
{
    register StackElem *ptr;

    (ptr=(StackElem *)MYmalloc(sizeof(StackElem)))->previous=top;
    (top=ptr)->var=var;
    ptr->count=count;
    ptr->knoten=knoten;
}

void pop(void)
{
    register StackElem *ptr;

    if ((ptr=top)!=NULL) {
        top=ptr->previous;
        free(ptr);
    }
}

StackElem *search(Var var)
{
    register StackElem *ptr;

    ptr=top;
    while (ptr && ptr->var!=var) {
        ptr=ptr->previous;
    }
    return ptr;
}

/*****************************************************************************/
/* Unterroutine von 'Synchronisation()' und 'Scoping()'.                     */
/* Durch die Synchronisation der Transitionen 'tr1' und 'tr2' bezueglich der */
/* Aktion 'action' entsteht im Netz 'net' eine neue Transition mit           */
/* dazugehoerigen neuen Kanten. Ueberschreitet das Kantengewicht einer dieser*/
/* neuen Kanten den Parameter 'max', so wird die neue Transition sofort      */
/* wieder geloescht; dadurch wird das Entstehen unendlich grosser Netze      */
/* (bzw. - realistisch gesehen - ein Programmabbruch wegen Speichermangels)  */
/* verhindert.                                                               */
/*****************************************************************************/

static Transition sub_Sync_Scope(LPNet net,Transition tr1,Transition tr2,
                                 Action action,Cardinal max)
{
    STEdge st1,st2,newst;
    TSEdge ts1,ts2,newts;

    /* neue Transition mit korrektem Label bilden */

    MScopy(mmLeaf,tr1->label);
    MSsum(mmLeaf,tr2->label);
    tr2=DuplicateTrans(net,tr2,&newst,&newts);
    tr2->label=MScreate_copy(MSremove_action_pairs(mmLeaf,action,1));
    tr2->numname=1;

    /* Neue Zeitbeschriftung setzen, tr2 hat schon eine Kopie */
    if (tr1->time_label)
      {
        if (!tr2->time_label) /* Label kopieren */
          {
            tr2->time_label=(Time_stamp*)malloc(sizeof(Time_stamp));
            tr2->time_label->low=tr1->time_label->low;
            tr2->time_label->high=tr1->time_label->high;
          }
        else /* Durchschnitt bilden */
          {
            if (tr2->time_label->low<tr1->time_label->low)
              tr2->time_label->low=tr1->time_label->low;
            if (tr2->time_label->high>tr1->time_label->high)
              tr2->time_label->high=tr1->time_label->high;
          }
      } /* if tr1 */

    /* Die durch Duplikation von Transition 'tr2' neu entstandene Transition */
    /* um den Preset von Transition 'tr1' ergaenzen.                         */
    st1=tr1->preset; st2=tr2->preset;
    while (st1) {
        if (st1->place->postset->trans!=tr2) { /* Weitere Kante hinzufuegen, */
            while (st2->nextplace &&
                   st2->place->numname<st1->place->numname)     /* aber wo ? */
                st2=st2->nextplace;
            (newst=LPNallocSTEdge())->prevtrans=NULL;
            newst->trans=tr2;
            newst->place=st1->place;
            newst->weight=st1->weight;
            newst->nexttrans=st1->place->postset;
            st1->place->postset=newst->nexttrans->prevtrans=newst;
            if (st2->place->numname>st1->place->numname) {      /* Vor st2 ! */
                if ((newst->prevplace=st2->prevplace)!=NULL)
                    newst->prevplace->nextplace=newst;
                else
                    tr2->preset=newst;
                (newst->nextplace=st2)->prevplace=newst;
            } else {                                         /* Hinter st2 ! */
                (st2->nextplace=newst)->prevplace=st2;
                newst->nextplace=NULL;
            }
            st2=newst;
        } else {                                   /* Kantengewicht erhoehen.*/
            if ((st1->place->postset->weight+=st1->weight)>max) {
                LPNRemoveTransition(net,tr2);
                return NULL;
            }
        }
        st1=st1->nextplace;
    }

    /* Ergaenzung um den Postset von Transition 'tr1' */
    ts1=tr1->postset; ts2=tr2->postset;
    while (ts1) {
        if (ts1->place->preset->trans!=tr2) {  /* Weitere Kante hinzufuegen, */
            while (ts2->nextplace &&
                   ts2->place->numname<ts1->place->numname)     /* aber wo ? */
                ts2=ts2->nextplace;
            (newts=LPNallocTSEdge())->prevtrans=NULL;
            newts->trans=tr2;
            newts->place=ts1->place;
            newts->weight=ts1->weight;
            newts->nexttrans=ts1->place->preset;
            ts1->place->preset=newts->nexttrans->prevtrans=newts;
            if (ts2->place->numname>ts1->place->numname) {      /* Vor ts2 ! */
                if ((newts->prevplace=ts2->prevplace)!=NULL)
                    newts->prevplace->nextplace=newts;
                else
                    tr2->postset=newts;
                (newts->nextplace=ts2)->prevplace=newts;
            } else {                                         /* Hinter ts2 ! */
                (ts2->nextplace=newts)->prevplace=ts2;
                newts->nextplace=NULL;
            }
            ts2=newts;
        } else {
            if ((ts1->place->preset->weight+=ts1->weight)>max) {
                LPNRemoveTransition(net,tr2);
                return NULL;
            }
        }
        ts1=ts1->nextplace;
    }

    /* Sofort testen, ob die neu entstandene Transition ein Duplikat ist ! */
    /* Das kann nur im Nachbereich des akt. Vorbereichs liegen             */
    newst=tr2->preset->place->postset->nexttrans;   /* Die erste ist tr2 ! */
    while (newst)
      {
        tr1=newst->trans;
        if (MSis_equal(tr2->label,tr1->label))      /* Gleiches Label ? */
          {
            /* Stimmt Postset ueberein ? */
            ts1=tr1->postset; ts2=tr2->postset;
            while (ts1 && ts2 && ts1->place==ts2->place &&
                   ts1->weight==ts2->weight)
              {
                ts1=ts1->nextplace;
                ts2=ts2->nextplace;
              }
            if (!ts1 && !ts2)
              {
                /* Der Preset etwa auch ? */
                st1=tr1->preset; st2=tr2->preset;
                while (st1 && st2 && st1->place==st2->place
                                  && st1->weight==st2->weight) {
                    st1=st1->nextplace;
                    st2=st2->nextplace;
                }
                if (!st1 && !st2)
                  {
                    if (tr1->time_label && tr2->time_label &&
                        tr1->time_label->low==tr2->time_label->low &&
                        tr1->time_label->high==tr2->time_label->high)
                      {
                        LPNRemoveTransition(net,tr2);   /* Dann loeschen ! */
                        return NULL;
                      } /* time_label gleich */
                  } /* if !st1 && !st2 */
            } /* if !ts1 && !ts2 */
        } /* if MSis_equal */
        newst=newst->nexttrans;
      } /* while newst */
    return tr2;
  }

/****************************************************************************/
/* Befindet sich an der Position 'x,y' bzw. in dessen vertikaler Umgebung   */
/* bereits eine Stelle oder Transition ?                                    */
/****************************************************************************/

static int occupied(LPNet net,int x,int y)
{
    register Transition tr;
    register Place pl;
    register int erg,yp1,ym1;

    yp1=y+1;
    ym1=y-1;
    erg=0;
    tr=net->Transitions;
    while (tr) {
        if (tr->x_pos==x && tr->y_pos>=ym1 && tr->y_pos<=yp1)
          {
            if (tr->y_pos==y)
                erg|=1;
            else if (tr->y_pos==ym1)
                erg|=2;
            else
                erg|=4;
          }
        tr=tr->nexttrans;
    }
    pl=net->Places;
    while (pl) {
        if (pl->x_pos==x && pl->y_pos>=ym1 && pl->y_pos<=yp1)
          {
            if (pl->y_pos==y)
                erg|=1;
            else if (pl->y_pos==ym1)
                erg|=2;
            else
                erg|=4;
          }
        pl=pl->nextplace;
    }
    return erg;
}

/****************************************************************************/
/* Ruft 'sub_Sync_Scope' auf und positioniert die eventuell neu entstandene */
/* Transition.                                                              */
/* tr1 ist unkonjugiert.                                                    */
/****************************************************************************/

static Transition sub_Sync(LPNet net,Transition tr1,Transition tr2,
                     Action action,Cardinal max)
{
    register Transition tr3;
    int x,y,erg;

    if ((tr3=sub_Sync_Scope(net,tr1,tr2,action,max))) {
        x=tr1->x_pos;
        y=tr1->y_pos;
        left_and_right(net,x,x-1,1);
        if ((erg=occupied(net,x+1,y))&1) {
            --x;
        } else if (erg&2) {
            --x;
            --y;
        } else if (erg&4) {
            --x;
            ++y;
        }
        tr3->x_pos=x;
        tr3->y_pos=y;
    }
    return tr3;
}

/*****************************************************************************/
/* Erzeugung einer Basisbox; die Transition erhaelt als Beschriftung eine    */
/* Kopie des uebergebenen Labels 'label', welches eine Multimenge oder       */
/* eine als Multimenge verkleidete Variable sein kann.                       */
/* Rueckgabewert ist das erzeugte Netz.                                      */
/*****************************************************************************/

LPNet CreateBasicClocknet(MultiSet label, Time_stamp *timelabel)
{
  Place      pl;
  Transition tr;
  STEdge     st;
  TSEdge     ts;
  LPNet      box;
  int i;

  box=LPNallocHead();
  box->NumComponents=1;
  if (timelabel==NULL)
    {
      timelabel=malloc(sizeof(Time_stamp));
      timelabel->low=0;
      timelabel->high=0;
    }
  box->NumPlaces=timelabel->high+2;
  box->NumTransitions=2*timelabel->high-timelabel->low+1;
  box->LastTransition=NULL;
  box->x_pos=box->y_pos=0;
  box->x_dim=2*(timelabel->high-timelabel->low+1)-1;
  box->y_dim=4*timelabel->low+4*(timelabel->high-timelabel->low+1)+1;
  pl=box->Places=box->LastPlace=LPNallocPlace();
  pl->prevplace=NULL;
  pl->name=NULL;
  pl->numname=1;
  pl->component=1;
  pl->label=ENTRY;
  pl->preset=NULL;
  pl->x_pos=pl->y_pos=0;
  pl->marks=0;
  pl->tokenage=0;

  /* Aktionen aktiviert */
  for (i=0;i<timelabel->low;i++)
    {
      tr=LPNallocTrans();
      if (i==0)
        {
          box->Transitions=tr;
        }
      else
        {
          box->LastTransition->nexttrans=tr;
        }
      tr->prevtrans=box->LastTransition;
      box->LastTransition=tr;
      tr->nexttrans=NULL;
      tr->numname=i+1;
      tr->label=NULL;
      tr->type=TIC;
      tr->bpn_label=NULL;
      tr->x_pos=0;
      tr->y_pos=4*i+2;
      tr->flags=0;
      st=pl->postset=tr->preset=LPNallocSTEdge();
      st->prevplace=st->prevtrans=NULL;
      st->nextplace=st->nexttrans=NULL;
      st->place=pl;
      st->trans=tr;
      st->weight=1;

      pl=LPNallocPlace();
      box->LastPlace->nextplace=pl;
      pl->prevplace=box->LastPlace;
      box->LastPlace=pl;
      pl->nextplace=NULL;
      pl->name=NULL;
      pl->numname=i+2;
      pl->component=1;
      pl->postset=NULL;
      pl->label=CHOICEPL;
      pl->x_pos=0;
      pl->y_pos=4*i+4;
      pl->marks=0;
      pl->tokenage=i+1;
      ts=pl->preset=tr->postset=LPNallocTSEdge();
      ts->prevplace=ts->prevtrans=NULL;
      ts->nextplace=ts->nexttrans=NULL;
      ts->place=pl;
      ts->trans=tr;
      ts->weight=1;
    } /* for */

  /* Erste Schalttransition */
  tr=LPNallocTrans();
  if (timelabel->low==0)
    {
      box->Transitions=tr;
    }
  else
    {
      box->LastTransition->nexttrans=tr;
    }
  tr->prevtrans=box->LastTransition;
  box->LastTransition=tr;
  tr->nexttrans=NULL;
  tr->numname=timelabel->low+1;
  tr->label=MScreate_copy(label);
  tr->type=NORMAL;
  tr->bpn_label=NULL;
  tr->x_pos=0;
  tr->y_pos=4*timelabel->high+2;
  tr->flags=0;
  st=pl->postset=tr->preset=LPNallocSTEdge();
  st->prevplace=st->prevtrans=NULL;
  st->nextplace=st->nexttrans=NULL;
  st->place=pl;
  st->trans=tr;
  st->weight=1;

  /* brige Aktionen schaltbereit */
  for (i=0;i<timelabel->high-timelabel->low;i++)
    {
      tr=LPNallocTrans();
      box->LastTransition->nexttrans=tr;
      tr->prevtrans=box->LastTransition;
      box->LastTransition=tr;
      tr->nexttrans=NULL;
      tr->numname=timelabel->low+2+2*i;
      tr->label=NULL;
      tr->type=TIC;
      tr->bpn_label=NULL;
      tr->x_pos=2*i+2;
      tr->y_pos=4*timelabel->low+2+4*i;
      tr->flags=0;
      st=tr->preset=LPNallocSTEdge(); /* Zweite Kante der Stelle! */
      pl->postset->nexttrans=st; /* Hinten anfuegen */
      st->prevtrans=pl->postset;
      st->nexttrans=NULL;
      st->prevplace=st->nextplace=NULL;
      st->place=pl;
      st->trans=tr;
      st->weight=1;

      pl=LPNallocPlace();
      box->LastPlace->nextplace=pl;
      pl->prevplace=box->LastPlace;
      box->LastPlace=pl;
      pl->nextplace=NULL;
      pl->name=NULL;
      pl->numname=timelabel->low+2+i;
      pl->component=1;
      pl->postset=NULL;
      pl->label=CHOICEPL;
      pl->x_pos=2*i+2;
      pl->y_pos=4*timelabel->low+4+4*i;
      pl->marks=0;
      pl->tokenage=timelabel->low+1+i;
      ts=pl->preset=tr->postset=LPNallocTSEdge();
      ts->prevplace=ts->prevtrans=NULL;
      ts->nextplace=ts->nexttrans=NULL;
      ts->place=pl;
      ts->trans=tr;
      ts->weight=1;

      tr=LPNallocTrans();
      box->LastTransition->nexttrans=tr;
      tr->prevtrans=box->LastTransition;
      box->LastTransition=tr;
      tr->nexttrans=NULL;
      tr->numname=timelabel->low+3+2*i;
      tr->label=MScreate_copy(label);
      tr->type=NORMAL;
      tr->bpn_label=NULL;
      tr->x_pos=2*i+2;
      tr->y_pos=4*timelabel->high+2;
      tr->flags=0;
      st=pl->postset=tr->preset=LPNallocSTEdge();
      st->prevplace=st->prevtrans=NULL;
      st->nextplace=st->nexttrans=NULL;
      st->place=pl;
      st->trans=tr;
      st->weight=1;
    } /* for */
  pl=LPNallocPlace();
  box->LastPlace->nextplace=pl;
  pl->prevplace=box->LastPlace;
  box->LastPlace=pl;
  pl->nextplace=NULL;
  pl->name=NULL;
  pl->numname=timelabel->high+2;
  pl->component=1;
  pl->postset=NULL;
  pl->preset=NULL;
  pl->label=EXIT;
  pl->x_pos=0;
  pl->y_pos=4*timelabel->high+4;
  pl->marks=0;
  pl->tokenage=0;
  ts=pl->preset;

  for(tr=box->Transitions;tr!=NULL;tr=tr->nexttrans)
    {
      if (tr->type==NORMAL)
        {
          tr->postset=LPNallocTSEdge();
          if (pl->preset==NULL)
          {
            ts=pl->preset=tr->postset;
            ts->prevtrans=NULL;
          }
          else
          {
            ts->nexttrans=tr->postset;
            tr->postset->prevtrans=ts;
            ts=tr->postset;
          }
          ts->nexttrans=NULL;
          ts->nextplace=ts->prevplace=NULL;
          ts->place=pl;
          ts->trans=tr;
          ts->weight=1;
        } /* if */
    } /* for */

  free(timelabel);
  return box;
}

/*****************************************************************************/
/* Erzeugung einer Basisbox; die Transition erhaelt als Beschriftung eine    */
/* Kopie des uebergebenen Labels 'label', welches eine Multimenge oder       */
/* eine als Multimenge verkleidete Variable sein kann.                       */
/* Rueckgabewert ist das erzeugte Netz.                                      */
/*****************************************************************************/

LPNet CreateBasicBox(MultiSet label, Time_stamp *time_label)
{
    register Place      pl;
    register Transition tr;
    register STEdge     st;
    register TSEdge     ts;
    register LPNet      box;

    /* Neue box hat zwei Stellen und eine Transition */
    (box=LPNallocHead())->NumComponents=1;
    box->NumPlaces=2;
    box->NumTransitions=1;
    box->x_pos=box->y_pos=0;
    box->x_dim=1;
    box->y_dim=5;
    /* Erste Stelle ist Eingangsstelle */
    (pl=box->Places=LPNallocPlace())->prevplace=NULL;
    pl->name=NULL;
    pl->numname=1;
    pl->component=1;
    pl->label=ENTRY;
    pl->preset=NULL;
    pl->x_pos=pl->y_pos=0;
    /* Die Transition liegt dazwischen */
    tr=box->Transitions=box->LastTransition=LPNallocTrans();
    tr->prevtrans=tr->nexttrans=NULL;
    tr->numname=1;
    tr->label=MScreate_copy(label);
    tr->bpn_label=NULL;
    tr->time_label=time_label;
    tr->x_pos=0;
    tr->y_pos=2;
    /* Dies ist die Kante zwischen Stelle und Transition */
    st=pl->postset=tr->preset=LPNallocSTEdge();
    st->prevplace=st->nextplace=st->prevtrans=st->nexttrans=NULL;
    st->place=pl;
    st->trans=tr;
    st->weight=1;
    /* Und die zweite (Ausgangs-)Stelle */
    ( pl=( (pl->nextplace=box->LastPlace=LPNallocPlace())
           ->prevplace=pl ) ->nextplace ) ->nextplace=NULL;
    pl->name=NULL;
    pl->numname=1;
    pl->component=1;
    pl->postset=NULL;
    pl->label=EXIT;
    pl->x_pos=0;
    pl->y_pos=4;
    /* Mit der passenden Kante */
    ts=pl->preset=tr->postset=LPNallocTSEdge();
    ts->prevplace=ts->nextplace=ts->prevtrans=ts->nexttrans=NULL;
    ts->place=pl;
    ts->trans=tr;
    ts->weight=1;
    return box;
}

/*****************************************************************************/
/* Anwendung der Synchronisations-Operation bezueglich der Aktion 'action'   */
/* auf das Netz 'net', welches auch den Rueckgabewert darstellt.             */
/* Diese erste Funktion sucht systematisch nach Transitions-Paaren, die sich */
/* synchronisieren koennen, und reicht diese (zusammen mit einigen anderen   */
/* Parametern) an die Funktion 'sub_Sync_Scope()' weiter.                */
/* Der Parameter 'max' spielt hier keine Rolle, er wird unveraendert an die  */
/* Funktion 'sub_Sync_Scope()' weitergegeben und auch dort erlaeutert.   */
/*****************************************************************************/

LPNet Synchronisation(LPNet net,MultiSet mm,Cardinal max)
{
    Transition tr1,tr2;
    Place      pl;
    Cardinal32 i;

    /* Durchnummerierung der Stellen notwendig fuer  */
    /* das Funktionieren von 'sub_Sync_Scope()'. */
    for (i=0, pl=net->Places; pl; pl=pl->nextplace)
    {
        pl->numname=++i;
    }
    while (*mm) {
        tr1=net->LastTransition;
        while (tr1) {
             if (MScount(*mm,tr1->label)) {
                 tr2=tr1;
                while (tr2) {
                    if (MScount_conj(*mm,tr2->label))
                        sub_Sync(net,tr1,tr2,*mm,max);
                    tr2=tr2->prevtrans;
                }
            } else if (MScount_conj(*mm,tr1->label)) {
                tr2=tr1;
                while (tr2) {
                    if (MScount(*mm,tr2->label))
                        sub_Sync(net,tr2,tr1,*mm,max);
                    tr2=tr2->prevtrans;
                }
            }
            tr1=tr1->prevtrans;
        }
        mm+=2;
    }
    return net;
}

/*****************************************************************************/
/* Anwendung der Restriktions-Operation bezueglich der Aktionen in 'mm'      */
/* auf das Netz 'net', welches auch den Rueckgabewert darstellt.             */
/*****************************************************************************/

LPNet Restriction(LPNet net, MultiSet mm)
  {
    Transition tr;

    tr=net->LastTransition;
    while (tr)
      {
        if (MSis_disjunct(mm,tr->label))
          tr=tr->prevtrans;
        else
          tr=LPNRemoveTransition(net,tr);
      }
    return net;
  } /* Restriction() */

/*****************************************************************************/
/* Spezielle Version der Restriktions-Operation fuer Uhrennetze              */
/*****************************************************************************/

LPNet CLOCKRestriction(LPNet net, MultiSet mm)
  {
    Transition tr;

    tr=net->LastTransition;
    while (tr)
      {
        if (MSis_disjunct(mm,tr->label))
          tr=tr->prevtrans;
        else
          tr=LPNRemoveTransition(net,tr);
      }
    return net;
  } /* CLOCKRestriction() */

/*****************************************************************************/
/* Spezialroutine aehnlich 'LPNRemoveTransition()' fuer die nachfolgende     */
/* Funktion 'Scoping()'. Nur beim label ein Unterschied.                     */
/* Entfernen der Transition 'tr' aus dem Netz 'net' zusammen mit allen       */
/* Kanten, die in/aus diese/r Transition fuehren.                            */
/* Rueckgabewert ist 'tr->prevtrans'.                                        */
/*****************************************************************************/

static Transition RemoveTransition(LPNet net,Transition tr)
{
    register STEdge st,stdel;
    register TSEdge ts,tsdel;

    if (net && tr) {
        ts=tr->postset;
        while (ts) {
            if (ts->prevtrans)
                ts->prevtrans->nexttrans=ts->nexttrans;
            else
                ts->place->preset=ts->nexttrans;
            if (ts->nexttrans)
                ts->nexttrans->prevtrans=ts->prevtrans;
            ts=(tsdel=ts)->nextplace;
            LPNremoveTSEdge(tsdel);
        }
        st=tr->preset;
        while (st) {
            if (st->prevtrans)
                st->prevtrans->nexttrans=st->nexttrans;
            else
                st->place->postset=st->nexttrans;
            if (st->nexttrans)
                st->nexttrans->prevtrans=st->prevtrans;
            st=(stdel=st)->nextplace;
            LPNremoveSTEdge(stdel);
        }
        if (tr->prevtrans)
            tr->prevtrans->nexttrans=tr->nexttrans;
        else
            net->Transitions=tr->nexttrans;
        if (tr->nexttrans)
            tr->nexttrans->prevtrans=tr->prevtrans;
        else
            net->LastTransition=tr->prevtrans;
        if (tr->label!=tr->bpn_label) /* sonst zeigt eine andere Transition  */
            MSremove(tr->label);      /* auf *(tr->label).                   */
        if (tr->time_label)
          free(tr->time_label);
        net->NumTransitions--;
        net=(LPNet)tr->prevtrans; /* kleiner Kunstgriff */
        LPNremoveTrans(tr);
    }
    return (Transition)net;       /* dito               */
}

/**************************************************************/
/* tr->numname & 1 : ' akt' befindet sich in der Beschriftung */
/* tr->numname & 2 : '^akt' befindet sich in der Beschriftung */
/**************************************************************/

static void set_numname(Transition tr,Action akt)
{
    MultiSet mm;

    mm=tr->label;
    akt&=~(Action)1;
    while (*mm && *mm<akt) {
        mm+=2;
    }
    if (*mm==akt) {
        tr->numname=1;
        if (*(mm+2)==(akt|1))
            tr->numname=3;
    } else if (*mm==(akt|1))
        tr->numname=2;
    else
        tr->numname=4;
}

/***********************************/
/* Anwendung der Scoping-Operation */
/* (analog zu 'Synchronisation()') */
/***********************************/

LPNet Scoping(LPNet net,MultiSet mm,Cardinal max)
{
    register Transition tr1,tr2,tr3;
    Place      pl;
    Cardinal32 i;

    /* Durchnummerierung der Stellen notwendig fuer  */
    /* das Funktionieren von 'sub_Sync_Scope()'. */
    for (i=0, pl=net->Places; pl; pl=pl->nextplace)
    {
        pl->numname=++i;
    }
    while (*mm) {
        tr1=net->LastTransition;
        for (tr2=tr1; tr2; tr2=tr2->prevtrans)
	{
            set_numname(tr2,*mm);
        }
        i=5;
        while (tr1) {
            if (tr1->numname&2) { /* konjugiertes Symbol */
                if (i&1) {
                    i&=~2; /* Neuer Durchgang */
                    tr2=tr1;
                    while (tr2) {
                        if (tr2->numname&1) { /* Paar gefunden */
                            i|=2; /* Paar gefunden */
                            if ((tr3=sub_Sync_Scope(net,tr1,tr2,*mm,max))!=NULL) {
                                tr3->bpn_label=(tr2->bpn_label)?
                                                tr2->bpn_label
                                               :(tr2->bpn_label=tr2->label);
                                set_numname(tr3,*mm);
                            }
                        }
                        tr2=tr2->prevtrans;
                    }
                    if (!(i&2)) /* Kein Paar mehr, weitere gleich loeschen */
                        i&=~1;
                } /* if i&1 */
                tr1=RemoveTransition(net,tr1);
            } else if (tr1->numname&1) { /* unkonjugiert */
                if (i&4) {
                    i&=~8;
                    tr2=tr1;
                    while (tr2) {
                        if (tr2->numname&2) { /* Paar gefunden */
                            i|=8;
                            if ((tr3=sub_Sync_Scope(net,tr2,tr1,*mm,max))!=NULL) {
                                tr3->bpn_label=(tr2->bpn_label)?
                                                tr2->bpn_label
                                               :(tr2->bpn_label=tr2->label);
                                set_numname(tr3,*mm);
                            }
                        }
                        tr2=tr2->prevtrans;
                    }
                    if (!(i&8))
                        i&=~4;
                }
                tr1=RemoveTransition(net,tr1);
            } else
                tr1=tr1->prevtrans;
        }
        mm+=2;
    }
    return net;
}

/*****************************************************************************/
/* Erzeugung eines Netzes mit einer Entry- und einer Exit-Stelle, sowie      */
/* n internen Stellen.                                                       */
/* Rueckgabewert ist das erzeugte Netz.                                      */
/*****************************************************************************/

LPNet OnlyPlaces(Cardinal n)
{
    register LPNet    box;
    register Place    pl;
    register Cardinal i;
             STEdge   stdummy;
             TSEdge   tsdummy;

    i=n+2;
    (box=LPNallocHead())->NumComponents=1;
    box->x_pos=box->y_pos=0;
    box->x_dim=box->y_dim=1;
    box->NumTransitions=0;
    box->NumPlaces=1;
    box->Transitions=box->LastTransition=NULL;
    (pl=box->Places=box->LastPlace=LPNallocPlace())->preset=NULL;
    pl->name=NULL;
    pl->postset=NULL;
    pl->nextplace=pl->prevplace=NULL;
    pl->numname=--i;
    pl->component=1;
    pl->label=EMPTY;
    pl->tokenage=0;
    pl->x_pos=0;
    pl->y_pos=i*2;
    while (i) {
        (pl=DuplicatePlace(box,pl,&stdummy,&tsdummy))->numname=--i;
        pl->y_pos-=2;
        box->y_dim+=2;
    }
    pl->label=ENTRY;
    box->LastPlace->label=EXIT;
    return box;
}

/********************************************************************/
/* Erzeuge Transition in Netz net mit Beschriftung mm an der Stelle */
/* x_pos,y_pos. Die Transition hat leeren Vor- und Nachbereich      */
/********************************************************************/

Transition CreateTr(LPNet net,MultiSet mm,int x_pos,int y_pos)
{
    Transition tr;

    (tr=LPNallocTrans())->prevtrans=NULL;
    tr->x_pos=x_pos;
    tr->y_pos=y_pos;
    if (net->Transitions) {
        (tr->nexttrans=net->Transitions)->prevtrans=tr;
    } else {
        (net->LastTransition=tr)->nexttrans=NULL;
    }
    (net->Transitions=tr)->preset=(STEdge)(tr->postset=NULL);
    tr->label=mm;
    tr->type=NORMAL;
    tr->bpn_label=NULL;
    tr->time_label=NULL;
    return tr;
}

/**********************************************************************/
/* Neue gemeinsam wartende Tic-Transitionen fuer Choice erstellen.    */
/* Getrennt schaltende auslassen, die neuen vorn in Netz1 eingefuegen */
/* Bei der ersten Schalttransition ist Schluss!                       */
/**********************************************************************/

LPNet CommonWait(LPNet net1, LPNet net2)
  {
    Transition tr1, tr2, newtr;
    STEdge     st1, st2;
    TSEdge     ts1, ts2;
    LPNet      net;

    net=LPNallocHead();
    net->NumPlaces=0;
    net->NumTransitions=0;
    net->Places=net->LastPlace=NULL;
    net->Transitions=net->LastTransition=NULL;

    for (tr1=net1->Transitions;tr1!=NULL;tr1=tr1->nexttrans)
      {
        if ((tr1->type & TIC))
          {
            for (tr2=net2->Transitions;tr2!=NULL;tr2=tr2->nexttrans)
              {
                if ((tr2->type & TIC))
                  {
                    newtr=DuplicateTrans(net,tr1,&st1,&ts1);
                    (st1->nextplace=CopyTransPreset(tr2,newtr,&st2))
                        ->prevplace=st1;
                    (ts1->nextplace=CopyTransPostset(tr2,newtr,&ts2))
                        ->prevplace=ts1;
                    newtr->x_pos=(tr1->x_pos+tr2->x_pos)/2;
                    newtr->y_pos=(tr1->y_pos+tr2->y_pos)/2;
                    newtr->type=TIC;
                  } /* if tr2 */
              } /* for tr2 */
          } /* if tr1 */
      } /* for tr1 */
    return net;
  }

/*************************/
/* END OF FILE box_sub.c */
/*************************/
