/******************************/
/* boxtypes.h                 */
/* Markus Wittwer und         */
/* Christian Stehno, 1.6.1999 */
/* basiert auf main.h von     */
/* Burkhard Graves, 17.12.'93 */
/******************************/

#ifndef __BOXTYPES_H__
#define __BOXTYPES_H__

#include "memlib.h"
#include "readlib.h"
#include "set.h"
/*****************************************************************************/
/*                 Allgemeine Typdefinitionen und Konstanten                 */
/*****************************************************************************/

typedef Cardinal Action, Var;
typedef Action  *MultiSet;
typedef MultiSet Function;


#define WARNING(__WARN_MSG__) fprintf(stderr,"WARNING: %s\n",__WARN_MSG__);

/*****************************************************************************/
/* Wenn SILENCE definiert, dann keine (kaum) Bildschirm-Ausgaben             */
/* Nur in box.c gebraucht                                                    */

#define SILENCE

/*****************************************************************************/
/* Datentyp fuer Funktionen                                                  */
/*****************************************************************************/

typedef Cardinal FuncCode;

/*****************************************************************************/
/* Datentyp zur Repraesentation von Zeitannotationen:                        */
/*****************************************************************************/

typedef double Time;

typedef struct {
  Time low;
  Time high;
} Time_stamp;


/************************************************************************/
/* Globale Variable, die Defaultwerte fuer die Rekursions- und          */
/* die Synchronisationstiefe enthalten.                                 */
/************************************************************************/

extern Cardinal RecDepth, SyncDepth;


/*****************************************************************************/
/* Datentyp ASList fuer die Constraint-bezogene Verwaltung von Eigenschaften */
/* der Aktionssymbole in den Syntaybaumknoten.                               */
/*****************************************************************************/

typedef struct _ASListEntryT
{
  Action     act; /* Aktionssymbol */
  bool       syn, res; /* wird ueber dem Symbol synchronisiert
                          bzw. restringiert */
  struct _SyntaxNodeList * leaflist; /* Zeiger auf den
                                        Syntaxbaumknoten, in dem 
                                        dieses Aktionssymbol
                                        benutzt wird */

  struct _ASListEntryT   * next; 
} ASListEntryT,
  * ASListT;


/*****************************************************************************/
/* Datentyp Tertiaerbaum fuer Syntaxbaum ( box expressions )                 */
/*****************************************************************************/

typedef struct TTNode
  {
    Cardinal      Operator,
                  Depth;
    struct TTNode * Operand1,
                  * Operand2,
                  * Operand3;

    Cardinal     Index; /* Index des Vor- und Nachbereiches */
    Cardinal     IterationDepth; /* enthaelt die Anzahl der
                                    Iterationsnetze in denen sich
                                    dieser Knoten befindet */
    ASListT      ASList;

    int fl, fc, ll, lc; /* 
                         * Zeilen- und Spaltenposition des Knotens im 
                         * Eingabe-PBC-File
                         */
    
  } TTreeNode,
    * TTree;

/*****************************************************************************/
/* Datentyp SyntaxNodeListT, um Knoten des Syntaxbaumes als lineare
   Liste unabhaengig von der Baumstruktur miteinander zu verknuepfen         */
/*****************************************************************************/

typedef struct _SyntaxNodeList 
{
  TTree Knoten;
  struct _SyntaxNodeList *next;
} SyntaxNodeListEntry; 
  
/*****************************************************************************/

#endif

/**************************/
/* END OF FILE boxtypes.h */
/**************************/
