/*****************************************************************************/
/*  boxxpars.y                                                               */
/*  YACC-Quelldatei: Parser fuer Box-Ausdruecke.                             */
/*  Di.,27.10.1992 - Di.,10.08.1993                           Stefan Roemer  */
/*****************************************************************************/

%{
#include <string.h>

#include "memlib.h"
#include "multiset.h"
#include "parser.h"
#include "boxtypes.h"

extern FILE *yyin;
extern int line,column,bpn_modus;
extern TTree Syntaxbaum;

int yylex(void);
int yyerror(char *msg);

%}

/* Moegliche Datentypen der Nichtterminalsymbole */
%union {
    TTree    KnotenPtr;
    MultiSet MulSet;
    Action   Aktion;
    Var      Variab;
    Function MapPtr;
    Cardinal Number;
    Time     RealNumber;
    FuncCode FunctCode;
    char     *String;
    Range    *Bereich;
}


/* Token ( Terminalsymbole ) der Grammatik, ausser 'Interpunktion' */
/* Diese Tabelle muss UNVERAENDERT bleiben; sie darf evtl.         */
/* VERLAENGERT werden.                                             */
/* Die Position der TOKEN bestimmt die auszufuehrenden Aktionen    */
/* Beim Abarbeiten des Syntaxbaums! Behandelt werden die Token     */
/* ACTION bis S_BOX                                                */
%token  <String> NUMBER
%token  <String> FUNCTION 
%token  MAP
%token  CONJUGATE
%token  <String> ACTION VARIABLE
%token  SEQUENCE CHOICE
%token  PARALLEL SYNCHRON
%token  RESTRICT SCOPE TIE
%token  RELABEL REFINE RECURSION
%token  ITERATION
%token  <String> FILENAME
%token  C_BOX D_BOX STOP_BOX
%token  ATOM_CHOICE
%token  S_BOX
%token  EMPTYSET
%token  ATTIME
%token  <String> REALNUMBER

/* Rueckgabetypen der nichtterminalen Grammatiksymbole */
%type  <KnotenPtr>  box_expr_decl
%type  <String>     funct_name
%type  <FunctCode>  funct_def func_decl
%type  <FunctCode>  declarations function
%type  <KnotenPtr>  box_expression
%type  <Number>     number depth init
%type  <Aktion>     single_action
%type  <MulSet>     action_list
%type  <MulSet>     num_action
%type  <MulSet>     multi_label_action
%type  <Variab>     variable
%type  <MapPtr>     mapping map_list
%type  <Bereich>    menge elem_list elem
%type  <RealNumber> realnumber

/* Startsymbol der Grammatik */
%start  box_expr_decl


/* Prioritaeten und Assoziativitaeten der Operatoren */
/* Mit den Prioritaeten koennen die unten mit 'S/R' gekennzeichneten */
/* Shift/Reduce-Konflikte der vorliegenden Grammatik geloest werden. */

%left     PARALLEL
%left     CHOICE
%left     SEQUENCE
%nonassoc SYNCHRON RESTRICT
%nonassoc '['
%nonassoc '.'
%left     '#'

%%

box_expr_decl      : '#' declarations '#' box_expression
                       {
			 Syntaxbaum = $4;
                         $$ = $4;
                       }
                   | box_expression
                       {
			 Syntaxbaum = $1;
                         $$ = $1;
                       }
		;

funct_name         : FUNCTION
                       {
                         $$ = MYstrdup( yylval.String );
                       }
		;

funct_def          : funct_name '=' '(' map_list ')'
                       {
                         $$ = StrToFunction ( $1, $4 );
                         free( $1 );
                       }
		;

func_decl          : funct_def
                       {
                         $$ = $1;
                       }
                   | func_decl funct_def
                       {
                         $$ = $2;
                       }
		;

declarations       : RECURSION number func_decl
                       {
                         if ( $2 )
                           {
                             RecDepth = $2;
                           } /* if */
                         $$ = $3;
                       }
                   | func_decl
                       {
                         $$ = $1;
                       }
		;

number             : NUMBER
                       {
                         if ( sscanf( yylval.String, FORMAT_CARD, &($$) ) != 1 )
                           {
                             yyerror( "Syntax Error in num. constant" );
                           } /* if */
                       }
		;

realnumber         : REALNUMBER
                       {
                         if ( sscanf( yylval.String, "%lf", &($$) ) != 1 )
                           {
                             yyerror( "Syntax Error in num. constant" );
                           } /* if */
                       }
		;

depth              : '<' number '>'
                       {
                         $$ = $2;
                       }
                   |
                       {
                         $$ = 3110;
                       }
		;

init               : number ','
                        {
                             $$ = $1;
                        }
                   |
                       {
                         $$ = 32767;
                       }
		;


box_expression     : multi_label_action
                       {
                         $$ = GenNewNode( ACTION, (TTree) $1, NULL, NULL,
					  @1.first_line, @1.first_column, 
					  @1.last_line, @1.last_column);
                       }
                   | multi_label_action ATTIME '[' number ',' number ']'
                       {
			 $$ = GenNewNode( ACTION, (TTree) $1, 
					  (TTree) CreateTimeStamp($4,$6),
					  NULL,
					  @1.first_line, @1.first_column,
					  @7.last_line, @7.last_column);
                       }
                   | multi_label_action ATTIME '[' realnumber ',' number ']'
                       {
			 $$ = GenNewNode( ACTION, (TTree) $1, 
					  (TTree) CreateTimeStamp($4,$6),
					  NULL,
					  @1.first_line, @1.first_column,
					  @7.last_line, @7.last_column);
                       }
                   | multi_label_action ATTIME '[' number ',' realnumber ']'
                       {
			 $$ = GenNewNode( ACTION, (TTree) $1, 
					  (TTree) CreateTimeStamp($4,$6),
					  NULL,
					  @1.first_line, @1.first_column,
					  @7.last_line, @7.last_column);
                       }
                   | multi_label_action ATTIME '[' realnumber ',' realnumber ']'
                       {
			 $$ = GenNewNode( ACTION, (TTree) $1, 
					  (TTree) CreateTimeStamp($4,$6),
					  NULL,
					  @1.first_line, @1.first_column,
					  @7.last_line, @7.last_column);
                       }
                   | variable
                       {
                         MultiSet mmvar;
                         mmvar = MScreate_singleton( 1 );
                         $$ = GenNewNode( VARIABLE,
                                          (TTree) MSset_single( mmvar, 1, $1 ),
                                          NULL, NULL,
					  @1.first_line, @1.first_column, 
					  @1.last_line, @1.last_column);

                       }
                   | '(' box_expression ')'
                       {
                         $$ = $2;
                       }
                   | box_expression SEQUENCE box_expression  /*S/R*/
                       {
                         $$ = GenNewNode( SEQUENCE, $1, $3, NULL,
					  @1.first_line, @1.first_column, 
					  @3.last_line, @3.last_column );
                       }
                   | box_expression CHOICE box_expression    /*S/R*/
                       {
                         $$ = GenNewNode( CHOICE, $1, $3, NULL,
					  @1.first_line, @1.first_column, 
					  @3.last_line, @3.last_column );
                       }
                   | box_expression '#' box_expression
                       {
                         $$ = GenNewNode( ATOM_CHOICE, $1, $3, NULL,
					  @1.first_line, @1.first_column, 
					  @3.last_line, @3.last_column );
                       }
                   | box_expression PARALLEL box_expression  /*S/R*/
                       {
                         $$ = GenNewNode( PARALLEL, $1, $3, NULL,
					  @1.first_line, @1.first_column, 
					  @3.last_line, @3.last_column );
                       }
                   | box_expression SYNCHRON depth multi_label_action
                       {
                         ( $$ = GenNewNode( SYNCHRON, $1, (TTree) $4, NULL,
					  @1.first_line, @1.first_column, 
					  @4.last_line, @4.last_column ) )
                                -> Depth = ($3==3110)?SyncDepth:$3;
                       }
                   | box_expression RESTRICT multi_label_action
                       {
                         $$ = GenNewNode( RESTRICT, $1, (TTree) $3, NULL,
					  @1.first_line, @1.first_column, 
					  @3.last_line, @3.last_column );
                       }
                   | '[' depth multi_label_action SCOPE box_expression ']'
                       {
                         ( $$ = GenNewNode( SCOPE, $5, (TTree) $3, NULL,
					  @1.first_line, @1.first_column, 
					  @6.last_line, @6.last_column ) )
                                -> Depth = ($2==3110)?SyncDepth:$2;
                       }
                   | box_expression TIE multi_label_action
                       {
			 $$ = GenNewNode( TIE, $1, (TTree) $3, NULL,
					  @1.first_line, @1.first_column, 
					  @3.last_line, @3.last_column );
		       }
                   | box_expression '[' function ']'  /* S/R */
                       {
                         $$ = GenNewNode( RELABEL,
                                           $1, (TTree) FUNCTION
                                             , (TTree) (Cardinal32) $3,
					  @1.first_line, @1.first_column, 
					  @4.last_line, @4.last_column );
                       }
                   | box_expression '[' map_list ']'  /* S/R */
                       {
                         $$ = GenNewNode( RELABEL, $1, (TTree) MAP ,
                                                 (TTree) MScreate_copy( $3 ),
					  @1.first_line, @1.first_column, 
					  @4.last_line, @4.last_column ); 
                   }
                   | box_expression '[' variable REFINE box_expression ']'
                       {                                              /* S/R */
                         $$ = GenNewNode( REFINE, $1, (TTree) (Cardinal32) $3, $5,
					  @1.first_line, @1.first_column, 
					  @6.last_line, @6.last_column );
                       }
                   | RECURSION variable depth '.' box_expression        /*S/R*/
                       {
                         ( $$ = GenNewNode( RECURSION, (TTree) (Cardinal32) $2
                                                     , $5, NULL,
					  @1.first_line, @1.first_column, 
					  @5.last_line, @5.last_column ) )
                                -> Depth = ($3==3110)?RecDepth:$3;
                       }
                   | '[' depth box_expression ITERATION box_expression
                                              ITERATION box_expression ']'
                       {
                         ( $$ = GenNewNode( ITERATION, $3, $5, $7,
					  @1.first_line, @1.first_column, 
					  @8.last_line, @8.last_column ) )
                                                                 -> Depth = 0;
                       }
                   | '(' box_expression ')' MAP FILENAME
                       {
                         $$ = GenNewNode( FILENAME, (TTree) MYstrdup( yylval.String ),
                                          $2, NULL,
					  @1.first_line, @1.first_column, 
					  @5.last_line, @5.last_column );
                       }
                   | FILENAME
                       {
                         $$ = GenNewNode( FILENAME, (TTree) MYstrdup( yylval.String ),
                                          NULL, NULL,
					  @1.first_line, @1.first_column, 
					  @1.last_line, @1.last_column );
                       }
                   | C_BOX '(' single_action ',' number ',' menge ')'
                       {
                         ( $$ = GenNewNode( C_BOX,
                                              (TTree) (Cardinal32) $3,
                                              (TTree) $7,
                                              NULL       ,
					  @1.first_line, @1.first_column, 
					  @8.last_line, @8.last_column ) 
                          )-> Depth = $5;
                       }
                   | S_BOX '(' single_action ',' number ',' menge ')'
                       {
                         ( $$ = GenNewNode( S_BOX,
                                              (TTree) (Cardinal32) $3,
                                              (TTree) $7,
                                              NULL,
					  @1.first_line, @1.first_column, 
					  @8.last_line, @8.last_column ) 
                           ) -> Depth = $5;
                       }
                   | D_BOX '(' single_action ',' init menge ')'
                       {
                         ( $$ = GenNewNode( D_BOX,
                                              (TTree) (Cardinal32) $3,
                                              (TTree) $6,
                                              NULL,       
					  @1.first_line, @1.first_column, 
					  @7.last_line, @7.last_column ) 
                         ) -> Depth = $5;
                       }
                   | STOP_BOX
                       {
                         $$ = GenNewNode( STOP_BOX, NULL, NULL, NULL,
					  @1.first_line, @1.first_column, 
					  @1.last_line, @1.last_column );
                       }
		;

/* Errorhandling to be done */
/*                   | error
                       {
                         printf("Error!\n");YYABORT;
                       }
                       */
menge              : '{' elem_list '}'     { $$=$2; }
		;

elem_list          : elem                  { $$=GenRangeList($1,NULL); }
                   | elem_list ',' elem    { $$=GenRangeList($3,$1);   }
		;

elem               : number                { $$=GenRange($1,$1); }
                   | number '.' '.' number { $$=GenRange($1,$4); }
		;

single_action      : ACTION
                       {

			 if (strrchr(yylval.String,'+'))
			   { /* Mark as link import */
			     yylval.String[strlen(yylval.String)-1]='\0';
			     $$ = (StrToAction( yylval.String )|2);
			   }
			 else if (strrchr(yylval.String,'-'))
			   { /* Mark as link export */
			     yylval.String[strlen(yylval.String)-1]='\0';
			     $$ = (StrToAction( yylval.String )|3);
			   }
			 else
			   {
			   $$ = StrToAction( yylval.String );
			   }
		       }
                   | CONJUGATE ACTION
                       {
                         $$ = MSconj_action( StrToAction( yylval.String ) );
                       }
		;

num_action         : single_action
                       {
                         $$ = MSset_single( mmSingle, $1, 1 );
                       }
                   | number single_action
                       {
                         $$ = MSset_single( mmSingle, $2, $1 );
                       }
		;

action_list        : num_action  /* linksrekursiv !!! */
                       {
                         $$ = MScopy( mmLeaf, $1 );
                       }
                   | action_list ',' num_action
                       {
                         $$ = MSsum( mmLeaf, $3 );
                       }
		;

multi_label_action : single_action
                       {
                         $$ = MScreate_singleton( $1 );
                       }
                   | '{' '}'
                       {
                         $$ = MScreate_empty_set();
                       }
                   | EMPTYSET
                       {
                         $$ = MScreate_empty_set();
                       }
                   | '{' action_list '}'
                       {
                         $$ = MScreate_copy( $2 );
                       }
		;


variable           : VARIABLE
                       {
                         $$ = StrToVariable( yylval.String );
                       }
		;


mapping            : single_action MAP single_action
                       {
                         if ( $1 & 1 )
                           {
                             $1 = MSconj_action( $1 );
                             $3 = MSconj_action( $3 );
                           } /* if */
                         $$ = MSset_single( mmSingle, $1 == $3 ? 0 : $1, $3 );
                       }
		;


map_list           : mapping  /* linksrekursiv !!! */
                       {
                         $$ = MScopy( mmLeaf, $1 );
                       }
                   | map_list ',' mapping
                       {
                         if ( MScount( *($3), mmLeaf ) )
                           {
                             yyerror( "Mapping double" );
                           } /* if */
                         $$ = MSunion( mmLeaf, $3 );
                       }
		;


function           : FUNCTION
                       {
                         if (($$=StrToFunction(yylval.String,NULL))==0)
                           {
                             yyerror( "Function not defined!" );
                           } /* if */
                       }
		;


%%


/*****************************************************************************/
/* Fehlerbehandlungsroutine, die von dem von "yacc" erzeugten Parser benoe-  */
/* tigt wird; diese wird bei Fehlern, die waehrend des Parsens des Ausdrucks */
/* auftreten, aufgerufen und gibt die Fehlermeldung `msg` auf dem Bildschirm */
/* mitsamt der passenden Zeile und Spalte (von Ende des gelesenen Token)     */
/*****************************************************************************/

int yyerror(char *msg)
  {
    fprintf(stderr,"\nError: line %i, row %i: %s !\n\n",line,column-1,msg);
    return 0;
  } /* yyerror() */

/**************************/
/* END OF FILE boxxpars.y */
/**************************/
