/**********************************************************************/
/* clockbox.c                                                         */
/* Funktionen, die bei Abarbeitung des Syntaxbaumes aufgerufen werden */
/* Christian Stehno, 1.9.1999, basierend auf box.c                    */
/**********************************************************************/

#include <stdio.h>
#include "clockbox.h"
#include "boxtypes.h"
#include "box_sub.h"
#include "dat_cha.h"
#include "iter.h"
#include "move.h"
#include "main.h"

#define __SimpleChoice__

/***********************************/
/* Basic Box mit Multimengen-Label */
/***********************************/

BOX CLOCKBOXMultiLabelAction(TTree Knoten)
{
    OUT("\tMultiLabelAction\n");
    return CreateBasicClocknet((MultiSet)Knoten->Operand1,
                             (Time_stamp *)Knoten->Operand2);
} /* CLOCKBOXMultiLabelAction() */

/*********************************/
/* Basic Box mit Variablen-Label */
/*********************************/

BOX CLOCKBOXVariable(TTree Knoten)
{
    LPNet net;
    StackElem *ptr;

    OUT("\tVariable\n");
    ptr=search((Var)*(((MultiSet)Knoten->Operand1)+1));
    if (ptr) {
        if (ptr->count) {                               /* Rekursion */
            if (--ptr->count)
                net=Eval(ptr->knoten);
            else
                (net=OnlyPlaces(0))->Places->numname=1;
            ++ptr->count;
        } else {                                        /* Verfeinerung */
            net=LPNCreateNetCopy((LPNet)ptr->knoten);
        }
    } else
        net=CreateBasicClocknet((MultiSet)Knoten->Operand1,
                              (Time_stamp *)Knoten->Operand2);
    return net;
} /* CLOCKBOXVariable() */

/***********/
/* Sequenz */
/***********/

BOX CLOCKBOXSequence(TTree Knoten)
{
    register LPNet    net1,net2;
    register Place    pl1,pl2;
             STEdge   stlast, newst, stdummy;
             TSEdge   tsdummy;
             Cardinal x_dim;
             int      x_pos;

    net1=Eval(Knoten->Operand1);
    net2=Eval(Knoten->Operand2);
    OUT("\tSequence\n");

    move_hv(net2,net1->x_pos-net2->x_pos
                 +(int)(net1->x_dim-net2->x_dim)/2,
                 net1->y_pos-net2->y_pos+net1->y_dim-1);
    if (net2->x_dim>net1->x_dim) {
        net1->x_dim=net2->x_dim;
        net1->x_pos=net2->x_pos;
    }
    net1->y_dim+=net2->y_dim-1;

    x_dim=0;
    PreparePlaces(net2,ENTRY,ENTRY);
    pl1=net1->LastPlace;
    while (pl1 && pl1->numname)
      {
        if (pl1->label&EXIT)      /* pl1 laeuft durch die net1-EXIT-Stellen  */
          {
            pl2=net2->LastPlace;  /* pl2 laeuft durch die net2-ENTRY-Stellen */
            while (pl2) {
                x_dim+=2;
                DuplicatePlace(net1,pl1,&stlast,&tsdummy)->label&=~EXIT;
                newst=CopyPlacePostset(pl2,net1->Places,&stdummy);
                if(stlast!=NULL)
                  stlast->nexttrans=newst;
                else
                  net1->Places->postset=newst;
                newst->prevtrans=stlast;
                pl2=pl2->prevplace;
            }
            pl1=LPNRemovePlace(net1,pl1);
          }
        else
          pl1=pl1->prevplace;
      } /* while pl1 */
    /* Im zweiten Netz wird kein Choice mehr aktiv */
    for (pl2=net2->Places;pl2;pl2=pl2->nextplace)
      {
        if (pl2->label&CHOICEPL)
          pl2->label=EMPTY;
      }

    x_pos=(int)(net1->x_dim-(--x_dim))/2+net1->x_pos;
    if (x_dim>net1->x_dim) {
        net1->x_dim=x_dim;
        net1->x_pos=x_pos;
    }
    while (pl1) {
        x_pos=(pl1->x_pos=x_pos)+2;
        pl1->numname=1;
        pl1=pl1->prevplace;
    }

    return DisjointNetUnion(net1,RestoreKillPlaces(net2,ENTRY));
} /* CLOCKBOXSequence() */

/***********/
/* Auswahl */
/***********/

BOX CLOCKBOXChoice(TTree Knoten)
  {
    register LPNet      net1,net2,newnet;
    register Place      pl1,pl2,pl,newplace;
    register Transition tr1,tr2,tr,newtr;
             STEdge     st1,st2,st;
             TSEdge     ts1,ts2,ts;
             Cardinal32 time;
             Cardinal32 max1=0, max2=0, max;

    net1=Eval(Knoten->Operand1);
    net2=Eval(Knoten->Operand2);
    OUT("\tChoice\n");

    juxtapose(net1,net2);

#ifndef __SimpleChoice__
    /* Gemeinsames Warten sicher stellen */
    newnet=CommonWait(net1,net2);
    /* Den Ursprung der Synchronisierten entfernen */
    /* gleichzeitig Kanten einfuegen bei Schalttransitionen */
    /* um den Nachbereich des anderen Netz zu versorgen */
    for (tr1=net1->LastTransition;tr1;tr1=tr1->prevtrans)
      {
        if (tr1->type==TIC)
          LPNRemoveTransition(net1,tr1);
        else if (tr1->type==NORMAL)
          {
            /* Hinten anhaengen */
            ts=tr1->postset;
            while(ts&&ts->nextplace!=NULL)
              ts=ts->nextplace;
            for (pl2=net2->Places;pl2!=NULL;pl2=pl2->nextplace)
              {
                if (pl2->label&EXIT)
                  {
                    ts1=LPNallocTSEdge();
                    ts1->nextplace=NULL;
                    ts->nextplace=ts1;
                    ts1->prevplace=ts;
                    ts=ts1;

                    ts2=pl2->preset;
                    ts1->nexttrans=ts2;
                    if (ts2!=NULL)
                      ts2->prevtrans=ts1;
                    ts1->prevtrans=NULL;
                    pl2->preset=ts1;
                    ts1->place=pl2;
                    ts1->trans=tr1;
                    ts1->weight=1;
                  } /* if pl2&EXIT */
              } /* for pl2 */
          } /* if tr1==TIC||tr1==NORMAL */
      } /* for tr1 */
    for (pl1=net1->LastPlace;pl1!=NULL;pl1=pl1->prevplace)
      {
        if (pl1->label&EXIT)
          {
            ts=pl1->preset;
            while(ts&&ts->nexttrans!=NULL)
              ts=ts->nexttrans;
            for (tr2=net2->Transitions;tr2!=NULL;tr2=tr2->nexttrans)
              {
                if (tr2->type==TIC)
                  LPNRemoveTransition(net2,tr2);
                else if (tr2->type==NORMAL)
                  {
                    ts1=LPNallocTSEdge();
                    ts2=tr2->postset;
                    ts1->prevplace=NULL;
                    ts2->prevplace=ts1;
                    ts1->nextplace=ts2;
                    tr2->postset=ts1;

                    ts1->nexttrans=NULL;
                    if (ts!=NULL)
                      ts->nexttrans=ts1;
                    else
                      pl1->preset=ts1;
                    ts1->prevtrans=ts;
                    ts=ts1;
                    ts1->place=pl1;
                    ts1->trans=tr2;
                    ts1->weight=1;
                  } /* if tr2==TIC||tr2==NORMAL */
              } /* for tr2 */
          } /* if pl1&EXIT */
      } /* for pl1 */

    /* Schalttransitionen duplizieren und bei einer eine Doppelkante,    */
    /* bei der anderen im Vorbereich die abzuziehenden Stellen einfuegen */
    for (tr1=net1->LastTransition;tr1;tr1=tr1->prevtrans)
      {
        if (tr1->type==NORMAL) 
          {
            newtr=DuplicateTrans(newnet,tr1,&st1,&ts1);
            newtr->x_pos++;
            newtr->y_pos++;
            newtr->label=MScreate_copy(tr1->label);
            /* Abziehende Kanten einfuegen */
            for (st=newtr->preset;st&&st->nextplace;st=st->nextplace)
              skip;
            time=tr1->preset->place->tokenage;
            for (pl2=net2->Places;pl2;pl2=pl2->nextplace)
              {
                if (pl2->label&(ENTRY|CHOICEPL) &&
                    pl2->tokenage==time)
                  {
                    st1=LPNallocSTEdge();
                    st1->nextplace=NULL;
                    st->nextplace=st1;
                    st1->prevplace=st;
                    st=st1;

                    st2=pl2->postset;
                    st1->nexttrans=st2;
                    if (st2!=NULL)
                      st2->prevtrans=st1;
                    st1->prevtrans=NULL;
                    pl2->postset=st1;
                    st1->place=pl2;
                    st1->trans=newtr;
                    st1->weight=1;
                  } /* if pl2 */
              } /* for pl2 */
            /* Doppelkanten einfuegen */
            for (st=tr1->preset;st&&st->nextplace;st=st->nextplace)
              skip;
            for (pl2=net2->Places;pl2!=NULL;pl2=pl2->nextplace)
              {
                if (pl2->label&EXIT)
                  {
                    st1=LPNallocSTEdge();
                    st1->nextplace=NULL;
                    st->nextplace=st1;
                    st1->prevplace=st;
                    st=st1;

                    st2=pl2->postset;
                    st1->nexttrans=st2;
                    if (st2!=NULL)
                      st2->prevtrans=st1;
                    st1->prevtrans=NULL;
                    pl2->postset=st1;
                    st1->place=pl2;
                    st1->trans=tr1;
                    st1->weight=1;
                  } /* if pl2&EXIT */
              } /* for pl2 */
          } /* if tr1==NORMAL */
      } /* for tr1 */
    for (tr2=net2->Transitions;tr2!=NULL;tr2=tr2->nexttrans)
      {
        if (tr2->type==NORMAL)
          {
            newtr=DuplicateTrans(newnet,tr2,&st2,&ts2);
            newtr->x_pos++;
            newtr->y_pos++;
            newtr->label=MScreate_copy(tr2->label);
            /* Abziehende Kanten einfuegen */
            time=tr2->preset->place->tokenage;
            for (pl1=net1->Places;pl1;pl1=pl1->nextplace)
              {
                if (pl1->label&(ENTRY|CHOICEPL) &&
                    pl1->tokenage==time)
                  {
                    st1=LPNallocSTEdge();
                    st2=tr2->preset;
                    st1->prevplace=NULL;
                    st2->prevplace=st1;
                    st1->nextplace=st2;
                    tr2->preset=st1;

                    st2=pl1->postset;
                    while(st2&&st2->nexttrans!=NULL)
                      st2=st2->nexttrans;
                    st1->nexttrans=NULL;
                    if (st2!=NULL)
                      st2->nexttrans=st1;
                    else
                      pl1->postset=st1;
                    st1->prevtrans=st2;
                    st1->place=pl1;
                    st1->trans=newtr;
                    st1->weight=1;
                  } /* if pl2 */
              } /* for pl2 */
            /* Doppelkanten einfuegen */
            for (pl1=net1->LastPlace;pl1!=NULL;pl1=pl1->prevplace)
              {
                if (pl1->label&EXIT)
                  {
                    st1=LPNallocSTEdge();
                    st2=tr2->preset;
                    st1->prevplace=NULL;
                    st2->prevplace=st1;
                    st1->nextplace=st2;
                    tr2->preset=st1;

                    st2=pl1->postset;
                    while(st2&&st2->nexttrans!=NULL)
                      st2=st2->nexttrans;
                    st1->nexttrans=NULL;
                    if (st2!=NULL)
                      st2->nexttrans=st1;
                    else
                      pl1->postset=st1;
                    st1->prevtrans=st2;
                    st1->place=pl1;
                    st1->trans=tr2;
                    st1->weight=1;
                  } /* if pl1&EXIT */
              } /* for pl1 */
          } /* if tr2==NORMAL */
      } /* for tr2 */
    DisjointNetUnion(net1, newnet);
#else
    /* Choice-Stellen reduzieren */
    for (pl=net1->Places;pl;pl=pl->nextplace)
      {
        if (pl->label&CHOICEPL && pl->tokenage>max1)
          max1=pl->tokenage;
      }
    for (pl=net2->Places;pl;pl=pl->nextplace)
      {
        if (pl->label&CHOICEPL && pl->tokenage>max2)
          max2=pl->tokenage;
      }
    max=(max1<max2)?max1:max2;
    for (pl=net1->Places;pl;pl=pl->nextplace)
      {
        if (pl->label&CHOICEPL && pl->tokenage>max)
          pl->label&=~CHOICEPL;
      }
    for (pl=net2->Places;pl;pl=pl->nextplace)
      {
        if (pl->label&CHOICEPL && pl->tokenage>max)
          pl->label&=~CHOICEPL;
      }

    for (pl1=net1->Places;pl1;pl1=pl1->nextplace)
      {
        if (pl1->label&(ENTRY|EXIT|CHOICEPL))
          {
            for (pl2=net2->Places;pl2;pl2=pl2->nextplace)
              {
                if (pl2->tokenage==pl1->tokenage && 
                    pl2->label&pl1->label)
                  {
                    pl1->label|=DELETE;
                    pl=DuplicatePlace(net1,pl1,&st,&ts);
                    pl->label&=~DELETE;
                    pl1->x_pos++;
                    if (ts && pl2->preset)
                      {
                        ts->nexttrans=CopyPlacePreset(pl2,pl,&ts1);
                        ts->nexttrans->prevtrans=ts;
                      }
                    else 
                      {
                        if (pl2->preset)
                          pl->preset=CopyPlacePreset(pl2,pl,&ts1);
                      }
                    if (st && pl2->postset)
                      {
                        st->nexttrans=CopyPlacePostset(pl2,pl,&st1);
                        st->nexttrans->prevtrans=st;
                      }
                    else 
                      {
                        if (pl2->postset)
                          pl->postset=CopyPlacePostset(pl2,pl,&st1);
                      }
                  } /* if label und tokenage gleich */
              } /* for pl2 */
            if (pl1->label&DELETE)
              pl1=LPNRemovePlace(net1, pl1);
          } /* if pl1->label */
      } /* for pl1 */
    /* Doppelte Stellen loeschen */
    RestoreKillPlaces(net2, (ENTRY|EXIT|CHOICEPL));

    /* Falsche TICs loeschen */
    for (tr1=net1->Transitions;tr1;tr1=tr1->nexttrans)
      {
        if (tr1->type==TIC)
          {
            tr1->type=NORMAL; /* zeigt an, dass diese zu loeschen ist */
            for (tr2=net2->Transitions;tr2;tr2=tr2->nexttrans)
              {
                if (tr2->type==TIC)
                  {
                    for (st1=tr1->preset,st2=tr2->preset;
                         st1 && st2 && st1->place==st2->place;
                         st1=st1->nextplace,st2=st2->nextplace)
                      skip;
                    /* Entferne Duplikat */
                    if (!(st1||st2))
                      {
                        LPNRemoveTransition(net2,tr2);
                        tr2=net2->LastTransition;
                        tr1->type=TIC;
                      }
                  } /* if tr2==TIC */
              } /* for tr2 */
            if (tr1->type==NORMAL)
              tr1=LPNRemoveTransition(net1,tr1);
          } /* if tr1==TIC */
      } /* for tr1 */
    for (tr2=net2->Transitions;tr2;skip)
      {
        if (tr2->type==TIC)
          tr2=LPNRemoveTransition(net2,tr2);
        if (tr2)
          tr2=tr2->nexttrans;
      }
#endif
    return DisjointNetUnion(net1,net2);
  } /* CLOCKBOXChoice() */

/********************/
/* Nebenlaeufigkeit */
/********************/

BOX CLOCKBOXParallel(TTree Knoten)
{
    LPNet      net1,net2;
    Transition tr,tr1,tr2,newtr;
    Place      pl;
    STEdge     st1,st2=NULL;
    TSEdge     ts1,ts2=NULL;

    net1=Eval(Knoten->Operand1);
    net2=Eval(Knoten->Operand2);
    OUT("\tConcurrentComposition\n");

    /* Netze nebeneinander stellen und Groessen anpassen */
    juxtapose(net1,net2);

    tr=net1->Transitions;
    /* Neue gemeinsame Tic-Transitionen erstellen */
    for (tr1=net1->Transitions;tr1!=NULL;tr1=tr1->nexttrans)
      {
        if (tr1->type & (TIC|TIC2))
          {
            for (tr2=net2->Transitions;tr2!=NULL;tr2=tr2->nexttrans)
              {
                if (tr2->type & (TIC|TIC2))
                  {
                    newtr=DuplicateTrans(net1,tr1,&st1,&ts1);
                    (st1->nextplace=CopyTransPreset(tr2,newtr,&st2))
                        ->prevplace=st1;
                    (ts1->nextplace=CopyTransPostset(tr2,newtr,&ts2))
                        ->prevplace=ts1;
                    newtr->x_pos=(tr1->x_pos+tr2->x_pos)/2;
                    newtr->y_pos=(tr1->y_pos+tr2->y_pos)/2;
                    newtr->type=TIC;
                  } /* if tr2 */
              } /* for tr2 */
          } /* if tr1 */
      } /* for tr1 */

    /* Erste (alte) Tic-Transition beachten */
    tr=tr->prevtrans;
    /* Einzeln schalten in Netz1 zulassen */
    for (tr1=net1->LastTransition;tr1!=tr;tr1=tr1->prevtrans)
      {
        if (tr1->type==TIC)
          {
            tr1->type=TIC2;
            for (pl=net2->Places;pl!=NULL;pl=pl->nextplace)
              {
                if (pl->label&EXIT)
                  {
                    ts1=LPNallocTSEdge();
                    ts2=tr1->postset;
                    while(ts2&&ts2->nextplace!=NULL)
                      ts2=ts2->nextplace;
                    ts1->nextplace=NULL;
                    ts2->nextplace=ts1;
                    ts1->prevplace=ts2;

                    ts2=pl->preset;
                    ts1->nexttrans=ts2;
                    if (ts2!=NULL)
                      ts2->prevtrans=ts1;
                    ts1->prevtrans=NULL;
                    pl->preset=ts1;
                    ts1->place=pl;
                    ts1->trans=tr1;
                    ts1->weight=1;

                    st1=LPNallocSTEdge();
                    st2=tr1->preset;
                    while(st2&&st2->nextplace!=NULL)
                      st2=st2->nextplace;
                    st1->nextplace=NULL;
                    st2->nextplace=st1;
                    st1->prevplace=st2;

                    st2=pl->postset;
                    st1->nexttrans=st2;
                    if (st2!=NULL)
                      st2->prevtrans=st1;
                    st1->prevtrans=NULL;
                    pl->postset=st1;
                    st1->place=pl;
                    st1->trans=tr1;
                    st1->weight=1;
                  } /* if pl&EXIT */
              } /* for pl */
          } /* if tr==TIC */
      } /* for tr */

    /* Einzeln schalten in Netz2 zulassen */
    for (pl=net1->LastPlace;pl!=NULL;pl=pl->prevplace)
      {
        if (pl->label&EXIT)
          {
            for (tr=net2->Transitions;tr!=NULL;tr=tr->nexttrans)
              {
                if (tr->type==TIC)
                  {
                    ts1=LPNallocTSEdge();
                    ts2=tr->postset;
                    ts1->prevplace=NULL;
                    ts2->prevplace=ts1;
                    ts1->nextplace=ts2;
                    tr->postset=ts1;

                    ts2=pl->preset;
                    while(ts2&&ts2->nexttrans!=NULL)
                      ts2=ts2->nexttrans;
                    ts1->nexttrans=NULL;
                    if (ts2!=NULL)
                      ts2->nexttrans=ts1;
                    else
                      pl->preset=ts1;
                    ts1->prevtrans=ts2;
                    ts1->place=pl;
                    ts1->trans=tr;
                    ts1->weight=1;

                    st1=LPNallocSTEdge();
                    st2=tr->preset;
                    st1->prevplace=NULL;
                    st2->prevplace=st1;
                    st1->nextplace=st2;
                    tr->preset=st1;

                    st2=pl->postset;
                    while(st2&&st2->nexttrans!=NULL)
                      st2=st2->nexttrans;
                    st1->nexttrans=NULL;
                    if (st2!=NULL)
                      st2->nexttrans=st1;
                    else
                      pl->postset=st1;
                    st1->prevtrans=st2;
                    st1->place=pl;
                    st1->trans=tr;
                    st1->weight=1;
                  } /* if pl&EXIT */
              } /* for pl */
          } /* if tr==TIC */
      } /* for tr */

    for (tr=net2->Transitions;tr!=NULL;tr=tr->nexttrans)
      {
        if (tr->type==TIC)
          tr->type=TIC2;
      }

    return DisjointNetUnion(net1,net2);

} /* CLOCKBOXParallel() */

/*******************/
/* Synchronisation */
/*******************/

BOX CLOCKBOXSynchronisation(TTree Knoten)
{
    LPNet net;

    net=Eval(Knoten->Operand1);
    OUT("\tSynchronisation\n");
    return Synchronisation(net,(MultiSet)Knoten->Operand2,Knoten->Depth);
} /* CLOCKBOXSynchronisation() */

/***************/
/* Restriktion */
/***************/

BOX CLOCKBOXRestriction(TTree Knoten)
{
    LPNet net;

    net=Eval(Knoten->Operand1);
    OUT("\tRestriction\n");
    return CLOCKRestriction(net,(MultiSet)Knoten->Operand2);
} /* CLOCKBOXRestriction() */

/***********/
/* Scoping */
/***********/

BOX CLOCKBOXScoping(TTree Knoten)
{
    LPNet net;

    net=Eval(Knoten->Operand1);
    OUT("\tScoping\n");

    return CLOCKRestriction(
        Synchronisation(net,(MultiSet)Knoten->Operand2,Knoten->Depth),
        (MultiSet)Knoten->Operand2);

} /* CLOCKBOXScoping() */

BOX CLOCKBOXTie(TTree Knoten)
{
  return (Eval(Knoten->Operand1));
} /* CLOCKBOXTie */

/***************/
/* Umbenennung */
/***************/

BOX CLOCKBOXRelabelling(TTree Knoten)
{
    LPNet      net;
    Function   f;
    Transition tr;

    net=Eval(Knoten->Operand1);
    OUT("\tRelabelling: ");
    if ((Cardinal)(Cardinal32)Knoten->Operand2==FUNCTION) {
        OUT("\tFUNCTION NAME\n");
        f=FunctionTable[(Cardinal)(Cardinal32)Knoten->Operand3-1].FuncMappings;
    } else {    /* (Cardinal)Knoten->Operand2==MAP ! */
        OUT("\tMAPPING\n");
        f=(Function)Knoten->Operand3;
    }

    if (*f)
      {
        tr=net->Transitions;
        while (tr)
          {
            tr->label=MSrelabel(tr->label,f);
            tr=tr->nexttrans;
          }
      }
    return net;
} /* CLOCKBOXRelabelling() */

/****************/
/* Verfeinerung */
/****************/

BOX CLOCKBOXRefinement(TTree Knoten)
{
    LPNet net,rnet;

    OUT("\tRefinement\n");
    rnet=Eval(Knoten->Operand3);
    push((Var)(Cardinal32)Knoten->Operand2,(Cardinal)0,(TTree)rnet);
    net=Eval(Knoten->Operand1);
    LPNRemove_net(&rnet);
    pop();
    return net;
} /* CLOCKBOXRefinement() */

/*************/
/* Rekursion */
/*************/

BOX CLOCKBOXRecursion(TTree Knoten)
{
    LPNet net;

    OUT("\tRecursion\n");
    push((Var)(Cardinal32)Knoten->Operand1,Knoten->Depth,Knoten->Operand2);
    net=Eval(Knoten->Operand2);
    pop();
    return net;
} /* CLOCKBOXRecursion() */

/*************/
/* Iteration */
/*************/

BOX CLOCKBOXIteration(TTree Knoten)
{
    OUT("\tIteration\n");
    return CLOCKSmallIteration(Knoten);
} /* CLOCKBOXIteration */

/************************/
/* Filename-Operationen */
/************************/

BOX CLOCKBOXFilename(TTree Knoten)
{
    OUT("\tFilename-Operation\n");
    PEPerror("\tFilename operation not yet implemented !!!");
    return NULL;
}

/***************/
/* Channel-Box */
/***************/

BOX CLOCKBOXChannel(TTree Knoten)
{
    OUT("\tChannel-Box\n");
    return ChannelBox(Knoten);
}

/*************/
/* Stack-Box */
/*************/

BOX CLOCKBOXStack(TTree Knoten)
{
    OUT("\tStack-Box\n");
    return (Knoten->Depth)?StackBox(Knoten):ChannelBox(Knoten);
}

/************/
/* Data-Box */
/************/

BOX CLOCKBOXData(TTree Knoten)
{
    OUT("\tData-Box\n");
    return DataBox(Knoten);
}

/************/
/* Stop-Box */
/************/

BOX CLOCKBOXStop(TTree Knoten)
{
    LPNet net;

    OUT("\tStop-Box\n");
    /* Um Warnungen zu vermeiden */
    Knoten=Knoten;
    (net=OnlyPlaces(0))->Places->numname=1;
    return net;
}

/*******************/
/* Atomare Auswahl */
/*******************/

BOX CLOCKBOXAtomChoice(TTree Knoten)
{
    return CLOCKBOXChoice(Knoten);
} /* CLOCKBOXAtomChoice() */

/****************************************/
/* Initialisierung der Funktionstabelle */
/****************************************/

void CLOCKBOXInitOpArray()
{
    unsigned char OpZaehler = 0;

    Operation[OpZaehler++] = CLOCKBOXMultiLabelAction;
    Operation[OpZaehler++] = CLOCKBOXVariable;
    Operation[OpZaehler++] = CLOCKBOXSequence;
    Operation[OpZaehler++] = CLOCKBOXChoice;
    Operation[OpZaehler++] = CLOCKBOXParallel;
    Operation[OpZaehler++] = CLOCKBOXSynchronisation;
    Operation[OpZaehler++] = CLOCKBOXRestriction;
    Operation[OpZaehler++] = CLOCKBOXScoping;
    Operation[OpZaehler++] = CLOCKBOXTie;
    Operation[OpZaehler++] = CLOCKBOXRelabelling;
    Operation[OpZaehler++] = CLOCKBOXRefinement;
    Operation[OpZaehler++] = CLOCKBOXRecursion;
    Operation[OpZaehler++] = CLOCKBOXIteration;
    Operation[OpZaehler++] = CLOCKBOXFilename;
    Operation[OpZaehler++] = CLOCKBOXChannel;
    Operation[OpZaehler++] = CLOCKBOXData;
    Operation[OpZaehler++] = CLOCKBOXStop;
    Operation[OpZaehler++] = CLOCKBOXAtomChoice;
    Operation[OpZaehler  ] = CLOCKBOXStack;

} /* CLOCKBOXInitOpArray() */

/**************************/
/* END OF FILE clockbox.c */
/**************************/
