/*****************************************************************/
/* dat_cha.c                                                     */
/* Burkhard Graves, 24.10.'93                                    */
/* Aenderung 6.6.1999: Tippfehler entfernt      -Christian Stehno*/
/* Aenderung 8.6.1999: CreateTr nach box_sub.c verschoben     -CS*/
/*****************************************************************/

#include <stdio.h>
#include <string.h>
#include "dat_cha.h"
#include "main.h"
#include "boxtypes.h"
#include "box.h"
#include "box_sub.h"
#include "parser.h"

/*****************************************************************/
/* Es gibt drei Versionen von Stackboxen, momentan tut's nur die */
/* zweite (ist aber auch die schoenste).                         */
/*****************************************************************/

#define VER2

#define act (Action)(Cardinal32)(Knoten->Operand1)
#define kap (Knoten->Depth)

/********************************************************/
/* Diverse Unterroutinen von 'DataBox' und 'ChannelBox' */
/********************************************************/

/* Transitionslabel erstellen: Variable_von_nach */
static MultiSet CreateDatMS(Action action,Cardinal vor,Cardinal nach)
{
    char str[32];

    ActionToStr(action,str);
    sprintf(strchr(str,0),"_%hu",(Cardinal)vor);
    sprintf(strchr(str,0),"_%hu",(Cardinal)nach);
    return MScreate_singleton(MSconj_action(StrToAction(str)));
}

/*****/
/*   */
/*****/

/* Transitionslabel erstellen: VariableZEICHENx, wobei Zeichen !,? sind und */
/*   x der Wert                                                             */
static MultiSet CreateChaMS(Action action,char zeichen,Cardinal wert)
{
    char str[32];

    ActionToStr(action,str);
    sprintf(strchr(str,0),"%c%hu",zeichen,(Cardinal)(wert));
    return MScreate_singleton(MSconj_action(StrToAction(str)));
}

/*****/
/*   */
/*****/

/* Transitionslabel mit zwei Zeichen: Variable!! oder Variable?? */
static MultiSet CreateChaMS_2(Action action,char *ptr)
{
    char str[32];

    ActionToStr(action,str);
    strcat(str,ptr);
    return MScreate_singleton(MSconj_action(StrToAction(str)));
}

/*****/
/*   */
/*****/

static Place CreatePreset(Cardinal preset,Place pl,LPNet net)
{
    STEdge st;

    /* Wenn preset, dann waehle diese Stelle */
    if (preset) {
        while (preset<pl->numname) pl=pl->prevplace;
        while (preset>pl->numname) pl=pl->nextplace;
    } else {
        pl=net->Places; /* Entry-Stelle hat keinen Vorbereich*/
    }
    /* Soll erste Kante werden */
    (st=LPNallocSTEdge())->prevtrans=NULL;
    /* Start ist pl */
    st->place=pl;
    /* Wenn es weitere Kanten gibt, dann passend verketten */
    if ((st->nexttrans=pl->postset)!=NULL)
        st->nexttrans->prevtrans=st;
    /* Als erste Kante einfuegen */
    (pl->postset=st)->prevplace=0;
    /* Ziel ist die erste Transition */
    st->trans=net->Transitions;
    /* Wenn es weitere Kanten gibt, dann passend verketten */
    if ((st->nextplace=st->trans->preset)!=NULL)
        st->nextplace->prevplace=st;
    /* Kante im Netz einfuegen mit Gewicht 1 */
    (net->Transitions->preset=st)->weight=1;
    return pl;
}

/*****/
/*   */
/*****/

static Place CreatePostset(Cardinal postset,Place pl,LPNet net)
{
    TSEdge ts;

    if (postset) {
        while (postset<pl->numname) pl=pl->prevplace;
        while (postset>pl->numname) pl=pl->nextplace;
    } else {
        pl=net->LastPlace; /* Exit-Stelle */
    }
    (ts=LPNallocTSEdge())->prevtrans=NULL;
    if ((ts->nexttrans=(ts->place=pl)->preset)!=NULL)
        ts->nexttrans->prevtrans=ts;
    (pl->preset=ts)->prevplace=0;
    if ((ts->nextplace=(ts->trans=net->Transitions)->postset)!=NULL)
        ts->nextplace->prevplace=ts;
    (net->Transitions->postset=ts)->weight=1;
    return pl;
}

/*****/
/*   */
/*****/

static Place CreateEdge(Cardinal preset,Cardinal postset,Place pl,LPNet net)
{
    return CreatePostset(postset,CreatePreset(preset,pl,net),net);
}

/************/
/* Data-Box */
/************/

LPNet DataBox(TTree Knoten)
{
    LPNet      net;
    Place      pl;
    Transition tr;
    Range      *range_ptr1,*range_ptr2;
    Cardinal   i,j,n,vor,nach;
    int        pos;
    char       str[16];

    n=0;
    range_ptr1=(Range *)(Knoten->Operand2);
    while (range_ptr1) {
        n+=(range_ptr1->bis)-(range_ptr1->von)+1;
        range_ptr1=range_ptr1->next;
    }
    (net=OnlyPlaces(n))->NumTransitions=
    (flag&SIMPLE_CD)?
        ((kap==32767)? n*(n+2)+1 : n*(n+1)+2)
    :
        ((kap==32767)? n*(2*n+1)+1 : n*(n+2)+1);

    pos=net->LastPlace->y_pos;
    net->LastPlace->y_pos=pos+2;
    net->y_dim+=2;
    /* Exit-Transition e->x */
    CreateTr(net,MScreate_singleton(MSconj_action(act)),0,pos);
    pl=CreateEdge(0,0,NULL,net);
    range_ptr1=(Range *)(Knoten->Operand2);
    vor=range_ptr1->von-1;
    for (i=1; i<=n; ++i) {
        if (++vor>range_ptr1->bis)
            vor=(range_ptr1=range_ptr1->next)->von;
        /* o<->[] */
        CreateTr(net,CreateDatMS(act,vor,vor),0,pos);
        pl=CreateEdge(i,i,pl,net);
        ActionToStr(act,str);
        sprintf(strchr(str,0),"=%hu",(Cardinal)vor);
        pl->name=(char *)strdup(str);
        /* Exit-Transition */
        CreateTr(net,MScreate_singleton(MSconj_action(act)),0,pos);
        pl=CreateEdge(i,0,pl,net);
        if (flag&SIMPLE_CD && (kap==32767 || kap==vor)) {
            /* Entry-Transition */
            CreateTr(net,MScreate_empty_set(),0,net->Places->y_pos);
            pl=CreateEdge(0,i,pl,net);
        }
        range_ptr2=(Range *)(Knoten->Operand2);
        nach=range_ptr2->von-1;
        for (j=1; j<=n; ++j) {
            if (++nach>range_ptr2->bis)
                nach=(range_ptr2=range_ptr2->next)->von;
            if (!(flag&SIMPLE_CD) && (kap==32767 || kap==nach)) {
                /* Entry-Transition */
                CreateTr(net,CreateDatMS(act,nach,vor),0,pos);
                pl=CreateEdge(0,i,pl,net);
            }
            if (j!=i) {
                /* normale Uebergaenge */
                CreateTr(net,CreateDatMS(act,nach,vor),0,pos);
                pl=CreateEdge(j,i,pl,net);
            }
        }
    }

    for ( pl=net->Places ; pl ; pl->marks|=1, pl=pl->nextplace );
    for ( tr=net->Transitions ; tr ; tr->flags|=1, tr=tr->nexttrans );

    return net;
}

/***************/
/* Channel-Box */
/***************/

LPNet ChannelBox(TTree Knoten)
{
    LPNet      net;
    Place      pl;
    Transition tr;
    Range      *range_ptr1;
    Cardinal   i,j,n,wert1;
    int        xpos,ypos;

    n=0;
    range_ptr1=(Range *)(Knoten->Operand2);
    while (range_ptr1) {
        n+=(range_ptr1->bis)-(range_ptr1->von)+1;
        range_ptr1=range_ptr1->next;
    }
    if (kap) {
        net=OnlyPlaces(kap*(n+2));
        net->NumTransitions=4+n*(kap+1)+((flag&SIMPLE_CD)?1:n);
    } else {
        net=OnlyPlaces(1);
        net->NumTransitions=2+n+((flag&SIMPLE_CD)?1:n);
    }

    /* Exit-Transition e->x (Positionierung fuer kap>0 erfolgt spaeter) */
    CreateTr(net,MScreate_singleton(MSconj_action(act)),0,6);
    pl=CreateEdge(0,0,NULL,net);
    if (kap) {

        /* Channel mit Kapazitaet > 0 */

        net->x_dim=-2*(net->x_pos=-2*kap)+1;
        i=net->y_dim=11+2*n;
        net->LastPlace->y_pos=i-1;
        net->LastTransition->y_pos=i-3;
        --net->LastTransition->x_pos;
        CreateTr(net,MScreate_singleton(MSconj_action(act)),1,i-3);
        for (i=kap; i; --i) {
            pl=CreatePreset(i,pl,net);
        }
        pl=CreatePostset(0,pl,net);
        pl=net->Places->nextplace;
        ypos=4;
        j=n+3;
        while (--j) {
            xpos=net->x_pos+2;
            for (i=1; i<=kap; ++i) {
                pl->x_pos=xpos;
                xpos+=4;
                pl->y_pos=ypos;
                pl=pl->nextplace;
            }
            ypos+=2;
        }
        CreateTr(net,CreateChaMS_2(act,"??"),-net->x_pos,4);
        for (i=kap; i; --i) {
            pl=CreateEdge(i,i,pl,net);
        }
        CreateTr(net,CreateChaMS_2(act,"!!"),net->x_pos,6);
        for (i=2*kap; i>kap; --i) {
            pl=CreateEdge(i,i,pl,net);
        }
        range_ptr1=(Range *)(Knoten->Operand2);
        wert1=range_ptr1->von-1;
        for (i=1; i<=n; ++i) {
            if (++wert1>range_ptr1->bis)
                wert1=(range_ptr1=range_ptr1->next)->von;
            /* normale Uebergaenge */
            CreateTr(net,CreateChaMS(act,'!',wert1),net->x_pos,i*2+6);
            pl=CreateEdge(1,kap*(i+1)+1,pl,net);
            pl=CreatePostset(kap+1,pl,net);
            CreateTr(net,CreateChaMS(act,'?',wert1),-net->x_pos,i*2+6);
            pl=CreateEdge(kap*(i+2),kap,pl,net);
            pl=CreatePreset(kap*2,pl,net);
            xpos=-net->x_pos;
            for (j=kap-1; j; --j) {
                CreateTr(net,MScreate_empty_set(),xpos-=4,i*2+6);
                pl=CreateEdge((i+1)*kap+j,(i+1)*kap+j+1,pl,net);
                pl=CreateEdge(j+kap,j+kap+1,pl,net);
                pl=CreateEdge(j+1,j,pl,net);
            }
        }
        /* Entry-Transition */
        if (flag&SIMPLE_CD) {
            CreateTr(net,MScreate_empty_set(),0,2);
            pl=CreatePreset(0,pl,net);
            for (i=kap; i; --i)
                pl=CreatePostset(i,pl,net);
        } else {
            j=2*n-1;
            xpos=(int)(net->x_dim-j)/2+net->x_pos;
            if (j>net->x_dim) {
                net->x_dim=j;
                net->x_pos=xpos;
            }
            range_ptr1=(Range *)(Knoten->Operand2);
            wert1=range_ptr1->von-1;
            for (i=1; i<=n; ++i) {
                if (++wert1>range_ptr1->bis)
                    wert1=(range_ptr1=range_ptr1->next)->von;
                CreateTr(net,CreateChaMS(act,'!',wert1),xpos,2);
                xpos+=2;
                pl=CreateEdge(0,kap*(i+1)+1,pl,net);
                pl=CreatePostset(kap+1,pl,net);
                for (j=kap; j>1; --j)
                    pl=CreatePostset(j,pl,net);
            }
        }
    } else {

        /* Channel mit Kapazitaet = 0 */

        net->Places->nextplace->y_pos=4;
        net->LastPlace->y_pos=8;
        /* Entry-Transition */
        if (flag&SIMPLE_CD) {
            CreateTr(net,MScreate_empty_set(),0,2);
            pl=CreateEdge(0,1,pl,net);
            net->x_dim=3;
        } else {
            net->x_dim=5;
            net->x_pos=-2;
        }
        /* Exit-Transition */
        CreateTr(net,MScreate_singleton(MSconj_action(act)),0,6);
        pl=CreateEdge(1,0,pl,net);
        j=0;
        range_ptr1=(Range *)(Knoten->Operand2);
        wert1=range_ptr1->von-1;
        for (i=1; i<=n; ++i) {
            if (++wert1>range_ptr1->bis)
                wert1=(range_ptr1=range_ptr1->next)->von;
            /* normale Uebergaenge */
            CreateTr(net,CreateChaMS(act,'!',wert1),2,j);
            net->Transitions->label=MSresize(net->Transitions->label,2);
            MSsum(net->Transitions->label,CreateChaMS(act,'?',wert1));
            pl=CreateEdge(1,1,pl,net);
            if (!(flag&SIMPLE_CD)) {
                /* Entry-Transition */
                CreateTr(net,MScreate_copy(net->Transitions->label),-2,j);
                pl=CreateEdge(0,1,pl,net);
            }
            j+=2;
        }
        net->y_dim=(--j>9)? j : 9;
    }

    for ( pl=net->Places ; pl ; pl->marks|=1, pl=pl->nextplace );
    for ( tr=net->Transitions ; tr ; tr->flags|=1, tr=tr->nexttrans );

    return net;
}

/*************/
/* Stack-Box */
/*************/

#ifdef VER1

LPNet StackBox(TTree Knoten)
{
    LPNet      net;
    Place      pl;
    Transition tr;
    Cardinal   i,j,k,n,wert1;
    int        xpos,ypos;
    MultiSet   mset;

    n=0;
    range_ptr1=(Range *)(Knoten->Operand2);
    while (range_ptr1) {
        n+=(range_ptr1->bis)-(range_ptr1->von)+1;
        range_ptr1=range_ptr1->next;
    }
    net=OnlyPlaces(kap*(n+2));
    net->NumTransitions=4+2*n*kap+((flag&SIMPLE_CD)?1:n);

    net->x_dim=-2*(net->x_pos=-kap)+1;
    i=net->y_dim=10+4*n;
    net->LastPlace->y_pos=i-1;
    /* Exit-Transition e->x */
    CreateTr(net,MScreate_singleton(MSconj_action(act)),-1,i-3);
    pl=CreateEdge(0,0,NULL,net);
    CreateTr(net,MScreate_singleton(MSconj_action(act)),1,i-3);
    for (i=kap; i; --i) {
        pl=CreatePreset(i,pl,net);
    }
    pl=CreatePostset(0,pl,net);
    pl=net->Places->nextplace;
    ypos=4;
    j=n+3;
    while (--j) {
        xpos=net->x_pos;
        for (i=1; i<=kap; ++i) {
            pl->x_pos=xpos;
            xpos+=2;
            pl->y_pos=ypos;
            pl=pl->nextplace;
        }
        if (j==n+2)
            ypos+=2;
        else if (j==n+1)
            ypos+=3;
        else
            ypos+=4;
    }
    CreateTr(net,CreateChaMS_2(act,"??"),-net->x_pos,4);
    for (i=kap; i; --i) {
        pl=CreateEdge(i,i,pl,net);
    }
    CreateTr(net,CreateChaMS_2(act,"!!"),-net->x_pos,6);
    for (i=2*kap; i>kap; --i) {
        pl=CreateEdge(i,i,pl,net);
    }
    /* normale Uebergaenge */
    for (i=1; i<=n; ++i) {
        mset=CreateChaMS(act,'!',wert1);
        for (j=1; j<=kap; ++j) {
            CreateTr(net,mset,-net->x_pos,i*4+4);
            pl=CreatePostset(kap*(i+1)+j,pl,net);
            pl=CreatePostset(kap+j,pl,net);
            for (k=j+kap-1; k>j; --k) {
                pl=CreateEdge(k,k,pl,net);
            }
            pl=CreatePreset(j,pl,net);
        }
        mset=CreateChaMS(act,'?',wert1);
        for (j=1; j<=kap; ++j) {
            CreateTr(net,mset,-net->x_pos,i*4+6);
            pl=CreatePreset(kap*(i+1)+j,pl,net);
            pl=CreatePreset(kap+j,pl,net);
            for (k=j+kap-1; k>j; --k) {
                pl=CreateEdge(k,k,pl,net);
            }
            pl=CreatePostset(j,pl,net);
        }
    }
    /* Entry-Transition */
    if (flag&SIMPLE_CD) {
        CreateTr(net,MScreate_empty_set(),0,2);
        pl=CreatePreset(0,pl,net);
        for (i=kap; i; --i)
            pl=CreatePostset(i,pl,net);
    } else {
        CreateTr(net,CreateChaMS_2(act,"??"),-net->x_pos,4);
        pl=CreateEdge(0,1,pl,net);
        j=2*n-1;
        xpos=(int)(net->x_dim-j)/2+net->x_pos;
        if (j>net->x_dim) {
            net->x_dim=j;
            net->x_pos=xpos;
        }
        for (i=1; i<=n; ++i) {
            CreateTr(net,CreateChaMS(act,'!',wert1),xpos,2);
            xpos+=2;
            pl=CreateEdge(0,kap*(i+1)+1,pl,net);
            pl=CreatePostset(kap+1,pl,net);
            for (j=kap; j>1; --j)
                pl=CreatePostset(j,pl,net);
        }
    }

    for ( pl=net->Places ; pl ; pl->marks|=1, pl=pl->nextplace );
    for ( tr=net->Transitions ; tr ; tr->flags|=1, tr=tr->nexttrans );

    return net;
}

#endif

#ifdef VER2

LPNet StackBox(TTree Knoten)
{
    LPNet      net;
    Place      pl;
    Transition tr;
    Range      *range_ptr1;
    Cardinal   i,j,n,wert1;
    int        xpos,ypos;
    MultiSet   mset;

    n=0;
    range_ptr1=(Range *)(Knoten->Operand2);
    while (range_ptr1) {
        n+=(range_ptr1->bis)-(range_ptr1->von)+1;
        range_ptr1=range_ptr1->next;
    }
    net=OnlyPlaces(kap*(n+1)+1);
    net->NumTransitions=5+2*n*kap;
    if (!(flag&SIMPLE_CD))
        net->NumTransitions+=n;

    net->x_dim=-2*(net->x_pos=-kap-1)+1;
    i=net->y_dim=9+4*n;
    net->LastPlace->y_pos=i-1;
    /* Exit-Transition e->x */
    CreateTr(net,MScreate_singleton(MSconj_action(act)),1,i-3);
    pl=CreateEdge(0,0,NULL,net);
    CreateTr(net,MScreate_singleton(MSconj_action(act)),-1,i-3);
    pl=CreateEdge(1,0,pl,net);
    pl=net->Places->nextplace;
    ypos=4;
    xpos=net->x_pos;
    j=n+2;
    while (--j) {
        for (i=1; i<=kap; ++i) {
            pl->x_pos=xpos;
            xpos+=2;
            pl->y_pos=ypos;
            pl=pl->nextplace;
        }
        if (j==n+1) {
            pl->x_pos=xpos;
            pl->y_pos=ypos;
            pl=pl->nextplace;
        }
        ypos+=4;
        xpos=net->x_pos+2;
    }
    CreateTr(net,CreateChaMS_2(act,"??"),net->x_pos,6);
    pl=CreateEdge(1,1,pl,net);
    CreateTr(net,CreateChaMS_2(act,"!!"),-net->x_pos,4);
    pl=CreateEdge(kap+1,kap+1,pl,net);
    /* normale Uebergaenge */
    range_ptr1=(Range *)(Knoten->Operand2);
    wert1=range_ptr1->von-1;
    for (i=1; i<=n; ++i) {
        if (++wert1>range_ptr1->bis)
            wert1=(range_ptr1=range_ptr1->next)->von;
        mset=CreateChaMS(act,'!',wert1);
        for (j=1; j<=kap; ++j) {
            CreateTr(net,mset,-net->x_pos,i*4+3);
            pl=CreateEdge(j,kap*i+j+1,pl,net);
            pl=CreatePostset(j+1,pl,net);
        }
        mset=CreateChaMS(act,'?',wert1);
        for (j=1; j<=kap; ++j) {
            CreateTr(net,mset,-net->x_pos,i*4+5);
            pl=CreateEdge(kap*i+j+1,j,pl,net);
            pl=CreatePreset(j+1,pl,net);
        }
    }
    /* Entry-Transition */
    if (flag&SIMPLE_CD) {
        CreateTr(net,MScreate_empty_set(),0,2);
        pl=CreateEdge(0,1,pl,net);
    } else {
        CreateTr(net,CreateChaMS_2(act,"??"),net->x_pos,6);
        pl=CreateEdge(0,1,pl,net);
        j=2*n-1;
        xpos=(int)(net->x_dim-j)/2+net->x_pos;
        if (j>net->x_dim) {
            net->x_dim=j;
            net->x_pos=xpos;
        }
        range_ptr1=(Range *)(Knoten->Operand2);
        wert1=range_ptr1->von-1;
        for (i=1; i<=n; ++i) {
            if (++wert1>range_ptr1->bis)
                wert1=(range_ptr1=range_ptr1->next)->von;
            CreateTr(net,CreateChaMS(act,'!',wert1),xpos,2);
            xpos+=2;
            pl=CreateEdge(0,kap*i+2,pl,net);
            pl=CreatePostset(2,pl,net);
        }
    }    

    for ( pl=net->Places ; pl ; pl->marks|=1, pl=pl->nextplace );
    for ( tr=net->Transitions ; tr ; tr->flags|=1, tr=tr->nexttrans );

    return net;
}

#endif

#ifdef VER3
 
LPNet StackBox(TTree Knoten)
{
    LPNet      net;
    Place      pl;
    Transition tr;
    Range      *range_ptr1;
    Cardinal   i,j,n,wert1;
    int        xpos,ypos;

    n=0;
    range_ptr1=(Range *)(Knoten->Operand2);
    while (range_ptr1) {
        n+=(range_ptr1->bis)-(range_ptr1->von)+1;
        range_ptr1=range_ptr1->next;
    }
    net=OnlyPlaces(kap*(n+2));
    net->NumTransitions=4+2*n*kap+((flag&SIMPLE_CD)?1:n);

    net->x_dim=-2*(net->x_pos=-2*kap+1)+1;
    i=net->y_dim=11+4*n;
    net->LastPlace->y_pos=i-1;
    /* Exit-Transition e->x */
    CreateTr(net,MScreate_singleton(MSconj_action(act)),-1,i-3);
    pl=CreateEdge(0,0,NULL,net);
    CreateTr(net,MScreate_singleton(MSconj_action(act)),1,i-3);
    for (i=kap; i; --i) {
        pl=CreatePreset(i,pl,net);
    }
    pl=CreatePostset(0,pl,net);
    pl=net->Places->nextplace;
    ypos=4;
    j=n+3;
    while (--j) {
        xpos=net->x_pos;
        for (i=1; i<=kap; ++i) {
            pl->x_pos=xpos;
            xpos+=4;
            pl->y_pos=ypos;
            pl=pl->nextplace;
        }
        if (j==n+2)
            ypos+=2;
        else if (j==n+1)
            ypos+=3;
        else
            ypos+=4;
    }
    CreateTr(net,CreateChaMS_2(act,"??"),-net->x_pos,4);
    for (i=kap; i; --i) {
        pl=CreateEdge(i,i,pl,net);
    }
    CreateTr(net,CreateChaMS_2(act,"!!"),-net->x_pos,6);
    for (i=2*kap; i>kap; --i) {
        pl=CreateEdge(i,i,pl,net);
    }
    for (i=1; i<=n; ++i) {
        /* normale Uebergaenge */
        CreateTr(net,CreateChaMS(act,'!',wert1),-net->x_pos,i*4+4);
        pl=CreateEdge(1,kap*(i+1)+1,pl,net);
        pl=CreatePostset(kap+1,pl,net);
        CreateTr(net,CreateChaMS(act,'?',wert1),-net->x_pos,i*4+6);
        pl=CreateEdge(kap*(i+2),kap,pl,net);
        pl=CreatePreset(kap*2,pl,net);
        xpos=-net->x_pos;
        for (j=kap-1; j; --j) {
            xpos-=4;
            CreateTr(net,MScreate_empty_set(),xpos,i*4+6);
            pl=CreateEdge((i+1)*kap+j,(i+1)*kap+j+1,pl,net);
            pl=CreateEdge(j+kap,j+kap+1,pl,net);
            pl=CreateEdge(j+1,j,pl,net);
            CreateTr(net,MScreate_empty_set(),xpos,i*4+4);
            pl=CreateEdge((i+1)*kap+j+1,(i+1)*kap+j,pl,net);
            pl=CreateEdge(j+kap+1,j+kap,pl,net);
            pl=CreateEdge(j,j+1,pl,net);
        }
    }
    /* Entry-Transition */
    if (flag&SIMPLE_CD) {
        CreateTr(net,MScreate_empty_set(),0,2);
        pl=CreatePreset(0,pl,net);
        for (i=kap; i; --i)
            pl=CreatePostset(i,pl,net);
    } else {
        j=2*n-1;
        xpos=(int)(net->x_dim-j)/2+net->x_pos;
        if (j>net->x_dim) {
            net->x_dim=j;
            net->x_pos=xpos;
        }
        for (i=1; i<=n; ++i) {
            CreateTr(net,CreateChaMS(act,'!',wert1),xpos,2);
            xpos+=2;
            pl=CreateEdge(0,kap*(i+1)+1,pl,net);
            pl=CreatePostset(kap+1,pl,net);
            for (j=kap; j>1; --j)
                pl=CreatePostset(j,pl,net);
        }
    }

    for ( pl=net->Places ; pl ; pl->marks|=1, pl=pl->nextplace );
    for ( tr=net->Transitions ; tr ; tr->flags|=1, tr=tr->nexttrans );

    return net;
}
 
#endif
 
/*************************/
/* END OF FILE dat_cha.c */
/*************************/
