/***************************************************************************/
/* > lpnet.c <                                                             */
/* Funktionen zur Manipulation von 'labelled petri nets'                   */
/* Burkhard Graves, 08.02.'94                                              */
/***************************************************************************/

#include <string.h>
#include "lpnet.h"

/* um evtl. Kommentare in den Ausgabedateien zu behalten, wird
   yyin benoetigt! */
extern FILE *yyin;

/*****************************************************************************/
/* Erstellt am Anfang der Transitionenliste des Netzes 'net' eine disjunkte  */
/* Kopie der Transition 'tr' mitsamt allen ein- und ausgehenden Kanten;      */
/* 'numname' wird als Kennzeichen einer Kopie auf 0 gesetzt,                 */
/* 'label' und 'bpn_label' werden auf NULL gesetzt.                          */
/* Rueckgabewert ist ein Zeiger auf die neue Transition, ausserdem wird in   */
/* '*stlast' bzw. '*tslast' ein Zeiger auf die letzte Kante des Presets bzw. */
/* des Postsets abgelegt.                                                    */
/* Aenderung 8.6.1999: register numtr initialisiert         -Christian Stehno*/
/*****************************************************************************/

Transition DuplicateTrans(LPNet net,Transition tr,STEdge *stlast,TSEdge *tslast)
  {
    register Transition newtr=NULL;

    if (net && tr)
      {
        ++net->NumTransitions;
        (newtr=LPNallocTrans())->nexttrans=net->Transitions;
        if (net->Transitions)
          net->Transitions->prevtrans=newtr;
        else
          net->LastTransition=newtr;
        (net->Transitions=newtr)->prevtrans=NULL;
        newtr->numname=0;
        newtr->label=newtr->bpn_label=NULL;
        /* Wenn nicht, ist so newtr->time_label NULL! */
        if ((newtr->time_label=tr->time_label))
          {
            newtr->time_label = (Time_stamp *) malloc(sizeof(Time_stamp));
            newtr->time_label->low=tr->time_label->low;
            newtr->time_label->high=tr->time_label->high;
          }
        newtr->type=tr->type;
        newtr->x_pos=tr->x_pos;
        newtr->y_pos=tr->y_pos;
        newtr->preset=CopyTransPreset(tr,newtr,stlast);
        newtr->postset=CopyTransPostset(tr,newtr,tslast);
      }
    else
      PEPerror("DuplicateTrans: No net or no transition");
    return newtr;
  }

/*****************************************************************************/
/* Alle in die Transition 'tr' eingehenden Kanten werden in der Kantenmatrix */
/* direkt vor die Stelle dupliziert, von der sie ausgehen. Als Transitions-  */
/* zeiger wird diesen Kanten (ein Zeiger auf die neu in die Transitionenliste*/
/* eingefuegte Transition) 'newtr' zugewiesen.                               */
/* Rueckgabewert ist ein Zeiger auf die erste Kante, '*stlast' wird ein      */
/* Zeiger auf die letzte Kante zugewiesen.                                   */
/*****************************************************************************/

STEdge CopyTransPreset(Transition tr,Transition newtr,STEdge *stlast)
{
    register STEdge st,newst;
    STEdge          stfirst;

    stfirst=*stlast=NULL;
    if (tr && newtr && ((st=tr->preset)!=NULL)) {
        (stfirst=newst=LPNallocSTEdge())->prevplace=NULL;
        while (newst) {
            newst->prevtrans=NULL;
            newst->weight=st->weight;
            newst->trans=newtr;
            newst->place=st->place;
            newst->nexttrans=st->place->postset;
            st->place->postset=newst->nexttrans->prevtrans=newst;
            newst=((st=st->nextplace)!=NULL)?
                  ((newst->nextplace=LPNallocSTEdge())
                   ->prevplace=newst)->nextplace
                  : ((*stlast=newst)->nextplace=NULL);
        }
    }
    return stfirst;
}

/*****************************************************************************/
/* Alle von der Transition 'tr' ausgehenden Kanten werden in der Kantenmatrix*/
/* direkt vor die Stelle dupliziert, in der sie enden. Als Transitionszeiger */
/* wird diesen Kanten (ein Zeiger auf die neu in die Transitionenliste ein-  */
/* gefuegte Transition) 'newtr' zugewiesen.                                  */
/* Rueckgabewert ist ein Zeiger auf die erste Kante, '*stlast' wird ein      */
/* Zeiger auf die letzte Kante zugewiesen.                                   */
/*****************************************************************************/

TSEdge CopyTransPostset(Transition tr,Transition newtr,TSEdge *tslast)
{
    register TSEdge ts,newts;
    TSEdge          tsfirst;

    tsfirst=*tslast=NULL;
    if (tr && newtr && ((ts=tr->postset)!=NULL)) {
        (tsfirst=newts=LPNallocTSEdge())->prevplace=NULL;
        while (newts) {
            newts->prevtrans=NULL;
            newts->weight=ts->weight;
            newts->trans=newtr;
            newts->place=ts->place;
            newts->nexttrans=ts->place->preset;
            ts->place->preset=newts->nexttrans->prevtrans=newts;
            newts=((ts=ts->nextplace)!=NULL)?
                  ((newts->nextplace=LPNallocTSEdge())
                   ->prevplace=newts)->nextplace
                  : ((*tslast=newts)->nextplace=NULL);
        }
    }
    return tsfirst;
}

/*****************************************************************************/
/* Erstellt am Anfang der Stellenliste des Netzes 'net' eine disjunkte Kopie */
/* der Stelle 'pl' mitsamt allen ein- und ausgehenden Kanten.                */
/* 'numname' wird als Kennzeichen einer Kopie auf 0 gesetzt.                 */
/* Rueckgabewert ist ein Zeiger auf die neue Stelle, ausserdem wird in       */
/* '*stlast' bzw. '*tslast' ein Zeiger auf die letzte Kante des Postsets     */
/* bzw. des Presets abgelegt.                                                */
/* Aenderung 8.6.1999: register numpl initialisiert         -Christian Stehno*/
/*****************************************************************************/

Place DuplicatePlace(LPNet net,Place pl,STEdge *stlast,TSEdge *tslast)
{
    register Place newpl=NULL;

    if (net && pl) {
        ++net->NumPlaces;
        (newpl=LPNallocPlace())->nextplace=net->Places;
	net->Places->prevplace=newpl;
        (net->Places=newpl)->prevplace=NULL;
        newpl->name=(pl->name)?(char *)strdup(pl->name):NULL;
        newpl->numname=0;
        newpl->label=pl->label;
        newpl->component=pl->component;
        newpl->tokenage=pl->tokenage;
        newpl->x_pos=pl->x_pos;
        newpl->y_pos=pl->y_pos;
        newpl->preset=CopyPlacePreset(pl,newpl,tslast);
        newpl->postset=CopyPlacePostset(pl,newpl,stlast);
    } else
        PEPerror("DuplicatePlace: No net or no place");
    return newpl;
}

/*****************************************************************************/
/* Alle in die Stelle 'pl' eingehenden Kanten werden in der Kantenmatrix     */
/* direkt vor die Transition dupliziert, von der sie ausgehen. Als Stellen-  */
/* zeiger wird diesen Kanten (ein Zeiger auf die neu in die Stellenliste     */
/* eingefuegte Stelle) 'newpl' zugewiesen.                                   */
/* Rueckgabewert ist ein Zeiger auf die erste Kante, '*tslast' wird ein      */
/* Zeiger auf die letzte Kante zugewiesen.                                   */
/*****************************************************************************/

TSEdge CopyPlacePreset(Place pl,Place newpl,TSEdge *tslast)
{
    register TSEdge ts,newts;
    TSEdge          tsfirst;

    tsfirst=*tslast=NULL;
    if (pl && newpl && ((ts=pl->preset)!=NULL)) {
        (tsfirst=newts=LPNallocTSEdge())->prevtrans=NULL;
        while (newts) {
            newts->prevplace=NULL;
            newts->weight=ts->weight;
            newts->place=newpl;
            newts->trans=ts->trans;
            newts->nextplace=ts->trans->postset;
            ts->trans->postset=newts->nextplace->prevplace=newts;
            newts=((ts=ts->nexttrans)!=NULL)?
                  ((newts->nexttrans=LPNallocTSEdge())
                   ->prevtrans=newts)->nexttrans
                  : ((*tslast=newts)->nexttrans=NULL);
        }
    }
    return tsfirst;
}

/*****************************************************************************/
/* Alle von der Stelle 'pl' ausgehenden Kanten werden in der Kantenmatrix    */
/* direkt vor die Transition dupliziert, in der sie enden. Als Stellenzeiger */
/* wird diesen Kanten (ein Zeiger auf die neu in die Stellenliste eingefuegte*/
/* Stelle) 'newpl' zugewiesen.                                               */
/* Rueckgabewert ist ein Zeiger auf die erste Kante, '*stlast' wird ein      */
/* Zeiger auf die letzte Kante zugewiesen.                                   */
/*****************************************************************************/

STEdge CopyPlacePostset(Place pl,Place newpl,STEdge *stlast)
{
    register STEdge st,newst;
    STEdge          stfirst;

    stfirst=*stlast=NULL;
    if (pl && newpl && ((st=pl->postset)!=NULL)) {
        (stfirst=newst=LPNallocSTEdge())->prevtrans=NULL;
        while (newst) {
            newst->prevplace=NULL;
            newst->weight=st->weight;
            newst->place=newpl;
            newst->trans=st->trans;
            newst->nextplace=st->trans->preset;
            st->trans->preset=newst->nextplace->prevplace=newst;
            newst=((st=st->nexttrans)!=NULL)?
                  ((newst->nexttrans=LPNallocSTEdge())
                   ->prevtrans=newst)->nexttrans
                  : ((*stlast=newst)->nexttrans=NULL);
        }
    }
    return stfirst;
}

/*****************************************************************************/
/* Entfernen der Stelle 'pl' aus dem Netz 'net' zusammen mit allen Kanten,   */
/* die in/aus diese/r Stelle fuehren.                                        */
/* Rueckgabewert ist 'pl->prevplace'.                                        */
/*****************************************************************************/

Place LPNRemovePlace(LPNet net,Place pl)
{
    register STEdge st,stdel;
    register TSEdge ts,tsdel;

    if (net && pl) {
        ts=pl->preset;
        while (ts) {
            if (ts->prevplace)
                ts->prevplace->nextplace=ts->nextplace;
            else
                ts->trans->postset=ts->nextplace;
            if (ts->nextplace)
                ts->nextplace->prevplace=ts->prevplace;
            ts=(tsdel=ts)->nexttrans;
            LPNremoveTSEdge(tsdel);
        }
        st=pl->postset;
        while (st) {
            if (st->prevplace)
                st->prevplace->nextplace=st->nextplace;
            else
                st->trans->preset=st->nextplace;
            if (st->nextplace)
                st->nextplace->prevplace=st->prevplace;
            st=(stdel=st)->nexttrans;
            LPNremoveSTEdge(stdel);
        }
        if (pl->prevplace)
            pl->prevplace->nextplace=pl->nextplace;
        else
            net->Places=pl->nextplace;
        if (pl->nextplace)
            pl->nextplace->prevplace=pl->prevplace;
        else
            net->LastPlace=pl->prevplace;
        if (pl->name)
            free(pl->name);
        net->NumPlaces--;
        net=(LPNet)pl->prevplace; /* kleiner Kunstgriff */
        LPNremovePlace(pl);
    }
    return (Place)net;            /* dito               */
}

/*****************************************************************************/
/* Entfernen der Transition 'tr' aus dem Netz 'net' zusammen mit allen       */
/* Kanten, die in/aus diese/r Transition fuehren.                            */
/* Rueckgabewert ist 'tr->prevtrans'.                                        */
/*****************************************************************************/

Transition LPNRemoveTransition(LPNet net,Transition tr)
{
    register STEdge st,stdel;
    register TSEdge ts,tsdel;

    if (net && tr) {
        ts=tr->postset;
        while (ts) {
            if (ts->prevtrans)
                ts->prevtrans->nexttrans=ts->nexttrans;
            else
                ts->place->preset=ts->nexttrans;
            if (ts->nexttrans)
                ts->nexttrans->prevtrans=ts->prevtrans;
            ts=(tsdel=ts)->nextplace;
            LPNremoveTSEdge(tsdel);
        }
        st=tr->preset;
        while (st) {
            if (st->prevtrans)
                st->prevtrans->nexttrans=st->nexttrans;
            else
                st->place->postset=st->nexttrans;
            if (st->nexttrans)
                st->nexttrans->prevtrans=st->prevtrans;
            st=(stdel=st)->nextplace;
            LPNremoveSTEdge(stdel);
        }
        if (tr->prevtrans)
            tr->prevtrans->nexttrans=tr->nexttrans;
        else
            net->Transitions=tr->nexttrans;
        if (tr->nexttrans)
            tr->nexttrans->prevtrans=tr->prevtrans;
        else
            net->LastTransition=tr->prevtrans;
        MSremove(tr->label);
        if (tr->bpn_label)
            MSremove(tr->bpn_label);
        if (tr->time_label)
          free(tr->time_label);
        net->NumTransitions--;
        net=(LPNet)tr->prevtrans; /* kleiner Kunstgriff */
        LPNremoveTrans(tr);
    }
    return (Transition)net;       /* dito               */
}

/*****************************************************************************/
/* Erzeugt eine Kopie des uebergebenen Netzes 'net'.                         */
/* Rueckgabewert ist ein Zeiger auf diese Kopie.                             */
/* In dem Algorithmus werden voruebergehend die Stellen-Vorgaenger-Zeiger im */
/* Original-Netz so verbogen, dass sie auf den Preset bzw. Postset der       */
/* korrespondierenden Stellen in der Kopie zeigen. Dies ist eine Hilfe beim  */
/* Einrichten des Postsets bzw. des Presets der neuen Transitionen.          */
/*****************************************************************************/

LPNet LPNCreateNetCopy(LPNet net)
{
    register TSEdge ts,ts2;
    register STEdge st,st2;
    Place           pl,pl2;
    Transition      tr,tr2;
    LPNet           net2;

    pl=net->Places;
    tr=net->Transitions;
    (net2=LPNallocHead())->NumComponents=net->NumComponents;
    net2->x_pos=net->x_pos;
    net2->y_pos=net->y_pos;
    net2->x_dim=net->x_dim;
    net2->y_dim=net->y_dim;
    if ((net2->NumPlaces=net->NumPlaces)!=0) {
        (pl2=net2->Places=LPNallocPlace())->prevplace=NULL;
        while (pl2) {
            pl2->x_pos=pl->x_pos;
            pl2->y_pos=pl->y_pos;
            pl2->component=pl->component;
            pl2->label=pl->label;
            pl2->name=(pl->name)?(char *)strdup(pl->name):NULL;
            pl2->numname=pl->numname;
            if ((ts=pl->preset)!=NULL) {       /* Preset der Stelle kopieren */
                (ts2=pl2->preset=LPNallocTSEdge())->prevtrans=NULL;
                while (ts2) {
                    ts2->place=pl2;
                    ts2->weight=ts->weight;
                    ts2=((ts=ts->nexttrans)!=NULL)?
                          ((ts2->nexttrans=LPNallocTSEdge())
                           ->prevtrans=ts2)->nexttrans
                        : (ts2->nexttrans=NULL);
                }
            } else {
                pl2->preset=NULL;
            }
            if ((st=pl->postset)!=NULL) {     /* PostSet der Stelle kopieren */
                (st2=pl2->postset=LPNallocSTEdge())->prevtrans=NULL;
                while (st2) {
                    st2->place=pl2;
                    st2->weight=st->weight;
                    st2=((st=st->nexttrans)!=NULL)?
                          ((st2->nexttrans=LPNallocSTEdge())
                           ->prevtrans=st2)->nexttrans
                        : (st2->nexttrans=NULL);
                }
            } else {
                pl2->postset=NULL;
            }
            pl->prevplace=(Place)pl2->postset;
            pl2=((pl=pl->nextplace)!=NULL)?
                  ((pl2->nextplace=LPNallocPlace())->prevplace=pl2)
                  ->nextplace
                : ((net2->LastPlace=pl2)->nextplace=NULL);
        }
    } else {
        net2->Places=net2->LastPlace=NULL;
    }
    if ((net2->NumTransitions=net->NumTransitions)!=0) {
        (tr2=net2->Transitions=LPNallocTrans())->prevtrans=NULL;
        while (tr2) {
            tr2->x_pos=tr->x_pos;
            tr2->y_pos=tr->y_pos;
            tr2->type=tr->type;
            tr2->numname=tr->numname;
            tr2->label=MScreate_copy(tr->label);
            tr2->bpn_label=(tr->bpn_label)?MScreate_copy(tr->bpn_label):NULL;
            if ((tr2->time_label=tr->time_label))
              {
                tr2->time_label = (Time_stamp *) malloc(sizeof(Time_stamp));
                tr2->time_label->low=tr->time_label->low;
                tr2->time_label->high=tr->time_label->high;
              }
            if ((st=tr->preset)!=NULL) { /* Preset der Transition einrichten */
                (st2=tr2->preset=(STEdge)st->place->prevplace)
                ->prevplace=NULL;
                while (st2) {
                    st2->trans=tr2;
                    st->place->prevplace=(Place)st2->nexttrans;
                    st2=((st=st->nextplace)!=NULL)?
                          ((st2->nextplace=(STEdge)st->place->prevplace)
                           ->prevplace=st2)->nextplace
                        : (st2->nextplace=NULL);
                }
            }
            tr2=((tr=tr->nexttrans)!=NULL)?
                  ((tr2->nexttrans=LPNallocTrans())->prevtrans=tr2)
                  ->nexttrans
                : ((net2->LastTransition=tr2)->nexttrans=NULL);
        }
    } else {
        net2->Transitions=net2->LastTransition=NULL;
    }
    if ((pl=net->Places)!=NULL) {
        if ((tr=net->Transitions)!=NULL) {
            pl2=net2->Places;
            while (pl) {
                pl->prevplace=(Place)pl2->preset;
                pl=pl->nextplace;
                pl2=pl2->nextplace;
            }
            tr2=net2->Transitions;
            while (tr2) {
                if ((ts=tr->postset)!=NULL) { /* Postset der Transition einr.*/
                    (ts2=tr2->postset=(TSEdge)ts->place->prevplace)
                    ->prevplace=NULL;
                    while (ts2) {
                        ts2->trans=tr2;
                        ts->place->prevplace=(Place)ts2->nexttrans;
                        ts2=((ts=ts->nextplace)!=NULL)?
                              ((ts2->nextplace=(TSEdge)ts->place->prevplace)
                               ->prevplace=ts2)->nextplace
                            : (ts2->nextplace=NULL);
                    }
                }
                tr=tr->nexttrans;
                tr2=tr2->nexttrans;
            }
        }
        (pl=net->Places)->prevplace=NULL; /* Missbrauchte Stellen-Vorgaenger-*/
        while (pl->nextplace) {           /* Zeiger restaurieren !           */
            pl=(pl->nextplace->prevplace=pl)->nextplace;
        }
    }
    return net2;
}

/*****************************************************************************/
/* Entfernt das Netz '*net' (schnell) aus dem Speicher.                      */
/*****************************************************************************/

void LPNRemove_net(LPNet *net)
  {
    register STEdge STPtr,STRem;
    register TSEdge TSPtr,TSRem;
    register Place  PlPtr,PlRem;
    Transition      TrPtr,TrRem;

    if (*net)
      {
        PlPtr=(*net)->Places;
        if ((TrPtr=(*net)->Transitions)!=NULL)
          {   /* sind Transitionen da ? */
            while (PlPtr)
              {
                TSPtr=PlPtr->preset;
                while (TSPtr)
                  {
                    TSPtr=(TSRem=TSPtr)->nexttrans;
                    LPNremoveTSEdge(TSRem);
                  }
                STPtr=PlPtr->postset;
                while (STPtr)
                  {
                    STPtr=(STRem=STPtr)->nexttrans;
                    LPNremoveSTEdge(STRem);
                  }
                PlPtr=(PlRem=PlPtr)->nextplace;
                if (PlRem->name)
                  free(PlRem->name);
                LPNremovePlace(PlRem);
              }
            while (TrPtr)
              {
                TrPtr=(TrRem=TrPtr)->nexttrans;
                MSremove(TrRem->label);
//                if (TrRem->bpn_label)
//                  MSremove(TrRem->bpn_label);
                if (TrRem->time_label)
                  free(TrRem->time_label);
                LPNremoveTrans(TrRem);
              }
          }
        else
          {   /* sind wenigstens Stellen da ? */
            while (PlPtr)
              {
                PlPtr=(PlRem=PlPtr)->nextplace;
                if (PlRem->name)
                  free(PlRem->name);
                LPNremovePlace(PlRem);
              }
          }
        LPNremoveHead(*net);
        *net=NULL;
      }
  }

/*****************************************************************************/
/* Veraenderung von 'net1', so dass 'net1' eine disjunkte Vereinigung der    */
/* Netze 'net1' und 'net2' darstellt. Achtung: Das Kopfelement von 'net2'    */
/* wird geloescht.                                                           */
/* Rueckgabewert ist 'net1'.                                                 */
/*****************************************************************************/

LPNet DisjointNetUnion(LPNet net1,LPNet net2)
{
    if (net1->Places!=NULL) {
        if ((net1->LastPlace->nextplace=net2->Places)!=NULL) {
            net2->Places->prevplace=net1->LastPlace;
            net1->LastPlace=net2->LastPlace;
        }
    } else {
        net1->Places=net2->Places;
        net1->LastPlace=net2->LastPlace;
    }
    if (net1->Transitions!=NULL) {
        if ((net1->LastTransition->nexttrans=net2->Transitions)!=NULL) {
            net2->Transitions->prevtrans=net1->LastTransition;
            net1->LastTransition=net2->LastTransition;
        }
    } else {
        net1->Transitions=net2->Transitions;
        net1->LastTransition=net2->LastTransition;
    }
    net1->NumPlaces+=net2->NumPlaces;
    net1->NumTransitions+=net2->NumTransitions;
    LPNremoveHead(net2);
    return net1;
}

/*****************************************************************************/
/* Numeriert die Stellen und Transitionen des Netzes 'net' durch.            */
/*****************************************************************************/

void numname_all(LPNet net)
{
    Place       pl;
    Transition  tr;
    Cardinal32  i;

    i=0; pl=net->Places;
    while (pl) { pl->numname=++i; pl=pl->nextplace; }
    i=0; tr=net->Transitions;
    while (tr) { tr->numname=++i; tr=tr->nexttrans; }
}

/*****************************************************************************/
/* Prueft das Netz auf logische Fehler und gibt eine Fehlernummer            */
/* (hoffentlich 0 = kein Fehler) zurueck.                                    */
/*****************************************************************************/

int CheckNet(LPNet net)
{
    Cardinal32  events,conds,inzi;
    TSEdge      ts,ts2;
    STEdge      st,st2;
    Place       pl;
    Transition  tr;

    numname_all(net);

    conds=0;
    if ((pl=net->Places)!=NULL) {
        if (pl->prevplace)
            return -1;
        while (pl) {
            ++conds;
            if (pl->nextplace && pl->nextplace->prevplace!=pl)
                return -2;
            ts=pl->preset; st=pl->postset;
            if (ts && ts->prevtrans)
                return -3;
            inzi=0;
            while (ts) {
                if (ts->place!=pl)
                    return -4;
                if (ts->nexttrans && ts->nexttrans->prevtrans!=ts)
                    return -5;
                tr=ts->trans;
                if (tr->numname<inzi)
                    return -6;
                else
                    inzi=tr->numname;
                ts2=tr->postset;
                while (ts2 && ts2!=ts)
                    ts2=ts2->nextplace;
                if (!ts2)
                    return -7;
                ts=ts->nexttrans;
            }
            if (st && st->prevtrans)
                return -8;
            inzi=0;
            while (st) {
                if (st->place!=pl)
                    return -9;
                if (st->nexttrans && st->nexttrans->prevtrans!=st)
                    return -10;
                tr=st->trans;
                if (tr->numname<inzi)
                    return -11;
                else
                    inzi=tr->numname;
                st2=tr->preset;
                while (st2 && st2!=st)
                    st2=st2->nextplace;
                if (!st2)
                    return -12;
                st=st->nexttrans;
            }
            if (pl->nextplace==NULL && net->LastPlace!=pl)
                return -13;
            pl=pl->nextplace;
        }
    } else {
        if (net->LastPlace) {
            return -14;
        }
    }
    if (net->NumPlaces!=conds)
        return -15;
    events=0;
    if ((tr=net->Transitions)!=NULL) {
        if (tr->prevtrans)
            return -21;
        while (tr) {
            ++events;
            if (tr->nexttrans && tr->nexttrans->prevtrans!=tr)
                return -22;
            st=tr->preset; ts=tr->postset;
            if (st && st->prevplace)
                return -23;
            inzi=0;
            while (st) {
                if (st->trans!=tr)
                    return -24;
                if (st->nextplace && st->nextplace->prevplace!=st)
                    return -25;
                pl=st->place;
                if (pl->numname<inzi)
                    return -26;
                else
                    inzi=pl->numname;
                st2=pl->postset;
                while (st2 && st2!=st)
                    st2=st2->nexttrans;
                if (!st2)
                    return -27;
                st=st->nextplace;
            }
            if (ts && ts->prevplace)
                return -28;
            inzi=0;
            while (ts) {
                if (ts->trans!=tr)
                    return -29;
                if (ts->nextplace && ts->nextplace->prevplace!=ts)
                    return -30;
                pl=ts->place;
                if (pl->numname<inzi)
                    return -31;
                else
                    inzi=pl->numname;
                ts2=pl->preset;
                while (ts2 && ts2!=ts)
                    ts2=ts2->nexttrans;
                if (!ts2)
                    return -32;
                ts=ts->nextplace;
            }
            if (tr->nexttrans==NULL && net->LastTransition!=tr)
                return -33;
            tr=tr->nexttrans;
        }
    } else {
        if (net->LastTransition) {
            return -34;
        }
    }
    if (net->NumTransitions!=events)
        return -35;
    return 0;
}

/*****************************************************************************/
/* Ausgabe des Netzes in einem Format, dass einen schnellen Ueberblick ueber */
/* das Netz ermoeglicht (Gewichte werden jedoch nicht mit ausgegeben).       */
/*****************************************************************************/

void PrintNet(LPNet net,FILE *ofp)
{
    STEdge      st;
    TSEdge      ts;
    Place       pl;
    Transition  tr;
    char        c;

    if (!net)
        return;
    rewind(yyin);
    while ((c=getc(yyin))!=EOF)
        if (c=='%') {
            while((c=getc(yyin))!='\n')
                if (c==EOF) {
                    ungetc(c,yyin);
                    break;
                }
        } else
            putc(c,ofp);
    numname_all(net);
    fprintf(ofp,"\nDas Netz besteht aus %u Stelle(n), ",net->NumPlaces);
    fprintf(ofp,"%u Transition(en).\n",net->NumTransitions);
    fprintf(ofp,"\nTransitionen:\n");
    fprintf(ofp,"| Name | Preset > Postset | Label\n");
    fprintf(ofp,"----------------------------------\n");
    tr=net->Transitions;
    while (tr) {
        fprintf(ofp,"|%5u | ",tr->numname);
        if ((st=tr->preset)!=NULL) {
            fprintf(ofp,"%u",st->place->numname);
            while ((st=st->nextplace)!=NULL)
                fprintf(ofp,",%u",st->place->numname);
        }
        fprintf(ofp," > ");
        if ((ts=tr->postset)!=NULL) {
            fprintf(ofp,"%u",ts->place->numname);
            while ((ts=ts->nextplace)!=NULL)
                fprintf(ofp,",%u",ts->place->numname);
        }
        fprintf(ofp," | ");
        MSWriteMultiset(ofp,tr->label);
        if (tr->bpn_label) {
            fprintf(ofp," > ");
            MSWriteMultiset(ofp,tr->bpn_label);
        }
        tr=tr->nexttrans;
    }
    fprintf(ofp,"\nStellen:\n");
    fprintf(ofp,"| Komponente | Name | Label | Preset > Postset\n");
    fprintf(ofp,"-----------------------------------------------\n");
    pl=net->Places;
    while(pl) {
        fprintf(ofp,"|%11u |",pl->component);
        switch(pl->label) {
        case EMPTY:
            c='0'; break;
        case ENTRY:
            c='e'; break;
        case EXIT:
            c='x'; break;
        case DELETE:
            c='d'; break;
        }
        fprintf(ofp,"%5u |  (%c)  | ",pl->numname,c);
        if ((ts=pl->preset)!=NULL) {
            fprintf(ofp,"%u",ts->trans->numname);
            while ((ts=ts->nexttrans)!=NULL)
                fprintf(ofp,",%u",ts->trans->numname);
        }
        fprintf(ofp," > ");
        if ((st=pl->postset)!=NULL) {
            fprintf(ofp,"%u",st->trans->numname);
            while ((st=st->nexttrans)!=NULL)
                fprintf(ofp,",%u",st->trans->numname);
        }
        if (pl->name)
            fprintf(ofp," | %s",pl->name);
        fprintf(ofp,"\n");
        pl=pl->nextplace;
    }
}

/*****************************************************************************/
/* Ausgabe des Netzes im .inf-Dateiformat.                                   */
/*****************************************************************************/

void PrintNet_inf(LPNet net,FILE *ofp)
{
    int        width;
    Place      pl;
    Transition tr;
    STEdge     st;
    TSEdge     ts;
    char       buffer[16];

    if (!net)
        return;
    numname_all(net);
    sprintf(buffer,"%u",net->NumPlaces);
    width=(int)strlen(buffer);
    fprintf(ofp,"%u %u\nInzidenzmatrix:\n%*s",
            net->NumPlaces,net->NumTransitions,width+3,"");
    tr=net->Transitions;
    while (tr) {
        fprintf(ofp,"T%-8u",tr->numname);
        tr=tr->nexttrans;
    }
    fprintf(ofp,"\n");
    pl=net->Places;
    while (pl) {
        tr=net->Transitions;
        st=pl->postset;
        ts=pl->preset;
        fprintf(ofp,"S%-*u",width,pl->numname);
        fprintf(ofp," |");
        while (tr) {
            if (ts && (ts->trans==tr)) {
                fprintf(ofp,"(%3d,",-ts->weight);
                ts=ts->nexttrans;
            } else {
                fprintf(ofp,"(  0,");
            }
            if (st && (st->trans==tr)) {
                fprintf(ofp,"%2d) ",st->weight);
                st=st->nexttrans;
            } else {
                fprintf(ofp," 0) ");
            }
            tr=tr->nexttrans;
        }
        fprintf(ofp,"\n");
        pl=pl->nextplace;
    }
    fprintf(ofp,"\nStartmarkierung:\n(");
    pl=net->Places;
    while (pl) {
        fprintf(ofp,"%c",(pl->label==ENTRY)?'1':'0');
        if ((pl=pl->nextplace)!=NULL)
            fprintf(ofp,", ");
    }
    fprintf(ofp,")\n");
}

/*****************************************************************************/
/* Veraendert die Stellenliste so, dass alle Stellen, die man von            */
/* 'net->LastPlace' bzw. dem Rueckgabewert (einer Stelle) aus rueckwaerts    */
/* besucht, die Beschriftung 'label1' bzw. 'label2' aufweisen.               */
/*****************************************************************************/

Place PreparePlaces(LPNet net,PLabel label1,PLabel label2)
{
    register Place pl,pl1,pl2;
             Place merke;

    pl=net->LastPlace;
    net->LastPlace=merke=pl1=pl2=NULL;
    while (pl) {
        if (pl->label==label1) {
            if (pl1) {
                pl1=pl1->prevplace=pl;
            } else {
                pl1=net->LastPlace=pl;
            }
        } else if (pl->label==label2) {
            if (pl2) {
                pl2=pl2->prevplace=pl;
            } else {
                pl2=merke=pl;
            }
        }
        pl=pl->prevplace;
    }
    if (pl1)
        pl1->prevplace=NULL;
    if (pl2)
        pl2->prevplace=NULL;
    return merke;
}

/*****************************************************************************/
/* Stellt die normale Stellenliste wieder her und loescht dabei Stellen mit  */
/* der Beschriftung 'label'.                                                 */
/* Rueckgabewert ist das uebergebene Netz.                                   */
/*****************************************************************************/

LPNet RestoreKillPlaces(LPNet net,PLabel label)
{
    register Place pl;

    pl=net->Places;
    while (pl->nextplace) {
        pl=(pl->nextplace->prevplace=pl)->nextplace;
        if (pl->prevplace->label & label)
            LPNRemovePlace(net,pl->prevplace);
    }
    if ((net->LastPlace=pl)->label & label)
        LPNRemovePlace(net,pl);
    return net;
}

/*****************************************************************************/
/* Entfernt Stellen- und Transitionsduplikate aus dem Netz 'net'.            */
/* In c_places bzw. c_transitions wird die Anzahl der Entfernungen notiert.  */
/*****************************************************************************/

void RemoveDuplicates(LPNet net,Cardinal32 *c_places,Cardinal32 *c_transitions)
{
    register Transition tr1;
    register Place      pl1;
    register TSEdge     ts;
    register STEdge     st;
             Transition tr2;
             Place      pl2;
             TSEdge     ts1,ts2;
             STEdge     st1,st2;

    *c_transitions=*c_places=0;
    tr1=net->Transitions;
    while (tr1) {
        st=tr1->preset->place->postset;
        while (st) {
            tr2=st->trans;
            st=st->nexttrans;
            if (tr2!=tr1 && MSis_equal(tr1->label,tr2->label)) {
                /* Test, ob postset uebereinstimmt */
                ts1=tr1->postset; ts2=tr2->postset;
                while (ts1 && ts2 && ts1->place==ts2->place
                                  && ts1->weight==ts2->weight) {
                    ts1=ts1->nextplace;
                    ts2=ts2->nextplace;
                }
                if (!ts1 && !ts2) {
                    /* Test, ob preset uebereinstimmt */
                    st1=tr1->preset; st2=tr2->preset;
                    while (st1 && st2 && st1->place==st2->place
                                      && st1->weight==st2->weight) {
                        st1=st1->nextplace;
                        st2=st2->nextplace;
                    }
                    if (!st1 && !st2) {
                        LPNRemoveTransition(net,tr2);
                        ++*c_transitions;
                    }
                }
            }
        }
        tr1=tr1->nexttrans;
    }
    pl1=net->LastPlace;
    while (pl1) {
        if ((st=pl1->postset)==NULL) {
            if((ts=pl1->preset)==NULL) {
                /* isolierte Stelle */
                pl2=pl1->prevplace;
                while (pl2) {
                    if (!pl2->preset && !pl2->postset
                                     && pl1->label==pl2->label) {
                        pl2=LPNRemovePlace(net,pl2);
                        ++*c_places;
                    } else {
                        pl2=pl2->prevplace;
                    }
                }
                pl1=pl1->prevplace;
                continue;
            }
            /* Stelle ohne Nachbereich, aber mit Vorbereich */
            ts=ts->trans->postset;
            while (ts) {
                pl2=ts->place;
                ts=ts->nextplace;
                if (pl2!=pl1 && pl1->label==pl2->label
                             && pl2->postset==NULL) {
                    /* Test, ob preset uebereinstimmt */
                    ts1=pl1->preset; ts2=pl2->preset;
                    while (ts1 && ts2 && ts1->trans==ts2->trans
                                      && ts1->weight==ts2->weight) {
                        ts1=ts1->nexttrans;
                        ts2=ts2->nexttrans;
                    }
                    if (!ts1 && !ts2) {
                        LPNRemovePlace(net,pl2);
                        ++*c_places;
                    }
                }
            }
            pl1=pl1->prevplace;
            continue;
        }
        /* Stelle mit Nachbereich */
        st=st->trans->preset;
        while (st) {
            pl2=st->place;
            st=st->nextplace;
            if (pl2!=pl1 && pl1->label==pl2->label) {
                /* Test, ob preset uebereinstimmt */
                ts1=pl1->preset; ts2=pl2->preset;
                while (ts1 && ts2 && ts1->trans==ts2->trans
                                  && ts1->weight==ts2->weight) {
                    ts1=ts1->nexttrans;
                    ts2=ts2->nexttrans;
                }
                if (!ts1 && !ts2) {
                    /* Test, ob postset uebereinstimmt */
                    st1=pl1->postset; st2=pl2->postset;
                    while (st1 && st2 && st1->trans==st2->trans
                                      && st1->weight==st2->weight) {
                        st1=st1->nexttrans;
                        st2=st2->nexttrans;
                    }
                    if (!st1 && !st2) {
                        LPNRemovePlace(net,pl2);
                        ++*c_places;
                    }
                }
            }
        }
        pl1=pl1->prevplace;
    }
}

/***********************/
/* END OF FILE lpnet.c */
/***********************/
