/*****************************/
/* lpnet.h                   */
/* Burkhard Graves, 26.6.'93 */
/*****************************/

#ifndef __LPNET_H__
#define __LPNET_H__

#include "memlib.h"
#include "boxtypes.h"
#include "multiset.h"

/*****************************/
/* Beschriftung einer Stelle */
/*****************************/
typedef char PLabel;

#define EMPTY     0
#define ENTRY     1
#define EXIT      2
#define INTERRUPT 3
#define DELETE    4
#define CHOICEPL  8

#define NORMAL 0
#define TIC    1
#define TIC2   2  /* Fuer getrenntes Schalten */

/* Speichert Koordinaten, die gecastet in inscript liegen */
typedef struct coords_
  {
    int            x,
                   y;
    struct coords_ * next;
  } CoordEntry,
    * Coords;

/**********/
/* Stelle */
/**********/

typedef struct PlElem_ {
    struct PlElem_ *prevplace, /* Zur (doppelten) Verkettung in der      */
                   *nextplace; /*   Stellenliste.                        */
    struct TSElem_ *preset;    /* Zeiger auf erste Kante des Vorbereichs */
    struct STElem_ *postset;   /*   bzw. des Nachbereichs.               */
    char           *name,      /* Bedeutung der Stelle ( -> Databox ).   */
                   *inscript;  /* Speichert Koordinaten und weitere Infos*/
    PLabel          label;     /* Beschriftung der Stelle.               */
    Cardinal32      numname,   /* Numerischer Name fuer div. Zwecke.     */
                    component, /* Zeigt Komponentenzugehoerigkeit an.    */
                    marks,     /* Anzahl der Marken auf dieser Stelle.   */
                    tokenage;  /* Zeit des Token in dieser Komponente    */
  /* nicht vorhanden in neunetze.h */
  /* Ecke oben links */
    int             x_pos,
                    y_pos;
} PlElem,*Place;

/**************/
/* Transition */
/**************/

typedef struct TrElem_ {
  struct TrElem_ *prevtrans, /* Zur (doppelten) Verkettung in der      */
                 *nexttrans; /*   Transitionenliste.                   */
  struct STElem_ *preset;    /* Zeiger auf erste Kante des Vorbereichs */
  struct TSElem_ *postset;   /*   bzw. des Nachbereichs.               */
  char           *inscript;  /* Soll die Koordinaten und andere Infos  */
                             /* tragen                                 */
  Cardinal32      numname;   /* Numerischer Name fuer div. Zwecke.     */
  MultiSet        label,     /* Beschriftung der Transition.           */
    /* nicht vorhanden in neunetze.h */
                  bpn_label; /* Backup der Beschriftung.               */
  Time_stamp     *time_label;/* Zeitbeschriftung der Transition        */
  int             type;      /* Diffenrenziert zw. Tic- und Schalttransitionen */
  /* nicht vorhanden in neunetze.h */
  /* Ecke oben links */
  int             x_pos,
                  y_pos,
                  flags;
} TrElem, *Transition;

/*****************************/
/* Kante Stelle-->Transition */
/*****************************/

typedef struct STElem_ {
    struct STElem_ *prevplace, /* Zur (doppelten, vertikalen) Verkettung   */
                   *nextplace, /*   in Bezug auf die Stellenliste.         */
                   *prevtrans, /* Zur (doppelten, horizontalen) Verkettung */
                   *nexttrans; /*   in Bezug auf die Transitionenliste.    */
    Place           place;     /* Zeiger auf den Ursprung                  */
    Transition      trans;     /*   bzw. auf das Ziel der Kante.           */
    Cardinal32      weight;    /* Gewicht der Kante.                       */
} STElem,*STEdge;

/*****************************/
/* Kante Transition-->Stelle */
/*****************************/

typedef struct TSElem_ {
    struct TSElem_ *prevplace, /* Zur (doppelten, vertikalen) Verkettung   */
                   *nextplace, /*   in Bezug auf die Stellenliste.         */
                   *prevtrans, /* Zur (doppelten, horizontalen) Verkettung */
                   *nexttrans; /*   in Bezug auf die Transitionenliste.    */
    Place           place;     /* Zeiger auf das Ziel                      */
    Transition      trans;     /*   bzw. auf den Ursprung der Kante.       */
    Cardinal32      weight;    /* Gewicht der Kante.                       */
} TSElem,*TSEdge;

/*********************/
/* Kopf eines Netzes */
/*********************/

typedef struct {
    Place       Places,         /* Zeiger auf erste bzw. auf letzte Stelle */
                LastPlace;      /*   in der Stellenliste.                  */
    Transition  Transitions,    /* Zeiger auf erste bzw. auf letzte Trans. */
                LastTransition; /*   in der Transitionenliste.             */
    Cardinal32  NumPlaces,      /* Anzahl der Stellen,                     */
                NumTransitions, /*        der Transitionen,                */
                NumComponents;  /*        der Komponenten.                 */
  /* nicht vorhanden in neunetze.h */
  /* Gesamtausdehnung (=Ecke unten rechts) */
    Cardinal    x_dim,
                y_dim;
  /* Ecke oben links */
    int         x_pos,
                y_pos;
} LPNetHead,*LPNet;

typedef LPNet BOX;

/*********************************************************************/
/* Funktionen zur Allokation und zur Freigabe von Speicherplatz fuer */
/* einzelne Elemente eines Netzes; realisiert als Makros.            */
/*********************************************************************/

#define LPNallocHead()   ( (LPNet) MYmalloc( sizeof( LPNetHead ) ) )
#define LPNallocPlace()  ( (Place) MYmalloc( sizeof( PlElem ) ) )
#define LPNallocTrans()  ( (Transition) MYmalloc( sizeof( TrElem ) ) )
#define LPNallocSTEdge() ( (STEdge) MYmalloc( sizeof( STElem ) ) )
#define LPNallocTSEdge() ( (TSEdge) MYmalloc( sizeof( TSElem ) ) )

#define LPNremoveHead( _Head )     free( _Head )
#define LPNremovePlace( _Place )   free( _Place )
#define LPNremoveTrans( _Trans )   free( _Trans )
#define LPNremoveSTEdge( _STEdge ) free( _STEdge )
#define LPNremoveTSEdge( _TSEdge ) free( _TSEdge )

/***********************/
/* Funktionsprototypen */
/***********************/

Transition      DuplicateTrans(LPNet net,Transition tr,
                               STEdge *stlast,TSEdge *tslast        );
    STEdge     CopyTransPreset(Transition tr,Transition newtr,
                               STEdge *stlast                       );
    TSEdge    CopyTransPostset(Transition tr,Transition newtr,
                               TSEdge *tslast                       );
     Place      DuplicatePlace(LPNet net,Place pl,STEdge *stlast,
                               TSEdge *tslast                       );
    TSEdge     CopyPlacePreset(Place pl,Place newpl,TSEdge *tslast  );
    STEdge    CopyPlacePostset(Place pl,Place newpl,STEdge *stlast  );
     Place      LPNRemovePlace(LPNet net,Place pl                   );
Transition LPNRemoveTransition(LPNet net,Transition tr              );
     LPNet    LPNCreateNetCopy(LPNet net                            );
      void       LPNRemove_net(LPNet *net                           );
     LPNet    DisjointNetUnion(LPNet net1, LPNet net2               );
      void         numname_all(LPNet net                            );
       int            CheckNet(LPNet net                            );
      void            PrintNet(LPNet net,FILE *ofp                  );
      void        PrintNet_inf(LPNet net,FILE *ofp                  );
     Place       PreparePlaces(LPNet net,PLabel label1,PLabel label2);
     LPNet   RestoreKillPlaces(LPNet net,PLabel label               );
      void    RemoveDuplicates(LPNet net,Cardinal32 *c_places,
                                     Cardinal32 *c_transitions      );

#endif

/***********************/
/* END OF FILE lpnet.h */
/***********************/
