/*****************************************************************/
/* main.c                                                        */
/* Burkhard Graves, 03.02.94                                     */
/* Aenderung 8.6.1999: radius auf 10 gesetzt -> uebersichtlicher */
/*                                              -Christian Stehno*/
/*****************************************************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include "main.h"
#include "boxtypes.h"
#include "box.h"
#include "clockbox.h"
#include "box_sub.h"
#include "moby_out.h"
#include "move.h"
#include "parser.h"

TTree      Syntaxbaum;
int flag;
/* Ausgabegroesse bestimmen, Minimum 10 */
int radius=10;
char *own_file,*inf_file,*mob_file;

extern int yyparse(void);
extern FILE *yyin;

/************************/
/* Aufruf demonstrieren */
/************************/

static void usage(char *myname)
{
    fprintf(stderr,"\nUsage: %s source [-o=fname] [-i=fname] ",myname);
    fprintf(stderr," [-m=fname] [-n=fname] [-RSbcepqrsvwt]\n\n");
    fprintf(stderr," o: specify OWN-file\n");
    fprintf(stderr," i: specify INF-file\n");
    fprintf(stderr," m: specify MOB-file\n");
    fprintf(stderr," n: specify output file (new PEP file format)\n");
    fprintf(stderr,"R#: specify depth of recursion\n");
    fprintf(stderr,"S#: specify depth of syncronisation\n");
    fprintf(stderr," b: bundle atomic actions\n");
    fprintf(stderr," c: check net\n");
    fprintf(stderr," e: extended labelling\n");
    fprintf(stderr," p: big loop\n");
    fprintf(stderr," q: medium loop\n");
    fprintf(stderr," r: remove duplicates\n");
    fprintf(stderr," s: simple C-&D-Boxes\n");
    fprintf(stderr," v: generate invisible arcs\n");
    fprintf(stderr," w: PEP file: wide output format\n");
    fprintf(stderr," t: output time labels\n");
    fprintf(stderr," u: use clocknets for calculation\n");
    fprintf(stderr,"\nWritten by Burkhard Graves, reworked\n");
    fprintf(stderr,"by Markus Wittwer and Christian Stehno.\n");
    fprintf(stderr,"Time added by Christian Stehno; made %s\n\n",__DATE__);
    exit(1);
}

/******************/
/* Zahl einlesen */
/******************/
 
static Cardinal read_card(char *ptr,char *myname)
{
    Cardinal zahl;
 
    if (!isdigit(*++ptr))
        usage(myname);
    zahl=0;
    do {
        zahl*=10;
        zahl+=*ptr-'0';
    } while (isdigit(*++ptr));
    return zahl;
}

/******************/
/* Namen einlesen */
/******************/
 
static char *read_name(char *ptr1,char *myname)
{
    if (*++ptr1!='=' || !*++ptr1)
        usage(myname);
    return ptr1;
}
 
/*****************/
/* Flag einlesen */
/*****************/
 
static void read_flags(char *ptr,char *myname)
  {
    while (*++ptr)
      {
        if (isdigit(*ptr))
            radius=(*ptr-'0')*3+7;
        else
          {
            switch(*ptr)
              {
            case 'S':   SyncDepth=read_card(ptr,myname);
                        return;
            case 'R':   RecDepth=read_card(ptr,myname);
                        return;
            case 'o':   flag|=OUT_OWN;
                        own_file=read_name(ptr,myname);
                        return;
            case 'i':   flag|=OUT_INF;
                        inf_file=read_name(ptr,myname);
                        return;
            case 'm':   flag|=OUT_MOB;
                        mob_file=read_name(ptr,myname);
                        return;
            case 'n':   flag|=OUT_NEW;
                        mob_file=read_name(ptr,myname);
                        return;
            case 'c':   flag|=CHECK_NET; break;
            case 'r':   flag|=REMOVE_DUPL; break;
            case 'b':   flag|=BUNDLE; break;
            case 's':   flag|=SIMPLE_CD; break;
            case 'e':   flag|=EXT_LABEL; break;
            case 'p':   flag|=BIG_LOOP; break;
            case 'q':   flag|=MEDIUM_LOOP; break;
            case 'v':   flag|=INVISIBLE_ARCS; break;
            case 'w':   flag|=WIDE_FORMAT; break;
            case 't':   flag|=WITH_TIME; break;
            case 'u':   flag|=CLOCKNET; break;
            default:    usage(myname);
              }
          } /* else */
      } /* while ptr */
  }

/*******************************/
/* Netz ASCII-maessig ausgeben */
/*******************************/

void ausgabe(char *file,LPNet net,void (*fkt)(LPNet,FILE *))
{
    FILE *yyout;

    if ((yyout=fopen(file,"w"))==NULL) {
        fprintf(stderr,"Error: File %s could not be opened !\n",
                file);
        exit(3);
    } else {
        printf("Writing in file %s ...\n",file);
        fkt(net,yyout);
        fclose(yyout);
    }
}

/*********************/
/* Das Hauptprogramm */
/*********************/

int main(int argc,char *argv[])
{
    BOX        net;
    clock_t    time;
    int        i;
    Cardinal32 c_transitions,c_places;
    char       *source;

    RecDepth=SyncDepth=10;
    source=NULL;
    flag=0;
    for (i=1; i<argc; ++i)
        if (*(argv[i])!='-')
            if (!source)
                source=argv[i];
            else
                usage(argv[0]);
        else
            read_flags(argv[i],argv[0]);

    if (!source)
        usage(argv[0]);
    if ((yyin=fopen(source,"r"))==NULL) {
        fprintf(stderr,"Error: File %s could not be opened!\n",
                source);
        exit(2);
    }

    printf("Reading from file %s ...\n",source);

    InitMultiSets();
    InitAllSymbolTables();
    if (flag&CLOCKNET)
      CLOCKBOXInitOpArray();
    else
      BOXInitOpArray();

    if (yyparse())
        exit(4);

    printf("Start of evaluation\n");
    time=clock();
    net=Eval(Syntaxbaum);
    printf("End of Evaluation (%f sec.).\n\n",(float)(clock()-time)/CLOCKS_PER_SEC);

    if (flag&CHECK_NET) {
        printf("Checking consistency of the net: ");
        if ((i=CheckNet(net))!=0) {
            fprintf(stderr,"Error: Net inconsistent (%i) !\n",i);
            exit(5);
        } else
            printf("Everything OK !\n\n");
    }

    if (flag&REMOVE_DUPL) {
        printf("Removing duplicates: ");
        RemoveDuplicates(net,&c_places,&c_transitions);
        printf("%u place(s), %u transition(s)\n\n",c_places,c_transitions);
    }

    if (flag&OUT_INF)
        ausgabe(inf_file,net,PrintNet_inf);
    if (flag&OUT_OWN)
        ausgabe(own_file,net,PrintNet);
    if (flag&OUT_MOB) {
        move_hv(net,1-net->x_pos,1-net->y_pos);
        ausgabe(mob_file,net,MOBY_Net);
    }
    if (flag&OUT_NEW) {
        move_hv(net,2-net->x_pos,2-net->y_pos);
        ausgabe(mob_file,net,PEP_Net);
    }

#if 0
    PrintSymbolTable(SymbolTable);
    PrintSymbolTable(VarTable);
#endif

    RemoveTTree(&Syntaxbaum);
    RemoveSymbolTable(&SymbolTable);
    RemoveSymbolTable(&VarTable);
    RemoveFunctionTable();
    LPNRemove_net(&net);
    fclose(yyin);
    return 0;
}

/**********************/
/* END OF FILE main.c */
/**********************/
