/******************************/
/* move.c                     */
/* Burkhard Graves, 20.12.93  */
/******************************/

#include "move.h"
#include "boxtypes.h"

/**********************************************/
/* Netz 'net2' neben Netz 'net1' legen, dabei */
/* EXIT-Stellen auf gleiche Hoehe bringen.    */
/* Ebenso die Schalttransitionen              */
/**********************************************/

void juxtapose(LPNet net1,LPNet net2)
{
    register Place pl;
    register int pos;

    /* Da viele Transitionen dazwischen erzeugt werden habe ich einen
       Abstand von drei Einheiten gewaehlt */
    move_hv(net2,net1->x_pos-net2->x_pos+net1->x_dim+3,
                 net1->y_pos-net2->y_pos);
    net1->x_dim+=net2->x_dim+3;
    if (net2->y_dim>net1->y_dim) {
        pos=(net1->y_dim=net2->y_dim)+net1->y_pos-1;
        for (pl=net1->Places; pl; pl=pl->nextplace)
	{
	  if (pl->label&EXIT)
	    pl->y_pos=pos;
	}
    } else if (net1->y_dim>net2->y_dim) {
        pos=(net2->y_dim=net1->y_dim)+net2->y_pos-1;
        for (pl=net2->Places; pl; pl=pl->nextplace)
	{
	  if (pl->label&EXIT)
	    pl->y_pos=pos;
        }
    }
}

/********************************************/
/* Netz horizontal und vertikal verschieben */
/********************************************/

void move_hv(LPNet net,int x_offset,int y_offset)
{
    register Transition tr;
    register Place pl;

    net->x_pos+=x_offset;
    net->y_pos+=y_offset;
    for (tr=net->Transitions; tr; tr=tr->nexttrans)
    {
        tr->x_pos+=x_offset;
        tr->y_pos+=y_offset;
    }
    for (pl=net->Places; pl; pl=pl->nextplace)
    {
        pl->x_pos+=x_offset;
        pl->y_pos+=y_offset;
    }
}

/*************************************/
/* Netz um 90 Grad nach links drehen */
/*************************************/

void turn_left(LPNet net)
{
    register Transition tr;
    register Place pl;
    register int i;
    register Cardinal c;

    i=net->x_pos; net->x_pos=net->y_pos; net->y_pos=-i-net->x_dim+1;
    c=net->x_dim; net->x_dim=net->y_dim; net->y_dim=c;
    for (tr=net->Transitions; tr; tr=tr->nexttrans)
    {
        i=tr->x_pos; tr->x_pos=tr->y_pos; tr->y_pos=-i;
    }
    for (pl=net->Places; pl; pl=pl->nextplace)
    {
        i=pl->x_pos; pl->x_pos=pl->y_pos; pl->y_pos=-i;
    }
    return;
}

/**************************************/
/* Netz um 90 Grad nach rechts drehen */
/**************************************/

void turn_right(LPNet net)
{
    register Transition tr;
    register Place pl;
    register int i;
    register Cardinal c;

    i=net->y_pos; net->y_pos=net->x_pos; net->x_pos=-i-net->y_dim+1;
    c=net->x_dim; net->x_dim=net->y_dim; net->y_dim=c;
    for (tr=net->Transitions; tr; tr=tr->nexttrans)
    {
        i=tr->y_pos; tr->y_pos=tr->x_pos; tr->x_pos=-i;
    }
    for (pl=net->Places; pl; pl=pl->nextplace)
    {
        i=pl->y_pos; pl->y_pos=pl->x_pos; pl->x_pos=-i;
    }
    return;
}

/****************************************************************************/
/* Verschiebt die Stellen und Transitionen, deren y-Position oberhalb von   */
/* 'y1' bzw. unterhalb von 'y2' liegt, gleichmaessig nach oben bzw. unten,  */
/* so dass der Abstand zwischen 'y1' und 'y2' um 'offset' vergroessert wird.*/
/****************************************************************************/

void up_and_down(LPNet net,int y1,int y2,int offset)
{
    register Transition tr;
    register Place pl;
    register int up,down;

    if (offset>0) {
        net->y_dim+=offset;
        down=offset/2;
        up=offset-down;
        net->y_pos-=up;

        for (tr=net->Transitions; tr; tr=tr->nexttrans)
	{
            if (tr->y_pos<y1)
                tr->y_pos-=up;
            else if (tr->y_pos>y2)
                tr->y_pos+=down;
        }
        for (pl=net->Places; pl; pl=pl->nextplace)
	{
            if (pl->y_pos<y1)
                pl->y_pos-=up;
            else if (pl->y_pos>y2)
                pl->y_pos+=down;
        }
    }
    return;
}

/****************************************************************************/
/* Verschiebt die Stellen und Transitionen, deren x-Position links von      */
/* 'x1' bzw. rechts von 'x2' liegt, gleichmaessig nach links bzw. rechts,   */
/* so dass der Abstand zwischen 'x1' und 'x2' um 'offset' vergroessert wird.*/
/****************************************************************************/

void left_and_right(LPNet net,int x1,int x2,int offset)
{
    register Transition tr;
    register Place pl;
    register int left,right;

    if (offset>0) {
        net->x_dim+=offset;
        right=offset/2;
        left=offset-right;
        net->x_pos-=left;

        for (tr=net->Transitions; tr; tr=tr->nexttrans)
	{
            if (tr->x_pos<x1)
                tr->x_pos-=left;
            else if (tr->x_pos>x2)
                tr->x_pos+=right;
        }
        for (pl=net->Places; pl; pl=pl->nextplace)
	{
            if (pl->x_pos<x1)
                pl->x_pos-=left;
            else if (pl->x_pos>x2)
                pl->x_pos+=right;
        }
    }
    return;
}

/****************************************************************************/
/* Kombination von 'up_and_down' und 'left_and_right', wobei der erstere    */
/* Vorgang Prioritaet hat.                                                  */
/****************************************************************************/

void explode(LPNet net,int y1,int y2,
                        int x1,int x2,int y_offset,int x_offset)
{
    register Transition tr;
    register Place pl;
    register int up,down;
    register int left,right;

    if (y_offset>0 || x_offset>0) {
        if (y_offset<0)
            y_offset=0;
        if (x_offset<0)
            x_offset=0;
        net->y_dim+=y_offset;
        net->x_dim+=x_offset;
        down=y_offset/2;
        up=y_offset-down;
        right=x_offset/2;
        left=x_offset-right;
        net->y_pos-=up;
        net->x_pos-=left;

        for (tr=net->Transitions; tr; tr=tr->nexttrans)
	{
            if (tr->y_pos<y1)
                tr->y_pos-=up;
            else if (tr->y_pos>y2)
                tr->y_pos+=down;
            else if (tr->x_pos<x1)
                tr->x_pos-=left;
            else if (tr->x_pos>x2)
                tr->x_pos+=right;
        }
        for (pl=net->Places; pl; pl=pl->nextplace)
	{
            if (pl->y_pos<y1)
                pl->y_pos-=up;
            else if (pl->y_pos>y2)
                pl->y_pos+=down;
            else if (pl->x_pos<x1)
                pl->x_pos-=left;
            else if (pl->x_pos>x2)
                pl->x_pos+=right;
        }
    }
    return;
}

/**********************/
/* END OF FILE move.c */
/**********************/
