/*****************************************************************************/
/*  multiset.c                                                               */
/*  Mi.,04.11.1992 - Mi.,04.11.1992                           Stefan Roemer  */
/*****************************************************************************/


#include <stdio.h>
#include <string.h>
#include "multiset.h"
#include "parser.h"

/*****************************************************************************/
/* Globale Variablen fuer die Behandlung von Multimengen.                    */
/*****************************************************************************/

Cardinal MaxActions   = ACTION_BUF_SIZE;

Cardinal MaxVars      = ACTION_BUF_SIZE;

Cardinal MaxFunctions = ACTION_BUF_SIZE;

Cardinal ActCounter  = 0;

Cardinal VarCounter  = 0;

Cardinal FuncCounter = 0;

MultiSet mmDummy    = NULL;       /* Hilfsmenge bei der Berechnung von sum,  */
                                  /* union,...                               */

MultiSet mmLeaf     = NULL;       /* Hilfsmenge bei der Berechnung der       */
                                  /* Blaetter des Parsetrees.                */

MultiSet mmSingle   = NULL;       /* Hilfsmenge fuer Multimengen mit maximal */
                                  /* einem Eintrag ( Aufbau Parsetree ).     */

bool  MSdisplay_style = MS_SHORT;


/*****************************************************************************/
/* Initialisieren der globalen Variablen des Multimengen-Funktionspakets.    */
/*****************************************************************************/

void InitMultiSets()

  {

    ActCounter = 0;
    MaxActions = ACTION_BUF_SIZE;
    if ( mmDummy )
      {
        MSremove( mmDummy );
      } /* if */
    mmDummy  = MSalloc( MaxActions );
    if ( mmLeaf )
      {
        MSremove( mmLeaf );
      } /* if */
    mmLeaf  = MSalloc( MaxActions );
    if ( ! mmSingle )
      {
        mmSingle = MSalloc( 1 );
      } /* if */
    MSdisplay_style = MS_SHORT;

  } /* InitMultiSets() */


/*****************************************************************************/
/* Funktion zum Kopieren einer Multimenge in eine bereits bestehende.        */
/*****************************************************************************/

MultiSet MScopy(MultiSet mm1,MultiSet mm2 )

  {

    register MultiSet mm1Ptr = mm1;

    while ((*(mm1Ptr++)=*(mm2++))!=0L)
      skip;

    return ( mm1 );

  } /* MScopy() */


/*****************************************************************************/
/* Funktion zur Initialisierung einer leeren Multimenge.                     */
/*****************************************************************************/

MultiSet MScreate_empty_set()

  {                                                       /* Gibt {} zurueck */

    MultiSet mm;

    *( mm = MSalloc( 0 ) ) = 0;

    return ( mm );

  } /* MScreate_empty_set() */


/*****************************************************************************/
/* Funktion zur Initialisierung einer neuen Multimenge, die als einziges     */
/* Element die uebergebene Aktion mit dem Code `akt` enthaelt.               */
/*****************************************************************************/

MultiSet MScreate_singleton(Action act)

  {

    register MultiSet mm;

    *( mm = MSalloc( 1 ) ) = act;
    *( mm + 1 ) = 1;
    *( mm + 2 ) = 0;

    return ( mm );

  } /* MScreate_singleton() */


/*****************************************************************************/
/* Funktion zur Initialisierung einer neuen Multimenge als Kopie einer be-   */
/* reits vorhandenen Multimenge; der Speicherbedarf der neuen Menge ent-     */
/* spricht ihrer tatsaechlichen "Laenge".                                    */
/*****************************************************************************/

MultiSet MScreate_copy(MultiSet mm)

  {

    MultiSet          mm2;        /* Zeiger auf die zu erzeugende Kopie.     */
    register MultiSet mmptr = mm, /* Laufzeiger durch die Originalmenge.     */
                      mm2ptr;     /* Laufzeiger durch die Kopie.             */
    register Cardinal len = 0;    /* Laenge der Originalmenge.               */

                                  /* "Laenge" der Multimenge ermitteln und   */
                                  /* in `len` ablegen.                       */
    while ( *mmptr )
      {
        len++;
        mmptr++;
        mmptr++;
      }
                                  /* Speicherplatz fuer die Kopie anfordern. */
    mm2ptr = mm2 = MSalloc( len );
    mmptr = mm;
                                  /* Eintraege einzeln kopieren.             */
    while ((*(mm2ptr++)=*(mmptr++))!=0)
      skip;
                                  /* Zeiger auf die Kopie zurueckliefern.    */
    return ( mm2 );

  } /* MScreate_copy() */


/*****************************************************************************/
/* Funktion zur Berechnung der Maechtigkeit einer Multimenge                 */
/*****************************************************************************/

Cardinal MScardinality(MultiSet mm)

  {

    Cardinal card = 0;            /* Kardinalitaet der Menge.                */

                                  /* Durch die Menge laufen ( in Zweier-     */
                                  /* schritten ) und dabei die Anzahlen der  */
                                  /* Vorkommen der einzelnen Aktionen auf-   */
                                  /* summieren.                              */
    while ( *(mm++) )
      {
        card += *(mm++);
      } /* while */

                                  /* Ermittelte Kardinalitaet zurueckgeben.  */
    return ( card );

  } /* MScardinality() */


/*****************************************************************************/
/* Funktion zur Berechnung der Anzahl verschiedener Aktionen in einer Multi- */
/* menge.                                                                    */
/*****************************************************************************/

Cardinal MSlength(MultiSet mm)

  {

    Cardinal len = 0;             /* "Laenge" der Menge.                     */

                                  /* Durch die Menge laufen ( in Zweier-     */
                                  /* schritten ) und dabei zaehlen, wieviele */
                                  /* Aktionen in der Menge sind.             */
    while ( *(mm++) )
      {
        len++;
        mm++;
      } /* while */

                                  /* Ermittelte Laenge zurueckliefern.       */
    return ( len );

  } /* MSlength() */


/*****************************************************************************/
/* Funktion zur Feststellung, ob die Aktion `akt` oder deren Konjugierte     */
/* `^akt` in der Multimenge `mm` vorkommen.                                  */
/*****************************************************************************/

bool MSis_in(Action akt,MultiSet mm)

  {

                                  /* Parameter auf Gueltigkeit testen und    */
                                  /* Aktionscode "ent-konjugieren".          */
    if ( mm && *mm && ( akt |= 1 ) && --akt )
      {
                                  /* In der Menge diejenige Stelle suchen,   */
                                  /* an der die Aktion `akt` stehen muss.    */
        while ( *mm && (*mm < akt) )
          {
            mm++;
            mm++;
          } /* while */
                                  /* Rueckgabe: True, falls `akt` oder `^akt`*/
                                  /*                  in `mm` enthalten;     */
                                  /*            False sonst.                 */
        return ( (akt|1) == ((*mm)|1) ? True
                                  : *mm && ((akt|1)==((*(mm+2))|1)) ? True
                                                                    : False );
      } /* if */

                                  /* Bei Parameterfehler 'False' liefern.    */
    return ( False );

  } /* MSis_in() */


/*****************************************************************************/
/* Funktion zur Berechnung, wie oft eine Aktion mit dem Aktionscode `akt` in */
/* der Multimenge `mm` enthalten ist.                                        */
/*****************************************************************************/

Cardinal MScount(Action akt,MultiSet mm)

  {

    if ( akt && mm )
      {
                                  /* In der Menge diejenige Stelle suchen,   */
                                  /* an der die Aktion `akt` stehen muss.    */
        while ( *mm && ( *mm < akt ) )
          {
            mm++;
            mm++;
          } /* while */
                                  /* Rueckgabe: falls die Aktion in der      */
                                  /* Menge enthalten ist, wird die Anzahl    */
                                  /* ihrer Vorkommen geliefert, sonst null.  */
        return ( ( akt == *mm ) ? *(++mm) : 0 );
      } /* if */
                                  /* Falls bei der Parameteruebergabe ein    */
                                  /* Fehler aufgetreten ist, wird null als   */
                                  /* ermittelte Anzahl zurueckgegeben.       */
    return ( 0 );

  } /* MScount() */


/*****************************************************************************/
/* Funktion zur Konjugation einer Multimenge `mm`.                           */
/*****************************************************************************/

MultiSet MSconjugation(MultiSet mm)

  {

    MultiSet mmPtr = mm - 1;      /* Hilfszeiger zum Durchlaufen der Menge.  */

                                  /* Durch die Menge laufen und dabei die    */
                                  /* einzelnen Aktionen konjugieren.         */
    while ( *++mmPtr )
      {
                                  /* Falls eine Aktion konjugiert und unkon- */
                                  /* jugiert in der Menge vorkommt,          */
        if ( ( *mmPtr ^= 1 ) == *(mmPtr + 2) )
          {
                                  /* muessen die Anzahlen der jeweiligen Vor-*/
                                  /* kommen ausgetauscht werden, so dass     */
                                  /* durch die Konjugation nicht die Ordnung */
                                  /* der Menge zerstoert wird.               */
            *mmPtr = *(mmPtr + 1);
            *(mmPtr + 1) = *(mmPtr + 3);
            *(mmPtr + 3) = *mmPtr;
            *mmPtr = *(mmPtr + 2) - 1;
            mmPtr += 3;
          } /* if */
        else
          {
                                  /* Ansonsten: Zeiger vorruecken.           */
            mmPtr++;
          } /* else */
      }

                                  /* Die Menge zurueckgeben.                 */
    return ( mm );

  } /* MSconjugation() */


/*****************************************************************************/
/* Funktion zur Feststellung, ob zwei Multimengen `mm1` und `mm2` gleich     */
/* sind.                                                                     */
/*****************************************************************************/

bool MSis_equal(MultiSet mm1,MultiSet mm2)

  {

                                  /* Die Mengen aktionsweise durchforsten:   */
    while ( *mm1 )
      {
                                  /* Bei der ersten Nicht-Uebereinstimmung   */
        if ( *(mm1++) != *(mm2++) )
          {
                                  /* wird die weitere Suche abgebrochen und  */
                                  /* 'False' als Ergebnis geliefert.         */
            return ( False );
          }
      } /* while */

                                  /* An dieser Stelle wurde die erste Menge  */
                                  /* bis zum Ende abgearbeitet; das Ergebnis */
                                  /* des Vergleichs ist 'True', wenn jetzt   */
                                  /* auch die zweite Menge 'leer' ist, sonst */
                                  /* 'False'.                                */
    return ( ! *mm2 );

  } /* MSis_equal() */


/*****************************************************************************/
/* Funktion zur Vereinigung zweier Multimengen.                              */
/*****************************************************************************/

MultiSet MSunion(MultiSet mm1,MultiSet mm2)

  {

    register MultiSet mm1Ptr = mm1,
                      mm2Ptr = mm2,
                      mmDum  = mmDummy;

                                  /* Solange nicht beide Mengen abgearbeitet */
                                  /* sind:                                   */
    while ( *mm1Ptr || *mm2Ptr )
      {
                                  /* Falls die aktuell betrachteten Aktionen */
                                  /* aus Menge 1 und 2 uebereinstimmen,      */
        if ( *mm1Ptr && (*mm1Ptr == *mm2Ptr) )
          {
                                  /* wird die Aktion einmal in die Hilfsmenge*/
                                  /* uebernommen und die Anzahl der Vorkommen*/
            *(mmDum++) = *(mm1Ptr++); mm2Ptr++;
                                  /* dort auf das Maximum der Vorkommen in   */
                                  /* den beiden Mengen gesetzt.              */
            *(mmDum++) = Max( *mm1Ptr, *mm2Ptr );
            mm1Ptr++, mm2Ptr++;
          } /* if */
                                  /* Falls der Aktionscode aus Menge 1 klei- */
                                  /* ner ist als der der Aktion aus Menge 2, */
                                  /* wird diejenige aus Menge 1 in die Hilfs-*/
                                  /* menge uebernommen.                      */
        else if ( ( ! *mm2Ptr ) || ( *mm1Ptr && ( *mm1Ptr < *mm2Ptr ) ) )
          {
            *(mmDum++) = *(mm1Ptr++);
            *(mmDum++) = *(mm1Ptr++);
          } /* else if */
                                  /* Ansonsten                               */
        else /* ( ( ! *mm1Ptr ) || ( *mm1Ptr > *mm2Ptr ) ) */
          {
                                  /* wird die Aktion aus Menge 2 uebernommen.*/
            *(mmDum++) = *(mm2Ptr++);
            *(mmDum++) = *(mm2Ptr++);
          } /* else */
      } /* while */

                                  /* Hilfsmenge abschliessen.                */
    *mmDum = 0;
                                  /* Hilfsmenge nach `mm1` kopieren. Der     */
                                  /* alte Inhalt von `mm1` wird ueberschrie- */
                                  /* ben!                                    */
    mmDum = mmDummy;
    mm1Ptr = mm1;
    while ((*(mm1Ptr++)=*(mmDum++))!=0L)
      skip;

                                  /* Ergebnismenge zurueckliefern.           */
    return ( mm1 );

  } /* MSunion() */


/*****************************************************************************/
/* Funktion zur Durchschnittbildung zweier Multimengen.                      */
/*****************************************************************************/

MultiSet MSintersection(MultiSet mm1,MultiSet mm2)

  {

    register MultiSet mm1rPtr = mm1, /* Lesezeiger    */
                      mm1wPtr = mm1, /* Schreibzeiger */
                      mm2Ptr  = mm2;

    while ( *mm1rPtr )
      {
        if ( *mm1rPtr == *mm2Ptr )
          {
            *(mm1wPtr++) = *(mm1rPtr++);
            mm2Ptr++;
            *(mm1wPtr++) = Min( *mm1rPtr, *mm2Ptr );
            mm1rPtr++;
            mm2Ptr++;
          } /* if */
        else if ( *mm2Ptr && ( *mm1rPtr > *mm2Ptr ) )
          {
            mm2Ptr++;
            mm2Ptr++;
          } /* else if */
        else /* ( ( ! *mm2Ptr ) || ( *mm1rPtr < *mm2Ptr ) ) */
          {
            mm1rPtr++;
            mm1rPtr++;
          } /* else */
      } /* while */

    *mm1wPtr = 0;

    return ( mm1 );

  } /* MSintersection() */


/*****************************************************************************/
/* Funktion zur Summenbildung zweier Multimengen.                            */
/*****************************************************************************/

MultiSet MSsum(MultiSet mm1,MultiSet mm2)

  {

    register MultiSet mm1Ptr = mm1,
                      mm2Ptr = mm2,
                      mmDum  = mmDummy;

    while ( *mm1Ptr || *mm2Ptr )
      {
        if ( *mm1Ptr && (*mm1Ptr == *mm2Ptr) )
          {
            *(mmDum++) = *(mm1Ptr++); mm2Ptr++;
            if ( ( *(mmDum++) = *mm1Ptr + *mm2Ptr ) < *mm1Ptr )
              {
                PEPerror( "Overflow: maximal number of single action exceeded!" );
              } /* if */
            mm1Ptr++, mm2Ptr++;
          } /* if */
        else if ( ( ! *mm2Ptr ) || ( *mm1Ptr && ( *mm1Ptr < *mm2Ptr ) ) )
          {
            *(mmDum++) = *(mm1Ptr++);
            *(mmDum++) = *(mm1Ptr++);
          } /* else if */
        else /* ( ( ! *mm1Ptr ) || ( *mm1Ptr > *mm2Ptr ) ) */
          {
            *(mmDum++) = *(mm2Ptr++);
            *(mmDum++) = *(mm2Ptr++);
          } /* else */
      } /* while */

    *mmDum = 0;
    mmDum = mmDummy;
    mm1Ptr = mm1;
    while ((*(mm1Ptr++)=*(mmDum++))!=0L)
      skip;

    return ( mm1 );

  } /* MSsum() */


/*****************************************************************************/
/* Funktion zur Differenzbildung zweier Multimengen.                         */
/*****************************************************************************/

MultiSet MSdifference(MultiSet mm1,MultiSet mm2)

  {

    register MultiSet mm1rPtr = mm1, /* Lesezeiger    */
                      mm1wPtr = mm1, /* Schreibzeiger */
                      mm2Ptr  = mm2;

    while ( *mm1rPtr )
      {
        if ( *mm1rPtr == *mm2Ptr )
          {
            *(mm1wPtr++) = *(mm1rPtr++);
            mm2Ptr++;
            *(mm1wPtr) = ( *mm1rPtr > *mm2Ptr ) ? *mm1rPtr - *mm2Ptr : 0;
            mm1rPtr++;
            mm2Ptr++;
            if ( *mm1wPtr ) mm1wPtr++; else mm1wPtr--;
          } /* if */
        else if ( *mm2Ptr && ( *mm1rPtr > *mm2Ptr ) )
          {
            mm2Ptr++;
            mm2Ptr++;
          } /* else if */
        else /* ( ( ! *mm2Ptr ) || ( *mm1rPtr < *mm2Ptr ) ) */
          {
            *(mm1wPtr++) = *(mm1rPtr++);
            *(mm1wPtr++) = *(mm1rPtr++);
          } /* else */
      } /* while */

    *mm1wPtr = 0;

    return ( mm1 );

  } /* MSdifference() */


/*****************************************************************************/
/* Funktion zum Entfernen von `num` Paaren `{ akt, ^akt }` aus der Multi-    */
/* menge `mm`. Hilfsfunktion fuer den Synchronisations-Operator.             */
/*****************************************************************************/

MultiSet MSremove_action_pairs(MultiSet mm,Action akt,Cardinal num)

  {

                                  /* Parameter auf Gueltigkeit testen.       */
    if ( mm && akt && num )
      {
                                  /* Hilfsmenge `{ num akt, num ^akt }`      */
                                  /* in `mmDummy` aufbauen und               */
        *mmDummy = ( mmDummy[2] = akt | 1 ) - 1;
        mmDummy[1] = mmDummy[3] = num;
        mmDummy[4] = 0;
                                  /* von der Menge `mm` abziehen.            */
        MSdifference( mm, mmDummy );
      } /* if */

                                  /* Ergebnismenge zurueckliefern.           */
    return ( mm );

  } /* MSremove_action_pairs() */


/*****************************************************************************/
/* Funktion zum Umbennen von Aktionen einer Multimenge bzw. Variablen mit    */
/* Funktionen.                                                               */
/*****************************************************************************/

MultiSet MSrelabel(MultiSet mm,Function f)

  {

    register MultiSet mmLeafPtr,
                      mmPtr;
    register Function fPtr;
    register Cardinal mmLen = 0;

                                  /* Parameter kurz ueberpruefen.            */
    if ( mm && *mm && f && *f )
      {
                                  /* Aktion behandeln.                       */
        if ( *mm > 1 )
          {
            fPtr = f;
            mmLeafPtr = mmLeaf;
            mmLen = 0;
                                  /* Die uebergebene Menge in eine Hilfs-    */
                                  /* menge kopieren.                         */
            MScopy( mmLeaf, mm );
                                  /* Die uebergebene Menge soll das Ergebnis */
                                  /* der Operation aufnehmen. Dazu wird sie  */
                                  /* zunaechst 'leer gemacht'.               */
            *mm = 0;
                                  /* Die Aktionen der Hilfsmenge durchgehen: */
            while ( *mmLeafPtr )
              {
                                  /* Laengenzaehler inkrementieren. Die Laen-*/
                                  /* ge der Menge wird spaeter fuer das      */
                                  /* Zurueckkopieren benoetigt.              */
                mmLen++;
                                  /* In der Funktionsdefinition diejenige    */
                                  /* Stelle suchen, an der die aktuelle      */
                                  /* Aktion fuer eine Umbenennung stehen     */
                                  /* muesste. ( Beachte: in den Funktions-   */
                                  /* definitionen kommen im Definitionsbe-   */
                                  /* reich stets nur UNKONJUGIERTE Aktionen  */
                                  /* vor! )                                  */
                while ( *fPtr && ( (Cardinal)( *fPtr | 1 ) < *mmLeafPtr ) )
                  {
                    fPtr++;
                    fPtr++;
                  } /* while */
                                  /* Falls die Aktion unkonjugiert ist und   */
                                  /* umbenannt werden soll,                  */
                if ( *fPtr == *mmLeafPtr )
                  {
                                  /* so wird dies gemacht;                   */
                    *mmLeafPtr = *( fPtr + 1 );
                  } /* if */
                                  /* Umbenennungen konjugierter Aktionen     */
                else if ( *fPtr == MSconj_action( *mmLeafPtr ) )
                  {
                                  /* werden hier vorgenommen.                */
                    *mmLeafPtr = MSconj_action( *( fPtr + 1 ) );
                  } /* else if */
                                  /* Ab zur naechsten Aktion der Hilfsmenge. */
                mmLeafPtr++;
                mmLeafPtr++;
              } /* while */
                                  /* Jetzt werden die ( umbenannten ) Aktio- */
                                  /* nen der Hilfsmenge sukzessive in die    */
                                  /* Ergenismenge 'hineinvereinigt'. Dies ge-*/
                                  /* schieht deshalb, weil die Ordnung der   */
                                  /* Multimenge erhalten bleiben soll.       */
            while ( mmLen-- )
              {
                mmLeafPtr--;
                mmLeafPtr--;
                MSunion( mm, mmLeafPtr );
                *mmLeafPtr = 0;
              } /* while */
          } /* if */
                                  /* Variablen "umbenennen":                 */
        else if ( *mm == 1 )
          {
                                  /* `mmPtr` zeigt auf die Funktion, die     */
                                  /* hinter dem Variablencode als verkappte  */
                                  /* Multimenge abgelegt ist ( im folgenden  */
                                  /* mit g(V) bezeichnet. )                  */
            mmPtr = mm + 2;
                                  /* Fuer jede der Abbildungen in g(V)...    */
            while ( *(mmPtr++) )
              {
                                  /* wird ueberprueft, ob ihr Bild in der    */
                                  /* Funktion f im Definitionsbereich vor-   */
                                  /* kommt ...                               */
                fPtr = f;
                while ( *fPtr && ( (Cardinal)( *fPtr | 1 ) < *mmPtr ) )
                  {
                    fPtr++;
                    fPtr++;
                  } /* while */
                                  /* und dann entsprechend f abgebildet.     */
                if ( *fPtr == *mmPtr )
                  {
                    *mmPtr = *( fPtr + 1 );
                  } /* if */
                else if ( *fPtr == MSconj_action( *mmPtr ) )
                  {
                    *mmPtr = MSconj_action( *( fPtr + 1 ) );
                  } /* else if */
                mmPtr++;
              } /* while */
                                  /* Jetzt wird die Komposition f( g(V) )    */
                                  /* zusammengestellt:                       */
            mmPtr = mm;
            mmLeafPtr = mmLeaf;
            *( mmLeafPtr++ ) = *( mmPtr++ );
            *( mmLeafPtr++ ) = *( mmPtr++ );
            fPtr = f;
            mmLen = 1;
                                  /* Solange in g(V) noch unbearbeitete      */
                                  /* Abbildungen existieren:                 */
            while ( *mmPtr )
              {
                                  /* Falls die aktuell betrachtete Abbildung */
                                  /* in f nicht vorkommt,...                 */
                if ( ! *fPtr || ( *mmPtr < *fPtr ) )
                  {
                                  /* wird sim Falle einer nicht-trivialen    */
                                  /* Abbildung ( wie z.B. b->b ) ...         */
                    if ( *mmPtr != *( mmPtr + 1 ) )
                      {
                                  /* in die Ergebnisfunktion uebernommen...  */
                        *( mmLeafPtr++ ) = *( mmPtr++ );
                        *( mmLeafPtr++ ) = *( mmPtr++ );
                                  /* und der Laengenzaehler erhoeht.         */
                        mmLen++;
                      } /* if */
                    else
                      {
                        mmPtr++;
                        mmPtr++;
                      } /* else */
                  } /* if */
                                  /* Falls die Abbildung auch in f vorkommt, */
                else if ( *mmPtr == *fPtr )
                  {
                                  /* so wird die Abbildung aus g(V) ueber-   */
                                  /* nommen ( da der abgebildete Name nicht  */
                                  /* mehr im Ausdruck vorkommen kann ), ...  */
                    fPtr += 2;
                    if ( *mmPtr != *( mmPtr + 1 ) )
                      {
                                  /* falls sie nicht-trivial ist.            */
                        *( mmLeafPtr++ ) = *( mmPtr++ );
                        *( mmLeafPtr++ ) = *( mmPtr++ );
                        mmLen++;
                      } /* if */
                    else
                      {
                        mmPtr++;
                        mmPtr++;
                      } /* else */
                  } /* else if */
                                  /* Andernfalls kommt diese Abbildung nur   */
                                  /* in f vor und wird von dort in die ...   */
                else
                  {
                                  /* Ergebnisfunktion uebernommen.           */
                    *( mmLeafPtr++ ) = *( fPtr++ );
                    *( mmLeafPtr++ ) = *( fPtr++ );
                    mmLen++;
                  } /* else */
              } /* while */
                                  /* g(V) bereits abgearbeitet; ein eventuel-*/
                                  /* ler Rest von f wird an die neue Funkti- */
                                  /* on einfach angehaengt.                  */
            while ( *fPtr )
              {
                *( mmLeafPtr++ ) = *( fPtr++ );
                *( mmLeafPtr++ ) = *( fPtr++ );
                mmLen++;
              } /* while */
                                  /* Variable mit Funktion ordentlich ab-    */
                                  /* schliessen und...                       */
            *mmLeafPtr = 0;
                                  /* in die Ergebnismenge kopieren, die      */
                                  /* laenger oder kuerzer sein kann als `mm` */
                                  /* zu Beginn dieser Funktion.              */
            MScopy( mm = MSresize( mm, mmLen ), mmLeaf );
          } /* else if */
      } /* if */
                                  /* Ergebnismenge zurueckgeben.             */
                                  /* Achtung! Bei Variablen-Umbennungen kann */
                                  /* der Zeiger `mm` wegen `MSresize()` auf  */
                                  /* einen anderen Speicherbereich weisen    */
                                  /* als zu Beginn dieser Funktion, deshalb  */
                                  /* muss mit dem Funktionswert weitergear-  */
                                  /* beitet werden!!!                        */
    return ( mm );

  } /* MSrelabel() */


/*****************************************************************************/
/* Funktion zur Ausgabe einer Multimenge ( Datei oder Bildschirm ).          */
/*****************************************************************************/

void MSWriteMultiset(FILE *fileptr,MultiSet mm)

  {

    Cardinal i;                   /* Zaehlvariable.                          */
    char     dum[ 50 ];           /* String zur Aufnahme des anzuzeigenden   */
                                  /* Namens.                                 */

    if ( *mm == 1 )
      {
                                  /* Keine Multimenge, sondern Variablenname.*/
        fprintf( fileptr, "VARIABLE %s", VariableToStr( *++mm, dum ) );
                                  /* Falls eine Relabelling-Funktion mit     */
                                  /* dieser Boxvariablen verbunden ist, so   */
                                  /* diese ausgegeben:                       */
        if ( *++mm )
          {
            fprintf( fileptr, " (" );
            fprintf( fileptr, " %s->", ActionToStr( *(mm++), dum ) );
            fprintf( fileptr, "%s", ActionToStr( *(mm++), dum ) );
            while ( *mm )
              {
                fprintf( fileptr, ", %s->", ActionToStr( *(mm++), dum ) );
                fprintf( fileptr, "%s", ActionToStr( *(mm++), dum ) );
              } /* while */
            fprintf( fileptr, " )." );
          } /* if */
        fprintf( fileptr, "\n" );
      } /* if */
    else
      {
                                  /* Tatsaechliche Multimenge. Ausgabe.      */
        fputc( '{', fileptr );
        if ( *mm )
          {
                                  /* Menge ist nicht leer.                   */
            if ( MSdisplay_style )
              {
                                  /* Ausgabe in Kurzschreibweise.            */
                if ( *(++mm) > 1 ) fprintf( fileptr, FORMAT_CARD, *mm );
                fprintf( fileptr, "%s", ActionToStr( *( (mm++) - 1 ), dum ) );
                while ( *mm )
                  {
                    fputc( ',', fileptr );
                    if ( *(++mm) > 1 ) fprintf( fileptr, FORMAT_CARD, *mm );
                    fprintf( fileptr, "%s",
                                         ActionToStr( *( (mm++) - 1 ), dum ) );
                  } /* while */
              } /* if */
            else /* WIDE */
              {
                                   /* Ausfuehrliche Ausgabe der Menge.       */
                fprintf( fileptr, "%s", ActionToStr( *mm, dum ) );
                for ( i = 2 ; i <= *(mm+1) ; i++ )
                  {
                    fprintf( fileptr, ",%s", ActionToStr( *mm, dum ) );
                  } /* for */
                mm++; mm++;
                while ( *mm )
                  {
                    for ( i = 1 ; i <= *(mm+1) ; i++ )
                      {
                        fprintf( fileptr, ",%s", ActionToStr( *mm, dum ) );
                      } /* for */
                    mm++; mm++;
                  } /* while */
              } /* else */
          } /* if */
        fprintf( fileptr, "}\n" );
      } /* else */

  } /* MSWriteMultiset() */


/*****************************************************************************/
/* Funktion zur Feststellung, ob zwei Multimengen `mm1` und `mm2` disjunkt   */
/* sind.                                                                     */
/*****************************************************************************/

bool MSis_disjunct(MultiSet mm1,MultiSet mm2)

  {

    Action act1, act2;

    while ( (act1=*mm1)!=0 && (act2=*mm2)!=0 )
      {
        if ( (act1|=1) < (act2|=1) )
          {
            mm1+=2;
          }
        else if ( act2 < act1 )
          {
            mm2+=2;
          }
        else
          {
            return ( False );
          }
      } /* while */
    return ( True );

  } /* MSis_disjunct() */

/*****************************************************************************/
/* END OF FILE multiset.c                                                    */
/*****************************************************************************/
