/*****************************************************************************/
/*  multiset.h                                                               */
/*  C-Headerdatei: Multimengen ueber Aktionen -- Datenstrukturen und         */
/*                                               Funktionsprototypen/Makros  */
/*  Di., 03.11.1992 - Di.,02.02.1993                          Stefan Roemer  */
/*****************************************************************************/

#ifndef __MULTISET_H__
#define __MULTISET_H__


#include <stdio.h>
#include <stdlib.h>
#include "boxtypes.h"
#include "memlib.h"

/*****************************************************************************/
/* Darstellung einer Multimenge bei der Ausgabe.                             */
/*****************************************************************************/

extern bool MSdisplay_style; /* globale Variable */

#define MS_WIDE False
#define MS_SHORT True

/*****************************************************************************/
/* Maximal vorgesehene Anzahl von Aktionsnamen in Symboltabelle; wird        */
/* dynamisch an die tatsaechlich benoetigte Groesse angepasst!               */
/*****************************************************************************/

#define ACTION_BUF_SIZE 100


/*****************************************************************************/
/* Globale Variablen fuer die Behandlung von Multimengen. Diese sind nur     */
/* deshalb hier global zusammengefasst worden, um die Aufrufe einiger        */
/* Funktionen einfacher zu halten. Sie duerfen auf keinen Fall veraendert    */
/* werden! Zu Beginn des Hauptprogramms sollte die Funktion `InitMultisets`  */
/* aufgerufen werden, um diese Variablen zu initialisieren!                  */
/*****************************************************************************/

extern Cardinal MaxActions;  /* Maximale Anzahl von Aktionsnamen, die in der */
                             /* Aktionen-Symboltabelle gespeichert werden    */
                             /* koennen; wird dynamisch an benoetigte Groes- */
                             /* se angepasst. Dieser Wert bestimmt auch die  */
                             /* Groesse der dynamischen Arrays, welche zur   */
                             /* Aufnahme von Zwischenergebnissen von Multi-  */
                             /* mengen-Operationen dienen ( globale Variable */
                             /* mmDummy bzw. mmLeaf ).                       */

extern Cardinal ActCounter;  /* Aktuelle, tatsaechliche Anzahl der in dem zu */
                             /* analysierenden Box Expression vorkommenden   */
                             /* ( verschiedenen ) Aktionsnamen, wobei konju- */
                             /* gierte Aktionen nicht ein zweites Mal mitge- */
                             /* zaehlt werden, falls die Aktion bereits un-  */
                             /* konjugiert aufgenommen worden ist ( und um-  */
                             /* gekehrt ). Die Aktionsnamen werden als Zei-  */
                             /* chenketten in der Aktionen-Symboltabelle     */
                             /* gespeichert.                                 */

extern Cardinal MaxVars;     /* Anzahl der maximal in der Variablen-Symbol-  */
                             /* tabelle abzulegenden Variablen; dieser Wert  */
                             /* wird dynamisch an die tatsaechlich benoetig- */
                             /* te Groesse angepasst und sollte nicht geaen- */
                             /* dert werden.                                 */

extern Cardinal VarCounter;  /* Aktuelle, tatsaechliche Anzahl vorkommender  */
                             /* Variablennamen, die in der Variablen-Symbol- */
                             /* tabelle gespeichert werden.                  */

extern Cardinal MaxFunctions;/* Anzahl der maximal in der Funktionen-Symbol- */
                             /* tabelle abzulegenden Funktionen dieser Wert  */
                             /* wird dynamisch an die tatsaechlich benoetig- */
                             /* te Groesse angepasst und sollte nicht geaen- */
                             /* dert werden.                                 */

extern Cardinal FuncCounter; /* Aktuelle, tatsaechliche Anzahl vorkommender  */
                             /* Funktionsnamen, die in der Funktions-Symbol- */
                             /* tabelle gespeichert werden.                  */

extern MultiSet mmDummy;     /* Hilfsmenge bei der Berechnung von Summe und  */
                             /* Vereinigung von Multimengen zur Aufnahme der */
                             /* Ergebnismenge, die "laenger" sein kann als   */
                             /* jeder der "Summanden".                       */

extern MultiSet mmSingle;    /* Hilfsmenge zur Aufnahme einer Multimenge mit */
                             /* nur einer einzigen Aktion ( die jedoch mehr- */
                             /* fach auftreten darf ).                       */

extern MultiSet mmLeaf;      /* Hilfsmenge zur voruebergehenden Aufnahme von */
                             /* Multimengen beim Aufbau des Syntaxbaumes aus */
                             /* dem Box Expression.                          */


/*****************************************************************************/
/* FUNKTIONEN AUF MULTIMENGEN:                                               */
/*****************************************************************************/

/*****************************************************************************/
/* Funktion zum Initialisieren/Allokieren der globalen Variablen des Multi-  */
/* mengen-Funktionspakets. Diese Funktion muss vom Hauptprogramm aufgerufen  */
/* werden, bevor mit Multimengen operiert wird!!!                            */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern void InitMultiSets( void );

#else

extern void InitMultiSets();

#endif


/*****************************************************************************/
/* Funktion zum Einrichten einer neuen Multimenge, hier wegen der Kuerze als */
/* Makro realisiert. Nur eine mit dieser Funktion initialisierte Multimenge  */
/* ist physikalisch im Speicher vorhanden, und zwar mit der uebergebenen     */
/* "Laenge" `n`, d.h. sie kann spaeter hoechstens `n` verschiedene Aktionen  */
/* aufnehmen, wobei konjugierte Aktionen extra mitgezaehlt werden muessen,   */
/* Mehrfachvorkommen eines Aktionsnamens in der entsprechenden Menge jedoch  */
/* nicht. Spaeter wird auf die so initialisierten Multimengen mit Hilfe von  */
/* Zeigern verwiesen.                                                        */
/* Rueckgabe: Zeiger auf den so bereitgestellten Speicherbereich zur Aufnahme*/
/*            einer Multimenge mit hoechstens `n` verschiedenen Aktionen.    */
/*****************************************************************************/

                        /* Funktionalitaet:  MultiSet MSalloc( Cardinal n )  */
#define MSalloc( n ) (MultiSet) MYmalloc( (((n)<<1)+1) * sizeof( Action ) )


/*****************************************************************************/
/* Funktion ( Makro ) zum Verlaengern einer Multimenge.                      */
/* Falls der fuer die Multimenge `mm` vorgesehene Platz nicht ausreicht,     */
/* muss mit dieser Funktion die "Laenge" von `mm` explizit auf die neue      */
/* "Laenge" `n` vergroessert werden, wobei der bisherige Inhalt der Menge    */
/* erhalten bleibt. Die Funktion prueft nicht explizit, ob der neue Wert `n` */
/* tatsaechlich groesser ist als die vorherige "Laenge der Menge `mm`!       */
/* Rueckgabe: Zeiger auf den entsprechenden Speicherbereich. Der alte Inhalt */
/*            der Multimenge bleibt erhalten, auch wenn sich die verlaengerte*/
/*            Menge nicht am gleichen Speicherplatz wie vorher befinden muss!*/
/*****************************************************************************/

                      /* Funktionalitaet:  MultiSet MSresize( MultiSet mm;   */
                      /*                                      Cardinal n   ) */
#define MSresize( mm, n ) ( mm = (MultiSet) MYrealloc( (char *) mm, \
                                        (((n)<<1)+1) * sizeof( Action ) ) )


/*****************************************************************************/
/* Funktion ( Makro ) zum Freigeben des von der Multimenge `mm` im Haupt-    */
/* speicher belegten Speicherplatzes.                                        */
/*****************************************************************************/

                           /* Funktionalitaet:  int MSremove( MultiSet mm )  */
                           /* Hier bewusst auf Indirektion `Multiset * mm`   */
                           /* verzichtet! Nach Ausfuehrung dieser Funktion   */
                           /* ist also `mm` undefiniert !!!                  */
#define MSremove( mm ) free( mm )


/*****************************************************************************/
/* Funktion zum Kopieren einer Multimenge; kopiert `mm1 := mm2`.             */
/* Der Inhalt der Multimenge `mm2` wird in den Speicherbereich kopiert, den  */
/* die Multimenge `mm1` "belegt". Dabei muss darauf geachtet werden, dass    */
/* `mm1` genuegend gross zur Aufnahme der Multimenge initialisiert wurde.    */
/* Vorzugsweise wird es sich bei `mm1` also meistens um die globale Menge    */
/* `mmLeaf` handeln, die gross genug dimensioniert ist, da sie sogar alle    */
/* moeglichen vorkommenden Aktionsnamen aufnehmen kann.                      */
/* Zurueckgegeben wird der Zeiger `mm1`.                                     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern MultiSet MScopy( MultiSet mm1,
                        MultiSet mm2  );

#else

extern MultiSet MScopy( /* MultiSet mm1,
                                    mm2  */ );

#endif


/*****************************************************************************/
/* Funktion zur Initialisierung einer leeren Multimenge `{}`.                */
/* Rueckgabe ist ein Zeiger auf einen mit der leeren Menge initialisierten   */
/* Speicherbereich der "Laenge" 0.                                           */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern MultiSet MScreate_empty_set( void );

#else

extern MultiSet MScreate_empty_set();

#endif


/*****************************************************************************/
/* Funktion zur Initialisierung einer neuen Multimenge, die als einzige      */
/* Aktion `akt` enthaelt, und zwar genau einmal: `{1 akt}` bzw. `{akt}`.     */
/* Rueckgabe: Zeiger auf den so initialisierten Speicherbereich der          */
/*            "Laenge" 1.                                                    */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern MultiSet MScreate_singleton( Action akt );

#else

extern MultiSet MScreate_singleton( /* Action akt */ );

#endif


/*****************************************************************************/
/* Funktion ( als Makro ), die eine bestehende Multimenge `mm` derart aen-   */
/* dert, dass diese nach dem Funktionsaufruf die Menge `mm := {num akt}`     */
/* darstellt. `mm` muss also auf eine mindestens einelementige Multimenge    */
/* zeigen. Rueckgabe ist ein Zeiger auf die so manipulierte Multimenge.      */
/*****************************************************************************/

                 /* Funktionalitaet:  Multiset MSset_single( Multiset mm;    */
                 /*                                          Action   akt;   */
                 /*                                          Cardinal num )  */
#define MSset_single( mm, akt, num ) ( *(mm) = (akt), \
                                       *((mm)+1) = num, \
                                       *((mm)+2) = 0, \
                                       mm \
                                     )


/*****************************************************************************/
/* Funktion zur Initialisierung einer neuen Multimenge als Kopie einer       */
/* bereits vorhandenen Multimenge `mm`; der Speicherbedarf der neu allokier- */
/* ten Menge entspricht ihrer tatsaechlichen "Laenge".                       */
/* Ein Zeiger auf diese neu eingerichtete Menge wird zurueckgegeben.         */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern MultiSet MScreate_copy( MultiSet mm );

#else

extern MultiSet MScreate_copy( /* MultiSet mm */ );

#endif


/*****************************************************************************/
/* Funktion zur Berechnung der Anzahl verschiedener Aktionen in einer Multi- */
/* menge `mm`, d.h. der "Laenge" von `mm`:  `| U  m |`                       */
/*                                           |m@mm  |  .                     */
/* Dabei werden Konjugierte natuerlich mitgezaehlt.                          */
/* Rueckgabe ist die ermittelte Anzahl ( als Cardinal ).                     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern Cardinal MSlength( MultiSet mm );

#else

extern Cardinal MSlength( /* MultiSet mm */ );

#endif


/*****************************************************************************/
/* Funktion zur Berechnung der Maechtigkeit einer Multimenge `mm`: `|mm|`.   */
/* Rueckgabe ist die so ermittelte Kardinalitaet ( als Cardinal ).           */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern Cardinal MScardinality( MultiSet mm );

#else

extern Cardinal MScardinality( /* MultiSet mm */ );

#endif


/*****************************************************************************/
/* Funktion zur Feststellung, ob in der Multimenge `mm` die Aktion `akt`     */
/* oder deren Konjugierte `^akt` enthalten ist.                              */
/* Rueckgabe: Boole'scher Wahrheitswert ( True bei Vorkommen der Aktion ).   */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern bool MSis_in( Action   akt,
                        MultiSet mm   );

#else

extern bool MSis_in( /* Action   akt;
                           MultiSet mm  */ );

#endif


/*****************************************************************************/
/* Funktion zur Berechnung, wie oft eine Aktion `akt` in einer Multimenge    */
/* `mm` enthalten ist: `#akt( mm )`.                                         */
/* Rueckgabe: ermittelte Anzahl Vorkommen ( als Cardinal ).                  */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern Cardinal MScount( Action   akt,
                         MultiSet mm   );

#else

extern Cardinal MScount( /* Action   akt;
                            MultiSet mm  */ );

#endif


/*****************************************************************************/
/* Funktion zur Berechnung der Konjugierten der uebergebenen Aktion `akt`,   */
/* also `^akt`, welche auch zurueckgegeben wird. Als Makro realisiert.       */
/*****************************************************************************/

                    /* Funktionalitaet:  Action MSconj_action( Action akt )  */
#define MSconj_action( akt ) ( (akt) ^ 1 )

/* liefert wahr, falls die Aktion konjugiert ist */
#define is_conjugate(__ACT__) (__ACT__ & 1)

/*****************************************************************************/
/* Funktion ( als Makro ) zur Berechnung, wie oft die zu `akt` konjugierte   */
/* Aktion ( also `^akt` ) in der Multimenge `mm` enthalten ist, d.h. es wird */
/* `#^akt( mm )` ermittelt.                                                  */
/* Rueckgabe ist die so ermittelte Anzahl ( als Cardinal ).                  */
/*****************************************************************************/

                     /* Funktionalitaet:  Cardinal MScount( Action   akt;    */
                     /*                                     MultiSet mm   )  */
#define MScount_conj( akt, mm ) MScount( MSconj_action( akt ), mm )


/*****************************************************************************/
/* Funktion zur Konjugation der gesamten Multimenge `mm`: berechnet `^mm`.   */
/* Rueckgabe ist ein Zeiger auf diese so manipulierte Multimenge.            */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern MultiSet MSconjugation( MultiSet mm );

#else

extern MultiSet MSconjugation( /* MultiSet mm */ );

#endif


/*****************************************************************************/
/* Funktion zur Feststellung, ob zwei Multimengen `mm1`, `mm2` gleich sind.  */
/* Rueckgabe: True,  falls `mm1 == mm2`, d.h. `mm1` enthaelt genau die glei- */
/*                   chen Elemente wie `mm2` ( in der gleichen Anzahl ).     */
/*            False, falls `mm1 != mm2`.                                     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern bool MSis_equal( MultiSet mm1,
                           MultiSet mm2  );

#else

extern bool MSis_equal( /* MultiSet mm1,
                                       mm2  */ );

#endif


/*****************************************************************************/
/* Funktion zur Vereinigung zweier Multimengen `mm1` und `mm2`; es wird      */
/* `mm1 := mm1 u mm2` berechnet, d.h. die zweite Menge wird zu der ersten    */
/* "hinzuvereinigt". Rueckgabe ist ein Zeiger auf die Ergebnismenge; `mm1`   */
/* ist somit nach dieser Operation zerstoert und muesste, falls gewuenscht,  */
/* vorher explizit in eine Hilfsmenge kopiert werden ( mit Funktion `MScopy` */
/* oder `MScreate_copy` ). Achtung! Multimenge `mm1` muss gross genug zur    */
/* Aufnahme des Ergebnisses sein!!!                                          */
/* Seiteneffekt: Benutzung der globalen Variablen `mmDummy`.                 */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern MultiSet MSunion( MultiSet mm1,
                         MultiSet mm2  );

#else

extern MultiSet MSunion( /* MultiSet mm1,
                                     mm2  */ );

#endif

  /* GLOBAL MultiSet mmDummy; (schreibend) */


/*****************************************************************************/
/* Funktion zur Durchschnittbildung zweier Multimengen `mm1` und `mm2`; es   */
/* wird `mm1 := mm1 n mm2` berechnet.                                        */
/* Rueckgabe: Zeiger auf die Schnittmenge. Der alte Inhalt von `mm1` wird    */
/*            ueberschrieben, also gegebenenfalls `mm1` vorher kopieren!     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern MultiSet MSintersection( MultiSet mm1,
                                MultiSet mm2 );

#else

extern MultiSet MSintersection( /* MultiSet mm1,
                                            mm2  */ );

#endif


/*****************************************************************************/
/* Funktion zur Summenbildung zweier Multimengen `mm1` und `mm2`: es wird    */
/* `mm1 := mm1 + mm2` berechnet, d.h. `mm2` wird zu `mm1` "hinzuaddiert".    */
/* Rueckgabe: Zeiger auf die Ergebnismenge `mm1`, deren alter Inhalt ueber-  */
/*            schrieben wird.                                                */
/* Seiteneffekt: Benutzung der globalen Variablen `mmDummy` zur Aufnahme     */
/*               des Zwischenergebnisses.                                    */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern MultiSet MSsum( MultiSet mm1,
                       MultiSet mm2  );

#else

extern MultiSet MSsum( /* MultiSet mm1,
                                   mm2  */ );

#endif

  /* GLOBAL MultiSet mmDummy; (schreibend) */


/*****************************************************************************/
/* Funktion zur Differenzbildung zweier Multimengen `mm1` und `mm2`; es wird */
/* `mm1 := mm1 \ mm2` berechnet, `mm2` wird also von `mm1` "abgezogen".      */
/* Rueckgabe: Zeiger auf die Ergebnismenge `mm1`, deren alter Inhalt somit   */
/*            ueberschrieben ist ( evtl. retten durch Kopieren! ).           */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern MultiSet MSdifference( MultiSet mm1,
                              MultiSet mm2  );

#else

extern MultiSet MSdifference( /* MultiSet mm1,
                                          mm2  */ );

#endif


/*****************************************************************************/
/* Funktion zur Berechnung von `mm := mm \ { num akt, num ^akt }`.           */
/* Rueckgabe: Zeiger auf die Ergebnismenge `mm`, deren alter Inhalt somit    */
/*            ueberschrieben ist ( evtl. retten durch Kopieren! ).           */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern MultiSet MSremove_action_pairs( MultiSet mm,
                                       Action   akt,
                                       Cardinal num  );

#else

extern MultiSet MSremove_action_pairs( /* MultiSet mm;
                                          Action   akt;
                                          Cardinal num;  */ );

#endif


/*****************************************************************************/
/* Funktion zur Umbenennung einzelner Aktionen einer Multimenge `mm`, wie    */
/* es die uebergebene Funktion `f` angibt: berechnet `mm := f( mm )`.        */
/* Falls es sich bei der uebergebenen "Multimenge" um eine Variable handelt, */
/* so wird die daran anhaengende Funktion "umbenannt"                        */
/* ACHTUNG: Bei Variablen-Multimengen kann sich die Laenge von `mm` vergroes-*/
/* sern ( mittels `MYresize()` ); deshalb muss der von dieser Funktion gelie-*/
/* ferte Zeiger der Variablen `mm` wieder zugewiesen werden !                */
/* Beispiel: MultiSet menge; // soll eine Variable enthalten                 */
/*           Function fnc;                                                   */
/*           menge = MSrelabel( menge, fnc );                                */
/*           =====                                                           */
/* Zurueckgeliefert wird ein Zeiger auf die veraenderte Menge `mm`.          */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern MultiSet MSrelabel( MultiSet mm,
                           Function f   );

#else

extern MultiSet MSrelabel( /* MultiSet mm;
                              Function f   */ );

#endif

  /* GLOBAL: MultiSet mmLeaf; (schreibend) */


/*****************************************************************************/
/* Funktion zum Schreiben einer Multimenge `mm` in eine Datei `fileptr`.     */
/* Dabei beeinflusst die globale Variable `MSdisplay_style` das Ausgabe-     */
/* format fuer mehrfach vorkommende Namen:                                   */
/* MSdisplay_style == MS_WIDE  ==>  {a, a, ^b, ^b, ^b}                       */
/*                 == MS_SHORT ==>  {2 a, 3 ^b}                              */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern void MSWriteMultiset( FILE     * fileptr,
                             MultiSet mm         );

#else

extern void MSWriteMultiset( /* FILE     * fileptr;
                                MultiSet mm        */ );

#endif

  /* GLOBAL: MSdisplay_style; (lesend) */


/*****************************************************************************/
/* Funktion (Makro) zur Darstellung einer Multimenge `mm` auf dem Bildschirm.*/
/* Die globale Variable `MSdisplay_style` bestimmt auch hierbei das          */
/* Ausgabeformat mehrfach vorkommender Namen:                                */
/* MSdisplay_style == MS_WIDE  ==>  {a, a, ^b, ^b, ^b}                       */
/*                 == MS_SHORT ==>  {2 a, 3 ^b}                              */
/*****************************************************************************/

                     /* Funktionalitaet:  void MSprint( MultiSet mm )  */
#define MSprint( _MM_ ) MSWriteMultiset( stdout, _MM_ );
  /* GLOBAL: MSdisplay_style; (lesend) */


/*****************************************************************************/
/* Funktion zur Feststellung, ob zwei Multimengen `mm1` und `mm2` disjunkt   */
/* sind.                                                                     */
/*****************************************************************************/

bool MSis_disjunct(MultiSet mm1,MultiSet mm2);



/*****************************************************************************/


#endif /* __MULTISET_H__ */


/*****************************************************************************/
/* END OF FILE multiset.h                                                    */
/*****************************************************************************/

