/*****************************************************************************/
/*  parser.c                                                                 */
/*  C-Quelldatei: Hilfsroutinen fuer den 'box expression'-Parser             */
/*  Di.,27.10.1992 - Mi.,04.11.1992                           Stefan Roemer  */
/*  Aenderung 6.6.1999: yyerror nach boxxpars.y verschoben -Christian Stehno */
/*****************************************************************************/


#include <stdio.h>
#include <string.h>
#include "parser.h"
#include "boxtypes.h"
#include "memlib.h"
#include "multiset.h"
/* scanner.h nicht mehr gebraucht, da yyerror verschoben */
#include "boxxpars.h"

/*****************************************************************************/
/*            Globale Variablen fuer den 'box expression'-Parser             */
/*****************************************************************************/

SymTab   SymbolTable = NULL;
SymTab   VarTable = NULL;
FuncTab  FunctionTable = NULL;

/* Defaultwert, der festlegt, bis zu welcher Tiefe Schleifen */
/* ( Iteration/Rekursion) berechnet werden sollen. Dieser    */
/* Wert kann global fuer einen Box-Ausdruck veraendert sowie */
/* lokal fuer einzelne Schleifen ueberlagert werden.         */
Cardinal RecDepth, SyncDepth;


/*****************************************************************************/
/*             Hilfsfunktionen fuer den 'box expression'-Parser              */
/*****************************************************************************/


/*****************************************************************************/
/* Erzeugen eines neuen Knotens fuer den Syntaxbaum der 'box expression's    */
/*****************************************************************************/

TTree GenNewNode(Cardinal operation,TTree expr1,TTree expr2,TTree expr3,
                 int fl, int fc, int ll, int lc)
  {

    TTree node;

    ( node = (TTree) MYmalloc( sizeof(TTreeNode) ) ) -> Operator = operation;
    node->Operand1 = expr1;
    node->Operand2 = expr2;
    node->Operand3 = expr3;
    node->Depth = 0;

    node->fl = fl;
    node->fc = fc;
    node->ll = ll;
    node->lc = lc;

    return ( node );

  } /* GenNewNode() */

/*****************************************************************************/
/* Entfernen eines Syntaxbaumes aus dem Hauptspeicher.                       */
/* Rekursive Funktion!                                                       */
/*****************************************************************************/

void RemoveTTree(TTree *root)

  {

    if ( *root )
      {
        switch ( (*root)->Operator )
          {
            case ACTION    :
            case VARIABLE  :
                             MSremove( (*root)->Operand1 );
                             break;
            case SEQUENCE  :
            case CHOICE    :
            case PARALLEL  :
                             RemoveTTree( &((*root)->Operand1) );
                             RemoveTTree( &((*root)->Operand2) );
                             break;
            case SYNCHRON  :
            case RESTRICT  :
            case SCOPE     :
	    case TIE       :
                             RemoveTTree( &((*root)->Operand1) );
                             MSremove( (*root)->Operand2 );
                             break;
            case RELABEL   :
                             RemoveTTree( &((*root)->Operand1) );
                             if ( (Cardinal)(Cardinal32)((*root)->Operand2)==MAP )
                                 MSremove( (*root)->Operand3 );
                             break;
            case REFINE    :
                             RemoveTTree( &((*root)->Operand1) );
                             RemoveTTree( &((*root)->Operand3) );
                             break;
            case RECURSION :
                             RemoveTTree( &((*root)->Operand2) );
                             break;
            case ITERATION :
                             RemoveTTree( &((*root)->Operand1) );
                             RemoveTTree( &((*root)->Operand2) );
                             RemoveTTree( &((*root)->Operand3) );
                             break;
            case FILENAME  :
                             free( (*root)->Operand1 );
                             break;
            case C_BOX     :
            case S_BOX     :
            case D_BOX     :
                             KillRange((Range *)(*root)->Operand2);
                             break;
            case STOP_BOX  :
                             break;
            default        :
                             break;
          } /* switch */
        DeleteNode( *root );
        *root = NULL;
      } /* if */

  } /* RemoveTTree() */

Time_stamp *CreateTimeStamp (Time low, Time high)
{
  Time_stamp *time_label;

  time_label=malloc(sizeof(Time_stamp));
  time_label->low=low;
  time_label->high=high;
  return time_label;
}


/*****************************************************************************/
/* Umwandeln eines Aktionsnamens ( String ) in entsprechenden Aktionscode    */
/* ( Cardinal ). Falls der Name noch nicht in der Symboltabelle enthalten    */
/* ist, wird er in derselben am Ende eingefuegt.                             */
/* Aktionssymbole weren ohne Konjugierung und ohne +\- (bei Links) abgegelgt */
/* Diese Infos werden in den beiden letzten Bits im Programm gespeichert:    */
/* 0x bei Aktionen (x=1: konjugiert) 1x bei Links (x=1: export)              */
/*****************************************************************************/

Action StrToAction(char *Name)

  {

    register SymTab TblPtr = SymbolTable;

                                /* Den Namen in der Symboltabelle suchen     */
                                /* Lineare Suche ... eventuell schnelleres   */
                                /* Verfahren fuer die Symboltabellenverwal-  */
                                /* tung verwenden !!!                        */
    while ( *TblPtr && ( ( *Name != **TblPtr ) || strcmp( Name, *TblPtr ) ) )
      {
        TblPtr++;
      } /* while */
                                /* Falls Name nicht in Liste vorhanden:      */
                                /* am Ende der Tabelle einfuegen.            */
    if ( ! *TblPtr )
      {
        if ( ++ActCounter >= MaxActions )
          {
            int OffSet=(int)(TblPtr-SymbolTable);
            TblPtr = ( SymbolTable = (SymTab) MYrealloc( (char *) SymbolTable,
                              (Cardinal32) ( MaxActions += ACTION_BUF_SIZE )
                                                     * sizeof( SymTabEntry ) )
                     ) + OffSet;
            MSresize( mmDummy, MaxActions );
            MSresize( mmLeaf, MaxActions );
          } /* if */
        * TblPtr    = MYstrdup( Name );
        *(TblPtr+1) = NULL;
      } /* if */
                                /* Position des Namens in der Symboltabelle  */
                                /* in den Aktionscode umrechnen und zurueck- */
                                /* geben; in der Tabelle sind alle Namen     */
                                /* unkonjugiert!                             */
    return ( (Action) ( ( TblPtr - SymbolTable + 1 ) << 2 ) );

  } /* StrToAction() */


/*****************************************************************************/
/* Umwandlung Aktion(scode) --> Aktionsname (String).                        */
/*****************************************************************************/

char * ActionToStr(Action akt,char *str)

  {

    if (!(akt & 2))
      return ( strcat( strcpy( str, akt & 1 ? "^" : ""),
                     SymbolTable[ ( akt >> 2 ) - 1 ] ) );
    else
      return ( strcat( strcpy( str, SymbolTable[ ( akt >> 2 ) - 1 ]),
	       akt & 1 ? "-" : "+"));

  } /* ActionToStr() */


/*****************************************************************************/
/* Umwandeln eines Variablennamens ( String ) in entsprechenden Variablencode*/
/* ( Cardinal ). Falls der Name noch nicht in der Symboltabelle enthalten    */
/* ist, wird er in derselben am Ende eingefuegt.                             */
/*****************************************************************************/

Var StrToVariable(char *Name)

  {

    register SymTab TblPtr = VarTable;

                                /* Den Namen in der Symboltabelle suchen     */
                                /* Lineare Suche ... eventuell schnelleres   */
                                /* Verfahren fuer die Symboltabellenverwal-  */
                                /* tung verwenden !!!                        */
    while ( *TblPtr && ( ( *Name != **TblPtr ) || strcmp( Name, *TblPtr ) ) )
      {
        TblPtr++;
      } /* while */
                                /* Falls Name nicht in Liste vorhanden:      */
                                /* am Ende der Tabelle einfuegen.            */
    if ( ! *TblPtr )
      {
        if ( ++VarCounter >= MaxVars )
          {
            int OffSet=(int)(TblPtr-VarTable);
            TblPtr = ( VarTable = (SymTab) MYrealloc( (char *) VarTable,
                                              ( MaxVars += ACTION_BUF_SIZE )
                                                     * sizeof( SymTabEntry ) )
                     ) + OffSet;
          } /* if */
        * TblPtr    = MYstrdup( Name );
        *(TblPtr+1) = NULL;
      } /* if */
                                /* Position des Namens in der Symboltabelle  */
                                /* in den Variablencode umrechnen und        */
                                /* zurueckgeben.                             */
    return ( (Var) ( TblPtr - VarTable + 1 ) );

  } /* StrToVariable() */


/*****************************************************************************/
/* Umwandlung Variable(ncode) --> Variablenname (String).                    */
/*****************************************************************************/

char * VariableToStr(Var var,char *str)

  {

    /* var enthaelt codierte Zahl der Variablen */
    return ( strcpy( str, VarTable[ var - 1 ] ) );

  } /* VarToStr() */


/*****************************************************************************/
/* Umwandeln eines Funktionsnamens ( String ) in entsprechenden Funktionscode*/
/* ( Cardinal ). Falls der Name noch nicht in der Symboltabelle enthalten    */
/* ist, wird er in derselben am Ende eingefuegt ( mitsamt Abbildungsliste ). */
/* Ansonsten erfolgt eine Fehlermeldung ( Doppelt deklarierte Funktion ).    */
/* Falls `Mappings == NULL` gilt, wird keine Fehlermeldung ausgegeben, wenn  */
/* der Name nicht in der Tabelle enthalten ist, sondern der Wert 0 ( fuer    */
/* nicht-deklarierte Funktion ) zurueckgegeben.                              */
/*****************************************************************************/

FuncCode StrToFunction(char *Name,Function Mappings)

  {

    register FuncTab TblPtr = FunctionTable;

                                /* Den Namen in der Symboltabelle suchen     */
                                /* Lineare Suche ... eventuell schnelleres   */
                                /* Verfahren fuer die Symboltabellenverwal-  */
                                /* tung verwenden !!!                        */
    while ( TblPtr->FuncName && strcmp( Name, TblPtr->FuncName ) )
      {
        TblPtr++;
      } /* while */
                                /* Falls Name nicht in Liste vorhanden:      */
                                /* am Ende der Tabelle einfuegen.            */
    if ( Mappings && ! TblPtr->FuncName )
      {
        if ( ++FuncCounter >= MaxFunctions )
          {
            int OffSet=(int)(TblPtr-FunctionTable);
            TblPtr = ( FunctionTable = (FuncTab)
                                       MYrealloc( (char *) FunctionTable,
                                            ( MaxFunctions += ACTION_BUF_SIZE )
                                                     * sizeof( FuncTabEntry ) )
                     ) + OffSet;
          } /* if */
        TblPtr->FuncName = MYstrdup( Name );
        TblPtr->FuncMappings = MScreate_copy( Mappings );
        ( TblPtr + 1 ) -> FuncName = NULL;
      } /* if */
    else if ( ! TblPtr->FuncName )
      {
        return ( 0 );
      } /* else if */

                                /* Position des Namens in der Symboltabelle  */
                                /* in den Funktionscode umrechnen und        */
                                /* zurueckgeben.                             */
    return ( (FuncCode) ( TblPtr - FunctionTable + 1 ) );

  } /* StrToFunction() */


/*****************************************************************************/
/* Umwandlung Funktion(scode) --> Funktionsname (String).                    */
/*****************************************************************************/

char * FunctionToStr(FuncCode funccode,char *str)

  {

    return ( strcpy( str, FunctionTable[ funccode - 1 ].FuncName ) );

  } /* FunctionToStr() */


/*****************************************************************************/
/* Entfernen einer Symboltabelle aus dem Speicher.                           */
/*****************************************************************************/

void RemoveSymbolTable(SymTab *table)

  {

    register SymTab STPtr;

    if ((STPtr=*table)!=NULL)
      {
        while ( *STPtr )
          {
            free( *(STPtr++) );
          } /* while */
        free( *table );
        SymbolTable = NULL;
      } /* if */

  } /* RemoveSymbolTable() */


/*****************************************************************************/
/* Entfernen der Funktions-Symboltabelle aus dem Speicher.                   */
/*****************************************************************************/

void RemoveFunctionTable()

  {

    register FuncTab tabptr;

    if ((tabptr=FunctionTable)!=NULL)
      {
        while ( tabptr->FuncName )
          {
            free( tabptr->FuncMappings );
            free( (tabptr++)->FuncName );
          } /* while */
        free( FunctionTable );
        FunctionTable = NULL;
      } /* if */

  } /* RemoveFunctionTable() */


/*****************************************************************************/
/* Initialisiere globale Variablen: Symboltabellen.                          */
/*****************************************************************************/

void InitAllSymbolTables()

  /* GLOBAL: SymTab   SymbolTable, VarTable;
             FuncTab  FunctionTable;
             Cardinal MaxActions, MaxVars, MaxFunctions,
                      ActCounter, VarCounter, FuncCounter */

  {

                                /* Initialisieren der Aktionen-Symboltabelle.*/
    if ( SymbolTable )
      {
        RemoveSymbolTable( &SymbolTable );
      } /* if */
    SymbolTable = (SymTab) MYmalloc( MaxActions * sizeof( SymTabEntry ) );
    * SymbolTable = NULL;
    MaxActions = ACTION_BUF_SIZE;
    ActCounter = 0;

                                /* Initialisieren der Variablen-Symbol-      */
                                /* tabelle.                                  */
    if ( VarTable )
      {
        RemoveSymbolTable( &VarTable );
      } /* if */
    VarTable = (SymTab) MYmalloc( MaxVars * sizeof( SymTabEntry ) );
    * VarTable = NULL;
    MaxVars = ACTION_BUF_SIZE;
    VarCounter = 0;

                                /* Initialisieren der Funktionen-Symbol-     */
                                /* tabelle.                                  */
    if ( FunctionTable )
      {
        RemoveFunctionTable();
      } /* if */
    FunctionTable = (FuncTab) MYmalloc( MaxFunctions * sizeof(FuncTabEntry) );
    FunctionTable->FuncName = NULL;
    MaxFunctions = ACTION_BUF_SIZE;
    FuncCounter = 0;

  } /* InitAllSymbolTables() */


/*****************************************************************************/
/* Ausgabe der Aktionen-Symboltabelle auf dem Bildschirm.                    */
/* Nur zu Analysezwecken eingerichtet.                                       */
/*****************************************************************************/

void PrintActionTable()

  {

    register SymTab table;
    register Cardinal i = 2;

    printf( "SYMBOLTABLE Actions:\n" );
    if ((table=SymbolTable)!=NULL)
      {
        while ( *table )
          {
            printf( "%6u. '%s'.\n", i++, *(table++) );
            i++;
          } /* while */
      } /* if */
    printf( "END SYMBOLTABLE Actions\n" );

  } /* PrintSymbolTable() */


/*****************************************************************************/
/* Ausgabe der Variablen-Symboltabelle auf dem Bildschirm.                   */
/* Nur zu Analysezwecken eingerichtet.                                       */
/*****************************************************************************/

void PrintVariableTable()

  {

    register SymTab table;
    register Cardinal i = 1;

    printf( "SYMBOLTABLE Variables:\n" );
    if ((table=VarTable)!=NULL)
      {
        while ( *table )
          {
            printf( "%6u. '%s'.\n", i++, *(table++) );
          } /* while */
      } /* if */
    printf( "END SYMBOLTABLE Variables\n" );

  } /* PrintVariableTable() */


/*****************************************************************************/
/* Ausgabe der Funktionen-Symboltabelle auf dem Bildschirm.                  */
/* Nur zu Analysezwecken eingerichtet.                                       */
/*****************************************************************************/

void PrintFunctionTable()

  {

    register FuncTab table;
    register Cardinal i = 1;
    Function fptr;
    char     dum[50];

    printf( "SYMBOLTABLE Functions:\n" );
    if ((table=FunctionTable)!=NULL)
      {
        while ( table->FuncName )
          {
            printf( "%6u. '%s' = (", i++, table->FuncName );
            if ((fptr=table->FuncMappings)!=NULL)
              {
                printf( " %s->", ActionToStr( *(fptr++), dum ) );
                printf( "%s", ActionToStr( *(fptr++), dum ) );
                while ( *fptr )
                  {
                    printf( ", %s->", ActionToStr( *(fptr++), dum ) );
                    printf( "%s", ActionToStr( *(fptr++), dum ) );
                  } /* while */
              } /* if */
            printf( " ).\n" );
            table++;
          } /* while */
      } /* if */
    printf( "END SYMBOLTABLE Functions\n" );

  } /* PrintFunctionTable() */


/*****************************************************************************/
/* END OF FILE parser.c                                                      */
/*****************************************************************************/

