/****************************************************************/
/*                                                              */
/*            C O D E - G E N E R A T O R  for B(PN)^2          */
/*                                                              */
/*            Universitaet Hildesheim                           */
/*            Institut fuer Informatik                          */
/*            Stephan Melzer                                    */
/****************************************************************/
/*            Diplomarbeit                                      */
/*            File        : algebra.y
              Version     : 1.0
	      Last Update : 18.10.1994
 ****************************************************************/
%{

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "algebra.h"
#include "bits.h"
#include "misc.h"

char* input_string;
int pos;

sequence* seq_tab;

int yylex(void);

sequence* init_seq(char* single)
{
  sequence* result = (sequence*) Malloc(sizeof(sequence));
  SET dummy = nil_set();

  result->seq = (char*) Malloc((strlen(single) + 2) * sizeof(char));
  strcpy(result->seq, single);
  result->seq = strcat(result->seq, " ");
  insert_element(dummy, atoi (single));
  result->procs = dummy;
  result->next = (sequence*) NULL;
  return result;
}

void print_seq(sequence* s)
{
  sequence* dummy = s;

  printf("\n");
  while (dummy != (sequence*) NULL)
  {
    printf("%s\n", dummy->seq);
    dummy = dummy->next;
  }
}

sequence* append_seq(sequence* left, sequence* right)
{
  sequence* dummy = left;

  if (left == (sequence*) NULL)
    return right;

  while (dummy->next != (sequence*) NULL)
    dummy = dummy->next;

  dummy->next = right;
  return left;
}

sequence* combine_seq(sequence* left, sequence* right)
{
  sequence *result    = (sequence*) NULL,
	   *left_ptr,
	   *right_ptr = right;

  while (right_ptr != (sequence*) NULL)
  {
    left_ptr = left;
    while (left_ptr != (sequence*) NULL)
    {
      if (is_disjunct(left_ptr->procs, right_ptr->procs))
      {
	sequence* dum = (sequence*) Malloc(sizeof(sequence));
	dum->seq = (char*) Malloc((strlen(right_ptr->seq) +
				   strlen(left_ptr->seq)  + 1) *
				   sizeof(char));
	strcpy(dum->seq, left_ptr->seq);
	dum->seq = strcat(dum->seq, right_ptr->seq);
	dum->procs = union_set(right_ptr->procs, left_ptr->procs);
	dum->next = result;
	result = dum;
      }
      left_ptr = left_ptr->next;
    } 
    right_ptr = right_ptr->next;
  }
  return result;
}

/* jj */
void yyerror(char* s)
{
  fprintf(stderr, "syntax error in interleaving term: %s\n", s);
}
/* jj */


%}

%union{
  char string[6];
  sequence* synchro;
}

%start begin
%token OPEN CLOSE CAT OR 
%token <string> PROC
%type <synchro> begin sequence sub_sequence list proc_list sequence_list

%%

begin      	: /* epsilon */ {seq_tab = (sequence*) NULL;}
             	| sequence {seq_tab = $1;}
             	;

sequence 	: sub_sequence PROC sub_sequence 
		  {sequence* single = init_seq($2);
	           if ($1 != (sequence*) -1)
		     $$ = combine_seq($1, single);
		   else
		     $$ = single;
		   if ($3 != (sequence*) -1)
		     $$ = combine_seq ($$, $3);
 		  }
  	 	;

sub_sequence 	: OPEN list CLOSE {$$ = $2;}
 	     	| /* epsilon */   {$$ = (sequence*) -1;}
	     	;

list		: proc_list OR list {$$ = append_seq($1, $3);}
	        | /* epsilon */     {$$ = (sequence*) NULL;}
		;

proc_list 	: OPEN sequence_list CLOSE {$$ = $2;}
 	  	;

sequence_list 	: sequence CAT sequence_list 
		  {if ($3 != (sequence*) -1)
		     $$ = combine_seq ($1, $3);
		   else
		     $$ = $1;
 		  }
		| /* epsilon */ {$$ = (sequence*) -1;}
 	      	;

%%

void get_term(char* inter_leaving)
{
  input_string = (char*) Malloc((strlen(inter_leaving) + 1) * sizeof(char));
  strcpy(input_string, inter_leaving);
  pos = 0;
  seq_tab = (sequence*) NULL;
}

int yylex(void)
{
  if (input_string[pos] == '\0')
    return (0);
  if (! isdigit(input_string[pos]))
    switch (input_string[pos])
    {
      case '(' : pos++ ; return (OPEN);
      case ')' : pos++ ; return (CLOSE); 
      case ';' : pos++ ; return (CAT);
      case '|' : pos++ ; return (OR);
      default : pos++ ; return (input_string[pos]);
    }
  else
  {
    int i;

    for (i=0 ; isdigit(input_string[pos]) && (i < 6); i++, pos++)
      yylval.string[i] = input_string[pos];
    yylval.string[i] = '\0';
    return (PROC);
  }
}

/*-------------------------------------------------------------------------*/
/* End of file algebra.y */
/*-------------------------------------------------------------------------*/
