/****************************************************************/
/*                                                              */
/*            C O D E - G E N E R A T O R  for B(PN)^2          */
/*                                                              */
/*            Universitaet Hildesheim                           */
/*            Institut fuer Informatik                          */
/*            Stephan Melzer                                    */
/****************************************************************/
/*            Diplomarbeit                                      */
/*            File        : bits.c                                          
              Version     : 1.0
	      Last Update : 18.10.1994
 ****************************************************************/
/*            Module information :                              */
/* This module is a libary for set handling based on bit-arrays.
   The basic routines such as union and intersection and more are
   implemented.
*/
/****************************************************************/
/* Johannes Jaeger -- Sommer 1997 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "bits.h"
#include "misc.h"
#include "configure.h"


/*                        bit0  1  2  3   4   5   6    7  */
unsigned short bit_mask[] = {1, 2, 4, 8, 16, 32, 64, 128};

SET nil_set(void)
/* Creates an empty set, with no element inside. 
*/
{
  short i;
  SET result = (SET) Malloc(BIT_LENGTH * sizeof(unsigned short));

  for (i = 0;i < BIT_LENGTH; i++)
    result[i] = 0;
  return result;
}

void print(SET a)
{
  int i = BIT_LENGTH - 1;
  printf("%u", a[i--]);
  while ( i >= 0 )
  {
    printf(" %u", a[i--]);
  }
}

SET intersection_set(SET left, SET right)
/* Intersection between the sets left and right.
   left, right : Sets
*/
{
  short	i;
  SET result = (SET) Malloc(BIT_LENGTH * sizeof(unsigned short));

  for (i = 0;i < BIT_LENGTH; i++)
    result[i] = left[i] & right[i];
  return result;
}

SET union_set(SET left, SET right)
/* Union of the sets left and right.
   left, right : Sets
*/
{
  short	i;
  SET result = (SET) Malloc(BIT_LENGTH * sizeof(unsigned short));

  for (i = 0;i < BIT_LENGTH; i++)
    result[i] = left[i] | right[i];
  return result;
}

SET difference_set(SET left, SET right)
/* Difference between left and right.
  left, right : Sets
*/
{
  short	i;
  SET result = (SET) Malloc(BIT_LENGTH * sizeof(unsigned short));

  for (i = 0; i < BIT_LENGTH; i++)
    result[i] = left[i] ^ (left[i] & right[i]);
  return result;
}

void insert_element(SET u, int element)
/* This function inserts an element in the set u.
   u : Set
   element : integer to insert
*/
{
  u[element/INT_WIDTH] = u[element/INT_WIDTH] | bit_mask[element % INT_WIDTH];
}

void delete_element(SET u, int element)
/* This function deletes an element in the set u.
   u : Set
   element : integer to delete
*/
{
  u[element/INT_WIDTH] = u[element/INT_WIDTH] ^ bit_mask[element % INT_WIDTH];
}


int is_element(SET u, int element)
{
  return (u[element / INT_WIDTH] & bit_mask[element % INT_WIDTH]);
}

int is_subset(SET left, SET right)
/* This function returns TRUE, if all elements in left
   are also in the right set.
   left, right : Sets
*/
{
  short	i;

  for (i = 0; i < BIT_LENGTH; i++)
    if ( !(left[i] == (left[i] & right[i])) )
      return 0;

  return 1;
}

int is_empty(SET u)
/* This function returns TRUE, if the set is empty.
   u : Set
*/
{
  SET empty = nil_set();
  int result = is_subset(u, empty);

  free(empty);
  return (result);
}

int is_disjunct(SET left, SET right)
/* This function returns TRUE, if intersection of both sets is empty 
   left, right : Sets
*/
{
  SET dummy = intersection_set(left, right);
  int result = is_empty(dummy);

  free(dummy);
  return result;
}

char* str_set(SET u)
/* Converts a set into a string.
  u : Set
*/
{
  char*	result = (char*) Malloc((size_t) (BIT_LENGTH * 6 + 1) );
  char tmp[7];
  int i = BIT_LENGTH - 1;

  (void) sprintf(result, "%u", u[i--]);
  while ( i >= 0)
  {
    sprintf(tmp, " %u", u[i--]);
    (void) strcat(result, tmp);
  }
  return result;
}

SET set_str(char* s)
/* Converts a string into a set.
  s : String
*/
{
  SET	r = nil_set();
  char *anfang, *ende;
  long number;
  int i;

  anfang = s;
  for (i = BIT_LENGTH - 1; i >= 0; i--)
  {
    number = strtol(anfang, &ende, 10);
    anfang = ende;
    r[i] = (unsigned short)number;
  }

  return r;
}

int elements(SET s)
/* Calculates the size of the set s.
   s : Set
*/
{
  int result = 0,
      i;

  for (i=0; i < INT_WIDTH * BIT_LENGTH; i++)
    if (is_element(s, i) != 0 )
      result++;

  return result;
}

int get_element(SET u, int number)
{
  int c, i;
  for (i=0, c=0; i < INT_WIDTH * BIT_LENGTH; i++)
  {
    if (is_element(u,i) != 0)
      c++;
    if (c == number) 
      return i;
  }
  return (-1);
}

 
conf* initialize_conf(unsigned short* sets, int number_of_sets)
{
  conf* result = (conf*) Malloc(sizeof(conf));
  int i;

  result->counter = 0;
  result->size = number_of_sets;
  result->cross_product = (unsigned short*) Malloc(number_of_sets *
						   BIT_LENGTH * 
                                                   sizeof(unsigned short)
                                                   );
  memcpy(result->cross_product, sets, number_of_sets *
	                        BIT_LENGTH * sizeof(unsigned short));

  result->sizes = (int*) Malloc(number_of_sets * sizeof(int));
  result->max_configs = (number_of_sets == 0) ? 0 : 1;
  for (i=0; i < number_of_sets; i++)
  {
    result->sizes[i] = elements(&(sets[i * BIT_LENGTH])); 
    result->max_configs = result->max_configs * result->sizes[i];
  }
  return result;
}

int* get_next_config(conf* act)
{
  int* result;
  int  dummy = act->counter,
       i;

  if (act->size == 0)
    return (int*) NULL;

  result = (int*) Malloc(act->size * sizeof(int));

  if (act->counter == act->max_configs)
  {
    free(result);
    return ((int *) NULL);
  }

  for (i = act->size - 1; i >= 0; i--)
  {
    result[i] = get_element(&(act->cross_product[i * BIT_LENGTH]),
			    dummy % act->sizes[i] + 1);
    dummy = dummy / act->sizes[i];
  }

  act->counter++;
  return result;
}

int* check_loop_synchro(conf* c, int* act_config)
{
  SET dummy = nil_set();
  int i;

  for (i=0; i < c->size; i++)
  {
    if (is_element(dummy, act_config[i]) == 0 )
      insert_element(dummy, act_config[i]);
    else 
    {
      free(dummy);
      return ((int*) NULL);
    }
  }
  return act_config;
}

/*-------------------------------------------------------------------------*/
/* End of file bits.c */
/*-------------------------------------------------------------------------*/
