/*-------------------------------------------------------------------------*/
/*  c_code.c  zusammengestellt aus dest_code_handling.c von S. Melzer      */
/*-------------------------------------------------------------------------*/
/*  B(PN)^2 -> C Codegenerator                                              */
/*                                                                         */
/*  Johannes Jaeger --- Sommer 1997                                        */
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include "frontend.h"
#include "error.h"
#include "tree_handling.h"
#include "misc.h"
#include "includes.h"
#include "symboltable.h"
#include "var_mem_manage.h"

/*------------------------------------------------------------------------*/

#define IWRITE(X) fprintf(H_File, X)
#define IWRITE1(X, Y) fprintf(H_File, X, Y)
#define IWRITE4(X, Y, Z, U, V) fprintf(H_File, X, Y, Z, U, V)
#define IWRITE5(X, Y, Z, U, V, W) fprintf(H_File, X, Y, Z, U, V, W)

#define WRITE(X) fprintf(C_File, X)
#define WRITE1(X, Y) fprintf(C_File, X, Y)
#define WRITE2(X, Y, Z) fprintf(C_File, X, Y, Z)
#define WRITE3(X, Y, Z, U) fprintf(C_File, X, Y, Z, U)
#define WRITE4(X, Y, Z, U, V) fprintf(C_File, X, Y, Z, U, V)
#define WRITE5(X, Y, Z, U, V, W) fprintf(C_File, X, Y, Z, U, V, W)

#define FWRITE(F, X) fprintf(F, X)
#define FWRITE1(F, X, Y) fprintf(F, X, Y)

/*------------------------------------------------------------------------*/
/* Definition of semantic node stack to store assignments out of
   the equations of the boolean expression of the atomic commands.
*/
#define stack_size 100

typedef SemNode*  STACK;


static void generate_expr(FILE*, SemNode*);
static void tree2c_code(SemNode* tree);


/************************** SemNode Stack ROUTINES *******************/
 
STACK sem_node_stack[stack_size];
int   stack_ptr = 0;

void init_stack(void)
/* Initializes the stack 
*/
{
  stack_ptr = 0;
}

/*------------------------------------------------------------------------*/
void push_sem_node(SemNode* element)
/* Pushs a semantic node on the stack.
  element : pointer to semantic node containing an assignment
*/
{
  if (stack_ptr == stack_size)
    runtime_error(ERR_STACK_FULL);
  sem_node_stack[stack_ptr++] = element;
}

/*------------------------------------------------------------------------*/
SemNode* pop_sem_node(int* is_empty)
/* This function returns the last element and pops it, in case of an empty 
   stack the flag is set on TRUE.
   is_empty : pointer to a flag
*/
{
  (*is_empty) = (stack_ptr == 0);
  if (! (*is_empty))
    return sem_node_stack[--stack_ptr];
  else
    return (SemNode*) NULL;
}


/************************ ROUTINES for the LOOP Construct **************/

int act_numbers_of_proc_list,
    act_is_c_call;

/*------------------------------------------------------------------------*/
void lookup_gc_command(SemNode* expr)
/* This functions searchs recursivlly for the possible guarded 
   commands in one alternative. These guards are enumerated in c-file
   as possible partners for choice.
   expr : pointer to semantic node containing one alternative
*/
{
  if (expr == (SemNode*) NULL)
    return;
  switch (expr->node_operator) 
  { 
    case BEGIN_ :
    case SEQ_   :
      lookup_gc_command(expr->left);
      break;
    case PARA_  :
      lookup_gc_command(expr->left);
      lookup_gc_command(expr->right);
      break;
    case DO_    :
      lookup_gc_command(expr->left);
      break;
    case IDENT_ :
      if (act_numbers_of_proc_list > 0)
	break;
      else
      {
	act_is_c_call = TRUE;
	break;
      }
    case EXPR_BEGIN_ :
      act_is_c_call = FALSE;
      if (act_numbers_of_proc_list > 0)
	WRITE(", ");
      act_numbers_of_proc_list++;
      WRITE1("{ check_%s, ", expr->info_text);
      WRITE1(" \"%s\" , ", expr->info_text);
      if (is_empty(expr->left->semantic.expr_constraint.channel_0_input) &&
	  is_empty(expr->left->semantic.expr_constraint.channel_0_output))
	WRITE(" FALSE, (char*) NULL, (char*) NULL, (char*) NULL }");
      else
      {
	WRITE1(" TRUE, \"%s\",", 
	       str_set(expr->left->semantic.expr_constraint.channel_0_input));
	WRITE1("\"%s\",",
	       str_set(expr->left->semantic.expr_constraint.channel_0_output));
	WRITE1("\"%s\" }", expr->right->info_text);
      }
      break;
  }
}

	
unsigned int gc_size;

char* init_string;

/*------------------------------------------------------------------------*/
void write_alt_set(SemNode* expr)
/* This function generates c-code for an alternative and prepares
   the guarded command list for the choice.
   expr : pointer to an alternative
*/
{
  char* dummy1,
        dummy2[200];

  gc_size++;
  WRITE1("EXPR_CHECK e_check%u[] = {\n", gc_size);
  act_numbers_of_proc_list = 0;
  act_is_c_call = FALSE;
  lookup_gc_command(expr->left);
  if (act_numbers_of_proc_list == 0)
    WRITE("{NULL, NULL, FALSE, NULL, NULL, NULL}");
  WRITE ("};\n");
  sprintf(dummy2, "  { NULL , NULL, NULL, FALSE, %d, %d, %d, 0 }\n",
	         (expr->right->node_operator == EXIT_) ? FALSE : TRUE, 
        	  act_is_c_call, act_numbers_of_proc_list);
  dummy1 = (char*) Malloc(sizeof(char) *
			  (1 + strlen(dummy2) + strlen(init_string)));
  if (gc_size != 1)
    dummy2[0] = ',';
  sprintf(dummy1, "%s%s", init_string, dummy2);
  free(init_string);
  init_string = (char*) Malloc(sizeof(char) * 
			       (1 + strlen(dummy1)));
  (void)strcpy(init_string, dummy1);
  free(dummy1);
}
	
/*------------------------------------------------------------------------*/
void write_choice_construct(SemNode* expr)
/* This function generates all alternatives of the loop-body.
   expr : pointer to the choice construct of the loop.
*/
{
  if (expr == (SemNode*) NULL)
    return;
  if (expr->node_operator == SEQ_)
    write_alt_set(expr);
  else
  {
    SemNode* dummy = expr;
    while (dummy->node_operator == CHOICE_)
    {
      write_alt_set(dummy->left);
      dummy = dummy->right;
    }
    write_alt_set(dummy);
  }
}
	
/*------------------------------------------------------------------------*/
void generate_do_construct(SemNode* expr)
/* This function generates the whole code for a loop construct.
   expr : pointer to do construct
*/
{
  unsigned int	i;

  WRITE("{\n");
  tree2c_code(expr->left);
  WRITE("};\n");
  WRITE("{\n");
  WRITE("	int goon = TRUE,\n");
  WRITE("	    goon_wait = TRUE,\n");
  WRITE("	    alt_set_numb;\n");
  gc_size = 0;
  init_string = Malloc(sizeof(char));
  init_string[0] = '\0';
  write_choice_construct(expr->right);
  WRITE1("\n guarded_control	gc[] = {\n %s };\n", init_string);
  free(init_string);
  WRITE1("int    gc_size = %d,\n",(int)gc_size);
  WRITE("        i;\n");
  for (i = 0; i < gc_size; i++)
  {
    WRITE2("  gc[%u].expr_list = e_check%u;\n",i, i+1);
    WRITE1(
	   "  gc[%u].pre=allocate_local_memory_for_copy(get_memory_table_size());\n", i);
    WRITE1(
"  gc[%u].post=allocate_local_memory_for_copy(get_memory_table_size());\n", i);
    WRITE1("  gc[%u].is_possible = FALSE;\n",i);
  }
  WRITE(" do {\n");
  WRITE("     goon_wait = TRUE;\n");
  WRITE("     do {\n");
  WRITE("	sem_wait(synchro_);\n");
  WRITE("	sem_wait(table_);\n");
  for (i=0;i<gc_size;i++)
    WRITE1("       gc[%u].is_possible = FALSE;\n",i);
  WRITE("  Testing = TRUE;\n");
  WRITE("	lookup_do_construct(gc, gc_size);\n");
  WRITE("  Testing = FALSE;\n");
  WRITE(
"	alt_set_numb = choose_one_possible_alt_set(gc, gc_size);\n"); /*checked muss noch hierher */
  WRITE("	if (alt_set_numb != -1)\n");
  WRITE("	  {goon = gc[alt_set_numb].goon_further;\n");
  WRITE("	   if ((! gc[alt_set_numb].is_c_call) &&\n");
  WRITE(
"! (gc[alt_set_numb].expr_list[gc[alt_set_numb].checked_bool_proc].is_channel_0_command))\n");
  WRITE("   { ExecGuard(gc, alt_set_numb);\n");
  WRITE(
"           rewrite_copies(gc[alt_set_numb].pre, gc[alt_set_numb].post);\n");
  WRITE("             sem_signal(table_);\n");
  WRITE("	     goon_wait = FALSE;\n");
  WRITE("             }\n");
  WRITE(" 	   else\n");
  WRITE(" 	     {sem_signal(table_);\n");
  WRITE("	      goon_wait = FALSE;\n");
  WRITE("	     };\n");
  WRITE("	  }\n");
  WRITE("	else\n");
  WRITE("	  {sem_signal(table_);\n");
  WRITE("	   sem_signal(synchro_);\n");
  WRITE("	  };\n");
  WRITE("	} while (goon_wait);\n");
  WRITE(
"if(gc[alt_set_numb].expr_list[gc[alt_set_numb].checked_bool_proc].is_channel_0_command)\n"
              );
  WRITE("	prepare_account(account_, gc, alt_set_numb);\n");
  WRITE("    else if (! gc[alt_set_numb].is_c_call)\n");
  WRITE("      (void) strcpy(account_, gc[alt_set_numb].expr_list[gc[alt_set_numb].checked_bool_proc].proc_name);\n");
  WRITE("    else \n");
  WRITE("      (void) strcpy(account_, \"\");\n");
  WRITE("    sem_signal(synchro_);\n");
  WRITE("    switch (alt_set_numb) \n");
  WRITE("    {\n");
  if (expr->right->node_operator == SEQ_)
  {
    WRITE("	case 0 : {\n");
    tree2c_code(expr->right->left);
    WRITE("		 };\n");
    WRITE("              break;\n");
  }
  else
  {
    int choice_counter = 0;
	
    expr = expr->right;
    while (expr->node_operator == CHOICE_)
    {
      WRITE1("	case %d : {\n", choice_counter++);
      tree2c_code(expr->left->left);
      WRITE("		  };\n");
      WRITE("		  break;\n");
      expr = expr->right;
    }
    WRITE1("	case %d : {\n", choice_counter++);
    tree2c_code(expr->left);
    WRITE("		  };\n");
    WRITE("		  break;\n");
  }
  WRITE("    };\n");
  WRITE("  } while (goon);\n");	
  WRITE("};\n");
}		
 
/*------------------------------------------------------------------------*/
void generate_assign_expr(void)
/* This function generates the assignment segments for an atomic command.
   All assignments are pushed on the SemNode stack.
*/
{
  SemNode* stack_element;
  int is__empty = FALSE;

  do
  {
    stack_element = pop_sem_node(&is__empty);
    if (is__empty)
      return;
    if (stack_element->node_operator == EQU_)
    {
      if (strcmp(stack_element->left->info_text, "X!") == 0)
	if (is_empty(stack_element->left->semantic.
		     expr_constraint.channel_0_output)
	   )
	{
	  IWRITE1("push_fifo(post, %d, \n", 
		  stack_element->left->node_operator);
	  generate_expr(H_File, stack_element->right);
	  IWRITE("\n , flag);\n ");
	}
	else
	{
	  IWRITE1("set_synchro(pre, post, %d, \n", 
		  stack_element->left->node_operator);
	  generate_expr(H_File, stack_element->right);
	  IWRITE("\n , flag);\n ");
	}
      else if (strcmp(stack_element->left->info_text, "X'") == 0)
      {
	IWRITE1("set_post_primed_value(post, %d, \n", 
		stack_element->left->node_operator);
	generate_expr(H_File, stack_element->right);
	IWRITE("\n, flag);\n ");
      }
      else if (strcmp(stack_element->left->info_text, "X") == 0)
      {
	IWRITE1("set_post_primed_value(post, %d, \n",  
		stack_element->left->node_operator);
	generate_expr(H_File, stack_element->right);
	IWRITE("\n, flag);\n ");
	IWRITE("result = result && (( ");
	generate_expr(H_File, stack_element->right);
	IWRITE1(" ) == get_pre_primed_value(pre, %d, flag ));", 
		stack_element->left->node_operator);
      }
    }
    else
      IWRITE("/* undefined */ \n");
  } 
  while (!is__empty);
}

/*------------------------------------------------------------------------*/
static void generate_expr(FILE* f, SemNode* expr)
/* This function generates code for the boolean expression of an atomic
   command.
   expr : pointer to the semantic node containing atomic command
*/
{
  if (strcmp(expr->info_text, "C") == 0)
  {
    FWRITE1(f, "%d", expr->node_operator);
    return;
  }
  if (strcmp(expr->info_text, "'X") == 0)
  {
    FWRITE1(f, "get_pre_primed_value(pre, %d, flag)\n", expr->node_operator);
    return;
  }
  if (strcmp(expr->info_text, "X?") == 0)
  {
    if (is_empty(expr->semantic.expr_constraint.channel_0_input))
    {
      FWRITE1(f,"top_fifo(pre, %d, flag)\n", expr->node_operator);
      return;
    }
    else
    {
      FWRITE1(f,"get_synchro(pre, post, %d, flag)\n", expr->node_operator);
      return;
    }
  }

  /* jj neu */
  if (strcmp(expr->info_text, "X??") == 0) {
    if (is_empty(expr->semantic.expr_constraint.channel_0_input))
    {
      FWRITE1(f,"empty_check(get_addr(%d))\n", expr->node_operator);
      return;
    }
  }

  if (strcmp(expr->info_text, "X!!") == 0) {
    if (is_empty(expr->semantic.expr_constraint.channel_0_input))
    {
      FWRITE1(f,"full_check(get_addr(%d))\n", expr->node_operator);
      return;
    }
  }
  /* jj neu Ende */

  if ( (strcmp(expr->info_text, "X'")  == 0) ||
       (strcmp(expr->info_text, "X!")  == 0) ||
       (strcmp(expr->info_text, "X")   == 0) )
    return;

  switch (expr->node_operator)
  {
    case AND_ :
      FWRITE(f, " ( ");
      generate_expr(f, expr->left);
      FWRITE(f, " && ");
      generate_expr(f, expr->right);
      FWRITE(f, " )\n ");
      return;
    case OR_  :
      FWRITE(f, " ( ");
      generate_expr(f, expr->left);
      FWRITE(f, " || ");
      generate_expr(f, expr->right);
      FWRITE(f, " ) \n");
      return;
    case NOT_ :
      FWRITE(f, " ( ! ");
      generate_expr(f, expr->left);
      FWRITE(f, " )\n ");
      return;
    case EQU_ :
      if (expr->semantic.expr_constraint.is_const)
      {
	FWRITE(f, " ( ");
	generate_expr(f, expr->left);
	FWRITE(f, " == ");
	generate_expr(f, expr->right);
	FWRITE(f,  ") \n");
	return;
      }
      else
      {
	push_sem_node(expr);
	FWRITE(f, "1");
	return;
      }
    case UNEQU_ :
      FWRITE(f, " ( ");
      generate_expr(f, expr->left);
      FWRITE(f, " != ");
      generate_expr(f, expr->right);
      FWRITE(f, " ) \n");
      return;
    case LESS_ :
      FWRITE(f, " ( ");
      generate_expr(f, expr->left);
      FWRITE(f, " < ");
      generate_expr(f, expr->right);
      FWRITE(f, " ) \n");
      return;
    case GREA_ :
      FWRITE(f, " ( ");
      generate_expr(f, expr->left);
      FWRITE(f, " > ");
      generate_expr(f, expr->right);
      FWRITE(f, " ) \n");
      return;
    case EGREA_ :
      FWRITE(f, " ( ");
      generate_expr(f, expr->left);
      FWRITE(f, " >= ");
      generate_expr(f, expr->right);
      FWRITE(f, " ) \n");
      return;
    case ELESS_ :
      FWRITE(f, " ( ");
      generate_expr(f, expr->left);
      FWRITE(f, " <= ");
      generate_expr(f, expr->right);
      FWRITE(f, " ) \n");
      return;
    case PLUS_ :
      FWRITE(f, " ( ");
      generate_expr(f, expr->left);
      FWRITE(f, " + ");
      generate_expr(f, expr->right);
      FWRITE(f, " ) \n ");
      return;
    case MINUS_ :
      FWRITE(f, " ( ");
      generate_expr(f, expr->left);
      FWRITE(f, " - ");
      generate_expr(f, expr->right);
      FWRITE(f, " ) \n");
      return;
    case MULT_ :
      FWRITE(f, " ( ");
      generate_expr(f, expr->left);
      FWRITE(f, " * ");
      generate_expr(f, expr->right);
      FWRITE(f, " ) \n");
      return;
    case DIV_ :
      FWRITE(f, " ( ");
      generate_expr(f, expr->left);
      FWRITE(f, " / ");
      generate_expr(f, expr->right);
      FWRITE(f, " ) \n");
      return;
    case MOD_ :
      FWRITE(f, " ( ");
      generate_expr(f, expr->left);
      FWRITE(f, " %% ");
      generate_expr(f, expr->right);
      FWRITE(f, " ) \n");
      return;
    case UNARYMINUS_ :
      FWRITE(f, " ( - ");
      generate_expr(f, expr->left);
      FWRITE(f, " ) \n");
      return;
    default   : return;
  }
}


/*------------------------------------------------------------------------*/
void generate_atomic_c_code(SemNode* atomic)
/* The function generates the code for the an atomic command.
   atomic : pointer to the semantic node containing the atomic command
*/
{
  if (atomic == (SemNode*) NULL)
    return;
  if (atomic->node_operator != EXPR_BEGIN_)
    return;

  IWRITE1("int check_%s(container* pre, container* post, int* flag)\n{\n", 
	  atomic->info_text);
  IWRITE("  int result;\n\n");
  IWRITE("  result = ");
  init_stack();
  generate_expr(H_File, atomic->left);
  IWRITE(";\n");
  generate_assign_expr();
  IWRITE("\n");
  /* falls globale Variable Testing gleich FALSE,  */
  /* Referenzwerte der atomaren Aktion in PMsg ein-*/
  /* tragen */
  IWRITE("  if (result && (*flag) && (!Testing))\n");
  IWRITE5("    PMsg.refs = AddRefListElem(PMsg.refs, %d, %d, %d, %d, %d);\n\n",
	   atomic->ref.start_l,
	   atomic->ref.start_c,
	   atomic->ref.end_l,
	   atomic->ref.end_c,
	   atomic->ref.RefNum
         );
  IWRITE("  return result;\n}\n");

  if (is_empty(atomic->left->semantic.expr_constraint.channel_0_input) &&
      is_empty(atomic->left->semantic.expr_constraint.channel_0_output))
  {
    WRITE("{ /* BEGIN ATOMIC C-CODE */ \n");
    WRITE("  container* pre,\n");
    WRITE("           * post;\n");
    WRITE("  int access_ok = TRUE,\n");
    WRITE("	 result,\n");
    WRITE("	 goon = TRUE;\n\n");
    WRITE("  sem_wait (synchro_);\n");
    WRITE1("  if (strcmp(\"%s\", account_) == 0)\n", atomic->info_text);
    WRITE("  {(void) strcpy (account_, \"\"); sem_signal(synchro_);}\n");
    WRITE("	else \n");
    WRITE("	{sem_signal(synchro_);\n");
    WRITE(
"	pre = allocate_local_memory_for_copy (get_memory_table_size () );\n");
    WRITE(
"	post= allocate_local_memory_for_copy (get_memory_table_size () );\n");
    WRITE("	do { \n");
    WRITE1("	lock (table_, set_str(\"%s\"));\n", atomic->right->info_text);
    WRITE("	if (strlen (account_) != 0)\n");
    WRITE1("	  {unlock (table_, set_str(\"%s\"));\n",
		 atomic->right->info_text);
    WRITE("	   (void) sleepy(1);\n");
    WRITE("	   continue;\n");
    WRITE("	  };\n");
    WRITE("	get_pre_primed_variables (pre);\n");
    WRITE("	get_post_primed_variables (post);\n");
    WRITE("	access_ok = TRUE;\n");
    WRITE1("	result = check_%s (pre, post, &access_ok);\n",
	   atomic->info_text);
    WRITE("	if (result && access_ok) {\n");
    WRITE("	  rewrite_copies(pre, post);\n");
    WRITE1("	  unlock (table_, set_str(\"%s\"));\n",
	   atomic->right->info_text);
    WRITE("	  deallocate_local_memory_copy (pre);\n");
    WRITE("	  deallocate_local_memory_copy (post);\n");
    WRITE("	  goon = FALSE;\n");
    WRITE("	}\n");
    WRITE("	else\n");
    WRITE1("	  {unlock (table_, set_str(\"%s\"));\n",
	   atomic->right->info_text);
    WRITE("	   (void) sleepy (1);\n");
    WRITE("	  }\n");
    WRITE("	}\n");
    WRITE("	while (goon);\n");
    WRITE("	};\n");
    WRITE("}; /* END ATOMIC C-CODE */\n");
  }
  else   /*------------------------------------------------*/
  {
    char* up_name = (char*) Malloc(sizeof(char) * 
				   (strlen(atomic->info_text) + 1));
    (void)strcpy(up_name, atomic->info_text);
    up_name = upper_name (up_name);

    WRITE("{ /* BEGIN ATOMIC C-CODE */\n");
    WRITE("	container* pre,\n");
    WRITE("	         * post;\n");
    WRITE("	sequence * s;\n");
    WRITE("	SET	   process_vars;\n");
    WRITE("	int	pid;\n");

    WRITE1("     SET InVars = set_str (\"%s\");\n",
	   str_set(atomic->left->semantic.expr_constraint.channel_0_input));
    WRITE1("     SET OutVars = set_str (\"%s\");\n",
	   str_set(atomic->left->semantic.expr_constraint.channel_0_output));
    WRITE1("     SET AllVars = set_str (\"%s\");\n", atomic->right->info_text);

    WRITE("     while (bswlist) {\n");
    WRITE("       if (is_element(InVars, bswlist->oldbucket)) {\n");
    WRITE("         delete_element(InVars, bswlist->oldbucket);\n");
    WRITE("         insert_element(InVars, bswlist->newbucket);\n");
    WRITE("       }\n");
    WRITE("       if (is_element(OutVars, bswlist->oldbucket)) {\n");
    WRITE("         delete_element(OutVars, bswlist->oldbucket);\n");
    WRITE("         insert_element(OutVars, bswlist->newbucket);\n");
    WRITE("       }\n");
    WRITE("       if (is_element(AllVars, bswlist->oldbucket)) {\n");
    WRITE("         delete_element(AllVars, bswlist->oldbucket);\n");
    WRITE("         insert_element(AllVars, bswlist->newbucket);\n");
    WRITE("       }\n");
    WRITE("       bswlist = bswlist->next;\n");
    WRITE("     } /* while (bswlist) */\n\n");

    WRITE("	sem_wait (synchro_);\n");
    WRITE1("if (strstr (account_, \"%s\" ) != (char*) NULL)\n", atomic->info_text);
    WRITE("{\n");
    WRITE(" int pos;\n");

    WRITE1("	pid = insert_process (1, check_%s, \n", atomic->info_text);
    WRITE("		InVars,\n");
    WRITE("		OutVars,\n");
    WRITE("		AllVars);\n");

    WRITE1("pos = ((int) strstr (account_, \"%s\")) - ((int) account_);\n", atomic->info_text);
    WRITE1("strcpy (&(account_[pos]), &(account_[pos+strlen (\"%s\")]));\n", 
		atomic->info_text);
    WRITE("sem_signal(synchro_);\n");
    WRITE("	   while (process[pid].semaphore)\n");
    WRITE("	     sleepy (1);\n");
    WRITE("	   process[pid].is_used = FALSE;\n");
    WRITE("}\n");
    WRITE1("else if (strstr (account_, \"%s\") != (char*) NULL)\n", up_name);
    WRITE("{\n");
    WRITE1(" while (strcmp (account_, \"%s\") != 0)\n",up_name);
    WRITE("	{sem_signal (synchro_);sleepy(1);sem_wait (synchro_);};\n");

    WRITE1("	pid = insert_process (1, check_%s, \n", atomic->info_text);
    WRITE("		InVars,\n");
    WRITE("		OutVars,\n");
    WRITE("		AllVars);\n");

    WRITE("	strcpy (account_, \"\");\n");
    WRITE("	process_vars = lock_for_synchronisation (table_);\n");
    WRITE(
"	pre = allocate_local_memory_for_copy (get_memory_table_size () );\n");
    WRITE(
"	post= allocate_local_memory_for_copy (get_memory_table_size () );\n");
    WRITE("	s = get_synchronisation (pre, post, pid);\n");
    WRITE("	if (s != (sequence*) NULL) {\n");
    WRITE("	  rewrite_copies (pre, post);\n");
    WRITE("	  deallocate_local_memory_copy (pre);\n");
    WRITE("	  deallocate_local_memory_copy (post);\n");
    WRITE("	  execute_sequence (s);\n");
    WRITE("	  unlock_for_synchronisation (table_, process_vars);\n");
    WRITE(" 	  process[pid].is_used = FALSE;\n");
    WRITE("	  sem_signal (synchro_);\n");
    WRITE("	}\n");
    WRITE("	else {\n"); 
    WRITE("	  unlock_for_synchronisation (table_, process_vars);\n");
    WRITE("	  sem_signal (synchro_);\n");
    WRITE("	  while (process[pid].semaphore)\n");
    WRITE("	    sleepy (1);\n");
    WRITE("	  process[pid].is_used = FALSE;\n");
    WRITE("	};\n");
    WRITE("}\n");
    WRITE("else \n");
    WRITE("{\n");
    WRITE(" sem_signal (synchro_);\n");
    WRITE(" while (strcmp (account_, \"\") != 0);\n");
    WRITE(" sem_wait (synchro_);\n");
    WRITE(
"	pre = allocate_local_memory_for_copy (get_memory_table_size () );\n");
    WRITE(
"	post= allocate_local_memory_for_copy (get_memory_table_size () );\n");

    WRITE1("	pid = insert_process (1, check_%s, \n", atomic->info_text);
    WRITE("		InVars,\n");
    WRITE("		OutVars,\n");
    WRITE("		AllVars);\n");

    WRITE("	process_vars = lock_for_synchronisation (table_);\n");
    WRITE("	s = get_synchronisation (pre, post, pid);\n");
    WRITE("	if (s != (sequence*) NULL)\n");
    WRITE("	  {rewrite_copies (pre, post);\n");
    WRITE("	   deallocate_local_memory_copy (pre);\n");
    WRITE("	   deallocate_local_memory_copy (post);\n");
    WRITE("	   execute_sequence (s);\n");
    WRITE("	   unlock_for_synchronisation (table_, process_vars);\n");
    WRITE(" 	   process[pid].is_used = FALSE;\n");
    WRITE("	   sem_signal (synchro_);\n");
    WRITE("	  }\n");
    WRITE("	else\n"); 
    WRITE("	  {unlock_for_synchronisation (table_, process_vars);\n");
    WRITE("	   sem_signal (synchro_);\n");
    WRITE("	   while (process[pid].semaphore)\n");
    WRITE("	     sleepy (1);\n");
    WRITE("	   process[pid].is_used = FALSE;\n");
    WRITE("	  };\n");
    WRITE("}\n");
    WRITE("};\n");
  }
}

/*------------------------------------------------------------------------*/
void generate_process_splitting(SemNode* tree)
/* This function generates c-code for the parallel construct.
   tree : pointer to the semantic node containing both branchs of 
          the construct
*/
{
  if (tree->node_operator != PARA_)
    return;
  WRITE("{\n");
  WRITE("  pid_t child1,\n");
  WRITE("        child2;\n");
  WRITE("  int first_at_first,\n");
  WRITE("      i,\n");
  WRITE("      status1,\n");
  WRITE("      status2;\n\n");
  WRITE("  first_at_first = (rand() %% 2);\n ");
  WRITE("  for (i = 0; i < 2; i++)\n");
  WRITE("  {\n");
  WRITE("    if (first_at_first) \n ");
  WRITE("    {\n      if ((child1 = fork() ) < 0)\n");
  WRITE("        runtime_error(ERR_FORK_DOES_NOT_WORK);\n");
  WRITE("      else if (child1 == 0)\n");
  WRITE("      {\n");
  /* Signalhandler auf Defaultwerte setzen */
  WRITE("        DeInstallSigHandler();\n");
  /* globale Variable PMsg initialisieren */
  WRITE("        InitPMsg();\n");
  /* globale Variable Testing auf FALSE setzen */
  WRITE("        Testing = FALSE;\n");
  tree2c_code(tree->left);
  WRITE("	 exit(EXIT_SUCCESS);\n");
  WRITE("      }\n");
  WRITE("    }\n ");
  WRITE("    else\n ");
  WRITE("    {\n      if ((child2 = fork() ) < 0)\n");
  WRITE("	 runtime_error(ERR_FORK_DOES_NOT_WORK);\n");
  WRITE("      else if (child2 == 0)\n");
  WRITE("      {\n");
  /* Signalhandler auf Defaultwerte setzen */
  WRITE("        DeInstallSigHandler();\n");
  /* globale Variable PMsg initialisieren */
  WRITE("        InitPMsg();\n");
  /* globale Variable Testing auf FALSE setzen */
  WRITE("        Testing = FALSE;\n");
  tree2c_code(tree->right);
  WRITE("	 exit(EXIT_SUCCESS);\n");
  WRITE("      }\n");
  WRITE("    }\n");
  WRITE("    first_at_first = !first_at_first;\n");
  WRITE("  }\n");
  WRITE("  while ((waitpid(child1, &status1, 0) == -1) && (errno==EINTR))\n");
  WRITE("    ;\n");
  WRITE("  while ((waitpid(child2, &status2, 0) == -1) && (errno==EINTR))\n");
  WRITE("    ;\n");
  WRITE("  if ( (WEXITSTATUS(status1)!=0) || (WEXITSTATUS(status2)!=0))\n");
  WRITE("    exit(EXIT_FAILURE);\n");
  WRITE("}\n");
}

/*------------------------------------------------------------------------*/
void generate_proc_call (SemNode* call)
/* This function generates c-code for a procedure call.
  call : pointer to a semantic node containing the procedure call
*/
{
  SemNode* formal_param_list = call->left;
  char*	proc_type; 
  int hash_number;
  unsigned int i;

  ProcHead *PHead;

  if (call == (SemNode*) NULL)
    return;
  WRITE("{ /* BEGIN PROC_CALL */\n");
  WRITE("  container* pre;\n");
  WRITE("  int*	   flag = (int*) Malloc(sizeof(int));\n");
  WRITE("  Reference ref;\n\n");
  /* Referenzwerte des Prozeduraufrufs */
  /* in ref eintragen */
  WRITE5("  SetRef(&ref, %d, %d, %d, %d, %d);\n",
	 call->ref.start_l, call->ref.start_c,
	 call->ref.end_l, call->ref.end_c, call->ref.RefNum);
  WRITE("  pre = allocate_local_memory_for_copy(get_memory_table_size() );\n");
  WRITE("  get_pre_primed_variables(pre);\n");
  WRITE("  *flag = TRUE;\n");

  /*----------------------------------------*/
  PHead = FindProcHead(Procedures, call->info_text );
  if (PHead->MaxCalls > 1) {
    WRITE("{\n  int inst;\n\n");
    WRITE("  sem_wait(funccall_);\n");
    WRITE1("  for(inst = 0; inst < %d; inst++)\n", PHead->MaxCalls);
    WRITE1("    if ( (*%s_array[inst].isrunning) == 0) {\n", PHead->ProcName);
    WRITE1("      (*%s_array[inst].isrunning) = 1;\n", PHead->ProcName);
    WRITE("      break;\n");
    WRITE("    }\n");
    WRITE("  sem_signal(funccall_);\n");
    /* falls _max_ zu klein, Fehlermeldung */
    WRITE1("  if (inst == %d) {\n", PHead->MaxCalls);
    WRITE1("    (void)fprintf(stderr, \"\\nERROR: procedure %s\\n\");\n",
	   PHead->ProcName);
    WRITE("    runtime_error(ERR_MAX_TOO_SMALL);\n");
    WRITE("  }\n");
    /* Prozeduraufruf mit Referenz protokollieren */
    WRITE3("  PCall2Prot(\"%s\",ref, %d, %d);\n",
	   call->info_text, PHead->RefNum, PROC_CALL);
    WRITE2("  %s_array[inst].%sfunc(\n", PHead->ProcName, PHead->ProcName);
  }
  else {
    /* Prozeduraufruf mit Referenz protokollieren */
    WRITE3("  PCall2Prot(\"%s\",ref, %d, %d);\n",
	   call->info_text, PHead->RefNum, PROC_CALL);
    WRITE1("  %s(", call->info_text);
  }
  hash_number = call->right->node_operator;
  proc_type = (char*) Malloc( (strlen(get_procedure_type(hash_number)) + 1) 
			      * sizeof(char));
  (void)strcpy(proc_type, get_procedure_type(hash_number));	
  for (i = 0; i < strlen(proc_type); i++)
  {
    switch (((int) proc_type[i]) - ((int) '0')) 
    {
      case 0 /* FORMAL_CONST_BOOL */ :
      case 1 /* FORMAL_CONST_INT  */ : 
	generate_expr(C_File, formal_param_list->left);
	formal_param_list = formal_param_list->right;
	break;
      case 2 /* FORMAL_VAR_BOOL */ :
      case 3 /* FORMAL_VAR_INT  */ :
      case 4 /* FORMAL_CHAN     */ :
	WRITE1("%d", formal_param_list->left->node_operator);
	formal_param_list = formal_param_list->right;
	break;
    }
    if (i+1 != strlen(proc_type))
      WRITE(" , ");
  }

  if (PHead->MaxCalls > 1)
    WRITE(");\n}\n");
  else
    WRITE(");\n\n");

    /* Rueckkehr vom Prozeduraufruf mit Referenz protokollieren */
    WRITE3("  PCall2Prot(\"%s\",ref, %d, %d);\n",
	   call->info_text, PHead->RefNum, PROC_RETURN);
  /*----------------------------------------*/
  free(proc_type);
  WRITE("	deallocate_local_memory_copy(pre);\n");
  WRITE("	free(flag);\n");
  WRITE("} /* END PROC_CALL */\n");
}


/*------------------------------------------------------------------------*/
static void tree2c_code(SemNode* root)
/* This function chooses a segment translating function, depending on the
   actual semantic node in the parse tree.
   root : actual root of the parse tree
*/
{
  if (root == (SemNode*) NULL)
    return;
  switch (root->node_operator)
  {
    case EXPR_BEGIN_ :
      generate_atomic_c_code(root);
      break;
    case IDENT_      :
      generate_proc_call(root);
      break;
    case SEQ_	 :
      WRITE("{\n");
      tree2c_code(root->left);
      WRITE("};\n{");
      tree2c_code(root->right);
      WRITE("};\n");
      break;
    case PARA_	 :
      generate_process_splitting(root);
      break;
    case BEGIN_	 :
      WRITE("{\n");
      WRITE1(" re_initialize(set_str(\"%s\"), FALSE);\n", root->info_text);
      tree2c_code(root->left);
      WRITE1(" \nre_initialize(set_str(\"%s\"), TRUE);\n", root->info_text);
      WRITE("}\n");
      break;
    case DO_	 :
      generate_do_construct(root);
      break;
    default : break;
  }
}

/*------------------------------------------------------------------------*/
/* erzeugt C-Funktionen fuer alle B(PN)^2-Prozeduren */
static void generate_proc_decls(void)
{
  ProcHead* ProcHeads = Procedures;

  while (ProcHeads) {
    ProcEntry* ProcDecl = ProcHeads->Procs;
    list* Param;

    while (ProcDecl) {

	SET LockSet = nil_set();
	SET InitSet = set_str(ProcDecl->ProcBlock->info_text);
	int ref_pars = 0,
	    val_pars = 0,
	    res_pars = 0,
            i;

	Param = ProcDecl->ParList;


	/* ------------------------------------------*/
	/* Prozedur-Kopf erzeugen */
	WRITE1("\nvoid %s(", ProcDecl->ProcName);
	while (Param) {
	  if (Param->partype != _NOPAR) {
	    WRITE1(" int %s", Param->identifier);
	    insert_element(LockSet, Param->bucket);
	    delete_element(InitSet, Param->bucket);
	    if (Param->partype == _REF)
	      ref_pars++;
	    else {
	      if (Param->partype == _RES)
		res_pars++;
	      val_pars++;
	    }
	  }
	  Param = Param->next;
	  if (Param) {
	    if (Param->partype != _NOPAR)
	      WRITE(",");
	  }
	}
	WRITE(" )\n");

	ProcDecl->ProcBlock->info_text = str_set(InitSet);

	WRITE("{\n");
        WRITE("  BucketSwitch* bswlist = (BucketSwitch*) NULL;\n");
	if (ref_pars > 0) {
	  WRITE1("  int tmpindex[%d];\n", ref_pars);
	  WRITE1("  int savindex[%d];\n", ref_pars);
	}

#ifdef DEBUG
	WRITE1("fprintf(stderr, \"%%d -- BEGIN %s\\n\", (int)getpid());\n",
	       ProcDecl->ProcName);
#endif /* DEBUG */

	/* ------------------------------------------*/
	/* Index-Tausch fuer Referenz-Parameter */
	if (ref_pars > 0) {
	  WRITE("  sem_wait(table_);\n");
	  Param = ProcDecl->ParList;
	  i = 0;
	  while (Param) {
	    if (Param->partype == _REF) {
              /* alten Index sichern */
	      WRITE2("  savindex[%d] = index_switch[%d];\n",
		     i, Param->bucket);
              /* neuen Index zwischenspeichern */
	      WRITE2("  tmpindex[%d] = index_switch[%s];\n",
		     i, Param->identifier);
              WRITE2("    bswlist = addbuckets(bswlist, %d, %s );\n",
		     Param->bucket, Param->identifier);
	      i++;
	    }
	    Param = Param->next;
	  }

	  Param = ProcDecl->ParList;
	  i = 0;
	  while (Param) {
	    if (Param->partype == _REF) {
              /* neuen Index eintragen */
	      WRITE2("  index_switch[%d] = tmpindex[%d];\n",
		     Param->bucket, i);
	      i++;
	    }
	    Param = Param->next;
	  }
	  WRITE("  sem_signal(table_);\n");
	} /* if (ref_pars > 0) */

	/* ------------------------------------------*/

	/* lokale Variablen fr Value-Parameter initialisieren */
	if (val_pars > 0) {
	  WRITE("{\n");
	  WRITE("  container* pre,\n");
	  WRITE("           * post;\n");
	  WRITE("  int dummy;\n");
	  WRITE("  int goon = TRUE;\n\n");
	  WRITE("  sem_wait (synchro_);\n");
	  WRITE1("  if (strcmp(\"%s\", account_) == 0) {\n",
		 ProcDecl->ProcName);
	  WRITE("    (void) strcpy (account_, \"\"); sem_signal(synchro_);\n");
	  WRITE("  }\n  else {\n");
	  WRITE("    sem_signal(synchro_);\n");
	  WRITE(
    "    pre = allocate_local_memory_for_copy (get_memory_table_size () );\n");
	  WRITE(
    "    post= allocate_local_memory_for_copy (get_memory_table_size () );\n");


	  WRITE("    do { \n");
	  WRITE1("      lock (table_, set_str(\"%s\"));\n", str_set(LockSet));
	  WRITE("      if (strlen (account_) != 0) {\n");
	  WRITE1(
     "        unlock (table_, set_str(\"%s\"));\n",str_set(LockSet));
	  WRITE("        (void) sleepy(1);\n");
	  WRITE("        continue;\n");
	  WRITE("      };\n");
	  WRITE("      get_pre_primed_variables (pre);\n");
	  WRITE("      get_post_primed_variables (post);\n");

	  Param = ProcDecl->ParList;
	  while (Param) {
	    if  ( (Param->partype != _NOPAR) && (Param->partype != _REF) ) { 
	      if (Param->partype == _RES)
		WRITE2(
"      set_post_primed_value(post, %d, getpreval_noprot(pre, %s, &dummy) , &dummy);\n", 
                       Param->bucket, Param->identifier);
	      else
		WRITE2("      set_post_primed_value(post, %d, %s, &dummy);\n", 
			 Param->bucket, Param->identifier);
	    }
	    Param = Param->next;
	  }
#ifdef DEBUG
          /* Initialisierung der VAL-Parameter mit Referenzwerten */
	  /* des BEGIN-Statements des Prozedurblocks protokollieren */
          WRITE("      (void)strcpy(PMsg.Msg, \"initialization\");\n");
	  WRITE5("      PMsg.refs=AddRefListElem(PMsg.refs,%d,%d,%d,%d,%d);\n",
		ProcDecl->ProcBlock->ref.start_l,
		ProcDecl->ProcBlock->ref.start_c,
		ProcDecl->ProcBlock->ref.end_l,
		ProcDecl->ProcBlock->ref.end_c,
		ProcDecl->ProcBlock->ref.RefNum
	       );
          WRITE("      rewrite_copies(pre, post);\n");
#else /* not DEBUG */
	  WRITE ("      rewrite_copies_noprot(post);\n");
#endif /* DEBUG */
	  WRITE1("      unlock (table_, set_str(\"%s\"));\n",str_set(LockSet));
	  WRITE("      deallocate_local_memory_copy (pre);\n");
	  WRITE("      deallocate_local_memory_copy (post);\n");
	  WRITE("      goon = FALSE;\n");
	  WRITE("    }\n");
	  WRITE("    while (goon);\n");
	  WRITE("  }\n");
	  WRITE("}\n");
	} /* if (val_pars > 0) */
	/* ------------------------------------------*/
      
	WRITE1("  re_initialize(set_str(\"%s\"), FALSE);\n",
	       ProcDecl->ProcBlock->info_text);
	WRITE("\n");

        /* der Prozedurblock */
	tree2c_code(ProcDecl->ProcBlock->left);

	WRITE1("  \nre_initialize(set_str(\"%s\"), TRUE);\n",
	       ProcDecl->ProcBlock->info_text);

	/* ------------------------------------------*/
	/* ggf. Werte an Result-Parameter zutrckgeben */
	if (res_pars > 0) {
	WRITE("{\n");
	WRITE("  container* pre,\n");
	WRITE("           * post;\n");
	WRITE("  int dummy;\n");
	WRITE("  int goon = TRUE;\n\n");
	WRITE("  sem_wait (synchro_);\n");
	WRITE1("  if (strcmp(\"%s\", account_) == 0) {\n", ProcDecl->ProcName);
	WRITE("    (void) strcpy (account_, \"\"); sem_signal(synchro_);\n");
	WRITE("  }\n  else {\n");
	WRITE("    sem_signal(synchro_);\n");
	WRITE(
    "    pre = allocate_local_memory_for_copy (get_memory_table_size () );\n");
	WRITE(
    "    post= allocate_local_memory_for_copy (get_memory_table_size () );\n");


	WRITE("    do { \n");
	WRITE1("      lock (table_, set_str(\"%s\"));\n", str_set(LockSet));
	WRITE("      if (strlen (account_) != 0) {\n");
	WRITE1("        unlock (table_, set_str(\"%s\"));\n",str_set(LockSet));
	WRITE("        (void) sleepy(1);\n");
	WRITE("        continue;\n");
	WRITE("      };\n");
	WRITE("      get_pre_primed_variables (pre);\n");
	WRITE("      get_post_primed_variables (post);\n");

	Param = ProcDecl->ParList;
	while (Param) {
	  if (Param->partype == _RES) {
	    WRITE2(
"      set_post_primed_value(post, %s, get_pre_primed_value(pre, %d, &dummy), &dummy);\n",
		   Param->identifier, Param->bucket);
	  }
	  Param = Param->next;
	}

	WRITE ("      rewrite_copies_noprot(post);\n");
	WRITE1("      unlock (table_, set_str(\"%s\"));\n", str_set(LockSet));
	WRITE("      deallocate_local_memory_copy (pre);\n");
	WRITE("      deallocate_local_memory_copy (post);\n");
	WRITE("      goon = FALSE;\n");
	WRITE("    }\n");
	WRITE("    while (goon);\n");
	WRITE("  }\n");
	WRITE("}\n");
      } /* if (res_pars > 0) */

	/* ------------------------------------------*/
        /* alte Werte in index_switch bei */
        /* Referenzparametern eintragen   */
	if (ref_pars > 0) {
	  WRITE("  sem_wait(table_);\n");
	  Param = ProcDecl->ParList;
	  i = 0;
	  while (Param) {
	    if (Param->partype == _REF) {
	      WRITE2("  index_switch[%d] = savindex[%d];\n",
		     Param->bucket, i);
	      i++;
	    }
	    Param = Param->next;
	  }
	  WRITE("  sem_signal(table_);\n");
	} /* if (ref_pars > 0) */

	/* ------------------------------------------*/
        /* ggf. das Feld *isrunning auf FALSE setzen */
        if (ProcHeads->MaxCalls > 1) {
	  WRITE("{\n  int i;\n");
	  WRITE("  sem_wait(funccall_);\n");
	  WRITE1("  for (i = 0; i < %d; i++) {\n", ProcHeads->MaxCalls);
	  WRITE2("    if (strcmp(%s_array[i].name, \"%s\") == 0) {\n",
		ProcHeads->ProcName, ProcDecl->ProcName);
          WRITE1("      (*%s_array[i].isrunning) = 0;\n", ProcHeads->ProcName);
	  WRITE("       break;\n");
	  WRITE("     }\n");
	  WRITE("   }\n");
	  WRITE("  sem_signal(funccall_);\n");
	  WRITE("}\n");
	}

#ifdef DEBUG
	WRITE1("fprintf(stderr, \"%%d -- END %s\\n\", (int)getpid());\n",
	       ProcDecl->ProcName);
#endif /* DEBUG */

	WRITE("}\n");

	/* naechste Instanz einer Prozedur */
      ProcDecl = ProcDecl->NextProc;
    } /* while (ProcDecl) */

    /* eine neue Prozedur */
    ProcHeads = ProcHeads->NextProcHead;
  } /* while (ProcHeads) */
}

/*------------------------------------------------------------------------*/
/* gibt das Feld isrunning im shared memory frei */
static void free_proc_fields()
{
  ProcHead *PHead = Procedures;
  int i;

  while (PHead) {
    if (PHead->MaxCalls > 1) {
      for (i = 0; i < PHead->MaxCalls; i++ )
	WRITE2("  shm_free(%s_array[%d].isrunning, sizeof(int));\n",
	       PHead->ProcName, i);
    }
    PHead = PHead->NextProcHead;
  }
}

/*------------------------------------------------------------------------*/
/* Initialisierung der Funktionsarrays */
static void  generate_proc_init(void)
{
  ProcHead *PHead = Procedures;
  int i;
  char name[MAX_IDENT_LENGTH+4];
  char number[4];

  while (PHead) {
    if (PHead->MaxCalls > 1) {
      for (i = 0; i < PHead->MaxCalls; i++) {
	(void)sprintf(number, "%d", i+1);
	(void)strcpy(name, PHead->ProcName);
	(void)strcat(name, number);
        /* eine Funktion zuweisen */
	WRITE4("  %s_array[%d].%sfunc = %s;\n",
	       PHead->ProcName, i,  PHead->ProcName, name);
        /* deren Namen eintragen */
	WRITE3("  (void)strcpy(%s_array[%d].name, \"%s\");\n",
	      PHead->ProcName, i, name);
        /* isrunning im shared memory anlegen ... */
	WRITE2("  %s_array[%d].isrunning = (int*) shm_alloc(sizeof(int));\n",
	      PHead->ProcName, i);
        /* ... und initialisieren */
	WRITE2("  (*%s_array[%d].isrunning) = 0;\n", PHead->ProcName, i);

      }
    }
    PHead = PHead->NextProcHead;
  }
  WRITE("\n");
}

/*------------------------------------------------------------------------*/
/* erzeugt Typdeklarationen fuer C-Funktionen, */
/* falls mehrere Instanzen benoetigt werden    */
static void generate_proc_types(void)
{
  ProcHead *PHead = Procedures;
  int i;

  while (PHead) {
    if (PHead->MaxCalls > 1) {
      /* Funktionstyp definieren */
      WRITE1("typedef void (*%s_func)(", PHead->ProcName);
      if (PHead->NumParams > 0) {
	i = 0;
	while (i < PHead->NumParams - 1) {
	  WRITE("int, ");
	  i++;
	}
	WRITE("int");
      }
      WRITE(");\n");

      /* Definition einer Struktur und */
      /* eines Arrays dieser Struktur  */
      WRITE("typedef struct {\n");
      WRITE2("  %s_func %sfunc;\n", PHead->ProcName, PHead->ProcName);
      WRITE("  char name[MAX_IDENT_LENGTH];\n");
      WRITE("  int* isrunning;\n");
      WRITE1("} %s_rec;\n", PHead->ProcName);
      WRITE3("%s_rec %s_array[%d];\n", PHead->ProcName,
	     PHead->ProcName, PHead->MaxCalls);
    }
    PHead = PHead->NextProcHead;
  }
  WRITE("\n");
}

/*------------------------------------------------------------------------*/

void generate_c_code(SemNode* tree)
/* This function prepares the header of c-code file and the include
   file of the boolean functions.
*/
{
  OpenFile(&C_File, C_fname, "w");
  OpenFile(&H_File, H_fname, "w");

  IWRITE("/*----------------------------------------------------------*/\n");
  IWRITE("/*      THIS FILE IS MACHINE CREATED, DO NOT EDIT !!!       */\n");
  IWRITE("/*----------------------------------------------------------*/\n");
  IWRITE("/*      I N C L U D E S    f o r    B( P N ) ^ 2            */\n");
  IWRITE("/* Copyright Stephan Melzer            Project : PEP        */\n");
  IWRITE("/*----------------------------------------------------------*/\n");
  IWRITE("/*                     RELEASE 1.0                          */\n");
  IWRITE("/*----------------------------------------------------------*/\n\n");
  IWRITE1("#include \"lib%s.h\"\n\n", RUN_LIB_NAME);
  IWRITE("\nint max(int a, int b)\n");
  IWRITE("{\n  if (a > b)\n    return a;\n  else\n    return b;\n}\n");

/********************** C-Code FILE ********************************/

  WRITE("/*-----------------------------------------------------------*/\n");
  WRITE("/*       THIS FILE IS MACHINE CREATED, DO NOT EDIT !!!       */\n");
  WRITE("/*-----------------------------------------------------------*/\n");
  WRITE("/*       C  -  C O D E   f o r    B( P N ) ^ 2               */\n");
  WRITE("/* Copyright Stephan Melzer              Project : PEP       */\n");
  WRITE("/*-----------------------------------------------------------*/\n");
  WRITE("/*                      RELEASE 1.0                          */\n");
  WRITE("/*-----------------------------------------------------------*/\n\n");
  WRITE("#define _POSIX_SOURCE 1\n");
  WRITE("#define _POSIX_C_SOURCE 2\n");
  WRITE("#define __EXTENSIONS__\n\n");
  WRITE("/*--------------------- system headers ----------------------*/\n\n");
  WRITE("#include <unistd.h>\n");
  WRITE("#include <errno.h>\n");
  WRITE("#include <sys/types.h>\n");
  WRITE("#include <sys/ipc.h>\n");
  WRITE("#include <sys/wait.h>\n");
  WRITE("#include <sys/shm.h>\n");
  WRITE("#include <stdlib.h>\n");
  WRITE("#include <signal.h>\n");
  WRITE("#include <string.h>\n\n");
  WRITE("/*--------------------- local headers -----------------------*/\n\n");
  WRITE1("#include \"%s\"\n\n", H_fname);
  WRITE("/*--------------------- global varables ---------------------*/\n\n");
  WRITE("int table_    = TABLE_NUM,\n");
  WRITE("    synchro_  = SYNCHRO_NUM,\n");
  WRITE("    funccall_ = FUNCCALL_NUM;\n");
  WRITE("char* account_;\n");
  WRITE("pid_t main_pid;\n\n");
  WRITE("BucketSwitch* bswlist = (BucketSwitch*) NULL;\n");

  /* fuer B(PN)^2-Prozeduren mit mehreren Instanzen */
  /* Typdeklarationen erzeugen                      */
  generate_proc_types();

  WRITE("/*-------------------   procedures --------------------------*/\n\n");

  /* Funktionen fuer B(PN)^2-Prozeduren erzeugen */
  generate_proc_decls();

  /* Exit-Handler */
  WRITE("\nvoid ExitHandler(void)\n{\n");
  WRITE("  if (main_pid != getpid()) return;\n\n");

  free_proc_fields();

  WRITE("  CloseFile(&prot_file);\n");
  WRITE("  sem_delete();\n");
  WRITE("  shm_free(account_, 200 * sizeof(char));\n");
  WRITE("  kill_timer_routine();\n");
  WRITE("  erase_memory_table();\n");
  WRITE("  release_shm(stady_page);\n");
  WRITE("  release_shm(tmp_page);\n");
  WRITE("  release_shm_control();\n}\n\n");
  /* Ende Exit-Handler */

  /* Usage-Aisgabe */
  WRITE("void printusage(char *cmd)\n{\n");
  WRITE(
  "  (void)fprintf(stderr, \"Usage: %%s [-s] [-p file] [-h]\\n\", cmd);\n");
  WRITE(
  "  (void)fprintf(stderr, \"       -s      write protocol to stdout\\n\");\n");
  WRITE(
  "  (void)fprintf(stderr, \"       -p file write protocol to file\\n\");\n");
  WRITE(
  "  (void)fprintf(stderr, \"       -h      show this usage\\n\");\n");
  WRITE("  exit(EXIT_FAILURE);\n}\n\n");
  /* Ende Usage-Aisgabe */

  /* Kommandozeilen-Parser */
  WRITE("void parsecmdline(int argc, char **argv)\n{\n");
  WRITE("  char options[5] = \"+sp:h\";\n");
  WRITE("  int c;\n\n");
  WRITE("  while ( (c = getopt(argc, argv, options)) != -1)\n  {\n");
  WRITE("    switch(c)\n    {\n");
  WRITE("      case 's': proto2stdout = !proto2stdout;\n");
  WRITE("        break;\n");
  WRITE("      case 'p': proto2file = TRUE;\n");
  WRITE("        if (optarg[0] == '-') {\n");
  WRITE(
"          fprintf(stderr, \"%%s: illegal filename %%s\\n\", argv[0], optarg);\n");
  WRITE("          printusage(argv[0]);\n");
  WRITE("        }\n");
  WRITE("        (void) strcpy(Prot_fname, optarg);\n");
  WRITE("        break;\n");
  WRITE("      case 'h': printusage(argv[0]);\n");
  WRITE("        break;\n");
  WRITE("      case '?': printusage(argv[0]);\n");
  WRITE("        break;\n    }\n");
  WRITE("  }\n");
  WRITE("  if (optind < argc)\n  {\n");
  WRITE("    (void)fprintf(stderr,\"%%s: too many arguments\\n\",argv[0]);\n");
  WRITE("    printusage(argv[0]);\n  }\n");
  WRITE("}\n\n");
  /* Ende Kommandozeilen-Parser */

  WRITE("/*--------------------- main program ------------------------*/\n\n");
  WRITE("int main(int argc, char **argv)\n{\n");
  WRITE("  parsecmdline(argc, argv);\n");
  WRITE("  main_pid = getpid();\n");
  WRITE("  /* install the signal handler */\n");
  WRITE("  InstallSigHandler();\n");
  WRITE("  /* install the exit handler */\n");
  WRITE("  if (atexit(&ExitHandler) != 0) {\n");
  WRITE("    perror(\"main().atexit()\");\n");
  WRITE("    exit(EXIT_FAILURE);\n  }\n");
  WRITE("  initialize_protokoll();\n");
  WRITE("  /* create a semaphore structure */\n");
  WRITE("  /* with NUM_SEMS semaphores     */\n");
  WRITE("  sem_create();\n");
  WRITE("  initialize_shm_control();\n");
  WRITE("  srand( (unsigned int) getpid() ); \n ");

  /* B(PN)^2-Variablen initialisieren */
  add_hash_table(C_File);

  WRITE("  initialize_shm(stady_page);\n");
  WRITE("  switch(stady_page);\n");
  WRITE("  account_ = (char*) shm_alloc(200 * sizeof(char));\n");
  WRITE("  (void) strcpy(account_, \"\");\n");
  WRITE("  init_scheduler();\n");
  WRITE("  switch (stady_page);\n");
  WRITE("  init_timer_routine(synchro_, table_, &account_);\n\n");

  /* ggf. Funktionsarrays initialisieren */
  generate_proc_init();

  /* der Hauptblock eines B(PN)^2-Programms */
  tree2c_code(tree);

  WRITE("  return(EXIT_SUCCESS);\n");
  WRITE("}\n");

  CloseFile(&C_File);
  CloseFile(&H_File);
}

/*------------------------------------------------------------------------*/
/* End of file c_code.c */
/*------------------------------------------------------------------------*/
