/****************************************************************/
/*                                                              */
/*            C O D E - G E N E R A T O R  for B(PN)^2          */
/*                                                              */
/*            Universitaet Hildesheim                           */
/*            Institut fuer Informatik                          */
/*            Stephan Melzer                                    */
/****************************************************************/
/*            Diplomarbeit                                      */
/*            File        : error.c                          
              Version     : 1.0
	      Last Update : 18.10.1994
 ****************************************************************/
/*            Module information :                              */
/* This module contains the error handling for the compile-time
   libary and also for the run-time system.
*/
/****************************************************************/
/* Johannes Jaeger -- Sommer 1997 */

#include <stdlib.h>
#include <stdio.h>
#include "misc.h"


char* error_msg[] =
{
  "ERROR: process is unable to fork",
  "ERROR: SemNode-Stack_overflow",
  "ERROR: not enough memory",
  "ERROR: Cannot open file\n",
  "ERROR: value of _max_ in procedure declaration too small\n",
  "ERROR: cannot execute sigemptyset()\n",
  "ERROR: cannot execute sigaction()\n"
}; 

char* runtime_warn_msg[] =
{
  "WARNING: channel/stack is empty\n",
  "WARNING: channel/stack is full\n"};

char* warn_msg[] =
{
  "WARNING: do construct may be infinite\n"
};

char* compile_err_msg[] =
{
  "ERROR: post-primed variable inside expression\n",
  "ERROR: inside expression -> both sides of '=' are postprimed\n",
  "ERROR: illegal channel-0 parameter\n",
  "ERROR: too many ranges for variable(s) - check configuration\n" 
};

void warning(int warn_number)
{
  (void) fprintf(stderr, "%s", warn_msg[warn_number]);
}

void compile_error(int number)
{
  (void) fprintf(stderr, "%s", compile_err_msg[number]);
  exit(EXIT_FAILURE);
}

void exec_error(int err_number, const char *name)
/* Displays an error concerning the UNIX enviroment (ex. file access, ...
   err_number : error code
   name : file name concerning this error
*/
{
  fprintf(stderr, "%s %s\n",error_msg[err_number], name);
  exit(EXIT_FAILURE);
}
 
void runtime_error(int no)
/* This function interrupts with an error display.
   no : number of the runtime error
*/
{
  fprintf(stderr, "%s\n", error_msg[no]);
  exit(EXIT_FAILURE);
}

void warning_illegal_value(char name[], int val, Tscope scope)
/* This function displays a warning related to a range overflow.
   name : identifier of the variable
   val : overflow value
   scope : defined ranges of the variable
*/
{
  unsigned int j = 0;

  fprintf(stderr, "WARNING : Value %i of Variable %s not valid!\n",
 	           val, name);
  fprintf(stderr, "Range: {");

  if (scope.bounds[j].low < scope.bounds[j].high)
    fprintf(stderr, "%i..%i", scope.bounds[j].low, scope.bounds[j].high);
  else
    fprintf(stderr, "%i", scope.bounds[j].low);
  j++;

  while (j < scope.num_bounds) {
    if (scope.bounds[j].low < scope.bounds[j].high)
      fprintf(stderr, ",%i..%i", scope.bounds[j].low, scope.bounds[j].high);
    else
      fprintf(stderr, ",%i", scope.bounds[j].low);
    j++;
  } 

  fprintf(stderr, "}\n");
}

void runtime_warning(int number)
{
  (void) fprintf(stderr, "%s", runtime_warn_msg[number]);
}

/*-------------------------------------------------------------------------*/
/* End of file error.c */
/*-------------------------------------------------------------------------*/
