/****************************************************************/
/*                                                              */
/*            C O D E - G E N E R A T O R  for B(PN)^2          */
/*                                                              */
/*            Universitaet Hildesheim                           */
/*            Institut fuer Informatik                          */
/*            Stephan Melzer                                    */
/****************************************************************/
/*            Diplomarbeit                                      */
/*            File        : gc_control.c                        */
/*                                                              */
/*            Version     : 1.0                                 */
/*            Last Update : 18.10.1994                          */
/****************************************************************/
/*            Module information :                              */
/*
   This module contains the runtime function for the removal of
   the undeterminism of the choice construct. At runtime an active
   guarded command has to choosen.
*/
/****************************************************************/

#include <stdlib.h>
#include <string.h>
#include "gc_control.h"
#include "var_mem_manage.h"
#include "bits.h"
#include "scheduler.h"
#include "algebra.h"
#include "includes.h"
#include "misc.h"





void lookup_do_construct (guarded_control gc[], int gc_size)
/* This function checks all alternatives wether they are active. The 
   pre and post containers of each guard is set by its boolean function.
   gc : a list of guards
   gc_size : the size of the vector gc 
*/
{
  int i;

  for (i=0; i < gc_size; i++)
    if (gc[i].is_c_call)
      gc[i].is_possible = TRUE;
    else
    {
      int access_ok, result, check_counter = 0,
          *process_list = (int*) Malloc(sizeof(int) * gc[i].number_of_proc_list),
	  l;

      for (l=0; l < gc[i].number_of_proc_list; l++)
	if (gc[i].expr_list[l].is_channel_0_command)
	  process_list[l] = insert_process (0, gc[i].expr_list[l].bool_check,
                                            set_str (gc[i].expr_list[l].input_vars),
                                            set_str (gc[i].expr_list[l].output_vars),
                                            set_str (gc[i].expr_list[l].all_vars)    );
	else 
	  process_list[l] = -1;

     do
     {
       get_pre_primed_variables(gc[i].pre);
       get_post_primed_variables(gc[i].post);
       access_ok = TRUE;

       if (! gc[i].expr_list[check_counter].is_channel_0_command)
       {
	 result = (gc[i].expr_list[check_counter].bool_check)
	    (gc[i].pre, gc[i].post, &access_ok);
	 if ((!result) || (!access_ok))
	   check_counter++;
	 else
	 {
	   gc[i].is_possible = TRUE;
	   gc[i].checked_bool_proc = check_counter;
	   break;
	 }
       }
       else
       {
	 sequence* s;

	 s = get_synchronisation(gc[i].pre, gc[i].post, 
				 process_list[check_counter]);
	 if (s == (sequence*) NULL)
	   check_counter++;
	 else
	 {
	   gc[i].is_possible = TRUE;
	   gc[i].checked_bool_proc = check_counter;
	   break;
	 }
       }
     } while (check_counter < gc[i].number_of_proc_list);

     for (l=0; l < gc[i].number_of_proc_list; l++)
	if (gc[i].expr_list[l].is_channel_0_command)
	  (void)delete_process(process_list[l]); 

     free(process_list); 	
    }
}

int choose_one_possible_alt_set(guarded_control gc[], int gc_size)
/* This function chooses randomly one possible guarded command.
   If no alternative is active -1 is returned.
   parameters : see above
*/
{
  int possible_alt_set = 0,
      rand_number,
      i;

  for (i=0; i < gc_size; i++)
    if (gc[i].is_c_call || gc[i].is_possible)
      possible_alt_set++;

  if (possible_alt_set == 0)
    return -1;

  do 
    rand_number = rand() % gc_size;
  while (! (gc[rand_number].is_c_call || gc[rand_number].is_possible));

  return rand_number;
}



void prepare_account(char* account, guarded_control gc[], int alt_set_numb)
{
  int i;

  account[0] = '\0';

  for (i=0; i < gc[alt_set_numb].number_of_proc_list; i++)
    if (gc[alt_set_numb].expr_list[i].is_channel_0_command)
      account = strcat(account,
                       (i == gc[alt_set_numb].checked_bool_proc) ?
                       upper_name (gc[alt_set_numb].expr_list[i].proc_name) :
                       gc[alt_set_numb].expr_list[i].proc_name 
                      );
}

void ExecGuard(guarded_control gc[], int num)
{
  int ok = TRUE;

  get_pre_primed_variables(gc[num].pre);
  get_post_primed_variables(gc[num].post);
  (void)(gc[num].expr_list[gc[num].checked_bool_proc].bool_check)
	    (gc[num].pre, gc[num].post, &ok);
}
/*-------------------------------------------------------------------------*/
/*  end of file gc_control.c                                               */
/*-------------------------------------------------------------------------*/
