/*-------------------------------------------------------------------------*/
/*  local_sem.c                                                            */
/*-------------------------------------------------------------------------*/
/*  B(PN)^2 -> C Codegenerator                                              */
/*                                                                         */
/*  Johannes Jaeger --- Sommer 1997                                        */
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include "local_sem.h"
#include "misc.h"

/* Id der Semaphorstruktur */
int Sem_Id = -1;

static struct sembuf op_op[NUM_SEMS] = {
/*  sem_num, sem_op, sem_flg  */
  { TABLE_NUM,    0, 0},
  { SYNCHRO_NUM,  0, 0},
  { SEMPROT_NUM,  0, 0},
  { FUNCCALL_NUM, 0, 0}
};
/* Struktur zur Uebergabe an semop(),     */
/* der Wert op_op.sem_op wird jeweils bei */
/* Aufruf von sem_op() gesetzt            */

/*-------------------------------------------------------------------------*/
static void Error(char err_msg[])
{
  (void)fprintf(stderr, err_msg);
  exit(EXIT_FAILURE);
}

/*-------------------------------------------------------------------------*/
/* erzeugt eine Semaphorstruktur mit */
/* NUM_SEMS Semaphoren               */
void sem_create(void)
{
  int i;

#if defined(__GNU_LIBRARY__) && !defined(_SEM_SEMUN_UNDEFINED)
/* union semun is defined by including <sys/sem.h> */
  union semun semctl_arg;
#else
/* according to X/OPEN we have to define it ourselves */
  union semun {
	int val;                    /* value for SETVAL */
	struct semid_ds *buf;       /* buffer for IPC_STAT, IPC_SET */
	unsigned short int *array;  /* array for GETALL, SETALL */
	struct seminfo *__buf;      /* buffer for IPC_INFO */
  } semctl_arg;
#endif


  /* Semaphorstruktur anfordern       */
  /* IPC_PRIVATE : eine neue erzeugen */
  /* NUM_SEMS : Anzahl der Semaphore  */
  /* 0660 : Zugriffsrechte            */
  if ( (Sem_Id = semget(IPC_PRIVATE, NUM_SEMS, 0660)) == -1) {
    perror("local_sem.sem_create().semget()");
    Error("Cannot get semaphore structure\n");;
  }

  /* alle Semaphore mit 1 initialisieren */
  semctl_arg.array = (unsigned short*)Malloc(NUM_SEMS * sizeof(unsigned short));
  for (i = 0; i < NUM_SEMS; i++)
    semctl_arg.array[i] = 1;  
  if (semctl(Sem_Id, 0, SETALL, semctl_arg) == -1) {
    perror("local_sem.sem_create().semctl()");
    Error("Cannot initialize (SETALL) semaphores\n");
  }
  free((void*)semctl_arg.array);
}

/*-------------------------------------------------------------------------*/
/* generelle Semaphoroperation */
static void sem_op(int sem_num, short int value)
{
  op_op[sem_num].sem_op = value;
  if (semop(Sem_Id, &op_op[sem_num], 1) == -1){
  } 
}

/*-------------------------------------------------------------------------*/
/* die bekannten Operationen sem_wait() und sem_signal() */
void sem_wait(int sem_num)
{
  sem_op(sem_num, -1);
}

void sem_signal(int sem_num)
{
  sem_op(sem_num, 1);
}

/*-------------------------------------------------------------------------*/
/* die Semaphorstruktur entfernen */
void sem_delete(void)
{
  if (Sem_Id > -1) {
    if (semctl(Sem_Id, 0, IPC_RMID, 0) == -1)
      perror("local_sem.sem_delete().semctl()");
  }
}


/*-------------------------------------------------------------------------*/
/* End of file local_sem.c */
/*-------------------------------------------------------------------------*/
