/*-------------------------------------------------------------------------*/
/*  main.c                                                                 */
/*-------------------------------------------------------------------------*/
/*  B(PN)^2 -> C Codegenerator                                             */
/*                                                                         */
/*  Johannes Jaeger --- Sommer 1997                                        */
/*-------------------------------------------------------------------------*/

#include <unistd.h>
#include <stdio.h>
#include "frontend.h"
#include "tree_handling.h"
#include "bool.h"
#include "includes.h"
#include "misc.h"
#include "translate.h"
#include "symboltable.h"
#include "c_code.h"


/*-------------------------------------------------------------------------*/

/* option chars */
#define N_OPTS    2
#define GEN_EXEC 'c'
#define INFO     'i'

char options[N_OPTS+2]={'+',
  GEN_EXEC,                     /* Option ohne Argument: -c     */
  INFO,':'                      /* Option mit Argument: -i file */
};


/*-------------------------------------------------------------------------*/
static void printUsage(char *cmd)
{
  (void) fprintf(stderr, "Usage: %s [-%c] [-%c file] <file%s>\n",
		 cmd, GEN_EXEC, INFO, BPN2_SUFFIX);
  (void) fprintf(stderr, "  -%c       do not generate executable\n", GEN_EXEC);
  (void) fprintf(stderr, "  -%c file  write symbols to file\n", INFO);

  exit(EXIT_FAILURE);
}


/*-------------------------------------------------------------------------*/
static void parsecmdline(int argc, char **argv)
{
  int c;
  boolean NoFileArg = TRUE;

  while ( (c = getopt(argc, argv, options)) != -1 )
  {
    switch (c)
    {
      case GEN_EXEC:
	generate_executable = FALSE;
	break;
      case INFO:
	info_on = (! info_on);
	strcpy(Info_fname, optarg);
	break;
      case '?':    /* wrong option detected */
	printUsage(argv[0]);
	break;
    }
  }

  /* there must be a filename argument behind the options */  
  if (optind < argc)
  {
    BPN2_fname = argv[optind++];
    NoFileArg = FALSE;
  }

  /* no file specified */
  if (NoFileArg)
  {
    fprintf(stderr, "%s: no filename specified\n", argv[0]);
    printUsage(argv[0]);
  }

  /* there shouldn't be any more arguments */
  if (optind < argc)
  {
    fprintf(stderr, "%s: too many arguments\n", argv[0]);
    printUsage(argv[0]);
  }
}

/*-------------------------------------------------------------------------*/
static void check_args(int argc, char* argv[])
{
  parsecmdline(argc, argv);

  /* correct suffix in BPN2-filename ? */
  if (strcmp(&BPN2_fname[strlen(BPN2_fname) - strlen(BPN2_SUFFIX)],
	     BPN2_SUFFIX) != 0)
  {
    fprintf(stderr,"%s: invalid filename suffix\n", argv[0]);
    printUsage(argv[0]);
  }
}


/*-------------------------------------------------------------------------*/
/* Ausgabe aller B(PN)^2-Symbole in eine Datei */
static void print_info(void)
{
  OpenFile(&Info_File, Info_fname, "w");
  print_hashtab(Info_File);
  CloseFile(&Info_File);
}

/*-------------------------------------------------------------------------*/
/* Aufruf des Compilers fuer das generierte Programm */
static void compile_c_code(void)
{
  char system_call[1024];
  char*	tmp;

  fprintf(stderr,"Compiling executable.\n");
  tmp=MYstrdup(C_fname);
  tmp[strlen(tmp) - strlen(C_SUFFIX)] = '\0';
  sprintf(system_call,
	  "gcc -Wall -W -ansi -pedantic -O3 -D_SVID_SOURCE -w -o %s %s -I%s -L%s -l%s",
	  tmp, C_fname, lib_path, lib_path, RUN_LIB_NAME);
#ifdef DEBUG
  fprintf(stderr,"\n%s\n",system_call);
#endif /* DEBUG */
  (void) system(system_call);

  free(tmp);
}

/*-------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
  int parseResult;
  /* Kommandozeile abarbieten */
  check_args(argc, argv);
  /* Dateinamen initialisieren */
  Init(argv[0]);

  OpenFile(&BPN2_File, BPN2_fname, "r");

  /* Ackermann-Parser aufrufen */
  parseResult = bpn2parse( BPN2_File, stdout, stderr );
  CloseFile(&BPN2_File);
  if ( parseResult != 0 )
    return parseResult;


#ifdef PRINT_TREE
  PrintBlock( tree );
#endif

#ifndef ONLY_FRONTEND
  /* Ackermann-Parsebaum in Melzer-Parsebaum uebertragen */
  InternalTree = translateParseTree(tree);

  delete_tree( tree );

  /* B(PN)^2-Symbole ausgeben */
  if (info_on)
    print_info();

  /* C-Code erzeugen */
  generate_c_code(InternalTree);

  /* C-Code compilieren */
  if (generate_executable)
    compile_c_code();

#endif /* ONLY_FRONTEND */

  return(EXIT_SUCCESS);
}

/*-------------------------------------------------------------------------*/
/* End of file main.c */
/*-------------------------------------------------------------------------*/
