/****************************************************************/
/*                                                              */
/*            C O D E - G E N E R A T O R  for B(PN)^2          */
/*                                                              */
/*            Universitaet Hildesheim                           */
/*            Institut fuer Informatik                          */
/*            Stephan Melzer                                    */
/****************************************************************/
/*            Diplomarbeit                                      */
/*            File        : scheduler.c                                   
              Version     : 1.0
	      Last Update : 18.10.1994
 ****************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <signal.h>
#include "scheduler.h"
#include "error.h"
#include "bits.h"
#include "local_shm.h"
#include "local_sem.h"
#include "misc.h"
#include "includes.h"
#include "sighandler.h"

#include "algebra.c"

int *entries_used,
    *first_entry,
    *first_free_entry;

int timer_routine_pid;

process_entry* process;


void init_scheduler(void)
{
  int i;

  initialize_shm(tmp_page);
  switch_page(tmp_page);
  first_entry = (int*) shm_alloc((int)sizeof(int));
  (*first_entry) = -1;
  entries_used = (int*) shm_alloc((int)sizeof(int));
  (*entries_used) = 0;
  first_free_entry = (int*) shm_alloc((int)sizeof(int));
  (*first_free_entry) = 0;
  process = (process_entry*) shm_alloc((int)(MAX_PROCESS * 
                                       sizeof(process_entry)) );
  switch_page(stady_page);
  for (i=0; i < MAX_PROCESS; i++)
  {
    process[i].next_entry = (i == (MAX_PROCESS - 1)) ? -1 : (i + 1);
    process[i].is_used = FALSE;
  }
}

void init_timer_routine(int syn_semaphore, int tab_semaphore, char ** account)
{
  if ((timer_routine_pid = fork()) < 0)
    runtime_error(ERR_FORK_DOES_NOT_WORK);
  else
    if (timer_routine_pid == 0)
    {
      DeInstallSigHandler();
      for ( ; ; )
      {
	sleep(1);
	sem_wait(syn_semaphore);
	if (strcmp(*account, "") == 0)
	{
	  sem_signal(syn_semaphore);
	  activate_sleeping_processes(syn_semaphore, tab_semaphore);
	}
	else sem_signal(syn_semaphore);
      }
    }
    else
      printf ("\nCHANNEL 0 - ROUTINE - SERVER  installed\n\n");
}

void kill_timer_routine(void)
{
  kill(timer_routine_pid, SIGKILL);
  printf("\nCHANNEL 0 - ROUTINE - SERVER  killed\n");
}


void set_copy(SET l, SET r)
{
  int i;

  for (i=0; i < BIT_LENGTH; i++)
    l[i] = r[i];
}


int insert_process(int sem, int (*bool_proc)(container*, container*, int*),
	           SET in, SET out, SET variables)
{
  if ((*entries_used) == MAX_PROCESS)
    return -1;
  else
  {
    int new_free_entry,
        i = *first_free_entry;

    while (process[i].is_used)
    {
      i = process[i].next_entry;
      if (i == -1)
        return -1;
    }

    new_free_entry = process[i].next_entry;

    process[i].semaphore = sem;
    process[i].expr = bool_proc;
    set_copy(process[i].input, in); 
    set_copy(process[i].output, out);
    set_copy(process[i].vars, variables);

    process[i].is_used = FALSE; 
    process[i].next_entry = (*first_entry);
    (*first_entry) = i;
    (*first_free_entry) = new_free_entry;
    (*entries_used)++;
    return i;
  }
}

void print_scheduler()
{
  int i = (*first_entry);

  while (i != -1)
  {
    fprintf(stderr,
	    "PID : %i \n  IN   : %s\n  OUT  : %s\n  VARS : %s\n  SEM  : %i\n", 
	    i,
	    str_set(process[i].input),
	    str_set(process[i].output),
	    str_set(process[i].vars),
	    process[i].semaphore);
    i = process[i].next_entry;
  }
}

void print_scheduler_ad()
{
  int i = (*first_free_entry);

  print_scheduler();
  fprintf (stderr, "First Entry : %i \n", *first_entry);
  fprintf (stderr, "First Free Entry : %i \n", *first_free_entry);
  while (i != -1)
  {
    fprintf(stderr, " %i", i);
    i = process[i].next_entry;
  }
  fprintf(stderr, "\n entries used : %i \n", *entries_used);
}


int delete_process(int number)
{
  int* ptr;

  if ((*entries_used) == 0)
    return -1;
  else
  {
    ptr = &(*first_entry);
  
    while (*ptr != number)
    {
      if (*ptr == -1)
        return -1;
      ptr = &(process[*ptr].next_entry);
    }
  }

  *ptr = process[number].next_entry;
  process[number].next_entry = (*first_free_entry);
  (*first_free_entry) = number;
  (*entries_used)--;
  return 0;
}

void get_synchro_partner(unsigned short* s, int chan_0, int mode)
/* This function searches through the whole process table to find 
   possible synchronisation partners related to chan_0.
   The mode indicates wether _INPUT_ partner or _OUTPUT_ partner
   should be collected.
   s : Set
   chan_0 : hash_number of the channel
   mode : _INPUT_ or _OUTPUT_
*/
{
  int ptr = (*first_entry);

  (void)memset(s, 0, BIT_LENGTH * sizeof(unsigned short));  /* empty set */
  while (ptr != -1)
  {
    if ((! process[ptr].is_used) && 
	is_element ( (mode == _INPUT_) ? process[ptr].input :
		     process[ptr].output,
		     chan_0))
      insert_element((SET) s, ptr);
   ptr = process[ptr].next_entry;
  }
}


int no_process_inside(char* inter)
{
  int i;

  for (i=0; i < (int) strlen(inter); i++)
    if (isdigit((int) inter[i]))
      return FALSE;

  return TRUE;
}


char* synchronize(int process_ptr)
{
  unsigned short *input_pairs,
		 *output_pairs;
  int input_size,
      output_size,
      i,j,k,
      *pos;
  char *dummy,
       *prefix;
  conf *c;

  dummy = (char*) Malloc(MAX_INTERLEAVING * sizeof(char));
  prefix = (char*) Malloc(MAX_INTERLEAVING * sizeof(char));

  prefix[0] = '\0';
   
  process[process_ptr].is_used = TRUE;
  input_size = elements(process[process_ptr].input);
  
  if (input_size != 0)
  {
    input_pairs = (unsigned short*) Malloc(input_size * BIT_LENGTH *
					   sizeof(unsigned short));

    for (i=0, j=0; i < BIT_LENGTH * INT_WIDTH; i++)
      if (is_element(process[process_ptr].input, i))
	get_synchro_partner( &(input_pairs[(j++) * BIT_LENGTH]), i, _OUTPUT_);

    c = initialize_conf(input_pairs, input_size);

    prefix = strcat(prefix, "(");

    if (c->max_configs == 0) 
    {
      strcpy(prefix, "");
      process[process_ptr].is_used = FALSE;
      return prefix;
    }

    for (i=0; i < c-> max_configs; i++)
    {
      pos = check_loop_synchro(c, get_next_config(c));
      if (pos != (int*) NULL)
      {
	int failure;
	char* single;

	single = (char*) Malloc(MAX_INTERLEAVING * sizeof(char));

	for (j=0; j < c->size; j++)
	  if ( ( k = get_element(process[process_ptr].input, j + 1) ) >=0 )
	    delete_element(process[pos[j]].output, k);

	failure = FALSE;
	strcpy(dummy, "(");
	for (j = 0; j < c->size; j++)
        {
	  strcpy(single, synchronize(pos[j]));
	  failure = no_process_inside(single);
	  if (! failure)
	    dummy = strcat(dummy, single);
	  else
	    break;
	  dummy = strcat(dummy, ";");
        }

	dummy = strcat(dummy, ")");

	for (j = 0; j < c->size; j++)
	  if ( ( k = get_element(process[process_ptr].input, j + 1) ) >= 0 )
	    insert_element(process[pos[j]].output, k);

	dummy = strcat(dummy, "|");
	free(pos);
	if (!failure)
	  prefix = strcat(prefix, dummy);

	free(single);
      }
    }
	
    free(c); 
    if (prefix[strlen(prefix) - 1] == '(')
    {
      strcpy(prefix, "");
      process[process_ptr].is_used = FALSE;
      return prefix;
    }
    else
      strcat(prefix, ")");
  }

  dummy [0] = '\0';

  sprintf(dummy, "%d", process_ptr);
  prefix = strcat(prefix, dummy);

  output_size = elements(process[process_ptr].output);

  if (output_size != 0)
  {
    output_pairs = (unsigned short*) Malloc(output_size * BIT_LENGTH *
					    sizeof(unsigned short));

    for (i = 0,j = 0; i < BIT_LENGTH * INT_WIDTH; i++)
      if (is_element(process[process_ptr].output, i))
	get_synchro_partner( &(output_pairs[(j++) * BIT_LENGTH]), i, _INPUT_);

    c = initialize_conf(output_pairs, output_size);
    prefix = strcat(prefix, "(");

    if (c->max_configs == 0) 
    {
      strcpy(prefix, "");
      process[process_ptr].is_used = FALSE;
      return prefix;
    }

    for (i=0; i < c-> max_configs; i++)
    {
      pos = check_loop_synchro(c, get_next_config(c));
      if (pos != (int*) NULL)
      {
	int failure;
	char* single;

	single = (char*) Malloc(MAX_INTERLEAVING * sizeof(char));
      
	for (j = 0; j < c->size; j++)
	  if ( ( k = get_element(process[process_ptr].output, j + 1) ) >= 0 )
	    delete_element(process[pos[j]].input, k);
	failure = FALSE;
	strcpy(dummy, "(");
	for (j = 0; j < c->size; j++)
	{
	  strcpy(single, synchronize(pos[j]));
	  failure = no_process_inside(single);
	  if (!failure)
	    dummy = strcat(dummy, single);
	  else
	    break;
	  dummy = strcat(dummy, ";");
	}

	dummy = strcat(dummy, ")");

	for (j = 0; j < c->size; j++)
	  if ( ( k = get_element(process[process_ptr].output, j + 1) ) >= 0 )
	    insert_element(process[pos[j]].input, k);
	dummy = strcat(dummy, "|");
	free(pos);
	if (!failure)
	  prefix = strcat(prefix, dummy);
	free(single);
      }
    }
  
    free (c);
    if (prefix [strlen (prefix) -1] == '(')
    {
      strcpy (prefix, "");
      process[process_ptr].is_used = FALSE;
      return prefix;
    }
    else
      prefix = strcat (prefix, ")");
  }

  process[process_ptr].is_used = FALSE;
  return prefix;
}
 

SET lock_for_synchronisation(int table_sem)
{
  int ptr = (*first_entry);
  SET all_vars = nil_set();

  while (ptr != -1)
  {
    all_vars = union_set(all_vars, process[ptr].vars);
    ptr = process[ptr].next_entry;
  }
  lock(table_sem, all_vars);
  return all_vars;
}

void unlock_for_synchronisation(int table_sem, SET all_vars)
{ 
  unlock(table_sem, all_vars);
  free(all_vars);
}

int global_bool_expr(container *pre_cont, container *post_cont, 
		     int *access_ok, sequence* s)
{	
  int str_ptr = 0,
      pid = 0,
      result = TRUE;

  while (str_ptr < (int) strlen(s->seq))
  {
    if (isdigit(s->seq[str_ptr]))
    {
      pid = pid * 10 + (int) (s->seq[str_ptr] - '0');
      str_ptr++;
    }
    else 
    {
      result = result && (process[pid].expr)(pre_cont, post_cont, access_ok);
      pid = 0;
      str_ptr++;
    }
  }
  return result && (*access_ok);
}

sequence* get_synchronisation(container* pre, container* post, int process)
{
  int* already_checked,
       number_of_sequences,
       access_ok,
       result,
       i,
       number_of_already_checked;

  sequence* ptr;
  container *pre_cont,
	    *post_cont;

  get_term(synchronize(process));
  (void) yyparse();
  number_of_sequences = 0;
  ptr = seq_tab;

  while (ptr != (sequence*) NULL)
  {
    number_of_sequences++;
    ptr = ptr->next;
  }

  if (number_of_sequences == 0) {
#ifdef DEBUG
    fprintf(stderr, "keine sequence\n");
#endif
    return (sequence*) NULL;
  }
  else
  {
#ifdef DEBUG
    fprintf(stderr, "sequence: %s\n", seq_tab->seq);
#endif
    already_checked = (int*) Malloc(number_of_sequences * sizeof(int));
    for (i=0; i < number_of_sequences; i++)
      already_checked[i] = FALSE;
    number_of_already_checked = 0;
  }

  pre_cont  = allocate_local_memory_for_copy(get_memory_table_size());
  post_cont = allocate_local_memory_for_copy(get_memory_table_size());
  
  do
  {
    get_pre_primed_variables(pre_cont);
    get_post_primed_variables(post_cont);
    access_ok = TRUE;

    do 
      i = rand() % number_of_sequences;
    while (already_checked[i]);

    already_checked[i] = TRUE;
    number_of_already_checked++;
    ptr = seq_tab;

    while (i != 0)
    {
      ptr = ptr->next;
      i--;
    }
    /* keine Referenzwerte im atomaren Kommando ubernehmen */
    Testing = TRUE;
    result = global_bool_expr(pre_cont, post_cont, &access_ok, ptr);
    Testing = FALSE;

    if (result)
    {
      get_pre_primed_variables(pre);
      get_post_primed_variables(post);

      (void) global_bool_expr(pre, post, &access_ok, ptr);
      return ptr;
    }
  } while (number_of_already_checked != number_of_sequences);

  return (sequence*) NULL;
}

void execute_sequence(sequence *s)
{
  int str_ptr = 0,
      pid     = 0;

#ifdef DEBUG
  print_scheduler();
#endif
  while (str_ptr < (int) strlen(s->seq))
  {
    if (isdigit(s->seq[str_ptr]))
    {
      pid = pid * 10 + (int) (s->seq[str_ptr] - '0');
      str_ptr++;
    }
    else 
    {
#ifdef DEBUG
      fprintf(stderr, "exe (%i) \n", pid);
#endif
      process[pid].semaphore = 0;
      (void) delete_process(pid);
      process[pid].is_used = TRUE; /* do not allow other process    */
      pid = 0;			   /* the synchronisation with this */
      str_ptr++;		   /* process, after activation     */
    }
  }
}

SET set_seq(sequence* s)
{
  int str_ptr = 0,
      pid     = 0;
  SET result = nil_set();

  if (s == (sequence*) NULL)
    return result;

  while (str_ptr < (int) strlen(s->seq))
  {
    if (isdigit(s->seq[str_ptr]))
    {
      pid = pid * 10 + (int) (s->seq[str_ptr] - '0');
      str_ptr++;
    }
    else 
    {
      insert_element(result, pid);
      pid = 0;
      str_ptr++;
    }
  }
  return result;
}

int first_sleeping_process(int start)
{ 
  while (start != -1)
  {
    if (is_empty(process[start].output) && (! process[start].is_used))
      return start;
    else
      start = process[start].next_entry;
  }
  return -1;
}


void activate_sleeping_processes(int syn_semaphore, int tab_semaphore)
{
  int pid;

  sem_wait(syn_semaphore);
  pid = first_sleeping_process(*first_entry);

  while (pid != -1)
  {
    sequence*  s;
    container* pre, * post;
    SET wake_up_pids = nil_set(),
        process_vars,
        dummy_set;

    process_vars = lock_for_synchronisation(tab_semaphore);
    pre = allocate_local_memory_for_copy( get_memory_table_size() );
    post= allocate_local_memory_for_copy( get_memory_table_size() );
    s = get_synchronisation(pre, post, pid);
    dummy_set = set_seq(s);
    if ((s != (sequence*) NULL) && is_disjunct(dummy_set, wake_up_pids))
    {
      rewrite_copies(pre, post);
      wake_up_pids = union_set(wake_up_pids, dummy_set);
      deallocate_local_memory_copy(pre);
      deallocate_local_memory_copy(post);
      execute_sequence(s);
      unlock_for_synchronisation(tab_semaphore, process_vars);
    }
    else
    {
      deallocate_local_memory_copy(pre);
      deallocate_local_memory_copy(post);
      unlock_for_synchronisation(tab_semaphore, process_vars);
    }
    pid = first_sleeping_process(process[pid].next_entry);
  }
  sem_signal(syn_semaphore);
}

/*-------------------------------------------------------------------------*/
/* End of file scheduler.c */
/*-------------------------------------------------------------------------*/
