/*-------------------------------------------------------------------------*/
/*  sighandler.c                                                           */
/*-------------------------------------------------------------------------*/
/*  B(PN)^2 -> C Codegenerator                                              */
/*                                                                         */
/*  Johannes Jaeger --- Sommer 1997                                        */
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <signal.h>
#include "error.h"


/* Signal-Handler fuer den Parent-Prozess;       */
/* bei den Signalen SIGINT, SIGTERM, SIGQUIT     */
/* soll kontrolliert abgebrochen werden, indem   */
/* per Exit-Handler Resourcen freigegeben werden */
void SigHandler( int sig )
{
#ifdef DEBUG
  (void)fprintf(stderr, "Inside SigHandler -- SIG: %d, pid: %d\n",
	  	sig, (int)getpid());
#endif /* DEBUG */
  exit(EXIT_FAILURE);
}


/* fuer die Signale SIGINT, SIGTERM und */
/* SIGQUIT einen Handler installieren;  */
/* wird nur in main() des generierten   */
/* Programms aufgeerufen                */
void InstallSigHandler(void)
{
  struct sigaction sigact;

  /* Signalhandler installieren */
  sigact.sa_handler = SigHandler;
  sigact.sa_flags = 0;
  if (sigemptyset(&sigact.sa_mask) == -1) {
    perror("sighandler.InstallSigHandler().sigemptyset()");
    runtime_error(ERR_SIGEMPTYSET);
  }
  if (sigaction(SIGINT, &sigact, NULL) == -1) {
    perror("sighandler.InstallSigHandler().sigaction()");
    runtime_error(ERR_SIGACTION);
  }
  if (sigaction(SIGTERM, &sigact, NULL) == -1) {
    perror("sighandler.InstallSigHandler().sigaction()");
    runtime_error(ERR_SIGACTION);
  }
  if (sigaction(SIGQUIT, &sigact, NULL) == -1) {
    perror("sighandler.InstallSigHandler().sigaction()");
    runtime_error(ERR_SIGACTION);
  }
}


/* fuer jeden Kind-Prozess wird    */
/* wieder das Default-Verhalten    */
/* bei den entsprechenden Signalen */
/* eingerichtet                    */
void DeInstallSigHandler(void)
{
  struct sigaction sigact;

  sigact.sa_handler = SIG_DFL;
  sigact.sa_flags = 0;
  if (sigemptyset(&sigact.sa_mask) == -1) {
    perror("sighandler.InstallSigHandler().sigemptyset()");
    runtime_error(ERR_SIGEMPTYSET);
  }
  if (sigaction(SIGINT, &sigact, NULL) == -1) {
    perror("sighandler.DeInstallSigHandler().sigaction()");
    runtime_error(ERR_SIGACTION);
  }
  if (sigaction(SIGTERM, &sigact, NULL) == -1) {
    perror("sighandler.DeInstallSigHandler().sigaction()");
    runtime_error(ERR_SIGACTION);
  }
  if (sigaction(SIGQUIT, &sigact, NULL) == -1) {
    perror("sighandler.DeInstallSigHandler().sigaction()");
    runtime_error(ERR_SIGACTION);
  }
}

/*-------------------------------------------------------------------------*/
/* End of file sighandler.c */
/*-------------------------------------------------------------------------*/

