/*-------------------------------------------------------------------------*/
/*  translate_misc.c                                                       */
/*-------------------------------------------------------------------------*/
/*  B(PN)^2 -> C Codegenerator                                              */
/*                                                                         */
/*  Johannes Jaeger --- Sommer 1997                                        */
/*-------------------------------------------------------------------------*/
/* Dieses Modul versammelt Funktionen aus Stephan Melzers Modulen    */
/* control_checking.c, declaration_checking.c, expression_checking.c,*/
/* procedure_call_checking.c. Diese werden in translate.c aufgerufen */
   
#include <stdlib.h>
#include <string.h>
#include "translate_misc.h"
#include "bool.h"
#include "includes.h"
#include "misc.h"
#include "tree_handling.h"
#include "error.h"
#include "bits.h"
#include "symboltable.h"


/*------------------------------------------------------------------------*/

#define ID_LENGTH 10

unsigned int numb = 0;

/*------------------------------------------------------------------------*/
char* new_identifier()
/* This function generates unique identifiers for the used B(PN)^2
   identifiers.
*/
{
  char*	result = (char*) Malloc(ID_LENGTH * sizeof(char));

  sprintf(result, "id_%u", numb++);
  return result;
}

/*------------------------------------------------------------------------*/
SemNode* init_sem_leaf(int type, int is_const_leaf, char text[], int op, 
		       SET ch_0_input, SET ch_0_output,
		       SET ch_X_input, SET ch_X_output, SET plain_var)
/* This function generates a leaf containing a identifier or a constant.
   type : datatype of symbol
   is_const_leaf : is TRUE if it is a constant
   text : information indicating the access to variable
   op : symbol code 
   ch_0_input : set of input variables of the type channel 0
   ch_0_output : set of output variables of the type channel 0
   ch_X_input : set of input variables of the type channel i (i>0)
   ch_X_output : set of output variables of the type channel i (i>0)
   plain_vars : set of plain variables
*/
{
  SemNode* result = init_tree(text, op);
  result->semantic.expr_constraint.datatype = type;
  result->semantic.expr_constraint.is_const = is_const_leaf;
  result->semantic.expr_constraint.is_complex = FALSE;
  result->semantic.expr_constraint.plain  = plain_var;
  result->semantic.expr_constraint.channel_0_input  = ch_0_input;
  result->semantic.expr_constraint.channel_0_output = ch_0_output;
  result->semantic.expr_constraint.channel_X_input  = ch_X_input;
  result->semantic.expr_constraint.channel_X_output = ch_X_output;

  /* if is_var then is not bounded */
  return result;
}

/*------------------------------------------------------------------------*/
SemNode* compose_sem_node(SemNode* l, SemNode* r, int type, int access, 
                          char text[], int op)
/* This function joins two semnatic node containing each a part of the
   boolean expression.
   l, r : pointer to semantic node of each operand
   type : type of operator
   access : indicates wether it is a constant expression or a variable 
            expression
   text : information string
   op : code of operator
*/
{
  SemNode* result = compose_tree(l, r, text, op);
  SET l_set, r_set;

  result->semantic.expr_constraint.datatype = type;
  result->semantic.expr_constraint.is_const = access;
  result->semantic.expr_constraint.is_complex = TRUE;
  l_set = (l != (SemNode*) NULL ?
                l->semantic.expr_constraint.channel_0_input : nil_set());
  r_set = (r != (SemNode*) NULL ?
	   r->semantic.expr_constraint.channel_0_input : nil_set());
  result->semantic.expr_constraint.channel_0_input = union_set(l_set, r_set);

  l_set = (l != (SemNode*) NULL ?
	   l->semantic.expr_constraint.channel_0_output : nil_set());
  r_set = (r != (SemNode*) NULL ?
	   r->semantic.expr_constraint.channel_0_output : nil_set());
  result->semantic.expr_constraint.channel_0_output = union_set(l_set, r_set);

  l_set = (l != (SemNode*) NULL ?
	   l->semantic.expr_constraint.channel_X_input : nil_set());
  r_set = (r != (SemNode*) NULL ?
	   r->semantic.expr_constraint.channel_X_input : nil_set());
  result->semantic.expr_constraint.channel_X_input = union_set(l_set, r_set);

  l_set = (l != (SemNode*) NULL ?
	   l->semantic.expr_constraint.channel_X_output : nil_set());
  r_set = (r != (SemNode*) NULL ?
	   r->semantic.expr_constraint.channel_X_output : nil_set());
  result->semantic.expr_constraint.channel_X_output = union_set(l_set, r_set);

  l_set = (l != (SemNode*) NULL ?
	   l->semantic.expr_constraint.plain : nil_set());
  r_set = (r != (SemNode*) NULL ?
	   r->semantic.expr_constraint.plain : nil_set());
  result->semantic.expr_constraint.plain = union_set(l_set, r_set);

  return result;
}

/*------------------------------------------------------------------------*/
SemNode* checked_type(SemNode* l, SemNode* r, 
	              int e_access, int x_type, int x_access, 
                      char text[], int op)
/* This function generates a inner sematic node out of the nodes l and r.
   The datatype and the access of l and r are checked. And the type of the 
   result of the operator op is specified.
   l, r : pointer to the semantic nodes of the operands
   remainder variables : see above
*/
{
  switch(e_access) 
  {
     case CONST : if ((! l->semantic.expr_constraint.is_const) || 
		      (! r->semantic.expr_constraint.is_const))
		  {
		    compile_error(ERR_ILLEGAL_POSTPRIMED);
		  }
                  break;
     case VAR   : if (l->semantic.expr_constraint.is_const || 
		      r->semantic.expr_constraint.is_const)
                  {
		    compile_error(ERR_ILLEGAL_POSTPRIMED);
                  }
                  break;
     default    : break;
  }
  return compose_sem_node(l, r, x_type, x_access, text, op);
}

/*------------------------------------------------------------------------*/
SemNode* checked_type_single(SemNode* l, 
	               int e_type, int e_access, char text[], int op)
/* This function generates a inner node of the monadic operator op.
   parameters : see above
*/
{
  SemNode* result = init_tree(text, op);

  switch (e_access) 
  {
    case CONST : if (! l->semantic.expr_constraint.is_const)
		 {
                   compile_error(ERR_ILLEGAL_POSTPRIMED);
                 }
                 break;
    case VAR   : if (l->semantic.expr_constraint.is_const)
                 {
		   compile_error(ERR_ILLEGAL_POSTPRIMED);
                 }
                 break;
    default    : break;
  }

  result->semantic.expr_constraint.datatype = e_type;
  result->left = l;	
  result->semantic.expr_constraint.is_const = e_access;
  result->semantic.expr_constraint.is_complex = TRUE;
  result->semantic.expr_constraint.plain = l->semantic.expr_constraint.plain;
  result->semantic.expr_constraint.channel_0_input = 
                                 l->semantic.expr_constraint.channel_0_input;
  result->semantic.expr_constraint.channel_0_output = 
                           	 l->semantic.expr_constraint.channel_0_output;
  result->semantic.expr_constraint.channel_X_input  = 
  	                         l->semantic.expr_constraint.channel_X_input;
  result->semantic.expr_constraint.channel_X_output = 
	                         l->semantic.expr_constraint.channel_X_output;
  return result;
}

/*------------------------------------------------------------------------*/
SemNode* create_ident_leaf(char sym[], int access,  char info[])
/* This function creates a semantic node - a leaf containing an identifier.
   sym : string of the identifier
   access : is TRUE, if it is a read access
   info : information string
*/
{
  int bucket = lookup(sym);
  int type, chan_size, dummy1, dummy2;
  Tscope scope;
  SET plain_set = nil_set();

  get_properties(bucket, &type, &scope, &chan_size, &dummy1, &dummy2);
  insert_element(plain_set, bucket);
  read_write_access(bucket);

  switch (type)
  {
    case USER_DEFINED  :
    case CARDINAL_TYPE :
    case INTEGER_TYPE  : type = NUMERIC; break;
    case BOOLEAN_TYPE  : type = BOOLEAN; break;
    default            : type = NUMERIC;
	              	 break;
  }
  return init_sem_leaf(type, access, info, bucket, nil_set(), nil_set(),
                       nil_set(), nil_set(), plain_set);
}

/*------------------------------------------------------------------------*/
SemNode* create_channel_leaf(char sym[], int access, char info[])
/* This function creates a semantic node - a leaf containing a channel access.
   sym : string of the identifier
   access : is TRUE, if it is a read access
   info : information string
*/
{
  int bucket = lookup(sym);
  int type, chan_size, dummy1, dummy2;
  Tscope scope;
  SET c_0_i = nil_set(),
      c_0_o = nil_set(),
      c_X_i = nil_set(), 
      c_X_o = nil_set();

  get_properties(bucket, &type, &scope, &chan_size, &dummy1, &dummy2);
  read_write_access(bucket);

  switch (type)
  {
    case USER_DEFINED  :
    case CARDINAL_TYPE :
    case INTEGER_TYPE  : type = NUMERIC; break;
    case BOOLEAN_TYPE  : type = BOOLEAN; break;
    default            : break;   
  }
  if (bucket != -1)
  {
    if (strcmp(info, "X?") == 0)
      if (chan_size == 0)
	insert_element(c_0_i, bucket);
      else
	insert_element(c_X_i, bucket);
    else
      if (chan_size == 0)
	insert_element(c_0_o, bucket);
      else
	insert_element(c_X_o, bucket);
  }

  return init_sem_leaf(type, access, info, bucket, c_0_i,
                       c_0_o, c_X_i, c_X_o, nil_set());
}

/*------------------------------------------------------------------------*/
SemNode* create_const_leaf(int type, char const_sym[], int operand)
/* This function creates a semantic node - a leaf containing an constant.
   type : datatype of the constant
   sym : string of the identifier
   operand : symbol code
*/
{
  return init_sem_leaf(type, CONST, const_sym, operand, nil_set(),
		       nil_set(), nil_set(), nil_set(), nil_set());
}

/*------------------------------------------------------------------------*/
SemNode* create_checked_atomic_command(SemNode* expr, int op_type)
/* This function generates an inner node for a atomic command.
   expr : pointer to the boolean expression of the atomic command
   op_type : symbol code
*/
{
  SemNode* result = init_tree("", op_type);
  SET used_vars = union_set(
                    expr->semantic.expr_constraint.plain, 
		    union_set(
		      union_set(
                        expr->semantic.expr_constraint.channel_X_input,
                        expr->semantic.expr_constraint.channel_X_output
		      ),
		      union_set(
			expr->semantic.expr_constraint.channel_0_input,
                        expr->semantic.expr_constraint.channel_0_output
                      )
		    )
		  );

  result->left = expr;
  result->right = init_tree(str_set(used_vars), 0);
  result->semantic.loop_control.operator = op_type;
  result->semantic.loop_control.is_exit  = FALSE;
  result->info_text = new_identifier();
  return result;
}

/*------------------------------------------------------------------------*/
SemNode* checked_equation(SemNode* l, SemNode* r)
/* This function generates an inner semantic node containing an equation.
   l, r : both operands of the equation
*/
{
  SemNode* result = NULL; 

  if (l->semantic.expr_constraint.is_const)
  {
    if (r->semantic.expr_constraint.is_const)
      result = checked_type(l, r, CONST, BOOLEAN, CONST, "=", EQU_);
    else
      result = checked_type(r, l, UNDEF, BOOLEAN, VAR, "=", EQU_);
  }
  else
  {
    if (r->semantic.expr_constraint.is_const) /* Melzer 3.3.2 */
      result = checked_type(l, r, UNDEF, BOOLEAN, VAR, "=", EQU_);
    else
      compile_error(ERR_ASSIGNMENT);
  }

  return result;
}

/*------------------------------------------------------------------------*/
 void recover_hidden_identifier(list* ident_list)
/* This routine recovers hidden identifiers stored in a identifier
   list ident_list after reaching the inner block end.
   ident_list : pointer to linear chaining list of identifiers
*/
{
  if (ident_list != (list*) -1)
  {
    list* list_ptr = ident_list;
    char id[MAX_IDENT_LENGTH];
    int bucket;
    int dummy;

    while (list_ptr != empty_list())
    {
      list_ptr = get_ident(list_ptr, id, &dummy);
      bucket = lookup(id);
      recover_entry(bucket);
    }
  }
}

/*------------------------------------------------------------------------*/
list* hide_identifier(list* main_list, list* sub_list,
		      data_description data, int RNum)
/* This function appends a list of identifiers, sub list, of a equal type, 
   data, to a block variable list declared at this block beginning, main_list.
   main_list : pointer to list of identifiers declared in this block
   sub_list : pointer to a list of identifiers of one and the same type
   data : the data description of the type of all variables of sub_list
*/
{
  if (data.data_type != -1)
    if (sub_list != (list*) -1)
    {
      list* list_ptr = sub_list;
      list* result   = (main_list != (list*) -1) ? main_list : empty_list();
      char id[MAX_IDENT_LENGTH];
      int partyp;

      while (list_ptr != (empty_list()))
      {
	list_ptr = get_ident(list_ptr, id, &partyp);
        result = append_list(result, id, partyp);

	(void) overriding_insert(id, data.data_type, data.scope,
				 data.channel_size, data.init_value,
				 data.stack_or_chan, RNum);
      }
      return result;
    }
    else 
      return main_list;
  else 
    return (list*) -1;
}

/*------------------------------------------------------------------------*/
data_description create_initialized_typed_set(data_description data_set, 
 					      data_description data_init)
/* Creates a data description consisting out of a set definition and a
   possible initialization. The coherence of the both data description
   is checked and merged to common data description.
   data_set : data description of the type of the set definition
   data_init : data description of the type of the initialization definition
*/
{
  data_description result;

  result.stack_or_chan = NO_S_OR_C;

  if (data_set.data_type == -1) 
    result = data_set;
  else
    if (data_init.data_type == -1)
      data_init.init_value = createValue(data_set.scope);
  switch (data_set.data_type)
  {
    case BOOLEAN_TYPE :
      if ((data_init.data_type == BOOLEAN_TYPE) ||
	  (data_init.data_type == -1))
      {
	result.data_type = BOOLEAN_TYPE;
	result.scope.num_bounds = 1;
	result.scope.bounds[0].low = 0;
	result.scope.bounds[0].high = 1;
	result.channel_size = -1;
	result.init_value = data_init.init_value;
      }
      break;
    case CARDINAL_TYPE :
    case INTEGER_TYPE :
    case USER_DEFINED : {
      int upper_bound = data_set.scope.bounds[data_set.scope.num_bounds-1].high;
      if ((data_set.scope.bounds[0].low <= data_init.init_value)
	  &&  (upper_bound >= data_init.init_value) 
	  &&  (data_init.data_type != BOOLEAN_TYPE))
      {
	result.data_type = data_set.data_type;
	result.scope = data_set.scope;
	result.channel_size = -1;
	result.init_value = data_init.init_value;
      }
      break;
    }
    default : break;
  }

  return result; 
}

/*------------------------------------------------------------------------*/
data_description create_typed_channel(data_description data_set, int capacity,
				      int s_or_c)
/* Creates the data description of channel box.
   data_set : data type of the channel
   capacity : capacity of the channel
   s_or_c   : indicates stack or channel
*/
{
  data_description 	result;

  if (data_set.data_type == -1)
    result.data_type = -1;
  else
  {
    result.data_type   = data_set.data_type;
    result.scope = data_set.scope;
    result.init_value  = -1;
    result.channel_size= capacity;
    result.stack_or_chan = s_or_c;
  }
  return result;
}

/*------------------------------------------------------------------------*/
data_description create_bool_set(void)
/* Creates a data description of a boolean variable box.
*/
{
  static data_description result;

  result.channel_size = -1;
  result.scope.num_bounds = 1;
  result.scope.bounds[0].low = 0;
  result.scope.bounds[0].high = 1;
  result.init_value = 0;
  result.data_type = BOOLEAN_TYPE;

  return result;
}

/*------------------------------------------------------------------------*/
data_description create_int_set(void)
/* Creates a data description of a integer variable box.
*/
{
  static data_description result;

  result.channel_size = -1;
  result.scope.num_bounds = 1;
  result.scope.bounds[0].low = MIN_INTEGER;
  result.scope.bounds[0].high = MAX_INTEGER;
  result.init_value = 0;
  result.data_type = INTEGER_TYPE;

  return result;
}

/*------------------------------------------------------------------------*/
data_description create_card_set(void)
/* Creates a data description of a cardinal variable box.
*/
{
  static data_description result;

  result.channel_size = -1;
  result.scope.num_bounds = 1;
  result.scope.bounds[0].low = 0;
  result.scope.bounds[0].high = MAX_CARD;
  result.init_value = 0;
  result.data_type = CARDINAL_TYPE;


  return result;
}

/*------------------------------------------------------------------------*/
data_description create_user_defined_set(data_description set)
/* Creates a data description of a user defined variable box.
*/
{
  data_description result;

  result.channel_size = -1;
  result.scope = set.scope;
  result.init_value = 0;
  result.data_type = USER_DEFINED;
  
  return result;
}

/*------------------------------------------------------------------------*/
SemNode* join_control(SemNode* l, SemNode* r, int op)
/* This function combines the two trees l and r to a new node with
   the operator op.
   l, r : Pointer to the operands of the construct op
   op : abbreviation code for the underlying B(PN)^2 construct
*/
{
  SemNode* result = compose_tree(l, r, "", op);

  if (l == (SemNode*) -1)
  {
    if (r == (SemNode*) -1)
      result->semantic.loop_control.is_exit = TRUE;
    else
      result->semantic.loop_control.is_exit = r->semantic.loop_control.is_exit;
  }
  else
  {
    if (r == (SemNode*) -1)
      result->semantic.loop_control.is_exit = l->semantic.loop_control.is_exit;
    else
      result->semantic.loop_control.is_exit =
	l->semantic.loop_control.is_exit ||
	r->semantic.loop_control.is_exit;
  }
  return result;
}

/*------------------------------------------------------------------------*/
SemNode* finite_loop_control(SemNode* i, SemNode* b, int op)
/* This function generates a tree for the loop (see op) consisting of
   an init-command i and a loop body b. If there is no exit construct
   a warning is given.
   i : pointer to the semantic node which contains the initializing command
   b : pointer to the semantic node which contains the body of the loop
   op : abbreviation code for the loop construct
*/
{
  SemNode* result = compose_tree(i, b, "", op);

  result->semantic.loop_control.operator = op;
  result->semantic.loop_control.is_exit = FALSE;
  if (b != (SemNode*) -1)
    if (! b->semantic.loop_control.is_exit)
      warning(WARN_INIFINITE_LOOP);
return result;
}

/*------------------------------------------------------------------------*/
SemNode* control_exit(SemNode* l, int op)
/* This function generates an alternative of the loop construct as an
   exit branch.
   l : pointer to the semantic node of alternative
   op : see above
*/
{
  SemNode* result = compose_tree(l, init_tree("", EXIT_), "", op);

  result->semantic.loop_control.is_exit = TRUE;
  result->semantic.loop_control.operator = op;
  return result;
}

/*------------------------------------------------------------------------*/
SemNode* control_repeat(SemNode* l, int op)
/* This function generates an alternative of the loop construct as an
   repeat branch.
   l : pointer to the semantic node of alternative
   op : see above
*/
{
  SemNode* result = compose_tree(l, init_tree("", REPEAT_), "", op);

  result->semantic.loop_control.is_exit = FALSE;
  result->semantic.loop_control.operator = op;
  return result;
}

/*------------------------------------------------------------------------*/
SemNode* control_block(SemNode *b, list* l)
/* This function generates a B(PN)^2 - block segment with declaration.
   b : pointer to the semantic node of the block
   l : pointer to a list of identifiers declared in this block
*/
{
  SemNode* result;
  SET	block_set = nil_set();
  list*	dummy = l;
  int	bucket;

  if (l == (list*) -1)
    dummy = empty_list();
	
  while (dummy != empty_list())
  {
    bucket = lookup(dummy->identifier);
    if (bucket != -1) {
      insert_element(block_set, bucket);
      dummy->bucket = bucket;
    }
    dummy = dummy->next;
  }
  result = compose_tree(b, init_tree("", 0), str_set(block_set), BEGIN_);
  result->semantic.loop_control.is_exit = FALSE;
  return result;
}

/*------------------------------------------------------------------------*/
SemNode* create_procedure_call(char proc_name[], SemNode* expr_list, 
                               int op_type)
/* Function to check a procedure  and generate a semantic node
   proc_name : Identifier of the called procedure
   expr_list : Semantic Node which is the root of the actual parameter
               list 
   op_type   : Type of the semantic node
*/
{
  SemNode* result = init_tree(proc_name, op_type);
  int	   bucket = lookup(proc_name);
  int type, dummy2, dummy3, dummy4;
  Tscope dummy1;
  SET vars = nil_set();
  char* actual_param_list = (char*) Malloc(sizeof(char)); 
  SemNode* expr_ptr;


  get_properties(bucket, &type, &dummy1, &dummy2, &dummy3, &dummy4);
  /* Protocol the access to this function */
  read_write_access(bucket);


  (void)strcpy(actual_param_list, "");
  expr_ptr = expr_list;
  while (expr_ptr != (SemNode*) NULL) {
    char* dummy = (char*) Malloc(
			 (strlen(actual_param_list) + 2) * sizeof(char)),
      * old_ptr = actual_param_list;

    actual_param_list = strcpy(dummy, actual_param_list);
    free(old_ptr);

    if (expr_ptr->left->semantic.expr_constraint.datatype == POINTER) {
      strcat(actual_param_list, FORMAL_CHAN);
      vars = union_set(vars, expr_ptr->left->semantic.
		       expr_constraint.channel_X_input);
    } else {
      vars = union_set(vars, expr_ptr->left->semantic.
		       expr_constraint.plain);
      if (expr_ptr->left->semantic.expr_constraint.is_const) {
	/* is a constant expression */  
	if (expr_ptr->left->semantic.expr_constraint.datatype == BOOLEAN)
	  strcat(actual_param_list, FORMAL_CONST_BOOL);
	else
	  strcat(actual_param_list, FORMAL_CONST_INT);
      }
      else {
	if (! expr_ptr->left->semantic.expr_constraint.is_complex) {
	  /* is a variable expression */  
	  if (expr_ptr->left->semantic.expr_constraint.datatype == BOOLEAN)
	    strcat(actual_param_list, FORMAL_VAR_BOOL);
	  else
	    strcat(actual_param_list, FORMAL_VAR_INT);
	}
      }
    }
    expr_ptr = expr_ptr->right;
  } /* while () */

  result->left = expr_list;
  result->right = init_tree(str_set(vars), bucket);
  result->semantic.loop_control.operator = op_type;
  result->semantic.loop_control.is_exit  = FALSE;
  return result;
}

/*------------------------------------------------------------------------*/
SemNode* empty_sem_node_list(void)
{	
  return (SemNode*) NULL;
}

/*------------------------------------------------------------------------*/
SemNode* init_sem_node_list(SemNode* expr)
/* Initialize a list of semantic nodes to store the actual parameter list
   expr : Pointer to a semantic node which stores one parameter of the
          parameter list
*/
{
  return compose_tree(expr, empty_sem_node_list(), "", COMMA_);
}

/*------------------------------------------------------------------------*/
SemNode* append_sem_node_list(SemNode* list, SemNode* expr)
/* Append one parameter to the actual parameter list 
   list : Pointer to the sematic node which containes the head of the actual
          parameter list
   expr : Pointer to the expression which will be append in the end of the
	  parameter list
*/
{
  SemNode* result = list;
  SemNode* dummy  = list;

  for ( ; dummy->right != (SemNode*) NULL ; dummy = dummy->right)
    ;
  dummy->right = expr;
  return result;
}

/*------------------------------------------------------------------------*/
SemNode* init_sem_node_list_channel(char channel_name[])
/* Initializes a list of a semantic node to store a channel parameter.
   channel_name : identifier of the channel variable
*/
{
  int bucket = lookup(channel_name);
  SemNode* result = init_tree(channel_name, bucket);
  int type, dummy2, chan_size, dummy4;
  Tscope  dummy1;
  SET c_0_i = nil_set(),
      c_0_o = nil_set(),
      c_X_i = nil_set(),
      c_X_o = nil_set();


  get_properties(bucket, &type, &dummy1, &dummy2, &chan_size, &dummy4);
  read_write_access(bucket);
  if (chan_size == 0) {
    insert_element(c_0_i, bucket);
    insert_element(c_0_o, bucket);
  }
  else {
    insert_element(c_X_i, bucket);
    insert_element(c_X_o, bucket);	
  }

  result->semantic.expr_constraint.datatype = POINTER;
  result->semantic.expr_constraint.is_const = FALSE;
  result->semantic.expr_constraint.is_complex = FALSE;
  result->semantic.expr_constraint.plain = nil_set();
  result->semantic.expr_constraint.channel_0_input = c_0_i;
  result->semantic.expr_constraint.channel_0_output = c_0_o;
  result->semantic.expr_constraint.channel_X_input = c_X_i;
  result->semantic.expr_constraint.channel_X_output = c_X_o;
  return compose_tree(result, empty_sem_node_list(), "", COMMA_);
}

/*-------------------------------------------------------------------------*/
/* End of file translate_misc.c */
/*-------------------------------------------------------------------------*/
