/*-------------------------------------------------------------------------*/
/*  translate_misc.h                                                       */
/*-------------------------------------------------------------------------*/
/*  B(PN)^2 -> C Codegenerator                                              */
/*                                                                         */
/*  Johannes Jaeger --- Sommer 1997                                        */
/*-------------------------------------------------------------------------*/

#ifndef TRANSLATE_MISC_H
#define TRANSLATE_MISC_H


#include "list.h"

/*-------------------------------------------------------------------*/
#include "tree_handling.h"


/* Global counter for unique identifiers */

extern unsigned int numb;

SemNode* checked_type(SemNode* l, SemNode* r, 
	              int e_access, int x_type, int x_access, 
                      char text[], int op);
SemNode* checked_type_single(SemNode* l, 
	                     int e_type, int e_access, char text[], int op);
SemNode* create_ident_leaf(char sym[], int access, char info[]);
SemNode* create_channel_leaf(char sym[], int access, char info[]);
SemNode* create_const_leaf(int type, char const_sym[], int operand);
SemNode* create_checked_atomic_command(SemNode* expr, int op_type);
SemNode* checked_equation(SemNode* l, SemNode* r);

/*-------------------------------------------------------------------*/
/* Datatype to store the information about a B(PN)^2 variable */ 

typedef struct {
  int channel_size;
  Tscope scope;
  int init_value;
  int data_type;
  int stack_or_chan;
} data_description;

void recover_hidden_identifier(list* ident_list);
list* hide_identifier(list* main_list, list* sub_list,
		      data_description data, int RNum);
data_description create_initialized_typed_set(data_description data_set, 
					      data_description data_init);
data_description create_typed_channel(data_description data_set, int capacity,
				      int s_or_c);
data_description create_bool_set(void);
data_description create_int_set(void);
data_description create_card_set(void);
data_description create_user_defined_set(data_description set);

/*-------------------------------------------------------------------*/
SemNode* join_control (SemNode* l, SemNode* r, int op);
SemNode* finite_loop_control (SemNode* i, SemNode* b, int op);
SemNode* control_exit (SemNode* l, int op);
SemNode* control_repeat (SemNode* l, int op);
SemNode* control_block (SemNode *b, list* l);
SemNode* create_procedure_call(char proc_name[], SemNode* expr_list, 
                               int op_type);
SemNode* empty_sem_node_list(void);
SemNode* init_sem_node_list_channel(char channel_name[]);
SemNode* append_sem_node_list(SemNode* list, SemNode* expr);
SemNode* init_sem_node_list(SemNode* expr);

#endif /* TRANSLATE_MISC_H */
/*-------------------------------------------------------------------------*/
/* End of file translate_misc.h */
/*-------------------------------------------------------------------------*/
