/****************************************************************/
/*                                                              */
/*            C O D E - G E N E R A T O R  for B(PN)^2          */
/*                                                              */
/*            Universitaet Hildesheim                           */
/*            Institut fuer Informatik                          */
/*            Stephan Melzer                                    */
/****************************************************************/
/*            Diplomarbeit                                      */
/*            File        : tree_handling.c                                        
              Version     : 1.0
	      Last Update : 18.10.1994
 ****************************************************************/
/*            Module information :                              */
/* This module makes the basical routines for the parse tree
   construction available. The nodes of the parse tree belongs 
   to the type SemNode, which are characterized by its op-type.
   This module is used in every parse tree generating context.
   The basic functions are : init_tree, compose_tree
*/
/****************************************************************/
/* Johannes Jaeger -- Sommer 1997 */

#include <stdlib.h>
#include <stdio.h>
#include "list.h"
#include "tree_handling.h"
#include "misc.h"


/* the parse tree from frontend will be translated into InternalTree */
SemNode* InternalTree;

/* globaler Zeiger auf die Prozedurdeklarationen */
ProcHead* Procedures = (ProcHead*) NULL;


SemNode* init_tree(char text[], int op)
/* This function initialize a leave as semantic node. The rudimantary
   attributes which are info_text and op-Code are setted here. 
   Special attributes of more special semantic nodes have additional
   functions which are presented in other modules.
   text : string which specifies an info text (ex. name of procedure)
   op : Type of semantic node
*/
{
  SemNode* result = (SemNode*) Malloc(sizeof(SemNode));

  result->left  = (SemNode*) NULL;
  result->right	= (SemNode*) NULL;
  result->node_operator = op;
  SetRef( &(result->ref), 0, 0, 0, 0, 0);
  if (text != (char*) NULL) 
  {
    if (strlen(text) > 0)
    {
      result->info_text = (char*) Malloc(sizeof(char) * (strlen(text) + 1));
      strcpy(result->info_text, text);
    }
  }
  else
    result->info_text = (char*) NULL;
  return result;
}


SemNode* compose_tree(SemNode* l, SemNode* r, char text[], int op)
/* This function composes an inner node of parse tree out of its childs
   here the sematic nodes l and r.
   l : pointer to the left child of the inner node
   r : pointer to the right child of the inner node
   text, op : see above
*/
{
  SemNode*	result = (SemNode*) Malloc(sizeof(SemNode));

  result->left  = l;
  result->right	= r;
  result->node_operator = op;
  if (text != (char*) NULL) 
  {
    if (strlen(text) > 0)
    {
      result->info_text = (char*) Malloc(sizeof(char) * (strlen(text) + 1));
      strcpy(result->info_text, text);
    }
  }
  else
    result->info_text = (char*) NULL;
  return result;
}

#ifdef DEBUG

void print_tree(SemNode* p, int i)
/* This function is useful for a visual overview of the parse tree. 
   Beside its aid to debug the behaviour of the code generation, it is 
   also useful for the generation of the tree inscription for the 
   dependicities of the Oldenburg part (see Mathias Damm in Berlin 94)
   p : pointer to the root of the parse tree
   i : number of space tab to indicate the tree deepness
*/
{
  int j;

  if (p == (SemNode*) NULL)
    return;
  print_tree(p->left, i+1);
  for (j= 0 ; j < i;j++)
    printf(" ");
  fprintf(stderr, "OPERATOR : %d INFO : %s -- i : %d\n", p->node_operator, 
	 p->info_text == (char*) NULL ? "" : p->info_text, i);
  if (p->node_operator != IDENT_)
    print_tree(p->right, i+1);
}

#endif



ProcEntry* AddProcEntry(ProcEntry* ProcListHead, char* Proc_Name, list* Params,
			   SemNode* Block)
{
  ProcEntry* result;
  ProcEntry* tmpEntry;

  if (!ProcListHead)
    result = tmpEntry = (ProcEntry*) Malloc(sizeof(ProcEntry));
  else {
    result = tmpEntry = ProcListHead;
    while (tmpEntry->NextProc)
      tmpEntry = tmpEntry->NextProc;
    tmpEntry->NextProc = (ProcEntry*) Malloc(sizeof(ProcEntry));
    tmpEntry = tmpEntry->NextProc;
  }

  (void)strcpy(tmpEntry->ProcName, Proc_Name);
  tmpEntry->ParList = Params;
  tmpEntry->NextProc = (ProcEntry*) NULL;
  tmpEntry->ProcBlock = Block;

  return result;
}

ProcHead* AddProcHead(ProcHead* listHead, char* name,
		      int calls, int nparams, ProcEntry* Proclist, int RNum)
{
  ProcHead *result, *tmpProcHead;

  if (!listHead)
    result = tmpProcHead = (ProcHead*) Malloc(sizeof(ProcHead));
  else {
    result = tmpProcHead = listHead;
    while (tmpProcHead->NextProcHead)
      tmpProcHead = tmpProcHead->NextProcHead;
    tmpProcHead->NextProcHead = (ProcHead*) Malloc(sizeof(ProcHead));
    tmpProcHead = tmpProcHead->NextProcHead;
  }

  tmpProcHead->MaxCalls = calls;
  tmpProcHead->NumParams = nparams;
  tmpProcHead->RefNum = RNum;
  (void)strcpy(tmpProcHead->ProcName, name);
  tmpProcHead->Procs = Proclist;
  tmpProcHead->NextProcHead = (ProcHead*) NULL;

  return result; 
}

ProcHead* FindProcHead(ProcHead* listHead, const char* searchname)
{
  ProcHead *PHead = listHead;

  while (PHead) {
    if(strcmp(PHead->ProcName, searchname) == 0)
      break;
    PHead = PHead->NextProcHead;
  }

  return PHead;
}

/*-------------------------------------------------------------------------*/
/* End of file tree_handling.c */
/*-------------------------------------------------------------------------*/
