#ifndef TREE_HANDLING_H
#define TREE_HANDLING_H

/****************************************************************/
/*                                                              */
/*            C O D E - G E N E R A T O R  for B(PN)^2          */
/*                                                              */
/*            Universitaet Hildesheim                           */
/*            Institut fuer Informatik                          */
/*            Stephan Melzer                                    */
/****************************************************************/
/*            Diplomarbeit                                      */
/*            File        : tree_handling.h
              Version     : 1.0
	      Last Update : 18.10.1994
 ****************************************************************/
/*            Module information :                              */
/* Header file for the parse tree constructing module.
*/
/****************************************************************/


#include "configure.h"
#include "bits.h"
#include "list.h"
#include "includes.h"


/* Datatype for the semantic nodes of the parse tree */

typedef struct SemNodePtr {
  union {
    struct { /* expression related infos */
      int datatype;
      int is_const;
      int is_complex;
      SET plain;
      SET channel_0_input;
      SET channel_0_output;
      SET channel_X_input;
      SET channel_X_output;
    } expr_constraint;
    struct { /* contructs related infos */
      int is_exit;
      int operator;
    } loop_control ;
  } semantic;
  Reference          ref;
  int    	     node_operator;
  char*  	     info_text;
  struct SemNodePtr* left;
  struct SemNodePtr* right;
} SemNode;

/* the parse tree from frontend will be translated into InternalTree */
extern SemNode* InternalTree;


/* Struktur zur Aufnahme einer Prozedurdeklaration */
typedef struct ProcEntryPtr {
  char                 ProcName[MAX_IDENT_LENGTH];
  list*                ParList;
  SemNode*             ProcBlock;
  struct ProcEntryPtr* NextProc;
} ProcEntry;

typedef struct ProcHeadPtr {
  char                ProcName[MAX_IDENT_LENGTH];
  int                 MaxCalls;
  int                 NumParams;
  int                 RefNum;
  ProcEntry          *Procs;
  struct ProcHeadPtr *NextProcHead;
} ProcHead;

/* globaler Zeiger auf die Prozedurdeklarationen */
extern ProcHead* Procedures;


#ifdef DEBUG

void print_tree(SemNode* p, int i);

#endif /* DEBUG */

SemNode* init_tree(char text[], int op);
SemNode* compose_tree(SemNode* l, SemNode* r, char text[], int op);

ProcEntry* AddProcEntry(ProcEntry* ProcListHead, char* Proc_Name, list* Params,
			   SemNode* Block);

ProcHead* AddProcHead(ProcHead* listHead, char* name,
		      int calls, int nparams, ProcEntry* Proclist, int RNum);
ProcHead* FindProcHead(ProcHead* listHead, const char* searchname);

#endif /* TREE_HANDLING_H */

/*-------------------------------------------------------------------------*/
/* End of file tree_handling.h */
/*-------------------------------------------------------------------------*/
