#ifndef VAR_MEM_MANAGE_H
#define VAR_MEM_MANAGE_H

/****************************************************************/
/*                                                              */
/*            C O D E - G E N E R A T O R  for B(PN)^2          */
/*                                                              */
/*            Universitaet Hildesheim                           */
/*            Institut fuer Informatik                          */
/*            Stephan Melzer                                    */
/****************************************************************/
/*            Diplomarbeit                                      */
/*            File        : var_mem_manage.h          
              Version     : 1.0
	      Last Update : 18.10.1994
 ****************************************************************/
/*            Module information :                              */
/* Header file for the runtime libary memory management of the 
   B(PN)^2 variables.
*/
/****************************************************************/

#include "bits.h"
#include "misc.h"
#include "configure.h"


 
/* MACROS */

#define UNKNOWN      32001 
#define FULL         32002
#define SYNCHRONIZED 32003
#define UNUSED          -1

/* Nummern fuer die Referenzen */
#define VAR_INIT       32
#define VAR_TERM       64
#define VAR_ACCESS    128
#define SC_READ       256
#define SC_WRITE      512
#define SC_FULL      1024
#define SC_EMPTY     2048
#define PROC_CALL    8192
#define PROC_RETURN 16384


/* Name der Protokolldatei - wird im generierten */
/* Programm per Kommandozeile gesetzt */
extern char Prot_fname[1024];
extern boolean proto2file;
extern boolean proto2stdout;
extern FILE* prot_file;

/* variable entry with basic information */
typedef struct{ 
  int* value;           /* pointer to the value */
  char name[MAX_IDENT_LENGTH];
  int  init;
  int  is_locked;
  int  lifo_ptr;        /* not used in std case */
  int  fifo_head; 	/* not used in std case */ 
  int  fifo_tail;	/* not used in std case */
  int  fifo_size;	/* not used in std case */
  Tscope scope;         /* range of the datatype*/
  int  stack_or_chan;   /* var. is stack/channel/none */
  int  checked_flag;
  int  RefNum;
} mem_entry; 

 
/* SWITCH table for an internal enumeration of the B(PN)^2 - variables */
#ifdef NO_SHARED_INDEX
extern int index_switch[HASH_TABLE_SIZE]; 
#else
extern int* index_switch;
#endif /* NO_SHARED_INDEX */

typedef struct{
  int value;
  int is_used;
} container;


void get_pre_primed_variables (container* copy);
int get_pre_primed_value(container* pre, int hash_number, int* flag);
int getpreval_noprot(container* pre, int hash_number, int* flag);
void set_post_primed_value(container* post, int hash_number, 
                           int new_value, int* flag);
void get_post_primed_variables (container* copy);
int get_post_primed_value(container* post, int hash_number);
void lock (int semaphore, SET vars);
void unlock (int semaphore, SET vars);
void rewrite_copies (container* pre, container* post);
void generate_memory_table(int size, int x_memory_size);
container* allocate_local_memory_for_copy (int size);
int get_memory_table_size (void);
void deallocate_local_memory_copy (container* page);
void initialize_variable(int hash_number, char var_name[], 
                         int channel_size, int init_value,
			 Tscope scope, int s_or_c, int RNum);
void initialize_protokoll(void);
void re_initialize(SET ident_set, boolean only2proto);
void sleepy(int j);
void erase_memory_table(void);
void enqueue(int* queue, int value);
int top_fifo(container* pre, int hash_number, int *flag);
void push_fifo(container* post, int hash_number, int new_value, int* flag);
int* get_addr(int hash_number);

void set_synchro(container* pre, container* post, int hash_number, 
	         int new_value, int* flag);
int get_synchro(container* pre, container* post, int hash_number, int* flag);
int empty_check(int* queue);
int full_check(int* queue);
void rewrite_copies_noprot(container* post);
int NoOfElements(int* queue);

void PCall2Prot(char* PName, Reference ref,
		int RNum, int Flag);

#endif /* VAR_MEM_MANAGE_H */

/*-------------------------------------------------------------------------*/
/* End of file var_mem_manage.h */
/*-------------------------------------------------------------------------*/
