/*****************************************************************************/
/* FILE bpn2pbc.c                                                            */
/* version 3.1 (completely reviewed version of file main.c, version 2.0)     */
/* Burkhard Graves, 10.10.93                                                 */
/* Lars Jenner, Juni 1994                                                    */
/* Stephan Merkel, Mar. 95                                                   */
/*****************************************************************************/
/* Note: In this version the default settings for standard initialisation and*/
/* sequential termination are: DISABLED.                                     */
/*****************************************************************************/


#include <stdio.h>
#include <string.h>
#include "bpn2pbc.h"
#include "symtab.h"
#include "calc.h"

#define QUEUE_LENGTH 81
#define LINE_TERM_CHARS " ,;:)}#"

/*****************************************************************************/
/* Datenstrukturen                                                           */
/*****************************************************************************/

/* these are all initially False to simplify the setting */
bool GL_SeqTerm   = False, /* apply sequential termination */
     GL_StdInit   = False, /* apply standard initialisation */
     GL_NoOutput  = False, /* don't create target file */
     GL_shrthnd   = False;

int num_old_syn;
  /* number of times old syntax is being used */

/*****************************************************************************/
/* Funktionen                                                                */
/*****************************************************************************/


/*****************************************************************************/
/*************************** usage() *****************************************/
/*****************************************************************************/
/* Gibt die korrekte Aufrufsyntax aus.                                       */
/*****************************************************************************/

static void usage(char* prgname)
  {
    fprintf(stderr, "\nusage: %s source target [-tio]\n", prgname);
    fprintf(stderr, "   or: %s source [target] -e[tio]\n\n", prgname);
    fprintf(stderr, "-t: enable sequential termination.\n");
    fprintf(stderr, "-i: enable standard initialization.\n");
    fprintf(stderr, "-s: enable 'shrthnd'.\n");
    fprintf(stderr, "-o: disable warnings of old syntax.\n" );
    fprintf(stderr, "-e: edit mode (check syntax only).\n\n");
    fprintf(stderr, "-f: edit mode (check syntax only) for fragments.\n\n");
    fprintf(stderr, "If the -e option is used, no box expression file will");
    fprintf(stderr, " be created.\n");
    fprintf(stderr, "If target is still specified, parsing error messages");
    fprintf(stderr, " will be printed\n");
    fprintf(stderr, "into it, else to the standard error output.\n\n");
    exit(2);
  } /* usage() */


/*****************************************************************************/
/**************************** read_opts() ************************************/
/*****************************************************************************/
/* Liest die Optionen (eingeleitet durch '-').                               */
/*****************************************************************************/

static void read_flags(char *ptr, char* progname)
{
    while (*++ptr) {
        switch(*ptr) {
	case 'o':
	    GL_OldSyn = True;
            break;
        case 'e':
	    GL_NoOutput = True;
            break;
        case 'f':
	    GL_NoOutput = True;
	    GL_Frags = True;
            break;
        case 'i':
	    GL_StdInit = True;
            break;
        case 't':
	    GL_SeqTerm = True;
            break;
        case 's':
	    GL_shrthnd = True;
            break;
        default:
            usage(progname);
        }
    }    
}

/*****************************************************************************/
/*************************** is_boxx_line_termin() ***************************/
/*****************************************************************************/
/*****************************************************************************/

static bool is_boxx_line_termin( char ch )
  {
    if ( ch == '\n' || strchr( LINE_TERM_CHARS, ch ) || ch == EOF )
      {
	return ( True );
      } /* if */
    else
      {
	return ( False );
      } /* else */
  } /* is_boxx_line_termin() */


/*****************************************************************************/
/******************************* dec() ***************************************/
/*****************************************************************************/
/*****************************************************************************/
static int dec( int i )
  {
    return ( --i >= 0 ? i : ( QUEUE_LENGTH - 1 ) );
  } /* dec() */


/*****************************************************************************/
/******************************** inc() **************************************/
/*****************************************************************************/
/*****************************************************************************/
static int inc( int i )
  {
    return ( ++i % QUEUE_LENGTH );
  } /* inc() */


/*****************************************************************************/
/******************************** beautify_boxx() ****************************/
/*****************************************************************************/
/*****************************************************************************/
static void beautify_boxx( FILE * ugly_boxx, FILE * nice_boxx )
  {
    int queue[QUEUE_LENGTH];
    int front = 0, rear, curr;

    rear = dec( front );

    do
      {
	do
	  {
	    rear = inc( rear );
	    queue[rear] = getc( ugly_boxx );
	  }
	while ( queue[rear] != '\n' && inc( inc( rear ) ) != front
		&& queue[rear] != EOF );

	curr = rear;
	while ( ! is_boxx_line_termin( queue[curr] ) && curr != front )
	  {
	    curr = dec( curr );
	  } /* while */

	if ( curr == front )
	  {
	    curr = rear;
	  } /* if */

	do
	  {
	    if ( queue[front] != EOF )
	      {
		putc( queue[front], nice_boxx );
	      } /* if */
	    front = inc( front );
	  } /* do */
	while ( dec( front ) != curr );
	if ( queue[curr] != '\n' )
	  {
	    putc( '\n', nice_boxx );
	  } /* if */
      } /* do */
    while ( queue[rear] != EOF );
  } /* beautify_boxx() */



/*****************************************************************************/
/* Hauptprogramm                                                             */
/*****************************************************************************/

int main(int argc,char *argv[])
{
  FILE * temp_file;
  char *src_name, *dest_name, * temp_file_name;
  int ret_val,i;
    /* just to preserve the return value of the compile function */

  src_name=dest_name=NULL;
 
  for (i=1; i<argc; ++i)
      if (*(argv[i])!='-')
          if (!src_name)
              src_name=argv[i];
          else if (!dest_name)
              dest_name=argv[i];
          else
              usage(argv[0]);
      else
          read_flags(argv[i],argv[0]);
  if (!src_name)
      usage(argv[0]);

  if (!(yyin=fopen(src_name,"r"))) {
    fprintf(stderr,"Cannot open file %s.\n",src_name);
    usage(argv[0]);
  }

  if (dest_name) { 
      if (!(yyout=fopen(dest_name,"w"))) {
          fprintf(stderr,"Cannot open file %s.\n",dest_name);
          usage(argv[0]);
      }
  } else {
      yyout=stderr;
      if ( ! GL_NoOutput ) {
          fprintf( stderr, "Target file was not specified.\n" );
          usage(argv[0]);
      }
  }

  init_elements();
  if ( GL_NoOutput )
    {
      ret_val = bpn2parse( yyin, stderr, yyout );
    }
  else
    {
      fprintf( yyout, "%%@+\n" );
      ret_val = bpn2parse( yyin, yyout, stderr );
      if (tree)
      {
	com(tree);
	delete_tree(tree);
      }
      if (yyout!=stderr)
	fclose( yyout );
      temp_file_name = MYmalloc( strlen( dest_name ) + 3 );
      strcpy( temp_file_name, dest_name );
      strcat( temp_file_name, "_t" );
      rename ( dest_name, temp_file_name );
      temp_file = fopen( dest_name, "w" );
      if ( temp_file ) /* if writeable */
	{
	  yyout = fopen( temp_file_name, "r" );
	  beautify_boxx( yyout, temp_file );
	  fclose( yyout );
	  fclose( temp_file );
	}
      remove ( temp_file_name );
      MYfree(temp_file_name);
    } /* else */
    if ( ! GL_NoOutput  )
      {
	fprintf( msg_out, "\nTarget file created.\n\n" );
      } /* if */
    else if ( ret_val )
      {
	fprintf( msg_out, "\nIncomplete target file created.\n\n" );
      } /* else if */
    else
      {
	fprintf( msg_out, "\nNo target file created.\n\n" );
      } /* else */

  fclose(yyin);
  remove_elements();
  exit(ret_val);
} /* main() */

/*****************************************************************************/
/* END OF FILE bpn2pbc.c                                                     */
/*****************************************************************************/
