/**********************************/
/* calc.c                         */
/* version 2.1                    */
/* Burkhard Graves, 06.10.93      */
/* slight modifications 03/95     */
/*              by Stephan Merkel */
/**********************************/

#include <string.h>
#include <stdio.h>
#include "calc.h"
#include "bpn2pbc.h"
#include "ops.h"
#include "tree.h"
#include "../frontend/set.h"

/*********************************/
/* Defines                       */
/*********************************/

#define TOP_MAX 10
#define MAX_ELEMENT 256u
#define MAX_STR_LEN 60

/*********************************/
/* Datenstrukturen               */
/*********************************/

FTupel *stack[TOP_MAX]; /* Ein Stack */
int top = TOP_MAX;

SymTabEntry **elements; /* Ueber diese Datenstruktur kann spaeter */
int max_element,   /* mittels des Element-Codes auf die      */
         act_element;   /* Symboltabelle zugegriffen werden.      */

/************************************************/
/* Initialisierung der Datenstruktur 'elements' */
/************************************************/

void init_elements(void)
{
    elements=(SymTabEntry **)MYmalloc(MAX_ELEMENT*sizeof(SymTabEntry *));
    max_element=MAX_ELEMENT;
    act_element=1;
}

void remove_elements(void)
{
    MYfree(elements);
}

/******************/
/* Offensichtlich */
/******************/

void set_act_element(int x)
{
	act_element=x;
}

/***********************************************/
/* Vergroesserung der Datenstruktur 'elements' */
/***********************************************/

static void pump_elements(void)
{
    elements=(SymTabEntry **)MYrealloc(elements,
                                       (max_element+=MAX_ELEMENT)
                                       *sizeof(SymTabEntry *));
}

/************************************************************************/
/* Fuer den neuen Symboleintrag 'entry' wird ab dem Index 'act_element' */
/* 'n'-mal 'entry' in der Datenstruktur 'elements' eingetragen.         */
/* Rueckgabewert ist der Index des ersten Eintrages.                    */
/************************************************************************/

int calc_elements(int n,SymTabEntry *entry)
{
    SymTabEntry **el_ptr;
    int old_act;

    old_act=act_element;
    act_element+=n;
    while (act_element>max_element) {
        pump_elements();
    }
    el_ptr=elements+old_act;
    while (n) {
        --n;
        *el_ptr++=entry;
    }
    return old_act;
}

/***************************************************************************/
/* In 'erg' wird die Vereinigung der Mengen 'op1' und 'op2' abgelegt,      */
/* falls die Maechtigkeit der Vereinigungsmenge 'max' nicht uebersteigt    */
/* (Fehlerfall).                                                           */
/* Rueckgabewert ist ein Zeiger, der hinter das Ende der Vereinigungsmenge */
/* zeigt. Im Fehlerfall wird ein NULL-Pointer zurueckgegeben.              */
/***************************************************************************/

static int *SetUnion(int *erg, int *op1,int *op2, int max)
{
    while (*op1 || *op2) {
        if (--max) {
            if (*op1==*op2) {
                *erg++=*op1++;
                op2++;
            } else if (!*op2 || (*op1 && *op1<*op2)) {
                *erg++=*op1++;
            } else {
                *erg++=*op2++;
            }
        } else {
            return NULL;
        }
    }
    *erg++=0;
    return erg;
}

/**************************************/
/* FTupel-Zeiger auf den Stack pushen */
/**************************************/

static void push(FTupel *ptr)
{
    if (top) {
        stack[--top]=ptr;
    } else {
	Msg(FAT,"stack overflow in <expr>-evaluation!");
    }
}

/**********************************/
/* FTupel-Zeiger vom Stack poppen */
/**********************************/

static FTupel *pop(void)
{
    if (top==TOP_MAX) {
	Msg(FAT,"stack underflow in <expr>-evaluation!");
    }
    return stack[top++];
}

/****************************************************************/
/* Eine Menge von FTupeln fuer eine un-primed Variable erzeugen */
/****************************************************************/

void un_primed(SymTabEntry *entry)
{
    int i;
    int n,elem,*menge;
    Set curr;
    FTupel *ftupel;

    chk_ident(entry,_VAR);

    if ( GL_NoOutput )
      /* if there have been errors upon parsing, some pointers may be nil */
      {
	return;
      }

    n = cardin( entry->s_set );
    if (!((entry->s_special|=_LG_UNPRIMED)&_L_PRIMED)) {
        entry->s_variations=n;
    }
    push(ftupel=(FTupel *)MYmalloc((n+2)*sizeof(FTupel)));
    ftupel->set=menge=(int *)MYmalloc(2*(n+1)*sizeof(int));
    *menge++=elem=entry->s_code;
    *menge++=0;
    ++n;
    curr = entry->s_set->next;
    i = curr->low - 1;
    do
      {
	if ( i < curr->high )
	  {
	    ++i;
	  } /* if */
	else
	  {
	    curr = curr->next;
	    i = curr->low;
	  } /* else */
        (++ftupel)->wert=i;
        ftupel->set=menge;
        *menge++=elem;
        *menge++=0;
        elem+=n;
      } /* do */
    while ( i < SET_MAX( entry->s_set ) );
    (++ftupel)->set=NULL; /* abschliessendes Element */
}

/*****************************************************************/
/* Eine Menge von FTupeln fuer eine pre-primed Variable erzeugen */
/*****************************************************************/

void pre_primed(SymTabEntry *entry)
{
    int i;
    int n,save_n,elem,*menge;
    Set curr;
    FTupel *ftupel;

    chk_ident(entry,_VAR);

    if ( GL_NoOutput )
      /* if there have been errors upon parsing, some pointers may be nil */
      {
	return;
      } /* if */

    n = save_n = cardin( entry->s_set );
    entry->s_special|=_LG_PRIMED;
    entry->s_variations=n*=n;
    push(ftupel=(FTupel *)MYmalloc((n+2)*sizeof(FTupel)));
    ftupel->set=menge=(int *)MYmalloc(2*(n+1)*sizeof(int));
    *menge++=elem=entry->s_code;
    *menge++=0;
    curr = entry->s_set->next;
    i = curr->low - 1;
    do
      {
	if ( i < curr->high )
	  {
	    ++i;
	  } /* if */
	else
	  {
	    curr = curr->next;
	    i = curr->low;
	  } /* else */

        n=save_n;
        while (n) {
            --n;
            (++ftupel)->wert=i;
            ftupel->set=menge;
            *menge++=elem++;
            *menge++=0;
        }
      } /* do ( curr ) */
    while ( i < SET_MAX( entry->s_set ) );
    (++ftupel)->set=NULL; /* abschliessendes Element */
}

/******************************************************************/
/* Eine Menge von FTupeln fuer eine post-primed Variable erzeugen */
/******************************************************************/

void post_primed(SymTabEntry *entry)
{
    int i;
    int n,elem,*menge;
    Set curr;
    FTupel *ftupel;

    chk_ident(entry,_VAR);

    if ( GL_NoOutput )
      /* if there have been errors upon parsing, some pointers may be nil */
      {
	return;
      } /* if */

    n = cardin( entry->s_set );
    entry->s_special|=_LG_PRIMED;
    entry->s_variations=n*=n;
    push(ftupel=(FTupel *)MYmalloc((n+2)*sizeof(FTupel)));
    ftupel->set=menge=(int *)MYmalloc(2*(n+1)*sizeof(int));
    *menge++=elem=entry->s_code;
    *menge++=0;
    n = cardin( entry->s_set );
    while (n) {
        --n;
        curr = entry->s_set->next;
        i = curr->low - 1;
        do
          {
	    if ( i < curr->high )
	      {
	        ++i;
	      } /* if */
	    else
	      {
	        curr = curr->next;
	        i = curr->low;
	      } /* else */
            (++ftupel)->wert=i;
            ftupel->set=menge;
            *menge++=elem++;
            *menge++=0;
          } /* do ( curr ) */
        while ( i < SET_MAX( entry->s_set ) );

    } /* while (n) */
    (++ftupel)->set=NULL; /* abschliessendes Element */
}

/***************************************************************/
/* Eine Menge von FTupeln fuer eine Channel-Variable erzeugen, */
/* wobei 'i' angibt, ob auf diese Variable lesend oder         */
/* schreibend zugegriffen wird.                                */
/***************************************************************/

void r_w_channel(SymTabEntry *entry,int i)
{
    int n,elem,*menge;
    Set curr;
    FTupel *ftupel;

    chk2_ident(entry,_CHANNEL,_STACK);

    if ( GL_NoOutput )
      /* if there have been errors upon parsing, some pointers may be nil */
      {
	return;
      } /* if */

    n = cardin( entry->s_set );
    push(ftupel=(FTupel *)MYmalloc((n+2)*sizeof(FTupel)));
    ftupel->set=menge=(int *)MYmalloc(2*(n+1)*sizeof(int));
    if (i==_LG_READING)
        *menge++=elem=entry->s_code;    /* c? */
    else
        *menge++=elem=n+entry->s_code;  /* c! */
    *menge++=0;
    if (((entry->s_special|=i)&(_L_RW))==_L_RW)
        entry->s_variations=2*n;
    else 
        entry->s_variations=n;

    curr = entry->s_set->next;
    i = curr->low - 1;
    do
      {
	if ( i < curr->high )
	  {
	    ++i;
	  } /* if */
	else
	  {
	    curr = curr->next;
	    i = curr->low;
	  } /* else */

        (++ftupel)->wert=i;
        ftupel->set=menge;
        *menge++=elem++;
        *menge++=0;
      } /* do */
    while ( i < SET_MAX( entry->s_set ) );
    (++ftupel)->set=NULL; /* abschliessendes Element */
}

/**************************************************/
/* Ein FTupel fuer einen konstanten Wert erzeugen */
/**************************************************/

void simple(int value)
{
    FTupel *ftupel;

    if ( GL_NoOutput )
      /* if there have been errors upon parsing, some pointers may be nil */
      {
	return;
      } /* if */

    push(ftupel=(FTupel *)MYmalloc(sizeof(FTupel)));
    ftupel->set=NULL;   /* Kennzeichen fuer konstanten Wert */
    ftupel->wert=value;
}

/**********************************/
/* Eine unaere Operation anwenden */
/**********************************/

void calc_un(int (*op)(int wert))
{
    FTupel *ftupel,*memft;

    if ( GL_NoOutput )
      /* if there have been errors upon parsing, some pointers may be nil */
      {
	return;
      }

    memft=ftupel=pop();
    if (ftupel->set) {
        while ((++ftupel)->set) {
            ftupel->wert=op(ftupel->wert);
        }
    } else {
        ftupel->wert=op(ftupel->wert);
    }
    push(memft);
}

/***********************************/
/* Eine binaere Operation anwenden */
/***********************************/

void calc_bin(int (*op) PROTO ((int wert1,int wert2)))
{
    register size_t size;
    register int max;
    register int *menge;
    register FTupel *ftupel1,*ftupel2,*ftupel3;
    FTupel *memft1,*memft2;
    int who[11];

    if ( GL_NoOutput )
      /* if there have been errors upon parsing, some pointers may be nil */
      {
	return;
      } /* if */

    memft1=ftupel1=pop(); memft2=ftupel2=pop();
    if (ftupel1->set) {
        if (ftupel2->set) {
            if ((menge=SetUnion(who,ftupel1->set,ftupel2->set,10))==NULL) {
		Msg(FAT,"using more than 10 variables in one <expr>!");
            }
            max=menge-who; /* max enthaelt die Anzahl der */
            menge=who;                 /* beteiligten Variablen + 1   */
            size=1;
            while (*menge) {
                size*=elements[*menge++]->s_variations;
            }
            push(ftupel3=(FTupel *)MYmalloc((size+2)*sizeof(FTupel)));
            ftupel3->set=menge=(int *)MYmalloc((size+1)*max*sizeof(int));
	    max=0;
            while ((*menge++=who[max++])!=0)
                skip;
            ++ftupel3;
            while ((++ftupel1)->set) {
                ftupel2=memft2;
                while ((++ftupel2)->set) {
                    ftupel3->set=menge;
                    if ((menge=SetUnion(ftupel3->set,
                                        ftupel1->set,ftupel2->set,max))==NULL) {
                        menge=ftupel3->set;
                        continue;
                    }
                    ftupel3->wert=op(ftupel2->wert,ftupel1->wert);
                    ++ftupel3;
                }
            }
            ftupel3->set=NULL;  /* abschliessendes Element */
            MYfree(memft1->set);
            MYfree(memft1);
            MYfree(memft2->set);
            MYfree(memft2);
        } else {
            while ((++ftupel1)->set) {
                ftupel1->wert=op(ftupel2->wert,ftupel1->wert);
            }
            push(memft1);
            MYfree(ftupel2);
        }
    } else if (ftupel2->set) {
        while ((++ftupel2)->set) {
            ftupel2->wert=op(ftupel2->wert,ftupel1->wert);
        }
        push(memft2);
        MYfree(ftupel1);
    } else {
        ftupel1->wert=op(ftupel2->wert,ftupel1->wert);
        push(ftupel1);
        MYfree(ftupel2);
    }
}


/*************************************************/
/* Aus einem FTupel eine Box-Expression erzeugen */
/*************************************************/

char *translate(void)
{
    register int n;
    register int *menge;
    register SymTabEntry *entry;
    register FTupel *ftupel;
    register char *txt;
    FTupel *memft;
    int done=0, min;
    char *txt_buffer,*max_txt;
    unsigned len;

#define SEGM 1000

    if ( GL_NoOutput )
      {
	return ( NULL );
      } /* if */
    memft=ftupel=pop();
    if (!(menge=ftupel->set)) {
      if (ftupel->wert==TT) {
        MYfree(ftupel);
	return MYstrdup("{}");
      } else {
        MYfree(ftupel);
	return MYstrdup("STOP");
      }
    } else {
      txt_buffer=(char *)MYmalloc(len=SEGM);
      max_txt = txt_buffer + len - (SEGM/4);
      txt=txt_buffer;
      while (*menge) {
	elements[*menge]->s_special&=(_G_RW | _G_PRUPR);
	elements[*menge]->s_variations=0;
	++menge;
      }
      menge=NULL;
      while ((++ftupel)->set) {
	if (ftupel->wert==TT) {
	  if (menge && !GL_shrthnd && !done) {
	    sprintf(txt," # ");
            txt+=3;
          }
	  if (!(GL_shrthnd && done)) {
	    sprintf(txt,"{");
            ++txt;
          }
	  menge=ftupel->set;
	  for (;;) {
	    if (txt>max_txt) {
              txt-=(unsigned)txt_buffer;
              txt_buffer=(char *)MYrealloc(txt_buffer,len+=SEGM);
              max_txt = txt_buffer + len - (SEGM/4);
              txt+=(unsigned)txt_buffer;
            }
	    entry=elements[*menge];
	    if (GL_shrthnd && !done) {
	      sprintf(txt,"%s",fn(entry));
              txt=strchr(txt,0);
            }
	    n = cardin( entry->s_set );
	    min = entry->s_set->next->low;
	    *menge-=entry->s_code;
	    if (!GL_shrthnd)
	    {
	      switch (entry->s_type) {
		case _VAR:
		  if ( is_bool( min ) )
		    {
		      sprintf( txt, "%s_%d_%d", fn( entry ),
		               transl( nth_elem( entry->s_set, *menge / n ) ),
		               transl( nth_elem( entry->s_set, *menge % n ) ) );
		    } /* if */
		  else
		    {
		      sprintf( txt, F_VAR, fn( entry ),
		                         nth_elem( entry->s_set, *menge / n ),
		                         nth_elem( entry->s_set, *menge % n ) );
		    } /* else */
		  txt=strchr(txt,0);
		break;
		case _CHANNEL:
		case _STACK:
		  sprintf(txt,"%s%c",fn(entry),(*menge>=n)?'!':'?');
		  txt=strchr(txt,0);
		  if (is_bool(min)) {
		    sprintf(txt,"%d",
		               transl( nth_elem( entry->s_set, *menge % n ) ) );
		  } else {
		    sprintf( txt, F_INT, nth_elem( entry->s_set, *menge % n ) );
		  }
		  txt=strchr(txt,0);
		break;
		default:
		Msg(FAT,"default translate ?");
	      }
	    }
	    if (!*++menge) {
	      break;
	    }
	    if (!(GL_shrthnd && done)) {
	      sprintf(txt,", ");
              txt+=2;
            }
	  }
	  if (!(GL_shrthnd && done)) {
	    sprintf(txt,"}");
            ++txt;
          }
	  if (GL_shrthnd)
	    done=1;
	}
      }
      if (!menge) {
	sprintf(txt,"STOP");
        txt+=4;
      }
      MYfree(memft->set);
      MYfree(memft);
    }
    txt=MYstrdup(txt_buffer);
    MYfree(txt_buffer);
    return txt;
}

/**********************************/
/* END OF FILE calc.c             */
/**********************************/
