/******************************************************************************/
/* FILE symtab.c                                                              */
/* version 2.1                                                                */
/* Burkhard Graves, 10.10.93                                                  */
/* Lars Jenner, Juni 1994                                                     */
/* last changes by Stephan Merkel, Mar. 95                                    */
/******************************************************************************/

#include <string.h>
#include "symtab.h"
#include "bpn2pbc.h"
#include "calc.h"
#include "ops.h"
#include "tree.h"

/******************************************************************************/
/* Datenstrukturen                                                            */
/******************************************************************************/

int blknum;         /* Verschachtelungstiefe */

SymTabEntry dummy,          /* Dummy-Element der Symboltabelle   */
		*s_start;   /* Start der Symboltabelle ('lokal') */

T_list  procgrplist;
T_list  forwardprocs;

/*******************************************************************/
/* Initialisierung der Symboltabelle (Aufruf aus 'bpn2pbc' heraus) */
/*******************************************************************/

void s_init(void)
{
    SymTabEntry *entry;

    (s_start=entry=&dummy)->s_next=NULL;
    entry->s_name="-";              /* kein gueltiger Bezeichner ! */
    entry->s_type=_VAR;             /* nicht _UDEC  !              */
    entry->s_blknum=blknum=0;       /* ungueltige Blocknummer !    */
    entry->s_body=NULL;
    entry->s_proc=NULL;
    entry->s_parlist=NULL;
    entry->s_partype=_NOPAR;
    entry->s_reclevel=0;
    entry->s_set=NULL;

 /* entry->s_special=undefined;     */
 /* entry->s_variations=undefined;  */
 /* entry->s_code=undefined;        */
 /* entry->s_init_or_cap=undefined; */
}

/****************************************************************************/
/* Absuchen der Symboltabelle nach dem Bezeichner 'ptr'. Konnte er gefunden */
/* werden, so wird ein Zeiger auf ihn zurueckgegeben, ansonsten wird ein    */
/* NULL-Pointer zurueckgegeben. (Aufruf durch 's_lookup')                   */
/****************************************************************************/

static SymTabEntry *s_find(char *ptr)
{
    SymTabEntry *entry;

    entry=s_start;
    while (entry && (*entry->s_name!=*ptr || strcmp(entry->s_name,ptr))) {
	entry=entry->s_next;
    }
    return entry;
}

/**************************************************************/
/* Erzeugung eines neuen Eintrages in der Symboltabelle       */
/* (Aufruf durch 's_lookup' und 'make_ident' und GenParList)  */
/**************************************************************/

static SymTabEntry *s_create(char *name)
{
    SymTabEntry *new_entry;

    new_entry=(SymTabEntry *)MYmalloc(sizeof(SymTabEntry));
    new_entry->s_next=s_start;
    s_start=new_entry;

    new_entry->s_name=MYstrdup(name);
    new_entry->s_type=_UDEC;
    new_entry->s_blknum=blknum;

    new_entry->s_body=NULL;
    new_entry->s_proc=NULL;
    new_entry->s_parlist=NULL;
    new_entry->s_partype=_NOPAR;
    new_entry->s_reclevel=0;
    new_entry->s_set=NULL;

 /* new_entry->s_special=undefined;      */
 /* new_entry->s_variations=undefined;   */
 /* new_entry->s_code=undefined;         */
 /* new->entry->s_init_or_cap=undefined; */

   return new_entry;
}

/***************************************************************************/
/* Feststellung, ob ein Bezeichner 'ptr' bereits in der Symboltabelle ist. */
/* Falls nicht, so wird ein entsprechender Eintrag erzeugt.                */
/* Rueckgabewert ist ein Zeiger auf den Eintrag. (Aufruf durch den Scanner)*/
/***************************************************************************/

void s_lookup(char *ptr, IdentStruct *ident)
{
    ident->sym=s_find(ptr);
    if(!ident->sym)
    {
	ident->sym=s_create(ptr);
	ident->is_old = False;
    }
    else
	ident->is_old = True;
}

/*************************************************************************/
/* Feststellung, ob eine Variable mehrfach definiert wurde, oder ob eine */
/* lokale Definition eine globale ueberdeckt.                            */
/* (Aufruf durch den Parser in der 'ident_list'-Regel)                   */
/*************************************************************************/

SymTabEntry *make_ident(IdentStruct *ident)
{
    SymTabEntry *entry=ident->sym;

    if (entry->s_type!=_UDEC) {
	if (entry->s_blknum==blknum) {
	  Msg(ERR,"multiple declaration of identifier %s.",entry->s_name);
	} else {
	  entry=s_create(entry->s_name);
	}
    }
    entry->s_special=0;
    entry->s_variations=0;
    entry->s_body=NULL;
    entry->s_proc=NULL;
    entry->s_parlist=NULL;
    entry->s_partype=_NOPAR;
    entry->s_reclevel=0;
    entry->s_set=NULL;

 /* entry->s_code=undefined;        */
 /* entry->s_init_or_cap=undefined; */

    return entry;
}

/*************************************************************************/

SymTabEntry *make_par(IdentStruct *ident)
{
    SymTabEntry *entry=ident->sym;

    if (entry->s_type!=_UDEC) {
		if (entry->s_blknum==blknum+1) {
			Msg(ERR,"multiple declaration of parameter %s.",entry->s_name);
		} else {
			entry=s_create(entry->s_name);
		}
	}

    entry->s_blknum =blknum+1;    /* lokal zum folgenden Prozedurrumpf! */

    entry->s_parlist=NULL;
    entry->s_partype=_VAL;        /* default */
    entry->s_reclevel=0;          /* default */
    entry->s_proc=ActProc;        /* doppeltgemoppelt, auch in GenParList */
    entry->s_body=NULL;

    entry->s_special=0;
    entry->s_variations=0;
    entry->s_set=NULL;

 /* entry->s_code=undefined;        */
 /* entry->s_init_or_cap=undefined; */

    return entry;
}

/*************************************************************************/

SymTabEntry *make_pardup(SymTabEntry *Entry)
{
  SymTabEntry *DupEntry;

  DupEntry = s_create(Entry->s_name);
  give_typ(Entry->s_type,Entry->s_set,Entry->s_init_or_cap);

  DupEntry->s_blknum      = Entry->s_blknum;
  DupEntry->s_proc        = Entry->s_proc;
  DupEntry->s_partype     = Entry->s_partype;
  DupEntry->s_reclevel    = 1;                    /* !!!! */
  DupEntry->s_special     = Entry->s_special;       
  DupEntry->s_variations  = Entry->s_variations;    

  return DupEntry;
}

/*************************************************************************/

void free_pardup(SymTabEntry *DupEntry)
{
  set_act_element(DupEntry->s_code);
  s_start = DupEntry->s_next;
  MYfree(DupEntry);
}

/************************************************************************/
/* Feststellung, ob eine Variable den geforderten Typ besitzt.          */
/* (Aufruf durch '?-primed' und 'r_w_channel')                          */
/* Bei Prozeduren wird noch deren Verwendung in s_special vermerkt, und */
/* bei Parameter-Prozeduren wird ActPar gesetzt.                        */
/************************************************************************/

void chk_ident(SymTabEntry *entry,int type)
{
    if ( ! GL_Frags )
      {
      if ( entry->s_type == _UDEC )
		  Msg(ERR,"undeclared identifier %s.",entry->s_name);
      else if (entry->s_type!=type) 
		  Msg(ERR,"incorrect use of identifier %s.",entry->s_name);
      }

    if (entry->s_type==_PROC || entry->s_type==_PPROC ) 
		entry->s_special=255;

    if (entry->s_type==_PPROC)
		ActPar=entry->s_parlist;
}

void chk2_ident(SymTabEntry *entry,int type1,int type2)
{
  if ( entry->s_type == _UDEC )
  {
    if ( ! GL_Frags )
      Msg(ERR,"undeclared identifier %s.",entry->s_name);
  }
  else if (entry->s_type!=type1 && entry->s_type!=type2) 
    Msg(ERR,"incorrect use of identifier %s.",entry->s_name);
}

int chk_proc_ident(SymTabEntry *entry,int type)
{
  if ( entry->s_type == _UDEC )
    { 
        if (!procgrplist)
            Msg(ERR,"undeclared identifier %s",entry->s_name);
        else
            list_add(forwardprocs,entry);
    }
  else if (entry->s_type!=type) 
    Msg(ERR,"incorrect use of identifier %s",entry->s_name);
  
  if (entry->s_type==_PROC || entry->s_type==_PPROC ) 
    entry->s_special |= ( _G_RW | _G_PRUPR );
  return entry->s_type == _UDEC;
}

/*********************************************************************/
/* Inkrementierung von 'blknum'.                                     */
/* (Aufruf durch den Parser am Beginn eines Blockes ('block'-Regel)) */
/*********************************************************************/

void inc_blknum(void)
{
    ++blknum;
}

/*******************************************************************/
/* Dekrementierung von 'blknum' und Entfernung von lokalen         */
/* Eintraegen aus der Symboltabelle. (Aber noch nicht loeschen,    */
/* sondern zurueckgeben !)                                         */
/* (Aufruf durch den Parser am Ende eines Blockes ('block'-Regel)) */
/*******************************************************************/

SymTabEntry *dec_blknum(void)
{
  SymTabEntry *ptr1,*ptr2,*memo;

  memo=ptr1=ptr2=s_start;

  while (ptr1->s_blknum==blknum) {
    set_act_element(ptr1->s_code);
    if (!(ptr1->s_special & (_G_RW | _G_PRUPR)) && (ptr1->s_type!=_UDEC) 
         && (ptr1->s_type!=_BLOCKNAME) && ! GL_NoOutput )
	/* diese Abfrage ist unsauber; eigentlich muesste s_special auch  */
	/* im NoOut-Fall gesetzt werden. Da mir das zu umstaendlich ist,  */
	/* verhindere ich es in der entsprechenden Funktion vermoege      */
	/* return und unterdruecke hier die Meldung. Sorry.               */
    {
      Msg(WAR,"identifier %s declared but not used.",ptr1->s_name);
#if 0
			ptr1->s_type=_UDEC; /* !!! */
#endif
    }
    ptr2=ptr1;
    ptr1=ptr1->s_next;
  }

  s_start=ptr1;
  --blknum;
  if (ptr1==ptr2) {
    memo=NULL;
  } else {
    ptr2->s_next=NULL;
  }

  return memo;
}

/************************************************************************/
/* Verleiht neuen Eintraegen in der Symboltabelle diverse Eigenschaften */
/* (Aufruf durch den Parser in der 'decl'-Regel)                        */
/* Bei Parameter-Prozedur-Deklarationen wird ActProc gesetzt.           */
/************************************************************************/

void give_typ(int type, SetRange * set, int init_or_cap)
{
  int i;
  SymTabEntry *ptr;

  if ((ptr=s_start)->s_type!=_UDEC && ! GL_Frags)
    Msg(ERR,"declaration of void identifier.");

  while (ptr->s_type==_UDEC) {
    ptr->s_set = set;
    if ( GL_StdInit && init_or_cap == NONE )
      init_or_cap =  set->next->low; /* Min. von set */
    switch(ptr->s_type=type) {

      case _VAR:
		ptr->s_code=calc_elements( ( i = cardin( set ) ) * i, ptr);
		if ( init_or_cap != NONE && ! is_elem( set, init_or_cap ) )
		{
			init_or_cap = NONE;
			Msg(ERR,"invalid initialization.");
		}
      case MNET_CONST:
		ptr->s_init_or_cap=init_or_cap;
	  break;

      case _CHANNEL:
      case _STACK:
		ptr->s_code=calc_elements( 2 * cardin( set ), ptr );
		/* There are no negative numbers so far...
		if (init_or_cap<0) {
			Msg(ERR,"invalid capacity.");
		}
		*/
		ptr->s_init_or_cap=init_or_cap;
	  break;

      case _PROC:
		ptr->s_init_or_cap=init_or_cap;
		ptr->s_code=0;
	  break;

      case _PPROC:
		ptr->s_init_or_cap=0;
		ptr->s_code=0;
		ActProc = ptr;
	  break;

      case _BLOCKNAME:
	  break;

      default:
		Msg(FAT,"default give_typ ?");
	}
	ptr=ptr->s_next;
  }
}
