/******************************************************************************/
/* FILE symtab.h                                                              */
/* version 2.2                                                                */
/* Burkhard Graves, 10.10.93                                                  */
/* Lars Jenner, Juni 1994                                                     */
/* last changes by Stephan Merkel, Mar. 95                                    */
/******************************************************************************/

#ifndef __SYMTAB_H__
#define __SYMTAB_H__

#include "../frontend/set.h"
#include "../frontend/list.h"

/******************************************************************************/
/* Globale Defines                                                            */
/******************************************************************************/

/* Fuer s_type (fortlaufende Nummerierung */

#define _UDEC       1
#define _VAR        2
#define _CHANNEL    3
#define _PROC       4
#define _PPROC      5
#define _STACK      6
#define MNET_CONST      7
#define _BLOCKNAME  8

/* ab hier nur verwendet fuer Referenzkennzeichnung */
#define _BEGIN      9
#define _ACTION    10
#define _PROCCALL  11

/******************************************************************************/
/* Globale Typdefinitionen                                                    */
/******************************************************************************/

typedef struct RefStruct_ {
  int                    start_l, start_c; /* Startzeile & Startspalte */
  int                      end_l,   end_c; /*   Endzeile &   Endspalte */
} RefStruct;

typedef struct ShortEntry_ {
  /* Achtung: Fuer das hl-backend muss s_ref an erster Stelle stehen und 
              s_type an zweiter Stelle!                                       */
  RefStruct              s_ref;         /* Definition im Quelltext            */
  char                   s_type;        /* Typ                                */
} ShortEntry, *ShortEntryPtr;

typedef struct symtabentry {
  RefStruct		 s_ref;		/* Definition im Quelltext	      */
  char     	      	 s_type;	/* Typ                                */
  char    	     	*s_name;	/* Name                               */
  int  	      		 s_blknum;	/* Blocknummer                        */
  struct symtabentry 	*s_next;	/* Verkettung                         */
  /*---- Falls Parameter : ---------------------------------------------------*/
  char                	 s_partype;	/* Par.typ (VAL,RES,NOPAR)            */
  struct symtabentry 	*s_proc; 	/* die zugehoerige Proz.              */
  char                	 s_reclevel;	/* fuer Rekursion                     */
  /*---- Falls Prozedur : ----------------------------------------------------*/
  struct symtabentry 	*s_parlist;	/* die Parameterliste                 */
  void		     	*s_body;	/* der Prozedurrumpf                  */
  /*--------------------------------------------------------------------------*/
  char     	      	 s_special; 	/* fuer diverse Zwecke, z.B. lokale/  */
					/* globale Benutzung von Variablen    */
  int 	      		 s_variations; 	/* Anzahl der Variationen             */
  int 	      		 s_code;     	/* Index des ersten Eintrages fuer    */ 
					/* diese symtabentry in elements[]    */
  SetRange    		*s_set;		/* Wertemenge                         */
  int  	      		 s_init_or_cap;	/* Initialisierungswert               */
					/*  oder Kapazitaet                   */
} SymTabEntry;

typedef struct IdentStruct_ {
    SymTabEntry         *sym;
    RefStruct            ref;
    bool                 is_old;
} IdentStruct;

extern T_list procgrplist;
extern T_list forwardprocs;
extern int blknum;

/******************************************************************************/
/* Globale Funktionen                                                         */
/******************************************************************************/

void              s_init PROTO (( void ));

void            s_lookup PROTO (( char *ptr, IdentStruct *ident));
SymTabEntry  *make_ident PROTO (( IdentStruct *ident ));
SymTabEntry    *make_par PROTO (( IdentStruct *ident ));
SymTabEntry *make_pardup PROTO (( SymTabEntry *Entry ));
void         free_pardup PROTO (( SymTabEntry *DupEntry ));

void           chk_ident PROTO (( SymTabEntry *entry,int type ));
void          chk2_ident PROTO (( SymTabEntry *entry, int type1, int type2 ));

int       chk_proc_ident PROTO (( SymTabEntry *entry,int type ));

void          inc_blknum PROTO (( void ));
SymTabEntry  *dec_blknum PROTO (( void ));

void            give_typ PROTO (( int type, SetRange * set, int init_or_cap ));
void        give_partype PROTO (( int partype ));


#endif /* __SYMTAB_H__ */

/******************************************************************************/
/* END OF FILE symtab.h                                                       */
/******************************************************************************/
