/******************************************************************************/
/* FILE tree.c                                                                */
/* version 2.2                                                                */
/* Burkhard Graves, 10.10.93                                                  */
/* Lars Jenner, Juni 1994                                                     */
/* last changes by Stephan Merkel, Mar. 95                                    */
/******************************************************************************/

#include <string.h>
#include <ctype.h>
#include "tree.h"
#include "calc.h"
#include "ops.h"
#include "bpn2pbc.h"
#include "../frontend/set.h"

/******************************************************************************/
/* Datenstrukturen                                                            */
/******************************************************************************/

extern FILE * yyout;
  /* defined in bpn2pbc.c */

char fullname[100];

SymTabEntry* ActPar=NULL;
SymTabEntry* ActProc=NULL;

/******************************************************************************/
/* Funktionen                                                                 */
/******************************************************************************/

/**********************************************************/
/* Invertierung der (lokalen) Symboltabelle (fuer's Auge) */
/**********************************************************/
static SymTabEntry *invert_list(SymTabEntry *ptr1)
{
	SymTabEntry *ptr2,*ptr3;

	ptr3=NULL;
	while (ptr1) {
		ptr1=(ptr2=ptr1)->s_next;
		ptr2->s_next=ptr3;
		ptr3=ptr2;
	}
	return ptr3;
}

/***************************************************************************/
/* LinkBody : Verknuepft den Bezeicher ProcIdent in der Symboltabelle mit  */
/*  dem Prozedurrumpf Body. Ausserdem wird s_special zurueckgesetzt, fuer  */
/*  fuer den Fall, dass es einen rekursiven Aufruf im Rumpf gab, der aber  */
/*  nicht als Verwendung zaehlt. (siehe chk_ident)                         */
/***************************************************************************/

void  LinkBody(SymTabEntry *ProcIdent,TreeNode *Body)
{
  ProcIdent->s_body    = Body;
  ProcIdent->s_special = 0;
}
/****************************************************************************/
/* LinkParlist: Verknuepft den Bezeicher ProcIdent in der Symboltabelle mit */
/*   der erzeugten Parameterliste, und setzt ActProc zurueck.(s.GenParList) */
/****************************************************************************/

void  LinkParlist(SymTabEntry *ProcIdent,SymTabEntry *ParList, int init_or_cap)
{
  ProcIdent->s_parlist = ParList;
  ProcIdent->s_init_or_cap = init_or_cap;
  ActProc = NULL;
}

/***************************************************************************/
/* GenParList : Haengt an die bisherige ParList den Parameter Par an, und  */
/*   gibt ihm den jeweiligen Parametertyp (RES oder VAL), sowie den        */
/*   ProcIdent ActProc der aktuellen Deklaration, der in give_typ gesetzt, */
/*   und in LinkParlist zurueckgesetzt wird.                               */
/***************************************************************************/

SymTabEntry 
  *GenParList(SymTabEntry *ParList, int Type, SymTabEntry *Par)
{
  SymTabEntry *ParPtr;

  if ( Par == NULL )
    {
      return ( NULL );
    } /* if */

  Par->s_parlist = NULL;
  Par->s_partype = Type;
  Par->s_proc    = ActProc;

  ParPtr = ParList;
  if (!ParPtr)
    return Par;
  else
  {
    while (ParPtr->s_parlist) 
      ParPtr = ParPtr->s_parlist;
    ParPtr->s_parlist = Par;
    return ParList;
  }
}

/***************************************************************************/
/* GenArgList : Haengt an die bisherige ArgList das Argument Arg an, wofuer*/
/*   ein neues Element NewArg geschaffen wird. Gibt NewArg den vom Para-   */
/*   meter geforderten Typ (_RES oder _VAL) und prueft die Vertraeglichkeit*/
/*   mit dem tatsaechlich uebergebenen Ausdruck (_IDENT oder _VAL). value  */
/*   und result werden berechnet, wofuer (Rekursion!) ein temporaeres      */ 
/*   Symbol DupPar in der Symboltabelle erzeugt wird.                      */
/***************************************************************************/

#define Ident ((SymTabEntry*)arg)
#define Arg   ((TreeNode*   )arg)
ArgListEntry *GenArgList( ArgListEntry *ArgList, int ArgType, void *arg )
{
  ArgListEntry *NewArg,*ArgPtr;
  SymTabEntry  *DupPar;


  if ( ActPar == NULL )
  {
    Msg(ERR,"too many arguments.");
    return ( NULL );
  } /* if */

  if ((NewArg=(ArgListEntry*)MYmalloc(sizeof(ArgListEntry)))==NULL)  
    Msg(FAT,"no more memory available for argument list!");


  NewArg->result  = NULL;
  NewArg->partype = ActPar->s_partype;

		/* Ident kann auch ActPar sein ! bei Rekursion ! */

  switch (ArgType) 
  {
    case _IDENT:
      NewArg->ident = Ident;
      switch (Ident->s_type)
      {
		case _UDEC :
		  if ( ! GL_Frags )
		    {
		      Msg(ERR,"undeclared identifier %s.",Ident->s_name);
		    } /* if */
		break;

		case _VAR:
		  switch (NewArg->partype)
		  {
			case _RES :
			  DupPar = make_pardup(ActPar);      
			  un_primed(DupPar);
			  post_primed(Ident);
			  calc_bin(equal);
			  NewArg->result = translate();
			  free_pardup(DupPar);
			case _VAL :
			case _CONSTPAR:
			  DupPar = make_pardup(ActPar);      
			  un_primed(Ident);
			  post_primed(DupPar);
			  calc_bin(equal);
			  NewArg->result = translate();
			  free_pardup(DupPar);
			break;

			case _REF :
			  if (ActPar->s_type!=_VAR && ! GL_Frags)
				Msg(ERR,"variable identifier expected as reference parameter %s.",
				ActPar->s_name);
			  if ( ! sub_set( ActPar->s_set, Ident->s_set ) )
				Msg(WAR,"parameter and argument have incompatible ranges.");
			  Ident->s_special |= _G_PRUPR;
			break;

			default:
			  Msg(FAT,"default VAR NewArg->partype ?");
		  }
		break;

		case _CHANNEL:
		case _STACK:
		  switch (NewArg->partype)
		  {
			case _RES :
			  DupPar = make_pardup(ActPar);      
			  un_primed(DupPar);
			  r_w_channel(Ident,_LG_WRITING);
			  calc_bin(equal);
			  NewArg->result = translate();
			  free_pardup(DupPar);
			case _VAL :
			case _CONSTPAR :
			  DupPar = make_pardup(ActPar);      
			  r_w_channel(Ident,_LG_READING);
			  post_primed(DupPar);
			  calc_bin(equal);
			  NewArg->result = translate();
			  free_pardup(DupPar);
			break;

			case _REF :
			  if (ActPar->s_type!=Ident->s_type /* _CHANNEL */ && ! GL_Frags)
				Msg(ERR,"%s identifier expected as reference parameter %s.",(ActPar->s_type!=_CHANNEL)?"channel":"stack",ActPar->s_name);
			  if ( ! sub_set( ActPar->s_set, Ident->s_set ) )
				Msg(WAR,"parameter and argument have incompatible ranges.");
			  Ident->s_special |= _G_RW;
			break;

			default:
			  Msg(FAT,"default CHANNEL NewArg->partype ?");

		  }                                                        
		break;

		case _PROC :
		case _PPROC :
			Msg(ERR," incorrect use of identifier %s as parameter.",Ident->s_name);       
		break;

		default:
			Msg(FAT,"default argidenttype ?");
	  }
	  break;

	  case _VAL:

		NewArg->ident = NULL;

		if (ActPar->s_partype==_RES)
			Msg(ERR,"pass by result expected for parameter %s.",ActPar->s_name);
		if (ActPar->s_partype==_REF)
			Msg(ERR,"pass by reference expected for parameter %s.",ActPar->s_name);
		DupPar = make_pardup(ActPar);      
		post_primed(DupPar);
		calc_bin(equal);
		NewArg->result = translate();
		free_pardup(DupPar);

	  break;

	  default:
		Msg(FAT,"default argtype ?");

  }

  NewArg->next = NULL;
  ActPar = ActPar->s_parlist;

  ArgPtr = ArgList;
  if (!ArgPtr)
  {
    return NewArg;
  }
  else
  {
    while (ArgPtr->next) 
      ArgPtr = ArgPtr->next;
    ArgPtr->next = NewArg;
    return ArgList;
  }
}
#undef Ident
#undef Arg

/***************************************************************************/
/* check_parm_list : Nachtraegliche Ueberpruefung der Korrektheit von	   */
/*   Argumenten, falls eine Vorwaertsreferenz innerhalb von procgrp	   */
/*   auftrat. Arglist ist sind die tatsaechlichen Parameter im Prozedur-   */
/*   Aufruf, ActPar die bei der Prozedur-Deklaration verwendeten.	   */
/***************************************************************************/

#define Ident (ArgList->ident)
#define Arg   (ArgList->arg)
void check_parm_list( ArgListEntry *ArgList, SymTabEntry *ActPar)
{
  if (!ArgList && !ActPar) return;
  if (!ArgList) { Msg(ERR,"too few arguments"); return; }
  if (!ActPar) { Msg(ERR,"too many arguments"); return; }

  ArgList->partype = ActPar->s_partype;

  if (Ident) /* case _IDENT: */
  {
      switch (Ident->s_type)
      {
      case _UDEC:
	  if ( !GL_Frags )
	    Msg(ERR,"undeclared identifier %s",Ident->s_name);
	break;
	
      case _VAR:
	switch (ArgList->partype)
	  {
	  case _RES:
	    if ( ! sub_set( ActPar->s_set, Ident->s_set ) )
	      Msg(WAR,"argument %s must have superset as range", Ident->s_name);
	    /* ActPar=Ident' */
	    /* !!! hier kein break !!! */
          case _CONSTPAR:
	  case _VAL:
	    if ( ! sub_set( Ident->s_set, ActPar->s_set ) )
	      Msg(WAR,"argument %s must have subset as range",Ident->s_name);
	    Ident->s_special |= _G_PRIMED;
	    /* Ident=ActPar' */
	    break;
	    
	  case _REF:
	    if (ActPar->s_type!=_VAR && ! GL_Frags)
	      Msg(ERR,"argument %s has incompatible type",Ident->s_name);
	    if ( ! sub_set( ActPar->s_set, Ident->s_set ) )
	      Msg(WAR,"argument %s must have superset as range",Ident->s_name);
	    Ident->s_special |= _G_PRUPR;
	    break;
	    
	  default:
	    Msg(FAT,"Internal Error: Unknown parameter type at VAR !");
	  }
	break;
	
      case _CHANNEL:
      case _STACK:
	switch (ArgList->partype)
	  {
	  case _RES :
          case _CONSTPAR :
	  case _VAL :
	    Msg(ERR,"argument %s has incompatible type",Ident->s_name);
	    break;

	  case _REF :
	    if (ActPar->s_type!=Ident->s_type && ! GL_Frags)
	      Msg(ERR,"argument %s has incompatible type",Ident->s_name);
	    if ( ! sub_set( ActPar->s_set, Ident->s_set ) )
	      Msg(WAR,"argument %s must have superset as range",Ident->s_name);
	    if ( ActPar->s_init_or_cap != Ident->s_init_or_cap )
	      Msg(ERR,"argument %s has incompatible capacity",Ident->s_name);
	    Ident->s_special |= _G_RW;
	    break;

	  default:
	    Msg(FAT,"Internal Error: Unknown parameter type at CHAN/STACK !");
	    
	  }                                                        
	break;

      case _PROC:
      case _PPROC:
	Msg(ERR,"incorrect use of procedure %s as argument",Ident->s_name);  
	break;
	
      default:
	Msg(FAT,"Internal Error: Unknown identifier type !");
      }
  }
  else if (Arg)	/* case _VAL: */
  {
    if (ActPar->s_partype==_RES)
      Msg(ERR,"pass by result expected for parameter %s",ActPar->s_name);
    if (ActPar->s_partype==_REF)
      Msg(ERR,"pass by reference expected for parameter %s",ActPar->s_name);
    /* Expr = ActPar' */
  }
  else  /* default: */
  {
    Msg(FAT,"Internal Error: Unknown argument type !");

  }

  check_parm_list(ArgList->next,ActPar->s_parlist);
}
#undef Ident
#undef Arg


/****************************************/
/* Loeschen der (lokalen) Symboltabelle */
/****************************************/

static void delete_list( char what, void *ptr1)
{
	void *ptr2;

	while (ptr1) {
	   switch (what)        
	   {
	     case _BLOCK :
	       ptr1=((SymTabEntry*)(ptr2=ptr1))->s_next;
	       MYfree(((SymTabEntry*)ptr2)->s_name);
	     break;

	     case _PCALL :
	       ptr1=((ArgListEntry*)(ptr2=ptr1))->next;
	     break;

	     default: Msg(FAT," what delete_list ?");
	   }
	   MYfree(ptr2);
	}

}

static int    alt_set PROTO ((TreeNode *node,int mode,int flag));
static int any_repeat PROTO ((TreeNode *node));
static void block(TreeNode *node);

/* create symbol names. Only safe to use with one call per line due to
   use of scope memory (should be global perhaps?) */

char *fn(SymTabEntry *x)
{ 
  char numtxt[10];

  if (x->s_proc!=NULL)
  {
    strcpy(fullname,x->s_proc->s_name);
    fullname[0]=tolower(fullname[0]);
    if (x->s_proc->s_blknum>1)
    {
	  sprintf(numtxt,"%d",x->s_proc->s_blknum);
	  strcat(fullname,numtxt);
    }
    sprintf(numtxt,"%d",x->s_reclevel);
    strcat(fullname,numtxt);
    strcat(fullname,x->s_name);
  }
  else
  {
    strcpy(fullname,x->s_name);
    if (x->s_blknum>1)
    {
	  sprintf(numtxt,"%d",x->s_blknum);
	  strcat(fullname,numtxt);
	}
  }

  return fullname;
}


/********************************/
/* Ausgabe aller Scope-Aktionen */
/********************************/

static int scopeactions(char where, SymTabEntry *ptr)
{
  int flag=0;
  int i,j;
  Set curr_i, curr_j;
  FILE *stream=yyout;

  while (ptr) 
  {
    if (where==_PCALL) 
	ptr->s_reclevel=1;
    if ((where==_PCALL && ptr->s_partype!=_REF) || ptr->s_partype==_NOPAR)
    switch (ptr->s_type) 
    {
      case _UDEC:
      case MNET_CONST:
      case _BLOCKNAME:
      break;

      case _VAR:
	  if (!flag) 
	  {
	     flag=1;
	     fprintf(stream,"[{%s",fn(ptr));
	  } 
	  else
	  {
	     fprintf(stream,",%s",fn(ptr));
	  } /* else */
	  if(!GL_shrthnd)
	  {
	    curr_i = ptr->s_set->next;
	    i = curr_i->low - 1;
	    do
	      {
		if ( i < curr_i->high )
		  {
		    ++i;
		  } /* if */
		else
		  {
		    curr_i = curr_i->next;
		    i = curr_i->low;
		  } /* else */

		curr_j = ptr->s_set->next;
		j = curr_j->low - 1;
		do
		  {
		    if ( j < curr_j->high )
		      {
			++j;
		      } /* if */
		    else
		      {
			curr_j = curr_j->next;
			j = curr_j->low;
		      } /* else */
		    fputc(',',stream);
		    fprintf(stream,F_VAR,fn(ptr),transl(i),transl(j));
		  } /* do ( curr_j ) */
		while ( j < SET_MAX( ptr->s_set ) );
	      } /* do ( curr_i ) */
	    while ( i < SET_MAX( ptr->s_set ) );
	  } /* if ( !GL_shrthnd ) */
      break;

      case _CHANNEL:
      case _STACK:
		if (!flag) 
		{
		  flag=1;

		  fprintf(stream,"[{%s",fn(ptr));
		} 
		else
		{
		  fprintf(stream,",%s",fn(ptr));
		} /* else */
		fprintf(stream,",%s!!",fn(ptr));
		fprintf(stream,",%s??",fn(ptr));
		if(!GL_shrthnd)
		{
		  curr_i = ptr->s_set->next;
		  i = curr_i->low - 1;
		  do
		    {
		      if ( i < curr_i->high )
			{
			  ++i;
			} /* if */
		      else
			{
			  curr_i = curr_i->next;
			  i = curr_i->low;
			} /* else */
		      fprintf(stream,F_CHAN_R,fn(ptr),transl(i));
		      fprintf(stream,F_CHAN_W,fn(ptr),transl(i));
		    } /* do */
		  while ( i < SET_MAX( ptr->s_set ) );
		} /* if ( !GL_shrthnd ) */
	  break;

      case _PROC:
      case _PPROC:
      break;

      default:
		Msg(FAT,"default scopeactions ?");
	}
	switch (where)
	{
	  case _BLOCK: ptr=ptr->s_next;                       break;
	  case _PCALL: ptr->s_reclevel=0; ptr=ptr->s_parlist; break;
	  default:     Msg(FAT,"default where scopeactions ?");
	}
  }

  if (flag)
  {
    fprintf(stream,"}:\n");
  } /* if */
  return flag;

}

/*****************************/
/* Ausgabe der Deklarationen */
/*****************************/

static void declarations(char where, SymTabEntry *ptr)
{
  int flag;
  FILE *stream=yyout;

  flag=0;
  while (ptr) {
    if (where==_PCALL) ptr->s_reclevel=1;
    if ((where==_PCALL && ptr->s_partype!=_REF) || ptr->s_partype==_NOPAR)
	switch(ptr->s_type) 
	{

	case _UDEC:
	case MNET_CONST:
	case _BLOCKNAME:
	break;

	case _VAR:
		  flag=1;

		  fprintf(stream,"D_BOX(%s,",fn(ptr));
		  if (ptr->s_init_or_cap!=NONE) 
		    {
		      fprintf(stream,F_INT_KOMMA,transl(ptr->s_init_or_cap));
		    } /* if */
		  write_set( stream, ptr->s_set );
		  fprintf( stream, ") || " );
	    break;

	case _CHANNEL:
	case _STACK:
		  flag=1;

		  fprintf(stream,"%c_BOX(%s,",(ptr->s_type==_STACK)?'S':'C',fn(ptr));
		  fprintf(stream,F_INT_KOMMA,ptr->s_init_or_cap);
		  write_set( stream, ptr->s_set );
		  fprintf( stream, ") || " );
	    break;

	case _PROC:
	case _PPROC:
	break;

	default:
		  Msg(FAT,"default declarations ?");
	}
	switch (where)
	{
	  case _BLOCK: ptr=ptr->s_next;                       break;
	  case _PCALL: ptr->s_reclevel=0; ptr=ptr->s_parlist; break;
	  default:     Msg(FAT,"default where declarations ?");
	}
  }
  if (flag)
  {
    fputc('\n',stream);
  } /* if */
}

/**************************************/
/* Ausgabe der Terminierungs-Aktionen */
/**************************************/

static void t_actions(char where, SymTabEntry *ptr)
{
  FILE *stream=yyout;
  Set curr;
  int i,first;

  first=1;
  while (ptr) {
    if (where==_PCALL) ptr->s_reclevel=1;
    if ((where==_PCALL && ptr->s_partype!=_REF) || ptr->s_partype==_NOPAR)
    if (ptr->s_type!=_PROC && ptr->s_type!=_PPROC && ptr->s_type != MNET_CONST)
    {
      if (first)
      {
	if ( GL_SeqTerm )
	{
	  fprintf(stream,";");
	} /* if */
	else 
	{
	  fprintf(stream,";{");
	} /* else */
	first=0;
      } /* if */
      else
      {
	if ( GL_SeqTerm )
	{
	  fprintf(stream,";");
	} /* if */
	else 
	{
	  fprintf(stream,",");
	} /* else */
      }/* else */

      if (ptr->s_init_or_cap
          && (ptr->s_type==_STACK || ptr->s_type==_CHANNEL))
        {
          fprintf(stream,"[{}*(");

          curr = ptr->s_set->next;
	  i = curr->low - 1;
	  do
	    {
	      if ( i < curr->high )
		{
		  ++i;
		} /* if */
	      else
		{
		  curr = curr->next;
		  i = curr->low;
		} /* else */
              fprintf(stream,"%s?%i",fn(ptr),transl(i));
              if ( i < SET_MAX( ptr->s_set ) )
		{
                  fprintf(stream,"[]");
		} /* if */
	    } /* do */
	  while ( i < SET_MAX( ptr->s_set ) );

          fprintf(stream,")*%s]",fn(ptr));
        } /* if */
      else
        {
	  fprintf(stream,fn(ptr));
        } /* else */

    }
    switch (where)
    {
      case _BLOCK: ptr=ptr->s_next;                       break;
      case _PCALL: ptr->s_reclevel=0; ptr=ptr->s_parlist; break;
      default:     Msg(FAT,"default where t_actions ?");
    }
  }
  if (!first)
    {
    if ( GL_SeqTerm )
      fprintf(stream,"]");
    else
      fprintf(stream,"}]");
    }
}

/**************************************************/
/* Sind ueberhaupt repeat-Anweisungen vorhanden ? */
/**************************************************/

static int any_repeat(TreeNode *node)
{
	switch(node->type) {
	case _REPEAT:
		return 1;
	case _EXIT:
		return 0;
	case _CHOICE:
		if (any_repeat(node->op1))
			return 1;
		return any_repeat(node->op2);
	default:
		Msg(FAT,"default any_repeat ?");
	}
	return 0;
}

/*****************************************************************************/

static void pcall ( SymTabEntry *proc, ArgListEntry * arglist )
{
  ArgListEntry *arg;
  SymTabEntry  *par;
  FILE         *stream = yyout;
  Set          curr_i, curr_j;
  char         first = 1;
  int      i,j;

  scopeactions(_PCALL,proc->s_parlist);
  if (!GL_shrthnd)
	declarations(_PCALL,proc->s_parlist);

  arg = arglist;
  while (arg) 
  {
	if ((arg->partype!=_RES) && arg->result) 
	{
	  fprintf(stream,"(%s);\n",arg->result);

	  MYfree(arg->result);
	}
	arg = arg->next;
  }

  fprintf(stream,"((%s) [",fn(proc));

  par = proc->s_parlist;
  arg = arglist; 
  while (par) 
  {
	par->s_reclevel=0;

	fprintf(stream,"%s%s->",(first ? " ":", "),fn(par));

	if (par->s_partype==_REF)
	{
	  fprintf(stream,"%s",fn(arg->ident));

	  switch (par->s_type)
	  {
		case _VAR:
		  curr_i = par->s_set->next;
		  i = curr_i->low - 1;
		  do
		    {
		      if ( i < curr_i->high )
			{
			  ++i;
			} /* if */
		      else
			{
			  curr_i = curr_i->next;
			  i = curr_i->low;
			} /* else */

		      curr_j = par->s_set->next;
		      j = curr_j->low - 1;
		      do
			{
			  if ( j < curr_j->high )
			    {
			      ++j;
			    } /* if */
			  else
			    {
			      curr_j = curr_j->next;
			      j = curr_j->low;
			    } /* else */
			  fprintf( stream, MAPVAR1,
				   fn(par), transl(i), transl(j));
			  fprintf( stream, MAPVAR2,
				   fn(arg->ident), transl(i), transl(j));
			} /* do */
		      while ( j < SET_MAX( par->s_set ) );
		    } /* do ( curr_i ) */
		  while ( i < SET_MAX( par->s_set ) );
		  break;
		case _CHANNEL:
		case _STACK:
		  curr_i = par->s_set->next;
		  i = curr_i->low - 1;
		  do
		    {
		      if ( i < curr_i->high )
			{
			  ++i;
			} /* if */
		      else
			{
			  curr_i = curr_i->next;
			  i = curr_i->low;
			} /* else */
		      fprintf( stream, MAPCHAN_R1, fn(par), transl(i));
		      fprintf( stream, MAPCHAN_R2, fn(arg->ident), transl(i));
		      fprintf( stream, MAPCHAN_W1, fn(par), transl(i));
		      fprintf( stream, MAPCHAN_W2, fn(arg->ident), transl(i));
		    } /* do */
		  while ( i < SET_MAX( par->s_set ) );
		break;
		default:
		  Msg(FAT,"default MAP ref pcall ?");
	  }
	}
	else
	{
	  par->s_reclevel=1;

	  fprintf(stream,"%s",fn(par));

	  curr_i = par->s_set->next;
	  i = curr_i->low - 1;
	  do
	    {
	      if ( i < curr_i->high )
		{
		  ++i;
		} /* if */
	      else
		{
		  curr_i = curr_i->next;
		  i = curr_i->low;
		} /* else */

	      curr_j = par->s_set->next;
	      j = curr_j->low - 1;
	      do
		{
		  if ( j < curr_j->high )
		    {
		      ++j;
		    } /* if */
		  else
		    {
		      curr_j = curr_j->next;
		      j = curr_j->low;
		    } /* else */
		  par->s_reclevel=0;
		  fprintf(stream,",%s_%d_%d->",fn(par),transl(i),transl(j));

		  par->s_reclevel=1;

		  fprintf(stream,"%s_%d_%d",fn(par),transl(i),transl(j));

		  par->s_reclevel=0;
		} /* do j */
	      while ( j < SET_MAX( par->s_set ) );

	    } /* do i */
	while ( i < SET_MAX( par->s_set ) );
	}
	first=0;
	par = par->s_parlist;
	arg = arg->next;
  }
  fprintf(stream," ])\n");

  arg = arglist;
  while (arg) 
  {
	if ((arg->partype==_RES) && arg->result)
	{
	  fprintf(stream,";(%s)\n",arg->result);

	  MYfree(arg->result);
	}
	arg = arg->next;
  }

  t_actions(_PCALL,proc->s_parlist);

}

/***************************************/
/* Abarbeitung eines com-Syntaxknotens */
/***************************************/

void com(TreeNode *node)
{
  FILE * stream = yyout;

  switch(node->type) {

    case _BLOCK:
      block(node);
    break;

    case _EXPR:
      fprintf(stream,"(%s)",(char *)node->op1);

      MYfree(node->op1);
    break;

    case N_VAR:
      fprintf(stream,"(%s",fn((SymTabEntry *)node->op1));
      switch((int)node->op2) {
	case _EMPTY:
          fprintf(stream,"??"); break;
	case _FULL:
          fprintf(stream,"!!"); break;
      }
     fprintf(stream,")");
     break;

    case _SEQ:
      com(node->op1);

      fprintf(stream," ; ");

      com(node->op2);
    break;

    case _PAR:
      com(node->op1);

      fprintf(stream," ||\n");

      com(node->op2);
    break;

    case _DO:
      if (any_repeat(node->op1)) {

        if (node->op2) {
          fprintf(stream,"[(");
          com(node->op2);
          fprintf(stream,") * ");
        } else {
	  fprintf(stream,"[ {} * ");
        }

	alt_set(node->op1,_REPEAT,0);

	fprintf(stream," * ");

	if (!alt_set(node->op1,_EXIT,0))
	{
	  fprintf(stream,"STOP");
	} /* if */

	fputc(']',stream);

      } else {
        if (node->op2) {
	  fputc('(',stream);
          com(node->op2);
	  fputc(')',stream);
          fprintf(stream," ; ");
        }

	fputc('(',stream);

	alt_set(node->op1,_EXIT,0);

	fputc(')',stream);
      }
    break;

    case _CALL:
	  fprintf(stream,"(%s)",fn((SymTabEntry*)(node->op1)));
	break;

	case _PCALL:
	  pcall((SymTabEntry*)(node->op1),(ArgListEntry*)(node->op2));
	break;

    default:
      Msg(FAT,"default com ?");
  }
}

/*******************************************/
/* Abarbeitung eines alt_set-Syntaxknotens */
/*******************************************/

static int alt_set(TreeNode *node,int mode,int flag)
{
	FILE *stream=yyout;

	switch(node->type) {
	case _REPEAT:
	case _EXIT:
		if (mode==node->type) {
			if (flag)
			  {
			    fprintf(stream," [] ");
			  } /* if */

			fputc('(',stream);

			com(node->op1);

			fputc(')',stream);

			flag=1;
		}
		break;
	case _CHOICE:
		flag=alt_set(node->op1,mode,flag);
		flag=alt_set(node->op2,mode,flag);
		break;
	default:
		Msg(FAT,"default alt_set ?");
	}
	return flag;
}

/*****************************************/
/* Abarbeitung eines block-Syntaxknotens */
/*****************************************/

static void block(TreeNode *node)
{
  FILE *stream = yyout;
  int i;
  SymTabEntry *ptr;

  fputc( '\n', stream );

  node->op1=(TreeNode *)invert_list((SymTabEntry *)node->op1);
  if ((i=scopeactions(_BLOCK,(SymTabEntry *)node->op1))!=0)
    if (!GL_shrthnd)
      declarations(_BLOCK,(SymTabEntry *)node->op1);
  fprintf(stream,"(\n");
  com(node->op2);
  fprintf(stream,"\n)");
  if (i)
    t_actions(_BLOCK,(SymTabEntry *)node->op1);

  ptr=invert_list((SymTabEntry *)node->op1);
  while (ptr)
  {
    if (ptr->s_type==_PROC || ptr->s_type==_PPROC)
    {
      fprintf(stream,"\n[%s",fn(ptr));
      fprintf(stream,"<-@%s.\n",fn(ptr));
      block((TreeNode*)ptr->s_body);
      delete_tree((TreeNode*)ptr->s_body);
      fprintf(stream,"]");
    }
    ptr=ptr->s_next;
  }
  fprintf( stream, "\n" );
}

/**************************************/
/* Erzeugung eines Syntaxbaum-Knotens */
/**************************************/

TreeNode *GenNode(int type,TreeNode *op1,TreeNode *op2)
{
    TreeNode *node;

    if ( ActPar != NULL ) 
    {
      Msg(ERR,"too few arguments.");
      return ( NULL );
    }

    if ( GL_NoOutput )
      {
	return ( NULL );
      } /* if */

    node=(TreeNode *)MYmalloc(sizeof(TreeNode));
    node->type=type;
    node->op1=op1;
    node->op2=op2;
    return node;
}

/*****************************/
/* Loeschen des Syntaxbaumes */
/*****************************/

void delete_tree(TreeNode *node)
{
  if ( node )
    {
	switch(node->type) {
	case _BLOCK:
		delete_list(_BLOCK,node->op1);
		delete_tree(node->op2);
		break;
	case _DO:
	case _REPEAT:
	case _EXIT:
	case _CALL:
		break;
	case _PCALL:
		delete_list(_PCALL,node->op2);
		break;
	case _SEQ:
	case _PAR:
	case _CHOICE:
		delete_tree(node->op1);
		delete_tree(node->op2);
		break;
	case _EXPR:      /* sind schon weg ! */
		break;
	default:
		Msg(FAT,"default delete_tree ?");
	} /* switch */
	MYfree(node);
    } /* if */
}

/******************************************************************************/
/* END OF FILE tree.c                                                         */
/******************************************************************************/
