// bpn2tex.cc, Pretty Printer for B(PN)^2 to LaTeX2e
// (c) 1996 by Burkhard Bieber
// started: 06.02.96, last-update: 04.06.96
// CS, 22.5.2001: Some updates for C++ style

  //##########
  // includes
  //##########

#include <iostream.h>
#include <fstream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

  //#########
  // defines
  //#########

const char *const VERSION_STR = "bpn2tex, Version 0.05 (20010522),\
 1996 by Burkhard Bieber,\n updates by Christian Stehno in 2001";

  // Fehlernummern
enum err_num {ERR_NO_FILE, ERR_IN_FILE, ERR_OUT_FILE, ERR_INTERNAL};

  // Statusgeschichten des Zeilenparsers
enum lparse_stat {SEARCH, WORD, NO_WORD}; 

  // buffer sizes, just guesses
#define IN_BUF_SIZE  1024
#define OUT_BUF_SIZE 4096

  // TeX indents in ex
#define INDENT_SPACE_SIZE 1.5
#define INDENT_TAB_SIZE (6.0*INDENT_SPACE_SIZE)

  // Schriftstilzuordnungen in LaTeX2e
  // Die Klammern am Ende sind wichtig!
#define COMMENT_STYLE "\\itshape{}"
#define KEYWORD_STYLE "\\bfseries{}"

  //###################
  // Globale Variablen
  //###################

//changed keyword to contain null string at the end to detect
//end without n_keywords
char *const keyword[] = {"begin", "end", "var", "const", "chan", "stack", "of",
  "init", "do", "od", "enter", "repeat", "exit", "parallel", "choice", 
  "not", "and", "or", "true", "false", "proc", "procgrp", "endgrp", "ref", "\0"};

  //############
  // Prototypen
  //############

void display_usage(char *prgname);
void display_error(int err_num, char *add_err_text);
int search_keyword(char *s);
void transform1line(char *out_str, char *in_str);
void totex(char *tex_str, char *no_tex_str);

  //############
  // Funktionen
  //############

void display_usage(char *prgname)
  // Ausgabe der Programmgebrauchsinformationen.
{
  cout << VERSION_STR << endl;
  cout << "  pretty printer for B(PN)^2-source" << endl;
  cout << "  output is formated in LaTeX2e style" << endl;
  cout << "  output file will be overwritten without warning" << endl;
  cout << "usage: " << prgname << " bpn2_filename latex_filename" << endl;
  exit(ERR_NO_FILE);
}  // display_usage

void display_error(int err_num, char *add_err_text)
  // Ausgabe einer Fehlermeldung und Programmende.
{
  switch (err_num) {
    case ERR_IN_FILE:
      cerr << "error: input file " << add_err_text << " unusable" << endl;
      break;
    case ERR_OUT_FILE:
      cerr << "error: output file " << add_err_text << " unusable" << endl;
      break;
    case ERR_INTERNAL:
      cerr << "error: " << add_err_text << endl;
      break;
    default:
      cerr << "error: strange, unknown error number " << err_num << endl;
      break;
  }  // switch on error number
  exit(err_num);
}  // display_error

int search_keyword(char *s)
  // Looking for keywords of B(PN)^2.
{
  int i = 0;

  while ((*keyword[i])) {
    if (strcasecmp(keyword[i], s) == 0)
      return i;
    else
      i++;
  }  // while i on keywords
  return -1;
}  // search_keyword

void totex(char *tex_str, char *no_tex_str)
  // Einen String in TeX-bar machen.
{
  char *s = no_tex_str;

  strcpy(tex_str, "");
  while (strlen(s) > 0) {
    if ((strlen(s) >= 2) && (s[0] == '[') && (s[1] == ']')) {
      strcat(tex_str, "\\framebox[1.1ex]{\\rule[-0.2ex]{0mm}{1.5ex}}");
      s += 2;
    }  // if evtl. die 2-Zeichen-Kombination []
    else if ((strlen(s) >= 2) && (s[0] == '|') && (s[1] == '|')) {
      strcat(tex_str,
  "\\rule[-0.7ex]{0.5mm}{2.5ex}\\hspace*{0.7mm}\\rule[-0.7ex]{0.5mm}{2.5ex}");
      s += 2;
    }  // if evtl. die 2-Zeichen-Kombination ||
    else {
      switch (*s) {
          // die blichen, nicht direkt TeX-baren Zeichen
        case '{':
        case '}':
          sprintf(tex_str, "%s\\ensuremath{\\%c}", tex_str, *s);
          break;
        case '_':
        case '#':
        case '$':
        case '&':
        case '~':
        case '^':
        case '%':
          sprintf(tex_str, "%s\\%c", tex_str, *s);
          break;
          // Umlaute und 
        case '':
          strcat(tex_str, "\\""a");
          break;
        case '':
          strcat(tex_str, "\\""o");
          break;
        case '':
          strcat(tex_str, "\\""u");
          break;
        case '':
          strcat(tex_str, "\\ss{}");
          break;
        case '':
          strcat(tex_str, "\\""A");
          break;
        case '':
          strcat(tex_str, "\\""O");
          break;
        case '':
          strcat(tex_str, "\\""U");
          break;
          // Mathezeichen
        case '<':
        case '>':
          sprintf(tex_str, "%s$%c$", tex_str, *s);
          break;
        case '|':
          strcat(tex_str, "$\\mid$");
          break;
        case '\\':
          strcat(tex_str, "$\\backslash$");
          break;
        case '-':
          strcat(tex_str, "-{}");
          break;
        case '':
          strcat(tex_str, "$^2$");
          break;
        default:  // no bad character
          sprintf(tex_str, "%s%c", tex_str, *s);
          break;
      }  // switch on character
      s++;
    }  // else, keine Choice oder Parallel
  }  // while noch nicht alle Zeichen bearbeitet

}  // totex

void transform1line(char *out_str, char *in_str)
  // Die B(PN)^2-Zeile in eine LaTeX2e-Zeile transformieren.
{
  char h_str[OUT_BUF_SIZE];
  char tok_str[OUT_BUF_SIZE];
  int state;
  char *be, *en;

    // Vorbelegung ist nichts
  strcpy(out_str, "");

  if (strlen(in_str) == 0) {
    return;
  }  // if Leerzeile

    // handle leading blanks and tabs
  float indent_size = 0;
  while ((*in_str == ' ') || (*in_str == '\t')) {
    if (*in_str == ' ')
      indent_size += INDENT_SPACE_SIZE;
    if (*in_str == '\t')
      indent_size += INDENT_TAB_SIZE;
    in_str++;
  }  // while leading blanks or tabs
  if (indent_size != 0) {
    sprintf(h_str, "\\hspace*{%0.0fex}", indent_size);
    strcat(out_str, h_str);
  }  // if indent needed

    // restliche Zeile behandeln
  while (*in_str != '\0') {
      // Kommentare erkennen
    if ((strlen(in_str) >= 2)
      && (*in_str == '/') && (*(in_str + 1) == '/')) {
      totex(h_str, in_str);
      sprintf(out_str, "%s{%s%s}", out_str, COMMENT_STYLE, h_str);
      *in_str = '\0';  // mal eben brutal das Ende markieren
    }  // if Kommentar erkannt '//'
    else {
     strcat(in_str, "\n");  // besser fr die Verarbeitung
     state = SEARCH;
     while (*in_str != '\0') {
        switch (state) {
          case SEARCH:
            if (isalpha(*in_str))
              state = WORD;
            else
              state = NO_WORD;
            be = in_str;
            en = in_str;
            break;
          case WORD:
            if (isalpha(*in_str))
              en = in_str;
            else {  // Wortende erreicht
              strncpy(tok_str, be, en - be + 1);
              tok_str[en - be + 1] = '\0';
              if (search_keyword(tok_str) == -1)
                totex(h_str, tok_str);
              else
                sprintf(h_str, "{%s%s}", KEYWORD_STYLE, tok_str);
              strcat(out_str, h_str);
              in_str--;  // unschn
              state = SEARCH;
            }  // else, Wortende erreicht
            break;
          case NO_WORD:
            if (isalpha(*in_str) || (*in_str == '\n')) {
              strncpy(tok_str, be, en - be + 1);
              tok_str[en - be + 1] = '\0';
              totex(h_str, tok_str);
              strcat(out_str, h_str);
              in_str--;  // unschn
              state = SEARCH;
            }  // if Kein-Wort-Ende erreicht
            else
              en = in_str;
            break;
          default:
            display_error(ERR_INTERNAL, "unknown search state");
            break;
        }  // switch ber den Status
        in_str++;  // nchstes Zeichen
     }  // while nicht Zeilenende
    }  // else, kein Kommentar
  }  // while nicht Zeile zu Ende
}  // transform1line

int main(int argc, char *argv[])
  // DAS Hauptprogramm.
{
    // Kommandozeile abfragen, kurz und knapp
  if (argc != 3)
    display_usage(argv[0]);

  ifstream in_file(argv[1]); // die B(PN)^2-Datei
  if (!in_file)
    display_error(ERR_IN_FILE, argv[1]);

  ofstream out_file(argv[2]); // die LaTeX-Datei
  if (!out_file)
    display_error(ERR_OUT_FILE, argv[2]);

  char in_buf[IN_BUF_SIZE];
  char out_buf[OUT_BUF_SIZE];
  // char c;

// flushleft ist problematisch, falls der Text als Block zentriert werden soll.
//  out_file << "\\begin{flushleft}" << endl;

  while (!in_file.eof()) {
    in_file.getline(in_buf, IN_BUF_SIZE);
    if (strlen(in_buf) > 0) {
      transform1line(out_buf, in_buf);
      out_file << out_buf;
      if (!in_file.eof())
        out_file << "\\mbox{}\\\\" << endl;
      else
        out_file << endl;
    }  // if keine Leerzeile
    else {
      if (!in_file.eof())
        out_file << "\\mbox{}\\\\" << endl;
      else
        out_file << endl;
    }  // else, Leerzeile
  }  // while nicht Dateiende

//  out_file << "\\end{flushleft}" << endl;

  return(0);
}  // main

  //### end of file bpn2tex.cc ###

