/*********************************************************/
/* > mnet.c <                                            */
/* Funktionen zur Manipulation von 'labelled petri nets' */
/* Burkhard Graves, 08.02.'94                            */
/*********************************************************/

#include <stdio.h>

#include "mnet.h"

/*###########################################################################*/
/*###                 BESCHLEUNIGTE EINFUEGE-OPERATIONEN                  ###*/
/*###########################################################################*/
/* AENDERUNG : 21.02.1994 Stefan Roemer                                      */
/*###########################################################################*/

/*****************************************************************************/
/* Fehler-Makros:                                                            */

#define WARNING_P_NUM(FUNCT, P_NUMNAME) \
  fprintf(stderr, \
  "WARNING: HL.%s(): a place numbered %u does not exist\n", FUNCT, P_NUMNAME)

#define WARNING_T_NUM(FUNCT, T_NUMNAME) \
  fprintf(stderr, \
  "WARNING: HL.%s(): a transition numbered %u does not exist\n",FUNCT,T_NUMNAME)

/*****************************************************************************/
/************************** M_empty_net() ************************************/
/*****************************************************************************/

M_Net M_empty_net(void)
{
	register M_Net net = M_allocHead();

	net->Blocks = net->LastBlock = NULL;
	net->Places = net->LastPlace = NULL;
	net->Transitions = net->LastTransition = NULL;
	net->PhTransitions = net->LastPhTransition = NULL;
	net->Texts = net->LastText = NULL;
	net->NumBlocks = net->NumPlaces = net->NumTransitions 
		       = net->NumPhTransitions = net->NumTexts = 0;
	net->type =  NULL;
	net->x = net->y = 0;

	return (net);

} /* M_empty_net() */


M_Text M_insert_text(M_Net net)
{
	M_Text tnew = M_allocText();

	if (net->LastText) net->LastText->nexttext = tnew;
	net->LastText = tnew;
	net->NumTexts++;

	tnew->nexttext = NULL;

	if (!net->Texts) net->Texts = tnew;

	return tnew;

}
	
/*****************************************************************************/
/************************ M_fast_insert_place ********************************/
/*****************************************************************************/

M_Place M_fast_insert_place(M_Net net,
			    Cardinal32 P_numname,
			    M_Place *pl_last)
{
	register M_Place pl,plnew;

	if (!(pl = *pl_last) || (pl->numname > P_numname))
		pl = net->Places;

	while (pl && (pl->numname < P_numname))
		pl = pl->nextplace;

	if (pl && (P_numname == pl->numname))
	{
		fprintf(stderr, "ERROR: HL.M_fast_insert_place(): "
				"place '%u' already exists.\n", P_numname);
		exit (1);
	} /* if */

	plnew = M_allocPlace();
	if (pl)
	{
		if ((plnew->prevplace = pl->prevplace))
			plnew->prevplace->nextplace = plnew;
		else
			net->Places = plnew;
		(plnew->nextplace = pl) -> prevplace = plnew;
	} /* if */
	else
	{
		if ((plnew->prevplace = net->LastPlace))
			net->LastPlace->nextplace = plnew;
		else
			net->Places = plnew;
		(net->LastPlace = plnew) -> nextplace = NULL;
	} /* else */

	plnew->numname = P_numname;
	plnew->preset = NULL;
	plnew->postset = NULL;
	plnew->phpreset = NULL;
	plnew->phpostset = NULL;
	plnew->other = NULL;
	plnew->charname = NULL;
	plnew->blocklist = NULL;
	net->NumPlaces++;

	return (*pl_last = plnew);

} /* M_fast_insert_place() */


/*****************************************************************************/
/*********************** M_fast_insert_transition ****************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

M_Transition M_fast_insert_transition(  M_Net        net,
					Cardinal32   T_numname,
					M_Transition *tr_last  )
{
	register M_Transition tr,trnew;

	if (!(tr = *tr_last) || (tr->numname > T_numname))
		tr = net->Transitions;

	while (tr && (tr->numname < T_numname))
		tr = tr->nexttrans;

	if (tr && (T_numname == tr->numname))
	{
		fprintf(stderr, "ERROR: HL.M_fast_insert_transition(): "
				"transition '%u' already exists.\n", T_numname);
		exit (1);
	} /* if */

	trnew = M_allocTrans();
	if (tr)
	{
		if ((trnew->prevtrans = tr->prevtrans))
			trnew->prevtrans->nexttrans = trnew;
		else
			net->Transitions = trnew;
		(trnew->nexttrans = tr) -> prevtrans = trnew;
	} /* if */
	else
	{
		if ((trnew->prevtrans = net->LastTransition))
			net->LastTransition->nexttrans = trnew;
		else
			net->Transitions = trnew;
		(net->LastTransition = trnew) -> nexttrans = NULL;
	} /* else */

	trnew->preset = NULL;
	trnew->postset = NULL;
	trnew->numname = T_numname;
	trnew->other = NULL;
	trnew->charname = NULL;
	trnew->corrtran = NULL;
	trnew->blocklist = NULL;
	net->NumTransitions++;

	return (*tr_last = trnew);

} /* M_fast_insert_transition() */


/*****************************************************************************/
/************************* M_fast_insert_TP_edge *****************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

M_TSEdge M_fast_insert_TP_edge( M_Net        net,
				Cardinal32   P_numname,
				Cardinal32   T_numname,
				M_Place      *pl_last,
				M_Transition *tr_last,
				M_TSEdge     *ts_last)
{
	register M_Place      pl;
	register M_Transition tr;
	register M_TSEdge     ts,tsnew;

	if ((ts = *ts_last) && (ts->place->numname == P_numname))
		pl = ts->place;
	else if (!((pl = *pl_last) && (pl->numname == P_numname)))
	{
		for (pl = net->Places ; pl && (pl->numname < P_numname);
					pl = pl->nextplace);
		if (!(pl && (pl->numname == P_numname)))
		{
			WARNING_P_NUM("fast_insert_TP_edge", P_numname);
			return (NULL);
		} /* if */
	} /* else if */

	if (ts && ( ts->trans->numname == T_numname))
		tr = ts->trans;
	else if (! ((tr = *tr_last) && (tr->numname == T_numname)))
	{
		for (tr = net->Transitions; tr && (tr->numname < T_numname);
					    tr = tr->nexttrans);
		if (! (tr && (tr->numname == T_numname)))
		{
			WARNING_T_NUM("fast_insert_TP_edge", T_numname);
			return (NULL);
		} /* if */
	} /* else if */

	if ((ts = pl->preset))
	{
		if (T_numname < ts->trans->numname)
			ts = NULL;
		else if (T_numname == ts->trans->numname)
		{
			/* Inkrementiere Kantengewicht */
			/* In diesen drei Zeilen stand vorher tsnew statt ts.
			   Das schien Probleme zu machen. */
			*pl_last = ts->place = pl;
			*tr_last = ts->trans = tr;
			return (*ts_last = ts);
		} /* if */
		else
		{
			while (ts->nexttrans
				&& (ts->nexttrans->trans->numname < T_numname))
			    ts = ts->nexttrans;

			if ((tsnew = ts->nexttrans)
				&& (tsnew->trans->numname == T_numname))
			{
				/* Inkrementiere Kantengewicht */
				*pl_last = tsnew->place = pl;
				*tr_last = tsnew->trans = tr;
				return (*ts_last = tsnew);
			} /* if */
		} /* else */
	} /* if */

	tsnew = M_allocTSEdge();
	tsnew->other = NULL;

	if (ts)
	{
		if ((tsnew->nexttrans = ts->nexttrans))
			tsnew->nexttrans->prevtrans = tsnew;
		(tsnew->prevtrans = ts) -> nexttrans = tsnew;
	} /* if */
	else
	{
		if ((tsnew->nexttrans = pl->preset))
			tsnew->nexttrans->prevtrans = tsnew;
		(pl->preset = tsnew) -> prevtrans = NULL;
	} /* else */

	if ((ts = tr->postset))
        {
		if (P_numname < ts->place->numname)
			ts = NULL;
		else
                {
			while (ts->nextplace
				&& (ts->nextplace->place->numname < P_numname))
			    ts = ts->nextplace;
                }
        }

	if (ts)
	{
		if ((tsnew->nextplace = ts->nextplace))
			tsnew->nextplace->prevplace = tsnew;
		(tsnew->prevplace = ts) -> nextplace = tsnew;
	} /* if */
	else
	{
		if ((tsnew->nextplace = tr->postset))
			tsnew->nextplace->prevplace = tsnew;
		(tr->postset = tsnew) -> prevplace = NULL;
	} /* else */

	*pl_last = tsnew->place = pl;
	*tr_last = tsnew->trans = tr;
	return (*ts_last = tsnew);

} /* M_fast_insert_TP_edge() */

/*****************************************************************************/
/************************* M_fast_insert_PT_edge *****************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

M_STEdge M_fast_insert_PT_edge( M_Net        net,
				Cardinal32   P_numname,
				Cardinal32   T_numname,
				M_Place      *pl_last,
				M_Transition *tr_last,
				M_STEdge     *st_last)
{
	register M_Place      pl;
	register M_Transition tr;
	register M_STEdge     st,stnew;

	if ((st = *st_last) && (st->place->numname == P_numname))
		pl = st->place;
	else if (!((pl = *pl_last) && (pl->numname == P_numname)))
	{
		for (pl = net->Places;  pl && (pl->numname < P_numname);
					pl = pl->nextplace);
		if (!(pl && (pl->numname == P_numname)))
		{
			WARNING_P_NUM("fast_insert_PT_edge", P_numname);
			return (NULL);
		} /* if */
	} /* else if */

	if (st && ( st->trans->numname == T_numname))
		tr = st->trans;
	else if (!((tr = *tr_last) && (tr->numname == T_numname)))
	{
		for (tr = net->Transitions; tr && (tr->numname < T_numname);
					    tr = tr->nexttrans);
		if (!(tr && (tr->numname == T_numname)))
		{
			WARNING_T_NUM("fast_insert_PT_edge", T_numname);
			return (NULL);
		} /* if */
	} /* else if */

	if ((st = pl->postset))
	{
		if (T_numname < st->trans->numname)
			st = NULL;
		else if (T_numname == st->trans->numname)
		{
			/* Inkrementiere Kantengewicht */
			/* stnew -> st substituiert (s.o.) */
			*pl_last = st->place = pl;
			*tr_last = st->trans = tr;
			return (*st_last = st);
		} /* if */
		else
		{
			while (st->nexttrans
				&& (st->nexttrans->trans->numname < T_numname))
			    st = st->nexttrans;

			if ((stnew = st->nexttrans)
				&& (stnew->trans->numname == T_numname))
			{
				/* Inkrementiere Kantengewicht */
				*pl_last = stnew->place = pl;
				*tr_last = stnew->trans = tr;
				return (*st_last = stnew);
			} /* if */
		} /* else */
	} /* if */

	stnew = M_allocSTEdge();
	stnew->other = NULL;

	if (st)
	{
		if ((stnew->nexttrans = st->nexttrans))
			stnew->nexttrans->prevtrans = stnew;
		(stnew->prevtrans = st) -> nexttrans = stnew;
	} /* if */
	else
	{
		if ((stnew->nexttrans = pl->postset))
			stnew->nexttrans->prevtrans = stnew;
		(pl->postset = stnew) -> prevtrans = NULL;
	} /* else */

	if ((st = tr->preset))
        {
		if (P_numname < st->place->numname)
			st = NULL;
		else
                {
			while (st->nextplace
				&& (st->nextplace->place->numname < P_numname))
			    st = st->nextplace;
                }
        }

	if (st)
	{
		if ((stnew->nextplace = st->nextplace))
			stnew->nextplace->prevplace = stnew;
		(stnew->prevplace = st) -> nextplace = stnew;
	} /* if */
	else
	{
		if ((stnew->nextplace = tr->preset))
			stnew->nextplace->prevplace = stnew;
		(tr->preset = stnew) -> prevplace = NULL;
	} /* else */

	*pl_last = stnew->place = pl;
	*tr_last = stnew->trans = tr;
	return (*st_last = stnew);

} /* M_fast_insert_PT_edge() */

/*###########################################################################*/
/*######           Operation on Blocks and Phantom Objects            #######*/
/*###########################################################################*/

/* Verbiegt einige Variablen und ruft dann die oberen Funktionen auf. */

#define swap(h,x1,x2) h=x1; x1=x2; x2=h;

M_Block M_insert_block (M_Net net,
			Cardinal32 B_numname,
			M_Block *bl_last)
{
	M_Block result,pl;
	Cardinal32 numpl;

	swap(pl,net->Blocks,net->Places);
	swap(pl,net->LastBlock,net->LastPlace);
	swap(numpl,net->NumBlocks,net->NumPlaces);

	result = M_fast_insert_place(net,B_numname,bl_last);

	swap(pl,net->Blocks,net->Places);
	swap(pl,net->LastBlock,net->LastPlace);
	swap(numpl,net->NumBlocks,net->NumPlaces);

	return result;
}

M_Transition M_insert_phantom_transition(M_Net        net,
					 Cardinal32   T_numname,
					 M_Transition *tr_last)
{
	M_Transition result,tr;
	Cardinal32 numtr;

	swap(tr,net->Transitions,net->PhTransitions);
	swap(tr,net->LastTransition,net->LastPhTransition);
	swap(numtr,net->NumTransitions,net->NumPhTransitions);

	result = M_fast_insert_transition(net,T_numname,tr_last);

	swap(tr,net->Transitions,net->PhTransitions);
	swap(tr,net->LastTransition,net->LastPhTransition);
	swap(numtr,net->NumTransitions,net->NumPhTransitions);

	return result;
}

M_TSEdge M_insert_PTP_edge(M_Net        net,
			   Cardinal32   P_numname,
			   Cardinal32   T_numname,
			   M_Place      *pl_last,
			   M_Transition *tr_last,
			   M_TSEdge     *ts_last)
{
	M_Transition tr;
	Cardinal32 numtr;
	M_STEdge st;
	M_TSEdge result,ts;

	swap(tr,net->Transitions,net->PhTransitions);
	swap(tr,net->LastTransition,net->LastPhTransition);
	swap(numtr,net->NumTransitions,net->NumPhTransitions);
	swap(ts,(*pl_last)->preset,(*pl_last)->phpreset);
	swap(st,(*pl_last)->postset,(*pl_last)->phpostset);

	result = M_fast_insert_TP_edge(net,P_numname,T_numname,
						pl_last,tr_last,ts_last);

	swap(tr,net->Transitions,net->PhTransitions);
	swap(tr,net->LastTransition,net->LastPhTransition);
	swap(numtr,net->NumTransitions,net->NumPhTransitions);
	swap(ts,(*pl_last)->preset,(*pl_last)->phpreset);
	swap(st,(*pl_last)->postset,(*pl_last)->phpostset);

	return result;
}

M_STEdge M_insert_PPT_edge(M_Net        net,
			   Cardinal32   P_numname,
			   Cardinal32   T_numname,
			   M_Place      *pl_last,
			   M_Transition *tr_last,
			   M_STEdge     *st_last)
{
	M_Transition tr;
	Cardinal32 numtr;
	M_STEdge result,st;
	M_TSEdge ts;

	swap(tr,net->Transitions,net->PhTransitions);
	swap(tr,net->LastTransition,net->LastPhTransition);
	swap(numtr,net->NumTransitions,net->NumPhTransitions);
	swap(ts,(*pl_last)->preset,(*pl_last)->phpreset);
	swap(st,(*pl_last)->postset,(*pl_last)->phpostset);

	result = M_fast_insert_PT_edge(net,P_numname,T_numname,
						pl_last,tr_last,st_last);

	swap(tr,net->Transitions,net->PhTransitions);
	swap(tr,net->LastTransition,net->LastPhTransition);
	swap(numtr,net->NumTransitions,net->NumPhTransitions);
	swap(ts,(*pl_last)->preset,(*pl_last)->phpreset);
	swap(st,(*pl_last)->postset,(*pl_last)->phpostset);

	return result;
}

/**********************/
/* END OF FILE mnet.c */
/**********************/
