/*****************************/
/* mnet.h                    */
/* Burkhard Graves, 26.6.'93 */
/*****************************/

#ifndef __LPNET_H__
#define __LPNET_H__

#include "mtupel.h"
#include "readPEP.h"

/**********/
/* Stelle */
/**********/

typedef struct M_PlElem_
{
    struct M_PlElem_ *prevplace,  /* Zur (doppelten) Verkettung in der	    */
		     *nextplace;  /*   Stellenliste.			    */
    struct M_Edge_   *preset;     /* Zeiger auf erste Kante des Vorbereichs */
    struct M_Edge_   *postset;    /*   bzw. des Nachbereichs.		    */
    struct M_Edge_   *phpreset;   /* dito fuer angrenzende		    */
    struct M_Edge_   *phpostset;  /*      Phantomtransitionen	            */
    Cardinal32	      numname;    /* Numerischer Name fuer div. Zwecke.	    */
    char	     *charname;	  /* Textueller Name: z.B. P19		    */
    t_coords	     *co;
    t_coords	     *absname,*absbed;
    char	     *blocklist;
    MPTupel	      blocktupel;
    char	     *other;
} M_PlElem,*M_Place,*M_Block;

/**************/
/* Transition */
/**************/

typedef struct M_TrElem_
{
    struct M_TrElem_ *prevtrans,  /* Zur (doppelten) Verkettung in der	    */
		     *nexttrans;  /*   Transitionenliste.		    */
    struct M_Edge_   *preset;     /* Zeiger auf erste Kante des Vorbereichs */
    struct M_Edge_   *postset;    /*   bzw. des Nachbereichs.		    */
    Cardinal32	      numname;    /* Numerischer Name fuer div. Zwecke.	    */
    char	     *charname;	  /* Textueller Name: z.B. T42		    */
    char	     *corrtran;	  /* zugehoerige Phantom/Synch-Transitionen */
    t_coords	     *co;
    t_coords	     *absname,*absbed,*absval;
    char	     *other;
    char	     *blocklist;
    MPTupel	      partners;
    MPTupel	      blocktupel;
} M_TrElem, *M_Transition;

/**********/
/* Kanten */
/**********/

typedef struct M_Edge_
{
    struct M_Edge_ *prevplace, /* Zur (doppelten, vertikalen) Verkettung   */
		   *nextplace, /*   in Bezug auf die Stellenliste.	   */
		   *prevtrans, /* Zur (doppelten, horizontalen) Verkettung */
		   *nexttrans; /*   in Bezug auf die Transitionenliste.    */
    M_Place	    place;     /* Zeiger auf den Ursprung		   */
    M_Transition    trans;     /*   bzw. auf das Ziel der Kante.	   */
    t_coords	   *midpoint,*absweight,*absins;
    char	   *other;
} M_STElem,M_TSElem,*M_Edge,*M_STEdge,*M_TSEdge;

/*********/
/* Texte */
/*********/

typedef struct M_Text_
{
	struct M_Text_  *nexttext;
	t_coords	*co;
	char		*content;
} M_TextElem, *M_Text;

/*********************/
/* Kopf eines Netzes */
/*********************/

typedef struct
{
    M_Block	 Blocks,         /* Zeiger auf erste bzw. auf letzte Stelle */
		 LastBlock;      /*   in der Blockliste.		    */
    M_Place	 Places,         /* Zeiger auf erste bzw. auf letzte Stelle */
		 LastPlace;      /*   in der Stellenliste.		    */
    M_Transition Transitions,    /* Zeiger auf erste bzw. auf letzte Trans. */
		 LastTransition; /*   in der Transitionenliste.		    */
    M_Transition PhTransitions,  /* Zeiger auf erste bzw. auf letzte Trans. */
		 LastPhTransition; /*   in der Phantomtransitionenliste.    */
    M_Text	 Texts, LastText; /* Auf ersten und letzten Text	    */
    Cardinal32	 NumBlocks,
		 NumPlaces,      /* Anzahl der Stellen,			    */
		 NumTransitions, /*        der Transitionen,		    */
		 NumPhTransitions, /*      der Phantomtransitionen.	    */
		 NumTexts; 	 /*      der Phantomtransitionen.	    */
    char	 *type;		 /* HLNet oder PetriBox			    */
    int	 	 x,y;		 /* Maximale x,y-Koordinaten		    */
} M_NetHead,*M_Net;

/**********************************************************************/
/* Funktionen zur Allokation von Speicherplatz fuer einzelne Elemente */
/* eines Netzes; realisiert als Makros.				      */
/**********************************************************************/

/* Funktionalitaet: M_Net   M_allocHead(void)   */
/*                  M_Place M_allocPlace(void)  */
/*             M_Transition M_allocTrans(void)  */
/*                 M_STEdge M_allocSTEdge(void) */
/*                 M_TSEdge M_allocTSEdge(void) */

#define M_allocHead()   ( (M_Net) MYmalloc( sizeof( M_NetHead ) ) )
#define M_allocPlace()  ( (M_Place) MYmalloc( sizeof( M_PlElem ) ) )
#define M_allocTrans()  ( (M_Transition) MYmalloc( sizeof( M_TrElem ) ) )
#define M_allocText()   ( (M_Text) MYmalloc( sizeof( M_TextElem ) ) )
#define M_allocSTEdge() ( (M_STEdge) MYmalloc( sizeof( M_STElem ) ) )
#define M_allocTSEdge() ( (M_TSEdge) MYmalloc( sizeof( M_TSElem ) ) )

#endif

/*****************************************************************************/

extern M_Net M_empty_net(void);
extern M_Block M_insert_block(M_Net, Cardinal32, M_Block*);
extern M_Place M_fast_insert_place(M_Net, Cardinal32, M_Place*);
extern M_Transition M_fast_insert_transition(M_Net, Cardinal32, M_Transition*);
extern M_TSEdge M_fast_insert_TP_edge(M_Net, Cardinal32, Cardinal32,
				      M_Place*, M_Transition*, M_TSEdge* );
extern M_STEdge M_fast_insert_PT_edge(M_Net, Cardinal32, Cardinal32,
				      M_Place*, M_Transition*, M_STEdge*);
extern M_Transition M_insert_phantom_transition(M_Net,Cardinal32,M_Transition*);
extern M_TSEdge M_insert_PTP_edge(M_Net, Cardinal32, Cardinal32,
				  M_Place*, M_Transition*, M_TSEdge*);
extern M_STEdge M_insert_PPT_edge(M_Net, Cardinal32, Cardinal32,
				  M_Place*, M_Transition*, M_STEdge*);
extern M_Text M_insert_text(M_Net);

/*****************************************************************************/

/**********************/
/* END OF FILE mnet.h */
/**********************/
