/*****************************************************************************/
/* readHL.c                                                                  */
/*****************************************************************************/
/*                                  Stefan Roemer,  06.02.1995 - 06.02.1995  */
/*                                  Stefan Schwoon, September 1997	     */
/*****************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "mnet.h"
#include "readPEP.h"
#include "readlib.h"

/*****************************************************************************/

#define MBSIZE 4096
#define NAMES_START 2000
#define NAMES_OFFSET 1000

/*****************************************************************************/
/***************************** read_HLnet ************************************/
/*****************************************************************************/

M_Net		net;
M_Place		*PlArray;
M_Transition	*TrArray,
		*PhTrArray;

Cardinal32	AnzPlNamen, MaxPlNamen,
		AnzTrNamen, MaxTrNamen,
		AnzPhTrNamen, MaxPhTrNamen;

char buffer[13];
int  swapped;
int  placecount, transcount;
bool autonumbering;

t_coords *co, *absname, *absbed, *absval;
char	 *name, *corrtran, *rest, *blocklist;
int	 ident;

int insert_place();
int insert_trans();
int insert_arc();
int insert_text();

M_Net read_HLnet(char *PEPfilename)
{
	t_dest place_dest[] =
		{ { '\'',&name },	/* Bezeichner		*/
		  { '"', &name },	/*     "		*/
		  { '@', &co },		/* Koordinaten		*/
		  { '0', &ident },	/* numerischer Name	*/
		  { 'u', &blocklist },	/* Blockliste		*/
		  { 'N', &absname },	/* abs.Pos.Name		*/
		  { 'A', &absbed },	/* abs.Pos.Bedeutung	*/
		  {  0 ,  0 } };
	  
	t_dest trans_dest[] =
		{ { '\'',&name },	/* Bezeichner		*/
		  { '"', &name },	/*     "		*/
		  { '@', &co },		/* Koordinaten		*/
		  { '0', &ident },	/* numerischer Name	*/
		  { 'P', &corrtran },	/* Liste der Ph.Trans.	*/
		  { 'u', &blocklist },	/* Blockliste		*/
		  { 'N', &absname },	/* abs.Pos.Name		*/
		  { 'A', &absbed },	/* abs.Pos.Bedeutung	*/
		  { 'H', &absval },	/* abs.Pos.Val.Term	*/
		  {  0 ,  0 } };
	  
	t_dest arc_dest[] =
		{ { '@', &co },		/* Quelle/Ziel		*/
		  { 'J', &absname },	/* Stuetzpunkt		*/
		  { 'N', &absbed },	/* abs.Pos.Gewicht	*/
		  { 'Q', &absval },	/* abs.Pos.Beschriftung	*/
		  {  0 ,  0 } };

	t_dest text_dest[] =
		{ { '\'',&name },	/* Bezeichner		*/
		  { '"', &name },	/*     "		*/
		  { 'N', &co },		/* Koordinaten		*/
		  {  0 ,  0 } };

	t_blockdest netdest[] =
		{ { "BL",  insert_place, &rest, place_dest },
		  { "PL",  insert_place, &rest, place_dest },
		  { "TR",  insert_trans, &rest, trans_dest },
		  { "PTR", insert_trans, &rest, trans_dest },
		  { "TP",  insert_arc,   &rest, arc_dest },
		  { "PT",  insert_arc,   &rest, arc_dest },
		  { "PTP", insert_arc,   &rest, arc_dest },
		  { "PPT", insert_arc,   &rest, arc_dest },
		  { "TX",  insert_text,  &rest, text_dest },
		  { NULL, NULL, NULL, NULL } };

	int		count;

	PlArray = (M_Place*) MYmalloc((count = MaxPlNamen = MaxTrNamen
		= MaxPhTrNamen = NAMES_START) * sizeof(M_Place));
	TrArray = (M_Transition*) MYmalloc(count * sizeof(M_Transition));
	PhTrArray = (M_Transition*) MYmalloc(count * sizeof(M_Transition));
	AnzPlNamen = AnzTrNamen = AnzPhTrNamen = 0;
	while (--count)
	{
		PlArray[count] = (M_Place) NULL;
		TrArray[count] = (M_Transition) NULL;
		PhTrArray[count] = (M_Transition) NULL;
	} /* while */

	/* Initialize net */
	net = M_empty_net();

	/* Initialize blocks */
	swapped = 0;
	net->x = net->y = 0;
	placecount = transcount = 0;

	/* read the net */
	read_PEP_net(PEPfilename, netdest);

	net->type = filetype;

	free(PlArray);
	free(TrArray);
	free(PhTrArray);

	return net;

} /* read_HLnet() */

#define swap(h,x1,x2) h=x1; x1=x2; x2=h;

void checksize(t_coords *co)
{
	if (!co) return;
	if (co->x > net->x) net->x = co->x;
	if (co->y > net->y) net->y = co->y;
}

int insert_place()
{
	int count;
	static M_Place pl_last = NULL;

	if (*blocktype == 'P')
	{
		*blocktype = '\0';
		pl_last = NULL;
		placecount = 0;
		for (count = 0; count <= AnzPlNamen; PlArray[count++] = NULL);
		AnzPlNamen = 0;
	}

	if (!placecount++) autonumbering = True;
	if (ident && ident != placecount) autonumbering = False;
	if (!ident && autonumbering) ident = placecount;
	if (!ident) Error((errbuf,"missing place identifier"));
		
	if (ident > AnzPlNamen)
		AnzPlNamen = ident;
	else if (PlArray[ident])
		Error((errbuf, "%s identifier %d used twice", 
				*blocktype == 'B'? "block" : "place", ident));

	while (AnzPlNamen >= MaxPlNamen)
	{
		PlArray = (M_Place*) MYrealloc((char*) PlArray,
				(MaxPlNamen += NAMES_OFFSET) * sizeof(M_Place));
		for (count = MaxPlNamen - NAMES_OFFSET; count < MaxPlNamen;)
			PlArray[count++] = (M_Place) NULL;

	}

	PlArray[ident] = (*blocktype == 'B'? M_insert_block
					   : M_fast_insert_place)
				(net,ident,&pl_last);
	if (!name) sprintf(buffer, "%c%u", *blocktype == 'B'? 'B' : 'P', ident);
	pl_last->charname = name? name : MYstrdup(buffer);
	pl_last->blocklist = blocklist;
	pl_last->co = co;		checksize(co);
	pl_last->absname = absname;	checksize(absname);
	pl_last->absbed = absbed;	checksize(absbed);
	pl_last->other = rest;
	return 0;
}

int insert_trans()
{
	int count;
	static M_Transition tr_last = NULL;

	if (*blocktype == 'P')
	{
		M_Transition *trtmp;
		
		*blocktype = '\0';
		tr_last = NULL;
		transcount = 0;
		swapped++;
		swap(trtmp,PhTrArray,TrArray);
		swap(count,AnzPhTrNamen,AnzTrNamen);
		swap(count,MaxPhTrNamen,MaxTrNamen);
	}

	if (!transcount++) autonumbering = True;
	if (ident && ident != transcount) autonumbering = False;
	if (!ident && autonumbering) ident = transcount;
	if (!ident) Error((errbuf, "missing transition identifier"));

	if (ident > AnzPhTrNamen)
		AnzPhTrNamen = ident;
	else if (PhTrArray[ident])
		Error((errbuf, "%stransition identifier %d used twice", 
			    *blocktype == 'T'? "" : "phantom ", ident));

	while (AnzPhTrNamen >= MaxPhTrNamen)
	{
		PhTrArray = (M_Transition *) MYrealloc((char*) PhTrArray,
			  (MaxPhTrNamen += NAMES_OFFSET) * sizeof(M_Transition));
		for (count = MaxPhTrNamen - NAMES_OFFSET; count < MaxPhTrNamen;)
			PhTrArray[count++] = (M_Transition) NULL;
	}

	PhTrArray[ident] = (*blocktype == 'T'? M_fast_insert_transition
					     : M_insert_phantom_transition)
				(net, ident, &tr_last);

	if (!name) sprintf(buffer,"%s%u", *blocktype == 'T'? "T" : "PT", ident);
	tr_last->charname = name? name : MYstrdup(buffer);
	tr_last->corrtran = corrtran;
	tr_last->blocklist = blocklist;
	tr_last->co = co;		checksize(co);
	tr_last->absname = absname;	checksize(absname);
	tr_last->absbed = absbed;	checksize(absbed);
	tr_last->absval = absval;	checksize(absval);
	tr_last->other = rest;
	return 0;
}

int insert_arc()
{
	static M_Edge arc_last = NULL;
	static int tp = 0, phantom = 0;
	int pl, tr, anz;
	M_Transition *arr;

	if (!swapped++)		/* oh no! ... pop! */
	{
		swap(arr,PhTrArray,TrArray);
		swap(tr,AnzPhTrNamen,AnzTrNamen);
		swap(tr,MaxPhTrNamen,MaxTrNamen);
	}

	if (isalpha(*blocktype))
	{
		arc_last = NULL;
		tp = strcmp(blocktype,"PT") && strcmp(blocktype,"PPT");
		phantom = strcmp(blocktype,"TP") && strcmp(blocktype,"PT");
		*blocktype = '\0';
	}
		
	pl = tp? co->y : co->x;
	tr = tp? co->x : co->y;
	arr = phantom? PhTrArray : TrArray;
	anz = phantom? AnzPhTrNamen : AnzTrNamen;

	if (!tr || (tr > anz) || !arr[tr])
		Error((errbuf, "arc: incorrect %stransition identifier",
			phantom? "phantom " : ""));
	if (!pl || (pl > AnzPlNamen) || !PlArray[pl] )
		Error((errbuf, "arc: incorrect place identifier"));

	(tp? (phantom? M_insert_PTP_edge : M_fast_insert_TP_edge)
	   : (phantom? M_insert_PPT_edge : M_fast_insert_PT_edge))
		(net, PlArray[pl]->numname, arr[tr]->numname,
		 &PlArray[pl], &arr[tr], &arc_last);

	arc_last->midpoint = absname;	checksize(absname);
	arc_last->absweight = absbed;	checksize(absbed);
	arc_last->absins = absval;	checksize(absval);

	arc_last->other = rest;
	return 0;
}

int insert_text()
{
	M_Text tx_last = M_insert_text(net);

	tx_last->co = co;
	tx_last->content = name;

	if (co->x > net->x) net->x = co->x;
	if (co->y > net->y) net->y = co->y;
	return 0;

}

/*****************************************************************************/

#undef Error
#define Error(msg) { PEPerror(msg); exit(10); }

/*****************************************************************************/
/***************************** write_HLnet ***********************************/
/*****************************************************************************/

void write_blocklist(FILE *file, MPTupel mpt)
{
	int i;
	f("u\"(");
	for (i = 0; i < mpt->count; )
	    fxy("%d%s", MT_PlaceP(mpt,i)->numname, ++i < mpt->count? "," : "");
	f(")\"");
}

void write_corrlist(FILE *file, MPTupel mpt)
{
	int i;
	f("P\"(");
	for (i = 0; i < mpt->count; )
	    fxy("%d%s", MT_PlaceT(mpt,i)->numname, ++i < mpt->count? "," : "");
	f(")\"");
}

void write_coords(FILE *file, char c, t_coords *co)
{
	if (!co) return;
	fx("%c",c);
	fxy("%d@%d",co->x,co->y);
}

/*****************************************************************************/

void write_HLnet(char *PEPfilename, char *type,
		 int numbl, M_Place *BlArray,
		 int numpl, M_Place *PlArray,
		 int numtr, M_Transition *TrArray,
		 int numph, M_Transition *PhArray,
		 int numtx, M_Text *TxArray)
{
	register M_Block	bl;
	register M_Place	pl;
	register M_Transition	tr;
	register M_STEdge	st;
	register M_TSEdge	ts;
	register M_Text		tx;
	FILE			*file;
	int			i;

	if (!(file = fopen(PEPfilename, "w")))
		Error("Could not open file for writing");

	f("PEP\n");
	fx("%s\n", type);
	fx("FORMAT_%s\n", numbl || numph? "N2" : "N");

	f("DPL s7n10@-9t2\n");
	f("DTR s7n10@-9t2\n");
	f("DPT w1t1\n");

	if (numbl) f("BL\n");
	for (i = 0; i < numbl; i++)
	{
		bl = BlArray[i];
		fxy("%u\"%s\"", bl->numname, bl->charname);
		fxy("%d@%d", bl->co->x, bl->co->y);
		if (bl->blocklist) write_blocklist(file,bl->blocktupel);
		write_coords(file,'N',bl->absname);
		write_coords(file,'A',bl->absbed);
		fx("%s\n", bl->other);
	}

	/* Liste der Stellen ausgeben */
	f("PL\n");
	for (i = 0; i < numpl; i++)
	{
		pl = PlArray[i];
		fxy("%u\"%s\"", pl->numname, pl->charname);
		fxy("%d@%d", pl->co->x, pl->co->y);
		if (pl->blocklist) write_blocklist(file,pl->blocktupel);
		write_coords(file,'N',pl->absname);
		write_coords(file,'A',pl->absbed);
		fx("%s\n", pl->other);
	}

	/* Liste der Transitionen ausgeben */
	f("TR\n");
	for (i = 0; i < numtr; i++)
	{
		tr = TrArray[i];
		fxy("%u\"%s\"", tr->numname, tr->charname);
		fxy("%d@%d", tr->co->x, tr->co->y);
		if (tr->corrtran) write_corrlist(file,tr->partners);
		if (tr->blocklist) write_blocklist(file,tr->blocktupel);
		write_coords(file,'N',tr->absname);
		write_coords(file,'A',tr->absbed);
		write_coords(file,'H',tr->absval);
		fx("%s\n", tr->other);
	}

	if (numph) f("PTR\n");
	for (i = 0; i < numph; i++)
	{
		tr = PhArray[i];
		fxy("%u\"%s\"", tr->numname, tr->charname);
		fxy("%d@%d", tr->co->x, tr->co->y);
		write_corrlist(file,tr->partners);
		if (tr->blocklist) write_blocklist(file,tr->blocktupel);
		write_coords(file,'N',tr->absname);
		write_coords(file,'A',tr->absbed);
		write_coords(file,'H',tr->absval);
		fx("%s\n", tr->other);
	}

	/* Liste der TS-Kanten ausgeben */
	f("TP\n");
	for (i = 0; i < numtr; i++)
	{
		tr = TrArray[i];
		for (ts = tr->postset; ts; ts = ts->nextplace)
		{
			fxy("%u<%u", tr->numname, ts->place->numname);
			write_coords(file,'J',ts->midpoint);
			write_coords(file,'N',ts->absweight);
			write_coords(file,'Q',ts->absins);
			fx("%s\n",ts->other);
		}
	}

	f("PT\n");
	for (i = 0; i < numtr; i++)
	{
		tr = TrArray[i];
		for (st = tr->preset; st; st = st->nextplace)
		{
			fxy("%u>%u", st->place->numname, tr->numname);
			write_coords(file,'J',st->midpoint);
			write_coords(file,'N',st->absweight);
			write_coords(file,'Q',st->absins);
			fx("%s\n",st->other);
		}
	}

	/* Liste der Phantom-Kanten ausgeben */
	if (numph) f("PTP\n");
	for (i = 0; i < numph; i++)
	{
		tr = PhArray[i];
		for (ts = tr->postset; ts; ts = ts->nextplace)
		{
			fxy("%u<%u", tr->numname, ts->place->numname);
			write_coords(file,'J',ts->midpoint);
			write_coords(file,'N',ts->absweight);
			write_coords(file,'Q',ts->absins);
			fx("%s\n",ts->other);
		}
	}

	if (numph) f("PPT\n");
	for (i = 0; i < numph; i++)
	{
		tr = PhArray[i];
		for (st = tr->preset; st; st = st->nextplace)
		{
			fxy("%u>%u", st->place->numname, tr->numname);
			write_coords(file,'J',st->midpoint);
			write_coords(file,'N',st->absweight);
			write_coords(file,'Q',st->absins);
			fx("%s\n",st->other);
		}
	}

	if (numtx) f("TX\n");
	for (i = 0; i < numtx; i++)
	{
		tx = TxArray[i];
		fx("\"%s\"",tx->content);
		fxy("N%u@%u\n",tx->co->x,tx->co->y);
	}

	fclose(file);

} /* write_HLnet() */

