/*-------------------------------------------------------------------------*/
/*--  DL_CHK.C                                                           --*/
/*--  Johannes Jaeger - Sommer 1994                                      --*/
/*-------------------------------------------------------------------------*/
/* Verbesserte & korrigierte Version: Stefan Roemer,   06.11.95 - 25.03.96 */
/*-------------------------------------------------------------------------*/
/* BG kleine Aenderungen an der Ausgabe 22.3.97                            */
/* BG kleine Aenderungen an der Ausgabe  5.6.97                            */

#include <stdlib.h>
#include <string.h>
#include "dl_chk.h"
#include "rw_prefix.h"
#include "readlib.h"

#define MAX_SPOILERS      0xFFFFFFFF /* fiktiver Max-Wert fuer Spoileranzahl */
#define DEADLOCK_FOUND    True
#define NO_DEADLOCK_FOUND False

/*-------------------------------------------------------------------------*/
/*   ein paar #defines fuer die Ergebnisausgabe                            */

#define NEW_PATHLINE    ",\n  "
#define TRANS_DELIMITER ","
#define PATH_END        "\n\n"
#define PATH_LINELEN    65
#define E_FOPEN         "WARNING -- Could not open %s.\n"


/*-------------------------------------------------------------------------*/
/* Markierung im Feld label einer Transition fuer nicht geloescht */

#define NOT_DELETED (Cardinal32) 0

/* globaler Zeiger auf das erste Element einer Liste von Cut-Off-Punkten */
static Transition * COPArray = NULL;

static bool DoCOPCheck;

static LMPNet     GL_net = NULL;
static Cardinal32 GL_numPFlag = 0;
static char       * GL_numTFlag = NULL;

#define IncPMark() \
    if ( ! ++GL_numPFlag ) \
      { \
        register Place _pl; \
        for ( _pl = GL_net->Places ; _pl ; _pl = _pl->nextplace );\
        GL_numPFlag++; \
      } /* if */

#define PMark( _PL_ ) ( (_PL_)->component = GL_numPFlag )

#define IsPMarked( _PL_ ) ( (_PL_)->component == GL_numPFlag )

#define IncTMark() \
    if ( ! ++GL_numTFlag ) \
      { \
        register Transition _tr; \
        for ( _tr = GL_net->Transitions ; _tr ; \
                               _tr->inscript = NULL, _tr = _tr->nexttrans ) skip;\
        GL_numTFlag++; \
      } /* if */

#define TMark( _TR_ ) ( ( _TR_ )->inscript = GL_numTFlag )

#define IsNotTMarked( _TR_ ) ( (_TR_)->inscript != GL_numTFlag )


/*-------------------------------------------------------------------------*/
/*-- prueft, ob die uebergebene Transition trans ein Cut-Off-Punkt ist,  --*/
/*-- d.h, ob trans in der Cut-Off-Punkt-Liste ist, auf die COPListHead   --*/
/*-- zeigt                                                               --*/
/*-- Parameter :                                                         --*/
/*--   Transition trans : Zeiger auf eine Transition, von der festge-    --*/
/*--                      stellt werden soll, ob sie Cut-Off-Punkt ist   --*/
/*--                                                                     --*/
/*-- Rueckgabe : True  - trans ist in der Cut-Off-Punkt-Liste            --*/
/*--             False - trans ist in nicht der Cut-Off-Punkt-Liste      --*/

#if defined(__STDC__) || defined(__cplusplus)

static bool IsCOP( Transition trans )

#else

static bool IsCOP( trans )

  Transition trans;

#endif

  {

    /* DoCOPCheck wird in DelInConflictWith() mit False (es   */
    /* interessiert nicht ob trans ein Cut-Off-Punkt ist) und */
    /* in GetAllSpoilers() mit True gesetzt                   */

    return ( DoCOPCheck ? trans->prevtrans ? True : False : False );

  }

/*-------------------------------------------------------------------------*/
/*-- ueberprueft, ob es einen Cut-Off-Punkt ohne Spoiler gibt            --*/
/*--                                                                     --*/
/*-- Rueckgabe : True  - es gibt einen ohne Spoiler                      --*/
/*--             False - alle Cut-Off-Punkte haben Spoiler               --*/

#if defined(__STDC__) || defined(__cplusplus)

static bool COPwithoutSpoiler( void )

#else

static bool COPwithoutSpoiler()

#endif

  {

    register Transition * ptr = COPArray;

    while ( *ptr )
      {
        /* hat aktueller Cut-Off-Punkt Spoiler? */
        if ( ! ( (COPAdd) ( *(ptr++) )->prevtrans )->NumSpoilers )
          {
            /* nein : fertig */
            return ( True );
          } /* if */
      } /* while */

    /* wenn wir hier ankommen, haben alle Cut-Off-Punkte Spoiler */
    return ( False );

  } /* COPwithoutSpoiler() */

/*-------------------------------------------------------------------------*/
/*-- erzeugt eine Liste von COPListElems mit Zeigern auf die Cut-Off-    --*/
/*-- Punkte, deren Namen per Parameter CutOffPoints uebergeben werden.   --*/
/*-- Parameter :                                                         --*/
/*--   LMPNet net : das Netz mit der Entfaltung                          --*/
/*--   LIST_POINTER CutOffPoints : eine Liste mit den Namen der Cut-Off- --*/
/*--                               Punkte                                --*/
/*--                                                                     --*/
/*-- Rueckgabe : Zeiger auf das erste Listenelement                      --*/

#if defined(__STDC__) || defined(__cplusplus)

static Transition * CreateCOPArray( LMPNet net, LIST_POINTER CutOffPoints )

#else

static Transition * CreateCOPArray( net, CutOffPoints )

  LMPNet       net;
  LIST_POINTER CutOffPoints;

#endif

  {

    register Transition   * array,
                          * ptr,
                          tr;
    register LIST_POINTER lst;

    ptr = array = (Transition *)
                  MYmalloc( ( GL_numCutoffs + 1 ) * sizeof( Transition ) );
    tr = net->Transitions;
    for ( lst = CutOffPoints ; lst ; lst = lst->next )
      {
        while ( tr && strcmp( tr->name, lst->name ) )
          {
            tr = tr->nexttrans;
          } /* while */
        if ( ! tr )
	{
          NNError( "Transition does not exist" );
	  exit(EXIT_FAILURE);
	}
        ( *(ptr++) = tr )->prevtrans
                             = (Transition) MYmalloc( sizeof( COPAddEntry ) );
	((COPAdd) ( tr->prevtrans ))->NumSpoilers=0;
	((COPAdd) ( tr->prevtrans ))->Spoilers=NULL;
	((COPAdd) ( tr->prevtrans ))->LastChoices=NULL;
      } /* for */
    *ptr = NULL;

    return ( array );

  } /* CreateCOPArray() */


/*****************************************************************************/
/*                 O P E R A T I O N S   O N   S T A C K S                   */
/*****************************************************************************/

static Place * GL_stackP,
             * GL_stackP2;

#define PushPlace( _PL_ ) ( (_PL_)->preset ? ( *++stackptr = _PL_ ) : _PL_ )

#define PopPlace() ( *stackptr ? *(stackptr--) : NULL )

#define InitPStack() ( *( stackptr = GL_stackP ) = NULL )


/*-------------------------------------------------------------------------*/

#if defined(__STDC__) || defined(__cplusplus)

static Cardinal32 GetCOPSpoilers( TransListPtr * Spoilers, Transition trans )

#else

static Cardinal32 GetCOPSpoilers( Spoilers, trans  )

  TransListPtr * Spoilers;
  Transition   trans;

#endif

  {

    register Place      * stackptr,
                        * stackptr2,
                        pl;
    register Transition tr;
    register STEdge     st;
    register TSEdge     ts;
    register Cardinal32 num = 0;

    IncPMark();
    IncTMark();
    InitPStack();
    *( stackptr2 = GL_stackP2 ) = NULL;

    TMark( trans );
    for ( st = trans->preset ; st ; st = st->nextplace )
      {
        if ( ( pl = st->place )->postset->nexttrans)
          *++stackptr2 = pl;
        PMark( PushPlace( pl ) );
      } /* for */
    while (( pl = PopPlace() ))
      {
        if ( IsNotTMarked( tr = pl->preset->trans ) )
          {
            TMark( tr );
            for ( st = tr->preset ; st ; st = st->nextplace )
              {
                PMark( pl = st->place );
                if ( ( ts = pl->preset ) && IsNotTMarked( ts->trans ))
                  PushPlace( pl );
                if ( pl->postset->nexttrans)
                  *++stackptr2 = pl;
              } /* for */
          } /* if */
      } /* while */

    stackptr = stackptr2;

    while (( pl = PopPlace() ))
      {
        for ( st = pl->postset ; st ; st = st->nexttrans )
          {
            if ( IsNotTMarked( tr = st->trans ) )
              {
                TMark( tr );
                if ( ! tr->prevtrans )
                  {
                    InsertTransition( Spoilers, tr );
                    num++;
                    for ( ts = tr->postset ; ts ; ts = ts->nextplace )
                      {
                        PushPlace( ts->place );
                      } /* for */
                  } /* if */
              } /* if */
          } /* for */
      } /* while */

    return ( num );

  } /* GetCOPSpoilers */

/*-------------------------------------------------------------------------*/

#if defined(__STDC__) || defined(__cplusplus)

static void DeleteSpoilers( Transition trans, MultiSet level )

#else

static void DeleteSpoilers( trans, level )

  TransListPtr * trans;
  MultiSet     level;

#endif

  {

    register Place      * stackptr,
                        * stackptr2,
                        pl;
    register Transition tr;
    register STEdge     st;
    register TSEdge     ts;

    IncPMark();
    IncTMark();
    InitPStack();
    *( stackptr2 = GL_stackP2 ) = NULL;

    TMark( trans );
    if (! trans->label)
      trans->label = level;
    for ( st = trans->preset ; st ; st = st->nextplace )
      {
        if ( ( pl = st->place )->postset->nexttrans)
          *++stackptr2 = pl;
        PMark( PushPlace( pl ) );
      } /* for */
    while (( pl = PopPlace() ))
      {
        if ( IsNotTMarked( tr = pl->preset->trans ) )
          {
            TMark( tr );
            if (! tr->label)
              tr->label = level;
            for ( st = tr->preset ; st ; st = st->nextplace )
              {
                PMark( pl = st->place );
                if ( ( ts = pl->preset ) && IsNotTMarked( ts->trans ))
                  PushPlace( pl );
                if ( pl->postset->nexttrans)
                  *++stackptr2 = pl;
              } /* for */
          } /* if */
      } /* while */

    stackptr = stackptr2;

    while (( pl = PopPlace() ))
      {
        for ( st = pl->postset ; st ; st = st->nexttrans )
          {
            if ( IsNotTMarked( tr = st->trans ) )
              {
                TMark( tr );
                if (! tr->label)
                  tr->label = level;
                for ( ts = tr->postset ; ts ; ts = ts->nextplace )
                  {
                    PushPlace( ts->place );
                  } /* for */
              } /* if */
          } /* for */
      } /* while */

  } /* DeleteSpoilers */

/*-------------------------------------------------------------------------*/
/* nur eine Prototyp-Deklaration - naeheres s. unten                       */

#if defined(__STDC__) || defined(__cplusplus)

static void GetTransSpoilers( TransListPtr * Spoilers   ,
                              Transition   trans       ,
                              PlaceListPtr * PlaceList  ,
                              TransListPtr * TransList  );

#else

static void GetTransSpoilers();

#endif

/*-------------------------------------------------------------------------*/
/*-- zu der Transition, auf die der uebergebene Zeiger STPtr zeigt,      --*/
/*-- werden alle Spoiler gesucht, indem geprueft wird, ob von der Stelle --*/
/*-- aus, auf die STPtr zeigt, weitere S/T-Kanten zu anderen Transitionen--*/
/*-- fuehren;                                                            --*/
/*-- des weiteren werden zu der Transition, die im Vorbereich dieser     --*/
/*-- dieser Stelle liegt (wenn denn vorhanden), wiederum alle Spoiler    --*/
/*-- gesucht                                                             --*/
/*-- Parameter :                                                         --*/
/*--   TransListPtr *Spoilers : Zeiger auf die Transitionenliste, in der --*/
/*--                            die Spoiler abgelegt werden              --*/
/*--   STEdge STPtr : Zeiger auf S/T-Kante, zu deren Transition die      --*/
/*--                  Spoiler gesucht werden                             --*/
/*--   PlaceListPtr *PlaceList : Referenz eines Zeigers auf eine Stel-   --*/
/*--                             lenliste, in der alle Stellen eingetra- --*/
/*--                             gen werden, die in der rekursiven       --*/
/*--                             Spoilersuche schon mal durchlaufen wur- --*/
/*--                             den; vermeidet doppelten Durchlauf      --*/
/*--   TransListPtr *TransList : analog zu *PlaceList                    --*/

#if defined(__STDC__) || defined(__cplusplus)

static void GetSpoilers( TransListPtr * Spoilers   ,
                         STEdge       STPtr       ,
                         PlaceListPtr * PlaceList  ,
                         TransListPtr * TransList  )

#else

static void GetSpoilers( Spoilers, STPtr, PlaceList, TransList  )

  TransListPtr * Spoilers;
  STEdge       STPtr;
  PlaceListPtr * PlaceList;
  TransListPtr * TransList;

#endif

  {
    register STEdge STConn,
                    st;

    /* gibt es ausgehend von der Stelle, auf die die uebergebene  */
    /* S/T-Kante STPtr zeigt, weitere S/T-Verbindungen zu anderen */
    /* Transitionen? - genau diese Transitionen werden als        */
    /* Spoiler gesucht                                            */

    /* falls die zu STPtr gehoerende Stelle schon mal durchlaufen */
    /* wurde, fertig -- sonst in PlaceList einfuegen              */
    if ( InPlaceList( *PlaceList, STPtr->place ) )
      {
        return;
      } /* if */

    InsertPlace( PlaceList, STPtr->place );

    /* eine S/T-Kante im Nachbereich der Stelle, auf die */
    /* die uebergebene S/T-Kante STPtr zeigt */
    STConn = STPtr->place->postset;

    /* per prevtrans und nexttrans werden ggf. weitere S/T-Kanten abgesucht */
    /* erst in die eine Richtung suchen... */
    for ( st = STConn ; st ; st = st->nexttrans )
      {
        /* die Transition, auf die STPtr zeigt, interessiert */
        /* natuerlich nicht -- wenn sie Cut-Off-Punkt ist,   */
        /* ggf. auch nicht                                   */
        if ( ( st != STPtr ) && ! IsCOP( st->trans ) )
          {
            /* Spoiler gefunden, in die Liste einfuegen */
            InsertTransition( Spoilers, st->trans );
          } /* if */
      } /* for */

    /* ... dann in die andere Richtung */
    for ( st = STConn->prevtrans ; st ; st = st->prevtrans )
      {
        /* die Transition, auf die STPtr zeigt, interessiert */
        /* natuerlich nicht -- wenn sie Cut-Off-Punkt ist,   */
        /* ggf. auch nicht                                   */
        if ( ( st != STPtr ) && ! IsCOP( st->trans ) )
          {
            /* Spoiler gefunden, in die Liste einfuegen */
            InsertTransition( Spoilers, st->trans );
          } /* if */
      } /* for */

    /* hat die Stelle, auf die STPtr zeigt, einen Vorbereich? */
    if ( STConn->place->preset )
      {
        /* alle Spoiler zu der Transition im  */
        /* Vorbereich dieser Stelle ermitteln */
        GetTransSpoilers( Spoilers, STConn->place->preset->trans,
                          PlaceList, TransList );
      } /* if */

  } /* GetSpoilers() */

/*-------------------------------------------------------------------------*/
/*-- ermittelt alle Spoiler zur Transition trans;                        --*/
/*-- Parameter :                                                         --*/
/*--   TransListPtr *Spoilers : Referenz auf einen Zeiger auf ein Tran-  --*/
/*--                            sitionslisten-Element;                   --*/
/*--                            ueber diesen Zeiger werden die ermittel- --*/
/*--                            ten Spoiler zurueckgegeben               --*/
/*--   Transition trans : die Transition, zu der alle Spoiler gesucht    --*/
/*--                      werden                                         --*/
/*--   PlaceListPtr *PlaceList : Referenz eines Zeigers auf eine Stel-   --*/
/*--                             lenliste, in der alle Stellen eingetra- --*/
/*--                             gen werden, die in der rekursiven       --*/
/*--                             Spoilersuche schon mal durchlaufen wur- --*/
/*--                             den; vermeidet doppelten Durchlauf      --*/
/*--   TransListPtr *TransList : analog zu *PlaceList                    --*/

#if defined(__STDC__) || defined(__cplusplus)

static void GetTransSpoilers( TransListPtr * Spoilers   ,
                              Transition   trans       ,
                              PlaceListPtr * PlaceList  ,
                              TransListPtr * TransList  )

#else

static void GetTransSpoilers( Spoilers, trans, PlaceList, TransList  )

  TransListPtr * Spoilers;
  Transition   trans;
  PlaceListPtr * PlaceList;
  TransListPtr * TransList;

#endif

  {

    register STEdge st;

    /* falls trans schon mal durchlaufen wurde, fertig -- */
    /* sonst in TransList einfuegen                       */
    if ( InTransList( *TransList, trans ) )
      {
        return;
      } /* if */

    InsertTransition( TransList, trans );

    /* es koennen mehrere Stellen im Vorbereich der Transition liegen; */
    /* bei jeder dieser Stellen wird geprueft, ob von diesen weitere   */
    /* S/T-Kanten zu anderen Transitionen, eben den Spoilern, ausgehen */

    /* die S/T-Kante im Vorbereich der Transition */
    for ( st = trans->preset ; st ; st = st->nextplace )
      {
        /* die aktuelle S/T-Kante wird zur Spoilersuche uebergeben */
        GetSpoilers( Spoilers, st, PlaceList, TransList );
      } /* for */

    /* auch in dieser Richtung koennen weitere Stellen liegen */
    for ( st = trans->preset->prevplace ; st ; st = st->prevplace )
      {
        /* die aktuelle S/T-Kante wird zur Spoilersuche uebergeben */
        GetSpoilers( Spoilers, st, PlaceList, TransList );
      } /* for */

  } /* GetTransSpoilers() */

/*-------------------------------------------------------------------------*/
/*-- zaehlt die nicht geloeschten Transitionen der uebergebenen Liste    --*/
/*-- und gibt das Ergebnis zurueck                                       --*/
/*-- Parameter :                                                         --*/
/*--   TransListPtr ListElem: Zeiger auf das erste Element der Liste von --*/
/*--                          Transitionszeigern                         --*/
/*--                                                                     --*/
/*-- Rueckgabe :                                                         --*/
/*--   Typ Cardinal32 : die Anzahl nicht geloeschter Transitionen in der --*/
/*--                    Liste                                            --*/

#if defined(__STDC__) || defined(__cplusplus)

static Cardinal32 CountTransitions( TransListPtr ListElem )

#else

static Cardinal32 CountTransitions( ListElem )

  TransListPtr ListElem;

#endif

  {
    register Cardinal32   counter = 0;
    register TransListPtr list;

    for ( list = ListElem ; list ; list = list->Next )
      {
        /* wenn nicht geloescht, Zaehler inkrementieren */
        if ( ! list->trans->label )
          {
            counter++;
          } /* if */
      } /* for */

    return ( counter );

  } /* CountTransitions() */

/*-------------------------------------------------------------------------*/
/*-- ermittelt fuer alle Cut-Off-Punkte die Spoiler;                     --*/

#if defined(__STDC__) || defined(__cplusplus)

static void GetAllSpoilers( void )

#else

static void GetAllSpoilers()

#endif

  {

    register Transition * ptr = COPArray - 1;
    register COPAdd     copinfo;

    while ( *++ptr )
      {
        copinfo = (COPAdd) ( (*ptr)->prevtrans );
        copinfo->NumSpoilers = GetCOPSpoilers( &( copinfo->Spoilers ), *ptr );
      } /* while */

  } /* GetAllSpoilers() */

/*-------------------------------------------------------------------------*/
/*-- prueft, ob alle Cut-Off-Punkte als geloescht markiert sind          --*/
/*-- Parameter :                                                         --*/
/*--   COPListPtr FirstCOP : Zeiger auf das erste Element der COP-Liste  --*/
/*--                                                                     --*/
/*-- Rueckgabe : True  - alle COPs sind als geloescht markiert           --*/
/*--             False - es gibt noch einen nicht geloeschten COP        --*/

#if defined(__STDC__) || defined(__cplusplus)

static bool COPSetEmpty( void )

#else

static bool COPSetEmpty()

#endif

  {

    register Transition * ptr = COPArray - 1;

    /* solange das aktuelle Element geloescht und noch weitere in der Liste */
    while ( *++ptr && (*ptr)->label )
      skip;

    /* nicht am Listenende ? Liste ist nicht leer : Liste ist leer */
    return ( *ptr ? False : True );

  } /* COPSetEmpty() */

/*-------------------------------------------------------------------------*/
/*-- waehlt aus allen nicht geloeschten Cut-Off-Punkten denjenigen mit   --*/
/*-- den wenigsten Spoilern aus                                          --*/
/*--                                                                     --*/
/*-- Rueckgabe : Zeiger auf das Listenelement mit den wenigsten Spoilern --*/
/*--             bzw. NULL                                               --*/

#if defined(__STDC__) || defined(__cplusplus)

static Transition ChooseCOP( void )

#else

static Transition ChooseCOP()

#endif

  {

    register Transition * ptr = COPArray - 1;
    register COPAdd     copinfo;
    Transition ChosenCOP = NULL;
    /* Variable mit einem fiktiven grossen Wert initialisieren */
    Cardinal32 NumSpoil = MAX_SPOILERS;

    /* solange nicht am Listenende */
    while ( *++ptr )
      {
        copinfo = (COPAdd) ( (*ptr)->prevtrans );
        /* wenn COP nicht geloescht */
        if ( ! (*ptr)->label )
          {
            /* zunaechst fuer alle nicht geloeschten COPs die Anzahl nicht */
            /* geloeschter Spoiler bestimmen */
            copinfo->NumSpoilers = CountTransitions( copinfo->Spoilers );

            /* hat aktueller COP weniger Spoiler? */
            if ( copinfo->NumSpoilers < NumSpoil )
              {
                /* diesen COP auswaehlen */
                ChosenCOP = *ptr;
                /* aktuelle Spoileranzahl als Minimum merken */
                NumSpoil = copinfo->NumSpoilers;
              } /* if */
          } /* if */
      } /* while */

    /* den gewaehlten COP bzw. NULL zurueckgeben */
    return ( ChosenCOP );

  } /* ChooseCOP() */

/*-------------------------------------------------------------------------*/
/*-- waehlt einen nicht geloeschten Spoiler (Transition) aus der Spoi-   --*/
/*-- lerliste des uebergebenen Cut-Off-Punktes aus, sofern dieser nicht  --*/
/*-- schon vorher einmal ausgewaehlt wurde                               --*/
/*-- Parameter :                                                         --*/
/*--   COPListPtr COPElem : Zeiger auf ein COP-Listenelement, aus dessen --*/
/*--                        Spoilerliste ein Spoiler ausgwaehlt werden   --*/
/*--                        soll                                         --*/
/*--                                                                     --*/
/*-- Rueckgabe : Zeiger auf eine Transition ( = Spoiler)                 --*/

#if defined(__STDC__) || defined(__cplusplus)

static Transition ChooseSpoiler( COPAdd copinfo )

#else

static Transition ChooseSpoiler( copinfo )

  COPAdd copinfo;

#endif

  {

    /* Anfang der Spoilerliste */
    register TransListPtr trList = copinfo->Spoilers;

    /* solange nicht am Listenende und aktuelle Transition geloescht */
    /* bzw. aktuelle Transition schon mal ausgewaehlt                */
    while (    trList
            && (    trList->trans->label
                 || InTransList( copinfo->LastChoices, trList->trans )
               )
          )
      {
        trList = trList->Next;
      } /* while */

    /* Listenende nicht erreicht ?               */
    /* ja   : Zeiger auf Transition zurueckgeben */
    /* nein : NULL zurueckgeben                  */
    return ( trList ? trList->trans : NULL );

  } /* ChooseSpoiler() */

/*-------------------------------------------------------------------------*/
/*-- nur eine Prototyp-Deklaration - naeheres s. unten                   --*/

#if defined(__STDC__) || defined(__cplusplus)

static void DelTransPath( Transition Trans, Cardinal32 Level );

#else

static void DelTransPath();

#endif

/*-------------------------------------------------------------------------*/
/*-- loescht alle Transitionen, die auf die Stelle folgen, von der die   --*/
/*-- uebergebene S/T-Kante ausgeht                                       --*/
/*-- Parameter :                                                         --*/
/*--   STEdge STConn : S/T-Kante von der Stelle, deren nachfolgende      --*/
/*--                   Transitionen geloescht werden sollen              --*/
/*--   Cardinal32 Level  : der Loeschstatus, mit dem die Transitionen als--*/
/*--                       geloescht markiert werden sollen              --*/

#if defined(__STDC__) || defined(__cplusplus)

static void DelPlacePostSet( STEdge STConn, Cardinal32 Level )

#else

static void DelPlacePostSet( STConn, Level )
  STEdge STConn;
  Cardinal32 Level;

#endif

  {

    register STEdge st;

    /* gibt es ueberhaupt eine S/T-Kante? */
    if ( STConn )
      {
        /* uebergebene S/T-Kante merken */
        /* fuer die uebergebene und alle evtl. folgenden */
        /* Kanten, die von der Stelle ausgehen           */
        for ( st = STConn ; st ; st = st->nexttrans )
          {
            /* die entsprechende Transition und alle ggf. folgenden loeschen */
            DelTransPath( st->trans, Level );
          } /* for */

        /* fuer alle evtl. vorhergehenden Kanten, die von der Stelle ausgehen */
        for ( st = STConn->prevtrans ; st ; st = st->prevtrans )
          {
            /* die entsprechende Transition und alle ggf. folgenden loeschen */
            DelTransPath( st->trans, Level );
          } /* for */
      } /* if */

  } /* DelPlacePostSet() */

/*-------------------------------------------------------------------------*/
/*-- markiert die uebergebene Transition und alle auf diese folgenden    --*/
/*-- Transitionen als geloescht                                          --*/
/*-- Parameter :                                                         --*/
/*--   Transition Trans : die Transition, die selbst und alle deren      --*/
/*--                      folgenden Transitionen geloescht werden sollen --*/
/*--   Cardinal32 Level : der Loeschstatus, mit dem die Transitionen als --*/
/*--                      geloescht markiert werden sollen               --*/

#if defined(__STDC__) || defined(__cplusplus)

static void DelTransPath( Transition Trans, Cardinal32 Level )

#else

static void DelTransPath( Trans, Level )
  Transition Trans;
  Cardinal32 Level;

#endif

  {

    register TSEdge ts;

    /* Zeiger ungleich NULL? */
    if ( ! Trans || Trans->label )
      return;

    /* die uebergebene Transition loeschen */
    if ( ! Trans->label )
      /* ... im Feld label den Wert DelLevel eintragen */
      Trans->label = (MultiSet) Level;

    /* fuer alle T/S-Kanten, die von der uebergebenen Transition ausgehen */
    for ( ts = Trans->postset ; ts ; ts = ts->nextplace )
      {
        /* alle Transitionen, die auf die Stelle folgen, auf die */
        /* die aktuelle T/S-Kante zeigt, loeschen                */
        DelPlacePostSet( ts->place->postset, Level );
      } /* for */

  } /* DelTransPath() */

/*-------------------------------------------------------------------------*/
/*-- markiert alle zur uebergebenen Transition Spoiler in Konflikt ste-  --*/
/*-- henden Transitionen und alle auf diese folgenden mit dem Loesch-    --*/
/*-- status Level als geloescht                                          --*/
/*-- Parameter :                                                         --*/
/*--   Transition Spoiler : die Transition, zu der alle in Konflikt      --*/
/*--                        stehenden Transitionen geloescht werden      --*/
/*--                        sollen                                       --*/
/*--   Cardinal32 Level : der Loeschstatus, mit dem die Transitionen als --*/
/*--                      geloescht markiert werden sollen               --*/

#if defined(__STDC__) || defined(__cplusplus)

static void DelInConflictWith( Transition Spoiler, Cardinal32 Level)

#else

static void DelInConflictWith( Spoiler, Level)

  Transition Spoiler;
  Cardinal32 Level;

#endif

{

  register TransListPtr trptr;
  TransListPtr          ConflictTransList = NULL;

  /* Zeiger auf eine Stellenliste, in der alle Stellen eingetra- */
  /* gen werden, die in der rekursiven Spoilersuche schon mal    */
  /* durchlaufen wurden; vermeidet doppelten Durchlauf;          */
  /* Analoges gilt fuer die Transitionenliste                    */
  PlaceListPtr plList = NULL;
  TransListPtr trList = NULL;

  /* es sollen alle zu Spoiler in Konflikt stehenden Transitionen */
  /* gesucht werden, dazu koennen auch Cut-Off-Punkte zaehlen;    */
  /* DoCOPCheck wird also auf False gesetzt, damit eine Pruefung, */
  /* ob eine gefundene Transition Cut-Off-Punkt ist, unterbleibt; */
  /* abgefragt wird DoCOPCheck in der Funktion IsCOP()            */
  DoCOPCheck = False;

  /* alle zur uebergebenen Transition Spoiler in */
  /* Konflikt stehenden Transitionen ermitteln   */
  /* und in _ConflictTransList ablegen           */
  GetTransSpoilers( &ConflictTransList, Spoiler, &plList, &trList );

  /* Stellen- und Transitionenliste wieder loeschen */
  DeletePlaceList( &plList );
  DeleteTransList( &trList );

  /* fuer alle mit Spoiler in Konflikt stehenden Transitionen... */
  for ( trptr = ConflictTransList ; trptr ; trptr = trptr->Next )
    {
      /* Transition und alle darauf folgenden loeschen */
      DelTransPath( trptr->trans, Level );
    } /* for */

  /* Transitionenliste loeschen */
  DeleteTransList( &ConflictTransList );

} /* DelInConflictWith() */

/*-------------------------------------------------------------------------*/
/*-- fuehrt den Algorithmus zur Deadlock-Suche aus                       --*/
/*-- Parameter :                                                         --*/
/*--   Transition Trans : Zeiger auf die erste Transition im Netz        --*/
/*--                                                                     --*/
/*-- Rueckgabe : True  : Deadlock gefunden                               --*/
/*--             False : kein Deadlock gefunden                          --*/

#if defined(__STDC__) || defined(__cplusplus)

static bool DoTheCheck( Transition Trans )

#else

static bool DoTheCheck( Trans )

  Transition Trans;

#endif

  {

    Transition          _Spoiler = NULL;
    register Transition tr;
    COPAdd              copinfo;
    Cardinal32          _Level = 0;
    bool                _NoMoreCOP;

    /* solange es noch einen nicht geloeschten Cut-Off-Punkt gibt */
    while ( ! ( _NoMoreCOP = COPSetEmpty() ) )
      {
        /* nicht geloeschtes COP-Element mit */
        /* den wenigsten Spoilern waehlen */
        copinfo = (COPAdd) ChooseCOP()->prevtrans;
/*
        _COPElem = ChooseCOP();
*/
    
        /* einen Spoiler dieses COP-Elements waehlen */
        /* gab es noch eine Wahl?                    */
        if (( _Spoiler = ChooseSpoiler( copinfo ) ))
          {
            /* den gewaehlten Spoiler in die Liste LastChoices      */
            /* des aktuellen COP eintragen, damit dieser nach       */
            /* einem evtl. Backtracking nicht nochmal gewaehlt wird */
            InsertTransition( &( copinfo->LastChoices ), _Spoiler );

            /* _Level (Anzahl der Durchlaeufe des Algorithmus) inkrementieren */
            _Level++;

            /* alle Transitionen in Konflikt mit dem _Spoiler loeschen */
            DelInConflictWith( _Spoiler, _Level );
/*
            DeleteSpoilers( _Spoiler, (MultiSet) _Level );
*/
          } /* if */
        else
          {
            /* gewaehlter Cut-Off-Punkt hatte keinen Spoiler; */
            /* also ggf. Backtracking durchfuehren            */

            /* nur solange _Level groesser Null, kann */
            /* Backtracking ausgefuehrt werden        */
            if ( _Level )
              {
                /* die Liste der schon mal gewaehlten Spoiler */
                /* wieder loeschen; wenn ein neuer Durchlauf  */
                /* wieder auf diesen COP trifft, duerfen alle */
                /* erneut ausgewaehlt werden                  */
                if ( _Level > 1 )
                  {
                    DeleteTransList( &( copinfo->LastChoices ) );
                  } /* if */

                /* alle beim Durchlauf _Level geloeschten        */
                /* Transitionen wieder als ungeloescht markieren */
                for ( tr = Trans ; tr ; tr = tr->nexttrans )
                  {
                    /* wenn das Feld label den Wert Level enthaelt ... */
                    if ( (Cardinal32) (tr->label) == _Level )
                      {
                        /* ... dort eine Null eintragen */
                        tr->label = (MultiSet) NOT_DELETED;
                      } /* if */
                  } /* for */

                /* Durchlaufzaehler dekrementieren */
                _Level--;
              } /* if */
            else
              {
                /* falls Backtracking nicht mehr moeglich, fertig */
                break;
              } /* else */
          } /* else */
      } /* while */

    return ( _NoMoreCOP ? DEADLOCK_FOUND : NO_DEADLOCK_FOUND );

  } /* DoTheCheck() */

/*-------------------------------------------------------------------------*/
/*-- Ergebnisausgabe in den Stream OutPut                                --*/
/*-- Parameter :                                                         --*/
/*--   Transition Path : Zeiger auf die erste Transition des Netzes;     --*/
/*--                     Path != NULL - es gibt einen Deadlock, alle     --*/
/*--                       nicht geloeschten Transitionen werden ausge-  --*/
/*--                       geben                                         --*/
/*--                     Path == NULL - es gibt keinen Deadlock          --*/
/*--  FILE *OutPut  : Ausgabe-Stream - entweder eine Datei oder stdout   --*/

#if defined(__STDC__) || defined(__cplusplus)

static void PrintPath( Transition Path, FILE *OutPut )

#else

static void PrintPath( Path, OutPut )
TransListPtr Path;
FILE *OutPut;

#endif
{
  Transition _Trans;
  Cardinal32 _NextLen, _Len = 0;

  if ( !Path )
  {
    /* es ist kein Deadlock-Pfad auszugeben */
    fprintf( OutPut, "TRUE\n\n" );
    return;
  }

  /* Deadlock gefunden */
  fprintf( OutPut, "FALSE\n" );
  fprintf( OutPut, "_SEQUENCE:\n" );

  /* von allen nicht geloeschten Transitionen den Namen */
  /* in der Entfaltung und im Originalnetz ausgeben     */
  for ( _Trans = Path ; _Trans ; _Trans = _Trans->nexttrans )
  {
    if ( ! _Trans->label )
    {
      /* Laenge der aktuellen Namen */
      if (_Trans->inscript)
        _NextLen = strlen(_Trans->inscript) + 1;
      else
	_NextLen = 4;

      /* Gesamtlaenge fuer eine Zeile */
      _Len += _NextLen;
      if (_Len > PATH_LINELEN)
      {
        /* neue Zeile, Gesamtlaenge auf aktuelle Laenge zuruecksetzen */
        /* BG leider erstmal rausgenommen, damit sich die Ergebnisse simulieren lassen. */
        fprintf( OutPut, NEW_PATHLINE );
        /* fprintf( OutPut, TRANS_DELIMITER ); */
        _Len = _NextLen;
      }

      if ( _Len > _NextLen )
        /* wenn nicht am Zeilenanfang, erstmal ein Komma ausgeben */
        fprintf( OutPut, TRANS_DELIMITER );

      /* den Namen ausgeben */
      if (_Trans->inscript)
        fprintf( OutPut, "%s", _Trans->inscript );
      else
        fprintf( OutPut, "Tic" );
    }
  } /* for */
  fprintf( OutPut, PATH_END );
} /* PrintPath() */

bool *ev_used,*frozen;

static bool freeze(Transition e)
{
  STEdge ce;
  TSEdge ec;

  if (ev_used[e->numname]) return False;
  if (frozen[e->numname]) return True;
  frozen[e->numname]=True;
  for(ec=e->postset;ec;ec=ec->nextplace)
    for (ce=ec->place->postset;ce;ce=ce->nexttrans)
      if (!freeze(ce->trans))
	return False;
  return True;
}
  
static bool max_conf(Transition e)
{
  STEdge ce,ce2;
  
  if (ev_used[e->numname]) return True;
  if (frozen[e->numname]) return False;
  ev_used[e->numname]=True;
  frozen[e->numname]=True;
  for(ce=e->preset;ce;ce=ce->nextplace)
  {
    if (ce->place->preset && !max_conf(ce->place->preset->trans))
      return False;
    if (ce->place->postset->nexttrans==NULL)
      continue;
    for (ce2=ce->place->postset;ce2;ce2=ce2->nexttrans)
      if ((ce2->trans!=e) && (!freeze(ce2->trans)))
	return False;
  }
  return True;
}
    
  
  
/*-------------------------------------------------------------------------*/
/*-- Konfigurationsausgabe in den Stream OutPut                          --*/
/*-- Parameter :                                                         --*/
/*--  LMPNet net          : Netz mit der Entfaltung                      --*/
/*--  FILE *OutPut  : Ausgabe-Stream - entweder eine Datei oder stdout   --*/

#if defined(__STDC__) || defined(__cplusplus)

static void PrintConfig( LMPNet net, FILE * OutPut )

#else

static void PrintConfig( net, OutPut )
  LMPNet net;
  FILE *OutPut;

#endif
{
  LIST_POINTER _Conf, Config=NULL;
  Transition _Trans, ev;
  Place pl;
  Cardinal32 _NextLen, _Len = 0;
  int i;

    ev_used=(bool*)MYmalloc((net->NumTransitions+1)*sizeof(bool));
    frozen=(bool*)MYmalloc((net->NumTransitions+1)*sizeof(bool));
    for(i=0;i<=net->NumTransitions;++i)
    {
      ev_used[i]=False;
      frozen[i]=False;
    }
  for(pl=net->Places; pl; pl=pl->nextplace)
    if(!pl->postset && pl->preset && !ev_used[pl->preset->trans->numname])
      max_conf(pl->preset->trans);

  for(ev=net->LastTransition;ev;ev=ev->prevtrans)
    if(ev_used[ev->numname])
    {
      if(!Config)
        Config=NewListElem(ev->name);
      else
      {
        _Conf=Config;
        Config=NewListElem(ev->name);
	Config->next=_Conf;
      }
    }
  free(ev_used);
  free(frozen);

  fprintf( OutPut, "FALSE\n" );
  fprintf( OutPut, "_SEQUENCE:\n" );

  /* von allen nicht geloeschten Transitionen den Namen */
  /* in der Entfaltung und im Originalnetz ausgeben     */
  for (_Conf = Config; _Conf; _Conf = _Conf->next)
  {
    if (( _Trans = find_T( net, _Conf->name ) ))
    {
      /* Laenge der aktuellen Namen bzw. Tic */
      if (_Trans->inscript)
        _NextLen = strlen(_Trans->inscript) + 1;
      else
	_NextLen = 4;

      /* Gesamtlaenge fuer eine Zeile */
      _Len += _NextLen;
      if (_Len > PATH_LINELEN)
      {
        /* neue Zeile, Gesamtlaenge auf aktuelle Laenge zuruecksetzen */
        fprintf( OutPut, NEW_PATHLINE );
        _Len = _NextLen;
      }

      if ( _Len > _NextLen )
        /* wenn nicht am Zeilenanfang, erstmal ein Komma ausgeben */
        fprintf( OutPut, TRANS_DELIMITER );

      /* den Namen ausgeben */
      if (_Trans->inscript)
        fprintf( OutPut, "%s", _Trans->inscript );
      else
        fprintf( OutPut, "Tic" );
    }
  }
  DeleteNameList(Config);
  fprintf( OutPut, PATH_END );
} /* PrintConfig() */

/*-------------------------------------------------------------------------*/
/*-- Hauptroutine zur Deadlockerkennung                                  --*/
/*-------------------------------------------------------------------------*/
/*-- Hauptroutine zur Deadlockerkennung                                  --*/
/*-- Parameter :                                                         --*/
/*--   LMPNet net : Netz mit der zu pruefenden Entfaltung                --*/
/*--   LIST_POINTER *CutOffPoints :                                      --*/
/*--                 Referenz eines Zeigers auf eine Liste mit den Namen --*/
/*--                 der Cut-Off-Punkte                                  --*/
/*--   char *OutFile : Name der Ausgabedatei                              --*/

#if defined(__STDC__) || defined(__cplusplus)

void CheckForDeadlock( LMPNet       net,
                       LIST_POINTER * CutOffPoints,
                       char         * OutFile )

#else

void CheckForDeadlock( net, CutOffPoints, OutFile )

  LMPNet       net;
  LIST_POINTER * CutOffPoints;
  char         * OutFile;

#endif

  {

    FILE *Output;
    Transition tr;
    Place      pl;
    char       ** inscriptarray,
               ** ptr;

    /* wurde ueberhaupt ein Name fuer die Ausgabedatei angegeben? */
    if ( !OutFile )
      Output=stdout;
    /* Ausgabedatei oeffnen */
    else if ( ! ( Output = fopen( OutFile, "w" ) ) )
    {
      /* bei Misserfolg Fehlermeldung */
      printf( E_FOPEN, OutFile );
      Output=stdout;
    }

    GL_net = net;
    /* wenn keine Cut-Off-Punkte in der Entfaltung,                  */
    /* Ergebnisausgabe: Ursprungsnetz enthaelt Deadlock - und fertig */
    if ( ! *CutOffPoints  )
      {
        PrintConfig( net, Output);
        return;
      } /* if */

    /* Sicherheitshalber in allen Transitionen    */
    /* das Feld label auf Null setzen; mit diesem */
    /* Feld werden sie als geloescht bzw. nicht   */
    /* geloescht markiert                         */
    inscriptarray = ptr = (char **) MYmalloc( net->NumTransitions
                                                          * sizeof( char * ) );
    for ( tr = net->Transitions ; tr ; tr = tr->nexttrans )
      {
        *( ptr++ ) = tr->inscript;
        tr->inscript = NULL;
        tr->prevtrans = NULL;
        tr->label = (MultiSet) NOT_DELETED;
      } /* for */
    for ( pl = net->Places ; pl ; pl = pl->nextplace )
      {
        pl->component = 0;
      } /* for */

    GL_stackP  = (Place *) MYmalloc( ( net->NumPlaces + 1 ) * sizeof( Place ) );
    GL_stackP2 = (Place *) MYmalloc( ( net->NumPlaces + 1 ) * sizeof( Place ) );

    /* Cut-Off-Points in eine COP-List uebertragen */
    COPArray = CreateCOPArray( net, *CutOffPoints );

    /* zu jedem Cut-Off-Point Spoiler suchen */
    GetAllSpoilers();

    if ( COPwithoutSpoiler() )
      {
        ptr = inscriptarray;
        for ( tr = net->Transitions ; tr ; tr = tr->nexttrans )
          {
            tr->inscript = *( ptr++ );
          } /* for */
        /* Es gibt Cut-Off-Punkte ohne Spoiler */
        /* Ergebnisausgabe in Datei und auf    */
        /* Monitor: es gibt keinen Deadlock    */
        PrintPath( NULL, Output );
      } /* if */
    else
      {
        /* Algorithmus zur Deadlocksuche durchfuehren */
        if ( DoTheCheck( net->Transitions ) )
          {
            ptr = inscriptarray;
            for ( tr = net->Transitions ; tr ; tr = tr->nexttrans )
              {
                tr->inscript = *( ptr++ );
              } /* for */
            /* es wurde Deadlock gefunden               */
            /* Ergebnisausgabe in Datei und auf Monitor */
            PrintPath( net->Transitions, Output );
          } /* if */
        else
          {
            ptr = inscriptarray;
            for ( tr = net->Transitions ; tr ; tr = tr->nexttrans )
              {
                tr->inscript = *( ptr++ );
              } /* for */
            /* Es wurde kein Deadlock gefunden:         */
            /* Ergebnisausgabe in Datei und auf Monitor */
            PrintPath( NULL, Output );
          } /* else */
      } /* else */

    /* Datei schliessen */
    fclose( Output );

    Free( (char *) inscriptarray );

    /* ggf. Cut-Off-Punkt-Liste aus Speicher entfernen */
    Free( COPArray );
    Free( GL_stackP );
    Free( GL_stackP2 );

    if (( tr = net->Transitions ))
      {
        if ( tr->prevtrans )
	{
	  DeleteTransList(&((COPAdd) (tr->prevtrans))->LastChoices);
	  free( tr->prevtrans );
	}
        for ( tr->prevtrans = NULL ; tr->nexttrans ; tr = tr->nexttrans )
          {
            if ( tr->nexttrans->prevtrans )
	    {
	      DeleteTransList(&((COPAdd) (tr->nexttrans->prevtrans))->Spoilers);
	      DeleteTransList(&((COPAdd) (tr->nexttrans->prevtrans))->LastChoices);
	      free( tr->nexttrans->prevtrans );
	    }
            tr->nexttrans->prevtrans = tr;
          } /* for */
      } /* if */

  } /* CheckForDeadlock() */

/*-------------------------------------------------------------------------*/
/*--                                                                     --*/
/*--  Ende DL_CHK.C                                                      --*/
/*--                                                                     --*/
/*-------------------------------------------------------------------------*/

