/*-------------------------------------------------------------------------*/
/*--                                                                     --*/
/*--  DL_CHK.H - Header-Datei fuer DL_CHK.C                              --*/
/*--                                                                     --*/
/*--  Schnittstelle fuer die Deadlock-Erkennung in Entfaltungen          --*/
/*--                                                                     --*/
/*--  Johannes Jaeger - Sommer 1994                                      --*/
/*-------------------------------------------------------------------------*/

#ifndef _DL_CHK_H
#define _DL_CHK_H

#include "neunetze.h"
#include "list.h"

/*-------------------------------------------------------------------------*/
/*-- Listenelement fuer eine einfach verkettete Liste von Transitions-   --*/
/*-- Zeigern auf Cut-Off-Punkte - zu einem Cut-Off-Punkt werden in einer --*/
/*-- Transitionenliste die zugehoerigen Spoiler gespeichert, unter Num-  --*/
/*-- Spoilers die Anzahl der Spoiler sowie unter LastChoices diejenigen  --*/
/*-- Transitionen, die im Suchalgorithmus zum Backtracking gefuehrt haben--*/

typedef struct COPAddEntry_
{
  Cardinal32          NumSpoilers;  /* Anzahl Spoiler zu diesem COP        */
  TransListPtr        Spoilers;     /* Liste der Spoiler zu diesem COP     */
  TransListPtr        LastChoices;  /* Liste der bei der Deadlock-Suche    */
                                    /* schon einmal ausgewaehlten Spoiler; */
                                    /* diese sollen nach einem Backtracking*/
                                    /* nicht noch einmal gwaehlt werden    */
  Cardinal32          marker;
} COPAddEntry,
  *COPAdd;

/*-------------------------------------------------------------------------*/
/*-- Hauptroutine zur Deadlockerkennung                                  --*/
/*-- Parameter :                                                         --*/
/*--   LMPNet net : Netz mit der zu pruefenden Entfaltung                --*/
/*--   LIST_POINTER *CutOffPoints :                                      --*/
/*--                 Referenz eines Zeigers auf eine Liste mit den Namen --*/
/*--                 der Cut-Off-Punkte                                  --*/
/*--   char *OutFile : Name der Ausgabedatei                             --*/


#if defined(__STDC__) || defined(__cplusplus)

extern void CheckForDeadlock( LMPNet net                ,
                              LIST_POINTER *CutOffPoints,
                              char *OutFile             );

#else

extern void CheckForDeadlock();

#endif

/*-------------------------------------------------------------------------*/

#endif /* _DL_CHK_H */

/*-------------------------------------------------------------------------*/
/*--                                                                     --*/
/*--  Ende DL_CHK.H                                                      --*/
/*--                                                                     --*/
/*-------------------------------------------------------------------------*/
