/*-------------------------------------------------------------------------*/
/*--                                                                     --*/
/*--  DL_MAIN.C    Hauptmodul zur Deadlocksuche in Entfaltungen          --*/
/*--                                                                     --*/
/*--  Johannes Jaeger - Sommer 1994                                      --*/
/*-------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "neunetze.h"
#include "pep_strings.h"
#include "rw_prefix.h"
#include "dl_chk.h"
#include "readlib.h"

/*-------------------------------------------------------------------------*/
/*-- Ausgabe der Aufruf-Syntax nach stdout                               --*/

#if defined(__STDC__) || defined(__cplusplus)

static void PrintSyntax( void )

#else

static void PrintSyntax( )

#endif
{
  printf( "\nUsage : dlcheck <unfolding> [<resultfile>]\n" );
  printf( "         unfolding : Input file with unfolding\n" );
  printf( "         resultfile: Output file (else stdout)\n" );
  exit( 0 );
} /* PrintSyntax() */

/*-------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------*/
#if defined(__STDC__) || defined(__cplusplus)

int main( int argc, char *argv[] )

#else

int main( argc, argv )
int argc;
char *argv[];

#endif
{
  LMPNet       net           = NULL;
  LIST_POINTER CutOffPoints  = NULL;
  LIST_POINTER MaxConfigList = NULL;
  clock_t      ttime;

  /* Anzahl der Aufruf-Parameter korrekt? */
  if ( (argc < 2) || (argc > 3) )
    PrintSyntax();

  /* Entfaltung aus einer Datei in ein LMPNet einlesen,   */
  /* in CutOffPoints ein Liste der Cut-Off-Punkte liefern */
  GL_numCutoffs = 0;
  net = readPrefix( argv[1], &CutOffPoints, &MaxConfigList );

  /* Einlesen erfolgreich? */
  if (net)
  {

    ttime=clock();

    /* Deadlocksuche ausfuehren */
    CheckForDeadlock( net, &CutOffPoints, argv[2] );

    printf("\nTime needed (sec): %10.2f\n"
           ,(float)(clock()-ttime)/CLOCKS_PER_SEC);

    /* Liste der Cut-Off-Punkte entfernen */
    remove_list( &CutOffPoints );

    /* Liste einer max. Konfiguration entfernen */
    remove_list( &MaxConfigList );

    /* Netz entfernen */
    {Place pl;
	for(pl=net->Places;pl;pl=pl->nextplace)
	  pl->inscript=NULL;
    }
    remove_net( &net );

    return 0;
  }
  else
  {
    PEPerror( " Unfolding could not be loaded!\n" );
    exit(1);
  }

} /* main() */

/*-------------------------------------------------------------------------*/
/*--                                                                     --*/
/*--  Ende DL_MAIN.C                                                     --*/
/*--                                                                     --*/
/*-------------------------------------------------------------------------*/
