/*****************************************************************************/
/* FILE bpn.y                                                                */
/* BISON source file: parser for b(pn)^2                                     */
/* 1993 by Burkhard Graves                                                   */
/* 1994 by Lars Jenner                                                       */
/* completely reviewed February 1995 by Stephan Merkel                       */
/* winter 1996/97 by Martin Ackermann                                        */
/*****************************************************************************/
/* Note: Rules containing square brackets ('[' and ']') are no longer        */
/* actively supported and provided only for historical continuity with older */
/* versions.                                                                 */
/* The default action '{ $$ = $1 }' is not explicitly given in rules.        */
/*****************************************************************************/

%{
#include <string.h>
#include "frontend.h"
#include "symtab.h"
#include "tree.h"

/* #define YYERROR_VERBOSE */
  /* this provides for verbose error messages with bison as well as with */
  /* yacc when using the proper driver file (yacc_dri.own)               */

#define MAX_EXPECT 20
  /* this sets a limit to the number of expected tokens printed into */
  /* error messages                                                  */

int yyerror( char * msg )
{
  Msg( ERR+64, msg ); /* +64 prevents double incrementation of yynerrs */
  return 1;
}

ShortEntryPtr ref;

#define _add_ref_to_node_( node, p1, p2, type_ )    \
  {                                                 \
    ref=MYmalloc( sizeof(ShortEntry) );             \
    ref->s_ref.start_l = p1.start_l;                \
    ref->s_ref.start_c = p1.start_c;                \
    ref->s_ref.end_l   = p2.end_l;                  \
    ref->s_ref.end_c   = p2.end_c;                  \
    ref->s_type        = type_;                     \
    *((ShortEntryPtr*)(node+1))=ref;                \
  }

#define _create_action_( erg, p1, p2, p3 )          \
  {                                                 \
    ref=MYmalloc( sizeof(ShortEntry) );             \
    ref->s_ref.start_l = p1.start_l;                \
    ref->s_ref.start_c = p1.start_c;                \
    ref->s_ref.end_l   = p3.end_l;                  \
    ref->s_ref.end_c   = p3.end_c;                  \
    ref->s_type        = _ACTION;                   \
    erg=GenNode(_EXPR,p2,(TreeNode*)ref);           \
  }

extern int   yylex( void );

%}

%union {
  SymTabEntry   *sym;    /* defined in file symtab.h */
  IdentStruct    ident;  /* defined in file ident.h  */
  TreeNode      *node;   /* defined in file tree.h   */
  ArgListEntry  *arg;    /* defined in file tree.h   */
  SetRange      *range;  /* defined in file set.h    */
  RefStruct      ref;    /* defined in file symtab.h */
  int            wert;
}

%token  <ref> BEGIN_
%token  END
%token  CONST
%token  VAR
%token  CHAN
%token  STACK
%token  PROC
%token  VAL
%token  RES
%token  REF
%token  OF
%token  INIT
%token  DO
%token  OD
%token  REPEAT
%token  EXIT
%token  ENTER
%token  PARALLEL
%token  CHOICE
%token  NOT
%token  AND
%token  OR
%token  TRUE
%token  FALSE
%token  MAX
%token  PROCGRP
%token  ENDGRP
%token  <wert>  NUMBER
%token  <wert>  CONST_IDENT
%token  <ident> IDENT
%token	<ref>   CA       /* closing action */
%token  <ref>   '<'
%token  <ref>   '>'
%token  <ref>   '['
%token  <ref>   ']'
%token  <ref>   ')'
%token  EMPTY           /* '??' */
%token  FULL            /* '!!' */
%token  GE              /* '>=' */
%token  LE              /* '<=' */
%token  MOD             /* '%'  */


%left           PARALLEL
%left           CHOICE
%left           ';' ','
%left           OR
%right          AND /* don't change to left - TreeToGuard needs right-order */
%right          NOT
%nonassoc       '=' '#' '<' '>' GE LE
%left           '+' '-'
%left           '*' '/' MOD /* bison and yacc are incompatible on '%' */
%right          Unaryminus


%type <sym>   proc_ident par_list val_par_decl ref_par_decl par_ident
%type <node>  program block good_block scope pscope com good_com alt_set
%type <node>  proc_call expr val_expr simple_expr
%type <node>  bad_block bad_com bad_alt_set 
%type <arg>   arg_list
%type <range> set good_set bad_set elem_list elem
%type <wert>  const_expr pos_expr init proc_ext
%type <ref>   begin

%start  start


%%
start
        : {
		procgrplist = NULL;
		forwardprocs = alloc_list();
	  }
	  program
         	{ if ( !$2 || ($2->type != _BLOCK) ) {
	            if (! GL_Frags)
	              {
	                Msg( ERR, "program is not correct block" );
	              } else
	              { /* pretend to have seen an END */
		        tree = GenNode(_BLOCK,(TreeNode *)dec_blknum(),$2);
			*((ShortEntryPtr*)(tree+1))=NULL;
	              }
		  } else
		    tree = $2;
		  if (forwardprocs->count)
			Msg(ERR,"undefined procedure %s",
				list_elem(procgrplist,0,SymTabEntry*)->s_name);
	        }
        ;

program    
        : decls { $$ = NULL; }
        | scope { $$ = $1; }
	;

block
	: good_block 
	| bad_block
	;

good_block
	: begin scope end
		{
		  $$=GenNode(_BLOCK,(TreeNode *)dec_blknum(),$2);

		  _add_ref_to_node_($$,$1,$1,_BEGIN);
		}
        | begin IDENT 
                {
		  if ( $2.sym->s_type != _UDEC )
		    Msg( ERR,"identifier %s already used", $2.sym->s_name );
                  give_typ(_BLOCKNAME, NULL, 0);
		}
          scope end IDENT
		{
		  $$=GenNode(_BLOCK,(TreeNode *)dec_blknum(),$4);

		  if ( $2.sym!=$6.sym )
		    Msg( ERR, "blocknames differ (BEGIN '%s', END '%s')",
                         $2.sym->s_name,$6.sym->s_name);

		  _add_ref_to_node_($$,$1,$2.ref,_BEGIN);
		}
	;

scope
	: pscope
        | decls pscope
                {
                  $$=$2;
                }
	;

decls
	: decl
	| decls decl
	;

decl
	: good_decl ';'
	| bad_decl
	;

good_decl
	: const_decl
	| var_decl
	;

pscope
	: com
		{       
		  $$=$1;
		}
        | pdecls com
                {
                  $$=$2;
                }
	;

pdecls
	: pdecl
	| pdecls pdecl
	;

pdecl
	: proc_decl ';'
	| procgrp ';'
      	| error proc_decl
	;

const_decl
	: const IDENT '=' const_expr
		{
		  if ( $2.sym->s_type != _UDEC )
		    Msg( ERR,"identifier %s already used", $2.sym->s_name );
		  give_typ( MNET_CONST, NULL, $4 );
		}
	;

var_decl
	: var ident_list ':' set init
		{
		  give_typ(_VAR,$4,$5);
		}
	| var ident_list ':' CHAN const_expr OF set
		{
		  give_typ(_CHANNEL,$7,$5);
		}
	| var ident_list ':' STACK const_expr OF set
		{
		  give_typ( _STACK, $7, $5 );
		}
	;

procgrp	: PROCGRP
	  {
		T_list tmp = procgrplist;
		procgrplist = alloc_list();
		list_add(procgrplist,tmp);
		list_add(procgrplist,blknum);
	  }
	  pdecls ENDGRP
	  {
		SymTabEntry *call,*proc;
		T_list reducedfp = alloc_list();
		int i;

		for_list (forwardprocs,call,SymTabEntry*)
		{
			for (i = procgrplist->count-1; i > 1; i--)
			{
				proc = list_elem(procgrplist,i,SymTabEntry*);
				if (strcmp(call->s_name,proc->s_name)) continue;
				*(SymTabEntry**)(call->s_set) = proc;
				check_parm_list((ArgListEntry*)call->s_parlist,
					proc->s_parlist);
				break;
			}
			if (i == 1) list_add(reducedfp,call);
		}
		free_list(forwardprocs);
		forwardprocs = reducedfp;
		reducedfp = procgrplist;
		procgrplist = list_elem(procgrplist,0,T_list);
		free_list(reducedfp);
	  }
	;

proc_decl
	: sim_proc_decl
	| par_proc_decl
	;

sim_proc_decl
	: proc proc_ident proc_ext
		/* this action may NOT go to the end of the rule */
		{
		  give_typ( _PROC, NULL, $3);
		}
	  block
		{
		  LinkBody($2,$5);
		}
	;

par_proc_decl
	: proc proc_ident '('
		/* this action may NOT go to the end of the rule */
		{
		  give_typ(_PPROC, NULL, 0 );
		}
	  par_list ')' proc_ext
		/* this action may NOT go to the end of the rule */
		{
		  LinkParlist($2,$5);
		  $2->s_init_or_cap = $7;
		}
	  block
		{
		  LinkBody($2,$9);
		}
	;

proc_ident
	: IDENT
		{
		  $$=make_ident(&$1);
		  if (procgrplist && list_elem(procgrplist,1,int) == blknum)
			list_add(procgrplist,$$);
		}
	;

proc_ext
        : /* empty rule */
                {
		  $$ = NONE;
		}
        | MAX const_expr
                {
		  if ($2<0) {
		    Msg(ERR,"Maximum of PROC-instances must be positive !");
		  }
		  if ($2>MAX_POS) {
		    Msg(ERR,"Maximum of PROC-instances must be a number !");
		  }
		  $$ = $2;
		}
	;

par_list
	: val val_par_decl
		{
		  $$=GenParList(NULL,_VAL,$2);
		}
	| par_list ',' val val_par_decl
		{
		  $$=GenParList($1,_VAL,$4);
		}
	| CONST val_par_decl
		{
		  $$=GenParList(NULL,_CONSTPAR,$2);
		}
	| par_list ',' CONST val_par_decl
		{
		  $$=GenParList($1,_CONSTPAR,$4);
		}
	| RES val_par_decl
		{
		  $$=GenParList(NULL,_RES,$2);
		}
	| par_list ',' RES val_par_decl
		{
		  $$=GenParList($1,_RES,$4);
		}
	| REF ref_par_decl
		{
		  $$=GenParList(NULL,_REF,$2);
		}
	| par_list ',' REF ref_par_decl
		{
		  $$=GenParList($1,_REF,$4);
		}
	;

val     : VAL
        | bad_val
        ;

val_par_decl
	: par_ident ':' set
		{ 
		  give_typ( _VAR, $3, NONE );
		}
	;

ref_par_decl
	: val_par_decl
	| par_ident ':' CHAN const_expr OF set
		{ 
		  give_typ(_CHANNEL,$6,$4);
		}
	| par_ident ':' STACK const_expr OF set 
		{
		  give_typ(_STACK,$6,$4); 
		} 
	;

par_ident
	: IDENT
		{
		  $$=make_par(&$1);
		}
	;

ident_list
	: IDENT
		{
		  make_ident(&$1);
		}
	| ident_list ',' IDENT   
		{
		  make_ident(&$3);
		}
	;

set
	: good_set
	| bad_set
	;

good_set
	: '{' elem_list '}'
		{
		  $$ = $2;
		}
	;

elem_list
	: elem
		/* insert element into empty list */
		{
		  $$ = insert_elem( $1, NULL );
		}
	| elem_list ',' elem
		/* insert element into elem_list */
		{
		  $$ = insert_elem( $3, $1 );
		}
	;

elem
	: const_expr
		{
		  $$ = GenRange( $1, $1 );
		}
	| const_expr '.' '.' const_expr
		{
		  $$ = GenRange( $1, $4 );
		}
	;

init
	: /* empty init */
		{
		  $$=NONE;
		}
	| INIT const_expr
		{
		  $$=$2;
		}
	;

com
	: good_com
	| bad_com
	;

good_com
	: '<' expr CA
		{
		  _create_action_( $$, $1, $2, $3 );
		}
	| com ';' com
		{
		  $$=GenNode(_SEQ,$1,$3);
		}
	| com PARALLEL com
		{
		  $$=GenNode(_PAR,$1,$3);
		}
	| good_block
	| do alt_set od
		{
		  $$=GenNode(_DO,$2,NULL);
		}
	| do com enter alt_set od
		{
		  $$ = GenNode(_DO,$4,$2);
		}
	| proc_call
		{
		  GL_ProcCall = True;
		}
	;

proc_call
	: IDENT
		{
		  $$=GenNode(_CALL,(TreeNode*)($1.sym),NULL);
		  if (chk_proc_ident($1.sym,_PROC))
			$1.sym->s_set = (SetRange*)&($$->op1);
		  _add_ref_to_node_($$,$1.ref,$1.ref,_PROCCALL);
		}
	| IDENT '('
		/* this action can NOT go to the end of the rule */
		{
		  ActPar = (chk_proc_ident($1.sym,_PPROC))?
			(SymTabEntry*) 1: $1.sym->s_parlist;
		}
	  arg_list ')'
		{
                  if ( (int) ActPar > 1)
		    Msg(ERR,"too few arguments");
		  $$=GenNode(_PCALL,(TreeNode*)($1.sym),$<node>4);
		  if (ActPar)
		  {
			$1.sym->s_set = (SetRange*)&($$->op1);
			$1.sym->s_parlist = (SymTabEntry*)($$->op2);
		  }
		  _add_ref_to_node_($$,$1.ref,$5,_PROCCALL);
		}
	;

arg_list
	: IDENT
		{
		  $$=GenArgList(NULL,_IDENT,$1.sym);
		}
	| arg_list ',' IDENT
		{
		  $$=GenArgList($1,_IDENT,$3.sym);
		}
	| val_expr
		{
		  $$=GenArgList(NULL,_VAL,$1);
		}
	| arg_list ',' val_expr
		{
		  $$=GenArgList($1,_VAL,$3);
		} 
	;

alt_set
	: com sc REPEAT
		{
		  $$=GenNode(_REPEAT,$1,NULL);
		}
	| com sc EXIT
		{
		  $$=GenNode(_EXIT,$1,NULL);
		}
	| alt_set CHOICE alt_set
		{
		  $$=GenNode(_CHOICE,$1,$3);
		}
	;

expr
	: '(' expr ')'
                {
		  $$=GenNode(N_KLAMM,$2,NULL);
		}
	| IDENT
		{
		  chk_ident($1.sym,_VAR);
		  $1.sym->s_special|=_LG_UNPRIMED;
		  $$=GenNode(N_VAR,(TreeNode *)$1.sym,(TreeNode *)_UNPRIMED);
		}
	| val_expr
	;

val_expr
	: simple_expr
	| expr OR expr
		{
		  $$=GenNode(N_OR,$1,$3);
		}
	| expr AND expr
		{
		  $$=GenNode(N_AND,$1,$3);
		}
	| '!' expr %prec NOT
		{
		  $$=GenNode(N_NOT,$2,NULL);		  
		}
	| NOT expr
		{
		  $$=GenNode(N_NOT,$2,NULL);		  
		}
	| expr '=' expr
		{
		  $$=GenNode(N_EQ,$1,$3);
		}
	| expr '#' expr
		{
		  $$=GenNode(N_UNEQ,$1,$3);
		}
	| expr '<' expr
		{
		  $$=GenNode(N_LT,$1,$3);
		}
	| expr '>' expr
		{
		  $$=GenNode(N_GT,$1,$3);
		}
	| expr GE expr
		{
		  $$=GenNode(N_GE,$1,$3);
		}
	| expr LE expr
		{
		  $$=GenNode(N_LE,$1,$3);
		}
	| expr '+' expr
		{
		  $$=GenNode(N_PLUS,$1,$3);
		}
	| expr '-' expr
		{
		  $$=GenNode(N_MIN,$1,$3);
		}
	| expr '*' expr
		{
		  $$=GenNode(N_MULT,$1,$3);
		}
	| expr '/' expr
		{
		  $$=GenNode(N_DIV,$1,$3);
		}
	| expr MOD expr
		{
		  $$=GenNode(N_MOD,$1,$3);
		}
	| '-' expr %prec Unaryminus
		{
		  $$=GenNode(N_SIGN,$2,NULL);
		}
	;

simple_expr
	: '\'' IDENT
		{
		  chk_ident($2.sym,_VAR);
		  $2.sym->s_special|=_LG_PRIMED;
		  $$=GenNode(N_VAR,(TreeNode *)$2.sym,(TreeNode *)_PREPRIMED);
		}
	| IDENT '\''
		{
		  chk_ident($1.sym,_VAR);
		  $1.sym->s_special|=_LG_PRIMED;
		  $$=GenNode(N_VAR,(TreeNode *)$1.sym,(TreeNode *)_POSTPRIMED);
		}
	| IDENT '?'
		{
		  chk2_ident($1.sym,_CHANNEL,_STACK);
		  $1.sym->s_special|=_LG_READING;
		  $$=GenNode(N_VAR,(TreeNode *)$1.sym,(TreeNode *)_READ);
		}
	| IDENT '!'
		{
		  chk2_ident($1.sym,_CHANNEL,_STACK);
                  $1.sym->s_special|=_LG_WRITING;
		  $$=GenNode(N_VAR,(TreeNode *)$1.sym,(TreeNode *)_WRITE);
		}
	| IDENT EMPTY
		{
		  chk2_ident($1.sym,_CHANNEL,_STACK);
		  if ( $1.sym->s_init_or_cap == 0 )
		    Msg(ERR,"illegal use of chan-0-identifier %s",
			$1.sym->s_name);
                  $1.sym->s_special|=_LG_EMPTY;
		  $$=GenNode(N_VAR,(TreeNode *)$1.sym,(TreeNode *)_EMPTY);
		}
	| IDENT FULL
		{
		  chk2_ident($1.sym,_CHANNEL,_STACK);
		  if ( $1.sym->s_init_or_cap == 0 )
		    Msg(ERR,"illegal use of chan-0-identifier %s",
			$1.sym->s_name);
                  $1.sym->s_special|=_LG_FULL;
		  $$=GenNode(N_VAR,(TreeNode *)$1.sym,(TreeNode *)_FULL);
		}
	| pos_expr
		{
		  $$=GenNode(N_CONST,(TreeNode *)$1,NULL);
		}
	;

const_expr
	: pos_expr
        | '-' NUMBER
                {
                  if (-$2 < MIN_NEG)   
                    Msg( FAT, "Constant out of range !" );
                  $$=-$2;
                }
        ;

pos_expr
        : NUMBER
	| TRUE
		{
		  $$=TT;
		}
	| FALSE
		{
		  $$=FF;
		}
	| CONST_IDENT
	;

begin
	: BEGIN_ { inc_blknum(); $$ = $1; }
	;

end
	: END { yyerrok; }
	;

sc                                                               /* semicolon */
	: ';' { yyerrok; }
	;

const
        : CONST { yyerrok; }
	;

var
	: VAR { yyerrok; }
	;

proc
	: PROC { yyerrok; GL_ProcDecl = True; }
	;

choice
	: CHOICE { yyerrok; }
	;
	
do
	: DO { yyerrok; }
	;

od
	: OD { yyerrok; }
	;

enter
	: ENTER { yyerrok; }
	;


/************************ error recovery rules *******************************/
/* Note: If return values of 'bad_'-rules are handed upward, it is merely to */
/* make sure they are seen by delete_tree in case they are not nil even      */
/* though some of them should be.                                            */
/*****************************************************************************/

bad_block
	: error scope end
		{
		  $$ = $2;
		}
	;

bad_decl
	: error sc
	| error const_decl
	| error var_decl
	| const CONST_IDENT '=' const_expr ';'
		{
		  Msg( WAR, "constant identifier redeclared" );
		}
	;

bad_set
	: '[' const_expr ',' const_expr ']'
		{
		  num_old_syn++;
		  if ( ! GL_OldSyn )
		    {
		      Msg( WAR, "old syntax in set declaration" );
		    } /* if */
		  $$ = insert_elem( GenRange( $2, $4 ), NULL );
		}
	;

bad_val
	: /* empty rule */
		{
		  num_old_syn++;
		  if ( ! GL_OldSyn )
		    {
		      Msg( WAR, "old syntax (missing keyword VAL)" );
		    } /* if */
		}
	;

bad_com
	: '[' expr ']'
		{
		  num_old_syn++;
		  if ( ! GL_OldSyn )
		    {
		      Msg( WAR, "old syntax in atomic action" );
		    }
		  _create_action_( $$, $1, $2, $3 );
		}
	| error '<' expr CA
		{
		  $$ = NULL;
		}
	| bad_seq_com com %prec ';'
		{
		  $$ = $2;
		}
	| error PARALLEL com
		{
		  $$ = $3;
		}
	| error do alt_set od
		{
		  $$ = $3;
		}
	| error do com enter alt_set od
		{
		  $$ = NULL;
		}
	| bad_alt_set od
	;

bad_seq_com
        : '<' error sc {}
	| do error sc
	;

bad_alt_set
	: com ';' error REPEAT
	| com ';' error EXIT
	| bad_alt_set choice alt_set %prec CHOICE
		{
		  $$ = $1;
		}
	;


/*****************************************************************************/
/* END OF FILE bpn.y                                                         */
/*****************************************************************************/
