/******************************************************************************/
/* FILE frontend.c                                                            */
/*                                                                            */
/* interface of the b(pn)2-frontend (Scanner, Parser and semantical analyser) */
/*                                                                            */
/* Burkhard Graves, 10.10.93                                                  */
/* Lars Jenner, Juni 1994                                                     */
/* Stephan Merkel, Mar. 95                                                    */
/* Martin Ackermann, Winter 1996/97                                           */
/******************************************************************************/


#include <stdio.h>
#include "frontend.h"


/******************************************************************************/
/* global variables                                                           */
/******************************************************************************/

                           /* meanings if flag == True: */
                           /*---------------------------*/
bool GL_Frags     = False, /* allow for code fragments  */
     GL_OldSyn    = False, /* allow for old syntax      */
     GL_ProcDecl  = False, /* there is at least one ... */
     GL_ProcCall  = False, /* ... and it was called     */
     GL_ExplFree  = False, /* check memory management   */
     GL_ParserMsg = False; /* Msg tells extra infos     */

int num_old_syn; /* number of times old syntax is being used */

TreeNode *tree;  /* the resulting tree */


/******************************************************************************/
/* Parses the source, writes parser errors in target and prints status        */
/* messages in err.                                                           */
/*                                                                            */
/* Result: 0 = correct program -> parse tree in *tree (global)                */
/*         1 = could finish parsing, but detected errors                      */
/*         2 = not able to finish parsing                                     */
/******************************************************************************/

int bpn2parse( FILE * source, FILE * target, FILE * err )

  /* Caution: Parameters are NOT checked for validity. Therefore all FILEs    */
  /* must be valid;                                                           */
  {
    extern int yynerrs;
    extern int yyparse( void );
      /* defined in bpn.c, respectively in the bison/yacc driver file */
    int parse_ret;

    yyin = source;
    yyout = target;
    msg_out=err;
    num_old_syn = 0;

    s_init();

    if ( (parse_ret = yyparse()) )
      {
        fprintf( msg_out, "Parse aborted.\n" );
      }
    else
      {
        fprintf( msg_out, "Parse complete.\n" );
      }

    if ( GL_ProcDecl || GL_ProcCall )
      {
        fprintf( msg_out, "Program with procedure(s).\n" );
      }
    else
      {
        fprintf( msg_out, "Program without procedures.\n" );
      }

    if ( ( num_old_syn > 0 ) && ( ! GL_OldSyn ) )
      {
        fprintf( msg_out, "Using old syntax %i time(s).\n", num_old_syn );
      }

    if ( yynerrs )
      /* program not correct */
      {
        delete_tree( tree );
        if ( parse_ret == 0 )
          return ( 1 ); /* parse complete */
        else
          return ( 2 ); /* parse incomplete */
      }
    else
      return ( 0 ); /* program correct */

  } /* bpn2parse() */

