/*****************************************************************************/
/* FILE scanner.l                                                            */
/*                                                                           */
/* FLEX source file: scanner for b(pn)^2                                     */
/*                                                                           */
/* winter 1996/97 by Martin Ackermann                                        */
/*****************************************************************************/

%option nounput
%option noyywrap

%{
#include <string.h>
#ifdef _AIX
#include <strings.h>
#endif
#include <unistd.h>
#include <ctype.h>

#include "bpn_tab.h"

int yylineno = 1;
int yycoluno = 1;

char *reserved[26]={"begin", "end", "const", "var", "chan", "stack", 
                    "proc", "val", "res", "ref", "of", "init", 
                    "do", "od", "repeat", "exit", "enter", 
                    "parallel", "not", "and", "or", "true", "false", "_max_",
		    "procgrp", "endgrp"};

int  _reserved[26]={BEGIN_, END, CONST, VAR, CHAN, STACK,
                     PROC, VAL, RES, REF, OF, INIT,
                    DO, OD, REPEAT, EXIT, ENTER,
                    PARALLEL, NOT, AND, OR, TRUE, FALSE, MAX,
		    PROCGRP, ENDGRP};

#define copy_ref( ref )               \
{                                     \
  ref.start_l = yylineno;             \
  ref.start_c = yycoluno;             \
  ref.end_l   = yylineno;             \
  ref.end_c   = yycoluno + yyleng;    \
}

/* Makes the distinction between normal identifiers and identifiers for 
   constants */
int ident( void ) {
  s_lookup(yytext,&(yylval.ident)); 
  if ( yylval.ident.sym->s_type == MNET_CONST ) {
    yylval.ident.sym->s_special |= ( _G_RW | _G_PRUPR );
    /* zeigt Verwendung des const-Bezeichners an */
    yylval.wert = yylval.ident.sym->s_init_or_cap;
    return CONST_IDENT;
  } else {
    copy_ref( yylval.ident.ref )
    return IDENT;
  }
}

%}

delim     [ \t]
ws        ({delim}|\n)*
digit     [0-9]
letter    [A-Za-z_]
ident     {letter}({letter}|{digit})*
number    [1-9]{digit}*|0
comment   "//".*

comfollow (;|parallel|PARALLEL|"||"|enter|ENTER|end|END)
          /* if a '>' is followed by this, we have a closing action */

          /* Bison / Yacc do not treat explicit priorities right in every case.
             If we don't distinguish GreaterThan and ClosingAction in the
             scanner, e.g. in the action <true or 4>5> "or" is parsed with
             a higher priority than the first ">" - which is wrong. */ 

gap       ({ws}|({comment}\n))*
          /* all which can occur between two consecutive symbols */

%x ET     

%%
                { yycoluno += yyleng; /* add length of previous token */ }

<*>\n           { yylineno++; yycoluno=1; }

<*>{delim}+     { yycoluno += yyleng; /* white space */ }

<*>{comment}/(\n?) { /* comment -> skip until end of line */ }

"||"            { return PARALLEL; }

"|"             { return OR; }

"[]"            { return CHOICE; }

"&"             { return AND; }

"<="            { return LE; }

">="            { return GE; }

"??"            { return EMPTY; }

"!!"            { return FULL; }

"%"             { return MOD; }

{number}        { int py; 
                  sscanf(yytext,"%d",&py); 
                  if (py > MAX_POS)
                    Msg( FAT, "Constant out of range !" );
                  yylval.wert = py;
                  return NUMBER;
                }

"BEGIN"         |
"begin"         { copy_ref( yylval.ref );
                  return BEGIN_;
                }

_?[A-Z]+_?      |
_?[a-z]+_?      { int i;
                  for (i=0; i<sizeof(reserved)/sizeof(*reserved); ++i)
                    if (!strcasecmp(yytext,reserved[i]))
                      return _reserved[i];
                  return ident();
                }

{ident}         { return ident(); }

">"/{gap}{comfollow} {
                  copy_ref( yylval.ref );
                  return CA; 
                }

">"             { copy_ref( yylval.ref );
                  BEGIN(ET); /* EofTest: test if next symbol is EOF */ 
                }


<ET><<EOF>>     { BEGIN(INITIAL);
                  return CA; 
                }

<ET>.           { yyless(0); /* reject everything */
                  yyleng=1;  /* consider the length of '>' */
                  BEGIN(INITIAL); 
                  return '>'; 
                }

[<)\[\]]        { copy_ref( yylval.ref );
                  return yytext[0];
                }

[-=#+*/:{},.(!?';] { return yytext[0]; }

 
%%
