/******************************************************************************/
/* FILE set.c                                                                 */
/* Stephan Merkel, May 95                                                     */
/* Martin Ackermann, Winter 1996/97                                           */
/******************************************************************************/

#include "set.h"

/******************************************************************************/
/* Funktionen                                                                 */
/******************************************************************************/


/******************************************************************************/
/********************* GenRange() *********************************************/
/******************************************************************************/
/* Liefert einen Zeiger auf eine Struktur, die die beiden Bereichsgrenzen     */
/* eines Set-Teilbereichs enthaelt.                                           */
/******************************************************************************/

Set GenRange( int low, int high )
  {
    Set range;

    range = ( Set ) MYmalloc( sizeof( SetRange ) );

    range->next = NULL;

    if ( low <= high )
      {
        range->low = low;
        range->high = high;
      } /* else if */
    else
      {
	range->low = high;
	range->high = low;
      } /* else */

    if ( is_bool( range->low ) != is_bool( range->high ) )
      {
	Msg( ERR, "true/false and numbers were mixed in a single declaration" );
	range->low  = FF;
	range->high = TT;
      } /* if */

    return ( range );
  } /* GenRange() */



/******************************************************************************/
/*************************** is_elem() ****************************************/
/******************************************************************************/

bool is_elem( Set set, int elem )
  {
    if ( elem < set->next->low || elem > SET_MAX( set ) )
      {
	return ( False );
      } /* if */

    set = set->next;
    while( elem >= set->low )
      {
	if ( elem <= set->high )
	  {
	    return ( True );
	  } /* if */
	else
	  {
	    set = set->next;
	  } /* else */
      } /* while */
    return ( False );
  } /* is_elem() */




/******************************************************************************/
/************************* sub_set() ******************************************/
/******************************************************************************/

bool sub_set( Set sub, Set super )
  {
    super = super->next;
    sub = sub->next;
    while ( sub )
      {
	while ( sub->low > super->high )
	  {
	    if ( super->next == NULL )
	      {
		return ( False );
	      } /* if */
	    super = super->next;
	  } /* while */

	if ( sub->low < super->low || sub->high > super->high )
	  {
	    return ( False );
	  } /* if */
	sub = sub->next;
      } /* while ( sub ) */
    
    return ( True );
  } /* sub_set() */



/******************************************************************************/
/************************ insert_elem() ***************************************/
/******************************************************************************/
/* fuegt ein neues Element in die geordnete Liste von Bereichen/einzelnen     */
/* Zahlen ein, falls es nicht schon in einem anderen enthalten ist.           */
/******************************************************************************/

Set insert_elem( Set new_elem, Set old_head )
  {
    Set curr1, curr2, mem, head;
    bool SkippingMoreThanOne;

    if ( old_head == NULL )
      /* neue Liste erzeugen */
      {
	head = GenRange( new_elem->high, new_elem->high );
	  /* dies ist der Kopf der Liste; sein low-Wert wird nicht verwendet */
	  /* und nicht gepflegt.                                             */
	head->next = new_elem;
	return ( head );
      } /* if */

    curr1 = head = old_head;

    if ( is_bool( curr1->next->low ) != is_bool( new_elem->low ) )
      {
	Msg( ERR, "true/false and numbers were mixed in a single declaration" );
	return ( head );
      } /* if */

    while ( curr1->next && new_elem->low > curr1->next->high + 1 )
      {
	curr1 = curr1->next;
      } /* while */
    
    if ( curr1->next == NULL )
      /* new_elem->low > max( old_head ) + 1 */
      {
	curr1->next = new_elem;
	head->high = new_elem->high;
	return ( head );
      } /* if */

    if ( curr1->next->low > new_elem->high + 1 )
      /* keine Ueberlappungen und new_elem muss dazwischen gefuegt werden */
      {
	new_elem->next = curr1->next;
	curr1->next = new_elem;
	return ( head );
      } /* if */

    curr1 = curr2 = curr1->next;
    SkippingMoreThanOne = False;
    while ( curr2->next && curr2->next->low - 1 <= new_elem->high )
      {
	mem = curr2;
	curr2 = curr2->next;
	if ( SkippingMoreThanOne )
	  {
	    curr1->next = curr2;
	    MYfree( mem );
	  } /* if */
	SkippingMoreThanOne = True;
      } /* while */
    curr1->high = Max( curr2->high, new_elem->high );

    if ( curr2->next == NULL )
      /* SET_MAX koennte sich geaendert haben */
      {
	head->high = curr1->high;
      } /* if */

    if ( curr1->low > new_elem->low )
      {
	curr1->low = new_elem->low;
      } /* if */

    if ( curr1 != curr2 )
      {
	curr1->next = curr2->next;
	MYfree( curr2 );
      } /* if */
    MYfree( new_elem );
    return ( head );

  } /* insert_elem() */


/******************************************************************************/
/*************************** cardin() *****************************************/
/******************************************************************************/

int cardin( Set set )
  {
    Set curr;
    int num_elems = 0;

    if(set)
    for( curr = set->next; curr; curr = curr->next )
      {
	num_elems += curr->high - curr->low + 1;
      }
    return ( num_elems );
  } /* cardin() */


/******************************************************************************/
/******************** nth_elem() **********************************************/
/******************************************************************************/
/* Returns the value of the element in set whose ordinal number is ord_num    */
/* as enumerated in C, i. e. number of first element is 0.                    */
/******************************************************************************/

int nth_elem( Set set, int ord_num )
  {
    int i = 0;

    for ( set = set->next; i + ( set->high - set->low + 1) < ord_num + 1; 
							    set = set->next )
      {
	i += set->high - set->low + 1;
      } /* for */
    return ( set->low - i + ord_num );
  } /* nth_elem() */



/******************************************************************************/
/*************************** write_set() **************************************/
/******************************************************************************/

void write_set( FILE * file, Set set )
  {
    set = set->next;
    fputc( '{', file ); fprintset( file, set->low );
    if ( set->high > set->low )
      {
	fprintf( file, ".." ); fprintset( file, set->high );
      } /* if */
    set = set->next;
    while ( set )
      {
	fputc( ',', file ); fprintset( file, set->low );
	if ( set->high > set->low )
	  {
	    fprintf( file, ".." ); fprintset( file, set->high );
	  } /* if */
	set = set->next;
      } /* while */
    fprintf( file, "}" );
  } /* write_set() */


/******************************************************************************/
/*************************** delete_set() *************************************/
/******************************************************************************/

void delete_set( Set set )
  {
    Set akt;

    while ( set )
      {
        akt=set;
        set=set->next;
        MYfree(akt);
      } /* while */
  } /* delete_set() */



/******************************************************************************/
/* END OF FILE set.c                                                          */
/******************************************************************************/
