/******************************************************************************/
/* FILE symtab.c                                                              */
/* Burkhard Graves, 10.10.93                                                  */
/* Lars Jenner, Juni 1994                                                     */
/* Stephan Merkel, Mar. 95                                                    */
/* Martin Ackermann, Winter 96/97                                             */
/******************************************************************************/

#include <string.h>
#include "symtab.h"
#include "frontend.h"

/******************************************************************************/
/* Datenstrukturen                                                            */
/******************************************************************************/

int          blknum;    /* Verschachtelungstiefe */

SymTabEntry  dummy,     /* Dummy-Element der Symboltabelle */
	    *s_start;   /* Start der Symboltabelle         */

T_list	procgrplist;
T_list	forwardprocs;

/**********************************************************************/
/* Initialisierung der Symboltabelle (Aufruf aus 'frontend.c' heraus) */
/**********************************************************************/

void s_init(void)
{
  SymTabEntry *entry;
  
  (s_start=entry=&dummy)->s_next=NULL;
  entry->s_name="-";              /* kein gueltiger Bezeichner ! */
  entry->s_type=_VAR;             /* nicht _UDEC  !              */
  entry->s_blknum=blknum=0;       /* ungueltige Blocknummer !    */
  entry->s_body=NULL;
  entry->s_parlist=NULL;
  entry->s_partype=_NOPAR;
  entry->s_set=NULL;
}

/****************************************************************************/
/* Absuchen der Symboltabelle nach dem Bezeichner 'ptr'. Konnte er gefunden */
/* werden, so wird ein Zeiger auf ihn zurueckgegeben, ansonsten wird ein    */
/* NULL-Pointer zurueckgegeben. (Aufruf durch 's_lookup')                   */
/****************************************************************************/

static SymTabEntry *s_find(char *ptr)
{
  SymTabEntry *entry;
  
  entry=s_start;
  while (entry && (*entry->s_name!=*ptr || strcmp(entry->s_name,ptr))) {
    entry=entry->s_next;
  }
  return entry;
}

/**************************************************************/
/* Erzeugung eines neuen Eintrages in der Symboltabelle       */
/* (Aufruf durch 's_lookup' und 'make_ident' und GenParList)  */
/**************************************************************/

SymTabEntry *s_create(char *name)
{
  SymTabEntry *new_entry;
  
  new_entry=(SymTabEntry *)MYmalloc(sizeof(SymTabEntry));
  new_entry->s_next=s_start;
  s_start=new_entry;
  
  new_entry->s_name=MYstrdup(name);
  new_entry->s_type=_UDEC;
  
  if ( GL_Frags & (blknum==0) ) 
    inc_blknum(); /* if we scan for fragments and an identifier occurs without a
	  	     preceding BEGIN, we pretend to have seen a BEGIN */
  
  new_entry->s_blknum=blknum;
  
  new_entry->s_body=NULL;
  new_entry->s_parlist=NULL;
  new_entry->s_partype=_NOPAR;
  new_entry->s_set=NULL;
  new_entry->s_init_or_cap=NONE;
  new_entry->s_special=0;
  
  return new_entry;
}

/***************************************************************************/
/* Feststellung, ob ein Bezeichner 'ptr' bereits in der Symboltabelle ist. */
/* Falls nicht, so wird ein entsprechender Eintrag erzeugt.                */
/* Rueckgabewert ist ein Zeiger auf den Eintrag. (Aufruf durch den Scanner)*/
/***************************************************************************/

void s_lookup(char *ptr, IdentStruct *ident)
{
  ident->sym=s_find(ptr);
  if (!ident->sym)
    {
      ident->sym=s_create(ptr);
      ident->is_old = False;
    }
  else
    ident->is_old = True;
}

/*************************************************************************/
/* Feststellung, ob eine Variable mehrfach definiert wurde, oder ob eine */
/* lokale Definition eine globale ueberdeckt.                            */
/* (Aufruf durch den Parser in der 'ident_list'-Regel)                   */
/*************************************************************************/

SymTabEntry *make_ident(IdentStruct *ident)
{
  SymTabEntry *entry=ident->sym;

  if (ident->is_old) {
    if (ident->sym->s_blknum==blknum) {
      Msg(ERR,"multiple declaration of identifier %s",ident->sym->s_name);
      return entry;
    } else {
      entry=s_create(ident->sym->s_name);
    }
  }

  memcpy(&(entry->s_ref),&(ident->ref),sizeof(RefStruct));

  return entry;
}

/*************************************************************************/
/* Feststellung, ob ein Parameter mehrfach definiert wurde, oder ob eine */
/* lokale Definition eine globale ueberdeckt.                            */
/* (Aufruf durch den Parser in der 'par_Ident'-Regel)                    */
/*************************************************************************/

SymTabEntry *make_par(IdentStruct *ident)
{
  SymTabEntry *entry=ident->sym;

  if (ident->is_old) {
    if (ident->sym->s_blknum==blknum+1) {
      Msg(ERR,"multiple declaration of parameter %s",ident->sym->s_name);
      return entry;
    } else {
      entry=s_create(ident->sym->s_name);
    }
  }
 
  memcpy(&(entry->s_ref),&(ident->ref),sizeof(RefStruct));
  entry->s_blknum =blknum+1;    /* lokal zum folgenden Prozedurrumpf! */
  entry->s_partype=_VAL;        /* default */

  return entry;
}

/************************************************************************/
/* Feststellung, ob eine Variable den geforderten Typ besitzt.          */
/* (Aufruf durch '?-primed' und 'r_w_channel')                          */
/* Bei Prozeduren wird noch deren Verwendung in s_special vermerkt.     */
/************************************************************************/

void chk_ident(SymTabEntry *entry,int type)
{
  if ( entry->s_type == _UDEC )
    { 
      if ( !GL_Frags || type==_PROC || type==_PPROC )
	Msg(ERR,"undeclared identifier %s",entry->s_name);
    }
  else if (entry->s_type!=type) 
    Msg(ERR,"incorrect use of identifier %s",entry->s_name);
  
  if (entry->s_type==_PROC || entry->s_type==_PPROC ) 
    entry->s_special |= ( _G_RW | _G_PRUPR );
}

void chk2_ident(SymTabEntry *entry,int type1,int type2)
{
  if ( entry->s_type == _UDEC )
    { 
      if ( !GL_Frags )
	Msg(ERR,"undeclared identifier %s",entry->s_name);
    }
  else if (entry->s_type!=type1 && entry->s_type!=type2) 
    Msg(ERR,"incorrect use of identifier %s",entry->s_name);
}

int chk_proc_ident(SymTabEntry *entry,int type)
{
  if ( entry->s_type == _UDEC )
    { 
	if (!procgrplist)
	    Msg(ERR,"undeclared identifier %s",entry->s_name);
	else
	    list_add(forwardprocs,entry);
    }
  else if (entry->s_type!=type) 
    Msg(ERR,"incorrect use of identifier %s",entry->s_name);
  
  if (entry->s_type==_PROC || entry->s_type==_PPROC ) 
    entry->s_special |= ( _G_RW | _G_PRUPR );
  return entry->s_type == _UDEC;
}

/*********************************************************************/
/* Inkrementierung von 'blknum'.                                     */
/* (Aufruf durch den Parser am Beginn eines Blockes ('block'-Regel)) */
/*********************************************************************/

void inc_blknum(void)
{
  ++blknum;
}

/*******************************************************************/
/* Dekrementierung von 'blknum' und Entfernung von lokalen         */
/* Eintraegen aus der Symboltabelle. (Aber noch nicht loeschen,    */
/* sondern zurueckgeben !)                                         */
/* (Aufruf durch den Parser am Ende eines Blockes ('block'-Regel)) */
/*******************************************************************/

SymTabEntry *dec_blknum(void)
{
  SymTabEntry *ptr1,*ptr2,*memo;

  if ( GL_Frags & (blknum==0) ) 
    inc_blknum(); /* if we scan for fragments and no identifiers occured,
	  	     we pretend now to have seen a BEGIN */
  
  memo=ptr1=ptr2=s_start;

  while (ptr1->s_blknum==blknum) {
    if ( !(ptr1->s_special & (_G_RW | _G_PRUPR)) && (ptr1->s_type!=_UDEC) 
                                                 && (ptr1->s_type!=_BLOCKNAME) )
    {
      Msg(WAR,"identifier %s declared but not used",ptr1->s_name);
    }
    ptr2=ptr1;
    ptr1=ptr1->s_next;
  }

  s_start=ptr1;
  --blknum;
  if (ptr1==ptr2) {
    memo=NULL;
  } else {
    ptr2->s_next=NULL;
  }

  return memo;
}

/************************************************************************/
/* Verleiht neuen Eintraegen in der Symboltabelle diverse Eigenschaften */
/* (Aufruf durch den Parser in der 'decl'-Regel)                        */
/************************************************************************/

void give_typ(int type, SetRange * set, int init_or_cap)
{
  SymTabEntry *ptr=s_start;

  while (ptr->s_type==_UDEC) {
    ptr->s_set = set;
    switch(ptr->s_type=type) {
      
      case _VAR:
		if ( init_or_cap != NONE && ! is_elem( set, init_or_cap ) )
		{
		  init_or_cap = NONE;
		  Msg(ERR,"invalid initialization");
		}
      case MNET_CONST:
		ptr->s_init_or_cap=init_or_cap;
	  break;

      case _CHANNEL:
      case _STACK:
		if (init_or_cap<0) {
		  Msg(ERR,"Capacity must be positive !");
		}
		if (init_or_cap>MAX_POS) {
		  Msg(ERR,"Capacity must be a number !");
		}
		ptr->s_init_or_cap=init_or_cap;
	  break;

      case _PROC:
		ptr->s_init_or_cap=init_or_cap;
	  break;

      case _PPROC:
	  break;
	
      case _BLOCKNAME:
	  break;

      default:
		Msg(FAT,"Internal error: Unknown type in give_typ !");
	}
	ptr=ptr->s_next;
  }
}
