/******************************************************************************/
/* FILE tree.c                                                                */
/* Burkhard Graves, 10.10.93                                                  */
/* Lars Jenner, Juni 1994                                                     */
/* Stephan Merkel, Mar. 95                                                    */
/* Martin Ackermann, Winter 96/97                                             */
/******************************************************************************/

#include "frontend.h"
#include "symtab.h"
#include "tree.h"
#include "set.h"

/******************************************************************************/
/* Datenstrukturen                                                            */
/******************************************************************************/

SymTabEntry* ActPar=NULL;

#ifdef PRINT_TREE
FILE* ofp=stdout;

#ifdef EXPR_AS_TREE
  int  tab_length=1;
  char ts[80]="\n";
#else
  char ts[]="";
#endif


/******************************************************************************/
/* Funktionen                                                                 */
/******************************************************************************/

static void PrintCom    PROTO ((TreeNode *node));
static void PrintAltSet PROTO ((TreeNode *node));


/*****************************/
/* Ausgabe der Deklarationen */
/*****************************/

static void PrintDecls(char where, SymTabEntry *ptr)
{
  while (ptr) {
    if (where==_PPROC || ptr->s_partype==_NOPAR)
	switch(ptr->s_type) 
	{

	case _UDEC:
	case MNET_CONST:
        case _BLOCKNAME:
	break;

	case _VAR:
		  fprintf(ofp,"VAR %s: ",ptr->s_name);
		  write_set( ofp, ptr->s_set );
		  if (ptr->s_init_or_cap!=NONE) 
		    {
                      fprintf(ofp," INIT "); fprintset(ofp,ptr->s_init_or_cap);
		    } /* if */
	          if (where!=_PPROC)
		    fprintf(ofp,";\n");
	          else
		    fprintf(ofp,",");
	    break;

	case _CHANNEL:
	case _STACK:
		  fprintf(ofp,"VAR %s: %s ",ptr->s_name,
			  (ptr->s_type==_STACK)?"STACK":"CHAN");
		  fprintset(ofp,ptr->s_init_or_cap);
	          fprintf(ofp," of ");
		  write_set( ofp, ptr->s_set );
	          if (where!=_PPROC)
		    fprintf(ofp,";\n");
	          else
		    fprintf(ofp,",");
	    break;

	case _PROC:
	case _PPROC:
	          fprintf(ofp,"PROC %s",ptr->s_name);
	          if (ptr->s_type==_PPROC) 
		    {
		      fputc( '(', ofp );
		      PrintDecls(_PPROC,ptr->s_parlist);
		      fputc( ')', ofp );
		    }
	          PrintBlock((TreeNode*)ptr->s_body);
	  break;

	default:
		  Msg(FAT,"Internal Error: Unknown declaration type !");
	}
	switch (where)
	{
	  case _BLOCK: ptr=ptr->s_next;    break;
	  case _PPROC: ptr=ptr->s_parlist; break;
	  default:     Msg(FAT,"Internal Error: Wrong value for PrintDecls !");
	}
  }
}

/****************************************/
/* Abarbeitung eines expr-Syntaxknotens */
/****************************************/

#define print( op ) {                                   \
	              PrintExpr( node->op1 );           \
	              fprintf( ofp, "%s%s", ts, op );   \
                      PrintExpr( node->op2 );           \
                      break;                            \
                    }

static void PrintExpr(TreeNode *node)
{
#ifdef EXPR_AS_TREE
  ts[tab_length++]=' ';
  ts[tab_length++]=' ';
  ts[tab_length  ]=0;

  switch ( node->type )
    {
      case N_CONST:
      case N_VAR:
      case N_KLAMM:
      case N_NOT:
      case N_SIGN: fprintf( ofp, "%s", ts );
                   break;
    }
#endif

  switch ( node->type )
    {
      case N_CONST  : {
	                fprintset( ofp, (int)node->op1 );
	                break;
                      }
      case N_VAR    : { if ( (int)node->op2 == _PREPRIMED )
	                  fputc( '\'', ofp );
                        fprintf( ofp, ((SymTabEntry *)node->op1)->s_name );
                        switch ( (int)node->op2 )
	                {
		          case  _POSTPRIMED: fputc( '\'', ofp  ); break;
			  case        _READ: fputc( '?' , ofp  ); break;
			  case       _WRITE: fputc( '!' , ofp  ); break;
			  case       _EMPTY: fprintf( ofp, "??"  ); break;
			  case        _FULL: fprintf( ofp, "!!"  ); break;
                          default:
		        }
                        break;
                      }
      case N_KLAMM  : {
	                fprintf( ofp, "()" );
                        PrintExpr( node->op1 );
                        break;
                      }
      case N_OR     : print( "|" );
      case N_AND    : print( "&" );
      case N_NOT    : {
	                fputc( '!', ofp );
                        PrintExpr( node->op1 );
                        break;
                      }
      case N_EQ     : print( "="  );
      case N_UNEQ   : print( "#"  );
      case N_LT     : print( "<"  );
      case N_GT     : print( ">"  );
      case N_LE     : print( "<=" );
      case N_GE     : print( ">=" );
      case N_PLUS   : print( "+"  );
      case N_MIN    : print( "-"  );
      case N_MULT   : print( "*"  );
      case N_DIV    : print( "/"  );
      case N_MOD    : print( "%" );
      case N_SIGN   : {
	                fputc( '-', ofp );
                        PrintExpr( node->op1 );
                        break;
                      }
      default: Msg(FAT, "Internal Error: Unknown node type in expr !" );
    }

#ifdef EXPR_AS_TREE
  tab_length-=2;
  ts[tab_length]=0;
#endif
}


/*******************************************/
/* Abarbeitung eines alt_set-Syntaxknotens */
/*******************************************/

static void PrintAltSet(TreeNode *node)
{
  switch(node->type) {

  case _REPEAT:
    PrintCom(node->op1);
    fprintf( ofp, ";REPEAT\n");
    break;

  case _EXIT:
    PrintCom(node->op1);
    fprintf( ofp, ";EXIT\n");
    break;

  case _CHOICE:
    PrintAltSet(node->op1);
    fprintf( ofp, "[] " );
    PrintAltSet(node->op2);
    break;

  default:
    Msg(FAT,"Internal Error: Unknown node type in alt_set !");
  }
}


/************************************************************/
/* Abarbeiten eines Syntaxknotens fuer einen Prozeduraufruf */
/************************************************************/

static void PrintPCall ( SymTabEntry *proc, ArgListEntry * arglist )
{
  ArgListEntry *arg;

  fprintf( ofp, "%s(", proc->s_name );

  arg = arglist;
  while (arg) 
  {
    if (arg!=arglist)
      fputc( ',', ofp );
    if (arg->ident) 
      fprintf( ofp, arg->ident->s_name );
    if (arg->arg)
      PrintExpr( arg->arg );
    arg = arg->next;
  }

  fputc( ')', ofp );
}


/***************************************/
/* Abarbeitung eines com-Syntaxknotens */
/***************************************/

static void PrintCom(TreeNode *node)
{
  switch(node->type) {

    case _BLOCK:
      PrintBlock(node);
    break;

    case _EXPR:
      fputc('<',ofp);
      PrintExpr(node->op1);
      fputc('>',ofp);
    break;

    case _SEQ:
      PrintCom(node->op1);
      fprintf(ofp," ;\n");
      PrintCom(node->op2);
    break;

    case _PAR:
      PrintCom(node->op1);
      fprintf(ofp," ||\n");
      PrintCom(node->op2);
    break;

    case _DO:
      fprintf( ofp, "DO " );
      if (node->op2) {
	PrintCom(node->op2);
	fprintf( ofp, " ENTER\n" );
      }
      PrintAltSet(node->op1);
      fprintf( ofp, "OD\n" );
    break;

    case _CALL:
      fprintf(ofp,"%s",((SymTabEntry*)(node->op1))->s_name);
    break;
      
    case _PCALL:
      PrintPCall((SymTabEntry*)(node->op1),(ArgListEntry*)(node->op2));
    break;

    default:
      Msg(FAT,"Internal Error: Unknown node type in com !");
  }
}

/*****************************************/
/* Abarbeitung eines block-Syntaxknotens */
/*****************************************/

void PrintBlock(TreeNode *node)
{
  fprintf(ofp,"\nBEGIN\n");

  node->op1=(TreeNode *)invert_list((SymTabEntry *)(node->op1));
  PrintDecls(_BLOCK,(SymTabEntry *)node->op1);

  if( node->op2 ) /* keine com vorhanden, falls nur decls */
    PrintCom( node->op2 );

  fprintf(ofp,"\nEND\n");
}
#endif /* PRINT_TREE */

/**************************************************************************/
/* Sortierung der (lokalen) Symboltabelle:                                */
/* - falls es einen Blocknamen gibt, soll er ganz vorne stehen            */
/* - danach sollen alle Eintraege fuer Kanaele mit Kapazitaet 0 folgen    */
/* - schliesslich folgen alle uebrigen Bezeichner in inverser Reihenfolge */
/**************************************************************************/

SymTabEntry *invert_list(SymTabEntry *ptr1)
{
	SymTabEntry *ptr2,*ptr3;

	ptr3=NULL;
	while (ptr1) {
		ptr1=(ptr2=ptr1)->s_next;
		ptr2->s_next=ptr3;
		ptr3=ptr2;
	}
	return ptr3;
}

/***************************************************************************/
/* LinkBody : Verknuepft den Bezeicher ProcIdent in der Symboltabelle mit  */
/*   dem Prozedurrumpf Body. Ausserdem wird s_special zurueckgesetzt, fuer */
/*   fuer den Fall, dass es einen rekursiven Aufruf im Rumpf gab, der aber */
/*   nicht als Verwendung zaehlt. (siehe chk_ident)                        */
/***************************************************************************/

void  LinkBody(SymTabEntry *ProcIdent,TreeNode *Body)
{
  ProcIdent->s_body    = Body;
  ProcIdent->s_special = 0;
}
/***************************************************************************/
/* LinkParlist: Verknuepft den Bezeicher ProcIdent in der Symboltabelle mit*/
/*   der erzeugten Parameterliste                                          */
/***************************************************************************/

void  LinkParlist(SymTabEntry *ProcIdent,SymTabEntry *ParList)
{
  ProcIdent->s_parlist = ParList;
}

/***************************************************************************/
/* GenParList : Haengt an die bisherige ParList den Parameter Par an, und  */
/*   gibt ihm den jeweiligen Parametertyp (VAL,RES,REF).                   */
/***************************************************************************/

SymTabEntry 
  *GenParList(SymTabEntry *ParList, int Type, SymTabEntry *Par)
{
  SymTabEntry *ParPtr;

  if ( Par == NULL )
    {
      return ( NULL );
    } /* if */

  Par->s_parlist = NULL;
  Par->s_partype = Type;

  ParPtr = ParList;
  if ( (!ParPtr) || 
       ParPtr == Par /* kann nach multiple declaration der Fall sein */ )
    return Par;
  else
  {
    while (ParPtr->s_parlist) 
      ParPtr = ParPtr->s_parlist;
    ParPtr->s_parlist = Par;
    return ParList;
  }
}

/***************************************************************************/
/* GenArgList : Hangt an die bisherige ArgList das Argument Arg an, wofur  */
/*   ein neues Element NewArg geschaffen wird. Gibt NewArg den vom Para-   */
/*   meter geforderten Typ (_RES,_REF,_VAL,_CONSTPAR) und pruft die        */
/*   Vertraglichkeit mit dem tatsachlich ubergebenen Ausdruck (_IDENT oder */
/*   _VAL). value und result werden berechnet, wofuer (Rekursion!) ein     */
/*   temporares Symbol DupPar in der Symboltabelle erzeugt wird.           */
/***************************************************************************/

#define Ident ((SymTabEntry*)arg)
#define Arg   ((TreeNode*   )arg)
ArgListEntry *GenArgList( ArgListEntry *ArgList, int ArgType, void *arg )
{
  ArgListEntry *NewArg,*ArgPtr;

  if ( ActPar == NULL )
  {
    Msg(ERR,"too many arguments");
    return ( NULL );
  } /* if */

  if ((NewArg=(ArgListEntry*)MYmalloc(sizeof(ArgListEntry)))==NULL)  
    Msg(FAT,"No more memory available for argument list !");


  if (ArgType == _IDENT)
  {
    NewArg->ident = Ident;
    NewArg->arg = NULL;
  }
  else if (ArgType == _VAL)
  {
    NewArg->arg   = Arg;
    NewArg->ident = NULL;
  }

  if ((int) ActPar > 1)
  {
    NewArg->partype = ActPar->s_partype;

		/* Ident kann auch ActPar sein ! bei Rekursion ! */

  switch (ArgType) 
  {
    case _IDENT:
      switch (Ident->s_type)
      {
      case _UDEC:
	  if ( !GL_Frags )
	    Msg(ERR,"undeclared identifier %s",Ident->s_name);
	break;
	
      case _VAR:
	switch (NewArg->partype)
	  {
	  case _RES:
	    if ( ! sub_set( ActPar->s_set, Ident->s_set ) )
	      Msg(WAR,"argument %s must have superset as range", Ident->s_name);
	    /* ActPar=Ident' */
	    /* !!! hier kein break !!! */
          case _CONSTPAR:
	  case _VAL:
	    if ( ! sub_set( Ident->s_set, ActPar->s_set ) )
	      Msg(WAR,"argument %s must have subset as range",Ident->s_name);
	    Ident->s_special |= _G_PRIMED;
	    /* Ident=ActPar' */
	    break;
	    
	  case _REF:
	    if (ActPar->s_type!=_VAR && ! GL_Frags)
	      Msg(ERR,"argument %s has incompatible type",Ident->s_name);
	    if ( ! sub_set( ActPar->s_set, Ident->s_set ) )
	      Msg(WAR,"argument %s must have superset as range",Ident->s_name);
	    Ident->s_special |= _G_PRUPR;
	    break;
	    
	  default:
	    Msg(FAT,"Internal Error: Unknown parameter type at VAR !");
	  }
	break;
	
      case _CHANNEL:
      case _STACK:
	switch (NewArg->partype)
	  {
	  case _RES :
          case _CONSTPAR :
	  case _VAL :
	    Msg(ERR,"argument %s has incompatible type",Ident->s_name);
	    break;

	  case _REF :
	    if (ActPar->s_type!=Ident->s_type && ! GL_Frags)
	      Msg(ERR,"argument %s has incompatible type",Ident->s_name);
	    if ( ! sub_set( ActPar->s_set, Ident->s_set ) )
	      Msg(WAR,"argument %s must have superset as range",Ident->s_name);
	    if ( ActPar->s_init_or_cap != Ident->s_init_or_cap )
	      Msg(ERR,"argument %s has incompatible capacity",Ident->s_name);
	    Ident->s_special |= _G_RW;
	    break;

	  default:
	    Msg(FAT,"Internal Error: Unknown parameter type at CHAN/STACK !");
	    
	  }                                                        
	break;

      case _PROC:
      case _PPROC:
	Msg(ERR,"incorrect use of procedure %s as argument",Ident->s_name);  
	break;
	
      default:
	Msg(FAT,"Internal Error: Unknown identifier type !");
      }
      break;
      
  case _VAL:
    
    if (ActPar->s_partype==_RES)
      Msg(ERR,"pass by result expected for parameter %s",ActPar->s_name);
    if (ActPar->s_partype==_REF)
      Msg(ERR,"pass by reference expected for parameter %s",ActPar->s_name);
    /* Expr = ActPar' */
    break;
      
  default:
    Msg(FAT,"Internal Error: Unknown argument type !");

  }

   ActPar = ActPar->s_parlist;
  }
  NewArg->next = NULL;

  ArgPtr = ArgList;
  if (!ArgPtr)
  {
    return NewArg;
  }
  else
  {
    while (ArgPtr->next) 
      ArgPtr = ArgPtr->next;
    ArgPtr->next = NewArg;
    return ArgList;
  }
}
#undef Ident
#undef Arg

/***************************************************************************/
/* check_parm_list : Nachtraegliche Ueberpruefung der Korrektheit von	   */
/*   Argumenten, falls eine Vorwaertsreferenz innerhalb von procgrp	   */
/*   auftrat. Arglist ist sind die tatsaechlichen Parameter im Prozedur-   */
/*   Aufruf, ActPar die bei der Prozedur-Deklaration verwendeten.	   */
/***************************************************************************/

#define Ident (ArgList->ident)
#define Arg   (ArgList->arg)
void check_parm_list( ArgListEntry *ArgList, SymTabEntry *ActPar)
{
  if (!ArgList && !ActPar) return;
  if (!ArgList) { Msg(ERR,"too few arguments"); return; }
  if (!ActPar) { Msg(ERR,"too many arguments"); return; }

  ArgList->partype = ActPar->s_partype;

  if (Ident) /* case _IDENT: */
  {
      switch (Ident->s_type)
      {
      case _UDEC:
	  if ( !GL_Frags )
	    Msg(ERR,"undeclared identifier %s",Ident->s_name);
	break;
	
      case _VAR:
	switch (ArgList->partype)
	  {
	  case _RES:
	    if ( ! sub_set( ActPar->s_set, Ident->s_set ) )
	      Msg(WAR,"argument %s must have superset as range", Ident->s_name);
	    /* ActPar=Ident' */
	    /* !!! hier kein break !!! */
          case _CONSTPAR:
	  case _VAL:
	    if ( ! sub_set( Ident->s_set, ActPar->s_set ) )
	      Msg(WAR,"argument %s must have subset as range",Ident->s_name);
	    Ident->s_special |= _G_PRIMED;
	    /* Ident=ActPar' */
	    break;
	    
	  case _REF:
	    if (ActPar->s_type!=_VAR && ! GL_Frags)
	      Msg(ERR,"argument %s has incompatible type",Ident->s_name);
	    if ( ! sub_set( ActPar->s_set, Ident->s_set ) )
	      Msg(WAR,"argument %s must have superset as range",Ident->s_name);
	    Ident->s_special |= _G_PRUPR;
	    break;
	    
	  default:
	    Msg(FAT,"Internal Error: Unknown parameter type at VAR !");
	  }
	break;
	
      case _CHANNEL:
      case _STACK:
	switch (ArgList->partype)
	  {
	  case _RES :
          case _CONSTPAR :
	  case _VAL :
	    Msg(ERR,"argument %s has incompatible type",Ident->s_name);
	    break;

	  case _REF :
	    if (ActPar->s_type!=Ident->s_type && ! GL_Frags)
	      Msg(ERR,"argument %s has incompatible type",Ident->s_name);
	    if ( ! sub_set( ActPar->s_set, Ident->s_set ) )
	      Msg(WAR,"argument %s must have superset as range",Ident->s_name);
	    if ( ActPar->s_init_or_cap != Ident->s_init_or_cap )
	      Msg(ERR,"argument %s has incompatible capacity",Ident->s_name);
	    Ident->s_special |= _G_RW;
	    break;

	  default:
	    Msg(FAT,"Internal Error: Unknown parameter type at CHAN/STACK !");
	    
	  }                                                        
	break;

      case _PROC:
      case _PPROC:
	Msg(ERR,"incorrect use of procedure %s as argument",Ident->s_name);  
	break;
	
      default:
	Msg(FAT,"Internal Error: Unknown identifier type !");
      }
  }
  else if (Arg)	/* case _VAL: */
  {
    if (ActPar->s_partype==_RES)
      Msg(ERR,"pass by result expected for parameter %s",ActPar->s_name);
    if (ActPar->s_partype==_REF)
      Msg(ERR,"pass by reference expected for parameter %s",ActPar->s_name);
    /* Expr = ActPar' */
  }
  else  /* default: */
  {
    Msg(FAT,"Internal Error: Unknown argument type !");

  }

  check_parm_list(ArgList->next,ActPar->s_parlist);
}
#undef Ident
#undef Arg

/****************************************/
/* Loeschen der (lokalen) Symboltabelle */
/****************************************/

static void delete_list( char what, void *ptr1)
{
  void     *ptr2;
  SetRange *last_set=NULL;
  
  while (ptr1) {
    ptr2=ptr1;
    switch (what)        
      {
      case _BLOCK:
	ptr1=((SymTabEntry*)ptr2)->s_next;
	MYfree(((SymTabEntry*)ptr2)->s_name);
	if ( ((SymTabEntry*)ptr2)->s_set != last_set )
	  /* bei VAR x,y,z: {...} muss der Wertebereich nur fuer x 
	     geloescht werden, weil y,z lediglich Zeigerkopien haben */
	  delete_set(last_set=((SymTabEntry*)ptr2)->s_set);
	if ( ((SymTabEntry*)ptr2)->s_type ==  _PROC ||
	     ((SymTabEntry*)ptr2)->s_type == _PPROC )
	  delete_tree( ((SymTabEntry*)ptr2)->s_body );
	break;
	
      case _PCALL:
	ptr1=((ArgListEntry*)ptr2)->next;
	if (((ArgListEntry*)ptr2)->arg)
	  delete_tree(((ArgListEntry*)ptr2)->arg);
	break;
	
      default: Msg(FAT,"Internal Error: Unknown list type !");
      }
    MYfree(ptr2);
  }
}

/**************************************/
/* Erzeugung eines Syntaxbaum-Knotens */
/**************************************/

TreeNode *GenNode(int type,TreeNode *op1,TreeNode *op2)
{
  TreeNode *node;
  
  node=(TreeNode *)MYmalloc(sizeof(TreeNode)+
			    ( (type==_BLOCK || type==_CALL || type==_PCALL )
			      ? sizeof(ShortEntryPtr):0 ) );
  node->type=type;
  node->op1=op1;
  node->op2=op2;
  return node;
}

/*****************************/
/* Loeschen des Syntaxbaumes */
/*****************************/

void delete_tree(TreeNode *node)
{
  if ( node )
    {
	switch(node->type) {
	case _BLOCK:
	        delete_list(_BLOCK,node->op1);
		delete_tree(node->op2);
		break;
	case _DO:
                if (node->op2) delete_tree(node->op2);
	case _REPEAT:
	case _EXIT:
	case _EXPR:
	case N_NOT:
        case N_KLAMM:
        case N_SIGN:
	        delete_tree(node->op1);
		break;
	case _CALL:
        case N_CONST:
        case N_VAR:
		break;
	case _PCALL:
	        delete_list(_PCALL,node->op2);
		break;
	case _SEQ:
	case _PAR:
	case _CHOICE:
        case N_AND:
        case N_OR:
        case N_EQ:
        case N_UNEQ:
        case N_GT:
        case N_LT:
        case N_GE:
        case N_LE:
        case N_PLUS:
        case N_MIN:
        case N_MULT:
        case N_DIV:
        case N_MOD:
		delete_tree(node->op1);
		delete_tree(node->op2);
		break;
	default:
		Msg(FAT,"Internal Error: Unknown node type in delete_tree !");
	} /* switch */
	MYfree(node);
    } /* if */
}

/******************************************************************************/
/* END OF FILE tree.c                                                         */
/******************************************************************************/
