/******************************************************************************/
/* FILE tree.h                                                                */
/* Burkhard Graves, 10.10.93                                                  */
/* Lars Jenner, Juni 1994                                                     */
/* Stephan Merkel, Mar. 95                                                    */
/* Martin Ackermann, Winter 96/97                                             */
/******************************************************************************/

#ifndef __TREE_H__
#define __TREE_H__

#include "symtab.h"

/******************************************************************************/
/* Globale Defines                                                            */
/******************************************************************************/

/* Knotentypen fuer Parse-Baeume */
#define N_HEAD  0

#define _BLOCK	1
#define _EXPR	2
#define _SEQ	3
#define _PAR	4
#define _DO	5
#define _REPEAT	6
#define _EXIT	7
#define _CHOICE	8
#define _CALL   9
#define _PCALL  10

#ifndef N_AND
  #define N_AND   101
  #define N_OR    102
  #define N_NOT   103
  #define N_CONST 104
  #define N_VAR   105
  #define N_KLAMM 106
  #define N_EQ    107
  #define N_UNEQ  108
  #define N_LT    109
  #define N_GT    110
  #define N_LE    111
  #define N_GE    112
  #define N_PLUS  113
  #define N_MIN   114
  #define N_MULT  115
  #define N_DIV   116
  #define N_MOD   117
  #define N_REM   118
  #define N_SIGN  119
#endif

#define N_MACRO 29

/* Arten des Identifier-Gebrauchs */
#define _UNPRIMED   1
#define _PREPRIMED  2
#define _POSTPRIMED 3
#define _READ       4
#define _WRITE      5
#define _EMPTY      6
#define _FULL       7

/* Typen fuer Parameter-Listen & Argumente-Listen oder Symboltabelle */
#define _VAL      1 /* fuer Parameter, Argumente          */
#define _CONSTPAR 2 /* fuer Parameter, Argumente          */
#define _RES      3 /* fuer Parameter, Argumente          */
#define _REF      4 /* fuer Parameter, Argumente, Symbole */
#define _NOPAR    5 /* fuer                       Symbole */

/* einzelne Argumente sind entweder _VAL oder _IDENT */
#define _IDENT    6


/* Basisdefinitionen (s.u., werden einzeln im Programmtext nicht verwendet */

#define _L_READING   1
#define _L_PRIMED    1
#define _L_WRITING   2
#define _L_UNPRIMED  2
#define _L_EMPTY     4
#define _L_FULL      8

#define _G_READING  16
#define _G_PRIMED   16
#define _G_WRITING  32
#define _G_UNPRIMED 32
#define _G_EMPTY    64
#define _G_FULL    128


/* Fuer s_special (bitweise Nummerierung) */

#define _LG_READING  (_L_READING  | _G_READING)
#define _LG_WRITING  (_L_WRITING  | _G_WRITING)
#define _LG_EMPTY    (_L_EMPTY    | _G_EMPTY)
#define _LG_FULL     (_L_FULL     | _G_FULL)
#define _LG_PRIMED   (_L_PRIMED   | _G_PRIMED)
#define _LG_UNPRIMED (_L_UNPRIMED | _G_UNPRIMED)

#define _G_RW    (_G_READING | _G_WRITING  | _G_EMPTY | _G_FULL)
#define _G_PRUPR (_G_PRIMED  | _G_UNPRIMED)

/******************************************************************************/
/* Globale Typdefinitionen                                                    */
/******************************************************************************/

typedef struct treenode {
    int type;
    struct treenode *op1,*op2;
} TreeNode;

typedef struct arglistentry {
    char                 partype;
    SymTabEntry         *ident;
    TreeNode            *arg;
    struct arglistentry *next;
} ArgListEntry;

/******************************************************************************/
/* Globale Datenstrukturen                                                    */
/******************************************************************************/

extern SymTabEntry* ActPar;


/******************************************************************************/
/* Globale Funktionen                                                         */
/******************************************************************************/

void   PrintBlock PROTO (( TreeNode *node ));
TreeNode *GenNode PROTO (( int type,TreeNode *op1,TreeNode *op2 ));
void  delete_tree PROTO (( TreeNode *node ));


void     LinkBody PROTO (( SymTabEntry *ProcIdent,TreeNode *Body ));
void  LinkParlist PROTO (( SymTabEntry *ProcIdent,SymTabEntry *Parlist ));

ArgListEntry *GenArgList PROTO (( ArgListEntry *ArgList, int ArgType, \
                                  void *arg ));

void check_parm_list PROTO (( ArgListEntry *ArgList, SymTabEntry *ActPar));

SymTabEntry  *GenParList PROTO (( SymTabEntry *ParList, int Type, \
				  SymTabEntry *Par ));
SymTabEntry *invert_list PROTO (( SymTabEntry *ptr1 ));


#endif /* __TREE_H__ */

/******************************************************************************/
/* END OF FILE tree.h                                                         */
/******************************************************************************/
