/****************************************************************************/
/* calc.c                                                                   */
/****************************************************************************/

#include "calc.h"
#include "unfoldHL.h"
#include "expr.h"

extern int yyerror (char *msg);
/****************************************************************************/

bool div_by_zero;

/****************************************************************************/

Integer calc(btree node)
{
	Integer v1 = V_DOT, v2;

	if (!node) return v1;

	switch (node->token)
	{
		case NUMBER: 
			v1 = (Integer) (int) node->left;
			break;
		case IDENT: 
			v1 = MT_Place(currvalue[(Cardinal32)node->left],0);
			break;
		case PROJ:
			v1 = MT_Place(currvalue[(Cardinal32) node->left],
				(Cardinal32) node->right);
			break;
		case DOTDOT:
			/* NOCH NICHT IMPLEMENTIERT: GEBUNDENE VARIABLE */
			v2 = calc(node->right);
			v1 = 0;
			break;
		case NOT:
			v1 = calc(node->left);
			if (v1 > V_TRUE) yyerror("Boolean value expected");
			v1 = (v1 == V_TRUE)? V_FALSE : V_TRUE;
			break;
		case AND:
			if ((v1 = calc(node->left)) == V_TRUE)
			{
				if ((v1 = calc(node->right)) > V_TRUE)
					yyerror("Boolean value expected");
			}
			else if (v1 != V_FALSE)
				yyerror("Boolean value expected");
			break;
		case OR:
			if ((v1 = calc(node->left)) == V_FALSE)
			{
				if ((v1 = calc(node->right)) > V_TRUE)
					yyerror("Boolean value expected");
			}
			else if (v1 != V_TRUE)
				yyerror("Boolean value expected");
			break;
		case '=':
			v1 = calc(node->left);
			v2 = calc(node->right);
			v1 = (v1 == v2)? V_TRUE : V_FALSE;
			break;
		case '#':
			v1 = calc(node->left);
			v2 = calc(node->right);
			v1 = (v1 != v2)? V_TRUE : V_FALSE;
			break;
		case '<':
			v1 = calc(node->left);
			v2 = calc(node->right);
			if ((v1 <= V_DOT) || (v2 <= V_DOT))
				v1 = V_FALSE;
			else
				v1 = (v1 < v2)? V_TRUE : V_FALSE;
			break;
		case '>':
			v1 = calc(node->left);
			v2 = calc(node->right);
			if ((v1 <= V_DOT) || (v2 <= V_DOT))
				v1 = V_FALSE;
			else
				v1 = (v1 > v2)? V_TRUE : V_FALSE;
			break;
		case LE:
			v1 = calc(node->left);
			v2 = calc(node->right);
			if ((v1 <= V_DOT) || (v2 <= V_DOT))
				v1 = (v1 == v2)? V_TRUE : V_FALSE;
			else
				v1 = (v1 <= v2)? V_TRUE : V_FALSE;
			break;
		case GE:
			v1 = calc(node->left);
			v2 = calc(node->right);
			if ((v1 <= V_DOT) || (v2 <= V_DOT))
				v1 = (v1 == v2)? V_TRUE : V_FALSE;
			else
				v1 = (v1 >= v2)? V_TRUE : V_FALSE;
			break;
		case '+':
			v1 = calc(node->left);
			v2 = calc(node->right);
			if ((v1 <= V_DOT) || (v2 <= V_DOT))
				yyerror("Numerical value expected");
			v1 = v1+v2;
			break;
		case '-':
			v1 = calc(node->left);
			v2 = calc(node->right);
			if ((v1 <= V_DOT) || (v2 <= V_DOT))
				yyerror("Numerical value expected");
			v1 = v1-v2;
			break;
		case '*':
			v1 = calc(node->left);
			v2 = calc(node->right);
			if ((v1 <= V_DOT) || (v2 <= V_DOT))
				yyerror("Numerical value expected");
			v1 = v1*v2;
			break;
		case '/':
			v1 = calc(node->left);
			v2 = calc(node->right);
			if ((v1 <= V_DOT) || (v2 <= V_DOT))
				yyerror("Numerical value expected");
			if (!v2) div_by_zero = True;
			v1 = v1/v2;
			break;
		case '%':
			v1 = calc(node->left);
			v2 = calc(node->right);
			if ((v1 <= V_DOT) || (v2 <= V_DOT))
				yyerror("Numerical value expected");
			if (!v2) div_by_zero = True;
			v1 = v1%v2;
			break;
		case UNARYMINUS:
			v1 = calc(node->left);
			if (v1 <= V_DOT)
				yyerror("Numerical value expected");
			v1 = -v1;
			break;
		default:
			break;
	} /* switch */

	return (v1);

} /* calc() */


/****************************************************************************/
/* END OF FILE calc.c                                                       */
/****************************************************************************/
