%{

#include <string.h>
#include <stdio.h>

#include "multiset.h"
#include "unfoldHL.h"
#include "parser.h"

#undef alloca
#define alloca MYmalloc

int yyerror(char *msg)
{
	fprintf(stderr, "transition id %u: %s\n", GLO_line, msg);
	return 1;
}

%}

%union
{
	char	string[129];
	int	wert;
	btree	bnode;
	Action	var;
	Integer	*eset;
}


%token DOTTOKEN
%token NOT
%token AND
%token OR
%token TRUE
%token FALSE
%token <wert> NUMBER
%token <string> IDENT
%token GE
%token LE
%token DOTDOT
%token PROJ
%token CROSS


%left	  OR
%left	  AND
%nonassoc NOT
%nonassoc '=' '#' '<' '>' GE LE
%left	  '+' '-'
%left	  '*' '/' '%'
%nonassoc UNARYMINUS '\''


%type <wert>	const
%type <bnode>	start normexpr expr basic compare bound mset
%type <var>	ident simple
%type <eset>	set elements

%start start

%%

start	: expr
	  {
		STree = $$ = $1;
	  }
	| bound ':' expr
	  {
		STree = $$ = new_node(DOTDOT, $1, $3);
	  }
	;

elements: const
	  {
		V_insert_value(&V_Dummy, $1);
	  }
	| const DOTDOT const 
	  {
		V_insert_interval(&V_Dummy, $1, $3);
	  }
	| '[' const ',' const  ']'
	  {
		V_insert_interval(&V_Dummy, $2, $4);
	  }
	| const DOTDOT const ',' elements
	  {
		V_insert_interval(&V_Dummy, $1, $3);
	  }
	| '[' const ',' const ']' ',' elements
	  {
		V_insert_interval(&V_Dummy, $2, $4);
	  }
	| const ',' elements 
	  {
		V_insert_value(&V_Dummy, $1);
	  }
	;

set	: '{'
	  {
		*V_Dummy = 0;
	  }
	  elements '}'
	  {
		$$ = V_create_copy(V_Dummy);
	  }
	;

mset	: set
	  {
		$$ = new_node(CROSS, (btree) $1, NULL);
	  }
	| set CROSS mset
	  {
		$$ = new_node(CROSS, (btree) $1, $3);
	  }
	;

bound	: simple mset
	  {
		MSset_single(mmSingle, $1, 1);
		MSsum(boundset, mmSingle);
		$$ = new_node(DOTDOT, NULL,
				new_node(DOTTOKEN, (btree)(Cardinal32) $1, $2));
	  }
	| bound AND simple mset
	  {
		MSset_single(mmSingle, $3, 1);
		MSsum(boundset, mmSingle);
		$$ = new_node(DOTDOT, $1,
				new_node(DOTTOKEN, (btree)(Cardinal32) $3, $4));
	  }
	;

ident	: IDENT
	  {
		$$ = StrToAction($1);
	  }
	;

basic	: '(' basic ')'
	  {
		$$ = $2;
	  }
	| const
	  {
		$$ = new_node(NUMBER, (btree) $1, NULL);
	  }
	| simple
	  {
		MSset_single(mmSingle, $1, 1);
		MSsum(annovarset, mmSingle);
		$$ = new_node(IDENT, (btree)(Cardinal32) $1, NULL);
	  }
	| PROJ NUMBER '(' simple ')'
	  {
		MSset_single(mmSingle, $4, 1);
		MSsum(annovarset, mmSingle);
		$$ = new_node(PROJ, (btree)(int) $4, (btree)(int) $2);
	  }
	| basic '+' basic
	  {
		$$ = new_node('+', $1, $3);
	  }
	| basic '-' basic
	  {
		$$ = new_node('-', $1, $3);
	  }
	| basic '*' basic
	  {
		$$ = new_node('*', $1, $3);
	  }
	| basic '/' basic
	  {
		$$ = new_node('/', $1, $3);
	  }
	| basic '%' basic
	  {
		$$ = new_node('%', $1, $3);
	  }
	| '-' basic %prec UNARYMINUS
	  {
		$$ = new_node(UNARYMINUS, $2, NULL);
	  }
	;

compare	: basic '=' normexpr
	  {
		$$ = new_node('=', $1, $3);
	  }
	| normexpr '=' basic
	  {
		$$ = new_node('=', $1, $3);
	  }
	| basic '#' basic
	  {
		$$ = new_node('#', $1, $3);
	  }
	| basic '<' basic
	  {
		$$ = new_node('<', $1, $3);
	  }
	| basic '>' basic
	  {
		$$ = new_node('>', $1, $3);
	  }
	| basic GE basic
	  {
		$$ = new_node(GE, $1, $3);
	  }
	| basic LE basic
	  {
		$$ = new_node(LE, $1, $3);
	  }
	| basic '=' compare
	  {
		$$ = new_node(AND, new_node('=', $1,
			$3->token == AND ? $3->left->left : $3->left), $3);
	  }
	| basic '#' compare
	  {
		$$ = new_node(AND, new_node('#', $1,
			$3->token == AND ? $3->left->left : $3->left), $3);
	  }
	| basic '<' compare
	  {
		$$ = new_node(AND, new_node('<', $1,
			$3->token == AND ? $3->left->left : $3->left), $3);
	  }
	| basic '>' compare
	  {
		$$ = new_node(AND, new_node('>', $1,
			$3->token == AND ? $3->left->left : $3->left), $3);
	  }
	| basic GE compare
	  {
		$$ = new_node(AND, new_node(GE, $1,
			$3->token == AND ? $3->left->left : $3->left), $3);
	  }
	| basic LE compare
	  {
		$$ = new_node(AND, new_node(LE, $1,
			$3->token == AND ? $3->left->left : $3->left), $3);
	  }
	;

normexpr: '(' expr ')'
	  {
		$$ = $2;
	  }
	| basic
	  {
		$$ = $1;
	  }
	| compare
	  {
		$$ = $1;
	  }
	| NOT normexpr 
	  {
		$$ = new_node(NOT, $2, NULL);
	  }
	| '!' normexpr 
	  {
		$$ = new_node(NOT, $2, NULL);
	  }
	;

expr	: normexpr
	  {
		$$ = $1;
	  }
	| expr OR expr
	  {
		$$ = new_node(OR, $1, $3);
	  }
	| expr AND expr
	  {
		$$ = new_node(AND, $1, $3);
	  }
	;

simple	: '\'' ident
	  {
		char str[41];
		*str = '\'';
		ActionToStr($2, str + 1);
		$$ = StrToAction(str);
	  }
	| ident '\''
	  {
		char str[41];
		ActionToStr($1, str);
		strcat(str, "\'");
		$$ = StrToAction(str);
	  }
	| ident '?'
	  {
		char str[41];
		ActionToStr($1, str);
		strcat(str, "?");
		$$ = StrToAction(str);
	  }
	| ident '!' %prec '\''
	  {
		char str[41];
		ActionToStr($1, str);
		strcat(str, "!");
		$$ = StrToAction(str);
	  }
	| ident '?' '?'
	  {
		char str[41];
		ActionToStr($1, str);
		strcat(str, "??");
		$$ = StrToAction(str);
	  }
	| ident '!' '!'
	  {
		char str[41];
		ActionToStr($1, str);
		strcat(str, "!!");
		$$ = StrToAction(str);
	  }
	| ident
	  {
		$$ = $1;
	  }
	;

const	: NUMBER
	  {
		$$ = $1;
	  }
	| TRUE
	  {
		$$ = V_TRUE;
	  }
	| FALSE
	  {
		$$ = V_FALSE;
	  }
	| DOTTOKEN
	  {
	  	$$ = V_DOT;
	  }
	;
%%

/****************************************************************************/
/* END OF FILE expr.y                                                       */
/****************************************************************************/
