/*********************************************************/
/* > mnet.c <                                            */
/* Funktionen zur Manipulation von 'labelled petri nets' */
/* Burkhard Graves, 08.02.'94                            */
/*********************************************************/

#include <stdio.h>

#include "mnet.h"

/*****************************************************************************/
/************************** M_empty_net() ************************************/
/*****************************************************************************/

M_Net M_empty_net(void)
{
	register M_Net net = M_allocHead();

	net->Places = net->LastPlace = NULL;
	net->Transitions = net->LastTransition = NULL;
	net->PhTransitions = net->LastPhTransition = NULL;
    net->Links = NULL;
	net->NumPlaces = net->NumTransitions = net->NumPhTransitions = 0;
    net->NumLinks = 0;
    net->MaxNumLinks = 0;
	net->blockinfo = net->definfo = net->refblockinfo = NULL;
        net->pfadefs = net->pfablockinfo = NULL;
	net->sdldefs = net->sdlblockinfo = NULL;

	return (net);

} /* M_empty_net() */


/*****************************************************************************/
/************************ M_fast_insert_place ********************************/
/*****************************************************************************/

M_Place M_fast_insert_place(M_Net net,
			    Cardinal32 P_numname,
			    M_Place *pl_last)
{
	register M_Place pl,plnew;

	if (!(pl = *pl_last) || (pl->numname > P_numname))
		pl = net->Places;

	while (pl && (pl->numname < P_numname))
		pl = pl->nextplace;

	if (pl && (P_numname == pl->numname))
	{
		fprintf(stderr, "ERROR: HL.M_fast_insert_place(): "
				"place '%u' already exists.\n", P_numname);
		exit (1);
	} /* if */

	plnew = M_allocPlace();
	if (pl)
	{
		if ((plnew->prevplace = pl->prevplace))
			plnew->prevplace->nextplace = plnew;
		else
			net->Places = plnew;
		(plnew->nextplace = pl) -> prevplace = plnew;
	} /* if */
	else
	{
		if ((plnew->prevplace = net->LastPlace))
			net->LastPlace->nextplace = plnew;
		else
			net->Places = plnew;
		(net->LastPlace = plnew) -> nextplace = NULL;
	} /* else */

	plnew->numname = P_numname;
	plnew->preset = NULL;
	plnew->postset = NULL;
	plnew->phpreset = NULL;
	plnew->phpostset = NULL;
	plnew->label = EMPTY;
	plnew->annotation = NULL;
	plnew->meaning = NULL;
	plnew->reference = NULL;
	plnew->blocklist = NULL;
	plnew->bpnref = NULL;
        plnew->pfaref = NULL;
	plnew->sdlref = NULL;
	plnew->init_marking = NULL;
	plnew->curr_marking = NULL;
	plnew->x_pos = plnew->y_pos = plnew->flags = 0;
	net->NumPlaces++;

	return (*pl_last = plnew);

} /* M_fast_insert_place() */


/*****************************************************************************/
/*********************** M_fast_insert_transition ****************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

M_Transition M_fast_insert_transition(  M_Net        net,
					Cardinal32   T_numname,
					M_Transition *tr_last  )
{
	register M_Transition tr,trnew;

	if (!(tr = *tr_last) || (tr->numname > T_numname))
		tr = net->Transitions;

	while (tr && (tr->numname < T_numname))
		tr = tr->nexttrans;

	if (tr && (T_numname == tr->numname))
	{
		fprintf(stderr, "ERROR: HL.M_fast_insert_transition(): "
				"transition '%u' already exists.\n", T_numname);
		exit (1);
	} /* if */

	trnew = M_allocTrans();
	if (tr)
	{
		if ((trnew->prevtrans = tr->prevtrans))
			trnew->prevtrans->nexttrans = trnew;
		else
			net->Transitions = trnew;
		(trnew->nexttrans = tr) -> prevtrans = trnew;
	} /* if */
	else
	{
		if ((trnew->prevtrans = net->LastTransition))
			net->LastTransition->nexttrans = trnew;
		else
			net->Transitions = trnew;
		(net->LastTransition = trnew) -> nexttrans = NULL;
	} /* else */

	trnew->preset = NULL;
	trnew->postset = NULL;
	trnew->numname = T_numname;
	trnew->label = NULL;
	trnew->annotation = NULL;
	trnew->meaning = NULL;
	trnew->links = NULL;
	trnew->vars = NULL;
	trnew->reference = NULL;
	trnew->blocklist = NULL;
	trnew->bpnref = NULL;
        trnew->pfaref = NULL;
	trnew->sdlref = NULL;
	trnew->corrtran = NULL;
	trnew->dead = False;
	trnew->time_label = NULL;
	trnew->x_pos = trnew->y_pos = trnew->flags = 0;
	net->NumTransitions++;

	return (*tr_last = trnew);

} /* M_fast_insert_transition() */


/*****************************************************************************/
/************************* M_fast_insert_TP_edge *****************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

M_TSEdge M_fast_insert_TP_edge( M_Net        net,
				Cardinal32   P_numname,
				Cardinal32   T_numname,
				M_Place      *pl_last,
				M_Transition *tr_last,
				M_TSEdge     *ts_last)
{
	register M_Place      pl;
	register M_Transition tr;
	register M_TSEdge     ts,tsnew;

	if ((ts = *ts_last) && (ts->place->numname == P_numname))
		pl = ts->place;
	else if (!((pl = *pl_last) && (pl->numname == P_numname)))
	{
		for (pl = net->Places ; pl && (pl->numname < P_numname);
					pl = pl->nextplace);
		if (!(pl && (pl->numname == P_numname)))
		{
			WARNING_P_NUM("fast_insert_TP_edge", P_numname);
			return (NULL);
		} /* if */
	} /* else if */

	if (ts && ( ts->trans->numname == T_numname))
		tr = ts->trans;
	else if (! ((tr = *tr_last) && (tr->numname == T_numname)))
	{
		for (tr = net->Transitions; tr && (tr->numname < T_numname);
					    tr = tr->nexttrans);
		if (! (tr && (tr->numname == T_numname)))
		{
			WARNING_T_NUM("fast_insert_TP_edge", T_numname);
			return (NULL);
		} /* if */
	} /* else if */

	if ((ts = pl->preset))
	{
		if (T_numname < ts->trans->numname)
			ts = NULL;
		else if (T_numname == ts->trans->numname)
		{
			/* Inkrementiere Kantengewicht */
			/* In diesen drei Zeilen stand vorher tsnew statt ts.
			   Das schien Probleme zu machen. */
			*pl_last = ts->place = pl;
			*tr_last = ts->trans = tr;
			return (*ts_last = ts);
		} /* if */
		else
		{
			while (ts->nexttrans
				&& (ts->nexttrans->trans->numname < T_numname))
			    ts = ts->nexttrans;

			if ((tsnew = ts->nexttrans)
				&& (tsnew->trans->numname == T_numname))
			{
				/* Inkrementiere Kantengewicht */
				*pl_last = tsnew->place = pl;
				*tr_last = tsnew->trans = tr;
				return (*ts_last = tsnew);
			} /* if */
		} /* else */
	} /* if */

	tsnew = M_allocTSEdge();
	tsnew->label = NULL;
	tsnew->flags = 0;

	if (ts)
	{
		if ((tsnew->nexttrans = ts->nexttrans))
			tsnew->nexttrans->prevtrans = tsnew;
		(tsnew->prevtrans = ts) -> nexttrans = tsnew;
	} /* if */
	else
	{
		if ((tsnew->nexttrans = pl->preset))
			tsnew->nexttrans->prevtrans = tsnew;
		(pl->preset = tsnew) -> prevtrans = NULL;
	} /* else */

	if ((ts = tr->postset))
        {
		if (P_numname < ts->place->numname)
			ts = NULL;
		else
                {
			while (ts->nextplace
				&& (ts->nextplace->place->numname < P_numname))
			    ts = ts->nextplace;
                }
        }

	if (ts)
	{
		if ((tsnew->nextplace = ts->nextplace))
			tsnew->nextplace->prevplace = tsnew;
		(tsnew->prevplace = ts) -> nextplace = tsnew;
	} /* if */
	else
	{
		if ((tsnew->nextplace = tr->postset))
			tsnew->nextplace->prevplace = tsnew;
		(tr->postset = tsnew) -> prevplace = NULL;
	} /* else */

	*pl_last = tsnew->place = pl;
	*tr_last = tsnew->trans = tr;
	return (*ts_last = tsnew);

} /* M_fast_insert_TP_edge() */

/*****************************************************************************/
/************************* M_fast_insert_PT_edge *****************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

M_STEdge M_fast_insert_PT_edge( M_Net        net,
				Cardinal32   P_numname,
				Cardinal32   T_numname,
				M_Place      *pl_last,
				M_Transition *tr_last,
				M_STEdge     *st_last)
{
	register M_Place      pl;
	register M_Transition tr;
	register M_STEdge     st,stnew;

	if ((st = *st_last) && (st->place->numname == P_numname))
		pl = st->place;
	else if (!((pl = *pl_last) && (pl->numname == P_numname)))
	{
		for (pl = net->Places;  pl && (pl->numname < P_numname);
					pl = pl->nextplace);
		if (!(pl && (pl->numname == P_numname)))
		{
			WARNING_P_NUM("fast_insert_PT_edge", P_numname);
			return (NULL);
		} /* if */
	} /* else if */

	if (st && ( st->trans->numname == T_numname))
		tr = st->trans;
	else if (!((tr = *tr_last) && (tr->numname == T_numname)))
	{
		for (tr = net->Transitions; tr && (tr->numname < T_numname);
					    tr = tr->nexttrans);
		if (!(tr && (tr->numname == T_numname)))
		{
			WARNING_T_NUM("fast_insert_PT_edge", T_numname);
			return (NULL);
		} /* if */
	} /* else if */

	if ((st = pl->postset))
	{
		if (T_numname < st->trans->numname)
			st = NULL;
		else if (T_numname == st->trans->numname)
		{
			/* Inkrementiere Kantengewicht */
			/* stnew -> st substituiert (s.o.) */
			*pl_last = st->place = pl;
			*tr_last = st->trans = tr;
			return (*st_last = st);
		} /* if */
		else
		{
			while (st->nexttrans
				&& (st->nexttrans->trans->numname < T_numname))
			    st = st->nexttrans;

			if ((stnew = st->nexttrans)
				&& (stnew->trans->numname == T_numname))
			{
				/* Inkrementiere Kantengewicht */
				*pl_last = stnew->place = pl;
				*tr_last = stnew->trans = tr;
				return (*st_last = stnew);
			} /* if */
		} /* else */
	} /* if */

	stnew = M_allocSTEdge();
	stnew->label = NULL;
	stnew->flags = 0;

	if (st)
	{
		if ((stnew->nexttrans = st->nexttrans))
			stnew->nexttrans->prevtrans = stnew;
		(stnew->prevtrans = st) -> nexttrans = stnew;
	} /* if */
	else
	{
		if ((stnew->nexttrans = pl->postset))
			stnew->nexttrans->prevtrans = stnew;
		(pl->postset = stnew) -> prevtrans = NULL;
	} /* else */

	if ((st = tr->preset))
        {
		if (P_numname < st->place->numname)
			st = NULL;
		else
                {
			while (st->nextplace
				&& (st->nextplace->place->numname < P_numname))
			    st = st->nextplace;
                }
        }

	if (st)
	{
		if ((stnew->nextplace = st->nextplace))
			stnew->nextplace->prevplace = stnew;
		(stnew->prevplace = st) -> nextplace = stnew;
	} /* if */
	else
	{
		if ((stnew->nextplace = tr->preset))
			stnew->nextplace->prevplace = stnew;
		(tr->preset = stnew) -> prevplace = NULL;
	} /* else */

	*pl_last = stnew->place = pl;
	*tr_last = stnew->trans = tr;
	return (*st_last = stnew);

} /* M_fast_insert_PT_edge() */

/*###########################################################################*/
/*######                Operationen auf Phantomobjekten               #######*/
/*###########################################################################*/

/* Verbiegt einige Variablen und ruft dann die oberen Funktionen auf. */

#define swap(h,x1,x2) h=x1; x1=x2; x2=h;

M_Transition M_insert_phantom_transition(M_Net        net,
					 Cardinal32   T_numname,
					 M_Transition *tr_last)
{
	M_Transition result,tr;
	Cardinal32 numtr;

	swap(tr,net->Transitions,net->PhTransitions);
	swap(tr,net->LastTransition,net->LastPhTransition);
	swap(numtr,net->NumTransitions,net->NumPhTransitions);

	result = M_fast_insert_transition(net,T_numname,tr_last);

	swap(tr,net->Transitions,net->PhTransitions);
	swap(tr,net->LastTransition,net->LastPhTransition);
	swap(numtr,net->NumTransitions,net->NumPhTransitions);

	return result;
}

M_TSEdge M_insert_PTP_edge(M_Net        net,
			   Cardinal32   P_numname,
			   Cardinal32   T_numname,
			   M_Place      *pl_last,
			   M_Transition *tr_last,
			   M_TSEdge     *ts_last)
{
	M_Transition tr;
	Cardinal32 numtr;
	M_STEdge st;
	M_TSEdge result,ts;

	swap(tr,net->Transitions,net->PhTransitions);
	swap(tr,net->LastTransition,net->LastPhTransition);
	swap(numtr,net->NumTransitions,net->NumPhTransitions);
	swap(ts,(*pl_last)->preset,(*pl_last)->phpreset);
	swap(st,(*pl_last)->postset,(*pl_last)->phpostset);

	result = M_fast_insert_TP_edge(net,P_numname,T_numname,
						pl_last,tr_last,ts_last);

	swap(tr,net->Transitions,net->PhTransitions);
	swap(tr,net->LastTransition,net->LastPhTransition);
	swap(numtr,net->NumTransitions,net->NumPhTransitions);
	swap(ts,(*pl_last)->preset,(*pl_last)->phpreset);
	swap(st,(*pl_last)->postset,(*pl_last)->phpostset);

	return result;
}

M_STEdge M_insert_PPT_edge(M_Net        net,
			   Cardinal32   P_numname,
			   Cardinal32   T_numname,
			   M_Place      *pl_last,
			   M_Transition *tr_last,
			   M_STEdge     *st_last)
{
	M_Transition tr;
	Cardinal32 numtr;
	M_STEdge result,st;
	M_TSEdge ts;

	swap(tr,net->Transitions,net->PhTransitions);
	swap(tr,net->LastTransition,net->LastPhTransition);
	swap(numtr,net->NumTransitions,net->NumPhTransitions);
	swap(ts,(*pl_last)->preset,(*pl_last)->phpreset);
	swap(st,(*pl_last)->postset,(*pl_last)->phpostset);

	result = M_fast_insert_PT_edge(net,P_numname,T_numname,
						pl_last,tr_last,st_last);

	swap(tr,net->Transitions,net->PhTransitions);
	swap(tr,net->LastTransition,net->LastPhTransition);
	swap(numtr,net->NumTransitions,net->NumPhTransitions);
	swap(ts,(*pl_last)->preset,(*pl_last)->phpreset);
	swap(st,(*pl_last)->postset,(*pl_last)->phpostset);

	return result;
}

#define M_removeHead( _Head )     MYfree( _Head )
#define M_removeSTEdge( _STEdge ) \
        { free( _STEdge->label ); free( _STEdge ); } skip
#define M_removeTSEdge( _TSEdge ) \
        { free( _TSEdge->label ); free( _TSEdge ); } skip
#define M_removePlace( _Place )  free( _Place )
#define M_removeTrans( _Trans )  free( _Trans )
#define MSremove( mm ) free( mm )

/***************************************************************************/
/* Entfernen der Stelle 'pl' aus dem Netz 'net' zusammen mit allen Kanten, */
/* die in/aus diese/r Stelle fuehren (inkl. Phantomkanten).                */
/* Rueckgabewert ist 'pl->prevplace'.                                      */
/***************************************************************************/

M_Place M_RemovePlace(M_Net net,M_Place pl)
{
  register M_STEdge st, stdel;
  register M_TSEdge ts, tsdel;
  M_Place prevplace=NULL;

  if (net && pl) {
      /* Eingangskanten wech */
    ts = pl->preset;
    while (ts) {
      if (ts->prevplace)
        ts->prevplace->nextplace = ts->nextplace;
      else
        ts->trans->postset = ts->nextplace;
      if (ts->nextplace)
        ts->nextplace->prevplace = ts->prevplace;
      ts = (tsdel = ts)->nexttrans;
      M_removeTSEdge(tsdel);
    }  /* while noch Eingangskanten */
      /* Ausgangskanten wech */
    st = pl->postset;
    while (st) {
      if (st->prevplace)
        st->prevplace->nextplace = st->nextplace;
      else
        st->trans->preset = st->nextplace;
      if (st->nextplace)
        st->nextplace->prevplace = st->prevplace;
      st = (stdel = st)->nexttrans;
      M_removeSTEdge(stdel);
    }  /* while noch Ausgangskanten */
      /* Phantomeingangskanten wech */
    ts = pl->phpreset;
    while (ts) {
      if (ts->prevplace)
        ts->prevplace->nextplace = ts->nextplace;
      else
        ts->trans->postset = ts->nextplace;
      if (ts->nextplace)
        ts->nextplace->prevplace = ts->prevplace;
      ts = (tsdel = ts)->nexttrans;
      M_removeTSEdge(tsdel);
    }  /* while noch Phantomeingangskanten */
      /* Phantomausgangskanten wech */
    st = pl->phpostset;
    while (st) {
      if (st->prevplace)
        st->prevplace->nextplace = st->nextplace;
      else
        st->trans->preset = st->nextplace;
      if (st->nextplace)
        st->nextplace->prevplace = st->prevplace;
      st = (stdel = st)->nexttrans;
      M_removeSTEdge(stdel);
    }  /* while noch Phantomausgangskanten */
      /* Stelle aus der Netzverzeigerung loesen */
    if (pl->prevplace)
      pl->prevplace->nextplace = pl->nextplace;
    else
      net->Places = pl->nextplace;
    if (pl->nextplace)
      pl->nextplace->prevplace = pl->prevplace;
    else
      net->LastPlace = pl->prevplace;
      /* sonstige Aufraeumarbeiten */
    net->NumPlaces--;
    prevplace = pl->prevplace;
    if (pl->init_marking)
      MYfree(pl->init_marking);
    if (pl->curr_marking)
      MYfree(pl->curr_marking);
    if (pl->meaning)
      MYfree(pl->meaning);
    free(pl->blocklist);
    free(pl->reference);
    M_removePlace(pl);
  }  /* if Netz und Stelle vorhanden */
  return prevplace;
}  /* M_RemovePlace */

/*****************************************************************************/
/* Entfernen der Transition 'tr' aus dem Netz 'net' zusammen mit allen       */
/* Kanten, die in/aus diese/r Transition fuehren.                            */
/* Bei normalen Transitionen werden auch ALLE zugehoerigen Phantome          */
/* geloescht. Das kann bei NORMAL_SCOPING zu Fehlern fuehren, wenn die       */
/* Phantome noch zu anderen Transitionen gehoeren !!!                        */
/* Rueckgabewert ist 'tr->prevtrans'.                                        */
/*****************************************************************************/

M_Transition M_RemoveTransition(M_Net net, M_Transition tr, bool tr_is_phant)
{
    register M_STEdge     st,stdel;
    register M_TSEdge     ts,tsdel;
             M_Transition tr_help;

    if (net && tr) {
        ts=tr->postset;
        while (ts) {
            if (ts->prevtrans)
                ts->prevtrans->nexttrans=ts->nexttrans;
            else
              if (tr_is_phant)
                ts->place->phpreset=ts->nexttrans;
              else
                ts->place->preset=ts->nexttrans;
            if (ts->nexttrans)
                ts->nexttrans->prevtrans=ts->prevtrans;
            ts=(tsdel=ts)->nextplace;
            M_removeTSEdge(tsdel);
        }
        st=tr->preset;
        while (st) {
            if (st->prevtrans)
                st->prevtrans->nexttrans=st->nexttrans;
            else
              if (tr_is_phant)
                st->place->phpostset=st->nexttrans;
              else
                st->place->postset=st->nexttrans;
            if (st->nexttrans)
                st->nexttrans->prevtrans=st->prevtrans;
            st=(stdel=st)->nextplace;
            M_removeSTEdge(stdel);
        }
        if (tr->prevtrans)
            tr->prevtrans->nexttrans=tr->nexttrans;
        else
          if (tr_is_phant)
            net->PhTransitions=tr->nexttrans;
          else
            net->Transitions=tr->nexttrans;
        if (tr->nexttrans)
            tr->nexttrans->prevtrans=tr->prevtrans;
        else
          if (tr_is_phant)
            net->LastPhTransition=tr->prevtrans;
          else
            net->LastTransition=tr->prevtrans;
        MSremove(tr->label);
        MYfree( tr->annotation);
        MYfree( tr->reference );
          /* Blockverweise wegwerfen */
        MYfree(tr->blocklist);
        MYfree(tr->vars);
        if (tr->meaning)   MYfree(tr->meaning);
        tr_help=tr->prevtrans;
        if (tr_is_phant) {
          net->NumPhTransitions--;
          MYfree(tr);
        } else {
          net->NumTransitions--;
          M_removeTrans(tr);
        }
        return tr_help;
    } else
      return NULL;
}

/********************************************************/
/* Entfernt das Netz '*net' (schnell) aus dem Speicher. */
/********************************************************/

void M_Remove_net(M_Net *net)
{
  register M_STEdge STPtr, STRem;
  register M_TSEdge TSPtr, TSRem;
  register M_Place PlPtr, PlRem;
  M_Transition TrPtr, TrRem;

  if (*net) {
    PlPtr = (*net)->Places;
    if ((TrPtr = (*net)->Transitions)) {  /* Sind Transitionen da? */
      while (PlPtr) {
          /* normale Kanten muellen */
        TSPtr = PlPtr->preset;
        while (TSPtr) {
          TSPtr = (TSRem = TSPtr)->nexttrans;
          M_removeTSEdge(TSRem);
        }  /* while Eingangskanten */
        STPtr = PlPtr->postset;
        while (STPtr) {
          STPtr = (STRem = STPtr)->nexttrans;
          M_removeSTEdge(STRem);
        }  /* while Ausgangskanten */
          /* Phantomkanten muellen */
        TSPtr = PlPtr->phpreset;
        while (TSPtr) {
          TSPtr = (TSRem = TSPtr)->nexttrans;
          M_removeTSEdge(TSRem);
        }  /* while Phantomeingangskanten */
        STPtr = PlPtr->phpostset;
        while (STPtr) {
          STPtr = (STRem = STPtr)->nexttrans;
          M_removeSTEdge(STRem);
        }  /* while Phantomausgangskanten */
        PlPtr = (PlRem = PlPtr)->nextplace;
           /* Blockverweise muellen */
        MYfree(PlRem->blocklist);
        MYfree(PlRem->reference);
        if (PlRem->init_marking) MYfree(PlRem->init_marking);
        if (PlRem->curr_marking) MYfree(PlRem->curr_marking);
        if (PlRem->meaning) MYfree(PlRem->meaning);
        M_removePlace(PlRem);
      }  /* while Stellen */
        /* normale Transitionen */
      while (TrPtr) {
        TrPtr = (TrRem = TrPtr)->nexttrans;
        MSremove(TrRem->label);
        MYfree(TrRem->reference);
        MYfree(TrRem->blocklist);
        MYfree(TrRem->vars);
        if (TrRem->meaning) MYfree(TrRem->meaning);
        M_removeTrans(TrRem);
      }  /* while Transitionen */
        /* Phantomtransitionen */
      TrPtr = (*net)->PhTransitions;
      while (TrPtr) {
        TrPtr = (TrRem = TrPtr)->nexttrans;
        MSremove(TrRem->label);
        MYfree(TrRem->reference);
        MYfree(TrRem->blocklist);
        MYfree(TrRem->vars);
        if (TrRem->meaning) MYfree(TrRem->meaning);
        MYfree(TrRem);
      }  /* while Phantomtransitionen */
    } else {  /* Sind wenigstens Stellen da? */
      while (PlPtr) {
        PlPtr = (PlRem = PlPtr)->nextplace;
          /* Blockverweise muellen */
        MYfree(PlRem->blocklist);
        MYfree(PlRem->reference);
        if (PlRem->init_marking) MYfree(PlRem->init_marking);
        if (PlRem->curr_marking) MYfree(PlRem->curr_marking);
        if (PlRem->meaning) MYfree(PlRem->meaning);
        M_removePlace(PlRem);
      }  /* while Stellen */
    }  /* else, sind wenigstens Stellen da? */
    M_removeHead(*net);
    *net = NULL;
  }  /* if Netz vorhanden */
}  /* M_Remove_net */

/**********************/
/* END OF FILE mnet.c */
/**********************/
