/*****************************/
/* mnet.h                    */
/* Burkhard Graves, 26.6.'93 */
/*****************************/

#ifndef __MNET_H__
#define __MNET_H__

#include "multiset.h"
#include "mtupel.h"

/*****************************************************************************/
/* Variable-Struktur                                                         */
/*****************************************************************************/

typedef char *VarMSet;

/*****************************************************************************/

typedef char *PAnnotation;
typedef char *TAnnotation;

/**********/
/* Stelle */
/**********/

typedef struct M_PlElem_ {
    struct M_PlElem_ *prevplace,  /* Zur (doppelten) Verkettung in der      */
                     *nextplace;  /*   Stellenliste.                        */
    struct M_Edge_   *preset;     /* Zeiger auf erste Kante des Vorbereichs */
    struct M_Edge_   *postset;    /*   bzw. des Nachbereichs.               */
    struct M_Edge_   *phpreset;   /* dito fuer angrenzende		    */
    struct M_Edge_   *phpostset;  /*      Phantomtransitionen		    */
    PLabel            label;      /* Label der Stelle                       */
    PAnnotation       annotation; /* Annotation der Stelle                  */
    Cardinal32        numname;    /* Numerischer Name fuer div. Zwecke.     */
    char	     *charname;	  /* Textueller Name: z.B. P19 */
    char	     *blocklist;
    struct M_PlElem_ **children;  /* Array mit den entfalteten Stellen	    */
    char             *init_marking,
                     *curr_marking,
                     *reference;
    MPTupel	      type;
    int               x_pos,
                      y_pos,
                      flags;
    char             *meaning;
    char	     *bpnref, *pfaref, *sdlref;
    struct M_PlElem_ *parentplace;
} M_PlElem,*M_Place;

/**************/
/* Transition */
/**************/

typedef struct M_TrElem_ {
    struct M_TrElem_ *prevtrans,  /* Zur (doppelten) Verkettung in der      */
                     *nexttrans;  /*   Transitionenliste.                   */
    struct M_Edge_   *preset;     /* Zeiger auf erste Kante des Vorbereichs */
    struct M_Edge_   *postset;    /*   bzw. des Nachbereichs.               */
    MultiSet          label;      /* Beschriftung der Transition.           */
    TAnnotation       annotation; /* Annotation der Transition              */
    Cardinal32        numname;    /* Numerischer Name fuer div. Zwecke.     */
    char	     *charname;	  /* Textueller Name: z.B. T42 */
    char	     *blocklist;
    char	     *corrtran;	  /* zugehoerige Phantom/Synch-Transitionen */
    MPTupel	      partners;	  /* zugehoerige Sync-/Ph-Tr		    */
    MPTupel	      children;	  /* fuer HL-Tr: zugehoerige Kinder	    */
    MTupel	      vars;	  /* Liste der Variablen auf den Kanten	    */
    MPTupel	      childvals;  /* Variablenbelegungen der Kinder	    */
    int               x_pos,
                      y_pos,
                      flags;
    char             *reference;
    char             *meaning;
    char             *links;
    char	     *bpnref, *pfaref, *sdlref;
    struct M_TrElem_ *parenttrans;
    bool   	      dead;	  /* fuer Trs mit "uebergewichtigen" Kanten */
    Time_stamp*       time_label;
} M_TrElem, *M_Transition;

/**********/
/* Kanten */
/**********/

typedef struct M_Edge_ {
    struct M_Edge_ *prevplace, /* Zur (doppelten, vertikalen) Verkettung   */
                   *nextplace, /*   in Bezug auf die Stellenliste.         */
                   *prevtrans, /* Zur (doppelten, horizontalen) Verkettung */
                   *nexttrans; /*   in Bezug auf die Transitionenliste.    */
    M_Place         place;     /* Zeiger auf den Ursprung                  */
    M_Transition    trans;     /*   bzw. auf das Ziel der Kante.           */
    VarMSet         label;     /* Label der Kante.                         */
    Cardinal        flags;     
    MTupel	    vars;	 /* Codierung der Beschriftung		   */
} M_STElem,M_TSElem,*M_Edge,*M_STEdge,*M_TSEdge;

  /****************/
  /* Link symbols */
  /****************/
typedef char *linkdef;

/*********************/
/* Kopf eines Netzes */
/*********************/

typedef struct {
    M_Place      Places,         /* Zeiger auf erste bzw. auf letzte Stelle */
                 LastPlace;      /*   in der Stellenliste.                  */
    M_Transition Transitions,    /* Zeiger auf erste bzw. auf letzte Trans. */
                 LastTransition; /*   in der Transitionenliste.             */
    M_Transition PhTransitions,  /* Zeiger auf erste bzw. auf letzte Trans. */
                 LastPhTransition; /*   in der Phantomtransitionenliste.    */
    linkdef *Links;                /* Zeiger auf erste Linksymbol in der    */
                                   /* Linksymbolenliste                     */
    Cardinal32   NumPlaces,        /* Anzahl der Stellen,		     */
                 NumTransitions,   /*        der Transitionen,               */
                 NumPhTransitions; /*        der Phantomtransitionen	     */
    Cardinal32 NumLinks,           /*        der Linksymbolen                */
            MaxNumLinks;           /*           and max of Link symbols      */
    char	 *blockinfo, *definfo, *refblockinfo;
    char         *pfadefs, *pfablockinfo;
    char	 *sdldefs, *sdlblockinfo;
} M_NetHead,*M_Net;

/**********************************************************************/
/* Funktionen zur Allokation von Speicherplatz fuer einzelne Elemente */
/* eines Netzes; realisiert als Makros.				      */
/**********************************************************************/

/* Funktionalitaet: M_Net   M_allocHead(void)   */
/*                  M_Place M_allocPlace(void)  */
/*             M_Transition M_allocTrans(void)  */
/*                 M_STEdge M_allocSTEdge(void) */
/*                 M_TSEdge M_allocTSEdge(void) */

#define M_allocHead()   ((M_Net) MYmalloc(sizeof(M_NetHead)))
#define M_allocPlace()  ((M_Place) MYmalloc(sizeof(M_PlElem)))
#define M_allocTrans()  ((M_Transition) MYmalloc(sizeof(M_TrElem)))
#define M_allocSTEdge() ((M_STEdge) MYmalloc(sizeof(M_STElem)))
#define M_allocTSEdge() ((M_TSEdge) MYmalloc(sizeof(M_TSElem)))

/*****************************************************************************/

extern M_Net M_empty_net(void);
extern M_Place M_fast_insert_place(M_Net, Cardinal32, M_Place*);
extern M_Transition M_fast_insert_transition(M_Net, Cardinal32, M_Transition*);
extern M_TSEdge M_fast_insert_TP_edge(M_Net, Cardinal32, Cardinal32,
				      M_Place*, M_Transition*, M_TSEdge*);
extern M_STEdge M_fast_insert_PT_edge(M_Net, Cardinal32, Cardinal32,
				      M_Place*, M_Transition*, M_STEdge*);
extern M_Transition M_insert_phantom_transition(M_Net,Cardinal32,M_Transition*);
extern M_TSEdge M_insert_PTP_edge(M_Net, Cardinal32, Cardinal32,
				  M_Place*, M_Transition*, M_TSEdge*);
extern M_STEdge M_insert_PPT_edge(M_Net, Cardinal32, Cardinal32,
				  M_Place*, M_Transition*, M_STEdge*);
void M_Remove_net(M_Net *net);

/*****************************************************************************/

#endif

/**********************/
/* END OF FILE mnet.h */
/**********************/
