/*****************************************************************************/
/*  multiset.c                                                               */
/*  Mi.,04.11.1992 - Mi.,15.11.1993                           Stefan Roemer  */
/*****************************************************************************/

#include <stdio.h>

#include "multiset.h"

/*****************************************************************************/
/* Globale Variablen fuer die Behandlung von Multimengen.                    */
/*****************************************************************************/

Cardinal MaxActions   = ACTION_BUF_SIZE;

Cardinal ActCounter  = 0;

MultiSet mmDummy    = NULL;       /* Hilfsmenge bei der Berechnung von sum,  */
                                  /* union,...                               */
MultiSet mmSingle   = NULL;       /* Hilfsmenge fuer Multimengen mit maximal */
                                  /* einem Eintrag (Aufbau Parsetree).       */


/*****************************************************************************/
/* Initialisieren der globalen Variablen des Multimengen-Funktionspakets.    */
/*****************************************************************************/

void InitMultiSets(void)
{
	ActCounter = 0;
	MaxActions = ACTION_BUF_SIZE;

	mmDummy  = MSalloc(MaxActions);

	if (!mmSingle) mmSingle = MSalloc(1);

} /* InitMultiSets() */


/*****************************************************************************/
/* Funktion zur Initialisierung einer neuen Multimenge als Kopie einer be-   */
/* reits vorhandenen Multimenge; der Speicherbedarf der neuen Menge ent-     */
/* spricht ihrer tatsaechlichen "Laenge".                                    */
/*****************************************************************************/

MultiSet MScreate_copy(MultiSet mm)
{
	MultiSet      mm2;	      /* Zeiger auf die zu erzeugende Kopie. */
	register MultiSet mmptr = mm, /* Laufzeiger durch die Originalmenge. */
			  mm2ptr;     /* Laufzeiger durch die Kopie.	     */
	register Cardinal len = 0;    /* Laenge der Originalmenge.	     */

	/* "Laenge" der Multimenge ermitteln und in `len` ablegen. */
	while (*mmptr)
	{
		len++;
		mmptr++;
		mmptr++;
	}

	/* Speicherplatz fuer die Kopie anfordern. */
	mm2ptr = mm2 = MSalloc(len);
	mmptr = mm;

	/* Eintraege einzeln kopieren.		   */
	while ((*(mm2ptr++) = *(mmptr++)));

	/* Zeiger auf die Kopie zurueckliefern.	   */
	return (mm2);

} /* MScreate_copy() */


/*****************************************************************************/
/* Funktion zur Feststellung, ob die Aktion `akt` oder deren Konjugierte     */
/* `^akt` in der Multimenge `mm` vorkommen.                                  */
/*****************************************************************************/

bool MSis_in(Action akt, MultiSet mm)
{
	/* Parameter auf Gueltigkeit testen */
	if (!mm || !*mm || !akt) return False;

	/* In der Menge die Stelle suchen, an der die Aktion stehen muss */
	while (*mm && (*mm < akt)) mm++, mm++;

	/* Rueckgabe: True, gwd. `akt` oder `^akt` in `mm` enthalten */
	return (akt == *mm); 

} /* MSis_in() */

/*****************************************************************************/
/* Funktion zur Vereinigung zweier Multimengen.                              */
/*****************************************************************************/

MultiSet MSunion(MultiSet mm1, MultiSet mm2)
/* GLOBAL MultiSet mmDummy; */		/* mmDummy nimmt Zwischenergebnis auf */
{
	register MultiSet mm1Ptr = mm1,
			  mm2Ptr = mm2,
			  mmDum  = mmDummy;

	/* Solange nicht beide Mengen abgearbeitet sind: */
	while (*mm1Ptr || *mm2Ptr)
	{
		/* Falls die aktuell betrachteten Aktionen */
		/* aus Menge 1 und 2 uebereinstimmen,      */
		if (*mm1Ptr && (*mm1Ptr == *mm2Ptr))
		{
			/* wird die Aktion einmal in die Hilfsmenge uebernommen
			   und die Anzahl der Vorkommen dort auf das Maximum
			   der Vorkommen in den beiden Mengen gesetzt.	     */
			*(mmDum++) = *(mm1Ptr++); mm2Ptr++;
			*(mmDum++) = Max(*mm1Ptr, *mm2Ptr);
			mm1Ptr++, mm2Ptr++;
		}
		/* Falls der Aktionscode aus Menge 1 kleiner ist als der der
		   Aktion aus Menge 2, wird diejenige aus Mange 1 in die
		   Hilfsmenge uebernommen.				     */
		else if ((!*mm2Ptr) || (*mm1Ptr && (*mm1Ptr < *mm2Ptr)))
		{
			*(mmDum++) = *(mm1Ptr++);
			*(mmDum++) = *(mm1Ptr++);
		}
		else
		{
			/* Ansonsten wird die Aktion aus Menge 2 uebernommen.*/
			*(mmDum++) = *(mm2Ptr++);
			*(mmDum++) = *(mm2Ptr++);
		}
	} /* while */

	/* Hilfsmenge abschliessen.*/
	*mmDum = 0;

	/* Hilfsmenge nach `mm1` kopieren.
	   Der alte Inhalt von `mm1` wird ueberschrieben! */
	mmDum = mmDummy;
	mm1Ptr = mm1;
	while ((*(mm1Ptr++) = *(mmDum++)));

	/* Ergebnismenge zurueckliefern. */
	return (mm1);

} /* MSunion() */


/*****************************************************************************/
/* Funktion zur Summenbildung zweier Multimengen.                            */
/*****************************************************************************/

MultiSet MSsum(MultiSet mm1, MultiSet mm2)
/* GLOBAL MultiSet mmDummy; */		/* mmDummy nimmt Zwischenergebnis auf */
{
	register MultiSet mm1Ptr = mm1,
			  mm2Ptr = mm2,
			  mmDum  = mmDummy;

	while (*mm1Ptr || *mm2Ptr)
	{
		if (*mm1Ptr && (*mm1Ptr == *mm2Ptr))
		{
		    *(mmDum++) = *(mm1Ptr++); mm2Ptr++;
		    if ((*(mmDum++) = *mm1Ptr + *mm2Ptr) < *mm1Ptr)
		    {
			fprintf(stderr, "\nUeberlauf: Aktionsname zu haeufig; "
				"Abh.: 'Action'\nProgram aborted.\n");
			exit (1);
		    }
		    mm1Ptr++, mm2Ptr++;
		}
		else if ((!*mm2Ptr) || (*mm1Ptr && (*mm1Ptr < *mm2Ptr)))
		{
		    *(mmDum++) = *(mm1Ptr++);
		    *(mmDum++) = *(mm1Ptr++);
		}
		else /* ((!*mm1Ptr) || (*mm1Ptr > *mm2Ptr)) */
		{
		    *(mmDum++) = *(mm2Ptr++);
		    *(mmDum++) = *(mm2Ptr++);
		}
	} /* while */

	*mmDum = 0;
	mmDum = mmDummy;
	mm1Ptr = mm1;

	while ((*(mm1Ptr++) = *(mmDum++)));
	return (mm1);

} /* MSsum() */


/*****************************************************************************/
/* Funktion zur Differenzbildung zweier Multimengen.                         */
/*****************************************************************************/

MultiSet MSdifference(MultiSet mm1, MultiSet mm2)
{
	register MultiSet mm1rPtr = mm1, /* Lesezeiger    */ 
			  mm1wPtr = mm1, /* Schreibzeiger */
			  mm2Ptr  = mm2;

	while (*mm1rPtr)
	{
		if (*mm1rPtr == *mm2Ptr)
		{
			*(mm1wPtr++) = *(mm1rPtr++);
			mm2Ptr++;
			*mm1wPtr = (*mm1rPtr > *mm2Ptr)? *mm1rPtr - *mm2Ptr : 0;
			mm1rPtr++;
			mm2Ptr++;
			if (*mm1wPtr) mm1wPtr++; else mm1wPtr--;
		} 
		else if (*mm2Ptr && (*mm1rPtr > *mm2Ptr))
		{
			mm2Ptr++;
			mm2Ptr++;
		}
		else /* ((!*mm2Ptr) || (*mm1rPtr < *mm2Ptr)) */
		{
			*(mm1wPtr++) = *(mm1rPtr++);
			*(mm1wPtr++) = *(mm1rPtr++);
		}
	} /* while */

	*mm1wPtr = 0;
	return (mm1);

} /* MSdifference() */


/*****************************************************************************/
/* END OF FILE multiset.c                                                    */
/*****************************************************************************/

