/*****************************************************************************/
/*  multiset.h                                                               */
/*  C-Headerdatei: Multimengen ueber Aktionen -- Datenstrukturen und         */
/*                                               Funktionsprototypen/Makros  */
/*  Di., 03.11.1992 - Di.,02.02.1993                          Stefan Roemer  */
/*****************************************************************************/

#ifndef __MULTISET_H__
#define __MULTISET_H__

#include "neunetze.h"

#define ACTION_BUF_SIZE 100

/*****************************************************************************/
/* Globale Variablen fuer die Behandlung von Multimengen. Diese sind nur     */
/* deshalb hier global zusammengefasst worden, um die Aufrufe einiger        */
/* Funktionen einfacher zu halten. Sie duerfen auf keinen Fall veraendert    */
/* werden! Zu Beginn des Hauptprogramms sollte die Funktion `InitMultisets`  */
/* aufgerufen werden, um diese Variablen zu initialisieren!                  */
/*****************************************************************************/

extern Cardinal MaxActions;  /* Maximale Anzahl von Aktionsnamen, die in der */
			     /* Aktionen-Symboltabelle gespeichert werden    */
			     /* koennen; wird dynamisch an benoetigte Groes- */
			     /* se angepasst. Dieser Wert bestimmt auch die  */
			     /* Groesse der dynamischen Arrays, welche zur   */
			     /* Aufnahme von Zwischenergebnissen von Multi-  */
			     /* mengen-Operationen dienen (globale Variable  */
			     /* mmDummy).				     */

extern Cardinal ActCounter;  /* Aktuelle, tatsaechliche Anzahl der in dem zu */
			     /* analysierenden Box Expression vorkommenden   */
			     /* (verschiedenen) Aktionsnamen, wobei konju-   */
			     /* gierte Aktionen nicht ein zweites Mal mitge- */
			     /* zaehlt werden, falls die Aktion bereits un-  */
			     /* konjugiert aufgenommen worden ist (und um-   */
			     /* gekehrt). Die Aktionsnamen werden als Zei-   */
			     /* chenketten in der Aktionen-Symboltabelle     */
			     /* gespeichert.				     */

extern MultiSet mmDummy;     /* Hilfsmenge bei der Berechnung von Summe und  */
			     /* Vereinigung von Multimengen zur Aufnahme der */
			     /* Ergebnismenge, die "laenger" sein kann als   */
			     /* jeder der "Summanden".			     */

extern MultiSet mmSingle;    /* Hilfsmenge zur Aufnahme einer Multimenge mit */
			     /* nur einer einzigen Aktion (die jedoch mehr-  */
			     /* fach auftreten darf.			     */


/*****************************************************************************/
/* FUNKTIONEN AUF MULTIMENGEN:                                               */
/*****************************************************************************/

/*****************************************************************************/
/* Funktion zum Initialisieren/Allokieren der globalen Variablen des Multi-  */
/* mengen-Funktionspakets. Diese Funktion muss vom Hauptprogramm aufgerufen  */
/* werden, bevor mit Multimengen operiert wird!!!                            */
/*****************************************************************************/

extern void InitMultiSets(void);

/*****************************************************************************/
/* Funktion zum Einrichten einer neuen Multimenge, hier wegen der Kuerze als */
/* Makro realisiert. Nur eine mit dieser Funktion initialisierte Multimenge  */
/* ist physikalisch im Speicher vorhanden, und zwar mit der uebergebenen     */
/* "Laenge" `n`, d.h. sie kann spaeter hoechstens `n` verschiedene Aktionen  */
/* aufnehmen, wobei konjugierte Aktionen extra mitgezaehlt werden muessen,   */
/* Mehrfachvorkommen eines Aktionsnamens in der entsprechenden Menge jedoch  */
/* nicht. Spaeter wird auf die so initialisierten Multimengen mit Hilfe von  */
/* Zeigern verwiesen.                                                        */
/* Rueckgabe: Zeiger auf den so bereitgestellten Speicherbereich zur Aufnahme*/
/*            einer Multimenge mit hoechstens `n` verschiedenen Aktionen.    */
/*****************************************************************************/

/* Funktionalitaet:  MultiSet MSalloc(Cardinal n) */
#define MSalloc(n) \
  (MultiSet) MYmalloc((((n) << 1) + 1) * sizeof(Action))

/*****************************************************************************/
/* Funktion (Makro) zum Verlaengern einer Multimenge.                      */
/* Falls der fuer die Multimenge `mm` vorgesehene Platz nicht ausreicht,     */
/* muss mit dieser Funktion die "Laenge" von `mm` explizit auf die neue      */
/* "Laenge" `n` vergroessert werden, wobei der bisherige Inhalt der Menge    */
/* erhalten bleibt. Die Funktion prueft nicht explizit, ob der neue Wert `n` */
/* tatsaechlich groesser ist als die vorherige "Laenge der Menge `mm`!       */
/* Rueckgabe: Zeiger auf den entsprechenden Speicherbereich. Der alte Inhalt */
/*            der Multimenge bleibt erhalten, auch wenn sich die verlaengerte*/
/*            Menge nicht am gleichen Speicherplatz wie vorher befinden muss!*/
/*****************************************************************************/

/* Funktionalitaet:  MultiSet MSresize(MultiSet mm, Cardinal n) */
#define MSresize(mm, n) \
  (mm = (MultiSet) MYrealloc((char *) mm, (((n) << 1) + 1) * sizeof(Action)))

/*****************************************************************************/
/* Funktion (als Makro), die eine bestehende Multimenge `mm` derart aen-   */
/* dert, dass diese nach dem Funktionsaufruf die Menge `mm := {num akt}`     */
/* darstellt. `mm` muss also auf eine mindestens einelementige Multimenge    */
/* zeigen. Rueckgabe ist ein Zeiger auf die so manipulierte Multimenge.      */
/*****************************************************************************/

/* Funktionalitaet: Multiset MSset_single(Multiset mm,Action akt,Cardinal num)*/
#define MSset_single(mm, akt, num) \
  ( *(mm) = (akt), *((mm)+1) = num, *((mm)+2) = 0, mm )

/*****************************************************************************/
/* Funktion zur Initialisierung einer neuen Multimenge als Kopie einer       */
/* bereits vorhandenen Multimenge `mm`; der Speicherbedarf der neu allokier- */
/* ten Menge entspricht ihrer tatsaechlichen "Laenge".                       */
/* Ein Zeiger auf diese neu eingerichtete Menge wird zurueckgegeben.         */
/*****************************************************************************/

extern MultiSet MScreate_copy(MultiSet mm);

/*****************************************************************************/
/* Funktion zur Feststellung, ob in der Multimenge `mm` die Aktion `akt`     */
/* oder deren Konjugierte `^akt` enthalten ist.                              */
/* Rueckgabe: Boole'scher Wahrheitswert ( True bei Vorkommen der Aktion ).   */
/*****************************************************************************/

extern bool MSis_in(Action akt, MultiSet mm);

/*****************************************************************************/
/* Funktion zur Vereinigung zweier Multimengen `mm1` und `mm2`; es wird      */
/* `mm1 := mm1 u mm2` berechnet, d.h. die zweite Menge wird zu der ersten    */
/* "hinzuvereinigt". Rueckgabe ist ein Zeiger auf die Ergebnismenge; `mm1`   */
/* ist somit nach dieser Operation zerstoert und muesste, falls gewuenscht,  */
/* vorher explizit in eine Hilfsmenge kopiert werden (mit Funktion `MScopy`  */
/* oder `MScreate_copy`). Achtung! Multimenge `mm1` muss gross genug zur     */
/* Aufnahme des Ergebnisses sein!!!                                          */
/* Seiteneffekt: Benutzung der globalen Variablen `mmDummy`.                 */
/*****************************************************************************/

extern MultiSet MSunion(MultiSet mm1, MultiSet mm2 );
/* GLOBAL MultiSet mmDummy; (schreibend) */

/*****************************************************************************/
/* Funktion zur Summenbildung zweier Multimengen `mm1` und `mm2`: es wird    */
/* `mm1 := mm1 + mm2` berechnet, d.h. `mm2` wird zu `mm1` "hinzuaddiert".    */
/* Rueckgabe: Zeiger auf die Ergebnismenge `mm1`, deren alter Inhalt ueber-  */
/*            schrieben wird.                                                */
/* Seiteneffekt: Benutzung der globalen Variablen `mmDummy` zur Aufnahme     */
/*               des Zwischenergebnisses.                                    */
/*****************************************************************************/

extern MultiSet MSsum(MultiSet mm1, MultiSet mm2);
/* GLOBAL MultiSet mmDummy; (schreibend) */

/*****************************************************************************/
/* Funktion zur Differenzbildung zweier Multimengen `mm1` und `mm2`; es wird */
/* `mm1 := mm1 \ mm2` berechnet, `mm2` wird also von `mm1` "abgezogen".      */
/* Rueckgabe: Zeiger auf die Ergebnismenge `mm1`, deren alter Inhalt somit   */
/*            ueberschrieben ist (evtl. retten durch Kopieren!).             */
/*****************************************************************************/

extern MultiSet MSdifference(MultiSet mm1, MultiSet mm2);

#endif /* __MULTISET_H__ */


/*****************************************************************************/
/* END OF FILE multiset.h                                                    */
/*****************************************************************************/

