/*****************************************************************************/
/*  parser.c                                                                 */
/*  C-Quelldatei: Hilfsroutinen fuer den 'box expression'-Parser             */
/*  Di.,27.10.1992 - Mi.,04.11.1992                           Stefan Roemer  */
/*****************************************************************************/

#include <string.h>
#include "parser.h"
#include "multiset.h"

/*****************************************************************************/
/*            Globale Variablen fuer den 'box expression'-Parser             */
/*****************************************************************************/

SymTab SymbolTable = NULL;
btree  STree;

/*****************************************************************************/
/*             Hilfsfunktionen fuer den 'box expression'-Parser              */
/*****************************************************************************/

/*****************************************************************************/

btree new_node(int token, btree left, btree right)
{
	register btree node = (btree) MYmalloc(sizeof(btree_elem));

	node->token = token;
	node->left  = left;
	node->right = right;

	return (node);

} /* new_node() */


/*****************************************************************************/
/* Umwandeln eines Aktionsnamens (String) in entsprechenden Aktionscode      */
/* (Cardinal). Falls der Name noch nicht in der Symboltabelle enthalten      */
/* ist, wird er in derselben am Ende eingefuegt.                             */
/*****************************************************************************/

Action StrToAction(char *Name)
{
	register SymTab TblPtr = SymbolTable;

	/* Den Namen in der Symboltabelle suchen.
	   Lineare Suche. Eventuell schnelleres Verfahren fuer die
	   Symboltabellenverwaltung verwenden !!!			*/
	while (*TblPtr && ((*Name != **TblPtr) || strcmp(Name, *TblPtr)))
		TblPtr++;

	/*Falls Name nicht in Liste vorhanden, am Ende der Tabelle einfuegen.*/
	if (!*TblPtr)
	{
	    if (++ActCounter >= MaxActions)
	    {
		int OffSet = (int)(TblPtr-SymbolTable);
		TblPtr = (SymbolTable = (SymTab) MYrealloc((char*) SymbolTable,
		  (Cardinal32)(MaxActions+=ACTION_BUF_SIZE)*sizeof(SymTabEntry))
			 ) + OffSet;
		MSresize(mmDummy, MaxActions);
	    }
	    *TblPtr = MYstrdup(Name);
	    *(TblPtr+1) = NULL;
	}

	/* Position des Namens in der Symboltabelle in den Aktionscode umrech-
	   nen und zurueckgeben; in der Tabelle sind alle Namen unkonjugiert! */
	return (Action)(TblPtr-SymbolTable+1);

} /* StrToAction() */


/*****************************************************************************/
/* Umwandlung Aktion(scode) --> Aktionsname (String).                        */
/*****************************************************************************/

char* ActionToStr(Action akt, char *str)
{
	return strcpy(str,SymbolTable[akt-1]);
} /* ActionToStr() */


/*****************************************************************************/
/* Initialisiere globale Variablen: Symboltabellen.                          */
/*****************************************************************************/

void InitAllSymbolTables()

/* GLOBAL: SymTab   SymbolTable;
	   Cardinal MaxActions, ActCounter; */

{
	/* Initialisieren der Aktionen-Symboltabelle.*/

	SymbolTable = (SymTab) MYmalloc(MaxActions * sizeof(SymTabEntry));
	*SymbolTable = NULL;
	MaxActions = ACTION_BUF_SIZE;
	ActCounter = 0;

} /* InitAllSymbolTables() */


/*****************************************************************************/
/* END OF FILE parser.c                                                      */
/*****************************************************************************/

