/*****************************************************************************/
/*  parser.h                                                                 */
/*  C-Headerdatei: Hilfsroutinen fuer den HL-Expression-Parser               */
/*  Mo.,13.02.1995 - Mo.,13.02.1995                           Stefan Roemer  */
/*****************************************************************************/


#ifndef __PARSER_H__
#define __PARSER_H__

#include "multiset.h"

/* some declarations for the scanner. */
extern int yylex(void);
extern char* strptr;

/*****************************************************************************/
/* Maximal vorgesehene Anzahl von Aktionsnamen in Symboltabelle; wird        */
/* dynamisch an die tatsaechlich benoetigte Groesse angepasst!               */
/*****************************************************************************/

typedef char *SymTabEntry;
typedef SymTabEntry *SymTab;

/*****************************************************************************/

typedef struct btree_
{
	int           token;
	struct btree_ *left,
		      *right;
} btree_elem, *btree;

extern btree STree;

/*****************************************************************************/

extern btree new_node( int, btree, btree );

/*****************************************************************************/
/* Initialisierungsfunktion, die die vom Parser benutzten globalen Variablen */
/* (insbesondere die Symboltabellen) einrichtet. Diese Funktion ist unbe-    */
/* dingt zu Beginn des Hauptprogramms (vor "Benutzung" des Parsers) aufzu-   */
/* rufen!								     */
/*****************************************************************************/

extern void InitAllSymbolTables(void);
/* GLOBAL	SymTab   SymbolTabelle, (schreibend)
		Cardinal MaxActions,    (schreibend)
			 ActCounter;    (schreibend) */


/*****************************************************************************/
/* Funktion zur Umwandlung eines Aktionsnamens `Name` (String) in den ent-   */
/* sprechenden Aktionscode (Cardinal bzw. Action), der als Ergebnis zurueck- */
/* geliefert wird. Falls der Name noch nicht in der Aktions-Symboltabelle    */
/* vorhanden ist, wird dieser hinzugefuegt und ein entsprechender Code fuer  */
/* ihn berechnet.							     */
/*****************************************************************************/

extern Action StrToAction(char *Name);
/* GLOBAL	SymTab	 SymbolTable; (schreibend)
		Cardinal ActCounter;  (schreibend)
		MultiSet mmDummy;     (schreibend) */


/*****************************************************************************/
/* Funktion zur Umwandlung eines Aktionscodes `akt` in den entsprechenden    */
/* Namen, der in der bereitgestellten Zeichenkette `str` abgelegt wird.      */
/* `str` sollte also ein Zeiger auf einen zur Aufnahme des Aktionsnamens ge- */
/* nuegend gross initialisierten Speicherbereich sein; `str` wird schliess-  */
/* lich als Funktionsergebnis geliefert.				     */
/*****************************************************************************/

extern char *ActionToStr(Action akt,char *str);
/* GLOBAL: SymTab SymbolTable; (lesend)  */

#endif

/*****************************************************************************/
/* END OF FILE parser.h                                                      */
/*****************************************************************************/

