/*****************************************************************************/
/* readHL.c								     */
/*****************************************************************************/
/*                                  Stefan Roemer,  06.02.1995 - 06.02.1995  */
/*****************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "mnet.h"
#include "readPEP.h"
#include "unfoldHL.h"
#include "readlib.h"
#include "readHL.h"

#define NAMES_START	2000
#define NAMES_OFFSET	1000
#define MBSIZE		4096

char buffer[10];
bool autonumbering;
int blockused, blockalloc, placecount, transcount;

M_Net		net;
M_Place		*PlArray;
M_Transition	*TrArray, *PhTrArray;

Cardinal32 AnzPlNamen, MaxPlNamen,
	   AnzTrNamen, MaxTrNamen,
	   AnzPhTrNamen, MaxPhTrNamen;

/* links: link terms on transitions,
   l_symb: link symbol declaration -> in the net */
char	 *name, *meaning, *block, *guard, *name2, *links,
	 *cmarking, *reference, *corrtran, *blk, *l_symb;
int	 ent_place, exit_place, invisible, ident;
double   eft, lft;
t_coords *co;

t_dest null_dest[] = { {  0 ,  0 } };

t_dest place_dest[] =
	{ { '\'',&name },	/* Bezeichner		*/
	  { '"', &name },	/*     "		*/
	  { '@', &co },		/* Position		*/
	  { '0', &ident },	/* numerischer Name	*/
	  { 'b', &meaning },	/* Bedeutung		*/
	  { 'u', &block },	/* Blockliste		*/
	  { 'Z', &guard },	/* Typ			*/
	  { 'z', &name2 },	/* Init-Marking		*/
	  { 'y', &cmarking },	/* Current-Marking	*/
	  { 'R', &reference },	/* Referenz		*/
	  { 'e', &ent_place },	/* Entry-Stelle		*/
	  { 'x', &exit_place },	/* Exit-Stelle		*/
	  { 'v', &invisible },	/* Flags		*/
	  {  0 ,  0 } };
  
t_dest trans_dest[] =
	{ { '\'',&name },	/* Bezeichner		*/
	  { '"', &name },	/*     "		*/
	  { '@', &co },		/* Position		*/
	  { '0', &ident },	/* numerischer Name	*/
	  { 'b', &meaning },	/* Bedeutung		*/
	  { 'l', &links },	/* Link terms		*/
	  { 'v', &invisible },	/* Flags		*/
	  { 'u', &block },	/* Blockliste		*/
	  { 'P', &corrtran },	/* Liste der Ph.Trans.	*/
	  { 'g', &guard },	/* Value Term		*/
	  { 'R', &reference },	/* Referenz		*/
	  { 'E', &eft },	/* Earliest firing time */
	  { 'L', &lft},		/* Latest firing time   */
	  { 'i', &name2 },	/* Aktionsterme		*/
	  {  0 ,  0 } };
  
t_dest arc_dest[] =
	{ { '@', &co },		/* Quelle/Ziel		*/
	  { 'p', &guard },	/* Beschriftung		*/
	  { 'v', &invisible },	/* Sichtbarkeit		*/
	  {  0 ,  0 } };

int HL_insert_block();
int HL_insert_link_symbol();
int HL_insert_place();
int HL_insert_trans();
int HL_insert_arc();

M_Net read_HL_net(char *PEPfilename)
{
	int count;

	t_blockdest netdest[] =
		{ { "BL",  HL_insert_block, &blk, null_dest },
                  { "LS",  HL_insert_link_symbol, &l_symb, null_dest },
		  { "PL",  HL_insert_place, NULL, place_dest },
		  { "TR",  HL_insert_trans, NULL, trans_dest },
		  { "PTR", HL_insert_trans, NULL, trans_dest },
		  { "TP",  HL_insert_arc,   NULL, arc_dest },
		  { "PT",  HL_insert_arc,   NULL, arc_dest },
		  { "PTP", HL_insert_arc,   NULL, arc_dest },
		  { "PPT", HL_insert_arc,   NULL, arc_dest },
		  { NULL, NULL, NULL, NULL } };

	PlArray = (M_Place*) MYmalloc((count = MaxPlNamen = MaxTrNamen
		= MaxPhTrNamen = NAMES_START) * sizeof(M_Place));
	TrArray = (M_Transition*) MYmalloc(count * sizeof(M_Transition));
	PhTrArray = (M_Transition*) MYmalloc(count * sizeof(M_Transition));
	AnzPlNamen = AnzTrNamen = AnzPhTrNamen = 0;
	while (--count)
	{
		PlArray[count] = (M_Place) NULL;
		TrArray[count] = (M_Transition) NULL;
		PhTrArray[count] = (M_Transition) NULL;
	}

	/* Initialize net */
	net = M_empty_net();

	/* Initialize blocks */
	net->blockinfo = MYmalloc(blockalloc = MBSIZE);
	*(net->blockinfo) = '\0';
	blockused = 0;
	placecount = transcount = 0;
	autonumbering = True;

	/* read the net */
	read_PEP_net(PEPfilename, netdest);

	free(PlArray);
	free(TrArray);
	free(PhTrArray);
	return net;

}

int HL_insert_block()
{
	if (!blk) return 0;
	blockused += strlen(blk);
	while (++blockused > blockalloc) 
		net->blockinfo = MYrealloc(net->blockinfo, blockalloc+=MBSIZE);
	sprintf(net->blockinfo, "%s%s\n", net->blockinfo, blk);
	free(blk);
	return 0;
}

int HL_insert_link_symbol()
{
	int count; /* counter */
	int MaxLinkSymbs = net -> MaxNumLinks;
	if (!l_symb || strlen(l_symb)<1) return 0;
	/* eventually (re)alloc memory to net->Link */
	if ( net -> NumLinks >= MaxLinkSymbs )
	 {
	    if (net->Links)
	      net -> Links = (linkdef *) MYrealloc( net -> Links,
	                   ( MaxLinkSymbs += 10 ) * sizeof( linkdef *) );
	    else
	      net -> Links = (linkdef *) MYmalloc
			   ((MaxLinkSymbs += 10) * sizeof (linkdef *) );
	    for ( count = MaxLinkSymbs - 10 ; count < MaxLinkSymbs ; count++ )
	     {
		       net -> Links [ count ] = (linkdef) NULL;
	     } /* for */
	    net -> MaxNumLinks = MaxLinkSymbs;
	  } /* if (net -> NumLinks >= MaxLinkSymbs) */

	net -> Links [net->NumLinks] =
		(char *) MYmalloc ((strlen(l_symb)+1) * sizeof(char) );
	strcpy (net -> Links [net->NumLinks], l_symb);

	net -> NumLinks++;

	return 0;
}

int HL_insert_place()
{
	static M_Place pl_last = NULL;

	placecount++;
	if (ident && ident != placecount) autonumbering = False;
	if (!ident && autonumbering) ident = placecount;
	if (!ident) PEPerror("missing place identifier");
		
	if (ident > AnzPlNamen)
		AnzPlNamen = ident;
	else if (PlArray[ident])
		Error((errbuf, "place identifier %d used twice", ident));

	while (AnzPlNamen >= MaxPlNamen)
	{
		int count;
		PlArray = (M_Place*) MYrealloc((char*) PlArray,
				(MaxPlNamen += NAMES_OFFSET) * sizeof(M_Place));
		for (count = MaxPlNamen - NAMES_OFFSET; count < MaxPlNamen;)
			PlArray[count++] = (M_Place) NULL;

	}

	PlArray[ident] = M_fast_insert_place(net,ident,&pl_last);
	pl_last->x_pos = co->x;
	pl_last->y_pos = co->y;
	pl_last->label = ent_place? ENTRY: exit_place? EXIT : EMPTY;
	if (!name) sprintf(buffer, "P%u", ident);
	pl_last->charname = name? name : MYstrdup(buffer);
	pl_last->blocklist = block;
	pl_last->annotation = guard;
	pl_last->flags = invisible;
	pl_last->reference = reference;
	pl_last->init_marking = name2;
	pl_last->curr_marking = cmarking;
	pl_last->meaning = meaning;
	return 0;
}

#define swap(h,x1,x2) h=x1; x1=x2; x2=h;
int swapped = 0;

int HL_insert_trans()
{
	int count;
	static M_Transition tr_last = NULL;

	if (*blocktype == 'P')
	{
		M_Transition *trtmp;
		
		*blocktype = '\0';
		tr_last = NULL;
		transcount = 0;
		swapped++;
		swap(trtmp,PhTrArray,TrArray);
		swap(count,AnzPhTrNamen,AnzTrNamen);
		swap(count,MaxPhTrNamen,MaxTrNamen);
	}

	if (!transcount++) autonumbering = True;
	if (ident && ident != transcount) autonumbering = False;
	if (!ident && autonumbering) ident = transcount;
	if (!ident) PEPerror("missing transition identifier");

	if (ident > AnzPhTrNamen)
		AnzPhTrNamen = ident;
	else if (PhTrArray[ident])
		Error((errbuf, "%stransition identifier %d used twice", 
			    *blocktype == 'T'? "" : "phantom ", ident));

	while (AnzPhTrNamen >= MaxPhTrNamen)
	{
		PhTrArray = (M_Transition *) MYrealloc((char*) PhTrArray,
			  (MaxPhTrNamen += NAMES_OFFSET) * sizeof(M_Transition));
		for (count = MaxPhTrNamen - NAMES_OFFSET; count < MaxPhTrNamen;)
			PhTrArray[count++] = (M_Transition) NULL;
	}

	PhTrArray[ident] = (*blocktype == 'T'? M_fast_insert_transition
					     : M_insert_phantom_transition)
				(net, ident, &tr_last);

	tr_last->x_pos = co->x;
	tr_last->y_pos = co->y;
	tr_last->flags = invisible;
	if (!name) sprintf(buffer,"%s%u", *blocktype == 'T'? "T" : "PT", ident);
	tr_last->charname = name? name : MYstrdup(buffer);
	tr_last->annotation = guard? guard : MYstrdup("tt");
	tr_last->label = (MultiSet) name2;
	tr_last->reference = reference;
	tr_last->meaning = meaning;
	tr_last->links = links;
	tr_last->blocklist = block;
	tr_last->corrtran = corrtran;
	if (eft||lft)
	{
	  tr_last->time_label=(Time_stamp*)MYmalloc(sizeof(Time_stamp));
	  tr_last->time_label->low=eft;
	  tr_last->time_label->high=lft;
	  eft=lft=0;
	}
	return 0;
}

int HL_insert_arc()
{
	static M_Edge arc_last = NULL;
	static int tp = 0, phantom = 0;
	int pl, tr, anz;
	M_Transition *arr;

	if (!swapped++)		/* oh no! ... pop! */
	{
		swap(arr,PhTrArray,TrArray);
		swap(tr,AnzPhTrNamen,AnzTrNamen);
		swap(tr,MaxPhTrNamen,MaxTrNamen);
	}

	if (isalpha(*blocktype))
	{
		arc_last = NULL;
		tp = strcmp(blocktype,"PT") && strcmp(blocktype,"PPT");
		phantom = strcmp(blocktype,"TP") && strcmp(blocktype,"PT");
		*blocktype = '\0';
	}
		
	pl = tp? co->y : co->x;
	tr = tp? co->x : co->y;
	arr = phantom? PhTrArray : TrArray;
	anz = phantom? AnzPhTrNamen : AnzTrNamen;

	if (!tr || (tr > anz) || !arr[tr])
		Error((errbuf, "arc: incorrect %stransition identifier",
			phantom? "phantom " : ""));
	if (!pl || (pl > AnzPlNamen) || !PlArray[pl] )
		PEPerror("arc: incorrect place identifier");

	(tp? (phantom? M_insert_PTP_edge : M_fast_insert_TP_edge)
	   : (phantom? M_insert_PPT_edge : M_fast_insert_PT_edge))
		(net, PlArray[pl]->numname, arr[tr]->numname,
		 &PlArray[pl], &arr[tr], &arc_last);

	arc_last->flags = invisible;
	arc_last->label = (VarMSet) guard;
	if (!guard) PEPerror("arc: missing arc annotation");
	return 0;
}

/*****************************************************************************/
/*****************************************************************************/

void ReadRefObject(FILE *infile, char ch, int *index, char **name, char **ref)
{
	static char buffer[1024];
	char *c = buffer;
	*c = ch;

	*index = 0;
	*name = buffer;
	*ref = NULL;
	
	if (isdigit(*c))
	{
		ungetc(*c,infile);
		*index=ReadNumber(infile);
		*name = NULL;
	}
	else
	{
		while (isalnum(*++c = getc(infile)));
		*c++ = '\0';
	}

	if (ReadCharComment(infile) != 'T') PEPerror("'T' expected");
	ReadCharComment(infile);

	*ref = c;
	while ((*c++ = getc(infile)) != '"' && !feof(infile));
	*--c = '\0';

	if (ReadCharComment(infile) != '\n')
		PEPerror("garbage characters after reference");
		
}

/*****************************************************************************/
/**************************** read_BPNHLref **********************************/
/*****************************************************************************/

void read_x_HL_ref(M_Net net, char *PEPfilename, char mode)
{
	register char	ch;
	char*		c;
	register FILE	*infile;
	int		limit,ident;
	M_Place		pl;
	M_Transition	tr;
	char		*name,*ref,*cmd;
	char		*filetype;
	char		**defptr, **blockptr, **refptr;

	NNinput_file = PEPfilename;
	NNinput_line = 1;

	/* Oeffne PEP-Datei */
	if (!(infile = fopen(PEPfilename, "r")))
	{
		fprintf(stderr, "ERROR: read_x_HLref(): could not open file "
				"'%s' for reading.\n", PEPfilename);
		exit(2);
	} /* if */

	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "PEP")) PEPerror("keyword `PEP' expected");

	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	filetype = (mode == REF_BPN2)? "References_BPN2_HL"
		  : (mode == REF_PFA)? "References_PFA_HL"
				     : "References_SDL_HL";
	if (strcmp(cmd, filetype))
	{
		Error((errbuf,"format identifier `%s' expected",filetype));
		exit(10);
	}

	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strncmp(cmd, "FORMAT_N", 8))
		PEPerror("keyword `FORMAT_N' or `FORMAT_N2' expected");

	/* Read in DEF settings */
	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "DEF")) PEPerror("keyword 'DEF' expected");
	ReadNewline(infile);

	defptr = (mode == REF_BPN2)? &(net->definfo)
		  : (mode == REF_PFA)? &(net->pfadefs) : &(net->sdldefs);
	limit = MBSIZE * 3/4;
	*defptr = MYmalloc(MBSIZE);
	c = *defptr;
		
	while ((*c = getc(infile)) != 'P' && *c != 'B' && *c != 'L' && c++)
	{
		while ((*c++ = getc(infile)) != '\n');
		if (c-*defptr > limit)
		{
			int offset = c - *defptr;
			limit += MBSIZE;
			*defptr = MYrealloc(*defptr,limit+MBSIZE*1/4);
			c = *defptr + offset;
		}
		NNinput_line++;
	}

	ungetc(*c,infile);
	cmd=ReadCmdToken(infile);
	*c = '\0';

	blockptr = (mode == REF_BPN2)? &(net->refblockinfo)
		  : (mode == REF_PFA)? &(net->pfablockinfo)
				     : &(net->sdlblockinfo);
	/* Read in BL settings */
	if (!strcmp(cmd, "BL"))
	{
		ReadNewline(infile);

		limit = MBSIZE * 3/4;
		*blockptr = MYmalloc(MBSIZE);
		c = *blockptr;
			
		while ((*c = getc(infile)) != 'P' && *c != 'L' && c++)
		{
			while ((*c++ = getc(infile)) != '\n');
			if (c-*blockptr > limit)
			{
			    int offset = c - *blockptr;
			    limit += MBSIZE;
			    *blockptr = MYrealloc(*blockptr,limit+MBSIZE*1/4);
			    c = *blockptr + offset;
			}
			NNinput_line++;
		}

		ungetc(*c,infile);
		cmd=ReadCmdToken(infile);
		*c = '\0';
	}

    /* Read Link symbols : we just ignore them **************************/
    if (!strcmp( cmd, "LS" ))
      {
        while (!feof(infile) && strcmp(cmd=ReadCmdToken(infile), "PL"))
          ReadNewline(infile);
        /* put back the last '\n' because of the ReadNewLine after
           strcmp(cmd,"PL")*/
        ungetc ('\n',infile);
      }; /* if ( strcmp( cmd, "LS" ) ) */

	/* Read in place nodes ***********************************************/
	if (strcmp(cmd, "PL"))
	  PEPerror("keyword 'PL' expected");
	ReadNewline(infile);
	pl = net->Places;
	while ((ch = ReadCharComment(infile)) && (ch != 'T'))
	{
		ReadRefObject(infile,ch,&ident,&name,&ref);
		if (!pl || (ident && ident != pl->numname) ||
		    (name && strcmp(name,pl->charname)))
		{
			for (pl = net->Places; pl; pl = pl->nextplace)
				if ((ident && ident == pl->numname) ||
				    (name && !strcmp(name,pl->charname)))
					break;
			if (!pl) PEPerror("invalid place identifier");
		}

		refptr = (mode == REF_BPN2)? &(pl->bpnref)
		       : (mode == REF_PFA)? &(pl->pfaref) : &(pl->sdlref);
	
		if (ref) *refptr = MYstrdup(ref);
		pl = pl->nextplace;
		if (!pl) pl = net->Places;
		NNinput_line++;
	}

	/* Read in transition nodes ******************************************/
	ungetc(ch, infile);
	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "TR")) PEPerror("keyword 'TR' expected");
	ReadNewline(infile);

	tr = net->Transitions;
	while ((ch = ReadCharComment(infile)) && (ch != 'P') && !feof(infile))
	{
		ReadRefObject(infile,ch,&ident,&name,&ref);
		if (!tr || (ident && ident != tr->numname) ||
		    (name && strcmp(name,tr->charname)))
		{
			for (tr = net->Transitions; tr; tr = tr->nexttrans)
				if ((ident && ident == tr->numname) ||
				    (name && !strcmp(name,tr->charname)))
					break;
			if (!tr) PEPerror("invalid transition identifier");
		}

		refptr = (mode == REF_BPN2)? &(tr->bpnref)
		       : (mode == REF_PFA)? &(tr->pfaref) : &(tr->sdlref);
	
		if (ref) *refptr = MYstrdup(ref);
		tr = tr->nexttrans;
		if (!tr) tr = net->Transitions;
		NNinput_line++;
	}

	/* Read in phantom transitions ***************************************/
	if (!feof(infile))
	{
		ungetc(ch, infile);
		cmd=ReadCmdToken(infile);
		if (strcmp(cmd, "PTR")) PEPerror("keyword `PTR' expected");
		ReadNewline(infile);

		tr = net->PhTransitions;
		while ((ch = ReadCharComment(infile)) != '\n' && !feof(infile))
		{
		    ReadRefObject(infile,ch,&ident,&name,&ref);
		    if (!tr || (ident && ident != tr->numname) ||
			    (name && strcmp(name,tr->charname)))
		    {
			for (tr = net->PhTransitions; tr; tr = tr->nexttrans)
				if ((ident && ident == tr->numname) ||
				    (name && !strcmp(name,tr->charname)))
					break;
			if (!tr) PEPerror("invalid phantom transition identifier");
		    }

		    refptr = (mode == REF_BPN2)? &(tr->bpnref)
			   : (mode == REF_PFA)? &(tr->pfaref) : &(tr->sdlref);
	
		    if (ref) *refptr = MYstrdup(ref);
		    tr = tr->nexttrans;
		    if (!tr) tr = net->PhTransitions;
		    NNinput_line++;
		}

	} /* if */

	fclose(infile);

} /* read_x_HL_ref() */


/*****************************************************************************/
/***************************** write_LLnet ***********************************/
/*****************************************************************************/
/*                                                  09.01.1995 Stefan Roemer */
/*****************************************************************************/

/* Liste der Stellen ausgeben */
void write_pllist(FILE *file, M_Place pl, int flags)
{
	for (; pl; pl = pl->nextplace)
	{
		fx("\"P%u\"", pl->numname);
		fxy("%u@%u", pl->x_pos, pl->y_pos);
		if (pl->label)
			fx("%c", (pl->label == ENTRY? 'e'
				: pl->label == EXIT? 'x' : ' '));
		if (pl->flags)
			fx("v%d", pl->flags);
		if (pl->init_marking)
			fx("M%u", (Cardinal32) pl->init_marking);
		if (pl->curr_marking)
			fx("m%u", (Cardinal32) pl->curr_marking);
		if (pl->meaning && *(pl->meaning))
			fx("b\"%s\"", pl->meaning);
		if (pl->reference && *(pl->reference))
			fx("R\"%s\"", pl->reference);
		if (pl->blocklist && *(pl->blocklist))
		    if (flags & FL_WRITE_BLOCKS) fx("u\"%s\"", pl->blocklist);
		f("\n");
	} /* for (pl ...) */
}

/* Liste der Transitionen ausgeben */
void write_trlist(FILE *file, M_Transition tr, int phantom, int flags)
{
	Cardinal32 i;
	for (; tr; tr = tr->nexttrans)
	{
	    if (!tr->dead && (!phantom || tr->partners->count))
	    {
		fx(phantom? "\"PT%u\"" : "\"T%u\"", tr->numname);
		fxy("%u@%u", tr->x_pos, tr->y_pos);
		if (tr->flags)
			fx("v%d", tr->flags);
		if (tr->meaning && *(tr->meaning))
			fx("b\"%s\"", tr->meaning);
	/* Printing of the links terms */
		if (tr->links && *(tr->links))
			fx("l\"%s\"", tr->links);
		if (tr->time_label)
		{
			fx("E%.0f", (tr->time_label->low)?tr->time_label->low:0);
			fx("L%.0f", (tr->time_label->high)?tr->time_label->high:0);
		}
		if (tr->reference && *(tr->reference))
			fx("R\"%s\"", tr->reference);
		if (tr->blocklist && *(tr->blocklist))
		    if (flags & FL_WRITE_BLOCKS) fx("u\"%s\"", tr->blocklist);
		if ((flags & FL_WRITE_PHANTOM) && tr->partners->count)
		{
			if (!phantom) f("S");
			f("P\"(");
			for (i=0; i<tr->partners->count; i++)
			{
				fx("%d",(MT_PlaceT(tr->partners,i))->numname);
				if (i < tr->partners->count-1) putc(',',file);
			}
			f(")\"");
		}
		f("\n");
	    }
	} /* for (tr ...) */
}

/* Liste der Transition->Stelle-Kanten ausgeben */
void write_tslist(FILE *file, M_Transition tr, int phantom)
{
	register M_TSEdge ts;
	register M_Place  pl;
	for (; tr; tr = tr->nexttrans)
	{
	    if (!tr->dead && (!phantom || tr->partners->count))
	    {
		for (ts = tr->postset; ts; ts = ts->nextplace)
		{
			pl = ts->place;
			fxy("%u<%u", tr->numname, pl->numname);
			if (ts->flags)
				fx("v%u", ts->flags);
			if (((Cardinal32) ts->label > 1)
			 && ((Cardinal32) ts->label < 1000))
				fx("w%u", (Cardinal32) ts->label);
			f("\n");
		} /* for */
	    }
	} /* for */
}

/* Liste der Stelle->Transition-Kanten ausgeben */
void write_stlist(FILE *file, M_Transition tr, int phantom)
{
	register M_STEdge st;
	register M_Place  pl;
	for (; tr; tr = tr->nexttrans)
	{
	    if (!tr->dead && (!phantom || tr->partners->count))
	    {
		for (st = tr->preset; st; st = st->nextplace)
		{
			pl = st->place;
			fxy("%u>%u", pl->numname, tr->numname);
			if (st->flags)
				fx("v%u", st->flags);
			if (((Cardinal32) st->label > 1)
			 && ((Cardinal32) st->label < 1000))
				fx("w%u", (Cardinal32) st->label);
			f("\n");
		} /* for */
	    }
	} /* for */
}

void write_LL_net(M_Net net, char *PEPfilename, int flags)
{
	FILE *file;

	if (!(file = fopen(PEPfilename, "w")))
		PEPerror("Could not open file for writing");

	f("PEP\n");
	fx("%s\n","PetriBox");
	fx("FORMAT_%s\n",
		(flags & (FL_WRITE_BLOCKS | FL_WRITE_PHANTOM))? "N2" : "N");

	f("DPL s7n10@-9t2\n");
	f("DTR s7n10@-9t2\n");
	f("DPT w1t1\n");

	if (flags & FL_WRITE_BLOCKS)
	{
		if (net->blockinfo)
		{
		  f("BL\n");
		  fputs(net->blockinfo,file);
		}
	}

	f("PL\n");
	write_pllist(file,net->Places,flags);

	f("TR\n");
	write_trlist(file,net->Transitions,0,flags);

	if (flags & FL_WRITE_PHANTOM)
	{
		f("PTR\n");
		write_trlist(file,net->PhTransitions,1,flags);
	}

	f("TP\n");
	write_tslist(file,net->Transitions,0);
	f("PT\n");
	write_stlist(file,net->Transitions,0);

	if (flags & FL_WRITE_PHANTOM)
	{
		f("PTP\n");
		write_tslist(file,net->PhTransitions,1);
		f("PPT\n");
		write_stlist(file,net->PhTransitions,1);
	}

	fclose(file);

} /* write_LLnet() */


/*****************************************************************************/
/**************************** write_HLLLref **********************************/
/*****************************************************************************/

void write_HL_LL_ref(M_Net net, char *PEPfilename, int flags)
{
	register M_Place      pl;
	register M_Transition tr;
	FILE                  *file;

	if (!(file = fopen(PEPfilename, "w")))
		PEPerror("Could not open file for writing");

	f("PEP\n");
	f("References_HL_LL\n");
	fx("FORMAT_%s\n", (flags & FL_WRITE_PHANTOM)? "N2" : "N");

	f("PL\n");
	for (pl = net->Places; pl; pl = pl->nextplace)
		fxy("P%u\"%s\"\n", pl->numname, pl->parentplace->charname);

	f("TR\n");
	for (tr = net->Transitions; tr; tr = tr->nexttrans)
		if (!tr->dead)
		    fxy("T%u\"%s\"\n",tr->numname,tr->parenttrans->charname);

	if (flags & FL_WRITE_PHANTOM)
	{
	    f("PTR\n");
	    for (tr = net->PhTransitions; tr; tr = tr->nexttrans)
		if (!tr->dead && tr->partners->count)
		    fxy("PT%u\"%s\"\n",tr->numname,tr->parenttrans->charname);
	}

	fclose(file);

} /* write_HLLLref */


/*****************************************************************************/
/*************************** write_BPNLLref **********************************/
/*****************************************************************************/

void write_x_LL_ref(M_Net net, char *PEPfilename, int flags, char mode)
{
	register M_Place      pl;
	register M_Transition tr;
	FILE                  *file;
	char		      *filetype, *defptr,
			      *blockptr, *refptr;

	switch (mode)
	{
	    case REF_BPN2:
		defptr = net->definfo; blockptr = net->refblockinfo;
		filetype = "References_BPN2_LL\n"; break;
	    case REF_PFA:
		defptr = net->pfadefs; blockptr = net->pfablockinfo;
		filetype = "References_PFA_LL\n"; break;
	    case REF_SDL:
		defptr = net->sdldefs; blockptr = net->sdlblockinfo;
		filetype = "References_SDL_LL\n"; break;
	    default:
		defptr = blockptr = filetype = NULL;
		break;	/* can't happen */
	}

	if (!(file = fopen(PEPfilename, "w")))
		PEPerror("Could not open file for writing");

	f("PEP\n");
	f(filetype);
	fx("FORMAT_%s\n",
		(flags & (FL_WRITE_BLOCKS | FL_WRITE_PHANTOM))? "N2" : "N");

	f("DEF\n");
	if (defptr) fputs(defptr,file);

	if (flags & FL_WRITE_BLOCKS)
	{
		if (blockptr)
		{
		  f("BL\n");
		  fputs(blockptr,file);
		}
	}

	f("PL\n");
	for (pl = net->Places; pl; pl = pl->nextplace)
	{
		refptr = (mode == REF_BPN2)? pl->bpnref
			: (mode == REF_PFA)? pl->pfaref : pl->sdlref;
		if (refptr) fxy("P%u T\"%s\"\n", pl->numname, refptr);
	}

	f("TR\n");
	for (tr = net->Transitions; tr; tr = tr->nexttrans)
	{
		refptr = (mode == REF_BPN2)? tr->bpnref
			: (mode == REF_PFA)? tr->pfaref : tr->sdlref;
		if (refptr && !tr->dead)
			fxy("T%u T\"%s\"\n", tr->numname, refptr);
	}

	if (flags & FL_WRITE_PHANTOM)
	{
	    f("PTR\n");
	    for (tr = net->PhTransitions; tr; tr = tr->nexttrans)
	    {
		refptr = (mode == REF_BPN2)? tr->bpnref
			: (mode == REF_PFA)? tr->pfaref : tr->sdlref;
		if (refptr && !tr->dead && tr->partners->count)
			fxy("PT%u T\"%s\"\n", tr->numname, refptr);
	    }
	}

	fclose(file);

} /* write_x_LL_ref */

/*****************************************************************************/
/* END OF FILE readHL.c                                                      */
/*****************************************************************************/

