/*****************************************************************************/
/* unfoldHL.c                                                                */
/*****************************************************************************/
/* Unfolding a High Level net into its corresponding Low Level net.          */
/*                                                             Stefan Roemer */
/*****************************************************************************/
/* TO DO:                                                                    */
/*        Optimieren der kombinatorischen Ermittlung moeglicher Bindungen.   */
/*	  Speicher wieder aufraeumen					    					 */
/*****************************************************************************/

/* Last modifications on 2nd Nov. 2001 by Cecile Bui Thanh */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <time.h>
#include "unfoldHL.h"
#include "multiset.h"
#include "expr.h"
#include "calc.h"
#include "readHL.h"
#include "mtupel.h"
#include "readlib.h"

/*****************************************************************************/

#define MOVE_TR_X 6
#define MOVE_TR_Y 6

/*****************************************************************************/

Integer  *V_Dummy = NULL;
MultiSet annovarset,
	 actionset,
         boundset;
Cardinal32 GLO_line = 0;


/*****************************************************************************/

typedef struct MarkArray_
{
	Integer count;
	Integer realsize;
	MTupel  mt;
} MarkArrayEntry, *MarkArray;


/*****************************************************************************/

static void usage(char *myname)
{
  fprintf(stderr,
    "\n%s -- Unfold high level net into the corresponding low level net.\n\n"

    "Usage: %s [options] <HLnetfile> [LLnetfile] [FileOptions]\n\n"

    "     Options:\n"
    "      -    to remove arcs with weight > 1\n"
    "      -b   to write block information\n"
    "      -p   to write phantom transitions\n\n"

    "     FileOptions:\n"
    "      -bh <filename>  specify an alternate BPN2->HL reference file\n"
    "      -bl <filename>  specify an alternate BPN2->LL reference file\n"
    "      -ph <filename>  specify an alternate PFA->HL reference file\n"
    "      -pl <filename>  specify an alternate PFA->LL reference file\n"
    "      -sh <filename>  specify an alternate SDL->HL reference file\n"
    "      -sl <filename>  specify an alternate SDL->LL reference file\n"
    "      -hl <filename>  specify an alternate HL->LL reference file\n\n"

    "If only HLnetfile is specified all other filenames will be assumed to\n"
    "be its basename plus appropriate extensions. If LLnetfile is given the\n"
    "names of the output files will be changed accordingly.\n\n"

    "Version 2.0.1 (20.02.2002)\n\n",
     myname, myname);

  exit(1);
} /* usage() */

/*****************************************************************************/

char errent[128];
void AnnoError(char *msg)
{
	fprintf(stderr, "Error in %s: %s\n", errent, msg);
	exit(1);
} /* AnnoError() */

/*****************************************************************************/

Integer* V_alloc(Cardinal size)
{
	register Integer * array;

	*(array = (Integer *) MYmalloc((size + 2) * sizeof(Integer))) = 0;
	*(array + 1) = size;

	return (array);

} /* V_alloc() */

/*****************************************************************************/

Integer* V_create_copy(Integer* s_array)
{
	Integer          * d_array_base;
	register Integer * d_array;
	Cardinal         count;

	count = *((Cardinal *) s_array++);
	*(d_array_base = d_array = (Integer *) MYmalloc((count + 1)
                                              * sizeof(Integer))) = count;
	while (count--)
		*++d_array = *++s_array;
    
	return (d_array_base);

} /* V_create_copy() */

/*****************************************************************************/

void V_insert_value(Integer **array, Integer value)
{
	register Integer *ptr, *move;
	register Cardinal len;

	if (**array)
	{
		ptr = (*array) + (**(Cardinal**)array) + 1;
		while ((ptr > (*array) + 1) && (*ptr > value))
			ptr--;

		if ((ptr == (*array) + 1) || (*ptr != value))
		{
			if (**array == (len = *((*(Cardinal**)array) + 1)))
			{
				ptr = (Integer *) ((int) ptr - (int) *array);
				*array = (Integer *) MYrealloc((char*) *array,
				  ((len += 200) + 2) * sizeof(Integer));
				*((*(Cardinal**)array) + 1) = len;
				ptr = (Integer *) ((int) ptr + (int) *array);
			} /* if */
			ptr++;
			move = (*array) + (**(Cardinal**)array) + 3;

			while (--move > ptr)
				*move = *(move - 1);

			*ptr = value;
			++**(Cardinal**)array;
		} /* if */
	} /* if */
	else
	{
		*((*array) + 2) = value;
		++**(Cardinal**)array;
	} /* else */

} /* V_insert_value() */

/*****************************************************************************/

void V_insert_interval(Integer **array, Integer value1, Integer value2)
{
	while (value1 <= value2)
		V_insert_value(array, value1++);

} /* V_insert_interval() */

/*****************************************************************************/

MPTupel leertyp;

void MPT_intersect(MPTupel m1, MPTupel m2)
{
	int i,k;
	if (!m2) m2 = leertyp;
	if (!m1->count)
	{ /* NOTE: whereas the name of this procedure says that it makes an
		intersection between the two MPTupels... what is done in this 
		loop is copying m2 into m1...*/
		for (i=0; i<m2->count; i++)
		{
			k = *MT_Place(m2,i);
			MPT_AddValue(m1,V_alloc(k));
			memcpy(MT_Place(m1,i),MT_Place(m2,i),
				sizeof(Integer) * (k+1));
		}
		return;
	}
	if (m1->count < 0) return;
	if (m1->count != m2->count)
	{
		m1->count *= -1;
		return;
	}
	for (i=0; i<m1->count; i++)
	{
	/* for each array in m1->place:
		m1->place[i] is replaced by his intersection with m2->place[i] */
		Integer *w,*p1,*p2;
		
		/* m1->place[i][0] is the number of elements in the set 
			represented by m1->place[i] */
		int count1 = *MT_Place(m1,i);
		int count2 = *MT_Place(m2,i);
		
		/* number of elements after intersection */
		int ncount1 = 0;
		w = p1 = MT_Place(m1,i) + 1;
		p2 = MT_Place(m2,i) + 1;
	
		while (count1 && count2)
		{ /* if pi is incremented, then counti should be decremented */
			if (*p1 == *p2 && p2++ && count1-- && ++ncount1)
				{
					*w++ = *p1++;
					count2--;
				}
			else if (*p1 < *p2 && count1--)
				p1++;
			else
				{
					p2++;
					count2--;
				}
		}

		*MT_Place(m1,i) = ncount1;
	}
}

/*****************************************************************************/

MarkArray Z_alloc(Cardinal size)
{
	register MarkArray array;

	(array = (MarkArray) MYmalloc((size+1) * sizeof(MarkArrayEntry))
								 )->count = 0;
	array->realsize = size;
	array->mt = NULL;

	return (array);

} /* Z_alloc() */

/*****************************************************************************/

MarkArray Z_create_copy(MarkArray s_array)
{
	MarkArray          d_array_base;
	register MarkArray d_array;
	Cardinal           count;

	count = s_array->count;
	(d_array_base = d_array = (MarkArray) MYmalloc((count + 1)
				 * sizeof(MarkArrayEntry)))->count = count;

	/* Von MTupels werden nur die Pointer kopiert */
	while (count--)
		*++d_array = *++s_array;

	return (d_array_base);

} /* Z_create_copy() */

/*****************************************************************************/

Integer Z_find(MarkArray array, MTupel mt)
{
	register MarkArray ptr;

	if (!array) return 0;
	if (array->count)
	{
		ptr = array + (Cardinal32) array->count;
		while (ptr > array && MT_Greater(ptr->mt,mt)) ptr--;
		if ((ptr > array) && MT_Equal(ptr->mt,mt))
			return ptr->count;
	}
	return 0;
}

/*****************************************************************************/

void Z_insert_value(MarkArray * array, Integer count, MTupel mt)
{

	register MarkArray ptr, move;
	register Cardinal  len;

	if ((*array)->count)
	{
		ptr = (*array) + (Cardinal32) (*array)->count;
		while (ptr > *array && MT_Greater(ptr->mt,mt)) ptr--;
		if ((ptr == *array) || !MT_Equal(ptr->mt,mt))
		{
			if ((*array)->count == (len = (*array)->realsize))
			{
				ptr = (MarkArray) ((int) ptr - (int) *array);
				*array = (MarkArray) MYrealloc((char*) *array,
				  ((len += 200) + 1) * sizeof(MarkArrayEntry));
				(*array)->realsize = len;
				ptr = (MarkArray) ((int) ptr + (int) *array);
			} /* if */
			ptr++;
			move = (*array) + (Cardinal32) (*array)->count + 2;
			while (--move > ptr)
				*move = *(move - 1);

			ptr->count = count;
			ptr->mt = mt;
			(*array)->count++;
		} /* if */
		else if (MT_Equal(ptr->mt,mt))
		ptr->count += count;
	} /* if */
	else
	{
		(ptr = *array)->count++;
		(++ptr)->count = count;
		ptr->mt = mt;
	} /* else */

} /* Z_insert_value() */

/*****************************************************************************/

#define GetOverSpace_()	while (*str && isspace(*str)) { str++; }
#define GetOverSpace()	while (**str && isspace(**str)) { (*str)++; }

/*****************************************************************************/
/*** rdig.h ******************************************************************/
/*****************************************************************************/

#define LBRACE '{'
#define RBRACE '}'

static Integer read_anyValue(char** str)
{
	if (isdigit(**str))
		return read_posint(str);
	if (**str == '-')
	{
		(*str)++;
		GetOverSpace();
		if (!isdigit(**str)) AnnoError("number expected after '-'");
		return -read_posint(str);
	}
	if (**str == 'f' && *(*str+1) == 'f')
	{
		*str += 2;
		GetOverSpace();
		return V_FALSE;
	}
	if (**str == 't' && *(*str+1) == 't')
	{
		*str += 2;
		GetOverSpace();
		return V_TRUE;
	}
	if (**str == 'd' && *(*str+1) == 'o' && *(*str+2) == 't')
	{
		*str += 3;
		GetOverSpace();
		return V_DOT;
	}
	AnnoError("unknown number");
	return 0;	/* to keep the compiler happy */
}

static MTupel hl2ll_read_MTupel(char** str)
{
        MTupel mt = MT_Create();

        if (**str == '(')
        {
                (*str)++;
                do {
                        GetOverSpace();
                        MT_AddValue(mt,read_anyValue(str));
                } while (**str == ',' && (*str)++);
                if (**str != ')')
                {
                        fprintf(stderr,"Parsing error in value:"
                                        " ')' expected\n");
                        exit(20);
                }
                (*str)++;
                GetOverSpace();
        }
        else
                MT_AddValue(mt,read_anyValue(str));

        return mt;
}


/*****************************************************************************/

MarkArray _markarray;

void convert_to_markarray(char** marktype)
{
	char* str;
	register int dimension;
	MTupel mt;
 
        if ((str = *marktype) && *str)
        {
            dimension = 0;
            _markarray->count = 0;
            GetOverSpace_();
            if (*(str++) != LBRACE)
            {
		sprintf(*marktype,"'%c' expected",LBRACE);
		AnnoError(*marktype);
            } /* if */
            do {
                 GetOverSpace_();
                 dimension = 1;
                 if (isdigit(*str))
		     dimension = read_posint(&str);

                 if (*str != ',' && *str != RBRACE)
                 {
                     if (*str == '*')
                     {
                         str++;
                         GetOverSpace_();
                     } /* if */
			
		     mt = hl2ll_read_MTupel(&str);
                     Z_insert_value(&_markarray, dimension, mt);
                 } /* if */
                 else
                 {
		     mt = MT_Create();
		     MT_AddValue(mt,dimension);
                     Z_insert_value(&_markarray, 1, mt);
                 } /* if */
            } while (*str == ',' && str++); 
            if (*(str++) != RBRACE)
            {
		sprintf(*marktype,"'%c' expected",RBRACE);
		AnnoError(*marktype);
            } /* if */
            free(*marktype);
            *marktype = (char*) Z_create_copy(_markarray);
        } /* if */
	else
	    *marktype = NULL;
}

/*****************************************************************************/

/*****************************************************************************/
/*** cprod.h *****************************************************************/
/*****************************************************************************/

Integer* _valarray;

MPTupel convert_to_cprod (char** annotation)
{
	char* str;
	int num,num2;
	MPTupel mt = NULL;
 
        if ((str = *annotation) && *str)
        {
	   mt = MPT_Create();
            do {
                 *_valarray = 0;
                 GetOverSpace_();
                 if (*(str++) != LBRACE)
                 {
		      sprintf(*annotation,"'%c' expected",LBRACE);
		      AnnoError(*annotation);
                 } /* if */
                 do {
                      GetOverSpace_();
		      num = read_anyValue(&str);
                      if ((*str == '.') && (*(str+1) == '.'))
                      {
                         str += 2;
                         GetOverSpace_();
			 num2 = read_anyValue(&str);
		         if (num > num2)
				 AnnoError("first number in range should be "
					   "smaller than the second");
                         V_insert_interval(&_valarray, num, num2);
		      }
                      else
                          V_insert_value(&_valarray, num);
                 } while ((*str == ',') && str++); 
                 if (*(str++) != RBRACE)
                 {
		     sprintf(*annotation,"'%c' expected",RBRACE);
		     AnnoError(*annotation);
                 } /* if */
                 GetOverSpace_();
		 MPT_AddValue(mt, V_create_copy(_valarray));
             } while (*str == 'X' && str++);
        }

	return mt;
}

/*****************************************************************************/

/*****************************************************************************/
/*** cplace.h ****************************************************************/
/*****************************************************************************/

char* val2string(Integer what)
{
	static char tmpstr[7]; /* -32768\0 */
	switch(what) {
		case V_DOT: return "dot";
		case V_FALSE: return "ff";
		case V_TRUE: return "tt";
		default:
			sprintf(tmpstr,"%d",what);
			return tmpstr;
	}
}

struct { int x,y; } dimvec[] = { {0,24}, {24,0}, {12,12} };

M_Place CreateNewPlace(M_Place pl, MTupel tp, MTupel pos, M_Net unfolding,int m)
{
	int i;
	static char tmpstr[500];
	static char foo[500];
	static M_Place upl = NULL;

	M_fast_insert_place(unfolding, unfolding->NumPlaces + 1, &upl);
	upl->reference = pl->reference;
	upl->blocklist = pl->blocklist;
	upl->label = pl->label;
	upl->flags = pl->flags;
	upl->parentplace = pl;

	upl->x_pos = pl->x_pos;
	upl->y_pos = pl->y_pos;
	for (i=0; i<pos->count; i++)
	{
		upl->x_pos += (MT_Place(pos,i)-1)*dimvec[i%3].x;
		upl->y_pos += (MT_Place(pos,i)-1)*dimvec[i%3].y;
	}

	*tmpstr = '\0';
	if (tp->count > 1) strcat(tmpstr,"(");
	for (i=0; i<tp->count; i++)
	{
		strcat(tmpstr,val2string(MT_Place(tp,i)));
		if (i < tp->count-1) strcat(tmpstr,",");
	}
	if (tp->count > 1) strcat(tmpstr,")");

	strcpy(foo,tmpstr);
	upl->meaning = pl->meaning? pl->meaning : "";
	sprintf(tmpstr,"%s=%s",upl->meaning,foo);
	upl->meaning = m? MYstrdup(tmpstr) : pl->meaning;

	sprintf(tmpstr,"{%s}",foo);
	upl->annotation = pl->type? MYstrdup(tmpstr) : NULL;

	if (pl->bpnref) sprintf(tmpstr,"%s;%s",pl->bpnref,foo);
	upl->bpnref = (pl->bpnref && *(pl->bpnref))? MYstrdup(tmpstr) : NULL;
	
	if (pl->pfaref) sprintf(tmpstr,"%s;%s",pl->pfaref,foo);
	upl->pfaref = (pl->pfaref && *(pl->pfaref))? MYstrdup(tmpstr) : NULL;
	
	if (pl->sdlref) sprintf(tmpstr,"%s;%s",pl->sdlref,foo);
	upl->sdlref = (pl->sdlref && *(pl->sdlref))? MYstrdup(tmpstr) : NULL;
	
	upl->init_marking
		= (char*)(Cardinal32) Z_find((MarkArray)pl->init_marking,tp);
	upl->curr_marking
		= (char*)(Cardinal32) Z_find((MarkArray)pl->curr_marking,tp);

	if (!pl->type && !pl->init_marking)
		 upl->init_marking = (char*)(pl->label == ENTRY);
	if (!pl->type && !pl->curr_marking)
		 upl->curr_marking = (char*)(pl->label == ENTRY);

	return upl;
}

void create_child_places(M_Net net, M_Net newnet)
{
	M_Place pl;
	MTupel worktp = MT_Create();
	MTupel dimtp = MT_Create();
	MTupel akttp = MT_Create();

	Integer* dottp = V_alloc(1);

	leertyp = MPT_Create();
	V_insert_value(&dottp,V_DOT);
	MPT_AddValue(leertyp,dottp);
		
	for (pl = net->Places ; pl ; pl = pl->nextplace)
	{
		int i,m,n;
		MPTupel mp = pl->type;

		if (!mp) mp = leertyp;

		worktp->count = dimtp->count = akttp->count = 0;
		for (i=0, n=1; i < mp->count; i++)
		{
			MT_AddValue(worktp,MT_Place(mp,i)[1]);
			MT_AddValue(dimtp,*MT_Place(mp,i));
			MT_AddValue(akttp,1);
			n *= *MT_Place(mp,i);
		}

		pl->children = MYmalloc(sizeof(M_Place)*n);

		for (m=--n;;n--)
		{
			pl->children[m-n] =
				 CreateNewPlace(pl,worktp,akttp,newnet,m);

			for (i=mp->count-1; 
				i>=0 && MT_Place(akttp,i)==MT_Place(dimtp,i);
				MT_Place(akttp,i) = 1,
				MT_Place(worktp,i) = MT_Place(mp,i)[1],
				i--
			   );
			if (i < 0) break;
			MT_Place(worktp,i) = MT_Place(mp,i)[++MT_Place(akttp,i)];
		}
	}
}

/*****************************************************************************/

/*****************************************************************************/
/*** cvarc.h *****************************************************************/
/*****************************************************************************/
void arc_parsevar(MTupel mt, char **str)
{
	static char anno[21];
	char *ptr = anno;
	int cnt = 1;

	if (isdigit(**str) || **str == '-')
	{
		MT_AddValue(mt,0);
		MT_AddValue(mt,read_anyValue(str));
		return;
	}

	if (!isalpha(*ptr = **str) && (*ptr != '@')
				   && (*ptr != '\'') && (*ptr != '_'))
		AnnoError("identifier or constant expected");
	ptr++;
	(*str)++;
	while (isalnum(*ptr = **str) || (*ptr == '_') || (*ptr == '@') ||
		 (*ptr == '\'') || (*ptr == '!') || (*ptr == '?'))
	{
		cnt++;
		ptr++;
		(*str)++;
	} /* while */
	*ptr = '\0';

	if (cnt == 2 && *anno == 't' && anno[1] == 't')
		cnt = V_TRUE;
	else if (cnt == 2 && *anno == 'f' && anno[1] == 'f')
		cnt = V_FALSE;
	else if (cnt == 3 && *anno == 'd' && anno[1] == 'o' && anno[2] == 't')
		cnt = V_DOT;
	else
		cnt = StrToAction(anno);

	MT_AddValue(mt,(cnt >= 0));
	MT_AddValue(mt,cnt);
	if (cnt >= 0) MSsum(actionset, MSset_single(mmSingle,cnt,1));
}

MTupel convert_arc_label(M_Edge st,int direction)
{
	char *str;
	int index, prenum, num;
	MTupel mt = MT_Create();
	VarMSet* label = &st->label;
	MPTupel type = st->place->type;

	sprintf(errent,"annotation of arc from %s to %s",
			direction? st->place->charname : st->trans->charname,
			direction? st->trans->charname : st->place->charname
	       );

	if (!(str = (char*) *label))
		AnnoError("annotation string is missing");

	*actionset = 0;
	GetOverSpace_();
	if (*str == LBRACE) str++;

	do {
	    GetOverSpace_();
	    num = 1;
	    if ((prenum = isdigit(*str)))
	    {
		num = read_posint(&str);
		if (*str == '*' && str++)
			GetOverSpace_();
	    } /* if */

	    index = mt->count;
	    MT_AddValue(mt,0);

	    if (prenum && (*str == ',' || !*str))
	    {
		MT_Place(mt,index) = 1;
		MT_AddValue(mt,1);
		MT_AddValue(mt,0);
		MT_AddValue(mt,num);
		if (type && type->count != 1)
		   AnnoError("inscription doesn't have the required dimension");
	    }
	    else
	    {
		MT_AddValue(mt,num);
		if (!(prenum = 1 - (*str == '(')) && str++)
		{
			do {
				prenum++;
				GetOverSpace_();
				arc_parsevar(mt,&str);
			} while (*str == ',' && str++);
			if (*str != ')')
				AnnoError("')' expected");
			if (prenum != type->count)
				AnnoError("inscription doesn't have the "
					  "required dimension");
			str++;
		}
		else
			arc_parsevar(mt,&str);

		GetOverSpace_();
		MT_Place(mt,index) = prenum;
	    }
		
	} while ((*str == ',') && str++);

	if (*str != RBRACE && *str != '\0')
		AnnoError("malformed inscription");

	free(*label);
	*label = (VarMSet) MScreate_copy(actionset);
	return mt;
}

/*****************************************************************************/
/* Phantom- <-> Synchronisationstransitions- (tolles Wort) Handling	     */
/*****************************************************************************/

void parse_corrlists(M_Net net)
{
	M_Transition tr, ptr, *TrArray;
	MTupel mt;
	int i = 0;

	for (tr = net->Transitions; tr; tr = tr->nexttrans)
		if (tr->numname > i)  i = tr->numname;

	TrArray = MYmalloc((i+1) * sizeof(M_Transition));

	for (tr = net->Transitions; tr; tr = tr->nexttrans)
		TrArray[tr->numname] = tr;

	for (ptr = net->PhTransitions; ptr; ptr = ptr->nexttrans)
	{
		ptr->partners = MPT_Create();
		if (ptr->corrtran)
		{
			sprintf(errent,"list of sync transitions of %s",
				ptr->charname);
			mt = hl2ll_read_MTupel(&ptr->corrtran);
			for (i=0; i<mt->count; i++)
				if ((tr = TrArray[MT_Place(mt,i)]))
					MPTT_AddValue(ptr->partners,tr);
			free(mt->place); free(mt);
		}
	}

	free(TrArray);
}
 

MTupel r_pmt;
M_Transition r_ptr, r_tr;

bool relate_arc(M_Edge arc, int flag)
{
	/* Es wird angenommen, dass eine Phantom-Kante mit hoechstens einer
	   Variablen beschriftet ist. */

	MultiSet mmptr;
	M_Edge st;
	int i;

	if (!arc) return flag? True : relate_arc(r_ptr->postset,1);
	if (!*(mmptr = (MultiSet) arc->label))
		return relate_arc(arc->nextplace,flag);

	if (*(mmptr+2))
		Error((errbuf,"more than one variable on inscription "
		      "of arc at phantom transition %s", r_ptr->charname));

	for (st = flag? r_tr->postset : r_tr->preset;
		st && arc->place != st->place; st = st->nextplace);

	if (!st)
		Error((errbuf,"%s is adjacent to %s, but %s is not",
		      r_ptr->charname, arc->place->charname, r_tr->charname));

	for (i = 0; i < r_ptr->vars->count; i++)
	    if (MT_Place(r_ptr->vars,i) == *mmptr)
		return MSis_in(MT_Place(r_pmt,i),(MultiSet) st->label)
			&& relate_arc(arc->nextplace,flag);

	MT_AddValue(r_ptr->vars,*mmptr);
	for (mmptr = (MultiSet) st->label; *mmptr; mmptr += 2)
	{
	    MT_AddValue(r_pmt,*mmptr);
	    if (relate_arc(arc->nextplace,flag))
	    {
		if (*(mmptr+2))
		   fprintf(stderr,"Warning: matching the variables of %s and %s"
			" is ambiguous\n", r_ptr->charname, r_tr->charname);
		return True;
	    }
	    r_pmt->count--;
	}

	r_ptr->vars->count--;
	return False;
}

void relate_variables(M_Net net)
{
	int i;

	for (r_ptr = net->PhTransitions; r_ptr; r_ptr = r_ptr->nexttrans)
	{
	    r_ptr->vars = MT_Create();
	    r_ptr->childvals = MPT_Create();
	    for (i = 0; i < r_ptr->partners->count; i++)
	    {
		r_tr = MT_PlaceT(r_ptr->partners,i);
		r_ptr->vars->count = 0;
		MPTT_AddValue(r_ptr->childvals, r_pmt = MT_Create());
		if (!relate_arc(r_ptr->preset,0))
			Error((errbuf,"cannot match variables of %s and %s",
			       r_ptr->charname, r_tr->charname));
	    }
	}
}

/*****************************************************************************/
/*** trans.h *****************************************************************/
/*****************************************************************************/

MPTupel *range;
MTupel  *currvalue;
MPTupel	 mptmp;

int *currindex, *maxindex;
Integer **value;
Integer **vrange;

void initvalues(MultiSet m, int *i, int *empty)
{
	MultiSet mmptr = m;
	while (*mmptr)
	{
		int j;
		MPTupel mpt = range[*mmptr];
		if (mpt->count < 0) *empty = 1;
		for (j=0; j<mpt->count; j++)
		{
			currindex[*i] = 1;
			maxindex[*i] = *MT_Place(mpt,j);
			if (!maxindex[*i]) *empty = 1;
			vrange[*i] = MT_Place(mpt,j);
			MT_AddValue(currvalue[*mmptr],MT_Place(mpt,j)[1]);
			(*i)++;
		}
		*i -= mpt->count;
		for (j=0; j<mpt->count; j++)
			value[(*i)++] = &MT_Place(currvalue[*mmptr],j);
		mmptr++; mmptr++;
	}
}

void maketypes(M_Edge st)
{
	int i,dim,count;
	Integer *curr = &MT_Place(st->vars,0);
	Integer *endp = curr + st->vars->count;
	Integer *tmp;

	MSunion(actionset, (MultiSet) st->label);
	while (curr < endp)
	{
		dim = *curr;
		curr += 2;
		for (i=0; i<dim; i++, curr++)
		    if (*curr++)
			if (dim == 1)
				MPT_intersect(range[*curr],st->place->type);
			else
			{
				MT_Place(mptmp,0) = MT_Place(st->place->type,i);
				MPT_intersect(range[*curr],mptmp);
			}
		    else
		    {
			for (count=*(tmp=MT_Place(st->place->type,i)); count--;)
				if (*(++tmp) == *curr) break;
			if (count < 0)
				Error((errbuf,"constant out of range on arc between "
				      "place %u and transition %u",
				      st->place->numname, st->trans->numname));
		    }
	}
}

void create_child_edges(M_Net net, M_Transition *utr, M_STEdge st, 
							int ptflag, int phantom)
{
	Integer *curr = &MT_Place(st->vars,0);
	Integer *endp = curr + st->vars->count;
	Integer *tmpptr;
	M_Place upl;
	M_Place pl = st->place;
	static M_STEdge ust = NULL;
	static M_TSEdge uts = NULL;
	int vdim, vnum;

	if (phantom) { ust = NULL; uts = NULL; }

	while (curr < endp)
	{
		vdim = *curr++;
		vnum = *curr++;

		if (!pl->type)
			upl = *pl->children;
		else
		{
			Integer v, *w;
			int dim, i, j, p = 0, wdim = 1;

			tmpptr = curr+2*(vdim-1);
			for (i=pl->type->count-1; i>=0; i--, tmpptr-=3)
			{
				if (*tmpptr++)
					v = MT_Place(currvalue[*tmpptr],
							(vdim==1)? i : 0);
				else
					v = *tmpptr;
				if (vdim == 1) tmpptr += 2;

				w = MT_Place(pl->type,i) + 1;
				dim = *(w-1);
				j = 0;

				while (*w++ != v) j++;
				p += wdim*j;
				wdim *= dim;
			}

			upl = pl->children[p];
		}

		if (ptflag)
		{
			(phantom? M_insert_PPT_edge : M_fast_insert_PT_edge)
			  (net, upl->numname, (*utr)->numname, &upl, utr, &ust);
			ust->flags = st->flags;
			ust->label = (VarMSet) ((Cardinal32) ust->label + vnum);
		}
		else
		{
			(phantom? M_insert_PTP_edge : M_fast_insert_TP_edge)
			  (net, upl->numname, (*utr)->numname, &upl, utr, &uts);
			uts->flags = st->flags;
			uts->label = (VarMSet) ((Cardinal32) uts->label + vnum);
		}

		curr += vdim;
		curr += vdim;

	}
}

void store_child_variables(M_Transition tr, M_Transition utr)
{
	MultiSet mmptr = actionset;
	MPTupel mpt = MPT_Create();
	int i;

	while (*mmptr)
	{
		MTupel dst = MT_Create();
		MTupel src = currvalue[*mmptr];

		for (i=0; i<src->count; i++) MT_AddValue(dst,MT_Place(src,i));
		MPTT_AddValue(mpt,dst);
		mmptr++; mmptr++;
	}

	MPTT_AddValue(tr->childvals,mpt);
}

void make_new_transition(M_Net unfolding, M_Transition tr,
			 M_Transition utr, int phantom)
{
	M_Edge arc;

	utr->flags = tr->flags;
	utr->x_pos = tr->x_pos+MOVE_TR_X*tr->children->count;
	utr->y_pos = tr->y_pos+MOVE_TR_Y*tr->children->count;
	utr->reference = tr->reference;
	utr->blocklist = tr->blocklist;
	utr->meaning = tr->meaning;
	utr->bpnref = tr->bpnref;
	utr->pfaref = tr->pfaref;
	utr->sdlref = tr->sdlref;
	utr->annotation = MYstrdup("TRUE");
	utr->parenttrans = tr;
	utr->partners = MPT_Create();
	if (tr->time_label)
	{
	  utr->time_label=(Time_stamp*)MYmalloc(sizeof(Time_stamp));
	  utr->time_label->low=tr->time_label->low;
	  utr->time_label->high=tr->time_label->high;
	}
	MPTT_AddValue(tr->children,utr);

	for (arc = tr->preset; arc; arc = arc->nextplace)
		create_child_edges(unfolding,&utr,arc,1,phantom);

	for (arc = tr->postset; arc; arc = arc->nextplace)
		create_child_edges(unfolding,&utr,arc,0,phantom);

}

void create_child_phantom_transitions(M_Net net, M_Net unfolding)
{
    register M_Transition ptr, tr;
    M_Transition cptr = NULL;
    int* xvar;
    int i,j,k,l;

    for (ptr = net->PhTransitions; ptr; ptr = ptr->nexttrans)
    {
	MPTupel xlat = ptr->childvals;
	ptr->childvals = MPT_Create();
	ptr->children = MPT_Create();

	xvar = MYmalloc(ptr->vars->count * sizeof(int));
	for (i = 0; i < ptr->partners->count; i++)
	{
	    MTupel mtx = MT_PlaceMT(xlat,i);
	    tr = MT_PlaceT(ptr->partners,i);

	    for (j = 0; j < ptr->vars->count; j++)
		for (k = 0, l = MT_Place(mtx,j);
		     l != MT_Place(tr->vars,xvar[j] = k++););

	    for (j = 0; j < tr->childvals->count; j++)
	    {
	        MPTupel sbind = MT_PlaceMPT(tr->childvals,j);
		M_Transition ctr = MT_PlaceT(tr->children,j);
		for (k = 0; k < ptr->childvals->count; k++)
		{
		    MPTupel pbind = MT_PlaceMPT(ptr->childvals,k);
		    cptr = MT_PlaceT(ptr->children,k);

		    for (l = 0; l < ptr->vars->count; l++)
			 if (MT_Compare(MT_PlaceMT(pbind,l),
					MT_PlaceMT(sbind,xvar[l]))) break;

		    if (l == ptr->vars->count) break;
		}
		if (k == ptr->childvals->count)
		{
		    MPTupel pbind = MPT_Create();

		    M_insert_phantom_transition(unfolding,
			unfolding->NumPhTransitions+1,&cptr);
		    for (l = 0; l < ptr->vars->count; l++)
			MPTT_AddValue(pbind, currvalue[MT_Place(ptr->vars,l)]
				= MT_PlaceMT(sbind,xvar[l]));
		    MPTT_AddValue(ptr->childvals,pbind);
		    make_new_transition(unfolding,ptr,cptr,1);
		}
		MPTT_AddValue(ctr->partners,cptr);
		MPTT_AddValue(cptr->partners,ctr);
	    }
	}
	free(xvar);
    }
}

void create_child_transitions(M_Net net, M_Net unfolding)
{
    MPTupel tmprange = MPT_Create();

    int i, j, acsize, bdsize;
    register MultiSet     mmptr;
    register M_STEdge     st;
    register M_TSEdge     ts;
    register M_Transition tr;
    M_Transition          utr = NULL;
    btree                 tree;
    Action                action;

    range = MYmalloc(MaxActions * sizeof(MPTupel));
    currvalue = MYmalloc(MaxActions * sizeof(MTupel));
    currindex = MYmalloc(MaxActions * sizeof(int));
    maxindex = MYmalloc(MaxActions * sizeof(int));
    value = MYmalloc(MaxActions * sizeof(Integer*));
    vrange = MYmalloc(MaxActions * sizeof(Integer*));

    for (i=0; i<MaxActions; range[i++] = MPT_Create());
    for (i=0; i<MaxActions; currvalue[i++] = MT_Create());

    mptmp = MPT_Create();
    MPT_AddValue(mptmp,NULL);

    for (tr = net->Transitions; tr; tr = tr->nexttrans)
    {
	int empty = 0;

	strptr = tr->annotation;
	*annovarset = *actionset = *boundset = 0;
	GLO_line = tr->numname;
	yyparse();

	for (st = tr->preset; st; st = st->nextplace) maketypes(st);
	for (ts = tr->postset; ts; ts = ts->nextplace) maketypes(ts);
	
	if ((tree = STree) && (tree->token == DOTDOT))
	{
		btree treedel;
		STree = tree->right;
		while ((tree = (treedel = tree)->left))
		{
		    btree rtree,tdel;

		    free(treedel);
		    action = (Action) (Cardinal32) tree->right->left;
		    rtree = tree->right->right;
		    free(tree->right);
		    while (rtree)
		    {
			MPT_AddValue(tmprange,(Integer*)rtree->left);
			rtree = (tdel=rtree)->right;
			free(tdel);
		    }
		    MPT_intersect(range[action],tmprange);

		    for (i=0; i<tmprange->count; i++)
			free(MT_Place(tmprange,i));
		    tmprange->count = 0;
		}  
		free(treedel);
		MSdifference(boundset,actionset);
	}  

	mmptr = annovarset;
	while (*mmptr)
	{
		if (!range[*mmptr]->count)
		{
			ActionToStr(*mmptr,tr->annotation);
			Error((errbuf,"Untyped variable %s in annotation of transition "
			      "%d.\n", tr->annotation, tr->numname));
		}
		mmptr++; mmptr++;
	}

	tr->children = MPT_Create();
	tr->vars = MT_Create();
	tr->childvals = MPT_Create();
	mmptr = actionset;
	while (*mmptr) MT_AddValue(tr->vars,*mmptr), mmptr++, mmptr++;

	acsize = 0;
	initvalues(actionset,&acsize,&empty);
	bdsize = acsize;
	initvalues(boundset,&bdsize,&empty);

	if (!empty) do {

	    do {
		div_by_zero = False;
		if (calc(STree) == V_TRUE && !div_by_zero)
		{
			M_fast_insert_transition(unfolding,
				unfolding->NumTransitions + 1, &utr);
			make_new_transition(unfolding,tr,utr,0);
			store_child_variables(tr,utr);
			break;
		} 
		else
		{
			for (i=bdsize-1; i>=acsize && currindex[i]==maxindex[i];
				currindex[i] = 1,
				*value[i] = vrange[i][1],
				i--
			);
			if (i>=acsize) *value[i] = vrange[i][++currindex[i]];
		}	
	    } while (i>=acsize);
		
	    for (i=bdsize-1; i>=acsize;
			currindex[i] = 1,
			*value[i] = vrange[i][1],
			i--
	   );

	    for (i=acsize-1; i>=0 && currindex[i]==maxindex[i];
			currindex[i] = 1,
			*value[i] = vrange[i][1],
			i--
	   );
	    if (i>=0) *value[i] = vrange[i][++currindex[i]];

	} while (i>=0);
 
	for (i=0; i<MaxActions; currvalue[i++]->count = 0);

	for (i=0; i<MaxActions; range[i++]->count = 0)
		for (j=0; j<abs(range[i]->count); j++)
			free(MT_Place(range[i],j));

	STree = NULL;

    }  

}

/*****************************************************************************/

/*****************************************************************************/

static M_Net unfold(M_Net net)
{

	M_Net                 unfolding;
	register M_Place      pl;
	register M_STEdge     st;
	register M_TSEdge     ts;

	if (!net) return NULL;

	InitAllSymbolTables();

	_valarray = V_alloc(200);
	_markarray = Z_alloc(200);
	V_Dummy = V_alloc(200);
	actionset = MSalloc(100);
	annovarset = MSalloc(100);
	boundset = MSalloc(50);

	unfolding = M_empty_net();

	unfolding->definfo = net->definfo;
	unfolding->blockinfo = net->blockinfo;
	unfolding->refblockinfo = net->refblockinfo;
	unfolding->pfadefs = net->pfadefs;
	unfolding->pfablockinfo = net->pfablockinfo;
	unfolding->sdldefs = net->sdldefs;
	unfolding->sdlblockinfo = net->sdlblockinfo;

	for (pl = net->Places; pl; pl = pl->nextplace)
	{
		sprintf(errent,"initial marking of place %u",pl->numname);
		convert_to_markarray(&(char*)(pl->init_marking));
		sprintf(errent,"current marking of place %u",pl->numname);
		convert_to_markarray(&(char*)(pl->curr_marking));
		sprintf(errent,"type of place %u",pl->numname);
		pl->type = convert_to_cprod(&pl->annotation);

		for (ts = pl->preset; ts; ts = ts->nexttrans)
			ts->vars = convert_arc_label(ts,0);
		for (st = pl->postset; st; st = st->nexttrans)
			st->vars = convert_arc_label(st,1);
		for (ts = pl->phpreset; ts; ts = ts->nexttrans)
			ts->vars = convert_arc_label(ts,0);
		for (st = pl->phpostset; st; st = st->nexttrans)
			st->vars = convert_arc_label(st,1);

	}

	parse_corrlists(net);
	relate_variables(net);

	create_child_places(net,unfolding);
	create_child_transitions(net,unfolding);
	create_child_phantom_transitions(net,unfolding);

	free(actionset);
	free(boundset);
	free(annovarset);

	free(_valarray);
	free(_markarray);

	return unfolding;

} /* unfold() */

void Mark_Arcs(M_Transition start)
{
	bool     remove_arcs = False;
	register M_Transition	tr;
	register M_STEdge	st;
	register M_TSEdge	ts;
	MPTupel			mpt;
	int			i,j;

	for (tr = start; tr; tr = tr->nexttrans)
	{
		remove_arcs = False;
		for (st = tr->preset; st && !remove_arcs; st = st->nextplace)
		    if (((Cardinal32) st->label > 1)
		     && ((Cardinal32) st->label < 1000))
				remove_arcs = True;

		for (ts = tr->postset; ts && !remove_arcs; ts = ts->nextplace)
		    if (((Cardinal32) ts->label > 1)
		     && ((Cardinal32) ts->label < 1000))
				remove_arcs = True;

		if (remove_arcs)
		{
		    tr->dead = True;
		    for (i = 0; i < tr->partners->count; i++)
		    {
			mpt = MT_PlaceT(tr->partners,i)->partners;
			for (j = 0; j < mpt->count; j++)
			    if (MT_PlaceT(mpt,j) == tr)
			    {
			      MT_PlaceT(mpt,j) = MT_PlaceT(mpt,mpt->count-1);
			      mpt->count--;
			    }
		    }
		    tr->partners->count = 0;
		}
	}
}

void set_numnames(M_Net net)
{
	int count;
	M_Place      pl;
	M_Transition tr;

	for (count = 0, pl = net->Places; pl; pl = pl->nextplace)
		pl->numname = ++count;
	for (count = 0, tr = net->Transitions; tr; tr = tr->nexttrans)
		if (!tr->dead) tr->numname = ++count;
	for (count = 0, tr = net->PhTransitions; tr; tr = tr->nexttrans)
		if (!tr->dead && tr->partners->count) tr->numname = ++count;

}

/*****************************************************************************/
/* Main program.                                                             */
/*****************************************************************************/
extern int yydebug;
int main(int argc, char **argv)
{

	int	  i;
	char    *source = NULL,
		*dest, *hlllref,
		*bpnhlref, *bpnllref,
		*pfahlref, *pfallref,
		*sdlhlref, *sdlllref;
	char    **dptr;
	char	tmpname[80], *idx;
	M_Net   net, unfolding;
	FILE    *bpnfile, *pfafile, *sdlfile;
	clock_t time;
	int flags = 0;
//yydebug=1;
	if (getenv("HL2LL_BLOCKS")) flags |= FL_WRITE_BLOCKS;
	if (getenv("HL2LL_PHANTOM")) flags |= FL_WRITE_PHANTOM;

	dptr = &source;
	for (i=1; i<argc; i++)
		if (!strcmp(argv[i],"-"))
			flags |= FL_REMOVE_ARCS;
		else if (!strcmp(argv[i],"-b"))
			flags |= FL_WRITE_BLOCKS;
		else if (!strcmp(argv[i],"-p"))
			flags |= FL_WRITE_PHANTOM;
		else if (!strcmp(argv[i],"-nb"))
			flags &= ~FL_WRITE_BLOCKS;
		else if (!strcmp(argv[i],"-np"))
			flags &= ~FL_WRITE_PHANTOM;
		else if (!strcmp(argv[i],"-hl"))
			dptr = &hlllref;
		else if (!strcmp(argv[i],"-bh"))
			dptr = &bpnhlref;
		else if (!strcmp(argv[i],"-bl"))
			dptr = &bpnllref;
		else if (!strcmp(argv[i],"-ph"))
			dptr = &pfahlref;
		else if (!strcmp(argv[i],"-pl"))
			dptr = &pfallref;
		else if (!strcmp(argv[i],"-sh"))
			dptr = &sdlhlref;
		else if (!strcmp(argv[i],"-sl"))
			dptr = &sdlllref;
		else
		{
			if (!dptr) usage(argv[0]);
			*dptr = argv[i];

			strcpy(tmpname,argv[i]);
			idx = strrchr(tmpname,'.');
			if (!idx) strcat(tmpname,".");
			idx = strrchr(tmpname,'.');

			if (dptr == &source)
			{
				strcpy(idx,".ll_net");
				dest = MYstrdup(tmpname);
				strcpy(idx,".bpn2_hl_ref");
				bpnhlref = MYstrdup(tmpname);
				strcpy(idx,".pfa_hl_ref");
				pfahlref = MYstrdup(tmpname);
				strcpy(idx,".sdl_hl_ref");
				sdlhlref = MYstrdup(tmpname);
			}
			if (dptr == &source || dptr == &dest)
			{
				strcpy(idx,".hl_ll_ref");
				hlllref = MYstrdup(tmpname);
				strcpy(idx,".bpn2_ll_ref");
				bpnllref = MYstrdup(tmpname);
				strcpy(idx,".pfa_ll_ref");
				pfallref = MYstrdup(tmpname);
				strcpy(idx,".sdl_ll_ref");
				sdlllref = MYstrdup(tmpname);
			}

			if (dptr == &source) dptr = &dest;
			else dptr = NULL;
		}

	if (!source) usage(argv[0]);
		
	if ((bpnfile = fopen(bpnhlref,"r"))) fclose(bpnfile);
	if ((pfafile = fopen(pfahlref,"r"))) fclose(pfafile);
	if ((sdlfile = fopen(sdlhlref,"r"))) fclose(sdlfile);

	if (!bpnfile && !pfafile && !sdlfile)
		fprintf(stderr,"Warning: neither %s nor %s nor %s found\n",
				bpnhlref,pfahlref,sdlhlref);

	InitMultiSets();

	time = clock();

	net = read_HL_net(source);
	if (bpnfile) read_x_HL_ref(net,bpnhlref,REF_BPN2);
	if (pfafile) read_x_HL_ref(net,pfahlref,REF_PFA);
	if (sdlfile) read_x_HL_ref(net,sdlhlref,REF_SDL);

	unfolding = unfold(net);
	if (flags & FL_REMOVE_ARCS)
	{
		Mark_Arcs(unfolding->Transitions);
		Mark_Arcs(unfolding->PhTransitions);
	}
	set_numnames(unfolding);

	write_LL_net(unfolding,dest,flags);
	write_HL_LL_ref(unfolding,hlllref,flags);
	if (bpnfile) write_x_LL_ref(unfolding,bpnllref,flags,REF_BPN2);
	if (pfafile) write_x_LL_ref(unfolding,pfallref,flags,REF_PFA);
	if (sdlfile) write_x_LL_ref(unfolding,sdlllref,flags,REF_SDL);

/*	printf("Time needed: %f sec.\n\n",(float)(clock()-time)/CLOCKS_PER_SEC);
*/
	M_Remove_net(&net);
//	free(dest);
//	free(bpnhlref);
//	free(pfahlref);
//	free(sdlhlref);
//	free(hlllref);
//	free(bpnllref);
//	free(pfallref);
//	free(sdlllref);
	free(mmDummy);
	free(mmSingle);
	free(V_Dummy);

	return 0;
}

/*****************************************************************************/
/* END OF FILE unfoldHL.c                                                    */
/*****************************************************************************/

