/*****************************/
/* mnet.h                    */
/* Burkhard Graves, 26.6.'93 */
/*****************************/

#ifndef __LPNET_H__
#define __LPNET_H__

#include "mtupel.h"
#include "readPEP.h"

#define EMPTY		0
#define ENTRY		1
#define EXIT_label	2

/**********/
/* Stelle */
/**********/

typedef struct M_PlElem_
{
    struct M_PlElem_ *prevplace,  /* Zur (doppelten) Verkettung in der	    */
		     *nextplace;  /*   Stellenliste.			    */
    struct M_Edge_   *preset;     /* Zeiger auf erste Kante des Vorbereichs */
    struct M_Edge_   *postset;    /*   bzw. des Nachbereichs.		    */
    Cardinal32	      numname;    /* Numerischer Name fuer div. Zwecke.	    */
    char	     *charname;	  /* Textueller Name: z.B. P19		    */
    char	      label;	  /* Entry/Exit/Internal		    */
    int		      invnum;	  /* Index der S-Inv, zu der pl gehoert	    */
    int		      index;	  /* Index innerhalb der S-Inv		    */
    MTupel	      edges;	  /* Verzeichnis der Tr (aus tr->index)	    */
    MTupel	      to;	  /* Zugehoerige Ziele (pl->index)	    */
} M_PlElem,*M_Place;

/**************/
/* Transition */
/**************/

typedef struct M_TrElem_
{
    struct M_TrElem_ *prevtrans,  /* Zur (doppelten) Verkettung in der	    */
		     *nexttrans;  /*   Transitionenliste.		    */
    struct M_Edge_   *preset;     /* Zeiger auf erste Kante des Vorbereichs */
    struct M_Edge_   *postset;    /*   bzw. des Nachbereichs.		    */
    Cardinal32	      numname;    /* Numerischer Name fuer div. Zwecke.	    */
    char	     *charname;	  /* Textueller Name: z.B. T42		    */
    int		      invcounter; /* wieviele S-Inv sind adjazent?	    */
    int		     *index;	  /* Index in jeder S-Inv		    */
} M_TrElem, *M_Transition;

/**********/
/* Kanten */
/**********/

typedef struct M_Edge_
{
    struct M_Edge_ *prevplace, /* Zur (doppelten, vertikalen) Verkettung   */
		   *nextplace, /*   in Bezug auf die Stellenliste.	   */
		   *prevtrans, /* Zur (doppelten, horizontalen) Verkettung */
		   *nexttrans; /*   in Bezug auf die Transitionenliste.    */
    M_Place	    place;     /* Zeiger auf den Ursprung		   */
    M_Transition    trans;     /*   bzw. auf das Ziel der Kante.	   */
} M_STElem,M_TSElem,*M_Edge,*M_STEdge,*M_TSEdge;

/*********************/
/* Kopf eines Netzes */
/*********************/

typedef struct
{
    M_Place	 Places,         /* Zeiger auf erste bzw. auf letzte Stelle */
		 LastPlace;      /*   in der Stellenliste.		    */
    M_Transition Transitions,    /* Zeiger auf erste bzw. auf letzte Trans. */
		 LastTransition; /*   in der Transitionenliste.		    */
    MPTupel	 invplist;	 /* Array der Platzliste der S-Invarianten  */
    MPTupel	 invtlist;	 /* Array der Platzliste der S-Invarianten  */
    Cardinal32	 NumPlaces,      /* Anzahl der Stellen,			    */
		 NumTransitions, /*        der Transitionen,		    */
		 NumInv;	 /*        der S-Invarianten		    */
    int		 numbridges;	 /* Anzahl Tr mit invcounter >= 2	    */
    char	 *type;		 /* HLNet oder PetriBox			    */

} M_NetHead,*M_Net;

/**********************************************************************/
/* Funktionen zur Allokation von Speicherplatz fuer einzelne Elemente */
/* eines Netzes; realisiert als Makros.				      */
/**********************************************************************/

/* Funktionalitaet: M_Net   M_allocHead(void)   */
/*                  M_Place M_allocPlace(void)  */
/*             M_Transition M_allocTrans(void)  */
/*                 M_STEdge M_allocSTEdge(void) */
/*                 M_TSEdge M_allocTSEdge(void) */

#define M_allocHead()   ( (M_Net) MYmalloc( sizeof( M_NetHead ) ) )
#define M_allocPlace()  ( (M_Place) MYmalloc( sizeof( M_PlElem ) ) )
#define M_allocTrans()  ( (M_Transition) MYmalloc( sizeof( M_TrElem ) ) )
#define M_allocSTEdge() ( (M_STEdge) MYmalloc( sizeof( M_STElem ) ) )
#define M_allocTSEdge() ( (M_TSEdge) MYmalloc( sizeof( M_TSElem ) ) )

#endif

/*****************************************************************************/

extern M_Net M_empty_net(void);
extern M_Place M_fast_insert_place(M_Net, Cardinal32, M_Place*);
extern M_Transition M_fast_insert_transition(M_Net, Cardinal32, M_Transition*);
extern M_TSEdge M_fast_insert_TP_edge(M_Net, Cardinal32, Cardinal32,
				      M_Place*, M_Transition*, M_TSEdge* );
extern M_STEdge M_fast_insert_PT_edge(M_Net, Cardinal32, Cardinal32,
				      M_Place*, M_Transition*, M_STEdge*);
extern M_Transition M_insert_phantom_transition(M_Net,Cardinal32,M_Transition*);
extern M_TSEdge M_insert_PTP_edge(M_Net, Cardinal32, Cardinal32,
				  M_Place*, M_Transition*, M_TSEdge*);
extern M_STEdge M_insert_PPT_edge(M_Net, Cardinal32, Cardinal32,
				  M_Place*, M_Transition*, M_STEdge*);

/*****************************************************************************/

/**********************/
/* END OF FILE mnet.h */
/**********************/
