/*****************************************************************************/
/* readHL.c                                                                  */
/*****************************************************************************/
/*                                  Stefan Roemer,  06.02.1995 - 06.02.1995  */
/*****************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "readHL.h"
#include "readPEP.h"
#include "readlib.h"

/*****************************************************************************/

#define NAMES_START 2000
#define NAMES_OFFSET 1000

/*****************************************************************************/
/***************************** read_HLnet ************************************/
/*****************************************************************************/
/* Neu eingefuegt (neues Format)       19.12.1994 - 09.01.1995 Stefan Roemer */
/*****************************************************************************/

M_Net		net;
M_Place		*PlArray;
M_Transition	*TrArray;

Cardinal32	AnzPlNamen, MaxPlNamen,
		AnzTrNamen, MaxTrNamen;

char buffer[10];
int  placecount;
int  transcount;
bool autonumbering;

char	 *name;
int	 ent_place,exit_place,ident;
t_coords *co;

t_dest place_dest[] =
	{ { '\'',&name },	/* Bezeichner		*/
	  { '"', &name },	/*     "		*/
	  { '0', &ident },	/* numerischer Name	*/
	  { 'e', &ent_place },	/* Entry-Stelle		*/
	  { 'x', &exit_place },	/* Exit-Stelle		*/
	  {  0 ,  0 } };
  
t_dest trans_dest[] =
	{ { '\'',&name },	/* Bezeichner		*/
	  { '"', &name },	/*     "		*/
	  { '0', &ident },	/* numerischer Name	*/
	  {  0 ,  0 } };
  
t_dest arc_dest[] =
	{ { '@', &co },		/* Quelle/Ziel		*/
	  {  0 ,  0 } };

int insert_place();
int insert_trans();
int insert_arc();

M_Net read_HLnet(char *PEPfilename)
{
	int count;

	t_blockdest netdest[] =
		{ { "PL",  insert_place, NULL, place_dest },
		  { "TR",  insert_trans, NULL, trans_dest },
		  { "TP",  insert_arc,   NULL, arc_dest },
		  { "PT",  insert_arc,   NULL, arc_dest },
		  { NULL, NULL, NULL, NULL } };

	PlArray = (M_Place*) MYmalloc((count = MaxPlNamen = MaxTrNamen
			= NAMES_START) * sizeof(M_Place));
	TrArray = (M_Transition*) MYmalloc(count * sizeof(M_Transition));
	AnzPlNamen = AnzTrNamen = 0;
	while (--count)
	{
		PlArray[count] = (M_Place) NULL;
		TrArray[count] = (M_Transition) NULL;
	}

	/* Initialize net */
	net = M_empty_net();

	/* Initialize blocks */
	placecount = transcount = 0;
	autonumbering = True;

	/* read the net */
	read_PEP_net(PEPfilename, netdest);
	net->type = filetype;

	free(PlArray);
	free(TrArray);
	return net;

}

int insert_place()
{
	static M_Place pl_last = NULL;

	placecount++;
	if (ident && ident != placecount) autonumbering = False;
	if (!ident && autonumbering) ident = placecount;
	if (!ident) Error((errbuf,"missing place identifier"));
		
	if (ident > AnzPlNamen)
		AnzPlNamen = ident;
	else if (PlArray[ident])
		Error((errbuf, "place identifier %d used twice", ident));

	while (AnzPlNamen >= MaxPlNamen)
	{
		int count;
		PlArray = (M_Place*) MYrealloc((char*) PlArray,
				(MaxPlNamen += NAMES_OFFSET) * sizeof(M_Place));
		for (count = MaxPlNamen - NAMES_OFFSET; count < MaxPlNamen;)
			PlArray[count++] = (M_Place) NULL;

	}

	PlArray[ident] = M_fast_insert_place(net,ident,&pl_last);
	pl_last->label = ent_place? ENTRY: exit_place? EXIT_label : EMPTY;
	if (!name) sprintf(buffer, "P%u", ident);
	pl_last->charname = name? name : MYstrdup(buffer);
	return 0;
}

#define swap(h,x1,x2) h=x1; x1=x2; x2=h;
int swapped = 0;

int insert_trans()
{
	int count;
	static M_Transition tr_last = NULL;

	if (!transcount++) autonumbering = True;
	if (ident && ident != transcount) autonumbering = False;
	if (!ident && autonumbering) ident = transcount;
	if (!ident) Error((errbuf, "missing transition identifier"));

	if (ident > AnzTrNamen)
		AnzTrNamen = ident;
	else if (TrArray[ident])
		Error((errbuf, "%stransition identifier %d used twice", 
			    *blocktype == 'T'? "" : "phantom ", ident));

	while (AnzTrNamen >= MaxTrNamen)
	{
		TrArray = (M_Transition *) MYrealloc((char*) TrArray,
			  (MaxTrNamen += NAMES_OFFSET) * sizeof(M_Transition));
		for (count = MaxTrNamen - NAMES_OFFSET; count < MaxTrNamen;)
			TrArray[count++] = (M_Transition) NULL;
	}

	TrArray[ident] = M_fast_insert_transition(net, ident, &tr_last);

	if (!name) sprintf(buffer,"%s%u", *blocktype == 'T'? "T" : "PT", ident);
	tr_last->charname = name? name : MYstrdup(buffer);
	return 0;
}

int insert_arc()
{
	static M_Edge arc_last = NULL;
	static int tp = 0;
	int pl, tr, anz;
	M_Transition *arr;

	if (isalpha(*blocktype))
	{
		arc_last = NULL;
		tp = !!strcmp(blocktype,"PT");
		*blocktype = '\0';
	}
		
	pl = tp? co->y : co->x;
	tr = tp? co->x : co->y;
	arr = TrArray;
	anz = AnzTrNamen;

	if (!tr || (tr > anz) || !arr[tr])
		Error((errbuf, "arc: incorrect transition identifier"));
	if (!pl || (pl > AnzPlNamen) || !PlArray[pl] )
		Error((errbuf, "arc: incorrect place identifier"));

	(tp? M_fast_insert_TP_edge : M_fast_insert_PT_edge)
		(net, PlArray[pl]->numname, arr[tr]->numname,
		 &PlArray[pl], &arr[tr], &arc_last);

	return 0;
}


/*****************************************************************************/

/*****************************************************************************/
/* END OF FILE readHL.c                                                      */
/*****************************************************************************/

