/********************************************************************/
/* Lib fuer Operationen zur Auswertung des Strukturbaumes     	    */
/* Datei		: eval.c				    */      
/* Autor		: Thomas Thielke                            */
/* letzte Aktualisierung: 22.08.1992                                */   
/********************************************************************/

/* Includes							*/
#include "eval.h"
#include "btree.h"
#include "error.h"
#include "util.h"
#include "load.h"
#include "mca.h"

/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: eval_tree(p1,p2,p3,p4)        	        	*/
/* Zweck   : Auswertung des Strukturbaumes  p1 fuer Matrix p2	*/
/*	     Rueckgabewert: Ergebnis der Auswertung (0,1,-1)	*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

int
eval_tree(root,imx,np,nt)

BTREE root;			/* Zeiger auf Strukturbaum	*/
IMX   imx;			/* Inzidenzmatrix		*/
int   np;			/* Anzahl Stellen		*/
int   nt;			/* Anzahl Transitionen		*/

{
   int val, val1, val2;

   if( root != NULL)
   {
     if( STREQ(root->info,"D") )
	return(-1); 
    
     val1 = eval_tree(root->left_child,imx,np,nt);
     val2 = eval_tree(root->right_child,imx,np,nt);

     if( STREQ(root->info,"A") )
     {
	if( val1<0   && val2<0) val = -1;    /* Wert unbekannt	*/
	else  val = val1 * val2;	     /* TRUE>0, FALSE=0	*/
     }
     else if( STREQ(root->info,"O") )
     {
	if( val1<0   && val2<0) val = -1;
	else if( val1>= 0 && val2>=0) val = val1 + val2;
	else if( val1<0   || val2<0)
	{
	  val =  val1 + val2;
	  if(val==0) val++;
	}
     }
     else if( STREQ(root->info,"N") )
     {
	if(val1 == 0) val=1;
	else if(val1 == 1) val=0;
	else if(val1 == -1) val=-1;
     }
     else 
        val=eval_node(root->info,imx,np,nt);
/*printf("\npost: %d  %d  %d   %s ",val,val1,val2,root->info); */ 
/* Vereinfache Baum und gib Speicher frei	*/
   if( val >= 0)
   {
     if(val>=1) root->info="W";
     if(val==0) root->info="F";
/*TTiii     free( (char *) root->left_child );
     free( (char *) root->right_child ); */
     root->left_child = root->right_child = NULL;
   } 
   if( val>=1 ) return(1);
   else	return(val);  
   } 
}



/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: eval_node(p1,p2,p3,p4)        	        	*/
/* Zweck   : Auswertung eines Blattknotens p1 im Strukturbaum	*/
/*	     fuer Matrix p2 (Transition,Stelle,WAHR,FALSCH) 	*/
/*	     Rueckgabewert: Ergebnis der Auswertung (0,1)	*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

int
eval_node(s,imx,np,nt)

char* s;			/* Wert des Blattknotens	*/
IMX     imx;			/* Inzidenzmatrix		*/
int      np;			/* Anzahl Stellen		*/
int      nt;			/* Anzahl Transitionen		*/

{
  int val; 			/* Rueckgabewert		*/ 
  int pt_num;			/* Nummer Stelle/Transition	*/
  int t_val;			/* Wert der Transitonsaussage	*/
  int pt_flag;			/* Flag fuer Stelle/Transition	*/
  int op_flag;			/* Typ des Operators der Tran. */
 
/* Unterformel WAHR oder FALSCH */
  if( STREQ(s,"W") ) return(1);
  if( STREQ(s,"F") ) return(0);

/* Transitionsstring in Komponenten zerkegen			*/
  get_ptval(s, &pt_flag, &pt_num, &op_flag, &t_val, np, nt);

/* Auswertung von Stellen/Transitonsaussagen			*/
    if( !pt_flag ) 	/* Stelle auswerten */
    {
      if( (int) imx[pt_num][0].element ) val=1;	/* Stelle hat Marke	*/
      else		      	         val=0;	/* Stelle ohne Marke	*/
    }
    else    		/* Transition auswerten */
    {
      if( op_flag == 1 )   /*<=*/
      {
         if( t_val >= 0 ) val=1;
         else 	 	val=0;
      }
      else if( op_flag == 2 )   /*>=*/
      {
  	  if( t_val <= 0 ) val=1;
	  else		 val=0;
      }
      else
      {
	mca_error(ERR8);
	exit(8);
      }
    }

    return(val);
  
}

