/********************************************************************/
/* Funktionen zur Erzeugung einer Inzidenzmatrix aus PEP-File	    */
/* Datei		: load.c				    */
/* Autor		: Thomas Thielke                            */
/* letzte Aktualisierung: 01.03.1995                          	    */   
/********************************************************************/

/* Includes							*/
#include "convertPEP.h"
#include "load.h"
#include "error.h"
#include "mca.h"

/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: load_net(p1,p2,p3)					*/
/* Zweck   : Einlesen der Netzspezikationsdatei p1 von GreatSPN	*/
/*	     und Erzeugen der entsprechenden Inzidenmatrix	*/
/*	     Rueckgabewert: Zeiger auf erzeugte Matrix		*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

IMX
load_net(source_file, np, nt)

char* source_file;		/* Name der Netzdatei	*/
int    *np;			/* Anzahl Stellen	*/
int    *nt;			/* Anzahl Transitionen	*/

{
  int  i,m;
  IMX  imx;			/* Inzidenzmatrix	*/
  LMPNet net;
  Place pl;
  Transition tr;
  STEdge st;
  TSEdge ts;
  /* BG
  int a,b; */

/* Eingabedatei vorbereiten/oeffnen				 */
 net = read_PEP( source_file );
 if(!net) {mca_error(ERR3);exit(3);}

/* Anzahl Marken, Stellen, Transitionen bestimmen */

/* Speicher fuer Inzidenzmatrix allokieren			*/
     imx = init_matrix(net->NumPlaces+1,net->NumTransitions+1);


     *np = net->NumPlaces;
     *nt = net->NumTransitions;

     plnamen = (char**) Malloc( (net->NumPlaces+1) * sizeof( char * ) );
     trnamen = (char**) Malloc( (net->NumTransitions+1) * sizeof( char * ) );
/* Markierung lesen, ablegen in imx[m][0]			*/
     pl = net->Places;
     m = 1;
     while ( pl )
       {
         pl->numname = m;
         plnamen[m] = strdup( pl->name );
         imx[m++][0].element = (int) pl->marks;
         pl = pl->nextplace;
       }

/* Transitionen lesen, Eintrag in Matrix			*/
     i = 1;
     tr = net->Transitions;
     while ( tr )
       {
         trnamen[i] = strdup( tr->name );
         tr->numname = i++;
         tr = tr->nexttrans;
       }

     pl = net->Places;
     while ( pl )
       {
         st = pl->postset;
         while ( st )
           {
             imx[pl->numname][st->trans->numname].element = -1;
             st = st->nexttrans;
           }
         ts = pl->preset;
         while ( ts )
           {
             if ( imx[pl->numname][ts->trans->numname].element )
               {
                 imx[pl->numname][ts->trans->numname].element = 0;
                 imx[pl->numname][ts->trans->numname].flag = 1;
               }
             else
               imx[pl->numname][ts->trans->numname].element = 1;
             ts = ts->nexttrans;
           }
         pl = pl->nextplace;
       }

     remove_net( &net );

/* Zeiger auf I-Matrix zurueckgegen	*/
     return(imx);
}

/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: init_matrix(p1,p2)					*/
/* Zweck   : Speicher fuer Inzidenzmatrix der Groesse p1xp2	*/
/*	     allokieren und Zeiger auf Matrix zurueckgeben 	*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

IMX
init_matrix(m,n)

int m;
int n;

{
  int  i,j;
  IMX  imx;


/* Speicher fuer Array bereitstellen */
  imx = (IMX) malloc( (unsigned) m*(sizeof(VECTOR)) );
  if(imx==NULL) mca_error(ERR1);

   for(i=0; i<m; i++)
      {
       imx[i] = (VECTOR) malloc( (unsigned) n*(sizeof(TUPEL)) );
       if(imx[i]==NULL) mca_error(ERR1);
      }

/* Array initialisieren  */
   for(i=0; i<m; i++)
     for(j=0; j<n; j++)
     {
	imx[i][j].element = 0;
	imx[i][j].flag    = 0;
     }

    /* BG Ausdrucken */
    /* printf("\nin init_matrix\n"); */
    /* print_matrix(imx,m+1,n+1); */

    return(imx);
}

/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: free_imatrix(head,n)					*/
/* Zweck   : Speicherfreigabe fuer Matrix mit Kopf head und Groesse n	*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

void
free_imatrix(head,n)

IMX  head;
int  n;

{
   
 int i;

   for(i=0; i<n; i++)
      {  
       free((char *) head[i]);
      }
   free((char *) head);
}

