/********************************************************************/
/* Berechnung fuer Modelchecker					    */
/* Datei		: mca1.c				    */
/* Autor		: Thomas Thielke			    */
/* letzte Aktualisierung: 07.04.1994				    */
/********************************************************************/

/* Includes							*/
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <locale.h>

#include "list.h"
#include "memlib.h"
#include "normal.h"
#include "load.h"
#include "error.h"
#include "eval.h"
#include "simplex.h"
#include "prep.h"
#include "stree.h"
#include "show.h"
#include "td2.h"
#include "util.h"
#include "mca.h" 

extern int yyparse(void);

/* #define  DEB 0			 Degugging-Modus aus	*/
#define  CLOCK 1 		/* Zeitmessung an	*/

int    changed = False; /* Kontr. der Schleife bei NF-Umformung	*/

void mca1(net_file,formula_file,out_file)

char *net_file;
char *formula_file;
FILE *out_file;

{

  int  np;		/* Anzahl von Stellen		*/
  int  nt;		/* Anzahl von Transitionen	*/

   int nd = 0;		/* Anzahl toter Transitionen	*/
   int m, n;  		/* Zeilen,Spalten der Inzidenzm.*/
   int m1, m2, m3;	/* Anzahl der Restriktionstypen	*/
   int l1, l2, l3;	/* Zeilenmerker Restriktionen	*/
   int icase;		/* Ergebnistyp des Simplex	*/
   int *izrov, *iposv;	/* Ergebnis des Simplex		*/
   int result=-1;	/* kein Ergebnis der Formel	*/

   LMX	  lmx;		/* Zeiger auf LP-Matrix		*/
   IMX    imx;		/* Zeiger auf Inzidenzmatrix	*/
   BTREE  btree;	/* Zeiger auf Binaerbaum	*/	
   BTREE  *info;	/* Adr. Unterformeln in NF	*/
   LINK   dlist;	/* Liste Adr. Unterformeln in NF*/

   long mca_time;       /*Zeitbedarf des MCA [ms]	*/
   extern FILE *yyin;	/* Name der Eingabe fuer Yacc	*/
			/* default: stdin		*/
extern BTREE SYNTREE;

/* Umlenken der Eingabe fuer Yacc			*/
yyin=fopen(formula_file,"r");
if(!yyin) {mca_error(ERR4);exit(4);}

/* Zeitmessung starten						*/
   if(CLOCK) mca_time=clock();


/* Laden des Netfiles *.net und Initialisierung der I-Matrix	*/
    imx = load_net(net_file, &np, &nt);

    /* BG in mca1.c nach dem Einlesen */
    /* printf("\nin mca1.c nach dem Einlesen\n\n"); */

    if(DEB) print_matrix(imx, np, nt);

    _np = np;
    _nt = nt;

/* Aufruf von LEX/YACC -> Lesen der Formel,Erzeugen Strukturbaum*/
  setlocale(LC_ALL,""); 
   yyparse();
/*
   btree = yyval.node_ptr;
*/
   btree = SYNTREE;

/* Ersetzen der abgeleiteten Operatoren in Formel 		*/
   replace(btree,np,nt);
if(DEB) print_tree(btree,0);

/* Auswertung der Formel, falls keine Modaloperatoren->Ergebnis	*/
   result = eval_tree(btree,imx,np,nt);
   if(DEB) printf("\nResultat vor Umformung: %d \n\n",result);

/* Bestimmung der Menge Td, schaltbarer Transitionen 		*/
   nd = determine_Td(imx, np, nt);

if(DEB)  printf("\n tote Transitionen: %d", nd);
/* Belegung der Matrix drucken 	*/
if(DEB){ printf("\n");
         print_matrix(imx, np, nt);}

/* NF erstellen und Modalop. entfernen mit Simplex		*/
  while( result < 0 )/*BEGINWHILE1*/
  { 
     /* Ersetzen konsekutiver NOTs in Formel & Normalform gen.  */
     do {
   	  changed = False;
	  replace_all(&btree);
        } while(changed);

if(DEB) { printf("\nStrukturbaum nach NF:\n");
         print_tree(btree,0);}

     /* List fuer Unterformeln, die bereits in Normalform sind	 */
     dlist = NULL;
     find_nf(&btree,0,&dlist); 

if(DEB){
	printf("\n Liste der Diamonds:");
	print_list(dlist);
	printf("\n");}

     /* Entfernung des Modaloperators D bei Unterformeln in NF	*/ 
     while( dlist != NULL && result<0)/*BEGINWHILE2*/
     {
        info = dlist->info;

       /* Anz. und Art(1,2,3) der Konjunktionen in Unterformel */   
	icase=0;
        m1 = m2 = m3 = 0;
        determine_conj(info, 0, &icase, &m1, &m2, &m3);

if(DEB){ printf("\n Konj.: %d %d %d fuer %p und %d",m1,m2,m3,info,icase);}

        /* LP-Matrix initialisieren				*/
	if(icase != -1)
	{
  /*       printf("Simplex vorbereiten...\n"); */
           l1 = np+1;
           l2 = np+1+m1;
           l3 = np+1+m1+m2;
	   m1+= np;
	   m3+= nd;
           m  = m1 + m2 + m3;
	   n  = nt ;
           lmx = init_lmx(imx,np,nt,m,n,info,&l1,&l2,&l3);

           /* Aufruf Simplex -> Bestimmung von Last_X		*/
           /* printf("Simplex ausfuehren...\n");*/
           izrov=ivector(1,n);
           iposv=ivector(1,m);
           simplx(lmx,m,n,m1,m2,m3,&icase,izrov,iposv); 
	}
        

/* BG !!!!!!! hiervor ist in btree alles OK nach mod_tree steht Bloedsinn drin */   
	/* neue Form des Strukturbaums erzeugen		*/
        mod_tree(icase,iposv,izrov,*info,lmx,m,n,nt);
    	/* Speicher fuer lmx freigeben				*/
	/* TTi  if(icase!=-1) free_lmatrix((lmx+1),m); */
       
/* Anzeigen der entstandenen Baumstruktur */ 
if(DEB) {  printf("\nBaumstruktur nach Simplex:\n");
           print_tree(btree,0);}
        /* Auswertung des Strukturbaums */
         result = eval_tree(btree,imx,np,nt);
	/* naechsten Modalop. in Liste betrachten		*/
        dlist = dlist->next;
     }/*ENDWHILE2*/
     /* Liste abgearbeitet, Speicher freigeben			*/
  /*   free(dlist); */
   }/*ENDWHILE1*/

  if(result)
	fprintf(out_file,"True\n");
  else
	fprintf(out_file,"False\n");
 
/* Zeitbedarf fuer Modelchecking anzeigen		*/
  if(CLOCK && DEB)
  {
    mca_time = clock() - mca_time;
    printf("Time used: %10.2f seconds",mca_time/1.0e6);
  }
 
 


/* Speicher fuer Strukturbaum  btree freigeben		*/
   free((char *) btree);	
/* Speicher fuer Inzidenzmatrix imx freigeben		*/
   free_imatrix(imx,np+1);	
/* offene Dateien schliessen				*/
  fclose(out_file);
  exit(0);


}/*ENDMCA1*/

