/********************************************************************/
/* Lib fuer Operationen im Binaer-Baum zur Herstellung der          */
/* Standardform 					            */
/* Datei		: normal.c				    */
/* Autor		: Thomas Thielke   	                    */
/* letzte Aktualisierung: 21.08.1992                                */   
/********************************************************************/

/* Includes							*/
#include <stdio.h>

#include "normal.h"
#include "memlib.h"
#include "list.h"
#include "util.h"
#include "mca.h"

extern int changed;		/* Flag, das Formel geaendert	*/
BTREE  last=NULL;	/* Var. fuer Suche der Unterformeln, die*/
			/* bereits in Normalform sind		*/

/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: replace(p1,p2,p3)         	        		*/
/* Zweck   : Entfernung aller abgeleiteten Operatoren aus	*/
/*	     Strukturbaum p1, sowie des =-Operators und des	*/
/*	     Sonderfalls "T>=0"					*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

void
replace(root,np,nt)

BTREE root;			/* Zeiger auf Baumknoten	*/
int   np;			/* Anzahl Stellen		*/
int   nt;			/* Anzahl Transitionen		*/

{
    if (root != NULL)
	{
	  if(STREQ(root-> info, "B"))
		kill_box(root);
	  if(STREQ(root->info, "->"))
		kill_implication(root); 
	  if(STREQ(root->info, "<->"))
		kill_equivalence(root);
	  if( (INSTR(root->info, 'T')) && 
	      (INSTR(root->info, '>')) )
		kill_zero(root,np,nt);
	  if( (INSTR(root->info, 'T')) &&
	      (strpbrk(root->info, "<>")==NULL) )
		kill_eq(root,np,nt);
		
 	  replace(root -> left_child,np,nt);
 	  replace(root -> right_child,np,nt);
  	}
}


/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: replace_all(p1)         	        		*/
/* Zweck   : Erstellen der Standarform des Strukturbaum p1	*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

void
replace_all(root)

BTREE *root;

{   
   if ( *root != NULL)
	{

	     if (STREQ((*root) -> info ,"N")
		 && STREQ((*root ) -> left_child -> info , "N"))
		{
		  *root = (*root) -> left_child -> left_child; 
		  changed = True;
		}
             if (STREQ((*root) -> info ,"D")
		 && STREQ((*root ) -> left_child -> info , "D"))
		{
 		  *root = (*root) -> left_child ; 
		  changed = True;
		}
	     if (STREQ((*root) -> info ,"N")
		 && STREQ((*root ) -> left_child -> info , "W"))
		{
		  *root = (*root) -> left_child; 
		  (*root)->info = "F";
		  changed = True;
		}
	     if (STREQ((*root) -> info ,"N")
		 && STREQ((*root ) -> left_child -> info , "F"))
		{
		  *root = (*root) -> left_child; 
		  (*root)->info = "W";
		  changed = True;
		}

	     if (STREQ((*root) -> info ,"N")
		 && STREQ((*root) -> left_child -> info , "A"))
 		  replace_NAFG(*root);

	     if (STREQ((*root) -> info ,"N")
		 && STREQ((*root) -> left_child -> info , "O"))
  		  replace_NOFG(*root);

	     if (STREQ((*root) -> info, "A") 
		 && STREQ((*root) -> right_child -> info, "O"))
		  replace_D1(*root);

	     if (STREQ((*root) -> info, "A") 
		 && STREQ((*root) -> left_child -> info, "O"))
		  replace_D2(*root);

	     if (STREQ((*root) -> info, "D") 
		 && STREQ((*root) -> left_child -> info, "O"))
		  replace_DO(*root);

 	     replace_all( &((*root) -> left_child) );
 	     replace_all( &((*root) -> right_child) );
  	}
}


/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: kill_box(p1)                           		*/
/* Zweck   : Entfernung des Box-Operators auf den p1 zeigt	*/
/*	     aus Formel    					*/
/*	     B-F  <==> N-D-N-F					*/	   
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
void
kill_box(node)

BTREE node;		/* Zeiger auf zu bearbeitenden Knoten 	*/

{
   BTREE left_ptr, right_ptr; 	/* temp. Variablen fuer Zeiger	*/

   left_ptr  = node -> left_child;  /* Speichern der Kinder	*/
   right_ptr = node -> right_child; /* des Box-Operators	*/

/* 1. Knoten zu NOT  aendern					*/
   node -> info		= "N";
   node -> right_child  = NULL; 
 
/* Knoten mit DIAMOND erzeugen					*/
   node = node -> left_child  = new_node("D", NULL, NULL);

/* 2. Knoten mit NOTerzeugen					*/  
   node ->left_child = new_node("N", left_ptr, right_ptr);
}

 

  
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: kill_implication(p1)                          	*/
/* Zweck   : Entfernung der Implikation, auf die p1 zeigt	*/
/*	     aus Formel    					*/
/*	     F->G  <==> (N-F)-O-(G)				*/	   
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

void
kill_implication(node)

BTREE node;     	 /* Zeiger auf zu bearbeitenden Knoten */

{

   BTREE left_ptr, right_ptr; 	/* temp. Variablen fuer Zeiger	*/

/* Speichern der Kindes von Implication 			*/
   left_ptr  = node -> left_child; 
   right_ptr = node -> right_child;  

/* OR Wurzel des Unterbaums abaendern				*/  
   node -> info        = "O";  
   
/*rechten Unterbaum anhaengen					*/
  node -> right_child = right_ptr;

/* NOT linken Unterbaum und gemerktes Kind anhaengen		*/
   node -> left_child  = new_node("N",left_ptr,NULL); 
}



/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: kill_equivalence(p1)                          	*/
/* Zweck   : Entfernung der Aequivalenz, auf die p1 zeigt    	*/
/*	     F<->G  <==> (F-A-G) -O- ((N-F)-A-(N-G))		*/	   
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

void
kill_equivalence(node)

BTREE node;    	 /*  Zeiger auf zu bearbeitenden Knoten */

{
   BTREE left_ptr, right_ptr;  	/* temp. Variablen fuer Zeiger 	*/
   BTREE r_node = node;
   BTREE sub_left, sub_right; /*Kopien rechter/linker Unterbaum */

/* Speichern der Kinder von equivalence				*/
   left_ptr  = node -> left_child;  
   right_ptr = node -> right_child; 


/* Wurzel aendern						 */
   node -> info  = "O";  
    
/* linker Unterbaum 						*/ 
   node -> left_child = new_node("A", left_ptr, right_ptr);

/* rechter Unterbaum						 */
   r_node = node -> right_child = new_node("A", NULL, NULL);

/* Kopien der Kinder von equ. erzeugen				 */
   sub_left  = copy_tree(left_ptr);
   sub_right = copy_tree(right_ptr);

/* Kopien in Unterbaum einhaengen 				*/ 
   r_node -> left_child  = new_node("N", sub_left, NULL);
   r_node -> right_child = new_node("N", sub_right, NULL);
}


/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: kill_zero(p1,p2,p3)          	               	*/
/* Zweck   : Ersetzen der Aussagen  p1 "T>=0" durch "W"	    	*/       
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

void
kill_zero(node,np,nt)

BTREE node;
int   np;
int   nt;

{
  int pt, num, op,  val;

/* Transitionsstring in Komponenten zerkegen			*/
  get_ptval(node->info, &pt, &num, &op, &val, np, nt);

/* Testen, ob Wert gleich 0 -> Aussage durch "W" ersetzen	*/
  if(val==0) node->info="W";
 
}


/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: kill_eq(p1,p2,p3)          	               	*/
/* Zweck   : Ersetzen der Aussagen "T=k" durch 	"T<=K"-A-"T>=k" */   
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

void
kill_eq(node,np,nt)

BTREE node;
int   np;
int   nt;

{
  int    pt, num, op,  val;
  char* s;

/* Transitionsstring in Komponenten zerkegen			*/
  get_ptval(node->info, &pt, &num, &op, &val, np, nt);

/* neue Knoten erzeugen						*/
  if(val==0)
  {
     node->info=gen_pt("T",num,"<=",val);
  }
  else
  {
    node->info="A";
    s=gen_pt("T",num,"<=",val);
    node->left_child=new_node(s,NULL,NULL);
    s=gen_pt("T",num,">=",val);
    node->right_child=new_node(s,NULL,NULL);
  }
}


/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: replace_NAFG(p1)          		       		*/
/* Zweck   : Entfernung der Teilformel auf die p1 zeigt, und NOT*/
/*           vor der Klammer und ein AND in der Klammer steht	*/
/*	     nach DeMorgan					*/
/*	     N-A-(F-G) <=> ((N-F)-O-(N-G))			*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* !!!!! Rueckgabe beim iterieren pruefen !!!!! */
/* Speicher fuer node -> left freigeben ????*/
	
void
replace_NAFG(node)

BTREE node;

{
   BTREE left_ptr, right_ptr;  	/* temp. Variablen fuer Zeiger 	*/

	changed = True;		/* Formel wurde geaendert 	*/

/* Speichern der Kinder von AND 				*/ 
	left_ptr  = node -> left_child  -> left_child; 
	right_ptr = node -> left_child -> right_child; 

	node -> info = "O";  	 /* NOT in OR umwandeln  */
	node -> left_child  = new_node("N", left_ptr, NULL);
	node -> right_child = new_node("N", right_ptr, NULL);
}





/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: replace_NOFG(p1) 	                		*/
/* Zweck   : Entfernung der Teilformel auf die p1 zeigt, bei und*/
/*           ein NOT vor der Klammer und OR in der Klammer steht*/
/*	     nach DeMorgan					*/
/*	     N-O-(F-G) <=> ((N-F)-A-(N-G))			*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

void
replace_NOFG(node)

BTREE node;

{
   BTREE left_ptr, right_ptr;  	/* temp. Variablen fuer Zeiger	*/  

	changed = True; 	/* Formel wurde geaendert 	*/

/* Speichern der Kinder von OR  				*/
	left_ptr  = node -> left_child  -> left_child; 
	right_ptr = node -> left_child -> right_child; 

	node -> info = "A"; 	 /* NOT in AND umwandeln */
	node -> left_child  = new_node("N", left_ptr, NULL);
	node -> right_child = new_node("N", right_ptr, NULL);	
}


/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: replace_D1(p1)                 			*/
/* Zweck   : Entfernung der Teilformel, auf die p1 zeigt und	*/
/*           ein Distributivgesetz D1 angewendet werden kann	*/
/*	     F-A-(G-O-H) <=> (F-A-G)-O-(G-A-H)			*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

void
replace_D1(node)   

BTREE node;

{
   BTREE left_ptr, right_ptr;	/* temp. Variablen fuer Zeiger	*/  

	changed = True;		/* Formel wurde geaendert	*/
 
/* Speichern der Kinder von OR					*/
	left_ptr  = node -> left_child;
	right_ptr = node -> right_child -> left_child;

/* Wurzel mit OR 						*/
	node -> info = "O";

/* neuer linker Teilbaum					*/
	node -> left_child = new_node("A", left_ptr, right_ptr);

/* neuer rechter Teilbaum 					*/
	node -> right_child -> info = "A";
	node -> right_child -> left_child = copy_tree(left_ptr);

}



/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: replace_D2(x) 	                		*/
/* Zweck   : Entfernung der Teilformeln auf die p1 zeigt und	*/
/*           ein Distributivgesetz D2 angewendet werden kann	*/
/*	     (F-O-G)-A-H <=> (F-A-G)-O-(G-A-H)			*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

void
replace_D2(node)   

BTREE node;

{
  BTREE left_ptr, right_ptr;	/* temp. Variablen fuer Zeiger	*/  

	changed = True; 	/* Formel wurde geaendert	*/

/* Speicher ben. Kinder von AND					*/
	left_ptr  = node -> left_child -> right_child;
	right_ptr = node -> right_child;

/* Wurzel mit OR 						*/
	node -> info = "O";

/* neuer linker Teilbaum					 */
	node -> left_child -> info = "A";
	node -> left_child -> right_child = copy_tree(right_ptr);

/* neuer rechter Teilbaum 					*/
	node -> right_child = new_node("A", left_ptr, right_ptr);

}


/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: replace_DO(x)                 			*/
/* Zweck   : Anwendung des Distributivgesetz auf Diamond-Op.	*/
/*	     D-(F-O-G) <=> (D-F)-O-(D-G)			*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

void
replace_DO(node)   

BTREE node;

{
   BTREE right_ptr;		/* temp. Variable fuer Zeiger	*/ 

	changed = True; 	/* Formel wurde geaendert 	*/

/* rechte Unterformel merken 					*/
	right_ptr = node -> left_child -> right_child;

/* Wurzel mit OR 						*/
	node -> info = "O";

/* neuer linker Teilbaum 					*/
	node -> left_child -> info = "D";
	node -> left_child -> right_child = NULL;

/*  neuer rechter Teilbaum 					*/
	node -> right_child = new_node("D", right_ptr, NULL);
}



/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: find_nf(p1,p2,p3)         	        		*/
/* Zweck   : Suche nach Unterformeln in Normalform im Baum p1	*/
/*	     Eintragen in Liste p3 sortiert nach Abstand von 	*/
/*	     der Wurzel des Strukturbaumes p2			*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

void
find_nf(root, level, list)
 
BTREE  *root;			/* Adresse des Knotens		*/
int    level;			/* Hoehe im Baum		*/
LINK   *list;			/* List gefundener UF in NF	*/

{
  if (*root != NULL)
	{
	   if(STREQ((*root)->info, "D"))
	      last = *root;

	   find_nf( &((*root)->left_child), level+1,list);
	   find_nf( &((*root)->right_child), level+1,list);

	   if( last==(*root) )
	     {
               if(DEB)
                 printf("\nAdresse root: %d %s %d",
	                  (int)root,(*root)->info,level);
		 insert_list(list,root,level); 
	         last  = NULL;	  
	     }	   
	}
}


