/****************************************************************/
/* Yacc-Definitionen						*/
/* author:         Thomas Thielke				*/
/* cd	      :    09.01.1992					*/
/* last update:    09.02.1994					*/   
/****************************************************************/

%{
#include <stdlib.h>

#include "btree.h"
#include "error.h"
#include "mca.h"

extern int yylex(void);
extern char* yytext;
void yyerror(char *s);

extern int _np;
int i;
static char name[MAXLEN];
BTREE SYNTREE;
%}

/* Definition des Typs der Symbole			*/
%union{  
        BTREE  node_ptr;
      }


/* Tokens entsprechen Terminalsymbolen der Grammatik 	*/        
%token  true false implication equivalence not
%token  sbox diamond or and
%token  place 

/* Startsymol der Grammatik */
%start  START

%type <node_ptr> EXPR START
 

/* Vorrang und Assoziativitaet der Operatoren */
%left	sbox diamond
%left	implication equivalence
%left	or and
%left	not
%nonassoc	false true place

%%
/* Aufbau des Sybtaxbaums der Formel			*/
START: EXPR
         {
           SYNTREE = $$ = $1;
         }
	;

EXPR:    '(' EXPR ')'
          { 
	     $$ = $2;
	  }
       | EXPR implication EXPR
          { 
             $$ = new_node("->", $1, $3); 
          }
       | EXPR equivalence EXPR
          {  
             $$ = new_node("<->", $1, $3);
          }
       | EXPR or EXPR
          {  
             $$ = new_node("O", $1, $3);
          }
       | EXPR and EXPR
          {  
             $$ = new_node("A", $1, $3);
          }
       | not EXPR
          {  
             $$ = new_node("N", $2, NULL);
          }
       | sbox EXPR
          {  
             $$ = new_node("B", $2, NULL);
          }
       | diamond EXPR
          {  
             $$ = new_node("D", $2, NULL);
          }
       | place
          {  
	     /* Konvertierung Stellenname -> Stellennummer	*/
	 	for(i=1; i<=_np; i++)  
		{
		   if(!strcmp( yytext, plnamen[i]))
		   {
			sprintf( name ,"S%u",i);
 		        $$ = new_node(name, NULL, NULL);
			break;
		   }
		   if(i==_np) {mca_error(ERR7);exit(7);}
		};
           }
       | true 
          {  
             $$ = new_node("W", NULL, NULL);
          }
       | false
          {  
             $$ = new_node("F", NULL, NULL);
          }
	;

%%


/* Fehlerroutine bei Syntaxfehlern				*/
void yyerror(char *s)
{
  mca_error(ERR2);
  mca_error(s);
  exit(2);
}

