/********************************************************************/
/* Lib fuer Operationen mit der Matrix zur Vorbereitung des    	    */
/* Simplex-Algorithmus					            */
/* Datei		: prep.c				    */
/* Autor		: Thomas Thielke                            */
/* letzte Aktualisierung: 10.09.1992                                */   
/********************************************************************/

/* Includes							*/
#include <stdio.h>

#include "prep.h"
#include "btree.h"
#include "show.h"
#include "util.h"
#include "load.h"
#include "error.h"
#include "mca.h"
#include "memlib.h"

/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: determine_conj(p1,p2,p3,p4,p5)          		*/
/* Zweck   : Anzahl der Konjuktionen in Unterformel p1 bestimmen*/
/*	     getrennt nach Art der Restriktion (s. Simplex)	*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

void 
determine_conj(root, not_flag, icase, l1,l2,l3)

BTREE   *root;
int     not_flag;
int	*icase;
int	*l1,*l2,*l3;

{
   char* s;

  if( (*root) != NULL )
  {
	s = (*root)->info;
	if( STREQ(s,"N") )
           not_flag = 1;

	/* Wenn eine Konj. falsch => Loesung Simplex -1	*/
	if( STREQ(s,"F") && !not_flag) *icase=-1;
	if( STREQ(s,"W") &&  not_flag) *icase=-1;

	/* Anzahlen der Restriktionstypen zaehlen	*/
	if( INSTR(s,'S') )
	      (*l3)++;

	if( INSTR(s,'T') )
	{
	     if( INSTR(s,'<') )
	     {
		if(not_flag) (*l2)++;
		else	     (*l1)++;
	     }
	     else if( INSTR(s,'>')  )
	     {
		if(not_flag) (*l1)++;
		else	     (*l2)++;
	     }    
	 }
         
      determine_conj( &((*root)->left_child),not_flag,icase,l1,l2,l3);
      determine_conj( &((*root)->right_child),not_flag,icase,l1,l2,l3);
   }
}



/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: init_lmx(p1,p2,p3,p4,p5,p6,p7,p8,p9)              	*/
/* Zweck   : Speicher fuer LP-Matrix reservieren und Matrix     */
/*	     mit Werten initialisieren				*/
/*	     Rueckgabewert: Zeiger auf neue LP-Matrix		*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

LMX
init_lmx(imx, np, nt, m, n, root,l1,l2,l3)

IMX   imx;			/* Adresse I-Matrix		*/
int   np;			/* Anzahl Stellen		*/
int   nt;			/* Anzahl Transitionen		*/
int   m;			/* Anzahl  Restriktionen	*/
int   n;			/* Anzahl Spalten 		*/
int   *l1,*l2,*l3;  		/* <=,=,>= - Restriktionen	*/
BTREE *root;			/* Zeiger auf Unterformeln in NF*/

{
   LMX lmx;			/* LP-Matrix			*/
   int i,j,k,l;			/* Laufvariablen		*/
   int l4;			/* groesste nicht belegte Zeile	*/

   /* Anzahl der Zeilen in LP-Matrix berechnen			*/
   /* (Anzahl Stellen + Restr.)+(Zeilen fuer Zielfkt. z,z' = 2)	*/		   m += 2;

   /* Anzahl der Spalten in LP-Matrix				*/
   /* Anzahl Transitionen + 1 Zeile fuer Konstanten b_i		*/
   n= nt+1;

   /* Startindex in Matrix fuer Eintrag nd-Bedingungen		*/
   l4 = m-1;

   /* Speicher LP-Matrix allokieren, Anfangsadressen je um 1 	*/ 		   /* dekrementieren,damit a[0] entspricht a[1] fuer Simplex	*/
   lmx = ((LMX) malloc( (unsigned) m*(sizeof(LVECTOR)) ) -1);
   if(lmx==NULL)
   {
      mca_error(ERR1);
      exit(1);
    }

   for(i=1; i<=m; i++)
      {
       lmx[i] = ((LVECTOR) malloc( (unsigned) n*(sizeof(double)) ) -1);
       if(lmx[i]==NULL) 
       {
          mca_error(ERR1);
          exit(1);
       }

      }

   /* LP-Matrix intialisieren:					*/
/*TTv   for(i=np+1; i<=m; i++)*/
	for(i=1; i<=m; i++)
	for(j=1; j<=n; j++)
	   lmx[i][j]=0.0;

if(DEB)
{
         /* BG nur zum Gucken */
         printf("\n\nBelegung der LP-Matrix 1:");
         for(i=1; i<=m; i++)
         {
            printf("\n  %d ",i);
            for(j=1; j<=n; j++)
 	    printf("%10.2f",lmx[i][j]);
         }        
}

   /* 1. Koeffizienten der Zielfunktion z in Matrix eintragen	*/
   /* BG geaendert: */
   /* lmx[1][1] = 0.0;
   for(i=1,j=2; i<n; i++,j++)
       lmx[1][j] = 1.0; */

   lmx[1][1] = 0.0;
   for(j=2; j<=n; j++)
       lmx[1][j] = 1.0;

if(DEB)
{
         /* BG nur zum Gucken */
         printf("\n\nBelegung der LP-Matrix 2:");
         for(i=1; i<=m; i++)
         {
            printf("\n  %d ",i);
            for(j=1; j<=n; j++)
 	    printf("%10.2f",lmx[i][j]);
         }        
}

   /* 1. Koeffizienten der Zielfunktion z in Matrix eintragen	*/

   /* 2. Restriktionen durch Netzstruktur in Matrix eintragen	*/
   /* und a_tr-Bedingungen vom Ende der Matrix beginnend	*/
   for(j=0,l=1; j<=nt; j++,l++)
   {
       for(i=1,k=2; i<=np; i++,k++)
	   lmx[k][l] = imx[i][j].element;
       if( imx[0][j].flag == 0  && j>0 )       
       {	
	    lmx[l4][l]=1.0;
	    l4--;
       }
   }
if(DEB){
         printf("\n\nBelegung der LP-Matrix 3:");
         for(i=1; i<=m; i++)
         {
            printf("\n  %d ",i);
            for(j=1; j<=n; j++)
 	    printf("%10.2f",lmx[i][j]);
         }        
        } 

if(DEB){ printf("\n neue Matrix: ");
         print_matrix(imx, np, nt);}


   /* Restriktionen durch Modalformel in lmx eintragen	 	*/
   find_subformulae(root,lmx, imx, n,  0, l1,l2,l3,np,nt);

/*Belegung der LP-Matrix drucken */
if(DEB){
         printf("\n\nBelegung der LP-Matrix 4:");
         for(i=1; i<=m; i++)
         {
            printf("\n  %d ",i);
            for(j=1; j<=n; j++)
 	    printf("%10.2f",lmx[i][j]);
         }        
        } 
   return(lmx);
}


/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: find_subformulae(p1,p2,p3,p4,p5,p6,p7,p8)         	*/
/* Zweck   : Konjunktionen der Unterformel p1 in Matrix p2	*/
/*	     eintragen						*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

void 
find_subformulae(root, lmx, imx, n, not_flag,l1,l2,l3,np,nt)

BTREE     *root;		/* Zeiger auf Knoten Unterformel*/
LMX       lmx;			/* LP-Matrix			*/
IMX       imx;			/* Inzidenzmatrix		*/
int	  n;			/* Anzahl Spalten in LP-Matrix	*/
int       not_flag;		/* Flag fuer neg. Konjunktion	*/
int	  *l1,*l2,*l3;		/* aktuelle naechste Zeile	*/
int	  np,nt;		/* Anz. Stellen/Transitionen	*/

{
   char* s;
   BTREE  *lr,*rr;

  if( (*root) != NULL )
   {

	s = (*root)->info;

	if( STREQ(s,"N") ) not_flag = 1;
           
 	if( INSTR(s,'S') || INSTR(s,'T') )
          add_restrictions(lmx, imx, s, n, not_flag,l1,l2,l3,np,nt);
 
	lr = &((*root)->left_child);
	rr = &((*root)->right_child);
	find_subformulae(lr,lmx,imx, n, not_flag,l1,l2,l3,np,nt);
	find_subformulae(rr,lmx,imx, n, not_flag,l1,l2,l3,np,nt);
   }
}




/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: add_restrictions(p1,p2,p3,p4,p5,p6,p7,p8)         	*/
/* Zweck   : Eintrag einer Restriktion fuer Konjunktion		*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

void 
add_restrictions(lmx, imx, s, n, not_flag, l1,l2,l3,np,nt)

LMX       lmx;		/* LP-Matrix			*/
IMX	  imx;		/* Inzidenzmatrix		*/
char* s;		/* Ablegen der Konjunktionen	*/
int	  n;		/* Anzahl Spalten in LP-Matrix	*/
int       not_flag;	/* 1, wenn Konjuktion negiert	*/
int       *l1,*l2,*l3;  /* <=,=,>= - Restriktionen	*/
int	  np,nt;	/* Anz. Stellen/Transitionen	*/

{
  int b_i;
  int j;
  int pt_num;		/* Nummer der Stelle/Transition	*/
  int t_val;		/* Wert einer Transitionsaussage*/
  int pt_flag;		/* 0,1   ==  Stelle,Transition	*/
  int op_flag;		/* 1,2,3 ==  <=,>=,=		*/
  int line;		/* in Zeile line ablegen	*/


/* Transitionsstring in Komponenten zerkegen			*/
  get_ptval(s, &pt_flag, &pt_num, &op_flag, &t_val, np, nt);


  /* Restriktion in LP-Matrix hinzufuegen	*/
  if( pt_flag )
  {
     if(not_flag)
     {
        if( op_flag == 1 )	
	  add_line(lmx, ++(*l2), pt_num, ++t_val);
        else if( op_flag == 2 )	
	  add_line(lmx, ++(*l1), pt_num, --t_val);
        else
	{
	  mca_error(ERR8);
	  exit(8);
	}
     }
     else
     {
        if( op_flag == 1 )
	  add_line(lmx, ++(*l1), pt_num, t_val);
        else if( op_flag == 2 )	
	  add_line(lmx, ++(*l2),pt_num,t_val);
        else  
	{
	  mca_error(ERR8);
	  exit(8);
	}
      }
   }
   else
   {
      (*l3)++; line=(*l3);
      if(not_flag) b_i = (-1.0)*(lmx[pt_num+1][1]);
      else         b_i = (-1.0)*(lmx[pt_num+1][1]) + 1.0;

      if( b_i >= 0 )
      {
	lmx[line][1] = b_i;
	for(j=2; j<=n; j++)
            lmx[line][j] = (-1.0)*lmx[pt_num+1][j]; 
      }
      else
      {
	lmx[line][1] = (-1.0)*b_i;
	for(j=2; j<=n; j++)
            lmx[line][j] = lmx[pt_num+1][j]; 	
      }
   }
}


							
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: add_line(p1,p2,p3,p4)              		*/
/* Zweck   : neue Zeile p2 bei Restriktionen , in LP-Matrix p1 	*/
/*	     hinzufuegen, wenn Transitionsrestriktion 		*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

void 
add_line( lmx, line, pt_num, t_val)

LMX  lmx;				/* LP-Matrix		*/		
int  line;				/* akt. Einfuegezeile	*/
int  pt_num;				/* Nr. der Transition	*/
int  t_val;				/* Wert der Transition	*/

{
  if(t_val >= 0)
  {
     lmx[line][1] = t_val;
     lmx[line][pt_num+1] = -1.0; 
  }
  else
	{
	  mca_error(ERR8);
	  exit(8);
	}
}

/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: free_lmatrix(head,n)                                       */
/* Zweck   : Speicherfreigabe fuer Matrix mit Kopf head und Groesse n   */
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
 
void
free_lmatrix(head,n)
 
LMX  head;
int  n;
 
{
  
 int i;

   for(i=0; i<n; i++)
      {  
       free((char *) head[i]);
      }  
   free((char *) head);
}

