/********************************************************************/
/* Lib fuer Operationen im Strukturbaum zum Eintrag der Loesung des */		/* Simplex-Verfahrens			    	     	            */
/* Datei		: stree.c				    */
/* Autor		: Thomas Thielke                            */
/* letzte Aktualisierung: 21.08.1992                           	    */   
/********************************************************************/

/* Includes							*/
#include <stdlib.h>
#include <stdio.h>

#include "stree.h"
#include "btree.h"
#include "util.h"
#include "prep.h"
#include "mca.h"
								
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: mod_tree(p1,p2,p3,p4,p5,p6,p7,p8)			*/
/* Zweck   : Ergebnis des Simplex (p1,p2,p3) in Strukturbaum p4	*/
/*	     eintragen 						*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

void
mod_tree(icase,iposv,izrov,sub_tree,lmx,m,n,nt)

int     icase;			/* Typ der Loesung des Simplex	*/
int     *iposv, *izrov;		/* Loesungsarrays des Simplex	*/
BTREE   sub_tree;		/* zu aendernder Unterbaum	*/
LMX     lmx;			/* LP-Ergebnismatrix		*/
int      m;			/* Anzahl Restriktionen		*/
int      n;			/* Anzahl Variablen		*/
int	 nt;			/* Anzahl der Transitionen	*/

{
   int   i;
   int   j=0;
   char* s;

 /* Ergebnis des Simplex in Strukturbaum eintragen */
   
      if (icase == 1)
      {
  	if(DEB){printf("\nunbounded objective function\n");}
	(*sub_tree).info = "W";
	free( (char *) (*sub_tree).left_child);
	free( (char *) (*sub_tree).right_child);
	(*sub_tree).left_child = NULL;
	(*sub_tree).right_child = NULL;
      }
      else if (icase == -1)
      {	
	if(DEB) printf("\nno solutions satisfy constraints given\n");
	(*sub_tree).info = "F";
	free( (char *) (*sub_tree).left_child);
	free( (char *) (*sub_tree).right_child);
	(*sub_tree).left_child = NULL;
	(*sub_tree).right_child = NULL;
      }
      else
      {	
	/* Variablen im Loesungsvektor	*/
	
	(*sub_tree).info = "A";
	free( (char *) (*sub_tree).left_child);
	free( (char *) (*sub_tree).right_child);
	(*sub_tree).left_child = NULL;
	(*sub_tree).right_child = NULL;
	
	for(i=2; i<=m+1; i++)
	{
	   if( iposv[i-1] <= n )
	   {
		j++;
		
	       /* Knoten erzeugen */
/*printf("\n Orginalvariable %d :  %10.2f ",iposv[i-1],lmx[i][1]);*/
		s=gen_pt("T",iposv[i-1],"<=", (int) lmx[i][1]);

		if( nt == 1)
		   (*sub_tree).info = s; 
		else /*BEGINELSE2*/
		{
	           if( j<nt-1)
		   {
		     sub_tree->left_child  = new_node(s,NULL,NULL);
	             sub_tree->right_child = new_node("A",NULL,NULL);
	             sub_tree = sub_tree -> right_child;
		   }
		   if( j== nt-1 )
		      sub_tree->left_child  = new_node(s,NULL,NULL);
		   if( j== nt )
		      sub_tree->right_child = new_node(s,NULL,NULL);
		}/*ENDELSE2*/
	   }/*ENDIF*/
	}/*ENDFOR*/

	/* Variablen mit Wert 0		*/
	for(i=1; i<=n; i++)
	{
	   if( izrov[i] <= n )
	   {
		j++;
/*printf("\n Orginalvariable %d :  %10.2f ",izrov[i-1],lmx[i][1]);*/
		s=gen_pt("T",izrov[i-1],"<=", 0);

		if( nt == 1)
		   (*sub_tree).info = s;
		else /*BEGINELSE2*/
		{
	           if( j<nt-1)
		   {
		     sub_tree->left_child  = new_node(s,NULL,NULL);
	             sub_tree->right_child = new_node("A",NULL,NULL);
	             sub_tree = sub_tree -> right_child;
		   }
		   if( j== nt-1 )
		      sub_tree->left_child  = new_node(s,NULL,NULL);
		   if( j== nt )
		      sub_tree->right_child = new_node(s,NULL,NULL);
		}/*ENDELSE2*/
	   }/*ENDIF*/
	}/*ENDFOR*/
      }/*ENDELSE1*/
}
