/*********************************************************************/
/* Lib fuer Bestimmung der Menge Td schaltbare Transitionen          */
/* Datei		: td.c					     */
/* Autor		: Thomas Thielke                             */
/* letzte Aktualisierung: 21.08.1992                                 */   
/*********************************************************************/

/* Includes							*/
#include <stdio.h>

#include "td2.h"
#include "memlib.h"
#include "load.h"
#include "error.h"
#include "mca.h"

#define BOUND 1			/* Grenze der Markenanzahl in	*/
				/* Stelle (=1, da Netz sicher)	*/

/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/
/* Funktion: determine_Td(p1,p2,p3)        	        	*/
/* Zweck   : Menge der nicht-schaltenden Transitionen bestimmen	*/
/*	     fuer Inzidenzmatrix p1				*/
/* 	     Rueckgabewert: Anzahl nichtschaltender Transitionen*/
/*FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF*/

int 
determine_Td(imx, np, nt)

IMX  imx;
int  np;
int  nt;

{

int  m;		/* Zaehler fuer Stellen, Transitionen 	*/
int  i;
int  run_flag = True;	/* Abbruch wenn == FALSE		*/
int  t_enabled;		/* aktivierte Transition		*/
int  *ma;		/* aktuelle Markierung			*/
int  active_trans;	/* aktive Transitionen			*/

/* Speicher fuer Anfangs-/Folgemarkierungen reservieren 	*/
/* -1 damit Kongruenz zwischen Adressen		       		*/
   ma =( (int *) malloc( (unsigned) np*(sizeof(int)) ) - 1);
   if(ma==NULL)
	{
	  mca_error(ERR1);
	  exit(1);
	}


/* Anfangsmarkierung einlesen 					*/
   for(m=1; m<=np; m++)
      ma[m] = imx[m][0].element;
   

if(DEB){
	 for(m=1; m<=np; m++)
     	    printf("\nIndex: %d Markierung: %d",m,ma[m]);
	}

/* Menge Td bestimmen						*/
   t_enabled    = 1;
   active_trans = 0;

   while( run_flag )/*BEGINWHILE1*/
   {
/* Testen, ob Transition t_enabled schalten kann	*/
	for(m=1; m<=np; m++)/*BEGINFOR1*/
	{
	  if( (imx[m][t_enabled].element == -1 && ma[m]<1) ||
	      (imx[m][t_enabled].element == 0  && 
	       imx[m][t_enabled].flag==1 && ma[m]<1) )
		break;
	   
	  if(m==np)  /* t_enabled kann schalten	 */
	  {
	    /* Transition t_enabled schalten				*/
	    for(i=1;i<=np;i++)
	    {
	       ma[i]  = ma[i] + imx[i][t_enabled].element;
	       if(ma[i]>BOUND)
		{
                  /* BG hier eingefuegt, da hier der fehler auf der inft
                        auftritt */
                  printf ("Error in td2.c\n");

		  mca_error(ERR6);
		  exit(6);
		}
	    }

	    /* Td als geschaltet markieren */
	    imx[0][t_enabled].flag = 1;

	    active_trans++;	/* Anzahl aktiver Transitionen erhoehen	*/
	    t_enabled=0;	/* Initialisierung fuer Suche nach	*/
				/* schaltbaren Transitionen       	*/
	   }
	}/*ENDFOR1*/

/* naechste noch nicht geschaltete Transition finden		*/
/* Abbruch, wenn t_enabled == nt, dann kann keine noch nicht 	*/
/* geschaltete Transition schalten				*/
	do
	{
	   if(t_enabled < nt)  t_enabled++;
	   else  run_flag = False; 	
    	}while(imx[0][t_enabled].flag && run_flag);
   }/*ENDWHILE1 */


/*TTii   free((char *) (ma+1) );i */
/* Anzahl nichtschaltender Transitionen zurueckgeben			*/
   return( nt - active_trans);
}


