/*****************************/
/* l1_mc.c                   */
/* Burkhard Graves, 26.10.94 */
/*****************************/

#include "l1_mc.h"
#include "basic_lf.h"
#include "mca.h"
#include "mc.tab.h"

/**************************************************************************/
/* Shiftet die Konfiguration 'configuration'                              */
/* bzgl. des virtuellen Cutoff-Events 'vce'.                              */
/**************************************************************************/

static Set *virt_finite_shift(VCEvent *vce,Set *configuration)
{
    unsigned element;
    Set *shifted,*c;
    IList *ptr;

    if (SetIsSubset(vce->causes_corr_event,configuration)) {
        shifted=SetDuplicate(vce->causes_event);
        c=SetDifference(SetDuplicate(configuration),vce->causes_corr_event);
        ptr=vce->ilist;
        element=0;
        for (;;) {
            element=SetGetElementNext(c,++element);
            if (!element)
                break;
            while (ptr && ptr->x<element)
                ptr=ptr->next;
            if (!ptr)
                break;
            if (ptr->x==element)
                SetInsertElement(shifted,ptr->y);
        }
        SetDelete(c);
    } else
        shifted=SetDuplicate(configuration);
    return shifted;
}

/**************************************************************************/
/* Shiftet die Konfiguration 'configuration'                              */
/* bzgl. des Cutoff-Events 'cutoff_event'.                                */
/**************************************************************************/

Set *FiniteShift(unsigned cutoff_event,Set *configuration)
{
    VCEvent *vce;
    unsigned element;
    Set *shifted,*c;
    IList *ptr;
    Info *info;

    if (cutoff_event>bprocess->num_events) {
        vce=bprocess->vcevents;
        assert(vce);
        while (vce->numname!=cutoff_event) {
            vce=vce->next;
            assert(vce);
        }
        return virt_finite_shift(vce,configuration);
    }

    info=(bprocess->events+cutoff_event)->info;
    assert(info);
    if (!info->corr_event || SetIsElement(configuration,info->corr_event)) {
        shifted=SetCreate();
        c=SetDuplicate(configuration);
        if (info->corr_event)
            SetDifference(c,info->causes_corr_event);
        ptr=info->ilist;
        element=0;
        for (;;) {
            element=SetGetElementNext(c,++element);
            if (!element)
                break;
            while (ptr && ptr->x<element)
                ptr=ptr->next;
            if (!ptr)
                break;
            if (ptr->x==element)
                SetInsertElement(shifted,ptr->y);
        }
        SetDelete(c);
        SetUnion(shifted,info->causes_event);
    } else
        shifted=SetDuplicate(configuration);
    return shifted;
}

/**************************************************************************/
/* Berechnet den Fixpunkt der Menge von Konfigurationen 'slist'.          */
/**************************************************************************/

SList *Fixpoint(SList *slist)
{
    unsigned cutoff_event;
    SList *ptr;
    VCEvent *vce;

    ptr=slist;
    if (!ptr)
        return ptr;
    while ((ptr=ptr->next)!=NULL) {
        vce=bprocess->vcevents;
        while (vce) {
                SListInsertElement(slist,virt_finite_shift(vce,ptr->set));
                vce=vce->next;
        }
        cutoff_event=0;
        for (;;) {
            cutoff_event=SetGetElementNext(cutoffs,++cutoff_event);
            if (!cutoff_event)
                break;
            SListInsertElement(slist,FiniteShift(cutoff_event,ptr->set));
        }
    }
    return slist;
}

/**************************************************************************/
/* Optimierte Version von 'BasicLast()' zur Berechnung von                */
/* Last(C1,gamma,C2), falls 'gamma' ueberhaupt keine Stellenaussagen      */
/* enthaelt, also gleich 'true' ist.                                      */
/* Wenn das 'flag' gesetzt ist, so werden Konfigurationen, die in die     */
/* Ergebnismenge eingefuegt werden, direkt aus 'C2' herausgenommen; ist   */
/* es nicht gesetzt, so werden entsprechende Duplikate der Konfigurationen*/
/* in 'C2' angefertigt, 'C2' wird also nicht veraendert.                  */
/**************************************************************************/

static SList *true_and(SList *C1,SList *C2,int flag)
{
    SList *slist;

#ifdef OUTPUT
    fprintf(yyout,"\n*true_and:\n");
    fprintf(yyout,"******C1:\n");
    SListDisplay(yyout,C1);
    fprintf(yyout,"******C2:\n");
    SListDisplay(yyout,C2);
#endif

    slist=NULL;
    if (C2) {
        while ((C2=C2->next)!=NULL) {
            if (!LowerEqual(C2->set,C1)) {

#ifdef OUTPUT
    fprintf(yyout,"\n-a-ZwErg: ");
    SetDisplay(yyout,C2->set);
#endif

                if (flag) {
                    if (slist) {
                        SListMaxInsert(slist,C2->set);
                        C2->set=NULL;
                    } else {
                        slist=SListCreate(C2->set);
                        C2->set=NULL;
                    }
                } else {
                    if (slist)
                        SListMaxInsert(slist,SetDuplicate(C2->set));
                    else
                        slist=SListCreate(SetDuplicate(C2->set));
                }
            }
        }
    }

#ifdef OUTPUT
    if (slist) {
        fprintf(yyout,"\n--a--Erg:\n");
        SListDisplay(yyout,slist);
    }
#endif

    return slist;
}

/**************************************************************************/
/* Liefert Last(C1,tree,C2).                                              */
/* Bemerkung: 'tree' ist die Wurzel eines Syntaxbaumes einer in           */
/*            Normalform befindlichen Formel der L1-Logik.                */
/**************************************************************************/

SList *Last(SList *C1,TreeNode *tree,SList *C2)
{
    TreeNode *sub_tree;
    SList *slist;
    SList *mem;

#ifdef OUTPUT
    fprintf(yyout,"\n****Last: ");
    TreeDisplay(yyout,tree);
    fprintf(yyout,"\n******C1:\n");
    SListDisplay(yyout,C1);
    fprintf(yyout,"******C2:\n");
    SListDisplay(yyout,C2);
#endif

    if ((sub_tree=DeliverDiamond(tree))==NULL) {
        slist=BasicLast(C1,tree,C2);
        SListDelete(C1);
        if (C2!=max_configs)
            SListDelete(C2);
        TreeDelete(tree);
        return slist;
    }
    if (sub_tree->info==DIAMOND) {
        slist=Fixpoint(Last(NULL,sub_tree->right,max_configs));

#ifdef OUTPUT
    fprintf(yyout,"\nFixpoint:\n");
    SListDisplay(yyout,slist);
#endif

        MyFree(sub_tree);
        mem=MaxTriangle(C2,slist);
        SListDelete(slist);
        if (C2!=max_configs)
            SListDelete(C2);
        C2=mem;

#ifdef OUTPUT
    fprintf(yyout,"\n   MaxTr:\n");
    SListDisplay(yyout,C2);
#endif

    } else {
        slist=Fixpoint(Last(NULL,sub_tree->right->right,max_configs));

#ifdef OUTPUT
    fprintf(yyout,"\nFixpoint:\n");
    SListDisplay(yyout,slist);
#endif

        MyFree(sub_tree->right);
        MyFree(sub_tree);
        C1=MaxUnion(C1,slist);
        SListDelete(slist);

#ifdef OUTPUT
    fprintf(yyout,"\n   MaxUn:\n");
    SListDisplay(yyout,C1);
#endif

    }
    if (tree!=sub_tree)
        return Last(C1,tree,C2);
    else {
        if (C2!=max_configs) {
            mem=true_and(C1,C2,1);
            SListDelete(C1);
            SListDelete(C2);
        } else {
            mem=true_and(C1,C2,0);
            SListDelete(C1);
        }
        return mem;
    }
}

/**************************************************************************/
/* Berechnung von Last({},tree,max_configs).                              */
/* Zurueckgeliefert wird der Wahrheitswert der Aussage                    */
/* 'Last({},tree,max_configs)!={}'.                                       */
/* Bemerkung: 'tree' ist die Wurzel eines Syntaxbaumes einer in           */
/*            Normalform befindlichen Formel der L1-Logik.                */
/**************************************************************************/

static void sub_fire(TreeNode *tree)
{
    FILE *out;

    out=OpenFile(path_file,"a");
    TreeDisplay(out,tree);
    fputc('\n',out);
    fclose(out);
}

static void fire(SList *slist)
{
    SList *ptr;
    Set *conf;
    FILE *out;
    unsigned max,mem;

    if (!path_file)
        return;
    out=OpenFile(path_file,"a");
    if ((ptr=slist)) {
        fprintf(out,"_SEQUENCE");
        if (multi)
            fprintf(out,"%u:\n",multi);
        else
            fprintf(out,":\n");
        if (flags&MIN_PATH) {
            max=bprocess->num_events+1;
            conf=NULL;
            while ((ptr=ptr->next)!=NULL) {
                mem=SetCountElements(ptr->set);
                if (mem<=max) {
                    max=mem;
                    conf=ptr->set;
                }
            }
            DisplayEventLabels(out,conf);
            fputc('\n',out);
        } else {
            while ((ptr=ptr->next)!=NULL) {
                DisplayEventLabels(out,ptr->set);
                fputc('\n',out);
            }
        }
    }
    fputc('\n',out);
    fclose(out);
}

int L1_ModelChecker(TreeNode *tree,unsigned mem)
{
    SList *slist;

    if (path_file)
        sub_fire(tree);

    while (mem) {
        tree=tree->right;
        --mem;
    }

    slist=Last(NULL,tree,max_configs);

#ifdef OUTPUT
    if (slist) {
        SListDisplay(yyout,slist);
        fprintf(yyout,"\nTRUE\n");
    } else
        fprintf(yyout,"\nFALSE\n");
#endif

    fire(slist);
    SListDelete(slist);
    return slist!=NULL;
}

/***********************/
/* END OF FILE l1_mc.c */
/***********************/
