/*****************************/
/* l2_mc.c                   */
/* Burkhard Graves, 26.10.94 */
/*****************************/

#include "l2_mc.h"
#include "basic_lf.h"
#include "mca.h"
#include "mc.tab.h"
#include "relation.h"

/********************/
/* Globale Variable */
/********************/

unsigned pl1,pl2;       /* fuer preorder()                  */
Set *help1,*help2;      /* fuer in_conflict()               */

SList *l2_configs=NULL; /* die L2-MC-Konfigurationen        */
                        /* (Initialisierung notwendig !)    */
unsigned anz_l2_c;      /* Anzahl der L2-MC-Konfigurationen */
                        /* (extern)                         */

/**************************************************************************/
/* Inverses shiften der Konfiguration 'configuration'                     */
/* bzgl. des virtuellen Cutoff-Events 'vce'.                              */
/* Konnte die Konfiguration komplett geshiftet werden, so wird eine '1'   */
/* zurueckgegeben, in 'configuration' steht das Ergebnis. Andernfalls     */
/* wird '0' zurueckgegeben, 'configuration' ist undefiniert.              */
/**************************************************************************/

static int virt_inv_finite_shift(VCEvent *vce,Set *configuration)
{
    Set *shifted;
    IList *ptr;

    shifted=SetDuplicate(vce->causes_corr_event);
    SetDifference(configuration,vce->causes_event);
    ptr=vce->ilist;
    while (ptr) {
        if (SetRemoveElement(configuration,ptr->y))
            SetInsertElement(shifted,ptr->x);
        ptr=ptr->next;
    }
    if (!SetIsEmpty(configuration)) {
        SetDelete(shifted);
        return 0;
    }
    SetSwap(configuration,shifted);
    SetDelete(shifted);
    return 1;
}

/**************************************************************************/
/* Inverses Shiften der Konfiguration 'configuration'                     */
/* bzgl. des Cutoff-Events 'cutoff_event'.                                */
/* Kann die Konfiguration nicht komplett geshiftet werden, so wird ein    */
/* virtuelles Cutoff-Event gesucht, welches 'cutoff_event' enthaelt       */
/* und Teilmenge von 'configuration' ist, so dass versucht werden kann,   */
/* 'configuration' bezueglich des virtuellen Cutoff-Events invers zu      */
/* shiften.                                                               */
/* Konnte die Konfiguration komplett geshiftet werden, so wird eine '1'   */
/* zurueckgegeben, in 'configuration' steht das Ergebnis. Andernfalls     */
/* wird '0' zurueckgegeben, 'configuration' ist undefiniert.              */
/**************************************************************************/

int ExtInvFiniteShift(unsigned cutoff_event,Set *configuration)
{
    Set *shifted,*mem;
    IList *ptr;
    Info *info;
    VCEvent *vce;

    info=(bprocess->events+cutoff_event)->info;
    assert(info);
    shifted=SetCreate();
    mem=SetDuplicate(configuration);
    SetDifference(configuration,info->causes_event);
    ptr=info->ilist;
    while (ptr) {
        if (SetRemoveElement(configuration,ptr->y))
            SetInsertElement(shifted,ptr->x);
        ptr=ptr->next;
    }
    if (SetIsEmpty(configuration)) {
        SetSwap(configuration,shifted);
        SetUnion(configuration,info->causes_corr_event);
        SetDelete(shifted);
        SetDelete(mem);
        return 1;
    }
    SetDelete(shifted);
    vce=bprocess->vcevents;
    while (vce) {
        if (SetIsElement(vce->causes_event,cutoff_event)
            && SetIsSubset(vce->causes_event,mem)) {
            shifted=SetCreate();
            SetCopy(configuration,mem);
            SetDifference(configuration,vce->causes_event);
            ptr=vce->ilist;
            while (ptr) {  
                if (SetRemoveElement(configuration,ptr->y))
                    SetInsertElement(shifted,ptr->x);
                ptr=ptr->next;
            }
            if (SetIsEmpty(configuration)) {
                SetSwap(configuration,shifted);
                SetUnion(configuration,vce->causes_corr_event);
                SetDelete(shifted);
                SetDelete(mem);
                return 1;
            } else {
                SetDelete(shifted);
            }
        }
        vce=vce->next;
    }
    SetDelete(mem);
    return 0;
}

/**************************************************************************/
/* Inverses Shiften der Konfiguration 'configuration'                     */
/* bzgl. des Cutoff-Events 'cutoff_event'.                                */
/* Konnte die Konfiguration komplett geshiftet werden, so wird eine '1'   */
/* zurueckgegeben, in 'configuration' steht das Ergebnis. Andernfalls     */
/* wird '0' zurueckgegeben, 'configuration' ist undefiniert.              */
/**************************************************************************/

int InvFiniteShift(unsigned cutoff_event,Set *configuration)
{
    Set *shifted;
    IList *ptr;
    Info *info;
    VCEvent *vce;

    if (cutoff_event>bprocess->num_events) {
        vce=bprocess->vcevents;
        assert(vce);
        while (vce->numname!=cutoff_event) {
            vce=vce->next;
            assert(vce);
        }
        return virt_inv_finite_shift(vce,configuration);
    }

    info=(bprocess->events+cutoff_event)->info;
    assert(info);
    shifted=SetDuplicate(info->causes_corr_event);
    SetDifference(configuration,info->causes_event);
    ptr=info->ilist;
    while (ptr) {
        if (SetRemoveElement(configuration,ptr->y))
            SetInsertElement(shifted,ptr->x);
        ptr=ptr->next;
    }
    if (!SetIsEmpty(configuration)) {
        SetDelete(shifted);
        return 0;
    }
    SetSwap(configuration,shifted);
    SetDelete(shifted);
    return 1;
}

/**************************************************************************/
/* Es wird versucht, die Konfiguration 'configuration' durch wiederholtes */
/* inverses Shiften (unter Invarianz der beschriebenen Markierung) von    */
/* cutoff-Events zu befreien.                                             */
/* Bei Erfolg wird eine '1' zurueckgegeben. Andernfalls wird eine '0'     */
/* zurueckgegeben, 'configuration' ist undefiniert.                       */
/**************************************************************************/

int CalcCorrConf(Set *configuration)
{
    register unsigned cutoff_event;
    register Set *mem;

    if (SetIsDisjunct(cutoffs,configuration))
        return 1;
    mem=SetDuplicate(configuration);
    cutoff_event=0;
    while ((cutoff_event=SetGetElementNext(cutoffs,++cutoff_event))!=0) {
        if (SetIsElement(configuration,cutoff_event)) {
            if (ExtInvFiniteShift(cutoff_event,configuration)) {
                if (SetIsDisjunct(cutoffs,configuration)) {
                    SetDelete(mem);
                    return 1;
                }
                SetCopy(mem,configuration);
                cutoff_event=0;
            } else {
                SetCopy(configuration,mem);
            }
        }
    }
    SetDelete(mem);
    return 0;
}

/**************************************************************************/
/* Ergaenzt die maximalen Konfigurationen um die fuer den L2-MC           */
/* notwendigen Konfigurationen.                                           */
/**************************************************************************/

static void complete_configs(void)
{
    register unsigned cutoff_event;

    l2_configs=SListHeadDuplicate(max_configs);
    anz_l2_c=anz_max_configs;
    cutoff_event=0;
    while ((cutoff_event=SetGetElementNext(cutoffs,++cutoff_event))!=0) {
        anz_l2_c+=SListInsertElement(l2_configs,SetDuplicate((bprocess->events+cutoff_event)->info->causes_event));
    }
    fprintf(stdout,"%u configuration(s) added.\n"
                  ,anz_l2_c-anz_max_configs);
}

/**************************************************************************/
/* Test, ob im Vorbereich der Geschichte des Ereignisses 'event' eine     */
/* Bedingung existiert, in deren Nachbereich ein Ereignis enthalten ist,  */
/* dass in der durch 'help2' bezeichneten' Konfiguration enthalten ist.   */
/**************************************************************************/

static int in_conflict(Event *event)
{
    Edge *edge_1;
    Edge *edge_2;

    if (SetInsertElement(help1,event->numname)) {
        edge_1=event->preset;
        while (edge_1) {
            edge_2=edge_1->condition->postset;
            while (edge_2) {
                if (SetIsElement(help2,edge_2->event->numname))
                    return 1;
                edge_2=edge_2->nextevent;
            }
            if (edge_1->condition->preset
                && in_conflict(edge_1->condition->preset->event))
                    return 1;
            edge_1=edge_1->nextcondition;
        }
    }
    return 0;
}

/**************************************************************************/
/* Berechnet die Art einer Kante zwischen zwei maximalen Konfigurationen, */
/* die miteinander in Relation stehen.                                    */
/**************************************************************************/

static Set *calc_s_edge(Set *d_1_0,Set *d2,unsigned *is_ok)
{
    register     Edge *edge;
    register      Set *s_edge,*cut;
    register unsigned event,condition;

    *is_ok=1;
    d2=SetDifference(SetDuplicate(d2),d_1_0);
    if (SetIsEmpty(d2)) {
        SetDelete(d2);
        return Mark(d_1_0);
    }
    cut=Cut(d_1_0);
    s_edge=SetCreate();
    condition=0;
    while ((condition=SetGetElementNext(cut,++condition))!=0) {
        SetInsertElement(s_edge,(bprocess->conditions+condition)
                                ->pl_numname);
        if (*is_ok) {
            if ((edge=(bprocess->conditions+condition)->postset)!=NULL) {
                do {
                    *is_ok=SetIsElement(d2,edge->event->numname);
                    edge=edge->nextevent;
                } while (edge && !*is_ok);
                if (!*is_ok) {
                    edge=(bprocess->conditions+condition)->postset;
                    help2=d2;
                    help1=SetCreate();
                    *is_ok=in_conflict(edge->event);
                    SetDelete(help1);
                    if (!*is_ok) {
                        *is_ok=!InPostCut(cut,edge->event->numname);
                    }
                }
            }
        }
    }
    event=0;
    while ((event=SetGetElementNext(d2,++event))!=0) {
        edge=(bprocess->events+event)->postset;
        while (edge) {
            SetInsertElement(s_edge,edge->condition->pl_numname);
            edge=edge->nextcondition;
        }
    }
    SetDelete(cut);
    SetDelete(d2);
    return s_edge;
}

/**************************************************************************/
/* Berechnet die Relation (genauer gesagt: alle Relationen) auf den       */
/* maximalen Konfigurationen.                                             */
/**************************************************************************/

void CalcRelations(void)
{
    register SList *ptr1,*ptr2;
    register Set *configuration,*edges;
    register unsigned i,j;
        auto unsigned is_ok;

    complete_configs();
    RelInit();
    i=0;
    ptr1=l2_configs;
    while ((ptr1=ptr1->next)!=NULL) {
        CalcCorrConf(configuration=SetDuplicate(ptr1->set));

        if (flags&SHOW_REL) {
            fprintf(stdout,"\n%u: {",i);
            SetDisplay(stdout,ptr1->set);
            fprintf(stdout,"} , {");
            SetDisplay(stdout,configuration);
            fprintf(stdout,"}\n");
        }

        j=0;
        ptr2=l2_configs;
        while ((ptr2=ptr2->next)!=NULL) {
            if (SetIsSubset(configuration,ptr2->set)) {
                edges=calc_s_edge(configuration,ptr2->set,&is_ok);
                RelAddKante(i,j,is_ok,edges);
            }
            ++j;
        }
        SetDelete(configuration);
        ++i;
    }

    if (flags&SHOW_REL)
        RelDisplay(stdout);
}

/**************************************************************************/
/* Test, ob configuration_1 <= {configuration_2}.                         */
/* Bemerkung: Optimierte Version von 'LowerEqual()' in 'mca.c'.           */
/**************************************************************************/

static int lower_equal(Set *configuration_1,Set *configuration_2)
{
    if (!configuration_1)
        return 1;
    if (!configuration_2)
        return 0;
    return SetIsSubset(configuration_1,configuration_2);
}

/**************************************************************************/
/* Berechnung der Gueltigkeit einer Formel des Typs                       */
/* E ^ ( S1 * ^ S2 )                                                      */
/**************************************************************************/

static int typ1(void)
{
    SList *ptr;
    Set *set1,*set2;
    unsigned count;
    int erg;

    erg=0;
    count=0;
    ptr=l2_configs;
    while (!erg && (ptr=ptr->next)!=NULL) {
        if ((set1=BFirstSingle(ptr->set,pl1))!=NULL) {
            erg=lower_equal(set1,set2=BLastSingle(ptr->set,pl2))
                || ExPath(count,pl2);
            if (set2)
                SetDelete(set2);
            SetDelete(set1);
        }
        ++count;
    }
    return erg;
}

/**************************************************************************/
/* Berechnung der Gueltigkeit einer Formel des Typs                       */
/* E ^ ( S1 * - ^ S2 )                                                    */
/**************************************************************************/

static int typ2(void)
{
    SList *ptr;
    Set *set1,*set2;
    unsigned count;
    int erg;

    erg=0;
    count=0;
    ptr=l2_configs;
    while (!erg && (ptr=ptr->next)!=NULL) {
        erg=!lower_equal(set1=BLastSingle(ptr->set,pl1)
                         ,set2=BLastSingle(ptr->set,pl2))
            && ExMaxFreePath(count,pl2);
        if (set1)
            SetDelete(set1);
        if (set2)
            SetDelete(set2);
        ++count;
    }
    return erg;
}

/**************************************************************************/
/* Berechnung der Gueltigkeit einer Formel des Typs                       */
/* E - ^ ( S1 * ^ S2 )                                                    */
/**************************************************************************/

static int typ3(void)
{
    SList *ptr;
    Set *set1,*set2;
    unsigned count;
    int erg;

    erg=0;
    count=0;
    ptr=l2_configs;
    while (!erg && (ptr=ptr->next)!=NULL) {
        if ((set1=BFirstSingle(ptr->set,pl1))==NULL
            && ExMaxFreePath(count,pl1))
            return 1;
        else {
            erg=!lower_equal(set1,set2=BLastSingle(ptr->set,pl2))
                && ExMaxFreePath(count,pl2);
            if (set2)
                SetDelete(set2);
            SetDelete(set1);
        }
        ++count;
    }
    return erg;
}

/**************************************************************************/
/* Berechnung der Gueltigkeit einer Formel des Typs                       */
/* E - ^ ( S1 * - ^ S2 )                                                  */
/**************************************************************************/

static int typ4(void)
{
    SList *ptr;
    Set *set1,*set2;
    unsigned count;
    int erg;

    erg=0;
    count=0;
    ptr=l2_configs;
    while (!erg && (ptr=ptr->next)!=NULL) {
        if (ExInfPath(count,pl2))
            return 1;
        erg=lower_equal(set1=BLastSingle(ptr->set,pl1)
                        ,set2=BLastSingle(ptr->set,pl2))
            && ExMaxFreePath(count,pl1);
        if (set1)
            SetDelete(set1);
        if (set2)
            SetDelete(set2);
        ++count;
    }
    return erg;
}

/**************************************************************************/
/* Berechnung der Gueltigkeit einer Formel des Typs                       */
/* E ^ S                                                                  */
/**************************************************************************/

static int typ5(void)
{
    SList *ptr;
    Set *set;
    unsigned count;

    count=0;
    ptr=l2_configs;
    while ((ptr=ptr->next)!=NULL) {
        if ((set=BFirstSingle(ptr->set,pl1))!=NULL) {
            SetDelete(set);
            return 1;
        }
        ++count;
    }
    return 0;
}

/**************************************************************************/
/* Berechnung der Gueltigkeit einer Formel des Typs                       */
/* E ^ - ^ S                                                              */
/**************************************************************************/

static int typ6(void)
{
    SList *ptr;
    unsigned count;

    count=0;
    ptr=l2_configs;
    while ((ptr=ptr->next)!=NULL) {
        if (ExMaxFreePath(count,pl1))
            return 1;
        ++count;
    }
    return 0;
}

/**************************************************************************/
/* Berechnung der Gueltigkeit einer Formel des Typs                       */
/* E - ^ S                                                                */
/**************************************************************************/

static int typ7(void)
{
    SList *ptr;
    Set *set;
    unsigned count;

    count=0;
    ptr=l2_configs;
    while ((ptr=ptr->next)!=NULL) {
        if (ExMaxFreePath(count,pl1)) {
            if ((set=BFirstSingle(ptr->set,pl1))==NULL)
                return 1;
            else
                SetDelete(set);
        }
        ++count;
    }
    return 0;
}

/**************************************************************************/
/* Berechnung der Gueltigkeit einer Formel des Typs                       */
/* E - ^ - ^ S                                                            */
/**************************************************************************/

static int typ8(void)
{
    SList *ptr;
    unsigned count;

    count=0;
    ptr=l2_configs;
    while ((ptr=ptr->next)!=NULL) {
        if (ExInfPath(count,pl1))
            return 1;
        ++count;
    }
    return 0;
}

/**************************************************************************/
/* Dient der (unkonventionellen) Feststellung des Typs der Formel, die im */
/* Syntaxbaum 'tree' abgelegt ist.                                        */
/* Dazu wird 'tree' rekursiv in Vorordnung durchlaufen.                   */
/* Wird ein Stellenknoten gefunden, so wird im Rueckgabewert ein Bit      */
/* entsprechend der aktuellen Rekursionstiefe gesetzt und der numerische  */
/* Name der Stelle in 'pl1' bzw. in 'pl2' abgelegt.                       */
/* Wird eine bestimmte Rekursionstiefe ueberschritten, so wird '-1'       */
/* zurueckgegeben.                                                        */
/**************************************************************************/

static unsigned preorder(TreeNode *tree,unsigned depth)
{
    if (tree) {
        if (depth>5 || (tree->info==NOT && tree->right->info!=DIAMOND))
            return -1;
        if (tree->info==PLACE) {
            if (pl1)
                pl2=(unsigned)(tree->left);
            else
                pl1=(unsigned)(tree->left);
            return 1<<depth;
        }
        return preorder(tree->left,++depth) | preorder(tree->right,depth);
    }
    return 0;
}

/**************************************************************************/
/* Checken einer Formel in der eingeschraenkten L2-Syntax.                */
/**************************************************************************/

int L2_ModelChecker(TreeNode *tree)
{
    int i;

    i=0;
    pl1=pl2=0;
    switch (preorder(tree,0)) {
    case 12: i=typ1(); break; /* E   ^ ( S1 *   ^ S2 ) */
    case 20: i=typ2(); break; /* E   ^ ( S1 * - ^ S2 ) */
    case 24: i=typ3(); break; /* E - ^ ( S1 *   ^ S2 ) */
    case 40: i=typ4(); break; /* E - ^ ( S1 * - ^ S2 ) */
    case  2: i=typ5(); break; /* E   ^   S             */
    case  8: i=typ6(); break; /* E   ^        - ^ S    */
    case  4: i=typ7(); break; /* E - ^   S             */
    case 16: i=typ8(); break; /* E - ^        - ^ S    */
    default:
        Fatal("Die Formel entspricht nicht der eingeschraenkten Syntax !");
        break;
    }
    return i;
}

/************************/
/* END OF FILE l2_mc.c  */
/************************/
