/*****************************/
/* main.c                    */
/* Burkhard Graves, 08.09.97 */
/*****************************/

#include <time.h>
#include <ctype.h>
#include <string.h>
#include <sys/stat.h>

#include "normal.h"
#include "mca.h"
#include "mc.tab.h"

/********************/
/* Globale Variable */
/********************/

FILE *yyin,*yyout;  /* Zu allgemeinen Ein/Ausgabe-Zwecken */
 int flags;         /* Diverse Flags.                     */
                    /* (extern)                           */

char *net_file=NULL;     /* Filenamen */
char *path_file=NULL;    /* File zur Ausgabe von Pfaden        */
char *formula_file=NULL;
char *solution_file=NULL;
char *maxconf_file=NULL;
char *iso_file=NULL;

clock_t ttime;      /* zur Zeitmessung */

extern YYSTYPE yyval;

/*******************************/
/* Prototyp des (yacc-)Parsers */
/*******************************/

int yyparse PROTO ((void));

#ifdef MEMORY

/*******************************/
/*-----------------------------*/
/*- Mit Speicher-Ueberwachung -*/
/*-----------------------------*/
/*******************************/

#define MAX 2500

void *memory[MAX],**bottom=memory;

/**********************/
/* Garbage Collection */
/**********************/

void garbage_collection(void)
{
    void **b,**t;

    b=memory+1;
    t=bottom;
    while (b<=t) {
        if (!*b) {
            while (!*t)
                --t;
            if (b>=t)
                break;
            *b=*t;
            *t=NULL;
            --t;
        }
        ++b;
    }
    bottom=--b;
}

/******************************************************/
/* Wurde der allozierte Speicher wieder dealloziert ? */
/******************************************************/

static void is_memory_empty(void)
{
    void **b,**t;

    b=memory+1;
    t=bottom;
    while (b<=t) {
        if (*t)
            Fatal("is_memory_empty(): No !");
        --t;
    }
}

/************************/
/* Speicher reservieren */
/************************/

void *MyMalloc(size_t size)
{
    if (++bottom>=memory+MAX) {
        --bottom;
        garbage_collection();
        if (++bottom>=memory+MAX)
            Fatal("MyMalloc(): memory[]-overflow !");
    }
    if ((*bottom=malloc(size))==NULL)
        Fatal("Memory allocation rejected !");
    return *bottom;
}

/***************************/
/* Speichergroesse aendern */
/***************************/

void *MyRealloc(void *ptr,size_t size)
{
    void **b,**t;

    b=memory+1;
    t=bottom;
    while (b<=t) {
        if (*t==ptr)
            break;
        --t;
    }
    if (b>t)
        Fatal("MyRealloc(): Unknown addresse !");
    if ((*t=realloc(ptr,size))==NULL)
        Fatal("Memory allocation rejected !");
    return *t;
}

/**********************/
/* Speicher freigeben */
/**********************/

void MyFree(void *ptr)
{
    void **b,**t;

    b=memory+1;
    t=bottom;
    while (b<=t) {
        if (*t==ptr)
            break;
        --t;
    }
    if (b>t)
        Fatal("MyFree(): Unknown addresse !");
    *t=NULL;
    free(ptr);
}

/**************************************/
/*------------------------------------*/
/*- END OF Mit Speicher-Ueberwachung -*/
/*------------------------------------*/
/**************************************/

#else

/********************************/
/*------------------------------*/
/*- Ohne Speicher-Ueberwachung -*/
/*------------------------------*/
/********************************/

/************************/
/* Speicher reservieren */
/************************/

void *MyMalloc(size_t size)
{
    void *m;

    if ((m=malloc(size))==NULL)
        Fatal("Memory allocation rejected !");
    return m;
}

/***************************/
/* Speichergroesse aendern */
/***************************/

void *MyRealloc(void *ptr,size_t size)
{
    void *r;

    if ((r=realloc(ptr,size))==NULL)
        Fatal("Memory allocation rejected !");
    return r;
}

/***************************************/
/*-------------------------------------*/
/*- END OF Ohne Speicher-Ueberwachung -*/
/*-------------------------------------*/
/***************************************/

#endif

/******************/
/* Fataler Fehler */
/******************/

void Fatal(char *msg)
{
    fprintf(stderr,"\n%s\nProgram stopped !\n",msg);
    exit(1);
}

/************************/
/* Aufruf demonstrieren */
/************************/

static void usage(char *myname)
{
    fprintf(stderr,"\nUsage: %s net_file [-cm] ",myname);
    fprintf(stderr,"[-f=fname] [-i=fname] [-s=fname] [-p=fname]\n\n");
    fprintf(stderr,"c: force calculation of informations\n");
    fprintf(stderr,"m: write only minimal firing-sequences\n");
    fprintf(stderr,"f: specify formula-file\n");
    fprintf(stderr,"i: specify iso-file\n");
    fprintf(stderr,"x: specify maxconf-file\n");
    fprintf(stderr,"s: specify solution-file\n");
    fprintf(stderr,"p: specify firing-sequences-file\n");
    fprintf(stderr,"\n                         Idea: "
                   "Javier Esparza ('93)\n");
    fprintf(stderr,"Implementation & Improvements: "
                   "Burkhard Graves ('94...'97)\n");
    fprintf(stderr,"                      Version: 4.3\n");
    fprintf(stderr,"          Date of compilation: %s\n\n",__DATE__);
    exit(1);
}

/******************/
/* Namen einlesen */
/******************/

static void read_name(char *ptr1,char **ptr2,char *myname)
{
    if (*++ptr1=='=' && *++ptr1)
        *ptr2=ptr1;
    else
        usage(myname);
}

/******************/
/* Flags einlesen */
/******************/

static void read_flags(char *ptr,char *myname)
{
    while (*++ptr) {
        switch(*ptr) {
        case 'l':
            flags|=LTL;
            break;
        case 'c':
            flags|=CALC_INFO;
            break;
        case 'm':
            flags|=MIN_PATH;
            break;
        case 'f':
            read_name(ptr,&formula_file,myname);
            return;
        case 'i':
            read_name(ptr,&iso_file,myname);
            return;
        case 's':
            read_name(ptr,&solution_file,myname);
            return;
        case 'p':
            read_name(ptr,&path_file,myname);
            return;
        case 'x':
            read_name(ptr,&maxconf_file,myname);
            return;
        case 'r':
            flags|=SHOW_REL;
            break;
        case 'w':
            yyout=NULL;
            break;
        default:
            usage(myname);
        }
    }
}

/****************/
/* File oeffnen */
/****************/

FILE *OpenFile(char *fname,char *mode)
{
    FILE *f;

    if ((f=fopen(fname,mode))==NULL) {
        fprintf(stderr,"File %s could not be opened!\n",fname);
        exit(1);
    }
    printf("%c: %s\n",*mode,fname);
    return f;
}

/*****************************************************************/
/* Zwei Files bzgl. des Datums der letzten Aenderung vergleichen */
/*****************************************************************/

static int first_older_second(char *file1,char *file2)
{
    struct stat fblock1;
    struct stat fblock2;

    return (!stat(file1,&fblock1) && !stat(file2,&fblock2)
#ifndef OLD_SUNOS
            && difftime(fblock1.st_mtime,fblock2.st_mtime)<0.0
#else
            && (unsigned long)(fblock1.st_mtime)
              <(unsigned long)(fblock2.st_mtime)
#endif
           )?1:0;
}

/****************************************************/
/* Test, ob Informationen geladen werden koennen    */
/*                   oder berechnet werden muessen. */
/****************************************************/

void Check(void (*read)(FILE *),void (*calc)(void),void (*save)(FILE *))
{
    FILE *f;
    static int count=2;     /* Funktion reagiert nur auf 2 Aufrufe. */

    if (count) {
        ttime=clock();
        if (!(flags&CALC_INFO) && first_older_second(net_file,iso_file)) {
            read(f=OpenFile(iso_file,"r"));
        } else {
            calc();
            save(f=OpenFile(iso_file,"w"));
        }
        fclose(f);
        printf("(%fs)\n",(float)(clock()-ttime)/CLOCKS_PER_SEC);
        if (--count)
            strcpy(iso_file+strlen(iso_file)-4,".re2");
        ttime=clock();
    }
}

/*****************/
/* Hauptprogramm */
/*****************/

int main(int argc,char *argv[])
{
    int i,erg=0;

#ifndef NDEBUG
    printf("\nchecking of assertions enabled !\n");
#endif
#ifdef OUTPUT
    printf("       debugging modus enabled !\n");
#endif
#ifdef MEMORY
    printf("       memory-watchdog enabled !\n");
#endif

    InitMaske();    /* Initialisierung und Test der Mengendarstellung */

    yyout=stdout;
    flags=0;        /* Alle Flags loeschen */

    for (i=1; i<argc; ++i)
        if (*(argv[i])!='-')
            if (!net_file)
                net_file=argv[i];
            else
                usage(argv[0]);
        else
            read_flags(argv[i],argv[0]);
    if (!net_file)
        usage(argv[0]);

    if (!yyout)
        yyout=OpenFile("mca.out","w");

    yyin=OpenFile(net_file,"r");

    ttime=clock();
    if (!maxconf_file) {
        maxconf_file=MYstrdup(net_file);
        strcpy(maxconf_file+strlen(maxconf_file)-4,".mxc");
    }
    ReadMciFile(yyin,maxconf_file);
    fclose(yyin);
    printf("(%fs)\n",(float)(clock()-ttime)/CLOCKS_PER_SEC);

    if (!iso_file) {
        iso_file=MYstrdup(net_file);
        strcpy(iso_file+strlen(iso_file)-4,".iso");
    }
    Check(ReadInfo,CalcInfo,SaveInfo);

#ifdef OUTPUT
    fprintf(yyout,"\n\nCutoffs:\n");
    SetDisplay(yyout,cutoffs);
    fprintf(yyout,"\n\nMaxConfigs:\n");
    SListDisplay(yyout,max_configs);
    DisplayMaxTrSequence(yyout,max_configs);
#endif
 
    if (path_file)
        fclose(OpenFile(path_file,"w")); 

    if (formula_file) {
        yyin=OpenFile(formula_file,"r");
    } else {
        yyin=stdin;
        printf("\nWhen using keyboard input a formula ");
        printf("has to be ended with a dot '.'\n");
        printf("'x' exits the input loop.\n\n");
        printf("    - : not    <=> : iff          1 : true\n");
        printf("    * : and     => : implicate    0 : false\n");
        printf("    + : or\n");
        printf("                 ! : ForAll       # : Box\n");
        printf("                 ? : Exists       ^ : Diamond\n\n");
    }
    for (;;) {
        if (!formula_file) {
            printf("Formula: ");
        }
        while (isspace(i=getc(yyin)))
            skip;
        if (i=='y' || i=='x' || (yyin!=stdin && feof(yyin))) {
            if (i=='y')
                Ask();
            break;
        }
        ungetc(i,yyin);

        if (yyparse()) {
            Fatal("Error while parsing expression !");
        }
        printf("Formula: ");
        TreeDisplay(stdout,tree);
        fputc('\n',stdout);

        Normalform(tree);

        printf(" in NF: ");
        TreeDisplay(stdout,tree);
        fputc('\n',stdout);

        ttime=clock();
        printf("----------------------------------------\n");

        erg=ModelChecker();

        printf("Result: %s (%fs)\n",(erg)?"True":"False"
               ,(float)(clock()-ttime)/CLOCKS_PER_SEC);
        printf("----------------------------------------\n");

#ifdef MEMORY
        garbage_collection();
#endif

    }

    FreeAll();
    Free(iso_file);

#ifdef MEMORY
    is_memory_empty();
#endif

    if (yyout!=stdout)
        fclose(yyout);
    if (yyin!=stdin)
        fclose(yyin);

    if (solution_file && yyin!=stdin) {
        yyout=OpenFile(solution_file,"w");
        fprintf(yyout,"%s\n",(erg)?"True":"False");
        fclose(yyout);
    }
    return 0;
}

/**********************/
/* END OF FILE main.c */
/**********************/
