/*****************************/
/* mc.y                      */
/* Burkhard Graves, 26.10.94 */
/*****************************/

%{
#include "tree.h"
#include "mca.h"

void yyerror(const char*);
int yylex(void);
%}

%union {
        TreeNode    *node;
        unsigned    wert;
}

/* Vorrang und Assoziativitaet der Operatoren */

%left       OR AND
%left       IFF IMPL
%nonassoc   FORALL EXISTS BOX DIAMOND
%nonassoc   NOT
%nonassoc   FALSE TRUE
%nonassoc   <wert> PLACE

%type <node> formel form

%%

start : formel { tree=$1; }
	;

formel  : form			  { $$=$1; }
        | FORALL form
            { $$=TreeCreateNode(NOT, NULL,
                     TreeCreateNode(EXISTS, NULL,
                         TreeCreateNode(NOT, NULL, $2)));
            }
        | EXISTS form           { $$=TreeCreateNode(EXISTS,NULL,$2); }
	;

form    : '(' form ')'          { $$=$2; }
        | '[' form ']'          { $$=$2; }
        | '{' form '}'          { $$=$2; }
        | BOX form
            {
                $$=TreeCreateNode(NOT, NULL,
                       TreeCreateNode(DIAMOND, NULL,
                           TreeCreateNode(NOT, NULL, $2)));
            }
        | DIAMOND form          { $$=TreeCreateNode(DIAMOND,NULL,$2); }
        | form IMPL form
            {
                $$=TreeCreateNode(OR,TreeCreateNode(NOT,NULL,$1),$3);
            }
        | form IFF form
            {
                $$=TreeCreateNode(OR,
                       TreeCreateNode(AND, TreeCopy($1), TreeCopy($3)),
                       TreeCreateNode(AND,
                           TreeCreateNode(NOT, NULL, $1),
                           TreeCreateNode(NOT, NULL, $3)));
            }
        | form OR form          { $$=TreeCreateNode(   OR,  $1,  $3); }
        | form AND form         { $$=TreeCreateNode(  AND,  $1,  $3); }
        | NOT form              { $$=TreeCreateNode(  NOT,NULL,  $2); }
        | FALSE                 { $$=TreeCreateNode(FALSE,NULL,NULL); }
        | TRUE                  { $$=TreeCreateNode( TRUE,NULL,NULL); }
        | PLACE       { $$=TreeCreateNode(PLACE,(TreeNode *)$1,NULL); }
	;

%%

void yyerror(const char *ptr)
{
    fprintf(stderr,"\nPARSING-ERROR (%s)!!!\n\n",ptr);
    exit(1);
}

/********************/
/* END OF FILE mc.y */
/********************/
