/*****************************/
/* mca.c                     */
/* Burkhard Graves, 08.04.95 */
/*****************************/

#include <ctype.h>
#include "mca.h"
#include "l1_mc.h"
#include "l2_mc.h"
#include "relation.h"
#include "basic_lf.h"
#include "mc.tab.h"

/********************/
/* Globale Variable */
/********************/

  TreeNode *tree;               /* der Syntaxbaum der Formel              */
BraProcess *bprocess;           /* der Branchingprozess                   */
       Set *cutoffs;            /* die Cutoff-Events                      */
       Set *min;                /* das Minimum des Branchingprozesses     */
       Set *initial_marking;    /* die Beschriftung des Minimums          */
     SList *max_configs;        /* die maximalen Konfigurationen          */
  unsigned anz_max_configs;     /* die Anzahl der max. Konfigurationen    */
  unsigned multi;               /* zaehlt die Konjunktionsglieder         */
                                /* (extern)                               */

  Wanted *first,*last;
     int count_wanted;
     Set *help1,*help2;   /* fuer sub_causes(), Causes(), CausesInSet()   */
unsigned gflag;           /* globales Flag                                */

/**************************************************************************/
/* Spezialfunktion, nur fuer den Meister persoenlich gedacht ...          */
/**************************************************************************/

#if 0 /* currently not needed*/

static void check_iso(void)
{
    Set *tmp;
    SList *slist;
    unsigned ce,count;

    count=0;
    tmp=NULL;
    slist=max_configs;
    while ((slist=slist->next)!=NULL) {
        printf("%u\n",++count);
        ce=0;
        for (;;) {
            ce=SetGetElementNext(cutoffs,++ce);
            if (!ce)
                break;
            if (SetIsElement(slist->set,ce)
                && !ExtInvFiniteShift(ce,tmp=SetDuplicate(slist->set))) {
                Fatal("No!");
            }
            if (tmp) {
                SetDelete(tmp);
                tmp=NULL;
            }
        }
    }
}

#endif

static void check_search_for_corr_conf(void)
{
    SList *slist;
    Set *conf,*mark1,*mark2;

    slist=max_configs;
    while ((slist=slist->next)!=NULL) {
        conf=SearchForCorrConf(Mark(slist->set));
        if (!conf || !SetIsDisjunct(conf,cutoffs))
            Fatal("Scheissdreck 1 !");
        mark1=Mark(conf);
        mark2=Mark(slist->set);
        if (!SetIsEqual(mark1,mark2))
            Fatal("Scheissdreck 2 !");
        SetDelete(conf);
        SetDelete(mark1);
        SetDelete(mark2);
        printf("Ok\n");
    }
}

static char *skip_space_digit(char *ptr)
{
    while (isspace(*ptr))
        ++ptr;
    while (isdigit(*ptr))
        ++ptr;
    return ptr;
}

void Ask(void)
{
    unsigned number;
    Set *cut,*causes,*conf,*shifted;
    char buffer[32],*ptr;
    Edge *edge;

    printf("\n(_x,_s,_ne,_nb,_ve,_vb,_c,_k,_i) ?\n\n");
    putchar('>');
    ptr=fgets(buffer,32,stdin);
    number=(unsigned)atol(ptr=fgets(buffer,32,stdin));
    while (number) {
        ptr=skip_space_digit(ptr);
        switch (*ptr) {
        case 'i':
            check_search_for_corr_conf();
            break;
        case 'x':
            ++ptr;
            conf=SetCreate();
            do {
                SetInsertElement(conf,number);
                number=(unsigned)atol(ptr);
                ptr=skip_space_digit(ptr);
            } while (number);
            for (;;) {
                SetDisplay(stdout,conf);
                fprintf(stdout,"\n<");
                number=(unsigned)atol(ptr=fgets(buffer,32,stdin));
                ptr=skip_space_digit(ptr);
                if (!number)
                    break;
                if (*ptr=='r') {
                    if (!InvFiniteShift(number,conf))
                        fprintf(stdout,"Backwardshift did not succeed!\n");
                } else {
                    shifted=FiniteShift(number,conf);
                    SetSwap(shifted,conf);
                    SetDelete(shifted);
                }
            }
            SetDelete(conf);
            break;
        case 's':
            cut=Cut(causes=Causes(number));
            SetDelete(causes);
            fprintf(stdout,"\ncut : ");
            SetDisplay(stdout,cut);
            fprintf(stdout,"\nmark : ");
            DisplayConditionLabels(stdout,cut);
            fprintf(stdout,"\n<");
            number=(unsigned)atol(ptr=fgets(buffer,32,stdin));
            while (number) {
                edge=(bprocess->events+number)->preset;
                while (edge && SetIsElement(cut,edge->condition->numname)) {
                    edge=edge->nextcondition;
                }
                if (edge) {
                    fprintf(stdout,"\nnot activated !");
                } else {
                    edge=(bprocess->events+number)->preset;
                    while (edge) {
                        SetRemoveElement(cut,edge->condition->numname);
                        edge=edge->nextcondition;
                    }
                    edge=(bprocess->events+number)->postset;
                    while (edge) {
                        SetInsertElement(cut,edge->condition->numname);
                        edge=edge->nextcondition;
                    }
                }
                fprintf(stdout,"\ncut : ");
                SetDisplay(stdout,cut);
                fprintf(stdout,"\nmark : ");
                DisplayConditionLabels(stdout,cut);
                fprintf(stdout,"\n<");
                ptr=fgets(buffer,32,stdin);
                if (*ptr=='-') {
                    edge=(bprocess->events+number)->postset;
                    while (edge) {
                        SetRemoveElement(cut,edge->condition->numname);
                        edge=edge->nextcondition;
                    }
                    edge=(bprocess->events+number)->preset;
                    while (edge) {
                        SetInsertElement(cut,edge->condition->numname);
                        edge=edge->nextcondition;
                    }
                    fprintf(stdout,"\ncut : ");
                    SetDisplay(stdout,cut);
                    fprintf(stdout,"\nmark : ");
                    DisplayConditionLabels(stdout,cut);
                    fprintf(stdout,"\n<");
                    number=(unsigned)atol(ptr=fgets(buffer,32,stdin));
                } else {
                    number=(unsigned)atol(ptr);
                }
            }
            SetDelete(cut);
            break;
        case 'v':
            if (*++ptr=='e') {
                fprintf(stdout,"\n.E%u(%s) : "
                        ,number
                        ,GetTrName((bprocess->events+number)->tr_numname));
                edge=(bprocess->events+number)->preset;
                while (edge) {
                    fprintf(stdout,"B%u(%s),"
                            ,edge->condition->numname
                            ,GetPlName(edge->condition->pl_numname));
                    edge=edge->nextcondition;
                }
            } else if (*ptr=='b') {
                fprintf(stdout,"\n.B%u(%s) : "
                        ,number
                        ,GetPlName((bprocess->conditions+number)->pl_numname));
                edge=(bprocess->conditions+number)->preset;
                while (edge) {
                    fprintf(stdout,"E%u(%s),"
                            ,edge->event->numname
                            ,GetTrName(edge->event->tr_numname));
                    edge=edge->nextevent;
                }
            }
            break;
        case 'n':
            if (*++ptr=='e') {
                fprintf(stdout,"\nE.%u(%s) : "
                        ,number
                        ,GetTrName((bprocess->events+number)->tr_numname));
                edge=(bprocess->events+number)->postset;
                while (edge) {
                    fprintf(stdout,"B%u(%s),"
                            ,edge->condition->numname
                            ,GetPlName(edge->condition->pl_numname));
                    edge=edge->nextcondition;
                }
            } else if (*ptr=='b') {
                fprintf(stdout,"\nB.%u(%s) : "
                        ,number
                        ,GetPlName((bprocess->conditions+number)->pl_numname));
                edge=(bprocess->conditions+number)->postset;
                while (edge) {
                    fprintf(stdout,"E%u(%s),"
                            ,edge->event->numname
                            ,GetTrName(edge->event->tr_numname));
                    edge=edge->nextevent;
                }
            }
            break;
        case 'c':
            cut=Cut(causes=Causes(number));
            fprintf(stdout,"\n[%u] : ",number);
            SetDisplay(stdout,causes);
            fprintf(stdout,"\ncut : ");
            SetDisplay(stdout,cut);
            fprintf(stdout,"\nmark : ");
            DisplayConditionLabels(stdout,cut);
            SetDelete(causes);
            SetDelete(cut);
            break;
        case 'k':
            fprintf(stdout,"\ncorr : %u",GetCorrespEvent(number));
            break;
        }
        fprintf(stdout,"\n\n");
        putchar('>');
        number=(unsigned)atol(ptr=fgets(buffer,32,stdin));
    }
}

/**************************************************************************/
/* Ausgabe der Beschriftungen einer Menge von Mengen von Ereignissen      */
/**************************************************************************/

void DisplayMaxTrSequence(FILE *out,SList *slist)
{
    assert(slist);
    while ((slist=slist->next)!=NULL) {
        DisplayEventLabels(out,slist->set);
        fputc('\n',out);
    }
}

/**************************************************************************/
/* Ausgabe der Beschriftungen von einer Menge von Ereignissen             */
/**************************************************************************/

void DisplayEventLabels(FILE *out,Set *events)
{
    unsigned event, num;

    event=SetGetElementNext(events,1);
    while (event) {
	if ((num=(bprocess->events+event)->tr_numname))
          fprintf(out,GetTrName(num));
	else
	  fprintf(out,"tic");
        event=SetGetElementNext(events,++event);
        if (event)
            fputc(',',out);
    }
}

/**************************************************************************/
/* Ausgabe der Beschriftungen von einer Menge von Bedingungen             */
/**************************************************************************/

void DisplayConditionLabels(FILE *out,Set *conditions)
{
    unsigned condition;

    condition=SetGetElementNext(conditions,1);
    while (condition) {
        fprintf(out,GetPlName((bprocess->conditions+condition)->pl_numname));
        condition=SetGetElementNext(conditions,++condition);
        if (condition)
            fputc(',',out);
    }
}

/**************************************************************************/
/* Ausgabe der Namen von einer Menge von numerischen Stellennamen         */
/**************************************************************************/

void DisplayPlaceNames(FILE *out,Set *places)
{
    unsigned place;

    place=SetGetElementNext(places,1);
    while (place) {
        fprintf(out,GetPlName(place));
        place=SetGetElementNext(places,++place);
        if (place)
            fputc(',',out);
    }
}

/**************************************************************************/
/* Berechnet max{slist1 triangle slist2}.                                 */
/**************************************************************************/

SList *MaxTriangle(SList *slist1,SList *slist2)
{
    SList *slist,*ptr;

    slist=NULL;
    if (slist1 && slist2) {
        while ((slist1=slist1->next)!=NULL) {
            ptr=slist2;
            while ((ptr=ptr->next)!=NULL) {
                if (slist)
                    SListMaxInsert(slist,
                                   SetIntersectionCreate(slist1->set
                                                         ,ptr->set));
                else
                    slist=SListCreate(SetIntersectionCreate(slist1->set
                                                            ,ptr->set));
            }
        }
    }
    return slist;
}

/**************************************************************************/
/* Berechnet max{slist1 union slist2}.                                    */
/* Bemerkung: Es wird davon ausgegangen, dass max{slist1}=slist1 gilt,    */
/*            ausserdem wird keine neue Set-Liste angelegt, sondern       */
/*            'slist1' entsprechend ergaenzt.                             */
/**************************************************************************/

SList *MaxUnion(SList *slist1,SList *slist2)
{
    assert(slist2!=max_configs);
    if (slist2) {
        while ((slist2=slist2->next)!=NULL) {
            if (slist1) {
                SListMaxInsert(slist1,slist2->set);
                slist2->set=NULL;
            } else {
                slist1=SListCreate(slist2->set);
                slist2->set=NULL;
            }
        }
    }
    return slist1;
}

/**************************************************************************/
/* Test, ob die Konfiguration 'configuration' kleiner oder gleich der     */
/* Menge von Konfigurationen 'slist' ist.                                 */
/**************************************************************************/

int LowerEqual(Set *configuration,SList *slist)
{
    if (slist)
        while ((slist=slist->next)!=NULL) {
            if (SetIsSubset(configuration,slist->set))
                return 1;
        }
    return 0;
}

/**************************************************************************/
/* Berechnung des Minimums des Branchingprozesses.                        */
/**************************************************************************/

Set *BpMin(BraProcess *bprocess)
{
    Set       *min;
    Condition *pl;

    min=SetCreate();
    pl=bprocess->conditions; /* Achtung: bprocess->conditions zeigt vor */
    ++pl;                    /* die erste Bedingung !                  */
    while (pl) {
        if (!pl->preset)
            SetInsertElement(min,pl->numname);
        pl=pl->next;
    }
    return min;
}

/**************************************************************************/
/* Kopiert den durch die Konfiguration 'configuration' determinierten     */
/* Schnitt in die Menge 'cut'.                                            */
/**************************************************************************/

void CutInSet(Set *cut,Set *configuration)
{
    unsigned element;
    Event *event;
    Edge *edge;

    SetCopy(cut,min);
    element=0;
    for (;;) {
        element=SetGetElementNext(configuration,++element);
        if (!element)
            break;
        event=bprocess->events+element;
        edge=event->postset;
        while (edge) {
            SetInsertElement(cut,edge->condition->numname);
            edge=edge->nextcondition;
        }
        edge=event->preset;
        while (edge) {
            SetRemoveElement(cut,edge->condition->numname);
            edge=edge->nextcondition;
        }
    }
}

/**************************************************************************/
/* Berechnet den durch die Konfiguration 'configuration' determinierten   */
/* Schnitt.                                                               */
/**************************************************************************/

Set *Cut(Set *configuration)
{
    Set *cut;

    CutInSet(cut=SetCreate(),configuration);
    return cut;
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/

Set *LocalCut(Set *basis,Set *configuration)
{
    unsigned element;
    Event *event;
    Edge *edge;
    Set *cut;

    if (!configuration)
        return configuration;
    cut=SetDuplicate(basis);
    element=0;
    for (;;) {
        element=SetGetElementNext(configuration,++element);
        if (!element)
            break;
        event=bprocess->events+element;
        edge=event->postset;
        while (edge) {
            SetInsertElement(cut,edge->condition->numname);
            edge=edge->nextcondition;
        }
        edge=event->preset;
        while (edge) {
            SetRemoveElement(cut,edge->condition->numname);
            edge=edge->nextcondition;
        }
    }
    return cut;
}

/**************************************************************************/
/* Kopiert die Beschriftungen der Bedingungen in der Menge 'conditions'   */
/* in die Menge 'labels'.                                                 */
/**************************************************************************/

void LCondInSet(Set *labels,Set *conditions)
{
    unsigned condition;
    Condition *cond;

    SetClear(labels);
    cond=bprocess->conditions;
    condition=0;
    for (;;) {
        condition=SetGetElementNext(conditions,++condition);
        if (!condition)
            break;
        SetInsertElement(labels,(cond+condition)->pl_numname);
    }
}

/**************************************************************************/
/* Berechnet die Beschriftungen der Bedingungen in der Menge 'conditions'.*/
/**************************************************************************/

Set *LCond(Set *conditions)
{
    Set *labels;

    LCondInSet(labels=SetCreate(),conditions);
    return labels;
}

/**************************************************************************/
/* Kopiert die durch die Konfiguration 'configuration' determinierte      */
/* Markierung in die Menge 'mark'.                                        */
/**************************************************************************/

Set *MarkInSet(Set *mark,Set *configuration)
{
    Set *cut;

    LCondInSet(mark,cut=Cut(configuration));
    SetDelete(cut);
    return mark;
}

/**************************************************************************/
/* Berechnet die durch die Konfiguration 'configuration' determinierte    */
/* Markierung.                                                            */
/**************************************************************************/

Set *Mark(Set *configuration)
{
    Set *cut;
    Set *mark;

    mark=LCond(cut=Cut(configuration));
    SetDelete(cut);
    return mark;
}

/**************************************************************************/
/* Berechnet die Geschichte des Ereignisses 'event'.                      */
/**************************************************************************/

static void sub_causes(Event *event)
{
    Edge *edge;

    if (SetInsertElement(help1,event->numname)) {
        edge=event->preset;
        while (edge) {
            if (edge->condition->preset)
                sub_causes(edge->condition->preset->event);
            edge=edge->nextcondition;
        }
    }
/*    else {                                                             */
/*      das gerade betrachtete Ereignis ist bereits in 'help1' enthalten */
/*      und somit auch dessen Geschichte !                               */
/*  }                                                                    */
}

void CausesInSet(Set *causes,unsigned event)
{
    help1=SetCreate();
    sub_causes(bprocess->events+event);
    SetCopy(causes,help1);
    SetDelete(help1);
}

Set *Causes(unsigned event)
{
    help1=SetCreate();
    sub_causes(bprocess->events+event);
    return help1;
}

/**************************************************************************/
/* Berechnet die Geschichte des Ereignisses 'event'                       */
/* jenseits eines Schnittes 'lcut'.                                       */
/**************************************************************************/

static void sub_local_causes(Event *event)
{
    Edge *edge;

    if (SetInsertElement(help1,event->numname)) {
        edge=event->preset;
        while (edge) {
            if (!SetIsElement(help2,edge->condition->numname)) {
                if (edge->condition->preset)
                    sub_local_causes(edge->condition->preset->event);
                else
                    gflag=1;
            }
            edge=edge->nextcondition;
        }
    }
/*    else {                                                             */
/*      das gerade betrachtete Ereignis ist bereits in 'help1' enthalten */
/*      und somit auch dessen lokale Geschichte !                        */
/*  }                                                                    */
}

Set *LocalCauses(Set *lcut,unsigned event)
{
    gflag=0;
    help2=lcut;
    help1=SetCreate();
    sub_local_causes(bprocess->events+event);
    if (gflag) {
        SetDelete(help1);
        return NULL;
    }
    return help1;
}

/**************************************************************************/
/* Berechnet das korrespondierende Ereignis zum Cutoff-Event 'event'.     */
/**************************************************************************/

unsigned GetCorrespEvent(unsigned event)
{
    unsigned corr_event,count;
    Set *mark_1,*mark_2,*causes;

    corr_event=0;
    if (SetIsEqual(mark_1=Mark(causes=Causes(event)),initial_marking)) {
        SetDelete(causes);
        SetDelete(mark_1);
        return corr_event;
    }
    count=SetCountElements(causes);
    mark_2=SetCreate();
    for (corr_event=event-1; corr_event; --corr_event) {
        CausesInSet(causes,corr_event);
        if (SetIsEqual(mark_1,MarkInSet(mark_2,causes))
            && SetCountElements(causes)<count ) {
            SetDelete(mark_2);
            SetDelete(causes);
            SetDelete(mark_1);
            return corr_event;
        }
    }
    Fatal("GetCorrespEvent(): Kein korresp. Event gefunden !");
    return corr_event;
}

/**************************************************************************/
/* Aufruf mit der ersten Kante im Nachbereich eines Events; es werden     */
/* alle Ereignisse aus follow (einer (Teil-)Konfiguration) entfernt,      */
/* die hinter diesem Event liegen.                                        */
/**************************************************************************/

static void remove_behind(Edge *edge1, Set *follow)
{
    register Edge *edge2;

    while (edge1) {
        edge2=edge1->condition->postset;
        while (edge2) {
            if (SetRemoveElement(follow,edge2->event->numname)) {
                remove_behind(edge2->event->postset,follow);
                break;
            }
            edge2=edge2->nextevent;
        }
        edge1=edge1->nextcondition;
    }
}

/**************************************************************************/
/* Test, ob einem Ereignis 'event' mind. ein weiteres Ereignis folgt.     */
/**************************************************************************/

static int something_behind(Event *event, Set *follow)
{
    register Edge *edge1, *edge2;

    edge1=event->postset;
    while (edge1) {
        edge2=edge1->condition->postset;
        while (edge2) {
            if (SetIsElement(follow, edge2->event->numname))
                return 1;
            edge2=edge2->nextevent;
        }
        edge1=edge1->nextcondition;
    }
    return 0;
}

/* optimieren */

/**************************************************************************/
/* Ein neues virtuelles Cutoff-Event beantragen.                          */
/**************************************************************************/

static void new_wanted(unsigned base, unsigned broken)
{
    Wanted *ptr1,*ptr2,*newev;

    ptr1=first;
    if (ptr1) {
        if (base>bprocess->num_events) { /* anhaengen    */
            while (ptr1) {
                if (ptr1->base==base && ptr1->broken==broken)
                    return;
                ptr1=ptr1->next;
            }
            (last=last->next=newev=(Wanted *)MyMalloc(sizeof(Wanted)))
            ->next=NULL;
        } else {                         /* einsortieren */
            ptr2=NULL;
            while (ptr1 && ptr1->base<base)
                ptr1=(ptr2=ptr1)->next;
            if (ptr1 && ptr1->base==base)
                while (ptr1 && ptr1->broken<broken)
                    ptr1=(ptr2=ptr1)->next;
            if (ptr1 && ptr1->base==base && ptr1->broken==broken)
                return;
            (newev=(Wanted *)MyMalloc(sizeof(Wanted)))->next=ptr1;
            if (!ptr1)
                last=newev;
            if (!ptr2)
                first=newev;
            else
                ptr2->next=newev;
        }
    } else {
        (newev=(Wanted *)MyMalloc(sizeof(Wanted)))->next=NULL;
        last=first=newev;
    }
    newev->base=base;
    newev->broken=broken;
    newev->count=0;
    ++count_wanted;
}

/**************************************************************************/
/* Den ersten Antrag an das Ende der Liste haengen, zurueckgegeben wird   */
/* ein Zeiger auf den zweiten Antrag.                                     */
/**************************************************************************/

static Wanted *skip_wanted(void)
{
    assert(first && last && first!=last);
    last=last->next=first;
    first=first->next;
    last->next=NULL;
    return first;
}

/**************************************************************************/
/* Den ersten Antrag loeschen, zurueckgegeben wird ein Zeiger auf den     */
/* darauffolgenden Antrag.                                                */
/**************************************************************************/

static Wanted *kill_wanted(void)
{
    Wanted *old;

    assert(first && last);
    old=first;
    first=first->next;
    MyFree(old);
    return first;
}

/**************************************************************************/
/* Ein neues virtuelles Cutoff-Event erstellen.                           */
/**************************************************************************/

static VCEvent *new_vce(Set *conf, Set *corr_conf)
{
    VCEvent *vce;

    ++bprocess->num_vce;
    (vce=(VCEvent *)MyMalloc(sizeof(VCEvent)))->next=bprocess->vcevents;
    bprocess->vcevents=vce;
    vce->numname=((vce->next)?vce->next->numname:bprocess->num_events)+1;
    vce->causes_event=conf;
    vce->causes_corr_event=corr_conf;
    return vce;
}

/**************************************************************************/
/* Die Geschichte des korrespondierenden Events eines Cutoff-Events       */
/* (welches virtuell sein kann) zurueckgeben.                             */
/**************************************************************************/

static Set *get_causes_corr_event(unsigned event)
{
    register BraProcess *bproc;
    register VCEvent *vce;

    bproc=bprocess;
    if (event<=bproc->num_events) {
        assert((bproc->events+event)->info);
        return (bproc->events+event)->info->causes_corr_event;
    }
    vce=bprocess->vcevents;
    assert(vce);
    while (vce->numname!=event) {
        vce=vce->next;
        assert(vce);
    }
    return vce->causes_corr_event;
}

/**************************************************************************/
/* Berechnung der Zusatzinformationen zu allen Cutoff-Events,             */
/* insbesondere des jeweiligen Isomorphismus.                             */
/**************************************************************************/

static int sub_calc_info_0(Set *cut,Event *event)
{
    Edge *edge;

    edge=event->preset;
    while (edge) {
        if (!SetIsElement(cut,edge->condition->numname))
            return 0;
        edge=edge->nextcondition;
    }
    edge=event->preset;
    while (edge) {
        SetRemoveElement(cut,edge->condition->numname);
        edge=edge->nextcondition;
    }
    edge=event->postset;
    while (edge) {
        SetInsertElement(cut,edge->condition->numname);
        edge=edge->nextcondition;
    }
    return 1;
}

static void sub_calc_info_1(Set *s_cut,Set *follow,IList *ilist
                            ,unsigned cutoff_event)
{
    unsigned event,src;
    Edge *edge;
    Condition *cond;

    event=0;
    for (;;) {
        event=SetGetElementNext(follow,++event);
        if (!event)
            break;
        cond=bprocess->conditions;
        src=0;
        do {
            src=SetGetElementNext(s_cut,++src);
            assert(src);
            edge=(cond+src)->postset;
            while (edge) {
                if (edge->event->tr_numname
                    ==(bprocess->events+event)->tr_numname
                    && sub_calc_info_0(s_cut,edge->event)) {
                    IListInsertElement(ilist
                                       ,edge->event->numname
                                       ,event);
                    if (edge->event->info
                        && something_behind(bprocess->events+event, follow)) {
                        remove_behind((bprocess->events+event)->postset,follow);
                        new_wanted(cutoff_event,edge->event->numname);
                    }
                    src=0;
                    break;
                }
                edge=edge->nextevent;
            }
        } while (src);
    }
}

static void sub_calc_info_2(void)
{
    Wanted *ptr;
    Set *temp,*corr_conf;
    VCEvent *vce;
    Set *s_cut,*ss_cut,*follow;
    SList *slist;
    IList *ilist;

    printf("%u virtual cut-off event(s) applied for.\n",count_wanted);
    ptr=first;
    while (ptr) {
        printf("%u (%u)\n",ptr->base,ptr->broken);
        corr_conf=SetDuplicate((bprocess->events+ptr->broken)
                               ->info->causes_event);
        SetUnion(corr_conf,get_causes_corr_event(ptr->base));
        if (ptr->count>0 && ptr->count==bprocess->num_vce+1) {
            printf("HELLO! I AM VERY INTERESTED IN THIS NET-SYSTEM\n");
            printf("AND ITS FINITE PREFIX, PLEASE CONTACT ME\n"); 
            printf("(pep@informatik.uni-oldenburg.de), THANKS!\n");
            corr_conf=SearchForCorrConf(Mark(corr_conf));
        } else {
            if (!CalcCorrConf(corr_conf)) {
                SetDelete(corr_conf);
                corr_conf=NULL;
                ptr->count=bprocess->num_vce+1;
                ptr=skip_wanted();
            }
        }
        if (corr_conf) {
            temp=SetDuplicate((bprocess->events+ptr->broken)
                              ->info->causes_event);
            SetUnion(temp,get_causes_corr_event(ptr->base));
            vce=new_vce(FiniteShift(ptr->base,temp),corr_conf);
            SetDelete(temp);
            s_cut=Cut(vce->causes_corr_event);
            ilist=IListCreate(0,0);
            ss_cut=SetCreate();

            slist=max_configs;
            while ((slist=slist->next)!=NULL) {
                if (SetIsSubset(vce->causes_event,slist->set)) {
                    follow=SetDifference(SetDuplicate(slist->set)
                                         ,vce->causes_event);
                    sub_calc_info_1(SetCopy(ss_cut,s_cut),follow,ilist
                                    ,vce->numname);
                    SetDelete(follow);
                }
            }
            vce->ilist=ilist->next;
            MyFree(ilist);
            SetDelete(s_cut);
            SetDelete(ss_cut);
            ptr=kill_wanted();
        }
    }

#ifdef OUTPUT
    vce=bprocess->vcevents;
    while (vce) {
        fprintf(yyout,"%u: ",vce->numname);
        SetDisplay(yyout,vce->causes_event);
        fputc('(',yyout);
        SetDisplay(yyout,vce->causes_corr_event);
        fprintf(yyout,")\n");
        IListDisplay(yyout,vce->ilist);
        vce=vce->next;
    }
#endif

}

#if 0 /* currently not needed*/

/**************************************************************************/
/* Sicherstellung, dass korrespondierende Events von cutoff-Events keine  */
/* cutoff-Events sind.                                                    */
/* Problem: Speicherung!                                                  */
/**************************************************************************/

static void adjust(Info *info1)
{
    unsigned event;
    Info *info2;

    event=info1->corr_event;
    while (event && (info2=(bprocess->events+event)->info)!=NULL)
        event=info2->corr_event;
    info1->corr_event=event;
}

#endif

void CalcInfo(void)
{
    register unsigned cutoff_event;
    Set *s_cut,*ss_cut,*follow;
    Info *info;
    SList *slist;
    IList *ilist;

    first=last=NULL;
    count_wanted=0;
    printf("calculating isomorphisms...\n");
    cutoff_event=0;
    for (;;) {
        cutoff_event=SetGetElementNext(cutoffs,++cutoff_event);
        if (!cutoff_event)
            break;
        info=(bprocess->events+cutoff_event)->info;
        assert(info);

        if (info->corr_event)
            s_cut=Cut(info->causes_corr_event=Causes(info->corr_event));
        else {
            info->causes_corr_event=SetCreate();
            s_cut=SetDuplicate(min);
        }
        info->causes_event=Causes(cutoff_event);

#ifdef OUTPUT
    fprintf(yyout,"\nE%u(E%u)\n",cutoff_event,info->corr_event);
#endif

        ilist=IListCreate(0,0);
        ss_cut=SetCreate();

        slist=max_configs;
        while ((slist=slist->next)!=NULL) {
            if (SetIsElement(slist->set,cutoff_event)) {
                follow=SetDifference(SetDuplicate(slist->set)
                                     ,info->causes_event);
                sub_calc_info_1(SetCopy(ss_cut,s_cut),follow,ilist
                                ,cutoff_event);
                SetDelete(follow);
            }
        }
        info->ilist=ilist->next;

#ifdef OUTPUT
    IListDisplay(yyout,info->ilist);
#endif

        MyFree(ilist);
        SetDelete(s_cut);
        SetDelete(ss_cut);
    }

    if (count_wanted)
        sub_calc_info_2();
}

/**************************************************************************/
/* Test, ob ein Ereignis mit dem numerischen Namen 'event', in dem        */
/* Branching-Prozess nach dem Schnitt 'cut' enthalten ist.                */
/**************************************************************************/

int InPostCut(Set *cut,unsigned event)
{
    Set *causes;

    causes=LocalCauses(cut,event);
    if (causes) {
        SetDelete(causes);
        return 1;
    } else
        return 0;
}

/**************************************************************************/
/* Berechnung der Zusatzinformationen zu allen Cutoff-Events,             */
/* und Einlesen des jeweiligen Isomorphismus.                             */
/**************************************************************************/

static IList *sub_read_info(FILE *in)
{
    IList *ptr,*start;
    unsigned x;

    fread((void *)&x,sizeof(x),1,in);
    if (x) {
        (start=ptr=(IList *)MyMalloc(sizeof(IList)))->x=x;
        fread((void *)&(ptr->y),sizeof(ptr->y),1,in);
        for (;;) {
            fread((void *)&x,sizeof(x),1,in);
            if (!x)
                break;
            (ptr=ptr->next=(IList *)MyMalloc(sizeof(IList)))->x=x;
            fread((void *)&(ptr->y),sizeof(ptr->y),1,in);
        }
        ptr->next=NULL;
        return start;
    } else
        return NULL;
}

void ReadInfo(FILE *in)
{
    unsigned cutoff_event,count;
    Info *info;
    VCEvent *vce;

    cutoff_event=0;
    for (;;) {
        cutoff_event=SetGetElementNext(cutoffs,++cutoff_event);
        if (!cutoff_event)
            break;
        info=(bprocess->events+cutoff_event)->info;
        assert(info);

        info->causes_event=Causes(cutoff_event);
        info->causes_corr_event
        =(info->corr_event)?Causes(info->corr_event):SetCreate();
        info->ilist=sub_read_info(in);
    }
    fread((void *)&(bprocess->num_vce),sizeof(bprocess->num_vce),1,in);
    count=bprocess->num_vce;
    while (count) {
        (vce=(VCEvent *)MyMalloc(sizeof(VCEvent)))
        ->next=bprocess->vcevents;
        bprocess->vcevents=vce;
        vce->causes_event=SetRead(in);
        vce->causes_corr_event=SetRead(in);
        vce->ilist=sub_read_info(in);
        --count;
    }
}

/**************************************************************************/
/* Speichern der Isomorphismen aller Cutoff-Events.                       */
/**************************************************************************/

static void sub_save_info(FILE *out,IList *ptr)
{
    unsigned x;

    while (ptr) {
        fwrite((void *)&(ptr->x),sizeof(ptr->x),1,out);
        fwrite((void *)&(ptr->y),sizeof(ptr->y),1,out);
        ptr=ptr->next;
    }
    x=0;
    fwrite((void *)&x,sizeof(x),1,out);
}

void SaveInfo(FILE *out)
{
    register unsigned cutoff_event;
    register VCEvent *vce;

    cutoff_event=0;
    for (;;) {
        cutoff_event=SetGetElementNext(cutoffs,++cutoff_event);
        if (!cutoff_event)
            break;
        assert((bprocess->events+cutoff_event)->info);
        sub_save_info(out,(bprocess->events+cutoff_event)->info->ilist);
    }
    fwrite((void *)&(bprocess->num_vce),sizeof(bprocess->num_vce),1,out);
    vce=bprocess->vcevents;
    while (vce) {
        SetSave(out,vce->causes_event);
        SetSave(out,vce->causes_corr_event);
        sub_save_info(out,vce->ilist);
        vce=vce->next;
    }
}

/**************************************************************************/
/* Test, ob alle direkten Stellenaussagen gueltig sind.                   */
/**************************************************************************/

int NoModalitiesAreOk(TreeNode *tree)
{
    if (tree->info==AND)
        return NoModalitiesAreOk(tree->left)
               & NoModalitiesAreOk(tree->right);
    else if (tree->info==PLACE)
        return SetIsElement(initial_marking,(unsigned)(tree->left));
    else if (tree->info==NOT && tree->right->info==PLACE)
        return !SetIsElement(initial_marking,(unsigned)(tree->right->left));
    else if (tree->info==FALSE)
        return 0;
    return 1;
}

/**************************************************************************/
/* Der L1-L2-Modelchecker                                                 */
/**************************************************************************/

void sub_model_checker(void)
{
    Fatal("Sorry, LTL modelchecking not yet supported.");
}

int ModelChecker(void)
{
    int erg;
    TreeNode *sub_tree,*sub_sub_tree;

    do {
        /* ein Disjunktionsglied holen */
        sub_tree=DeliverSubformula(tree);
        /* Test der nicht-temporalen Aussagen */
        erg=NoModalitiesAreOk(sub_tree);
        /* eine L1- oder L2-Teilformel holen */
        sub_sub_tree=DeliverDiamOrQuan(sub_tree);
        /* multi gesetzt, wenn Teilformel nicht in der Wurzel beginnt */
        multi=(sub_sub_tree!=sub_tree);
        /* solange die bisher betrachteten Teilformeln gueltig sind */
        /* und weitere Teilformeln existieren ...                   */
        while (erg && sub_sub_tree) {
            if (sub_sub_tree->info==NOT
                && sub_sub_tree->right->info==DIAMOND) {
                erg&=!L1_ModelChecker(sub_sub_tree,2);
                sub_sub_tree->right->right=NULL;
            } else if (sub_sub_tree->info==DIAMOND) {
                erg&=L1_ModelChecker(sub_sub_tree,1);
                sub_sub_tree->right=NULL;
            } else if (sub_sub_tree->info==EXISTS) {
                /* wie steht's mit den Infos fuer den L2-MC ? */
                Check(RelLoad,CalcRelations,RelSave);
                if (flags&LTL) {
                    erg&=L2_ModelChecker(sub_sub_tree->right);
                } else {
                    sub_model_checker();
                }
            } else {
                /* wie steht's mit den Infos fuer den L2-MC ? */
                Check(RelLoad,CalcRelations,RelSave);
                if (flags&LTL) {
                    erg&=!L2_ModelChecker(sub_sub_tree->right->right);
                } else {
                    sub_model_checker();
                }
            }
            /* Teilformel loeschen */
            TreeDelete(sub_sub_tree);
            /* multi gesetzt, wenn Teilformel nicht in der Wurzel beginnt */
            if (sub_sub_tree==sub_tree)
                multi=0;
            else
                ++multi;
/* vorher:  multi=(sub_sub_tree!=sub_tree); */
            /* naechste Teilformel holen */
            sub_sub_tree=(multi)?DeliverDiamOrQuan(sub_tree):NULL;
        }
        if (multi) /* Disjunktionsglied noch nicht vollstaendig geloescht */
            TreeDelete(sub_tree);
    /* solange die bisher betrachteten Disjunktionsglieder ungueltig sind */
    /* und weitere existieren ...                                         */
    } while (!erg && tree!=sub_tree);
    if (tree!=sub_tree) /* Syntaxbaum noch nicht vollstaendig geloescht */
        TreeDelete(tree);
    return erg;
}

/*********************/
/* END OF FILE mca.c */
/*********************/
