/*****************************/
/* occ_net.c                 */
/* Burkhard Graves, 27.05.98 */
/*****************************/

#include <string.h>
#include <sys/types.h>
#include <netinet/in.h>
#include "occ_net.h"
#include "mca.h"
#include "l2_mc.h"
#include "relation.h"
#include "rw_prefix.h"
#include "rw_mci.h"
#include "maxconf.h"

/********************/
/* Globale Variable */
/********************/

char **pl_names; /* fuer ReadMciFile(), FreeAll() */
                 /* GetPlName(), GetPlNumname()   */
char **tr_names; /* fuer ReadMciFile(), FreeAll() */
                 /* GetTrName(), GetTrNumname()   */
char *names;     /* fuer ReadMciFile(), FreeAll() */
                 /* (extern)                      */

extern char* net_file;
/**************************************************************************/
/* Erzeugung einer Kante.                                                 */
/**************************************************************************/

static Edge *create_edge(Condition *condition,Event *event,
                         Edge *nextcondition,Edge *nextevent)
{
    Edge *edge;

    edge=(Edge *)MyMalloc(sizeof(Edge));
    edge->condition=condition;
    edge->event=event;
    edge->nextcondition=nextcondition;
    edge->nextevent=nextevent;
    return edge;
}

/**************************************************************************/
/* Loeschen eines Netzes.                                                 */
/**************************************************************************/

static void sub_del(VCEvent *vce)
{
    if (vce) {
        sub_del(vce->next);
        SetDelete(vce->causes_corr_event);
        SetDelete(vce->causes_event);
        IListDelete(vce->ilist);
        MyFree(vce);
    }
}

void DeleteBraProcess(BraProcess *bprocess)
{
    register Edge    *edge,*edge_mem;
    register Event   *event;

    if (bprocess) {
        sub_del(bprocess->vcevents);
        event=++bprocess->events;
        while (event) {
            edge=event->preset;
            while (edge) {
                edge=(edge_mem=edge)->nextcondition;
                MyFree(edge_mem);
            }
            edge=event->postset;
            while (edge) {
                edge=(edge_mem=edge)->nextcondition;
                MyFree(edge_mem);
            }
            if (event->info) {
                SetDelete(event->info->causes_corr_event);
                SetDelete(event->info->causes_event);
                if (event->info->ilist)
                    IListDelete(event->info->ilist);
                MyFree(event->info);
            }
            event=event->next;
        }
        if (bprocess->events)
            MyFree(bprocess->events);
        if (bprocess->conditions)
            MyFree(++bprocess->conditions);
        MyFree(bprocess);
    }
}

/**************************************************************************/
/* Den numerischen Namen einer Stellenbeschriftung liefern.               */
/**************************************************************************/

unsigned GetPlNumname(char *ptr)
{
    char **calc_ptr;

    calc_ptr=pl_names;
    while (*calc_ptr) {
        if (!strcmp(*calc_ptr,ptr))
            return ((unsigned)(calc_ptr-pl_names))+1;
        ++calc_ptr;
    }
    return 0;
}

/**************************************************************************/
/* Die Stellenbeschriftung zu einem numerischen Namen liefern.            */
/**************************************************************************/

char *GetPlName(unsigned numname)
{
    return *(pl_names+numname-1);
}

/**************************************************************************/
/* Den numerischen Namen einer Transitionsbeschriftung liefern.           */
/**************************************************************************/

unsigned GetTrNumname(char *ptr)
{
    char **calc_ptr;

    calc_ptr=tr_names;
    while (*calc_ptr) {
        if (!strcmp(*calc_ptr,ptr))
            return ((unsigned)(calc_ptr-tr_names))+1;
        ++calc_ptr;
    }
    return 0;
}

/**************************************************************************/
/* Die Transitionsbeschriftung zu einem numerischen Namen liefern.        */
/**************************************************************************/

char *GetTrName(unsigned numname)
{
    return *(tr_names+numname-1);
}

/**************************************************************************/
/* Einlesen einer mci-Datei.                                              */
/**************************************************************************/

#define read_num ReadNumMCI
extern bool GL_SwapNum;
extern Cardinal32 swap_endian( Cardinal32 in);

/* Read maximal configs */
int GetMaxConfigs( FILE *in, BraProcess *bprocess, char *maxconf_file )
{
  unsigned i=0,num;
  Set *set;

  max_configs=SListCreate(set=SetCreate());
#ifndef IGNORE_MCI_MAXCONF
  while ((num=read_num(in))) {
    ++i;
    if (!set)
      SListNoTestInsert(max_configs,set=SetCreate());
    do {
      SetInsertElement(set,num);
    } while ((num=read_num(in)));
    set=NULL;
  }
#endif
  if (set)
  {
    FILE *mc;
    char buffer[32768],*bufptr,*tmpptr;

printf("read maxconf from file %s\n",maxconf_file);
#ifdef IGNORE_MCI_MAXCONF
    // skip configs
    while ((num=read_num(in))) {
      while ((num=read_num(in)))
      skip;
    }
#endif
    /* Read maximal configurations from generated file */
    if (!(mc=fopen(maxconf_file, "r"))) {
      if (!(outfile=fopen(maxconf_file, "w"))) {
        printf("Could not get maxconfs!\n");
        exit(1);
      } else {
        printf("Calculating maxconfs!\n");
	configs(readPrefix(net_file, NULL, NULL));
	if (!(mc=fopen(maxconf_file, "r"))) {
	  printf("Could not read calculated maxconfs!\n");
	  exit(1);
	}
      }
    }
    while (fgets(buffer, 32768, mc))
    {
      tmpptr=buffer;
      if (!set)
        SListNoTestInsert(max_configs,set=SetCreate());
      do {
	bufptr=tmpptr;
	num=strtol(bufptr,&tmpptr,10);
	if (num==0) break;
	SetInsertElement(set,num);
	++i;
      } while (tmpptr!=bufptr);
      set=NULL;
    }
  } else {
printf("read maxconf from unfolding\n");
  }
printf("Found %d maxconfs.\n",i);
  return (anz_max_configs=i);
}

void ReadMciFile(FILE *in, char *maxconf_file)
{
    Condition  *condition;
    Event      *event;
    char       *ptr;
    char       **calc_ptr;
    unsigned    i,num;

    bprocess=(BraProcess *)MyMalloc(sizeof(BraProcess));
    bprocess->vcevents=NULL;
    bprocess->num_vce=0;
    bprocess->num_conditions=read_num(in);
    /* Try to be clever ;-) */
    if (((bprocess->num_conditions)>>24)!=0)
      {
	fprintf(stderr,"Byte swapping in action!\n");
	GL_SwapNum=True;
	bprocess->num_conditions=swap_endian(bprocess->num_conditions);
      }
    bprocess->num_events=read_num(in);
    bprocess->conditions
    =(Condition *)MyMalloc(sizeof(Condition)*bprocess->num_conditions);
    if (bprocess->num_events) {
        bprocess->events
        =(Event *)MyMalloc(sizeof(Event)*bprocess->num_events);
        event=bprocess->events;
        for (i=1; i<=bprocess->num_events; ++i) {
            event->numname=i;
            event->tr_numname=read_num(in);
            event->preset=event->postset=NULL;
            event->info=NULL;
            event=event->next=event+1;
        }
        (--event)->next=NULL;
        event=bprocess->events-1;
    } else
        event=bprocess->events=NULL;
    condition=bprocess->conditions;
    for (i=1; i<=bprocess->num_conditions; ++i) {
        condition->numname=i;
        condition->pl_numname=read_num(in);
        condition->preset=condition->postset=NULL;
        num=read_num(in);
        if (num)
            (event+num)->postset=condition->preset
            =create_edge(condition,event+num,
                         (event+num)->postset,condition->preset);
        num=read_num(in);
        while(num) {
            (event+num)->preset=condition->postset
            =create_edge(condition,event+num,
                         (event+num)->preset,condition->postset);
            num=read_num(in);
        }
        condition=condition->next=condition+1;
    }
    (--condition)->next=NULL;
    --bprocess->conditions;  /* Damit spaeter z.B. ((bprocess->events)+1)  */
    --bprocess->events;      /* auf das erste Ereignis in der Liste zeigt. */
    initial_marking=LCond(min=BpMin(bprocess));
    cutoffs=SetCreate();
    num=read_num(in);
    while (num) {
        SetInsertElement(cutoffs,num);
        i=num; num=read_num(in);
        ((bprocess->events+i)->info=(Info *)MyMalloc(sizeof(Info)))
        ->corr_event=num;
        num=read_num(in);
    }
    GetMaxConfigs(in, bprocess, maxconf_file);
    num=read_num(in); /* # Places */
    pl_names=(char **)MyMalloc(sizeof(char *)*(num+1));
    i=num;
    num=read_num(in); /* # Transitions */
    tr_names=(char **)MyMalloc(sizeof(char *)*(num+1));
    i+=num;
    num=read_num(in); /* max. Laenge eines Namens */
    names=(char *)MyMalloc(num=(num+1)*i+2);
    names=(char *)MyRealloc(names,fread((void *)names,1,num,in));
    calc_ptr=pl_names;
    ptr=names;
    while (*ptr) {
        *calc_ptr=ptr;
        ++calc_ptr;
        while (*++ptr)
            ;
        ++ptr;
    }
    *calc_ptr=NULL;
    calc_ptr=tr_names;
    ++ptr;
    while (*ptr) {
        *calc_ptr=ptr;
        ++calc_ptr;
        while (*++ptr)
            ;
        ++ptr;
    }
    *calc_ptr=NULL;
}

/**********************************************************************/
/* Deallocate all main structures.                                    */
/**********************************************************************/

void FreeAll(void)
{
    DeleteBraProcess(bprocess);
    SetDelete(cutoffs);
    if (l2_configs) {
        SListDelete(l2_configs);
        MyFree(max_configs);
    } else
        SListDelete(max_configs);
    SetDelete(min);
    SetDelete(initial_marking);
    MyFree(pl_names);
    MyFree(tr_names);
    MyFree(names);
    RelDelete();
}

/*************************/
/* END OF FILE occ_net.c */
/*************************/
