/*****************************/
/* relation.c                */
/* Burkhard Graves, 26.10.94 */
/*****************************/

#include <limits.h>
#include "relation.h"
#include "mca.h"
#include "l2_mc.h"

/*******************/
/* Typdefinitionen */
/*******************/

/*---------------------*/
/* Knoten der Relation */
/*---------------------*/

typedef struct knoten {
    unsigned color;         /* Farbe des Knotens              */
    struct kante *first;    /* Zeiger auf Liste von Kanten    */
} Knoten;

/*--------------------*/
/* Kante der Relation */
/*--------------------*/

typedef struct kante {
    Set *edges;             /* Art der Kante                */
    Knoten *to;             /* Ziel der Kante               */
    struct kante *next;     /* Zeiger auf naechste Kante    */
    unsigned is_ok;         /* Flag, ob Kante ok            */
} Kante;

/********************/
/* Globale Variable */
/********************/

unsigned the_s;      /* fuer ExInfPath(), ExMaxFreePath(), ExPath()  */
unsigned found;      /* fuer ExInfPath(), ExMaxFreePath()            */

  Knoten *node=NULL; /* fuer "alle" (Initialisierung notwendig !)    */

/**************************************************************************/
/* Initialisierung der (noch leeren) Relation.                            */
/**************************************************************************/

void RelInit(void)
{
    unsigned max;
    Knoten *ptr;

    ptr=node=(Knoten *)MyMalloc(sizeof(Knoten)*(max=anz_l2_c));
    while (max) {
        ptr->color=0;
        ptr->first=NULL;
        ++ptr;
        --max;
    }
}

/**************************************************************************/
/* Markierungen der Knoten der Relation loeschen.                         */
/* Bemerkung: Unterroutine von 'ExPath()', 'ExMaxFreePath()'              */
/*                         und 'ExInfPath()'                              */
/**************************************************************************/

static void rel_clean(void)
{
    Knoten *ptr;
    unsigned i;

    ptr=node;
    for (i=anz_l2_c; i; --i) {
        ptr->color=0;
        ++ptr;
    }
}

/**************************************************************************/
/* Entfernen der Relation.                                                */
/**************************************************************************/

void RelDelete(void)
{
    Kante *kante,*kante_mem;
    Knoten *ptr;
    unsigned i;

    if ((ptr=node)!=NULL) {
        for (i=anz_l2_c; i; --i) {
            kante=ptr->first;
            while (kante) {
                SetDelete(kante->edges);
                kante=(kante_mem=kante)->next;
                MyFree(kante_mem);
            }
            ++ptr;
        }
        MyFree(node);
    }
}

/**************************************************************************/
/* Speichern der Relation.                                                */
/**************************************************************************/

void RelSave(FILE *out)
{
    Kante *kante;
    Knoten *ptr,*mem;
    unsigned max;
    SList *slist;

    max=anz_l2_c-anz_max_configs;
    fwrite((void *)&max,sizeof(max),1,out);
    slist=l2_configs;
    while (max) {
        slist=slist->next;
        assert(slist && slist->set);
        SetSave(out,slist->set);
        --max;
    }
    fwrite((void *)(ptr=node),max=anz_l2_c,sizeof(*ptr),out);
    while (max) {
        kante=ptr->first;
        while (kante) {
            (kante->to)=(Knoten *)((mem=kante->to)-node);
            fwrite((void *)kante,sizeof(*kante),1,out);
            kante->to=mem;
            if (kante->edges) {
                SetSave(out,kante->edges);
            }
            kante=kante->next;
        }
        ++ptr;
        --max;
    }
}

/**************************************************************************/
/* Laden der Relation.                                                    */
/**************************************************************************/

void RelLoad(FILE *in)
{
    Kante *kante;
    Knoten *ptr;
    Set *set;
    unsigned max;

    l2_configs=SListHeadDuplicate(max_configs);
    fread((void *)&max,1,sizeof(max),in);
    anz_l2_c=max+anz_max_configs;
    while (max) {
        set=SetRead(in);
        SListNoTestInsert(l2_configs,set);
        --max;
    }
    ptr=node=(Knoten *)MyMalloc(sizeof(Knoten)*(max=anz_l2_c));
    fread((void *)ptr,max,sizeof(*ptr),in);
    while (max) {
        if (ptr->first) {
            kante=ptr->first=(Kante *)MyMalloc(sizeof(Kante));
            for (;;) {
                fread((void *)kante,sizeof(*kante),1,in);
                kante->to=node+(unsigned)(kante->to);
                if (kante->edges) {
                    kante->edges=SetRead(in);
                }
                if (kante->next) {
                    kante=kante->next=(Kante *)MyMalloc(sizeof(Kante));
                } else {
                    break;
                }
            }
        }
        ++ptr;
        --max;
    }

#ifdef OUTPUT
    RelDisplay(yyout);
#endif

}

/**************************************************************************/
/* Eine mit der Menge 'set' beschriftete Kante von 'source' nach 'target' */
/* zur Relation hinzufuegen.                                              */
/**************************************************************************/

void RelAddKante(unsigned source,unsigned target,unsigned is_ok,Set *set)
{
    Knoten *knoten;
    Kante *kante;

    (kante=(Kante *)MyMalloc(sizeof(Kante)))->to=((knoten=node)+target);
    kante->is_ok=is_ok;
    kante->next=(knoten+=source)->first;
    (knoten->first=kante)->edges=set;
}

/**************************************************************************/
/* Relation ausgeben.                                                     */
/**************************************************************************/

void RelDisplay(FILE *out)
{
    Kante *kante;
    Knoten *ptr;
    unsigned i;

    fprintf(out,"|H|=%u\n",anz_l2_c);
    SListDisplay(out,l2_configs);
    ptr=node;
    for (i=0; i<anz_l2_c; ++i) {
        fprintf(out,"%u: ",i);
        kante=ptr->first;
        while (kante) {
            fprintf(out,"%u(%c)",(unsigned)((kante->to)-node)
                                ,(kante->is_ok)?'+':'-');
            fputc('{',out);
            DisplayPlaceNames(out,kante->edges);
            fprintf(out,"} ");
            kante=kante->next;
        }
        ++ptr;
        fputc('\n',out);
    }
}

/**************************************************************************/
/* Unterroutine von 'ExPath()'; modifizierte Tiefensuche.                 */
/**************************************************************************/

static int sub_ex_path(Knoten *ptr)
{
    Kante *kante;

    ptr->color=1;
    kante=ptr->first;
    while (kante) {
        if (SetIsElement(kante->edges,the_s)
            || (!kante->to->color && sub_ex_path(kante->to))) {
            return 1;
        }
        kante=kante->next;
    }
    return 0;
}

/**************************************************************************/
/* Test, ob - ausgehend von der 'c'-ten maximalen Konfiguration - ein     */
/* Pfad existiert, der eine mit 's' markierte Kante enthaelt.             */
/**************************************************************************/

int ExPath(unsigned c,unsigned s)
{
    rel_clean();
    the_s=s;
    return sub_ex_path(node+c);
}

/**************************************************************************/
/* Unterroutine von 'ExMaxFreePath()'; modifizierte Tiefensuche.          */
/**************************************************************************/

static int sub_ex_max_free_path(Knoten *ptr)
{
    Kante *kante;

    ptr->color=found;
    kante=ptr->first;
    while (kante) {
        if (kante->is_ok)
            if (!++found)
                Fatal("ExMaxFreePath(): Overflow !");
        if (!SetIsElement(kante->edges,the_s)
            && ((!kante->to->color && sub_ex_max_free_path(kante->to))
                || kante->to->color<found)) {
            return 1;
        }
        if (kante->is_ok)
            --found;
        kante=kante->next;
    }
    ptr->color=UINT_MAX;
    return 0;
}

/**************************************************************************/
/* Test, ob - ausgehend von der 'c'-ten maximalen Konfiguration - ein     */
/* maximaler Pfad existiert, der keine mit 's' markierte Kante enthaelt.  */
/**************************************************************************/

int ExMaxFreePath(unsigned c,unsigned s)
{
    rel_clean();
    found=1;
    the_s=s;
    return sub_ex_max_free_path(node+c);
}

/**************************************************************************/
/* Unterroutine von 'ExInfPath()'; modifizierte Tiefensuche.              */
/**************************************************************************/

static int sub_ex_inf_path(Knoten *ptr)
{
    Kante *kante;

    ptr->color=found;
    kante=ptr->first;
    while (kante) {
        if (SetIsElement(kante->edges,the_s)) {
            if (!++found)
                Fatal("ExMaxFreePath(): Overflow !");
            if ((!kante->to->color && sub_ex_inf_path(kante->to))
                || kante->to->color<found) {
                return 1;
            }
            --found;
        } else {
            if ((!kante->to->color && sub_ex_inf_path(kante->to))
                || kante->to->color<found) {
                return 1;
            }
        }
        kante=kante->next;
    }
    ptr->color=UINT_MAX;
    return 0;
}

/**************************************************************************/
/* Test, ob - ausgehend von der 'c'-ten maximalen Konfiguration -         */
/* ein Pfad mit unendlich vielen 's'-Kanten existiert.                    */
/**************************************************************************/

int ExInfPath(unsigned c,unsigned s)
{
    rel_clean();
    found=1;
    the_s=s;
    return sub_ex_inf_path(node+c);
}

/**************************/
/* END OF FILE relation.c */
/**************************/
