/*****************************************************************************/
/* convertPEP.c                                 (c) Stefan Roemer 1993-1997  */
/*                                              Version h-1.5.1 (07.03.1997) */
/*****************************************************************************/
/* Schnittstellen-Bibliothek zum Lesen (Schreiben) des PEP-Netzdateiformats  */
/* in die (aus den) grundlegenden Datenstrukturen fuer Petri-Netze.          */
/*****************************************************************************/
/*                                   Stefan Roemer, 13.12.1993 - 01.08.1994  */
/* Neues PEP-Netzdateiformat:        Stefan Roemer, 19.12.1994 - 24.01.1995  */
/* Erweiterungen FORMAT_N2(Phantom): Stefan Roemer, 31.05.1996 - 11.06.1996  */
/* Erweiterungen FORMAT_N2(Bloecke): Stefan Roemer, 07.03.1997 - 07.03.1997  */
/*****************************************************************************/
/*                                                                           */
/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
/*                                                                           */
/* ACHTUNG: Diese Datei enthaelt elementare Routinen, die von vielen ver-    */
/* schiedenen Programmen benoetigt werden.                                   */
/*                                                                           */
/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "convertPEP.h"
#include "readlib.h"
#include "readPEP.h"
#include "rw_pnml.h"

/*****************************************************************************/

char       CM_nettype[ 31 ];
bool       CM_storeMeaning      = False,
           CM_storeCoordinates  = False,
           CM_storeReferences   = False,
           CM_use_imarks        = True;
M_LIST     CM_placeMList        = NULL,
           CM_transMList        = NULL;
Cardinal32 CM_countMPlaces      = 0,
           CM_countPMeanings    = 0,
           CM_countMTransitions = 0,
           CM_countTMeanings    = 0;

/*****************************************************************************/
/*                                Funktionen                                 */
/*****************************************************************************/

/*****************************************************************************/
/* Einlesen einer Zeichenkette aus einer Textdatei; fuehrende Leerzeichen    */
/* werden ueberlesen und nicht dem String zugeordnet; Kommentarzeilen werden */
/* ueberlesen. Funktion fuer das neue Dateiformat.                           */
/*                                                             Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static int ReadEnclString( FILE * file, char * string, int maxlen )

#else

static int ReadEnclString( file, string, maxlen )

  FILE * file;
  char * string;
  int  maxlen;

#endif

  {

    register char ch    = '\0',
                  * str = string;
    register int  len    = 0;
    char          delimiter;

    if (    ( ( delimiter = ReadCharComment( file ) ) != '\'' )
         && ( delimiter != '"' ) )
      {
        NNError( "ReadEnclString(): string leading ' or \" expected" );
      } /* if */
    while ( ! feof( file ) && ( ( *str = getc( file ) ) != delimiter )
            && ( len < maxlen ) )
      {
        str++;
        len++;
      } /* while */
    if ( ( len == maxlen ) && ( *str != delimiter ) )
      {
        while ( ! feof( file ) && ( ( ch = getc( file ) ) != delimiter ) )
          skip;
      } /* if */
    else
      {
        ch = *str;
      } /* else */
    if ( ch != delimiter )
      {
        sprintf( string, "ReadEnclString(): closing %c is missing",
                                                                   delimiter );
        NNError( string );
      } /* if */
    *str = '\0';
    
    return len;

  } /* ReadEnclString() */


/*****************************************************************************/
/* Einlesen einer Zeichenkette aus einer Textdatei; fuehrende Leerzeichen    */
/* werden ueberlesen und nicht dem String zugeordnet.                        */
/*                                                             Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static void ReadCommand( FILE * file, char * string, int maxlen )

#else

static void ReadCommand( file, string, maxlen )

  FILE * file;
  char * string;
  int  maxlen;

#endif

  {

    if ( ReadAlphaName( file, string, maxlen ) != 3 )
      {
        NNError( "ReadCommand(): unknown command found" );
      } /* if */
    ReadSymbol( file, ':' );

  } /* ReadCommand() */


/*****************************************************************************/
/*********************** insert_place_meaning() ******************************/
/*****************************************************************************/
/*                                                  08.08.1994 Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static M_LIST insert_place_meaning( char * meaning, Place pl )

#else

static M_LIST insert_place_meaning( meaning, pl )

  char  * meaning;
  Place pl;

#endif

  {

    register M_LIST mptr,
                    newmptr;
    register P_LIST pptr;
    register char   * rptr,
                    * wptr;
    register int    comp = -1;

    rptr = wptr = meaning;
    while ( *rptr )
      {
        if ( ! isspace( *rptr ) )
          {
            *( wptr++ ) = *rptr;
          } /* if */
        rptr++;
      } /* while */
    *wptr = *rptr;
    if (    ! ( mptr = CM_placeMList )
         || ( strcmp( meaning, mptr->meaning ) < 0 ) )
      {
        ( newmptr = (M_LIST) Malloc( sizeof( M_LIST_ELEM ) ) )
                                                        ->next = CM_placeMList;
        ( CM_placeMList = newmptr )->number = 1;
        newmptr->meaning = Strdup( meaning );
        CM_countPMeanings++;
        ( pptr = newmptr->list = (P_LIST) Malloc( sizeof( P_LIST_ELEM ) ) )
                                                                 ->next = NULL;
        pptr->item = pl;
      } /* if */
    else
      {
        while (    ( newmptr = mptr->next )
                && ( ( comp = strcmp( newmptr->meaning, meaning ) ) < 0 ) )
          {
            mptr = newmptr;
          } /* while */
        if ( newmptr && ! comp )
          {
            ( pptr = (P_LIST) Malloc( sizeof( P_LIST_ELEM ) ) )
                                                        ->next = newmptr->list;
            ( newmptr->list = pptr )->item = pl;
            newmptr->number++;
          } /* if */
        else
          {
            ( newmptr = (M_LIST) Malloc( sizeof( M_LIST_ELEM ) ) )
                                                           ->next = mptr->next;
            mptr->next = newmptr;
            newmptr->meaning = Strdup( meaning );
            newmptr->number = 1;
            CM_countPMeanings++;
            ( pptr = newmptr->list = (P_LIST) Malloc( sizeof( P_LIST_ELEM ) ) )
                                                                 ->next = NULL;
            pptr->item = pl;
          } /* else */
      } /* else */

    CM_countMPlaces++;

    return ( CM_placeMList );

  } /* insert_place_meaning() */


/*****************************************************************************/
/*********************** insert_trans_meaning() ******************************/
/*****************************************************************************/
/*                                                  08.08.1994 Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static M_LIST insert_trans_meaning( char * meaning, Transition tr )

#else

static M_LIST insert_trans_meaning( meaning, tr )

  char       * meaning;
  Transition tr;

#endif

  {

    register M_LIST mptr,
                    newmptr;
    register P_LIST pptr;
    register char   * rptr,
                    * wptr;
    register int    comp = -1;

    rptr = wptr = meaning;
    while ( *rptr )
      {
        if ( ! isspace( *rptr ) )
          {
            *( wptr++ ) = *rptr;
          } /* if */
        rptr++;
      } /* while */
    *wptr = *rptr;
    if (    ! ( mptr = CM_transMList )
         || ( strcmp( meaning, mptr->meaning ) < 0 ) )
      {
        ( newmptr = CM_transMList = (M_LIST) Malloc( sizeof( M_LIST_ELEM ) ) )
                                                                 ->next = NULL;
        newmptr->meaning = Strdup( meaning );
        newmptr->number = 1;
        CM_countTMeanings++;
        ( pptr = newmptr->list = (P_LIST) Malloc( sizeof( P_LIST_ELEM ) ) )
                                                                 ->next = NULL;
        pptr->item = (Place) tr;
      } /* if */
    else
      {
        while (    ( newmptr = mptr->next )
                && ( ( comp = strcmp( newmptr->meaning, meaning ) ) < 0 ) )
          {
            mptr = newmptr;
          } /* while */
        if ( newmptr && ! comp )
          {
            ( pptr = (P_LIST) Malloc( sizeof( P_LIST_ELEM ) ) )
                                                        ->next = newmptr->list;
            ( newmptr->list = pptr )->item = (Place) tr;
            newmptr->number++;
          } /* if */
        else
          {
            ( newmptr = (M_LIST) Malloc( sizeof( M_LIST_ELEM ) ) )
                                                           ->next = mptr->next;
            mptr->next = newmptr;
            newmptr->meaning = Strdup( meaning );
            newmptr->number = 1;
            CM_countTMeanings++;
            ( pptr = newmptr->list = (P_LIST) Malloc( sizeof( P_LIST_ELEM ) ) )
                                                                 ->next = NULL;
            pptr->item = (Place) tr;
          } /* else */
      } /* else */

    CM_countMTransitions++;

    return ( CM_transMList );

  } /* insert_trans_meaning() */


/*****************************************************************************/
/************************ delete_meaning_list() ******************************/
/*****************************************************************************/
/*                                                  08.08.1994 Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void delete_meaning_list( M_LIST list )

#else

void delete_meaning_list( list )

  M_LIST list;

#endif

  {

    register M_LIST mdel;
    register P_LIST pptr,
                    pdel;

    mdel = list;
    while ( mdel )
      {
        Free( mdel->meaning );
        pdel = mdel->list;
        while ( pdel )
          {
            pdel = ( pptr = pdel )->next;
            Free( pptr );
          } /* while */
        mdel = ( list = mdel )->next;
        Free( list );
      } /* while */

  } /* delete_meaning_list() */


/*****************************************************************************/
/******************************* read_PEP ************************************/
/*****************************************************************************/
/*                                                  13.12.1993 Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LMPNet read_PEP( char * PEPfilename )

#else

LMPNet read_PEP( PEPfilename )

  char * PEPfilename;

#endif

  {

    int                 x, y = 0,
                        r1, r2,
                        tx, ty,
                        cmarks, imarks, width, capacity,
                        num, num2, weight;
    register int        ch;
    register char       ** Namen,
                        * S_T_Flags;
    char                cmd[513],
                        name[4097],
                        meaning[4097],
                        name2[NAMELENGTH];
    register FILE       * infile;
    register LMPNet     net;
    register Cardinal32 AnzNamen,
                        MaxNamen,
                        count;
    Place               pl_last = NULL;
    Transition          tr_last = NULL;
    STEdge              st_last = NULL;
    TSEdge              ts_last = NULL;

    NNinput_file = PEPfilename;
    NNinput_line = 1;

    /* Oeffne PEP-Datei */
    if ( ! ( infile = fopen( PEPfilename, "r" ) ) )
      {
        fprintf( stderr,
              "ERROR: CM.read_PEP(): could not open file '%s' for reading.\n",
                 PEPfilename );
        exit ( 4 );
      } /* if */

    fgets( cmd, 512, infile );
    if ( strncmp( cmd, "PEP", 3 ) && strncmp( cmd, "MOBY", 4 ))
      {
        if (strncmp( cmd, "<?xml", 5 ))
          NNError( "keyword 'PEP' or 'MOBY' expected" );
	else
	{
	  fclose ( infile );
	  return readPNML(PEPfilename);
	}
      } /* if */
    ReadAlphaName( infile, cmd, 512 );
    if (    strcmp( cmd, "PetriBox" ) && strcmp( cmd, "Net" )
         && strcmp( cmd, "PTNet" )    && strcmp( cmd, "CENet" )
         && strcmp( cmd, "HLNet" ) )
      {
        NNError( "file identifier ( 'PTNet', 'PetriBox', ... ) expected" );
      } /* if */
    strncpy( CM_nettype, cmd, 30 );
    ReadAlphaName( infile, cmd, 512 );
    if ( ! ( strncmp( cmd, "FORMAT_N", 8 ) ) )
      {
	fclose ( infile ); /* Hack: read_new_PEP opens again */
        return ( read_new_PEP( PEPfilename ) );
      } /* if */
    else if ( strcmp( cmd, "FORMAT_A" ) )
      {
        NNError( "keyword 'FORMAT_A' expected" );
        return ( NULL );
      } /* if */

    /* Initialisiere net */
    Namen = (char **) Malloc( ( count = MaxNamen = NAMES_START )
                                                          * sizeof( char * ) );
    S_T_Flags = (char *) Malloc( count * sizeof( char ) );
    AnzNamen = 0;
    while ( --count )
      {
        Namen[ count ] = (char *) NULL;
        S_T_Flags[ count ] = '\0';
      } /* while */
    net = empty_net();
    net->NumComponents = 1;

    ReadCommand( infile, cmd, 512 );
    while ( strcmp( cmd, "END" ) )
      {
        if ( ! strcmp( cmd, "PLA" ) )
          {
            ReadCommand( infile, cmd, 512 );
            num = 0;
            imarks = cmarks = 0;
            *name = *meaning = '\0';
            while ( strcmp( cmd, "end" ) )
              {
                if ( ! strcmp( cmd, "idt" ) )
                  {
                    num = ReadNumber( infile );
                  } /* if */
                else if ( ! strcmp( cmd, "nam" ) )
                  {
                    ReadEnclString( infile, name, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "mea" ) )
                  {
                    ReadEnclString( infile, meaning, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dnp" ) )
                  {
                    ReadCoordinates( infile, &tx, &ty );
                  } /* else if */
                else if ( ! strcmp( cmd, "dsz" ) )
                  {
                    ReadCoordinates( infile, &r1, &r2 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dpt" ) )
                  {
                    ReadCoordinates( infile, &x, &y );
                  } /* else if */
                else if ( ! strcmp( cmd, "dwd" ) )
                  {
                    width = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "cap" ) )
                  {
                    while ( ( ch = getc( infile ) ) == ' ' )
                      skip;
                    if ( isdigit( ch ) )
                      {
                        ungetc( ch, infile );
                        capacity = ReadNumber( infile );
                      } /* if */
                    else
                      {
                        ungetc( ch, infile );
                        ReadAlphaName( infile, name2, NAMELENGTH-1 );
                        if ( strcmp( name2, "Omega" ) )
                          {
                            NNError( "incorrect capacity" );
                          } /* if */
                      } /* else */
                  } /* else if */
                else if ( ! strcmp( cmd, "ima" ) )
                  {
                    imarks = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "cma" ) )
                  {
                    cmarks = ReadNumber( infile );
                  } /* else if */
                ReadCommand( infile, cmd, 512 );
              } /* while */
            if ( ! num )
              {
                NNError( "missing place identifier" );
              } /* if */
            if ( num > AnzNamen )
              {
                AnzNamen = num;
              } /* if */
            else if ( Namen[ num ] )
              {
                NNError( "multiply defined place identificator" );
              } /* else if */
            if ( AnzNamen >= MaxNamen )
              {
                Namen = (char **) Realloc( (char *) Namen,
                             ( MaxNamen += NAMES_OFFSET ) * sizeof( char * ) );
                S_T_Flags = (char *) Realloc( (char *) S_T_Flags,
                                                   MaxNamen * sizeof( char ) );
                for ( count = MaxNamen - NAMES_OFFSET ;
                                                   count < MaxNamen ; count++ )
                  {
                    Namen[ count ] = (char *) NULL;
                    S_T_Flags[ count ] = '\0';
                  } /* for */
              } /* if */
            if ( ! *name )
              {
                sprintf( name, "S%d", num );
              } /* if */
            Namen[ num ] = fast_insert_place( net, name,
                           CM_use_imarks ? imarks : cmarks, &pl_last ) -> name;
            if ( CM_storeMeaning && *meaning )
              {
                insert_place_meaning( meaning, pl_last );
              } /* if */
            pl_last->label = EMPTY;
            pl_last->component = 0;
            S_T_Flags[ num ] = 'S';
            if ( CM_storeCoordinates )
              {
                Coords co;
                pl_last->inscript = (char *)
                              ( co = (Coords) Malloc( sizeof( CoordEntry ) ) );
                co->next = NULL;
                co->x = x;
                co->y = y;
              } /* if */
          } /* if */
        else if (    ! strcmp( cmd, "TRA" )
                  || ! strcmp( cmd, "RTR" ) )
          {
            ReadCommand( infile, cmd, 512 );
            num = 0;
            *name = *meaning =  '\0';
            while ( strcmp( cmd, "end" ) )
              {
                if ( ! strcmp( cmd, "idt" ) )
                  {
                    num = ReadNumber( infile );
                  } /* if */
                else if ( ! strcmp( cmd, "nam" ) )
                  {
                    ReadEnclString( infile, name, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "mea" ) )
                  {
                    ReadEnclString( infile, meaning, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dnp" ) )
                  {
                    ReadCoordinates( infile, &tx, &ty );
                  } /* else if */
                else if ( ! strcmp( cmd, "dsz" ) )
                  {
                    ReadCoordinates( infile, &r1, &r2 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dpt" ) )
                  {
                    ReadCoordinates( infile, &x, &y );
                  } /* else if */
                else if ( ! strcmp( cmd, "dwd" ) )
                  {
                    width = ReadNumber( infile );
                  } /* else if */
                ReadCommand( infile, cmd, 512 );
              } /* while */
            if ( ! num )
              {
                NNError( "missing transition identifier" );
              } /* if */
            if ( num > AnzNamen )
              {
                AnzNamen = num;
              } /* if */
            else if ( Namen[ num ] )
              {
                NNError( "multiply defined transition identifier" );
              } /* else if */
            if ( AnzNamen >= MaxNamen )
              {
                Namen = (char **) Realloc( (char *) Namen,
                                     ( MaxNamen += NAMES_OFFSET ) * sizeof( char * ) );
                S_T_Flags = (char *) Realloc( (char *) S_T_Flags,
                                                   MaxNamen * sizeof( char ) );
                for ( count = MaxNamen - NAMES_OFFSET ; count < MaxNamen ; count++ )
                  {
                    Namen[ count ] = (char *) NULL;
                    S_T_Flags[ count ] = '\0';
                  } /* for */
              } /* if */
            if ( ! *name )
              {
                sprintf( name, "T%d", num );
              } /* if */
            Namen[ num ] = fast_insert_transition( net, name, &tr_last )
                                                                       -> name;
            if ( CM_storeMeaning && *meaning )
              {
                insert_trans_meaning( meaning, tr_last );
              } /* if */
            tr_last->label = NULL;
            S_T_Flags[ num ] = 'T';
            if ( CM_storeCoordinates )
              {
                Coords co;
                tr_last->inscript = (char *)
                              ( co = (Coords) Malloc( sizeof( CoordEntry ) ) );
                co->next = NULL;
                co->x = x;
                co->y = y;
              } /* if */
          } /* else if */
        else if ( ! strcmp( cmd, "ARC" ) )
          {
            Coords lastco  = NULL,
                   firstco = NULL;
            ReadCommand( infile, cmd, 512 );
            num = num2 = weight = 0;
            *name = *meaning = '\0';
            while ( strcmp( cmd, "end" ) )
              {
                if ( ! strcmp( cmd, "lpt" ) )
                  {
                    ReadCoordinates( infile, &x, &y );
                  } /* else if */
                else if ( ! strcmp( cmd, "dsz" ) )
                  {
                    width = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "wgt" ) )
                  {
                    weight = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "sou" ) )
                  {
                    num = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "des" ) )
                  {
                    num2 = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "nam" ) )
                  {
                    ReadEnclString( infile, name, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "mea" ) )
                  {
                    ReadEnclString( infile, meaning, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "vsf" ) )
                  {
                    char flag[2];
                    ReadAlphaName( infile, flag, 1 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dpt" ) )
                  {
                    ReadCoordinates( infile, &x, &y );
                    if ( CM_storeCoordinates )
                      {
                        if ( lastco )
                          {
                            lastco = lastco->next
                                   = (Coords) Malloc( sizeof( CoordEntry ) );
                          } /* if */
                        else
                          {
                            firstco = lastco
                                    = (Coords) Malloc( sizeof( CoordEntry ) );
                          } /* else */
                        lastco->next = NULL;
                        lastco->x = x;
                        lastco->y = y;
                      } /* if */
                  } /* else if */
                ReadCommand( infile, cmd, 512 );
              } /* while */
            if ( ! num || ( num > AnzNamen ) || ! Namen[ num ] )
              {
                NNError( "arc: missing source identifier" );
              } /* if */
            if ( ! num2 || ( num2 > AnzNamen ) || ! Namen[ num2 ]  )
              {
                NNError( "arc: missing destination identifier" );
              } /* if */
            if ( ( S_T_Flags[ num ] == 'S' ) && ( S_T_Flags[ num2 ] == 'T' ) )
              {
                fast_insert_PT_con( net, Namen[ num ], Namen[ num2 ],
                                        weight, &pl_last, &tr_last, &st_last );
                if ( CM_storeCoordinates )
                  {
                    lastco = (Coords) Malloc( sizeof( CoordEntry ) );
                    lastco->x = weight;
                    lastco->y = 0;
                    lastco->next = firstco;
                    st_last->weight = (Cardinal32) lastco;
                  } /* if */
              } /* if */
            else if (    ( S_T_Flags[ num ] == 'T' )
                      && ( S_T_Flags[ num2 ] == 'S' ) )
              {
                fast_insert_TP_con( net, Namen[ num2 ], Namen[ num ],
                                        weight, &pl_last, &tr_last, &ts_last );
                if ( CM_storeCoordinates )
                  {
                    lastco = (Coords) Malloc( sizeof( CoordEntry ) );
                    lastco->x = weight;
                    lastco->y = 0;
                    lastco->next = firstco;
                    ts_last->weight = (Cardinal32) lastco;
                  } /* if */
              } /* if */
            else
              {
                NNError( "faulty PEP net file: S->S resp. T->T arc found" );
              } /* else */
          } /* else if */
        else if ( ! strcmp( cmd, "IPL" ) )
          {
            ReadCommand( infile, cmd, 512 );
            num = 0;
            imarks = 1;
            cmarks = 0;
            *name = *meaning = '\0';
            while ( strcmp( cmd, "end" ) )
              {
                if ( ! strcmp( cmd, "idt" ) )
                  {
                    num = ReadNumber( infile );
                  } /* if */
                else if ( ! strcmp( cmd, "nam" ) )
                  {
                    ReadEnclString( infile, name, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "mea" ) )
                  {
                    ReadEnclString( infile, meaning, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dnp" ) )
                  {
                    ReadCoordinates( infile, &tx, &ty );
                  } /* else if */
                else if ( ! strcmp( cmd, "dsz" ) )
                  {
                    ReadCoordinates( infile, &r1, &r2 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dpt" ) )
                  {
                    ReadCoordinates( infile, &x, &y );
                  } /* else if */
                else if ( ! strcmp( cmd, "dwd" ) )
                  {
                    width = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "cap" ) )
                  {
                    while ( ( ch = getc( infile ) ) == ' ' )
                      skip;
                    if ( isdigit( ch ) )
                      {
                        ungetc( ch, infile );
                        capacity = ReadNumber( infile );
                      } /* if */
                    else
                      {
                        ungetc( ch, infile );
                        ReadAlphaName( infile, name2, NAMELENGTH-1 );
                        if ( strcmp( name2, "Omega" ) )
                          {
                            fprintf( stderr,
                                            "ERROR: CM.read_PEP(): capacity" );
                            fprintf( stderr, "'Omega' expected.\n" );
                          } /* if */
                      } /* else */
                  } /* else if */
                else if ( ! strcmp( cmd, "ima" ) )
                  {
                    imarks = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "cma" ) )
                  {
                    cmarks = ReadNumber( infile );
                  } /* else if */
                ReadCommand( infile, cmd, 512 );
              } /* while */
            if ( ! num )
              {
                NNError( "missing place identifier" );
              } /* if */
            if ( num > AnzNamen )
              {
                AnzNamen = num;
              } /* if */
            else if ( Namen[ num ] )
              {
                NNError( "multiply defined place identifer" );
              } /* else if */
            if ( AnzNamen >= MaxNamen )
              {
                Namen = (char **) Realloc( (char *) Namen,
                              ( MaxNamen += NAMES_OFFSET ) * sizeof( char * ) );
                S_T_Flags = (char *) Realloc( (char *) S_T_Flags,
                                                    MaxNamen * sizeof( char ) );
                for ( count = MaxNamen - NAMES_OFFSET ;
                                                    count < MaxNamen ; count++ )
                  {
                    Namen[ count ] = (char *) NULL;
                    S_T_Flags[ count ] = '\0';
                  } /* for */
              } /* if */
            if ( ! *name )
              {
                sprintf( name, "S%d", num );
              } /* if */
            Namen[ num ] = fast_insert_place( net, name,
                                 CM_use_imarks ? 1 : cmarks, &pl_last ) -> name;
            if ( CM_storeMeaning && *meaning )
              {
                insert_place_meaning( meaning, pl_last );
              } /* if */
            pl_last->label = ENTRY;
            pl_last->component = 0;
            S_T_Flags[ num ] = 'S';
            if ( CM_storeCoordinates )
              {
                Coords co;
                pl_last->inscript = (char *)
                              ( co = (Coords) Malloc( sizeof( CoordEntry ) ) );
                co->next = NULL;
                co->x = x;
                co->y = y;
              } /* if */
          } /* else if */
        else if ( ! strcmp( cmd, "OPL" ) )
          {
            ReadCommand( infile, cmd, 512 );
            num = imarks = cmarks = 0;
            *name = *meaning = '\0';
            while ( strcmp( cmd, "end" ) )
              {
                if ( ! strcmp( cmd, "idt" ) )
                  {
                    num = ReadNumber( infile );
                  } /* if */
                else if ( ! strcmp( cmd, "nam" ) )
                  {
                    ReadEnclString( infile, name, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "mea" ) )
                  {
                    ReadEnclString( infile, meaning, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dnp" ) )
                  {
                    ReadCoordinates( infile, &tx, &ty );
                  } /* else if */
                else if ( ! strcmp( cmd, "dsz" ) )
                  {
                    ReadCoordinates( infile, &r1, &r2 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dpt" ) )
                  {
                    ReadCoordinates( infile, &x, &y );
                  } /* else if */
                else if ( ! strcmp( cmd, "dwd" ) )
                  {
                    width = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "cap" ) )
                  {
                    while ( ( ch = getc( infile ) ) == ' ' )
                      skip;
                    if ( isdigit( ch ) )
                      {
                        ungetc( ch, infile );
                        capacity = ReadNumber( infile );
                      } /* if */
                    else
                      {
                        ungetc( ch, infile );
                        ReadAlphaName( infile, name2, NAMELENGTH-1 );
                        if ( strcmp( name2, "Omega" ) )
                          {
                            fprintf( stderr,
                                            "ERROR: CM.read_PEP(): capacity" );
                            fprintf( stderr, "'Omega' expected.\n" );
                          } /* if */
                      } /* else */
                  } /* else if */
                else if ( ! strcmp( cmd, "ima" ) )
                  {
                    imarks = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "cma" ) )
                  {
                    cmarks = ReadNumber( infile );
                  } /* else if */
                ReadCommand( infile, cmd, 512 );
              } /* while */
            if ( ! num )
              {
                NNError( "missing place identifier" );
              } /* if */
            if ( num > AnzNamen )
              {
                AnzNamen = num;
              } /* if */
            else if ( Namen[ num ] )
              {
                NNError( "multiply defined place identifier" );
              } /* else if */
            if ( AnzNamen >= MaxNamen )
              {
                Namen = (char **) Realloc( (char *) Namen,
                              ( MaxNamen += NAMES_OFFSET ) * sizeof( char * ) );
                S_T_Flags = (char *) Realloc( (char *) S_T_Flags,
                                                    MaxNamen * sizeof( char ) );
                for ( count = MaxNamen - NAMES_OFFSET ;
                                                    count < MaxNamen ; count++ )
                  {
                    Namen[ count ] = (char *) NULL;
                    S_T_Flags[ count ] = '\0';
                  } /* for */
              } /* if */
            if ( ! *name )
              {
                sprintf( name, "S%d", num );
              } /* if */
            Namen[ num ] = fast_insert_place( net, name,
                           CM_use_imarks ? imarks : cmarks, &pl_last ) -> name;
            if ( CM_storeMeaning && *meaning )
              {
                insert_place_meaning( meaning, pl_last );
              } /* if */
            pl_last->label = EXIT;
            pl_last->component = 0;
            S_T_Flags[ num ] = 'S';
            if ( CM_storeCoordinates )
              {
                Coords co;
                pl_last->inscript = (char *)
                              ( co = (Coords) Malloc( sizeof( CoordEntry ) ) );
                co->next = NULL;
                co->x = x;
                co->y = y;
              } /* if */
          } /* else if */
        else if ( ! strcmp( cmd, "TXT" ) )
          {
            ReadCommand( infile, cmd, 512 );
            num = 0;
            while ( strcmp( cmd, "end" ) )
              {
                if ( ! strcmp( cmd, "str" ) )
                  {
                    ReadEnclString( infile, name, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dpt" ) )
                  {
                    ReadCoordinates( infile, &x, &y );
                  } /* else if */
                ReadCommand( infile, cmd, 512 );
              } /* while */
          } /* else if */
        else if ( ! strcmp( cmd, "FML" ) )
          {
            ReadNewline( infile );
            ungetc( '\n', infile );
          } /* else if */
        else if ( ! strcmp( cmd, "haf" ) )
          {
            ReadNewline( infile );
          } /* else if */
        else if ( ! strcmp( cmd, "set" ) )
          {
            ReadNewline( infile );
          } /* else if */
        else if ( ! strcmp( cmd, "pap" ) )
          {
            ReadNewline( infile );
          } /* else if */
        else if ( ! strcmp( cmd, "com" ) )
          {
            ReadEnclString( infile, name, 4096 );
          } /* else if */
        else if ( ! strcmp( cmd, "wsz" ) )
          {
            ReadNewline( infile );
          } /* else if */
        else if ( ! strcmp( cmd, "end" ) )
          {
          } /* else if */
        else
          {
            fprintf( stderr,
            "ERROR: CM.read_PEP(): unknown PEP command found: '%s'.\n", cmd );
          } /* else */
        ReadNewline( infile );
        ReadCommand( infile, cmd, 512 );
      } /* while */

    fclose( infile );

    Free( Namen );

    return( net );

  } /* read_PEP() */


/*****************************************************************************/
/****************************** write_PEP ***********************************/
/*****************************************************************************/
/*                                                  13.12.1993 Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void write_PEP( LMPNet net, char * PEPfilename )

#else

void write_PEP( net, PEPfilename )
  /* Gibt die mit net bezeichnete Matrix als Inzidenzmatrix nach 'file' aus  */

  LMPNet net;
  char   * PEPfilename;

#endif

  {

    /* NOCH NICHT IMPLEMENTIERT !!! */

  } /* write_PEP() */


/*****************************************************************************/
/***************************** read_new_PEP **********************************/
/*****************************************************************************/
/* Neu eingefuegt (neues Format)       19.12.1994 - 09.01.1995 Stefan Roemer */
/*                                     Bloecke      07.03.1997 Stefan Roemer */
/* Neue Einleseroutine				   April 1997 Stefan Schwoon */
/*****************************************************************************/

/*
#define NEW_READFUNC
*/
#ifdef NEW_READFUNC
char		buffer[10];
bool		autonumbering;
int		placecount, transcount;

LMPNet		net;
Place		*PlArray;
Transition	*TrArray, *PhTrArray;

Cardinal32	AnzPlNamen, MaxPlNamen,
		AnzTrNamen, MaxTrNamen,
		AnzPhTrNamen, MaxPhTrNamen;

char		*name, *meaning, *block, *guard, *name2,
		*cmarking, *reference, *corrtran, *blk;
int		entry_place, exit_place, invisible, ident,
		imarks, cmarks, weight;
t_coords	*xy, *mrc, *mc;

int		def_mrx = 10,
		def_mry = 10;

int rp_insert_place();
int rp_insert_trans();
int rp_insert_tparc();
int rp_insert_ptarc();

t_dest place_dest[] =
	{ { '\'',&name },	/* Bezeichner		*/
	  { '"', &name },	/*     "		*/
	  { '@', &xy },		/* Position		*/
	  { '0', &ident },	/* numerischer Name	*/
	  { 'M', &imarks },	/* Init-Marking (num)	*/
	  { 'm', &cmarks },	/* Current-Markung	*/
	  { 'e', &entry_place },/* Entry-Stelle		*/
	  { 'x', &exit_place },	/* Exit-Stelle		*/
	  { 'Z', &guard },	/* Typ			*/
	  { 'z', &name2 },	/* Init-Marking		*/
	  { 'y', &cmarking },	/* Current-Marking	*/
	  { 'a', &mrc },	/* rel. Pos. Bedeutung  */
	  { 'A', &mc },		/* abs. Pos. Bedeutung  */
	  { 'b', &meaning },	/* Bedeutung		*/
	  { 'R', &reference },	/* Referenz		*/
	  {  0 ,  0 } };
  
t_dest trans_dest[] =
	{ { '\'',&name },	/* Bezeichner		*/
	  { '"', &name },	/*     "		*/
	  { '@', &xy },		/* Position		*/
	  { '0', &ident },	/* numerischer Name	*/
	  { 'b', &meaning },	/* Bedeutung		*/
	  { 'a', &mrc },	/* rel. Pos. Bedeutung  */
	  { 'i', &name2 },	/* Aktionsterme		*/
	  { 'g', &guard },	/* Value Term		*/
	  { 'R', &reference },	/* Referenz		*/
	  {  0 ,  0 } };

t_dest arc_dest[] =
	{ { '@', &xy },		/* Quelle/Ziel		*/
	  { 'w', &weight },	/* Gewicht		*/
	  { 'Q', &mc },		/* abs. Pos. Beschr.	*/
	  { 'p', &guard },	/* Beschriftung		*/
	  { 'v', &invisible },	/* Sichtbarkeit		*/
	  {  0 ,  0 } };

t_blockdest netdest[] =
	{ { "PL", rp_insert_place, NULL, place_dest },
	  { "TR", rp_insert_trans, NULL, trans_dest },
	  { "TP", rp_insert_tparc, NULL, arc_dest },
	  { "PT", rp_insert_ptarc, NULL, arc_dest },
	  { NULL, NULL, NULL, NULL } };

#endif /* NEW_READFUNC */

#if defined(__STDC__) || defined(__cplusplus)

LMPNet read_new_PEP( char * PEPfilename )

#else

LMPNet read_new_PEP( PEPfilename )

  char * PEPfilename;

#endif

  {
#ifdef NEW_READFUNC
	int count;

	PlArray = (Place*) Malloc((count = MaxPlNamen = MaxTrNamen
			= MaxPhTrNamen = NAMES_START) * sizeof(Place));
	TrArray = (Transition*) Malloc(count * sizeof(Transition));
	AnzPlNamen = AnzTrNamen = AnzPhTrNamen = 0;
	while (--count)
	{
		PlArray[count] = (Place) NULL;
		TrArray[count] = (Transition) NULL;
	} /* while */

	placecount = 0;
	transcount = 0;
	autonumbering = True;

	net = empty_net();
	net->NumComponents = 1;

	read_PEP_net(PEPfilename,netdest);
	strncpy(CM_nettype,filetype,30);

	free(PlArray);
	free(TrArray);

	return net;

}

static char nullstring = '\0';

int rp_insert_place()
{
	static Place pl_last = NULL;

	placecount++;
	if (ident && ident != placecount) autonumbering = False;
	if (!ident && autonumbering) ident = placecount;
	if (!ident) NNError("missing place identifier");

        if (ident > AnzPlNamen)
		AnzPlNamen = ident;
        else if (PlArray[ident])
		Error((errbuf,"place identifier %d muliply used",ident));

        while (AnzPlNamen >= MaxPlNamen)
	{
		int count;
		PlArray = (Place*) Realloc((char*) PlArray,
                            (MaxPlNamen += NAMES_OFFSET) * sizeof(Place));
		for (count = MaxPlNamen - NAMES_OFFSET; count < MaxPlNamen;)
			PlArray[count++] = (Place) NULL;
	} /* if */

	if (!name)
	{
		sprintf(buffer, "PL%d", ident);
		name = Strdup(buffer);
	} /* if */

	PlArray[ident] = fast_insert_place(net, name,
			CM_use_imarks? imarks : cmarks, &pl_last);

        if (CM_storeMeaning && meaning)
		insert_place_meaning(meaning, pl_last);

        if (!strcmp(filetype, "PetriBox"))
	{
		if (CM_use_imarks)
		    pl_last->marks = entry_place? 1 : 
			(guard || name2 || cmarking)? 0 : imarks;

		pl_last->label = entry_place? ENTRY : exit_place? EXIT : EMPTY;
	}

	pl_last->component = 0;

	if (!mc)
	{
		mc = (t_coords*) Malloc(sizeof(t_coords));
		mc->x = (xy? xy->x : 0) + (mrc? mrc->x : def_mrx);
		mc->y = (xy? xy->y : 0) + (mrc? mrc->y : def_mry);
	}

        if (CM_storeCoordinates || CM_storeReferences)
	{
		Coords co;

		if (!reference) reference = Strdup(&nullstring);
		if (!meaning) meaning = Strdup(&nullstring);
		if (!guard) guard = Strdup(&nullstring);
		if (!name2) name2 = Strdup(&nullstring);

		pl_last->inscript = (char*)
			(co = (Coords) Malloc(sizeof(CoordEntry)));
		co->x = xy->x;
		co->y = xy->y;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = mc->x;
		co->y = mc->y;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = (int) meaning;
		co->y = (int) reference;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = (int) guard;
		co->y = (int) name2;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = (int) cmarks;
		co->next = NULL;
	} /* if */

	free(mc);
	if (xy) free(xy);
	if (mrc) free(mrc);
	if (cmarking) free(cmarking);
	return 0;

}

int rp_insert_trans()
{
	static Transition tr_last = NULL;

	if (!transcount++) autonumbering = True;
	if (ident && ident != transcount) autonumbering = False;
	if (!ident && autonumbering) ident = transcount;
	if (!ident) NNError("missing transition identifier");

        if (ident > AnzTrNamen)
		AnzTrNamen = ident;
        else if (TrArray[ident])
		Error((errbuf,"transition identifier %d muliply used",ident));

	while (AnzTrNamen >= MaxTrNamen)
	{
		int count;
		TrArray = (Transition*) Realloc((char*) TrArray,
			(MaxTrNamen += NAMES_OFFSET) * sizeof(Transition));
		for (count = MaxTrNamen - NAMES_OFFSET; count < MaxTrNamen;)
			TrArray[count++] = (Transition) NULL;

	} /* if */
	if (!name)
	{
		sprintf(buffer, "TR%d", ident);
		name = Strdup(buffer);
	} /* if */
        TrArray[ident] = fast_insert_transition(net, name, &tr_last);

	if (CM_storeMeaning && meaning)
		insert_trans_meaning(meaning, tr_last);
        tr_last->label = NULL;

	if (CM_storeCoordinates || CM_storeReferences)
	{
		Coords co;

		if (!reference) reference = Strdup(&nullstring);
		if (!meaning) meaning = Strdup(&nullstring);
		if (!guard) guard = Strdup(&nullstring);
		if (!name2) name2 = Strdup(&nullstring);

		tr_last->inscript = (char *)
			(co = (Coords) Malloc(sizeof(CoordEntry)));
		co->x = xy->x;
		co->y = xy->y;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = xy->x + (mrc? mrc->x : def_mrx);
		co->y = xy->y + (mrc? mrc->y : def_mry);
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = (int) name2;
		co->y = (int) meaning;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = (int) guard;
		co->y = (int) reference;
		co->next = NULL;
	} /* if */

	free(xy);
	if (mrc) free(mrc);
	return 0;

}

int rp_insert_tparc()
{
	static TSEdge ts_last = NULL;
	Place pl_last;
	Transition tr_last;

        if (!xy->x|| (xy->x > AnzTrNamen) || !TrArray[xy->x])
		NNError("arc: incorrect transition identifier");
        if (!xy->y || (xy->y > AnzPlNamen) || !PlArray[xy->y])
		NNError("arc: incorrect place identifier");

	if (!weight) weight = 1;
        tr_last = TrArray[xy->x];
        pl_last = PlArray[xy->y];
	fast_insert_TP_con(net, pl_last->name, tr_last->name, weight,
					&pl_last, &tr_last, &ts_last);

	if (CM_storeCoordinates)
	{
		Coords co;

		if (!guard) guard = Strdup(&nullstring);

		ts_last->weight = (Cardinal32)
			(co = (Coords) Malloc(sizeof(CoordEntry)));
		co->x = mc? mc->x : 0;
		co->y = mc? mc->y : 0;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = (int) guard;
		co->y = (int) invisible;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = (int) weight;
		co->next = NULL;
	} /* if */

	free(xy);
	if (mc) free(mc);
	return 0;

}

int rp_insert_ptarc()
{
	static STEdge st_last = NULL;
	Place pl_last;
	Transition tr_last;

        if (!xy->x || (xy->x > AnzPlNamen) || !PlArray[xy->x])
		NNError("arc: incorrect place identifier");
        if (!xy->y|| (xy->y > AnzTrNamen) || !TrArray[xy->y])
		NNError("arc: incorrect transition identifier");

	if (!weight) weight = 1;
	pl_last = PlArray[xy->x];
	tr_last = TrArray[xy->y];
	fast_insert_PT_con(net, pl_last->name, tr_last->name, weight,
					&pl_last, &tr_last, &st_last);

	if (CM_storeCoordinates)
	{
		Coords co;

		if (!guard) guard = Strdup(&nullstring);

		st_last->weight = (Cardinal32)
			(co = (Coords) Malloc(sizeof(CoordEntry)));
		co->x = mc? mc->x : 0;
		co->y = mc? mc->y : 0;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = (int) guard;
		co->y = (int) invisible;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = (int) weight;
		co->next = NULL;
	} /* if */

	free(xy);
	if (mc) free(mc);
	return 0;

#else /* not NEW_READFUNC */

    int                 cx, cy,
                        sx, sy,
                        px, py,
                        prx, pry,
                        gx, gy,
                        grx, gry,
                        wrx, wry,
                        arx, ary,
                        mx, my,
                        mrx, mry,
                        color, thickness,
                        cmarks, imarks, capacity,
                        num, num2, weight,
                        def_pl_radius = 10,
                        def_pl_prx = 10,
                        def_pl_pry = 0,
                        def_pl_mrx = 10,
                        def_pl_mry = 10,
                        def_pl_wrx = 10,
                        def_pl_wry = 10,
                        def_pl_imarks = 0,
                        def_pl_cmarks = 0,
                        def_pl_capacity = OMEGA,
                        def_pl_thickness = 2,
                        def_pl_color = 1,
                        def_tr_size_x = 10,
                        def_tr_size_y = 10,
                        def_tr_prx = 10,
                        def_tr_pry = 0,
                        def_tr_mrx = 10,
                        def_tr_mry = 10,
                        def_tr_thickness = 2,
                        def_tr_color = 1;
    register int        ch;
    Place               * PlArray;
    Transition          * TrArray,
                        * PhTrArray;
    char                *cmd,
			name[NAMESIZE+1],
                        meaning[MEANINGSIZE+1],
                        marking[MEANINGSIZE+1],
                        guard[GUARDSIZE+1],
                        reference[REFSIZE+1],
                        treference[REFSIZE+1],
                        blocklist[REFSIZE+1],
                        synclist[REFSIZE+1],
                        name2[MEANINGSIZE+1];
    register FILE       * infile;
    register LMPNet     net;
    register Cardinal32 AnzPlNamen,
                        MaxPlNamen,
                        AnzTrNamen,
                        MaxTrNamen,
                        AnzPhTrNamen,
                        MaxPhTrNamen,
                        autocounter = 0,
                        ident,
                        count;
    Cardinal32          max_ident = 0,
                        invisible;
    Place               pl_last = NULL;
    Transition          tr_last = NULL;
    STEdge              st_last = NULL;
    TSEdge              ts_last = NULL;
    bool                entry_place,
                        exit_place,
                        refined,
                        is_sync_trans,
                        autonumbering,
                        abs_meaning,
                        abs_name,
                        num_marking,
                        specified;
    Time_stamp          * time_label;

    NNinput_file = PEPfilename;
    NNinput_line = 1;
    PlArray = (Place *) Malloc( ( count = MaxPlNamen = MaxTrNamen
                            = MaxPhTrNamen = NAMES_START ) * sizeof( Place ) );
    TrArray = (Transition *) Malloc( count * sizeof( Transition ) );
    PhTrArray = (Transition *) Malloc( count * sizeof( Transition ) );
    AnzPlNamen = AnzTrNamen = AnzPhTrNamen = 0;
    while ( --count )
      {
        PlArray[ count ] = (Place) NULL;
        TrArray[ count ] = (Transition) NULL;
        PhTrArray[ count ] = (Transition) NULL;
      } /* while */

    if ( ! ( infile = fopen( PEPfilename, "r" ) ) )
      {
        fprintf( stderr,
               "ERROR: CM.read_PEP(): could not open file '%s' for reading.\n",
                 PEPfilename );
        exit ( 4 );
      } /* if */

    cmd=ReadCmdToken( infile );
    if ( strcmp( cmd, "PEP" ) )
      {
        NNError( "keyword 'PEP' expected" );
        return ( NULL );
      } /* if */
    ReadNewline( infile );
    cmd=ReadCmdToken( infile );
    if (    strcmp( cmd, "PetriBox" ) && strcmp( cmd, "HLNet" )
         && strcmp( cmd, "PTNet" ) )
      {
      NNError( "format identifier ( 'PTNet', 'PetriBox', 'HLNet' ) expected" );
      } /* if */
    strncpy( CM_nettype, cmd, 30 );
    ReadNewline( infile );
    cmd=ReadCmdToken( infile );
    if ( strncmp( cmd, "FORMAT_N", 8 ) )
      {
        NNError( "keyword 'FORMAT_N' or 'FORMAT_N2' expected" );
        return ( NULL );
      } /* if */

    net = empty_net();
    net->NumComponents = 1;

    ReadNewline( infile );
    cmd=ReadCmdToken( infile );
    while ( *cmd== 'D' )
      {
        if ( ! strcmp( cmd, "DPL" ) )
          {
            /* To be done */
            ReadNewline( infile );
          } /* if */
        else if ( ! strcmp( cmd, "DTR" ) )
          {
            /* To be done */
            ReadNewline( infile );
          } /* else if */
        else if ( ! strcmp( cmd, "DPT" ) )
          {
            /* To be done */
            ReadNewline( infile );
          } /* else if */
        else
          {
            sprintf( name, "read_new_PEP(): unknown keyword '%s'", cmd);
            NNError( name );
            return ( NULL );
          } /* else */
        cmd=ReadCmdToken( infile );
      } /* while */

    /* Read in block information *********************************************/
    if ( ! strcmp( cmd, "BL" ) )
      {
        ReadNewline( infile );
 
        while ( ( ch = ReadCharComment( infile ) ) && ( ch != 'P' ) && (ch!='L') )
          {
            while ( ch != '\n' )
              {
                if ( isdigit( ch ) || ( ch == '-' ) )
                  {
                    ungetc( ch, infile );
                    num = ReadNumber( infile );
                    if ( TestSymbol( infile, '@' ) )
                      {
                        cx = num;
                        cy = ReadNumber( infile );
                      } /* if */
                    else
                      {
                        if ( ( ident = num ) != autocounter )
                          {
                            autonumbering = False;
                          } /* if */
                        specified = True;
                      } /* else */
                  } /* if */
                else
                  {
                    switch ( ch )
                      {
                        case '\'':
                        case '"' : ungetc( ch, infile );
                                   ReadEnclString( infile, name, NAMESIZE );
                                   break;
                        case 'n' : ReadCoordinates( infile, &prx, &pry );
                                   break;
                        case 'N' : ReadCoordinates( infile, &px, &py );
                                   abs_name = True;
                                   break;
                        case 'b' : ReadEnclString( infile, meaning,
                                                                 MEANINGSIZE );
                                   break;
                        case 'a' : ReadCoordinates( infile, &mrx, &mry );
                                   break;
                        case 'A' : ReadCoordinates( infile, &mx, &my );
                                   abs_meaning = True;
                                   break;
                        case 'R' : ReadEnclString( infile, reference, REFSIZE );
                                   break;
                        case 'T' : ReadEnclString( infile, treference,REFSIZE );
                                   break;
                        case 'u' : ReadEnclString( infile, blocklist, REFSIZE );
                                   break;
                        default  : sprintf( name, "unknown token '%c'", ch );
                                   NNError( name );
                      } /* switch */
                  } /* else */
                ch = ReadCharComment( infile );
              } /* while */
 
            NNinput_line++;
          } /* while */

        ungetc( ch, infile );
        cmd=ReadCmdToken( infile );
      } /* if */
    /* Read Link symbols : we just ignore them **************************/
    if ( !strcmp( cmd, "LS" ) )
      {
        while (!feof(infile) && strcmp(cmd=ReadCmdToken(infile), "PL"))
          ReadNewline(infile);
      } /* if ( !strcmp( cmd, "LS" ) ) */

    /* Read in place nodes ***************************************************/
    if ( strcmp( cmd, "PL" ) )
      {
        NNError( "keyword 'PL' expected" );
        return ( NULL );
      } /* if */
    ReadNewline( infile );

    autocounter = 0;
    autonumbering = True;
    while ( ( ch = ReadCharComment( infile ) ) && ( ch != 'T' ) )
      {
        autocounter++; 
        ident = 0;
        cx = cy =  mx = my = px = py = 0;
        prx = def_pl_prx;
        pry = def_pl_pry;
        wrx = def_pl_wrx;
        wry = def_pl_wry;
        mrx = def_pl_mrx;
        mry = def_pl_mry;
        cmarks = def_pl_cmarks;
        imarks = def_pl_imarks;
        num_marking = True;
        invisible = 0;
        abs_name = abs_meaning = specified = False;
        *marking = *guard = *name = *name2 = *meaning = *reference
                                             = *treference = *blocklist = '\0';
        capacity = def_pl_capacity;
        color = def_pl_color;
        sx = def_pl_radius;
        thickness = def_pl_thickness;
        entry_place = exit_place = False;
        while ( ch != '\n' )
          {
            if ( isdigit( ch ) || ( ch == '-' ) )
              {
                ungetc( ch, infile );
                num = ReadNumber( infile );
                if ( TestSymbol( infile, '@' ) )
                  {
                    cx = num;
                    cy = ReadNumber( infile );
                  } /* if */
                else
                  {
                    if ( ( ident = num ) != autocounter )
                      {
                        autonumbering = False;
                      } /* if */
                    if ( ident > max_ident )
                      {
                        max_ident = ident;
                      } /* if */
                    specified = True;
                  } /* else */
              } /* if */ 
            else
              {
                switch ( ch )
                  {
                    case '\'':
                    case '"' : ungetc( ch, infile );
                               ReadEnclString( infile, name, NAMESIZE );
                               break;
                    case 'n' : ReadCoordinates( infile, &prx, &pry );
                               break;
                    case 'N' : ReadCoordinates( infile, &px, &py );
                               abs_name = True;
                               break;
                    case 'b' : ReadEnclString( infile, meaning, MEANINGSIZE );
                               break;
                    case 'a' : ReadCoordinates( infile, &mrx, &mry );
                               break;
                    case 'A' : ReadCoordinates( infile, &mx, &my );
                               abs_meaning = True;
                               break;
                    case 'M' : imarks = ReadNumber( infile );
                               num_marking = True;
                               break;
                    case 'm' : cmarks = ReadNumber( infile );
                               num_marking = True;
                               break;
                    case 'Z' : ReadEnclString( infile, guard, GUARDSIZE );
                               num_marking = False;
                               break;
                    case 'B' : ReadCoordinates( infile, &wrx, &wry );
                               break;
                    case 'z' : ReadEnclString( infile, name2, MEANINGSIZE );
                               num_marking = False;
                               break;
                    case 'y' : ReadEnclString( infile, marking, MEANINGSIZE );
                               num_marking = False;
                               break;
                    case 'R' : ReadEnclString( infile, reference, REFSIZE );
                               break;
                    case 'T' : ReadEnclString( infile, treference, REFSIZE );
                               break;
                    case 'u' : ReadEnclString( infile, blocklist, REFSIZE );
                               break;
                    case 'k' : capacity = ReadNumber( infile );
/* POSITION KAPAZITAET */
                               break;
                    case 'e' : if ( exit_place )
                                 {
                                   NNError( "both exit and entry flag found" );
                                 } /* else */
                               else
                                 {
                                   entry_place = True;
                                 } /* else */
                               break;
                    case 'x' : if ( entry_place )
                                 {
                                   NNError( "both entry and exit flag found" );
                                 } /* else */
                               else
                                 {
                                   exit_place = True;
                                 } /* else */
                               break;
                    case 'v' : invisible = ReadNumber( infile );
                               break;
                    case 's' : sx = ReadNumber( infile );
                               break;
                    case 't' : thickness = ReadNumber( infile );
                               break;
                    case 'c' : color = ReadNumber( infile );
                               break;
                    default  : sprintf( name, "unknown token '%c'", ch );
                               NNError( name );
                  } /* switch */
              } /* else */
            ch = ReadCharComment( infile );
          } /* while */

        /* insert place node in the net */
        if ( autonumbering )
          {
            ident = autocounter;
          } /* if */
        else if ( ! specified )
          {
            NNError( "missing place identifier" );
          } /* if */
        if ( ident > AnzPlNamen )
          {
            AnzPlNamen = ident;
          } /* if */
        else if ( PlArray[ ident ] )
          {
            sprintf( guard, "place identifier %d multiply used", ident );
            NNError( guard );
          } /* else if */
        while ( AnzPlNamen >= MaxPlNamen )
          {
            PlArray = (Place *) Realloc( (char *) PlArray,
                            ( MaxPlNamen += NAMES_OFFSET ) * sizeof( Place ) );
            for ( count = MaxPlNamen - NAMES_OFFSET ;
                                                 count < MaxPlNamen ; count++ )
              {
                PlArray[ count ] = (Place) NULL;
              } /* for */
          } /* if */
        if ( ! *name )
          {
            sprintf( name, "PL%d", ident );
          } /* if */
        PlArray[ ident ] = fast_insert_place( net, name,
                                   CM_use_imarks ? imarks : cmarks, &pl_last );
        if ( CM_storeMeaning && *meaning )
          {
            insert_place_meaning( meaning, pl_last );
          } /* if */
        if ( ! strcmp( CM_nettype, "PetriBox" ) )
          {
            if ( CM_use_imarks )
              {
                pl_last->marks = entry_place ? 1 : num_marking ? imarks : 0;
              } /* if */
            pl_last->label = entry_place ? ENTRY : exit_place ? EXIT : EMPTY;
          }
        pl_last->component = 0;
        if ( ! abs_name )
          {
            px = cx + prx;
            py = cy + pry;
          } /* if */
        if ( ! abs_meaning )
          {
            mx = cx + mrx;
            my = cy + mry;
          } /* if */
        if ( CM_storeCoordinates || CM_storeReferences )
          {
            Coords co;
            pl_last->inscript = (char *)
                              ( co = (Coords) Malloc( sizeof( CoordEntry ) ) );
            co->x = cx;
            co->y = cy;
            co = co->next = (Coords) Malloc( sizeof( CoordEntry ) );
            co->x = mx;
            co->y = my;
            co = co->next = (Coords) Malloc( sizeof( CoordEntry ) );
            co->x = (int) Strdup( meaning );
            co->y = (int) Strdup( reference );
            co = co->next = (Coords) Malloc( sizeof( CoordEntry ) );
            co->x = (int) Strdup( guard );
            co->y = (int) Strdup( name2 );
            co = co->next = (Coords) Malloc( sizeof( CoordEntry ) );
            co->x = (int) cmarks;
            co->next = NULL;
          } /* if */

        NNinput_line++;
      } /* while */

    /* Read in transition nodes **********************************************/
    ungetc( ch, infile );
    cmd=ReadCmdToken( infile );
    if ( strcmp( cmd, "TR" ) )
      {
        NNError( "keyword 'TR' expected" );
        return ( NULL );
      } /* if */
    ReadNewline( infile );

    autocounter = 0;
    autonumbering = True;
    while ( ( ch = ReadCharComment( infile ) ) && ( ch != 'T' )
                                               && ( ch != 'P' ) )
      {
        autocounter++; 
        ident = 0;
        cx = cy =  mx = my = px = py = 0;
        arx = ary = 0;
        prx = def_tr_prx;
        pry = def_tr_pry;
        mrx = def_tr_mrx;
        mry = def_tr_mry;
        num_marking = True;
        invisible = 0;
        abs_name = abs_meaning = specified = is_sync_trans = False;
        *guard = *name = *name2 = *meaning = *reference = *treference
                                                           = *blocklist = '\0';
        color = def_tr_color;
        sx = def_tr_size_x;
        sy = def_tr_size_y;
        thickness = def_tr_thickness;
        time_label=NULL;
        while ( ch != '\n' )
          {
            refined = False;
            if ( isdigit( ch ) || ( ch == '-' ) )
              {
                ungetc( ch, infile );
                num = ReadNumber( infile );
                if ( TestSymbol( infile, '@' ) )
                  {
                    cx = num;
                    cy = ReadNumber( infile );
                  } /* if */
                else
                  {
                    if ( ( ident = num ) != autocounter )
                      {
                        autonumbering = False;
                      } /* if */
                    if ( ident > max_ident )
                      {
                        max_ident = ident;
                      } /* if */
                    specified = True;
                  } /* else */
              } /* if */ 
            else
              {
                switch ( ch )
                  {
                    case '\'':
                    case '"' : ungetc( ch, infile );
                               ReadEnclString( infile, name, NAMESIZE );
                               break;
                    case 'n' : ReadCoordinates( infile, &prx, &pry );
                               break;
                    case 'N' : ReadCoordinates( infile, &px, &py );
                               break;
                    case 'b' : ReadEnclString( infile, meaning, MEANINGSIZE );
                               break;
                    case 'a' : ReadCoordinates( infile, &mrx, &mry );
                               break;
                    case 'A' : ReadCoordinates( infile, &mx, &my );
                               break;
                    case 'v' : invisible = ReadNumber( infile );
                               break;
                    case 'g' : ReadEnclString( infile, guard, GUARDSIZE );
                               break;
                    case 'h' : ReadCoordinates( infile, &grx, &gry );
                               break;
                    case 'i' : ReadEnclString( infile, name2, MEANINGSIZE );
                               break;
                    case 'j' : ReadCoordinates( infile, &arx, &ary );
                               break;
                    case 'H' : ReadCoordinates( infile, &gx, &gy );
                               break;
                    case 'R' : ReadEnclString( infile, reference, REFSIZE );
                               break;
                    case 'T' : ReadEnclString( infile, treference, REFSIZE );
                               break;
                    case 'u' : ReadEnclString( infile, blocklist, REFSIZE );
                               break;
                    case 'P' : ReadEnclString( infile, synclist, REFSIZE );
                               break;
                    case 's' : sy = sx = ReadNumber( infile );
                               break;
#undef _OLD_VERSION_
#ifdef _OLD_VERSION_
                    case 'S' : sx = ReadNumber( infile );
                               if ( TestSymbol( infile, '@' ) )
                                 {
                                   sy = ReadNumber( infile );
                                 } /* if */
                               else
                                 {
                                   NNError( "missing '@'" );
                                 } /* else */
                               break;
#else  /* _OLD_VERSION_ */
                    case 'S' : is_sync_trans = True;
                               break;
#endif /* _OLD_VERSION_ */
                    case 't' : thickness = ReadNumber( infile );
                               break;
                    case 'r' : if ( refined )
                                 {
                                   NNError( "refined flag doubly defined" );
                                 } /* else */
                               else
                                 {
                                   refined = True;
                                 } /* else */
                               break;
                    case 'c' : color = ReadNumber( infile );
                               break;
                    case 'E' : if (!time_label)
                                 {
                                   time_label = (Time_stamp *) Malloc (sizeof(Time_stamp));
                                   time_label->high = 0;
                                 }
                               time_label->low = ReadDouble( infile );
                               break;
                    case 'L' : if (!time_label)
                                 {
                                   time_label = (Time_stamp *) Malloc (sizeof(Time_stamp));
                                   time_label->low = 0;
                                 }
                               time_label->high = ReadDouble (infile);
                               break;
                    default  : sprintf( name, "unknown token '%c'", ch );
                               NNError( name );
                  } /* switch */
              } /* else */
            ch = ReadCharComment( infile );
          } /* while */

        /* insert transition node in the net */
        if ( autonumbering )
          {
            ident = autocounter;
          } /* if */
        else if ( ! specified )
          {
            NNError( "missing transition identifier" );
          } /* if */
        if ( ident > AnzTrNamen )
          {
            AnzTrNamen = ident;
          } /* if */
        else if ( TrArray[ ident ] )
          {
            sprintf( guard, "transition identifier %d multiply used", ident );
            NNError( guard );
          } /* else if */
        while ( AnzTrNamen >= MaxTrNamen )
          {
            TrArray = (Transition *) Realloc( (char *) TrArray,
                       ( MaxTrNamen += NAMES_OFFSET ) * sizeof( Transition ) );
            for ( count = MaxTrNamen - NAMES_OFFSET ;
                                                 count < MaxTrNamen ; count++ )
              {
                TrArray[ count ] = (Transition) NULL;
              } /* for */
          } /* if */
        if ( ! *name )
          {
            sprintf( name, "TR%d", ident );
          } /* if */
        TrArray[ ident ] = fast_insert_transition( net, name, &tr_last );
        if ( CM_storeMeaning && *meaning )
          {
            insert_trans_meaning( meaning, tr_last );
          } /* if */
        tr_last->label = NULL;

        if ( ! abs_name )
          {
            px = cx + prx;
            py = cy + pry;
          } /* if */
        if ( ! abs_meaning )
          {
            mx = cx + mrx;
            my = cy + mry;
          } /* if */
        if ( CM_storeCoordinates || CM_storeReferences )
          {
            Coords co;
            tr_last->inscript = (char *)
                              ( co = (Coords) Malloc( sizeof( CoordEntry ) ) );
            co->x = cx;
            co->y = cy;
            co = co->next = (Coords) Malloc( sizeof( CoordEntry ) );
            co->x = mx;
            co->y = my;
            co = co->next = (Coords) Malloc( sizeof( CoordEntry ) );
            co->x = (int) Strdup( name2 );
            co->y = (int) Strdup( meaning );
            co = co->next = (Coords) Malloc( sizeof( CoordEntry ) );
            co->x = (int) Strdup( guard );
            co->y = (int) Strdup( reference );
            co->next = NULL;
          } /* if */
        tr_last->time_label=time_label;
        time_label=NULL;

        NNinput_line++;
      } /* while */

    /* Read in phantom transitions *******************************************/
    if ( ch == 'P' )
      {
        ungetc( ch, infile );
        cmd=ReadCmdToken( infile );
        if ( strcmp( cmd, "PTR" ) )
          {
            NNError( "keyword 'PTR' expected" );
            return ( NULL );
          } /* if */
        ReadNewline( infile );

        autocounter = 0;
        autonumbering = True;
        while ( ( ch = ReadCharComment( infile ) ) && ( ch != 'T' ) )
          {
            autocounter++; 
            ident = 0;
            cx = cy =  mx = my = px = py = 0;
            arx = ary = 0;
            prx = def_tr_prx;
            pry = def_tr_pry;
            mrx = def_tr_mrx;
            mry = def_tr_mry;
            num_marking = True;
            invisible = 0;
            abs_name = abs_meaning = specified = False;
            *guard = *name = *name2 = *meaning = *reference
                                            = *treference = *blocklist = '\0';
            color = def_tr_color;
            sx = def_tr_size_x;
            sy = def_tr_size_y;
            thickness = def_tr_thickness;
            while ( ch != '\n' )
              {
                refined = False;
                if ( isdigit( ch ) || ( ch == '-' ) )
                  {
                    ungetc( ch, infile );
                    num = ReadNumber( infile );
                    if ( TestSymbol( infile, '@' ) )
                      {
                        cx = num;
                        cy = ReadNumber( infile );
                      } /* if */
                    else
                      {
                        if ( ( ident = num ) != autocounter )
                          {
                            autonumbering = False;
                          } /* if */
                        if ( ident > max_ident )
                          {
                            max_ident = ident;
                          } /* if */
                        specified = True;
                      } /* else */
                  } /* if */ 
                else
                  {
                    switch ( ch )
                      {
                        case '\'':
                        case '"' : ungetc( ch, infile );
                                   ReadEnclString( infile, name, NAMESIZE );
                                   break;
                        case 'n' : ReadCoordinates( infile, &prx, &pry );
                                   break;
                        case 'N' : ReadCoordinates( infile, &px, &py );
                                   break;
                        case 'b' : ReadEnclString( infile, meaning,
                                                                  MEANINGSIZE );
                                   break;
                        case 'a' : ReadCoordinates( infile, &mrx, &mry );
                                   break;
                        case 'A' : ReadCoordinates( infile, &mx, &my );
                                   break;
                        case 'v' : invisible = ReadNumber( infile );
                                   break;
                        case 'g' : ReadEnclString( infile, guard, GUARDSIZE );
                                   break;
                        case 'h' : ReadCoordinates( infile, &grx, &gry );
                                   break;
                        case 'i' : ReadEnclString( infile, name2, MEANINGSIZE );
                                   break;
                        case 'j' : ReadCoordinates( infile, &arx, &ary );
                                   break;
                        case 'H' : ReadCoordinates( infile, &gx, &gy );
                                   break;
                        case 'R' : ReadEnclString( infile, reference, REFSIZE );
                                   break;
                        case 'T' : ReadEnclString( infile, treference,REFSIZE );
                                   break;
                        case 'u' : ReadEnclString( infile, blocklist, REFSIZE );
                                   break;
                        case 'P' : ReadEnclString( infile, synclist, REFSIZE );
                                   break;
                        case 's' : sy = sx = ReadNumber( infile );
                                   break;
                        case 'S' : sx = ReadNumber( infile );
                                   if ( TestSymbol( infile, '@' ) )
                                     {
                                       sy = ReadNumber( infile );
                                     } /* if */
                                   else
                                     {
                                       NNError( "missing '@'" );
                                     } /* else */
                                   break;
                        case 't' : thickness = ReadNumber( infile );
                                   break;
                        case 'r' : if ( refined )
                                     {
                                       NNError( "refined flag doubly defined" );
                                     } /* else */
                                   else
                                     {
                                       refined = True;
                                     } /* else */
                                   break;
                        case 'c' : color = ReadNumber( infile );
                                   break;
                        default  : sprintf( name, "unknown token '%c'", ch );
                                   NNError( name );
                      } /* switch */
                  } /* else */
                ch = ReadCharComment( infile );
              } /* while */

            /* insert phantom transition node in the net */
            if ( autonumbering )
              {
                ident = autocounter;
              } /* if */
            else if ( ! specified )
              {
                NNError( "missing phantom transition identifier" );
              } /* if */
            if ( ident > AnzPhTrNamen )
              {
                AnzPhTrNamen = ident;
              } /* if */
            else if ( PhTrArray[ ident ] )
              {
                sprintf( guard,
                     "phantom transition identifier %d multiply used", ident );
                NNError( guard );
              } /* else if */
            if ( AnzPhTrNamen >= MaxPhTrNamen )
              {
                PhTrArray = (Transition *) Realloc( (char *) PhTrArray,
                     ( MaxPhTrNamen += NAMES_OFFSET ) * sizeof( Transition ) );
                for ( count = MaxPhTrNamen - NAMES_OFFSET ;
                                               count < MaxPhTrNamen ; count++ )
                  {
                    PhTrArray[ count ] = (Transition) NULL;
                  } /* for */
              } /* if */
            if ( ! *name )
              {
                sprintf( name, "TR%d", ident );
              } /* if */
            PhTrArray[ ident ] = (Transition) 1;
            NNinput_line++;
          } /* while */
      } /* if */

    /* Read in T->P arc nodes ************************************************/
    ungetc( ch, infile );
    cmd=ReadCmdToken( infile );
    if ( strcmp( cmd, "TP" ) )
      {
        NNError( "keyword 'TP' expected" );
        return ( NULL );
      } /* if */
    ReadNewline( infile );

    while ( ( ch = ReadCharComment( infile ) ) && ( ch != 'P' ) )
      {
        weight = 1;
        gx = gy = mx = my = 0;
        invisible = 0;
        *guard = *name = *name2 = *meaning = '\0';
        num = num2 = 0;
        while ( ch != '\n' )
          {
            if ( isdigit( ch ) )
              {
                ungetc( ch, infile );
                num = ReadNumber( infile );
                if ( TestSymbol( infile, '<' ) )
                  {
                    num2 = ReadNumber( infile );
                  } /* if */
                else
                  {
                    NNError( "'<' expected" );
                  } /* else */
		if ( ! num || ( num > AnzTrNamen ) || ! TrArray[ num ] )
		  NNError( "arc: incorrect transition identifier" );
		if ( ! num2 || ( num2 > AnzPlNamen ) || ! PlArray[ num2 ]  )
		  NNError( "arc: incorrect place identifier" );
		tr_last = TrArray[ num ];
		pl_last = PlArray[ num2 ];
              } /* if */ 
            else
              {
		if (!num)
                  NNError( "Node number expected" );
                switch ( ch )
                  {
                    case 'J' :
                    case ',' : ReadCoordinates( infile, &mx, &my );
                               break;
                    case 'w' : weight = ReadNumber( infile );
                               break;
                    case 'n' : ReadCoordinates( infile, &prx, &pry );
                               break;
                    case 'N' : ReadCoordinates( infile, &px, &py );
                               break;
                    case 'p' : ReadEnclString( infile, guard, GUARDSIZE );
                               break;
                    case 'q' : ReadCoordinates( infile, &gx, &gy );
                               break;
                    case 'Q' : ReadCoordinates( infile, &gx, &gy );
			       gx -= ((Coords)tr_last->inscript)->x;
			       gy -= ((Coords)tr_last->inscript)->y;
                               break;
                    case 'v' : invisible = ReadNumber( infile );
                               break;
                    case 't' : thickness = ReadNumber( infile );
                               break;
                    case 'c' : color = ReadNumber( infile );
                               break;
                    default  : sprintf( name, "unknown token '%c'", ch );
                               NNError( name );
                  } /* switch */
              } /* else */
            ch = ReadCharComment( infile );
          } /* while */
        fast_insert_TP_con( net, pl_last->name, tr_last->name, weight,
                                                &pl_last, &tr_last, &ts_last );
        if ( CM_storeCoordinates )
          {
            Coords co;
            ts_last->weight = (Cardinal32)
                              ( co = (Coords) Malloc( sizeof( CoordEntry ) ) );
            co->x = gx;
            co->y = gy;
            co = co->next = (Coords) Malloc( sizeof( CoordEntry ) );
            co->x = (int) Strdup( guard );
            co->y = (int) invisible;
            co = co->next = (Coords) Malloc( sizeof( CoordEntry ) );
            co->x = (int) weight;
	    co->next = NULL;
	    if (mx||my)
	    {
              co = co->next = (Coords) Malloc( sizeof( CoordEntry ) );
              co->x = mx;
              co->y = my;
	      co->next = NULL;
	    }
          } /* if */

        NNinput_line++;
      } /* while */

    /* Read in P->T arc nodes ************************************************/
    ungetc( ch, infile );
    cmd=ReadCmdToken( infile );
    if ( strcmp( cmd, "PT" ) )
      {
        NNError( "keyword 'PT' expected" );
        return ( NULL );
      } /* if */
    ReadNewline( infile );

    while ( ( ch = ReadCharComment( infile ) ) && ( ch != 'T' )
            && ( ch != 'P' ) && ! feof( infile ) )
      {
        weight = 1;
        gx = gy = mx = my = 0;
        invisible = 0;
        *guard = *name = *name2 = *meaning = '\0';
        num = num2 = 0;
        while ( ch != '\n' )
          {
            if ( isdigit( ch ) )
              {
                ungetc( ch, infile );
                num = ReadNumber( infile );
                if ( TestSymbol( infile, '>' ) )
                  {
                    num2 = ReadNumber( infile );
                  } /* if */
                else
                  {
                    NNError( "'>' expected" );
                  } /* else */
		if ( ! num || ( num > AnzPlNamen ) || ! PlArray[ num ] )
		  NNError( "arc: incorrect place identifier" );
		if ( ! num2 || ( num2 > AnzTrNamen ) || ! TrArray[ num2 ]  )
		  NNError( "arc: incorrect transition identifier" );
		pl_last = PlArray[ num ];
		tr_last = TrArray[ num2 ];
              } /* if */ 
            else
              {
		if (!num)
                  NNError( "Node number expected" );
                switch ( ch )
                  {
                    case 'J' :
                    case ',' : ReadCoordinates( infile, &mx, &my );
                               break;
                    case 'w' : weight = ReadNumber( infile );
                               break;
                    case 'n' : ReadCoordinates( infile, &prx, &pry );
                               break;
                    case 'N' : ReadCoordinates( infile, &px, &py );
                               break;
                    case 'p' : ReadEnclString( infile, guard, GUARDSIZE );
                               break;
                    case 'q' : ReadCoordinates( infile, &gx, &gy );
                               break;
                    case 'Q' : ReadCoordinates( infile, &gx, &gy );
			       gx -= ((Coords)pl_last->inscript)->x;
			       gy -= ((Coords)pl_last->inscript)->y;
                               break;
                    case 'v' : invisible = ReadNumber( infile );
                               break;
                    case 't' : thickness = ReadNumber( infile );
                               break;
                    case 'c' : color = ReadNumber( infile );
                               break;
                    default  : sprintf( name, "unknown token '%c'", ch );
                               NNError( name );
                  } /* switch */
              } /* else */
            ch = ReadCharComment( infile );
          } /* while */
        fast_insert_PT_con( net, pl_last->name, tr_last->name, weight,
                                                &pl_last, &tr_last, &st_last );
        if ( CM_storeCoordinates )
          {
            Coords co;
            st_last->weight = (Cardinal32)
                              ( co = (Coords) Malloc( sizeof( CoordEntry ) ) );
            co->x = gx;
            co->y = gy;
            co = co->next = (Coords) Malloc( sizeof( CoordEntry ) );
            co->x = (int) Strdup( guard );
            co->y = (int) invisible;
            co = co->next = (Coords) Malloc( sizeof( CoordEntry ) );
            co->x = (int) weight;
            co->next = NULL;
	    if (mx||my)
	    {
              co = co->next = (Coords) Malloc( sizeof( CoordEntry ) );
              co->x = mx;
              co->y = my;
	      co->next = NULL;
	    }
          } /* if */

        NNinput_line++;
      } /* while */

    /* Read in phantom T->P arc nodes ****************************************/
    if ( ch == 'P' )
      {
        ungetc( ch, infile );
        cmd=ReadCmdToken( infile );
        if ( strcmp( cmd, "PTP" ) )
          {
            NNError( "keyword 'PTP' expected" );
            return ( NULL );
          } /* if */
        ReadNewline( infile );

        while ( ( ch = ReadCharComment( infile ) ) && ( ch != 'P' ) )
          {
            weight = 1;
            gx = gy = 0;
            invisible = 0;
            *guard = *name = *name2 = *meaning = '\0';
            num = num2 = 0;
            while ( ch != '\n' )
              {
                if ( isdigit( ch ) )
                  {
                    ungetc( ch, infile );
                    num = ReadNumber( infile );
                    if ( TestSymbol( infile, '<' ) )
                      {
                        num2 = ReadNumber( infile );
                      } /* if */
                    else
                      {
                        NNError( "'<' expected" );
                      } /* else */
                  } /* if */ 
                else
                  {
                    switch ( ch )
                      {
                        case 'J' :
                        case ',' : ReadCoordinates( infile, &mx, &my );
                                   break;
                        case 'w' : weight = ReadNumber( infile );
                                   break;
                        case 'n' : ReadCoordinates( infile, &prx, &pry );
                                   break;
                        case 'N' : ReadCoordinates( infile, &px, &py );
                                   break;
                        case 'p' : ReadEnclString( infile, guard, GUARDSIZE );
                                   break;
                        case 'q' : ReadCoordinates( infile, &grx, &gry );
                                   break;
                        case 'Q' : ReadCoordinates( infile, &gx, &gy );
                                   break;
                        case 'v' : invisible = ReadNumber( infile );
                                   break;
                        case 't' : thickness = ReadNumber( infile );
                                   break;
                        case 'c' : color = ReadNumber( infile );
                                   break;
                        default  : sprintf( name, "unknown token '%c'", ch );
                                   NNError( name );
                      } /* switch */
                  } /* else */
                ch = ReadCharComment( infile );
              } /* while */
            if ( ! num || ( num > AnzPhTrNamen ) || ! PhTrArray[ num ] )
              {
                NNError( "arc: incorrect phantom transition identifier" );
              } /* if */
            if ( ! num2 || ( num2 > AnzPlNamen ) || ! PlArray[ num2 ]  )
              {
                NNError( "arc: incorrect place identifier" );
              } /* if */

            NNinput_line++;
          } /* while */
      } /* if */

    /* Read in phantom P->T arc nodes ****************************************/
    if ( ch == 'P' )
      {
        ungetc( ch, infile );
        cmd=ReadCmdToken( infile );
        if ( strcmp( cmd, "PPT" ) )
          {
            NNError( "keyword 'PPT' expected" );
            return ( NULL );
          } /* if */
        ReadNewline( infile );

        while ( ( ch = ReadCharComment( infile ) ) && ( ch != 'T' )
                && ! feof( infile ) )
          {
            weight = 1;
            gx = gy = 0;
            invisible = 0;
            *guard = *name = *name2 = *meaning = '\0';
            num = num2 = 0;
            while ( ch != '\n' )
              {
                if ( isdigit( ch ) )
                  {
                    ungetc( ch, infile );
                    num = ReadNumber( infile );
                    if ( TestSymbol( infile, '>' ) )
                      {
                        num2 = ReadNumber( infile );
                      } /* if */
                    else
                      {
                        NNError( "'>' expected" );
                      } /* else */
                  } /* if */ 
                else
                  {
                    switch ( ch )
                      {
                        case 'J' :
                        case ',' : ReadCoordinates( infile, &mx, &my );
                                   break;
                        case 'w' : weight = ReadNumber( infile );
                                   break;
                        case 'n' : ReadCoordinates( infile, &prx, &pry );
                                   break;
                        case 'N' : ReadCoordinates( infile, &px, &py );
                                   break;
                        case 'p' : ReadEnclString( infile, guard, GUARDSIZE );
                                   break;
                        case 'q' : ReadCoordinates( infile, &grx, &gry );
                                   break;
                        case 'Q' : ReadCoordinates( infile, &gx, &gy );
                                   break;
                        case 'v' : invisible = ReadNumber( infile );
                                   break;
                        case 't' : thickness = ReadNumber( infile );
                                   break;
                        case 'c' : color = ReadNumber( infile );
                               break;
                        default  : sprintf( name, "unknown token '%c'", ch );
                                   NNError( name );
                      } /* switch */
                  } /* else */
                ch = ReadCharComment( infile );
              } /* while */
            if ( ! num || ( num > AnzPlNamen ) || ! PlArray[ num ] )
              {
                NNError( "arc: incorrect place identifier" );
              } /* if */
            if ( ! num2 || ( num2 > AnzPhTrNamen ) || ! PhTrArray[ num2 ]  )
              {
                NNError( "arc: incorrect phantom transition identifier" );
              } /* if */

            NNinput_line++;
          } /* while */
      } /* if */

    /* Read in additional text strings ***************************************/
    if ( ! feof( infile ) )
      {
        ungetc( ch, infile );
        cmd=ReadCmdToken( infile );

        if ( ! strcmp( cmd, "TX" ) )
          {
            ReadNewline( infile );
            while ( ( ch = ReadCharComment( infile ) ) && ! feof( infile ) )
              {
                while ( ch != '\n' )
                  {
                    switch ( ch )
                      {
                        case '\'':
                        case '"' : ungetc( ch, infile );
                                   ReadEnclString( infile, guard, NAMESIZE );
                                   break;
                        case 'N' : ReadCoordinates( infile, &px, &py );
                                   break;
                        default  : sprintf( name, "unknown token '%c'", ch );
                                   NNError( name );
                      } /* switch */
                    ch = ReadCharComment( infile );
                  } /* while */
                NNinput_line++;
              } /* while */
          } /* if */
      } /* if */

    Free( PlArray );
    Free( TrArray );
    Free( PhTrArray );

    fclose( infile );

    return( net );

#endif /* not NEW_READFUNC */
  } /* read_new_PEP() */


/*****************************************************************************/
/***************************** write_new_PEP *********************************/
/*****************************************************************************/
/*                                                  09.01.1995 Stefan Roemer */
/*                              Letzte Aenderungen: 13.06.1996 Stefan Roemer */
/*****************************************************************************/


#if defined(__STDC__) || defined(__cplusplus)

void write_new_PEP( LMPNet net, char * PEPfilename )

#else

void write_new_PEP( net, PEPfilename )

  LMPNet net;
  char   * PEPfilename;

#endif

  {

    register Place      pl;
    register Transition tr;
    register STEdge     st;
    register TSEdge     ts;
    Cardinal32          count,
                        x1,
                        nx;     /* Koordinaten Kopfknoten */
    Coords              co;
    char                * ptr;
    FILE                * file;

    if ( ! ( file = fopen( PEPfilename, "w" ) ) )
      {
        NNError( "Could not open file for writing" );
      } /* if */
    if ( ! CM_storeCoordinates )
      {
        NNError( "Internal Error: 'CM_storeCoordinates' not set" );
      } /* if */

    f( "PEP\n" );
    fx( "%s\n", CM_nettype );
    f( "FORMAT_N\n" );
    f( "DPL s7n10@-9t2\n" );
    f( "DTR s7n10@-9t2\n" );
    f( "DPT w1t1\n" );

                                               /* Liste der Stellen ausgeben */
    f( "PL\n" );
    for ( count = 0, pl = net->Places ; pl ; pl = pl->nextplace )
      {
        pl->numname = ++count;
        fx( "\"%s\"", pl->name );
        co = (Coords) pl->inscript;
        fxy( "%u@%u", co->x, co->y );
        if ( pl->label && ( ! strcmp( CM_nettype, "PetriBox" )
                                         || ! strcmp( CM_nettype, "HLNet" ) ) )
          {
            fx( "%s", ( pl->label == ENTRY ? "e"
                                             : pl->label == EXIT ? "x" : "" ) );
          } /* if */
        else if ( pl->marks )
          {
            fx( "M%u", pl->marks );
          } /* else if */
        if ( ( co = co->next ) )
	{
	  if (co->x && co->y)
	    fxy("A%d@%d",co->x, co->y);
	}
	if ( ( co = co->next ) )
          {
            if ( ( ptr = (char *) co->x ) && *ptr)
              fx( "b\"%s\"", ptr );
            if ( ( ptr = (char *) co->y ) && *ptr)
              fx( "R\"%s\"", ptr );
            if (( co = co->next ))
              {
                if ( ( ptr = (char *) co->x ) && *ptr)
                  fx( "Z\"%s\"", ptr );
                if ( ( ptr = (char *) co->y ) && *ptr)
                  fx( "z\"%s\"", ptr );
                if (( co = co->next ))
                  {
                    if ( ( x1 = (Cardinal32)co->x ))
                      fxy( "M%um%u", x1, x1 );
                  } /* if */
              } /* if */
          } /* if */
        f( "\n" );
      } /* for ( pl ... ) */

    f( "TR\n" );
    for ( count = 0, tr = net->Transitions ; tr ; tr = tr->nexttrans )
      {
        tr->numname = ++count;

        fx("\"%s\"",tr->name);
        co = (Coords) tr->inscript;
        fxy( "%u@%u", co->x, co->y );
        if ( tr->label && ( ! strcmp( CM_nettype, "PetriBox" )
                                         || ! strcmp( CM_nettype, "HLNet" ) ) )
          {
	    /* refined flag */
            fx( "%s", "r" + ! ( *tr->label == 1 ) );
          } /* if */
        if ( ( co = co->next ) )
	{
	  if (co->x && co->y)
	    fxy( "A%d@%d", co->x, co->y );
	  if ( ( co = co->next ) )
          {
	    /* Action symbols */
            if ( ( ptr = (char *) co->x ) && *ptr)
              fx( "i\"%s\"", ptr );
	    /* meaning string */
            if ( ( ptr = (char *) co->y ) && *ptr)
              fx( "b\"%s\"", ptr );
            if (( co = co->next ))
              {
		/* Guards */
                if ( ( ptr = (char *) co->x ) && *ptr)
                  fx( "g\"%s\"", ptr );
		/* Reference string */
                if ( ( ptr = (char *) co->y ) && *ptr)
                  fx( "R\"%s\"", ptr );
              } /* if */
	    }
          } /* if */
        if (tr->time_label)
          {
            fx( "E%u",(int)tr->time_label->low);
	    if (!is_inf_lft(tr))
              fx( "L%u",(int)tr->time_label->high);
	    else
	      f("Linf");
          } /* if */
        f( "\n" );
      } /* for ( tr ... ) */

    f( "TP\n" );
    for ( tr = net->Transitions ; tr ; tr = tr->nexttrans )
      {
        for ( ts = tr->postset ; ts ; ts = ts->nextplace )
          {
            pl = ts->place;
            fxy( "%u<%u", tr->numname, pl->numname );
            co = (Coords) ts->weight;
            if ( co && co->next )
              {
		if (co->x || co->y )
                  fxy( "q%d@%d", co->x, co->y);
                if ( ( ptr = (char *) (co=co->next)->x ) && *ptr)
                  fx( "p\"%s\"", ptr );
                if ( ( nx = (Cardinal32) co->y ))
                  fx( "v%u", nx );
                if (( co = co->next ))
                  {
                    if ( ( nx = (Cardinal32) co->x )>1 )
                      fx( "w%u", nx );
                  } /* if */
                if (( co = co->next ))
                  {
                    fxy( "J%d@%d", co->x, co->y);
                  } /* if */
              } /* if */
            f( "\n" );
          } /* for */
      } /* for */

    f( "PT\n" );
    for ( pl = net->Places ; pl ; pl = pl->nextplace )
      {
        for ( st = pl->postset ; st ; st = st->nexttrans )
          {
            tr = st->trans;
            fxy( "%u>%u", pl->numname, tr->numname );
            co = (Coords) st->weight;
            if ( co && co->next )
              {
		if (co->x || co->y )
                  fxy( "q%d@%d", co->x, co->y);
                if ( ( ptr = (char *) (co=co->next)->x ) && *ptr)
                  fx( "p\"%s\"", ptr );
                if ( ( nx = (Cardinal32) co->y ))
                  fx( "v%u", nx );
                if (( co = co->next ))
                  {
                    if ( ( nx = (Cardinal32) co->x )>1 )
                      fx( "w%u", nx );
                  } /* if */
                if (( co = co->next ))
                  {
                    fxy( "J%u@%u", co->x, co->y);
                  } /* if */
              } /* if */
            f( "\n" );
          } /* for */
      } /* for */

    f( "TX\n" );
    f( "\n" );

    fclose( file );

  } /* write_new_PEP() */

/*****************************************************************************/
/* END OF FILE convertPEP.c                                                  */
/*****************************************************************************/
