/*****************************************************************************/
/* convertPEP.c                                 (c) Stefan Roemer 1993-1997  */
/*                                              Version h-1.5.1 (07.03.1997) */
/*****************************************************************************/
/* Schnittstellen-Bibliothek zum Lesen (Schreiben) des PEP-Netzdateiformats  */
/* in die (aus den) grundlegenden Datenstrukturen fuer Petri-Netze.          */
/*****************************************************************************/
/*                                   Stefan Roemer, 13.12.1993 - 01.08.1994  */
/* Neues PEP-Netzdateiformat:        Stefan Roemer, 19.12.1994 - 24.01.1995  */
/* Erweiterungen FORMAT_N2(Phantom): Stefan Roemer, 31.05.1996 - 11.06.1996  */
/* Erweiterungen FORMAT_N2(Bloecke): Stefan Roemer, 07.03.1997 - 07.03.1997  */
/*****************************************************************************/
/*                                                                           */
/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
/*                                                                           */
/* ACHTUNG: Diese Datei enthaelt elementare Routinen, die von vielen ver-    */
/* schiedenen Programmen benoetigt werden.                                   */
/*                                                                           */
/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "neunetze.h"
#include "convertPEP.h"
#include "readPEP.h"
#include "readlib.h"

/*****************************************************************************/

#define NAMESIZE 4096
#define MEANINGSIZE 4096
#define GUARDSIZE 8192
#define REFSIZE 8192
#define OMEGA 0x7FFFFFFF
#define NAMES_START 2000
#define NAMES_OFFSET 1000


/*****************************************************************************/

char       CM_nettype[ 31 ];
bool       CM_storeMeaning      = False,
           CM_storeCoordinates  = False,
           CM_storeReferences   = False,
           CM_use_imarks        = True;
M_LIST     CM_placeMList        = NULL,
           CM_transMList        = NULL;
Cardinal32 CM_countMPlaces      = 0,
           CM_countPMeanings    = 0,
           CM_countMTransitions = 0,
           CM_countTMeanings    = 0;

/*****************************************************************************/
/*                                Funktionen                                 */
/*****************************************************************************/

/*****************************************************************************/
/* Einlesen einer Zeichenkette aus einer Textdatei; fuehrende Leerzeichen    */
/* werden ueberlesen und nicht dem String zugeordnet; Kommentarzeilen werden */
/* ueberlesen. Funktion fuer das neue Dateiformat.                           */
/*                                                             Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static void ReadEnclString( FILE * file, char * string, int maxlen )

#else

static void ReadEnclString( file, string, maxlen )

  FILE * file;
  char * string;
  int  maxlen;

#endif

  {

    register char ch    = '\0',
                  * str = string;
    register int  len    = 0;
    char          delimiter;

    if (    ( ( delimiter = ReadCharComment( file ) ) != '\'' )
         && ( delimiter != '"' ) )
      {
        NNError( "ReadEnclString(): string leading ' or \" expected" );
      } /* if */
    while ( ! feof( file ) && ( ( *str = getc( file ) ) != delimiter )
            && ( len < maxlen ) )
      {
        str++;
        len++;
      } /* while */
    if ( ( len == maxlen ) && ( *str != delimiter ) )
      {
        while ( ! feof( file ) && ( ( ch = getc( file ) ) != delimiter ) )
          skip;
      } /* if */
    else
      {
        ch = *str;
      } /* else */
    if ( ch != delimiter )
      {
        sprintf( string, "ReadEnclString(): closing %c is missing",
                                                                   delimiter );
        NNError( string );
      } /* if */
    *str = '\0';
    
  } /* ReadEnclString() */


/*****************************************************************************/
/* Einlesen einer Zeichenkette aus einer Textdatei; fuehrende Leerzeichen    */
/* werden ueberlesen und nicht dem String zugeordnet.                        */
/*                                                             Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static void ReadCommand( FILE * file, char * string, int maxlen )

#else

static void ReadCommand( file, string, maxlen )

  FILE * file;
  char * string;
  int  maxlen;

#endif

  {

    if ( ReadAlphaName( file, string, maxlen ) != 3 )
      {
        NNError( "ReadCommand(): unknown command found" );
      } /* if */
    ReadSymbol( file, ':' );

  } /* ReadCommand() */


/*****************************************************************************/
/*********************** insert_place_meaning() ******************************/
/*****************************************************************************/
/*                                                  08.08.1994 Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static M_LIST insert_place_meaning( char * meaning, Place pl )

#else

static M_LIST insert_place_meaning( meaning, pl )

  char  * meaning;
  Place pl;

#endif

  {

    register M_LIST mptr,
                    newmptr;
    register P_LIST pptr;
    register char   * rptr,
                    * wptr;
    register int    comp = -1;

    rptr = wptr = meaning;
    while ( *rptr )
      {
        if ( ! isspace( *rptr ) )
          {
            *( wptr++ ) = *rptr;
          } /* if */
        rptr++;
      } /* while */
    *wptr = *rptr;
    if (    ! ( mptr = CM_placeMList )
         || ( strcmp( meaning, mptr->meaning ) < 0 ) )
      {
        ( newmptr = (M_LIST) Malloc( sizeof( M_LIST_ELEM ) ) )
                                                        ->next = CM_placeMList;
        ( CM_placeMList = newmptr )->number = 1;
        newmptr->meaning = Strdup( meaning );
        CM_countPMeanings++;
        ( pptr = newmptr->list = (P_LIST) Malloc( sizeof( P_LIST_ELEM ) ) )
                                                                 ->next = NULL;
        pptr->item = pl;
      } /* if */
    else
      {

	for (newmptr = mptr; 
		newmptr && ( comp = strcmp(newmptr->meaning, meaning) ) < 0;
		mptr = newmptr, newmptr = mptr->next);

        if ( newmptr && ! comp )
          {
            ( pptr = (P_LIST) Malloc( sizeof( P_LIST_ELEM ) ) )
                                                        ->next = newmptr->list;
            ( newmptr->list = pptr )->item = pl;
            newmptr->number++;
          } /* if */
        else
          {
            ( newmptr = (M_LIST) Malloc( sizeof( M_LIST_ELEM ) ) )
                                                           ->next = mptr->next;
            mptr->next = newmptr;
            newmptr->meaning = Strdup( meaning );
            newmptr->number = 1;
            CM_countPMeanings++;
            ( pptr = newmptr->list = (P_LIST) Malloc( sizeof( P_LIST_ELEM ) ) )
                                                                 ->next = NULL;
            pptr->item = pl;
          } /* else */
      } /* else */

    CM_countMPlaces++;

    return ( CM_placeMList );

  } /* insert_place_meaning() */


/*****************************************************************************/
/*********************** insert_trans_meaning() ******************************/
/*****************************************************************************/
/*                                                  08.08.1994 Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static M_LIST insert_trans_meaning( char * meaning, Transition tr )

#else

static M_LIST insert_trans_meaning( meaning, tr )

  char       * meaning;
  Transition tr;

#endif

  {

    register M_LIST mptr,
                    newmptr;
    register P_LIST pptr;
    register char   * rptr,
                    * wptr;
    register int    comp = -1;

    rptr = wptr = meaning;
    while ( *rptr )
      {
        if ( ! isspace( *rptr ) )
          {
            *( wptr++ ) = *rptr;
          } /* if */
        rptr++;
      } /* while */
    *wptr = *rptr;
    if (    ! ( mptr = CM_transMList )
         || ( strcmp( meaning, mptr->meaning ) < 0 ) )
      {
        ( newmptr = (M_LIST) Malloc( sizeof( M_LIST_ELEM ) ) )
							 ->next = CM_transMList;
        newmptr->meaning = Strdup( meaning );
        (CM_transMList = newmptr)->number = 1;
        CM_countTMeanings++;
        ( pptr = newmptr->list = (P_LIST) Malloc( sizeof( P_LIST_ELEM ) ) )
                                                                 ->next = NULL;
        pptr->item = (Place) tr;
      } /* if */
    else
      {

	for (newmptr = mptr; 
		newmptr && ( comp = strcmp(newmptr->meaning, meaning) ) < 0;
		mptr = newmptr, newmptr = mptr->next);

        if ( newmptr && ! comp )
          {
            ( pptr = (P_LIST) Malloc( sizeof( P_LIST_ELEM ) ) )
                                                        ->next = newmptr->list;
            ( newmptr->list = pptr )->item = (Place) tr;
            newmptr->number++;
          } /* if */
        else
          {
            ( newmptr = (M_LIST) Malloc( sizeof( M_LIST_ELEM ) ) )
                                                           ->next = mptr->next;
            mptr->next = newmptr;
            newmptr->meaning = Strdup( meaning );
            newmptr->number = 1;
            CM_countTMeanings++;
            ( pptr = newmptr->list = (P_LIST) Malloc( sizeof( P_LIST_ELEM ) ) )
                                                                 ->next = NULL;
            pptr->item = (Place) tr;
          } /* else */
      } /* else */

    CM_countMTransitions++;

    return ( CM_transMList );

  } /* insert_trans_meaning() */


/*****************************************************************************/
/************************ delete_meaning_list() ******************************/
/*****************************************************************************/
/*                                                  08.08.1994 Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void delete_meaning_list( M_LIST list )

#else

void delete_meaning_list( list )

  M_LIST list;

#endif

  {

    register M_LIST mdel;
    register P_LIST pptr,
                    pdel;

    mdel = list;
    while ( mdel )
      {
        Free( mdel->meaning );
        pdel = mdel->list;
        while ( pdel )
          {
            pdel = ( pptr = pdel )->next;
            Free( pptr );
          } /* while */
        mdel = ( list = mdel )->next;
        Free( list );
      } /* while */

  } /* delete_meaning_list() */


/*****************************************************************************/
/******************************* read_PEP ************************************/
/*****************************************************************************/
/*                                                  13.12.1993 Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LMPNet read_PEP( char * PEPfilename )

#else

LMPNet read_PEP( PEPfilename )

  char * PEPfilename;

#endif

  {

    int                 x, y = 0,
                        r1, r2,
                        tx, ty,
                        cmarks, imarks, width, capacity,
                        num, num2, weight;
    register char       ch,
                        ** Namen,
                        * S_T_Flags;
    char                cmd[513],
                        name[4097],
                        meaning[4097],
                        name2[NAMELENGTH];
    register FILE       * infile;
    register LMPNet     net;
    register Cardinal32 AnzNamen,
                        MaxNamen,
                        count;
    Place               pl_last = NULL;
    Transition          tr_last = NULL;
    STEdge              st_last = NULL;
    TSEdge              ts_last = NULL;

    NNinput_file = PEPfilename;
    NNinput_line = 1;

    /* Oeffne PEP-Datei */
    if ( ! ( infile = fopen( PEPfilename, "r" ) ) )
      {
        fprintf( stderr,
              "ERROR: CM.read_PEP(): could not open file '%s' for reading.\n",
                 PEPfilename );
        exit ( 4 );
      } /* if */

    ReadAlphaName( infile, cmd, 512 );
    if ( strcmp( cmd, "PEP" ) )
      {
        NNError( "keyword 'PEP' expected" );
      } /* if */
    ReadAlphaName( infile, cmd, 512 );
    if (    strcmp( cmd, "PetriBox" ) && strcmp( cmd, "Net" )
         && strcmp( cmd, "PTNet" )    && strcmp( cmd, "CENet" )
         && strcmp( cmd, "HLNet" ) )
      {
        NNError( "file identifier ( 'PTNet', 'PetriBox', ... ) expected" );
      } /* if */
    strncpy( CM_nettype, cmd, 30 );
    ReadAlphaName( infile, cmd, 512 );
    if ( ! ( strncmp( cmd, "FORMAT_N", 8 ) ) )
      {
        return ( read_new_PEP( PEPfilename ) );
      } /* if */
    else if ( strcmp( cmd, "FORMAT_A" ) )
      {
        NNError( "keyword 'FORMAT_A' expected" );
        return ( NULL );
      } /* if */

    /* Initialisiere net */
    Namen = (char **) Malloc( ( count = MaxNamen = NAMES_START )
                                                          * sizeof( char * ) );
    S_T_Flags = (char *) Malloc( count * sizeof( char ) );
    AnzNamen = 0;
    while ( --count )
      {
        Namen[ count ] = (char *) NULL;
        S_T_Flags[ count ] = '\0';
      } /* while */
    net = empty_net();
    net->NumComponents = 1;

    ReadCommand( infile, cmd, 512 );
    while ( strcmp( cmd, "END" ) )
      {
        if ( ! strcmp( cmd, "PLA" ) )
          {
            ReadCommand( infile, cmd, 512 );
            num = 0;
            imarks = cmarks = 0;
            *name = *meaning = '\0';
            while ( strcmp( cmd, "end" ) )
              {
                if ( ! strcmp( cmd, "idt" ) )
                  {
                    num = ReadNumber( infile );
                  } /* if */
                else if ( ! strcmp( cmd, "nam" ) )
                  {
                    ReadEnclString( infile, name, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "mea" ) )
                  {
                    ReadEnclString( infile, meaning, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dnp" ) )
                  {
                    ReadCoordinates( infile, &tx, &ty  );
                  } /* else if */
                else if ( ! strcmp( cmd, "dsz" ) )
                  {
                    ReadCoordinates( infile, &r1, &r2 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dpt" ) )
                  {
                    ReadCoordinates( infile, &x, &y );
                  } /* else if */
                else if ( ! strcmp( cmd, "dwd" ) )
                  {
                    width = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "cap" ) )
                  {
                    while ( ( ch = getc( infile ) ) == ' ' )
                      skip;
		    if ( isdigit( ch ) )
                      {
                        ungetc( ch, infile );
                        capacity = ReadNumber( infile );
                      } /* if */
                    else
                      {
                        ungetc( ch, infile );
                        ReadAlphaName( infile, name2, NAMELENGTH-1 );
                        if ( strcmp( name2, "Omega" ) )
                          {
                            NNError( "incorrect capacity" );
                          } /* if */
                      } /* else */
                  } /* else if */
                else if ( ! strcmp( cmd, "ima" ) )
                  {
                    imarks = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "cma" ) )
                  {
                    cmarks = ReadNumber( infile );
                  } /* else if */
                ReadCommand( infile, cmd, 512 );
              } /* while */
            if ( ! num )
              {
                NNError( "missing place identifier" );
              } /* if */
            if ( num > AnzNamen )
              {
                AnzNamen = num;
              } /* if */
            else if ( Namen[ num ] )
              {
                NNError( "multiply defined place identificator" );
              } /* else if */
            if ( AnzNamen >= MaxNamen )
              {
                Namen = (char **) Realloc( (char *) Namen,
                             ( MaxNamen += NAMES_OFFSET ) * sizeof( char * ) );
                S_T_Flags = (char *) Realloc( (char *) S_T_Flags,
                                                   MaxNamen * sizeof( char ) );
                for ( count = MaxNamen - NAMES_OFFSET ;
                                                   count < MaxNamen ; count++ )
                  {
                    Namen[ count ] = (char *) NULL;
                    S_T_Flags[ count ] = '\0';
                  } /* for */
              } /* if */
            if ( ! *name )
              {
                sprintf( name, "S%d", num );
              } /* if */
            Namen[ num ] = fast_insert_place( net, name,
                           CM_use_imarks ? imarks : cmarks, &pl_last ) -> name;
            if ( CM_storeMeaning && *meaning )
              {
                insert_place_meaning( meaning, pl_last );
              } /* if */
            pl_last->label = EMPTY;
            pl_last->component = 1;
            S_T_Flags[ num ] = 'S';
            if ( CM_storeCoordinates )
              {
                Coords co;
                pl_last->inscript = (char *)
                              ( co = (Coords) Malloc( sizeof( CoordEntry ) ) );
                co->next = NULL;
                co->x = x;
                co->y = y;
              } /* if */
          } /* if */
        else if (    ! strcmp( cmd, "TRA" )
                  || ! strcmp( cmd, "RTR" ) )
          {
            ReadCommand( infile, cmd, 512 );
            num = 0;
            *name = *meaning =  '\0';
            while ( strcmp( cmd, "end" ) )
              {
                if ( ! strcmp( cmd, "idt" ) )
                  {
                    num = ReadNumber( infile );
                  } /* if */
                else if ( ! strcmp( cmd, "nam" ) )
                  {
                    ReadEnclString( infile, name, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "mea" ) )
                  {
                    ReadEnclString( infile, meaning, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dnp" ) )
                  {
                    ReadCoordinates( infile, &tx, &ty );
                  } /* else if */
                else if ( ! strcmp( cmd, "dsz" ) )
                  {
                    ReadCoordinates( infile, &r1, &r2 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dpt" ) )
                  {
                    ReadCoordinates( infile, &x, &y );
                  } /* else if */
                else if ( ! strcmp( cmd, "dwd" ) )
                  {
                    width = ReadNumber( infile );
                  } /* else if */
                ReadCommand( infile, cmd, 512 );
              } /* while */
            if ( ! num )
              {
                NNError( "missing transition identifier" );
              } /* if */
            if ( num > AnzNamen )
              {
                AnzNamen = num;
              } /* if */
            else if ( Namen[ num ] )
              {
                NNError( "multiply defined transition identifier" );
              } /* else if */
            if ( AnzNamen >= MaxNamen )
              {
                Namen = (char **) Realloc( (char *) Namen,
                                     ( MaxNamen += NAMES_OFFSET ) * sizeof( char * ) );
                S_T_Flags = (char *) Realloc( (char *) S_T_Flags,
                                                   MaxNamen * sizeof( char ) );
                for ( count = MaxNamen - NAMES_OFFSET ; count < MaxNamen ; count++ )
                  {
                    Namen[ count ] = (char *) NULL;
                    S_T_Flags[ count ] = '\0';
                  } /* for */
              } /* if */
            if ( ! *name )
              {
                sprintf( name, "T%d", num );
              } /* if */
            Namen[ num ] = fast_insert_transition( net, name, &tr_last )
                                                                       -> name;
            if ( CM_storeMeaning && *meaning )
              {
                insert_trans_meaning( meaning, tr_last );
              } /* if */
            tr_last->label = NULL;
            S_T_Flags[ num ] = 'T';
            if ( CM_storeCoordinates )
              {
                Coords co;
                tr_last->inscript = (char *)
                              ( co = (Coords) Malloc( sizeof( CoordEntry ) ) );
                co->next = NULL;
                co->x = x;
                co->y = y;
              } /* if */
          } /* else if */
        else if ( ! strcmp( cmd, "ARC" ) )
          {
            Coords lastco  = NULL,
                   firstco = NULL;
            ReadCommand( infile, cmd, 512 );
            num = num2 = weight = 0;
            *name = *meaning = '\0';
            while ( strcmp( cmd, "end" ) )
              {
                if ( ! strcmp( cmd, "lpt" ) )
                  {
                    ReadCoordinates( infile, &x, &y );
                  } /* else if */
                else if ( ! strcmp( cmd, "dsz" ) )
                  {
                    width = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "wgt" ) )
                  {
                    weight = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "sou" ) )
                  {
                    num = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "des" ) )
                  {
                    num2 = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "nam" ) )
                  {
                    ReadEnclString( infile, name, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "mea" ) )
                  {
                    ReadEnclString( infile, meaning, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "vsf" ) )
                  {
                    char flag[2];
                    ReadAlphaName( infile, flag, 1 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dpt" ) )
                  {
                    ReadCoordinates( infile, &x, &y );
                    if ( CM_storeCoordinates )
                      {
                        if ( lastco )
                          {
                            lastco = lastco->next
                                   = (Coords) Malloc( sizeof( CoordEntry ) );
                          } /* if */
                        else
                          {
                            firstco = lastco
                                    = (Coords) Malloc( sizeof( CoordEntry ) );
                          } /* else */
                        lastco->next = NULL;
                        lastco->x = x;
                        lastco->y = y;
                      } /* if */
                  } /* else if */
                ReadCommand( infile, cmd, 512 );
              } /* while */
            if ( ! num || ( num > AnzNamen ) || ! Namen[ num ] )
              {
                NNError( "arc: missing source identifier" );
              } /* if */
            if ( ! num2 || ( num2 > AnzNamen ) || ! Namen[ num2 ]  )
              {
                NNError( "arc: missing destination identifier" );
              } /* if */
            if ( ( S_T_Flags[ num ] == 'S' ) && ( S_T_Flags[ num2 ] == 'T' ) )
              {
                fast_insert_PT_con( net, Namen[ num ], Namen[ num2 ],
                                        weight, &pl_last, &tr_last, &st_last );
                if ( CM_storeCoordinates )
                  {
                    lastco = (Coords) Malloc( sizeof( CoordEntry ) );
                    lastco->x = weight;
                    lastco->y = 0;
                    lastco->next = firstco;
                    st_last->weight = (Cardinal32) lastco;
                  } /* if */
              } /* if */
            else if (    ( S_T_Flags[ num ] == 'T' )
                      && ( S_T_Flags[ num2 ] == 'S' ) )
              {
                fast_insert_TP_con( net, Namen[ num2 ], Namen[ num ],
                                        weight, &pl_last, &tr_last, &ts_last );
                if ( CM_storeCoordinates )
                  {
                    lastco = (Coords) Malloc( sizeof( CoordEntry ) );
                    lastco->x = weight;
                    lastco->y = 0;
                    lastco->next = firstco;
                    ts_last->weight = (Cardinal32) lastco;
                  } /* if */
              } /* if */
            else
              {
                NNError( "faulty PEP net file: S->S resp. T->T arc found" );
              } /* else */
          } /* else if */
        else if ( ! strcmp( cmd, "IPL" ) )
          {
            ReadCommand( infile, cmd, 512 );
            num = 0;
            imarks = 1;
            cmarks = 0;
            *name = *meaning = '\0';
            while ( strcmp( cmd, "end" ) )
              {
                if ( ! strcmp( cmd, "idt" ) )
                  {
                    num = ReadNumber( infile );
                  } /* if */
                else if ( ! strcmp( cmd, "nam" ) )
                  {
                    ReadEnclString( infile, name, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "mea" ) )
                  {
                    ReadEnclString( infile, meaning, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dnp" ) )
                  {
                    ReadCoordinates( infile, &tx, &ty );
                  } /* else if */
                else if ( ! strcmp( cmd, "dsz" ) )
                  {
                    ReadCoordinates( infile, &r1, &r2 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dpt" ) )
                  {
                    ReadCoordinates( infile, &x, &y );
                  } /* else if */
                else if ( ! strcmp( cmd, "dwd" ) )
                  {
                    width = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "cap" ) )
                  {
                    while ( ( ch = getc( infile ) ) == ' ' )
                      skip;
		    if ( isdigit( ch ) )
                      {
                        ungetc( ch, infile );
                        capacity = ReadNumber( infile );
                      } /* if */
                    else
                      {
                        ungetc( ch, infile );
                        ReadAlphaName( infile, name2, NAMELENGTH-1 );
                        if ( strcmp( name2, "Omega" ) )
                          {
                            fprintf( stderr,
                                            "ERROR: CM.read_PEP(): capacity" );
                            fprintf( stderr, "'Omega' expected.\n" );
                          } /* if */
                      } /* else */
                  } /* else if */
                else if ( ! strcmp( cmd, "ima" ) )
                  {
                    imarks = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "cma" ) )
                  {
                    cmarks = ReadNumber( infile );
                  } /* else if */
                ReadCommand( infile, cmd, 512 );
              } /* while */
            if ( ! num )
              {
                NNError( "missing place identifier" );
              } /* if */
            if ( num > AnzNamen )
              {
                AnzNamen = num;
              } /* if */
            else if ( Namen[ num ] )
              {
                NNError( "multiply defined place identifer" );
              } /* else if */
            if ( AnzNamen >= MaxNamen )
              {
                Namen = (char **) Realloc( (char *) Namen,
                              ( MaxNamen += NAMES_OFFSET ) * sizeof( char * ) );
                S_T_Flags = (char *) Realloc( (char *) S_T_Flags,
                                                    MaxNamen * sizeof( char ) );
                for ( count = MaxNamen - NAMES_OFFSET ;
                                                    count < MaxNamen ; count++ )
                  {
                    Namen[ count ] = (char *) NULL;
                    S_T_Flags[ count ] = '\0';
                  } /* for */
              } /* if */
            if ( ! *name )
              {
                sprintf( name, "S%d", num );
              } /* if */
            Namen[ num ] = fast_insert_place( net, name,
                                 CM_use_imarks ? 1 : cmarks, &pl_last ) -> name;
            if ( CM_storeMeaning && *meaning )
              {
                insert_place_meaning( meaning, pl_last );
              } /* if */
            pl_last->label = ENTRY;
            pl_last->component = 0;
            S_T_Flags[ num ] = 'S';
            if ( CM_storeCoordinates )
              {
                Coords co;
                pl_last->inscript = (char *)
                              ( co = (Coords) Malloc( sizeof( CoordEntry ) ) );
                co->next = NULL;
                co->x = x;
                co->y = y;
              } /* if */
          } /* else if */
        else if ( ! strcmp( cmd, "OPL" ) )
          {
            ReadCommand( infile, cmd, 512 );
            num = imarks = cmarks = 0;
            *name = *meaning = '\0';
            while ( strcmp( cmd, "end" ) )
              {
                if ( ! strcmp( cmd, "idt" ) )
                  {
                    num = ReadNumber( infile );
                  } /* if */
                else if ( ! strcmp( cmd, "nam" ) )
                  {
                    ReadEnclString( infile, name, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "mea" ) )
                  {
                    ReadEnclString( infile, meaning, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dnp" ) )
                  {
                    ReadCoordinates( infile, &tx, &ty );
                  } /* else if */
                else if ( ! strcmp( cmd, "dsz" ) )
                  {
                    ReadCoordinates( infile, &r1, &r2 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dpt" ) )
                  {
                    ReadCoordinates( infile, &x, &y );
                  } /* else if */
                else if ( ! strcmp( cmd, "dwd" ) )
                  {
                    width = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "cap" ) )
                  {
                    while ( ( ch = getc( infile ) ) == ' ' )
                      skip;
		    if ( isdigit( ch ) )
                      {
                        ungetc( ch, infile );
                        capacity = ReadNumber( infile );
                      } /* if */
                    else
                      {
                        ungetc( ch, infile );
                        ReadAlphaName( infile, name2, NAMELENGTH-1 );
                        if ( strcmp( name2, "Omega" ) )
                          {
                            fprintf( stderr,
                                            "ERROR: CM.read_PEP(): capacity" );
                            fprintf( stderr, "'Omega' expected.\n" );
                          } /* if */
                      } /* else */
                  } /* else if */
                else if ( ! strcmp( cmd, "ima" ) )
                  {
                    imarks = ReadNumber( infile );
                  } /* else if */
                else if ( ! strcmp( cmd, "cma" ) )
                  {
                    cmarks = ReadNumber( infile );
                  } /* else if */
                ReadCommand( infile, cmd, 512 );
              } /* while */
            if ( ! num )
              {
                NNError( "missing place identifier" );
              } /* if */
            if ( num > AnzNamen )
              {
                AnzNamen = num;
              } /* if */
            else if ( Namen[ num ] )
              {
                NNError( "multiply defined place identifier" );
              } /* else if */
            if ( AnzNamen >= MaxNamen )
              {
                Namen = (char **) Realloc( (char *) Namen,
                              ( MaxNamen += NAMES_OFFSET ) * sizeof( char * ) );
                S_T_Flags = (char *) Realloc( (char *) S_T_Flags,
                                                    MaxNamen * sizeof( char ) );
                for ( count = MaxNamen - NAMES_OFFSET ;
                                                    count < MaxNamen ; count++ )
                  {
                    Namen[ count ] = (char *) NULL;
                    S_T_Flags[ count ] = '\0';
                  } /* for */
              } /* if */
            if ( ! *name )
              {
                sprintf( name, "S%d", num );
              } /* if */
            Namen[ num ] = fast_insert_place( net, name,
                           CM_use_imarks ? imarks : cmarks, &pl_last ) -> name;
            if ( CM_storeMeaning && *meaning )
              {
                insert_place_meaning( meaning, pl_last );
              } /* if */
            pl_last->label = EXIT;
            pl_last->component = 0;
            S_T_Flags[ num ] = 'S';
            if ( CM_storeCoordinates )
              {
                Coords co;
                pl_last->inscript = (char *)
                              ( co = (Coords) Malloc( sizeof( CoordEntry ) ) );
                co->next = NULL;
                co->x = x;
                co->y = y;
              } /* if */
          } /* else if */
        else if ( ! strcmp( cmd, "TXT" ) )
          {
            ReadCommand( infile, cmd, 512 );
            num = 0;
            while ( strcmp( cmd, "end" ) )
              {
                if ( ! strcmp( cmd, "str" ) )
                  {
                    ReadEnclString( infile, name, 512 );
                  } /* else if */
                else if ( ! strcmp( cmd, "dpt" ) )
                  {
                    ReadCoordinates( infile, &x, &y );
                  } /* else if */
                ReadCommand( infile, cmd, 512 );
              } /* while */
          } /* else if */
        else if ( ! strcmp( cmd, "FML" ) )
          {
            ReadNewline( infile );
            ungetc( '\n', infile );
          } /* else if */
        else if ( ! strcmp( cmd, "haf" ) )
          {
            ReadNewline( infile );
          } /* else if */
        else if ( ! strcmp( cmd, "set" ) )
          {
            ReadNewline( infile );
          } /* else if */
        else if ( ! strcmp( cmd, "pap" ) )
          {
            ReadNewline( infile );
          } /* else if */
        else if ( ! strcmp( cmd, "com" ) )
          {
            ReadEnclString( infile, name, 4096 );
          } /* else if */
        else if ( ! strcmp( cmd, "end" ) )
          {
          } /* else if */
        else
          {
            fprintf( stderr,
            "ERROR: CM.read_PEP(): unknown PEP command found: '%s'.\n", cmd );
          } /* else */
        ReadNewline( infile );
        ReadCommand( infile, cmd, 512 );
      } /* while */

    fclose( infile );

    Free( Namen );

    return( net );

  } /* read_PEP() */


/*****************************************************************************/
/****************************** write_PEP ***********************************/
/*****************************************************************************/
/*                                                  13.12.1993 Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void write_PEP( LMPNet net, char * PEPfilename )

#else

void write_PEP( net, PEPfilename )
  /* Gibt die mit net bezeichnete Matrix als Inzidenzmatrix nach 'file' aus  */

  LMPNet net;
  char   * PEPfilename;

#endif

  {

    /* NOCH NICHT IMPLEMENTIERT !!! */

  } /* write_PEP() */


/*****************************************************************************/
/***************************** read_new_PEP **********************************/
/*****************************************************************************/
/* Neu eingefuegt (neues Format)      19.12.1994 - 09.01.1995 Stefan Roemer  */
/*                                    Bloecke      07.03.1997 Stefan Roemer  */
/* Neue Einleseroutine				   April 1997 Stefan Schwoon */
/*****************************************************************************/

char		buffer[10];
bool		autonumbering;
int		placecount, transcount;

LMPNet		rnet;
Place		*PlArray;
Transition	*TrArray, *PhTrArray;

Cardinal32	AnzPlNamen, MaxPlNamen,
		AnzTrNamen, MaxTrNamen,
		AnzPhTrNamen, MaxPhTrNamen;

char		*name, *meaning, *block, *guard, *name2,
		*cmarking, *reference, *corrtran, *blk;
int		entry_place, exit_place, invisible, ident,
		imarks, cmarks, weight;
t_coords	*xy, *mrc, *mc;

int		def_mrx = 10,
		def_mry = 10;

int rp_insert_place();
int rp_insert_trans();
int rp_insert_tparc();
int rp_insert_ptarc();

t_dest place_dest[] =
	{ { '\'',&name },	/* Bezeichner		*/
	  { '"', &name },	/*     "		*/
	  { '@', &xy },		/* Position		*/
	  { '0', &ident },	/* numerischer Name	*/
	  { 'M', &imarks },	/* Init-Marking (num)	*/
	  { 'm', &cmarks },	/* Current-Markung	*/
	  { 'e', &entry_place },/* Entry-Stelle		*/
	  { 'x', &exit_place },	/* Exit-Stelle		*/
	  { 'Z', &guard },	/* Typ			*/
	  { 'z', &name2 },	/* Init-Marking		*/
	  { 'y', &cmarking },	/* Current-Marking	*/
	  { 'a', &mrc },	/* rel. Pos. Bedeutung  */
	  { 'A', &mc },		/* abs. Pos. Bedeutung  */
	  { 'b', &meaning },	/* Bedeutung		*/
	  { 'R', &reference },	/* Referenz		*/
	  {  0 ,  0 } };
  
t_dest trans_dest[] =
	{ { '\'',&name },	/* Bezeichner		*/
	  { '"', &name },	/*     "		*/
	  { '@', &xy },		/* Position		*/
	  { '0', &ident },	/* numerischer Name	*/
	  { 'b', &meaning },	/* Bedeutung		*/
	  { 'a', &mrc },	/* rel. Pos. Bedeutung  */
	  { 'i', &name2 },	/* Aktionsterme		*/
	  { 'g', &guard },	/* Value Term		*/
	  { 'R', &reference },	/* Referenz		*/
	  {  0 ,  0 } };

t_dest arc_dest[] =
	{ { '@', &xy },		/* Quelle/Ziel		*/
	  { 'w', &weight },	/* Gewicht		*/
	  { 'Q', &mc },		/* abs. Pos. Beschr.	*/
	  { 'p', &guard },	/* Beschriftung		*/
	  { 'v', &invisible },	/* Sichtbarkeit		*/
	  {  0 ,  0 } };

t_blockdest netdest[] =
	{ { "PL", rp_insert_place, NULL, place_dest },
	  { "TR", rp_insert_trans, NULL, trans_dest },
	  { "TP", rp_insert_tparc, NULL, arc_dest },
	  { "PT", rp_insert_ptarc, NULL, arc_dest },
	  { NULL, NULL, NULL, NULL } };

#if defined(__STDC__) || defined(__cplusplus)
LMPNet read_new_PEP(char *PEPfilename)
#else
LMPNet read_new_PEP(PEPfilename)
  char *PEPfilename;
#endif
{
	int count;

	PlArray = (Place*) Malloc((count = MaxPlNamen = MaxTrNamen
			= MaxPhTrNamen = NAMES_START) * sizeof(Place));
	TrArray = (Transition*) Malloc(count * sizeof(Transition));
	AnzPlNamen = AnzTrNamen = AnzPhTrNamen = 0;
	while (--count)
	{
		PlArray[count] = (Place) NULL;
		TrArray[count] = (Transition) NULL;
	} /* while */

	placecount = 0;
	transcount = 0;
	autonumbering = True;

	rnet = empty_net();
	rnet->NumComponents = 1;

	read_PEP_net(PEPfilename,netdest);
	strncpy(CM_nettype,filetype,30);

	free(PlArray);
	free(TrArray);

	return rnet;

}

#define Error(msg) { sprintf msg; PEPerror(errbuf); return 1; }
static char nullstring = '\0';

int rp_insert_place()
{
	static Place pl_last = NULL;

	placecount++;
	if (ident && ident != placecount) autonumbering = False;
	if (!ident && autonumbering) ident = placecount;
	if (!ident) Error((errbuf,"missing place identifier"));

        if (ident > AnzPlNamen)
		AnzPlNamen = ident;
        else if (PlArray[ident])
		Error((errbuf,"place identifier %d muliply used",ident));

        while (AnzPlNamen >= MaxPlNamen)
	{
		int count;
		PlArray = (Place*) Realloc((char*) PlArray,
                            (MaxPlNamen += NAMES_OFFSET) * sizeof(Place));
		for (count = MaxPlNamen - NAMES_OFFSET; count < MaxPlNamen;)
			PlArray[count++] = (Place) NULL;
	} /* if */

	if (!name)
	{
		sprintf(buffer, "PL%d", ident);
		name = Strdup(buffer);
	} /* if */

	PlArray[ident] = fast_insert_place(rnet, name,
			CM_use_imarks? imarks : cmarks, &pl_last);

        if (CM_storeMeaning && meaning)
		insert_place_meaning(meaning, pl_last);

        if (!strcmp(filetype, "PetriBox"))
	{
		if (CM_use_imarks)
		    pl_last->marks = entry_place? 1 : 
			(guard || name2 || cmarking)? 0 : imarks;

		pl_last->label = entry_place? ENTRY : exit_place? EXIT : EMPTY;
	}

	pl_last->component = 1;

	if (!mc)
	{
		mc = (t_coords*) Malloc(sizeof(t_coords));
		mc->x = xy->x + (mrc? mrc->x : def_mrx);
		mc->y = xy->y + (mrc? mrc->y : def_mry);
	}

        if (CM_storeCoordinates || CM_storeReferences)
	{
		Coords co;

		if (!reference) reference = Strdup(&nullstring);
		if (!meaning) meaning = Strdup(&nullstring);
		if (!guard) guard = Strdup(&nullstring);
		if (!name2) name2 = Strdup(&nullstring);

		pl_last->inscript = (char*)
			(co = (Coords) Malloc(sizeof(CoordEntry)));
		co->x = xy->x;
		co->y = xy->y;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = mc->x;
		co->y = mc->y;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = (int) meaning;
		co->y = (int) reference;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = (int) guard;
		co->y = (int) name2;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = (int) cmarks;
		co->next = NULL;
	} /* if */

	free(xy); free(mc);
	if (mrc) free(mrc);
	if (cmarking) free(cmarking);
	return 0;

}

int rp_insert_trans()
{
	static Transition tr_last = NULL;

	if (!transcount++) autonumbering = True;
	if (ident && ident != transcount) autonumbering = False;
	if (!ident && autonumbering) ident = transcount;
	if (!ident) Error((errbuf,"missing transition identifier"));

        if (ident > AnzTrNamen)
		AnzTrNamen = ident;
        else if (TrArray[ident])
		Error((errbuf,"transition identifier %d muliply used",ident));

	while (AnzTrNamen >= MaxTrNamen)
	{
		int count;
		TrArray = (Transition*) Realloc((char*) TrArray,
			(MaxTrNamen += NAMES_OFFSET) * sizeof(Transition));
		for (count = MaxTrNamen - NAMES_OFFSET; count < MaxTrNamen;)
			TrArray[count++] = (Transition) NULL;

	} /* if */
	if (!name)
	{
		sprintf(buffer, "TR%d", ident);
		name = Strdup(buffer);
	} /* if */
        TrArray[ident] = fast_insert_transition(rnet, name, &tr_last);

	if (CM_storeMeaning && meaning)
		insert_trans_meaning(meaning, tr_last);
        tr_last->label = NULL;

	if (CM_storeCoordinates || CM_storeReferences)
	{
		Coords co;

		if (!reference) reference = Strdup(&nullstring);
		if (!meaning) meaning = Strdup(&nullstring);
		if (!guard) guard = Strdup(&nullstring);
		if (!name2) name2 = Strdup(&nullstring);

		tr_last->inscript = (char *)
			(co = (Coords) Malloc(sizeof(CoordEntry)));
		co->x = xy->x;
		co->y = xy->y;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = xy->x + (mrc? mrc->x : def_mrx);
		co->y = xy->y + (mrc? mrc->y : def_mry);
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = (int) name2;
		co->y = (int) meaning;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = (int) guard;
		co->y = (int) reference;
		co->next = NULL;
	} /* if */

	free(xy);
	if (mrc) free(mrc);
	return 0;

}

int rp_insert_tparc()
{
	static TSEdge ts_last = NULL;
	Place pl_last;
	Transition tr_last;

        if (!xy->x|| (xy->x > AnzTrNamen) || !TrArray[xy->x])
		Error((errbuf,"arc: incorrect transition identifier"));
        if (!xy->y || (xy->y > AnzPlNamen) || !PlArray[xy->y])
		Error((errbuf,"arc: incorrect place identifier"));

	if (!weight) weight = 1;
        tr_last = TrArray[xy->x];
        pl_last = PlArray[xy->y];
	fast_insert_TP_con(rnet, pl_last->name, tr_last->name, weight,
					&pl_last, &tr_last, &ts_last);

	if (CM_storeCoordinates)
	{
		Coords co;

		if (!guard) guard = Strdup(&nullstring);

		ts_last->weight = (Cardinal32)
			(co = (Coords) Malloc(sizeof(CoordEntry)));
		co->x = mc? mc->x : 0;
		co->y = mc? mc->y : 0;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = (int) guard;
		co->y = (int) invisible;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = (int) weight;
		co->next = NULL;
	} /* if */

	free(xy);
	if (mc) free(mc);
	return 0;

}

int rp_insert_ptarc()
{
	static STEdge st_last = NULL;
	Place pl_last;
	Transition tr_last;

        if (!xy->x || (xy->x > AnzPlNamen) || !PlArray[xy->x])
		Error((errbuf,"arc: incorrect place identifier"));
        if (!xy->y|| (xy->y > AnzTrNamen) || !TrArray[xy->y])
		Error((errbuf,"arc: incorrect transition identifier"));

	if (!weight) weight = 1;
	pl_last = PlArray[xy->x];
	tr_last = TrArray[xy->y];
	fast_insert_PT_con(rnet, pl_last->name, tr_last->name, weight,
					&pl_last, &tr_last, &st_last);

	if (CM_storeCoordinates)
	{
		Coords co;

		if (!guard) guard = Strdup(&nullstring);

		st_last->weight = (Cardinal32)
			(co = (Coords) Malloc(sizeof(CoordEntry)));
		co->x = mc? mc->x : 0;
		co->y = mc? mc->y : 0;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = (int) guard;
		co->y = (int) invisible;
		co = co->next = (Coords) Malloc(sizeof(CoordEntry));
		co->x = (int) weight;
		co->next = NULL;
	} /* if */

	free(xy);
	if (mc) free(mc);
	return 0;

}


/*****************************************************************************/
/***************************** write_new_PEP *********************************/
/*****************************************************************************/
/*                                                  09.01.1995 Stefan Roemer */
/*                              Letzte Aenderungen: 13.06.1996 Stefan Roemer */
/*****************************************************************************/

#   define f( STRING ) fprintf( file, STRING )
#   define fx( STRING, X ) fprintf( file, STRING, X )
#   define fxy( STRING, X, Y ) fprintf( file, STRING, X, Y )
#   define fxyz( STRING, X, Y, Z ) fprintf( file, STRING, X, Y, Z )
#   define fxyxy( STRING, X1, Y1, X2, Y2 ) fprintf( file, STRING, X1,Y1,X2,Y2 )
#   define fxyxyz( STRING, X1,Y1,X2,Y2,Z ) fprintf( file, STRING,X1,Y1,X2,Y2,Z )

/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void write_new_PEP( LMPNet net, char * PEPfilename )

#else

void write_new_PEP( net, PEPfilename )

  LMPNet net;
  char   * PEPfilename;

#endif

  {

    register Place      pl;
    register Transition tr;
    register STEdge     st;
    register TSEdge     ts;
    Cardinal32          count,
                        x1,
                        nx;     /* Koordinaten Kopfknoten */
    Coords              co;
    char                * ptr;
    FILE                * file;

    if ( ! ( file = fopen( PEPfilename, "w" ) ) )
      {
        NNError( "Could not open file for writing" );
      } /* if */
    if ( ! CM_storeCoordinates )
      {
        NNError( "Internal Error: 'CM_storeCoordinates' not set" );
      } /* if */

    f( "PEP\n" );
    fx( "%s\n", CM_nettype );
    f( "FORMAT_N\n" );
    f( "DPL s7n10@-9t2\n" );
    f( "DTR s7n10@-9t2\n" );
    f( "DPT w1t1\n" );

                                               /* Liste der Stellen ausgeben */
    f( "PL\n" );
    for ( count = 0, pl = net->Places ; pl ; pl = pl->nextplace )
      {
        pl->numname = ++count;
        fx( "\"%s\"", pl->name );
        co = (Coords) pl->inscript;
        fxy( "%u@%u", co->x, co->y );
        if ( pl->label && ( ! strcmp( CM_nettype, "PetriBox" )
                                         || ! strcmp( CM_nettype, "HLNet" ) ) )
          {
            fx( "%s", ( pl->label == ENTRY ? "e"
                                             : pl->label == EXIT ? "x" : "" ) );
          } /* if */
        else if ( pl->marks )
          {
            fx( "M%u", pl->marks );
          } /* else if */
        if ( ( co = co->next ) && ( co = co->next ) )
          {
            ( *( ptr = (char *) co->x ) && fx( "b\"%s\"", ptr ) );
            ( *( ptr = (char *) co->y ) && fx( "R\"%s\"", ptr ) );
            if ( co = co->next )
              {
                ( *( ptr = (char *) co->x ) && fx( "Z\"%s\"", ptr ) );
                ( *( ptr = (char *) co->y ) && fx( "z\"%s\"", ptr ) );
                if ( co = co->next )
                  {
                    ( ( x1 = (Cardinal32)co->x ) && fxy( "M%um%u", x1, x1 ) );
                  } /* if */
              } /* if */
          } /* if */
        f( "\n" );
      } /* for ( pl ... ) */

    f( "TR\n" );
    for ( count = 0, tr = net->Transitions ; tr ; tr = tr->nexttrans )
      {
        tr->numname = ++count;

        fx("\"%s\"",tr->name);
        co = (Coords) tr->inscript;
        fxy( "%u@%u", co->x, co->y );
        if ( tr->label && ( ! strcmp( CM_nettype, "PetriBox" )
                                         || ! strcmp( CM_nettype, "HLNet" ) ) )
          {
            fx( "%s", "r" + ! ( tr->label && ( *tr->label == 1 ) ) );
          } /* if */
        if ( ( co = co->next ) && ( co = co->next ) )
          {
            ( *( ptr = (char *) co->x ) && fx( "i\"%s\"", ptr ) );
            ( *( ptr = (char *) co->y ) && fx( "b\"%s\"", ptr ) );
            if ( co = co->next )
              {
                ( *( ptr = (char *) co->x ) && fx( "g\"%s\"", ptr ) );
                ( *( ptr = (char *) co->y ) && fx( "R\"%s\"", ptr ) );
              } /* if */
          } /* if */
        f( "\n" );
      } /* for ( tr ... ) */

    f( "TP\n" );
    for ( tr = net->Transitions ; tr ; tr = tr->nexttrans )
      {
        for ( ts = tr->postset ; ts ; ts = ts->nextplace )
          {
            pl = ts->place;
            fxy( "%u<%u", tr->numname, pl->numname );
            co = (Coords) ts->weight;
            if ( co && ( co = co->next ) )
              {
                ( *( ptr = (char *) co->x ) && fx( "p\"%s\"", ptr ) );
                ( ( nx = (Cardinal32) co->y ) && fx( "v%u", nx ) );
                if ( co = co->next )
                  {
                    ( ( ( nx = (Cardinal32) co->x )>1 ) && fx( "w%u", nx ) );
                  } /* if */
              } /* if */
            f( "\n" );
          } /* for */
      } /* for */

    f( "PT\n" );
    for ( pl = net->Places ; pl ; pl = pl->nextplace )
      {
        for ( st = pl->postset ; st ; st = st->nexttrans )
          {
            tr = st->trans;
            fxy( "%u>%u", pl->numname, tr->numname );
            co = (Coords) st->weight;
            if ( co && ( co = co->next ) )
              {
                ( *( ptr = (char *) co->x ) && fx( "p\"%s\"", ptr ) );
                ( ( nx = (Cardinal32) co->y ) && fx( "v%u", nx ) );
                if ( co = co->next )
                  {
                    ( ( ( nx = (Cardinal32) co->x )>1 ) && fx( "w%u", nx ) );
                  } /* if */
              } /* if */
            f( "\n" );
          } /* for */
      } /* for */

    f( "TX\n" );
    f( "\n" );

    fclose( file );

  } /* write_new_PEP() */


/*****************************************************************************/
/* Erzeugen von Namen fuer die Elemente eines Netzes:                        */
/* Stellenknoten werden mit "Sn" bezeichnet, wobei 'n' eine laufende Nummer  */
/* ( 1..net->NumPlaces ) ist; die Transitionen werden entsprechend mit       */
/* "Tm", m = 1..net->NumTransitions, benannt.                                */
/*                                                             Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LMPNet generate_S_T_names( LMPNet net )

#else

LMPNet generate_S_T_names( net )

  LMPNet net;

#endif

  {

    register Cardinal32 counter;
    register Place      pl;
    register Transition tr;
    char                str[30];

    counter = 0;
    pl = net->Places;
    while ( pl )
      {
        if ( pl->name )
          {
            Free( pl->name );
          } /* if */
        sprintf( str, "S%u", ++counter );
        pl->name = Strdup( str );
        pl = pl->nextplace;
      } /* while */

    counter = 0;
    tr = net->Transitions;
    while ( tr )
      {
        if ( tr->name )
          {
            Free( tr->name );
          } /* if */
        sprintf( str, "T%u", ++counter );
        tr->name = Strdup( str );
        tr = tr->nexttrans;
      } /* while */

    return ( net );

  } /* generate_S_T_names() */


/*****************************************************************************/

static Cardinal32 CP_AnzTrNamen,
                  CP_AnzTrExtra,
                  CP_MaxTrExtra;
static char       ** CP_TrNamen,
                  ** CP_TrExtra;
static Place      CP_pllast;
static Transition CP_trlast,
                  * CP_TransArray = NULL,
                  * CP_TransExtraArray = NULL;


/*****************************************************************************/
/* Einlesen einer Zeichenkette aus einer Textdatei; fuehrende Leerzeichen    */
/* werden ueberlesen und nicht dem String zugeordnet.                        */
/*                                                             Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static char * Read_STG_Name( FILE * file, char * string, int maxlen,
                             bool * endofline )

#else

static char * Read_STG_Name( file, string, maxlen, endofline )

  FILE * file;
  char * string;
  int  maxlen;
  bool * endofline;

#endif

  {

    register char ch;
    register int  len = 0;
    char          * pos = NULL;

    while( ( ( ch = getc( file ) ) == ' ' ) || ( ch == '\n' ) || ( ch == '#' ) )
      {
        if ( ch == '\n' )
          {
            NNinput_line++;
          } /* if */
        else if ( ch == '#' )
          {
            while ( ( ch = getc( file ) ) != '\n' )
              skip;
            NNinput_line++;
          } /* else */
      } /* while */
    ungetc( ch, file );
    while(    ( ( *string = ch = getc( file ) ) != ' ' ) && ( ch != '\n' )
           && ( ch != ',' ) && ( ch != '>' ) && ( ch != '}' ) && ( ch != '=' )
           && ( ch != EOF ) && ( len < maxlen ) )
      {
        if ( ! pos && ( ( ch == '+' ) || ( ch == '-' ) || ( ch == '~' ) ) )
          {
            pos = string;
          } /* if */
        string++;
        len++;
      } /* while */
    if ( ( ch == '>' ) || ( ch == ',' ) || ( ch == '}' ) || ( ch == '=' ) )
      ungetc( ch, file );
    if ( len == maxlen )
      {
        NNError( "read_STG(): length of identifier too long" );
        return ( NULL );
      } /* if */
    *endofline = (bool) ( *string != ' ' );
    ( ( *string == '\n' ) && NNinput_line++ );
    ( pos || ( pos = string ) );
    *string = '\0';
    
    return ( pos );

  } /* Read_STG_Name() */


/*****************************************************************************/
/* Lesen eines Stellenknotens.                                               */
/*                                                             Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static bool Read_STG_Node( FILE   * file,
                           char   * string,
                           LMPNet net,
                           bool   * endofline )

#else

static bool Read_STG_Node( file, string, net, endofline )

  FILE   * file;
  char   * string;
  LMPNet net;
  bool   * endofline;

#endif

  {

    register char       ch,
                        ** nptr;
    register Cardinal32 count;
    char                * end;
    unsigned short int  index;
    Transition          * adr;

    ch = *( end = Read_STG_Name( file, string, NAMESIZE, endofline ) );
    *end = '\0';
    for ( nptr = CP_TrNamen, count = 0 ;
                 ( ++count <= CP_AnzTrNamen ) && strcmp( string, *++nptr ) ; )
      /* skip */;
    *end = ch;
    if ( count <= CP_AnzTrNamen )
      {
        if ( ( index = ch == '+'? 0 : ch == '-'? 1 : ch == '~'? 2 : 3 ) == 3 )
          {
            char msg[200];
            sprintf( msg,
            "CP_read_STG(): transition name without trailing +/-/~ found (%s)",
                                                                      string );
printf( "%s\n", msg );
/*
            NNError( msg );
*/
          } /* if */
        if ( *( end + 1 ) == '/' )
          {
            for ( nptr = CP_TrExtra, count = 0 ;
                 ( ++count <= CP_AnzTrExtra ) && strcmp( string, *++nptr ) ; )
              /* skip */;
            if ( count <= CP_AnzTrExtra )
              {
                CP_trlast = CP_TransExtraArray[ count ];
              } /* if */
            else if ( ( CP_AnzTrExtra = count ) >= CP_MaxTrExtra )
              {
                CP_TrExtra = (char **) Realloc( (char *) CP_TrExtra,
                        ( CP_MaxTrExtra += NAMES_OFFSET ) * sizeof( char * ) );
                CP_TransExtraArray = (Transition *) Realloc( (char *)
                    CP_TransExtraArray, CP_MaxTrExtra * sizeof( Transition ) );
                CP_TrExtra[ CP_AnzTrExtra ] = Strdup( string );
                CP_TransExtraArray[ CP_AnzTrExtra ] =
                             fast_insert_transition( net, string, &CP_trlast );
              } /* else if */
            else
              {
                CP_TrExtra[ CP_AnzTrExtra ] = Strdup( string );
                CP_TransExtraArray[ CP_AnzTrExtra ] =
                             fast_insert_transition( net, string, &CP_trlast );
              } /* else */
          } /* if */
        else if ( ! ( CP_trlast = *( adr = CP_TransArray + count*3 + index ) ) )
          {
            *adr = fast_insert_transition( net, string, &CP_trlast );
          } /* else if */
        return ( True );
      } /* if */
    else
      {
        if ( ! ( net->Places && ( CP_pllast = find_P( net, string ) ) ) )
          {
            fast_insert_place( net, string, 0, &CP_pllast );
          } /* if */
        return ( False );
      } /* else */

  } /* Read_STG_Node() */


/*****************************************************************************/
/***************************** read_STG **************************************/
/*****************************************************************************/
/* Neu eingefuegt                      18.08.1995 - 31.01.1996 Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LMPNet read_STG( char * STGfilename )

#else

LMPNet read_STG( STGfilename )

  char * STGfilename;

#endif

  {

    Place               * PlArray,
                        pl;
    char                ** nptr;
    char                cmd[513],
                        name[NAMESIZE+1],
                        plname[NAMESIZE+1],
                        trname[NAMESIZE+1],
                        guard[GUARDSIZE+1];
    register FILE       * STGfile;
    register LMPNet     net;
    register Cardinal32 AnzPlNamen,
                        MaxPlNamen,
                        MaxTrNamen,
                        count;
    Transition          tr1,
                        tr2;
    STEdge              stlast = NULL;
    TSEdge              tslast = NULL;
    bool                endofline;
    Cardinal32          placecounter = 0;

    CP_trlast = NULL;
    CP_pllast = NULL;
    NNinput_file = STGfilename;
    NNinput_line = 1;
    PlArray = (Place *) Malloc( ( count = MaxPlNamen = MaxTrNamen
                           = CP_MaxTrExtra = NAMES_START ) * sizeof( Place ) );
    CP_TrNamen = (char **) Malloc( count * sizeof( char * ) );
    CP_TrExtra = (char **) Malloc( count * sizeof( char * ) );
    CP_TransExtraArray = (Transition *) Malloc( count * sizeof( Transition ) );
    AnzPlNamen = CP_AnzTrNamen = CP_AnzTrExtra = 0;
    while ( --count )
      {
        PlArray[ count ] = (Place) NULL;
      } /* while */

    /* Oeffne STG-Datei */
    if ( ! ( STGfile = fopen( STGfilename, "r" ) ) )
      {
        fprintf( stderr,
               "ERROR: CM.read_STG(): could not open file '%s' for reading.\n",
                 STGfilename );
        exit ( 4 );
      } /* if */

    net = empty_net();
    net->NumComponents = 1;

    while ( True )
      {
        Read_STG_Name( STGfile, cmd, 512, &endofline );
        if ( *cmd != '.' )
          {
            sprintf( guard,
                "read_STG(): command with prefix '.' expected, but '%s' found",
                                                                         cmd );
            NNError( guard );
            return ( NULL );
          } /* if */
        if ( ! strcmp( cmd, ".end" ) )
          {
            break;
          } /* if */
        else if ( ! strcmp( cmd, ".name" ) )
          {
            Read_STG_Name( STGfile, name, NAMESIZE, &endofline );
          } /* else if */
        else if ( ! strcmp( cmd, ".model" ) )
          {
            Read_STG_Name( STGfile, name, NAMESIZE, &endofline );
          } /* else if */
        else if ( ! (    strcmp( cmd, ".dummy" )
                      && strcmp( cmd, ".internal" )
                      && strcmp( cmd, ".inputs" )
                      && strcmp( cmd, ".outputs" ) ) )
          {
            do {
                 Read_STG_Name( STGfile, name, NAMESIZE, &endofline );
                 for ( nptr = CP_TrNamen, count = 0 ;
                    ( ++count <= CP_AnzTrNamen ) && strcmp( name, *++nptr ) ; )
                   /* skip */;
                 if ( count <= CP_AnzTrNamen )
                   {
                     sprintf( guard, "transition '%s' multiply used", name );
                     NNError( guard );
                   } /* if */
                 if ( ( CP_AnzTrNamen = count ) >= MaxTrNamen )
                   {
                     CP_TrNamen = (char **) Realloc( (char *) CP_TrNamen,
                           ( MaxTrNamen += NAMES_OFFSET ) * sizeof( char * ) );
                   } /* if */
                 CP_TrNamen[ CP_AnzTrNamen ] = Strdup( name );
               } while ( ! endofline );
          } /* else if */
        else if ( ! strcmp( cmd, ".graph" ) )
          {
            CP_TransArray = (Transition *) Malloc( 3 * ( CP_AnzTrNamen + 1 )
                                                      * sizeof( Transition ) );
            while ( ! ( feof( STGfile ) || TestSymbol( STGfile, '.' ) ) )
              {
                if ( TestSymbol( STGfile, '\n' ) )
                  {
                    continue;
                  } /* if */
                if ( Read_STG_Node( STGfile, name, net, &endofline ) )
                  {
                    tr1 = CP_trlast;
                    strcpy( trname, name );
                    if ( endofline )
                      {
                        NNError( "CP_read_STG(): missing destination name" );
                      } /* if */
                    while ( ! endofline )
                      {
                        if ( Read_STG_Node( STGfile, name, net, &endofline ) )
                          {
                            tr2 = CP_trlast;
                            sprintf( plname, "zzzplace%u",
                                                              ++placecounter );
                            CP_pllast = net->LastPlace;
                            fast_insert_place( net, plname, 0,
                                                                  &CP_pllast );
                            CP_trlast = tr1;
                            fast_insert_TP_con( net, plname, trname, 1, 
                                             &CP_pllast, &CP_trlast, &tslast );
                            CP_trlast = tr2;
                            fast_insert_PT_con( net, plname, name, 1,
                                             &CP_pllast, &CP_trlast, &stlast );
                          } /* if */
                        else
                          {
                            CP_trlast = tr1;
                            fast_insert_TP_con( net, name, trname, 1, 
                                             &CP_pllast, &CP_trlast, &tslast );
                          } /* else */
                      } /* while */;
                  } /* if */
                else
                  {
                    /* Place name found */
                    strcpy( plname, name );
                    if ( endofline )
                      {
                        NNError( "CP_read_STG(): missing destination name" );
                      } /* if */
                    while ( ! endofline )
                      {
                        if ( ! Read_STG_Node( STGfile, name, net, &endofline ) )
                          {
                            NNError( "CP_read_STG(): transition name expected" );
                          } /* if */
                        fast_insert_PT_con( net, plname, name, 1,
                                             &CP_pllast, &CP_trlast, &stlast );
                      } /* while */;
                  } /* else */
              } /* while */
            ( feof( STGfile ) || ungetc( '.', STGfile ) );
            Free( CP_TransArray );
          } /* else if */
        else if ( ! strcmp( cmd, ".marking" ) )
          {
            if ( ! TestSymbol( STGfile, '{' ) )
              {
                NNError( "read_STG(): '{' expected" );
              } /* if */
            while ( ! TestSymbol( STGfile, '}' ) )
              {
                if ( TestSymbol( STGfile, '<' ) )
                  {
                    if ( ! Read_STG_Node( STGfile, name, net, &endofline ) )
                      {
                        NNError( "CP_read_STG(): transition name expected" );
                      } /* if */
                    tr1 = CP_trlast;
                    ReadSymbol( STGfile, ',' );
                    if ( ! Read_STG_Node( STGfile, name, net, &endofline ) )
                      {
                        NNError( "CP_read_STG(): transition name expected" );
                      } /* if */
                    ReadSymbol( STGfile, '>' );
                    for ( tslast = tr1->postset ; tslast ;
                                                   tslast = tslast->nextplace )
                      {
                        if (    tslast->place
                             && ! tslast->place->preset->nexttrans
                             && tslast->place->postset
                             && ! tslast->place->postset->nexttrans
                             && tslast->place->postset->trans == CP_trlast )
                          {
                            tslast->place->marks++;
                            break;
                          } /* if */
                      } /* for */
                  } /* if */
                else
                  {
                    if ( Read_STG_Node( STGfile, name, net, &endofline ) )
                      {
                        NNError( "CP_read_STG(): place name expected" );
                      } /* if */
                    CP_pllast->marks++;
                  } /* else */
              } /* while */
          } /* else if */
        else if ( ! strcmp( cmd, ".capacity" ) )
          {
            while ( ! TestSymbol( STGfile, '.' ) )
              {
                pl = NULL;
                if ( TestSymbol( STGfile, '<' ) )
                  {
                    if ( ! Read_STG_Node( STGfile, name, net, &endofline ) )
                      {
                        NNError( "CP_read_STG(): transition name expected" );
                      } /* if */
                    tr1 = CP_trlast;
                    ReadSymbol( STGfile, ',' );
                    if ( ! Read_STG_Node( STGfile, name, net, &endofline ) )
                      {
                        NNError( "CP_read_STG(): transition name expected" );
                      } /* if */
                    ReadSymbol( STGfile, '>' );
                    for ( tslast = tr1->postset ; tslast ;
                                                   tslast = tslast->nextplace )
                      {
                        if (    tslast->place
                             && ! tslast->place->preset->nexttrans
                             && tslast->place->postset
                             && ! tslast->place->postset->nexttrans
                             && tslast->place->postset->trans == CP_trlast )
                          {
                            pl = tslast->place;
                            break;
                          } /* if */
                      } /* for */
                  } /* if */
                else
                  {
                    if ( Read_STG_Node( STGfile, name, net, &endofline ) )
                      {
                        NNError( "CP_read_STG(): place name expected" );
                      } /* if */
                    pl = CP_pllast;
                  } /* else */
                ReadSymbol( STGfile, '=' );
                count = ReadNumber( STGfile );
                if ( pl )
                  {
/*
                    pl->capacity = count;
*/
                  } /* if */
              } /* while */
          } /* else if */
        else
          {
            sprintf( name, "read_STG(): unknown keyword '%s'", cmd );
            NNError( name );
            return ( NULL );
          } /* else */
      } /* while */

    Free( PlArray );
    for ( nptr = CP_TrNamen, count = 0 ; ++count <= CP_AnzTrNamen ;
                                                              Free( *++nptr ) )
      /* skip */;
    Free( CP_TrNamen );
    for ( nptr = CP_TrExtra, count = 0 ; ++count <= CP_AnzTrExtra ;
                                                              Free( *++nptr ) )
      /* skip */;
    Free( CP_TrExtra );
    Free( CP_TransArray );
    Free( CP_TransExtraArray );

    fclose( STGfile );

    return( net );

  } /* read_STG() */


/*****************************************************************************/
/***************************** write_STG *************************************/
/*****************************************************************************/
/*                                                  18.08.1995 Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void write_STG( LMPNet net, char * STGfilename )

#else

void write_STG( net, STGfilename )
  /* Gibt die mit net bezeichnete Matrix als Inzidenzmatrix nach 'file' aus  */

  LMPNet net;
  char   * STGfilename;

#endif

  {

    register Place      pl;
    register Transition tr;
    register STEdge     st;
    register TSEdge     ts;
    Cardinal32          count,
                        x1,
                        nx;
    Coords              co;
    char                * ptr;
    FILE                * file;

    if ( ! ( file = fopen( STGfilename, "w" ) ) )
      {
        NNError( "Could not open file for writing" );
      } /* if */
    if ( ! CM_storeCoordinates )
      {
        NNError( "Internal Error: 'CM_storeCoordinates' not set" );
      } /* if */

    f( "STG\n" );
    fx( "%s\n", CM_nettype );
    f( "FORMAT_N\n" );
    f( "DPL s7n10@-9t2\n" );
    f( "DTR s7n10@-9t2\n" );
    f( "DPT w1t1\n" );

    f( "PL\n" );
    for ( count = 0, pl = net->Places ; pl ; pl = pl->nextplace )
      {
        pl->numname = ++count;
        fx( "\"%s\"", pl->name );
        co = (Coords) pl->inscript;
        fxy( "%u@%u", co->x, co->y );
        if ( pl->label && ( ! strcmp( CM_nettype, "PetriBox" )
                                         || ! strcmp( CM_nettype, "HLNet" ) ) )
          {
            fx( "%c", ( pl->label == ENTRY ? 'e'
                                           : pl->label == EXIT ? 'x' : ' ' ) );
          } /* if */
        else if ( pl->marks )
          {
            fx( "M%u", pl->marks );
          } /* else if */
        if ( ( co = co->next ) && ( co = co->next ) )
          {
            ( *( ptr = (char *) co->x ) && fx( "b\"%s\"", ptr ) );
            ( *( ptr = (char *) co->y ) && fx( "R\"%s\"", ptr ) );
            if ( co = co->next )
              {
                ( *( ptr = (char *) co->x ) && fx( "Z\"%s\"", ptr ) );
                ( *( ptr = (char *) co->y ) && fx( "z\"%s\"", ptr ) );
                if ( co = co->next )
                  {
                    ( ( x1 = (Cardinal32) co->x ) && fx( "m%u", x1 ) );
                  } /* if */
              } /* if */
          } /* if */
        f( "\n" );
      } /* for ( pl ... ) */

    f( "TR\n" );
    for ( count = 0, tr = net->Transitions ; tr ; tr = tr->nexttrans )
      {
        tr->numname = ++count;
        fx("\"%s\"",tr->name);
        co = (Coords) tr->inscript;
        fxy( "%u@%u", co->x, co->y );
        if ( tr->label && ( ! strcmp( CM_nettype, "PetriBox" )
                                         || ! strcmp( CM_nettype, "HLNet" ) ) )
          {
            fx( "%s", "r" + ! ( tr->label && ( *tr->label == 1 ) ) );
          } /* if */
        if ( ( co = co->next ) && ( co = co->next ) )
          {
            ( *( ptr = (char *) co->x ) && fx( "i\"%s\"", ptr ) );
            ( *( ptr = (char *) co->y ) && fx( "b\"%s\"", ptr ) );
            if ( co = co->next )
              {
                ( *( ptr = (char *) co->x ) && fx( "g\"%s\"", ptr ) );
                ( *( ptr = (char *) co->y ) && fx( "R\"%s\"", ptr ) );
              } /* if */
          } /* if */
        f( "\n" );
      } /* for ( tr ... ) */
                                             /* Liste der TS-Kanten ausgeben */
    f( "TP\n" );
    for ( tr = net->Transitions ; tr ; tr = tr->nexttrans )
      {
        for ( ts = tr->postset ; ts ; ts = ts->nextplace )
          {
            pl = ts->place;
            fxy( "%u<%u", tr->numname, pl->numname );
            co = (Coords) ts->weight;
            if ( co && ( co = co->next ) )
              {
                ( *( ptr = (char *) co->x ) && fx( "p\"%s\"", ptr ) );
                ( ( nx = (Cardinal32) co->y ) && fx( "v%u", nx ) );
                if ( co = co->next )
                  {
                    ( ( ( nx = (Cardinal32) co->x )>1 ) && fx( "w%u", nx ) );
                  } /* if */
              } /* if */
            f( "\n" );
          } /* for */
      } /* for */

    f( "PT\n" );
    for ( pl = net->Places ; pl ; pl = pl->nextplace )
      {
        for ( st = pl->postset ; st ; st = st->nexttrans )
          {
            tr = st->trans;
            fxy( "%u>%u", pl->numname, tr->numname );
            co = (Coords) st->weight;
            if ( co && ( co = co->next ) )
              {
                ( *( ptr = (char *) co->x ) && fx( "p\"%s\"", ptr ) );
                ( ( nx = (Cardinal32) co->y ) && fx( "v%u", nx ) );
                if ( co = co->next )
                  {
                    ( ( ( nx = (Cardinal32) co->x )>1 ) && fx( "w%u", nx ) );
                  } /* if */
              } /* if */
            f( "\n" );
          } /* for */
      } /* for */

    f( "TX\n" );
    f( "\n" );

    fclose( file );

  } /* write_STG() */


/*****************************************************************************/
/* Einlesen eines Namens aus einer Textdatei.                                */
/*                                                             Stefan Roemer */
/*****************************************************************************/
 
#if defined(__STDC__) || defined(__cplusplus)
 
static unsigned char ReadFSAName( FILE * file, char * string, int maxlen )
 
#else
 
static unsigned char ReadFSAName( file, string, maxlen )
 
  FILE * file;
  char * string;
  int  maxlen;
 
#endif
 
  {
 
    register unsigned char len = 1;
 
    if ( isalnum( *string = ReadWhiteSpace( file ) ) )
      {
        while ( ( isalnum( *++string = getc( file ) ) || ( *string == '-' )
               || ( *string == '_' ) ) && ( (int) ++len < maxlen ) )
          skip;
        if ( len == maxlen )
          {
            NNError( "ReadFSAName(): identifier string size exceeded" );
          } /* if */
      } /* if */
    else if ( ! feof( file ) )
      {
        NNError( "ReadFSAName(): identifier expected" );
      } /* else if */
    ungetc( *string, file );
    *string = '\0';
 
    return ( len );
 
  } /* ReadFSAName() */
 

/*****************************************************************************/
/* Lesen einer Zahl ( im Dezimalsystem ) aus einer Textdatei.                */
/*                                                             Stefan Roemer */
/*****************************************************************************/
 
#if defined(__STDC__) || defined(__cplusplus)
 
static Cardinal32 ReadFSANumber( FILE * file )
 
#else
 
static Cardinal32 ReadFSANumber( file )
 
  FILE * file;
 
#endif
 
  {
 
    register Cardinal32 number;
    register char       digit = '\0';
 
    while (    ! feof( file )
            && ( ( ( digit = getc( file ) ) == ' ' ) || ( digit == '\t' ) ) )
      skip;
    if ( isdigit( digit ) )
      {
        number = digit - '0';
        while ( isdigit( digit = getc( file ) ) )
          {
            number = number * 10 + digit - '0';
          } /* while */
        if ( digit != ' ' )
          {
            ungetc( digit, file );
          } /* if */
        return ( number );
      } /* if */
    else if ( digit == '\n' )
      {
        NNinput_line++;
        return ( 0 );
      } /* else if */
    else
      {
        NNError( "ReadFSANumber(): digit expected" );
      } /* else */

    return ( 0 );
 
  } /* ReadFSANumber() */
 

/*****************************************************************************/
/***************************** read_FSA **************************************/
/*****************************************************************************/
/* Neu eingefuegt                      04.09.1995 - 04.09.1995 Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LMPNet read_FSA( char * FSAfilename )

#else

LMPNet read_FSA( FSAfilename )

  char * FSAfilename;

#endif

  {

    Place               pl,
                        * PlArray;
    register Transition tr,
                        tr2;
    Transition          lasttr,
                        * trarray,
                        * trarrayptr;
    char                cmd[513],
                        name[NAMESIZE+1],
                        fsaname[NAMESIZE+1],
                        numericname[50];
    register FILE       * FSAfile;
    register LMPNet     net;
    LMPNet              * netarray,
                        * netarrayptr;
    char                ** netname,
                        ** identifier;
    register char       ** netnameptr,
                        ** idptr;
    register Cardinal32 AnzPlNamen,
                        MaxPlNamen,
                        MaxTrNamen,
                        NumNets,
                        MaxNets,
                        NumIdents,
                        MaxIdents,
                        id,
                        ident,
                        count,
                        count2,
                        comtype = 0,
                        * comlabel,
                        * comlabel2;
    Cardinal32          startstate;
    STEdge              stlast = NULL;
    TSEdge              tslast = NULL;

    CP_trlast = NULL;
    CP_pllast = NULL;
    NNinput_file = FSAfilename;
    NNinput_line = 1;
    PlArray = (Place *) Malloc( ( count = MaxPlNamen = MaxTrNamen
                                           = NAMES_START ) * sizeof( Place ) );
    CP_TrNamen = (char **) Malloc( count * sizeof( char * ) );
    AnzPlNamen = CP_AnzTrNamen = 0;
    while ( --count )
      {
        PlArray[ count ] = (Place) NULL;
      } /* while */

    netarrayptr = netarray = (LMPNet *) Malloc( ( MaxNets = 100 )
                                                           * sizeof( LMPNet ) );
    netname = netnameptr = (char **) Malloc( MaxNets * sizeof( char * ) );
    identifier = idptr = (char **) Malloc( ( MaxIdents = NAMES_START )
                                                           * sizeof( char * ) );
    NumNets = NumIdents = 0;

    if ( ! ( FSAfile = fopen( FSAfilename, "r" ) ) )
      {
        fprintf( stderr,
                "ERROR: CM.read_FSA(): could not open file '%s' for reading.\n",
                 FSAfilename );
        exit ( 4 );
      } /* if */

    ReadFSAName( FSAfile, cmd, 512 );
    while ( ! feof( FSAfile ) )
      {
        if ( strcmp( cmd, "FSA" )  )
          {
            NNError( "CM.read_FSA(): keyword 'FSA' expected" );
            return ( NULL );
          } /* if */
        /* Initialisiere net */
        if ( ++NumNets >= MaxNets )
          {
            netarrayptr = &( ( netarray = (LMPNet *) Realloc( (char *) netarray,
                     ( MaxNets += 100 ) * sizeof( LMPNet ) ) )[ NumNets - 1 ] );
            netnameptr = &( ( netname = (char **) Realloc( (char *) netname,
                                MaxNets * sizeof( char * ) ) )[ NumNets - 1 ] );
          } /* if */
        *( netarrayptr++ ) = net = empty_net();
        net->NumComponents = 1;
        ReadFSAName( FSAfile, fsaname, NAMESIZE );
        *( netnameptr++ ) = Strdup( fsaname );
        ReadSymbol( FSAfile, ':' );

        ReadFSAName( FSAfile, cmd, 512 );
        if ( strcmp( cmd, "STATES" )  )
          {
            NNError( "CM.read_FSA(): keyword 'STATES' expected" );
            return ( NULL );
          } /* if */
        ReadSymbol( FSAfile, '=' );
        count = ReadFSANumber( FSAfile );

        ReadFSAName( FSAfile, cmd, 512 );
        if ( strcmp( cmd, "TRANSITIONS" )  )
          {
            NNError( "CM.read_FSA(): keyword 'TRANSITIONS' expected" );
            return ( NULL );
          } /* if */
        ReadSymbol( FSAfile, '=' );
        count = ReadFSANumber( FSAfile );

        ReadFSAName( FSAfile, cmd, 512 );
        if ( strcmp( cmd, "START" )  )
          {
            NNError( "CM.read_FSA(): keyword 'START' expected" );
            return ( NULL );
          } /* if */
        ReadSymbol( FSAfile, '=' );
        startstate = ReadFSANumber( FSAfile );

        ReadFSAName( FSAfile, cmd, 512 );
        if ( strcmp( cmd, "ACCEPT" )  )
          {
            NNError( "CM.read_FSA(): keyword 'ACCEPT' expected" );
            return ( NULL );
          } /* if */
        ReadSymbol( FSAfile, '=' );
        count = ReadFSANumber( FSAfile );
        ident = 0;

        ReadFSAName( FSAfile, cmd, 512 );
        while ( ! strcmp( cmd, "STATE" ) )
          {
            count = ReadFSANumber( FSAfile );
            sprintf( numericname, "%05u%016u", NumNets, count );
            fast_insert_place( net, numericname, startstate == count,
                                                  &CP_pllast )->numname = count;
            ReadSymbol( FSAfile, ':' );
            ReadFSAName( FSAfile, cmd, 512 );
            while ( ! strcmp( cmd, "TRANS" )  )
              {
                ReadFSAName( FSAfile, cmd, 512 );
                ReadSymbol( FSAfile, '(' );
                ReadFSAName( FSAfile, name, NAMESIZE );
                count = 1;
                if ( ! strcmp( cmd, "call" ) )
                  {
                    if ( strcmp( name, fsaname ) )
                      {
                        NNError( "CM.read_FSA(): name of FSA expected" );
                      } /* if */
                    comtype = 1;
                    if ( TestSymbol( FSAfile, ';' ) )
                      {
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        count++;
                        ReadSymbol( FSAfile, '.' );
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        count++;
                        for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                          {
                            if ( ! strcmp( *idptr, name ) )
                              {
                                break;
                              } /* if */
                          } /* for */
                        if ( id == NumIdents )
                          {
                            if ( ++NumIdents >= MaxIdents )
                              {
                                idptr = &( ( identifier = (char **) Realloc(
                                               (char *) identifier, MaxIdents
                                     * sizeof( char * ) ) ) [ NumIdents - 1 ] );
                              } /* if */
                            *idptr = Strdup( name );
                          } /* if */
                        while ( TestSymbol( FSAfile, ';' ) )
                          {
                            ReadFSAName( FSAfile, name, NAMESIZE );
                            count++;
                            for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                              {
                                if ( ! strcmp( *idptr, name ) )
                                  {
                                    break;
                                  } /* if */
                              } /* for */
                            if ( id == NumIdents )
                              {
                                if ( ++NumIdents >= MaxIdents )
                                  {
                                    idptr = &( ( identifier = (char **) Realloc(
                                               (char *) identifier, MaxIdents
                                     * sizeof( char * ) ) ) [ NumIdents - 1 ] );
                                  } /* if */
                                *idptr = Strdup( name );
                              } /* if */
                          } /* while */
                      } /* if */
                  } /* if */
                else if ( ! strcmp( cmd, "accept" ) )
                  {
                    comtype = 2;
                    if ( TestSymbol( FSAfile, ';' ) )
                      {
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        if ( strcmp( name, fsaname ) )
                          {
                            NNError( "CM.read_FSA(): name of FSA expected" );
                          } /* if */
                        count++;
                        ReadSymbol( FSAfile, '.' );
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        count++;
                        for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                          {
                            if ( ! strcmp( *idptr, name ) )
                              {
                                break;
                              } /* if */
                          } /* for */
                        if ( id == NumIdents )
                          {
                            if ( ++NumIdents >= MaxIdents )
                              {
                                idptr = &( ( identifier = (char **) Realloc(
                                               (char *) identifier, MaxIdents
                                     * sizeof( char * ) ) ) [ NumIdents - 1 ] );
                              } /* if */
                            *idptr = Strdup( name );
                          } /* if */
                        while ( TestSymbol( FSAfile, ';' ) )
                          {
                            ReadFSAName( FSAfile, name, NAMESIZE );
                            count++;
                            for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                              {
                                if ( ! strcmp( *idptr, name ) )
                                  {
                                    break;
                                  } /* if */
                              } /* for */
                            if ( id == NumIdents )
                              {
                                if ( ++NumIdents >= MaxIdents )
                                  {
                                    idptr = &( ( identifier = (char **) Realloc(
                                               (char *) identifier, MaxIdents
                                     * sizeof( char * ) ) ) [ NumIdents - 1 ] );
                                  } /* if */
                                *idptr = Strdup( name );
                              } /* if */
                          } /* while */
                      } /* if */
                  } /* else if */
                else if ( ! (    strcmp( cmd, "internal" )
                              && strcmp( cmd, "exception" ) ) )
                  {
                    if ( strcmp( name, fsaname ) )
                      {
                        NNError( "CM.read_FSA(): name of FSA expected" );
                      } /* if */
                    if ( TestSymbol( FSAfile, ';' ) )
                      {
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        count++;
                        for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                          {
                            if ( ! strcmp( *idptr, name ) )
                              {
                                break;
                              } /* if */
                          } /* for */
                        if ( id == NumIdents )
                          {
                            if ( ++NumIdents >= MaxIdents )
                              {
                                idptr = &( ( identifier = (char **) Realloc(
                                               (char *) identifier, MaxIdents
                                     * sizeof( char * ) ) ) [ NumIdents - 1 ] );
                              } /* if */
                            *idptr = Strdup( name );
                          } /* if */
                      } /* if */
                    comtype = 0;
                  } /* else if */
                else if ( ! strcmp( cmd, "terminate" ) )
                  {
                    if ( strcmp( name, fsaname ) )
                      {
                        NNError( "CM.read_FSA(): name of FSA expected" );
                      } /* if */
                    comtype = 0;
                  } /* else if */
                else
                  {
                    sprintf( name,
                                "CM.read_FSA(): unexpected keyword '%s' found",
                                                                          cmd );
                    NNError( name );
                  } /* else */
                sprintf( numericname, "%05u%016u", NumNets, ++ident );
                fast_insert_transition( net, numericname, &CP_trlast )
                                                              ->numname = count;
                CP_trlast->label = (MultiSet) NumNets;
                comlabel = (Cardinal32 *) ( CP_trlast->inscript
                      = (char *) Malloc( (count + 2) * sizeof( Cardinal32 ) ) );
                *( comlabel + 1 ) = count;
                if ( *comlabel = comtype )
                  {
                    *( comlabel + comtype + 1 ) = NumNets - 1;
                  } /* if */
                else
                  {
                    *( comlabel + 2 ) = NumNets - 1;
                  } /* else */
                fast_insert_PT_con( net, CP_pllast->name,
                          CP_trlast->name, 1, &CP_pllast, &CP_trlast, &stlast );
                ReadSymbol( FSAfile, ')' );
                ReadSymbol( FSAfile, '-' );
                ReadSymbol( FSAfile, '-' );
                ReadSymbol( FSAfile, '-' );
                ReadSymbol( FSAfile, '>' );
                count = ReadFSANumber( FSAfile );

                ReadFSAName( FSAfile, cmd, 512 );
              } /* while ( ! strcmp( cmd, "TRANS" )  ) */
          } /* while ( ! strcmp( cmd, "STATE" ) ) */

      } /* while ( ! feof( FSAfile ) ) */
    fclose( FSAfile );

    NNinput_line = 1;
    comtype = 0;
    /* Open FSA file */
    if ( ! ( FSAfile = fopen( FSAfilename, "r" ) ) )
      {
        fprintf( stderr,
               "ERROR: CM.read_FSA(): could not open file '%s' for reading.\n",
                 FSAfilename );
        exit ( 4 );
      } /* if */

    netarrayptr = netarray;
    ReadFSAName( FSAfile, cmd, 512 );
    while ( ! feof( FSAfile ) )
      {
        /* Initialisiere net */
        tr = ( net = *( netarrayptr++ ) )->Transitions;
        comtype++;
        if ( strcmp( cmd, "FSA" )  )
          {
            NNError( "CM.read_FSA(): keyword 'FSA' expected" );
            return ( NULL );
          } /* if */
        ReadFSAName( FSAfile, name, NAMESIZE );
        ReadSymbol( FSAfile, ':' );

        ReadFSAName( FSAfile, cmd, 512 );
        if ( strcmp( cmd, "STATES" )  )
          {
            NNError( "CM.read_FSA(): keyword 'STATES' expected" );
            return ( NULL );
          } /* if */
        ReadSymbol( FSAfile, '=' );
        count = ReadFSANumber( FSAfile );

        ReadFSAName( FSAfile, cmd, 512 );
        if ( strcmp( cmd, "TRANSITIONS" )  )
          {
            NNError( "CM.read_FSA(): keyword 'TRANSITIONS' expected" );
            return ( NULL );
          } /* if */
        ReadSymbol( FSAfile, '=' );
        count = ReadFSANumber( FSAfile );

        ReadFSAName( FSAfile, cmd, 512 );
        if ( strcmp( cmd, "START" )  )
          {
            NNError( "CM.read_FSA(): keyword 'START' expected" );
            return ( NULL );
          } /* if */
        ReadSymbol( FSAfile, '=' );
        count = ReadFSANumber( FSAfile );

        ReadFSAName( FSAfile, cmd, 512 );
        if ( strcmp( cmd, "ACCEPT" )  )
          {
            NNError( "CM.read_FSA(): keyword 'ACCEPT' expected" );
            return ( NULL );
          } /* if */
        ReadSymbol( FSAfile, '=' );
        count = ReadFSANumber( FSAfile );

        ReadFSAName( FSAfile, cmd, 512 );
        while ( ! strcmp( cmd, "STATE" ) )
          {
            count = ReadFSANumber( FSAfile );
            ReadSymbol( FSAfile, ':' );

            ReadFSAName( FSAfile, cmd, 512 );
            while ( ! strcmp( cmd, "TRANS" )  )
              {
                ReadFSAName( FSAfile, cmd, 512 );
                ReadSymbol( FSAfile, '(' );
                ReadFSAName( FSAfile, name, NAMESIZE );
                if ( ! strcmp( cmd, "call" ) )
                  {
                    comlabel = (Cardinal32 *) tr->inscript;
                    comlabel++;
                    comlabel++;
                    if ( TestSymbol( FSAfile, ';' ) )
                      {
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        for ( netnameptr = netname, count = 0 ;
                                       count < NumNets ; netnameptr++, count++ )
                          {
                            if ( ! strcmp( *netnameptr, name ) )
                              {
                                break;
                              } /* if */
                          } /* for */
                        *++comlabel = count;
                        ReadSymbol( FSAfile, '.' );
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                          {
                            if ( ! strcmp( *idptr, name ) )
                              {
                                break;
                              } /* if */
                          } /* for */
                        *++comlabel = id;
                        while ( TestSymbol( FSAfile, ';' ) )
                          {
                            ReadFSAName( FSAfile, name, NAMESIZE );
                            for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                              {
                                if ( ! strcmp( *idptr, name ) )
                                  {
                                    break;
                                  } /* if */
                              } /* for */
                            *++comlabel = id;
                          } /* while */
                      } /* if */
                  } /* if */
                else if ( ! strcmp( cmd, "accept" ) )
                  {
                    comlabel = (Cardinal32 *) tr->inscript;
                    comlabel++;
                    for ( netnameptr = netname, count = 0 ; count < NumNets ;
                                                         netnameptr++, count++ )
                      {
                        if ( ! strcmp( *netnameptr, name ) )
                          {
                            break;
                          } /* if */
                      } /* for */
                    *++comlabel = count;
                    comlabel++;
                    if ( TestSymbol( FSAfile, ';' ) )
                      {
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        ReadSymbol( FSAfile, '.' );
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                          {
                            if ( ! strcmp( *idptr, name ) )
                              {
                                break;
                              } /* if */
                          } /* for */
                        *++comlabel = id;
                        while ( TestSymbol( FSAfile, ';' ) )
                          {
                            ReadFSAName( FSAfile, name, NAMESIZE );
                            for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                              {
                                if ( ! strcmp( *idptr, name ) )
                                  {
                                    break;
                                  } /* if */
                              } /* for */
                            *++comlabel = id;
                          } /* while */
                      } /* if */
                  } /* else if */
                else if ( ! (    strcmp( cmd, "internal" )
                              && strcmp( cmd, "exception" ) ) )
                  {
                    if ( TestSymbol( FSAfile, ';' ) )
                      {
                        comlabel = (Cardinal32 *) tr->inscript;
                        comlabel++;
                        comlabel++;
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                          {
                            if ( ! strcmp( *idptr, name ) )
                              {
                                break;
                              } /* if */
                          } /* for */
                        *++comlabel = id;
                      } /* if */
                  } /* else if */
                else if ( ! strcmp( cmd, "terminate" ) )
                  {
                  } /* else if */
                else
                  {
                    sprintf( name,
                                "CM.read_FSA(): unexpected keyword '%s' found",
                                                                          cmd );
                    NNError( name );
                  } /* else */
                ReadSymbol( FSAfile, ')' );
                ReadSymbol( FSAfile, '-' );
                ReadSymbol( FSAfile, '-' );
                ReadSymbol( FSAfile, '-' );
                ReadSymbol( FSAfile, '>' );
                count = ReadFSANumber( FSAfile );
                sprintf( numericname, "%05u%016u", comtype, count );
                CP_pllast = NULL;
                fast_insert_TP_con( net, numericname, tr->name, 1, 
                                              &CP_pllast, &CP_trlast, &tslast );
                tr = tr->nexttrans;
                ReadFSAName( FSAfile, cmd, 512 );
              } /* while ( ! strcmp( cmd, "TRANS" )  ) */
          } /* while ( ! strcmp( cmd, "STATE" ) ) */
      } /* while ( ! feof( FSAfile ) ) */
    fclose( FSAfile );

    ident = 0;
    *( trarrayptr = trarray = (Transition *) Malloc( ( NumNets + 1 )
                          * sizeof( Transition ) ) ) = (*netarray)->Transitions;
    for ( netarrayptr = netarray + 1, count = NumNets - 1 ; count-- ;
                                                                 netarrayptr++ )
      {
        *++trarrayptr = ( net = *netarrayptr )->Transitions;
        (*netarray)->NumPlaces += net->NumPlaces;
        (*netarray)->NumTransitions += net->NumTransitions;
        if ( ! (*netarray)->Places )
          {
            (*netarray)->Places = net->Places;
            (*netarray)->LastPlace = net->LastPlace;
          } /* if */
        else if ( (*netarray)->LastPlace->nextplace = net->Places )
          {
            net->Places->prevplace = (*netarray)->LastPlace;
            (*netarray)->LastPlace = net->LastPlace;
          } /* else if */
        if ( ! (*netarray)->Transitions )
          {
            (*netarray)->Transitions = net->Transitions;
            (*netarray)->LastTransition = net->LastTransition;
          } /* if */
        else if ( (*netarray)->LastTransition->nexttrans = net->Transitions )
          {
            net->Transitions->prevtrans = (*netarray)->LastTransition;
            (*netarray)->LastTransition = net->LastTransition;
          } /* else if */
        net->NumPlaces = net->NumTransitions = 0;
        net->Places = net->LastPlace = NULL;
        net->Transitions = net->LastTransition = NULL;
      } /* for */

    net = *netarray;
    lasttr = net->LastTransition;
    for ( tr = net->Transitions ; tr != lasttr ; tr = tr->nexttrans )
      {
        if (    *( comlabel = (Cardinal32 *) tr->inscript )
             && (    ( ( *comlabel == 1 ) && ( comlabel[2] < comlabel[3] ) )
                  || ( ( *comlabel == 2 ) && ( comlabel[2] > comlabel[3] ) ) ) )
          {
            count = comlabel[ 4 - *comlabel ];
            for ( tr2 = trarray[ count ] ; tr2 && ( tr2 != trarray[ count+1 ] ) ;
                                                          tr2 = tr2->nexttrans )
              {
                if (    *( comlabel2 = (Cardinal32 *) tr2->inscript )
                     && ( *comlabel != *comlabel2 ) )
                  {
                    if ( ( count2 = comlabel[1] ) == comlabel2[1] )
                      {
                        while ( count2 && ( comlabel[ count2 + 1 ]
                                                   == comlabel2[ count2 + 1 ] ) )
                          {
                            count2--;
                          } /* while */
                        if ( ! count2 )
                          {
                            sprintf( numericname, "%05u%016u", NumNets + 1,
                                                                      ++ident );
                            CP_trlast = NULL;
                            fast_insert_transition( net, numericname,
                                                                   &CP_trlast );
                            if ( ident == 1 )
                              {
                                trarray[ NumNets ] = CP_trlast;
                              } /* if */
                            CP_pllast = NULL;
                            CP_trlast = NULL;
                            fast_insert_PT_con( net,
                                      tr->preset->place->name, numericname, 1,
                                              &CP_pllast, &CP_trlast, &stlast );
                            CP_pllast = NULL;
                            CP_trlast = NULL;
                            fast_insert_PT_con( net,
                                     tr2->preset->place->name, numericname, 1,
                                              &CP_pllast, &CP_trlast, &stlast );
                            CP_pllast = NULL;
                            CP_trlast = NULL;
                            fast_insert_TP_con( net, 
                                     tr->postset->place->name, numericname, 1,
                                              &CP_pllast, &CP_trlast, &tslast );
                            CP_pllast = NULL;
                            CP_trlast = NULL;
                            fast_insert_TP_con( net, 
                                    tr2->postset->place->name, numericname, 1,
                                              &CP_pllast, &CP_trlast, &tslast );
                          } /* if */
                      }
                  } /* if */
              } /* for ( tr2 ... ) */
          } /* if */
      } /* for ( tr ... ) */

    lasttr = trarray[ NumNets ];
    for ( tr = net->Transitions ; tr != lasttr ; skip )
      {
        if ( *( (Cardinal32 *) tr->inscript ) )
          {
            Free( tr->inscript );
            tr->inscript = NULL;
            tr = delete_transition_direct( net, tr );
          } /* if */
        else
          {
            tr = tr->nexttrans;
          } /* else */
      } /* for ( tr ... ) */
    for ( pl = net->Places, count = 0 ; pl ; pl = pl->nextplace )
      {
        pl->numname = ++count;
      } /* for */
    for ( tr = net->Transitions, count = 0 ; tr ; tr = tr->nexttrans )
      {
        tr->numname = ++count;
      } /* for */

    Free( *netname );
    for ( netarrayptr = netarray + 1, netnameptr = netname, count = NumNets ;
                      --count ; Free( *netnameptr ), remove_net( netarrayptr ) )
      skip;
    Free( netarray );
    Free( netname );
    for ( idptr = identifier, count = NumIdents ; count-- ; Free( *idptr ) )
      skip;
    Free( identifier );
    Free( trarray );

    return( net );

  } /* read_FSA() */


/*****************************************************************************/
/***************************** write_FSA *************************************/
/*****************************************************************************/
/*                                                  18.08.1995 Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void write_FSA( LMPNet net, char * FSAfilename )

#else

void write_FSA( net, FSAfilename )

  LMPNet net;
  char   * FSAfilename;

#endif

  {

    register Place      pl;
    register Transition tr;
    register STEdge     st;
    register TSEdge     ts;
    Cardinal32          count;
    FILE                * file;

    if ( ! ( file = fopen( FSAfilename, "w" ) ) )
      {
        NNError( "Could not open file for writing" );
      } /* if */

    f( "PEP\n" );
    f( "PTNet\n" );
    f( "FORMAT_N\n" );
    f( "DPL s7n10@-9t2\n" );
    f( "DTR s7n10@-9t2\n" );
    f( "DPT w1t1\n" );

    f( "PL\n" );
    for ( count = 0, pl = net->Places ; pl ; pl = pl->nextplace )
      {
        pl->numname = ++count;
        fx( "\"%s\"", pl->name );
        fxy( "%u@%u", 20 + count * 10, 20 + count * 10 );
        if ( pl->marks )
          {
            fx( "M%u", pl->marks );
          } /* else if */
        f( "\n" );
      } /* for ( pl ... ) */
    f( "TR\n" );

    for ( count = 0, tr = net->Transitions ; tr ; tr = tr->nexttrans )
      {
        tr->numname = ++count;
        fx("\"%s\"",tr->name);
        fxy( "%u@%u", 160 + count * 10, 20 + count * 10 );
        f( "\n" );
      } /* for ( tr ... ) */

    f( "TP\n" );
    for ( tr = net->Transitions ; tr ; tr = tr->nexttrans )
      {
        for ( ts = tr->postset ; ts ; ts = ts->nextplace )
          {
            pl = ts->place;
            fxy( "%u<%u\n", tr->numname, pl->numname );
          } /* for */
      } /* for */

    f( "PT\n" );
    for ( pl = net->Places ; pl ; pl = pl->nextplace )
      {
        for ( st = pl->postset ; st ; st = st->nexttrans )
          {
            tr = st->trans;
            fxy( "%u>%u\n", pl->numname, tr->numname );
          } /* for */
      } /* for */

    f( "TX\n" );
    f( "\n" );

    fclose( file );

  } /* write_FSA() */


/*****************************************************************************/
/* END OF FILE convertPEP.c                                                  */
/*****************************************************************************/

