/*****************************************************************************/
/* FILE memlib.h                                                             */
/* Burkhard Graves, 10.10.93                                                 */
/* Lars Jenner, Juni 1994                                                    */
/* Stephan Merkel, Feb. 95                                                   */
/* Martin Ackermann, Winter 1996/97                                          */
/*****************************************************************************/

#ifndef __MEMLIB_H__
#define __MEMLIB_H__

#include <stdlib.h>
#include <limits.h>

/*****************************************************************************/
/* Global defines                                                            */
/*****************************************************************************/

#define skip

#if defined( __STDC__ ) || defined( __cplusplus )
#define PROTO(x) x
#else
#define PROTO(x) ()
#endif

#ifndef CLOCKS_PER_SEC
#define CLOCKS_PER_SEC 1000000
#define OLD_SUNOS
#endif

#define INF 0
	/* information */
#define WAR 1
	/* warning */
#define ERR 2
	/* error */
#define FAT 3
	/* fatal */

/***************************/
/* Global type definitions */
/***************************/

#undef False
#undef True
#ifndef __cplusplus
typedef enum { False = 0, True } bool;
#else
#define False false
#define True true
#endif
typedef short int Integer;		/* 16bit */
typedef unsigned short int Cardinal;	/* 16bit */
#ifdef ATARI
  typedef unsigned long int Cardinal32;
#else
  typedef unsigned int Cardinal32;	/* 32bit */
#endif

/* Format string for defined type Cardinal32 with 'printf()' */
#define FORMAT_CARD "%hu"

/************************************/
/* Special values in defined ranges */
/************************************/

#define MAX_INT 32767
#define NONE (MAX_INT-0)  /* Marks a missing init */
#define  DOT (MAX_INT-1)
#define   TT (MAX_INT-2)
#define   FF (MAX_INT-3)
#define MAX_POS (FF-1)    /* maximal allowed positive value */
#define MIN_NEG (-10000)  /* least allowed negative value */

/*********************************************************************/

#define NAMESIZE 4096
#define MEANINGSIZE 4096
#define GUARDSIZE 8192
#define REFSIZE 8192
#define OMEGA INT_MAX
#define NAMES_START 2000
#define NAMES_OFFSET 1000

/*********************************************************************/
#define f( STRING ) fprintf( file, STRING )
#define fx( STRING, X ) fprintf( file, STRING, X )
#define fxy( STRING, X, Y ) fprintf( file, STRING, X, Y )
#define fxyz( STRING, X, Y, Z ) fprintf( file, STRING, X, Y, Z )
#define fxyxy( STRING, X1, Y1, X2, Y2 ) fprintf( file, STRING, X1,Y1,X2,Y2 )
#define fxyxyz( STRING, X1,Y1,X2,Y2,Z ) fprintf( file, STRING,X1,Y1,X2,Y2,Z )

/*********************************************************************/
/* Makros zur Bestimmung von Minimum bzw. Maximum zweier beliebiger, */
/* jedoch vergleichbarer Werte irgendeines Aufzaehltyps.             */
/*********************************************************************/

#define Min(a, b) ((a)<(b) ? (a) : (b))
#define Max(a, b) ((a)>(b) ? (a) : (b))

/*****************************************************************************/
/*                          Allokationsfunktionen                            */
/*****************************************************************************/

#ifdef Free
#undef Free
#endif
#define Free( _X_ ) (void) free( (char *) (_X_) )
#define MYfree(ptr) free( ptr )

#define Malloc MYmalloc
#define Realloc MYrealloc
#define Strdup MYstrdup

void  *MYmalloc PROTO (( size_t size ));
void *MYrealloc PROTO (( void *ptr, size_t size ));
char  *MYstrdup PROTO (( char *str ));

#endif /* __MEMLIB_H__ */

/******************************************************************************/
/* END OF FILE memlib.h                                                     */
/******************************************************************************/
