/*****************************************************************************/
/* readlib.h								     */
/*****************************************************************************/
/*				    Stefan Roemer,  06.02.1995 - 06.02.1995  */
/*				    Stefan Schwoon, April 1997		     */
/*****************************************************************************/

#ifndef __READLIB_H__
#define __READLIB_H__

#include <stdio.h>
#include "memlib.h"

/*****************************************************************************/
/* Global variables.							     */
/*****************************************************************************/

extern char *NNinput_file;        /* Name of file currently being processed. */
extern int   NNinput_line;        /* Number of current input line in file.   */

extern char *sbuf;    /* Some functions (ReadCmdToken, ReadEnclString)       */
extern int  sballoc;  /* store strings in this buffer. Memory is allocated   */
                      /* (and enlarged) automatically.                       */

extern char errbuf[];
extern void (*PEPerror)(char*); /* Function pointer variable; this is called
                                   whenever an error in a PEP file is discovered
                                   during parsing. Default behaviour is to
                                   display the error given in char* and quit,
                                   but if you want to change that, just redefine
                                   this variable. */

#define Error(msg) { sprintf msg; PEPerror(errbuf); }
#define PEPwarn(msg) fprintf(stderr,msg)

/*****************************************************************************/
/* Auxiliary functions for reading PEP files. 				     */
/*****************************************************************************/

char* RL_ReadEnclString(FILE *file);
char* ReadCmdToken(FILE *file);

#if defined(__STDC__) || defined(__cplusplus)
char ReadCharComment( FILE * );
#else
char ReadCharComment();
#endif

#if defined(__STDC__) || defined(__cplusplus)
void ReadNewline( FILE * );
#else
void ReadNewline();
#endif

#if defined(__STDC__) || defined(__cplusplus)
char ReadWhiteSpace( FILE * );
#else
char ReadWhiteSpace();
#endif

#if defined(__STDC__) || defined(__cplusplus)
int ReadNumber( FILE * );
#else
int ReadNumber();
#endif

#if defined(__STDC__) || defined(__cplusplus)
double ReadDouble( FILE * );
#else
double ReadDouble();
#endif

#if defined(__STDC__) || defined(__cplusplus)
unsigned char ReadAlphaName( FILE * file, char * string, int maxlen );
#else
unsigned char ReadAlphaName();
#endif

#if defined(__STDC__) || defined(__cplusplus)
void ReadCoordinates(FILE *file, int *x, int *y);
#else
void ReadCoordinates();
#endif

/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)
bool TestSymbol( FILE *, char );
#else
bool TestSymbol();
#endif

#if defined(__STDC__) || defined(__cplusplus)
bool TestDigit( FILE * );
#else
bool TestDigit();
#endif

/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)
extern void ReadSymbol( FILE *, char );
#else
extern void ReadSymbol();
#endif

/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)
extern unsigned char ReadName( FILE *, char *, int );
#else
extern unsigned char ReadName();
#endif

#endif

