/*-------------------------------------------------------------------------*/
/*--                                                                     --*/
/*--  LIST.C                                                             --*/
/*--                                                                     --*/
/*--  Routinen zur allgemeinen Verwendung                                --*/
/*--                                                                     --*/
/*--  Johannes Jaeger - Sommer 1994                                      --*/
/*--  Stefan Roemer   - 1995/1996                                        --*/
/*-------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "list.h"


#define EXITCODE 1
#define E_ERROR "\nERROR -- "

/*-------------------------------------------------------------------------*/
/*-- reserviert Speicher fuer ein TransListElem;                         --*/
/*-- der uebergebene Transitionszeiger wird im Feld trans abgelegt;      --*/
/*-- das Feld Next wird mit NULL initialisiert;                          --*/
/*-- Parameter :                                                         --*/
/*--   Transition trans : Zeiger auf eine Transition, der im Element     --*/
/*--                      eingetragen werden soll                        --*/
/*--                                                                     --*/
/*-- Rueckgabe : Zeiger vom Typ TransListPtr auf das neu erzeugte Element--*/

#if defined(__STDC__) || defined(__cplusplus)

TransListPtr NewTransListElem( Transition trans )

#else

TransListPtr NewTransListElem( trans )

  Transition trans;

#endif

  {

    register TransListPtr trptr;

    ( trptr = (TransListPtr) MYmalloc( sizeof( TransListElem ) ) )
                                                            ->trans = trans;
    trptr->Next = NULL;
    return ( trptr );

  } /* NewTransListElem() */

/*-------------------------------------------------------------------------*/
/*-- fuegt den Transitions-Zeiger trans an das Ende der Transitions-Liste--*/
/*-- tl an, falls trans noch nicht in der Liste ist;                     --*/
/*-- Parameter :                                                         --*/
/*--   TransListPtr *tl : Referenz des Zeigers auf das erste Listenele-  --*/
/*--                      ment, darf auch NULL sein                      --*/
/*--   Transition trans : Zeiger auf die in die Liste einzufuegende      --*/
/*--                      Transition                                     --*/

#if defined(__STDC__) || defined(__cplusplus)

void InsertTransition( TransListPtr *trlist, Transition trans )

#else

void InsertTransition( trlist, trans )

  TransListPtr * trlist;
  Transition   trans;

#endif

  {

    register TransListPtr trptr, trnew;
    register Cardinal32   numname;

    if ( ! trans )
      {
        return;
      } /* if */

    numname = trans->numname;

    if (    ( ! ( trptr = *trlist ) )
         || ( numname < trptr->trans->numname ) )
      {
        ( trnew = NewTransListElem( trans ) )->Next = trptr;
        *trlist = trnew;
      } /* if */
    else if ( trans != trptr->trans )
      {
        while (    ( trnew = trptr->Next )
                && ( trnew->trans->numname < numname ) )
          {
            trptr = trnew;
          } /* while */
        if ( ! ( trnew && ( trnew->trans == trans ) ) )
          {
            ( trnew = NewTransListElem( trans ) )->Next = trptr->Next;
            trptr->Next = trnew;
          } /* if */
      } /* else if */

} /* InsertTransition() */

/*-------------------------------------------------------------------------*/
/*-- loescht die uebergebene Transition trans aus der Transitionsliste   --*/
/*-- tl;                                                                 --*/
/*-- Parameter :                                                         --*/
/*--   TransListPtr *tl : Referenz auf einen Zeiger auf das erste Li-    --*/
/*--                      stenelement                                    --*/
/*--   Transition trans : der zu loeschende Transitionszeiger            --*/


#if defined(__STDC__) || defined(__cplusplus)

void DeleteTransition( TransListPtr * trlist, Transition trans )

#else

void DeleteTransition( trlist, trans )

  TransListPtr * trlist;
  Transition   trans;

#endif

  {

    register TransListPtr trptr, trdel;
    register Cardinal32   numname;

    /* enthaelt die Liste ueberhaupt Elemente? */
    if ( *trlist && trans )
      {
        if ( (*trlist)->trans == trans )
          {
            *trlist = ( trdel = *trlist )->Next;
            free( (char *) trdel );
          } /* if */
      } /* if */
    else
      {
        numname = trans->numname;
        trptr = *trlist;
        while ( trptr->Next && ( trptr->Next->trans->numname < numname ) )
          {
            trptr = trptr->Next;
          } /* while */
        if ( trptr->Next && ( trptr->Next->trans == trans ) )
          {
            trptr->Next = ( trdel = trptr->Next )->Next;
            free( (char *) trdel );
          } /* if */
      } /* else */

  } /* DeleteTransition() */

/*-------------------------------------------------------------------------*/
/*-- loescht eine Liste von TransListElems ab dem uebergebenen Zeiger auf--*/
/*-- ein TransListElem;                                                  --*/
/*-- Parameter :                                                         --*/
/*--   TransListPtr *tl : Referenz auf einen Zeiger auf ein TransListElem--*/
/*--                      ab dem die Liste geloescht werden soll         --*/

#if defined(__STDC__) || defined(__cplusplus)

void DeleteTransList( TransListPtr * trlist )

#else

void DeleteTransList( trlist )

  TransListPtr * trlist;

#endif

  {

    register TransListPtr trdel;

    while ( *trlist )
      {
        *trlist = ( trdel = *trlist )->Next;
        free( (char *) trdel );
      } /* while */

    *trlist = NULL;

  } /* DeleteTransList() */

/*-------------------------------------------------------------------------*/
/*-- prueft, ob der uebergebene Transitionszeiger trans in der ueberge-  --*/
/*-- benen Transitionenliste tr enthalten ist                            --*/
/*-- Parameter :                                                         --*/
/*--   TransListPtr tr  : Zeiger auf das erste Listenelement der zu      --*/
/*--                      durchsuchenden Liste                           --*/
/*--   Transition trans : der zu suchende Transitionszeiger              --*/
/*--                                                                     --*/
/*-- Rueckgabe : True  - trans ist in der Liste                          --*/
/*--             False - trans ist nicht in der Liste                    --*/

#if defined(__STDC__) || defined(__cplusplus)

bool InTransList( TransListPtr trlist, Transition trans )

#else

bool InTransList( trlist, trans )

  TransListPtr trlist;
  Transition   trans;

#endif

  {

    register TransListPtr trptr;
    register Cardinal32   numname;

    if ( ( trptr = trlist ) && trans )
      {
        numname = trans->numname;
        while ( trptr && ( trptr->trans->numname < numname ) )
         {
           trptr = trptr->Next;
         } /* while */

        return ( trptr && ( trptr->trans == trans ) ? True : False );
      } /* if */

    return ( False );

  } /* InTransList() */

/*-------------------------------------------------------------------------*/
/*-- reserviert Speicher fuer ein PlaceListElem;                         --*/
/*-- der uebergebene Stellenzeiger wird im Feld place abgelegt;          --*/
/*-- das Feld Next wird mit NULL initialisiert;                          --*/
/*-- Parameter :                                                         --*/
/*--   Place place : Zeiger auf eine Stelle, die im Element              --*/
/*--                 eingetragen werden soll                             --*/
/*--                                                                     --*/
/*-- Rueckgabe : Zeiger vom Typ PlaceListPtr auf das neu erzeugte Element--*/

#if defined(__STDC__) || defined(__cplusplus)

PlaceListPtr NewPlaceListElem( Place place )

#else

PlaceListPtr NewPlaceListElem( place )

  Place place;

#endif

  {

    register PlaceListPtr plptr;

    ( plptr = (PlaceListPtr) MYmalloc( sizeof( PlaceListElem ) ) )
                                                             ->place = place;
    plptr->Next = NULL;
    return ( plptr );

  } /* NewPlaceListElem() */

/*-------------------------------------------------------------------------*/
/*-- fuegt den Stellen-Zeiger place an das Ende der Stellen-Liste        --*/
/*-- pl an, falls place noch nicht in der Liste ist;                     --*/
/*-- Parameter :                                                         --*/
/*--   PlaceListPtr *pl : Referenz des Zeigers auf das erste Listenele-  --*/
/*--                      ment, darf auch NULL sein                      --*/
/*--   Place place      : Zeiger auf die in die Liste einzufuegende      --*/
/*--                      Stelle                                         --*/

#if defined(__STDC__) || defined(__cplusplus)

void InsertPlace( PlaceListPtr * pllist, Place place )

#else

void InsertPlace( pllist, place )

  PlaceListPtr * pllist;
  Place        place;

#endif

  {

    register PlaceListPtr plptr, plnew;
    register Cardinal32   numname;

    if ( ! place )
      {
        return;
      } /* if */

    numname = place->numname;

    if (    ( ! ( plptr = *pllist ) )
         || ( numname < plptr->place->numname ) )
      {
        ( plnew = NewPlaceListElem( place ) )->Next = plptr;
        *pllist = plnew;
      } /* if */
    else if ( place != plptr->place )
      {
        while (    ( plnew = plptr->Next )
                && ( plnew->place->numname < numname ) )
          {
            plptr = plnew;
          } /* while */
        if ( ! ( plnew && ( plnew->place == place ) ) )
          {
            ( plnew = NewPlaceListElem( place ) )->Next = plptr->Next;
            plptr->Next = plnew;
          } /* if */
      } /* else if */
 
  } /* InsertPlace() */

/*-------------------------------------------------------------------------*/
/*-- loescht eine Liste von PlaceListElems ab dem uebergebenen Zeiger auf--*/
/*-- ein PlaceListElem;                                                  --*/
/*-- Parameter :                                                         --*/
/*--   PlaceListPtr *pl : Referenz auf einen Zeiger auf ein PlaceListElem--*/
/*--                      ab dem die Liste geloescht werden soll         --*/


#if defined(__STDC__) || defined(__cplusplus)

void DeletePlaceList( PlaceListPtr * pllist )

#else

void DeletePlaceList( pllist )

  PlaceListPtr * pllist;

#endif

  {

    register PlaceListPtr pldel;

    while ( *pllist )
      {
        *pllist = ( pldel = *pllist )->Next;
        free( (char *) pldel );
      } /* while */
    *pllist = NULL;

  } /* DeletePlaceList() */

/*-------------------------------------------------------------------------*/
/*-- prueft, ob der uebergebene Stellenzeiger place in der ueberge-      --*/
/*-- benen Stellenliste pl enthalten ist                                 --*/
/*-- Parameter :                                                         --*/
/*--   PlaceListPtr pl  : Zeiger auf das erste Listenelement der zu      --*/
/*--                      durchsuchenden Liste                           --*/
/*--   Place      place : der zu suchende Stellenzeiger                  --*/
/*--                                                                     --*/
/*-- Rueckgabe : True  - place ist in der Liste                          --*/
/*--             False - place ist nicht in der Liste                    --*/

#if defined(__STDC__) || defined(__cplusplus)

bool InPlaceList( PlaceListPtr pllist, Place place )

#else

bool InPlaceList( pllist, place )

  PlaceListPtr pllist;
  Place        place;

#endif

  {

    register PlaceListPtr plptr;
    register Cardinal32   numname;

    if ( ( plptr = pllist ) && place )
      {
        numname = place->numname;
        while ( plptr && ( plptr->place->numname < numname ) )
         {
           plptr = plptr->Next;
         } /* while */

        return ( plptr && ( plptr->place == place ) ? True : False );
      } /* if */

    return ( False );

  } /* InPlaceList() */


/*-------------------------------------------------------------------------*/
/*-- erzeugt ein neues Listen-Element vom Typ LIST_ELEMENT, traegt       --*/
/*-- Newname im Feld name ein, initialisiert das Feld next mit NULL und  --*/
/*-- gibt einen Zeiger auf das Element zurueck                           --*/
/*-- Parameter :                                                         --*/
/*--   char *NewName : der in das neue Element einzutragende String      --*/
/*--                                                                     --*/
/*-- Rueckgabe : Zeiger vom Typ LIST_POINTER auf das neu erzeugte Element--*/

#if defined(__STDC__) || defined(__cplusplus)

LIST_POINTER NewListElem( char * NewName )

#else

LIST_POINTER NewListElem( NewName )

  char * NewName;

#endif

  {

    register LIST_POINTER list;

    ( list = (LIST_POINTER) MYmalloc( sizeof( LIST_ELEMENT ) ) )->next = NULL;
    strcpy( list->name, NewName );
    return ( list );

  } /* NewListElem() */

void DeleteNameList (LIST_POINTER list)
{
  LIST_POINTER tmp;
  while ((tmp=list))
  {
    list=list->next;
    free(tmp);
  }
}

/*-------------------------------------------------------------------------*/
/*-- fuegt ein neues Listenelement mit NewName an das Ende der Liste     --*/
/*-- lp an, falls NewName noch nicht in der Liste ist;                   --*/
/*-- Parameter :                                                         --*/
/*--   LIST_POINTER *lp : Referenz des Zeigers auf das erste Listenele-  --*/
/*--                      ment, darf auch NULL sein                      --*/
/*--   char *NewName    : der in das Feld name des neuen Elements einzu- --*/
/*--                      fuegende String                                --*/

#if defined(__STDC__) || defined(__cplusplus)

void InsertListElem( LIST_POINTER * lp, char *NewName )

#else

void InsertListElem( lp, NewName )

  LIST_POINTER * lp;
  char         * NewName;

#endif

  {

    register LIST_POINTER _lp, _tmp;

    /* falls kein neuer Name, fertig */
    if ( ! NewName )
      return;

    /* ist die Liste noch leer? */
    if ( ! *lp )
      {
        /* neues Element erzeugen, NewName eintragen */
        *lp = NewListElem( NewName );
        return;
      }
    else    /* Liste enthaelt schon Elemente */
      {
        /* Listenanfang */
        _lp = *lp;

        do
          {
            /* wenn Name schon in Liste, fertig */
            if ( ! strcmp(_lp->name, NewName) )
              return;

            /* letzten Listenelement-Zeiger ungleich NULL merken */
            _tmp = _lp;

            /* ein Listen-Element weiter */
            _lp = _lp->next;

            /* bis zum Ende der Liste */
          } while ( _lp != NULL );

        /* NewName war noch nicht in der Liste            */
        /* also: an das letzte Element ein neues anfuegen */
        /* und NewName eintragen                          */
        _tmp->next = NewListElem( NewName );
      }

  } /* InsertListElem() */

/*-------------------------------------------------------------------------*/
/*--                                                                     --*/
/*--  Ende LIST.C                                                        --*/
/*--                                                                     --*/
/*-------------------------------------------------------------------------*/

