/*-------------------------------------------------------------------------*/
/*--                                                                     --*/
/*--  LIST.H - Header-Datei fuer LIST.C                                  --*/
/*--                                                                     --*/
/*--  Datenstrukturen und Routinen zur allgemeinen Verwendung            --*/
/*--                                                                     --*/
/*--  Johannes Jaeger - Sommer 1994                                      --*/
/* 16.6.2000, CS: type Bool changed to PEP type bool                       */
/*-------------------------------------------------------------------------*/

#ifndef __LIST_H__
#define __LIST_H__

#include "neunetze.h"

/*-------------------------------------------------------------------------*/
/*-- Listenelement fuer eine einfach verkettete Liste von Transitions-   --*/
/*-- Zeigern                                                             --*/

typedef struct TransListElem_
{
  Transition             trans;            /* Zeiger auf Transition */
  struct TransListElem_  *Next;            /* Zeiger auf naechstes Element */
} TransListElem,
  *TransListPtr;

/*-------------------------------------------------------------------------*/
/*-- Listenelement fuer eine einfach verkettete Liste von Stellen-       --*/
/*-- Zeigern                                                             --*/

typedef struct PlaceListElem_
{
  Place                  place;            /* Zeiger auf Stelle */
  struct PlaceListElem_  *Next;            /* Zeiger auf naechstes Element */
} PlaceListElem,
  *PlaceListPtr;

/*-------------------------------------------------------------------------*/
/*-- Prototyp-Deklarationen                                              --*/
/*-------------------------------------------------------------------------*/
/*-- reserviert Speicher fuer ein TransListElem;                         --*/
/*-- der uebergebene Transitionszeiger wird im Feld trans abgelegt;      --*/
/*-- das Feld Next wird mit NULL initialisiert;                          --*/
/*-- Parameter :                                                         --*/
/*--   Transition trans : Zeiger auf eine Transition, der im Element     --*/
/*--                      eingetragen werden soll                        --*/
/*--                                                                     --*/
/*-- Rueckgabe : Zeiger vom Typ TransListPtr auf das neu erzeugte Element--*/

#if defined(__STDC__) || defined(__cplusplus)

extern TransListPtr NewTransListElem( Transition trans );

#else

extern TransListPtr NewTransListElem();

#endif

/*-------------------------------------------------------------------------*/
/*-- fuegt den Transitions-Zeiger trans an das Ende der Transitions-Liste--*/
/*-- tl an, falls trans noch nicht in der Liste ist;                     --*/
/*-- Parameter :                                                         --*/
/*--   TransListPtr *tl : Referenz des Zeigers auf das erste Listenele-  --*/
/*--                      ment, darf auch NULL sein                      --*/
/*--   Transition trans : Zeiger auf die in die Liste einzufuegende      --*/
/*--                      Transition                                     --*/

#if defined(__STDC__) || defined(__cplusplus)

extern void InsertTransition( TransListPtr *tl, Transition trans );

#else

extern void InsertTransition();

#endif

/*-------------------------------------------------------------------------*/
/*-- loescht die uebergebene Transition trans aus der Transitionsliste   --*/
/*-- tl;                                                                 --*/
/*-- Parameter :                                                         --*/
/*--   TransListPtr *tl : Referenz auf einen Zeiger auf das erste Li-    --*/
/*--                      stenelement                                    --*/
/*--   Transition trans : die zu loeschende Transition                   --*/


#if defined(__STDC__) || defined(__cplusplus)

extern void DeleteTransition( TransListPtr * tl, Transition trans );

#else

extern void DeleteTransition();

#endif

/*-------------------------------------------------------------------------*/
/*-- loescht eine Liste von TransListElems ab dem uebergebenen Zeiger auf--*/
/*-- ein TransListElem;                                                  --*/
/*-- Parameter :                                                         --*/
/*--   TransListPtr *tl : Referenz auf einen Zeiger auf ein TransListElem--*/
/*--                      ab dem die Liste geloescht werden soll         --*/


#if defined(__STDC__) || defined(__cplusplus)

extern void DeleteTransList( TransListPtr * tl );

#else

extern void DeleteTransList();

#endif

/*-------------------------------------------------------------------------*/
/*-- prueft, ob der uebergebene Transitionszeiger trans in der ueberge-  --*/
/*-- benen Transitionenliste tr enthalten ist                            --*/
/*-- Parameter :                                                         --*/
/*--   TransListPtr tr  : Zeiger auf das erste Listenelement der zu      --*/
/*--                      durchsuchenden Liste                           --*/
/*--   Transition trans : der zu suchende Transitionszeiger              --*/
/*--                                                                     --*/
/*-- Rueckgabe : True  - trans ist in der Liste                          --*/
/*--             False - trans ist nicht in der Liste                    --*/

#if defined(__STDC__) || defined(__cplusplus)

extern bool InTransList( TransListPtr tr, Transition trans );

#else

extern bool InTransList();

#endif

/*-------------------------------------------------------------------------*/
/*-- reserviert Speicher fuer ein PlaceListElem;                         --*/
/*-- der uebergebene Stellenzeiger wird im Feld place abgelegt;          --*/
/*-- das Feld Next wird mit NULL initialisiert;                          --*/
/*-- Parameter :                                                         --*/
/*--   Place place : Zeiger auf eine Stelle, der im Element              --*/
/*--                 eingetragen werden soll                             --*/
/*--                                                                     --*/
/*-- Rueckgabe : Zeiger vom Typ PlaceListPtr auf das neu erzeugte Element--*/

#if defined(__STDC__) || defined(__cplusplus)

extern PlaceListPtr NewPlaceListElem( Place place );

#else

extern PlaceListPtr NewPlaceListElem();

#endif

/*-------------------------------------------------------------------------*/
/*-- fuegt den Stellen-Zeiger place an das Ende der Stellen-Liste        --*/
/*-- pl an, falls place noch nicht in der Liste ist;                     --*/
/*-- Parameter :                                                         --*/
/*--   PlaceListPtr *pl : Referenz des Zeigers auf das erste Listenele-  --*/
/*--                      ment, darf auch NULL sein                      --*/
/*--   Place place      : Zeiger auf die in die Liste einzufuegende      --*/
/*--                      Stelle                                         --*/

#if defined(__STDC__) || defined(__cplusplus)

extern void InsertPlace( PlaceListPtr *pl, Place place );

#else

extern void InsertPlace();

#endif

/*-------------------------------------------------------------------------*/
/*-- loescht eine Liste von PlaceListElems ab dem uebergebenen Zeiger auf--*/
/*-- ein PlaceListElem;                                                  --*/
/*-- Parameter :                                                         --*/
/*--   PlaceListPtr *pl : Referenz auf einen Zeiger auf ein PlaceListElem--*/
/*--                      ab dem die Liste geloescht werden soll         --*/

#if defined(__STDC__) || defined(__cplusplus)

extern void DeletePlaceList( PlaceListPtr * pl );

#else

extern void DeletePlaceList();

#endif

/*-------------------------------------------------------------------------*/
/*-- prueft, ob der uebergebene Stellenzeiger place in der ueberge-      --*/
/*-- benen Stellenliste pl enthalten ist                                 --*/
/*-- Parameter :                                                         --*/
/*--   PlaceListPtr pl  : Zeiger auf das erste Listenelement der zu      --*/
/*--                      durchsuchenden Liste                           --*/
/*--   Place      place : der zu suchende Stellenzeiger                  --*/
/*--                                                                     --*/
/*-- Rueckgabe : True  - place ist in der Liste                          --*/
/*--             False - place ist nicht in der Liste                    --*/

#if defined(__STDC__) || defined(__cplusplus)

extern bool InPlaceList( PlaceListPtr pl, Place place );

#else

extern bool InPlaceList();

#endif

/*-------------------------------------------------------------------------*/
/*-- erzeugt ein neues Listen-Element vom Typ LIST_ELEMENT, traegt       --*/
/*-- NewName im Feld name ein, initialisiert das Feld next mit NULL und  --*/
/*-- gibt einen Zeiger auf das Element zurueck                           --*/
/*-- Parameter :                                                         --*/
/*--   char *NewName : der in das neue Element einzutragende String      --*/
/*--                                                                     --*/
/*-- Rueckgabe : Zeiger vom Typ LIST_POINTER auf das neu erzeugte Element--*/

#if defined(__STDC__) || defined(__cplusplus)

extern LIST_POINTER NewListElem( char *NewName );

#else

extern LIST_POINTER NewListElem();

#endif

/*-------------------------------------------------------------------------*/
/*-- fuegt ein neues Listenelement mit NewName an das Ende der Liste     --*/
/*-- lp an, falls NewName noch nicht in der Liste ist;                   --*/
/*-- Parameter :                                                         --*/
/*--   LIST_POINTER *lp : Referenz des Zeigers auf das erste Listenele-  --*/
/*--                      ment, darf auch NULL sein                      --*/
/*--   char *NewName    : der in das Feld name des neuen Elements einzu- --*/
/*--                      fuegende String                                --*/

#if defined(__STDC__) || defined(__cplusplus)

extern void InsertListElem( LIST_POINTER *lp, char *NewName );

#else

extern void InsertListElem();

#endif

void DeleteNameList(LIST_POINTER list);

/*-------------------------------------------------------------------------*/

#endif

/*-------------------------------------------------------------------------*/
/*--                                                                     --*/
/*--  Ende LIST.H                                                        --*/
/*--                                                                     --*/
/*-------------------------------------------------------------------------*/
