/************************************************************************/
/* mci2ll.c                                                             */
/************************************************************************/
/* November 2001, Christian Stehno                                      */
/************************************************************************/

#include <stdio.h>
#include <string.h>
#include "memlib.h"
#include "rw_prefix.h"
#include "convertPEP.h"

int obj_size=80;

void move_right(Transition tr,int x)
{
  STEdge st;
  TSEdge ts;

  ((Coords)tr->inscript)->x+=obj_size*x;
  for(ts=tr->postset;ts;ts=ts->nextplace)
  {
    ((Coords)ts->place->inscript)->x+=obj_size*x;
    for(st=ts->place->postset;st;st=st->nexttrans)
    {
      if ((st->trans->preset->place->numname!=ts->place->numname) ||
	  !(ts->place->postset->nexttrans || st->trans->preset->prevtrans))
	    	move_right(st->trans,x);
    }
  }
}


int main( int argc, char **argv )
{

  LMPNet net; 
  Place pl;
  Transition tr,tr2;
  int x,x1,y,min,max,left;
  int i;
  bool layout=False,changed=True;
  STEdge st;
  TSEdge ts;
  char *infile=NULL, *outfile=NULL;

  if ( argc < 4 )
    {
      printf( "Convert unfoldings to ll_net files.\n" );
      printf( "Coordinates are calculated from configuration depth.\n" );
      printf( "Usage: %s [-L] -o <net file> <unfolding>\n", *argv );
      printf( "  -L: Try to compute a nice layout.\n" );
      exit ( 1 );
    } /* if */

  CM_storeCoordinates = True;
  strcpy(CM_nettype,"PetriBox");
  for (i=1; i<argc; ++i)
  {
    if(!strcmp(argv[i],"-L"))
      layout=True;
    else if(!strcmp(argv[i],"-o"))
      outfile=argv[++i];
    else if(argv[i][0]!='-')
      infile=argv[i];
  }
  net = readPrefix( infile, NULL, NULL );

  x=y=1;
  for (tr=net->Transitions;tr;tr=tr->nexttrans)
  {
	  Coords co;
	  co=(Coords)MYmalloc(sizeof(Coords));
	  co->x=0;
	  co->y=0;
	  co->next=(Coords)MYmalloc(sizeof(Coords));
	  co->next->next=(Coords)MYmalloc(sizeof(Coords));
	  co->next->next->x=(int)MYstrdup("");
	  if (tr->inscript)
	    co->next->next->y=(int)MYstrdup(tr->inscript);
	  else
	    co->next->next->y=(int)MYstrdup("Tic");
	  tr->label=NULL;
#if 0
	  if (tr->label)
	  {
		co->next->next->y=(int)MYmalloc(sizeof(char[9+strlen(tr->inscript)]));
		strcat((char*)co->next->next->y,tr->inscript);
	        strcat((char*)co->next->next->y,"(cut-off)");
	  }
#endif
	  tr->inscript=(char*)co;
  }

  for (pl=net->Places;pl;pl=pl->nextplace)
  {
	  char corr[10];
	  sprintf(corr,"P%d",(Cardinal32)pl->inscript);
	  pl->inscript=(char*)MYmalloc(sizeof(Coords));
	  ((Coords)pl->inscript)->next=(Coords)MYmalloc(sizeof(Coords));
	  ((Coords)pl->inscript)->next->next=(Coords)MYmalloc(sizeof(Coords));
	  ((Coords)pl->inscript)->next->next->x=(int)strdup(corr);
	  ((Coords)pl->inscript)->next->next->y=(int)strdup("");
	  if (!pl->preset)
	  {
		  ((Coords)pl->inscript)->x=obj_size*x++;
		  ((Coords)pl->inscript)->y=obj_size*y;
		  pl->marks=1;
	  }
	  else
	  {
		  ((Coords)pl->inscript)->x=0;
		  ((Coords)pl->inscript)->y=0;
	  }
	  for(st=pl->postset;st;st=st->nexttrans)
		  st->weight=0;
	  for(ts=pl->preset;ts;ts=ts->nexttrans)
	  {
		  ts->weight=0;
		  if ((int)ts->trans->label==1)
			  pl->label=EXIT;
	  }
  }

  if (layout)
  {
  while (changed) 
  {
    x=0;
    for(changed=False,tr=net->Transitions;tr;tr=tr->nexttrans)
    {
      min=left=MAX_INT;
      max=0;

      if (((Coords)tr->inscript)->y!=0)
	      continue;

      tr->label=NULL;

      for(st=tr->preset;st;st=st->nextplace)
      {
	min=Min(min,((Coords)st->place->inscript)->y);
	max=Max(max,((Coords)st->place->inscript)->y);
	left=Min(left,((Coords)st->place->inscript)->x);
      }
      if (min)
      {
	((Coords)tr->inscript)->y=max+obj_size;
      	((Coords)tr->inscript)->x=left;
	for (tr2=net->Transitions; tr2; tr2=tr2->nexttrans)
	{
	  if (tr2==tr)
	    continue;
	  if (((Coords)tr->inscript)->y==((Coords)tr2->inscript)->y)
	  {
	    if (((Coords)tr2->inscript)->x>=((Coords)tr->inscript)->x)
	      ((Coords)tr->inscript)->x=((Coords)tr2->inscript)->x+obj_size;
	  }
	}
	    
	for (x1=0,ts=tr->postset;ts;ts=ts->nextplace)
    	  {
      	    if (((Coords)ts->place->inscript)->x==0)
	      {
	  	((Coords)ts->place->inscript)->x=((Coords)tr->inscript)->x+x1++*(obj_size/2);
	  	((Coords)ts->place->inscript)->y=max+2*obj_size;
	      }
          }
	changed=True;
      }
    }

  }
  
  for (pl=net->Places;pl;pl=pl->nextplace)
  {
	  x=1;
	  if (pl->postset && (st=pl->postset->nexttrans))
	  {
		  while (st)
		  {
	    if (st->trans->preset && (st->trans->preset->place->numname!=pl->numname))
	    {
			  move_right(st->trans,x);
			  x++;
	    }
			  st=st->nexttrans;
		  }
	  }
  }
  }

  write_new_PEP( net, outfile );
    
  remove_net( &net );

  exit ( 0 );

} /* main() */

/************************************************************************/
/* END OF FILE mci2ll.c                                                 */
/************************************************************************/
