#include "netarray.h"
#include "readlib.h"
#include "convertPEP.h"
#include "pep_strings.h"

/*****************************************************************************/
/* Store pointers to the places of the net in an array.                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)
 
Place * GeneratePlArray( LMPNet net )

#else

Place * GeneratePlArray( net )

  LMPNet net;

#endif

  {

    register Place pl,
                   * ptr,
		   *array;

    array = ptr
        = (Place *) Malloc( ( net->NumPlaces + 1 ) * sizeof( Place ) );
    for ( pl = net->Places ; pl ; pl = ( *++ptr = pl )->nextplace ) skip;

    return ( array );

  } /* GeneratePlArray() */


/*****************************************************************************/
/* Store pointers to the transitions of the net in an array.                 */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)
 
Transition * GenerateTrArray( LMPNet net )

#else

Transition * GenerateTrArray( net )

  LMPNet net;

#endif

  {

    register Transition tr,
                        * ptr,
			*array;

    array = ptr = (Transition *)
               Malloc( ( net->NumTransitions + 1 ) * sizeof( Transition ) );
    for ( tr = net->Transitions ; tr ; tr = ( *++ptr = tr )->nexttrans )
      skip;

    return ( array );

  } /* GenerateTrArray() */


/*****************************************************************************/
/* Copy the place meaning list into an array.                                */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)
 
PlMArray GeneratePlMArray( void )

#else

PlMArray GeneratePlMArray()

#endif

  {

    register PlMArray ptr,
		      array;
    register M_LIST   mptr,
                      mdel;
    register P_LIST   pptr,
                      pdel;

    if ( CM_countPMeanings )
      {
        array = ptr = (PlMArray) Malloc( ( CM_countPMeanings + 1 )
                                         * sizeof( PlMArrayEntry ) );
        mptr = CM_placeMList;
        while ( mptr )
          {
            ptr->meaning = mptr->meaning;
            ptr->set = NULL;
            pptr = mptr->list;
            while ( pptr )
              {
                pdel = pptr->next;
                pptr->next = (P_LIST) ptr->set;
                ptr->set = (P_LIST) pptr;
                pptr = pdel;
              } /* while */
            mptr = ( mdel = mptr )->next;
            Free( mdel );
            ptr++;
          } /* while */
        ptr->meaning = NULL;
        return ( array );
      } /* if */
    else
      {
        return(NULL);
      } /* else */


  } /* GeneratePlMArray() */


/*****************************************************************************/
/* Remove the place meaning array.                                           */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void RemovePLMArray( PlMArray array )

#else

void RemovePLMArray( array )

  PlMArray array;

#endif

  {

    register PlMArray   ptr;
    register Cardinal32 count = CM_countPMeanings;
    register P_LIST del;

    ptr = array;
    while ( count-- )
      {
        if ( ptr->meaning )
          {
            Free( ptr->meaning );
          } /* if */
	del=ptr->set;
	while ( del )
	{
	  del = ( ptr->set = del )->next;
	  Free( ptr->set );
	} /* while */
	ptr++->set = NULL;
      } /* while */
    Free( array );
  } /* RemovePLMArray() */


/*****************************************************************************/
/* Find a place ( binary search ).                                           */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Place find_place( char * name, Place* array, LMPNet net )

#else

Place find_place( name, array, net )

  char  *name;
  Place *array;
  LMPNet net;

#endif

  {

    register Cardinal32 lower = 1,
                        upper = net->NumPlaces,
                        index;

    while ( upper >= lower )
      {
        index = ( upper + lower ) >> 1;
        if ( Kleiner( name, array[ index ]->name ) )
          {
            upper = index - 1;
          } /* if */
        else
          {
            lower = index + 1;
          } /* else */
        if ( Gleich( name, array[ index ]->name ) )
          {
            return ( array[ index ] );
          } /* if */
      } /* while */

    return ( NULL );

  } /* find_place() */


/*****************************************************************************/
/* Find a place ( binary search ).                                           */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Transition find_transition( char * name, Transition* array, LMPNet net )

#else

Transition find_transition( name, array, net )

  char * name;
  Transtition *array;
  LMPNet net;

#endif

  {

    register Cardinal32 lower = 1,
                        upper = net->NumTransitions,
                        index;

    while ( upper >= lower )
      {
        index = ( upper + lower ) >> 1;
        if ( Kleiner( name, array[ index ]->name ) )
          {
            upper = index - 1;
          } /* if */
        else
          {
            lower = index + 1;
          } /* else */
        if ( Gleich( name, array[ index ]->name ) )
          {
            return ( array[ index ] );
          } /* if */
      } /* while */

    return ( NULL );

  } /* find_transition() */


/*****************************************************************************/
/* Find an extended place labelling.                                         */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

P_LIST find_extPLabel( char * name, PlMArray array )

#else

P_LIST find_extPLabel( name, array )

  char * name;
  PlMArray array;

#endif

  {

    register PlMArray   ptr;
    register Cardinal32 count;
    register int        cmp;
 
    if ((count = CM_countPMeanings))
      {
        ptr = array;
        while ( count-- )
          {
            if ( ( cmp = strcmp( name, ptr->meaning ) ) < 0 )
              {
                return ( NULL );
              } /* if */
            else if ( ! cmp )
              {
		if ( ptr->set->next )
		  {
		    PEPerror( "extended place labelling is ambiguous" );
		  }
                return ( ptr->set );
              } /* else if */
            ptr++;
          } /* while */
      } /* if */
    
    return ( NULL );

  } /* find_extPLabel() */

/*** End of file netarray.c ***/
