/*****************************************************************************/
/* neunetze.c                                                                */
/*****************************************************************************/
/*              Schnittstellen Bibliothek zwischen GreatSPN und              */
/*                 Praktikum Werkzege zur Petri Netz Analyse                 */
/*****************************************************************************/
/* Robert Riemann, April 1992                                                */
/* Version 1.7.3                       Stefan Roemer,  04.05.1993-09.01.1995 */
/*****************************************************************************/

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "neunetze.h"
#include "convertPEP.h"
#include "readlib.h"
#include "pep_strings.h"

#if defined(__STDC__) || defined(__cplusplus)

void delete_PT_con_node( LMPNet, STEdge );
void delete_TP_con_node( LMPNet, TSEdge );

#else

void delete_PT_con_node();
void delete_TP_con_node();

#endif

/*****************************************************************************/
/*                              Deklarationen                                */
/*****************************************************************************/

LIST_POINTER  list;
MARKING       mark_list;


/*****************************************************************************/
/*                                Funktionen                                 */
/*****************************************************************************/

/*****************************************************************************/
/*                             Loeschfunktionen                              */
/*****************************************************************************/

/*****************************************************************************/
/***************************** remove_list() *********************************/
/*****************************************************************************/
/* NEUE FUNKTION: 10.05.1993 Stefan Roemer                                   */
/*****************************************************************************/

/* Entfernen einer Liste vom Typ 'LIST_POINTER' aus dem Speicher.          */
#if defined(__STDC__) || defined(__cplusplus)

void remove_list( LIST_POINTER * list )

#else

void remove_list( list )

  LIST_POINTER * list; /* call-by-reference ! */

#endif

  {
    register LIST_POINTER del = *list;

    while ( del )
      {
        del = ( *list = del ) -> next;
        Free( *list );
      } /* while */
    *list = NULL;

  } /* remove_list() */


/*****************************************************************************/
/****************************** remove_marklist() ****************************/
/*****************************************************************************/
/* NEUE FUNKTION: 10.05.1993 Stefan Roemer                                   */
/*****************************************************************************/

/* Entfernen einer Markierungsliste aus dem Speicher.                      */
#if defined(__STDC__) || defined(__cplusplus)

void remove_marklist( MARKING * list )

#else

void remove_marklist( list )

  MARKING * list; /* call-by-reference ! */

#endif

  {
    register MARKING del = *list;

    while ( del )
      {
        del = ( *list = del ) -> next;
        Free( *list );
      } /* while */
    *list = NULL;

  } /* remove_marklist() */


/*****************************************************************************/
/****************************** remove_net() *********************************/
/*****************************************************************************/
/* NEUE FUNKTION: 10.05.1993 Stefan Roemer                                   */
/*****************************************************************************/

/* Entfernen eines Netzes aus dem Speicher.                                */
#if defined(__STDC__) || defined(__cplusplus)

void remove_net( LMPNet * net )

#else

void remove_net( net )

  LMPNet * net; /* call-by-reference ! */

#endif

  {
    if ( *net )
      {
        while ( (*net)->Transitions )
          {
            (void) delete_transition_direct( *net, (*net)->Transitions );
          } /* while */
        while ( (*net)->Places )
          {
            (void) delete_place_direct( *net, (*net)->Places );
          } /* while */
        Free( *net );
        *net = NULL;
      } /* if */

  } /* remove_net() */

/*###########################################################################*/
/*  Matrixinterne Funktionen, die auf die 'numname'-Eintraege zurueckgreifen */
/*###########################################################################*/
/* NEU EINGEFUEGT: 15.02.1994 Stefan Roemer                                  */
/*###########################################################################*/

/*****************************************************************************/
/**************************** num_net_nodes() ********************************/
/* This function numbers the net nodes, i.e. places and transitions of the   */
/* given net in their numname component from 1 to NumPlaces (resp.           */
/* NumTransitions).                                                          */
/*****************************************************************************/
/* New: 15.02.1994 Stefan Roemer                                             */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void num_net_nodes( LMPNet net )

#else

void num_net_nodes( net )

  LMPNet     net;

#endif

  {
    register Cardinal32 count;
    register Place      pl;
    register Transition tr;

    for (count = 1,pl = net->Places;pl;pl = pl->nextplace)
      { 
        pl->numname = count++;
      } /* for pl */

    for (count = 1,tr = net->Transitions;tr; tr = tr->nexttrans)
      {
        tr->numname = count++;
      } /* for tr */

  } /* num_net_nodes() */


/*****************************************************************************/
/*********************************** find_T_num ******************************/
/*****************************************************************************/
/* KONVERSION: 15.02.1994 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Transition find_T_num( LMPNet net, Cardinal32 T_numname )

#else

Transition find_T_num( net, T_numname )
  /* liefert Pointer auf Knoten mit T_name in der ersten Zeile.              */
  /* Ergebnis ist NULL-Pointer, wenn: Keine Transitionenliste in der Matrix  */
  /*                                                              existiert  */
  /*                                  Kein Knoten mit T_name in der ersten   */
  /*                                                        Zeile existiert  */

  LMPNet     net;
  Cardinal32 T_numname;    /* String fuer Name der Transition */

#endif

  {
    register Transition tr = NULL;

    if ( net )
      {
        if (( tr = net->Transitions ))
          {
            while ( tr && ( tr->numname < T_numname ) )
              {
                tr = tr->nexttrans;
              } /* while */
            if ( ! ( tr && ( tr->numname == T_numname ) ) )
              {
                WARNING_T_NUM( "find_T_num", T_numname );
                tr = NULL;
              } /* if */
          } /* if */
        else
          {
            WARNING_NO_T( "find_T_num" );
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET( "find_T_num" );
      } /* else */

    return ( tr );

  } /* find_T_num() */


/*****************************************************************************/
/*********************************** find_P_num ******************************/
/*****************************************************************************/
/* KONVERSION: 15.02.1994 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Place find_P_num( LMPNet net, Cardinal32 P_numname )

#else

Place find_P_num( net, P_numname )
  /* liefert Pointer auf Stellenknoten mit P_numname, also Knoten in erster  */
  /* Spalte der Matrix.                                                      */
  /* Ergebnis ist NULL-Pointer, wenn: keine Stellen im Netz vorhanden sind,  */
  /* also die erste Spalte leer ist kein Stellenknoten mit Namen P_numname   */
  /* in der Liste vorhanden ist                                              */

  LMPNet     net;
  Cardinal32 P_numname;

#endif

  {
    register Place pl = NULL;

    if ( net )
      {
        if (( pl = net->Places ))
          {
            while ( pl && ( pl->numname < P_numname ) )
              {
                pl = pl->nextplace;
              } /* while */
            if ( ! ( pl && ( pl->numname == P_numname ) ) )
              {
                WARNING_P_NUM( "find_P_num", P_numname );
                pl = NULL;
              } /* if */
          } /* if */
        else
          {
            WARNING_NO_P( "find_P_num" );
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET( "find_P_num" );
      } /* else */

    return ( pl );

  } /* find_P_num() */


/*****************************************************************************/
/***************************** find_PT_connection_num ************************/
/*****************************************************************************/
/* KONVERSION: 15.02.1994 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

STEdge find_PT_connection_num( LMPNet net,
                               Cardinal32 P_numname, Cardinal32 T_numname )

#else

STEdge find_PT_connection_num( net, P_numname, T_numname )
  /* liefert Pointer auf Verbindungsknoten fuer die Verbindung von Stelle    */
  /* 'P_name' nach Transition 'T_name'.                                      */
  /* Ergebnis ist NULL-Pointer, wenn: P_name oder T_name Stellen- bzw.       */
  /* Transitionsknoten sind oder kein solcher Verbindungsknoten vorhanden ist*/

  LMPNet     net;
  Cardinal32 P_numname,
             T_numname;

#endif

  {
    register Place  pl;
    register STEdge st;

    if ( net )
      {
        if (( pl = find_P_num( net, P_numname ) ))
          {
            if (( st = pl->postset ))
              {
                while ( st && ( st->trans->numname < T_numname ) )
                  {
                    st = st->nexttrans;
                  } /* while */
                if ( st && ( T_numname == st->trans->numname ) )
                  {
                    return ( st );
                  } /* if */
              } /* if */
          } /* if */
        else
          {
            WARNING_P_NUM( "find_PT_connection_num", P_numname );
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET( "find_PT_connection_num" );
      } /* else */

    return ( NULL );

  } /* find_PT_connection_num() */


/*****************************************************************************/
/***************************** find_TP_connection_num ************************/
/*****************************************************************************/
/* KONVERSION: 15.02.1994 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

TSEdge find_TP_connection_num( LMPNet net,
                               Cardinal32 P_numname, Cardinal32 T_numname )

#else

TSEdge find_TP_connection_num( net, P_numname, T_numname )
  /* liefert Pointer auf Verbindungsknoten fuer die Verbindung von           */
  /* Transition 'T_name' nach Stelle 'P_name'.                               */
  /* Ergebnis ist NULL-Pointer, wenn: P_name oder T_name Stellen- bzw.       */
  /* Transitionsknoten sind oder kein solcher Verbindungsknoten vorhanden ist*/

  LMPNet     net;
  Cardinal32 P_numname,
             T_numname;

#endif

  {
    register Place  pl;
    register TSEdge ts;

    if ( net )
      {
        if (( pl = find_P_num( net, P_numname ) ))
          {
            if (( ts = pl->preset ))
              {
                while ( ts && ( ts->trans->numname < T_numname ) )
                  {
                    ts = ts->nexttrans;
                  } /* while */
                if ( ts && ( T_numname == ts->trans->numname ) )
                  {
                    return ( ts );
                  } /* if */
              } /* if */
          } /* if */
        else
          {
            WARNING_P_NUM( "find_TP_connection_num", P_numname );
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET( "find_TP_connection_num" );
      } /* else */

    return ( NULL );

  } /* find_TP_connection_num() */


/*****************************************************************************/
/*                         Matrixinterne Funktionen                          */
/*****************************************************************************/

/*****************************************************************************/
/*********************************** find_T **********************************/
/*****************************************************************************/
/* KONVERSION: 04.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Transition find_T( LMPNet net, char * T_name )

#else

Transition find_T( net, T_name )
  /* liefert Pointer auf Knoten mit T_name in der ersten Zeile.              */
  /* Ergebnis ist NULL-Pointer, wenn: 					     */
  /* Keine Transitionenliste in der Matrix existiert                         */
  /* Kein Knoten mit T_name in der ersten Zeile existiert                    */

  LMPNet net;
  char   * T_name;    /* String fuer Name der Transition */

#endif

  {
    register Transition tr = NULL;

    if ( net && T_name && *T_name)
      {
        if (( tr = net->Transitions ))
          {
            while ( tr && Kleiner( tr->name, T_name ) )
              {
                tr = tr->nexttrans;
              } /* while */
            if ( ! ( tr && Gleich( tr->name, T_name ) ) )
              {
/*
                WARNING_T( "find_T", T_name );
*/
                tr = NULL;
              } /* if */
          } /* if */
        else
          {
            WARNING_NO_T( "find_T" );
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET( "find_T" );
      } /* else */

    return ( tr );

  } /* find_T() */


/*****************************************************************************/
/*********************************** find_P **********************************/
/*****************************************************************************/
/* KONVERSION: 04.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Place find_P( LMPNet net, char * P_name )

#else

Place find_P( net, P_name )
  /* liefert Pointer auf Stellenknoten mit P_name, also Knoten in erster     */
  /* Spalte der Matrix.                                                      */
  /* Ergebnis ist NULL-Pointer, wenn: keine Stellen im Netz vorhanden sind,  */
  /*                                          also die erste Spalte leer ist */
  /*                                  kein Stellenknoten mit Namen P_name in */
  /*                                                 der Liste vorhanden ist */

  LMPNet net;
  char   * P_name;

#endif

  {
    register Place pl = NULL;

    if ( net && P_name && *P_name )
      {
        if (( pl = net->Places ))
          {
            while ( pl && Kleiner( pl->name, P_name ) )
              {
                pl = pl->nextplace;
              } /* while */
            if ( ! ( pl && Gleich( pl->name, P_name ) ) )
              {
/*
                WARNING_P( "find_P", P_name );
*/
                pl = NULL;
              } /* if */
          } /* if */
        else
          {
            WARNING_NO_P( "find_P" );
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET( "find_P" );
      } /* else */

    return ( pl );

  } /* find_P() */


/*****************************************************************************/
/***************************** find_PT_connection ****************************/
/*****************************************************************************/
/* KONVERSION: 04.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

STEdge find_PT_connection( LMPNet net, char * P_name, char * T_name )

#else

STEdge find_PT_connection( net, P_name, T_name )
  /* liefert Pointer auf Verbindungsknoten fuer die Verbindung von Stelle    */
  /* 'P_name' nach Transition 'T_name'.                                      */
  /* Ergebnis ist NULL-Pointer, wenn: P_name oder T_name Stellen- bzw.       */
  /* Transitionsknoten sind oder kein solcher Verbindungsknoten vorhanden ist*/

  LMPNet net;
  char   * P_name,
         * T_name;

#endif

  {
    register Place  pl;
    register STEdge st;

    if ( net && P_name && T_name && *P_name && *T_name )
      {
        if (( pl = find_P( net, P_name ) ))
          {
            if (( st = pl->postset ))
              {
                while ( st && Kleiner( st->trans->name, T_name ) )
                  {
                    st = st->nexttrans;
                  } /* while */
                if ( st && Gleich( T_name, st->trans->name ) )
                  {
                    return ( st );
                  } /* if */
              } /* if */
          } /* if */
        else
          {
            WARNING_P( "find_PT_connection", P_name );
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET( "find_PT_connection" );
      } /* else */

    return ( NULL );

  } /* find_PT_connection() */


/*****************************************************************************/
/***************************** find_TP_connection ****************************/
/*****************************************************************************/
/* KONVERSION: 04.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

TSEdge find_TP_connection( LMPNet net, char * P_name, char * T_name )

#else

TSEdge find_TP_connection( net, P_name, T_name )
  /* liefert Pointer auf Verbindungsknoten fuer die Verbindung von           */
  /* Transition 'T_name' nach Stelle 'P_name'.                               */
  /* Ergebnis ist NULL-Pointer, wenn: P_name oder T_name Stellen- bzw.       */
  /* Transitionsknoten sind oder kein solcher Verbindungsknoten vorhanden ist*/

  LMPNet net;
  char   * P_name,
         * T_name;

#endif

  {
    register Place  pl;
    register TSEdge ts;

    if ( net && P_name && T_name && *P_name && *T_name )
      {
        if (( pl = find_P( net, P_name ) ))
          {
            if (( ts = pl->preset ))
              {
                while ( ts && Kleiner( ts->trans->name, T_name ) )
                  {
                    ts = ts->nexttrans;
                  } /* while */
                if ( ts && Gleich( T_name, ts->trans->name ) )
                  {
                    return ( ts );
                  } /* if */
              } /* if */
          } /* if */
        else
          {
            WARNING_P( "find_TP_connection", P_name );
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET( "find_TP_connection" );
      } /* else */

    return ( NULL );

  } /* find_TP_connection() */


/*****************************************************************************/
/********************************** row_index ********************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Cardinal32 row_index( LMPNet net, char * P_name )

#else

Cardinal32 row_index( net, P_name )
  /* liefert die Zeilennummer der Matrix, in der 'P_name' als Stellenname im */
  /* ersten Knoten (0-te Spalte) steht.                                      */

  LMPNet net;
  char   * P_name;

#endif

  {
    register Place      pl;
    register Cardinal32 index = 1;

    if ( net && P_name && *P_name )
      {
        if (( pl = net->Places ))
          {
            while ( pl && Kleiner( pl->name, P_name ) )
              {
                index++;
                pl = pl->nextplace;
              } /* while */
            if ( ! pl || ! Gleich( pl->name, P_name ) )
              {
                WARNING_P( "row_index", P_name );
                index = 0;
              } /* if */
          } /* if */
        else
          {
            WARNING_NO_P( "row_index" );
            index = 0;
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET( "row_index" );
        index = 0;
      } /* else */

    return ( index );

  } /* row_index() */


/*****************************************************************************/
/******************************** column_index *******************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Cardinal32 column_index( LMPNet net, char * T_name )

#else

Cardinal32 column_index( net, T_name )
  /* liefert die Spaltennummer, in der 'T_name' als Transitionsname steht.   */
  /* Ist 'T_name' kein Transitionsname, so wird 0 zurueckgeliefert.          */

  LMPNet net;
  char   * T_name;    /* String fuer Name der Transition */

#endif

  {
    register Transition tr;
    register Cardinal32 index = 1;

    if ( net && T_name && *T_name )
      {
        if (( tr = net->Transitions ))
          {
            while ( tr && Kleiner( tr->name, T_name ) )
              {
                index++;
                tr = tr->nexttrans;
              } /* while */
            if ( ! tr || ! Gleich( tr->name, T_name ) )
              {
                WARNING_T( "column_index", T_name );
                index = 0;
              } /* if */
          } /* if */
        else
          {
            WARNING_NO_T( "column_index" );
            index = 0;
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET( "column_index" );
        index = 0;
      } /* else */

    return ( index );

  } /* column_index() */

bool is_inf_lft(Transition tr)
{
  return ((tr->time_label==NULL)||(tr->time_label->high == HUGE_VAL));
}

void set_inf_lft(Transition tr)
{
  if (!tr->time_label)
  {
    tr->time_label=(Time_stamp *)MYmalloc (sizeof(Time_stamp));
    tr->time_label->low=0;
  }
  tr->time_label->high=HUGE_VAL;
}

/*****************************************************************************/
/*                         Initialisierungsfunktionen                        */
/*****************************************************************************/

/*****************************************************************************/
/******************************** empty_net **********************************/
/*****************************************************************************/
/* NEUE FUNKTION: 06.05.1993 Stefan Roemer                                   */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LMPNet empty_net( void )

#else

LMPNet empty_net()
  /* Diese Funktion richtet ein neues Netz ( genauer: nur das Kopfelement )  */
  /* ohne Stellen und ohnen Transitionen ein und gibt einen Zeiger darauf    */
  /* als Funktionswert zurueck.                                              */

#endif

  {
    register LMPNet net;

    net = (LMPNet) Malloc( sizeof( LMPNetHead ) );
    net->Places = net->LastPlace = NULL;
    net->Transitions = net->LastTransition = NULL;
    net->NumPlaces = net->NumTransitions = net->NumComponents = 0;

    return ( net );

  } /* empty_net() */


/*###########################################################################*/
/*###                 BESCHLEUNIGTE EINFUEGE-OPERATIONEN                  ###*/
/*###########################################################################*/
/* AENDERUNG : 21.02.1994 Stefan Roemer                                      */
/*###########################################################################*/

/*****************************************************************************/
/************************** fast_insert_place ********************************/
/*****************************************************************************/
/* NEU EINGEFUEGT : 21.02.1994, Stefan Roemer                                */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Place fast_insert_place( LMPNet net, char * P_name, Cardinal32 marks,
                         Place * pl_last )

#else

Place fast_insert_place( net, P_name, marks, pl_last )
  /* setzt einen Knoten fuer die Stelle mit Namen P_name in die Stellenliste */
  /* an lexikographisch richtiger Stelle ein.                                */

  LMPNet      net;
  char        * P_name;
  Cardinal32  marks;
  Place       * pl_last;

#endif

  {
    register Place pl,
                   plnew;

    if ( ! ( pl = *pl_last ) || ! Kleiner( pl->name, P_name ) )
      {
        pl = net->Places;
      } /* if */
    while ( pl && Kleiner( pl->name, P_name ) )
      {
        pl = pl->nextplace;
      } /* while */
    if ( pl && Gleich( P_name, pl->name ) )
      {
#ifdef _DOPPELTE_NICHT_ZULASSEN_
        fprintf( stderr,
               "ERROR: NN.fast_insert_place(): place '%s' already exists.\n",
                P_name );
        exit ( 1 );
#else
        fprintf( stderr,
               "WARNING: NN.fast_insert_place(): place '%s' already exists.\n",
                P_name );
        return ( *pl_last = pl );
#endif
      } /* if */
    plnew = (Place) Malloc( sizeof( PlElem ) );
    if ( pl )
      {
        if (( plnew->prevplace = pl->prevplace ))
          {
            plnew->prevplace->nextplace = plnew;
          } /* if */
        else
          {
            net->Places = plnew;
          } /* else */
        ( plnew->nextplace = pl ) -> prevplace = plnew;
      } /* if */
    else
      {
        if (( plnew->prevplace = net->LastPlace ))
          {
            net->LastPlace->nextplace = plnew;
          } /* if */
        else
          {
            net->Places = plnew;
          } /* else */
        ( net->LastPlace = plnew ) -> nextplace = NULL;
      } /* else */
    plnew->name = Strdup( P_name );
    plnew->component = plnew->numname = 0;
    plnew->preset = NULL;
    plnew->postset = NULL;
    plnew->inscript = NULL;
    plnew->label = EMPTY;
    plnew->marks = marks;
    /* Neu fuer Zeit-Petrinetze */
    plnew->tokenage=0;

    net->NumPlaces++;

    return ( *pl_last = plnew );

  } /* fast_insert_place() */


/*****************************************************************************/
/************************* fast_insert_transition ****************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Transition fast_insert_transition( LMPNet net, char * T_name,
                                   Transition * tr_last )

#else

Transition fast_insert_transition( net, T_name, tr_last )
  /* Setzt einen neuen Knoten fuer eine Transition an der lexikographisch    */
  /* richtigen Stelle in die Matrix ein, also in die erste Zeile.            */

  LMPNet     net;
  char       * T_name;
  Transition * tr_last;

#endif

  {

    register Transition tr,
                        trnew;

    if ( ! ( tr = *tr_last ) || ! Kleiner( tr->name, T_name ) )
      {
        tr = net->Transitions;
      } /* if */
    while ( tr && Kleiner( tr->name, T_name ) )
      {
        tr = tr->nexttrans;
      } /* while */
    if ( tr && Gleich( T_name, tr->name ) )
      {
#ifdef _DOPPELTE_NICHT_ZULASSEN_
        fprintf( stderr,
       "ERROR: NN.fast_insert_transition(): transition '%s' already exists.\n",
                 T_name );
        exit ( 1 );
#else
        fprintf( stderr,
     "WARNING: NN.fast_insert_transition(): transition '%s' already exists.\n",
                 T_name );
        return ( *tr_last = tr );
#endif
      } /* if */
    trnew = (Transition) Malloc( sizeof( TrElem ) );
    if ( tr )
      {
        if (( trnew->prevtrans = tr->prevtrans ))
          {
            trnew->prevtrans->nexttrans = trnew;
          } /* if */
        else
          {
            net->Transitions = trnew;
          } /* else */
        ( trnew->nexttrans = tr ) -> prevtrans = trnew;
      } /* if */
    else
      {
        if (( trnew->prevtrans = net->LastTransition ))
          {
            net->LastTransition->nexttrans = trnew;
          } /* if */
        else
          {
            net->Transitions = trnew;
          } /* else */
        ( net->LastTransition = trnew ) -> nexttrans = NULL;
      } /* else */
    trnew->name = Strdup( T_name );
    trnew->numname = 0;
    trnew->label = NULL;
    trnew->preset = NULL;
    trnew->postset = NULL;
    trnew->inscript = NULL;
    /* Neu fuer Zeit-Petrinetze */
    trnew->time_label=NULL;

    net->NumTransitions++;

    return ( *tr_last = trnew );

  } /* fast_insert_transition() */


/*****************************************************************************/
/******************************** insert_TP_con ******************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

TSEdge fast_insert_TP_con( LMPNet net, char * P_name, char * T_name,
                           Cardinal32 tsweight, Place * pl_last,
                           Transition * tr_last, TSEdge * ts_last )

#else

TSEdge fast_insert_TP_con( net, P_name, T_name, tsweight,
                           pl_last , tr_last , ts_last )
  /* setzt einen Verbindungsknoten mit dem Gewicht tsweight in die Matrix    */
  /* ein; falls ein solcher schon vorhanden, wird das Gewicht entsprechend   */
  /* geaendert.                                                              */

  LMPNet     net;
  char       * P_name,
             * T_name;
  Cardinal32 tsweight;
  Place      * pl_last;
  Transition * tr_last;
  TSEdge     * ts_last;

#endif

  {
    register Place      pl;
    register Transition tr;
    register TSEdge     ts,
                        tsnew;
    register bool       kl1,
                        kl2;

    pl = *pl_last;
    if ( ( ts = *ts_last ) && Gleich( ts->place->name, P_name ) )
      {
        pl = ts->place;
      } /* if */
    else if ( pl && Gleich( pl->name, P_name ) )
      {
        ts = NULL;
      } /* else if */
    else if ( pl && ts && ( ( kl1 = Kleiner( pl->name, P_name ) ),
                            ( kl2 = Kleiner( ts->place->name, P_name ) ),
                            ( kl1 || kl2 ) ) )
      {
        if ( ( ! kl1 ) || ( kl2 && Kleiner( pl->name, ts->place->name ) ) )
          {
            pl = ts->place;
          } /* if */
        else
          {
            ts = NULL;
          } /* else */
      } /* else if */
    else
      {
        pl = net->Places;
        ts = NULL;
      } /* else */
    while ( pl && Kleiner( pl->name, P_name ) )
      {
ts = NULL;
        pl = pl->nextplace;
      } /* while */

    if ( ! ( pl && Gleich( pl->name, P_name ) ) )
      {
        WARNING_P( "fast_insert_TP_con", P_name );
        return ( NULL );
      } /* if */

    if ( ( ts && Kleiner( ts->trans->name, T_name ) ) || ( ts = pl->preset ) )
      {
        if ( Kleiner( T_name, ts->trans->name ) )
          {
            ts = NULL;
          } /* if */
        else
          {
            while ( ts->nexttrans
                    && Kleiner( ts->nexttrans->trans->name, T_name ) )
              {
                ts = ts->nexttrans;
              } /* while */
            if ( ( tsnew = ts->nexttrans )
                 && Gleich( tsnew->trans->name, T_name ) )
              {
                *pl_last = tsnew->place;
                *tr_last = tsnew->trans;
                if ( ! ( tsnew->weight = tsweight ) )
                  {
                    delete_TP_con_node( net, tsnew );
                    return ( NULL );
                  } /* if */
                else
                  {
                    return ( *ts_last = tsnew );
                  } /* else */
              } /* if */
          } /* else */
      } /* if */
    tr = ( *tr_last && Gleich( (*tr_last)->name, T_name ) ) ? *tr_last
                                           : ts ? ts->trans : net->Transitions;
    while ( tr && Kleiner( tr->name, T_name ) )
      {
        tr = tr->nexttrans;
      } /* while */
    if ( ! ( tr && Gleich( tr->name, T_name ) ) )
      {
        WARNING_T( "fast_insert_TP_con", T_name );
        return ( NULL );
      } /* if */

    tsnew = (TSEdge) Malloc( sizeof( TSElem ) );
    if ( ts )
      {
        if (( tsnew->nexttrans = ts->nexttrans ))
          {
            tsnew->nexttrans->prevtrans = tsnew;
          } /* if */
        ( tsnew->prevtrans = ts ) -> nexttrans = tsnew;
      } /* if */
    else
      {
        if (( tsnew->nexttrans = pl->preset ))
          {
            tsnew->nexttrans->prevtrans = tsnew;
          } /* if */
        ( pl->preset = tsnew ) -> prevtrans = NULL;
      } /* else */
    if (( ts = tr->postset ))
      {
        if ( Kleiner( P_name, ts->place->name ) )
          {
            ts = NULL;
          } /* if */
        else
          {
            while ( ts->nextplace
                    && Kleiner( ts->nextplace->place->name, P_name ) )
              {
                ts = ts->nextplace;
              } /* while */
          } /* else */
      } /* if */
    if ( ts )
      {
        if (( tsnew->nextplace = ts->nextplace ))
          {
            tsnew->nextplace->prevplace = tsnew;
          } /* if */
        ( tsnew->prevplace = ts ) -> nextplace = tsnew;
      } /* if */
    else
      {
        if (( tsnew->nextplace = tr->postset ))
          {
            tsnew->nextplace->prevplace = tsnew;
          } /* if */
        ( tr->postset = tsnew ) -> prevplace = NULL;
      } /* else */
    tsnew->weight = tsweight;
    *pl_last = tsnew->place = pl;
    *tr_last = tsnew->trans = tr;

    return ( *ts_last = tsnew );

  } /* fast_insert_TP_con() */


/*****************************************************************************/
/********************************* insert_PT_con *****************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

STEdge fast_insert_PT_con( LMPNet net, char * P_name, char * T_name,
                           Cardinal32 stweight, Place * pl_last,
                           Transition * tr_last, STEdge * st_last )

#else

STEdge fast_insert_PT_con( net, P_name, T_name, stweight,
                           pl_last, tr_last, st_last )
  /* setzt einen Verbindungsknoten mit dem Gewicht stweight in die Matrix ein; */
  /* falls ein solcher schon vorhanden, wird das Gewicht entsprechend        */
  /* geaendert.                                                              */

  LMPNet     net;
  char       * P_name,
             * T_name;
  Cardinal32 stweight;                  /* Anzahl der Verbindungen von einer */
                                        /* Stelle zu einer Transition        */
  Place      * pl_last;
  Transition * tr_last;
  STEdge     * st_last;

#endif

  {
    register Place      pl;
    register Transition tr;
    register STEdge     st,
                        stnew;
    register bool       kl1,
                        kl2;

    pl = *pl_last;
    if ( ( st = *st_last ) && Gleich( st->place->name, P_name ) )
      {
        pl = st->place;
      } /* if */
    else if ( pl && Gleich( pl->name, P_name ) )
      {
        st = NULL;
      } /* else if */
    else if ( pl && st && ( ( kl1 = Kleiner( pl->name, P_name ) ),
                            ( kl2 = Kleiner( st->place->name, P_name ) ),
                            ( kl1 || kl2 ) ) )
      {
        if ( ( ! kl1 ) || ( kl2 && Kleiner( pl->name, st->place->name ) ) )
          {
            pl = st->place;
          } /* if */
        else
          {
            st = NULL;
          } /* else */
      } /* else if */
    else
      {
        pl = net->Places;
        st = NULL;
      } /* else */
    while ( pl && Kleiner( pl->name, P_name ) )
      {
st = NULL;
        pl = pl->nextplace;
      } /* while */

    if ( ! ( pl && Gleich( pl->name, P_name ) ) )
      {
        WARNING_P( "fast_insert_PT_con", P_name );
        return ( NULL );
      } /* if */

    if ( ( st && Kleiner( st->trans->name, T_name ) ) || ( st = pl->postset ) )
      {
        if ( Kleiner( T_name, st->trans->name ) )
          {
            st = NULL;
          } /* if */
        else
          {
            while ( st->nexttrans
                    && Kleiner( st->nexttrans->trans->name, T_name ) )
              {
                st = st->nexttrans;
              } /* while */
            if ( ( stnew = st->nexttrans )
                 && Gleich( stnew->trans->name, T_name ) )
              {
                *pl_last = stnew->place;
                *tr_last = stnew->trans;
                if ( ! ( stnew->weight = stweight ) )
                  {
                    delete_PT_con_node( net, stnew );
                    return ( NULL );
                  } /* if */
                else
                  {
                    return ( *st_last = stnew );
                  } /* else */
              } /* if */
          } /* else */
      } /* if */
    tr = ( *tr_last && Gleich( (*tr_last)->name, T_name ) ) ? *tr_last
                                           : st ? st->trans : net->Transitions;
    while ( tr && Kleiner( tr->name, T_name ) )
      {
        tr = tr->nexttrans;
      } /* while */
    if ( ! ( tr && Gleich( tr->name, T_name ) ) )
      {
        WARNING_T( "fast_insert_PT_con", T_name );
        return ( NULL );
      } /* if */

    stnew = (STEdge) Malloc( sizeof( STElem ) );
    if ( st )
      {
        if (( stnew->nexttrans = st->nexttrans ))
          {
            stnew->nexttrans->prevtrans = stnew;
          } /* if */
        ( stnew->prevtrans = st ) -> nexttrans = stnew;
      } /* if */
    else
      {
        if (( stnew->nexttrans = pl->postset ))
          {
            stnew->nexttrans->prevtrans = stnew;
          } /* if */
        ( pl->postset = stnew ) -> prevtrans = NULL;
      } /* else */
    if (( st = tr->preset ))
      {
        if ( Kleiner( P_name, st->place->name ) )
          {
            st = NULL;
          } /* if */
        else
          {
            while ( st->nextplace
                    && Kleiner( st->nextplace->place->name, P_name ) )
              {
                st = st->nextplace;
              } /* while */
          } /* else */
      } /* if */
    if ( st )
      {
        if (( stnew->nextplace = st->nextplace ))
          {
            stnew->nextplace->prevplace = stnew;
          } /* if */
        ( stnew->prevplace = st ) -> nextplace = stnew;
      } /* if */
    else
      {
        if (( stnew->nextplace = tr->preset ))
          {
            stnew->nextplace->prevplace = stnew;
          } /* if */
        ( tr->preset = stnew ) -> prevplace = NULL;
      } /* else */
    stnew->weight = stweight;
    *pl_last = stnew->place = pl;
    *tr_last = stnew->trans = tr;

    return ( *st_last = stnew );
  } /* fast_insert_PT_con() */


/*****************************************************************************/
/************************** fast_insert_con **********************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void fast_insert_con( LMPNet net, char * P_name, char * T_name,
                      Cardinal32 tsweight, Cardinal32 stweight,
                      Place * pl_last, Transition * tr_last,
                      STEdge * st_last, TSEdge * ts_last )

#else

void fast_insert_con( net, P_name, T_name, tsweight, stweight,
                      pl_last, tr_last, st_last, ts_last )
  /* setzt die beiden Verbindungsknoten mit den Gewichten tsweight und       */
  /* stweight in die Matrix ein. Falls solche schon vorhanden sind, werden   */
  /* die Gewichte entsprechend geaendert.                                    */

  LMPNet     net;
  char       * P_name,
             * T_name;
  Cardinal32 tsweight,                  /* Anzahl der Verbindungen von einer */
                                        /* Transition zu einer Stelle        */
             stweight;                  /* Anzahl der Verbindungen von einer */
                                        /* Stelle zu einer Transition        */
  Place      * pl_last;
  Transition * tr_last;
  STEdge     * st_last;
  TSEdge     * ts_last;

#endif

  {
    fast_insert_PT_con( net, P_name, T_name, stweight,
                        pl_last, tr_last, st_last );
    fast_insert_TP_con( net, P_name, T_name, tsweight,
                        pl_last, tr_last, ts_last );
  } /* fast_insert_con() */

/*****************************************************************************/
/*                             Ausgabefunktionen                             */
/*****************************************************************************/

/*****************************************************************************/
/******************************** show_multiset ******************************/
/*****************************************************************************/
/* Gibt die Liste 'list' als Menge auf dem Bildschirm aus */
/* 'list' ist als Datenstruktur fuer Vor- und Nachbereiche konzipiert. */
/* In der Regel wird also eine Menge von Stellen oder Transitionen angezeigt */

#if defined(__STDC__) || defined(__cplusplus)

void show_multiset( LIST_POINTER list )

#else

void show_multiset( list )

LIST_POINTER list;

#endif

  {
    printf( "{" );
    if ( list )
      {
        while ( list )
          {
            printf("%s", list->name);
            if (( list = list->next ))
             {
               printf( ", " );
             } /* if */
          } /* while */
        printf("}\n");
      } /* if */
    else
      {
        printf( "}\n" );
      } /* else */
  } /* show_multiset() */


/*****************************************************************************/
/*                          Netzinformationsfunktionen                       */
/*****************************************************************************/

/*****************************************************************************/
/******************************* is_place_successor **************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

bool is_place_successor( LMPNet net, char * P_name, char * T_name )

#else

bool is_place_successor( net, P_name, T_name )
  /* liefert True, falls Verbindung P_name --> T_name besteht, sonst False.  */

  LMPNet net;
  char   * P_name,
         * T_name;

#endif

  {
    return ( (bool) !! ( find_PT_connection( net, P_name, T_name ) ) );
  } /* is_place_successor() */


/*****************************************************************************/
/******************************* is_trans_successor **************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

bool is_trans_successor( LMPNet net, char * P_name, char * T_name )

#else

bool is_trans_successor( net, P_name, T_name )
  /* liefert True, falls Verbindung T_name --> P_name besteht, sonst False.  */

  LMPNet net;
  char   * P_name,
         * T_name;

#endif

  {
    return ( (bool) !! ( find_TP_connection( net, P_name, T_name ) ) );
  } /* is_trans_successor() */


/*****************************************************************************/
/************************ max_placename_length *******************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Cardinal max_placename_length( LMPNet net )

#else

Cardinal max_placename_length( net )
  /* liefert die Laenge des laengsten Stellennamens.                         */
  /* Wird benoetigt, um die uebersichtlich auszugeben.                       */

  LMPNet net;

#endif

  {
    register Cardinal length,
                      maxlength = 0;
    register Place    pl;

    if ( net )
      {
        for(pl = net->Places; pl; pl = pl->nextplace)
          {
            if ( ( length = strlen( pl->name ) ) > maxlength )
              {
                maxlength = length;
              } /* if */
          }
      } /* if */
    else
      {
        WARNING_NO_NET( "max_placename_length" );
      } /* else */

    return ( maxlength );
  } /* max_placename_length() */


/*****************************************************************************/
/*********************** max_transitionname_length ***************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Cardinal max_transitionname_length( LMPNet net )

#else

Cardinal max_transitionname_length( net )
/* liefert die Laenge des laengsten Transitionsnamens.                       */
/* Wird fuer die Matrixausgabe benoetigt.                                    */

  LMPNet net;

#endif

  {
    register Cardinal   length,
                        maxlength = 0;
    register Transition tr;

    if ( net )
      {
        tr = net->Transitions;
        while ( tr )
          {
            if ( ( length = strlen( tr->name ) ) > maxlength )
              {
                maxlength = length;
              } /* if */
            tr = tr->nexttrans;
          } /* while */
      } /* if */
    else
      {
        WARNING_NO_NET( "max_transitionname_length" );
      } /* else */

    return ( maxlength );
  } /* max_transitionname_length() */


/*****************************************************************************/
/*                         Expandierungsfunktionen                           */
/*****************************************************************************/

/*****************************************************************************/
/******************************* insert_place ********************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Place insert_place( LMPNet net, char * P_name, Cardinal32 marks )

#else

Place insert_place( net, P_name, marks )
  /* setzt einen Knoten fuer die Stelle mit Namen P_name in die Stellenliste */
  /* an lexikographisch richtiger Stelle ein.                                */

  LMPNet      net;
  char        * P_name;
  Cardinal32  marks;

#endif

  {
    register Place pl,
                   plnew;

    if ( net && P_name && *P_name )
      {
        pl = net->Places;
        while ( pl && Kleiner( pl->name, P_name ) )
          {
            pl = pl->nextplace;
          } /* while */
        if ( pl && Gleich( P_name, pl->name ) )
          {
            fprintf( stderr,
                    "WARNING: NN.insert_place(): place '%s' already exists.\n",
                     P_name );
            return ( pl );
          } /* if */
        plnew = (Place) Malloc( sizeof( PlElem ) );
        if ( pl )
          {
            if (( plnew->prevplace = pl->prevplace ))
              {
                plnew->prevplace->nextplace = plnew;
              } /* if */
            else
              {
                net->Places = plnew;
              } /* else */
            ( plnew->nextplace = pl ) -> prevplace = plnew;
          } /* if */
        else
          {
            if (( plnew->prevplace = net->LastPlace ))
              {
                net->LastPlace->nextplace = plnew;
              } /* if */
            else
              {
                net->Places = plnew;
              } /* else */
            ( net->LastPlace = plnew ) -> nextplace = NULL;
          } /* else */
        plnew->name = Strdup( P_name );
        plnew->component = plnew->numname = 0;
        plnew->preset = NULL;
        plnew->postset = NULL;
        plnew->label = EMPTY;
        plnew->marks = marks;
        /* Neu fuer Zeit-Petrinetze */
        plnew->tokenage=0;

        net->NumPlaces++;
      } /* if */
    else
      {
        WARNING_NO_NET( "insert_place" );
        return ( NULL );
      } /* else */

    return ( plnew );
  } /* insert_place() */

/*****************************************************************************/
/****************************** insert_transition ****************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Transition insert_transition( LMPNet net, char * T_name )

#else

Transition insert_transition( net, T_name )
  /* Setzt einen neuen Knoten fuer eine Transition an der lexikographisch    */
  /* richtigen Stelle in die Matrix ein, also in die erste Zeile.            */

  LMPNet net;
  char   * T_name;

#endif

  {
    register Transition tr,
                        trnew;

    if ( net && T_name && *T_name )
      {
        tr = net->Transitions;
        while ( tr && Kleiner( tr->name, T_name ) )
          {
            tr = tr->nexttrans;
          } /* while */
        if ( tr && Gleich( T_name, tr->name ) )
          {
            fprintf( stderr,
          "WARNING: NN.insert_transition(): transition '%s' already exists.\n",
                     T_name );
            return ( tr );
          } /* if */
        trnew = (Transition) Malloc( sizeof( TrElem ) );
        if ( tr )
          {
            if (( trnew->prevtrans = tr->prevtrans ))
              {
                trnew->prevtrans->nexttrans = trnew;
              } /* if */
            else
              {
                net->Transitions = trnew;
              } /* else */
            ( trnew->nexttrans = tr ) -> prevtrans = trnew;
          } /* if */
        else
          {
            if (( trnew->prevtrans = net->LastTransition ))
              {
                net->LastTransition->nexttrans = trnew;
              } /* if */
            else
              {
                net->Transitions = trnew;
              } /* else */
            ( net->LastTransition = trnew ) -> nexttrans = NULL;
          } /* else */
        trnew->name = Strdup( T_name );
        trnew->numname = 0;
        trnew->label = NULL;
        trnew->preset = NULL;
        trnew->postset = NULL;
        /* Neu fuer Zeit-Petrinetze */
        trnew->time_label=NULL;

        net->NumTransitions++;
      } /* if */
    else
      {
        WARNING_NO_NET( "insert_transition" );
        return ( NULL );
      } /* else */

    return ( trnew );
  } /* insert_transition() */


/*****************************************************************************/
/******************************* insert_con **********************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void insert_con( LMPNet net, char * P_name, char * T_name,
                 Cardinal32 tsweight, Cardinal32 stweight )

#else

void insert_con( net, P_name, T_name, tsweight, stweight )
  /* setzt die beiden Verbindungsknoten mit den Gewichten tsweight und stweight  */
  /* in die Matrix ein. Falls solche schon vorhanden sind, werden die        */
  /* Gewichte entsprechend geaendert.                                        */

  LMPNet     net;
  char       * P_name,
             * T_name;
  Cardinal32 tsweight,                  /* Anzahl der Verbindungen von einer */
                                        /* Transition zu einer Stelle        */
             stweight;                  /* Anzahl der Verbindungen von einer */
                                        /* Stelle zu einer Transition        */

#endif

  {
    insert_PT_con( net, P_name, T_name, stweight );
    insert_TP_con( net, P_name, T_name, tsweight );
  } /* insert_con() */


/*****************************************************************************/
/******************************** insert_TP_con ******************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

TSEdge insert_TP_con( LMPNet net, char * P_name, char * T_name,
                      Cardinal32 tsweight )

#else

TSEdge insert_TP_con( net, P_name, T_name, tsweight )
  /* setzt einen Verbindungsknoten mit dem Gewicht tsweight in die Matrix ein; */
  /* falls ein solcher schon vorhanden, wird das Gewicht entsprechend        */
  /* geaendert.                                                              */

  LMPNet     net;
  char       * P_name,
             * T_name;
  Cardinal32 tsweight;                  /* Anzahl der Verbindungen von einer */
                                        /* Transition zu einer Stelle        */

#endif

  {
    register Place      pl;
    register Transition tr;
    register TSEdge     ts,
                        tsnew;

    if ( net && P_name && T_name && *P_name && *T_name )
      {
        if ( ! tsweight )
          {
            delete_TP_con_node( net,
                                   find_TP_connection( net, P_name, T_name ) );
            return ( NULL );
          } /* if */
        if ( ( pl = find_P( net, P_name ) ) && ( tr = find_T( net, T_name ) ) )
          {
            /* Position des Knotens suchen */
            if (( ts = pl->preset ))
              {
                if ( Kleiner( T_name, ts->trans->name ) )
                  {
                    ts = NULL;
                  } /* if */
                else
                  {
                    while ( ts->nexttrans
                            && Kleiner( ts->nexttrans->trans->name, T_name ) )
                      {
                        ts = ts->nexttrans;
                      } /* while */
                    if ( ( tsnew = ts->nexttrans )
                         && Gleich( tsnew->trans->name, T_name ) )
                      {
                        tsnew->weight = tsweight;
                        return ( tsnew );
                      } /* if */
                  } /* else */
              } /* if */
            tsnew = (TSEdge) Malloc( sizeof( TSElem ) );
            if ( ts )
              {
                if (( tsnew->nexttrans = ts->nexttrans ))
                  {
                    tsnew->nexttrans->prevtrans = tsnew;
                  } /* if */
                ( tsnew->prevtrans = ts ) -> nexttrans = tsnew;
              } /* if */
            else
              {
                if (( tsnew->nexttrans = pl->preset ))
                  {
                    tsnew->nexttrans->prevtrans = tsnew;
                  } /* if */
                ( pl->preset = tsnew ) -> prevtrans = NULL;
              } /* else */
            if (( ts = tr->postset ))
              {
                if ( Kleiner( P_name, ts->place->name ) )
                  {
                    ts = NULL;
                  } /* if */
                else
                  {
                    while ( ts->nextplace
                            && Kleiner( ts->nextplace->place->name, P_name ) )
                      {
                        ts = ts->nextplace;
                      } /* while */
                  } /* else */
              } /* if */
            if ( ts )
              {
                if (( tsnew->nextplace = ts->nextplace ))
                  {
                    tsnew->nextplace->prevplace = tsnew;
                  } /* if */
                ( tsnew->prevplace = ts ) -> nextplace = tsnew;
              } /* if */
            else
              {
                if (( tsnew->nextplace = tr->postset ))
                  {
                    tsnew->nextplace->prevplace = tsnew;
                  } /* if */
                ( tr->postset = tsnew ) -> prevplace = NULL;
              } /* else */
            tsnew->weight = tsweight;
            tsnew->place = pl;
            tsnew->trans = tr;
          }
        else
          {
            fprintf( stderr,
"WARNING: NN.insert_TP_con(): place '%s' or transition '%s' does not exist.\n",
                     P_name, T_name );
            return ( NULL );
          }
      }
    else
      {
        WARNING_NO_NET( "insert_TP_con" );
        return ( NULL );
      }
    return ( tsnew );
  } /* insert_TP_con() */


/*****************************************************************************/
/********************************* insert_PT_con *****************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

STEdge insert_PT_con( LMPNet net, char * P_name, char * T_name,
                      Cardinal32 stweight )

#else

STEdge insert_PT_con( net, P_name, T_name, stweight )
  /* setzt einen Verbindungsknoten mit dem Gewicht stweight in die Matrix ein; */
  /* falls ein solcher schon vorhanden, wird das Gewicht entsprechend        */
  /* geaendert.                                                              */

  LMPNet     net;
  char       * P_name,
             * T_name;
  Cardinal32 stweight;                  /* Anzahl der Verbindungen von einer */
                                        /* Stelle zu einer Transition        */

#endif

  {
    register Place      pl;
    register Transition tr;
    register STEdge     st,
                        stnew;

    if ( net && P_name && T_name && *P_name && *T_name )
      {
        if ( ! stweight )
          {
            delete_PT_con_node( net,
                                   find_PT_connection( net, P_name, T_name ) );
            return ( NULL );
          }
        if ( ( pl = find_P( net, P_name ) ) && ( tr = find_T( net, T_name ) ) )
          {
            /* Position des Knotens suchen */
            if (( st = pl->postset ))
              {
                if ( Kleiner( T_name, st->trans->name ) )
                  {
                    st = NULL;
                  }
                else
                  {
                    while ( st->nexttrans
                            && Kleiner( st->nexttrans->trans->name, T_name ) )
                      {
                        st = st->nexttrans;
                      }
                    if ( ( stnew = st->nexttrans )
                         && Gleich( stnew->trans->name, T_name ) )
                      {
                        stnew->weight = stweight;
                        return ( stnew );
                      }
                  }
              }
            stnew = (STEdge) Malloc( sizeof( STElem ) );
            if ( st )
              {
                if (( stnew->nexttrans = st->nexttrans ))
                  {
                    stnew->nexttrans->prevtrans = stnew;
                  }
                ( stnew->prevtrans = st ) -> nexttrans = stnew;
              }
            else
              {
                if (( stnew->nexttrans = pl->postset ))
                  {
                    stnew->nexttrans->prevtrans = stnew;
                  }
                ( pl->postset = stnew ) -> prevtrans = NULL;
              }
            if (( st = tr->preset ))
              {
                if ( Kleiner( P_name, st->place->name ) )
                  {
                    st = NULL;
                  }
                else
                  {
                    while ( st->nextplace
                            && Kleiner( st->nextplace->place->name, P_name ) )
                      {
                        st = st->nextplace;
                      }
                  }
              }
            if ( st )
              {
                if (( stnew->nextplace = st->nextplace ))
                  {
                    stnew->nextplace->prevplace = stnew;
                  } /* if */
                ( stnew->prevplace = st ) -> nextplace = stnew;
              }
            else
              {
                if (( stnew->nextplace = tr->preset ))
                  {
                    stnew->nextplace->prevplace = stnew;
                  }
                ( tr->preset = stnew ) -> prevplace = NULL;
              }
            stnew->weight = stweight;
            stnew->place = pl;
            stnew->trans = tr;
          }
        else
          {
            fprintf( stderr,
"WARNING: NN.insert_PT_con(): place '%s' or transition '%s' does not exist.\n",
                     P_name, T_name );
            return ( NULL );
          }
      }
    else
      {
        WARNING_NO_NET( "insert_PT_con" );
        return ( NULL );
      }
    return ( stnew );
  } /* insert_PT_con() */

/*****************************************************************************/
/*                              Reduktionsfunktionen                         */
/*****************************************************************************/

/*****************************************************************************/
/***************************** delete_PT_con_node ****************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void delete_PT_con_node( LMPNet net, STEdge st )

#else

void delete_PT_con_node( net, st )
  /* loescht in Matrix 'net' den Verbindungsknoten, auf den st zeigt, */
  /* und aendert die Verzeigerung der Matrix entsprechend.            */

  LMPNet net;
  STEdge st;

#endif

  {
    if ( net && st )
      {
        if ( st->nextplace )
          { st->nextplace->prevplace = st->prevplace; }
        if ( st->prevplace )
          { st->prevplace->nextplace = st->nextplace; }
        else
          { st->trans->preset = st->nextplace; }
        if ( st->nexttrans )
          { st->nexttrans->prevtrans = st->prevtrans; }
        if ( st->prevtrans )
          { st->prevtrans->nexttrans = st->nexttrans; }
        else
          { st->place->postset = st->nexttrans; }
      }
    else
      {
        fprintf( stderr,
            "WARNING: NN.delete_PT_con_node(): not a net or no node.\n" );
      }
  } /* delete_PT_con_node() */

/*****************************************************************************/
/***************************** delete_TP_con_node ****************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void delete_TP_con_node( LMPNet net, TSEdge ts )

#else

void delete_TP_con_node( net, ts )
  /* loescht in Matrix 'net' den Verbindungsknoten, auf den ts zeigt, */
  /* und aendert die Verzeigerung der Matrix entsprechend.            */

  LMPNet net;
  TSEdge ts;

#endif

  {
    if ( net && ts )
      {
        if ( ts->nextplace )
          { ts->nextplace->prevplace = ts->prevplace; }
        if ( ts->prevplace )
          { ts->prevplace->nextplace = ts->nextplace; }
        else
          { ts->trans->postset = ts->nextplace; }
        if ( ts->nexttrans )
          { ts->nexttrans->prevtrans = ts->prevtrans; }
        if ( ts->prevtrans )
          { ts->prevtrans->nexttrans = ts->nexttrans; }
        else
          { ts->place->preset = ts->nexttrans; }
      }
    else
      {
        WARNING_NO_NET( "delete_TP_con_node" );
      }
  } /* delete_TP_con_node() */

/*****************************************************************************/
/*************************** delete_TP_connection ****************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

bool delete_TP_connection( LMPNet net, char * P_name, char * T_name )

#else

bool delete_TP_connection( net, P_name, T_name )
  /* loescht eine Verbindung von T_name zu P_name, dekrementier 'weight' im  */
  /* zugehoerigen Knoten.                                                    */

  LMPNet net;
  char   * P_name,
         * T_name;

#endif

  {
    TSEdge ts;

    if ( net && P_name && T_name && *P_name && *T_name )
      {
        /* suche passenden Verbindungsknoten in der Matrix */
        if (( ts = find_TP_connection( net, P_name, T_name ) ))
          {
            /* Verbindungsknoten vorhanden */
            if ( ! --ts->weight )
              {
                delete_TP_con_node( net, ts );
              }
          }
      }
    else
      {
        fprintf( stderr,
            "WARNING: NN.delete_PT_connection(): not a net or no P/T.\n" );
        return ( False );
      }
    return ( True );
  } /* delete_TP_connection() */


/*****************************************************************************/
/************************** delete_PT_connection *****************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

bool delete_PT_connection( LMPNet net, char * P_name, char * T_name )

#else

bool delete_PT_connection( net, P_name, T_name )
  /* loescht Verbindung von P_name zu T_name, reduziert weight im zugehoe-   */
  /* rigen Knoten.                                                           */

  LMPNet net;
  char   * P_name,
         * T_name;

#endif

  {
    STEdge st;

    if ( net && P_name && T_name && *P_name && *T_name )
      {
        /* suche passenden Verbindungsknoten in der Matrix */
        if (( st = find_PT_connection( net, P_name, T_name ) ))
          {
            /* Verbindungsknoten vorhanden */
            if ( ! --st->weight )
              {
                delete_PT_con_node( net, st );
              }
          }
      }
    else
      {
        fprintf( stderr,
            "WARNING: NN.delete_TP_connection(): not a net or no P/T.\n" );
        return ( False );
      }
    return ( True );
  } /* delete_TP_connection() */


/*****************************************************************************/
/************************* delete_place_direct *******************************/
/*****************************************************************************/
/* NEU: 17.10.1994 Stefan Roemer                                             */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Place delete_place_direct( LMPNet net, Place place )

#else

Place delete_place_direct( net, place )
  /* loescht die Zeile fuer die Stelle `pl` aus der Matrix                   */
  /* 'net'. Alle Verbindungsknoten werden geloescht und die Spalten entspre- */
  /* chend neu verzeigert.                                                   */
  /* Rueckgabe ist ein Zeiger auf die der geloeschten Stelle folgende Stelle.*/

  LMPNet net;
  Place  place;

#endif

  {
    register Place  pl;
    register STEdge st,
                    stdel;
    register TSEdge ts,
                    tsdel;

    place = ( pl = place )->nextplace;
    ts = pl->preset;
    while ( ts )
      {
        if ( ts->prevplace )
          {
            ts->prevplace->nextplace = ts->nextplace;
          }
        else
          {
            ts->trans->postset = ts->nextplace;
          }
        if ( ts->nextplace )
          {
            ts->nextplace->prevplace = ts->prevplace;
          }
        ts = ( tsdel = ts ) -> nexttrans;
        Free( tsdel );
      }
    st = pl->postset;
    while ( st )
      {
        if ( st->prevplace )
          {
            st->prevplace->nextplace = st->nextplace;
          }
        else
          {
            st->trans->preset = st->nextplace;
          }
        if ( st->nextplace )
          {
            st->nextplace->prevplace = st->prevplace;
          }
        st = ( stdel = st ) -> nexttrans;
        Free( stdel );
      }
    if ( pl->prevplace )
      {
        pl->prevplace->nextplace = pl->nextplace;
      }
    else
      {
        net->Places = pl->nextplace;
      }
    if ( pl->nextplace )
      {
        pl->nextplace->prevplace = pl->prevplace;
      }
    else
      {
        net->LastPlace = pl->prevplace;
      }
    Free( pl->name );
    if ( pl->inscript )
      {
	if ( CM_storeCoordinates || CM_storeReferences )
	{
	  Coords co,co2;
	  co2=(Coords)pl->inscript;
          co = co2->next;
	  free(co2);
          co = (co2=co)->next;
	  free(co2);
          co = (co2=co)->next;
	  free((char*)co2->x);
	  free((char*)co2->y);
	  free(co2);
          co = (co2=co)->next;
	  free((char*)co2->x);
	  free((char*)co2->y);
	  free(co2);
          co = (co2=co)->next;
	  free(co2);
	} else 
          free( pl->inscript );
      }
    free( pl );
    net->NumPlaces--;
    return ( place );
  } /* delete_place_direct() */


/*****************************************************************************/
/************************** delete_transition_direct *************************/
/*****************************************************************************/
/* NEU: 17.10.1994 Stefan Roemer                                             */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Transition delete_transition_direct( LMPNet net, Transition trans )

#else

Transition delete_transition_direct( net, trans )
  /* loescht die Spalte fuer die Transition `tr` aus der Matrix 'net'.       */
  /* Alle Verbindungknoten werden geloescht und die Zeilen entsprechend neu  */
  /* verzeigert.                                                             */
  /* Rueckgabe ist ein Zeiger auf die naechste Transition.                   */

  LMPNet     net;
  Transition trans;

#endif

  {
    register Transition tr;
    register STEdge     st,
                        stdel;
    register TSEdge     ts,
                        tsdel;

    trans = ( tr = trans )->nexttrans;
    st = tr->preset;
    while ( st )
      {
        if ( st->prevtrans )
          { st->prevtrans->nexttrans = st->nexttrans; }
        else
          { st->place->postset = st->nexttrans; }
        if ( st->nexttrans )
          { st->nexttrans->prevtrans = st->prevtrans; }
        st = ( stdel = st ) -> nextplace;
        Free( stdel );
      }
    ts = tr->postset;
    while ( ts )
      {
        if ( ts->prevtrans )
          { ts->prevtrans->nexttrans = ts->nexttrans; }
        else
          { ts->place->preset = ts->nexttrans; }
        if ( ts->nexttrans )
          { ts->nexttrans->prevtrans = ts->prevtrans; }
        ts = ( tsdel = ts ) -> nextplace;
        Free( tsdel );
      }
    if ( tr->prevtrans )
      { tr->prevtrans->nexttrans = tr->nexttrans; }
    else
      { net->Transitions = tr->nexttrans; }
    if ( tr->nexttrans )
      { tr->nexttrans->prevtrans = tr->prevtrans; }
    else
      { net->LastTransition = tr->prevtrans; }
    Free( tr->name );
    if ( tr->inscript )
      {
	if ( CM_storeCoordinates || CM_storeReferences )
	{
	  Coords co,co2;
	  co2=(Coords)tr->inscript;
          co = co2->next;
	  free(co2);
          co = (co2=co)->next;
	  free(co2);
          co = (co2=co)->next;
	  free((char*)co2->x);
	  free((char*)co2->y);
	  free(co2);
          co = (co2=co)->next;
	  free((char*)co2->x);
	  free((char*)co2->y);
	  free(co2);
	} else 
          free( tr->inscript );
      }
    if ( tr->time_label )
      {
        free( tr->time_label );
      }
    free( tr );
    net->NumTransitions--;
    return ( trans );
  } /* delete_transition_direct() */


/*****************************************************************************/
/****************************** delete_place *********************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

bool delete_place( LMPNet net, char * P_name )

#else

bool delete_place( net, P_name )
  /* loescht die Zeile fuer die Stelle mit Namen 'P_name' aus der Matrix     */
  /* 'net'. Alle Verbindungsknoten werden geloescht und die Spalten entspre- */
  /* chend neu verzeigert.                                                   */

  LMPNet net;
  char   * P_name;

#endif

  {
    Place pl;

    if ( net && P_name && *P_name )
      {
        if (( pl = find_P( net, P_name ) ))
          {
            (void) delete_place_direct( net, pl );
          } /* if */
        else
          {
            WARNING_P( "delete_place", P_name );
          } /* else */
      }
    else
      {
        WARNING_NO_NET_OR_P( "delete_place" );
        return ( False );
      } /* else */
    return ( True );
  } /* delete_place() */


/*****************************************************************************/
/****************************** delete_transition ****************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

bool delete_transition( LMPNet net, char * T_name )

#else

bool delete_transition( net, T_name )
  /* loescht die Spalte fuer die Transition 'T_name' aus der Matrix 'net'.   */
  /* Alle Verbindungknoten werden geloescht und die Zeilen entsprechend neu  */
  /* verzeigert.                                                             */

  LMPNet net;
  char   * T_name;

#endif

  {
    Transition tr;

    if ( net && T_name && *T_name )
      {
        if (( tr = find_T( net, T_name ) ))
          {
            (void) delete_transition_direct( net, tr );
          } /* if */
        else
          {
            WARNING_T( "delete_transition", T_name );
          } /* else */
      }
    else
      {
        WARNING_NO_NET_OR_T( "delete_transition" );
        return ( False );
      } /* else */
    return ( True );
  } /* delete_transition() */


/*****************************************************************************/
/************************** place_successor_number ***************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Cardinal32 place_successor_number( LMPNet net, char * P_name )

#else

Cardinal32 place_successor_number( net, P_name )
  /* zaehlt die Kanten im Nachbereich von 'P_name' */

  LMPNet net;
  char   * P_name;

#endif

  {
    Place      pl;
    STEdge     st;
    Cardinal32 number = 0;

    if ( net && P_name && *P_name )
      {
        if (( pl = find_P( net, P_name ) ))
          {
            for(st = pl->postset; st; st = st->nexttrans)
              {
                number += st->weight;
              }
          }
        else
          {
            WARNING_P( "place_successor_number", P_name );
          }
      }
    else
      {
        WARNING_NO_NET_OR_P( "place_successor_number" );
      }
    return( number );
  } /* place_successor_number() */

/*****************************************************************************/
/************************** trans_successor_number ***************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

/* zaehlt die Kanten im Nachbereich von 'T_name' */
#if defined(__STDC__) || defined(__cplusplus)

Cardinal32 trans_successor_number( LMPNet net, char * T_name )

#else

Cardinal32 trans_successor_number( net, T_name )

  LMPNet net;
  char   * T_name;

#endif

  {
    Transition tr;
    TSEdge     ts;
    Cardinal32 number = 0;

    if ( net && T_name && *T_name )
      {
        if (( tr = find_T( net, T_name ) ))
          {
            for(ts = tr->postset; ts; ts = ts->nextplace)
              {
                number += ts->weight;
              }
          }
        else
          {
            WARNING_T( "trans_successor_number", T_name );
          }
      }
    else
      {
        WARNING_NO_NET_OR_T( "trans_successor_number" );
      }
    return( number );
  } /* trans_successor_number() */

/*****************************************************************************/
/************************** place_predecessor_number *************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Cardinal32 place_predecessor_number( LMPNet net, char * P_name )

#else

Cardinal32 place_predecessor_number( net, P_name )
  /* zaehlt die Kanten im Vorbereich von 'P_name' */

  LMPNet net;
  char   * P_name;

#endif

  {
    Place      pl;
    TSEdge     ts;
    Cardinal32 number = 0;

    if ( net && P_name && *P_name )
      {
        if (( pl = find_P( net, P_name ) ))
          {
            ts = pl->preset;
            while ( ts )
              {
                number += ts->weight;
                ts = ts->nexttrans;
              } /* while */
          } /* if */
        else
          {
            WARNING_P( "place_predecessor_number", P_name );
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET_OR_P( "place_predecessor_number" );
      } /* else */
    return( number );
  } /* place_predecessor_number() */


/*****************************************************************************/
/************************** trans_predecessor_number *************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Cardinal32 trans_predecessor_number( LMPNet net, char * T_name )

#else

Cardinal32 trans_predecessor_number( net, T_name )
  /* zaehlt die Kanten im Vorbereich von 'T_name' */

  LMPNet net;
  char   * T_name;

#endif

  {
    Transition tr;
    STEdge     st;
    Cardinal32 number = 0;

    if ( net && T_name && *T_name )
      {
        if (( tr = find_T( net, T_name ) ))
          {
            st = tr->preset;
            while ( st )
              {
                number += st->weight;
                st = st->nextplace;
              } /* while */
          } /* if */
        else
          {
            WARNING_T( "trans_predecessor_number", T_name );
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET_OR_T( "trans_predecessor_number" );
      } /* else */
    return( number );
  } /* trans_predecessor_number() */


/*****************************************************************************/
/************************ place_successor_list *******************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/* Diese Funktion sollte nur selten eingesetzt werden, da die vorliegende    */
/* Datenstruktur einen direkten Zugriff auf den Nachbereich einer Stelle     */
/* erlaubt. ACHTUNG: DIE LISTE, DIE MIT DIESER FUNKTION ANGEFORDERT WIRD,    */
/* SOLLTE NACH IHRER "BENUTZUNG" AUCH WIEDER AUS DEM SPEICHER GELOESCHT      */
/* WERDEN ( durch Aufruf der Funktion 'remove_list()' ) !                    */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LIST_POINTER place_successor_list( LMPNet net, char * P_name )

#else

LIST_POINTER place_successor_list( net, P_name )
  /* liefert einen Pointer auf eine Liste mit den Namen der Elemente im      */
  /* Nachbereich der Stelle 'P_name'. Ist der Nachbereich von 'P_name' leer, */
  /* so wird ein nil-Zeiger zurueckgegeben. Die Liste enthaelt die Namen     */
  /* der Nachfolgetransitionen entsprechend oft der Staerke der Verbindung,  */
  /* die das jeweilige Kantengewicht angibt.                                 */

  LMPNet net;
  char   * P_name;

#endif

  {
    Place        pl;
    STEdge       st;
    LIST_POINTER list = NULL,
                 listptr;
    Cardinal32   counter;

    if ( net && P_name && *P_name )
      {
        if (( pl = find_P( net, P_name ) ))
          {
            if (( st = pl->postset ))
              {
                listptr = list = (LIST_POINTER) Malloc( sizeof(LIST_ELEMENT) );
                Strcpy( list->name, st->trans->name );
                counter = st->weight;
                while ( --counter )
                  {
                    listptr = listptr->next
                               = (LIST_POINTER) Malloc( sizeof(LIST_ELEMENT) );
                    Strcpy( listptr->name, st->trans->name );
                  } /* while */
                while (( st = st->nexttrans ))
                  {
                    counter = st->weight;
                    while ( counter-- )
                      {
                        listptr = listptr->next
                               = (LIST_POINTER) Malloc( sizeof(LIST_ELEMENT) );
                        Strcpy( listptr->name, st->trans->name );
                      } /* while */
                  } /* while */
                listptr->next = NULL;
              } /* if */
          } /* if */
        else
          {
            WARNING_P( "place_successor_list", P_name );
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET_OR_P( "place_successor_list" );
      } /* else */
    return ( list );
  } /* place_successor_list() */


/*****************************************************************************/
/************************ trans_successor_list *******************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/* Diese Funktion sollte nur selten eingesetzt werden, da die vorliegende    */
/* Datenstruktur einen direkten Zugriff auf den Nachbereich einer Transition */
/* erlaubt. ACHTUNG: DIE LISTE, DIE MIT DIESER FUNKTION ANGEFORDERT WIRD,    */
/* SOLLTE NACH IHRER "BENUTZUNG" AUCH WIEDER AUS DEM SPEICHER GELOESCHT      */
/* WERDEN ( durch Aufruf der Funktion 'remove_list()' ) !                    */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LIST_POINTER trans_successor_list( LMPNet net, char * T_name )

#else

LIST_POINTER trans_successor_list( net, T_name )
  /* liefert einen Pointer auf eine Liste mit den Namen der Elemente im      */
  /* Nachbereich der Transition 'P_name'. Ist der Nachbereich von 'T_name'   */
  /* leer, so wird ein nil-Zeiger zurueckgegeben. Die Liste enthaelt die     */
  /* Namen der Nachfolgestellen entsprechend oft der Staerke der Verbindung, */
  /* die das jeweilige Kantengewicht angibt.                                 */

  LMPNet net;
  char   * T_name;

#endif

  {
    Transition   tr;
    TSEdge       ts;
    LIST_POINTER list = NULL,
                 listptr;
    Cardinal32   counter;

    if ( net && T_name && *T_name )
      {
        if (( tr = find_T( net, T_name ) ))
          {
            if (( ts = tr->postset ))
              {
                listptr = list = (LIST_POINTER) Malloc( sizeof(LIST_ELEMENT) );
                Strcpy( list->name, ts->place->name );
                counter = ts->weight;
                while ( --counter )
                  {
                    listptr = listptr->next
                               = (LIST_POINTER) Malloc( sizeof(LIST_ELEMENT) );
                    Strcpy( listptr->name, ts->place->name );
                  } /* while */
                while (( ts = ts->nextplace ))
                  {
                    counter = ts->weight;
                    while ( counter-- )
                      {
                        listptr = listptr->next
                               = (LIST_POINTER) Malloc( sizeof(LIST_ELEMENT) );
                        Strcpy( listptr->name, ts->place->name );
                      } /* while */
                  } /* while */
                listptr->next = NULL;
              } /* if */
          } /* if */
        else
          {
            WARNING_T( "trans_successor_list", T_name );
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET_OR_T( "trans_successor_list" );
      } /* else */
    return ( list );
  } /* trans_successor_list() */


/*****************************************************************************/
/************************ place_predecessor_list *****************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/* Diese Funktion sollte nur selten eingesetzt werden, da die vorliegende    */
/* Datenstruktur einen direkten Zugriff auf den Vorbereich einer Stelle      */
/* erlaubt. ACHTUNG: DIE LISTE, DIE MIT DIESER FUNKTION ANGEFORDERT WIRD,    */
/* SOLLTE NACH IHRER "BENUTZUNG" AUCH WIEDER AUS DEM SPEICHER GELOESCHT      */
/* WERDEN ( durch Aufruf der Funktion 'remove_list()' ) !                    */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LIST_POINTER place_predecessor_list( LMPNet net, char * P_name )

#else

LIST_POINTER place_predecessor_list( net, P_name )
  /* liefert einen Pointer auf eine Liste mit den Namen der Elemente im      */
  /* Vorbereich der Stelle 'P_name'. Ist der Vorbereich von 'P_name' leer,   */
  /* so wird ein nil-Zeiger zurueckgegeben. Die Liste enthaelt die Namen     */
  /* der Vorgaengertransitionen entsprechend oft der Staerke der Verbindung, */
  /* die das jeweilige Kantengewicht angibt.                                 */

  LMPNet net;
  char   * P_name;

#endif

  {
    register Place        pl;
    register TSEdge       ts;
    register LIST_POINTER list = NULL,
                          listptr;
    register Cardinal32   counter;

    if ( net && P_name && *P_name )
      {
        if (( pl = find_P( net, P_name ) ))
          {
            if (( ts = pl->preset ))
              {
                listptr = list = (LIST_POINTER) Malloc( sizeof(LIST_ELEMENT) );
                Strcpy( list->name, ts->trans->name );
                counter = ts->weight;
                while ( --counter )
                  {
                    listptr = listptr->next
                               = (LIST_POINTER) Malloc( sizeof(LIST_ELEMENT) );
                    Strcpy( listptr->name, ts->trans->name );
                  } /* while */
                while (( ts = ts->nexttrans ))
                  {
                    counter = ts->weight;
                    while ( counter-- )
                      {
                        listptr = listptr->next
                               = (LIST_POINTER) Malloc( sizeof(LIST_ELEMENT) );
                        Strcpy( listptr->name, ts->trans->name );
                      } /* while */
                  } /* while */
                listptr->next = NULL;
              } /* if */
          } /* if */
        else
          {
            WARNING_P( "place_predecessor_list", P_name );
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET_OR_P( "place_predecessor_list" );
      } /* else */
    return ( list );
  } /* place_predecessor_list() */

/*****************************************************************************/
/************************ trans_predecessor_list *****************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/* Diese Funktion sollte nur selten eingesetzt werden, da die vorliegende    */
/* Datenstruktur einen direkten Zugriff auf den Vorbereich einer Transition  */
/* erlaubt. ACHTUNG: DIE LISTE, DIE MIT DIESER FUNKTION ANGEFORDERT WIRD,    */
/* SOLLTE NACH IHRER "BENUTZUNG" AUCH WIEDER AUS DEM SPEICHER GELOESCHT      */
/* WERDEN ( durch Aufruf der Funktion 'remove_list()' ) !                    */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LIST_POINTER trans_predecessor_list( LMPNet net, char * T_name )

#else

LIST_POINTER trans_predecessor_list( net, T_name )
  /* liefert einen Pointer auf eine Liste mit den Namen der Elemente im      */
  /* Vorbereich der Transition 'P_name'. Ist der Vorbereich von 'T_name'     */
  /* leer, so wird ein nil-Zeiger zurueckgegeben. Die Liste enthaelt die     */
  /* Namen der Vorgaengerstellen entsprechend oft der Staerke der Verbindung,*/
  /* die das jeweilige Kantengewicht angibt.                                 */

  LMPNet net;
  char   * T_name;

#endif

  {

    register Transition   tr;
    register STEdge       st;
    register LIST_POINTER list = NULL,
                          listptr;
    register Cardinal32   counter;

    if ( net && T_name && *T_name )
      {
        if (( tr = find_T( net, T_name ) ))
          {
            if (( st = tr->preset ))
              {
                listptr = list = (LIST_POINTER) Malloc( sizeof(LIST_ELEMENT) );
                Strcpy( list->name, st->place->name );
                counter = st->weight;
                while ( --counter )
                  {
                    listptr = listptr->next
                               = (LIST_POINTER) Malloc( sizeof(LIST_ELEMENT) );
                    Strcpy( listptr->name, st->place->name );
                  } /* while */
                while (( st = st->nextplace ))
                  {
                    counter = st->weight;
                    while ( counter-- )
                      {
                        listptr = listptr->next
                               = (LIST_POINTER) Malloc( sizeof(LIST_ELEMENT) );
                        Strcpy( listptr->name, st->place->name );
                      } /* while */
                  } /* while */
                listptr->next = NULL;
              } /* if */
          } /* if */
        else
          {
            WARNING_T( "trans_predecessor_list", T_name );
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET_OR_T( "trans_predecessor_list" );
      } /* else */
    return ( list );
  } /* trans_predecessor_list() */


/*****************************************************************************/
/*************************** TP_connection ***********************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Cardinal32 TP_connection( LMPNet net, char * P_name, char * T_name )

#else

Cardinal32 TP_connection( net, P_name, T_name )
  /* liefert die Staerke der Verbindung zwischen Transition T_name und       */
  /* Stelle P_name.                                                          */

  LMPNet net;
  char   * P_name,
         * T_name;

#endif

  {

    register TSEdge ts;

    if ( net && P_name && T_name && *P_name && *T_name )
      {
        if (( ts = find_TP_connection( net, P_name, T_name ) ))
          {
            return( ts->weight );
          } /* if */
      } /* if */
    else
      {
        fprintf( stderr,
                 "WARNING: NN.TP_connection(): not a net or no P/T name.\n" );
      } /* else */
    return( 0 );
  } /* end of function TP_connection */

/*****************************************************************************/
/*************************** PT_connection ***********************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

/* liefert die Staerke der Verbindung zwischen Stelle P_name und Transi-   */
/* tion T_name.                                                            */
#if defined(__STDC__) || defined(__cplusplus)

Cardinal32 PT_connection( LMPNet net, char * P_name, char * T_name )

#else

Cardinal32 PT_connection( net, P_name, T_name )

  LMPNet net;
  char   * P_name,
         * T_name;

#endif

  {
    register STEdge st;

    if ( net && P_name && T_name && *P_name && *T_name )
      {
        if (( st = find_PT_connection( net, P_name, T_name ) ))
          {
            return ( st->weight );
          } /* if */
      } /* if */
    else
      {
        fprintf( stderr,
                 "WARNING: NN.PT_connection(): not a net or no P/T name.\n" );
      } /* else */
    return( 0 );
  } /* PT_connection() */


/*****************************************************************************/
/*************************** name_free ***************************************/
/*****************************************************************************/
/* KONVERSION: 06.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

/* liefert True zurueck, falls 'name' noch nicht vergeben; False sonst.      */
#if defined(__STDC__) || defined(__cplusplus)

bool name_free( LMPNet net, char * name )

#else

bool name_free( net, name )

  LMPNet net;
  char   * name;

#endif

  {
    return ( (bool) ! ( find_P( net, name ) || find_T( net, name ) ) );
  } /* name_free */

/*****************************************************************************/
/*                                 Markierungen                              */
/*****************************************************************************/

/*****************************************************************************/
/******************************** marking ************************************/
/*****************************************************************************/
/* KONVERSION: 10.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

/* liefert die Startmarkierung der Stelle mit Namen 'P_name'                 */
#if defined(__STDC__) || defined(__cplusplus)

Cardinal32 marking( LMPNet net, char * P_name )

#else

Cardinal32 marking( net, P_name )

  LMPNet net;
  char   * P_name;

#endif

  {
    Place pl;

    if ( net && P_name && *P_name )
      {
        /* Netz vorhanden */
        if (( pl = find_P( net, P_name ) ))
          {
            return ( pl->marks );
          } /* if */
        else
          {
            WARNING_P( "marking", P_name );
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET_OR_P( "marking" );
      } /* else */
    return ( 0 );
  } /* marking() */


/*****************************************************************************/
/***************************** size_of_marking *******************************/
/*****************************************************************************/
/* KONVERSION: 10.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Cardinal32 size_of_marking( MARKING marking )

#else

Cardinal32 size_of_marking( marking )
  /* liefert die Anzahl der Stellen in der Markierungsliste                  */

  MARKING marking;

#endif

  {
    Cardinal32 places = 0;

    while ( marking )
      {
        places++;
        marking = marking->next;
      } /* while */
    return( places );
  } /* size_of_marking() */

/*****************************************************************************/
/***************************** change_place_marking **************************/
/*****************************************************************************/
/* KONVERSION: 10.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

bool change_place_marking( LMPNet net, char * P_name, Cardinal32 value )

#else

bool change_place_marking( net, P_name, value )
  /* aendert die Markierung der einzelnen Stelle mit Namen 'P_name' in der   */
  /* Matrix 'net' auf den Wert 'value'.                                      */

  LMPNet     net;
  char       * P_name;
  Cardinal32 value;

#endif

  {
    Place pl;

    if ( net && P_name && *P_name )
      {
        if (( pl = find_P( net, P_name ) ))
          {
            pl->marks = value;
            return ( True );
          } /* if */
        else
          {
            WARNING_P( "change_place_marking", P_name );
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET_OR_P( "change_place_marking" );
      } /* else */
   return ( False );
  } /* change_place_marking() */

/*****************************************************************************/
/***************************** copy_start_marking ****************************/
/*****************************************************************************/
/* KONVERSION: 10.05.1993 Stefan Roemer                                      */
/* ACHTUNG: DIE LISTE MUSS NACH IHRER BENUTZUNG AUS DEM SPEICHER ENTFERNT    */
/* WERDEN ( mit 'remove_marklist()' ) !                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

MARKING copy_start_marking( LMPNet net )

#else

MARKING copy_start_marking( net )
  /* liefert eine Liste mit der Startmarkierung der Matrix 'net'.            */

  LMPNet net;       /* Matrix */

#endif

  {

    /* Deklarationen */
    Place   pl;
    MARKING list,
            mhelp = NULL;

    if ( net )
      {
        if (( pl = net->Places ))
          {
            mhelp = list = (MARKING) Malloc( sizeof( MARK_LIST ) );
            Strcpy( list->name, pl->name );
            list->value = pl->marks;
            /* baue Liste mit Hilfe von 'list' auf, merke die gesamte    */
            /* Liste unter 'mhelp'                                       */
            while (( pl = pl->nextplace ))
              {
                ( list = list->next = (MARKING) Malloc( sizeof( MARK_LIST ) ) )
                                                             -> value = pl->marks;
                Strcpy( list->name, pl->name );
              } /* while */
            list->next = NULL;
          } /* end of if */
        else
          {
            WARNING_NO_P( "copy_start_marking" );
          } /* end of else */

      } /* if */
    else
      {
        WARNING_NO_NET( "copy_start_marking" );
      } /* else */
    return( mhelp );
  } /* copy_start_marking() */

/*****************************************************************************/
/*************************** change_start_marking ****************************/
/*****************************************************************************/
/* KONVERSION: 10.05.1993 Stefan Roemer                                      */
/* ACHTUNG: DIE LISTE MUSS NACH IHRER BENUTZUNG AUS DEM SPEICHER ENTFERNT    */
/* WERDEN ( mit 'remove_marklist()' ) !                                      */
/*****************************************************************************/

/* setzt 'marking' als neue Startmarkierung in die Matrix 'net' ein */
#if defined(__STDC__) || defined(__cplusplus)

bool change_start_marking( LMPNet net, MARKING marking )

#else

bool change_start_marking( net, marking )

  LMPNet  net;
  MARKING marking;

#endif

  {
    register bool success = False;

    if ( net )
      {
        if ( net->NumPlaces == size_of_marking( marking ) )
          {
            if ( net->NumPlaces )
              {
                success = True;
                while ( marking )
                  {
                    if ( ! change_place_marking( net, marking->name,
                                                            marking->value ) )
                      {
                    fprintf( stderr, "WARNING: NN.change_start_marking(): " );
      fprintf( stderr, "number of marks of place '%s' has not been changed.\n",
                                 marking->name);
                        success = False;
                      } /* if */
                    marking = marking->next;
                  } /* end of while */
              } /* end of if */
          } /* if */
        else
          {
            /* Fehler: Markierungsliste zu kurz oder zu lang */
            fprintf( stderr,
                "WARNING: NN.change_start_marking(): marking list does not " );
            fprintf( stderr, "fit to the number of places.\n" );
          } /* else */
      } /* if */
    else
      {
        WARNING_NO_NET( "change_start_marking" );
      } /* else */
    return( success );
  } /* change_start_marking() */

/*****************************************************************************/
/* END OF FILE neunetze.c                                                    */
/*****************************************************************************/
