#include "convertPEP.h"
#include <string.h>

unsigned int flags;

#define TIME_NET 1
#define DURATION_NET 2

/*****************************************************************************/
/***************************** write_elan ************************************/
/*****************************************************************************/

#   define f( STRING ) fprintf( file, STRING )
#   define fx( STRING, X ) fprintf( file, STRING, X )

/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void write_ELAN( LMPNet net, char * filename )

#else

void write_ELAN( net, filename )

  LMPNet net;
  char   * filename;

#endif

  {

    register Place      pl;
    register Transition tr;
    register STEdge     st;
    register TSEdge     ts;
    register int   counter;
    FILE            * file;

    /* places and transitions have to be numbered */
    num_net_nodes(net);

    /* open output file */
    if ( ! ( file = fopen( filename, "w" ) ) )
      {
        PEPerror( "Could not open file for writing" );
      } /* if */

    /* write the header */
    fx( "module %s\n\n",strtok(filename,".") );
    f( "import global\n         PNbasic;\nend\n\n" );
    f( "rules for conf\n    pn, pn2 : pnet;\n");
    f( "    lt      : list[trans];\n" );
    if (flags&(TIME_NET|DURATION_NET))
      f("    n,m,e,l : int;\n");
    f( "\nglobal\n\n");

    /* For each transition print rewriting rule(s) */
    for ( tr = net->Transitions ; tr ; tr = tr->nexttrans )
      {
	fx( "//** Transition rule(s) for Transition T%u **//\n",tr->numname );

	/* fire rule rsp. token output rule */
	f("[tr]     <");
	if (!(flags&DURATION_NET))
	  {
	    for ( st = tr->preset; st; st=st->nextplace )
	      {
		fx( "p(%u) # ",st->place->numname );
	      }
	  }
	if (flags&TIME_NET)
	  fx("t(%u)[m,e,l] # ",tr->numname);
	if (flags&DURATION_NET)
	  fx("t(%u)[m,e,l,true] # ",tr->numname);
	f("pn>[lt] => <");
	if (flags&TIME_NET)
	  fx("t(%u)[0,e,l] # ",tr->numname);
	if (flags&DURATION_NET)
	  fx("t(%u)[0,e,l,false] # ",tr->numname);
	for ( ts = tr->postset ; ts ; ts = ts->nextplace )
	  {
	    fx( "p(%u) # ", ts->place->numname );
	  } /* for */
	fx( "pn>[t(%u).lt]", tr->numname );
	if (flags&(TIME_NET|DURATION_NET))
	  f("\n        if m >= e and m <=l   ");
	else
	  f("   ");
	f("end\n\n");

	/* time step for duration nets */
	if (flags&DURATION_NET)
	  {
	    fx("[tr]     <t(%u)[m,e,l,true] # pn>[lt] => ",tr->numname);
	    fx("<T(t(%u)[m+1,e,l,true]) # pn>[lt]\n",tr->numname);
	    f("        if m < 1    end\n\n");
	  }

	/* time step for time nets */
	/* consume token for duration nets */
	if (flags&(TIME_NET|DURATION_NET))
	  {
	    f("[tr]     <");
	    for ( st = tr->preset; st; st=st->nextplace )
	      {
		fx( "p(%u) # ",st->place->numname );
	      }
	    if (flags&TIME_NET)
	      fx("t(%u)[m,e,l] # ",tr->numname);
	    else
	      fx("t(%u)[0,e,l,false] # ",tr->numname);
	    f("pn>[lt] => <");
	    if (flags&TIME_NET)
	      fx("T(t(%u)[m+1,e,l]) # ",tr->numname);
	    else
	      fx("t(%u)[0,e,l,true] # ",tr->numname);
	    if (flags&TIME_NET)
	      {
		for ( ts = tr->postset ; ts ; ts = ts->nextplace )
		  {
		    fx( "p(%u) # ", ts->place->numname );
		  } /* for */
		fx( "pn>[t(%u).lt]\n", tr->numname );
		f( "        if m < l   end\n\n");
	      }
	    else
	      f( "pn>[lt]  end\n\n" );
	  } /* if TIME_NET */
      } /* for tr */

    f( "end\n\nend\n" );

    fclose( file );

  } /* write_ELAN() */

static void show_usage()
{
  printf( "Transform PEP netfiles into ELAN modules\n" );
  printf( "usage: pep2elan [-d] [-t] <PEP netfile> <Elan module>\n");
  printf( "       -d:  generate rules for Duration nets\n");
  printf( "       -t:  generate rules for Time Petri nets\n");
  exit ( 1 );
} /* usage() */

int main( int argc, char **argv )
{
  int i;
  LMPNet net; 
  char *netfile=NULL;
  char *elanfile=NULL;

  flags=0;
  if ( (argc<3) || (argc >5) )
    show_usage();

  for (i=1; i<argc; i++)
    {
      if ( argv[i][0]=='-' )
	{
	  if (argv[i][1]=='t')
	    flags|=TIME_NET;
	  else if (argv[i][1]=='d')
	    flags|=DURATION_NET;
	  else
	    show_usage();
	}
      else if (!netfile)
	netfile=argv[i];
      else if (!elanfile)
	elanfile=argv[i];
      else
	show_usage();
    }
  
  net = read_PEP( netfile );
  write_ELAN( net, elanfile );

  remove_net( &net );
  
  exit ( 0 );
  
} /* main() */
