/*****************************************************************************/
/* Defines the layout of a PEP net file for use with read_PEP_file.          */
/*****************************************************************************/

#ifndef __PEPNET_H__
#define __PEPNET_H__

/* Allowable types of nets. */
char *type_hlnet[] = { "HLNet", NULL };
char *type_llnet[] = { "PetriBox", "PTNet", NULL };
char *type_pepnet[] = { "HLNet", "PetriBox", "PTNet", NULL };

/* Defaults for blocks/places/transitions. */
t_fieldinfo nodedefs[] =
	{ { 'n', FT_COORDS },		/* rel pos of name        */
	  { 'a', FT_COORDS },		/* rel pos of meaning     */
	  { 's', FT_NUMBER },		/* object size            */
	  { 't', FT_NUMBER },		/* line weight            */
	  {  0 ,  0 } };

/* Defaults for arcs. */
t_fieldinfo arcdefs[] =
	{ { 'n', FT_COORDS },		/* rel pos of name        */
	  { 't', FT_NUMBER },		/* line weight            */
	  { 'w', FT_NUMBER },		/* default weight         */
	  {  0 ,  0 } };

/* Data fields for blocks. */
t_fieldinfo blockfields[] =
	{ { '\'',FT_STRING },		/* identifier             */
	  { '"', FT_STRING },		/*     "		  */
	  { '@', FT_COORDS },		/* coordinates		  */
	  { '0', FT_NUMBER },		/* numeric identifier     */
	  { 'n', FT_COORDS },		/* rel pos of name        */
	  { 'N', FT_COORDS },		/* abs pos of name        */
	  { 'b', FT_STRING },		/* meaning  		  */
	  { 'a', FT_COORDS },		/* rel pos of meaning     */
	  { 'A', FT_COORDS },		/* abs pos of meaning     */
	  { 'R', FT_STRING },		/* reference		  */
	  { 'T', FT_STRING },		/* BPN->HL ref (obsolete) */
	  { 'u', FT_STRING },		/* block list		  */
	  {  0 ,  0 } };

/* Data fields for places. */
t_fieldinfo placefields[] =
	{ { '\'',FT_STRING },		/* identifier             */
	  { '"', FT_STRING },		/*     "		  */
	  { '@', FT_COORDS },		/* coordinates		  */
	  { '0', FT_NUMBER },		/* numeric identifier     */
	  { 'n', FT_COORDS },		/* rel pos of name	  */
	  { 'N', FT_COORDS },		/* abs pos of name	  */
	  { 'b', FT_STRING },		/* meaning		  */
	  { 'a', FT_COORDS },		/* rel pos of meaning     */
	  { 'A', FT_COORDS },		/* abs pos of meaning     */
	  { 'M', FT_NUMBER },		/* initial marking (int)  */
	  { 'm', FT_NUMBER },		/* current marking (int)  */
	  { 'u', FT_STRING },		/* block list		  */
	  { 'Z', FT_STRING },		/* type			  */
	  { 'B', FT_COORDS },		/* rel pos of type	  */
	  { 'z', FT_STRING },		/* initial marking (str)  */
	  { 'y', FT_STRING },		/* current marking (str)  */
	  { 'R', FT_STRING },		/* reference		  */
	  { 'k', FT_NUMBER },		/* capacity  		  */
	  { 'e', FT_FLAG   },		/* entry place 		  */
	  { 'x', FT_FLAG   },		/* exit place 		  */
	  { 'v', FT_NUMBER },		/* flags		  */
	  { 's', FT_NUMBER },		/* object size  	  */
	  { 't', FT_NUMBER },		/* line weight  	  */
	  { 'c', FT_NUMBER },		/* colour  		  */
	  { 'T', FT_STRING },		/* BPN->HL ref (obsolete) */
	  {  0 ,  0 } };
 
/* Data fields for transitions. */
t_fieldinfo transfields[] =
	{ { 'S', FT_FLAG   },		/* synch transition       */
	  { '\'',FT_STRING },		/* identifier		  */
	  { '"', FT_STRING },		/*     "		  */
	  { '@', FT_COORDS },		/* coordinates		  */
	  { '0', FT_NUMBER },		/* numeric identifier     */
	  { 'n', FT_COORDS },		/* rel pos of name	  */
	  { 'N', FT_COORDS },		/* abs pos of name	  */
	  { 'b', FT_STRING },		/* meaning                */
	  { 'a', FT_COORDS },		/* rel pos of meaning     */
	  { 'A', FT_COORDS },		/* abs pos of meaning     */
	  { 'v', FT_NUMBER },		/* flags		  */
	  { 'u', FT_STRING },		/* block list		  */
	  { 'P', FT_STRING },		/* list of phantom trans  */
	  { 'g', FT_STRING },		/* value term		  */
	  { 'h', FT_COORDS },		/* rel pos of value term  */
	  { 'H', FT_COORDS },		/* rel pos of value term  */
	  { 'R', FT_STRING },		/* reference		  */
	  { 'i', FT_STRING },		/* action terms		  */
	  { 'j', FT_COORDS },		/* rel pos of action terms*/
	  { 's', FT_NUMBER },		/* object size  	  */
	  { 't', FT_NUMBER },		/* line weight  	  */
	  { 'c', FT_NUMBER },		/* colour  		  */
	  { 'T', FT_STRING },		/* BPN->HL ref (obsolete) */
	  { 'r', FT_FLAG   },		/* refined flag 	  */
	  { 'E', FT_DOUBLE },		/* Eft			  */
	  { 'L', FT_DOUBLE },		/* Lft			  */
	  { 'l', FT_STRING },		/* link terms */
	  { 'k', FT_COORDS },		/* rel pos of link terms */
	  {  0 ,  0 } };
 
/* Data fields for arcs. */
t_fieldinfo arcfields[] =
	{ { '@', FT_COORDS },		/* source/dest    	  */
	  { 'J', FT_COORDS },		/* coordinates     	  */
	  { ',', FT_COORDS },		/* more coordinates       */
	  { 'w', FT_NUMBER },		/* weight		  */
	  { 'n', FT_COORDS },		/* rel pos weight 	  */
	  { 'N', FT_COORDS },		/* abs pos weight 	  */
	  { 'p', FT_STRING },		/* inscription  	  */
	  { 'q', FT_COORDS },		/* rel pos of inscription */
	  { 'Q', FT_COORDS },		/* abs pos of inscription */
	  { 'v', FT_NUMBER },		/* visibility  		  */
	  { 't', FT_NUMBER },		/* line weight    	  */
	  { 'c', FT_NUMBER },		/* colour  		  */
	  {  0 ,  0 } };

/* Data fields for texts. */
t_fieldinfo textfields[] =
	{ { '\'',FT_STRING },		/* text			  */
	  { '"', FT_STRING },		/*   "			  */
	  { 'N', FT_COORDS },		/* absolute position	  */
	  {  0 ,  0 } };

/* empty */
t_fieldinfo emptyfield[] =
    { { 0 , 0 } };

/* This structure describes the layout of a file by listing all the
   data blocks such a file might comprise. More details about the
   blocks are given by providing an array of data field descritions
   for each block.

   Each entry consists of four fields (t_blockinfo is defined in readPEP.c):
      name            the identifier that starts the block
      optional        whether a block is mandatory or optional (an error
                      is displayed if a mandatory block is not present)
      line            whether the block consists of one single line or
                      of a sequence of lines, each defining one object
      field           an array of t_fieldinfo structs defining the data
                      fields that might occur in the block              */
t_blockinfo netblocks[] =
	{ { "DBL", TB_OPTIONAL,  TB_LINE,  nodedefs },
	  { "DPL", TB_OPTIONAL,  TB_LINE,  nodedefs },
	  { "DTR", TB_OPTIONAL,  TB_LINE,  nodedefs },
	  { "DPT", TB_OPTIONAL,  TB_LINE,  arcdefs },
	  { "BL",  TB_OPTIONAL,  TB_BLOCK, blockfields },
	  { "LS",  TB_OPTIONAL,  TB_BLOCK, emptyfield },
	  { "PL",  TB_MANDATORY, TB_BLOCK, placefields },
	  { "TR",  TB_MANDATORY, TB_BLOCK, transfields },
	  { "PTR", TB_OPTIONAL,  TB_BLOCK, transfields+1 },
	  { "TP",  TB_MANDATORY, TB_BLOCK, arcfields },
	  { "PT",  TB_MANDATORY, TB_BLOCK, arcfields },
	  { "PTP", TB_OPTIONAL,  TB_BLOCK, arcfields },
	  { "PPT", TB_OPTIONAL,  TB_BLOCK, arcfields },
	  { "TX",  TB_OPTIONAL,  TB_BLOCK, textfields },
	  { NULL, 0, 0, NULL } };

#endif

