/*****************************************************************************/
/* readPEP.c                                                                 */
/*****************************************************************************/
/*                                  Stefan Roemer,  06.02.1995 - 06.02.1995  */
/*                                  Stefan Schwoon, April 1997		     */
/*****************************************************************************/
/* Generic function for reading a file in PEP format. The function           */
/* read_PEP_net is used in readHL.c.                                         */
/*****************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "readPEP.h"
#include "readlib.h"
#include "memlib.h"

/*****************************************************************************/
/* Global variables: File and block identifier.                              */
/*****************************************************************************/

char *filetype;
char *blocktype;


/*****************************************************************************/

#define FT_STRING 1
#define FT_COORDS 2
#define FT_NUMBER 3
#define FT_FLAG	  4
#define FT_DOUBLE 5

typedef struct
{
	char c;
	int type;
} t_fieldinfo;

#define TB_MANDATORY 0
#define TB_OPTIONAL  1
#define TB_BLOCK     0
#define TB_LINE	     1

typedef struct
{
	char *name;
	int  optional;
	int  line;
	t_fieldinfo *field;
} t_blockinfo;

typedef struct
{
	int type;
	void *ptr;
} t_lookup;

/*****************************************************************************/
/* read_PEP_file                                                             */
/* This is a function for reading a file in PEP's general layout, i.e. it's  */
/* not restricted to nets (though in practice that's the only thing we'll    */
/* use it for). blocks is a data structure telling us how the layout of a    */
/* file should look like, and dest tells us what we should do with the data  */
/* we find (what and where to store it).                                     */

#define RL_ReadCmdToken ReadCmdToken

void read_PEP_file(char *filename, char **types,
		   t_blockinfo *blocks, t_blockdest *dest)
{
	FILE *infile;
	t_lookup *tbl;
	int i,len=0;
	int ch;
	int num,num2;
	double floatnum;
	t_fieldinfo *fld;
	t_dest *dst;
	char *restptr, *rtmp;
	int ralloc;
	t_blockdest *sdest = dest;

	NNinput_file = filename;
	NNinput_line = 1;

	/* Open the file, read the header. */
	if (!(infile = fopen(filename, "r")))
		Error((errbuf, "could not open file for reading"));

	RL_ReadCmdToken(infile);
	if (strcmp(sbuf, "PEP"))
		Error((errbuf, "keyword `PEP' expected"));

	/* Check if the file's type (second line of file) is one of those
	   that are allowed. */
	ReadNewline(infile);
	RL_ReadCmdToken(infile);
	for (; *types && strcmp(sbuf,*types); types++);
	if (!*types) Error((errbuf, "unexpected format identifier `%s'", sbuf));
	filetype = MYstrdup(sbuf);
	
	ReadNewline(infile);
	RL_ReadCmdToken(infile);
	if (strncmp(sbuf, "FORMAT_N", 8))
		Error((errbuf, "keyword `FORMAT_N' or `FORMAT_N2' expected"));

	tbl = (t_lookup*) MYmalloc(sizeof(t_lookup)*(128-' ')) - ' ';
	ReadNewline(infile);

	while (!feof(infile))
	{
		/* Read next block id. */
		RL_ReadCmdToken(infile);

		blocktype = MYstrdup(sbuf);

		/* Identify block. */
		for (; blocks->name && strcmp(blocks->name,sbuf); blocks++)
		    if (!blocks->optional)
			Error((errbuf, "keyword '%s' expected", blocks->name));

		if (!blocks->name) Error((errbuf, "unknown keyword `%s'", sbuf));

		for (dest = sdest; dest->name; dest++)
			if (!strcmp(blocks->name,dest->name)) break;

		/* Set up tbl. */
		for (i=' '; i<128; tbl[i++].type = 0);
		for (i=' '; i<128; tbl[i++].ptr = NULL);

		for (fld = blocks->field; fld->c; fld++)
			tbl[(int)fld->c].type = fld->type;

		if (!blocks->line) ReadNewline(infile);

        /* Special: Read Link symbols.
           Read and eventualy stock into block->restptr */
                if ( strcmp( (blocks->name), "LS" ) == 0)
                  {
                    if (!feof(infile))
                          {fscanf(infile,"%s\n",sbuf); NNinput_line++;}
                    while (!feof(infile) && sbuf && strchr( sbuf, '"' )) {
                       if (dest -> name)
                       /* if  LS is in the set of destination blocks */
                             {
                               *(dest ->restptr) = sbuf;
                               dest->hookfunc(); // call the function which takes this new link symbol
                             };
                       ch = ReadCharComment(infile);
                       ungetc( ch, infile );
                       fscanf(infile,"%s\n",sbuf);
                       NNinput_line++;
                     }; /* while (!feof(infile) && cmd && !strchr( cmd, '"' )) */
                    /* re-put the last read line into infile */
                    ungetc( '\n', infile );
                    for (i=(strlen(sbuf) - 1);i>-1;i--)
                     {
                       ch = sbuf[i];
                       /* printf ("\tch = '%c'\n",ch); */
                       ungetc( ch , infile );
                     }
                    NNinput_line --;
                  }; /* if ( strcmp( cmd, "LS" ) ) */


		/* Read lines until next block begins. Every line yields one
		   new entity of the type determined by the current block. */
		for(;;)
		{
		    if (isupper(ch = ReadCharComment(infile)))
		    {
			int ch2 = getc(infile);
			ungetc(ch2,infile);
		        /* We assume that uppercase letters at the start of
		           a line always indicate a new block. */
			if (isupper(ch2)) break;
		    }
		    if (feof(infile)) break;
		    if (ch == '\n') continue;

		    *(rtmp = restptr = (char*)MYmalloc(ralloc = 64)) = '\0';

		    /* If information about this block is wanted, take
		       the information where to store data from dest. */
		    if (dest->name)
			for (dst = dest->destarray; dst->c; dst++)
			{
			    t_lookup *l = tbl + dst->c;
			    l->ptr = dst->ptr;
			    switch(l->type)
			    {
				case FT_STRING:
				case FT_COORDS:
					*(char**)(l->ptr) = NULL;
					break;
				case FT_NUMBER:
				case FT_FLAG:
					*(int*)(l->ptr) = 0;
					break;
				case FT_DOUBLE:
					*(double*)(l->ptr) = 0;
					break;
				default:
					Error((errbuf, "internal error: don't "
					"know the type of field `%c'", dst->c));
				break;
			    }
			}

		    /* Parse until end of line. We assume that all information
		       about an entity (place, transition etc.) is stored in
		       one single line. */
		    while (ch != '\n')
		    {
			if (isdigit(ch) || ch == '-')
			{
				/* Numbers are treated specially.
				   x@y gets stored in the '@' field, and
				   plain numbers are stored in the '0' field. */
				ungetc(ch,infile);
				num=ReadNumber(infile);
				if ((ch=ReadWhiteSpace(infile)) == '@'
						  || ch == '<' || ch == '>')
				{
					ch = '@';
					num2=ReadNumber(infile);
					len = 24;
				}
				else
				{
					ungetc(ch,infile);
					ch = '0';
					len = 12;
				}
			}
			else
			{
				if (ch == '\'' || ch == '"') ungetc(ch,infile);
				switch(tbl[ch].type)
				{
				    case FT_STRING:
					RL_ReadEnclString(infile);
					len = 3+strlen(sbuf);
					break;
				    case FT_DOUBLE:
					floatnum=ReadDouble(infile);
					len = 12;
					break;
				    case FT_NUMBER:
					num=ReadNumber(infile);
					len = 12;
					break;
				    case FT_COORDS:
					ReadCoordinates(infile, &num, &num2);
					len = 24;
					break;
				    case FT_FLAG:
					len = 1;
					break;
				    default:
					Error((errbuf,"unknown token `%c'",ch));
					break;
				}
			}

			/* Store data in the memory locations specified by
			   the dest array. If no location is given for a
			   particular field, its contents are written to
			   a 'rest' string. */
			while (!tbl[ch].ptr && rtmp-restptr+len >= ralloc)
			{
				restptr = (char*)MYrealloc(restptr, ralloc += 64);
				rtmp = restptr + strlen(restptr);
			}
			switch(tbl[ch].type)
			{
			    case FT_STRING:
				if (tbl[ch].ptr)
					*(char**)(tbl[ch].ptr) = MYstrdup(sbuf);
				else if (strchr("'\"",ch))
					sprintf(rtmp,"\"%s\"",sbuf);
				else
					sprintf(rtmp,"%c\"%s\"",ch,sbuf);
				break;
			    case FT_NUMBER:
				if (tbl[ch].ptr)
					*(int*)(tbl[ch].ptr) = num;
				else if (ch == '0')
					sprintf(rtmp,"%d ",num);
				else
					sprintf(rtmp,"%c%d",ch,num);
				break;
			    case FT_DOUBLE:
				if (tbl[ch].ptr)
					*(double*)(tbl[ch].ptr) = floatnum;
				else if (ch == '0')
					sprintf(rtmp,"%.0f ",floatnum);
				else
					sprintf(rtmp,"%c%.0f",ch,floatnum);
				break;
			    case FT_COORDS:
				if (tbl[ch].ptr)
				{
				    *((t_coords**)(tbl[ch].ptr)) =
					(t_coords *)MYmalloc(sizeof(t_coords));
				    (*(t_coords**)(tbl[ch].ptr))->x = num;
				    (*(t_coords**)(tbl[ch].ptr))->y = num2;
				}
				else if (ch == '@')
					sprintf(rtmp,"%d@%d ",num,num2);
				else
					sprintf(rtmp,"%c%d@%d",ch,num,num2);
				break;
			    case FT_FLAG:
				if (tbl[ch].ptr)
					(*(int*)(tbl[ch].ptr))++;
				else
					sprintf(rtmp,"%c",ch);
				break;
			}
			if (!tbl[ch].ptr) rtmp += strlen(rtmp);
			ch = ReadCharComment(infile);
		    } /* end of while */

		    /* When the whole line has been read, see if information
		       about the current block is wanted - if so, call the
		       corresponding hook function. */
		    if (dest->name)
		    {
			if (dest->restptr) *(dest->restptr) = restptr;
			if (dest->hookfunc()) Error((errbuf, "read aborted"));
		    }
		    else
			free(restptr);

		    NNinput_line++;

		}

		free(blocktype);
		if (!feof(infile)) ungetc(ch,infile);
		blocks++;

	}

	/* Check for mandatory blocks that didn't occur in the file. */
	for (; blocks->name; blocks++)
	    if (!blocks->optional)
		Error((errbuf, "section `%s' not found", blocks->name));

	free(tbl+' ');
	fclose(infile);

}

/*****************************************************************************/
/* read_PEP_net                                                              */
/* Read a PEP net. This is accomplished by using the more general function   */
/* read_PEP_file with the net layout data given in pepnet.h. dest specifies  */
/* which data sections of the net are of interest to the caller, and where   */
/* data fields of interest should be stored.                                 */

#include "pepnet.h"

void read_PEP_net(char *filename, t_blockdest *dest)
{
	read_PEP_file(filename, type_pepnet, netblocks, dest);
}

/*****************************************************************************/

