/* FSA-Dateiformat lesen:            Stefan Roemer, 04.09.1995 - 28.02.1996  */
#include <string.h>
#include <ctype.h>
#include "rw_fsa.h"
#include "readlib.h"

static Cardinal32 CP_AnzTrNamen;
static char       ** CP_TrNamen;
static Place      CP_pllast;
static Transition CP_trlast;


/*****************************************************************************/
/* Einlesen eines Namens aus einer Textdatei.                                */
/*                                                             Stefan Roemer */
/*****************************************************************************/
 
#if defined(__STDC__) || defined(__cplusplus)
 
static unsigned char ReadFSAName( FILE * file, char * string, int maxlen )
 
#else
 
static unsigned char ReadFSAName( file, string, maxlen )
 
  FILE * file;
  char * string;
  int  maxlen;
 
#endif
 
  {
 
    register unsigned char len = 1;
 
    if ( isalnum( *string = ReadWhiteSpace( file ) ) )
      {
        while ( ( isalnum( *++string = getc( file ) ) || ( *string == '-' )
               || ( *string == '_' ) ) && ( (int) ++len < maxlen ) )
          skip;
        if ( len == maxlen )
          {
            NNError( "ReadFSAName(): identifier string size exceeded" );
          } /* if */
      } /* if */
    else if ( ! feof( file ) )
      {
        NNError( "ReadFSAName(): identifier expected" );
      } /* else if */
    ungetc( *string, file );
    *string = '\0';
 
    return ( len );
 
  } /* ReadFSAName() */
 

/*****************************************************************************/
/* Lesen einer Zahl ( im Dezimalsystem ) aus einer Textdatei.                */
/*                                                             Stefan Roemer */
/*****************************************************************************/
 
#if defined(__STDC__) || defined(__cplusplus)
 
static Cardinal32 ReadFSANumber( FILE * file )
 
#else
 
static Cardinal32 ReadFSANumber( file )
 
  FILE * file;
 
#endif
 
  {
 
    register Cardinal32 number;
    register int        digit = '\0';
 
    while (    ! feof( file )
            && ( ( ( digit = getc( file ) ) == ' ' ) || ( digit == '\t' ) ) )
      skip;
    if ( isdigit( digit ) )
      {
        number = digit - '0';
        while ( isdigit( digit = getc( file ) ) )
          {
            number = number * 10 + digit - '0';
          } /* while */
        if ( digit != ' ' )
          {
            ungetc( digit, file );
          } /* if */
        return ( number );
      } /* if */
    else if ( digit == '\n' )
      {
        NNinput_line++;
        return ( 0 );
      } /* else if */
    else
      {
        NNError( "ReadFSANumber(): digit expected" );
      } /* else */

    return ( 0 );
 
  } /* ReadFSANumber() */
 

/*****************************************************************************/
/***************************** read_FSA **************************************/
/*****************************************************************************/
/* Neu eingefuegt                      04.09.1995 - 04.09.1995 Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LMPNet read_FSA( char * FSAfilename )

#else

LMPNet read_FSA( FSAfilename )

  char * FSAfilename;

#endif

  {

    Place               pl,
                        * PlArray;
    register Transition tr,
                        tr2;
    Transition          lasttr,
                        * trarray,
                        * trarrayptr;
    char                cmd[513],
                        name[NAMESIZE+1],
                        fsaname[NAMESIZE+1],
                        numericname[50];
    register FILE       * FSAfile;
    register LMPNet     net;
    LMPNet              * netarray,
                        * netarrayptr;
    char                ** netname,
                        ** identifier;
    register char       ** netnameptr,
                        ** idptr;
    register Cardinal32 AnzPlNamen,
                        MaxPlNamen,
                        MaxTrNamen,
                        NumNets,
                        MaxNets,
                        NumIdents,
                        MaxIdents,
                        id,
                        ident,
                        count,
                        count2,
                        comtype = 0,
                        * comlabel,
                        * comlabel2;
    Cardinal32          startstate;
    STEdge              stlast = NULL;
    TSEdge              tslast = NULL;

    CP_trlast = NULL;
    CP_pllast = NULL;
    NNinput_file = FSAfilename;
    NNinput_line = 1;
    PlArray = (Place *) Malloc( ( count = MaxPlNamen = MaxTrNamen
                                           = NAMES_START ) * sizeof( Place ) );
    CP_TrNamen = (char **) Malloc( count * sizeof( char * ) );
    AnzPlNamen = CP_AnzTrNamen = 0;
    while ( --count )
      {
        PlArray[ count ] = (Place) NULL;
      } /* while */

    netarrayptr = netarray = (LMPNet *) Malloc( ( MaxNets = 100 )
                                                           * sizeof( LMPNet ) );
    netname = netnameptr = (char **) Malloc( MaxNets * sizeof( char * ) );
    identifier = idptr = (char **) Malloc( ( MaxIdents = NAMES_START )
                                                           * sizeof( char * ) );
    NumNets = NumIdents = 0;

    if ( ! ( FSAfile = fopen( FSAfilename, "r" ) ) )
      {
        fprintf( stderr,
                "ERROR: CM.read_FSA(): could not open file '%s' for reading.\n",
                 FSAfilename );
        exit ( 4 );
      } /* if */

    ReadFSAName( FSAfile, cmd, 512 );
    while ( ! feof( FSAfile ) )
      {
        if ( strcmp( cmd, "FSA" )  )
          {
            NNError( "CM.read_FSA(): keyword 'FSA' expected" );
            return ( NULL );
          } /* if */
        /* Initialisiere net */
        if ( ++NumNets >= MaxNets )
          {
            netarrayptr = &( ( netarray = (LMPNet *) Realloc( (char *) netarray,
                     ( MaxNets += 100 ) * sizeof( LMPNet ) ) )[ NumNets - 1 ] );
            netnameptr = &( ( netname = (char **) Realloc( (char *) netname,
                                MaxNets * sizeof( char * ) ) )[ NumNets - 1 ] );
          } /* if */
        *( netarrayptr++ ) = net = empty_net();
        net->NumComponents = 1;
        ReadFSAName( FSAfile, fsaname, NAMESIZE );
        *( netnameptr++ ) = Strdup( fsaname );
        ReadSymbol( FSAfile, ':' );

        ReadFSAName( FSAfile, cmd, 512 );
        if ( strcmp( cmd, "STATES" )  )
          {
            NNError( "CM.read_FSA(): keyword 'STATES' expected" );
            return ( NULL );
          } /* if */
        ReadSymbol( FSAfile, '=' );
        count = ReadFSANumber( FSAfile );

        ReadFSAName( FSAfile, cmd, 512 );
        if ( strcmp( cmd, "TRANSITIONS" )  )
          {
            NNError( "CM.read_FSA(): keyword 'TRANSITIONS' expected" );
            return ( NULL );
          } /* if */
        ReadSymbol( FSAfile, '=' );
        count = ReadFSANumber( FSAfile );

        ReadFSAName( FSAfile, cmd, 512 );
        if ( strcmp( cmd, "START" )  )
          {
            NNError( "CM.read_FSA(): keyword 'START' expected" );
            return ( NULL );
          } /* if */
        ReadSymbol( FSAfile, '=' );
        startstate = ReadFSANumber( FSAfile );

        ReadFSAName( FSAfile, cmd, 512 );
        if ( strcmp( cmd, "ACCEPT" )  )
          {
            NNError( "CM.read_FSA(): keyword 'ACCEPT' expected" );
            return ( NULL );
          } /* if */
        ReadSymbol( FSAfile, '=' );
        count = ReadFSANumber( FSAfile );
        ident = 0;

        ReadFSAName( FSAfile, cmd, 512 );
        while ( ! strcmp( cmd, "STATE" ) )
          {
            count = ReadFSANumber( FSAfile );
            sprintf( numericname, "%05u%016u", NumNets, count );
            fast_insert_place( net, numericname, startstate == count,
                                                  &CP_pllast )->numname = count;
            ReadSymbol( FSAfile, ':' );
            ReadFSAName( FSAfile, cmd, 512 );
            while ( ! strcmp( cmd, "TRANS" )  )
              {
                ReadFSAName( FSAfile, cmd, 512 );
                ReadSymbol( FSAfile, '(' );
                ReadFSAName( FSAfile, name, NAMESIZE );
                count = 1;
                if ( ! strcmp( cmd, "call" ) )
                  {
                    if ( strcmp( name, fsaname ) )
                      {
                        NNError( "CM.read_FSA(): name of FSA expected" );
                      } /* if */
                    comtype = 1;
                    if ( TestSymbol( FSAfile, ';' ) )
                      {
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        count++;
                        ReadSymbol( FSAfile, '.' );
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        count++;
                        for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                          {
                            if ( ! strcmp( *idptr, name ) )
                              {
                                break;
                              } /* if */
                          } /* for */
                        if ( id == NumIdents )
                          {
                            if ( ++NumIdents >= MaxIdents )
                              {
                                idptr = &( ( identifier = (char **) Realloc(
                                               (char *) identifier, MaxIdents
                                     * sizeof( char * ) ) ) [ NumIdents - 1 ] );
                              } /* if */
                            *idptr = Strdup( name );
                          } /* if */
                        while ( TestSymbol( FSAfile, ';' ) )
                          {
                            ReadFSAName( FSAfile, name, NAMESIZE );
                            count++;
                            for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                              {
                                if ( ! strcmp( *idptr, name ) )
                                  {
                                    break;
                                  } /* if */
                              } /* for */
                            if ( id == NumIdents )
                              {
                                if ( ++NumIdents >= MaxIdents )
                                  {
                                    idptr = &( ( identifier = (char **) Realloc(
                                               (char *) identifier, MaxIdents
                                     * sizeof( char * ) ) ) [ NumIdents - 1 ] );
                                  } /* if */
                                *idptr = Strdup( name );
                              } /* if */
                          } /* while */
                      } /* if */
                  } /* if */
                else if ( ! strcmp( cmd, "accept" ) )
                  {
                    comtype = 2;
                    if ( TestSymbol( FSAfile, ';' ) )
                      {
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        if ( strcmp( name, fsaname ) )
                          {
                            NNError( "CM.read_FSA(): name of FSA expected" );
                          } /* if */
                        count++;
                        ReadSymbol( FSAfile, '.' );
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        count++;
                        for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                          {
                            if ( ! strcmp( *idptr, name ) )
                              {
                                break;
                              } /* if */
                          } /* for */
                        if ( id == NumIdents )
                          {
                            if ( ++NumIdents >= MaxIdents )
                              {
                                idptr = &( ( identifier = (char **) Realloc(
                                               (char *) identifier, MaxIdents
                                     * sizeof( char * ) ) ) [ NumIdents - 1 ] );
                              } /* if */
                            *idptr = Strdup( name );
                          } /* if */
                        while ( TestSymbol( FSAfile, ';' ) )
                          {
                            ReadFSAName( FSAfile, name, NAMESIZE );
                            count++;
                            for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                              {
                                if ( ! strcmp( *idptr, name ) )
                                  {
                                    break;
                                  } /* if */
                              } /* for */
                            if ( id == NumIdents )
                              {
                                if ( ++NumIdents >= MaxIdents )
                                  {
                                    idptr = &( ( identifier = (char **) Realloc(
                                               (char *) identifier, MaxIdents
                                     * sizeof( char * ) ) ) [ NumIdents - 1 ] );
                                  } /* if */
                                *idptr = Strdup( name );
                              } /* if */
                          } /* while */
                      } /* if */
                  } /* else if */
                else if ( ! (    strcmp( cmd, "internal" )
                              && strcmp( cmd, "exception" ) ) )
                  {
                    if ( strcmp( name, fsaname ) )
                      {
                        NNError( "CM.read_FSA(): name of FSA expected" );
                      } /* if */
                    if ( TestSymbol( FSAfile, ';' ) )
                      {
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        count++;
                        for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                          {
                            if ( ! strcmp( *idptr, name ) )
                              {
                                break;
                              } /* if */
                          } /* for */
                        if ( id == NumIdents )
                          {
                            if ( ++NumIdents >= MaxIdents )
                              {
                                idptr = &( ( identifier = (char **) Realloc(
                                               (char *) identifier, MaxIdents
                                     * sizeof( char * ) ) ) [ NumIdents - 1 ] );
                              } /* if */
                            *idptr = Strdup( name );
                          } /* if */
                      } /* if */
                    comtype = 0;
                  } /* else if */
                else if ( ! strcmp( cmd, "terminate" ) )
                  {
                    if ( strcmp( name, fsaname ) )
                      {
                        NNError( "CM.read_FSA(): name of FSA expected" );
                      } /* if */
                    comtype = 0;
                  } /* else if */
                else
                  {
                    sprintf( name,
                                "CM.read_FSA(): unexpected keyword '%s' found",
                                                                          cmd );
                    NNError( name );
                  } /* else */
                sprintf( numericname, "%05u%016u", NumNets, ++ident );
                fast_insert_transition( net, numericname, &CP_trlast )
                                                              ->numname = count;
                CP_trlast->label = (MultiSet) NumNets;
                comlabel = (Cardinal32 *) ( CP_trlast->inscript
                      = (char *) Malloc( (count + 2) * sizeof( Cardinal32 ) ) );
                *( comlabel + 1 ) = count;
                if (( *comlabel = comtype ))
                  {
                    *( comlabel + comtype + 1 ) = NumNets - 1;
                  } /* if */
                else
                  {
                    *( comlabel + 2 ) = NumNets - 1;
                  } /* else */
                fast_insert_PT_con( net, CP_pllast->name,
                          CP_trlast->name, 1, &CP_pllast, &CP_trlast, &stlast );
                ReadSymbol( FSAfile, ')' );
                ReadSymbol( FSAfile, '-' );
                ReadSymbol( FSAfile, '-' );
                ReadSymbol( FSAfile, '-' );
                ReadSymbol( FSAfile, '>' );
                count = ReadFSANumber( FSAfile );

                ReadFSAName( FSAfile, cmd, 512 );
              } /* while ( ! strcmp( cmd, "TRANS" )  ) */
          } /* while ( ! strcmp( cmd, "STATE" ) ) */

      } /* while ( ! feof( FSAfile ) ) */
    fclose( FSAfile );

    NNinput_line = 1;
    comtype = 0;
    /* Open FSA file */
    if ( ! ( FSAfile = fopen( FSAfilename, "r" ) ) )
      {
        fprintf( stderr,
               "ERROR: CM.read_FSA(): could not open file '%s' for reading.\n",
                 FSAfilename );
        exit ( 4 );
      } /* if */

    netarrayptr = netarray;
    ReadFSAName( FSAfile, cmd, 512 );
    while ( ! feof( FSAfile ) )
      {
        /* Initialisiere net */
        tr = ( net = *( netarrayptr++ ) )->Transitions;
        comtype++;
        if ( strcmp( cmd, "FSA" )  )
          {
            NNError( "CM.read_FSA(): keyword 'FSA' expected" );
            return ( NULL );
          } /* if */
        ReadFSAName( FSAfile, name, NAMESIZE );
        ReadSymbol( FSAfile, ':' );

        ReadFSAName( FSAfile, cmd, 512 );
        if ( strcmp( cmd, "STATES" )  )
          {
            NNError( "CM.read_FSA(): keyword 'STATES' expected" );
            return ( NULL );
          } /* if */
        ReadSymbol( FSAfile, '=' );
        count = ReadFSANumber( FSAfile );

        ReadFSAName( FSAfile, cmd, 512 );
        if ( strcmp( cmd, "TRANSITIONS" )  )
          {
            NNError( "CM.read_FSA(): keyword 'TRANSITIONS' expected" );
            return ( NULL );
          } /* if */
        ReadSymbol( FSAfile, '=' );
        count = ReadFSANumber( FSAfile );

        ReadFSAName( FSAfile, cmd, 512 );
        if ( strcmp( cmd, "START" )  )
          {
            NNError( "CM.read_FSA(): keyword 'START' expected" );
            return ( NULL );
          } /* if */
        ReadSymbol( FSAfile, '=' );
        count = ReadFSANumber( FSAfile );

        ReadFSAName( FSAfile, cmd, 512 );
        if ( strcmp( cmd, "ACCEPT" )  )
          {
            NNError( "CM.read_FSA(): keyword 'ACCEPT' expected" );
            return ( NULL );
          } /* if */
        ReadSymbol( FSAfile, '=' );
        count = ReadFSANumber( FSAfile );

        ReadFSAName( FSAfile, cmd, 512 );
        while ( ! strcmp( cmd, "STATE" ) )
          {
            count = ReadFSANumber( FSAfile );
            ReadSymbol( FSAfile, ':' );

            ReadFSAName( FSAfile, cmd, 512 );
            while ( ! strcmp( cmd, "TRANS" )  )
              {
                ReadFSAName( FSAfile, cmd, 512 );
                ReadSymbol( FSAfile, '(' );
                ReadFSAName( FSAfile, name, NAMESIZE );
                if ( ! strcmp( cmd, "call" ) )
                  {
                    comlabel = (Cardinal32 *) tr->inscript;
                    comlabel++;
                    comlabel++;
                    if ( TestSymbol( FSAfile, ';' ) )
                      {
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        for ( netnameptr = netname, count = 0 ;
                                       count < NumNets ; netnameptr++, count++ )
                          {
                            if ( ! strcmp( *netnameptr, name ) )
                              {
                                break;
                              } /* if */
                          } /* for */
                        *++comlabel = count;
                        ReadSymbol( FSAfile, '.' );
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                          {
                            if ( ! strcmp( *idptr, name ) )
                              {
                                break;
                              } /* if */
                          } /* for */
                        *++comlabel = id;
                        while ( TestSymbol( FSAfile, ';' ) )
                          {
                            ReadFSAName( FSAfile, name, NAMESIZE );
                            for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                              {
                                if ( ! strcmp( *idptr, name ) )
                                  {
                                    break;
                                  } /* if */
                              } /* for */
                            *++comlabel = id;
                          } /* while */
                      } /* if */
                  } /* if */
                else if ( ! strcmp( cmd, "accept" ) )
                  {
                    comlabel = (Cardinal32 *) tr->inscript;
                    comlabel++;
                    for ( netnameptr = netname, count = 0 ; count < NumNets ;
                                                         netnameptr++, count++ )
                      {
                        if ( ! strcmp( *netnameptr, name ) )
                          {
                            break;
                          } /* if */
                      } /* for */
                    *++comlabel = count;
                    comlabel++;
                    if ( TestSymbol( FSAfile, ';' ) )
                      {
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        ReadSymbol( FSAfile, '.' );
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                          {
                            if ( ! strcmp( *idptr, name ) )
                              {
                                break;
                              } /* if */
                          } /* for */
                        *++comlabel = id;
                        while ( TestSymbol( FSAfile, ';' ) )
                          {
                            ReadFSAName( FSAfile, name, NAMESIZE );
                            for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                              {
                                if ( ! strcmp( *idptr, name ) )
                                  {
                                    break;
                                  } /* if */
                              } /* for */
                            *++comlabel = id;
                          } /* while */
                      } /* if */
                  } /* else if */
                else if ( ! (    strcmp( cmd, "internal" )
                              && strcmp( cmd, "exception" ) ) )
                  {
                    if ( TestSymbol( FSAfile, ';' ) )
                      {
                        comlabel = (Cardinal32 *) tr->inscript;
                        comlabel++;
                        comlabel++;
                        ReadFSAName( FSAfile, name, NAMESIZE );
                        for ( idptr = identifier, id = 0 ; id < NumIdents ;
                                                                 idptr++, id++ )
                          {
                            if ( ! strcmp( *idptr, name ) )
                              {
                                break;
                              } /* if */
                          } /* for */
                        *++comlabel = id;
                      } /* if */
                  } /* else if */
                else if ( ! strcmp( cmd, "terminate" ) )
                  {
                  } /* else if */
                else
                  {
                    sprintf( name,
                                "CM.read_FSA(): unexpected keyword '%s' found",
                                                                          cmd );
                    NNError( name );
                  } /* else */
                ReadSymbol( FSAfile, ')' );
                ReadSymbol( FSAfile, '-' );
                ReadSymbol( FSAfile, '-' );
                ReadSymbol( FSAfile, '-' );
                ReadSymbol( FSAfile, '>' );
                count = ReadFSANumber( FSAfile );
                sprintf( numericname, "%05u%016u", comtype, count );
                CP_pllast = NULL;
                fast_insert_TP_con( net, numericname, tr->name, 1,
                                              &CP_pllast, &CP_trlast, &tslast );
                tr = tr->nexttrans;
                ReadFSAName( FSAfile, cmd, 512 );
              } /* while ( ! strcmp( cmd, "TRANS" )  ) */
          } /* while ( ! strcmp( cmd, "STATE" ) ) */
      } /* while ( ! feof( FSAfile ) ) */
    fclose( FSAfile );

    ident = 0;
    *( trarrayptr = trarray = (Transition *) Malloc( ( NumNets + 1 )
                          * sizeof( Transition ) ) ) = (*netarray)->Transitions;
    for ( netarrayptr = netarray + 1, count = NumNets - 1 ; count-- ;
                                                                 netarrayptr++ )
      {
        *++trarrayptr = ( net = *netarrayptr )->Transitions;
        (*netarray)->NumPlaces += net->NumPlaces;
        (*netarray)->NumTransitions += net->NumTransitions;
        if ( ! (*netarray)->Places )
          {
            (*netarray)->Places = net->Places;
            (*netarray)->LastPlace = net->LastPlace;
          } /* if */
        else if (( (*netarray)->LastPlace->nextplace = net->Places ))
          {
            net->Places->prevplace = (*netarray)->LastPlace;
            (*netarray)->LastPlace = net->LastPlace;
          } /* else if */
        if ( ! (*netarray)->Transitions )
          {
            (*netarray)->Transitions = net->Transitions;
            (*netarray)->LastTransition = net->LastTransition;
          } /* if */
        else if (( (*netarray)->LastTransition->nexttrans = net->Transitions ))
          {
            net->Transitions->prevtrans = (*netarray)->LastTransition;
            (*netarray)->LastTransition = net->LastTransition;
          } /* else if */
        net->NumPlaces = net->NumTransitions = 0;
        net->Places = net->LastPlace = NULL;
        net->Transitions = net->LastTransition = NULL;
      } /* for */

    net = *netarray;
    lasttr = net->LastTransition;
    for ( tr = net->Transitions ; tr != lasttr ; tr = tr->nexttrans )
      {
        if (    *( comlabel = (Cardinal32 *) tr->inscript )
             && (    ( ( *comlabel == 1 ) && ( comlabel[2] < comlabel[3] ) )
                  || ( ( *comlabel == 2 ) && ( comlabel[2] > comlabel[3] ) ) ) )
          {
            count = comlabel[ 4 - *comlabel ];
            for ( tr2 = trarray[ count ] ; tr2 && ( tr2 != trarray[ count+1 ] ) ;
                                                          tr2 = tr2->nexttrans )
              {
                if (    *( comlabel2 = (Cardinal32 *) tr2->inscript )
                     && ( *comlabel != *comlabel2 ) )
                  {
                    if ( ( count2 = comlabel[1] ) == comlabel2[1] )
                      {
                        while ( count2 && ( comlabel[ count2 + 1 ]
                                                   == comlabel2[ count2 + 1 ] ) )
                          {
                            count2--;
                          } /* while */
                        if ( ! count2 )
                          {
                            sprintf( numericname, "%05u%016u", NumNets + 1,
                                                                      ++ident );
                            CP_trlast = NULL;
                            fast_insert_transition( net, numericname,
                                                                   &CP_trlast );
                            if ( ident == 1 )
                              {
                                trarray[ NumNets ] = CP_trlast;
                              } /* if */
                            CP_pllast = NULL;
                            CP_trlast = NULL;
                            fast_insert_PT_con( net,
                                      tr->preset->place->name, numericname, 1,
                                              &CP_pllast, &CP_trlast, &stlast );
                            CP_pllast = NULL;
                            CP_trlast = NULL;
                            fast_insert_PT_con( net,
                                     tr2->preset->place->name, numericname, 1,
                                              &CP_pllast, &CP_trlast, &stlast );
                            CP_pllast = NULL;
                            CP_trlast = NULL;
                            fast_insert_TP_con( net,
                                     tr->postset->place->name, numericname, 1,
                                              &CP_pllast, &CP_trlast, &tslast );
                            CP_pllast = NULL;
                            CP_trlast = NULL;
                            fast_insert_TP_con( net,
                                    tr2->postset->place->name, numericname, 1,
                                              &CP_pllast, &CP_trlast, &tslast );
                          } /* if */
                      }
                  } /* if */
              } /* for ( tr2 ... ) */
          } /* if */
      } /* for ( tr ... ) */

    lasttr = trarray[ NumNets ];
    for ( tr = net->Transitions ; tr != lasttr ; skip )
      {
        if ( *( (Cardinal32 *) tr->inscript ) )
          {
            Free( tr->inscript );
            tr->inscript = NULL;
            tr = delete_transition_direct( net, tr );
          } /* if */
        else
          {
            tr = tr->nexttrans;
          } /* else */
      } /* for ( tr ... ) */
    for ( pl = net->Places, count = 0 ; pl ; pl = pl->nextplace )
      {
        pl->numname = ++count;
      } /* for */
    for ( tr = net->Transitions, count = 0 ; tr ; tr = tr->nexttrans )
      {
        tr->numname = ++count;
      } /* for */

    Free( *netname );
    for ( netarrayptr = netarray + 1, netnameptr = netname, count = NumNets ;
                      --count ; Free( *netnameptr ), remove_net( netarrayptr ) )
      skip;
    Free( netarray );
    Free( netname );
    for ( idptr = identifier, count = NumIdents ; count-- ; Free( *idptr ) )
      skip;
    Free( identifier );
    Free( trarray );

    return( net );

  } /* read_FSA() */


/*****************************************************************************/
/***************************** write_FSA *************************************/
/*****************************************************************************/
/*                                                  18.08.1995 Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void write_FSA( LMPNet net, char * FSAfilename )

#else

void write_FSA( net, FSAfilename )

  LMPNet net;
  char   * FSAfilename;

#endif

  {

    register Place      pl;
    register Transition tr;
    register STEdge     st;
    register TSEdge     ts;
    Cardinal32          count;
    FILE                * file;

    if ( ! ( file = fopen( FSAfilename, "w" ) ) )
      {
        NNError( "Could not open file for writing" );
      } /* if */

    f( "PEP\n" );
    f( "PTNet\n" );
    f( "FORMAT_N\n" );
    f( "DPL s7n10@-9t2\n" );
    f( "DTR s7n10@-9t2\n" );
    f( "DPT w1t1\n" );

    f( "PL\n" );
    for ( count = 0, pl = net->Places ; pl ; pl = pl->nextplace )
      {
        pl->numname = ++count;
        fx( "\"%s\"", pl->name );
        fxy( "%u@%u", 20 + count * 10, 20 + count * 10 );
        if ( pl->marks )
          {
            fx( "M%u", pl->marks );
          } /* else if */
        f( "\n" );
      } /* for ( pl ... ) */
    f( "TR\n" );

    for ( count = 0, tr = net->Transitions ; tr ; tr = tr->nexttrans )
      {
        tr->numname = ++count;
        fx("\"%s\"",tr->name);
        fxy( "%u@%u", 160 + count * 10, 20 + count * 10 );
        f( "\n" );
      } /* for ( tr ... ) */

    f( "TP\n" );
    for ( tr = net->Transitions ; tr ; tr = tr->nexttrans )
      {
        for ( ts = tr->postset ; ts ; ts = ts->nextplace )
          {
            pl = ts->place;
            fxy( "%u<%u\n", tr->numname, pl->numname );
          } /* for */
      } /* for */

    f( "PT\n" );
    for ( pl = net->Places ; pl ; pl = pl->nextplace )
      {
        for ( st = pl->postset ; st ; st = st->nexttrans )
          {
            tr = st->trans;
            fxy( "%u>%u\n", pl->numname, tr->numname );
          } /* for */
      } /* for */

    f( "TX\n" );
    f( "\n" );

    fclose( file );

  } /* write_FSA() */
