/*****************************************************************************/
/******************************** gspn_ini ***********************************/
/*****************************************************************************/
/* KONVERSION: 10.05.1993 Stefan Roemer                                      */
/*****************************************************************************/
#include <string.h>
#include "rw_gspn.h"
#include "pep_strings.h"

#if defined(__STDC__) || defined(__cplusplus)

LMPNet gspn_ini( char * filename )

#else

LMPNet gspn_ini( filename )
  /* Initialisiert eine Matrix mit dem in der Datei filename abgelegten Netz */
  /* Die Datei filename ist im GSPN-Format.                                  */

  char * filename; /* Name der Datei, in der das Netz abgelegt ist */

#endif

  {

    /* Deklarationen */
    LMPNet net;
    FILE *ifp;               /* Pointer auf Eingabedatei (InputFilePointer)  */
    char       line[LINELENGTH],
               name[NAMELENGTH],
               comment[1024];/* String fuer den Kommentar zum Netz */
    int i, j, k;             /* Laufvariablen */
    int trnum;       /* Anzahl der Transitionen */
    int plnum;            /* Anzahl der Stellen (places)  */
    int group_nr;            /* Anzahl der  Groups */
    int rate_nr;             /* Anzahl der  Rates*/
    int mark_nr;             /* Anzahl der  Marks */
    int marking;             /* Nimmt Anfangsmarkierung einer Stelle auf     */
    float TRATE;             /* */
    int TSERV, TKND;         /* */
    int TINP;                /* Anzahl der Verbindungen ZU einer Transition  */
    int TINH;                /* Anzahl Inhibitor Arcs                        */
    int TROT;                /* */
    int TOUT;                /* Anzahl der Verbindungen VON einer Transition */
    int AMULT;               /* Wertigkeit einer Verbindung                  */
    int APLACE;              /* zur Aufnahme des anderen Endes der Verbindung*/
    int APOINTS;             /* Anzahl der Zeilen mit Koordinaten von        */
                             /* Zwischenpunkten einer Verbindung             */
    int LEVELS;              /* */
    LIST_POINTER place_list,
                 help_list,
                 list_old=NULL;

    /* Oeffnen der Datei im GSPN-Format */
    if ( ! ( ifp = fopen( filename,"r" ) ) )
      {
        ERROR_FILE_READ( "gspn_ini", filename );
      } /* if */

    /* Initialisiere net */
    net = empty_net();

    /*********************** NONTERMINAL COMMENT *****************************/
    for ( i = 1 ; i <= 2; i++ )
      {
        fgets( line, LINELENGTH, ifp );
      } /* end of for i */
    while ( strncmp( line, "|\n", 2 ) )
      {
        strcat( comment, line );
        fgets( line, LINELENGTH, ifp );
      } /* end of while */
    /* Zeilen mit NONTERMINAL COMMENT gelesen und Kommentar in               */
    /* string comment abgelegt                                               */

    /*********************** NONTERMINAL NOOBJS  *****************************/
    fgets( line, LINELENGTH, ifp );
    sscanf( line, " f   %d %d %d %d %d 0\n",
                  &mark_nr, &plnum, &rate_nr, &trnum, &group_nr );

    /*********************** NONTERMINAL MARKS *******************************/
    for ( i=1 ; i <= mark_nr; i++ )
      {
        fgets( line,LINELENGTH, ifp );
      } /* end of for i */
    /* Zeilen mit Nonterminal MARKS ueberlesen */

    /*********************** NONTERMINAL PLACES ******************************/
    /* Baue Stellenliste in die Matrix ein */
    /* unterhalte in place_list eine Liste der tatsaechlichen Namen der      */
    /* Stellen */
    help_list = place_list = (LIST_POINTER) Malloc( sizeof( LIST_ELEMENT ) );
    for ( i = 1 ; i <= plnum ; i++ )
      {
        fgets( line, LINELENGTH, ifp );
        sscanf( line,"%[A-Za-z0-9]   %d \n", name, &marking );
        Strcpy( help_list->name, name );
        help_list->next = (LIST_POINTER) Malloc( sizeof( LIST_ELEMENT ) );
        list_old = help_list;
        help_list = help_list->next;
        insert_place( net, name, marking );
      } /* end of for i */
    /* schliesse Liste ab */
    list_old->next = NULL;
    /* Stellenliste in die Matrix eingebaut, erste Spalte der Matrix steht.  */

    /*********************** NONTERMINAL GROUPS ******************************/
    for ( i=1; i<=group_nr; i++ )
      {
        fgets( line, LINELENGTH, ifp );
      } /* end of for i */
    /* Zeilen mit Nonterminal GROUPS ueberlesen */

    /*********************** NONTERMINAL TRANS *******************************/
    /* Baue Transitionsliste und Verbindungen in die Matrix ein */
    /* Situation: Bild 1. */
    /* Aufbau von hinten, da GSPN die Transitionen umgekehrt ihrer Benennung */
    /* ablegt */
    for ( i=1; i<= trnum; i++ )
      {
        fgets( line, LINELENGTH, ifp );
        sscanf( line, "%[A-Za-z0-9] %f %d %d %d %d  \n",
                      name, &TRATE, &TSERV, &TKND, &TINP, &TROT );
        insert_transition( net, name );
        /* verarbeite die Zeilen fuer die Verbindungen zu dieser Transition */
        for ( j = 1 ; j<= TINP ; j++ )
          {
            fgets( line, LINELENGTH, ifp );
            sscanf( line, "    %d  %d  %d %d \n",
                          &AMULT, &APLACE, &APOINTS, &LEVELS );
            /* Konstruiere den Namen der Stelle,von der die Verbindung kommt */
            help_list = place_list;
            for ( k = 1 ; k < APLACE ; k++ )
              {
                help_list = help_list->next;
              } /* end of for k */
            insert_PT_con( net, help_list->name, name, AMULT );
            /* Ueberlese die Koordinatenzeilen mit Zwischenpunkten */
            for ( k = 1 ; k <= APOINTS; k++ )
              {
                fgets( line, LINELENGTH, ifp );
              } /* end of for k */
          } /* end of for j */
        /* Zu-Verbindungen eingebaut */ 
    
        /* Verarbeite die Zeilen fuer die Verbindungen von dieser Transition */
        fgets( line, LINELENGTH, ifp );
        sscanf( line, "%d \n", &TOUT );
        /* TOUT : Anzahl der Von-Verbindungen ( # Output Arcs) */
        for ( j = 1 ; j <= TOUT; j++ )
          {
            fgets( line, LINELENGTH, ifp );
            sscanf( line, "   %d  %d  %d %d \n",
                          &AMULT, &APLACE, &APOINTS, &LEVELS );
            /* AMULT : Wertigkeit der Verbindung */
            /* APLACE :Name der Stelle, von der die Verbindung kommt */
            /* APOINTS : Anzahl der Zwischenpunkte dieser Verbindung */
            /* Konstruiere Name, von der die Verbindung kommt */
            help_list = place_list;
            for ( k = 1 ; k < APLACE ; k++ )
              {
                help_list = help_list->next;
              } /* end of for k */
            insert_TP_con( net, help_list->name, name, AMULT );
        
            /* Ueberlese die Koordinatenzeilen mit Zwischenpunkten */
            for ( k = 1 ; k <= APOINTS ; k++ )
              {
                fgets( line, LINELENGTH, ifp );
              } /* end of for k */
          } /* end of for j */
        /* Von-Verbindungen eingebaut */

        /* verarbeite Zeilen mit Anzahl der Inhibitor-Verbindungen */
        fgets( line, LINELENGTH, ifp );
        sscanf( line, "%d \n", &TINH );
        if ( TINH != 0 )
          {
            /* Fehler: es existieren Inhibitor-Verbindungen */
            fprintf( stderr,
                    "WARNING: NN.gspn_ini(): net contains inhibitor arcs.\n" );
            remove_net( &net );
            fclose( ifp );
            return( NULL );
          } /* end of if */

      } /* end of for i */
    fclose( ifp );

    return( net );

  } /* gspn_ini() */
