#include <string.h>
#include "rw_inf.h"
#include "readlib.h"

/*****************************************************************************/
/****************************** praktikum_ini ********************************/
/*****************************************************************************/
/* KONVERSION: 10.05.1993 Stefan Roemer                                      */
/* AENDERUNG : 14.02.1994 Stefan Roemer: Keine Beschraenkung mehr bzgl.      */
/*                                       der Zeilenlaenge!                   */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LMPNet praktikum_ini( char * filename )

#else

LMPNet praktikum_ini( filename )
  /* liefert einen Pointer auf eine Matrix, in der das Netz gespeichert ist. */
  /* Ergebnis ist NULL-Pointer, wenn eine entsprechend benannte .inf Datei   */
  /* nicht existiert.                                                        */

  char * filename;

#endif

  {

    LMPNet		net;
    register FILE       * ifp;
    char                str[101],
                        name[NAMELENGTH],
                        P_name[NAMELENGTH];
    register Cardinal32 trnum,
                        plnum,
                        tsweight,
                        stweight,
                        i, j;
    register Place      phelp;
    register Transition thelp;
    register bool       parenth;
    Place               pl_last = NULL;
    Transition          tr_last = NULL;
    STEdge              st_last = NULL;
    TSEdge              ts_last = NULL;

    /* Oeffne .inf-Datei */
    if ( ! ( ifp = fopen( filename, "r" ) ) )
      {
        ERROR_FILE_READ( "praktikum_ini", filename );
        exit ( 4 );
      } /* if */
    NNinput_line = 1;
    NNinput_file = filename;

    /* Initialisiere net */
    net = empty_net();

    /* Lese Zeile mit Anzahl-Angaben */
    plnum = ReadNumber( ifp );
    trnum = ReadNumber( ifp );

    /* Lese Zeile mit 'Inzidenzmatrix' */
    ReadName( ifp, str, 100 );
    if ( strcmp( str, "Inzidenzmatrix:" ) )
      {
        fclose( ifp );
        NNError( "Schluesselwort 'Inzidenzmatrix:' erwartet" );
      } /* if */

    /* Lese erste Zeile der Matrix */
    for ( i = 1 ; i <= trnum ; i++ )
      {
        ReadName( ifp, name, NAMELENGTH-1 );
        /* setze neue Transition in die Matrix ein */
        fast_insert_transition( net, name, &tr_last );
      } /* end of for i */
    /* erste Zeile verarbeitet, Transitionsliste in der Matrix initialisiert */
    /* verarbeite weitere Zeilen */
    for ( i = 1; i <= plnum ; i++ )
      {
        ReadName( ifp, P_name, NAMELENGTH-1 );
        fast_insert_place( net, P_name, 0, &pl_last );
        ReadSymbol( ifp, '|' );
        /* lege eine Hilfszeiger in die Transitionsliste, um auf die         */
        /*  Transitionsnamen Zugriff zu haben */
        thelp = net->Transitions;
        for ( j = 1 ; j <= trnum ; j++ )
          {
            ReadSymbol( ifp, '(' );
            tsweight = abs( ReadNumber( ifp ) );
            ReadSymbol( ifp, ',' );
            stweight = abs( ReadNumber( ifp ) );
            ReadSymbol( ifp, ')' );
            if ( stweight )
              { fast_insert_PT_con( net, P_name, thelp->name, stweight,
                                    &pl_last, &tr_last, &st_last ); }
            if ( tsweight )
              { fast_insert_TP_con( net, P_name, thelp->name, tsweight,
                                    &pl_last, &tr_last, &ts_last ); }
            thelp = thelp->nexttrans;
          } /* end of for j */
      } /* end of for i */
    /* Inzidenzmatrix verarbeitet */

    /* Lese Anfangsmarkierung ein */
    /* Lese Zeile mit 'Startmarkierung' */
    ReadName( ifp, str, 100 );
    if ( strcmp( str, "Startmarkierung:" ) )
      {
        fclose( ifp );
        NNError( "Schluesselwort 'Startmarkierung:' erwartet" );
      } /* if */

    parenth = False;
    if ( TestSymbol( ifp, '(' ) )
      {
        parenth = True;
      } /* if */
    phelp = net->Places;
    phelp->marks = ReadNumber( ifp );
    for ( i = 2 ; i <= plnum; i++ )
      {
        ReadSymbol( ifp, ',' );
        ( phelp = phelp->nextplace ) -> marks = ReadNumber( ifp );
      } /* for */
    if ( parenth && ! TestSymbol( ifp, ')' ) )
      {
        fclose( ifp );
        NNError( "Markierungsvektor: ')' erwartet" );
      } /* if */

    fclose( ifp );

    return( net );

  } /* praktikum_ini() */

/*****************************************************************************/
/******************************** write_matrix *******************************/
/*****************************************************************************/
/* KONVERSION: 10.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void output_matrix( LMPNet net, FILE * ofp )

#else

void output_matrix( net, ofp )
  /* Gibt die mit net bezeichnete Matrix als Inzidenzmatrix nach 'file' aus  */

  LMPNet net;        /* Zeiger auf Matrix */
  FILE   * ofp;

#endif

  {

    register Cardinal32 trnum,
                        plnum;
    register int        i,
                        width1,
                        width2;
    register Place      pl;
    register Transition tr,
                        trhelp;
    register STEdge     st;
    register TSEdge     ts;

    if ( net )
      {
        trnum = net->NumTransitions;
        plnum = net->NumPlaces;
        /* Bestimme Wert fuer Spaltenbreite */
        width1 = max_placename_length( net );
        fprintf( ofp, "%d %d \n", plnum, trnum ); 
        fprintf( ofp, "Inzidenzmatrix: \n" );
        tr = net->Transitions;
        for ( i = 1 ; i <= width1+4 ; i++ )
          {
            fprintf( ofp, " " );
          } /* end of for i */
        while ( tr )
          {
            fprintf( ofp, "%s ", tr->name );
            for ( i = 1 ; i <= (int)( 6 - strlen( tr->name ) ) ; i++ )
              {
                fprintf( ofp, " " );
              } /* end of for i */
            tr = tr->nexttrans;
          } /* while */
        fprintf( ofp, " \n" );
        pl = net->Places;
        while ( pl )
          {
            trhelp = net->Transitions;
            st = pl->postset;
            ts = pl->preset;
            fprintf( ofp, "%s", pl->name );
            for ( i = 1 ; i <= (int)( width1 - strlen( pl->name ) ) ; i++ )
              {
                fprintf( ofp, " " );
              } /* end of for i */
            fprintf( ofp, " |" );
            while ( trhelp )
              {
                width2 = strlen( trhelp->name );
                /* falls die Verbindungsliste noch nicht zuende ist */
                if ( ts && ( ts->trans == trhelp ) )
                  {
                    fprintf( ofp, "(-%d,", ts->weight );
                    ts = ts->nexttrans;
                  } /* if */
                else
                  {
                    fprintf( ofp, "( 0," );
                  } /* else */
                if ( st && ( st->trans == trhelp ) )
                  {
                    fprintf( ofp, "%d) ", st->weight );
                    st = st->nexttrans;
                  } /* if */
                else
                  {
                    fprintf( ofp, "0) " );
                  } /* else */
                for ( i = 1; i <= ( width2 - 5 ) ; i++ )
                  {
                    fprintf( ofp, " " );
                  } /* end of for i */
                trhelp = trhelp->nexttrans;
              } /* end of while */
            fprintf( ofp, " \n" );
            /* eine Zeile geschrieben */
            pl = pl->nextplace;
          } /* end of while */
      } /* end of if net */
    else
      {
        WARNING_NO_NET( "output_matrix" );
      } /* else */

  } /* output_matrix() */


/*---------------------------------------------------------------------------*/

#if defined(__STDC__) || defined(__cplusplus)

void write_matrix( LMPNet net, char * filename )

#else

void write_matrix( net, filename )
  /* Gibt die mit net bezeichnete Matrix als Inzidenzmatrix in die Datei     */
  /* filename aus. */ 

  LMPNet net;        /* Zeiger auf Matrix */
  char   * filename; /* String mit Namen der Ausgabedatei */

#endif

  {

    register FILE * ofp;

    if (( ofp = fopen( filename, "w" ) ))
      {
        output_matrix( net, ofp );
        fclose( ofp );
      } /* if */
    else
      {
        ERROR_FILE_WRITE( "write_matrix", filename );
      } /* else */

  } /* write_matrix() */


/*****************************************************************************/
/******************************* write_netmarking ****************************/
/*****************************************************************************/
/* KONVERSION: 10.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void write_netmarking( LMPNet net, char * filename )

#else

void write_netmarking( net, filename )
  /* Gibt die Anfangsmarkierung der Matrix net in die Datei filename aus */

  LMPNet net;
  char   filename[NAMELENGTH];

#endif

  {

    Place pl;
    FILE  * ofp;

    if ( net )
      {
        if (( ofp = fopen( filename, "a" ) ))
          {
            fprintf( ofp, "\nStartmarkierung: \n" );
            if (( pl = net->Places ))
              {
                fprintf( ofp, "( %d", pl->marks );
                while (( pl = pl->nextplace ))
                  {
                    /* mittlere Marke, mit Komma */
                    fprintf( ofp, " ,%d", pl->marks);
                  } /* while */
                fprintf( ofp, " ) \n" );
              } /* if */
            fclose( ofp );
          } /* if */
        else
          {
            ERROR_FILE_READ( "write_netmarking", filename );
          } /* end of else */
      } /* if */
    else
      {
        WARNING_NO_NET( "write_netmarking" );
      } /* else */
   
  } /* write_netmarking() */


/*****************************************************************************/
/****************************** show_matrix **********************************/
/*****************************************************************************/
/* KONVERSION: 10.05.1993 Stefan Roemer                                      */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void show_matrix( LMPNet net )

#else

void show_matrix( net )
  /* Gibt die mit net bezeichnete Matrix als Inzidenzmatrix auf dem          */
  /* Bildschirm aus. */ 

  LMPNet net; /* Zeiger auf Matrix */

#endif

  {

    output_matrix( net, stdout );

  } /* show_matrix() */


/*****************************************************************************/
/******************************** show_marking *******************************/
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void show_marking( MARKING marking )

#else

void show_marking(marking)
               /* gibt die Markierungsliste 'marking' auf dem Bildschirm aus */

  MARKING marking;

#endif

  {

    register MARKING mpointer;

    if ( marking )
      {
        mpointer = marking;
        while ( mpointer )
          {
            if ( mpointer == marking )
              {
                printf("(");
              } /* if */
            if ( ! mpointer->next )
              {
                printf( "%d)\n", mpointer->value );
              } /* if */
            else
              {
                printf( "%d, ", mpointer->value );
              } /* else */
            mpointer = mpointer->next;
          } /* while */
      } /* if */
    else
      {
        fprintf( stderr, "WARNING: NN.show_marking(): not a marking.\n" );
      } /* else */

  } /* show_marking() */
