/*-------------------------------------------------------------------------*/
/*--                                                                     --*/
/*-- READ_MCI.C                                                          --*/
/*--                                                                     --*/
/*-- Routines for reading and writing finite prefices from binary files. --*/
/*-- Based on readMCI.c from S. Roemer.                                  --*/
/*-- Changed and extended by Johannes Jaeger - Summer 1994               --*/
/*-- Adopted for libreadpep by Christian Stehno, July 2001               --*/
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#ifdef __linux
#include <endian.h>
#include <byteswap.h>
#else
#include <sys/param.h>
#endif
#include "rw_mci.h"
#include "list.h"
#include "readlib.h"
#ifdef WITH_ZLIB
#include "zlib.h"
#endif

extern Cardinal32 GL_numCutoffs;
bool GL_SwapNum;

#define E_RDMCI_NOCOND   "readMCI(): Net without conditions!"

/* eingefuegte Kanten erhalten Gewicht 1 */
#define TP_WEIGHT    1
#define PT_WEIGHT    1
/* eine neu ins Netz eingefuegte Stelle enthaelt keine Markierung */
#define INITIAL_MARK 0

/*-------------------------------------------------------------------------*/
/*-- Diese Funktion wurde von H.Truee uebernommen --
*/

#if defined(__STDC__) || defined(__cplusplus)

Cardinal32 swap_endian( Cardinal32 in)

#else

Cardinal32 swap_endian( in )
Cardinal32 in;

#endif
{
#ifdef bswap_32
  return bswap_32(in);
#else
  Cardinal32 out;
  unsigned char *t_in;
  unsigned char *t_out;

  t_in = (unsigned char *)&in;
  t_out = (unsigned char *)&out;
  t_out[0] = t_in[3];
  t_out[1] = t_in[2];
  t_out[2] = t_in[1];
  t_out[3] = t_in[0];
  return( out );
#endif
} /* swap_endian() */

/* Set swap according to byte order */
#ifdef __BYTE_ORDER
  #if __BYTE_ORDER == __LITTLE_ENDIAN
    #define swap(x) swap_endian(x)
  #endif
#else
  #ifdef _LITTLE_ENDIAN
    #define swap(x) swap_endian(x)
  #else
    #ifdef BYTE_ORDER
      #if BYTE_ORDER == LITTLE_ENDIAN
        #define swap(x) swap_endian(x)
      #endif
    #endif
  #endif
#endif
#ifndef swap
  #define swap(x) x
#endif /* else BIG_ENDIAN */

/*-------------------------------------------------------------------------*/
/*-- Einlesen einer Zahl vom Typ Cardinal32 aus MCIfile                  --*/
/*-- Parameter :                                                         --*/
/*--   FILE *MCIfile : aus dieser Datei wird gelesen                     --*/
/*--                                                                     --*/
/*-- Rueckgabe : die gelesene Zahl vom Typ Cardinal32                    --*/

Cardinal32 ReadNum( FILE * MCIfile )
{
  Cardinal32 number;

#ifdef WITH_ZLIB
  gzread(MCIfile, (char *) &number, sizeof (Cardinal32));
#else
  fread( (char *) &number, sizeof( Cardinal32 ), 1, MCIfile );
#endif

  return swap( number );
} /* ReadNum() */

Cardinal32 ReadNumMCI( FILE * MCIfile )
{
  Cardinal32 number;

#ifdef WITH_ZLIB
  gzread(MCIfile, (char *) &number, sizeof (Cardinal32));
#else
  fread( (char *) &number, sizeof( Cardinal32 ), 1, MCIfile );
#endif

  return GL_SwapNum?swap_endian( number ):number;
} /* ReadNum() */

void WriteNum( Cardinal32 number, FILE* MCIfile )
{
  Cardinal32 num;
  
  num = swap( number );
#ifdef WITH_ZLIB
  gzwrite(MCIfile, (char *) &number, sizeof (Cardinal32));
#else
  fwrite( (char *) &(num), sizeof( Cardinal32 ), 1, MCIfile );
#endif
}

void WriteNumMCI( Cardinal32 number, FILE* MCIfile )
{
#ifdef WITH_ZLIB
  gzwrite(MCIfile, (char *) &number, sizeof (Cardinal32));
#else
  fwrite( (char *) &(number), sizeof( Cardinal32 ), 1, MCIfile );
#endif
}

/*-------------------------------------------------------------------------*/
/*-- Interface-Routine zum Lesen einer Entfaltung aus einer Binaer-Datei.--*/
/*-- Parameter:                                                          --*/
/*--   char* MCIfilename : Dateiname der Binaerdatei mit einer Entfaltung--*/
/*--   LIST_POINTER *CutOffPoints:                                       --*/
/*--                    Adresse eines LIST_POINTERS, ueber den eine Liste--*/
/*--                    der Namen der Cut-Off-Punkte zurueckgegeben wird --*/
/*--   LIST_POINTER *MaxConfig:                                          --*/
/*--                    Adresse eines LIST_POINTERS, ueber den eine Liste--*/
/*--                    der Namen einer max. Konfiguration zurueckgegeben--*/
/*--                    wird, falls keine Cutoffpunkte existieren        --*/
/*--                                                                     --*/
/*-- Rueckgabe:                                                          --*/
/*--     Zeiger vom Typ LMPNet auf das eingelesene Netz                  --*/

LMPNet readMCI( FILE* MCIfile,
                LIST_POINTER *CutOffPoints,
                LIST_POINTER *MaxConfig   )
{

  LMPNet     net = NULL;
  Place      pl,co_last = NULL;
  Transition ev,
             ev_last = NULL;
  STEdge     be_last = NULL;
  TSEdge     eb_last = NULL;
  Cardinal32 conum,
             evnum,
             i,
             num;
  char       coname[NAMELENGTH],
             evname[NAMELENGTH];

  char       * all_names,
             ** tr_names,
             * tmpchar,
             ** tmpptr;

  /* Anzahl Bedingungen lesen */
  GL_SwapNum=False;
  if (( conum = ReadNumMCI( MCIfile ) ))
  {
    /* Try to be clever ;-) */
    if ((conum>>24)!=0)
      {
	fprintf(stderr,"Byte swapping in action!\n");
	GL_SwapNum=True;
	conum=swap_endian(conum);
      }

    /* Anzahl Ereignisse lesen */
    evnum = ReadNumMCI( MCIfile );

    /* Netzheader anlegen */
    net = empty_net();

    /* Ereignisse ins Netz einfuegen */
    for ( i = 1 ; i <= evnum ; i++ )
    {
      /* aus "E" und i den Transitionsnamen bilden */
      sprintf( evname, "E%u", i );

      /* Transition ins Netz einfuegen, Feld numname setzen */
      (ev = fast_insert_transition( net, evname, &ev_last ))->numname = i;

      /* Nummer der entsprechenden Transition im Originalnetz lesen */
      /* und zunaechst im Feld inscript eintragen                   */
      ev->inscript = (char *) ReadNumMCI( MCIfile );
      ev->label = NULL;
    } /* for ... */

    /* Bedingungen und Kanten ins Netz einfuegen */
    for ( i = 1 ; i <= conum ; i++ )
    {
      /* aus "B" und i den Bedingungsnamen bilden */
      sprintf( coname, "B%u", i );

      /* Bedingung ins Netz einfuegen, Feld numname setzen */
      (pl=fast_insert_place( net, coname, INITIAL_MARK, &co_last ))->numname = i;


      /* Nummer der entsprechenden Stelle im Originalnetz, */
      /* lesen, wird hier nicht benoetigt                  */
      pl->inscript=(char*)ReadNumMCI( MCIfile );
      pl->label=EMPTY;

      /* Eine ggf. vorhandene E->B-Kante einfuegen */
      if (( evnum = ReadNumMCI( MCIfile ) ))
      {
	/* Transition zu evnum suchen */
        ev_last = find_T_num( net, evnum );
	/* Kante einfuegen */
        fast_insert_TP_con( net, coname, ev_last->name, TP_WEIGHT,
                            &co_last, &ev_last, &eb_last );
      }

      /* Nachbereich der Bedingung einfuegen; */
      /* solange Nummern der Transitionen im  */
      /* Nachbereich gelesen werden...        */
      while (( evnum = ReadNumMCI( MCIfile ) ))
      {
	/* Transition zu evnum suchen */
        ev_last = find_T_num( net, evnum );
	/* Kante einfuegen */
        fast_insert_PT_con( net, coname, ev_last->name, PT_WEIGHT,
                            &co_last, &ev_last, &be_last );
      }
    } /* for ... */

    /* ggf. Nummern der Cutoff-Events einlesen */
    ev = net->Transitions;
    GL_numCutoffs = 0;
    while (( num = ReadNumMCI( MCIfile ) ))
    {
      GL_numCutoffs++;

      if ((CutOffPoints))
      {
          /* in num steht die Nummer des Cutoff-Events; */ 
	  /* aus "E" und num den Namen bilden */
	  sprintf( evname, "E%u", num );

          /* in die Liste eintragen */
          InsertListElem( CutOffPoints, evname );
      }

      /* Event als Cutoff markieren */
      while ( ev && ( num > ev->numname ) )
        {
          ev = ev->nexttrans;
        } /* while */
      if ( ev && ( num == ev->numname ) )
        {
          ev->label = (MultiSet) 1;
        } /* if */

      /* korrespondierendes Event interessiert nicht */
      evnum = ReadNumMCI( MCIfile );
    }

  } /* if ( conum = ... ) */
  else
    NNError( E_RDMCI_NOCOND );

  /* falls Entfaltung keine Cutoffpunkte enthaelt, */
  /* eine(!) maximale Konfiguration einlesen, wenn */
  /* vorhanden                                     */    
  /* num: erstes Ereignis einer Konfiguration      */
  while (( num = ReadNumMCI( MCIfile ) ))
  {
    if ( !(GL_numCutoffs) && (MaxConfig) && !(*MaxConfig) )
    {
      do
      {
	/* aus "E" und num den Transitionsnamen bilden */
        sprintf( evname, "E%u", num );

	/* in die Liste eintragen */
        InsertListElem( MaxConfig, evname );
      } while (( num = ReadNumMCI( MCIfile ) ));
    }
    else
      /* evnum: weitere Ereignisse dieser Konfiguration */
      while (( evnum = ReadNumMCI( MCIfile ) ))
        skip;
  }

  /* Laenge der nachfolgenden Stellenliste, d.h. */
  /* Anzahl der Stellen im Originalnetz, lesen   */
  conum = ReadNumMCI( MCIfile );
  
  /* Laenge der nachfolgenden Transitionenliste, d.h. */
  /* Anzahl der Transitionen im Originalnetz, lesen   */
  evnum = ReadNumMCI( MCIfile );

  /* Laenge des laengsten Stellen-/Transitionsnamens */
  num = ReadNumMCI( MCIfile );

  /* Array von char-Zeigern zur Aufnahme der Zeiger auf */
  /* die Transitionsnamen anlegen                       */
  tr_names = (char**) MYmalloc( sizeof(char*) * (evnum + 1) );

  /* Speicher zur Aufnahme aller Namen reservieren */
  all_names = (char*) MYmalloc( i = (num+1) * (evnum+conum) + 2 );
  all_names = (char*) MYrealloc( all_names,
                                fread( (char*)all_names, 1, i, MCIfile )
                              );

  /* alle Stellennamen in all_names uebergehen */
  tmpchar = all_names;
  /* solange Stellennamen vorhanden... */
  while ( *tmpchar )
  {
    /* den ganzen Namen ueberlesen */
    while( *++tmpchar )
      skip;
    /* Zeiger ggf. auf den naechsten Namen setzen */
    ++tmpchar;
  }

  /* die Zeiger des Arrays tr_names jetzt auf */
  /* die Transitionsnamen in all_names setzen */
  tmpptr = tr_names;

  /* auf das erste Zeichen des ersten Transitionsnamen gehen */
  ++tmpchar;

  /* solange Namen in all_names ... */
  while ( *tmpchar )
  {
    /* Zeiger auf das erste Zeichen des Namens */
    *tmpptr = tmpchar;

    /* einen Zeiger weiter fuer evtl. naechsten Namen */
    ++tmpptr;

    /* die restlichen Zeichen des Namens uebergehen */
    while( *++tmpchar )
      skip;

    /* evtl. das erste Zeichen des naechsten Namens */
    ++tmpchar;
  }

  /* der letzte Zeiger in tr_names erhaelt NULL */
  *tmpptr = NULL;

  /* und nun die Namen aus tr_names ins Netz kopieren */
  for ( ev = net->Transitions ; ev ; ev = ev->nexttrans )
  {
    if (ev->inscript)
      ev->inscript = MYstrdup( tr_names[ ( (Cardinal32) ev->inscript ) - 1 ] );
  }

  free( (char *) tr_names );
  free( (char *) all_names );

  /* Datei schliessen */
  fclose( MCIfile );

  /* das eingelesene Netz (Entfaltung) zurueckgeben */
  return ( net );

} /* readMCI() */

/*-------------------------------------------------------------------------*/
/*--                                                                     --*/
/*-- Ende READ_MCI.C                                                     --*/
/*--                                                                     --*/
/*-------------------------------------------------------------------------*/
