
#include <string.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include "rw_pnml.h"
#include "memlib.h"
#include "convertPEP.h"
#include "readlib.h"

//TODO: Memory leaks by xmlEncodeSpecialChars, allocates new strings
int writePNML(FILE* file, LMPNet net)
{
  Place pl;
  Transition tr;
  TSEdge ts;
  STEdge st;
  Cardinal32 count;
  Coords co;

  num_net_nodes(net);

  /* Write XML header */
  f("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n");
  f("<!DOCTYPE pnml PUBLIC \"-//PEP//DTD PEP PNML 1.0//EN\"\n");
  f("\t\"pep_pnml.dtd\">\n");
  f("<pnml xmlns=\"http://www.informatik.hu-berlin.de/top/pnml\">\n");

  /* Write net header */
  fx("<net id=\"n1\" type=\"%s\">\n",CM_nettype);
  fx("  <name><text>%s</text></name>\n",xmlEncodeSpecialChars(NULL,NNinput_file));

  for (pl=net->Places;pl;pl=pl->nextplace)
  {
    co=(Coords)pl->inscript;
    fx("  <place id=\"p%d\">\n",pl->numname);
    fx("    <name><text>%s</text>",pl->name);
    /* The name position is currently not stored. */
    f("</name>\n");
    if (pl->marks)
      fx("    <initialMarking><text>%d</text></initialMarking>\n",pl->marks);
    else if (!strcmp(CM_nettype, "HLNet"))
    {
      if (co && *(char*)co->next->next->next->x)
	fx("    <sort><text>%s</text></sort>\n",
		xmlEncodeSpecialChars(NULL,(char*)co->next->next->next->x));
      if (co && *(char*)co->next->next->next->y)
	fx("    <initialMarking><text>%s</text></initialMarking>\n",
		xmlEncodeSpecialChars(NULL,(char*)((Coords)pl->inscript)->next->next->next->y));
    }
    if (pl->label)
      fx("    <label><text>%s</text></label>\n",
		(pl->label==ENTRY)?"entry":"exit");
    if (pl->inscript)
    {
      fxy("    <graphics><position x=\"%d\" y=\"%d\"/></graphics>\n",
		    co->x,co->y);
      if ((co->next) && (*(char*)co->next->next->x))
      {
	f("    <meaning>\n");
	fx("      <text>%s</text>\n", xmlEncodeSpecialChars(NULL,(char*)co->next->next->x));
	fxy("      <graphics><offset x=\"%d\" y=\"%d\"/></graphics>\n",
		co->next->x, co->next->y);
	f("    </meaning>\n");
      }
    }
    f("  </place>\n");
  }

  for (tr=net->Transitions;tr;tr=tr->nexttrans)
  {
    fx("  <transition id=\"t%d\">\n",tr->numname);
    fx("    <name><text>%s</text></name>\n",tr->name);
    if (tr->inscript)
    {
      co=(Coords)tr->inscript;
      fxy("    <graphics><position x=\"%d\" y=\"%d\" /></graphics>\n",
		    co->x,co->y);
      if ((co=co->next))
      {
	f("    <meaning>\n");
	fx("      <text>%s</text>\n", xmlEncodeSpecialChars(NULL,(char*)co->next->y));
	fxy("      <graphics><offset x=\"%d\" y=\"%d\" /></graphics>\n",
		co->x, co->y);
	f("    </meaning>\n");
	if (*(char *)co->next->x)
	  fx("    <actionterm><text>%s</text></actionterm>\n",
		xmlEncodeSpecialChars(NULL,(char*)co->next->x));
	if (*(char *)co->next->next->x)
	  fx("    <guard><text>%s</text></guard>\n",
		xmlEncodeSpecialChars(NULL,(char*)co->next->next->x));
      }
    }
    if (tr->time_label)
    {
      f("    <interval><text>");
      fx("(%.0f,", tr->time_label->low);
      if (is_inf_lft(tr))
        f("inf");
      else
        fx("%.0f", tr->time_label->high);
      f(")</text></interval>\n");
    }
    f("  </transition>\n");
  }

  for (count=0,pl=net->Places;pl;pl=pl->nextplace)
  {
    for (st=pl->postset;st;st=st->nexttrans)
    {
      fxyz("  <arc id=\"a%d\" source=\"p%d\" target=\"t%d\">\n",count++,
		      pl->numname,st->trans->numname);
      if (st->weight)
      {
	co=(Coords)st->weight;
	if (co->next)
	{
	  if (*(char*)co->next->x)
	  {
	    // variables
	    f("      <inscription>\n");
	    fx("        <text>%s</text>\n",(char*)co->next->x);
	    if ((co->x)||(co->y))
	      fxy("        <graphics><offset x=\"%d\" y=\"%d\" /></graphics>\n",co->x,co->y);
	    f("      </inscription>\n");
	  }
	  if ((int)co->next->y)
	    fx("      <visibility><text>%d</text></visibility>\n",(int)co->next->y);
	  if ((int)co->next->next->x>1)
	  {
	    f("      <weight>\n");
	    fx("        <text>%d</text>\n",(int)co->next->next->x);
	    f("      </weight>\n");
	  }
	  if ((co=co->next->next->next))
	  {
	    f("    <graphics>\n");
	    fxy("      <position page=\"1\" x=\"%d\" y=\"%d\" />\n", co->x,co->y);
//	fx("      <color><value>%d</value></color>\n",(int)co->next->x);
//	fx("      <line><value>%d</value></line>\n",(int)co->next->x);
	    f("    </graphics>\n");
	  }
	}
      }
      f("  </arc>\n");
    }
  }

  for (tr=net->Transitions;tr;tr=tr->nexttrans)
  {
    for (ts=tr->postset;ts;ts=ts->nextplace)
    {
      fxyz("  <arc id=\"a%d\" source=\"t%d\" target=\"p%d\">\n",count++,
		      tr->numname,ts->place->numname);
      if (ts->weight)
      {
	co=(Coords)ts->weight;
	if (co->next)
	{
	  if (*(char*)co->next->x)
	  {
	    f("      <inscription>\n");
	    fx("        <text>%s</text>\n",(char*)co->next->x);
	    if ((co->x)||(co->y))
	      fxy("        <graphics><offset x=\"%d\" y=\"%d\" /></graphics>\n",co->x,co->y);
	    f("      </inscription>\n");
	  }
	  if ((int)co->next->y)
	    fx("      <visibility><text>%d</text></visibility>\n",(int)co->next->y);
	  if ((int)co->next->next->x>1)
	  {
	    f("      <weight>\n");
	    fx("        <text>%d</text>\n",(int)co->next->next->x);
	    f("      </weight>\n");
	  }
	  if ((co=co->next->next->next))
	  {
	    f("    <graphics>\n");
	    fxy("      <position page=\"1\" x=\"%d\" y=\"%d\" />\n", co->x,co->y);
//	fx("      <color><value>%d</value></color>\n",(int)co->next->x);
//	fx("      <line><value>%d</value></line>\n",(int)co->next->x);
	    f("    </graphics>\n");
	  }
	}
      }
      f("  </arc>\n");
    }
  }

  f("</net>\n</pnml>\n");

 return 0;
}

static Coords parseGraphics(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr node)
{
  Coords co=Malloc(sizeof(CoordEntry));

  while (node != NULL)
  {
    if ((!xmlStrcmp(node->name, (const xmlChar *)"graphics")) &&
	(node->ns == ns))
    {
      co->x=atoi(xmlGetProp(node->xmlChildrenNode, (const xmlChar *) "x"));
      co->y=atoi(xmlGetProp(node->xmlChildrenNode, (const xmlChar *) "y"));
    }
    node=node->next;
  }
  co->next=NULL;
  return co;
}

static char* getValue(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr node)
{
  node=node->xmlChildrenNode;
  while (node != NULL)
  {
    if (((!xmlStrcmp(node->name, (const xmlChar *)"value")) || 
	(!xmlStrcmp(node->name, (const xmlChar *)"text"))) &&
	(node->ns == ns))
    {
      return xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
    }
    node=node->next;
  }
  return NULL;
}

static void parsePlace(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr node, LMPNet net)
{
  xmlChar *id;
  xmlNodePtr placenode=node;
  char	*name=NULL,
	*type=NULL,
	*imark=NULL,
	*mean=NULL,
	*label=NULL;
  Coords pos=NULL, meanpos=NULL;
  int numname;
  Place pl=NULL;
  
  id=xmlGetProp(node, (const xmlChar *) "id");
  xmlAddID(NULL, doc, id, xmlHasProp(node, (const xmlChar *) "id"));
  node = node->xmlChildrenNode;
  while (node != NULL) {
    if ((!xmlStrcmp(node->name, (const xmlChar *)"name")) &&
	(node->ns == ns))
      name = getValue(doc, ns, node);
    if ((!xmlStrcmp(node->name, (const xmlChar *)"sort")) &&
	(node->ns == ns))
      type = getValue(doc, ns, node);
    if ((!xmlStrcmp(node->name, (const xmlChar *)"initialMarking")) &&
	(node->ns == ns))
      imark = getValue(doc, ns, node);
    if ((!xmlStrcmp(node->name, (const xmlChar *)"meaning")) &&
	(node->ns == ns))
    {
      mean = getValue(doc, ns, node);
      meanpos = parseGraphics(doc, ns, node->xmlChildrenNode);
    }
    if ((!xmlStrcmp(node->name, (const xmlChar *)"graphics")) &&
	(node->ns == ns))
      pos = parseGraphics(doc, ns, node);
    node= node->next;
  }
  numname=net->NumPlaces+1;
  if (!name)
  {
    name=MYmalloc(20*sizeof(char));
    sprintf(name, "P%d", numname);
  }
  if (imark && strcmp(CM_nettype, "HLNet"))
    fast_insert_place( net, name, atoi(imark), &pl );
  else
    fast_insert_place( net, name, 0, &pl );
  placenode->_private=pl;
  if (label)
  {
    if (!strcmp(label, "entry"))
      pl->label=ENTRY;
    else if (!strcmp(label, "exit"))
      pl->label=EXIT;
    Free(label);
  }
  else
    pl->label = EMPTY;

  if (pos)
    pl->inscript=(char*) pos;
  else
  {
    pl->inscript=(char*)(pos=(Coords)Malloc(sizeof(CoordEntry)));
    pos->x=0;
    pos->y=0;
  }
  if (meanpos)
    pos->next=meanpos;
  else
  {
    pos->next=(meanpos=(Coords)Malloc(sizeof(CoordEntry)));
    meanpos->x=0;
    meanpos->y=0;
  }
  meanpos->next=(pos=(Coords)Malloc(sizeof(CoordEntry)));
  if (mean)
    pos->x=(int)mean;
  else
    pos->x=(int)NULL, pos->y=(int)NULL;
  if ((!(strcmp(CM_nettype, "HLNet")))&&(imark||type))
  {
    pos->next=(meanpos=(Coords)Malloc(sizeof(CoordEntry)));
    meanpos->x=(int)type;
    meanpos->y=(int)imark;
  }
}

static void parseTransition(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr node, LMPNet net)
{
  xmlChar *id;
  xmlNodePtr transnode=node;
  char	*name=NULL,
	*action=NULL,
	*mean=NULL,
	*guard=NULL;
  Coords pos=NULL, meanpos=NULL;
  Time_stamp *time_label=NULL;
  int numname;
  Transition tr=NULL;
  
  id=xmlGetProp(node, (const xmlChar *) "id");
  xmlAddID(NULL, doc, id, xmlHasProp(node, (const xmlChar *) "id"));
  node = node->xmlChildrenNode;
  while (node != NULL) {
    if ((!xmlStrcmp(node->name, (const xmlChar *)"name")) &&
	(node->ns == ns))
      name = getValue(doc, ns, node);
    if ((!xmlStrcmp(node->name, (const xmlChar *)"actionterm")) &&
	(node->ns == ns))
      action = getValue(doc, ns, node);
    if ((!xmlStrcmp(node->name, (const xmlChar *)"guard")) &&
	(node->ns == ns))
      guard = getValue(doc, ns, node);
    if ((!xmlStrcmp(node->name, (const xmlChar *)"meaning")) &&
	(node->ns == ns))
    {
      mean = getValue(doc, ns, node);
      meanpos = parseGraphics(doc, ns, node->xmlChildrenNode);
    }
    if ((!xmlStrcmp(node->name, (const xmlChar *)"interval")) &&
	(node->ns == ns))
    {
      char *interval;

      time_label=(Time_stamp *)MYmalloc (sizeof(Time_stamp));
      interval = getValue(doc, ns, node);
      sscanf(interval, "(%lf,%lf)",&(time_label->low),&(time_label->high));
    }
    if ((!xmlStrcmp(node->name, (const xmlChar *)"graphics")) &&
	(node->ns == ns))
      pos = parseGraphics(doc, ns, node);
    node= node->next;
  }
  numname=net->NumTransitions+1;
  if (!name)
  {
    name=MYmalloc(20*sizeof(char));
    sprintf(name, "T%d", numname);
  }
  fast_insert_transition( net, name, &tr );
  transnode->_private=tr;
  tr->label = NULL;
  tr->time_label=time_label;
  time_label=NULL;

  if (pos)
    tr->inscript=(char*) pos;
  else
  {
    tr->inscript=(char*)(pos=(Coords)Malloc(sizeof(CoordEntry)));
    pos->x=0;
    pos->y=0;
  }
  if (meanpos)
    pos->next=meanpos;
  else
  {
    pos->next=(meanpos=(Coords)Malloc(sizeof(CoordEntry)));
    meanpos->x=0;
    meanpos->y=0;
  }
  meanpos->next=(pos=(Coords)Malloc(sizeof(CoordEntry)));
  if (action)
    pos->x=(int)action;
  else
    pos->x=(int)NULL;
  if (mean)
    pos->y=(int)mean;
  else
    pos->y=(int)NULL;
  if (guard)
  {
    pos->next=(meanpos=(Coords)Malloc(sizeof(CoordEntry)));
    meanpos->x=(int)guard;
    meanpos->y=(int)NULL;
  }
}

static void parseArc(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr node, LMPNet net)
{
  xmlChar *source, *target;
  xmlAttrPtr sourceattr, targetattr;
  xmlNodePtr sourcenode, targetnode;
  char *label=NULL;
  int visibility=0, weight=1;
  Coords pos=NULL, labelpos=NULL, weightpos=NULL;
  Place pl=NULL;
  Transition tr=NULL;
  bool is_pt=True;
  STEdge ptarc=NULL;
  TSEdge tparc=NULL;

  source=xmlGetProp(node, (const xmlChar *) "source");
  sourceattr=xmlGetID(doc, source);
  if (sourceattr)
  {
    sourcenode=sourceattr->parent;
    if ((!xmlStrcmp(sourcenode->name, (const xmlChar *)"place")) &&
        (sourcenode->ns == ns))
      pl=(Place)sourcenode->_private;
    else if ((!xmlStrcmp(sourcenode->name, (const xmlChar *)"transition")) &&
        (sourcenode->ns == ns))
    {
      is_pt=False;
      tr=(Transition)sourcenode->_private;
    }
    else
    {
      printf("Wrong node type referenced. Arc is ignored!\n");
      return;
    }
  }
  else
  {
    fprintf(stderr, "Referenced node %s not found. Arc is ignored!\n", source);
    return;
  }
  target=xmlGetProp(node, (const xmlChar *) "target");
  targetattr=xmlGetID(doc, target);
  if (targetattr)
    targetnode=targetattr->parent;
    if ((!xmlStrcmp(targetnode->name, (const xmlChar *)"place")) &&
        (targetnode->ns == ns))
    {
      if (!is_pt)
        pl=(Place)targetnode->_private;
      else
      {
	printf("Arc from place to place is not allowed!\n");
	return;
      }
    }
    else if ((!xmlStrcmp(targetnode->name, (const xmlChar *)"transition")) &&
        (targetnode->ns == ns))
    {
      if (is_pt)
        tr=(Transition)targetnode->_private;
      else
      {
	printf("Arc from transition to transition is not allowed!\n");
	return;
      }
    }
  else
  {
    fprintf(stderr, "Referenced node %s not found. Arc is ignored!\n", target);
    return;
  }

  node = node->xmlChildrenNode;
  while (node != NULL) {
    if ((!xmlStrcmp(node->name, (const xmlChar *)"graphics")) &&
	(node->ns == ns))
      pos = parseGraphics(doc, ns, node);
    if ((!xmlStrcmp(node->name, (const xmlChar *)"inscription")) &&
	(node->ns == ns))
    {
      label = getValue(doc, ns, node);
      labelpos = parseGraphics(doc, ns, node->xmlChildrenNode);
    }
    if ((!xmlStrcmp(node->name, (const xmlChar *)"visibility")) &&
	(node->ns == ns))
      visibility = atoi(getValue(doc, ns, node));
    if ((!xmlStrcmp(node->name, (const xmlChar *)"weight")) &&
	(node->ns == ns))
    {
      weight = atoi(getValue(doc, ns, node));
//      weightpos = parseGraphics(doc, ns, node);
    }
    node= node->next;
  }
  if (!labelpos)
  {
    labelpos=(Coords) Malloc( sizeof( CoordEntry ) );
    labelpos->x=0;
    labelpos->y=0;
  }
  if (is_pt)
  {
    fast_insert_PT_con(net, pl->name, tr->name, 0, &pl, &tr, &ptarc);
    ptarc->weight=(Cardinal32)labelpos;
  }
  else
  {
    fast_insert_TP_con(net, pl->name, tr->name, 0, &pl, &tr, &tparc);
    tparc->weight=(Cardinal32)labelpos;
  }
  labelpos=(labelpos->next=(Coords) Malloc( sizeof( CoordEntry ) ));
  labelpos->x=(int)label;
  labelpos->y=visibility;
  labelpos=(labelpos->next=(Coords) Malloc( sizeof( CoordEntry ) ));
  labelpos->x=weight;
  if (pos)
    labelpos->next=pos;
  else
    labelpos->next=NULL;
}

static LMPNet parseNet(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr node)
{
  xmlNodePtr netnode=node;
  LMPNet net=empty_net();

  strncpy(CM_nettype, xmlGetProp(node, (const xmlChar *) "type"), 30);
  node=node->xmlChildrenNode;
  while (node!=NULL)
  {
    if ((!xmlStrcmp(node->name, (const xmlChar *) "place")) &&
	(node->ns == ns))
      parsePlace(doc, ns, node, net);
    if ((!xmlStrcmp(node->name, (const xmlChar *) "transition")) &&
	(node->ns == ns))
      parseTransition(doc, ns, node, net);
    node=node->next;
  }
  /* To be sure that all source and target nodes exist, parse arcs
   * in a second traversal. */
  node=netnode->xmlChildrenNode;
  while (node!=NULL)
  {
    if ((!xmlStrcmp(node->name, (const xmlChar *) "arc")) &&
	(node->ns == ns))
      parseArc(doc, ns, node, net);
    node=node->next;
  }

  return net;
}

static LMPNet parsePNMLFile(char *filename)
{
  xmlDocPtr doc;
  xmlNsPtr ns;
  xmlNodePtr node;

  doc = xmlParseFile(filename);
  if (doc==NULL)
  {
    return NULL;
  }

  node=xmlDocGetRootElement(doc);
  if (node==NULL)
  {
    fprintf(stderr,"empty document\n");
    xmlFreeDoc(doc);
    return NULL;
  }
  ns = xmlSearchNsByHref(doc, node , 
	(const xmlChar *) "http://www.informatik.hu-berlin.de/top/pnml");
  if (ns == NULL) {
    fprintf(stderr, "Document of the wrong type, default namespace not found\n");
    xmlFreeDoc(doc);
    return(NULL);
  }
  if ((xmlStrcmp(node->name, (const xmlChar *) "pnml")) ||
	(node->ns != ns)) {
    fprintf(stderr,"document of the wrong type, root node != pnml\n");
    xmlFreeDoc(doc);
    return(NULL);
  }

  /* First level we expect just nets */
  node = node->xmlChildrenNode;
  while ( node && xmlIsBlankNode ( node ) )
    {
      node = node->next;
    }
  if ( node == 0 )
    return ( NULL );
  if ((xmlStrcmp(node->name, (const xmlChar *) "net")) ) {
    fprintf(stderr,"document of the wrong type, was '%s', net expected",
            node->name);
    xmlFreeDoc(doc);
    return(NULL);
  }
  return(parseNet(doc, ns, node));
}

LMPNet readPNML( char* filename )
{
  LMPNet net;
  
  LIBXML_TEST_VERSION
  xmlKeepBlanksDefault(0);

  if ((net=parsePNMLFile(filename)))
  {
    xmlCleanupParser();
    return (net);
  }
  else
  {
    fprintf(stderr, "Error parsing file %s\n", filename);
    return NULL;
  }
}
