/* STG-Dateiformat lesen:            Stefan Roemer, 18.08.1995 - 28.02.1996  */
#include <string.h>
#include "rw_stg.h"
#include "readlib.h"
#include "convertPEP.h"

/*****************************************************************************/
static Cardinal32 CP_AnzTrNamen,
                  CP_AnzTrExtra,
                  CP_MaxTrExtra;
static char       ** CP_TrNamen,
                  ** CP_TrExtra;
static Place      CP_pllast;
static Transition CP_trlast,
                  * CP_TransArray = NULL,
                  * CP_TransExtraArray = NULL;


/*****************************************************************************/
/* Einlesen einer Zeichenkette aus einer Textdatei; fuehrende Leerzeichen    */
/* werden ueberlesen und nicht dem String zugeordnet.                        */
/*                                                             Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static char * Read_STG_Name( FILE * file, char * string, int maxlen,
                             bool * endofline )

#else

static char * Read_STG_Name( file, string, maxlen, endofline )

  FILE * file;
  char * string;
  int  maxlen;
  bool * endofline;

#endif

  {

    register char ch;
    register int  len = 0;
    char          * pos = NULL;

    /* skip space, newline and comments */
    while( ( ( ch = getc( file ) ) == ' ' ) || ( ch == '\n' ) || ( ch == '#' ) )
      {
        if ( ch == '\n' )
          {
            NNinput_line++;
          } /* if */
        else if ( ch == '#' )
          {
            while ( ( ch = getc( file ) ) != '\n' )
              skip;
            NNinput_line++;
          } /* else */
      } /* while */
    /* put back non-whitespace */
    ungetc( ch, file );
    while(    ( ( *string = ch = getc( file ) ) != ' ' ) && ( ch != '\n' )
           && ( ch != ',' ) && ( ch != '>' ) && ( ch != '}' ) && ( ch != '=' )
           && ( ch != EOF ) && ( len < maxlen ) )
      {
        if ( ! pos && ( ( ch == '+' ) || ( ch == '-' ) || ( ch == '~' ) ) )
          {
            pos = string;
          } /* if */
        string++;
        len++;
      } /* while */
    if ( ( ch == '>' ) || ( ch == ',' ) || ( ch == '}' ) || ( ch == '=' ) )
      ungetc( ch, file );
    if ( len == maxlen )
      {
        NNError( "read_STG(): length of identifier too long" );
        return ( NULL );
      } /* if */
    *endofline = (bool) ( *string != ' ' );
    if ( *string == '\n' )
      NNinput_line++;
    if ( !pos )
      pos = string;
    *string = '\0';
    
    return ( pos );

  } /* Read_STG_Name() */


/*****************************************************************************/
/* Lesen eines Stellenknotens.                                               */
/*                                                             Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

static bool Read_STG_Node( FILE   * file,
                           char   * string,
                           LMPNet net,
                           bool   * endofline )

#else

static bool Read_STG_Node( file, string, net, endofline )

  FILE   * file;
  char   * string;
  LMPNet net;
  bool   * endofline;

#endif

  {

    register char       ch,
                        ** nptr;
    register Cardinal32 count;
    char                * end;
    unsigned short int  index;
    Transition          * adr;

    ch = *( end = Read_STG_Name( file, string, NAMESIZE, endofline ) );
    *end = '\0';
    for ( nptr = CP_TrNamen, count = 0 ;
                 ( ++count <= CP_AnzTrNamen ) && strcmp( string, *++nptr ) ; )
      /* skip */;
    *end = ch;
    if ( count <= CP_AnzTrNamen )
      {
        if ( ( index = ch == '+'? 0 : ch == '-'? 1 : ch == '~'? 2 : 3 ) == 3 )
          {
            char msg[200];
            sprintf( msg,
            "CP_read_STG(): transition name without trailing +/-/~ found (%s)",
                                                                      string );
printf( "%s\n", msg );
/*
            NNError( msg );
*/
          } /* if */
        if ( *( end + 1 ) == '/' )
          {
            for ( nptr = CP_TrExtra, count = 0 ;
                 ( ++count <= CP_AnzTrExtra ) && strcmp( string, *++nptr ) ; )
              /* skip */;
            if ( count <= CP_AnzTrExtra )
              {
                CP_trlast = CP_TransExtraArray[ count ];
              } /* if */
            else if ( ( CP_AnzTrExtra = count ) >= CP_MaxTrExtra )
              {
                CP_TrExtra = (char **) Realloc( (char *) CP_TrExtra,
                        ( CP_MaxTrExtra += NAMES_OFFSET ) * sizeof( char * ) );
                CP_TransExtraArray = (Transition *) Realloc( (char *)
                    CP_TransExtraArray, CP_MaxTrExtra * sizeof( Transition ) );
                CP_TrExtra[ CP_AnzTrExtra ] = Strdup( string );
                CP_TransExtraArray[ CP_AnzTrExtra ] =
                             fast_insert_transition( net, string, &CP_trlast );
              } /* else if */
            else
              {
                CP_TrExtra[ CP_AnzTrExtra ] = Strdup( string );
                CP_TransExtraArray[ CP_AnzTrExtra ] =
                             fast_insert_transition( net, string, &CP_trlast );
              } /* else */
          } /* if */
        else if ( ! ( CP_trlast = *( adr = CP_TransArray + count*3 + index ) ) )
          {
            *adr = fast_insert_transition( net, string, &CP_trlast );
          } /* else if */
        return ( True );
      } /* if */
    else
      {
        if ( ! ( net->Places && ( CP_pllast = find_P( net, string ) ) ) )
          {
            fast_insert_place( net, string, 0, &CP_pllast );
          } /* if */
        return ( False );
      } /* else */

  } /* Read_STG_Node() */


/*****************************************************************************/
/***************************** read_STG **************************************/
/*****************************************************************************/
/* Neu eingefuegt                      18.08.1995 - 31.01.1996 Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

LMPNet read_STG( char * STGfilename )

#else

LMPNet read_STG( STGfilename )

  char * STGfilename;

#endif

  {

    Place               * PlArray,
                        pl;
    char                ** nptr;
    char                cmd[513],
                        name[NAMESIZE+1],
                        plname[NAMESIZE+1],
                        trname[NAMESIZE+1],
                        guard[GUARDSIZE+1];
    register FILE       * STGfile;
    register LMPNet     net;
    register Cardinal32 AnzPlNamen,
                        MaxPlNamen,
                        MaxTrNamen,
                        count;
    Transition          tr1,
                        tr2;
    STEdge              stlast = NULL;
    TSEdge              tslast = NULL;
    bool                endofline;
    Cardinal32          placecounter = 0;

    CP_trlast = NULL;
    CP_pllast = NULL;
    NNinput_file = STGfilename;
    NNinput_line = 1;
    PlArray = (Place *) Malloc( ( count = MaxPlNamen = MaxTrNamen
                           = CP_MaxTrExtra = NAMES_START ) * sizeof( Place ) );
    CP_TrNamen = (char **) Malloc( count * sizeof( char * ) );
    CP_TrExtra = (char **) Malloc( count * sizeof( char * ) );
    CP_TransExtraArray = (Transition *) Malloc( count * sizeof( Transition ) );
    AnzPlNamen = CP_AnzTrNamen = CP_AnzTrExtra = 0;
    while ( --count )
      {
        PlArray[ count ] = (Place) NULL;
      } /* while */

    /* Oeffne STG-Datei */
    if ( ! ( STGfile = fopen( STGfilename, "r" ) ) )
      {
        fprintf( stderr,
               "ERROR: CM.read_STG(): could not open file '%s' for reading.\n",
                 STGfilename );
        exit ( 4 );
      } /* if */

    net = empty_net();
    net->NumComponents = 1;

    while ( True )
      {
        Read_STG_Name( STGfile, cmd, 512, &endofline );
        if ( *cmd != '.' )
          {
            sprintf( guard,
                "read_STG(): command with prefix '.' expected, but '%s' found",
                                                                         cmd );
            NNError( guard );
            return ( NULL );
          } /* if */
        if ( ! strcmp( cmd, ".end" ) )
          {
            break;
          } /* if */
        else if ( ! strcmp( cmd, ".name" ) )
          {
            Read_STG_Name( STGfile, name, NAMESIZE, &endofline );
          } /* else if */
        else if ( ! strcmp( cmd, ".model" ) )
          {
            Read_STG_Name( STGfile, name, NAMESIZE, &endofline );
          } /* else if */
        else if ( ! (    strcmp( cmd, ".dummy" )
                      && strcmp( cmd, ".internal" )
                      && strcmp( cmd, ".inputs" )
                      && strcmp( cmd, ".outputs" ) ) )
          {
            do {
                 Read_STG_Name( STGfile, name, NAMESIZE, &endofline );
                 for ( nptr = CP_TrNamen, count = 0 ;
                    ( ++count <= CP_AnzTrNamen ) && strcmp( name, *++nptr ) ; )
                   /* skip */;
                 if ( count <= CP_AnzTrNamen )
                   {
                     sprintf( guard, "transition '%s' multiply used", name );
                     NNError( guard );
                   } /* if */
                 if ( ( CP_AnzTrNamen = count ) >= MaxTrNamen )
                   {
                     CP_TrNamen = (char **) Realloc( (char *) CP_TrNamen,
                           ( MaxTrNamen += NAMES_OFFSET ) * sizeof( char * ) );
                   } /* if */
                 CP_TrNamen[ CP_AnzTrNamen ] = Strdup( name );
               } while ( ! endofline );
          } /* else if */
        else if ( ! strcmp( cmd, ".graph" ) )
          {
            CP_TransArray = (Transition *) Malloc( 3 * ( CP_AnzTrNamen + 1 )
                                                      * sizeof( Transition ) );
            while ( ! ( feof( STGfile ) || TestSymbol( STGfile, '.' ) ) )
              {
                if ( TestSymbol( STGfile, '\n' ) )
                  {
                    continue;
                  } /* if */
                if ( Read_STG_Node( STGfile, name, net, &endofline ) )
                  {
                    tr1 = CP_trlast;
                    strcpy( trname, name );
                    if ( endofline )
                      {
                        NNError( "CP_read_STG(): missing destination name" );
                      } /* if */
                    while ( ! endofline )
                      {
                        if ( Read_STG_Node( STGfile, name, net, &endofline ) )
                          {
                            tr2 = CP_trlast;
                            sprintf( plname, "zzzplace%u",
                                                              ++placecounter );
                            CP_pllast = net->LastPlace;
                            fast_insert_place( net, plname, 0,
                                                                  &CP_pllast );
                            CP_trlast = tr1;
                            fast_insert_TP_con( net, plname, trname, 1,
                                             &CP_pllast, &CP_trlast, &tslast );
                            CP_trlast = tr2;
                            fast_insert_PT_con( net, plname, name, 1,
                                             &CP_pllast, &CP_trlast, &stlast );
                          } /* if */
                        else
                          {
                            CP_trlast = tr1;
                            fast_insert_TP_con( net, name, trname, 1,
                                             &CP_pllast, &CP_trlast, &tslast );
                          } /* else */
                      } /* while */;
                  } /* if */
                else
                  {
                    /* Place name found */
                    strcpy( plname, name );
                    if ( endofline )
                      {
                        NNError( "CP_read_STG(): missing destination name" );
                      } /* if */
                    while ( ! endofline )
                      {
                        if ( ! Read_STG_Node( STGfile, name, net, &endofline ) )
                          {
                            NNError( "CP_read_STG(): transition name expected" );
                          } /* if */
                        fast_insert_PT_con( net, plname, name, 1,
                                             &CP_pllast, &CP_trlast, &stlast );
                      } /* while */;
                  } /* else */
              } /* while */
            if ( !feof( STGfile ))
              ungetc( '.', STGfile );
            Free( CP_TransArray );
          } /* else if */
        else if ( ! strcmp( cmd, ".marking" ) )
          {
            if ( ! TestSymbol( STGfile, '{' ) )
              {
                NNError( "read_STG(): '{' expected" );
              } /* if */
            while ( ! TestSymbol( STGfile, '}' ) )
              {
                if ( TestSymbol( STGfile, '<' ) )
                  {
                    if ( ! Read_STG_Node( STGfile, name, net, &endofline ) )
                      {
                        NNError( "CP_read_STG(): transition name expected" );
                      } /* if */
                    tr1 = CP_trlast;
                    ReadSymbol( STGfile, ',' );
                    if ( ! Read_STG_Node( STGfile, name, net, &endofline ) )
                      {
                        NNError( "CP_read_STG(): transition name expected" );
                      } /* if */
                    ReadSymbol( STGfile, '>' );
                    for ( tslast = tr1->postset ; tslast ;
                                                   tslast = tslast->nextplace )
                      {
                        if (    tslast->place
                             && ! tslast->place->preset->nexttrans
                             && tslast->place->postset
                             && ! tslast->place->postset->nexttrans
                             && tslast->place->postset->trans == CP_trlast )
                          {
                            tslast->place->marks++;
                            break;
                          } /* if */
                      } /* for */
                  } /* if */
                else
                  {
                    if ( Read_STG_Node( STGfile, name, net, &endofline ) )
                      {
                        NNError( "CP_read_STG(): place name expected" );
                      } /* if */
                    CP_pllast->marks++;
                  } /* else */
              } /* while */
          } /* else if */
        else if ( ! strcmp( cmd, ".capacity" ) )
          {
            while ( ! TestSymbol( STGfile, '.' ) )
              {
                pl = NULL;
                if ( TestSymbol( STGfile, '<' ) )
                  {
                    if ( ! Read_STG_Node( STGfile, name, net, &endofline ) )
                      {
                        NNError( "CP_read_STG(): transition name expected" );
                      } /* if */
                    tr1 = CP_trlast;
                    ReadSymbol( STGfile, ',' );
                    if ( ! Read_STG_Node( STGfile, name, net, &endofline ) )
                      {
                        NNError( "CP_read_STG(): transition name expected" );
                      } /* if */
                    ReadSymbol( STGfile, '>' );
                    for ( tslast = tr1->postset ; tslast ;
                                                   tslast = tslast->nextplace )
                      {
                        if (    tslast->place
                             && ! tslast->place->preset->nexttrans
                             && tslast->place->postset
                             && ! tslast->place->postset->nexttrans
                             && tslast->place->postset->trans == CP_trlast )
                          {
                            pl = tslast->place;
                            break;
                          } /* if */
                      } /* for */
                  } /* if */
                else
                  {
                    if ( Read_STG_Node( STGfile, name, net, &endofline ) )
                      {
                        NNError( "CP_read_STG(): place name expected" );
                      } /* if */
                    pl = CP_pllast;
                  } /* else */
                ReadSymbol( STGfile, '=' );
                count = ReadNumber( STGfile );
                if ( pl )
                  {
/*
                    pl->capacity = count;
*/
                  } /* if */
              } /* while */
          } /* else if */
        else
          {
            sprintf( name, "read_STG(): unknown keyword '%s'", cmd );
            NNError( name );
            return ( NULL );
          } /* else */
      } /* while */

    Free( PlArray );
    for ( nptr = CP_TrNamen, count = 0 ; ++count <= CP_AnzTrNamen ;
                                                              Free( *++nptr ) )
      /* skip */;
    Free( CP_TrNamen );
    for ( nptr = CP_TrExtra, count = 0 ; ++count <= CP_AnzTrExtra ;
                                                              Free( *++nptr ) )
      /* skip */;
    Free( CP_TrExtra );
    Free( CP_TransArray );
    Free( CP_TransExtraArray );

    fclose( STGfile );

    return( net );

  } /* read_STG() */


/*****************************************************************************/
/***************************** write_STG *************************************/
/*****************************************************************************/
/*                                                  18.08.1995 Stefan Roemer */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void write_STG( LMPNet net, char * STGfilename )

#else

void write_STG( net, STGfilename )
  /* Gibt die mit net bezeichnete Matrix als Inzidenzmatrix nach 'file' aus  */

  LMPNet net;
  char   * STGfilename;

#endif

  {

    register Place      pl;
    register Transition tr;
    register STEdge     st;
    register TSEdge     ts;
    Cardinal32          count,
                        x1,
                        nx;
    Coords              co;
    char                * ptr;
    FILE                * file;

    if ( ! ( file = fopen( STGfilename, "w" ) ) )
      {
        NNError( "Could not open file for writing" );
      } /* if */
    if ( ! CM_storeCoordinates )
      {
        NNError( "Internal Error: 'CM_storeCoordinates' not set" );
      } /* if */

    f( "STG\n" );
    fx( "%s\n", CM_nettype );
    f( "FORMAT_N\n" );
    f( "DPL s7n10@-9t2\n" );
    f( "DTR s7n10@-9t2\n" );
    f( "DPT w1t1\n" );

    f( "PL\n" );
    for ( count = 0, pl = net->Places ; pl ; pl = pl->nextplace )
      {
        pl->numname = ++count;
        fx( "\"%s\"", pl->name );
        co = (Coords) pl->inscript;
        fxy( "%u@%u", co->x, co->y );
        if ( pl->label && ( ! strcmp( CM_nettype, "PetriBox" )
                                         || ! strcmp( CM_nettype, "HLNet" ) ) )
          {
            fx( "%c", ( pl->label == ENTRY ? 'e'
                                           : pl->label == EXIT ? 'x' : ' ' ) );
          } /* if */
        else if ( pl->marks )
          {
            fx( "M%u", pl->marks );
          } /* else if */
        if ( ( co = co->next ) && ( co = co->next ) )
          {
            if ( *( ptr = (char *) co->x ))
              fx( "b\"%s\"", ptr );
            if ( *( ptr = (char *) co->y ))
              fx( "R\"%s\"", ptr );
            if (( co = co->next ))
              {
                if ( *( ptr = (char *) co->x ))
                  fx( "Z\"%s\"", ptr );
                if ( *( ptr = (char *) co->y ))
                  fx( "z\"%s\"", ptr );
                if (( co = co->next ))
                  {
                    if ( ( x1 = (Cardinal32) co->x) )
                      fx( "m%u", x1 );
                  } /* if */
              } /* if */
          } /* if */
        f( "\n" );
      } /* for ( pl ... ) */

    f( "TR\n" );
    for ( count = 0, tr = net->Transitions ; tr ; tr = tr->nexttrans )
      {
        tr->numname = ++count;
        fx("\"%s\"",tr->name);
        co = (Coords) tr->inscript;
        fxy( "%u@%u", co->x, co->y );
        if ( tr->label && ( ! strcmp( CM_nettype, "PetriBox" )
                                         || ! strcmp( CM_nettype, "HLNet" ) ) )
          {
            fx( "%s", "r" + ! ( tr->label && ( *tr->label == 1 ) ) );
          } /* if */
        if ( ( co = co->next ) && ( co = co->next ) )
          {
            if ( *( ptr = (char *) co->x ))
              fx( "i\"%s\"", ptr );
            if ( *( ptr = (char *) co->y ))
              fx( "b\"%s\"", ptr );
            if (( co = co->next ))
              {
                if ( *( ptr = (char *) co->x ))
                  fx( "g\"%s\"", ptr );
                if ( *( ptr = (char *) co->y ))
                  fx( "R\"%s\"", ptr );
              } /* if */
          } /* if */
        f( "\n" );
      } /* for ( tr ... ) */
                                             /* Liste der TS-Kanten ausgeben */
    f( "TP\n" );
    for ( tr = net->Transitions ; tr ; tr = tr->nexttrans )
      {
        for ( ts = tr->postset ; ts ; ts = ts->nextplace )
          {
            pl = ts->place;
            fxy( "%u<%u", tr->numname, pl->numname );
            co = (Coords) ts->weight;
            if ( co && ( co = co->next ) )
              {
                if ( *( ptr = (char *) co->x ))
                  fx( "p\"%s\"", ptr );
                if ( ( nx = (Cardinal32) co->y) )
                  fx( "v%u", nx );
                if (( co = co->next ))
                  {
                    if ( ( nx = (Cardinal32) co->x )>1 )
                      fx( "w%u", nx );
                  } /* if */
              } /* if */
            f( "\n" );
          } /* for */
      } /* for */

    f( "PT\n" );
    for ( pl = net->Places ; pl ; pl = pl->nextplace )
      {
        for ( st = pl->postset ; st ; st = st->nexttrans )
          {
            tr = st->trans;
            fxy( "%u>%u", pl->numname, tr->numname );
            co = (Coords) st->weight;
            if ( co && ( co = co->next ) )
              {
                if ( *( ptr = (char *) co->x ))
                  fx( "p\"%s\"", ptr );
                if ( ( nx = (Cardinal32) co->y ))
                  fx( "v%u", nx );
                if (( co = co->next ))
                  {
                    if ( ( nx = (Cardinal32) co->x )>1 )
                      fx( "w%u", nx );
                  } /* if */
              } /* if */
            f( "\n" );
          } /* for */
      } /* for */

    f( "TX\n" );
    f( "\n" );

    fclose( file );

  } /* write_STG() */
