/*****************************************************************************/
/* FILE pep2ina.c                                                            */
/*****************************************************************************/
/* Konvertierungsprogramm zur Umwandlung von Petri-Netzen aus PEP in das     */
/* Dateiformat des INA Programms                                             */
/*                                           Stephan Merkel, 07.11.94        */
/* Added time file generation, July 2000, Christian Stehno		     */
/*****************************************************************************/


/*****************************************************************************/
/*                                                                           */
/*                             header files                                  */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "neunetze.h"
#include "convertPEP.h"


/*****************************************************************************/
/*                                                                           */
/*                               defines                                     */
/*                                                                           */
/*****************************************************************************/

#define PGWIDTH 80
#define OMEGASTRING "oo"
#define STDNETNUM 42

#define COLOURED 1
#define REFERENCES 2
#define WITH_TIME 4

int GL_flags=0;

#define MEMOUT {fprintf(stderr, "out of memory\n"); exit(1);}

/* Boah, cool ey */
#define GETPREF(X) ((char *)((Coords)(X->inscript))->next->next->y)
#define GETTREF(X) ((char *)((Coords)(X->inscript))->next->next->next->y)

/*****************************************************************************/
/*                                                                           */
/*                              functions                                    */
/*                                                                           */
/*****************************************************************************/

struct col_nod {
	char *ref_str;
	Cardinal32 col_num;
	struct col_el *col_list;
	struct col_nod *next;
};

struct col_el {
	Cardinal32 el_num;
	struct col_el *next_el;
};

/*****************************************************************************/
/**************************** usage_err() ************************************/
/*****************************************************************************/
/* prints the message *msg and information about the usage of pep2ina to the */
/* standard output and halts the program returning an error exit status      */
/*****************************************************************************/

#if defined (__STDC__) || defined (__cplusplus)

void usage_err( char * msg )

#else

void usage_err( msg )

char * msg;

#endif

{

	printf( "\n%s\n\n", msg );
	printf( "Usage: \n" );
	printf( "\t\t pep2ina [-c] [-r] [-t] [path]PEP_file [path]INA_file \n\n");
	printf( "where PEP_file is a file containing a net in the PEP format ");
	printf( "and INA_file\n" );
	printf( "is the target file. Path may be a valid pathname.\n" );
        printf( "-c: write coloured nets\n");
	printf( "-r: write references\n");
	printf( "-t: write time file\n");
	printf( "\n\n" );

	exit(1);

} /* usage_err() */


/*****************************************************************************/
/************************* is_dos_ch() ***************************************/
/*****************************************************************************/
/* returns True if ch is a valid character for a DOS filename, else False    */
/*****************************************************************************/

#if defined (__STDC__) || defined (__cplusplus)

bool is_dos_ch( char ch )

#else

bool is_dos_ch( ch )

char ch;

#endif

{

	if ( isalnum( ch ) || strchr( "-_$%(){#&@!}\'`^", ch ) )
	{
		return ( True );
	} /* if */
	else
	{
		return ( False );
	} /* else */

} /* is_dos_ch() */


/*****************************************************************************/
/************************ is_ina_pt_name() ***********************************/
/*****************************************************************************/
/* returns True, if *name is a valid INA st-net filename, else False         */
/*****************************************************************************/

#if defined (__STDC__) || defined (__cplusplus)

bool is_ina_pt_name( char * name )

#else

bool is_ina_pt_name( name )

char * name;

#endif

{

	int i = 0;

	if ( isdigit( *name ) )
	{
		return ( False );
	} /* if */

	while ( is_dos_ch( *name ) ) {
		i++;
		name++;
	} /* while */

	if ( ( i > 8 ) || ( i == 0 ) ) {
		return ( False );
	} /* if */

	if ( strcmp( name, ".pnt" ) ) {
		return ( False );
	} /* if */

	return ( True );
} /* is_ina_pt_name() */


/*****************************************************************************/
/************************** count_digits() ***********************************/
/*****************************************************************************/

#if defined (__STDC__) || defined (__cplusplus)

int count_digits( int num )

#else

int count_digits( num )

int num;

#endif

{
	int i = 1;

	while ( num /= 10 ) {
		i++;
	} /* while */
	return ( i );
} /* count_digits() */



#if defined (__STDC__) || defined (__cplusplus)

void ins_node_col(struct col_nod **head, char *ref, Cardinal32 num)

#else

void ins_node_col(head, ref, num)

struct col_nod **head;
char *ref;
Cardinal32 num;

#endif

{
	Cardinal32 act_col_num=0;
	struct col_el **act_el;

	for (; *head; head = &((*head)->next)) {
		act_col_num = (*head)->col_num;
		if (*ref && *((*head)->ref_str) && strcmp((*head)->ref_str, ref)==0) break;
	}
	if (!*head) {
		act_col_num++;
		*head = MYmalloc(sizeof(struct col_nod));
		if (!*head) MEMOUT;
		(*head)->ref_str = MYstrdup(ref);
		if (!(*head)->ref_str) MEMOUT;
		(*head)->next = (struct col_nod *)0;
		(*head)->col_list = (struct col_el *)0;
		(*head)->col_num = act_col_num;
	}
	for (act_el = &((*head)->col_list); *act_el; act_el = &((*act_el)->next_el) ); 
	*act_el = MYmalloc(sizeof(struct col_el));
	if (!*act_el) MEMOUT;
	(*act_el)->next_el = (struct col_el *)0;
	(*act_el)->el_num = num;
}

#if defined (__STDC__) || defined (__cplusplus)

void print_col(FILE *file, struct col_nod *node)

#else

void print_col(file, node)

FILE *file;
struct col_nod *node;

#endif

{
	struct col_el *act_el,*ptr;

	fprintf(file, "%5d:        ", node->col_num);
	for (act_el = node->col_list; act_el; act_el = ptr) {
		fprintf(file, "%5d ", act_el->el_num);
		ptr=act_el->next_el;
		free(act_el);
	}
	fprintf(file, "\n");
}

/*****************************************************************************/
/************************** write_INA() **************************************/
/*****************************************************************************/
/* writes the net *net represented in the PEP format into a file called      */
/* *filename in the INA representation                                       */
/*****************************************************************************/
/* Caution: If no valid DOS filename is found behind an optional path in     */
/* *filename, the net name printed into the INA file may be void!            */
/*****************************************************************************/

#if defined (__STDC__) || defined (__cplusplus)

void write_INA( char * filename, LMPNet net, char *bpn_ref_name, char *ll_ref_name, int cols, int refs )

#else

void write_INA( filename, net, bpn_ref_name, ll_ref_name, cols, refs )

char * filename;
LMPNet net;
char * bpn_ref_name;
char * ll_ref_name;
int cols, refs;

#endif

{
	int        i = 0,
	cap = OMEGA,
	prio = 0,
	time = 0,
	row_pos;
	FILE       *file, *bpn_ref_file=stdout, *ll_ref_file=stdout;
	char       net_name[9];
	char       net_name_long[64];
	char *     curr_ch;
	char *     old_curr_ch;
	Place      curr_pl;
	Transition curr_tr;
	TSEdge     curr_ts;
	STEdge     curr_st;
	struct col_nod *place_cols = (struct col_nod *)0, *trans_cols = (struct col_nod *)0;
	struct col_nod *cur_col;

	num_net_nodes( net );
	if (( curr_ch = strrchr( filename, '/' ) )) {
		curr_ch++;
	} /* if */
	else {
		curr_ch = filename;
	} /* else */

	old_curr_ch = curr_ch;
	while ( is_dos_ch( *curr_ch ) && ( i < 8 ) ) {
		/* fehler in ina bezeichnern sind keine {}| erlaubt */
		net_name[i++] = *(curr_ch++);
	} /* while */

	net_name[i] = '\0';

	i = 0;
	while ( old_curr_ch[i] && i < 64 && old_curr_ch[i] != '.') {
		net_name_long[i] = old_curr_ch[i];
		i++;
	} /* while */
	net_name_long[i] = '\0';

	if ( ! ( file = fopen( filename, "w" ) ) ) {
		usage_err( "Cannot open INA_file." );
	} /* if */

	if (refs) {
		if ( ! ( bpn_ref_file = fopen( bpn_ref_name, "w" ) ) ) {
			usage_err( "Cannot open bpn_ref_file." );
		} /* if */
		if ( ! ( ll_ref_file = fopen( ll_ref_name, "w" ) ) ) {
			usage_err( "Cannot open ll_ref_file." );
		} /* if */
	}

	/***************** write node list ***********************************/

	fprintf( file, "P   M    PRE,POST    NETZ   %d:%s\n", STDNETNUM, net_name );
	curr_pl = net->Places;

	while ( curr_pl ) {
		fprintf( file, "%3d %-5d", curr_pl->numname,\
							 marking( net, curr_pl->name ) );

		row_pos = 10;

		/******************* write preset ****************************/

		curr_ts = curr_pl->preset;

		while ( curr_ts ) {
			if ( curr_ts->weight ) {
				if ( curr_ts->weight > 1 ) {
					row_pos += ( count_digits( curr_ts->weight ) + 1 );
				} /* if */
					/* das ist mindestens bei ina fuer sun nicht noetig */
					/* macht's aber schoener Lutz */
				/*
				if ( ( row_pos += count_digits( curr_ts->trans->numname ) ) > ( PGWIDTH - 3 ) ) {
					fprintf( file, " >\n");
					row_pos = 0;
				} 
				*/
				fprintf( file, " %d", curr_ts->trans->numname );

				if ( curr_ts->weight > 1 ) {
					fprintf( file, ":%d", curr_ts->weight );
				} /* if */

				row_pos++;

			} /* if ( curr_ts->weight ) */

			curr_ts = curr_ts->nexttrans;

		} /* while ( curr_ts ) */

		/******************** write postset **************************/

		curr_st = curr_pl->postset;

		/* wenn keine post-list darf kein komma */
		/* Lutz Pogrell Mittwoch, 15. Februar 1995, 11:05:35 Uhr MET */

		if (curr_st) {
			fprintf( file, "," );
		}

		while ( curr_st ) {
			if ( curr_st->weight ) {
				if ( curr_st->weight > 1 ) {
					row_pos += ( count_digits( curr_st->weight ) + 1 );
				} /* if */
				/* das ist mindestens bei ina fuer sun nicht noetig */
				/* macht's aber schoener Lutz */
				/*
				if ( ( row_pos += count_digits( curr_st->trans->numname ) ) > ( PGWIDTH - 3 ) ) {
					fprintf( file, " >\n");
					row_pos = 0;
				}
				*/
				fprintf( file, " %d", curr_st->trans->numname );

				if ( curr_st->weight > 1 ) {
					fprintf( file, ":%d", curr_st->weight );
				} /* if */

				row_pos++;

			} /* if ( curr_st->weight ) */

			curr_st = curr_st->nexttrans;

		} /* while ( curr_st ) */

		fprintf( file, "\n" );
		curr_pl = curr_pl->nextplace;

	} /* while ( curr_pl ) */

	fprintf( file, "@\n" );

	/******************** write place list *******************************/

	fprintf( file, "place nr.     name  capacity  time\n" );
	curr_pl = net->Places;

	while ( curr_pl ) {
		/* OMEGASTRING neu eingefuert */
		if (cap == OMEGA) {
			fprintf( file, "%8d: %-9s %8s  %4d\n", 
			    curr_pl->numname, curr_pl->name, OMEGASTRING, time );
		} else {
			fprintf( file, "%8d: %-9s %8u  %4d\n", 
			    curr_pl->numname, curr_pl->name, cap, time );
			/* die laenge von genau acht Ziffern ist eigentlich nicht noetig */
		}
		if (refs) {
			fprintf(bpn_ref_file, "set ref(%s,place,%d) \"%s\"\n", 
							net_name_long, curr_pl->numname, GETPREF(curr_pl));
			fprintf(ll_ref_file, "set ll_ref(%s,place,%d) \"%s\"\n", 
							net_name_long, curr_pl->numname, curr_pl->name);
		}
		if (cols) {
			ins_node_col(&place_cols, GETPREF(curr_pl), curr_pl->numname);
		}
		curr_pl = curr_pl->nextplace;

	} /* while ( curr_pl ) */

	fprintf( file, "@\n" );

	/********************* write transition list *************************/

	fprintf( file, "trans nr.     name  priority  time\n" );
	curr_tr = net->Transitions;

	while ( curr_tr ) {
		fprintf( file, "%8d: %-9s %8d  %4d\n", curr_tr->numname, curr_tr->name,
                       prio, time);
		if (refs) {
			fprintf(bpn_ref_file, "set ref(%s,trans,%d) \"%s\"\n", 
							net_name_long, curr_tr->numname, GETTREF(curr_tr));
			fprintf(ll_ref_file, "set ll_ref(%s,trans,%d) \"%s\"\n", 
							net_name_long, curr_tr->numname, curr_tr->name);
		}
		if (cols) {
			ins_node_col(&trans_cols, GETTREF(curr_tr), curr_tr->numname);
		}
		curr_tr = curr_tr->nexttrans;

	} /* while ( curr_tr ) */

	fprintf( file, "@\n" );
	if (cols) {
		struct col_nod* ptr;
		fprintf( file, "AGGREGATION:\n" );
		fprintf( file, "places:\n" );
		for (cur_col = place_cols; cur_col; cur_col = ptr) {
			print_col(file, cur_col);
			ptr=cur_col->next;
			free(cur_col->ref_str);
			free(cur_col);
		}
		fprintf( file, "@\n" );
		fprintf( file, "transitions:\n" );
		for (cur_col = trans_cols; cur_col; cur_col = ptr) {
			print_col(file, cur_col);
			ptr=cur_col->next;
			free(cur_col->ref_str);
			free(cur_col);
		}
		fprintf( file, "@\n" );
	}

	fclose(file);

	if (GL_flags & WITH_TIME)
	{
		strtok(filename, ".");
		strcat(filename, ".tim");
		file=fopen(filename, "w");
		
		for(curr_tr=net->Transitions;curr_tr;curr_tr=curr_tr->nexttrans) {
			fprintf( file, "  %d:",curr_tr->numname);
			fprintf( file, "%d,", (curr_tr->time_label==NULL)?
				0:(int)curr_tr->time_label->low);
			if (is_inf_lft(curr_tr))
			  fprintf( file, "oo\n");
			else
			  fprintf( file, "%d\n",(int)curr_tr->time_label->high);
		}
		fclose(file);
	}
} /* write_INA() */


#if defined (__STDC__) || defined (__cplusplus)

static void parse_args(int argc, char **argv, char **ll_np, char **ina_np, 
		       char **ref_np, char **ll_ref_np, int *cop, int *rep)

#else

static void parse_args(argc, argv, ll_np, ina_np, ref_np, ll_ref_np, cop, rep)

int argc;
char **argv;
char **ll_np;
char **ina_np;
char **ref_np;
char **ll_ref_np;
int *cop, *rep;

#endif

{
	int i;
	char *name,*d, *e;

	for (i = 1; i < argc; i++) {
		if (strcmp(argv[i], "-c")==0) {
			*cop = 1;
			GL_flags |= COLOURED;
			continue;
		}
		if (strcmp(argv[i], "-r")==0) {
			*rep = 1;
			GL_flags |= REFERENCES;
			continue;
		}
		if (strcmp(argv[i], "-t")==0) {
			GL_flags |= WITH_TIME;
			continue;
		}
		if (argv[i][0] == '-') {
			usage_err( "unknown option!" );
		}
		break;
	}
	if (i == argc) {
		usage_err( "name required!" );
	}
	if ((d = strchr(argv[i], '.'))) {
		*ll_np = argv[i];
	} else {
		name = calloc(strlen(argv[i]) + 8,1);
		strcpy(name, argv[i]);
		strcat(name, ".ll_net");
		*ll_np = name;
	}
	if (i+1 < argc) {
		if ((e = strchr(argv[i+1], '.'))) {
			if (*rep) {
				*ref_np = calloc((e - argv[i+1]) + 14,1);
				strncpy(*ref_np, argv[i+1], e - argv[i+1]);
				strcat(*ref_np, ".bpn2_ina_ref");
				*ll_ref_np = calloc((e - argv[i+1]) + 12,1);
				strncpy(*ll_ref_np, argv[i+1], e - argv[i+1]);
				strcat(*ll_ref_np, ".ll_ina_ref");
			}
			*ina_np = argv[i+1];
		} else {
			name = calloc(strlen(argv[i+1]) + 5,1);
			strcpy(name, argv[i+1]);
			if (*rep) {
				*ref_np = calloc(strlen(argv[i+1]) + 14,1);
				strcpy(*ref_np, name);
				strcat(*ref_np, ".bpn2_ina_ref");
				*ll_ref_np = calloc(strlen(argv[i+1]) + 12,1);
				strcpy(*ll_ref_np, name);
				strcat(*ll_ref_np, ".ll_ina_ref");
			}
			strcat(name, (*cop ? ".cnt" : ".pnt"));
			*ina_np = name;
		}
	} else {
		if (d) {
			name = calloc((d - argv[i]) + 5,1);
			strncpy(name, argv[i], d - argv[i]);
			name[d - argv[i]+1] = '\0';
			if (*rep) {
				*ref_np = calloc((d - argv[i]) + 14,1);
				strcpy(*ref_np, name);
				strcat(*ref_np, ".bpn2_ina_ref");
				*ll_ref_np = calloc((d - argv[i]) + 12,1);
				strcpy(*ll_ref_np, name);
				strcat(*ll_ref_np, ".ll_ina_ref");
			}
			strcat(name, (*cop ? ".cnt" : ".pnt"));
			*ina_np = name;
		} else {
			name = calloc(strlen(argv[i]) + 5,1);
			strcpy(name, argv[i]);
			if (*rep) {
				*ref_np = calloc(strlen(argv[i]) + 14,1);
				strcpy(*ref_np, name);
				strcat(*ref_np, ".bpn2_ina_ref");
				*ll_ref_np = calloc(strlen(argv[i]) + 12,1);
				strcpy(*ll_ref_np, name);
				strcat(*ll_ref_np, ".ll_ina_ref");
			}
			strcat(name, (*cop ? ".cnt" : ".pnt"));
			*ina_np = name;
		}
	}
}


/*****************************************************************************/
/*************************** main ********************************************/
/*****************************************************************************/
/* creates a file called *arg_list[2] containing the st-net *arg_list[1] as a*/
/* PEP file in the INA format if exactly 2 valid parameters are given, else  */
/* a message about correct usage is printed to the standard output           */
/*****************************************************************************/

#if defined (__STDC__) || defined (__cplusplus)

int main( int arg_num, char * arg_list[] )

#else

int main( arg_num, arg_list )

int arg_num;
char * arg_list[];

#endif

{

	char * ina_name = NULL;
	char * bpn_ref_name = NULL;
	char * ll_ref_name = NULL;
	char * ll_name = NULL;
	int colored = 0, refs = 0;
	FILE * file;
	LMPNet net;

	parse_args(arg_num, arg_list, &ll_name, &ina_name, &bpn_ref_name, &ll_ref_name, &colored, &refs);

	if (( file = fopen( ina_name, "w" ) )) {
		fclose( file );
		remove( ina_name );
	} /* if */
	else {
		usage_err( "Cannot create INA_file. Possibly wrong path." );
	} /* else */
	CM_storeReferences = colored || refs;
	net = read_PEP( ll_name );

	if ( ! net ) {
		usage_err( "First parameter is not a valid PEP net." );
	} /* if */

	write_INA( ina_name, net, bpn_ref_name, ll_ref_name, colored, refs );

	remove_net( &net );

	free(ina_name);
	exit(0);

} /* main() */


/*****************************************************************************/
/* END OF FILE pep2ina.c                                                     */
/*****************************************************************************/
