/*****************************************************************************/
/* FILE pep2lola.c                                                           */
/*****************************************************************************/
/* Converter between Petri nets in PEP file format to those in LoLA file     */
/* format and vice versa. Based on pep2ina.                                  */
/*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "convertPEP.h"

/*****************************************************************************/
/*                               defines                                     */
/*****************************************************************************/

#define COLOURED 1
#define REFERENCES 2
#define WITH_TIME 4

int GL_flags=0;

/*****************************************************************************/
/*                              functions                                    */
/*****************************************************************************/

/*****************************************************************************/
/**************************** usage_err() ************************************/
/*****************************************************************************/
/* prints the message *msg and information about the usage on                */
/* standard output and stops the program returning an error exit status      */
/*****************************************************************************/

void usage_err( char * msg )
{
	printf( "\n%s\n", msg );
	printf( "Usage:" );
	printf( "\t\tpep2lola <PEP_file> <LoLA_file> \n\n");

	exit(1);
}

/*****************************************************************************/
/************************** write_lola ***************************************/
/*****************************************************************************/
/* writes the net *net represented in the PEP format into a file called      */
/* *filename in LoLA representation                                          */
/*****************************************************************************/

void write_lola( LMPNet net, char * filename)
{
	bool       first=True;
	FILE       *file;
	Place      curr_pl;
	Transition curr_tr;
	TSEdge     curr_ts;
	STEdge     curr_st;

	num_net_nodes( net );
	if ( ! ( file = fopen( filename, "w" ) ) ) {
		usage_err( "Cannot open INA_file." );
	} /* if */

	/***************** write node list ***********************************/

	fprintf( file, "PLACE ");
	for (curr_pl = net->Places;curr_pl; curr_pl=curr_pl->nextplace)
	{
		fprintf( file, "P%d%s", curr_pl->numname,curr_pl->nextplace?", ":"");
	}
	fprintf(file, ";\n");

	first=True;
	fprintf( file, "MARKING ");
	for (curr_pl = net->Places;curr_pl; curr_pl=curr_pl->nextplace)
	{

	if (curr_pl->marks)
	{
		fprintf( file, "%sP%d: %d", (first)?"":", ",curr_pl->numname,curr_pl->marks);
		first=False;
	}
	}
	fprintf(file, ";\n");

	for(curr_tr = net->Transitions;curr_tr;curr_tr=curr_tr->nexttrans)
	{
		fprintf( file, "TRANSITION T%d\n",curr_tr->numname );
		fprintf( file, "CONSUME ");
		for(curr_st = curr_tr->preset;curr_st;curr_st=curr_st->nextplace)
			fprintf( file, " P%d: %d%s", curr_st->place->numname,curr_st->weight?curr_st->weight:1,curr_st->nextplace?",":"" );
		fprintf(file, ";\n");

		fprintf( file, "PRODUCE ");
		for(curr_ts = curr_tr->postset;curr_ts;curr_ts=curr_ts->nextplace)
			fprintf( file, " P%d: %d%s", curr_ts->place->numname,curr_ts->weight?curr_ts->weight:1,curr_ts->nextplace?",":"" );
		fprintf(file, ";\n");

	}

	fclose(file);

} /* write_LoLA() */

/*************************** main ********************************************/
int main( int argc, char * argv[] )
{
	LMPNet net;

	if (argc!=2)
		usage_err( "Not enough files" );
	
	net = read_PEP( argv[0] );

	if ( ! net ) {
		usage_err( "First parameter is not a valid PEP net." );
	}

	write_lola(net, argv[1]);

	remove_net(&net);
	exit(0);
} /* main() */

/*****************************************************************************/
/* END OF FILE pep2lola.c                                                    */
/*****************************************************************************/
