#include "lp_misc.h"

char *cplex_mip_msg[] = {
	"Optimal integer solution found.",
	"Optimal sol. within epgap or epagap tolerance found.",
       	"Integer infeasible.",
	"Mixed integer solutions limit exceeded.",
	"Node limit exceeded, integer solution exists.",
     	"Node limit exceeded, no integer solution exists.",
     	"Time limit exceeded, integer solution exists.",
     	"Time limit exceeded, no integer solution exists.",
     	"Unrecoverable failure, integer solution exists.",
     	"Unrecoverable failure, no integer solution exists.",
     	"Treememory limit, integer solution exists.",
     	"Treememoty limit, no integer solution exists.",
     	"Aborted, integer solution exists.",
     	"Aborted, no integer solution exists.",
     	"Problem optimal with unscaled infeasibilities.",
     	"Out of memory, no tree, solution may exist." };

char *cplex_msg[] = {
	"Cplex simplex msg (see reference manual).",
 	"Something really bad happend ;-( ." };

REAL *init_marking=NULL;

unsigned int max_tokens = 1;

FILE *out_f;

#if defined(__STDC__) || defined (__cplusplus)
int print_sequence (LMPNet net, DSet fire, FILE *out_f)
#else
int print_sequence (net, fire, out_f)
#endif
{
	DSet d = fire,
	     single = SETalloc (1);
	Place pl = net->Places;
	Transition t;
	STEdge pre;
	TSEdge post;

	while ( pl )
	  { if ( pl->label == ENTRY )
	      pl->marks = 1;
	    else
	      pl->marks = 0;
	    pl = pl->nextplace;
	  };

 	fprintf (out_f, "_SEQUENCE:\n");


	while ( *d )
	  {DSet dd = d;
	   while ( *dd )
	     {t = find_T_num (net, *dd - 1);
	      pre = t->preset;
	      while ( pre )
	        if ( pre->place->marks == 0 )
	          break;
	        else pre=pre->nextplace;
	      if ( pre )
	        dd++;
	      else
	        {pre = t->preset;
	         while ( pre )
	           {pre->place->marks = 0;
	            pre = pre->nextplace;
	           };
	         post = t->postset;
		 while ( post )
	           {post->place->marks = 1;
	            post = post->nextplace;
	           };
		 *single = *dd;
	         *(single + 1) = 0;
		 d = SETdifference (d, single);
		 fprintf (out_f, (*d?"%s,":"%s\n"), t->inscript);
                 break;
	        };
	      };
	    if ( ! *dd && *d)
	     ERROR ("Fatal error during sequence construction.");
	    }
  return 0;
 }

#if defined(__STDC__) || defined (__cplusplus)
void print_results (lprec *lp)
#else
void print_results (lp)
	lprec *lp;
#endif
{
  int i;

  fprintf(stdout, "Value of objective function: %e\n",
	  (double)lp->best_solution[0]);

  /* print normal variables */
  for (i = 1; i <= lp->columns; i++)
    if (lp->best_solution[lp->rows+i] != 0.0)
      {if (lp->names_used)
      fprintf(stdout, "%-10s%e\n", lp->col_name[i],
	    (double)lp->best_solution[lp->rows+i]);
    else
      fprintf(stdout, 
              "Var [%4d]  %e\n",i,(double)lp->best_solution[lp->rows+i]);
      };

  /* print achieved constraint values 
  if(lp->verbose)
    {
      fprintf(stdout, "\nActual values of the constraints:\n");
      for (i = 1; i <= lp->rows; i++)
        if (lp->names_used)
	  fprintf(stdout, "%-10s%e\n", lp->row_name[i],
		(double)lp->best_solution[i]);
        else
          fprintf(stdout, 
                  "%Row [%4d]  %e\n",i,(double)lp->best_solution[i]);  
    }*/

 /* if((lp->verbose || lp->print_duals)) */
    {
      if (lp->max_level != 1)
        fprintf(stdout, 
        "These are the duals from the node that gave the optimal solution.\n");
      else
        fprintf(stdout, "\nDual values:\n");
      for (i = 1; i <= lp->rows; i++)
	if (lp->duals[i] != 0.0) {
        if (lp->names_used)
	  fprintf(stdout, 
                  "%-10s%e\n", lp->row_name[i], (double)lp->duals[i] );
        else
          fprintf(stdout, "Row [%4d]  %e\n",i,(double)lp->duals[i]); 
	};
    }
} /* print_results */

#if defined(__STDC__) || defined(__cplusplus)
void print_marking (lprec *lp)
#else
void print_marking (lp)
	lprec *lp;
#endif
{
  int i;

  /*printf ("\nThe fullspecified marking is:\n");*/

   for (i = 1; i <= lp->columns; i++)
    if (lp->names_used && (lp->col_name[i][0] == 'm') && 
	(my_abs (lp->best_solution[lp->rows+i]) > DEF_EPSILON))
    {
      if (max_tokens != 1)
	fprintf(out_f, "%s(%d) ", &(lp->col_name[i][1]), (int) 
		(my_abs (lp->best_solution[lp->rows+i])));
      else
        fprintf(out_f, "%s ", &(lp->col_name[i][1]));
    }
}

#if defined(__STDC__) || defined(__cplusplus)
void print_sub_marking (LMPNet net)
#else
void print_sub_marking (net)
	LMPNet net;
#endif
{
	Place pl = net->Places;

	while (pl != (Place) NULL)
	  {if (pl->marks == 1)
	     fprintf (out_f, "%s ", pl->name);
	   pl = pl->nextplace;
	  }
	fprintf (out_f, "\n");
}


