/*****************************************************************************/
/* mat_cplex.c                                                               */
/* Bereitstellung eines LGS zur Deadlockueberpruefung auf Entfaltungen       */
/*                                                                           */
/*****************************************************************************/
/*                                           Do., 03.03.1994, Stefan Roemer  */
/*                                           Dezember   1996, Stephan Melzer */
/*****************************************************************************/

 
#include <stdlib.h>
#include "mat_cplex.h"
#include "rw_prefix.h"
#include "cpxdefs.inc"
#include "setop.h"
 
#ifndef CPXMIP_FAIL_FEAS_NO_TREE
#define CPXMIP_FAIL_FEAS_NO_TREE 116
#endif

#define DEF_EPSILON 1.0e-5
#ifndef CPX_ABORT_CROSSOVER
#define CPX_ABORT_CROSSOVER 18
#endif

#define pos(x) ( (x) < 0 ? -(x) : (x) )

/*****************************************************************************/

char     probname[NAMELENGTH];
int      mac;                             /* Anzahl Zeilen/Gleichungen */
int      mar;                             /* Anzahl Spalten */
int      objsen;                          /* -1 => Maximierungsproblem */
double   * objx;                          /* Koeffizienten der Zielfunktion */
double   * rhsx;                          /* Koeffizienten der rechten Seite */
char     * senx;                          /* Typ Un-/Gleichung: L/E */
char     * ctype;
                                   /* Beschreibung der Matrix: */
                                   /* Nur Eintraege != 0 werden gespeichert! */
int      * matbeg;                        /* "Startindizes" */
int      * matcnt;                        /* "Anzahlen" pro Spalte */
int      * matind;                        /* Zeilennummern */
double   * matval;                        /* entspr. Eintraege */

double   * bdl;                               /* untere Schranken */
double   * bdu;                               /* obere Schranken */
char     * dataname = NULL;
char     * objname = NULL;
char     * rhsname = NULL;
char     * rngname = NULL;
char     * bndname = NULL;
char     * cstore = NULL;                     /* Namen der Variablen */
char     ** cname = NULL;
char     * rstore = NULL;                     /* Namen der "Zeilen"*/
char     ** rname = NULL;
int      macsz;                            /* Wert >= mac: Groesse von objx,
                                             matbeg, matcnt, bdl, bdu, cname */
int      marsz;                          /* Wert >= mar: Groesse von rhsx,senx,
                                                               rname, rngval */
int      matsz;                            /* Groesse von matind, matval */
unsigned cstorsz;                          /* Laenge von cstore */
unsigned rstorsz;                          /* Laenge von rstore */
int      lpstat;
double   obj;
double   * x;
double   * pi;
double   * slack;
double   * dj;

unsigned int 	 * conv;
int 	*ts_size;

FILE *out_file;

/*****************************************************************************/


#if defined(__STDC__) || defined(__cplusplus)
int calc_simplex(LMPNet net )
#else
int calc_simplex( net )
	LMPNet net;
#endif

  {

    struct cpxlp  * lp = NULL;
    FILE          * logfile = NULL,
                  * changefile = NULL;
    int           status = 1,
                  ntoosmall,
                  ntoobig,
		  mipstat = 1,
 	          nodecnt,
     		  itercnt,
	          curmac, 
		  curmar;
    double   	  obj,
		  toleranz,
	          dtoosmall,
		  dtoobig;


 
    /* Set up CPLEX message output.  setlogfile sends messages
       on all four CPLEX-defined channels to the file specified.
       setscr_ind sends messages on all four channels to stdout.
       The CPLEX Message Handler must be initialized for CPLEX to
       generate any messages, even error messages. */

    if (    ! ( logfile = fopen( "cplex_logfile.log", "a" ) )
         || setlogfile( logfile )
         || setscr_ind( 1 ) )
      {
        printf ("Failure to connect logging channel.\n");
        goto TERMINATE;
      }

 
    /* Loading, optimizing and obtaining a solution to the problem. */
    lp = loadmprob( probname, mac, mar, 0, objsen, objx, rhsx,
                   senx, matbeg, matcnt, matind, matval,
                   bdl, bdu, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                   dataname, objname, rhsname, rngname, bndname,
                   cname, cstore, rname, rstore, NULL, NULL,
                   macsz, marsz, matsz, 0, 0, cstorsz, rstorsz, 0, ctype );

    /* A returned pointer of NULL may mean that not enough memory
       was available or there was some other problem.  For most 
       other CPLEX functions, a return status of 0 means success,
       and 1 means failure.  In the case of failure, an error message
       will have been written to the error channel, which for this
       example is both stdout and the logfile, from inside CPLEX. */

 
    if ( ! lp )
      {
        goto TERMINATE;
      } /* if */

/*    if ( status = loadctype (lp, ctype) )
      {
	goto TERMINATE;
      }
*/
 
     settilim (-10.0, &dtoosmall, &dtoobig);
     settilim (dtoobig, &dtoosmall, &dtoobig);

     setndlim (-10, &ntoosmall, &ntoobig);
     setndlim (ntoobig, &ntoosmall, &ntoobig);

     getepint (&toleranz);

     /* Optimize the problem and obtain solution. */
    if ( status = mipoptimize( lp ) )
      {
        goto TERMINATE;
      } /* if */

     curmac = getmac (lp);
     curmar = getmar (lp);

     x     = (double *) malloc ((unsigned) curmac * sizeof (double));
     pi    = (double *) malloc ((unsigned) curmar * sizeof (double));
     slack = (double *) malloc ((unsigned) curmar * sizeof (double));
     dj    = (double *) malloc ((unsigned) curmac * sizeof (double));

     if ( !x || !pi || !slack || !dj )  
       {
	 goto TERMINATE;
       }
 
     mipstat = getstat (lp);
     nodecnt = getndc (lp);
     itercnt = getitcm (lp);

     if ( mipstat == CPXMIP_INFEASIBLE      ||
          mipstat == CPXMIP_NODE_LIM_INFEAS ||
          mipstat == CPXMIP_TIME_LIM_INFEAS ||
          mipstat == CPXMIP_FAIL_INFEAS       ) 
        getbobjval (lp, &obj); 
     else
    	getmobjval (lp, &obj);

     getmx (lp, x, 0, curmac-1);
 
     if ( mipstat == CPXMIP_INFEASIBLE ) 
       fprintf (out_file, "Net is deadlock-free\n"); 
     else if ( mipstat == CPXMIP_OPTIMAL ) 
		{DSet fire,
		      single;
		int j,
		    size = 0;

		SETinitSets ();

		for (j = 0; j < curmac; j++)
	         if ((pos (x[j] - 1.0) < toleranz) && (ctype[j] == 'B'))
                   size++;

	        fire = SETalloc (size);
	        *fire = 0;
	        single = SETalloc (1);
	        *(single + 1) = 0;

	        for (j = 0; j < curmac; j++)
	         if ((pos (x[j] - 1.0) < toleranz) && (ctype[j] == 'B'))
	            {*single = (Cardinal) conv[j] + 1;
		     *(single + 1) = 0;
	             fire = SETunion (fire, single);
	                  
	            }

		fprintf (out_file, "Net has a deadlock\n"); 
	        print_sequence (net, fire, out_file);

	        SETremove (fire);
	        SETremove (single);
 	       }
           else {if ((mipstat > CPXMIP_OPTIMAL) && 
	       (mipstat <= CPXMIP_FAIL_FEAS_NO_TREE))
	     fprintf (stderr, 
		"CPLEX exists with status: %s\n", 
		cplex_mip_msg[mipstat - CPXMIP_OPTIMAL]);
	   else if ((mipstat > CPX_OPTIMAL) && 
		  (mipstat <= CPX_ABORT_CROSSOVER))
	     fprintf (stderr, 
		"CPLEX exists with status: %s\n", cplex_msg[0]);
	   else
		fprintf (stderr, "%s\n", cplex_msg[1]);

	      goto TERMINATE;
	     };

     printf ("Result: %s Objective %s %g.\n Node: %d Iteration: %d\n", 
		(mipstat >= CPXMIP_OPTIMAL) && 
		(mipstat <= CPXMIP_FAIL_FEAS_NO_TREE)   ? 
		cplex_mip_msg[mipstat - CPXMIP_OPTIMAL] :
		((mipstat >= CPX_OPTIMAL) && 
		  (mipstat <= CPX_ABORT_CROSSOVER) ? cplex_msg[0] : "??"),

	     	mipstat == CPXMIP_INFEASIBLE      ||
             	mipstat == CPXMIP_NODE_LIM_INFEAS ||
             	mipstat == CPXMIP_TIME_LIM_INFEAS ||
             	mipstat == CPXMIP_FAIL_INFEAS ? "<=" : "=", 
	        obj, nodecnt, itercnt);

    
TERMINATE:
    /* Flush all channels before deleting files as destinations. */
    flushchannel( cpxresults );
    flushchannel( cpxlog );
    flushchannel( cpxerror );
    flushchannel( cpxwarning );

    freeprob (&lp);

    /* Delete log file from all four channels and close. */
    setlogfile( NULL );     
    if ( logfile )
      {
        fclose( logfile );	   
      } /* if */
 
    /* Delete changefile from results channel, then close. */
    delfpdest( cpxresults, changefile );
    if ( changefile )
      {
        fclose( changefile );
      } /* if */
    
    return ( status );

  } /* calc_simplex() */


/*****************************************************************************/
/* Konversion einer Netzmatrix in das CPLEX-Matrixformat.                    */
/*****************************************************************************/
  
#if defined(__STDC__) || defined(__cplusplus)

void convert_matrix( LMPNet net )

#else

void convert_matrix( net )
  LMPNet net;

#endif

  {

    Place      pl;
    Transition tr;
    STEdge     st;
    TSEdge     ts;
    int        i, j,
               count = 0,
	       s_size,
	       c_count,
               * ptr_matbeg,
               * ptr_matcnt,
               * ptr_matind;
    double     * ptr_matval;

    /* show_matrix( net ); */
    strcpy( probname, "net_matrix" );

    mac = (int) net->NumPlaces + (net->NumTransitions - GL_numCutoffs);        
    mar = (int) net->NumTransitions + net->NumPlaces;    
    macsz = mac + 1;                   /* Wert >= mac: Groesse von objx,
                                            matbeg, matcnt, bdl, bdu, cname */
    marsz = mar + 2;                   /* Wert >= mar: Groesse von rhsx,senx,
                                                              rname, rngval */
    objsen = -1;                       /* -1 => Maximierungsproblem */
    dataname = objname = rhsname = rngname = bndname = NULL;
    cstore = rstore = NULL;
    cname = rname = NULL;
    cstorsz = rstorsz = 0;

    /* Zielfunktion und Schranken festlegen */
    objx = (double *) MYmalloc( macsz * sizeof( double ) );
    bdl = (double *) MYmalloc( macsz * sizeof( double ) );
    bdu = (double *) MYmalloc( macsz * sizeof( double ) );
    ctype = (char *) MYmalloc ( macsz * sizeof( double ) );
 
    conv = (unsigned int *) MYmalloc ((net->NumTransitions - GL_numCutoffs) * 
				    sizeof (unsigned int));
    ts_size = (int *) MYmalloc (net->NumTransitions  * sizeof (int));

    i = 0;
    while ( i < mac )
      {
	if ( i >= (net->NumTransitions - GL_numCutoffs) )
          {
	    objx[i]  = (double) 1.0;
	    bdl[i]   = (double) 0.0;
	    bdu[i]   = (double) 1.0;
	    ctype[i] = 'C';
	  }
	else
	  {
	    objx[i]  = (double) 0.0;
	    bdl[i]   = (double) 0.0;
	    bdu[i]   = (double) 1.0;
	    ctype[i] = 'B';
	  }
	i++;
      } /* while */
 
    /* Anzahl der Knoten mit Kantengewicht > 0 ermitteln */

    num_net_nodes(net);
    i = 0;
    j = 0;
    tr = net->Transitions;
    while ( tr )
      {
	if ( !tr->label )
	  conv[j++] = tr->numname;
        tr = tr->nexttrans;
      } /* while */

    count = 0;
    c_count = 0;
    tr = net->Transitions;
    while ( tr )
      {
        st = tr->preset;
	s_size = 0;
        while ( st )
          {
            s_size++;
            st = st->nextplace;
          } /* while */
	ts_size[tr->numname] = s_size;
/*tr->inscript = (char*) s_size; */
	c_count += s_size;
        ts = tr->postset;
        while ( ts )
          {
            s_size++;
            ts = ts->nextplace;
          } /* while */
	if (!tr->label)
	  count += s_size;
        tr = tr->nexttrans;
      } /* while */
     matsz = count + net->NumPlaces + c_count;      
     /*matsz = count;    */     
					/* Groesse von matind, matval */

    /* Rechte Seiten und Art der (Un-)Gleichungen bestimmen */
    rhsx = (double *) MYmalloc( marsz * sizeof( double ) );
    senx = (char *) MYmalloc( marsz * sizeof( char ) );
    i = 0;
    pl = net->Places;
    while ( pl )
      {
        rhsx[i] = (pl->preset ? -0.0 : -1.0);
 				      /* Koeffizienten der rechten Seite */
        senx[i] = 'E';                /* Typ: Gleichung (E) */
	i++;
	pl = pl->nextplace;
      } /* while */
    tr = net->Transitions;
    while ( tr ) 
      {
	rhsx[i] = (double) (((int) ts_size[tr->numname]) - 1) ;       
				      /* Koeffizienten der rechten Seite */
        senx[i] = 'L';                /* Typ: Gleichung (E) */
	i++;
	tr = tr->nexttrans;
      } /* while */

    
    /* Konstruktion der Matrix und der notwendigen Hilfsfelder */
    *( ptr_matbeg = matbeg
                  = (int *) MYmalloc( ( macsz + 1 ) * sizeof( int ) ) ) = 0;
    ptr_matcnt = matcnt = (int *) MYmalloc( ( macsz + 1 ) * sizeof( int ) );
    ptr_matind = matind = (int *) MYmalloc( ( matsz ) * sizeof( int ) );
    ptr_matval = matval
               = (double *) MYmalloc( ( matsz ) * sizeof( double ) );
    tr = net->Transitions;
    while ( tr )
      {
	if ( !tr->label )
	{  
        st = tr->preset;
        ts = tr->postset;
        i = 0;
        while ( st || ts )
          {
            i++;
            if ( ts && st && ( st->place->numname == ts->place->numname ) )
              {
                fprintf( stderr, "ERROR: (Matrixkonversion) " );
                fprintf( stderr, "Kein schlingenfreies Netz!\n");
                exit( 1 );
              } /* if */
            if ( !ts || ( st && ( st->place->numname < ts->place->numname ) ) )
              {
                *( ptr_matval++ ) = - 1.0;
                *( ptr_matind++ ) = (int) st->place->numname;
                st = st->nextplace;
              } /* if */
            else
              {
                *( ptr_matval++ ) = (double) 1.0 ;
                *( ptr_matind++ ) = (int) ts->place->numname;
                ts = ts->nextplace;
              } /* else */
          } /* while ( st || ts ) */
        *( ptr_matcnt++ ) = i;
        i += *ptr_matbeg;
        *++ptr_matbeg = i;
	}
        tr = tr->nexttrans;
      } /* while ( tr ) */
    pl = net->Places;
    while ( pl )
	{ st = pl->postset;
	  i = 0;
	  i++;
	  *( ptr_matval++ ) = (double) -1.0;
          *( ptr_matind++ ) = (int) pl->numname;
	  while ( st )
	    { i++;
	      *( ptr_matval++ ) = (double) 1.0;
              *( ptr_matind++ ) = net->NumPlaces + (int) st->trans->numname;
              st = st->nexttrans;
            } 
	  *( ptr_matcnt++ ) = i;
          i += *ptr_matbeg;
          *++ptr_matbeg = i;
        pl = pl->nextplace;
      }  

#ifdef _O_U_T_P_U_T_
    ptr_matbeg = matbeg;
    ptr_matcnt = matcnt;
    ptr_matind = matind;
    ptr_matval = matval;
    i = mac;
    printf( "matbeg = (%d", *ptr_matbeg );
    while ( --i )
      {
        printf( ", %d", *++ptr_matbeg );
      } /* while */
    printf( ")\n" );
    i = mac;
    printf( "matcnt = (%d", *ptr_matcnt );
    while ( --i )
      {
        printf( ", %d", *++ptr_matcnt );
      } /* while */
    printf( ")\n" );
    i = matsz;
    printf( "matind = (%d", *ptr_matind );
    while ( --i )
      {
        printf( ", %d", *++ptr_matind );
      } /* while */
    printf( ")\n" );
    i = matsz;
    printf( "matval = (%f", *ptr_matval );
    while ( --i )
      {
        printf( ", %f", *++ptr_matval );
      } /* while */
    printf( ")\n" );
#endif

  } /* convert_matrix() */


/*****************************************************************************/
/* Freigabe des fuer die Felder angeforderten Speicherplatzes.               */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void free_matrix( void )

#else

void free_matrix()

#endif

  {

    free( (char *) objx );
    free( (char *) bdl );
    free( (char *) bdu );
    free( (char *) x );
    free( (char *) dj );
    free( (char *) rhsx );
    free( (char *) senx );
    free( (char *) pi );
    free( (char *) slack );
    free( (char *) matbeg );
    free( (char *) matcnt );
    free( (char *) matind );
    free( (char *) matval );
    free( (char *) conv);
    free( (char *) ts_size);

  } /* free_matrix() */


/*****************************************************************************/
/* Testroutine.                                                              */
/*****************************************************************************/

 
#if defined(__STDC__) || defined(__cplusplus)

void main( int argc, char ** argv )

#else

void main( argc, argv )

  int argc;
  char ** argv;

#endif

  {

    /* Deklarationen */ 

    LMPNet net; 
    char   in_filename[NAMELENGTH];
    char   out_filename[NAMELENGTH];

    if ( argc == 2 && !strncmp (argv[1], "-n=", 3) && (strlen(argv[1]) > 3) )
      {
        strcpy( in_filename, &(argv[1][3]) );
	out_file = stdout;
 	net = readPrefix( in_filename, NULL, NULL );
   	convert_matrix( net );
    	calc_simplex( net );
    	free_matrix();
     
    	remove_net(&net);
    	exit (0);
      } 
    else if (argc == 3 && 
           !strncmp (argv[1], "-n=", 3) && (strlen(argv[1]) > 3) &&
	   !strncmp (argv[2], "-o=", 3) && (strlen(argv[2]) > 3) )
      {
        strcpy( in_filename, &(argv[1][3]) );
	out_file = fopen (&(argv[2][3]), "w");
 	net = readPrefix( in_filename, NULL, NULL );
    	convert_matrix( net );
    	calc_simplex( net );
    	free_matrix();
        fclose (out_file);
    	remove_net(&net);
    	exit (0);
      }
   else if (argc == 3 && 
           !strncmp (argv[2], "-n=", 3) && (strlen(argv[2]) > 3) &&
	   !strncmp (argv[1], "-o=", 3) && (strlen(argv[1]) > 3) )
      {
        strcpy( in_filename, &(argv[2][3]) );
	out_file = fopen (&(argv[1][3]), "w");
 	net = readPrefix( in_filename, NULL, NULL );
    	convert_matrix( net );
    	calc_simplex( net );
    	free_matrix();
       	fclose (out_file);
    	remove_net(&net);
    	exit (0);
      }
    else
      {
   
	printf ("dl_check_lin    Version 1.0\n");
	printf ("Aufruf  dl_check_lin -n=<net> [-o=<file>]    \n\n");
	printf ("<net> = pep net file            \n");
	printf ("<file> = output result file     \n");
	printf ("(c) SFB 342 Tp A3               \n");
	exit (0);
      } /* else */
} /* main() */

 
/*****************************************************************************/
/* END OF FILE mat.c                                                         */
/*****************************************************************************/
