/*****************************************************************************/
/* mat_cplex.h                                                               */
/* Konversion von Netzmatrizen in das CPLEX-Format.                          */
/*****************************************************************************/
/*                                            Do., 03.03.1994, Stefan Roemer */
/*****************************************************************************/

#ifndef __MAT_CPLEX_H__
#define __MAT_CPLEX_H__

#include "neunetze.h"

/*****************************************************************************/

extern char     probname[];
extern int      mac;                             /* Anzahl Zeilen/Gleichungen */
extern int      mar;                             /* Anzahl Spalten */
extern int      objsen;                          /* -1 => Maximierungsproblem */
extern double   * objx;                          /* Koeffizienten der Zielfunktion */
extern double   * rhsx;                          /* Koeffizienten der rechten Seite */
extern char     * senx;                          /* Typ Un-/Gleichung: L/E */
                                   /* Beschreibung der Matrix: */
                                   /* Nur Eintraege != 0 werden gespeichert! */
extern int      * matbeg;                        /* "Startindizes" */
extern int      * matcnt;                        /* "Anzahlen" pro Spalte */
extern int      * matind;                        /* Zeilennummern */
extern double   * matval;                        /* entspr. Eintraege */

extern double   * bdl;                               /* untere Schranken */
extern double   * bdu;                               /* obere Schranken */
extern char     * dataname;
extern char     * objname;
extern char     * rhsname;
extern char     * rngname;
extern char     * bndname;
extern char     * cstore;                     /* Namen der Variablen */
extern char     ** cname;
extern char     * rstore;                     /* Namen der "Zeilen"*/
extern char     ** rname;
extern int      macsz;                            /* Wert >= mac: Groesse von objx,
                                             matbeg, matcnt, bdl, bdu, cname */
extern int      marsz;                          /* Wert >= mar: Groesse von rhsx,senx,
                                                               rname, rngval */
extern int      matsz;                            /* Groesse von matind, matval */
extern unsigned cstorsz;                          /* Laenge von cstore */
extern unsigned rstorsz;                          /* Laenge von rstore */
extern int      lpstat;
extern double   obj;
extern double   * x;
extern double   * pi;
extern double   * slack;
extern double   * dj;

unsigned int GL_numCutoffs;

/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern int calc_simplex();

#else

extern int calc_simplex();

#endif 

/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern void convert_matrix();

#else

extern void convert_matrix();

#endif 

/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern void free_matrix();

#else

extern void free_matrix();

#endif 

/*****************************************************************************/

#endif /* __MAT_CPLEX_H__ */

/*****************************************************************************/
/* END OF FILE mat_cplex.h                                                   */
/*****************************************************************************/
