/*******************************************************************************

      	pep2lp                                                         14/8/97

	1.) Implementation of the trap-equation:

	    theoretical background:

	    Stephan Melzer and Javier Esparza, 
	    Checking system properties via Integer Programming, 
            in LNCS 1058, pp. 250 -- 264.
            also available via anonymous-ftp:
	    flop.informatik.tu-muenchen.de
            /pub/theory/melzer/sfb-342-13-95.ps.gz

 	2.) Implementation of the improved deadlock-algorithm using net
            unfoldings

	    theoretical background:

	    Stephan Melzer and Stefan Roemer, 
	    Deadlock Checking Using Net Unfoldings,
	    in LNCS 1254, pp. 352 -- 363.
	    also available via anonymous-ftp:
	    flop.informatik.tu-muenchen.de
	    /pub/theory/melzer/cav_97.ps.gz

	Stephan Melzer (supported by SFB 342 A3)
	Technical University of Munich, Germany
	E-mail: melzers@informatik.tu-muenchen.de

        The implementation makes use of 'lp_solve' that is written by:

	M.R.C.M. Berkelaar
      	Eindhoven University of Technology
      	Design Automation Section
      	P.O. Box 513
      	NL-5600 MB Eindhoven, The Netherlands
      	phone ...-31-40-473345
      	E-mail: michel@es.ele.tue.nl



*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <ctype.h>

#include "lp_misc.h"
#include "neunetze.h"
#include "convertPEP.h"
#include "mat_cplex.h"
#include "rw_prefix.h"
#include "setop.h"
#include "conv_pep.h"
#ifndef _NO_CPLEX
#include "cpxdefs.inc"
#endif
 
#define MAX 1000.0
#define MAX_NAME 200
#define MAX_MARK 200
#define MAX_LEN (MAXSTRL - 5)
#define MAX_ARGS 100
#define MAX_ADD 100

#define REACHABLE 1
#define DEADLOCK  2
#define DEADLOCK_UNF 3

#define TRAP 1
#define DEADLOCK 2

#define write_mps  1
#define write_lp   2
#define write_both 3

#define EXT ".mps"    

#define pos(x) ( (x) < 0 ? -(x) : (x) )

#define LINE_LEN 80
 
#ifndef CPXERR_BOUNDS_INFEAS
#define CPXERR_BOUNDS_INFEAS 1100
#endif

#ifndef CPXERR_PRESLV_INForUNBD
#define CPXERR_PRESLV_INForUNBD 1101
#endif

#ifndef CPXMIP_FAIL_FEAS_NO_TREE
#define CPXMIP_FAIL_FEAS_NO_TREE 116
#endif

#ifndef CPX_ABORT_CROSSOVER
#define CPX_ABORT_CROSSOVER 18
#endif

float time__[4] = { 0.0, 0.0, 0.0, 0.0 };


#define error(X) error1(X,"")
#if defined(__STDC__) || defined (__cplusplus)
void error1 (char * n1, char *n2)
#else
void error1 (n1, n2)
	char *n1, *n2;
#endif
{
	fprintf (stderr, "\n%s%s\n", n1, n2);
	exit(1);
}

#if defined(__STDC__) || defined (__cplusplus)
int dual_is_trap (lprec *lp, LMPNet net, unsigned int num_st)
#else
int dual_is_trap (lp, net, num_st)
	lprec *lp;
	LMPNet net;
	unsigned int num_st;
#endif
{
	Place pl = net->Places;

	while ( pl != (Place) NULL )
	  {if ( (pl->label == ENTRY) && (lp->duals[pl->numname + 1] != 0.0) )
	     break;
	   pl = pl->nextplace;
	  }

	if ( pl == (Place) NULL )
	  return FALSE;

	pl = net->Places;

	while ( pl != (Place) NULL )
	  {if ( (pl->marks == undef) && (lp->duals[pl->numname + 1] > 0.0) && 
		(lp->best_solution[lp->rows + num_st + pl->component + 2] > 0.0))
	     return FALSE;
	   else if ( (pl->marks == 1) && (lp->duals[pl->numname + 1] > 0.0) )
	     return FALSE;
	   pl = pl->nextplace;
	  }

	pl = net->Places;

	while ( pl != (Place) NULL )
	  {if ( lp->duals[pl->numname + 1] > 0.0 )
	    {STEdge post = pl->postset;
	     while ( post != (STEdge) NULL )
	       {TSEdge post2 = post->trans->postset;
	        unsigned short found = FALSE;
	        while ( post2 != (TSEdge) NULL )
	          {if ( lp->duals[post2->place->numname + 1] > 0.0 )
	             {found = TRUE; 
		      post2 = (TSEdge) NULL;
                     }
		   else
	             post2 = post2->nextplace;
	          }                          /* while */
 	        if (!found)
	           return FALSE;
	        post = post->nexttrans;
               }                             /* while */
	    } 				     /* if */
	  pl = pl->nextplace;
	  } 				     /* while */

	return TRUE;
}

#if defined(__STDC__) || defined (__cplusplus)
int dual_is_trap_cplex (double *dual, double *sol,
                        LMPNet net, unsigned int num_st)
#else
int dual_is_trap_cplex (dual, sol, net, num_st)
	double *lp;
	double *sol;
	LMPNet net;
	unsigned int num_st;
#endif
{
	Place pl = net->Places;

	while ( pl != (Place) NULL )
	  {if ( (pl->label == ENTRY) && (dual[pl->numname] != 0.0) )
	     break;
	   pl = pl->nextplace;
	  }
 
	if ( pl == (Place) NULL )
	  return FALSE;

	pl = net->Places;

	while ( pl != (Place) NULL )
	  {if ( (pl->marks == undef) && (dual[pl->numname] < 0.0) && 
		(sol[num_st + pl->component + 1] > 0.0))
	     return FALSE;
	   else if ( (pl->marks == 1) && (dual[pl->numname] < 0.0) )
	     return FALSE;
	   pl = pl->nextplace;
	  }
 
	pl = net->Places;

	while ( pl != (Place) NULL )
	  {if ( dual[pl->numname] < 0.0 )
	    {STEdge post = pl->postset;
 	     while ( post != (STEdge) NULL )
	       {TSEdge post2 = post->trans->postset;
	        unsigned short found = FALSE;
	        while ( post2 != (TSEdge) NULL )
	          {if ( dual[post2->place->numname] < 0.0 )
	             {found = TRUE; 
		      post2 = (TSEdge) NULL;
                     }
		   else
	             post2 = post2->nextplace;
	          }                          /* while */
	        if (!found)
	           return FALSE;
	        post = post->nexttrans;
               }                             /* while */
	    } 				     /* if */
	  pl = pl->nextplace;
	  } 				     /* while */

	return TRUE;
}

typedef struct {REAL *val;
		int  *pos;
		REAL  rhs;
		int   space;
		short sense;
	       } *ROW, ROW_elm;

#if defined(__STDC__) || defined(__cplusplus)
int str2cut (char *str, ROW_elm *row, LMPNet net)
#else
int str2cut ( str, row, net )
	char     *str;
	ROW_elm  *row;
	LMPNet    net;
#endif
{
	char *dum;
	char dum_str[1000];
	long int numb;
	Place pl;
	int i;
	double sign = 1.0;
	int *ptr_matind = row->pos;
	REAL *ptr_matval = row->val;

	row->space = 0;
 
	while ( *str )
	  {while ( (*str == ' ') && str ) str++;
	   if (! str)
	     ERROR ("Identifier or digit is missing.");

	   if ( *str == '-' )
	     {str++;
	      sign = -1.0;
	     }
	   else if ( *str == '+' )
	     {str++; 
	      sign = 1.0;
	     };

	   while ( (*str == ' ') && str ) str++;
	   if (! str)
	     ERROR ("Identifier is missing.");

	   if ( isdigit (*str) )
	     {dum = str;
 	      numb = strtol (dum, &str, 0);
	     }
	   else 
	     numb = 1;
	   while ( (*str == ' ') && str ) str++;
	   if (! str)
	     ERROR ("Identifier is missing.");

	   dum = dum_str;
	   while ( (*str != ' ') && str )
	     *dum++ = *str++;
	   *dum = '\0';

           if ( (pl = find_P (net, dum_str)) == (Place) NULL)
	     ERROR ("String no place identifier.")
	   else
	     {*( ptr_matval++) = ((REAL) numb) * sign;
	      sign = 1.0;
	      *( ptr_matind++) = pl->numname;
	      row->space += 1;
 	     };
	   while ( *str == ' ') str++;
	
	   if (! str)
	     ERROR ("Operator is missing.");
	
	   if ( (*str == '=') || (*str == '<') || (*str == '>') )
	     break;
	  };
	if (! str)
	  ERROR ("Operator mis missing.");
	
	if (*str == '<')
	  row->sense = LE;
	else if (*str == '>')
	  row->sense = GE;
	else if (*str == '=')
	  row->sense = EQ;
	else
	  ERROR ("Wrong operator.");

	str++;

	while ((*str == ' ') && str) str++;
	if (! str) 
	  ERROR ("Digit is missing.");

	if ( *str == '-' )
	  {str++;
	   sign = -1.0;
	  }
	else if ( *str == '+' )
	  {str++; 
	   sign = 1.0;
	  };

	while ( (*str == ' ') && str ) str++;
	if (! str)
	  ERROR ("Identifier is missing.");

	if ( isdigit (*str) )
	  {dum = str;
	   numb = strtol (dum, &str, 0);
	  }
	else  
	  ERROR ("Digit is missing.");

	row->rhs = ((REAL) numb) * sign;

/*
	printf ("space=%i\n", row->space);
	printf ("rsh = %f\n", row->rhs);
	printf ("sense = %d\n", row->sense);
	for (i=0; i < row->space; i++)
	  printf ("x[%d] = %f\n", row->pos[i], row->val[i]);
*/

	return (0);
}

#if defined(__STDC__) || defined(__cplusplus)
int read_extra_constraints ( char *file_name, ROW_elm *e_rows, 
			     unsigned int *e_num, LMPNet net)
#else 
int read_extra_constraints ( file_name, e_row, e_num, net )
	char         *file_name;
	ROW_elm      *e_rows;
	unsigned int *e_num;
	LMPNet       net;
#endif
{
	FILE *f;
	char dummy[1000];
	 
	(*e_num) = 0;
 
	if ( (f = fopen (file_name, "r")) == (FILE*) NULL )
	  {fprintf (stderr, "Cannot open file.");
	   return (1);
	  };
	
	while ( ! feof (f) )
	  {do {
	       fgets (dummy, 1000, f);
	      }
	   while ( (! feof (f)) && (! *dummy) );
	   if ( feof (f) )
	     break;
	    e_rows[*e_num].val = (REAL*) malloc (300 * sizeof (REAL));
	    e_rows[*e_num].pos = (int*)  malloc (300 * sizeof (int));

 	   if ( str2cut (dummy, &(e_rows[*e_num]), net) )
	     {fprintf (stderr, "String is no constraint.");
	      return (1);
	     };

 	   (*e_num)++;

           if ( *e_num == MAX_ADD )
             {fprintf (stderr, 
                       "No space left for extra constraints.");
	      fclose (f);
	      return (1);
	     }
	  };
	
 	fclose (f);
 	return (0);
}


#ifndef _NO_CPLEX

#if defined(__STDC__) || defined (__cplusplus)
void cplex_solve (lprec *lp_, int without_traps, 
		  int debugging_on, char *mps_file, float *copy_time_,
		  int copy_in_file, char *sent1, char *sent2, LMPNet net, 
                  unsigned int num_st)
#else
void cplex_solve (lp_, without_traps, debugging_on, mps_file, 
		  copy_time_, copy_in_file, sent1, sent2, net, num_st)
	lprec* lp_;
	int    without_traps;
	int    debugging_on;
	char*  mps_file;
	float* copy_time_;
	int    copy_in_file;
	char*  sent1;
	char*  sent2;
	LMPNet net;
	unsigned int num_st;
#endif
{
	char     *name 		= "/tmp/xxx.mps";
	int      mac;
	int      mar;
	int      mae;
	int      nsos;
	int      nsosnz;
	int      objsen;
	double   *objx 		= NULL;
	double   *rhsx 		= NULL;
	char     *senx 		= NULL;
	double   *rngval 	= NULL;
	int      *nrowind 	= NULL;
	int      *matbeg 	= NULL;
	int      *matcnt 	= NULL;
	int      *matind 	= NULL;
	double   *matval 	= NULL;
	double   *bdl 		= NULL;
	double   *bdu 		= NULL;
	int      *etype 	= NULL;
	int      *enzbeg 	= NULL;
	int      *enzcnt 	= NULL;
	int      *enzind 	= NULL;
	double   *enzval 	= NULL;
	char     *dataname 	= NULL;
	char     *objname 	= NULL;
	char     *rhsname 	= NULL;
	char     *rngname 	= NULL;
	char     *bndname 	= NULL;
	char     **cname 	= NULL;
	char     *cstore 	= NULL;
	char     **rname 	= NULL;
	char     *rstore 	= NULL;
	char     *estore 	= NULL;
	char     **ename 	= NULL;
	char     *ctype  	= NULL;
	char     *sostype 	= NULL;
	int      *sosbeg 	= NULL;
	int      *sosind	= NULL;
	double   *sosref 	= NULL;
	int      macsz 		= 0;
	int      marsz 		= 0;
	int      matsz 		= 0;
	int      maesz 		= 0;
	int      enzsz		 = 0;
	unsigned cstorsz 	= 0;
	unsigned rstorsz 	= 0;
	unsigned estorsz 	= 0;
	int      lpstat;
	int      mipstat;
	int      nodecnt;
	int 	 itercnt;
	double   obj;
	double   *pi 		= NULL;
	double   *x 		= NULL;
	double   *slack 	= NULL;
	double   *dj 		= NULL;

	struct cpxlp *lp 	= NULL;
	char 	message_string[LINE_LEN + 1];
	int  	status;
	int     i, j;
	int     curmac, curmar;

	FILE     *f_mps   	= NULL;
  	double 	toleranz;
	double 	dtoosmall, dtoobig = 0.0;
	int     ntoosmall, ntoobig = 0;
	time_t	time_;


	if (copy_in_file != write_mps)
	  {time_ = time((time_t *) NULL);
	   if ((f_mps = fopen (name, "w+")) == (FILE*) NULL)
             error1 ("Error: Can't open MPS-file: ", name);
	   write_MPS (lp_, f_mps);
	   fclose (f_mps);
	   *copy_time_ = (float) (time((time_t *) NULL) - time_) ;
	  };

	time_ = time((time_t *) NULL);
        setscr_ind (1);   
	
	settilim (-10.0, &dtoosmall, &dtoobig);
	settilim (dtoobig, &dtoosmall, &dtoobig);

 	setndlim (-10, &ntoosmall, &ntoobig);
	setndlim (ntoobig, &ntoosmall, &ntoobig);

        status = mpsmread (copy_in_file != write_mps ? name : mps_file, 
		      &mac, &mar, &mae, &objsen, &objx, &rhsx,
                      &senx, &matbeg, &matcnt, &matind, &matval,
                      &bdl, &bdu, &rngval, &nrowind,
                      &etype, &enzbeg, &enzcnt, &enzind, &enzval,
                      &dataname, &objname, &rhsname, &rngname,
                      &bndname, &cname, &cstore, &rname, &rstore,
                      &ename, &estore,
                      &macsz, &marsz, &matsz, &maesz, &enzsz,
                      &cstorsz, &rstorsz, &estorsz,
                      &ctype, &nsos, &nsosnz, &sostype, &sosbeg, &sosind,
                      &sosref);
        if ( status )  
          goto TERMINATE;
 
	*copy_time_ += (float) (time((time_t *) NULL) - time_) ;
	time_ = time((time_t *) NULL);

        lp = loadmprob (copy_in_file != write_mps ? name : mps_file, 
                   mac, mar, mae, objsen, objx, rhsx,
                   senx, matbeg, matcnt, matind, matval,
                   bdl, bdu, rngval, nrowind,
                   etype, enzbeg, enzcnt, enzind, enzval,
                   dataname, objname, rhsname, rngname,
                   bndname, cname, cstore, rname, rstore,
                   ename, estore,
                   macsz, marsz, matsz, maesz, enzsz,
                   cstorsz, rstorsz, estorsz, ctype);
        if ( lp == NULL )  
          goto TERMINATE;
 
        status = mipoptimize (lp);
        if ( status && (status != CPX_INFEASIBLE) 
		    && (status != CPXERR_BOUNDS_INFEAS)
		    && (status != CPXERR_PRESLV_INForUNBD) )  
          goto TERMINATE;
 
	getepint (&toleranz);

        curmac = getmac (lp);
        curmar = getmar (lp);

        x     = (double *) malloc ((unsigned) curmac * sizeof (double));
        pi    = (double *) malloc ((unsigned) curmar * sizeof (double));
        slack = (double *) malloc ((unsigned) curmar * sizeof (double));
        dj    = (double *) malloc ((unsigned) curmac * sizeof (double));

        if ( !x || !pi || !slack || !dj )  goto TERMINATE;
 
   	mipstat = getstat (lp);
   	nodecnt = getndc (lp);
	itercnt = getitcm (lp);

	if ( mipstat == CPXMIP_INFEASIBLE       ||
	     status  == CPXERR_BOUNDS_INFEAS    ||
	     status  == CPXERR_PRESLV_INForUNBD ||
             mipstat == CPXMIP_NODE_LIM_INFEAS  ||
             mipstat == CPXMIP_TIME_LIM_INFEAS  ||
             mipstat == CPXMIP_FAIL_INFEAS        ) 
          getbobjval (lp, &obj); 
	else
    	  getmobjval (lp, &obj);

   	getmx (lp, x, 0, curmac-1);

	obj = pos(obj);

	if ((mipstat == CPXMIP_OPTIMAL) && (obj == 0.0) && (!without_traps))
          {unsigned int curmar;
	   unsigned int curmac;
	   int      pstat;
	   double   *dual;
	   double   *sol;
 
	   chgprtype (lp, 3);
	   optimize (lp);

	   curmar = getmar (lp);
	   curmac = getmac (lp);
	   dual   = (double *) malloc ((unsigned) curmar * sizeof (double));
	   sol    = (double *) malloc ((unsigned) curmac * sizeof (double));
	   solution (lp, &pstat, NULL, sol, dual, NULL, NULL);
	   
	   if ( pstat == CPX_OPTIMAL && !dual_is_trap_cplex (dual, sol, 
                                                             net, num_st) )
	     obj = 1.0;

	   free (dual);
	   free (sol);
	  }
	   

	time__[2] = (float) (time((time_t *) NULL) - time_) ;

 	if ((mipstat == CPXMIP_INFEASIBLE)       ||
	    (status  == CPXERR_BOUNDS_INFEAS )   ||
	    (status  == CPXERR_PRESLV_INForUNBD) || 
           ((mipstat == CPXMIP_OPTIMAL) && (obj == 0.0) && (!without_traps)))
	       fprintf (out_f, "%s\n", sent1); 
	else if ((mipstat == CPXMIP_OPTIMAL) && 
		 (obj > 0.0))
	   {fprintf (out_f, "%s\n", sent2); 
	    for (j = 0; j < curmac; j++)
	       if ((pos (x[j]) > 1.0e-05) && (cname[j][0] == 'm'))
                 if (max_tokens != 1 )
	           fprintf (out_f, "%s(%d) ", &(cname[j][1]), (int) pos(x[j]));
 		 else fprintf (out_f, "%s ", &(cname[j][1]));
  	   }
	else {if ((mipstat >= CPXMIP_OPTIMAL) && 
		  (mipstat <= CPXMIP_FAIL_FEAS_NO_TREE))
		fprintf (stderr, 
		"CPLEX exists with status: %s\n", 
		cplex_mip_msg[mipstat - CPXMIP_OPTIMAL]);
	      else if ((mipstat >= CPX_OPTIMAL) && 
		  (mipstat <= CPX_ABORT_CROSSOVER))
		fprintf (stderr, 
		"CPLEX exists with status: %s\n", cplex_msg[0]);
	      else
		fprintf (stderr, "%s\n", cplex_msg[1]);

	      goto TERMINATE;
	     };
 
	printf ("Result: %s Objective %s %e.\nNode: %d Iteration: %d\n", 
		(mipstat >= CPXMIP_OPTIMAL) && 
		(mipstat <= CPXMIP_FAIL_FEAS_NO_TREE)   ? 
		cplex_mip_msg[mipstat - CPXMIP_OPTIMAL] :
		((mipstat >= CPX_OPTIMAL) && 
		  (mipstat <= CPX_ABORT_CROSSOVER) ? cplex_msg[0] : "??"),

	     	mipstat == CPXMIP_INFEASIBLE       ||
		status  == CPXERR_PRESLV_INForUNBD ||
		status  == CPXERR_BOUNDS_INFEAS    ||
             	mipstat == CPXMIP_NODE_LIM_INFEAS  ||
             	mipstat == CPXMIP_TIME_LIM_INFEAS  ||
             	mipstat == CPXMIP_FAIL_INFEAS ? "~" : "=", 
	        obj, nodecnt, itercnt);

	if (debugging_on)
 	  for (j = 0; j < curmac; j++)
             printf ("%s:  Value = %e \n",
                           cname[j], x[j]);

   freeprob (&lp);  
   if ( objx )     free ((char *) objx);
   if ( rhsx )     free ((char *) rhsx);
   if ( senx )     free ((char *) senx);
   if ( matbeg )   free ((char *) matbeg);
   if ( matcnt )   free ((char *) matcnt);
   if ( matind )   free ((char *) matind);
   if ( matval )   free ((char *) matval);
   if ( bdl )      free ((char *) bdl);
   if ( bdu )      free ((char *) bdu);
   if ( rngval )   free ((char *) rngval);
   if ( nrowind )  free ((char *) nrowind);
   if ( etype )    free ((char *) etype);
   if ( enzbeg )   free ((char *) enzbeg);
   if ( enzcnt )   free ((char *) enzcnt);
   if ( enzind )   free ((char *) enzind);
   if ( enzval )   free ((char *) enzval);
   if ( dataname ) free ((char *) dataname);
   if ( objname )  free ((char *) objname);
   if ( rhsname )  free ((char *) rhsname);
   if ( rngname )  free ((char *) rngname);
   if ( bndname )  free ((char *) bndname);
   if ( cname )    free ((char *) cname);
   if ( cstore )   free ((char *) cstore);
   if ( rname )    free ((char *) rname);
   if ( rstore )   free ((char *) rstore);
   if ( ename )    free ((char *) ename);
   if ( estore )   free ((char *) estore);
   if ( ctype )    free ((char *) ctype);
   if ( sostype )  free ((char *) sostype);
   if ( sosbeg )   free ((char *) sosbeg);
   if ( sosind )   free ((char *) sosind);
   if ( sosref )   free ((char *) sosref);

   if ( x )        free ((char *) x);
   if ( pi )       free ((char *) pi);
   if ( slack )    free ((char *) slack);
   if ( dj )       free ((char *) dj);
   {char dummy[200];
    sprintf (dummy, "\\rm %s", name);
    system (dummy);
    free (name);
   };
   return;
	 
TERMINATE:

   freeprob (&lp);  
   if ( objx )     free ((char *) objx);
   if ( rhsx )     free ((char *) rhsx);
   if ( senx )     free ((char *) senx);
   if ( matbeg )   free ((char *) matbeg);
   if ( matcnt )   free ((char *) matcnt);
   if ( matind )   free ((char *) matind);
   if ( matval )   free ((char *) matval);
   if ( bdl )      free ((char *) bdl);
   if ( bdu )      free ((char *) bdu);
   if ( rngval )   free ((char *) rngval);
   if ( nrowind )  free ((char *) nrowind);
   if ( etype )    free ((char *) etype);
   if ( enzbeg )   free ((char *) enzbeg);
   if ( enzcnt )   free ((char *) enzcnt);
   if ( enzind )   free ((char *) enzind);
   if ( enzval )   free ((char *) enzval);
   if ( dataname ) free ((char *) dataname);
   if ( objname )  free ((char *) objname);
   if ( rhsname )  free ((char *) rhsname);
   if ( rngname )  free ((char *) rngname);
   if ( bndname )  free ((char *) bndname);
   if ( cname )    free ((char *) cname);
   if ( cstore )   free ((char *) cstore);
   if ( rname )    free ((char *) rname);
   if ( rstore )   free ((char *) rstore);
   if ( ename )    free ((char *) ename);
   if ( estore )   free ((char *) estore);
   if ( ctype )    free ((char *) ctype);
   if ( sostype )  free ((char *) sostype);
   if ( sosbeg )   free ((char *) sosbeg);
   if ( sosind )   free ((char *) sosind);
   if ( sosref )   free ((char *) sosref);


   if ( x )        free ((char *) x);
   if ( pi )       free ((char *) pi);
   if ( slack )    free ((char *) slack);
   if ( dj )       free ((char *) dj);
   {char dummy[200];
    sprintf (dummy, "\\rm %s", name);
    system (dummy);
    free (name);
   }
   printf ("\nNr.: %u %u\n", mipstat, status);
   error1 ("Error occurred using CPLEX.\n", "");
}

#if defined(__STDC__) || defined (__cplusplus)
void cplex_solve_unf (lprec *lp_, LMPNet net,
		      int debugging_on, char *mps_file, float *copy_time_,
		      int copy_in_file, int mark_size)
#else
void cplex_solve_unf (lp_, net, debugging_on, mps_file, 
		      copy_time_, copy_in_file, mark_size)
	LMPNet net;
	lprec* lp_;
	int    without_traps;
	int    debugging_on;
	char*  mps_file;
	float* copy_time_;
	int    copy_in_file;
	int    mark_size;
#endif
{
	char     *name 		= "/tmp/xxx.mps";
	int      mac;
	int      mar;
	int      mae;
	int      nsos;
	int      nsosnz;
	int      objsen;
	double   *objx 		= NULL;
	double   *rhsx 		= NULL;
	char     *senx 		= NULL;
	double   *rngval 	= NULL;
	int      *nrowind 	= NULL;
	int      *matbeg 	= NULL;
	int      *matcnt 	= NULL;
	int      *matind 	= NULL;
	double   *matval 	= NULL;
	double   *bdl 		= NULL;
	double   *bdu 		= NULL;
	int      *etype 	= NULL;
	int      *enzbeg 	= NULL;
	int      *enzcnt 	= NULL;
	int      *enzind 	= NULL;
	double   *enzval 	= NULL;
	char     *dataname 	= NULL;
	char     *objname 	= NULL;
	char     *rhsname 	= NULL;
	char     *rngname 	= NULL;
	char     *bndname 	= NULL;
	char     **cname 	= NULL;
	char     *cstore 	= NULL;
	char     **rname 	= NULL;
	char     *rstore 	= NULL;
	char     *estore 	= NULL;
	char     **ename 	= NULL;
	char     *ctype  	= NULL;
	char     *sostype 	= NULL;
	int      *sosbeg 	= NULL;
	int      *sosind	= NULL;
	double   *sosref 	= NULL;
	int      macsz 		= 0;
	int      marsz 		= 0;
	int      matsz 		= 0;
	int      maesz 		= 0;
	int      enzsz		 = 0;
	unsigned cstorsz 	= 0;
	unsigned rstorsz 	= 0;
	unsigned estorsz 	= 0;
	int      lpstat;
	int      mipstat;
	int      nodecnt;
 	int 	 itercnt;
	double   obj;
	double   *pi 		= NULL;
	double   *x 		= NULL;
	double   *slack 	= NULL;
	double   *dj 		= NULL;

	struct cpxlp *lp 	= NULL;
	char 	message_string[LINE_LEN + 1];
	int  	status;
	int     i, j;
	int     curmac, curmar;

	int	*colindex	= NULL;
	int 	*direction	= NULL;
	int 	*priority	= NULL;

	FILE     *f_mps   	= NULL;
  	double 	toleranz;
	double 	dtoosmall, dtoobig = 0.0;
	int     ntoosmall, ntoobig = 0;
	time_t	time_;


	if (copy_in_file != write_mps)
	  {time_ = time((time_t *) NULL);
	   if ((f_mps = fopen (name, "w+")) == (FILE*) NULL)
             error1 ("Error: Can't open MPS-file: ", name);
	   write_MPS (lp_, f_mps);
	   fclose (f_mps);
	   *copy_time_ = (float) (time((time_t *) NULL) - time_) ;
	  };

	time_ = time((time_t *) NULL);
        setscr_ind (1);   
	
	settilim (-10.0, &dtoosmall, &dtoobig);
	settilim (dtoobig, &dtoosmall, &dtoobig);

	/*setmslim (1, &ntoosmall, &ntoobig);*/

	/*setdpriind (CPX_DPRIIND_STEEPQSTART, &ntoosmall, &ntoobig);
	setdpriind (CPX_DPRIIND_STEEP, &ntoosmall, &ntoobig);*/

 	setndlim (-10, &ntoosmall, &ntoobig);
	setndlim (ntoobig, &ntoosmall, &ntoobig);

        status = mpsmread (copy_in_file != write_mps ? name : mps_file, 
		      &mac, &mar, &mae, &objsen, &objx, &rhsx,
                      &senx, &matbeg, &matcnt, &matind, &matval,
                      &bdl, &bdu, &rngval, &nrowind,
                      &etype, &enzbeg, &enzcnt, &enzind, &enzval,
                      &dataname, &objname, &rhsname, &rngname,
                      &bndname, &cname, &cstore, &rname, &rstore,
                      &ename, &estore,
                      &macsz, &marsz, &matsz, &maesz, &enzsz,
                      &cstorsz, &rstorsz, &estorsz,
                      &ctype, &nsos, &nsosnz, &sostype, &sosbeg, &sosind,
                      &sosref);
        if ( status )  
          goto TERMINATE;

	*copy_time_ += (float) (time((time_t *) NULL) - time_) ;
	time_ = time((time_t *) NULL);

        lp = loadmprob (copy_in_file != write_mps ? name : mps_file, 
                   mac, mar, mae, objsen, objx, rhsx,
                   senx, matbeg, matcnt, matind, matval,
                   bdl, bdu, rngval, nrowind,
                   etype, enzbeg, enzcnt, enzind, enzval,
                   dataname, objname, rhsname, rngname,
                   bndname, cname, cstore, rname, rstore,
                   ename, estore,
                   macsz, marsz, matsz, maesz, enzsz,
                   cstorsz, rstorsz, estorsz, ctype);
        if ( lp == NULL )  
          goto TERMINATE;


	colindex = (int*) malloc ((unsigned) mac * sizeof (int));
	priority = (int*) malloc ((unsigned) mac * sizeof (int));
	direction = (int*) malloc ((unsigned) mac * sizeof (int));
	if ( !colindex || !priority || !direction ) goto TERMINATE;

	time__[2] = (float) (time((time_t *) NULL) - time_) ;
	time_ = time((time_t *) NULL);
	/*for (j = 0; j < mac; j++)
	  {colindex[j] = j;
	   if (j < mark_size)
	     {priority[j] = 0;
	      direction[j] = 1;
	     }
	   else
	     {priority[j] = (j - mark_size) + 1;
	      direction[j] = 2;
	     }
	  };

	status = loadorder (lp, mac, colindex, priority, direction);
	if ( status )  
          goto TERMINATE;
	*/
	time__[1] += (float) (time((time_t *) NULL) - time_) ;
	time_ = time((time_t *) NULL);

	status = mipoptimize (lp);
        if ( status && (status != CPX_INFEASIBLE) 
		    && (status != CPXERR_BOUNDS_INFEAS)
		    && (status != CPXERR_PRESLV_INForUNBD) )  
          goto TERMINATE;

	getepint (&toleranz);

        curmac = getmac (lp);
        curmar = getmar (lp);

        x     = (double *) malloc ((unsigned) curmac * sizeof (double));
        pi    = (double *) malloc ((unsigned) curmar * sizeof (double));
        slack = (double *) malloc ((unsigned) curmar * sizeof (double));
        dj    = (double *) malloc ((unsigned) curmac * sizeof (double));

        if ( !x || !pi || !slack || !dj )  goto TERMINATE;

   	mipstat = getstat (lp);
   	nodecnt = getndc (lp);
	itercnt = getitcm (lp);

	if ( mipstat == CPXMIP_INFEASIBLE       ||
	     status  == CPXERR_BOUNDS_INFEAS    ||
	     status  == CPXERR_PRESLV_INForUNBD ||
             mipstat == CPXMIP_NODE_LIM_INFEAS  ||
             mipstat == CPXMIP_TIME_LIM_INFEAS  ||
             mipstat == CPXMIP_FAIL_INFEAS       ) 
          getbobjval (lp, &obj); 
	else
    	  getmobjval (lp, &obj);

   	getmx (lp, x, 0, curmac-1);

	obj = pos(obj);

	time__[2] = (float) (time((time_t *) NULL) - time_) ;
	
 	if ( (mipstat == CPXMIP_INFEASIBLE)       ||
	     (status  == CPXERR_PRESLV_INForUNBD) ||
	     (status  == CPXERR_BOUNDS_INFEAS) )
	    fprintf (out_f, "Net is deadlock-free\n"); 
	else if (mipstat == CPXMIP_OPTIMAL)   
               {DSet fire,
		     single;
		int i, 
		    size = 0;

		SETinitSets ();

	        for (j = 0; j < curmac; j++)
	         if ((pos (x[j] - 1.0) < toleranz) && (cname[j][0] == 't'))
                   size++;

	        fire = SETalloc (size);
	        *fire = 0;
	        single = SETalloc (1);
	        *(single + 1) = 0;

	        for (j = 0; j < curmac; j++)
	         if ((pos (x[j] - 1.0) < toleranz) && (cname[j][0] == 't'))
	            {Transition tr = find_T (net, &(cname[j][1]));
	 
	             if ( ! tr )
	               ERROR ("Could not find event.")
	             else {*single = tr->numname + 1;
		           *(single + 1) = 0;
	                   fire = SETunion (fire, single);
	                  }
	            }
   
		fprintf (out_f, "Net has a deadlock\n"); 
	        print_sequence (net, fire, out_f);

	        SETremove (fire);
	        SETremove (single);
 	       }
	else {if ((mipstat >= CPXMIP_OPTIMAL) && 
		  (mipstat <= CPXMIP_FAIL_FEAS_NO_TREE))
		fprintf (stderr, 
		"CPLEX exists with status: %s\n", 
		cplex_mip_msg[mipstat - CPXMIP_OPTIMAL]);
	      else if ((mipstat >= CPX_OPTIMAL) && 
		  (mipstat <= CPX_ABORT_CROSSOVER))
		fprintf (stderr, 
		"CPLEX exists with status: %s\n", cplex_msg[0]);
	      else
		fprintf (stderr, "%s\n", cplex_msg[1]);

	      goto TERMINATE;
	     };

	printf ("Result: %s Nodes: %d Iteration: %d\n", 
		(mipstat >= CPXMIP_OPTIMAL) && 
		(mipstat <= CPXMIP_FAIL_FEAS_NO_TREE)   ? 
		cplex_mip_msg[mipstat - CPXMIP_OPTIMAL] :
		((mipstat >= CPX_OPTIMAL) && 
		  (mipstat <= CPX_ABORT_CROSSOVER) ? cplex_msg[0] : "??"),
		nodecnt, itercnt);

	if (debugging_on)
 	  for (j = 0; j < curmac; j++)
             printf ("%s:  Value = %10f  Reduced cost = %10f\n",
                           cname[j], x[j], dj[j]);

    freeprob (&lp);  
   if ( objx )     free ((char *) objx);
   if ( rhsx )     free ((char *) rhsx);
   if ( senx )     free ((char *) senx);
   if ( matbeg )   free ((char *) matbeg);
   if ( matcnt )   free ((char *) matcnt);
   if ( matind )   free ((char *) matind);
   if ( matval )   free ((char *) matval);
   if ( bdl )      free ((char *) bdl);
   if ( bdu )      free ((char *) bdu);
   if ( rngval )   free ((char *) rngval);
   if ( nrowind )  free ((char *) nrowind);
   if ( etype )    free ((char *) etype);
   if ( enzbeg )   free ((char *) enzbeg);
   if ( enzcnt )   free ((char *) enzcnt);
   if ( enzind )   free ((char *) enzind);
   if ( enzval )   free ((char *) enzval);
   if ( dataname ) free ((char *) dataname);
   if ( objname )  free ((char *) objname);
   if ( rhsname )  free ((char *) rhsname);
   if ( rngname )  free ((char *) rngname);
   if ( bndname )  free ((char *) bndname);
   if ( cname )    free ((char *) cname);
   if ( cstore )   free ((char *) cstore);
   if ( rname )    free ((char *) rname);
   if ( rstore )   free ((char *) rstore);
   if ( ename )    free ((char *) ename);
   if ( estore )   free ((char *) estore);
   if ( ctype )    free ((char *) ctype);
   if ( sostype )  free ((char *) sostype);
   if ( sosbeg )   free ((char *) sosbeg);
   if ( sosind )   free ((char *) sosind);
   if ( sosref )   free ((char *) sosref);

   if ( x )        free ((char *) x);
   if ( pi )       free ((char *) pi);
   if ( slack )    free ((char *) slack);
   if ( dj )       free ((char *) dj);

   if ( priority ) free ((int *) priority);
   if ( colindex ) free ((int *) colindex);
   if ( direction ) free ((int *) direction);
   {char dummy[200];
    sprintf (dummy, "\\rm %s", name);
    system (dummy);
    free (name);
    };
     return;
	 
TERMINATE:

    freeprob (&lp);  
   if ( objx )     free ((char *) objx);
   if ( rhsx )     free ((char *) rhsx);
   if ( senx )     free ((char *) senx);
   if ( matbeg )   free ((char *) matbeg);
   if ( matcnt )   free ((char *) matcnt);
   if ( matind )   free ((char *) matind);
   if ( matval )   free ((char *) matval);
   if ( bdl )      free ((char *) bdl);
   if ( bdu )      free ((char *) bdu);
   if ( rngval )   free ((char *) rngval);
   if ( nrowind )  free ((char *) nrowind);
   if ( etype )    free ((char *) etype);
   if ( enzbeg )   free ((char *) enzbeg);
   if ( enzcnt )   free ((char *) enzcnt);
   if ( enzind )   free ((char *) enzind);
   if ( enzval )   free ((char *) enzval);
   if ( dataname ) free ((char *) dataname);
   if ( objname )  free ((char *) objname);
   if ( rhsname )  free ((char *) rhsname);
   if ( rngname )  free ((char *) rngname);
   if ( bndname )  free ((char *) bndname);
   if ( cname )    free ((char *) cname);
   if ( cstore )   free ((char *) cstore);
   if ( rname )    free ((char *) rname);
   if ( rstore )   free ((char *) rstore);
   if ( ename )    free ((char *) ename);
   if ( estore )   free ((char *) estore);
   if ( ctype )    free ((char *) ctype);
   if ( sostype )  free ((char *) sostype);
   if ( sosbeg )   free ((char *) sosbeg);
   if ( sosind )   free ((char *) sosind);
   if ( sosref )   free ((char *) sosref);


   if ( x )        free ((char *) x);
   if ( pi )       free ((char *) pi);
   if ( slack )    free ((char *) slack);
   if ( dj )       free ((char *) dj);

   if ( priority ) free ((int *) priority);
   if ( colindex ) free ((int *) colindex);
   if ( direction ) free ((int *) direction);
   {char dummy[200];
    sprintf (dummy, "\\rm %s", name);
    system (dummy);
    free (name);
   };
   printf ("\nNr.: %u %u\n", mipstat, status);
   error1 ("Error occurred using CPLEX.\n", "");
}

#endif

#if defined(__STDC__) || defined (__cplusplus)
void check_net (LMPNet net)
#else
void check_net (net)
	LMPNet net;
#endif
{
	Place       pl = net->Places;
	Transition  tr = net->Transitions;
	unsigned int i = 0, j = 0;

	init_marking = (REAL*) malloc (sizeof (REAL) * net->NumPlaces);

	while (pl != (Place) NULL)
	  {if (pl->marks > 0)
	     {pl->label = ENTRY;
	      init_marking[j++] = (REAL) pl->marks;
	     }
	   else init_marking[j++] = 0.0;

	   if (strlen (pl->name) >= MAX_LEN)
	     {fprintf (stderr, 
              "Warning: Place name too long, modified! \"%s\" --> ",
		pl->name);
	      sprintf (pl->name, "x%u", ++i);
	      fprintf (stderr, "\"%s\" \n", pl->name);
	      
	     };
 	   pl = pl->nextplace;
	  }

	while (tr != (Transition) NULL)
	  {if (strlen (tr->name) >= MAX_LEN)
	     {fprintf (stderr, 
              "Warning: Transition name too long, modified! \"%s\" --> ",
		tr->name);
	      sprintf (tr->name, "x%u", ++i);
	      fprintf (stderr, "\"%s\" \n", tr->name);
	     };
	   tr = tr->nexttrans;
	  }
}


#if defined(__STDC__) || defined (__cplusplus)
void initialize_current_marking (LMPNet net, 
                                 unsigned int num_sub_marking, 
                                 char **argv)
#else
void initialize_current_marking (net, num_sub_marking, argv)
	LMPNet net;
	unsigned int num_sub_marking;
	char **argv;
#endif
{
 	Place p;
	int i;

	i = 0;
	p = net->Places;
 	while (p != (Place) NULL)
	  {p->marks = undef;
	   p->numname = (Cardinal32) i++;
  	   p = p->nextplace;
	  };

	for (i = 0; i < num_sub_marking; i++)
	  {p = find_P (net, &(argv[i][1]));
	   if (p == (Place) NULL)
	     {fprintf (stderr, 
                       "\nError: undefined place name %s.\n", &(argv[i][1]));
	      exit (1);
	     };
	   if (p->marks != undef)
	     {fprintf (stderr, "\nError: multiple option %s.\n", &(argv[i][0]));
	      exit (1);
	     };
	   if (argv[i][0] == '+')
	     p->marks = 1;
	   else if (argv[i][0] == '-')
	     p->marks = 0;
	   else
	     {fprintf (stderr, 
                       "\nError: undefined option delimiter %c.\n", argv[i][0]);
	      exit (1);
	     };
	  };
}

#if defined(__STDC__) || defined(__cplusplus)
void find_cutoffs (LMPNet net, unsigned int *size)
#else
void find_cutoffs (net, size)
	LMPNet net;
        unsigned int *size;
#endif
{
        Transition tr = net->Transitions;
	Place      pl = net->Places;

 	Cardinal32 pre_size, numb = 0;
 	STEdge pre;

	(*size) = 0;
 	while (tr != (Transition) NULL)
          {tr->numname = numb++;
/*           if (strncmp (tr->name, "CUT_", 4) == 0 )   */
	   if ( tr->label )
             tr->preset->weight = 0;
	   else
             tr->preset->weight = ++(*size);
           for (pre_size = 0,
                pre = tr->preset;
                pre != (STEdge) NULL;
                pre_size++,
                pre = pre->nextplace);
	   tr->postset->weight = pre_size;
	   tr = tr->nexttrans;
          };

	while (pl != (Place) NULL)
/*	  {if (pl->marks > 0) */
	  {if (! pl->preset )
	     pl->label = ENTRY;
 	   pl = pl->nextplace;
	  };
}

#if defined(__STDC__) || defined(__cplusplus)
void theta_transform_net (LMPNet net, int type)
#else
void theta_transform_net (net, type)
	LMPNet net;
        unsigned int type;
#endif
{
        Transition tr = net->Transitions;
        unsigned int size;
	Cardinal32 numb = 0;
        STEdge pre;
        TSEdge post;

	if (type == TRAP)
	  while (tr != (Transition) NULL)
            {for (size = 0,
                pre = tr->preset;
                pre != (STEdge) NULL;
                size++,
                pre = pre->nextplace);
             for (post = tr->postset;
                post != (TSEdge) NULL;
                post->weight = size,
                post = post->nextplace);
	     tr->numname = numb++; 
	     tr = tr->nexttrans;
           }
	else if (type == DEADLOCK)
          while (tr != (Transition) NULL)
            {for (size = 0,
                post = tr->postset;
                post != (TSEdge) NULL;
                size++,
                post = post->nextplace);
             for (pre = tr->preset;
                pre != (STEdge) NULL;
                pre->weight = size,
                pre = pre->nextplace);
	     tr->numname = numb++; 
	     tr = tr->nexttrans;
            }
	else
	  error ("Wrong type for <<theta_transform_net>>\n");

}

#if defined(__STDC__) || defined(__cplusplus)
void gentzen_transform_net (LMPNet net, unsigned int *num_st_arcs)
#else
void gentzen_transform_net (net, num_st_arcs)
	LMPNet net;
        unsigned int *num_st_arcs;
#endif
{
        Transition tr = net->Transitions;
 	Cardinal32 numb = 0;
	unsigned int size_pre, size_post;
        STEdge pre;
	TSEdge post;
 
	*num_st_arcs = 0;
 	while (tr != (Transition) NULL)
          {for (pre = tr->preset, size_pre = 0; 
                pre != (STEdge) NULL; 
                pre = pre->nextplace, size_pre++)
             pre->weight = (*num_st_arcs)++;
           for (post = tr->postset, size_post = 0;
                post != (TSEdge) NULL;
                post->weight = size_pre,
                post = post->nextplace, size_post++);
	   tr->numname = numb++; 
 	   tr = tr->nexttrans;
           }
}


#if defined(__STDC__) || defined(__cplusplus)
int b0_failure (lprec *lp, LMPNet net, unsigned int num_st)
#else
int b0_failure (lp, net, num_st)
	lprec *lp;
	LMPNet net;
	unsigned int num_st;
#endif
{
  int i;

  for (i = 1; i <= lp->columns; i++)
    if (lp->names_used              && 
        (lp->col_name[i][0] == 'b') && 
	(lp->best_solution[lp->rows+i] == 0.0))
	return dual_is_trap (lp, net, num_st);
   return FALSE;
 }

#if defined(__STDC__) || defined(__cplusplus)
void initialize_MIP_ext
                    (lprec **lp, 
                     LMPNet net, 
                     unsigned int num_unspecified_places,
		     unsigned int *s0,
		     unsigned int *s1,
		     unsigned int *s2,
		     unsigned int *s3,
		     unsigned int num_constr,
		     int          anti_degen,
		     int          without_traps,
		     int          without_invariants,
		     REAL	  max_coeff,
		     int          solve_LP,
		     REAL	  tolerance)
#else
void initialize_MIP_ext 
                    (lp, net, num_unspecified_places, s0, s1, s2, s3, 
		     num_constr, anti_degen, without_traps, without_invariants,
                     max_coeff, solve_LP, tolerance)
	lprec        **lp;
	LMPNet       net;
	unsigned int num_unspecified_places;
	unsigned int *s0;
	unsigned int *s1;
	unsigned int *s2;
	unsigend int *s3;
	unsigned int num_constr;
	int	     anti_degen;
	int          without_traps;
	int 	     without_invariants;
	REAL	     max_coeff;
	int	     solve_LP;
	REAL	     tolerance;
#endif
{
	nstring      name;
	unsigned int i;
	Place        pl = net->Places;
	Transition   tr = net->Transitions;


	(*lp) = make_lp (num_constr + num_unspecified_places, 
 	without_traps ? (num_unspecified_places + net->NumTransitions) 
                      : (without_invariants ? 
                        (num_unspecified_places + net->NumTransitions + 1) 
                      : (num_unspecified_places + 2 * net->NumTransitions + 1)));
    	 
	(*lp)->debug   = FALSE;
	(*lp)->verbose = FALSE;
	(*lp)->trace   = FALSE;
        (*lp)->anti_degen = anti_degen;
	(*lp)->epsilon = tolerance;

	set_maxim (*lp);
	(*s0) = 1;
	(*s1) = (without_traps ? 1 : (net->NumTransitions + 1));
	(*s2) = (without_traps ? 1 : ((*s1) + 1));
	(*s3) = (*s2) + num_unspecified_places;

	if (!without_traps)
	  {strcpy (name, "b0");
	   set_col_name (*lp, *s1, name);
/*	   set_mat (*lp, 0, *s1, max_coeff);	   define objective function */
	   set_mat (*lp, 0, *s1, 1.0);	        /* define objective function */
	  };
	if ( without_traps )
	  for (i = (*s2); i < (*s3); i++)
	    set_mat (*lp, 0, i, 1.0);

	while (tr != (Transition) NULL)
	  {if (!without_traps)
	     {/*sprintf (name, "x%u", (unsigned int) tr->numname + 1);*/
	      sprintf (name, "x%s", tr->name);
	      set_col_name (*lp, tr->numname + (*s0), name);
	     };
	   if (!without_invariants)
	     {/*sprintf (name, "t%u", (unsigned int) tr->numname + 1);*/
	      sprintf (name, "t%s", tr->name);	
              set_col_name (*lp, tr->numname + (*s3), name);
#ifdef _PAPER
	      set_int (*lp, tr->numname + (*s3), TRUE);
#endif
	     };
	   tr = tr->nexttrans;
	  };

	i = 0;
	while (pl != (Place) NULL)
	  {if (pl->marks == undef)
	  /*   {sprintf (name, "m%u", pl->numname + 1);*/
	     {sprintf (name, "m%s", pl->name);
	      set_col_name (*lp, (*s2) + i, name);

	      set_rh          (*lp, num_constr + i + 1, 1.0);
	      set_constr_type (*lp, num_constr + i + 1, LE);
	      set_mat         (*lp, num_constr + i + 1, (*s2) + i, 1.0);
	      set_upbo        (*lp, (*s2) + i, (REAL) max_tokens);
#ifndef _PAPER
	      if (!solve_LP)
 	        set_int      (*lp, (*s2) + i, TRUE);
#endif
	      pl->component = (Cardinal32) i++;
	     };
	   pl = pl->nextplace;
	  };
}

#if defined(__STDC__) || defined(__cplusplus)
void initialize_MIP (lprec **lp, LMPNet net, unsigned int num_unspecified_places,
		     unsigned int *s0,
		     unsigned int *s1,
		     unsigned int *s2,
		     unsigned int *s3,
		     unsigned int num_constr,
		     int          anti_degen,
		     int          without_traps,
		     int          without_invariants,
		     REAL	  max_coeff,
		     int	  solve_LP,
		     REAL         tolerance,
		     ROW_elm	  *rows,
		     unsigned int extra_constr)
#else
void initialize_MIP (lp, net, num_unspecified_places, s0, s1, s2, s3, 
		     num_constr, anti_degen, without_traps, without_invariants,
                     max_coeff, solve_LP, tolerance, rows, extra_constr)
	lprec        **lp;
	LMPNet       net;
	unsigned int num_unspecified_places;
	unsigned int *s0;
	unsigned int *s1;
	unsigned int *s2;
	unsigend int *s3;
	unsigned int num_constr;
	int	     anti_degen;
	int          without_traps;
	int 	     without_invariants;
	REAL	     max_coeff;
	int	     solve_LP;
	REAL	     tolerance;
	ROW_elm	     *rows;
	unsigned int extra_constr;
#endif
{
	nstring      name;
	unsigned int i;
	Place        pl = net->Places;
	Transition   tr = net->Transitions;


	(*lp) = make_lp (num_constr + extra_constr, 
 	without_traps ? (num_unspecified_places + net->NumTransitions) 
                      : (without_invariants ? 
                        (num_unspecified_places + net->NumTransitions + 1) 
                      : (num_unspecified_places + 2 * net->NumTransitions + 1)));
    	 
	(*lp)->debug   = FALSE;
	(*lp)->verbose = FALSE;
	(*lp)->trace   = FALSE;
        (*lp)->anti_degen = anti_degen;
	(*lp)->epsilon = tolerance;

	set_maxim (*lp);
	(*s0) = 1;
	(*s1) = (without_traps ? 1 : (net->NumTransitions + 1));
	(*s2) = (without_traps ? 1 : ((*s1) + 1));
	(*s3) = (*s2) + num_unspecified_places;

	if (!without_traps)
	  {
#ifndef _POLYHEDRON
           strcpy (name, "b0");
	   set_col_name (*lp, *s1, name);
#endif
	  /* sprintf (name, "x%u", *s1);
	   set_col_name (*lp, *s1, name);*/
/*	   set_mat (*lp, 0, *s1, max_coeff);	   define objective function */
	   set_mat (*lp, 0, *s1, 1.0);	        /* define objective function */
 	  }; 
	if ( without_traps )
	  for (i = (*s2); i < (*s3); i++)
	    set_mat (*lp, 0, i, 1.0);

	while (tr != (Transition) NULL)
	  {if (!without_traps)
	     {
#ifndef _POLYHEDRON
	      /*sprintf (name, "x%u", (unsigned int) tr->numname + 1);*/
	      sprintf (name, "x%s", tr->name);
	      set_col_name (*lp, tr->numname + (*s0), name);
#endif
	     };
	   if (!without_invariants)
	     {
#ifndef _POLYHEDRON
              /*sprintf (name, "t%u", (unsigned int) tr->numname + 1);*/
              sprintf (name, "t%s", tr->name);	
              set_col_name (*lp, tr->numname + (*s3), name);
#endif

#ifdef _PAPER
	      set_int (*lp, tr->numname + (*s3), TRUE);
#endif
	     };
	   tr = tr->nexttrans;
	  };

	i = 0;
	while (pl != (Place) NULL)
	  {if (pl->marks == undef)
	     {
#ifndef _POLYHEDRON
              /*sprintf (name, "m%u", pl->numname + 1);*/
              sprintf (name, "m%s", pl->name);
	      set_col_name (*lp, (*s2) + i, name);
#endif
	      set_upbo     (*lp, (*s2) + i, (REAL) max_tokens);
 
#ifndef _PAPER
	      if (!solve_LP)
	        set_int      (*lp, (*s2) + i, TRUE);
#endif
	      pl->component = (Cardinal32) i++;
	     };
	   pl = pl->nextplace;
	  };

	for (i = 0; i < extra_constr; i++)
	  {int j;
	   for (j = 0; j < rows[i].space; j++)
	     set_mat (*lp, num_constr + i + 1, 
		      (*s2) + rows[i].pos[j], rows[i].val[j]);
	   set_rh (*lp, num_constr + i + 1, rows[i].rhs);
	   set_constr_type (*lp, num_constr + i + 1, rows[i].sense);
	   free (rows[i].pos);
	   free (rows[i].val);
          }
}

#if defined(__STDC__) || defined(__cplusplus)
void initialize_MIP_unf (lprec **lp, LMPNet net, 
                         unsigned int num_unspecified_trans,
		         unsigned int *s0,
		         unsigned int *s1,
 		         unsigned int num_constr,
 		         REAL	      max_coeff,
		         int	      solve_LP)
#else
void initialize_MIP_unf (lp, net, num_unspecified_trans, s0, s1,  
		         num_constr, max_coeff, solve_LP)
	lprec        **lp;
	LMPNet       net;
	unsigned int num_unspecified_trans;
	unsigned int *s0;
	unsigned int *s1;
 	unsigned int num_constr;
 	REAL	     max_coeff;
	int	     solve_LP;
#endif
{
	nstring      name;
	unsigned int i;
	Place        pl = net->Places;
	Transition   tr = net->Transitions;


	(*lp) = make_lp (num_constr, net->NumPlaces + num_unspecified_trans);

	(*lp)->debug   = FALSE;
	(*lp)->verbose = FALSE;
	(*lp)->trace   = FALSE;
        (*lp)->anti_degen = TRUE;

	set_maxim (*lp);
	(*s0) = 1;
	(*s1) = (*s0) + net->NumPlaces;

 
	for (i = (*s0); i < (*s1); i++)
	  set_mat (*lp, 0, i, 1.0);

	/*for (i = (*s1); i < (*s1) + num_unspecified_trans; i++)
	  set_mat (*lp, 0, i, 1.0);*/
       
 
	while (tr != (Transition) NULL)
	  {if (tr->preset->weight != 0)
             {sprintf (name, "t%s", tr->name);	
              set_col_name (*lp, tr->preset->weight + (*s1) - 1, name);
 	      /*if (!solve_LP) */
 	        set_int (*lp, tr->preset->weight + (*s1) - 1, TRUE);
	      set_upbo (*lp, tr->preset->weight + (*s1) - 1, 1.0);
	     };
	   tr = tr->nexttrans;
	  };

 	i = 0;
	while (pl != (Place) NULL)
	  {if (pl->marks == undef)
	     {
#ifndef _POLYHEDRON
              /*sprintf (name, "m%u", pl->numname + 1);*/
              sprintf (name, "m%s", pl->name);
	      set_col_name (*lp, (*s0) + i, name);
#endif
	      /*if (solve_LP) */
	      /*  set_int (*lp, (*s0) + i, TRUE);*/
	      set_upbo  (*lp, (*s0) + i, 1.0);
 	      pl->component = (Cardinal32) i++;
	     };
	   pl = pl->nextplace;
	  };
}

#if defined(__STDC__) || defined(__cplusplus)
void initialize_MIP_stabil (lprec **lp, LMPNet net, 
			    unsigned int num_unspecified_places,
			    unsigned int num_st_arcs,
		            unsigned int *s0,
		            unsigned int *s1,
		            unsigned int *s2,
		            unsigned int *s3,
		            unsigned int num_constr,
		            int          anti_degen,
		            int          without_traps,
		            int          without_invariants,
		            REAL	 max_coeff,
		            int	         solve_LP, 
			    REAL         tolerance,
			    ROW_elm      *rows,
			    unsigned int extra_constr)
#else
void initialize_MIP_stabil (lp, net, num_unspecified_places, num_st_arcs, 
		     s0, s1, s2, s3, 
		     num_constr, anti_degen, without_traps, without_invariants,
                     max_coeff, solve_LP, tolerance, rows, extra_constr)
	lprec        **lp;
	LMPNet       net;
	unsigned int num_unspecified_places;
	unsigned int num_st_arcs;
	unsigned int *s0;
	unsigned int *s1;
	unsigned int *s2;
	unsigend int *s3;
	unsigned int num_constr;
	int	     anti_degen;
	int          without_traps;
	int 	     without_invariants;
	REAL	     max_coeff;
	int	     solve_LP;
	REAL         tolerance;
	ROW_elm	     *rows;
	unsigned int extra_constr;
#endif
{
	nstring      name;
	unsigned int i;
	STEdge       pre;
	Place        pl = net->Places;
	Transition   tr = net->Transitions;


	(*lp) = make_lp (num_constr + extra_constr, num_unspecified_places +
	(without_traps ? 0 : num_st_arcs + 1) +
	(without_invariants ? 0 : net->NumTransitions));
     	 
	(*lp)->debug   = FALSE;
	(*lp)->verbose = FALSE;
	(*lp)->trace   = FALSE;
        (*lp)->anti_degen = anti_degen;
	(*lp)->epsilon = tolerance;

	set_maxim (*lp);
	(*s0) = 1;
	(*s1) = (without_traps ? 1 : (num_st_arcs + 1));
	(*s2) = (without_traps ? 1 : ((*s1) + 1));
	(*s3) = (*s2) + num_unspecified_places;

	if (!without_traps)
	  {
#ifndef _POLYHEDRON
           strcpy (name, "b0");
	   set_col_name (*lp, *s1, name);
#endif
	  /* sprintf (name, "x%u", *s1);
	   set_col_name (*lp, *s1, name);*/
/*	   set_mat (*lp, 0, *s1, max_coeff);	   define objective function */
	   set_mat (*lp, 0, *s1, 1.0);	         /*define objective function */
 	  }; 
	if ( without_traps )
	  for (i = (*s2); i < (*s3); i++)
	    set_mat (*lp, 0, i, 1.0);

	while (tr != (Transition) NULL)
	  {if (!without_traps)
	     {
#ifndef _POLYHEDRON
	      /*sprintf (name, "x%u", (unsigned int) tr->numname + 1);*/
	      pre = tr->preset;
	      while (pre != (STEdge) NULL)
		{/*sprintf (name, "x%u", pre->weight);*/
		 sprintf (name, "%s_%s", pre->place->name, pre->trans->name);
	         set_col_name (*lp, pre->weight + (*s0), name);
		 pre = pre->nextplace;
	        }
#endif
	     };
	   if (!without_invariants)
	     {
#ifndef _POLYHEDRON
              /*sprintf (name, "t%u", (unsigned int) tr->numname + 1);*/
              sprintf (name, "t%s", tr->name);	
              set_col_name (*lp, tr->numname + (*s3), name);
#endif

#ifdef _PAPER
	      set_int (*lp, tr->numname + (*s3), TRUE);
#endif
	     };
	   tr = tr->nexttrans;
	  };

	i = 0;
	while (pl != (Place) NULL)
	  {if (pl->marks == undef)
	     {
#ifndef _POLYHEDRON
              /*sprintf (name, "m%u", pl->numname + 1);*/
              sprintf (name, "m%s", pl->name);
	      set_col_name (*lp, (*s2) + i, name);
#endif
	      set_upbo     (*lp, (*s2) + i, (REAL) max_tokens);
#ifndef _PAPER
	      if (!solve_LP)
	        set_int      (*lp, (*s2) + i, TRUE);
#endif
	      pl->component = (Cardinal32) i++;
	     };
	   pl = pl->nextplace;
	  };

	for (i = 0; i < extra_constr; i++)
	  {int j;
	   for (j = 0; j < rows[i].space; j++)
	     set_mat (*lp, num_constr + i + 1, 
		      (*s2) + rows[i].pos[j], rows[i].val[j]);
	   set_rh (*lp, num_constr + i + 1, rows[i].rhs);
	   set_constr_type (*lp, num_constr + i + 1, rows[i].sense);
	   free (rows[i].pos);
	   free (rows[i].val);
          };
}

#if defined(__STDC__) || defined(__cplusplus)
void check_reachable (char *file_name, char *con_file,
                      int anti_degen, int without_traps,
		      int without_invariants, 
	              char **submarking,
		      int num_sub_marking, int debugging_on, REAL max_coeff,
		      REAL tolerance, int solve_LP, int copy_in_file, 
		      int do_not_solve, int use_cplex)
#else
void check_reachable (file_name, con_file, 
                      anti_degen, without_traps, without_invariants,
                      submarking, num_sub_marking, debugging_on, max_coeff,
                      tolerance, solve_LP, copy_in_file, do_not_solve,
		      use_cplex)

	char *file_name;
	char *con_file;
	int anti_degen;
	int without_traps;
	int without_invariants;
	char **submarking;
	int num_sub_marking;
	int debugging_on;
	REAL max_coeff;
	REAL tolerance;
	int solve_LP;
	int copy_in_file;
        int do_not_solve;
	int use_cplex;
#endif
{
	LMPNet net;
	lprec  *lp;
	unsigned int start_col_trap,
		     start_col_b,
	             start_col_mark,
		     start_col_inv,
		     start_row_trap,
	             start_row_inv,
		     num_unspecified_places;

	int	     result;
  	time_t	     time_;

	FILE 	     *f;
	char         *output_file;

	ROW_elm      rows[MAX_ADD];
	unsigned int extra_constr = 0;
 
  	time_ = time((time_t *) NULL);
	output_file = (char*) malloc (sizeof (char) * 
                                (strlen (file_name) + 1 + strlen (EXT)) );
	output_file[0] = '\0';

	if ((net = read_PEP (file_name)) == (LMPNet) NULL)
	  error1 ("Cannot open file:", file_name);
	check_net (net);

	time__[0] = (time((time_t *) NULL) - time_);
	time_ = time((time_t *) NULL);

	theta_transform_net (net, TRAP);
	initialize_current_marking (net, num_sub_marking, submarking);
	num_unspecified_places = net->NumPlaces - num_sub_marking;

	if ( con_file != (char*) NULL )
	  read_extra_constraints (con_file, rows, &extra_constr, net);

	initialize_MIP (&lp, net, num_unspecified_places, 
			  &start_col_trap,
			  &start_col_b,
			  &start_col_mark,
		          &start_col_inv,
			  (without_traps || without_invariants) ?
			  net->NumPlaces : 2 * net->NumPlaces,
		          anti_degen, without_traps, without_invariants,
                          max_coeff, solve_LP, tolerance, rows, extra_constr);

	start_row_trap = 1;
	start_row_inv  = (without_traps ? 1 : (net->NumPlaces + 1));

	convertPEP2reach (lp, net, start_row_trap, 
				   start_row_inv,
				   start_col_trap,
				   start_col_b,
				   start_col_mark,
				   start_col_inv, 
                                   without_traps, without_invariants, max_coeff);	
	time__[1] = (time((time_t *) NULL) - time_);

	if (copy_in_file)
	  {strcpy (output_file, file_name);
	   strcpy (&(output_file[strlen (file_name)]), ".lp");
	  
	   f = fopen (output_file, "w+");
	   if ((copy_in_file == write_lp) || (copy_in_file == write_both))
	     {f = fopen (output_file, "w+");
              write_LP  (lp, f);
	      fclose (f);
	     }
 
 	   strcpy (&(output_file[strlen (file_name)]), ".mps");
	  
	   f = fopen (output_file, "w+");
	   if ((copy_in_file == write_mps) || (copy_in_file == write_both))
	     {f = fopen (output_file, "w+");
              write_MPS (lp, f);
	      fclose (f);
	     }
 	  };

	if (!do_not_solve)
	 {if (!use_cplex)
	    {time_ = time((time_t *) NULL);
	     result = solve (lp);
             if (result == INFEASIBLE && !without_traps)
	       {fprintf (out_f, "Marking should not be reachable\n"); 
	        print_marking (lp);
		printf ("Try 'pep2lp' without traps (use option -i)!\n");
 	       }
	     else if ((result == INFEASIBLE && without_traps) ||
                (result == OPTIMAL && b0_failure(lp, net, net->NumTransitions) 
                 && !without_traps))
	       fprintf (out_f, "Marking is not reachable\n"); 
	     else
	       {fprintf (out_f, "Marking may be reachable\n"); 
	        print_marking (lp);
	        print_sub_marking (net);
	       }

	     printf ("Result = %i and b0 = %e.\n", result, lp->best_solution[0]);

	     time__[2] = (time((time_t *) NULL) - time_);

	     if (debugging_on)
 	       print_results (lp);
	    }
	  else
#ifdef _NO_CPLEX
	    {printf ("CPLEX not yet implemented.\n");
	    }
#else
	    {cplex_solve (lp, without_traps, 
                          debugging_on, output_file, 
                          &(time__[3]), copy_in_file, 
			  "Marking is not reachable",
			  "Marking may be reachable", net, net->NumTransitions);
	    } 
#endif
	 };

	free (output_file);
	remove_net (&net);
	delete_lp (lp);

}

#if defined(__STDC__) || defined(__cplusplus)
void check_reachable_stabil (char *file_name, char *con_file,
			     int anti_degen, 
			     int without_traps,
		             int without_invariants, 
	                     char **submarking,
		             int num_sub_marking, int debugging_on, 
			     REAL max_coeff,
		             REAL tolerance, int solve_LP, 
			     int copy_in_file, int do_not_solve, int use_cplex)
#else
void check_reachable_stabil (file_name, con_file, anti_degen, without_traps, 
			     without_invariants,
                             submarking, num_sub_marking, debugging_on, 
			     max_coeff,
                             tolerance, solve_LP, copy_in_file, 
			     do_not_solve, use_cplex)

	char *file_name;
	char *con_file;
	int anti_degen;
	int without_traps;
	int without_invariants;
	char **submarking;
	int num_sub_marking;
	int debugging_on;
	REAL max_coeff;
	REAL tolerance;
	int solve_LP;
	int copy_in_file;
        int do_not_solve;
	int use_cplex;
#endif
{
	LMPNet net;
	lprec  *lp;
	unsigned int start_col_trap,
		     start_col_b,
	             start_col_mark,
		     start_col_inv,
		     start_row_trap,
	             start_row_inv,
		     num_st_arcs,
		     num_unspecified_places;

	int	     result;
  	time_t	     time_;
	FILE 	     *f;
	char         *output_file;
	ROW_elm	     rows[MAX_ADD];
	unsigned int extra_constr = 0;
 
  	time_ = time((time_t *) NULL);
	output_file = (char*) malloc (sizeof (char) * 
                                (strlen (file_name) + 1 + strlen (EXT)) );
	output_file[0] = '\0';

	if ((net = read_PEP (file_name)) == (LMPNet) NULL)
	  error1 ("Cannot open file:", file_name);
	check_net (net);

	time__[0] = (time((time_t *) NULL) - time_);
	time_ = time((time_t *) NULL);

 	gentzen_transform_net (net, &(num_st_arcs));
	initialize_current_marking (net, num_sub_marking, submarking);
	num_unspecified_places = net->NumPlaces - num_sub_marking;

	if ( con_file != (char*) NULL )
	  read_extra_constraints (con_file, rows, &extra_constr, net);

 	initialize_MIP_stabil (&lp, net, num_unspecified_places, num_st_arcs,
			       &start_col_trap,
			       &start_col_b,
			       &start_col_mark,
		               &start_col_inv,
			       (without_traps || without_invariants) ?
			       net->NumPlaces : 2 * net->NumPlaces,
		               anti_degen, without_traps, without_invariants,
                               max_coeff, solve_LP, tolerance, 
			       rows, extra_constr);

	start_row_trap = 1;
	start_row_inv  = (without_traps ? 1 : (net->NumPlaces + 1));
 
	convertPEP2reach_stabil (lp, net, start_row_trap, 
				 start_row_inv,
				 start_col_trap,
				 start_col_b,
				 start_col_mark,
				 start_col_inv, 
                                 without_traps, without_invariants, max_coeff);	

	time__[1] = (time((time_t *) NULL) - time_);

	if (copy_in_file)
	  {strcpy (output_file, file_name);
	   strcpy (&(output_file[strlen (file_name)]), ".lp");
	  
	   f = fopen (output_file, "w+");
	   if ((copy_in_file == write_lp) || (copy_in_file == write_both))
	     {f = fopen (output_file, "w+");
              write_LP  (lp, f);
	      fclose (f);
	     }
 
 	   strcpy (&(output_file[strlen (file_name)]), ".mps");
	  
	   f = fopen (output_file, "w+");
	   if ((copy_in_file == write_mps) || (copy_in_file == write_both))
	     {f = fopen (output_file, "w+");
              write_MPS (lp, f);
	      fclose (f);
	     }
 	  };

	if (!do_not_solve)
	 {if (!use_cplex)
	    {time_ = time((time_t *) NULL);
	     result = solve (lp);
             if (result == INFEASIBLE && !without_traps)
	       {fprintf (out_f, "Marking should not be reachable\n"); 
	        print_marking (lp);
		printf ("Try 'pep2lp' without traps (use option -i)!\n");
 	       }
	     else if ((result == INFEASIBLE && without_traps) ||
                (result == OPTIMAL && b0_failure(lp, net, num_st_arcs) 
                 && !without_traps))
	       fprintf (out_f, "Marking is not reachable\n"); 
	     else
	       {fprintf (out_f, "Marking may be reachable\n"); 
	        print_marking (lp);
	        print_sub_marking (net);
	       }

	     printf ("Result = %i and b0 = %e.\n", result, lp->best_solution[0]);

	     time__[2] = (time((time_t *) NULL) - time_);

	     if (debugging_on)
 	       print_results (lp);
	    }
	  else
#ifdef _NO_CPLEX
	    {printf ("CPLEX not yet implemented.\n");
	    }
#else
	    {cplex_solve (lp, without_traps, 
                          debugging_on, output_file, 
                          &(time__[3]), copy_in_file,
			  "Marking is not reachable",
			  "Marking may be reachable", net, num_st_arcs);
	    }
#endif
	 };

	free (output_file);
	remove_net (&net);
	delete_lp (lp);
}
 
#if defined(__STDC__) || defined(__cplusplus)
void check_deadlock (char *file_name, char *con_file,
		     int anti_degen, int without_traps,
		     int without_invariants, int debugging_on, REAL max_coeff,
                     REAL tolerance, int solve_LP, int copy_in_file, 
		     int do_not_solve, int use_cplex)
#else
void check_deadlock (file_name, con_file, anti_degen, without_traps, 
		     without_invariants, debugging_on, 
                     max_coeff, tolerance, solve_LP, copy_in_file, 
		     do_not_solve, int use_cplex)
	 char *file_name;
	 char *con_file;
	 int   anti_degen;
	 int   without_traps;
	 int   without_invariants;
	 int   debugging_on;
	 REAL  max_coeff;
	 REAL  tolerance;
	 int   solve_LP;
	 int   copy_in_file;
	 int   do_not_solve;
	 int   use_cplex;
#endif
{
	LMPNet net;
	lprec  *lp;
	unsigned int start_col_trap,
		     start_col_b,
	             start_col_mark,
		     start_col_inv,
		     start_row_trap,
	             start_row_inv,
		     start_row_dead,
		     num_unspecified_places;

	int	     result;
  	time_t	     time_;
	FILE 	     *f;
	char         *output_file;
	ROW_elm	     rows[MAX_ADD];
	unsigned int extra_constr = 0;
 
  	time_ = time((time_t *) NULL);
	output_file = (char*) malloc (sizeof (char) * 
                                (strlen (file_name) + 1 + strlen (EXT)) );
	output_file[0] = '\0';

  	if ((net = read_PEP (file_name)) == (LMPNet) NULL)
	  error1 ("Cannot open file:", file_name);
	check_net (net);

	time__[0] = (time((time_t *) NULL) - time_);
	time_ = time((time_t *) NULL);

 	theta_transform_net (net, TRAP);
	initialize_current_marking (net, 0, (char**) NULL);
	num_unspecified_places = net->NumPlaces;

	if ( con_file != (char*) NULL )
	  read_extra_constraints (con_file, rows, &extra_constr, net);

	initialize_MIP  (&lp, net, num_unspecified_places, 
			  &start_col_trap,
			  &start_col_b,
			  &start_col_mark,
		          &start_col_inv, 
			  ((without_traps || without_invariants) ? 
                          net->NumPlaces :
		          2 * net->NumPlaces) + net->NumTransitions,
		          anti_degen, without_traps, without_invariants, 
			  max_coeff, solve_LP, tolerance, rows, extra_constr);

	start_row_trap =                                      1;
	start_row_inv  = without_traps ? 1 : net->NumPlaces + 1;
	start_row_dead = (without_traps || without_invariants)  
                                           ? net->NumPlaces + 1: 
                                         2 * net->NumPlaces + 1;

	convertPEP2dead (lp, net, start_row_trap, 
				  start_row_inv,
			          start_row_dead,
				  start_col_trap,
				  start_col_b,
				  start_col_mark,
				  start_col_inv,
				  without_traps, without_invariants, max_coeff);		
	time__[1] = (time((time_t *) NULL) - time_);

	if (copy_in_file)
	  {strcpy (output_file, file_name);
	   strcpy (&(output_file[strlen (file_name)]), ".lp");
	  
	   f = fopen (output_file, "w+");
	   if ((copy_in_file == write_lp) || (copy_in_file == write_both))
	     {f = fopen (output_file, "w+");
              write_LP  (lp, f);
	      fclose (f);
	     }
 
 	   strcpy (&(output_file[strlen (file_name)]), ".mps");
	  
	   f = fopen (output_file, "w+");
	   if ((copy_in_file == write_mps) || (copy_in_file == write_both))
	     {f = fopen (output_file, "w+");
              write_MPS (lp, f);
	      fclose (f);
	     }
 	  };
 	 
	if (!do_not_solve)
	 {if (!use_cplex)
	    {time_ = time((time_t *) NULL);
	     result = solve (lp);
             if (result == INFEASIBLE && !without_traps)
	       {fprintf (out_f, "Net should be deadlock-free\n"); 
	        print_marking (lp);
		printf ("Try 'pep2lp' without traps (use option -i)!\n");
 	       }
	     else if ((result == INFEASIBLE && without_traps) ||
                (result == OPTIMAL && b0_failure(lp, net, net->NumTransitions) 
                 && !without_traps))
	       fprintf (out_f, "Net is deadlock-free\n"); 
	     else
	       {fprintf (out_f, "Net may have a deadlock\n"); 
	        print_marking (lp);
 	       }

	     printf ("Result = %i and b0 = %e.\n", result, lp->best_solution[0]);

	     time__[2] = (time((time_t *) NULL) - time_);

	     if (debugging_on)
 	       print_results (lp);
	    }
	  else
#ifdef _NO_CPLEX
	    {printf ("CPLEX not yet implemented.\n");
	    }
#else
	    {cplex_solve (lp, without_traps, 
			  debugging_on, output_file, 
			  &(time__[3]), copy_in_file,
			  "Net is deadlock-free",
			  "Net may have a deadlock", net, net->NumTransitions);
	    }
#endif
	 };

	free (output_file);
 	remove_net (&net);
	delete_lp (lp);
}

#if defined(__STDC__) || defined(__cplusplus)
void check_deadlock_unf (char *file_name, int anti_degen, int without_traps,
		         int without_invariants, int debugging_on, 
                         REAL max_coeff,
                         REAL tolerance, int solve_LP, int copy_in_file, 
                         int do_not_solve, int use_cplex)
#else
void check_deadlock_unf (file_name, anti_degen, without_traps, 
		         without_invariants, debugging_on, 
                         max_coeff, tolerance, solve_LP, copy_in_file,
                         do_not_solve, use_cplex)
	 char *file_name;
	 int   anti_degen;
	 int   without_traps;
	 int   without_invariants;
	 int   debugging_on;
	 REAL  max_coeff;
	 REAL  tolerance;
	 int   solve_LP;
	 int   copy_in_file;
	 int   do_not_solve;
	 int   use_cplex;
#endif
{
	LMPNet net;
	lprec  *lp;
	unsigned int start_col_mark,
		     start_col_inv,
 	             start_row_inv,
		     start_row_dead,
		     num_unspecified_trans;

	int	     result;
  	time_t	     time_;
	FILE 	     *f;
	char         *output_file;

 
  	time_ = time((time_t *) NULL);
	output_file = (char*) malloc (sizeof (char) * 
                                (strlen (file_name) + 1 + strlen (EXT)) );
	output_file[0] = '\0';
 
	if ((net = readPrefix(file_name, NULL, NULL)) == (LMPNet) NULL)
	  error1 ("Cannot open file:", file_name);
 	time__[0] = (time((time_t *) NULL) - time_);
	time_ = time((time_t *) NULL);

	find_cutoffs (net, &num_unspecified_trans);
	initialize_current_marking (net, 0, (char**) NULL);
	
        initialize_MIP_unf (&lp, net, num_unspecified_trans, 
 			    &start_col_mark,
		            &start_col_inv, 
			    net->NumPlaces + net->NumTransitions,
 			    max_coeff, solve_LP);

  	start_row_inv  =  1;
	start_row_dead =  net->NumPlaces + 1;

 	convertPEP2dead_unf (lp, net,   
		             start_row_inv,
			     start_row_dead,
 			     start_col_mark,
			     start_col_inv,
			     max_coeff);	

	time__[1] = (time((time_t *) NULL) - time_);

	if (copy_in_file)
	  {strcpy (output_file, file_name);
	   strcpy (&(output_file[strlen (file_name)]), ".lp");
	  
	   f = fopen (output_file, "w+");
	   if ((copy_in_file == write_lp) || (copy_in_file == write_both))
	     {f = fopen (output_file, "w+");
              write_LP  (lp, f);
	      fclose (f);
	     }
 
 	   strcpy (&(output_file[strlen (file_name)]), ".mps");
	  
	   f = fopen (output_file, "w+");
	   if ((copy_in_file == write_mps) || (copy_in_file == write_both))
	     {f = fopen (output_file, "w+");
              write_MPS (lp, f);
	      fclose (f);
	     }
 	  };

	if (!do_not_solve)
	 {if (!use_cplex)
	    {time_ = time((time_t *) NULL);
	     result = solve (lp);
   	     if (result == INFEASIBLE)  
	       fprintf (out_f, "Net is deadlock-free\n"); 
	     else
	       {DSet fire,
		     single;
		int i, 
		    size = 0;

		SETinitSets ();

		for (i=1; i <= lp->columns; i++)
	          if (lp->names_used && (lp->col_name[i][0] == 't') &&
		      (my_abs (lp->best_solution[lp->rows+i] - 1.0) < DEF_EPSILON))
		    size++;

	        fire = SETalloc (size);
	        *fire = 0;
	        single = SETalloc (1);
	        *(single + 1) = 0;

	        for (i=1; i <= lp->columns; i++)
	          if (lp->names_used && (lp->col_name[i][0] == 't') &&
		      (my_abs (lp->best_solution[lp->rows+i] - 1.0) < DEF_EPSILON))
	            {Transition tr = find_T (net, &(lp->col_name[i][1]));
	 
	             if ( ! tr )
	               ERROR ("Could not find event.")
	             else {*single = tr->numname + 1;
		           *(single + 1) = 0;
	                   fire = SETunion (fire, single);
	                  }
	            }

		fprintf (out_f, "Net has a deadlock\n"); 
	        print_sequence (net, fire, out_f);

	        SETremove (fire);
	        SETremove (single);
 	       }

	     printf ("Result = %i.\n", result);

	     time__[2] = (time((time_t *) NULL) - time_);

	     if (debugging_on)
 	       print_results (lp);
	    }
	  else
#ifdef _NO_CPLEX
	    {printf ("CPLEX not yet implemented.\n");
	    }
#else
	    {cplex_solve_unf (lp, net,
			  debugging_on, output_file, 
			  &(time__[3]), copy_in_file, (int) net->NumPlaces);
 	    }
#endif
	 };

	free (output_file);
{ Place pl;
for (pl=net->Places;pl;pl=pl->nextplace)
pl->inscript=NULL;
}
   	remove_net (&net);
	delete_lp (lp);
}

#if defined(__STDC__) || defined(__cplusplus)
void check_deadlock_stabil (char *file_name,
			    char *con_file,
                            int anti_degen, 
                            int without_traps,
		            int without_invariants, 
                            int debugging_on, 
                            REAL max_coeff,
                            REAL tolerance, 
                            int solve_LP, 
                            int copy_in_file, 
                            int do_not_solve,
			    int use_cplex)
#else
void check_deadlock_stabil (file_name, con_file, anti_degen, without_traps, 
		            without_invariants, debugging_on, 
                            max_coeff, tolerance, solve_LP, 
                            copy_in_file, do_not_solve, use_cplex)
	 char *file_name;
	 char *con_file;
	 int   anti_degen;
	 int   without_traps;
	 int   without_invariants;
	 int   debugging_on;
	 REAL  max_coeff;
	 REAL  tolerance;
	 int   solve_LP;
	 int   copy_in_file;
	 int   do_not_solve;
	 int   use_cplex;
#endif
{
	LMPNet net;
	lprec  *lp;
	unsigned int start_col_trap,
		     start_col_b,
	             start_col_mark,
		     start_col_inv,
		     start_row_trap,
	             start_row_inv,
		     start_row_dead,
		     num_st_arcs,
		     num_unspecified_places;

	int	     result;
  	time_t	     time_;

	FILE 	     *f;
	char 	     *output_file; 
	ROW_elm	     rows[MAX_ADD];
	unsigned int extra_constr = 0;
 
  	time_ = time((time_t *) NULL);
	output_file = (char*) malloc (sizeof (char) * 
                              (strlen (file_name) + 1 + strlen (EXT)) );
	output_file[0] = '\0';

  	if ((net = read_PEP (file_name)) == (LMPNet) NULL)
	  error1 ("Cannot open file:", file_name);
	check_net (net);

	time__[0] = (time((time_t *) NULL) - time_);
	time_ = time((time_t *) NULL);

	gentzen_transform_net (net, &num_st_arcs);
	initialize_current_marking (net, 0, (char**) NULL);
	num_unspecified_places = net->NumPlaces;

	if ( con_file != (char*) NULL )
	  read_extra_constraints (con_file, rows, &extra_constr, net);

	initialize_MIP_stabil  (&lp, net, num_unspecified_places, 
				num_st_arcs,
			        &start_col_trap,
			        &start_col_b,
			        &start_col_mark,
		                &start_col_inv, 
				net->NumTransitions +
			        (without_traps      ? 0 : net->NumPlaces) +
			        (without_invariants ? 0 : net->NumPlaces),
 		                anti_degen, without_traps, without_invariants, 
			        max_coeff, solve_LP, tolerance,
				rows, extra_constr);

	start_row_trap =                                      1;
	start_row_inv  = without_traps ? 1 : net->NumPlaces + 1;
	start_row_dead = (without_traps || without_invariants)  
                                           ? net->NumPlaces + 1: 
                                         2 * net->NumPlaces + 1;

	convertPEP2dead_stabil (lp, net, start_row_trap, 
				start_row_inv,
			        start_row_dead,
				start_col_trap,
				start_col_b,
				start_col_mark,
				start_col_inv,
				without_traps, without_invariants, max_coeff);	

	time__[1] = (time((time_t *) NULL) - time_);

	if (copy_in_file)
	  {strcpy (output_file, file_name);
	   strcpy (&(output_file[strlen (file_name)]), ".lp");
	  
	   f = fopen (output_file, "w+");
	   if ((copy_in_file == write_lp) || (copy_in_file == write_both))
	     {f = fopen (output_file, "w+");
              write_LP  (lp, f);
	      fclose (f);
	     }
 
 	   strcpy (&(output_file[strlen (file_name)]), ".mps");
	  
	   f = fopen (output_file, "w+");
	   if ((copy_in_file == write_mps) || (copy_in_file == write_both))
	     {f = fopen (output_file, "w+");
              write_MPS (lp, f);
	      fclose (f);
	     }
 	  };

	if (!do_not_solve)
	 {if (!use_cplex)
	    {time_ = time((time_t *) NULL);
	     result = solve (lp);
   	     if (result == INFEASIBLE && !without_traps)
	       {fprintf (out_f, "Net should be deadlock-free\n"); 
	        print_marking (lp);
		printf ("Try 'pep2lp' without traps (use option -i)!\n");
 	       }
	     else if ((result == INFEASIBLE && without_traps) ||
                (result == OPTIMAL && b0_failure(lp, net, num_st_arcs) 
                 && !without_traps))
	       fprintf (out_f, "Net is deadlock-free\n"); 
	     else
	       {fprintf (out_f, "Net may have a deadlock\n"); 
	        print_marking (lp);
 	       }

	     printf ("Result = %i and b0 = %e.\n", result, lp->best_solution[0]);

	     time__[2] = (time((time_t *) NULL) - time_);

	     if (debugging_on)
 	       print_results (lp);
	    }
	  else
#ifdef _NO_CPLEX
	    {printf ("CPLEX not yet implemented.\n");
	    }
#else
	    {cplex_solve (lp, without_traps, 
			  debugging_on, output_file, 
			  &(time__[3]), copy_in_file,
		          "Net is deadlock-free",
 			  "Net may have a deadlock", net, num_st_arcs);
  	    }
#endif
 	 };
 
	free (output_file);
   	remove_net (&net);
	delete_lp (lp);
}

#if defined(__STDC__) || defined(__cplusplus)
void read_marking (char* file_name, int *argc, char **argv)
#else
void read_marking (file_name, argc, argv)
	char *file_name;
	int  *argc;
	char **argv;
#endif
{
	FILE *f;
	char c;

	if ((f = fopen (file_name, "r")) == (FILE*) NULL)
	  error1 ("Unable to open file : ", file_name);

	c = getc (f);
	ungetc (c, f);

	while (c != EOF)
	  {argv[*argc] = (char*) malloc (sizeof (char) * MAX_NAME);
	   fscanf (f, "%s", argv[*argc]);
	   *argc = *argc + 1;
 	   if (*argc == MAX_ARGS)
             error1 ("Too many arguments in file : ", file_name);
           while (isspace(c = getc(f)));
	   ungetc(c, f);
	  };

	fclose(f);
}

 
#if defined(__STDC__) || defined(__cplusplus)
int main (int argc, char **argv)
#else
int main (argc, argv)
	int argc;
	char **argv;
#endif
{
	int	anti_degen         = FALSE,
	        without_traps      = FALSE,
	        without_invariants = FALSE,
		debugging_on       = FALSE,
 	        solve_LP           = FALSE,
		copy_in_file       = FALSE,
                use_instabil_descr = FALSE,
	        do_not_solve       = FALSE,
	        use_cplex          = FALSE,
 		goal		   = 0,
		i, j;

	REAL	max_coeff	   = 1.0;
	REAL    tolerance          = 1.0e-8;
	float	dummy;
	int     ddummy;

	char	*net_file,
		*out_file,
		*in_file,
		*con_file;

	int	net_file_spec = FALSE,
		out_file_spec = FALSE,
		in_file_spec  = FALSE,
		con_file_spec = FALSE;

	char	**marking;
	int     size_of_m;

    marking = (char**) malloc (MAX_ARGS * sizeof (char*));
	
    if (argc == 1)
  {printf ("pep2lp -- reachability and deadlock-checker -- Version <2.1>\n\n");
   printf ("\nUsage: pep2lp <parameters>                                \n\n");
   
   printf ("-n=<file-name>   specify Petri net file,                    \n");
   printf ("-o=<file-name>   output in this file,                       \n");
   printf ("-m=<file-name>   specify the input file (submarking),       \n");
   printf ("-k=<file-name>   specify the input file (constraints),      \n");
   printf ("-a               use random perturbations,                  \n"); 
   printf ("                 to decrease numerical instability.         \n");
   printf ("-e=<real-number> tolerance for integrality:                 \n");
   printf ("                 0 < e < 0.5                                \n");
   printf ("-s               solve problem as a LP (no Integer LP).     \n");
   printf ("-d               makes debugging information available.     \n");
   printf ("-t               do only consider trap properties.          \n");
   printf ("-i               do only consider s-invariant properties.   \n");
   printf ("-c=<real-number> scaling factor for trap equation (improves \n");
   printf ("                 numerical stability of the LP).            \n");
   printf ("-q=<int-number>  upper bound for the maximal token number.  \n");
   printf ("-w               write LP to MPS-file.                      \n");
   printf ("-v               write LP to LP-file.                       \n");
   printf ("-@               use the trap characterization of Ezpeleta and \n");
   printf ("                 Couvreur (default is the Alaiwan and Toudic   \n");
   printf ("                 description).                              \n");
   printf ("-x               do not envoke the LP-solver.               \n");
   printf ("-y               use CPLEX instead of lp_solve.             \n");
   printf ("The following options specify the underlying LP:            \n");
   printf ("-R               check reachability.                        \n");
   printf ("-D               check deadlock-freedom of safe Petri nets. \n");
   printf ("-U               check deadlock-freedom of unfolded nets.   \n");
   printf ("\nwritten by Stephan Melzer, Technical University of Munich (SFB 342 A3).\n");
   printf ("using lp_solve written by M.R.C.M. Berkelaar.\n\n");
   return(1);
   }

	for (j = 1; j < argc; j++)
	  {if ((argv[j][0] != '-') || (strlen (argv[j]) <= 1))
	     error1 ("Wrong option : ", argv[j]);
	   else if (argv[j][1] == 'c')
	     {sscanf (argv[j], "-c=%g", &dummy);
	      max_coeff = (REAL) dummy;
 	     }
	   else if (argv[j][1] == 'e')
	     {sscanf (argv[j], "-e=%g", &dummy);
	      tolerance = (REAL) (dummy <= 0.0 ? 1.0e-8 : 
                                               (dummy >= 0.5 ? 0.49999 : dummy));
 	     }
	   else if (argv[j][1] == 'q')
	     {sscanf (argv[j], "-q=%d", &ddummy);
	      max_tokens = (unsigned int) ddummy;
 	     }
	   else if ((argv[j][1] == 'n') && (strlen (argv[j]) > 3 ))
	     {net_file_spec = TRUE;
	      net_file = (char*) malloc (sizeof (char) * strlen (argv[j]));
	      strcpy (net_file, &(argv[j][3]));
	     }
	  else if ((argv[j][1] == 'o') && (strlen (argv[j]) > 3 ))
	     {out_file_spec = TRUE;
	      out_file = (char*) malloc (sizeof (char) * strlen (argv[j]));
	      strcpy (out_file, &(argv[j][3]));
	     }
	  else if ((argv[j][1] == 'k') && (strlen (argv[j]) > 3 ))
	     {con_file_spec = TRUE;
	      con_file = (char*) malloc (sizeof (char) * strlen (argv[j]));
	      strcpy (con_file, &(argv[j][3]));
	     }
	  else if ((argv[j][1] == 'm') && (strlen (argv[j]) > 3 ))
	     {in_file_spec = TRUE;
	      in_file = (char*) malloc (sizeof (char) * strlen (argv[j]));
	      strcpy (in_file, &(argv[j][3]));
	     }
	  else
	   for (i = 1; i < strlen (argv[j]); i++)
	     if (argv[j][i] == 'a')
	       anti_degen = TRUE;
	     else if ((argv[j][i] == 'R') && (goal == 0))
	       goal = REACHABLE;
	     else if ((argv[j][i] == 'R') && (goal != REACHABLE))
	       error ("Multiple command option (-DUR).\n");
 	     else if ((argv[j][i] == 'D') && (goal == 0))
	       goal = DEADLOCK;
	     else if ((argv[j][i] == 'D') && (goal != DEADLOCK))
	       error ("Multiple command option (-DUR).\n");
	     else if ((argv[j][i] == 'U') && (goal == 0))
	       goal = DEADLOCK_UNF;
	     else if ((argv[j][i] == 'U') && (goal != DEADLOCK_UNF))
	       error ("Multiple command option (-DUR).\n");

  	     else if ((argv[j][i] == 't') && (! without_traps))
	       without_invariants = TRUE;
	     else if ((argv[j][i] == 't') && without_traps)
	       error ("Multiple exclusion of traps and s-invariants \
property.\n");
	     else if ((argv[j][i] == 'i') && (! without_invariants))
	       without_traps = TRUE;
	     else if ((argv[j][i] == 'i') && without_invariants)
	       error ("Multiple exclusion of traps and s-invariants \
property.\n");
	     else if (argv[j][i] == 'd')
		debugging_on = TRUE;
	     else if (argv[j][i] == 's')
	        solve_LP = TRUE;
	     else if (argv[j][i] == 'w')
		copy_in_file = ((copy_in_file == write_lp) ? 
				 write_both :write_mps);
	     else if (argv[j][i] == 'v')
		copy_in_file = ((copy_in_file == write_mps) ? 
				 write_both :write_lp);
	     else if (argv[j][i] == '@')
		use_instabil_descr = TRUE;
	     else if (argv[j][i] == 'x')
		do_not_solve = TRUE;
	     else if (argv[j][i] == 'y')
		use_cplex = TRUE;
       	};

	if ( (goal == DEADLOCK) && (max_tokens > 1) )
	  error ("Deadlock-Constraints are only valid for safe Petri nets.");

	if (goal == 0) 
	  error ("Unspecified command option.\n");
 
	if (!(net_file_spec))
	  error ("LL-net is not specified.\n");

	size_of_m = 0;

	if ( con_file_spec && in_file_spec )
	  error ("Do not use option '-k' and '-m' simultaneously.");

	if ( !con_file_spec )
	  con_file = (char*) NULL;

	if (in_file_spec)
	  read_marking (in_file, &size_of_m, marking);

 	if (out_file_spec)
	  {if ((out_f = fopen (out_file, "w")) == (FILE *) NULL)
	     error1 ("Unable to open file : ", out_file);
	  }
	else out_f = stdout;

	if ((goal == REACHABLE) && use_instabil_descr)
          check_reachable(net_file, con_file, anti_degen,
	  	  	  without_traps,
                          without_invariants, marking, size_of_m, 
	                  debugging_on,
			  max_coeff, tolerance, solve_LP, 
			  copy_in_file, do_not_solve, use_cplex);
	else if ((goal == REACHABLE) && !use_instabil_descr)
	  check_reachable_stabil 
			 (net_file, con_file, anti_degen,
	  	  	  without_traps,
                          without_invariants, marking, size_of_m, 
	                  debugging_on,
			  max_coeff, tolerance, solve_LP, 
			  copy_in_file, do_not_solve, use_cplex);
	else if ((goal == DEADLOCK) && use_instabil_descr)
          check_deadlock (net_file, con_file, anti_degen, 
			  without_traps,
			  without_invariants, debugging_on, max_coeff,
                          tolerance, solve_LP, 
  			  copy_in_file, do_not_solve, use_cplex);
	else if ((goal == DEADLOCK) && !use_instabil_descr)
          check_deadlock_stabil
                         (net_file, con_file, anti_degen, 
			  without_traps,
			  without_invariants, debugging_on, max_coeff,
                          tolerance, solve_LP, 
			  copy_in_file, do_not_solve, use_cplex);
        else if (goal == DEADLOCK_UNF)
          check_deadlock_unf 
	                 (net_file, anti_degen, 
			  TRUE, FALSE, debugging_on, max_coeff,
                          FALSE, solve_LP, copy_in_file, 
                          do_not_solve, use_cplex);
 
	
	if (net_file_spec)
	  free (net_file);
	if (in_file_spec)
	  free (in_file);
	if (out_file_spec)
	  free (out_file);

	for (j = 0; j < size_of_m; j++);
	  if (marking[j])
            free (marking[j]);
    
    free (marking);

    if ( init_marking )
	free ( init_marking );
    
    printf("Time for (read/gen/solve/copy) = (%.0f / %.0f / %.0f / %.0f) = %.0f + %.0f = %.0f sec.\n", 
	time__[0], time__[1], time__[2], time__[3], 
	time__[0] + time__[1] + time__[2], time__[3], 
	time__[0] + time__[1] + time__[2] + time__[3]);
	
    exit(0);

}
	 
