/*****************************************************************************/
/*  setop.c                                                                  */
/*  Mi.,04.11.1992 - fr.,10.01.1997                           Stefan Roemer  */
/*****************************************************************************/


#include <stdio.h>
#include <string.h>
#include "neunetze.h"
#include "setop.h"


/*****************************************************************************/
/* Globale Variablen fuer die Behandlung von Mengen.                         */
/*****************************************************************************/

Cardinal MaxEntries  = SET_MAX_ENTRIES;

DSet setDummy    = NULL;          /* Hilfsmenge bei der Berechnung von sum,  */
                                  /* union,...                               */

DSet setSingle   = NULL;          /* Hilfsmenge mit max. 1 Eintrag.          */


/*****************************************************************************/
/* Initialisieren der globalen Variablen der Mengenoperationen.              */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void SETinitSets( void )

#else

void SETinitSets()

#endif

  {

    MaxEntries = SET_MAX_ENTRIES;
    if ( setDummy )
      {
        SETremove( setDummy );
      } /* if */
    setDummy  = SETalloc( MaxEntries );
    if ( ! setSingle )
      {
        setSingle = SETalloc( 1 );
      } /* if */

  } /* SETinitSets() */


/*****************************************************************************/
/* Funktion zum Kopieren einer Menge in eine bereits bestehende.             */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

DSet SETcopy( DSet set1,
              DSet set2 )

#else

DSet SETcopy( set1, set2 )

  DSet set1,
       set2;                                         /* Kopiert set1 := set2 */

#endif

  {

    register DSet set1ptr = set1;

    while (( *(set1ptr++) = *(set2++) ))
      skip;

    return ( set1 );

  } /* SETcopy() */


/*****************************************************************************/
/* Funktion zur Initialisierung einer leeren Menge.                          */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

DSet SETcreate_empty_set( void )

#else

DSet SETcreate_empty_set()

#endif

  {                                                      /* Gibt {} zurueck */

    DSet set;

    *( set = SETalloc( 0 ) ) = 0;

    return ( set );

  } /* SETcreate_empty_set() */


/*****************************************************************************/
/* Funktion zur Initialisierung einer neuen Menge, die als einziges Element  */
/* 'akt' enthaelt.                                                           */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

DSet SETcreate_singleton( SetEntry act )

#else

DSet SETcreate_singleton( act )

  SetEntry act;                                        /* Gibt {akt} zurueck */

#endif

  {

    register DSet set;

    *( set = SETalloc( 1 ) ) = act;
    *( set + 1 ) = 0;

    return ( set );

  } /* SETcreate_singleton() */


/*****************************************************************************/
/* Funktion zur Initialisierung einer neuen Menge als Kopie einer bereits    */
/* vorhandenen Menge; der Speicherbedarf der neuen Menge entspricht ihrer    */
/* tatsaechlichen "Laenge".                                                   */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

DSet SETcreate_copy( DSet set )

#else

DSet SETcreate_copy( set )

  DSet set;

#endif

  {

    DSet              set2;         /* Zeiger auf die zu erzeugende Kopie.   */
    register DSet     setptr = set, /* Laufzeiger durch die Originalmenge.   */
                      set2ptr;      /* Laufzeiger durch die Kopie.           */
    register Cardinal len = 0;      /* Laenge der Originalmenge.             */

                                    /* "Laenge" der Menge ermitteln.         */
    while ( *setptr )
      {
        len++;
        setptr++;
      }
                                  /* Speicherplatz fuer die Kopie anfordern. */
    set2ptr = set2 = SETalloc( len );
    setptr = set;
                                  /* Eintraege einzeln kopieren.             */
    while (( *(set2ptr++) = *(setptr++) ))
      skip;
                                  /* Zeiger auf die Kopie zurueckliefern.    */
    return ( set2 );

  } /* SETcreate_copy() */


/*****************************************************************************/
/* Funktion zur Berechnung der Maechtigkeit einer Menge.                     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

Cardinal SETcardinality( DSet set )

#else

Cardinal SETcardinality( set )

  DSet set;                                               /* Berechnet |set| */

#endif

  {

    register Cardinal card = 0;

    while ( *(set++) )
      {
        card++;
      } /* while */

    return ( card );

  } /* SETcardinality() */


/*****************************************************************************/
/* Funktion zur Feststellung, ob das element 'akt' in der Menge 'set' ist.   */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

bool SETis_in( SetEntry akt,
                  DSet     set  )

#else

bool SETis_in( akt, set )

  SetEntry akt;
  DSet     set;

#endif

  {

    if ( set && *set && akt )
      {
        while ( *set && ( *set < akt ) )
          {
            set++;
          } /* while */
        return ( akt == *set ? True : False );
      } /* if */
                                  /* Bei Parameterfehler 'False' liefern.    */
    return ( False );

  } /* SETis_in() */


/*****************************************************************************/
/* Funktion zur Feststellung, ob zwei Mengen `set1` und `set2` gleich sind.  */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

bool SETis_equal( DSet set1,
                     DSet set2 )

#else

bool SETis_equal( set1, set2 )

  DSet set1,                                      /* Berechnet set1 == set2 */
       set2;

#endif

  {

                                  /* Die Mengen aktionsweise durchforsten:   */
    while ( *set1 )
      {
                                  /* Bei der ersten Nicht-Uebereinstimmung   */
        if ( *(set1++) != *(set2++) )
          {
                                  /* wird die weitere Suche abgebrochen und  */
                                  /* 'False' als Ergebnis geliefert.         */
            return ( False ); 
          }
      } /* while */

                                  /* An dieser Stelle wurde die erste Menge  */
                                  /* bis zum Ende abgearbeitet; das Ergebnis */
                                  /* des Vergleichs ist 'True', wenn jetzt   */
                                  /* auch die zweite Menge 'leer' ist, sonst */
                                  /* 'False'.                                */
    return ( ! *set2 );

  } /* SETis_equal() */


/*****************************************************************************/
/* Funktion zur Vereinigung zweier Mengen.                                   */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

DSet SETunion( DSet set1,
               DSet set2 )

#else

DSet SETunion( set1, set2 )

  DSet set1,
       set2;                                /* Berechnet set1 := set1 u set2 */

#endif

  /* GLOBAL DSet setDummy; */         /* setDummy nimmt Zwischenergebnis auf */

  {

    register DSet set1Ptr = set1,
                  set2Ptr = set2,
                  setDum  = setDummy;

                                  /* Solange nicht beide Mengen abgearbeitet */
                                  /* sind:                                   */
    while ( *set1Ptr || *set2Ptr )
      {
                                  /* Falls die aktuell betrachteten Aktionen */
                                  /* aus Menge 1 und 2 uebereinstimmen,      */
        if ( *set1Ptr && (*set1Ptr == *set2Ptr) )
          {
                                  /* wird die Aktion einmal in die Hilfsmenge*/
                                  /* uebernommen.                            */
            *(setDum++) = *(set1Ptr++); set2Ptr++;
          } /* if */
                                  /* Falls der Aktionscode aus Menge 1 klei- */
                                  /* ner ist als der der Aktion aus Menge 2, */
                                  /* wird diejenige aus Mange 1 in die Hilfs-*/
                                  /* menge uebernommen.                      */
        else if ( ( ! *set2Ptr ) || ( *set1Ptr && ( *set1Ptr < *set2Ptr ) ) )
          {
            *(setDum++) = *(set1Ptr++);
          } /* else if */
                                  /* Ansonsten                               */
        else /* ( ( ! *set1Ptr ) || ( *set1Ptr > *set2Ptr ) ) */
          {
                                  /* wird die Aktion aus Menge 2 uebernommen.*/
            *(setDum++) = *(set2Ptr++);
          } /* else */
      } /* while */

                                  /* Hilfsmenge abschliessen.                */
    *setDum = 0;
                                  /* Hilfsmenge nach `set1` kopieren. Der     */
                                  /* alte Inhalt von `set1` wird ueberschrie- */
                                  /* ben!                                    */
    setDum = setDummy;
    set1Ptr = set1;
    while (( *(set1Ptr++) = *(setDum++) ))
      skip;

                                  /* Ergebnismenge zurueckliefern.           */
    return ( set1 );

  } /* SETunion() */


/*****************************************************************************/
/* Funktion zur Durchschnittbildung zweier Mengen.                           */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

DSet SETintersection( DSet set1,
                      DSet set2 )

#else

DSet SETintersection( set1, set2 )

  DSet set1,
       set2;                                /* Berechnet set1 := set1 n set2 */

#endif

  {

    register DSet set1rPtr = set1, /* Lesezeiger    */ 
                  set1wPtr = set1, /* Schreibzeiger */
                  set2Ptr  = set2;

    while ( *set1rPtr )
      {
        if ( *set1rPtr == *set2Ptr )
          {
            *(set1wPtr++) = *(set1rPtr++);
            set2Ptr++;
          } /* if */
        else if ( *set2Ptr && ( *set1rPtr > *set2Ptr ) )
          {
            set2Ptr++;
          } /* else if */
        else /* ( ( ! *set2Ptr ) || ( *set1rPtr < *set2Ptr ) ) */
          {
            set1rPtr++;
          } /* else */
      } /* while */

    *set1wPtr = 0;

    return ( set1 );

  } /* SETintersection() */


/*****************************************************************************/
/* Funktion zur Differenzbildung zweier Mengen.                              */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

DSet SETdifference( DSet set1,
                    DSet set2 )

#else

DSet SETdifference( set1, set2 )

  DSet set1,
       set2;                                /* Berechnet set1 := set1 \ set2 */

#endif

  {

    register DSet set1rPtr = set1, /* Lesezeiger    */ 
                  set1wPtr = set1, /* Schreibzeiger */
                  set2Ptr  = set2;

    while ( *set1rPtr )
      {
        if ( *set1rPtr == *set2Ptr )
          {
            set1rPtr++;
            set2Ptr++;
          } /* if */
        else if ( *set2Ptr && ( *set1rPtr > *set2Ptr ) )
          {
            set2Ptr++;
          } /* else if */
        else /* ( ( ! *set2Ptr ) || ( *set1rPtr < *set2Ptr ) ) */
          {
            *(set1wPtr++) = *(set1rPtr++);
          } /* else */
      } /* while */

    *set1wPtr = 0;

    return ( set1 );

  } /* SETdifference() */


/*****************************************************************************/
/* Funktion zur Ausgabe einer Menge ( Datei oder Bildschirm ).               */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

void SETwrite( FILE * fileptr,
               DSet set )

#else

void SETwrite( fileptr, set )

  FILE * fileptr;
  DSet set;

#endif

  {

    if ( set )
      {
        fputc( '{', fileptr );
        if ( *set )
          {
            fprintf( fileptr, "%hu", *set );
            while ( *++set )
              {
                fprintf( fileptr, ",%hu", *set );
              } /* while */
          } /* if */
        fprintf( fileptr, "}\n" );
      } /* else */

  } /* SETwrite() */


/*****************************************************************************/
/* END OF FILE setop.c                                                       */
/*****************************************************************************/

 
