/*****************************************************************************/
/*  setop.h                                                                  */
/*  C-Headerdatei: Datenstruktor/Operationen auf einfachen Mengen            */
/*                                               Funktionsprototypen/Makros  */
/*  Di., 03.11.1992 - Fr.,10..01.1997                         Stefan Roemer  */
/*****************************************************************************/

#ifndef __SETOP_H__
#define __SETOP_H__


#include <stdio.h>
#include <stdlib.h>


/*****************************************************************************/
/* Globale Variablen fuer die Behandlung von Mengen.                         */
/*****************************************************************************/

#define SET_MAX_ENTRIES 1000

typedef Cardinal SetEntry;

typedef SetEntry * DSet;

extern Cardinal MaxEntries;  /* Maximale Anzahl von Eintraegen               */

extern DSet setDummy;        /* Hilfsmenge bei der Berechnung von Summe und  */
                             /* Vereinigung von Mengen.                      */

extern DSet setSingle;       /* Hilfsmenge zur Aufnahme einer Menge aus nur  */
                             /* einem Element.                               */


/*****************************************************************************/
/* Funktion zum Initialisieren/Allokieren der globalen Variablen.            */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern void SETinitSets( void );

#else

extern void SETinitSets();

#endif


/*****************************************************************************/
/* Funktion zum Einrichten einer neuen Menge.                                */
/*****************************************************************************/

                        /* Funktionalitaet:  DSet SETalloc( Cardinal n ) */
#define SETalloc( n ) (DSet) MYmalloc( ( (n) + 1 ) * sizeof( SetEntry ) )


/*****************************************************************************/
/* Funktion ( Makro ) zum Verlaengern einer Menge.                           */
/*****************************************************************************/

                      /* Funktionalitaet:  DSet SETresize( DSet     mm;      */
                      /*                                   Cardinal n   )    */
#define SETresize( mm, n ) ( mm = (DSet) Realloc( (char *) mm, \
                                           ( (n) + 1 ) * sizeof( SetEntry ) ) )


/*****************************************************************************/
/* Funktion ( Makro ) zum Freigeben des von der Menge `mm` im Hauptspeicher  */
/* belegten Speicherplatzes.                                                 */
/*****************************************************************************/

                           /* Funktionalitaet:  int SETremove( DSet mm )     */
                           /* Hier bewusst auf Indirektion `DSet * mm`       */
                           /* verzichtet! Nach Ausfuehrung dieser Funktion   */
                           /* ist also `mm` undefiniert !!!                  */
#define SETremove( mm ) Free( mm )


/*****************************************************************************/
/* Funktion zum Kopieren einer Menge; kopiert `mm1 := mm2`.                  */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern DSet SETcopy( DSet mm1,
                     DSet mm2  );

#else

extern DSet SETcopy( /* DSet mm1,
                             mm2  */ );

#endif


/*****************************************************************************/
/* Funktion zur Initialisierung einer leeren Menge `{}`.                     */
/* Rueckgabe ist ein Zeiger auf einen mit der leeren Menge initialisierten   */
/* Speicherbereich der "Laenge" 0.                                           */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern DSet SETcreate_empty_set( void );

#else

extern DSet SETcreate_empty_set();

#endif


/*****************************************************************************/
/* Funktion zur Initialisierung einer neuen Menge, die als einziges Element  */
/* `akt` enthaelt.                                                           */
/* Rueckgabe: Zeiger auf den so initialisierten Speicherbereich der          */
/*            "Laenge" 1.                                                    */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern DSet SETcreate_singleton( SetEntry akt );

#else

extern DSet SETcreate_singleton( /* SetEntry akt */ );

#endif


/*****************************************************************************/
/* Funktion ( als Makro ), die eine bestehende Menge `mm` derart aendert,    */
/* dass diese nach dem Funktionsaufruf die Menge `mm := {akt}` darstellt.    */
/* `mm` muss also auf eine mindestens einelementige Menge zeigen.            */
/* Rueckgabe ist ein Zeiger auf die so manipulierte Menge.                   */
/*****************************************************************************/

                     /* Funktionalitaet:  DSet SETset_single( DSet mm;       */
                     /*                                       SetEntry akt;  */
                     /*                                       Cardinal num ) */
#define SETset_single( mm, akt, num ) ( *(mm) = (akt), \
                                        *((mm)+1) = 0, \
                                        mm \
                                      )


/*****************************************************************************/
/* Funktion zur Initialisierung einer neuen Menge als Kopie einer bereits    */
/* vorhandenen Menge `mm`; der Speicherbedarf der neu allokierten Menge      */
/* entspricht ihrer tatsaechlichen "Laenge".                                 */
/* Ein Zeiger auf diese neu eingerichtete Menge wird zurueckgegeben.         */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern DSet SETcreate_copy( DSet mm );

#else

extern DSet SETcreate_copy( /* DSet mm */ );

#endif


/*****************************************************************************/
/* Funktion zur Berechnung der Maechtigkeit einer Menge `mm`: `|mm|`.        */
/* Rueckgabe ist die so ermittelte Kardinalitaet ( als Cardinal ).           */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern Cardinal SETcardinality( DSet mm );

#else

extern Cardinal SETcardinality( /* DSet mm */ );

#endif


/*****************************************************************************/
/* Funktion zur Feststellung, ob in der Menge `mm` das Element 'akt' ent-    */
/* ten ist.                                                                  */
/* Rueckgabe: Boole'scher Wahrheitswert ( True bei Vorkommen des Elements ). */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern bool SETis_in( SetEntry akt,
                         DSet     mm );

#else

extern bool SETis_in( /* SetEntry akt;
                            DSet     mm  */ );

#endif


/*****************************************************************************/
/* Funktion zur Feststellung, ob zwei Mengen `mm1`, `mm2` gleich sind.       */
/* Rueckgabe: True,  falls `mm1 == mm2`, d.h. `mm1` enthaelt genau die glei- */
/*                   chen Elemente wie `mm2`.                                */
/*            False, falls `mm1 != mm2`.                                     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern bool SETis_equal( DSet mm1,
                            DSet mm2  );

#else

extern bool SETis_equal( /* DSet mm1,
                                    mm2  */ );

#endif


/*****************************************************************************/
/* Funktion zur Vereinigung zweier Mengen `mm1` und `mm2`; es wird           */
/* `mm1 := mm1 u mm2` berechnet, d.h. die zweite Menge wird zu der ersten    */
/* "hinzuvereinigt". Rueckgabe ist ein Zeiger auf die Ergebnismenge; `mm1`   */
/* ist somit nach dieser Operation zerstoert und muesste, falls gewuenscht,  */
/* vorher explizit in eine Hilfsmenge kopiert werden ( mit Funktion `SETcopy`*/
/* oder `SETcreate_copy` ). Achtung! Die Menge `mm1` muss gross genug zur    */
/* Aufnahme des Ergebnisses sein!!!                                          */
/* Seiteneffekt: Benutzung der globalen Variablen `setDummy`.                */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern DSet SETunion( DSet mm1,
                      DSet mm2  );

#else

extern DSet SETunion( /* DSet mm1,
                              mm2  */ );

#endif

  /* GLOBAL DSet setDummy; (schreibend) */


/*****************************************************************************/
/* Funktion zur Durchschnittbildung zweier Mengen `mm1` und `mm2`; es        */
/* wird `mm1 := mm1 n mm2` berechnet.                                        */
/* Rueckgabe: Zeiger auf die Schnittmenge. Der alte Inhalt von `mm1` wird    */
/*            ueberschrieben, also gegebenenfalls `mm1` vorher kopieren!     */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern DSet SETintersection( DSet mm1,
                             DSet mm2 );

#else

extern DSet SETintersection( /* DSet mm1,
                                     mm2  */ );

#endif


/*****************************************************************************/
/* Funktion zur Differenzbildung zweier Mengen `mm1` und `mm2`; es wird      */
/* `mm1 := mm1 \ mm2` berechnet, `mm2` wird also von `mm1` "abgezogen".      */
/* Rueckgabe: Zeiger auf die Ergebnismenge `mm1`, deren alter Inhalt somit   */
/*            ueberschrieben ist ( evtl. retten durch Kopieren! ).           */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern DSet SETdifference( DSet mm1,
                           DSet mm2  );

#else

extern DSet SETdifference( /* DSet mm1,
                                   mm2  */ );

#endif


/*****************************************************************************/
/* Funktion zum Schreiben einer Menge `mm` in eine Datei `fileptr`.          */
/*****************************************************************************/

#if defined(__STDC__) || defined(__cplusplus)

extern void SETWriteSet( FILE * fileptr,
                         DSet mm         );

#else

extern void SETWriteSet( /* FILE * fileptr;
                            DSet mm        */ );

#endif


/*****************************************************************************/
/* Funktion (Makro) zur Darstellung einer Menge `mm` auf dem Bildschirm.     */
/*****************************************************************************/

                               /* Funktionalitaet:  void SETprint( DSet mm ) */
#define SETprint( _MM_ ) SETwrite( stdout, _MM_ );


/*****************************************************************************/


#endif /* __SETOP_H__ */


/*****************************************************************************/
/* END OF FILE setop.h                                                       */
/*****************************************************************************/


/*****************************************************************************/

