/*****************************************************************************/
/* FILE pep2spin.c                                                           */
/*****************************************************************************/
/* Konvertierungsprogramm zur Umwandlung von Petri-Netzen aus PEP in das     */
/* Dateiformat des SPIN Programms                                            */
/*                                    Carola Pohl, last update: 17.09.97     */
/*****************************************************************************/


/*****************************************************************************/
/*                                                                           */
/*                             header files                                  */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "neunetze.h"
#include "convertPEP.h"

/*****************************************************************************/
/**************************** usage_err() ************************************/
/*****************************************************************************/
/* prints the message *msg and information about the usage of pep2spin to the*/
/* standard output and halts the program returning an error exit status      */
/*****************************************************************************/

#if defined (__STDC__) || defined (__cplusplus)

void usage_err( char * msg )

#else

void usage_err( msg )

char * msg;

#endif

{

	printf( "\n%s\n\n", msg );
	printf( "Usage: \n" );
	printf( "\t\t pep2spin [-s] [path]PEP_file [path]SPIN_file \n\n");
	printf( "where PEP_file is a file containing a net in the PEP format ");
	printf( "and SPIN_file\n" );
	printf( "is the target file. Path may be a valid pathname.\n" );
	printf( "\n\n" );

	exit(1);

} /* usage_err() */


/*****************************************************************************/
/************************* is_dos_ch() ***************************************/
/*****************************************************************************/
/* returns True if ch is a valid character for a DOS filename, else False    */
/*****************************************************************************/

#if defined (__STDC__) || defined (__cplusplus)

bool is_dos_ch( char ch )

#else

bool is_dos_ch( ch )

char ch;

#endif

{

	if ( isalnum( ch ) || strchr( "-_$%(){#&@!}\'`^", ch ) )
	{
		return ( True );
	} /* if */
	else
	{
		return ( False );
	} /* else */

} /* is_dos_ch() */


/*****************************************************************************/
/************************ is_spin_pt_name() **********************************/
/*****************************************************************************/
/* returns True, if *name is a valid SPIN st-net filename, else False        */
/*****************************************************************************/

#if defined (__STDC__) || defined (__cplusplus)

bool is_spin_pt_name( char * name )

#else

bool is_spin_pt_name( name )

char * name;

#endif

{

	int i = 0;

	if ( isdigit( *name ) )
	{
		return ( False );
	} /* if */

	while ( is_dos_ch( *name ) ) {
		i++;
		name++;
	} /* while */

	if ( ( i > 8 ) || ( i == 0 ) ) {
		return ( False );
	} /* if */

	if ( strcmp( name, ".promela" ) ) {
		return ( False );
	} /* if */

	return ( True );
} /* is_spin_pt_name() */



/*****************************************************************************/
/************************** write_SPIN() *************************************/
/*****************************************************************************/
/* writes the net *net represented in the PEP format into a file called      */
/* *filename in the SPIN representation                                      */
/*****************************************************************************/
/* Caution: If no valid DOS filename is found behind an optional path in     */
/* *filename, the net name printed into the SPIN file may be void!           */
/*****************************************************************************/

#if defined (__STDC__) || defined (__cplusplus)

void write_SPIN( char * filename, LMPNet net, int safs )

#else

void write_SPIN( filename, net, safs )

char * filename;
LMPNet net;
int safs;

#endif

{
        bool pre_pl = 0;
	char *first_pl;
        int  first_pl_weight;

	int        i = 0;
        int        line = 1;
	int        anz = 0;
	FILE       *file;
        FILE       *file_tr;
	char       net_name[9];
	char       net_name_long[65];
	char *     curr_ch;
	char *     old_curr_ch;
	char *     file_tr_name;
	Place      curr_pl;
	Transition curr_tr;
	TSEdge     curr_ts;
	STEdge     curr_st;
	
	num_net_nodes( net );
	if (( curr_ch = strrchr( filename, '/' ) )) {
		curr_ch++;
	} /* if */
	else {
		curr_ch = filename;
	} /* else */

	old_curr_ch = curr_ch;
	while ( is_dos_ch( *curr_ch ) && ( i < 8 ) ) {
		/* fehler in spin bezeichnern sind keine {}| erlaubt */
		net_name[i++] = *(curr_ch++);
	} /* while */

	net_name[i] = '\0';

	i = 0;
	while ( old_curr_ch[i] && i < 64 && old_curr_ch[i] != '.') {
		net_name_long[i] = old_curr_ch[i];
		i++;
	} /* while */
	net_name_long[i] = '\0';

	if ( ! ( file = fopen( filename, "w" ) ) ) {
		usage_err( "Cannot open SPIN_file." );
	} /* if */

	file_tr_name = calloc(strlen(filename)+12,1);
	i = 0;
	while ( filename[i] != '.') {
		file_tr_name[i] = filename[i];
		i++;
	}
	file_tr_name[i] = '\0';
	strcat(file_tr_name,".transition");
	file_tr = fopen(file_tr_name,"w");


	/******************** write spin program *****************************/

        curr_pl=net->Places;

	if (!safs) {
          fprintf(file,"/* NO 1-safe */\n");line++;
	} else {
          fprintf(file,"/* 1-safe */\n");line++;
        }
 
	while (curr_pl) {
	  
	          if (!safs) {
	            fprintf(file,"byte %s",curr_pl->name);
		  }
		  else {
		    fprintf(file,"bool %s",curr_pl->name);
		  }

		  if (curr_pl->marks) {
		    
		    fprintf(file,"=%d;\n",curr_pl->marks);line++;
		  }
		  else {
		    fprintf(file,"=0;\n");line++;
		  }
		  curr_pl=curr_pl->nextplace;

	} /* while */

        fprintf(file,"bool DEADLOCK=0;\n\n");line++;line++;
        fprintf(file,"proctype Netz()\n{\n");line++;line++;

        curr_tr=net->Transitions;

	if (curr_tr) {

	  fprintf(file,"\tdo\n");line++;
	  
	  while (curr_tr) {

	    fprintf(file,"\t::atomic { ");

           /*** In(curr_tr) ***/
	    curr_st=curr_tr->preset;

	    if (curr_st) {
	      pre_pl=1;
              anz=1;
	      first_pl=curr_st->place->name;
              first_pl_weight=curr_st->weight;
	      curr_st=curr_st->nextplace;
	    }

	    if (curr_st) { /* more then one preplace */

	      if (!safs) {
	        fprintf(file,"((%s>=%d)",first_pl,curr_st->weight);
	      }
	      else {
		fprintf(file,"(%s",first_pl);
	      }
	      while (curr_st) {
		if (!safs) {
		fprintf(file,"&&(%s>=%d)",curr_st->place->name,curr_st->weight);
		}
		else {
		  fprintf(file,"&&%s",curr_st->place->name);
		}
		curr_st=curr_st->nextplace;
	      }
	      fprintf(file,")");
	    }
	    else { /* only one preplace */
	      if (!safs) {
	        fprintf(file,"(%s>=%d)",first_pl,first_pl_weight);
	      }
	      else {
		fprintf(file,"%s",first_pl);
	      }
	    }
	   /********************/

	    if (pre_pl) {
	      fprintf(file," -> ");pre_pl=0;
	    }
	    
           /*** Clear(curr_tr) ***/
	    curr_st=curr_tr->preset;
	    while (curr_st) {
	      if (!safs) {
	        fprintf(file,"%s=%s-%d;",curr_st->place->name,curr_st->place->name,curr_st->weight);
	      }
	      else {
		fprintf(file,"%s=0;",curr_st->place->name);
	      }
	      anz++;
	      curr_st=curr_st->nextplace;
	    }
	    fprintf(file,"\n\t\t\t");

	   /********************/

	   /*** Set_Out(curr_tr)***/
	    curr_ts=curr_tr->postset;
	    if (curr_ts) {
	      while (curr_ts) {
		if (!safs) {
		  fprintf(file,"%s=%s+%d;",curr_ts->place->name,curr_ts->place->name,curr_ts->weight);
		}
		else {
		  fprintf(file,"%s=1;",curr_ts->place->name);
		}
	        anz++;
		curr_ts=curr_ts->nextplace;
	      }
	    }
	    else {
	      fprintf(file,"skip");
	    }

	   /********************/ 

	    fprintf(file," }\n");
	    fprintf(file_tr,"%d %s %d\n",line,curr_tr->name,anz);
	    line = line+2; 
	    anz = 0;
	    curr_tr=curr_tr->nexttrans;

	  } /*while*/

          fprintf(file,"\t::else -> goto dead\n");
	  fprintf(file,"\tod;\n");
	  fprintf(file,"dead: DEADLOCK=1\n}\n");

       } /*if*/

       fprintf(file,"\ninit{run Netz()}");
       fclose(file_tr);

} /* write_SPIN() */


#if defined (__STDC__) || defined (__cplusplus)

static void parse_args(int argc, char **argv, char **ll_np, char **spin_np,
		       int *sap)

#else

static void parse_args(argc, argv, ll_np, spin_np, sap)

int argc;
char **argv;
char **ll_np;
char **spin_np;
int *sap;

#endif

{
	int i;
	char *name,*d, *e;

	for (i = 1; i < argc; i++) {
		if (strcmp(argv[i], "-s")==0) {
			*sap = 1;
			continue;
		}
		if (argv[i][0] == '-') {
			usage_err( "unknown option!" );
		}
		break;
	}
	if (i == argc) {
		usage_err( "name required!" );
	}
	if ((d = strchr(argv[i], '.'))) {
		*ll_np = argv[i];
	} else {
		name = calloc(strlen(argv[i]) + 8,1);
		strcpy(name, argv[i]);
		strcat(name, ".ll_net");
		*ll_np = name;
	}
	if (i+1 < argc) {
		if ((e = strchr(argv[i+1], '.'))) {
			*spin_np = argv[i+1];
		} else {
			name = calloc(strlen(argv[i+1]) + 9,1);
			strcpy(name, argv[i+1]);
			strcat(name, ".promela");
			*spin_np = name;
		}
	} else {
		if (d) {
			name = calloc((d - argv[i]) + 9,1);
			strncpy(name, argv[i], d - argv[i]);
			name[d - argv[i]] = '\0';
			strcat(name, ".promela");
			*spin_np = name;
		} else {
			name = calloc(strlen(argv[i]) + 9,1);
			strcpy(name, argv[i]);
			strcat(name, ".promela");
			*spin_np = name;
		}
	}
}


/*****************************************************************************/
/*************************** main ********************************************/
/*****************************************************************************/
/* creates a file called *arg_list[2] containing the st-net *arg_list[1] as a*/
/* PEP file in the SPIN format if exactly 2 valid parameters are given, else */
/* a message about correct usage is printed to the standard output           */
/*****************************************************************************/

#if defined (__STDC__) || defined (__cplusplus)

int main( int arg_num, char * arg_list[] )

#else

int main( arg_num, arg_list )

int arg_num;
char * arg_list[];

#endif

{

	char * spin_name = NULL;
	char * ll_name = NULL;
	int safe = 0;
	FILE * file;
	LMPNet net;

	parse_args( arg_num, arg_list, &ll_name, &spin_name, &safe);
	
	if (( file = fopen( ll_name, "r" ) )) {
		fclose( file );
	} /* if */
	else {
		usage_err( "PEP_file does not exist. Possibly wrong path." );
	} /* else */

	/* warum nicht stat() ? */
	/*
    if ( file = fopen( spin_name, "r" ) ) {
        fclose( file );
        usage_err( "SPIN_file already exists." );
		} 
	*/

	if (( file = fopen( spin_name, "w" ) )) {
		fclose( file );
		remove( spin_name );
	} /* if */
	else {
		usage_err( "Cannot create SPIN_file. Possibly wrong path." );
	} /* else */


	net = read_PEP( ll_name );

	if ( ! net ) {
		usage_err( "First parameter is not a valid PEP net." );
	} /* if */


	write_SPIN( spin_name, net, safe );


	remove_net( &net );

	exit(0);

} /* main() */


/*****************************************************************************/
/* END OF FILE pep2spin.c                                                    */
/*****************************************************************************/
