/* pep2tex.cpp */
/* Ez a file tartalmazza a pep2tex program main() fuggvenyet */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include <conio.h>
#include "pep_cons.h"
#include "pep_def.h"
#include "pep_bl.h"
#include "pep_arc.h"
#include "pep_tx.h"
#include "pep_list.h"
#include "pep_tree.h"


typedef struct {
		 ticoord DCBlockName,
			 DCBlockMean,
			 DCPlaceName,                //Alap ertekek
			 DCPlaceMean,
			 DCTransName,
			 DCTransMean,
			 DCArcWeight;
		 int DArcWeight;
		} tdefaults, *ptdefaults;

typedef struct {
		 boolean BName,
			 BMean,
			 Bezier,                 //Parancssorban beallithato
			 AInvisIn,               // opciok
			 PhTRInvis,
			 SyncTRInvis,
			 ExplAllBL,
			 Independ;
		 int     Magnif;
	       } toption, *ptoption;

	//Filekezelessel kapcsolatos segedfuggvenyek deklaracioi
FILE *OpenInput ( char *InName);
FILE *OpenOutput(  char *InName, char *OutName);
void ReadHeader( FILE *InF);
int ReadDefault( FILE *InF, tphase *StartPh, ptdefaults DF);

       // A kep befoglalo meretet szamito fgv
		// visszaadja, hogy el kell-e forditani a kepet
		// MaxX, MaxY a kep maximalis koordinatai keppontokban
		// Fact a nagyitas/kicsinyites merteke
boolean PictSize(int MaxX, int MaxY, int *FactInd);

		// A parancssorban adott argumentumokat ertelmezo fgv)
		// visszateresi erteke az input file nevet tartalmazo
		// argumentum indexe
int ProcArg(int argc, char *argv[], ptoption Opt);

int main (int argc, char *argv[])
 {
	//Definiciok
   FILE *InpFile, *OutFile;
   char NEOF;   //Nem file vege
   int InpFileInd; // Az argumentum lista input file nevet tartalmazo
		       // elemenek indexe

   tline ActLine;
   tphase ActPhase, StartPhase;
   boxtypes ActBoxType;
   arctypes ActArcType;

   tbox ActBox;		//Aktualis strukturak
   tarc ActArc;
   ttext ActText;

   boolean ArcsStart, PhArcsStart;

   int CountBL=0, CountPL=0, CountTR=0, CountPTR=0;
			// A kulonbozo box-ok szamlaloi az automatikus szamozashoz

   ptlist ListBL, ListPL, ListTR, ListPTR;

   blocktree BlockTree;

			// Opciok alapertelmezese
   toption Options = { TRUE, TRUE, TRUE, FALSE, FALSE, FALSE, FALSE, TRUE, 2 };

   tdefaults Defaults = {{-BLOCKSIZE/4,-BLOCKSIZE/4+NAMEHEIGHT/2},{0,0},
			{-PLACESIZE/4,-PLACESIZE/4+NAMEHEIGHT/2},{0,0},
			{-TRANSSIZE/4,-TRANSSIZE/4+NAMEHEIGHT/2},{0,0},
			{0,0},1};

   int NoDefLine;     // A default sorok szama

   int GlobMaxX=0, GlobMaxY=0, LocMaxX=0, LocMaxY=0;
				//Globalis es lokalis maximalis koordinatak

   int Magn; //Az abra nagyitas/kicsinyites merteke
		// Factors vektor indexe (a pep_def.h-ban lett definialva)
   fontind Fonts;   // Az abran hasznalt betumeretek indexei
		  // a FontSizes vektorban (a pep_def.h-ban lett definialva)
   boolean Landscape; // Fekvo lap
   short i;

//----------------------
	//Kod
			// Listak inicializalasa
   ListBL  = InitList();
   ListPL  = InitList();
   ListTR  = InitList();
   ListPTR = InitList();
		// Blokk-fa inicializalasa
   InitBlockTree(BlockTree);

			 // Argumentumlista elemzese
  InpFileInd = ProcArg(argc,  argv,  &Options);

			 // Filemegnyitas
  InpFile = OpenInput(argv[InpFileInd]);
  OutFile = OpenOutput(argv[InpFileInd], argv[InpFileInd+1]);

			 // Input file elejenek olvasasa
  ReadHeader(InpFile);
  NoDefLine = ReadDefault(InpFile, &ActPhase, &Defaults);
  StartPhase = ActPhase; // A kezdoallapot tarolasa a masodik olvasashoz

  NEOF = *fgets(ActLine,MAXLINE, InpFile);  //Eloreolvas az input filebol
   while (NEOF)                      // Input file elso vegigolvasasa
    {				// a befoglalo meretek meghatarozasa celjabol
        switch (ActPhase)
	{
	 case PhBL: ActBoxType = BL;
		    InitBox(&ActBox, ActBoxType, &(Defaults.DCBlockName),
					&(Defaults.DCBlockMean), CountBL);
		    ActPhase = ProcLineBox(&ActBox, ActLine);
		    if (IsValidBox(&ActBox))
		     {
		      MaxBoxCoord(&ActBox, &LocMaxX, &LocMaxY);
		      StoreBoxToTree(&ActBox, BlockTree);
		     }
		    break;
	 case PhPL: ActBoxType = PL;
		    InitBox(&ActBox, ActBoxType, &(Defaults.DCPlaceName),
					&(Defaults.DCPlaceMean), CountPL);
		    ActPhase = ProcLineBox(&ActBox, ActLine);
		    if (IsValidBox(&ActBox))
		     {
		      MaxBoxCoord(&ActBox, &LocMaxX, &LocMaxY);
		     }
		    break;
	 case PhTR: ActBoxType = TR;
		    InitBox(&ActBox, ActBoxType, &(Defaults.DCTransName),
					&(Defaults.DCTransMean), CountTR);
		    ActPhase = ProcLineBox(&ActBox, ActLine);
		    if (IsValidBox(&ActBox))
		     {
		      MaxBoxCoord(&ActBox, &LocMaxX, &LocMaxY);
		     }
		    break;
	 case PhPTR: ActBoxType = PTR;
		     InitBox(&ActBox, ActBoxType, &(Defaults.DCTransName),
					&(Defaults.DCTransMean), CountPTR);
		     ActPhase = ProcLineBox(&ActBox, ActLine);
		     if (IsValidBox(&ActBox))
		     {
		      MaxBoxCoord(&ActBox, &LocMaxX, &LocMaxY);
		     }
		     break;
	 case PhTP: ActArcType = TP;
		    InitArc(&ActArc, ActArcType, Defaults.DArcWeight,
						&(Defaults.DCArcWeight));
		    ActPhase = ProcLineArc(&ActArc, ActLine);
		    if (IsValidArc(&ActArc))
		     {
		      MaxArcCoord(&ActArc, &LocMaxX, &LocMaxY);
		     }
		    break;
	 case PhPT: ActArcType = PT;
		    InitArc(&ActArc, ActArcType, Defaults.DArcWeight,
						&(Defaults.DCArcWeight));
		    ActPhase = ProcLineArc(&ActArc, ActLine);
		    if (IsValidArc(&ActArc))
		     {
		      MaxArcCoord(&ActArc, &LocMaxX, &LocMaxY);
		     }
		    break;
	 case PhPTP: ActArcType = PTP;
		     InitArc(&ActArc, ActArcType, Defaults.DArcWeight,
						&(Defaults.DCArcWeight));
		     ActPhase = ProcLineArc(&ActArc, ActLine);
		     if (IsValidArc(&ActArc))
		     {
		      MaxArcCoord(&ActArc, &LocMaxX, &LocMaxY);
		     }
		     break;
	 case PhPPT: ActArcType = PPT;
		     InitArc(&ActArc, ActArcType, Defaults.DArcWeight,
						&(Defaults.DCArcWeight));
		     ActPhase = ProcLineArc(&ActArc, ActLine);
		     if (IsValidArc(&ActArc))
		     {
		      MaxArcCoord(&ActArc, &LocMaxX, &LocMaxY);
		     }
		     break;
	 case PhTX:
		     ActPhase = ProcLineText(&ActText, ActLine);
		     if (IsValidText(&ActText))
		     {
		      MaxTextCoord(&ActText, &LocMaxX, &LocMaxY);
		     }
		     break;
	} //end switch

      GlobMaxX = max(GlobMaxX, LocMaxX);
      GlobMaxY = max(GlobMaxY, LocMaxY);

      fgets(ActLine,MAXLINE, InpFile);  //Olvas az input filebol
      NEOF = !feof(InpFile);
    }//end while


		   // Ha nincs megadva opcoval, hogy az osszes
		   //blokk ki legyen bontva,
      if (!Options.ExplAllBL)
       {
	FindRelatives(BlockTree);
	AskExplBl(BlockTree); // akkor rakerdez
       }
		    // Ha minden blokkot ki kell bontani, akkor
		    // nem kell semmit csinalni, mert ugy vannak inicializalva
		    // a blokkok

     // A kep meretenek meghatarozasa
  Landscape = PictSize(GlobMaxX, GlobMaxY, &Magn);

  if (Options.Magnif != 2)  // Ha a felhasznalo eloirt nagyitast
   Magn = Options.Magnif;

     // Fontok indexeinek beallitasa
  switch (Magn)
   {
    case 0:
	   Fonts[0] =  0;
	   Fonts[1] =  1;
	   Fonts[2] =  3;
	   break;
    case 1:
	   Fonts[0] =  1;
	   Fonts[1] =  2;
	   Fonts[2] =  4;
	   break;
    case 2:
	   Fonts[0] =  2;
	   Fonts[1] =  3;
	   Fonts[2] =  5;
	   break;
    case 3:
	   Fonts[0] =  3;
	   Fonts[1] =  4;
	   Fonts[2] =  6;
	   break;
    case 4:
	   Fonts[0] =  4;
	   Fonts[1] =  5;
	   Fonts[2] =  7;
	   break;

   }


     // A lapmeret kiirasa az output fileba
  if (Landscape)
   {
    fprintf(OutFile, "\\hsize=300mm\n\\vsize=200mm\n");
    fprintf(OutFile, "\\typeout{***************************}\n");
    fprintf(OutFile, "\\typeout{A \\thepage . oldal fekvo }\n");
    fprintf(OutFile, "\\typeout{***************************}\n");
   }
  else
   fprintf(OutFile, "\\hsize=200mm\n\\vsize=300mm\n");

  fprintf(OutFile, "\\unitlength=%5.3fmm\n",0.33*Factors[Magn]);

  fprintf(OutFile, "\\begin{picture}(%d, %d)\n",GlobMaxX, GlobMaxY);

  //------------- Masodik olvasas ---------------

  fseek(InpFile, 0L, SEEK_SET); // File elejere allitja a mutatot
  for (i=0; i<NoDefLine+4; i++)
     NEOF = *fgets(ActLine,MAXLINE, InpFile); // az elso sorok atolvasasa

  ActPhase = StartPhase; // Beallitja a kezdoallapotot, (elso olvasas eredmenye)
  NEOF = *fgets(ActLine,MAXLINE, InpFile);  //Eloreolvas az input filebol
   while (NEOF)                      // Input file masodik vegigolvasasa
    {					// tenyleges feldolgozas celjabol
      switch (ActPhase)
	{
	 case PhBL: ActBoxType = BL;
		    CountBL++;
		    InitBox(&ActBox, ActBoxType, &(Defaults.DCBlockName),
					&(Defaults.DCBlockMean), CountBL);
		    ActPhase = ProcLineBox(&ActBox, ActLine);
		    if (IsValidBox(&ActBox))
		     {
		      GetBLTreeInfo(&ActBox, BlockTree);
		      ConvertBox(&ActBox, ListBL, GlobMaxY);
		      StoreBoxToList(&ActBox, ListBL );
		      DrawBox(&ActBox, OutFile, Fonts, Options.BName,
							Options.BMean);
		     }
		    break;
	 case PhPL: ActBoxType = PL;
		    CountPL++;
		    InitBox(&ActBox, ActBoxType, &(Defaults.DCPlaceName),
					&(Defaults.DCPlaceMean), CountPL);
		    ActPhase = ProcLineBox(&ActBox, ActLine);
		    if (IsValidBox(&ActBox))
		     {
		      ConvertBox(&ActBox, ListBL, GlobMaxY);
		      StoreBoxToList(&ActBox, ListPL);
		      DrawBox(&ActBox, OutFile, Fonts, Options. BName,
							Options.BMean);
		     }
		    break;
	 case PhTR: ActBoxType = TR;
		    CountTR++;
		    InitBox(&ActBox, ActBoxType, &(Defaults.DCTransName),
					&(Defaults.DCTransMean), CountTR);
		    ActPhase = ProcLineBox(&ActBox, ActLine);
		    if (IsValidBox(&ActBox))
		     {
		      if (IsSyncBox(&ActBox) && Options.SyncTRInvis)
		       {
			 SetForbid(&ActBox, TRUE);
		       }
		      ConvertBox(&ActBox, ListBL, GlobMaxY);
		      StoreBoxToList(&ActBox, ListTR);
		      DrawBox(&ActBox, OutFile, Fonts, Options.BName,
							Options.BMean);
		     }
		    break;
	 case PhPTR: ActBoxType = PTR;
		     CountPTR++;
		     InitBox(&ActBox, ActBoxType, &(Defaults.DCTransName),
					&(Defaults.DCTransMean), CountPTR);
		     ActPhase = ProcLineBox(&ActBox, ActLine);
		     if (IsValidBox(&ActBox))
		     {
		      if (Options.PhTRInvis)
		       {
			SetForbid(&ActBox, TRUE);
		       }
		      ConvertBox(&ActBox, ListBL, GlobMaxY);
		      StoreBoxToList(&ActBox, ListPTR);
		      DrawBox(&ActBox, OutFile, Fonts, Options.BName,
							Options.BMean);
		     }
		     break;
	 case PhTP: ActArcType = TP;
		    if (!ArcsStart)
		     {
		      fprintf(OutFile, "\\special{em:linewidth 0.4pt} \\linethickness{0.4pt}\n");
		      ArcsStart = TRUE;
		     }
		    InitArc(&ActArc, ActArcType, Defaults.DArcWeight,
						&(Defaults.DCArcWeight));
		    ActPhase = ProcLineArc(&ActArc, ActLine);
		    if (IsValidArc(&ActArc))
		     {
		      ConvertArc(&ActArc, GlobMaxY, ListTR, ListPL, ListBL);
		      DrawArc(&ActArc, OutFile, Fonts, Options.Bezier,
							Options.AInvisIn);
		     }
		    break;
	 case PhPT: ActArcType = PT;
		    InitArc(&ActArc, ActArcType, Defaults.DArcWeight,
						&(Defaults.DCArcWeight));
		    ActPhase = ProcLineArc(&ActArc, ActLine);
		    if (IsValidArc(&ActArc))
		     {
		      ConvertArc(&ActArc, GlobMaxY, ListPL, ListTR, ListBL);
		      DrawArc(&ActArc, OutFile, Fonts, Options.Bezier,
							Options.AInvisIn);
		     }
		    break;
	 case PhPTP: ActArcType = PTP;
		     if (!PhArcsStart)
			  {
				fprintf(OutFile, "\\special{em:linewidth 0.1pt} \\linethickness{0.1pt}\n");
				PhArcsStart = TRUE;
			  }
		     InitArc(&ActArc, ActArcType, Defaults.DArcWeight,
						&(Defaults.DCArcWeight));
		     ActPhase = ProcLineArc(&ActArc, ActLine);
		     if (IsValidArc(&ActArc))
		     {
		      ConvertArc(&ActArc, GlobMaxY, ListPTR, ListPL, ListBL);
		      DrawArc(&ActArc, OutFile, Fonts, Options.Bezier,
							Options.AInvisIn);
		     }
		     break;
	 case PhPPT: ActArcType = PPT;
		     InitArc(&ActArc, ActArcType, Defaults.DArcWeight,
						&(Defaults.DCArcWeight));
		     ActPhase = ProcLineArc(&ActArc, ActLine);
		     if (IsValidArc(&ActArc))
		     {
		      ConvertArc(&ActArc, GlobMaxY, ListPL, ListPTR, ListBL);
		      DrawArc(&ActArc, OutFile, Fonts, Options.Bezier,
							Options.AInvisIn);
		     }
		     break;
	 case PhTX:
		     InitText(&ActText);
		     ActPhase = ProcLineText(&ActText, ActLine);
		     if (IsValidText(&ActText))
		     {
		      ConvertText(&ActText, GlobMaxY);
		      DrawText(&ActText, OutFile, Fonts);
		     }
		     break;
	} //end switch


      fflush(OutFile);
      fgets(ActLine,MAXLINE, InpFile);  //Olvas az input filebol
      NEOF = !feof(InpFile);
    }//end while
				   // File lezarasok
    fprintf(OutFile, "\\end{picture}\n");
    fprintf(OutFile, "\\end{document}\n");
    fclose(OutFile);
    fclose(InpFile);
    printf("Conversion is done.\n");
 }

FILE *OpenInput( char *InName)
  {
   char InN[MAXSTRING];
   FILE *InF;

   strcpy(InN, InName);
   if ((/*FirstOcc = */strchr(InN, '.')) == NULL)   //Ha nincs '.' a megadott
    strcat(InN, ".lln");			//input file neveben, kiegesziti ".lln"-nel

   if ((InF = fopen( InName, "r")) == NULL)
    if ((InF = fopen( InN, "r")) == NULL)
     {
      printf("Input file does not exist\n");
      exit(1);
     }
    else
     {
      printf("Input file: %s\n",InN);
     }
   else
    {
      printf("Input file: %s\n", InName);
    }

   return InF;
  }  // end OpenInput


FILE *OpenOutput( char *InName, char *OutName)
  {
     FILE *OutF;
   //  char FirstOcc; // A '.' karakter elso elofordulasat jelzi az InName-ben
     char OutN[MAXSTRING];  // Output file nevet tarolja
     int NameBaseLen;           // Input file nevenek hossza,
				//ill. kiterjesztes nelkuli hossza

     if (OutName != NULL) //Adott-e az output file neve (igen, ha a feltetel igaz)
      {
       strcpy(OutN, OutName); //ha adott az output file neve
      }
     else
      {                       // ha nem adott az output file neve
	if ((/*FirstOcc =*/ strchr(InName, '.')) == NULL)
	 {
	   strcpy(OutN,InName); //ha a megadott output filenak nincs kiterjesztese
	 }
	else
	 {                  // ha a megadott output filenak van kiterjesztese
	    for(NameBaseLen=0; (InName[NameBaseLen] != '.'); NameBaseLen++);
				//megkeresi a '.'-ot a filenevben
	    strncpy(OutN,InName,NameBaseLen);    // '.' elotti reszt OutN-be masolja
	    OutN[NameBaseLen] = '\0';           // lezaro karakter
	 }// end-else

	strcat(OutN, ".tex");  // kiterjesztes konkatenalasa a filenevhez
       } //end-else

	if ((OutF = fopen (OutN, "w")) == NULL) // Megnyitja irasra az output file-t
	 {
	   printf("Output file can not be created\n");
	   exit(1);
	 }

	printf("Outputfile: %s\n", OutN);
					    //Megkezdi az output file-t
	fprintf(OutF, "\\documentstyle[bezier, emlines2]{article}\n");
	fprintf(OutF, "\\begin{document}\n");

	return OutF;
  } //end OpenOutput

 void ReadHeader (FILE *InF)
  {
   tline ActLine;
   int LineLength;

   fgets(ActLine, MAXLINE, InF);
   LineLength = strlen(ActLine) - 1;
   if (strncmp(ActLine, "PEP",LineLength)) // Ha az elso sor nem "PEP"
    {
     printf("Irregular output file\n");
     exit(3);
    }

   fgets(ActLine, MAXLINE, InF);
   LineLength = strlen(ActLine) - 1;
   if ((strncmp(ActLine, "PTNet",LineLength)) &&
		(strncmp(ActLine,"PetriBox",LineLength)))
    {                          //Ha a masodik sor nem PTNet vagy PetriBox
     printf("Irregular output file\n");
     exit(3);
    }

    fgets(ActLine, MAXLINE, InF);
    LineLength =  strlen(ActLine) - 1;
    if ((strncmp(ActLine, "FORMAT_N", LineLength)) &&
			(strncmp(ActLine,"FORMAT_N2", LineLength)))
    {                    //Ha a harmadik sor nem FORMAT_N vagy FORMAT_N2
     printf("Irregular output file\n");
     exit(3);
    }

    printf("Header OK\n");

  }  // end ReadHeader


int ReadDefault( FILE *InF, tphase *StartPh, ptdefaults DF)
 {

  typedef enum { num, crd, drc, phch, whitespace} tstate; // Lexikalis egysegek fajtai
  typedef enum { DBL, DPL, DTR, DPT} tdef;

  tline Line;
  int Count;

	tdef ActDef; // Az aktualis alapertelmezes-sor targya
  short CountLine; // Az aktualis sorbol mar elolvasott karakterek szama
	short CountChar ;  //Az eppen olvasott lex. egys. hossza
	tstate ActState; // Az aktualis lexikai egys. tipusa
	string ActString; // Az eppen olvasott lex. egyseg tarolasara
	char ActCommand; // Az aktualis opcio karaktere
	char ActChar;     //Az aktualis karakter
	string TempX, TempY; // A koordinatak karaktereinek ideiglenes tarolasara
	short Cnt, LenX, i;  //Szamlalo, X coordinata hossza,
					// lista-karakter szamlalo
					// lista elem szamlalo

  Count = 0;
  fgets(Line, MAXLINE, InF);
  while ((strncmp(Line, "BL",2)) && (strncmp(Line,"PL",2)))
	{                                 //  Amig nem BL-t vagy PL-t olvas

	CountLine = 0;
	CountChar = 0;
	ActChar = Line[CountLine++];
	ActString[CountChar++] = ActChar;
	while (ActChar != '\n')
	 {
	  switch (ActChar)
		{
		// case '\t':
		 case ' ' :             //whitespace-k atolvasasa
			 ActState = whitespace;
			 ActChar = Line[CountLine++];
			 ActString[CountChar++] = ActChar;
			 break;

		 case '-':
			ActState = crd;   // Ha '-'-al kezdodik a lex. egys.,
			while (ActChar != '@')	// akkor koordinatanak kell lennie
			  {
				ActChar = Line[CountLine++];
				ActString[CountChar++] = ActChar;
			  }
			 ActChar = Line[CountLine++];
			 ActString[CountChar++] = ActChar;
			 if (ActChar == '-')
			  {                // Ha a masodik koordinata negativ
				ActChar = Line[CountLine++];
				ActString[CountChar++] = ActChar;
				while (isdigit(ActChar))
				 {
				  ActChar = Line[CountLine++];
				  ActString[CountChar++] = ActChar;
				 }
			  }
			 else
			  {		// Ha a masodik koordinata pozitiv
				while (isdigit(ActChar))
				 {
				  ActChar = Line[CountLine++];
				  ActString[CountChar++] = ActChar;
				 }
			  }
			 break;

		 case '0':
		 case '1':
		 case '2':
		 case '3':       // Szamot olvas
		 case '4':
		 case '5':
		 case '6':
		 case '7':
		 case '8':
		 case '9':
			 ActState = num;
			 while (isdigit(ActChar))
			  {
				ActChar = Line[CountLine++];
				ActString[CountChar++] = ActChar;
			  }
			 if (ActChar == '@')
			  {
				ActState = crd;
				ActChar = Line[CountLine++];
				ActString[CountChar++] = ActChar;
				if (ActChar == '-')
				 {      // Ha a masodik koordinata negativ
				  ActChar = Line[CountLine++];
				  ActString[CountChar++] = ActChar;
				  while (isdigit(ActChar))
					{
					 ActChar = Line[CountLine++];
					 ActString[CountChar++] = ActChar;
					}
				 }
				else
				 {			// Ha a masodik koordinata pozitiv
				  while (isdigit(ActChar))
					{
					 ActChar = Line[CountLine++];
					 ActString[CountChar++] = ActChar;
					}
				 }
			  }
			 break;

		// case 's':
		// case 't':
		// case 'w':
		// case 'n':
		// case 'a':       // Vezerlo karaktert olvas

		/*	ActState = drc;
			ActChar = Line[CountLine++];
			ActString[CountChar++] = ActChar;
			break; */

		 case 'D':
			 ActState = phch;


			 for (i=0; i<3; i++)
			  {
				ActChar = Line[CountLine++];
				ActString[CountChar++] = ActChar;
			  }
			break;

		 case '%':
			ActChar = '\n'; //Ami a % utan van az erdektelen
			break;

		 default :  // nem csinal semmit, mert feltettuk, hogy helyes az input
		   ActState = drc;
			ActChar = Line[CountLine++];
			ActString[CountChar++] = ActChar;
			 break;
		} // end switch

		ActString[CountChar-1] = '\0'; //Lezarja az aktualis stringet

		switch (ActState)
		 {			// A lex. egys fajtajatoltol fuggoen csinal vele
					//valamit

		  case num:
			 if (ActCommand == 'w')	// a szamot beirja
			  DF->DArcWeight =  atoi(ActString);
			 break;

		  case crd:

			 Cnt = 0;            // Ha koordinata az aktualis lex. egys.
			 while (ActString[Cnt] != '@')
			  {                  // TempX-be masolja az elso koordinatat
				TempX[Cnt] = ActString[Cnt];
				Cnt++;
			  }
			 TempX[Cnt] = '\0';       // Lezarja TempX-t
			 LenX = Cnt+1;
			 Cnt++;
			 while ( ActString[Cnt] != '\0')
			  {
				TempY[Cnt-LenX] = ActString[Cnt];
				Cnt++;
			  }
			 TempY[Cnt-LenX] = '\0'; //Lezarja TempY-t
			 if (ActCommand == 'n')
			  switch (ActDef)
			  {
				case DBL:
							SetICoord(&(DF->DCBlockName), atoi(TempX), atoi(TempY));
							break;
				case DPL:
							SetICoord(&(DF->DCPlaceName), atoi(TempX), atoi(TempY));
							break;
				case DTR:
							SetICoord(&(DF->DCTransName), atoi(TempX), atoi(TempY));
							break;
				case DPT:
							SetICoord(&(DF->DCArcWeight), atoi(TempX), atoi(TempY));
							break;
			 }

			 if (ActCommand == 'a')
			  switch (ActDef)
			  {
				case DBL:
							SetICoord(&(DF->DCBlockMean), atoi(TempX), atoi(TempY));
							break;
				case DPL:
							SetICoord(&(DF->DCPlaceMean), atoi(TempX), atoi(TempY));
							break;
				case DTR:
							SetICoord(&(DF->DCTransMean), atoi(TempX), atoi(TempY));
							break;
				}



			 break;


			case drc:
			 ActCommand = ActString[0];

			 break;

			case phch:
			 if (!strncmp(ActString, "DBL", 3))
				ActDef = DBL;
			 if (!strncmp(ActString, "DPL", 3))
				ActDef = DPL;
			 if (!strncmp(ActString, "DTR", 3))
				ActDef = DTR;
			 if (!strncmp(ActString, "DPT", 3))
				ActDef = DPT;
			 break;
		 }

		 CountChar = 0;
		 ActString[CountChar++] = ActChar; // Az aktualis string elejere irja
				// az utoljara olvasott karaktert,
				//ami mar nem tartozik az elozo lex elemhez
				// (ez a ciklus invarians)

	 } // end while

		 fgets(Line, MAXLINE, InF);
		 Count++;
	}

  if (strncmp(Line, "BL",2))	//Beallitja a kezdofazist (Ha a feltetel
	*StartPh = PhPL;              //igaz, akkor kulonbozik a ket string)
  else
	*StartPh = PhBL;
  printf("Default OK\n");
  return Count;         //Visszaadja a Header es az BL/PL kozott sorok szamat
 } // end ReadDefault




boolean PictSize(int MaxX, int MaxY, int *FactInd)
 {
  boolean Land;
  double Ratio;
  short i=0;

  if ((MaxX>PAGEWIDTH) || (MaxY>PAGEHEIGHT))
   {                    // ha valamelyik iranyban tul nagy  az eredeti meret
     if ((MaxX>PAGEHEIGHT) || (MaxY>PAGEWIDTH))
      {                // ha elforditve sem fer bele
	if (MaxX>MaxY)
	 {            // ha fekvo kep
	  Ratio = min(double(PAGEWIDTH)/double(MaxY),
				double(PAGEHEIGHT)/(double)MaxX);
	  Land = TRUE;
	 }
	else
	 {           // ha allo kep
	  Ratio = min(double(PAGEWIDTH)/double(MaxX),
				double(PAGEHEIGHT)/(double)MaxY);
	  Land = FALSE;
	 }

	if (Ratio<Factors[0])
	  {         // ha sehogy sem fer el
	   printf("Too big picture\n");
	  // exit(8);
	  *FactInd = 0;
	  }
	else
	 {
	  while(Factors[i] < Ratio)
	   i++; // megkeresi melyik factorhoz van legkozelebb
	   *FactInd = i-1;
	  }

      }
     else
      {               // ha elforditva belefer
       *FactInd = 2;
       Land = TRUE;
      }

   }
  else
   {                  // ha ugy, ahogy van elfer
    *FactInd = 2;    // 1:1
    Land = FALSE;
   }
  return Land;
 }


int ProcArg(int argc, char *argv[], ptoption Opt)
 {
  short i=1;
  boolean ArgOption = TRUE; //az argumentum opcio, azaz / jellel kezdodik
  boolean BN = FALSE,
	  BM = FALSE,
	  Magn = FALSE,
	  AB  = FALSE,
	  AI  = FALSE,
	  PhTR = FALSE,
	  SynTR = FALSE,
	  Indep = FALSE,
	  BL = FALSE, // flagek, jelzik, hogy az
				// adott opcio szerpelt-e mar
	  ERR = FALSE; // argumentumlista-hibajelzo

  if (argc == 1)
    {
     printf("Program usage:\n pep2tex {option} <input file> [<output file>]\n");
     printf("The program converts PEP Tool files to LaTeX format.\n");
     printf("Options: (The first one is the default.)\n");
     printf("n+, n- : Box name represented/not represented\n");
     printf("m+, m- : Box mean represented/not represented\n");
     printf("l-, l+ : Arcs represented by Bezier curves/by straigth lines\n");
     printf("i-, i+ : Invisible arcs not represented/represented\n");
     printf("x+, x- : All the blocks exploded/Not all the block exploded\n");
     printf("p+, p- : Phantom transitions represented/not represented\n");
     printf("s+, s- : Synchron transitions represented/not represented\n");
     printf("d+, d- : Conversion to independent document/embedded document\n");
     printf("+, ++ : Magnification by 150, 200%%\n");
     printf("-, -- : Reduction by 75, 50%%\n");
     exit(3);
    }

  while (((i<argc) && ArgOption) && (!ERR))
   {
    if (argv[i][0] == '/')
     {
	switch (argv[i][1])
	 {
	  case 'n':    // nev ki- bekapcsolas
		   if (BN)
		    ERR = TRUE;
		   else
		    if (argv[i][2] == '-')
		      Opt->BName = FALSE;
		    else
		     if (argv[i][2] == '+')
		      Opt->BName = TRUE;
		     else
		      ERR = TRUE;

		    BN = TRUE;
		    break;

	   case 'm':    // jelentes ki- bekapcsolas
		   if (BM)
		    ERR = TRUE;
		   else
		    if (argv[i][2] == '-')
		      Opt->BMean = FALSE;
		    else
		     if (argv[i][2] == '+')
		      Opt->BMean = TRUE;
		     else
		      ERR = TRUE;

		    BM = TRUE;
		    break;

	   case 'l':       // Ivek Bezier gorbevel, vagy egyenesekkel
		     if (AB)
		      ERR = TRUE;
		     else
		      if (argv[i][2] == '-')
		       Opt->Bezier = TRUE;
		      else
		       if (argv[i][2] == '+')
			Opt->Bezier = FALSE;
		       else
			ERR = TRUE;

		     AB = TRUE;
		     break;

	   case 'i':      //   lathatatlan elek ki- bekapcsolasa
		     if (AI)
		      ERR = TRUE;
		     else
		      if (argv[i][2] == '+')
		       Opt->AInvisIn =TRUE;
		      else
		       if (argv[i][2] == '-')
			Opt->AInvisIn = FALSE;
		       else
			ERR = TRUE;

		     AI = TRUE;
		     break;

	    case 'x':
		     if (BL)
		      ERR = TRUE;
		     else
		      if (argv[i][2] == '+')
		       {
			Opt->ExplAllBL = TRUE;
		       }
		      else
		       if (argv[i][2] == '-')
			{
			 Opt->ExplAllBL = FALSE;
			}
		       else
			ERR = TRUE;

		     BL = TRUE;
		     break;

	    case 'p':
		     if (PhTR)
		      ERR = TRUE;
		     else
		      if (argv[i][2] == '+')
		       Opt-> PhTRInvis =FALSE;
		      else
		       if (argv[i][2] == '-')
			Opt->PhTRInvis = TRUE;
		       else
			ERR = TRUE;

		     PhTR = TRUE;
		     break;

	    case 's':
		     if (SynTR)
		      ERR = TRUE;
		     else
		      if (argv[i][2] == '+')
		       Opt-> SyncTRInvis =FALSE;
		      else
		       if (argv[i][2] == '-')
			Opt->SyncTRInvis = TRUE;
		       else
			ERR = TRUE;

		     SynTR = TRUE;
		     break;

	    case 'd':
		     if (Indep)
		      ERR = TRUE;
		     else
		      if (argv[i][2] == '+')
		       Opt-> Independ = TRUE;
		      else
		       if (argv[i][2] == '-')
			Opt->Independ = FALSE;
		       else
			ERR = TRUE;

		     Indep = TRUE;
		     break;

	    case '+':
		     if (Magn)
		      ERR = TRUE;
		     else
		      {
		       if (strlen(argv[i]) > 2)
			{
			 if (argv[i][2] == '+')
			  Opt->Magnif = 4;
			 else
			  ERR = TRUE;
			}
		       else
			{
			 Opt->Magnif = 3;
			}
		      }
		     Magn = TRUE;
		     break;

	    case '-':
		     if (Magn)
		      ERR = TRUE;
		     else
		      {
		       if (strlen(argv[i]) > 2)
			{
			 if (argv[i][2] == '-')
			  Opt->Magnif = 0;
			 else
			  ERR = TRUE;
			}
		       else
			{
			 Opt->Magnif = 1;
			}
		      }
		     Magn = TRUE;
		     break;

	    default:
		     ERR = TRUE;
		     break;
	 }
     }
    else
     {
      ArgOption = FALSE;
     }

    i++;
   }
   if (ERR)
    {
     printf ("Error in the commandline\n");
     exit(9);

    }
   return i-1;
 }

