/* Ez a file tartalmazza a pep_tree.h fileban deklaralt tipus
implementaciojat */

#include "pep_tree.h"

void InitBlockTree(blocktree BlTree)
 {
  short i;

  for (i=0; i<MAXBLOCK; i++)
   BlTree[i] = NULL;
 }

void InsertBlockTree(blocktree BlTree, int ID, boxlist ChList, int NoBL)
 {
   ptbltree_elem PElem;
	short i;

		// Memoriafogalalas az uj csucsnak
	if ((PElem = (ptbltree_elem)malloc(sizeof(tbltree_elem))) == NULL )
	  {
		printf("Not enough memory\n");
		exit(4);
	  }

	  // Kezdoertekadas az uj csucsnak
	PElem->ID = ID;     //Megadott azonosito
	PElem->AncID = 0;  // Ismeretlen os
	for(i=0; i<NoBL; i++)
	 {
	  (PElem->Children[i]).ChID = ChList[i];   // Adott azonositoju leszarmazottak
	  (PElem->Children[i]).ChPtr = NULL;      //Ismeretlen cimu leszarmazottak
    }

	PElem->NoCh = NoBL;  // Leszarmazottak szama
	PElem->Expl = TRUE;  // Alapertelmezesben a blokk kibontott
	PElem->Invis = TRUE; //ezert nem is latszik
	PElem->Valid = FALSE;   // Meg nem dolt el, hogy latszik-e

	 // Uj csucs elhelyezese a blokk-faban
	i=0;
	while(BlTree[i])  // megkeresi az elso ures helyet a blokk-fa vektoraban
	 i++;

	BlTree[i] = PElem;
	return;
 }

void FindRelatives(blocktree BlTree)
 {
  ptbltree_elem PChild;
  short i=0, j;

  while (BlTree[i])
	{              // minden blokkon vegigmegy
	  j=0;
	  while(j<BlTree[i]->NoCh)
		{           // minden leszarmazotton vegigmegy
	PChild = GetBlock(BlTree, (BlTree[i]->Children[j]).ChID);
	(BlTree[i]->Children[j]).ChPtr = PChild;
	PChild->AncID = BlTree[i]->ID;    //Ost beallitja
		 j++;
		}
	  i++;
	}
 }



void ExplAnc(blocktree BlTree, int ID)
 {                          // rekurziv fuggveny a blokk oseinek kibontasara
  ptbltree_elem Elem, Anc;

  Elem = GetBlock(BlTree, ID);   // megkeresi az ID azonositoju elemet
  if (Elem->AncID)
	{
	 Anc = GetBlock(BlTree, Elem->AncID);  // megkeresi az oset
	 Anc->Expl = TRUE;                     // kibontja
	 Anc->Invis = TRUE;                   // lathatatlanna teszi
	 Anc->Valid = TRUE;                    // ervenyesiti
	 ExplAnc(BlTree, Elem->AncID);         // az oset is kibontja
	}
 }

void CloseChilds(blocktree BlTree, int ID)
 {                          // rekurziv fuggveny a leszarmazottak bezarasara
	ptbltree_elem Elem, Child;
	short i;

  Elem = GetBlock(BlTree, ID);   // megkeresi az ID azonositoju elemet
  for (i=0; i<Elem->NoCh; i++)
	{
	 Child = (ptbltree_elem)Elem->Children[i].ChPtr;  // megkeresi a leszarmazottjat
	 Child->Expl = FALSE;                     // bezarja
	 Child->Invis = TRUE;                    // lathatatlanna teszi
	 Child->Valid = TRUE;                    // ervenyesiti
	 CloseChilds(BlTree, Child->ID);         // bezarja a leszarmazottakat is

	}
 }



ptbltree_elem GetBlock(blocktree BlTree, int ID)
 {
	short i=0;

	while (BlTree[i]->ID != ID)
	 i++;

	return BlTree[i];
 }

void AskExplBl(blocktree BlTree)
 {
  short i=0;
  char Ans;

  while (BlTree[i])
	{
	 if (!BlTree[i]->Valid)
	  {                  // ha meg se kozvetlenul, se kozvetetten nem
			// lett meghatarozva, hogy a blokk ki legyen-e bontva
		printf("Do you wish block %d to be opened? (Y/N)\n", BlTree[i]->ID);
		Ans = getchar();
		getchar();
		while ((Ans != 'Y') && (Ans !='N') && (Ans != 'y') && (Ans != 'n'))
		 {
	 printf("Do you wish block %d to be opened? (Y/N)\n", BlTree[i]->ID);
	 Ans = getchar();
		 getchar();
		 }
		if ((Ans == 'Y') || (Ans == 'y'))
		 {                           // ha ki kell bontani
	 BlTree[i]->Expl = TRUE;   // kibontja
	 BlTree[i]->Invis = TRUE;  // lathatatlanna teszi
	 BlTree[i]->Valid = TRUE;  // ervenyesiti
	 ExplAnc(BlTree, BlTree[i]->ID); // az osoket is ki kell bontani
		 }
		else
		 {                         // ha nem kell kibontani
	 BlTree[i]->Expl = FALSE;  // bezarja
	 BlTree[i]->Invis = FALSE; // lathatova teszi
	 BlTree[i]->Valid = TRUE;  // ervenyesiti
	 CloseChilds(BlTree, BlTree[i]->ID); // leszarmazottakat is bezarja
		 }
	  }
	 i++;
	}

 }

 void ModBlock(blocktree BlTree, int ID)
  {
	ptbltree_elem Elem, Anc;
	short i;

	Elem = GetBlock(BlTree, ID);
	if (IsExplBlock(BlTree, ID))
	 {                   // Ha ki volt bontva, akkor becsukja
	  Elem->Expl = FALSE;
	  Elem->Invis = FALSE;
	  CloseChilds(BlTree, Elem->ID);
	 }
	else
	 {                  // kulonben kibontja
	  Elem->Expl = TRUE;
	  Elem->Invis = TRUE;
	  ExplAnc(BlTree,Elem->ID);
	  i=0;
	  while(BlTree[i])
		{
		if ((BlTree[i]->AncID))
		 {
		  Anc = GetBlock(BlTree, BlTree[i]->AncID);
		  if(Anc->Expl && !(BlTree[i]->Expl))
		   BlTree[i]->Invis = FALSE;
		 }
		i++;
		}
	 }

  }

  void CalcBlockCoords(blocktree BlTree, int *Width, int *Height, int *No)
	{
	  short i, j, k, MaxLevel;
	  short Level[]= {0,0,0,0,0}; // A kulonbozo szinteken levo blokkok szama
	  int ActPos[] = {0,0,0,0,0,0};
	  ptbltree_elem PElem;
	  int MaxWidth;

		 j=0;         // A blokkok szintjenek szamitasa
		 while (BlTree[j])
		  {
			k=0;
			PElem = BlTree[j];
			while (PElem->AncID)
			 {
			  k++;
			  PElem = GetBlock(BlTree, PElem->AncID);
			 }
			Level[k]++;
			BlTree[j]->BlockLevel = k;
			j++;
		  }
								 // Legszelesebb szint keresese
		 MaxLevel = 0;
		 for (i=1; i<5; i++)
		  if (Level[MaxLevel] < Level[i])
			MaxLevel = i;

		 MaxWidth = (2*Level[MaxLevel]-1)*WBLOCKSIDE;


		  j=0;
		  i=0;                // A 0. szint koordinatainak meghatarozasa
		  while (BlTree[j])
			{
			 if (BlTree[j]->BlockLevel == 0)
			  {
				BlTree[j]->X = (MaxWidth-Level[0]*WBLOCKSIDE)/(Level[0]+1)*(ActPos[0]+1)
											+ActPos[0]*WBLOCKSIDE+WBLOCKSIDE/2;
				ActPos[0]++;
				BlTree[j]->Y = WBLOCKSIDE;
				BlTree[j]->BlockPos = i;
				i++;
			  }
			 j++;
			}

		  *No = j;


		  for (k=0; k<4; k++)
			{                        // Az egyes szintek leszzarmazottainak
											// koordinatainak meghatarozasa
			 j=0;
			 while (BlTree[j])
			  {
				if (BlTree[j]->BlockLevel == k)
				 {
				  for(i=0; i<BlTree[j]->NoCh; i++)
					{
					 PElem = GetBlock(BlTree, BlTree[j]->Children[i].ChID);
					 PElem->X = (MaxWidth-Level[k+1]*WBLOCKSIDE)/(Level[k+1]+1)*(ActPos[k+1]+1)+
									 ActPos[k+1]*WBLOCKSIDE+WBLOCKSIDE/2;
					 ActPos[k+1]++;
					 PElem->Y = 2*(k+1)*WBLOCKSIDE+WBLOCKSIDE;
					}
				 }
				j++;
			  }
			}


	*Width = MaxWidth;

	i=0;
	while (Level[i])
	i++;

	*Height = 2*i*WBLOCKSIDE;

	}

 void ExplNoBl(blocktree BlTree)
  {
	short i=0;

	while (BlTree[i])
	 {
	  BlTree[i]->Expl = FALSE;
	  BlTree[i]->Valid = TRUE;
	  i++;
	 }
  }


 boolean IsExplBlock(blocktree BlTree, int ID)
  {
	ptbltree_elem PElem;

	PElem = GetBlock(BlTree, ID);
	return PElem->Expl;
  }

boolean IsInvisBlock(blocktree BlTree, int ID)
 {
  ptbltree_elem Elem;

	Elem = GetBlock(BlTree, ID);
	return  Elem->Invis;
 }

int GetBlockNo(blocktree BlTree)
 {
  int k=0;

  while(BlTree[k])
	k++;

  return k;
 }

void GetBlockInfoByNum(blocktree BlTree, int Num, int *ID, int *X, int *Y,
						boolean *Expl)
  {
	*ID = BlTree[Num]->ID;
	*X = BlTree[Num]->X;
	*Y = BlTree[Num]->Y;
	*Expl = BlTree[Num]->Expl;
  }

void GetBlockInfoByID(blocktree BlTree, int ID, int *X, int *Y,
						boolean *Expl)
  {
	ptbltree_elem PElem;

	PElem = GetBlock(BlTree, ID);

	*X = PElem->X;
	*Y = PElem->Y;
	*Expl = PElem->Expl;

  }

int GetBlockRelations(blocktree BlTree, int *Relations)
 {
  int i=0, j=0, k;

  while(BlTree[i])
	{
	 if (BlTree[i]->NoCh)
	  {
		 for (k=0; k<BlTree[i]->NoCh; k++)
		  {
			Relations[2*j] = BlTree[i]->ID;
			Relations[2*j+1] = BlTree[i]->Children[k].ChID;
			j++;
		  }
	  }
	 i++;
	}
  return j;
 }

void DestroyBlockTree(blocktree BlTree)
 {
  short i;
  for (i=0; i<MAXBLOCK; i++)
	{
	 if (BlTree[i])
	  {
		free((ptbltree_elem) BlTree[i]);
		BlTree[i] = NULL;
	  }	
	}
 }
