package provide pep_edit 1.0

###############################################################
# Hier kommt der Teil, der die neue Referenzdatei lesen soll. #
###############################################################

############################
# Konvertierungsprozeduren #
############################

#----------------------------------
proc convert_def {def_string} {
#
# Convert a line from definition area and return reference
# number and string.
#
# Example: convert_def 1 R"(2,3;4,5)"V{2..5}
#           -> 1 (2,3;4,5)

 # if string contains a V, it's a variable
 if {[string first {"V"} $def_string] != -1} {
 	set is_var 1
 } else {
 	set is_var 0
 }

 # set def_string to return result, i.e. number and pos
 set end [string first ")" $def_string]
 set def_string [string range $def_string 0 $end]

 # remove R" from string, final " is already removed
 regsub {R"\(} $def_string {(} return_string

 set return_string "$return_string $is_var"

 # return list
 return $return_string
}

#----------------------------------
proc convert_ar_pfa {pfa_ar_string} {
  regsub {T"} $pfa_ar_string {} return_string
  regsub {"} $return_string {} return_string

  return $return_string
} ;# convert_ar_pfa

#----------------------------------
proc convert_pl_hl {hl_pl_string} {
 regsub {T"} $hl_pl_string {} return_string
 regsub {"} $return_string {} return_string

 set return_string [sort_references $return_string]
 return $return_string
}

#----------------------------------
proc convert_pl_ll {ll_pl_string} {
 regsub {T"} $ll_pl_string {} return_string
 regsub {"} $return_string {} return_string
 regsub {;} $return_string {+} return_string

 set return_string [sort_references $return_string]
 return $return_string
}


#####################
# Einleseprozeduren #
#####################

#proc read_references { } {
#
# global gvar hl_ref_ext ll_ref_ext def
#
# if {[file exists $gvar(prjname)$hl_ref_ext] == 1} {
#      read_reference_from_hl_file $gvar(prjname)$hl_ref_ext
# }
# if {[file exists $gvar(prjname)$ll_ref_ext] == 1} {
#      read_reference_from_ll_file $gvar(prjname)$ll_ref_ext
# }
#}

# DEF
######

#----------------------------------
proc read_def_line {refid} {
# Liest eine Zeile aus dem Definitionsbereich und legt den Inhalt
# in das globale Array def(x). Die Datei MUSS geoeffnet sein und
# die refid wird uebergeben.

 global def is_var

 set def_line [read_line $refid]

 # Wenn der Definitionsbereich verlassen wird...
 if {$def_line == "BL" || $def_line == "PL" || $def_line == "AR"} {
    return $def_line
  }

 set def_list [convert_def $def_line]
  # es gab mal eine s2m-Version, die ':' statt ';' als Trennzeichen
  # in der Referenz verwendete...  das ist nicht erlaubt!
 if {[string first ":" [lindex $def_list 1]]!=-1} {
  return "OLD_STYLE"
 }
 set def([lindex $def_list 0]) [lindex $def_list 1]
 set is_var([lindex $def_list 0]) [lindex $def_list 2]

 return 1
}

# PFA
######
#--------- Kanten -----------------
proc read_ar_pfa_line {refid} {
  global ref_ar_pfa

  set ar_pfa_line [read_line $refid]

  if {$ar_pfa_line==""} {
    return $ar_pfa_line
  }

  set ar_pfa [convert_ar_pfa $ar_pfa_line]
  set ref_ar_pfa([lindex $ar_pfa 0]) [lindex $ar_pfa 1]

  return 1
} ;# read_ar_pfa_line

# HL
#####

#--------- Stellen ----------------
proc read_pl_hl_line {refid} {
 global ref_pl_hl

 set pl_hl_line [read_line $refid]

 # Wenn der Stellenbereich betreten wird...
 if {$pl_hl_line == "PL"} {
    set pl_hl_line [read_line $refid]
 }

 # Wenn der Stellenbereich verlassen wird...
 if {$pl_hl_line == "TR"} {
    return $pl_hl_line
 }

 set pl_hl_list [convert_pl_hl $pl_hl_line]
 set ref_pl_hl([lindex $pl_hl_list 0]) [lindex $pl_hl_list 1]

 return 1
}

#-------- Transitionen ------------
proc read_tr_hl_line {refid} {
 global ref_tr_hl

 set tr_hl_line [read_line $refid]

  # Ueberlesen
  if {$tr_hl_line == "PTR"} {
     set tr_hl_line [read_line $refid]
  }
  # Wenn der Transitionenbereich verlassen wird...
  if {[eof $refid]} {
     return $tr_hl_line
  }

 # Konvertierung wie bei den Stellen...
 set tr_hl_list [convert_pl_hl $tr_hl_line]
 set ref_tr_hl([lindex $tr_hl_list 0]) [lindex $tr_hl_list 1]

 return 1
}

# LL
#####

#----------- Stellen --------------
proc read_pl_ll_line {refid} {
 global ref_pl_ll

 set pl_ll_line [read_line $refid]

 # Wenn der Stellenbereich betreten wird...
 if {$pl_ll_line == "PL"} {
    set pl_ll_line [read_line $refid]
 }

 # Wenn der Stellenbereich verlassen wird...
 if {$pl_ll_line == "TR"} {
    return $pl_ll_line
 }

 set pl_ll_list [convert_pl_ll $pl_ll_line]
 set ref_pl_ll([lindex $pl_ll_list 0]) [lindex $pl_ll_list 1]

 return 1
}

#----------- Transitionen ---------
proc read_tr_ll_line {refid} {
 global ref_tr_ll

 set tr_ll_line [read_line $refid]

  # Ueberlesen
  if {$tr_ll_line == "PTR"} {
     set tr_ll_line [read_line $refid]
  }

  # Wenn der Transitionenbereich verlassen wird...
  if {[eof $refid]} {
     return $tr_ll_line
  }
 # Konvertierung wie bei den Stellen...
 set tr_ll_list [convert_pl_ll $tr_ll_line]
 set ref_tr_ll([lindex $tr_ll_list 0]) [lindex $tr_ll_list 1]

 return 1
}


#----------------------------------
# Liest den Definitionsbereich aus der Datei $name.
# Dazu: 1. Ueberlesen bis "DEF"
#       2. Zeilen in Array $def schreiben
#       3. bis "BL" oder "PL" Bereich.
#	4. Verlassen immer beim PL Bereich
#
# Ist alles normal eingelesen, so wird eine 1, sonst
# eine 0 zurueckgegeben.
proc read_def_file {name} {
 global def refid gvar

 # Ein evtl. altes def-Array wird geloescht (man weiss ja nie).
 if {[array exists def]} {
     delete_ref_tags
 }
 if {[array exists is_var]} {
     unset is_var
 }

 if {[file exists $name]} {
    set refid [open $name]
    set line [read_line $refid]
 }

 # Ueberlesen bis zum Definitionsbereich...
 skip_to $refid DEF

 set ret_value 1
 while {![eof $refid] && $ret_value == 1} {
    set ret_value [read_def_line $refid]
 }

  # es gab mal eine s2m-Version, die ':' statt ';' als Trennzeichen
  # in der Referenz verwendete...  das ist nicht erlaubt!
 if {$ret_value == "OLD_STYLE"} {
   puts "Your Reference file is old fashioned. Please recompile the net."
   return 0
 }

 # Wenn kein Array geschrieben wurde, ist wohl auch etwas
 # nicht in Ordnung.
 if {[array exists def]} {
    # Setzten der Tags
    set_ref_tags
    return 1
 } else {
    return 0
    puts "Error while reading file..."
 }
}

### EINLESEN DER PFA KANTEN                     ###
###################################################
#
proc read_ar_pfa_file {} {
  global refid ref_ar_pfa

  # ggf. altes Array loeschen
  if {[array exists ref_ar_pfa]} {
    unset ref_ar_pfa
  }

  set ret_value 1
  while {![eof $refid] && $ret_value==1} {
    set ret_value [read_ar_pfa_line $refid]
  }
} ;# read_ar_pfa_file

### EINLESEN DER HL-STELLEN UND HL-TRANSITIONEN ###
###################################################
#
# Hier werden auch die Blockinformationen eingelesen...

#----------------------------------
# Einlesen der Stellen-Referenzen in des Array ref_pl_hl
proc read_pl_hl_file { } {
 global refid ref_pl_hl

 # Ein evtl. altes ref_pl_hl-Array wird geloescht (man weiss ja nie).
 if {[array exists ref_pl_hl]} {
     unset ref_pl_hl
 }

 set ret_value 1
 while {![eof $refid] && $ret_value == 1} {
    set ret_value [read_pl_hl_line $refid]
 }
    
 if {$ret_value != "TR"} {
        puts "Error while reading file..."
	return 0
 }
}
        
proc read_tr_hl_file { } {
# Einlesen der Stellen-Referenzen in das Array ref_tr_hl

 global refid ref_tr_hl

 # Ein evtl. altes ref_tr_hl-Array wird geloescht (man weiss ja nie).
 if {[array exists ref_tr_hl]} {
     unset ref_tr_hl
 }
 set ret_value 1
 while {![eof $refid] && $ret_value == 1} {
    set ret_value [read_tr_hl_line $refid]
 }
 close $refid
}

### EINLESEN DER LL-STELLEN UND LL-TRANSITIONEN ###
###################################################

# Einlesen der Stellen-Referenzen in des Array ref_pl_ll
proc read_pl_ll_file { } {
 global refid ref_pl_ll

 # Ein evtl. altes ref_pl_ll-Array wird geloescht (man weiss ja nie).
 if {[array exists ref_pl_ll]} {
     unset ref_pl_ll
 }

 set ret_value 1
 while {![eof $refid] && $ret_value == 1} {
    set ret_value [read_pl_ll_line $refid]
 }
    
 if {$ret_value != "TR"} {
        puts "Error while reading file..."
	return 0
 }
}
        
# Einlesen der Stellen-Referenzen in des Array ref_tr_ll
proc read_tr_ll_file { } {
 global refid ref_tr_ll

 # Ein evtl. altes ref_tr_ll-Array wird geloescht (man weiss ja nie).
 if {[array exists ref_tr_ll]} {
     unset ref_tr_ll
 }
 set ret_value 1
 while {![eof $refid] && $ret_value == 1} {
    set ret_value [read_tr_ll_line $refid]
 }
 close $refid
}

#----------------------------------
# liest eine Zeile von $refid ein und gibt eine getrimmte
# Zeile zurueck... (Zum Ueberlesen und so...)
proc read_line {refid} {
 gets $refid line
 return [string trim $line]
}

#----------------------------------
# Ueberlesen bis $value
proc skip_to {refid value} {
 set line [read_line $refid]
 while {$line != $value} { 
    set line [read_line $refid]
    # Wenn hier Dateiende erreicht wird, ist etwas
    # nicht ok.
    if {[eof $refid]} {
        puts "Error while reading file..."
        return 0
    }
 }
 return 1
}

#----------------------------------
# Sortieren der Referenzen (Element Nr.1)
# Bsp.: P11 3:1*1:4 -> P11 1:4*3:1
proc sort_references {string} {
 set unsorted_list [lindex $string 1]
 regsub -all -- {\*} $unsorted_list { } unsorted_list
 set sorted_list [sort_positions $unsorted_list]
 regsub -all -- { } $sorted_list {*} sorted_list
 set string [lreplace $string 1 1 $sorted_list]
 return $string
}

# Sort ref string such that variables are last
proc sort_positions {string} {
 set pos [lsearch $string "*+*"]
 if {$pos != -1} {
 	set var [lindex $string $pos]
 	set string [lreplace $string $pos $pos]
 	set sorted_list [lsort $string]
	lappend sorted_list $var
 } else {
 	set sorted_list [lsort $string]
 }
 return $sorted_list
}


#################################################
### Erstellen der Referenz Liste mit der Maus ###
#################################################

# Setzt das item in die globale Liste ref_string
# und sortiert diese.
proc insert_in_ref_string {item} {
 global ref_string

 lappend ref_string $item
 set ref_string [sort_positions $ref_string]
 check_name_button
 abstr_name_dlg update
}

# Anhaengen einer Variablenbelegung
proc append_ref_string {item} {
 global ref_string

 set pos [lsearch $ref_string "*+*"]
 if {$pos != -1} {
	set oldvar [lindex $ref_string $pos]
	set oldtag [string range $oldvar 0 [expr {[string first : $oldvar] -1}]]
	delete_var_from_ref_string 
	unmark_tag $oldtag
 }
 lappend ref_string $item
 check_name_button
 abstr_name_dlg update
}

# Loescht item aus der globalen Liste ref_string
proc delete_from_ref_string {item} {
 global ref_string

 set pos [lsearch $ref_string $item]
 set ref_string [lreplace $ref_string $pos $pos]
 check_name_button
 abstr_name_dlg update
}

# loescht die gesetzte Variable mit dem Wert
proc delete_var_from_ref_string { } {
 global ref_string

 if {$ref_string != ""} {
    set ref_string [lreplace $ref_string end end]
 }
 check_name_button
 abstr_name_dlg update
}

proc change_ref_string {item oldvalue newvalue} {
 global ref_string

 set pos [lsearch $ref_string $item:$oldvalue]
 set ref_string [lreplace $ref_string $pos $pos $item:$newvalue]
 check_name_button
 abstr_name_dlg update
}

###################### Beginn des Abstrakten Teils ########################

# aktiviert/deaktiviert den set_name Button
proc check_name_button { } {
 global ref_string

 if {[winfo exists .insert.set_name]} {
    if {$ref_string == "" || [llength $ref_string] != 1} {
       if {[winfo exists .abstract.buttonframe.ok]} {
          .abstract.buttonframe.ok configure -state disabled
          bind .abstract.entryframe.nameentry <Return> {}
       }
       .insert.set_name configure -state disabled
    } else {
       .insert.set_name configure -state normal
       if {[winfo exists .abstract.buttonframe.ok]} {
          .abstract.buttonframe.ok configure -state normal
          bind .abstract.entryframe.nameentry <Return> {set_abstract 1}
          .insert.set_name configure -state disabled
       }
    }
 }
}

proc get_current_tag_nr { } {
  global ref_string

  regsub {:} $ref_string { } tag_nr
  return [lindex $tag_nr 0]
}

# Gibt die Positionen des aktuellen Tags zurueck
proc get_tag_pos { tag_nr } {
 set range [.tedit tag range tag$tag_nr]
 return [incr_pos $range]
}

# Positionskonverter
######################

proc def2abstract {def_string} {
 regsub -all -- {\(|\)|,|;} $def_string " " def_string
 
 set v0 [lindex $def_string 0]
 set v1 [lindex $def_string 1]
 set v2 [lindex $def_string 2]
 set v3 [lindex $def_string 3]

 return "$v0.$v1+$v2.$v3"
}

proc abstract2def {abs_string} {
 regsub -all -- {\+|\.} $abs_string " " abs_string
 
 set v0 [lindex $abs_string 0]
 set v1 [lindex $abs_string 1]
 set v2 [lindex $abs_string 2]
 set v3 [lindex $abs_string 3]

 return "($v0,$v1;$v2,$v3)"
}

# Konvertiert OpenDoor:0 -> 17:0
# oder        Lift:4;5   -> 18:4;5
proc abs_ref2def_ref {abs_ref} {
 set def_ref ""
 regsub -all -- {\*} $abs_ref " " abs_ref

 foreach i $abs_ref {
    regsub -all -- {:|;} $i " " i 

    set name  [lindex $i 0]
    set type  [lindex $i 1]
    set value [lindex $i 2]
    set number [find_abs_number $name]

    if {$value == ""} {
      set ret "$number:$type"
    } else {
      set ret "$number:$type;$value"
    }
    if {$def_ref == ""} {
       set def_ref "$ret"
    } else {
       set def_ref "$def_ref*$ret"
    }
 }
 return $def_ref
}

# Suchfunktionen
################

proc find_abs_name { def_pos } {
 global abs_pos abs

 for {set i 1} {$i <= [array size abs_pos]} {incr i} {
    if { $def_pos == [abstract2def $abs_pos($i)]} {
       return $abs($i)
    }
 }
 return ""
}

proc find_abs_number {abs_name} {
 global abstr

 set abs_list [array names abstr]
 foreach i $abs_list {
   if { $abstr($i) == $abs_name} {
	return $i
   }
 }
 return -1
}


# Eingabe der abstrakten Namen
###################################

proc abstr_name_dlg { {what ""} } {
 global abstr_name abstr ref_string tag_nr tag_pos

 set abstr_name ""
 set tag_pos "0.0+0.0"

 # Ermitteln der Tagnummer und Position
 if { [llength $ref_string] == 1} {
    set tag_nr  [get_current_tag_nr]
    set tag_pos [get_tag_pos $tag_nr]

    if {[array exists abstr]} {
      catch {
        set abstr_name $abstr($tag_nr)
      }
    }
 }

 if {[winfo exists .abstract] || $what == "update"} {
   return
 }

  toplevel .abstract -class Toplevel
  wm maxsize .abstract 1009 745
  wm minsize .abstract 1009 745
  wm resizable .abstract 0 0
  wm deiconify .abstract
  wm title .abstract "Abstract Name"
  frame .abstract.entryframe -borderwidth 2 -height 75 -width 125
  label .abstract.entryframe.name \
    -borderwidth 1 -text {Enter the abstract name:}
  entry .abstract.entryframe.nameentry -textvariable abstr_name
  frame .abstract.buttonframe \
    -borderwidth 2 -height 29 -relief groove -width 125
  button .abstract.buttonframe.ok \
    -padx 9 -pady 3 -text Ok -command {set_abstract 1}
  button .abstract.buttonframe.close \
    -padx 9 -pady 3 -text "Close" -command {set_abstract 0}
  pack .abstract.entryframe\
    -in .abstract -anchor center -expand 0 -fill x -side top
  pack .abstract.entryframe.name \
    -in .abstract.entryframe -anchor center -expand 0 -fill none -side left
  pack .abstract.entryframe.nameentry \
    -in .abstract.entryframe -anchor center -expand 1 -fill x -side top
  pack .abstract.buttonframe \
    -in .abstract -anchor center -expand 0 -fill x -side top
  pack .abstract.buttonframe.ok -in .abstract.buttonframe \
    -anchor center -expand 0 -fill none -ipadx 10 -padx 70 -side left
  pack .abstract.buttonframe.close -in .abstract.buttonframe \
    -anchor center -expand 0 -fill none -padx 60 -side left

  bind .abstract.entryframe.nameentry <Return> {set_abstract 1}
  check_name_button
}

proc set_abstract { save } {
 global abstr_name abstr tag_nr tag_pos

 if {$save} {
   set abstr($tag_nr) $abstr_name
   focus .tedit
 } else {
   destroy .abstract
 }
 check_name_button
}

###################### Ende des Abstrakten Teils ########################

# Erzeugen des Uebertragungsstrings fuer das peptool
####################################################
#
# highlight -hl_net=P11,P12 -ll_net=P12 -fo P12
#
proc send_ref_name { } {
 global rc ref_string
 
 # Initialisierung
 set hl_string ""
 set ll_string ""
 set send_string ""

 set temp_string $ref_string
 set comp_string $temp_string
 while {[regexp "(\[^\ \]+):3(.*)" $comp_string tmp tag comp_string]} {
   change_ref_string $tag 3 2 
   insert_in_ref_string $tag:1
 }

 set comp_string $ref_string
 set ref_string $temp_string
 # HL Referenzen suchen
 ######################
 if { $rc(show_ref_in_hl) } {
   set hl_string [ref_string2hl_name_ap $comp_string]
 }

 # LL Referenzen suchen
 ######################
 if { $rc(show_ref_in_ll) || $rc(show_ref_in_formula) } {
   set ll_string [ref_string2ll_name_ap $comp_string]
 }

 # Hier sollte der String zum Uebertragen fertig sein. Jetzt
 # muss noch ueberprueft werden, was denn nun gehighlightet werden
 # soll.

 if { $rc(show_ref_in_hl) && $hl_string != "" } {
   set hl_string "-hl_net=$hl_string"
   set send_string $hl_string
 }
 if { $rc(show_ref_in_ll) && $ll_string != "" } {
   set ll_string "-ll_net=$ll_string"
   set send_string "$send_string $ll_string"
 }
 if {$rc(show_ref_in_formula) && $ll_string != "" } {
   set ll_string "-fo_net=$ll_string"
   set send_string "$send_string $ll_string"
 }

 # Leerzeichen am Anfang loeschen
 set send_string [string trimleft $send_string]

 # Wenn jetzt noch was drin steht, dann wech damit.
 if {$send_string != ""} {
   highlight_reference $send_string
 }
}

#######################################
# Konvertierungen zwischen den Arrays #
#######################################

# Diese Prozedur ueberprueft ob alle Elemente von List1
# an einer beliebigen Stelle in List2 vorkommen
proc partial_in {list1 list2} {
 foreach elem $list1 {
	if { [lsearch -exact $list2 $elem] == -1} {
		return 0
	}
	# Element ist nicht in List2
 }
 return 1
}

proc intersect {list1 list2} {
 foreach elem $list1 {
	if { [lsearch -exact $list2 $elem] != -1} {
		return 1
	}
 }
 return 0
}

# Loescht den Variablenwert aus string
# "1:2 3:2+dot" -> "1:2 3:2"
proc delete_value {string} {
 set pos [string range $string 0 [expr {[string first "+" $string] -1}]]
 if {$pos == ""} {
 	return $string
 } else {
 	return $pos
 }
}

# Gibt den Wert des Strings zurueck, wenn String nicht leer ist.
proc get_value {string} {
 set pos [string first "+" $string]
 if {$pos == -1} {
 	return ""
 } else {
 	return [string range $string [expr {$pos+1}] end]
 }
}

# Ueberprueft die Arrays ref_pl_hl und ref_tr_hl nach dem Vorkommen
# aller Elemente des ref_strings. Der Value wird hier vergessen.
proc ref_string2hl_name_ap {ref_string} {
 global ref_pl_hl ref_tr_hl rc

 set name_list ""

 if {$ref_string == ""} {
 	return
 }
 
 # Remove variable value from string
 set ref_string [delete_value $ref_string]
 
 # Erstellen der Referenzliste aus dem ref_string
 if {[array exists ref_pl_hl]} {
 	set id [array startsearch ref_pl_hl]
	while {[set a_index [array nextelement ref_pl_hl $id]] != ""} {
		set ref_list $ref_pl_hl($a_index)
                regsub -all -- {\*} $ref_list { } ref_list
		if { [intersect $ref_string $ref_list] }  {
		   set name_list "$name_list $a_index"
		}
			
	}
 }
 if {[array exists ref_tr_hl]} {
 	set id [array startsearch ref_tr_hl]
	while {[set a_index [array nextelement ref_tr_hl $id]] != ""} {
		set ref_list $ref_tr_hl($a_index)
                regsub -all -- {\*} $ref_list { } ref_list
		if { [intersect $ref_string $ref_list] }  {
		   set name_list "$name_list $a_index"
		}
			
	}
 }
 regsub { } $name_list {} name_list
 regsub -all -- { } $name_list {,} name_list
 return $name_list
}

# Ueberprueft die Arrays ref_pl_ll und ref_tr_ll nach dem Vorkommen
# aller Elemente des ref_strings. Hierbei ist allerdings der Value
# wichtig.
proc ref_string2ll_name_ap {ref_string} {
 global ref_pl_ll ref_tr_ll rc glob_value

 set name_list ""

 if {$ref_string == ""} {
 	return
 }

 # Wenn ein globaler Wert eingegeben wurde, so wird dieser
 # fuer den "Partialvergleich" genommen. Sonst der Wert
 # einer Variable

 if {$glob_value != ""} {
 	set value $glob_value
 } else {
 	set value [get_value $ref_string]
 }
 set ref_string [delete_value $ref_string]

 if {[array exists ref_pl_ll]} {
 	set id [array startsearch ref_pl_ll]
	while {[set a_index [array nextelement ref_pl_ll $id]] != ""} {
		set ref_list $ref_pl_ll($a_index)
		set ref_value [get_value $ref_list]
		# Always show dot types
		if {$ref_value == "dot"} {
		  set ref_value $value
		}
		set ref_list [delete_value $ref_list]
                regsub -all -- {\*} $ref_list { } ref_list

		if { [intersect $ref_string $ref_list] }  {
			if {$rc(show_all_values) || ($value == $ref_value)} {
			     set name_list "$name_list $a_index"
			}
		}
	}
 }
 if {[array exists ref_tr_ll]} {
 	set id [array startsearch ref_tr_ll]
	while {[set a_index [array nextelement ref_tr_ll $id]] != ""} {
		set ref_list $ref_tr_ll($a_index)
		set ref_value [get_value $ref_list]
		set ref_list [delete_value $ref_list]
                regsub -all -- {\*} $ref_list { } ref_list

		if { [intersect $ref_string $ref_list] }  {
			if {$rc(show_all_values) || ($value == $ref_value) } {
			     set name_list "$name_list $a_index"
			}
		}
			
	}
 }

 regsub { } $name_list {} name_list
 regsub -all -- { } $name_list {,} name_list
 return $name_list
}


######################
## Textmarkierungen ##
######################


proc mark_tag {where tag_id} {
 global is_var rc

 if {$is_var($tag_id)} {
 	mark_tag_var $tag_id
	return
 }

 switch $where {
 	before {mark_tag_before $tag_id}
	normal {mark_tag_normal $tag_id}
	after  {mark_tag_after  $tag_id}
 }
}

proc unmark_tag {tag_id} {
 .tedit tag configure tag$tag_id -background ""
}

##############
# Simulation #
##############

proc set_name { } {
}

#####################################
# Highlighten vom LL oder HL Editor #
#####################################

proc highlight_reference_from_hl {hl_ref} {
 global ref_pl_hl ref_tr_hl

 # An dieser Stelle werden erstmal alle Klammern aus
 # dem String genommen. Das MUSS noch geaendert werden,
 # aber mit faellt jetzt keine nette Loesung ein

 regsub -all -- {\{} $hl_ref {} hl_ref
 regsub -all -- {\}} $hl_ref {} hl_ref

 set first_index [.tedit index end]
 for {set i 0} {$i < [llength $hl_ref]} {incr i} {
 	set item ""
   	if {[partial_in [lindex $hl_ref $i] [array names ref_pl_hl]]} {
		set item $ref_pl_hl([lindex $hl_ref $i])
	} elseif {[partial_in [lindex $hl_ref $i] [array names ref_tr_hl]]} {
		set item $ref_tr_hl([lindex $hl_ref $i])
	}

	if {$item != ""} {
		# Hier steht jetzt ein Ref_String der Form:
		# 1:1*3:8
 		regsub -all -- {\*} $item { } item
		#puts "highlight_list: $item"
		for {set j 0} {$j < [llength $item]} {incr j} {
		  set highlight_item [lindex $item $j]
 		  regsub {:} $highlight_item { } highlight_item
		  set style [lindex $highlight_item 1]
		  set highlight_tag [lindex $highlight_item 0]

		  if {$style == 1} {
		  	mark_tag_before $highlight_tag
		  } elseif {$style == 2} {
			mark_tag_after $highlight_tag
		  } else {
			mark_tag_normal $highlight_tag
		  }
		  set act_index \
		    [lindex [.tedit tag ranges tag$highlight_tag] 0]
		  if {$act_index!=""} {
		    set first_index [expr \
		      [expr $act_index<$first_index]?$act_index:$first_index ]
		  }
		}
	}
 }
 .tedit see $first_index
}

proc highlight_reference_from_ll {ll_ref} {
 global ref_pl_ll ref_tr_ll

 # An dieser Stelle werden erstmal alle Klammern aus
 # dem String genommen. Das MUSS noch geaendert werden,
 # aber mit faellt jetzt keine nette Loesung ein

 regsub -all -- {\{} $ll_ref {} ll_ref
 regsub -all -- {\}} $ll_ref {} ll_ref

 set first_index [.tedit index end]
 for {set i 0} {$i < [llength $ll_ref]} {incr i} {
 	set item ""
   	if {[partial_in [lindex $ll_ref $i] [array names ref_pl_ll]]} {
		set item $ref_pl_ll([lindex $ll_ref $i])
	} elseif {[partial_in [lindex $ll_ref $i] [array names ref_tr_ll]]} {
		set item $ref_tr_ll([lindex $ll_ref $i])
	}

	if {$item != ""} {
		# Hier steht jetzt ein Ref_String der Form:
		# 1:1*3:8
 		regsub -all {\*} $item { } item
		set item [delete_value $item]
		#puts "highlight_list: $item"
		#puts "Anzahl der Stellen [llength $item]"
		for {set j 0} {$j < [llength $item]} {incr j} {
		  set highlight_item [lindex $item $j]
		  #puts "highlight_item $highlight_item"
 		  regsub {:} $highlight_item { } highlight_item
		  set style [lindex $highlight_item 1]
		  set highlight_tag [lindex $highlight_item 0]

		  if {$style == 1} {
		  	mark_tag_before $highlight_tag
		  } elseif {$style == 2} {
			mark_tag_after $highlight_tag
		  } else {
			mark_tag_normal $highlight_tag
		  }
		  set act_index \
		    [lindex [.tedit tag ranges tag$highlight_tag] 0]
		  if {$act_index!=""} {
		    set first_index [expr \
		      [expr $act_index<$first_index]?$act_index:$first_index ]
		  }
		}
	}
 }
 .tedit see $first_index
}

proc highlight_reference_from_pfa {pfa_ref} {
 #puts "highlight_reference_from_pfa: $pfa_ref"
 
 global ref_ar_pfa
 
 if {[lsearch -exact [array names ref_ar_pfa] $pfa_ref] != -1} {
   puts " ref_ar_pfa($pfa_ref) = $ref_ar_pfa($pfa_ref)"
   regsub {:} $ref_ar_pfa($pfa_ref) { } highlight_item
   set highlight_tag [lindex $highlight_item 0]
   set style [lindex $highlight_item 1]

   if {$style == 1} {
     mark_tag_before $highlight_tag
   } elseif {$style == 2} {
     mark_tag_after $highlight_tag
   } else {
     mark_tag_normal $highlight_tag
   }

   .tedit see [lindex [.tedit tag ranges tag$highlight_tag] 0]
 } else {
   puts "No reference for item $pfa_ref"
 }
} ;# highlight_reference_from_pfa

proc mark_tag_before {tag_id} {
 global color

 if {$tag_id == ""} {return}

 set current_color [get_current_color $tag_id]

 switch $current_color {
 	before {
	# if already marked unmark
 		set current_color $color(ref.active)
		delete_from_ref_string $tag_id:1
	}
	active -
	"" {
	# if active or empty
		set current_color $color(ref.before)
		insert_in_ref_string $tag_id:1
	}
	normal {
	# if marked normal change to before
		set current_color $color(ref.before)
		change_ref_string $tag_id 0 1
	}
	after {
	# if marked after change to both
		set current_color $color(ref.both)
		change_ref_string $tag_id 2 3
	}
	both {
	# if marked both change to after
		set current_color $color(ref.after)
		change_ref_string $tag_id 3 2
	}
 }
 .tedit tag configure tag$tag_id -background $current_color
}

proc mark_tag_normal {tag_id} {
 global color

 if {$tag_id == ""} {return}

 set current_color [get_current_color $tag_id]

 switch $current_color {
 	before {
	# if marked before
 		set current_color $color(ref.normal)
		change_ref_string $tag_id 1 0
	}
	active -
	"" {
	# if cursor on tag or nothing
		set current_color $color(ref.normal)
		insert_in_ref_string $tag_id:0
	}
	normal {
	# if marked before unmark, but set active
		set current_color $color(ref.active)
		delete_from_ref_string $tag_id:0
	}
	after {
	# if marked after, mark normal
		set current_color $color(ref.normal)
		change_ref_string $tag_id 2 0
	}
	both {
	# if marked before and after, mark normal
		set current_color $color(ref.normal)
		change_ref_string $tag_id 3 0
	}
 }
 .tedit tag configure tag$tag_id -background $current_color
}

proc mark_tag_after {tag_id} {
 global color

 if {$tag_id == ""} {return}

 set current_color [get_current_color $tag_id]

 switch $current_color {
 	before {
	# if marked before, mark before and after
 		set current_color $color(ref.both)
		change_ref_string $tag_id 1 3
	}
	active -
	"" {
	# if active or empty
		set current_color $color(ref.after)
		insert_in_ref_string $tag_id:2
	}
	normal {
		set current_color $color(ref.after)
		change_ref_string $tag_id 0 2
	}
	after {
		set current_color $color(ref.active)
		delete_from_ref_string $tag_id:2
	}
	both {
		set current_color $color(ref.before)
		change_ref_string $tag_id 3 1
	}
 }
 .tedit tag configure tag$tag_id -background $current_color
}

proc mark_tag_var {tag_id} {
 global rc glob_value color

 if {$tag_id == ""} {return}
 
 set current_color [get_current_color $tag_id]

 if {$rc(show_ref_in_ll)} {
 	switch $current_color {
		active {
			set current_color $color(ref.variable)
			append_ref_string $tag_id:4+$glob_value
		}
		variable -
		normal {
			set current_color $color(ref.active)
			delete_var_from_ref_string
		}
 	}
 } else {
 	# Nur HL
 	switch $current_color {
		active {
			set current_color $color(ref.variable)
			insert_in_ref_string $tag_id:4
		}
		variable -
		normal {
			set current_color $color(ref.active)
			delete_from_ref_string $tag_id:4
		}
 	}
 }
 .tedit tag configure tag$tag_id -background $current_color
}

proc toggle_tag_color {tag_id direction} {
 global color

 set cur_color_name [get_current_color $tag_id]
 if {$cur_color_name != ""} {
   set cur_color $color(ref.$cur_color_name)
 } else {
   set cur_color ""
 }
 set new_color [string range $cur_color 0 [expr {[string length $cur_color] -2}]]
 if {$direction == "in"} {
   set num 1
   if {$cur_color == ""} {
	set cur_color $color(ref.active)
   } else {
 	set cur_color "$new_color$num"
   }
 } else {
   set num 3
   if {$cur_color == ""} {
        return
   } elseif {$cur_color == $color(ref.active)} {
        set cur_color ""
   } else {
        set cur_color "$new_color$num"
   }
 }
 .tedit tag configure tag$tag_id -background $cur_color
}

proc get_current_color {tag_id} {
 global color

 set colorname [lindex [.tedit tag configure tag$tag_id -background] end]
 set colorname [string range $colorname 0 [expr [string length $colorname]-2]]1
 if {$colorname == $color(ref.active)} {
   return active
 } elseif {$colorname == $color(ref.before)} {
   return before
 } elseif {$colorname == $color(ref.after)} {
   return after
 } elseif {$colorname == $color(ref.both)} {
   return both
 } elseif {$colorname == $color(ref.normal)} {
   return normal
 } elseif {$colorname == $color(ref.variable)} {
   return variable
 }
 return ""
}

###################
## Variablenkram ##
###################

#proc value_window { } {
#global value r_val
#
# toplevel .value
# message .value.msg -text "Enter Value:" -width 100
# entry   .value.entry -relief sunken -textvariable value -width 50
# frame   .value.button_fr
# button  .value.ok -text "Ok" -command {set r_val $value; set ok 1}
# button  .value.cancel -text "Cancel" -command {set r_val "--cancel"; set ok 1}
#
# pack .value.msg
# pack .value.entry
# pack .value.button_fr
# pack .value.ok -side left -in .value.button_fr -pady 3 -padx 2m
# pack .value.cancel -side left -in .value.button_fr -pady 3 -padx 2m
#
# bind .value.entry <Return> {set r_val $value; set ok 1}
#
#tkwait variable ok
#destroy .value
#return $r_val
#}

