package provide pep_edit 1.0

proc show_bpn2_error {error_file} {

# Anzeigen der Fehler, die der bpn2pbc erzeugt.
#
# Parameter: error_file = Fehlerdatei
#
# Bemerkung: Die Fehlermeldungen werden im Array bpn2err abgelegt.
#            $bpn2err(x,1) ist die ganze Zeile
#            $bpn2err(x,2) ist die gesuchte Zeilennummer

global bpn2err line

   # Fehler aus Datei in bpn2err-array schreiben
   set i [get_error_from_file $error_file]
   if {$i == 0} {
      tk_dialog .d { } {No Errors Found !} info 0 {Ok}
      return
   }

   toplevel .syntax
   wm title .syntax "B(PN) - Syntax Errors"

   frame .syntax.buttonfr -borderwidth 2 
   frame .syntax.okfr -relief sunken -borderwidth 1
   frame .syntax.cancelfr -borderwidth 1
   frame .syntax.helpfr -borderwidth 1

   scrollbar .syntax.scroll_side -orient horiz -command ".syntax.listbox xview"
   scrollbar .syntax.scroll_up -command ".syntax.listbox yview"
   listbox .syntax.listbox -yscroll ".syntax.scroll_up set"\
           -xscroll ".syntax.scroll_side set" -relief sunken -setgrid yes\
           -width 55
   button .syntax.ok -text "Ok" -command "highlight_error"

   button .syntax.cancel -text "Close" -command "destroy .syntax"
   button .syntax.help -text "Help" -command "mini_help syntax"

   pack .syntax.buttonfr -side bottom -fill x -expand yes
   pack .syntax.scroll_side -side bottom -fill x
   pack .syntax.scroll_up -side right -fill y
   pack .syntax.listbox -fill both -expand yes
   pack .syntax.okfr .syntax.cancelfr .syntax.helpfr\
        -side left -in .syntax.buttonfr -pady 3 -padx 10m -expand yes
   pack .syntax.ok -side left -in .syntax.okfr -padx 1m -pady 1m\
        -ipady 1m -ipadx 1m
   pack .syntax.cancel -side left -in .syntax.cancelfr -padx 1m -pady 1m\
        -ipady 1m -ipadx 1m   
   pack .syntax.help -side left -in .syntax.helpfr -padx 1m -pady 1m\
        -ipady 1m -ipadx 1m   

   ############
   # Bindings #
   ############

   bind .syntax <Double-Button-1> {
       highlight_error
   }

   # Listbox erstellen
   .syntax.listbox delete 0 end
   for {set j 1} {$j <= $i} {incr j} {
        .syntax.listbox insert end $bpn2err($j,1)
   }

   .syntax.listbox selection clear 0 end
   .syntax.listbox selection set 0
}

proc bpn2_syntax_check { } {

# Ruft B(PN)2 Syntax Checker auf, der Fehlerdatei schreibt. Die Fehler
# werden im Fenster ".syntax" angezeigt. Beim Verlassen wird die
# Fehlerdatei wieder geloescht.
#
# Parameter: keine
#
# Rueckgabe: nichts
#
# Bemerkung: Die Endung der Fehlerdatei ist ".err"

   global editor

   if {[.tedit edit modified]} {
       set ok [tk_dialog .d {Warning} {To Perform The Syntax-Check You \
          First Have To Save Your Document} warning 0 {Save Changes} {Cancel}]

       if {$ok == 0} {
           Save
       } else {
           return
       }
   }

   # Aufruf des Syntaxcheckers
   catch {exec bpn2pbc $editor(filename) $editor(filename).err -e}
   
   # Anzeigen der Fehler
   show_bpn2_error $editor(filename).err
 
   # Fehlerdatei loeschen
   file delete $editor(filename).err
}

proc get_error_from_file {error_file} {

# Oeffnet Fehlerdatei und schreibt die Fehler in das Array bpn2err(x,y),
# wobei $bpn2err(x,1) = Fehlerstring und $bpn2err(x,2) = Fehlerzeile (s.o.)
#
# Parameter: error_file = Name der Fehlerdatei
#
# Rueckgabe: Anzahl der Fehler
#
# Bemerkung: Zwischen Fehlern und dem Rest muss eine Leerzeile stehen.

   global bpn2err
   
   # Fehlerdatei oeffnen
   set fileid [open $error_file]
   set i 1
   # Einlesen bis EOF oder "Leerzeile"
   while {[gets $fileid bpn2err($i,1)] >= 1} {
      # Das erste "]" suchen
      set pos [string first "\]" $bpn2err($i,1)]
      # Zeilennummer holen
      set bpn2err($i,2) [string range $bpn2err($i,1) 5 [incr pos -1 ]]
      incr i
   }
   # Fehlerdatei schliessen
   close $fileid
   incr i -1
   return $i
}

proc highlight_error { } {
   global bpn2err

   set line [.syntax.listbox index active]
   incr line 1
   
   .tedit mark set insert [string trim $bpn2err($line,2)].0
   .where configure -text [.tedit index insert]
   .tedit tag add sel insert lineend
}

