# optdia.tcl, Dialogs for changeable options of the compilers
# started: 04.03.95, last-update: 24.03.97

# BG 18.3.97 Defaults changed
# BG 18.3.97 negate -b -x with bpn2mnet
# BG 24.3.97 tried to integrate simplify. Original in OK
# BG 25.3.97 Defaults changed
# MK 28.4.97 SMV-Model-Checker integrated
# CP 28.4.97 SPIN-Model-Checker integrated
# BG 18.7.97 Set instead of Stack as Default
# BG 27.11.97 Flags for no_disjunct_idsets + procedure_box_flag/number +
#             channel_box_flag/number with bpn2mnet
# BG 27.4.98 Options for pfa_bpn2 changed: S removed and -n for N, default w/o
# CS 30.11.1999 Options for box changed, now with time
# CS 1.7.2000 Options for EVunfold changed, now with time
# CS 29.3.2001 Removed old reset functions since generic funtion works

  ####################################
  # Functions for windows management #
  ####################################

proc add_dlg_widget {prjnum widget} {
  # Add Widget to the list of open projects.
  # prjnum - number of project or -1 for main window
  # widget - Name of widget
  if {$prjnum != -1} {
    upvar #0 prj$prjnum prjx
    set prjx(open_dlg_widget.$widget) $widget
  } else {
    global gvar
    set gvar(open_dlg_widget.$widget) $widget
  }  ;# else main window was used
}  ;# add_dlg_widget

proc del_dlg_widget {prjnum widget} {
  # Remove Widget from the list of open projects.
  # prjnum - Number of Project or -1 for main window
  # widget - Name of Widget
  if {$prjnum != -1} {
    upvar #0 prj$prjnum prjx
    unset prjx(open_dlg_widget.$widget)
  } else {
    global gvar
    unset gvar(open_dlg_widget.$widget)
  }  ;# else main window was used
}  ;# del_dlg_widget

proc display_open_dlg_widgets {prjnum} {
  # Display list of open dialogs on the screen.
  # Main purpose is for debugging.
  # prjnum - Number of project or -1 for main window
  if {$prjnum != -1} {
    upvar #0 prj$prjnum prjx
    set names [array names prjx]
    puts "***** Open dialogs for project number $prjnum *****"
    foreach n $names {
      if {[string match open_dlg_widget.* $n]} {
        puts "$prjx($n)"
      }  ;# if Widget
    }  ;# foreach
    puts "***** End of list *****"
  } else {
    global gvar
    set names [array names gvar]
    puts "***** Open dialogs for the main window *****"
    foreach n $names {
      if {[string match open_dlg_widget.* $n]} {
        puts "$gvar($n)"
      }  ;# if Widget
    }  ;# foreach
    puts "***** End of list *****"
  }  ;# else main window was used
}  ;# display_open_dlg_widgets

proc dlg_option_wm_delete {prjnum widget {opt_cmd "NONE"}} {
  # The user wants to close the options dialog
  # This was a call from the window manager.
  # prjnum - projectnumber or -1 for main window
  # widget - Name of widgets
  # opt_cmd - optional command to be evaluated before 
  #    destruction of the widget
  del_dlg_widget $prjnum $widget
  if {$opt_cmd != "NONE"} {
    eval $opt_cmd
  }  ;# if optional command
  destroy $widget
}  ;# dlg_option_wm_delete

proc dlg_option_prj_close {prjnum} {
  # Close all open windows, as the project window
  # is closed.
  # prjnum - Number of project
  upvar #0 prj$prjnum prjx
  set names [array names prjx]
  foreach n $names {
    if {[string match open_dlg_widget.* $n]} {
      destroy $prjx($n)
    }  ;# if Widget
  }  ;# foreach
}  ;# dlg_option_prj_close

  ###########################################################
  # Functions for resetting of the values to their defaults #
  ###########################################################

# generic reset function
# used as a replacement for those many different functions
# formerly used. Now everything is working on the array
# structure of gvar. All default values are stored in 
# default.dlg.COMPILER.* while current values are stored in
# option.dlg.COMPILER.* with COMPILER replaced by the
# correct value. This function searches the array for those
# fields and sets the values accordingly.
# Note: search for default values as current ones are
# created by calling this function for the first time.
# name - The variable name for the specified compiler

proc dlg_option_set_defaults {name} {
  global gvar

  set options [array names gvar "default.dlg.$name.*"]
  foreach defaultname $options {
    regsub default $defaultname option optionname
    set gvar($optionname) $gvar($defaultname)
  }
}

proc dlg_option_ana_clp_set_defaults {} {
  global gvar

  set gvar(option.dlg.ana_clp.safe_net) \
    $gvar(default.dlg.ana_clp.safe_net)
  set gvar(option.dlg.ana_clp.shortest_trail) \
    $gvar(default.dlg.ana_clp.shortest_trail)
  set gvar(option.dlg.ana_clp.save_mem) \
    $gvar(default.dlg.ana_clp.save_mem)
# the following should not be altered by Default button
    if {[catch {set gvar(option.dlg.ana_clp.mode)}]} {
	set gvar(option.dlg.ana_clp.mode) \
		$gvar(default.dlg.ana_clp.mode)
    }
    if {[catch {set gvar(option.dlg.ana_clp.dead_tokens_allowed)}]} {
	set gvar(option.dlg.ana_clp.dead_tokens_allowed) \
		$gvar(default.dlg.ana_clp.dead_tokens_allowed)
    }
    if {[catch {set gvar(option.dlg.ana_clp.handle_terminal_marking)}]} {
	set gvar(option.dlg.ana_clp.handle_terminal_marking) \
		$gvar(default.dlg.ana_clp.handle_terminal_marking)
    }
} ;# set options for clp

  ###############################
  # Initialisation of variables #
  ###############################

proc init_options_dialog_values {} {
  global gvar

    # Defaults and Options for the Syntax Checker
  set gvar(default.dlg.syntax_check.skip) 1

    # Defaults and Options for B(PN) -> BoxExpr
  set gvar(default.dlg.bpn2_box_expr.seq_term) 1
  set gvar(default.dlg.bpn2_box_expr.std_init) 1
  set gvar(default.dlg.bpn2_box_expr.no_warn_old_syntax) 0

    # Defaults and Options for BoxExpr -> LL-Net
  set gvar(default.dlg.box_expr_ll_net.check_net) 0
  set gvar(default.dlg.box_expr_ll_net.rem_dup) 1
  set gvar(default.dlg.box_expr_ll_net.bundle_atomic_actions) 0
  set gvar(default.dlg.box_expr_ll_net.simple_cd_boxes) 0
  set gvar(default.dlg.box_expr_ll_net.ext_label) 1
  set gvar(default.dlg.box_expr_ll_net.gen_inv_arcs) 1
  set gvar(default.dlg.box_expr_ll_net.wide_output) 0
  set gvar(default.dlg.box_expr_ll_net.withtime) 0
  set gvar(default.dlg.box_expr_ll_net.rec_depth) 5
  set gvar(default.dlg.box_expr_ll_net.sync_depth) 5
  set gvar(default.dlg.box_expr_ll_net.loop_size) "small_loop"

    # Defaults and Options for B(PN) -> HL Net
  set gvar(default.dlg.bpn2_hl_net.check_net) 0
  set gvar(default.dlg.bpn2_hl_net.rem_dup) 1
  set gvar(default.dlg.bpn2_hl_net.dead_trans) 1
  set gvar(default.dlg.bpn2_hl_net.seq_term) 1
  set gvar(default.dlg.bpn2_hl_net.bpn2_names) 1
  set gvar(default.dlg.bpn2_hl_net.no_scoping) 0
  set gvar(default.dlg.bpn2_hl_net.scale) 1
  set gvar(default.dlg.bpn2_hl_net.loop_size) "small_loop"
  set gvar(default.dlg.bpn2_hl_net.transition_flag) 1
  set gvar(default.dlg.bpn2_hl_net.transition_number) 100
  set gvar(default.dlg.bpn2_hl_net.disable_blocks) 0
  set gvar(default.dlg.bpn2_hl_net.disable_phantoms) 0
  set gvar(default.dlg.bpn2_hl_net.use_set) 0
  set gvar(default.dlg.bpn2_hl_net.no_disjunct_idsets) 0
  set gvar(default.dlg.bpn2_hl_net.procedure_box_flag) 0
  set gvar(default.dlg.bpn2_hl_net.procedure_box_number) 0
  set gvar(default.dlg.bpn2_hl_net.channel_box_flag) 0
  set gvar(default.dlg.bpn2_hl_net.channel_box_number) 0
  set gvar(default.dlg.bpn2_hl_net.no_warnings_old_syntax) 1
  set gvar(default.dlg.bpn2_hl_net.allow_fragments) 0
  set gvar(default.dlg.bpn2_hl_net.always_procedures) 0
  set gvar(default.dlg.bpn2_hl_net.procedure_instances_flag) 0
  set gvar(default.dlg.bpn2_hl_net.procedure_instances_number) 3

    # Defaults and Options for HL Net -> LL Net
  set gvar(default.dlg.hl_net_ll_net.remove_edges) 1
  set gvar(default.dlg.hl_net_ll_net.disable_blocks) 0
  set gvar(default.dlg.hl_net_ll_net.disable_phantoms) 0

    # Defaults and Options for Sync HL Net
  set gvar(default.dlg.sync_hl_net.rem_dup) 1
  set gvar(default.dlg.sync_hl_net.dead_trans) 1
  set gvar(default.dlg.sync_hl_net.bpn2_names) 1
  set gvar(default.dlg.sync_hl_net.no_sync) 0
  set gvar(default.dlg.sync_hl_net.disable_blocks) 0
  set gvar(default.dlg.sync_hl_net.disable_phantoms) 0
  set gvar(default.dlg.sync_hl_net.check_net) 0
  set gvar(default.dlg.sync_hl_net.no_restriction) 0
  set gvar(default.dlg.sync_hl_net.transition_flag) 1
  set gvar(default.dlg.sync_hl_net.transition_number) 100
  set gvar(default.dlg.sync_hl_net.max_vars_arc_flag) 0
  set gvar(default.dlg.sync_hl_net.max_vars_arc_number) 10

    # Defaults and Options for unfolder
  set gvar(default.dlg.unfolding_text.unfolder) $gvar(ll_net_to_unfolding.name)
  set gvar(default.dlg.unfolding_text.display_time) 0
  set gvar(default.dlg.unfolding_text.display_progress) 0
  set gvar(default.dlg.unfolding_text.use_bit_arrays) 1
  set gvar(default.dlg.unfolding_text.b_conflict_algorithm_flag) 1
  set gvar(default.dlg.unfolding_text.b_conflict_algorithm_number) 1
  set gvar(default.dlg.unfolding_text.max_events_flag) 0
  set gvar(default.dlg.unfolding_text.max_events_number) 10000
  set gvar(default.dlg.unfolding_text.display_statistics) 1
  set gvar(default.dlg.unfolding_text.display_version) 0
  set gvar(default.dlg.unfolding_text.take_last_events) 0
  set gvar(default.dlg.unfolding_text.take_current_marking) 0
  set gvar(default.dlg.unfolding_text.display_unused_transitions) 0
  set gvar(default.dlg.unfolding_text.minimize_hl_net) 1
  set gvar(default.dlg.unfolding_text.minimize_net) 1
  set gvar(default.dlg.unfolding_text.use_esparza_vogler) 1
  set gvar(default.dlg.unfolding_text.time_unfolding) 0
  set gvar(default.dlg.unfolding_text.time_unfolding_optimized) 0 

    # Defaults and Options for boundedness
  set gvar(default.dlg.ana_bounded.display_statistics) 1
  set gvar(default.dlg.ana_bounded.display_progress) 1
  set gvar(default.dlg.ana_bounded.display_version) 0

    # Defaults and Options for reachability
  set gvar(default.dlg.ana_reachability.calc_all) 0
  set gvar(default.dlg.ana_reachability.calc_part_mark) 1

    # Defaults and Options for LP-Analysis
  set gvar(default.dlg.ana_lp.considered_prop) 0
  set gvar(default.dlg.ana_lp.use_cplex) 0
  set gvar(default.dlg.ana_lp.solve_noint_lp) 0
  set gvar(default.dlg.ana_lp.use_ezpeleta) 0
  set gvar(default.dlg.ana_lp.objective_func_coeff_power) 0
  set gvar(default.dlg.ana_lp.objective_func_coeff_power_flag) 0
  set gvar(default.dlg.ana_lp.write_lp_file) 0
  set gvar(default.dlg.ana_lp.write_mps_file) 0
  set gvar(default.dlg.ana_lp.do_nothing) 0
  set gvar(default.dlg.ana_lp.display_debug) 0
  set gvar(default.dlg.ana_lp.use_perturbation) 0
  set gvar(default.dlg.ana_lp.use_max_token_number) 1
  set gvar(default.dlg.ana_lp.max_token_number) 1
  set gvar(default.dlg.ana_lp.lp_solve_coeff) 0
  set gvar(default.dlg.ana_lp.lp_solve_coeff_flag) 0
#  set gvar(default.dlg.ana_lp.use_parikh_vector_heuristics) 0
  set gvar(default.dlg.ana_lp.use_prefix) 0
  set gvar(default.dlg.ana_lp.use_invariants) 0

  # Defaults and Options for clp
  set gvar(default.dlg.ana_clp.safe_net) 1
  set gvar(default.dlg.ana_clp.dead_tokens_allowed) 0
  set gvar(default.dlg.ana_clp.shortest_trail) 0
  set gvar(default.dlg.ana_clp.save_mem) 0
  # default is 0 for coverability, 1 is reachability, 
  # 2 extended reachability
  set gvar(default.dlg.ana_clp.mode) 0
  set gvar(default.dlg.ana_clp.handle_terminal_marking) 0

  #Defaults for mcsmodels
  set gvar(default.dlg.ana_mcsmodels.optimize) 3
  set gvar(default.dlg.ana_mcsmodels.nolookahead) 1

  #Defaults for boundsmodels
  set gvar(default.dlg.ana_boundsmodels.bound) 5
  set gvar(default.dlg.ana_boundsmodels.semantics) step
  set gvar(default.dlg.ana_boundsmodels.optimize) full

    # Defaults and Options for Model-Checker Safe
  set gvar(default.dlg.model_check_safe.force_calc) 0
  set gvar(default.dlg.model_check_safe.disp_fire_seqs) 1
  set gvar(default.dlg.model_check_safe.only_min_fire_seqs) 1
  set gvar(default.dlg.model_check_safe.log_results) 0

    # Defaults and Options for Model-Checker SPIN
  set gvar(default.dlg.model_check_spin.po_typ) -1
  set gvar(default.dlg.model_check_spin.vb_typ) 0
  set gvar(default.dlg.model_check_spin.pr_typ) 0
  set gvar(default.dlg.model_check_spin.vr_typ) 0
  set gvar(default.dlg.model_check_spin.et_typ) 1
  set gvar(default.dlg.model_check_spin.st_typ) 0
  set gvar(default.dlg.model_check_spin.se_typ) 0
  set gvar(default.dlg.model_check_spin.err_nr) 1
  set gvar(default.dlg.model_check_spin.an_typ) -1
  set gvar(default.dlg.model_check_spin.cyc_typ) 0
  set gvar(default.dlg.model_check_spin.nv_typ) 1
  set gvar(default.dlg.model_check_spin.ur_typ) 1
  set gvar(default.dlg.model_check_spin.xu_typ) 1
  set gvar(default.dlg.model_check_spin.ct_typ) 0
  set gvar(default.dlg.model_check_spin.oct_typ) -1
  set gvar(default.dlg.model_check_spin.ol_typ) -1
  set gvar(default.dlg.model_check_spin.l_typ) 0
  set gvar(default.dlg.model_check_spin.tl_stat) 0
  set gvar(default.dlg.model_check_spin.max_mem) 32
  set gvar(default.dlg.model_check_spin.state_space) 500
  set gvar(default.dlg.model_check_spin.max_search_depth) 10000
  set gvar(default.dlg.model_check_spin.comp_time) ""
  set gvar(default.dlg.model_check_spin.run_time) ""
  set gvar(default.dlg.model_check_spin.log_results) 0

    # Defaults and Options for Model-Checker SMV
  set gvar(default.dlg.model_check_smv.switch) 5
  set gvar(default.dlg.model_check_smv.inv) 2
  set gvar(default.dlg.model_check_smv.invariants_check) 0
  set gvar(default.dlg.model_check_smv.additional) "-f"
  set gvar(default.dlg.model_check_smv.log_results) 0

    # Defaults and Options for B(PN)-Expander
  set gvar(default.dlg.bpn2_exp.std_init) 0
  set gvar(default.dlg.bpn2_exp.rec_depth) 5

    # Defaults and Options for pep2ina
  set gvar(default.dlg.pep_to_ina.references) 1
  set gvar(default.dlg.pep_to_ina.colored) 0
  set gvar(default.dlg.pep_to_ina.withtime) 0

    # Defaults and Options for pep2spin
  set gvar(default.dlg.pep_to_spin.safe) 1

    # Defaults and Options for pfa_hl_net
  set gvar(default.dlg.pfa_hl_net.no_scoping) 0
  set gvar(default.dlg.pfa_hl_net.disable_blocks) 0
  set gvar(default.dlg.pfa_hl_net.disable_phantoms) 0
  set gvar(default.dlg.pfa_hl_net.procedure_net_version_flag) 0
  set gvar(default.dlg.pfa_hl_net.procedure_net_version) 0
  set gvar(default.dlg.pfa_hl_net.scale) 1
  set gvar(default.dlg.pfa_hl_net.use_bpn2_proc_sem) 0
  set gvar(default.dlg.pfa_hl_net.bpn2_proc_sem_idset) "1..2"

    # Defaults and Options for pfa_bpn2
  set gvar(default.dlg.pfa_bpn2.struct) 1

    # Defaults and Options for sdl_hl_net
  set gvar(default.dlg.sdl_hl_net.no_scoping) 0
  set gvar(default.dlg.sdl_hl_net.no_implicit_local_procedure_subtypes) 0
#  set gvar(default.dlg.sdl_hl_net.no_var_rename) 0
  set gvar(default.dlg.sdl_hl_net.channel_net_version) 0
  set gvar(default.dlg.sdl_hl_net.process_net_version) 0
  set gvar(default.dlg.sdl_hl_net.default_channel_capacity) 1
  set gvar(default.dlg.sdl_hl_net.default_maximum_process_instances) 1
  set gvar(default.dlg.sdl_hl_net.default_input_queue_capacity) 2
  set gvar(default.dlg.sdl_hl_net.default_maximum_procedure_instances) 1

 foreach compiler {syntax_check bpn2_box_expr box_expr_ll_net\
    bpn2_hl_net hl_net_ll_net sync_hl_net unfolding_text ana_bounded\
    ana_reachability ana_lp ana_clp model_check_safe model_check_spin\
    model_check_smv bpn2_exp pep_to_ina pep_to_spin pfa_hl_net\
    pfa_bpn2 sdl_hl_net ana_boundsmodels ana_mcsmodels} {
   dlg_option_set_defaults $compiler
 }

}  ;# init_options_dialog_values


  #################################################################
  # Functions for calculation of the argument strings from values #
  #################################################################

proc get_options_syntax_check {} {
  global gvar

  return $gvar(option.dlg.syntax_check.skip)
}  ;# get_options_syntax_check

proc get_options_bpn2_box_expr {} {
  global gvar

    # Start with empty string.
  set res {}

    # Append option flags to the end.
  if {$gvar(option.dlg.bpn2_box_expr.seq_term)} {
    set res "$res -t"}
  if {$gvar(option.dlg.bpn2_box_expr.std_init)} {
    set res "$res -i"}
  if {$gvar(option.dlg.bpn2_box_expr.no_warn_old_syntax)} {
    set res "$res -o"}

    # Return option string.
  return $res
}  ;# get_options_bpn2_box_expr

proc get_options_box_expr_ll_net {} {
  global gvar

    # Start with empty string.
  set res {}

    # Append option flags to the end.
  if {$gvar(option.dlg.box_expr_ll_net.check_net)} {
    set res "$res -c"}
  if {$gvar(option.dlg.box_expr_ll_net.rem_dup)} {
    set res "$res -r"}
  if {$gvar(option.dlg.box_expr_ll_net.bundle_atomic_actions)} {
    set res "$res -b"}
  if {$gvar(option.dlg.box_expr_ll_net.simple_cd_boxes)} {
    set res "$res -s"}
  if {$gvar(option.dlg.box_expr_ll_net.ext_label)} {
    set res "$res -e"}
  if {$gvar(option.dlg.box_expr_ll_net.gen_inv_arcs)} {
    set res "$res -v"}
  if {$gvar(option.dlg.box_expr_ll_net.wide_output)} {
    set res "$res -w"}
  if {$gvar(option.dlg.box_expr_ll_net.withtime)} {
    set res "$res -t"}
  set res "$res -R$gvar(option.dlg.box_expr_ll_net.rec_depth)"
  set res "$res -S$gvar(option.dlg.box_expr_ll_net.sync_depth)"
  switch $gvar(option.dlg.box_expr_ll_net.loop_size) {
    "small_loop"   {}
    "medium_loop"  {set res "$res -q"}
    "big_loop"     {set res "$res -p"}
  }  ;# switch Loop-Size

    # Return option string.
  return $res
}  ;# get_options_box_expr_ll_net

proc get_options_bpn2_hl_net {} {
  global gvar

    # Start with empty string.
  set res {}

    # Append option flags to the end.
  if {$gvar(option.dlg.bpn2_hl_net.check_net)} {
    set res "$res -c"}
  if {$gvar(option.dlg.bpn2_hl_net.rem_dup)} {
    set res "$res -r"}
  if {$gvar(option.dlg.bpn2_hl_net.dead_trans)} {
    set res "$res -e"}
  if {!$gvar(option.dlg.bpn2_hl_net.seq_term)} {
    set res "$res -t"}
  if {$gvar(option.dlg.bpn2_hl_net.bpn2_names)} {
    set res "$res -n"}
  if {$gvar(option.dlg.bpn2_hl_net.no_scoping)} {
    set res "$res -!"}
  if {$gvar(option.dlg.bpn2_hl_net.transition_flag)} {
    set res "$res -m$gvar(option.dlg.bpn2_hl_net.transition_number)"}
  set res "$res -s$gvar(option.dlg.bpn2_hl_net.scale)"
  switch $gvar(option.dlg.bpn2_hl_net.loop_size) {
    "small_loop"   {}
    "medium_loop"  {set res "$res -q"}
    "big_loop"     {set res "$res -l"}
  }  ;# switch Loop-Size
  if {!$gvar(option.dlg.bpn2_hl_net.disable_blocks)} {
    set res "$res -b"}
  if {!$gvar(option.dlg.bpn2_hl_net.disable_phantoms)} {
    set res "$res -x"}
  if {!$gvar(option.dlg.bpn2_hl_net.use_set)} {
    set res "$res -u"}
  if {$gvar(option.dlg.bpn2_hl_net.no_disjunct_idsets)} {
    set res "$res -J"}
  if {$gvar(option.dlg.bpn2_hl_net.procedure_box_flag)} {
    set res "$res -P$gvar(option.dlg.bpn2_hl_net.procedure_box_number)"}
  if {$gvar(option.dlg.bpn2_hl_net.channel_box_flag)} {
    set res "$res -C$gvar(option.dlg.bpn2_hl_net.channel_box_number)"}
  if {$gvar(option.dlg.bpn2_hl_net.no_warnings_old_syntax)} {
    set res "$res -o"}
  if {$gvar(option.dlg.bpn2_hl_net.allow_fragments)} {
    set res "$res -a"}
  if {$gvar(option.dlg.bpn2_hl_net.always_procedures)} {
    set res "$res -g"}
  if {$gvar(option.dlg.bpn2_hl_net.procedure_instances_flag)} {
    set res "$res -i$gvar(option.dlg.bpn2_hl_net.procedure_instances_number)"}

    # Return option string.
  return $res
}  ;# get_options_bpn2_hl_net

proc get_options_hl_net_ll_net {} {
  global gvar

    # Start with empty string.
  set res {}

    # Append option flags to the end.
  if {$gvar(option.dlg.hl_net_ll_net.remove_edges)} {
    set res "$res - "}
  if {!$gvar(option.dlg.hl_net_ll_net.disable_blocks)} {
    set res "$res -b"}
  if {!$gvar(option.dlg.hl_net_ll_net.disable_phantoms)} {
    set res "$res -p"}

    # Return option string.
  return $res
}  ;# get_options_hl_net_ll_net

proc get_options_sync_hl_net {} {
  global gvar

    # Start with empty string.
  set res {}

    # Append option flags to the end.
  if {$gvar(option.dlg.sync_hl_net.check_net)} {
    set res "$res -c"}
  if {$gvar(option.dlg.sync_hl_net.rem_dup)} {
    set res "$res -r"}
  if {$gvar(option.dlg.sync_hl_net.dead_trans)} {
    set res "$res -e"}
  if {$gvar(option.dlg.sync_hl_net.bpn2_names)} {
    set res "$res -n"}
  if {$gvar(option.dlg.sync_hl_net.no_sync)} {
    set res "$res -!"}
  if {!$gvar(option.dlg.sync_hl_net.no_restriction)} {
    set res "$res -d"}
  if {$gvar(option.dlg.sync_hl_net.transition_flag)} {
    set res "$res -m$gvar(option.dlg.sync_hl_net.transition_number)"}
  if {$gvar(option.dlg.sync_hl_net.max_vars_arc_flag)} {
    set res "$res -v$gvar(option.dlg.sync_hl_net.max_vars_arc_number)"}
  if {!$gvar(option.dlg.sync_hl_net.disable_blocks)} {
    set res "$res -b"}
  if {!$gvar(option.dlg.sync_hl_net.disable_phantoms)} {
    set res "$res -x"}

    # Return option string.
  return $res
}  ;# get_options_sync_hl_net

proc get_options_unfolding_text \
 {unused_transitions_filename hl_ll_ref_filename \
  unused_transitions_flag \
  minimized_hl_net_flag minimized_net_flag} {
  # Returns program option string according to chosen options.
  # unused_transitions_filename - file name for list of unused transitions (IN)
  # unused_transitions_flag - true, if list of unused transitions should be
  #   written (OUT)
  # minimized_net_flag - true, if minimized net should be written (OUT)
  # minimized_hl_net_flag - dito for hl nets (OUT)
  global gvar
  upvar $unused_transitions_flag utf
  upvar $minimized_hl_net_flag hl_mnf
  upvar $minimized_net_flag mnf

    # Start with empty string.
  set res {}

  #no options
  if {$gvar(option.dlg.unfolding_text.unfolder)!=$gvar(ll_net_to_unfolding.name)} {
    set utf false
    set mnf false
    set hl_mnf false
    return
  } 
    # Append option flags to the end.
  if {$gvar(option.dlg.unfolding_text.display_time)} {
    set res "$res -t"}
  if {$gvar(option.dlg.unfolding_text.display_progress)} {
    set res "$res -q"}
  if {$gvar(option.dlg.unfolding_text.use_bit_arrays)} {
    set res "$res -b"}
  if {$gvar(option.dlg.unfolding_text.b_conflict_algorithm_flag)} {
    set res \
      "$res -n=$gvar(option.dlg.unfolding_text.b_conflict_algorithm_number)"}
  if {$gvar(option.dlg.unfolding_text.max_events_flag)} {
    set res "$res -x=$gvar(option.dlg.unfolding_text.max_events_number)"}
  if {$gvar(option.dlg.unfolding_text.display_statistics)} {
    set res "$res -s"}
  if {$gvar(option.dlg.unfolding_text.display_version)} {
    set res "$res -v"}
  if {$gvar(option.dlg.unfolding_text.take_last_events)} {
    set res "$res -l"}
  if {$gvar(option.dlg.unfolding_text.take_current_marking)} {
    set res "$res -M"}
  if {$gvar(option.dlg.unfolding_text.use_esparza_vogler)} {
    set res "$res -@"}
  if {$gvar(option.dlg.unfolding_text.time_unfolding)} {
    set res "$res -X"}
#  if {$gvar(option.dlg.unfolding_text.time_unfolding) && $gvar(option.dlg.unfolding_text.time_unfolding_optimized)} {
#    set res "$res+"}

# TODO
# BG tried to use simplify
# always use unused_transitions_filename, if something is minimized.
  if {$gvar(option.dlg.unfolding_text.display_unused_transitions)} {
    set res "$res -T=$unused_transitions_filename"
    set utf true
    if {$gvar(option.dlg.unfolding_text.minimize_net)} {
      set mnf true
    } else {
      set mnf false
    }
    if {$gvar(option.dlg.unfolding_text.minimize_hl_net)} {
      set hl_mnf true
    } else {
      set hl_mnf false
    } 
  } else {
    set utf false
    if {$gvar(option.dlg.unfolding_text.minimize_net)} {
      set res "$res -T=$unused_transitions_filename"
      set mnf true
      if {$gvar(option.dlg.unfolding_text.minimize_hl_net)} {
        set hl_mnf true
      } else {
        set hl_mnf false
      } 
    } else {
      set mnf false
      if {$gvar(option.dlg.unfolding_text.minimize_hl_net)} {
        set res "$res -T=$unused_transitions_filename"
        set hl_mnf true
      } else {
        set hl_mnf false
      } 
    }  ;# else, do nothing
  }  ;# if no putput of unused transitions
  # if HL is to be minimized without hl -> ll reference
  # nothing is done
  if {($hl_mnf=="true") && (![file exists $hl_ll_ref_filename])} {
    set hl_mnf false
    DEBUG "No Reference from HL to LL Net found. Cannot minimize HL Net."
  }

DEBUG "res=$res"
DEBUG "utf=$utf"
DEBUG "mnf=$mnf"
DEBUG "hl_mnf=$hl_mnf"
    # Return option string.
  return $res
}  ;# get_options_unfolding_text

proc get_options_ana_bounded {display_statistics_flag} {
  # Returns program option string according to chosen options.
  # display_statistics_flag - Flag for output of statistics (OUT)
  global gvar
  upvar $display_statistics_flag dsf

    # Start with empty string.
  set res {}

    # Append option flags to the end.
  if {$gvar(option.dlg.ana_bounded.display_statistics)} {
    set res "$res -s"}
  set dsf $gvar(option.dlg.ana_bounded.display_statistics)
  if {$gvar(option.dlg.ana_bounded.display_progress)} {
    set res "$res -p"}
  if {$gvar(option.dlg.ana_bounded.display_version)} {
    set res "$res -v"}

    # Return option string.
  return $res
}  ;# get_options_ana_bounded

proc get_options_ana_reachability {} {
  global gvar

    # Start with empty string.
  set res {}

    # Append option flags to the end.
  if {$gvar(option.dlg.ana_reachability.calc_all)} {
    set res "$res -A"}
  if {$gvar(option.dlg.ana_reachability.calc_part_mark)} {
    set res "$res -T"}

    # Return option string.
  return $res
}  ;# get_options_ana_reachability

proc get_options_ana_lp {{reachability_flag 0}} {
  # Returns program option string according to chosen options.
  # reachability_flag: 1 if reachability test, 0 for deadlock test
  #   was: Parameter -R or -H depending on ana_lp.use_parikh_vector_heuristics
  #   now: -D or -U dep. on ana_lp.use_prefix if not reachability
  #        and -R for reachability
  global gvar

    # Start with empty string.
  set res {}

    # Append option flags to the end.
  switch $gvar(option.dlg.ana_lp.considered_prop) {
    0  {}
    1  {set res "$res -t"}
    2  {set res "$res -i"}
  }
  if {$gvar(option.dlg.ana_lp.use_ezpeleta)} {
    set res "$res -@"}
  if {$gvar(option.dlg.ana_lp.solve_noint_lp)} {
    set res "$res -s"}
  if {$gvar(option.dlg.ana_lp.use_cplex)} {
    set res "$res -y"}
  if {$gvar(option.dlg.ana_lp.objective_func_coeff_power_flag)} {
    set coeff \
      [expr pow(10, $gvar(option.dlg.ana_lp.objective_func_coeff_power))]
    set res "$res -c=$coeff"
  }

   if {$gvar(option.dlg.ana_lp.write_lp_file)} {
	 set res "$res -w"
   }
   if {$gvar(option.dlg.ana_lp.write_mps_file)} {
	 set res "$res -v"
   }

  if {$gvar(option.dlg.ana_lp.do_nothing)} {
    set res "$res -x"}
  if {$gvar(option.dlg.ana_lp.display_debug)} {
    set res "$res -d"}
  if {$gvar(option.dlg.ana_lp.use_perturbation)} {
    set res "$res -a"}
  if {$gvar(option.dlg.ana_lp.lp_solve_coeff_flag)} {
    set coeff_lp_solve \
      [expr pow(10, $gvar(option.dlg.ana_lp.lp_solve_coeff))]
    set res "$res -e=$coeff_lp_solve"
  }
  if {$reachability_flag} {
     set res "$res -R"
  } else {
     if {$gvar(option.dlg.ana_lp.use_prefix)} {
        set res "$res -U"
     } else {
        set res "$res -D"
     }  ;# else use Prefix or plain LL Net
  }  ;# if reachability

  if {$gvar(option.dlg.ana_lp.use_max_token_number)} {
     set res "$res -q=$gvar(option.dlg.ana_lp.max_token_number)"
  }

    # Return option string.
  return $res
}  ;# get_options_ana_lp

proc get_options_ana_clp {reachability_flag} {
  # Returns program option string according to chosen options.
  # reachability_flag - specify mode

  global gvar

    # Start with empty string.
  set res {}

    # Append option flags to the end.
  if {$gvar(option.dlg.ana_clp.shortest_trail)} {
    set res "$res -t"}
  if {$gvar(option.dlg.ana_clp.safe_net)} {
    set res "$res -s"}
  if {$gvar(option.dlg.ana_clp.save_mem)} {
    set res "$res -M"}
  switch $reachability_flag {
      0 {
	  if {$gvar(option.dlg.ana_clp.handle_terminal_marking)} {
	      if {$gvar(option.dlg.ana_clp.dead_tokens_allowed)} {
		  set res "$res -i"	
	      }
	      set res "$res -d"  
	  }
      }
      1 {
	  switch $gvar(option.dlg.ana_clp.mode) {
	      0  {set res "$res -c"}
	      1  {set res "$res -r"}
	      2  {set res "$res -e"}
	  }
      }
      2 {
	  set res "$res -m"
      }
  }

    # Return option string.
  return $res
}  ;# get_options_ana_clp

proc get_options_mcsmodels {deadlock} {
  # Create command line options for mcsmodels
  # deadlock is true if generating options for deadlock checking

  global gvar

  set res "-O$gvar(option.dlg.ana_mcsmodels.optimize)"
  if {$gvar(option.dlg.ana_mcsmodels.nolookahead)} {
    set res "$res -n"
  }
  return $res
}

proc get_options_boundsmodels {deadlock} {
  # Create command line options for boundsmodels
  # deadlock is true if generating options for deadlock checking

  global gvar

  set res "-b=$gvar(option.dlg.ana_boundsmodels.bound)"
  if {$gvar(option.dlg.ana_boundsmodels.semantics)=="interleave"} {
    set res "$res -i"
  } else {
    set res "$res -s"
  }
  switch $gvar(option.dlg.ana_boundsmodels.optimize) {
    "none" {set res "$res -O0"}
    "simple" {set res "$res -O1"}
    "full" {set res "$res -O2"}
  }
  return $res
}

proc get_options_model_check_safe {display_firing_sequences_flag} {
  # Returns program option string according to chosen options.
  # display_firing_sequences_flag - true if sequences are to be
  # displayed. Needed for filename generation.
  global gvar
  upvar $display_firing_sequences_flag dfsf

    # Start with empty string.
  set res {}

    # Append option flags to the end.
  if {$gvar(option.dlg.model_check_safe.force_calc)} {
    set res "$res -c"}
  set dfsf $gvar(option.dlg.model_check_safe.disp_fire_seqs)
  if {$gvar(option.dlg.model_check_safe.disp_fire_seqs)
    && $gvar(option.dlg.model_check_safe.only_min_fire_seqs)} {
    set res "$res -m"}

    # Return option string.
  return $res
}  ;# get_options_model_check_safe

proc init_options_var_model_check_spin {prjnum} {
  # Initialisierung der Optionsvariablen, je nachdem
  # was in der Promela-Datei vorkommt
  global gvar

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"

  if {$gvar(option.dlg.model_check_spin.an_typ)==-1} {
    set gvar(option.dlg.model_check_spin.an_typ) 0
    set gvar(option.dlg.model_check_spin.nv_typ) [pattern_in_file $prjx(promela_form.filename) "never"]
    if {$gvar(option.dlg.model_check_spin.nv_typ)==1} {
      if {[pattern_in_file $prjx(promela_form.filename) "accept.*:"]} {
        set gvar(option.dlg.model_check_spin.an_typ) 2
      }
    } elseif {[pattern_in_file $prjx(promela_form.filename) "progress.*:"]} {
        set gvar(option.dlg.model_check_spin.an_typ) 1
    }
  }
  if {$gvar(option.dlg.model_check_spin.po_typ)==-1} {
    if {[pattern_in_file $prjx(promela_form.filename) "_last"] \
    ||  [pattern_in_file $prjx(promela_form.filename) "provided.*\\("] \
    ||  [pattern_in_file $prjx(promela_form.filename) "enabled\\("]} {
      set gvar(option.dlg.model_check_spin.po_typ) 0
    } else {
      set gvar(option.dlg.model_check_spin.po_typ) 1
    }
  }
}  ;# init_options_var_model_check_spin


proc get_a_options_model_check_spin {prjnum} {
  # Returns program option string according to chosen options.
  global gvar

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"

    # Start with empty string.
  set res {}

  set res "$res -a -X"
  if {$gvar(option.dlg.model_check_spin.l_typ)==1} {
    set res "$res -m"
  }

    # Return option string.
  return $res
}  ;# get_a_options_model_check_spin


proc get_c_options_model_check_spin {prjnum} {
  # Returns program option string according to chosen options.
  global gvar

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"

    # Start with empty string.
  set res {}

  set gvar(option.dlg.model_check_spin.ol_typ) $gvar(option.dlg.model_check_spin.l_typ)
  set gvar(option.dlg.model_check_spin.oct_typ) $gvar(option.dlg.model_check_spin.ct_typ)

    # the Mbytes scale resolution is in millions: 2^20
  set Mbytes $gvar(option.dlg.model_check_spin.max_mem)
  set Mbytes [expr 20 + log($Mbytes) / log(2) ]
  set foo [string first "." $Mbytes]
  if {$foo >= 0} {
		incr foo -1
		set Mbytes [string range $Mbytes 0 $foo]
  }
  unset foo

    # the Mstate scale resolution is in thousands: 2^10
  set Mstate [expr 10+[log $gvar(option.dlg.model_check_spin.state_space)]]
  set Mdepth $gvar(option.dlg.model_check_spin.max_search_depth)

  if {$gvar(option.dlg.model_check_spin.ct_typ)==1} {
    set res "$res -DBITSTATE -DMEMCNT=$Mbytes"
     # in supertrace mode: #bits ~~ 128x #states
     # (effectively the #bytes will be ~~ 16x #states)
    set Mstate [expr 7+$Mstate]
  } elseif {$gvar(option.dlg.model_check_spin.ct_typ)==2} {
    set res "$res -DHC -DMEMCNT=$Mbytes"
     # in hash-compact mode
  } else {
    set res "$res -DMEMCNT=$Mbytes"
     # in exhaustive mode: #hashtable ~~ #states
  }

  set res "-D_POSIX_SOURCE $res"

    # Append option flags to the end.
  if {$gvar(option.dlg.model_check_spin.an_typ)==0} {
    set res "$res -DSAFETY"}
  if {$gvar(option.dlg.model_check_spin.an_typ)==1} {
    set res "$res -DNP"}
  if {$gvar(option.dlg.model_check_spin.po_typ)==0} {
    set res "$res -DNOREDUCE"}
  if {$gvar(option.dlg.model_check_spin.vb_typ)==1} {
    set res "$res -DCHECK"}
  if {$gvar(option.dlg.model_check_spin.nv_typ)==0} {
    set res "$res -DNOCLAIM"}
  if {$gvar(option.dlg.model_check_spin.se_typ)!=0} {
    set res "$res -DREACH"}
  if {$gvar(option.dlg.model_check_spin.xu_typ)==0} {
    set res "$res -DXUSAFE"}
  if {$gvar(option.dlg.model_check_spin.pr_typ)==1} {
    set res "$res -DPEG"}
  if {$gvar(option.dlg.model_check_spin.vr_typ)==1} {
    set res "$res -DVAR_RANGES"}
  if {$gvar(option.dlg.model_check_spin.cyc_typ)==1} {
    set res "$res -DNFAIR=3"
  } else {
    set res "$res -DNOFAIR"}

  set res "$res $gvar(option.dlg.model_check_spin.comp_time)"

    # Return option string.
  return $res
}  ;# get_c_options_model_check_spin

proc get_v_options_model_check_spin {prjnum} {
  # Returns program option string according to chosen options.
  global gvar

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"

    # Start with empty string.
  set res {}

    # the Mbytes scale resolution is in millions: 2^20
  set Mbytes $gvar(option.dlg.model_check_spin.max_mem)
  set Mbytes [expr 20 + log($Mbytes) / log(2) ]
  set foo [string first "." $Mbytes]
  if {$foo >= 0} {
		incr foo -1
		set Mbytes [string range $Mbytes 0 $foo]
  }
  unset foo

    # the Mstate scale resolution is in thousands: 2^10
  set Mstate [expr 10+[log $gvar(option.dlg.model_check_spin.state_space)]]
  set Mdepth $gvar(option.dlg.model_check_spin.max_search_depth)

  if {$gvar(option.dlg.model_check_spin.ct_typ)==1} {
     # in supertrace mode: #bits ~~ 128x #states
     # (effectively the #bytes will be ~~ 16x #states)
    set Mstate [expr 7+$Mstate]
    set res "$res -X -m$Mdepth -w$Mstate"
  } elseif {$gvar(option.dlg.model_check_spin.ct_typ)==2} {
    set res "$res -X -m$Mdepth -w$Mstate"
     # in hash-compact mode
  } else {
     # in exhaustive mode: #hashtable ~~ #states
    set res "$res -X -m$Mdepth -w$Mstate"
  }

  set res "$res $gvar(option.dlg.model_check_spin.run_time)"

    # Append option flags to the end.
  if {$gvar(option.dlg.model_check_spin.an_typ)==1} {
    set res "$res -l"}
  if {$gvar(option.dlg.model_check_spin.an_typ)==2} {
    set res "$res -a"}
  if {$gvar(option.dlg.model_check_spin.cyc_typ)==1} {
    set res "$res -f"}
  if {$gvar(option.dlg.model_check_spin.ur_typ)==0} {
    set res "$res -n"}
  if {$gvar(option.dlg.model_check_spin.st_typ)==1} {
    set res "$res -e"}
  if {$gvar(option.dlg.model_check_spin.se_typ)==1} {
    set res "$res -I"}
  if {$gvar(option.dlg.model_check_spin.se_typ)==2} {
    set res "$res -i"}
  if {$gvar(option.dlg.model_check_spin.et_typ)==0} {
    set res "$res -c0"}
  if {$gvar(option.dlg.model_check_spin.et_typ)==1} {
    set res "$res -c$gvar(option.dlg.model_check_spin.err_nr)"}

    # Return option string.
  return $res
}  ;# get_v_options_model_check_spin

proc get_options_bpn2_exp {} {
  global gvar

    # Start with empty string.
  set res {}

    # Append option flags to the end.
  if {$gvar(option.dlg.bpn2_exp.std_init)} {
    set res "$res -i"}
  set res "$res -r$gvar(option.dlg.bpn2_exp.rec_depth)"

    # Return option string.
  return $res
}  ;# get_options_bpn2_exp

proc get_options_pep_to_ina {} {
  global gvar

    # Start with empty string.
  set res {}

    # Append option flags to the end.
  if {$gvar(option.dlg.pep_to_ina.references)} {
    set res "$res -r"}
  if {$gvar(option.dlg.pep_to_ina.colored)} {
    set res "$res -c"}
  if {$gvar(option.dlg.pep_to_ina.withtime)} {
    set res "$res -t"}

    # Return option string.
  return $res
}  ;# get_options_pep_to_ina

proc get_options_pep_to_spin {} {
  global gvar

    # Start with empty string.
  set res {}

    # Append option flags to the end.
  if {$gvar(option.dlg.pep_to_spin.safe)} {
    set res "$res -s"}

    # Return option string.
  return $res
}  ;# get_options_pep_to_spin

proc get_options_pfa_hl_net {} {
  global gvar

    # Start with empty string.
  set res {}

  if {!$gvar(option.dlg.pfa_hl_net.disable_blocks)} {
    set res "$res -b"}
  if {!$gvar(option.dlg.pfa_hl_net.disable_phantoms)} {
    set res "$res -p"}
  set res "$res -s$gvar(option.dlg.pfa_hl_net.scale)"
  if {$gvar(option.dlg.pfa_hl_net.no_scoping)} {
    set res "$res -S"}
  if {$gvar(option.dlg.pfa_hl_net.use_bpn2_proc_sem)} {
    set res "$res -g$gvar(option.dlg.pfa_hl_net.bpn2_proc_sem_idset)"}
  if {$gvar(option.dlg.pfa_hl_net.procedure_net_version_flag)} {
      set res "$res -u$gvar(option.dlg.pfa_hl_net.procedure_net_version)"}

    # Return option string.
  return $res
}  ;# get_options_pfa_hl_net

proc get_options_pfa_bpn2 {} {
  global gvar

  set res {}

  if {$gvar(option.dlg.pfa_bpn2.struct)} {
      set res "$res -n"}
  
  return $res
} ;# get_options_pfa_bpn2

proc get_options_sdl_hl_net {} {
  global gvar

  set res {}

#  if {$gvar(option.dlg.sdl_hl_net.no_var_rename)} {
#    set res "$res -n"
#  }
  if {$gvar(option.dlg.sdl_hl_net.no_scoping)} {
    set res "$res -S"
  }
  if {$gvar(option.dlg.sdl_hl_net.no_implicit_local_procedure_subtypes)} {
    set res "$res -L"
  }
  set res "$res -BC$gvar(option.dlg.sdl_hl_net.channel_net_version)"
  set res "$res -BP$gvar(option.dlg.sdl_hl_net.process_net_version)"
  set res "$res -DC$gvar(option.dlg.sdl_hl_net.default_channel_capacity)"
  set res "$res -DP$gvar(option.dlg.sdl_hl_net.default_maximum_process_instances)"
  set res "$res -DQ$gvar(option.dlg.sdl_hl_net.default_input_queue_capacity)"
  set res "$res -DZ$gvar(option.dlg.sdl_hl_net.default_maximum_procedure_instances)"
  return $res
} ;# get_options_sdl_hl_net

  ######################################
  # And finally the dialogs themselves #
  ######################################

proc restore_options {module} {
  global gvar

  set options [array names gvar "option.dlg.$module.*"]
  foreach name $options {
    set gvar($name) $gvar(save.$name)
  }
}

proc buttonframe {dialog module prjnum} {
    # Container for commandbuttons
  global gvar

  frame $dialog.commands -relief raised -bd 1
  pack $dialog.commands -side bottom -fill both
    # Commandbuttons
  button $dialog.commands.ok -text "Ok" \
    -command "del_dlg_widget $prjnum $dialog; destroy $dialog"
  button $dialog.commands.defaults -text "Defaults" \
    -command "dlg_option_set_defaults $module"
  button $dialog.commands.cancel -text "Cancel" \
    -command "restore_options $module;del_dlg_widget $prjnum $dialog; 
    destroy $dialog"

  # save current settings for restoring when cancelling  
  set options [array names gvar "option.dlg.$module.*"]
  foreach name $options {
    set gvar(save.$name) $gvar($name) 
  }

  pack $dialog.commands.ok \
    $dialog.commands.defaults \
    $dialog.commands.cancel \
    -side left -expand 1
}
 
proc option_syntax_check_dlg {} {
  global gvar

  set prjnum $gvar(actprj)
    # new TopLevel
  if [catch {set d [toplevel .dlg_option_syntax_check$prjnum]}] {
    raise .dlg_option_syntax_check$prjnum
    return
  }  ;# if already open
  add_dlg_widget $prjnum $d
  wm title $d "Syntax Check"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container for the Options
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Options
  checkbutton $d.options.skip \
    -text "Skip Syntax Check" -anchor w \
    -variable gvar(option.dlg.syntax_check.skip)
  pack $d.options.skip \
    -side top -anchor w -fill x

  buttonframe $d syntax_check $prjnum
}  ;# option_syntax_check_dlg

proc option_bpn2_box_expr_dlg {} {
  global gvar

  set prjnum $gvar(actprj)
    # new TopLevel
  if [catch {set d [toplevel .dlg_option_bpn2_box_expr$prjnum]}] {
    raise .dlg_option_bpn2_box_expr$prjnum
    return
  }  ;# if already open
  add_dlg_widget $prjnum $d
  wm title $d "B(PN) -> PBC"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
  checkbutton $d.options.seq_term \
    -text "Sequential Termination" -anchor w \
    -variable gvar(option.dlg.bpn2_box_expr.seq_term)
  checkbutton $d.options.std_init \
    -text "Standard Initialisation" -anchor w \
    -variable gvar(option.dlg.bpn2_box_expr.std_init)
  checkbutton $d.options.no_warn_old_syntax \
    -text "Suppress Warnings (e.g. Old Syntax)" -anchor w \
    -variable gvar(option.dlg.bpn2_box_expr.no_warn_old_syntax)
  pack $d.options.seq_term \
    $d.options.std_init \
    $d.options.no_warn_old_syntax \
    -side top -anchor w -fill x

  buttonframe $d bpn2_box_expr $prjnum
}  ;# option_bpn2_box_expr_dlg

proc option_box_expr_ll_net_dlg {} {
  # Den Dialog oeffnen.
  global gvar

  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .dlg_option_box_expr_ll_net$prjnum]}] {
    raise .dlg_option_box_expr_ll_net$prjnum
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $d
  wm title $d "PBC -> LL Net"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
  checkbutton $d.options.check_net \
    -text "Check Net" -anchor w \
    -variable gvar(option.dlg.box_expr_ll_net.check_net)
  checkbutton $d.options.rem_dup \
    -text "Remove Duplicates" -anchor w \
    -variable gvar(option.dlg.box_expr_ll_net.rem_dup)
  checkbutton $d.options.bundle_atomic_actions \
    -text "Bundle Atomic Actions" -anchor w \
    -variable gvar(option.dlg.box_expr_ll_net.bundle_atomic_actions)
  checkbutton $d.options.simple_cd_boxes \
    -text "Simple C-&D-Boxes" -anchor w \
    -variable gvar(option.dlg.box_expr_ll_net.simple_cd_boxes)
  checkbutton $d.options.ext_label \
    -text "Extended Labelling" -anchor w \
    -variable gvar(option.dlg.box_expr_ll_net.ext_label)
  checkbutton $d.options.gen_inv_arcs \
    -text "Control / Data Arcs invisible" -anchor w \
    -variable gvar(option.dlg.box_expr_ll_net.gen_inv_arcs)
#  checkbutton $d.options.wide_output \
#    -text "Wide Output Format" -anchor w \
#    -variable gvar(option.dlg.box_expr_ll_net.wide_output)
  checkbutton $d.options.withtime \
    -text "Write Time Boxes" -anchor w \
    -variable gvar(option.dlg.box_expr_ll_net.withtime)
  scale $d.options.rec_depth \
    -label "Recursion Depth" \
    -from 0 -to 20 -length 250 -orient horizontal -relief ridge \
    -variable gvar(option.dlg.box_expr_ll_net.rec_depth)
  scale $d.options.sync_depth \
    -label "Synchronisation Depth" \
    -from 0 -to 20 -length 250 -orient horizontal -relief ridge \
    -variable gvar(option.dlg.box_expr_ll_net.sync_depth)
  radiobutton $d.options.small_loop \
    -text "Small Loop" -value "small_loop" -anchor w \
    -variable gvar(option.dlg.box_expr_ll_net.loop_size)
  radiobutton $d.options.medium_loop \
    -text "Medium Loop" -value "medium_loop" -anchor w \
    -variable gvar(option.dlg.box_expr_ll_net.loop_size)
  radiobutton $d.options.big_loop \
    -text "Big Loop" -value "big_loop" -anchor w \
    -variable gvar(option.dlg.box_expr_ll_net.loop_size)
  pack $d.options.check_net \
    $d.options.rem_dup \
    $d.options.bundle_atomic_actions \
    $d.options.simple_cd_boxes \
    $d.options.ext_label \
    $d.options.gen_inv_arcs \
    $d.options.withtime \
    $d.options.rec_depth \
    $d.options.sync_depth \
    $d.options.small_loop \
    $d.options.medium_loop \
    $d.options.big_loop \
    -side top -anchor w -fill x
#    $d.options.wide_output

  buttonframe $d box_expr_ll_net $prjnum
}  ;# option_box_expr_ll_net_dlg

proc option_bpn2_hl_net_dlg {} {
  # Den Dialog "offnen.
  global gvar

  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .dlg_option_bpn2_hl_net$prjnum]}] {
    raise .dlg_option_bpn2_hl_net$prjnum
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $d
  wm title $d "B(PN) -> HL Net"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container f"ur die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
  checkbutton $d.options.check_net \
    -text "Check Net" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.check_net)
  checkbutton $d.options.rem_dup \
    -text "Remove Duplicates" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.rem_dup)
  checkbutton $d.options.dead_trans \
    -text "Eliminate Dead Transitions" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.dead_trans)
  checkbutton $d.options.seq_term \
    -text "Use Sequential Termination" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.seq_term)
  checkbutton $d.options.bpn2_names \
    -text "Use B(PN) Names for Communication Variables" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.bpn2_names)
  checkbutton $d.options.no_scoping \
    -text "No Scoping" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.no_scoping)
  checkbutton $d.options.disable_blocks \
    -text "No Block Information" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.disable_blocks)
  checkbutton $d.options.disable_phantoms \
    -text "No Phantom Transitions" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.disable_phantoms)
  checkbutton $d.options.use_set \
    -text "Use Stack instead of Set" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.use_set)

  checkbutton $d.options.no_disjunct_idsets \
    -text "No disjunct Identifier Sets" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.no_disjunct_idsets)
  checkbutton $d.options.procedure_box_flag \
    -text "Override Default: Procedure Net Version" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.procedure_box_flag)
  scale $d.options.procedure_box_number \
    -label "Procedure Net Version" -from 0 -to 2 \
    -orient horizontal \
    -variable gvar(option.dlg.bpn2_hl_net.procedure_box_number)
  checkbutton $d.options.channel_box_flag \
    -text "Override Default: Channel Net Version" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.channel_box_flag)
  scale $d.options.channel_box_number \
    -label "Channel Net Version" -from 0 -to 2 \
    -orient horizontal \
    -variable gvar(option.dlg.bpn2_hl_net.channel_box_number)

  checkbutton $d.options.no_warnings_old_syntax \
    -text "No Warnings at old Syntax" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.no_warnings_old_syntax)
  checkbutton $d.options.allow_fragments \
    -text "Allow Program Fragments" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.allow_fragments)
  checkbutton $d.options.always_procedures \
    -text "Always use Procedure Semantics" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.always_procedures)

  checkbutton $d.options.procedure_instances_flag \
    -text "Override Default: Maximum Number of concurrent Instances" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.procedure_instances_flag)
  scale $d.options.procedure_instances_number \
    -label "Maximum Number of concurrent Instances" -from 1 -to 10 \
    -orient horizontal \
    -variable gvar(option.dlg.bpn2_hl_net.procedure_instances_number)

  scale $d.options.scale \
    -label "Scale" \
    -from 1 -to 50 -length 200 -orient horizontal -relief ridge \
    -variable gvar(option.dlg.bpn2_hl_net.scale)
  radiobutton $d.options.small_loop \
    -text "Small Loop" -value "small_loop" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.loop_size)
  radiobutton $d.options.medium_loop \
    -text "Medium Loop" -value "medium_loop" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.loop_size)
  radiobutton $d.options.big_loop \
    -text "Big Loop" -value "big_loop" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.loop_size)
  checkbutton $d.options.transition_flag \
    -text "Restrict Number of Transitions" -anchor w \
    -variable gvar(option.dlg.bpn2_hl_net.transition_flag)
  scale $d.options.transition_number \
    -label "Maximum Number of Transitions" -from 1 -to 1000 \
    -orient horizontal \
    -variable gvar(option.dlg.bpn2_hl_net.transition_number)
  pack $d.options.check_net \
    $d.options.rem_dup \
    $d.options.dead_trans \
    $d.options.seq_term \
    $d.options.bpn2_names \
    $d.options.no_scoping \
    $d.options.disable_blocks \
    $d.options.disable_phantoms \
    $d.options.use_set \
    $d.options.no_disjunct_idsets \
    $d.options.procedure_box_flag \
    $d.options.procedure_box_number \
    $d.options.channel_box_flag \
    $d.options.channel_box_number \
    $d.options.no_warnings_old_syntax \
    $d.options.allow_fragments \
    $d.options.always_procedures \
    $d.options.procedure_instances_flag \
    $d.options.procedure_instances_number \
    $d.options.scale \
    $d.options.small_loop \
    $d.options.medium_loop \
    $d.options.big_loop \
    $d.options.transition_flag \
    $d.options.transition_number \
    -side top -anchor w -fill x

  buttonframe $d bpn2_hl_net $prjnum
}  ;# option_bpn2_hl_net_dlg

proc option_hl_net_ll_net_dlg {} {
  # Den Dialog oeffnen.
  global gvar

  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .dlg_option_hl_net_ll_net$prjnum]}] {
    raise .dlg_option_hl_net_ll_net$prjnum
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $d
  wm title $d "HL Net -> LL Net"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
  checkbutton $d.options.remove_edges \
    -text "Remove Edges with Weight > 1" -anchor w \
    -variable gvar(option.dlg.hl_net_ll_net.remove_edges)
  checkbutton $d.options.disable_blocks \
    -text "No Block Information" -anchor w \
    -variable gvar(option.dlg.hl_net_ll_net.disable_blocks)
  checkbutton $d.options.disable_phantoms \
    -text "No Phantom Transitions" -anchor w \
    -variable gvar(option.dlg.hl_net_ll_net.disable_phantoms)
  pack $d.options.remove_edges \
    $d.options.disable_blocks \
    $d.options.disable_phantoms \
    -side top -anchor w -fill x

  buttonframe $d hl_net_ll_net $prjnum
}  ;# option_hl_net_ll_net_dlg

proc option_sync_hl_net_dlg {} {
  # Den Dialog oeffnen.
  global gvar

  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .dlg_option_sync_hl_net$prjnum]}] {
    raise .dlg_option_sync_hl_net$prjnum
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $d
  wm title $d "Sync HL Net"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
  checkbutton $d.options.check_net \
    -text "Check Net" -anchor w \
    -variable gvar(option.dlg.sync_hl_net.check_net)
  checkbutton $d.options.rem_dup \
    -text "Remove Duplicates" -anchor w \
    -variable gvar(option.dlg.sync_hl_net.rem_dup)
  checkbutton $d.options.dead_trans \
    -text "Eliminate Dead Transitions" -anchor w \
    -variable gvar(option.dlg.sync_hl_net.dead_trans)
  checkbutton $d.options.bpn2_names \
    -text "Use B(PN) Names for Communication Variables" -anchor w \
    -variable gvar(option.dlg.sync_hl_net.bpn2_names)
  checkbutton $d.options.disable_blocks \
    -text "No Block Information" -anchor w \
    -variable gvar(option.dlg.sync_hl_net.disable_blocks)
  checkbutton $d.options.disable_phantoms \
    -text "No Phantom Transitions" -anchor w \
    -variable gvar(option.dlg.sync_hl_net.disable_phantoms)
  checkbutton $d.options.no_sync \
    -text "No Synchronisation" -anchor w \
    -variable gvar(option.dlg.sync_hl_net.no_sync)
  checkbutton $d.options.no_restriction \
    -text "No Restriction" -anchor w \
    -variable gvar(option.dlg.sync_hl_net.no_restriction)
  checkbutton $d.options.transition_flag \
    -text "Restrict Number of Transitions" -anchor w \
    -variable gvar(option.dlg.sync_hl_net.transition_flag)
  scale $d.options.transition_number \
    -label "Maximum Number of Transitions" -from 1 -to 1000 \
    -orient horizontal \
    -variable gvar(option.dlg.sync_hl_net.transition_number)
  checkbutton $d.options.max_vars_arc_flag \
    -text "Restrict Number of Variables in an Arc Label" -anchor w \
    -variable gvar(option.dlg.sync_hl_net.max_vars_arc_flag)
  scale $d.options.max_vars_arc_number \
    -label "Maximum Number of Variables" -from 1 -to 20 \
    -orient horizontal \
    -variable gvar(option.dlg.sync_hl_net.max_vars_arc_number)
  pack $d.options.check_net \
    $d.options.rem_dup \
    $d.options.dead_trans \
    $d.options.bpn2_names \
    $d.options.no_sync \
    $d.options.disable_blocks \
    $d.options.disable_phantoms \
    $d.options.no_restriction \
    $d.options.transition_flag \
    $d.options.transition_number \
    $d.options.max_vars_arc_flag \
    $d.options.max_vars_arc_number \
    -side top -anchor w -fill x

  buttonframe $d sync_hl_net $prjnum
}  ;# option_sync_hl_net_dlg

proc option_unfolding_text_dlg {} {
  # Den Dialog oeffnen.
  global gvar

  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .dlg_option_unfolding_text$prjnum]}] {
    raise .dlg_option_unfolding_text$prjnum
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $d
  wm title $d "Prefix Builder"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
  radiobutton $d.options.use_unfold -value $gvar(ll_net_to_unfolding.name)\
    -text "Use original unfolder" -anchor w \
    -variable gvar(option.dlg.unfolding_text.unfolder)
  radiobutton $d.options.use_punf -value $gvar(parallel_unfolder.name)\
    -text "Use parallel unfolder (ignores rest of options)" -anchor w \
    -variable gvar(option.dlg.unfolding_text.unfolder)
  radiobutton $d.options.use_unfsmodels -value $gvar(unfolding_ana_unfsmodels.name)\
    -text "Use unfsmodels (ignores rest of options)" -anchor w \
    -variable gvar(option.dlg.unfolding_text.unfolder)
  checkbutton $d.options.display_time \
    -text "Display Time" -anchor w \
    -variable gvar(option.dlg.unfolding_text.display_time)
  checkbutton $d.options.display_progress \
    -text "Display Progress" -anchor w \
    -variable gvar(option.dlg.unfolding_text.display_progress)
  checkbutton $d.options.use_bit_arrays \
    -text "Use Bit Arrays for Hash Table Entries" -anchor w \
    -variable gvar(option.dlg.unfolding_text.use_bit_arrays)
  checkbutton $d.options.b_conflict_algorithm_flag \
    -text "Change B-Conflict Algorithm" -anchor w \
    -variable gvar(option.dlg.unfolding_text.b_conflict_algorithm_flag)
  scale $d.options.b_conflict_algorithm_number \
    -label "Number of Events before Change" -from 1 -to 10000 \
    -orient horizontal \
    -variable gvar(option.dlg.unfolding_text.b_conflict_algorithm_number)
  checkbutton $d.options.max_events_flag \
    -text "Restrict Number of Maximal Generated Events" -anchor w \
    -variable gvar(option.dlg.unfolding_text.max_events_flag)
  scale $d.options.max_events_number \
    -label "Maximum Number of Generated Events" -from 1 -to 100000 \
    -orient horizontal \
    -variable gvar(option.dlg.unfolding_text.max_events_number)
  checkbutton $d.options.display_statistics \
    -text "Display Statistics" -anchor w \
    -variable gvar(option.dlg.unfolding_text.display_statistics)
  checkbutton $d.options.display_version \
    -text "Display Program Version" -anchor w \
    -variable gvar(option.dlg.unfolding_text.display_version)
# laut Burkhard Graves useless
#  checkbutton $d.options.take_last_events \
#    -text "Take Last Corresponding Events" -anchor w \
#    -variable gvar(option.dlg.unfolding_text.take_last_events)
  checkbutton $d.options.take_current_marking \
    -text "Take Current Marking Instead of Initial" -anchor w \
    -variable gvar(option.dlg.unfolding_text.take_current_marking)
  checkbutton $d.options.display_unused_transitions \
    -text "Display List of Unused Transitions" -anchor w \
    -variable gvar(option.dlg.unfolding_text.display_unused_transitions)

  checkbutton $d.options.minimize_hl_net \
    -text "Minimize HL Net (Replace Old Net)" -anchor w \
    -variable gvar(option.dlg.unfolding_text.minimize_hl_net)

  checkbutton $d.options.minimize_net \
    -text "Minimize LL Net (Replace Old Net)" -anchor w \
    -variable gvar(option.dlg.unfolding_text.minimize_net)
  checkbutton $d.options.use_esparza_vogler \
    -text "Use Esparza/Rmer/Vogler Algorithm" -anchor w \
    -variable gvar(option.dlg.unfolding_text.use_esparza_vogler)
  checkbutton $d.options.time_unfolding \
    -text "Use Time Unfolding" -anchor w \
    -variable gvar(option.dlg.unfolding_text.time_unfolding)
#    -command "if {\$gvar(option.dlg.unfolding_text.time_unfolding)} {
#	$d.options.time_unfolding_optimized configure -state normal
#	} else {$d.options.time_unfolding_optimized configure -state disabled
#        }"
  checkbutton $d.options.time_unfolding_optimized \
    -text "Use Optimized Time Unfolding" -anchor w
#    -variable gvar(option.dlg.unfolding_text.time_unfolding_optimized)
  if {$gvar(option.dlg.unfolding_text.time_unfolding)} {
    $d.options.time_unfolding_optimized configure -state normal
  } else {
    $d.options.time_unfolding_optimized configure -state disabled
  }
  pack $d.options.use_unfold $d.options.use_punf \
    $d.options.use_unfsmodels $d.options.display_time \
    $d.options.display_progress \
    $d.options.use_bit_arrays \
    $d.options.b_conflict_algorithm_flag \
    -side top -anchor w -fill x
  pack $d.options.b_conflict_algorithm_number \
    -side top -anchor w -fill x -expand 1
  pack $d.options.max_events_flag \
    -side top -anchor w -fill x
  pack $d.options.max_events_number \
    -side top -anchor w -fill x -expand 1
  pack $d.options.display_statistics \
    $d.options.display_version \
    $d.options.take_current_marking \
    $d.options.display_unused_transitions \
    $d.options.minimize_hl_net \
    $d.options.minimize_net \
    $d.options.use_esparza_vogler \
    $d.options.time_unfolding \
    -side top -anchor w -fill x
#    $d.options.take_last_events \
#    $d.options.time_unfolding_optimized \


  buttonframe $d unfolding_text $prjnum
}  ;# option_unfolding_text_dlg

proc option_ana_bounded_dlg {} {
  # Den Dialog oeffnen.
  global gvar

  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .dlg_option_ana_bounded$prjnum]}] {
    raise .dlg_option_ana_bounded$prjnum
    return
  }  ;# if return
  add_dlg_widget $prjnum $d
  wm title $d "Boundedness Analyser"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
  checkbutton $d.options.display_statistics \
    -text "Display Statistics" -anchor w \
    -variable gvar(option.dlg.ana_bounded.display_statistics)
  checkbutton $d.options.display_progress \
    -text "Display Progress" -anchor w \
    -variable gvar(option.dlg.ana_bounded.display_progress)
  checkbutton $d.options.display_version \
    -text "Display Version" -anchor w \
    -variable gvar(option.dlg.ana_bounded.display_version)
  pack $d.options.display_statistics \
    $d.options.display_progress \
    $d.options.display_version \
    -side top -anchor w -fill x

  buttonframe $d ana_bounded $prjnum
}  ;# option_ana_bounded_dlg

proc option_ana_reachability_dlg {} {
  # Den Dialog oeffnen.
  global gvar

  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .dlg_option_ana_reachability$prjnum]}] {
    raise .dlg_option_ana_reachability$prjnum
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $d
  wm title $d "Reachability Analyser"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
  checkbutton $d.options.calc_all \
    -text "Calculate all Paths" -anchor w \
    -variable gvar(option.dlg.ana_reachability.calc_all)
  checkbutton $d.options.calc_part_mark \
    -text "Calculate Submarking Reachability" -anchor w \
    -variable gvar(option.dlg.ana_reachability.calc_part_mark)
  pack $d.options.calc_all \
    $d.options.calc_part_mark \
    -side top -anchor w -fill x

  buttonframe $d ana_reachability $prjnum
}  ;# option_ana_reachability_dlg

proc option_ana_mcsmodels_dlg {} {
  # Den Dialog oeffnen.
  global gvar

  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .dlg_option_ana_mcsmodels$prjnum]}] {
    raise .dlg_option_ana_mcsmodels$prjnum
    return
  }
  add_dlg_widget $prjnum $d
  wm title $d "CTL Model Checker mcsmodels"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
  checkbutton $d.options.lookahead -variable gvar(option.dlg.ana_mcsmodels.nolookahead) -text "No Lookahead"
  pack $d.options.lookahead
  labelframe $d.options.optimize -bd 1 -text "Optimization"
  pack $d.options.optimize -fill x -expand true
  radiobutton $d.options.optimize.none \
    -text "None" -anchor w -value 0 \
    -variable gvar(option.dlg.ana_mcsmodels.optimize)
  radiobutton $d.options.optimize.simple \
    -text "Linear Time" -anchor w -value 1 \
    -variable gvar(option.dlg.ana_mcsmodels.optimize)
  radiobutton $d.options.optimize.cheap \
    -text "Cheap Polynomial" -anchor w -value 2 \
    -variable gvar(option.dlg.ana_mcsmodels.optimize)
  radiobutton $d.options.optimize.closure \
    -text "Enabledness Closure" -anchor w -value 3 \
    -variable gvar(option.dlg.ana_mcsmodels.optimize)
  radiobutton $d.options.optimize.full \
    -text "Expensive Optimization" -anchor w -value 4 \
    -variable gvar(option.dlg.ana_mcsmodels.optimize)
  pack $d.options.optimize.none $d.options.optimize.simple \
    $d.options.optimize.cheap $d.options.optimize.closure \
    $d.options.optimize.full -anchor w -padx 5
  buttonframe $d ana_mcsmodels $prjnum
}

proc option_ana_boundsmodels_dlg {} {
  # Den Dialog oeffnen.
  global gvar

  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .dlg_option_ana_boundsmodels$prjnum]}] {
    raise .dlg_option_ana_boundsmodels$prjnum
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $d
  wm title $d "LTL Model Checker boundsmodels"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
  labelframe $d.options.bound -bd 1 -text Bound
  pack $d.options.bound -fill x -expand yes
  spinbox $d.options.bound.spinbox -from 1 -to inf -increment 1 \
    -textvariable gvar(option.dlg.ana_boundsmodels.bound)
  pack $d.options.bound.spinbox -anchor w -padx 5
  labelframe $d.options.semantics -bd 1 -text "Semantics"
  pack $d.options.semantics -fill x -expand true
  radiobutton $d.options.semantics.step \
    -text "Step semantics" -anchor w -value step \
    -variable gvar(option.dlg.ana_boundsmodels.semantics)
  radiobutton $d.options.semantics.interleave \
    -text "Interleaving semantics" -anchor w -value interleave \
    -variable gvar(option.dlg.ana_boundsmodels.semantics)
  pack $d.options.semantics.step $d.options.semantics.interleave -side left -padx 5
  labelframe $d.options.optimize -bd 1 -text "Optimization"
  pack $d.options.optimize -fill x -expand true
  radiobutton $d.options.optimize.none \
    -text "None" -anchor w -value none \
    -variable gvar(option.dlg.ana_boundsmodels.optimize)
  radiobutton $d.options.optimize.simple \
    -text "Simple" -anchor w -value simple \
    -variable gvar(option.dlg.ana_boundsmodels.optimize)
  radiobutton $d.options.optimize.full \
    -text "Full" -anchor w -value full \
    -variable gvar(option.dlg.ana_boundsmodels.optimize)
  pack $d.options.optimize.none $d.options.optimize.simple \
    $d.options.optimize.full -anchor w -padx 5
  buttonframe $d ana_boundsmodels $prjnum
}

proc option_ana_clp_dlg {} {
  # Den Dialog oeffnen.
  global gvar

  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .dlg_option_ana_clp$prjnum]}] {
    raise .dlg_option_ana_clp$prjnum
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $d
  wm title $d "PO Reachability Analyser"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
  checkbutton $d.options.shortest_trail \
    -text "Look for a shortest trail" -anchor w \
    -variable gvar(option.dlg.ana_clp.shortest_trail)
  checkbutton $d.options.safe_net \
    -text "Suppose safety of the net" -anchor w \
    -variable gvar(option.dlg.ana_clp.safe_net)
  checkbutton $d.options.save_mem \
    -text "Memory saving mode" -anchor w \
    -variable gvar(option.dlg.ana_clp.save_mem)
  pack $d.options.shortest_trail \
    $d.options.safe_net \
    $d.options.save_mem \
    -side top -anchor w -fill x

  buttonframe $d ana_clp $prjnum
}  ;# option_ana_clp_dlg

proc option_ana_lp_dlg {} {
  # Den Dialog oeffnen.
  global gvar

  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .dlg_option_ana_lp$prjnum]}] {
    raise .dlg_option_ana_lp$prjnum
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $d
  wm title $d "LP Analyser"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -padx 5 -pady 5 -side top -fill both

    # allgemeine Optionen zu den Verfahren
  frame $d.options.general_methods -relief sunken -bd 1
  pack $d.options.general_methods -ipadx 2 -ipady 2 -side top -fill both
  label $d.options.general_methods.title \
    -text "General Method Options" -anchor w
  radiobutton $d.options.general_methods.considered_prop_all \
    -text "Consider Trap and S-Invariant Properties" -anchor w \
    -variable gvar(option.dlg.ana_lp.considered_prop) -value 0
  radiobutton $d.options.general_methods.considered_prop_trap \
    -text "Consider only Trap Properties" -anchor w \
    -variable gvar(option.dlg.ana_lp.considered_prop) -value 1
  radiobutton $d.options.general_methods.considered_prop_s_inv \
    -text "Consider only S-Invariant Properties" -anchor w \
    -variable gvar(option.dlg.ana_lp.considered_prop) -value 2
  pack $d.options.general_methods.title \
    -padx 2 -pady 4 -side top -anchor w -fill x
  pack $d.options.general_methods.considered_prop_all \
    $d.options.general_methods.considered_prop_trap \
    $d.options.general_methods.considered_prop_s_inv \
    -padx 8 -side top -anchor w -fill x

    # allgemeine Optionen zum LP und Debug
  frame $d.options.general_lp -relief sunken -bd 1
  pack $d.options.general_lp -ipadx 2 -ipady 2 -side top -fill both
  label $d.options.general_lp.title \
    -text "General LP / Debug Options" -anchor w
  checkbutton $d.options.general_lp.use_cplex \
    -text "Use CPLEX instead of lp_solve" -anchor w \
    -variable gvar(option.dlg.ana_lp.use_cplex)
  checkbutton $d.options.general_lp.solve_noint_lp \
    -text "Solve non Integer LP" -anchor w \
    -variable gvar(option.dlg.ana_lp.solve_noint_lp)
  checkbutton $d.options.general_lp.use_ezpeleta \
    -text "Use Ezpeleta/Couvreur instead of Alaiwan/Toudic for Traps" -anchor w \
    -variable gvar(option.dlg.ana_lp.use_ezpeleta)
  checkbutton $d.options.general_lp.objective_func_coeff_power_flag \
    -text "Specify Objective Function Coefficient" -anchor w \
    -variable gvar(option.dlg.ana_lp.objective_func_coeff_power_flag)
  scale $d.options.general_lp.objective_func_coeff_power \
    -label "Specify Scaling Factor for Trap Equation" -from 1 -to 20 \
    -orient horizontal \
    -variable gvar(option.dlg.ana_lp.objective_func_coeff_power)

#  radiobutton $d.options.general_lp.write_no_lp_file \
#    -text "Write no LP or MPS File" -anchor w \
#    -variable gvar(option.dlg.ana_lp.write_lp_file) -value 0
#  radiobutton $d.options.general_lp.write_lp_file \
#    -text "Write LP File" -anchor w \
#    -variable gvar(option.dlg.ana_lp.write_lp_file) -value 1
#  radiobutton $d.options.general_lp.write_mps_file \
#    -text "Write MPS File" -anchor w \
#    -variable gvar(option.dlg.ana_lp.write_lp_file) -value 2
#  radiobutton $d.options.general_lp.write_lp_mps_file \
#    -text "Write LP and MPS File" -anchor w \
#    -variable gvar(option.dlg.ana_lp.write_lp_file) -value 3
  checkbutton $d.options.general_lp.write_lp_file \
    -text "Write LP File" -anchor w \
    -variable gvar(option.dlg.ana_lp.write_lp_file)
  checkbutton $d.options.general_lp.write_mps_file \
    -text "Write MPS File" -anchor w \
    -variable gvar(option.dlg.ana_lp.write_mps_file)

  checkbutton $d.options.general_lp.do_nothing \
    -text "Do not invoke LP Solver" -anchor w \
    -variable gvar(option.dlg.ana_lp.do_nothing)
  checkbutton $d.options.general_lp.display_debug \
    -text "Display Debug" -anchor w \
    -variable gvar(option.dlg.ana_lp.display_debug)
  pack $d.options.general_lp.title \
    -padx 2 -pady 4 -side top -anchor w -fill x
  pack $d.options.general_lp.use_cplex \
    $d.options.general_lp.solve_noint_lp \
    $d.options.general_lp.use_ezpeleta \
    $d.options.general_lp.objective_func_coeff_power_flag \
    $d.options.general_lp.objective_func_coeff_power \
    $d.options.general_lp.write_lp_file \
    $d.options.general_lp.write_mps_file \
    $d.options.general_lp.do_nothing \
    $d.options.general_lp.display_debug \
    -padx 8 -side top -anchor w -fill x

    # Optionen nur fuer lp_solve
  frame $d.options.only_lp_solve -relief sunken -bd 1
  pack $d.options.only_lp_solve -ipadx 2 -ipady 2 -side top -fill both
  label $d.options.only_lp_solve.title \
    -text "LP_solve Options" -anchor w
  checkbutton $d.options.only_lp_solve.use_perturbation \
    -text "Use Perturbation" -anchor w \
    -variable gvar(option.dlg.ana_lp.use_perturbation)
  checkbutton $d.options.only_lp_solve.lp_solve_coeff_flag \
    -text "Specify LP_solve Coefficient" -anchor w \
    -variable gvar(option.dlg.ana_lp.lp_solve_coeff_flag)
  scale $d.options.only_lp_solve.lp_solve_coeff \
    -label "Specify lp_solve Integrality Parameter" -from 0.01 -to 0.49 \
    -resolution 0.01 -orient horizontal \
    -variable gvar(option.dlg.ana_lp.lp_solve_coeff)

  checkbutton $d.options.only_lp_solve.use_max_token_number \
    -text "Use upper bound for the maximal token number" -anchor w \
    -variable gvar(option.dlg.ana_lp.use_max_token_number)

  scale $d.options.only_lp_solve.max_token_number \
    -from 0 -to 100 \
    -orient horizontal -variable gvar(option.dlg.ana_lp.max_token_number)

  pack $d.options.only_lp_solve.title \
    -padx 2 -pady 4 -side top -anchor w -fill x
  pack $d.options.only_lp_solve.use_perturbation \
    $d.options.only_lp_solve.lp_solve_coeff_flag \
    $d.options.only_lp_solve.lp_solve_coeff \
    $d.options.only_lp_solve.use_max_token_number \
    $d.options.only_lp_solve.max_token_number \
    -padx 8 -side top -anchor w -fill x

    # spezielle Optionen zur Erreichbarkeit
  frame $d.options.special_reachability -relief sunken -bd 1
  pack $d.options.special_reachability -ipadx 2 -ipady 2 \
    -side top -fill both
  label $d.options.special_reachability.title \
    -text "Deadlock-free Options" -anchor w
  checkbutton $d.options.special_reachability.use_prefix \
    -text "Use Prefix instead of LL Net" -anchor w \
    -variable gvar(option.dlg.ana_lp.use_prefix)
  pack $d.options.special_reachability.title \
    -padx 2 -pady 4 -side top -anchor w -fill x
  pack $d.options.special_reachability.use_prefix \
    -padx 8 -side top -anchor w -fill x

  buttonframe $d ana_lp $prjnum
}  ;# option_ana_lp_dlg

proc option_ana_model_check_safe_dlg {} {
  # Den Dialog oeffnen.
  global gvar

  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .dlg_option_model_check_safe$prjnum]}] {
    raise .dlg_option_model_check_safe$prjnum
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $d
  wm title $d "Model Checker (Safe)"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -padx 5 -pady 5 -side top -fill both

    # Allgemeine Optionen
  frame $d.options.general -relief sunken -bd 1
  pack $d.options.general -ipadx 2 -ipady 2 -side top -fill both
  label $d.options.general.title -text "General Options" -anchor w
  checkbutton $d.options.general.force_calc \
    -text "Force Calculation of Information" -anchor w \
    -variable gvar(option.dlg.model_check_safe.force_calc)
  checkbutton $d.options.general.disp_fire_seqs \
    -text "Display Firing Sequences" -anchor w \
    -variable gvar(option.dlg.model_check_safe.disp_fire_seqs)
  checkbutton $d.options.general.only_min_fire_seqs \
    -text "Display only minimal Firing-Sequences" -anchor w \
    -variable gvar(option.dlg.model_check_safe.only_min_fire_seqs)
  checkbutton $d.options.general.log_results \
    -text "Add results to doc file" -anchor w \
    -variable gvar(option.dlg.model_check_safe.log_results)
  pack $d.options.general.title \
    -padx 2 -pady 4 -side top -anchor w -fill x
  pack $d.options.general.force_calc \
    $d.options.general.disp_fire_seqs \
    $d.options.general.only_min_fire_seqs \
    $d.options.general.log_results \
    -padx 8 -side top -anchor w -fill x

  buttonframe $d model_check_safe $prjnum
}  ;# option_ana_model_check_safe_dlg

proc do_command_1 {prjnum} {
  # Kommando fuer Radiobutton "Acceptance Cycles (Liveness)"
  global gvar

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"

  if {![pattern_in_file $prjx(promela_form.filename) "accept"]} {
    set gvar(option.dlg.model_check_spin.an_typ) 0; \
    puts stderr "specification contains no accept labels" \
  }
} ;# do_command_1

proc do_command_2 {prjnum} {
  # Kommando fuer Checkbutton "Apply Never Claim (If Present)"
  global gvar

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"

  if {![pattern_in_file $prjx(promela_form.filename) "never"]} {
    set gvar(option.dlg.model_check_spin.nv_typ) 0; \
    puts stderr "specification contains no never labels" \
  }
} ;# do_command_2

proc do_command_3 {} {
  # Kommando fuer Checkbutton "Weak Fairness (for Liveness)"
  global gvar

  if {$gvar(option.dlg.model_check_spin.an_typ)==0} {
    set gvar(option.dlg.model_check_spin.cyc_typ) 0; \
    puts stderr "weak fairness is irrelevant to state properties" \
  }
} ;# do_command_3

proc option_ana_model_check_spin_dlg_choose1 {} {
  global gvar

  #Neues Toplevel erzeugen, falls dies bereits existiert: raise
  if [catch {set w [toplevel .dlg_option_model_check_spin_choose1]}] {
    raise .dlg_option_model_check_spin_choose1
    return
  }
  wm title $w "Extra Compile-Time Directives"
  wm transient $w .dlg_option_model_check_spin$gvar(actprj)

  label $w.head -text "Choose an Extra Compile-Time Directive\nBy Double-Clicking it:"
  pack $w.head -side top -pady 10 -padx 10

  frame $w.b
  pack $w.b -side bottom -expand yes -fill x
  button $w.b.explain -text "Explain" \
    -command "text_window $w.exp_ectd \
      {Help: Extra Compile-Time Directives} {Use only when prompted:\nNFAIR=N		size memory used for enforcing weak fairness (default N=2)\nVECTORSZ=N	allocates memory (in bytes) for state vector (default N=1024) \n\nRelated to partial order reduction:

CTL		limit p.o.reduction to subset consistent with branching time logic
GLOB_ALPHA	consider process death a global action
XUSAFE		disable validity checks of xr/xs assertions

Speedups:
NOBOUNDCHECK	don't check array bound violations
NOCOMP		don't compress states with fullstate storage (uses more memory)
NOSTUTTER	disable stuttering rules (warning: changes semantics)

Memory saving (slower):
MA=N		use a minimized DFA encoding for state vectors up to N bytes

Experimental:
BCOMP		when in BITSTATE mode, computes hash over compressed state-vector
HYBRID_HASH	can shave a word (4 bytes) from state vectors with N%4=1
NIBIS		apply a small optimization of partial order reduction
NOVSZ		risky - removes 4 bytes from state vector - its length field.
PRINTF		enables printfs during verification runs
RANDSTORE=N	in BITSTATE mode, -DRANDSTORE=33 lowers prob of storing a state to 33%
W_XPT=N		with MA, write checkpoint files every multiple of N states stored
R_XPT		with MA, restart run from last checkpoint file written

Debugging:
SDUMP		with CHECK: adds ascii dumps of state vectors
SVDUMP		add run option -pN to write N-byte state vectors into file sv_dump

Already set by the other xspin options:
BITSTATE	use supertrace/bitstate instead of exhaustive exploration
HC		use hash-compact instead of exhaustive exploration
COLLAPSE	collapses state vector size by up to 80% to 90%
MEMCNT=N	set upperbound of 2^N bytes to memory that can be allocated
NOCLAIM		exclude never claim from the verification, if present
NOFAIR		disable the code for weak-fairness (is faster)
NOREDUCE	disables the partial order reduction algorithm
NP		enable non-progress cycle detection (option -l, replacing -a),
PEG		add complexity profiling (transition counts)
REACH		guarantee absence of errors within the -m depth-limit
SAFETY		optimize for the case where no cycle detection is needed
VAR_RANGES	compute the effective value range of byte variables
CHECK		generate debugging information (see also below)
VERBOSE		elaborate debugging printouts}"

  button $w.b.cancel -text "Cancel" -command "destroy $w; return"
  pack $w.b.explain $w.b.cancel -side left -expand yes

  listbox $w.list -width 10 -height 10 -yscroll "$w.scroll set" -setgrid 1
  scrollbar  $w.scroll -command "$w.list yview"
  pack $w.list -side left -fill both -expand yes
  pack $w.scroll -side right -fill y

  set liste {"-DBCOMP" "-DCTL" "-DGLOB_ALPHA" "-DHYBRID_HASH" "-DMA=?" "-DNFAIR=?" "-DNIBIS" "-DNOBOUNDCHECK" "-DNOCOMP" "-DNOSTUTTER" "-DNOVSZ" "-DPRINTF" "-DRANDSTORE=?" "-DR_XPT" "-DSDUMP" "-DSVDUMP" "-DVECTORSZ=?" "-DW_XPT=?" "-DXUSAFE"}
  foreach el $liste {
    $w.list insert end $el
  }
  bind $w.list <Double-1> {
      global gvar
      set gvar(option.dlg.model_check_spin.comp_time) \
        "$gvar(option.dlg.model_check_spin.comp_time) [%W get @%x,%y]"
      destroy .dlg_option_model_check_spin_choose1
    }
} ;# option_ana_model_check_spin_dlg_choose1

proc option_ana_model_check_spin_dlg_choose2 {} {
  global gvar

  #Neues Toplevel erzeugen, falls dies bereits existiert: raise
  if [catch {set w [toplevel .dlg_option_model_check_spin_choose2]}] {
    raise .dlg_option_model_check_spin_choose2
    return
  }
  wm title $w "Extra Run-Time Options"
  wm transient $w .dlg_option_model_check_spin$gvar(actprj)

  label $w.head -text "Choose an Extra Run-Time Option:"
  pack $w.head -side top -pady 10 -padx 10

  foreach el {{ 1 {-d} {print state tables and stop}}
              { 2 {-q} {require all chans to be empty in valid endstates}}
	      { 3 {-l} {try to find shortest trail}}
	      { 4 {-h?} {choose another hash-function 1..32 (default 1)}}
	      { 5 {-s} {use 1-bit hashing (default is 2-bit)}}} {
    set no [lindex $el 0]
    set sw [lindex $el 1]
    set descr [lindex $el 2]
    frame $w.f$no
    button $w.f$no.b -text $sw -width 5 \
      -command "global gvar; \
                append gvar(option.dlg.model_check_spin.run_time) { } $sw; \
          destroy .dlg_option_model_check_spin_choose2"
    label $w.f$no.l -text $descr
    pack $w.f$no.b $w.f$no.l -side left 
    pack $w.f$no -expand yes -fill x
  }
  button $w.cancel -text "Cancel" -command "destroy $w; return"
  pack $w.cancel -side bottom -expand yes

} ;# option_ana_model_check_spin_dlg_choose2

proc option_ana_model_check_spin_dlg {} {
  # Den Dialog oeffnen.
  global gvar

  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .dlg_option_model_check_spin$prjnum]}] {
    raise .dlg_option_model_check_spin$prjnum
    return
  }  ;# if doppelt

  init_options_var_model_check_spin $prjnum

set s $d
  add_dlg_widget $prjnum $d
  wm title $d "Model Checker (SPIN)"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -padx 5 -pady 5 -side top -fill both

    # Allgemeine Optionen

    # Optionen "Memory Options"
  frame $d.options.memory -relief sunken -bd 1

  frame $d.options.memory.choise0
  label $d.options.memory.choise0.e0 -text "Maximum Memory Usage (Mbytes): "
  entry $d.options.memory.choise0.e1 -relief sunken -width 35 \
     -textvariable gvar(option.dlg.model_check_spin.max_mem)
  button $d.options.memory.choise0.e2 -text "Explain" \
    -command "text_window $d.exp_mmu \
      {Help: Maximum Memory Usage} {Physical Memory Available:\n\nSet this number to amount of physical (not virtual) memory in your system, and leave it there for all runs.\n\nWhen the limit is reached, the verification is stopped to avoid trashing.\n\nThe number entered here is rounded to the nearest power of two (e.g., to 4, 8, 16, 32, 64, 128, 256 Mbytes etc.) Increase it by doubling the number, decrease it by halfing.\n\nIf an exhaustive verification cannot be completed due to lack of memory, use compression, or switch to a supertrace/bitstate run under basic options.}"

  frame $d.options.memory.choise1
  label $d.options.memory.choise1.e0 -text "Estimated State Space Size (states x 10^3): "
  entry $d.options.memory.choise1.e1 -relief sunken -width 35 \
     -textvariable gvar(option.dlg.model_check_spin.state_space)
  button $d.options.memory.choise1.e2 -text "Explain" \
    -command "text_window $d.exp_esss \
      {Help: Estimated State Space Size} {Estimated State Space Size:\n\nThis parameter is used to calculate the size of the hash-table.  Setting it too high may cause an out-of-memory error with zero states reached (meaning that the verification cannot even be started). Setting it too low can cause inefficiencies due to hash collisions.\n\nIn Bitstate runs begin with the default estimate for this parameter.  After a run completes successfully, double the estimate, and see if the number of reached stated changes much.  Continue to do this until it stops changing, or until you overflow the memory bound and run out of memory.\n\nThe closest power of two is taken by xspin to set the true number used for the number of reachable states.  (The selected power of two is visible as the -w argument in the run-line that xspin synthesizes).}"

  frame $d.options.memory.choise2
  label $d.options.memory.choise2.e0 -text "Maximum Search Depth (steps): "
  entry $d.options.memory.choise2.e1 -relief sunken -width 35 \
     -textvariable gvar(option.dlg.model_check_spin.max_search_depth)
  button $d.options.memory.choise2.e2 -text "Explain" \
    -command "text_window $d.exp_msd \
      {Help: Maximum Search Depth} {Maximum Search Depth:\n\nThis number determines the size of the depth-first search stack that is used during the verification.  The stack uses memory, so a larger number increases the memory requirements, and a lower number decreases it.  In a tight spot, when there seems not to be sufficient memory for the search depth needed, you can reduce the estimated state space size to free some more memory for the stack.\n\nIf you hit the maximum search depth during a verification (noted as 'Search not completed' or 'Search Truncated' in the verification output) without finding an error, double the search depth parameter and repeat the run.}"

  frame $d.options.memory.choise3
  label $d.options.memory.choise3.e0 -text "Extra Compile-Time Directives (Optional): "
  entry $d.options.memory.choise3.e1 -relief sunken -width 35 \
     -textvariable gvar(option.dlg.model_check_spin.comp_time)
  button $d.options.memory.choise3.e2 -text "Choose" \
    -command "option_ana_model_check_spin_dlg_choose1"
  frame $d.options.memory.choise4
  label $d.options.memory.choise4.e0 -text "Extra Run-Time Options (Optional): "
  entry $d.options.memory.choise4.e1 -relief sunken -width 35 \
     -textvariable gvar(option.dlg.model_check_spin.run_time)
  button $d.options.memory.choise4.e2 -text "Choose" \
    -command "option_ana_model_check_spin_dlg_choose2"

  for {set i 0} {$i <= 4} {incr i} {
     pack $d.options.memory.choise$i \
       -padx 2 -pady 4 -side top -anchor w -fill x
     pack $d.options.memory.choise$i.e0 -side left -anchor w -fill x
     pack $d.options.memory.choise$i.e2 \
          $d.options.memory.choise$i.e1 \
	 -side right -anchor w -fill x
  }

    # Optionen "Correctness Properties"
  frame $d.options.m
  frame $d.options.m.correct -relief sunken -bd 1
  label $d.options.m.correct.title -text "Correctness Properties" -anchor w \
	-relief sunken -borderwidth 1m
  radiobutton $d.options.m.correct.safety \
    -text "Safety (state properties)" -anchor w \
    -variable gvar(option.dlg.model_check_spin.an_typ) \
    -value 0 \
    -command {set gvar(option.dlg.model_check_spin.cyc_typ) 0
              set gvar(option.dlg.model_check_spin.an_typ_2) 0
             }
  checkbutton $d.options.m.correct.assertions \
    -text "Assertions" -anchor w \
    -state disabled
  checkbutton $d.options.m.correct.inv_end \
    -text "Invalid Endstates" -anchor w \
    -state disabled

  radiobutton $d.options.m.correct.liveness \
    -text "Liveness (cycles/sequences)" -anchor w \
    -variable gvar(option.dlg.model_check_spin.an_typ_2) \
    -value 1 \
    -command {global gvar
              set gvar(option.dlg.model_check_spin.an_typ) -1
	     }
  radiobutton $d.options.m.correct.non_pro \
    -text "Non-Progress Cycles" -anchor w \
    -variable gvar(option.dlg.model_check_spin.an_typ) \
    -value 1 \
    -command { set gvar(option.dlg.model_check_spin.an_typ_2) 1 }
  radiobutton $d.options.m.correct.acceptance \
    -text "Acceptance Cycles" -anchor w \
    -variable gvar(option.dlg.model_check_spin.an_typ) \
    -value 2 \
    -command {
      set gvar(option.dlg.model_check_spin.an_typ_2) 1
      do_command_1 $gvar(actprj)
    }
  checkbutton $d.options.m.correct.weak_fair \
    -text "Weak Fairness (for Liveness)" -anchor w \
    -variable gvar(option.dlg.model_check_spin.cyc_typ) \
    -command {
      set gvar(option.dlg.model_check_spin.an_typ_2) 1
      do_command_3
    }
#  radiobutton $d.options.m.correct.state_prop \
#    -text "State Properties (Safety)" -anchor w \
#    -variable gvar(option.dlg.model_check_spin.an_typ) \
#    -value 0 \
#    -command {set gvar(option.dlg.model_check_spin.cyc_typ) 0}
#  radiobutton $d.options.m.correct.no_prog_cyc \
#    -text "Non-Progress Cycles (Liveness)" -anchor w \
#    -variable gvar(option.dlg.model_check_spin.an_typ) \
#    -value 1
#  radiobutton $d.options.m.correct.accept_cyc \
#    -text "Acceptance Cycles (Liveness)" -anchor w \
#    -variable gvar(option.dlg.model_check_spin.an_typ) \
#    -value 2 \
#    -command {do_command_1 $gvar(actprj)}
#  checkbutton $d.options.m.correct.weak_fair \
#    -text "Weak Fairness (for Liveness)" -anchor w \
#    -variable gvar(option.dlg.model_check_spin.cyc_typ) \
#    -command {do_command_3}

  checkbutton $d.options.m.correct.app_never_claim \
    -text "Apply Never Claim (If Present)" -anchor w \
    -variable gvar(option.dlg.model_check_spin.nv_typ) \
    -command {do_command_2 $gvar(actprj)}
  checkbutton $d.options.m.correct.rep_unreach_cod \
    -text "Report Unreachable Code" -anchor w \
    -variable gvar(option.dlg.model_check_spin.ur_typ) 
  checkbutton $d.options.m.correct.check_xr_xs \
    -text "Check xr/xs Assertions" -anchor w \
    -variable gvar(option.dlg.model_check_spin.xu_typ) 

  pack $d.options.m.correct.title \
    -padx 2 -pady 4 -side top -anchor w -fill x
  pack $d.options.m.correct.safety \
       -side top -anchor w -fill x
  pack $d.options.m.correct.assertions \
       $d.options.m.correct.inv_end \
       -padx 10 -side top -anchor w -fill x
  pack $d.options.m.correct.liveness \
       -side top -anchor w -fill x
  pack $d.options.m.correct.non_pro \
       $d.options.m.correct.acceptance \
       $d.options.m.correct.weak_fair\
       -padx 10 -side top -anchor w -fill x
  pack $d.options.m.correct.app_never_claim \
    $d.options.m.correct.rep_unreach_cod \
    $d.options.m.correct.check_xr_xs \
    -side top -anchor w -fill x

#  pack $d.options.m.correct.state_prop \
#    $d.options.m.correct.no_prog_cyc \
#    $d.options.m.correct.accept_cyc \
#    $d.options.m.correct.weak_fair \
#    $d.options.m.correct.app_never_claim \
#    $d.options.m.correct.rep_unreach_cod \
#    $d.options.m.correct.check_xr_xs \
#    -side top -anchor w -fill x

    # Optionen "Error Trapping"
  frame $d.options.m.e_s
  frame $d.options.m.e_s.error -relief sunken -bd 1
  label $d.options.m.e_s.error.title -text "Error Trapping" -anchor w \
	-relief sunken -borderwidth 1m
  radiobutton $d.options.m.e_s.error.no_stop_at_err \
    -text "Don't Stop at Errors" -anchor w \
    -variable gvar(option.dlg.model_check_spin.et_typ) \
    -value 0 
  checkbutton $d.options.m.e_s.error.save_all_err_tr \
    -text "Save All Error-trails" -anchor w \
    -variable gvar(option.dlg.model_check_spin.st_typ)
  checkbutton $d.options.m.e_s.error.find_short_tr \
    -text "Find Shortest Trail" -anchor w \
    -variable gvar(option.dlg.model_check_spin.se_typ)
  radiobutton $d.options.m.e_s.error.stop_at_err \
    -text "Stop at Error Nr:" -anchor w \
    -variable gvar(option.dlg.model_check_spin.et_typ) \
    -value 1
  entry $d.options.m.e_s.error.err_nr -relief sunken -width 8 \
    -textvariable gvar(option.dlg.model_check_spin.err_nr) 
#  $d.options.error.err_nr insert end "1"

  pack $d.options.m.e_s.error.title \
    -padx 2 -pady 4 -side top -anchor w -fill x
  pack $d.options.m.e_s.error.save_all_err_tr \
    $d.options.m.e_s.error.find_short_tr \
    $d.options.m.e_s.error.no_stop_at_err \
    -side top -anchor w -fill x
  pack $d.options.m.e_s.error.stop_at_err -side left -anchor w
  pack $d.options.m.e_s.error.err_nr -side right

    # Optionen "Search Mode"
  frame $d.options.m.e_s.search -relief sunken -bd 1
  label $d.options.m.e_s.search.title -text "Search Mode" -anchor w \
	-relief sunken -borderwidth 1m
  radiobutton $d.options.m.e_s.search.exhaust\
    -text "Exhaustive" -anchor w \
    -variable gvar(option.dlg.model_check_spin.ct_typ) \
    -value 0 
  radiobutton $d.options.m.e_s.search.super \
    -text "Supertrace" -anchor w \
    -variable gvar(option.dlg.model_check_spin.ct_typ) \
    -value 1
  radiobutton $d.options.m.e_s.search.hash \
    -text "Hash-Compact" -anchor w \
    -variable gvar(option.dlg.model_check_spin.ct_typ) \
    -value 2

  pack $d.options.m.e_s.search.title \
    -padx 2 -pady 4 -side top -anchor w -fill x
  pack $d.options.m.e_s.search.exhaust \
    $d.options.m.e_s.search.super \
    $d.options.m.e_s.search.hash \
    -side top -anchor w -fill x

    # Optionen "Type of Verification Run"
  frame $d.options.m.r_q
  frame $d.options.m.r_q.run -relief sunken -bd 1
  label $d.options.m.r_q.run.title -text "Type of Verification Run" -anchor w \
	-relief sunken -borderwidth 1m
  checkbutton $d.options.m.r_q.run.red_part_ord \
    -text "Reduced (Partial Orders)" -anchor w \
    -variable gvar(option.dlg.model_check_spin.po_typ)
  checkbutton $d.options.m.r_q.run.verb_for_deb \
    -text "Verbose (For Debugging)" -anchor w \
    -variable gvar(option.dlg.model_check_spin.vb_typ)
  checkbutton $d.options.m.r_q.run.add_compl_prof \
    -text "Add Complexity Profiling" -anchor w \
    -variable gvar(option.dlg.model_check_spin.pr_typ)
  checkbutton $d.options.m.r_q.run.comp_var_rang \
    -text "Compute Variable Ranges" -anchor w \
    -variable gvar(option.dlg.model_check_spin.vr_typ)

  pack $d.options.m.r_q.run.title \
    -padx 2 -pady 4 -side top -anchor w -fill x
  pack $d.options.m.r_q.run.red_part_ord \
    $d.options.m.r_q.run.verb_for_deb \
    $d.options.m.r_q.run.add_compl_prof \
    $d.options.m.r_q.run.comp_var_rang \
    -side top -anchor w -fill x

    # Optionen "A Full Queue"
  frame $d.options.m.r_q.queue -relief sunken -bd 1
  label $d.options.m.r_q.queue.title -text "A Full Queue" -anchor w \
	-relief sunken -borderwidth 1m
  radiobutton $d.options.m.r_q.queue.bl_new_msg\
    -text "Blocks New Msgs" -anchor w \
    -variable gvar(option.dlg.model_check_spin.l_typ) \
    -value 0 
  radiobutton $d.options.m.r_q.queue.los_new_msg \
    -text "Loses New Msgs" -anchor w \
    -variable gvar(option.dlg.model_check_spin.l_typ) \
    -value 1

  pack $d.options.m.r_q.queue.title \
    -padx 2 -pady 4 -side top -anchor w -fill x
  pack $d.options.m.r_q.queue.bl_new_msg \
    $d.options.m.r_q.queue.los_new_msg \
    -side top -anchor w -fill x

    # Optionen fuer LOG
  frame $d.options.m.r_q.log -relief sunken -bd 1
  label $d.options.m.r_q.log.title -text "Log" -anchor w \
      -relief sunken -borderwidth 1m
  checkbutton $d.options.m.r_q.log.check \
    -text "Add results to doc file" \
    -variable gvar(option.dlg.model_check_spin.log_results)
  pack $d.options.m.r_q.log.title \
    -padx 2 -pady 4 -side top -anchor w -fill x
  pack $d.options.m.r_q.log.check \
    -side top -anchor w -fill x

   # Option "LTL Formula"
  frame $d.options.formula -relief sunken -bd 1
  label $d.options.formula.title -text "LTL Formula" -anchor w \
	-relief sunken -borderwidth 1m
  label $d.options.formula.text -text "This Property Is: " 
  radiobutton $d.options.formula.alw_satis\
    -text "Always Satisfied" -anchor w \
    -variable gvar(option.dlg.model_check_spin.tl_stat) \
    -value 0 
  radiobutton $d.options.formula.nev_satis\
    -text "Never Satisfied" -anchor w \
    -variable gvar(option.dlg.model_check_spin.tl_stat) \
    -value 1

  pack $d.options.formula.title \
    -padx 2 -pady 4 -side top -anchor w -fill x
  pack $d.options.formula.text \
    $d.options.formula.alw_satis \
    $d.options.formula.nev_satis \
    -side left -anchor w -fill x


  pack $d.options.memory -ipadx 2 -ipady 2 -side top -fill both
  pack $d.options.m -ipadx 2 -ipady 2 -side top -fill both
  pack $d.options.m.correct -ipadx 2 -ipady 2 -side left -fill both
  pack $d.options.m.r_q -ipadx 2 -ipady 2 -side left -fill both
  pack $d.options.m.r_q.run \
     $d.options.m.r_q.queue \
     $d.options.m.r_q.log \
     -ipadx 2 -ipady 2 -side top -fill both
  pack $d.options.m.e_s -ipadx 2 -ipady 2 -side left -fill both
  pack $d.options.m.e_s.error \
     $d.options.m.e_s.search -ipadx 2 -ipady 2 -side top -fill both
  pack $d.options.formula -ipadx 2 -ipady 2 -side top -fill both


  buttonframe $d model_check_spin $prjnum
}  ;# option_ana_model_check_spin_dlg

proc option_ana_model_check_smv_dlg {} {
  # Den Dialog oeffnen.
  global gvar

  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .dlg_option_model_check_smv$prjnum]}] {
    raise .dlg_option_model_check_smv$prjnum
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $d
  wm title $d "Model Checker (SMV)"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -padx 5 -pady 5 -side top -fill both

    # Allgemeine Optionen
  frame $d.options.general -relief sunken -bd 1
  pack $d.options.general -ipadx 2 -ipady 2 -side top -fill both
  label $d.options.general.title -text "General Options" -anchor w
  label $d.options.general.warn1 -text "We highly recommend to use Models 4 and 5. Other models may" -anchor w
  label $d.options.general.warn2 -text "cause wrong results.\n" -anchor w

  radiobutton $d.options.general.switch1 \
    -text "1. transitions as processes, FAIRNESS running for transition"\
    -anchor w -variable gvar(option.dlg.model_check_smv.switch) -value 1
  radiobutton $d.options.general.switch2 \
    -text "2. transitions as processes, global FAIRNESS"\
    -anchor w -variable gvar(option.dlg.model_check_smv.switch) -value 2
  radiobutton $d.options.general.switch3 \
    -text "3. places and transitions as modules"\
    -anchor w -variable gvar(option.dlg.model_check_smv.switch) -value 3
  radiobutton $d.options.general.switch4 \
    -text "4. explicit transition relation, selfloops in deadlock states"\
    -anchor w -variable gvar(option.dlg.model_check_smv.switch) -value 4
  radiobutton $d.options.general.switch5 \
    -text "5. same as 4, 'next(x)=x' expressions compressed"\
    -anchor w -variable gvar(option.dlg.model_check_smv.switch) -value 5
  radiobutton $d.options.general.switch6 \
    -text "6. same as 4; no selfloops in deadlock states"\
    -anchor w -variable gvar(option.dlg.model_check_smv.switch) -value 6
  radiobutton $d.options.general.switch7 \
    -text "7. same as 5; no selfloops in deadlock states"\
    -anchor w -variable gvar(option.dlg.model_check_smv.switch) -value 7

  checkbutton $d.options.general.invariants_check \
	-text "Use S-Invariants" -command "switch_inv $d"\
	-variable gvar(option.dlg.model_check_smv.invariants_check) -anchor w

  radiobutton $d.options.general.switch8 \
    -text "Use normal S-Invariants"\
    -anchor w -variable gvar(option.dlg.model_check_smv.inv) -value 1\
    -state disabled
  radiobutton $d.options.general.switch9 \
    -text "Use non-overlapping Invariants"\
    -anchor w -variable gvar(option.dlg.model_check_smv.inv) -value 2\
    -state disabled
  radiobutton $d.options.general.switch10 \
    -text "Use B(PN)^2 related S-Invariants"\
    -anchor w -variable gvar(option.dlg.model_check_smv.inv) -value 3\
    -state disabled

  checkbutton $d.options.general.log_results \
        -text "Add results to doc file" \
	-variable gvar(option.dlg.model_check_smv.log_results) -anchor w

  label $d.options.general.additional -text "Additional SMV Options"\
     -anchor w
  label $d.options.general.additional2 \
	-text "For instance: -f -reorder -inc -v 1 -r -c cachesize" -anchor w
  entry $d.options.general.add_entry \
     -textvariable gvar(option.dlg.model_check_smv.additional)
  pack $d.options.general.title  \
   -padx 2 -pady 4 -side top -anchor w -fill x
  pack $d.options.general.title $d.options.general.warn1 $d.options.general.warn2 \
   -side top -anchor w -fill x

  foreach i {1 2 3 4 5 6 7} {
     pack $d.options.general.switch$i \
     -padx 8 -side top -anchor w -fill x
  }
  pack $d.options.general.invariants_check\
   -padx 8 -side top -anchor w -fill x 
  foreach i {8 9 10} {
     pack $d.options.general.switch$i \
     -padx 8 -side top -anchor w -fill x
  }
  pack $d.options.general.log_results \
    -padx 8 -side top -anchor w -fill x
  pack $d.options.general.additional \
   -padx 8 -pady 4 -side top -anchor w -fill x   
  pack $d.options.general.additional2 \
   -padx 8 -pady 4 -side top -anchor w -fill x   
  pack $d.options.general.add_entry \
   -padx 8 -side top -anchor w -fill x   

  buttonframe $d model_check_smv $prjnum

  switch_inv $d
}  ;# option_ana_model_check_smv_dlg


proc option_bpn2_exp_dlg {} {
  # Den Dialog oeffnen.
  global gvar

  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .dlg_option_bpn2_exp$prjnum]}] {
    raise .dlg_option_bpn2_exp$prjnum
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $d
  wm title $d "B(PN) Expander"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
  checkbutton $d.options.std_init \
    -text "Standard Initialisation" -anchor w \
    -variable gvar(option.dlg.bpn2_exp.std_init)
  scale $d.options.rec_depth \
    -label "Maximal Nesting Depth (Recursion and/or Blocks)" \
    -from 0 -to 20 -length 320 -orient horizontal -relief ridge \
    -variable gvar(option.dlg.bpn2_exp.rec_depth)
  pack $d.options.std_init \
    $d.options.rec_depth \
    -side top -anchor w -fill x

  buttonframe $d bpn2_exp $prjnum
}  ;# option_bpn2_exp_dlg

proc option_pep_to_ina_dlg {} {
  # Den Dialog oeffnen.
  global gvar

  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .dlg_option_pep_to_ina$prjnum]}] {
    raise .dlg_option_pep_to_ina$prjnum
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $d
  wm title $d "PEP -> INA"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
  checkbutton $d.options.references \
    -text "Generate references" -anchor w \
    -variable gvar(option.dlg.pep_to_ina.references)
  checkbutton $d.options.color \
    -text "Generate colored net" -anchor w \
    -variable gvar(option.dlg.pep_to_ina.colored)
  checkbutton $d.options.withtime \
    -text "Generate time interval file" -anchor w \
    -variable gvar(option.dlg.pep_to_ina.withtime)
  pack $d.options.references \
    $d.options.color \
    $d.options.withtime \
    -side top -anchor w -fill x

  buttonframe $d pep_to_ina $prjnum
}  ;# option_pep_to_ina_dlg

proc option_pep_to_spin_dlg {} {
  # Den Dialog oeffnen.
  global gvar

  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .dlg_option_pep_to_spin$prjnum]}] {
    raise .dlg_option_pep_to_spin$prjnum
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $d
  wm title $d "PEP -> SPIN"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"

    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
  checkbutton $d.options.safe \
    -text "Net is 1-safe" -anchor w \
    -variable gvar(option.dlg.pep_to_spin.safe)
  pack $d.options.safe \
    -side top -anchor w -fill x

  buttonframe $d pep_to_spin $prjnum
}  ;# option_pep_to_spin_dlg

proc option_pfa_hl_net_dlg {} {
  # Den Dialog "offnen.
  global gvar
 
  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .dlg_option_pfa_hl_net$prjnum]}] {
    raise .dlg_option_pfa_hl_net$prjnum
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $d
  wm title $d "PFA -> HL Net"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"
 
    # Container f"ur die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen

  checkbutton $d.options.no_scoping \
    -text "No Scoping" -anchor w \
    -variable gvar(option.dlg.pfa_hl_net.no_scoping)
  checkbutton $d.options.disable_blocks \
    -text "No Block Information" -anchor w \
    -variable gvar(option.dlg.pfa_hl_net.disable_blocks)
  checkbutton $d.options.disable_phantoms \
    -text "No Phantom Transitions" -anchor w \
    -variable gvar(option.dlg.pfa_hl_net.disable_phantoms)

  checkbutton $d.options.procedure_net_version_flag \
    -text "Override Default: Procedure Net Version" -anchor w \
    -variable gvar(option.dlg.pfa_hl_net.procedure_net_version_flag)

  scale $d.options.procedure_net_version \
    -label "Procedure Net Version" -from 1 -to 2 \
    -orient horizontal \
    -variable gvar(option.dlg.pfa_hl_net.procedure_net_version)

  scale $d.options.scale \
    -label "Scale" \
    -from 1 -to 50 -length 200 -orient horizontal -relief ridge \
    -variable gvar(option.dlg.pfa_hl_net.scale)

  checkbutton $d.options.use_bpn2_proc_sem \
    -text "Use B(PN)^2 Procedure Semantics with ID set:" -anchor w \
    -variable gvar(option.dlg.pfa_hl_net.use_bpn2_proc_sem)
  entry $d.options.bpn2_proc_sem_idset \
    -relief sunken \
    -textvariable gvar(option.dlg.pfa_hl_net.bpn2_proc_sem_idset)

  pack $d.options.no_scoping \
    $d.options.disable_blocks \
    $d.options.disable_phantoms \
    $d.options.procedure_net_version_flag \
    $d.options.procedure_net_version \
    $d.options.scale \
    $d.options.use_bpn2_proc_sem \
    $d.options.bpn2_proc_sem_idset \
    -side top -anchor w -fill x
 
  buttonframe $d pfa_hl_net $prjnum
}  ;# option_pfa_hl_net_dlg

proc option_pfa_bpn2_dlg {} {
  # Den Dialog oeffnen.
  global gvar
 
  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .option_pfa_bpn2_dlg$prjnum]}] {
    raise .option_pfa_bpn2_dlg$prjnum
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $d
  wm title $d "PFA -> B(PN)"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"
 
    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
  checkbutton $d.options.struct \
    -text "Structure Preserving Compilation" -anchor w \
    -variable gvar(option.dlg.pfa_bpn2.struct)
  pack $d.options.struct \
    -side top -anchor w -fill x
 
  buttonframe $d pfa_bpn2 $prjnum
} ;# option_pfa_bpn2_dlg

proc option_sdl_hl_net_dlg {} {
  # Den Dialog oeffnen.
  global gvar
 
  set prjnum $gvar(actprj)
    # neues TopLevel erzeugen
  if [catch {set d [toplevel .option_sdl_hl_net_dlg$prjnum]}] {
    raise .option_sdl_hl_net_dlg$prjnum
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $d
  wm title $d "SDL -> HL Net"
  wm protocol $d WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $d"
 
    # Container fuer die Optionen
  frame $d.options -relief raised -bd 1
  pack $d.options -side top -fill both
    # Optionen
#  checkbutton $d.options.no_var_rename \
#    -text "do not rename variables" -anchor w \
#    -variable gvar(option.dlg.sdl_hl_net.no_var_rename)
  checkbutton $d.options.no_scoping \
    -text "no scoping" -anchor w \
    -variable gvar(option.dlg.sdl_hl_net.no_scoping)
  checkbutton $d.options.no_implicit_local_procedure_subtypes \
    -text "No implicit local Procedure Subtypes" -anchor w \
    -variable gvar(option.dlg.sdl_hl_net.no_implicit_local_procedure_subtypes)
  scale $d.options.channel_net_version \
    -label "Channel Net Version" \
    -from 0 -to 1 -length 200 -orient horizontal -relief ridge \
    -variable gvar(option.dlg.sdl_hl_net.channel_net_version)
  scale $d.options.process_net_version \
    -label "Process Net Version" \
    -from 0 -to 3 -length 200 -orient horizontal -relief ridge \
    -variable gvar(option.dlg.sdl_hl_net.process_net_version)
  scale $d.options.default_channel_capacity \
    -label "Default Channel Capacity" \
    -from 1 -to 10 -length 200 -orient horizontal -relief ridge \
    -variable gvar(option.dlg.sdl_hl_net.default_channel_capacity)
  scale $d.options.default_maximum_process_instances \
    -label "Default Maximum Number of Process Instances" \
    -from 1 -to 10 -length 200 -orient horizontal -relief ridge \
    -variable gvar(option.dlg.sdl_hl_net.default_maximum_process_instances)
  scale $d.options.default_input_queue_capacity \
    -label "Default Input Queue Capacity" \
    -from 1 -to 10 -length 200 -orient horizontal -relief ridge \
    -variable gvar(option.dlg.sdl_hl_net.default_input_queue_capacity)
  scale $d.options.default_maximum_procedure_instances \
    -label "Default Maximum Number of Procedure Instances" \
    -from 1 -to 10 -length 200 -orient horizontal -relief ridge \
    -variable gvar(option.dlg.sdl_hl_net.default_maximum_procedure_instances)

  pack $d.options.no_scoping $d.options.no_implicit_local_procedure_subtypes \
    $d.options.channel_net_version \
    $d.options.process_net_version \
    $d.options.default_channel_capacity \
    $d.options.default_maximum_process_instances \
    $d.options.default_input_queue_capacity \
    $d.options.default_maximum_procedure_instances \
    -side top -anchor w -fill x
 
  buttonframe $d sdl_hl_net $prjnum
} ;# option_sdl_hl_net_dlg

  #*** end of file optdia.tcl ***

