proc format_ana_result_string {result {number -1}} {
  # Formatiert ein Ergebnis der Analyse in einem anzeigbaren String.
  # result - Zeichenkette aus dem Analysevorgang (YES, NO, UNKNOWN, ...)
  # number - Zahl fuer Bounded-Check, default ist -1 => keine Anzeige
  # return - obiges, nur besser formatiert
  if {$number >= 0} {
    set number_str " $number"
  } else {
    set number_str {}
  }  ;# else, keine Ausgabe
  switch $result {
    "YES"      {return "<YES>$number_str"}
    "MAYBEYES" {return "<MAY BE YES>$number_str"}
    "NO"       {return "<NO>$number_str"}
    "MAYBENO"  {return "<MAY BE NO>$number_str"}
    "YESBUT"   {return "<YES, try S-Invariants>$number_str"}
    "UNKNOWN"  {return "<?>"}
    default    {return "<??>"}
  }  ;# switch $result
}  ;# format_ana_result_string

proc reset_project_ll_net_ana_results prjnum {
  # Die Resultate der Analysen zuruecksetzen.
  global gvar

    # Zugriff auf die Projektvariable erlangen
  upvar #0 prj$prjnum prjx
    # Werte zuruecksetzen
  set prjx(ll_net.ana.t_system) "UNKNOWN"
  set prjx(ll_net.ana.free_choice) "UNKNOWN"
  set prjx(ll_net.ana.safe) "UNKNOWN"
  set prjx(ll_net.ana.bounded) "UNKNOWN"
  set prjx(ll_net.ana.bounded.markings) " "
  set prjx(ll_net.ana.bounded.arcs) " "
  set prjx(ll_net.ana.model_check_t_system) "UNKNOWN"
  set prjx(ll_net.ana.bounded) "UNKNOWN"
  set prjx(ll_net.ana.reachability) "UNKNOWN"
  set prjx(ll_net.ana.reachability.marking) " "
  set prjx(ll_net.ana.reachability.fire_sequence) " "
  set prjx(ll_net.ana.lp_reachability) "UNKNOWN"
  set prjx(ll_net.ana.lp_reachability.marked_places) " "
  set prjx(ll_net.ana.lp_reachability.not_marked_places) " "
  set prjx(ll_net.ana.lp_reachability.result_marking) " "
  set prjx(ll_net.ana.lp_deadlock_free) "UNKNOWN"
  set prjx(ll_net.ana.lp_deadlock_free.marking) " "
  set prjx(ll_net.ana.clp_reachability) "UNKNOWN"
  set prjx(ll_net.ana.clp_mutex) "UNKNOWN"
  set prjx(ll_net.ana.clp_deadlock_free) "UNKNOWN"
}  ;# reset_project_ll_net_ana_results

proc reset_project_promela_text_ana_results {prjnum} {
  # Die Resultate der Analysen zuruecksetzen.
  global gvar

    # Zugriff auf die Projektvariable erlangen
  upvar #0 prj$prjnum prjx
    # Werte zuruecksetzen
  set prjx(promela.text.ana.model_check_spin) "UNKNOWN"
}  ;# reset_project_promela_text_ana_results

proc reset_project_fc2_ana_results {prjnum} {
  # Resultate der Analysen zur"ucksetzen
  global gvar

  # Zugriff auf die Projektvariable erlangen
  upvar #0 prj$prjnum prjx
  # Werte zur"ucksetzen
  set prjx(fc2.ana.deadlock_free) "UNKNOWN"
  set prjx(fc2.ana.deadlock_free.events) ""
}  ;# reset_project_fc2_ana_results

proc reset_project_unfolding_text_ana_results prjnum {
  # Die Resultate der Analysen zuruecksetzen.
  global gvar

    # Zugriff auf die Projektvariable erlangen
  upvar #0 prj$prjnum prjx
    # Werte zuruecksetzen
  set prjx(unfolding.text.ana.deadlock_free) "UNKNOWN"
  set prjx(unfolding.text.ana.deadlock_free.events) " "
  set prjx(unfolding.text.ana.model_check_safe) "UNKNOWN"
  set prjx(unfolding.text.ana.model_check_smv) "UNKNOWN"
}  ;# reset_project_unfolding_text_ana_results

proc sim_area {tl} {
 # Simulationsbereich
  frame $tl.simulate  -relief raised  -bd 2
  pack $tl.simulate  -ipadx 5  -ipady 5  -fill x -expand no
  button $tl.simulate.sim_bpn2  -text "Sim B(PN)^2" \
     -command "simulate $tl.output.textfr.l bpn2" -state disabled
  button $tl.simulate.sim_exp_bpn2  -text "Sim Exp-B(PN)^2" \
     -command "simulate $tl.output.textfr.l exp_bpn2" -state disabled
  button $tl.simulate.sim_hl  -text "Sim HL" \
     -command "simulate $tl.output.textfr.l hl_net" -state disabled
  button $tl.simulate.sim_ll  -text "Sim LL" \
     -command "simulate $tl.output.textfr.l ll_net" -state disabled
  button $tl.simulate.sim_sdl -text "Sim SDL" \
     -command "simulate $tl.output.textfr.l sdl" -state disabled
  pack $tl.simulate.sim_bpn2 \
       $tl.simulate.sim_sdl \
       $tl.simulate.sim_hl \
       $tl.simulate.sim_ll \
       -side left  -expand 1
}

proc formula_transform {actdir} {
  global gvar
  global errorCode

    # get project number
  set prjnum $gvar(actprj)
    # set local name for project variable
  upvar #0 prj$prjnum prjx
    # expand formula
  set prjx(formula.expanded.filename \
    [change_filename_extension $prjx(formula.normal.filename) form_exp]
  if {[file exists $prjx(sdl_ll_ref.filename)] \
      && [file exists $prjx(sdl_info.filename)]} {
    set sdl_opt "-s=$prjx(sdl_ll_ref.filename) -x=$prjx(sdl_info.filename)"
  } else {
    set sdl_opt ""
  }
  set sys_str "exec $gvar(formula_2_formula_exp.name) \
    $prjx(ll_net.filename) -i=$prjx(formula.normal.filename) \
    -o=$prjx(formula.expanded.filename) \
    $sdl_opt"
  DEBUG "Call: $sys_str"
  set errorCode "NONE"
  set sys_res [catch {eval $sys_str} result_text]
  DEBUG " results: $sys_res $result_text"
  if [string compare $errorCode "NONE"] {
    cd $actdir
    error_msg "Error while expanding Formula: $result_text"
    return false
  }  ;# if expansion failed
  return true
}
  
  ###############################
  # Callbacks fuer Analyse-Menu #
  ###############################

proc prj_ana_t_system {} {
  # Falls die LL-Netzdatei vorhanden ist, auf T-System testen.
  global gvar
  global errorCode

    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(ll_net.filename)"] {
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(ll_net.filename) ll_net_t_system_res]
      # Testen, ob LL-Netz ein T-System ist
      # TODO: Optionen verwalten, Aufrufzeile verbessern
    set sys_str "exec $gvar(ll_net_ana_t_system.name) $prjx(ll_net.filename) \
      $result_filename"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while testing LL Net\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    set result [cat $result_filename]
    switch -regexp $result {
      "SYSTEM IST T-SYSTEM"  {set prjx(ll_net.ana.t_system) "YES"}
      "SYSTEM KEIN T-SYSTEM"  {set prjx(ll_net.ana.t_system) "NO"}
      default  {set prjx(ll_net.ana.t_system) "UNKNOWN"}
    }  ;# switch $result
    file delete $result_filename
    new_project_msg $prjnum "LL Net tested for T-System \
      [format_ana_result_string $prjx(ll_net.ana.t_system)]"
    update_project_infos $prjnum
  } else {
    error_msg "LL Net $prjx(actdir)/$prjx(ll_net.filename) does not exists"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_ana_t_system

proc prj_ana_free_choice {} {
  # Falls die LL-Netzdatei vorhanden ist, auf Free-Choice testen.
  global gvar
  global errorCode

    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(ll_net.filename)"] {
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(ll_net.filename) ll_net_free_choice_res]
      # Testen, ob LL-Netz ein Free-Choice-Netz ist
      # TODO: Optionen verwalten, Aufrufzeile verbessern
    set sys_str "exec $gvar(ll_net_ana_free_choice.name) \
      $prjx(ll_net.filename) $result_filename"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while testing LL Net\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    set result [cat $result_filename]
    switch -regexp $result {
      "SYSTEM IS FREE CHOICE"    {set prjx(ll_net.ana.free_choice) "YES"}
      "SYSTEM IS NOT FREE CHOICE"  {set prjx(ll_net.ana.free_choice) "NO"}
      default  {set prjx(ll_net.ana.free_choice) "UNKNOWN"}
    }  ;# switch $result
    file delete $result_filename
    new_project_msg $prjnum "LL Net tested for Free Choice \
      [format_ana_result_string $prjx(ll_net.ana.free_choice)]"
    update_project_infos $prjnum
  } else {
    error_msg "LL Net $prjx(actdir)/$prjx(ll_net.filename) does not exists"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_ana_free_choice

proc prj_ana_safe {} {
  # Falls die LL-Netzdatei vorhanden ist, auf Safe testen.
  global gvar
  global errorCode

    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(ll_net.filename)"] {
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(ll_net.filename) ll_net_safe_res]
      # Testen, ob LL-Netz ein Safe-Netz ist
      # TODO: Optionen verwalten, Aufrufzeile verbessern
    set sys_str "exec $gvar(ll_net_ana_bounded.name) \
      $prjx(ll_net.filename) $result_filename"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while testing LL Net\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    set result [cat $result_filename]
    switch -regexp $result {
      "SYSTEM IST N-BESCHRAENKT"   {set prjx(ll_net.ana.safe) "YES"}
      "SYSTEM NICHT N-BESCHRAENKT" {set prjx(ll_net.ana.safe) "NO"}
      default  {set prjx(ll_net.ana.safe) "UNKNOWN"}
    }  ;# switch $result
    file delete $result_filename
    new_project_msg $prjnum "LL Net tested for Safe \
      [format_ana_result_string $prjx(ll_net.ana.safe)]"
    update_project_infos $prjnum
  } else {
    error_msg "LL Net $prjx(actdir)/$prjx(ll_net.filename) does not exists"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_ana_safe

proc prj_ana_bounded {prjnum toplevel} {
  # Falls die LL-Netzdatei vorhanden ist auf Beschraenktheit testen.
  # prjnum - Die Nummer des Projektes (IN)
  # toplevel - Name des Fensters fuer das Ergebnis (IN, wird veraendert)
  # aus gvar wird option.upper_bound_number benutzt
  global gvar
  global errorCode

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(ll_net.filename)"] {
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(ll_net.filename) ll_net_bounded_res]
      # Testen, ob LL-Netz ein N-beschraenktes-Netz ist
    set opt [get_options_ana_bounded statistics_flag]
      # die derzeitige Zahl zwischenspeichern
    set ubn $gvar(option.ana_bounded.upper_bound_number)
    set sys_str "exec $gvar(ll_net_ana_bounded.name) \
      $prjx(ll_net.filename) -r=$result_filename -b=$ubn $opt"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while testing LL Net\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    set result [cat $result_filename]
    set result_list [split $result "\n"]
    set pos [string first "SYSTEM IST N-BESCHRAENKT" [lindex $result_list 0]]
    if {$pos == -1} {
      set prjx(ll_net.ana.bounded) "NO"
      set prjx(ll_net.ana.bounded.number) $ubn
    } else {  ;# Netz ist beschraenkt
      set prjx(ll_net.ana.bounded) "YES"
      set prjx(ll_net.ana.bounded.number) $ubn
    }  ;# else, Netz ist beschraenkt
    file delete $result_filename
    set display_result [format_ana_result_string $prjx(ll_net.ana.bounded) \
      $prjx(ll_net.ana.bounded.number)]
    new_project_msg $prjnum "LL Net tested for Boundedness $display_result"
    $toplevel.output.result configure  -text $display_result
    if {$statistics_flag} {
      set prjx(ll_net.ana.bounded.markings) [lindex $result_list 1]
      set prjx(ll_net.ana.bounded.arcs) [lindex $result_list 2]
      $toplevel.output.markings configure  -text [lindex $result_list 1]
      $toplevel.output.arcs configure  -text [lindex $result_list 2]
    } else {  ;# keine Statistik
      set prjx(ll_net.ana.bounded.markings) " "
      set prjx(ll_net.ana.bounded.arcs) " "
      $toplevel.output.markings configure  -text " "
      $toplevel.output.arcs configure  -text " "
    }  ;# else, keine Statistik
    update_project_infos $prjnum
  } else {
    error_msg "LL Net $prjx(actdir)/$prjx(ll_net.filename) does not exists"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_ana_bounded

proc prj_ana_bounded_dlg {} {
  # Den Dialog fuer die Eingabe der Zahl fuer den Beschraenktheitstest aufbauen.
  global gvar

    # Die Projektnummer abholen. WICHTIG, dass jenes jetzt passiert!
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx

    # Menupunkt abschalten, spaeter wieder einschalten
  $prjx(project.ana.menu.widget) entryconfigure "Bounded System*" \
    -state disabled

    # Dialog aufbauen
  if [catch {set tl [toplevel .ana_bounded_dlg$prjnum]}] {
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $tl
  wm title $tl "Boundedness Analysis"
  wm protocol $tl WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $tl \
    {$prjx(project.ana.menu.widget) entryconfigure {Bounded System*} \
    -state normal}"

    # Eingabebereich
  frame $tl.input  -bd 2
  pack $tl.input  -side top  -fill both
  label $tl.input.upper_bound_label  -text "Upper Bound"  -bd 2
  set gvar(option.ana_bounded.upper_bound_number) \
    $prjx(ll_net.ana.bounded.number)
#  entry $tl.input.upper_bound_number  -bd 2 
  spinbox $tl.input.upper_bound_number -from 1 -to inf -increment 1 \
    -textvariable gvar(option.ana_bounded.upper_bound_number)
  pack $tl.input.upper_bound_label \
    -side left
  pack $tl.input.upper_bound_number \
    -side left  -fill x  -expand 1

    # Ausgabebereich
  frame $tl.output  -relief sunken  -bd 2
  pack $tl.output  -side top  -fill both  -expand 1
  label $tl.output.result_label  -text "Result of the previous Test:"
  label $tl.output.result \
    -text [format_ana_result_string $prjx(ll_net.ana.bounded) \
      $prjx(ll_net.ana.bounded.number)]
  label $tl.output.markings  -text $prjx(ll_net.ana.bounded.markings)
  label $tl.output.arcs  -text $prjx(ll_net.ana.bounded.arcs)
  pack $tl.output.result_label \
    $tl.output.result \
    -side top  -anchor w
  pack $tl.output.markings \
    $tl.output.arcs \
    -side top  -anchor w  -fill both  -expand 1

  button_frame $tl $prjnum prj_ana_bounded \
	  option_ana_bounded_dlg $prjx(project.ana.menu.widget) \
	  {Bounded System*}
}  ;# prj_ana_bounded_dlg

proc prj_ana_reachability {prjnum toplevel} {
  # Falls die LL-Netzdatei vorhanden ist auf Erreichbarkeit testen.
  # prjnum - Die Nummer des Projektes (IN)
  # toplevel - Name des Fensters fuer das Ergebnis (IN, wird veraendert)
  global gvar
  global errorCode

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(ll_net.filename)"] {
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(ll_net.filename) ll_net_reachability_res]
      # Testen, ob Markierung erreichbar (LL-Netz)
    set opt [get_options_ana_reachability]
      # die derzeitige Markierung zwischenspeichern
    set marking $gvar(option.ana_reachability.marking)
    set marking_filename \
      [change_filename_extension $prjx(ll_net.filename) ll_net_marking]
    echo "$marking\n" $marking_filename
    set sys_str "exec $gvar(ll_net_ana_reachability.name) \
      -I$prjx(ll_net.filename) -M$marking_filename -O$result_filename $opt"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename $marking_filename
      cd $actdir
      error_msg "Error while testing LL Net\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    file delete $marking_filename
    set result [cat $result_filename]
      # NewLines sind in der Projektdatei nicht erlaubt, Trennzeichen deshalb #
    set result [string trim [join [split $result "\n"] "#"]]

    if {$result == ""} {
      set prjx(ll_net.ana.reachability) "NO"
      set add_info {}
    } elseif {[lindex [split $result "#"] 0] == "Net is not bounded"} {
      set prjx(ll_net.ana.reachability) "UNKNOWN"
      set add_info "Net is not bounded"
    } else  {  ;# Netz ist beschraenkt
      set prjx(ll_net.ana.reachability) "YES"
      set add_info {}
    }  ;# else, Netz ist beschraenkt
    set prjx(ll_net.ana.reachability.marking) $marking
    set prjx(ll_net.ana.reachability.fire_sequence) $result
    convert_string_2_text $toplevel.output.textfr.l $result
    file delete $result_filename
    set display_result [format_ana_result_string \
      $prjx(ll_net.ana.reachability)]
    if {[string length $add_info] > 0} {
      set display_result "$display_result ($add_info)"
    }  ;# if sonstnochwas
    new_project_msg $prjnum "LL Net tested for Reachability $display_result"
    $toplevel.output.result configure -text $display_result
    update_project_infos $prjnum
  } else {
    error_msg "LL Net $prjx(actdir)/$prjx(ll_net.filename) does not exists"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_ana_reachability

proc prj_ana_reachability_dlg {} {
  # Den Dialog fuer die Eingabe der Markierung fuer den
  # Erreichbarkeitstest aufbauen.
  global gvar

    # Die Projektnummer abholen. WICHTIG, dass jenes jetzt passiert!
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx

    # Menupunkt abschalten, spaeter wieder einschalten
  $prjx(project.ana.menu.widget) entryconfigure "Reachability*" \
    -state disabled

    # Dialog aufbauen
  if [catch {set tl [toplevel .ana_reachability_dlg$prjnum]}] {
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $tl
  wm title $tl "Reachability Analysis"
  wm protocol $tl WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $tl \
    {$prjx(project.ana.menu.widget) entryconfigure {Reachability*} \
    -state normal}"

  wm minsize $tl 397 400

    # Eingabebereich
  frame $tl.input -bd 2
  pack $tl.input -side top -fill both
  label $tl.input.marking_label -anchor w \
    -text "Marking (List of Places and Number of Tokens)" -bd 2
  label $tl.input.marking_example -anchor w \
    -text "Example: P10, 3 P4, 2 P11 (read: 3xP4, 1xP10, 2xP11)" -bd 2
  set gvar(option.ana_reachability.marking) \
    $prjx(ll_net.ana.reachability.marking)
  entry $tl.input.marking -bd 2 \
    -textvariable gvar(option.ana_reachability.marking)
  pack $tl.input.marking_label \
    $tl.input.marking_example $tl.input.marking \
    -side top -anchor w -fill x -expand 1

    # Ausgabebereich
  frame $tl.output -relief sunken -bd 2
  pack $tl.output -side top -fill both -expand 1
  label $tl.output.result_label -text "Result of the previous Test:"
  label $tl.output.result \
    -text [format_ana_result_string $prjx(ll_net.ana.reachability)]
  pack $tl.output.result_label $tl.output.result \
    -side top -anchor w

    # Textbox fuer die Schaltfolgen
  frame $tl.output.textfr
  pack $tl.output.textfr -fill both -expand 1
  text $tl.output.textfr.l -state disabled -width 60 -height 15 \
    -xscrollcommand "$tl.output.textfr.pad.sx set" \
    -yscrollcommand "$tl.output.textfr.sy set"
  scrollbar $tl.output.textfr.sy -orient vertical \
    -command "$tl.output.textfr.l yview"
  frame $tl.output.textfr.pad
  scrollbar $tl.output.textfr.pad.sx -orient horizontal \
    -command "$tl.output.textfr.l xview"
  set pad [expr [$tl.output.textfr.sy cget -width] + 2 * \
    ([$tl.output.textfr.sy cget -bd] + \
     [$tl.output.textfr.sy cget -highlightthickness])]
  frame $tl.output.textfr.pad.it -width $pad -height $pad
  pack $tl.output.textfr.pad -side bottom -fill x
  pack $tl.output.textfr.pad.it -side right
  pack $tl.output.textfr.pad.sx -side bottom -fill x
  pack $tl.output.textfr.sy -side right -fill y
  pack $tl.output.textfr.l -side left -fill both -expand 1
  convert_string_2_text $tl.output.textfr.l \
    $prjx(ll_net.ana.reachability.fire_sequence)

  sim_area $tl

  button_frame $tl $prjnum prj_ana_reachability \
	  option_ana_reachability_dlg $prjx(project.ana.menu.widget) \
	  {Reachability*}
}  ;# prj_ana_reachability_dlg

proc prj_ana_lp_reachability {prjnum toplevel} {
  # Falls die LL-Netzdatei vorhanden ist auf Erreichbarkeit testen.
  # prjnum - Die Nummer des Projektes (IN)
  # toplevel - Name des Fensters fuer das Ergebnis (IN, wird veraendert)
  global gvar 
  global errorCode
  

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(ll_net.filename)"] {
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(ll_net.filename) \
        ll_net_lp_reachability_res]
      # Testen, ob Markierung erreichbar (LL-Netz)
    set opt [get_options_ana_lp 1]
      # die derzeitige Markierungen zwischenspeichern
    set marking_filename \
      [change_filename_extension $prjx(ll_net.filename) ll_net_lp_marking]

  if {$gvar(option.dlg.ana_lp.use_invariants)} {
    echo "$gvar(option.ana_lp_reachability.extended_marked_places)" $marking_filename
    DEBUG $gvar(option.ana_lp_reachability.extended_marked_places)
    set sys_str "exec $gvar(ll_net_ana_lp.name) \
      -n=$prjx(ll_net.filename) -k=$marking_filename -o=$result_filename $opt"

  } else {
    regsub  -all {,} $gvar(option.ana_lp_reachability.marked_places) \
      {} dummy
    regsub -all {([ ]+)([^ ]+)} " $dummy" {\1+\2} marked_places
    regsub  -all {,} $gvar(option.ana_lp_reachability.not_marked_places) \
      {} dummy
    regsub -all {([ ]+)([^ ]+)} " $dummy" {\1-\2} not_marked_places
      # die derzeitige Markierung zwischenspeichern
    echo "$marked_places\n$not_marked_places" $marking_filename
    set sys_str "exec $gvar(ll_net_ana_lp.name) \
      -n=$prjx(ll_net.filename) -m=$marking_filename -o=$result_filename $opt"
  }

    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename $marking_filename
      cd $actdir
      error_msg "Error while testing LL Net\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    file delete $marking_filename
    set result [cat $result_filename]
    DEBUG " Ergebnisdateiinhalt: $result"
    set ans [lindex [split $result "\n"] 0]
   
    if {$ans == "Marking is not reachable"} {
      set prjx(ll_net.ana.lp_reachability) "NO"
      set add_info {}
    } elseif { $ans == "Marking should not be reachable"} {
      set prjx(ll_net.ana.lp_reachability) "NO"
      set add_info {}
    } else {  ;# Markierung ist evtl. erreichbar
      set prjx(ll_net.ana.lp_reachability) "MAYBEYES"
      set add_info {}
    }  ;# else, Markierung ist evtl. erreichbar
    set prjx(ll_net.ana.lp_reachability.marked_places) \
      $gvar(option.ana_lp_reachability.marked_places)
    set prjx(ll_net.ana.lp_reachability.not_marked_places) \
      $gvar(option.ana_lp_reachability.not_marked_places)
      # NewLines sind in der Projektdatei nicht erlaubt, Trennzeichen deshalb #
    set result [string trim [join [lrange [split $result "\n"] 1 end] "#"]]
    set prjx(ll_net.ana.lp_reachability.result_marking) $result
    set gvar(option.ana_lp_reachability.result_marking) $result
    file delete $result_filename
    set display_result [format_ana_result_string \
      $prjx(ll_net.ana.lp_reachability)]
    if {[string length $add_info] > 0} {
      set display_result "$display_result ($add_info)"
    }  ;# if sonstnochwas
    new_project_msg $prjnum "LL Net tested for LP Reachability $display_result"
    $toplevel.output.result configure -text $display_result
    update_project_infos $prjnum
  } else {
    error_msg "LL Net $prjx(actdir)/$prjx(ll_net.filename) does not exist"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_ana_lp_reachability

proc prj_ana_lp_reachability_dlg {} {
  # Den Dialog fuer die Eingabe der Markierung fuer den
  # Erreichbarkeitstest aufbauen (LP-Version).
  global gvar

    # Die Projektnummer abholen. WICHTIG, dass jenes jetzt passiert!
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx

    # Menupunkt abschalten, spaeter wieder einschalten
  $prjx(project.ana.menu.widget) entryconfigure "LP Reachability*" \
    -state disabled

    # Dialog aufbauen
  if [catch {set tl [toplevel .ana_lp_reachability_dlg$prjnum]}] {
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $tl
  wm title $tl "LP Reachability Analysis"
  wm protocol $tl WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $tl \
    {$prjx(project.ana.menu.widget) entryconfigure {LP Reachability*} \
    -state normal}"

  wm minsize $tl  397 393

    # Eingabebereich
  frame $tl.input -relief sunken -bd 1
  pack $tl.input -ipadx 2 -ipady 2 -side top -fill both
  label $tl.input.marked_label -anchor w \
    -text "Marked Places (List of Places)" -bd 2
  pack $tl.input.marked_label \
    -padx 5 -pady 5 -side top -anchor w -fill x -expand 1
  label $tl.input.marked_example -anchor w \
    -text "Example: P10, P4, P11" -bd 2
  pack $tl.input.marked_example \
    -padx 10 -side top -anchor w -fill x -expand 1
  set gvar(option.ana_lp_reachability.marked_places) \
    $prjx(ll_net.ana.lp_reachability.marked_places)
  entry $tl.input.marked -bd 2 \
    -textvariable gvar(option.ana_lp_reachability.marked_places)
  pack $tl.input.marked \
    -padx 10 -side top -anchor w -fill x -expand 1
  label $tl.input.separator -text " "
  pack $tl.input.separator \
    -side top -anchor w -fill x -expand 1
  label $tl.input.not_marked_label -anchor w \
    -text "Not Marked Places (List of Places)" -bd 2
  pack $tl.input.not_marked_label \
    -padx 5 -pady 5 -side top -anchor w -fill x -expand 1
  label $tl.input.not_marked_example -anchor w \
    -text "Example: P9, P3" -bd 2
  pack $tl.input.not_marked_example \
    -padx 10 -side top -anchor w -fill x -expand 1
  set gvar(option.ana_lp_reachability.not_marked_places) \
    $prjx(ll_net.ana.lp_reachability.not_marked_places)
  entry $tl.input.not_marked -bd 2 \
    -textvariable gvar(option.ana_lp_reachability.not_marked_places)
  pack $tl.input.not_marked \
    -padx 10 -side top -anchor w -fill x -expand 1

    # Eingabebereich fuer Extended
  frame $tl.extended -relief sunken -bd 1
  pack $tl.extended -ipadx 2 -ipady 2 -side top -fill both
  checkbutton $tl.extended.switch -anchor w \
	-text "Extended Reachability Analysis"\
	-variable gvar(option.dlg.ana_lp.use_invariants)\
	-command "switch_lp $tl"
  pack $tl.extended.switch -padx 5 -pady 5 -side top -anchor w  \
	-fill x -expand 1

  label $tl.extended.marked_example -anchor w \
    -text "Example: P10 + P23 + P9 = 2" -bd 2 -state disabled
  pack $tl.extended.marked_example \
    -padx 10 -side top -anchor w -fill x -expand 1 
  entry $tl.extended.marked -bd 2 \
    -textvariable gvar(option.ana_lp_reachability.extended_marked_places)\
	-state disabled
  pack $tl.extended.marked \
    -padx 10 -side top -anchor w -fill x -expand 1

    # Ausgabebereich
  frame $tl.output -relief sunken -bd 1
  pack $tl.output -ipadx 2 -ipady 2 -side top -fill both
  label $tl.output.result_label -anchor w \
    -text "Result of the previous Test:"
  pack $tl.output.result_label \
    -padx 5 -pady 5 -side top -anchor w -fill x -expand 1
  label $tl.output.result -anchor w \
    -text [format_ana_result_string $prjx(ll_net.ana.lp_reachability)]
  set gvar(option.ana_lp_reachability.result_marking) \
    $prjx(ll_net.ana.lp_reachability.result_marking)
  entry $tl.output.result_marking -bd 2 \
    -textvariable gvar(option.ana_lp_reachability.result_marking) \
    -state disabled -width 50
  pack $tl.output.result \
    $tl.output.result_marking \
    -padx 10 -side top -anchor w -fill x -expand 1

  button_frame $tl $prjnum prj_ana_lp_reachability \
	  option_ana_lp_dlg $prjx(project.ana.menu.widget) \
	  {LP Reachability*}
  switch_lp $tl
}  ;# prj_ana_lp_reachability_dlg

proc prj_ana_clp_reachability {prjnum toplevel} {
  # Falls die LL-Netzdatei vorhanden ist auf Erreichbarkeit testen.
  # prjnum - Die Nummer des Projektes (IN)
  # toplevel - Name des Fensters fuer das Ergebnis (IN, wird veraendert)
  global gvar 
  global errorCode
  

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(ll_net.filename)"] {
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(ll_net.filename) \
        ll_net_lp_reachability_res]
      # Testen, ob Markierung erreichbar (LL-Netz)
    set opt [get_options_ana_clp 1]
      # die derzeitige Markierungen zwischenspeichern
    set marking_filename \
      [change_filename_extension $prjx(ll_net.filename) ll_net_lp_marking]

      if {$gvar(option.dlg.ana_clp.mode)==2} {
	  DEBUG [$toplevel.extended.textfr.text get 0.0 end]
	  echo [$toplevel.extended.textfr.text get 0.0 end] $marking_filename
      } else {
    echo "$gvar(option.ana_clp_reachability.marked_places)" $marking_filename
      }
    set sys_str "exec $gvar(ll_net_ana_clp.name) $opt \
      $marking_filename $prjx(unfolding.text.filename) $result_filename "

    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename $marking_filename
      cd $actdir
      error_msg "Error while testing LL Net\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    file delete $marking_filename
    set result [cat $result_filename]
    DEBUG " Ergebnisdateiinhalt: $result"

    set ans [lindex [split $result "\n"] 0]
   
    if {$ans == "NO"} {
      set prjx(ll_net.ana.clp_reachability) "NO"
      set add_info {}
    } else {  ;# Markierung ist erreichbar
      set prjx(ll_net.ana.clp_reachability) "YES"
      set add_info {}
    }  ;# else, Markierung ist evtl. erreichbar
    set prjx(ll_net.ana.clp_reachability.marked_places) \
      $gvar(option.ana_clp_reachability.marked_places)
      # NewLines sind in der Projektdatei nicht erlaubt, Trennzeichen deshalb #
    set result [string trim [join [lrange [split $result "\n"] 1 end] "#"]]
    set prjx(ll_net.ana.clp_reachability.result_sequence) $result
    set gvar(option.ana_clp_reachability.result_sequence) $result
    convert_string_2_text $toplevel.output.textfr.l $result
    file delete $result_filename
    set display_result [format_ana_result_string \
      $prjx(ll_net.ana.clp_reachability)]
    if {[string length $add_info] > 0} {
      set display_result "$display_result ($add_info)"
    }  ;# if sonstnochwas
    new_project_msg $prjnum "LL Net tested for PO Reachability $display_result"
    $toplevel.output.result configure -text $display_result
    update_project_infos $prjnum
  } else {
    error_msg "LL Net $prjx(actdir)/$prjx(ll_net.filename) does not exist"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_ana_clp_reachability

proc prj_ana_clp_reachability_dlg {} {
  # Den Dialog fuer die Eingabe der Markierung fuer den
  # Erreichbarkeitstest aufbauen (PO-Version).
  global gvar

    # Die Projektnummer abholen. WICHTIG, dass jenes jetzt passiert!
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx

    # Menupunkt abschalten, spaeter wieder einschalten
  $prjx(project.ana.menu.widget) entryconfigure "PO Reachability*" \
    -state disabled

    # Dialog aufbauen
  if [catch {set tl [toplevel .ana_clp_reachability_dlg$prjnum]}] {
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $tl
  wm title $tl "PO Reachability Analysis"
  wm protocol $tl WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $tl \
    {$prjx(project.ana.menu.widget) entryconfigure {PO Reachability*} \
    -state normal}"

  wm minsize $tl  397 393

  frame $tl.mode -relief sunken -bd 1
  pack $tl.mode -ipadx 2 -ipady 2 -side top -fill both
  radiobutton $tl.mode.cover -text "Coverability" -value "0" \
    -variable gvar(option.dlg.ana_clp.mode) -anchor w \
    -command "switch_clp $tl"
  radiobutton $tl.mode.reach -text "Reachability" -value "1" \
    -variable gvar(option.dlg.ana_clp.mode) -anchor w \
    -command "switch_clp $tl"
  radiobutton $tl.mode.ext_reach -text "Extended Reachability" -value "2" \
    -variable gvar(option.dlg.ana_clp.mode) -anchor w \
    -command "switch_clp $tl"
  pack $tl.mode.cover $tl.mode.reach $tl.mode.ext_reach -side top -fill both
    # Eingabebereich
  frame $tl.input -relief sunken -bd 1
  pack $tl.input -ipadx 2 -ipady 2 -side top -fill both
  label $tl.input.marked_label -anchor w \
    -text "Marked Places (List of Places)" -bd 2
  pack $tl.input.marked_label \
    -padx 5 -pady 5 -side top -anchor w -fill x -expand 1
  label $tl.input.marked_example -anchor w \
    -text "Example: P10, 3 P4, 2 P11" -bd 2
  pack $tl.input.marked_example \
    -padx 10 -side top -anchor w -fill x -expand 1
  set gvar(option.ana_clp_reachability.marked_places) \
    $prjx(ll_net.ana.clp_reachability.marked_places)
  entry $tl.input.marked -bd 2 \
    -textvariable gvar(option.ana_clp_reachability.marked_places)
  pack $tl.input.marked \
    -padx 10 -side top -anchor w -fill x -expand 1
  label $tl.input.separator -text " "
  pack $tl.input.separator \
    -side top -anchor w -fill x -expand 1

    # Eingabebereich fuer Extended
  frame $tl.extended -relief sunken -bd 1
  pack $tl.extended -ipadx 2 -ipady 2 -side top -fill both

  label $tl.extended.marked_label -anchor w \
    -text "System of linear constraints" -bd 2 -state disabled
  pack $tl.extended.marked_label \
    -padx 5 -pady 5 -side top -anchor w -fill x -expand 1
  label $tl.extended.marked_example -anchor w \
    -text "Example: 2*P10 - 3*P23 + P9 <= 2" -bd 2 -state disabled
  pack $tl.extended.marked_example \
    -padx 10 -side top -anchor w -fill x -expand 1 
  label $tl.extended.marked_example2 -anchor w \
    -text "-P11 - P23 + 4*P6 > 2" -bd 2 -state disabled
  pack $tl.extended.marked_example2 \
    -padx 63 -side top -anchor w -fill x -expand 1 
  frame $tl.extended.textfr
  pack $tl.extended.textfr -fill both -expand 1
  text $tl.extended.textfr.text -height 6 \
    -xscrollcommand "$tl.extended.textfr.pad.sx set" \
    -yscrollcommand "$tl.extended.textfr.sy set" -state disabled
  scrollbar $tl.extended.textfr.sy -orient vertical \
    -command "$tl.extended.textfr.text yview"
  frame $tl.extended.textfr.pad
  scrollbar $tl.extended.textfr.pad.sx -orient horizontal \
    -command "$tl.extended.textfr.text xview"
  set pad [expr [$tl.extended.textfr.sy cget -width] + 2 * \
    ([$tl.extended.textfr.sy cget -bd] + \
     [$tl.extended.textfr.sy cget -highlightthickness])]
  frame $tl.extended.textfr.pad.it -width $pad -height $pad
  pack $tl.extended.textfr.pad -side bottom -fill x
  pack $tl.extended.textfr.pad.it -side right
  pack $tl.extended.textfr.pad.sx -side bottom -fill x
  pack $tl.extended.textfr.sy -side right -fill y
  pack $tl.extended.textfr.text -side left -fill both -expand 1

    # Ausgabebereich
  frame $tl.output -relief sunken -bd 1
  pack $tl.output -ipadx 2 -ipady 2 -side top -fill both
  label $tl.output.result_label -anchor w \
    -text "Result of the previous Test:"
  label $tl.output.result -anchor w \
    -text [format_ana_result_string $prjx(ll_net.ana.clp_reachability)]
  pack $tl.output.result_label \
    $tl.output.result \
    -padx 5 -pady 5 -side top -anchor w -fill x -expand 1
  set gvar(option.ana_clp_reachability.result_sequence) \
    $prjx(ll_net.ana.clp_reachability.result_sequence)
  frame $tl.output.textfr
  pack $tl.output.textfr -fill both -expand 1
  text $tl.output.textfr.l -height 3 -state disabled \
    -xscrollcommand "$tl.output.textfr.pad.sx set" \
    -yscrollcommand "$tl.output.textfr.sy set"
  scrollbar $tl.output.textfr.sy -orient vertical \
    -command "$tl.output.textfr.l yview"
  frame $tl.output.textfr.pad
  scrollbar $tl.output.textfr.pad.sx -orient horizontal \
    -command "$tl.output.textfr.l xview"
  set pad [expr [$tl.output.textfr.sy cget -width] + 2 * \
    ([$tl.output.textfr.sy cget -bd] + \
     [$tl.output.textfr.sy cget -highlightthickness])]
  frame $tl.output.textfr.pad.it -width $pad -height $pad
  pack $tl.output.textfr.pad -side bottom -fill x
  pack $tl.output.textfr.pad.it -side right
  pack $tl.output.textfr.pad.sx -side bottom -fill x
  pack $tl.output.textfr.sy -side right -fill y
  pack $tl.output.textfr.l -side left -fill both -expand 1
  convert_string_2_text $tl.output.textfr.l \
    $prjx(ll_net.ana.clp_reachability.result_sequence)

  sim_area $tl

  button_frame $tl $prjnum prj_ana_clp_reachability \
	  option_ana_clp_dlg $prjx(project.ana.menu.widget) \
	  {PO Reachability*}
  switch_clp $tl
}  ;# prj_ana_clp_reachability_dlg

proc prj_ana_clp_mutex {prjnum toplevel} {
  # Falls die LL-Netzdatei vorhanden ist auf Erreichbarkeit testen.
  # prjnum - Die Nummer des Projektes (IN)
  # toplevel - Name des Fensters fuer das Ergebnis (IN, wird veraendert)
  global gvar 
  global errorCode
  

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(ll_net.filename)"] {
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(ll_net.filename) \
        ll_net_lp_reachability_res]
      # Testen, ob Markierung erreichbar (LL-Netz)
    set opt [get_options_ana_clp 2]
      # die derzeitige Markierungen zwischenspeichern
    set marking_filename \
      [change_filename_extension $prjx(ll_net.filename) ll_net_lp_marking]

    echo "$gvar(option.ana_clp_mutex.mutex_places)" $marking_filename
    set sys_str "exec $gvar(ll_net_ana_clp.name) $opt \
      $marking_filename $prjx(unfolding.text.filename) $result_filename "

    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename $marking_filename
      cd $actdir
      error_msg "Error while testing LL Net\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    file delete $marking_filename
    set result [cat $result_filename]
    DEBUG " Ergebnisdateiinhalt: $result"

    set ans [lindex [split $result "\n"] 0]
   
    if {$ans == "NO"} {
      set prjx(ll_net.ana.clp_mutex) "YES"
      set add_info {}
    } else {  ;# Markierung ist erreichbar
      set prjx(ll_net.ana.clp_mutex) "NO"
      set add_info {}
    }  ;# else, Markierung ist evtl. erreichbar
    set prjx(ll_net.ana.clp_mutex.mutex_places) \
      $gvar(option.ana_clp_mutex.mutex_places)
      # NewLines sind in der Projektdatei nicht erlaubt, Trennzeichen deshalb #
    set result [string trim [join [lrange [split $result "\n"] 1 end] "#"]]
    set prjx(ll_net.ana.clp_mutex.result_sequence) $result
    set gvar(option.ana_clp_mutex.result_sequence) $result
    convert_string_2_text $toplevel.output.textfr.l $result
    file delete $result_filename
    set display_result [format_ana_result_string \
      $prjx(ll_net.ana.clp_mutex)]
    if {[string length $add_info] > 0} {
      set display_result "$display_result ($add_info)"
    }  ;# if sonstnochwas
    new_project_msg $prjnum "LL Net tested for PO Mutual Exclusion $display_result"
    $toplevel.output.result configure -text $display_result
    update_project_infos $prjnum
  } else {
    error_msg "LL Net $prjx(actdir)/$prjx(ll_net.filename) does not exist"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_ana_clp_mutex

proc prj_ana_clp_mutex_dlg {} {
  # Den Dialog fuer die Eingabe der Markierung fuer den
  # Mutextest aufbauen (PO-Version).
  global gvar

    # Die Projektnummer abholen. WICHTIG, dass jenes jetzt passiert!
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx

    # Menupunkt abschalten, spaeter wieder einschalten
  $prjx(project.ana.menu.widget) entryconfigure "PO Mutual Exclusion*" \
    -state disabled

    # Dialog aufbauen
  if [catch {set tl [toplevel .ana_clp_mutex_dlg$prjnum]}] {
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $tl
  wm title $tl "PO Mutual Exclusion Analysis"
  wm protocol $tl WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $tl \
    {$prjx(project.ana.menu.widget) entryconfigure {PO Mutual Exclusion*} \
    -state normal}"

  wm minsize $tl  397 393

    # Eingabebereich
  frame $tl.input -relief sunken -bd 1
  pack $tl.input -ipadx 2 -ipady 2 -side top -fill both
  label $tl.input.mutex_label -anchor w \
    -text "Mutually exclusive Places (List of Places)" -bd 2
  pack $tl.input.mutex_label \
    -padx 5 -pady 5 -side top -anchor w -fill x -expand 1
  label $tl.input.mutex_example -anchor w \
    -text "Example: P10, P4, P11" -bd 2
  pack $tl.input.mutex_example \
    -padx 10 -side top -anchor w -fill x -expand 1
  set gvar(option.ana_clp_mutex.mutex_places) \
    $prjx(ll_net.ana.clp_mutex.mutex_places)
  entry $tl.input.mutex -bd 2 \
    -textvariable gvar(option.ana_clp_mutex.mutex_places)
  pack $tl.input.mutex \
    -padx 10 -side top -anchor w -fill x -expand 1

    # Ausgabebereich
  frame $tl.output -relief sunken -bd 1
  pack $tl.output -ipadx 2 -ipady 2 -side top -fill both -expand yes
  label $tl.output.result_label -anchor w \
    -text "Result of the previous Test:"
  label $tl.output.result -anchor w \
    -text [format_ana_result_string $prjx(ll_net.ana.clp_mutex)]
  pack $tl.output.result_label \
    $tl.output.result \
    -padx 5 -pady 5 -side top -anchor w -fill x -expand 1
  set gvar(option.ana_clp_mutex.result_sequence) \
    $prjx(ll_net.ana.clp_mutex.result_sequence)
  frame $tl.output.textfr
  pack $tl.output.textfr -fill both -expand 1
  text $tl.output.textfr.l -height 3 -state disabled \
    -xscrollcommand "$tl.output.textfr.pad.sx set" \
    -yscrollcommand "$tl.output.textfr.sy set"
  scrollbar $tl.output.textfr.sy -orient vertical \
    -command "$tl.output.textfr.l yview"
  frame $tl.output.textfr.pad
  scrollbar $tl.output.textfr.pad.sx -orient horizontal \
    -command "$tl.output.textfr.l xview"
  set pad [expr [$tl.output.textfr.sy cget -width] + 2 * \
    ([$tl.output.textfr.sy cget -bd] + \
     [$tl.output.textfr.sy cget -highlightthickness])]
  frame $tl.output.textfr.pad.it -width $pad -height $pad
  pack $tl.output.textfr.pad -side bottom -fill x
  pack $tl.output.textfr.pad.it -side right
  pack $tl.output.textfr.pad.sx -side bottom -fill x
  pack $tl.output.textfr.sy -side right -fill y
  pack $tl.output.textfr.l -side left -fill both -expand 1
  convert_string_2_text $tl.output.textfr.l \
    $prjx(ll_net.ana.clp_mutex.result_sequence)

  sim_area $tl

  button_frame $tl $prjnum prj_ana_clp_mutex option_ana_clp_dlg \
	  $prjx(project.ana.menu.widget) {PO Mutual Exclusion*}
}  ;# prj_ana_clp_mutex_dlg

proc prj_ana_deadlock_free {prjnum toplevel} {
  # Falls die Entfaltung vorhanden ist, auf Deadlock-free testen.
  # prjnum - Die Nummer des Projektes (IN)
  # toplevel - Name des Fensters fuer das Ergebnis (IN, wird veraendert)
  #TODO testen ob Entfaltung zu LL Net gehoert!
  global gvar
  global errorCode

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if {![file exists "$prjx(unfolding.text.filename)"]} {
    if {[catch prj_exe_ll_net_unfolding]} {
      error_msg "Prefix $prjx(actdir)/$prjx(unfolding.text.filename) \
        does not exist"
      return
    }
  }
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(unfolding.text.filename) \
      unfolding_deadlock_free_res]
      # Testen, ob eine Entfaltung Deadlock-free ist
      # TODO: Optionen verwalten, Aufrufzeile verbessern
    set sys_str "exec $gvar(unfolding_text_ana_deadlock_free.name) \
      $prjx(unfolding.text.filename) $result_filename"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while testing Prefix\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    set result [cat $result_filename]
    switch -glob [string toupper $result] {
      "TRUE*"   {set prjx(unfolding.text.ana.deadlock_free) "YES"}
      "FALSE*"  {set prjx(unfolding.text.ana.deadlock_free) "NO"}
      default  {set prjx(unfolding.text.ana.deadlock_free) "UNKNOWN"}
    }  ;# switch $result
      # NewLines sind in der Projektdatei nicht erlaubt, Trennzeichen deshalb #
      # Die erste Zeile mit dem TRUE oder FALSE entfernen!
    set prjx(unfolding.text.ana.deadlock_free.events) $result
    set result [string trim [join [lrange [split $result "\n"] 1 end] "#"]]
    convert_string_2_text $toplevel.output.textfr.l $result
    file delete $result_filename
    set display_result [format_ana_result_string \
      $prjx(unfolding.text.ana.deadlock_free)]
    new_project_msg $prjnum "Prefix tested for Deadlock-free $display_result"
    $toplevel.output.result configure -text $display_result
    update_project_infos $prjnum
  cd $actdir
}  ;# prj_ana_deadlock_free

proc prj_ana_mcs_deadlock_free {prjnum toplevel} {
  # Check unfolding (if existent) for deadlock
  # using mcsmodels
  # prjnum - Die Nummer des Projektes (IN)
  # toplevel - Name des Fensters fuer das Ergebnis (IN, wird veraendert)
  #TODO testen ob Entfaltung zu LL Net gehoert!
  global gvar
  global errorCode

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if {![file exists "$prjx(unfolding.text.filename)"]} {
    if {[catch prj_exe_ll_net_unfolding]} {
      error_msg "Prefix $prjx(actdir)/$prjx(unfolding.text.filename) \
        does not exist"
      return
    }
  }
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(unfolding.text.filename) \
      unfolding_deadlock_free_res]
      # Testen, ob eine Entfaltung Deadlock-free ist
      # TODO: Optionen verwalten, Aufrufzeile verbessern
    set options [get_options_mcsmodels true]
    set sys_str "exec $gvar(unfolding_ana_mcs_deadlock_free.name) \
      -D $options $prjx(unfolding.text.filename) $result_filename"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while testing Prefix\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    set result [cat $result_filename]
    switch -glob [string toupper $result] {
      "TRUE*"   {set prjx(unfolding.ana.mcs_deadlock_free) "YES"}
      "FALSE*"  {set prjx(unfolding.ana.mcs_deadlock_free) "NO"}
      default  {set prjx(unfolding.ana.mcs_deadlock_free) "UNKNOWN"}
    }  ;# switch $result
      # NewLines sind in der Projektdatei nicht erlaubt, Trennzeichen deshalb #
      # Die erste Zeile mit dem TRUE oder FALSE entfernen!
    set list [split $result "\n"]
    set time [lindex $list end]
    set list [lrange $list 1 [expr [llength $list]-3] ]
    set result [string trim [join $list]]
    regsub -all { } $result "" result
    if {[regsub -all {E[0-9]+\(([^)]+)\)} $result {\1} new_result]} {
      regsub -all ",\n" $new_result {,} result
      set result "$time##_SEQUENCE:#$result"
    }
    set prjx(unfolding.ana.mcs_deadlock_free.events) $result
    convert_string_2_text $toplevel.output.textfr.l $result
    file delete $result_filename
    set display_result [format_ana_result_string \
      $prjx(unfolding.ana.mcs_deadlock_free)]
    new_project_msg $prjnum "Prefix tested for Deadlock-free $display_result"
    $toplevel.output.result configure  -text $display_result
    update_project_infos $prjnum
  cd $actdir
}  ;# prj_ana_mcs_deadlock_free

proc prj_ana_bound_deadlock_free {prjnum toplevel} {
  # Check unfolding (if existent) for deadlock
  # using mcsmodels
  # prjnum - Die Nummer des Projektes (IN)
  # toplevel - Name des Fensters fuer das Ergebnis (IN, wird veraendert)
  #TODO testen ob Entfaltung zu LL Net gehoert!
  global gvar
  global errorCode

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(ll_net.filename)"] {
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(ll_net.filename) \
      unfolding_deadlock_free_res]
      # Testen, ob eine Entfaltung Deadlock-free ist
    set options [get_options_boundsmodels yes]
    set sys_str "exec $gvar(ll_net_ana_boundsmodels.name) \
      -D $options $prjx(ll_net.filename) $gvar(ll_net_ana_boundsmodels_extras.name) >$result_filename"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while testing Prefix\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    set result [cat $result_filename]
    switch -glob [string toupper $result] {
      "*TRUE*"   {set prjx(unfolding.ana.bound_deadlock_free) "NO"}
      "*FALSE*"  {set prjx(unfolding.ana.bound_deadlock_free) "YES"}
      default  {set prjx(unfolding.ana.bound_deadlock_free) "UNKNOWN"}
    }  ;# switch $result
read_file_2_textwidget $toplevel.output.textfr.l \
$result_filename end true
    file delete $result_filename
    set display_result [format_ana_result_string \
      $prjx(unfolding.ana.bound_deadlock_free)]
    new_project_msg $prjnum "Prefix tested for Deadlock-free $display_result"
    $toplevel.output.result configure -text $display_result
    update_project_infos $prjnum
  } else {
    error_msg "Prefix $prjx(actdir)/$prjx(ll_net.filename) \
      does not exists"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_ana_bound_deadlock_free

proc prj_ana_prod_deadlock_free {prjnum toplevel} {
  # Generate prod file from ll-net. Call prod to generate
  # reduced state space after inserting deadlock search
  # code. In case of deadlock generate sequence with probe
  # prjnum - Die Nummer des Projektes (IN)
  # toplevel - Name des Fensters fuer das Ergebnis (IN, wird veraendert)
  global gvar
  global errorCode
  global env

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(ll_net.filename)"] {
      # generate prod file
    set prod_filename \
      [change_filename_extension $prjx(ll_net.filename) prod]
    set sys_str "exec $gvar(ll_net_to_prod.name) \
      $prjx(ll_net.filename) $prod_filename"
    DEBUG "Call: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " resulted: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while generating prod file\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    # Copy prod file to temporary file with .net ending
    set temp_filename \
     [lindex [split $prjx(ll_net.filename) .] 0]_prod.net
    file copy -force $prod_filename $temp_filename

    #Append deadlock check sequence to temp file
    set temp_filehandler [open $temp_filename a]
    puts $temp_filehandler "#place tester lo(<.0.>) hi(<.0.>) mk(<.0.>)"
    puts $temp_filehandler "#tester tester deadlock(<.0.>)"
    close $temp_filehandler

    set env(PRODPATH) $gvar(bin.path)
    set prod_filename \
      [change_filename_extension $temp_filename init]
    set sys_str "exec $gvar(bin.path)/prod $prod_filename"
    DEBUG "Call: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG "resulted: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      DEBUG "Error while generating reach graph generator: $errorCode"
      cd $actdir
      return
    } ;# Fehler beim Konvertieren

    #check for deadlock
    set prod_filename [file rootname $temp_filename]
    set sys_str "exec ./$prod_filename"
    DEBUG "Call: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG "resulted: $sys_res $result_text"
#    if [string compare $errorCode "NONE"] {
#      DEBUG "Error while checking for deadlock: $errorCode"
#      cd $actdir
#      return
#    } ;# Fehler beim Konvertieren
    set result ""
    
    if {$result_text!=""} {
      # Deadlock exists
      set prjx(ll_net.ana.prod_deadlock_free) "NO"

      #Generate sequence
      set result_filename \
        [change_filename_extension $temp_filename out]
      echo "build volatile verbose %1\nquery volatile verbose bspan(true) %1\n"\
	      $prod_filename.probe
      set sys_str "exec probe -d $prod_filename <$prod_filename.probe >$result_filename"
      DEBUG "Aufruf: $sys_str"
      set errorCode "NONE"
      set sys_res [catch {eval $sys_str} result_text]
      DEBUG " ergab: $sys_res $result_text"
#      if [string compare $errorCode "NONE"] {
#        DEBUG "Error while checking firing sequence: $errorCode"
#        cd $actdir
#        return
#      } ;# Fehler beim Konvertieren

	# search all Arrow lines (Transitions)
      pattern_in_file $result_filename Arrow result_text
	# remove everything except transition name and preceeding comma
      regsub -all {[^,]+( |$)} $result_text "" result
      set result [string trim $result ,]
      if {$result!=""} {
        set result_text "_SEQUENCE:\n$result"
      } else {
        set result_text ""
      }
      # Ergebnis zum Speichern im Projekt aufbereiten
      # (keine \n erlaubt!)
      set result [string trim [join [lrange [split $result_text "\n"] 0 end] "#"]]
    } else {
      # No deadlock
      set prjx(ll_net.ana.prod_deadlock_free) "YES"
    } ;# if deadlock exists
    set prjx(ll_net.ana.prod_deadlock_free.events) $result
    convert_string_2_text $toplevel.output.textfr.l $result
    set display_result [format_ana_result_string \
      $prjx(ll_net.ana.prod_deadlock_free)]
    new_project_msg $prjnum "Tested for PROD Deadlock-free $display_result"
    $toplevel.output.result configure -text $display_result
    update_project_infos $prjnum

    #cleanup
    set prod_filename \
      [change_filename_extension $temp_filename clean]
    set sys_str "exec $gvar(bin.path)/prod $prod_filename"
    set sys_res [catch {eval $sys_str}]
    set prod_filename \
      [change_filename_extension $temp_filename src]
    file delete $prod_filename $temp_filename
  } else {
    error_msg "File $prjx(actdir)/$prjx(ll_net.filename) \
      does not exist"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_ana_prod_deadlock_free

proc prj_ana_deadlock_dlg {} {
  # Den Dialog fuer die Ausgabe des Deadlock-Tests aufbauen.
  # checker = safe, fc2, lp, clp, prod, mcs, bound
  global gvar Dialog

  if {[catch {set gvar(option.dlg.ana_deadlock.checker)}]} {
    set gvar(option.dlg.ana_deadlock.checker) mcmillan
  }

    # Die Projektnummer abholen. WICHTIG, dass jenes jetzt passiert!
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx

  if [winfo exists .ana_deadlock_dlg] {
	set tl .ana_deadlock_dlg
	raise $tl
  } else {
	set tl [toplevel .ana_deadlock_dlg]
  }
  set Dialog $tl

    # Menupunkt abschalten, spaeter wieder einschalten
   $prjx(project.ana.menu.widget) entryconfigure "Deadlock-free System*" \
    -state disabled

  add_dlg_widget $prjnum $tl
  wm protocol $tl WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $tl \
	  {$prjx(project.ana.menu.widget) entryconfigure {Deadlock-free System*} \
    -state normal}"

  wm minsize $tl 500 350

  frame $tl.checker -relief sunken -bd 0
  pack $tl.checker -side top -fill both -expand yes

  radiobutton $tl.checker.mcmillan -variable \
    gvar(option.dlg.ana_deadlock.checker) -anchor w \
    -text "McMillan" -value mcmillan -command \
    {prj_ana_deadlock_set $gvar(option.dlg.ana_deadlock.checker)}
  radiobutton $tl.checker.lp -variable \
    gvar(option.dlg.ana_deadlock.checker) -anchor w \
    -text "LP" -value lp -command \
    {prj_ana_deadlock_set $gvar(option.dlg.ana_deadlock.checker)}
  radiobutton $tl.checker.fc2 -variable \
    gvar(option.dlg.ana_deadlock.checker) -anchor w \
    -text "FC2" -value fc2 -command \
    {prj_ana_deadlock_set $gvar(option.dlg.ana_deadlock.checker)}
  radiobutton $tl.checker.clp -variable \
    gvar(option.dlg.ana_deadlock.checker) -anchor w \
    -text "CLP" -value clp -command \
    {prj_ana_deadlock_set $gvar(option.dlg.ana_deadlock.checker)}
  radiobutton $tl.checker.prod -variable \
    gvar(option.dlg.ana_deadlock.checker) -anchor w \
    -text "PROD" -value prod -command \
    {prj_ana_deadlock_set $gvar(option.dlg.ana_deadlock.checker)}
  radiobutton $tl.checker.mcs -variable \
    gvar(option.dlg.ana_deadlock.checker) -anchor w \
    -text "MCSmodels" -value mcs -command \
    {prj_ana_deadlock_set $gvar(option.dlg.ana_deadlock.checker)}
  radiobutton $tl.checker.bound -variable \
    gvar(option.dlg.ana_deadlock.checker) -anchor w \
    -text "BoundSmodels" -value bound -command \
    {prj_ana_deadlock_set $gvar(option.dlg.ana_deadlock.checker)}

  pack $tl.checker.mcmillan $tl.checker.lp $tl.checker.fc2 -side left -anchor w
  pack $tl.checker.clp $tl.checker.prod $tl.checker.mcs $tl.checker.bound -side left -anchor w

  if {$gvar(option.dlg.ana_deadlock.checker)=="clp"} {
    wm minsize $tl 500 405
    frame $tl.options -relief sunken -bd 2
    pack $tl.options -side top -fill both -expand yes

    checkbutton $tl.options.dead_tokens_allowed -variable \
      gvar(option.dlg.ana_clp.dead_tokens_allowed) -anchor w \
      -text "Allow dead tokens on internal places"
    checkbutton $tl.options.handle_terminal_marking -variable \
      gvar(option.dlg.ana_clp.handle_terminal_marking) -anchor w \
      -text "Handle terminal markings"
#      -command "$tl.options.dead_tokens_allowed configure -state \
#      if {$gvar(option.dlg.ana_clp.handle_terminal_marking)} {enabled} else {disabled}"
    pack $tl.options.handle_terminal_marking -side top -anchor w
    pack $tl.options.dead_tokens_allowed -padx 20 -side top -anchor w
  }

  # Ausgabebereich
  frame $tl.output  -relief sunken  -bd 2
  pack $tl.output  -side top  -fill both -expand yes
  frame $tl.output.resf -relief sunken -bd 0
  pack $tl.output.resf -fill x -expand yes
  label $tl.output.result_label -text "Result of the previous Test:"
  label $tl.output.result  -text ""
  pack $tl.output.result_label $tl.output.result -in $tl.output.resf \
    -side left -anchor nw -fill none

    # Textbox fuer die verschiedenen Schaltfolgen
  frame $tl.output.textfr
  pack $tl.output.textfr -fill both -expand 1
  text $tl.output.textfr.l \
    -xscrollcommand "$tl.output.textfr.pad.sx set" \
    -yscrollcommand "$tl.output.textfr.sy set" \
    -width 60 -height 15 -state disabled
  scrollbar $tl.output.textfr.sy -orient vertical \
    -command "$tl.output.textfr.l yview"
  frame $tl.output.textfr.pad
  scrollbar $tl.output.textfr.pad.sx -orient horizontal \
    -command "$tl.output.textfr.l xview"
  set pad [expr [$tl.output.textfr.sy cget -width] + 2 * \
    ([$tl.output.textfr.sy cget -bd] + \
     [$tl.output.textfr.sy cget -highlightthickness])]
  frame $tl.output.textfr.pad.it -width $pad -height $pad
  pack $tl.output.textfr.pad -side bottom -fill x
  pack $tl.output.textfr.pad.it -side right
  pack $tl.output.textfr.pad.sx -side bottom -fill x
  pack $tl.output.textfr.sy -side right -fill y
  pack $tl.output.textfr.l -side left -fill both -expand 1

  prj_ana_deadlock_set $gvar(option.dlg.ana_deadlock.checker)

  sim_area $tl

  button_frame $tl $prjnum deadlock_button_ok deadlock_button_opts $prjx(project.ana.menu.widget) \
	  "Deadlock-free System*"
}  ;# prj_ana_deadlock_free_dlg

proc deadlock_button_ok {prjnum tl} {
  global dl_opts Dialog
  $Dialog.output.textfr.l delete 1.0 end
  set dl_opts(runtime) [regexp -inline {[0-9]*} [time {$dl_opts(command) $prjnum $tl}]]
  $Dialog.output.result configure -text "[$Dialog.output.result cget -text] $dl_opts(runtime) ms"
}

proc deadlock_button_opts {} {
  global dl_opts
  if {$dl_opts(option)!=""} {
    $dl_opts(option)
  }
}

proc prj_ana_deadlock_set {{tool mcmillan}} {
  global gvar dl_opts Dialog

    # Die Projektnummer abholen. WICHTIG, dass jenes jetzt passiert!
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
  switch $tool {
	mcmillan "set dl_opts(result) $prjx(unfolding.text.ana.deadlock_free)
             set dl_opts(command) prj_ana_deadlock_free
             set dl_opts(events) $prjx(unfolding.text.ana.deadlock_free.events)
             set dl_opts(option) {}
             wm title $Dialog {Deadlock-free Analysis}"
	lp "set dl_opts(result) $prjx(ll_net.ana.lp_deadlock_free)
             set dl_opts(command) prj_ana_lp_deadlock_free
             set dl_opts(events) $prjx(unfolding.text.ana.deadlock_free.events)
             set dl_opts(option) option_ana_lp_dlg
             wm title $Dialog {LP Deadlock-free Analysis}"
	fc2 "set dl_opts(result) $prjx(fc2.ana.deadlock_free)
             set dl_opts(command) prj_ana_fc2_deadlock_free
             set dl_opts(events) $prjx(fc2.ana.deadlock_free.events)
             set dl_opts(option) {}
             wm title $Dialog {FC2 Deadlock-free Analysis}"
	clp "set dl_opts(result) $prjx(ll_net.ana.clp_deadlock_free)
             set dl_opts(command) prj_ana_clp_deadlock_free
             set dl_opts(events) $prjx(ll_net.ana.clp_deadlock_free.result_sequence)
             set dl_opts(option) option_ana_clp_dlg
             wm title $Dialog {PO Deadlock-free Analysis}"
	prod "set dl_opts(result) $prjx(ll_net.ana.prod_deadlock_free)
             set dl_opts(command) prj_ana_prod_deadlock_free
             set dl_opts(events) $prjx(ll_net.ana.prod_deadlock_free.result_sequence)
             set dl_opts(option) {}
             wm title $Dialog {PROD Deadlock-free Analysis}"
	mcs "set dl_opts(result) $prjx(unfolding.ana.mcs_deadlock_free)
             set dl_opts(command) prj_ana_mcs_deadlock_free
             set dl_opts(events) $prjx(unfolding.ana.mcs_deadlock_free.result_sequence)
             set dl_opts(option) {option_ana_mcsmodels_dlg}
	     wm title $Dialog {mcsmodels Deadlock-free Analysis}"
	bound "set dl_opts(result) $prjx(unfolding.ana.bound_deadlock_free)
             set dl_opts(command) prj_ana_bound_deadlock_free 
             set dl_opts(events) $prjx(unfolding.ana.bound_deadlock_free.result_sequence)
             set dl_opts(option) option_ana_boundsmodels_dlg
	     wm title $Dialog {boundsmodels Deadlock-free Analysis}"
  }
#  if {$dl_opts(option)!=""} {
#    $Dialog.commands.options configure -state normal
#  } else {
#    $Dialog.commands.options configure -state disabled
#  }
  set dl_opts(runtime) ""
  convert_string_2_text $Dialog.output.textfr.l $dl_opts(events)
  $Dialog.output.result configure -text "[format_ana_result_string $dl_opts(result)] $dl_opts(runtime)"
}
 
proc prj_ana_lp_deadlock_free {prjnum toplevel} {
  # Falls das LL Netz vorhanden ist, auf Deadlock-free testen.
  # prjnum - Die Nummer des Projektes (IN)
  # toplevel - Name des Fensters fuer das Ergebnis (IN, wird veraendert)
  global gvar
  global errorCode

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob das LL Net da ist
  if [file exists "$prjx(ll_net.filename)"] {
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(ll_net.filename) \
      ll_net_lp_deadlock_free_res]
    set opt [get_options_ana_lp]
      # Testen, ob das LL Netz Deadlock-free ist
    set use_net [lsearch $opt "-D"]
    if {$use_net == -1} {
      DEBUG "Use unfolding for deadlock-check"
      if {![file exists "$prjx(unfolding.text.filename)"]} {
	if {[catch prj_exe_ll_net_unfolding]} {
	  error_msg "Prefix $prjx(actdir)/$prjx(unfolding.text.filename) \
	    does not exist"
	  return
	}
      }
      if [file exists "$prjx(unfolding.text.filename)"] {
        set sys_str "exec $gvar(ll_net_ana_lp.name) \
          -n=$prjx(unfolding.text.filename) -o=$result_filename $opt"
        DEBUG "Aufruf: $sys_str"
      } else {
        error_msg "Prefix $prjx(actdir)/$prjx(unfolding.text.filename) \
          does not exists"
      }
    } else {
       DEBUG "Use LL net for deadlock-check"
       set sys_str "exec $gvar(ll_net_ana_lp.name) \
         -n=$prjx(ll_net.filename) -o=$result_filename $opt"
       DEBUG "Aufruf: $sys_str"
    }  ;# use unfolding
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while testing LL Net\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    set result [cat $result_filename]
    DEBUG " Ergebnisdateiinhalt: $result"

   set ans [lindex [split $result "\n"] 0]

    if {$ans == "Net is deadlock-free"} {
      set prjx(ll_net.ana.lp_deadlock_free) "YES"
      set add_info {}
    } elseif { $ans == "Net should be deadlock-free"} {
         set prjx(ll_net.ana.lp_deadlock_free) "YESBUT"
         set add_info {}
    } elseif { $ans == "Net has a deadlock"} {
         set prjx(ll_net.ana.lp_deadlock_free) "NO"
         set add_info {}
    } else { ;# Deadlock ist evtl. vorhanden
         set prjx(ll_net.ana.lp_deadlock_free) "MAYBENO"
         set add_info {}
    }  ;# else, Deadlock ist evtl. vorhanden
      # NewLines sind in der Projektdatei nicht erlaubt, Trennzeichen deshalb #
    set result [string trim [join [lrange [split $result "\n"] 1 end] "#"]]
    set prjx(ll_net.ana.lp_deadlock_free.marking) $result
    set gvar(option.ana_lp_deadlock_free.marking) $result
    convert_string_2_text $toplevel.output.textfr.l $result
    file delete $result_filename
    set display_result [format_ana_result_string \
      $prjx(ll_net.ana.lp_deadlock_free)]
    new_project_msg $prjnum \
      "LL Net tested for Deadlock-free (LP) $display_result"
    $toplevel.output.result configure  -text $display_result
    update_project_infos $prjnum
  } else {
    error_msg "LL Net $prjx(actdir)/$prjx(ll_net.filename) does not exists"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_ana_lp_deadlock_free

proc toggle_slaves {list state} {
  # list is list of widgets to toggle
  foreach i list {
      [i configure -state $state]
  }
}

proc prj_ana_clp_deadlock_free {prjnum toplevel} {
  # Falls die Entfaltung vorhanden ist, auf Deadlock-free testen.
  # prjnum - Die Nummer des Projektes (IN)
  # toplevel - Name des Fensters fuer das Ergebnis (IN, wird veraendert)
  #TODO testen ob Entfaltung zu LL Net gehoert!
  global gvar
  global errorCode

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if {![file exists "$prjx(unfolding.text.filename)"]} {
    if {[catch prj_exe_ll_net_unfolding]} {
      error_msg "Prefix $prjx(actdir)/$prjx(unfolding.text.filename) \
        does not exist"
      return
    }
  }
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(unfolding.text.filename) \
      unfolding_deadlock_free_res]
      # Testen, ob eine Entfaltung Deadlock-free ist
      # TODO: Optionen verwalten, Aufrufzeile verbessern
    set opt [get_options_ana_clp 0]
      # 0 for deadlock check
    if {$gvar(option.dlg.ana_clp.handle_terminal_marking)} {
	set opt "$opt $prjx(ll_net.filename)" }
    set sys_str "exec $gvar(ll_net_ana_clp.name) $opt \
      $prjx(unfolding.text.filename) $result_filename"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while testing Prefix\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    set result [cat $result_filename]
    switch -glob [string toupper $result] {
      "YES*"   {set prjx(ll_net.ana.clp_deadlock_free) "NO"}
      "NO*"  {set prjx(ll_net.ana.clp_deadlock_free) "YES"}
      default  {set prjx(ll_net.ana.clp_deadlock_free) "UNKNOWN"}
    }  ;# switch $result
      # NewLines sind in der Projektdatei nicht erlaubt, Trennzeichen deshalb #
      # Die erste Zeile mit dem TRUE oder FALSE entfernen!
    set result [string trim [join [lrange [split $result "\n"] 1 end] "#"]]
    set prjx(ll_net.ana.clp_deadlock_free.result_sequence) $result
    convert_string_2_text $toplevel.output.textfr.l $result
    file delete $result_filename
    set display_result [format_ana_result_string \
      $prjx(ll_net.ana.clp_deadlock_free)]
    new_project_msg $prjnum "Prefix tested for Deadlock-free $display_result"
    $toplevel.output.result configure  -text $display_result
    update_project_infos $prjnum
  cd $actdir
}  ;# prj_ana_clp_deadlock_free

proc prj_ana_fc2_deadlock_free {prjnum toplevel} {
  # Falls FC2 vorhanden ist, auf Deadlock-free testen.
  # prjnum - Die Nummer des Projektes (IN)
  # toplevel - Name des Fensters f"ur das Ergebnis (IN, wird ver"andert)
  # TODO: testen ob FC2 zu LL Net gehoert!
  global gvar
  global errorCode
 
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if {![file exists "$prjx(fc2.filename)"]} {
    if [catch prj_exe_ll_net_fc2] {
      error_msg "Failed to create FC2 input $prjx(actdir)/$prjx(fc2.filename)"
      return
    }
  }
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
     [lindex [split $prjx(fc2.filename) .] 0]_dead.fc2
    file delete $result_filename
      # TODO: Optionen verwalten
    set opt "-dead -no_global "
    set sys_str "exec $gvar(fc2_to_fc2_dead.name) \
      $opt $prjx(fc2.filename) "
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      DEBUG "errorCode: $errorCode"
      cd $actdir
      return
    } ;# Fehler beim Konvertieren

    if [file exists "$result_filename"] {
      # Deadlock existiert
      set sys_str "exec $gvar(fc2_ana_deadlock_free.name) \
        $result_filename "
      DEBUG "Aufruf: $sys_str"
      set errorCode "NONE"
      set sys_res [catch {eval $sys_str} result_text]
      DEBUG " ergab: $sys_res $result_text"
      if [string compare $errorCode "NONE"] {
        DEBUG "errorCode: $errorCode"
        cd $actdir
        return
      } ;# if Fehler beim berechnen der Sequenz
      file delete $result_filename
      # Ergebnis zum Speichern im Projekt aufbereiten
      # (keine \n erlaubt!)
      set result [string trim [join [lrange [split $result_text "\n"] 0 end] "#"]]

      # wenn das Ergebnis nicht mit "_SEQUENCE:" beginnt,
      # gilt das Netz als FC2-Deadlock-Free!
      if {[string match "_SEQUENCE*" $result]} {
        set prjx(fc2.ana.deadlock_free) "NO"
      } else {
        set prjx(fc2.ana.deadlock_free) "YES"
      };#
    } else {
      # Kein Deadlock
      # result <- Ausgabe von fc2_to_fc2_dead
      set result [string trim [join [lrange [split $result_text "\n"] 0 end] "#"]]
      set prjx(fc2.ana.deadlock_free) "YES"
    } ;# if Ergebnisdatei existiert (deadlock existiert)
    set prjx(fc2.ana.deadlock_free.events) $result
    convert_string_2_text $toplevel.output.textfr.l $result
    set display_result [format_ana_result_string \
      $prjx(fc2.ana.deadlock_free)]
    new_project_msg $prjnum "Tested for FC2 Deadlock-free $display_result"
    $toplevel.output.result configure  -text $display_result
    update_project_infos $prjnum
  
  cd $actdir
}  ;# prj_ana_fc2_deadlock_free

proc prj_ana_model_check_t_system {} {
  # Falls die LL-Netzdatei und die Formeldatei vorhanden sind
  # und das Netz ein T-System ist mit Hilfe des Model-Checkers testen.
  global gvar
  global errorCode

    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Sourcen da sind.
    # Getrennt fuer eine ausfuehrliche Fehlermeldung
  set ll_net_file_flag [file exists "$prjx(ll_net.filename)"]
  set formula_file_flag [file exists "$prjx(formula.normal.filename)"]
  if {$ll_net_file_flag && $formula_file_flag} {
      # Test auf T-System fuer das LL-Netz
    if {$prjx(ll_net.ana.t_system) == "NO"} {
      error_msg "LL Net is not a T-System\n\
        this Model Checker works only for T-Systems"
      return
    }  ;# if LL-Netz ist kein T-System
    if {![formula_transform $actdir]} {
      return
    } ; # if expansion failed
    if {$prjx(ll_net.ana.t_system) == "UNKNOWN"} {
        # der Test auf T-System muss erst durchgefuehrt werden
      set result_filename \
        [change_filename_extension $prjx(ll_net.filename) ll_net_t_system_res]
        # Testen, ob LL-Netz ein T-System ist
        # TODO: Optionen verwalten, Aufrufzeile verbessern
      set sys_str "exec $gvar(ll_net_ana_t_system.name) $prjx(ll_net.filename) \
        $result_filename"
      DEBUG "Aufruf: $sys_str"
      set errorCode "NONE"
      set sys_res [catch {eval $sys_str} result_text]
      DEBUG " ergab: $sys_res $result_text"
      if [string compare $errorCode "NONE"] {
        file delete $result_filename
        cd $actdir
        error_msg "Error while testing LL Net\n$result_text"
        return
      }  ;# if Fehler beim Konvertieren
      set result [cat $result_filename]
      switch -regexp $result {
        "SYSTEM IST T-SYSTEM"  {set prjx(ll_net.ana.t_system) "YES"}
        "SYSTEM KEIN T-SYSTEM"  {set prjx(ll_net.ana.t_system) "NO"}
        default  {set prjx(ll_net.ana.t_system) "UNKNOWN"}
      }  ;# switch $result
      file delete $result_filename
      new_project_msg $prjnum "LL Net tested for T-System \
        [format_ana_result_string $prjx(ll_net.ana.t_system)]"
      update_project_infos $prjnum
    }  ;# if T-System-Status unbekannt
    if {$prjx(ll_net.ana.t_system) != "YES"} {
      error_msg "LL Net is not a T-System\n\
        this Model Checker works only for T-Systems"
      return
    }  ;# if LL-Netz ist kein T-System
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(ll_net.filename) \
      model_check_t_system_res]
      # Testen, ob die Formel gueltig ist
      # TODO: Optionen verwalten, Aufrufzeile verbessern
    set sys_str "exec $gvar(ll_net_ana_model_check_t_system.name) \
      $prjx(ll_net.filename) $prjx(formula.expanded.filename) \
      $result_filename"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while Model checking LL Net\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    set result [cat $result_filename]
    switch -regexp [string toupper $result] {
      "TRUE"   {set prjx(ll_net.ana.model_check_t_system) "YES"}
      "FALSE"  {set prjx(ll_net.ana.model_check_t_system) "NO"}
      default  {set prjx(ll_net.ana.model_check_t_system) "UNKNOWN"}
    }  ;# switch $result
    file delete $result_filename
    new_project_msg $prjnum "LL Net Model checked (T-System) \
      [format_ana_result_string $prjx(ll_net.ana.model_check_t_system)]"
    update_project_infos $prjnum
  } else {
    if {!$ll_net_file_flag} {
      set error_txt_1 \
        "LL Net $prjx(actdir)/$prjx(ll_net.filename) does not exists"
    } else {
      set error_txt_1 " "
    }  ;# else, kein Fehler hier
    if {!$formula_file_flag} {
      set error_txt_2 \
        "Formula $prjx(actdir)/$prjx(formula.normal.filename) does not exists"
    } else {
      set error_txt_2 " "
    }  ;# else, kein Fehler hier
    error_msg [string trim "$error_txt_1\n$error_txt_2"]
  }  ;# else, Sourcen ist nicht vorhanden
  cd $actdir
}  ;# prj_ana_model_check_t_system

proc prj_ana_model_check_unfsmodels {prjnum toplevel} {
  # Falls die LL-Netzdatei und die Formeldatei vorhanden sind
  # mit Hilfe des Model-Checkers testen.
  global gvar
  global errorCode

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Sourcen da sind.
    # Getrennt fuer eine ausfuehrliche Fehlermeldung
  set ll_net_file_flag [file exists "$prjx(ll_net.filename)"]
  set formula_file_flag [file exists "$prjx(formula.normal.filename)"]
  if {$ll_net_file_flag && $formula_file_flag} {
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(ll_net.filename) \
      model_check_unfsmodels_res]
      # Testen, ob die Formel gueltig ist
      # TODO: Optionen verwalten, Aufrufzeile verbessern
    # First synchronize net and formula with qq
    set sys_str "exec $gvar(ll_net_ana_model_check_qq.name) \
      $prjx(ll_net.filename) $prjx(formula.normal.filename) -S"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while synchronizing LL Net and formula\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    set sync_net_filename \
      [change_filename_extension $prjx(ll_net.filename) sync.ll_net]
    set sys_str "exec $gvar(unfolding_ana_unfsmodels.name) \
      -v -l $sync_net_filename >$result_filename"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while checking synchronized LL Net\n$result_text"
      return
    }  ;# if Fehler beim Checken
    set result [cat $result_filename]
    switch -regexp [string toupper $result] {
      "TRUE"   {set prjx(ll_net.ana.model_check_unfsmodels) "YES"}
      "FALSE"  {set prjx(ll_net.ana.model_check_unfsmodels) "NO"}
      default  {set prjx(ll_net.ana.model_check_unfsmodels) "UNKNOWN"}
    }  ;# switch $result
read_file_2_textwidget $toplevel.output.textfr.l \
$result_filename end true
    file delete $result_filename
    new_project_msg $prjnum "LL Net Model checked (unfsmodels) \
      [format_ana_result_string $prjx(ll_net.ana.model_check_unfsmodels)]"
    update_project_infos $prjnum
  } else {
    if {!$ll_net_file_flag} {
      set error_txt_1 \
        "LL Net $prjx(actdir)/$prjx(ll_net.filename) does not exist"
    } else {
      set error_txt_1 " "
    }  ;# else, kein Fehler hier
    if {!$formula_file_flag} {
      set error_txt_2 \
        "Formula $prjx(actdir)/$prjx(formula.normal.filename) does not exist"
    } else {
      set error_txt_2 " "
    }  ;# else, kein Fehler hier
    error_msg [string trim "$error_txt_1\n$error_txt_2"]
  }  ;# else, Sourcen ist nicht vorhanden
  cd $actdir
}  ;# prj_ana_model_check_unfsmodels

proc prj_ana_model_check_boundsmodels {prjnum toplevel} {
  # Falls die LL-Netzdatei und die Formeldatei vorhanden sind
  # mit Hilfe des Model-Checkers testen.
  global gvar
  global errorCode

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Sourcen da sind.
    # Getrennt fuer eine ausfuehrliche Fehlermeldung
  set ll_net_file_flag [file exists "$prjx(ll_net.filename)"]
  set formula_file_flag [file exists "$prjx(formula.normal.filename)"]
  if {$ll_net_file_flag && $formula_file_flag} {
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(ll_net.filename) \
      model_check_boundsmodels_res]
      # Testen, ob die Formel gueltig ist
      # TODO: Optionen verwalten
    set sys_str "exec $gvar(ll_net_ana_boundsmodels.name) \
      -b=5 -f=$prjx(formula.normal.filename) $prjx(ll_net.filename) $gvar(ll_net_ana_boundsmodels_extras.name) >$result_filename"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while checking LL Net\n$result_text"
      return
    }  ;# if Fehler beim Checken
    set result [cat $result_filename]
    switch -regexp [string toupper $result] {
      "TRUE"   {set prjx(ll_net.ana.model_check_boundsmodels) "YES"}
      "FALSE"  {set prjx(ll_net.ana.model_check_boundsmodels) "NO"}
      default  {set prjx(ll_net.ana.model_check_boundsmodels) "UNKNOWN"}
    }  ;# switch $result
read_file_2_textwidget $toplevel.output.textfr.l \
$result_filename end true
    file delete $result_filename
    new_project_msg $prjnum "LL Net Model checked (boundsmodels) \
      [format_ana_result_string $prjx(ll_net.ana.model_check_boundsmodels)]"
    update_project_infos $prjnum
  } else {
    if {!$ll_net_file_flag} {
      set error_txt_1 \
        "LL Net $prjx(actdir)/$prjx(ll_net.filename) does not exist"
    } else {
      set error_txt_1 " "
    }  ;# else, kein Fehler hier
    if {!$formula_file_flag} {
      set error_txt_2 \
        "Formula $prjx(actdir)/$prjx(formula.normal.filename) does not exist"
    } else {
      set error_txt_2 " "
    }  ;# else, kein Fehler hier
    error_msg [string trim "$error_txt_1\n$error_txt_2"]
  }  ;# else, Sourcen ist nicht vorhanden
  cd $actdir
}  ;# prj_ana_model_check_boundsmodels

proc prj_ana_model_check_qq {prjnum toplevel} {
  # Falls die LL-Netzdatei und die Formeldatei vorhanden sind
  # mit Hilfe des Model-Checkers testen.
  global gvar
  global errorCode

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Sourcen da sind.
    # Getrennt fuer eine ausfuehrliche Fehlermeldung
  set ll_net_file_flag [file exists "$prjx(ll_net.filename)"]
  set formula_file_flag [file exists "$prjx(formula.normal.filename)"]
  if {$ll_net_file_flag && $formula_file_flag} {
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(ll_net.filename) \
      model_check_qq_res]
      # Testen, ob die Formel gueltig ist
      # TODO: Optionen verwalten, Aufrufzeile verbessern
    set sys_str "exec $gvar(ll_net_ana_model_check_qq.name) \
      $prjx(ll_net.filename) $prjx(formula.normal.filename) >$result_filename"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while Model checking LL Net\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    set result [cat $result_filename]
    switch -regexp [string toupper $result] {
      "YES"   {set prjx(ll_net.ana.model_check_qq) "YES"}
      "NO"  {set prjx(ll_net.ana.model_check_qq) "NO"}
      default  {set prjx(ll_net.ana.model_check_qq) "UNKNOWN"}
    }

    read_file_2_textwidget $toplevel.output.textfr.l $result_filename end true

    file delete $result_filename
    set display_result [format_ana_result_string \
      $prjx(ll_net.ana.model_check_qq)]
    $toplevel.output.result configure -text $display_result
    new_project_msg $prjnum "LL Net Model checked (qq) $display_result"
    update_project_infos $prjnum
  } else {
    if {!$ll_net_file_flag} {
      set error_txt_1 \
        "LL Net $prjx(actdir)/$prjx(ll_net.filename) does not exist"
    } else {
      set error_txt_1 " "
    }  ;# else, kein Fehler hier
    if {!$formula_file_flag} {
      set error_txt_2 \
        "Formula $prjx(actdir)/$prjx(formula.normal.filename) does not exist"
    } else {
      set error_txt_2 " "
    }  ;# else, kein Fehler hier
    error_msg [string trim "$error_txt_1\n$error_txt_2"]
  }  ;# else, Sourcen ist nicht vorhanden
  cd $actdir
}  ;# prj_ana_model_check_qq

proc prj_ana_model_check_safe {prjnum toplevel} {
  # Falls die Entfaltung und die Formeldatei vorhanden sind,
  # mit Hilfe des Model-Checkers testen.
  # prjnum - Die Nummer des Projektes (IN)
  # toplevel - Name des Fensters fuer das Ergebnis (IN, wird veraendert)
  #TODO testen ob Entfaltung zu LL Net gehoert!
  global gvar
  global errorCode

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Sourcen da sind
    # Getrennt fuer eine ausfuehrliche Fehlermeldung
  set unfolding_file_flag [file exists "$prjx(unfolding.text.filename)"]
  set formula_file_flag [file exists "$prjx(formula.normal.filename)"]
  if {$unfolding_file_flag && $formula_file_flag} {
    if {![formula_transform $actdir]} {
      return
    }  ;# if expansion failed
      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(unfolding.text.filename) \
      model_check_safe_res]
      # Testen, ob die Formel gueltig ist
    set opt [get_options_model_check_safe display_firing_sequences_flag]
    if {$display_firing_sequences_flag} {
      set firing_sequences_filename \
      [change_filename_extension $prjx(unfolding.text.filename) \
      model_check_safe_firing_sequences]
      set opt "$opt -p=$firing_sequences_filename"
    }  ;# if Schaltfolgen angeben
    set sys_str "exec $gvar(unfolding_text_ana_model_check_safe.name) \
      $prjx(unfolding.text.filename) -f=$prjx(formula.expanded.filename) \
      -s=$result_filename $opt"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while Model checking LL Net (Prefix)\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    set result [cat $result_filename]
    switch -regexp [string toupper $result] {
      "TRUE"   {set prjx(unfolding.text.ana.model_check_safe) "YES"}
      "FALSE"  {set prjx(unfolding.text.ana.model_check_safe) "NO"}
      default  {set prjx(unfolding.text.ana.model_check_safe) "UNKNOWN"}
    }
    file delete $result_filename
      # Ausgabe der Schaltfolgen, falls gew"unscht.
    if {$display_firing_sequences_flag} {
      set firing_sequences [cat $firing_sequences_filename]
        # NewLines sind in der Projektdatei nicht erlaubt,
        # Trennzeichen ist deshalb #
      set firing_sequences [string trim [join [lrange \
        [split $firing_sequences "\n"] 1 end] "#"]]
      set prjx(ll_net.ana.model_check_safe.firing_sequences) $firing_sequences
      set gvar(option.ana_model_check_safe.firing_sequences) $firing_sequences

      read_file_2_textwidget $toplevel.output.textfr.l \
      $firing_sequences_filename end true

      file delete $firing_sequences_filename
    }  ;# if Anzeige der Schaltfolgen
    if {$gvar(option.dlg.model_check_safe.log_results)} {
      #Ergebnisse an .doc datei anhaengen
      #TODO:  doc editor ggf. schliessen!
      if {[file exists $prjx(doc.filename)]} {
        set datei [open $prjx(doc.filename) a+]
      } else {
        set datei [open $prjx(doc.filename) w+]
      }
      DEBUG "Adding results to $prjx(doc.filename)"
      puts $datei "\nModel Checker (Safe) result:"
      puts $datei [$toplevel.output.textfr.l get 0.0 end]
      close $datei
    }

    set display_result [format_ana_result_string \
      $prjx(unfolding.text.ana.model_check_safe)]
    $toplevel.output.result configure  -text $display_result
    new_project_msg $prjnum "LL Net (Prefix) Model checked (Safe) \
      $display_result"
    update_project_infos $prjnum
  } else {
    if {!$unfolding_file_flag} {
      set error_txt_1 \
        "Prefix $prjx(actdir)/$prjx(ll_net.filename) does not exists"
    } else {
      set error_txt_1 " "
    }  ;# else, kein Fehler hier
    if {!$formula_file_flag} {
      set error_txt_2 \
        "Formula $prjx(actdir)/$prjx(formula.normal.filename) does not exists"
    } else {
      set error_txt_2 " "
    }  ;# else, kein Fehler hier
    error_msg [string trim "$error_txt_1\n$error_txt_2"]
  }  ;# else, Sourcen sind nicht vorhanden
  cd $actdir
}  ;# prj_ana_model_check_safe

proc create_promela_f {prjnum display_option} {
   # erzeugt aus *.promela und *.form die zu
   # analysierende Datei *.promela_f
   # falls display_option=1 soll das Optionsfenster geoeffnet werden
  global gvar
  global errorCode

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"

    # Testen, ob die Sourcen da sind
  set promela_file_flag [file exists "$prjx(promela.filename)"]
  set formula_file_flag [file exists "$prjx(formula.normal.filename)"]
  if {$promela_file_flag && $formula_file_flag} {
     # Expand formula
    set prjx(formula.expanded.filename) \
    [change_filename_extension $prjx(formula.normal.filename) form_exp]

    #Transform formula macros, only for 1-safe!
    if {[pattern_in_file $prjx(promela.filename) "NO 1-safe"]} {
      DEBUG "simple copy, because not 1-safe"
      file copy -force $prjx(formula.normal.filename) $prjx(formula.expanded.filename)
    } else {
      DEBUG "TransForm, because 1-safe"
      if {[file exists $prjx(sdl_ll_ref.filename)] \
         && [file exists $prjx(sdl_info.filename)]} {
        set sdl_opt "-s=$prjx(sdl_ll_ref.filename) -x=$prjx(sdl_info.filename)"
      } else {
        set sdl_opt " "
      }
      set sys_str "exec $gvar(formula_2_formula_exp.name) \
        $prjx(ll_net.filename) -i=$prjx(formula.normal.filename) \
        -o=$prjx(formula.expanded.filename) -l $sdl_opt"
      DEBUG "Aufruf: $sys_str"
      set errorCode "NONE"
      set sys_res [catch {eval $sys_str} result_text]
      DEBUG " ergab: $sys_res $result_text"
      if [string compare $errorCode "NONE"] {
        cd $actdir
        error_msg "Error while expanding Formula: $result_text"
        return 1
      }  ;# if Expandieren ging in die Hose
    } ;# if TransForm is called.

    # Umwandlung der in der Formel evtl. vorkommenden 
    # grossgeschriebenen Stellennamen in kleingeschriebene
    # und merken der alten Stellennamen
    # Ergebnis in *.form_ch
    if {![pattern_in_file $prjx(promela.filename) "NO 1-safe"]} {
      DEBUG "simple case, because 1-safe"
      set sys_str "exec $gvar(upper_to_lower.name) $prjx(formula.expanded.filename) >$prjx(formula.change.filename)"
      DEBUG "Aufruf: $sys_str"
      exec $gvar(upper_to_lower.name) $prjx(formula.expanded.filename) >$prjx(formula.change.filename)
    } else {
      DEBUG "difficult case, because not 1-safe"
      set sys_str "exec $gvar(formel_to_LTL.name) <$prjx(formula.expanded.filename) >$prjx(formula.change.filename)"
      DEBUG "Aufruf: $sys_str"
      exec $gvar(formel_to_LTL.name) <$prjx(formula.expanded.filename) >$prjx(formula.change.filename)
      set errorCode "NONE"
      set sys_res [catch {eval $sys_str} result_text]
      DEBUG " ergab: $sys_res $result_text"
      if [string compare $errorCode "NONE"] {
        cd $actdir
        error_msg "Error while executing [file tail $gvar(formel_to_LTL.name)].\n$result_text\n\n"
        return 0
      } 
      if {[pattern_in_file $prjx(formula.change.filename) "not classifiable"]} {
        error_msg "Error while executing [file tail $gvar(formel_to_LTL.name)].\nSyntax of formula is wrong.\n"
        return 0
      }
    }

    # uebersetze die Formel in einen never claim
    # Ergebnis in *.form_promela
    set form_file $prjx(formula.change.filename)
    if [file exist $form_file] {
      set f [open $form_file]
      gets $f formel
    }
    if {$gvar(option.dlg.model_check_spin.tl_stat)==0} {
       set fo [format "\"!(%s)\"" $formel]
    } else {
       set fo [format "\"%s\"" $formel]
    }
    set form_promela_filename [change_filename_extension $prjx(ll_net.filename) form_promela]
    set sys_str "exec $gvar(promela_text_ana_model_check_spin.name) \
      -f $fo >$form_promela_filename" 
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      cd $actdir
      error_msg "Error during execution of SPIN\n$result_text\n\nMaybe $formel is not an LTL formula."
      return 0
    }  ;# if Fehler bei Ausfuehrung von spin -f
      # Anhaengen der notwendigen define Anweisungen und
      # des never claim an die zu analysierende Promela-Datei (*.promela)
      # Ergebnis in *.promela_f
    if {[file exist $form_promela_filename]} {
       file copy -force $prjx(promela.filename) $prjx(promela_form.filename)
       set p [open $prjx(promela_form.filename) a]
       set f_p [open $form_promela_filename {RDONLY}]
       puts $p "\n\n"
       # define Anweisungen anh"angen
       if {![pattern_in_file $prjx(promela.filename) "NO 1-safe"]} {
         while {[gets $f old_place_name]>=0} {
          set new_place_name [string tolower $old_place_name]
          if {$new_place_name!=""} { 
               puts $p [format "#define %s %s" $new_place_name $old_place_name]
          }
         }
       } else {
         set def [open merke_define {RDONLY}]
         set line [read $def]
         puts $p $line
         close $def
         file delete merke_define
       }
       puts $p ""
       # never claim anh"angen
       set line [read $f_p]
       puts $p $line
       close $p
       close $f_p
       file delete $form_promela_filename 
     }
    close $f

    if {$display_option} {option_ana_model_check_spin_dlg}
    return 1

  } else {
    if {!$promela_file_flag} {
       set error_txt_1 "File $prjx(actdir)/$prjx(promela.filename) does not exists"
    } else {
       set error_txt_1 " "
    }  
    if {!$formula_file_flag} {
       set error_txt_2 "File $prjx(actdir)/$prjx(formula.expanded.filename) does not exists"
    } else {
       set error_txt_2 " "
    }  
    error_msg [string trim "$error_txt_1\n$error_txt_2"]
  } ;# else, Sourcen ist nicht vorhanden
  cd $actdir
return 0

}  ;# proc create_promela_f


proc prj_ana_model_check_spin {prjnum toplevel} {
  # Falls die Promela-Datei, die die zu untersuchende Formel enthaelt, 
  # vorhanden ist,
  # mit Hilfe des Model-Checkers testen.
  # prjnum - Die Nummer des Projektes (IN)
  # toplevel - Name des Fensters fuer das Ergebnis (IN, wird veraendert)
  global gvar
  global errorCode

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"

    # Testen, ob die Sourcen da sind
  set promela_file_flag [file exists "$prjx(promela.filename)"]
  set formula_file_flag [file exists "$prjx(formula.normal.filename)"]
  if {$promela_file_flag && $formula_file_flag} {

      # zu analysierende Datei *.promela_f erzeugen
      # falls dies nicht moeglich war, Prozedur beenden
    if {![create_promela_f $prjnum 0]} {return}

      # Ergebnisdateinamen zusammensetzen
    set result_filename \
      [change_filename_extension $prjx(promela_form.filename) \
      model_check_spin_res]

      # spin-Aufruf: 
      # erzeugt C-Sourcen (pan.*) des Verifizierers 
    init_options_var_model_check_spin $prjnum
    set a_opt [get_a_options_model_check_spin $prjnum]
    set sys_str "exec $gvar(promela_text_ana_model_check_spin.name) \
	$a_opt $prjx(promela_form.filename) >$result_filename" 
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while Execution of SPIN\n$result_text\n\nMay be that there is a mistake \nin the PROMELA model $prjx(promela_form.filename)."
      return
    }  ;# if Fehler bei Ausfuehrung von spin

      # Erzeugen des Verifizierers
    set c_opt [get_c_options_model_check_spin $prjnum]
    if $gvar(installed.gcc) {
      set Compile "gcc -o $prjx(project.name) $c_opt pan.c"
    } elseif $gvar(installed.cc) {
      DEBUG "No gcc installed, please install! Using cc now."
      set Compile "cc -o $prjx(project.name) $c_opt pan.c"
    } else {
      DEBUG "No gcc or cc found! Sorry, but I can't compile."
      cd $actdir
      return
    }
    set sys_str "exec $Compile >$result_filename"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while Compilation of pan.c\n$result_text"
      return
    }  ;# if Fehler beim Kompilieren

      # Aufruf des Verifizierers
    set v_opt [get_v_options_model_check_spin $prjnum]
    set sys_str "exec time ./$prjx(project.name) $v_opt >$result_filename"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      file delete $result_filename
      cd $actdir
      error_msg "Error while Execution of Verifier\n$result_text"
      return
    }  ;# if Fehler bei der Ausfuehrung des Verifizierers

      # Ausgabe des SPIN-Resultats
    if [file exists "$result_filename"] {
	read_file_2_textwidget $toplevel.output.textfr.l \
	$result_filename end true
    }
    set file_string [cat $result_filename]
    if {[regexp "errors: 0" $file_string]} {
       if {([regexp "pan: out of memory" $file_string]||[regexp "Bit statespace search" $file_string]||[regexp "error: max search depth too small" $file_string])} {
         set prjx(promela.text.ana.model_check_spin) "UNKNOWN"
       } else {
         set prjx(promela.text.ana.model_check_spin) "YES"
       }
    } else {
       set prjx(promela.text.ana.model_check_spin) "NO"
    }
    if {![regexp "errors:" $file_string]} {
       set prjx(promela.text.ana.model_check_spin) "UNKNOWN"
    }

    # gegebenenfalls Anzeige des Error Trail
    if {[regexp "promela_f.trail" $file_string]} {
        # Aufruf des Simulators (using error trail)
        # Ergebnis in *.out
	set sim_outfile [change_filename_extension "$prjx(promela.filename)" model_check_spin_sim_out]
        set sys_str "exec $gvar(promela_text_ana_model_check_spin.name) -p -t $prjx(promela_form.filename) >$sim_outfile"
        DEBUG "Aufruf: $sys_str"
        set errorCode "NONE"
        set sys_res [catch {eval $sys_str} result_text]
        DEBUG " ergab: $sys_res $result_text"
        if [string compare $errorCode "NONE"] {
          file delete $result_filename
          cd $actdir
          error_msg "Error while Execution of SPIN Simulation\n$result_text"
          return
        }  ;# if Fehler bei der Ausfuehrung der SPIN Simulation
        # Ermitteln der Schaltfolge
        set schaltfolge {}
        $toplevel.output.textfr.l configure -state normal
        set f_out [open $sim_outfile {RDONLY}]
        set loopexist 0
        while {[gets $f_out line]>=0} {
           if {[regexp "START OF CYCLE" $line]} {
               set loopexist 1
               set folge [join $schaltfolge ,]
               $toplevel.output.textfr.l insert end "_SEQUENCE:\n$folge"
               set schaltfolge {}
           } ;# if 
	   # after line keyword, transition is found
           if {[regexp "line" $line]} {
               set part [string range $line [string first "line" $line] end]
               scan $part "line %d" nr
	       # search transition file for nr
               set f_transition [open $prjx(transition.filename) {RDONLY}]
               while {[gets $f_transition l]>=0} {
                  if {[string first $nr $l]==0} {
                     set liste [split $l " "]
                     set schaltfolge [concat $schaltfolge [lindex $liste 1]]
                     set anz [lindex $liste 2]
                     while {$anz>1} {
                        gets $f_out line
                        incr anz -1
                     } ;# while
                  } ;# if
               } ;# while
               close $f_transition
            } ;# if
         } ;# while
	 close $f_out
         set folge [join $schaltfolge ","]
         if {$loopexist==0} {
            $toplevel.output.textfr.l insert end "_SEQUENCE:\n$folge"
         } else {
            if {$folge!=""} {
              $toplevel.output.textfr.l insert end "\n_LOOP:\n$folge"
	    }
         }
      $toplevel.output.textfr.l configure -state disabled
      if {$gvar(option.dlg.model_check_spin.log_results)} {
        #Ergebnisse an .doc datei anhaengen
        #TODO:  doc editor ggf. schliessen!
        if {[file exists $prjx(doc.filename)]} {
          set datei [open $prjx(doc.filename) a+]
        } else {
          set datei [open $prjx(doc.filename) w+]
        }
        DEBUG "Adding results to $prjx(doc.filename)"
        puts $datei "\nModel Checker (SPIN) result:"
        puts $datei [$toplevel.output.textfr.l get 0.0 end]
        close $datei
      }
    file delete $sim_outfile 
    } ;# if neuer error trail erzeugt

    file delete $result_filename
    set display_result [format_ana_result_string \
      $prjx(promela.text.ana.model_check_spin)]
    $toplevel.output.result configure -text $display_result

    new_project_msg $prjnum "Promela Net Model checked (SPIN) $display_result"
    update_project_infos $prjnum
  } else {
    if {!$promela_file_flag} {
       set error_txt_1 "File $prjx(actdir)/$prjx(promela.filename) does not exists"
    } else {
       set error_txt_1 " "
    }  
    if {!$formula_file_flag} {
       set error_txt_2 "File $prjx(actdir)/$prjx(formula.normal.filename) does not exists"
    } else {
       set error_txt_2 " "
    }  
    error_msg [string trim "$error_txt_1\n$error_txt_2"]
  } ;# else, Sourcen ist nicht vorhanden
  cd $actdir
}  ;# prj_ana_model_check_spin


# Allgemeines Fenster fuer die Model-Checker: Safe, SPIN, SMV
##############################################################
proc prj_ana_model_check_dlg { checker } {
# Den Dialog fuer die Ausgabe des Model Checkings aufbauen.
#
# checker = safe, spin, smv

  global gvar

    # Die Projektnummer abholen. WICHTIG, dass jenes jetzt passiert!
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx

  if [catch {set tl [toplevel .ana_model_check_dlg_$checker]}] {
    raise .ana_model_check_dlg_$checker
    return
  }  ;# if doppelt

 switch $checker {
 	spin { set menu_entry "LTL Model Checker (SPIN)*"
	       set result $prjx(promela.text.ana.model_check_spin)
	       set checker_command "prj_ana_model_check_spin"
	       set option_command "create_promela_f $prjnum 1"
	       set window_title "(SPIN)"}
	safe { set menu_entry "CTL Model Checker (Safe)*"
	       set result $prjx(unfolding.text.ana.model_check_safe)
	       set checker_command "prj_ana_model_check_safe"
	       set option_command "option_ana_model_check_safe_dlg"
	       set window_title "(Safe)"}
	smv {  set menu_entry "CTL Model Checker (SMV)*"
	       set result $prjx(unfolding.text.ana.model_check_smv)
	       set checker_command "prj_ana_model_check_smv"
	       set option_command "option_ana_model_check_smv_dlg"
	       set window_title "(SMV)"}
	unfsmodels { set menu_entry "LTL Model Checker (unfsmodels)*"
	       set result $prjx(unfolding.text.ana.model_check_unfsmodels)
	       set checker_command "prj_ana_model_check_unfsmodels"
	       set option_command ""
	       set window_title "(unfsmodels)"}
	boundsmodels { set menu_entry "LTL Model Checker (boundsmodels)*"
	       set result $prjx(ll_net.ana.model_check_boundsmodels)
	       set checker_command "prj_ana_model_check_boundsmodels"
	       set option_command option_ana_boundsmodels_dlg
	       set window_title "(boundsmodels)"}
	qq {  set menu_entry "LTL Model Checker (qq)*"
	       set result $prjx(unfolding.text.ana.model_check_qq)
	       set checker_command "prj_ana_model_check_qq"
	       set option_command ""
	       set window_title "(qq)"}
 }

  # Menupunkt abschalten, spaeter wieder einschalten
  $prjx(project.ana.menu.widget) entryconfigure $menu_entry -state disabled

  # Dialog aufbauen
  add_dlg_widget $prjnum $tl
  wm title $tl "Model Checking Analysis $window_title"
  wm protocol $tl WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $tl \
    {$prjx(project.ana.menu.widget) entryconfigure {$menu_entry} \
    -state normal}"

  wm minsize $tl 500 350

  # Ausgabebereich
  frame $tl.output  -relief sunken  -bd 2
  pack $tl.output  -side top  -fill both -expand yes
  label $tl.output.result_label  -text "Result of Modelchecking:"
  label $tl.output.result  -text [format_ana_result_string $result]

  pack $tl.output.result_label $tl.output.result -side top  -anchor w
  frame $tl.output.textfr
  pack $tl.output.textfr  -fill both  -expand 1
  text $tl.output.textfr.l \
    -xscrollcommand "$tl.output.textfr.pad.sx set" \
    -yscrollcommand "$tl.output.textfr.sy set" \
    -width 60 -height 15 -state disabled
  scrollbar $tl.output.textfr.sy  -orient vertical \
    -command "$tl.output.textfr.l yview"
  frame $tl.output.textfr.pad
  scrollbar $tl.output.textfr.pad.sx  -orient horizontal \
    -command "$tl.output.textfr.l xview"
  set pad [expr [$tl.output.textfr.sy cget -width] + 2 * \
    ([$tl.output.textfr.sy cget -bd] + \
     [$tl.output.textfr.sy cget -highlightthickness])]
  frame $tl.output.textfr.pad.it  -width $pad  -height $pad 
  pack $tl.output.textfr.pad  -side bottom  -fill x 
  pack $tl.output.textfr.pad.it  -side right 
  pack $tl.output.textfr.pad.sx  -side bottom  -fill x
  pack $tl.output.textfr.sy  -side right  -fill y 
  pack $tl.output.textfr.l  -side left  -fill both  -expand 1

  sim_area $tl

  button_frame $tl $prjnum $checker_command "$option_command" \
	  $prjx(project.ana.menu.widget) "$menu_entry"
}

proc button_frame { tl prjnum cmd1 cmd2 menuwidget menuentry } {
    # Button-Bereich
  if {[catch {frame $tl.commands -relief raised -bd 2}]} {
    $tl.commands.ok configure -command "disable_dlg_buttons $tl; \
      $cmd1 $prjnum $tl; enable_dlg_buttons $tl;"
    if {$cmd2 != ""} {
      $tl.commands.options configure -state normal -command "$cmd2"
    } else {
      $tl.commands.options configure -state disabled
    }
    return
  }
  pack $tl.commands  -side bottom  -fill x
  button $tl.commands.ok  -text "Check" \
    -command "disable_dlg_buttons $tl; \
      $cmd1 $prjnum $tl; enable_dlg_buttons $tl;"
  button $tl.commands.options  -text "Options" \
    -command "$cmd2"
  button $tl.commands.cancel  -text "Close" \
    -command "del_dlg_widget $prjnum $tl; \
      destroy $tl; \
      $menuwidget entryconfigure {$menuentry} \
        -state normal"

  # display options only if available
  if {$cmd2==""} {
       pack $tl.commands.ok \
	 $tl.commands.cancel \
	 -side left -expand yes
  } else {
       pack $tl.commands.ok \
	 $tl.commands.options \
	 $tl.commands.cancel \
	 -side left -expand yes
  }
  bind $tl <Return> "$tl.commands.ok invoke"
  focus $tl.commands.ok
}

proc disable_dlg_buttons { tl } {
   global gvar

   set gvar(cursor) [$tl cget -cursor]
   $tl configure -cursor watch
   $tl.commands.ok configure  -state disabled
   $tl.commands.options configure  -state disabled
   $tl.commands.cancel configure  -state disabled
   catch {
   $tl.simulate.sim_bpn2 configure  -state disabled
   $tl.simulate.sim_exp_bpn2 configure  -state disabled
   $tl.simulate.sim_sdl configure -state disabled
   $tl.simulate.sim_hl configure  -state disabled
   $tl.simulate.sim_ll configure  -state disabled
   }
   update
}

proc enable_dlg_buttons { tl } {
   global gvar
   upvar #0 prj$gvar(actprj) prjx

   $tl configure -cursor $gvar(cursor)
   $tl.commands.ok configure  -state normal
   $tl.commands.options configure  -state normal
   $tl.commands.cancel configure  -state normal
   if {[catch { set sequence [expr [$tl.output.textfr.l search SEQUENCE 1.0]!=""] }]} {
     set sequence false
   }
   if {$sequence} {
   catch {
   $tl.simulate.sim_ll configure  -state normal
   if {[exists_ll_to_bpn2]} {
      $tl.simulate.sim_bpn2 configure -state normal
   }
   if {[exists_ll_to_exp_bpn2]} {
      $tl.simulate.sim_exp_bpn2 configure  -state normal
   }
   if {[file exists $prjx(sdl_ll_ref.filename)]} {
     $tl.simulate.sim_sdl configure -state normal
   }
   if {[file exists $prjx(hl_ll_ref.filename)]} {
     $tl.simulate.sim_hl configure -state normal
   }
   }}
   update
}

proc prj_ana_model_check_smv {prjnum toplevel} {
  # 
  # 
  # prjnum - Die Nummer des Projektes (IN)
  # toplevel - Name des Fensters fuer das Ergebnis (IN, wird veraendert)
  global gvar
  global errorCode

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"

    # Testen, ob die Sourcen da sind
    # Getrennt fuer eine ausfuehrliche Fehlermeldung
  set ll_file_flag [file exists "$prjx(ll_net.filename)"]
  set formula_file_flag [file exists "$prjx(formula.normal.filename)"]

  if {$ll_file_flag && $formula_file_flag} {

   if {$gvar(option.dlg.model_check_smv.invariants_check)} {
     if {$gvar(option.dlg.model_check_smv.inv) == 1} {
         set pre_sys_str "exec gencomp $prjx(ll_net.filename) -o=$prjx(slist.filename)"
         set sys_str "exec $gvar(pep_to_smv.name) \
           $prjx(ll_net.filename) -f=$prjx(formula.normal.filename) \
           -o=$prjx(smv.filename) -b=$prjx(slist.filename) -m=$gvar(option.dlg.model_check_smv.switch)"
     } elseif {$gvar(option.dlg.model_check_smv.inv) == 2} {
         set pre_sys_str "exec gencomp -r $prjx(ll_net.filename) -o=$prjx(slist.filename)"
         set sys_str "exec $gvar(pep_to_smv.name) \
           $prjx(ll_net.filename) -f=$prjx(formula.normal.filename) \
           -o=$prjx(smv.filename) -b=$prjx(slist.filename) -m=$gvar(option.dlg.model_check_smv.switch)"
     } elseif {$gvar(option.dlg.model_check_smv.inv) == 3} {
         # PTAK
         # Welche Referenzen sollen verwendet werden?
        if [file exists $prjx(bpn2_ll_ref.filename)] {
	  set ref_mode "-bpn2"
	  set ref_file_name "-r $prjx(bpn2_ll_ref.filename)"
	} elseif [file exists $prjx(exp_bpn2_ll_ref.filename)] {
	  set ref_mode "-bpn2"
	  set ref_file_name "-r $prjx(exp_bpn2_ll_ref.filename)"
	} elseif [file exists $prjx(sdl_ll_ref.filename)] {
	  set ref_mode "-sdl"
	  set ref_file_name "-r $prjx(sdl_ll_ref.filename)"
	} else {
	  DEBUG "Could not create new slist. No References found!"
	  cd $actdir
	  return
	}
        set pre_sys_str "exec $gvar(ll_net_to_slist.name) $ref_mode\
                              $prjx(ll_net.filename) $ref_file_name\
                              -o $prjx(slist.filename)"
         set sys_str "exec $gvar(pep_to_smv.name) $prjx(ll_net.filename)\
		-f=$prjx(formula.normal.filename) -o=$prjx(smv.filename) -p\
	 -b=$prjx(slist.filename) -m=$gvar(option.dlg.model_check_smv.switch)"
     }
    DEBUG "Aufruf: $pre_sys_str"
    set errorCode "NONE"
    set pre_sys_res [catch {eval $pre_sys_str} result_text]
   } else {
    set sys_str "exec $gvar(pep_to_smv.name) \
      $prjx(ll_net.filename) -f=$prjx(formula.normal.filename) \
      -o=$prjx(smv.filename) -m=$gvar(option.dlg.model_check_smv.switch)"
  }

    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      cd $actdir
      error_msg "Error ..."
      return
    }  ;# if 

    set sys_str "exec $gvar(ana_model_check_smv.name) \
      $gvar(option.dlg.model_check_smv.additional) $prjx(smv.filename) | \
      $gvar(ana_model_check_trans.name) -n=$prjx(ll_net.filename)"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"

    if [string compare $errorCode "NONE"] {
      file delete $prjx(smv.filename)
    }
    # Erste Zeile   
    set ret_line [lindex [split $result_text "\n"] 0]
    # Letztes Wort
    set result [lindex $ret_line end]

    switch -regexp [string toupper $result] {
      "TRUE"   {set prjx(unfolding.text.ana.model_check_smv) "YES"}
      "FALSE"  {set prjx(unfolding.text.ana.model_check_smv) "NO"}
      default  {set prjx(unfolding.text.ana.model_check_smv) "UNKNOWN"}
    }  ;# switch $result

	# Ergebnisse werden in TextWidget eingetragen:

	$toplevel.output.textfr.l configure -state normal
	$toplevel.output.textfr.l delete 0.0 end
	$toplevel.output.textfr.l insert end $result_text
	$toplevel.output.textfr.l configure -state disabled
    if {$gvar(option.dlg.model_check_smv.log_results)} {
      #Ergebnisse an .doc datei anhaengen
      #TODO:  doc editor ggf. schliessen!
      if {[file exists $prjx(doc.filename)]} {
        set datei [open $prjx(doc.filename) a+]
      } else {
        set datei [open $prjx(doc.filename) w+]
      }
      DEBUG "Adding results to $prjx(doc.filename)"
      puts $datei "\nModel Checker (SMV) result:"
      puts $datei [$toplevel.output.textfr.l get 0.0 end]
      close $datei
    }

    set display_result [format_ana_result_string \
      $prjx(unfolding.text.ana.model_check_smv)]
    $toplevel.output.result configure  -text $display_result
    new_project_msg $prjnum "LL Net (Prefix) Model checked (Safe) \
      $display_result"
    update_project_infos $prjnum
    update_project_files $prjnum

  } else {
    if {!$ll_file_flag} {
      set error_txt_1 \
        "Prefix $prjx(actdir)/$prjx(ll_net.filename) does not exists"
    } else {
      set error_txt_1 " "
    }  ;# else, kein Fehler hier
    if {!$formula_file_flag} {
      set error_txt_2 \
        "Formula $prjx(actdir)/$prjx(formula.normal.filename) does not exists"
    } else {
      set error_txt_2 " "
    }  ;# else, kein Fehler hier
    error_msg [string trim "$error_txt_1\n$error_txt_2"]
  }  ;# else, Sourcen ist nicht vorhanden
  cd $actdir
}  ;# prj_ana_model_check_smv
