  ###############################
  # Callbacks fuer Execute-Menu #
  ###############################

proc prj_exe_bpn2_boxexpr {} {
  # Die evtl. vorhandene B(PN)-Datei checken und 
  # dann ggf. in eine BoxExpr umwandeln.
  global gvar
  global errorCode

    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(bpn2.normal.filename)"] {
    if [get_options_syntax_check] {
        # kein Syntax-Check vornehmen
      DEBUG "NO Syntax-Check"
      set prjx(bpn2.normal.syntax_ok) "Skipped"
      set prjx(bpn2.normal.procs) "Unknown"
      update_project_files $prjnum
    } else {
        # temporaeren Dateinamen fuer den Syntax-Checker basteln
      set result_file \
        [change_filename_extension $prjx(bpn2.normal.filename) bpn2_res]
        # Syntax-Checker aufufen
      set sys_str "exec $gvar(bpn2_syntaxchecker.name) \
        $prjx(bpn2.normal.filename) $result_file -e"
      DEBUG "Aufruf: $sys_str"
      set errorCode "NONE"
      set sys_res [catch {eval $sys_str} result_text]
      DEBUG " ergab: $sys_res $result_text *$errorCode*"
      new_project_msg $prjnum "B(PN) Syntax checked"
      if [string compare $errorCode "NONE"] {
        set prjx(bpn2.normal.syntax_ok) "Failed"
        update_project_files $prjnum
        file delete $result_file
        cd $actdir
        error_msg "Error while parsing B(PN) Programm (Syntax Checker)"
        return 0
      }  ;# if kein Fehler aufgetreten, auch wenn errorCode > 0
        # Ergebnis handeln
      
      set sys_res [pattern_in_file $result_file {parse error}]
      file delete $result_file
      if {$sys_res == 1} {
        set prjx(bpn2.normal.syntax_ok) "Errors"
        update_project_files $prjnum
        cd $actdir
        error_msg "B(PN) Program has Error(s)"
        return 0
      }  ;# if Fehler im Programm
      set prjx(bpn2.normal.syntax_ok) "Ok"
    }  ;# else, Syntax-Check vornehmen

      # Konverter von B(PN) nach BoxExpr aufrufen, kein neuer Dateiname
    set opt [get_options_bpn2_box_expr]
    set sys_str "exec $gvar(bpn2_to_boxexpr.name) $prjx(bpn2.normal.filename) \
      $prjx(boxexpr.filename) $opt"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text *$errorCode*"
    if [string compare $errorCode "NONE"] {
      set prjx(boxexpr.ref) "None"
      update_project_files $prjnum
      cd $actdir
      error_msg "Error while making PBC\n$result_text"
      return 0
    }  ;# if Fehler beim Konvertieren
    set prjx(boxexpr.ref) "B(PN)"
    update_project_files $prjnum
    new_project_msg $prjnum "B(PN) -> PBC finished"
  } else {
    error_msg \
    "B(PN) Program $prjx(actdir)/$prjx(bpn2.normal.filename) does not exists"
    return 0
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
  return 1
}  ;# prj_exe_bpn2_boxexpr

proc prj_exe_boxexpr_ll_net {} {
  # Die evtl. vorhandene BoxExpr-Datei in ein LL-Netz umwandeln.
  global gvar
  global errorCode

    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln

    # Referenzdateien loeschen
    remove_ref_files ll_net $prjx(project.name)
    remove_ref_files pbc_ll_net $prjx(project.name)

  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(boxexpr.filename)"] {
      # Konverter von BoxExpr nach LL-Netz aufrufen, kein neuer Netzdateiname
    set opt [get_options_box_expr_ll_net]
    set sys_str "exec $gvar(boxexpr_to_ll_net.name) $prjx(boxexpr.filename) \
      -n=$prjx(ll_net.filename) $opt"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      if [string compare $prjx(ll_net.ref) "HL-Net"] {
      } else {
        set prjx(ll_net.ref) "None"
        update_project_files $prjnum
      }  ;# else, Verweis war da
      cd $actdir
      error_msg "Error while making LL Net\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    set prjx(ll_net.ref) "PBC"
    set prjx(unfolding.text.ref) "None"
    set prjx(spin.ref) "None"
    set prjx(ina.ref) "None"
    set prjx(smv.ref) "None"
    set prjx(fc2.ref) "None"
    reset_project_ll_net_ana_results $prjnum
    reset_project_unfolding_text_ana_results $prjnum
    update_project_infos $prjnum
    update_project_files $prjnum
    new_project_msg $prjnum "PBC -> LL Net finished"
  } else {
    error_msg "PBC $prjx(actdir)/$prjx(boxexpr.filename) does not exists"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_exe_boxexpr_ll_net

proc prj_exe_bpn2_ll_net_via_boxexpr {} {
  # Die zwei anderen Funktionen aufrufen und evtl. Fehler beruecksichtigen.
  if [prj_exe_bpn2_boxexpr] {
    prj_exe_boxexpr_ll_net
  }  ;# if erste Aktion erfolgreich
}  ;# prj_exe_bpn2_ll_net_via_boxexpr

proc prj_exe_bpn2_hl_net {} {
  # Die evtl. vorhandene B(PN)-Datei checken und 
  # dann ggf. in ein HL-Netz umwandeln.
  global gvar
  global errorCode

    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Referenzdateien loeschen
  remove_ref_files bpn2_hl_net $prjx(project.name)
  remove_ref_files hl_net $prjx(project.name)
    # Testen, ob die Source da ist
  if [file exists "$prjx(bpn2.normal.filename)"] {
    if [get_options_syntax_check] {
        # kein Syntax Check
      DEBUG "NO Syntax Check"
      set prjx(bpn2.normal.syntax_ok) "Skipped"
      set prjx(bpn2.normal.procs) "Unknown"
      update_project_files $prjnum
        # die naechsten beiden Variablen werden fuer die Referenzen gebraucht
      set bpn2_filename $prjx(bpn2.normal.filename)
      set woher "B(PN)"
    } else {  ;# Syntax-Check durchfuehren
        # temporaeren Dateinamen fuer den Syntax-Checker basteln
      set result_file \
        [change_filename_extension $prjx(bpn2.normal.filename) bpn2_res]
        # Syntax-Checker aufufen
      set sys_str "exec $gvar(bpn2_syntaxchecker.name) \
        $prjx(bpn2.normal.filename) $result_file -e"
      DEBUG "Aufruf: $sys_str"
      set errorCode "NONE"
      set sys_res [catch {eval $sys_str} result_text]
      DEBUG " ergab: $sys_res $result_text *$errorCode*"
      new_project_msg $prjnum "B(PN) Syntax checked"
      if [string compare $errorCode "NONE"] {
        set prjx(bpn2.normal.syntax_ok) "Failed"
        set prjx(bpn2.normal.procs) "Unknown"
        update_project_files $prjnum
        file delete $result_file
        cd $actdir
        error_msg "Error while parsing B(PN) Programm (Syntax Checker)"
        return 0
      } else {  ;# kein Fehler
          # Ergebnis handeln
        set sys_res [pattern_in_file $result_file {parse error}]
        if {$sys_res == 1} {
          set prjx(bpn2.normal.syntax_ok) "Errors"
          set prjx(bpn2.normal.procs) "Unknown"
          update_project_files $prjnum
          file delete $result_file
          cd $actdir
          error_msg "B(PN) Program has Error(s)"
          return 0
        }  ;# if Fehler im Programm
        set prjx(bpn2.normal.syntax_ok) "Ok"
        update_project_files $prjnum
          # feststellen, ob Prozeduren oder nicht
        set sys_res [pattern_in_file $result_file {Program without procedures}]
          # temporaere Datei wieder entfernen
        file delete $result_file
          # den norm. B(PN) Dateinamen merken, wird im if evtl. ueberschrieben
        set bpn2_filename $prjx(bpn2.normal.filename)
        set woher "B(PN)"  ;# wird spaeter evtl. als Referenz eingetragen
          # OK, so far, evtl. expandieren
        if {$sys_res == 0} {
          set prjx(bpn2.normal.procs) "Yes"
          update_project_files $prjnum
            # Programm expandieren
          set $prjx(bpn2.expanded.filename) [change_filename_extension \
            $prjx(bpn2.normal.filename) bpn2_exp]
          set opt [get_options_bpn2_exp]
          set sys_str "exec $gvar(bpn2_exp.name) \
            $prjx(bpn2.normal.filename) $prjx(bpn2.expanded.filename) $opt"
          DEBUG "Aufruf: $sys_str"
          set errorCode "NONE"
          set sys_res [catch {eval $sys_str} result_text]
          DEBUG " ergab: $sys_res $result_text"
          if {$sys_res != 0} {
            set prjx(bpn2.expanded.ref) "None"
            update_project_files $prjnum
            cd $actdir
            error_msg "Error while expanding B(PN) Programm (Expander)\n\
               $result_text"
            return 0
          }  ;# if Fehler beim Expandieren
          set prjx(bpn2.expanded.ref) "B(PN)"
          update_project_files $prjnum
            # den Dateinamen des expandierten Programmes merken
          set bpn2_filename $prjx(bpn2.expanded.filename)
          set woher "Exp-B(PN)"  ;# wird spaeter evtl. als Referenz eingetragen
            # Hinweis ausgeben
          new_project_msg $prjnum "B(PN) Program expanded"
        } else {
          set prjx(bpn2.normal.procs) "No"
          update_project_files $prjnum
        }  ;# else, Programm hat keine Prozeduren
      }  ;# else, kein Fehler
    }  ;# else, Syntax-Check machen
      # Konverter von B(PN) nach HL-Netz aufrufen, kein neuer Netzdateiname
    set opt [get_options_bpn2_hl_net]
    set bpn2_hl_net_ref_filename \
      [change_filename_extension $bpn2_filename bpn2_hl_ref]
    set sys_str "exec $gvar(bpn2_to_hl_net.name) $bpn2_filename \
      -p=$prjx(hl_net.filename) -z=$bpn2_hl_net_ref_filename $opt"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if {$sys_res != 0} {
	# warnings abgefangen...
	if {$errorCode != "NONE" } {
	  if {![string compare $prjx(ll_net.ref) "HL-Net"]} {
       	    set prjx(ll_net.ref) "None"
          }  ;# if Verweis bestand
      	  set prjx(hl_net.ref) "None"
      	  update_project_files $prjnum
      	  cd $actdir
      	  error_msg "Error while making HL Net\n$result_text"
      	  return 0
	} else {
      	  error_msg "Warning while making HL Net\n$result_text"
        }
    }

    if {![string compare $prjx(ll_net.ref) "HL-Net"]} {
      set prjx(ll_net.ref) "None"
    }
    set prjx(hl_net.ref) $woher
    update_project_files $prjnum
    new_project_msg $prjnum "B(PN) -> HL Net finished"
    # Der B(PN)2 Editor sollte jetzt die neue Referenz laden 
    update_text_editors
    
  } else {
    error_msg \
    "B(PN)-Program $prjx(actdir)/$prjx(bpn2.normal.filename) does not exists"
    return 0
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
  return 1
}  ;# prj_exe_bpn2_hl_net

proc prj_exe_bpn2_exp_hl_net {} {
  # Die evtl. vorhandene Expanded-B(PN)-Datei checken und 
  # dann ggf. in ein HL-Netz umwandeln.
  global gvar
  global errorCode

    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
   
  if [file exists "$prjx(bpn2.expanded.filename)"] {
#Erst spaeter loeschen, hier nur Syntaxcheck
#    remove_ref_files exp_bpn2_hl_net $prjx(project.name)
#    remove_ref_files hl_net $prjx(project.name)

    if [get_options_syntax_check] {
        # kein Syntax Check
      DEBUG "NO Syntax Check"
    } else {  ;# Syntax-Check durchfuehren
        # temporaeren Dateinamen fuer den Syntax-Checker basteln
      set result_file \
        [change_filename_extension $prjx(bpn2.expanded.filename) bpn2_exp_res]
        # Syntax-Checker aufufen
      set sys_str "exec $gvar(bpn2_syntaxchecker.name) \
        $prjx(bpn2.expanded.filename) $result_file -e"
      DEBUG "Aufruf: $sys_str"
      set errorCode "NONE"
      set sys_res [catch {eval $sys_str} result_text]
      DEBUG " ergab: $sys_res $result_text"
      new_project_msg $prjnum "Expanded B(PN) Syntax checked"
      if [string compare $errorCode "NONE"] {
        file delete $result_file
        cd $actdir
        error_msg \
          "Error while parsing Expanded B(PN) Programm (Syntax Checker)"
        return
      } else {  ;# kein Fehler
          # Ergebnis handeln
        set sys_res [pattern_in_file $result_file {parse error}]
        file delete $result_file
        if {$sys_res == 1} {
          cd $actdir
          error_msg "Expanded B(PN) Program has Error(s)"
          return
        }  ;# if Fehler im Programm
      }  ;# else, kein Fehler beim Checken
    }  ;# else, Syntax-Check durchfuehren


      # Alte Referenzen loeschen (?)
    remove_ref_files exp_bpn2_hl_net $prjx(project.name)
    remove_ref_files hl_net $prjx(project.name)

      # Konverter von Expanded-B(PN) nach HL-Netz aufrufen
      # kein neuer Netzdateiname
    set opt [get_options_bpn2_hl_net]
    set bpn2_hl_net_ref_filename \
      [change_filename_extension $prjx(bpn2.expanded.filename) exp_bpn2_hl_ref]
    set sys_str "exec $gvar(bpn2_to_hl_net.name) \
      $prjx(bpn2.expanded.filename) -p=$prjx(hl_net.filename) \
      -z=$bpn2_hl_net_ref_filename $opt"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if {$sys_res != 0} {
      if [string compare $prjx(ll_net.ref) "HL-Net"] {
      } else {
        set prjx(ll_net.ref) "None"
      }  ;# else, Verweis bestand
      set prjx(hl_net) "None"
      update_project_files $prjnum
      cd $actdir
      error_msg "Error while making HL Net\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    if [string compare $prjx(ll_net.ref) "HL-Net"] {
    } else {
      set prjx(ll_net.ref) "None"
    }  ;# else, Verweis bestand
    set prjx(hl_net.ref) "Exp-B(PN)"
    update_project_files $prjnum
    new_project_msg $prjnum "Expanded B(PN) -> HL Net finished"
  } else {
    error_msg "Expanded B(PN) Program \
      $prjx(actdir)/$prjx(bpn2.expanded.filename) does not exists"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_exe_bpn2_exp_hl_net

proc prj_exe_hl_net_ll_net {} {
  # Die evtl. vorhandene HL Netzdatei in ein LL Netz umwandeln.
  global gvar
  global errorCode

    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(hl_net.filename)"] {
    # Loeschen der Referenzdateien
  remove_ref_files hl2ll $prjx(project.name)
  remove_ref_files ll_net $prjx(project.name)
    set opt [get_options_hl_net_ll_net]

	# Verbindungen testen
  	set ext [get_ref_ext $prjx(project.name)]
	set hl "_hl_ref"
	set ll "_ll_ref"
	if {$ext == "NONE"} {
    		set sys_str "exec $gvar(hl_net_to_ll_net.name) $opt \
      			$prjx(hl_net.filename) $prjx(ll_net.filename) \
      			-hl $prjx(hl_ll_ref.filename)"
	} else {
    		set sys_str "exec $gvar(hl_net_to_ll_net.name) $opt \
      			$prjx(hl_net.filename) $prjx(ll_net.filename) \
			-bh $prjx(project.name).$ext$hl \
      			-hl $prjx(hl_ll_ref.filename) \
			-bl $prjx(project.name).$ext$ll"
	}

    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"

    if [string compare $errorCode "NONE"] {
      if [string compare $prjx(ll_net.ref) "HL-Net"] {
      } else {
        set prjx(ll_net.ref) "None"
        update_project_files $prjnum
      }  ;# else, Verweis bestand
      cd $actdir
      error_msg "Error while making LL Net\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    set prjx(ll_net.ref) "HL-Net"
    set prjx(unfolding.text.ref) "None"
    set prjx(spin.ref) "None"
    set prjx(ina.ref) "None"
    set prjx(smv.ref) "None"
    set prjx(fc2.ref) "None"
    reset_project_ll_net_ana_results $prjnum
    reset_project_unfolding_text_ana_results $prjnum
    reset_project_promela_text_ana_results $prjnum
    update_project_infos $prjnum
    update_project_files $prjnum
    new_project_msg $prjnum "HL Net -> LL Net finished"
    # Der B(PN)2 Editor sollte jetzt die neue (LL) Referenz laden
    update_text_editors

  } else {
    error_msg "HL Net $prjx(actdir)/$prjx(hl_net.filename) does not exists"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_exe_hl_net_ll_net

proc prj_exe_bpn2_ll_net_via_hl_net {} {
  # Die zwei anderen Funktionen aufrufen und evtl. Fehler beruecksichtigen.
  if [prj_exe_bpn2_hl_net] {
    prj_exe_hl_net_ll_net
  }  ;# if erste Aktion erfolgreich
}  ;# prj_exe_bpn2_ll_net_via_hl_net

proc prj_exe_bpn2_bpn2_exp {} {
  # Falls die B(PN)-Datei vorhanden ist, diese expandieren.
  global gvar
  global errorCode

    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(bpn2.normal.filename)"] {
    if [get_options_syntax_check] {
        # kein Syntax Check
      DEBUG "NO Syntax Check"
      set prjx(bpn2.normal.syntax_ok) "Skipped"
      set prjx(bpn2.normal.procs) "Unknown"
      update_project_files $prjnum
    } else {  ;# Syntax-Check vornehmen
        # temporaeren Dateinamen fuer den Syntax-Checker basteln
      set result_file \
        [change_filename_extension $prjx(bpn2.normal.filename) bpn2_res]
        # Syntax-Checker aufufen
      set sys_str "exec $gvar(bpn2_syntaxchecker.name) \
        $prjx(bpn2.normal.filename) $result_file -e"
      DEBUG "Aufruf: $sys_str"
      set errorCode "NONE"
      set sys_res [catch {eval $sys_str} result_text]
      DEBUG " ergab: $sys_res $result_text"
      new_project_msg $prjnum "B(PN) Syntax checked"
      if [string compare $errorCode "NONE"] {
        set prjx(bpn2.normal.syntax_ok) "Failed"
        set prjx(bpn2.normal.procs) "Unknown"
        update_project_files $prjnum
        file delete $result_file
        cd $actdir
        error_msg "Error while parsing B(PN) Programm (Syntax Checker)"
        return
      } else {  ;# kein Fehler
          # Ergebnis handeln
        set sys_res [pattern_in_file $result_file {parse error}]
        file delete $result_file
        if {$sys_res == 1} {
          set prjx(bpn2.normal.syntax_ok) "Errors"
          set prjx(bpn2.normal.procs) "Unknown"
          update_project_files $prjnum
          cd $actdir
          error_msg "B(PN) Program has Error(s)"
          return
        }  ;# if Fehler im Programm
        set prjx(bpn2.normal.syntax_ok) "Ok"
        update_project_files $prjnum
      }  ;# else, keine Fehler
    }  ;# else, Syntax-Check vornehmen
      #
      # Loeschen der alten Referenzen
      remove_ref_files exp_bpn2 $prjx(project.name)
      remove_ref_files bpn2_exp_bpn2 $prjx(project.name)

      # Programm expandieren
    set $prjx(bpn2.expanded.filename) [change_filename_extension \
      $prjx(bpn2.normal.filename) bpn2_exp]
    set opt [get_options_bpn2_exp]
    set sys_str "exec $gvar(bpn2_exp.name) \
      $prjx(bpn2.normal.filename) $prjx(bpn2.expanded.filename) $opt"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if {$sys_res != 0} {
      set prjx(bpn2.expanded.ref) "None"
      if [string compare $prjx(hl_net.ref) "Exp-B(PN)"] {
      } else {
        set prjx(hl_net.ref) "None"
      }  ;# else, Verweis bestand
      update_project_files $prjnum
      cd $actdir
      error_msg "Error while expanding B(PN) Programm (Expander)\n\
        $result_text"
      return
    }  ;# if Fehler beim Expandieren
    set prjx(bpn2.expanded.ref) "B(PN)"
    if [string compare $prjx(hl_net.ref) "Exp-B(PN)"] {
    } else {
      set prjx(hl_net.ref) "None"
    }  ;# else, Verweis bestand
    update_project_files $prjnum
    new_project_msg $prjnum "B(PN) Program expanded"
  } else {
    error_msg "B(PN) $prjx(actdir)/$prjx(hl_net.filename) does not exists"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_exe_bpn2_bpn2_exp

proc prj_exe_bpn2_c_source {} {
  # Die evtl. vorhandene B(PN)-Datei in eine C-Source-Datei umwandeln.
  global gvar
  global errorCode

    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(bpn2.normal.filename)"] {
      # Konverter von B(PN) nach C-Source aufrufen, kein neuer Dateiname
      # TODO: Optionen verwalten
    set sys_str "exec $gvar(bpn2_2_c_source.name) \
      $prjx(bpn2.normal.filename)"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      set prjx(c_source.ref) "None"
      update_project_files $prjnum
      cd $actdir
      error_msg "Error while making C Source\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    set prjx(c_source.ref) "B(PN)"
    update_project_files $prjnum
    new_project_msg $prjnum "B(PN) -> C Source finished"
  } else {
    error_msg "B(PN) $prjx(actdir)/$prjx(bpn2.normal.filename) does not exists"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_exe_bpn2_c_source

# Aufrufparameter fuer syncmnet
proc get_sync_sys_str {inc_opt exc_opt opt} {

  global gvar
  set prjnum $gvar(actprj)
  upvar #0 prj$prjnum prjx

    set ext [get_ref_ext $prjx(project.name)]
    set hl "_hl_ref"
    if {$ext == "NONE"} {
   	set sys_str "exec $gvar(hl_net_sync.name) $prjx(hl_net.filename) \
     		-p=$prjx(hl_net.filename) $inc_opt $exc_opt $opt"
    } else {
   	set sys_str "exec $gvar(hl_net_sync.name) $prjx(hl_net.filename) \
     		-p=$prjx(hl_net.filename) $inc_opt $exc_opt $opt \
 		-Z=$prjx(project.name).$ext$hl -z=$prjx(project.name).$ext$hl"
    }
}
##################################################################################



proc prj_exe_sync_hl_net {prjnum toplevel} {
  # Falls die HL-Netzdatei vorhanden ist diese synchronisieren.
  # prjnum - Die Nummer des Projektes (IN)
  # toplevel - Name des Fensters fuer das Ergebnis (IN, wird evtl. veraendert)
  # aus gvar wird option.exe_sync_hl_net.* benutzt
  global gvar
  global errorCode

    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Referenzdateien loeschen
  remove_ref_files ll_net $prjx(project.name)
#TODO sdl soll speater nicht geloescht werden
  remove_ref_files sdl $prjx(project.name)
    # Testen, ob die Source da ist
  if [file exists "$prjx(hl_net.filename)"] {
      # HL Netz synchronisieren
    set opt [get_options_sync_hl_net]
      # feststellen, ob alle Aktionen gemeint sind
    set gvar(option.exe_sync_hl_net.include_actions) \
      [string trim $gvar(option.exe_sync_hl_net.include_actions)]
    if {$gvar(option.exe_sync_hl_net.all_actions)} {
      set inc_opt "-j"
    } else {
      if {[string length $gvar(option.exe_sync_hl_net.include_actions)] == 0} {
        cd $actdir
        error_msg "Error while sync HL Net\nThere are no actions specified."
        return
      } else {
      set inc_opt "-j=$gvar(option.exe_sync_hl_net.include_actions)"
      }  ;# else, Include angegeben
    }  ;# else, es waren nicht alle gemeint

    set gvar(option.exe_sync_hl_net.exclude_actions) \
      [string trim $gvar(option.exe_sync_hl_net.exclude_actions)]
    if {[string length $gvar(option.exe_sync_hl_net.exclude_actions)] == 0} {
      set exc_opt {}
    } else {
      set exc_opt "-X=$gvar(option.exe_sync_hl_net.exclude_actions)"
    }  ;# else, Exclude angegeben

    set bpn2_hl_net_ref_filename \
      [change_filename_extension $prjx(hl_net.filename) bpn2_hl_ref]

    set sys_str [get_sync_sys_str $inc_opt $exc_opt $opt]

    #set sys_str "exec $gvar(hl_net_sync.name) $prjx(hl_net.filename) \
    #  -p=$prjx(hl_net.filename) $inc_opt $exc_opt $opt \
    #  -Z=$bpn2_hl_net_ref_filename -z=$bpn2_hl_net_ref_filename"

    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      cd $actdir
      error_msg "Error while sync HL Net\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
      # Nur den Verweis auf das LL Netz loeschen, nicht auf (Exp-) B(PN)!
    if [string compare $prjx(ll_net.ref) "HL-Net"] {
    } else {
      set prjx(ll_net.ref) "None"
    }  ;# else, Verweis bestand
    update_project_infos $prjnum
    new_project_msg $prjnum "Sync HL Net finished"
  } else {
    error_msg "HL Net $prjx(actdir)/$prjx(ll_net.filename) does not exists"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_exe_sync_hl_net

proc prj_exe_sync_hl_net_dlg {} {
  # Den Dialog fuer die Eingabe der Aktionen aufbauen.
  global gvar

    # Die Projektnummer abholen. WICHTIG, dass jenes jetzt passiert!
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx

    # Menupunkt abschalten, spaeter wieder einschalten
  $prjx(project.exe.menu.widget) entryconfigure "Sync HL Net" \
    -state disabled

    # Dialog aufbauen
  if [catch {set tl [toplevel .exe_sync_hl_net_dlg$prjnum]}] {
    return
  }  ;# if doppelt
  add_dlg_widget $prjnum $tl
  wm title $tl "Sync HL Net"
  wm protocol $tl WM_DELETE_WINDOW "dlg_option_wm_delete $prjnum $tl \
    {$prjx(project.exe.menu.widget) entryconfigure {Sync HL Net} \
    -state normal}"

    # Eingabebereich
  frame $tl.input  -bd 2
  pack $tl.input  -side top  -fill both
  label $tl.input.general1  -text "Give the Actions with their Arities," -anchor w 
  label $tl.input.general2  -text "e.g.: sem(x,x),X(x,x,x),Y(x)" -anchor w 

  set gvar(option.exe_sync_hl_net.all_actions) \
    $prjx(hl_net.exe.sync_hl_net.all_actions)
  checkbutton $tl.input.include_all  -text "Include all Actions" \
    -anchor w \
    -variable gvar(option.exe_sync_hl_net.all_actions)
  pack $tl.input.general1 $tl.input.general2 $tl.input.include_all \
    -side top  -anchor w  -fill x

  frame $tl.input.include_actions
  pack $tl.input.include_actions \
    -side top  -expand 1  -fill x
  label $tl.input.include_actions.label  -text "Include Actions"  -bd 2
  set gvar(option.exe_sync_hl_net.include_actions) \
    $prjx(hl_net.exe.sync_hl_net.include_actions)
  entry $tl.input.include_actions.entry  -bd 2 \
    -textvariable gvar(option.exe_sync_hl_net.include_actions)
  pack $tl.input.include_actions.label \
    -side left
  pack $tl.input.include_actions.entry \
    -side left  -fill x  -expand 1

  frame $tl.input.exclude_actions
  pack $tl.input.exclude_actions \
    -side top  -expand 1  -fill x
  label $tl.input.exclude_actions.label  -text "Exclude Actions"  -bd 2
  set gvar(option.exe_sync_hl_net.exclude_actions) \
    $prjx(hl_net.exe.sync_hl_net.exclude_actions)
  entry $tl.input.exclude_actions.entry  -bd 2 \
    -textvariable gvar(option.exe_sync_hl_net.exclude_actions)
  pack $tl.input.exclude_actions.label \
    -side left
  pack $tl.input.exclude_actions.entry \
    -side left  -fill x  -expand 1

    # Ausgabebereich ist leer

    # Button-Bereich
  frame $tl.commands  -relief raised  -bd 2
  pack $tl.commands  -side bottom  -fill x
  button $tl.commands.ok  -text "Ok" \
    -command " \
      $tl.commands.ok configure  -state disabled; \
      $tl.commands.cancel configure  -state disabled; \
      update; \
      prj_exe_sync_hl_net $prjnum $tl; \
      $tl.commands.ok configure  -state normal; \
      $tl.commands.cancel configure  -state normal; \
      update \
    "
  button $tl.commands.cancel  -text "Cancel" \
    -command " \
      del_dlg_widget $prjnum $tl; \
      destroy $tl; \
      $prjx(project.exe.menu.widget) entryconfigure {Sync HL Net} \
        -state normal \
    "
  pack $tl.commands.ok \
    $tl.commands.cancel \
    -side left  -expand 1
}  ;# prj_exe_sync_hl_net_dlg

# Simplify Aufruf mit Referenzen
# get_hl_ref_ext steht in der util.tcl
# hl_ und ll_flag entsprechen den Prefix-Optionen
proc get_simplify_sys_str { hl_flag ll_flag } {
  global gvar

  set prjnum $gvar(actprj)
  upvar #0 prj$prjnum prjx

  set files ""
  set refs  ""

  # Verbindungen testen
  set ext_hl [get_hl_ref_ext $prjx(project.name)]
  set ext_ll [get_ll_ref_ext $prjx(project.name)]

  if [file exists $prjx(hl_ll_ref.filename)] {
     set refs "-hl $prjx(hl_ll_ref.filename) -ohl $prjx(hl_ll_ref.filename)"
  }

  if {$hl_flag} {
  	set files "$files $prjx(hl_net.filename)"
	if [file exists $prjx(project.name).$ext_hl] {
	   set refs "$refs -bh $prjx(project.name).$ext_hl -obh $prjx(project.name).$ext_hl"
	}
  }

  if {$ll_flag} {
  	set files "$files $prjx(ll_net.filename)"
	if [file exists $prjx(project.name).$ext_ll] {
	   set refs "$refs -bl $prjx(project.name).$ext_ll -obl $prjx(project.name).$ext_ll"
	}
  }

  DEBUG "Simplify: $files"
  DEBUG "Ref:      $refs"

  return "exec simplify -p $files $refs"
}

proc prj_exe_ll_net_unfolding {} {
  # Die evtl. vorhandene LL-Netzdatei in endl. Praefix umwandeln.
  global gvar
  global errorCode

    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"


    # Testen, ob die Source da ist
  if [file exists "$prjx(ll_net.filename)"] {
      # Konverter von LL-Netz nach Entfaltung aufrufen
    set unused_transitions_filename \
      [change_filename_extension $prjx(ll_net.filename) ll_net_unused_trans]
    set opt [get_options_unfolding_text $unused_transitions_filename \
      $prjx(hl_ll_ref.filename) \
      unused_transitions_flag minimized_hl_net_flag minimized_net_flag]
    if {$minimized_hl_net_flag} {
       # Referenzdateien loeschen
        remove_ref_files minimize_hl $prjx(project.name)
    } 
    if {$minimized_net_flag} {
       # Referenzdateien loeschen
        remove_ref_files minimize_ll $prjx(project.name)
       # Da .fc2, .pnt und .cpt  geloescht wird, sollte die Referenz auch weg...
        set prjx(fc2.ref) "None"
	set prjx(ina.ref) "None"
    } 
    # Testen, ob die HL -> LL Referenzen da sind
#    if {$minimized_hl_net_flag} {
#      if ![file exists $prjx(project.name).hl_ll_ref] {
#         if {$use_esparza_vogler_flag} {
#	   tk_dialog .d { } \
#	     {You have chosen to minimise the HL Net,\
#	      but the LL Net is not connected to\
#              the HL Net. Please, change this option\
#	      of the prefix-builder, if you would\
#              like to calculate the prefix.} \
#	      warning 0 {OK}
#           return
#	 } else {
#           error_msg "The LL Net is not connected to the HL Net"
#           return
#         }
#      }
#    }
    set sys_str "exec $gvar(option.dlg.unfolding_text.unfolder) \
      $prjx(ll_net.filename) -m=$prjx(unfolding.text.filename) $opt"

    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      set prjx(unfolding.text.ref) "None"
      update_project_files $prjnum
      cd $actdir
      error_msg "Error while building Prefix\n$result_text"
      return
    }  ;# if Fehler beim Konvertieren
    set prjx(unfolding.text.ref) "LL-Net"
    update_project_files $prjnum
    
# TODO
    if {$minimized_net_flag} {
      if {$minimized_hl_net_flag} {
        DEBUG "Minimize the HL and LL Net"

  	set sys_str "[get_simplify_sys_str $minimized_hl_net_flag $minimized_net_flag] \
		-dt $unused_transitions_filename"
        DEBUG "Aufruf: $sys_str"
        set errorCode "NONE"
        set sys_res [catch {eval $sys_str} result_text]
        DEBUG " ergab: $sys_res $result_text"
        if [string compare $errorCode "NONE"] {
          update_project_files $prjnum
          cd $actdir
          error_msg "Error while minimizing HL and LL Net\n$result_text"
          return
        }  ;# if Fehler beim Minimieren

        if [file exists $prjx(hl_net.filename).min] {
          file rename -force $prjx(hl_net.filename).min $prjx(hl_net.filename)
            # TODO Reload of net editor (if needed)
        } else {
          error_msg "Minimzed HL Net is gone with the Wind \
            ($prjx(hl_net.filename).min)"
        }  ;# else, Datei nicht vorhanden
        if [file exists $prjx(ll_net.filename).min] {
          file rename -force $prjx(ll_net.filename).min $prjx(ll_net.filename)
            # TODO Reload of net editor (if needed)
        } else {
          error_msg "Minimzed LL Net is gone with the Wind \
            ($prjx(ll_net.filename).min)"
        }  ;# else, Datei nicht vorhanden

#TODO Namen der Ref-Dateien allgemeiner
#        if [file exists $prjx(project.name).bpn2_ll_ref.min] {
#          file rename -force $prjx(project.name).bpn2_ll_ref.min $prjx(project.name).bpn2_ll_ref
#            # TODO Reload des Netzedis (evtl.)
#        } else {
#          error_msg "Minimized B(PN)^2->LL References are gone with the Wind \
#            ($prjx(project.name).bpn2_ll_ref.min)"
#        }  ;# else, Datei nicht vorhanden
#        if [file exists $prjx(project.name).bpn2_hl_ref.min] {
#          file rename -force $prjx(project.name).bpn2_hl_ref.min $prjx(project.name).bpn2_hl_ref
#            # TODO Reload des Netzedis (evtl.)
#        } else {
#          error_msg "Minimized B(PN)^2->HL References are gone with the Wind \
#            ($prjx(project.name).bpn2_hl_ref.min)"
#        }  ;# else, Datei nicht vorhanden
#
#        if [file exists $prjx(project.name).hl_ll_ref.min] {
#          file rename -force $prjx(project.name).hl_ll_ref.min $prjx(project.name).hl_ll_ref
#            # TODO Reload des Netzedis (evtl.)
#        } else {
#          error_msg "Minimized HL->LL References are gone with the Wind \
#            ($prjx(project.name).hl_ll_ref.min)"
#        }  ;# else, Datei nicht vorhanden

        # Der B(PN)2 Editor sollte jetzt die neue Referenz laden
	update_text_editors

      } else {
        DEBUG "Minimize the LL Net"
	# Simplify mit Referenzen
  	set sys_str "[get_simplify_sys_str $minimized_hl_net_flag $minimized_net_flag] \
		-dt $unused_transitions_filename"
        DEBUG "Aufruf: $sys_str"
        set errorCode "NONE"
        set sys_res [catch {eval $sys_str} result_text]
        DEBUG " ergab: $sys_res $result_text"
        if [string compare $errorCode "NONE"] {
          update_project_files $prjnum
          cd $actdir
          error_msg "Error while minimizing LL Net\n$result_text"
          return
        }  ;# if Fehler beim Minimieren
        if [file exists $prjx(ll_net.filename).min] {
          file rename -force $prjx(ll_net.filename).min $prjx(ll_net.filename)
            # TODO Reload des Netzedis (evtl.)
        } else {
          error_msg "Minimzed LL Net is gone with the Wind \
            ($prjx(ll_net.filename).min)"
        }  ;# else, Datei nicht vorhanden

#        if [file exists $prjx(project.name).bpn2_ll_ref.min] {
#          file rename -force $prjx(project.name).bpn2_ll_ref.min $prjx(project.name).bpn2_ll_ref
#            # TODO Reload des Netzedis (evtl.)
#        } else {
#          error_msg "Minimzed B(PN)^2->LL References are gone with the Wind \
#            ($prjx(project.name).bpn2_ll_ref.min)"
#        }  ;# else, Datei nicht vorhanden
#        if [file exists $prjx(project.name).hl_ll_ref.min] {
#          file rename -force $prjx(project.name).hl_ll_ref.min $prjx(project.name).hl_ll_ref
#            # TODO Reload des Netzedis (evtl.)
#        } else {
#          error_msg "Minimzed HL->LL References are gone with the Wind \
#            ($prjx(project.name).hl_ll_ref.min)"
#        }  ;# else, Datei nicht vorhanden

	update_text_editors

      }  ;# if minimiertes HL Netz erstellen
    } else {

      if {$minimized_hl_net_flag} {
        DEBUG "Minimize the HL Net"
	# Simplify mit Referenzen
  	set sys_str "[get_simplify_sys_str $minimized_hl_net_flag $minimized_net_flag] \
		-dt $unused_transitions_filename"
        DEBUG "Aufruf: $sys_str"
        set errorCode "NONE"
        set sys_res [catch {eval $sys_str} result_text]
        DEBUG " ergab: $sys_res $result_text"
        if [string compare $errorCode "NONE"] {
          update_project_files $prjnum
          cd $actdir
          error_msg "Error while minimizing HL Net\n$result_text"
          return
        }  ;# if Fehler beim Minimieren
        if [file exists $prjx(hl_net.filename).min] {
          file rename -force $prjx(hl_net.filename).min $prjx(hl_net.filename)
            # TODO Reload des Netzedis (evtl.)
        } else {
          error_msg "Minimzed HL Net is gone with the Wind \
            ($prjx(hl_net.filename).min)"
        }  ;# else, Datei nicht vorhanden
        
#        if [file exists $prjx(project.name).bpn2_hl_ref.min] {
#          file rename -force $prjx(project.name).bpn2_hl_ref.min $prjx(project.name).bpn2_hl_ref
#            # TODO Reload des Netzedis (evtl.)
#        } else {
#          error_msg "Minimzed B(PN)^2->HL References are gone with the Wind \
#            ($prjx(project.name).bpn2_hl_ref.min)"
#        }  ;# else, Datei nicht vorhanden
#        if [file exists $prjx(project.name).hl_ll_ref.min] {
#          file rename -force $prjx(project.name).hl_ll_ref.min $prjx(project.name).hl_ll_ref
#            # TODO Reload des Netzedis (evtl.)
#        } else {
#          error_msg "Minimzed HL->LL References are gone with the Wind \
#            ($prjx(project.name).hl_ll_ref.min)"
#        }  ;# else, Datei nicht vorhanden

	update_text_editors

      }  ;# if minimiertes HL Netz erstellen
    }  ;# if minimiertes Netz erstellen

    if {$unused_transitions_flag} {
      DEBUG "Display the unused Transitions"
      if [file exists $unused_transitions_filename] {
        set result_text [cat $unused_transitions_filename]
        if {![string length [string trim $result_text]]} {
          set result_text "There are no unused Transitions in the LL Net."
        } ;# else, keine Transitionen
        tk_dialog .unused_transitions_dlg {List of unused transitions} \
          "$result_text" {} 0 {Ok}
        file delete $unused_transitions_filename
      } else {
        error_msg "List of unused Transitions is gone with the Wind \
            ($unused_transitions_filename)"
      }  ;# else, Datei war nicht da
    }  ;# if Liste ausgeben

    file delete $unused_transitions_filename

    new_project_msg $prjnum "LL Net -> Prefix finished"
  } else {
    error_msg "LL Net $prjx(actdir)/$prjx(ll_net.filename) does not exists"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_exe_ll_net_unfolding


proc prj_exe_ll_net_minimize {} {
  # Die evtl. vorhandene LL-Netzdatei per Entfalter minimieren.
  global gvar
  global errorCode

    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(ll_net.filename)"] {
      # Konverter von LL-Netz nach Entfaltung aufrufen mit einer Dummy-Datei
    set dummy_unfolding "$prjx(ll_net.filename).dummy.mci"
    set unused_transitions_filename \
      [change_filename_extension $prjx(ll_net.filename) ll_net_unused_trans]
    set opt [get_options_unfolding_text $unused_transitions_filename \
      $prjx(hl_ll_ref.filename) \
      unused_transitions_flag minimized_hl_net_flag minimized_net_flag]
    if {$minimized_hl_net_flag} {
       # Referenzdateien loeschen
        remove_ref_files minimize_hl $prjx(project.name)
    } 
    if {$minimized_net_flag} {
       # Referenzdateien loeschen
        remove_ref_files minimize_ll $prjx(project.name)
       # Da .fc2, .cpt und .pnt geloescht wird, sollte die Referenz auch weg...
        set prjx(fc2.ref) "None"
	set prjx(ina.ref) "None"
    } 
    if {$gvar(option.dlg.unfolding_text.use_esparza_vogler)} {
      set prgname "$gvar(ll_net_to_unfolding.name) -@"
    } else {
      set prgname "$gvar(ll_net_to_unfolding.name)"
    }  ;# else, den alten Entfalter nehmen
    set sys_str "exec $prgname $prjx(ll_net.filename) \
      -m=$dummy_unfolding -sn=1b -T=$unused_transitions_filename "
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      update_project_files $prjnum
      cd $actdir
      error_msg "Error while minimizing LL Net\n$result_text"
      return
    }  ;# if Fehler beim Minimieren
    
    set sys_str "[get_simplify_sys_str $minimized_hl_net_flag $minimized_net_flag] \
	-dt $unused_transitions_filename"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      update_project_files $prjnum
      cd $actdir
      error_msg "Error while minimizing LL Net\n$result_text"
      return
    }  ;# if Fehler beim Minimieren

    update_project_files $prjnum
    DEBUG "Minimize the LL Net"
    if [file exists $prjx(ll_net.filename).min] {
      file rename -force $prjx(ll_net.filename).min $prjx(ll_net.filename)
        # TODO Reload des Netzedis (evtl.)
    } else {
      error_msg "Minimzed Net is gone with the Wind \
        ($prjx(ll_net.filename).min)"
    }  ;# else, Datei nicht vorhanden
    file delete $dummy_unfolding
    new_project_msg $prjnum "Minimizing LL Net finished"
  } else {
    error_msg "LL Net $prjx(actdir)/$prjx(ll_net.filename) does not exists"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_exe_ll_net_minimize

proc prj_exe_ll_net_ina {} {
  # Die evtl. vorhandene LL-Netzdatei in eine Datei f"ur INA umwandeln.
  global gvar
  global errorCode

    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(ll_net.filename)"] {
     # Referenzdateien loeschen
    remove_ref_files ll_ina $prjx(project.name)

      # Konverter von LL-Netz nach Entfaltung aufrufen
    if {$gvar(option.dlg.pep_to_ina.colored)} {
      set extension "cnt"
    } else {
      set extension "pnt"
    }  ;# File extension anpassen
    set prjx(ina.filename) [change_filename_extension $prjx(ll_net.filename) \
      $extension]
    file delete $prjx(ina.filename)
    set opt [get_options_pep_to_ina]
    set sys_str "exec $gvar(ll_net_to_ina.name) \
      $opt $prjx(ll_net.filename) $prjx(ina.filename)"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      DEBUG "errorCode: $errorCode"
      set prjx(ina.ref) "None"
      update_project_files $prjnum
      cd $actdir
      return
#TODO:  das TODO hier drunter ist jetzt bestimmt erledigt...
# TODO: Ref verwalten INA -> LL
#       set prjx(unfolding.text.ref) "None"
#       update_project_files $prjnum
#       cd $actdir
#       error_msg "Error while building Prefix\n$result_text"
#       return
     }  ;# if Fehler beim Konvertieren
# TODO das wird spaeter auch fuer ina wieder gebraucht
    set prjx(ina.ref) "LL-Net"
    update_project_files $prjnum
    new_project_msg $prjnum "LL Net -> INA finished"
  } else {
    error_msg "LL Net $prjx(actdir)/$prjx(ll_net.filename) does not exist"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_exe_ll_net_ina


proc prj_exe_ll_net_spin {} {
  # Die evtl. vorhandene LL-Netzdatei in eine Datei f"ur SPIN umwandeln.
  global gvar
  global errorCode

    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(ll_net.filename)"] {
      # Konverter von LL-Netz nach SPIN aufrufen
      set extension "promela"
      set prjx(promela.filename) [change_filename_extension $prjx(ll_net.filename) \
      $extension]
    file delete $prjx(promela.filename)
     # Referenzdateien loeschen
    remove_ref_files ll_spin $prjx(project.name)
    set opt [get_options_pep_to_spin]
    set sys_str \
      "exec $gvar(ll_net_to_spin.name) \
      $opt $prjx(ll_net.filename) $prjx(promela.filename)"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set result_text ""
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
      DEBUG "errorCode: $errorCode"
       # Referenz setzen
      set prjx(spin.ref) "None"
    } else {
       # Referenz setzen
      set prjx(spin.ref) "LL-Net"
    } ;# if Fehler beim Konvertieren
    update_project_files $prjnum
    reset_project_promela_text_ana_results $prjnum
    update_project_infos $prjnum
    new_project_msg $prjnum "LL Net -> SPIN finished"
  } else {
    error_msg "LL Net $prjx(actdir)/$prjx(ll_net.filename) does not exists"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_exe_ll_net_spin

proc prj_exe_ll_net_fc2 {} {
  # Die evtl. vorhandene LL-Netzdatei in FC2 umwandeln.
  global gvar
  global errorCode

    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(ll_net.filename)"] {
      # Testen ob die slist da ist
      if [file exists $prjx(slist.filename)] {
        # Ueberpruefen ob slist aelter als ll-net
        set slist_mtime [file mtime $prjx(slist.filename)]
        set ll_net_mtime [file mtime $prjx(ll_net.filename)]
        set create_new_slist [expr $ll_net_mtime>=$slist_mtime]
      } else {
        set create_new_slist 1
      } ;# if slist not exists
      if $create_new_slist {
        # nicht da oder zu alt? -> slist erzeugen
	 # Welche Referenzen sollen verwendet werden?
        if [file exists $prjx(bpn2_ll_ref.filename)] {
	  set ref_mode "-bpn2"
	  set ref_file_name "-r $prjx(bpn2_ll_ref.filename)"
	} elseif [file exists $prjx(exp_bpn2_ll_ref.filename)] {
	  set ref_mode "-bpn2"
	  set ref_file_name "-r $prjx(exp_bpn2_ll_ref.filename)"
	} elseif [file exists $prjx(sdl_ll_ref.filename)] {
	  set ref_mode "-sdl"
	  set ref_file_name "-r $prjx(sdl_ll_ref.filename)"
	} else {
	  set ref_mode "-pfa"
	  set ref_file_name ""
	  #DEBUG "Could not create new slist. No References found!"
	  #cd $actdir
	  #return
	}
        set sys_str "exec $gvar(ll_net_to_slist.name)\
	                  $ref_mode\
	                  $prjx(ll_net.filename)\
			  $ref_file_name\
			  -o $prjx(slist.filename)"
        DEBUG "Aufruf: $sys_str"
        set errorCode "NONE"
        set result_text ""
        set sys_res [catch {eval $sys_str} result_text]
        DEBUG " ergab: $sys_res $result_text"
        if [string compare $errorCode "NONE"] {
          DEBUG "errorCode: $errorCode"
          cd $actdir
          return
        }  ;# if Fehler beim Konvertieren
      } ;# if create_new_slist
      
      # Uebersetzer von LL-Netz nach FC2 aufrufen
      set extension "fc2"
      set prjx(fc2.filename) [change_filename_extension $prjx(ll_net.filename) $extension]
    if [file exists $prjx(fc2.filename)] {
      file delete $prjx(fc2.filename)
      set prjx(fc2.ref) "None"
    }  ;# if Datei da
    set sys_str "exec $gvar(ll_net_to_fc2.name)  $prjx(ll_net.filename)"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set result_text ""
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if [string compare $errorCode "NONE"] {
       # if Fehler beim Konvertieren
      DEBUG "errorCode: $errorCode"
    } else {
      set prjx(fc2.ref) "LL-Net"
    }
    # Analyseergebnisse zur"ucksetzen
    reset_project_fc2_ana_results $prjnum
    new_project_msg $prjnum "LL Net -> FC2 finished"
  } else {
    error_msg "LL Net $prjx(actdir)/$prjx(ll_net.filename) does not exists"
  }  ;# else, Source ist nicht vorhanden
  update_project_infos $prjnum
  update_project_files $prjnum
  cd $actdir
}  ;# prj_exe_ll_net_fc2

proc prj_exe_pfa_bpn2 {} {
  # Falls die pfa-Projektdatei vorhanden ist, compiler anwerfen.
  global gvar
  global errorCode

    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(pfa.filename)"] {
    if [get_options_syntax_check] {
        # kein Syntax Check
      DEBUG "NO Syntax Check"
      set prjx(pfa.syntax_ok) "Skipped"
      update_project_files $prjnum
    } else {  ;# Syntax-Check vornehmen
        # $$$TODO
    }
      #
      # Loeschen der alten Referenzen
      remove_ref_files pfa_bpn2 $prjx(project.name)
      remove_ref_files bpn2 $prjx(project.name)

      # PFA kompilieren
    set opt [get_options_pfa_bpn2]
    set sys_str "exec $gvar(pfa_bpn2.name) \
                      $prjx(pfa.filename) \
		      $prjx(bpn2.normal.filename) \
		      -r $prjx(pfa_bpn2_ref.filename) \
		      $opt"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    set prjx(hl_net.ref) "None"
    set prjx(box_expr.ref) "None"
    set prjx(bpn2.expanded.ref) "None"
    set prjx(c_source.ref) "None"
    if {$sys_res != 0} {
      set prjx(bpn2.normal.ref) "None"
      update_project_files $prjnum
      cd $actdir
      error_msg "Error while compiling PFA to B(PN) program\n$result_text"
      return
    }  ;# if Fehler beim Kompilieren
    set prjx(bpn2.normal.ref) "PFA"
    update_project_files $prjnum
    new_project_msg $prjnum "PFA compiled to B(PN)"
  } else {
    error_msg "PFA $prjx(actdir)/$prjx(pfa.filename) does not exists"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_exe_pfa_bpn2

proc prj_exe_pfa_bpn2_struct {} {
  # Falls die pfa-Projektdatei vorhanden ist, compiler anwerfen.
  global gvar
  global errorCode

    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(pfa.filename)"] {
    if [get_options_syntax_check] {
        # kein Syntax Check
      DEBUG "NO Syntax Check"
      set prjx(pfa.syntax_ok) "Skipped"
      update_project_files $prjnum
    } else {  ;# Syntax-Check vornehmen
        # $$$TODO
    }
      #
      # Loeschen der alten Referenzen
      remove_ref_files bpn2 $prjx(project.name)
      remove_ref_files pfa_bpn2 $prjx(project.name)

      # PFA kompilieren
    set opt [get_options_pfa_bpn2]
    set sys_str "exec $gvar(pfa_bpn2.name) \
      $prjx(pfa.filename) $prjx(bpn2.normal.filename) S"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    set prjx(hl_net.ref) "None"
    set prjx(box_expr.ref) "None"
    set prjx(bpn2.expanded.ref) "None"
    set prjx(c_source.ref) "None"
    if {$sys_res != 0} {
      set prjx(bpn2.normal.ref) "None"
      update_project_files $prjnum
      cd $actdir
      error_msg "Error while compiling PFA to B(PN) program\n$result_text"
      return
    }  ;# if Fehler beim Kompilieren
    set prjx(bpn2.normal.ref) "PFA"
    update_project_files $prjnum
    new_project_msg $prjnum "PFA compiled to B(PN)"
  } else {
    error_msg "PFA $prjx(actdir)/$prjx(pfa.filename) does not exists"
  }  ;# else, Source ist nicht vorhanden
  cd $actdir
}  ;# prj_exe_pfa_bpn2_struct

proc prj_exe_pfa_hl_net {} {
  # Uebersetzung von PFA in HL-Net
  global gvar
  global errorCode
    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(pfa.filename)"] {
    #TODO: Soll der Syntax-check abgefragt werden?
    #    bei prj_exe_pfa_bpn2 eingebaut...
    if [get_options_syntax_check] {
        # kein Syntax Check
      DEBUG "NO Syntax Check"
      set prjx(pfa.syntax_ok) "Skipped"
      update_project_files $prjnum
    } else {  ;# Syntax-Check vornehmen
        #TODO
    }
    # Loeschen der alten Referenzen
    remove_ref_files pfa_hl_net $prjx(project.name)
    remove_ref_files hl_net $prjx(project.name)
    # Optionen setzen
    set opt [get_options_pfa_hl_net]
    set sys_str "exec $gvar(pfa_to_hl_net.name) \
                      $opt \
                      $prjx(pfa.filename) \
                      -o $prjx(hl_net.filename) \
		      -r $prjx(pfa_hl_ref.filename)"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    set prjx(ll_net.ref) "None"
    if {$sys_res != 0} {
      set prjx(hl_net.ref) "None"
      update_project_files $prjnum
      cd $actdir
      error_msg "Error while compiling PFA to HL Net:\n$result_text"
      return
    }  ;# if Fehler beim Kompilieren
    set prjx(hl_net.ref) "PFA"
    update_project_files $prjnum
    new_project_msg $prjnum "PFA compiled to HL Net"
  } else {
    error_msg "PFA File $prjx(actdir)/$prjx(pfa.filename) does not exist!"
  }  ;# if src exists
  cd $actdir
}  ;# prj_exe_pfa_hl_net

proc prj_exe_sdl_hl_net {} {

  # Uebersetzung von PFA in HL-Net
  global gvar
  global errorCode
    # die Projektnummer abholen
  set prjnum $gvar(actprj)
    # der Projectvariable einen Namen geben
  upvar #0 prj$prjnum prjx
    # Verzeichnis merken
  set actdir [pwd]
    # in das passende Verzeichnis wechseln
  cd "$prjx(actdir)"
    # Testen, ob die Source da ist
  if [file exists "$prjx(sdl.filename)"] {
    # Loeschen der alten Referenzen
    remove_ref_files sdl_hl_net $prjx(project.name)
    remove_ref_files hl_net $prjx(project.name)
    # Optionen setzen
    set opt [get_options_sdl_hl_net]
    set sys_str "exec $gvar(sdl_hl_net.name) \
                      $opt \
                      $prjx(sdl.filename) \
                      -o $prjx(hl_net.filename) \
		      -r $prjx(sdl_hl_ref.filename)"
    DEBUG "Aufruf: $sys_str"
    set errorCode "NONE"
    set sys_res [catch {eval $sys_str} result_text]
    DEBUG " ergab: $sys_res $result_text"
    if {$sys_res != 0} {
      if {$errorCode != "NONE" } {
        set prjx(hl_net.ref) "None"
        update_project_files $prjnum
        cd $actdir
        error_msg "Error while compiling SDL to HL Net:\n$result_text"
        return 0
      } else {
        error_msg "Warning while compiling SDL to HL Net:\n$result_text"
      } ;# if Fehler beim Kompilieren
    }
    set prjx(hl_net.ref) "SDL"
    set prjx(ll_net.ref) "None"
    update_project_files $prjnum
    new_project_msg $prjnum "SDL compiled to HL Net"
     # Der SDL Editor sollte ggf. die neuen Referenzen laden
    update_text_editors
  } else {
    error_msg "SDL File $prjx(actdir)/$prjx(sdl.filename) does not exist!"
  }  ;# if src exists
  cd $actdir

} ;# prj_exe_sdl_hl_net

#*** end of file projexe.tcl ***

