# projio.tcl, read/write/etc. projects
# started: 06.02.95, last-update: 06.01.97
# BG integrated Andre's PFA, Ptak's FC2, Peedy's Animator 10.2.98
#TODO: replace move with copy/delete

  ################################
  # Misc procedures for projects #
  ################################

proc well_formated_project_file_name {project_name result_text} {
  # Check, if the given filename is a good project name.
  # If no ".project" postfix append it automatically!
  # project_name - filename to check/change (IN/OUT)
  # result_text  - Errortext, if name is Ok, then = "Ok" (OUT)
  # return       - 1 if name is ok, else 0
  upvar $project_name name
  upvar $result_text res_txt

  set name [string trim $name]
  if {[string length $name] == 0} {
    set res_txt "Empty project name not allowed"
    return 0
  }  ;# bad if empty project name
  if [string match "*.project" $name] {
    # OK, ".project" already appended
  } else {
    set name "$name.project"
  }  ;# else, add postfix
  if [string match "*/*" $name] {
    set res_txt "Project name not allowed (no slashes)!"
    return 0
  }  ;# bad if name contains /
  set parts [split $name .]
  if {[llength $parts] > 2} {
    set res_txt "Project name not allowed (too many points)!"
    return 0
  }  ;# bad if more than one point
  set res_txt "Ok"
  return 1
}  ;# well_formated_project_file_name

proc format_std_file_name {prj_file project_name} {
  # Einen netten Standarddateinamen fuer die betreffende Projektdatei
  # zusammenbasteln.
  # prj_file     - Projektvariablenindex (bpn2.normal.filename oder so)
  # project_name - Name des Projektes (inkl. ".project")
  # return - eben der normale Dateinamen fuer diesen Typ

    # den ersten Teil des Dateinames holen
  set name [lindex [split $project_name .] 0]
    # die Liste explizit durchgehen
  switch -exact $prj_file {
    bpn2.normal.filename
      {return "$name.bpn2"}
    bpn2.expanded.filename
      {return "$name.bpn2_exp"}
    boxexpr.filename
      {return "$name.box_expr"}
    ll_net.filename
      {return "$name.ll_net"}
    hl_net.filename
      {return "$name.hl_net"}
    unfolding.text.filename
      {return "$name.mci"}
    formula.normal.filename
      {return "$name.form"}
    formula.expanded.filename
      {return "$name.form_exp"}
    c_source.filename
      {return "$name.c"}
    doc.filename
      {return "$name.doc"}
    ina.filename
      {return "$name.ina"}
    pfa.filename
      {return "$name.prj"}
    hl_ll_ref.filename
      {return "$name.hl_ll_ref"}
    fc2.filename
      {return "$name.fc2"}
    slist.filename
      {return "$name.slist"}
    bpn2.tex.filename
      {return "$name.bpn2.tex"}
    bpn2_ina_ref.filename
      {return "$name.bpn2_ina_ref"}
    exp_bpn2_hl_ref.filename
      {return "$name.exp_bpn2_hl_ref"}
    exp_bpn2_ll_ref.filename
      {return "$name.exp_bpn2_ll_ref"}
    bpn2_ll_ref.filename
      {return "$name.bpn2_ll_ref"}
    pfa_hl_ref.filename
      {return "$name.pfa_hl_ref"}
    sdl_hl_ref.filename
      {return "$name.sdl_hl_ref"}
    sdl_ll_ref.filename
      {return "$name.sdl_ll_ref"}
    sdl_info.filename
      {return "$name.sdl_info"}
    sdl_properties.filename
      {return "$name.sdl_prop"}
    pfa_bpn2_ref.filename
      {return "$name.pfa_bpn2_ref"}
    c_header.filename
      {return "$name.h"}
    ll_ina_ref.filename
      {return "$name.ll_ina_ref"}
    executable.filename
      {return $name}
    promela.filename
      {return "$name.promela"}
    promela_form.filename
      {return "$name.promela_f"}
    transition.filename
      {return "$name.transition"}
    formula.change.filename
      {return "$name.form_ch"}
    model_check_spin_res.filename
      {return "$name.model_check_spin_res"}
    smv.filename
      {return "$name.smv"}
    sdl.filename
      {return "$name.sdl"}
    simulator.filename
      {return "$name.move"}
    animator.filename
      {return "$name.move"}
    default {
      error_msg "Internal Error: unknown PrjIdx $prj_file"
      return "BLUBB"
    }  ;# default
  }  ;# switch ueber die Indexbezeichner
}  ;# format_std_file_name

proc search_open_project_filename {directory project_filename} {
  # Pruefen, ob das angegebene Projekt schon geoeffnet ist.
  # directory        - Verzeichnis des gesuchten Objektes
  # project_filename - Dateiname des Projektes
  # Liefert eine 1, falls bereits vorhanden, 0 sonst.
  global open_projects

  set name [file join $directory $project_filename]
  if [catch {set open_projects($name)}] {
    return 0
  } else {
    return 1
  }  ;# else, ist vorhanden
}  ;# search_open_project

proc insert_open_project {directory project_filename project_name project_level} {
  # Das Project wird in die Liste eingefuegt.
  # directory        - Verzeichnis fuer das Projekt
  # project_filename - Dateiname des Projektes
  # project_name     - Name des Projektes
  # project_level    - Level des Projektes
  global open_projects

  set name [file join $directory $project_filename]
  set open_projects($name) "$project_name $project_level"
  incr open_projects(open_count)
}  ;# insert_open_project

proc delete_open_project {directory project_filename} {
  # Das Project wird aus der Liste entfernt.
  # directory        - Verzeichnis fuer das Projekt
  # project_filename - Dateiname des Projektes
  global open_projects

  set name [file join $directory $project_filename]
  catch {unset open_projects($name)}
  set open_projects(open_count) [expr $open_projects(open_count)-1]
}  ;# delete_open_project

  ###################
  # Dateiprozeduren #
  ###################

proc read_project {project_name destination} {
  # Read file project_name and use as project file.
  # Put all Values read into array destination.
  global gvar

  # Get destination array.
  upvar $destination prjx
  # Open project file.
  if [catch {set project_file [open $project_name r]}] {
    error_msg "read_project: Could not open file $project_name"
    return -code error
  }
  # Check if first line contains "PEP".
  if {[gets $project_file input_line] < 0} {
    error_msg "read_project: Empty project file $project_name"
    return -code error
  }  ;# if empty file
  set input_line [string trim $input_line]
  if {$input_line != "PEP"} {
    error_msg "read_project: No regular project file $project_name"
    return -code error
  }  ;# if no PEP file
  # The second line should contain the project file identifier,
  # version and date.
  if {[gets $project_file input_line] < 0} {
    error_msg "read_project: Truncated project file $project_name"
    return -code error
  }  ;# if bad file
  set input_line [string trim $input_line]
  scan $input_line "%s %s %s %s %s" a b c d e
  set a [string trim $a]
  set c [string trim $c]
  set e [string trim $e]
  if {$a != "Project-File"} {
    error_msg "read_project: No project file $project_name"
    return -code error
  }  ;# if no project file
  set prjx(VERSION) $c
  set prjx(DATE) $e
  # Read rest of file and store input.
  while {[gets $project_file input_line] >= 0} {
    set input_line [string trim $input_line] ;# Remove trailing whitespace
    if {[string length $input_line] > 0} {
      set input_list [split $input_line " "]
      set a [lindex $input_list 0]
      set b [lrange $input_list 2 end]
      set prjx($a) $b
    }  ;# if something to read
  }  ;# while not eof

  close $project_file
  # Store project's filename
  set prjx(project.filename) $project_name
  set prjx(project.name) [lindex [split $project_name .] 0]
  # Set possibly missing values for old project files
  if [catch {set prjx(project.comment_1)}] {
    set prjx(project.comment_1) "New feature implemented!"
  }  ;
  if [catch {set prjx(project.comment_2)}] {
    set prjx(project.comment_2) "Comments in Projects"
  }  ;
  # compiler issues
  if [catch {set prjx(hl_net.exe.sync_hl_net.all_actions)}] {
    set prjx(hl_net.exe.sync_hl_net.all_actions) 1
  }  ;
  if [catch {set prjx(hl_net.exe.sync_hl_net.include_actions)}] {
    set prjx(hl_net.exe.sync_hl_net.include_actions) {}
  }  ;
  if [catch {set prjx(hl_net.exe.sync_hl_net.exclude_actions)}] {
    set prjx(hl_net.exe.sync_hl_net.exclude_actions) {}
  }  ;
  # analyser issues
  if [catch {set prjx(ll_net.ana.t_system)}] {
    set prjx(ll_net.ana.t_system) "UNKNOWN"
  }  ;
  if [catch {set prjx(ll_net.ana.free_choice)}] {
    set prjx(ll_net.ana.free_choice) "UNKNOWN"
  }  ;
  if [catch {set prjx(ll_net.ana.safe)}] {
    set prjx(ll_net.ana.safe) "UNKNOWN"
  }  ;
  if [catch {set prjx(ll_net.ana.bounded)}] {
    set prjx(ll_net.ana.bounded) "UNKNOWN"
    set prjx(ll_net.ana.bounded.number) 1
    set prjx(ll_net.ana.bounded.markings) " "
    set prjx(ll_net.ana.bounded.arcs) " "
  }  ;
  if [catch {set prjx(ll_net.ana.reachability)}] {
    set prjx(ll_net.ana.reachability) "UNKNOWN"
    set prjx(ll_net.ana.reachability.marking) " "
    set prjx(ll_net.ana.reachability.fire_sequence) " "
  }  ;
  init_lola_defaults
  if [catch {set prjx(ll_net.ana.lp_reachability)}] {
    set prjx(ll_net.ana.lp_reachability) "UNKNOWN"
    set prjx(ll_net.ana.lp_reachability.marked_places) " "
    set prjx(ll_net.ana.lp_reachability.not_marked_places) " "
    set prjx(ll_net.ana.lp_reachability.result_marking) " "
    set prjx(ll_net.ana.lp_reachability.result_sequence) " "
  }  ;
  if [catch {set prjx(ll_net.ana.clp_reachability)}] {
    set prjx(ll_net.ana.clp_reachability) "UNKNOWN"
    set prjx(ll_net.ana.clp_reachability.marked_places) " "
    set prjx(ll_net.ana.clp_reachability.result_sequence) " "
  }  ;
  if [catch {set prjx(ll_net.ana.clp_mutex)}] {
    set prjx(ll_net.ana.clp_mutex) "UNKNOWN"
    set prjx(ll_net.ana.clp_mutex.mutex_places) " "
    set prjx(ll_net.ana.clp_mutex.result_sequence) " "
  }  ;
  if [catch {set prjx(unfolding.text.ana.deadlock_free)}] {
    set prjx(unfolding.text.ana.deadlock_free) "UNKNOWN"
    set prjx(unfolding.text.ana.deadlock_free.events) " "
  }  ;
  if [catch {set prjx(unfolding.ana.mcs_deadlock_free)}] {
    set prjx(unfolding.ana.mcs_deadlock_free) "UNKNOWN"
    set prjx(unfolding.ana.mcs_deadlock_free.result_sequence) " "
  }  ;
  if [catch {set prjx(unfolding.ana.bound_deadlock_free)}] {
    set prjx(unfolding.ana.bound_deadlock_free) "UNKNOWN"
    set prjx(unfolding.ana.bound_deadlock_free.result_sequence) " "
  }  ;
  if [catch {set prjx(ll_net.ana.lp_deadlock_free)}] {
    set prjx(ll_net.ana.lp_deadlock_free) "UNKNOWN"
    set prjx(ll_net.ana.lp_deadlock_free.marking) " "
  }  ;
  if [catch {set prjx(ll_net.ana.clp_deadlock_free)}] {
    set prjx(ll_net.ana.clp_deadlock_free) "UNKNOWN"
    set prjx(ll_net.ana.clp_deadlock_free.result_sequence) " "
  }  ;
  if [catch {set prjx(fc2.ana.deadlock_free)}] {
    set prjx(fc2.ana.deadlock_free) "UNKNOWN"
    set prjx(fc2.ana.deadlock_free.events) " "
  }  ;
  if [catch {set prjx(ll_net.ana.prod_deadlock_free)}] {
    set prjx(ll_net.ana.prod_deadlock_free) "UNKNOWN"
    set prjx(ll_net.ana.prod_deadlock_free.result_sequence) " "
  }  ;
  if [catch {set prjx(ll_net.ana.model_check_t_system)}] {
    set prjx(ll_net.ana.model_check_t_system) "UNKNOWN"
  }  ;
  if [catch {set prjx(ll_net.ana.model_check_boundsmodels)}] {
    set prjx(ll_net.ana.model_check_boundsmodels) "UNKNOWN"
  }  ;
  if [catch {set prjx(unfolding.text.ana.model_check_safe)}] {
    set prjx(unfolding.text.ana.model_check_safe) "UNKNOWN"
    set prjx(unfolding.text.ana.model_check_safe.firing_sequences) " "
  }  ;
  if [catch {set prjx(promela.text.ana.model_check_spin)}] {
    set prjx(promela.text.ana.model_check_spin) "UNKNOWN"
    set prjx(promela.text.ana.model_check_spin.result_sequences) " "
  }  ;
  if [catch {set prjx(unfolding.text.ana.model_check_smv)}] {
    set prjx(unfolding.text.ana.model_check_smv) "UNKNOWN"
  }  ;
  if [catch {set prjx(unfolding.text.ana.model_check_unfsmodels)}] {
    set prjx(unfolding.text.ana.model_check_unfsmodels) "UNKNOWN"
  }  ;
  if [catch {set prjx(unfolding.text.ana.model_check_qq)}] {
    set prjx(unfolding.text.ana.model_check_qq) "UNKNOWN"
  }  ;
  # Set filenames / extensions
  if [catch {set prjx(bpn2.normal.filename)}] {
    set prjx(bpn2.normal.filename) $prjx(project.name).bpn2
  }  ;
  if [catch {set prjx(bpn2.expanded.filename)}] {
    set prjx(bpn2.expanded.filename) $prjx(project.name).bpn2_exp
  }  ;
  if [catch {set prjx(boxexpr.filename)}] {
    set prjx(boxexpr.filename) $prjx(project.name).box_expr
  }  ;
  if [catch {set prjx(ll_net.filename)}] {
    set prjx(ll_net.filename) $prjx(project.name).ll_net
  }  ;
  if [catch {set prjx(hl_net.filename)}] {
    set prjx(hl_net.filename) $prjx(project.name).hl_net
  }  ;
  if [catch {set prjx(doc.filename)}] {
    set prjx(doc.filename) $prjx(project.name).doc
  }  ;
  if [catch {set prjx(unfolding.text.filename)}] {
    set prjx(unfolding.text.filename) $prjx(project.name).mci
  }  ;
  if [catch {set prjx(c_source.filename)}] {
    set prjx(c_source.filename) $prjx(project.name).c
  }  ;
  if [catch {set prjx(c_header.filename)}] {
    set prjx(c_header.filename) $prjx(project.name).h
  }  ;
  if [catch {set prjx(formula.normal.filename)}] {
    set prjx(formula.normal.filename) $prjx(project.name).form
  }  ;
  if [catch {set prjx(formula.expanded.filename)}] {
    set prjx(formula.expanded.filename) $prjx(project.name).form_exp
  }  ;
  if [catch {set prjx(formula.change.filename)}] {
    set prjx(formula.change.filename) $prjx(project.name).form_ch
  }  ;
  if [catch {set prjx(sdl.filename)}] {
    set prjx(sdl.filename) $prjx(project.name).sdl
  }  ;
  if [catch {set prjx(bpn2.tex.filename)}] {
    set prjx(bpn2.tex.filename) $prjx(project.name).bpn2.tex
  }  ;
  if [catch {set prjx(bpn2_ina_ref.filename)}] {
    set prjx(bpn2_ina_ref.filename) $prjx(project.name).bpn2_ina_ref
  }  ;
  if [catch {set prjx(exp_bpn2_hl_ref.filename)}] {
    set prjx(exp_bpn2_hl_ref.filename) $prjx(project.name).exp_bpn2_hl_ref
  }  ;
  if [catch {set prjx(exp_bpn2_ll_ref.filename)}] {
    set prjx(exp_bpn2_ll_ref.filename) $prjx(project.name).exp_bpn2_ll_ref
  }  ;
  if [catch {set prjx(bpn2_ll_ref.filename)}] {
    set prjx(bpn2_ll_ref.filename) $prjx(project.name).bpn2_ll_ref
  }  ;
  if [catch {set prjx(hl_ll_ref.filename)}] {
    set prjx(hl_ll_ref.filename) $prjx(project.name).hl_ll_ref
  }  ;
  if [catch {set prjx(pfa_hl_ref.filename)}] {
    set prjx(pfa_hl_ref.filename) $prjx(project.name).pfa_hl_ref
  }  ;
  if [catch {set prjx(sdl_hl_ref.filename)}] {
    set prjx(sdl_hl_ref.filename) $prjx(project.name).sdl_hl_ref
  }  ;
  if [catch {set prjx(sdl_ll_ref.filename)}] {
    set prjx(sdl_ll_ref.filename) $prjx(project.name).sdl_ll_ref
  }  ;
  if [catch {set prjx(sdl_info.filename)}] {
    set prjx(sdl_info.filename) $prjx(project.name).sdl_info
  }  ;
  if [catch {set prjx(sdl_properties.filename)}] {
    set prjx(sdl_properties.filename) $prjx(project.name).sdl_prop
  }  ;
  if [catch {set prjx(pfa_bpn2_ref.filename)}] {
    set prjx(pfa_bpn2_ref.filename) $prjx(project.name).pfa_bpn2_ref
  }  ;
  if [catch {set prjx(ina.filename)}] {
    set prjx(ina.filename) $prjx(project.name).ina
  }  ;
  if [catch {set prjx(ll_ina_ref.filename)}] {
    set prjx(ll_ina_ref.filename) $prjx(project.name).ll_ina_ref
  }  ;
  if [catch {set prjx(executable.filename)}] {
    set prjx(executable.filename) $prjx(project.name)
  }  ;
  if [catch {set prjx(promela.filename)}] {
    set prjx(promela.filename) $prjx(project.name).promela
  }  ;
  if [catch {set prjx(promela_form.filename)}] {
    set prjx(promela_form.filename) $prjx(project.name).promela_f
  }  ;
  if [catch {set prjx(model_check_spin_res.filename)}] {
    set prjx(model_check_spin_res.filename) $prjx(project.name).model_check_spin_res
  }  ;
  if [catch {set prjx(transition.filename)}] {
    set prjx(transition.filename) $prjx(project.name).transition
  }  ;
  if [catch {set prjx(smv.filename)}] {
    set prjx(smv.filename) $prjx(project.name).smv
  }  ;
  if [catch {set prjx(fc2.filename)}] {
    set prjx(fc2.filename) $prjx(project.name).fc2
  }  ;
  if [catch {set prjx(prod.filename)}] {
    set prjx(prod.filename) $prjx(project.name).prod
  }  ;
  if [catch {set prjx(slist.filename)}] {
    set prjx(slist.filename) $prjx(project.name).slist
  }  ;
  if [catch {set prjx(simulator.filename)}] {
    set prjx(simulator.filename) $prjx(project.name).move
  }  ;
  if [catch {set prjx(animator.filename)}] {
    set prjx(animator.filename) $prjx(project.name).move
  }  ;
  if [catch {set prjx(pfa.filename)}] {
    set prjx(pfa.filename) $prjx(project.name).prj
  } else {
     # there might be still some project files where extension for PFA
     # is pfa, so change that in case
    if {$prjx(pfa.filename)=="$prjx(project.name).pfa"} {
      DEBUG "Changing filename extension from pfa to prj!"
       # Rename file
      if [file exists $prjx(pfa.filename)] {
        file rename $prjx(pfa.filename) $prjx(project.name).prj
      }
       # Now change extension in project variable
      set prjx(pfa.filename) $prjx(project.name).prj
    }
  }
  # Set references to inital value None
  if [catch {set prjx(bpn2.expanded.ref)}] {
    set prjx(bpn2.expanded.ref) "None"
  }  ;
  if [catch {set prjx(boxexpr.ref)}] {
    set prjx(boxexpr.ref) "None"
  }  ;
  if [catch {set prjx(ll_net.ref)}] {
    set prjx(ll_net.ref) "None"
  }  ;
  if [catch {set prjx(unfolding.text.ref)}] {
    set prjx(unfolding.text.ref) "None"
  }  ;
  if [catch {set prjx(hl_net.ref)}] {
    set prjx(hl_net.ref) "None"
  }  ;
  if [catch {set prjx(c_source.ref)}] {
    set prjx(c_source.ref) "None"
  }  ;
  if [catch {set prjx(formula.expanded.ref)}] {
    set prjx(formula.expanded.ref) "None"
  }  ;
  if [catch {set prjx(bpn2.normal.ref)}] {
    set prjx(bpn2.normal.ref) "None"
  }  ;
  if [catch {set prjx(ina.ref)}] {
    set prjx(ina.ref) "None"
  }  ;
  if [catch {set prjx(smv.ref)}] {
    set prjx(smv.ref) "None"
  }  ;
  if [catch {set prjx(spin.ref)}] {
    set prjx(spin.ref) "None"
  }  ;
  if [catch {set prjx(fc2.ref)}] {
    set prjx(fc2.ref) "None"
  }  ;
  if [catch {set prjx(bpn2.normal.syntax_ok)}] {
    set prjx(bpn2.normal.syntax_ok) "Unknown"
  }  ;
  if [catch {set prjx(bpn2.normal.procs)}] {
    set prjx(bpn2.normal.procs) "Unknown"
  }  ;
}  ;# read_project

proc write_project {project_name project} {
  # Die Datei mit dem Namen project_name wird ueberschrieben.
  # Die Angaben werden aus dem Array project geholt.
  global gvar

    # Die Zielliste schnappen.
  upvar $project prj

    # aktuellen Pfad merken
  set actdir [pwd]
    # den Projektpfad setzen
  cd $prj(actdir)
    # Projektdatei oeffnen.
#TODO: Fehler abfangen! Evtl. nicht beschreibbar!
  if {[catch {open $project_name w} project_file]} {
    error_msg "Project file $project_name not writeable.\nChanges skipped!" 
  } else {
    # den vorherigen Pfad wieder herstellen
  cd $actdir
    # Erste Zeile schreiben, Inhalt: "PEP"
  puts $project_file "PEP"
    # Zweite Zeile schreiben, Inhalt: "Project-File Version xy.z Date 31.12.99
  set date_time [clock format [clock seconds] -format "%d.%m.%y %H:%M:%S"]
  set date [lindex $date_time 0]
  puts $project_file \
    "Project-File Version $gvar(project.version) Date $date"
    # Die Liste aller gespeicherten Elementbezeichnungen fuer das Projekt holen.
  set els [array names prj]
  foreach el $els {
    switch -glob $el {
    "*.host" -
    "*.port" -
    "*.sock" -
    "*.server" -
    "*.comstate" -
    "*.widget" -
    "open_dlg_widget.*" -
    "*.filestate_text_1" -
    "*.filestate_text_2" -
    "*.filestate_flag" -
    "DATE" -
    "VERSION" -
    "prjmsg*" -
    "project.level" -
    "project.filename" -
    "actdir" {
	# do nothing
    }
    default {
      puts $project_file "$el = $prj($el)"
    }
    }
  }  ;# die Liste der Elementbezeichnungen durchgehen
  close $project_file
  }
}  ;# write_project

proc copy_project_file {project_old_name project_new_name copy_all_flag} {
  # Die Projektdatei mit dem Namen $project_name kopieren.
  # Die zugehoerigen Dateien werden nur kopiert, wenn
  # copy_all_flag gesetzt ist.
  global gvar

    # Gueltige Dateinamen?
  if {![well_formated_project_file_name project_old_name txt]} {
    error_msg "Old Project Name `$project_old_name`\n  $txt"
    return
  }  ;# if kein guter Name fuer ein altes Projekt
  if {![file exists $project_old_name]} {
    error_msg "The Project File Name `$project_old_name`\n\
      is not a valid Project File Name"
    return
  }  ;# if alte Datei ist nicht vorhanden
  if {![well_formated_project_file_name project_new_name txt]} {
    error_msg "New Project Name `$project_new_name`\n  $txt"
    return
  }  ;# if kein guter Name fuer ein neues Projekt
  if [file exists $project_new_name] {
    error_msg "The Project File Name `$project_new_name` is used"
    return
  }  ;# if neue Datei ist schon vorhanden
    # Ist das Projekt etwa offen?
  if [search_open_project_filename [pwd] $project_old_name] {
    error_msg "The Project `$project_old_name`\n\
      is open, close it before making the copy"
    return
  }  ;# if altes Projekt ist offen
  if [search_open_project_filename [pwd] $project_new_name] {
    error_msg "The Project `$project_new_name`\n\
      is open, close it before making the copy"
    return
  }  ;# if neues Projekt ist offen

    # evtl. Fehlermeldung vorbereiten
  set error_txt " "

    # Projektdatei selber kopieren
    # Das zuerst, denn wenn das nicht geht, ist die ganze Aktion
    # voellig sinnlos, denn die neuen Dateinamen koennen dann nicht
    # in die Projektdatei geschrieben werden.
  if {![file isfile $project_old_name]} {
    set error_txt "$error_txt\n\
      Old Project File `$project_old_name` is not a regular file"
  } elseif {![file owned $project_old_name]} {
    set error_txt "$error_txt\n\
      Old Project File `$project_old_name` is not owned by user"
  } else {
    file copy -force $project_old_name $project_new_name
  }  ;# else, Projektdatei koennte kopiert werden
    # Fehlermeldungen ausgeben
  set error_txt [string trim $error_txt]
  if {[string length $error_txt] > 0} {
    error_msg $error_txt
  }  ;# if Fehlermeldungen vorhanden

    # Sollen auch die Datendateien kopiert werden?
  if {$copy_all_flag} {
      # Projektvariable vorbereiten
    set prj(blubb) "Dummy"
    if {[catch {read_project $project_new_name prj}]} {
      return
    }
    unset prj(blubb)
      # Alle Bezeichner fuer Dateien raussuchen und
      # "project.filename" entfernen.
    set prj_names [array names prj "*.filename"]
    set prj_index [lsearch -exact $prj_names "project.filename"]
    set prj_names [lreplace $prj_names $prj_index $prj_index]

      # 1. Runde: alle Dateinamen durchgehen und pruefen
    set ok_to_copy 1
    foreach prj_file $prj_names {
        # Versuchsweise den neuen Dateinamen eintragen und
        # alte Datei als nicht vorhanden markieren.
      set prj(copy.$prj_file.old_exists) 0
      set prj(copy.$prj_file.new_name) \
        [format_std_file_name $prj_file $project_new_name]
        # alte Datei checken
      if [file exists $prj($prj_file)] {
        if {![file isfile $prj($prj_file)]} {
          set error_txt "$error_txt\n\
            Old File `$prj($prj_file)` is not a regular file"
          set ok_to_copy 0
        } else {
          set prj(copy.$prj_file.old_exists) 1
        }  ;# else, Kopieren sollte moeglich sein
      }  ;# if Datei vorhanden
      if [file exists $prj(copy.$prj_file.new_name)] {
        set error_txt "$error_txt\n\
          New File $prj(copy.$prj_file.new_name) already exists"
        set ok_to_copy 0
      }  ;# if new file exists
    }  ;# foreach Dateibezeichner im Projekt
    if {!$ok_to_copy} {
        # neue Projektdatei entfernen
      file delete $project_new_name
    }  ;# if Konflikte oder Fehler aufgetreten

      # Kopieren sollte jetzt moeglich sein
      # 2. Runde eben jetzt
    foreach prj_file $prj_names {
      if {($prj(copy.$prj_file.old_exists) && $prj(copy.$prj_file.new_name)!="")} {
        file copy -force $prj($prj_file) $prj(copy.$prj_file.new_name)
      }  ;# if alte Datei vorhanden
      set prj($prj_file) $prj(copy.$prj_file.new_name)
      unset prj(copy.$prj_file.new_name)
      unset prj(copy.$prj_file.old_exists)
    }  ;# foreach Dateibezeicher im Projekt

      # 3. Runde: Projektdatei restbearbeiten und schreiben
    set prj(project.filename) $project_new_name
    set prj(project.name) [lindex [split $project_new_name .] 0]
    set prj(actdir) .
    write_project $project_new_name prj
     # Einige Dateien muessen besonders behandelt werden
     #  (Umbenennungen in den Dateien)
     # z.B. PFA
    copy_pfa_files $project_old_name $project_new_name
     # C-Source
    if {[file exists $prj(project.name).c]} {
      change_c_file_content $project_old_name $project_new_name
    }
     # FC2-File
    if {[file exists $prj(project.name).fc2]} {
      change_fc2_file_content $project_old_name $project_new_name
    }
  }  ;# if Datendateien sollen kopiert werden
    # Fehlermeldungen ausgeben
  set error_txt [string trim $error_txt]
  if {[string length $error_txt] > 0} {
    error_msg $error_txt
  }  ;# if Fehlermeldungen vorhanden
}  ;# copy_project_file

proc rename_project_file {project_old_name project_new_name rename_all_flag} {
  # Die Projektdatei mit dem Namen $project_name umbennen.
  # Die zugehoerigen Dateien werden nur umbenannt, wenn
  # rename_all_flag gesetzt ist.
  global gvar

    # Gueltige Dateinamen?
  if {![well_formated_project_file_name project_old_name txt]} {
    error_msg "Old Project Name `$project_old_name`\n  $txt"
    return
  }  ;# if kein guter Name fuer ein altes Projekt
  if {![file exists $project_old_name]} {
    error_msg "The Project File Name `$project_old_name`\n\
      is not a valid Project File Name"
    return
  }  ;# if alte Datei ist nicht vorhanden
  if {![well_formated_project_file_name project_new_name txt]} {
    error_msg "New Project Name `$project_new_name`\n  $txt"
    return
  }  ;# if kein guter Name fuer ein neues Projekt
  if [file exists $project_new_name] {
    error_msg "The Project File Name `$project_new_name` is used"
    return
  }  ;# if neue Datei ist schon vorhanden
    # Ist das Projekt etwa offen?
  if [search_open_project_filename [pwd] $project_old_name] {
    error_msg "The Project `$project_old_name`\n\
      is open, close it before renaming"
    return
  }  ;# if altes Projekt ist offen
  if [search_open_project_filename [pwd] $project_new_name] {
    error_msg "The Project `$project_new_name`\n\
      is open, close it before renaming"
    return
  }  ;# if neues Projekt ist offen

    # evtl. Fehlermeldung vorbereiten
  set error_txt " "

    # Projektdatei selber umbenennen
    # Das zuerst, denn wenn das nicht geht, ist die ganze Aktion
    # voellig sinnlos, denn die neuen Dateinamen koennen dann nicht
    # in die Projektdatei geschrieben werden.
  if {![file isfile $project_old_name]} {
    set error_txt "$error_txt\n\
      Old Project File `$project_old_name` is not a regular file"
  } elseif {![file owned $project_old_name]} {
    set error_txt "$error_txt\n\
      Old Project File `$project_old_name` is not owned by user"
  } else {
    file rename $project_old_name $project_new_name
  }  ;# else, Projektdatei koennte umbenannt werden
    # Fehlermeldungen ausgeben
  set error_txt [string trim $error_txt]
  if {[string length $error_txt] > 0} {
    error_msg $error_txt
  }  ;# if Fehlermeldungen vorhanden

    # Sollen auch die Datendateien umbenannt werden?
  if {$rename_all_flag} {
      # Projektvariable vorbereiten
    set prj(blubb) "Dummy"
    if {[catch {read_project $project_new_name prj}]} {
      return
    }
    unset prj(blubb)
      # Alle Bezeichner fuer Dateien raussuchen und
      # "project.filename" entfernen.
    set prj_names [array names prj "*.filename"]
    set prj_index [lsearch -exact $prj_names "project.filename"]
    set prj_names [lreplace $prj_names $prj_index $prj_index]

      # 1. Runde: alle Dateinamen durchgehen und pruefen
    set ok_to_rename 1
    foreach prj_file $prj_names {
        # Versuchsweise den neuen Dateinamen eintragen und
        # alte Datei als nicht vorhanden markieren.
      set prj(rename.$prj_file.old_exists) 0
      set prj(rename.$prj_file.new_name) \
        [format_std_file_name $prj_file $project_new_name]
        # alte Datei checken
      if [file exists $prj($prj_file)] {
        if {![file isfile $prj($prj_file)]} {
          set error_txt "$error_txt\n\
            Old File `$prj($prj_file)` is not a regular file"
          set ok_to_rename 0
        } elseif {![file owned $prj($prj_file)]} {
          set error_txt "$error_txt\n\
            Old File `$prj($prj_file)` is not owned by user"
          set ok_to_rename 0
        } else {
          set prj(rename.$prj_file.old_exists) 1
        }  ;# else, Umbenennen sollte moeglich sein
      }  ;# if Datei vorhanden
      if [file exists $prj(rename.$prj_file.new_name)] {
        set error_txt "$error_txt\n\
          New File $prj(rename.$prj_file.new_name) already exists"
        set ok_to_rename 0
      }  ;# if new file exists
    }  ;# foreach Dateibezeichner im Projekt
    if {!$ok_to_rename} {
        # neue Projektdatei in alte Projektdatei zurueck
      file rename $project_new_name $project_old_name
    }  ;# if Konflikte oder Fehler aufgetreten

      # Umbenennen sollte jetzt moeglich sein
      # 2. Runde eben jetzt
    foreach prj_file $prj_names {
      if {$prj(rename.$prj_file.old_exists)} {
        file rename $prj($prj_file) $prj(rename.$prj_file.new_name)
      }  ;# if alte Datei vorhanden
      set prj($prj_file) $prj(rename.$prj_file.new_name)
      unset prj(rename.$prj_file.new_name)
      unset prj(rename.$prj_file.old_exists)
    }  ;# foreach Dateibezeicher im Projekt

     # 3. Runde: Projektdatei restbearbeiten und schreiben
    set prj(project.filename) $project_new_name
    set prj(project.name) [lindex [split $project_new_name .] 0]
    set prj(actdir) .
    write_project $project_new_name prj
  }  ;# if Datendateien sollen umbenannt werden
    # Fehlermeldungen ausgeben
  set error_txt [string trim $error_txt]
  if {[string length $error_txt] > 0} {
    error_msg $error_txt
  }  ;# if Fehlermeldungen vorhanden
}  ;# rename_project_file

proc delete_project_file {project_name delete_all_flag} {
  # Die Projektdatei mit dem Namen $project_name loeschen.
  # Die zugehoerigen Dateien werden nur geloescht, wenn
  # delete_all_flag gesetzt ist.
  global gvar

    # Gueltiger Dateiname?
  if {![well_formated_project_file_name project_name txt]} {
    error_msg "Project Name `$project_name`\n  $txt"
    return
  }  ;# if kein guter Name fuer ein Projekt
  if {![file exists $project_name]} {
    error_msg "The Project File Name `$project_name`\n\
      is not a valid Project File Name"
    return
  }  ;# if Datei nicht vorhanden
    # Ist das Projekt etwa offen?
  if [search_open_project_filename [pwd] $project_name] {
    error_msg "The Project `$project_name`\n\
      is open, close it before deletion"
    return
  }  ;# if Projekt ist offen
    # evtl. Fehlermeldung vorbereiten
  set error_txt " "
    # Sollen auch die Datendateien geloescht werden?
  if {$delete_all_flag} {
      # Projektvariable vorbereiten
    set prj(blubb) "Dummy"
    if {[catch {read_project $project_name prj}]} {
      return
    }
      # Alle Bezeichner fuer Dateien raussuchen und
      # "project.filename" entfernen.
    set prj_names [array names prj "*.filename"]
    set prj_index [lsearch -exact $prj_names "project.filename"]
    set prj_names [lreplace $prj_names $prj_index $prj_index]
    foreach prj_file $prj_names {
      if [file exists $prj($prj_file)] {
        if {![file isfile $prj($prj_file)]} {
          set error_txt "$error_txt\n\
            File `$prj($prj_file)` is not a regular file"
        } elseif {![file owned $prj($prj_file)]} {
          set error_txt "$error_txt\n\
            File `$prj($prj_file)` is not owned by user"
        } elseif {![file writable $prj($prj_file)]} {
          set error_txt "$error_txt\n\
            File `$prj($prj_file)` has no write access"
        } else {
            # Loeschen sollte jetzt moeglich sein
          file delete -force $prj($prj_file)
        }  ;# else, Loeschen sollte moeglich sein
      }  ;# if Datei ueberhaupt vorhanden
    }  ;# foreach Dateibezeichner im Projekt
  }  ;# if Datendateien sollen geloescht werden
    # Projektdatei selber killen
  if {![file isfile $project_name]} {
    set error_txt "$error_txt\n\
      File `$project_name` is not a regular file"
  } elseif {![file owned $project_name]} {
    set error_txt "$error_txt\n\
      File `$project_name` is not owned by user"
  } elseif {![file writable $project_name]} {
    set error_txt "$error_txt\n\
      File `$project_name` has no write access"
  } else {
    file delete -force $project_name
  }  ;# else, Projektdatei konnte geloescht werden
    # Fehlermeldungen ausgeben
  set error_txt [string trim $error_txt]
  if {[string length $error_txt] > 0} {
    error_msg $error_txt
  }  ;# if Fehlermeldungen vorhanden
}  ;# delete_project_file

