# scom.tcl, Server communication for pepm.tcl
######################################################

package require Tcl_RPC

###############################################################
proc init_server_com {} {
###############################################################
  #############################
  # Eigenen Server einrichten #
  #############################
  global gvar

  set gvar(server_host) "localhost"
  set gvar(server_sock) [RPC_server 0]
  set gvar(server_port) [lindex [fconfigure $gvar(server_sock) -sockname] 2]
}

###############################################################
proc display_com_state {} {
###############################################################
  # Die gespeicherten Angaben zur Kommunikation werden angezeigt.
  global gvar

  toplevel .com_state
  frame .com_state.f
  pack .com_state.f
  label .com_state.f.l1  -text "Server Host: $gvar(server_host)"
  label .com_state.f.l2  -text "Server Port: $gvar(server_port)"
  label .com_state.f.l3  -text "Server Sock: $gvar(server_sock)"
  pack .com_state.f.l1  .com_state.f.l2  .com_state.f.l3 -side top  -anchor w
  button .com_state.f.ok  -text "OK"  -command {destroy .com_state}
  pack .com_state.f.ok  -side bottom  -fill x
  grab .com_state.f.ok
}

  #################################################
  # Prozeduren, die vom Server aufgerufen werden. #
  #################################################

###############################################################
proc raise_an_editor {prjnum name} {
###############################################################
  # Den betreffenden Editor auf den Bildschirm anweisen sein
  # Fensterlein nach oben zu erheben.
  # prjnum - die Nummer des Projektes
  # name   - Name des Editors (bpn2.normal oder so)
    # Projektvariable bestimmen
  upvar #0 prj$prjnum prjx

  RPC "raise_window_from_server" localhost $prjx($name.port) 
}

  ######################################################
  # Prozeduren, die von den Clients aufgerufen werden. #
  ######################################################

###############################################################
proc init_client_peer {prjnum program hostname port sock} {
###############################################################
  # Den Host und den Port f"ur den Client registrieren.
  # prjnum   - Nummer des zugeh"origen Projektes
  # program  - Programm (B(PN) Normaleditor oder so)
  # hostname - Host des Clients
  # port     - Port-Nr des Clients
  # sock     - Socket-Nr des Clients
    # Projektvariable bestimmen
  upvar #0 prj$prjnum prjx

    # Daten eintragen
  set prjx($program.host) $hostname
  set prjx($program.port) $port
  set prjx($program.sock) $sock
  set prjx($program.comstate) "connected"
  update_project_files $prjnum
}  ;# init_client_peer


proc ping_server {prjnum program} {
  # Der Server wird aufgefordert "alive" zu sagen.
  # prjnum   - Nummer des zugehoerigen Projektes
  # program  - Programm (B(PN) Normaleditor oder so)
  # return   - in jedem Fall "alive"

  DEBUG "PEP Tool Server ist being pinged by client $prjnum $program"
  return "alive"
}  ;# ping_server

proc new_filename_from_client {prjnum program filename} {
  # Vom Client wird ein neuer Dateiname gemeldet.
  # prjnum   - Nummer des zugehoerigen Projektes
  # program  - Programm (B(PN) Normaleditor oder so)
  # filename - neuer Dateiname
    # Projektvariable bestimmen
  upvar #0 prj$prjnum prjx
    # Daten eintragen
  switch $program {
    bpn2.normal     -
    bpn2.expanded   -
    boxexpr         -
    ll_net          -
    hl_net          -
    c_source        -
    formula.normal  -
    doc             -
    pfa             -
    sdl             -
    sdl_properties  -
    animator        -
    simulator       {set prjx($program.filename) $filename}
    help.view \
      {error_msg "new_filename_from_client: \
        $prjnum $program $filename is not allowed"}
    default \
      {error_msg "new_filename_from_client: \
        $prjnum $program $filename is unknown"} 
  }  ;# switch ueber den Programmnamen
}  ;# new_filename_from_client

proc file_changed_from_client {prjnum program} {
  # Vom Client wird eine Ver"anderung an der Datei gemeldet.
  # prjnum   - Nummer des zugeh"origen Projektes
  # program  - Programm (B(PN) Normaleditor oder so)
    # Projektvariable bestimmen
  upvar #0 prj$prjnum prjx

    # Daten eintragen
  switch $program {
    bpn2.normal {
      set prjx(bpn2.normal.syntax_ok) "Unknown"
      set prjx(bpn2.normal.procs) "Unknown"
      set prjx(bpn2.expanded.ref) "None"
      set prjx(boxexpr.ref) "None"
      set prjx(c_source.ref) "None"
      set prjx(bpn2.normal.ref) "None"
      if {$prjx(hl_net.ref) == "B(PN)"} {
        set prjx(hl_net.ref) "None"
      }  ;# if Verweis bestand
      set prjx(c_source.ref) "None"
      # Referenzdateien loeschen
      remove_ref_files bpn2 $prjx(project.name)
    }
    bpn2.expanded {
      set prjx(bpn2.expanded.ref) "None"
      if {$prjx(hl_net.ref) == "Exp-B(PN)"} {
        set prjx(hl_net.ref) "None"
      }  ;# if Verweis bestand
      # Referenzdateien loeschen
      remove_ref_files exp_bpn2 $prjx(project.name)
    }
    boxexpr {
      set prjx(boxexpr.ref) "None"
      if {$prjx(ll_net.ref) == "PBC"} {
        set prjx(ll_net.ref) "None"
      }  ;# if Verweis bestand
      # Referenzdateien loeschen
      remove_ref_files pbc $prjx(project.name)
    }
    ll_net {
      set prjx(ll_net.ref) "None"
      set prjx(unfolding.text.ref) "None"
      set prjx(ina.ref) "None"
      set prjx(spin.ref) "None"
      set prjx(smv.ref) "None"
      set prjx(fc2.ref) "None"
      reset_project_ll_net_ana_results $prjnum
      reset_project_unfolding_text_ana_results $prjnum
      reset_project_promela_text_ana_results $prjnum
      # Referenzdateien loeschen
      remove_ref_files ll_net $prjx(project.name)
    }
    hl_net {
      set prjx(hl_net.ref) "None"
      if {$prjx(ll_net.ref) == "HL-Net"} {
        set prjx(ll_net.ref) "None"
      }  ;# if Verweis bestand
      # Referenzdateien loeschen
      remove_ref_files hl_net $prjx(project.name)
    }
    c_source {
      set prjx(c_source.ref) "None"
    }
    formula.normal {
      set prjx(formula.expanded.ref) "None"
      set prjx(ll_net.ana.model_check_t_system) "UNKNOWN"
      set prjx(unfolding.text.ana.model_check_safe) "UNKNOWN"
      set prjx(promela.text.ana.model_check_spin) "UNKNOWN"
    }
    doc {}
    pfa {
      if {$prjx(bpn2.normal.ref) == "PFA"} {
        set prjx(bpn2.normal.ref) "None"
      }
      if {$prjx(hl_net.ref) == "PFA"} {
        set prjx(hl_net.ref) "None"
      }
      # Referenzdateien loeschen
      remove_ref_files pfa $prjx(project.name)
    }
    sdl {
      if {$prjx(hl_net.ref) == "SDL"} {
        set prjx(hl_net.ref) "None"
      }
      # Referenzdateien loeschen
      remove_ref_files sdl $prjx(project.name)
    }
    sdl_properties {
    }
    help.view {
      error_msg "file_changed_from_client: \
        $prjnum $program $filename is not allowed"
    }
    default {
      error_msg "file_changed_from_client: $prjnum $program is unknown"
    } 
  }  ;# switch "uber den Programmnamen
  update_project_infos $prjnum
  update_project_files $prjnum
}  ;# file_changed_from_client

proc highlight_in_bpn2_from_client {prjnum program ref_str} {
  # Ein Client m"ochte in einem B(PN)-Programm etwas highlighten.
  # Diese Funktion pr"uft zuerst, ob die Dateien zusammengeh"oren
  # und "offnet evtl. den B(PN)-Editor, so denn er bisher nicht
  # ge"offnet war. Anschlie"send wird der Highlight-Aufruf abgeschickt.
  # Die Funktion mu"s per RPC aufgerufen werden.
  # prjnum  - Nummer des zugeh"origen Projektes
  # program - Programmtyp des Clients
  # ref_str - Referenz
  global gvar

    # Projektvariable
  upvar #0 prj$prjnum prjx
    # Die Aktion einleiten
  switch $program {
    ll_net {
        # Projektverweis pr"ufen
      if {[string compare $prjx(ll_net.ref) "HL-Net"]} {
        error_msg "The LL Net is not connected to the HL Net"
        return
      }  ;# if Anzeige nicht m"oglich
      switch $prjx(hl_net.ref) {
        "B(PN)" {
            # Verweise Ok, ist der Editor auf?
          if {$prjx(bpn2.normal.comstate) == "disconnected"} {
              # Editor ist noch nicht auf, also "offnen
            open_an_editor $prjnum "bpn2.normal"
          }  ;# if B(PN)-Editor war noch nicht auf
            # Highlight im B(PN) Editor  m"oglich
          RPC "show_ref_in_bpn2 -ll_net=$ref_str" \
            $prjx(bpn2.normal.host) $prjx(bpn2.normal.port)
	}
        "Exp-B(PN)" {
            # Verweis ist Ok, ist der Editor auf?
          if {$prjx(bpn2.expanded.comstate) == "disconnected"} {
              # Editor ist noch nicht auf, also "offnen
            open_an_editor $prjnum "bpn2.expanded"
          }  ;# if Exp-B(PN)-Editor war noch nicht auf
              # Highlight im Exp-B(PN) Editor  m"oglich
          RPC "show_ref_in_bpn2 -ll_net=$ref_str" \
            $prjx(bpn2.expanded.host) $prjx(bpn2.expanded.port)
	}
	"SDL" {
            # Verweise Ok, ist der Editor auf?
          if {$prjx(sdl.comstate) == "disconnected"} {
              # Editor ist noch nicht auf, also "offnen
            open_an_editor $prjnum "sdl"
          }  ;# if SDL-Editor war noch nicht auf
            # Highlight im SDL Editor  m"oglich
          RPC "show_ref_in_bpn2 -ll_net=$ref_str" \
            $prjx(sdl.host) $prjx(sdl.port)
	}
	default {
          error_msg "The LL Net is not connected to the SDL Program"
          return
	}
      } ;# switch ueber prjx(hl_net.ref)
    }

    hl_net {
       # Projektverweis pruefen
      switch $prjx(hl_net.ref) {
        "B(PN)" {
            # Verweise auf b(pn)^2, ist der Editor auf?
          if {$prjx(bpn2.normal.comstate) == "disconnected"} {
              # Editor ist noch nicht auf, also "offnen
            open_an_editor $prjnum "bpn2.normal"
          }  ;# if B(PN)-Editor war noch nicht auf
            # Highlight im B(PN) Editor  m"oglich
          RPC "show_ref_in_bpn2 -hl_net=$ref_str" \
            $prjx(bpn2.normal.host) $prjx(bpn2.normal.port)
	 }
        "Exp-B(PN)" {
            # Verweis auf exp b(pn)^2, ist der Editor auf?
          if {$prjx(bpn2.expanded.comstate) == "disconnected"} {
              # Editor ist noch nicht auf, also "offnen
            open_an_editor $prjnum "bpn2.expanded" 
          }  ;# if Exp-B(PN)-Editor war noch nicht auf
            # Highlight im Exp-B(PN) Editor  m"oglich
          RPC "show_ref_in_bpn2 -hl_net=$ref_str" \
            $prjx(bpn2.expanded.host) $prjx(bpn2.expanded.port)
	 }
        "SDL" {
            # Verweis auf sdl, ist der Editor auf?
          if {$prjx(sdl.comstate) == "disconnected"} {
              # Editor ist noch nicht auf, also "offnen
            open_an_editor $prjnum "sdl" 
          }  ;# if SDL-Editor war noch nicht auf
            # Highlight im SDL Editor  m"oglich
          RPC "show_ref_in_bpn2 -hl_net=$ref_str" \
            $prjx(sdl.host) $prjx(sdl.port)
         }
        default {
          error_msg "The HL Net is not connected to the B(PN) Program\
	             or to the SDL Program."
	 }
      } ;# switch ueber hl_net.ref
    }

    pfa {
        # Projektverweis pr"ufen
      if {$prjx(bpn2.normal.ref) == "PFA"} {
          # Verweise Ok, ist der Editor auf?
        if {$prjx(bpn2.normal.comstate) == "disconnected"} {
            # Editor ist noch nicht auf, also "offnen
          open_an_editor $prjnum "bpn2.normal"
        }  ;# if B(PN)-Editor war noch nicht auf
          # Highlight im B(PN) Editor m"oglich
          RPC "show_ref_in_bpn2 -pfa=$ref_str" \
            $prjx(bpn2.normal.host) $prjx(bpn2.normal.port)
      } else {
        error_msg "The B(PN) Program is not connected to the PFA"
        return
      }  ;# else, Anzeige nicht m"oglich
    }

    ina {
        # Projektverweis pr"ufen
      if {[string compare $prjx(ll_net.ref) "HL-Net"]} {
        error_msg "The LL Net is not connected to the HL Net"
        return
      }  ;# if Anzeige nicht m"oglich
      if {[string compare $prjx(hl_net.ref) "B(PN)"]} {
        if [string compare $prjx(hl_net.ref) "Exp-B(PN)"] {
          error_msg "The HL Net is not connected to the B(PN) Program"
          return
        }  ;# if Anzeige nicht m"oglich
          # Verweis ist Ok, ist der Editor auf?
        if {$prjx(bpn2.expanded.comstate) == "disconnected"} {
            # Editor ist noch nicht auf, also "offnen
          open_an_editor $prjnum "bpn2.expanded"
        }  ;# if Exp-B(PN)-Editor war noch nicht auf
          # Highlight im Exp-B(PN) Editor  m"oglich
        RPC "show_ref_in_bpn2 -ll_net=$ref_str" \
          $prjx(bpn2.expanded.host) $prjx(bpn2.expanded.port)
      } else {
          # Verweise Ok, ist der Editor auf?
        if {$prjx(bpn2.normal.comstate) == "disconnected"} {
            # Editor ist noch nicht auf, also "offnen
          open_an_editor $prjnum "bpn2.normal"
        }  ;# if B(PN)-Editor war noch nicht auf
          # Highlight im B(PN) Editor  m"oglich
        RPC "show_ref_in_bpn2 -ll_net=$ref_str" \
          $prjx(bpn2.normal.host) $prjx(bpn2.normal.port)
      }  ;# else, HL-Netz ist klar
    }
    default {
      error_msg "Internal Error:\n\
        highlight_in_bpn2_from_client:\n\
        Highlighting only for LL/HL Net, INA or PFA,\n\
        not for $program"
      return 0
    }
  }  ;# switch "uber den Programmtyp
}  ;# highlight_in_bpn2_from_client

proc highlight_reference_from_client {prjnum program args} {
  # Ein B(PN)-Client m"ochte in einem Programm etwas highlighten.
  # Diese Funktion pr"uft zuerst, ob die Dateien zusammengeh"oren
  # und "offnet evtl. den HL/LL-Editor, so denn er bisher nicht
  # ge"offnet war. Anschlie"send wird der Highlight-Aufruf abgeschickt.
  # Die Funktion mu"s per RPC aufgerufen werden.
  # prjnum  - Nummer des zugeh"origen Projektes
  # program - Programmtyp des Clients
  # args - Referenz der Form -ll_net=P1,P2 -hl_net=P3,T5
  global gvar

  # puts "SCOM: highlight_reference_from_client: $prjnum $program $args"

    # Projektvariable
  upvar #0 prj$prjnum prjx
    # Die Aktion einleiten
      foreach arg $args {
        set parlist [split $arg "="]
	puts $parlist
        set destprg [string range [lindex $parlist 0] 1 end]
        set refs [lindex $parlist 1]
        switch $destprg {
          ll_net {
            if [string compare $prjx(ll_net.ref) "HL-Net"] {
              error_msg "The LL Net is not connected to the HL Net"
              return
            }  ;# if Anzeige nicht m"oglich
            switch $prjx(hl_net.ref) {
	     "B(PN)" { }
             "Exp-B(PN)" { }
	     "SDL" { }
	     default {
                error_msg "The LL Net is not connected to the B(PN) or the SDL Program"
                return
	     }
	    }
              # Verweise Ok, ist der Editor auf?
            if {$prjx(ll_net.comstate) == "disconnected"} {
                # Editor ist noch nicht auf, also "offnen
              open_an_editor $prjnum "ll_net"
            }  ;# if LL-Net-Editor war noch nicht auf
              # Highlight im LL-Net-Editor  m"oglich
            RPC "Highlight {$refs}" \
              $prjx(ll_net.host) $prjx(ll_net.port)
          }

          hl_net {
              # Projektverweis pr"ufen
# TODO: geht noch einfacher, da Referenz nur gepr"uft werden mu"s,
# aber jeweils nur der HL-Editor aufgemacht werden mu"s
            switch $prjx(hl_net.ref) {
	     "B(PN)" -
             "Exp-B(PN)" -
             "SDL" {
                  # Verweis ist Ok, ist der Editor auf?
                if {$prjx(hl_net.comstate) == "disconnected"} {
                    # Editor ist noch nicht auf, also "offnen
                  open_an_editor $prjnum "hl_net"
                }  ;# if HL-Net-Editor war noch nicht auf
                   # Highlight im HL-Net-Editor  m"oglich
                RPC "Highlight {$refs}" \
                  $prjx(hl_net.host) $prjx(hl_net.port)
	      }
             default {
                error_msg "The HL Net is not connected to the\
		B(PN), Exp. B(PN) or SDL Program."
                return
	      }
            } ;# switch
          }
	  fo_net {
	  	show_ref_in_formula_from_client $prjnum $program $refs
		# s.u.
	  }
          default {
            error_msg "Internal Error:\n\
              highlight_reference_from_client:\n\
              Highlighting only to LL/HL Net\n\
              and formulae, not to $program"
            return 0
          }
        }  ;# switch "uber den Zielprogrammtyp
      }  ;# foreach arg
}  ;# highlight_reference_from_client

proc highlight_reference_from_ina {prjnum program args} {
   global gvar
 
 # BW: Gesamte Procedure wegen highlighting von INA 
 # im wesentlichen eine Kopie von highlight_reference_from_client
 
 #puts "SCOM: highlight_reference_from_ina: $prjnum $program $args"
 
     # Projektvariable
   upvar #0 prj$prjnum prjx
     # Die Aktion einleiten
       foreach arg $args {
         set parlist [split $arg "="]
       puts $parlist
         set destprg [string range [lindex $parlist 0] 1 end]
         set refs [lindex $parlist 1]
         switch $destprg {
           ll_net {
               if {$prjx(ll_net.comstate) == "disconnected"} {
                   # Editor ist noch nicht auf, also "offnen
                 open_an_editor $prjnum "ll_net"
               }  ;# if LL-Net-Editor war noch nicht auf
                 # Highlight im LL-Net-Editor  moeglich
               RPC "Highlight {$refs}" \
                 $prjx(ll_net.host) $prjx(ll_net.port)
           }
 
           hl_net {
                       # Projektverweis pruefen
             if [string compare $prjx(ll_net.ref) "HL-Net"] {
                 error_msg "The LL Net is not connected to the Hl Net"
                 return
             }  ;# if Anzeige nicht moeglich
                      # Verweis ist Ok, ist der Editor auf?
             if {$prjx(hl_net.comstate) == "disconnected"} {
                 # Editor ist noch nicht auf, also "offnen
                 open_an_editor $prjnum "hl_net"
             }  ;# if HL-Net-Editor war noch nicht auf
                      # Highlight im HL-Net-Editor  m"oglich
             RPC "Highlight {$refs}" \
                 $prjx(hl_net.host) $prjx(hl_net.port)
           }
 
         fo_net {
               show_ref_in_formula_from_client $prjnum $program $refs
               # s.u.
         }
           default {
             error_msg "Internal Error:\n\
               highlight_reference_from_client:\n\
               Highlighting only to LL/HL Net\n\
               not to $program"
             return 0
           }
         }  ;# switch "uber den Zielprogrammtyp
       }  ;# foreach arg
}  ;# highlight_reference_from_ina


proc show_ref_in_formula_from_client {prjnum program ref_str} {
# Da ref2place und ref2trans nicht mehr benoetigt werden, da sowieso
# die Namen uebermittelt werden, habe ich mal eine neue Proc gemacht,
# die die Namen nur weitergibt. Das sicherlich noch vereinfacht werden

  global gvar
  global errorCode

    # Projektvariable
  upvar #0 prj$prjnum prjx
    # Die Aktion einleiten
  switch -glob $program {
    "bpn2*" {
        # Projektverweis pr"ufen
      if [string compare $prjx(ll_net.ref) "HL-Net"] {
        error_msg "The LL Net is not connected to the HL Net"
        return
      } else {
        if [string compare $prjx(hl_net.ref) "B(PN)"] {
          if [string compare $prjx(hl_net.ref) "Exp-B(PN)"] {
            error_msg "The LL Net is not connected to the B(PN) Program"
            return
          } else {
            set ok_flag 1
          }  ;# else, Verweis besteht
        } else {
          set ok_flag 1
        }  ;# else, dieser Verweis besteht
        if {$ok_flag} {
            # in das Projektverzeichnis wechseln
            # Verweis ist Ok, ist der Editor auf?
          if [string compare $prjx(formula.normal.comstate) "connected"] {
              # Editor ist noch nicht auf, also "offnen
            open_an_editor $prjnum "formula.normal"
          }  ;# if Formeleditor war noch nicht auf
            # Einf"ugen im Formeleditor aufrufen
          RDO "insert_string {$ref_str}" \
            $prjx(formula.normal.host) $prjx(formula.normal.port)
        }  ;# if Verweis von LL-Netz zu B(PN) oder Exp-B(PN) besteht
      }  ;# else, Netze sind verbunden
    }
    "sdl" {
        # Projektverweis pr"ufen
      if {$prjx(ll_net.ref)!="HL-Net"} {
        error_msg "The LL Net is not connected to the HL Net"
        return
      } else {
        if {$prjx(hl_net.ref)=="SDL"} {
            # in das Projektverzeichnis wechseln
            # Verweis ist Ok, ist der Editor auf?
          if {$prjx(formula.normal.comstate)!="connected"} {
              # Editor ist noch nicht auf, also "offnen
            open_an_editor $prjnum "formula.normal"
          }  ;# if Formeleditor war noch nicht auf
            # Einf"ugen im Formeleditor aufrufen
          RDO "insert_string {$ref_str}" \
            $prjx(formula.normal.host) $prjx(formula.normal.port)
        } else {
          error_msg "The HL Net is not connected to the SDL Program"
          return
	};# if Verweis von HL-Netz zu SDL besteht
      }  ;# else, Netze sind verbunden
    }
    default {
      error_msg "Internal Error:\n\
        show_ref_in_formula_from_client:\n\
        Show reference only for B(PN) or\n\
        Expanded B(PN) and SDL, not for $program"
    }
  }  ;# switch "uber den Programmtyp
}  ;# show_ref_in_formula_from_client

proc ll2hl {filename string} {
  #string - List of Transitions from ll net
  # returns corr. list of transitions from hl net

  if {$string==""} {
    return ""
  }

  set ref_filename \
    [change_filename_extension $filename hl_ll_ref]

  #read references
  set allrefs [cat $ref_filename]
  #strip prefix up to transition part
  set end [expr [string first PTR $allrefs] -1]
  if {$end==-2} {
    set end end
  }
  set allrefs [string range $allrefs [expr 3+[string first {TR} $allrefs]] $end]
  #remove whitespace and replace " by space forming a list
  regsub -all "\n|\t| " $allrefs {} allrefs
  regsub -all "\"" $allrefs { } allrefs
  # place list into array
  array set refarray [split [string trim $allrefs]]
  
  set simlist [split $string ,]
  foreach trans $simlist {
    if {$trans != "tic"} {
      lappend newlist $refarray($trans)
    } else {
      lappend newlist "tic"
    }
  }
  return [join $newlist ,]
} 

proc send_sim {prjnum args} {
  # prjnum  - Nummer des zugeh"origen Projektes
  # args - Referenz der Form -dest=bpn2 -sim=T11,T12 -loop=T10,T19
  global gvar

  # Projektvariable
  upvar #0 prj$prjnum prjx

  regsub -all -- {-} $args {} args 

  foreach arg $args {
    set parlist [split $arg "="]
    set [lindex $parlist 0] [lindex $parlist 1]
  }
  if {$dest == "ll_net"} {
      if {$prjx(ll_net.comstate) == "disconnected"} {
          open_an_editor $prjnum "ll_net"
      } 
      RPC "FireTransitionLists -sequence=$sim -loop=$loop" \
          $prjx(ll_net.host) $prjx(ll_net.port)

  } elseif {$dest == "hl_net"} {
      if {$prjx(hl_net.comstate) == "disconnected"} {
          open_an_editor $prjnum "hl_net"
      } 
      #translate ll to hl nodes
      set sim [ll2hl $prjx(ll_net.filename) $sim] 
      set loop [ll2hl $prjx(ll_net.filename) $loop] 
      RPC "FireTransitionLists -sequence=$sim -loop=$loop" \
          $prjx(hl_net.host) $prjx(hl_net.port)

  } elseif {$dest == "exp_bpn2"} {
      if {$prjx(bpn2.expanded.comstate) == "disconnected"} {
          open_an_editor $prjnum "bpn2.expanded"
      }
       RPC "simulate_trans -sequence=$sim -loop=$loop" \
           $prjx(bpn2.expanded.host) $prjx(bpn2.expanded.port)

  } elseif {$dest == "bpn2"} {
      if {$prjx(bpn2.normal.comstate) == "disconnected"} {
          open_an_editor $prjnum "bpn2.normal"
      }
       RPC "simulate_trans -sequence=$sim -loop=$loop" \
           $prjx(bpn2.normal.host) $prjx(bpn2.normal.port)
  } elseif {$dest == "sdl"} {
      if {$prjx(sdl.comstate) == "disconnected"} {
          open_an_editor $prjnum "sdl"
      }
       RPC "simulate_trans -sequence=$sim -loop=$loop" \
           $prjx(sdl.host) $prjx(sdl.port)
  }
} 

proc goodbye_from_client {prjnum program} {
  # Ein Client teilt sein ableben mit.
  # prjnum   - Nummer des zugehoerigen Projektes
  # program  - Programm (B(PN) Normaleditor oder so)
  upvar #0 prj$prjnum prjx
    # Daten eintragen
  set prjx($program.comstate) "disconnected"
  switch $program {
    bpn2.normal {
      new_project_msg $prjnum "B(PN) Editor closed"
      .prjtoplvl$prjnum.mbar.edit.menu.open entryconfigure "B(PN)" \
        -state normal
    }
    bpn2.expanded {
      new_project_msg $prjnum "Expanded B(PN) Editor closed"
      .prjtoplvl$prjnum.mbar.edit.menu.open entryconfigure "Expanded B(PN)" \
        -state normal
    }
    boxexpr {
      new_project_msg $prjnum "PBC Editor closed"
      .prjtoplvl$prjnum.mbar.edit.menu.open entryconfigure "PBC" \
        -state normal
    }
    ll_net {
      new_project_msg $prjnum "LL Net Editor closed"
      # Simulator
      send_close_to_simulator $prjnum $program

      .prjtoplvl$prjnum.mbar.edit.menu.open entryconfigure "LL Net" \
        -state normal
    }
    hl_net {
      new_project_msg $prjnum "HL Net Editor closed"
      .prjtoplvl$prjnum.mbar.edit.menu.open entryconfigure "HL Net" \
        -state normal
    }
    c_source {
      new_project_msg $prjnum "C Source Editor closed"
      .prjtoplvl$prjnum.mbar.edit.menu.open entryconfigure "C Source" \
        -state normal
    }
    formula.normal {
      new_project_msg $prjnum "Formula Editor closed"
      .prjtoplvl$prjnum.mbar.edit.menu.open entryconfigure "Formula" \
        -state normal
    }
    doc {
      new_project_msg $prjnum "Document Editor closed"
      .prjtoplvl$prjnum.mbar.edit.menu.open entryconfigure "Document" \
        -state normal
    }
    ina {
      new_project_msg $prjnum "INA closed"
      .prjtoplvl$prjnum.mbar.edit.menu.open entryconfigure "INA" \
        -state normal
    }
    pfa {
      new_project_msg $prjnum "PFA Editor closed"
      .prjtoplvl$prjnum.mbar.edit.menu.open entryconfigure "PFA" \
        -state normal
    }
    help.view {
      new_project_msg $prjnum "Help Viewer closed"
      .prjtoplvl$prjnum.mbar.help.menu entryconfigure "Help Viewer" \
        -state normal
    }
    simulator {
      new_project_msg $prjnum "Simulator closed"
      .prjtoplvl$prjnum.mbar.edit.menu.open entryconfigure "Simulator" \
        -state normal
    }
    animator {
      new_project_msg $prjnum "Animator closed"
      .prjtoplvl$prjnum.mbar.edit.menu.open entryconfigure "Animator" \
        -state normal
    }
    sdl {
      new_project_msg $prjnum "SDL Editor closed"
      .prjtoplvl$prjnum.mbar.edit.menu.open entryconfigure "SDL" \
        -state normal
    }
    sdl_properties {
      new_project_msg $prjnum "SDL Properties Editor closed"
      .prjtoplvl$prjnum.mbar.edit.menu.open entryconfigure "SDL Properties" \
        -state normal
    }
    default \
      {error_msg "goodbye_from_client: \
        $prjnum $program is unknown!"} 
  }  ;# switch "uber den Programmnamen
  update_project_files $prjnum
}  ;# goodbye_from_client


# Simulator Teil
########################################################

proc get_gvar_from_client {prjnum gvar_value} {
  upvar #0 prj$prjnum prjx

  if ![catch {set value $prjx($gvar_value)} ret] {
         return $value
  } else {
         return 0
  }
}

proc send_open_to_simulator { prjnum editor} {

# Wenn der Simulator geoeffnet ist, wird der Editorname
# uebergeben.

 global gvar

 upvar #0 prj$prjnum prjx

 if {$prjx(simulator.comstate) == "connected"} {
    if {$editor == "ll_net"} {
       RPC "setEditorOpen $editor" $prjx(simulator.host) $prjx(simulator.port)
    }
 }
}

proc send_close_to_simulator { prjnum editor} {

# Wenn der Simulator geoeffnet ist, wird der Editorname
# uebergeben.

 global gvar

 upvar #0 prj$prjnum prjx

 if {$prjx(simulator.comstate) == "connected"} {
    if {$editor == "ll_net"} {
       RPC "setEditorClose $editor" $prjx(simulator.host) $prjx(simulator.port)
    }
 }
}

proc send_to_ll_edit_from_client { prjnum command {par1 ""} {par2 ""}} {

 global gvar

 upvar #0 prj$prjnum prjx

 if {$prjx(ll_net.comstate) == "disconnected"} {
    # Editor ist noch nicht auf, also "offnen
    open_an_editor $prjnum "ll_net"
 }  ;# if LL-Net-Editor war noch nicht auf

 RPC "$command $par1 $par2" $prjx(ll_net.host) $prjx(ll_net.port)

}

proc send_to_simulator_from_client { prjnum command {par1 ""} {par2 ""}} {

 global gvar

 upvar #0 prj$prjnum prjx

 if {$prjx(simulator.comstate) == "connected"} {
    RPC "$command $par1 $par2" $prjx(simulator.host) $prjx(simulator.port)
 }
}

proc get_value_from_ll_edit_from_client { prjnum command } {

 global gvar

 upvar #0 prj$prjnum prjx

 if {$prjx(ll_net.comstate) == "disconnected"} {
    # Editor ist noch nicht auf, also "offnen
    open_an_editor $prjnum "ll_net"
 }  ;# if LL-Net-Editor war noch nicht auf

 set str [RPC "$command" $prjx(ll_net.host) $prjx(ll_net.port)]

 return $str
}

proc open_or_raise_sdl_prop_editor {prjnum} {
  upvar #0 prj$prjnum prjx

  if {$prjx(sdl_properties.comstate) == "disconnected"} {
     # Editor oeffnen...
    open_an_editor $prjnum sdl_properties
  }
}
 
  #*** end of file scom.tcl ***
