# util.tcl, misc functions and windows for PEP
# started: 06.02.95, last-update: 30.09.95
# BG Integrated Andre's PFA, Ptak FC2 and Peedy Animator 10.2.98
# 30.11.2000, CS: introduced some new file commands

  #########
  # Inits #
  #########

proc init_util {} {
  # do some init
  global gvar

    # current number for error messages shown in a window
  set gvar(errorcount) 0
    # current number for variables beeing watched in a window
  set gvar(watchcount) 0
}  ;# init_util

  ############################
  # Procedures for something #
  ############################

proc print_array name {
  # Print contents of array (cf.p.89).
  upvar $name a
  set count 0
  DEBUG "***** Contents of array $name *****"
  foreach el [lsort [array names a]] {
    incr count
    DEBUG "$el = $a($el)"
  }  ;# foreach element of the sorted list
  DEBUG "***** End of content list, $count elements *****"
}  ;# print_array

proc change_filename_extension {filename new_ext} {
  # Delete old extension from filename and return
  # it with new extension.
  # filename - current name
  # new_ext  - new extension without .
  # return   - filename with new extension
  set h [file rootname $filename]
  return "$h.$new_ext"
}  ;# change_filename_extension

proc convert_string_2_text {lb str} {
  # Convert string str into rows of listbox lb.
  # Items are separated by \n or "#". The listbox is
  # emptied before this.
  # lb - Textbox (IN/OUT)
  # str - String containing the text to display (IN)
  regsub -all "#" $str "\n" str_list
  $lb configure -state normal
  $lb delete 1.0 end
  $lb insert 1.0 $str_list
  $lb configure -state disabled
}

proc read_file_2_textwidget {text file {pos end} {delete false}} {
  # Reads the given file (not anymore in blocks of 1000 Byte) 
  # into the text widget text.  text is emptied before.
  # text - Widget for the text (IN/OUT)
  # file - File containing the text (IN)
  set textstate [$text cget -state]
  $text configure -state normal
  if {$delete} {
    $text delete 1.0 end
  }
  $text insert $pos [cat $file]
  $text configure -state $textstate
}  ;# read_file_2_textwidget

proc cat {file} {
  #works like unix cat so just return file contents
  if {[file exists $file]} {
    set fileid [open $file r]
    set file_string [read -nonewline $fileid]
    close $fileid
  } else {
    set file_string ""
  }
  return $file_string
}

proc echo {string {file ""}} {
  #works like unix echo with optional redirection to file
  if {$file!=""} {
    if {[catch {open $file w} fileid]} { 
      return
    }
    puts -nonewline $fileid $string
    close $fileid
  } else {
    puts $string
  }
}

proc sed {pattern replacement infile outfile} {
  #replace pattern with replacement
  #reading from infile and writing to outfile
  set infileid [open $infile r]
  set outfileid [open $outfile w]

  while {![eof $infileid]} {
  regsub $pattern [gets $infileid] $replacement changed_line
  puts $outfileid $changed_line
  }
  close $infileid
  close $outfileid
}

proc pattern_in_file {file pattern {var ""}} {
  # returns 1, if pattern is contained in file
  # else 0. If var is given it contains all
  # lines matching pattern (as grep does)
  if {$var!=""} {
    upvar 1 $var dest
    set dest ""
    set fileid [open $file r]
    while {![eof $fileid]} {
      gets $fileid string
      if {[regexp $pattern $string]} {
	append dest $string "\n"
      }
    }
    close $fileid
    return [string length $dest]
  } else {
  set file_string [cat $file]
  if {[regexp $pattern $file_string]} {
     return 1}
  return 0
  }
}  ;# proc pattern_in_file

proc log {n} {
	set m 1
	set cnt 0
	while {$m<$n} {
		set m [expr $m*2]
		incr cnt
	}
	return $cnt
}

proc check_version {} {

  if {[set tcl_version [info tclversion]] != "8.0"} {
    version_warning_msg $tcl_version
  }
}

proc check_utils {} {
 # Checks for some external tools...
  global gvar env

   # assume that nothing is installed
  set gvar(installed.gcc) "0"
  set gvar(installed.cc)  "0"

  set path [split $env(PATH) :]

  foreach dir $path {
    if {[file exists [file join $dir gcc]]} {
      set gvar(installed.gcc) "1"
    }
    if {[file exists [file join $dir cc]]} {
      set gvar(installed.cc) "1"
    }
  }
} ;# check_utils

# delete reference files
proc remove_ref_files { who project_name} {

  set filelist ""
  switch $who {
    pfa {
	set filelist "$project_name.pfa_bpn2_ref $project_name.pfa_hl_ref"
    }
    bpn2 {
	set filelist "$project_name.pfa_bpn2_ref $project_name.bpn2_ina_ref\
                      $project_name.bpn2_ll_ref $project_name.bpn2_hl_ref"
    }
    exp_bpn2 {
	set filelist "$project_name.exp_bpn2_hl_ref $project_name.exp_bpn2_ll_ref"
    }
    sdl {
        set filelist "$project_name.sdl_hl_ref $project_name.sdl_ll_ref\
                      $project_name.sdl_info"
    }
    hl_net {
	set filelist "$project_name.bpn2_hl_ref $project_name.bpn2_ll_ref\
		      $project_name.hl_ll_ref $project_name.exp_bpn2_hl_ref\
		      $project_name.exp_bpn2_ll_ref\
		      $project_name.bpn2_ina_ref\
		      $project_name.sdl_info\
		      $project_name.pfa_hl_ref\
		      $project_name.sdl_ll_ref $project_name.sdl_hl_ref"
    } 
    pbc {
    }
    ll_net {
	set filelist "$project_name.bpn2_ll_ref $project_name.hl_ll_ref\
		       $project_name.ll_ina_ref $project_name.exp_bpn2_ll_ref\
		       $project_name.ll_spin_ref\
		       $project_name.bpn2_ina_ref\
		       $project_name.fc2 $project_name.smv\
		       $project_name.cpt $project_name.pnt $project_name.tim\
		       $project_name.promela $project_name.promela_f\
		       $project_name.form_promela $project_name.promela_f_trail\
		       $project_name.slist\
		       $project_name.mci $project_name.sdl_ll_ref\
		       $project_name.ll_net.mps $project_name.ll_net.lp"
    }
    pfa_bpn2 {
	set filelist "$project_name.bpn2_mv_ref"
    }
    pfa_hl_net {
    }
    sdl_hl_net {
    }
    bpn2_hl_net {
    }
    bpn2_exp_bpn2 {
    }
    exp_bpn2_hl_net {
    }
    bpn2_pbc {
    }
    pbc2_ll_net {
    }
    hl2ll {
    } 
    minimize_hl {
#Do NOT delete all of HL!!!
    }
    minimize_ll {
#Do NOT delete all of LL!!!
	set filelist "$project_name.bpn2_ina_ref $project_name.ll_ina_ref\
		       $project_name.ll_spin_ref\
		       $project_name.fc2 $project_name.smv\
		       $project_name.cpt $project_name.pnt $project_name.tim\
		       $project_name.promela $project_name.promela_f\
		       $project_name.form_promela $project_name.promela_f_trail\
		       $project_name.slist\
		       $project_name.ll_net.mps $project_name.ll_net.lp"
    }
    ll_spin {
        set filelist "$project_name.ll_spin_ref\
		       $project_name.promela $project_name.promela_f\
		       $project_name.form_promela $project_name.promela_f_trail"
    }
    ll_ina {
        set filelist "$project_name.ll_ina_ref\
		       $project_name.cpt $project_name.pnt\
		       $project_name.bpn2_ina_ref"
    }
  }

   DEBUG "Try to remove: $filelist"

   foreach file $filelist {
	if [file exists $file] {
		DEBUG "remove: $file"
		file delete -force $file
	}
   }
}

# Check references

proc get_hl_ref_ext { project_name } {

	# Check connections
	if [file exists $project_name.bpn2_hl_ref] {
    		set res_str "bpn2_hl_ref"
	} elseif [file exists $project_name.exp_bpn2_hl_ref] {
    		set res_str "exp_bpn2_hl_ref"
	} else {
    		set res_str "NONE"
	}

  return $res_str
}

proc get_ll_ref_ext { project_name } {

	# Check connections
	if [file exists $project_name.bpn2_ll_ref] {
    		set res_str "bpn2_ll_ref"
	} elseif [file exists $project_name.exp_bpn2_ll_ref] {
    		set res_str "exp_bpn2_ll_ref"
	} else {
    		set res_str "NONE"
	}
  return $res_str
}

proc get_ref_ext { project_name } {

	# Check connections
	if [file exists $project_name.bpn2_hl_ref] {
    		set res_str "bpn2"
	} elseif [file exists $project_name.exp_bpn2_hl_ref] {
    		set res_str "exp_bpn2"
	} else {
    		set res_str "NONE"
	}
  return $res_str
}

proc exists_ll_to_bpn2 { } {
  global gvar

  set prjnum $gvar(actprj)
  upvar #0 prj$prjnum prjx

  set ll_ref [get_ll_ref_ext $prjx(project.name)]

  DEBUG "exists_ll_to_bpn2: $ll_ref"

  if {$ll_ref == "bpn2_ll_ref"} {
  	return 1
  } else {
  	return 0
  }
}

proc exists_ll_to_exp_bpn2 { } {
  global gvar

  set prjnum $gvar(actprj)
  upvar #0 prj$prjnum prjx

  set ll_ref [get_ll_ref_ext $prjx(project.name)]
  DEBUG "exists_ll_to_exp_bpn2: $ll_ref"

  if {$ll_ref == "exp_bpn2_ll_ref"} {
  	return 1
  } else {
  	return 0
  }
}


# Update Reference files for B(PN)2 Editors
proc update_text_editors { } {
  
  global gvar

  # get project number
  set prjnum $gvar(actprj)
  # Name Project variable
  upvar #0 prj$prjnum prjx

  set ll_ref [get_ll_ref_ext $prjx(project.name)]
  set hl_ref [get_hl_ref_ext $prjx(project.name)]

  if {$prjx(bpn2.normal.comstate) == "connected"} {

#BG Attention: Only use read_references
#   not read_references_from ...
#   read_references is in InOut.tcl

   if {$ll_ref == "bpn2_ll_ref"} {
        catch {RPC "read_references" \
            $prjx(bpn2.normal.host) $prjx(bpn2.normal.port)}
   } else {
        catch {RPC "delete_reference_ll" \
            $prjx(bpn2.normal.host) $prjx(bpn2.normal.port)}
        catch {RPC "enter_edit_mode" \
            $prjx(bpn2.normal.host) $prjx(bpn2.normal.port)}
   }
   if {$hl_ref == "bpn2_hl_ref"} {
        catch {RPC "read_references" \
            $prjx(bpn2.normal.host) $prjx(bpn2.normal.port)}
   } else {
        catch {RPC "delete_reference_hl" \
            $prjx(bpn2.normal.host) $prjx(bpn2.normal.port)}
        catch {RPC "enter_edit_mode" \
            $prjx(bpn2.normal.host) $prjx(bpn2.normal.port)}
   }
  }

  if {$prjx(bpn2.expanded.comstate) == "connected"} {

   if {$ll_ref == "exp_bpn2_ll_ref"} {
        catch {RPC "read_references" \
            $prjx(bpn2.expanded.host) $prjx(bpn2.expanded.port)}
   } else {
        catch {RPC "delete_reference_ll" \
            $prjx(bpn2.expanded.host) $prjx(bpn2.expanded.port)}
        catch {RPC "enter_edit_mode" \
            $prjx(bpn2.expanded.host) $prjx(bpn2.expanded.port)}
   }
   if {$hl_ref == "exp_bpn2_hl_ref"} {
        catch {RPC "read_references" \
            $prjx(bpn2.expanded.host) $prjx(bpn2.expanded.port)}
   } else {
        catch {RPC "delete_reference_hl" \
            $prjx(bpn2.expanded.host) $prjx(bpn2.expanded.port)}
        catch {RPC "enter_edit_mode" \
            $prjx(bpn2.expanded.host) $prjx(bpn2.expanded.port)}
   }
  }

  if {$prjx(sdl.comstate) == "connected"} {
    if {[file exists $prjx(sdl_hl_ref.filename)]} {
        catch {RPC "read_references" \
            $prjx(sdl.host) $prjx(sdl.port)}
    } else {
        catch {RPC "delete_reference_hl" \
            $prjx(sdl.host) $prjx(sdl.port)}
        catch {RPC "enter_edit_mode" \
            $prjx(sdl.host) $prjx(sdl.port)}
    }
    if {[file exists $prjx(sdl_ll_ref.filename)]} {
        catch {RPC "read_references" \
            $prjx(sdl.host) $prjx(sdl.port)}
    } else {
        catch {RPC "delete_reference_ll" \
            $prjx(sdl.host) $prjx(sdl.port)}
        catch {RPC "enter_edit_mode" \
            $prjx(sdl.host) $prjx(sdl.port)}
    }
  }
  
}


  ##################################
  # Procedures for warning windows #
  ##################################

proc strip_win_num window_name {
  # Returns number of window from window name
    # get first name, should be .xyzNNN
  set a [split $window_name .]
  set b [lindex $a 1]
    # regular Expression for numbers at the end
  regexp {[0-9]+} $b c
  return [set c]
}  ;# strip_win_num

proc not_yet_msg {} {
  # Not-yet-implemented window
  toplevel .box
  wm title .box "Announce"
  frame .box.fr
  label .box.fr.l -text "Not now darling!"
  button .box.fr.b -text "Alright my dear ..." \
    -command {destroy .box}
  pack .box.fr
  pack .box.fr.l -side top
  pack .box.fr.b -side bottom
  grab .box.fr.b
}  ;# not_yet_msg

proc version_warning_msg {} {
  global gvar tcl_version tcl_patchLevel rc

  read_rc_file
  if {[catch {set rc(version_warn)}]} {
    set rc(version_warn) yes
  }
  if {$rc(version_warn)} {
  toplevel .fr 
  wm title .fr "Please note!"

  label .fr.l -text " Welcome to $gvar(pep_version) \n\n\
 PEP needs Tcl/Tk 8.4 or higher; Version $tcl_patchLevel was found on your system.\n\
 Please have a look at our webpage for latest infos and versions:\n\n\
 http://peptool.sourceforge.net \n" -justify left
  button .fr.b -text "Ok" \
    -command {destroy .fr}
  pack .fr.l -side top
  pack .fr.b -side bottom
  update idletasks
  set rc(version_warn) no
  save_preferences silent
  }
  after 10000 {destroy .fr}
}  ;# version_warning_msg

# This procedure switches between the needed input areas
# of lp-Reachability. This should be later on be
# possible for different windows so call with list of
# active and inactive widgets...
proc switch_lp { tl } {

 global gvar

 if {$gvar(option.dlg.ana_lp.use_invariants)} {
	$tl.input.marked_label configure -state disabled
	$tl.input.marked_example configure -state disabled
	$tl.input.marked configure -state disabled
	$tl.input.not_marked_label configure -state disabled
	$tl.input.not_marked_example configure -state disabled
	$tl.input.not_marked configure -state disabled

	$tl.extended.marked_example configure -state normal
	$tl.extended.marked configure -state normal
 } else {
	$tl.input.marked_label configure -state normal
	$tl.input.marked_example configure -state normal
	$tl.input.marked configure -state normal
	$tl.input.not_marked_label configure -state normal
	$tl.input.not_marked_example configure -state normal
	$tl.input.not_marked configure -state normal

	$tl.extended.marked_example configure -state disabled
	$tl.extended.marked configure -state disabled
 }
}

proc switch_clp { tl } {
 
  global gvar
 
  if {$gvar(option.dlg.ana_clp.mode)==2} {
       $tl.input.marked_label configure -state disabled
       $tl.input.marked_example configure -state disabled
       $tl.input.marked configure -state disabled
 
       $tl.extended.marked_label configure -state normal
       $tl.extended.marked_example configure -state normal
       $tl.extended.marked_example2 configure -state normal
       $tl.extended.textfr.text configure -state normal
  } else {
       $tl.input.marked_label configure -state normal
       $tl.input.marked_example configure -state normal
       $tl.input.marked configure -state normal
 
       $tl.extended.marked_label configure -state disabled
       $tl.extended.marked_example configure -state disabled
       $tl.extended.marked_example2 configure -state disabled
       $tl.extended.textfr.text configure -state disabled
  }
}

proc switch_inv {d} {
#BG turn to enabled when slist is ok.

 global gvar

 if {$gvar(option.dlg.model_check_smv.invariants_check)} {
	$d.options.general.switch8 configure -state normal
	$d.options.general.switch9 configure -state normal
	$d.options.general.switch10 configure -state normal
 } else {
	$d.options.general.switch8 configure -state disabled
	$d.options.general.switch9 configure -state disabled
	$d.options.general.switch10 configure -state disabled
 }
}

proc error_msg error_text {
  # Error window with text from global variable $error_text
  global gvar

  incr gvar(errorcount)
    # New TopLevel for this message
  set t [toplevel .error_msg_box$gvar(errorcount)]
  wm title $t "PEPTool Error Message"

  frame $t.fr
  pack $t.fr  -side top  -fill both  -expand 1

  label $t.fr.bi  -bitmap error  -bd 10
  pack $t.fr.bi  -side left

  text $t.fr.text  -font fixed  -width 50  -height 10 \
    -yscrollcommand "$t.fr.s set"
  $t.fr.text insert end $error_text
  $t.fr.text configure  -state disabled
  scrollbar $t.fr.s  -orient vertical \
    -command "$t.fr.text yview"
  pack $t.fr.text  -side left  -fill both  -expand 1
  pack $t.fr.s  -side left  -fill y

  frame $t.m  -relief raised  -bd 2
  pack $t.m  -side bottom  -fill x
  button $t.m.ok  -text "OK" \
    -command {destroy .error_msg_box$gvar(acterrormsg)}
  pack $t.m.ok
  bind $t.m.ok <Enter> {+
    set gvar(acterrormsg) [strip_win_num %W]
  }  ;# new Binding for button on Enter
}  ;# error_msg

proc watch name {
  # Display contents of variable (cf.p.178).
  # Changed for output in many windows.
  global gvar

  incr gvar(watchcount)
    # new TopLevel for Watch
  toplevel .watch$gvar(watchcount)
  label .watch$gvar(watchcount).label  -text "Contents of \"$name\": "
  label .watch$gvar(watchcount).value  -textvariable $name
  pack .watch$gvar(watchcount).label \
    .watch$gvar(watchcount).value \
    -side left
}  ;# watch


  ################################################## 
  # Procedures for generation of simulation string # 
  ##################################################

proc simulate {w dest} {

  global gvar

  set ok 0;

  set prjnum $gvar(actprj)
  upvar #0 prj$prjnum prjx

  set sim_str ""
  set loop_str ""
  set target_str ""

  # Check of widget

  if {[winfo class $w] == "Text"} {

     # Search for simulation string
     set sim_row [lindex [split [$w search "_SEQUENCE" 1.0] "."] 0]
     set loop_row [lindex [split [$w search "_LOOP:" 1.0] "."] 0]
     # Simulation string is in the next line
     if {$sim_row != ""} {
        incr sim_row
        set sim_str [$w get $sim_row.0 $sim_row.end]
     }
     if {$loop_row != ""} {
        incr loop_row
        set loop_str [$w get $loop_row.0 $loop_row.end]
     }
  } else {
     # Ok, we assume it is a listbox

     set box_list [$w get 0 end]
     set box_list_target [lsearch $box_list "_SEQUENCE"]
     incr box_list_target

     if {!$box_list_target} {
        return
     }

     while {!$ok} {
	set target_str [string trim [$w get $box_list_target]]
	set str_length [string length $target_str]
	incr str_length -1
	set last_char [string range $target_str $str_length $str_length]
	DEBUG $target_str
	DEBUG $last_char
        set sim_str $sim_str$target_str
       
	if {$last_char != ","} {
		set ok 1
        } else {
		incr box_list_target
	}
     }     

     regsub -all { } $sim_str "" sim_str
     regsub -all {\}} $sim_str "" sim_str
     regsub -all {\{} $sim_str "" sim_str
  }

  if {$sim_str != "" || $loop_str != ""} {
	DEBUG "Call: send_sim $prjnum -dest=$dest -sim=$sim_str -loop=$loop_str"
	send_sim $prjnum -dest=$dest -sim=$sim_str -loop=$loop_str
  }
}

###########################
## Load and save options ##
###########################

proc preferences {} {

  global rc gvar
 
  toplevel .preferences -class Toplevel
  wm geometry .preferences 46x24
  wm grid .preferences 20 10 7 17
  wm maxsize .preferences 46 24
  wm minsize .preferences 46 24
  wm overrideredirect .preferences 0
  wm resizable .preferences 1 1
  wm deiconify .preferences
  wm title .preferences "Preferences"
  frame .preferences.listframe \
    -borderwidth 2 -height 75 -relief groove -width 125 
  frame .preferences.listframe.left \
    -borderwidth 1 -height 30 -relief raised -width 30 
  listbox .preferences.listframe.left.01 \
    -font -Adobe-Helvetica-Medium-R-Normal-*-*-120-*-*-*-*-*-* -setgrid 1 \
    -xscrollcommand {.preferences.listframe.left.02 set} \
    -yscrollcommand {.preferences.listframe.left.03 set} 
  scrollbar .preferences.listframe.left.02 \
    -borderwidth 1 -command {.preferences.listframe.left.01 xview} -orient horiz \
    -width 10 
  scrollbar .preferences.listframe.left.03 \
    -borderwidth 1 -command {.preferences.listframe.left.01 yview} -orient vert \
    -width 10 
  frame .preferences.listframe.right \
    -borderwidth 1 -height 30 -relief raised -width 30 
  listbox .preferences.listframe.right.01 \
    -font -Adobe-Helvetica-Medium-R-Normal-*-*-120-*-*-*-*-*-* -setgrid 1 \
    -xscrollcommand {.preferences.listframe.right.02 set} \
    -yscrollcommand {.preferences.listframe.right.03 set} 
  scrollbar .preferences.listframe.right.02 \
    -borderwidth 1 -command {.preferences.listframe.right.01 xview} -orient horiz \
    -width 10 
  scrollbar .preferences.listframe.right.03 \
    -borderwidth 1 -command {.preferences.listframe.right.01 yview} -orient vert \
    -width 10 
  frame .preferences.buttonframe \
    -borderwidth 2 -height 75 -width 125 -relief groove 
  # note that in command the variable color_palette will get the old
  # value due to variable expansion when generating the button
  # Do not delete the first command - it's really useful (only replace it
  # with a better version ;-)
  button .preferences.buttonframe.cancel \
    -padx 9 -pady 3 -text Cancel  -command "
	set gvar(color_palette) $gvar(color_palette);
	viewSetPalette $gvar(color_palette);
	set_color_palette;
	destroy .preferences"
  button .preferences.buttonframe.apply \
    -padx 9 -pady 3 -text Apply -command set_color_palette
  button .preferences.buttonframe.save \
    -padx 9 -pady 3 -text Save  -command "
	viewSavePalette;
  	save_preferences;
	set_color_palette;
	destroy .preferences"
  frame .preferences.header \
    -borderwidth 2 -height 75 -relief groove -width 175
  label .preferences.header.left \
    -borderwidth 1 -text "All Colors" 
  label .preferences.header.right \
    -borderwidth 1 -text "My Colors"
  button .preferences.toright \
    -padx 9 -pady 3 -text -> -command add_color_entry -state disabled
  button .preferences.toleft \
    -padx 9 -pady 3 -text <- -command delete_color_entry -state disabled

  message .preferences.headline \
    -aspect 375 -justify center -padx 5 -pady 2 -text "Select Colorpalette"
  label .preferences.current -borderwidth 1 -text "Current:"
  label .preferences.cur_color \
    -borderwidth 1 -relief raised -textvariable gvar(color_palette)

  ###################
  # SETTING GEOMETRY
  ###################
  place .preferences.listframe \
    -x 5 -y 88 -width 375 -height 310 -anchor nw -bordermode ignore 
  place .preferences.listframe.left \
    -x 5 -y 5 -width 190 -height 300 -anchor nw -bordermode ignore 
  grid columnconf .preferences.listframe.left 0 -weight 1
  grid rowconf .preferences.listframe.left 0 -weight 1
  grid .preferences.listframe.left.01 \
    -in .preferences.listframe.left -column 0 -row 0 -columnspan 1 -rowspan 1 \
    -sticky nesw 
  grid .preferences.listframe.left.02 \
    -in .preferences.listframe.left -column 0 -row 1 -columnspan 1 -rowspan 1 \
    -sticky ew 
  grid .preferences.listframe.left.03 \
    -in .preferences.listframe.left -column 1 -row 0 -columnspan 1 -rowspan 1 \
    -sticky ns 
  place .preferences.listframe.right \
    -x 195 -y 5 -width 175 -height 300 -anchor nw -bordermode ignore 
  grid columnconf .preferences.listframe.right 0 -weight 1
  grid rowconf .preferences.listframe.right 0 -weight 1
  grid .preferences.listframe.right.01 \
    -in .preferences.listframe.right -column 0 -row 0 -columnspan 1 -rowspan 1 \
    -sticky nesw 
  grid .preferences.listframe.right.02 \
    -in .preferences.listframe.right -column 0 -row 1 -columnspan 1 -rowspan 1 \
    -sticky ew 
  grid .preferences.listframe.right.03 \
    -in .preferences.listframe.right -column 1 -row 0 -columnspan 1 -rowspan 1 \
    -sticky ns 
  place .preferences.buttonframe \
    -x 5 -y 395 -width 375 -height 40 -anchor nw -bordermode ignore 
  place .preferences.buttonframe.cancel \
    -x 20 -y 5 -anchor nw -bordermode ignore 
  place .preferences.buttonframe.apply \
    -x 155 -y 5 -anchor nw -bordermode ignore 
  place .preferences.buttonframe.save \
    -x 285 -y 5 -anchor nw -bordermode ignore 

  place .preferences.header \
    -x 5 -y 60 -width 375 -height 30 -anchor nw -bordermode ignore 
  place .preferences.header.left \
    -x 5 -y 5 -anchor nw -bordermode ignore 
  place .preferences.header.right \
    -x 270 -y 5 -anchor nw -bordermode ignore 
  place .preferences.toright \
    -x 116 -y 63 -width 39 -height 23 -anchor nw -bordermode ignore 
  place .preferences.toleft \
    -x 223 -y 63 -width 39 -height 23 -anchor nw -bordermode ignore 

  place .preferences.headline \
    -x 5 -y 8 -width 375 -height 20 -anchor nw -bordermode ignore 
  place .preferences.current \
    -x 6 -y 32 -width 101 -height 18 -anchor nw -bordermode ignore 
  place .preferences.cur_color \
    -x 115 -y 33 -width 261 -height 18 -anchor nw -bordermode ignore 
#########################################################################
#    $w.colors.list insert 0 gray60 gray70 gray80 gray85 gray90 gray95 \

  bind .preferences.listframe.left.01 <1> {
    .preferences.toright configure -state normal
    .preferences.toleft configure -state disabled
  }
  bind .preferences.listframe.right.01 <1> {
    .preferences.toleft configure -state normal
    .preferences.toright configure -state disabled
  }

  bind .preferences.listframe.left.01 <Double-1> {
    set color [%W get @%x,%y]
    tk_setPalette $color
    set gvar(color_palette) $color
    set rc(color_palette) $color
    viewSetPalette $color
  }
  bind .preferences.listframe.right.01 <Double-1> {
    set color [%W get @%x,%y]
    tk_setPalette $color
    set gvar(color_palette) $color
    set rc(color_palette) $color
    viewSetPalette $color
  }

  if {$rc(my_colors) == ""} {
    set rc(my_colors) "bisque1"
  }

  foreach color_entry $rc(my_colors) {
    .preferences.listframe.right.01 insert end $color_entry
  }

  .preferences.listframe.left.01 insert 0 gray60 gray70 gray80 gray85 gray90 gray95 \
    snow1 snow2 snow3 snow4 seashell1 seashell2 \
    seashell3 seashell4 AntiqueWhite1 AntiqueWhite2 AntiqueWhite3 \
    AntiqueWhite4 bisque1 bisque2 bisque3 bisque4 PeachPuff1 \
    PeachPuff2 PeachPuff3 PeachPuff4 NavajoWhite1 NavajoWhite2 \
    NavajoWhite3 NavajoWhite4 LemonChiffon1 LemonChiffon2 \
    LemonChiffon3 LemonChiffon4 cornsilk1 cornsilk2 cornsilk3 \
    cornsilk4 ivory1 ivory2 ivory3 ivory4 honeydew1 honeydew2 \
    honeydew3 honeydew4 LavenderBlush1 LavenderBlush2 \
    LavenderBlush3 LavenderBlush4 MistyRose1 MistyRose2 \
    MistyRose3 MistyRose4 azure1 azure2 azure3 azure4 \
    SlateBlue1 SlateBlue2 SlateBlue3 SlateBlue4 RoyalBlue1 \
    RoyalBlue2 RoyalBlue3 RoyalBlue4 blue1 blue2 blue3 blue4 \
    DodgerBlue1 DodgerBlue2 DodgerBlue3 DodgerBlue4 SteelBlue1 \
    SteelBlue2 SteelBlue3 SteelBlue4 DeepSkyBlue1 DeepSkyBlue2 \
    DeepSkyBlue3 DeepSkyBlue4 SkyBlue1 SkyBlue2 SkyBlue3 \
    SkyBlue4 LightSkyBlue1 LightSkyBlue2 LightSkyBlue3 \
    LightSkyBlue4 SlateGray1 SlateGray2 SlateGray3 SlateGray4 \
    LightSteelBlue1 LightSteelBlue2 LightSteelBlue3 \
    LightSteelBlue4 LightBlue1 LightBlue2 LightBlue3 \
    LightBlue4 LightCyan1 LightCyan2 LightCyan3 LightCyan4 \
    PaleTurquoise1 PaleTurquoise2 PaleTurquoise3 PaleTurquoise4 \
    CadetBlue1 CadetBlue2 CadetBlue3 CadetBlue4 turquoise1 \
    turquoise2 turquoise3 turquoise4 cyan1 cyan2 cyan3 cyan4 \
    DarkSlateGray1 DarkSlateGray2 DarkSlateGray3 \
    DarkSlateGray4 aquamarine1 aquamarine2 aquamarine3 \
    aquamarine4 DarkSeaGreen1 DarkSeaGreen2 DarkSeaGreen3 \
    DarkSeaGreen4 SeaGreen1 SeaGreen2 SeaGreen3 SeaGreen4 \
    PaleGreen1 PaleGreen2 PaleGreen3 PaleGreen4 SpringGreen1 \
    SpringGreen2 SpringGreen3 SpringGreen4 green1 green2 \
    green3 green4 chartreuse1 chartreuse2 chartreuse3 \
    chartreuse4 OliveDrab1 OliveDrab2 OliveDrab3 OliveDrab4 \
    DarkOliveGreen1 DarkOliveGreen2 DarkOliveGreen3 \
    DarkOliveGreen4 khaki1 khaki2 khaki3 khaki4 \
    LightGoldenrod1 LightGoldenrod2 LightGoldenrod3 \
    LightGoldenrod4 LightYellow1 LightYellow2 LightYellow3 \
    LightYellow4 yellow1 yellow2 yellow3 yellow4 gold1 gold2 \
    gold3 gold4 goldenrod1 goldenrod2 goldenrod3 goldenrod4 \
    DarkGoldenrod1 DarkGoldenrod2 DarkGoldenrod3 DarkGoldenrod4 \
    RosyBrown1 RosyBrown2 RosyBrown3 RosyBrown4 IndianRed1 \
    IndianRed2 IndianRed3 IndianRed4 sienna1 sienna2 sienna3 \
    sienna4 burlywood1 burlywood2 burlywood3 burlywood4 wheat1 \
    wheat2 wheat3 wheat4 tan1 tan2 tan3 tan4 chocolate1 \
    chocolate2 chocolate3 chocolate4 firebrick1 firebrick2 \
    firebrick3 firebrick4 brown1 brown2 brown3 brown4 salmon1 \
    salmon2 salmon3 salmon4 LightSalmon1 LightSalmon2 \
    LightSalmon3 LightSalmon4 orange1 orange2 orange3 orange4 \
    DarkOrange1 DarkOrange2 DarkOrange3 DarkOrange4 coral1 \
    coral2 coral3 coral4 tomato1 tomato2 tomato3 tomato4 \
    OrangeRed1 OrangeRed2 OrangeRed3 OrangeRed4 red1 red2 red3 \
    red4 DeepPink1 DeepPink2 DeepPink3 DeepPink4 HotPink1 \
    HotPink2 HotPink3 HotPink4 pink1 pink2 pink3 pink4 \
    LightPink1 LightPink2 LightPink3 LightPink4 PaleVioletRed1 \
    PaleVioletRed2 PaleVioletRed3 PaleVioletRed4 maroon1 \
    maroon2 maroon3 maroon4 VioletRed1 VioletRed2 VioletRed3 \
    VioletRed4 magenta1 magenta2 magenta3 magenta4 orchid1 \
    orchid2 orchid3 orchid4 plum1 plum2 plum3 plum4 \
    MediumOrchid1 MediumOrchid2 MediumOrchid3 MediumOrchid4 \
    DarkOrchid1 DarkOrchid2 DarkOrchid3 DarkOrchid4 purple1 \
    purple2 purple3 purple4 MediumPurple1 MediumPurple2 \
    MediumPurple3 MediumPurple4 thistle1 thistle2 thistle3 \
    thistle4
}

proc delete_color_entry {} {
 global rc

  set item [.preferences.listframe.right.01 curselection]
  if {$item != ""} {
    .preferences.listframe.right.01 delete $item
    set rc(my_colors) [.preferences.listframe.right.01 get 0 end]
  }
}

proc add_color_entry {} {
 global rc
 
  .preferences.listframe.right.01 insert end [.preferences.listframe.left.01 get anchor]
  set rc(my_colors) [.preferences.listframe.right.01 get 0 end]
}

proc save_preferences {{mode verbose}} {

 global gvar rc env
 
 set rc_string "
##################################################################
#                                                                #
#  .peprc                                                        #
#  -----------                                                   #
#                                                                #
#  Preferences for PEP                                           # 
#                                                                #
##################################################################

##################################################################
# Colors

color_palette = $rc(color_palette)
my_colors = $rc(my_colors)
version_warn = $rc(version_warn)
"

 set rcfile [file join $env(HOME) .peprc]
 if {[file exists $rcfile]} {
      file rename -force $rcfile $rcfile$gvar(bak_extension)
  }

  echo $rc_string $rcfile
  set rc(rcfile) 1

  if {$mode!="silent"} {
    tk_dialog .d { } {Options Saved} warning 0 {Ok}
  }
}

proc save_viewrc_file {prj_num} {
#TODO: comment!!!

  upvar #0 prj$prj_num prjx
  global gvar env viewrc

   # Save window size into project variable
  scan [wm geometry .prjtoplvl$prj_num] "%dx%d%s" width height position
  set prjx(view.window,width) $width
  set prjx(view.window,height) $height

  set viewrc_string "
##################################################################
#                                                                #
#  .pep_viewrc                                                   #
#  -----------                                                   #
#                                                                #
#  Preferences for the view in the Project Frame                 # 
#  of the fabulous PEP Tool                                      #
#                                                                #
##################################################################
"

  set viewrcfile [file join $env(HOME) .pep_viewrc]
  if {[file exists $viewrcfile]} {
      file rename $viewrcfile $viewrcfile$gvar(bak_extension)
  }

  foreach variable [array names prjx view.*] {
    set viewrc_string "$viewrc_string\n$variable = $prjx($variable)"
    set viewrc($variable) $prjx($variable)
  }

  echo $viewrc_string $viewrcfile

  set viewrc(viewrcfile) 1

  tk_dialog .d { } {Saved current Project View as User Default} warning 0 {Ok}

} ;# save_viewrc_file

#
# Read preferences file
#
proc read_rc_file { } {

 global rc gvar env

 # Search for file in HOME and then in PEP directory
 if {[file exists [file join $env(HOME) .peprc]]} {
   set fileid [open $env(HOME)/.peprc]
   set rc(rcfile) 1
 } elseif {[file exists [file join $gvar(pephome.path) .peprc]]} {
   set fileid [open [file join $gvar(pephome.path) .peprc]]
   set rc(rcfile) 1
 }

 if {$rc(rcfile)} {
	while {![eof $fileid]} {
		gets $fileid rcline
		set rcline [string trim $rcline]
		# skip comments
		if {$rcline != "" && [string range $rcline 0 0] != "#"} {
			regexp "=" $rcline delimiter
			set delimiter_pos [string first $delimiter $rcline]
			set rcvar [string range $rcline 0 [expr ( $delimiter_pos - 1 ) ] ]
			set rcvalue [string range $rcline [expr ( $delimiter_pos + 1 )] end]

			set rc([string trim $rcvar]) [string trim $rcvalue]
			#DEBUG "[string trim $rcvar] [string trim $rcvalue]"
		};#end if
	};#end while
 close $fileid
 }
}

proc read_viewrc_file { } {
#TODO: comment!!!

 global viewrc gvar env

 # Search for file in HOME and then in PEP directory
 if {[file exists [file join $env(HOME) .pep_viewrc]]} {
   set fileid [open [file join $env(HOME) .pep_viewrc]]
   set viewrc(viewrcfile) 1
   DEBUG "Loading User Defaults for ProjectView (\$HOME/.pep_viewrc)"
 } elseif {[file exists [file join $gvar(pephome.path) .pep_viewrc]]} {
   set fileid [open [file join $gvar(pephome.path) .pep_viewrc]]
   set viewrc(viewrcfile) 1
   DEBUG "Loading User Defaults for ProjectView (\$PEPHOME/.pep_viewrc)"
 }

 if {$viewrc(viewrcfile)} {
	while {![eof $fileid]} {
		gets $fileid rcline
		set rcline [string trim $rcline]
		# skip comments
		if {$rcline != "" && [string range $rcline 0 0] != "#"} {
			regexp "=" $rcline delimiter
			set delimiter_pos [string first $delimiter $rcline]
			set rcvar [string range $rcline 0 [expr ( $delimiter_pos - 1 ) ] ]
			set rcvalue [string range $rcline [expr ( $delimiter_pos + 1 )] end]

			set viewrc([string trim $rcvar]) [string trim $rcvalue]
			#DEBUG "[string trim $rcvar] [string trim $rcvalue]"
		};#end if
	};#end while
 close $fileid
 } else {
   DEBUG "No User Defaults for ProjectView"
 }
} ;# read_viewrc_file

proc save_view_to_file {prj_num filename} {
#TODO: comment!!!

  upvar #0 prj$prj_num prjx
  global gvar

 if {[file exists $filename]} {
      file rename $filename $filename$gvar(bak_extension)
      DEBUG "File already exists. Old file has been copied to $filename$gvar(bak_extension)!"
  }

  set viewrc_string [cat $filename]
  foreach variable [array names prjx view.*] {
    set viewrc_string "$viewrc_string\n  set gvar(default.$variable) $prjx($variable)"
  }
  echo $viewrc_string $filename

  tk_dialog .d { } "Saved current Project View to file $filename" warning 0 {Ok}
} ;# save_view_to_file

proc GetTclEnvInfoStr {} {
   # tclEnvInfo.tcl
   # Information about current Tcl(/Tk) environment data.
   # by Burkhard Bieber
   # start:       1997-08-28
   # last-change: 1998-05-08

  global tcl_patchLevel tk_patchLevel tcl_platform

  set ev(1) {"Tcl patch level is" \$tcl_patchLevel}
  set ev(2) {"Tk patch level is" \$tk_patchLevel}
  set ev(3) {"Platform name is" \$tcl_platform(platform)}
  set ev(4) {"Machine name is" \$tcl_platform(machine)}
  set ev(5) {"OS name is" \$tcl_platform(os)}
  set ev(6) {"OS version is" \$tcl_platform(osVersion)}
   # collect data if available
  set res {}
  foreach i [lsort -integer [array names ev]] {
    if {[catch {set v [expr [lindex $ev($i) 1]]}] != 0} {
      set v "unavailable"
    }
    set res "$res[lindex $ev($i) 0] $v\n"
  }  ;# foreach env-var to get
   # remove trailing newline
  set l [string length $res]
  if {$l > 0} {
    set res [string range $res 0 [expr $l - 2]]
  }  ;# if res != ""
  return $res
}  ;# GetTclEnvInfoStr

proc text_window {w title contents} {
  if {[catch "toplevel $w"]} {
    #toplevel exists
    raise $w
    return 0
  }
  wm title $w $title
  frame $w.f
  text $w.f.t -height 15 -width 40 -wrap word -yscrollcommand "$w.f.s set" -setgrid 1
  scrollbar $w.f.s -command "$w.f.t yview"
  pack $w.f.s -side right -fill y
  pack $w.f.t -expand yes -fill both
  $w.f.t insert 0.0 $contents

  pack $w.f -fill both -expand 1

  button $w.b -text "Ok" \
    -command "destroy $w; return 0"
  pack $w.b
} ;# text_window

################
## set colors ##
################

proc init_color_palette { } {
  global rc gvar

  if { $rc(rcfile) } {
  	set gvar(color_palette) $rc(color_palette)
  } else {
  	set gvar(color_palette) gray85
  }
  tk_setPalette $gvar(color_palette)
  viewSetPalette $gvar(color_palette)
}

proc set_color_palette { } {
  global gvar

  # send colors to other interps
  for {set x 1} {$x<=$gvar(prjlevel)} {incr x} {
    upvar #0 prj$x prjx
    foreach program [array names prjx -glob "*.comstate"] {
      if {$prjx($program)=="connected"} {
        RPC [list tk_setPalette $gvar(color_palette)] localhost $prjx([string range $program 0 [string last . $program]]port)
      }
    }
  }
  # and do it local
  tk_setPalette $gvar(color_palette)
}

proc viewSetPalette color {
 # set background of images frame_pattern_round and
 # frame_pattern_oval to $color
  global gvar

   # List of {x y} coordinates, where image_frame_pattern_round
   # needs background color
  set point_list { {0 0} \
    {0 1} {0 2} {0 3} {0 4} {0 5} {0 6} {0 7} {0 8} \
    {0 9} {0 10} {0 11} {0 12} {0 13} {0 14} {0 15} {0 16} {0 17} \
    {0 18} {0 19} {0 20} {0 21} {0 22} {0 23} {0 24} {0 25} {0 26} \
    {0 27} {0 28} {0 29} {0 40} {0 41} {0 42} {0 43} {0 44} {0 45} \
    {0 46} {0 47} {0 48} {0 49} {0 50} {0 51} {0 52} {0 53} {0 54} \
    {0 55} {0 56} {0 57} {0 58} {0 59} {0 60} {0 61} {0 62} {0 63} \
 {0 64} {0 65} {0 66} {0 67} {0 68} {0 69} {0 70} {1 0} {1 1} \
 {1 2} {1 3} {1 4} {1 5} {1 6} {1 7} {1 8} {1 9} {1 10} \
 {1 11} {1 12} {1 13} {1 14} {1 15} {1 16} {1 17} {1 18} {1 19} \
 {1 20} {1 21} {1 22} {1 23} {1 24} {1 45} {1 46} {1 47} {1 48} \
 {1 49} {1 50} {1 51} {1 52} {1 53} {1 54} {1 55} {1 56} {1 57} \
 {1 58} {1 59} {1 60} {1 61} {1 62} {1 63} {1 64} {1 65} {1 66} \
 {1 67} {1 68} {1 69} {1 70} {2 0} {2 1} {2 2} {2 3} {2 4} \
 {2 5} {2 6} {2 7} {2 8} {2 9} {2 10} {2 11} {2 12} {2 13} \
 {2 14} {2 15} {2 16} {2 17} {2 18} {2 19} {2 20} {2 21} {2 48} \
 {2 49} {2 50} {2 51} {2 52} {2 53} {2 54} {2 55} {2 56} {2 57} \
 {2 58} {2 59} {2 60} {2 61} {2 62} {2 63} {2 64} {2 65} {2 66} \
 {2 67} {2 68} {2 69} {2 70} {3 0} {3 1} {3 2} {3 3} {3 4} \
 {3 5} {3 6} {3 7} {3 8} {3 9} {3 10} {3 11} {3 12} {3 13} \
 {3 14} {3 15} {3 16} {3 17} {3 18} {3 19} {3 50} {3 51} {3 52} \
 {3 53} {3 54} {3 55} {3 56} {3 57} {3 58} {3 59} {3 60} {3 61} \
 {3 62} {3 63} {3 64} {3 65} {3 66} {3 67} {3 68} {3 69} {3 70} \
 {4 0} {4 1} {4 2} {4 3} {4 4} {4 5} {4 6} {4 7} {4 8} \
 {4 9} {4 10} {4 11} {4 12} {4 13} {4 14} {4 15} {4 16} {4 17} \
 {4 52} {4 53} {4 54} {4 55} {4 56} {4 57} {4 58} {4 59} {4 60} \
 {4 61} {4 62} {4 63} {4 64} {4 65} {4 66} {4 67} {4 68} {4 69} \
 {4 70} {5 0} {5 1} {5 2} {5 3} {5 4} {5 5} {5 6} {5 7} \
 {5 8} {5 9} {5 10} {5 11} {5 12} {5 13} {5 14} {5 15} {5 54} \
 {5 55} {5 56} {5 57} {5 58} {5 59} {5 60} {5 61} {5 62} {5 63} \
 {5 64} {5 65} {5 66} {5 67} {5 68} {5 69} {5 70} {6 0} {6 1} \
 {6 2} {6 3} {6 4} {6 5} {6 6} {6 7} {6 8} {6 9} {6 10} \
 {6 11} {6 12} {6 13} {6 14} {6 55} {6 56} {6 57} {6 58} {6 59} \
 {6 60} {6 61} {6 62} {6 63} {6 64} {6 65} {6 66} {6 67} {6 68} \
 {6 69} {6 70} {7 0} {7 1} {7 2} {7 3} {7 4} {7 5} {7 6} \
 {7 7} {7 8} {7 9} {7 10} {7 11} {7 12} {7 57} {7 58} {7 59} \
 {7 60} {7 61} {7 62} {7 63} {7 64} {7 65} {7 66} {7 67} {7 68} \
 {7 69} {7 70} {8 0} {8 1} {8 2} {8 3} {8 4} {8 5} {8 6} \
 {8 7} {8 8} {8 9} {8 10} {8 11} {8 58} {8 59} {8 60} {8 61} \
 {8 62} {8 63} {8 64} {8 65} {8 66} {8 67} {8 68} {8 69} {8 70} \
 {9 0} {9 1} {9 2} {9 3} {9 4} {9 5} {9 6} {9 7} {9 8} \
 {9 9} {9 10} {9 59} {9 60} {9 61} {9 62} {9 63} {9 64} {9 65} \
 {9 66} {9 67} {9 68} {9 69} {9 70} {10 0} {10 1} {10 2} {10 3} \
 {10 4} {10 5} {10 6} {10 7} {10 8} {10 9} {10 60} {10 61} {10 62} \
 {10 63} {10 64} {10 65} {10 66} {10 67} {10 68} {10 69} {10 70} {11 0} \
 {11 1} {11 2} {11 3} {11 4} {11 5} {11 6} {11 7} {11 8} {11 61} \
 {11 62} {11 63} {11 64} {11 65} {11 66} {11 67} {11 68} {11 69} {11 70} \
 {12 0} {12 1} {12 2} {12 3} {12 4} {12 5} {12 6} {12 7} {12 62} \
 {12 63} {12 64} {12 65} {12 66} {12 67} {12 68} {12 69} {12 70} {13 0} \
 {13 1} {13 2} {13 3} {13 4} {13 5} {13 6} {13 63} {13 64} {13 65} \
 {13 66} {13 67} {13 68} {13 69} {13 70} {14 0} {14 1} {14 2} {14 3} \
 {14 4} {14 5} {14 64} {14 65} {14 66} {14 67} {14 68} {14 69} {14 70} \
 {15 0} {15 1} {15 2} {15 3} {15 4} {15 5} {15 64} {15 65} {15 66} \
 {15 67} {15 68} {15 69} {15 70} {16 0} {16 1} {16 2} {16 3} {16 4} \
 {16 65} {16 66} {16 67} {16 68} {16 69} {16 70} {17 0} {17 1} {17 2} \
 {17 3} {17 66} {17 67} {17 68} {17 69} {17 70} {18 0} {18 1} {18 2} \
 {18 3} {18 66} {18 67} {18 68} {18 69} {18 70} {19 0} {19 1} {19 2} \
 {19 67} {19 68} {19 69} {19 70} {20 0} {20 1} {20 2} {20 67} {20 68} \
 {20 69} {20 70} {21 0} {21 1} {21 68} {21 69} {21 70} {22 0} {22 1} \
 {22 68} {22 69} {22 70} {23 0} {23 68} {23 69} {23 70} {24 0} {24 69} \
 {24 70} {25 0} {25 69} {25 70} {26 0} {26 69} {26 70} {27 70} {28 70} \
    {29 70} {30 70} {31 70} {32 70} {33 70} {52 70} {53 70} {54 70} {55 70} \
    {56 70} {57 70} {58 70} {59 69} {59 70} {60 0} {60 69} {60 70} {61 0} \
    {61 69} {61 70} {62 0} {62 69} {62 70} {63 0} {63 1} {63 68} {63 69} \
    {63 70} {64 0} {64 1} {64 68} {64 69} {64 70} {65 0} {65 1} {65 67} \
    {65 68} {65 69} {65 70} {66 0} {66 1} {66 2} {66 67} {66 68} {66 69} \
 {66 70} {67 0} {67 1} {67 2} {67 66} {67 67} {67 68} {67 69} {67 70} \
 {68 0} {68 1} {68 2} {68 3} {68 66} {68 67} {68 68} {68 69} {68 70} \
 {69 0} {69 1} {69 2} {69 3} {69 4} {69 65} {69 66} {69 67} {69 68} \
 {69 69} {69 70} {70 0} {70 1} {70 2} {70 3} {70 4} {70 65} {70 66} \
 {70 67} {70 68} {70 69} {70 70} {71 0} {71 1} {71 2} {71 3} {71 4} \
 {71 5} {71 64} {71 65} {71 66} {71 67} {71 68} {71 69} {71 70} {72 0} \
 {72 1} {72 2} {72 3} {72 4} {72 5} {72 6} {72 63} {72 64} {72 65} \
 {72 66} {72 67} {72 68} {72 69} {72 70} {73 0} {73 1} {73 2} {73 3} \
 {73 4} {73 5} {73 6} {73 62} {73 63} {73 64} {73 65} {73 66} {73 67} \
 {73 68} {73 69} {73 70} {74 0} {74 1} {74 2} {74 3} {74 4} {74 5} \
 {74 6} {74 7} {74 62} {74 63} {74 64} {74 65} {74 66} {74 67} {74 68} \
 {74 69} {74 70} {75 0} {75 1} {75 2} {75 3} {75 4} {75 5} {75 6} \
 {75 7} {75 8} {75 61} {75 62} {75 63} {75 64} {75 65} {75 66} {75 67} \
 {75 68} {75 69} {75 70} {76 0} {76 1} {76 2} {76 3} {76 4} {76 5} \
 {76 6} {76 7} {76 8} {76 9} {76 60} {76 61} {76 62} {76 63} {76 64} \
 {76 65} {76 66} {76 67} {76 68} {76 69} {76 70} {77 0} {77 1} {77 2} \
 {77 3} {77 4} {77 5} {77 6} {77 7} {77 8} {77 9} {77 10} {77 59} \
 {77 60} {77 61} {77 62} {77 63} {77 64} {77 65} {77 66} {77 67} {77 68} \
 {77 69} {77 70} {78 0} {78 1} {78 2} {78 3} {78 4} {78 5} {78 6} \
 {78 7} {78 8} {78 9} {78 10} {78 11} {78 57} {78 58} {78 59} {78 60} \
 {78 61} {78 62} {78 63} {78 64} {78 65} {78 66} {78 67} {78 68} {78 69} \
 {78 70} {79 0} {79 1} {79 2} {79 3} {79 4} {79 5} {79 6} {79 7} \
 {79 8} {79 9} {79 10} {79 11} {79 12} {79 13} {79 56} {79 57} {79 58} \
 {79 59} {79 60} {79 61} {79 62} {79 63} {79 64} {79 65} {79 66} {79 67} \
 {79 68} {79 69} {79 70} {80 0} {80 1} {80 2} {80 3} {80 4} {80 5} \
 {80 6} {80 7} {80 8} {80 9} {80 10} {80 11} {80 12} {80 13} {80 14} \
 {80 55} {80 56} {80 57} {80 58} {80 59} {80 60} {80 61} {80 62} {80 63} \
 {80 64} {80 65} {80 66} {80 67} {80 68} {80 69} {80 70} {81 0} {81 1} \
 {81 2} {81 3} {81 4} {81 5} {81 6} {81 7} {81 8} {81 9} {81 10} \
 {81 11} {81 12} {81 13} {81 14} {81 15} {81 16} {81 53} {81 54} {81 55} \
 {81 56} {81 57} {81 58} {81 59} {81 60} {81 61} {81 62} {81 63} {81 64} \
 {81 65} {81 66} {81 67} {81 68} {81 69} {81 70} {82 0} {82 1} {82 2} \
 {82 3} {82 4} {82 5} {82 6} {82 7} {82 8} {82 9} {82 10} {82 11} \
 {82 12} {82 13} {82 14} {82 15} {82 16} {82 17} {82 18} {82 51} {82 52} \
 {82 53} {82 54} {82 55} {82 56} {82 57} {82 58} {82 59} {82 60} {82 61} \
 {82 62} {82 63} {82 64} {82 65} {82 66} {82 67} {82 68} {82 69} {82 70} \
 {83 0} {83 1} {83 2} {83 3} {83 4} {83 5} {83 6} {83 7} {83 8} \
    {83 9} {83 10} {83 11} {83 12} {83 13} {83 14} {83 15} {83 16} {83 17} \
    {83 18} {83 19} {83 20} {83 49} {83 50} {83 51} {83 52} {83 53} {83 54} \
    {83 55} {83 56} {83 57} {83 58} {83 59} {83 60} {83 61} {83 62} {83 63} \
    {83 64} {83 65} {83 66} {83 67} {83 68} {83 69} {83 70} {84 0} {84 1} \
    {84 2} {84 3} {84 4} {84 5} {84 6} {84 7} {84 8} {84 9} {84 10} \
    {84 11} {84 12} {84 13} {84 14} {84 15} {84 16} {84 17} {84 18} {84 19} \
    {84 20} {84 21} {84 22} {84 47} {84 48} {84 49} {84 50} {84 51} {84 52} \
    {84 53} {84 54} {84 55} {84 56} {84 57} {84 58} {84 59} {84 60} {84 61} \
    {84 62} {84 63} {84 64} {84 65} {84 66} {84 67} {84 68} {84 69} {84 70} \
    {85 0} {85 1} {85 2} {85 3} {85 4} {85 5} {85 6} {85 7} {85 8} \
    {85 9} {85 10} {85 11} {85 12} {85 13} {85 14} {85 15} {85 16} {85 17} \
    {85 18} {85 19} {85 20} {85 21} {85 22} {85 23} {85 24} {85 25} {85 26} \
    {85 43} {85 44} {85 45} {85 46} {85 47} {85 48} {85 49} {85 50} {85 51} \
    {85 52} {85 53} {85 54} {85 55} {85 56} {85 57} {85 58} {85 59} {85 60} \
    {85 61} {85 62} {85 63} {85 64} {85 65} {85 66} {85 67} {85 68} {85 69} \
    {85 70} }

  foreach pnt $point_list {
    set x [lindex $pnt 0]
    set y [lindex $pnt 1]
    image_frame_pattern_round put $color -to $x $y
  } ;# foreach pnt

   # List of {x y} coordinates, where image_frame_pattern_oval
   # needs background colors
  set point_list { {0 0} {0 1} {0 2} \
    {0 3} {0 4} {0 5} {0 6} {0 7} {0 8} \
    {0 9} {0 10} {0 11} {0 12} {0 13} {0 14} {0 15} {0 16} {0 17} \
    {0 18} {0 19} {0 63} {0 64} {0 65} {0 66} {0 67} {0 68} {0 69} \
    {0 70} {0 71} {0 72} {0 73} {0 74} {0 75} {0 76} {0 77} {0 78} \
    {0 79} {0 80} {0 81} {0 82} {0 83} {1 0} {1 1} {1 2} {1 3} \
    {1 4} {1 5} {1 6} {1 7} {1 8} {1 9} {1 10} {1 11} {1 12} \
    {1 13} {1 69} {1 70} {1 71} {1 72} {1 73} {1 74} {1 75} {1 76} \
    {1 77} {1 78} {1 79} {1 80} {1 81} {1 82} {1 83} {2 0} {2 1} \
    {2 2} {2 3} {2 4} {2 5} {2 6} {2 7} {2 8} {2 9} {2 10} \
    {2 11} {2 71} {2 72} {2 73} {2 74} {2 75} {2 76} {2 77} {2 78} \
    {2 79} {2 80} {2 81} {2 82} {2 83} {3 0} {3 1} {3 2} {3 3} \
    {3 4} {3 5} {3 6} {3 7} {3 8} {3 9} {3 73} {3 74} {3 75} \
    {3 76} {3 77} {3 78} {3 79} {3 80} {3 81} {3 82} {3 83} {4 0} \
    {4 1} {4 2} {4 3} {4 4} {4 5} {4 6} {4 7} {4 75} {4 76} \
    {4 77} {4 78} {4 79} {4 80} {4 81} {4 82} {4 83} {5 0} {5 1} \
    {5 2} {5 3} {5 4} {5 5} {5 6} {5 76} {5 77} {5 78} {5 79} \
    {5 80} {5 81} {5 82} {5 83} {6 0} {6 1} {6 2} {6 3} {6 4} \
    {6 5} {6 77} {6 78} {6 79} {6 80} {6 81} {6 82} {6 83} {7 0} \
    {7 1} {7 2} {7 3} {7 4} {7 78} {7 79} {7 80} {7 81} {7 82} \
    {7 83} {8 0} {8 1} {8 2} {8 3} {8 79} {8 80} {8 81} {8 82} \
    {8 83} {9 0} {9 1} {9 2} {9 80} {9 81} {9 82} {9 83} {10 0} \
    {10 1} {10 2} {10 80} {10 81} {10 82} {10 83} {11 0} {11 1} {11 81} \
    {11 82} {11 83} {12 0} {12 1} {12 81} {12 82} {12 83} {13 0} {13 82} \
    {13 83} {14 0} {14 82} {14 83} {15 83} {16 83} {17 83} {18 83} {19 83} \
    {75 83} {76 83} {77 83} {78 83} {79 83} {80 0} {80 82} {80 83} {81 0} \
    {81 82} {81 83} {82 0} {82 1} {82 81} {82 82} {82 83} {83 0} {83 1} \
    {83 81} {83 82} {83 83} {84 0} {84 1} {84 2} {84 80} {84 81} {84 82} \
    {84 83} {85 0} {85 1} {85 2} {85 80} {85 81} {85 82} {85 83} {86 0} \
    {86 1} {86 2} {86 3} {86 79} {86 80} {86 81} {86 82} {86 83} {87 0} \
    {87 1} {87 2} {87 3} {87 4} {87 78} {87 79} {87 80} {87 81} {87 82} \
    {87 83} {88 0} {88 1} {88 2} {88 3} {88 4} {88 5} {88 77} {88 78} \
    {88 79} {88 80} {88 81} {88 82} {88 83} {89 0} {89 1} {89 2} {89 3} \
    {89 4} {89 5} {89 6} {89 76} {89 77} {89 78} {89 79} {89 80} {89 81} \
    {89 82} {89 83} {90 0} {90 1} {90 2} {90 3} {90 4} {90 5} {90 6} \
    {90 7} {90 8} {90 74} {90 75} {90 76} {90 77} {90 78} {90 79} {90 80} \
    {90 81} {90 82} {90 83} {91 0} {91 1} {91 2} {91 3} {91 4} {91 5} \
    {91 6} {91 7} {91 8} {91 9} {91 73} {91 74} {91 75} {91 76} {91 77} \
    {91 78} {91 79} {91 80} {91 81} {91 82} {91 83} {92 0} {92 1} {92 2} \
    {92 3} {92 4} {92 5} {92 6} {92 7} {92 8} {92 9} {92 10} {92 11} \
    {92 71} {92 72} {92 73} {92 74} {92 75} {92 76} {92 77} {92 78} {92 79} \
    {92 80} {92 81} {92 82} {92 83} {93 0} {93 1} {93 2} {93 3} {93 4} \
    {93 5} {93 6} {93 7} {93 8} {93 9} {93 10} {93 11} {93 12} {93 13} \
    {93 14} {93 68} {93 69} {93 70} {93 71} {93 72} {93 73} {93 74} {93 75} \
    {93 76} {93 77} {93 78} {93 79} {93 80} {93 81} {93 82} {93 83} }

  foreach pnt $point_list {
    set x [lindex $pnt 0]
    set y [lindex $pnt 1]
    image_frame_pattern_oval put $color -to $x $y
  } ;# foreach pnt
  unset point_list
} ;# viewSetPalette

proc viewSavePalette {} {
  global gvar
#TODO:   WHAT A PROBLEM!!!    :(
return
  image_frame_pattern_oval write [file join $gvar(my_bitmap.path) frame_pattern_oval.ppm]
  image_frame_pattern_round write [file join $gvar(my_bitmap.path) frame_pattern_round.ppm]
} ;# viewSavePalette

  #*** End of file util.tcl ***
