#include <string.h>
#include <stdlib.h>
#include "pfa.h"
#include "project.h"
#include "misc.h"
#include "errors.h"

char buffer[160];

// Return-Zeichen im Ausgabestream loeschen
void LoescheZeichen(FILE *f)
{
  fflush(f);
  fseek(f, -1, SEEK_CUR);
}

FA::FA()
{
  KnotenFont=KantenFont=NULL;
  EdgeNumber=0;
  version=0;
}

FA::FA(const FA& x)
{
  KnotenFont=strdup(x.KnotenFont);
  KantenFont=strdup(x.KantenFont);
  Vars.Copy(x.Vars);
  Vertices.Copy(x.Vertices);
  CodeArt=x.CodeArt;
/*  nrrealstartstates=x.nrrealstartstates;
  nrrealfinalstates=x.nrrealfinalstates;
  queue.Copy(x.queue);
  PML.Copy(x.PML);
  StructQ.Copy(x.StructQ);
  exzulkanteendstart=x.exzulkanteendstart;
  EdgeNumber=x.EdgeNumber;*/
  version=x.version;
}

FA& FA::operator=(const FA &x)
{
  KnotenFont=strdup(x.KnotenFont);
  KantenFont=strdup(x.KantenFont);
  Vars.Copy(x.Vars);
  Vertices.Copy(x.Vertices);
  CodeArt=x.CodeArt;
/*  nrrealstartstates=x.nrrealstartstates;
  nrrealfinalstates=x.nrrealfinalstates;
  queue.Copy(x.queue);
  PML.Copy(x.PML);
  StructQ.Copy(x.StructQ);
  exzulkanteendstart=x.exzulkanteendstart;
  EdgeNumber=x.EdgeNumber;*/
  version=x.version;
/*  KnotenFont=strdup(x.KnotenFont);
  KantenFont=strdup(x.KantenFont);
  Vertices.Copy(x.Vertices);
  CodeArt=x.CodeArt;
  Vars.Copy(x.Vars);*/
  return *this;
}

FA::~FA()
{
  if (KnotenFont)
    free(KnotenFont);
  if (KantenFont)
    free(KantenFont);
}

int FA::Load(char *fname)
{
  FILE *f;
  char buffer[MAXLEN],token[MAXLEN], *bufptr;
  
  // Datei oeffnen

  f=fopen(fname,"rt");
  if (!f)
  {
    fprintf(stderr, FILE_OPEN_ERR, fname);
    return 0;
  }

  // PEP-Format : 1. Zeile == "PEP"
  if (!fgets(buffer, MAXLEN, f))
  {
    fprintf(stderr, FIRST_LINE_ERR);
    fclose(f);
    return 0;
  }
  if (strncmp(buffer, "PEP", 3))
  {
    fprintf(stderr, NO_PEP); 
    fclose(f);
    return 0;
  }

  // PEP-Format : 2. Zeile == "FiniteAutomaton"
  if (!fgets(buffer, MAXLEN, f))
  {
    fprintf(stderr, SECOND_LINE_ERR); 
    fclose(f);
    return 0;
  }
  if (strncmp(buffer, "FiniteAutomaton", 15))
  {
    fprintf(stderr, NO_FA);
    fclose(f);
    return 0;
  }

  // PEP-Format : 3. Zeile == "FormatN"
  if (!fgets(buffer, MAXLEN, f))
  {
    fprintf(stderr, THIRD_LINE_ERR);
    fclose(f);
    return 0;
  }
  if (strncmp(buffer, "FORMAT_N", 8))
  {
    fprintf(stderr, INVALID_FORMAT);
    fclose(f);
    return 0;
  }

  if (!strncmp(buffer, "FORMAT_N2", 9))
  {
    version=2;
  }

  // Rest der Datei einlesen
  for (;;)
  {
    if (!fgets(token, MAXLEN, f))
    {
      fprintf(stderr, EOF_ERR);
      fclose(f);
      return 0;
    }

    if (!strncmp(token, "DPL", 3))
    {
      // DPL : Knoten-Font
      if (KnotenFont)
        free(KnotenFont);
      if (!fgets(buffer, MAXLEN, f))
      {
        fprintf(stderr, NFONT_ERR); 
        fclose(f);
        return 0;
      }
      ConvertString(buffer);
      KnotenFont=strdup(buffer);
    }

    if (!strncmp(token, "DAR", 3))
    {
      // DAR : Kanten-Font
      if (KantenFont)
        free(KantenFont);
      if (!fgets(buffer, MAXLEN, f))
      {
        fprintf(stderr, EFONT_ERR);
        fclose(f);
        return 0;
      }
      ConvertString(buffer);
      KantenFont=strdup(buffer);
    }

    int id,x,y,typ;
 
    if (!strncmp(token, "PL", 2))
    {
      // PL : Alle Knoten
      while (fgets(token, MAXLEN, f))
      {
        if (!strncmp(token, "AR", 2))
          break;
        bufptr=token;
        bufptr=ReadInt(bufptr, id);
        bufptr=ReadInt(bufptr, x);
        if (*bufptr!='@')
        {
          fprintf(stderr, NODE_ERR, token);
          fclose(f);
          return 0;
        }
        bufptr=ReadInt(bufptr, y);
        bufptr=ReadInt(bufptr, typ);
	ReadString(bufptr, buffer);
        AddVertex(id,typ,buffer);
      }
    }

    int from,to,offsetx, offsety;

    if (!strncmp(token, "AR", 2))
    {
      // AR : Alle Kanten
      while (fgets(token, MAXLEN, f))
      {
        if (!strncmp(token, "TX", 2))
          break;
        bufptr=token;
	if (version==2)
	  bufptr=ReadInt(bufptr, id);
        bufptr=ReadInt(bufptr, from);
        if (*bufptr!='>')
        {
          fprintf(stderr, EDGE_ERR, token);
          fclose(f);
          return 0;
        }
        bufptr=ReadInt(bufptr, to);

        bufptr=ReadInt(bufptr, x);
        if (*bufptr!='@')
        {
          fprintf(stderr, EDGE_ERR, token);
          fclose(f);
          return 0;
        }
        bufptr=ReadInt(bufptr, y);

        bufptr=ReadInt(bufptr, offsetx);
        if (*bufptr!='@')
        {
          fprintf(stderr, EDGE_ERR, token);
          fclose(f);
          return 0;
        }
        bufptr=ReadInt(bufptr, offsety);
	ReadString(bufptr, buffer);
	AddEdge(id,from,to,buffer);
      }
    }

    if (!strncmp(token, "TX", 2))
    {
      // Zeile ueberlesen
      fgets(buffer, MAXLEN, f);

      // Code-Art lesen
      fgets(buffer, MAXLEN, f);
      ConvertString(buffer);
      if (!strncmp(buffer, "Struct", 6))
        CodeArt=0;
      else      
        if (!strncmp(buffer, "B(PN)2", 6))
          CodeArt=1;
        else
        {
          fprintf(stderr, INVALID_CODE, buffer);
          fclose(f);
          return 0;
        }

      // Zeile ueberlesen
      fgets(buffer, MAXLEN, f);

      // Variablen einlesen
      while (!feof(f))
      {
        fgets(buffer, MAXLEN, f);
        if (buffer[0]=='%')
          break;
        ConvertString(buffer);
        Vars.Add(strdup(buffer));
      }
      // der rest sind nur kommentare
      fclose(f);
      return 1;
    }
  }
}
    
void FA::Dump()
{
  int i,j;

  printf("Dump of FA :\n\n");
  printf("Edgefont : %s\n", KantenFont);
  printf("Vertexfont : %s\n\n", KnotenFont);
  printf("Vertex table :\n");
  printf(" Id |Type|Name\n");
  printf("----|----|--------------------\n");
  for (i=0; i<Vertices.GetSize(); i++)
    printf("%4d|%4d|%-20s\n", Vertices[i].id,
                              Vertices[i].typ,
                              Vertices[i].name);
  printf("\n");
  printf("Edge table :\n");
  printf("From| To |Name\n");
  printf("----|----|--------------------\n");
  for (i=0; i<Vertices.GetSize(); i++)
  {
    for (j=0; j<Vertices[i].EdgeSame.GetSize(); j++)
      printf("%4d|%4d|%-20s\n", Vertices[i].id,
                                Vertices[i].id,
                                Vertices[i].EdgeSame[j].name);
    for (j=0; j<Vertices[i].EdgeOut.GetSize(); j++)
      printf("%4d|%4d|%-20s\n", Vertices[i].id,
                                Vertices[Vertices[i].EdgeOut[j].target].id,
                                Vertices[i].EdgeOut[j].name);
  }
  printf("\n");
  if (!CodeArt)
    printf("Type : Struct\n");
  else
    printf("Type : B(PN)2\n");
  printf("Variables :\n");
  for (i=0; i<Vars.GetSize(); i++)
    printf("  %s\n", Vars[i]);
  printf("\n");
}

int FA::ExistArcs()
{
  int i;
  for (i=0; i<Vertices.GetSize(); i++)
    if (Vertices[i].EdgeSame.GetSize()+Vertices[i].EdgeOut.GetSize()!=0)
      return 1;
  return 0;
}

int FA::GetVertexIndex(int vindex)
{
  int i;
  for (i=0; i<Vertices.GetSize(); i++)
  {
    if (Vertices[i].id==vindex)
      return i;
  }
  return -1;
}

void FA::AddVertex(int id, int typ, char *name)
{
  Vertex v;
  v.id=id;
  v.typ=typ;
  v.name=strdup(name);
  Vertices.Add(v);
}

void FA::AddEdge(int id, int from, int to, char *name)
{
  int findex,tindex;
  Edge e;
  findex=GetVertexIndex(from);
  tindex=GetVertexIndex(to);
  if ((findex==-1)||(tindex==-1))
    return;
  if (version==2)
    e.id=id;
  else
  {
    e.id=EdgeNumber;
    EdgeNumber++;
  }
  e.target=tindex;
  e.name=strdup(name);
  if (findex!=tindex)
    Vertices[findex].EdgeOut.Add(e);
  else
    Vertices[findex].EdgeSame.Add(e);

  if (findex!=tindex)
  {
    e.target=findex;
    Vertices[tindex].EdgeIn.Add(e);
  }
}

void FA::PrintDeclStruct(FILE *f, int spaces)
{
  char buffer[160];

  for (int i=0; i<Vars.GetSize(); i++)
  {
    if (*Vars[i])
    {
      if (spaces==4)
	sprintf(buffer, "    %s", Vars[i]);
      else
	sprintf(buffer, "        %s", Vars[i]);
      PrintVar(f, reffile, buffer, 1);
    }
  }
}  

void FA::PcPrint(FILE *f, int nrstates, char *parsym)
{
  char buffer[160];
  fprintf(f, "    %s\n", parsym);
  fprintf(f, "    begin\n");
  zeilenr+=2;
  sprintf(buffer, "        var   pc : {0..%d} init 0;", nrstates);
  PrintVar(f, reffile, buffer, 1);
  PrintDeclStruct(f,8);
  fprintf(f, "        do   <true> enter\n");
  zeilenr++;
}

void FA::MultiStartPrint(FILE *f, int nrstartstates, char *brackets)
{
  int i;
  char buffer[160];
  strcpy(brackets, "  ");
  if (nrstartstates>1)
  {
    for (i=1; i<=nrstartstates; i++)
    {
      sprintf(buffer, "        %s <'pc = 0 and pc' = %d>; repeat", brackets, i);
      PrintBlock(f, reffile, buffer, nrstartstates>1);
      strcpy(brackets, "[]");
    }
  }
}

void FA::SchlingenPrint(FILE *f, int v, int start_compile_id, char *brackets)
{
  char buf[160],buffer[160];
  int i;

  for (i=0; i<Vertices[v].EdgeSame.GetSize(); i++)
  {
    if (strlen(Vertices[v].EdgeSame[i].name))
    {
      if (!CodeArt)
      {
        // kommt noch
      }
      else
      {
        strcpy(buf, Vertices[v].EdgeSame[i].name);
        DeleteBrackets(buf);
        sprintf(buffer, "        %s <pc = %d and (%s)>; repeat", brackets,
	           start_compile_id, buf);
        Vertices[v].EdgeSame[i].ref=PrintAction(f, reffile, buffer);
      }
    }
    else
    {
      sprintf(buffer, "        %s <pc = %d>; repeat", brackets, start_compile_id);
      Vertices[v].EdgeSame[i].ref=PrintAction(f, reffile, buffer);
    }
    brackets[0]='[';
    brackets[1]=']';
  }
}

void FA::KantenPrint(FILE *f, int v, int start_compile_id, char *brackets)
{
  char chanpart[160]="                    ";
  char textpart[160]="                    ";
  char buf[160], buffer[160];
  int i,ziel_id;

  for (i=0; i<Vertices[v].EdgeOut.GetSize(); i++)
  {
    strcpy(chanpart,"                    ");
    strcpy(textpart,"                    ");
    ziel_id=Vertices[Vertices[v].EdgeOut[i].target].compile_id;
    if (strlen(Vertices[v].EdgeOut[i].name))
    {
      if (!CodeArt)
      {
        // kommt noch
      }
      else
      {
        strcpy(buf, Vertices[v].EdgeOut[i].name);
        DeleteBrackets(buf);
        sprintf(buffer, "        %s <'pc = %d and (%s) and pc' = %d>; repeat", 
          brackets, start_compile_id, buf,ziel_id);
        Vertices[v].EdgeOut[i].ref=PrintAction(f, reffile, buffer);
      }
    }
    else
    {
      sprintf(buffer, "        %s <'pc = %d and pc' = %d>; repeat", 
        brackets, start_compile_id, ziel_id);
      Vertices[v].EdgeOut[i].ref=PrintAction(f, reffile, buffer);
    }
    brackets[0]='[';
    brackets[1]=']';
  }
}

void FA::FinalPlacePrint(FILE *f, int v, int start_compile_id, char *brackets)
{
  char buffer[160];
  if (Vertices[v].typ==3)
  {
    sprintf(buffer, "        %s <pc = %d>; exit", brackets, start_compile_id);
    PrintAction(f, reffile, buffer);
    brackets[0]='[';
    brackets[1]=']';
  }
}
          
void FA::Compile(FILE *f, char *parsym)
{
  int nrstartstates;
  int nrstates;
  int i;
  int start_compile_id;
  char brackets[160]="  ";

  nrstartstates=NrStartStates();
  nrstates=NrStates(nrstartstates);

  PcPrint(f, nrstates-1, parsym);

  MultiStartPrint(f, nrstartstates, brackets);

  for (i=0; i<Vertices.GetSize(); i++)
  {
    start_compile_id=Vertices[i].compile_id;
    SchlingenPrint(f, i, start_compile_id, brackets);
    KantenPrint(f, i, start_compile_id, brackets);
    FinalPlacePrint(f, i, start_compile_id, brackets);
  }
 
  fprintf(f, "        od\n");
  fprintf(f, "    end\n\n");
  zeilenr+=2;
}
  
void FA::DeleteBrackets(char *s)
{
  char *ptr=s;

  if (s[0]=='<')
    while(*ptr++)
      *(ptr-1)=*ptr;
  if (*--ptr=='>')
    ptr=0;
}

// Struct - Compiler

int FA::GetStartState()
{
  int i;
  for (i=0; i<Vertices.GetSize(); i++)
    if (Vertices[i].typ==1)
      return i;
  return -1;
}

int FA::GetEndPlace()
{
  int i;
  for (i=0; i<Vertices.GetSize(); i++)
    if (Vertices[i].typ==3)
      return i;
  return -1;
}

int FA::NrRealStartStates()
{
  int i,number=0;

  for (i=0; i<Vertices.GetSize(); i++)
  {
    if ((Vertices[i].typ==1) && 
        (Vertices[i].EdgeOut.GetSize()+Vertices[i].EdgeSame.GetSize()))
      number++;
  }
  return number;
}

int FA::NrRealFinalStates()
{
  int i,number=0;

  for (i=0; i<Vertices.GetSize(); i++)
  {
    if ((Vertices[i].typ==3) && (Vertices[i].EdgeIn.GetSize()))
      number++;
  }
  return number;
}

int FA::NrStates(int nrstartstates)
{
  int i,number=0,start=0;
  if (nrstartstates < 2)
  {
    for (i=0; i<Vertices.GetSize(); i++)
    {
      number++;
      if (Vertices[i].typ==1)
      {
	Vertices[i].compile_id=0;
	start=1;
      }
      else
	Vertices[i].compile_id=number-start;
    }
    return (number+1-start);
  }
  else
  {
    number = nrstartstates;
    for (i=0; i<Vertices.GetSize(); i++)
    {
      if (Vertices[i].typ==1)
	Vertices[i].compile_id=nrstartstates--;
      else
      {
	Vertices[i].compile_id=number-start;
	number++;
      }
    }
    return (number+1);
  }
}

int FA::NrStartStates()
{
  int i,number=0;

  for (i=0; i<Vertices.GetSize(); i++)
    if (Vertices[i].typ==1)
      number++;
   
  return number;
}

int FA::ExZulKanteEndStart()
{
  int i,v;

  if ((nrrealstartstates!=1)||(nrrealfinalstates!=1))
    return 0;

  v=GetEndPlace();
  for (i=0; i<Vertices[v].EdgeOut.GetSize(); i++)
  {
    if (Vertices[Vertices[v].EdgeOut[i].target].typ==1)
    {
      if (strlen(Vertices[v].EdgeOut[i].name))
        return -1;
      return 1;
    }
  }
  return 0;
}

void FA::EnQueue(int i, StructArray q, int plusinnr, int changed)
{
  queue.Enqueue(Cell(i,q,plusinnr,changed));
}

void FA::EnQueue(int i, Struct q, int plusinnr, int changed)
{
  StructArray sa;
 
  sa.Add(q);
  queue.Enqueue(Cell(i,sa,plusinnr,changed));
}

Cell FA::DeQueue()
{
  Cell c;
  if (queue.GetSize())
    return queue.Dequeue();
  else
  {
    c.v_index=-1;
    return c;
  }
}    

// PlaceMarkList ....

void FA::PutMarkToPlace(int v_index, StructArray &q, int plusinnr)
{
  int i,j,k,l,m;
  EdgeArray ea;

#ifdef DEBUG
  printf("... PutMarkToPlace(v_index=%s, plusinnr=%d\n", Vertices[v_index].name, plusinnr);
#endif
  int gefunden=0;
  for (i=0; i<PML.GetSize(); i++)
  {
    if (PML[i].id == v_index)
    {
      gefunden=1;
      break;
    }
  }

  if (!gefunden)
  {
    PlaceMarkArray pma;
    pma.id=v_index;
    pma.Data.Add(PlaceMark(0, plusinnr, 0, 1, ea, q));
    PML.Add(pma);
    return;
  }

  // PlaceMark aus Liste holen
  PlaceMarkArray &pma=PML[i];

  // PlaceMark vorhanden, wenn abgearbeitet => Ende
  if (pma.Data[0].done)
    return;

  // PlaceMark vorhanden, komplette StructQueue ueberpruefen
  
  for (l=0; l<pma.Data.GetSize(); l++)
  {
    for (j=0,k=0; j<pma.Data[l].StructQ.GetSize(); j++)
      if ((k<q.GetSize()) &&
	  (pma.Data[l].StructQ[j].v_id==q[k].v_id) &&
	  (pma.Data[l].StructQ[j].count==q[k].count))
	k++;
      else
	break;
    if (k==q.GetSize())
      return;
  }
#ifdef DEBUG
  printf("... PlaceMark: Eintrag noch nicht vorhanden !\n");
#endif
  // PlaceMark einfuegen und alle nachfolgenden Eintraege anpassen
  m=pma.Data[pma.Data.GetSize()-1].plusinnr;
  pma.Data.Add(PlaceMark(0, m, 0, 1, ea, q));
  for (j=0; j<pma.Data.GetSize(); j++)
    pma.Data[j].plusinnr+=plusinnr;
}

PlaceMarkArray FA::GetPlaceMarkList(int v_index)
{
  int i;
  PlaceMarkArray p;

  for (i=0; i<PML.GetSize(); i++)
    if (PML[i].id==v_index)
      return PML[i];
  return p;
}

int FA::GetPlaceMarkIndex(int v_index)
{
  int i;

  for (i=0; i<PML.GetSize(); i++)
    if (PML[i].id==v_index)
      return i;
  return -1;
}

EdgeArray FA::GetRepeatKanten(int v_index)
{
  int i;
  EdgeArray x;

  for (i=0; i<PML.GetSize(); i++)
    if (PML[i].id==v_index)
      return PML[i].Data[0].RepeatEdges;
  return x;
}

AllInEdge FA::AllInEdges(int v_index, int innr, StructArray q)
{
   /* 
    *  branchinfo == -1 : Schleife schliesst, 
    *                     aber noch nicht abgearbeitet,
    *  branchinfo ==  0 : Schleife schliesst und schon abgearbeitet,
    *  branchinfo  >  0 : Schleife schliesst nicht.
    *  fuer Zyklus an Einfach-Startknoten : branchinfo == number == 2.
    */
  int pmi,help,help2;
  StructArray que;
  EdgeArray ea;
  int plusinnr=0, number=0, flag=0, counter, returnwert, zusammengefasste,i,j;

  help=0;
  pmi=GetPlaceMarkIndex(v_index);
  PlaceMarkArray &pm=PML[pmi];
  if (pm.Data[0].done)
  {
    for (j=0; j<pm.Data.GetSize(); j++)
    {
      PlaceMark &pmd=pm.Data[j];
      if ((pmd.branchinfo<=0) && (pmd.StructQ[0].v_id==q[0].v_id))
      {
	if (!pmd.branchinfo)
	{
	  if (IdentStruct(q, pmd.StructQ))
	  {
	    return AllInEdge(1,0);
	  }
	}
	else
	{
	  if (IdentStruct(q, pmd.StructQ))
	  {
	    flag=0;
	    returnwert=2;
	    i=0;
	    while ((!flag) && (i<q.GetSize()))
	    {
	      que.Copy(q, i);
	      flag=1;
	      help=0;
	      while (help<pm.Data.GetSize())
	      {
		PlaceMark &ph=pm.Data[help];
		if ((ph.branchinfo==-1)&&(AlmostIdentStruct(que,ph.StructQ)))
		{
		  returnwert++;
		  flag=0;
		  ph.branchinfo=0;
		  help++;
		  while (help<pm.Data.GetSize())
		  {
		    PlaceMark &ph=pm.Data[help];
		    if ((ph.branchinfo==-1)&&(AlmostIdentStruct(que,ph.StructQ)))
		      ph.branchinfo=0;
		    help++;
		  }
		  break;
		}
		help++;
	      }
	      i++;
	    }
	    return AllInEdge(returnwert, 0);
	  }
	}
      }
      help++;
      number++;
    }
    return AllInEdge(2,0);
  }
  else
  {
    while (help<pm.Data.GetSize())
    {
      help++;
      number++;
    }
#ifdef DEBUG
    printf("... AllInEdges: number=%d, innr=%d, plusinnr=%d\n", number, innr, pm.Data[0].plusinnr);
#endif
    if (number < (innr + pm.Data[0].plusinnr))
    {
      return AllInEdge(0,0);
    }
    while (!flag)
    {
      flag=1;
      help=0;
      while (help<pm.Data.GetSize())
      {
	PlaceMark &ph=pm.Data[help];
	if (ph.branchinfo==-1)
	{
          help++;
	  continue;
	}
	help2=help+1;
	counter=ph.StructQ[0].number-1;
	zusammengefasste=ph.united;
	while (help2<pm.Data.GetSize())
	{
	  if (pm.Data[help2].branchinfo!=-1)
	  {
	    if (AlmostIdentStruct(ph.StructQ, pm.Data[help2].StructQ))
	    {
	      counter--;
	      zusammengefasste+=pm.Data[help2].united;
	    }
	  }
	  help2++;
	}
	if (!counter)
	{
	  plusinnr+=(1-zusammengefasste);
	  ph.branchinfo=-1;
	  help2=help+1;
	  while (help2<pm.Data.GetSize())
	  {
	    if (pm.Data[help2].branchinfo!=-1)
	    {
	      if (AlmostIdentStruct(ph.StructQ, pm.Data[help2].StructQ))
	      {
	        pm.Data[help2].branchinfo=-1;
	      }
	    }
	    help2++;
	  }
	  if (ph.StructQ.GetSize()>1)
	  {
	    StructArray q;
	    q.Copy(ph.StructQ, 1);
	    pm.Data.Add(PlaceMark(0, pm.Data[pm.Data.GetSize()-1].plusinnr, 1, 1, ea, q));
	    flag=0;
	  }
	}
	help++;
      }
    }
  }
  help=0;
//  pm=PlaceMarkList[help];
  while (help<pm.Data.GetSize())
  {
    pm.Data[help].done=1;
    help++;
  }
  flag=0;
  i=0;
  returnwert=2;
  while ((!flag) && (i<q.GetSize()))
  {
    que.Copy(q, i);
    flag=1;
    help=0;
    while (help<pm.Data.GetSize())
    {
      PlaceMark &ph=pm.Data[help];
      if ((ph.branchinfo==-1)&&(AlmostIdentStruct(ph.StructQ,que)))
      {
	returnwert++;
	flag=0;
	ph.branchinfo=0;
	help++;
	while (help<pm.Data.GetSize())
	{
	  PlaceMark &ph=pm.Data[help];
	  if ((ph.branchinfo==-1)&&(AlmostIdentStruct(ph.StructQ,que)))
	  {
	    ph.branchinfo=0;
	  }
	  help++;
	}
	break;
      }
      help++;
    }
    i++;
  }
  help=0;
  while (help<pm.Data.GetSize())
  {
    PlaceMark &ph=pm.Data[help];
    if ((ph.branchinfo>0) && (!ph.united))
    {
      plusinnr++;
    }
    help++;
  }
  plusinnr--;
  return AllInEdge(returnwert, plusinnr);
}

int FA::IdentStruct(StructArray q1, StructArray q2)
{
  int i;

  if (q1.GetSize()!=q2.GetSize())
    return 0;
  for (i=0; i<q1.GetSize(); i++)
  {
    if (q1[i].v_id!=q2[i].v_id)
      return 0;
    if (q1[i].number!=q2[i].number)
      return 0;
    if (q1[i].count!=q2[i].count)
      return 0;
  }
  return 1;
}

int FA::AlmostIdentStruct(StructArray q1, StructArray q2)
{
  int i;
#ifdef DEBUG
  printf("AlmostIdentStruct:\nq1:\n");
  DumpStruct(q1);
  printf("q2:\n");
  DumpStruct(q2);
#endif
  if (q1.GetSize()!=q2.GetSize())
    return 0;
  if (q1[0].v_id!=q2[0].v_id)
    return 0;
  if (q1[0].count==q2[0].count)
    return 0;
  if (q1[0].number!=q2[0].number)
    return 0;
  for (i=1; i<q1.GetSize(); i++)
  {
    if (q1[i].v_id!=q2[i].v_id)
      return 0;
    if (q1[i].number!=q2[i].number)
      return 0;
    if (q1[i].count!=q2[i].count)
      return 0;
  }
#ifdef DEBUG
  printf("AlmostIdentStruct: true\n");
#endif
  return 1;
}
  

int FA::InPlusinnrStartList(int v_index, IntArray ar)
{
  int i;
  for (i=0; i<ar.GetSize(); i++)
    if (ar[i]==v_index)
      return 1;
  return 0;
}

void FA::PutPlusinnrStartList(int v_index, IntArray ar)
{
  ar.AddHead(v_index);
}

int FA::InRepeatKanten(Edge &kante, int v_id)
{
  int i;
  EdgeArray repeatedges=GetRepeatKanten(v_id);
  for (i=0; i<repeatedges.GetSize(); i++)
    if (repeatedges[i].id==kante.id)
      return 1;
  return 0;
}

void FA::InArcIdList(IntArray inarcs, Edge e)
{
  inarcs.AddHead(e.id);
}

IntArray FA::CircleRepeat(int v_id, int s_id)
{
  int wegezaehler,kante,i,outnr;
  Circle circle;
  QueueCircle CircleQ;
  IntArray inarcs,ia;

  wegezaehler=1;
  while (wegezaehler>0)
  {
    if (v_id==s_id)
    {
      inarcs.AddHead(kante);
      wegezaehler--;
      if (CircleQ.GetSize())
      {
	circle=CircleQ.Dequeue();
	v_id=circle.v_index;
      }
      else
	v_id=-1;
      continue;
    }
    else
      if (Vertices[v_id].typ==3) 
      {
	return ia;
      }
      else {
	if (GetPlaceMarkIndex(v_id)!=-1)
	    return ia;
      }
    outnr=Vertices[v_id].EdgeOut.GetSize();
    if (outnr == 1)
    {
      kante=Vertices[v_id].EdgeOut[0].id;
      v_id=Vertices[v_id].EdgeOut[0].target;
    }
    else
      if (outnr > 1)
      {
	if (circle.history.contains(v_id)==1)
	{
	  wegezaehler--;
	  if (CircleQ.GetSize())
	  {
	    circle=CircleQ.Dequeue();
	    v_id=circle.v_index;
	  }
	  else
	    v_id=-1;
	  continue;
	}
	circle.history.AddHead(v_id);
	wegezaehler += (outnr-1);
	int old_v_id=v_id;
	for (i=0; i<outnr-1; i++)
	{
	  kante=Vertices[old_v_id].EdgeOut[i].id;
	  v_id=Vertices[old_v_id].EdgeOut[i].target;
	  if (v_id == s_id)
	  {
	    inarcs.AddHead(kante);
	    wegezaehler--;
	  }
	  else
	    CircleQ.Enqueue(Circle(v_id, circle.history));
	}
        kante=Vertices[old_v_id].EdgeOut[outnr-1].id;
	v_id=Vertices[old_v_id].EdgeOut[outnr-1].target;
      }
  }
  if (v_id!=-1)
    printf("Fehler in CircleRepeat !\n");
  return inarcs;
}

int FA::Circle_(int v_index)
{
  int i,j,pmi,outnr,resultgive,innumber;
  Edge kante;
  StructArray q;
  IntArray inarcs,allinarcs;
  EdgeArray RepeatEdges;

#ifdef DEBUG
  printf("\nCircle_: %s\n", Vertices[v_index].name);
#endif


  pmi=GetPlaceMarkIndex(v_index);

  if (pmi==-1)
  {
    q.Add(Struct(v_index, 0, 0));
    PutMarkToPlace(v_index,q,0);
    pmi=GetPlaceMarkIndex(v_index);
  }

  if (PML[pmi].Data[0].RepeatEdges.GetSize())
  {
    return 0;
  }


  outnr=Vertices[v_index].EdgeOut.GetSize();
  for (i=0; i<outnr; i++)
  {
    kante=Vertices[v_index].EdgeOut[i];
    inarcs=CircleRepeat(kante.target, v_index);
    if (inarcs.GetSize())
    {
      resultgive=1;
      RepeatEdges=PML[pmi].Data[0].RepeatEdges;
      PML[pmi].Data[0].RepeatEdges.Add(kante);
#ifdef DEBUG
      printf("Circle_: %s->%s\n", Vertices[v_index].name, Vertices[kante.target].name); 
#endif
      for (j=0; j<inarcs.GetSize(); j++)
      {
	if (!allinarcs.contains(inarcs[j]))
	  allinarcs.Add(inarcs[j]);
      }
    }
  }    
  if (PML[pmi].Data[0].RepeatEdges.GetSize())
  {
    if (!CircleStructNaming(v_index,v_index))
    {
      PML[pmi].Data[0].RepeatEdges.Clear();
      allinarcs.Clear();
      resultgive=0;
    }
  }
  innumber=allinarcs.GetSize();
  if (innumber)
  {
    for (i=0; i<PML[pmi].Data.GetSize(); i++)
      PML[pmi].Data[i].plusinnr -= innumber;
  }
  return resultgive;
}

int FA::CircleStructNaming(int v_id, int s_id)
{
  EdgeArray repeatedges;
  int innr,outnr,z_id,allinedges,i,anzahlechtekanten,j;
  int changed=0;
  int plusinnr=0;
  int circle=0;
  QueueCell q;
  Cell c;
  StructArray StructQ,StructQ1;
  AllInEdge allinedgesresult;

  repeatedges=GetRepeatKanten(v_id);
  outnr=repeatedges.GetSize();
  for (i=0; i<repeatedges.GetSize()-1; i++)
  {
    z_id=repeatedges[i].target;
    changed++;
    Struct str1(v_id, i, 2*outnr);
    Cell c1(z_id, str1, plusinnr, changed);
    q.Enqueue(c1);
//    q.Enqueue(Cell(z_id, Struct(v_id, i, 2*outnr), plusinnr, changed));
  }
  i++;
  changed++;
  z_id=repeatedges[outnr-1].target;
  StructQ.Clear();
  StructQ.Add(Struct(v_id, i, 2*outnr));
  v_id=z_id;
  while (v_id>=0)
  {
    if (v_id==s_id)
    {
      changed++;
      if (q.GetSize())
      {
	c=q.Dequeue();
	v_id=c.v_index;
	StructQ.Copy(c.StructA);
	plusinnr=c.plusinnr;
      }
      else
	v_id=-1;
    }
    else
    {
      innr=Vertices[v_id].EdgeIn.GetSize();
      outnr=Vertices[v_id].EdgeOut.GetSize();
      if (innr>1)
      {
	PutMarkToPlace(v_id, StructQ, plusinnr);
	allinedgesresult=AllInEdges(v_id, innr, StructQ);
	allinedges=allinedgesresult.allinedges;
	plusinnr=allinedgesresult.plusinnr;
	if (allinedges == 0)
	{
	  q.Enqueue(Cell(v_id, StructQ, 0, changed));
	  c=q.Dequeue();
	  v_id=c.v_index;
	  StructQ.Copy(c.StructA);
	  plusinnr=c.plusinnr;
	  if ((changed==c.changed) && (circle==1))
	  {
	    q.Enqueue(Cell(v_id, StructQ, 0, changed));
	    return 0;
	  }
	  else
	    if ((changed==c.changed) && (circle==0))
	    {
	      circle=1;
	      changed++;
	      if (Circle_(v_id)==1)
	      {
		circle=0;
	      }
	    }
	    else
	      if ((changed!=c.changed) && (circle==1))
	      {
		if (Circle_(v_id)==1)
		{
		  changed++;
		  circle=0;
		}
	      }
	  continue;
	}
	else
	  if (allinedges == 1)
	  {
	    changed++;
	    if (q.GetSize())
	    {
	      c=q.Dequeue();
	      v_id=c.v_index;
	      StructQ.Copy(c.StructA);
	      plusinnr=c.plusinnr;
	    }
	    else
	      v_id=-1;
	    continue;
	  }
	  else
	    if (allinedges == 2)
	    {
	      changed++;
	    }
	    else
	      if (allinedges >= 3)
	      {
		changed++;
		while (allinedges > 2)
		{
		  StructQ.DeleteHead();
		  allinedges--;
		}
	      }
      }
      anzahlechtekanten=AnzahlEchteKanten(v_id);
      if (anzahlechtekanten==1)
      {
	changed++;
	v_id=Get1EchteKnoten(v_id);
      }
      else
	if (anzahlechtekanten>1)
	{
	  i=0;
	  z_id=Vertices[v_id].EdgeOut[0].target;
	  for (j=0; j<Vertices[v_id].EdgeOut.GetSize()-1; j++)
	  {
	    z_id=Vertices[v_id].EdgeOut[j].target;
	    changed++;
	    if (!InRepeatKanten(Vertices[v_id].EdgeOut[j], v_id))
	    {
	      i++;
	      StructQ1.Copy(StructQ);
	      StructQ1.AddHead(Struct(v_id, i, anzahlechtekanten));
	      q.Enqueue(Cell(z_id, StructQ1, plusinnr, changed));
	    }
	  }
	  changed++;
	  if (InRepeatKanten(Vertices[v_id].EdgeOut[j], v_id)==1)
	  {
	    if (q.GetSize())
	    {
	      c=q.Dequeue();
	      v_id=c.v_index;
	      StructQ.Copy(c.StructA);
	      plusinnr=c.plusinnr;
	    }
	    else
	      v_id=-1;
	  }
	  else
	  {
	    StructQ.AddHead(Struct(v_id, anzahlechtekanten, anzahlechtekanten));
	    v_id=z_id;
	  }
	}
	else
        {
	  printf("Endknoten");
	}
    }
  }
  return 1;
}
	    
  
// Einfuegen eines virtuellen Start-Knotens, der e-Kanten zu den "realen"
// Start-Knoten legt.
int FA::StructNamingStart()
{
  int i,changed=0;

  for (i=0; i<Vertices.GetSize(); i++)
  {
    // Alle Knoten durchgehen
    if (Vertices[i].typ==1)
    {
      // Knoten ist Start-Knoten
      if (Vertices[i].EdgeOut.GetSize())
      {
        // Start-Knoten hat ausgehende Kanten, Kante vom virtuellen Knoten zu
        // diesem Knoten legen
        EnQueue(i, Struct(-i,2,2), 0, changed);
	changed++;
      }
    }
  }
  return changed;
}   


int FA::StructNaming(int v_index)
{
  StructArray StructA;
  int changed = 0,plusinnr = 0,innr,outnr,circle=0;
  IntArray startlist;
  Cell popresult;
  EdgeArray repeatedges;
  AllInEdge allinedgesresult;
  int allinedges,anzahlechtekanten,i,index,z_id;

  // Initialisierung des ersten Knotens
  StructA.Add(Struct(-v_index, 2, 2));

  // Bei mehreren Startknoten virtuellen Startknoten erzeugen
  if (NrStartStates()>1)
  {
    changed=StructNamingStart();
    popresult=DeQueue();
    v_index=popresult.v_index;
    StructA.Copy(popresult.StructA);
  }
#ifdef DEBUG
  printf("NrRealFinalStates = %d\n", nrrealfinalstates);
#endif
  while (v_index>=0)
  {
#ifdef DEBUG
    printf("v_index=%s, innr=%d, outnr=%d, anzahlechtekanten=%d, changed %d\n", Vertices[v_index].name, Vertices[v_index].EdgeIn.GetSize(), Vertices[v_index].EdgeOut.GetSize(), AnzahlEchteKanten(v_index), changed);
    DumpProcesses();
    printf("Struct :\n");
    DumpStruct(StructA);
#endif
    if ((Vertices[v_index].typ==3) && (nrrealfinalstates==1))
    {
      // Knoten ist Endknoten und es ist nur ein Endknoten vorhanden
      repeatedges=GetRepeatKanten(v_index);
      if ((Vertices[v_index].EdgeOut.GetSize()) && (!repeatedges.GetSize()))
      {
        if ((!exzulkanteendstart) || (Vertices[v_index].EdgeOut.GetSize()>1))
	{
          // Test, ob zulaessiger Zyklus am Endknoten
          if (!Circle_(v_index))
	  {
            fprintf(stderr, CIRCLE_ERR); 
            return 0;
          }
        }
      }
      // Prozess ist am Endknoten und terminiert, neuen Prozess aktivieren
      changed++;
      popresult=DeQueue();
      v_index=popresult.v_index;
      StructA=popresult.StructA;
      plusinnr=popresult.plusinnr;
    }
    else
    {
#ifdef DEBUG
      DumpPML();
#endif
      innr=Vertices[v_index].EdgeIn.GetSize();
      outnr=Vertices[v_index].EdgeOut.GetSize();
      if (((Vertices[v_index].typ!=1) && (innr>1)) ||
          ((Vertices[v_index].typ==1) && (innr>0) && (!exzulkanteendstart)) ||
          ((Vertices[v_index].typ==1) && (innr>1) && (exzulkanteendstart)))
      {
        if (Vertices[v_index].typ==1)
	{
          // Knoten ist Startknoten
          if (!InPlusinnrStartList(v_index, startlist))
	  {
            // Jeder Startknoten, der eingehende Kante hat, soll genau einmal
            // sein plusinnr um eins erhoehen
            PutPlusinnrStartList(v_index, startlist);
            plusinnr=1;
          }
        }

        // Ablegen der Knoteninformationen
        PutMarkToPlace(v_index, StructA, plusinnr);
        
	// Feststellen der 
        allinedgesresult=AllInEdges(v_index, innr, StructA);
        allinedges=allinedgesresult.allinedges;
        plusinnr=allinedgesresult.plusinnr;
#ifdef DEBUG
        printf("allinedges : %d, plusinnr : %d, circle : %d\n", allinedges, plusinnr, circle);
#endif
        if (!allinedges)
	{
          EnQueue(v_index, StructA, 0, changed);
          popresult=DeQueue();
          v_index=popresult.v_index;
          StructA=popresult.StructA;
          plusinnr=popresult.plusinnr;
          if ((changed==popresult.changed) && (circle==1))
	  {
            EnQueue(v_index, StructA, 0, changed);
            fprintf(stderr, INVALID_CIRCLE);
            return 0;
          }
	  else
	    if ((changed==popresult.changed) && (!circle))
	    {
	      circle=1;
	      changed++;
	      if (Circle_(v_index)==1)
	      {
		circle=0;
	      }
	    }
	    else
	      if ((changed!=popresult.changed) && (circle==1))
	      {
	        if (Circle_(v_index)==1)
		{
		  changed++;
		  circle=0;
		}
	      }
	      continue;
	}
	else
	  if (allinedges==1)
	  {
	    changed++;
	    popresult=DeQueue();
	    v_index=popresult.v_index;
	    StructA=popresult.StructA;
	    plusinnr=popresult.plusinnr;
	    continue;
	  }
	  else
	    if (allinedges==2)
	    {
	      changed++;
	    }
	    else
	      if (allinedges>=3)
	      {
		changed++;
		while (allinedges>2)
		{
		  StructA.DeleteHead();
		  allinedges--;
		}
	      }
      }
      anzahlechtekanten=AnzahlEchteKanten(v_index);
      if (!anzahlechtekanten)
      {
	if (Vertices[v_index].typ!=3)
	{
	  fprintf(stderr, NAMING_ERR);
	  return 0;
	}
	popresult=DeQueue();
        v_index=popresult.v_index;
	StructA=popresult.StructA;
	plusinnr=popresult.plusinnr;
	changed++;
	continue;
      }
      if ((Vertices[v_index].typ==3) && (nrrealfinalstates>1))
      {
	StructA.AddHead(Struct(-v_index, 2, 2));
      }
      if (anzahlechtekanten==1)
      {
	changed++;
	v_index=Get1EchteKnoten(v_index);
      }
      else
        if (anzahlechtekanten>1)
	{
	  i=0;

	  for (index=0; index<Vertices[v_index].EdgeOut.GetSize()-1; index++)
	  {
	    z_id=Vertices[v_index].EdgeOut[index].target;
	    changed++;
	    if (!InRepeatKanten(Vertices[v_index].EdgeOut[index], v_index))
	    {
	      i++;
	      StructArray helpA;
	      helpA.Copy(StructA);
	      helpA.AddHead(Struct(v_index, i, anzahlechtekanten));
	      EnQueue(z_id, helpA, plusinnr, changed);
	    }
	  }
	  z_id=Vertices[v_index].EdgeOut[index].target;
	  changed++;
	  if (InRepeatKanten(Vertices[v_index].EdgeOut[index], v_index))
	  {
	    popresult=DeQueue();
	    v_index=popresult.v_index;
	    StructA=popresult.StructA;
	    plusinnr=popresult.plusinnr;
	  }
          else
	  {
	    StructA.AddHead(Struct(v_index, 
				   anzahlechtekanten, anzahlechtekanten));
	    v_index=z_id;
	  }
	}
	else
	  /* CS, 7.6.2000: Last argument was missing, fixed without sense */
	  fprintf(stderr, NODE_ERR, "Unknown");
    }
  }
  return 1;
}

void FA::DumpProcesses()
{
  printf("Number of Processes : %d\n", queue.GetSize());
  for (int i=0; i<queue.GetSize(); i++)
  {
    printf("%d: Node %s, %d pathlen, used %d, changed %d\n", i, Vertices[queue[i].v_index].name, queue[i].StructA.GetSize(), queue[i].plusinnr, queue[i].changed);
    for (int j=0; j<queue[i].StructA.GetSize(); j++)
    {
      printf(".. %d: Node %s, count %d, number %d\n", j, Vertices[abs(queue[i].StructA[j].v_id)].name, queue[i].StructA[j].count, queue[i].StructA[j].number);
    } 
  }
}

void FA::DumpStruct(StructArray x)
{
  for (int j=0; j<x.GetSize(); j++)
  {
    printf(".. %d: Node %s, count %d, number %d\n", j, Vertices[abs(x[j].v_id)].name, x[j].count, x[j].number);
  } 
}


int FA::Get1EchteKanten(int v_index)
{
  EdgeArray repeatedges;
  int i,j,gleich;

  repeatedges=GetRepeatKanten(v_index);
  if (!repeatedges.GetSize())
  {
    return 0;
  }
  for (i=0,j=0; i<Vertices[v_index].EdgeOut.GetSize(); i++)
  {
    gleich=0;
    while (j<repeatedges.GetSize())
    {
      if (repeatedges[j].id==Vertices[v_index].EdgeOut[i].id)
      {
	gleich = 1;
	break;
      }
      j++;
    }
    if (!gleich)
      return i;
  }
  return -1;
}

int FA::CompileStruct(FILE *f, int spaces)
{
  StructArray StructA;
  int startstate;

  startstate=GetStartState();
  nrrealstartstates=NrRealStartStates();
  if (!nrrealstartstates)
  {
    printf("No real start node found.\n");
    return 1;
  }
  nrrealfinalstates=NrRealFinalStates();
  exzulkanteendstart=ExZulKanteEndStart();
  if (exzulkanteendstart == -1)
  {
    printf("No path found from start to exit node.\n");
    return -1;
  }
  if (!StructNaming(startstate)) 
  {
    return -2;
  }
  StructA.Add(Struct(-startstate, 2, 2));
  StructStartPrint(f, startstate, StructA, spaces);  
  return 1;
}  

int FA::StructStartPrint(FILE *f, int v_id, StructArray StructA, int spaces)
{
  char space[160]="";
  int i,check;
  int firststate=1;
  int klammer=0;
  for (i=0; i<spaces; i++)
    strcat(space, " ");
  if (exzulkanteendstart==1)
  {
    sprintf(buffer, "%sdo  <true> enter", space);
    PrintAction(f, reffile, buffer);
    if ((GetRepeatKanten(v_id).GetSize()>0) ||
	(AnzahlEchteKanten(v_id)!=1))
    {
      sprintf(buffer, "%s    <TRUE>;", space);
      PrintAction(f, reffile, buffer);
    }
    check=StructPrint(f, v_id, StructA, spaces+4, 1, 0);
    LoescheZeichen(f);
    fprintf(f, " repeat\n");
    if ((GetRepeatKanten(v_id).GetSize()>0) ||
	(AnzahlEchteKanten(v_id)!=1))
    {
      sprintf(buffer, "%s    <TRUE>;", space);
      PrintAction(f, reffile, buffer);
    }
    else
      klammer=1;
    check=StructPrint(f, v_id, StructA, spaces+4, 1, 0, klammer);
    LoescheZeichen(f);
    fprintf(f, " exit\n");
    fprintf(f, "%sod;\n", space);
    zeilenr++;
    return check;
  }
  if (nrrealstartstates==1)
  {
    check=StructPrint(f, v_id, StructA, spaces, 1, 0);
  }
  else
  {
    sprintf(buffer, "%sdo  <true> enter", space);
    PrintAction(f, reffile, buffer);
    for (i=0; i<Vertices.GetSize(); i++)
    {
      if (Vertices[i].typ==1)
      {
	if (Vertices[i].EdgeOut.GetSize()>0)
	{
	  if (!firststate)
	  {
	    klammer=1;
	  }
          else
            klammer=0;
	  firststate=0;
	  if (klammer)
	    sprintf(buffer, "%s [] <TRUE>;", space);
	  else
	    sprintf(buffer, "%s    <TRUE>;", space);
	  PrintAction(f, reffile, buffer);
          zeilenr++;
//	  StructA[0].v_id=-i;
	  check=StructPrint(f, i, StructA, spaces+4, 1, 0);
          LoescheZeichen(f);
	  fprintf(f, " exit\n");
	}
      }
    }
    fprintf(f, "%sod;\n", space);
    zeilenr++;
  }
  return check;
}

int FA::StructPrint(FILE *f, int v_id, StructArray StructA, int nrspaces, int first, int end, int klammer)
{
  char space[160]="";
  int i,anzahlechtekanten,kante,z_id,check,firstkante;
  EdgeArray repeatedges;

  for (i=0; i<nrspaces; i++)
    strcat(space, " ");

  repeatedges = GetRepeatKanten(v_id);

  if (end==0)
  {
    if ((Vertices[v_id].EdgeSame.GetSize()>0) || (repeatedges.GetSize()>0))
    {
      StructPrintRepeat(f, v_id, nrspaces);
    }
  }

  if ((end==1) || (nrrealfinalstates == 1) || (Vertices[v_id].typ!=3))
  {
    if (Vertices[v_id].EdgeOut.GetSize()==0)
    {
      return v_id;
    }
    else
    {
      anzahlechtekanten=AnzahlEchteKanten(v_id);
      if ((anzahlechtekanten==1) && (exzulkanteendstart==1) && (Vertices[v_id].typ==3))
      {
	return v_id;
      }
      if ((anzahlechtekanten==1) || ((anzahlechtekanten==2) && (exzulkanteendstart==1) && (Vertices[v_id].typ==3)))
      {
	kante=Get1EchteKanten(v_id);
	z_id=Vertices[v_id].EdgeOut[kante].target;
	if (((Vertices[v_id].EdgeSame.GetSize()>0) || (repeatedges.GetSize()>0)) && (Vertices[v_id].typ!=3))
	{
	  KantenStructPrint(f, Vertices[v_id].EdgeOut[kante], nrspaces+4, 1);
	  LoescheZeichen(f);
	  fprintf(f, " exit\n");
	  fprintf(f, "%sod;\n", space);
          zeilenr++;
	}
	else
	{
	  KantenStructPrint(f, Vertices[v_id].EdgeOut[kante], nrspaces, klammer);
	}
	if (Vertices[z_id].EdgeOut.GetSize()==1)
	{
	  check = StructPrint(f, z_id, StructA, nrspaces, first, 0);
	}
	else
	{
	  if (BranchClose(z_id, StructA)==1)
	  {
	    repeatedges=GetRepeatKanten(z_id);
	    if ((Vertices[z_id].typ==3) && (first) && ((Vertices[z_id].EdgeSame.GetSize()>0) || (repeatedges.GetSize()>0)))
	    {
	      StructPrintRepeat(f, z_id, nrspaces);
	    }
	    return z_id;
	  }
	  else
	  {
	    check = StructPrint(f, z_id, StructA, nrspaces, 0, 0, klammer);
	  }
	}
      }
      else
      {
	if (!(((Vertices[v_id].EdgeSame.GetSize()>0) || (repeatedges.GetSize()>0)) && (Vertices[v_id].typ!=3)))
	{
	  sprintf(buffer, "%sdo  <true> enter", space);
	  PrintAction(f, reffile, buffer);
	}
	firstkante=1;
	for (i=0; i<Vertices[v_id].EdgeOut.GetSize(); i++)
	{
	  if (InRepeatKanten(Vertices[v_id].EdgeOut[i], v_id))
	  {
	    continue;
	  }
	  if (!firstkante)
	  {
	    klammer=1;
	  }
	  else
	    klammer=0;
	  firstkante=0;
	  z_id=Vertices[v_id].EdgeOut[i].target;
	  KantenStructPrint(f, Vertices[v_id].EdgeOut[i], nrspaces+4, klammer);
	  StructA.AddHead(Struct(v_id, 0, 0));
	  if (Vertices[z_id].EdgeIn.GetSize()==1)
	  {
	    check = StructPrint(f, z_id, StructA, nrspaces+4, 0, 0);
	  }
	  else
	  {
	    if (BranchClose(z_id, StructA)==1)
	    {
	      check=z_id;
	    }
	    else
	    {
	      check = StructPrint(f, z_id, StructA, nrspaces+4, 0, 0, klammer);
	    }
	  }
	  LoescheZeichen(f);
	  fprintf(f, " exit\n");
	}
	repeatedges.Clear();
	if (GetPlaceMarkIndex(check)!=-1)
	{
	  repeatedges.Copy(GetPlaceMarkList(check).Data[0].RepeatEdges);
	}
	if (BranchClose(check, StructA))
        {
	  fprintf(f, "%sod;\n", space);
          zeilenr++;
	  if ((Vertices[check].typ==3) && ((Vertices[check].EdgeSame.GetSize()>0) || (repeatedges.GetSize()>0)) && (first))
	  {
	    StructPrintRepeat(f, check, nrspaces);
	  }
	  return check;
	}
	fprintf(f, "%sod;\n", space);
        zeilenr++;
	if (Vertices[check].typ!=3)
        {
	  check=StructPrint(f, check, StructA, nrspaces, 0, 0, klammer);
	}
	repeatedges=GetRepeatKanten(check);
	if ((Vertices[check].typ==3) && ((Vertices[check].EdgeSame.GetSize()>0) || (repeatedges.GetSize()>0)))
        {
	  StructPrintRepeat(f, check, nrspaces);
	}
      }
    }
    v_id=check;
  }
  else
  {
    check=v_id;
  }
  anzahlechtekanten=AnzahlEchteKanten(v_id);
  if ((nrrealfinalstates>1) && (anzahlechtekanten>0))
  {
    check=WSPFState(f, anzahlechtekanten, v_id, StructA, nrspaces);
  }
  return check;
}

int FA::WSPFState(FILE *f, int anzahlechtekanten, int v_id, StructArray StructA, int nrspaces)
{
  int i;
  char space[160]="";
  char space1[160]="";
  int klammer;
  for (i=0; i<nrspaces; i++)
    strcat(space, " ");
  for (i=0; i<nrspaces-3; i++)
    strcat(space1, " ");

  StructA.AddHead(Struct(-v_id, 2, 2));
  sprintf(buffer, "%sdo  <true> enter", space);
  PrintAction(f, reffile, buffer);
  sprintf(buffer, "%s    <TRUE>;", space);
  PrintAction(f, reffile, buffer);
  LoescheZeichen(f);
  fprintf(f, " exit\n");
  klammer=1;
  if (anzahlechtekanten>1)
  {
    sprintf(buffer, "%s[] <TRUE>;\n", space1);
    PrintAction(f, reffile, buffer);
    klammer=0;
  }
  v_id=StructPrint(f, v_id, StructA, nrspaces+4, 1, 1, klammer);
  LoescheZeichen(f);
  fprintf(f, " exit\n");
  fprintf(f, "%sod;\n", space);
  zeilenr+=2;
  return v_id;
}

int FA::BranchClose(int v_id, StructArray StructA)
{
  int i;

  if ((Vertices[v_id].typ==3) && (nrrealfinalstates==1))
  {
    return 1;
  }
  if (!StructA.GetSize())
  {
    return 0;
  }

  if (GetPlaceMarkIndex(v_id)==-1)
    return 0;
  PlaceMarkArray &pml=PML[GetPlaceMarkIndex(v_id)];
  for (i=0; i<pml.Data.GetSize(); i++)
  {
    if ((pml.Data[i].branchinfo==0) && (pml.Data[i].StructQ[0].v_id==v_id))
    {
      return 1;
    }
  }
  return 0;
}

void FA::StructPrintRepeat(FILE *f, int v_id, int nrspaces)
{
  char space[160]="";
  int i,kante,z_id,check,j,k;
  int first=1;
  EdgeArray repeatedges;

  for (i=0; i<nrspaces; i++)
    strcat(space, " ");

  repeatedges=GetRepeatKanten(v_id);
  sprintf(buffer, "%sdo  <true> enter", space);
  PrintAction(f, reffile, buffer);
  if (Vertices[v_id].typ==3)
  {
    sprintf(buffer, "    %s<TRUE>;", space);
    PrintAction(f, reffile, buffer);
    LoescheZeichen(f);
    fprintf(f, " exit\n");
  }
  if (Vertices[v_id].EdgeSame.GetSize())
  {
    for (kante=0; kante<Vertices[v_id].EdgeSame.GetSize(); kante++)
    {
      KantenStructPrint(f, Vertices[v_id].EdgeSame[kante], nrspaces+4, (Vertices[v_id].typ==3)||(!first));
      first=0;
      LoescheZeichen(f);
      fprintf(f, " repeat\n");
    }
  }

#ifdef DEBUG
  printf("repeatedges : %d\n", repeatedges.GetSize());
#endif
  
  for (i=0; i<repeatedges.GetSize(); i++)
  {
    z_id=repeatedges[i].target;
    KantenStructPrint(f, repeatedges[i], nrspaces+4, (Vertices[v_id].typ==3)||(i));
    // Uebertragen der Referenz in die Kante des Automaten
    for (j=0; j<Vertices.GetSize(); j++)
    {
      for (k=0; k<Vertices[j].EdgeOut.GetSize(); k++)
      {
	if (Vertices[j].EdgeOut[k].id==repeatedges[i].id)
	  Vertices[j].EdgeOut[k].ref=repeatedges[i].ref;
      }
      for (k=0; k<Vertices[j].EdgeSame.GetSize(); k++)
      {
	if (Vertices[j].EdgeSame[k].id==repeatedges[i].id)
	  Vertices[j].EdgeSame[k].ref=repeatedges[i].ref;
      }
    }
    StructArray ar;
    check=StructPrintCircle(f, z_id, v_id, ar, nrspaces+4);
    if ((Vertices[check].typ!=3) && (Vertices[check].id!=Vertices[v_id].id))
    {
      printf("Unerw. Fall in StructPrintRepeat.\n");
    }
    LoescheZeichen(f);
    fprintf(f, " repeat\n");
  }
  if (Vertices[v_id].typ==3)
  {
    fprintf(f, "%sod;\n", space);
    zeilenr++;
  }
}

void FA::KantenStructPrint(FILE *f, Edge &kante, int nrspaces, int klammer)
{
  int i;
  char space[160]="";
  char textpart[160];
  char chanpart[160];
  char buffer1[160];

  for (i=0; i<nrspaces-3; i++)
    strcat(space, " ");
  if (klammer)
    strcat(space, "[] ");
  else
    strcat(space, "   ");

  sprintf(textpart, "                    ");
  sprintf(chanpart, "                    ");
  
  if (strlen(kante.name))
  {
    {
      sprintf(buffer1, "%s%s;", space, kante.name);
      kante.ref=PrintAction(f, reffile, buffer1);
#ifdef DEBUG
      printf("Adding Edge : %s , ref = %d\n", kante.name, kante.ref);
#endif
    }
  }
  else
  {
    sprintf(buffer1, "%s<TRUE>;", space);      
    kante.ref=PrintAction(f, reffile, buffer1);
  }
}

int FA::StructPrintCircle(FILE *f, int v_id, int start_id, StructArray StructA, int nrspaces, int klammer)
{
  char space[160]="";
  int i,kante,z_id,check;
  int first=1;
  EdgeArray repeatedges;

  for (i=0; i<nrspaces; i++)
    strcat(space, " ");
  
  if (v_id==start_id)
  {
    return v_id;
  }

  repeatedges=GetRepeatKanten(v_id);
  if ((Vertices[v_id].EdgeSame.GetSize()) || (repeatedges.GetSize()))
  {
    StructPrintRepeat(f, v_id, nrspaces);
  }
  if (!Vertices[v_id].EdgeOut.GetSize())
  {
    return v_id;
  }
  if (AnzahlEchteKanten(v_id)==1)
  {
    kante=Get1EchteKanten(v_id);
    z_id=Vertices[v_id].EdgeOut[kante].target;
    if ((Vertices[v_id].EdgeSame.GetSize()) || (repeatedges.GetSize()))
    {
      KantenStructPrint(f, Vertices[v_id].EdgeOut[kante], nrspaces+4, 1);
      LoescheZeichen(f);
      fprintf(f," exit\n");
      fprintf(f, "%sod;\n", space);
      zeilenr++;
    }
    else
    {
      KantenStructPrint(f, Vertices[v_id].EdgeOut[kante], nrspaces, klammer);
    }
    if (Vertices[z_id].EdgeIn.GetSize()>1)
    {
      check=StructPrintCircle(f, z_id, start_id, StructA, nrspaces, klammer);
    }
    else
    {
      if (BranchClose(z_id, StructA))
      {
	return z_id;
      }
      else
      {
	check=StructPrintCircle(f, z_id, start_id, StructA, nrspaces, klammer);
      }
    }
  }
  else
  {
    first=1;
    if (!((Vertices[v_id].EdgeSame.GetSize()) || (repeatedges.GetSize())))
    {
      sprintf(buffer, "%sdo  <true> enter", space);
      PrintAction(f, reffile, buffer);
    }
    for (i=0; i<Vertices[v_id].EdgeOut.GetSize(); i++)
    {
      if (InRepeatKanten(Vertices[v_id].EdgeOut[i], v_id))
	continue;
      first=0;
      KantenStructPrint(f, Vertices[v_id].EdgeOut[i], nrspaces+4, !first);
      z_id=Vertices[v_id].EdgeOut[kante].target;
      StructA.AddHead(Struct(v_id, 0, 0));
      if (Vertices[z_id].EdgeIn.GetSize()>1)
      {
	check=StructPrintCircle(f, z_id, start_id, StructA, nrspaces+4);
      }
      else
      {
	if (BranchClose(z_id, StructA))
	{
	  check=z_id;
	}
	else
	{
	  check=StructPrintCircle(f, z_id, start_id, StructA, nrspaces+4);
	}
      }
      LoescheZeichen(f);
      fprintf(f, " exit\n");
    }
    repeatedges=GetRepeatKanten(v_id);
    if (BranchClose(check, StructA))
    {
      fprintf(f, "%sod;\n", space);
      zeilenr++;
      return check;
    }
    fprintf(f, "%sod;\n", space);
    zeilenr++;
    if (check!=start_id)
    {
      return StructPrintCircle(f, check, start_id, StructA, nrspaces);
    }
  }
  return check;
}

int FA::AnzahlEchteKanten(int v_index)
{
  EdgeArray repeatedges;
  repeatedges=GetRepeatKanten(v_index);
  return Vertices[v_index].EdgeOut.GetSize()-repeatedges.GetSize();
}

int FA::Get1EchteKnoten(int v_id)
{
  int i;

  for (i=0; i<Vertices[v_id].EdgeOut.GetSize(); i++)
  {
    if (InRepeatKanten(Vertices[v_id].EdgeOut[i], v_id)==1)
      continue;
    return Vertices[v_id].EdgeOut[i].target;
  }
  fprintf(stderr, INV_CASE);
  return -1;
}

void FA::DumpPML()
{
  int i,j,k;
  for (i=0; i<PML.GetSize(); i++)
  {
    printf("PlaceMarkList fuer Knoten %s :\n", Vertices[PML[i].id].name);
    for (j=0; j<PML[i].Data.GetSize(); j++)
    {
      printf(".. done=%d, plusinnr=%d, branchinfo=%d\n", PML[i].Data[j].done, PML[i].Data[j].plusinnr, PML[i].Data[j].branchinfo);
      for (k=0; k<PML[i].Data[j].RepeatEdges.GetSize(); k++)
	printf("... RepeatEdge : %d\n", PML[i].Data[j].RepeatEdges[k].id);
      for (k=0; k<PML[i].Data[j].StructQ.GetSize(); k++)
	printf("... Struct : Node %s, Count %d, Number %d\n", Vertices[abs(PML[i].Data[j].StructQ[k].v_id)].name, PML[i].Data[j].StructQ[k].count, PML[i].Data[j].StructQ[k].number);
    }
  }
}

