#ifndef PFA_H
#define PFA_H

#include <stdio.h>
#include "array.h"
#include "vertex.h"
#include "edge.h"
#include "struct.h"
#include "cell.h"
#include "place.h"
#include "allinedges.h"
#include "circle.h"

extern char buffer[160];
extern char *refname;

class FA
{
  protected:
    char *KnotenFont;
    char *KantenFont;
    SArray Vars;
    int CodeArt;
    int nrrealstartstates;
    int nrrealfinalstates;
    QueueCell queue;
    PlaceMarkList PML;
    StructArray StructQ;
    int exzulkanteendstart;
    int EdgeNumber;
    int version;

  public:
    VertexArray Vertices;
    FA();
    virtual ~FA();
    FA(const FA& x);
    FA& operator=(const FA& x);
    int Load(char *fname);
    void Dump();
    int ExistArcs();
    int NrStartStates();
    int NrStates(int states);
    void PrintDeclStruct(FILE *f, int spaces);
    void PcPrint(FILE *f, int nrstates, char *parsym);
    void Compile(FILE *f, char *parsym);
    void MultiStartPrint(FILE *f, int nrstartstates, char *brackets);
    void SchlingenPrint(FILE *f, int v, int c, char *b);
    void KantenPrint(FILE *f, int v, int c, char *b);
    void FinalPlacePrint(FILE *f, int v, int c, char *b);
    void DeleteBrackets(char *s);
    int GetVertexIndex(int vindex);
    void AddVertex(int id, int typ, char *name);
    void AddEdge(int id, int from, int to, char *name);
    int GetStartState();
    int GetEndState();
    int GetEndPlace();
    int NrRealStartStates();
    int NrRealFinalStates();
    int ExZulKanteEndStart();
    void EnQueue(int i, StructArray q, int plusinnnr, int changed);
    void EnQueue(int i, Struct q, int plusinnnr, int changed);
    Cell DeQueue();
    void PutMarkToPlace(int v_index, StructArray &q, int plusinnr);
    PlaceMarkArray GetPlaceMarkList(int v_index);
    int GetPlaceMarkIndex(int v_index);
    EdgeArray GetRepeatKanten(int v_index);
    AllInEdge AllInEdges(int v_index, int innr, StructArray q);
    int IdentStruct(StructArray q1, StructArray q2);
    int AlmostIdentStruct(StructArray q1, StructArray q2);
    int InPlusinnrStartList(int v_index, IntArray ar);
    void PutPlusinnrStartList(int v_index, IntArray ar);
    int InRepeatKanten(Edge &kante, int v_id);
    void InArcIdList(IntArray inarcs, Edge e);
    IntArray CircleRepeat(int v_id, int s_id);
    int Circle_(int v_index);
    int StructNamingStart();
    int CircleStructNaming(int v_id, int s_id);
    int StructNaming(int v_index);
    int AnzahlEchteKanten(int v_index);
    int CompileStruct(FILE *f, int spaces);
    int Get1EchteKanten(int v_index);
    int StructStartPrint(FILE *f, int v_id, StructArray StructA, int spaces);
    int StructPrint(FILE *f, int v_id, StructArray StructA, int nrspaces, int first, int end, int klammer=0);
    void StructPrintRepeat(FILE *f, int v_id, int nrspaces);
    void KantenStructPrint(FILE *f, Edge& kante, int nrspaces, int klammer=0);
    int BranchClose(int v_id, StructArray StructA);
    int WSPFState(FILE *f, int anzahlechtekanten, int v_id, StructArray StructA, int nrspaces);
    int StructPrintCircle(FILE *f, int v_id, int start_id, StructArray StructA, int nrspaces, int klammer=0);
    int Get1EchteKnoten(int v_id);
    virtual char *GetClassName() { return "FA"; };
    void DumpProcesses();
    void DumpStruct(StructArray x);
    void DumpPML();
};

typedef Array<class FA> FAArray;

#endif


