/******************************************************************************/
/* FILE bpn.y                                                                 */
/* BISON source file: parser for b(pn)^2                                      */
/* 1993 by Burkhard Graves                                                    */
/* 1994 by Lars Jenner                                                        */
/* completely reviewed February 1995 by Stephan Merkel                        */
/* winter 1996/97 by Martin Ackermann                                         */
/******************************************************************************/
/* Note: Rules containing square brackets ('[' and ']') are no longer         */
/* actively supported and provided only for historical continuity with older  */
/* versions.                                                                  */
/* The default action '{ $$ = $1 }' is not explicitly given in rules.         */
/******************************************************************************/

%{
#include <string.h>
#include "frontend.h"
#include "symtab.h"
#include "tree.h"
#include "set.h"

SymTabEntry *ProcId;
TreeNode *ExprTree;
int	  ParseType;
int	  varcounter = 0;

#define YYERROR_VERBOSE 
#undef  YYERROR_VERBOSE 
  /* this provides for verbose error messages with bison as well as with */
  /* yacc when using the proper driver file (yacc_dri.own)               */

#define MAX_EXPECT 20
  /* this sets a limit to the number of expected tokens printed into */
  /* error messages                                                  */

int yyerror(char *msg)
{
	fprintf(stderr,"error while parsing: %s\n", msg);
	exit(1);
}

ShortEntryPtr ref;

#define _add_ref_to_node(_node, p1, p2, type_)	\
  {						\
    ref=MYmalloc(sizeof(ShortEntry));		\
    ref->s_ref.start_l = p1.start_l;		\
    ref->s_ref.start_c = p1.start_c;		\
    ref->s_ref.end_l   = p2.end_l;		\
    ref->s_ref.end_c   = p2.end_c;		\
    ref->s_type        = type_;			\
    *((ShortEntryPtr*)(node+1))=ref;		\
  }

#define _create_action_(erg, p1, p2, p3)	\
  {						\
    ref=MYmalloc(sizeof(ShortEntry));		\
    ref->s_ref.start_l = p1.start_l;		\
    ref->s_ref.start_c = p1.start_c;		\
    ref->s_ref.end_l   = p3.end_l;		\
    ref->s_ref.end_c   = p3.end_c;		\
    ref->s_type        = _ACTION;		\
    erg=GenNode(_EXPR,p2,(TreeNode*)ref);	\
  }

extern int   yylex(void);
extern char *yytext;

%}

%union {
  SymTabEntry   *sym;    /* defined in file symtab.h */
  IdentStruct    ident;  /* defined in file ident.h  */
  TreeNode      *node;   /* defined in file tree.h   */
  ArgListEntry  *arg;    /* defined in file tree.h   */
  SetRange      *range;  /* defined in file set.h    */
  RefStruct      ref;    /* defined in file symtab.h */
  int            wert;
}

%token  <ref> BEGIN_
%token  END
%token  CONST
%token  VAR
%token  CHAN
%token  STACK
%token  PROC
%token  VAL
%token  RES
%token  REF
%token  OF
%token  INIT
%token  DO
%token  OD
%token  REPEAT
%token  EXIT_
%token  ENTER
%token  PARALLEL
%token  CHOICE
%token  NOT
%token  AND
%token  OR
%token  TRUE
%token  FALSE
%token  MAX
%token  <wert>  NUMBER
%token  <wert>  CONST_IDENT
%token  <ident> IDENT
%token	<ref>   CA       /* closing action */
%token  <ref>   '<'
%token  <ref>   '>'
%token  <ref>   '['
%token  <ref>   ']'
%token  <ref>   ')'
%token  EMPTY_          /* '??' */
%token  FULL            /* '!!' */
%token  GE              /* '>=' */
%token  LE              /* '<=' */
%token  MOD             /* '%'  */


%left           PARALLEL
%left           CHOICE
%left           ';' ','
%left           OR
%right          AND /* don't change to left - TreeToGuard needs right-order */
%right          NOT
%nonassoc       '=' '#' '<' '>' GE LE
%left           '+' '-'
%left           '*' '/' MOD /* bison and yacc are incompatible on '%' */
%right          Unaryminus

%type <sym>   proc_ident par_list val_par_decl ref_par_decl par_ident
%type <range> set elem_list elem
%type <wert>  const_expr pos_expr init
%type <node>  com expr val_expr simple_expr

%start  alt


%%

alt	: decls 
		{ ParseType = 0; }
	| com
		{ ParseType = 1; }
	| proc_decl
		{ ParseType = 2; }
	;

decls
	: decl
	| decls decl
	;

decl
	: const_decl ';'
	| var_decl ';'
	;

const_decl
	: const IDENT '=' const_expr
		{
		  if ($2.sym->s_type != _UDEC)
		    Msg(ERR,"identifier %s already used", $2.sym->s_name);
		  give_typ(MNET_CONST, NULL, $4);
		}
	;

var_decl
	: var ident_list ':' set init
		{
		  give_typ(_VAR,$4,$5);
		}
	| var ident_list ':' CHAN const_expr OF set
		{
		  give_typ(_CHANNEL,$7,$5);
		}
	| var ident_list ':' STACK const_expr OF set
		{
		  give_typ(_STACK, $7, $5);
		}
	;

ident_list
	: IDENT
		{
		  make_ident(&$1)->s_ref.end_c |= (++varcounter) << 16;
		  
		}
	| ident_list ',' IDENT   
		{
		  make_ident(&$3)->s_ref.end_c |= (++varcounter) << 16;
		}
	;

set
	: '{' elem_list '}'
		{
		  $$ = $2;
		}
	;

elem_list
	: elem
		/* insert element into empty list */
		{
		  $$ = insert_elem($1, NULL);
		}
	| elem_list ',' elem
		/* insert element into elem_list */
		{
		  $$ = insert_elem($3, $1);
		}
	;

elem
	: const_expr
		{
		  $$ = GenRange($1, $1);
		}
	| const_expr '.' '.' const_expr
		{
		  $$ = GenRange($1, $4);
		}
	;

init
	: /* empty init */
		{
		  $$=NONE;
		}
	| INIT const_expr
		{
		  $$=$2;
		}
	;

const_expr
	: pos_expr
	| '-' NUMBER
		{
		  if (-$2 < MIN_NEG) Msg(FAT, "Constant out of range !");
                  $$=-$2;
		}
	;

const
	: CONST { yyerrok; }
	;

var
	: VAR { yyerrok; }
	;

com
	: '<' expr CA ';'
		{
		  _create_action_($$, $1, $2, $3);
		  ExprTree = $$;
		}
	;

expr
	: '(' expr ')'
		{
		  $$=GenNode(N_KLAMM,$2,NULL);
		}
	| IDENT
		{
		  chk_ident($1.sym,_VAR);
		  $1.sym->s_special|=_LG_UNPRIMED;
		  $$=GenNode(N_VAR,(TreeNode *)$1.sym,(TreeNode *)_UNPRIMED);
		}
	| val_expr
	;

val_expr
	: simple_expr
	| expr OR expr
		{
		  $$=GenNode(N_OR,$1,$3);
		}
	| expr AND expr
		{
		  $$=GenNode(N_AND,$1,$3);
		}
	| '!' expr %prec NOT
		{
		  $$=GenNode(N_NOT,$2,NULL);		  
		}
	| NOT expr
		{
		  $$=GenNode(N_NOT,$2,NULL);		  
		}
	| expr '=' expr
		{
		  $$=GenNode(N_EQ,$1,$3);
		}
	| expr '#' expr
		{
		  $$=GenNode(N_UNEQ,$1,$3);
		}
	| expr '<' expr
		{
		  $$=GenNode(N_LT,$1,$3);
		}
	| expr '>' expr
		{
		  $$=GenNode(N_GT,$1,$3);
		}
	| expr GE expr
		{
		  $$=GenNode(N_GE,$1,$3);
		}
	| expr LE expr
		{
		  $$=GenNode(N_LE,$1,$3);
		}
	| expr '+' expr
		{
		  $$=GenNode(N_PLUS,$1,$3);
		}
	| expr '-' expr
		{
		  $$=GenNode(N_MIN,$1,$3);
		}
	| expr '*' expr
		{
		  $$=GenNode(N_MULT,$1,$3);
		}
	| expr '/' expr
		{
		  $$=GenNode(N_DIV,$1,$3);
		}
	| expr MOD expr
		{
		  $$=GenNode(N_MOD,$1,$3);
		}
	| '-' expr %prec Unaryminus
		{
		  $$=GenNode(N_SIGN,$2,NULL);
		}
	;

simple_expr
	: '\'' IDENT
		{
		  chk_ident($2.sym,_VAR);
		  $2.sym->s_special|=_LG_PRIMED;
		  $$=GenNode(N_VAR,(TreeNode *)$2.sym,(TreeNode *)_PREPRIMED);
		}
	| IDENT '\''
		{
		  chk_ident($1.sym,_VAR);
		  $1.sym->s_special|=_LG_PRIMED;
		  $$=GenNode(N_VAR,(TreeNode *)$1.sym,(TreeNode *)_POSTPRIMED);
		}
	| IDENT '?'
		{
		  chk2_ident($1.sym,_CHANNEL,_STACK);
		  $1.sym->s_special|=_LG_READING;
		  $$=GenNode(N_VAR,(TreeNode *)$1.sym,(TreeNode *)_READ);
		}
	| IDENT '!'
		{
		  chk2_ident($1.sym,_CHANNEL,_STACK);
		  $1.sym->s_special|=_LG_WRITING;
		  $$=GenNode(N_VAR,(TreeNode *)$1.sym,(TreeNode *)_WRITE);
		}
	| IDENT EMPTY_
		{
		  chk2_ident($1.sym,_CHANNEL,_STACK);
		  if ($1.sym->s_init_or_cap == 0)
		    Msg(ERR,"illegal use of chan-0-identifier %s",
			$1.sym->s_name);
		  $1.sym->s_special|=_LG_EMPTY;
		  $$=GenNode(N_VAR,(TreeNode *)$1.sym,(TreeNode *)_EMPTY);
		}
	| IDENT FULL
		{
		  chk2_ident($1.sym,_CHANNEL,_STACK);
		  if ($1.sym->s_init_or_cap == 0)
		    Msg(ERR,"illegal use of chan-0-identifier %s",
			$1.sym->s_name);
		  $1.sym->s_special|=_LG_FULL;
		  $$=GenNode(N_VAR,(TreeNode *)$1.sym,(TreeNode *)_FULL);
		}
	| pos_expr
		{
		  $$=GenNode(N_CONST,(TreeNode *)$1,NULL);
		}
	;

pos_expr
	: NUMBER
	| TRUE
		{
		  $$=TT;
		}
	| FALSE
		{
		  $$=FF;
		}
	| CONST_IDENT
	;

proc_decl
	: proc proc_ident
	| proc proc_ident '(' par_list ')'
		{
		  LinkParlist($2,$4);
		}
	;

proc_ident
	: IDENT
		{
		  $$=make_ident(&$1);
		  give_typ(_BLOCKNAME,NULL,0);
		  ProcId = $$;
		}
	;

par_list
	: VAL val_par_decl
		{
		  $$=GenParList(NULL,_VAL,$2);
		}
	| par_list ',' VAL val_par_decl
		{
		  $$=GenParList($1,_VAL,$4);
		}
	| CONST val_par_decl
		{
		  $$=GenParList(NULL,_CONSTPAR,$2);
		}
	| par_list ',' CONST val_par_decl
		{
		  $$=GenParList($1,_CONSTPAR,$4);
		}
	| RES val_par_decl
		{
		  $$=GenParList(NULL,_RES,$2);
		}
	| par_list ',' RES val_par_decl
		{
		  $$=GenParList($1,_RES,$4);
		}
	| REF ref_par_decl
		{
		  $$=GenParList(NULL,_REF,$2);
		}
	| par_list ',' REF ref_par_decl
		{
		  $$=GenParList($1,_REF,$4);
		}
	;

val_par_decl
	: par_ident ':' set
		{ 
		  give_typ(_VAR,$3,NONE);
		}
	;

ref_par_decl
	: val_par_decl
	| par_ident ':' CHAN const_expr OF set
		{ 
		  give_typ(_CHANNEL,$6,$4);
		}
	| par_ident ':' STACK const_expr OF set 
		{
		  give_typ(_STACK,$6,$4); 
		} 
	;

par_ident
	: IDENT
		{
		  $$=make_par(&$1);
		}
	;

proc
	: PROC { yyerrok; };

%%
/******************************************************************************/
/* END OF FILE bpn.y                                                          */
/******************************************************************************/
