/****************************************************************************/
/* main.c                                                                   */
/*                                                                          */
/* Burkhard Graves, 10.10.93                                                */
/* Lars Jenner, Juni 1994                                                   */
/* Martin Ackermann, Fruehjahr 95                                           */
/* Burkhard Bieber 1995/96:                                                 */
/*   - Blockbehandlung ergaenzt                                             */
/*   - Phantomtransitionsbehandlung ergaenzt                                */
/*   - Schoenheitskorrekturen                                               */
/* Martin Ackermann, Winter 1996/97                                         */
/*   - Prozeduren, andere Referenzen, mehr Effizienz, Fehler beseitigt      */
/****************************************************************************/

  /************/
  /* includes */
  /************/

#include <string.h>
#include <time.h>
#include <ctype.h>
#include <stdlib.h>

#include "main.h"
#include "multiset.h"
#include "mnet.h"
#include "tables.h"
#include "macros.h"
#include "block.h"
#include "evaltree.h"
#include "compose.h"
#include "readHL.h"
#include "f_parse.h"
#include "box_sub.h"
#include "pfaFile.h"
#include "vars.h"
#include "decls.h"
#include "pfaref.h"

  /*******************/
  /* Datenstrukturen */
  /*******************/

Cardinal var_style = VAR_BOTH;   /* Ausgabe-Stil f. Kommunikationsvariablen */

char *SrcBuffer = NULL;          /* Buffer fuer den Quelltext               */
char **SrcLine;                  /* Array von Zeigern auf die einzelnen 
                                    Zeilenanfaenge im Quelltext             */
FILE *yytmp;			 /* um die Quelltextfragmente zu sammeln    */

Cardinal scale = 1,              /* Skalierungsfaktor fuer den Netz-Graphen */
         max_arc_vars = 1,       /* maximale Anzahl von Variablen in einem 
                                                                Kantenlabel */
         max_proc_inst = 2;      /* maximale Anzahl von Aktivierungen einer
                                                                   Prozedur */
Cardinal32 flag;                 /* globale Programm-Flags                  */

char    *actions_string;         /* die beim Aufruf von syncmnet ggf. 
                                    spezifizierten Aktionssymbole           */
char    *exclude_string = NULL;  /* ggf. auszuscliessende Aktionssymbole    */
char	*pfa_ref_file = NULL;	 /* Ausgabedatei fuer die Referenz-Tabelle  */
char    *pep_file = NULL;        /* Ausgabedatei im Standard-PEP-Format     */

char	*strptr;		 /* String zum Parsen			    */
char	*source,
        *source_ref = NULL;      /* Referenzen fuer das Eingabe-Netz        */

extern int yyparse(void);

  /************************/
  /* Aufruf demonstrieren */
  /************************/

static void usage(char *myname)
{
  fprintf(stderr,
    "\n%s -- translate a set of finite automata into an M-Net\n\n"

    "Usage: %s [options] <prjfile> -o <outfile> [-r <reffile>]\n\n"

    "     Options:\n"
    "      -b        write block information\n"
    "      -p        write phantom transitions\n"
    "      -s#       set scaling factor\n"
    "      -S        suppress scoping\n"
    "      -g[#..#]  use BPN procedure semantic with specified idset\n"
    "      -u[0,1,2] use either no procedure box, a stack, or a set\n\n"

    "The project file must contain information about global variables\n"
    "and the set of parallel finite automata which are to be translated.\n"
    "The resulting high level net is written to outfile.\n\n"
 
    "Version 0.9.5 (27.04.1998) Stefan Schwoon\n\n", myname, myname);

  exit(1);
} /* usage() */


/*****************************************************************************/
/* Main program.                                                             */
/*****************************************************************************/

int main(int argc, char **argv)
{

	int		i, num;
	char    	*prjfile = NULL;
	char		*c;
	char    	**dptr;
	clock_t 	time;
	t_pfahdr	pfahdr;
	M_Net		fa, net = NULL;
	int		id1 = 0, id2 = 0;
	int		procbox = 0;
	char		*procdecl = NULL;
	int		tmpfd;
	int		*refs;

	source=strdup("pfa2mnetXXXXXX"); 
	dptr = &prjfile;
	var_style = VAR_REAL;
	for (i = 1; i < argc; i++)
	{
		if (*argv[i] == '-')
			for (c=argv[i]; *++c;) switch(*c)
			{
				case 'b': flag |= BLOCK_OUTPUT;
					  break;
				case 'o': dptr = &pep_file;
					  break;
				case 'r': dptr = &pfa_ref_file;
					  break;
				case 'p': flag |= PHANTOM_TRANS_OUTPUT;
					  break;
				case 's': scale = 0;
					  while (isdigit(*++c) || !c--)
					  { scale = scale*10 + *c - '0'; }
					  break;
				case 'S': flag |= NO_SCOPING;
					  break;
				case 'g': GL_ProcDecl = True;
					  if (!isdigit(*(c+1))) break;
					  id1 = 0;
					  while (isdigit(*++c) || !c--)
					  { id1 = id1*10 + *c - '0'; }
					  id2 = 0; c += 2;
					  while (isdigit(*++c) || !c--)
					  { id2 = id2*10 + *c - '0'; }
					  break;
				case 'u': GL_ProcDecl = True;
					  procbox = 1;
					  switch (*(c+1))
					  {
					    case '1': procbox = 1;
						      c++; break;
					    case '2': procbox = 2;
						      c++; break;
					  }
					  break;
				case 'P': procdecl = c+1;
					  while (*(c+1)) c++;
					  break;
				default:  fprintf(stderr,"unknown flag: "
						         "'%c'\n",*c);
					  exit(1);
			}
		else
		{
			if (!dptr) usage(argv[0]);
			*dptr = argv[i];
			dptr = (dptr != &prjfile)? &prjfile : NULL;
		}
	}

	if (!prjfile) usage(argv[0]);

	if (!procdecl)
	{
		procdecl = MYmalloc(strlen(prjfile)+6);
		sprintf(procdecl,"proc %s",prjfile);
		c = strrchr(procdecl,'.');
		if (c) *c = '\0';
		/* next line changed after 1.8e */
                if ((c = strrchr(procdecl,'/'))) strcpy(procdecl+5,c+1);
	}

	strcpy(entry_type,"{dot}");
	if (id1 && id2)
	{
		macro_id_set = MYmalloc((id2-id1+2) * sizeof(Integer));
		for (i = id1; i <= id2; i++) macro_id_set[i-id1+1] = i;
		*macro_id_set = id2-id1+2;
		if (!procbox) sprintf(entry_type,"{%u..%u}",id1,id2);
	}

	if ((tmpfd=mkstemp(source))==-1)
	  exit(2);
	else
	  yytmp = fdopen(tmpfd,"w");

	time = clock();
	pfahdr = read_prj(prjfile);

	InitMultiSets();
	InitVarMSets();
	InitAllSymbolTables();
	InitBlockTable();
	s_init();

	yyleng = 0; strptr = procdecl; yyparse();
	if (ParseType != 2) yyerror("incorrect procedure declaration");

	parse_vars(pfahdr->variables);

	num = 0;
	for (i = 0; i < pfahdr->count; i++)
		if (*PH_Place(pfahdr,i))
		{
			fa = read_fa(PH_Place(pfahdr,i));
			net = !num++? fa : M_ConcurrentComposition(net,fa);
		}

	net = Scope(net,handle_vars());

	net = add_procbox(net,procbox,id1,id2);

	fclose(yytmp);

	refs = MYmalloc(RefCounter * sizeof(int));
	for (i = 0; i < RefCounter; i++)
		refs[i] = RefToStruct(i+1)->s_ref.end_c;
	for (i = 0; i < RefCounter; i++)
		RefToStruct(i+1)->s_ref.end_c &= 0xffff;

	HandleNet(net);

	for (i = 0; i < RefCounter; i++)
		RefToStruct(i+1)->s_ref.end_c = refs[i];
	free(refs);

	if (pfa_ref_file) print_pfa_refs(net,pfa_ref_file);
	remove(source);
	free(source);

	if (!num)
	{
		fprintf(stderr, "no PFAs specified, nothing to do\n");
		exit(0);
	}

/*	printf("Time needed: %f sec.\n\n",(float)(clock()-time)/CLOCKS_PER_SEC);
*/

	return 0;

}

/*****************************************************************************/
/* END OF FILE main.c           	                                     */
/*****************************************************************************/

