/****************************************************************************/
/* main.h                                                                   */
/* Burkhard Graves, 10.10.93                                                */
/* Lars Jenner, Juni 1994                                                   */
/* Martin Ackermann, Fruehjahr 95                                           */
/* Burkhard Bieber 1995/96:                                                 */
/*   - Blockbehandlung ergaenzt                                             */
/*   - Phantomtransitionsbehandlung ergaenzt                                */
/*   - Schoenheitskorrekturen                                               */
/* Martin Ackermann, Winter 1996/97                                         */
/*   - Prozeduren, andere Referenzen, mehr Effizienz, Fehler beseitigt      */
/****************************************************************************/

#ifndef __MAIN_H__
#define __MAIN_H__

  /************/
  /* includes */
  /************/

#include <stdio.h>
#include "../frontend/frontend.h"

  /*********************************************/
  /* Allgemeine Typdefinitionen und Konstanten */
  /*********************************************/

  /* Bit-Darstellung fuer die Kommandozeilenparameter */

#define USE_PROC_SET                1
#define CHECK_NET                   2
#define REMOVE_DUPL                 4
#define ELIMINATE_DEAD              8
#define PAR_TERM                   16
#define NO_SCOPING                 32
#define BIG_LOOP                   64
#define MEDIUM_LOOP               128
#define SHORT_FILE                256
#define BLOCK_OUTPUT              512
#define PHANTOM_TRANS_OUTPUT     1024
#define CHECK_NAMES              2048
#define INTERSECT_WARNING        4096

#define SYNCHRONIZE              8192
#define DO_RESTRICTION          16384

extern Cardinal scale,            /* Skalierungsfaktor fuer das Netz */
                max_arc_vars,     /* maximale Anzahl von Variablen in einem 
				                              Kantenlabel */
                max_proc_inst;    /* maximale Anzahl von Aktivierungen einer
				                                 Prozedur */
extern Cardinal32 flag;           /* globale Programm-Flags */
extern char *source,              /* Quelldatei b(pn)2 bzw. bei syncmnet
                                     das zu synchronisierende Netz          */
            *pep_file;            /* Ausgabedatei im Std-PEP-Format */

extern char *strptr;		  /* zeigt auf das zu parsende Code-Fragment */

  /**********************************/
  /* Defines fuer das Ausgabeformat */
  /**********************************/

#define exit_postfix	"@e"   /* Postfix fuer die Exit-Aktion */
#define and_sym		" & "  /* Symbol fuer logisches AND */

extern Cardinal var_style;  /* Ausgabe-Style fuer Kommunikationsvariablen */

#define VAR_BOTH  0  /* real_name UND com_nr ausgeben */
#define VAR_COM	  1  /* nur com_nr ausgeben */
#define VAR_REAL  2  /* nur real_name ausgeben */

  /******************************/
  /* Defines fuer Stringlaengen */
  /******************************/

#define MAX_ANNO_LENGTH  2048  /* maximale Laenge einer Annotation  */
#define MAX_IDENT_LENGTH  256  /* maximale Laenge eines Bezeichners */

  /***************************/
  /* Globale Datenstrukturen */
  /***************************/

extern FILE *yyin;
extern FILE *yytmp;

extern char *SrcBuffer;  /* Buffer fuer den Quelltext */
extern char **SrcLine;   /* Array von Zeigern auf die einzelnen */
                         /* Zeilenanfaenge im Quelltext */
extern int yyleng;

  /**********************************************************/
  /* Datentyp zur internen Darstellung von Variablen(namen) */
  /**********************************************************/

typedef unsigned short int Var;

  /**********************/
  /* HL-Action-Struktur */
  /**********************/

#define MaxAnzComVars  256 /* max. Anz. Com.-Vars. in einer Annotation */
#define MaxArity        10 /* maximale Aritaet einer HL-Aktion */

typedef Cardinal Action_nr;
typedef Cardinal ComVar;

typedef struct Action_
{ Action_nr	nr;		/* Aktionsnummer */
  Cardinal	arity;		/* Aritaet der Aktion */
  ComVar	com[MaxArity];	/* Kommunikationsvariablen */
  Cardinal	anzahl;		/* Anzahl des Auftretens in der Multimenge */ 
} Action, *MultiSet;

  /*********************/
  /* Variable-Struktur */
  /*********************/

typedef struct Variable_
{ ComVar	nr;      /* Nummer der Kommunikationsvariablen */
  Cardinal	anzahl;	 /* Anzahl des Auftretens in der Multimenge */ 
} Variable, *VarMSet;

  /************************************************/
  /* Darstellung einer Multimenge bei der Ausgabe */
  /************************************************/

extern bool MSdisplay_style;  /* globale Variable */

#define MS_WIDE  False
#define MS_SHORT True

  /*********************************************************************/
  /* Maximal vorgesehene Anzahl von Aktionsnamen in der Symboltabelle; */
  /* wird dynamisch an die tatsaechlich benoetigte Groesse angepasst!  */
  /*********************************************************************/

#define ACTION_BUF_SIZE 100

  /********************************************************************/
  /* Datentyp Symboltabelle fuer die Aktions-/Variablennamen (Parser) */
  /********************************************************************/

typedef char             *SymTabEntry_Pars;
typedef SymTabEntry_Pars *SymTab;

extern SymTab  ActTable;
extern SymTab  VarTable;
extern SymTab  RefTable;
extern SymTab  TplTable;

  /********************************************************************/
  /* Datentyp fuer eingelesene Referenzen, die zum Quellnetz gehoeren */
  /********************************************************************/

typedef struct SyncRefEntry_ {
  /* Achtung: Fuer das hl-backend muss s_ref an erster Stelle stehen und 
              s_type an zweiter Stelle!                                       */
  RefStruct              s_ref;         /* Definition im Quelltext            */
  char     	      	 s_type;	/* Typ                                */
  char                  *s_description; /* naehere Spezifikation des Typs     */
  char                  *s_meaning;     /* Bedeutung                          */
} SyncRefEntry, *SyncRefEntryPtr;

#endif  /* #ifndef __MAIN_H__ */

  /*** Dateiende main.h ***/

