#ifndef pfa_h
#define pfa_h

/*
###########################################################################
###########################################################################
##                                                                       ##
##  Datei   :    `pfaOptions.c'                                          ##
##               --------------                                          ##
##                                                                       ##
##  Inhalt  :    Hilfsfunktionen, die von mehreren Befehlsfunktionen     ##
##               aus den anderen Files ben"otigt werden.                 ##
##                                                                       ##
###########################################################################
###########################################################################
*/

/* #include "tk.h" */
#include "pfaMultList.h"


#define MALLOC(x)   ((x *) malloc( sizeof(x) ))	
#define FREE(x)     free( x )
#define LINE_LENGTH  1024


/*
 ##########################################################################
 #                                                                        #
 #  Deklaration der Datenstrukturen                                       #
 #                                                                        #
 ##########################################################################
 */

typedef struct Vertex_type Vertex_type;
typedef struct Point_type {
    int x,y;
} Point_type;

/* 
 *  In EdgeMove_type wird die Kurvenform der Kanten gespeichert, um
 *  die Kanten beim verschieben der entsprechenden Knoten schneller
 *  aktualisieren zu k"onnen. 
 */

typedef struct EdgeMove_type {
    double cos_alpha;
    double sin_alpha;
    double fraction;
} EdgeMove_type;


/* 
 *  In Edge_type werden alle ben"otigten Informationen einer Kante
 *  gespeichert. 
 */

typedef struct Edge_type
{
    int start;			/* Canvas-Id des Ausgangsknotens */
    int end;			/* Canvas-Id des Zielknotens */
    int e_id;                   /* Canvas-Id der Kante */
    int label_id;               /* Canvas-Id der Kantenbeschriftung */
    char *label;                /* Kantenbeschriftung */
    Point_type center;		/* Mittelpunkt der Kante */
    Point_type label_center;    /* Differenz des Textes vom Mittelpunkt */ 
    EdgeMove_type move_infos;   /* Informationen zur Form der Kante  */ 
} Edge_type;


/* 
 *  Jede Knotenstruktur vom Type Vertex_type enth"alt drei Listen vom 
 *  Typ EdgeList_type, in denen alle zu dem Knoten geh"orenden Kanten 
 *  einsortiert werden.
 */

typedef struct EdgeList_type {
    int arc_id;
    Edge_type *edge_object;
    int other_pid;
    Vertex_type *other_vertex;
    struct EdgeList_type *next;
} EdgeList_type;


/* 
 *  In Vertex_type werden alle ben"otigten Informationen eines Knotens
 *  gespeichert. 
 */

struct Vertex_type {
    int v_id;			/* Canvas-Id des Knotens */
    int type;			/* Typ des Knotens */
    int label_id;               /* Canvas-Id des Knotentextes */
    char *label;                /* Kantenbeschriftung */
    Point_type center;          /* Mittelpunkt der Kante */ 
    EdgeList_type *in;          /* Liste aller eingehenden Kanten */ 
    EdgeList_type *out;         /* Liste aller ausgehenden Kanten */ 
    EdgeList_type *own_edge;    /* Liste aller Schlingen */ 
    int compile_id;             /* Information f"ur den Compiler */ 
};


/* 
 *  Font_type enth"alt die Informationen zum Font eines Kommentares
 */

typedef struct Font_type
{
    char *family;
    char *weight;
    int slant;
    int size;

} Font_type;


/* 
 *  In Comment_type werden alle ben"otigten Informationen eines 
 *  Kommentares gespeichert. 
 */

typedef struct Comment_type
{
    int comment_id;     /* Canvas-Id des Kommentares */
    Point_type center;  /* Koordinaten des Kommentares */
    char *comment;	/* Text */
    Font_type *font;	/* Font des Kommentares */

} Comment_type;


/* 
 *  Es gibt zwei Typen von Unset-Aktionen:
 *  
 *    1. Ein Verschiebe-Vorgang wird rueckgaengig gemacht
 *    2. Ein gesetztes Element wird wieder geloescht
 */


/* 
 *  In UnsetTop_type werden alle Elemente einer Unset-Aktion in der
 *  Liste elementList einsortiert. Die Struktur vom Typ UnsetTop_type
 *  wird dann in der Unset-Liste der Struktur Editor_type, des aktuellen
 *  Editors, am Anfang eingef"ugt.
 */

enum Unset_action
{ move_element, remove_element };
typedef struct UnsetTop_type
{
    enum Unset_action action;	/* Unset-Aktion */
    List_type *elementList;     /* Liste der Unset-Elemente */

} UnsetTop_type;


/* 
 *  In Unset_type werden die Daten eines Elementes einer Unset-Aktion
 *  gespeichert. Es kann sich bei den Elementen um Knoten, Kanten,
 *  Kommentare und Kantentext handeln.
 */

enum Unset_element
{ place, edge, comment, edge_text };
typedef struct Unset_type
{
    enum Unset_element element; /* Typ des Elementes */
    int id;			/* Id des Elementes */
    Point_type last_point;      /* Alte Koordinaten des Elementes */
} Unset_type;


/* 
 *  Jedes Element eines L"oschvorganges wird in die ihm zugeh"orige
 *  Liste der Struktur vom Typ Undelete_type einsortiert. 
 *  Die Struktur vom Typ Undelete_type wird dann in der Unset-Liste der
 *  Struktur Editor_type, des aktuellen Editors, am Anfang eingef"ugt.
 */

typedef struct Undelete_type
{
    List_type *killedVertexList;  /* Liste der gel"oschten Knoten */
    List_type *killedEdgeList;    /* Liste der gel"oschten Kanten */
    List_type *killedCommentList; /* Liste der gel"oschten Kommentare */
} Undelete_type;


/*
 *  In der Struktur Copy_type werden alle mit 'copy' oder 'cut' 
 *  kopierten Elemente zwischengespeichert.
 */

typedef struct Copy_type
{
    Point_type area_start ;	/* Bezugspunkt f"ur das sp"atere Ein- */
				/* setzen                             */
    List_type *copyVertexList;	/* kopierte Knotenelemente */
    List_type *copyEdgeList;    /* kopierte Kantenelemente */  
    List_type *copyCommentList; /* kopierte Kommentarelemente */
} Copy_type;



/*
 ##########################################################################
 #                                                                        #
 #  Die globalen Variablen                                                #
 #                                                                        #
 ##########################################################################
 */

extern int vertexRad;		  /* Radius der Knoten                  */

extern Copy_type *copyObject;     /* Zeiger auf den Zwischenspeicher    */

extern int maxUndeleteCount;	  /* Anzahl der max. Undelete-Schritte  */

extern int maxUnsetCount;         /* Anzahl der max. Unset-Schritte     */

#endif /* pfa_h */


