/*
###########################################################################
###########################################################################
##                                                                       ##
##  Datei   :    `pfaFile.c'                                             ##
##               -----------                                             ##
##                                                                       ##
###########################################################################
###########################################################################
*/

#include <string.h>

#include "symtab.h"
#include "pfaFile.h"
#include "pfa.h"
#include "mnet.h"
#include "macros.h"
#include "multiset.h"
#include "var_mset.h"
#include "compose.h"
#include "vars.h"
#include "evaltree.h"
#include "decls.h"
#include "tables.h"
#include "block.h"
#include "main.h"

/*  lineStart
 *--------------------------------------------------------------
 *   
 *  "uberliest die nicht relevanten Zeichen einer Zeile
 *  des zu ladenden Files. Ein Zeiger auf den relevanten
 *  Text wird zur"uckgegeben.
 *
 *--------------------------------------------------------------
 */

void lineStart(register char *str, char **line, unsigned *line_number)
{
  for (;;) {
    switch(*str)
      {
	case '\n':
		(*line_number)++;
	case ' ' :
	case '\t':
		str++;
		break;
	case '%':		/* Kommentare uebergehen */
		for (str++; *str && *str != '\n'; str++);
		break;
	default:
		/* Zeiger auf den Anfang einer relevanten Zeile */
		*line = str;
		return;
      }
  }
}


/*  lineEnd
 *--------------------------------------------------------------
 *   
 *  "uberpr"uft das Ende einer eingelesenen Zeile auf nicht
 *  erlaubte Zeichen. Es d"urfen nur noch '\0', '\n' und 
 *  Kommentare vorkommen.
 *
 *--------------------------------------------------------------
 */

int lineEnd(register char *str, char **rest)
{
	while (*str == ' ' || *str == '\t')
		str++;

	/* das Ende der Zeile */
	*rest = str;
	return *str && *str != '\n' && *str != '%';
}


/*  getString
 *--------------------------------------------------------------
 *   
 *  gibt einen eingelesenen String zur"uck.
 *
 *--------------------------------------------------------------
 */

int getString(register char *str, char **rest,
	      char **name, unsigned *line_number) 
{
	char     *string_start;
	char     *string;
	unsigned  string_count;

	if (*str != '"') return 1;
	str++;
    
	string_start = str;
	for (string_count = 0; *str != '"'; string_count++, str++)
	{
		if (!*str) return 1;
		if (*str == '\n') (*line_number)++;
	}
    
	*name = string = MYmalloc(sizeof(char) * (string_count + 1));
	strncpy(string, string_start, string_count);
	string[string_count] = '\0'; 

	*rest = ++str;
	return 0;
}


/*  getNumber
 *--------------------------------------------------------------
 *  gibt eine eingelesene Nummer zur"uck
 *--------------------------------------------------------------
 */

int getNumber(register char *str, char **line, int *id)
{
	char     *string_start;  
	char     *string;  
	unsigned  string_count;

    
	while (*str == ' ') str++;
	string_start = str;
	for (string_count = 0; *str != ' '; string_count++, str++)
		if (!*str) return 1;
	
	string = MYmalloc(sizeof(char) * (string_count + 1));
	strncpy(string, string_start, string_count);
	string[string_count] = '\0';

	*id = atoi(string);

	free(string);
	*line = str;
	return 0;
}


/*  getXY
 *--------------------------------------------------------------
 *   
 *  gibt ein eingelesenes Koordinatenpaar zur"uck.
 *
 *--------------------------------------------------------------
 */

int getXY(register char *str, char **line, int *x, int *y)
{
	char     *string_start;    
	char     *string;    
	unsigned  string_count;

	string_start = str;
	for (string_count = 0; *str != '@'; string_count++, str++)
		if (!*str) return 1;

	string = malloc(sizeof(char) * (string_count + 1));
	strncpy(string, string_start, string_count);
	string[string_count] = '\0'; 

	*x = atoi(string);
	free(string);

	/* @ uebergehen */
	str++;
    
	string_start = str;
	for (string_count = 0; *str != ' '; string_count++, str++)
		if (!*str) return 1;

	string = malloc(sizeof(char) * (string_count + 1));
	strncpy(string, string_start, string_count);
	string[string_count] = '\0';

	*y = atoi(string);
	free(string);

	*line = str;
	return 0;

}


/*  getOldStart_End
 *--------------------------------------------------------------
 *   
 *  gibt ein eingelesenes Koordinatenpaar zur"uck.
 *
 *--------------------------------------------------------------
 */

int getOldStart_End(register char *str, char **rest, int *start, int *end)
{
	char     *string_start;    
	char     *string;    
	unsigned  string_count;

	string_start = str;
	for (string_count = 0; *str != '>'; string_count++, str++)
		if (!*str) return 1;

	string = malloc(sizeof(char) * (string_count + 1));
	strncpy(string, string_start, string_count);
	string[string_count] = '\0'; 
    
	*start = atoi(string);
	free(string);
    
	/* > uebergehen */
	str++;
    
	string_start = str;
	for (string_count = 0; *str != ' '; string_count++, str++)
		if (!*str) return 1;

	string = malloc(sizeof(char) * (string_count + 1));
	strncpy(string, string_start, string_count);
	string[string_count] = '\0';

	*end = atoi(string);
	free(string);

	*rest = str;
	return 0;

}

/*****************************************************************************/

t_pfahdr PH_Create()
{
	t_pfahdr mt = MYmalloc(sizeof(t_pfahdr_elem));
#if defined(sun)
	mt->count = 0;
	mt->realsize = 4;
	mt->place = MYmalloc(sizeof(char*) * mt->realsize);
#else
	mt->count = mt->realsize = 0;
	mt->place = NULL;
#endif
	return mt;
}

void PH_AddValue(t_pfahdr mt, char* what)
{
	if (mt->count >= mt->realsize)
	{
		mt->realsize += 4;
		mt->place = MYrealloc(mt->place,
				sizeof(char*) * mt->realsize);
	}
	mt->place[mt->count++] = what;
}

/*****************************************************************************/

void setmax(M_Net net, int cx, int cy)
{
	if (cx < net->x_pos) net->x_pos = cx;
	if (cy < net->y_pos) net->y_pos = cy;
	if (cx+RASTER > net->x_dim) net->x_dim = cx+RASTER;
	if (cy+RASTER > net->y_dim) net->y_dim = cy+RASTER;
}

/*
 ##########
 # LoadFA #
 ##########
*/


/*  tk_LoadFA
 *--------------------------------------------------------------
 *   
 *  l"adt den zum "ubergebenen Namen geh"orenden FA und 
 *  erstellt diesen auf der Zeichenfl"ache.
 *
 *--------------------------------------------------------------
 */

M_Net read_fa(char *faname)
{
	static int	 blockcounter = 0;
	char		 buffer[LINE_LENGTH];
	char		*rest;
	char		*line;
	char		*code;
	char		*font;
	char		*variables;
	unsigned	 line_number = 1;
	FILE		*fp;
	int		 used, alloced;
	char		*c, *fbuf;

	M_Net		 net;
	M_Place		 pl = NULL;
	M_Transition	 tr = NULL;
	M_Place          startplace, exitplace;
	M_Transition	 starttrans, exittrans;
	M_STEdge	 st = NULL;
	M_TSEdge	 ts = NULL;
	BlockNr		 bl;
	int 		 plident = 0;
	int 		 trident = 0;
	char		 pep_ver = 1;
	int		 brefnr;
	decl_type	 decls;


	fp = fopen(faname, "r");
	if (!fp)
	{
		fprintf(stderr,"could not open %s\n",faname);
		exit(1);
	}

	used = 0;
	c = fbuf = MYmalloc(alloced = 512);

	while (fgets(buffer,LINE_LENGTH,fp))
	{
		if (used+strlen(buffer) >= alloced)
		{
			fbuf = MYrealloc(fbuf, alloced += 512);
			c = fbuf + used;
		}
		strcpy(c,buffer);
		c += strlen(buffer);
		used = c - fbuf;
	}

	fclose (fp);
	rest = fbuf;

	/*  Zunaechst wird getestet, ob die Einleitung dem neuen
	    PEP-Dateiformat entspricht				 */

	lineStart(rest, &line, &line_number); 
	if (strncmp(line, "PEP", 3)) goto error;
	if (lineEnd(line += 3, &line)) goto error;
  
	lineStart(line, &line, &line_number);
	if (strncmp(line, "FiniteAutomaton", 15)) goto error;
	if (lineEnd(line += 15, &line)) goto error;
  
	lineStart(line, &line, &line_number);
	if (strncmp(line, "FORMAT_N", 8)) goto error;
	if (line[8] == '2') pep_ver = 2;
	if (lineEnd(line += (8+(line[8] == '2')), &line)) goto error;
	if (pep_ver == 1)
		fprintf(stderr,"warning: file %s has old format; "
		    "references will be bogus.\n"
		    "please load the project in the PFA editor and save it\n",
		    faname);
  
	net = M_empty_net();
	net->NumComponents = 1;

	/* Startstelle erzeugen */
	startplace = M_fast_insert_place(net,++plident,&pl);
	M_CopySingleRange(pl->range,macro_id_set);
	pl->range[1] = NULL;
	pl->label = ENTRY;
	setmax(net,pl->x_pos,pl->y_pos);

	starttrans = M_fast_insert_transition(&net->NumTransitions,
	      &net->Transitions, &net->LastTransition, ++trident, &tr, False);
	tr->label = MScreate_empty_set();
	tr->guard = MYmalloc(sizeof(node));
	tr->guard->typ = N_HEAD;
	tr->guard->op1 = 0;
	tr->guard->op2 = 1;
	tr->def = MYmalloc(2*sizeof(Integer));
	tr->def[0] = 2;
	tr->def[1] = -1;
	tr->x_pos = 100;
	Create_empty_Variables(&tr->vars,1);
	setmax(net,tr->x_pos,tr->y_pos);

	M_fast_insert_PT_edge(net->Places, net->Transitions,
		pl->numname, tr->numname, &pl, &tr, &st, False);
	st->label = V_MScreate_singleton(0);

	/* Exitstelle erzeugen */
	exitplace = M_fast_insert_place(net,++plident,&pl);
	M_CopySingleRange(pl->range,macro_id_set);
	pl->range[1] = NULL;
	pl->label = EXIT;
	setmax(net,pl->x_pos,pl->y_pos);

	exittrans = M_fast_insert_transition(&net->NumTransitions,
	      &net->Transitions, &net->LastTransition, ++trident, &tr, False);
	tr->label = MScreate_empty_set();
	tr->guard = MYmalloc(sizeof(node));
	tr->guard->typ = N_HEAD;
	tr->guard->op1 = 0;
	tr->guard->op2 = 1;
	tr->def = MYmalloc(2*sizeof(Integer));
	tr->def[0] = 2;
	tr->def[1] = -1;
	Create_empty_Variables(&tr->vars,1);
	setmax(net,tr->x_pos,tr->y_pos);
	
	M_fast_insert_TP_edge(net->Places, net->Transitions,
		pl->numname, tr->numname, &pl, &tr, &ts, False);
	ts->label = V_MScreate_singleton(0);

	/* Danach wird der Knoten-Font eingeleitet durch DPL angegeben. 
	   Falls DPL in der Datei auftaucht, muss in der naechsten Zeile 
	   auch der Font angegeben werden. Es besteht auch die Moeglichkeit 
	   beides wegzulassen.						    */

	lineStart(line, &line, &line_number);
	if (!strncmp(line, "DPL", 3))
	{	
		if (lineEnd(line += 3, &line)) goto error;
	
		/* Der Font wird gelesen ... */

		lineStart(line, &line, &line_number);
		if (getString(line, &line, &font, &line_number)) goto error;
		if (lineEnd(line, &line)) goto error;

		lineStart(line, &line, &line_number);

	}

	/* Dann wird der Kanten-Font eingeleitet durch DAR angegeben. Falls 
	   DAR in der Datei auftaucht, muss in der naechsten Zeile  der Font 
	   angegeben werden. Es besteht auch die Moeglichkeit beides wegzu-
	   lassen.							    */

	if (!strncmp(line, "DAR", 3))
	{	
		if (lineEnd(line += 3, &line)) goto error;

		/* Der Font wird gelesen ... */

		lineStart(line, &line, &line_number);
		if (getString(line, &line, &font, &line_number)) goto error;
		if (lineEnd(line, &line)) goto error;

		lineStart(line, &line, &line_number);

	}

	if (strncmp(line, "PL", 2)) goto error;
	if (lineEnd(line += 2, &line)) goto error;

	/* Eingeleitet durch PL folgen die gesamten Knoten-Informationen */

	lineStart(line, &line, &line_number);    
	while (strncmp(line, "AR", 2))
	{ 
		int id;

		M_fast_insert_place(net, ++plident, &pl);
		M_CopySingleRange(pl->range,macro_id_set);
		pl->range[1] = NULL;
		pl->label = EMPTY;

		/* Es wird eine neue Knotenstruktur erstellt und dann zunaechst
		   die alte Canvas-Id eingelesen, um spaeter die Kanten richtig 
		   zuordnen zu koennen.					      */

		if (getNumber(line, &line, &id)) goto error;
		pl->component = id;
		while (*line == ' ' || *line == '\t') line++;
      
		/* Dann folgt der Knoten-Mittelpunkt, ... */
		if (getXY(line, &line, &pl->x_pos, &pl->y_pos)) goto error;
		while (*line == ' ' || *line == '\t') line++;
		setmax(net,pl->x_pos,pl->y_pos);

		/* der Typ des Knotens (Start-, normaler oder Endknoten) ... */
		if (*line < '1' || *line > '3') goto error;
		
		if (*line == '1')	/* Startknoten */
		{
		    M_fast_insert_TP_edge(net->Places, net->Transitions,
					  pl->numname, starttrans->numname,
					  &pl, &starttrans, &ts, False);
		    ts->label = V_MScreate_singleton(0);
		}
		else if (*line == '3')	/* Endknoten */
		{
		    M_fast_insert_PT_edge(net->Places, net->Transitions,
					  pl->numname, exittrans->numname,
					  &pl, &exittrans, &st, False);
		    st->label = V_MScreate_singleton(0);
		}

		do { line++; } while (*line == ' ' || *line == '\t');

		/* und zum Schluss die Beschriftung. */
		if (getString(line, &line, &font, &line_number)) goto error;
		s_create(font);
		s_start->s_type = _BEGIN;
		s_start->s_ref.end_c = pl->component << 16;
		flag |= SYNCHRONIZE;
/* added after 1.8e */
                pl->RefEntries = 1;
                pl->ref = MYmalloc(sizeof(Ref));
		pl->ref->nr = StructToRef((ShortEntryPtr)s_start);
                pl->ref->flag = 0;
/* */
		flag &= ~SYNCHRONIZE;
		s_start = s_start->s_next;

		if (lineEnd(line, &line)) goto error;
		lineStart(line, &line, &line_number);    

	}

	/* Die AR-Zeile fuer die Kanten muss uebergangen werden */
	if (lineEnd(line += 2, &line)) goto error;

	/* Dann folgen die gesamten Kanten-Informationen. */
	lineStart(line, &line, &line_number);    
	while (strncmp (line, "TX", 2))
	{
		int arcid = 1, start, end;

		M_fast_insert_transition(&net->NumTransitions,
				&net->Transitions, &net->LastTransition,
				++trident, &tr, False);

		/* Es wird eine neue Knotenstruktur erstellt und dann zunaechst
		   die alte Id's der Anfangs- und Endknoten eingelesen.	      */
		if (pep_ver == 2)  /* neue Kantenid */
			if (getNumber(line, &line, &arcid)) goto error;
		tr->meaning = (char*) arcid;
		if (getOldStart_End(line, &line, &start, &end)) goto error;
		while (*line == ' ' || *line == '\t') line++;
      
		for (pl = net->Places; pl; pl = pl->nextplace)
			if (pl->component == start) break;
		if (!pl) goto error;
		M_fast_insert_PT_edge(net->Places, net->Transitions,
				      pl->numname, tr->numname,
				      &pl, &tr, &st, False);
		st->label = V_MScreate_singleton(0);

		for (pl = net->Places; pl; pl = pl->nextplace)
			if (pl->component == end) break;
		if (!pl) goto error;
		M_fast_insert_TP_edge(net->Places, net->Transitions,
				      pl->numname, tr->numname,
				      &pl, &tr, &ts, False);
		ts->label = V_MScreate_singleton(0);

		/* Dann folgt der Mittelpunkt der Kante, ... */
		if (getXY(line, &line, &tr->x_pos, &tr->y_pos)) goto error;
		while (*line == ' ' || *line == '\t') line++;
		setmax(net,tr->x_pos,tr->y_pos);

		/* ... die Differenz mit der der Text abgesetzt ist ... */ 
		if (getXY(line, &line, &start, &end)) goto error;
		while (*line == ' ' || *line == '\t') line++;

		/* ... und schliesslich die Kantenbeschriftung. */
		if (getString(line, &line, &rest, &line_number)) goto error;
		if (lineEnd(line, &line)) goto error;
		tr->def = MYmalloc(strlen(rest)+2);
		strcpy((char*)tr->def,rest);
		free(rest);

		lineStart(line, &line, &line_number);    

	}

	/* Die TX-Zeile fuer die Kommentare muss uebergangen werden */
	if (strncmp(line, "TX", 2)) goto error;
	if (lineEnd(line += 2, &line)) goto error;
  
	/* Die Art der Kantenbeschriftung wird eingelesen */
	lineStart(line, &line, &line_number);    
	if (getString(line, &line, &code, &line_number)) goto error;
	if (lineEnd(line, &line)) goto error;
	if (strncmp(code,"Struct",6) && strncmp(code,"B(PN)2",6)) goto error;

	/* Die Variablen werden eingelesen */
	lineStart(line, &line, &line_number);    
	if (getString(line, &line, &variables, &line_number)) goto error;
	if (lineEnd(line, &line)) goto error;

	net->x_dim -= net->x_pos; /* korrigiere x_max zu x_dim */
	net->y_dim -= net->y_pos; /* korrigiere y_max zu y_dim */
	exittrans->x_pos = net->x_dim - 100;
	net->Places->nextplace->x_pos = net->x_dim;

	for (pl = net->Places; pl; pl = pl->nextplace) pl->component = 0;

	parse_vars(variables);

	for (tr = net->Transitions->nexttrans; (tr = tr->nexttrans); )
	{
		RefNr ref;

		strptr = (char*) tr->def;
		strcat(strptr,";");
		yyleng = 0;
		yyparse();
		if (ParseType!=1) yyerror("got vardecls instead of expression");

		RenameRefPars(ProcId->s_parlist,ProcId->s_name);
		init_ExpressionParser(NULL);
		HandleExpr(ExprTree->op1);
		exit_ExpressionParser();
		RenameBack(ProcId->s_parlist);

		ref = StructToRef((ShortEntryPtr) ExprTree->op2);
		tr->label = action_set;
		tr->guard = expr_tree;
		tr->def = expr_def;
		tr->vars.ComEntries = expr_vars.ComEntries;
		tr->vars.ComArray = expr_vars.ComArray;
		tr->RefEntries = !!ref;
		RefToStruct(ref)->s_ref.end_c |= ((int) tr->meaning) << 16;
		if (!tr->guard)
		{
			tr->guard = MYmalloc(sizeof(node));
			tr->guard->typ = N_HEAD;
			tr->guard->op1 = 0;
			tr->guard->op2 = 1;
		}
		if (!tr->def)
		{
			tr->def = MYmalloc(2*sizeof(Integer));
			tr->def[0] = 2;
			tr->def[1] = -1;
		}
		if (tr->RefEntries)
		{
			tr->ref = MYmalloc(sizeof(Ref));
			tr->ref->nr = ref;
			tr->ref->flag = 0;
		}
		tr->meaning = NULL;

		pl = tr->preset->place;
		if (pl->RefEntries++)
			pl->ref = MYrealloc(pl->ref,pl->RefEntries*sizeof(Ref));
		else
			pl->ref = MYmalloc(sizeof(Ref));
		pl->ref[pl->RefEntries-1].nr = ref;
		pl->ref[pl->RefEntries-1].flag = PRE_REF;
		
		pl = tr->postset->place;
		if (pl->RefEntries++)
			pl->ref = MYrealloc(pl->ref,pl->RefEntries*sizeof(Ref));
		else
			pl->ref = MYmalloc(sizeof(Ref));
		pl->ref[pl->RefEntries-1].nr = ref;
		pl->ref[pl->RefEntries-1].flag = POST_REF;

		Modify_Transition(tr);

	}

	flag |= SYNCHRONIZE; /* damit StructToRef einen Eintrag erzeugt */
	s_create(faname);
	give_typ(_BLOCKNAME,NULL,0);

	add_block(s_start,&bl,brefnr = StructToRef((ShortEntryPtr)s_start),0);
	s_start->s_type = _BLOCKNAME;
	s_start->s_ref.end_c = ++blockcounter << 16;
	M_AddAllPlacesBlock(net,bl);
	M_AddAllTransBlock(net,bl);
	M_AppendBlock(net,bl);
	flag &= ~SYNCHRONIZE;
/* added after 1.8e */
        startplace->RefEntries = 1;
        startplace->ref = MYmalloc(sizeof(Ref));
        startplace->ref->nr = brefnr;
        startplace->ref->flag = PRE_REF;
  
        exitplace->RefEntries = 1;
        exitplace->ref = MYmalloc(sizeof(Ref));
        exitplace->ref->nr = brefnr;
        exitplace->ref->flag = POST_REF;
/* */

	decls = handle_vars();
	net = Scope(net,decls);

	/* Dann folgen die gesamten Informationen der Kommentare, 
	   falls vorhanden.					  */

	lineStart(line, &line, &line_number);    
	while (*line)
	{ 
		int dummy1, dummy2;
	
		/* Es wird eine neue Kommentarstruktur und Fontstruktur erstellt
		   und dann zunaechst der Font des Kommentars eingelesen      */

		if (getString(line, &line, &rest, &line_number)) goto error;
		while (*line == ' ' || *line == '\t') line++;

		/* Dann folgt der Mittelpunkt des Kommentars, ... */
		if (getXY(line, &line, &dummy1, &dummy2)) goto error;
		while (*line == ' ' || *line == '\t') line++;

		/*  ... und schliesslich der Kommentar selbst. */
		if (getString(line, &line, &rest, &line_number)) goto error;
		if (lineEnd(line, &line)) goto error;

		lineStart(line, &line, &line_number);    

	}
  
	/* Der String in dem das File gespeichert war kann geloescht werden */
	free(fbuf);
  
	return net;

	/* Falls beim Einlesen ein Fehler aufgetreten ist, wird an diese
	   Stelle verzweigt 						 */
 
error: 
	fprintf(stderr,"error at line number %d in file %s\n",
		line_number,faname);
	exit(1);

}

t_pfahdr read_prj(char *prjname)
{
	char		 buffer[LINE_LENGTH];
	char		*rest;
	char		*line;
	char		*tmp;
	unsigned	 line_number = 1;
	FILE		*fp;
	int		 used, alloced;
	char		*c, *fbuf;
	t_pfahdr	result;

	fp = fopen(prjname, "r");
	if (!fp)
	{
		fprintf(stderr,"could not open project file %s\n",prjname);
		exit(1);
	}

	used = 0;
	c = fbuf = MYmalloc(alloced = 512);
	result = PH_Create();

	while (fgets(buffer,LINE_LENGTH,fp))
	{
		if (used+strlen(buffer) >= alloced)
		{
			fbuf = MYrealloc(fbuf, alloced += 512);
			c = fbuf + used;
		}
		strcpy(c,buffer);
		c += strlen(buffer);
		used = c - fbuf;
	}

	fclose (fp);
	rest = fbuf;

	/*  Zunaechst wird getestet, ob die Einleitung dem neuen
	    PEP-Dateiformat entspricht				 */

	lineStart(rest, &line, &line_number); 
	if (strncmp(line, "PEP", 3)) goto error;
	if (lineEnd(line += 3, &line)) goto error;
  
	lineStart(line, &line, &line_number);
	if (strncmp(line, "PROJECT_pepPFA", 14)) goto error;
	if (lineEnd(line += 14, &line)) goto error;
  
	lineStart(line, &line, &line_number);
	if (strncmp(line, "VAR", 3)) goto error;
	if (lineEnd(line += 3, &line)) goto error;
  
	lineStart(line, &line, &line_number);
	tmp = line;
	while (strncmp(line, "FA", 2))
	{
		while (*line && *line != '\n') line++;
		if (lineEnd(line, &line)) goto error;
		lineStart(line, &line, &line_number);    
	}
	*line = '\0';
	result->variables = MYstrdup(tmp);

	line += 2;
	while (!lineEnd(line, &line))
	{
		lineStart(line, &line, &line_number);    
		if ((tmp = strchr(line,'\n'))) *tmp = '\0';
		PH_AddValue(result, MYstrdup(line));
		if (tmp) *(line = tmp) = '\n'; else break;
	}

	free(fbuf);
	return result;

error: 
	fprintf(stderr,"project file: error at line number %d",line_number);
	exit(1);

}

