#include <stdio.h>
#include "symtab.h"
#include "function.h"
#include "mnet.h"
#include "evaltree.h"
#include "tables.h"
#include "block.h"

void print_var_range(FILE *f, ShortEntryPtr ptr, bool twodim)
{
	Integer *range;

	fprintf(f,"\"");
	if (twodim) fprintf(f,"{%u}",((SymTabEntry*)ptr)->s_init_or_cap);
	range = Set_Parser2Intern(((SymTabEntry*)ptr)->s_set);
	PrintSingleRange(range,f);
	fprintf(f,"\"");
	free(range);
}

void print_ref_meaning(FILE *f, RefNr i)
{
	fprintf(f,"b\"");
	PrintMeaning(i,0,f);
	fprintf(f,"\"");
}

void output_refs(FILE *f, char *type, int numname, int RefEntries, Ref *ref)
{
	int i;

	if (!RefEntries) return;
	fprintf(f,"%s%u ",type,numname);
	for (i = 0; i < RefEntries; i++)
		fprintf(f,"%s%u:%u",i? "*" : "T\"",ref[i].nr,ref[i].flag);
	fprintf(f,"\"\n");
}

void print_pfa_refs(M_Net net, char *filename)
{
	ShortEntryPtr ptr;
	M_Place pl;
	M_Block bl;
	M_Transition tr;
	FILE *f = fopen(filename,"w");
	RefNr i;

	if (!f)
	{
		fprintf(stderr,"could not open file %s for writing",filename);
		exit(3);
	}

	printf("Writing references to file %s...\n",filename);

	fprintf(f,"PEP\nReferences_PFA_HL\nFORMAT_N2\nDEF\n");
	for (i = 1; (ptr = RefToStruct(i)); i++)
	{
		fprintf(f,"%u \"%u\"",i,ptr->s_ref.end_c >> 16);
		ptr->s_ref.end_c &= 0xffff;
		switch (ptr->s_type)
		{
		    case _CHANNEL:
			fputc('C',f);
			print_var_range(f,ptr,True);
			print_ref_meaning(f,i);
			break;
		    case _STACK:
			fputc('S',f);
			print_var_range(f,ptr,True);
			print_ref_meaning(f,i);
			break;
		    case _VAR:
			fprintf(f,"V");
			print_var_range(f,ptr,False);
			print_ref_meaning(f,i);
			break;
		    case _BLOCKNAME:
			fprintf(f,"E\"0\"b\"%s\"",((SymTabEntry*)ptr)->s_name);
			break;
		    case _ACTION:
			fprintf(f,"A");
			print_ref_meaning(f,i);
			break;
		    case _BEGIN:	/* Kante */
			fprintf(f,"Kb\"%s\"",((SymTabEntry*)ptr)->s_name);
			break;
		}
		fprintf(f,"\n");
	}

	if (flag & BLOCK_OUTPUT)
	{
		fprintf(f,"BL\n");
		for (bl = net->Blocks; bl; bl = bl->nextblock)
			fprintf(f,"B%u T\"%u:0\"\n",bl->blocknr,
				BlockToStruct(bl->blocknr)->ref);
	}

	fprintf(f,"PL\n");
	for (pl = net->Places; pl; pl = pl->nextplace)
		output_refs(f,"P",pl->numname,pl->RefEntries,pl->ref);

	fprintf(f,"TR\n");
	for (tr = net->Transitions; tr; tr = tr->nexttrans)
		output_refs(f,"T",tr->numname,tr->RefEntries,tr->ref);

	if (flag & PHANTOM_TRANS_OUTPUT)
	{
		fprintf(f,"PTR\n");
		for (tr = net->PTransitions; tr; tr = tr->nexttrans)
			output_refs(f,"PT",tr->numname,tr->RefEntries,tr->ref);
	}

	fclose(f);
}
