#ifndef pfa_h
#define pfa_h

/*
###########################################################################
###########################################################################
##                                                                       ##
##  Datei   :    `pfaOptions.c'                                          ##
##               --------------                                          ##
##                                                                       ##
##  Inhalt  :    Hilfsfunktionen, die von mehreren Befehlsfunktionen     ##
##               aus den anderen Files ben"otigt werden.                 ##
##                                                                       ##
###########################################################################
###########################################################################
*/

#include <tk.h>
#include "pfaMultList.h"


#define MALLOC(x)   ((x *) Tcl_Alloc( sizeof(x) ))	
#define FREE(x)     Tcl_Free( x )
#define LINE_LENGTH  1024

#if (TCL_MAJOR_VERSION == 8) && (TCL_MINOR_VERSION == 0) /* Tcl8.0 stuff */
#define Tcl_GetString(objPtr)	Tcl_GetStringFromObj(objPtr, (int *)NULL)
#endif

/* This EXTERN declaration is needed for Tcl < 8.0.3 */
#ifndef EXTERN
# ifdef __cplusplus
#  define EXTERN extern "C"
# else
#  define EXTERN extern
# endif
#endif

#ifdef TCL_STORAGE_CLASS
# undef TCL_STORAGE_CLASS
#endif
#ifdef BUILD_pfa
# define TCL_STORAGE_CLASS DLLEXPORT
#else
# define TCL_STORAGE_CLASS DLLIMPORT
#endif

#ifdef WIN32
#   define WIN32_LEAN_AND_MEAN
#   include <windows.h>
#   undef WIN32_LEAN_AND_MEAN
/* VC++ has an entry point called DllMain instead of DllEntryPoint */
#   if defined(_MSC_VER)
#	define DllEntryPoint DllMain
#   endif
#endif

/*
 ##########################################################################
 #                                                                        #
 #  Deklaration der Datenstrukturen                                       #
 #                                                                        #
 ##########################################################################
 */

typedef struct Vertex_type Vertex_type;
typedef struct Point_type {
    int x,y;
} Point_type;

/* 
 *  In EdgeMove_type wird die Kurvenform der Kanten gespeichert, um
 *  die Kanten beim verschieben der entsprechenden Knoten schneller
 *  aktualisieren zu k"onnen. 
 */

typedef struct EdgeMove_type {
    double cos_alpha;
    double sin_alpha;
    double fraction;
} EdgeMove_type;


/* 
 *  In Edge_type werden alle ben"otigten Informationen einer Kante
 *  gespeichert. 
 */

typedef struct Edge_type
{
    int start;			/* Canvas-Id des Ausgangsknotens */
    int end;			/* Canvas-Id des Zielknotens */
    int e_id;                   /* Canvas-Id der Kante */
    int label_id;               /* Canvas-Id der Kantenbeschriftung */
    char *label;                /* Kantenbeschriftung */
    Point_type center;		/* Mittelpunkt der Kante */
    Point_type label_center;    /* Differenz des Textes vom Mittelpunkt */ 
    EdgeMove_type move_infos;   /* Informationen zur Form der Kante  */ 
    int save_id;                /* Kanten-Id fuers Speichern */
    int editor_num;             /* Nummer des zugehoerigen Editors/Netzes */
} Edge_type;


/* 
 *  Jede Knotenstruktur vom Type Vertex_type enth"alt drei Listen vom 
 *  Typ EdgeList_type, in denen alle zu dem Knoten geh"orenden Kanten 
 *  einsortiert werden.
 */

typedef struct EdgeList_type {
    int arc_id;
    Edge_type *edge_object;
    int other_pid;
    Vertex_type *other_vertex;
    struct EdgeList_type *next;
} EdgeList_type;


/* 
 *  In Vertex_type werden alle ben"otigten Informationen eines Knotens
 *  gespeichert. 
 */

struct Vertex_type {
    int v_id;			/* Canvas-Id des Knotens */
    int type;			/* Typ des Knotens */
    int label_id;               /* Canvas-Id des Knotentextes */
    char *label;                /* Kantenbeschriftung */
    Point_type center;          /* Mittelpunkt der Kante */ 
    EdgeList_type *in;          /* Liste aller eingehenden Kanten */ 
    EdgeList_type *out;         /* Liste aller ausgehenden Kanten */ 
    EdgeList_type *own_edge;    /* Liste aller Schlingen */ 
    int compile_id;             /* Information f"ur den Compiler */ 
    int save_id;                /* Knoten-Id fuers Speichern */
};


/* 
 *  Font_type enth"alt die Informationen zum Font eines Kommentares
 */

typedef struct Font_type
{
    char *family;
    char *weight;
    int slant;
    int size;

} Font_type;


/* 
 *  In Comment_type werden alle ben"otigten Informationen eines 
 *  Kommentares gespeichert. 
 */

typedef struct Comment_type
{
    int comment_id;     /* Canvas-Id des Kommentares */
    Point_type center;  /* Koordinaten des Kommentares */
    char *comment;	/* Text */
    Font_type *font;	/* Font des Kommentares */

} Comment_type;


/* 
 *  Es gibt zwei Typen von Unset-Aktionen:
 *  
 *    1. Ein Verschiebe-Vorgang wird rueckgaengig gemacht
 *    2. Ein gesetztes Element wird wieder geloescht
 */


/* 
 *  In UnsetTop_type werden alle Elemente einer Unset-Aktion in der
 *  Liste elementList einsortiert. Die Struktur vom Typ UnsetTop_type
 *  wird dann in der Unset-Liste der Struktur Editor_type, des aktuellen
 *  Editors, am Anfang eingef"ugt.
 */

enum Unset_action
{ move_element, remove_element };
typedef struct UnsetTop_type
{
    enum Unset_action action;	/* Unset-Aktion */
    List_type *elementList;     /* Liste der Unset-Elemente */

} UnsetTop_type;


/* 
 *  In Unset_type werden die Daten eines Elementes einer Unset-Aktion
 *  gespeichert. Es kann sich bei den Elementen um Knoten, Kanten,
 *  Kommentare und Kantentext handeln.
 */

enum Unset_element
{ place, edge, comment, edge_text };
typedef struct Unset_type
{
    enum Unset_element element; /* Typ des Elementes */
    int id;			/* Id des Elementes */
    Point_type last_point;      /* Alte Koordinaten des Elementes */
} Unset_type;


/* 
 *  Jedes Element eines L"oschvorganges wird in die ihm zugeh"orige
 *  Liste der Struktur vom Typ Undelete_type einsortiert. 
 *  Die Struktur vom Typ Undelete_type wird dann in der Unset-Liste der
 *  Struktur Editor_type, des aktuellen Editors, am Anfang eingef"ugt.
 */

typedef struct Undelete_type
{
    List_type *killedVertexList;  /* Liste der gel"oschten Knoten */
    List_type *killedEdgeList;    /* Liste der gel"oschten Kanten */
    List_type *killedCommentList; /* Liste der gel"oschten Kommentare */
} Undelete_type;


/* 
 *  Die Struktur vom Typ Editor_type enth"alt die gesamte ben"otigte
 *  Information eines Editors. F"ur jeden neuen Editor wird eine solche
 *  Struktur angelegt und in der Editor-Liste eingef"ugt.
 */

typedef struct Editor_type
{
    int number ;		 /* Nummer des Editors */
    char *canvas;		 /* Name der zugeh"origen Zeichenfl"ache */
    Tcl_HashTable vertex_table;	 /* Hash-Table aller Knoten */
    Tcl_HashTable edge_table;    /* Hash-Table aller Kanten */
    Tcl_HashTable comment_table; /* Hash-Table aller Kommentare */
    int undeleteCount;		 /* Anzahl der Undelete-Schritte */
    List_type *undeleteList;	 /* Undelete-Liste */
    int unsetCount;              /* Anzahl der Unset-Schritte */
    List_type *unsetList;        /* Unset-Liste */
    struct Editor_type *next;
} Editor_type;


/* 
 *  In der Struktur Copy_type werden alle mit 'copy' oder 'cut' 
 *  kopierten Elemente zwischengespeichert.
 */

typedef struct Copy_type
{
    Point_type area_start ;	/* Bezugspunkt f"ur das sp"atere Ein- */
				/* setzen                             */
    List_type *copyVertexList;	/* kopierte Knotenelemente */
    List_type *copyEdgeList;    /* kopierte Kantenelemente */  
    List_type *copyCommentList; /* kopierte Kommentarelemente */
} Copy_type;



/*
 ##########################################################################
 #                                                                        #
 #  Die globalen Variablen                                                #
 #                                                                        #
 ##########################################################################
 */

extern int vertexRad;		  /* Radius der Knoten                  */

extern Editor_type *firstEd;	  /* Zeiger auf die erste Editor-       */
				  /* Struktur                           */

extern Editor_type *actualEd;     /* Zeiger auf die aktuelle Editor-    */
                                  /* Struktur                           */

extern Copy_type *copyObject;     /* Zeiger auf den Zwischenspeicher    */

extern int maxUndeleteCount;	  /* Anzahl der max. Undelete-Schritte  */

extern int maxUnsetCount;         /* Anzahl der max. Unset-Schritte     */

extern Tcl_HashTable id2cidTable; /* Hash-Table, in dem z.B. beim Laden */
				  /* von FA's die alte und neue         */ 
				  /* Canvas-Id der Knoten zwischenge-   */
				  /* speichert wird, um die Kanten      */
				  /* korrekt zuordnen zu k"onnen.       */

extern Tcl_HashTable NodeIds;
extern Tcl_HashTable EdgeIds;


/*
 *  Neues Canvas-Item
 */

extern Tk_ItemType  tkEndplaceType ; 



/*
 ##########################################################################
 #                                                                        #
 #  Anmeldung der global verwendeten Funktionen                           #
 #                                                                        #
 ##########################################################################
 */

EXTERN int Pfa_Init(Tcl_Interp* interp);

/*
 *  Funktionen aus pfaFile.c
 *  ------------------------
 */

extern int tk_NewFA( ClientData cd, Tcl_Interp * interp,\
			 int objc, Tcl_Obj *CONST objv[]);
extern int tk_LoadFA( ClientData cd, Tcl_Interp * interp,\
		       int objc, Tcl_Obj *CONST objv[]);
extern int tk_SaveFA( ClientData cd, Tcl_Interp * interp,\
		       int objc, Tcl_Obj *CONST objv[] );
extern int tk_DeleteFA( ClientData cd, Tcl_Interp * interp,\
			int objc, Tcl_Obj *CONST objv[]);


/*
 *  Funktionen aus pfaEdit.c
 *  ------------------------
 */

extern int tk_Cut(ClientData cd, Tcl_Interp * interp,\
		  int objc, Tcl_Obj *CONST objv[]);
extern int tk_Copy(ClientData cd, Tcl_Interp * interp,\
		   int objc, Tcl_Obj *CONST objv[]);
extern int tk_Paste(ClientData cd, Tcl_Interp * interp,\
		    int objc, Tcl_Obj *CONST objv[]);
extern int tk_Unset(ClientData cd, Tcl_Interp * interp,\
		    int objc, Tcl_Obj *CONST objv[]);
extern int tk_Undelete( ClientData cd, Tcl_Interp * interp,\
			int objc, Tcl_Obj *CONST objv[]);


/*
 *  Funktionen aus pfaModes.c
 *  -------------------------
 */

extern int tk_VertexInfos(ClientData cd, Tcl_Interp * interp,\
			 int objc, Tcl_Obj *CONST objv[]);
extern int tk_NormalEdgeInfos(ClientData cd, Tcl_Interp * interp,\
			     int objc, Tcl_Obj *CONST objv[]);
extern int tk_OwnEdgeInfos(ClientData cd, Tcl_Interp * interp,\
			  int objc, Tcl_Obj *CONST objv[]);
extern int tk_CommentInfos(ClientData cd, Tcl_Interp * interp,\
			    int objc, Tcl_Obj *CONST objv[]);
extern int tk_VertexDelete(ClientData cd, Tcl_Interp * interp,\
			  int objc, Tcl_Obj *CONST objv[]);
extern int tk_EdgeDelete(ClientData cd, Tcl_Interp * interp,\
			int objc, Tcl_Obj *CONST objv[]);
extern int tk_CommentDelete(ClientData cd, Tcl_Interp * interp,\
			    int objc, Tcl_Obj *CONST objv[]);
extern int tk_AreaDelete(ClientData cd, Tcl_Interp * interp,\
			 int objc, Tcl_Obj *CONST objv[]);
extern int tk_LineUpX(ClientData cd, Tcl_Interp * interp,\
		      int objc, Tcl_Obj *CONST objv[]);
extern int tk_LineUpY(ClientData cd, Tcl_Interp * interp,\
		      int objc, Tcl_Obj *CONST objv[]);
extern int tk_UnsetTextCoords(ClientData cd, Tcl_Interp * interp,\
			      int objc, Tcl_Obj *CONST objv[]);
extern int tk_ChangeEdgeTextCoords( ClientData cd, Tcl_Interp * interp,\
				   int objc, Tcl_Obj *CONST objv[]);
extern int tk_Rename(ClientData cd, Tcl_Interp * interp,\
		     int objc, Tcl_Obj *CONST objv[]);
extern int tk_ResetEdge( ClientData cd, Tcl_Interp * interp,\
			 int objc, Tcl_Obj *CONST objv[]);


/*
 *  Funktionen in pfaOptions.c
 *  --------------------------
 */

extern int tk_UpdateLists(ClientData cd, Tcl_Interp * interp,\
			  int objc, Tcl_Obj *CONST objv[]);

extern int tk_CommentSaveFont(ClientData cd, Tcl_Interp * interp,\
			      int objc, Tcl_Obj *CONST objv[]);


/*
 *  Funktion in pfaEditor.c
 *  -----------------------
 */

extern int tk_ChangeFA( ClientData cd, Tcl_Interp * interp,\
			int objc, Tcl_Obj *CONST objv[]);


/*
 *  Funktionen in pfaMove.c
 *  -----------------------
 */

extern int tk_UnsetVertexCoords(ClientData cd, Tcl_Interp * interp,\
			       int objc, Tcl_Obj *CONST objv[]);
extern int tk_VertexChange(ClientData cd, Tcl_Interp * interp,\
			  int objc, Tcl_Obj *CONST objv[]);
extern int tk_SaveEdgeAngle(ClientData cd, Tcl_Interp * interp,\
			    int objc, Tcl_Obj *CONST objv[]);
extern int tk_EdgeMove( ClientData cd, Tcl_Interp * interp,\
			int objc, Tcl_Obj *CONST objv[]);
extern int tk_Angle( ClientData cd, Tcl_Interp * interp,\
		     int objc, Tcl_Obj *CONST objv[]);
extern int tk_UnsetCommentCoords(ClientData cd, Tcl_Interp * interp,\
				 int objc, Tcl_Obj *CONST objv[]);
extern int tk_CommentChange(ClientData cd, Tcl_Interp * interp,\
			    int objc, Tcl_Obj *CONST objv[]);
extern int tk_UnsetAreaCoords(ClientData cd, Tcl_Interp * interp,\
			      int objc, Tcl_Obj *CONST objv[]);
extern int tk_AreaChange(ClientData cd, Tcl_Interp * interp,\
			 int objc, Tcl_Obj *CONST objv[]);


/*
 *  Funktionen in pfaSupport.c
 *  --------------------------------
 */
extern int printVertex( Tcl_Interp *interp, Vertex_type *vertexObject,\
			int *place_id, int *text_id);
extern int printEdge( Tcl_Interp *interp, Edge_type *edge_object,\
		      int *arc_id, int *text_id);
extern int printOwnEdge( Tcl_Interp *interp, Edge_type *edge_object,\
			 int *arc_id, int *text_id);
extern int printComment( Tcl_Interp *interp, Comment_type *comment_object,\
			 int *text_id);
extern int calculateAngles( Tcl_Interp *interp, int arc_id );
extern int calculateStartFraction( Tcl_Interp *interp, Point_type start,\
				   Point_type mid,Point_type end,\
				   double *fract);
extern int changeEdgePos( Tcl_Interp *interp, Edge_type *edge_object,\
			 Point_type p_start , \
			 Point_type p_end, int calc_mid);
extern int changeOwnEdgePos( Tcl_Interp *interp, Edge_type *edge_object,\
			     Point_type p_start);
extern int changeVertexPos(Tcl_Interp * interp, Vertex_type *vertexObject,\
		   Point_type new_center);
extern int edgeMove( Tcl_Interp * interp, int cid, Point_type mid);
extern int changeCommentPos(Tcl_Interp * interp,\
			    Comment_type *comment_object,\
			    Point_type point );
extern int changeTextPos( Tcl_Interp * interp, int cid,\
			  Point_type center );
extern int vertexDelete( Tcl_Interp * interp, int cid,\
			Undelete_type *undeleteElement);
extern int edgeDelete( Tcl_Interp * interp, int arc_cid,\
	       Undelete_type *undeleteElement);
extern int commentDelete( Tcl_Interp * interp, int cid,\
		   Undelete_type *undeleteElement);
extern Vertex_type *getVertexStruct(char *string);
extern int getVertexCoords( Tcl_Interp *interp, int place_cid,\
			    Point_type *center );
extern Edge_type *getEdgeStruct(char *string);
extern Comment_type *getCommentStruct(char *string);

extern int hashVertexInsert(Tcl_Interp *interp, Vertex_type *vertexObject);
extern int hashEdgeInsert( Tcl_Interp * interp, Edge_type *edge_object);
extern int hashCommentInsert( Tcl_Interp * interp,\
			      Comment_type *comment_object);
extern int EdgeToVertexList(Tcl_Interp *interp, Edge_type *edge_object );

extern UnsetTop_type *makeUnsetElement( enum Unset_action type );
extern void insertUnsetStruct(UnsetTop_type *partList_object,\
			      enum Unset_element type,\
			      int id, int x, int y);
extern void cleanUnsetList( int object_id ); 
extern void cleanUnsetWithoutDelete( int object_id ); 
extern Undelete_type *makeUndeleteElement( void );
extern void updateUndeleteList( int old_id, int new_id );
extern int cleanUndeleteList( int start_id, int end_id ) ;
extern int get_cid (char *string);
extern int add_cid (int id, int cid, Tcl_Interp * interp);

/*
 * end of header
 * reset TCL_STORAGE_CLASS to DLLIMPORT.
 */

#undef TCL_STORAGE_CLASS
#define TCL_STORAGE_CLASS DLLIMPORT

#endif /* pfa_h */
