###########################################################################
###########################################################################
##                                                                       ##
##  Datei   :     `pfaAction.tcl'                                        ##
##                ---------------                                        ##
##                                                                       ##
##  Inhalt  :     Prozeduren, die die Funktionalit"at                    ##
##                des `Action'-Men"us gew"ahrleisten.                    ##
##                                                                       ##
###########################################################################
###########################################################################


###########################################################################
##                                                                       ##
##  Prozeduren zum Men"upunkt:                                           ##
##                                                                       ##
##  Show B(PN)                        	       	       	       	       	 ##
##                                                                       ##
###########################################################################


##  TextEditor
##-----------------------------------------------------------
##  Es wird ein Toplevel-Fenster eingerichtet mit einem
##  Textfenster, um die B(PN)2-Programme darzustellen.
##  Ausserdem werden Menuepunkte fuer das Laden, Beenden
##  und weitere Sonderfunktionen bereitgestellt.
##-----------------------------------------------------------
##
proc TextEditor {} {
    global textArea opt textline histList

    if { [llength $histList] == 0 } {
	InitialQuestion
	return
    }

    ##  Das Toplevel-Window wird erstellt
    ##
    destroy .watch
    set watch_bpn2 [toplevel .watch -relief {raised}]
    wm positionfrom $watch_bpn2 user
    wm geometry $watch_bpn2 350x400+500+0
    wm title .watch "Show B(PN)"

    ##  Dieses Fenster soll nur ueber den 
    ##  'Exit' Menuepunkt verlassen werden 
    ##
    wm protocol .watch WM_DELETE_WINDOW {KillTextEditor}

    set textline 0
    .fr1a.fr2m.mb4.m entryconfigure "Show B(PN)"\
	-state disabled

    ##  Das Textfenster soll nur einmal erstellt werden.
    set opt(bpn2Editor) on

    ##  Das Menue wird erstellt und gepackt.
    frame $watch_bpn2.frame -borderwidth {2} -relief {raised}
    menubutton $watch_bpn2.frame.mb1 -font $opt(edFont)\
	-menu {.watch.frame.mb1.file} -text {File}
    menu $watch_bpn2.frame.mb1.file -font {8x13}
    $watch_bpn2.frame.mb1.file add command\
	    -command { 
		SomethingHasChanged "" highlight
		FSBox LoadTextFile "Open..." $editor(filter,bpn2)
		}\
	    -label {Open...} -state {active}
    $watch_bpn2.frame.mb1.file add command\
	-command { KillTextEditor } -label {Exit}
    menubutton $watch_bpn2.frame.mb2 -font $opt(edFont)\
	-menu {.watch.frame.mb2.action} -text {Action}
    menu $watch_bpn2.frame.mb2.action -font {8x13}
    $watch_bpn2.frame.mb2.action add command\
	-command { PFAdel_multi_sel }\
	-label {Remove Selection} -state {active}
    $watch_bpn2.frame.mb2.action add separator
    $watch_bpn2.frame.mb2.action add command\
	-command { Search Search }\
	-label {Search} -state {active}
    pack $watch_bpn2.frame.mb1  $watch_bpn2.frame.mb2\
	-side left -padx 4
    pack $watch_bpn2.frame -side top -fill x

    ##  Das Text-Widget mit den Scrollbars
    ##  wird erstellt und gepackt.
    ##
    set textframe [frame $watch_bpn2.text_frame -relief {raised}]
    set textRight [frame $textframe.right]
    set textScrollY [scrollbar $textRight.scrollbarY \
			 -command { .watch.text_frame.t yview }\
			 -relief {raised} -width {12}]
    set sizeFrame [frame $textRight.f -borderwidth 0\
			-width 12 -height 20]
    set textScrollX [ scrollbar $textframe.scrollbarX \
			  -command { .watch.text_frame.t xview }\
			  -orient {horizontal}\
			  -relief {raised} -width {12}]
    set textArea [text ${textframe}.t\
		      -insertofftime {600}\
		      -relief {raised} -borderwidth 0\
		      -state disabled\
		      -width 500 -height 300\
		      -wrap none\
		      -font $opt(edFont)\
		      -yscrollcommand \
		      { .watch.text_frame.right.scrollbarY set}\
		      -xscrollcommand \
		      { .watch.text_frame.scrollbarX set}]
    pack $textScrollY -side top -fill y -expand yes
    pack $sizeFrame -side bottom
    pack $textRight -side right -fill y
    pack $textScrollX -in $textframe -side bottom -fill x
    pack $textArea -in $textframe -expand yes -fill both
    pack $textframe -side top -expand true -fill both

    ##  Die Bindings fuer die Multi-Selektion von 
    ##  Textzeilen und fuer das Highlighten werden
    ##  gesetzt.
    ##
    bind .watch.text_frame.t <Button-3> {PFAmulti_sel}
    bind .watch.text_frame.t <Motion> {
        highlight_edge [.watch.text_frame.t index current]
    }
    bind .watch <Enter> {
        if {($opt(highlight) == "on") && ($opt(changed) == "no")} {
            DeleteMarks
        }
    }
    bind .watch <Leave> {
        set opt(messageText) ""
        if {($opt(highlight) == "on") && ($opt(changed) == "no")} {
            ResetLineInfo
        }
    }
}

##  LoadTextFile
##-----------------------------------------------------------
##  Das File mit dem Namen 'name' wird geladen, falls es
##  existiert und im Text-Widget angezeigt. 
##-----------------------------------------------------------
##
proc LoadTextFile { name } {
    global textArea fs_filepath 

    if {[file isfile $fs_filepath] == 1} {
	.watch.text_frame.t configure -state normal
	$textArea delete 1.0 end
	set d [open $name]
	while {! [eof $d] } {
	    $textArea insert end [read $d 1000]
	}
	close $d
	wm title .watch  "Show B(PN) - [file tail $name]"
	.watch.text_frame.t configure -state disabled
	
	## B(PN) Schluesselwoerter markieren
	##
	PFAscan_text
    } elseif { [file isdirectory $fs_filepath] == 1 } {

	set info_text "\"$fs_filepath\"\n Is A Directory!" 
	set question [win_dialog .d {Load Error}\
			  $info_text largeinfo 0 {Continue}]
	if {$question == 0} {
	    return 
	}
    } elseif { [file isfile $fs_filepath] != 1 } {
	set info_text "\"$fs_filepath\"\n Does Not Exist" 
	set question [win_dialog .d {Load Error}\
			  $info_text largeinfo 0 {Continue}]
	if {$question == 0} {
	    return
	}
    }
}

##  KillTextEditor
##-----------------------------------------------------------
##  wird aufgerufen, wenn der Editor ueber den Menuepunkt 
##  exit beendet wurde.
##-----------------------------------------------------------
##
proc KillTextEditor {} {
    global opt 

    SomethingHasChanged "" highlight
    .fr1a.fr2m.mb4.m\
	entryconfigure 1\
	-state normal
    destroy .watch
    set opt(bpn2Editor) off
}


###########################################################################
##                                                                       ##
##  Prozedur zum Menuepunkt:                                             ##
##                                                                       ##
##  CheckNFA                                                             ##
##                                                                       ##
###########################################################################


##  CheckNFA
##-----------------------------------------------------------
##  gibt die Meldung aus, da"s jeder FA ein NFA ist.  
##-----------------------------------------------------------
##
proc CheckNFA {} {
    global histList opt

    if { [llength $histList] == 0 } {
	InitialQuestion  
	return
    }

    DeleteMarks
    if { ($opt(examineFA) == "visible") || ( ![pfa_Editors]) } {
	set message "Every graph created by the FA-Editor\
                     is a NFA, so this is a NFA."
    } else {
	set message "Every graph created by the FA-Editor\
                    is a NFA, so these are NFAs."
    }
    win_dialog .d {CheckNFA Result} $message largeinfo 0 {Continue}
}


###########################################################################
##                                                                       ##
##  Prozedur zum Menuepunkt:                                             ##
##                                                                       ##
##  CheckNFAPath                                                         ##
##                                                                       ##
###########################################################################


##  CheckNFAPath
##-----------------------------------------------------------
##  ueberprueft, ob der ausgewaelte oder alle Automaten zur
##  Klasse NFAPath gehoeren.
##-----------------------------------------------------------
##
proc CheckNFAPath {} {
    global histList editor call faNumber opt

    if { [llength $histList] == 0 } {
	InitialQuestion  
	return
    }

    set call NFAPath
    set currentFANumber $faNumber(actual)

    ## loesche Markierungen
    ##
    DeleteMarks

    if { ($opt(examineFA) == "all") && ([pfa_Editors]) } {

            ## checke alle Automaten
            ##
	for { set i 1} { $i <= $faNumber(sum)} {incr i} {
	    if {[pfa_ExistEditor $i]} {        

		if {$faNumber(actual) != $i} {
		    set faNumber(actual) $i
		    TakeOtherFA $i
		}

		## ueberpruefe Start- und Endknoten
		##
		if {[CheckStartFinal]} {
		    ##
		    ## ueberpruefe Pfadeigenschaft

		    if {[CheckPath]} {
			continue
		    }
		}
		return
	    }
	}
	if {$faNumber(actual) != $currentFANumber} {
	    set faNumber(actual) $currentFANumber
	    TakeOtherFA $currentFANumber
	}
	win_dialog .d {CheckNFAPath Result} \
	    "O.K. Every Vertex Is On A Path\
                From A Start- To A Final-Vertex." largeinfo 0 {Continue}
    } else {

        ##checke nur aktuellen Automaten
	##
	if {[CheckStartFinal]} {

            ## ueberpruefe Start- und Endknoten
	    ##
	    if {[CheckPath]} {

		## ueberpruefe Pfadeigenschaft
		##
		win_dialog .d {CheckNFAPath Result} \
		    "O.K. Every Vertex Is On A Path\
                         From A Start- To A Final-Vertex." \
		    largeinfo 0 {Continue}
	    }
	}
    }
}


###########################################################################
##                                                                       ##
##  Prozedur zum Menuepunkt:                                             ##
##                                                                       ##
##  CheckDFA                                                             ##
##                                                                       ##
###########################################################################


##  CheckDFA
##-----------------------------------------------------------
##  "uberpr"uft, ob der ausgew"alte oder alle Automaten zur
##  Klasse DFA geh"oren.
##-----------------------------------------------------------
##
proc CheckDFA {} {
    global histList  editor call faNumber opt

    if { [llength $histList] == 0 } {
	InitialQuestion  
	return
    }

    set call DFA
    set currentFANumber $faNumber(actual)
    DeleteMarks
    if { ($opt(examineFA) == "all") && ([pfa_Editors]) } {
	##checke alle Automaten
	##
	for { set i 1} { $i <= $faNumber(sum)} {incr i} {
	    if {[pfa_ExistEditor $i]} {        
		if {$faNumber(actual) != $i} {
		    set faNumber(actual) $i
		    TakeOtherFA $i
		}
		if {[CheckStart]} {

		    ## ueberpruefe Startknoten
		    ##
		    if {[CheckDeterminism]} {

			## ueberpruefe Determinismus
			##
			continue
		    }
		}
		return
	    }
	}
	if {$faNumber(actual) != $currentFANumber} {
	    set faNumber(actual) $currentFANumber
	    TakeOtherFA $currentFANumber
	}
	win_dialog .d {CheckDFA Result} "O.K. These Are DFAs." \
	    largeinfo 0 {Continue}
    } else {

	## checke nur ahtuellen Automaten
	##
	if {[CheckStart]} {
	    
	    ## teste Startknoten
	    ##
	    if {[CheckDeterminism]} {

		## ueberpruefe determinismus
		##
		win_dialog .d {CheckDFA Result} "O.K. This Is A DFA." \
		    largeinfo 0 {Continue}
	    }
	}
    }
}


###########################################################################
##                                                                       ##
##  Prozedur zum Menuepunkt:                                             ##
##                                                                       ##
##  CheckDFAPath                                                         ##
##                                                                       ##
###########################################################################


##  CheckDFAPath
##-----------------------------------------------------------
##  "uberpr"uft, ob der ausgew"alte oder alle Automaten zur
##  Klasse DFAPath geh"oren.
##-----------------------------------------------------------
##
proc CheckDFAPath {} {
    global histList editor call faNumber opt

    if { [llength $histList] == 0 } {
	InitialQuestion  
	return
    }

    DeleteMarks
    set call DFAPath
    set currentFANumber $faNumber(actual)
    DeleteMarks
    if { ($opt(examineFA) == "all") && ([pfa_Editors]) } {
	
	##checke alle FA
	##
	for { set i 1} { $i <= $faNumber(sum)} {incr i} {
	    if {[pfa_ExistEditor $i]} {        
		if {$faNumber(actual) != $i} {
		    set faNumber(actual) $i
		    TakeOtherFA $i
		}
		if {[CheckStart]} {

		    ## teste Startknoten
		    ##
		    if {[CheckFinal]} {

			## teste Endknoten
			##
			if {[CheckDeterminism]} {

			    ## ueberpruefe Determinismus
			    ##
			    if {[CheckPath]} {

				## ueberpruefe Pfadeigenschaft
				##
				continue
			    }
			}
		    }
		}
		return
	    }
	}
	if {$faNumber(actual) != $currentFANumber} {
	    set faNumber(actual) $currentFANumber
	    TakeOtherFA $currentFANumber
	}
	win_dialog .d {CheckDFAPath Result} \
	    "O.K. Every Vertex Is On A Path\
                From A Start- To A Final-Vertex." largeinfo 0 {Continue}
    } else {

	## checke nur aktuellen FA
	##
	if {[CheckStart]} {

	    ## teste Startknoten
	    ##
	    if {[CheckFinal]} {

		## teste Endknoten
		##
		if {[CheckDeterminism]} {

		    ## ueberpruefe Determinismus
		    ##
		    if {[CheckPath]} {

			## ueberpruefe Pfadeigenschaft
			##
			win_dialog .d {CheckDFAPath Result} \
			    "O.K. Every Vertex Is On A Path\
                                 From A Start- To A Final-Vertex." \
			    largeinfo 0 {Continue}
		    }
		}
	    }
	} 
    }
}


###########################################################################
##                                                                       ##
##  Prozeduren zum Menuepunkt:                                           ##
##                                                                       ##
##  Set-Variables                                                        ##
##                                                                       ##
###########################################################################


##  SetVariables
##-----------------------------------------------------------
##  Es wird ein Toplevel-Fenster eingerichtet mit dessen 
##  Hilfe die Variablendeklarationen des FA eingegeben
##  werden koennen.
##-----------------------------------------------------------
##
proc SetVariables { scope } {
    global editor opt faNumber histList varText inf_text

    if { [llength $histList] == 0 } { 
	InitialQuestion
	return

    } elseif { $opt(code) != "B(PN)2"} {
	##  Der Aufruf des Variablen-Fensters ist nur sinnvoll,
	##  wenn die Option $opt(code) auf B(PN) steht. Falls dies
	##  nicht der Fall ist erscheint ein Meldungsfenster.
	##
	append info_text "Variable-Input Is Only Useful\n"
	append info_text "With Option \"Label-Code: B(PN)\""
	set question [win_dialog .d {Load Error}\
			  $info_text largeinfo 0\
			  {Continue}]
	if {$question == 0} {
	    return
	}
    }
    
    ##  Das Toplevel-Window wird erstellt
    ##
    toplevel .var -relief raised

    wm title .var " "
    wm geometry .var 320x390+500+0
    wm minsize .var 320 390
    wm maxsize .var 520 590
    
    ##  Die Canvas fuer die Ueberschrift 
    ##  und Hilfstexte wird erstellt und 
    ##  gepackt.
    ##
    canvas .var.inf_can -height 50\
	    -relief ridge -borderwidth 4
    pack .var.inf_can -fill x -side left
    
    ##  Die "Uberschrift wird gesetzt.
    ##
    if { $scope == "local"} {
	set xv 35
	set text "Declaration Of Local Variables"
    } else {
	set xv 30
	set text "Declaration Of Global Variables"
    }
    .var.inf_can create text $xv 30 -text $text\
	-anchor w -font "-*-times-bold-I-Normal--*-180-*"

    ##  Bitmap-Button, um Variablen-Text  
    ##  schneller einf"ugen zu k"onnen.
    ##
    frame .var.bu -relief ridge -borderwidth 4
    button .var.bu.var \
      -bitmap "@$opt(bitmapPath)/var.bm"\
      -command {VarInsert var}
    button .var.bu.chan \
      -bitmap "@$opt(bitmapPath)/chan.bm"\
      -command {VarInsert chan}
    button .var.bu.stack \
      -bitmap "@$opt(bitmapPath)/stack.bm"\
      -command {VarInsert stack}
    button .var.bu.bool \
	-bitmap "@$opt(bitmapPath)/bool.bm"\
	-command {VarInsert bool}
    pack .var.bu.var .var.bu.chan .var.bu.stack .var.bu.bool\
	-side left -padx 2

    ##  Das Textfenster zur Variableneingabe
    ##  und die Scrollbars werden erstellt
    ##  und gepackt.
    ##
    frame .var.te -relief ridge -borderwidth 4
    set textRight [frame .var.te.right]
    set textScrollY [scrollbar $textRight.scrollbarY \
			 -command { .var.te.text yview }\
			 -relief {raised} -width {12}]
    set sizeFrame [frame $textRight.f\
		       -borderwidth 0\
		       -width 13 -height 20]
    set textScrollX [ scrollbar .var.te.scrollbarX \
			  -command { .var.te.text xview }\
			  -orient {horizontal}\
			  -relief {raised} -width {12}]
    set textArea [text .var.te.text\
		      -insertofftime {600}\
		      -relief {raised} -borderwidth 0\
		      -height 11 -width 40\
		      -wrap none\
		      -yscrollcommand { .var.te.right.scrollbarY set}\
		      -xscrollcommand { .var.te.scrollbarX set}]
    pack $textScrollY -side top -fill y -expand yes
    pack $sizeFrame -side bottom
    pack $textRight -side right -fill y
    pack $textScrollX -side bottom -fill x
    pack $textArea -expand yes -fill both

    ##  Erstellen und Packen des unteren Bereichs 
    ##  mit den zwei Abschlussbutton.
    ##
    frame .var.exit -relief ridge -borderwidth 4
    set sunk [frame .var.exit.sa -bd 1 -relief sunken ]
    button $sunk.save -text "Save"\
	    -command { CheckQuestion [.var.te.text get 1.0 end]} 
    pack $sunk.save -padx 2m -pady 2m
    button .var.exit.discard\
	    -text "Cancel" -command {set opt(ok) 0}
    pack $sunk .var.exit.discard\
	    -side left -expand 1\
	    -padx 3m -pady 2m
    pack .var.inf_can .var.bu .var.te .var.exit\
	    -side top -fill both -expand yes\
	    -padx 1m -pady 1m
    
    ##  Falls eine Variablendeklaration des aktuellen
    ##  FA bereits existiert, wird sie in das Text-
    ##  fenster eingefuegt.
    ##
    .var.te.text delete 1.0 end
    if { $scope == "local" } {
	set opt(scope) "local"
	set text [join $editor($faNumber(actual),decl) \n]
    } else {
	set opt(scope) "global"
	set text [join $opt(globalDecl) \n]
    }
    .var.te.text insert end $text

    ##  Die Arbeit mit dem Editor kann erst 
    ##  fortgefuehrt werden, wenn das Variablen-Fenster
    ##  ueber den Save- oder Cancel-Button geschlossen wird.
    ##  D.h., wenn die Variable opt(ok) geaendert wird.
    ##
    DeleteKeyBindings $editor(canvas)
    tkwait visibility .var
    grab .var
    focus .var.te.text
    tkwait variable opt(ok)
    grab release .var
    if { [llength $histList] != 0 } { 	
	focus $editor(canvas)
    } else {
	focus .fr1a.fr2ru			
    }
    HotKeyBindings $editor(canvas)
    destroy .var  
}


##  VarInsert
##-----------------------------------------------------------
##  f"ugt Variablen-Deklarationen in das Textfenster ein
##  wenn die Bitmap-Button gedr"uckt werden.
##-----------------------------------------------------------
##
proc VarInsert {item} {
    global textArea

    if {$item == "var"} {
	.var.te.text insert insert "var name: {1,3..5} init 1;"
    } elseif {$item == "chan"} {
	.var.te.text insert insert "var name: chan capacity of {1,3..5};"
    } elseif {$item == "stack"} {
	.var.te.text insert insert "var name: stack capacity of {1,3..5};"
    } elseif {$item == "bool"} {
	.var.te.text insert insert "var name: {false,true} init true;"
    }
}


##  CheckQuestion
##-----------------------------------------------------------
##  testet durch den Aufruf von SyntaxCheckBPN2, ob die  
##  Variableneingabe korrekt ist. Wenn ja, wird der 
##  Variablen-Text "ubergeben.
##-----------------------------------------------------------
##
proc CheckQuestion { text } {
    global editor faNumber opt 

    while { [string first \" $text)] != -1 } {
	regsub \" $text ` text
	if { [string first \" $text] != -1} {
	    regsub \" $text ' text
	}
    }
    if {[SyntaxCheckBPN2 $text] == 1} {
        if { $opt(scope) == "local" } {
	    set editor($faNumber(actual),decl) [split $text \n]
	    set editor($faNumber(actual),decl) \
		[CutEmptyElements $editor($faNumber(actual),decl)]
	    SomethingHasChanged FA ""
        } else {
	    set opt(globalDecl) [split $text \n]
	    set opt(globalDecl) \
		[CutEmptyElements $opt(globalDecl)]
        }
	SomethingHasChanged "" highlight
	set opt(ok) 1
    }
}


###########################################################################
##                                                                       ##
##  Prozedur zum Menuepunkt:                                             ##
##                                                                       ##
##  PFA=>B(PN)                                                          ##
##  PFAStruct=>B(PN)                                                    ##
##                                                                       ##
###########################################################################

##  PFAtoBPN2
##-------------------------------------------------------------
##  trifft die notwendigen Vorbereitungen zum Aufruf
##  des Compilers zur Uebersetzung aller mit pepPFA editierten
##  Systeme von endlichen Automaten, bzw. der Klasse NFAStruct bei
##  gesetztem Parameter struct, und ruft den Compiler aus der FSBox
##  ueber die Prozedur InvokeC_Compile bzw. InvokeC_Compile_Struct auf. 
##-----------------------------------------------------------
##
proc PFAtoBPN2 {{struct "no"}} {
    global histList compile_answer opt faNumber editor\
	fs_filepath

    if { [llength $histList] == 0 } {
	InitialQuestion  
	return
    }

    ## Alte Markierungen werden gel"oscht.
    ##
    DeleteMarks

    if {$struct} {
      ##  Die einzelnen FA werden vorgeschaltet auf die 
      ##  Pfadeigenschaft hin ueberprueft
      ##
      if {![CheckCompile]} {
          return
      }
    }
    set compile_answer {}
    if {$opt(examineFA) == "all"} {

	##  Wenn alle FA kompilierte werden sollen, wird
	##  gepr"uft, ob sie den gleiche Code-Typ haben.
	##
	if {![SameCodeType]} {
	    ## Test auf Uebereinstimmung der Code-Typen
	    return
	}
	
	##  Wenn Struct-Code eingestellt ist wird zusaetzlich ein
	##  Syntax-Check durchgefuehrt.
	##
	if {![SyntaxCheckBeforeComp]} {
	    return
	}

	##  In der File-Select-Box kann der Name des B(PN)-Files
	##  angegeben werden. Dann wird der Compiler aufgerufen.
	##
	if {$opt(project) == "yes"} {
	    cd [file dirname $opt(projectName)]
	    set fs_filepath \
		"[file rootname $opt(projectName)].bpn2"
	} else {
	    cd [file dirname $editor($faNumber(actual),fullPath)]
	    set fs_filepath \
		"[file rootname $editor($faNumber(actual),fullPath)].bpn2"
	}
    } else {
	## zusaetzlicher Syntaxcheck
	##
	if {![SyntaxCheckBeforeComp]} {
	    return
	}
	set fs_filepath \
            "[file rootname $editor($faNumber(actual),fullPath)].bpn2"
	cd [file dirname $editor($faNumber(actual),fullPath)]
    }
    if {$struct} {
      FSBox InvokeC_Compile "Save As..." $editor(filter,bpn2) . save
    } else {
      FSBox InvokeC_Compile_Struct "Save As..." $editor(filter,bpn2) . save
    }
    ExecuteCompileAnswer
}


###########################################################################
##                                                                       ##
##  Prozedur zum Menuepunkt:                                             ##
##                                                                       ##
##  DeleteMarks                                                          ##
##                                                                       ##
###########################################################################

##  DeleteMarks
##-----------------------------------------------------------
##  l"oscht alle durch die Check-Algorithmen oder das 
##  Highlighten hervorgerufenen Markierungen an Knoten 
##  oder Kanten.
##-----------------------------------------------------------
##
proc DeleteMarks {} {
    global histList editor faNumber opt

    if { [llength $histList] == 0 } {
	InitialQuestion  
	return
    }

    if {$opt(redMarked)} {
	set currentFANumber $faNumber(actual)
	for { set i 1} { $i <= $faNumber(sum)} {incr i} {
	    if {[pfa_ExistEditor $i]} {        
		if {$faNumber(actual) != $i} {
		    set faNumber(actual) $i
		    TakeOtherFA $i
                    }
		$editor(canvas) itemconfigure knotenBase -fill white
		$editor(canvas) itemconfigure doubleselected -fill brown
		$editor(canvas) itemconfigure selected -fill tan
		$editor(canvas) itemconfigure kante -fill black
                }
	}
	if {$faNumber(actual) != $currentFANumber} {
	    set faNumber(actual) $currentFANumber
	    TakeOtherFA $currentFANumber
	}
	set opt(redMarked) 0
    }
}
