###########################################################################
###########################################################################
##                                                                       ##
##  Hilfs-Prozeduren zum Menue:						 ##
##									 ##
##  Action								 ##
##									 ##
###########################################################################
###########################################################################


###########################################################################
##									 ##
##  Hilfsprozeduren zu den 						 ##
##									 ##
##  Check-Menuepunkten							 ##
##									 ##
###########################################################################


##  DeleteMarksActFA
##-----------------------------------------------------------
##  loescht die durch die CheckAlgorithmen oder das
##  Highlighten oder die Compiler hervorgerufenen 
##  Markierungen des aktuellen FA.
##-----------------------------------------------------------
##
proc DeleteMarksActFA {} {

    global editor opt

    if {$opt(redMarked)} {
        $editor(canvas) itemconfigure knotenBase -fill white
        $editor(canvas) itemconfigure doubleselected -fill brown
        $editor(canvas) itemconfigure selected -fill tan
        $editor(canvas) itemconfigure kante -fill black
        set opt(redMarked) 0
    }


}


##  CheckCompile
##-----------------------------------------------------------
##  ueberprueft, ob alle zu kompilierenden FA die 
##  Pfadeigenschaft aufweisen.
##-----------------------------------------------------------
##
proc CheckCompile {} {

    global editor faNumber opt

    DeleteMarks
    set currentFANumber $faNumber(actual)
    if { ($opt(examineFA) == "all") && ([pfa_Editors]) } {

	## checke alle FA
	##
        for { set i 1} { $i <= $faNumber(sum)} {incr i} {
            if {[pfa_ExistEditor $i]} {
                if {$faNumber(actual) != $i} {
                    set faNumber(actual) $i
                    TakeOtherFA $i
                }
                if {[FALeer]} {
                    continue
                }
                if {[CheckStartFinal_Compile]} {

		    ## Start- und Endknoten-Test
		    ##
                    if {[CheckPath_Compile]} {

			## ueberpruefe Pfadeigenschaft
			##
                        continue
                    }
                }
                return 0
            }
        }
        if {$faNumber(actual) != $currentFANumber} {
            set faNumber(actual) $currentFANumber
            TakeOtherFA $currentFANumber
        }
        return 1
    } else {

	## checke nur aktuellen FA
	##
        if {[FALeer]} {
            return 1
        }
        if {[CheckStartFinal_Compile]} {

	    ## teste auf Start- und Endknoten
	    ##
            if {[CheckPath_Compile]} {

		## ueberpruefe Pfadeigenschaft
		##
                return 1
            }
        }
        return 0
    }
}


##  FALeer
##-----------------------------------------------------------
##  ueberprueft, ob ein zu untersuchender FA Elemente 
##  enthaelt.
##-----------------------------------------------------------
##
proc FALeer {} {
    global editor

    if { [expr [llength [$editor(canvas) find withtag startknoten]] \
             + [llength [$editor(canvas) find withtag endknoten]] \
             + [llength [$editor(canvas) find withtag knoten]] ] == 0 } {
        return 1
    }
    return 0
}


##  CheckPath_Compile
##-----------------------------------------------------------
##  gibt eine Meldung zurueck, wenn Knoten existieren, die
##  sich nicht auf einem Pfad vom Start- zum Endknoten
##  befinden, d.h. testet die Pfadeigenschaft
##-----------------------------------------------------------
##
proc CheckPath_Compile {} {
    global editor opt

    ##  Vorwaerts- und Rueckwaertslauf durch die FA
    ##
    CheckPathWork
    ClearSoloVertices
    if {[lindex [$editor(canvas) find withtag forward] 0] != "" || \
	    [lindex [$editor(canvas) find withtag backward] 0] != ""} {
	$editor(canvas) itemconfigure forward -fill Red
	$editor(canvas) itemconfigure backward -fill Red

	append info_text "The Vertices In Red Are Not On A\n"
	append info_text "Path From A Start- To A Final-\n"
	append info_text "Vertex. This FA Is Not Compilable\n"
	append info_text "With 'PFAStruct=>B(PN)'." 
	win_dialog .d "Compile Result" $info_text\
                      largeinfo 0 {Continue}
        $editor(canvas) dtag forward
        $editor(canvas) dtag backward
        set opt(redMarked) 1
        return 0
    }
    $editor(canvas) dtag forward
    $editor(canvas) dtag backward
    return 1
}


##  ClearSoloVertices
##-----------------------------------------------------------
##  loescht die Markierungen forward und backward von Knoten,
##  die keine inzidierenden Kanten aufweisen
##-----------------------------------------------------------
##
proc ClearSoloVertices {} {
    global editor

    set entirelist [concat [$editor(canvas) find withtag forward] \
                           [$editor(canvas) find withtag backward] ]
    foreach v $entirelist {
        if { ([llength [$editor(canvas) find withtag kantevon$v]] == 0) &&
             ([llength [$editor(canvas) find withtag kantenach$v]] == 0)} {
            if {[ItemType $v] != "startknoten"} {
                $editor(canvas) dtag $v forward
                $editor(canvas) dtag $v backward
            }
        }
    }
}


##  CheckPath
##-----------------------------------------------------------
##  gibt eine Meldung zurueck, wenn Knoten existieren, die
##  sich nicht auf einem Pfad vom Start- zum Endknoten
##  befinden. Test auf Pfadeigenschaft.
##-----------------------------------------------------------
##
proc CheckPath {} {
    global editor call opt

    ## Vorwaerts- und Rueckwaertslauf durch die FA
    ##
    CheckPathWork
    if {[lindex [$editor(canvas) find withtag forward] 0] != "" || \
        [lindex [$editor(canvas) find withtag backward] 0] != ""} {
        $editor(canvas) itemconfigure forward -fill Red
        $editor(canvas) itemconfigure backward -fill Red

    append info_text "The Vertices In Red Are Not\n"
    append info_text "On A Path From A Start- To\n"
    append info_text "A Final-Vertex.\n"
    win_dialog .d "Check$call Result" $info_text\
                      largeinfo 0 {Continue}
        set opt(redMarked) 1
        return 0
    }
    $editor(canvas) dtag forward
    $editor(canvas) dtag backward
    return 1
}

##  CheckPathWork
##-----------------------------------------------------------
##  fuehrt je einen Vorwaerts- und Rueckwaertslauf durch die
##  die FA durch und hinterlaesst an jedem besuchten Knoten
## die Marke forward bzw. backward
##-----------------------------------------------------------
##
proc CheckPathWork {} {
    global editor
    $editor(canvas) addtag forward withtag knotenBase
    $editor(canvas) addtag backward withtag knotenBase
    
    ## gehe von Startknoten aus
    ##
    foreach v [$editor(canvas) find withtag startknoten] {
        if {[string range [lindex [$editor(canvas) gettags $v] 2] 0 14] !=
           "startknotentext"} {
            lappend vertexlist $v
            $editor(canvas) dtag $v forward
        }
    }
    ## Vorwaertsdurchlauf
    ##
    while {[llength $vertexlist] != 0} {
        set vertexworklist $vertexlist
        set vertexlist {}
        foreach v $vertexworklist {
            set edgelist [$editor(canvas) find withtag kantevon$v]
            foreach e $edgelist {
                set kantenziel [KantenZiel $e]
                if {[lsearch [$editor(canvas) gettags $kantenziel] forward]
		    != -1} {
                    $editor(canvas) dtag $kantenziel forward
                    lappend vertexlist $kantenziel
                }
            }
        }
    }

    ## gehe von Endknoten aus
    ##
    foreach v [$editor(canvas) find withtag endknoten] {
        if {[string range [lindex [$editor(canvas) gettags $v] 2] 0 14] !=
           "endknotentext"} {
            lappend vertexlist $v
            $editor(canvas) dtag $v backward
        }
    }

    ## Rueckwaertsdurchlauf
    ##
    while {[llength $vertexlist] != 0} {
        set vertexworklist $vertexlist
        set vertexlist {}
        foreach v $vertexworklist {
            set edgelist [$editor(canvas) find withtag kantenach$v]
            foreach e $edgelist {
                set kantenstart [KantenStart $e]
                if {[ lsearch [$editor(canvas) gettags $kantenstart] \
			  backward] != -1} {
                    $editor(canvas) dtag $kantenstart backward
                    lappend vertexlist $kantenstart
                }
            }
        }
    }
}


##  CheckStartFinal
##-----------------------------------------------------------
##  testet, ob der zu untersuchende FA
##  keinen Start- bzw. Endknoten enthaelt und gibt eine
##  entsprechende Meldung zur"uck.
##-----------------------------------------------------------
##
proc CheckStartFinal {} {
    global editor

    if {[lindex [$editor(canvas) find withtag startknoten] 0] == ""} {
        if {[lindex [$editor(canvas) find withtag endknoten] 0] == ""} {
	    append info_text "No Start- Or Final-\n"
	    append info_text "Vertex Is Specified"
            win_dialog .d {CheckNFAPath Result} $info_text\
		    largeinfo 0 {Continue}
        } else {
            win_dialog .d {CheckNFAPath Result} "No Start-Vertex\
                          Is Specified." largeinfo 0 {Continue}
        }
        return 0
    } else {
        if {[lindex [$editor(canvas) find withtag endknoten] 0] == ""} {
            win_dialog .d {CheckNFAPath Result} "No Final-Vertex\
                          Is Specified." largeinfo 0 {Continue}
            return 0
        } else {
            return 1
        }
    }
}


##  CheckStart
##-----------------------------------------------------------
##  ueberprueft, ob der aktuelle FA keinen oder mehr als
##  einen Startknoten enthaelt und gibt eine entsprechende
##  Meldung zurueck.
##-----------------------------------------------------------
##
proc CheckStart {} {
    global editor call

    set number [llength [$editor(canvas) find withtag startknoten]]
    if { $number == 0} {
	append info_text "This Is Not A DFA !\n"
	append info_text "No Start-Vertex Is Specified."
        win_dialog .d "Check$call Result" $info_text\
		largeinfo 0 {Continue}
        return 0
    } elseif { $number > 2 } {
	append info_text "This Is Not A DFA ! More Than\n"
	append info_text "One Start-Vertex Is Specified." 
        win_dialog .d "Check$call Result" $info_text \
                       largeinfo 0 {Continue}
        return 0
    } else {
        return 1
    }
}


##  CheckFinal
##-----------------------------------------------------------
##  ueberprueft, ob der aktuelle FA keinen Endknoten enthaelt
##  und gibt eine entsprechende Meldung zurueck.
##-----------------------------------------------------------
##
proc CheckFinal {} {
    global editor
    if {[llength [$editor(canvas) find withtag endknoten]] == 0} {
	
        win_dialog .d {CheckDFAPath Result} "No Final-Vertex\
                      Is Specified." largeinfo 0 {Continue}
        return 0
    }
    return 1
}


##  CheckStartFinal
##-----------------------------------------------------------
##  Testet fuer den Checklauf des Compiler, ob Start- und 
##  Endknoten im Automaten vorliegen.
##-----------------------------------------------------------
##
proc CheckStartFinal_Compile {} {

    global editor

    set number [llength [$editor(canvas) find withtag startknoten]]
    if { $number == 0} {
	append info_text "In This FA Is No Start-Vertex\n"
	append info_text "Specified. The FA Is Not Compilable\n"
	append info_text "With 'PFAStruct=>B(PN)'."
        win_dialog .d "Compile Result"  $info_text \
		largeinfo 0 {Continue}
        return 0
    } else {
        set number [llength [$editor(canvas) find withtag endknoten]]
        if { $number == 0} {
	    append info_text "In This FA Is No Final-Vertex\n"
	    append info_text "Specified. The FA Is Not Compilable\n"
	    append info_text "With 'PFAStruct=>B(PN)'."
            win_dialog .d "Compile Result" $info_text \
		    largeinfo 0 {Continue}
            return 0
        }
    }
    return 1
}

##  CheckDeterminism
##-----------------------------------------------------------
##  Testet die Determinismus-Eigenschaft von FA, d.h. es 
##  duerfen keine epsilon-Kanten und nicht mehr als eine 
##  Kante mit gleicher Beschriftung von einem Knoten ausgehen
##-----------------------------------------------------------
##
proc CheckDeterminism {} {

    global editor call opt

    set deterministic 1
    foreach v [$editor(canvas) find withtag knotenBase] {
        set deterministic [expr $deterministic && [CheckEdges $v]]
    }
    if {! $deterministic} {
        $editor(canvas) itemconfigure nondet -fill Red
        win_dialog .d "Check$call Result" "This Is Not A DFA Because Of\
                      The Edges With Their Input In Red." \
	    largeinfo 0 {Continue}
        $editor(canvas) dtag nondet
        set opt(redMarked) 1
    }
    return $deterministic
}

##  CheckEdges
##-----------------------------------------------------------
##  ueberprueft, ob mehrere ausgehende Kanten des Knotens
##  `vertex' die gleiche Beschriftung besitzen und gibt
##  in diesem Fall eine 0 an die aufrufende Prozedur
##  `CheckDeterminism' zur"uck.
##-----------------------------------------------------------
##
proc CheckEdges {vertex} {
    global editor textlist
    set textlist {}
    set deterministic 1
    set itemlist [$editor(canvas) find withtag kantetextvon$vertex]
    if {[lindex $itemlist 0] == ""} {
        return $deterministic
    }
    foreach item $itemlist {
       lappend textlist \
               [concat \{$item\}\
	       \{[lindex [$editor(canvas) itemconfigure $item -text] 4]\}]
    }
    set textlist [lsort -command {compare} $textlist]
    set lastno [expr [llength $textlist] -1]
    for {set i 0} {$i < $lastno} {incr i} {
        set item [lindex $textlist $i]
        if {[lindex $item 1] == ""} {
            $editor(canvas) addtag \
		nondet withtag [ItemName [lindex $item 0]]
            set deterministic 0
        } else {
            set nextitem [lindex $textlist [expr $i + 1]]
            if {[string compare \
		     [lindex $item 1] [lindex $nextitem 1]] == 0} {
              
		$editor(canvas) addtag \
		    nondet withtag [ItemName [lindex $item 0]]
                $editor(canvas) addtag \
		    nondet withtag [ItemName [lindex $nextitem 0]]
                set deterministic 0
            }
        }
    }
    set lastitem [lindex $textlist $lastno]
    if {[lindex $lastitem 1] == ""} {
        $editor(canvas) addtag \
	    nondet withtag [ItemName [lindex $lastitem 0]]
        set deterministic 0
    }
    return $deterministic
}


##-------------------------------------------------------
##  Vergleichsprozedur
##-------------------------------------------------------
proc compare { x y } {
    return [ string compare [lindex $x 1] [lindex $y 1] ]
}




###########################################################################
##									 ##
##  Hilfsprozeduren zu den 						 ##
##									 ##
##  Compile-Men"upunkten						 ##
##									 ##
###########################################################################



##  SameCodeType
##-----------------------------------------------------------
##  prueft, ob bei allen geoeffneten FA die gleiche
##  Code-Option gesetzt ist. Wenn diese Bedingung
##  nicht erfuellt ist, erscheint ein Meldungsfenster
##  und der Compiler wird nicht aufgerufen.
##-----------------------------------------------------------
##
proc SameCodeType {} {
    global opt editor histList

    set first [lindex $histList 0]
    foreach nr $histList {
	if {$editor($first,code) != $editor($nr,code)} {
	    TakeOtherFA $nr

	    append info_text "The FA Are Not Of The Same Label Code\n"
	    append info_text "Type. They Are Not Compilable As A\n"
	    append info_text "System Of Parallel FA."
	    win_dialog .d {Compile Result} $info_text\
		largewarn 0 {Continue}
	    return 0
	}
    }
    return 1
}

##  InvokeC_Compile_Struct
##-----------------------------------------------------------
##  ueberprueft ob alle FA kompiliert werden sollen
##  und welcher Code-Typ eingestellt ist. Anschliessend
##  wird der Struct-Compiler aufgerufen.
##-----------------------------------------------------------
##
proc InvokeC_Compile_Struct { file } {

    global  faNumber opt

    if {$opt(examineFA) == "all"} {
        set fa_number 0
    } else {
        set fa_number $faNumber(actual)
    }
    if {$opt(code) == "B(PN)2"} {
        set code 1
    } else {
        set code 0
    }

    pfa_Compile_Struct $code $fa_number $file
}


##  InvokeC_Compile
##-----------------------------------------------------------
##  ueberprueft ob alle FA kompiliert werden sollen
##  und welcher Code-Typ eingestellt ist. Anschliessend
##  wird der Compiler, der fuer alle FA geeignet ist, 
##  aufgerufen.
##-----------------------------------------------------------
##
proc InvokeC_Compile { file } {

    global  faNumber opt

    if {$opt(examineFA) == "all"} {
        set fa_number 0
    } else {
        set fa_number $faNumber(actual)
    }
    if {$opt(code) == "B(PN)2"} {
        set code 1
    } else {
        set code 0
    }
    pfa_Compile $code $fa_number $file
}


##  ExecuteCompileAnswer
##-----------------------------------------------------------
##  ermoeglicht Ausgabe von Informationen von C an Tcl
##
##-----------------------------------------------------------
##
proc ExecuteCompileAnswer {} {
    global compile_answer

    eval [join $compile_answer]
}


##  Mark-FA-WS
##-----------------------------------------------------------
##  wird von C aus aufgerufen, markiert Problemstrukturen,
##  die waehrend des Compile-Vorganges erkannt wurden
##
##-----------------------------------------------------------
##
proc Mark-FA-WS { FANumber placelist } {
    global editor faNumber opt

    if {$faNumber(actual) != $FANumber} {
        set faNumber(actual) $FANumber
        TakeOtherFA $FANumber
    }

    set placelist [lsort $placelist]
    set lastplace -1
    foreach place $placelist {
        if {$place != $lastplace } {
            set lastplace $place
            lappend placeunion $place
        }
    }
    foreach place $placeunion {
        $editor(canvas) itemconfigure $place -fill Red
    }
    Error-FA-WS-Window
    set opt(redMarked) 1
}


##  Error-FA-WS-Window
##-----------------------------------------------------------
##  gibt ueber ein Meldungsfenster eine Fehlermeldung aus,
##  dass der zu kompilierende FA nicht die notwendige
##  Struktur besitzt.
##-----------------------------------------------------------
##
proc Error-FA-WS-Window {} {

    append info_text "This FA doesn't have the necessary structure\n"
    append info_text "because of (mutual) dependencies between the red-\n"
    append info_text "colored node(s) and others. It is not compilable\n"
    append info_text "by 'PFAStruct=>B(PN)'."
    win_dialog .d {Struct-Compile Result} $info_text\
        largewarn 0 {Continue}
}

##  FA-WS-EmptyWindow
##-----------------------------------------------------------
##  wird von C aus aufgerufen, wenn ein zu kompilierender
##  FA keine Kanten besitzt.
##-----------------------------------------------------------
##
proc FA-WS-EmptyWindow {} {

    win_dialog .d {Compile Result} "There are no edges to compile." \
    largewarn 0 {Continue}
}


##  FA-WS-FaultLabel
##-----------------------------------------------------------
##  wird von C aus aufgerufen, wenn eine Kante vom End- zum
##  Startknoten verbotenerweise ein Text-Label besitzt.
##-----------------------------------------------------------
##
proc FA-WS-FaultLabel {FANumber} {

    global faNumber

    if {$faNumber(actual) != $FANumber} {
        set faNumber(actual) $FANumber
        TakeOtherFA $FANumber
    }
    MarkFaultLabel

    append info_text "As Input Of The Edge from Final-\n"
    append info_text "To Start-Vertex Is Only 'Epsilon'\n"
    append info_text "Permitted. This FA is not compilable\n"
    append info_text "by 'PFAStruct=>B(PN)'." 

    win_dialog .d {Compile Result} $info_text\
                  largewarn 0 {Continue}
}


##  MarkFaultLabel
##-----------------------------------------------------------
##  markiert eine Kante mit verbotenener Beschriftung.
##-----------------------------------------------------------
##
proc MarkFaultLabel {} {

    global editor opt

    foreach endvertex [$editor(canvas) find withtag endknoten] {
        foreach kante [$editor(canvas) find withtag kantevon$endvertex] {
            set zv [KantenZiel $kante]
            if {[ string range \
		      [lindex [$editor(canvas) gettags $zv] 0] 0 10] \
		    == "startknoten"} {
                $editor(canvas) itemconfigure [ItemName $kante] -fill Red
                set opt(redMarked) 1
            }
        }
    }
}


##  CompileDone
##-----------------------------------------------------------
##  wird von C aus aufgerufen, wenn der Kompiliervorgang
##  erfolgreich beendet wurde und gibt eine entsprechende
##  Meldung aus. Im folgenden ist es moeglich, dass
##  kompilierte B(PN)- Programm im B(PN)2-Editor zu
##  betrachten.
##-----------------------------------------------------------
##
proc CompileDone {} {
    global fs_filepath opt

    append info_text "Compilation Succeeded. Do You\n"
    append info_text "Like To See The B(PN)-File ?" 
    set question [win_dialog .d {Compile Success}\
	    $info_text largeinfo 0 {Load File} {Cancel}]
    if {$question == 1} {
        set opt(changed) "yes"
        return
    }
    if {$question == 0} {

	if { $opt(bpn2Editor) != "on" } {

	    TextEditor
	    LoadTextFile $fs_filepath

	} else {
	    LoadTextFile $fs_filepath
	}
    }
}



###########################################################################
##									 ##
##  Hilfsprozeduren zum							 ##
##									 ##
##  Highlighten								 ##
##									 ##
###########################################################################





##  highlight_lines
##-----------------------------------------------------------
##  wird von der C-Funktion tk_Highlight aufgerufen und
##  markiert die angegebenen Zeilen im B(PN)-Editor
##-----------------------------------------------------------
##
proc highlight_lines {lines} {

    foreach line $lines {
        PFAhighlight_line $line
        PFAmulti_sel
    }
}

##  highlight
##-----------------------------------------------------------
##  Wenn die Highlight-Option eingestellt ist, wird die
##  Kante mit der Id `kante' farblich hervorgehoben, wenn
##  sie mit dem Mauszeiger beruehrt wird. Ueber den Tcl-
##  Befehl pfa_Highlight wird daraufhin die zu dieser Kante
##  gehoerende Zeile ebenfalls markiert
##-----------------------------------------------------------
##
proc highlight {kante} {

    global opt faNumber

    if {$opt(highlight) == "on"} {
## muss wegen peptool hier hin
	pfa_Highlight $faNumber(actual) $kante 
        if {$opt(changed) == "no"} {
##            pfa_Highlight $faNumber(actual) $kante
        } elseif {$opt(changed) == "yes"} {
            set opt(messageText) "No Reference To B(PN)"
        } else {
            set opt(messageText) "FA Not Compiled"
        }
    }
}

##  highlightend
##-----------------------------------------------------------
##  Wenn der Mauszeiger den Bereich der Kante verlaesst, wird
##  die Markierung der Kante und der entsprechenden Zeile im
##  B(PN)-Editor rueckgaengig gemacht.
##-----------------------------------------------------------
##
proc highlightend {} {

    global opt

    if {($opt(messageText) == "FA Not Compiled") || \
        ($opt(messageText) == "No Reference To B(PN)")} {
        set opt(messageText) ""
    }
    if {($opt(highlight) == "on") && ($opt(changed) == "no")} {
        PFAdel_multi_sel
    }

}

##  set_changed_no
##-----------------------------------------------------------
##  wird von C aus aufgerufen und gibt an, dass Highlighten
##  moeglich ist.
##-----------------------------------------------------------
##
proc set_changed_no {} {

    global opt

    set opt(changed) "no"
}


##  highlight_edge
##-----------------------------------------------------------
##  markiert die zu einer B(PN)-Zeile im Text-Editor
##  gehoerende Kante des entsprechenden Automaten, wenn
##  die Highlight-Option eingestellt ist.
##-----------------------------------------------------------
##
proc highlight_edge {textpos} {

    global opt textline faNumber

    if {$opt(highlight) == "on"} {
        if {$opt(changed) == "no"} {
            set textpos [.watch.text_frame.t index current]
            set line \
            [string range $textpos 0 [expr [string first . $textpos] - 1]]
            if {$line != $textline} {
                PFAdel_multi_sel
                highlightend
                DeleteMarksActFA
                set textline $line
                set erg [pfa_GetEdgeToLine $line]
                if {$erg != ""} {
                    set ednumber [lindex $erg 0]
                    set edge [lindex $erg 1]
                    if {$faNumber(actual) != $ednumber} {
                        set faNumber(actual) $ednumber
                        TakeOtherFA $ednumber
                    }
                    set opt(redMarked) 1
                    PFAhighlight_line $line
                    PFAmulti_sel
                    markedge $edge
                }
            }
        } elseif {$opt(changed) == "yes"} {
            set opt(messageText) "No Reference To B(PN)"
        } else {
            set opt(messageText) "FA Not Compiled"
        }
    }
}

##  ResetLineInfo
##-----------------------------------------------------------
##  loescht die Zeileninformationen bezueglich des
##  Highlightens, wenn der Mauszeiger den Bereich des
##  B(PN)-Editors verlaesst.
##-----------------------------------------------------------
##
proc ResetLineInfo {} {

    global textline

    DeleteMarksActFA
    highlightend
    set textline 0
}

##  markedge
##-----------------------------------------------------------
##  markiert die Kante mit der Id `edge' in der Farbe
##  magenta.
##-----------------------------------------------------------
##
proc markedge {edge} {

    global editor

    $editor(canvas) itemconfigure [ItemName $edge] -fill magenta
}



###########################################################################
##									 ##
##  Hilfsprozeduren zum							 ##
##									 ##
##  Checken der Syntax							 ##
##									 ##
###########################################################################



##  CodeChange
##-----------------------------------------------------------
##  ruft ein Meldungsfenster auf, wenn die Variable, die den 
##  Code-Typ repraesentiert, geaendert wurde.
##-----------------------------------------------------------
##
proc CodeChange {} {

    append info_text "Changing The Code Type Results In\n"
    append info_text "Deleting All Existing Edge Labels\n"
    append info_text "Of This FA! Do You Really Want To\n"
    append info_text "Change The Code Type ?" 
    set result [win_dialog .d {Warning Code Change} \
	    $info_text largewarn 0 {Cancel} {Change Code} ]
    if {$result == 1} {
        DeleteAllEdgeLabels
    }
    return $result
}


##  DeleteAllEdgeLabels
##-----------------------------------------------------------
##  loescht alle Kanten-Label, wenn der Code-Typ geaendert
##  werden soll.
##-----------------------------------------------------------
##
proc DeleteAllEdgeLabels {} {
    global  editor faNumber opt

    set ktextliste [$editor(canvas) find withtag kantetext]
    if {$ktextliste != 0} {
	SomethingHasChanged FA highlight
        foreach kantetext $ktextliste {
            set baseId [ItemBase $kantetext]
            pfa_Rename $baseId ""
        }
    }
}


##  SyntaxCheckBeforeComp
##-----------------------------------------------------------
##  Zusaetzlicher Syntax-Check vor dem Compilieren
##  
##-----------------------------------------------------------
##
proc SyntaxCheckBeforeComp {} {

    global opt

    if {$opt(code) == "B(PN)2"} {
        return 1
    }
    return [SyntaxCheckBC]
}


##  SyntaxCheckBC
##-----------------------------------------------------------
##  Zusaetzlicher Syntax-Check vor dem Compilieren
##  
##-----------------------------------------------------------
##
proc SyntaxCheckBC {} {

    global faNumber opt editor

    set currentFANumber $faNumber(actual)
    if { ($opt(examineFA) == "all") && ([pfa_Editors]) } {
        for { set i 1} { $i <= $faNumber(sum) } {incr i} {
            if {[pfa_ExistEditor $i]} {
                if {$faNumber(actual) != $i} {
                    set faNumber(actual) $i
                    TakeOtherFA $i
                }
                if {[FALeer]} {
                    continue
                }
                foreach kantetext \
		    [$editor(canvas) find withtag kantetext] {

                    if {![SyntaxCheckStructBC $kantetext]} {
                        return 0
                    }
                }
            }
        }
        if {$faNumber(actual) != $currentFANumber} {
            set faNumber(actual) $currentFANumber
            TakeOtherFA $currentFANumber
        }
        return 1
    } else {
        if {[FALeer]} {
            return 1
        }

        foreach kantetext [$editor(canvas) find withtag kantetext] {
            if {![SyntaxCheckStructBC $kantetext]} {
                return 0
            }
        }
        return 1
    }
}


##  SyntaxCheck
##-----------------------------------------------------------
##  Teil des Syntaxcheckers
##  
##-----------------------------------------------------------
##
proc SyntaxCheck {kantetext} {

    global opt

    if {$kantetext == "<>"} {
        return 1
    }

    if {$opt(code) == "B(PN)2"} {
        if {![SyntaxCheckBPN2 $kantetext]} {
            return 0
        }
    } else {
        if {![SyntaxCheckStruct $kantetext]} {
            return 0
        }
    }
    return 1
}


##  SyntaxCheckBPN2
##-----------------------------------------------------------
##  ruft den Syntax-Checker bpn2pbc auf, um korrekten B(PN)-
##  Code zu garantieren. Wenn 1 zurueckgegeben wird, kann das
##  Kantenlabel eingetragen werden
##-----------------------------------------------------------
##

proc SyntaxCheckBPN2 {text} {

    global opt

    if {$text == ""} {
        return 1
    }

    set datei [open $opt(filename) w]
    puts $datei $text
    close $datei

    catch {exec bpn2pbc -f $opt(filename) $opt(filename.erg)}

    return [SyntaxResultCheck]

}

##  SyntaxCheckStructBC
##-----------------------------------------------------------
##  Teil des SyntaxCheck vor dem Compilieren
##  
##-----------------------------------------------------------
##
proc SyntaxCheckStructBC {kantetext} {
    global editor opt 


    set label [GetText $kantetext]

    if {$label == ""} {
        return 1
    }
    set label [string range $label 1 [expr [string length $label] - 2]]

    set sycheckresult [SyntaxCheckActionStruct $label]

    if {$sycheckresult != 1} {
        set opt(redMarked) 1
        $editor(canvas) itemconfigure [ItemName $kantetext] -fill Red
        win_dialog .d {Syntax Check Result} \
        "Syntax Error In The Red Marked Edge Label :\n\n$sycheckresult" \
        largewarn 0 {Continue}
        return 0
    }
    return 1
}


##  SyntaxCheckStruct
##-----------------------------------------------------------
##  Teil des Syntax-Checkers
##  
##-----------------------------------------------------------
##
proc SyntaxCheckStruct {kantetext} {

    global editor opt

    if {$kantetext == ""} {
        return 1
    }


    set kantetext \
    [string range $kantetext 1 [expr [string length $kantetext] - 2]]

    set sycheckresult [SyntaxCheckActionStruct $kantetext]

    if {$sycheckresult != 1} {
        win_dialog .d {Syntax Check Result} \
        "Syntax Error : \n\n$sycheckresult" \
        largewarn 0 {Continue}
        return 0
    }
    return 1
}


##  SyntaxCheckActionStruct
##-----------------------------------------------------------
##  Syntax-Check fuer B(PN)2-Aktionen
##  
##-----------------------------------------------------------
##
proc SyntaxCheckActionStruct {text} {


    set zustand 0

    for {set i 0} {$i < [string length $text]} {incr i} {
        set zeichen [string index $text $i]

        if {$zustand == 0} {
            if {$zeichen == " "} {
                return "An Action Must Not Begin With 'skip'."
            } elseif {($zeichen == "!") || ($zeichen == "?")} {
                return "An Action Has To Begin With A\
                        Channel, \nNot With '!' Or '?'."
            } else {
                set zustand 1
            }
        } elseif {$zustand == 1} {
            if {$zeichen == " "} {
                return "The Name Of A Channel Must Not Contain A 'skip'."
            } elseif {($zeichen == "!") || ($zeichen == "?")} {
                set zustand 2
            } else {
                set zustand 1
            }
        } elseif {$zustand == 2} {
            if {$zeichen != " "} {
                return "'!' Or '?' Must Be Followed By One 'skip'."
            } else {
                set zustand 3
            }
        } elseif {$zustand == 3} {
            if {$zeichen == " "} {
                return "'!' Or '?' Must Be Followed By Only One 'skip'."
            } elseif {($zeichen == "!") || ($zeichen == "?")} {
                return "An Action Must Contain Only One '!' Or '?'."
            } else {
                set zustand 4
            }
        } elseif {$zustand == 4} {
            if {$zeichen == " "} {
                return \
		    "The Action's Second Part Must Not Contain A 'skip'."
            } elseif {($zeichen == "!") || ($zeichen == "?")} {
                return "An Action Must Contain Only One '!' Or '?'."
            } else {
                set zustand 4
            }
        }
    }
    if {$zustand != 4} {
        return "Action Incomplete."
    }
    return 1
}


##  SyntaxResultCheck
##-----------------------------------------------------------
##  Teil des Syntax-Checkers : Ergebnis-Abfrage
##  
##-----------------------------------------------------------
##
proc SyntaxResultCheck {} {

    global editor opt

    catch {exec grep error $opt(filename.erg) >& $opt(filename.res)}
    if {[file size $opt(filename.res)] == 0} {
        return 1
    }
    set datei [open $opt(filename.res) r]

    win_dialog .d {Syntax Check Result} \
        "Syntax Error : \n\n[read $datei]" \
        largewarn 0 {Continue}
    close $datei

    return 0
}


## Hiernach folgen Routinen fuers Highlighten vom peptool aus

proc show_ref_in_pfa {str} {
    set erg [scan "-bpn=%s" pfa_str]
    puts [format "show_ref_in_pfa: arg=%s, ref_str = %s" $str $pfa_str]
    if ($erg == 1) {
        PFAdel_multi_sel
        highlightend
        DeleteMarksActFA
        foreach i $ref_str {
	    set erg [PFAHighlightEdge $i]
	    if {$erg != ""} {
		set ednumber [lindex $erg 0]
		set edge [lindex $erg 1]
		if {$faNumber(actual) != $ednumber} {
		    set faNumber(actual) $ednumber
		    TakeOtherFA $ednumber
		}
		set opt(redMarked) 1
		PFAhighlight_line $line
		PFAmulti_sel
		markedge $edge
	    }
	}
    }
}






