###########################################################################
###########################################################################
##                                                                       ##
##  Datei   :     `pfaBindings.tcl'                                      ##
##                -----------------                                      ##
##                                                                       ##
##  Inhalt  :     Prozeduren, die die Verkn"upfung der                   ##
##                Maustasten, Keyboardtasten und des                     ##
##                Mauszeigers mit bestimmten Funktionen                  ##
##                festlegen.                                             ##
##                                                                       ##
###########################################################################
###########################################################################



## DeleteBindings 
##-----------------------------------------------------------
## wird aufgerufen, nachdem in einen anderen Modus gewechselt 
## wurde und loescht alle Bindings, die mit der mittleren 
## und rechten Maustaste assoziiert sind, um keine falschen 
## Bindings im neuen Modus zu verwenden.
##-----------------------------------------------------------  
proc DeleteBindings {} { 
    global editor opt

    bind $editor(canvas) <3> ""
    bind $editor(canvas) <B3-Motion> ""
    bind $editor(canvas) <ButtonRelease-3> ""
    bind $editor(canvas) <2> ""
    bind $editor(canvas) <B2-Motion> ""
    bind $editor(canvas) <ButtonRelease-2> ""
    $editor(canvas) bind startknoten <3>  ""
    $editor(canvas) bind startknoten <2> ""
    $editor(canvas) bind startknoten <B3-Motion> ""
    $editor(canvas) bind startknoten <ButtonRelease-3> ""
    $editor(canvas) bind knoten <3>  ""
    $editor(canvas) bind knoten <2> ""
    $editor(canvas) bind knoten <B3-Motion> ""
    $editor(canvas) bind knoten <ButtonRelease-3> ""
    $editor(canvas) bind endknoten <3>  ""
    $editor(canvas) bind endknoten <2> ""
    $editor(canvas) bind endknoten <B3-Motion> ""
    $editor(canvas) bind endknoten <ButtonRelease-3> ""
    $editor(canvas) bind kante <3>  ""
    $editor(canvas) bind kante <2> ""
    $editor(canvas) bind kante <B3-Motion> ""
    $editor(canvas) bind kante <ButtonRelease-3> ""
    $editor(canvas) bind kantetext <3>  ""
    $editor(canvas) bind kantetext <2> ""
    $editor(canvas) bind kantetext <B3-Motion> ""
    $editor(canvas) bind kantetext <ButtonRelease-3> ""
    $editor(canvas) bind area <3>  ""
    $editor(canvas) bind area <2> ""
    $editor(canvas) bind area <B3-Motion> ""
    $editor(canvas) bind area <ButtonRelease-3> ""
    $editor(canvas) bind comment <3>  ""
    $editor(canvas) configure -cursor left_ptr
    set opt(mouse2) "Free"

    $editor(canvas) itemconfigure selected -fill white
    $editor(canvas) itemconfigure doubleselected -fill white
}


##  HotKeyBindings
##-----------------------------------------------------------
##  legt die Bindings der einzelnen Tastaturkuerzel fest. Die
##  Prozedur wird bei Erstellung jedes neuen Editors auf-
##  gerufen und einmal ganz am Anfang, wenn noch kein Editor
##  aktiviert ist. Wenn in diesem Zustand die Hot-Keys
##  verwendet werden, erscheint ein Meldungsfenster.
##-----------------------------------------------------------
##
proc HotKeyBindings { area } {
    global editor

    bind $area <Control-n> "AddNewFA" 
    bind $area <Control-o> "FilemanagerOpenFA" 
    bind $area <Control-r> "ReloadFA" 
    bind $area <Control-s> "SaveFA" 
    bind $area <Control-a> "FilemanagerSaveAs" 
    bind $area <Control-d> "CloseDialog" 
    bind $area <Control-i> "FilemanagerOpenProject" 
    bind $area <Control-j> "FilemanagerSaveProject" 
    bind $area <Control-q> "ExitDialog" 
    bind $area <Control-v> "PasteInit" 
    bind $area <Insert> "PasteInit" 
    bind $area <Control-u> "Unset" 
    bind $area <Control-y> "Undelete" 
    bind $area <Control-c> "set depp 0" 
    bind $area <Control-x> "set depp 0" 
    bind $area <Control-p> "Preferences" 
    bind $area <Control-f> "FontSelection" 
    bind $area <Control-z> "ZoomWindow" 
    bind $area <Control-t> "TextEditor" 
    bind $area <Control-w> "AskOptCode" 
    bind $area <Control-g> "PFAtoBPN2" 
    bind $area <Control-h> "PFAtoBPN2 yes" 
    bind $area <Control-m> "DeleteMarks" 

    bind $area <s> "StartVertexMode" 
    bind $area <v> "NormalVertexMode" 
    bind $area <f> "FinalVertexMode" 
    bind $area <e> "EdgeMode" 
    bind $area <c> "CommentMode" 
    bind $area <a> "AreaMode" 
    bind $area <d> "DeleteMode" 
    bind $area <t> "LineUpMode" 
    bind $area <m> "TextMoveMode"
    bind $area <r> "RenameMode" 
    bind $area <g> "ResetEdgeMode"  
}


##  DeleteKeyBindings
##-----------------------------------------------------------
##  l"oscht die Key-Bindings.
##-----------------------------------------------------------
##
proc DeleteKeyBindings { area } {
    global editor

    bind $area <Control-n> "" 
    bind $area <Control-o> "" 
    bind $area <Control-r> "" 
    bind $area <Control-s> "" 
    bind $area <Control-a> "" 
    bind $area <Control-d> "" 
    bind $area <Control-i> "" 
    bind $area <Control-j> "" 
    bind $area <Control-q> "" 
    bind $area <Control-v> "" 
    bind $area <Insert> "" 
    bind $area <Control-u> "" 
    bind $area <Control-y> "" 
    bind $area <Control-c> "" 
    bind $area <Control-x> "" 
    bind $area <Control-p> "" 
    bind $area <Control-f> "" 
    bind $area <Control-z> "" 
    bind $area <Control-t> "" 
    bind $area <Control-w> "" 
    bind $area <Control-g> "" 
    bind $area <Control-h> "" 
    bind $area <Control-m> "" 

    bind $area <s> "" 
    bind $area <v> "" 
    bind $area <f> "" 
    bind $area <e> ""
    bind $area <c> "" 
    bind $area <a> "" 
    bind $area <d> "" 
    bind $area <t> "" 
    bind $area <m> ""
    bind $area <r> "" 
    bind $area <g> ""  
}


##  FreeMode
##-----------------------------------------------------------
##  wird bei Erstellung eines neuen oder Wechsel in einen
##  anderen Editor aufgerufen, wenn noch kein Modus ein-
##  gestellt ist.
##-----------------------------------------------------------
##
proc FreeMode {} {
    global editor opt

    DeleteBindings
    set opt(mouse3) Free
}


##  MoveMode
##-----------------------------------------------------------
##  wird aufgerufen, nachdem ein neuer Editor erstellt wurde, 
##  um dauerhafte Bindings fuer dessen canvas festzulegen.
##-----------------------------------------------------------
##
proc MoveMode {} {
    global editor opt

    set opt(mouse1) Move

    ##  Die Bindings der Tastaturkuerzel werden fuer jeden
    ##  neuen Editor gesetzt.
    ##
    HotKeyBindings $editor(canvas)
  
    ##  Tastaturkuerzel, die nur Gueltigkeit besitzen, 
    ##  wenn bereits ein Editor aktiviert wurde
    ##
    bind $editor(canvas) <Delete> \
	{Delete [$editor(canvas) find withtag current]}
    bind $editor(canvas) <Home> "TakeFirstFA" 
    bind $editor(canvas) <End> "TakeLastFA" 
    bind $editor(canvas) <Control-plus> "TakeNextFA" 
    bind $editor(canvas) <Next> "TakeNextFA" 
    bind $editor(canvas) <Control-minus> "TakePreviousFA" 
    bind $editor(canvas) <Prior> "TakePreviousFA" 

    ##  Die Bindings zum Veraendern der Farbe 
    ##  der Elemente, wenn der Mauszeiger sie beruehrt.
    ##

    $editor(canvas) bind startknoten <Any-Enter> {
	VertexEnterColor [$editor(canvas) find withtag current]
    }
    $editor(canvas) bind startknoten <Any-Leave> {
	VertexLeaveColor [$editor(canvas) find withtag current]
    }

    $editor(canvas) bind knoten <Any-Enter> {
	VertexEnterColor [$editor(canvas) find withtag current]
    }
    $editor(canvas) bind knoten <Any-Leave> {
	VertexLeaveColor [$editor(canvas) find withtag current]
    }

    $editor(canvas) bind endknoten <Any-Enter> {
	VertexEnterColor [$editor(canvas) find withtag current]
    }
    $editor(canvas) bind endknoten <Any-Leave> {
	VertexLeaveColor [$editor(canvas) find withtag current]
    }
    $editor(canvas) bind kante <Any-Enter> {
        EdgeEnterColor [$editor(canvas) find withtag current]
        highlight [ItemBase [$editor(canvas) find withtag current]]
    }
    $editor(canvas) bind kante <Any-Leave> {
        EdgeLeaveColor [$editor(canvas) find withtag current]
        highlightend
    }

    $editor(canvas) bind comment <Any-Enter> { 
	$editor(canvas) itemconfigure current -fill red}
    $editor(canvas) bind comment <Any-Leave> { 
	$editor(canvas) itemconfigure current -fill black}


    ##  Die Bindings zum Verschieben der Elemente.
    ##
    $editor(canvas) bind startknoten <1> \
	    "XYSet %x %y;  VertexGet"
    $editor(canvas) bind startknoten <B1-Motion> \
	"XYSet %x %y; VertexMove"
    $editor(canvas) bind startknoten <ButtonRelease-1> \
	    "VertexSet"
    $editor(canvas) bind knoten <1> \
	    "XYSet %x %y;  VertexGet"
    $editor(canvas) bind knoten <B1-Motion> \
	    "XYSet %x %y; VertexMove"
    $editor(canvas) bind knoten <ButtonRelease-1> \
	    "VertexSet"
    $editor(canvas) bind endknoten <1> \
	    "XYSet %x %y;  VertexGet"
    $editor(canvas) bind endknoten <B1-Motion> \
	    "XYSet %x %y; VertexMove"
    $editor(canvas) bind endknoten <ButtonRelease-1> \
	    "VertexSet"



    $editor(canvas) bind kante <Button-1> { EdgeGet  }
    $editor(canvas) bind kante <B1-Motion> { XYSet %x %y; EdgeMove }
    $editor(canvas) bind kante <ButtonRelease-1> { EdgeSet } 

    $editor(canvas) bind comment <Button-1> { XYSet %x %y; CommentGet  }
    $editor(canvas) bind comment <B1-Motion> { XYSet %x %y; CommentMove }
    $editor(canvas) bind comment <ButtonRelease-1> { CommentSet } 

    $editor(canvas) bind area <1> {AreaMoveStart %x %y}
    $editor(canvas) bind area <B1-Motion> {AreaMove %x %y}
    $editor(canvas) bind area <ButtonRelease-1> {AreaMovePut}


    ##  Die Bindings zum Verschieben der gesamten Zeichenfl"ache.
    ##
    bind $editor(canvas) <Control-Button-1> {
	$editor(canvas) scan mark %x %y
    }
    bind $editor(canvas) <Control-B1-Motion> {
	$editor(canvas) scan dragto %x %y
    }
}


##  BitmapBindings
##-----------------------------------------------------------
##  Bindings fuer Erklaerung der Bedeutung der Bitmap-Button
##  im Textfenster links unten
##-----------------------------------------------------------
##
proc BitmapBindings {} {
    global opt

    bind .fr1a.fr2b.fr3.vertex \
	<Any-Enter> {set opt(messageText) "Make Vertex"}
    bind .fr1a.fr2b.fr3.startvertex\
	<Any-Enter> {set opt(messageText) "Make Start-Vertex"}
    bind .fr1a.fr2b.fr3.finalvertex\
    <Any-Enter> {set opt(messageText) "Make Final-Vertex"}
    bind .fr1a.fr2b.fr3.edge\
	<Any-Enter> {set opt(messageText) "Make Edge"}
    bind .fr1a.fr2b.fr3.comment\
	<Any-Enter> {set opt(messageText) "Make Comment"}
    bind .fr1a.fr2b.fr4.delete\
	<Any-Enter> {set opt(messageText) "Delete"}
    bind .fr1a.fr2b.fr4.settags\
	<Any-Enter> {set opt(messageText) "Set Tags"}
    bind .fr1a.fr2b.fr4.rename\
	<Any-Enter> {set opt(messageText) "Rename"}
    bind .fr1a.fr2b.fr4.copy\
	<Any-Enter> {set opt(messageText) "Area"}
    bind .fr1a.fr2b.fr3.vertex\
	<Any-Leave> {set opt(messageText) ""}
    bind .fr1a.fr2b.fr3.startvertex\
	<Any-Leave> {set opt(messageText) ""}
    bind .fr1a.fr2b.fr3.finalvertex\
	<Any-Leave> {set opt(messageText) ""}
    bind .fr1a.fr2b.fr3.edge\
	<Any-Leave> {set opt(messageText) ""}
    bind .fr1a.fr2b.fr3.comment\
	<Any-Leave> {set opt(messageText) ""}
    bind .fr1a.fr2b.fr4.delete\
	<Any-Leave> {set opt(messageText) ""}
    bind .fr1a.fr2b.fr4.settags\
	<Any-Leave> {set opt(messageText) ""}
    bind .fr1a.fr2b.fr4.rename\
	<Any-Leave> {set opt(messageText) ""}
    bind .fr1a.fr2b.fr4.copy\
	<Any-Leave> {set opt(messageText) ""}
    bind .fr1a.fr2m <Any-Leave> {catch {focus $editor(canvas)}}
}


##  DeleteBitmapBindings
##-----------------------------------------------------------
##  Bindings fuer Erkl"arung der Bedeutung der Bitmap-Button
##  im Textfenster links unten werden gel"oscht
##-----------------------------------------------------------
##
proc DeleteBitmapBindings {} {

    bind .fr1a.fr2b.fr3.vertex <Any-Enter>  ""
    bind .fr1a.fr2b.fr3.startvertex <Any-Enter> ""
    bind .fr1a.fr2b.fr3.finalvertex <Any-Enter> ""
    bind .fr1a.fr2b.fr3.edge <Any-Enter> ""
    bind .fr1a.fr2b.fr3.comment <Any-Enter> ""
    bind .fr1a.fr2b.fr4.delete <Any-Enter> ""
    bind .fr1a.fr2b.fr4.settags	<Any-Enter> ""
    bind .fr1a.fr2b.fr4.rename <Any-Enter> ""
    bind .fr1a.fr2b.fr4.copy <Any-Enter> ""
}
