/*
**************************************************************************
**************************************************************************
**                                                                      **
**  Datei   :    `pfaAction.c'                                          **
**               -------------                                          **
**                                                                      **
**  Inhalt  :    Funktionen, die die neuen Tcl-Befehle des              **
**               `Action'-Men"us implementieren.                        **
**                                                                      **
**************************************************************************
**************************************************************************
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "pfaCompile.h"
#include "pfaStructure.h"

/*
 *************************************************************************
 *
 *  Deklaration der lokalen Strukturen
 *
 *************************************************************************
 */

typedef struct LineInfo {
    int              ednumber;
    int              arc_id;
    IntList         *linelist;
    struct LineInfo *next;
} LineInfo;

typedef struct ChannelTableEntry {
    int        channel;
    IntList   *inputlist;
} ChannelTableEntry;


/*
 *************************************************************************
 *
 *  Deklaration der globalen Variablen
 *
 *************************************************************************
 */

Tcl_HashTable channelTable;
Tcl_HashTable inputTable;
Tcl_HashTable structTable; 

char     *globalDecl = "";
Queue     queue;
Queue     problemnodes;
int       linecounter;
LineInfo *lineinfo = NULL;
int       nrrealstartstates;
int       nrrealfinalstates;
int       exzulkanteendstart; 
char      buffer[LINE_LENGTH];
int       code;
int       ednumberglob;
char      fourspace[LINE_LENGTH]   = "    ";
char      twentyspace[LINE_LENGTH] = "                    ";


/*
 *##############################################################
 *
 *  Deklaration der lokalen Funtionen
 *
 *##############################################################
 */

int NumberEditors(void);

void PutLineInfo(int arc_id, int linenumber);

int Compile_OneEd(Tcl_Interp *interp, FILE *fp, Editor_type *editor,
		   char *parallelsymbol, int parnum);

int Compile_Struct_OneEd (FILE *fp, Editor_type *editor, int nrspace,
                          Tcl_Interp *interp);

void CutBuffer(void);

int ExistArcs (Editor_type *editor);

IntList *InArcIdList(IntList *inarcs, EdgeList_type *kante);

Vertex_type *StructStartPrint(FILE *fp, Vertex_type *vertexObject, 
                                 StructEntry *structentry, int nrspaces,
                                 int first, Tcl_HashTable placetable,
                                 Tcl_HashTable arctable);

Vertex_type *StructPrint(FILE *fp, Vertex_type *vertexObject, 
                            StructEntry *structentry, int nrspaces,
                            int first, int end,
                            Tcl_HashTable placetable,
                            Tcl_HashTable arctable);

Vertex_type *WSPFState(FILE *fp, int anzahlechtekanten,
                      Vertex_type *vertexObject, StructEntry *structentry,
                      int nrspaces, Tcl_HashTable placetable,
                      Tcl_HashTable arctable);

Vertex_type *StructPrintCircle(FILE *fp, Vertex_type *vertexObject,
                               int start_id, StructEntry *structentry,
                               int nrspaces, Tcl_HashTable placetable,
                               Tcl_HashTable arctable);

void StructPrintRepeat(FILE *fp, Vertex_type *vertexObject,
                       int nrspaces, Tcl_HashTable placetable,
                       Tcl_HashTable arctable);

void ChannelPrint(FILE *fp, int nrchan);

int PcPrint(Tcl_Interp *interp, FILE *fp, Editor_type *editor,
            int nrstates, char parallelsymbol[], int parnum);

void MultiStartPrint(FILE *fp, int nrstartstates, char brackets[]);

void SchlingenPrint(FILE *fp, Vertex_type *vertexObject,
                    Tcl_HashTable arctable,
                    int start_compile_id, char brackets[]);

void KantenPrint(FILE *fp, Vertex_type *vertexObject,
                 Tcl_HashTable arctable, Tcl_HashTable placetable,
                 int start_compile_id, char brackets[]);

void KantenStructPrint(FILE *fp, EdgeList_type *kante, int nrspaces,
                            Tcl_HashTable arctable);

void FinalPlacePrint(FILE *fp, Vertex_type *vertexObject, int start_compile_id,
                     char brackets[]);

int ExistArcsEntire (int ednumber);

int NrChanInput(int ednumber);

void GetChanPart(char label[], char chanpart[]);

void GetTextPart(char label[], char textpart[]);

void GetParts(char label[], char chanpart[], char mark[], char textpart[]);

void ClearLineInfo(void);

/*  GetWithoutBrackets
 *--------------------------------------------------------------
 *  liest eine Zeile bis zum Zeichen '>' ein und gibt einen  
 *  neuen String, der die eingelesenen Zeichen enth"alt, zur"uck.
 *--------------------------------------------------------------
 */
/* TODO: Hier kann wohl einiges einfacher gehen:
   strchr benutzen, ausnutzen, dass name=str beim Aufruf... */
static void GetWithoutBrackets(char *str, char **name)

{
    char *string_start;
    char *string;
    unsigned string_count;

    str++;

    string_start = str;
    for( string_count = 0; *str != '>'; string_count++, str++ )  {
    }

    string = malloc( sizeof( char *) * (string_count+1));
    strncpy( string, string_start, string_count);
    string[string_count] = '\0'; 
    str++;

    *name = string;
}

/*  PrintDeclGlobal
 *--------------------------------------------------------------
 *  schreibt die globalen Variablen in das entsprechende   
 *  B(PN)2-File.
 *--------------------------------------------------------------
 */

static int PrintDeclGlobal(Tcl_Interp *interp, FILE *fp)
{
    CONST char *string;
    char *newString;
    CONST char **listElems;
    int listSum, index;

    string = Tcl_GetVar2(interp, "opt", "globalDecl",
			TCL_GLOBAL_ONLY|TCL_LEAVE_ERR_MSG);
    if (!string)
	return TCL_ERROR;
    newString = strdup(string);

    if( Tcl_SplitList(interp, newString, &listSum, &listElems) != TCL_OK) {
	return TCL_ERROR;
    }

    for( index = 0; index < listSum; index++ ) {
        fprintf (fp, "    %s\n", listElems[index]);
        ++linecounter;
    }

    return TCL_OK;
}


/*  PrintDeclLocal
 *--------------------------------------------------------------
 *  schreibt die lokalen Variablen in das entsprechende   
 *  B(PN)2-File. Diese Prozedur wird aufgerufen, wenn der
 *  (Struktur)Compiler aufgerufen wird. space gibt die Einrcktiefe an.
 *--------------------------------------------------------------
 */

static int PrintDeclLocal(Tcl_Interp *interp, FILE *fp,
			  Editor_type *editor, int space) 
{
    char command[LINE_LENGTH];
    Tcl_Obj **listElems,
	     *objptr;
    int listSum, index;

    sprintf( command, "%d,decl",editor->number);
    objptr = Tcl_ObjGetVar2(interp, Tcl_NewStringObj("editor", -1),
		Tcl_NewStringObj(command, -1),
		TCL_GLOBAL_ONLY|TCL_LEAVE_ERR_MSG);
    if (!objptr)
	return TCL_ERROR;

    if( Tcl_ListObjGetElements(interp, objptr, &listSum, &listElems) != TCL_OK ) {
	return TCL_ERROR;
    }

    for( index = 0; index < listSum; index++ ) {
	if( space == 4 ) {
	    fprintf (fp, "    ");
	} else {
	    fprintf (fp, "        ");
	}
        fprintf(fp, "%s\n",Tcl_GetStringFromObj(listElems[index], NULL));
        ++linecounter;
    }
    return TCL_OK;
}

/*  tk_Highlight
 *--------------------------------------------------------------
 *  erstellt Referenzen zwischen Automat und Zielcode durch
 *  Einfuegen von Zeilennummern in die Kantenliste
 *--------------------------------------------------------------
 */

int tk_Highlight(ClientData cd, Tcl_Interp * interp, int objc,
                 Tcl_Obj *CONST objv[])
{
    int       ednumber;
    int       arc_id;
    LineInfo *helpli;
    IntList  *helpil;
    char      command[LINE_LENGTH];

    int i;
    Editor_type *editor;
    Tcl_HashTable  arctable;
    Edge_type   *edgeObject;
    Tcl_HashEntry *place_entryPtr;
    Tcl_HashSearch search;
    int editor_num;
    int max_editor_num;
    Tcl_Obj *objptr;

    if (objc != 3) {
	Tcl_WrongNumArgs(interp, 1, objv, "ednum arcid");
        return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[1], &ednumber ) != TCL_OK)
      return TCL_ERROR;
    /*
     *  Wir lassen uns die Editor-Nummer geben
     */

    objptr=Tcl_ObjGetVar2(interp, Tcl_NewStringObj("faNumber", -1),
		Tcl_NewStringObj("sum", -1), TCL_GLOBAL_ONLY);

    if(Tcl_GetIntFromObj (interp, objptr, &editor_num) != TCL_OK) {
        return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[2], &arc_id) != TCL_OK)
      return TCL_ERROR;

/* AS: Befehl fuers highlighten in dem BPN2-Editor ueber PEP-Tool absetzen */
    /* Editor heraussuchen */

    /*
     *  Wir lassen uns die Anzahl der Editoren geben
     */

    objptr=Tcl_ObjGetVar2(interp, Tcl_NewStringObj("faNumber", -1),
		Tcl_NewStringObj("sum", -1), TCL_GLOBAL_ONLY);
    if(Tcl_GetIntFromObj (interp, objptr, &max_editor_num) != TCL_OK) {
        return TCL_ERROR;
    }

    for (editor = firstEd, i = max_editor_num;
	 editor && (i > editor_num);
	 editor = editor->next, i-- );

    /* Jetzt alle Kanten durchgehen */
    arctable   = editor->edge_table;
    for (place_entryPtr = Tcl_FirstHashEntry (&arctable, &search), i = 1;
         place_entryPtr;
         place_entryPtr = Tcl_NextHashEntry(&search), i++ ) {
      edgeObject = Tcl_GetHashValue(place_entryPtr);

      if (edgeObject->e_id == arc_id && edgeObject->editor_num == editor_num) {
	sprintf(command, "highlight_in_bpn2 %d", edgeObject->save_id);
	if (Tcl_GlobalEval(interp, command) != TCL_OK) {
	  printf("Error while highlighting : command = %s\n", command);
	}
      }
    }
/* AS: Ende */
    helpli = lineinfo;
    while (helpli) {
        if ((helpli->ednumber == ednumber) && (helpli->arc_id == arc_id)) {
            sprintf (command, "highlight_lines {");
            for (helpil = helpli->linelist; helpil; helpil = helpil->next) {
                sprintf (command, "%s %d", command, helpil->id);
            }
            strcat(command, "}");
            Tcl_GlobalEval(interp, command);
            return TCL_OK;
        }
        helpli = helpli->next;
    }
    return TCL_OK;
}

/*  tk_GetEdgeToLine
 *--------------------------------------------------------------
 *  Gibt als Ergebnis die Kantennumer mit Editornummer zu
 *  einer gewaehlten Zeile
 *--------------------------------------------------------------
 */

int tk_GetEdgeToLine(ClientData cd, Tcl_Interp * interp,
                     int objc, Tcl_Obj *CONST objv[])
{
    int       line;
    LineInfo *helpli;
    IntList  *helpil;
    char      erg[LINE_LENGTH];

    if (objc != 2) {
	Tcl_WrongNumArgs(interp, 1, objv, "line");
        return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[1], &line) != TCL_OK)
      return TCL_ERROR;
    helpli = lineinfo;
    while (helpli) {
        helpil = helpli->linelist;
        while (helpil) {
            if (helpil->id == line) {
                sprintf (erg, "%d %d", helpli->ednumber, helpli->arc_id);
                interp->result = erg;
                return TCL_OK;
            }
            helpil = helpil->next;
        }
        helpli = helpli->next;
    }
    interp->result = "";
    return TCL_OK;
}

/*  tk_ExistEditor
 *--------------------------------------------------------------
 *  ueberprueft, ob ein Editor mit der Nummer des uebergebenen   
 *  Argumentes 'objv[1]' existiert.
 *--------------------------------------------------------------
 */

int tk_ExistEditor(ClientData cd, Tcl_Interp * interp, int objc,
                   Tcl_Obj *CONST objv[])
{
    int          number;
    Editor_type *editor;

    if (objc != 2) {
	Tcl_WrongNumArgs(interp, 1, objv, "editor");
        return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[1], &number) != TCL_OK)
      return TCL_ERROR;

    for ( editor = firstEd; editor; editor = editor->next ) {
        if (editor->number == number) {
            Tcl_SetIntObj(Tcl_GetObjResult(interp), 1);
            return TCL_OK;
        }
    }
    Tcl_SetIntObj(Tcl_GetObjResult(interp), 0);
    return TCL_OK;
}

/*  tk_Editors
 *--------------------------------------------------------------
 *  testet, ob mehr als ein Editor geoeffnet ist. 
 *--------------------------------------------------------------
 */

int tk_Editors(ClientData cd, Tcl_Interp * interp, int objc, 
	Tcl_Obj *CONST objv[])
{
    if (objc != 1) {
	Tcl_WrongNumArgs(interp, 1, objv, NULL);
        return TCL_ERROR;
    }

    if (NumberEditors() > 1) {
      Tcl_SetIntObj(Tcl_GetObjResult(interp), 1);
    } else {
      Tcl_SetIntObj(Tcl_GetObjResult(interp), 0);
    }
    return TCL_OK;
}

/*  NumberEditors
 *--------------------------------------------------------------
 *  berechnet die gesamte Anzahl der geoeffneten Editoren und  
 *  gibt diese ueber die Variable 'number' zurueck.
 *--------------------------------------------------------------
 */

int NumberEditors(void)
{
    int          number = 0;
    Editor_type *editor;

    for ( editor = firstEd; editor; editor = editor->next ) {
        ++number;
    }
    return number;
}

/*  tk_GetEdgeToRef
 *--------------------------------------------------------------
 *  Gibt als Ergebnis die Kantennumer mit Editornummer zu
 *  einer gewaehlten Referenz
 *--------------------------------------------------------------
 */

int tk_GetEdgeToRef(ClientData cd, Tcl_Interp * interp, \
                    int objc, Tcl_Obj *CONST objv[])
{
    int       line, i, j;
    char      erg[LINE_LENGTH];
    Editor_type *editor;
    Tcl_HashTable  arctable;
    Edge_type   *edgeObject;
    Tcl_HashEntry *place_entryPtr;
    Tcl_HashSearch search;

    if (objc != 2) {
	Tcl_WrongNumArgs(interp, 1, objv, "line");
        return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[1], &line) != TCL_OK)
      return TCL_ERROR;
    i=0;
    for (editor = firstEd; editor; editor = editor->next ) {
      arctable = editor->edge_table;
      j=0;
      for (place_entryPtr = Tcl_FirstHashEntry (&arctable, &search);
	   place_entryPtr != NULL;
	   place_entryPtr = Tcl_NextHashEntry(&search) ) {
        edgeObject = Tcl_GetHashValue(place_entryPtr);
	if (edgeObject->save_id == line)
	  sprintf (erg, "%d %d", i, j);
	j++;
      }
      i++;
    }  
    interp->result = erg;
    return TCL_OK;
}

/*  tk_GetEdgeToLine
 *--------------------------------------------------------------
 *  Gibt als Ergebnis die Kantennumer mit Editornummer zu
 *  einer gewaehlten Zeile
 *--------------------------------------------------------------
 */

int tk_HighlightEdge(ClientData cd, Tcl_Interp * interp, \
                     int objc, Tcl_Obj *CONST objv[])
{
    int i,j,arc_id;
    Editor_type *editor;
    Tcl_HashTable  arctable;
    Edge_type   *edgeObject;
    Tcl_HashEntry *place_entryPtr;
    Tcl_HashSearch search;

    if (objc != 2) {
	Tcl_WrongNumArgs(interp, 1, objv, "edge");
        return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[1], &arc_id) != TCL_OK)
      return TCL_ERROR;

    /* Editor heraussuchen */
    for (editor = firstEd, i = 1; editor; editor = editor->next, i++ )
      skip;

    /* Jetzt alle Kanten durchgehen */
    arctable   = editor->edge_table;
    for (place_entryPtr = Tcl_FirstHashEntry (&arctable, &search), j = 1;
         place_entryPtr;
         place_entryPtr = Tcl_NextHashEntry(&search), i++ ) {
      edgeObject = Tcl_GetHashValue(place_entryPtr);
      if (edgeObject->e_id == arc_id) {
	sprintf(interp->result, "{ %d %d }", i, j);
	return TCL_OK;
      }
    }
    interp->result = "";
    return TCL_OK;
}

/*  PutLineInfo
 *--------------------------------------------------------------
 *   Fuegt die Zeilennummer in die Kantenliste, Teil des 
 *   Erstellens von Referenzen zwischen FA und Zielcode
 *--------------------------------------------------------------
 */

void PutLineInfo(int arc_id, int linenumber)
{
    LineInfo *lilocal;
    IntList  *helpil;
    LineInfo *helpli;

    if (!lineinfo) {
        lineinfo = malloc(sizeof(LineInfo));
        lineinfo->ednumber = ednumberglob;
        lineinfo->arc_id = arc_id;
        lineinfo->linelist = malloc(sizeof(IntList));
        lineinfo->linelist->id = linenumber;
        lineinfo->linelist->next = NULL;
        lineinfo->next = NULL;
        return;
    }
    lilocal = lineinfo;
    while (lilocal) {
        if ((lilocal->ednumber == ednumberglob) &&
            (lilocal->arc_id == arc_id)) {
            helpil = malloc(sizeof(IntList));
            helpil->id = linenumber;
            helpil->next = lilocal->linelist;
            lilocal->linelist = helpil;
            return;
        }
        lilocal = lilocal->next;
    }
    helpli = malloc(sizeof(LineInfo));
    helpli->ednumber = ednumberglob;
    helpli->arc_id = arc_id;
    helpli->linelist = malloc(sizeof(IntList));
    helpli->linelist->id = linenumber;
    helpli->linelist->next = NULL;
    helpli->next = lineinfo;
    lineinfo = helpli;
}

/*  tk_Compile
 *--------------------------------------------------------------
 *  Von Tcl aus aufzurufende Funktion zur Verwaltung der 
 *  Uebersetzung mit dem Compiler zur Uebersetzung aller mit
 *  pepPFA editierten Systeme von endlichen Automaten.
 *  Verwaltung des Ergebnisses.
 *--------------------------------------------------------------
 */

int tk_Compile(ClientData cd, Tcl_Interp * interp, int objc, 
	Tcl_Obj *CONST objv[])
{
    FILE        *fp;
    int          ednumber;
    int          nrchan;
    Editor_type *editor;
    char        *parallelsymbol = "    \n";
    int          parnum = 1;
    
    if (objc != 4) {
	Tcl_WrongNumArgs(interp, 1, objv, "code editor filename");
        return TCL_ERROR;
    }

    /*
     * code == 1 -> B(PN)2-Code,
     * == 0 -> Struct-Code 
     */

    if (Tcl_GetIntFromObj(interp, objv[1], &code) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetIntFromObj(interp, objv[2], &ednumber) != TCL_OK)
      return TCL_ERROR;

    /*
     * Ausgabedatei
     */

    if ((fp = fopen(Tcl_GetStringFromObj(objv[3], NULL), "w")) == NULL) {
	printf("Error while opening file\n");
	return TCL_ERROR;
    }

    /*
     * Generieren der Tabelle zur Variablenverawltung
     */

    if (code == 0) {
        nrchan = NrChanInput(ednumber);
    } else {
        nrchan = ExistArcsEntire(ednumber);
    }

    if (nrchan == -1) {
        fclose(fp);
        Tcl_SetVar(interp, "compile_answer", "{ {FA-WS-EmptyWindow} }", TCL_GLOBAL_ONLY);
        return TCL_OK;
    }

    /*
     * Initialisiere Liste und Zaehler fuer Zeilen sowie changed 
     * -> Highlighten moeglich 
    */

    ClearLineInfo();
    Tcl_ObjSetVar2(interp, Tcl_NewStringObj("opt(changed)", -1), NULL,
		Tcl_NewStringObj("no", -1), TCL_GLOBAL_ONLY|TCL_PARSE_PART1);
    linecounter = 1;

    fprintf (fp, "begin\n");
    ++linecounter;
    if (code == 0) {
        ChannelPrint(fp, nrchan); 
    } else {
	if( PrintDeclGlobal(interp, fp) != TCL_OK ) {
	    return TCL_ERROR;
	}
    }
    if (ednumber == 0) {
        /*
	 * mehrere zu uebersetzende Automaten
         */

        for (editor = firstEd; editor; editor = editor->next) {
            if (ExistArcs(editor) == 1) {
                ednumberglob = editor->number;

		/*
		 * Uebersetzung eines Automaten
		 */
                if( Compile_OneEd(interp, fp, editor,
				  parallelsymbol, parnum) != TCL_OK ) {
		    return TCL_ERROR;
		}
                parallelsymbol = "||\n\n";
                parnum = 2;
           }
        }
    } else {

	/*
         * ednumber gibt actFANumber des zu uebersetzenden FAs an
	 */

        for (editor = firstEd; editor; editor = editor->next) {
            if (editor->number == ednumber) {
                ednumberglob = ednumber;

		/*
		 * Uebersetzung eines Automaten
		 */

                if( Compile_OneEd(interp, fp, editor,
				  parallelsymbol, parnum) != TCL_OK ) {
		    return TCL_ERROR;
		}
                exit(0);
            }
        }
    }
    fprintf (fp, "end\n");
    ++linecounter;
    fclose(fp);
    if (code == 0) {
        ClearSymbolTables();
    }
    Tcl_SetVar(interp, "compile_answer", "{ {CompileDone} }", TCL_GLOBAL_ONLY);
    return TCL_OK;
}

/*  Compile_OneEd
 *--------------------------------------------------------------
 *  Uebersetzung eines einzelnen Automaten. 
 *  Wird von tk_Compile aufgerufen.
 *--------------------------------------------------------------
 */

int Compile_OneEd(Tcl_Interp *interp, FILE *fp, Editor_type *editor,
		   char *parallelsymbol, int parnum)
{
    int            nrstartstates;
    int            nrstates;
    Vertex_type    *vertexObject;
    int            start_compile_id;
    Tcl_HashEntry *place_entryPtr;
    Tcl_HashSearch search;
    Tcl_HashTable  placetable;
    Tcl_HashTable  arctable;
    char           brackets[LINE_LENGTH] = "  ";

    placetable = editor->vertex_table;
    arctable   = editor->edge_table;
    nrstartstates = NrStartStates(placetable);
    nrstates = NrStates(placetable, nrstartstates);
    if( PcPrint(interp, fp, editor, nrstates-1,
		parallelsymbol, parnum) != TCL_OK ) {
	return TCL_ERROR;
    }
    MultiStartPrint(fp, nrstartstates, brackets);
    for ( place_entryPtr = Tcl_FirstHashEntry (&placetable, &search);
          place_entryPtr;
          place_entryPtr = Tcl_NextHashEntry(&search) ) {
        vertexObject = (Vertex_type *) Tcl_GetHashValue(place_entryPtr);
        start_compile_id = vertexObject->compile_id;
        SchlingenPrint(fp, vertexObject, arctable, start_compile_id,
                       brackets);
        KantenPrint(fp, vertexObject, arctable, placetable,
                    start_compile_id, brackets);
        FinalPlacePrint(fp, vertexObject, start_compile_id, brackets);
    }
    /* BG fprintf (fp, "\n        od\n\n");
    linecounter+=3;
    fprintf (fp, "    end\n\n");
    linecounter+=2; */
    fprintf (fp, "        od\n");
    linecounter++;
    fprintf (fp, "    end\n");
    linecounter++;

    return TCL_OK;
}

/*  tk_Compile_Struct
 *--------------------------------------------------------------
 *  Von Tcl aus aufzurufende Verwaltungsfunktion des Compilers
 *  zur Uebersetzung von Systemen von endlichen Automaten der
 *  Klasse NFAStruct.
 *  Von hier wird die Funktion Compile_Struct_OneEd aufgerufen,
 *  um jeweils einen Automaten des Systems zu uebersetzen.
 *  Verwaltung des Ergebnisses.
 *--------------------------------------------------------------
 */

int tk_Compile_Struct(ClientData cd, Tcl_Interp * interp,
                          int objc, Tcl_Obj *CONST objv[])

{
    FILE        *fp;
    int          ednumber;
    int          nrchan;
    Editor_type *editor;
    char        *parallelsymbol = "        \n";
    int          parnum = 1;

    /* ednumber gibt die Editor-Nummer des zu uebersetzenden FA an,
       bei ednumber == 0 werden alle FAs uebersetzt.
    */
    if (objc != 4) {
	Tcl_WrongNumArgs(interp, 1, objv, "code editor filename");
        return TCL_ERROR;
    }

    /*
     * code == 1 -> B(PN)2-Code,
     * == 0 -> Struct-Code 
     */

    if (Tcl_GetIntFromObj(interp, objv[1], &code) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetIntFromObj(interp, objv[2], &ednumber) != TCL_OK)
      return TCL_ERROR;

    /*
     * Ausgabedatei
     */

    if ((fp = fopen(Tcl_GetStringFromObj(objv[3], NULL), "w")) == NULL) {
		printf("Error while opening file\n");
	return TCL_ERROR;
    }
    if (code == 0) {
        /* Anlegen der Variablen-Tabelle bei Channel-Notation */
        nrchan = NrChanInput(ednumber);
    } else {
        /* ueberpruefe, ob Kanten existieren */
        nrchan = ExistArcsEntire(ednumber);
    }
    if (nrchan == -1) {
        fclose(fp);
        Tcl_SetVar(interp, "compile_answer", "{ {FA-WS-EmptyWindow} }", TCL_GLOBAL_ONLY);
        return TCL_OK;
    }
    /* 
     * Initialisiere Liste und Zaehler fuer Zeilen sowie changed 
     * -> Highlighten moeglich 
     */

    ClearLineInfo();
    linecounter = 1;
    
    fprintf (fp, "begin\n");
    ++linecounter;

    /* Ausgabe der globalen Variablendeklaration */
    if (code == 0) {
        ChannelPrint(fp, nrchan); 
    } else {
	if( PrintDeclGlobal(interp, fp) != TCL_OK ) {
	    return TCL_ERROR;
	}
    }
    if (NumberEditors() == 1) {
        /* nur ein Automat existent */
	if( PrintDeclLocal(interp, fp, firstEd, 4 ) != TCL_OK )
	    return TCL_ERROR;
        ednumberglob = firstEd->number;
        if (Compile_Struct_OneEd(fp, firstEd, 4, interp) == 0) {
            if (code == 0) {
                ClearSymbolTables();
            }
            ClearQueue(problemnodes);
            return TCL_OK;
        }
        CutBuffer();
        fprintf (fp, buffer);
    } else if (ednumber == 0) {
        /* alle Automaten sollen uebersetzt werden */
        for ( editor = firstEd; editor; editor = editor->next ) {
            if (ExistArcs(editor) == 1) {
                *buffer='\0';
                fprintf (fp, "%s\n", parallelsymbol);
                linecounter += parnum;
                linecounter++;
                fprintf (fp, "    begin\n");
                linecounter++;
		if(PrintDeclLocal(interp, fp, editor, 8) != TCL_OK )
		    return TCL_ERROR;
                ednumberglob = editor->number;

		/* Uebersetzung jeweils eines einzelnen FA */
                if (Compile_Struct_OneEd(fp, editor, 8, interp) == 0) {
                    if (code == 0) {
                        ClearSymbolTables();
                    }
                    ClearQueue(problemnodes);
                    return TCL_OK;
                }
                CutBuffer();
                fprintf (fp, buffer);
                /* BG fprintf (fp, "\n    end\n");
                linecounter+=2; */
                fprintf (fp, "    end\n");
                linecounter++;
                parallelsymbol = "\n    ||\n";
                parnum = 2;
            }
        }
    } else {
        /* ednumber gibt actFANumber des zu uebersetzenden FAs an */
        for ( editor = firstEd; editor; editor = editor->next ) {
            if (editor->number == ednumber) {
                /* BG sprintf(buffer, "\n\n");
                linecounter+=2; */
                sprintf(buffer, "\n");
                linecounter++;
                if(PrintDeclLocal(interp, fp, editor, 8) != TCL_OK )
		    return TCL_ERROR;
                ednumberglob = ednumber;

		/* Uebersetzung des FA */
                if (Compile_Struct_OneEd(fp, editor, 4, interp) == 0) {
                    if (code == 0) {
                        ClearSymbolTables();
                    }
                    ClearQueue(problemnodes);            
                    return TCL_OK;
                }
                CutBuffer();
                fprintf (fp, buffer);
                exit(0);
            }
        }
    }
    /* BG fprintf (fp, "\nend\n");
    linecounter+=2; */
    fprintf (fp, "end\n");
    linecounter++;
    fclose(fp);
    if (code == 0) {
        ClearSymbolTables();
    }
    Tcl_ObjSetVar2(interp, Tcl_NewStringObj("opt(changed)", -1), NULL,
		Tcl_NewStringObj("no", -1), TCL_GLOBAL_ONLY|TCL_PARSE_PART1);
    Tcl_ObjSetVar2(interp, Tcl_NewStringObj("compile_answer", -1), NULL,
		Tcl_NewStringObj("{ {CompileDone} }", -1), TCL_GLOBAL_ONLY);
    return TCL_OK;
}

/*  CutBuffer
 *--------------------------------------------------------------
 *  Loeschen eines Semikolons aus dem Zeichenspeicher, wenn 
 *  die Syntax von B(PN)2 kein Semikolon zulaesst
 *--------------------------------------------------------------
 */

void CutBuffer(void)
{
    int  i = 0;

    while (i <= LINE_LENGTH) {
        if (buffer[i] == ';') {
            buffer[i] = ' ';
            exit(0);
        }
        ++i;
    }
}

/*  ExistArcs
 *--------------------------------------------------------------
 *  ueberprueft, ob im Hashtable 'arctable' Kanten gespeichert
 *  sind oder ob er leer ist.
 *--------------------------------------------------------------
 */

int ExistArcs (Editor_type *editor) 
{
    Tcl_HashTable  arctable;
    Tcl_HashSearch search;

    arctable = editor->edge_table;
    if (!Tcl_FirstHashEntry (&arctable, &search)) {
        return 0;
    }
    return 1;
}

/*  Compile_Struct_OneEd
 *--------------------------------------------------------------
 *  Uebersetzt einen einzelnen Automaten der Klasse NFAStruct. 
 *  Aufruf durch tk_Compile_Struct.
 *  Arbeitsweise in zwei Laeufen.
 *--------------------------------------------------------------
 */

int Compile_Struct_OneEd (FILE *fp, Editor_type *editor, int nrspace,
                          Tcl_Interp *interp)
{
    Tcl_HashTable  placetable;
    Tcl_HashTable  arctable;
    QueueCell     *queueptr;
    Vertex_type    *check;
    Vertex_type    *startstate;
    char           command[LINE_LENGTH];
    StructEntry   *structentry;

    placetable = editor->vertex_table;
    arctable   = editor->edge_table;
    startstate = GetStartState(placetable);
    nrrealstartstates = NrRealStartStates(placetable);
    if ( nrrealstartstates == 0 ) {
        return 1;
    }
    nrrealfinalstates = NrRealFinalStates(placetable);
    exzulkanteendstart = ExZulKanteEndStart(placetable, arctable);
    if (exzulkanteendstart == -1) {
        fclose(fp);
	/* TODO: Hier soll wohl die Datei geloescht werden. filename
	   ist aber unbekannt und daher nicht zu benutzen! */
        /*fp = fopen(filename, "w");
        fclose(fp);
	*/
        sprintf(command, "{FA-WS-FaultLabel %d}", editor->number);
        Tcl_SetVar(interp, "compile_answer", command, TCL_GLOBAL_ONLY);
        Tcl_GlobalEval(interp, command);            
        return 0;
    }
    
    /*
     * Initialisierung der Schlange
     */

    queue.front = NULL;
    queue.end   = NULL;
    problemnodes.front = NULL;
    problemnodes.end = NULL;

    /*
     * Aufruf der Hauptfunktion des ersten Laufs
     */

    if (StructNaming(startstate, placetable, arctable) == 0) {
        /*
	 * FA ist nicht in der Klasse NFAStruct
	 */

        fclose(fp);
	/* dito, s.o. 
        fp = fopen(filename, "w");
        fclose(fp);
	*/
        sprintf(command, "{ { Mark-FA-WS %d {", editor->number);
        for (queueptr = problemnodes.front; queueptr;
             queueptr = queueptr->next) {
            sprintf(command, "%s %d", command,
                                      queueptr->vertexObject->v_id);
        }
        strcat(command, "} } }");
        Tcl_SetVar(interp, "compile_answer", command, TCL_GLOBAL_ONLY);
        ClearStructTable();
        return 0;
    }
    structentry = malloc(sizeof(StructEntry));
    structentry->canvas_id = -(startstate->v_id);
    structentry->count     = 2;
    structentry->number    = 2;
    structentry->next      = NULL;
    check = StructStartPrint(fp, startstate, structentry, nrspace, 1,
                                 placetable, arctable);
    if (check->type != 3) {
        printf("StructPrint does not return end vertices!\n");
    }
    ClearStructTable();
    return 1;
}

/*  CopyKante
 *--------------------------------------------------------------
 *  erstellt ein neues Element vom Typ 'EdgeList_type' und 
 *  uebergibt diesem die Informationen des Parameters 'kante'.
 *--------------------------------------------------------------
 */

EdgeList_type *CopyKante(EdgeList_type *kante)
{
    EdgeList_type *newkante;

    newkante = malloc(sizeof(EdgeList_type));
    newkante->edge_object = NULL;
    newkante->arc_id = kante->arc_id;
    newkante->other_vertex = NULL;
    newkante->other_pid = kante->other_pid;
    newkante->next = NULL;
    return newkante;
}

/*  CopyArcIdElem
 *--------------------------------------------------------------
 *  erstellt ein neues Element vom Typ 'IntList' und uebergibt
 *  diesem die Informationen des Parameters 'kante'.
 *--------------------------------------------------------------
 */

IntList *CopyArcIdElem(IntList *kante) 
{
    IntList *newarc;

    newarc = malloc(sizeof(IntList));
    newarc->id = kante->id;
    newarc->next = NULL;
    return newarc;
}

/*  StructStartPrint
 *--------------------------------------------------------------
 *  Ausgangsfunktion des zweiten Laufs. Diese Funktion ruft  
 *  die Funktion StructPrint auf, die die Hauptverwaltung
 *  des zweiten Laufs uebernimmt
 *--------------------------------------------------------------
 */

Vertex_type *StructStartPrint(FILE *fp, Vertex_type *vertexObject, 
                                 StructEntry *structentry, int nrspaces,
                                 int first, Tcl_HashTable placetable,
                                 Tcl_HashTable arctable)
{
    Tcl_HashEntry *place_entryPtr;
    Tcl_HashSearch search;
    Vertex_type    *check=NULL;
    int            firststate = 1;
    char           space[LINE_LENGTH] = "";
    int            i;

    for (i = 4; i <= nrspaces; i += 4) 
        strcat(space, fourspace);

    if (exzulkanteendstart == 1) {
        /* Zulaessige Kante von End zu Start : zweimal durchlaufen */
        fprintf (fp, buffer);
        sprintf (buffer, "%sdo  <true> enter\n", space);
        ++linecounter;
        if ( GetRepeatKanten(vertexObject) ||
             (AnzahlEchteKanten(vertexObject) != 1) ) {
            fprintf(fp, buffer);
            sprintf (buffer, "%s    <TRUE>;\n", space);
            ++linecounter;
        }
        check = StructPrint(fp, vertexObject, CopyStructList(structentry),
                            (nrspaces + 4), 1, 0, placetable, arctable);
        fprintf(fp, buffer);
        fprintf (fp, "%s    repeat\n", space);
        ++linecounter;
        sprintf (buffer, "%s[]", space);
        if ( GetRepeatKanten(vertexObject) ||
             (AnzahlEchteKanten(vertexObject) != 1) ) {
            fprintf(fp, buffer);
            sprintf (buffer, "%s    <TRUE>;\n", space);
            ++linecounter;
        }
        check = StructPrint(fp, vertexObject, CopyStructList(structentry),
                                (nrspaces + 4), 1, 0, placetable, arctable);
        fprintf (fp, buffer);
        fprintf (fp, "%s    exit\n", space);
        ++linecounter;
        sprintf (buffer, "%sod;\n", space);
        ++linecounter;
        ClearStructList(structentry);
        return check;
    }

    if ( nrrealstartstates == 1) {
         check = StructPrint(fp, vertexObject, CopyStructList(structentry), 
			     nrspaces, 1, 0, placetable, arctable);
    } else {
        fprintf (fp, buffer);
        sprintf (buffer, "%sdo  <true> enter\n", space);
        ++linecounter;
        for ( place_entryPtr = Tcl_FirstHashEntry (&placetable, &search);
            place_entryPtr;
            place_entryPtr = Tcl_NextHashEntry(&search) ) {
            vertexObject = Tcl_GetHashValue(place_entryPtr);
            if (vertexObject->type == 1) {
                if (CountOut(vertexObject) > 0) {
                    if (firststate == 0) {
                        fprintf (fp, buffer);
                        sprintf (buffer, "%s[]", space);
                    }
                    firststate = 0;
                    fprintf (fp, buffer);
                    sprintf (buffer, "%s    <TRUE>;\n", space);
                    ++linecounter;
                    structentry->canvas_id = -(vertexObject->v_id);
                    check = StructPrint(fp, vertexObject,
                                        CopyStructList(structentry),
                                        (nrspaces + 4), 1, 0,
                                        placetable, arctable);
                    fprintf (fp, buffer);
                    sprintf (buffer, "%s    exit\n", space);
                    ++linecounter;
                }
            }
        }
        fprintf (fp, buffer);
        sprintf (buffer, "%sod;\n", space);
        ++linecounter;
    }
    ClearStructList(structentry);
    return check;
}

/*  StructPrint
 *--------------------------------------------------------------
 *  Verwaltungsfunktion des zweiten Laufs zur Generierung des 
 *  B(PN)2-Zielcodes fuer den Struct-Compiler
 *--------------------------------------------------------------
 */

Vertex_type *StructPrint(FILE *fp, Vertex_type *vertexObject, 
                            StructEntry *structentry, int nrspaces,
                            int first, int end,
                            Tcl_HashTable placetable,
                            Tcl_HashTable arctable)
{
    EdgeList_type  *kante;    
    Vertex_type    *ziel_object;
    int            i; 
    char           space[LINE_LENGTH] = "";
    Vertex_type    *check=NULL;
    PlaceMarkList *placemarklist;
    EdgeList_type  *repeatedges;
    int            firstkante;
    StructEntry   *structentrynew;
    int            anzahlechtekanten;

    for (i = 4; i <= nrspaces; i += 4) 
        strcat(space, fourspace);
    repeatedges = GetRepeatKanten(vertexObject);
    if (end == 0) {
      if ( vertexObject->own_edge || repeatedges ) {
          StructPrintRepeat(fp, vertexObject, nrspaces, placetable, arctable);
      }
    }
    if ((end == 1) || (nrrealfinalstates == 1) || (vertexObject->type != 3)) {
        if (!vertexObject->out) {
            ClearStructList(structentry);
            return vertexObject;
        } else {
            anzahlechtekanten = AnzahlEchteKanten(vertexObject);
            if ( (anzahlechtekanten == 1) && (exzulkanteendstart == 1) && 
                 (vertexObject->type == 3) ) {
                ClearStructList(structentry);
                return vertexObject;
            }
            if ((anzahlechtekanten == 1) || 
                ((anzahlechtekanten == 2) && (exzulkanteendstart == 1)
                  && (vertexObject->type == 3)) ) {
                kante = Get1EchteKante(vertexObject);
                ziel_object = GetKantenZiel(kante, placetable, arctable);
        if ( (vertexObject->own_edge || repeatedges ) &&
             (vertexObject->type != 3) ) {
            KantenStructPrint(fp, kante, (nrspaces + 4), arctable);
            fprintf (fp, buffer);
            fprintf (fp, "    %sexit\n", space);
            ++linecounter;
            sprintf (buffer, "%sod;\n", space);
            ++linecounter;
        } else {
            KantenStructPrint(fp, kante, nrspaces, arctable);
        }
                if (!ziel_object->in->next) {
                    check = StructPrint(fp, ziel_object,
                                        CopyStructList(structentry),
                                        nrspaces, first, 0, placetable,
                                        arctable);
                } else {
                    if ( BranchClose(ziel_object, structentry) == 1) {
                        repeatedges = GetRepeatKanten(ziel_object);
                        if ( (ziel_object->type == 3) && (first == 1) &&
                             ( ziel_object->own_edge || repeatedges ) ) {
                            StructPrintRepeat(fp, ziel_object, nrspaces,
                                                   placetable, arctable);
                        }
                        ClearStructList(structentry);
                        return ziel_object;
                    } else {
                        check = StructPrint(fp, ziel_object,
                                            CopyStructList(structentry),
                                            nrspaces, 0, 0, placetable,
                                            arctable);
                    }
                }
            } else {
                if (!((vertexObject->own_edge || repeatedges ) && 
		      (vertexObject->type != 3))) {
                    fprintf (fp, buffer);
                    sprintf (buffer, "%sdo  <true> enter\n", space);
                    ++linecounter;
                }
                firstkante = 1;            
                for (kante = vertexObject->out; kante; kante = kante->next) {
                    if (InRepeatKanten(kante, vertexObject) == 1) {
                        continue;
                    }
                    if (firstkante == 0) {
                        fprintf (fp, buffer);
                        sprintf (buffer, "%s[]", space);
                    }
                    firstkante = 0;
                    ziel_object = GetKantenZiel(kante, placetable, arctable);
                    KantenStructPrint(fp, kante, (nrspaces + 4), arctable);
                    structentrynew = malloc(sizeof(StructEntry));
                    structentrynew->canvas_id = vertexObject->v_id;
                    structentrynew->count    = 0;
                    structentrynew->number   = 0;
                    structentrynew->next     = CopyStructList(structentry);
                    if (!ziel_object->in->next) {
                        check = StructPrint(fp, ziel_object,
                                            structentrynew, nrspaces+4,
                                            0, 0, placetable, arctable);
                    } else {
                        if (BranchClose(ziel_object, structentrynew) == 1) {
                            check = ziel_object;
                        } else {
                            /* Sonderfall behandeln !!! */ 
                            check = StructPrint(fp, ziel_object,
                                    CopyStructList(structentrynew),
                                    nrspaces+4, 0, 0, placetable,
                                    arctable);
                        }
                    }
                    fprintf (fp, buffer);
                    sprintf (buffer, "%s    exit\n", space);
                    ++linecounter;
                }
                placemarklist = GetPlaceMarkList(check);
                if (!placemarklist) {
                    repeatedges = NULL;
                } else {
                    repeatedges = placemarklist->repeatedges;
                }
                if ( BranchClose(check, structentry) == 1) {
                    fprintf (fp, buffer);
                    sprintf (buffer, "%sod;\n", space);
                    ++linecounter;
                    if ( (check->type == 3) && (first == 1) &&
                         (check->own_edge || repeatedges) ) {
                        StructPrintRepeat(fp, check, nrspaces,
                                               placetable, arctable);
                    }
                    ClearStructList(structentry);
                    return check;
                }
                fprintf (fp, buffer);
                sprintf (buffer, "%sod;\n", space);
                ++linecounter;
                if (check->type != 3) {
                    check = StructPrint(fp, check, CopyStructList(structentry),
                            nrspaces, 0, 0, placetable, arctable);
                }
                repeatedges = GetRepeatKanten(check);
                if ( (check->type == 3) &&
                     (check->own_edge || repeatedges) ) {
                    StructPrintRepeat(fp, check, nrspaces,
                                           placetable, arctable);
                }
            }
        }
        vertexObject = check;
    } else {
        check = vertexObject;
    }
    anzahlechtekanten = AnzahlEchteKanten(vertexObject);
    if ( (nrrealfinalstates > 1) && (anzahlechtekanten > 0) ) {
        check = WSPFState(fp, anzahlechtekanten, vertexObject,
                          CopyStructList(structentry), nrspaces,
                          placetable, arctable);
    }
    ClearStructList(structentry);
    return check;
}

/*  WSPFState
 *--------------------------------------------------------------
 *  Funktion zur Behandlung von Endknoten im zweiten Lauf des 
 *  Struct-Compilers
 *--------------------------------------------------------------
 */

Vertex_type *WSPFState(FILE *fp, int anzahlechtekanten,
                      Vertex_type *vertexObject, StructEntry *structentry,
                      int nrspaces, Tcl_HashTable placetable,
                      Tcl_HashTable arctable)
{
    StructEntry *structentrynew;
    char         space[LINE_LENGTH] = "";
    int          i;
       
    for (i = 4; i <= nrspaces; i += 4) 
        strcat(space, fourspace);

    structentrynew = malloc(sizeof(StructEntry));
    structentrynew->canvas_id = -(vertexObject->v_id);
    structentrynew->count     = 2;
    structentrynew->number    = 2;
    structentrynew->next      = structentry;
    structentry = structentrynew;

    fprintf (fp, buffer);
    fprintf (fp, "%sdo  <true> enter\n", space);
    ++linecounter;
    fprintf (fp, "%s    <TRUE>;\n", space);
    ++linecounter;
    fprintf (fp, "%s    exit\n", space);
    ++linecounter;
    sprintf (buffer, "%s[]", space);
    if (anzahlechtekanten > 1) {
        fprintf (fp, buffer);
        sprintf (buffer, "%s    <TRUE>;\n", space);
        ++linecounter;
    }
    vertexObject =  StructPrint(fp, vertexObject, structentry,
                                nrspaces+4, 1, 1, placetable, arctable);    
    fprintf (fp, buffer);
    fprintf (fp, "%s    exit\n", space);
    ++linecounter;
    sprintf (buffer, "%sod;\n", space);
    ++linecounter;
    return vertexObject;
}

/*  StructPrintCircle
 *--------------------------------------------------------------
 *  Funktion des zweiten Laufs des Struct-Compilers zur 
 *  Bearbeitung von Zyklen
 *--------------------------------------------------------------
 */

Vertex_type *StructPrintCircle(FILE *fp, Vertex_type *vertexObject,
                          int start_id,
                          StructEntry *structentry, int nrspaces,
                          Tcl_HashTable placetable, Tcl_HashTable arctable)
{
    EdgeList_type  *kante;
    Vertex_type    *ziel_object;
    int            i; 
    char           space[LINE_LENGTH] = "";
    Vertex_type    *check=NULL;
    EdgeList_type  *repeatedges;
    int            first;
    StructEntry   *structentrynew;

    for (i = 4; i <= nrspaces; i += 4) 
        strcat(space, fourspace);

    if (vertexObject->v_id == start_id) {
        ClearStructList(structentry);
        return vertexObject;
    }

    repeatedges = GetRepeatKanten(vertexObject);

    if ( vertexObject->own_edge || repeatedges ) {
        StructPrintRepeat(fp, vertexObject, nrspaces, placetable, arctable);
    }
    if (!vertexObject->out) {
        ClearStructList(structentry);
        return vertexObject;
    } else {
        if (AnzahlEchteKanten(vertexObject) == 1) {
            kante = Get1EchteKante(vertexObject);
            ziel_object = GetKantenZiel(kante, placetable, arctable);
        if ( vertexObject->own_edge || repeatedges ) {
            KantenStructPrint(fp, kante, (nrspaces + 4), arctable);
            fprintf (fp, buffer);
            fprintf (fp, "    %sexit\n", space);
            ++linecounter;
            sprintf (buffer, "%sod;\n", space);
            ++linecounter;
        } else {
            KantenStructPrint(fp, kante, nrspaces, arctable);
        }
            if (!ziel_object->in->next) {
                check = StructPrintCircle(fp, ziel_object, start_id,
                                          CopyStructList(structentry),
                                          nrspaces,
                                          placetable, arctable);
            } else {
               if ( BranchClose(ziel_object, structentry) == 1) {
                    ClearStructList(structentry);
                    return ziel_object;
                } else {
                    check = StructPrintCircle(fp, ziel_object, start_id, \
                                          CopyStructList(structentry), \
                                          nrspaces, placetable, arctable);
                }
            }
        } else {
            first = 1;
            if (!(vertexObject->own_edge || repeatedges)) {
                fprintf (fp, buffer);
                sprintf (buffer, "%sdo  <true> enter\n", space);
                ++linecounter;
            }
            for ( kante = vertexObject->out; kante; kante = kante->next ) {
                if (InRepeatKanten(kante, vertexObject) == 1) {
                    continue;
                }
                if (first == 0) {
                    fprintf (fp, buffer);
                    /* BG sprintf (buffer, "%s[]\n", space);
                    ++linecounter; */
                    sprintf (buffer, "%s[]", space);
                }
                first = 0;
                ziel_object = GetKantenZiel(kante, placetable, arctable);
                KantenStructPrint(fp, kante, (nrspaces + 4), arctable);
                structentrynew = malloc(sizeof(StructEntry));
                structentrynew->canvas_id = vertexObject->v_id;
                structentrynew->count    = 0;
                structentrynew->number   = 0;
                structentrynew->next     = CopyStructList(structentry);
                if (!ziel_object->in->next) {
                    check = StructPrintCircle(fp, ziel_object, start_id, \
                                           structentrynew, nrspaces+4, \
                                           placetable, arctable);
                } else {
                    if ( BranchClose(ziel_object, structentrynew) == 1) {
                        check = ziel_object;
                    } else {
                        /* Sonderfall behandeln !!! */ 
                        check = StructPrintCircle(fp, ziel_object, \
                                start_id, \
                                CopyStructList(structentrynew), \
                                nrspaces+4, placetable, arctable);
                    }
                }
                fprintf (fp, buffer);
                sprintf (buffer, "%s    exit\n", space);
                ++linecounter;
            }
            repeatedges = GetRepeatKanten(vertexObject);
            if ( BranchClose(check, structentry) == 1) {
                fprintf (fp, buffer);
                sprintf (buffer, "%sod;\n", space);
                ++linecounter;
                ClearStructList(structentry);
                return check;
            }
            fprintf (fp, buffer);
            sprintf (buffer, "%sod;\n", space);
            ++linecounter;
            if (check->v_id != start_id) {
                return StructPrintCircle(fp, check, start_id, \
                                         structentry,\
                                         nrspaces, placetable, arctable);
            }
        }
    }
    ClearStructList(structentry);
    return check;
}

/*  StructPrintRepeat
 *--------------------------------------------------------------
 *  Teil des zweiten Laufs zur Ueberstzung von zyklischen
 *  Strukturen.
 *--------------------------------------------------------------
 */

void StructPrintRepeat(FILE *fp, Vertex_type *vertexObject, int nrspaces,
                       Tcl_HashTable placetable, Tcl_HashTable arctable)
{
    EdgeList_type *kante;
    char           space[LINE_LENGTH] = "";
    int            i;
    EdgeList_type *repeatedges;
    Vertex_type   *ziel_object;
    Vertex_type   *check;
    int            first = 1;

    for (i = 4; i <= nrspaces; i += 4) 
        strcat(space, fourspace);

    repeatedges = GetRepeatKanten(vertexObject);
    fprintf (fp, buffer);
    sprintf (buffer, "%sdo  <true> enter\n", space);
    ++linecounter;
    if (vertexObject->type == 3) {
        fprintf (fp, buffer);
        fprintf (fp, "    %s<TRUE>;\n", space);
        ++linecounter;
        sprintf (buffer, "    %sexit\n", space);
        ++linecounter;
    }

    if (vertexObject->own_edge) {
        if (vertexObject->type == 3) {
            fprintf (fp, buffer);
            sprintf (buffer, "%s[]", space);
        }
        kante = vertexObject->own_edge;
        while (kante) {
            if (first == 0) {
                fprintf (fp, buffer);
                sprintf (buffer, "%s[]", space);
            }
            first = 0;
            ziel_object = GetKantenZiel(kante, placetable, arctable);
            KantenStructPrint(fp, kante, (nrspaces + 4), arctable);
            fprintf (fp, buffer);
            sprintf (buffer, "    %srepeat\n", space);
            ++linecounter;
            kante = kante->next;
        }
        if (vertexObject->type != 3) {
            fprintf (fp, buffer);
            sprintf (buffer, "%s[]", space);
        }
    }
    while (repeatedges) {
        if (vertexObject->type == 3) {
            fprintf (fp, buffer);
            sprintf (buffer, "%s[]", space);
        }
        ziel_object = GetKantenZiel(repeatedges, placetable, arctable);
        KantenStructPrint(fp, repeatedges, (nrspaces + 4), arctable);
        check = StructPrintCircle(fp, ziel_object, vertexObject->v_id, \
                              NULL, nrspaces + 4, placetable, arctable);
        if ( (check->type != 3) && (check->v_id != vertexObject->v_id) ) {
            printf("Error in StructPrintRepeat.\n");
        }
        fprintf (fp, buffer);
        sprintf (buffer, "    %srepeat\n", space);
        ++linecounter;
        repeatedges = repeatedges->next;
        if (vertexObject->type != 3) {
            fprintf (fp, buffer);
            sprintf (buffer, "%s[]", space);
        }
    }
    if (vertexObject->type == 3) {
        fprintf (fp, buffer);
        sprintf (buffer, "%sod;\n", space);
        ++linecounter;
    }
}

/*  ChannelPrint
 *--------------------------------------------------------------
 *  Ausgabe der Variablendeklaration bei gewaehlter Channel-
 *  Notation.
 *--------------------------------------------------------------
 */

void ChannelPrint(FILE *fp, int nrchan) 
{
    Tcl_HashSearch     search;
    Tcl_HashEntry     *chan_entry;
    ChannelTableEntry *chanobject;
    IntList           *inputlist;
    int                i;
  
    for ( i = 0; i < nrchan; ++i) {
        chan_entry = Tcl_FirstHashEntry (&channelTable, &search);
        chanobject = (ChannelTableEntry *) Tcl_GetHashValue(chan_entry);
        while (chanobject->channel != i) {
            chan_entry = Tcl_NextHashEntry(&search);
            chanobject = (ChannelTableEntry *) Tcl_GetHashValue(chan_entry);
        }
        inputlist = chanobject->inputlist;
        if (i < 10) {
            fprintf (fp, "    var channel%d   : chan 0 of {%d", \
                     i, inputlist->id);
        } else if ((9 < i) && (i < 100)) {
            fprintf (fp, "    var channel%d  : chan 0 of {%d", \
                     i, inputlist->id);
        } else {
            fprintf (fp, "    var channel%d : chan 0 of {%d", \
                     i, inputlist->id);
        }
        inputlist = inputlist->next;
        while (inputlist) {
            fprintf (fp, ",%d", inputlist->id);
            inputlist = inputlist->next;
        }
        fprintf (fp, "};\n");
        ++linecounter;
    }
}

/*  PcPrint
 *--------------------------------------------------------------
 *  Ausgabe des lokalen Programmzaehlers als lokale Variablen-
 *  Deklaration bei Einsatz des allgemeinen Compilers. 
 *--------------------------------------------------------------
 */

int PcPrint(Tcl_Interp *interp, FILE *fp, Editor_type *editor, \
            int nrstates, char parallelsymbol[], int parnum)
{
    fprintf (fp, "    %s", parallelsymbol);
    linecounter += parnum; 
    fprintf (fp, "    begin\n");
    ++linecounter;

    fprintf (fp, "        var   pc : {0..%d} init 0;\n",nrstates);
    ++linecounter;
    if( PrintDeclLocal(interp, fp, editor, 8) != TCL_OK )
	return TCL_ERROR;

    fprintf (fp, "        do  <true> enter\n");
    ++linecounter;

    return TCL_OK;
}

/*  MultiStartPrint
 *--------------------------------------------------------------
 *  Behandlung von mehreren Startknoten bei Einsatz des 
 *  allgemeinen Compilers.
 *--------------------------------------------------------------
 */

void MultiStartPrint(FILE *fp, int nrstartstates, char brackets[])
{
    int i;
        
    brackets = "  ";
    if (nrstartstates > 1) {
        for (i = 1;
             i <= nrstartstates;
             ++i) {
            fprintf (fp, "        %s <'pc = 0 and pc' = %d>; repeat\n", \
                     brackets, i);
            ++linecounter;
            brackets[0] = '[';
            brackets[1] = ']';
        }
    }
}

/*  SchlingenPrint
 *--------------------------------------------------------------
 *  Behandlung von Schlingen bei Einsatz des allgemeinen
 *  Compilers.
 *--------------------------------------------------------------
 */

void SchlingenPrint(FILE *fp, Vertex_type *vertexObject, \
                    Tcl_HashTable arctable,\
                    int start_compile_id, char brackets[])
{
    EdgeList_type  *kante;
    char           *kante_label;
    char            chanpart[LINE_LENGTH] = "";
    char            textpart[LINE_LENGTH] = "";
    char            mark[LINE_LENGTH] = "";
    int             chansymbol;
    int             textsymbol;

    sprintf(chanpart, twentyspace);
    sprintf(textpart, twentyspace);

    for (kante = vertexObject->own_edge; kante; kante = kante->next) {
        kante_label = strdup(GetKantenLabel(kante, arctable));
        if (strcmp(kante_label, "") != 0) {
            if (code == 0) {
                GetParts(kante_label,chanpart,mark,textpart);
                chansymbol = *GetChanObject(chanpart);
                textsymbol = *GetTextObject(textpart);
                fprintf (fp, \
                "        %s <pc = %d and channel%d%s = %d>; repeat\n",\
                    brackets, start_compile_id, chansymbol, mark, textsymbol);
                PutLineInfo(kante->arc_id, linecounter);
                ++linecounter;
            } else {
                GetWithoutBrackets(kante_label, &kante_label);
                fprintf (fp, "        %s <pc = %d and (%s)>; repeat\n", \
                         brackets, start_compile_id, kante_label);
                PutLineInfo(kante->arc_id, linecounter);
                ++linecounter;
            }
        } else {
            fprintf (fp, "        %s <pc = %d>; repeat\n", \
                     brackets, start_compile_id);
            PutLineInfo(kante->arc_id, linecounter);
            ++linecounter;
        }
        brackets[0] = '[';
        brackets[1] = ']';
    }
}

/*  KantenPrint
 *--------------------------------------------------------------
 *  Uebersetzung eines Kantenlabels bei Einsatz des allgemeinen 
 *  Compilers.
 *--------------------------------------------------------------
 */

void KantenPrint(FILE *fp, Vertex_type *vertexObject, \
                 Tcl_HashTable arctable, Tcl_HashTable placetable, \
                 int start_compile_id, char brackets[])
{
    EdgeList_type  *kante;    
    char           *kante_label;
    int             ziel_compile_id;
    char            chanpart[LINE_LENGTH] = "";
    char            textpart[LINE_LENGTH] = "";
    char            mark[LINE_LENGTH] = "";
    int             chansymbol;
    int             textsymbol; 

    for ( kante = vertexObject->out; kante; kante = kante->next ) {
        sprintf(chanpart, twentyspace);
        sprintf(textpart, twentyspace);

        ziel_compile_id = (GetKantenZiel(kante, placetable, \
                                                arctable))->compile_id;
        kante_label = strdup(GetKantenLabel(kante, arctable));
        if (strcmp(kante_label, "") != 0) {
            if (code == 0) {
                GetParts(kante_label, chanpart, mark, textpart);
                chansymbol = *GetChanObject(chanpart);
                textsymbol = *GetTextObject(textpart);
                fprintf (fp, \
               "        %s <'pc = %d and channel%d%s = %d and pc' = %d>; \
               repeat\n",\
               brackets, start_compile_id, chansymbol, mark, textsymbol, \
               ziel_compile_id);
                PutLineInfo(kante->arc_id, linecounter); 
                ++linecounter;
            } else {
                GetWithoutBrackets(kante_label, &kante_label);
                fprintf (fp, \
                "        %s <'pc = %d and (%s) and pc' = %d>; repeat\n",\
                brackets, start_compile_id, kante_label, \
                ziel_compile_id);
                PutLineInfo(kante->arc_id, linecounter);
                ++linecounter;
            }
        } else {
            fprintf (fp, "        %s <'pc = %d and pc' = %d>; repeat\n", \
            brackets, start_compile_id, ziel_compile_id);
            PutLineInfo(kante->arc_id, linecounter);
            ++linecounter;
        }
        brackets[0] = '[';
        brackets[1] = ']';
    }
}

/*  KantenStructPrint
 *--------------------------------------------------------------
 *  Uebersetzung eines Kantenlabels bei Einsatz des
 *  Struktur-Compilers.
 *--------------------------------------------------------------
 */

/* BG !!! Hier Ausgabe + Referenzen einbauen */
void KantenStructPrint(FILE *fp, EdgeList_type *kante, int nrspaces, \
                            Tcl_HashTable arctable)
{
    char *kante_label;
    char  chanpart[LINE_LENGTH] = "";
    char  textpart[LINE_LENGTH] = "";
    char  mark[LINE_LENGTH] = "";
    int   chansymbol;
    int   textsymbol;
    char  space[LINE_LENGTH] = "";
    int   i;

    for (i = 4; i <= nrspaces; i += 4) {
        strcat(space, fourspace);
    }
    sprintf(chanpart, twentyspace);
    sprintf(textpart, twentyspace);

    kante_label = GetKantenLabel(kante, arctable);
    if (strcmp(kante_label, "") != 0) {
        if (code == 0) {
            GetParts(kante_label,chanpart,mark,textpart);
            chansymbol = *GetChanObject(chanpart);
            textsymbol = *GetTextObject(textpart);
            fprintf (fp, buffer);
            sprintf (buffer, "%s<channel%d%s = %d>;\n", \
                     space, chansymbol, mark, textsymbol);
            PutLineInfo(kante->arc_id, linecounter);
            ++linecounter;
        } else {
            fprintf (fp, buffer);
            sprintf (buffer, "%s%s;\n", space, kante_label);
            PutLineInfo(kante->arc_id, linecounter);
            ++linecounter;
        }
    } else {
        fprintf (fp, buffer);
        sprintf (buffer, "%s<TRUE>;\n", space);
        PutLineInfo(kante->arc_id, linecounter);
        ++linecounter;
    }
}

/*  FinalPlacePrint
 *--------------------------------------------------------------
 *  Behandlung von Endknoten bei Einsatz des allgemeinen  
 *  Compilers.
 *--------------------------------------------------------------
 */

void FinalPlacePrint(FILE *fp, Vertex_type *vertexObject, \
                     int start_compile_id, char brackets[])
{
    if (vertexObject->type == 3) {
        fprintf (fp, "        %s <pc = %d>; exit\n", \
                 brackets, start_compile_id);
        ++linecounter;
        brackets[0] = '[';
        brackets[1] = ']';
    }
}

/*  ExistArcsEntire
 *--------------------------------------------------------------
 *  Testet, ob im gesamten System von endlichen Automaten 
 *  zu uebersetzende Kante vorhanden sind.
 *--------------------------------------------------------------
 */

int ExistArcsEntire (int ednumber)
{
    Tcl_HashEntry     *arc_entryPtr;
    Tcl_HashSearch     search;
    Editor_type       *editor;
    Tcl_HashTable      arctable;

    /* gib -1 wenn keine zu uebersetzende Kante vorhanden, 1 sonst */
    for ( editor = firstEd; editor; editor = editor->next ) {
        if ( (ednumber != 0) && (ednumber != editor->number) ) {
            /* 
             * wenn ednumber != 0 -> nur fuer den mit ednumber angegebenen
             * Editor abarbeiten
             */
             continue;
         }
         arctable = editor->edge_table;   
         for ( arc_entryPtr = Tcl_FirstHashEntry (&arctable, &search);
               arc_entryPtr;
               arc_entryPtr = Tcl_NextHashEntry(&search) ) {
             return 1;
        }
   }
    return -1;
}

/*  NrChanInput
 *--------------------------------------------------------------
 *  Legt bei vorgewaehlter Channel-Notation fuer die Variablen
 *  in den Kantenlabel die Uebersetzungstabell an und gibt 
 *  die Anzahl der Variablen zurueck.
 *--------------------------------------------------------------
 */

int NrChanInput(int ednumber) 
{
    int                channumber = 0;
    ChannelTableEntry *channeltableentry;
    int                inputnumber = 0;
    int               *inputPtr;
    Edge_type          *edge_object;
    char               arc_label[LINE_LENGTH];
    char              *arc_label2;
    Tcl_HashEntry     *channelentryPtr;
    Tcl_HashEntry     *input_entryPtr;
    Tcl_HashEntry     *arc_entryPtr;
    Tcl_HashSearch     search;
    int                channew;
    int                textnew;
    Editor_type       *editor;
    Tcl_HashTable      arctable;
    char               chanpart[LINE_LENGTH] = "";
    char               textpart[LINE_LENGTH] = "";
    IntList           *inputlist;
    IntList           *inputlist2nd;
    int                arcs = 0;
    int                inputtochan;

    Tcl_InitHashTable( &channelTable, TCL_STRING_KEYS);
    Tcl_InitHashTable( &inputTable,   TCL_STRING_KEYS);

    sprintf(chanpart, twentyspace);
    sprintf(textpart, twentyspace);

    for ( editor = firstEd; editor; editor = editor->next ) {
        if ( (ednumber != 0) && (ednumber != editor->number) ) {
            /* wenn ednumber != 0 -> nur fuer den mit ednumber angegebenen
             * Editor abarbeiten */
            continue;
        }
        arctable = editor->edge_table;   
        for ( arc_entryPtr = Tcl_FirstHashEntry (&arctable, &search);
              arc_entryPtr;
              arc_entryPtr = Tcl_NextHashEntry(&search) ) {
            sprintf(chanpart, twentyspace);
            sprintf(textpart, twentyspace);
            arcs = 1;
            edge_object = Tcl_GetHashValue(arc_entryPtr);
            arc_label2 = edge_object->label;
            sprintf(arc_label, edge_object->label);
            if (strcmp(arc_label2,"") != 0) {
                GetChanPart(arc_label, chanpart);
                GetTextPart(arc_label, textpart);
                channelentryPtr = \
                Tcl_CreateHashEntry(&channelTable, chanpart, &channew);
                input_entryPtr = Tcl_CreateHashEntry(&inputTable, \
                                                     textpart, &textnew);
                if ( textnew ) {
                    inputPtr = malloc(sizeof(int));
                    *inputPtr = inputnumber;
                    inputtochan = inputnumber;
                    Tcl_SetHashValue(input_entryPtr, inputPtr);
                    ++inputnumber;
                } else {
                    inputtochan = \
                    *((int *) Tcl_GetHashValue(input_entryPtr));
                }

                if ( channew ) {
                    channeltableentry = malloc(sizeof(ChannelTableEntry));
                    channeltableentry->channel = channumber;
                    inputlist = malloc(sizeof(IntList));
                    inputlist->id = inputtochan;
                    inputlist->next = NULL;
                    channeltableentry->inputlist = inputlist;
                    Tcl_SetHashValue(channelentryPtr, channeltableentry);
                    ++channumber;
                } else {
                    channeltableentry = Tcl_GetHashValue(channelentryPtr);
                    inputlist = channeltableentry->inputlist;
                    inputlist2nd = inputlist;
                    if (inputlist->id <= inputtochan) {
                        inputlist = inputlist->next;
                        while ( inputlist && (inputlist->id <= inputtochan) ) {
                            inputlist2nd = inputlist;
                            inputlist = inputlist->next;
                        }
                        if (inputlist2nd->id != inputtochan) {
                            inputlist = malloc(sizeof(IntList));
                            inputlist->id = inputtochan;
                            inputlist->next = inputlist2nd->next;
                            inputlist2nd->next = inputlist;
                        }
                    } else {
                        inputlist = malloc(sizeof(IntList));
                        inputlist->id = inputtochan;
                        inputlist->next = inputlist2nd;
                        channeltableentry->inputlist = inputlist;
                    }       
                }
            }
        }
    }
    if (arcs == 0) {
        return -1;
    }
    return channumber;
}

/*  GetChanPart
 *--------------------------------------------------------------
 *  Liefert den Channel-Teil einer Kantenbeschriftung bei 
 *  vorgewaehlter Channel-Notation
 *--------------------------------------------------------------
 */

void GetChanPart(char label[], char chanpart[]) 
{
    int i = 0;  

    while ( (label[i] != '?') && (label[i] != '!') ) {
        chanpart[i] = label[i];
        ++i;
    }
}

/*  GetTextPart
 *--------------------------------------------------------------
 *  Liefert den zweiten Teil einer Kantenbeschriftung bei 
 *  vorgewaehlter Channel-Notation.
 *--------------------------------------------------------------
 */

void GetTextPart(char label[], char textpart[]) 
{
    int i = 0;
    int j = 0;

    while ( (label[i] != '?') && (label[i] != '!') ) 
        ++i;
    ++i;
    while (i < LINE_LENGTH) {
        textpart[j++] = label[i++];
    }    
}

/*  GetParts
 *--------------------------------------------------------------
 *  Zerlegt die Kantenbeschriftung bei vorgewaehlter
 *  Channel-Notation.
 *--------------------------------------------------------------
 */

void GetParts(char label[], char chanpart[], char mark[], char textpart[]) 
{
    int i = 0;
    int j = 0;

    while ( (label[i] != '?') && (label[i] != '!') ) {
        chanpart[i] = label[i];
        ++i;
    }
    mark[0] = label[i++];
    while (i < LINE_LENGTH) {
        textpart[j++] = label[i++];
    }    
}

/*  GetKantenZiel
 *--------------------------------------------------------------
 *  liefert einen Zeiger auf die Struktur des Zielknotens 
 *  der Kante 'kante' zurueck.
 *--------------------------------------------------------------
 */

Vertex_type *GetKantenZiel( EdgeList_type *kante, Tcl_HashTable placetable,
			    Tcl_HashTable arctable)
{
    char        arc_kante[LINE_LENGTH];
    Edge_type   *kante_object;
    char        knoten_ziel[LINE_LENGTH];

    sprintf(arc_kante, "arc%d", kante->arc_id);
    kante_object = GetKanteObject(arctable, arc_kante);
    sprintf(knoten_ziel, "knoten%d", kante_object->end);
    return GetPlaceObject(placetable, knoten_ziel);
}

/*  GetKantenLabel
 *--------------------------------------------------------------
 *  sucht im Hash-Table 'arctable' nach einer Kante mit der  
 *  Id 'kante->arc_id' und liefert deren Label zurueck.
 *--------------------------------------------------------------
 */

char *GetKantenLabel( EdgeList_type *kante, Tcl_HashTable arctable)
{
    char      arc_kante[LINE_LENGTH];
    Edge_type *kante_object;

    sprintf(arc_kante, "arc%d", kante->arc_id);
    kante_object = GetKanteObject (arctable, arc_kante);
    return kante_object->label;
}

/*  
 *##############################################################
 *--------------------------------------------------------------
 *  Die folgenden Prozeduren l"oschen die verwendeten Hash-Table
 *  bzw. Listen und geben den gesamten belegten Speicherplatz 
 *  frei.
 *--------------------------------------------------------------
 *##############################################################
 */

void ClearLineInfo(void)
{
    LineInfo *help;

    while (lineinfo) {
        help = lineinfo;
        lineinfo = lineinfo->next;
        ClearIntList(help->linelist);
        free(help);
    }
}

