/*
***************************************************************************
***************************************************************************
**                                                                       **
**  Datei   :    `pfaEdit.c'                                             **
**               -----------                                             **
**                                                                       **
**  Inhalt  :    Funktionen, die die neuen Tcl-Befehle des               **
**               `Edit'-Men"us implementieren.                           **
**                                                                       **
***************************************************************************
***************************************************************************
*/

#include <stdio.h>	
#include <string.h>		
#include <stdlib.h>
#include <math.h>
#include "pfa.h"

/*
 ##########################################################################
 #                                                                        #
 #  Deklaration der lokalen Funktionen                                    #
 #                                                                        #
 ##########################################################################
 */

Vertex_type  *createVertexObject( Vertex_type *vertexObject );

Edge_type    *createEdgeObject( Edge_type *edgeObject );

Comment_type *createCommentObject( Comment_type *commentObject );

/*
 ##########################################################################
 #                                                                        #
 #  C-Befehlsfunktion zum Men"upunkt:                                     #
 #                                                                        #
 #  Cut                                                                   #
 #                                                                        #
 ##########################################################################
 */

/*  tk_Cut
 *--------------------------------------------------------------
 *   
 *  l"oscht die Elemente des markierten Bereiches von der 
 *  Zeichenfl"ache und kopiert sie in einen Zwischenspeicher.
 *
 *--------------------------------------------------------------
 */

int tk_Cut(ClientData cd, Tcl_Interp * interp,\
		  int objc, Tcl_Obj *CONST objv[])
{
    Tcl_Obj       **listElems;
    int             cid, listSum, index;
    double	    x;
    Comment_type   *commentObject;
    Comment_type   *copyCommentObject;
    EdgeList_type  *arcs;
    Edge_type      *edgeObject;
    Edge_type      *copyEdgeObject;
    List_element   *compareListElement;
    List_element   *list_element;
    Undelete_type  *undeleteElement;
    Vertex_type    *vertexObject;
    Vertex_type    *copyvertexObject;
    Vertex_type    *comparevertexObject;

    if (objc != 5) {
	Tcl_WrongNumArgs(interp, 1, objv, "nodelist commentlist x y");
        return TCL_ERROR;
    }

    /* 
     *  Die alten kopierten Elemente muessen vollstaendig entfernt werden
     */

    if( copyObject != NULL ) {
	listFree( copyObject->copyVertexList );
	listFree( copyObject->copyEdgeList );
	listFree( copyObject->copyCommentList );
	free( copyObject );
    }

    /* 
     *  Ein neues Objekt fuer die kopierten Elemente wird angelegt
     */

    copyObject = MALLOC( Copy_type );
    if (Tcl_GetDoubleFromObj( interp, objv[3], &x ) != TCL_OK)
      return TCL_ERROR;
    copyObject->area_start.x = x;
    if (Tcl_GetDoubleFromObj( interp, objv[4], &x ) != TCL_OK)
      return TCL_ERROR;
    copyObject->area_start.y = x;
    copyObject->copyVertexList =createList();
    copyObject->copyEdgeList = createList();
    copyObject->copyCommentList = createList();

    /*  Ein Listenelement der Undelete-Liste wird erstellt
     *  und am Anfang der Liste eingefuegt
     */

    undeleteElement = makeUndeleteElement();
    if( undeleteElement == NULL ) {
	return TCL_ERROR;
    }

    /*
     *  Die kopierten Knoten werden der Reihe nach in die Knoten-Liste
     *  des Copy-Objektes uebertragen
     */

    if( Tcl_ListObjGetElements( interp, objv[1], &listSum,\
		       &listElems ) != TCL_OK ) {
	return TCL_ERROR;
    }

    index = 0;
    while( index < listSum ) {
        if (Tcl_GetIntFromObj(interp, listElems[index++], &cid ) != TCL_OK)
          return TCL_ERROR;
	sprintf(interp->result, "knoten%d",cid);
	vertexObject =  getVertexStruct( interp->result);
	if( vertexObject == NULL ) {
	    return TCL_ERROR;
	}

	/*
	 *  Die Struktur des kopierten Knoten muss uebertragen werden
	 */
	copyvertexObject = createVertexObject( vertexObject );

	/* 
	 *  Der kopierte Knoten wird in die Knoten-Liste des Copy-Objektes
	 *  eingefuegt
	 */
	copyvertexObject = listFirst( copyObject->copyVertexList,\
					    copyvertexObject );
    }
    //free( (char *) listElems );

    /* 
     *  Wenn die Knoten-Liste des Copy-Objektes nicht leer ist muessen
     *  die evtl. kopierten Kanten behandelt werden
     */

    if( copyObject->copyVertexList->head->next !=\
	copyObject->copyVertexList->head ) {
	for( list_element  = copyObject->copyVertexList->head->next;
	     list_element != copyObject->copyVertexList->head;
	     list_element  = list_element->next ) {
	    vertexObject = (Vertex_type *) (list_element->data);

	    /* 
	     *  Zuerst werden die Schlingen behandelt. Da sie jeweils nur 
	     *  zu einem Knoten gehoeren, koennen sie komplett uebernommen
	     *  werden. 
	     */

	    arcs = vertexObject->own_edge;
	    while( arcs != NULL ) {
		sprintf(interp->result, "arc%d",arcs->edge_object->e_id);
		edgeObject =  getEdgeStruct( interp->result);
		if( edgeObject == NULL ) {
		    return TCL_ERROR;
		}

		/*
		 *  Die Strukturen der zum jeweiligen Knoten gehoerenden
		 *  Schlingen werden komplett kopiert ...
		 */

		copyEdgeObject = createEdgeObject( edgeObject );

		/* 
		 *  ... und in die Kanten-Liste des Copy-Objektes
		 *  uebertragen.
		 */

		copyEdgeObject = listFirst( copyObject->copyEdgeList,\
					    copyEdgeObject );
		arcs = arcs->next;
	    }

	    /* 
	     *  Dann werden die normalen Kanten des Knotens behandelt.
	     *  Diese werden nur aufgenommen, wenn der zweite zur Kante 
	     *  gehoerende Knoten ebenfalls in der kopierten Area liegt.
	     *  Speziell wird fuer jede ausgehende Kante jedes in der
	     *  Area befindlichen Knotens die gesamte Knotenliste nach dem
	     *  zweiten zu dieser Kante gehoerenden Knoten durchsucht
	     */

	     arcs = vertexObject->out;
	     while( arcs != NULL ) {
		 for( compareListElement = \
			  copyObject->copyVertexList->head->next;
		      compareListElement != \
			  copyObject->copyVertexList->head;
		      compareListElement  = compareListElement->next ) {
		     comparevertexObject = \
			 (Vertex_type *)(compareListElement->data);
		     if( arcs->other_vertex->v_id == \
			 comparevertexObject->v_id) {
			 sprintf(interp->result,\
				 "arc%d",arcs->edge_object->e_id);
			 edgeObject =  getEdgeStruct( interp->result);
			 if( edgeObject == NULL ) {
			     return TCL_ERROR;
			 }
			 
			 /*
			  *  Die Struktur der zum jeweiligen Knoten 
			  *  gehoerenden Kante wird komplett kopiert ...
			  */

			 copyEdgeObject = createEdgeObject( edgeObject );
		    
			 /* 
			  *  ... und in die Kanten-Liste des Copy-Objektes
			  *   uebertragen
			  */

			 copyEdgeObject = \
			     listFirst( copyObject->copyEdgeList,\
					copyEdgeObject );
			 break;
		     }
		 }
		 arcs = arcs->next;
	     }
	}
    }

    if( Tcl_ListObjGetElements( interp, objv[1], &listSum,\
		       &listElems ) != TCL_OK ) {
	return TCL_ERROR;
    }

    index = 0;
    while( index < listSum ) {
        if (Tcl_GetIntFromObj(interp, listElems[index++], &cid ) != TCL_OK)
          return TCL_ERROR;

	/* 
	 *  Die Knoten duerfen erst geloescht werden, wenn die Bearbeitung
	 *  der Kanten vollstaendig abgeschlossen ist.
	 */

	if( vertexDelete( interp, cid, undeleteElement ) != TCL_OK ) {
	    return TCL_ERROR;
	}
    }

    /*
     *   Die als zweites Argument uebergebene Tcl-Liste der zu loeschenden 
     *   Kommentare wird in ein Array mit Zeigern auf die einzelnen 
     *   Listenelemente umgewandelt
     */

    if( Tcl_ListObjGetElements( interp, objv[2], &listSum,\
		       &listElems ) != TCL_OK ) {
	return TCL_ERROR;
    }

    /*
     *  Die kopierten Kommentare werden der Reihe nach in die Kommentar-
     *  Liste des Copy-Objektes uebertragen
     */

    index = 0;
    while( index < listSum ) {
        if (Tcl_GetIntFromObj(interp, listElems[index++], &cid ) != TCL_OK)
          return TCL_ERROR;
	sprintf(interp->result, "comment%d",cid);
	commentObject =  getCommentStruct( interp->result);
	if( commentObject == NULL ) {
	    return TCL_ERROR;
	}

	/*
	 *  Die Struktur des kopierten Kommentares muss uebertragen 
	 *  werden ...
	 */

	copyCommentObject = createCommentObject( commentObject );

	/* 
	 *  ... und wird dann in die Kommentar-Liste des Copy-Objektes
	 *  eingefuegt.
	 */

	copyCommentObject = listFirst( copyObject->copyCommentList,\
					copyCommentObject );

	/* 
	 *  Der alte Kommentar wird zum Schluss geloescht
	 */

	if( commentDelete( interp, cid, undeleteElement ) != TCL_OK ) {
	    return TCL_ERROR;
	}
    }
    //free( (char *) listElems );
    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Befehlsfunktion zum Men"upunkt:                                     #
 #                                                                        #
 #  Copy                                                                  #
 #                                                                        #
 ##########################################################################
 */


/*  tk_Copy
 *--------------------------------------------------------------
 *   
 *  kopiert die Elemente des markierten Bereiches in einen
 *  Zwischenspeicher.
 *
 *--------------------------------------------------------------
 */

int tk_Copy(ClientData cd, Tcl_Interp * interp,\
		  int objc, Tcl_Obj *CONST objv[])
{
    Tcl_Obj      **listElems;
    int            cid, listSum, index;
    double	   x;
    Comment_type  *commentObject;
    Comment_type  *copyCommentObject;
    EdgeList_type *arcs;
    Edge_type     *edgeObject;
    Edge_type     *copyEdgeObject;
    List_element  *compareListElement;
    List_element  *list_element;
    Vertex_type   *vertexObject;
    Vertex_type   *copyvertexObject;
    Vertex_type   *comparevertexObject = NULL;

    if (objc != 5) {
	Tcl_WrongNumArgs(interp, 1, objv, "nodelist commentlist x y");
        return TCL_ERROR;
    }
    
    /* 
     *  Die alten kopierten Elemente muessen vollstaendig entfernt werden
     */

    if( copyObject != NULL ) {
	listFree( copyObject->copyVertexList );
	listFree( copyObject->copyEdgeList );
	listFree( copyObject->copyCommentList );
	free( copyObject );
    }

    /* 
     *  Ein neues Objekt fuer die kopierten Elemente wird angelegt
     */

    copyObject = MALLOC( Copy_type );
    if (Tcl_GetDoubleFromObj(interp, objv[3], &x) != TCL_OK)
      return TCL_ERROR;
    copyObject->area_start.x = x;
    if (Tcl_GetDoubleFromObj(interp, objv[4], &x) != TCL_OK)
      return TCL_ERROR;
    copyObject->area_start.y = x;
    copyObject->copyVertexList =createList();
    copyObject->copyEdgeList = createList();
    copyObject->copyCommentList = createList();

    /*
     *   Die als erstes Argument uebergebende Tcl-Liste der zu loeschenden
     *   Knoten wird in ein Array mit Zeigern auf die einzelnen 
     *   Listenelemente umgewandelt.
     */
    if( Tcl_ListObjGetElements( interp, objv[1], &listSum,\
		       &listElems ) != TCL_OK ) {
	return TCL_ERROR;
    }

    /*
     *  Die kopierten Knoten werden der Reihe nach in die Knoten-Liste
     *  des Copy-Objektes uebertragen
     */

    index = 0;
    while( index < listSum ) {
        if (Tcl_GetIntFromObj(interp, listElems[index++], &cid ) != TCL_OK)
          return TCL_ERROR;
	sprintf(interp->result, "knoten%d",cid);
	vertexObject =  getVertexStruct( interp->result);
	if( vertexObject == NULL ) {
	    return TCL_ERROR;
	}

	/*
	 *  Die Struktur des kopierten Knoten muss uebertragen werden
	 */

	copyvertexObject = createVertexObject( vertexObject );

	/* 
	 *  Der kopierte Knoten wird in die Knoten-Liste des Copy-Objektes
	 *  eingefuegt
	 */

	copyvertexObject = listFirst( copyObject->copyVertexList,\
					    copyvertexObject );
    }
    //free( (char *) listElems );

    /* 
     *  Wenn die Knoten-Liste des Copy-Objektes nicht leer ist muessen
     *  die evtl. kopierten Kanten behandelt werden
     */

    if( copyObject->copyVertexList->head->next !=\
	copyObject->copyVertexList->head ) {
	for( list_element  = copyObject->copyVertexList->head->next;
	     list_element != copyObject->copyVertexList->head;
	     list_element  = list_element->next ) {
	    vertexObject = (Vertex_type *) (list_element->data);

	    /* 
	     *  Zuerst werden die Schlingen behandelt. Da sie jeweils nur
	     *  zu einem Knoten gehoeren, koennen sie komplett uebernommen
	     *  werden. 
	     */

	    arcs = vertexObject->own_edge;
	    while( arcs != NULL ) {
		sprintf(interp->result, "arc%d",arcs->edge_object->e_id);
		edgeObject =  getEdgeStruct( interp->result);
		if( edgeObject == NULL ) {
		    return TCL_ERROR;
		}

		/*
		 *  Die Strukturen der zum jeweiligen Knoten gehoerenden
		 *  Schlingen werden komplett kopiert ...
		 */

		copyEdgeObject = createEdgeObject( edgeObject );

		/* 
		 *  ... und in die Kanten-Liste des Copy-Objektes ueber-
		 *  tragen.
		 */

		copyEdgeObject = listFirst( copyObject->copyEdgeList,\
					    copyEdgeObject );
		arcs = arcs->next;
	    }

	    /* 
	     *  Dann werden die normalen Kanten des Knotens behandelt.
	     *  Diese werden nur aufgenommen, wenn der zweite zur Kante 
	     *  gehoerende Knoten ebenfalls in der kopierten Area liegt.
	     *  Speziell wird fuer jede ausgehende Kante jedes in der
	     *  Area befindlichen Knotens die gesamte Knotenliste nach dem
	     *  zweiten zu dieser Kante gehoerenden Knoten durchsucht
	     */

	    arcs = vertexObject->out;
	    while( arcs != NULL ) {
		for( compareListElement  = \
			 copyObject->copyVertexList->head->next;
		     compareListElement != \
			 copyObject->copyVertexList->head;
		     compareListElement  = compareListElement->next ) {
		    comparevertexObject = \
			(Vertex_type *)(compareListElement->data);
		    if( arcs->other_vertex->v_id == \
			comparevertexObject->v_id) {
			sprintf(interp->result,"arc%d",\
				arcs->edge_object->e_id);
			edgeObject = \
			    getEdgeStruct( interp->result);
			if( edgeObject == NULL ) {
			    return TCL_ERROR;
			}

			/*
			 *  Die Struktur der zum jeweiligen Knoten 
			 *  gehoerenden Kante wird komplett kopiert ...
			 */

			copyEdgeObject = createEdgeObject( edgeObject );

			/* 
			 *  ... und in die Kanten-Liste des Copy-Objektes
			 *   uebertragen
			 */
			copyEdgeObject = \
			    listFirst( copyObject->copyEdgeList,\
				       copyEdgeObject );
			break;
		    }
		}
		arcs = arcs->next;
	    }
	}
    }

    /*
     *   Die als zweites Argument uebergebende Tcl-Liste der zu 
     *   loeschenden Kommentare wird in ein Array mit Zeigern auf die 
     *   einzelnen Listenelemente umgewandelt.
     */

    if( Tcl_ListObjGetElements( interp, objv[2], &listSum,\
		       &listElems ) != TCL_OK ) {
	return TCL_ERROR;
    }

    /*
     *  Die kopierten Kommentare werden der Reihe nach in die Kommentar-
     *  Liste des Copy-Objektes uebertragen.
     */

    index = 0;
    while( index < listSum ) {
        if (Tcl_GetIntFromObj(interp, listElems[index++], &cid ) != TCL_OK)
          return TCL_ERROR;
	sprintf(interp->result, "comment%d",cid);
	commentObject =  getCommentStruct( interp->result);
	if( commentObject == NULL ) {
	    return TCL_ERROR;
	}

	/*
	 *  Die Struktur des kopierten Kommentare muss uebertragen werden
	 */

	copyCommentObject = createCommentObject( commentObject );

	/* 
	 *  Der kopierte Kommentar wird in die Kommentar-Liste des Copy-
	 *  Objektes eingefuegt.
	 */

	copyCommentObject = listFirst( copyObject->copyCommentList,\
					copyCommentObject );
    }
    //free( (char *) listElems );

    return TCL_OK;
}



/*
 ##########################################################################
 #                                                                        #
 #  C-Befehlsfunktion zum Men"upunkt:                                     #
 #                                                                        #
 #  Paste                                                                 #
 #                                                                        #
 ##########################################################################
 */


/*  tk_Paste
 *--------------------------------------------------------------
 *   
 *  erstellt die Elemente aus dem Zwischenspeicher an einer
 *  angegebenen Position auf der Zeichenfl"ache.
 *
 *--------------------------------------------------------------
 */
int tk_Paste(ClientData cd, Tcl_Interp * interp,\
		  int objc, Tcl_Obj *CONST objv[])
{
    int            xd, yd;
    int            place_cid;
    int            label_cid;
    int            arc_cid;
    int            comment_cid;
    int            cid;
    double 	   x;
    Comment_type  *newCommentObject;
    Comment_type  *copyCommentObject;
    Edge_type     *edgeObject;
    Edge_type     *copyEdgeObject;
    List_element  *list_element;
    Point_type     new_start;
    Point_type     new_end;
    UnsetTop_type *partList_object;
    Vertex_type   *newvertexObject;
    Vertex_type   *vertexObject;
    Vertex_type   *copyvertexObject;

    if (objc != 3) {
	Tcl_WrongNumArgs(interp, 1, objv, "x y");
        return TCL_ERROR;
     }

    /* 
     *  Wurde schon etwas kopiert?
     */

    if( copyObject != NULL ) {

	/* 
	 *  Ein Listenelement der Unset-Liste wird erstellt
	 *  und am Anfang der Liste eingefuegt
	 */

	partList_object = makeUnsetElement( remove_element );
	if( partList_object == NULL ) {
	    return TCL_ERROR;
	}

	/* 
	 *  Im id2cidTable werden die alten und neuen Canvas-Id's
	 *  der Knoten eingetragen, um den Kanten spaeter die neuen
	 *  Id's zuweisen zu koennen
	 */

	Tcl_InitHashTable (&id2cidTable, TCL_STRING_KEYS);

	/* 
	 *  Die x,y-Differenzen werden ermittelt, um die Objekte spaeter 
	 *  an die richtige Stelle setzen zu koennen
	 */

        if (Tcl_GetDoubleFromObj(interp, objv[1], &x) != TCL_OK)
	  return TCL_ERROR;
	xd = x - copyObject->area_start.x;
        if (Tcl_GetDoubleFromObj(interp, objv[2], &x) != TCL_OK)
	  return TCL_ERROR;
	yd = x - copyObject->area_start.y;

	/* 
	 *  Die kopierten Knoten werden eingesetzt  
	 */

	for( list_element = copyObject->copyVertexList->head->next;
	     list_element != copyObject->copyVertexList->head;
	     list_element = list_element->next ) {
	    copyvertexObject = (Vertex_type *)(list_element->data);

	    /*
	     *  Die Strukturen der kopierten Knoten muessen uebertragen
	     *  werden, um ein mehrmaliges Einsetzen zu gewaehrleisten 
	     */

	    newvertexObject = createVertexObject( copyvertexObject );
	    newvertexObject->center.x += xd;
	    newvertexObject->center.y += yd;

	    /* 
	     *  Die alten Kantenliste muessen geloescht werden 
	     */

	    newvertexObject->own_edge = NULL;
	    newvertexObject->in = NULL;
	    newvertexObject->out = NULL;
	    
	    /* 
	     *  Der neue Knoten auf der Canvas gezeichnet und die neuen  
	     *  Canvas-Id's des Knotens und des Textes zurueckgegeben
	     */

	    if( printVertex( interp, newvertexObject,\
			     &place_cid, &label_cid ) != TCL_OK ) {
		return TCL_ERROR;
	    }

	    /* 
	     *  Die alte und neue Canvas-Id wird im id2cid-Table einge-
	     *  tragen.
	     */

	    if( add_cid (newvertexObject->v_id,\
			 place_cid, interp) != TCL_OK ) {
		return TCL_ERROR;
	    }
	    newvertexObject->v_id = place_cid;
	    newvertexObject->label_id = label_cid;

	    /* 
	     *  Der Knoten wird in die Liste des zuvor initialisierten 
	     *  Elementes der Unset-Liste eingetragen 
	     */

	    insertUnsetStruct(partList_object, place,\
			      newvertexObject->v_id, -1, -1 );

	    /* 
	     *  Die Knotenstruktur wird in den Knoten-Hash-Table einge-
	     *  tragen.
	     */

	    if( hashVertexInsert (interp, newvertexObject ) != TCL_OK ) {
		return TCL_ERROR;	    
	    }
	}

	/* 
	 *  Die kopierten Kanten werden eingesetzt  
	 */

	for( list_element = copyObject->copyEdgeList->head->next;
	     list_element != copyObject->copyEdgeList->head;
	     list_element = list_element->next ) {
	    copyEdgeObject = (Edge_type *)(list_element->data);

	    /*
	     *  Die Strukturen der kopierten Kanten muessen uebertragen
	     *  werden, um ein mehrmaliges Kopieren zu gewaehrleisten 
	     */

	    edgeObject = createEdgeObject( copyEdgeObject );

	    /* 
	     *  Die Id's der neuen Anfangs- und Endknoten der Kante
	     *  werden aus dem id2cid-Table geholt
	     */

	    sprintf (interp->result, "id%d", copyEdgeObject->start);
	    cid = get_cid (interp->result);
	    if( cid != -1 ) {
		edgeObject->start = cid;		
	    } else { 
		return TCL_ERROR;
	    }
	    sprintf (interp->result, "id%d", copyEdgeObject->end);
	    cid = get_cid (interp->result);
	    if( cid != -1 ) {
		edgeObject->end = cid;		
	    } else {
		return TCL_ERROR;
	    }

	    /* 
	     *  Wenn es sich nicht um eine Schlinge handelt ...
	     */

	    if (edgeObject->start != edgeObject->end) {

		/* 
		 *  ... wird die Kante auf der Canvas gezeichnet und die 
		 *  neuen Canvas-Id's der Kante und des Textes zurueck-
		 *  gegeben.
		 */

		if( printEdge( interp, edgeObject,\
			       &arc_cid, &label_cid ) != TCL_OK ) {
		    return TCL_ERROR;
		}

		edgeObject->e_id = arc_cid;
		edgeObject->label_id = label_cid;

		/* 
		 *  Die Strukturen des ersten und zweiten Knotens der Kante
		 *  werden geholt ...
		 */

		if( getVertexCoords (interp, edgeObject->start,\
				     &new_start) != TCL_OK ) {
		    return TCL_ERROR;
		}
		if( getVertexCoords (interp, edgeObject->end,\
				     &new_end) != TCL_OK ) {
		    return TCL_ERROR;
		}
				
		/* 
		 *  ... und die Kante wird an die richtige Position gesetzt
		 */

		if( changeEdgePos (interp, edgeObject, new_start, \
				   new_end, 1) != TCL_OK ) {
		    return TCL_ERROR;
		}
	    } else { /* Wenn es sich um eine Schlinge handelt ... */
		sprintf(interp->result, "knoten%d",edgeObject->start);
		vertexObject = getVertexStruct( interp->result);
		if( vertexObject == NULL ) {
		    return TCL_ERROR;
		}

		/* 
		 *  ... wird diese auf der Canvas gezeichnet und die 
		 *  neuen Canvas-Id's der Kante und des Textes zurueck-
		 *  gegeben ...
		 */

		if( printOwnEdge( interp, edgeObject,\
				  &arc_cid, &label_cid ) != TCL_OK ) {
 		    return TCL_ERROR;
		}

		edgeObject->e_id = arc_cid;
		edgeObject->label_id = label_cid;

		/*
		 *  ... zum Schluss wird die Schlinge noch an der 
		 *  richtigen Position plaziert.
		 */

		edgeObject->center.x += xd;
		edgeObject->center.y += yd;
	  
		if( changeOwnEdgePos( interp,edgeObject,\
				      vertexObject->center) != TCL_OK ) {
		    return TCL_ERROR;
		}
	    }

	    /* 
	     *  Die Kante wird in die Kanten-Listen der zugehoerigen 
	     *  Knoten ...
	     */

	    if( EdgeToVertexList (interp, edgeObject) != TCL_OK ) {
		return TCL_ERROR;
	    }

	    /* 
	     *  ... in die Liste des zuvor initialisierten 
	     *  Elementes der Unset-Liste ...
	     */

	    insertUnsetStruct(partList_object, edge,
			      edgeObject->e_id, -1, -1 );

	    /* 
	     *  ... und in den Kanten-Hash-Table eingetragen
	     */

	    if( hashEdgeInsert (interp, edgeObject) != TCL_OK ) {
		return TCL_ERROR;
	    }
	}

	/*
	 *  Alle neuen Knoten-Cid's sind den Kanten bekannt. Der id2cid-
	 *  Table ist also nicht mehr noetig
	 */

	Tcl_DeleteHashTable (&id2cidTable);
	
	/* 
	 *  Die kopierten Kommentare werden eingesetzt  
	 */

	for( list_element = copyObject->copyCommentList->head->next;
	     list_element != copyObject->copyCommentList->head;
	     list_element = list_element->next ) {
	    copyCommentObject = (Comment_type *)(list_element->data);

	    /*
	     *  Die Strukturen der kopierten Kommentare muessen uebertragen
	     *  werden, um ein mehrmaliges Kopieren zu gewaehrleisten 
	     */

	    newCommentObject = createCommentObject( copyCommentObject );
	    newCommentObject->center.x += xd;
	    newCommentObject->center.y += yd;

	    /* 
	     *  Der Kommentar wird auf der Canvas gezeichnet und die neue  
	     *  Canvas-Id des Kommentars zurueckgegeben
	     */

	    if( printComment( interp, newCommentObject,\
			      &comment_cid) != TCL_OK ) {
		return TCL_ERROR;
	    }

	    newCommentObject->comment_id = comment_cid;

	    /* 
	     *  Der Kommentar wird in die Liste des zuvor initialisierten 
	     *  Elementes der Unset-Liste eingetragen 
	     */

	    insertUnsetStruct(partList_object, comment,
			      newCommentObject->comment_id, -1, -1 );

	    /* 
	     *  Die Struktur wird in den Kommentar-Hash-Table eingetragen
	     */

	    if( hashCommentInsert (interp, newCommentObject) != TCL_OK ) {
		return TCL_ERROR;
	    }
	}
    } 
    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Befehlsfunktion zum Men"upunkt:                                     #
 #                                                                        #
 #  Unset                                                                 #
 #                                                                        #
 ##########################################################################
 */

/*  tk_Unset
 *--------------------------------------------------------------
 *   
 *  macht einen Set-Vorgang (Setzen oder Verschieben von 
 *  Knoten, Kanten, Kommentaren oder markierten Bereichen)
 *  r"uckg"angig.
 *
 *--------------------------------------------------------------
 */

int tk_Unset(ClientData cd, Tcl_Interp * interp,\
	     int objc, Tcl_Obj *CONST objv[])
{
    Comment_type  *commentObject;
    List_element  *list_element;
    Undelete_type *undeleteElement;
    UnsetTop_type *partList_object;
    Unset_type    *set_element;
    Vertex_type   *vertexObject;

    if (objc != 1) {
	Tcl_WrongNumArgs(interp, 1, objv, NULL);
        return TCL_ERROR;
    }

    /*
     *  Falls die Unset-Liste nicht leer ist ...
     */

    if( actualEd->unsetList->head->next != actualEd->unsetList->head ) {

	/* 
	 *  ... wird das erste Unset-Element aus der Liste entfernt.
	 */

	partList_object = listRemoveFirst(actualEd->unsetList);
	(actualEd->unsetCount) -= 1;

	/* 
	 *  Es gibt zwei Moeglichkeiten:
	 *    1. Ein Verschiebe-Vorgang wird rueckgaengig gemacht
	 *    2. Ein gesetztes Element wird wieder geloescht
	 */

	if( partList_object->action == move_element ) {
	    
	    /* 
	     *  Ein verschobenes Element wird an die alte Position
	     *  zur"uckgesetzt:
	     */
	
	    for( list_element = partList_object->elementList->head->next;
		 list_element != partList_object->elementList->head;
		 list_element = list_element->next ) {
		set_element = (Unset_type *) (list_element->data);

		switch( set_element->element ) {
		    
		    /* 
		     *  Es wird jeweils die alte gemerkte Position der
		     *  Objekte wiederhergestellt
		     */

		case place:
		    sprintf(interp->result, "knoten%d",set_element->id);
		    vertexObject = getVertexStruct( interp->result);
		    if( vertexObject == NULL ) {
			return TCL_ERROR;
		    }

		    if( changeVertexPos( interp, vertexObject,\
				    set_element->last_point) != TCL_OK ) {
			return TCL_ERROR;
		    }
		    break;

		case edge:
		    if( edgeMove(interp, set_element->id,\
				 set_element->last_point) != TCL_OK ) {
			return TCL_ERROR;
		    }
		    if (calculateAngles(interp,\
					set_element->id) != TCL_OK ) {
			return TCL_ERROR;
		    }
		    break;

		case comment:
		    sprintf(interp->result, "comment%d",set_element->id);
		    commentObject = getCommentStruct( interp->result);
		    if( commentObject == NULL ) {
			return TCL_ERROR;
		    }
		    if( changeCommentPos( interp, commentObject,\
				     set_element->last_point) != TCL_OK ) {
			return TCL_ERROR;
		    }
		    break;

		case edge_text:
		    if( changeTextPos(interp, set_element->id,\
				     set_element->last_point) != TCL_OK ) {
			return TCL_ERROR;
		    }
		    break;
		    
		default:
		    break;
		}
	    }
	} else if( partList_object->action == remove_element ) {

	    /* 
	     *  Ein erstelltes Element wird gel"oscht:
	     */

	    undeleteElement = makeUndeleteElement();
	    if( undeleteElement == NULL ) {
		return TCL_ERROR;
	    }

	    for( list_element = partList_object->elementList->head->next;
		 list_element != partList_object->elementList->head;
		 list_element = list_element->next ) {
		set_element = (Unset_type *) (list_element->data);

		switch(set_element->element) {

		case place:
		    if( vertexDelete( interp, set_element->id,\
				      undeleteElement) != TCL_OK ) {
			return TCL_ERROR;
		    }
		    break;

		case edge:
		    if( edgeDelete( interp, set_element->id,\
				    undeleteElement ) != TCL_OK ) {
			return TCL_ERROR;
		    }
		    break;

		case comment:
		    if( commentDelete(interp, set_element->id,\
				      undeleteElement) != TCL_OK ) {
			return TCL_ERROR;
		    }
		    break;

		default:
		    break;
		}
	    }
	} else {
	    return TCL_ERROR;
	}
    }
    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Befehlsfunktion zum Men"upunkt:                                     #
 #                                                                        #
 #  Undelete                                                              #
 #                                                                        #
 ##########################################################################
 */

/*  tk_Undelete
 *--------------------------------------------------------------
 *   
 *  macht einen Delete-Vorgang (l"oschen von 
 *  Knoten, Kanten, Kommentaren oder markierten Bereichen)
 *  r"uckg"angig.
 *
 *--------------------------------------------------------------
 */
int tk_Undelete( ClientData cd, Tcl_Interp * interp,\
		 int objc, Tcl_Obj *CONST objv[])
{
    int            place_cid;
    int            label_cid;
    int            arc_cid;
    int            comment_cid;
    Comment_type  *commentObject;
    Edge_type     *edgeObject;
    List_element  *list_element;
    Point_type     new_start;
    Point_type     new_end;
    Undelete_type *delete_object;
    UnsetTop_type *partList_object;
    Vertex_type   *vertexObject;


    if (objc != 1) {
	Tcl_WrongNumArgs(interp, 1, objv, NULL);
        return TCL_ERROR;
    }

    /*
     *  Falls die Undelete-Liste nicht leer ist ...
     */

    if( actualEd->undeleteList->head->next != \
	actualEd->undeleteList->head ) {

	/*
	 *  ... wird das erste Element der Liste geholt.
	 */

	delete_object = \
	    (Undelete_type *)(actualEd->undeleteList->head->next->data);

	/* 
	 *  Ein Listenelement der Unset-Liste wird erstellt
	 *  und am Anfang der Liste eingefuegt
	 */

	partList_object = makeUnsetElement( remove_element );
	if( partList_object == NULL ) {
	    return TCL_ERROR;
	}

	/* 
	 *  Die zugehoerigen Knoten werden wiederhergestellt  
	 */

	for( list_element  = delete_object->killedVertexList->head->next;
	     list_element != delete_object->killedVertexList->head;
	     list_element  = list_element->next ) {

	    (vertexObject) = (Vertex_type *)(list_element->data);
	    vertexObject->in = NULL;
	    vertexObject->out = NULL;

	    /* 
	     *  Der Knoten wird auf der Canvas gezeichnet und die neuen  
	     *  Canvas-Id's des Knotens und des Textes werden zurueckge-
	     *  geben.
	     */

	    if( printVertex( interp, vertexObject,\
			     &place_cid, &label_cid ) != TCL_OK) {
		return TCL_ERROR;
	    }

	    /* 
	     *  In der gesamten Undelete-Liste muss die neue Canvas-Id des
	     *  Knotens den zugehoerigen Kanten mitgeteilt werden!
	     */

	    updateUndeleteList( vertexObject->v_id, place_cid );
	    vertexObject->v_id = place_cid;
	    vertexObject->label_id = label_cid;

	    /* 
	     *  Der Knoten wird in die Liste des zuvor initialisierten 
	     *  Elementes der Unset-Liste eingetragen 
	     */

	    insertUnsetStruct(partList_object, place,
			      vertexObject->v_id, -1, -1 );

	    /* 
	     *  Die Knotenstruktur wird in den Knoten-Hash-Table einge-
	     *  tragen.
	     */

	    if( hashVertexInsert (interp, vertexObject) != TCL_OK ) {
		return TCL_ERROR;	     
	    }
	}

	/* 
	 *  Die zugehoerigen Kanten werden wiederhergestellt  
	 */

	for( list_element = delete_object->killedEdgeList->head->next;
	     list_element != delete_object->killedEdgeList->head;
	     list_element = list_element->next ) {
	    edgeObject = (Edge_type *)(list_element->data);
	    
	    /* 
	     *  Wenn es sich nicht um eine Schlinge handelt ...
	     */

	    if (edgeObject->start != edgeObject->end) {

		/* 
		 *  ... wird die Kante auf der Canvas gezeichnet und die
		 *  neuen Canvas-Id's der Kante und des Textes zurueck-
		 *  gegeben
		 */

		if( printEdge( interp, edgeObject,\
			       &arc_cid, &label_cid ) != TCL_OK ) {
		    return TCL_ERROR;
		}

		edgeObject->e_id = arc_cid;
		edgeObject->label_id = label_cid;

		/* 
		 *  Die Strukturen des ersten und zweiten Knotens der Kante
		 *  werden geholt ...
		 */

		if( getVertexCoords (interp, edgeObject->start,\
				     &new_start) != TCL_OK ) {
		    return TCL_ERROR;
		}
		if( getVertexCoords (interp, edgeObject->end,\
				     &new_end) != TCL_OK ) {
		    return TCL_ERROR;
		}

		/* 
		 *  ... und die Kante wird an die richtige Position gesetzt
		 */

		if( changeEdgePos (interp, edgeObject, new_start, \
				   new_end, 1) != TCL_OK) {
		    return TCL_ERROR;
		}
	    } else { /* Wenn es sich um eine Schlinge handelt ... */
		sprintf(interp->result, "knoten%d",edgeObject->start);
		vertexObject = getVertexStruct( interp->result);
		if( vertexObject == NULL ) {
		    return TCL_ERROR;
		}

		/* 
		 *  ... wird diese auf der Canvas gezeichnet und die 
		 *  neuen Canvas-Id's der Kante und des Textes zurueck-
		 *  gegeben ...
		 */

		if( printOwnEdge( interp, edgeObject,\
				  &arc_cid, &label_cid ) != TCL_OK ) {
		    return TCL_ERROR;
		}

		edgeObject->e_id = arc_cid;
		edgeObject->label_id = label_cid;

		/* 
		 *  ... dann wird der Mittelpunkt der Schlinge 
		 *  aktualisiert ...
		 */

		edgeObject->center.x += vertexObject->center.x;
		edgeObject->center.y += vertexObject->center.y;

		/*
		 *  ... und zum Schluss wird die Schlinge noch an 
		 *  der richtigen Position plaziert.
		 */

		if( changeOwnEdgePos( interp, edgeObject,\
				      vertexObject->center) != TCL_OK ) {
		    return TCL_ERROR;
		}
	    }

	    /* 
	     *  Die Kante wird in die Kanten-Listen der zugehoerigen 
	     *  Knoten ...
	     */

	    if( EdgeToVertexList (interp, edgeObject) != TCL_OK ) {
		return TCL_ERROR;
	    }

	    /* 
	     *  ... in die Liste des zuvor initialisierten 
	     *  Elementes der Unset-Liste ...
	     */

	    insertUnsetStruct(partList_object, edge,
			      edgeObject->e_id, -1, -1 );
	    /* 
	     *  ... und in den Kanten-Hash-Table eingetragen
	     */

	    if( hashEdgeInsert (interp, edgeObject) != TCL_OK ) {
		return TCL_ERROR;
	    }
	}

	/* 
	 *  Die zugehoerigen Kommentare werden wiederhergestellt  
	 */

	for( list_element = delete_object->killedCommentList->head->next;
	     list_element != delete_object->killedCommentList->head;
	     list_element = list_element->next ) {
	    commentObject = (Comment_type *)(list_element->data);
	    
	    /* 
	     *  Der Kommentar wird auf der Canvas gezeichnet und die neue  
	     *  Canvas-Id des Kommentars zurueckgegeben
	     */

	    if( printComment( interp, commentObject,\
			      &comment_cid) != TCL_OK ) {
		return TCL_ERROR;
	    }

	    commentObject->comment_id = comment_cid;

	    /* 
	     *  Die Struktur wird in den Kommentar-Hash-Table eingetragen
	     */

	    if( hashCommentInsert (interp, commentObject) != TCL_OK ) {
		return TCL_ERROR;
	    }

	    /* 
	     *  Der Kommentar wird in die Liste des zuvor initialisierten 
	     *  Elementes der Unset-Liste eingetragen 
	     */

	    insertUnsetStruct(partList_object, comment,
			      commentObject->comment_id, -1, -1 );
	}

	/*
	 *  Das Undelete-Element wird aus der Undelete-Liste geloescht
	 *  und der zugehoerige Zaehler um eins vermindert
	 */

	(actualEd->undeleteCount) -= 1;
	listRemoveFirst(actualEd->undeleteList);

    }
    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  Hilfsfunktionen                                                       #
 #                                                                        #
 ##########################################################################
*/

/*  createVertexObject
 *--------------------------------------------------------------
 *   
 *  erstellt eine neue Knotenstruktur und "ubergibt ihr
 *  die Daten der Struktur vertexObject.
 *
 *--------------------------------------------------------------
 */

Vertex_type *createVertexObject( Vertex_type *vertexObject )
{
    Vertex_type *newvertexObject;

    newvertexObject = MALLOC(Vertex_type);
    newvertexObject->v_id = vertexObject->v_id;
    newvertexObject->center.x = vertexObject->center.x;
    newvertexObject->center.y = vertexObject->center.y;
    newvertexObject->label_id = vertexObject->label_id;
    newvertexObject->type = vertexObject->type;
    newvertexObject->label = strdup( vertexObject->label );
    newvertexObject->own_edge = vertexObject->own_edge;
    newvertexObject->in = vertexObject->in;
    newvertexObject->out = vertexObject->out; 
    newvertexObject->save_id = -1;

    return( newvertexObject );
}


/*  createEdgeObject
 *--------------------------------------------------------------
 *   
 *  erstellt eine neue Kantenstruktur und "ubergibt ihr
 *  die Daten der Struktur edgeObject.
 *
 *--------------------------------------------------------------
 */

Edge_type *createEdgeObject( Edge_type *edgeObject )
{
    Edge_type *newEdgeObject;

    newEdgeObject = MALLOC( Edge_type );
    newEdgeObject->start = edgeObject->start;
    newEdgeObject->end = edgeObject->end;
    newEdgeObject->label_center = edgeObject->label_center;
    newEdgeObject->label = strdup(edgeObject->label);
    newEdgeObject->move_infos = edgeObject->move_infos;
    newEdgeObject->center = edgeObject->center;
    newEdgeObject->save_id = -1;

    return( newEdgeObject );
}


/*  createCommentObject
 *--------------------------------------------------------------
 *   
 *  erstellt eine neue Kommentarstruktur und "ubergibt ihr
 *  die Daten der Struktur commentObject.
 *
 *--------------------------------------------------------------
 */

Comment_type *createCommentObject( Comment_type *commentObject )
{
    Comment_type *newCommentObject;
    Font_type    *font_object;

    newCommentObject = MALLOC(Comment_type);
    newCommentObject->comment_id = commentObject->comment_id;
    newCommentObject->center.x = commentObject->center.x;
    newCommentObject->center.y = commentObject->center.y;
    newCommentObject->comment = strdup(commentObject->comment);

    font_object = MALLOC(Font_type);
    font_object->family = strdup( commentObject->font->family );
    font_object->weight = strdup( commentObject->font->weight );
    font_object->slant =  commentObject->font->slant;
    font_object->size =  commentObject->font->size;

    newCommentObject->font = font_object; 

    return( newCommentObject );
}
