###########################################################################
###########################################################################
##                                                           		 ##
##  Datei   :     `pfaEdit.tcl'                                          ##
##                -------------                                          ##
##                                                                       ##
##  Inhalt  :     Prozeduren, die die Funktionalit"at        		 ##
##                des `Edit'-Men"us gew"ahrleisten.                      ##
##                                                           	       	 ##
###########################################################################
###########################################################################



###########################################################################
##									 ##
##  Prozedur zum Men"upunkt:						 ##
##									 ##
##  Cut									 ##
##									 ##
###########################################################################


##  Cut
##-----------------------------------------------------------
##  Es wird eine C-Funktion aufgerufen, die alle Elemente
##  im markierten Bereich in einen Zwischenspeicher packt
##  und dann vom Bildschirm entfernt. Die Cut- und Copy-
##  Funktionen lassen sich daraufhin erst nach erneutem
##  Markieren eines Bereichs aufrufen
##-----------------------------------------------------------
##
proc Cut {} {
    global editor box

    pfa_Cut  [$editor(canvas) find withtag nodesInArea]\
	[$editor(canvas) find withtag commentsInArea]\
	[lindex $box 0] [lindex $box 1]
    $editor(canvas) delete area
    .fr1a.fr2m.mb2.m\
	entryconfigure "Cut"\
	-state disabled
    bind $editor(canvas) <Control-x> ""
    .fr1a.fr2m.mb2.m\
	entryconfigure "Copy"\
	-state disabled
    bind $editor(canvas) <Control-c> ""
    .fr1a.fr2m.mb2.m\
	entryconfigure "Paste"\
	-state normal

    SomethingHasChanged FA highlight
}


###########################################################################
##									 ##
##  Prozedur zum Men"upunkt:						 ##
##									 ##
##  Copy								 ##
##									 ##
###########################################################################


##  Copy
##-----------------------------------------------------------
##  Es wird eine C-Funktion aufgerufen, die alle Elemente
##  im markierten Bereich in einen Zwischenspeicher packt.
##  Die Cut- und Copy-Funktionen lassen sich daraufhin
##  erst nach erneutem Markieren eines Bereichs aufrufen.
##-----------------------------------------------------------
##
proc Copy {} {
    global editor opt box

    pfa_Copy  [$editor(canvas) find withtag nodesInArea]\
	[$editor(canvas) find withtag commentsInArea]\
	[lindex $box 0] [lindex $box 1]
    $editor(canvas) delete area
    .fr1a.fr2m.mb2.m\
	entryconfigure "Cut"\
	-state disabled
    bind $editor(canvas) <Control-x> ""
    .fr1a.fr2m.mb2.m\
	entryconfigure "Copy"\
	-state disabled
    bind $editor(canvas) <Control-c> ""
    .fr1a.fr2m.mb2.m\
	entryconfigure "Paste"\
	-state normal
}


###########################################################################
##									 ##
##  Prozeduren zum Men"upunkt:						 ##
##									 ##
##  Paste								 ##
##									 ##
###########################################################################


##  PasteInit
##-----------------------------------------------------------
##  Es werden Vorbereitungen getroffen, um den kopierten
##  Bereich an einer gew"unschten Stelle wieder einzu-
##  setzen.
##-----------------------------------------------------------
##
proc PasteInit {} {
    global pos opt histList editor

    if { [llength $histList] == 0 } {
	InitialQuestion
	return
    }
    DeleteBindings
    set opt(mouse3) Paste
    bind $editor(canvas) <3> "XYSet %x %y; Paste"
}


##  Paste
##-----------------------------------------------------------
##  Es wird eine C-Funktion aufgerufen, die alle Elemente
##  des Zwischenspeichers an der Mausposition wieder
##  auf den Bildschirm bringt.
##-----------------------------------------------------------
##
proc Paste {} {
    global pos opt

    pfa_Paste $pos(x) $pos(y)
    DeleteBindings
    set opt(mouse3) Free
    SomethingHasChanged	FA highlight
}


###########################################################################
##									 ##
##  Prozedur zum Men"upunkt:						 ##
##									 ##
##  Unset								 ##
##									 ##
###########################################################################


##  Unset
##-----------------------------------------------------------
##  Es wird eine C-Funktion aufgerufen, um das letzte ge-
##  aenderte Element wieder zur"uckzuversetzen.
##-----------------------------------------------------------
##
proc Unset {} {
    global histList

    if { [llength $histList] == 0 } {
	InitialQuestion
	return
    }
    pfa_Unset
    SomethingHasChanged FA highlight
}


###########################################################################
##									 ##
##  Prozedur zum Men"upunkt:						 ##
##									 ##
##  Undelete								 ##
##									 ##
###########################################################################


##  Undelete
##-----------------------------------------------------------
##  Es wird eine C-Funktion aufgerufen, um das letzte ge-
##  loeschte Element wieder auf den Bildschirm zu bringen.
##-----------------------------------------------------------
##
proc Undelete {} {
    global histList

    if { [llength $histList] == 0 } {
	InitialQuestion
	return
    }
    pfa_Undelete
    SomethingHasChanged FA highlight
}
