/*
###########################################################################
###########################################################################
##                                                                       ##
##  Datei   :    `pfaFile.c'                                             ##
##               -----------                                             ##
##                                                                       ##
##  Inhalt  :    Funktionen, die die neuen Tcl-Befehle des               ##
##               `File'-Men"us implementieren.                           ##
##                                                                       ##
###########################################################################
###########################################################################
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>	
#include <ctype.h>
#include "pfa.h"

/*
 ##########################################################################
 #                                                                        #
 #  Deklaration der lokalen Funktionen                                    #
 #                                                                        #
 ##########################################################################
 */

void          clearUndeleteList(List_type *undeleteList); 
int           getFont( Tcl_Interp *interp, register char *str, 
		       char **rest, char **family, char **weight, 
		       int *slant, int *size );
Editor_type  *deleteEd(Editor_type *first, int number);
void          clearHashTable(Tcl_Interp *interp, Editor_type *delEd);
void          clearList(EdgeList_type *first);
void          clearUnsetList(List_type *unsetList); 


int currEdge=1;
int currNode=1;


/*  lineStart
 *--------------------------------------------------------------
 *   
 *  "uberliest die nicht relevanten Zeichen einer Zeile
 *  des zu ladenden Files. Ein Zeiger auf den relevanten
 *  Text wird zur"uckgegeben.
 *
 *--------------------------------------------------------------
 */

int lineStart( register char *str, char **line, unsigned *line_number )
{
    for(;;) {
	switch( *str ) {
	case '\n':
	    (*line_number)++;
	case ' '  :
	case '\t' :
	    str++;
	    break;
	case '%':		/* Kommentare uebergehen */
	    for(str++; *str  && *str != '\n'; str++);
	    break;
	default:
		/* Zeiger auf den Anfang einer relevanten Zeile */
		*line = str;
		return 0;
	}
    }
}


/*  lineEnd
 *--------------------------------------------------------------
 *   
 *  "uberpr"uft das Ende einer eingelesenen Zeile auf nicht
 *  erlaubte Zeichen. Es d"urfen nur noch '\0', '\n' und 
 *  Kommentare vorkommen.
 *
 *--------------------------------------------------------------
 */

int lineEnd( register char *str, char **rest )
{
    while( (*str == ' ') || (*str == '\t') ) {
	str++;
    }
    /* das Ende der Zeile */
    *rest = str;
    return *str && *str != '\n' && *str != '%';
}


/*  getString
 *--------------------------------------------------------------
 *   
 *  gibt einen eingelesenen String zur"uck.
 *
 *--------------------------------------------------------------
 */

int getString( register char *str, char **rest, char **name,\
	       unsigned *line_number ) 
{
    char     *string_start;
    char     *string;
    unsigned  string_count;

    if( *str != '"' ) {
	return 1;
    }
    str++;
    
    string_start = str;
    for( string_count = 0; *str != '"'; string_count++, str++ )  {
	if( *str == '\0' ) {
	    return 1;
	} else if( *str == '\n' ) {
	    (*line_number)++;
	}
    }

    *name = string = malloc( sizeof( char *) * (string_count+1));
    strncpy( string, string_start, string_count);
    string[string_count] = '\0'; 

    *rest = ++str;
    return 0;
}

/*  getNumber
 *--------------------------------------------------------------
 *  gibt eine eingelesene Nummer zur"uck
 *  TODO: replace by strtol
 *--------------------------------------------------------------
 */

static int getNumber( Tcl_Interp *interp, register char *str, char **line, int *id )
{
    char     *string_start;  
    char     *string;  
    unsigned  string_count;

    string_start = str;
    for( string_count = 0; *str != ' '; string_count++, str++ )  {
	if( !*str ) {
	    return 1;
	}
    }
    string = malloc( sizeof( char ) * (string_count+1));
    strncpy( string, string_start, string_count);
    string[string_count] = '\0'; 

    if (Tcl_GetInt(interp, string, id) != TCL_OK) {
	return TCL_ERROR;
    }

    free(string);
    *line = str;

    return TCL_OK;
}


/*  getXY
 *--------------------------------------------------------------
 *   
 *  gibt ein eingelesenes Koordinatenpaar zur"uck.
 *
 *--------------------------------------------------------------
 */

int getXY(Tcl_Interp *interp, register char *str, char **line,\
	  int *x, int *y )
{
    char     *string_start;    
    char     *string;    
    unsigned  string_count;

    string_start = str;
    for( string_count = 0; *str != '@'; string_count++, str++ )  {
	if( !*str ) {
	    return 1;
	}
    }

    string = malloc( sizeof( char *) * (string_count+1));
    strncpy( string, string_start, string_count);
    string[string_count] = '\0'; 

    if (Tcl_GetInt(interp, string, x) != TCL_OK) {
	return 1;
    }
    free(string);

    /* @ uebergehen */
    str++;
    
    string_start = str;
    for( string_count = 0; *str != ' '; string_count++, str++ )  {
	if( !*str ) {
	   return 1;
	}
    }
    string = malloc( sizeof( char *) * (string_count+1));
    strncpy( string, string_start, string_count);
    string[string_count] = '\0';

    if (Tcl_GetInt(interp, string, y) != TCL_OK) {
	return 1;
    }
    free(string);

    *line = str;
    return 0;
}


/*  getOldStart_End
 *--------------------------------------------------------------
 *   
 *  gibt ein eingelesenes Koordinatenpaar zur"uck.
 *
 *--------------------------------------------------------------
 */

int getOldStart_End(Tcl_Interp *interp,\
		    register char *str, char **rest, int *start, int *end )
{
    char     *string_start;    
    char     *string;    
    unsigned  string_count;

    string_start = str;
    for( string_count = 0; *str != '>'; string_count++, str++ )  {
	if( *str == '\0' ) {
	    return 1;
	}
    }

    string = malloc( sizeof( char *) * (string_count+1));
    strncpy( string, string_start, string_count);
    string[string_count] = '\0'; 
    
    if (Tcl_GetInt(interp, string, start) != TCL_OK) {
	return 1;
    }
    free(string);
    
    /* > uebergehen */
    str++;
    
    string_start = str;
    for( string_count = 0; *str != ' '; string_count++, str++ )  {
	if( *str == '\0' ) {
	   return 1;
	}
    }
    string = malloc( sizeof( char *) * (string_count+1));
    strncpy( string, string_start, string_count);
    string[string_count] = '\0';

    if (Tcl_GetInt(interp, string, end) != TCL_OK) {
	return 1;
    }
    free(string);

    *rest = str;
    return 0;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Funktion zum  Men"upunkt:                                           #
 #                                                                        #
 #  NewFA                                                                 #
 #                                                                        #
 ##########################################################################
 */


/*  tk_NewFA
 *------------------------------------------------------------
 *   
 *  erstellt eine neue Editor-Struktur vom Typ Editor_type
 *  und initialisiert alle Felder. Die Struktur wird dann
 *  am Anfang der Liste eingef"ugt und "uber die 
 *  globale Zeiger-Variable actualEd kann  auf sie
 *  zugegriffen werden.
 *
 *------------------------------------------------------------
 */

int tk_NewFA( ClientData cd, Tcl_Interp * interp,\
		  int objc, Tcl_Obj *CONST objv[])
{
    Editor_type *actual;
    char canvname[30];

    if (objc != 2) {
	Tcl_WrongNumArgs(interp, 1, objv, "num");
        return TCL_ERROR;
    }

    actual = MALLOC( Editor_type );
    actual->next = firstEd;
    firstEd = actual;  
    actualEd = actual;

    if (Tcl_GetIntFromObj( interp, objv[1], &actual->number ) !=
	TCL_OK )
    {
      return TCL_ERROR;
    }
    sprintf(canvname, ".fr1a.fr2ru.fr3FA%d.canv", actual->number);
    actual->canvas = strdup(canvname);
    Tcl_SetStringObj(Tcl_GetObjResult(interp), canvname, -1);
    actual->undeleteCount = 0;
    actual->undeleteList = createList();
    actual->unsetCount = 0;
    actual->unsetList = createList();
    Tcl_InitHashTable( &(actual->vertex_table), TCL_STRING_KEYS);
    Tcl_InitHashTable( &(actual->edge_table), TCL_STRING_KEYS);
    Tcl_InitHashTable( &(actual->comment_table), TCL_STRING_KEYS);

    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Funktionen zum Men"upunkt:                                          #
 #                                                                        #
 #  LoadFA und ReloadFA                                                   #
 #                                                                        #
 ##########################################################################
 */


/*  tk_LoadFA
 *--------------------------------------------------------------
 *   
 *  l"adt den zum "ubergebenen Namen geh"orenden FA und 
 *  erstellt diesen auf der Zeichenfl"ache.
 *
 *--------------------------------------------------------------
 */

int tk_LoadFA( ClientData cd, Tcl_Interp * interp, int objc, 
		Tcl_Obj *CONST objv[])
{
  char          buffer[LINE_LENGTH];
  char          command[2][LINE_LENGTH];
  char         *rest, *line, *font, *code;
  char         *variables;
  int           object_cid;
  int           label_cid;
  int           isok;
  unsigned      line_number = 1;
  Comment_type *commentObject;
  Edge_type    *edgeObject;
  FILE         *fp;
  Font_type    *comment_font;
  Point_type    new_start;
  Point_type    new_end;
  Tcl_DString   tmp;
  Vertex_type  *vertexObject;
  int           version = 0;
  int           alreadyIn;
  long nr;
  Tcl_HashEntry *entry;

  if (objc != 2) {
      Tcl_WrongNumArgs(interp, 1, objv, "filename");
      return TCL_ERROR;
  }

  if ((fp = fopen (Tcl_GetStringFromObj(objv[1], NULL), "r")) == NULL) {
      Tcl_SetStringObj(Tcl_GetObjResult(interp), "Error while opening file",-1);
      return TCL_ERROR;
  }

  Tcl_InitHashTable (&id2cidTable, TCL_STRING_KEYS);
  Tcl_DStringInit (&tmp);

  /* 
   *  Es werden solange Zeilen aus dem File an den String-Buffer
   *  angehaengt, bis das Ende des Files erreicht ist
   */

  while (fgets (buffer, LINE_LENGTH, fp) != NULL) {
      Tcl_DStringAppend (&tmp, buffer, -1);
  }
  fclose (fp);
  rest = Tcl_DStringValue (&tmp);

  /* 
   *  Zunaechst wird getestet, ob die Einleitung dem neuen PEP-Dateiformat
   *  entspricht
   */

  lineStart( rest, &line, &line_number ); 
  if ((strncmp (line, "PEP", 3) != 0)) 
      goto error;
 
  line += 3;
  if( lineEnd( line, &line) != 0 ) {
      goto error;
  }
  lineStart( line, &line, &line_number );
  if((strncmp (line, "FiniteAutomaton", 15) != 0)) {
      goto error;
  }
  line += 15;
  if( lineEnd( line, &line) != 0 ) {
      goto error;
  }

  lineStart( line, &line, &line_number );
  if((strncmp (line, "FORMAT_N2", 9) == 0)) {
    version = 1;
  }
  if((strncmp (line, "FORMAT_N", 8) != 0)) {
      goto error;
  }
  line += 8;
  if (version)
    line++;
  if( lineEnd( line, &line) != 0 ) {
      goto error;
  }

  /* 
   *  Danach wird der Knoten-Font eingeleitet durch DPL angegeben. 
   *  Falls DPL in der Datei auftaucht, muss in der naechsten Zeile 
   *  auch der Font angegeben werden. Es besteht auch die Moeglichkeit 
   *  beides wegzulassen.
   */

  lineStart( line, &line, &line_number );
  if ((strncmp (line, "DPL", 3) == 0)) {	
      line += 3;
      if( lineEnd( line, &line) != 0 ) {
	  goto error;
      }

      /* 
       *  Der Font wird gelesen ...
       */

      lineStart( line, &line, &line_number );
      if( getString( line, &line, &(font), &line_number ) != 0 ) {
	  goto error;
      }
      if( lineEnd( line, &line) != 0 ) {
	  goto error;
      }

      sprintf (command[0], "editor(%d,vertexFont)", actualEd->number );

      /* 
       *  ... und es wird getestet, ob der Font existiert.
       */

      sprintf (command[1], "AskForFont \"%s\"", font );
      if (Tcl_GlobalEval (interp, command[1]) != TCL_OK) {
	  return TCL_ERROR;
      }
      if( Tcl_GetInt (interp, interp->result, &isok) != TCL_OK) {
	  return TCL_ERROR;
      }

      if( isok == 1) {
	  /* 
	   *  Wenn ja, kann er gesetzt werden.
	   */
	  if( Tcl_SetVar(interp,command[0],font, TCL_GLOBAL_ONLY) == NULL ) {
	      return TCL_ERROR;
	  }
      } else {
	  /* 
	   *  Wenn nein, wird eine Fehlermeldung ausgegeben.
	   */
	  goto error;
      }

      lineStart( line, &line, &line_number );
  }


  /* 
   *  Dann wird der Kanten-Font eingeleitet durch DAR angegeben. Falls 
   *  DAR in der Datei auftaucht, muss in der naechsten Zeile  der Font 
   *  angegeben werden. Es besteht auch die Moeglichkeit beides 
   *  wegzulassen.
   */

  if ((strncmp (line, "DAR", 3) == 0)) {	
      line += 3;
      if( lineEnd( line, &line) != 0 ) {
	  goto error;
      }

      /* 
       *  Der Font wird gelesen ...
       */

      lineStart( line, &line, &line_number );
      if( getString( line, &line, &(font), &line_number ) != 0) {
	  goto error;
      }
      if( lineEnd( line, &line) != 0 ) {
	  goto error; 
      }

      sprintf (command[0], "editor(%d,edgeFont)", actualEd->number );

      /* 
       *  ... und es wird getestet, ob der Font existiert.
       */

      sprintf (command[1], "AskForFont \"%s\"", font );
      if (Tcl_GlobalEval (interp, command[1]) != TCL_OK) {
	  return TCL_ERROR;
      }

      if( Tcl_GetInt (interp, interp->result, &isok) != TCL_OK) {
	  return TCL_ERROR;
      }

      if( isok == 1) {
	  /* 
	   *  Wenn ja, kann er gesetzt werden.
	   */
	  if( Tcl_SetVar(interp,command[0],font, TCL_GLOBAL_ONLY) == NULL) {
	      return TCL_ERROR;
	  }
      } else {
	  /* 
	   *  Wenn nein, wird eine Fehlermeldung ausgegeben.
	   */
	  goto error;
      }
      lineStart( line, &line, &line_number );
  }
  if ((strncmp (line, "PL", 2) != 0)) {	
      goto error;
  }
  line += 2;
  if( lineEnd( line, &line) != 0 ) {
      goto error;
  }

  /* 
   *  Eingeleitet durch PL folgen die gesamten Knoten-Informationen
   */

  lineStart( line, &line, &line_number );    
  while( strncmp (line, "AR", 2) != 0) { 
      vertexObject = MALLOC(struct Vertex_type);
      vertexObject->in = NULL;
      vertexObject->out = NULL; 
      vertexObject->own_edge = NULL;
      vertexObject->label = strdup("");

      /* 
       *  Es wird eine neue Knotenstruktur erstellt un dann zunaechst
       *  die alte Canvas-Id eingelesen, um spaeter die Kanten richtig 
       *  zuordnen zu koennen.
       */

      if( getNumber( interp, line, &line, &(vertexObject->v_id) ) != 0 ) {
	  goto error;
      }
      while( (*line == ' ') || (*line == '\t') ) {
	  line++;
      }
      /* 
       * Ok, falls die Knoten-Id noch nicht in der Knoten-Liste ist, diese
       * eintragen, ansonsten eine neue Id vergeben
       */
      entry = Tcl_CreateHashEntry(&NodeIds, (char *)vertexObject->v_id, &alreadyIn);
      if (!alreadyIn)
      {
	/* Der Knoten war schon in der Liste, also neue Id suchen */
	nr = 1;
	while (!alreadyIn)
	{	  
	  entry = Tcl_CreateHashEntry(&NodeIds, (char *)nr, &alreadyIn);
	  if (!alreadyIn)
	    nr++;
	  else
	    vertexObject->save_id=nr;
	}
      }
      else
      {
	vertexObject->save_id=vertexObject->v_id;
	nr=vertexObject->save_id;
      }
      Tcl_SetHashValue(entry, nr);

      /* 
       *  Dann folgt der Knoten-Mittelpunkt, ...
       */

      if( getXY( interp, line, &line, &(vertexObject->center.x),\
		 &(vertexObject->center.y) ) != 0 ) {
	  goto error;
      }
      while( (*line == ' ') || (*line == '\t') ) {
	  line++;
      }

      /* 
       *  ... der Typ des Knotens ( Start-, normaler oder Endknoten ) ...
       */

      if( *line == '1' ) {
	  vertexObject->type = 1;
      } else if( *line == '2' ) {
	  vertexObject->type = 2;
      } else if( *line == '3' )  {
	  vertexObject->type = 3;
      } else {
	  goto error;
      }
      line++;
      while( (*line == ' ') || (*line == '\t') ) {
	  line++;
      }

      /* 
       *  und zum Schluss die Beschriftung.
       */

      if( getString( line, &line, &(vertexObject->label),\
		     &line_number) != 0) {
	  goto error;
      }
      if( lineEnd( line, &line) != 0 ) {
	  goto error;
      }

      /* 
       *  Der Knoten wird auf der Canvas gezeichnet und die neuen  
       *  Canvas-Id's des Knotens und des Textes werden zurueckgegeben
       */

      if( printVertex( interp, vertexObject,\
		       &object_cid, &label_cid ) != TCL_OK ) {
	  return TCL_ERROR;
      }

      /*  Die Canvas-Id des Knotens wird zusammen mit der alten Id des 
       *  Knotens im id2cid-Hash-Table abgespeichert, um spaeter die 
       *  Kanten richtig setzten zu koennen.
       */

      if( add_cid (vertexObject->v_id, object_cid, interp) != TCL_OK ) {
	  return TCL_ERROR;
      }

      /* 
       * Die neuen Id's werden in die Struktur eingetragen, ... 
       */

      vertexObject->v_id = object_cid;
      vertexObject->label_id = label_cid;

      /* 
       *  die dann im Knoten-Hash-Table eingegliedert wird.
       */

      if( hashVertexInsert (interp, vertexObject) != TCL_OK ) {
	  return TCL_ERROR;
      }

      lineStart( line, &line, &line_number );    
  }

  /* 
   *  Die AR-Zeile fuer die Kanten muss uebergangen werden
   */

  line += 2;
  if( lineEnd( line, &line) != 0 ) {
      goto error;
  }

  /* 
   *  Dann folgen die gesamten Kanten-Informationen.
   */

  lineStart( line, &line, &line_number );    
  while( (strncmp (line, "TX", 2) != 0)) { /* && (*line != 0)*/
      edgeObject = MALLOC (Edge_type);
      edgeObject->move_infos.cos_alpha = 0.0;
      edgeObject->move_infos.sin_alpha = 0.0;
      edgeObject->move_infos.fraction  = 0.0;

      if ((version) && ( getNumber( interp, line, &line, &(object_cid) ) != 0 )) {
	  goto error;
      }
      /* 
       * Ok, falls die Knoten-Id noch nicht in der Knoten-Liste ist, diese
       * eintragen, ansonsten eine neue Id vergeben
       */
      alreadyIn=0;
      if (version)
      {
	nr=object_cid;
	entry = Tcl_CreateHashEntry(&EdgeIds, (char *)nr, &alreadyIn);
	if (!alreadyIn)
	  nr=1;
      }
      else
	nr=1;
      while (!alreadyIn)
      {	  
	entry = Tcl_CreateHashEntry(&EdgeIds, (char *)nr, &alreadyIn);
	if (!alreadyIn)
	  nr++;
	else
	  edgeObject->save_id=nr;
      }
      edgeObject->save_id=nr;
      Tcl_SetHashValue(entry, nr);

      /* 
       *  Es wird eine neue Knotenstruktur erstellt und dann zunaechst
       *  die alte Id's der Anfangs- und Endknoten eingelesen.
       */

      if( getOldStart_End( interp, line, &line, &(edgeObject->start),\
			   &(edgeObject->end) ) != 0 ) {
	  goto error;
      }
      while( (*line == ' ') || (*line == '\t') ) {
	  line++;
      }
      
      /* 
       *  Dann folgt der Mittelpunkt der Kante, ...
       */

      if( getXY( interp, line, &line, &(edgeObject->center.x),\
		 &(edgeObject->center.y) ) != 0 ) {
	  goto error;
      }
      while( (*line == ' ') || (*line == '\t') ) {
	  line++;
      }
      
      /* 
       *  ... die Differenz mit der der Text abgesetzt ist ...
       */

      if( getXY( interp, line, &line, &(edgeObject->label_center.x),\
		 &(edgeObject->label_center.y) ) != 0 ) {
	  goto error;
      }
      while( (*line == ' ') || (*line == '\t') ) {
	  line++;
      }

      /* 
       *  ... und schliesslich die Kantenbeschriftung.
       */

      if( getString( line, &line, &(edgeObject->label),\
		     &line_number ) != 0) {
	  goto error;
      }
      if( lineEnd( line, &line) != 0 ) {
	  goto error;
      }

      /* 
       *  Die neuen Id's des Anfangs- und Endpunktes der Kante werden aus
       *  dem id2cid-Table geholt.
       */

      sprintf (interp->result, "id%d", edgeObject->start);
      edgeObject->start = get_cid (interp->result);
      sprintf (interp->result, "id%d", edgeObject->end);
      edgeObject->end = get_cid (interp->result);

      if (edgeObject->start != edgeObject->end) {
	  /* 
	   *  Die Kante wird auf der Canvas gezeichnet und die neuen  
	   *  Canvas-Id's der Kante und des Textes zurueckgegeben
	   */
	  if( printEdge( interp, edgeObject,\
			 &object_cid, &label_cid ) != TCL_OK ) {
	      return TCL_ERROR;
	  }

	  edgeObject->e_id = object_cid;
	  edgeObject->label_id = label_cid;

	  /* 
	   *  Die Mittelpunkte des Anfangs- und Endpunktes der Kante 
	   *  werden geholt ...
	   */

	  if( getVertexCoords (interp, edgeObject->start,\
			       &new_start) != TCL_OK ) {
	      return TCL_ERROR;
	  }
	  if( getVertexCoords (interp, edgeObject->end,\
			       &new_end) != TCL_OK ) {
	      return TCL_ERROR;
	  }

	  /* 
	   *  ... um diese dann an die richtige Stelle zu plazieren.
	   */

	  if( changeEdgePos (interp, edgeObject, new_start, \
			     new_end, 0) != TCL_OK ) {
	      return TCL_ERROR;
	  }

      } else {

	  /*
	   *  Falls es sich um eine Schlinge handelt wird zunaechst die
	   *  Struktur des zugehoerigen Knotens geholt ...
	   */

	  sprintf(interp->result, "knoten%d",edgeObject->start);
	  vertexObject = getVertexStruct( interp->result);
	  if( vertexObject == NULL ) {
	      return TCL_ERROR;
	  }

	  /* 
	   *  ... dann die Kante auf der Canvas gezeichnet ...
	   */

	  if( printOwnEdge( interp, edgeObject,\
			    &object_cid, &label_cid ) != TCL_OK ) {
	      return TCL_ERROR;
	  }

	  edgeObject->e_id = object_cid;
	  edgeObject->label_id = label_cid;

	  /*
	   *  und schliesslich an der richtigen Stelle plaziert  
	   */

	  if( changeOwnEdgePos( interp,edgeObject,\
				vertexObject->center) != TCL_OK ) {
	      return TCL_ERROR;
	  }
      }
      
      /* 
       *  Die Kante wird in die Kanten-Listen des zugehoerigen 
       *  Knotens ...
       */

      if( EdgeToVertexList (interp, edgeObject ) != TCL_OK ) {
	  return TCL_ERROR;
      }

      /* 
       *  ... und in den Kanten-Hash-Table eingetragen
       */

      if( hashEdgeInsert (interp, edgeObject) != TCL_OK ) {
	  return TCL_ERROR;
      }

      /*
       *  Dann werden die Winkel der Kante fuer ein spaeteres Verschieben
       *  berechnet 
       */

      if( calculateAngles (interp, edgeObject->e_id) != TCL_OK ) {
	  return TCL_ERROR;
      }
      lineStart( line, &line, &line_number );    
  }

  /* 
   *  Die TX-Zeile fuer die Kommentare muss uebergangen werden
   */

  if((strncmp(line, "TX", 2) != 0) ) {
      goto error;
  }
  line += 2;
  if( lineEnd( line, &line) != 0 ) {
      goto error;
  }
  
  /* 
   *  Die Art der Kantenbeschriftung wird eingelesen
   */

  lineStart( line, &line, &line_number );    
  if( getString( line, &line, &(code), &line_number ) != 0) {
      goto error;
  }
  if( lineEnd( line, &line) != 0 ) {
      goto error;
  }
  if((strncmp(code,"Struct",6) != 0) && (strncmp(code,"B(PN)2",6) != 0)) {
      goto error;
  }

  /* 
   *  Die Variablen werden eingelesen
   */

  lineStart( line, &line, &line_number );    
  if( getString( line, &line, &(variables), &line_number ) != 0) {
      goto error;
  }
  if( lineEnd( line, &line) != 0 ) {
      goto error;
  }

  /* 
   *  Dann folgen die gesamten Informationen der Kommentare, falls
   *  vorhanden. 
   */

  lineStart( line, &line, &line_number );    
  while( *line != '\0') { 
      commentObject = MALLOC (Comment_type);
      comment_font = MALLOC (Font_type);

      /* 
       *  Es wird eine neue Kommentarstruktur und Fontstruktur erstellt und
       *  dann zunaechst der Font des Kommentars eingelesen
       */

      if( getFont( interp, line, &line, &(comment_font->family),\
		   &(comment_font->weight),\
		   &(comment_font->slant), &(comment_font->size) ) != 0 ) {
	  goto error;
      }
      while( (*line == ' ') || (*line == '\t') ) {
	  line++;
      }
      commentObject->font = comment_font;

      /* 
       *  Es wird getestet, ob der Font existiert.
       */

      sprintf (command[0], "AskForFont \"-*-%s-%s-%c-Normal--*-%d-*\"",\
	  comment_font->family,comment_font->weight,comment_font->slant,\
	  comment_font->size);
      if (Tcl_GlobalEval (interp, command[0]) != TCL_OK) {
	  return TCL_ERROR;
      }
      if( Tcl_GetInt (interp, interp->result, &isok) != TCL_OK) {
	  return TCL_ERROR;
      }

      if( isok == 0 ) {
	  goto error;
      }

      /* 
       *  Dann folgt der Mittelpunkt des Kommentars, ...
       */

      if( getXY( interp, line, &line, &(commentObject->center.x),\
		 &(commentObject->center.y) ) != 0 ) {
	  goto error;
      }
      while( (*line == ' ') || (*line == '\t') ) {
	  line++;
      }
      /* 
       *  ... und schliesslich der Kommentar selbst.
       */

      if( getString( line, &line, &(commentObject->comment),\
		     &line_number ) != 0) {
	  goto error;
      }
      if( lineEnd( line, &line) != 0 ) {
	  goto error;
      }

      /* 
       *  Der Kommentar wird auf der Canvas gezeichnet und die neue  
       *  Canvas-Id zurueckgegeben
       */

      if( printComment( interp, commentObject,&object_cid) != TCL_OK ) {
	  return TCL_ERROR;
      }

      commentObject->comment_id = object_cid;

      /* 
       *  Dann wird die Struktur in den Kommentar-Hash-Table eingetragen
       */

      if( hashCommentInsert (interp, commentObject) != TCL_OK ) {
	  return TCL_ERROR;
      }
      lineStart( line, &line, &line_number );    
  }
  
  /*
   *  Der String in dem das File gespeichert war kann geloescht werden
   */

  Tcl_DStringFree (&tmp);
  
  /* 
   *  Alle neuen Knoten-Id's sind den Kanten bekannt -> 
   *  Der id2cid-Hash-Table wird nicht mehr benoetigt
   */

  Tcl_DeleteHashTable (&id2cidTable);	

  /* 
   *  Die lokalen Variablen und der Code werden uebergeben
   */

  sprintf (command[0], "editor(%d,code)", actualEd->number );
  if( Tcl_SetVar(interp,command[0],code, TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG) == NULL)
      return TCL_ERROR;

  sprintf(command[0], "editor(%d,decl)"
	   , actualEd->number );
  if( Tcl_SetVar(interp,command[0],variables,\
		TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG) == NULL)
      return TCL_ERROR;

  /* 
   *  Fehlerbehandlung
   */

  Tcl_SetStringObj(Tcl_GetObjResult(interp),"ok 0",-1);
  return TCL_OK;

  /* 
   *  Falls beim Einlesen ein Fehler aufgetreten ist, wird an diese
   *  Stelle verzweigt 
   */
 
error: 

  Tcl_DStringFree (&tmp);
  Tcl_DeleteHashTable (&id2cidTable);	

  sprintf(buffer,"err %d",line_number);
  Tcl_SetStringObj(Tcl_GetObjResult(interp),buffer,-1);
  // TODO: Why TCL_OK and not _ERROR?
  return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Funktionen zu den Men"upunkten:                                     #
 #                                                                        #
 #  Save FA und Save As                                                   #
 #                                                                        #
 ##########################################################################
 */


/*  tk_SaveFA
 *--------------------------------------------------------------
 *   
 *  speichert den aktuellen FA unter dem als Parameter 
 *  angegebenen Namen ab.
 *
 *--------------------------------------------------------------
 */

int tk_SaveFA( ClientData cd, Tcl_Interp * interp, int objc, Tcl_Obj *CONST objv[] )

{
  Comment_type   *commentObject;
  Edge_type      *edgeObject;
  FILE           *fp;
  Tcl_HashEntry  *place_entry, *arc_entry, *comment_entry;
  Tcl_HashSearch  place_search, arc_search, comment_search;
  Vertex_type    *vertexObject;
  int start,end;
  int nr, alreadyIn;
  Tcl_HashEntry *entry;

  if (objc != 7) {
    Tcl_WrongNumArgs(interp, 1, objv, "filename vertexdef edgedef labels vars F");
    return TCL_ERROR;
  }

  if ((fp = fopen (Tcl_GetStringFromObj(objv[1], NULL), "w")) == NULL) {
    Tcl_SetStringObj(Tcl_GetObjResult(interp), "Error while opening file",-1);
    return TCL_ERROR;
  }

  if (Tcl_GetStringFromObj(objv[6],NULL) == "F")
  {
  }

  for (place_entry = Tcl_FirstHashEntry (&(actualEd->vertex_table), 
                                         &place_search);
       place_entry != NULL;
       place_entry = Tcl_NextHashEntry (&place_search)) {

    vertexObject = (Vertex_type *) Tcl_GetHashValue (place_entry);
    /* 
     * Ok, falls die Knoten-Id noch nicht in der Knoten-Liste ist, diese
     * eintragen, ansonsten eine neue Id vergeben
     */
    if (vertexObject->save_id == -1)
    {
      /* Der Knoten war schon in der Liste, also neue Id suchen */
      nr = 1;
      alreadyIn = 0;
      while (!alreadyIn)
      {	  
        entry = Tcl_CreateHashEntry(&NodeIds, (char *)nr, &alreadyIn);
        if (!alreadyIn)
          nr++;
        else
          vertexObject->save_id=nr;
      }
      Tcl_SetHashValue(entry, nr);
    }
  }


  for(arc_entry = Tcl_FirstHashEntry (&(actualEd->edge_table),\
				       &arc_search);
      arc_entry != NULL;
      arc_entry = Tcl_NextHashEntry (&arc_search)) {

    edgeObject = (Edge_type *) Tcl_GetHashValue (arc_entry);
    /* 
     * Ok, falls die Knoten-Id noch nicht in der Knoten-Liste ist, diese
     * eintragen, ansonsten eine neue Id vergeben
     */
    if (edgeObject->save_id == -1)
    {
      /* Der Knoten war schon in der Liste, also neue Id suchen */
      nr = 1;
      alreadyIn = 0;
      while (!alreadyIn)
      {	  
        entry = Tcl_CreateHashEntry(&EdgeIds, (char *)nr, &alreadyIn);
        if (!alreadyIn)
          nr++;
        else
          edgeObject->save_id=nr;
      }
      Tcl_SetHashValue(entry, nr);
    }
  }
      
  /*
   *  Die Einleitung des neuen PEP-Dateiformates wird abgespeichert
   */

  fprintf(fp, "PEP\n");
  fprintf(fp, "FiniteAutomaton\n");
  fprintf(fp, "FORMAT_N2\n");

  /* 
   *  Dann folgt der Knoten-Font, ...
   */

  fprintf(fp, "DPL %% defaults for vertices\n");
  fprintf(fp, "\"%s\"\n", Tcl_GetStringFromObj(objv[2],NULL));

  /* 
   *  ... der Kanten-Font, ...
   */

  fprintf(fp, "DAR %% defaults for edges\n");
  fprintf(fp, "\"%s\"\n", Tcl_GetStringFromObj(objv[3],NULL));

  /* 
   *  ... die gesamten Knoten aus dem Knoten-Hash-Table, ...
   */

  fprintf(fp, "PL %%all vertices\n");
  for (place_entry = Tcl_FirstHashEntry (&(actualEd->vertex_table), 
                                         &place_search);
       place_entry != NULL;
       place_entry = Tcl_NextHashEntry (&place_search)) {

    vertexObject = (Vertex_type *) Tcl_GetHashValue (place_entry);
    fprintf( fp, "%d  %d@%d %d \"%s\"\n",\
	     vertexObject->save_id,
	     vertexObject->center.x,vertexObject->center.y,\
	     vertexObject->type,\
	     vertexObject->label );
  }

  /* 
   *  ... die gesamten Kanten aus dem Kanten-Hash-Table ...
   */

  fprintf (fp, "AR  %% all edges\n");
  for(arc_entry = Tcl_FirstHashEntry (&(actualEd->edge_table),\
				       &arc_search);
      arc_entry != NULL;
      arc_entry = Tcl_NextHashEntry (&arc_search)) {

    edgeObject = (Edge_type *) Tcl_GetHashValue (arc_entry);
      
    sprintf(interp->result, "knoten%d", edgeObject->start);
    vertexObject = getVertexStruct( interp->result);
    start = vertexObject->save_id;

    sprintf(interp->result, "knoten%d", edgeObject->end);
    vertexObject = getVertexStruct( interp->result);
    end = vertexObject->save_id;

    fprintf( fp, "%d %d>%d %d@%d %d@%d \"%s\"\n", edgeObject->save_id,
             start, end,\
             edgeObject->center.x, edgeObject->center.y,\
             edgeObject->label_center.x, edgeObject->label_center.y,\
             edgeObject->label);
  }

  /* 
   *  ... und schliesslich alle Kommentare.
   */

  fprintf (fp, "TX\n");
  fprintf(fp, "%%Label - Code\n");
  fprintf(fp, "\"%s\"\n", Tcl_GetStringFromObj(objv[4],NULL));
  fprintf(fp, "%%Var - Decl\n");
  fprintf(fp, "\"%s\"\n", Tcl_GetStringFromObj(objv[5],NULL));
  fprintf(fp, "%%Comments\n");
  for(comment_entry =\
      Tcl_FirstHashEntry (&(actualEd->comment_table), &comment_search);
      comment_entry != NULL;
      comment_entry = Tcl_NextHashEntry (&comment_search)) {

      commentObject = (Comment_type *) Tcl_GetHashValue (comment_entry);
      fprintf(fp, "\"%s-%s-%c-%d\" %d@%d \"%s\"\n",\
	      commentObject->font->family,\
	      commentObject->font->weight,\
	      commentObject->font->slant,\
	      commentObject->font->size,\
	      commentObject->center.x,\
	      commentObject->center.y,\
	      commentObject->comment);
  }

  fclose (fp);

  return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Funktionen zu den Men"upunkten:                                     #
 #                                                                        #
 #  Close FA und Exit                                                     #
 #                                                                        #
 ##########################################################################
 */


/*  tk_DeleteFA
 *--------------------------------------------------------------
 *   
 *  l"oscht die Editor-Struktur, die zur "ubergenen Nummer 
 *  geh"ort und gibt den gesamten von ihr belegten Speicher
 *  frei.
 *
 *--------------------------------------------------------------
 */

int tk_DeleteFA( ClientData cd, Tcl_Interp * interp,\
		  int objc, Tcl_Obj *CONST objv[])
{
    Editor_type *actualDel;
    int num;

    if (objc != 2) {
	Tcl_WrongNumArgs(interp, 1, objv, "num");
	return TCL_ERROR;
    }

    /* 
     *  Es wird nach dem zu l"oschenden FA gesucht.
     */

    actualDel = firstEd;
    while( actualDel != NULL ) {
	if (Tcl_GetIntFromObj(interp, objv[1], &num) != TCL_OK)
	  return TCL_ERROR;
	if( actualDel->number != num ) {
	   actualDel = actualDel->next;
	} else {
	    break;
	}
    }

    /* 
     *  Der gesamte vom Editor belegte Speicher wird gel"oscht.
     */

    if( actualDel != NULL ) {
	actualEd = firstEd;

	clearHashTable(interp, actualDel);    
	clearUnsetList(actualDel->unsetList);
	clearUndeleteList(actualDel->undeleteList);

	/* 
	 *  Bevor der letzte Editor gel"oscht wird mu"s der
	 *  Speicher des copyObects freigegeben werden.
	 */

	if((firstEd->next == NULL) && (copyObject != NULL)) {
		listFree( copyObject->copyVertexList );
		listFree( copyObject->copyEdgeList );
		listFree( copyObject->copyCommentList );
		free(copyObject);
	}

	firstEd = deleteEd(firstEd, actualDel->number);	
	return TCL_OK;
    } else {
	return TCL_ERROR;
    }
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Funktionen zu den Men"upunkten:                                     #
 #                                                                        #
 #  Close FA und Exit                                                     #
 #                                                                        #
 ##########################################################################
 */


/*  getFont
 *--------------------------------------------------------------
 *   
 *  gibt die Strings zu einem eingelesenen Font zur"uck.
 *
 *--------------------------------------------------------------
 */

int getFont( Tcl_Interp *interp, register char *str, char **rest,\
	     char **family, char **weight, int *slant, int *size )
{
    char     *string_start;
    char     *string;
    unsigned  string_count;

    if( *str != '"' ) {
	return 1;
    }
    str++;

    string_start = str;
    for( string_count = 0; *str != '-'; string_count++, str++ )  {
	if( !isgraph(*str) ) {
	    return 1;
	}
    }

    string = malloc( sizeof( char *) * (string_count+1));
    strncpy( string, string_start, string_count);
    string[string_count] = '\0'; 
    *family = string;

    str++;
    string_start = str;
    for( string_count = 0; *str != '-'; string_count++, str++ )  {
	if( !isgraph(*str) ) {
	    return 1;
	}
    }

    string = malloc( sizeof( char *) * (string_count+1));
    strncpy( string, string_start, string_count);
    string[string_count] = '\0'; 
    str++;
    *weight = string;

    switch( *str ) {
    case 'r':
    case 'R':
	*slant = 'R';
	str++;
	break;
    case 'i':
    case 'I':
	*slant = 'I';
	str++;
	break;
    case 'o':
    case 'O':
	*slant = 'O';
	str++;
	break;
    default: return 1;
    }
    if( *str == '-' ) {
	str++;
    } else {
	return 1;
    }

    string_start = str;
    for( string_count = 0; *str != '"'; string_count++, str++ )  {
	if( *str == '\0' ) {
	    return 1;
	}
    }

    string = malloc( sizeof( char *) * (string_count+1));
    strncpy( string, string_start, string_count);
    string[string_count] = '\0'; 
    
    if (Tcl_GetInt(interp, string, size) != TCL_OK) {
	return 1;
    }
    free(string);

    str++;
    *rest = str;

    return 0;
}


/*  deleteEd
 *--------------------------------------------------------------
 *   
 *  sucht eine zu l"oschende Editor-Struktur und gibt deren
 *  Speicher frei.
 *
 *--------------------------------------------------------------
 */

Editor_type  *deleteEd(Editor_type *first, int number)
{
  int          found;
  Editor_type *help, *actual;  

  help = NULL;
  actual = first;
  found = 0;
  while( (actual != NULL) && (!found) ) {
      if( number != actual->number ) {
	  help = actual;
	  actual = actual->next;
      } else {
	found = 1;
      }
  }
  if( found > 0 ) {
      if( actual == first ) {
	  first = first->next;
      } else {
	  help->next = actual->next;
      }
      free( actual->canvas );
      free( actual );
  } else {
      return NULL;
  }

  return first;
}


/*  clearHashTable
 *--------------------------------------------------------------
 *   
 *  gibt den gesamten Speicher, der von den Hash-Table's eines
 *  Editor's belegt wird frei.
 *
 *--------------------------------------------------------------
 */

void clearHashTable(Tcl_Interp *interp, Editor_type *delEd) 
{
    Comment_type   *commentObject;
    Edge_type      *edgeObject;
    Tcl_HashEntry  *entryPtr;
    Tcl_HashSearch  search;
    Vertex_type    *vertexObject;

    for( entryPtr = Tcl_FirstHashEntry (&(delEd->vertex_table), &search);
	  entryPtr!= NULL;
	  entryPtr = Tcl_NextHashEntry(&search) )
    {
      vertexObject = (Vertex_type *) Tcl_GetHashValue( entryPtr );
      clearList( vertexObject->in );
      clearList( vertexObject->out ); 
      clearList( vertexObject->own_edge ); 
      free( vertexObject->label );
      free( vertexObject );
    }
    Tcl_DeleteHashTable(&(delEd->vertex_table));

    for( entryPtr = Tcl_FirstHashEntry (&(delEd->edge_table), &search);
	 entryPtr!= NULL;
	 entryPtr = Tcl_NextHashEntry(&search) )
    {
      edgeObject = (Edge_type *) Tcl_GetHashValue( entryPtr );
      free( edgeObject->label );
      free( edgeObject );
    }
    Tcl_DeleteHashTable(&(delEd->edge_table));

    for( entryPtr = Tcl_FirstHashEntry (&(delEd->comment_table), &search);
	 entryPtr!= NULL;
	 entryPtr = Tcl_NextHashEntry(&search) )
    {
	commentObject = (Comment_type *) Tcl_GetHashValue( entryPtr );
	free( commentObject->comment );
	free( commentObject->font->family );
	free( commentObject->font->weight );
	free( commentObject->font );
	free( commentObject );
    }
    Tcl_DeleteHashTable(&(delEd->comment_table));
}

void clearList (EdgeList_type *first)
{
    EdgeList_type *help;

    while (first != NULL) {
        help = first;
        first = first->next;
        free(help);
    }
}


/*  clearUnsetList
 *--------------------------------------------------------------
 *   
 *  gibt den gesamten Speicher, der Unset-Liste frei
 *
 *--------------------------------------------------------------
 */

void clearUnsetList(List_type *unsetList) 
{    
    List_element  *p;
    List_element  *q;
    UnsetTop_type *element;

    q = unsetList->head->next;
    for( p = q->next; p != unsetList->head;
		      q = p, p = p->next) {
	element = q->data;
	listFree( element->elementList );
	free(q); 
    }
    if (q != unsetList->head) free(q);
    free(unsetList->head);
    free(unsetList);
}


/*  clearUndeleteList
 *--------------------------------------------------------------
 *   
 *  gibt den gesamten Speicher, der Undelete-Liste frei
 *
 *--------------------------------------------------------------
 */

void clearUndeleteList(List_type *undeleteList) 
{    
    List_element  *p;
    List_element  *q;
    Undelete_type *element;

    q = undeleteList->head->next;
    for( p = q->next; p != undeleteList->head;
		      q = p, p = p->next) {
	element = q->data;
	listFree( element->killedVertexList );
	listFree( element->killedEdgeList );
	listFree( element->killedCommentList );
	free(q); 
    }
    if (q != undeleteList->head) free(q);
    free(undeleteList->head);
    free(undeleteList);

}

