###########################################################################
###########################################################################
##                                                                       ##
##  Datei   :     `pfaFile.tcl'                              		 ##
##                -------------                              		 ##
##                                                           		 ##
##  Inhalt  :     Prozeduren, die die Funktionalit"at        		 ##
##                des `File'-Men"us gew"ahrleisten.          		 ##
##                                                           		 ##
###########################################################################
###########################################################################
# verndert von Burkhard Bieber 1996
# last-update: 19.03.96


###########################################################################
##									 ##
##  Prozedur zum Men"upunkt:						 ##
##									 ##
##  New FA								 ##
##									 ##
###########################################################################


##  AddNewFA
##-----------------------------------------------------------
##  stellt einen neuen Editor mit Zeichenflaeche bereit.
##  Der zuvor aktuelle Editor wird in den Hintergrund
##  verlagert.
##-----------------------------------------------------------
##
proc AddNewFA {} {
    global faNumber histList opt editor

    incr faNumber(sum)

    ##  Die Zeichenfl"ache des alten FA
    ##  wird in den Hintergrund gebracht.
    ##
    if {$faNumber(sum) > 1} {
	pack forget $editor(frame)
	if { $opt(ruler) == "on"} {
	    CanvasNormalBindings
	}
    } else {
	.fr1a.fr2m.mb5 configure -state normal
    }

    lappend histList $faNumber(sum)
    set faNumber(actual) $faNumber(sum)
    pfa_NewFA $faNumber(actual)

    ##  Neue Zeichefl"ache wird erstellt
    ##  und in den Vordergrund gebracht.
    ##
    CreateNewCanvas
    pack propagate $editor(frame) 0
    pack $editor(frame)\
 	-in .fr1a.fr2ru\
	-expand true\
	-fill both
    update idletask
    MoveMode
    FreeMode

    ##  Die Optionen, die den FA betreffen
    ##  werden gesetzt.
    ##
    set editor($faNumber(actual),edited) "no"
    set editor($faNumber(actual),needQuestion) "yes"
    set editor($faNumber(actual),fullPath) \
	"$opt(workingDir)/NoName$faNumber(actual).net"
    set editor($faNumber(actual),decl) ""
    set editor($faNumber(actual),vertexFont)\
	"-*-Courier-Medium-R-Normal--*-100-*"
    set editor($faNumber(actual),edgeFont)\
	"-*-Courier-Medium-R-Normal--*-100-*"
    set editor($faNumber(actual),code) $opt(code)
    SomethingHasChanged "" highlight


    ##  Der Titel des Hauptfensters wird
    ##  auf den neuen Namen gesetzt und
    ##  ein neuer Men"ueintrag im Editormen"u
    ##  wird erstellt.
    ##
# TODO: Hauptfenstertitel
    wm title . "pepPFA - NoName$faNumber(actual).net"
    .fr1a.fr2m.mb5.m add radiobutton\
	-label "$faNumber(actual). $editor($faNumber(actual),fullPath)"\
	-variable faNumber(actual)\
	-value $faNumber(actual)\
	-command { TakeOtherFA $faNumber(actual) }

    ##  Wenn das Lineal eingeschaltet ist
    ##  werden die Bindings f"ur den neuen
    ##  FA gesetzt.
    ##
    if {($opt(ruler) == "on")} {
	if { ($faNumber(sum) == 1)}  {
	    SetRuler
	} else {
	    CanvasRulerBindings
	}
    }
}


###########################################################################
##									 ##
##  Prozeduren zum Men"upunkt:						 ##
##									 ##
##  Open FA								 ##
##									 ##
###########################################################################


##  FilemanagerOpenFA
##-----------------------------------------------------------
##  ruft die File-Select-Box zur Eingabe eines File-Namens
##  auf.
##-----------------------------------------------------------
##
proc FilemanagerOpenFA {} {
    global opt fs_filepath faNumber editor

    set fs_filepath $opt(workingDir)
    cd $fs_filepath
    set oldnumber $faNumber(sum)
    FSBox OpenFA "Open ..." $editor(filter,net)
	
    if {$faNumber(sum) != $oldnumber} {
	set info_text "Is this automaton used in other projects ?"
	set question [win_dialog .d {Question} $info_text largeinfo 0 {Yes} {No}]
	if {$question == 1} {
	    set editor($faNumber(actual),edited) "yes"
	    SaveFA
	}
	if {$question == 0} {
	    set info_text "Since this automaton is used in more than one project, it must be saved under a different name."
	    set question [win_dialog .d {Information} $info_text largeinfo 0 {Continue}]
	    set editor($faNumber(actual),edited) "yes"
	    FilemanagerSaveAs
	    if {$editor($faNumber(actual),edited) == "yes"} {
		DeleteActualFA
	    }
	    ## Es muss noch verhindert werden, dass der Automat unter demselben
	    ## Namen abgespeichert wird.
	}
    }
}

##  OpenFA
##-----------------------------------------------------------
##  erstellt einen neuen Editor und l"adt den ausgew"ahlten
##  FA. Falls w"ahrend des Ladevorganges Fehler auftreten,
##  wird der Ausgangszustand wieder hergestellt.
##-----------------------------------------------------------
##
proc OpenFA {} {
    global fs_file faNumber histList editor opt

    if {$fs_file == ""} {return}

    incr faNumber(sum)

    ##  Die alte Zeichenfl"ache und einige Werte werden gesichert,
    ##  f"ur den Fall, da"s das zu ladende File nicht in Ordnung ist.
    ##
    if {$faNumber(sum) > 1} {
	set oldframe $editor(frame)
	set oldcan $editor(canvas)
	set oldScrollX $editor(scrollX)
	set oldScrollY $editor(scrollY)
    }
    set old_actFANumber $faNumber(actual)
    set faNumber(actual) $faNumber(sum)
    set editor($faNumber(actual),vertexFont)\
	"-*-Courier-Medium-R-Normal--*-100-*"
    set editor($faNumber(actual),edgeFont)\
	"-*-Courier-Medium-R-Normal--*-100-*"

    ##  Neue C-Strukturen und neue Zeichenfl"ache werden
    ##  erstellt aber noch nicht in den Vordergrund
    ##  gebracht.
    ##

    pfa_NewFA $faNumber(actual)
    CreateNewCanvas

    set result [pfa_LoadFA $fs_file]
    update idletask
    if { [lindex $result 0] == "ok" } {

	if {$faNumber(sum) > 1} {
	    pack forget $oldframe
	    if { $opt(ruler) == "on"} {
		CanvasNormalBindings
	    }
	} else {
	    .fr1a.fr2m.mb5 configure -state normal
	}
	lappend histList $faNumber(sum)

	##  Neue Zeichenfl"ache wird in den Vordergrund
	##  gebracht.
	##
	pack propagate $editor(frame) 0
	pack $editor(frame) -in .fr1a.fr2ru -expand true -fill both
	MoveMode
	FreeMode

	##  Die lokalen Variablen werden von C aus
	##  mit Tcl_SetVar gesetzt und im folgenden
	##  in eine Liste verwandelt.
	##
	set editor($faNumber(actual),decl) \
			[split $editor($faNumber(actual),decl) \n]

	## $editor($faNumber(actual),code) wird von C aus gesetzt
	## $editor($faNumber(actual),vertexFont) wird von C aus gesetzt
	## $editor($faNumber(actual),edgeFont) wird von C aus gesetzt
	##
	set opt(code) $editor($faNumber(actual),code)
	set editor($faNumber(actual),fullPath) $fs_file
	set editor($faNumber(actual),edited) "no"
	set editor($faNumber(actual),needQuestion) "no"
	SomethingHasChanged "" highlight


	##  Der Titel des Hauptfensters wird
	##  auf den neuen Namen gesetzt und
	##  ein neuer Men"ueintrag im Editormen"u
	##  wird erstellt.
	##
	set fileName [file tail $fs_file]
# TODO: Hauptfenstertitel
	wm title . "pepPFA - $fileName"
	.fr1a.fr2m.mb5.m add radiobutton -label \
	    "$faNumber(actual). $editor($faNumber(actual),fullPath)"\
	    -variable faNumber(actual) -value $faNumber(actual)\
	    -command { TakeOtherFA $faNumber(actual)}

	##  Wenn das Lineal eingeschaltet ist
	##  werden die Bindings f"ur den neuen
	##  FA gesetzt.
	##
	if {($opt(ruler) == "on")} {
	    if { ($faNumber(sum) == 1)}  {
		SetRuler
	    } else {
		CanvasRulerBindings
	    }
	}

    } else {

	##  Wenn das File nicht in Ordnung war, wird
	##  der Ausgangszustand wieder hergestellt.
	##
	append info_text "File \"[file tail $fs_file]\"\n"
	append info_text "has no valid PEP-Format.\n"
	append info_text "Error in line [lindex $result 1]!"
	set question [win_dialog .d {Load Error}\
			  $info_text largeinfo 0\
			  {Continue}]
	if {$question == 0} {

	    destroy $editor(frame)
	    pfa_DeleteFA $faNumber(sum)

	    if {$faNumber(sum) > 1} {
		set editor(frame) $oldframe
		set editor(canvas) $oldcan
		set editor(scrollX) $oldScrollX
		set editor(scrollY) $oldScrollY
	    }

	    incr faNumber(sum) -1
	    if { $faNumber(sum) > 0 } {
		set faNumber(actual) $old_actFANumber
		pfa_ChangeFA $faNumber(actual)
	    }

	}
    }
}


###########################################################################
##									 ##
##  Prozedur zum Men"upunkt:						 ##
##									 ##
##  Reload FA								 ##
##									 ##
###########################################################################


##  ReloadFA
##-----------------------------------------------------------
##  stellt den gespeicherten Zustand eines FA wieder her.
##-----------------------------------------------------------
##
proc ReloadFA {} {
    global faNumber editor fs_filepath histList opt

    if { [llength $histList] == 0 } {
	InitialQuestion
	return
    }

    set filename  $editor($faNumber(actual),fullPath)
    if { $editor($faNumber(actual),edited) == "no"} {

	append info_text "There is no need to reload\n"
	append info_text "since nothing has changed\n"
	append info_text "in \"[file tail $filename]\""
	set question [win_dialog .d {Reload Message}\
			  $info_text largeinfo 0 {Continue}]
	if {$question == 0} {
	    return


	}
    } elseif { $editor($faNumber(actual),needQuestion) == "yes" } {
	append info_text "You can't reload this FA\n"
	append info_text "since it has not been saved!"
	set question [win_dialog .d {Reload Message}\
			  $info_text largeinfo 0 {Continue}]
	if {$question == 0} {
	    return
	}
    } else {

	append info_text "You will loose all changes in\n"
	append info_text "\"[file tail $filename]\""
	set question [win_dialog .d {Reload Message}\
			  $info_text largewarn 0 {Reload} {Cancel}]
	if {$question == 0} {

	    ##  Die Elemente der Zeichenfl"ache und die C-Strukturen
	    ##  werden gel"oscht und dann wird der Ausgangszustand
	    ##  wieder hergestellt.
	    ##
	    $editor(canvas) delete all
	    pfa_DeleteFA $faNumber(actual)
	    pfa_NewFA $faNumber(actual)

	    set result [pfa_LoadFA $filename ]
	    if { [lindex $result 0] != "ok" } {
		append info_text "File \"[file tail $filename]\"\n"
		append info_text "has no valid PEP-Format.\n"
		append info_text "Error in line [lindex $result 1]!"
		set question [win_dialog .d {Load Error}\
				  $info_text largeinfo 0 {Continue}]
		if {$question == 0} {
		    DeleteActualFA
		    if { [llength $histList] == 0 } {
			unset editor
			set faNumber(actual) 0
			set faNumber(sum) 0
			set histList {}
			.fr1a.fr2m.mb5 configure -state disabled
			focus .fr1a.fr2ru
			wm title . "pepPFA"
			set opt(mouse1) Free
		    }
		}
	    } else {

		
		FreeMode
		##  Die lokalen Variablen werden von C aus
		##  mit Tcl_SetVar gesetzt und im folgenden
		##  in eine Liste verwandelt.
		##
		set editor($faNumber(actual),decl) \
			[split $editor($faNumber(actual),decl) \n]

		## $editor($faNumber(actual),code) wird von C aus gesetzt.
		## $editor($faNumber(actual),vertexFont) wird von C aus.
		## gesetzt.
		## $editor($faNumber(actual),edgeFont) wird von C aus
		## gesetzt.
		##
		set opt(code) $editor($faNumber(actual),code)
		set editor($faNumber(actual),edited) "no"
		SomethingHasChanged "" highlight
	    }

	} elseif {$question == 1} {
	    return
	}
    }
}


###########################################################################
##									 ##
##  Prozedur zum Men"upunkt:						 ##
##									 ##
##  Save FA								 ##
##									 ##
###########################################################################


##  SaveFA
##-----------------------------------------------------------
##  speichert den aktuellen FA unter seinem Namen ab.
##-----------------------------------------------------------
##
proc SaveFA {} {
    global editor faNumber histList

    if { [llength $histList] == 0 } {
	InitialQuestion
	return
    }
    if { $editor($faNumber(actual),edited) == "no"} {

	append info_text "There is no need to save\n"
	append info_text "since nothing has changed\n"
	append info_text \
	    "In\"[file tail $editor($faNumber(actual),fullPath)]\""
	set question [win_dialog .d {Save Message}\
			  $info_text largeinfo 0 {Continue}]
	if {$question == 0} {
	    return
	}
    } else {
	if { $editor($faNumber(actual),needQuestion) == "yes" } {
	    FilemanagerSaveAs
	} else {
	    set locVars $editor($faNumber(actual),decl)
	    pfa_SaveFA $editor($faNumber(actual),fullPath)\
		$editor($faNumber(actual),vertexFont)\
		$editor($faNumber(actual),edgeFont)\
		$editor($faNumber(actual),code)\
		[join $locVars \n]\
		"N"
	    file_changed_server
	    set editor($faNumber(actual),edited) "no"
	}
    }
}


###########################################################################
##									 ##
##  Prozeduren zum Men"upunkt:						 ##
##									 ##
##  Save As								 ##
##									 ##
###########################################################################

##  FilemanagerSaveAs
##-----------------------------------------------------------
##  ruft die File-Select-Box zur Eingabe eines File-Namens
##  auf.
##-----------------------------------------------------------
##
proc FilemanagerSaveAs {} {
    global  editor faNumber fs_filepath oldPath histList

    if { [llength $histList] == 0 } {
	InitialQuestion
	return
    }
    set oldPath $editor($faNumber(actual),fullPath)
    set fs_filepath $editor($faNumber(actual),fullPath)
    cd [file dirname $editor($faNumber(actual),fullPath)]
    FSBox "SaveFA_As $fs_filepath" "Save As..." $editor(filter,net) . save
}

##  SaveFA_As
##-----------------------------------------------------------
##  speichert einen FA unter neuem Namen ab.
##-----------------------------------------------------------
##
proc SaveFA_As { name } {
    global editor faNumber oldPath

    if {$name==""} {return}

    set editor($faNumber(actual),fullPath) $name
    set editor($faNumber(actual),edited) "no"
    set editor($faNumber(actual),needQuestion) "no"

    set locVars $editor($faNumber(actual),decl)
    pfa_SaveFA $editor($faNumber(actual),fullPath) \
	$editor($faNumber(actual),vertexFont)\
	$editor($faNumber(actual),edgeFont)\
	$editor($faNumber(actual),code)\
	[join $locVars \n]\
	"N"
    file_changed_server

    set fileName [file tail $editor($faNumber(actual),fullPath)]
# TODO: Hauptfenstertitel
    wm title . "pepPFA - $fileName"
    .fr1a.fr2m.mb5.m\
	entryconfigure "$faNumber(actual). $oldPath"\
	-label "$faNumber(actual). $editor($faNumber(actual),fullPath)"
}


###########################################################################
##									 ##
##  Prozeduren zum Men"upunkt:						 ##
##									 ##
##  Close FA								 ##
##									 ##
###########################################################################


##  CloseDialog
##-----------------------------------------------------------
##  stellt mit einer Sicherheitsabfrage klar, ob der FA
##  wirklich gel"oscht werden soll. Wenn ja, k"onnen
##  Ver"anderungen abgespeichert werden.
##-----------------------------------------------------------
##
proc CloseDialog { } {
    global faNumber editor histList opt

    if { [llength $histList] == 0 } {
	InitialQuestion
	return
    }

    set fileName [file tail $editor($faNumber(actual),fullPath)]
    if {$editor($faNumber(actual),edited) == "yes"} {

	append info_text "You have unsaved changes in\n"
	append info_text "\"$fileName\" !"
	set question [win_dialog .d {Close}\
			  $info_text largeinfo 0\
			  {Save Changes} {Discard Changes} {Cancel}]
	if {$question == 0} {
	    set locVars $editor($faNumber(actual),decl)
	    pfa_SaveFA $editor($faNumber(actual),fullPath) \
		$editor($faNumber(actual),vertexFont)\
		$editor($faNumber(actual),edgeFont)\
		$editor($faNumber(actual),code)\
		[join $locVars \n]
	    file_changed_server

	} elseif {$question == 2} {
	    return $question
	}
    }
    DeleteActualFA
}


##  DeleteActualFA
##-----------------------------------------------------------
##  l"oscht den FA und bringt den vor dem gel"oschten FA
##  im Vordergrund befindlichen FA wieder in den Vordergrund
##-----------------------------------------------------------
##
proc DeleteActualFA {} {
    global faNumber histList opt editor fs_filepath

    ##  Der zu l"oschende Editor wird aus der Liste aller
    ##  Editore entfernt.
    ##
    set rightPlace [ lsearch $histList $faNumber(actual) ]
    set histList [lreplace $histList $rightPlace $rightPlace]
    set length [llength $histList]

    ##  Die Strukturen werden gel"oscht.
    ##
    .fr1a.fr2m.mb5.m delete \
	"$faNumber(actual). $editor($faNumber(actual),fullPath)"
    pfa_DeleteFA $faNumber(actual)
    unset editor($faNumber(actual),fullPath)
    unset editor($faNumber(actual),edited)
    unset editor($faNumber(actual),needQuestion)
    unset editor($faNumber(actual),decl)
    unset editor($faNumber(actual),code)
    unset editor($faNumber(actual),vertexFont)
    unset editor($faNumber(actual),edgeFont)

    if { $length > 0 } {

	if {$opt(ruler) == "on"} {
	    CanvasNormalBindings
	}

	destroy $editor(frame)
	##  Die Nummer des neuen Editor wird gesucht, und
	##  dann wird dieser in den Vordergrund gebracht.
	##
	set faNumber(actual) [lindex $histList [expr $length - 1]]
	pfa_ChangeFA $faNumber(actual)
	set name ".fr1a.fr2ru.fr3FA$faNumber(actual)"
	set editor(frame) $name
	set editor(canvas) $name.canv
	set editor(scrollX) "$name.scrollbarX"
	set editor(scrollY) "$name.f_right.scrollbarY"
	pack $editor(frame)\
	    -in .fr1a.fr2ru\
	    -expand true\
	    -fill both
	focus $editor(canvas)
	FreeMode
	set fileName [file tail $editor($faNumber(actual),fullPath)]
# TODO: Hauptfenstertitel
	wm title . "pepPFA - $fileName"
	SomethingHasChanged "" highlight

	if {$opt(ruler) == "on"} {
	    CanvasRulerBindings
	}

    } else {

	##  Der letzte Editor wurde gel"oscht ->
	##  pepPFA wird wieder in den Ausgangszustand
	##  versetzt.
	##
	if {$opt(ruler) == "on"}  {
	    NoRuler
	}

# TODO: Hauptfenstertitel
	wm title . "pepPFA"
	set opt(mouse1) Free
	set opt(mouse2) Free
	set opt(mouse3) Free
	set opt(project) no
	set opt(globalDecl) ""
	set faNumber(actual) 0
	set faNumber(sum) 0
	set histList {}
	.fr1a.fr2m.mb5 configure -state disabled
	focus .fr1a.fr2ru
	destroy $editor(frame)
    }


}


###########################################################################
###########################################################################
##									 ##
##  Prozeduren zum Untermen"u:						 ##
##									 ##
##  Project								 ##
##									 ##
###########################################################################
###########################################################################



###########################################################################
##									 ##
##  Prozeduren zum Untermen"upunkt:					 ##
##									 ##
##  Open Project							 ##
##									 ##
###########################################################################


##  ExistProject
##-----------------------------------------------------------
##  pr"uft, ob bereits an einem Projekt gearbeitet wird oder
##  ob sich bereits ge"offnete FA in pepPFA befinden. Falls
##  dies zutrifft erscheint ein Meldungsfenster.
##-----------------------------------------------------------
##
proc ExistProject {} {
    global opt histList

    if { $opt(project) == "yes" } {
	append info_text "Only one project is allowed.\n"
	append info_text "Do you want to open a new project?"
	set question [win_dialog .d {Project}\
		$info_text largeinfo 0 {Load} {Cancel}]
	if {$question == 0} {
	    foreach i $histList {
		TakeOtherFA $i
		if {[CloseDialog] == 2} {
		    return
		}
		update idletasks
	    }
	} elseif {$question == 1} {
	    return
	}
    } elseif { [llength $histList] != 0 } {
	append info_text "There are open editors in\n"
	append info_text "pepPFA. Do you want to close them\n"
	append info_text "before opening the new project?"
	set question [win_dialog .d {Project}\
	$info_text largeinfo 0 {Close FA} {Don't Close Them} {Cancel}]
	if {$question == 0} {
	    foreach i $histList {
		TakeOtherFA $i
		if {[CloseDialog] == 2} {
		    return
		}
		update idletasks
	    }
	} elseif {$question == 2} {
	    return
	}
    }

    FilemanagerOpenProject
}


##  FilemanagerOpenProject
##-----------------------------------------------------------
##  ruft die File-Select-Box zur Eingabe eines File-Namens
##  auf.
##-----------------------------------------------------------
##
proc FilemanagerOpenProject {} {
    global editor fs_filepath opt

    set fs_filepath $opt(workingDir)
    cd $fs_filepath
    FSBox OpenProject "Open project" $editor(filter,prj)
}

##  OpenProject
##-----------------------------------------------------------
##  Die globalen Variablen und die einzelnen Files des
##  Projektes werden der Reihe nach in pepPFA geladen.
##-----------------------------------------------------------
##
proc OpenProject { name } {
    global fs_file opt

    if {$name == ""} {return}

    set g [open $name r]

    ##  Laden der Headerzeilen
    ##
    while { [gets $g Zeile] >= 0 } {
	if { $Zeile != "" } {
	    lappend header $Zeile
	}	 

	if { $Zeile == "VAR" } {
	    break
	}
    } 
    if { $Zeile != "VAR" } {

	set header {error}
    }

    ##  Laden der globalen Variablen
    ##
    while {[gets $g Zeile] >= 0} {
	if { $Zeile != "FA" } {
	    lappend globVars $Zeile
	} else {
	    break
	}
    }

    if { $Zeile != "FA" } {
	set header {error}
    }

    ## Laden der Projekt-Files
    ##
    while {[gets $g Zeile] >= 0} {
	lappend files $Zeile
    }
    close $g

    ##  Test auf korrektes File-Format
    ##
    if { ([lindex $header 0] != "PEP") || 
	 ([lindex $header 1] != "PROJECT_pepPFA") || 
	 ([lindex $header 2] != "VAR") } {

	append info_text "$name\n"
	append info_text "Has no correct project format!"
	set question [win_dialog .d {Load Error}\
			  $info_text largewarn 0\
			  {Continue}]
	if {$question == 0} {
	    return
	} 
    }

    ##  "Ubergabe der globalen Variablen
    ##
    set opt(globalDecl) $globVars
    set opt(project) yes
    set opt(projectName) $name

    foreach element $files {
	if { $element != "" } {
	    if { ([file exists $element] == 1) } {
		set fs_file $element
		OpenFA
		update idletasks
	    } else {
		set info_text "File \"$element\"\n Does not exist!"
		set question [win_dialog .d {Load Error}\
				  $info_text largewarn 0\
				  {Continue} {Cancel}]
		if {$question == 0} {
		    continue
		} elseif {$question == 1} {
		    return
		}
	    }
	}
    }
}


###########################################################################
##									 ##
##  Prozeduren zum Untermen"upunkt:					 ##
##									 ##
##  Save Project							 ##
##									 ##
###########################################################################


##  FilemanagerSaveProject
##-----------------------------------------------------------
##  ruft die File-Select-Box zur Eingabe eines File-Namens
##  auf.
##-----------------------------------------------------------
##
proc FilemanagerSaveProject {} {
    global editor fs_filepath histList opt

    if { [llength $histList] == 0 } {
	InitialQuestion
	return
    }

    if {$opt(project) == "yes"} {
	cd [file dirname $opt(projectName)]
	set fs_filepath $opt(projectName)
    } else {
	cd $opt(workingDir)
	set fs_filepath $opt(workingDir)
    }
    FSBox SaveProject "Save..." $editor(filter,prj) . save
}


##  SaveProject
##-----------------------------------------------------------
##  speichert die globalen Variablen des Projektes und alle
##  in pepPFA befindlichen Editore ab.
##-----------------------------------------------------------
##
proc SaveProject { name } {
    global histList editor fs_filepath opt

    set g [open $fs_filepath w]
    puts $g "PEP"
    puts $g "PROJECT_pepPFA"
    puts $g "VAR"

    set globVars $opt(globalDecl)
    puts $g [join $globVars \n]

    puts $g "FA"
    set first 1
    foreach element $histList {
        set editor($element,edited) "yes"
	if { ($editor($element,needQuestion) == "yes") || \
	     ($editor($element,edited) == "yes")} {

	    TakeOtherFA $element
	    update idletasks

	    set locVars $editor($element,decl)
	    if { ($first == 1)} {
		pfa_SaveFA $editor($element,fullPath) \
			$editor($element,vertexFont)\
			$editor($element,edgeFont)\
			$editor($element,code)\
			[join $locVars \n]\
			"F"
		set first 0
	    } else {
		pfa_SaveFA $editor($element,fullPath) \
			$editor($element,vertexFont)\
			$editor($element,edgeFont)\
			$editor($element,code)\
			[join $locVars \n]\
			"N"
	    }


	    set editor($element,needQuestion) "no"
	    set editor($element,edited) "no"
	    file_changed_server


	}
	puts $g $editor($element,fullPath)
    }
    close $g
    set opt(project) yes
    set opt(projectName) $fs_filepath
    new_filename_server $fs_filepath
}


###########################################################################
###########################################################################
##									 ##
##  Prozeduren zum Untermenue:						 ##
##									 ##
##  PostScript								 ##
##									 ##
###########################################################################
###########################################################################



###########################################################################
##								         ##
##  Prozeduren zum Untermen"upunkt:					 ##
##									 ##
##  Export FA								 ##
##									 ##
###########################################################################


##  FilemanagerPostscript
##-----------------------------------------------------------
##  ruft die File-Select-Box zur Eingabe eines File-Namens
##  auf.
##-----------------------------------------------------------
##
proc FilemanagerPostscript { cmd } {
    global editor fs_filepath histList faNumber

    if { [llength $histList] == 0 } {
	InitialQuestion
	return
    }

    set fs_filepath \
	"[file rootname $editor($faNumber(actual),fullPath)].ps"
    cd [file dirname $editor($faNumber(actual),fullPath)]
    FSBox $cmd "Export to Postscript" $editor(filter,ps)
}


##  PostscriptAll
##-----------------------------------------------------------
##  Die gesamte Zeichenfl"ache wird unter dem angegebenen
##  File-Namen als Postscript-File abgespeichert.
##-----------------------------------------------------------
##
proc PostscriptAll { filename } {
    global editor

    set area [$editor(canvas) bbox all]
    if { [llength $area] > 0 } {
	set xlo [lindex $area 0]
	set ylo [lindex $area 1]
	set xru [lindex $area 2]
	set yru [lindex $area 3]
	set wi [expr $xru - $xlo]
	set he [expr $yru - $ylo]

	$editor(canvas) postscript -file $filename\
	    -x $xlo\
	    -y $ylo\
	    -width $wi\
	    -height $he
    }
}

##  PostscriptAllA4
##-----------------------------------------------------------
##  bietet die M"oglichkeit einen gro"sen FA auf mehrere
##  Files aufzuteilen, die jeweils die Gr"o"se einer
##  DIN-A4 Seite enthalten.
##-----------------------------------------------------------
##
proc PostscriptAllA4 { filename } {
    global editor

    set area [$editor(canvas) bbox all]
    if { [llength $area] > 0 } {
	set xlo [lindex $area 0]
	set ylo [lindex $area 1]
	set xru [lindex $area 2]
	set yru [lindex $area 3]
	set wi [expr $xru - $xlo]
	set he [expr $yru - $ylo]
	set din4w 597.44 ;# 210 mm (1pt: 0.3515 mm)
	set din4h 842.11 ;# 296 mm

	set wiCount [expr int([expr $wi / $din4w])]
	set heCount [expr int([expr $he / $din4h])]
	set nameCount 0
	for { set i 0 } { $i <= $wiCount } { incr i } {
	    for { set j 0 } { $j <= $heCount } { incr j } {

		incr nameCount
		$editor(canvas) postscript \
		    -file [file rootname $filename]${nameCount}.ps\
		    -x [expr $xlo + [expr $din4w * $i]]\
		    -y [expr $ylo + [expr $din4h * $j]]\
		    -width $din4w\
		    -height $din4h
	    }
	}
    }
}


##  PostscriptArea
##-----------------------------------------------------------
##  exportiert die Elemente innerhalb eines markierten
##  Bereiches nach Postscript.
##-----------------------------------------------------------
##
proc PostscriptArea { filename } {
    global editor areaCoords

    $editor(canvas) delete area
    $editor(canvas) addtag inarea overlapping\
	$areaCoords(X1) $areaCoords(Y1) $areaCoords(X2) $areaCoords(Y2)
    set inside [$editor(canvas) bbox inarea]
    $editor(canvas) dtag inarea
    if { [llength $inside] > 0 } {
	set xlo [lindex $inside 0]
	set ylo [lindex $inside 1]
	set xru [lindex $inside 2]
	set yru [lindex $inside 3]
	set wi [expr $xru - $xlo]
	set he [expr $yru - $ylo]
	$editor(canvas) postscript -file $filename\
	    -x $xlo\
	    -y $ylo\
	    -width $wi\
	    -height $he
    }
}



###########################################################################
##									 ##
##  Prozedur zum Men"upunkt:						 ##
##									 ##
##  Exit								 ##
##									 ##
###########################################################################


##  ExitDialog
##-----------------------------------------------------------
##  ruft ein Dialogfenster auf "uber das pepPFA verlassen
##  werden kann. F"ur jeden FA k"onnen ungesicherten
##  "Anderungen gesichert werden.
##-----------------------------------------------------------
##
proc ExitDialog {} {
    global faNumber histList

    set info_text "Do You Really Want To Leave The Editor?"
    set question [win_dialog .d {Exit}\
		      $info_text largewarn 0\
		      {Exit} {Cancel}]

    if {$question == 0} {
	foreach i $histList {
	    TakeOtherFA $i
	    if {[CloseDialog] == 2} {
		return
	    }
	    update idletasks
	}
        goodbye_server
	exit
    } else {
	return
    }
}
