/*
###########################################################################
###########################################################################
##                                                                       ##
##  Datei   :    `pfaModes.c'                                            ##
##               ------------                                            ##
##                                                                       ##
##  Inhalt  :    Funktionen, die die neuen Tcl-Befehl des Unter-         ##
##               men"us `Modes' implementieren.                          ##
##                                                                       ##
###########################################################################
###########################################################################
*/

#include <stdio.h>
#include <string.h>	
#include <stdlib.h>
#include <math.h>
#include "pfa.h"

/*
 ##########################################################################
 #                                                                        #
 #  Deklaration der lokalen Funktionen                                    #
 #                                                                        #
 ##########################################################################
 */

int doubleEdge( Tcl_Interp * interp, int start, int end );


/*
 ##########################################################################
 #                                                                        #
 #  C-Funktion zu den Men"upunkten:                                       #
 #                                                                        #
 #  S-Vertex, Vertex, F-Vertex                                            #
 #                                                                        #
 ##########################################################################
 */

/*  tk_VertexInfos
 *--------------------------------------------------------------
 *   
 *  "ubergibt die Knoteninformationen an den Hash-Table
 *  des aktuellen Editors actualEd->vertexTable.
 *
 *--------------------------------------------------------------
 */

int tk_VertexInfos(ClientData cd, Tcl_Interp * interp,\
		int objc, Tcl_Obj *CONST objv[])
{
    UnsetTop_type *partList_object;
    Vertex_type   *vertexObject;
    double x;

    if (objc != 6) {
	Tcl_WrongNumArgs(interp, 1, objv, "nodenum x y label type");
        return TCL_ERROR;
    }

    /* 
     *  Ein neues Object vom Typ Place wird erstellt, und die uebergebenen
     *  Parameter werden eingetragen
     */

    vertexObject = MALLOC(Vertex_type);
    if (Tcl_GetIntFromObj(interp, objv[1], &vertexObject->v_id ) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetDoubleFromObj(interp, objv[2], &x) != TCL_OK)
      return TCL_ERROR;
    vertexObject->center.x = x;
    if (Tcl_GetDoubleFromObj(interp, objv[3], &x) != TCL_OK)
      return TCL_ERROR;
    vertexObject->center.y = x;
    if (Tcl_GetIntFromObj(interp, objv[4], &vertexObject->label_id ) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetIntFromObj(interp, objv[5], &vertexObject->type ) != TCL_OK)
      return TCL_ERROR;
    vertexObject->label = strdup("");
    vertexObject->in = NULL;
    vertexObject->out = NULL; 
    vertexObject->own_edge = NULL;
    vertexObject->save_id = -1;
    /* 
     *  Ein Listenelement der Unset-Liste wird erstellt und am Anfang
     *  der Liste eingefuegt. Dann wird der Knoten in die Liste des
     *  zuvor initialisierten Listenelementes eingetragen
     */

    partList_object = makeUnsetElement( remove_element );
    if( partList_object == NULL ) {
	return TCL_ERROR;
    }

    insertUnsetStruct(partList_object, place, vertexObject->v_id,-1,-1);
    
    /* 
     *  Die Knotenstruktur wird in den Knoten-Hash-Table eingetragen
     */

    if( hashVertexInsert( interp, vertexObject ) != TCL_OK ) {
	return TCL_ERROR;
    }
    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Funktionen zum Men"upunkt:                                          #
 #                                                                        #
 #  Edge                                                                  #
 #                                                                        #
 ##########################################################################
 */

/*  tk_NormalEdgeInfos
 *--------------------------------------------------------------
 *   
 *  setzt die Kante an die richtige Position und "ubergibt 
 *  die Kanteninformationen an den Hash-Table des aktuellen 
 *  Editors actualEd->edgeTable.
 *
 *--------------------------------------------------------------
 */
int tk_NormalEdgeInfos(ClientData cd, Tcl_Interp * interp,\
		      int objc, Tcl_Obj *CONST objv[])
{
    double         fraction;
    int            number;
    int            quest;
    Edge_type     *edgeObject;
    Edge_type     *other_arc;
    EdgeList_type *testy;
    Point_type     start_point, end_point;
    Point_type     other_start, other_end;
    UnsetTop_type *partList_object;
    Vertex_type   *place_start, *place_end;
    int            editor_num;
    Tcl_Obj *objptr;

    if (objc != 5) {
	Tcl_WrongNumArgs(interp, 1, objv, "start end edgenum label");
        return TCL_ERROR;
    }
    /*
     *  Wir lassen uns die Editor-Nummer geben
     */

    objptr=Tcl_ObjGetVar2(interp, Tcl_NewStringObj("faNumber", -1),
		Tcl_NewStringObj("actual", -1), TCL_GLOBAL_ONLY);
    if(Tcl_GetIntFromObj(interp, objptr, &editor_num) != TCL_OK) {
        return TCL_ERROR;
    }

    /* 
     *  Ein neues Object vom Typ Arc wird erstellt, und die uebergebenen
     *  Parameter werden eingetragen
     */

    edgeObject = MALLOC(struct Edge_type);
    if (Tcl_GetIntFromObj(interp, objv[1], &edgeObject->start) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetIntFromObj(interp, objv[2], &edgeObject->end) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetIntFromObj(interp, objv[3], &edgeObject->e_id) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetIntFromObj(interp, objv[4], &edgeObject->label_id) != TCL_OK)
      return TCL_ERROR;
    edgeObject->label = strdup("");
    edgeObject->label_center.x = 0;
    edgeObject->label_center.y = 0;
    edgeObject->move_infos.cos_alpha = 0.0;
    edgeObject->move_infos.sin_alpha = 0.0;
    edgeObject->move_infos.fraction  = 0.0;
    edgeObject->save_id=-1;
    edgeObject->editor_num=editor_num;

    /* 
     *  Die Strukturen der zu dieser Kante gehoerenden Knoten werden
     *  geholt, um den Anfangs- und Endpunkt zu erhalten
     */

    sprintf(interp->result, "knoten%d",edgeObject->start);
    place_start = getVertexStruct( interp->result);
    if( place_start == NULL ) {
	return TCL_ERROR;
    }
    sprintf(interp->result, "knoten%d",edgeObject->end);
    place_end = getVertexStruct( interp->result);
    if( place_end == NULL ) {
	return TCL_ERROR;
    }

    start_point = place_start->center;
    end_point = place_end->center;

    /* 
     *  Die Anzahl der Kanten, die bereits von place_start nach place_end
     *  existieren, wird ermittelt 
     */

    number = 0;
    for( testy = place_start->out; testy != NULL; testy = testy->next ) {
	if( testy->other_vertex->v_id == edgeObject->end ) {
	    number += 1;
	}
    }

    if( number == 0 ) {	
	/*
	 *  Es handelt sich um die erste Kante. Als naechstes wird 
	 *  ermittelt, ob zu dieser Kante bereits eine andere in 
	 *  Gegenrichtung existiert.
	 */
	quest = doubleEdge( interp, edgeObject->start, edgeObject->end );
	if( quest > 0 ) { 
            /*
	     *  In diesem Fall handelt es sich um eine Doppelkante, d.h.
	     *  beide Kanten muessen abgesetzt werden, um sich nicht zu
	     *  ueberlappen
	     */
	    
	    /* 
	     *  Zunaechst wird die neu hinzugekommene Kante behandelt.
	     *  Dazu werden die neuen Winkel der Kante berechnet ... 
	     */

	    fraction =  0.12;
	    edgeObject->move_infos.cos_alpha = cos(fraction);
	    edgeObject->move_infos.sin_alpha = sin(fraction);
	    edgeObject->move_infos.fraction = 1.0/(2.0 * cos(fraction)); 

	    /* 
	     *  ... und schliesslich wird die Kante an die richtige 
	     *  Position gesetzt.
	     */

	    if( changeEdgePos( interp,edgeObject,start_point,\
			       end_point, 1) != TCL_OK ) {
		return TCL_ERROR; 
	    }

	    /* 
	     *  Danach wird die alte Kante behandelt
	     */

	    other_start = place_end->center;
	    other_end = place_start->center;
	    sprintf(interp->result, "arc%d",quest);
	    other_arc = getEdgeStruct( interp->result);
	    if( other_arc == NULL ) {
		return TCL_ERROR;
	    }

	    /* 
	     *  Die Winkel der alten Kante muessen an die der neuen Kante
	     *  angepasst werden ...
	     */

	    other_arc->move_infos.cos_alpha = \
		edgeObject->move_infos.cos_alpha;
	    other_arc->move_infos.sin_alpha = \
		edgeObject->move_infos.sin_alpha;
	    other_arc->move_infos.fraction = \
		edgeObject->move_infos.fraction;

	    /* 
	     *  ... der Text der alten Kante darf nicht mehr abgesetzt
	     *  erscheinen ...
	     */

	    other_arc->label_center.x  = 0;
	    other_arc->label_center.y  = 0;

	    /* 
	     *  ... und die alte Kante wird an die richtige Position
	     *  gesetzt
	     */

	    if( changeEdgePos( interp,other_arc,other_start,\
			       other_end, 1) != TCL_OK ) {
		return TCL_ERROR; 
	    }
	} else {
	    /* 
	     *  Es handelt sich um eine normale Kante, die als Strecke
	     *  zwischen Anfangs- und Endpunkt dargestellt wird
	     */

    	    edgeObject->center.x = (start_point.x + end_point.x)/2;
	    edgeObject->center.y = (start_point.y + end_point.y)/2;
	    edgeObject->move_infos.cos_alpha = 1.0;
	    edgeObject->move_infos.sin_alpha = 0.0;
	    edgeObject->move_infos.fraction = -4711.0;

	    if( changeEdgePos( interp,edgeObject,start_point,\
			       end_point, 1) != TCL_OK ) {
		return TCL_ERROR; 
	    }
	}
    } else {
	/* 
	 *  Es handelt sich nicht um die erste Kante von place_start nach
	 *  place_end. Damit sich diese Kante nicht mit den vorhergehende
	 *  ueberlappt, muss sie abgesetzt dargestellt werden
	 */

	fraction = 0.12 + ((double)number)*0.08;
	edgeObject->move_infos.cos_alpha = cos(fraction);
	edgeObject->move_infos.sin_alpha = sin(fraction);
	edgeObject->move_infos.fraction = 1.0/(2.0 * cos(fraction)); 

	if( changeEdgePos( interp,edgeObject,start_point, \
			   end_point, 1) != TCL_OK ) {
	    return TCL_ERROR; 
	}
    }
    
    /* 
     *  Ein Listenelement der Unset-Liste wird erstellt und am Anfang
     *  der Liste eingefuegt. Dann wird die Kante in die Liste des
     *  zuvor initialisierten Listenelementes eingetragen
     */

    partList_object = makeUnsetElement( remove_element );
    if( partList_object == NULL ) {
	return TCL_ERROR;
    }
    insertUnsetStruct(partList_object, edge,\
		      edgeObject->e_id,-1,-1);
    
    /* 
     *  Die Kante wird in die Kanten-Listen der zugehoerigen 
     *  Knoten ...
     */

    if( EdgeToVertexList( interp, edgeObject ) != TCL_OK ) {
	return TCL_ERROR;
    }

    /* 
     *  ... und in den Kanten-Hash-Table eingetragen
     */

    if( hashEdgeInsert( interp, edgeObject) != TCL_OK ) {
	return TCL_ERROR;
    }
    sprintf(interp->result,"1");
    return TCL_OK;
}

/*  tk_OwnEdgeInfos
 *--------------------------------------------------------------
 *   
 *  setzt die Schlinge an die richtige Position und "ubergibt 
 *  die Schlingeninformationen an den Hash-Table des aktuellen 
 *  Editors actualEd->edgeTable.
 *
 *--------------------------------------------------------------
 */

int tk_OwnEdgeInfos(ClientData cd, Tcl_Interp * interp,\
		   int objc, Tcl_Obj *CONST objv[])
{
    int            own_edgeCount;
    Edge_type     *edgeObject;
    EdgeList_type *testy;
    UnsetTop_type *partList_object;
    Vertex_type   *vertexObject;
    int            editor_num;
    Tcl_Obj *objptr;

    if (objc != 4) {
	Tcl_WrongNumArgs(interp, 1, objv, "start edgenum label");
        return TCL_ERROR;
    }

    /*
     *  Wir lassen uns die Editor-Nummer geben
     */

    objptr=Tcl_ObjGetVar2(interp, Tcl_NewStringObj("faNumber", -1),
		Tcl_NewStringObj("actual", -1), TCL_GLOBAL_ONLY);
    if(Tcl_GetIntFromObj(interp, objptr, &editor_num) != TCL_OK) {
        return TCL_ERROR;
    }

    /* 
     *  Ein neues Object vom Typ Arc wird erstellt, und die uebergebenen
     *  Parameter werden eingetragen
     */

    edgeObject = MALLOC(struct Edge_type);
    if (Tcl_GetIntFromObj(interp, objv[1], &edgeObject->start) != TCL_OK)
      return TCL_ERROR;
    edgeObject->end   = edgeObject->start;
    if (Tcl_GetIntFromObj(interp, objv[2], &edgeObject->e_id) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetIntFromObj(interp, objv[3], &edgeObject->label_id) != TCL_OK)
      return TCL_ERROR;
    edgeObject->label = strdup("");
    edgeObject->label_center.x = 0;
    edgeObject->label_center.y = 0;
    edgeObject->move_infos.cos_alpha = 1.0;
    edgeObject->move_infos.sin_alpha = 0.0;
    edgeObject->move_infos.fraction = 4711.0; 
    edgeObject->save_id=-1;
    edgeObject->editor_num=editor_num;

    /* 
     *  Die Strukturen des zu dieser Kante gehoerenden Knoten wird
     *  geholt, um den Anfangs- und Endpunkt zu erhalten
     */

    sprintf(interp->result, "knoten%d",edgeObject->start);
    vertexObject = getVertexStruct( interp->result);
    if( vertexObject == NULL ) {
	return TCL_ERROR;
    }
    
    /* 
     *  Es wird getestet wieviele Schlingen bereits zu diesem Knoten 
     *  gehoeren.
     */

    own_edgeCount = 0;
    for( testy = vertexObject->own_edge;
	 testy != NULL;
	 testy = testy->next ) {
	own_edgeCount++;
    }

    /* 
     *  Die spaetere Ausrichtung der Kante wird festgelegt:
     *  - die erste Kante zeigt nach oben
     *  - die zweite nach links
     *  - die dritte nach unten
     *  - die vierte nach rechts
     *  - alle weiteren werden nach oben abgesetzt gezeichnet
     */

    switch( own_edgeCount ) {
    case 0:
	edgeObject->center.x = vertexObject->center.x;
	edgeObject->center.y = vertexObject->center.y - 60 ;
	break;
    case 1:
	edgeObject->center.x = vertexObject->center.x - 60;
	edgeObject->center.y = vertexObject->center.y;
	break;
    case 2:
	edgeObject->center.x = vertexObject->center.x;
	edgeObject->center.y = vertexObject->center.y + 60 ;
	break;
    case 3:
	edgeObject->center.x = vertexObject->center.x + 60;
	edgeObject->center.y = vertexObject->center.y;
	break;
    default:
	edgeObject->center.x = vertexObject->center.x;
	edgeObject->center.y = vertexObject->center.y - \
	    30 - own_edgeCount*10;
	break;
    }
	    
    /* 
     *  Die Schlinge wird auf der Canvas plaziert
     */

    if( changeOwnEdgePos( interp,edgeObject,\
			  vertexObject->center) != TCL_OK ) {
	return TCL_ERROR;
    }

    /* 
     *  Ein Listenelement der Unset-Liste wird erstellt und am Anfang
     *  der Liste eingefuegt. Dann wird die Kante in die Liste des
     *  zuvor initialisierten Listenelementes eingetragen
     */

    partList_object = makeUnsetElement( remove_element );
    if( partList_object == NULL ) {
	return TCL_ERROR;
    }
    insertUnsetStruct(partList_object, edge, edgeObject->e_id,-1,-1);

    /* 
     *  Die Kante wird in die Kanten-Listen der zugehoerigen 
     *  Knoten ...
     */

    if (EdgeToVertexList(interp, edgeObject ) != TCL_OK ) {
	return TCL_ERROR;
    }

    /* 
     *  ... und in den Kanten-Hash-Table eingetragen
     */

    if( hashEdgeInsert( interp, edgeObject) != TCL_OK ) {
	return TCL_ERROR;
    }

    sprintf(interp->result,"1");
    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Funktion zum Men"upunkt:                                            #
 #                                                                        #
 #  Comment                                                               #
 #                                                                        #
 ##########################################################################
 */

/*  tk_CommentInfos
 *--------------------------------------------------------------
 *   
 *  "ubergibt die Informationen der Kommentare an den Hash-Table
 *  des aktuellen Editors actualEd->commentTable.
 *
 *--------------------------------------------------------------
 */

int tk_CommentInfos(ClientData cd, Tcl_Interp * interp,\
		       int objc, Tcl_Obj *CONST objv[])
{
    Comment_type  *commentObject;
    Font_type     *comment_font;
    UnsetTop_type *partList_object;
    double x;

    if (objc != 9) {
	Tcl_WrongNumArgs(interp, 1, objv, "commentnum x y comment fontfamily fontweight fontslant fontsize");
        return TCL_ERROR;
    }

    /* 
     *  Ein neues Object vom Typ Comment wird erstellt, und die 
     *  uebergebenen Parameter werden eingetragen
     */

    commentObject = MALLOC (Comment_type);
    if (Tcl_GetIntFromObj(interp, objv[1], &commentObject->comment_id) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetDoubleFromObj(interp, objv[2], &x) != TCL_OK)
      return TCL_ERROR;
    commentObject->center.x = x;
    if (Tcl_GetDoubleFromObj(interp, objv[3], &x) != TCL_OK)
      return TCL_ERROR;
    commentObject->center.y = x;
    commentObject->comment = strdup(Tcl_GetStringFromObj(objv[4], NULL));

    comment_font = MALLOC (Font_type);
    comment_font->family = strdup(Tcl_GetStringFromObj(objv[5], NULL));
    comment_font->weight = strdup(Tcl_GetStringFromObj(objv[6], NULL));
    comment_font->slant = *(Tcl_GetStringFromObj(objv[7], NULL));
    if (Tcl_GetIntFromObj(interp, objv[8], &comment_font->size) != TCL_OK)
      return TCL_ERROR;

    commentObject->font = comment_font;

    /* 
     *  Ein Listenelement der Unset-Liste wird erstellt und am Anfang
     *  der Liste eingefuegt. Dann wird der Kommentar in die Liste des
     *  zuvor initialisierten Listenelementes eingetragen
     */

    partList_object = makeUnsetElement( remove_element );
    if( partList_object == NULL ) {
	return TCL_ERROR;
    }
    insertUnsetStruct(partList_object, comment,\
		      commentObject->comment_id, -1, -1);

    /* 
     *  Die Struktur des Komentars wird in den Kommentar-Hash-Table 
     *  eingetragen.
     */

    if( hashCommentInsert( interp, commentObject ) != TCL_OK ) {
	return TCL_ERROR;
    }

    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Funktionen zum Men"upunkt:                                          #
 #                                                                        #
 #  Delete                                                                #
 #                                                                        #
 ##########################################################################
 */

/*  tk_VertexDelete
 *--------------------------------------------------------------
 *   
 *  ist f"ur das L"oschen der Knoten zust"andig.
 *
 *--------------------------------------------------------------
 */
int tk_VertexDelete(ClientData cd, Tcl_Interp * interp,\
		   int objc, Tcl_Obj *CONST objv[])
{
    Undelete_type *undeleteElement;
    int nodenum;

    if (objc != 2) {
	Tcl_WrongNumArgs(interp, 1, objv, "nodenum");
        return TCL_ERROR;
    }

    /* 
     *  Ein Listenelement der Undelete-Liste wird erstellt
     *  und am Anfang der Liste eingefuegt
     */

    undeleteElement = makeUndeleteElement();
    if( undeleteElement == NULL ) {
	return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[1], &nodenum) != TCL_OK)
      return TCL_ERROR;
    if( vertexDelete( interp, nodenum, undeleteElement ) != TCL_OK ) {
	return TCL_ERROR;
    }

    return TCL_OK;
}


/*  tk_EdgeDelete
 *--------------------------------------------------------------
 *   
 *  ist f"ur das L"oschen der Kanten zust"andig.
 *
 *--------------------------------------------------------------
 */

int tk_EdgeDelete(ClientData cd, Tcl_Interp * interp,\
		 int objc, Tcl_Obj *CONST objv[])
{
    Undelete_type *undeleteElement;
    int edgenum;

    if (objc != 2) {
	Tcl_WrongNumArgs(interp, 1, objv, "edgenum");
        return TCL_ERROR;
    }

    /* 
     *  Ein Listenelement der Undelete-Liste wird erstellt
     *  und am Anfang der Liste eingefuegt
     */

    undeleteElement = makeUndeleteElement();
    if( undeleteElement == NULL ) {
	return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[1], &edgenum) != TCL_OK)
      return TCL_ERROR;
    if( edgeDelete( interp, edgenum, undeleteElement ) != TCL_OK ) {
	return TCL_ERROR;
    }

    return TCL_OK;
}


/*  tk_CommentDelete
 *--------------------------------------------------------------
 *   
 *  ist f"ur das L"oschen der Kommentare zust"andig.
 *
 *--------------------------------------------------------------
 */

int tk_CommentDelete(ClientData cd, Tcl_Interp * interp,\
		     int objc, Tcl_Obj *CONST objv[])
{
    Undelete_type *undeleteElement;
    int commentnum;

    if (objc != 2) 
    {
	Tcl_WrongNumArgs(interp, 1, objv, "commentnum");
        return TCL_ERROR;
    }

    /* 
     *  Ein Listenelement der Undelete-Liste wird erstellt
     *  und am Anfang der Liste eingefuegt
     */

    undeleteElement = makeUndeleteElement();
    if( undeleteElement == NULL ) {
	return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[1], &commentnum) != TCL_OK)
      return TCL_ERROR;
    if( commentDelete( interp, commentnum, undeleteElement ) != TCL_OK ) {
	return TCL_ERROR;
    }

    return TCL_OK;
}


/*  tk_AreaDelete
 *--------------------------------------------------------------
 *   
 *  ist f"ur das L"oschen von markierten Bereichen zust"andig.
 *
 *--------------------------------------------------------------
 */

int tk_AreaDelete(ClientData cd, Tcl_Interp * interp,\
		  int objc, Tcl_Obj *CONST objv[])
{
    Tcl_Obj       **listElems;
    int             cid;
    int             listSum;
    int             index;
    Undelete_type  *undeleteElement;

    if (objc != 3) {
	Tcl_WrongNumArgs(interp, 1, objv, "nodelist commentlist");
        return TCL_ERROR;
    }

    /* 
     *  Ein Listenelement der Undelete-Liste wird erstellt
     *  und am Anfang der Liste eingefuegt
     */

    undeleteElement = makeUndeleteElement();
    if( undeleteElement == NULL ) {
	return TCL_ERROR;
    }

    /*
     *  Die als erstes Argument uebergebende Tcl-Liste der zu loeschenden
     *  Knoten wird in ein Array mit Zeigern auf die einzelnen 
     *  Listenelemente umgewandelt.
     */

    if( Tcl_ListObjGetElements( interp, objv[1], &listSum,\
		       &listElems ) != TCL_OK ) { 
	return TCL_ERROR;
    }

    /*
     *   Die einzelnen Knoten der Liste werden der Reihe nach geloescht.
     */

    index = 0;
    while( index < listSum ) {
	if (Tcl_GetIntFromObj(interp, listElems[index++], &cid) != TCL_OK)
	  return TCL_OK;
	if( vertexDelete(interp, cid, undeleteElement) != TCL_OK ) {
	    return TCL_ERROR;
	}
    }
    //free( (char *) listElems );

    /*
     *   Die als zweites Argument uebergebende Tcl-Liste der zu 
     *   loeschenden Kommentare wird in ein Array mit Zeigern auf die 
     *   einzelnen Listenelemente umgewandelt
     */

    if( Tcl_ListObjGetElements( interp, objv[2], &listSum,\
		       &listElems ) != TCL_OK ) {
	return TCL_ERROR;
    }

    /*
     *   Die einzelnen Kommentare der Liste werden der Reihe nach
     *   geloescht.
     */
    index = 0;
    while( index < listSum ) {
        if (Tcl_GetIntFromObj(interp, listElems[index++], &cid) != TCL_OK)
	  return TCL_ERROR;
	if( commentDelete( interp, cid, undeleteElement) != TCL_OK ) {
	    return TCL_ERROR;
	}
    }
    //free( (char *) listElems );

    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Funktionen zum Men"upunkt:                                          #
 #                                                                        #
 #  Line-Up                                                               #
 #                                                                        #
 ##########################################################################
 */

/*  tk_LineUpX
 *--------------------------------------------------------------
 *   
 *  setzt die x-Position aller auszurichtenden Knoten auf die
 *  x-Position des Referenzknotens.
 *
 *--------------------------------------------------------------
 */

int tk_LineUpX(ClientData cd, Tcl_Interp * interp,\
		 int objc, Tcl_Obj *CONST objv[])
{
    Tcl_Obj       **listElems;
    int             cid;
    int             listSum;
    int             index;
    Point_type      new_center;
    UnsetTop_type  *partList_object;
    Vertex_type    *vertexObject;
    Vertex_type    *comparevertexObject;

    if (objc != 3) {
	Tcl_WrongNumArgs(interp, 1, objv, "node nodelist");
        return TCL_ERROR;
    }

    /* 
     *  Ein Listenelement der Unset-Liste wird erstellt
     *  und am Anfang der Liste eingefuegt
     */

    partList_object = makeUnsetElement( move_element );
    if( partList_object == NULL ) {
	return TCL_ERROR;
    }

    /*
     *   Die als zweites Argument uebergebende Tcl-Liste der markierten 
     *   Knoten wird in ein Array mit Zeigern auf die einzelnen 
     *   Listenelemente umgewandelt.
     */

    if( Tcl_ListObjGetElements( interp, objv[2], &listSum,\
		       &listElems ) != TCL_OK ) {
	return TCL_ERROR;
    }

    /* 
     *  Die Struktur des Knotens, an dem die anderen ausgerichtet werden
     *  wird geholt
     */

    if (Tcl_GetIntFromObj(interp, objv[1], &cid) != TCL_OK)
      return TCL_ERROR;
    sprintf(interp->result, "knoten%d", cid);
    comparevertexObject = getVertexStruct( interp->result);
    if( comparevertexObject == NULL ) {
	return TCL_ERROR;
    }

    /*
     *   Die einzelnen Knoten der Liste werden der Reihe nach an den
     *   x-Wert des Vergleichsknotens angepasst
     */

    index = 0;
    while( index < listSum ) {
        if (Tcl_GetIntFromObj(interp, listElems[index++], &cid) != TCL_OK)
          return TCL_ERROR;
	sprintf(interp->result, "knoten%d",cid);
	vertexObject =  getVertexStruct( interp->result);
	if( vertexObject == NULL ) {
	    return TCL_ERROR;
	}

	/* 
	 *  Der Knoten wird in die Liste des zuvor initialisierten 
	 *  Elementes der Unset-Liste eingetragen 
	 */

	 insertUnsetStruct(partList_object, place, cid,
			   vertexObject->center.x,
			   vertexObject->center.y );

	/* 
	 *  Neue Knotenposition
	 */

	new_center.x = comparevertexObject->center.x;
	new_center.y = vertexObject->center.y;

	/* 
	 *  Knoten, Knotentext und Kanten werden auf der Canvas verschoben
	 */

	if( changeVertexPos( interp,vertexObject,\
			     new_center) != TCL_OK ) {
	    return TCL_ERROR;
	}
    }
    //free( (char *) listElems );

    return TCL_OK;
}


/*  tk_LineUpY
 *--------------------------------------------------------------
 *   
 *  setzt die y-Position aller auszurichtenden Knoten auf die
 *  y-Position des Referenzknotens.
 *
 *--------------------------------------------------------------
 */

int tk_LineUpY(ClientData cd, Tcl_Interp * interp,\
	       int objc, Tcl_Obj *CONST objv[])
{
    Tcl_Obj       **listElems;
    int             cid;
    int             listSum;
    int             index;
    Point_type      new_center;
    UnsetTop_type  *partList_object;
    Vertex_type    *vertexObject;
    Vertex_type    *comparevertexObject;

    if (objc != 3) {
	Tcl_WrongNumArgs(interp, 1, objv, "node nodelist");
        return TCL_ERROR;
    }

    /* 
     *  Ein Listenelement der Unset-Liste wird erstellt
     *  und am Anfang der Liste eingefuegt
     */

    partList_object = makeUnsetElement( move_element );
    if( partList_object == NULL ) {
	return TCL_ERROR;
    }

    /*
     *   Die als zweites Argument uebergebende Tcl-Liste der markierten
     *   Knoten wird in ein Array mit Zeigern auf die einzelnen 
     *   Listenelemente umgewandelt.
     */

    if( Tcl_ListObjGetElements( interp, objv[2], &listSum,\
		       &listElems ) != TCL_OK ) {
	return TCL_ERROR;
    }

    /* 
     *  Die Struktur des Knotens, an dem die anderen ausgerichtet werden
     *  wird geholt.
     */

    if (Tcl_GetIntFromObj(interp, objv[1], &cid) != TCL_OK)
      return TCL_ERROR;
    sprintf(interp->result, "knoten%d", cid);
    comparevertexObject = getVertexStruct( interp->result);
    if( comparevertexObject == NULL ) {
	return TCL_ERROR;
    }

    /*
     *   Die einzelnen Knoten der Liste werden der Reihe nach an den
     *   y-Wert des Vergleichsknotens angepasst.
     */

    index = 0;
    while( index < listSum ) {
        if (Tcl_GetIntFromObj(interp, listElems[index++], &cid) != TCL_OK)
          return TCL_ERROR;
	sprintf(interp->result, "knoten%d",cid);
	vertexObject =  getVertexStruct( interp->result);
	if( vertexObject == NULL ) {
	    return TCL_ERROR;
	}

	/* 
	 *  Der Knoten wird in die Liste des zuvor initialisierten 
	 *  Elementes der Unset-Liste eingetragen. 
	 */

	insertUnsetStruct(partList_object, place, cid,
			  vertexObject->center.x,
			  vertexObject->center.y );

	/* 
	 *  Neue Knotenposition
	 */

	new_center.x = vertexObject->center.x;
	new_center.y = comparevertexObject->center.y;

	/* 
	 *  Knoten, Knotentext und Kanten werden auf der Canvas verschoben
	 */

	if( changeVertexPos( interp,vertexObject,\
			     new_center) != TCL_OK ) {
	    return TCL_ERROR;
	}
    }
    //free( (char *) listElems );

    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Funktionen zum Men"upunkt:                                          #
 #                                                                        #
 #  Move-Text                                                             #
 #                                                                        #
 ##########################################################################
 */

/*  tk_UnsetTextCoords
 *--------------------------------------------------------------
 *   
 *  speichert die Position des Kantentextes vor dem Verschieben
 *  in der Unset-Liste, um den Text sp"ater wieder auf die
 *  alte Position zur"ucksetzen zu k"onnen.
 *
 *--------------------------------------------------------------
 */

int tk_UnsetTextCoords(ClientData cd, Tcl_Interp * interp,\
		       int objc, Tcl_Obj *CONST objv[])
{
    Edge_type     *edgeObject;
    Point_type     text;
    UnsetTop_type *partList_object;
    int arcnum;

    if (objc != 2) {
	Tcl_WrongNumArgs(interp, 1, objv, "arcnum");
        return TCL_ERROR;
    }

    /* 
     *  Die Struktur der Kante wird geholt, um die aktuelle Position des
     *  Textes zu erhalten.
     */

    if (Tcl_GetIntFromObj(interp, objv[1], &arcnum) != TCL_OK)
      return TCL_ERROR;
    sprintf(interp->result, "arc%d", arcnum);
    edgeObject = getEdgeStruct( interp->result);
    if( edgeObject == NULL ) {
	return TCL_ERROR;
    }

    text.x = edgeObject->center.x + edgeObject->label_center.x;
    text.y = edgeObject->center.y + edgeObject->label_center.y;

    /* 
     *  Ein Listenelement der Unset-Liste wird erstellt und am Anfang
     *  der Liste eingefuegt. Dann werden die Daten der Kante mit der 
     *  Textposition in die Liste des zuvor initialisierten Listenelementes
     *  eingetragen
     */

    partList_object = makeUnsetElement( move_element );
    if( partList_object == NULL ) {
	return TCL_ERROR;
    }

    insertUnsetStruct(partList_object,
		      edge_text, edgeObject->e_id,\
		      text.x, text.y);

    return TCL_OK;
}


/*  tk_ChangeEdgeTextCoords
 *--------------------------------------------------------------
 *   
 *  leitet das Verschieben des Kantentextes ein.
 *
 *--------------------------------------------------------------
 */

int tk_ChangeEdgeTextCoords( ClientData cd, Tcl_Interp * interp,\
			   int objc, Tcl_Obj *CONST objv[])
{
    Point_type center;
    int textnum;
    double x;

    if (objc != 4) {
	Tcl_WrongNumArgs(interp, 1, objv, "annonum x y");
        return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[2], &textnum) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetDoubleFromObj(interp, objv[2], &x) != TCL_OK)
      return TCL_ERROR;
    center.x = x;
    if (Tcl_GetDoubleFromObj(interp, objv[3], &x) != TCL_OK)
      return TCL_ERROR;
    center.y = x;
    
    if( changeTextPos( interp, textnum, center ) != TCL_OK ) {
	return TCL_ERROR; 
    }

    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Funktion zum Men"upunkt:                                            #
 #                                                                        #
 #  Rename                                                                #
 #                                                                        #
 ##########################################################################
 */

/*  tk_Rename
 *--------------------------------------------------------------
 *   
 *  ver"andert das Label von Knoten und Kanten bzw. Kommmentare
 *  in den jeweiligen Datenstrukturen und auf der Zeichen-
 *  fl"ache.
 *
 *--------------------------------------------------------------
 */

int tk_Rename(ClientData cd, Tcl_Interp * interp,\
		      int objc, Tcl_Obj *CONST objv[])
{
    char         *label;
    char          command[LINE_LENGTH];
    int           cid;
    int           label_id;
    Comment_type *commentObject;
    Edge_type    *edgeObject;
    Vertex_type  *vertexObject;

    if (objc != 3) {
	Tcl_WrongNumArgs(interp, 1, objv, "id string");
        return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj( interp, objv[1], &cid ) != TCL_OK)
      return TCL_ERROR;
    label = strdup( Tcl_GetStringFromObj(objv[2], NULL) );

    /* 
     *  Zuerst wird getestet, ob der Text einer Kante geaendert werden
     *  soll ... 
     */

    sprintf(interp->result, "arc%d",cid);
    edgeObject = getEdgeStruct( interp->result);
    if( edgeObject != NULL ) {
	edgeObject->label = label;
	label_id = edgeObject->label_id;
    } else {
	/* 
	 *  ... falls die Suche nicht erfolgreich war, wird getestet, ob 
	 *  der Text eines Knotens geaendert werden soll ... 
	 */

	sprintf(interp->result, "knoten%d",cid);
	vertexObject = getVertexStruct( interp->result);
	if( vertexObject != NULL ) { 
	    vertexObject->label = label;
	    label_id = vertexObject->label_id;
	} else {
	    /* 
	     *  ... falls die Suche nicht erfolgreich war, wird getestet, 
	     *  ob der Text eines Kommentars geaendert werden soll ... 
	     */

	    sprintf(interp->result, "comment%d",cid);
	    commentObject = getCommentStruct( interp->result);
	    if( commentObject != NULL ) {
		commentObject->comment = label;
		label_id = commentObject->comment_id;
	    }  else {
		/* 
		 *  ... falls die Suche nicht erfolgreich war, soll der 
		 *  Text eines Objektes geaendert werden, dass es nicht 
		 *  gibt -> Fehler.
		 */

		sprintf(interp->result,\
			"No existing Object with ID %d\n",cid);	
		return TCL_ERROR;
	    }
	}
    }
    
    /* 
     *  Der Text muss noch auf der Canvas geaendert werden.  
     */

    sprintf( command, "%s itemconfigure %d -text \"%s\"",\
	     actualEd->canvas,\
	     label_id,label);
    if( Tcl_GlobalEval( interp,command ) != TCL_OK ) {
	return TCL_ERROR; 
    }
    
    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Funktion zum Men"upunkt:                                            #
 #                                                                        #
 #  Reset-Edge                                                            #
 #                                                                        #
 ##########################################################################
 */

/*  tk_ResetEdge
 *--------------------------------------------------------------
 *   
 *  setzt die Form einer Kante auf den urspr"unglichen Zustand
 *  zur"uck.
 *
 *--------------------------------------------------------------
 */

int tk_ResetEdge( ClientData cd, Tcl_Interp * interp,\
		 int objc, Tcl_Obj *CONST objv[])
{
    double         fraction;
    int            quest;
    int            number;
    int            own_edgeCount;
    Edge_type     *edgeObject;
    EdgeList_type *testy;
    Point_type     start_point;
    Point_type     end_point;
    Vertex_type   *vertexObject;
    Vertex_type   *place_start;
    Vertex_type   *place_end;

    if (objc != 2) {
	Tcl_WrongNumArgs(interp, 1, objv, "arcnum");
        return TCL_ERROR;
    }

    /* 
     *  Die Struktur der Kante wird geholt
     */

    if (Tcl_GetIntFromObj(interp, objv[1], &number) != TCL_OK)
      return TCL_ERROR;
    sprintf(interp->result, "arc%d", number);
    edgeObject = getEdgeStruct( interp->result);
    if( edgeObject == NULL ) {
	return TCL_ERROR;
    }

    if( edgeObject->start != edgeObject->end ) {

	/* 
	 *  Die Strukturen der zu dieser Kante gehoerenden Knoten werden
	 *  geholt, um den Anfangs- und Endpunkt zu erhalten
	 */

	sprintf(interp->result, "knoten%d",edgeObject->start);
	place_start = getVertexStruct( interp->result);
	if( place_start == NULL ) {
	    return TCL_ERROR;
	}
	sprintf(interp->result, "knoten%d",edgeObject->end);
	place_end = getVertexStruct( interp->result);
	if( place_end == NULL ) {
	    return TCL_ERROR;
	}

	start_point = place_start->center;
	end_point = place_end->center;

	/* 
	 *  Die Anzahl der Kanten, die bereits von place_start nach 
	 *  place_end existieren, wird ermittelt. 
	 */

	number = 0;
	for( testy = place_start->out; 
	     testy != NULL; 
	     testy = testy->next ) {
	    if( testy->other_vertex->v_id == edgeObject->end) {
		number += 1;		
		if( testy->edge_object->e_id == edgeObject->e_id ) {
		    break;
		}
	    }
	}
	if( number == 1 ) {	
	    quest = doubleEdge( interp, edgeObject->start,\
				edgeObject->end); 
	    if( quest > 0 )  {
		
		/*
		 *  In diesem Fall handelt es sich um eine Doppelkante, 
		 *  d.h. beide Kanten muessen abgesetzt werden, um sich 
		 *  nicht zu ueberlappen.
		 */

		fraction = 0.12;
		edgeObject->move_infos.cos_alpha = cos(fraction);
		edgeObject->move_infos.sin_alpha = sin(fraction);
		edgeObject->move_infos.fraction = \
		    1.0/(2.0 * cos(fraction)); 
		
		/*  
		 *  Der Text muss auch zurueckgesetzt werden
		 */  

		edgeObject->label_center.x  = 0;
		edgeObject->label_center.y  = 0;
		if( changeTextPos(interp, edgeObject->e_id,\
				  edgeObject->center) != TCL_OK ) {
		    return TCL_ERROR;
		}
		if( changeEdgePos( interp,edgeObject,start_point,\
				   end_point, 1) != TCL_OK ) {
		    return TCL_ERROR; 
		}
	    } else {
		/*  
		 *  Der Text muss auch zurueckgesetzt werden
		 */  

		edgeObject->label_center.x  = 0;
		edgeObject->label_center.y  = 0;
		if( changeTextPos(interp, edgeObject->e_id,\
				  edgeObject->center) != TCL_OK ) {
		    return TCL_ERROR;
		}

		edgeObject->move_infos.cos_alpha = 1.0;
		edgeObject->move_infos.sin_alpha = 0.0;
		edgeObject->move_infos.fraction = -4711.0;
		if( changeEdgePos( interp, edgeObject, start_point,\
				   end_point, 1) != TCL_OK ) {
		    return TCL_ERROR;
		}
	    }
	} else {
	    /* 
	     *  Es handelt sich nicht um die erste Kante von place_start 
	     *  nach place_end. Die Kante wird an ihren zuerst belegten
	     *  Platz zurueckgesetzt 
	     */

	    fraction = 0.12 + ((double)number-1)*0.08 ;
	    edgeObject->move_infos.cos_alpha = cos(fraction);
	    edgeObject->move_infos.sin_alpha = sin(fraction);
	    edgeObject->move_infos.fraction = 1.0/(2.0 * cos(fraction)); 

	    if( changeEdgePos( interp,edgeObject,start_point,\
			       end_point, 1) != TCL_OK ) {
		return TCL_ERROR; 
	    }
    
	    edgeObject->label_center.x  = 0;
	    edgeObject->label_center.y  = 0;
	    if( changeTextPos(interp, edgeObject->e_id,\
			      edgeObject->center) != TCL_OK ) {
		return TCL_ERROR;
	    }
	}	 
    } else { /* Schlinge */
	/* 
	 *  Die Strukturen des zu dieser Kante gehoerenden Knoten wird
	 *  geholt, um den Anfangs- und Endpunkt zu erhalten
	 */

	sprintf(interp->result, "knoten%d",edgeObject->start);
	vertexObject = getVertexStruct( interp->result);
	if( vertexObject == NULL ) {
	    return TCL_ERROR;
	}

	/* 
	 *  Es wird getestet, um die wievielte Schlinge es sich handelt.
	 */

	own_edgeCount = 0;
	for( testy = vertexObject->own_edge;
	     testy != NULL;
	     testy = testy->next ) {
	    if( testy->edge_object->e_id == edgeObject->e_id ) {
		break;
	    }
	    own_edgeCount++;
	}

	/* 
	 *  Die Ausrichtung der Schlinge wird festgelegt
	 */

	switch( own_edgeCount ) {
	case 0:
	    edgeObject->center.x = vertexObject->center.x;
	    edgeObject->center.y = vertexObject->center.y - 60 ;
	    break;
	case 1:
	    edgeObject->center.x = vertexObject->center.x - 60;
	    edgeObject->center.y = vertexObject->center.y;
	    break;
	case 2:
	    edgeObject->center.x = vertexObject->center.x;
	    edgeObject->center.y = vertexObject->center.y + 60 ;
	    break;
	case 3:
	    edgeObject->center.x = vertexObject->center.x + 60;
	    edgeObject->center.y = vertexObject->center.y;
	    break;
	default:
	    edgeObject->center.x = vertexObject->center.x;
	    edgeObject->center.y = vertexObject->center.y - \
		30 - own_edgeCount*10;
	    break;
	}

	/*
	 *  Die Kante wird an die alte Position gesetzt
	 */

	if( edgeMove(interp, edgeObject->e_id,\
		     edgeObject->center) != TCL_OK ) {
	    return TCL_ERROR;
	}
	if( calculateAngles(interp, edgeObject->e_id) != TCL_OK ) {
	    return TCL_ERROR;
	}
	
	/*  
	 *  Der Text muss auch zurueckgesetzt werden
	 */  

	if( changeTextPos(interp, edgeObject->e_id,\
			  edgeObject->center) != TCL_OK ) {
	    return TCL_ERROR;
	}
	edgeObject->label_center.x  = 0;
	edgeObject->label_center.y  = 0;
    }
    
    /* 
     *  In der Unset-Liste muessen die alten Verschiebungen
     *  geloescht werden
     */

    cleanUnsetWithoutDelete( edgeObject->e_id );

    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Hilfsfunktion                                                       #
 #                                                                        #
 ##########################################################################
 */

/*  doubleEdge
 *--------------------------------------------------------------
 *   
 *  testet, ob zu einer neuen Kante bereits eine andere
 *  in entgegengesetzter Richtung existiert.
 *
 *--------------------------------------------------------------
 */

int doubleEdge( Tcl_Interp * interp, int start, int end )
{
    EdgeList_type *in_arcs;
    Vertex_type   *place_start=NULL;

    /* 
     *  Struktur des Startknotens der Kante holen ...
     */

    sprintf(interp->result, "knoten%d",start);
    place_start = getVertexStruct( interp->result );
    if( place_start == NULL ) {
	return TCL_ERROR;
    }

    /* 
     *  ...  daraus die Liste aller eingehenden Kanten.
     */

    in_arcs = place_start->in;

    /* 
     *  Wenn eine eingehende Kante vom Endknoten 
     *  kommt handelt es sich bei der neuen Kante
     *  um eine Doppelkante und deren Canvas-id wird 
     *  zurueckgegeben. 
     */

    while( in_arcs != NULL ) {
	if( in_arcs->other_vertex->v_id != end ) {
	    in_arcs = in_arcs->next;
	} else {
	    return in_arcs->edge_object->e_id;
	}
    }
    return 0;
}

