###########################################################################
###########################################################################
##                                                                       ##
##  Datei   :     `pfaModes.tcl'                             		 ##
##                -------------                              		 ##
##                                                           		 ##
##  Inhalt  :     Prozeduren, die die Funktionalit"at        		 ##
##                des `Modes'-Untermen"us gew"ahrleisten.    		 ##
##                                                           		 ##
###########################################################################
###########################################################################


###########################################################################
##									 ##
##  Prozeduren zum:							 ##
##									 ##
##  Startvertex - Mode							 ##
##									 ##
###########################################################################


##  StartVertexMode
##-----------------------------------------------------------
##  Einstellen des Modus zum Setzen von Startknoten mit
##  Hilfe der rechten Maustaste.
##-----------------------------------------------------------
##
proc StartVertexMode {} {
    global editor opt histList

    ##  Aufruf wird erst ermoeglicht, wenn
    ##  mindestens ein Editor aktiviert wurde
    ##
    while { [llength $histList] == 0 } {
	if { [InitialQuestion] == "end" }  {
	    set opt(mouse3) Free
	    return
	}
    }

    ##  Alte Bindings der rechten Maustaste loeschen
    ##
    DeleteBindings

    ##  Information "uber Modus im Info-Fenster
    ##  rechts unten
    ##
    set opt(mouse3) SVertex

    ##  Binding f"ur rechte Maustaste wird gesetzt
    ##
    bind $editor(canvas) <3> "XYSet %x %y; StartVertex"
}


##  StartVertex
##-----------------------------------------------------------
##  wird im StartVertexMode aufgerufen, wenn man die rechte
##  Maustaste  dr"uckt. Es wird daraufhin ein Start-
##  knoten kreiert
##-----------------------------------------------------------
##
proc StartVertex {} {
    global opt pos

    ##  Startknoten auf der Canvas zeichnen und
    ##  die Id zur"uckholen
    ##
    set item [MkStartVertex $pos(x) $pos(y)]

    ##  Startknotentext auf der Canvas zeichnen und
    ##  die Id zur"uckholen
    ##
    set textitem [MkStartText $item $pos(x) $pos(y)]

    ##  Infos "uber den Startknoten an die Struktur
    ##  in C "ubergeben
    ##
    pfa_VertexInfos $item $pos(x) $pos(y) $textitem 1

    ##  Wenn die Label-Option gesetzt ist, wird
    ##  das Texteingabefenster aufgerufen
    ##
    if { $opt(label) == "on" } {
	set placelabel [ EntryWindow $textitem]
    } else {
	set placelabel ""
    }

    ##  Der Labeltext wird an die Struktur
    ##  in C "ubergeben
    ##
    pfa_Rename $item $placelabel

    ##  Es haben Aenderungen auf der Canvas
    ##  stattgefunden
    ##
    SomethingHasChanged FA highlight
}

##  MkStartVertex
##-----------------------------------------------------------
##  Startknoten wird auf der Canvas kreiert und gibt die
##  Canvas-Id zur"uck
##-----------------------------------------------------------
##
proc MkStartVertex {x y } {
    global editor opt

    set item [$editor(canvas) create oval \
		  [expr $x - $opt(vertexRad)]\
		  [expr $y - $opt(vertexRad)]\
		  [expr $x + $opt(vertexRad)]\
		  [expr $y + $opt(vertexRad)]\
		  -outline black -fill white -width 3]
    $editor(canvas) addtag startknoten$item withtag $item
    $editor(canvas) addtag startknoten withtag $item
    $editor(canvas) addtag knotenBase withtag $item
    return $item
}


##  MkStartText
##-----------------------------------------------------------
## Startknotentext wird auf der Canvas kreiert und gibt die
##  Canvas-Id zur"uck
##-----------------------------------------------------------
##
proc MkStartText { item x y {text_label ""} } {
    global editor faNumber

    set textitem  [$editor(canvas) create text $x $y \
		       -text $text_label -fill black \
		       -font $editor($faNumber(actual),vertexFont) ]

    $editor(canvas) addtag startknoten$item withtag $textitem
    $editor(canvas) addtag startknoten withtag $textitem
    $editor(canvas) addtag startknotentext$item withtag $textitem
    $editor(canvas) addtag startknotentext withtag $textitem

    return $textitem
}


###########################################################################
##									 ##
##  Prozeduren zum:							 ##
##									 ##
##  Vertex - Mode							 ##
##									 ##
###########################################################################


##  NormalVertexMode
##-----------------------------------------------------------
##  Einstellen des Modus zum Setzen von inneren Knoten mit
##  Hilfe der rechten Maustaste.
##-----------------------------------------------------------
##
proc NormalVertexMode {} {
    global editor opt histList

    while { [llength $histList] == 0 } {
	if { [InitialQuestion] == "end" }  {
	    set opt(mouse3) Free
	    return
	}
    }
    DeleteBindings
    set opt(mouse3) Vertex
    bind $editor(canvas) <3> "XYSet %x %y; Vertex"
}


##  Vertex
##-----------------------------------------------------------
##  wird im VertexMode aufgerufen, wenn man die rechte
##  Maustaste  dr"uckt. Es wird daraufhin ein innerer
##  Knoten kreiert
##-----------------------------------------------------------
##
proc Vertex {} {
    global  pos opt

    set item [MkVertex $pos(x) $pos(y)]
    set textitem [ MkText $item $pos(x) $pos(y) ]
    pfa_VertexInfos $item $pos(x) $pos(y) $textitem 2
    if { $opt(label) == "on" } {
	set placelabel [ EntryWindow $textitem]
    } else {
	set placelabel ""
    }
    pfa_Rename $item $placelabel
    SomethingHasChanged FA highlight
}


##  MkVertex
##-----------------------------------------------------------
##  Innerer Knoten wird auf der Canvas kreiert und gibt die
##  Canvas-Id zur"uck
##-----------------------------------------------------------
##
proc MkVertex {xWert yWert } {
    global editor opt

    set item [$editor(canvas) create oval\
		  [expr $xWert - $opt(vertexRad)]\
		  [expr $yWert - $opt(vertexRad)]\
		  [expr $xWert + $opt(vertexRad)]\
		  [expr $yWert + $opt(vertexRad)]\
		  -outline black -fill white]
    $editor(canvas) addtag knoten$item withtag $item
    $editor(canvas) addtag knoten withtag $item
    $editor(canvas) addtag knotenBase withtag $item
    return $item
}


##  MkText
##-----------------------------------------------------------
##  Knotentext wird auf der Canvas kreiert und gibt die
##  Canvas-Id zur"uck
##-----------------------------------------------------------
##
proc MkText { item x y {text_label ""} } {
    global editor faNumber

    return  [$editor(canvas) create text $x $y -text $text_label\
		 -fill black\
		 -tags "knoten$item knoten knotentext$item knotentext"\
		 -font $editor($faNumber(actual),vertexFont)]
}


###########################################################################
##									 ##
##  Prozeduren zum:							 ##
##									 ##
##  Finalvertex - Mode							 ##
##									 ##
###########################################################################


##  FinalVertexMode
##-----------------------------------------------------------
##  Einstellen des Modus zum Setzen von Endknoten mit
##  Hilfe der rechten Maustaste.
##-----------------------------------------------------------
##
proc FinalVertexMode {} {
    global editor opt histList

    while { [llength $histList] == 0 } {
	if { [InitialQuestion] == "end" }  {
	    set opt(mouse3) Free
	    return
	}
    }
    DeleteBindings
    set opt(mouse3) FVertex
    bind $editor(canvas) <3> "XYSet %x %y; FinalVertex"
}


##  FinalVertex
##-----------------------------------------------------------
##  wird im FinalVertexMode aufgerufen, wenn man die rechte
##  Maustaste  dr"uckt. Es wird daraufhin ein End-
##  knoten kreiert
##-----------------------------------------------------------
##
proc FinalVertex {} {
    global opt pos

    set item [MkFinalVertex $pos(x) $pos(y)]
    set textitem [MkFinalText $item $pos(x) $pos(y)]
    pfa_VertexInfos $item $pos(x) $pos(y) $textitem 3
    if { $opt(label) == "on" } {
	set placelabel [ EntryWindow $textitem]
    } else {
	set placelabel ""
    }
    pfa_Rename $item $placelabel
    SomethingHasChanged FA highlight
}

##  MkFinalVertex
##-----------------------------------------------------------
##  Endknoten wird auf der Canvas kreiert und gibt die
##  Canvas-Id zur"uck
##-----------------------------------------------------------
##
proc MkFinalVertex {x y} {
    global editor opt

    set item [$editor(canvas) create finalvertex\
		  [expr $x - $opt(vertexRad)]\
		  [expr $y - $opt(vertexRad)]\
		  [expr $x + $opt(vertexRad)]\
		  [expr $y + $opt(vertexRad)]\
		  -outline black -fill white]

    $editor(canvas) addtag endknoten$item withtag $item
    $editor(canvas) addtag endknoten withtag $item
    $editor(canvas) addtag knotenBase withtag $item

    return $item
}

##  MkFinalText
##-----------------------------------------------------------
##  Endknotentext wird auf der Canvas kreiert und gibt die
##  Canvas-Id zur"uck
##-----------------------------------------------------------
##
proc MkFinalText { item x y {text_label ""} } {
    global  editor faNumber

    set textitem [$editor(canvas) create text $x $y \
		      -text $text_label\
		      -fill black\
		 -font $editor($faNumber(actual),vertexFont) ]

    $editor(canvas) addtag endknoten$item withtag $textitem
    $editor(canvas) addtag endknoten withtag $textitem
    $editor(canvas) addtag endknotentext$item withtag $textitem
    $editor(canvas) addtag endknotentext withtag $textitem

    return $textitem
}


#############################################################
##
##  Prozeduren zum:
##
##  Edge-Mode
##
#############################################################


##  EdgeMode
##-----------------------------------------------------------
##  Einstellen des Modus zum Setzen von Kanten mit
##  Hilfe der rechten Maustaste.
##-----------------------------------------------------------
##
proc EdgeMode {} {
    global editor opt histList

    while { [llength $histList] == 0 } {
	if { [InitialQuestion] == "end" }  {
	    set opt(mouse3) Free
	    return
	}
    }
    DeleteBindings
    set opt(mouse3) Edge
    $editor(canvas) bind knoten <3> " XYSet %x %y; EdgeStart"
    $editor(canvas) bind startknoten <3> " XYSet %x %y; EdgeStart"
    $editor(canvas) bind endknoten <3> " XYSet %x %y; EdgeStart "
    $editor(canvas) bind knoten <B3-Motion> " XYSet %x %y; EdgeMotion "
    $editor(canvas) bind startknoten <B3-Motion> \
	" XYSet %x %y; EdgeMotion "
    $editor(canvas) bind endknoten <B3-Motion> 	" XYSet %x %y; EdgeMotion "
    $editor(canvas) bind knoten <ButtonRelease-3> " XYSet %x %y; EdgePut "
    $editor(canvas) bind startknoten <ButtonRelease-3> \
	" XYSet %x %y; EdgePut "
    $editor(canvas) bind endknoten <ButtonRelease-3> \
	" XYSet %x %y; EdgePut "
}


##  EdgeStart
##-----------------------------------------------------------
##  Wenn man im Ausgangsknoten die rechte Maustaste dr"uckt
##  wird mit dem Zeichnen der Kante begonnen
##-----------------------------------------------------------
##
proc EdgeStart {} {
    global editor pos

    set start [ItemBase [$editor(canvas) find withtag current]]
    $editor(canvas) addtag kantestart withtag $start
    eval "$editor(canvas) create line [ItemCenter $start] $pos(x) $pos(y)\
	      -arrow last -tag kantekurz"
}


##  EdgeMotion
##-----------------------------------------------------------
##  Die Kante wird beim Verschieben des Mauszeigers bei
##  gedr"uckter Maustaste auf der Canvas staendig aktualisiert
##-----------------------------------------------------------
##
proc EdgeMotion {} {
    global editor pos

    if {[set kantekurz [$editor(canvas) find withtag kantekurz]] != ""} {
	eval "$editor(canvas) coords $kantekurz \
                    [lrange [$editor(canvas) coords $kantekurz] 0 1] \
                    $pos(x) $pos(y)"
    }
}

##  EdgePut
##-----------------------------------------------------------
##  Die Maustaste muss losgelassen werden, wenn sich der Maus-
##  zeiger "uber dem Zielknoten befindet. Dann wird mit
##  NewEdge eine ne"u Kante vom Ausgangs- zum Zielknoten
##  gezeichnet. Wenn die Maustaste auf der freien Zeichen-
##  flaeche losgelassen wird, wird die vorher gezogene
##  Teilkante wieder geloescht.
##-----------------------------------------------------------
##
proc EdgePut {} {
    global editor pos opt

    set kantekurz [$editor(canvas) find withtag kantekurz]
    set start [$editor(canvas) find withtag kantestart]
    set zielort [lrange [$editor(canvas) coords $kantekurz] 2 3]
    $editor(canvas) delete $kantekurz
    $editor(canvas) dtag kantekurz
    $editor(canvas) dtag kantestart
    set zielort1 [lindex $zielort 0]
    set zielort2 [lindex $zielort 1]
    set itemlist [$editor(canvas) find overlapping \
                      $zielort1 $zielort2 $zielort1 $zielort2]
    if {$itemlist == ""} {
    } else {
	foreach item $itemlist {
	    if {([lsearch [$editor(canvas) gettags $item] kante] == -1) 
	     && ([lsearch [$editor(canvas) gettags $item]  area] == -1) } {
		set ziel [ItemBase $item]
		NewEdge $start $ziel
	    }
	}
    }
}

##  NewEdge
##-----------------------------------------------------------
##  Wenn der Ausgangs- und Zielknoten verschieden sind wird
##  eine normale Kante gezeichnet, sonst eine Schlinge.
##-----------------------------------------------------------
##
proc NewEdge { start ziel } {
    global opt editor


    ##  Es haben Aenderungen auf der Canvas
    ##  stattgefunden
    ##
    SomethingHasChanged FA highlight
    set ready 0
    if { $start != $ziel } {

	##  Es wird eine normale Kante auf der Canvas
	##  zunaechst unsichtbar gesetzt, um die
	##  Canvas-Id zu erhalten
	##
	set item [MkEdge 0 0 0 0 0 0 $start $ziel]

	##  Das gleiche mit dem Kantentext
	##
	set textitem [MkEdgeText $item 0 0 "" $start $ziel]

	##  Die gesamten Kanteninfos werden in C
	##  in die Struktur eingetragen und dann
	##  wird die Kante von dort aus an die richtige
	##  Stelle gesetzt
	##
	set ready [pfa_NormalEdgeInfos $start $ziel $item $textitem]
    } else {

	##  Es wird eine Schlinge auf der Canvas
	##  zunaechst unsichtbar gesetzt, um die
	##  Canvas-Id zu erhalten
	##
	set item [MkOwnEdge 0 0 $start $ziel]

	##  Das gleiche mit dem Kantentext
	##
	set textitem [MkEdgeText $item 0 0 "" $start $ziel]

	##  Die gesamten Schlingeninfos werden in C
	##  in die Struktur eingetragen und dann
	##  wird die Schlinge von dort aus an die richtige
	##  Stelle gesetzt
	##
	set ready [pfa_OwnEdgeInfos $start $item $textitem ]
    }


    ##  Wenn die Label-Option gesetzt ist wird das
    ##  Eingabefenster aufgerufen.
    ##
    if { ($opt(label) == "on") || ($opt(label) == "edges")  } {
	set arclabel [ EntryWindow $textitem]
    } else {
	set arclabel ""
    }

    ##  Der neue Text wird in
    ##  die C-Struktur eingetragen
    ##
    pfa_Rename $item $arclabel
    tkwait variable $ready
}

##  MkEdge
##-----------------------------------------------------------
##  Zeichnet eine normale Kante als Spline von start nach
##  ziel und gibt die Canvas-Id f"ur diese Kante zur"uck.
##-----------------------------------------------------------
##
proc MkEdge {x1 y1 xm ym x2 y2 start ziel}  {
    global editor

    set Kante [$editor(canvas) create line \
		   $x1 $y1 \
		   $xm $ym \
		   $x2 $y2 \
		   -smooth true -splinesteps 20 \
		   -arrow last \
		   -width 1]

    $editor(canvas) addtag kante$Kante withtag $Kante
    $editor(canvas) addtag kante withtag $Kante
    $editor(canvas) addtag kantevon$start withtag $Kante
    $editor(canvas) addtag kantenach$ziel withtag $Kante
    $editor(canvas) lower $Kante
    $editor(canvas) lower area
    return $Kante
}

##  MkOwnEdge
##-----------------------------------------------------------
##  Zeichnet eine Schlinge als Spline und gibt die Canvas-Id
##  zur"uck.
##-----------------------------------------------------------
##
proc MkOwnEdge { startx starty start ziel }  {
    global editor

    set Kante [$editor(canvas) create line \
		   0 0\
		   0 0\
		   0 0\
		   0 0\
		   0 0\
		   0 0\
		   0 0\
		   -smooth 1 \
		   -splinesteps 20 \
		   -width 1\
		   -arrow first]
    $editor(canvas) addtag kante$Kante withtag $Kante
    $editor(canvas) addtag kante withtag $Kante
    $editor(canvas) addtag kantevon$start withtag $Kante
    $editor(canvas) addtag kantenach$ziel withtag $Kante
    $editor(canvas) lower $Kante
    $editor(canvas) lower area
    return $Kante
}

##  MkEdgeText
##-----------------------------------------------------------
##  Zeichnet den Kantentext an der Stelle (x,y) mit dem Text
##  $textlabel und gibt die zugehoerige Canvas-Id zur"uck
##-----------------------------------------------------------
##
proc MkEdgeText { Kante x y {textlabel ""} start ziel} {
    global editor faNumber

    set textitem [$editor(canvas) create text $x $y \
		      -text $textlabel \
		      -fill black\
		      -font $editor($faNumber(actual),edgeFont) ]

    $editor(canvas) addtag kante$Kante withtag $textitem
    $editor(canvas) addtag kante withtag $textitem
    $editor(canvas) addtag kantetext$Kante withtag $textitem
    $editor(canvas) addtag kantetext withtag $textitem
    $editor(canvas) addtag kantetextvon$start withtag $textitem
    $editor(canvas) addtag kantetextnach$ziel withtag $textitem

    return $textitem
}


###########################################################################
##									 ##
##  Prozeduren zum:							 ##
##									 ##
##  Comment-Mode							 ##
##									 ##
###########################################################################

##  CommentMode
##-----------------------------------------------------------
##  Einstellen des Modus zum Setzen von Kommentaren
##  mit Hilfe der rechten Maustaste.
##-----------------------------------------------------------
##
proc CommentMode {} {
    global editor opt histList


    while { [llength $histList] == 0 } {
	if { [InitialQuestion] == "end" }  {
	    set opt(mouse3) Free
	    return
	}
    }
    DeleteBindings
    set opt(mouse3) Comment
    bind $editor(canvas) <3> "XYSet %x %y; Comment"
}



##  Comment
##-----------------------------------------------------------
##  wird im CommentMode aufgerufen, wenn die rechte
##  Maustaste auf der Zeichenflaeche gedr"uck wird. Es
##  wird daraufhin das Texteingabefenster geoeffnet
##-----------------------------------------------------------
##
proc Comment {} {
    global pos

    set text [ EntryWindow ""]
    set comment [MkComment $pos(x) $pos(y) $text Courier Medium R 140]
    pfa_CommentInfos $comment $pos(x) $pos(y) $text Courier Medium R 140

    SomethingHasChanged FA ""
}

##  MkComment
##-----------------------------------------------------------
##  Ein Textitem mit dem Text des Kommentars wird an
##  der Stelle (x,y) plaziert und die Canvas-Id wird
##  zur"uckgegeben.
##-----------------------------------------------------------
##
proc MkComment { x y {text_label ""} family weight slant size} {
    global editor

    return  [$editor(canvas) create text $x $y\
		 -text $text_label\
		 -fill black \
		 -tags "comment"\
		 -font -*-$family-$weight-$slant-Normal--*-$size-*]
}



###########################################################################
##									 ##
##  Prozeduren zum:							 ##
##									 ##
##  Area-Mode								 ##
##									 ##
###########################################################################


##  AreaMode
##-----------------------------------------------------------
##  Einstellen des Modus zum Markieren von Bereichen mit
##  Hilfe der rechten Maustaste.
##-----------------------------------------------------------
##
proc AreaMode {} {
    global editor opt histList

    while { [llength $histList] == 0 } {
	if { [InitialQuestion] == "end" }  {
	    set opt(mouse3) Free
	    return
	}
    }
    DeleteBindings
    set opt(mouse3) Area
    bind $editor(canvas) <3> "AreaStart %x %y"
    bind $editor(canvas) <B3-Motion> {XYSet %x %y; AreaMotion %x %y}
    bind $editor(canvas) <ButtonRelease-3> "AreaPut"
}


##  AreaStart
##-----------------------------------------------------------
##  Der Startpunkt des zu markierenden Bereiches wird
##  "ubergeben und es werden Vorbereitungen getroffen,
##  um die Area darzustellen
##-----------------------------------------------------------
##
proc AreaStart { x y } {
    global editor areaCoords view opt

    $editor(canvas) delete area
    set areaCoords(X1) [$editor(canvas) canvasx $x]
    set areaCoords(Y1) [$editor(canvas) canvasy $y]
    $editor(canvas) dtag inarea
    $editor(canvas) dtag nodesInArea
    $editor(canvas) dtag commentsInArea
    set view(right) \
	[expr int($opt(canScroll) * [lindex [$editor(canvas) xview] 1])]
    set view(left) \
	[expr int($opt(canScroll) * [lindex [$editor(canvas) xview] 0])]
    set view(top) \
	[expr int($opt(canScroll) * [lindex [$editor(canvas) yview] 0])]
    set view(bottom) \
	[expr int($opt(canScroll) * [lindex [$editor(canvas) yview] 1])]
    set view(xr) [lindex [$editor(canvas) xview] 0]
    set view(yb) [lindex [$editor(canvas) yview] 0]
}

##  AreaMotion
##-----------------------------------------------------------
##  Die Area wird durch Veraendern der Maus bei gedr"uckter
##  rechter Maustaste als Rechteck dargestellt. Falls man
##  dabei den sichtbaren Bereich der Canvas "uberschreitet
##  wird automatisch mitgescrollt.
##-----------------------------------------------------------
##
proc AreaMotion {x y} {
    global editor areaCoords pos view opt

    set x [$editor(canvas) canvasx $x]
    set y [$editor(canvas) canvasy $y]
    if {($areaCoords(X1) != $x) && ($areaCoords(Y1) != $y)} {
        $editor(canvas) delete area
        $editor(canvas) addtag area withtag [$editor(canvas) create rect \
            $areaCoords(X1) $areaCoords(Y1) $x $y -outline black]
        set areaCoords(X2) $x
        set areaCoords(Y2) $y
    }

    if { $pos(x) >= $view(right) } {
	set view(xr) \
	    [expr $view(xr) + ($pos(x) - $view(right)) / $opt(canScroll)]
	set view(right) $pos(x)
	if { ($view(xr) > 0) && ($view(right) < $opt(canScroll)) } {
	    if {$opt(ruler) == "off"} {
		$editor(canvas) xview moveto $view(xr)
	    } else {
		$editor(canvas) xview moveto $view(xr)
		.fr1a.fr2ru.top.topscale_canv xview moveto $view(xr)
	    }
	}
    } elseif { $pos(x) <= $view(left) } {
	set view(xr) \
	    [expr $view(xr) - ($view(left) - $pos(x)) / $opt(canScroll)]
	set view(left) $pos(x)
	if { ($view(xr) >= 0) && ($view(left) > 0) } {
	    if {$opt(ruler) == "off"} {
		$editor(canvas) xview moveto $view(xr)
	    } else {
		$editor(canvas) xview moveto $view(xr)
		.fr1a.fr2ru.top.topscale_canv xview moveto $view(xr)
	    }
	}
    } elseif { $pos(y) >= $view(bottom) } {
	set view(yb) \
	    [expr $view(yb) + ($pos(y) - $view(bottom)) / $opt(canScroll)]
	set view(bottom) $pos(y)
	if { ($view(yb) > 0) && ($view(bottom) < $opt(canScroll)) } {
	    if {$opt(ruler) == "off"} {
		$editor(canvas) yview moveto $view(yb)
	    } else  {
		$editor(canvas) yview moveto $view(yb)
		.fr1a.fr2ru.left.leftscale_canv yview moveto $view(yb)
	    }
	}
    } elseif { $pos(y) <= $view(top) } {
	set view(yb) \
	    [expr $view(yb) - ($view(top) - $pos(y)) / $opt(canScroll)]
	set view(top) $pos(y)
	if { ($view(yb) >= 0) && ($view(top) > 0) } {
	    if {$opt(ruler) == "off"} {
		$editor(canvas) yview moveto $view(yb)
	    } else  {
		$editor(canvas) yview moveto $view(yb)
		.fr1a.fr2ru.left.leftscale_canv yview moveto $view(yb)
	    }
	}
    }
}

##  AreaPut
##-----------------------------------------------------------
##  Beim Loslassen der rechten Maustaste wird die Area durch
##  Veraendern der Farbe gekennzeichnet und alle in ihr
##  enthaltenen Elemente werden sich gemerkt.
##-----------------------------------------------------------
##
proc AreaPut {} {
    global editor areaCoords box

    $editor(canvas) addtag inarea enclosed \
	$areaCoords(X1) $areaCoords(Y1) $areaCoords(X2) $areaCoords(Y2)
    foreach item [$editor(canvas) find withtag inarea] {
	if { ([$editor(canvas) type $item] == "oval") ||\
		 ([$editor(canvas) type $item] == "finalvertex")  } {
            $editor(canvas) addtag nodesInArea withtag  $item
	} elseif { [lindex [$editor(canvas) gettags $item] 0] \
		       == "comment"} {
            $editor(canvas) addtag commentsInArea withtag $item
	} else {
	    $editor(canvas) dtag $item inarea
	}
    }
    if { [llength [$editor(canvas) find withtag inarea]] > 0 } {
	set box [$editor(canvas) bbox inarea]
	$editor(canvas) itemconfigure area \
	-fill DarkTurquoise -stipple gray25
	$editor(canvas) coords area [lindex $box 0] [lindex $box 1]\
	    [lindex $box 2] [lindex $box 3]
	$editor(canvas) lower area
    } else {
	$editor(canvas) delete area
    }
    bind $editor(canvas) <Control-c> "Copy"
    .fr1a.fr2m.mb2.m\
	entryconfigure "Copy"\
	-state normal
    bind $editor(canvas) <Control-x> "Cut"
    .fr1a.fr2m.mb2.m\
	entryconfigure "Cut"\
	-state normal
}



###########################################################################
##									 ##
##  Prozeduren zum:							 ##
##									 ##
##  Delete-Mode								 ##
##									 ##
###########################################################################

##  DeleteMode
##-----------------------------------------------------------
##  Einstellen des Modus zum Loeschen von Elementen mit
##  Hilfe der rechten Maustaste.
##-----------------------------------------------------------
##
proc DeleteMode {} {
    global editor opt histList

    while { [llength $histList] == 0 } {
	if { [InitialQuestion] == "end" }  {
	    set opt(mouse3) Free
	    return
	}
    }
    DeleteBindings
    set opt(mouse3) Delete

    bind $editor(canvas) <3> {
	Delete [$editor(canvas) find withtag current] }
    $editor(canvas) configure -cursor pirate
}



##  Delete
##-----------------------------------------------------------
##  Die folgende Prozedur loescht die auf der Zeichenflaeche
##  vorkommenden Elemente jeweils durch den Aufruf einer
##  C-Funktion.
##-----------------------------------------------------------
##
proc Delete { item } {
    global editor opt

    if { ([$editor(canvas) type [ItemBase $item]] == "oval") ||\
	 ([$editor(canvas) type [ItemBase $item]] == "finalvertex")  } {

	pfa_VertexDelete [ItemBase $item]
	SomethingHasChanged FA highlight

    } elseif {[lindex [$editor(canvas) gettags $item] 1]  == "kante"} {

	pfa_EdgeDelete [ItemBase $item]
	SomethingHasChanged FA highlight
	if {($opt(highlight) == "on") && ($opt(bpn2Editor) == "on")} {

	    PFAdel_multi_sel
	}

    } elseif { [lindex [$editor(canvas) gettags $item] 0]  == "comment"} {

	pfa_CommentDelete $item
	SomethingHasChanged FA ""

    } elseif { [lindex [$editor(canvas) gettags $item] 0]  == "area"} {

	pfa_AreaDelete [$editor(canvas) find withtag nodesInArea] \
	    [$editor(canvas) find withtag commentsInArea]
	SomethingHasChanged FA highlight
	.fr1a.fr2m.mb2.m\
	    entryconfigure "Cut"\
	    -state disabled
	.fr1a.fr2m.mb2.m\
	    entryconfigure "Copy"\
	    -state disabled
    }

    if { [$editor(canvas) find withtag area] != "" } {

	$editor(canvas) delete area
    }
}


###########################################################################
##									 ##
##  Prozeduren zum:							 ##
##									 ##
##  Line-Up - Mode							 ##
##									 ##
###########################################################################

##  LineUpMode
##-----------------------------------------------------------
##  Einstellen des Modus zum Gleichsetzen von x- bzw. y-
##  Position mit Hilfe der mittleren und rechten Maustaste.
##-----------------------------------------------------------
##
proc LineUpMode {} {
    global editor opt histList

    while { [llength $histList] == 0 } {
	if { [InitialQuestion] == "end" }  {
	    set opt(mouse3) Free
	    return
	}
    }
    DeleteBindings
    set opt(mouse2) "1st Tag"
    set opt(mouse3) "2nd Tag"
    $editor(canvas) bind knoten <2> { FirstTag [ItemBase current] }
    $editor(canvas) bind startknoten <2> { FirstTag [ItemBase current] }
    $editor(canvas) bind endknoten <2> { FirstTag [ItemBase current] }
    $editor(canvas) bind knoten <3> { SecondTags [ItemBase current] }
    $editor(canvas) bind startknoten <3> { SecondTags [ItemBase current] }
    $editor(canvas) bind endknoten <3> { SecondTags [ItemBase current] }
}



##  FirstTag
##-----------------------------------------------------------
##  Mit der mittleren Maustaste wird der Knoten markiert,
##  an dem die anderen ausgerichtet werden sollen. Bei
##  nochmaliger Betaetigung der Maustaste wird die Markierung
##  wieder geloescht.
##-----------------------------------------------------------
##
proc FirstTag { vertex } {
   global editor opt

   set itembase $vertex
   if {[lsearch [$editor(canvas) gettag $itembase] doubleselected] == -1} {
	$editor(canvas) itemconfigure doubleselected -fill white
	$editor(canvas) dtag doubleselected
	$editor(canvas) dtag $itembase selected
	$editor(canvas) itemconfigure $itembase -fill brown
	$editor(canvas) addtag doubleselected withtag $itembase
	set opt(itemColor) brown

	bind $editor(canvas) <x> "LineUpX"
	bind $editor(canvas) <y> "LineUpY"
	.fr1a.fr2m.mb2.m.modes.lineup\
	    entryconfigure "Line-Up-X  "\
	    -state active
	.fr1a.fr2m.mb2.m.modes.lineup\
	    entryconfigure "Line-Up-Y  "\
	    -state active
    } else {
	$editor(canvas) dtag $itembase doubleselected
	$editor(canvas) itemconfigure $itembase -fill white
	set opt(itemColor) white
    }
}

##  SecondTags
##-----------------------------------------------------------
##  Mit der rechten Maustaste werden die Knoten markiert,
##  die ausgerichtet werden sollen. Bei nochmaliger
##  Betaetigung der Maustaste kann  auch deren Markierung
##  wieder geloescht werden.
##-----------------------------------------------------------
##
proc SecondTags { vertex } {
   global editor opt

    set itembase $vertex
    if {[lsearch [$editor(canvas) gettag $itembase] selected] == -1} {
	$editor(canvas) dtag $itembase doubleselected
	$editor(canvas) itemconfigure $itembase -fill tan
	$editor(canvas) addtag selected withtag $itembase
	set opt(itemColor) tan

    } else {
	$editor(canvas) dtag $itembase selected
	$editor(canvas) itemconfigure $itembase -fill white
	set opt(itemColor) white
    }
}

##  LineUpX
##-----------------------------------------------------------
##  Die x-Position der auszurichtenden Knoten wird in
##  einer C-Funktion angepasst.
##-----------------------------------------------------------
##
proc LineUpX {} {
    global editor opt

    if {[llength [$editor(canvas) find withtag doubleselected]] == 1} {

	pfa_LineUpX [$editor(canvas) find withtag doubleselected]\
	    [$editor(canvas) find withtag selected]
	SomethingHasChanged FA ""
    }
    set opt(itemColor) white
    $editor(canvas) itemconfigure selected -fill white
    $editor(canvas) itemconfigure doubleselected -fill white
    $editor(canvas) dtag selected
    $editor(canvas) dtag doubleselected

    bind $editor(canvas) <x> ""
    bind $editor(canvas) <y> ""
    .fr1a.fr2m.mb2.m.modes.lineup\
	entryconfigure "Line-Up-X  "\
	-state disabled
    .fr1a.fr2m.mb2.m.modes.lineup\
	entryconfigure "Line-Up-Y  "\
	-state disabled
}

##  LineUpY
##-----------------------------------------------------------
##  Die y-Position der auszurichtenden Knoten wird in
##  einer C-Funktion angepasst.
##-----------------------------------------------------------
##
proc LineUpY {} {
    global editor opt

    if {[llength [$editor(canvas) find withtag doubleselected]] == 1} {

	pfa_LineUpY [$editor(canvas) find withtag doubleselected]\
	    [$editor(canvas) find withtag selected]
	SomethingHasChanged FA ""
    }
    set opt(itemColor) white
    $editor(canvas) itemconfigure selected -fill white
    $editor(canvas) itemconfigure doubleselected -fill white
    $editor(canvas) dtag selected
    $editor(canvas) dtag doubleselected

    bind $editor(canvas) <x> ""
    bind $editor(canvas) <y> ""
    .fr1a.fr2m.mb2.m.modes.lineup\
	entryconfigure "Line-Up-X  "\
	-state disabled
    .fr1a.fr2m.mb2.m.modes.lineup\
	entryconfigure "Line-Up-Y  "\
	-state disabled
}



###########################################################################
##									 ##
##  Prozeduren zum:							 ##
##									 ##
##  Move-Text -Mode							 ##
##									 ##
###########################################################################


##  TextMoveMode
##-----------------------------------------------------------
##  Einstellen des Modus zum Versetzen von Kantenttexten
##  mit Hilfe der rechten Maustaste.
##-----------------------------------------------------------
##
proc TextMoveMode {} {
    global editor opt histList

    while { [llength $histList] == 0 } {
	if { [InitialQuestion] == "end" }  {
	    set opt(mouse3) Free
	    return
	}
    }
    DeleteBindings
    set opt(mouse3) "T-Move"
    $editor(canvas) bind kantetext <Button-3>\
	{ EdgeTextGet  }
    $editor(canvas) bind kantetext <B3-Motion>\
	{ XYSet %x %y; EdgeTextMove }
    $editor(canvas) bind kantetext <ButtonRelease-3>\
	{ EdgeTextSet }
}



##  EdgeTextGet
##-----------------------------------------------------------
##  Die Id des Textes wird ermittelt. Ausserdem wird die
##  Position des Textes gespeichert, um ihn nach dem
##  Verschieben mit Hilfe von Unset wieder in die alte
##  Position bringen zu koennen.
##-----------------------------------------------------------
##
proc EdgeTextGet {} {
    global editor elem

    set elem [$editor(canvas) find withtag current]
    $editor(canvas) itemconfigure $elem  -fill white

    pfa_UnsetTextCoords [ItemBase $elem]
}

##  EdgeTextMove
##-----------------------------------------------------------
##  Der Text kann durch Verschieben des Mauszeigers bei
##  gedr"uckter rechter Maustaste veraendert werden. Dies
##  erledigt eine C-Funktion.
##-----------------------------------------------------------
##
proc EdgeTextMove {} {
    global  pos elem

    pfa_ChangeEdgeTextCoords [ItemBase $elem] $pos(x) $pos(y)
}

##  EdgeTextSet
##-----------------------------------------------------------
##  Es hat durch das Verschieben eine Veraenderung auf der
##  Canvas stattgefunden.
##-----------------------------------------------------------
##
proc EdgeTextSet {} {

    SomethingHasChanged FA ""
}


###########################################################################
##									 ##
##  Prozeduren zum:							 ##
##									 ##
##  Rename - Mode							 ##
##									 ##
###########################################################################

##  RenameMode
##-----------------------------------------------------------
##  Einstellen des Modus zum Aendern von Knoten- oder Kanten-
##  label bzw. Kommentaren mit Hilfe der rechten Maustaste.
##-----------------------------------------------------------
##
proc RenameMode {} {
    global editor opt histList

    while { [llength $histList] == 0 } {
	if { [InitialQuestion] == "end" }  {
	    set opt(mouse3) Free
	    return
	}
    }
    DeleteBindings
    set opt(mouse3) Rename
    $editor(canvas) bind startknoten <3> {
	set baseId [ItemBase [$editor(canvas) find withtag current]]
	set textId [ItemText $baseId]
	Rename $baseId $textId
    }
    $editor(canvas) bind knoten <3> {
	set baseId [ItemBase [$editor(canvas) find withtag current]]
	set textId [ItemText $baseId]
	Rename $baseId $textId
    }
    $editor(canvas) bind endknoten <3> {
	set baseId [ItemBase [$editor(canvas) find withtag current]]
	set textId [ItemText $baseId]
	Rename $baseId $textId
    }
    $editor(canvas) bind kante <3> {
	set baseId [ItemBase [$editor(canvas) find withtag current]]
	set textId [ItemText $baseId]
	Rename $baseId $textId
    }
    $editor(canvas) bind comment <3> {
	set baseId [$editor(canvas) find withtag current]
	Rename $baseId $baseId
    }
}



##  Rename
##-----------------------------------------------------------
##  Das Eingabefenster fragt nach dem neuen Text und "ubergibt
##  diesen mit der Id des umzubennenenden Elementes an eine
##  C-Funktion.
##-----------------------------------------------------------
##
proc Rename { baseId textId } {

    set text [ EntryWindow $textId]

    pfa_Rename $baseId $text
    if {[ItemType $baseId] == "kante"} {
	SomethingHasChanged FA highlight
    } else {
	SomethingHasChanged FA ""
    }
}


###########################################################################
##									 ##
##  Prozeduren zum:							 ##
##									 ##
##  Reset-Edge - Mode							 ##
##									 ##
###########################################################################

##  ResetEdgeMode
##-----------------------------------------------------------
##  Einstellen des Modus zum Rueckversetzen von Kanten mit
##  Hilfe der rechten Maustaste.
##-----------------------------------------------------------
##
proc ResetEdgeMode {} {
    global editor opt histList

    while { [llength $histList] == 0 } {
	if { [InitialQuestion] == "end" }  {
	    set opt(mouse3) Free
	    return
	}
    }
    DeleteBindings
    set opt(mouse3) Reset
    $editor(canvas) bind kante <3> {
	ResetEdge [$editor(canvas) find withtag current]
    }
}



##  ResetEdge
##-----------------------------------------------------------
##  Eine Kante wird auf die Position direkt nach der
##  Erstellung zurueckgesetzt.
##-----------------------------------------------------------
##

proc ResetEdge { id } {

    pfa_ResetEdge [ItemBase $id]
    SomethingHasChanged FA ""
}
