/*
###########################################################################
###########################################################################
##                                                                       ##
##  Datei   :    `pfaMove.c'                                             ##
##               -----------                                             ##
##                                                                       ##
##  Inhalt  :    Funktionen, die zum Verschieben der Elemente auf der    ##
##               Zeichenfl"ache ben"otigt werden.                        ##
##                                                                       ##
###########################################################################
###########################################################################
*/

#include <stdio.h>
#include <string.h>	
#include <stdlib.h>
#include <math.h>
#include "pfa.h"

/*
 ##########################################################################
 #                                                                        #
 #  C-Funktionen zum Verschieben von Knoten                               #
 #                                                                        #
 ##########################################################################
 */


/*  tk_UnsetVertexCoords
 *--------------------------------------------------------------
 *  
 *  speichert die Position des Knotens vor dem Verschieben
 *  in der Unset-Liste, um ihn sp"ater wieder auf die
 *  alte Position zur"ucksetzen zu k"onnen.
 *
 *--------------------------------------------------------------
 */

int tk_UnsetVertexCoords(ClientData cd, Tcl_Interp * interp,\
			int objc, Tcl_Obj *CONST objv[])
{
    UnsetTop_type *partList_object;
    Vertex_type   *vertexObject;
    int num;

    if (objc != 2) {
	Tcl_WrongNumArgs(interp, 1, objv, "nodenum");
	return TCL_ERROR;
    }

    /* 
     *  Die Struktur des Knoten wird geholt, um die aktuelle Position zu
     *  erhalten.
     */

    if (Tcl_GetIntFromObj(interp, objv[1], &num ) != TCL_OK)
      return TCL_ERROR;
    sprintf(interp->result, "knoten%d", num);
    vertexObject = getVertexStruct( interp->result);
    if( vertexObject == NULL ) {
	return TCL_ERROR;
    }

    /* 
     *  Ein Listenelement der Unset-Liste wird erstellt und am Anfang
     *  der Liste eingefuegt. Dann werden die Daten des Knoten in die 
     *  Liste des zuvor initialisierten Listenelementes eingetragen.
     */

    partList_object = makeUnsetElement( move_element );
    if( partList_object == NULL ) {
	return TCL_ERROR;
    }

    insertUnsetStruct(partList_object,\
		      place,\
		      vertexObject->v_id,\
		      vertexObject->center.x,\
		      vertexObject->center.y);
    return TCL_OK;
}

/*  tk_VertexChange
 *--------------------------------------------------------------
 *  
 *  leitet das Verschieben des Knotens ein.
 *
 *--------------------------------------------------------------
 */

int tk_VertexChange(ClientData cd, Tcl_Interp * interp,\
		  int objc, Tcl_Obj *CONST objv[])
{
    Point_type   new_center;
    Vertex_type *vertexObject;
    int num;
    double x;

    if (objc != 4) {
	Tcl_WrongNumArgs(interp, 1, objv, "nodenum x y");
        return TCL_ERROR;
    }

    if (Tcl_GetDoubleFromObj(interp, objv[2], &x) != TCL_OK)
      return TCL_ERROR;
    new_center.x=x;
    if (Tcl_GetDoubleFromObj(interp, objv[3], &x) != TCL_OK)
      return TCL_ERROR;
    new_center.y=x;

    /* 
     *  Die Struktur des Knoten wird geholt ...
     */
    if (Tcl_GetIntFromObj(interp, objv[1], &num ) != TCL_OK)
      return TCL_ERROR;
    sprintf(interp->result, "knoten%d",num);
    vertexObject = getVertexStruct( interp->result);
    if( vertexObject == NULL ) {
	return TCL_ERROR;
    }

    /* 
     *  ... und zusammen mit den neuen Koordinaten an
     *  die eigentliche Verschiebeprozedur "ubergeben.
     */

    if( changeVertexPos( interp, vertexObject, new_center ) != TCL_OK ) {
	return TCL_ERROR;
    }
    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Funktionen zum Verschieben von Kanten                               #
 #                                                                        #
 ##########################################################################
 */


/*  tk_SaveEdgeAngles
 *--------------------------------------------------------------
 *  
 *  speichert den Mittelpunkt der Kante  vor dem Verschieben
 *  in der Unset-Liste, um sie sp"ater wieder auf die
 *  alte Position zur"ucksetzen zu k"onnen.
 *
 *--------------------------------------------------------------
 */
int tk_SaveEdgeAngle( ClientData cd, Tcl_Interp * interp,\
			     int objc, Tcl_Obj *CONST objv[])
{
    Edge_type     *edgeObject;
    UnsetTop_type *partList_object;
    int num;

    if (objc != 2) {
	Tcl_WrongNumArgs(interp, 1, objv, "edgenum");
        return TCL_ERROR;
    }

    /* 
     *  Die Struktur der verschobenen Kante wird geholt.
     */
    if (Tcl_GetIntFromObj(interp, objv[1], &num ) != TCL_OK)
      return TCL_ERROR;
    sprintf(interp->result, "arc%d", num);
    edgeObject = getEdgeStruct( interp->result);
    if( edgeObject == NULL ) {
	return TCL_ERROR;
    }

    /* 
     *  Ein Listenelement der Unset-Liste wird erstellt und am Anfang
     *  der Liste eingefuegt. Dann werden die Daten der Kante in die 
     *  Liste des zuvor initialisierten Listenelementes eingetragen.
     */

    partList_object = makeUnsetElement( move_element );
    if( partList_object == NULL ) {
	return TCL_ERROR;
    }

    insertUnsetStruct(partList_object, edge,\
		      edgeObject->e_id,\
		      edgeObject->center.x,edgeObject->center.y);
    return TCL_OK;
}

/*  tk_EdgeMove
 *--------------------------------------------------------------
 *  
 *  leitet das Verschieben der Kante ein.
 *
 *--------------------------------------------------------------
 */

int tk_EdgeMove( ClientData cd, Tcl_Interp * interp,\
		 int objc, Tcl_Obj *CONST objv[])
{
    Point_type mid;
    int num;
    double x;

    if (objc != 4) {
	Tcl_WrongNumArgs(interp, 1, objv, "edgenum x y");
        return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[1], &num ) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetDoubleFromObj(interp, objv[2], &x ) != TCL_OK)
      return TCL_ERROR;
    mid.x=x;
    if (Tcl_GetDoubleFromObj(interp, objv[3], &x ) != TCL_OK)
      return TCL_ERROR;
    mid.y=x;
    if( edgeMove(interp, num, mid) != TCL_OK ) {
	return TCL_ERROR;
    }
    return TCL_OK;
}

/*  tk_Angle
 *--------------------------------------------------------------
 *  
 *  Die neue Form der Kante wird mit Hilfe der Funktion
 *  calculateAngles gespeichert, damit sie beim
 *  Verschieben der zugehoerigen Knoten beibehalten werden kann.
 *
 *--------------------------------------------------------------
 */

int tk_Angle( ClientData cd, Tcl_Interp * interp,\
		 int objc, Tcl_Obj *CONST objv[])
{
    int num;

    if (objc != 2) {
	Tcl_WrongNumArgs(interp, 1, objv, "edgenum");
        return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[1], &num ) != TCL_OK)
      return TCL_ERROR;
    if( calculateAngles( interp, num) != TCL_OK ) {
	return TCL_ERROR;
    }
    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Funktionen zum Verschieben von Kommentaren                          #
 #                                                                        #
 ##########################################################################
 */


/*  tk_UnsetCommentCoords
 *--------------------------------------------------------------
 *  
 *  speichert die Position des Kommentars  vor dem Verschieben
 *  in der Unset-Liste, um ihn sp"ater wieder auf die
 *  alte Position zur"ucksetzen zu k"onnen.
 *
 *--------------------------------------------------------------
 */

int tk_UnsetCommentCoords(ClientData cd, Tcl_Interp * interp,\
			  int objc, Tcl_Obj *CONST objv[])
{
    Comment_type  *commentObject;
    UnsetTop_type *partList_object;
    int num;

    if (objc != 2) {
	Tcl_WrongNumArgs(interp, 1, objv, "coordsnum");
        return TCL_ERROR;
    }

    /* 
     *  Die Struktur des Kommentars wird geholt, um dessen aktuelle 
     *  Position zu erhalten.
     */

    if (Tcl_GetIntFromObj(interp, objv[1], &num ) != TCL_OK)
      return TCL_ERROR;
    sprintf(interp->result, "comment%d", num);
    commentObject = getCommentStruct( interp->result);
    if( commentObject == NULL ) {
	return TCL_ERROR;
    }

    /* 
     *  Ein Listenelement der Unset-Liste wird erstellt und am Anfang
     *  der Liste eingefuegt. Dann werden die Daten des Kommentars mit der 
     *  in die Liste des zuvor initialisierten Listenelementes eingetragen
     */

    partList_object = makeUnsetElement( move_element );
    if( partList_object == NULL ) {
	return TCL_ERROR;
    }

    insertUnsetStruct(partList_object,\
		      comment,\
		      commentObject->comment_id,\
		      commentObject->center.x,\
		      commentObject->center.y);
    return TCL_OK;
}

/*  tk_CommentChange
 *--------------------------------------------------------------
 *  
 *  leitet das Verschieben des Kommentars ein.
 *
 *--------------------------------------------------------------
 */

int tk_CommentChange(ClientData cd, Tcl_Interp * interp,\
		     int objc, Tcl_Obj *CONST objv[])
{
    Comment_type *commentObject;
    Point_type    center;
    int num;
    double x;

    if (objc != 4) {
	Tcl_WrongNumArgs(interp, 1, objv, "x y commentnum");
        return TCL_ERROR;
    }
    if (Tcl_GetDoubleFromObj(interp, objv[1], &x ) != TCL_OK)
      return TCL_ERROR;
    center.x=x;
    if (Tcl_GetDoubleFromObj(interp, objv[2], &x ) != TCL_OK)
      return TCL_ERROR;
    center.y=x;
    if (Tcl_GetIntFromObj(interp, objv[3], &num ) != TCL_OK)
      return TCL_ERROR;
    sprintf(interp->result, "comment%d", num);
    commentObject = getCommentStruct( interp->result);
    if( commentObject == NULL ) {
	return TCL_ERROR;
    }

    if( changeCommentPos( interp,commentObject, center ) != TCL_OK ) {
	return TCL_ERROR;
    }
    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Funktionen zum Verschieben von markierten Bereichen                 #
 #                                                                        #
 ##########################################################################
 */


/*  tk_UnsetAreaCoords
 *--------------------------------------------------------------
 *  
 *  speichert die Positionen aller Elemente des markierten
 *  Bereiches vor dem Verschieben in der Unset-Liste, um die
 *  Elemente sp"ater wieder zur"ucksetzen zu k"onnen.
 *
 *--------------------------------------------------------------
 */
//TODO: List to Obj!
int tk_UnsetAreaCoords(ClientData cd, Tcl_Interp * interp,\
		       int objc, Tcl_Obj *CONST objv[])
{
    Tcl_Obj       **listElems;
    int             listSum;
    int             i;
    int             cid;
    Comment_type   *commentObject;
    UnsetTop_type  *partList_object;
    Vertex_type    *vertexObject;

    if (objc != 3) {
	Tcl_WrongNumArgs(interp, 1, objv, "nodelist commentlist");
        return TCL_ERROR;
    }

    /* 
     *  Ein Listenelement der Unset-Liste wird erstellt
     *  und am Anfang der Liste eingefuegt
     */

    partList_object = makeUnsetElement( move_element );
    if( partList_object == NULL ) {
	return TCL_ERROR;
    }

    /*
     *   Die als erstes Argument uebergebende Tcl-Liste der zu 
     *   loeschenden Knoten wird in ein Array mit Zeigern auf die 
     *   einzelnen Listenelemente umgewandelt.
     */

    if( Tcl_ListObjGetElements( interp, objv[1], &listSum,\
		       &listElems ) != TCL_OK ) {
	return TCL_ERROR;
    }

    i = 0;
    while( i < listSum ) {
        if (Tcl_GetIntFromObj(interp, listElems[i++], &cid ) != TCL_OK)
          return TCL_ERROR;
	sprintf(interp->result, "knoten%d", cid);
	vertexObject = getVertexStruct( interp->result);
	if( vertexObject == NULL )  {
	    return TCL_ERROR;
	}

	/*
	 *  Die aktuelle Position der Knoten wird der Reihe nach in die 
	 *  Unset-Liste uebertragen
	 */

	insertUnsetStruct(partList_object,\
			  place,\
			  cid,\
			  vertexObject->center.x,\
			  vertexObject->center.y);
    }
    //free( (char *) listElems );

    /*
     *   Die als zweites Argument uebergebende Tcl-Liste der zu loeschenden
     *   Kommentare wird in ein Array mit Zeigern auf die einzelnen 
     *   Listenelemente umgewandelt
     */

    if( Tcl_ListObjGetElements( interp, objv[2], &listSum,\
		       &listElems ) != TCL_OK ) {
	return TCL_ERROR;
    }

    i = 0;
    while( i < listSum ) {
        if (Tcl_GetIntFromObj(interp, listElems[i++], &cid ) != TCL_OK)
          return TCL_ERROR;
	sprintf(interp->result, "comment%d", cid);
	commentObject = getCommentStruct( interp->result);
	if( commentObject == NULL )  {
	    return TCL_ERROR;
	}

	/*
	 *  Die aktuelle Position der Kommentare wird der Reihe nach in 
	 *  die Unset-Liste uebertragen
	 */

	insertUnsetStruct(partList_object,\
			  comment,\
			  commentObject->comment_id,\
			  commentObject->center.x,\
			  commentObject->center.y);
    }
    //free( (char *) listElems );
    return TCL_OK;
}

/*  tk_AreaChange
 *--------------------------------------------------------------
 *  
 *  ist f"ur das Verschieben der Elemente eines markierten
 *  Bereiches zust"andig.
 *
 *--------------------------------------------------------------
 */

int tk_AreaChange(ClientData cd, Tcl_Interp * interp,\
		  int objc, Tcl_Obj *CONST objv[])
{
    Tcl_Obj      **listElems;
    int            cid;
    int            listSum;
    int            i;
    Comment_type  *commentObject;
    Point_type     new_center;
    Point_type     delta;
    Vertex_type   *vertexObject;
    double x;
   
    if (objc != 5) {
	Tcl_WrongNumArgs(interp, 1, objv, "x y nodelist commentlist");
        return TCL_ERROR;
    }

    /* 
     *  Der Wert, um den die einzelnen Elemente verschoben wurden
     */

    if (Tcl_GetDoubleFromObj(interp, objv[1], &x ) != TCL_OK)
      return TCL_ERROR;
    delta.x=x;
    if (Tcl_GetDoubleFromObj(interp, objv[2], &x ) != TCL_OK)
      return TCL_ERROR;
    delta.y=x;

    /*
     *  Die als drittes Argument uebergebende Tcl-Liste der zu 
     *  loeschenden Knoten wird in ein Array mit Zeigern auf die einzelnen
     *   Listenelemente umgewandelt.
     */

    if( Tcl_ListObjGetElements( interp, objv[3], &listSum,\
		       &listElems ) != TCL_OK ) {
	return TCL_ERROR;
    }

    /*
     *  Die einzelnen Knoten der Liste werden mit den Kanten der Reihe
     *  nach verschoben
     */

    i = 0;
    while( i < listSum ) {
        if (Tcl_GetIntFromObj(interp, listElems[i], &cid ) != TCL_OK)
          return TCL_ERROR;
	i++;

	sprintf(interp->result, "knoten%d",cid);
	vertexObject = getVertexStruct( interp->result);
	if( vertexObject == NULL ) {
	    return TCL_ERROR;
	}

	new_center.x = vertexObject->center.x + delta.x;
	new_center.y = vertexObject->center.y + delta.y;

	if( changeVertexPos( interp, vertexObject,\
			     new_center ) != TCL_OK ) {
	    return TCL_ERROR;
	}
    }
    //free( (char *) listElems );

    /*
     *  Die als viertes Argument uebergebende Tcl-Liste der zu loeschenden
     *  Kommentare wird in ein Array mit Zeigern auf die einzelnen 
     *  Listenelemente umgewandelt.
     */

    if( Tcl_ListObjGetElements( interp, objv[4], &listSum,\
		       &listElems ) != TCL_OK ) {
	return TCL_ERROR;
    }

    /*
     *  Die einzelnen Kommentare der Liste werden Reihe nach verschoben
     */

    i = 0;
    while( i < listSum ) {
        if (Tcl_GetIntFromObj(interp, listElems[i], &cid ) != TCL_OK)
          return TCL_ERROR;
	i++;
	sprintf(interp->result, "comment%d",cid);
	commentObject = getCommentStruct( interp->result);
	if( commentObject == NULL ) {
	    return TCL_ERROR;
	}

	new_center.x = commentObject->center.x + delta.x;
	new_center.y = commentObject->center.y + delta.y;

	if( changeCommentPos( interp, commentObject,\
			      new_center ) != TCL_OK ) {
	    return TCL_ERROR;
	}

    }
    //free( (char *) listElems );
    return TCL_OK;
}
