###########################################################################
###########################################################################
##                                                                       ##
##  Datei   :     `pfaMove.tcl'                                          ##
##                -------------                                          ##
##                                                                       ##
##  Inhalt  :     Prozeduren, die f"ur die Positions-                    ##
##                ver"anderung von Knoten, Kanten, Kommen-               ##
##                taren und markierten Bereichen auf der                 ##
##                Canvas verantwortlich sind.                            ##
##                                                                       ##
###########################################################################
###########################################################################



###########################################################################
##                                                                       ##
##  Knoten verschieben                                                   ##
##                                                                       ##
###########################################################################

##  VertexGet
##-----------------------------------------------------------
##  Bei Aktivierung der linken Maustaste innerhalb eines 
##  Knotens wird die Id ermittelt und der Knoten springt
##  mit dem Mittelpunkt an die Mausposition 
##-----------------------------------------------------------
##
proc VertexGet {} {
    global editor pos elem opt

    if { [llength [$editor(canvas) find withtag inarea]] > 0 } {
	$editor(canvas) delete area 
    }
    set elem [ItemBase [$editor(canvas) find withtag current]]
    pfa_UnsetVertexCoords $elem 
    pfa_VertexChange $elem $pos(x) $pos(y)

    SomethingHasChanged FA ""
    $editor(canvas) bind startknoten <Any-Enter> ""
    $editor(canvas) bind startknoten <Any-Leave> ""
    $editor(canvas) bind knoten <Any-Enter> ""
    $editor(canvas) bind knoten <Any-Leave> ""
    $editor(canvas) bind endknoten <Any-Enter> ""
    $editor(canvas) bind endknoten <Any-Leave> ""
}


##  VertexMove
##-----------------------------------------------------------
##  Beim Verschieben der Maus wird der Knoten und die evtl.
##  dazugehoerenden Kanten mit Hilfe der C-Prozedur verschoben 
##-----------------------------------------------------------
##
proc VertexMove {} {
    global elem pos 

    pfa_VertexChange $elem $pos(x) $pos(y)
}


##  VertexSet
##-----------------------------------------------------------
##  Die Bindings zum Ver"andern der Farbe beim Eintritt und
##  Verlassen des Mauszeigers in den Knoten wird wieder 
##  gesetzt.
##-----------------------------------------------------------
##
proc VertexSet {} {
    global editor

    $editor(canvas) bind startknoten <Any-Enter> {
	VertexEnterColor [$editor(canvas) find withtag current]
    }
    $editor(canvas) bind startknoten <Any-Leave> {
	VertexLeaveColor [$editor(canvas) find withtag current]
    }
    $editor(canvas) bind knoten <Any-Enter> {
	VertexEnterColor [$editor(canvas) find withtag current]
    }
    $editor(canvas) bind knoten <Any-Leave> {
	VertexLeaveColor [$editor(canvas) find withtag current]
    }
    $editor(canvas) bind endknoten <Any-Enter> {
	VertexEnterColor [$editor(canvas) find withtag current]
    }
    $editor(canvas) bind endknoten <Any-Leave> {
	VertexLeaveColor [$editor(canvas) find withtag current]
    }
}



#############################################################
##
##  Kanten verschieben
##
#############################################################


##  EdgeGet
##-----------------------------------------------------------
##  Die Id der Kante wird ermittelt. Ausserdem wird die
##  Form der Kante gespeichert, um sie nach dem Verschieben
##  mit Hilfe von Unset wieder in die alte Position bringen
##  zu koennen.
##-----------------------------------------------------------
##
proc EdgeGet {} {
    global elem editor opt
    
    if { [llength [$editor(canvas) find withtag inarea]] > 0 } {
	$editor(canvas) delete area 
    }
    set elem [ItemBase [$editor(canvas) find withtag current]]
    pfa_SaveEdgeAngle $elem

    ##  Das Ver"andern der Kantenfarbe, wenn der Mauszeiger in
    ##  den Bereich der Kante eintritt wird w"ahrend des Verschiebe-
    ##  vorganges unterbunden
    ##
    $editor(canvas) bind kante <Any-Enter> {}
    $editor(canvas) bind kante <Any-Leave> {}
}


##  EdgeMove
##-----------------------------------------------------------
##  Die Kante kann durch Verschieben des Mauszeigers bei
##  gedrueckter erster Maustaste veraendert werden. Dies  
##  erledigt eine C-Funktion.
##-----------------------------------------------------------
##
proc EdgeMove {} {
    global elem pos 

    pfa_EdgeMove $elem $pos(x) $pos(y) 
}


##  EdgeSet
##-----------------------------------------------------------
##  Die neue Form der Kante wird gespeichert, damit sie 
##  beim Verschieben der zugehoerigen Knoten beibehalten
##  werden kann.
##-----------------------------------------------------------
##
proc EdgeSet {} {
    global elem editor

    pfa_Angle $elem 

    ##  Die Bindings zum Ver"andern der Kantenfarbe werden wieder
    ##  gesetzt 
    ##
    $editor(canvas) bind kante <Any-Enter> {
        EdgeEnterColor [$editor(canvas) find withtag current]
        highlight [ItemBase [$editor(canvas) find withtag current]]
    }
    $editor(canvas) bind kante <Any-Leave> {
        EdgeLeaveColor [$editor(canvas) find withtag current]
        highlightend
    }

    SomethingHasChanged FA ""
}



###########################################################################
##                                                                       ##
##  Kommentare verschieben                                               ##
##                                                                       ##
###########################################################################


##  CommentGet
##-----------------------------------------------------------
##  Die Id des Kommentars wird ermittelt. Ausserdem wird die
##  Position des Kommentars gespeichert, um ihn nach dem 
##  Verschieben mit Hilfe von Unset wieder in die alte 
##  Position bringen zu koennen.
##-----------------------------------------------------------
##
proc CommentGet {} {
    global elem editor opt
    
    if { [llength [$editor(canvas) find withtag inarea]] > 0 } {
	$editor(canvas) delete area 
    }
    set elem [$editor(canvas) find withtag current]
    pfa_UnsetCommentCoords $elem
}


##  CommentMove
##-----------------------------------------------------------
##  Der Kommentar kann durch Verschieben des Mauszeigers bei
##  gedrueckter erster Maustaste veraendert werden. Dies  
##  erledigt eine C-Funktion.
##-----------------------------------------------------------
##
proc CommentMove {} {
    global editor pos elem xA yA

    pfa_CommentChange  $pos(x) $pos(y) $elem
}


##  CommentSet
##-----------------------------------------------------------
##  Es hat durch das Verschieben eine Veraenderung auf der
##  Canvas stattgefunden.
##-----------------------------------------------------------
##
proc CommentSet {} {

    SomethingHasChanged FA ""
}


###########################################################################
##                                                                       ##
##  Markierte Bereiche verschieben                                       ##
##                                                                       ##
###########################################################################


##  AreaMoveStart
##-----------------------------------------------------------
##  Die Positionen aller Element in der Area werden 
##  gespeichert, um sie nach dem 
##  Verschieben mit Hilfe von Unset wieder in die alte 
##  Position bringen zu koennen.
##-----------------------------------------------------------
##
proc AreaMoveStart {x y} {
    global editor xA yA 

    set xA [$editor(canvas) canvasx $x]
    set yA [$editor(canvas) canvasy $y]
    pfa_UnsetAreaCoords \
	[$editor(canvas) find withtag nodesInArea]\
	[$editor(canvas) find withtag commentsInArea] 
}


##  AreaMove
##-----------------------------------------------------------
##  Die gesamte Area mit den Elementen wird verschoben.
##  Fuer das Verschieben der Elemente ist eine C-Funktion
##  zustaendig.
##-----------------------------------------------------------
##
proc AreaMove {x y} {
    global editor xA yA 

    set xdiff [expr [$editor(canvas) canvasx $x] - $xA] 
    set ydiff [expr [$editor(canvas) canvasy $y] - $yA]
    $editor(canvas) move area $xdiff $ydiff
    pfa_AreaChange $xdiff $ydiff \
	[$editor(canvas) find withtag nodesInArea]\
	[$editor(canvas) find withtag commentsInArea] 

    set xA [$editor(canvas) canvasx $x]
    set yA [$editor(canvas) canvasy $y]
}


##  AreaMovePut
##-----------------------------------------------------------
##  Nach dem Verschieben wird die Markierung der Area
##  geloescht.
##-----------------------------------------------------------
##
proc AreaMovePut {} {
    global editor 

    $editor(canvas) delete area
    .fr1a.fr2m.mb2.m\
	entryconfigure "Cut"\
	-state disabled
    .fr1a.fr2m.mb2.m\
	entryconfigure "Copy"\
	-state disabled
    SomethingHasChanged FA ""
}














