/*
###########################################################################
###########################################################################
##                                                                       ##
##  Datei   :    `pfaMultList.c'                                         ##
##               ---------------                                         ##
##                                                                       ##
##  Inhalt  :    Funktionen, zur Erstellung universeller Listen und zur  ##
##               Bearbeitung dieser Listen.                              ##
##                                                                       ##
###########################################################################
###########################################################################
*/


#include <stdio.h>
#include <stdlib.h>
#include "pfaMultList.h"



List_type *newList() {
	return((List_type *)malloc(sizeof(List_type)));
}

List_element *newListElement() {
	return((List_element *)malloc(sizeof(List_element)));
}


/*  createList
 *--------------------------------------------------------------
 *   
 *  Eine neue Liste mit dem zugeh"origen Listenkopf wird
 *  erschaffen.
 *
 *--------------------------------------------------------------
 */

List_type *createList(void)
{

	List_type *list = newList();
	list->head = newListElement();
	list->head->next = list->head->prev = list->head;
	return(list);
}


/*  listFirst
 *--------------------------------------------------------------
 *   
 *  Ein neues Listenelement wird erschaffen und mit den
 *  zugeh"origen Daten am Anfang der Liste eingefuegt.
 *
 *--------------------------------------------------------------
 */

void *listFirst(List_type *list, void * data)
{

	List_element *link = newListElement();
	link->data = data;
	(link->next = list->head->next)->prev = link;
	(list->head->next = link)->prev = list->head;
	return data;
}


/*  listLast
 *--------------------------------------------------------------
 *   
 *  Ein neues Listenelement wird erschaffen und mit den
 *  zugeh"origen Daten am Ende der Liste eingefuegt.
 *
 *--------------------------------------------------------------
 */

void *listLast(List_type *list, void * data)
{

	List_element *link = newListElement();
	link->data = data;
	(link->prev = list->head->prev)->next = link;
	(list->head->prev = link)->next = list->head;
	return data;
}


/*  listRemoveFirst
 *--------------------------------------------------------------
 *   
 *  Das erste Element einer Liste wird entfernt und
 *  dessen Daten werden freigegeben.
 *
 *--------------------------------------------------------------
 */

void *listRemoveFirst(List_type *list)
{
	void         *data;
	List_element *link = list->head->next;


	if (link == list->head) {
		return NULL;
	}
	data = link->data;
	(list->head->next = link->next)->prev = list->head;
	free( (void *) link);
	return data;
}


/*  listRemoveFirst
 *--------------------------------------------------------------
 *   
 *  Das letzte Element einer Liste wird entfernt und
 *  dessen Daten werden freigegeben.
 *
 *--------------------------------------------------------------
 */

void *listRemoveLast(List_type *list)
{
	void         *data;
	List_element *link = list->head->prev;

	if (link == list->head) {
		fprintf(stderr,"empty list: can't remove element\n");
		return NULL;
	}

	data = link->data;
	(list->head->prev = link->prev)->next = list->head;
	free( (void *) link);
	return data;
}


/*  listRemoveFirst
 *--------------------------------------------------------------
 *   
 *  Das als Parameter "ubergenene Element einer Liste wird
 *  entfernt und dessen Daten werden freigegeben.
 *
 *--------------------------------------------------------------
 */

void *listRemove(List_type *list, List_element *link)
{
	void * data;

	link->next->prev = link->prev;
	link->prev->next = link->next;
	data = link->data;
	free( (void *) link);
	return data;
}


/*  listFree
 *--------------------------------------------------------------
 *   
 *  Eine Liste wird gel"oscht und deren Speicher wird
 *  freigegben
 *
 *--------------------------------------------------------------
 */

void listFree(List_type *list)
{
	List_element *p;
	List_element *q = list->head->next;


	for (p = q->next; p != list->head; q = p, p = p->next) {

		free((void *) q);
	}
	if (q != list->head) free((void *) q);
	free((void *) list->head);
	free((void *) list);
}


