/*
###########################################################################
###########################################################################
##                                                                       ##
##  Datei   :    `pfaOptions.c'                                          ##
##               --------------                                          ##
##                                                                       ##
##  Inhalt  :    Funktionen, die die neuen Tcl-Befehle des               ##
##               `Option'-Men"us implementieren.                         ##
##                                                                       ##
###########################################################################
###########################################################################
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "pfa.h"

/*
 ##########################################################################
 #                                                                        #
 #  C-Funktion zum  Men"upunkt:                                           #
 #                                                                        #
 #  Preferences                                                           #
 #                                                                        #
 ##########################################################################
 */


/*  tk_UpdateLists
 *--------------------------------------------------------------
 *   
 *  korrigiert die L"ange der Unset- bzw. Undelete-Liste
 *  nachdem die Unset- bzw. Undelete-Scale im Preferences-
 *  Window ver"andert wurde.
 *
 *--------------------------------------------------------------
 */

int tk_UpdateLists(ClientData cd, Tcl_Interp * interp,\
		   int objc, Tcl_Obj *CONST objv[])
{
    Editor_type   *testEd;
    UnsetTop_type *unset_element;
    Undelete_type *undelete_element;

    if (objc != 3) {
	Tcl_WrongNumArgs(interp, 1, objv, "maxunset maxundelete");
        return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[1], &maxUnsetCount) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetIntFromObj(interp, objv[2], &maxUndeleteCount) != TCL_OK)
      return TCL_ERROR;

    /* 
     *  In allen Editoren muss die Anzahl der Unset- sowie der Undelete-
     *  Schritte auf den neuen Stand gesetzt werden
     */

    testEd = firstEd;
    while( testEd != NULL ) {
	while( maxUnsetCount < testEd->unsetCount ) {

	    /*
	     *  Solange der neue Max-Wert der Unset-Liste kleiner ist
	     *  als der alte Max-Wert, muss das letztes Element der Liste
	     *  geloescht werden.
	     */

	    unset_element = listRemoveLast( actualEd->unsetList );
	    if( unset_element != NULL ) {
		listFree( unset_element->elementList );
		free( unset_element );
	    }
	    testEd->unsetCount -= 1;
	}

	while( maxUndeleteCount < testEd->undeleteCount ) {

	    /*
	     *  Solange der neue Max-Wert der Undelete-Liste kleiner ist
	     *  als der alte Max-Wert, muss das letztes Element der Liste
	     *  geloescht werden.
	     */

	    undelete_element = listRemoveLast( actualEd->undeleteList );
	    if( undelete_element != NULL ) {
		listFree( undelete_element->killedVertexList );
		listFree( undelete_element->killedEdgeList );
		listFree( undelete_element->killedCommentList );
		free( undelete_element );
	    }
	    testEd->undeleteCount -= 1;
	}
	testEd = testEd->next;
    }

    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  C-Funktion zum  Men"upunkt:                                           #
 #                                                                        #
 #  Font-Selection                                                        #
 #                                                                        #
 ##########################################################################
 */


/*  tk_CommentSaveFont
 *--------------------------------------------------------------
 *   
 *  tr"agt den im Fontselection-Window gew"ahlten neuen
 *  Font in die entsprechenden C-Strukturen der Kommentare
 *  ein.
 *
 *--------------------------------------------------------------
 */

int tk_CommentSaveFont(ClientData cd, Tcl_Interp * interp,\
		       int objc, Tcl_Obj *CONST objv[])

{
    char           *family;
    char           *weight;
    char            slant;
    int             cid;
    int             size;
    Comment_type   *commentObject;
    Tcl_HashEntry  *comment_entry;
    Tcl_HashSearch  comment_search;

    if (objc != 6) {
	Tcl_WrongNumArgs(interp, 1, objv,"which family weight slant size");
        return TCL_ERROR;
    }

    family = strdup( Tcl_GetStringFromObj(objv[2],NULL) );
    weight = strdup( Tcl_GetStringFromObj(objv[3],NULL) );
    slant = *(Tcl_GetStringFromObj(objv[4],NULL));
    if (Tcl_GetIntFromObj( interp, objv[5], &size ) != TCL_OK)
      return TCL_ERROR;

    if( strcmp(Tcl_GetStringFromObj(objv[1],NULL),"all") == 0 ) { 
	
	/* 
	 *  In die Strukturen aller Kommentare wird der neue Font 
	 *  eingetragen.
	 */

    	for (comment_entry =\
             Tcl_FirstHashEntry (&(actualEd->comment_table),
				 &comment_search);
	     comment_entry != NULL;
	     comment_entry = Tcl_NextHashEntry (&comment_search)) {
	        commentObject = \
		(Comment_type *) Tcl_GetHashValue (comment_entry);
	    commentObject->font->family = family; 
	    commentObject->font->weight = weight;
	    commentObject->font->slant = slant;
	    commentObject->font->size = size;
	}
    } else {

	/* 
	 * Nur der Kommentar mit der Canvas-Id cid bekommt einen neuen Font
	 */

	if (Tcl_GetIntFromObj(interp, objv[1], &cid) != TCL_OK)
	  return TCL_ERROR;
	sprintf(interp->result, "comment%d",cid);
	commentObject = getCommentStruct( interp->result);
	if( commentObject == NULL ) {
	    return TCL_ERROR;
	}

	commentObject->font->family = family; 
	commentObject->font->weight = weight;
	commentObject->font->slant = slant;
	commentObject->font->size = size;
    }
    return TCL_OK;
}

