###########################################################################
###########################################################################
##                                                                       ##
##  Datei   :     `pfaOptions.tcl'                                       ##
##                -------------                                          ##
##                                                                       ##
##  Inhalt  :     Prozeduren, die die Funktionalit"at des                ##
##                `Option'-Men"us gew"ahrleisten.                        ##
##                                                                       ##
###########################################################################
###########################################################################



###########################################################################
##                                                                       ##
##  Funktionen, die im Zusammenhang                                      ##
##  mit dem Preferences-Fenster im                                       ##
##  Option-Menue stehen.                                                 ##
##                                                                       ##
###########################################################################


##  Preferences
##-----------------------------------------------------------
##  Es wird ein Toplevel-Fenster eingerichtet in dem 
##  die vorher geladenen Voreinstellungen verschiedener
##  Aktionen geaendert werden koennen
##-----------------------------------------------------------
##
proc Preferences {} {
    global editor opt saveOpt faNumber histList
   


    ##  Die alten Optionen werden gesichert
    ##
    set saveOpt(label) $opt(label)
    set saveOpt(mouseKeys) $opt(mouseKeys)
    set saveOpt(code) $opt(code)
    set saveOpt(examineFA) $opt(examineFA)
    set saveOpt(highlight) $opt(highlight)
    set saveOpt(unsetScale) $opt(unsetScale) 
    set saveOpt(undeleteScale) $opt(undeleteScale) 
    set labelFont \
	"-*-times-bold-I-Normal--*-140-*"


    ##  Das Toplevel-Window wird erstellt
    ##
    destroy .opt
    toplevel .opt -relief raised
    wm title .opt " "	
    wm geometry .opt 300x590+500+0
    wm minsize .opt 300 590
    wm maxsize .opt 300 590

    ##  Die Canvas fuer die Ueberschrift 
    ##  wird erstellt und gepackt
    ##
    canvas .opt.inf_can -height 50\
	-relief ridge -borderwidth 4

    .opt.inf_can create text 60 30\
	-text "Preferences - Window"\
	-anchor w -font "-*-times-bold-I-Normal--*-180-*" 

    ##  Die gesamten Radiobutton zum Setzen 
    ##  der einzelnen Optionen werden erstellt
    ##  und gepackt
    ##
    frame .opt.radio_opts -relief ridge\
	-borderwidth 4
    set text [frame .opt.radio_opts.te]
    label $text.la -text "Label-Input :"\
	-width 16 -anchor nw\
	-padx 5 -font $labelFont
    set text1 [frame $text.te1]
    radiobutton $text1.edges -text "Only Edges"\
	-value edges -variable opt(label)
    radiobutton $text1.on -text "Always"\
	-value on -variable opt(label)
    radiobutton $text1.off -text "Never"\
	-value off -variable opt(label)
    pack $text1.edges $text1.on $text1.off -side top -anchor w
    pack $text.la $text1 -side left -fill y

    set code [frame .opt.radio_opts.co]
    label $code.la -text "Label-Code :"\
	-width 16 -anchor nw\
	-padx 5 -font $labelFont
    set code1 [frame $code.co1]
    radiobutton $code1.on -text "B(PN)"\
	-value B(PN)2 -variable opt(code)
    radiobutton $code1.off -text "Channel-\nNotation"\
	-value Struct -variable opt(code)
    pack $code1.on $code1.off -side top -anchor w
    pack $code.la $code1 -side left -fill y

    set compile [frame .opt.radio_opts.comp]
    label $compile.la -text "Check/Compile:"\
	-width 16 -anchor nw\
	-padx 5 -font $labelFont
    set compile1 [frame $compile.comp1]
    radiobutton $compile1.on -text "The Visible FA"\
	-value visible -variable opt(examineFA)
    radiobutton $compile1.off -text "All FA's"\
	-value all -variable opt(examineFA)
    pack $compile1.on $compile1.off  -side top -anchor w
    pack $compile.la $compile1 -side left -fill y

    set highlight [frame .opt.radio_opts.hi]
    label $highlight.la -text "Highlight :"\
	-width 16 -anchor w\
	-padx 5 -font $labelFont
    radiobutton $highlight.on -text "On"\
	-value on -variable opt(highlight)
    radiobutton $highlight.off -text "Off"\
	-value off -variable opt(highlight)
    pack $highlight.la $highlight.on $highlight.off -side left

    set mouse [frame .opt.radio_opts.mou]
    label $mouse.la -text "Mouse-Keys :"\
	-width 16 -anchor w\
	-padx 5 -font $labelFont
    radiobutton $mouse.on -text On\
	-value on -variable opt(mouseKeys)
    radiobutton $mouse.off -text Off\
	-value off -variable opt(mouseKeys)
    pack $mouse.la $mouse.on $mouse.off -side left
    pack $text  $code $compile $highlight $mouse\
	-side top -pady 2m -fill x 
    
    ##  Die zwei Scales fuer die Einstellung der 
    ##  Unset- und Undelete-History werden erstellt
    ##  und gepackt
    ##
    frame .opt.undelset -relief ridge\
	-borderwidth 4
    set unset [frame .opt.undelset.uns]
    label $unset.la -text "Unset-History :"\
	-width 16 -anchor sw\
	-padx 5 -font $labelFont
    scale $unset.scale -from 5 -to 250\
	-resolution 5 -length 140\
	-orient horizontal\
	-variable opt(unsetScale)
    pack $unset.la $unset.scale\
	-side left -fill y

    set undelete [frame .opt.undelset.und]
    label $undelete.la -text "Undelete-History :"\
	-width 16 -anchor sw\
	-padx 5 -pady 5 -font $labelFont
    scale $undelete.scale -from 5 -to 250\
	-resolution 5 -length 140\
	-orient horizontal\
	-variable opt(undeleteScale)
    pack $undelete.la $undelete.scale\
	-side left -fill y  
    pack $unset $undelete -side top\
	-pady 2m -fill x 

    ##  Erstellen und Packen des unteren Bereichs 
    ##  mit den zwei Abschlussbutton.
    ##
    frame .opt.exit -relief ridge\
	-borderwidth 4
    set sunk [frame .opt.exit.sa -bd 1\
		  -relief sunken ]
    button $sunk.save\
	-text "Save Changes"\
	-command {set opt(ok) 1}
    pack $sunk.save\
	-padx 2m -pady 2m
    button .opt.exit.discard\
	-text "Cancel" -command {set opt(ok) 0}
    pack $sunk .opt.exit.discard\
	-side left\
	-expand 1\
	-padx 3m -pady 2m

    ##  Packen der vier Bereiche in das
    ##  Toplevel-Fenster
    ##
    pack .opt.inf_can .opt.radio_opts .opt.undelset .opt.exit\
	-side top\
	-fill both\
	-expand yes\
	-padx 1m\
	-pady 1m

    bind .opt <Return> {set opt(ok) 1}
    bind .opt <Control-c> {set opt(ok) 0}

    ##  Die Arbeit mit dem Editor kann erst 
    ##  fortgefuehrt werden, wenn das Preferences-Fenster
    ##  ueber den Save- oder Cancel-Button geschlossen wird.
    ##  D.h., wenn die Variable opt(ok) geaendert wird.
    ##
    if { [llength $histList] > 0 } { 
	DeleteKeyBindings $editor(canvas)
    } else {
	DeleteKeyBindings .fr1a.fr2ru
    }
    tkwait visibility .opt
    grab .opt
    focus .opt
    tkwait variable opt(ok)
    grab release .opt

    if { [llength $histList] > 0 } { 
	focus $editor(canvas)
	HotKeyBindings $editor(canvas)
    } else {
	focus .fr1a.fr2ru
	HotKeyBindings .fr1a.fr2ru
    }

    if { $opt(ok) } {

	##  Wenn mit dem Save-Button abgeschlossen wird,
	##  werden die geaenderten Optionen uebernommen
	##
	if { ($faNumber(sum) > 0) && 
	     ($opt(code) != $saveOpt(code)) &&
	     ([llength [$editor(canvas) find withtag kante]] > 0)  } {

	    if { [CodeChange] != 0 } {
		set editor($faNumber(actual),code) $opt(code)
		SomethingHasChanged FA highlight
	    } else {
		set opt(code) $saveOpt(code)
	    }
    	}

	SetNewValues
    } else {

	##  Wenn mit dem Cancel-Button abgeschlossen wird,
	##  werden die alten Optionen wieder gesetzt.
	##
	SetOldValues
    }
    destroy .opt
}


##  SetOldValues
##-----------------------------------------------------------
##  Die Optionen, die vor dem Aufruf des Preferences-Fensters
##  galten, werden wieder aktualisiert.
##-----------------------------------------------------------
##
proc SetOldValues {} {
    global opt saveOpt

    set opt(label) $saveOpt(label) 
    set opt(mouseKeys) $saveOpt(mouseKeys) 
    set opt(code) $saveOpt(code)
    set opt(examineFA) $saveOpt(examineFA)
    set opt(highlight) $saveOpt(highlight)
    set opt(unsetScale) $saveOpt(unsetScale) 
    set opt(undeleteScale) $saveOpt(undeleteScale) 
}


##  SetNewValues
##-----------------------------------------------------------
##  Die geaenderten Optionen muessen gesetzt und 
##  dann im File preferencesPFA.dat gesichert werden
##-----------------------------------------------------------
##
proc SetNewValues {} {
    global env opt saveOpt histList  

    if {$opt(mouseKeys) != $saveOpt(mouseKeys)} {
	if {$opt(mouseKeys) == "on" } {
	    SetMouse
	} else { 
	    NoMouse
	} 
    }
    if { ($opt(undeleteScale) != $saveOpt(undeleteScale)) ||
	 ($opt(unsetScale) != $saveOpt(unsetScale)) } {
	pfa_UpdateLists $opt(unsetScale) $opt(undeleteScale)
    }

    ##  Fuer die anderen Optionen muessen keine Aktionen
    ##  ausgefuehrt werden.
    ##

    set g [open [file join $env(HOME) $opt(rcfile)] w]
    puts $g "label $opt(label)"
    puts $g "mouseKeys $opt(mouseKeys)"
    puts $g "code $opt(code)"
    puts $g "examineFA $opt(examineFA)"
    puts $g "highlight $opt(highlight)"
    puts $g "unsetScale $opt(unsetScale)"
    puts $g "undeleteScale $opt(undeleteScale)"
    close $g	    
}


###########################################################################
##                                                                       ##
##  Funktionen, die im Zusammenhang                                      ##
##  mit dem Font-Selection-Fenster im                                    ##
##  Option-Menue stehen.                                                 ##
##                                                                       ##
###########################################################################


##  FontSelection
##-----------------------------------------------------------
##  Es wird ein Toplevel-Fenster bereitgestellt, mit dessen
##  Hilfe neue Fonts fuer Kanten- und Knotenbeschriftungen
##  und fuer Kommentare ausgewaehlt werden koennen.
##-----------------------------------------------------------
##
proc FontSelection {} {
    global fontType histList 

    if { [llength $histList] == 0 } { 
	InitialQuestion
	return
    }

    ## Das Toplevel-Fenster wird erstellt.
    ##
    destroy .font
    toplevel .font

    wm title .font  "Font Selection"
    wm geometry .font 350x640+500+0
    wm minsize .font 350 640
    wm maxsize .font 350 640


    set labelFont \
	"-*-times-bold-I-Normal--*-140-*"

    ##  Die Canvas fuer die Darstellung des geaenderten
    ##  Fonts wird erstellt und gepackt
    ##
    canvas .font.inf_can -height 65\
	-relief ridge -borderwidth 4

    ##  Die gesamten Radiobutton zum Setzen 
    ##  des gewuenschten Font werden erstellt
    ##  und gepackt.
    ##
    frame .font.sel_font\
	-relief ridge\
	-borderwidth 4
    set family [frame .font.sel_font.fa]
    label $family.la -text "Family:"\
	-width 7 -anchor w\
	-padx 5 -font $labelFont
    radiobutton $family.times\
	-text "Times" -value Times\
	-variable fontType(family) \
	-command {ExistFont} 
    radiobutton $family.courier\
	-text "Courier" -value Courier\
	-variable fontType(family) \
	-command {ExistFont} 
    radiobutton $family.helvetica\
	-text "Helvetica" -value Helvetica\
	-variable fontType(family)\
	-command {ExistFont} 
    pack $family.la $family.times $family.courier $family.helvetica\
	-side left 

    set weight [frame .font.sel_font.we]
    label $weight.la -text "Weight:"\
	-width 7 -anchor w -padx 5\
	-font $labelFont
    radiobutton $weight.normal -text "Medium" -value Medium\
	-variable fontType(weight)\
	-command ExistFont 
    radiobutton $weight.bold -text "Bold" -value Bold\
	-variable fontType(weight)\
	-command ExistFont 
    pack $weight.la $weight.normal $weight.bold\
	-side left

    set slant [frame .font.sel_font.sla]
    label $slant.la -text "Slant:"  -width 7 -anchor w -padx 5\
	-font $labelFont
    radiobutton $slant.it -text "Italic" -value I\
	-variable fontType(slant)\
	-command ExistFont 
    radiobutton $slant.ro -text "Roman" -value R\
	-variable fontType(slant)\
	-command ExistFont 
    radiobutton $slant.ob -text "Oblique" -value O\
	-variable fontType(slant)\
	-command ExistFont 
    pack $slant.la $slant.it $slant.ro $slant.ob\
	-side left

    set size [frame .font.sel_font.si]
    label $size.la -text "Size:" -width 7 -anchor nw -pady 4 -padx 5\
	-font $labelFont
    set size1 [frame $size.si1]
    radiobutton $size1.8\
	-text " 8 Points"\
	-value 80\
	-variable fontType(size)\
	-command ExistFont 
    radiobutton $size1.10 -text "10 Points" -value 100\
	-variable fontType(size)\
	-command ExistFont 
    radiobutton $size1.12 -text "12 Points" -value 120\
	-variable fontType(size)\
	-command ExistFont 
    radiobutton $size1.14 -text "14 Points" -value 140\
	-variable fontType(size)\
	-command ExistFont 
    radiobutton $size1.18 -text "18 Points" -value 180\
	-variable fontType(size)\
	-command ExistFont 
    radiobutton $size1.24 -text "24 Points" -value 240\
	-variable fontType(size)\
	-command ExistFont 
    radiobutton $size1.36 -text "36 Points" -value 360\
	-variable fontType(size)\
	-command ExistFont 
    pack $size1.8 $size1.10 $size1.12 $size1.14\
	$size1.18 $size1.24 $size1.36\
	-side top -anchor w
    pack $size.la $size1 -side left -fill y

    pack $family $weight $slant $size\
	-side top\
	-pady 2m -fill x 


    ##  Erstellen und Packen des unteren Bereichs 
    ##  mit mehreren Button ueber die die Elemente 
    ##  auf der Zeichenflaeche des Editors geaendert 
    ##  werden koennen.
    ##
    frame .font.but\
	-relief ridge\
	-borderwidth 4
    set change [frame .font.but.ch -border 2]
    label $change.la -text "Change Font Of:"  -anchor nw -pady 7 -width 17\
	-font $labelFont
    set change1 [frame $change.buts -border 2]
    button $change1.edge\
	-text "Edge-Labels"\
	-command {"ChangeEdgeLabelFont"} 
    button $change1.vertex\
	-text "Vertice-Labels"\
	-command {ChangeVertexLabelFont}
    button $change1.comment1\
	-text "Comments"\
	-command {ChangeCommentFont}
    button $change1.comment2\
	-text "Selected Comments"\
	-command {ChangeSelCommentFont}
    pack $change1.edge $change1.vertex\
	$change1.comment1 $change1.comment2 \
	-side top -fill x
    pack $change.la $change1 -side left -fill y
    pack $change\
	-side top -pady 1m -fill x

    ##  Erstellung des unteren Bereichs mit den
    ##  einem Abschlussbutton
    ##
    frame .font.exit -relief ridge\
	-borderwidth 4
    set sunk [frame .font.exit.ca\
		  -bd 1 -relief sunken ]
    button $sunk.cancel\
	-text "Close" -command { ExitFontWindow}
    pack $sunk.cancel -padx 2m -pady 2m
    pack $sunk -side left\
	-expand 1 -padx 3m -pady 2m

    ##  Packen der drei Bereiche in das
    ##  Toplevel-Fenster.
    ##
    pack .font.inf_can .font.sel_font .font.but .font.exit\
	-side top -fill both -expand yes\
	-padx 1m -pady 1m


    ##  Die Ueberschrift wird auf der Canvas  
    ##  gesetzt.
    ##
    set fontType(family) Times
    set fontType(weight) Bold
    set fontType(slant) I
    set fontType(size) 180
    append newFont "-*-$fontType(family)-$fontType(weight)"
    append newFont "-R-Normal--*-$fontType(size)-*"
	
    .font.inf_can create text 145 35\
	-anchor center\
	-text "Font - Selection"\
	-font $newFont\
	-tags fontText 

    ExistFont
}


##  ExistFont
##-----------------------------------------------------------
##  Es wird getestet, ob ein ueber die Radiobutton aus-
##  gewaehlter Font auf dem System existiert. Falls dies der
##  Fall ist, wird dieser Font oben auf der Canvas angezeigt
##  und man hat die Moeglichkeit die Elemente auf der 
##  Zeichenflaeche zu aendern. Sonst wird eine Meldung 
##  ausgegeben, dass dieser Font nicht existiert.
##-----------------------------------------------------------
##
proc ExistFont {} {
    global fontType 

    append newFont "-*-$fontType(family)-$fontType(weight)"
    append newFont "-$fontType(slant)-normal--*-$fontType(size)-*"

    if { [AskForFont $newFont] == 1} {

	.font.inf_can itemconfigure fontText\
	    -text "Font - Selection" -font $newFont
	.font.but.ch.buts.edge configure -state normal
	.font.but.ch.buts.vertex configure -state normal
	.font.but.ch.buts.comment1 configure -state normal
	.font.but.ch.buts.comment2 configure -state normal
	return yes
    } else {
	.font.inf_can itemconfigure fontText\
	    -font -*-Courier-Medium-R-Normal--*-140-*\
	    -text "This Font Does Not Exist"
	.font.but.ch.buts.edge configure -state disabled
	.font.but.ch.buts.vertex configure -state disabled
	.font.but.ch.buts.comment1 configure -state disabled
	.font.but.ch.buts.comment2 configure -state disabled
	return no
    }
}


##  AskForFont
##-----------------------------------------------------------
##  testet direkt, ob ein Font existiert. Diese Prozedur ist
##  nicht in 'ExistFont' integriert, weil sie auch von C aus,
##  beim Laden eines FA aufgerufen wird.
##-----------------------------------------------------------
##
proc AskForFont { newFont } {
    global opt

   if {[catch {.fr1a.fr2bo.entry1 configure \
		    -font $newFont}] == 0} {
       .fr1a.fr2bo.entry1 configure\
	   -font $opt(edFont)
       return 1
   } else {
       .fr1a.fr2bo.entry1 configure\
	   -font $opt(edFont)
       return 0
   }
}


##  ChangeEdgeLabelFont
##-----------------------------------------------------------
##  Nach Betaetigung des Edges-Button werden die Fonts der
##  Kantenbeschriftungen auf der Zeichenflaeche geaendert.
##-----------------------------------------------------------
##
proc ChangeEdgeLabelFont {} {
    global editor fontType faNumber


    append font "-*-$fontType(family)-$fontType(weight)-"
    append font "$fontType(slant)-Normal--*-$fontType(size)-*"

    $editor(canvas) itemconfigure kantetext\
	-font $font
    SomethingHasChanged FA ""
    
    set editor($faNumber(actual),edgeFont) $font
}


##  ChangeVertexLabelFont
##-----------------------------------------------------------
##  Nach Betaetigung des Vertices-Button werden die Fonts der
##  Knotenbeschriftungen auf der Zeichenflaeche geaendert.
##-----------------------------------------------------------
##
proc ChangeVertexLabelFont {} {
    global editor faNumber fontType 


    append font "-*-$fontType(family)-$fontType(weight)-"
    append font "$fontType(slant)-Normal--*-$fontType(size)-*"

    $editor(canvas) itemconfigure startknotentext  -font $font
    $editor(canvas) itemconfigure knotentext  -font $font
    $editor(canvas) itemconfigure endknotentext  -font $font
    SomethingHasChanged FA ""
    set editor($faNumber(actual),vertexFont) $font
    
}


##  ChangeCommentFont
##-----------------------------------------------------------
##  Nach Betaetigung des Comments-Button werden die Fonts der 
##  Kommentare auf der Zeichenflaeche geaendert.
##-----------------------------------------------------------
##
proc ChangeCommentFont {} {
    global editor fontType 


    append font "-*-$fontType(family)-$fontType(weight)-"
    append font "$fontType(slant)-Normal--*-$fontType(size)-*"

    $editor(canvas) itemconfigure comment\
	-font $font
    pfa_CommentSaveFont all\
	$fontType(family) $fontType(weight) $fontType(slant) \
	$fontType(size)

    SomethingHasChanged FA ""

}


##  ChangeSelCommentFont
##-----------------------------------------------------------
##  Nach Betaetigung des Selected-Comments-Button koennen
##  die Fonts ausgewaehlter Kommentare mit der rechten
##  Maustaste veraendert werden
##-----------------------------------------------------------
##
proc ChangeSelCommentFont {} {
    global editor opt fontType 

    set opt(mouse3) changeF
    bind $editor(canvas) <3> ""
    $editor(canvas) bind comment <3> {

	set font ""
	append font "-*-$fontType(family)-$fontType(weight)-"
	append font "$fontType(slant)-Normal--*-$fontType(size)-*"
	$editor(canvas) itemconfigure current\
	    -font $font
	    
	pfa_CommentSaveFont [$editor(canvas) find withtag current]\
		$fontType(family) $fontType(weight) $fontType(slant) \
		$fontType(size)

	SomethingHasChanged FA ""
    
    }
}


##  ExitFontWindow
##-----------------------------------------------------------
##  Das Font-Window wird ueber den Cancel-Button verlassen.
##-----------------------------------------------------------
##
proc ExitFontWindow {} {
    global opt

    set opt(mouse3) Free
    DeleteBindings
    destroy .font
}


###########################################################################
##                                                                       ##
##  Funktionen, die im Zusammenhang                                      ##
##  mit dem Zoom-Fenster im                                              ##
##  Option-Menue stehen.                                                 ##
##                                                                       ##
# #########################################################################


##  ZoomWindow
##-----------------------------------------------------------
##  Es wird ein Toplevel-Fenster eingerichtet mit dessen 
##  Hilfe ein FA auf verschiedene Groessen gezoomt werden
##  kann.
##-----------------------------------------------------------
##
proc ZoomWindow {} {
    global histList oldZoom opt editor

    if { [llength $histList] == 0 } { 
	InitialQuestion
	return
    }
    set oldZoom 1.0

    ##  Das Toplevel-Window wird erstellt
    ##
    destroy .zoom
    toplevel .zoom

    wm title .zoom  " "
    wm geometry .zoom 290x350+500+0
    wm minsize .zoom 290 350
    wm maxsize .zoom 290 350

    ##  Dieses Fenster soll nur ueber den 
    ##  'Cancel' Button verlassen werden
    ##


    ##  Die Canvas fuer die Ueberschrift 
    ##  wird erstellt und gepackt
    ##
    canvas .zoom.inf_can -height 40\
	-relief ridge -borderwidth 4

    ##  Setzen der Ueberschrift auf der
    ##  Canvas
    ##  
    .zoom.inf_can create text 80 25\
	-text "Zoom - Window" \
	-anchor w\
	-font "-*-times-bold-I-Normal--*-180-*"  


    ##  Das Scale-Widget zum Setzen 
    ##  der einzelnen Skalierfaktoren 
    ##  wird erstellt und gepackt
    ##
    frame .zoom.sel -relief ridge\
	-borderwidth 4
    set value [frame .zoom.sel.va]
    label $value.la -text "Zoom-Factor :" -width 15\
	    -anchor nw -pady 4 -padx 5\
	-font "-*-times-bold-I-Normal--*-140-*"
    scale $value.choose\
	-from 0.25 -to 2.0 \
	-resolution 0.05\
	-showvalue 1\
	-length 150\
	-variable opt(zoom)\
	-command {ZoomNet}
    canvas $value.can\
	-width 60 -height 150
    pack $value.la $value.choose $value.can\
	-side left\
	-pady 3m -fill y
    pack $value\
	-side top\
	-pady 2m -fill x 


    ##  Die Skala wird auf der Canvas
    ##  rechts neben dem Scale-Widget
    ##  erzeugt.
    ##
    $value.can create line 0 21 10 21 -width 2.0
    $value.can create text 55 21 -anchor e -text 25%
    $value.can create line 0 38 10 38 -width 2.0
    $value.can create text 55 38 -anchor e -text 50%
    $value.can create line 0 54 10 54 -width 2.0
    $value.can create text 55 54 -anchor e -text 75%
    $value.can create line 0 71 10 71 -width 2.0
    $value.can create text 55 71 -anchor e -text 100%
    $value.can create line 0 87 10 87 -width 2.0
    $value.can create text 55 87 -anchor e -text 125%
    $value.can create line 0 104 10 104 -width 2.0
    $value.can create text 55 104 -anchor e -text 150%
    $value.can create line 0 120 10 120 -width 2.0
    $value.can create text 55 120 -anchor e -text 175%
    $value.can create line 0 137 10 137 -width 2.0
    $value.can create text 55 137 -anchor e -text 200%

    ##  Erstellen und Packen des unteren Bereichs 
    ##  mit dem Abschlussbutton
    ##
    frame .zoom.but -relief ridge\
	-borderwidth 4 -width 10m 
    set sunk [frame .zoom.but.su\
		  -bd 1 -relief sunken ]
    button $sunk.cancel\
	-text "Cancel" -command {set opt(ok) 1}
    pack $sunk.cancel -padx 2m -pady 2m
    pack $sunk -side top\
	-pady 2m -padx 3m\
	-expand 1

    ##  Packen der drei Bereiche in das
    ##  Toplevel-Fenster
    ##
    pack .zoom.inf_can .zoom.sel .zoom.but\
	-side top -fill both -expand yes\
	-padx 1m -pady 1m

    bind .zoom <Return> {set opt(ok) 1}
#    bind .zoom <Control-c> {set opt(ok) 1}

    ##  Die Arbeit mit dem Editor kann erst 
    ##  fortgefuehrt werden, wenn das Zoom Fenster
    ##  ueber Cancel-Button abgeschlossen wird.
    ##  D.h., wenn die Variable opt(ok) geaendert wird.
    ##
    DeleteKeyBindings $editor(canvas)
    tkwait visibility .zoom
    grab .zoom
    focus .zoom
    tkwait variable opt(ok)
    grab release .zoom

    focus $editor(canvas)
    HotKeyBindings $editor(canvas)

    destroy .zoom
    
    ##  Nach Beendigung wird die normale Groesse 
    ##  des FA wieder hergestellt.
    ##  
    set opt(zoom) 1.0
    ZoomNet 1.0


}


##  ZoomNet
##-----------------------------------------------------------
##  Die einzelnen Elemente der Zeichenflaeche werden
##  in ihrer Groesse geaendert
##-----------------------------------------------------------
##
proc ZoomNet { value } {
    global editor opt oldZoom

    set factor [ expr $value / $oldZoom]
    set oldZoom $value
    $editor(canvas) scale startknoten 0 0  $factor $factor
    $editor(canvas) scale endknoten 0 0  $factor $factor
    $editor(canvas) scale knoten 0 0  $factor $factor
    $editor(canvas) scale kante 0 0  $factor $factor      
    $editor(canvas) scale comment 0 0  $factor $factor      
} 


###########################################################################
##                                                                       ##
##  Funktionen, die im Zusammenhang                                      ##
##  mit der Darstellung der Lineale                                      ##
##  oben und links neben der Zeichen-                                    ##
##  flaeche stehen.                                                      ##
##                                                                       ##
###########################################################################


##  Scroll_Hor
##-----------------------------------------------------------
##  Die Bindings des horizontalen Scrollbar werden veraendert.
##  Bei Benutzung des Scrollbar wird nach der Aenderung die
##  Canvas und das horizontale Lineal zusammen gescrollt
##-----------------------------------------------------------
##
proc Scroll_Hor { canv1 n o } {
    global editor

    $editor(canvas) xview $n $o 
    $canv1 xview $n $o 
}


##  Scroll_Vert
##-----------------------------------------------------------
##  Die Bindings des vertikalen Scrollbar werden veraendert.
##  Bei Benutzung des Scrollbar wird nach der Aenderung die
##  Canvas und das vertikale Lineal zusammen gescrollt
##-----------------------------------------------------------
##
proc Scroll_Vert { canv1 n o } {
    global editor

    $editor(canvas) yview $n $o 
    $canv1 yview $n $o 
}


##  SetRuler
##-----------------------------------------------------------
##  Es werden zwei Lineale erstellt und an die linke und
##  obere Seite der Zeichenflaeche gepackt.
##-----------------------------------------------------------
##
proc SetRuler {} {
    global histList editor opt ruler

    if { [llength $histList] == 0 } { 
	InitialQuestion
	return
    }


    .fr1a.fr2m.mb3.m.ruler entryconfigure 1\
	-state disabled
    .fr1a.fr2m.mb3.m.ruler entryconfigure 2\
	-state normal

    ##  Erstellung der Canvas des horizontalen
    ##  Lineals.
    ##
    frame .fr1a.fr2ru.top
    frame .fr1a.fr2ru.top.edgeR \
	-width 17 -height 11
    set ruler(topc) [canvas .fr1a.fr2ru.top.topscale_canv \
		  -insertofftime 600\
		  -relief sunken\
		  -border 1\
		  -scrollregion "0 0 $opt(canScroll) 0"\
		  -xscrollcommand {$editor(scrollX) set}\
		  -xscrollincrement 11\
		  -closeenough 4.0\
		  -height 29\
		  -width 340]

    pack .fr1a.fr2ru.top.topscale_canv \
	    -in .fr1a.fr2ru.top \
	    -side left -fill x -expand 1
    pack .fr1a.fr2ru.top.edgeR \
	    -in .fr1a.fr2ru.top \
	    -side right

    ##  Erstellung der Canvas des vertikalen
    ##  Lineals.
    ##
    frame .fr1a.fr2ru.left

    frame .fr1a.fr2ru.left.edgeT \
	-width 34 -height 34

    frame .fr1a.fr2ru.left.edgeD \
	-width 5 -height 17
    set ruler(leftc) [canvas .fr1a.fr2ru.left.leftscale_canv \
			  -insertofftime 600\
			  -relief sunken\
			  -border 1\
			  -scrollregion "0 0 0 $opt(canScroll)"\
			  -yscrollcommand { $editor(scrollY) set}\
			  -yscrollincrement 3m\
			  -closeenough 4.0\
			  -height 165\
			  -width 29]

    pack .fr1a.fr2ru.left.edgeT \
	    -side top
    pack .fr1a.fr2ru.left.leftscale_canv \
	    -side top -fill y -expand 1
    pack .fr1a.fr2ru.left.edgeD \
	    -side bottom


    ##  Einsetzen der Lineale um die 
    ##  Zeichenflaeche.
    ##
    pack forget $editor(frame)

    pack .fr1a.fr2ru.left \
	-side left -anchor nw -fill y
    pack .fr1a.fr2ru.top \
	-side top -anchor nw -fill x
    pack $editor(frame)\
	-expand true -fill both

    ##  Berechnung der L"ange des Lineals in cm.
    ##
    set length [ expr int($opt(canScroll) * 0.03515) +1 ] ;# in cm 


    ##  Darstellung des horizontalen Lineals 
    ##  auf der entsprechenden Canvas
    ##
    $ruler(topc) create line  0 24 ${length}c 24 
    for {set i 0} {$i < $length} {incr i} {
	if { $i > 0 } {
	    $ruler(topc) create line ${i}c  24 ${i}c  12 
	}
	$ruler(topc) create line ${i}.25c 24 ${i}.25c 19 
	$ruler(topc) create line ${i}.5c  24 ${i}.5c  16 
	$ruler(topc) create line ${i}.75c 24 ${i}.75c 19
	$ruler(topc) create text ${i}.12c 16 -text $i\
	    -anchor sw -font 6x9
    }

    ##  Darstellung des vertikalen Lineals 
    ##  auf der entsprechenden Canvas
    ##
    $ruler(leftc) create line 24 0 24 ${length}c 
    for {set i 0} {$i < $length} {incr i} {
	if { $i > 0 } {
	    $ruler(leftc) create line 24 ${i}c    12 ${i}c  
	}
	$ruler(leftc) create line 24 ${i}.25c 19 ${i}.25c  
	$ruler(leftc) create line 24 ${i}.5c  16 ${i}.5c  
	$ruler(leftc) create line 24 ${i}.75c 19 ${i}.75c 
	$ruler(leftc) create text  16 ${i}.42c   -text $i\
	    -anchor se -font 6x9
    }

    CanvasRulerBindings
}


##  NoRuler
##-----------------------------------------------------------
##  Loeschen der Lineale, wenn die Option opt(ruler) auf
##  no gesetzt wird.
##-----------------------------------------------------------
##
proc AskNoRuler {} {
    global histList 

    if { [llength $histList] == 0 } { 
	InitialQuestion
	return
    }

    .fr1a.fr2m.mb3.m.ruler entryconfigure 2\
	-state disabled
    .fr1a.fr2m.mb3.m.ruler entryconfigure 1\
	-state normal
    NoRuler
}
proc NoRuler {} {

    CanvasNormalBindings
    destroy .fr1a.fr2ru.left
    destroy .fr1a.fr2ru.top
}


##  CanvasNormalBindings
##-----------------------------------------------------------
##  Wenn zwischen Editoren umgeschaltet wird, muss der alte 
##  Editor wieder die normalen Bindings erhalten, falls
##  opt(ruler) auf on steht.
##-----------------------------------------------------------
##
proc CanvasNormalBindings {} {

    global editor faNumber ruler

    bind $editor(canvas) <Any-Enter> {}
    bind $editor(canvas) <Any-Leave> {}
    bind $editor(canvas) <Motion> {}

    $ruler(topc) delete $ruler(hori)
    $ruler(leftc) delete $ruler(verti)

    $editor(scrollY) configure\
	-command "$editor(canvas) yview"
    $editor(scrollX) configure\
	-command "$editor(canvas) xview"

}


##  CanvasRulerBindings
##-----------------------------------------------------------
##  Wenn zwischen Editoren umgeschaltet wird, muss der neue 
##  Editor die Ruler-Bindings erhalten, falls
##  opt(ruler) auf on steht.
##-----------------------------------------------------------
##
proc CanvasRulerBindings {} {
    global editor ruler

    ##  Zeichenfl"ache und Scrollbars werden auf den Ausgangspunkt 
    ##  gesetzt.
    ##
    .fr1a.fr2ru.left.leftscale_canv yview moveto 0
    .fr1a.fr2ru.top.topscale_canv xview moveto 0 
    $editor(canvas) xview moveto 0
    $editor(canvas) yview moveto 0

    ##  Beim Verschieben der Scrollbars wandert die Skala der beiden
    ##  Lineale im exakten Verh"altnis mit.
    ##
    $editor(scrollY) configure\
	    -command {Scroll_Vert .fr1a.fr2ru.left.leftscale_canv }
    $editor(scrollX) configure\
	    -command {Scroll_Hor .fr1a.fr2ru.top.topscale_canv}
    

    ##  Bei Eintreten des Mauszeigers in die Zeichenfl"ache erscheinen
    ##  zwei Markierungen auf den Linealen durch die die exakte 
    ##  Position festgestellt werden kann.
    ##
    bind $editor(canvas) <Any-Enter> { 

	set xw [$editor(canvas) canvasx %x 0.5m] 
	set ruler(hori) [$ruler(topc) create line $xw 0 $xw 8m]
	set ruler(verti) [$ruler(leftc) create line \
			      0 [$editor(canvas) canvasy %y 0.5m]\
			      8m [$editor(canvas) canvasy %y] ]

    }
    bind $editor(canvas) <Motion> {

	set pos(x) [$editor(canvas) canvasx %x]
	set pos(y) [$editor(canvas) canvasy %y]
	set xall [$editor(canvas) canvasx %x 0.5m ]
	set yall [$editor(canvas) canvasy %y 0.5m]
	$ruler(topc) coords $ruler(hori) $xall 0 $xall 8m
	$ruler(leftc) coords $ruler(verti) 0 $yall 8m $yall

    }

    ##  Beim Verlassen der Zeichenfl"ache verschwinden die Markierungen
    ##  wieder.
    ##
    bind $editor(canvas) <Any-Leave> { 

	$ruler(topc) delete $ruler(hori)
	$ruler(leftc) delete $ruler(verti)
    }
}
