#####################################################################
##
##  Die folgenden Prozeduren implementieren eine komfortable  
##  Suchfunktion f"ur den B(PN)-Editor.
##
#####################################################################

proc Search {cmd} {
#
# Parameter:
#
# Rueckgabe:
#
# Bemerkung:

  global direction wrap regular

  toplevel .search
  wm title .search $cmd

  frame		.search.searchfr -borderwidth 10
  frame		.search.replacefr -borderwidth 10
  frame		.search.optionfr -borderwidth 2 
  frame		.search.optionleftfr -borderwidth 4
  frame		.search.optionrightfr -borderwidth 4
  frame		.search.buttonfr -borderwidth 10
  frame		.search.nextfr -borderwidth 1 -relief sunken
  frame		.search.cancelfr -borderwidth 1
  frame		.search.helpfr -borderwidth 1
  frame		.search.verifyfr -borderwidth 1
  frame		.search.allfr -borderwidth 1

  message	.search.search_msg -text "Search" -width 55
  entry		.search.search_entry -relief sunken \
	  -textvariable search_string
  message	.search.replace_msg -text "Replace" -width 50
  entry		.search.replace_entry -relief sunken\
		-textvariable replace_string

  message	.search.direction_msg -text "Direction" -width 150
  message	.search.option_msg -text "Options" -width 100
  radiobutton	.search.direction_for -text "Forward" -variable direction\
		-relief flat -value "-forwards" -command "selection clear"
  radiobutton	.search.direction_back \
	  -text "Backward" -variable direction\
		-relief flat -value "-backwards" -command "selection clear"
  checkbutton	.search.option_wrap -text "Wrap" \
	  -variable wrap -relief flat
  checkbutton	.search.option_reg -text "Regular Expr." -variable regular\
		-relief flat
  checkbutton	.search.option_word -text "Whole Word" -variable word\
		-relief flat
  button	.search.help -text "Help"
  button	.search.cancel -text "Cancel" -command {destroy .search}
  button	.search.verify -text "Replace"\
		-command {PFAverify $search_string $replace_string}
  button	.search.next -text "Search" -command "PFAnext"
  button	.search.all -text "Replace all"\
		-command {PFAall $search_string $replace_string}
  ##########
  # Packer #
  ##########

  pack	.search.searchfr -side top -fill x

  if {$cmd == "Replace"} {
    pack .search.replacefr -side top -fill x
    pack .search.replace_msg -side left -in .search.replacefr
    pack .search.replace_entry -side right -expand yes\
          -fill x -in .search.replacefr
  }

  pack	.search.optionfr -side top -fill x
  pack	.search.buttonfr -side bottom -fill x

  pack	.search.optionleftfr -side left -anchor n -padx 10\
	-in .search.optionfr
  pack	.search.optionrightfr -side right -anchor n -padx 10\
	-in .search.optionfr

  pack	.search.search_msg -side left -in .search.searchfr
  pack	.search.search_entry -side right -expand yes -fill x\
	-in .search.searchfr
  pack	.search.direction_msg -side top -anchor w -in .search.optionleftfr
  pack	.search.option_msg -side top -anchor w -in .search.optionrightfr
  pack	.search.direction_for .search.direction_back -side top -anchor w\
	-in .search.optionleftfr
  pack	.search.option_wrap .search.option_reg .search.option_word\
	-side top -anchor w -in .search.optionrightfr

  if {$cmd == "Search"} {
	pack	.search.nextfr .search.cancelfr .search.helpfr\
		-side left -in .search.buttonfr -pady 3 -padx 2m
	pack	.search.next -side left -in .search.nextfr -fill x\
		-padx 1m -pady 1m -ipady 1m -ipadx 1m
  } else {
        pack	.search.verifyfr .search.allfr \
		.search.cancelfr .search.helpfr\
		-side left -in .search.buttonfr -pady 3 -padx 2m
	pack	.search.verify -side left -in .search.verifyfr -fill x\
		-padx 1m -pady 1m -ipady 1m -ipadx 1m
	pack	.search.all -side left -in .search.allfr -fill x\
		-padx 1m -pady 1m -ipady 1m -ipadx 1m
  }

  pack	.search.cancel -side left -in .search.cancelfr -fill x\
	-padx 1m -pady 1m -ipady 1m -ipadx 1m
  pack	.search.help -side left -in .search.helpfr -fill x\
	-padx 1m -pady 1m -ipady 1m -ipadx 1m

  ####################
  # Defaultselection #
  ####################

  .search.direction_for select
  .search.option_wrap select
  .search.option_reg select

  ############
  # Bindings #
  ############

  bind .search.search_entry <Return> {PFAnext}

}

proc PFAnext { } {

# Sucht einen String, setzt Cursor und markiert Wort.
#
# Rueckgabe: 0, wenn search_string nicht gefunden wurde
#            1, sonst
# Bemerkung: search_string = Gesuchter String
#            direction = Suchrichtung, -forawrds oder -backwards
#            wrap = Suche wieder am Anfang beginnen ? (0 oder 1)
#            regular = Suche nach regulaerem Ausdruck ? (0 oder 1)
#            word = Ganzes Wort markieren ? (0 oder 1)

   global search_string direction wrap regular word

     # Optionen
     
     if {($regular)} {
        set op1 "-regexp"
     } else {
        set op1 "-exact"
     }
     
     # Cursorpositionierung
     # --------------------
     # forward	-> Cursor hinter Suchstring
     #			(mit regexp oder wort-> hinter Wort)
     # backward	-> Cursor vor Suchstring
     #			(mit regexp oder wort-> vor Wort)

     tkTextSetCursor .watch.text_frame.t \
	     [.watch.text_frame.t index {insert -1c}]

     # Setzen der Endmarkierung. Bei -forwards -> index end
     #                               -backwars -> index first
     if {($wrap)} {
         set pos [.watch.text_frame.t search $direction $op1 \
                 $search_string [.watch.text_frame.t index insert]]
      } else {
   
         if { $direction == "-forwards"} { 
            set pos [.watch.text_frame.t search $direction $op1 \
                     $search_string [.watch.text_frame.t index insert]\
		     [.watch.text_frame.t index end]]
         } else {
            set pos [.watch.text_frame.t search $direction $op1 \
                     $search_string [.watch.text_frame.t index insert] 1.0]
	puts $direction
	puts [.watch.text_frame.t index first]
         }
      }


     if {$pos == ""} {
         tk_dialog .d { } {String Not Found !} warning 0 {Ok} 
         return 0
     }

     if {$direction == "-forwards"} {
         if {($word) | ($regular)} {
             # ganzes Wort, forward
             tkTextSetCursor .watch.text_frame.t "$pos wordstart"
             tkTextKeySelect .watch.text_frame.t \
		     [.watch.text_frame.t index "insert wordend"]
         } else {
             # Teilstring,forward
             tkTextSetCursor .watch.text_frame.t $pos
             tkTextKeySelect .watch.text_frame.t \
		     [.watch.text_frame.t index insert+[string length\
                              $search_string]c ]
         }
     } else {
         if {(!$word) && (!$regular)} {
            # Teilstring, backward
            tkTextSetCursor .watch.text_frame.t \
		    $pos+[string length $search_string]c
            tkTextKeySelect .watch.text_frame.t \
		    [.watch.text_frame.t index insert-[string length\
                            $search_string]c ]
         } else {
            # ganzes Wort, backward
            tkTextSetCursor .watch.text_frame.t "$pos wordend"
            tkTextKeySelect .watch.text_frame.t \
		    [.watch.text_frame.t index "insert -1c wordstart"]
         }
     }

 return 1
}

proc PFAexists_string {search_string regular} {

# Durchsucht den Text nach $search_string.
#
# Parameter: search_string = Gesucher String
#	     regular = regexp 
#
# Rueckgabe: 0, wenn $search_string nicht gefunden wurde
#            1, sonst
#
# Bemerkung: Der Text wird von Position 1.0 bis zum Ende nach 
#            $search_string
#            durchsucht. Danach wird der Cursor wieder auf die alte 
#            Position
#            gesetzt und (0|1) zurueckgegeben.
#

   set oldpos [.watch.text_frame.t index insert]
   if {$regular} {
       set op1 "-regexp"
   } else {
       set op1 "--"
   }
   set pos [.watch.text_frame.t search -forwards -nowrap $op1\
            $search_string 1.0 end]
   if {$pos == ""} {
       tkTextSetCursor .watch.text_frame.t $oldpos
       return 0
   }
   tkTextSetCursor .watch.text_frame.t $oldpos
   return 1
}

proc PFAverify {search_string replace_string} {

# Sucht "search_string" und ersetzt ihn mit "replace_string",
# wenn gewuenscht.
#
# Parameter: search_string = zu ersetzender String
#            replace_string = String, der search_string ersetzt
#
# Rueckgabe: nichts
#
# Bemerkung: 

   while {[PFAnext] != 0} {
       set ok [tk_dialog .d { } {Replace Selection ?} question 1\
               {Replace} {Next} {Cancel}]
       if {$ok == 2} {return}
       if {$ok == 0} {
           # Selection loeschen
           Cut
           # Ersetzen
           .watch.text_frame.t insert insert $replace_string
       }
   }
}

proc PFAall {search_string replace_string} {

# Jeder $search_string wird durch $replace_string ersetzt
#
# Parameter: search_string = zu ersetzender String
#            replace_string = String der search_string ersetzt
#
# Rueckgabe: nichts
#
# Bemerkung: Macht das Gleiche wie verify, nur ohne Nachfragen

   global wrap

   while {[PFAnext] != 0} {
           Cut
           .watch.text_frame.t insert insert $replace_string
   }
}

# ende Search.tcl
