/*
**************************************************************************
**************************************************************************
**                                                                      **
**  Datei   :    `pfaAction.c'                                          **
**               -------------                                          **
**                                                                      **
**  Inhalt  :    Funktionen, die die neuen Tcl-Befehle des              **
**               `Action'-Men"us implementieren.                        **
**                                                                      **
**************************************************************************
**************************************************************************
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "pfaCompile.h"
#include "pfaStructure.h"

/*
 *************************************************************************
 *
 *  Deklaration der lokalen Strukturen
 *
 *************************************************************************
 */

typedef struct QueueCircleCell {
    Vertex_type             *vertexObject;
    IntList                *history;
    struct QueueCircleCell *next;
} QueueCircleCell;

typedef struct QueueCircle {
    QueueCircleCell *front;
    QueueCircleCell *end;
} QueueCircle;

typedef struct PopResult {
    Vertex_type  *vertexObject;
    StructEntry *structentry;
    int          plusinnr;
    int          changed;
} PopResult;

typedef struct AllInEdgesResult {
    int allinedges;
    int plusinnr;
} AllInEdgesResult;

typedef struct ChannelTableEntry {
    int        channel;
    IntList   *inputlist;
} ChannelTableEntry;


/*
 *************************************************************************
 *
 *  Deklaration der globalen Variablen
 *
 *************************************************************************
 */

Tcl_HashTable channelTable;
Tcl_HashTable inputTable;
Tcl_HashTable structTable; 

Queue     queue;
Queue     problemnodes;
int       nrrealstartstates;
int       nrrealfinalstates;
int       exzulkanteendstart; 

/*
 *##############################################################
 *
 *  Deklaration der lokalen Funtionen
 *
 *##############################################################
 */

int StructNamingStart(StructEntry *structentry, Tcl_HashTable placetable);

int InPlusinnrStartList(Vertex_type *vertexObject, IntList *startlist);

IntList *PutPlusinnrStartList(Vertex_type *vertexObject, IntList *startlist);

StructEntry *CopyStructElement( StructEntry *structentry);

int Circle (Vertex_type *vertexObject, Tcl_HashTable placetable,
            Tcl_HashTable arctable);

int CircleStructNaming (Vertex_type *vertexObject, int start_canvas_id,
                        Tcl_HashTable placetable, Tcl_HashTable arctable);

IntList *CircleRepeat (Vertex_type *vertexObject, int start_canvas_id,
                       Tcl_HashTable placetable, Tcl_HashTable arctable);

IntList *CopyHistory(IntList *history);

int InHistory(IntList *history, int currentid);

IntList *PutHistory(IntList *history, int newid);

QueueCircleCell* PopCircleQueue(QueueCircle *queuecircle);

void PutCircleQueue(Vertex_type *vertexObject, IntList *history,
                    QueueCircle *queuecircle);

AllInEdgesResult AllInEdgesCircle(Vertex_type *vertexObject, int innr,
                                  StructEntry *structentry,
                                  Tcl_HashTable circlestructTable);

PopResult DeQueue(Vertex_type *vertexObject, StructEntry *structentry,
		  Queue *queue);

void EnQueue(Vertex_type *vertexObject, StructEntry *structentry,
             int plusinnr, int changed, Queue *queuecircle);

void PutMarkToPlace(Vertex_type *vertexObject, StructEntry *structentry,
                    int plusinnr);

AllInEdgesResult AllInEdges(Vertex_type *vertexObject, int innr,
                            StructEntry *structentry);

int AlmostIdentStruct (StructEntry *structentry1, StructEntry *structentry2);

int IdentStruct(StructEntry *structentry1, StructEntry *structentry2);

int CountIn (Vertex_type *vertexObject);

Vertex_type *GetEndPlace(Tcl_HashTable placetable);

Vertex_type *Get1EchtenKnoten(Vertex_type *vertexObject,
                             Tcl_HashTable placetable,
                             Tcl_HashTable arctable);

void ClearStructElement (StructEntry *structelement);

void ClearArcList(EdgeList_type *arclist);

void ClearQueuePtr(Queue *queuelocal);

void ClearQueueCircle(QueueCircle *queuecircle);

/*  StructNaming
 *--------------------------------------------------------------
 *  Hauptfunktion des ersten Laufs des Struct-Compilers zur  
 *  Uebersetzung von Automaten der klasse NFAStruct.
 *  Legt Informationen fuer den zweiten Lauf bzw. erkennt
 *  Automaten, die nicht die geforderten Eigenschaften 
 *  aufweisen.
 *--------------------------------------------------------------
 */

int StructNaming (Vertex_type *vertexObject, Tcl_HashTable placetable,
                   Tcl_HashTable arctable)
{
    StructEntry     *structentry = NULL;
    StructEntry     *structentrynew;
    StructEntry     *help;
    Vertex_type     *ziel_object;
    EdgeList_type   *kante;
    EdgeList_type   *repeatedges;
    PopResult        popresult;
    AllInEdgesResult allinedgesresult;
    IntList         *startlist = NULL;
    int              changed = 0;
    int              circle = 0;
    int              innr;
    int              outnr;
    int              allinedges;
    int              plusinnr = 0;     /* auf jedem Weg mitziehen */
    int              anzahlechtekanten;
    int              i;

    Tcl_InitHashTable(&structTable, TCL_STRING_KEYS);

    structentry = malloc(sizeof(StructEntry));
    structentry->canvas_id = -(vertexObject->v_id);
    structentry->count    = 2;
    structentry->number   = 2;
    structentry->next     = NULL;

    if (NrStartStates(placetable) > 1) {
        changed =
        StructNamingStart(CopyStructList(structentry), placetable);
        popresult = DeQueue(vertexObject, structentry, &queue);
        vertexObject = popresult.vertexObject;
        structentry  = popresult.structentry;
    }
    while (vertexObject) {
        if ( (vertexObject->type == 3) && (nrrealfinalstates == 1) ) {
            repeatedges = GetRepeatKanten(vertexObject);
            if ( (vertexObject->out) && (!repeatedges) ) {
                /* pruefe noch Fall der zul. Kante von End zu Start */
                if ( (exzulkanteendstart == 0) ||
                     (vertexObject->out->next) ) {
                    /* Test, ob zul. Zyklus am Endknoten */
                    if (Circle(vertexObject, placetable, arctable) == 0) {
                        if (!problemnodes.front) {
                            problemnodes = queue;
                        } else {
                            ClearQueue(queue);
                        }
                        ClearStructList(structentry);
                        ClearIntList(startlist);
                        return 0;
                    }
                }
            }
            ++changed;
            popresult = DeQueue(vertexObject, structentry, &queue);
            vertexObject = popresult.vertexObject;
            structentry  = popresult.structentry;
            plusinnr = popresult.plusinnr;
        } else {
            innr  = CountIn(vertexObject);
            outnr = CountOut(vertexObject);
            if (   ((vertexObject->type != 1) && (innr > 1))
                || ((vertexObject->type == 1) && 
                    (exzulkanteendstart == 0) && (innr > 0))
                || ((vertexObject->type == 1) &&
                    (exzulkanteendstart == 1) && (innr > 1)) ) {
                if (vertexObject->type == 1) {
                    if (InPlusinnrStartList(vertexObject, startlist) == 0) {
                            /* 
			     * Jeder Startknoten, der eingehende Kante
			     * hat, soll genau einmal sein plusinnr um
			     * eins erhoehen
                             */

                            startlist =
                            PutPlusinnrStartList(vertexObject, startlist);
                            plusinnr = 1;
                    }
                }

		/*
		 * Aufruf der Funktion zum Ablegen der mitgefuehrten 
		 * Inforationen an dem aktuellen Knoten
		 */

                PutMarkToPlace(vertexObject,
                               CopyStructList(structentry), plusinnr);
		/* 
		 * Vollstaendigkeits- und Vereinigungstest
		 */

                allinedgesresult = AllInEdges(vertexObject, innr, structentry);
                allinedges = allinedgesresult.allinedges;
                plusinnr = allinedgesresult.plusinnr;
                if ( allinedges == 0 ) {

		    /*
		     * warte an aktuellem Knoten
		     */
                    EnQueue(vertexObject, CopyStructList(structentry),
                             0, changed, &queue);
                    popresult = DeQueue(vertexObject, structentry, &queue);
                    vertexObject = popresult.vertexObject;
                    structentry  = popresult.structentry;
                    plusinnr = popresult.plusinnr;
                    if ( (changed == popresult.changed) &&
                         (circle == 1) ) {

                        /*
			 * keine Verkuerzung moeglich und 
			 * circle einmal herum
			 */
                        
                        EnQueue(vertexObject, structentry, 0, changed, &queue);
                        if (!problemnodes.front) {
                            problemnodes = queue;
                        } else {
                            ClearQueue(queue);
                        }

                        /* 
			 * Automat ist nicht in der Klasse NFAStruct,
			 * somit NICHT UEBERSETZBAR 
			 */

                        ClearIntList(startlist);
                        return 0;
                    } else if ( (changed == popresult.changed) &&
                                (circle == 0) ) {
                        /* keine Verkuerzung bis jetzt -> circle starten */
                        circle = 1;
                        ++changed;
                        if (Circle(vertexObject, placetable, arctable) == 1) {
                            /* mind. 1 Zyklus (am ersten Knoten) gefunden */
                            circle = 0;
                        }
                    } else if ( (changed != popresult.changed) &&
                                (circle == 1) ) {
                        /* circle laeuft, bisher noch kein Zyklus gefunden */
                        if (Circle(vertexObject, placetable, arctable) == 1) {
                            /* mind. 1 Zyklus (am ersten Knoten) gefunden */
                            ++changed;
                            circle = 0;
                        }
                    }
                    continue;
                } else if ( allinedges == 1 ) {
		   /* aktueller Prozess endet hier */
                    ++changed;
                    popresult = DeQueue(vertexObject, structentry, &queue);
                    vertexObject = popresult.vertexObject;
                    structentry  = popresult.structentry;
                    plusinnr = popresult.plusinnr;
                    continue;
                } else if ( allinedges == 2 ) {

		    /*
		     * gehe von Knoten weiter
		     */

                    ++changed;
                } else if ( allinedges >= 3) {

		    /*
		     * Vereinigung erfolgreich -> verkuerze und gehe 
		     * weiter
		     */

                    ++changed;
                    while (allinedges > 2) {
                        help = structentry;
                        structentry = structentry->next;
                        ClearStructElement(help);
                        --allinedges;
                    }
                }
            }

	    /*
	     * Verlassen des Knotens moeglich
	     */

            anzahlechtekanten = AnzahlEchteKanten(vertexObject);
            if (anzahlechtekanten == 0) {
                /* Nur im Fall mehrerer Endknoten */
                if (vertexObject->type != 3) {
                    printf("Error in StructNaming.\n");
                    return 0;
                }
                popresult = DeQueue(vertexObject, structentry, &queue);
                vertexObject = popresult.vertexObject;
                structentry  = popresult.structentry;
                plusinnr = popresult.plusinnr;
                ++changed;
                continue;
            }

            if ( (vertexObject->type == 3) && (nrrealfinalstates > 1) ) {
                structentrynew = malloc(sizeof(StructEntry));
                structentrynew->canvas_id = -(vertexObject->v_id);
                structentrynew->count     = 2;
                structentrynew->number    = 2;
                structentrynew->next      = structentry;
                structentry = structentrynew;
            }
            if (anzahlechtekanten == 1) {
		/* nur eine ausgehende Kante */
                ++changed;
                vertexObject = Get1EchtenKnoten(vertexObject,
                                                placetable, arctable);
            } else if (anzahlechtekanten > 1) {
		/* mehrere ausgehende Kanten */
                kante = vertexObject->out;
                ziel_object = GetKantenZiel(kante, placetable, arctable);
                i = 0;
                while (kante->next) {

		    /*
		     * starte fuer jede ausgehende Kante (-1) einen
		     * neuen Prozess
		     */

                    ++changed;
                    if ( InRepeatKanten(kante, vertexObject) == 0 ) {
                        ++i;
                        structentrynew = malloc(sizeof(StructEntry));
                        structentrynew->canvas_id = vertexObject->v_id;
                        structentrynew->count    = i;
                        structentrynew->number   = anzahlechtekanten;
                        structentrynew->next     = CopyStructList(structentry);
                        EnQueue(ziel_object, structentrynew, plusinnr, 
				changed, &queue);
                    }
                    kante = kante->next;
                    ziel_object = GetKantenZiel(kante, placetable, arctable);
                }
                ++changed;
                if ( InRepeatKanten(kante, vertexObject) == 1 ) {
                    popresult = DeQueue(vertexObject, structentry, &queue);
                    vertexObject = popresult.vertexObject;
                    structentry  = popresult.structentry;
                    plusinnr = popresult.plusinnr;
                } else {
                    structentrynew = malloc(sizeof(StructEntry));
                    structentrynew->canvas_id = vertexObject->v_id;
                    structentrynew->count    = anzahlechtekanten;
                    structentrynew->number   = anzahlechtekanten;
                    structentrynew->next     = structentry;
                    structentry = structentrynew;
                    vertexObject = ziel_object;
                }
            } else {
                printf("End vertex");
            }
        } 
    }
    ClearStructList(structentry);
    ClearQueue(queue);
    ClearIntList(startlist);
    return 1;
}

/*  StructNamingStart
 *--------------------------------------------------------------
 *  Vorbereitung der Strukturanalyse fuer die Startknoten 
 *--------------------------------------------------------------
 */

int StructNamingStart(StructEntry *structentry, Tcl_HashTable placetable)
{
    Tcl_HashEntry *place_entryPtr;
    Vertex_type    *vertexObject;
    Tcl_HashSearch search;
    int            changed = 0;

    for ( place_entryPtr = Tcl_FirstHashEntry (&placetable, &search);
          place_entryPtr;
          place_entryPtr = Tcl_NextHashEntry(&search) ) {
        vertexObject = Tcl_GetHashValue(place_entryPtr);
        if (vertexObject->type == 1) {
             if (CountOut(vertexObject) != 0) {
                 structentry->canvas_id = -(vertexObject->v_id);
                 EnQueue(vertexObject, CopyStructList(structentry), 0, 
			 changed, &queue);
                 ++changed;
             }
        }
    }
    ClearStructList(structentry);
    return changed;
}

/*  ExZulKanteEndStart
 *--------------------------------------------------------------
 *  Teste, ob es eine zulaessige Kante vom End- zum Startknoten
 *  gibt. Dieser darf nur eine epsilon-Beschriftung aufweisen 
 *--------------------------------------------------------------
 */

int ExZulKanteEndStart(Tcl_HashTable placetable, Tcl_HashTable arctable)
{
    EdgeList_type *kante;
    char          *kante_label;

    if ( (nrrealstartstates != 1) || (nrrealfinalstates != 1) ) {
        return 0;
    }
    for (kante = (GetEndPlace(placetable))->out;
         kante ;
         kante = kante->next) {
        if ( (GetKantenZiel(kante, placetable, arctable))->type == 1) {
            kante_label = GetKantenLabel(kante, arctable);
            if (strcmp(kante_label, "")) {
                return -1;
            }
            return 1;
        }
    }
    return 0;
}

/*  InPlusinnrStartList
 *--------------------------------------------------------------
 *   Hilfsfunktion bei mehreren Startknoten
 *--------------------------------------------------------------
 */

int InPlusinnrStartList(Vertex_type *vertexObject, IntList *startlist) 
{
    while (startlist) {
        if (startlist->id == vertexObject->v_id) {
            return 1;
        }
        startlist = startlist->next;
    }
    return 0;
}


/*  PutPlusinnrStartList
 *--------------------------------------------------------------
 *   Hilfsfunktion bei mehreren Startknoten
 *--------------------------------------------------------------
 */

IntList *PutPlusinnrStartList(Vertex_type *vertexObject, IntList *startlist) 
{
    IntList *new;

    new = malloc(sizeof(IntList));
    new->id   = vertexObject->v_id;
    new->next = startlist;
    return new;
}

/*  InRepeatKanten
 *--------------------------------------------------------------
 *  Test, ob Repeatkanten existieren.
 *--------------------------------------------------------------
 */

int InRepeatKanten(EdgeList_type *kante, Vertex_type *vertexObject)
{
    EdgeList_type      *repeatedges;

    repeatedges = GetRepeatKanten(vertexObject);
    while (repeatedges) {
        if (repeatedges->arc_id == kante->arc_id) {
            return 1;
        }
        repeatedges = repeatedges->next;
    }
    return 0;
}

/*  Circle
 *--------------------------------------------------------------
 *   Funktion zum Zyklus-Test
 *--------------------------------------------------------------
 */

int Circle (Vertex_type *vertexObject, Tcl_HashTable placetable,
            Tcl_HashTable arctable)
{
    PlaceMarkList  *placemarklist;
    StructEntry    *structentry;
    int             innumber = 0;
    int             resultgive = 0;
    IntList        *allinarcs = NULL;
    IntList        *inarcs;
    IntList        *helpinarcs;
    IntList        *help;
    int             outnr;
    EdgeList_type *kante;
    int             i;
    char            key[LINE_LENGTH];
    Tcl_HashEntry  *struct_entryPtr;
    EdgeList_type  *repeatedges;
    Vertex_type    *ziel_object;

    sprintf(key, "%d", vertexObject->v_id);
    struct_entryPtr = Tcl_FindHashEntry(&structTable, key);
    if (!struct_entryPtr) {
        /* Zyklus im Zyklus */
        structentry = malloc(sizeof(StructEntry));
        structentry->canvas_id = vertexObject->v_id;
        structentry->count    = 0;
        structentry->number   = 0;
        structentry->next     = NULL;
        PutMarkToPlace(vertexObject, structentry, 0);
        struct_entryPtr = Tcl_FindHashEntry(&structTable, key);
    }
    placemarklist = (PlaceMarkList *) Tcl_GetHashValue(struct_entryPtr);
    if (placemarklist->repeatedges) {

        /*
	 * Fall, dass fur diesen Knoten nur einige Kanten
	 *  Repeatkanten waren.
 	 */

        return 0;
    }

    outnr = CountOut(vertexObject);
    kante = vertexObject->out;
    
    for ( i = 1; i <= outnr; ++i) {
        ziel_object = GetKantenZiel(kante, placetable, arctable); 
        inarcs = CircleRepeat (ziel_object, vertexObject->v_id,
                               placetable, arctable);
        if (inarcs) {

            /* 
	     * markiere Knoten vertexObject mit der ausgehenden 
	     * Kante durch Anhaengen der kante an Feld repeatkante;
             */

            resultgive = 1;
            repeatedges = placemarklist->repeatedges;
            if (!repeatedges) {
                placemarklist->repeatedges = CopyKante(kante);
            } else {
                while (repeatedges->next) {
                    repeatedges = repeatedges->next;
                }
                repeatedges->next = CopyKante(kante);
            }

            /* 
	     * uebernehme die inarcs, falls noch nicht vorhanden 
	     */

            for (helpinarcs = inarcs; helpinarcs;
                 helpinarcs = helpinarcs->next) {
                if (!allinarcs) {
                    allinarcs = CopyArcIdElem(helpinarcs);
                } else {
                    help = allinarcs;
                    while (help->next) {
                        if (help->id == helpinarcs->id) {
                            break;
                        }
                        help = help->next;
                    }
                    if (help->id != helpinarcs->id) {
                        help->next = CopyArcIdElem(helpinarcs);
                    }
                }
            }
            ClearIntList(inarcs);
        }
        kante = kante->next;
    }
    if (placemarklist->repeatedges) {
        if ( CircleStructNaming(vertexObject, vertexObject->v_id,
                                placetable, arctable) == 0) {;
            ClearArcList(placemarklist->repeatedges);
            placemarklist->repeatedges = NULL;
            ClearIntList(allinarcs);
            allinarcs = NULL;
            resultgive = 0;
        }
    }
    for (help = allinarcs; help; help = help->next) {
        ++innumber;
    }
    ClearIntList(allinarcs);

    /*
     * Anzahl der durch repeat eingehenden Kanten von plusinnr
     * subtrahieren
     */

    if (innumber != 0) {
        while (placemarklist) {
            placemarklist->plusinnr -= innumber;
            placemarklist = placemarklist->next;
        }
    }
    return resultgive;
}

/*  CircleStructNaming
 *--------------------------------------------------------------
 *  Fuehrt die Strukturanalyse von Zyklen durch, nachdem 
 *  festgestellt wurde, dass ein Zyklus vorliegt.
 *  Aehnliche Arbeitsweise wie die Funktion StructNaming.
 *  Fuer detaillierteres Verstaendnis siehe dort.
 *--------------------------------------------------------------
 */

int CircleStructNaming (Vertex_type *vertexObject, int start_canvas_id,
                        Tcl_HashTable placetable, Tcl_HashTable arctable)
{
    StructEntry     *structentry = NULL;
    StructEntry     *structentrynew;
    StructEntry     *help;
    Vertex_type      *ziel_object;
    EdgeList_type    *repeatedges;
    EdgeList_type    *kante;
    PopResult        popresult;
    AllInEdgesResult allinedgesresult;
    int              changed = 0;
    int              circle = 0;
    int              innr, outnr;
    int              allinedges;
    int              plusinnr = 0;     /* auf jedem Weg mitziehen */
    int              anzahlechtekanten;
    int              i;
    Queue           *queuecircle;

    queuecircle = malloc(sizeof(Queue));
    queuecircle->front = NULL;
    queuecircle->end   = NULL;
    repeatedges = GetRepeatKanten(vertexObject);
    i = 0;
    for ( kante = repeatedges; kante; kante = kante->next) {
        ++i;
    }
    outnr = i;
    i = 0;
    for ( kante = repeatedges; kante->next; kante = kante->next) {
        ++i;
        ziel_object = GetKantenZiel(kante, placetable, arctable);
        ++changed;
        structentrynew = malloc(sizeof(StructEntry));
        structentrynew->canvas_id = vertexObject->v_id;
        structentrynew->count    = i;
        structentrynew->number   = (2 * outnr);
        structentrynew->next     = NULL;
        EnQueue(ziel_object, structentrynew, plusinnr, changed,
                        queuecircle);
    }
    ++i;
    ++changed;
    ziel_object = GetKantenZiel(kante, placetable, arctable);
    structentrynew = malloc(sizeof(StructEntry));
    structentrynew->canvas_id = vertexObject->v_id;
    structentrynew->count    = i;
    structentrynew->number   = (2 * outnr);
    structentrynew->next     = NULL;
    structentry = structentrynew;
    vertexObject = ziel_object;

    while (vertexObject) {
        if (vertexObject->v_id == start_canvas_id) {
            ++changed;
            popresult = DeQueue(vertexObject, structentry, queuecircle);
            vertexObject = popresult.vertexObject;
            structentry  = popresult.structentry;
            plusinnr = popresult.plusinnr;
        } else {
            innr  = CountIn(vertexObject);
            outnr = CountOut(vertexObject); 
            if (innr > 1) {
                PutMarkToPlace(vertexObject, CopyStructList(structentry),
                               plusinnr);
                allinedgesresult = AllInEdges(vertexObject, innr, structentry);
                allinedges = allinedgesresult.allinedges;
                plusinnr = allinedgesresult.plusinnr;
                if ( allinedges == 0 ) {
                    EnQueue(vertexObject,
                      CopyStructList(structentry), 0, changed, queuecircle);
                    popresult = DeQueue(vertexObject, structentry, queuecircle);
                    vertexObject = popresult.vertexObject;
                    structentry  = popresult.structentry;
                    plusinnr = popresult.plusinnr;
                    if ( (changed == popresult.changed) && (circle == 1) ) {
			 /* keine Verkuerzung moeglich und circle 
			  * einmal herum */

                        EnQueue(vertexObject, structentry,
   				        0, changed, queuecircle);
                        problemnodes = *queuecircle;
                        
			/* 
			 * Zyklus weist nicht die regulaere Struktur 
			 * auf -> NICHT UEBERSETZBAR 
			 */

                        return 0;
                    } else if ( (changed == popresult.changed) &&
                                (circle == 0) ) {

                        /* 
			 * keine Verkuerzung bis jetzt -> circle starten 
			 */

                        circle = 1;
                        ++changed;
                        if (Circle(vertexObject, placetable, arctable) == 1) {
                            /* mind. 1 Zyklus (am ersten Knoten) gefunden */
                            circle = 0;
                        }
                    } else if ( (changed != popresult.changed) &&
                                (circle == 1) ) {
                        /* circle laeuft, bisher noch kein Zyklus gefunden */
                        if (Circle(vertexObject, placetable, arctable) == 1) {
                            /* mind. 1 Zyklus (am ersten Knoten) gefunden */
                            ++changed;
                            circle = 0;
                        }
                    }
                    continue;
                } else if ( allinedges == 1 ) {
                    ++changed;
                    popresult = DeQueue(vertexObject, structentry, queuecircle);
                    vertexObject = popresult.vertexObject;
                    structentry  = popresult.structentry;
                    plusinnr = popresult.plusinnr;
                    continue;
                } else if ( allinedges == 2 ) {
                    ++changed;
                } else if ( allinedges >= 3) {
                    ++changed;
                    while (allinedges > 2) {
                        help = structentry;
                        structentry = structentry->next;
                        ClearStructElement(help);
                        --allinedges;
                    }
                }
            }
            anzahlechtekanten = AnzahlEchteKanten(vertexObject);
            if (anzahlechtekanten == 1) {
                ++changed;
                vertexObject = Get1EchtenKnoten(vertexObject,
                                                placetable, arctable);
            } else if (anzahlechtekanten > 1) {
                kante = vertexObject->out;
                ziel_object = GetKantenZiel(kante, placetable, arctable);
                i = 0; 
                while (kante->next) {
                    ++changed;
                    if ( InRepeatKanten(kante, vertexObject) == 0 ) {
                        ++i;
                        structentrynew = malloc(sizeof(StructEntry));
                        structentrynew->canvas_id = vertexObject->v_id;
                        structentrynew->count    = i;
                        structentrynew->number   = anzahlechtekanten;
                        structentrynew->next     =
                                         CopyStructList(structentry);
                        EnQueue(ziel_object, structentrynew,
                                plusinnr, changed, queuecircle);
                    }
                    kante = kante->next;
                    ziel_object = GetKantenZiel(kante, placetable,
 					        arctable);
                }
                ++changed;
                if ( InRepeatKanten(kante, vertexObject) == 1 ) {
                    popresult = DeQueue(vertexObject, structentry, queuecircle);
                    vertexObject = popresult.vertexObject;
                    structentry  = popresult.structentry;
                    plusinnr = popresult.plusinnr;
                } else {
                    structentrynew = malloc(sizeof(StructEntry));
                    structentrynew->canvas_id = vertexObject->v_id;
                    structentrynew->count    = anzahlechtekanten;
                    structentrynew->number   = anzahlechtekanten;
                    structentrynew->next     = structentry;
                    structentry = structentrynew;
                    vertexObject = ziel_object;
                }
            } else {
                printf("End vertex");
            }
        } 
    }
    ClearStructList(structentry);
    ClearQueuePtr(queuecircle);
    return 1;
}

/*  InArcIdList
 *--------------------------------------------------------------
 *  erstellt ein neues Element vom Typ 'IntList' und uebergibt
 *  diesem die Informationen des Parameters 'kante'. Das neue
 *  Element wird dann vor dem Element 'inarcs' in die Liste
 *  eingef"ugt. Hilfsfunktion von CircleRepeat.
 *--------------------------------------------------------------
 */

static IntList *InArcIdList(IntList *inarcs, EdgeList_type *kante)
{
    IntList *newarc;

    newarc = malloc(sizeof(IntList));
    newarc->id = kante->arc_id;
    newarc->next = inarcs;
    return newarc;
}

/*  CircleRepeat
 *--------------------------------------------------------------
 *  Teilfunktion des Tests auf Zyklen an einem Knoten.
 *--------------------------------------------------------------
 */

IntList *CircleRepeat (Vertex_type *vertexObject, int start_canvas_id,
                         Tcl_HashTable placetable, Tcl_HashTable arctable)
{
    QueueCircle     *queuecircle;
    QueueCircleCell *popcircleresult;
    int              wegezaehler;
    int              outnr;
    EdgeList_type   *kante=NULL; /* maybe used uninitalized? */
    int              i;
    char             key[LINE_LENGTH];
    IntList         *inarcs = NULL;
    IntList         *history = NULL;

    queuecircle = malloc(sizeof(QueueCircle));
    queuecircle->front = NULL;
    queuecircle->end = NULL;
    wegezaehler = 1;
    while (wegezaehler > 0) {
        sprintf(key, "%d", vertexObject->v_id);
        if (vertexObject->v_id == start_canvas_id) {
            inarcs = InArcIdList(inarcs, kante);
            --wegezaehler;
            ClearIntList(history);
            popcircleresult = PopCircleQueue(queuecircle);
            vertexObject = popcircleresult->vertexObject;
            history = popcircleresult->history;
	    free(popcircleresult);
            continue;
        } else if (vertexObject->type == 3) {
            ClearIntList(history);
            ClearIntList(inarcs);
            ClearQueueCircle(queuecircle);
            return NULL;
        } else if ( Tcl_FindHashEntry(&structTable, key) ) {
            /* Knoten im Normal-Graph bzw. umgebenden Zyklus */
            ClearIntList(history);
            ClearIntList(inarcs);
            ClearQueueCircle(queuecircle);
            return NULL;
        }
        outnr = CountOut(vertexObject);
        if (outnr == 1) {
            kante = vertexObject->out;
            vertexObject = GetKantenZiel(kante, placetable, arctable); 
        } else if (outnr > 1) {
            if (InHistory(history, vertexObject->v_id) == 1) {
                /* dieser Knoten wurde auf diesem Weg schon begangen
                   -> Zyklus im Zyklus -> hierfuer Ende
                */
                --wegezaehler;
                ClearIntList(history);
                popcircleresult = PopCircleQueue(queuecircle);
                vertexObject = popcircleresult->vertexObject;
                history = popcircleresult->history;
		free(popcircleresult);
                continue;
            }
            history = PutHistory(history, vertexObject->v_id);
            kante = vertexObject->out;
            vertexObject = GetKantenZiel(kante, placetable, arctable); 
            wegezaehler += (outnr-1);
            for ( i = 1; i < outnr; ++i) {
                if (vertexObject->v_id == start_canvas_id) {
                    inarcs = InArcIdList(inarcs, kante);
                    --wegezaehler;
                } else {
                    PutCircleQueue(vertexObject, CopyHistory(history),
                                   queuecircle);
                }
                kante = kante->next;
                vertexObject = GetKantenZiel(kante, placetable, arctable);
            }
        }
    }
    if (vertexObject) {
        printf("Error CircleRepeat\n");
    }
    ClearIntList(history);
    ClearQueueCircle(queuecircle);
    return inarcs;
}

/*  CopyHistory
 *--------------------------------------------------------------
 *  Kopierfunktion fuer die mitgefuehrte Historie ueber
 *  geoeffnete schon begangene Knoten. Teil der Zyklusanalyse
 *--------------------------------------------------------------
 */

IntList *CopyHistory(IntList *history)
{
    IntList *helpnew;
    IntList *historynew;
    IntList *help;

    if (history) {
        historynew = malloc(sizeof(IntList));
        historynew->id   = history->id;
        historynew->next = NULL;
        helpnew = historynew;
        history = history->next;
        while (history) {
            help = malloc(sizeof(IntList));
            help->id   = history->id;
            help->next = NULL;
            helpnew->next = help;
            helpnew = helpnew->next;
            history = history->next;
        }
    } else {
        historynew = NULL;
    }
    return historynew;
}

/*  InHistory
 *--------------------------------------------------------------
 *  Test, ob Element in Historie enthalten 
 *--------------------------------------------------------------
 */

int InHistory(IntList *history, int currentid)
{
    while (history) {
        if (history->id == currentid) {
            return 1;
        }
        history = history->next;
    }
    return 0;
}

/*  PutHistory
 *--------------------------------------------------------------
 *  erstellt ein neues Element vom Typ 'IntList' und "ubergibt
 *  diesem die Informationen des Parameters 'newid'. Das neue
 *  Element wird dann vor dem Element 'history in die Liste
 *  eingefuegt.
 *--------------------------------------------------------------
 */

IntList *PutHistory(IntList *history, int newid) 
{
    IntList *newele;

    newele = malloc(sizeof(IntList));
    newele->id = newid;
    newele->next = history;
    return newele;
}

/*  PopCircleQueue
 *--------------------------------------------------------------
 *  Entnahme des ersten Elementes der uebergebenen Schlange und
 *  Rueckgabe dieses Elementes. Funktion im Einsatz waehrend
 *  Zyklusanalyse
 *--------------------------------------------------------------
 */

QueueCircleCell* PopCircleQueue(QueueCircle *queuecircle)
{
    QueueCircleCell *help;

    if (!queuecircle->front) {
	help=(QueueCircleCell*)MYmalloc(sizeof(QueueCircleCell));
        help->vertexObject = NULL;
        help->history = NULL;
    } else {
        help = queuecircle->front;
        if (queuecircle->front->next) {
            queuecircle->front = queuecircle->front->next;
        } else {
            queuecircle->front = NULL;
            queuecircle->end = NULL;
        }            
    }
    return help;
}

/*  PutCircleQueue
 *--------------------------------------------------------------
 *  Anfuegen eines Elementes an die mitgegebene Schlange.  
 *  Funktion im Einsatz waehrend der Zyklenanalyse
 *--------------------------------------------------------------
 */

void PutCircleQueue(Vertex_type *vertexObject, IntList *history,
                    QueueCircle *queuecircle)
{
    QueueCircleCell *queuecell;

    queuecell = malloc(sizeof(QueueCircleCell));
    queuecell->vertexObject = vertexObject;
    queuecell->history = history;
    queuecell->next = NULL;
    if (queuecircle->end) {
        queuecircle->end->next = queuecell;
        queuecircle->end = queuecell;
    } else {
        queuecircle->front = queuecell;
        queuecircle->end = queuecell;
    }
}

/*  DeQueue
 *--------------------------------------------------------------
 *  Entnahme des ersten Elememtes der global deklarierten 
 *  Schlange queue und Rueckgabe dieses Elementes. 
 *--------------------------------------------------------------
 */

PopResult DeQueue(Vertex_type *vertexObject, StructEntry *structentry,
		  Queue *queue)
{
    QueueCell *help;
    PopResult  popresult; 

    ClearStructList(structentry);
    if (!queue->front) {
        popresult.vertexObject = NULL;
        popresult.structentry  = NULL;
        popresult.plusinnr = 0;
        popresult.changed = 0;
    } else {
        popresult.vertexObject = queue->front->vertexObject;
        popresult.structentry  = queue->front->structentry;
        popresult.plusinnr = queue->front->plusinnr;
        popresult.changed = queue->front->changed;
        help = queue->front;
        if (queue->front->next) {
            queue->front = queue->front->next;
        } else {
            queue->front = NULL;
            queue->end = NULL;
        }            
        free(help);
    }
    return popresult;
}

/*  EnQueue
 *--------------------------------------------------------------
 *  Anfuegen eines Elementes an die global deklarierte Schlange 
 *  queue.
 *--------------------------------------------------------------
 */

void EnQueue(Vertex_type *vertexObject, StructEntry *structentry,
              int plusinnr, int changed, Queue *queue)
{
    QueueCell *queuecell;

    queuecell = malloc(sizeof(QueueCell));
    queuecell->vertexObject = vertexObject;
    queuecell->structentry = structentry;
    queuecell->plusinnr = plusinnr;
    queuecell->changed = changed;
    queuecell->next = NULL;
    if (queue->end) {
        queue->end->next = queuecell;
        queue->end = queuecell;
    } else {
        queue->front = queuecell;
        queue->end = queuecell;
    }
}

/*  PutMarkToPlace
 *--------------------------------------------------------------
 *  Funktion zum Ablegen von Weg-Informationen an dem aktuellen 
 *  Knoten, wenn nicht schon vorhanden. Immer an Knoten, an 
 *  denen mehrere Kanten eingehen.
 *--------------------------------------------------------------
 */

void PutMarkToPlace(Vertex_type *vertexObject, StructEntry *structentry,
                    int  plusinnr)
{
    Tcl_HashEntry *struct_entryPtr;
    PlaceMarkList *instructentry;
    PlaceMarkList *help;
    char           key[LINE_LENGTH];
    PlaceMarkList *placemarklist;
    StructEntry   *deeper1;
    StructEntry   *deeper2;
    int            goon;
    int            isok;

    sprintf(key, "%d", vertexObject->v_id);
    struct_entryPtr = Tcl_CreateHashEntry(&structTable, key, &isok);
    if( isok ) {
        placemarklist = malloc(sizeof(PlaceMarkList));
        placemarklist->done = 0;
        placemarklist->plusinnr = plusinnr;
        placemarklist->united = 0;
        placemarklist->branchinfo = 1;
        placemarklist->repeatedges = NULL;
        placemarklist->structentry = structentry;
        placemarklist->next = NULL;
        Tcl_SetHashValue(struct_entryPtr, placemarklist);
    } else {
        struct_entryPtr = Tcl_FindHashEntry(&structTable, key);
        instructentry = (PlaceMarkList *) Tcl_GetHashValue(struct_entryPtr);
        help = instructentry;
        if (help->done == 1) {
            ClearStructList(structentry);
            return;
        }

        /*
         * pruefe, ob derselbe Eintrag schon vorhanden 
         */

        if ( (help->structentry->canvas_id == structentry->canvas_id) &&
             (help->structentry->count == structentry->count) ) {
            deeper1 = help->structentry->next;
            deeper2 = structentry->next;
            goon = 1;
            while ( deeper1 && deeper2 && (goon == 1) ) {
                if ( (deeper1->canvas_id == deeper2->canvas_id) &&
                     (deeper1->count == deeper2->count) ) {
                    deeper1 = deeper1->next;
                    deeper2 = deeper2->next;
                } else {
                    goon = 0;
                }
            }
            if ( (!deeper1) && (!deeper2) ) {
                ClearStructList(structentry); 
               return;
            }
        }
        while (help->next) {
            help = help->next;
            if ( (help->structentry->canvas_id == structentry->canvas_id)
               && (help->structentry->count == structentry->count) ) {
                deeper1 = help->structentry->next;
                deeper2 = structentry->next;
                goon = 1;
                while ( deeper1 && deeper2 && goon ) {
                    if ( (deeper1->canvas_id == deeper2->canvas_id) &&
                         (deeper1->count == deeper2->count) ) {
                        deeper1 = deeper1->next;
                        deeper2 = deeper2->next;
                    } else {
                        goon = 0;
                    }
                }
                if ( (!deeper1) && (!deeper2) ) {
                    ClearStructList(structentry);
                    return;
                }
            }
        }

        /* 
	 * Eintrag noch nicht vorhanden -> eintragen 
	 */

        placemarklist = malloc(sizeof(PlaceMarkList));
        placemarklist->done = 0;
        placemarklist->plusinnr = help->plusinnr;
        placemarklist->united = 0;
        placemarklist->branchinfo = 1;
        placemarklist->repeatedges = NULL;
        placemarklist->structentry = structentry;
        placemarklist->next = NULL;
        help->next = placemarklist;
        /* addiere plusinnr auf */
        help = instructentry;
        while (help) {
            help->plusinnr += plusinnr;
            help = help->next;
        }
    }
}

/*  AllInEdges
 *--------------------------------------------------------------
 *  Funktion zum Test, ob alle erwarteten Prozesse an einem 
 *  Knoten mit mehreren eingehenden Kanten schon eingetroffen
 *  sind und im positiven Falle Test auf 
 *  Vereinigungsmoeglichkeit von Prozessen. 
 *--------------------------------------------------------------
 */

AllInEdgesResult AllInEdges(Vertex_type *vertexObject, int innr,
                            StructEntry *structentry)
{
    AllInEdgesResult allinedgesresult;
    PlaceMarkList   *inentry;
    PlaceMarkList   *help;
    PlaceMarkList   *help2;
    PlaceMarkList   *lastentry;
    PlaceMarkList   *placemarklist;
    StructEntry     *structentryhelp;
    int              plusinnr = 0;
    int              zusammengefasste;
    int              counter;
    int              number = 0;   
    int              flag = 0;
    int              returnwert;

   /* 
    *  branchinfo == -1 : Schleife schliesst, 
    *                     aber noch nicht abgearbeitet,
    *  branchinfo ==  0 : Schleife schliesst und schon abgearbeitet,
    *  branchinfo  >  0 : Schleife schliesst nicht.
    *  fuer Zyklus an Einfach-Startknoten : branchinfo == number == 2.
    */

    inentry = GetPlaceMarkList(vertexObject);
    help = inentry;
    if (inentry->done == 1) {
        while (help) {    
            if ((help->branchinfo <= 0) &&
                (help->structentry->canvas_id == structentry->canvas_id)) {
                if (help->branchinfo == 0) {
                   if (IdentStruct(help->structentry, structentry) == 1) {

                        /* 
			 * Eintraege abgearbeitet und Schleife schliet
			 * und schon abgearbeitet -> Pop
                         */

                        allinedgesresult.allinedges = 1;
                        allinedgesresult.plusinnr = 0;
                        return allinedgesresult;
                    }
                } else {
                    if (IdentStruct(help->structentry, structentry) == 1) {
                        /*
			 * Eintraege abgearbeitet und Schleife schliet,
			 * aber diese noch nicht abgearbeitet -> 
			 * weiter mit returnwert-2 Verkuerzungen, 
                         * plusinnr muss hier immer 0 sein
                        */

                        flag = 0;
                        structentryhelp = structentry;
                        returnwert = 2;
                        while ( (flag == 0) && structentryhelp ) {

                        /*
			 * bis in die Tiefe vergleichen und bei 
			 * Gleichheit alle mit 0 markieren 
			 */

                            flag = 1;
                            help = inentry;
                            while (help) {    
                                if ( (help->branchinfo == -1) &&
                                (AlmostIdentStruct(help->structentry,
                                 structentryhelp) == 1) ){
                                    ++returnwert;
                                    flag = 0;
                                    help->branchinfo = 0;

                                    /* 
				     * auch alle anderen der Schleife mit 
				     * 0 markieren 
                                     */

                                    help = help->next;
                                    while (help) {
                                        if ( (help->branchinfo == -1)
                                && (AlmostIdentStruct(help->structentry,
                                        structentryhelp) == 1) ) {
                                            help->branchinfo = 0;
                                        }
                                        help = help->next;
                                    }
                                    break;
                                }            
                                help = help->next;
                            }
                            structentryhelp = structentryhelp->next;
                        }
                        allinedgesresult.allinedges = returnwert;
                        allinedgesresult.plusinnr = 0;
                        return allinedgesresult;
                    }
                }
            }            
            help = help->next;
            ++number;
        }

        /*
	 * Eintraege abgearbeitet, Schleife schliesst nicht -> 
	 * weiter 
	 */

        allinedgesresult.allinedges = 2;
        allinedgesresult.plusinnr = 0;
        return allinedgesresult;
    } else {
        /* 
	 * noch nicht abgearbeitet 
	 */

        while (help) {    
            help = help->next;
            ++number;
        }
        if (number < (innr + inentry->plusinnr) ) {

            /* 
	     * noch nicht alle Eintraege vorhanden -> warten mit Put/Pop
             */

            allinedgesresult.allinedges = 0;
            allinedgesresult.plusinnr = 0;
            return allinedgesresult;
        }

        /* 
	 * alle Eingaenge vorhanden, jetzt verkuerzen und evtl. weiter
         * verkuerzen spaeter in Prozedur 
         */

        while (flag == 0) {
            /*
	     * solange bis nach einem Komplett-Durchlauf 
	     * keine Verkuerzung moegl.
             */ 

            flag = 1;
            help = inentry;
            while (help) {
                /*
		 * jeweils von vorger. start alle Nachfolger
	         * ueberpruefen
		 */

                if (help->branchinfo == -1) {
                    help = help->next;
                    continue;
                }
                help2 = help->next;
                counter = help->structentry->number - 1;
                zusammengefasste = 0;
                zusammengefasste += help->united;
                while (help2) {
                    if (help2->branchinfo != -1) {
                        if ( AlmostIdentStruct(help->structentry,
                                               help2->structentry) == 1) {
                            --counter;
                            zusammengefasste += help2->united;
                        }
                    }
                    help2 = help2->next;
                }
                if (counter == 0) {

                    /* 
		     * Schleife schliesst, count mit -1 markieren und 
                     * verkuerztes Element anhaengen, wenn nicht == NULL
                     */

                    plusinnr += (1 - zusammengefasste);

                    /* gehe wieder durch und setze count auf -1 */
                    help->branchinfo = -1;

                    /* aehnlich oben -> Prozedur */
                    help2 = help->next;
                    while (help2) {
                        if (help2->branchinfo != -1) {
                            if ( AlmostIdentStruct(help->structentry,
                                              help2->structentry) == 1) {
                                help2->branchinfo = -1;
                            }
                        }
                        help2 = help2->next;
                    }
                    if (help->structentry->next) {
                        lastentry = help;
                        while (lastentry->next) {
                            lastentry = lastentry->next;
                        }               
                        placemarklist = malloc(sizeof(PlaceMarkList));
                        placemarklist->done = 0;
                        placemarklist->plusinnr = lastentry->plusinnr;
                        placemarklist->united = 1;
                        placemarklist->branchinfo = 1;
                        placemarklist->repeatedges = NULL;
                        placemarklist->structentry =
                        CopyStructList(help->structentry->next);
                        placemarklist->next = NULL;
                        lastentry->next = placemarklist;
                        flag = 0;
                    }
                }
                help = help->next;
                /* vorgaenger gleich aber count unterschiedlich ! */
            }
        }
        help = inentry;
        while (help) {
            help->done = 1;
            help = help->next;
        }

        /*
	 * jetzt fuer den konkreten Fall checken, ob und wenn ja welche 
         * Verkuerzung moeglich (Tiefe)
         */

        flag = 0;
        structentryhelp = structentry;
        returnwert = 2;
        while ( (flag == 0) && structentryhelp ) {

        /* 
	 * bis in die Tiefe vergleichen und bei Gleichheit 
	 * alle mit 0 markieren 
         */
            flag = 1;
            help = inentry;
            while (help) {    
                if ( (help->branchinfo == -1) &&
                    (AlmostIdentStruct(help->structentry,
                                       structentryhelp) == 1) ) {
                    ++returnwert;
                    flag = 0;
                    help->branchinfo = 0;

                    /* 
		     * auch alle anderen der Schleife mit 0 markieren 
		     */

                    help = help->next;
                    while (help) {
                        if ( (help->branchinfo == -1) &&
                        (AlmostIdentStruct(help->structentry,
                                           structentryhelp) == 1) ){
                            help->branchinfo = 0;
                        }
                        help = help->next;
                    }
                    break;
                }            
                help = help->next;
            }
            structentryhelp = structentryhelp->next;
        }

        /* 
	 * returnwert ist 2, falls keine Verkuerzung moeglich -> weiter 
         * oder >= 3 und gibt mit -2 die Anzahl der Verkuerzungen an
         */

        /* 
	 * plusinnr hier entspricht der Anzahl der auslaufenden 
	 * Wege durch zusammengefasste Schleifen, jetzt die nicht 
	 * zusammenfassbaren Kanten dazuaddieren und 1 subtrahieren
         */

        help = inentry;
        while (help) {
            if ( (help->branchinfo > 0) && (help->united == 0) ){
                ++plusinnr;
            }
            help = help->next;
        }
        --plusinnr;
        allinedgesresult.allinedges = returnwert;
        allinedgesresult.plusinnr = plusinnr;
        return allinedgesresult;
    }
    return allinedgesresult;
}

/*  IdentStruct
 *--------------------------------------------------------------
 *  Ueberprueft uebergebene Elemente vom Typ StructEntry auf 
 *  Uebereinstimmung in canvas_id und count
 *--------------------------------------------------------------
 */

int IdentStruct(StructEntry *structentry1, StructEntry *structentry2)
{
    while ( structentry1 && structentry2 ) {
        if (structentry1->canvas_id != structentry2->canvas_id) {
            return 0;
        }
        if (structentry1->count != structentry2->count) {
            return 0;
        }
        if (structentry1->number != structentry2->number) {
            return 0;
        }
        structentry1 = structentry1->next;
        structentry2 = structentry2->next;
    }
    if ( (!structentry1) && (!structentry2) ) {
        return 1;
    }
    return 0;
}

/*  AlmostIdentStruct
 *--------------------------------------------------------------
 *  Ueberprueft uebergebene Elemente vom Typ StructEntry auf 
 *  Uebereinstimmung in canvas_id und Unterschied in letztem
 *  count und Gleichheit aller davorliegenden count
 *--------------------------------------------------------------
 */

int AlmostIdentStruct (StructEntry *structentry1, StructEntry *structentry2)
{
    if ( (!structentry1) && (!structentry2) ) {
        printf("Error in AlmostIdentStruct!\n");
        return 1;
    }
    if ( (!structentry1) || (!structentry2) ) {
        return 0;
    }
    if (structentry1->canvas_id != structentry2->canvas_id) {
        return 0;
    }
    if (structentry1->count == structentry2->count) {
        return 0;
    }
    if (structentry1->number != structentry2->number) {
        return 0;
    }
    structentry1 = structentry1->next;
    structentry2 = structentry2->next;

    while ( structentry1 && structentry2 ) {
        if (structentry1->canvas_id != structentry2->canvas_id) {
            return 0;
        }
        if (structentry1->count != structentry2->count) {
            return 0;
        }
        if (structentry1->number != structentry2->number) {
            return 0;
        }
        structentry1 = structentry1->next;
        structentry2 = structentry2->next;
    }
    if ( (!structentry1) && (!structentry2) ) {
        return 1;
    }
    return 0;
}

/*  CopyStructList
 *--------------------------------------------------------------
 *   Kopiert eine uebergebene List vom Typ StructEntry
 *--------------------------------------------------------------
 */

StructEntry *CopyStructList( StructEntry *structentry) 
{
    StructEntry *helpnew;
    StructEntry *structentrynew;
    StructEntry *help;

    if (structentry) {
        structentrynew = malloc(sizeof(StructEntry));
        structentrynew->canvas_id = structentry->canvas_id;
        structentrynew->count    = structentry->count;
        structentrynew->number   = structentry->number;
        structentrynew->next     = NULL;
        helpnew = structentrynew;
        structentry = structentry->next;
        while (structentry) {
            help = malloc(sizeof(StructEntry));
            help->canvas_id = structentry->canvas_id;
            help->count     = structentry->count;
            help->number    = structentry->number;
            help->next      = NULL;
            helpnew->next = help;
            helpnew = helpnew->next;
            structentry = structentry->next;
        }
    } else {
        structentrynew = NULL;
    }
    return structentrynew;
}

/*  CopyStructElement
 *--------------------------------------------------------------
 *   Kopiert ein uebergebenes Element vom Typ StructEntry
 *--------------------------------------------------------------
 */

StructEntry *CopyStructElement( StructEntry *structentry) 
{
    StructEntry *structentrynew;

    structentrynew = malloc(sizeof(StructEntry));
    structentrynew->canvas_id = structentry->canvas_id;
    structentrynew->count    = structentry->count;
    structentrynew->number   = structentry->number;
    structentrynew->next     = NULL;
    return structentrynew;
}

/*  CountIn
 *--------------------------------------------------------------
 *  zaehlt alle eingehenden Kanten eines Knotens und gibt 
 *  deren Gesamtanzahl zurueck.
 *--------------------------------------------------------------
 */

int CountIn (Vertex_type *vertexObject)
{
    EdgeList_type *kante;
    int      i = 0;

    for ( kante = vertexObject->in; kante; kante = kante->next) {
        ++i;
    }
    return i;
}

/*  CountOut
 *--------------------------------------------------------------
 *  zaehlt alle ausgehenden Kanten eines Knotens und gibt 
 *  deren Gesamtanzahl zurueck.
 *--------------------------------------------------------------
 */

int CountOut (Vertex_type *vertexObject)
{
    EdgeList_type *kante;
    int      i = 0;

    for ( kante = vertexObject->out; kante; kante = kante->next) {
        ++i;
    }
    return i;
}

/*  Get1EchtenKnoten
 *--------------------------------------------------------------
 *  Gibt den Nachfolgerknoten eines Knotens zurueck 
 *--------------------------------------------------------------
 */

Vertex_type *Get1EchtenKnoten(Vertex_type *vertexObject,
                              Tcl_HashTable placetable,
                              Tcl_HashTable arctable)
{
    EdgeList_type *kante;    
    char        arc_kante[LINE_LENGTH];
    Edge_type   *kante_object;
    int         kante_ziel;
    char        knoten_ziel[LINE_LENGTH];
    Vertex_type *ziel_object;

    for (kante = vertexObject->out; kante; kante = kante->next) {
        if (InRepeatKanten(kante, vertexObject) == 1) {
            continue;
        }
        sprintf(arc_kante, "arc%d", kante->arc_id);
        kante_object = GetKanteObject (arctable, arc_kante);
        kante_ziel = kante_object->end;
        sprintf(knoten_ziel, "knoten%d", kante_ziel);
        ziel_object = GetPlaceObject(placetable, knoten_ziel);
        return ziel_object;
    }
    printf("Error in Get1EchtenKnoten!\n");
    return NULL;
}

/*  GetStartState
 *--------------------------------------------------------------
 *  durchsucht den Hash-Table 'placetable' nach einem 
 *  Startknoten und gibt, falls dieser gefunden wurde,
 *  einen Zeiger auf dessen Struktur zur"uck.
 *--------------------------------------------------------------
 */

Vertex_type *GetStartState(Tcl_HashTable placetable) 
{
    Tcl_HashEntry *place_entryPtr;
    Vertex_type    *vertexObject;
    Tcl_HashSearch search;

    for ( place_entryPtr = Tcl_FirstHashEntry (&placetable, &search);
          place_entryPtr; place_entryPtr = Tcl_NextHashEntry(&search) ) {
        vertexObject = Tcl_GetHashValue(place_entryPtr);
        if (vertexObject->type == 1) 
            return vertexObject;
    }
    return NULL;
}

/*  AnzahlEchteKanten
 *--------------------------------------------------------------
 *  Berechnet die Anzahl der ausgehenden Kanten fuer einen 
 *  Knoten, wobei Repeatkanten, d.h. Kanten mit denen ein 
 *  Zyklus beginnt, nicht mitgerechnet werden.
 *--------------------------------------------------------------
 */

int AnzahlEchteKanten(Vertex_type *vertexObject)
{
    EdgeList_type  *repeatedges;
    EdgeList_type  *kante;
    int            number = 0;
 
    for (kante = vertexObject->out; kante; kante = kante->next) {
        ++number;
    }
    repeatedges = GetRepeatKanten(vertexObject);
    if (!repeatedges) {
        return number;
    }
    while (repeatedges) {
        --number;
        repeatedges = repeatedges->next;
    }
    return number;
}    

/*  Get1EchteKante
 *--------------------------------------------------------------
 *  Gibt als Ergebnis die einzige von einem Knoten ausgehende 
 *  Kante, die keine Repeatkante ist.
 *--------------------------------------------------------------
 */

EdgeList_type *Get1EchteKante(Vertex_type *vertexObject)
{
    EdgeList_type *repeatedges;
    EdgeList_type *kante;
    int      gleich;
 
    repeatedges = GetRepeatKanten(vertexObject);
    if (!repeatedges) {
        return vertexObject->out;
    }
    for (kante = vertexObject->out; kante; kante = kante->next) {
        gleich = 0;
        while (repeatedges) {
            if (repeatedges->arc_id == kante->arc_id) {
                gleich = 1;
                break;
            }
            repeatedges = repeatedges->next;
        }
        if (gleich == 0) {
            return kante;
        }
    }
    return NULL;
}

/*  GetPlaceMarkList
 *--------------------------------------------------------------
 *  Gibt die mit einem Knoten assoziierte Liste vom Typ
 *  PlaceMarkList zurueck
 *--------------------------------------------------------------
 */

PlaceMarkList *GetPlaceMarkList(Vertex_type *vertexObject)
{
    char           key[LINE_LENGTH];
    Tcl_HashEntry *struct_entry; 

    sprintf(key, "%d", vertexObject->v_id);
    struct_entry = Tcl_FindHashEntry(&structTable, key);
    if (!struct_entry) {
        return NULL;
    }
    return ( (PlaceMarkList *) Tcl_GetHashValue(struct_entry) );
}

/*  GetRepeatKanten
 *--------------------------------------------------------------
 *  Gibt die Liste der Repeatkanten eines Knotens zurueck 
 *--------------------------------------------------------------
 */

EdgeList_type *GetRepeatKanten(Vertex_type *vertexObject)
{
    char           key[LINE_LENGTH];
    Tcl_HashEntry *struct_entry; 

    sprintf(key, "%d", vertexObject->v_id);
    struct_entry = Tcl_FindHashEntry(&structTable, key);
    if (!struct_entry) {
        return NULL;
    }
    return ( (PlaceMarkList *) Tcl_GetHashValue(struct_entry) )->repeatedges;
}

/*  BranchClose
 *--------------------------------------------------------------
 *  Ueberprueft aufgrund der Informationen des ersten Laufs, ob
 *  sich die zuletzt geoeffnete Alternative des aktuellen
 *  Prozesses schliesst.
 *--------------------------------------------------------------
 */

int BranchClose(Vertex_type *vertexObject, StructEntry *structentry) 
{
    PlaceMarkList *inentry;
    PlaceMarkList *help;

    if ( (vertexObject->type == 3) && (nrrealfinalstates == 1) ) {
        return 1;
    }
    if (!structentry) {
        return 0;
    }

    inentry = GetPlaceMarkList(vertexObject);
    help = inentry;
    while (help) {    
        if ((help->branchinfo == 0) &&
            (help->structentry->canvas_id == structentry->canvas_id)) {
            /* Schleife schliet und fuer diese Schleife schon abgearbeitet */
            return 1;
        }            
        help = help->next;
    }
    return 0;
}

/*  GetEndPlace
 *--------------------------------------------------------------
 *  durchsucht den Hash-Table 'placetable' nach einem 
 *  Endknoten und gibt, falls dieser gefunden wurde,
 *  einen Zeiger auf dessen Struktur zur"uck.
 *--------------------------------------------------------------
 */

Vertex_type *GetEndPlace(Tcl_HashTable placetable) 
{ 
    Tcl_HashEntry *place_entryPtr;
    Vertex_type    *vertexObject=NULL;
    Tcl_HashSearch search;

    for ( place_entryPtr = Tcl_FirstHashEntry (&placetable, &search);
          place_entryPtr; place_entryPtr = Tcl_NextHashEntry(&search) ) {
        vertexObject = Tcl_GetHashValue(place_entryPtr);
        if (vertexObject->type == 3) 
            return vertexObject;
    }
    printf("Error : Automaton without end vertex!\n");
    return vertexObject;
}

/*  GetKanteObject
 *--------------------------------------------------------------
 *  durchsucht den Hash-Table 'arctable' nach einer 
 *  Kante und gibt, falls sie gefunden wurde,
 *  einen Zeiger auf deren Struktur zurueck.
 *--------------------------------------------------------------
 */

Edge_type *GetKanteObject (Tcl_HashTable arctable, char *arc_kante) 
{
    Tcl_HashEntry *kante_entry;  

    kante_entry = Tcl_FindHashEntry(&arctable, arc_kante);
    return (Edge_type *) Tcl_GetHashValue(kante_entry);
}

/*  GetPlaceObject
 *--------------------------------------------------------------
 *  durchsucht den Hash-Table 'placetable' nach einem 
 *  Knoten mit dem namen 'knoten_ziel' und gibt, falls 
 *  dieser gefunden wurde, einen Zeiger auf dessen
 *  Struktur zurueck.
 *--------------------------------------------------------------
 */

Vertex_type *GetPlaceObject (Tcl_HashTable placetable, char *knoten_ziel) 
{
    Tcl_HashEntry *ziel_entry;

    ziel_entry = Tcl_FindHashEntry(&placetable, knoten_ziel);
    return (Vertex_type *) Tcl_GetHashValue(ziel_entry);
}

/*  GetChanObject
 *--------------------------------------------------------------
 *  durchsucht den Hash-Table 'channelTable' nach dem
 *  Object mit dem Namen 'chan_label' und gibt, falls 
 *  dieses gefunden wurde, einen Zeiger darauf zurueck.
 *--------------------------------------------------------------
 */

int *GetChanObject (char *chan_label) 
{
    Tcl_HashEntry *chan_entry;
  
    chan_entry = Tcl_FindHashEntry(&channelTable, chan_label);
    return \
    &(((ChannelTableEntry *) Tcl_GetHashValue(chan_entry))->channel);
}

/*  GetTextObject
 *--------------------------------------------------------------
 *  durchsucht den Hash-Table 'inputTable' nach dem
 *  Object mit dem Namen 'text_label' und gibt, falls 
 *  dieses gefunden wurde, einen Zeiger darauf zurueck.
 *--------------------------------------------------------------
 */

int *GetTextObject (char *text_label) 
{
    Tcl_HashEntry *text_entry;

    text_entry = Tcl_FindHashEntry(&inputTable, text_label);
    return (int *) Tcl_GetHashValue(text_entry);
}

/*  NrStartStates
 *--------------------------------------------------------------
 *  zaehlt die Anzahl der Startknoten im Hash-Table 
 *  'placetable'. 
 *--------------------------------------------------------------
 */

int NrStartStates(Tcl_HashTable placetable) 
{ 
    Tcl_HashEntry *place_entryPtr;
    Vertex_type    *vertexObject;
    Tcl_HashSearch search;
    int            number = 0;

    for ( place_entryPtr = Tcl_FirstHashEntry (&placetable, &search);
          place_entryPtr; place_entryPtr = Tcl_NextHashEntry(&search) ) {
        vertexObject = Tcl_GetHashValue(place_entryPtr);
        if (vertexObject->type == 1) 
            ++number;
    }
    return number;
}

/*  NrRealStartStates
 *--------------------------------------------------------------
 *  Berechnet die Anzahl der Startknoten, die Initialknoten 
 *  fuer mindestens eine Kante sind.
 *--------------------------------------------------------------
 */

int NrRealStartStates(Tcl_HashTable placetable) 
{ 
    Tcl_HashEntry *place_entryPtr;
    Vertex_type    *vertexObject;
    Tcl_HashSearch search;
    int            number = 0;

    for ( place_entryPtr = Tcl_FirstHashEntry (&placetable, &search);
          place_entryPtr; place_entryPtr = Tcl_NextHashEntry(&search) ) {
        vertexObject = Tcl_GetHashValue(place_entryPtr);
        if ((vertexObject->type == 1) && (CountOut(vertexObject) > 0)) {
          ++number;
        }
    }
    return number;
}

/*  NrRealFinalStates
 *--------------------------------------------------------------
 *  Berechnet die Anzahl der Endknoten, die Terminalknoten 
 *  fuer mindestens eine Kante sind.
 *--------------------------------------------------------------
 */

int NrRealFinalStates(Tcl_HashTable placetable) 
{ 
    Tcl_HashEntry *place_entryPtr;
    Vertex_type    *vertexObject;
    Tcl_HashSearch search;
    int            number = 0;

    for ( place_entryPtr = Tcl_FirstHashEntry (&placetable, &search);
          place_entryPtr; place_entryPtr = Tcl_NextHashEntry(&search) ) {
        vertexObject = Tcl_GetHashValue(place_entryPtr);
        if ((vertexObject->type == 3) && (CountIn(vertexObject) > 0)) {
          ++number;
        }
    }
    return number;
}

/*  NrStates
 *--------------------------------------------------------------
 *  Berechnet die Anzahl der Knoten eines Automaten und  
 *  numeriert diese eindeutig.
 *--------------------------------------------------------------
 */

int NrStates(Tcl_HashTable placetable, int nrstartstates) 
{
    Tcl_HashEntry *place_entryPtr;
    Vertex_type    *vertexObject;
    Tcl_HashSearch search;
    int            number = 0;
    int            start = 0;

    if (nrstartstates < 2) {
        for ( place_entryPtr = Tcl_FirstHashEntry (&placetable, &search);
              place_entryPtr; place_entryPtr = Tcl_NextHashEntry(&search) ) {
            ++ number;
            vertexObject = Tcl_GetHashValue(place_entryPtr);
            if (vertexObject->type == 1) {
                vertexObject->compile_id = 0; 
                start = 1;
            } else {
                vertexObject->compile_id = number - start;
            }
        }
        return (number + 1 - start);
    } else {
        number = nrstartstates;
        for ( place_entryPtr = Tcl_FirstHashEntry (&placetable, &search);
              place_entryPtr; place_entryPtr = Tcl_NextHashEntry(&search) ) {
            vertexObject = Tcl_GetHashValue(place_entryPtr);
            if (vertexObject->type == 1) {
                vertexObject->compile_id = nrstartstates--; 
            } else {
                ++number;
                vertexObject->compile_id = number - start;
            }
        }
        return (number + 1);
    }
}

/*  
 *##############################################################
 *--------------------------------------------------------------
 *  Die folgenden Prozeduren l"oschen die verwendeten Hash-Table
 *  bzw. Listen und geben den gesamten belegten Speicherplatz 
 *  frei.
 *--------------------------------------------------------------
 *##############################################################
 */

void ClearSymbolTables (void)
{
    Tcl_HashEntry     *entryPtr;
    Tcl_HashSearch     search;
    ChannelTableEntry *symbol_object;
    int               *symbol_objectinput;

    for ( entryPtr = Tcl_FirstHashEntry (&channelTable, &search);
	  entryPtr; entryPtr = Tcl_NextHashEntry(&search) ) {
	    symbol_object = (ChannelTableEntry *) Tcl_GetHashValue(entryPtr);
            ClearIntList (symbol_object->inputlist);
	    free(symbol_object);
    }
    Tcl_DeleteHashTable(&channelTable);
    for ( entryPtr = Tcl_FirstHashEntry (&inputTable, &search);
	  entryPtr; entryPtr = Tcl_NextHashEntry(&search) ) {
	    symbol_objectinput = (int *) Tcl_GetHashValue( entryPtr );
	    free( symbol_objectinput );
    }
    Tcl_DeleteHashTable(&inputTable);
} 

void ClearIntList (IntList *intlist)
{
    IntList *help;

    while (intlist) {
        help = intlist;
        intlist = intlist->next;
        free(help);
    }
}

void ClearStructTable(void) 
{
    Tcl_HashEntry  *entryPtr;
    Tcl_HashSearch  search;
    PlaceMarkList  *struct_object;

    for ( entryPtr = Tcl_FirstHashEntry (&structTable, &search);
	  entryPtr; entryPtr = Tcl_NextHashEntry(&search) ) {
        struct_object = (PlaceMarkList *) Tcl_GetHashValue(entryPtr);
        ClearArcList (struct_object->repeatedges);
        ClearStructList(struct_object->structentry);
	free (struct_object);
    }
    Tcl_DeleteHashTable(&structTable);
}

void ClearArcList(EdgeList_type *arclist)
{
    EdgeList_type *help;

    while (arclist) {
        help = arclist;
        arclist = arclist->next;
        free(help);
    }
}

void ClearQueue(Queue queuelocal)
{
    QueueCell *queuecell;
    QueueCell *help;

    queuecell = queuelocal.front;
    while (queuecell) {
        help = queuecell;
        queuecell = queuecell->next;
        ClearStructList(help->structentry);
        free(help);
    }
}

void ClearQueuePtr(Queue *queuelocal)
{
    ClearQueue(*queuelocal);
    free(queuelocal);
}

void ClearQueueCircle(QueueCircle *queuecircle)
{
    QueueCircleCell *queuecell;
    QueueCircleCell *help;

    queuecell = queuecircle->front;
    while (queuecell) {
        help = queuecell;
        queuecell = queuecell->next;
        ClearIntList(help->history);
        free(help);
    }
    free(queuecircle);
}

void ClearStructList(StructEntry *structlist)
{
    StructEntry *help;

    while (structlist) {
        help = structlist;
        structlist = structlist->next;
        free(help);
   }

}

void ClearStructElement (StructEntry *structelement)
{
    if (structelement) {
        free(structelement);
    }
}

