/*
###########################################################################
###########################################################################
##                                                                       ##
##  Datei   :    `pfaSupport.c'                                          ##
##               --------------                                          ##
##                                                                       ##
##  Inhalt  :    Hilfsfunktionen, die von mehreren Befehlsfunktionen     ##
##               aus den anderen Files ben"otigt werden.                 ##
##                                                                       ##
###########################################################################
###########################################################################
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "pfa.h"

/*
 ##########################################################################
 #                                                                        #
 #  Deklaration der lokalen Funktionen                                    #
 #                                                                        #
 ##########################################################################
 */

EdgeList_type  *backInsert(EdgeList_type  *first,\
		    Edge_type *edgeObject, Vertex_type *other_vertex);
EdgeList_type  *deleteObject(int daten,EdgeList_type  *first);

/*
 ##########################################################################
 #                                                                        #
 #  Funktionen zum Setzen von Knoten, Kanten und Kommentaren auf der      #
 #  Zeichenflaeche.                                                       #
 #                                                                        #
 ##########################################################################
 */

/*  printVertex
 *--------------------------------------------------------------
 *   
 *  setzt Start-, End-, und innere Knoten auf der Canvas
 *  und gibt die Canvas-Id's des Knotens und des Knoten-
 *  textes zur"uck.
 *
 *--------------------------------------------------------------
 */
int printVertex( Tcl_Interp *interp, Vertex_type *vertexObject,\
		 int *vertex_id, int *text_id)
{
    char command[LINE_LENGTH];
    int  v_cid;
    int  t_cid;

    if (1 == vertexObject->type) {
	
	/* 
	 *  Es wird ein Start-Knoten auf der Canvas gezeichnet ...
	 */

	sprintf(command, "MkStartVertex %d %d",\
		vertexObject->center.x, vertexObject->center.y);
	if (Tcl_GlobalEval (interp, command) != TCL_OK) {
	    return TCL_ERROR;
	}
	  
	/*
	 *  ... MkStartVertex gibt die neue Canvas-Id des Knotens zurueck,
	 *  damit sie spaeter in die Struktur des Knotens eingetragen
	 *  werden kann.
	 */ 

	if (Tcl_GetIntFromObj(interp, Tcl_GetObjResult(interp), &v_cid) != TCL_OK) {
	    return TCL_ERROR;
	}

	/* 
	 *  Dann wird der zugehoerige Text gezeichnet ...
	 */

	sprintf (command, "MkStartText %d %d %d \"%s\"",\
		 v_cid,\
		 vertexObject->center.x,  vertexObject->center.y,\
		 vertexObject->label);
	if (Tcl_GlobalEval (interp, command) != TCL_OK) {
	    return TCL_ERROR;
	}

	/* 
	 *  ... und auch hier die Canvas-Id zurueckgegeben.
	 */

	if( Tcl_GetIntFromObj(interp, Tcl_GetObjResult(interp), &t_cid) != TCL_OK) {
	    return TCL_ERROR;
	}
    } else if (2 == vertexObject->type) {
	
	/* 
	 *  Es wird ein normaler Knoten auf der Canvas gezeichnet ...
	 */

	sprintf (command, "MkVertex %d %d",\
		 vertexObject->center.x, vertexObject->center.y);
	if (Tcl_GlobalEval (interp, command) != TCL_OK) {
	    return TCL_ERROR;
	}
	      
	/*
	 *  ... MkVertex gibt die neue Canvas-Id des Knotens zurueck,
	 *  damit sie spaeter in die Struktur des Knotens eingetragen
	 *  werden kann.
	 */ 

	if( Tcl_GetIntFromObj(interp, Tcl_GetObjResult(interp), &v_cid) != TCL_OK) {
	    return TCL_ERROR;
	}

	/* 
	 *  Dann wird der zugehoerige Text gezeichnet ...
	 */

	sprintf (command, "MkText %d %d %d \"%s\"",\
		 v_cid,\
		 vertexObject->center.x,  vertexObject->center.y,\
		 vertexObject->label);
	if (Tcl_GlobalEval (interp, command) != TCL_OK) {
	    return TCL_ERROR;
	}

	/* 
	 *  ... und auch hier die Canvas-Id zurueckgegeben.
	 */

	if( Tcl_GetIntFromObj(interp, Tcl_GetObjResult(interp), &t_cid) != TCL_OK) {
	    return TCL_ERROR;
	}
    } else if (3 == vertexObject->type) {
	/* 
	 *  Es wird ein Endknoten auf der Canvas gezeichnet ...
	 */

	sprintf (command, "MkFinalVertex %d %d",\
		 vertexObject->center.x, vertexObject->center.y);
	if (Tcl_GlobalEval (interp, command) != TCL_OK) {
	    return TCL_ERROR;
	}
	      
	/*
	 *  ... MkFinalVertex gibt die neue Canvas-Id des Knotens zurueck,
	 *  damit sie spaeter in die Struktur des Knotens eingetragen
	 *  werden kann.
	 */ 

	if( Tcl_GetIntFromObj(interp, Tcl_GetObjResult(interp), &v_cid) != TCL_OK) {
	    return TCL_ERROR;
	}
	  
	/* 
	 *  Dann wird der zugehoerige Text gezeichnet ...
	 */

	sprintf (command, "MkFinalText %d %d %d \"%s\"",\
		 v_cid,\
		 vertexObject->center.x, vertexObject->center.y,
		 vertexObject->label);
	if (Tcl_GlobalEval (interp, command) != TCL_OK) {
	    return TCL_ERROR;
	}

	/* 
	 *  ... und auch hier die Canvas-Id zurueckgegeben.
	 */

	if( Tcl_GetIntFromObj(interp, Tcl_GetObjResult(interp), &t_cid) != TCL_OK) {
	    return TCL_ERROR; 
	}
    } else {
	/* 
	 *  Darf nicht vorkommen!
	 */

	return TCL_ERROR;
    }

    *vertex_id = v_cid;
    *text_id = t_cid;

    return TCL_OK;
}

/*  printEdge
 *--------------------------------------------------------------
 *   
 *  setzt normale Kanten auf der Canvas
 *  und gibt die Canvas-Id's der Kante und des Kanten-
 *  textes zur"uck.
 *
 *--------------------------------------------------------------
 */

int printEdge( Tcl_Interp *interp, Edge_type *edgeObject,\
		 int *arc_id, int *text_id)
{
    char command[LINE_LENGTH];
    int  a_cid;
    int  t_cid;
    int  editor_num;
    Tcl_Obj *objptr;

    /*
     *  Wir lassen uns die Editor-Nummer geben
     */

    objptr=Tcl_ObjGetVar2(interp, Tcl_NewStringObj("faNumber", -1),
		Tcl_NewStringObj("actual", -1), TCL_GLOBAL_ONLY);
    if( Tcl_GetIntFromObj(interp, objptr, &editor_num) != TCL_OK) {
	return TCL_ERROR;
    }

    /* 
     *  Es wird eine normale Kante auf der Canvas gezeichnet ...
     */

    sprintf (command, "MkEdge %d %d %d %d %d %d %d %d", \
	     0, 0, 0, 0, 0, 0, \
	     edgeObject->start,\
	     edgeObject->end);
    if (Tcl_GlobalEval (interp, command) != TCL_OK) {
	return TCL_ERROR;
    }

    /*
     *  ... MkEdge gibt die neue Canvas-Id der Kante zurueck,
     *  damit sie spaeter in die Struktur der Kante eingetragen
     *  werden kann.
     */ 

    if( Tcl_GetIntFromObj(interp, Tcl_GetObjResult(interp), &a_cid) != TCL_OK) {
	return TCL_ERROR;
    }
    
    /* 
     *  Dann wird der zugehoerige Text gezeichnet ...
     */

    sprintf (command, "MkEdgeText %d %d %d {%s} %d %d",\
	     a_cid,\
             0,0,\
             edgeObject->label,\
             edgeObject->start,
             edgeObject->end);
    if (Tcl_GlobalEval (interp, command) != TCL_OK) {
	return TCL_ERROR;
    }

    edgeObject->editor_num=editor_num;

    /* 
     *  ... und auch hier die Canvas-Id zurueckgegeben.
     */

    if( Tcl_GetIntFromObj(interp, Tcl_GetObjResult(interp), &t_cid) != TCL_OK) {
	return TCL_ERROR;
    }

    *arc_id = a_cid;
    *text_id = t_cid;

    return TCL_OK;
}

/*  printOwnEdge
 *--------------------------------------------------------------
 *   
 *  setzt Schlingen auf der Canvas
 *  und gibt die Canvas-Id's der Kante und des Kanten-
 *  textes zur"uck.
 *
 *--------------------------------------------------------------
 */

int printOwnEdge( Tcl_Interp *interp, Edge_type *edgeObject,\
		 int *arc_id, int *text_id)
{
    char command[LINE_LENGTH];
    int  a_cid;
    int  t_cid;
    int  editor_num;
    Tcl_Obj *objptr;

    /*
     *  Wir lassen uns die Editor-Nummer geben
     */

    objptr=Tcl_ObjGetVar2(interp, Tcl_NewStringObj("faNumber", -1),
		Tcl_NewStringObj("actual", -1), TCL_GLOBAL_ONLY);
    if( Tcl_GetIntFromObj(interp, objptr, &editor_num) != TCL_OK) {
	return TCL_ERROR;
    }

    /* 
     *  Es wird eine Schlinge auf der Canvas gezeichnet ...
     */

    sprintf( command, "MkOwnEdge %d %d %d %d",\
	     0,0,\
	     edgeObject->start,\
	     edgeObject->end);
    if( Tcl_GlobalEval( interp,command ) != TCL_OK ) {
	return TCL_ERROR; 
    }
    
    /*
     *  ... MkOwnEdge gibt die neue Canvas-Id der Kante zurueck,
     *  damit sie spaeter in die Struktur der Kante eingetragen
     *  werden kann.
     */ 

    if( Tcl_GetIntFromObj(interp, Tcl_GetObjResult(interp), &a_cid) != TCL_OK) {
	return TCL_ERROR; 
    }

    /* 
     *  Dann wird der zugehoerige Text gezeichnet ...
     */

    sprintf (command, "MkEdgeText %d %d %d {%s} %d %d",\
	     a_cid, \
	     0, 0,\
	     edgeObject->label,\
	     edgeObject->start,\
	     edgeObject->end);
    if (Tcl_GlobalEval (interp, command) != TCL_OK) {
	return TCL_ERROR;
    }

    edgeObject->editor_num=editor_num;

    /* 
     *  ... und auch hier die Canvas-Id zurueckgegeben.
     */

    if( Tcl_GetIntFromObj(interp, Tcl_GetObjResult(interp), &t_cid) != TCL_OK) {
	return TCL_ERROR;
    }

    *arc_id = a_cid;
    *text_id = t_cid;

    return TCL_OK;
}


/*  printComment
 *--------------------------------------------------------------
 *   
 *  setzt Kommentare auf der Canvas
 *  und gibt die Canvas-Id des Kommentars zur"uck.
 *
 *--------------------------------------------------------------
 */

int printComment( Tcl_Interp *interp, Comment_type *commentObject,\
		  int *text_id)
{
    char command[LINE_LENGTH];
    int  c_cid;

    /* 
     *  Es wird ein Kommentar auf der Canvas gezeichnet ...
     */

    sprintf (command, "MkComment %d %d \"%s\" %s %s %c %d",\
	     commentObject->center.x, commentObject->center.y,\
	     commentObject->comment,\
	     commentObject->font->family,\
	     commentObject->font->weight,\
	     commentObject->font->slant,\
	     commentObject->font->size);
    if (Tcl_GlobalEval (interp, command) != TCL_OK) {
	return TCL_ERROR;
    }

    /*
     *  ... MkComment gibt die neue Canvas-Id der Kante zurueck,
     *  damit sie spaeter in die Struktur der Kante eingetragen
     *  werden kann.
     */ 

    if( Tcl_GetIntFromObj(interp, Tcl_GetObjResult(interp), &c_cid) != TCL_OK) {
	return TCL_ERROR;
    }
	
    *text_id = c_cid;

    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  Funktionen zum Verschieben von Knoten, Kanten, Kommentaren,           #
 #  Kantenbeschriftungen und markierten Bereichen auf der                 #
 #  Zeichenflaeche.                                                       #
 #                                                                        #
 ##########################################################################
 */

/*  changeVertexPos
 *--------------------------------------------------------------
 *   
 *  verschiebt einen Knoten mit den zugeh"orenden Kanten
 *  an die Position new_center
 *
 *--------------------------------------------------------------
 */

int changeVertexPos(Tcl_Interp * interp, Vertex_type *vertexObject,\
		   Point_type new_center) 
{
    char           command[LINE_LENGTH];
    EdgeList_type *edges;

    /*
     *  Der Knoten und wird auf der Canvas an die richtige Position gesetzt
     */
    sprintf( command,\
	     "%s coords %d %d %d %d %d",\
	     actualEd->canvas,\
	     vertexObject->v_id,\
	     (new_center.x - vertexRad),\
	     (new_center.y - vertexRad),\
	     (new_center.x + vertexRad),\
	     (new_center.y + vertexRad));
    if( Tcl_GlobalEval( interp,command ) != TCL_OK ) {
	return TCL_ERROR; 
    }
    sprintf( command, "%s coords %d %d %d",\
	     actualEd->canvas,\
	     vertexObject->label_id,\
	     new_center.x,\
	     new_center.y);
    if( Tcl_GlobalEval( interp,command ) != TCL_OK ) {
	return TCL_ERROR; 
    }

    /* 
     *  Die zugehoerigen Kanten muessen aktualisiert werden
     */

    /* 
     *  Zunaechst wird die Position aller von dem Knoten
     *  ausgehenden Kanten angepa"st ...
     */
    
    for( edges = vertexObject->out;
	 edges != NULL;
	 edges = edges->next) {
	if( changeEdgePos( interp, edges->edge_object,new_center,\
			   edges->other_vertex->center, 1) != TCL_OK ) {
	    	    return TCL_ERROR;
	}
    } 

    /* 
     *  ... dann wird die Position aller eingehenden Kanten
     *  angepa"st ...
     */

    for( edges = vertexObject->in; 
	 edges != NULL; 
	 edges = edges->next) {
	if( changeEdgePos( interp, edges->edge_object,\
			   edges->other_vertex->center,\
			   new_center, 1) != TCL_OK ) {
	    return TCL_ERROR;
	}
    } 

    /* 
     *  ... und schlie"slich die Position aller Schlingen.
     */

    for( edges = vertexObject->own_edge;
	 edges != NULL;
	 edges = edges->next) {
	edges->edge_object->center.x += \
	    (new_center.x - vertexObject->center.x);
	edges->edge_object->center.y += \
	    (new_center.y - vertexObject->center.y); 
	if( changeOwnEdgePos( interp,edges->edge_object,\
			      new_center) != TCL_OK ) {
	    return TCL_ERROR;
	}
    } 

    /* 
     *  Die neue Knotenposition wird an die Struktur uebergeben
     */

    vertexObject->center.x =  new_center.x;
    vertexObject->center.y =  new_center.y;

    return TCL_OK;
}


/*  changeEdgePos
 *--------------------------------------------------------------
 *   
 *  bekommt Mittelpunkt des Anfangs und Zielknotens und
 *  die Kantenstruktur "ubergeben.
 *  Berechnet den korrekten Anfangs-, Mittel-  und Endpunkt
 *  der Kante und verschiebt diese samt Text auf der 
 *  Zeichenfl"ache.
 *
 *--------------------------------------------------------------
 */

int changeEdgePos( Tcl_Interp *interp, Edge_type *edgeObject,\
		   Point_type p_start , Point_type p_end, int calc_mid )
{
    char       command[256];
    double     rad;
    double     cos_gamma;
    double     sin_gamma;
    double     hypoth;
    double     xinc1;
    double     yinc1;
    double     nenn;
    int        xd;
    int        yd;
    int        text_x;
    int        text_y;
    Point_type p_mid;

    if( calc_mid == 1 ) {

	/*
	 *  1.: Berechnen des neuen Mittelpunktes der Kante, 
	 *      wenn calc_mid == 1.
	 */

	if( edgeObject->move_infos.fraction < -4710 ) {
	    /* 
	     *  Wenn es sich um eine gerade noch nicht in der Form
	     *  ver"anderten Kante handelt, wird 
	     *  der neue Mittelpunkt dieser Kante mit der folgenden
	     *  einfachen Formel berechnet:
	     */

	    edgeObject->center.x = (p_end.x + p_start.x)/2;
	    edgeObject->center.y = (p_end.y + p_start.y)/2;
	} else {
	    xd = p_end.x - p_start.x;
	    yd = p_end.y - p_start.y;

	    if( (xd != 0) || (yd != 0) ) {
		/* 
		 *  Bild:  
		 *     	   
		 *      mid         end
		 *         -------/|	    
		 *        |     b/ |	    
		 *        |     /  |	    
		 *        |    /   |	    
		 *        |   /    | yd	    
		 *        |  /     |	    
		 *        |a/      |	    
		 *        |/g      | 	    
		 *         --------|  	    
		 *     start  xd          
		 *                          
		 *  a : Winkel alpha
		 *  b : Winkel beta
		 *  g : Winkel gamma
		 *  
		 *  Wenn die Kante als Spline dargestellt wird,
		 *  wird der neue Mittelpunkt folgendenderma"sen
		 *  berechnet:
		 */
		
		hypoth = sqrt(pow(xd,2)+pow(yd,2));
		rad = edgeObject->move_infos.fraction * hypoth;
		cos_gamma = (double)xd/hypoth;
		sin_gamma = (double)yd/hypoth;
		
		xinc1 = rad * \
		    ( edgeObject->move_infos.cos_alpha * cos_gamma - \
		      edgeObject->move_infos.sin_alpha * sin_gamma ) ;
		yinc1 = rad * \
		    ( edgeObject->move_infos.sin_alpha * cos_gamma + \
		      edgeObject->move_infos.cos_alpha * sin_gamma ) ;

		edgeObject->center.x = (int) (p_end.x - xinc1);
		edgeObject->center.y = (int) (p_end.y - yinc1);
	    } 
	}
    }

    /*
     *  2.: Berechnen des korrekten Endpunktes der Kante
     *  aus Mittelpunkt der Kante, Mittelpunkt des
     *  Zielknotens und Radius des Knotens    
     */

    xd = p_end.x - edgeObject->center.x;
    yd = p_end.y - edgeObject->center.y;
    if( (xd != 0) || (yd != 0) ) {
	nenn = sqrt(pow(xd,2)+pow(yd,2));
	xinc1 = ((double) (vertexRad * xd))/nenn;
	yinc1 = ((double) (vertexRad * yd))/nenn;
	p_end.x = p_end.x - (int) xinc1;
	p_end.y = p_end.y - (int) yinc1;
    }

    /*
     *  Der f"ur das Verschieben der Kante auf der Zeichenfl"ache 
     *  anzugebende Mittelpunkt ergibt sich aus folgender Formel. Er ist 
     *  entscheidend f"ur die Darstellung als Spline.
     */

    p_mid.x = 2 * edgeObject->center.x - ((p_start.x + p_end.x) / 2); 
    p_mid.y = 2 * edgeObject->center.y - ((p_start.y + p_end.y) / 2); 

    /*
     *  Die Kante wird auf der Zeichenfl"ache verschoben
     */

    sprintf( command,\
	     "%s coords %d %d %d %d %d %d %d",\
	     actualEd->canvas,\
	     edgeObject->e_id,\
	     p_start.x, p_start.y,\
	     p_mid.x, p_mid.y,\
	     p_end.x, p_end.y);
    if( Tcl_GlobalEval( interp,command ) != TCL_OK ) {
	return TCL_ERROR; 
    }

    /*
     *  Die Position des Kantentextes wird bestimmt.
     */

    text_x = edgeObject->center.x + edgeObject->label_center.x;
    text_y = edgeObject->center.y + edgeObject->label_center.y;

    /*
     *  Der Kantentexte wird verschoben.
     */

    sprintf( command, "%s coords %d %d %d",\
		     actualEd->canvas,\
	     edgeObject->label_id, text_x, text_y);
    if( Tcl_GlobalEval( interp,command ) != TCL_OK ) {
	return TCL_ERROR; 
    }

    return TCL_OK; 
}


/*  changeOwnEdgePos
 *--------------------------------------------------------------
 *   
 *  bekommt den Mittelpunkt des zugeh"origen Knotens "ubergeben.
 *  Berechnet die ben"otigten Punkte um die Schlinge an
 *  der gew"unschten Position darstellen zu k"onnen.
 *
 *--------------------------------------------------------------
 */

int changeOwnEdgePos( Tcl_Interp *interp, Edge_type *edgeObject,\
		      Point_type p_start)
{
    char command[256];
    double xinc1;
    double yinc1;
    double nenn;
    double cos1, cos2;
    double sin1, sin2;
    double xd1, yd1;
    double x1, y1;
    double x2, y2;
    int xd;
    int yd;
    int text_x;
    int text_y;
    Point_type p_end; 
    Point_type m1,m2,m3,m4; 
    Point_type p_mid; 

    p_end = p_start;
    text_x = edgeObject->center.x  + edgeObject->label_center.x;
    text_y = edgeObject->center.y  + edgeObject->label_center.y;

    /* 
     *  Die Winkel bestimmen die Form der Schlinge
     */
    cos1 = 0.9781476;	
    sin1 = 0.20791169;	
    cos2 = 0.965925826;
    sin2 = 0.258819045;

    xd1 = edgeObject->center.x - p_start.x;
    yd1 = edgeObject->center.y - p_start.y;

    x1 = 0.9 * xd1;
    y1 = 0.9 * yd1;
    x2 = 0.85 * xd1;
    y2 = 0.85 * yd1;

    /*
     *  Berechnung der 5 Bezugspunkte, die f"ur die Darstellung
     *  der Schlinge als Spline n"otig sind.
     */
    m1.x = x2 * cos2 - y2 * sin2 + p_start.x ;
    m1.y = y2 * cos2 + x2 * sin2 + p_start.y;
    m2.x = x1 * cos1 - y1 * sin1 + p_start.x ;
    m2.y = y1 * cos1 + x1 * sin1 + p_start.y ;

    p_mid.x = edgeObject->center.x ;
    p_mid.y = edgeObject->center.y ;

    m3.x = (x1 * cos1) + y1 * sin1 + p_start.x ;  
    m3.y = (y1 * cos1) - x1 * sin1 + p_start.y ;
    m4.x = (x2 * cos2) + y2 * sin2 + p_start.x ;
    m4.y = (y2 * cos2) - x2 * sin2 + p_start.y ;

    /*
     *  Berechnung des Anfangspunktes der Schlinge am Rand 
     *  des Knotens.
     */

    xd = m1.x - p_start.x;
    yd = m1.y - p_start.y;

    if( (xd != 0) || (yd != 0) ) {
	nenn = sqrt(pow(xd,2)+pow(yd,2));
	xinc1 = (double) (vertexRad * xd)/nenn;
	yinc1 = (double) (vertexRad * yd)/nenn;

	p_start.x =  p_start.x + (int) xinc1;
	p_start.y =  p_start.y + (int) yinc1;
    } 

    /*
     *  Berechnung des Endpunktes der Schlinge am Rand 
     *  des Knotens.
     */

    xd = p_end.x - m4.x ;
    yd = p_end.y - m4.y;

    if( (xd != 0) || (yd != 0) ) {
	nenn = sqrt(pow(xd,2)+pow(yd,2));
	xinc1 = (double) (vertexRad * xd)/nenn;
	yinc1 = (double) (vertexRad * yd)/nenn;

	p_end.x = p_end.x - (int) xinc1;
	p_end.y = p_end.y - (int) yinc1;
    } 

    /*
     *  Die Schlinge und der Text werden auf der Zeichenfl"ache
     *  verschoben
     */

    sprintf( command,\
	     "%s coords %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",\
	     actualEd->canvas,\
	     edgeObject->e_id,\
	     p_start.x, p_start.y,\
	     m1.x, m1.y,\
	     m2.x, m2.y,\
	     p_mid.x, p_mid.y,\
	     m3.x, m3.y,\
	     m4.x, m4.y,\
	     p_end.x, p_end.y);
    if( Tcl_GlobalEval( interp,command ) != TCL_OK ) {
	return TCL_ERROR; 
    }
    sprintf( command, "%s coords %d %d %d",\
	     actualEd->canvas,\
	     edgeObject->label_id,\
	     text_x, text_y);
    if( Tcl_GlobalEval( interp,command ) != TCL_OK ) {
	return TCL_ERROR; 
    }

    return TCL_OK;
}

/*  edgeMove
 *--------------------------------------------------------------
 *   
 *  ver"andert die Form der Kante mit der Id cid, so da"s deren 
 *  Mittelpunkt die Position mid erh"alt.
 *
 *--------------------------------------------------------------
 */
int edgeMove( Tcl_Interp * interp, int cid, Point_type mid)
{
    Edge_type  *edgeObject;
    Point_type  new_start;
    Point_type  new_end;

    /* 
     *  Struktur der Kante holen ...
     */

    sprintf(interp->result, "arc%d", cid);
    edgeObject = getEdgeStruct( interp->result);
    if( edgeObject == NULL ) {
	return TCL_ERROR;
    }

    /* 
     *  ... und den neuen Mittelpunkt eintragen
     */

    edgeObject->center.x =  mid.x;
    edgeObject->center.y =  mid.y;
    
    if( edgeObject->start != edgeObject->end ) {
	/* 
	 *  Wenn es sich nicht um eine Schlinge handelt werden zunaechst
	 *  die Positionen des Anfangs- und Endpunktes geholt ...
	 */

	if( getVertexCoords( interp, edgeObject->start,\
			     &new_start ) != TCL_OK ) {
	    return TCL_ERROR;
	}
	if( getVertexCoords( interp, edgeObject->end,\
			     &new_end ) != TCL_OK ) {
	    return TCL_ERROR;
	}

	/* 
	 * ... und dann die Kante neu gezeichnet.
	 */

	if( changeEdgePos( interp, edgeObject, new_start,\
			   new_end, 0) != TCL_OK ) {
	    return TCL_ERROR;
	}
    } else {
	/* 
	 *  Wenn es sich um eine Schlinge handelt wird zunaechst die 
	 *  Position des Anfangspunktes geholt ...
	 */

	if( getVertexCoords( interp, edgeObject->start,\
			     &new_start ) != TCL_OK ) {
	    return TCL_ERROR;
	}

	/* 
	 * ... und dann die Kante neu gezeichnet.
	 */

	if( changeOwnEdgePos( interp,edgeObject,new_start) != TCL_OK ) {
	    return TCL_ERROR;
	}
    }

    return TCL_OK;
}


/*  changeCommentPos
 *--------------------------------------------------------------
 *   
 *  verschiebt den Kommentar aus commentObject an die Position
 *  point.
 *
 *--------------------------------------------------------------
 */

int changeCommentPos(Tcl_Interp * interp, Comment_type *commentObject,\
		     Point_type point )

{
    char command[LINE_LENGTH];

    commentObject->center.x = point.x;
    commentObject->center.y = point.y;

    /*
     *  Der Kommentar und wird auf der Canvas an die richtige Position 
     *  gesetzt.
     */

    sprintf( command,\
	     "%s coords %d %d %d",\
	     actualEd->canvas,\
	     commentObject->comment_id,\
	     commentObject->center.x,commentObject->center.y);
    if( Tcl_GlobalEval( interp,command ) != TCL_OK ) {
	return TCL_ERROR; 
    }

    return TCL_OK;
}


/*  changeTextPos
 *--------------------------------------------------------------
 *   
 *  verschiebt den Kantentext mit der Id cid an die Position
 *  center.
 *
 *--------------------------------------------------------------
 */

int changeTextPos( Tcl_Interp * interp, int cid, Point_type center )
{
    char       command[LINE_LENGTH];
    Edge_type *edgeObject;

    /* 
     *  Zunaechst wird sie Struktur der Kante des zu aendernden Textes
     *  geholt ...
     */

    sprintf(interp->result, "arc%d", cid);
    edgeObject = getEdgeStruct( interp->result);
    if( edgeObject == NULL )  {
	return TCL_ERROR;
    }

    /* 
     *  ... dann die Differenz der neuen Textposition zum neuen Mittelpunkt
     *   berechnet und abgespeichert ...
     */

    edgeObject->label_center.x = center.x - edgeObject->center.x;
    edgeObject->label_center.y = center.y - edgeObject->center.y;
	
    /* 
     *  ... und schliesslich der Text auf der Canvas an die richtige 
     *  Position gesetzt.
     */

    sprintf( command, "%s coords %d %d %d",\
	     actualEd->canvas,\
	     edgeObject->label_id,\
	     center.x, center.y);
    if( Tcl_GlobalEval( interp,command ) != TCL_OK ) {
	return TCL_ERROR; 
    }

    return TCL_OK;
}

/*
 ##########################################################################
 #                                                                        #
 #  Funktionen zum Loeschen von Knoten, Kanten und Kommentaren            #
 #  von der Zeichenflaeche.                                               #
 #                                                                        #
 ##########################################################################
 */

/*  vertexDelete
 *--------------------------------------------------------------
 *   
 *  l"oscht den Knoten mit der Id cid und alle dazugeh"orenden 
 *  Kanten von der Zeichenfl"ache.
 *
 *--------------------------------------------------------------
 */

int vertexDelete( Tcl_Interp * interp, int cid,\
		  Undelete_type *undeleteElement)

{
    char           command[LINE_LENGTH];
    EdgeList_type *testy = NULL;
    Tcl_HashEntry *place_entry;
    Vertex_type   *vertexObject;

    sprintf(interp->result, "knoten%d", cid);
    place_entry = Tcl_FindHashEntry( &(actualEd->vertex_table),\
				     interp->result );
    if( place_entry == NULL ) {
	return TCL_ERROR;
    }
    vertexObject =  (Vertex_type *) Tcl_GetHashValue( place_entry );

    /*
     *  Aus der Unset-Liste muessen alle Eintragungen, die den 
     *  zu loeschenden Knoten betreffen entfernt werden 
     */

    cleanUnsetList( cid );

    /*
     *  Alle den Knoten betreffenden Kanten muessen ebenfalls geloescht
     *  werden 
     */

    testy = vertexObject->out;
    while( testy != NULL ) {

	if( edgeDelete( interp, testy->edge_object->e_id,\
			undeleteElement ) != TCL_OK ) {
	    return TCL_ERROR;
	}
	testy = testy->next;
    }

    testy = vertexObject->in;
    while( testy != NULL ) {
	if( edgeDelete( interp, testy->edge_object->e_id,\
			undeleteElement ) != TCL_OK ) {
	    return TCL_ERROR;
	}
	testy = testy->next;
    }

    testy = vertexObject->own_edge;
    while( testy != NULL ) {

	if( edgeDelete( interp, testy->edge_object->e_id,\
			undeleteElement ) != TCL_OK ) {
	    return TCL_ERROR;
	}
	testy = testy->next;
    }

    /* 
     *  Der Knoten wird in die Knotenliste des zuvor initialisierten 
     *  Elementes der Undelete-Liste eingetragen 
     */

    vertexObject = \
	listFirst(undeleteElement->killedVertexList,(void *) vertexObject);

    /*
     *  Zum Schluss muss der Knoten selbst von der Canvas und aus dem
     *  Hash-Table geloescht werden 
     */

    sprintf( command, "%s delete %d",
	     actualEd->canvas, vertexObject->v_id);
    if( Tcl_GlobalEval( interp,command ) != TCL_OK ) {
	return TCL_ERROR; 
    }
    sprintf( command, "%s delete %d",
	     actualEd->canvas, vertexObject->label_id);
    if( Tcl_GlobalEval( interp,command ) != TCL_OK ) {
	return TCL_ERROR; 
    }

    /* Save-Id aus der Liste entfernen */
    if (vertexObject->save_id != -1)
    {
      Tcl_HashEntry *entry = Tcl_FindHashEntry(&NodeIds, (char *)(vertexObject->save_id));
      if (entry)
	Tcl_DeleteHashEntry(entry);
    }
    Tcl_DeleteHashEntry( place_entry );

    return TCL_OK;
}


/*  edgeDelete
 *--------------------------------------------------------------
 *   
 *  l"oscht die Kante mit der Id cid von der Zeichenfl"ache.
 *
 *--------------------------------------------------------------
 */

int edgeDelete( Tcl_Interp * interp, int cid,Undelete_type *undeleteElement)
{

    char           command[LINE_LENGTH];
    Edge_type     *edgeObject;
    Tcl_HashEntry *arc_entry;
    Vertex_type   *place_start;
    Vertex_type   *place_end;
    Tcl_HashEntry *entry;

    sprintf(interp->result, "arc%d",cid);
    arc_entry = Tcl_FindHashEntry( &(actualEd->edge_table),\
				   interp->result );   
    if( arc_entry == NULL ) {
	Tcl_AppendResult( interp, "no object named ",\
			  interp->result, (char *) NULL);
	return TCL_ERROR;
    }
    edgeObject =  (Edge_type *) Tcl_GetHashValue( arc_entry );

    /*
     *  Aus der Unset-Liste muessen alle Eintragungen, die die 
     *  zu loeschende Kante betreffen, entfernt werden 
     */

    cleanUnsetList( cid );

    /* 
     *  Aus der Struktur der Kante wird die Canvas-Id des zugehoerigen
     *  Anfangsknotens besorgt, um dessen Struktur zu erhalten
     */

    sprintf(interp->result, "knoten%d",edgeObject->start);
    place_start =  getVertexStruct( interp->result);
    if( place_start == NULL ) {
	return TCL_ERROR;
    }

    /* 
     *  Danach wird die Struktur des zugehoerigen zweiten Knotens geholt
     */

    sprintf(interp->result, "knoten%d",edgeObject->end);
    place_end =  getVertexStruct( interp->result);
    if( place_end == NULL ) {
	return TCL_ERROR;
    }

    /*
     *  Wenn es sich nicht um eine Schlinge handelt ... 
     */

    if( place_start->v_id != place_end->v_id ) {
	
	/* 
	 *  wird aus der ausgehenden Kantenliste des ersten Knotens die 
	 *  Id der zu loeschenden Kante gestrichen ...
	 */
	place_start->out = deleteObject( edgeObject->e_id, place_start->out);

        /* 
	 *  ... und dann aus der eingehenden Kantenliste des zweiten 
	 *  Knotens die Id der zu loeschenden Kante gestrichen 
	 */

        place_end->in = deleteObject( edgeObject->e_id,place_end->in);
    } else { /* Wenn es sich um eine Schlinge handelt ... */
	/* 
	 *  wird aus der Schlingenliste des betreffenden Knotens die Id 
	 *  der zu loeschenden Kante gestrichen ...
	 */

	place_start->own_edge = deleteObject( edgeObject->e_id,\
					     place_start->own_edge);

	/*
	 *  Der Mittelpunkt der Schlinge wird in relative Koordinaten
	 *  in Abh"angigkeit vom Mittelpunkt des zugeh"origen Knotens
	 *  umgewandelt. Dies ist notwendig, damit die Schlinge nach
	 *  dem Verschieben des Knotens bei Anwendung der Funktion 
	 *  Undelete wieder an die richtige Position gesetzt werden kann.
	 */

	edgeObject->center.x -= place_start->center.x;
	edgeObject->center.y -= place_start->center.y;
    }   

    /* 
     *  Die Kante wird in die Kantenliste des zuvor initialisierten 
     *  Elementes der Undelete-Liste eingetragen. 
     */

    edgeObject = \
	listFirst(undeleteElement->killedEdgeList, (void *) edgeObject);

    /*
     *  Zum Schluss muss die Kante selbst von der Canvas und aus dem
     *  Hash-Table geloescht werden 
     */

    sprintf( command, "%s delete %d",
	     actualEd->canvas, edgeObject->e_id);
    if( Tcl_GlobalEval( interp,command ) != TCL_OK ) {
	return TCL_ERROR; 
    }
    sprintf( command, "%s delete %d", actualEd->canvas, edgeObject->label_id);
    if( Tcl_GlobalEval( interp,command ) != TCL_OK ) {
	return TCL_ERROR; 
    }

    /* Kante-Id aus der Liste entfernen */
    if (edgeObject->save_id != -1)
    {
      int nr = edgeObject->save_id;
      entry = Tcl_FindHashEntry(&EdgeIds, (char *)nr);
      if (entry)
	Tcl_DeleteHashEntry(entry);
    }
    Tcl_DeleteHashEntry( arc_entry ); 
    return TCL_OK;
}


/*  commentDelete
 *--------------------------------------------------------------
 *   
 *  l"oscht die Kommentare mit der Id cid von der Zeichenfl"ache.
 *
 *--------------------------------------------------------------
 */

int commentDelete( Tcl_Interp * interp, int cid,\
		   Undelete_type *undeleteElement )
{
    char           command[LINE_LENGTH];
    Comment_type  *commentObject;
    Tcl_HashEntry *comment_entry;

    sprintf(interp->result, "comment%d", cid);
    comment_entry = \
	Tcl_FindHashEntry( &(actualEd->comment_table), interp->result );
    if( comment_entry == NULL ) {
	return TCL_ERROR;
    }

    commentObject =  (Comment_type *) Tcl_GetHashValue( comment_entry );

    /*
     *  Aus der Unset-Liste muessen alle Eintragungen, die den 
     *  zu loeschenden Kommentar betreffen, entfernt werden 
     */

    cleanUnsetList( cid );

    /* 
     *  Der Kommentar wird in die Kommentarliste des zuvor initialisierten 
     *  Elementes der Undelete-Liste eingetragen 
     */

    commentObject = \
	listFirst(undeleteElement->killedCommentList,\
		  (void *) commentObject);

    /*
     *  Zum Schluss muss der Knoten selbst von der Canvas und aus dem
     *  Hash-Table geloescht werden 
     */

    sprintf( command, "%s delete %d",\
	     actualEd->canvas,\
	     cid );
    if( Tcl_GlobalEval( interp,command ) != TCL_OK ) {
	return TCL_ERROR; 
    }
    Tcl_DeleteHashEntry( comment_entry );
    return TCL_OK;
}

/*
 ##########################################################################
 #                                                                        #
 #  Funktionen durch die auf den Inhalt der Strukturen                    #
 #  von Knoten, Kanten und Kommentaren zugegriffen werden                 #
 #  kann.                                                                 #
 #                                                                        #
 ##########################################################################
 */

/*  getVertexStruct
 *--------------------------------------------------------------
 *  gibt einen Zeiger auf die Struktur eines gesuchten Knoten
 *  zur"uck.
 *--------------------------------------------------------------
 */

Vertex_type * getVertexStruct(char *string)
{
    Tcl_HashEntry  *place_entry;
    
    /* Search for node string in hash table of current editor */
    place_entry = Tcl_FindHashEntry( &(actualEd->vertex_table), string );
    if( place_entry == NULL ) {
	return NULL;
    }
    /* If found return struct, else NULL. */
    return (Vertex_type *)Tcl_GetHashValue( place_entry );
}


/*  getVertexCoords
 *--------------------------------------------------------------
 *  gibt die Koordinaten des Knotens mit der Id vertex_cid
 *  zur"uck.
 *--------------------------------------------------------------
 */

int getVertexCoords( Tcl_Interp *interp, int vertex_cid,\
		     Point_type *center )
{
    Vertex_type *vertexObject;

    sprintf(interp->result, "knoten%d",vertex_cid);
    vertexObject =  getVertexStruct( interp->result);
    if( vertexObject == NULL ) {
	return TCL_ERROR;
    }

    (* center).x = vertexObject->center.x;
    (* center).y = vertexObject->center.y;
		
    return TCL_OK;
}


/*  getEdgeStruct
 *--------------------------------------------------------------
 *  gibt einen Zeiger auf die Struktur einer gesuchten Kante
 *  zur"uck.
 *--------------------------------------------------------------
 */

Edge_type *getEdgeStruct(char *string)
{
    Tcl_HashEntry  *arc_entry;

    /* Search for edge in hash table of current editor. */
    arc_entry = Tcl_FindHashEntry( &(actualEd->edge_table), string );
    if( arc_entry == NULL ) {
	return NULL;
    }
    /* If found return struct, else NULL. */
    return  (Edge_type *) Tcl_GetHashValue( arc_entry );
}


/*  getCommentStruct
 *--------------------------------------------------------------
 *  gibt einen Zeiger auf die Struktur eines gesuchten Kommentars
 *  zur"uck.
 *--------------------------------------------------------------
 */

Comment_type *getCommentStruct(char *string)
{
    Tcl_HashEntry  *comment_entry;

    /* Search comment in hash table of current editor. */
    comment_entry = Tcl_FindHashEntry( &(actualEd->comment_table), string );
    if( comment_entry == NULL ) {
	return NULL;
    }
    /* If found return struct, else NULL. */
    return  (Comment_type *) Tcl_GetHashValue( comment_entry );
}

/*
 ##########################################################################
 #                                                                        #
 #  Funktionen die die Strukturen von Knoten, Kanten und                  #
 #  Kommentaren in den jeweiligen Hash-Table eingliedern                  #
 #                                                                        #
 ##########################################################################
 */

/*  hashVertexInsert
 *--------------------------------------------------------------
 *   
 *  tr"agt einen Knoten in den Knoten-Hash-Table des aktuellen
 *  Editors ein.
 *
 *--------------------------------------------------------------
 */

int hashVertexInsert(Tcl_Interp *interp, Vertex_type *vertexObject)
{
    int            isok;
    Tcl_HashEntry *place_entry;

    /*
     *  Die Struktur des Knotens wird in den zugehoerigen Hash-Table
     *  des aktuellen Editors eingegliedert 
     */

    sprintf(interp->result, "knoten%d",vertexObject->v_id );
    place_entry = Tcl_CreateHashEntry( &(actualEd->vertex_table),\
				       interp->result, &isok );
    if( !isok ) {
	sprintf(interp->result,"Node Id %d already exists!\n",\
		vertexObject->v_id);
	return TCL_ERROR;
    } else {
	Tcl_SetHashValue( place_entry, vertexObject );
    }
    return TCL_OK;
}

/*  hashEdgeInsert
 *--------------------------------------------------------------
 *   
 *  tr"agt eine Kante in den Kanten-Hash-Table des aktuellen
 *  Editors ein.
 *
 *--------------------------------------------------------------
 */

int hashEdgeInsert( Tcl_Interp * interp, Edge_type *edgeObject)
{
    int            test;
    Tcl_HashEntry *arc_entry;

    /*
     *  Die Struktur der Kante wird in den zugehoerigen Hash-Table
     *  des aktuellen Editors eingegliedert 
     */

    sprintf(interp->result, "arc%d",edgeObject->e_id );
    arc_entry = Tcl_CreateHashEntry( &(actualEd->edge_table),\
				     interp->result, &test );
    if( !test ) {
	sprintf(interp->result,"Edge Id %d aleady exists!\n",\
		edgeObject->e_id);
	return TCL_ERROR;
    } else {
	Tcl_SetHashValue( arc_entry, edgeObject );
    }
    return TCL_OK;
}

/*  hashCommentInsert
 *--------------------------------------------------------------
 *   
 *  tr"agt einen Kommentar in den Hash-Table aller Kommentare
 *  des aktuellen Editors ein.
 *
 *--------------------------------------------------------------
 */

int hashCommentInsert( Tcl_Interp * interp, Comment_type *commentObject)
{
    int            test;
    Tcl_HashEntry *comment_entry;

    /*
     *  Die Struktur des Kommentars wird in den zugehoerigen Hash-Table
     *  des aktuellen Editors eingegliedert 
     */

    sprintf(interp->result, "comment%d",commentObject->comment_id );
    comment_entry = Tcl_CreateHashEntry( &(actualEd->comment_table),\
				     interp->result, &test );
    if( !test ) {
	sprintf(interp->result,"Comment Id %d already exists!\n",\
		commentObject->comment_id);
	return TCL_ERROR;
    } else {
	Tcl_SetHashValue( comment_entry, commentObject );
    }

    return TCL_OK;
}
     
/*  calculateAngles
 *--------------------------------------------------------------
 *   
 *  berechnet zwei Winkel und einen Bruch fuer die Struktur 
 *  move_infos in edgeObject. Mit diesen Angaben ist es beim  
 *  Verschieben von Knoten schneller moeglich die zu diesen
 *  Knoten gehoerenden Kanten in ihrer alten Form 
 *  (als Spline) mitzuveraendern. 
 *  Bild:  
 *     	   
 *      mid         alt_move
 *         -------/|	    
 *        |     b/ |	    
 *        |     /  |	    
 *        |    /   |	    
 *      h1|   /h   | 	    
 *        |  /	   |	    
 *        |a/	   |	    
 *        |/g  	   | 	    
 *         --------|  	    
 * other_center            
 *                          
 *  a : Winkel alpha
 *  b : Winkel beta
 *  h : hyp
 *  h1: hyp1
 *--------------------------------------------------------------
 */    	   

int calculateAngles( Tcl_Interp *interp, int arc_id )
{          
    double      hyp;
    double      hyp1;
    int         xd, yd;
    int         xd1, yd1;
    Edge_type  *edgeObject;
    Point_type  alt_move;
    Point_type  other_center;
    Point_type  mid;

    /* 	   
     *  Die Kantenstruktur wird geholt.
     */	   

    sprintf(interp->result, "arc%d",arc_id);
    edgeObject = getEdgeStruct( interp->result);
    if( edgeObject == NULL ) {
	return TCL_ERROR;
    }
	   
    if( edgeObject->start != edgeObject->end ) {
	/* 
	 *  Wenn es sich nicht um eine Schlinge handelt, werden
	 *  die Mittelpunkte der zur Kante geh"orenden Knoten
	 *  geholt:
	 *  alt_move : Mittelpunkt des Anfangsknoten
	 *  other_center : Mittelpunkt des Endknoten
	 */

	if( getVertexCoords(interp,edgeObject->start,\
			    &alt_move) != TCL_OK ) {
	    return TCL_ERROR;
	}
	if( getVertexCoords(interp,edgeObject->end,\
			    &other_center) != TCL_OK ) { 
	    return TCL_ERROR;
	}
	   
	/* 
	 *  Mittelpunkt der Kante
	 */

	mid = edgeObject->center;
	   
	xd = alt_move.x - other_center.x;
	yd = alt_move.y - other_center.y;
	xd1 =  mid.x - other_center.x;
	yd1 =  (mid.y - other_center.y);
	if( ((xd != 0) || (yd != 0)) && ((xd1 != 0) || (yd1 != 0))) {
	    hyp = sqrt(pow(xd,2)+pow(yd,2));
	    hyp1 = sqrt(pow(xd1,2)+pow(yd1,2));

	    /* 
	     *  Cosinus, Sinus sollte bekannt sein. Zur 
	     *  Veranschaulichung (hoffentlich!) siehe Bild.
	     */
	    edgeObject->move_infos.cos_alpha =\
		(xd1/hyp1 * xd/hyp) + (yd1/hyp1*yd/hyp);
	    edgeObject->move_infos.sin_alpha =\
		(yd1/hyp1 * xd/hyp) - (xd1/hyp1*yd/hyp);
	    edgeObject->move_infos.fraction = hyp1/hyp;
	} 
    } else {
	/* 
	 *  Schlingen 
	 */

	edgeObject->move_infos.cos_alpha = 1.0;
	edgeObject->move_infos.sin_alpha = 0.0;
	edgeObject->move_infos.fraction = 4711.0; 
    }
    return TCL_OK;
}


/*
 ##########################################################################
 #                                                                        #
 #  Hilfsfunktionen                                                       #
 #                                                                        #
 ##########################################################################
 */

/*  EdgeToVertexList
 *--------------------------------------------------------------
 *   
 *  tr"agt eine neu erschaffene Kante in die Kantenliste 'out'
 *  ihres Ausgangs- und in die Kantenliste 'in' ihres Zielknotens
 *  ein, wenn es sich um eine normale Kante handelt. Eine Schlinge
 *  wird in die Kantenliste 'own_edge' ihres zugeh"orenden
 *  Knotens einsortiert. 
 *
 *--------------------------------------------------------------
 */

int EdgeToVertexList(Tcl_Interp *interp, Edge_type *edgeObject )
{
    Vertex_type *place_start;
    Vertex_type *place_end;

    /* 
     *  Struktur des Startknotens der Kante holen 
     */

    sprintf(interp->result, "knoten%d",edgeObject->start);
    place_start = getVertexStruct( interp->result );
    if( place_start == NULL ) {
	return TCL_ERROR;
    }

    /* 
     *  Struktur des Endknotens der Kante holen
     */

    sprintf(interp->result, "knoten%d",edgeObject->end);
    place_end = getVertexStruct( interp->result );
    if( place_end == NULL ) {
	return TCL_ERROR;
    }
    
    if( edgeObject->start != edgeObject->end ) { /* keine Schlinge */
	/* 
	 *  Die Kante wird am Ende der jeweiligen Kantenliste des
	 *  Ausgangs- bzw. Zielknotens einsortiert.
	 */

	place_start->out = backInsert( place_start->out, edgeObject,\
				       place_end );
	place_end->in = backInsert( place_end->in, edgeObject,\
				    place_start );
    } else { 
	/* 
	 *  Die Schlinge wird am Ende der Schlingenliste einsortiert.
	 */

	place_start->own_edge = backInsert( place_start->own_edge,\
					    edgeObject, place_start);
    }
    return 0; 
}

/*  makeUnsetElement
 *--------------------------------------------------------------
 *   
 *  erstellt ein Listenelement der Unset-Liste und f"ugt es
 *  an deren Anfang ein.
 *
 *--------------------------------------------------------------
 */

UnsetTop_type *makeUnsetElement( enum Unset_action type )
{
    UnsetTop_type *list_element;

    if( actualEd->unsetCount < maxUnsetCount ) {
	(actualEd->unsetCount) += 1;
    } else {
	/* 
	 *  Das letzte Element der Unset-Liste muss geloescht und
	 *  die dazugehoerenden Daten freigegeben werden
	 */

	list_element = listRemoveLast( actualEd->unsetList );
	if( list_element != NULL ) {
	    listFree( list_element->elementList );
	    free( list_element );
	} else { 
	    return NULL;
	}
    }
    
    /*
     *  Ein neues Element wird erstellt und am Anfang eingefuegt
     */

    list_element = MALLOC( UnsetTop_type );
    list_element->action = type;
    list_element->elementList = createList();
    list_element = listFirst( actualEd->unsetList, (void *) list_element);
    return list_element;
}

/*  insertUnsetStruct
 *--------------------------------------------------------------
 *   
 *  erstellt ein Listenelement der 'elementList', die zu einem   
 *  ListenElement der Unset-Liste geh"ort
 *
 *--------------------------------------------------------------
 */

void insertUnsetStruct(UnsetTop_type *partList_object,\
		       enum Unset_element type,\
		       int id, int x, int y)
{
    Unset_type *set_object;

    /* 
     *  Ein neues Element der elementList wird erstellt 
     */

    set_object = MALLOC( Unset_type );
    set_object->element = type;
    set_object->id = id;
    set_object->last_point.x = x;
    set_object->last_point.y = y;
    set_object =
	listFirst( partList_object->elementList, (void *) set_object );
}

/*  cleanUnsetList
 *--------------------------------------------------------------
 *   
 *  durchsucht die gesamte Unset-Liste nach Elementen mit
 *  der Id 'object_id' und l"oscht diese. 
 *
 *--------------------------------------------------------------
 */

void cleanUnsetList( int object_id )
{
    List_element  *element;
    List_element  *part_element;
    List_element  *help_element;
    List_type     *partList;
    Unset_type    *test;
    UnsetTop_type *test1;

    element = actualEd->unsetList->head->next;
    while( element != actualEd->unsetList->head ) {
	partList = ((UnsetTop_type *)(element->data))->elementList;
	part_element = partList->head->next;
	for( ; part_element != partList->head;
	       part_element=part_element->next) {
	    test = (Unset_type *)(part_element->data);
	    if( test->id == object_id ) {
		test = listRemove( partList, part_element);
		free(test);
		break;
	    }
	}
	help_element = element;
	element = element->next; 
	if( partList->head == partList->head->next ) {
	    test1 = listRemove( actualEd->unsetList, help_element );
	    free(test1);
	    (actualEd->unsetCount) -= 1;
	}
    }
} 

/*  cleanUnsetWithoutDelete
 *--------------------------------------------------------------
 *   
 *  durchsucht die gesamte Unset-Liste nach Elementen mit
 *  der Id 'object_id' und l"oscht diese nur, wenn es sich
 *  um Elemente vom Typ 'move_element' handelt. 
 *
 *--------------------------------------------------------------
 */

void cleanUnsetWithoutDelete( int object_id )
{
    List_element  *element;
    List_element  *part_element;
    List_element  *help_element;
    List_type     *partList;
    Unset_type    *test;
    UnsetTop_type *test1;

    element = actualEd->unsetList->head->next;
    while( element != actualEd->unsetList->head ) {
	partList = ((UnsetTop_type *)(element->data))->elementList;
	if(((UnsetTop_type *)(element->data))->action != remove_element) {
	    part_element = partList->head->next;
	    for(; part_element != partList->head;
		  part_element=part_element->next) {
		test = (Unset_type *)(part_element->data);
		if( test->id == object_id ) {
		    test = listRemove( partList, part_element);
		    free(test);
		    break;
		}
	    }
	
	}
	help_element = element;
	element = element->next; 
	if( partList->head == partList->head->next ) {
	    test1 = listRemove( actualEd->unsetList, help_element );
	    free(test1);
	    (actualEd->unsetCount) -= 1;
	}
    }
} 

/*  makeUndeleteElement
 *--------------------------------------------------------------
 *   
 *  erstellt ein Listenelement der Undelete-Liste und f"ugt es
 *  an deren Anfang ein.
 *
 *--------------------------------------------------------------
 */

Undelete_type *makeUndeleteElement( void )
{
    Undelete_type *list_element;

    if( actualEd->undeleteCount < maxUndeleteCount ) {
	(actualEd->undeleteCount) += 1;
    } else {
	/* 
	 *  Das letzte Element der Undelete-Liste muss geloescht und
	 *  die dazugehoerenden Daten freigegeben werden
	 */

	list_element = listRemoveLast( actualEd->undeleteList );
	if( list_element != NULL ) {
	    listFree( list_element->killedVertexList );
	    listFree( list_element->killedEdgeList );
	    listFree( list_element->killedCommentList );
	    free( list_element );
	} else { 
	    return NULL;
	}
    }
    
    /*
     *  Ein neues Element wird erstellt und am Anfang eingefuegt
     */

    list_element = MALLOC( Undelete_type );
    list_element->killedVertexList = createList();
    list_element->killedEdgeList = createList();
    list_element->killedCommentList = createList();
    list_element = \
	listFirst(actualEd->undeleteList,(void *) list_element);
    return list_element;
}

/*  updateUndeleteList
 *--------------------------------------------------------------
 *   
 *  teilt in der gesamten Liste allen Kanten die neue Id ihres
 *  mit 'Undelete' wieder erstellten Knotens mit.
 *
 *--------------------------------------------------------------
 */

void updateUndeleteList( int old_id, int new_id ) 
{
    Edge_type *test;
    List_element *element;
    List_element *part_element;
    List_type *partList;

    element = actualEd->undeleteList->head->next;
    while( element != actualEd->undeleteList->head ) {
	    partList = ((Undelete_type *)(element->data))->killedEdgeList;
	    part_element = partList->head->next;
	    for(; part_element != partList->head;
		  part_element=part_element->next) {
		test = (Edge_type *)(part_element->data);
		if( test->start == old_id ) {
		    test->start = new_id;
		}
		if( test->end == old_id) {
		    test->end = new_id;
		}
	    }
	    element = element->next; 
    }
} 

/*  add_cid
 *--------------------------------------------------------------
 *   
 *  nimmt Eintragungen in den id2cid Hash-Table vor.
 *
 *--------------------------------------------------------------
 */

int add_cid (int id, int cid, Tcl_Interp * interp)
{
  int test;
  Tcl_HashEntry *id2cid_entry;
  
  /* 
   *  Die alte Knoten-Id wird im id2cid-Table zusammen mit der neuen Id
   *  eingetragen
   */

  sprintf (interp->result, "id%d", id);
  id2cid_entry = Tcl_CreateHashEntry (&id2cidTable, interp->result, &test);

  if (!test) {
    sprintf(interp->result, "id%d already exists!\n",id);
    return TCL_ERROR;
  } else {
    Tcl_SetHashValue (id2cid_entry, cid);
  }
  return TCL_OK;
}

/*  get_cid
 *--------------------------------------------------------------
 *   
 * gibt Daten aus dem id2cid-Hash-Table zur"uck. 
 *
 *--------------------------------------------------------------
 */

int get_cid (char *string)
{
  Tcl_HashEntry *id2cid_entry;

  /* 
   *  Zu einer uebergebenen alten Id wird, falls vorhanden, die neue
   *  Canvas-Id eines Knoten zurueckgegeben
   */

  id2cid_entry = Tcl_FindHashEntry (&id2cidTable, string);
  if (id2cid_entry != NULL) {
      return (int) Tcl_GetHashValue (id2cid_entry);
  } else {
      return -1;
  }
}

/*  backInsert
 *--------------------------------------------------------------
 *   
 *  f"ugt ein neues Element der EdgeList am Ende der Liste ein.
 *
 *--------------------------------------------------------------
 */

EdgeList_type *backInsert(EdgeList_type  *first, Edge_type *edgeObject,\
			  Vertex_type *other_vertex) 
{
  EdgeList_type  *neu;
  EdgeList_type  *help;
  
  help = first;
  if( (neu = MALLOC(  EdgeList_type )) != NULL ) {
      neu->edge_object = edgeObject;
      neu->arc_id = edgeObject->e_id;
      neu->other_vertex = other_vertex;
      neu->other_pid = other_vertex->v_id;
      neu->next = NULL;

      if( help != NULL ) {
	    while( help->next != NULL ) {
		help = help->next;
	    }
	    help->next = neu;
	}
      else
	first = neu;
      return( first );
    }
  else 
      return( NULL );
}

/*  deleteObject
 *--------------------------------------------------------------
 *   
 *  l"oscht das Element der EdgeList mit der Id aid
 *
 *--------------------------------------------------------------
 */

EdgeList_type  *deleteObject(int aid, EdgeList_type  *first)
{
  int             found;
  EdgeList_type  *help;
  EdgeList_type  *actual;

  help = NULL;
  actual = first;
  found = 0;
      
  while( (actual != NULL) && (!found) ) {
      if( aid != actual->edge_object->e_id ) {
	  help = actual;
	  actual = actual->next;
      } else {
	found = 1;
      }
  }
  if( found > 0 ) {
      if( actual == first ) {
	first = first->next;
      } else {
	  help->next = actual->next;
      }
      free( actual );
    }
  else
    fprintf(stderr,"\nElement is not in list\n");
  return first;
  
}

