###########################################################################
###########################################################################
##                                                                       ##
##  Datei   :     `pfaSupport.tcl'                                       ##
##                ----------------                                       ##
##                                                                       ##
##  Inhalt  :     Prozeduren, die von mehreren anderen                   ##
##                Files aus aufgerufen werden oder sonst                 ##
##                nirgends dazupassen.                                   ##
##                                                                       ##
###########################################################################
###########################################################################


##  InitialQuestion
##-----------------------------------------------------------
##  ruft ein Meldungsfenster auf, wenn eine Funktion von
##  pepPFA aktiviert wird, ohne da"s ein Editor aktiviert 
##  ist.
##-----------------------------------------------------------
##

proc InitialQuestion {} {
    append info_text "First Of All You Have To Open An Editor!\n"
    append info_text "There Are Three Possibilities:\n\n"
    append info_text "1. Create A New FA.\n"
    append info_text "2. Open An Existing FA.\n"
    append info_text "3. Open A Project\n"

    set question [win_dialog .d {Initial Question}\
		      $info_text largeinfo 0\
		      {Create FA} {Open FA} {Open Project} {Cancel}]
	    if {$question == 0} {
		AddNewFA
	    }  elseif {$question == 1} {
		FilemanagerOpenFA
	    } elseif {$question == 2} {
		FilemanagerOpenProject
	    } elseif {$question == 3} {
		set opt(mouse3) Free
		return end
	    }
}


##  LoadPreferences
##-----------------------------------------------------------
##  laedt die abgespeicherten Optionen beim Aufruf des
##  Programms 
##-----------------------------------------------------------
##
proc LoadPreferences {} {
    global opt saveOpt env

    set saveOpt(label) ""
    set saveOpt(mouseKeys) ""
    set saveOpt(code) ""
    set saveOpt(examineFA) ""
    set saveOpt(highlight) ""
    set saveOpt(unsetScale) "" 
    set saveOpt(undeleteScale) "" 

    set Datei [file join $env(HOME) $opt(rcfile)]
    if {![file exists $Datei]} {
      set Datei [file join $opt(autoPath) $opt(rcfile)]
    }
    if { ([file exists $Datei] == 1) } {
	set g [open $Datei r]
	while {[gets $g Zeile] >= 0} {	
	    lappend all [lindex $Zeile 0]
	    lappend all [lindex $Zeile 1]
	}
	close $g

	set opt(label)         [lindex $all 1]
	set opt(mouseKeys)     [lindex $all 3]
        set opt(code)          [lindex $all 5]
        set opt(examineFA)     [lindex $all 7]
        set opt(highlight)     [lindex $all 9]
	set opt(unsetScale)    [lindex $all 11]
	set opt(undeleteScale) [lindex $all 13]

	## Test, ob die Optionen korrekt eingelesen  
	## wurden
	##
	if { 
	    ([lsearch $all label] == 0) &&
	     (($opt(label) == "on") || ($opt(label) == "off")\
		  || ($opt(label) == "edges")) &&
	     ([lsearch $all mouseKeys] == 2) &&
	     (($opt(mouseKeys) == "on") || ($opt(mouseKeys) == "off")) &&
             ([lsearch $all code] == 4) &&
	     (($opt(code) == "B(PN)2") || ($opt(code) == "Struct")) &&
             ([lsearch $all examineFA] == 6) &&
	     (($opt(examineFA) == "all") || \
	      ($opt(examineFA) == "visible")) &&
             ([lsearch $all highlight] == 8) &&
	     (($opt(highlight) == "on") || ($opt(highlight) == "off")) &&
	     ([lsearch $all unsetScale] == 10) &&
	     (($opt(unsetScale) > 2) && ($opt(unsetScale) < 252)) &&
	     ([lsearch $all undeleteScale] == 12) &&
	     (($opt(undeleteScale) > 2) && ($opt(undeleteScale) < 252)) } {

	    if {$opt(mouseKeys) == "on"} {
		SetMouse
	    }
	    pfa_UpdateLists $opt(unsetScale) $opt(undeleteScale)
            set opt(scope) local	
	    set opt(project) no
	    set opt(projectName) ""
	}
    } else {
	## File ist nicht da oder im Eimer -> Voreinstellung
	##
	set opt(label) on
	set opt(mouseKeys) on
	set opt(code) B(PN)2
	set opt(examineFA) all
#BG TODO:
	set opt(highlight) off
	set opt(unsetScale) 25
	set opt(undeleteScale) 25
        set opt(scope) local
	set opt(project) no
	set opt(projectName) ""
	SetMouse
	pfa_UpdateLists $opt(unsetScale) $opt(undeleteScale)
    }
}


##  XYSet
##-----------------------------------------------------------
##  berechnet die globale Position aus der lokalen Position 
##  der sichtbaren Zeichenfl"ache und "ubergibt sie "uber
##  die globale Variable pos an die aufrufenden Prozeduren.
##-----------------------------------------------------------
##
proc XYSet {x y} {
    global pos editor
    set pos(x) [$editor(canvas) canvasx $x]
    set pos(y) [$editor(canvas) canvasy $y]
}


##  ItemName
##-----------------------------------------------------------
##  liefert den gemeinsamen Tag eines Itemverbundes (z.B.
##  Kante und Kantentext).
##-----------------------------------------------------------
##
proc ItemName {item} {
    global editor
  return [lindex [$editor(canvas) gettags $item] 0]
}


##  ItemBase
##-----------------------------------------------------------
##  liefert die Id der Itemverbund-Basis (bei Kante und 
##  Kantentext ist dies die Id der Kante).
##-----------------------------------------------------------
##
proc ItemBase {item} {
    global editor
    return [lindex [$editor(canvas) \
                     find withtag [ItemName $item]] 0]
}


##  ItemText
##-----------------------------------------------------------
##  liefert die Id des Itemverbund-Textes.
##-----------------------------------------------------------
##
proc ItemText {item} {
    global editor
    return [lindex \
           [$editor(canvas) gettags $item] 1]text[ItemBase $item]
}


##  GetText
##-----------------------------------------------------------
##  liefert den Text eines Items.
##-----------------------------------------------------------
##
proc GetText {item} {
    global editor

    set text [$editor(canvas) itemconfigure [ItemText $item] -text]
    return [lindex $text 4]
}

##  itemExists
##-----------------------------------------------------------
##  "uberpr"uft, ob ein Item existiert.
##-----------------------------------------------------------
##
proc itemExists {tag} {
    global editor
    if {[$editor(canvas) find withtag $tag] == ""} {
        return 0
    } else {
        return 1
    }
}


##  ItemCenter
##-----------------------------------------------------------
##  liefert den Mittelpunkt eines Knotens.
##-----------------------------------------------------------
##
proc ItemCenter {item} {
    global editor

    set coords [$editor(canvas) coords [ItemBase $item]]
    if {[llength $coords]==2} {
        return $coords
    } else {
        return [eval "between $coords"]
    }
}


##  between
##-----------------------------------------------------------
##  liefert den Mittelpunkt zwischen zwei Punkten.
##-----------------------------------------------------------
##
proc between {x1 y1 x2 y2} {
    return [list [expr floor([expr {($x1 + $x2) / 2}])] \
                 [expr floor([expr {($y1 + $y2) / 2}])]]
}


##  ItemType
##-----------------------------------------------------------
##  liefert den Verbund-Typ eines Items.
##-----------------------------------------------------------
##
proc ItemType {item} {
   global editor
  
   if { [lsearch [$editor(canvas) gettags $item] knoten] != -1 } {
       return knoten
   } elseif {[lsearch [$editor(canvas) gettags $item] startknoten] != -1} {
       return startknoten
   } elseif { [lsearch [$editor(canvas) gettags $item] endknoten] != -1 } {
       return endknoten
   } elseif { [lsearch [$editor(canvas) gettags $item] kante] != -1 } {
       return kante
   } 
}


##  KantenStart
##-----------------------------------------------------------
##  liefert die Id des Ausgangsknotens einer Kante.
##-----------------------------------------------------------
##
proc KantenStart {kante} {
    global editor
    return [string range [lindex [$editor(canvas) gettags $kante] 2] 8 end]
}


##  KantenZiel
##-----------------------------------------------------------
##  liefert die Id des Zielknotens einer Kante.
##-----------------------------------------------------------
##
proc KantenZiel {kante} {
    global editor
    return [string range [lindex [$editor(canvas) gettags $kante] 3] 9 end]
}


##  VertexEnterColor
##-----------------------------------------------------------
##  Wenn der Mauszeiger in den Bereich des Knotens kommt
##  aendert dieser seine Farbe.
##-----------------------------------------------------------
##
proc VertexEnterColor { node } {
   global editor opt

   set itembase [ItemBase $node]
   set opt(itemColor) \
       [lindex [$editor(canvas) itemconfigure $itembase -fill] 4]
   $editor(canvas) itemconfigure $itembase -fill MediumVioletRed

}


##  VertexLeaveColor
##-----------------------------------------------------------
##  Wenn der Mauszeiger den Bereich des Knotens verlaesst
##  bekommt dieser wieder seine Ursprungsfarbe.
##-----------------------------------------------------------
##
proc VertexLeaveColor { node } {
    global editor opt

    $editor(canvas) itemconfigure [ItemBase $node] \
	-fill $opt(itemColor)
}


##  EdgeEnterColor
##-----------------------------------------------------------
##  Wenn der Mauszeiger in den Bereich der Kante kommt
##  aendert diese ihre Farbe.
##-----------------------------------------------------------
##
proc EdgeEnterColor { edge } {
    global editor opt

    set opt(itemColor) \
	[lindex [$editor(canvas) itemconfigure $edge -fill] 4]
    $editor(canvas) itemconfigure [ItemName $edge] -fill magenta
}


##  EdgeLeaveColor
##-----------------------------------------------------------
##  Wenn der Mauszeiger den Bereich der Kante verlaesst
##  bekommt diese wieder Ihre Ursprungsfarbe.
##-----------------------------------------------------------
##
proc EdgeLeaveColor { edge } {
    global editor opt

    $editor(canvas) itemconfigure [ItemName $edge] \
	-fill $opt(itemColor)
}


##  SomethingHasChanged
##-----------------------------------------------------------
##  wird bei "Anderungen auf der Zeichenfl"ache aufgerufen.
##-----------------------------------------------------------
##
proc SomethingHasChanged { what1 what2 } {
    global  editor faNumber opt 

    if { $what1 == "FA" } {
	if { $editor($faNumber(actual),edited) == "no"} {
	    set editor($faNumber(actual),edited) "yes"
	}
    }
    if { $what2 == "highlight" } {
	if { $opt(changed) == "no" } {
	    set opt(changed) "yes"
	} 
    }
}


##  CutEmptyElements
##-----------------------------------------------------------
##  l"oscht leere Elemente am Ende einer Liste.
##-----------------------------------------------------------
##
proc CutEmptyElements { list } { 
    set length [llength $list]
    for { set i [expr $length -1]} { $i >= 0 } { incr i -1} {
	if { [lindex $list $i] != {} } {
	    break
	} else {
	    set list [lreplace $list $i $i]
	}
    }
    return $list
}
