/*
***************************************************************************
***************************************************************************
**                                                                       **
**  Datei   :    `pfa.c'                                                 **
**               -------                                                 **
**                                                                       **
**  Inhalt  :    `pfa.c' stellt die Schnittstelle zwischen Tcl und C     **
**               dar. Es handelt sich um die applikationsspezifische     **
**               Initialisierung wozu z.B. die Registrierung aller       **
**               neuen Tcl-Befehle geh"ort.                              **
**                                                                       **
***************************************************************************
***************************************************************************
last-update: 05.02.95
*/

#include <tcl.h>
#include <tk.h>
#include <math.h>
#include "pfa.h"
#include "pfaCompile.h"

/*
 *  global variables
 */

Tcl_HashTable id2cidTable, EdgeIds, NodeIds;
Editor_type  *firstEd = NULL;
Editor_type  *actualEd = NULL;
Copy_type *copyObject = NULL;
int maxUndeleteCount = 25;
int maxUnsetCount = 25;
int vertexRad = 14;

#ifdef BUILD_pfa
#   undef TCL_STORAGE_CLASS
#   define TCL_STORAGE_CLASS DLLEXPORT
#endif

EXTERN int 
Pfa_Init(Tcl_Interp* interp)
{
    /* 
     * Create Stub initialisation if required. 
     */
#ifdef USE_TCL_STUBS
    if (Tcl_InitStubs(interp, "8.0", 0) == NULL) {
#else
    if (Tcl_PkgRequire(interp, "Tcl", "8.0", 0) == NULL) {
#endif
	return TCL_ERROR;
    }
#ifdef USE_TK_STUBS
    if (Tk_InitStubs(interp, "8.0", 0) == NULL) {
#else
    if (Tcl_PkgRequire(interp, "Tk", "8.0", 0) == NULL) {
#endif
	return TCL_ERROR;
    }

    /* Currently disabled. */
    /* tcl_RcFileName = "~/.tclrc"; */

    if (Tcl_PkgProvide(interp, "pfa", "1.0") != TCL_OK) {
	return TCL_ERROR;
    }

    /*
     * Allocate node and edge hash tables for saving.
     */
    Tcl_InitHashTable (&EdgeIds, TCL_ONE_WORD_KEYS);
    Tcl_InitHashTable (&NodeIds, TCL_ONE_WORD_KEYS);
    
    /* 
     * Register new cancas item for final vertex.
     */
    Tk_CreateItemType( &tkEndplaceType );


    /* 
     * Register all new tcl functions from library.
     */


    /*
     *  Neue Befehle des File-Men"us:
     *  -----------------------------
     */

    Tcl_CreateObjCommand( interp, "pfa_NewFA", tk_NewFA,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_LoadFA", tk_LoadFA,\
		       (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_SaveFA", tk_SaveFA,\
		       (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_DeleteFA", tk_DeleteFA,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    /*
     *  Neue Befehle des Edit-Men"us:
     *  -----------------------------
     */

    Tcl_CreateObjCommand( interp, "pfa_Cut", tk_Cut,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_Copy", tk_Copy,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_Paste", tk_Paste,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_Unset", tk_Unset,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_Undelete", tk_Undelete,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    /*
     *  Neue Befehle des Modes-Untermen"us:
     *  -----------------------------------
     */

    Tcl_CreateObjCommand( interp, "pfa_VertexInfos", tk_VertexInfos,\
		       (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_NormalEdgeInfos", tk_NormalEdgeInfos,\
		       (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_OwnEdgeInfos", tk_OwnEdgeInfos,\
		       (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL); 

    Tcl_CreateObjCommand( interp, "pfa_VertexDelete", tk_VertexDelete,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_EdgeDelete", tk_EdgeDelete,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_CommentDelete", tk_CommentDelete,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_AreaDelete", tk_AreaDelete,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_LineUpX", tk_LineUpX,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_LineUpY", tk_LineUpY,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_UnsetTextCoords", tk_UnsetTextCoords,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_ChangeEdgeTextCoords",\
		       tk_ChangeEdgeTextCoords,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_Rename", tk_Rename,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_ResetEdge", tk_ResetEdge,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_CommentInfos", tk_CommentInfos,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    /*
     *  Neue Befehle des Option-Men"us:
     *  -------------------------------
     */

    Tcl_CreateObjCommand( interp, "pfa_UpdateLists", tk_UpdateLists,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_CommentSaveFont", tk_CommentSaveFont,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    /*
     *  Neue Befehle des Action-Men"us:
     *  -------------------------------
     */

    Tcl_CreateObjCommand( interp, "pfa_Highlight", tk_Highlight,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);

    Tcl_CreateObjCommand( interp, "pfa_GetEdgeToLine", tk_GetEdgeToLine,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);

    Tcl_CreateObjCommand( interp, "pfa_ExistEditor", tk_ExistEditor,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);

    Tcl_CreateObjCommand( interp, "pfa_Editors", tk_Editors,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_Compile", tk_Compile,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_Compile_Struct", tk_Compile_Struct,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);    

    /* For highlighting in b(pn)^2 */
    Tcl_CreateObjCommand( interp, "pfa_GetEdgeToRef", tk_GetEdgeToRef,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_HighlightEdge", tk_HighlightEdge,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    /*
     *  Neuer Befehl des Editor-Men"us:
     *  -------------------------------
     */

    Tcl_CreateObjCommand( interp, "pfa_ChangeFA", tk_ChangeFA,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    /*
     *  Neue Befehle zum Verschieben von Elementen:
     *  -------------------------------------------
     */

    Tcl_CreateObjCommand( interp, "pfa_UnsetVertexCoords",\
		       tk_UnsetVertexCoords,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_VertexChange", tk_VertexChange,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_SaveEdgeAngle",\
		       tk_SaveEdgeAngle,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_EdgeMove", tk_EdgeMove,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_Angle", tk_Angle,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_UnsetCommentCoords",\
		       tk_UnsetCommentCoords,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_CommentChange", tk_CommentChange,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_UnsetAreaCoords", tk_UnsetAreaCoords,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_AreaChange", tk_AreaChange,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    return TCL_OK;
}

#ifdef WIN32
/*
 *----------------------------------------------------------------------
 *
 * DllEntryPoint --
 *
 *	This wrapper function is used by Windows to invoke the
 *	initialization code for the DLL.  If we are compiling
 *	with Visual C++, this routine will be renamed to DllMain.
 *	routine.
 *
 * Results:
 *	Returns TRUE;
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

BOOL APIENTRY
DllEntryPoint(HINSTANCE hInst, DWORD reason, LPVOID reserved)
{
    return TRUE;
}
#endif
