/*
***************************************************************************
***************************************************************************
**                                                                       **
**  Datei   :    `pfa.c'                                                 **
**               -------                                                 **
**                                                                       **
**  Inhalt  :    `pfa.c' stellt die Schnittstelle zwischen Tcl und C     **
**               dar. Es handelt sich um die applikationsspezifische     **
**               Initialisierung wozu z.B. die Registrierung aller       **
**               neuen Tcl-Befehle geh"ort.                              **
**                                                                       **
***************************************************************************
***************************************************************************
last-update: 05.02.95
*/

#include <tcl.h>
#include <tk.h>
#include <math.h>
#include "pfa.h"


/*
 *  globale Variablen
 */

Tcl_HashTable id2cidTable, EdgeIds, NodeIds;
Editor_type  *firstEd = NULL;
Editor_type  *actualEd = NULL;
Copy_type *copyObject = NULL;
int maxUndeleteCount = 25;
int maxUnsetCount = 25;
int vertexRad = 14;


int
Tcl_AppInit(Tcl_Interp* interp)
{
    Tk_Window main;

    /* 
     *  Ein Root-Window wird geschaffen.
     */
    main = Tk_MainWindow(interp);


    /* 
     *  Es wird unter anderem ein neuer Interpreter geschaffen.
     */
    if (Tcl_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    if (Tk_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }

    /*
     * Die Knoten- und Kantentabellen fuers Speichern anlegen
     */
    Tcl_InitHashTable (&EdgeIds, TCL_ONE_WORD_KEYS);
    Tcl_InitHashTable (&NodeIds, TCL_ONE_WORD_KEYS);
    

    /* 
     *  Das neue Canvas-Item wird angemeldet.
     */
    Tk_CreateItemType( &tkEndplaceType );


    /* 
     *  Alle neuen Tcl-Befehle werden angemeldet.
     */


    /*
     *  Neue Befehle des File-Men"us:
     *  -----------------------------
     */

    Tcl_CreateObjCommand( interp, "pfa_NewFA", tk_NewFA,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_LoadFA", tk_LoadFA,\
		       (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_SaveFA", tk_SaveFA,\
		       (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_DeleteFA", tk_DeleteFA,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    /*
     *  Neue Befehle des Edit-Men"us:
     *  -----------------------------
     */

    Tcl_CreateObjCommand( interp, "pfa_Cut", tk_Cut,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_Copy", tk_Copy,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_Paste", tk_Paste,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_Unset", tk_Unset,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_Undelete", tk_Undelete,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    /*
     *  Neue Befehle des Modes-Untermen"us:
     *  -----------------------------------
     */

    Tcl_CreateObjCommand( interp, "pfa_VertexInfos", tk_VertexInfos,\
		       (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_NormalEdgeInfos", tk_NormalEdgeInfos,\
		       (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_OwnEdgeInfos", tk_OwnEdgeInfos,\
		       (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL); 

    Tcl_CreateObjCommand( interp, "pfa_VertexDelete", tk_VertexDelete,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_EdgeDelete", tk_EdgeDelete,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_CommentDelete", tk_CommentDelete,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_AreaDelete", tk_AreaDelete,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_LineUpX", tk_LineUpX,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_LineUpY", tk_LineUpY,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_UnsetTextCoords", tk_UnsetTextCoords,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_ChangeEdgeTextCoords",\
		       tk_ChangeEdgeTextCoords,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_Rename", tk_Rename,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_ResetEdge", tk_ResetEdge,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_CommentInfos", tk_CommentInfos,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    /*
     *  Neue Befehle des Option-Men"us:
     *  -------------------------------
     */

    Tcl_CreateObjCommand( interp, "pfa_UpdateLists", tk_UpdateLists,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_CommentSaveFont", tk_CommentSaveFont,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    /*
     *  Neue Befehle des Action-Men"us:
     *  -------------------------------
     */

    Tcl_CreateObjCommand( interp, "pfa_Highlight", tk_Highlight,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);

    Tcl_CreateObjCommand( interp, "pfa_GetEdgeToLine", tk_GetEdgeToLine,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);

    Tcl_CreateObjCommand( interp, "pfa_ExistEditor", tk_ExistEditor,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);

    Tcl_CreateObjCommand( interp, "pfa_Editors", tk_Editors,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_Compile", tk_Compile,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_Compile_Struct", tk_Compile_Struct,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);    

    /* Fuers highlighten */
    Tcl_CreateObjCommand( interp, "pfa_GetEdgeToRef", tk_GetEdgeToRef,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_HighlightEdge", tk_HighlightEdge,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    /*
     *  Neuer Befehl des Editor-Men"us:
     *  -------------------------------
     */

    Tcl_CreateObjCommand( interp, "pfa_ChangeFA", tk_ChangeFA,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    /*
     *  Neue Befehle zum Verschieben von Elementen:
     *  -------------------------------------------
     */

    Tcl_CreateObjCommand( interp, "pfa_UnsetVertexCoords",\
		       tk_UnsetVertexCoords,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_VertexChange", tk_VertexChange,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_SaveEdgeAngle",\
		       tk_SaveEdgeAngle,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_EdgeMove", tk_EdgeMove,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_Angle", tk_Angle,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_UnsetCommentCoords",\
		       tk_UnsetCommentCoords,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_CommentChange", tk_CommentChange,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_UnsetAreaCoords", tk_UnsetAreaCoords,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 

    Tcl_CreateObjCommand( interp, "pfa_AreaChange", tk_AreaChange,\
		       (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);	 


    /* Peedy: Diese Zeile muss leider erstmal raus */
    /* tcl_RcFileName = "~/.tclrc"; */

    return TCL_OK;
}



int
main(int argc, char **argv)
{
    Tcl_Main(argc, argv, Tcl_AppInit);
    return 0;
}
