# RPC-client-dispatch.tcl:
# Client dispatch for the RPC user interface
#
# $Id: RPC-client-dispatch.tcl,v 1.1 2001/07/11 12:21:49 pep Exp $
# $State: Exp $
#
# Copyright (c) 1996 Michael E. Shorter (mike@globalware.com)
#
# See the file "LICENSE.TERMS" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

global rpc_vars;
if [info exists rpc_vars(sourced,RPC_client_dispatch_tcl_)] {
    return;
}

package provide Tcl_RPC 1.3.0;

if {("[info proc init_var]" == "init_var") || ([auto_load init_var] == 1)} {
    init_var rpc_vars(sourced,RPC_client_dispatch_tcl_) 1;
    init_var rpc_vars(version,RPC_client_dispatch_tcl_) 1.3.0;
}

proc client_dispatch {code cmd host port} {
    global rpc_vars;

    # open our socket to the server
    if [catch "RPC_client $host $port" channel] {
	return -code error "$channel";
    }

    # this is for the logging service
    set myid "client_dispatch|$port:$channel|";

    if [catch {RPC_write $channel $code $cmd exception} ret] {
	close $channel;
	return -code error "$ret";
    }

    # if it's just a one way transmission, return
    if {($code == $rpc_vars(RPC_code,RDO)) \
	    || ($code == $rpc_vars(RPC_code,SHUTDOWN))} {
	close $channel;
	return;
    }

    rpclog 0x0040 "$myid:waiting for $channel to become readable...";

    if {[chan_readable $channel $rpc_vars(RPC_READABLE_TIMEOUT)] == 0} {
	rpclog 0x0040 "$myid:channel readable timeout";
	close $channel;
	return -code error "waiting for readable timeout";
    }

    # get our stuff back from the server.  Pretend that any error was on our
    # side

    if [catch {RPC_read $channel returncode returnval exception} ret] {
	close $channel;
	return -code error "$ret";
    }

    if {$exception == 1} {
	rpclog 0x0002 "$myid:got an exception! \"$returnval\"";
    }

    close $channel;
    return -code $returncode "$returnval";
}

# Local Variables:
# mode:tcl
# End:
