# RPC-dispatch.tcl:
# Default server dispatch routine
#
# $Id: RPC-dispatch.tcl,v 1.1 2001/07/11 12:21:49 pep Exp $
# $State: Exp $
#
# Copyright (c) 1996 Michael E. Shorter (mike@globalware.com)
#
# See the file "LICENSE.TERMS" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

global rpc_vars;
if [info exists rpc_vars(sourced,RPC-dispatch_tcl_)] {
    return;
}

package provide Tcl_RPC 1.3.0;

if {("[info proc init_var]" == "init_var") || ([auto_load init_var] == 1)} {
    set rpc_vars(sourced,RPC-dispatch_tcl_) 1;
    set rpc_vars(version,RPC-dispatch_tcl_) 1.3.0;
}

proc dispatch {sock client_addr client_port} {
    global rpc_vars;

    set server_port [lindex [fconfigure $sock -sockname] 2];

    # this is for the logging service
    set myid "<dispatch|$server_port:$sock|>";

    incr_var rpc_vars(RPC_Accepts,all);
    incr_var rpc_vars(RPC_Accepts,$server_port);

    # now pass on all of our options
    fconfigure $sock -blocking $rpc_vars(RPC_BLOCKINGIO) \
        -buffering $rpc_vars(RPC_BUFFERING) \
        -buffersize $rpc_vars(RPC_BUFFERSIZE) \
        -translation $rpc_vars(RPC_TRANSLATION);

    # some useful info...
    rpclog 0x0010 "$myid:$sock => [fconfigure $sock]";

    # wait until the client has written to the channel
    rpclog 0x0040 "$myid:waiting for $sock to become readable...";
    if {[chan_readable $sock $rpc_vars(RPC_READABLE_TIMEOUT)] == 0} {
	# we timed out waiting
	rpclog 0x0002 "$myid:channel readable timeout";
	incr_var rpc_vars(RPC_IO_Errors,all);
	incr_var rpc_vars(RPC_IO_Errors,$server_port);
	close $sock;
	return;
    }

    # get the first part of the packet so we'll know which type of packet this
    # is
    if [catch {RPC_read $sock code cmd exception} ret] {
	rpclog 0x0002 "$myid:read error \"$ret\"";
	incr_var rpc_vars(RPC_IO_Errors,all);
	incr_var rpc_vars(RPC_IO_Errors,$server_port);
	close $sock;
	return;
    }

   # looks like a valid code, let's keep track of how many we process
    incr_var rpc_vars(RPC_Connects,all);
    incr_var rpc_vars(RPC_Connects,$server_port);

    # now, which code did we get?
    if {$code == $rpc_vars(RPC_code,EXCEPTION)} {
	incr_var rpc_vars(RPC_Stats,EXCEPTION,all);
	incr_var rpc_vars(RPC_Stats,$server_port,EXCEPTION);
	rpclog 0x0002 "$myid:got an exception! \"$cmd\"";
	close $sock;
	return;
    }

    #
    if {$code == $rpc_vars(RPC_code,PING)} {
	rpclog 0x0020 "$myid:got a ping";
	if [eof $sock] {
	    rpclog 0x0002 "$myid:eof found";
	    incr_var rpc_vars(RPC_IO_Errors,all);
	    incr_var rpc_vars(RPC_IO_Errors,$server_port);
	    close $sock;
	    return;
	}

	if [catch {RPC_write $sock 0 1 exception} ret] {
	    rpclog 0x0002 "$myid:write error \"$ret\"";
	    incr_var rpc_vars(RPC_IO_Errors,all);
	    incr_var rpc_vars(RPC_IO_Errors,$server_port);
	    close $sock;
	    return;
	}
	incr_var rpc_vars(RPC_Stats,PING,all);
	incr_var rpc_vars(RPC_Stats,$server_port,PING);
	rpclog 0x0020 "$myid:ping done";
	close $sock;
	return;
    }

    #
    if {$code == $rpc_vars(RPC_code,RPC)} {
	rpclog 0x0020 "$myid:server RPC starting";
	set retcode [catch "uplevel #0 $cmd" ret];
	rpclog 0x0100 "$myid:server RPC writing";
	if {$retcode} {
	    incr_var rpc_vars(RPC_eval_Errors,all);
	    incr_var rpc_vars(RPC_eval_Errors,$server_port);
	    rpclog 0x0002 "$myid:RPC eval error \"$ret\"";
	}

	if [catch {RPC_write $sock $retcode $ret exception} ret] {
	    rpclog 0x0002 "$myid:write error \"$ret\"";
	    incr_var rpc_vars(RPC_IO_Errors,all);
	    incr_var rpc_vars(RPC_IO_Errors,$server_port);
	    close $sock;
	    return;
	}
	rpclog 0x0020 "$myid:server RPC close";

	incr_var rpc_vars(RPC_Stats,RPC,all);
	incr_var rpc_vars(RPC_Stats,RPC,$server_port);
	rpclog 0x0020 "$myid:server RPC done";
	close $sock;
	return;
    }

    #
    if {$code == $rpc_vars(RPC_code,RDO)} {
	rpclog 0x0020 "$myid:server RDO starting";
	incr_var rpc_vars(RPC_Stats,RDO,all);
	incr_var rpc_vars(RPC_Stats,RDO,$server_port);
	if [catch "uplevel #0 $cmd" ret] {
	    incr_var rpc_vars(RPC_eval_Errors,all);
	    incr_var rpc_vars(RPC_eval_Errors,$server_port);
	    rpclog 0x0002 "$myid:RDO eval error \"$ret\"";
	}
	rpclog 0x0020 "$myid:server RDO done";
	close $sock;
	return;
    }

    #
    if {$code == $rpc_vars(RPC_code,STATS)} {
	rpclog 0x0020 "$myid:server STATS starting";
	incr_var rpc_vars(RPC_Stats,STATS,all);
	incr_var rpc_vars(RPC_Stats,STATS,$server_port);
	set stats "NYI";
#	lappend stats [list COUNT $RPC_Counter($server_port)];
#	foreach {element} [array names RPC_code] {
#	    lappend stats [list $element $RPC_Stats($RPC_code($element),all)];
#	}
#	lappend stats [list ERRORS $RPC_Errors($server_port)];
	rpclog 0x0100 "$myid:server STATS writing"

	if [catch {RPC_write $sock 0 $stats exception} ret] {
	    rpclog 0x0002 "$myid:write error \"$ret\"";
	    incr_var rpc_vars(RPC_IO_Errors,all);
	    incr_var rpc_vars(RPC_IO_Errors,$server_port);
	    close $sock;
	    return;
	}

	rpclog 0x0020 "$myid:server STATS close";
	catch {unset stats};
	rpclog 0x0020 "$myid:server STATS done";
	close $sock;
	return;
    }

    #
    if {$code == $rpc_vars(RPC_code,SHUTDOWN)} {
	rpclog 0x0020 "$myid:doing server shutdown...";
	incr_var rpc_vars(RPC_Stats,SHUTDOWN,all);
	incr_var rpc_vars(RPC_Stats,SHUTDOWN,$server_port);
	close $sock;
	return [incr_var rpc_vars(RPC_Server_Halted,$server_port)];
    }

    incr_var rpc_vars(RPC_Stats,UNKNOWN,all);
    incr_var rpc_vars(RPC_Stats,UNKNOWN,$server_port);
    rpclog 0x0002 "$myid:we got a strange code $code";
    
    rpclog 0x0010 "$myid:server dispatch done";
    close $sock;
    return;
}

# Local Variables:
# mode:tcl
# End:
