# RPC-log.tcl:
# Simple logging facility
#
# $Id: RPC-log.tcl,v 1.1 2001/07/11 12:21:50 pep Exp $
# $State: Exp $
#
# Copyright (c) 1996 Michael E. Shorter (mike@globalware.com)
#
# See the file "LICENSE.TERMS" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

global rpc_vars;
if [info exists rpc_vars(sourced,RPC_log_tcl_)] {
    return;
}

package provide Tcl_RPC 1.3.0;

if {("[info proc init_var]" == "init_var") || ([auto_load init_var] == 1)} {
    init_var rpc_vars(version,RPC_log_tcl_) 1.3.0;
    init_var rpc_vars(sourced,RPC_log_tcl_) 1;
}

proc rpclog {level args} {
    global rpc_vars;

    if {![info exists rpc_vars(RPC_LOGFILE)]} {
	set rpc_vars(RPC_LOGFILE) stdout;
    }

    if [info exists rpc_vars(RPC_DEBUG)] {
	if {[expr $rpc_vars(RPC_DEBUG) & $level] != 0} {
	    puts $rpc_vars(RPC_LOGFILE) "\
	    [file root [file tail [info script]]]\
            \[[pid]\]\{[clock format [clock seconds]\
	    -format "%m%d %H%M%S"]\}: [lindex $args 0]";
	    flush $rpc_vars(RPC_LOGFILE);
	}
    }
}

proc rpc_debug {{level 0}} {
    global rpc_vars;

    if {$level == "help"} {
	puts "0x00000 no logging";
	puts "";
	puts "User Interface:";
	puts "---------------------------------";
	puts "0x00001 background errors";
	puts "0x00002 errors";
	puts "0x00004 port creation and svc_run";
	puts "0x00008 user commands";
	puts "";
	puts "Dispatchers:";
	puts "----------------------------------";
	puts "0x00010 dispatch enter and exit";
	puts "0x00020 dispatch commands";
	puts "0x00040 channel readable";
	puts "";
	puts "IO:"
	puts "----------------------------------";
	puts "0x00100 read/write";
	puts "0x00200 channel configuration data";
	puts "0x00400 data stream";
	puts "";
	puts "Queues:"
	puts "----------------------------------";
	puts "0x01000 all queueing events";
	puts "";
	puts "Nameserver:"
	puts "----------------------------------";
	puts "0x10000 all ns events";
	return;
    }

    if {$level == "all"} {
	set rpc_vars(RPC_DEBUG) 0xfffff;
    } else {
	set rpc_vars(RPC_DEBUG) $level;
    }
    return;
}

# Local Variables:
# mode:tcl
# End:
