# RPC-queue.tcl:
# Queueing facilities
#
# $Id: RPC-queue.tcl,v 1.1 2001/07/11 12:21:50 pep Exp $
# $State: Exp $
#
# Copyright (c) 1996 Michael E. Shorter (mike@globalware.com) and
#                    Lisa M. Beale (lisa@globalware.com)
#
# See the file "LICENSE.TERMS" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

global rpc_vars;
if [info exists rpc_vars(sourced,RPC-queue_tcl_)] {
    return;
}

package provide Tcl_RPC 1.3.0;

if {("[info proc init_var]" == "init_var") || ([auto_load init_var] == 1)} {
    init_var rpc_vars(version,RPC_queue_tcl_) 1.3.0;
    init_var rpc_vars(sourced,RPC_queue_tcl_) 1;
}

proc enqueue {resource} {
    global rpc_queue;

    set mytid [hrt];
    set myid "enqueue|$resource|$mytid";

    # add myself to the queue
    set rpc_queue($resource,$mytid) 0;

    rpclog 0x1000 "$myid|entering queue \"$resource\", [lqueue $resource]\
                   in queue";

    # wait until we come to the head of the queue
    rpclog 0x1000 "$myid|head of queue => [head_of_queue $resource]";
    if {[head_of_queue $resource] != $mytid} {
	vwait rpc_queue($resource,$myid);
    }
    rpclog 0x1000 "$myid|I'm up with [lqueue $resource]\
                   others left in queue";

    if {![info exists rpc_queue(clerk,$resource)]} {
	set rpc_queue(clerk,$resource) [every 10000 "queue_clerk $resource"];
    }
}

proc dequeue {resource {clerk 0}} {
    global rpc_queue;

    set myid "dequeue|$resource";

    if {[lqueue $resource] == 0} {
	rpclog 0x1000 "$myid|no one in queue \"$resource\"";
	if {[info exists rpc_queue(clerk,$resource)]} {
	    cancel $rpc_queue(clerk,$resource);
	    catch "unset rpc_queue(clerk,$resource)";
	}
	return;
    }

    incr_var rpc_queue(counter,$resource);

#    if {$clerk == 0} {
	catch "unset rpc_queue($resource,[head_of_queue $resource])";
#    }
    if {[lqueue $resource] > 0} {
	set rpc_queue($resource,[head_of_queue $resource]) 1;
    }
#    parray rpc_queue;
    rpclog 0x1000 "$myid|done in queue \"$resource\"";
}

proc lqueue {resource} {
    global rpc_queue;

    catch {set ret [llength [array names rpc_queue "$resource,*"]]};
    return $ret;
}

proc head_of_queue {resource} {
    global rpc_queue;

    set head [lindex [lsort [array names rpc_queue "$resource,*"]] 0];
    set head [string range $head [expr [string last "," $head] + 1] end];
    return $head;
}

proc queue_clerk {resource} {
    global rpc_queue;

    rpclog 0x1000 "queue_clerk|$resource|checking for someone in queue...";

    dequeue $resource 1;
}

proc pqueue {resource} {
    global rpc_queue;

    foreach {element} [array names rpc_queue "${resource},*"] {
	puts "rpc_queue($element) = $rpc_queue($element)";
    }
    return;
}

# Local Variables:
# mode:tcl
# End:
