# RPC-timing.tcl:
# Every and delay procs
#
# $Id: RPC-timer.tcl,v 1.1 2001/07/11 12:21:50 pep Exp $
# $State: Exp $
#
# Copyright (c) 1996 Michael E. Shorter (mike@globalware.com)
#
# See the file "LICENSE.TERMS" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

global rpc_vars;
if [info exists rpc_vars(sourced,RPC_timer_tcl_)] {
    return;
}

package provide Tcl_RPC 1.3.0;

if {("[info proc init_var]" == "init_var") || ([auto_load init_var] == 1)} {
    init_var rpc_vars(version,RPC_timer_tcl_) 1.3.0;
    init_var rpc_vars(sourced,RPC_timer_tcl_) 1;
}

# high resolution timer good for time ids (tids)
proc hrt {} {
    return [clock clicks];
}

# do a nonblocking wait for ms milliseconds
proc delay {ms} {
    global rpc_vars;

    set mytime [hrt];
    set rpc_vars(delay,$mytime) 0;
    catch {after $ms "set rpc_vars(delay,$mytime) 1"};
    vwait rpc_vars(delay,$mytime);
    catch "unset rpc_vars(delay,$mytime)";
    return;
}

# main workhorse for every command.
proc every_body {tid ms args} {
    global rpc_vars;

    init_var rpc_vars(every,$tid);

    if {$rpc_vars(every,$tid) == 1} {
	if [catch "uplevel #0 $args"] {
	    catch "unset rpc_vars(every,$tid)";
	}
	after $ms "every_body $tid $ms $args";
    } else {
	catch "unset rpc_vars(every,$tid)";
    }
    return;
}

# execute a script after every ms milliseconds
proc every {ms args} {
    global rpc_vars;

    set mytid [hrt];
    set rpc_vars(every,$mytid) 1;
    catch {after $ms "every_body $mytid $ms $args"};
    return $mytid;
}

# cancel a previously started every command, or all of them
proc cancel {mytid} {
    global rpc_vars;

    if {$mytid == "all"} {
	foreach {element} [array names rpc_vars] {
	    if [regexp "^every" $element] {
		set rpc_vars($element) 0;
	    }
	}
    } else {
	set rpc_vars(every,$mytid) 0;
    }
}

# Local Variables:
# mode:tcl
# End:
