# RPC.tcl:
# Variables tcl-RPC
#
# $Id: RPC-var.tcl,v 1.1 2001/07/11 12:21:50 pep Exp $
# $State: Exp $
#
# Copyright (c) 1996 Michael E. Shorter (mike@globalware.com)
#
# See the file "LICENSE.TERMS" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

global rpc_vars;
if [info exists rpc_vars(sourced,RPC_var_tcl_)] {
    return;
}

package provide Tcl_RPC 1.3.0;

# initialize a variable if it isn't defined, otherwise just leave it be
proc init_var {varname {initial_value 0}} {
    upvar #0 $varname var;

    if {![info exists var]} {
	catch {set var $initial_value};
    }
    return;
}

# incr a variable if it is found, else init_var it
proc incr_var {varname {increment 1}} {
    upvar #0 $varname var;

    if {![info exists var]} {
	catch {init_var $varname $increment} ret;
	return;
    }

    catch {incr var $increment};
    return;
}

init_var rpc_vars(sourced,RPC_var_tcl_) 1;
init_var rpc_vars(version,RPC_var_tcl_) 1.3.0

# set RPC_BLOCKINGIO to 1 for blocking I/O (default)
# set RPC_BLOCKINGIO to 0 for non-blocking I/O
init_var rpc_vars(RPC_BLOCKINGIO)  	1;

# set RPC_BUFFERING to full, line or none.  See fconfigure(n)
init_var rpc_vars(RPC_BUFFERING)    	full;

# set RPC_BUFFERSIZE to 10 <= size <= 1000000.  See fconfigure(n)
init_var rpc_vars(RPC_BUFFERSIZE)   	4096

# set RPC_TRANSLATION to end-of-line translation.  See fconfigure(n)
init_var rpc_vars(RPC_TRANSLATION)  	auto

# this is the number of times to return 0 for reads
# non-blocking I/O only
init_var rpc_vars(MAXNULLREADS)     	500;

init_var rpc_vars(RPC_MAX_READS)	500;

# time in ms to pause after each non-blocking read
init_var rpc_vars(RPC_READ_TIMEOUT) 500;
init_var rpc_vars(RPC_READABLE_TIMEOUT) 20000;

init_var rpc_vars(RPC_PORT)         	3210;
init_var rpc_vars(LOG_PORT)         	3211;
init_var rpc_vars(MAINTENANCE_PORT)	3212;
init_var rpc_vars(RPC_MIN_AUTO_PORT)	30000;

init_var rpc_vars(MAXCODEBITS) 3;
init_var rpc_vars(MAXDATABITS) 9;
init_var rpc_vars(MAXRPCREADS) 5;

init_var rpc_vars(RPC_ACK)   "\x06";	# good ol' ACK
init_var rpc_vars(RPC_NAK)   "\x15";	# the same goes for NAK
init_var rpc_vars(RPC_HELLO) "Hallo";	# greeting string
init_var rpc_vars(RPC_HEY)   "Hej";	# acknowledgment string

init_var rpc_vars(RPC_code,EXCEPTION)   0;
init_var rpc_vars(RPC_code,PING)        1;
init_var rpc_vars(RPC_code,RPC)         2;
init_var rpc_vars(RPC_code,RDO)         4;
init_var rpc_vars(RPC_code,STATS)       8;
init_var rpc_vars(RPC_code,UNKNOWN)    16;
init_var rpc_vars(RPC_code,SHUTDOWN)  512;

init_var rpc_vars(format_codebits) "%$rpc_vars(MAXCODEBITS)d";
init_var rpc_vars(format_databits) "%$rpc_vars(MAXDATABITS)d";

proc rpc_blockingio {{io 1}} {
    global rpc_vars;

    set rpc_vars(RPC_BLOCKINGIO) $io;
}

proc rpc_buffering {{buffering full}} {
    global rpc_vars;

    set rpc_vars(RPC_BUFFERING) $buffering;
}

proc rcp_buffersize {{buffersize 4096}} {
    global rpc_vars;

    set rpc_vars(RPC_BUFFERSIZE) $buffersize;
}

proc rpc_translation {{translation auto}} {
    global rpc_vars;

    set rpc_vars(RPC_TRANSLATION) $translation;
}

proc rpc_maxnullreads {{maxnullreads 500}} {
    global rpc_vars;

    set rpc_vars(MAXNULLREADS) $maxnullreads;
}

proc rpc_maxreads {{maxreads 500}} {
    global rpc_vars;

    set rpc_vars(RPC_MAX_READS) $maxreads;
}

proc rpc_readtimeout {{readtimeout 500}} {
    global rpc_vars;

    set rpc_vars(RPC_READ_TIMEOUT) $readtimeout;
}

proc rpc_readabletimeout {{readabletimeout 20000}} {
    global rpc_vars;

    set rpc_vars(RPC_READABLE_TIMEOUT) $readabletimeout;
}

proc rpc_rpcport {{rpcport 3210}} {
    global rpc_vars;

    set rpc_vars(RPC_PORT) $rpcport;
}

proc rpc_logport {{logport 3211}} {
    global rpc_vars;

    set rpc_vars(LOG_PORT) $logport;
}

proc rpc_maintport {{maintport 3212}} {
    global rpc_vars;

    set rpc_vars(MAINTENANCE_PORT) $maintport;
}

proc rpc_minautoport {{minautoport 30000}} {
    global rpc_vars;

    set rpc_vars(RPC_MIN_AUTO_PORT) $minautoport;
}

proc show_rpcdebug {{output stdout}} {
    global rpc_vars;

    puts $output $rpc_vars(RPC_DEBUG);
}

proc show_rpcblockingio {{output stdout}} {
    global rpc_vars;

    puts $output $rpc_vars(RPC_BLOCKINGIO);
}

proc show_rpcbuffering {{output stdout}} {
    global rpc_vars;

    puts $output $rpc_vars(RPC_BUFFERING);
}

proc show_rpcbuffersize {{output stdout}} {
    global rpc_vars;

    puts $output $rpc_vars(RPC_BUFFERSIZE);
}

proc show_rpctranslation {{output stdout}} {
    global rpc_vars;

    puts $output $rpc_vars(RPC_TRANSLATION);
}

proc show_rpcmaxnullreads {{output stdout}} {
    global rpc_vars;

    puts $output $rpc_vars(MAXNULLREADS);
}

proc show_rpcmaxreads {{output stdout}} {
    global rpc_vars;

    puts $stdout $rpc_vars(RPC_MAX_READS);
}

proc show_rpcreadtimeout {{output stdout}} {
    global rpc_vars;

    puts $output $rpc_vars(RPC_READ_TIMEOUT);
}

proc show_rpcreadabletimeout {{output stdout}} {
    global rpc_vars;

    puts $output $rpc_vars(RPC_READABLE_TIMEOUT);
}

proc show_rpcport {{output stdout}} {
    global rpc_vars;

    puts $output $rpc_vars(RPC_PORT);
}

proc show_rpclogport {{output stdout}} {
    global rpc_vars;

    puts $output $rpc_vars(LOG_PORT);
}

proc show_rpcmaintport {{output stdout}} {
    global rpc_vars;

    puts $output $rpc_vars(MAINTENANCE_PORT);
}

proc show_rpcminautoport {{output stdout}} {
    global rpc_vars;

    puts $output $rpc_vars(RPC_MIN_AUTO_PORT);
}

# Local Variables:
# mode:tcl
# End:
