# RPC.tcl:
# User interface for tcl-RPC
#
# Copyright (c) 1996 Michael E. Shorter (mike@globalware.com)
#
# See the file "LICENSE.TERMS" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#
# I hacked something in RPC_server so that it runs on tcl 7.6
# *for me*. Lutz Pogrell

global rpc_vars;
if [info exists rpc_vars(sourced,RPC_tcl_)] {
    return;
}

package provide Tcl_RPC 1.3.0;

if {("[info proc init_var]" == "init_var") || ([auto_load init_var] == 1)} {
    init_var rpc_vars(sourced,RPC_tcl_) 1;
    init_var rpc_vars(version,RPC_tcl_) 1.3.0;
}

if {("[info proc bgerror]" != "bgerror") && ([auto_load bgerror] == 0)} {
#    puts "loading RPC bgerror";

    proc bgerror {errmsg} {
	global rpc_vars;
	global errorInfo;

	rpclog 0x0001 "bgerror: $errmsg";
	rpclog 0x0001 "bgerror: $errorInfo";
    }
}

# wait for a channel to become readable, or a timeout occurs
proc chan_readable {chan {timeout 0}} {
    global rpc_vars;

    if [catch "fconfigure $chan" ret] {
	return -code error "$ret";
    }

    rpclog 0x0040 "waiting for \"$chan\" to become readable in $timeout ms";

    set mytid [hrt];

    # not readable so far
    set rpc_vars(chanreadable,$mytid) -1;

    # fire only once.
    if [catch {fileevent $chan readable "set rpc_vars(chanreadable,$mytid) 1;\
                                         fileevent $chan readable {}"}] {
	catch "unset rpc_vars(chanreadable,$mytid)";
	return 0;
    }

	if {$timeout != -1} {
    set afterid [after $timeout "set rpc_vars(chanreadable,$mytid) 0"];
	}

    # see if file has become readable
    if {$rpc_vars(chanreadable,$mytid) != -1} {
	if {$timeout != -1} {
		after cancel $afterid;
	}
		}

    # if not, wait for something to happen
    vwait rpc_vars(chanreadable,$mytid);
    set tmp $rpc_vars(chanreadable,$mytid);
    catch "unset rpc_vars(chanreadable,$mytid)";
    return $tmp;
}

proc RPC_server {{port -1} {addr localhost} {dispatch dispatch}} {
    global rpc_vars;

    if {$port == -1} {set port $rpc_vars(RPC_PORT)}

    # this is for autoport selection
    # and it is nonsens! Lutz
    #if {$port == 0} {
	#init_var rpc_vars(server_auto_port) $rpc_vars(RPC_MIN_AUTO_PORT);

	#while {[catch "socket -server $dispatch -myaddr $addr \
                       #$rpc_vars(server_auto_port)" sock] != 0} {
	    #incr_var rpc_vars(server_auto_port);
	#}
    #} else {
	if [catch "socket -server $dispatch -myaddr $addr $port" sock] {
	    return -code error "$sock";
	}
    #}

    # now pass on all of our options
    #fconfigure $sock -blocking $rpc_vars(RPC_BLOCKINGIO) \
	#-buffering $rpc_vars(RPC_BUFFERING) \
	#-buffersize $rpc_vars(RPC_BUFFERSIZE) \
	#-translation $rpc_vars(RPC_TRANSLATION);

    # get our port number
    set ourport [lindex [fconfigure $sock -sockname] 2];
    # das war flash! index war 1 aber 2 ist richtig

    # initialize all variables for this server
    set rpc_vars(RPC_Server_Halted,$ourport) -1;

    foreach {element} [array names rpc_vars] {
	# look for RPC_Code at the beginning of the element
	if [regexp "^RPC_Code" $element] {
	    # grab the code name from the end of the element name
	    set code [string range $element \
			  [expr [string last "," $element] + 1] end];

	    init_var rpc_vars(RPC_Stats,$code,all);
	    set rpc_vars(RPC_Stats,$ourport,$code) 0;
	}
    }

    init_var rpc_vars(RPC_Connects,all);
    init_var rpc_vars(RPC_Accepts,all);
    init_var rpc_vars(RPC_IO_Errors,all);
    init_var rpc_vars(RPC_eval_Errors,all);

    rpclog 0x0004 "RPC_server $sock => [fconfigure $sock]";
    return "$sock";
}

proc RPC_svc_run {{server_port -1}} {
    global rpc_vars;

    if {$server_port == -1} {
	rpclog 0x0004 "RPC_svc_run: using default port $rpc_vars(RPC_PORT)";
	set server_port $rpc_vars(RPC_PORT);
    }

    set rpc_vars(RPC_Server_Halted,$server_port) 0;
    rpclog 0x0004 "RPC_svc_run: vwaiting for\
                   rpc_vars(RPC_Server_Halted,$server_port)";
    vwait rpc_vars(RPC_Server_Halted,$server_port);
    rpclog 0x0004 "RPC_svc_run: server halting returning\
                   $rpc_vars(RPC_Server_Halted,$server_port)";
}

proc RPC_client {{host localhost} {port 0}} {
    global rpc_vars;

    if {$port == 0} {set port $rpc_vars(RPC_PORT)}

    if [catch "socket $host $port" sock] {
	return -code error "$sock";
    }

    fconfigure $sock -blocking $rpc_vars(RPC_BLOCKINGIO) \
	-buffering $rpc_vars(RPC_BUFFERING) \
	-buffersize $rpc_vars(RPC_BUFFERSIZE) \
	-translation $rpc_vars(RPC_TRANSLATION);

    rpclog 0x0004 "RPC_client $sock => [fconfigure $sock]";
    return "$sock";
}

proc RPC {cmd {host localhost} {port 0}} {
    global rpc_vars;

    if {$port == 0} {set port $rpc_vars(RPC_PORT)}

    rpclog 0x0008 "RPC to $host:$port starting cmd: \"$cmd\"";
    if [catch "client_dispatch $rpc_vars(RPC_code,RPC) \"{$cmd}\" $host\
               $port" ret] {
	return -code error "$ret";
    }
    rpclog 0x0008 "RPC to $host:$port done, returning \"$ret\"";
    return "$ret";
}

proc RDO {cmd {host localhost} {port 0}} {
    global rpc_vars;

    if {$port == 0} {set port $rpc_vars(RPC_PORT)}

    rpclog 0x0008 "RDO to $host:$port starting";
    if [catch "client_dispatch $rpc_vars(RPC_code,RDO) \"{$cmd}\" $host\
               $port" ret] {
	return -code error "$ret";
    }
    rpclog 0x0008 "RDO to $host:$port done, returning \"$ret\"";
    return "$ret";
}

proc RPC_ping {{host localhost} {port 0}} {
    global rpc_vars;

    if {$port == 0} {set port $rpc_vars(RPC_PORT)};

    rpclog 0x0008 "RPC_ping to $host:$port starting";
    if [catch "client_dispatch $rpc_vars(RPC_code,PING) \"\" $host $port" ret] {
	return -code error "$ret";
    }
    rpclog 0x0008 "RPC_ping to $host:$port done, returning \"$ret\"";
    return "$ret";
}

proc RPC_stats {{host localhost} {port 0}} {
    global rpc_vars;

    if {$port == 0} {set port $rpc_vars(RPC_PORT)}

    rpclog 0x0008 "RPC_stats to $host:$port starting";
    if [catch "client_dispatch $rpc_vars(RPC_code,STATS) \"\" $host $port" ret] {
	return -code error "$ret";
    }
    rpclog 0x0008 "RPC_stats to $host:$port done, returning \"$ret\"";
    return "$ret";
}

proc RPC_server_shutdown {{host localhost} {port 0}} {
    global rpc_vars;

    if {$port == 0} {set port $rpc_vars(RPC_PORT)}

    rpclog 0x0008 "RPC_server_shutdown to $host:$port starting";
    if [catch "client_dispatch $rpc_vars(RPC_code,SHUTDOWN) \"\" $host $port" ret] {
	return -code error "$ret";
    }
    rpclog 0x0008 "RPC_server_shutdown to $host:$port done, returning \"$ret\"";
    return "$ret";
}

# Local Variables:
# mode:tcl
# End:
