#include "symtab.h"
#include "signet.h"
#include "intset.h"
#include "sortrange.h"
#include "info.h"

intset env_pid_set;

static void comm_handle_sigdefs(T_symbol sigdef)
{
	static int total_number_of_signals = 0;

	T_list list = sigdef->e.list;
	T_symbol sym;

	for_list (list,sym,T_symbol)
	{
		T_signal siginf = sym->e.signal;
		T_list args = siginf->args;
		T_syntype type;
		int i = 0, total = 1;

		siginf->number = siginf->highest = total_number_of_signals++;
		if (!args)
		{
			add_signal_info(sym);
			continue;
		}
		siginf->ground = MYmalloc(args->count * sizeof(Integer));
		siginf->width = MYmalloc(args->count * sizeof(Integer));
		for_list (args,type,T_syntype)
		{
			siginf->ground[i] = (type->set == SORT_PID)?
					-1 : intset_elem(type->set,1);
			siginf->width[i] = (type->set == SORT_PID)?
				num_pi_ids : intset_elem(type->set,
						intset_length(type->set))
					    - intset_elem(type->set,1) + 1;
			total *= siginf->width[i];
			i++;
		}
		siginf->highest += (total-1);
		total_number_of_signals += (total-1);
		add_signal_info(sym);
	}
}

T_symbol check_connect(T_symbol relay, T_symbol endpoint, T_symbol scope)
{
	T_structure scopeinf = scope->e.struc;
	T_symbol sym;
	T_symbol c1;
	int count = 0;
	bool    to_outside;

	if (!strcmp(endpoint->name,"env"))
	{
		if (scopeinf->subtype == TYPE_SYSTEM)
		{
			char *tmpstr = MYmalloc(strlen(scope->name)+3);
			T_ident dummy;

			sprintf(tmpstr,"%s@%c",scope->name,
				'a'+scopeinf->num_connects++);
			dummy = s_dummy(tmpstr);
			c1 = define_ident(dummy,TYPE_ENVPORT,NULL);
			list_add(scopeinf->entities,c1);
			free(tmpstr);
			sig_add_pids(c1,env_pid_set);
			return c1;
		}
		to_outside = True;
	}
	else if (endpoint->e.struc->subtype != TYPE_PROCESS)	/* fixme */
/*	else if (endpoint->e.struc->num_connects) */
		to_outside = False;
	else
	{
		return endpoint;
	}

	if (!to_outside) scopeinf = endpoint->e.struc;
	for_list(scopeinf->entities,sym,T_symbol)
		if (sym->type == TYPE_CONNECT)
		{
			T_symbol chn;
			T_connect conn = sym->e.connect;
			T_list chnlist = (to_outside? conn->inlist
						    : conn->outlist);

			for_list (chnlist,chn,T_symbol)
				if (chn == relay)
				{
					count++;
					c1 = sym;
				}
		}

	if (count != 1)
		Msg(FAT,"%s must be in exactly one connection of structure %s",
			relay->name,endpoint->name);

	return c1;
}

void comm_handle_relay(T_symbol relay,T_symbol scope)
{
	T_relay chninf = relay->e.relay;
	T_symbol c1,c2;

	c1 = check_connect(relay,chninf->ep1,scope);
	c2 = check_connect(relay,chninf->ep2,scope);
			
	sig_add_route(relay,c1,c2,chninf->siglist1);
	if (chninf->bidirectional) sig_add_route(relay,c2,c1,chninf->siglist2);

	chninf->ep1 = c1;
	chninf->ep2 = c2;

}

void comm_handle_connect(T_symbol connect,T_symbol scope)
{
	T_structure scopeinf = scope->e.struc;

	free(connect->name);
	connect->name = MYmalloc(strlen(scope->name)+3);
	sprintf(connect->name,"%s@%c",scope->name,'a'+scopeinf->num_connects++);
}

void comm_handle_process(T_symbol block)
{
	static int process_ids = 0;
	int maximum = block->e.struc->prcsinf->maximum;

	intset pids = alloc_intset();
	intset_add_interval(&pids,process_ids + 1,process_ids + maximum);
	process_ids += maximum;
	sig_add_pids(block,pids);
	add_pid_info(block,pids);
	free(pids);
}

void comm_handle_scope(T_symbol block)
{
	T_symbol sym;
	T_structure blkinf = block->e.struc;

	if (!strcmp(block->name,"env")) return;

	/* fixme */
	if (blkinf->subtype == TYPE_PROCESS)
	{
		comm_handle_process(block);
		return;
	}

	for_list (blkinf->entities,sym,T_symbol)
	switch (sym->type)
	{
	    case TYPE_SIGDEF:
		comm_handle_sigdefs(sym);
		break;
	    case TYPE_RELAY:	/* erst nachher, s.u. */
		break;
	    case TYPE_CONST:
	    case TYPE_SYNTYPE:
	    case TYPE_PROCEDURE:
		break;
	    case TYPE_STRUCTURE:
		comm_handle_scope(sym);
		break;
	    case TYPE_CONNECT:
		comm_handle_connect(sym,block);
		break;
	    default:
		Msg(FAT,"can't happen: %s:%u (%u)",
			__FILE__,__LINE__,sym->type);
		break;
	}

	for_list (blkinf->entities,sym,T_symbol)
		if (sym->type == TYPE_RELAY)
		{
			int i = list_pos(blkinf->entities);
			comm_handle_relay(sym,block);
			list_set_pos(blkinf->entities,i);
		}

}
		
void comm_handle_system(T_symbol sys)
{
	sig_init_net();
	env_pid_set = alloc_intset();
	intset_add(&env_pid_set,0);	/* fixme */

	comm_handle_scope(sys);
	sig_simnet();

/*	signale und blockzugehoerigkeit der connects ueberpruefen
*/
/*	sig_delete_net();
	wird in sys2net gemacht
*/
}
