
#include "actions.h"
#include "proc2net.h"
#include "sdliter.h"
#include "sortrange.h"
#include "strbuf.h"
#include "mnet.h"
#include "box_sub.h"
#include "f_parse.h"
#include "macros.h"
#include "info.h"

void copy_to_comarray(Variables dst, Variables src)
{
	size_t memsize = src->ComEntries * sizeof(ComArrayEntry);
	dst->ComEntries = src->ComEntries;
	dst->ComArray = MYrealloc(dst->ComArray,memsize);
	memcpy(dst->ComArray,src->ComArray,memsize);
}

M_Net create_basic_prenet(RefNr ref)
{
	static VarMSet prenet_label = NULL;
	static VariableStruct prenet_vars;

	M_Net net;
	M_Place pl;
	M_Transition tr;

	net = M_CreateBasicBox(NULL, NULL, NULL,
			Create_empty_Variables(&expr_vars,1), ref);

	tr = net->Transitions;
	if (!prenet_label)
	{
		prenet_label = StrToVarMSet("(pid,piid)",tr);
		copy_to_comarray(&prenet_vars,&tr->vars);
	}
	/* fixme: die Variable @c wird nutzlos herumgeschleppt */
	copy_to_comarray(&tr->vars,&prenet_vars);
	free(tr->postset->label);
	free(tr->preset->label);
	tr->preset->label = V_MScreate_copy(prenet_label,NULL);
	tr->postset->label = V_MScreate_copy(prenet_label,NULL);

	pl = net->Places;
	M_CopySingleRange(pl->range+1,macro_new_ids);
	pl->range[2] = NULL;

	pl = net->LastPlace;
	M_CopySingleRange(pl->range+1,macro_new_ids);
	pl->range[2] = NULL;

	return net;
}

void create_transform(char *tmpstr, int ground, int sigbase, int mod, int div)
{
	char *c = tmpstr;

	if (ground)
		c += sprintf(tmpstr,"%d+",ground);

	if (sigbase)
		c += sprintf(c,"(sig-%d)",sigbase);
	else
		c += sprintf(c,"%s","sig");

	if (div > 1) c += sprintf(c,"/%d",div);
	if (mod) c += sprintf(c,"%%%d",mod);
}

M_Net create_input_net(T_list signals, RefNr refnr)
{
	static MultiSet label = NULL;
	static MultiSet spontlabel;
	static VariableStruct vars;
	static node_ptr guard = NULL;

	T_signaluse siguse;
	M_Net net = create_basic_prenet(0);
	M_Transition tr = net->Transitions;
	int i;

	if (macro_new_ids == dot_set)
		Msg(FAT,"input request outside of any process");

	if (!label)
	{
		label = StrToMultiSet("in?(sig,sender',piid)",tr);
		spontlabel = StrToMultiSet("sender('sender,piid,pid,piid)",tr);
		copy_to_comarray(&vars,&tr->vars);
	}
	copy_to_comarray(&tr->vars,&vars);
	tr->label = MScreate_copy(label);
	add_trans_ref(tr,refnr,0);

	/* nur gebraucht bei parameterlosen signalen */
	if (!guard) guard = ParseAnnotation("sig=0",tr);

	for (i = 1; i < signals->count; i++)
	  DuplicateTrans(net,tr);

	tr = net->Transitions;
	for_list (signals,siguse,T_signaluse)
	{
		T_signal sig;
		char tmpstr[32];
		int j, modulo, divisor = 1;

		if (!siguse)	/* input none */
		{
			free(tr->label);
			tr->label = MScreate_copy(spontlabel);
			initbuf();
			addbuf("input none");
			tr->meaning = MYstrdup(strbuf);
			tr = tr->nexttrans;
			continue;
		}

		sig = siguse->sym->e.signal;
		tr->meaning = MYmalloc(7+strlen(siguse->sym->name));
		sprintf(tr->meaning,"input %s",siguse->sym->name);

		if (!sig->args)
		{
			tr->guard = MYrealloc(tr->guard,5*sizeof(node));
			memcpy(tr->guard,guard,5*sizeof(node));
			tr->guard[2].op1 = sig->number;
			tr = tr->nexttrans;
			continue;
		}

		initbuf();
		sprintf(tmpstr,"sig>=%u&sig<=%u",sig->number,sig->highest);
		addbuf(tmpstr);

		for (j = 0; j < sig->args->count; j++)
		{
			T_parameter parm = siguse->parm + j;
		
			modulo = sig->width[j];
			if (j == sig->args->count-1) modulo = 0;
			create_transform(tmpstr,sig->ground[j],sig->number,
					 modulo,divisor);
			divisor *= modulo;

			/* fixme: das hier kann bestimmt auch kuerzer fassen */
			if (parm->flag == PARM_UNUSED || sig->width[j] == 1)
				continue;
			else if (parm->flag == PARM_CONST && parm->value == DOT)
			{
				addbuf("&");
				addbuf(tmpstr);
				addbuf("=-1");
			}
			else if (parm->flag == PARM_CONST
				&& (parm->value == TT || parm->value == FF))
			{
				sprintf(tmpstr,"%s=%u",strchr(tmpstr,'+')+1,
						parm->value == TT? 1 : 0);
				addbuf("&");
				addbuf(tmpstr);
			}
			else if (parm->flag == PARM_VARIABLE
				&& parm->var->e.syntype->set == SORT_PID)
			{
				addbuf("&(");
				addbuf(tmpstr);
				addbuf("=-1&");
				addbuf(parm->var->name);
				addbuf("'=dot|");
				addbuf(parm->var->name);
				addbuf("'=");
				addbuf(tmpstr);
				addbuf("&");
				addbuf(parm->var->name);
				addbuf("'>=0)");
				add_var_use(tr,parm->var,True);
			}
			else if (parm->flag == PARM_VARIABLE
				&& parm->var->e.syntype->set == boolset)
			{
				sprintf(tmpstr,"%s",strchr(tmpstr,'+')+1);
				addbuf("&(");
				addbuf(tmpstr);
				addbuf("=0&");
				addbuf(parm->var->name);
				addbuf("'=ff|");
				addbuf(tmpstr);
				addbuf("=1&");
				addbuf(parm->var->name);
				addbuf("'=tt)");
				add_var_use(tr,parm->var,True);
			}
			else if (parm->flag == PARM_VARIABLE)
			{
				addbuf("&");
				addbuf(parm->var->name);
				addbuf("'=");
				addbuf(tmpstr);
				add_var_use(tr,parm->var,True);
			}
			else if (parm->flag == PARM_CONST)
			{
				addbuf("&");
				addbuf(tmpstr);
				addbuf("=");
				sprintf(tmpstr,"%u",parm->value);
				addbuf(tmpstr);
			}
			else
				Msg(FAT,"can't happen: %s:%u (%u)",
					__FILE__,__LINE__,j);
		}

		tr->guard = ParseAnnotation(strbuf,tr);
		tr = tr->nexttrans;
	}

	return net;
}

M_Net create_conditions_net(T_transition trans)
{
	static MultiSet action = NULL;
	static MultiSet dashaction;

	/* todo: andere */
	M_Net net = create_input_net(trans->signals,trans->ref->nr);
	M_Transition tr,tr2;
	T_list trs = alloc_list();
	T_symbol state;

	int i, new_size;
	
	for (tr = net->Transitions; tr; tr = tr->nexttrans)
		list_add(trs,tr);

	tr = net->Transitions;
	if (!action)
	{
		action = StrToMultiSet("q(dot,dot,pid,piid)",tr);
		dashaction = StrToMultiSet("dash(0,dot,pid,piid)",tr);
	}
		
	new_size = MSlength(tr->label)+1;
	for_list (trs,tr,M_Transition)
	{
		i = trans->states->count;
// Handle asterisk
if (!trans->states->elems)
{
for_list(state_list,state,T_symbol)
list_add(trans->states,state);
}
		for_list (trans->states,state,T_symbol)
		{
			tr2 = --i? DuplicateTrans(net,tr) : tr;
			action->com[0] = V_CONST_NULL + state->e.num;
			MSresize(tr2->label,new_size);
			MSsum(tr2->label,action);

			initbuf();
			addbuf("state ");
			addbuf(state->name);
			addbuf("; ");
			addbuf(tr2->meaning);
			free(tr2->meaning);
			tr2->meaning = MYstrdup(strbuf);

			if (!use_nextstate) continue;
			
			dashaction->com[1] = V_CONST_NULL + state->e.num;
			MSresize(tr2->label,new_size+1);
			MSsum(tr2->label,dashaction);
		}
	}
	free_list(trs);

	return net;
}

M_Net body2net(T_list transitions)
{
	T_transition trans;
	alt_set_type alt_set;
	M_Net net, pre_net;
	M_Net start_net=NULL, end_net=NULL, trans_net=NULL;

	for_list (transitions,trans,T_transition)
	{
		net = create_trans_net(trans->actions);
		if (trans->states)
		{
			pre_net = create_conditions_net(trans);
			net = net? (pre_net? M_Sequence(pre_net,net) : net)
				 : (pre_net? pre_net : NULL);
			trans_net = M_Choice(trans_net,net);
		}
		else
			start_net = net;
	}

	end_net = GetBox(SDL_PROCESS_EXIT,True);

	alt_set.repeat_net = trans_net;
	alt_set.exit_net = end_net;
	net = SDL_Iteration(alt_set,start_net);

	return net;
}

