#include "proc2net.h"
#include "symtab.h"
#include "list.h"
#include "mnet.h"
#include "macros.h"
#include "signet.h"
#include "intset.h"
#include "function.h"
#include "multiset.h"
#include "tables.h"
#include "box_sub.h"

static T_symbol c_sym;
static intset c_receivers;
static RefNr c_refnr;

void prepare_connection(T_symbol sym, intset *set, char flag, char dir)
{
	char *tmpstr;
	intset sigs;

	sig_get_relaysets(sym,&sigs,NULL,NULL,dir);
	intset_union(set,sigs);

	if (!flag) return;

	tmpstr = MYmalloc(strlen(sym->name)+4);
	sprintf(tmpstr,"%s@%u!",sym->name,dir);
	intset_add(&c_receivers,StrToAction(tmpstr));
	free(tmpstr);
}

void check_side(T_list list, intset *set, char flag)
{
	T_symbol sym;
	T_relay relinf;

	*set = alloc_intset();
	for_list (list,sym,T_symbol)
	{
		relinf = sym->e.relay;
		if ((flag? relinf->ep1 : relinf->ep2) == c_sym)
			prepare_connection(sym,set,flag,0);
		if ((flag? relinf->ep2 : relinf->ep1) == c_sym
		     && relinf->bidirectional)
			prepare_connection(sym,set,flag,1);
	}
}

void check_direction(T_list list1, T_list list2,
		     char *direction, char *blkname)
{
	int i;
	intset set1, set2;
	M_Net net;
	M_Transition tr, tr2;
	MultiSet action;
	char *tmpstr;

	c_receivers = alloc_intset();
	check_side(list1,&set1,0);
	check_side(list2,&set2,1);

	if (!intset_equal(set1,set2))
	    Msg(FAT,"%s signals of connection no.%u in block %s do not match",
		     direction,c_sym->name[strlen(c_sym->name)-1]-'a'+1,
		     blkname);

	free(set1);
	free(set2);

	if (!intset_isempty(c_receivers)) return;

	macro_name = MYmalloc(strlen(c_sym->name)+7);
	tmpstr = macro_name + sprintf(macro_name,"%s%c",c_sym->name,*direction);

	macro_ref = global_ref;
	net = GetBox(SDL_INIT,True);
	global_init = global_init? M_Sequence(global_init,net) : net;
	net = GetBox(SDL_TERM,True);
	global_term = global_term? M_Sequence(global_term,net) : net;

	macro_ref = c_refnr + (*direction == 'o');
	trans_mark[MARK_PAR_PRE] = NULL;
	net = GetBox(SDL_CONNECT,True);
	tr = check_mark(MARK_PAR_PRE,SDL_CONNECT);

	strcpy(tmpstr,"!");
	if (*direction == 'o') add_action(&current_actions,macro_name,3);
	strcpy(tmpstr,"@init");
	add_action(&global_actions,macro_name,0);
	strcpy(tmpstr,"@term");
	add_action(&global_actions,macro_name,0);
	free(macro_name);

	/* hier werden einige sehr spezielle Annahmen
	   ueber das SDL_CONNECT-Netz gemacht */
	action = MScreate_copy(tr->label);
	for (i = intset_length(c_receivers)-1; i >= 0; i--)
	{
		tr2 = i? DuplicateTrans(net,tr) : tr;
		action->nr = intset_elem(c_receivers,i+1);
		MSresize(tr2->label,2);
		MSsum(tr2->label,action);
	}
	add_to_current(net);

}

void handle_connect(T_symbol connect, T_symbol block)
{
	T_list out = connect->e.connect->outlist;
	T_list in = connect->e.connect->inlist;
	T_symbol sym;
	T_relay relinf;

	c_sym = connect;
	c_refnr = connect->ref->nr;

	/* todo: muessen signale disjunkt sein? */

	for_list (out,sym,T_symbol)
	{
		relinf = sym->e.relay;
		if (sym->blknum != connect->blknum-1)
			Msg(FAT,"cannot connect channel/signalroute %s "
				"to inner block %s",sym->name,block->name);
		if (relinf->ep1 != connect && relinf->ep2 != connect)
			Msg(FAT,"channel/signalroute %s is not connected "
				"to %s",sym->name,block->name);
	}
	for_list (in,sym,T_symbol)
	{
		relinf = sym->e.relay;
		if (sym->blknum != connect->blknum)
			Msg(FAT,"cannot connect channel/signalroute %s "
				"to outer block %s",sym->name,block->name);
		if (relinf->ep1 != connect && relinf->ep2 != connect)
			Msg(FAT,"channel/signalroute %s is not connected to"
				"the environment of %s",sym->name,block->name);
	}

	/* zunaechst Fluss von aussen nach innen behandeln */
	check_direction(out,in,"inbound",block->name);
	/* dann Fluss von innen nach aussen */
	check_direction(in,out,"outbound",block->name);

}

