#include "proc2net.h"
#include "symtab.h"
#include "list.h"
#include "macros.h"
#include "signet.h"
#include "function.h"
#include "box_sub.h"
#include "compose.h"
#include "f_parse.h"
#include "sortrange.h"
#include "actions.h"
#include "tables.h"
#include "sdliter.h"
#include "strbuf.h"

#include "debug.h"

extern T_list create_list;
T_symbol current_proc;
bool    use_nextstate;

/********************/
/***** Variable *****/
/********************/

void create_variable_net(T_symbol sym, M_Net *var_net, M_Net *var_init,
				       M_Net *var_term, MultiSet *scope)
{
	M_Net net;
	T_syntype type = sym->e.syntype;

	macro_decl_set = type->set;
	macro_init = type->def_init;
	if (type->set == SORT_PID)
		macro_decl_set = sig_get_all_pids();
	else if (type->def_init == NONE)
		macro_init = intset_elem(type->set,1);
	
	macro_name = MYmalloc(strlen(sym->name)+6);
	strcpy(macro_name,sym->name);
	macro_ref = sym->ref->nr;
	sym->ref->proc = current_ref;

	net = GetBox(SDL_VARIABLE,True);
	*var_net = *var_net? M_ConcurrentComposition(*var_net,net) : net;

	net = GetBox(SDL_VAR_INIT,True);
	*var_init = *var_init? M_Sequence(*var_init,net) : net;

	net = GetBox(SDL_VAR_TERM,True);
	*var_term = *var_term? M_Sequence(net,*var_term) : net;

	add_action(scope,macro_name,4);
	strcat(macro_name,"@init");
	add_action(scope,macro_name,3);
	sprintf(macro_name,"%s@term",sym->name);
	add_action(scope,macro_name,2);
	free(macro_name);
}

/******************************/
/* Verschiedene Unterroutinen */
/******************************/

void find_dash_state(T_symbol scope)
{
	T_symbol sym;

	use_nextstate = True;
	for (sym = scope->e.struc->symtab; sym; sym = sym->next)
		if (sym->type == TYPE_STATE && sym->e.num == -1) return;
	use_nextstate = False;
}

void add_label_actions(T_symbol scope, MultiSet *actions)
{
	T_symbol sym;

	for (sym = scope->e.struc->symtab; sym; sym = sym->next)
		if (sym->type == TYPE_LABEL)
		{
			initbuf();
			addbuf(sym->name);
			addbuf("@join");
			add_action(actions,strbuf,2);
		}
}

/********************/
/***** Prozedur *****/
/********************/

void add_call_action(M_Transition tr, Action *act)
{
	MultiSet set;
	char *tmpstr = MYmalloc(18+strlen(macro_name)*2);

	if (!act) Msg(FAT,"no call action in procedure net?");
	act->com[act->arity++] = StrToCom(macro_name,&tr->vars,tr);

	MSresize(tr->label,MSlength(tr->label)+1);
	sprintf(tmpstr,"%s@init(proc,piid,%s)",macro_name,macro_name);
	set = StrToMultiSet(tmpstr,tr);
	MSsum(tr->label,set);
	free(tmpstr);
}

void add_procedure_parms(M_Net *prcdnet, T_prcsinf prcdinf, MultiSet *scope)
{
	M_Net net;
	M_Transition tr_call1 = check_mark(MARK_PAR_PRE,SDL_PROCEDURE);
	M_Transition tr_call2 = check_mark(MARK_PAR_POST,SDL_PROCEDURE);
	T_fmlparm parm;
	Action *act;
	Action_nr callnr, retnr;
	char *add = macro_name + strlen(macro_name);

	strcpy(add,"@call");
	callnr = StrToAction(macro_name);
	strcpy(add,"@return");
	retnr = StrToAction(macro_name);

	for_list (prcdinf->fmlparms,parm,T_fmlparm)
	{
		/* hier nur in-parms behandeln */
		if (parm->type) continue;
		macro_name = parm->sym->name;
		macro_decl_set = parm->sym->e.syntype->set;
		macro_ref = parm->sym->ref->nr;
		parm->sym->ref->proc = current_ref;

		net = GetBox(SDL_PRCD_IN_PARM,True);
		*prcdnet = M_ConcurrentComposition(*prcdnet,net);

		act = MSconj_is_in(callnr, 2+prcdinf->initial, tr_call1->label);
		add_call_action(tr_call1,act);
		act = MSconj_is_in(callnr, 2+prcdinf->initial, tr_call2->label);
		add_call_action(tr_call2,act);
		prcdinf->initial++;	/* ahem, leichter Missbrauch */

		add = MYmalloc(strlen(macro_name)+6);
		add_action(scope,macro_name,4);
		sprintf(add,"%s@init",macro_name);
		add_action(scope,add,3);
		sprintf(add,"%s@term",macro_name);
		add_action(scope,add,2);
		free(add);
	}
}

void add_parameter_term_nets(M_Net *term_net, T_list fmlparms)
{
	M_Net net;
	T_fmlparm parm;

	for_list (fmlparms,parm,T_fmlparm)
	{
		if (parm->type) continue;
		macro_name = parm->sym->name;
		net = GetBox(SDL_VAR_TERM,True);
		*term_net = *term_net? M_Sequence(net,*term_net) : net;
	}
}

void relabel_parms(T_symbol proc, T_list fmlparms)
{
	T_fmlparm parm;

	for_list (fmlparms,parm,T_fmlparm)
	{	/* nur in/out-Parameter umbenennen */
		if (!parm->type) continue;
		initbuf();
		addbuf(parm->sym->name);
		addbuf("@");
		addbuf(proc->name);
		free(parm->sym->name);
		parm->sym->name = MYstrdup(strbuf);
	}
}

void restore_parm_names(T_list fmlparms)
{
	T_fmlparm parm;

	for_list (fmlparms,parm,T_fmlparm)
		if (parm->type) *strrchr(parm->sym->name,'@') = 0;
}

void create_procedure_net(T_symbol proc, M_Net *prcsproc_net, MultiSet *scope)
{
	M_Net net;
	M_Net proc_net, proc_init, proc_term;
	MultiSet proc_scope = MScreate_empty_set();
	T_symbol sym, var;
	T_list body;
	T_prcsinf prcdinf = proc->e.struc->prcsinf;
	T_fmlparm parm;
	intset old_pids = macro_id_set;
	char *save_name;
	int maximum = prcdinf->maximum;

	proc_net = proc_init = proc_term = NULL;
	macro_id_set = alloc_intset();

	if (maximum < 0)
		maximum = intset_length(macro_new_ids) * (-prcdinf->maximum); 

	intset_add_interval(&macro_id_set,1,maximum);
	prcdinf->idset = macro_id_set;

	relabel_parms(proc,prcdinf->fmlparms);

	for_list (proc->e.struc->entities,sym,T_symbol) switch (sym->type)
	{
	    case TYPE_BODY:
		body = sym->e.list;
		break;
	    case TYPE_VARDEF:
		current_ref = proc->ref->nr;
		for_list (sym->e.list,var,T_symbol)
			create_variable_net(var,&proc_net,&proc_init,
						&proc_term,&proc_scope);
		break;
	    case TYPE_PROCEDURE:
		create_procedure_net(sym,prcsproc_net,&proc_scope);
		break;
	    case TYPE_CONST:
	    case TYPE_SYNTYPE:
		break;
	    default:
		Msg(FAT,"can't happen: %s:%u (%u)",
			__FILE__,__LINE__,sym->type);
	}

	add_parameter_term_nets(&proc_term,prcdinf->fmlparms);

	find_dash_state(proc);
	current_ref = proc->ref->nr;
	macro_body = body2net(body);
	if (proc_init) macro_body = SDL_Sequence(proc_init,macro_body);
	if (proc_term) macro_body = SDL_Sequence(macro_body,proc_term);
	if (proc_net) macro_body = M_ConcurrentComposition(proc_net,macro_body);

	macro_init = prcdinf->num_states;
	macro_name = MYmalloc(strlen(proc->name)+8);
	strcpy(macro_name,proc->name);
	macro_ref = proc->ref->nr;

	trans_mark[MARK_PAR_PRE] = trans_mark[MARK_PAR_POST]
			= trans_mark[MARK_RETURN_PAR] = NULL;
	net = GetBox(SDL_PROCEDURE,True);

	save_name = macro_name;
	add_procedure_parms(&net,prcdinf,&proc_scope);
	macro_name = save_name;
	strcpy(macro_name,proc->name);

	add_label_actions(proc,&proc_scope);
	add_action(&proc_scope,"q",4);
	add_action(&proc_scope,"dash",4);
	net = sync_net(net,proc_scope);
	set_correct_dimensions(net);
	net->x_dim += RASTER;
	*prcsproc_net = *prcsproc_net? 
		M_ConcurrentComposition(*prcsproc_net,net) : net;

	macro_ref = global_ref;
	net = GetBox(SDL_INIT,True);
	global_init = global_init? M_Sequence(global_init,net) : net;

	net = GetBox(SDL_TERM,True);
	global_term = global_term? M_Sequence(net,global_term) : net;

	sprintf(macro_name,"%s@call",proc->name);
	add_action(scope,macro_name,2+prcdinf->initial);
	sprintf(macro_name,"%s@return",proc->name);
	add_action(scope,macro_name,2);
	sprintf(macro_name,"%s@init",proc->name);
	add_action(&global_actions,macro_name,0);
	sprintf(macro_name,"%s@term",proc->name);
	add_action(&global_actions,macro_name,0);
	free(macro_name);

	for_list (prcdinf->fmlparms,parm,T_fmlparm)
		if (parm->type) add_action(scope,parm->sym->name,4);

	/* muss wieder rueckgaengig gemacht werden, da eine Prozedur
	   mehrmals uebersetzt werden kann, falls nicht -L gesetzt ist */
	restore_parm_names(prcdinf->fmlparms);

	macro_id_set = old_pids;
}

/*******************/
/***** Prozess *****/
/*******************/

void add_channel_guard(M_Transition tr, int cap)
{
	int i,j;
	char tmpstr[16];

	initbuf();
	for (i = 0; i <= cap; i++)
	{
		if (i) addbuf("|");
		sprintf(tmpstr,"i=%u",i);
		addbuf(tmpstr);
		for (j = 1; j <= cap; j++)
		{
			sprintf(tmpstr,"&sig%u%cdot",j, (cap-j < i)? '#' : '=');
			addbuf(tmpstr);
			strncpy(tmpstr,"&sen",4);
			addbuf(tmpstr);
		}
	}
	add_guard(tr,ParseAnnotation(strbuf,tr));
}

void create_initial_instances(T_prcsinf prcsinf)
{
	M_Net net;
	T_fmlparm parm;
	char tmpstr[16];
	int i;

	for (i = 1; i <= prcsinf->initial; i++)
	{
		initbuf();
		addbuf(macro_name);
		addbuf("@create(");
		/* fixme: pin1 statt konstanten */
		sprintf(tmpstr,"%u",intset_elem(macro_new_ids,i));
		addbuf(tmpstr);
		addbuf(",dot");
		for_list (prcsinf->fmlparms,parm,T_fmlparm)
		{
			intset type = parm->sym->e.syntype->set;
			int value = (type == SORT_PID)?
					DOT : intset_elem(type,1);
			switch (value)
			{
			    case FF: addbuf(",ff"); break;
			    case TT: addbuf(",tt"); break;
			    case DOT:addbuf(",dot"); break;
			    default: sprintf(tmpstr,",%u",value);
				     addbuf(tmpstr); break;
			}
		}
		addbuf(")");
		
		macro_ref = global_ref;
		net = GetBox(SDL_CREATE,True);
		free(net->Transitions->label);
		net->Transitions->label
			= StrToMultiSet(strbuf,net->Transitions);
		global_create = global_create? M_Sequence(global_create,net)
					     : net;
	}
}

void set_create_sets(T_symbol process)
{
	T_createreq cr;
	intset idset;

	macro_decl_set = alloc_intset();	/* parent */
	if (process->e.struc->prcsinf->initial)
		intset_add(&macro_decl_set,DOT);
	for_list (create_list,cr,T_createreq)
		if (cr->child == process)
		{
			sig_get_pointsets(cr->parent,&idset,NULL,NULL);
			intset_union(&macro_decl_set,idset);
		}

	macro_new_ids = alloc_intset();		/* offspring */
	intset_add(&macro_new_ids,DOT);	/* fehlgeschlagene creates */
	for_list (create_list,cr,T_createreq)
		if (cr->parent == process)
		{
			if (cr->child->e.struc->subtype != TYPE_PROCESS)
				Msg(FAT,"tried to create %s, but it's not "
					"a process (doh!)",cr->child->name);
			sig_get_pointsets(cr->child,&idset,NULL,NULL);
			intset_union(&macro_new_ids,idset);
		}
}

void add_process_parms(T_list fmlparms, M_Net *var_net, M_Net *var_term,
					MultiSet *scope)
{
	T_fmlparm parm;
	M_Net net;

	for_list (fmlparms,parm,T_fmlparm)
	{
		macro_name = MYmalloc(strlen(parm->sym->name)+6);
		strcpy(macro_name,parm->sym->name);
		macro_decl_set = parm->sym->e.syntype->set;
		macro_ref = parm->sym->ref->nr;
		parm->sym->ref->proc = current_ref;

		net = GetBox(SDL_PRCD_IN_PARM,True);
		*var_net = *var_net? M_ConcurrentComposition(*var_net,net):net;
		net = GetBox(SDL_VAR_TERM,True);
		*var_term = *var_term? M_Sequence(net,*var_term) : net;

		add_action(scope,macro_name,4);
		strcat(macro_name,"@init");
		add_action(scope,macro_name,3);
		sprintf(macro_name,"%s@term",parm->sym->name);
		add_action(scope,macro_name,2);
		free(macro_name);
	}
}

void modify_create_actions(T_list fmlparms, char *name)
{
	T_fmlparm parm;
	M_Transition tr;
	Action_nr create_nr;
	MultiSet label;
	int arity = 2;

	tr = check_mark(MARK_PAR_PRE,SDL_PROCESS);
	initbuf();
	addbuf(name);
	addbuf("@create");
	create_nr = StrToAction(strbuf);
	label = StrToMultiSet("dummy@init(dot,piid,dummy)",tr);

	for_list (fmlparms,parm,T_fmlparm)
	{
		Action *act = MSconj_is_in(create_nr,arity,tr->label);
		char *name = parm->sym->name;
		ComVar varnr = StrToCom(name,&tr->vars,tr);

		if (!act) Msg(FAT,"create action missing in procedure net");
		act->com[act->arity++] = varnr;
		arity++;

		initbuf();
		addbuf(name);
		addbuf("@init");
		label->nr = StrToAction(strbuf);
		label->com[2] = varnr;
		MSresize(tr->label,MSlength(tr->label)+1);
		MSsum(tr->label,label);
	}

	free(label);
	tr = check_mark(MARK_PAR_POST,SDL_PROCESS);
	arity = 2;

	for_list (fmlparms,parm,T_fmlparm)
	{
		Action *act = MSconj_is_in(create_nr,arity,tr->label);
		char *name = parm->sym->name;
		ComVar varnr = StrToCom(name,&tr->vars,tr);

		if (!act) Msg(FAT,"create action missing in procedure net");
		act->com[act->arity++] = varnr;
		arity++;
	}
}

void handle_process(T_symbol process)
{
	M_Net net;
	MultiSet scope;
	T_symbol sym,var;
	T_prcsinf prcsinf = process->e.struc->prcsinf;
	T_list body, created_procs;
	char *tmpstr; 
	M_Net var_net, var_init, var_term, prcsproc_net;

	if (!strcmp(process->name,"env")) return;
	scope = MScreate_empty_set();
	var_net = var_init = var_term = prcsproc_net = NULL;

	current_proc = process;
	macro_id_set = dot_set;
	sig_get_pointsets(process,&macro_new_ids,NULL,NULL);
	prcsinf->idset = alloc_intset();
	intset_union(&prcsinf->idset,macro_new_ids);

	for_list (process->e.struc->entities,sym,T_symbol) switch (sym->type)
	{
	    case TYPE_BODY:
		body = sym->e.list;
		break;
	    case TYPE_VARDEF:
		current_ref = process->ref->nr;
		for_list (sym->e.list,var,T_symbol)
			create_variable_net(var,&var_net,&var_init,
						&var_term,&scope);
		break;
	    case TYPE_PROCEDURE:
		if (!(sdlflag & SDLF_NO_PRCD_COPIES)) break;
		create_procedure_net(sym,&prcsproc_net,&scope);
		break;
	    case TYPE_CONST:
	    case TYPE_SYNTYPE:
		break;
	    default:
		Msg(FAT,"can't happen: %s:%u (%u)",
			__FILE__,__LINE__,sym->type);
	}

	if (!(sdlflag & SDLF_NO_PRCD_COPIES))
	{
		created_procs = alloc_list();
		for_list (prcsinf->called_procs,sym,T_symbol)
			if (sym->blknum <= process->blknum+1)
			{	/* verschachtelte Prozeduren werden in
				   create_procedure_net erzeugt */
				for_list (created_procs,var,T_symbol)
					if (var == sym) goto already_created;
				list_add(created_procs,sym);
				create_procedure_net(sym,&prcsproc_net,&scope);
already_created:
			}
		free_list(created_procs);
	}

	current_ref = process->ref->nr;
	add_process_parms(prcsinf->fmlparms,&var_net,&var_term,&scope);

	macro_name = MYmalloc(strlen(process->name)+8);
	tmpstr = macro_name + sprintf(macro_name,"%s",process->name);

	macro_ref = global_ref;
	net = GetBox(SDL_INIT,True);
	global_init = global_init? M_Sequence(global_init,net) : net;

	create_initial_instances(prcsinf);

	strcpy(tmpstr,"@init");
	add_action(&global_actions,macro_name,0);
	strcpy(tmpstr,"@create");
	add_action(&global_actions,macro_name,2+prcsinf->fmlparms->count);
	strcpy(tmpstr,"@term");
	add_action(&process_actions,macro_name,0);
	strcpy(tmpstr,"!");
	add_action(&current_actions,macro_name,3);
	free(macro_name);

	find_dash_state(process);
	macro_body = body2net(body);
	if (var_init) macro_body = SDL_Sequence(var_init,macro_body);
	if (var_term) macro_body = SDL_Sequence(macro_body,var_term);
	if (var_net) macro_body = M_ConcurrentComposition(var_net,macro_body);

	macro_name = process->name;
	sig_get_pointsets(process,&macro_id_set,&macro_signals,&macro_senders);

	/* todo: nur wenn "input none" genutzt wird */
	macro_senders2 = alloc_intset();
	intset_union(&macro_senders2,macro_senders);
	intset_union(&macro_senders2,macro_id_set);

	macro_max = intset_length(macro_id_set);
	set_create_sets(process);
	macro_init = prcsinf->num_states;
	macro_capacity = prcsinf->capacity;
	macro_ref = process->ref->nr;

	/* nur fuer unbounded version */
	macro_capacity_plus_eins = prcsinf->capacity + 1;
	macro_baseid = intset_elem(macro_id_set,1) - 1;

	trans_mark[MARK_PAR_PRE] = trans_mark[MARK_PAR_POST]
		= trans_mark[MARK_RETURN_PAR] = NULL;
	change_net_version(SDL_PROCESS,&sdl_lastprocused,prcsinf->version);
	net = GetBox(SDL_PROCESS,True);

	free(macro_senders2);
	if (trans_mark[MARK_RETURN_PAR])
	    add_channel_guard(trans_mark[MARK_RETURN_PAR],prcsinf->capacity);
	modify_create_actions(prcsinf->fmlparms,process->name);
	set_correct_dimensions(net);
	net->x_dim += RASTER;
	if (prcsproc_net) net = M_ConcurrentComposition(net,prcsproc_net);

	add_label_actions(process,&scope);
	add_action(&scope,"self",4);
	add_action(&scope,"parent",4);
	add_action(&scope,"offspring",4);
	add_action(&scope,"offspring@get",3);
	add_action(&scope,"offspring@put",3);
	add_action(&scope,"sender",4);
	add_action(&scope,"q",4);
	add_action(&scope,"dash",4);
	add_action(&scope,"in?",3);
	net = sync_net(net,scope);

	add_to_current(net);
}

