#include "proc2net.h"
#include "f_parse.h"
#include "function.h"
#include "symtab.h"
#include "mnet.h"
#include "macros.h"
#include "actions.h"
#include "strbuf.h"
#include "signet.h"
#include "sortrange.h"
#include "tables.h"
#include "compose.h"
#include "box_sub.h"
#include "node.h"

M_Net create_nextstate_net(T_symbol state, T_ref ref)
{
	static MultiSet label = NULL;
	static MultiSet dashlabel;
	static VariableStruct vars;
	static Action_nr q_nr;

	M_Net net = create_basic_prenet(ref->nr);
	M_Transition tr = net->Transitions;
	Action *q;

	if (!label)
	{
		label = StrToMultiSet("q(dot,dot,pid,piid)",tr);
		dashlabel = StrToMultiSet("q(dot,'dash,pid,piid),"
					  "dash('dash,0,pid,piid)",tr);
		copy_to_comarray(&vars,&tr->vars);
		q_nr = StrToAction("q");
	}
	copy_to_comarray(&tr->vars,&vars);

	tr->label = MScreate_copy(use_nextstate? dashlabel : label);
	q = MSis_in(q_nr,4,tr->label);

	if (!state || state->e.num > 0)
		q->com[1] = V_CONST_NULL + (state? state->e.num : 0);

	if (state)
	{
	  tr->meaning = MYmalloc(11+strlen(state->name));
	  sprintf(tr->meaning,"nextstate %s",state->name);
	}
	else
	  tr->meaning = MYstrdup(macro_id_set == dot_set? "stop" : "return");

	return net;
}

M_Net create_single_output_net(T_list via, T_symbol dst, T_signaluse siguse)
{
	static MultiSet dstlabel = NULL;
	static MultiSet reclabel;
	static intset pidcset, boolcset;
	static node_ptr recguard;
	static VariableStruct vars;

	M_Net net = create_basic_prenet(0);
	M_Transition tr = net->Transitions;
	T_signal sig = siguse->sym->e.signal;
	T_list list, route;
	T_symbol goal;
	intset pids;
	int i, newsize;
	char *tmpstr;

	add_trans_ref(tr,siguse->sym->ref->nr,0);

	/* sender und signal setzen */
	if (!dstlabel)
	{
		dstlabel = StrToMultiSet("dummy!(sig,piid,rec)",tr);
		reclabel = StrToMultiSet("parent(rec,rec,pid,piid)",tr);
		recguard = ParseAnnotation("rec=0",tr);
		pidcset = alloc_intset();
		intset_add_interval(&pidcset,0,num_pi_ids-1);
		boolcset = alloc_intset();
		intset_add_interval(&boolcset,0,1);
		copy_to_comarray(&vars,&tr->vars);
	}
	copy_to_comarray(&tr->vars,&vars);
	tr->label = MScreate_empty_set();

	tmpstr = MYmalloc(16);
	if (!sig->args)
	{
		sprintf(tmpstr,"sig=%u",sig->number);
		free(tr->guard);
		tr->guard = ParseAnnotation(tmpstr,tr);
	}
	else
	{
		int mult = 1;
		intset pidvars = alloc_intset();

		initbuf();
		sprintf(tmpstr,"sig=%u",sig->number);
		addbuf(tmpstr);
		for (i = 0; i < sig->args->count; i++)
		{
			T_parameter parm = siguse->parm + i;

			if (i) mult *= sig->width[i-1];
			if (parm->flag == PARM_UNUSED) continue;
			if (parm->flag == PARM_CONST && parm->value == DOT)
				continue;
			if (parm->flag == PARM_VARIABLE
				&& (parm->var->e.syntype->set == SORT_PID
				 || parm->var->e.syntype->set == boolset))
			{
				intset_add(&pidvars,i);
				add_var_use(tr,parm->var,False);
				sprintf(tmpstr,"+c%u",i);
				addbuf(tmpstr);
			}
			else if (parm->flag == PARM_VARIABLE)
			{
				add_var_use(tr,parm->var,False);
				addbuf("+");
				if (sig->ground[i]) addbuf("(");
				addbuf(parm->var->name);
				if (sig->ground[i] > 0)
					sprintf(tmpstr,"-%u",sig->ground[i]);
				else if (sig->ground[i] < 0)
					sprintf(tmpstr,"+%u",-sig->ground[i]);
				if (sig->ground[i]) addbuf(tmpstr);
				if (sig->ground[i]) addbuf(")");
			}
			else if (parm->flag == PARM_CONST)
			{
				int j = parm->value - sig->ground[i];
				if (!j) continue;
				sprintf(tmpstr,"+%u",j);
				addbuf(tmpstr);
			}
			else
				Msg(FAT,"can't happen: %s:%u (%u)",
					__FILE__,__LINE__,i);

			if (!i) continue;
			sprintf(tmpstr,"*%u",mult);
			addbuf(tmpstr);
		}

		for_intset (pidvars,pids)
		{
			T_parameter parm = siguse->parm + *pids;
			bool boolvar = (parm->var->e.syntype->set == boolset);
			char *expr;
			
			sprintf(tmpstr,"c%u",*pids);
			expr = MYmalloc(strlen(parm->var->name)*3
				+strlen(tmpstr)*2+19);

			if (boolvar)
				sprintf(expr,"&(%s=ff&%s=0|%s=tt&%s=1)",
					parm->var->name,tmpstr,
					parm->var->name,tmpstr);
			else
				sprintf(expr,"&(%s=dot&%s=0|%s>=0&%s=%s+1)",
					parm->var->name,tmpstr,
					parm->var->name,tmpstr,parm->var->name);

			addbuf(expr);
			free(expr);

			AddSingleRangeToDef(&tr->def,
				StrToCom(tmpstr,&tr->vars,tr),
				-1, boolvar? boolcset : pidcset, True);
		}

		free(tr->guard);
		pids = tr->def;
		tr->guard = ParseAnnotation(strbuf,tr);
		free(tr->def);
		tr->def = pids;
		free(pidvars);
	}

	free(tmpstr);
	tr->meaning = MYmalloc(8+strlen(siguse->sym->name));
	sprintf(tr->meaning,"output %s",siguse->sym->name);

	newsize = MSlength(tr->label)+2;

	list = dst? sig_make_unique(via) : via;
	if (!list->count)
		Msg(FAT,"no way to send signal %s in structure %s",
			siguse->sym->name, current_proc->name);

	for (i = 1; i < list->count; i++) DuplicateTrans(net,tr);

	tr = net->Transitions;
	for_list (list,route,T_list)
	{
		/* richtigen aktionsterm zum senden aussuchen */
		T_symbol relay = sig_get_head(route);
		tmpstr = MYmalloc(strlen(relay->name)+4);

		sprintf(tmpstr,"%s@%u!",relay->name,
			current_proc == relay->e.relay->ep2);
		dstlabel->nr = StrToAction(tmpstr);
		MSresize(tr->label,newsize);
		MSsum(tr->label,dstlabel);
		free(tmpstr);

		/* empfaenger richtig setzen */
		switch ((int)dst)
		{
		    case 0:	/* processident */
			goal = sig_get_tail(route);
			sig_get_pointsets(goal,&pids,NULL,NULL);
			if (intset_length(pids) != 1)
				Msg(WAR,"several instances of process %s exist;"
					" sending signal %s to first instance",
					goal->name, siguse->sym->name);
			recguard[2].op1 = intset_elem(pids,1);
			add_guard(tr,recguard);
			break;
		    default:	/* variable */
			reclabel->nr = StrToAction(dst->name);
			MSsum(tr->label,reclabel);
			break;
		}
		tr = tr->nexttrans;
	}
	if (dst) free_list(list);

	return net;
}

M_Net create_signal_output_net(T_output output, T_signaluse sig)
{
	M_Net net = NULL;
	T_list pathlist = sig_find_paths(current_proc,
				sig->sym->e.signal->highest);
		/* hoechste Nummer nehmen wegen Schnelligkeit */
	T_list list = alloc_list(), newlist;
	T_symbol via, dst = output->dst;

	for_list (pathlist,newlist,T_list) list_add(list,newlist);

	for_list (output->via,via,T_symbol)
	{
		newlist = sig_restrict_via(list,via);
		free_list(list);
		list = newlist;
	}

	if (dst && (dst->type == TYPE_STRUCTURE
		/* wegen refer_offscope */  || dst->type == TYPE_UNDECLARED))
	{
		newlist = sig_restrict_goal(list,dst);
		free_list(list);
		list = newlist;
		dst = NULL;
	}

	net = create_single_output_net(list,dst,sig);

/* TODO: core dump
	free_list(list);
	for_list (pathlist,list,T_list) free_list(list);
	free_list(pathlist);
*/
	return net;
}

M_Net create_output_net(T_output output, T_ref ref)
{
	M_Net net = NULL, single_net;
	M_Place pl;
	M_Transition tr;
	T_signaluse sig;

	if (macro_new_ids == dot_set)
		Msg(FAT,"output request outside of any process");

	for_list (output->signals,sig,T_signaluse)
	{
		single_net = create_signal_output_net(output,sig);
		net = net? M_Sequence(net,single_net) : single_net;
	}

	for (pl = net->Places; pl; pl = pl->nextplace)
		add_place_ref(pl,ref->nr,(pl->label == ENTRY)? PRE_REF :
					  (pl->label == EXIT)? POST_REF : 0);
	for (tr = net->Transitions; tr; tr = tr->nexttrans)
		add_trans_ref(tr,ref->nr,0);

	return net;
}

M_Net create_assignment_net(T_assignment assign)
{
	M_Net net = create_basic_prenet(assign->ref->nr);
	M_Transition tr = net->Transitions;

	int last_node;
	char *tmpstr = MYmalloc(strlen(assign->var->name)+2);
	sprintf(tmpstr,"%s'",assign->var->name);

	expr_tree = MYmalloc(100 * sizeof(node));
	expr_tree->typ = N_HEAD;
	expr_tree->op1 = 1;
	expr_tree->op2 = 100;

	tr->label = MScreate_empty_set();
	add_var_use(tr,assign->var,True);
	last_node = create_guard_nodes(tr,assign->var,assign->expr);
	
	add_node(N_EQ,
		add_node(N_VAR,StrToCom(tmpstr,&tr->vars,tr),-1),
		add_node(N_KLAMM,last_node,0));

	free(tr->guard);
	tr->guard = TreeToGuard(expr_tree,False);
	free(tmpstr);

	tr->meaning = MYmalloc(11+strlen(assign->var->name));
	sprintf(tr->meaning,"task %s:=...",assign->var->name);

	return net;
}

M_Net create_task_net(T_list list)
{
	M_Net net = NULL, single_net;
	T_assignment assign;

	for_list (list,assign,T_assignment)
	{
		assign->ref->proc = current_ref;
		single_net = create_assignment_net(assign);
		net = net? M_Sequence(net,single_net) : single_net;
	}

	return net;
}

int add_range_node(int r, int qnode)
{
	int lower = lower_bound(r);
	int upper = upper_bound(r);
	int n1 = (lower != MIN_NEG)? add_node(N_CONST,lower,0) : 0;
	int n2 = (upper != MAX_POS)? add_node(N_CONST,upper,0) : 0;

	if (lower == upper) return add_node(N_EQ,qnode,n1);
	if (n2) n2 = add_node(N_LE,qnode,n2);
	if (n1) n1 = add_node(N_GE,qnode,n1);

	return n1? n2? add_node(N_AND,n1,n2) : n1 : n2;
}

M_Net create_decision_net(T_decision dec, T_ref ref)
{
	M_Net retnet;
	M_Net *net = MYmalloc(dec->answers->count * sizeof(M_Net));
	M_Place pl;
	M_Transition tr;
	node_ptr thisexpr = NULL;

	int qnode, node, lastor, lastand = 0;
	int i, r;

	net[0] = create_basic_prenet(0);
	tr = net[0]->Transitions;
	add_trans_ref(tr,ref->nr,0);

	tr->label = MScreate_empty_set();
	if (dec->question)
	{
		expr_tree = MYmalloc(100 * sizeof(node));
		expr_tree->typ = N_HEAD;
		expr_tree->op1 = 1;
		expr_tree->op2 = 100;

		qnode = add_node(N_KLAMM,
				 create_guard_nodes(tr,NULL,dec->question),0);

		thisexpr = expr_tree;
	}

	for (i = 1; i < dec->answers->count; i++)
		net[i] = M_CreateNetCopy(net[0]);

	for (i = 0; i < dec->answers->count; i++)
	{
		T_list answer = list_elem(dec->answers,i,T_list);
		T_list actions = list_elem(dec->transitions,i,T_list);
		T_ref answer_ref = list_elem(dec->refs,i,T_ref);
		M_Net transnet = create_trans_net(actions);
		M_Transition tr = net[i]->Transitions;

		if (transnet) net[i] = M_Sequence(net[i],transnet);

		add_trans_ref(tr,answer_ref->nr,0);

		if (!dec->question) continue;
		if (!answer)
		{
			tr->guard = TreeToGuard(thisexpr,False);
			tr->meaning = MYstrdup("else");
			thisexpr = NULL;
			continue;
		}
		tr->meaning = MYstrdup("decision");
		expr_tree = thisexpr;
		lastor = 0;
		for_list (answer,r,int)
		{
			node = add_range_node(r,qnode);
			if (lastor) node = add_node(N_OR,node,lastor);
			lastor = node;
		}
		thisexpr = MYmalloc(expr_tree->op2 * sizeof(node));
		memcpy(thisexpr,expr_tree,expr_tree->op2 * sizeof(node));
		tr->guard = TreeToGuard(expr_tree,False);

		/* else-Baum vorbereiten */
		expr_tree = thisexpr;
		node = add_node(N_KLAMM,expr_tree->op1-1,0);
		node = add_node(N_NOT,node,0);
		if (lastand) node = add_node(N_AND,node,lastand);
		lastand = node;
		thisexpr = expr_tree;	/* realloc war moeglich */
	}

	for (i = 1; i < dec->answers->count; i++)
		net[0] = M_Choice(net[0],net[i]);

	retnet = net[0];

	for (pl = retnet->Places; pl; pl = pl->nextplace)
	{
		if (pl->label == ENTRY) add_place_ref(pl,ref->nr,PRE_REF);
		if (pl->label == EXIT) add_place_ref(pl,ref->nr,POST_REF);
	}

	free(net);
	free(thisexpr);
	return retnet;
}

void modify_call(T_call call, M_Transition tr, Action_nr callnr)
{
	ComVar varnr;
	Action *act;
	T_fmlparm parm;
	T_node node;
	int i = 0, arity = 0;

	for_list (call->fmlparms,parm,T_fmlparm)
	{
		if (parm->type) continue;

		node = list_elem(call->actparms,i++,T_node);
		if (!node)
		{	/* uninitialisiert: nimm niedrigsten Wert */
			intset type = parm->sym->e.syntype->set;
			varnr = V_CONST_NULL +
				((type == SORT_PID)? DOT : intset_elem(type,1));
		}
		else if (node->constant)
		{
			varnr = V_CONST_NULL + eval_node(node);
		}
		else if (node->type == NODE_VAR)
		{
			T_symbol var = (T_symbol) node->op1;
			varnr = StrToCom(var->name,&tr->vars,tr);
			add_var_use(tr,var,False);
		}
		else
		{
			int lastnode;
			char tmpstr[16];

			sprintf(tmpstr,"@pin%u",arity);
			varnr = StrToCom(tmpstr,&tr->vars,tr);

			expr_tree = MYmalloc(100 * sizeof(node));
			expr_tree->typ = N_HEAD;
			expr_tree->op1 = 1;
			expr_tree->op2 = 100;

			lastnode = create_guard_nodes(tr,NULL,node);

			add_node(N_EQ,
				add_node(N_VAR, varnr, -1),
				add_node(N_KLAMM, lastnode, 0));

			add_guard(tr,TreeToGuard(expr_tree,False));
		}

		act = MSis_in(callnr,2+arity,tr->label);
		act->com[act->arity++] = varnr;
		arity++;
	}
}

M_Net create_call_net(T_call call, T_ref ref)
{
	M_Net net;
	M_Place pl;
	M_Transition calltr, returntr;
	MultiSet parmscope;
	MultiSet set;
	T_symbol proc = call->sym;
	T_fmlparm parm;
	int i = 0;

	macro_name = proc->name;
	macro_decl_set = proc->e.struc->prcsinf->idset;
	macro_ref = ref->nr;

	place_mark[MARK_REF_PAR] = NULL;
	trans_mark[MARK_CALL_PAR] = trans_mark[MARK_RETURN_PAR] = NULL;
	net = GetBox(SDL_PROC_CALL,True);
	calltr = check_mark(MARK_CALL_PAR,SDL_PROC_CALL);
	returntr = check_mark(MARK_RETURN_PAR,SDL_PROC_CALL);

	pl = place_mark[MARK_REF_PAR];
	add_place_ref(pl,ref->nr,0);
	add_place_ref(pl,current_ref,32768);
	add_place_ref(pl,proc->ref->nr,64);

	if (proc->blknum <= current_proc->blknum
			&& (sdlflag & SDLF_NO_PRCD_COPIES))
	{
		calltr->guard[2].typ = N_CONST;
		calltr->guard[2].op1 = DOT;
		returntr->guard[2].typ = N_CONST;
		returntr->guard[2].op1 = DOT;
	}

	relabel_parms(proc,call->fmlparms);

	/* erst die in-Parameter */
	initbuf();
	addbuf(proc->name);
	addbuf("@call");
	modify_call(call,calltr,StrToAction(strbuf));

	/* jetzt die in/out-Parameter */
	parmscope = MScreate_empty_set();
	for_list (call->fmlparms,parm,T_fmlparm)
	{
		T_node node = list_elem(call->actparms,i++,T_node);
		char *tmpstr;
		if (!parm->type) continue;

		macro_arg = parm->sym->name;
		macro_name = ((T_symbol)node->op1)->name;
		macro_ref = parm->sym->ref->nr;
		parm->sym->ref->proc = proc->ref->nr;

		net = M_ConcurrentComposition(net,
				GetBox(SDL_PRCD_INOUT_PARM,True));

		add_place_ref(net->Places,((T_symbol)node->op1)->ref->nr,32);
		if (macro_id_set != dot_set)
			add_place_ref(net->Places,current_ref,32768);
		tmpstr = MYmalloc(strlen(parm->sym->name)+25);

		sprintf(tmpstr,"%s@init(proc,pid,usedpiid)",parm->sym->name);
		MSresize(calltr->label,MSlength(calltr->label)+1);
		set = StrToMultiSet(tmpstr,calltr);
		MSsum(calltr->label,set);
		*strchr(tmpstr,'(') = 0;
		add_action(&parmscope,tmpstr,3);

		sprintf(tmpstr,"%s@term(proc,pid,usedpiid)",parm->sym->name);
		MSresize(returntr->label,MSlength(returntr->label)+1);
		set = StrToMultiSet(tmpstr,returntr);
		MSsum(returntr->label,set);
		*strchr(tmpstr,'(') = 0;
		add_action(&parmscope,tmpstr,3);

		free(tmpstr);
	}

	restore_parm_names(call->fmlparms);

	return sync_net(net,parmscope);
}

M_Net create_create_net(T_call call, T_ref ref)
{
	M_Net net;
	M_Transition tr;
	Action_nr callnr;
	T_fmlparm parm;
	int i = 0;

	macro_name = call->sym->name;
	macro_ref = ref->nr;
	trans_mark[MARK_PAR_PRE] = NULL;
	net = GetBox(SDL_PROC_CREATE,True);
	tr = check_mark(MARK_PAR_PRE,SDL_PROC_CREATE);

	initbuf();
	addbuf(call->sym->name);
	addbuf("@create");
	callnr = StrToAction(strbuf);

	call->fmlparms = call->sym->e.struc->prcsinf->fmlparms;
	if (call->fmlparms->count < call->actparms->count)
		Msg(FAT,"too many arguments in create request "
			"for process %s",call->sym->name);
	for_list (call->fmlparms,parm,T_fmlparm)
	{
		T_node node = list_elem(call->actparms,i++,T_node);
		intset type = parm->sym->e.syntype->set;

		if (!node) continue;
		if ((node->sort == ETYPE_INTEGER
			&& type != SORT_PID && type != boolset)
		   || (node->sort == ETYPE_BOOLEAN && type == boolset)
		   || (node->sort == ETYPE_PID && type == SORT_PID))
			continue;
		Msg(FAT,"argument type mismatch in create request "
			"for process %s",call->sym->name);
	}
	while (call->actparms->count < call->fmlparms->count)
		list_add(call->actparms,NULL);

	modify_call(call,tr,callnr);

	return net;
}

M_Net create_join_net(T_symbol goal, T_ref ref)
{
	macro_name = goal->name;
	macro_ref = ref->nr;
	return GetBox(SDL_JOIN,True);
}

M_Net create_trans_net(T_list actions)
{
	T_action action;
	M_Net t_net = NULL;
	M_Net net;

	if (!actions->count) return NULL;

	for_list (actions,action,T_action)
	{
		switch(action->type)
		{
		    case ACTION_NEXTSTATE:
			net = create_nextstate_net(action->e.state,action->ref);
			break;
		    case ACTION_OUTPUT:
			net = create_output_net(action->e.output,action->ref);
			break;
		    case ACTION_TASK:
			net = create_task_net(action->e.list);
			break;
		    case ACTION_DECISION:
			net=create_decision_net(action->e.decision,action->ref);
			break;
		    case ACTION_CALL:
			net = create_call_net(action->e.call,action->ref);
			break;
		    case ACTION_CREATE:
			net = create_create_net(action->e.call,action->ref);
			break;
		    case ACTION_JOIN:
			net = create_join_net(action->e.goal,action->ref);
		}

		if (action->ref)
			action->ref->proc = current_ref;

		if (action->label)
		{
			M_Net label_net;

			macro_name = action->label->name;
			macro_ref = action->label->ref->nr;
			action->label->ref->proc = current_ref;
			label_net = GetBox(SDL_LABEL,True);
			net = net? M_Sequence(label_net,net) : label_net;
		}

		t_net = t_net? (net? M_Sequence(t_net,net) : t_net)
			     : (net? net : NULL);
	}

	return t_net;
}

