#include "list.h"
#include "info.h"
#include "symtab.h"
#include "intset.h"
#include "function.h"
#include "strbuf.h"
#include "sortrange.h"

T_list state_list;
static T_list pid_list;
static T_list sigsym_list;
static T_list signal_list;

void init_info(void)
{
	state_list = alloc_list();
	pid_list = alloc_list();
	sigsym_list = alloc_list();
	signal_list = alloc_list();
}

void delete_info(void)
{
    free_list(state_list);
    free_list(pid_list);
    free_list(sigsym_list);
    free_list(signal_list);
}

void add_state_info(T_symbol proc)
{
	T_symbol sym;

	for (sym = proc->e.struc->symtab; sym; sym = sym->next)
		if (sym->type == TYPE_STATE && sym->e.num > 0)
		{
			T_info state = alloc_info();
			state->proc = proc->name;
			state->name = sym->name;
			state->id = sym->e.num;
			list_add(state_list,state);
		}
}

void add_pid_info(T_symbol process, intset pids)
{
	Integer *id;
	int i = 0;

	for_intset (pids,id)
	{
		T_info pid = alloc_info();
		pid->proc = process->name;
		pid->index = ++i;
		pid->id = *id;
		list_add(pid_list,pid);
	}
}

void add_signal_info(T_symbol sigsym)
{
	T_info signal = alloc_info();
	signal->sym = sigsym;
	list_add(sigsym_list,signal);
}

void expand_signal_info(T_symbol sigsym)
{
	T_syntype type;
	T_signal siginf = sigsym->e.signal;
	T_info signal;
	int i, j, index, modulo, div;
	char number[16];

	if (siginf->args) revert_list(siginf->args);

	for (i = 0; i <= siginf->highest - siginf->number; i++)
	{
	    initbuf();
	    addbuf(sigsym->name);

	    div = siginf->highest - siginf->number + 1;
	    if (siginf->args) addbuf("(");

	    if (siginf->args) for_list (siginf->args,type,T_syntype)
	    {
		j = siginf->args->count - list_pos(siginf->args) - 1;

		index = i;
		modulo = siginf->width[j];
		div /= modulo;
		index /= div;
		if (j < siginf->args->count - 1) index %= modulo;

		if (type->set == SORT_PID)
		{
			if (index == 0) addbuf("Null");
			else if (index == 1) addbuf("System");
			else
			{
				T_info pid = list_elem(pid_list,index-2,T_info);
				addbuf(pid->proc);
				sprintf(number,"[%u]",pid->index);
				addbuf(number);
			}
		}
		else if (type->set == boolset)
		{
			addbuf(index? "True" : "False");
		}
		else
		{
			sprintf(number,"%d",siginf->ground[j] + index);
			addbuf(number);
		}
		if (j) addbuf(",");
	    }

	    if (siginf->args) addbuf(")");

	    signal = alloc_info();
	    signal->name=NULL;
	    signal->proc = MYstrdup(strbuf);
	    signal->id = siginf->number + i;
	    list_add(signal_list,signal);
	}

	if (siginf->args) revert_list(siginf->args);
}

void write_info_file(char *filename)
{
	T_info state, pid, signal;

	FILE *file;

	if (!(file = fopen(filename,"w")))
		Msg(FAT,"could not open file %s for writing\n",filename);

	printf("Writing state information to %s\n",filename);

	fprintf(file,"PEP\n");
	fprintf(file,"SDL_Information\n");
	fprintf(file,"FORMAT_N\n");

	fprintf(file,"STATE\n");
	for_list (state_list,state,T_info)
		fprintf(file,"%s.%s = %u\n",state->proc,state->name,state->id);
	
	fprintf(file,"PID\n");
	for_list (pid_list,pid,T_info)
		fprintf(file,"%s[%u] = %u\n",pid->proc,pid->index,pid->id);
	
	for_list (sigsym_list,signal,T_info)
		expand_signal_info(signal->sym);
	fprintf(file,"SIGNAL\n");
	for_list (signal_list,signal,T_info)
		fprintf(file,"%s = %u\n",signal->proc,signal->id);
	
	fclose(file);
}

