#include <stdio.h>
#include "intset.h"
#include "function.h"

intset alloc_intset()
{
	intset set = MYmalloc(4*sizeof(Integer));
	*set = 1;
	return set;
}

void intset_add(intset *set, Integer value)
{
	Integer i, *ip, *jp;

	for (i = 1, ip = *set+1; i < **set; i++, ip++)
	{
		if (*ip == value) return;	
		if (*ip < value) continue;
		break;
	}

	if (!((**set)++ & 3))	/* mod 4 = 0 */
	{
		*set = MYrealloc(*set,(3+**set)*sizeof(Integer));
		ip = *set+i;
	}
	for (jp = *set+**set-1; jp > ip; jp--) *jp = *(jp-1);
	*ip = value;
}

void intset_add_interval(intset *set, Integer val1, Integer val2)
{
	intset newset = MYmalloc((val2-val1+2)*sizeof(Integer));
	Integer i, *wp = newset+1;

	*newset = val2-val1+2;
	for (i = val1; i <= val2; i++) *wp++ = i;
	intset_union(set,newset);
	free(newset);
}

void intset_union(intset *set1, intset set2)
{
	intset newset = MYmalloc((**set1+*set2+2)*sizeof(Integer));
	Integer i = **set1-1, j = *set2-1;
	Integer *ip = *set1+1, *jp = set2+1;
	Integer *wp = newset+1;

	*newset = 1;
	while (i && j && ++(*newset))
	{
		*wp++ = (*ip < *jp)? *ip : *jp;
		if (*ip < *jp)
		{	i--; ip++;	}
		else if (*ip > *jp)
		{	j--; jp++;	}
		else
		{	i--; j--; ip++; jp++;	}
	}

	if (!i) { i = j; ip = jp; }

	while (i-- && ++(*newset)) *wp++ = *ip++;

	free(*set1);
	*set1 = newset;
}

void intset_subtract(intset *set1, intset set2)
{
	intset newset = MYmalloc((**set1+3)*sizeof(Integer));
	Integer i = **set1-1, j = *set2-1;
	Integer *ip = *set1+1, *jp = set2+1;
	Integer *wp = newset+1;

	*newset = 1;
	while (i && j)
	{
		if (*ip == *jp)
		{	i--; j--; ip++; jp++;	}
		else if (*ip < *jp)
		{	*wp++ = *ip++;	i--;
			(*newset)++;		}
		else
		{	jp++; j--;		}
	}

	while (i-- && ++(*newset)) *wp++ = *ip++;

	free(*set1);
	*set1 = newset;
}

void intset_print(intset set)
{
	Integer i, *ip = set+1;

	for (i = *set-1; i > 0; i--) printf("%u ",*ip++);
	printf("\n");
}

/* fixme: nicht gerade effizient */
bool    intset_isin(intset set, Integer value)
{
	Integer *ip;

	for (ip = set+*set-1; ip > set; ip--)
		if (*ip <= value) break;

	return *ip == value && ip > set;
}

bool    intset_issubset(intset super, intset sub)
{
	Integer i = *super-1, j = *sub-1;
	Integer *ip = super+1, *jp = sub+1;

	while (i && j)
	{
		if (*ip < *jp)
		{	i--; ip++;	}
		else if (*ip > *jp)
		{	return False;	}
		else
		{	i--; j--; ip++; jp++;	}
	}

	return !j;
}
