/****************************************************************************/
/* main.c                                                                   */
/*                                                                          */
/* Hauptprogramm des SDL-Compilers					    */
/*									    */
/* Stefan Schwoon, Sommer 98						    */
/* (hervorgegangen aus backend/main.c)					    */
/****************************************************************************/

  /************/
  /* includes */
  /************/

#include <string.h>
#include <time.h>
#include <ctype.h>
#include <unistd.h>

#include "main.h"
#include "mnet.h"
#include "commnet.h"
#include "proc2net.h"
#include "tables.h"
#include "block.h"
#include "compose.h"
#include "ref.h"
#include "info.h"
#include "property.h"
#include "macros.h"
#include "strbuf.h"

/****************************************************************************/

extern int yyparse(void);
extern int yynerrs;

/* kann diese Funktionsdeklarationen nicht durch ref.h einlesen, da sonst
   ein heilloser Kreis von includes entsteht =0 */
extern void hide_refs(M_Net);
extern void restore_refs(M_Net);
extern void write_ref_file(char*,M_Net,char*);

/****************************************************************************/

  /*******************/
  /* Datenstrukturen */
  /*******************/

Cardinal32 flag = 0;		/* globale Programm-Flags                  */
Cardinal32 sdlflag = 0;		/* SDL-spezifische Flags		   */

char    *source;		/* Quelldatei b(pn)2 bzw. bei syncmnet	    */
char	*pep_file = NULL;

char	*SrcBuffer = NULL;
char   **SrcLine;

Cardinal var_style = VAR_BOTH;	/* Default: @Index nur wenn noetig */
Cardinal scale = 1;
Cardinal max_arc_vars = 2;

int sdl_maxnum = 8;
int sdl_defcap = 1;
int sdl_inqcap = 1;
int sdl_prcsinst = 1;
int sdl_prcdinst = -1;

int sdl_procver = 0;
int sdl_chanver = 0;
int sdl_lastprocused = 0;
int sdl_lastchanused = 0;

			   /* meanings if flag == True: */
			   /*---------------------------*/
bool GL_Frags     = False, /* allow for code fragments  */
     GL_OldSyn    = False, /* allow for old syntax      */
     GL_ProcDecl  = False, /* there is at least one ... */
     GL_ProcCall  = False, /* ... and it was called     */
     GL_ExplFree  = False, /* check memory management   */
     GL_ParserMsg = False; /* Msg tells extra infos     */

T_symbol STree;		/* Ergebnis des Parsens */

/****************************************************************************/

static void usage(char *myname)
{
  fprintf(stderr,
    "\n%s -- translate an SDL program into an M-Net\n\n"

    "Usage: %s [options] <sdlfile> [FileOptions]\n\n"

    "     Options:\n"
    "      -L        no implicit local procedure subtypes\n"
    "      -S        no scoping\n"
    "      -BC#      use different channel net\n"
    "      -BP#      use different process net\n"
    "      -DC#      set default channel capacity\n"
    "      -DN#      set default number range\n"
    "      -DP#      set default maximum number of process instances\n"
    "      -DQ#      set default capacity of input queues\n"
    "      -DZ#      set default maximum number of procedure instances\n"

    "\n     FileOptions:\n"
    "      -o <filename>   specify alternate M-Net file\n"
    "      -r <filename>   specify alternate reference file\n"
    "      -i <filename>   specify alternate file for additional information\n"
    "      -p <filename>   specify alternate file for additional properties\n"

    "\nAll file names not explicitly stated on the command line will\n"
    "default to the input file's basename plus appropriate extensions.\n\n"
 
    "Version 0.1.4 (17.06.1998) Stefan Schwoon\n\n", myname, myname);

  exit(EXIT_FAILURE);
}


/****************************************************************************/
/* parse_def_arg						    	    */
/* Einlesen von -Dx-Parametern						    */

static void parse_def_arg(char **c, char *name)
{
	int *goal = NULL;
	char **tmp=NULL;

	switch (*++(*c))
	{
	    case 'N': goal = &sdl_maxnum; break;
	    case 'C': goal = &sdl_defcap; break;
	    case 'Q': goal = &sdl_inqcap; break;
	    case 'Z': goal = &sdl_prcdinst; break;
	    case 'P': goal = &sdl_prcsinst; break;
	    default: usage(name);
	}

	*goal=labs(strtol(++(*c), tmp, 10));
	if (tmp == c)
	  usage(name);
	else
	  c=tmp;
}

/****************************************************************************/
/* parse_def_arg						    	    */
/* Einlesen von -Bx-Parametern						    */

static void parse_box_arg(char **c, char *name)
{
	int *goal = NULL;

	switch (*++(*c))
	{
	    case 'C': goal = &sdl_chanver; break;
	    case 'P': goal = &sdl_procver; break;
	    default: usage(name);
	}

	if (!isdigit(*++*c)) usage(name);
	*goal = **c - '0';
}

/*****************************************************************************/
/* Main program.                                                             */
/*****************************************************************************/

int main(int argc, char **argv)
{

	int		i;
	char		*c, *idx;
	char    	**dptr;
	char		*sdl_ref_file = NULL;
	char		*sdl_info_file = NULL;
	char		*sdl_prop_file = NULL;
	int		props_stated = 0;
	clock_t 	time;
	M_Net		net;

	bpn_templates=False;
	/* Kommandozeilen-Parameter durchgehen, Dateinamen setzen */
	source = NULL;
	dptr = &source;

	for (i = 1; i < argc; i++)
	{
		if (*argv[i] == '-')
			for (c=argv[i]; *++c;) switch(*c)
			{
				case 'o': dptr = &pep_file;
					  break;
				case 'r': dptr = &sdl_ref_file;
					  break;
				case 'i': dptr = &sdl_info_file;
					  break;
				case 'p': dptr = &sdl_prop_file;
					  props_stated++;
					  break;
				case 'S': flag |= NO_SCOPING;
					  break;
				case 'L': sdlflag |= SDLF_NO_PRCD_COPIES;
					  break;
				case 'D': parse_def_arg(&c,argv[0]);
					  break;
				case 'B': parse_box_arg(&c,argv[0]);
					  break;
				default:  fprintf(stderr,"unknown flag: "
						         "'%c'\n",*c);
					  usage(argv[0]);
			}
		else	/* kein Flag, also Dateiname */
		{
			if (!dptr) usage(argv[0]);
			*dptr = argv[i];

			c = MYmalloc(strlen(argv[i])+12);
			strcpy(c,argv[i]);
			idx = strrchr(c,'.');
			if (!idx)
			  strcat(c,".");
			idx = strrchr(c,'.');

			/* Standardendungen setzen */
			if (dptr == &source)
			{
				strcpy(idx,".hl_net");
				pep_file = MYstrdup(c);
				strcpy(idx,".sdl_prop");
				sdl_prop_file = MYstrdup(c);
			}
			if (dptr == &source || dptr == &pep_file)
			{
				strcpy(idx,".sdl_hl_ref");
				sdl_ref_file = MYstrdup(c);
				strcpy(idx,".sdl_info");
				sdl_info_file = MYstrdup(c);
			}

			free(c);
			dptr = NULL;
		}
	}

	/* kein Dateiname angegeben? */
	if (!source) usage(argv[0]);
	if (sdl_maxnum>MAX_POS) sdl_maxnum=MAX_POS;

	time = clock();

	if (!(yyin = fopen(source,"r")))
	{
		fprintf(stderr,"Error: Could not open file %s!\n",source);
		exit(2);
	}

	/* Datei mit Zusatzinformationen einlesen */
	read_sdl_properties(sdl_prop_file,props_stated);

	printf("Parsing program %s ...\n",source);
  
	GL_ParserMsg = True;
	s_init();
	init_references();
	init_info();

#ifdef YYDEBUG
	{
		extern int yydebug;
		yydebug = 1;
	}
#endif

	if (yyparse() || yynerrs)		/* hier Aufruf des Parsers */
	{
		fclose(yyin);
		fprintf(stderr,"Parser indicated errors; aborting\n");
		exit(3);
	}

	GL_ParserMsg = False;
	fclose(yyin);

	InitMultiSets();
	InitVarMSets();
	InitAllSymbolTables();
	InitBlockTable();

	/* Zwischenschritt: Kommunikationsstrukturen untersuchen */
	comm_handle_system(STree);

	/* hier Einleitung der Netzerzeugung */
	net = sys2net(STree);

	hide_refs(net);
	HandleNet(net);		/* erzeugtes Netz in Datei schreiben */
	restore_refs(net);

	/* zusaetzliche Dateien erzeugen */
	if (sdl_ref_file) write_ref_file(sdl_ref_file,net,source);
	if (sdl_info_file) write_info_file(sdl_info_file);

	/* todo: Speicherfreigabe */
	delete_sdl_tree(STree);
	M_Remove_net(&net);
#if 0
	if (sdl_ref_file)
	  MYfree(sdl_ref_file);
	if (sdl_info_file)
	  MYfree(sdl_info_file);
	if (sdl_prop_file)
	  MYfree(sdl_prop_file);
	if (pep_file)
	  MYfree(pep_file);
	delete_info();
#endif
	RemoveAllSymbolTables();
	RemoveBlockTable();
	RemoveTemplates();
	MSremove(mmDummy);
	MSremove(mmLeaf);
	MSremove(mmSub);
	MSremove(mmSingle);
	VSremove(V_mmDummy);
	VSremove(V_mmLeaf);
	delbuf();

/*	printf("Time needed: %f sec.\n\n",(float)(clock()-time)/CLOCKS_PER_SEC);
*/
	exit(EXIT_SUCCESS);
}

