/****************************************************************************/
/* main.h                                                                   */
/* Burkhard Graves, 10.10.93                                                */
/* Lars Jenner, Juni 1994                                                   */
/* Martin Ackermann, Fruehjahr 95                                           */
/* Burkhard Bieber 1995/96:                                                 */
/*   - Blockbehandlung ergaenzt                                             */
/*   - Phantomtransitionsbehandlung ergaenzt                                */
/*   - Schoenheitskorrekturen                                               */
/* Martin Ackermann, Winter 1996/97                                         */
/*   - Prozeduren, andere Referenzen, mehr Effizienz, Fehler beseitigt      */
/****************************************************************************/

#ifndef __MAIN_H__
#define __MAIN_H__

  /************/
  /* includes */
  /************/

#include <stdio.h>
#include "symtab.h"
#include "memlib.h"
#include "frontend.h"

  /*********************************************/
  /* Allgemeine Typdefinitionen und Konstanten */
  /*********************************************/

  /* Bit-Darstellung fuer die Kommandozeilenparameter */

#define USE_PROC_SET                1
#define CHECK_NET                   2
#define REMOVE_DUPL                 4
#define ELIMINATE_DEAD              8
#define PAR_TERM                   16
#define NO_SCOPING                 32
#define BIG_LOOP                   64
#define MEDIUM_LOOP               128
#define SHORT_FILE                256
#define BLOCK_OUTPUT              512
#define PHANTOM_TRANS_OUTPUT     1024
#define CHECK_NAMES              2048
#define INTERSECT_WARNING        4096
#define NO_DISJUNCT_IDSETS	32768

#define SYNCHRONIZE              8192
#define DO_RESTRICTION          16384

/* SDL-spezifische Flags */
#define SDLF_NO_PRCD_COPIES	1

extern Cardinal scale,            /* Skalierungsfaktor fuer das Netz */
                max_arc_vars,     /* maximale Anzahl von Variablen in einem 
				                              Kantenlabel */
                max_proc_inst;    /* maximale Anzahl von Aktivierungen einer
				                                 Prozedur */
extern Cardinal32 flag, sdlflag;  /* globale Programm-Flags */

extern int sdl_maxnum;		/* Max. number for sdl types Integer and Natural */
extern int sdl_defcap;		/* Kapazitaet von Kanaelen */
extern int sdl_inqcap;		/* Kapazitaet von Input-Queues */
extern int sdl_prcsinst;	/* Anzahl Instanzen von Prozessen */ 
extern int sdl_prcdinst;	/* Anzahl Instanzen von Prozeduren */

extern int sdl_procver;		/* Version des Prozedurnetzes */
extern int sdl_chanver;		/* Version des Channelnetzes */
extern int sdl_lastprocused;	/* zuletzt benutzte Version des Prozedurnetzes*/
extern int sdl_lastchanused;	/* zuletzt benutzte Version des Channelnetzes */

extern char *source;              /* Quelldatei b(pn)2 bzw. bei syncmnet
                                     das zu synchronisierende Netz          */
extern char *pep_file;		  /* Ausgabedatei */

extern T_symbol STree;

  /**********************************/
  /* Defines fuer das Ausgabeformat */
  /**********************************/

#define exit_postfix	"@e"   /* Postfix fuer die Exit-Aktion */
#define and_sym		" & "  /* Symbol fuer logisches AND */

extern Cardinal var_style;  /* Ausgabe-Style fuer Kommunikationsvariablen */

#define VAR_BOTH	0  /* real_name UND com_nr ausgeben */
#define VAR_COM		1  /* nur com_nr ausgeben */
#define VAR_REAL	2  /* nur real_name ausgeben */
#define VAR_SAFEREAL	3  /* nur real_name ausgeben, indizieren falls noetig */

  /******************************/
  /* Defines fuer Stringlaengen */
  /******************************/

#define MAX_ANNO_LENGTH  2048  /* maximale Laenge einer Annotation  */
#define MAX_IDENT_LENGTH  256  /* maximale Laenge eines Bezeichners */

  /***************************/
  /* Globale Datenstrukturen */
  /***************************/

extern FILE *yyin;

extern char *SrcBuffer;  /* Buffer fuer den Quelltext */
extern char **SrcLine;   /* Array von Zeigern auf die einzelnen */
                         /* Zeilenanfaenge im Quelltext */

  /*********************************************/
  /* Leeranweisung fuer leere Schleifenruempfe */
  /*********************************************/

#define skip

  /**********************************************************/
  /* Datentyp zur internen Darstellung von Variablen(namen) */
  /**********************************************************/

typedef unsigned short int Var;

  /**********************/
  /* HL-Action-Struktur */
  /**********************/

#define MaxAnzComVars  256 /* max. Anz. Com.-Vars. in einer Annotation */
#define MaxArity        10 /* maximale Aritaet einer HL-Aktion */

typedef Cardinal Action_nr;
typedef Cardinal ComVar;

typedef struct Action_
{ Action_nr	nr;		/* Aktionsnummer */
  Cardinal	arity;		/* Aritaet der Aktion */
  ComVar	com[MaxArity];	/* Kommunikationsvariablen */
  Cardinal	anzahl;		/* Anzahl des Auftretens in der Multimenge */ 
} Action, *MultiSet;

  /*********************/
  /* Variable-Struktur */
  /*********************/

typedef struct Variable_
{ ComVar	nr;      /* Nummer der Kommunikationsvariablen */
  Cardinal	anzahl;	 /* Anzahl des Auftretens in der Multimenge */ 
} Variable, *VarMSet;

  /************************************************/
  /* Darstellung einer Multimenge bei der Ausgabe */
  /************************************************/

extern bool MSdisplay_style;  /* globale Variable */

#define MS_WIDE  False
#define MS_SHORT True

  /*********************************************************************/
  /* Maximal vorgesehene Anzahl von Aktionsnamen in der Symboltabelle; */
  /* wird dynamisch an die tatsaechlich benoetigte Groesse angepasst!  */
  /*********************************************************************/

#define ACTION_BUF_SIZE 100

  /********************************************************************/
  /* Datentyp Symboltabelle fuer die Aktions-/Variablennamen (Parser) */
  /********************************************************************/

typedef char             *SymTabEntry_Pars;
typedef SymTabEntry_Pars *SymTab;

extern SymTab  ActTable;
extern SymTab  VarTable;
extern SymTab  RefTable;
extern SymTab  TplTable;

  /********************************************************************/
  /* Datentyp fuer eingelesene Referenzen, die zum Quellnetz gehoeren */
  /********************************************************************/

typedef struct SyncRefEntry_ {
  /* Achtung: Fuer das hl-backend muss s_ref an erster Stelle stehen und 
              s_type an zweiter Stelle!                                       */
  RefStruct              s_ref;         /* Definition im Quelltext            */
  char     	      	 s_type;	/* Typ                                */
  char                  *s_description; /* naehere Spezifikation des Typs     */
  char                  *s_meaning;     /* Bedeutung                          */
} SyncRefEntry, *SyncRefEntryPtr;

#endif  /* #ifndef __MAIN_H__ */

  /*** Dateiende main.h ***/

