#include "symtab.h"
#include "function.h"
#include "mnet.h"
#include "compose.h"
#include "node.h"
#include "f_parse.h"
#include "proc2net.h"

bool booleval(T_node node)
{
	int eval = eval_node(node);

	if (eval != TT && eval != FF)
		Msg(FAT,"can't happen: %s:%u (%u)",__FILE__,__LINE__,eval);

	return eval == TT;
}

int eval_node(T_node node)
{
	switch (node->type)
	{
	    case NODE_CONST:
		return (int) node->op1;
	    case NODE_VAR:
		Msg(FAT,"can't happen: %s:%u",__FILE__,__LINE__);
	    case NODE_PAREN:
		return eval_node(node->op1);
	    case NODE_IMPLIES:
		return (!booleval(node->op1) || booleval(node->op2))?  TT : FF;
	    case NODE_OR:
		return (booleval(node->op1) || booleval(node->op2))? TT : FF;
	    case NODE_XOR:
		return (booleval(node->op1) != booleval(node->op2))? TT : FF;
	    case NODE_AND:
		return (booleval(node->op1) && booleval(node->op2))? TT : FF;
	    case NODE_EQ:
		return (eval_node(node->op1) == eval_node(node->op2))? TT : FF;
	    case NODE_NE:
		return (eval_node(node->op1) != eval_node(node->op2))? TT : FF;
	    case NODE_GT:
		return (eval_node(node->op1) > eval_node(node->op2))? TT : FF;
	    case NODE_GE:
		return (eval_node(node->op1) >= eval_node(node->op2))? TT : FF;
	    case NODE_LT:
		return (eval_node(node->op1) < eval_node(node->op2))? TT : FF;
	    case NODE_LE:
		return (eval_node(node->op1) <= eval_node(node->op2))? TT : FF;
	    case NODE_PLUS:
		return eval_node(node->op1) + eval_node(node->op2);
	    case NODE_MINUS:
		return eval_node(node->op1) - eval_node(node->op2);
	    case NODE_MULT:
		return eval_node(node->op1) * eval_node(node->op2);
	    case NODE_DIV:
		return eval_node(node->op1) / eval_node(node->op2);
	    case NODE_MOD:
		return eval_node(node->op1) % eval_node(node->op2);
	    case NODE_REM:
		{
		  int a,b;
		  a=eval_node(node->op1);
		  b=eval_node(node->op2);
		  return(a-b*(a/b));
		}
	    case NODE_NEG:
		return -eval_node(node->op1);
	    case NODE_NOT:
		return (!booleval(node->op1))? TT : FF;
	    default:
		Msg(FAT,"can't happen: %s:%u(%u)",__FILE__,__LINE__,node->type);
	}
	return 0;	/* gammel */
}

T_node create_node(char type,T_node op1, T_node op2,
		   char reqsort, char newsort)
{
	T_node node = alloc_node();

	node->type = type;
	node->constant = op1->constant && (!op2 || op2->constant);
	node->op1 = op1;
	node->op2 = op2;

	if ((op2 && op1->sort != op2->sort)
			|| (reqsort != ETYPE_ANY && op1->sort != reqsort))
		Msg(ERR,"type mismatch in expression");

	node->sort = (newsort == ETYPE_ANY)? op1->sort : newsort;

	return node;
}

#define child(x) create_guard_nodes(tr,var,node->x)

int create_guard_nodes(M_Transition tr, T_symbol var, T_node node)
{
	switch (node->type)
	{
	    case NODE_CONST:
		return add_node(N_CONST, (int) node->op1, 0);
	    case NODE_VAR:
	    {
		ComVar nr;
		T_symbol sym = (T_symbol) node->op1;
		char *tmpstr = MYmalloc(strlen(sym->name)+2);

		add_var_use(tr,sym,False);
		sprintf(tmpstr,"%s%s",(sym == var)? "'" : "", sym->name);
		nr = StrToCom(tmpstr,&tr->vars,tr);
		free(tmpstr);

		return add_node(N_VAR, nr, -1);
	    }
	    case NODE_PAREN:
		return add_node(N_KLAMM, child(op1), 0);
	    case NODE_IMPLIES:
		return add_node(N_KLAMM, add_node(N_OR, add_node(N_KLAMM,
			add_node(N_NOT, child(op1), 0), 0), child(op2)), 0);
	    case NODE_OR:
		return add_node(N_OR,child(op1),child(op2));
	    case NODE_XOR:
		return add_node(N_KLAMM, add_node(N_UNEQ,
			add_node(N_KLAMM, child(op1), 0),
			add_node(N_KLAMM, child(op2), 0)), 0);
	    case NODE_AND:
		return add_node(N_AND,child(op1),child(op2));
	    case NODE_EQ:
		return add_node(N_EQ,child(op1),child(op2));
	    case NODE_NE:
		return add_node(N_UNEQ,child(op1),child(op2));
	    case NODE_GT:
		return add_node(N_GT,child(op1),child(op2));
	    case NODE_GE:
		return add_node(N_GE,child(op1),child(op2));
	    case NODE_LT:
		return add_node(N_LT,child(op1),child(op2));
	    case NODE_LE:
		return add_node(N_LE,child(op1),child(op2));
	    case NODE_PLUS:
		return add_node(N_PLUS,child(op1),child(op2));
	    case NODE_MINUS:
		return add_node(N_MIN,child(op1),child(op2));
	    case NODE_MULT:
		return add_node(N_MULT,child(op1),child(op2));
	    case NODE_DIV:
		return add_node(N_DIV,child(op1),child(op2));
	    case NODE_MOD:
		return add_node(N_MOD,child(op1),child(op2));
	    case NODE_REM:
		return add_node(N_REM,child(op1),child(op2));
	    case NODE_NEG:
		return add_node(N_SIGN, child(op1), 0);
	    case NODE_NOT:
		return add_node(N_KLAMM, add_node(N_NOT,
			add_node(N_KLAMM, child(op1), 0), 0), 0);
	    default:
		Msg(FAT,"can't happen: %s:%u(%u)",__FILE__,__LINE__,node->type);
	}
	return 0;	/* dito */
}

void free_node(T_node node)
{
	switch (node->type)
	{
	    case NODE_IMPLIES:
	    case NODE_OR:
	    case NODE_XOR:
	    case NODE_AND:
	    case NODE_EQ:
	    case NODE_NE:
	    case NODE_GT:
	    case NODE_GE:
	    case NODE_LT:
	    case NODE_LE:
	    case NODE_PLUS:
	    case NODE_MINUS:
	    case NODE_MULT:
	    case NODE_DIV:
	    case NODE_MOD:
	    case NODE_REM:
		free_node(node->op2);
	    case NODE_PAREN:
	    case NODE_NEG:
	    case NODE_NOT:
		free_node(node->op1);
	    case NODE_CONST:
	    case NODE_VAR:
		break;
	    default:
		Msg(FAT,"can't happen: %s:%u(%u)",__FILE__,__LINE__,node->type);
	}
	free(node);
}
