#include "proc2net.h"
#include "macros.h"
#include "signet.h"
#include "tables.h"
#include "box_sub.h"
#include "f_parse.h"

M_Net global_init;
M_Net global_create;
M_Net global_term;
M_Net current_net;

MultiSet global_actions;
MultiSet process_actions;
MultiSet current_actions;
RefNr global_ref;
RefNr current_ref;

#define add_to_current(net) \
    current_net = current_net? M_ConcurrentComposition(current_net,net) : net;

void add_action(MultiSet *set, char *name, int arity)
{ 
	Action_nr nr;
	if (!MSis_in(nr = StrToAction(name),arity,*set));
	{ 
		mmSingle->anzahl    = 1;
		mmSingle->nr        = nr;
		mmSingle->arity     = arity;
		MSresize(*set,MSlength(*set)+1);
		MSsum(*set,mmSingle);
	}
}

void add_var_use(M_Transition tr, T_symbol sym, bool change)
{
	MultiSet set;
	char *tmpstr = MYmalloc(15+strlen(sym->name)*3);

	sprintf(tmpstr,"%s(%s%s,%s%s,pid,piid)",sym->name,
		change? "'" : "", sym->name, sym->name, change? "'" : "");

	MSresize(tr->label,MSlength(tr->label)+1);
	set = StrToMultiSet(tmpstr,tr);
	if (!MSis_in(set->nr,4,tr->label)) MSsum(tr->label,set);
	free(tmpstr);
}

void add_guard(M_Transition tr, node_ptr extra)
{
	int num_nodes = tr->guard->op2 + extra->op2 - 1;

	tr->guard = MYrealloc(tr->guard, num_nodes * sizeof(node));
	memcpy(tr->guard+tr->guard->op2-1, extra, extra->op2 * sizeof(node));
	tr->guard->op2 = num_nodes;
}

M_Transition check_mark(int mark, int netnum)
{
	M_Transition tr = trans_mark[mark];
	if (!tr) Msg(FAT,"missing mark in template net %s",tmpl[netnum].file);
	return tr;
}

M_Transition DuplicateTrans(M_Net net, M_Transition tr)
{
	M_STEdge st;
	M_TSEdge ts;
	M_Transition tr_new = M_DuplicateTrans(net,tr,&st,&ts,NULL);
	int memsize;

	tr_new->label = MScreate_copy(tr->label);
	tr_new->meaning = tr->meaning? MYstrdup(tr->meaning) : NULL;

	tr_new->vars.ComEntries = tr->vars.ComEntries;
	memsize = tr->vars.ComEntries * sizeof(ComArrayEntry);
	tr_new->vars.ComArray = MYmalloc(memsize);
	memcpy(tr_new->vars.ComArray,tr->vars.ComArray,memsize);

	tr_new->def = MYmalloc(memsize = *(tr->def) * sizeof(Integer));
	memcpy(tr_new->def,tr->def,memsize);

	tr_new->guard = MYmalloc(memsize = tr->guard->op2 * sizeof(node));
	memcpy(tr_new->guard,tr->guard,memsize);

	tr_new->RefEntries = tr->RefEntries;
	tr_new->ref = MYmalloc(memsize = tr->RefEntries * sizeof(Ref));
	memcpy(tr_new->ref,tr->ref,memsize);
	
	tr_new->BlockEntries = tr->BlockEntries;
	tr_new->blocks = MYmalloc(memsize = tr->BlockEntries * sizeof(BlockNr));
	memcpy(tr_new->blocks,tr->blocks,memsize);
	
	/* todo: Phantom-Transitionen */
	
	return tr_new;
}

void add_place_ref(M_Place pl, RefNr nr, Cardinal flag)
{
	if (pl->RefEntries)
		pl->ref = MYrealloc(pl->ref, (pl->RefEntries+1) * sizeof(Ref));
	else
		pl->ref = MYmalloc(sizeof(Ref));

	pl->ref[pl->RefEntries].nr   = nr;
	pl->ref[pl->RefEntries].flag = flag;
	pl->RefEntries++;
}

void add_trans_ref(M_Transition tr, RefNr nr, Cardinal flag)
{
	if (tr->RefEntries)
		tr->ref = MYrealloc(tr->ref, (tr->RefEntries+1) * sizeof(Ref));
	else
		tr->ref = MYmalloc(sizeof(Ref));

	tr->ref[tr->RefEntries].nr   = nr;
	tr->ref[tr->RefEntries].flag = flag;
	tr->RefEntries++;
}

M_Net sync_net(M_Net net, MultiSet scope)
{
	if (flag & NO_SCOPING) return net;
#ifndef NORMAL_SCOPING
	net = M_Scoping(net,scope,max_arc_vars);
#else
	net = M_Synchronisation(net,scope,max_arc_vars);
	net = M_Restriction(net,scope);
#endif
	MSremove(scope);
	return net;
}

void set_dim(M_Net net, int x, int y)
{
	if (x < net->x_pos) net->x_pos = x;
	if (y < net->y_pos) net->y_pos = y;
	if (x > net->x_dim) net->x_dim = x;
	if (y > net->y_dim) net->y_dim = y;
}

void set_correct_dimensions(M_Net net)
{
	/* Fehler von M_Replacement wieder ausgleichen */
	M_Transition tr;
	M_Place pl;

	net->x_dim = net->y_dim = 0;
	net->x_pos = net->y_pos = MAX_INT;
	for (pl = net->Places; pl; pl = pl->nextplace)
		set_dim(net,pl->x_pos,pl->y_pos);
	for (tr = net->Transitions; tr; tr = tr->nexttrans)
		set_dim(net,tr->x_pos,tr->y_pos);
	net->x_dim += RASTER - net->x_pos;
	net->y_dim += RASTER - net->y_pos;
}

void change_net_version(int index, int *lastver, int newver)
{
	if (*lastver == newver) return;

	tmpl[index].file = MYstrdup(tmpl[index].file);
	*(strchr(tmpl[index].file,'.')-1) = newver + '0';

	tmpl[index].net = NULL;
	tmpl[index].cmd = NULL;
	tmpl[index].cmd_entries = 0;
	tmpl[index].netcmd = NULL;
	tmpl[index].netcmd_entries = 0;

	*lastver = newver;
}

void handle_scope(T_symbol block)
{
	T_symbol sym;
	T_structure blkinf = block->e.struc;
	M_Net save_net = current_net;
	MultiSet save_actions = current_actions;

	if (!strcmp(block->name,"env")) return;

	current_net = NULL;
	current_actions = MScreate_empty_set();

	for_list (blkinf->entities,sym,T_symbol) switch (sym->type)
	{
	    case TYPE_SIGDEF:
		break;
	    case TYPE_RELAY:
		handle_relay(sym);
		break;
	    case TYPE_STRUCTURE:
		if (sym->e.struc->subtype == TYPE_PROCESS)
			handle_process(sym);
		else
			handle_scope(sym);
		break;
	    case TYPE_CONNECT:
		handle_connect(sym,block);
		break;
	    case TYPE_ENVPORT:
		Msg(INF,"detected port to environment with "
			"action symbol %s", sym->name);
		break;
	    case TYPE_PROCEDURE:
		if (!(sdlflag & SDLF_NO_PRCD_COPIES)) break;
		macro_new_ids = dot_set;
		create_procedure_net(sym,&current_net,&current_actions);
		break;
	    case TYPE_CONST:
	    case TYPE_SYNTYPE:
		break;
	    default:
		Msg(FAT,"can't happen: %s:%u (%u)",
			__FILE__,__LINE__,sym->type);
		break;
	}

	if (!current_net)
	{
		current_net = save_net;
		current_actions = save_actions;
		return;
	}

	current_net = sync_net(current_net,current_actions);
	current_net = save_net? M_ConcurrentComposition(save_net,current_net)
			      : current_net;
	current_actions = save_actions;
	
	for_list (blkinf->entities,sym,T_symbol)
		if (sym->type == TYPE_CONNECT)
		{
			char *tmpstr = MYmalloc(strlen(sym->name)+3);
			sprintf(tmpstr,"%si!",sym->name);
			add_action(&current_actions,tmpstr,3);
			free(tmpstr);
		}
}
		
M_Net sys2net(T_symbol sys)
{
	M_Net net;

	global_init = NULL;
	global_create = NULL;
	global_term = NULL;
	process_actions = MScreate_empty_set();
	global_actions = MScreate_empty_set();
	global_ref = sys->ref->nr;

	handle_scope(sys);
	sig_delete_net();

	if (global_create)
		global_init = M_Sequence(global_init,global_create);

	if (process_actions->anzahl)
	{
		macro_name = "dummy";
		macro_ref = global_ref;

		net = GetBox(SDL_TERM,True);
		/* hier wird angenommen, dass das Term-Netz nur eine
		   Transition hat! */
		MSremove(net->Transitions->label);
		free(net->Transitions->meaning);
		net->Transitions->meaning = MYstrdup("process termination");
		/* R.I.P. Mother of All Core Dumps 12/03/98 */
		net->Transitions->label = MScreate_copy(process_actions);
		global_init = M_Sequence(global_init,net);
	}

	if (global_term)
		global_init = M_Sequence(global_init,global_term);
	
	if (!current_net) return NULL;

	current_net = M_ConcurrentComposition(global_init,current_net);

	/* free stuff (symtabs,waehrend der netzerzeugung alle structs,
		die nicht mehr gebraucht werden)			*/
	current_net = sync_net(current_net,process_actions);
	current_net = sync_net(current_net,global_actions);

	return current_net;
}
