
#include "function.h"
#include "ref.h"
#include "actions.h"
#include "sortrange.h"
#include "list.h"
#include "mnet.h"

T_list reflist;
T_list num_refs;

char ref_type_char[] = REF_TYPE_CHARS;

void init_references()
{
	reflist = alloc_list();
}

void make_ref(T_ref ref, int line, int col, int length)
{
	ref->start_l = ref->end_l = line;
	ref->start_c = col;
	ref->end_c = col + length - 1;
}

T_ref add_ref(void *ptr, char type, T_ref start, T_ref end)
{
	T_ref ref = alloc_ref();

	list_add(reflist,ref);
	ref->type = type;
	ref->nr = reflist->count;
	ref->start_l = start->start_l;
	ref->start_c = start->start_c;
	ref->end_l = end->end_l;
	ref->end_c = end->end_c;
	ref->ptr = ptr;
	ref->proc = 0;
	return ref;
}

void hide_refs(M_Net net)
{
	M_Place pl;
	M_Transition tr;
	
	num_refs = alloc_list();
	for (pl = net->Places; pl; pl = pl->nextplace)
		list_add(num_refs,(int) pl->RefEntries);
	for (tr = net->Transitions; tr; tr = tr->nexttrans)
		list_add(num_refs,(int) tr->RefEntries);
	for (tr = net->PTransitions; tr; tr = tr->nexttrans)
		list_add(num_refs,(int) tr->RefEntries);

	for (pl = net->Places; pl; pl = pl->nextplace) pl->RefEntries = 0;
	for (tr = net->Transitions; tr; tr = tr->nexttrans) tr->RefEntries = 0;
	for (tr = net->PTransitions; tr; tr = tr->nexttrans) tr->RefEntries = 0;
}

void restore_refs(M_Net net)
{
	M_Place pl;
	M_Transition tr;
	int i = 0;
	
	for (pl = net->Places; pl; pl = pl->nextplace)
		pl->RefEntries = list_elem(num_refs,i++,int);
	for (tr = net->Transitions; tr; tr = tr->nexttrans)
		tr->RefEntries = list_elem(num_refs,i++,int);
	for (tr = net->PTransitions; tr; tr = tr->nexttrans)
		tr->RefEntries = list_elem(num_refs,i++,int);
	free_list(num_refs);
}

void write_ref_line(FILE *f, char *prefix, int numname, int entries, Ref *ref)
{
	int i;

	if (!entries) return;
	fprintf(f,"%s%u ",prefix,numname);
	for (i = 0; i < entries; i++)
	{
		fprintf(f,"%s%u:%u", i? "*" : "T\"", ref[i].nr, ref[i].flag);
        }
	fprintf(f,"\"\n");
}

void write_ref_file(char *filename, M_Net net, char *sdlsource)
{
	M_Place pl;
	M_Transition tr;
	T_symbol sig;
	T_ref ref;
	T_list siglist, linestart = alloc_list();

	FILE *file, *src;
	int i;
	char c, direction = 0;

	if (!(file = fopen(filename,"w")))
		Msg(FAT,"could not open file %s for writing\n",filename);

	printf("Writing references to %s\n",filename);

	if (!(src = fopen(sdlsource,"r")))
		Msg(FAT,"could not re-open %s?!?",sdlsource);

	list_add(linestart,ftell(src));
	while ((c = fgetc(src)) != EOF)
		if (c == '\n') list_add(linestart,ftell(src));
	
	fprintf(file,"PEP\n");
	fprintf(file,"References_SDL_HL\n");
	fprintf(file,"FORMAT_N\n");

	fprintf(file,"DEF\n");
	for_list (reflist,ref,T_ref)
	{
		T_symbol sym = (T_symbol) ref->ptr;

		fprintf(file,"%d ",ref->nr);
		fprintf(file,"R\"(%u,%u;%u,%u)\"%c",
			ref->start_l,ref->start_c,ref->end_l,ref->end_c,
			ref_type_char[(int) ref->type]);

		switch (ref->type)
		{
		    case REF_CHANNEL:
		    case REF_SIGNALROUTE:
			fprintf(file,"\"");
			if (sym->e.relay->bidirectional)
				fprintf(file,(direction=!direction)? ">" : "<");
			siglist = sym->e.relay->siglist1;
			if (sym->e.relay->bidirectional && !direction)
				siglist = sym->e.relay->siglist2;
			i = 0;
			for_list (siglist,sig,T_symbol)
				fprintf(file,"%c%s",i++? ',' : '{', sig->name);
			fprintf(file,"}\"");
			if (ref->type != REF_CHANNEL) break;
			fprintf(file,"v\"%u\"c\"%u\"",
				sym->e.relay->version,sym->e.relay->capacity);
			break;
		    case REF_VARIABLE:
		    case REF_IN_PARM:
		    case REF_IN_OUT_PARM:
			fprintf(file,"\"");
			if (sym->e.syntype->set == SORT_PID)
				fprintf(file,"PId");
			else
				PrintSingleRange(sym->e.syntype->set,file);
			fprintf(file,"\"");
			break;
		    case REF_PROCESS:
		    case REF_PROCEDURE:
			fprintf(file,"\"");
			PrintSingleRange(sym->e.struc->prcsinf->idset,file);
			fprintf(file,"\"");
			if (ref->type == REF_PROCEDURE) break;
			fprintf(file,"v\"%u\"c\"%u\"",
				sym->e.struc->prcsinf->version,
				sym->e.struc->prcsinf->capacity);
			break;
		    case REF_CONNECT:
			fprintf(file,"\"%c\"",(direction=!direction)? '>':'<');
			break;
		    case REF_SYSTEM:
		    case REF_BLOCK:
		    case REF_SIGNAL:
		    case REF_ACTION:
		    case REF_ANSWER:
		    case REF_TRIGGER:
		    case REF_LABEL:
			break;
		    default:
			Msg(WAR,"can't happen: %s:%u (%u)",
				__FILE__,__LINE__,ref->type);
			break;
		}

		if (ref->proc)
			fprintf(file,"u\"%u\"",ref->proc);

		if (ref->type == REF_ACTION || ref->type == REF_ANSWER
			|| ref->type == REF_TRIGGER)
		{
			fprintf(file,"b\"");
			fseek(src,list_elem(linestart,ref->start_l-1,long)
				  + ref->start_c - 1, SEEK_SET);

			if (ref->start_l == ref->end_l)
			{
				for (i = ref->start_c; i <= ref->end_c; i++)
					fputc(fgetc(src),file);
			}
			else
			{
				for (i = ref->start_l; i <= ref->end_l-1; i++)
					while ((c = fgetc(src) != '\n'))
						fputc(c,file);

				for (i = 1; i <= ref->end_c; i++)
					fputc(fgetc(src),file);
			}
			
			fprintf(file,"\"\n");
		}
		else
			fprintf(file,"b\"%s\"\n",sym->name);
	}

	fprintf(file,"PL\n");
	for (pl = net->Places; pl; pl = pl->nextplace)
		write_ref_line(file,"P",pl->numname,pl->RefEntries,pl->ref);

	fprintf(file,"TR\n");
	for (tr = net->Transitions; tr; tr = tr->nexttrans)
		write_ref_line(file,"T",tr->numname,tr->RefEntries,tr->ref);

	if (flag & PHANTOM_TRANS_OUTPUT)
	{
		fprintf(file,"PTR\n");
		for (tr = net->PTransitions; tr; tr = tr->nexttrans)
		  write_ref_line(file,"PT",tr->numname,tr->RefEntries,tr->ref);
	}

	fclose(file);
	fclose(src);
}

