/******************************************************************************/
/* FILE scanner.l                                                             */
/*                                                                            */
/* FLEX source file: scanner for b(pn)^2                                      */
/*                                                                            */
/* winter 1996/97 by Martin Ackermann                                         */
/******************************************************************************/

%option noyywrap

%{
#include <string.h>
#ifdef _AIX
#include <strings.h>
#endif
#include <unistd.h>
#include <ctype.h>

#include "symtab.h"
#include "actions.h"
#include "sdl.h"

#ifndef NO_FLEX_HACKS
#define YY_NO_UNPUT
void* (*yydummy)(void*,yy_size_t) = yy_flex_realloc;
#endif

int yylineno = 1;
int yycoluno = 1;

char *reserved[]
	= {	"system", "endsystem",
		"block", "endblock",
		"referenced",
		"channel", "endchannel", "from", "to", "with",
		"nodelay",
		"signal",
		"connect", "and",
		"process", "endprocess", 
		"start", "state", "input",
		"output",
		"nextstate", "stop",
		"via",
		"synonym", "syntype", "endsyntype",
		"default", "constants",
		"dcl",
		"task",
		"or", "xor", "and", "mod", "not", "rem",
		"decision", "any", "else", "enddecision",
		"procedure", "endprocedure", "call", "return",
		"in", "out",
		"create", "none",
		"join"
	  };

int  _reserved[]
	= {	SYSTEM, ENDSYSTEM,
		BLOCK, ENDBLOCK,
		REFERENCED,
		CHANNEL, ENDCHANNEL, FROM, TO, WITH,
		NODELAY,
		SIGNAL,
		CONNECT, AND,
		PROCESS, ENDPROCESS,
		START, STATE, INPUT,
		OUTPUT,
		NEXTSTATE, STOP,
		VIA,
		SYNONYM, SYNTYPE, ENDSYNTYPE,
		DEFAULT, CONSTANTS,
		DCL,
		TASK,
		OR, XOR, AND, MOD, NOT, REM,
		DECISION, ANY, ELSE, ENDDECISION,
		PROCEDURE, ENDPROCEDURE, CALL, RETURN,
		IN, OUT,
		CREATE, INPUT_NONE,
		JOIN
	  };

#define copy_ref( ref )               \
{                                     \
  ref.start_l = yylineno;             \
  ref.start_c = yycoluno;             \
  ref.end_l   = yylineno;             \
  ref.end_c   = yycoluno + yyleng;    \
}

/* return pointer to ident struct, create if identifier doesn't exist */
static int ident()
{
	make_ref(&yylval.ident.ref,yylineno,yycoluno,yyleng);
	s_lookup(yytext,&(yylval.ident)); 
	if (yylval.ident.sym->type != TYPE_CONST) return IDENT;

	yylval.value = yylval.ident.sym->e.num;
	return CONST;
}

%}

delim		[ \t]
ws		({delim}|\n)*
digit		[0-9]
letter		[A-Za-z_]

ident		{letter}({letter}|{digit})*
number		[1-9]{digit}*|0

%x note

%%
		{ yycoluno += yyleng; /* add length of previous token */ }

<*>\n		{ ++yylineno; yycoluno=1; }

<*>{delim}+	{ yycoluno += yyleng; /* white space */ }

"/*"		BEGIN(note);

<note>[^*\n]*		/* eat anything that's not a '*' */
<note>"*"+[^*/\n]*	/* eat up '*'s not followed by '/'s */
<note>"*"+"/"		BEGIN(INITIAL);

{number}	{ int py; 
		  sscanf(yytext,"%d",&py); 
		  if (py > MAX_POS) Msg(FAT,"Constant out of range!");
		  yylval.value = py;
		  return CONST;
		}

Null    {return NULL_PID;}
False   {return FALSE;}
True    {return TRUE;}
"comment '"[^;\n]*"'" {return COMMENT;}

_?[a-z]+_?	|
_?[A-Za-z]+_?	{ int i;	/* todo: das hier effizienter machen */
		  for (i = 0; i < sizeof(reserved)/sizeof(*reserved); ++i)
		  /* TODO: Check only all uppercase and/or all lowercase */
			if (!strcasecmp(yytext,reserved[i]))
			{
			  int j=0;
			  while(yytext[j]&&islower(yytext[0])?islower(yytext[j]):isupper(yytext[j]))++j;
			    if(!yytext[j])
			      {
				make_ref(&yylval.ref,yylineno,yycoluno,yyleng);
				return _reserved[i];
			      }
			}
		  return ident();
		}

{ident}		{ return ident(); }

"<="		{ return LE; }
">="		{ return GE; }
"/="		{ return NE; }
"=>"		{ return IMPLIES; }
":="		{ return ASSIGN; }

[(;,:)]		{ make_ref(&yylval.ref,yylineno,yycoluno,1);
		  return yytext[0];
		}

[-=#<>+*/{}.!?'] { return yytext[0]; }

%%
