/******************************************************************************/
/* iter.c                                                                     */
/* Burkhard Graves, 15.07.94                                                  */
/* Martin Ackermann, Fruehjahr 95                                             */
/* (modifiziert fuer M-Netze, enter-Netz eingefuegt)                          */
/* Burkhard Bieber:                                                           */
/*   - Schoenheitskorrekturen                                                 */
/* Martin Ackermann, Winter 1996/97                                           */
/*   - Prozeduren, andere Referenzen, mehr Effizienz, Fehler beseitigt        */
/******************************************************************************/

  /************/
  /* includes */
  /************/

#include "mnet.h"
#include "move.h"
#include "compose.h"
#include "box_sub.h"
#include "multiset.h"

extern void M_PlaceMultiplication(M_Net,M_Net,PLabel,PLabel);

M_Net SDL_Iteration(alt_set_type alt, M_Net enter)
{
	M_Place   pl1,pl2,pl3;
	M_Place   pl,pl_entry,pl_exit;
	M_Net     net_a,net_b,net_c;

	M_STEdge  stlast,pstlast;
	M_TSEdge  tslast,ptslast;

	if (!alt.repeat_net)
		return enter? M_Sequence(enter,alt.exit_net) : alt.exit_net;

	if (!enter)
	{
		/* falls kein enter-Netz uebergeben wird, ein leeres erzeugen */
		VariableStruct vars;
		enter = M_CreateBasicBox(MScreate_empty_set(),NULL,NULL,
			Create_empty_Variables(&vars,1),0);
	}

	net_a = enter;
	net_b = alt.repeat_net;
	net_c = alt.exit_net;

	M_PreparePlaces(net_a,EXIT,EXIT);
	M_PreparePlaces(net_c,ENTRY,ENTRY);

	pl2=net_a->LastPlace;
	pl3=net_c->LastPlace;
	for (pl1 = net_b->Places; pl1; pl1 = pl1->nextplace)
	{
		if (pl1->label==ENTRY) pl_entry = pl1;
		if (pl1->label==EXIT) pl_exit = pl1;
	}

	pl = M_DuplicatePlace(net_b,pl_entry,&stlast,&tslast,&pstlast,&ptslast);
	pl->numname = 1;
	pl->label = EMPTY;
	M_SumPlaceRefs(pl,pl2);
	M_SumPlaceRefs(pl,pl3);
	M_SumPlaceRefs(pl,pl_exit);

	pl->preset = M_CopyPlacePreset(pl2,pl,&tslast,False);
	tslast->nexttrans = M_CopyPlacePreset(pl_exit,pl,&tsdummy,False);
	tslast->nexttrans->prevtrans = tslast;

	pl->ppreset = M_CopyPlacePreset(pl2,pl,&ptslast,True);
	if (ptslast)
	{
		ptslast->nexttrans = M_CopyPlacePreset(pl_exit,pl,&tslast,True);
		ptslast->nexttrans->prevtrans = ptslast;
	}
	else
		pl->ppreset = M_CopyPlacePreset(pl_exit,pl,&tslast,True);

	stlast->nextplace = M_CopyPlacePostset(pl3,pl,&stdummy,False);
	stlast->nextplace->prevplace = stlast;

	pl->ppostset = M_CopyPlacePostset(pl3,pl,&pstlast,True);
	if (pstlast)
	{
		pstlast->nextplace = M_CopyPlacePostset(pl3,pl,&stlast,True);
		pstlast->nextplace->prevplace = pstlast;
	}
	else
		pl->ppostset = M_CopyPlacePostset(pl3,pl,&stlast,True);

	for (tslast = pl->preset; tslast; tslast = tslast->nexttrans)
		tslast->flags |= EDGE_INV;
	for (tslast = pl->ppreset; tslast; tslast = tslast->nexttrans)
		tslast->flags |= EDGE_INV;

	M_RemovePlace(net_b,pl_entry);
	M_RemovePlace(net_b,pl_exit);

	M_move_hv(net_c,-RASTER,net_a->y_pos-net_c->y_pos+net_a->y_dim);

	M_move_hv(net_b,net_a->x_dim+2*RASTER,0);
	net_b->x_dim += net_a->x_dim+2*RASTER;
	net_b->x_pos = net_a->x_pos;
	net_a = M_RestoreKillPlaces(net_a,EXIT);
	net_c = M_RestoreKillPlaces(net_c,ENTRY);
	return M_DisjointNetUnion(net_b,M_DisjointNetUnion(net_a,net_c));

}

M_Net SDL_Sequence(M_Net net1, M_Net net2)
{
	int netsize = net1->x_dim;

	M_move_hv(net2,net1->x_pos-net2->x_pos,
		       net1->y_pos-net2->y_pos+net1->y_dim-RASTER);
	net1->x_dim = RASTER;
	M_PlaceMultiplication(net1,net2,ENTRY,EXIT);
   
	if (net2->x_dim > (net1->x_dim = netsize))
	{
		net1->x_dim = net2->x_dim;
		net1->x_pos = net2->x_pos;
	}
	net1->y_dim += net2->y_dim-RASTER;

	return M_DisjointNetUnion(net1,net2);
}

