
#include "function.h"
#include "signet.h"

T_list net_points;
T_list net_routes;

intset all_pids;

typedef struct {
	T_symbol sym;
	T_list outgoing;
	T_list incoming;
	intset tosend,sent;
	intset torecv,recvd;
} *T_point, point_instance;

typedef struct {
	T_symbol sym;
	T_point ep1,ep2;
	intset signals;
	intset senders;
	intset receivers;
} *T_route, route_instance;

#define alloc_point() (MYmalloc(sizeof(point_instance)))
#define alloc_route() (MYmalloc(sizeof(route_instance)))

void sig_init_net()
{
	net_points = alloc_list();
	net_routes = alloc_list();
	all_pids = alloc_intset();
	intset_add(&all_pids,DOT);
}

T_point new_point()
{
	T_point p = alloc_point();
	p->outgoing = alloc_list();
	p->incoming = alloc_list();
	p->tosend = alloc_intset();
	p->sent = alloc_intset();
	p->torecv = alloc_intset();
	p->recvd = alloc_intset();
	return p;
}

T_route new_route()
{
	T_route r = alloc_route();
	r->signals = alloc_intset();
	r->senders = alloc_intset();
	r->receivers = alloc_intset();
	return r;
}

T_point lookup_point(T_symbol sym, char dir)
{
	T_point p=NULL;

	for_list (net_points,p,T_point)
		if (p->sym == sym && !dir--) return p;

	while(dir-- + 1)
	{
		list_add(net_points,p = new_point());
		p->sym = sym;
	}

	return p;
}
	
void sig_add_route(T_symbol relay, T_symbol conn1, T_symbol conn2, T_list sigs)
{
	T_symbol sig;
	T_point p1,p2;
	T_route route;

	list_add(net_routes,route = new_route());
	for_list (sigs,sig,T_symbol)
		intset_add_interval(&route->signals,sig->e.signal->number,
						    sig->e.signal->highest);

	p1 = lookup_point(conn1,conn1->type == TYPE_CONNECT
				&& conn1->blknum - relay->blknum);
	p2 = lookup_point(conn2,conn2->type == TYPE_CONNECT
				&& !(conn2->blknum - relay->blknum));
	
	route->sym = relay;
	route->ep1 = p1;
	route->ep2 = p2;
	
	list_add(p1->outgoing,route);
	list_add(p2->incoming,route);

}

void sig_add_pids(T_symbol sym,intset pids)
{
	T_point p = lookup_point(sym,0);
	intset_union(&p->tosend,pids);
	intset_union(&all_pids,pids);
}

void send_pids(T_point p,intset pids)
{
	T_route r;
	T_point to;

	for_list (p->outgoing,r,T_route)
	{
		to = r->ep2;
		intset_union(&r->senders,pids);
		if (to->sym->type == TYPE_CONNECT)
		{
			intset_union(&to->tosend,pids);
			intset_subtract(&to->tosend,to->sent);
		}
		else
			intset_union(&to->recvd,pids);
	}
}

void recv_pids(T_point p,intset pids)
{
	T_route r;
	T_point from;

	for_list (p->incoming,r,T_route)
	{
		from = r->ep1;
		intset_union(&r->receivers,pids);
		intset_union(&from->torecv,pids);
		if (from->sym->type == TYPE_CONNECT)
			intset_subtract(&from->torecv,from->recvd);
	}
}

void sig_simnet()
{
	bool alldone;
	T_point p;
	T_route r;

	for_list (net_points,p,T_point)
		if (p->sym->type != TYPE_CONNECT)
		{
			send_pids(p,p->tosend);
			recv_pids(p,p->tosend);
		}

	alldone = False;
	while (!alldone)
	{
		alldone = True;
		for_list (net_points,p,T_point)
		    if (p->sym->type == TYPE_CONNECT)
		    {
			if (intset_isempty(p->tosend))
			{
				alldone = False;
				send_pids(p,p->tosend);
				intset_union(&p->sent,p->tosend);
				free(p->tosend);
				p->tosend = alloc_intset();
			}
			if (intset_isempty(p->torecv))
			{
				alldone = False;
				recv_pids(p,p->torecv);
				intset_union(&p->recvd,p->torecv);
				free(p->torecv);
				p->torecv = alloc_intset();
			}
		    }
	}

	for_list (net_points,p,T_point)
		if (p->sym->type == TYPE_STRUCTURE)
			for_list (p->incoming,r,T_route)
				intset_union(&p->sent,r->signals);

/*	for_list (net_points,p,T_point)
		if (p->sym->type == TYPE_STRUCTURE)
		{
			printf("struc %s:\n  pids: ",p->sym->name);
			intset_print(p->recvd);
			printf("  sigs: ");
			intset_print(p->sent);
		}

	for_list (net_routes,r,T_route)
	{
		printf("relay %s:\n  senders: ",r->sym->name);
		intset_print(r->senders);
		printf("  receivers: ");
		intset_print(r->receivers);
	}
*/
}

/* fixme: diese lineare Suche ist nicht besonders effizient */

void sig_get_relaysets(T_symbol sym, intset *signals, intset *senders,
				     intset *receivers, char count)
{
	T_route r;

	for_list (net_routes,r,T_route)
		if (r->sym == sym && !count--)
		{
			if (signals) *signals = r->signals;
			if (senders) *senders = r->senders;
			if (receivers) *receivers = r->receivers;
			return;
		}

	Msg(FAT,"can't happen: %s:%u (%s)",__FILE__,__LINE__,sym->name);

}

void sig_get_pointsets(T_symbol sym, intset *pids, intset *sigs,
				     intset *senders)
{
	T_point p;

	for_list (net_points,p,T_point)
		if (p->sym == sym)
		{
			if (pids) *pids = p->tosend;
			if (sigs) *sigs = p->sent;
			if (senders) *senders = p->recvd;
			return;
		}

	Msg(FAT,"can't happen: %s:%u (%s)",__FILE__,__LINE__,sym->name);

}

void pursue_path(T_list pathlist, T_list currlist, T_route route, int signal)
{
	T_list newlist;
	T_point ep = route->ep2;
	T_route r;

	list_add(currlist,route);
	if (ep->sym->type != TYPE_CONNECT)
	{
		list_add(pathlist,newlist = alloc_list());
		for_list (currlist,r,T_route)
			list_add(newlist,r);
	}
	else for_list (ep->outgoing,r,T_route)
		if (intset_isin(r->signals,signal))
			pursue_path(pathlist,currlist,r,signal);
	
	list_remove_tail(currlist);
}

T_list sig_find_paths(T_symbol src, int signal)
{
	T_list pathlist = alloc_list();
	T_list newlist = alloc_list();
	T_point p = lookup_point(src,0);
	T_route r;

	for_list (p->outgoing,r,T_route)
		if (intset_isin(r->signals,signal))
			pursue_path(pathlist,newlist,r,signal);

	free_list(newlist);
	return pathlist;
}

T_symbol sig_get_head(T_list list)
{
	return list_elem(list,0,T_route)->sym;
}

T_symbol sig_get_tail(T_list list)
{
	return list_elem(list,list->count-1,T_route)->ep2->sym;
}

T_list sig_restrict_via(T_list pathlist, T_symbol via)
{
	T_list list, newlist = alloc_list();
	T_route r;
	int count = 0;

	if (via->type == TYPE_UNDECLARED)
	{
		for_list (net_routes,r,T_route)
			if (!strcmp(r->sym->name,via->name) && ++count)
				via = r->sym;

		if (via->type == TYPE_UNDECLARED)
			Msg(FAT,"undefined name %s in via clause",via->name);
		if (count > 1)
			Msg(WAR,"ambiguous name %s in via clause",via->name);
	}

	for_list (pathlist,list,T_list)
		for_list (list,r,T_route)
			if (r->sym == via)
			{
				list_add(newlist,list);
				break;
			}

	return newlist;
}

T_list sig_restrict_goal(T_list pathlist,T_symbol goal)
{
	T_list list, newlist = alloc_list();
	T_route r;

	if (goal->type == TYPE_UNDECLARED)
	{
		for_list (net_routes,r,T_route)
			if (!strcmp(r->sym->name,goal->name))
			{
				goal = r->sym;
				break;
			}
		if (goal->type == TYPE_UNDECLARED)
			Msg(FAT,"undefined name %s in to clause",goal->name);
	}

	for_list (pathlist,list,T_list)
		if (sig_get_tail(list) == goal)
			list_add(newlist,list);

	return newlist;
}

void sig_print_pathlist(T_list pathlist)
{
	T_list list;
	T_route r;

	for_list (pathlist,list,T_list)
	{
		for_list (list,r,T_route)
			printf("%s ",r->sym->name);
		printf(sig_get_tail(list)->name);
		printf("\n");
	}
}

T_list sig_make_unique(T_list pathlist)
{
	T_list list, newlist = alloc_list();
	T_symbol sym, lastsym = NULL;

	for_list (pathlist,list,T_list)
		if ((sym = sig_get_head(list)) != lastsym)
		{
			list_add(newlist,list);
			lastsym = sym;
		}
	return newlist;
}

intset sig_get_all_pids()
{
	return all_pids;
}

void sig_delete_net()
{
	T_point p;
	T_route r;

	for_list (net_points,p,T_point)
	{
		free_list(p->outgoing);
		free_list(p->incoming);
		free(p->tosend);
		free(p->sent);
		free(p->torecv);	/* fixme: was wird noch gebraucht? */
		free(p->recvd);
		free(p);
	}

	for_list (net_routes,r,T_route)
	{
		free(r->signals);	/* fixme: dito */
		free(r->senders);
		free(r->receivers);
		free(r);
	}

	free_list(net_points);
	free_list(net_routes);

}
