#include "sortrange.h"
#include "main.h"

intset intsset=NULL,natsset=NULL,boolset;

void check_sort(T_syntype sort, Integer value)
{
	int r;

	if (!sort) return;

printf("Size: %d\n",sort->range->count);
	for_list (sort->range,r,int)
{
printf("%d : %d (%d)\n",lower_bound(r),upper_bound(r),value);
		if (value >= lower_bound(r) && value <= upper_bound(r)) return;
}

	Msg(ERR,"constant out of declared range");
}

void check_subset(intset super, intset sub)
{
	if (super == sub) return;
	if (super != SORT_PID && sub != SORT_PID
			&& intset_issubset(super,sub)) return;
	Msg(ERR,"argument variable is not a subset of the required type");
}

/* wenn warn gesetzt ist, werden Warnungen ausgegeben, falls
   range1 (auch teilweise) nicht in range2 enthalten ist	*/
T_list intersect_range(T_list range1, T_list range2, bool warn)
{
	T_list newrange = alloc_list();
	int count, r1, r2, l1, u1, l2, u2;

	for_list (range1,r1,int)
	{
		l1 = lower_bound(r1);
		u1 = upper_bound(r1);
		count = newrange->count;

		if (warn && l1 > u1)
			Msg(WAR,"lower bound greater than upper bound");
		for_list (range2,r2,int)
		{
			l2 = lower_bound(r2);
			u2 = upper_bound(r2);
			if (l1 > l2) l2 = l1;
			if (u1 < u2) u2 = u1;
			
			if (warn && l2 <= u2 && (l1 < l2 || u1 > u2))
				Msg(WAR,"range is not completely contained "
					"in parent range");

			if (l2 <= u2) list_add(newrange,create_range(l2,u2));
		}
		if (warn && newrange->count == count)
			Msg(WAR,"range is completely outside parent range");
	}

	return newrange;
}

intset create_set(T_list range)
{
	int r;
	intset newset = alloc_intset();

	for_list (range,r,int)
		intset_add_interval(&newset,lower_bound(r),upper_bound(r));

	return newset;
}

bool range_isfinite(T_list range)
{
	int r;

	for_list (range,r,int)
	{
		if (lower_bound(r) == MIN_NEG) return False;
		if (upper_bound(r) == MAX_POS) return False;
	}

	return True;
}

void init_default_types()
{
	ident_instance dummy;
	T_syntype syn;
	int i;

	s_lookup("Integer",&dummy);
	if (sdl_maxnum)
	{
	  intsset = alloc_intset();
	  for (i=-sdl_maxnum;i<sdl_maxnum;++i)
	    intset_add(&intsset,i);
	}  
	(syn = alloc_syntype())->set = intsset;
	syn->def_init = NONE;
	syn->range = alloc_list();
	if (sdl_maxnum)
	  list_add(syn->range,create_range(-sdl_maxnum,sdl_maxnum-1));
	else
	  list_add(syn->range,create_range(MIN_NEG,MAX_POS));
	define_ident(&dummy,TYPE_SYNTYPE,syn);

	s_lookup("Natural",&dummy);
	if (sdl_maxnum)
	{
	  natsset = alloc_intset();
	  for (i=0;i<=sdl_maxnum;++i)
	    intset_add(&natsset,i);
	}  
	(syn = alloc_syntype())->set = natsset;
	syn->def_init = NONE;
	syn->range = alloc_list();
	if (sdl_maxnum)
	  list_add(syn->range,create_range(0,sdl_maxnum));
	else
	  list_add(syn->range,create_range(0,MAX_POS));
	define_ident(&dummy,TYPE_SYNTYPE,syn);

	s_lookup("Boolean",&dummy);
	boolset = alloc_intset();
	intset_add(&boolset,TT);
	intset_add(&boolset,FF);
	(syn = alloc_syntype())->set = boolset;
	syn->def_init = NONE;
	syn->range = alloc_list();
	list_add(syn->range,create_range(FF,TT));
	define_ident(&dummy,TYPE_SYNTYPE,syn);

	s_lookup("PId",&dummy);
	(syn = alloc_syntype())->set = SORT_PID;
	syn->def_init = NONE;
	syn->range = alloc_list();
	list_add(syn->range,create_range(DOT,DOT));
	define_ident(&dummy,TYPE_SYNTYPE,syn);
}

void init_implicit_vars()
{
	ident_instance dummy;
	T_syntype syn;
	
	s_lookup("PId",&dummy);
	syn = dummy.sym->e.syntype;
	s_lookup("self",&dummy);
	define_ident(&dummy,TYPE_VARIABLE,syn);
	s_lookup("parent",&dummy);
	define_ident(&dummy,TYPE_VARIABLE,syn);
	s_lookup("offspring",&dummy);
	define_ident(&dummy,TYPE_VARIABLE,syn);
	s_lookup("sender",&dummy);
	define_ident(&dummy,TYPE_VARIABLE,syn);
}
