/******************************************************************************/
/* FILE symtab.c                                                              */
/* Burkhard Graves, 10.10.93                                                  */
/* Lars Jenner, Juni 1994                                                     */
/* Stephan Merkel, Mar. 95                                                    */
/* Martin Ackermann, Winter 96/97                                             */
/******************************************************************************/

#include "symtab.h"
#include "frontend.h"

/******************************************************************************/
/* Datenstrukturen							      */
/******************************************************************************/

int	 blknum;	/* Verschachtelungstiefe */

T_symbol s_start;	/* Start der Symboltabelle */


/**********************************************************************/
/* Initialisierung der Symboltabelle (Aufruf aus 'frontend.c' heraus) */
/**********************************************************************/

void s_init()
{
	s_start = alloc_symbol();
	s_start->next = NULL;
	s_start->name = "-";
	s_start->type = TYPE_TAIL;
	s_start->blknum = blknum = 0;	/* ungueltige Blocknummer! */
}

/****************************************************************************/
/* Absuchen der Symboltabelle nach dem Bezeichner 'ptr'. Konnte er gefunden */
/* werden, so wird ein Zeiger auf ihn zurueckgegeben, ansonsten wird ein    */
/* NULL-Pointer zurueckgegeben. (Aufruf durch 's_lookup')                   */
/****************************************************************************/

T_symbol s_find(char *ptr)
{
	T_symbol entry;
  
	entry = s_start;
	while (entry && (*entry->name != *ptr || strcmp(entry->name,ptr)))
	{
		entry = entry->next;
	}
	return entry;
}

/**************************************************************/
/* Erzeugung eines neuen Eintrages in der Symboltabelle       */
/* (Aufruf durch 's_lookup' und 'make_ident' und GenParList)  */
/**************************************************************/

T_symbol s_create(char *name)
{
	T_symbol new_entry;
  
	new_entry = alloc_symbol();
	new_entry->next = s_start;	/* vorne anhaengen */
	s_start = new_entry;
  
	new_entry->name = MYstrdup(name);
	new_entry->type = TYPE_UNDECLARED;
  
	new_entry->blknum = blknum;
  
	/* set other stuff */  
	new_entry->e.data = NULL;
	new_entry->set = NULL;

	return new_entry;
}

/***************************************************************************/
/* Feststellung, ob ein Bezeichner 'ptr' bereits in der Symboltabelle ist. */
/* Falls nicht, so wird ein entsprechender Eintrag erzeugt.                */
/* Rueckgabewert ist ein Zeiger auf den Eintrag. (Aufruf durch den Scanner)*/
/***************************************************************************/

void s_lookup(char *ptr, T_ident ident)
{
	ident->sym = s_find(ptr);
	if (!ident->sym)
	{
		ident->sym = s_create(ptr);
		ident->is_old = False;
	}
	else
		ident->is_old = True;
}

/***************************************************************************/
/* Erzeugt einen neuen Dummy-Eintrag in der Symbolliste			   */
/***************************************************************************/

static ident_instance dummy;

T_ident s_dummy(char *ptr)
{
	dummy.sym = s_create(ptr);
	dummy.is_old = False;
	return &dummy;
}

/*************************************************************************/
/* Stellt fest, ob bereits ein Symbol dieses Namens auf der derzeitigen  */
/* Ebene existiert. Wenn ja, wird der Typ ueberprueft. Wenn nein, wird	 */
/* ein neues Symbol erzeugt und der Typ festgelegt.			 */
/*************************************************************************/

T_symbol decl_ident(T_ident ident, char type)
{
	T_symbol entry = ident->sym;

	if (ident->is_old)
	{
		if (entry->blknum == blknum)
		{
			if (entry->type != type)
				Msg(ERR,"%s is of wrong type",entry->name);
			return entry;
		}
		entry = s_create(entry->name);
	}
	entry->type = type;

	return entry;
}

/*************************************************************************/
/* Legt ein neues Symbol an.						 */
/* Wenn dieses Symbol auf der derzeitigen Ebene bereits referenziert	 */
/* wurde, wird der Typ ueberprueft und festgestellt, ob das Symbol	 */
/* bereits definiert wurde.						 */
/*************************************************************************/

T_symbol define_ident(T_ident ident, char type, void *e)
{
	T_symbol entry = ident->sym;

	if (ident->is_old)
	{
		if (entry->blknum == blknum)
		{
			if (entry->type != type)
				Msg(ERR,"%s previously declared",entry->name);
			if (entry->e.data)
				Msg(ERR,"%s previously defined",entry->name);
			entry->e.data = e;
			return entry;
		}
		entry = s_create(entry->name);
	}
	entry->type = type;
	entry->e.data = e;

	return entry;
}

/*************************************************************************/
/* Stellt fest, ob ein Symbol bereits deklariert und definiert wurde	 */
/* und fuehrt eine Typ-Pruefung durch.					 */
/*************************************************************************/

T_symbol refer_ident(T_ident ident, char type)
{
	T_symbol entry;

	if (ident && (entry=ident->sym) && (entry->type == type) && 
		entry->e.data)
	  return entry;

	Msg(ERR,"identifier %s undeclared or used incorrectly",entry->name);
	return NULL;
}

T_symbol refer_offscope(T_ident ident, char type)
{
	T_symbol entry = ident->sym;

	if (entry->type != TYPE_UNDECLARED)
          {
	    if (entry->type == type)
	      return entry;
	    else
	      entry = s_create(entry->name);
          }
	entry->e.num = 1;
	return entry;
}

int make_states()
{
	T_symbol sym;
	int num_states = 0;

	for(sym=s_start;sym->blknum == blknum; sym = sym->next)
	{
		if (sym->type == TYPE_STATE)
			sym->e.num = strcmp(sym->name,"-")? ++num_states : -1;
	}

	return num_states;
}

/*********************************************************************/
/* Inkrementierung von 'blknum'.                                     */
/* (Aufruf durch den Parser am Beginn eines Blockes ('block'-Regel)) */
/*********************************************************************/

void inc_blknum()
{
	++blknum;
}

/*******************************************************************/
/* Dekrementierung von 'blknum' und Entfernung von lokalen         */
/* Eintraegen aus der Symboltabelle. (Aber noch nicht loeschen,    */
/* sondern zurueckgeben !)                                         */
/* (Aufruf durch den Parser am Ende eines Blockes ('block'-Regel)) */
/*******************************************************************/

T_symbol dec_blknum()
{
	T_symbol ptr1,ptr2,memo;

	if (GL_Frags && !blknum) inc_blknum();
		/* if we scan for fragments and no identifiers occured,
			we pretend now to have seen a BEGIN */
  
	memo = ptr1 = ptr2 = s_start;

	while (ptr1->blknum == blknum)
	{
		if (!ptr1->e.data)
			Msg(ERR,"%s declared but not defined",ptr1->name);
		/* todo: declared but not used */
		ptr2 = ptr1;
		ptr1 = ptr1->next;
	}

	s_start = ptr1;
	blknum--;
	if (ptr1 == ptr2) memo = NULL; else ptr2->next = NULL;

	return memo;
}

void delete_sdl_tree(T_symbol node)
{
  if ( node )
    {
	switch(node->type) {
	case TYPE_PROCEDURE:
		free_list(node->e.struc->prcsinf->fmlparms);
		MYfree(node->e.struc->prcsinf);
		break;
	case TYPE_STRUCTURE:
	        MYfree(node->name);
	        MYfree(node->ref);
		delete_sdl_tree(node->e.struc->symtab);
		MYfree(node->e.data);
		break;
	case TYPE_SYSTEM:
	case TYPE_BLOCK:
	        MYfree(node->name);
		delete_sdl_tree(node->e.struc->symtab);
		break;
	case TYPE_BODY:
	        MYfree(node->name);
		free_list(node->e.list);
		break;
	case TYPE_SIGDEF:
		MYfree(node->e.data);
	        MYfree(node->name);
		break;
	case TYPE_SIGNAL:
	case TYPE_CONNECT:
	case TYPE_CONST:
	case TYPE_SYNTYPE:
//		MYfree(node->e.data);
	case TYPE_STATE:
	case TYPE_LABEL:
	case TYPE_ENVPORT:
	case TYPE_VARIABLE:
	case TYPE_VARDEF:
	        MYfree(node->name);
		break;
	case TYPE_RELAY:
	        MYfree(node->name);
		MYfree(node->e.relay);
		break;
	case TYPE_UNDECLARED:
	case TYPE_TAIL:
		break;
	default:
		Msg(INF,"Internal Error: Unknown node type in delete_sdl_tree !");
	} /* switch */
//	if((node->ref)&&(node->type!=TYPE_BODY))
//	  MYfree(node->ref);
	delete_sdl_tree(node->next);
	MYfree(node);
    } /* if */
}

