/******************************************************************************/
/* FILE symtab.h                                                              */
/* Burkhard Graves, 10.10.93                                                  */
/* Lars Jenner, Juni 1994                                                     */
/* Stephan Merkel, Mar. 95                                                    */
/* Martin Ackermann, Winter 96/97                                             */
/******************************************************************************/

#ifndef __SYMTAB_H__
#define __SYMTAB_H__

#include <stdio.h>	/* fixme: nur fuer set.h      */
#include "function.h"
#include "set.h"	/* fixme: nur fuer evaltree.h */
#include "list.h"
#include "ref.h"

/******************************************************************************/
/* Globale Defines                                                            */
/******************************************************************************/

#define structdef(x) *T_##x, x##_instance

/* Symboltypen */
enum {	TYPE_UNDECLARED,
	TYPE_TAIL,
	TYPE_STRUCTURE,
	TYPE_RELAY,
	TYPE_SIGNAL,
	TYPE_CONNECT,
	TYPE_STATE,
	TYPE_CONST,
	TYPE_SYNTYPE,
	TYPE_VARIABLE,
	TYPE_PROCEDURE,
	TYPE_LABEL,

	TYPE_SYSTEM,
	TYPE_BLOCK,
	TYPE_PROCESS,

	TYPE_SIGDEF,
	TYPE_VARDEF,
	TYPE_ENVPORT,
	TYPE_BODY
     };

#include "oldtypes.h"	/* alte Defines, aus Kompatibilitaetsgruenden */

/******************************************************************************/
/* Globale Typdefinitionen                                                    */
/******************************************************************************/

#include "refstuff.h"

typedef union {
	int num;
	void *data;
	T_list list;
	struct t_struc_type *struc;
	struct t_relay_type *relay;
	struct t_signal_type *signal;
	struct t_connect_type *connect;
	struct t_syntype_type *syntype;
	struct t_prcsinf_type *prcsinf;
} symbol_data;

typedef struct this_type {
	char		   type;	/* Typ, siehe TYPE_*	 */
	char		  *name;	/* Name			 */
	int		   blknum;	/* Verschachtelungstiefe */
	symbol_data	   e;		/* Zeiger auf Zusatz-Inf */
	SetRange	  *set;		/* fuer Kompatibilitaet	 */
	struct this_type  *next;	/* Verkettung            */
	T_ref		   ref;
} structdef(symbol);

typedef struct {
	T_symbol sym;
	bool    is_old;
	ref_instance ref;
} structdef(ident);

#include "parsedefs.h"

/******************************************************************************/
/* Definitionen aus Kompatibilitaetsgruenden				      */
/******************************************************************************/

typedef symbol_instance SymTabEntry;

#define s_name name
#define s_blknum blknum
#define s_init_or_cap blknum
#define s_set set
#define s_partype type

/******************************************************************************/
/* Globale Variablen                                                          */
/******************************************************************************/

extern T_symbol s_start;


/******************************************************************************/
/* Globale Funktionen                                                         */
/******************************************************************************/

extern void s_init();
extern void s_lookup(char*,T_ident);
extern T_ident s_dummy(char*);
extern T_symbol decl_ident(T_ident,char);
extern T_symbol define_ident(T_ident,char,void*);
extern T_symbol refer_ident(T_ident,char);
extern T_symbol refer_offscope(T_ident,char);
extern int make_states();
extern void assign_type(char);
extern void inc_blknum();
extern T_symbol dec_blknum();

void delete_sdl_tree(T_symbol node);
#define alloc_symbol() (MYmalloc(sizeof(symbol_instance)))
#define alloc_ident() (MYmalloc(sizeof(ident_instance)))

#endif /* __SYMTAB_H__ */

/******************************************************************************/
/* END OF FILE symtab.h                                                       */
/******************************************************************************/
