
#include "Channel.h"
#include "Helper.h"
#include "Converter.h"

//*******************************************************************************************************
olSDL::ClsChannel::ClsChannel(const int Symbol_ID, const int Page_ID, e_DelayType DelayType,
					     ClsChanName ChanName, ClsChanSourceSigList ChanSourceSigList,
					     ClsChanSourceCon ChanSourceCon, ClsChanDestSigList ChanDestSigList,
					     ClsChanDestCon ChanDestCon, const list<int> & ChanCoords,
					     bool bSourceIsEnv, const int ChanSourceModel_ID,
					     bool bDestIsEnv, const int ChanDestModel_ID)
	: ClsObjectWithModel_ID(Symbol_ID), ClsPageObject(Page_ID),
	  m_DelayType(DelayType) {

	m_DirectionType = both ;

	m_pChanName = new ClsChanName(ChanName) ;
	m_pChanSourceSigList = new ClsChanSourceSigList(ChanSourceSigList);
	m_pChanSourceCon = new ClsChanSourceCon(ChanSourceCon) ;
	m_pChanDestSigList = new ClsChanDestSigList(ChanDestSigList) ;
	m_pChanDestCon = new ClsChanDestCon(ChanDestCon) ;

	m_ChanCoords = ChanCoords ;

	m_bSourceIsEnv = bSourceIsEnv ;
	m_bDestIsEnv = bDestIsEnv ;


	if (m_bSourceIsEnv) {
		m_ChanSourceModel_ID = -100 ;
	} else {
		m_ChanSourceModel_ID = ChanSourceModel_ID ;
	} ;


	if (m_bDestIsEnv) {
		m_ChanDestModel_ID = -100 ;
	} else {
		m_ChanDestModel_ID = ChanDestModel_ID ;
	} ;

} ;
//*******************************************************************************************************
olSDL::ClsChannel::ClsChannel(const int Symbol_ID, const int Page_ID, e_DelayType DelayType, e_DirectionType DirectionType,
						ClsChanName ChanName, ClsChanSourceSigList ChanSourceSigList,
						ClsChanSourceCon ChanSourceCon, ClsChanDestSigList ChanDestSigList,
						ClsChanDestCon ChanDestCon, const list<int> & ChanCoords,
						bool bSourceIsEnv, const int ChanSourceModel_ID,
						bool bDestIsEnv, const int ChanDestModel_ID)
	: ClsObjectWithModel_ID(Symbol_ID), ClsPageObject(Page_ID),
	  m_DelayType(DelayType), m_DirectionType(DirectionType) {

	m_pChanName = new ClsChanName(ChanName) ;
	m_pChanSourceSigList = new ClsChanSourceSigList(ChanSourceSigList);
	m_pChanSourceCon = new ClsChanSourceCon(ChanSourceCon) ;
	m_pChanDestSigList = new ClsChanDestSigList(ChanDestSigList) ;
	m_pChanDestCon = new ClsChanDestCon(ChanDestCon) ;

	m_ChanCoords = ChanCoords ;

	m_bSourceIsEnv = bSourceIsEnv ;
	m_bDestIsEnv = bDestIsEnv ;


	if (m_bSourceIsEnv) {
		m_ChanSourceModel_ID = -100 ;
	} else {
		m_ChanSourceModel_ID = ChanSourceModel_ID ;
	} ;


	if (m_bDestIsEnv) {
		m_ChanDestModel_ID = -100 ;
	} else {
		m_ChanDestModel_ID = ChanDestModel_ID ;
	} ;

} ;
//*******************************************************************************************************
olSDL::ClsChannel::~ClsChannel() {

	delete m_pChanName ;
	m_pChanName = NULL ;

	delete m_pChanSourceSigList ;
	m_pChanSourceSigList = NULL ;

	delete m_pChanSourceCon ;
	m_pChanSourceCon = NULL ;

	delete m_pChanDestSigList ;
	m_pChanDestSigList = NULL ;

	delete m_pChanDestCon ;
	m_pChanDestCon = NULL ;

	m_ChanCoords.clear( ) ;
} ;
//*******************************************************************************************************
// modify
//*******************************************************************************************************
void olSDL::ClsChannel::modify_Chan_DirectionType (e_DirectionType DirectionType) {
	m_DirectionType = DirectionType ;
} ;
//*******************************************************************************************************
void olSDL::ClsChannel::modify_Chan_DelayType (e_DelayType DelayType) {
	m_DelayType = DelayType ;
} ;
//*******************************************************************************************************
void olSDL::ClsChannel::modify_Chan_Coords(const list<int> & ChanCoords)  {
	m_ChanCoords = ChanCoords ;
} ;


//*******************************************************************************************************
void olSDL::ClsChannel::modify_Comp_Name(const string content, const ClsRecPaS & PaS) {

	m_pChanName->set_content( content) ;
	m_pChanName->set_PaS(PaS) ;
} ;
//*******************************************************************************************************
void olSDL::ClsChannel::modify_Comp_SourceSigList(const string content, const ClsRecPaS & PaS) {
	m_pChanSourceSigList->set_content( content) ;
	m_pChanSourceSigList->set_PaS(PaS) ;
} ;
//*******************************************************************************************************
void olSDL::ClsChannel::modify_Comp_SourceCon(const string content, const ClsRecPaS & PaS) {
	m_pChanSourceCon->set_content( content) ;
	m_pChanSourceCon->set_PaS(PaS) ;
} ;
//*******************************************************************************************************
void olSDL::ClsChannel::modify_Comp_DestSigList(const string content, const ClsRecPaS & PaS) {
	m_pChanDestSigList->set_content( content) ;
	m_pChanDestSigList->set_PaS(PaS) ;
} ;
//*******************************************************************************************************
void olSDL::ClsChannel::modify_Comp_DestCon(const string content, const ClsRecPaS & PaS) {
	m_pChanDestCon->set_content( content) ;
	m_pChanDestCon->set_PaS(PaS) ;
} ;
//*******************************************************************************************************
// END: modify
//*******************************************************************************************************

//*******************************************************************************************************
olSDL::ClsMoEleType::e_MoEle olSDL::ClsChannel::get_MoEleType() const {
	return olSDL::ClsMoEleType::ChanSym ;
} ;
//*******************************************************************************************************
olSDL::ClsChannel::e_DirectionType olSDL::ClsChannel::get_DirectionType() const {

	return m_DirectionType ;
} ;
//*******************************************************************************************************
olSDL::ClsChannel::e_DelayType olSDL::ClsChannel::get_DelayType() const {

	return m_DelayType ;
} ;
//*******************************************************************************************************
string olSDL::ClsChannel::get_DirectionType_as_String() const {

	switch (m_DirectionType) {
		case both:
		{
			return "both" ;
			break ;
		}
		case from_source:
		{
			return "from_source" ;
			break ;
		}
		case from_destination:
		{
			return "from_destination" ;
			break ;
		}
		default:
		{
			return "none" ;
		}
	} ;

} ;
//*******************************************************************************************************
string olSDL::ClsChannel::get_DelayType_as_String() const {

	switch (m_DelayType) {
		case delay:
		{
			return "delay" ;
			break ;
		}
		case no_delay:
		{
			return "no_delay" ;
			break ;
		}
		default:
		{
			return "none" ;
		}
	} ;

} ;
//*******************************************************************************************************
bool olSDL::ClsChannel::Source_is_Env() const {
	return m_bSourceIsEnv ;
} ;
//*******************************************************************************************************
bool olSDL::ClsChannel::Destination_is_Env() const {
	return m_bDestIsEnv ;
} ;
//*******************************************************************************************************
int olSDL::ClsChannel::get_SourceID_as_Int() const {

	if (m_bSourceIsEnv) {
		return -1 ;
	} else {
		return m_ChanSourceModel_ID ;
	}
} ;
//*******************************************************************************************************
int olSDL::ClsChannel::get_DestID_as_Int() const {

	if (m_bDestIsEnv) {
		return -1 ;
	} else {
		return m_ChanDestModel_ID ;
	} ;
} ;
//*******************************************************************************************************
string olSDL::ClsChannel::get_SourceID_as_String() const {

	string str_SourceID ;

	if (m_bSourceIsEnv) {
		str_SourceID = "ENV" ;
	} else {
		str_SourceID = olSDL::Converter::Int_to_String(m_ChanSourceModel_ID) ;
	} ;

	return str_SourceID ;
} ;
//*******************************************************************************************************
string olSDL::ClsChannel::get_DestID_as_String() const {

	string str_DestID ;

	if (m_bDestIsEnv) {
		str_DestID = "ENV" ;
	} else {
		str_DestID = olSDL::Converter::Int_to_String(m_ChanDestModel_ID) ;
	} ;

	return str_DestID ;
} ;
//*******************************************************************************************************
string olSDL::ClsChannel::get_NameContent( ) const {
	return (m_pChanName->get_content( )) ;
} ;
//*******************************************************************************************************
string olSDL::ClsChannel::get_SourceSigListContent( ) const {
	return (m_pChanSourceSigList->get_content( )) ;
} ;
//*******************************************************************************************************
string olSDL::ClsChannel::get_SourceConContent( ) const {
	return (m_pChanSourceCon->get_content( )) ;
} ;
//*******************************************************************************************************
string olSDL::ClsChannel::get_DestSigListContent( ) const {
	return (m_pChanDestSigList->get_content( )) ;
} ;
//*******************************************************************************************************
string olSDL::ClsChannel::get_DestConContent( ) const {
	return (m_pChanDestCon->get_content( )) ;
} ;
//*******************************************************************************************************
//*******************************************************************************************************
const olSDL::ClsChanName* olSDL::ClsChannel::get_ChanName() const {
	return m_pChanName;
} ;
//*******************************************************************************************************
const olSDL::ClsChanSourceSigList* olSDL::ClsChannel::get_ChanSourceSigList() const {
	return m_pChanSourceSigList ;
} ;
//*******************************************************************************************************
const olSDL::ClsChanSourceCon* olSDL::ClsChannel::get_ChanSourceCon() const {
	return m_pChanSourceCon ;
} ;
//*******************************************************************************************************
const olSDL::ClsChanDestSigList* olSDL::ClsChannel::get_ChanDestSigList() const {
	return m_pChanDestSigList;
} ;
//*******************************************************************************************************
const olSDL::ClsChanDestCon* olSDL::ClsChannel::get_ChanDestCon() const {
	return m_pChanDestCon;
} ;
//*******************************************************************************************************
//*******************************************************************************************************
list<int> olSDL::ClsChannel::get_NamePas( ) const {
	return (m_pChanName->get_PaS_as_IntList()) ;
} ;
//*******************************************************************************************************
list<int> olSDL::ClsChannel::get_SourceSigListPas( ) const {
	return (m_pChanSourceSigList->get_PaS_as_IntList()) ;
} ;
//*******************************************************************************************************
list<int> olSDL::ClsChannel::get_SourceConPas( ) const {
	return (m_pChanSourceCon->get_PaS_as_IntList()) ;
} ;
//*******************************************************************************************************
list<int> olSDL::ClsChannel::get_DestSigListPas( ) const {
	return (m_pChanDestSigList->get_PaS_as_IntList()) ;
} ;
//*******************************************************************************************************
list<int> olSDL::ClsChannel::get_DestConPas( ) const {
	return (m_pChanDestCon->get_PaS_as_IntList()) ;
} ;
//*******************************************************************************************************
void olSDL::ClsChannel::get_Coords_as_IntList(list<int> & IntList) const {

	IntList = m_ChanCoords ;
} ;
//*******************************************************************************************************
//*******************************************************************************************************
void olSDL::ClsChannel::print() const {

	DebugPrinter::print("PRINTING CHANNEL") ;

	DebugPrinter::print("Source: ",false) ;
	DebugPrinter::print_iNumber(m_ChanSourceModel_ID) ;
	DebugPrinter::print("Dest: ",false) ;
	DebugPrinter::print_iNumber(m_ChanDestModel_ID) ;

	DebugPrinter::print("Coords: ",false) ;
	DebugPrinter::print_iList(m_ChanCoords) ;

	/*
	DebugPrinter::print("ChanNameContent: ",false) ;
	DebugPrinter::print(m_pChanName->get_content( )) ;
	DebugPrinter::print("ChanSourceSigListContent: ",false) ;
	DebugPrinter::print(m_pChanSourceSigList->get_content( )) ;
	DebugPrinter::print("ChanSourceConContent: ",false) ;
	DebugPrinter::print(m_pChanSourceCon->get_content( )) ;
	DebugPrinter::print("ChanDestSigListContent: ",false) ;
	DebugPrinter::print(m_pChanDestSigList->get_content( )) ;
	DebugPrinter::print("ChanDestConContent: ",false) ;
	DebugPrinter::print(m_pChanDestCon->get_content( )) ;
	*/

	DebugPrinter::print("ChanNamePaS: ",false) ;
	DebugPrinter::print(m_pChanName->get_PaS_as_String( )) ;
	DebugPrinter::print("ChanSourceSigListPaS: ",false) ;
	DebugPrinter::print(m_pChanSourceSigList->get_PaS_as_String( )) ;
	DebugPrinter::print("ChanSourceConPaS: ",false) ;
	DebugPrinter::print(m_pChanSourceCon->get_PaS_as_String( )) ;
	DebugPrinter::print("ChanDestConPaS: ",false) ;
	DebugPrinter::print(m_pChanDestCon->get_PaS_as_String( )) ;
	DebugPrinter::print("ChanDestSigListPaS: ",false) ;
	DebugPrinter::print(m_pChanDestSigList->get_PaS_as_String( )) ;

	DebugPrinter::print("END PRINTING CHANNEL") ;

} ;


