
#include "Diagram.h"
#include "Helper.h"
#include "MoAccessInfo.h"
#include "Converter.h"

const string olSDL::Diagram::STR_DIA_TYPE_SYS = "system" ;
const string olSDL::Diagram::STR_DIA_TYPE_BLK = "block" ;
const string olSDL::Diagram::STR_DIA_TYPE_BLK_TYPE = "block type" ;
const string olSDL::Diagram::STR_DIA_TYPE_PRO_DIA = "process" ;
const string olSDL::Diagram::STR_DIA_TYPE_PRO_TYPE = "process type" ;
const string olSDL::Diagram::STR_DIA_TYPE_PROCEDURE = "procedure" ;

//*******************************************************************************************************
olSDL::Diagram::Diagram(const int Diagram_ID, const EnumDia::Type DiagramType, const string & HeaderPart_Name, const int First_Page_ID,
	const Symbol * const pReferencingSymbol)
	: ClsObjectWithModel_ID(Diagram_ID), m_PageManager(First_Page_ID), m_pReferencingSymbol(pReferencingSymbol) {

	#ifdef olSDL_DL_ONE
	//DebugPrinter::print("olSDL::Diagram::Diagram(const int Model_ID)") ;
	//DebugPrinter::print("Diagram-ID: ") ;
	//DebugPrinter::print_iNumber(get_ID()) ;
	#endif

	m_DiagramType = DiagramType ;
	m_str_HeaderPart_Name = HeaderPart_Name ;

	switch (DiagramType) {
		case EnumDia::SysDia:
		{
			m_str_HeaderPart_DiaType = STR_DIA_TYPE_SYS ;
			break ;
		}
		case EnumDia::BlkDia:
		{
			m_str_HeaderPart_DiaType = STR_DIA_TYPE_BLK ;
			break ;
		}
		case EnumDia::ProDia:
		{
			m_str_HeaderPart_DiaType = STR_DIA_TYPE_PRO_DIA ;
			break ;
		}
		case EnumDia::BlkTypeDia:
		{
			m_str_HeaderPart_DiaType = STR_DIA_TYPE_BLK_TYPE ;
			break ;
		}
		case EnumDia::ProTypeDia:
		{
			m_str_HeaderPart_DiaType = STR_DIA_TYPE_PRO_TYPE ;
			break ;
		}
		case EnumDia::ProcedureDia:
		{
			m_str_HeaderPart_DiaType = STR_DIA_TYPE_PROCEDURE ;
			break ;
		}
		default:
			{
				m_str_HeaderPart_DiaType = "unknown" ;
			}
	} ; // switch (DiagramType)

} ;
//*******************************************************************************************************
olSDL::Diagram::~Diagram( ) {

	#ifdef olSDL_DL_ONE
	//DebugPrinter::print("olSDL::Diagram::~Diagram( )") ;
	//DebugPrinter::print("Diagram-ID: ") ;
	//DebugPrinter::print_iNumber(get_Model_ID()) ;
	#endif

	// delete member

	type_SymbolContainer::iterator iterSymbolContainer ;
	type_StateMachineSymbolContainer::iterator iterStateMachineSymbol ;
	type_FlowLineContainer::iterator iterFlowLine ;

	// ==================================================

 	for (iterSymbolContainer=m_Symbols.begin(); iterSymbolContainer != m_Symbols.end(); iterSymbolContainer++) {

		switch ((*iterSymbolContainer)->get_SymbolType()) {

			case EnumSym::TextSym:
			{
				delete ((ClsTextSym *) (*iterSymbolContainer)) ;
				break ;
			}
			case EnumSym::ProInstRefSym:
			{
				delete ((ClsProInstRefSym *) (*iterSymbolContainer)) ;
				break ;
			}
			case EnumSym::ProTypeRefSym:
			{
				delete ((ClsProTypeRefSym *) (*iterSymbolContainer)) ;
				break ;
			}
			case EnumSym::BlkInstRefSym:
			{
				delete ((ClsBlkInstRefSym *) (*iterSymbolContainer)) ;
				break ;
			}
			case EnumSym::BlkTypeRefSym:
			{
				delete ((ClsBlkTypeRefSym *) (*iterSymbolContainer)) ;
				break ;
			}
			case EnumSym::ProcedureInstRefSym:
			{
				delete ((ClsProcedureInstRefSym *) (*iterSymbolContainer)) ;
				break ;
			}
			default:
				#ifdef olSDL_DL_TWO
				{
				DebugPrinter::print("error in: olSDL::Diagram::~Diagram()") ;
				DebugPrinter::print("unconsidered Symbol Type !!!") ;
				}
				#endif
				;
		} ; // switch (SymbolType)

	} ; // for iterSymbolContainer

	// ==================================================

 	for (iterStateMachineSymbol=m_StateMachineSymbols.begin(); iterStateMachineSymbol != m_StateMachineSymbols.end(); iterStateMachineSymbol++) {

		switch ((*iterStateMachineSymbol)->get_SymbolType()) {
			case ClsStateMachineSymbol::start:
			{
				delete ((ClsStateMachineStartSymbol*) (*iterStateMachineSymbol)) ;
				break ;
			}
			case ClsStateMachineSymbol::state:
			{
				delete ((ClsStateMachineStateSymbol*) (*iterStateMachineSymbol)) ;
				break ;
			}
			case ClsStateMachineSymbol::output:
			{
				delete ((ClsStateMachineOutputSymbol*) (*iterStateMachineSymbol)) ;
				break ;
			}
			case ClsStateMachineSymbol::input:
			{
				delete ((ClsStateMachineInputSymbol*) (*iterStateMachineSymbol)) ;
				break ;
			}
			case ClsStateMachineSymbol::stop:
			{
				delete ((ClsStateMachineStopSymbol*) (*iterStateMachineSymbol)) ;
				break ;
			}
			case ClsStateMachineSymbol::task:
			{
				delete ((ClsStateMachineTaskSymbol*) (*iterStateMachineSymbol)) ;
				break ;
			}
			case ClsStateMachineSymbol::decision:
			{
				delete ((ClsStateMachineDecisionSymbol*) (*iterStateMachineSymbol)) ;
				break ;
			}
			case ClsStateMachineSymbol::create_pro:
			{
				delete ((ClsStateMachineCreateProSymbol*) (*iterStateMachineSymbol)) ;
				break ;
			}
			case ClsStateMachineSymbol::call_procedure:
			{
				delete ((ClsStateMachineCallProcedureSymbol*) (*iterStateMachineSymbol)) ;
				break ;
			}
			case ClsStateMachineSymbol::procedure_start_state:
			{
				delete ((ClsStateMachineProcedureStartStateSymbol*) (*iterStateMachineSymbol)) ;
				break ;
			}
			case ClsStateMachineSymbol::procedure_return:
			{
				delete ((ClsStateMachineProcedureReturnSymbol*) (*iterStateMachineSymbol)) ;
				break ;
			}
			default:
				#ifdef olSDL_DL_TWO
				{
				DebugPrinter::print("error in: olSDL::Diagram::~Diagram()") ;
				DebugPrinter::print("unconsidered Process Symbol Type !!!") ;
				}
				#endif
				;
		} ; // switch (SymbolType)

	} ; // for (iterStateMachineSymbol=m_StateMachineSymbols.begin(); iterStateMachineSymbol != m_StateMachineSymbols.end(); iterStateMachineSymbol++)

	// ==================================================

	for (iterFlowLine=m_FlowLines.begin(); iterFlowLine != m_FlowLines.end(); iterFlowLine++) {

		switch ((*iterFlowLine)->get_FlowLineType()) {

			case ClsFlowLine::normal:
			{
				delete ( (ClsNormalFlowLine*) (*iterFlowLine) ) ;
				break ;
			}
			case ClsFlowLine::decision:
			{
				delete ( (ClsDecisionFlowLine*) (*iterFlowLine) ) ;
				break ;
			}
			default:
			{
				#ifdef olSDL_DL_TWO
				{
				DebugPrinter::print("error in: olSDL::Diagram::~Diagram()") ;
				DebugPrinter::print("unconsidered Flow Line Type !!!") ;
				}
				#endif
				;
			}
		} ;

	} ; // 	for (iterFlowLine=m_FlowLines.begin(); iterFlowLine != m_FlowLines.end(); iterFlowLine++)

} ;
//*******************************************************************************************************
void olSDL::Diagram::modify_Chan_DirectionType(const int Channel_ID, const ClsChannel::e_DirectionType DirectionType, ClsMoAccessInfo & Info) {

	type_ChannelContainer::iterator iterChannel ;

	iterChannel = find_if(m_Channels.begin(), m_Channels.end(), ObjectHasID(Channel_ID));
	if (iterChannel == m_Channels.end()) {
		Info.set_Info(ClsMoAccessInfo::modify_object_unknown) ;
		return ;
	} ;

	(*iterChannel)->modify_Chan_DirectionType(DirectionType)  ;
	Info.set_Info(ClsMoAccessInfo::modify_OK) ;

	// TEST
	// printChannels() ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::modify_Chan_DelayType(const int Channel_ID, const ClsChannel::e_DelayType DelayType, ClsMoAccessInfo & Info) {

	type_ChannelContainer::iterator iterChannel ;

	iterChannel = find_if(m_Channels.begin(), m_Channels.end(), ObjectHasID(Channel_ID));
	if (iterChannel == m_Channels.end()) {
		Info.set_Info(ClsMoAccessInfo::modify_object_unknown) ;
		return ;
	} ;

	(*iterChannel)->modify_Chan_DelayType(DelayType) ;
	Info.set_Info(ClsMoAccessInfo::modify_OK) ;

	// TEST
	// printChannels() ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::modify_Chan_Coords(const int Channel_ID, list<int> & ChanCoords, ClsMoAccessInfo & Info) {

	type_ChannelContainer::iterator iterChannel ;

	iterChannel = find_if(m_Channels.begin(), m_Channels.end(), ObjectHasID(Channel_ID));
	if (iterChannel == m_Channels.end()) {
		Info.set_Info(ClsMoAccessInfo::modify_object_unknown) ;
		return ;
	} ;

	(*iterChannel)->modify_Chan_Coords(ChanCoords) ;
	Info.set_Info(ClsMoAccessInfo::modify_OK) ;

	// TEST
	// printChannels() ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::modify_Comp_Name(const int Channel_ID, const string content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) {

	type_ChannelContainer::iterator iterChannel ;

	iterChannel = find_if(m_Channels.begin(), m_Channels.end(), ObjectHasID(Channel_ID));
	if (iterChannel == m_Channels.end()) {
		Info.set_Info(ClsMoAccessInfo::modify_object_unknown) ;
		return ;
	} ;

	(*iterChannel)->modify_Comp_Name(content, PaS) ;
	Info.set_Info(ClsMoAccessInfo::modify_OK) ;

	// TEST
	// printChannels() ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::modify_Comp_SourceSigList(const int Channel_ID, const string content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) {

	type_ChannelContainer::iterator iterChannel ;

	iterChannel = find_if(m_Channels.begin(), m_Channels.end(), ObjectHasID(Channel_ID));
	if (iterChannel == m_Channels.end()) {
		Info.set_Info(ClsMoAccessInfo::modify_object_unknown) ;
		return ;
	} ;

	(*iterChannel)->modify_Comp_SourceSigList(content, PaS) ;
	Info.set_Info(ClsMoAccessInfo::modify_OK) ;

	// TEST
	// printChannels() ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::modify_Comp_SourceCon(const int Channel_ID, const string content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) {

	type_ChannelContainer::iterator iterChannel ;

	iterChannel = find_if(m_Channels.begin(), m_Channels.end(), ObjectHasID(Channel_ID));
	if (iterChannel == m_Channels.end()) {
		Info.set_Info(ClsMoAccessInfo::modify_object_unknown) ;
		return ;
	} ;

	(*iterChannel)->modify_Comp_SourceCon(content, PaS) ;
	Info.set_Info(ClsMoAccessInfo::modify_OK) ;

	// TEST
	// printChannels() ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::modify_Comp_DestSigList(const int Channel_ID, const string content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) {

	type_ChannelContainer::iterator iterChannel ;

	iterChannel = find_if(m_Channels.begin(), m_Channels.end(), ObjectHasID(Channel_ID));
	if (iterChannel == m_Channels.end()) {
		Info.set_Info(ClsMoAccessInfo::modify_object_unknown) ;
		return ;
	} ;

	(*iterChannel)->modify_Comp_DestSigList(content, PaS) ;
	Info.set_Info(ClsMoAccessInfo::modify_OK) ;

	// TEST
	// printChannels() ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::modify_Comp_DestCon(const int Channel_ID, const string content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) {

	type_ChannelContainer::iterator iterChannel ;

	iterChannel = find_if(m_Channels.begin(), m_Channels.end(), ObjectHasID(Channel_ID));
	if (iterChannel == m_Channels.end()) {
		Info.set_Info(ClsMoAccessInfo::modify_object_unknown) ;
		return ;
	} ;

	(*iterChannel)->modify_Comp_DestCon(content, PaS) ;
	Info.set_Info(ClsMoAccessInfo::modify_OK) ;

	// TEST
	// printChannels() ;
} ;
//*******************************************************************************************************
const olSDL::Symbol * olSDL::Diagram::get_Symbol(const int Symbol_ID, ClsMoAccessInfo & Info) const {

	type_SymbolContainer::const_iterator iterSymbolContainer ;
	iterSymbolContainer = find_if(m_Symbols.begin(), m_Symbols.end(), ObjectHasID(Symbol_ID));

	if (iterSymbolContainer != m_Symbols.end()) {
		Info.set_Info(ClsMoAccessInfo::get_OK) ;
		return (*iterSymbolContainer) ;
	} ;

	Info.set_Info(ClsMoAccessInfo::get_wrong_symbol_type);

	#ifdef olSDL_DL_TWO
		DebugPrinter::print("error in: const olSDL::Symbol * olSDL::Diagram::get_Symbol(const int Symbol_ID, ClsMoAccessInfo & Info)") ;
		DebugPrinter::print("Symbol not found!!!") ;
	#endif

	return NULL ;

} ;
//*******************************************************************************************************
void olSDL::Diagram::get_Syms(const EnumSym::Type SymbolType, const int Page_ID,
					     type_ListOf_Poi_to_ConstSym & ResultList,
					     ClsMoAccessInfo & Info) const {

	if ( ! m_PageManager.Page_exists(Page_ID) )
	{
		Info.set_Info(ClsMoAccessInfo::get_page_unknown) ;
		return ;
	} ;

	type_SymbolContainer::const_iterator iterSymbolContainer ;

	switch (SymbolType) {
		case EnumSym::TextSym:
		case EnumSym::ProInstRefSym:
		case EnumSym::ProTypeRefSym:
		case EnumSym::BlkInstRefSym:
		case EnumSym::BlkTypeRefSym:
		case EnumSym::ProcedureInstRefSym:
		{
			for (iterSymbolContainer=m_Symbols.begin(); iterSymbolContainer!=m_Symbols.end(); iterSymbolContainer++)
			{
				if ( (*iterSymbolContainer)->get_Page_ID() == Page_ID ) {
					if ( (*iterSymbolContainer)->get_SymbolType() == SymbolType ) {
						ResultList.push_back(*iterSymbolContainer) ;
					} ;
				} ;
			} ;
			Info.set_Info(ClsMoAccessInfo::get_OK);
			break;
		}
		default:
			Info.set_Info(ClsMoAccessInfo::get_wrong_symbol_type);
			#ifdef olSDL_DL_TWO
			{
			DebugPrinter::print("error in: void olSDL::Diagram::get_Syms(") ;
			DebugPrinter::print("unconsidered e_SymbolType !!!") ;
			}
			#endif
			;
	} ; // switch

} ;
//*******************************************************************************************************
const olSDL::ClsChannel * olSDL::Diagram::get_Channel(const int Channel_ID, ClsMoAccessInfo & Info) const {

	type_ChannelContainer::const_iterator iterChannel ;
	iterChannel = find_if(m_Channels.begin(), m_Channels.end(), ObjectHasID(Channel_ID));
	if (iterChannel != m_Channels.end()) {
		Info.set_Info(ClsMoAccessInfo::get_OK) ;
		return (*iterChannel) ;
	} else {
		Info.set_Info(ClsMoAccessInfo::get_channel_unknown) ;
		return NULL ;
	} ;

} ;
//*******************************************************************************************************
void olSDL::Diagram::get_Channels(const int Page_ID, type_ListOf_Poi_to_ConstChannel & ResultList,
						ClsMoAccessInfo & Info) const {

	if (! m_PageManager.Page_exists(Page_ID) ) {
		Info.set_Info(ClsMoAccessInfo::get_page_unknown) ;
		return ;
	} ;

	type_ChannelContainer::const_iterator iterChannel ;
	for (iterChannel=m_Channels.begin(); iterChannel != m_Channels.end(); iterChannel++)
	{
		if ((*iterChannel)->get_Page_ID() == Page_ID) {
			ResultList.push_back(*iterChannel) ;
		} ;
	} ;

	Info.set_Info(ClsMoAccessInfo::get_OK);

} ;
//*******************************************************************************************************
const olSDL::Symbol * olSDL::Diagram::insert_Symbol(const EnumSym::Type SymbolType, const int Symbol_ID, const int Page_ID,
						   		   const int x_left_up, const int y_left_up, const int width, const int height , const string content,
								   const int ReferencedDiagramID, const int ReferencedDiagramFirstPageID,
					           		   ClsMoAccessInfo & Info) {

	// page exists ?
	if ( ! m_PageManager.Page_exists(Page_ID) )
	{
		Info.set_Info(ClsMoAccessInfo::insert_page_unknown) ;
		return NULL ;
	} ;

	switch (SymbolType) {
		case EnumSym::TextSym:
		{
			ClsTextSym *  pTextSymbol ;
			pTextSymbol = new ClsTextSym(Symbol_ID, x_left_up, y_left_up, width, height , content, Page_ID, ReferencedDiagramID, ReferencedDiagramFirstPageID) ;
			m_Symbols.push_back(pTextSymbol) ;
			Info.set_Info(ClsMoAccessInfo::insert_OK) ;
			return pTextSymbol ;
			break ;
		}
		case EnumSym::ProInstRefSym:
		{
			ClsProInstRefSym * pProInstRefSym ;
			pProInstRefSym = new ClsProInstRefSym(Symbol_ID, x_left_up, y_left_up, width, height , content, Page_ID, ReferencedDiagramID, ReferencedDiagramFirstPageID) ;
			m_Symbols.push_back(pProInstRefSym) ;
			Info.set_Info(ClsMoAccessInfo::insert_OK) ;
			return pProInstRefSym ;
			break ;
		}
		case EnumSym::ProTypeRefSym:
		{
			ClsProTypeRefSym * pProTypeRefSym ;
			pProTypeRefSym = new ClsProTypeRefSym(Symbol_ID, x_left_up, y_left_up, width, height , content, Page_ID, ReferencedDiagramID, ReferencedDiagramFirstPageID) ;
			m_Symbols.push_back(pProTypeRefSym) ;
			Info.set_Info(ClsMoAccessInfo::insert_OK) ;
			return pProTypeRefSym ;
			break ;
		}
		case EnumSym::BlkInstRefSym:
		{
			ClsBlkInstRefSym * pBlkInstRefSym ;
			pBlkInstRefSym = new ClsBlkInstRefSym(Symbol_ID, x_left_up, y_left_up, width, height , content, Page_ID,
				ReferencedDiagramID, ReferencedDiagramFirstPageID) ;
			m_Symbols.push_back(pBlkInstRefSym) ;
			Info.set_Info(ClsMoAccessInfo::insert_OK) ;
			return pBlkInstRefSym ;
			break ;
		}
		case EnumSym::BlkTypeRefSym:
		{
			ClsBlkTypeRefSym * pBlkTypeRefSym ;
			pBlkTypeRefSym = new ClsBlkTypeRefSym(Symbol_ID, x_left_up, y_left_up, width, height , content, Page_ID,
				ReferencedDiagramID, ReferencedDiagramFirstPageID) ;
			m_Symbols.push_back(pBlkTypeRefSym) ;
			Info.set_Info(ClsMoAccessInfo::insert_OK) ;
			return pBlkTypeRefSym ;
			break ;
		}
		case EnumSym::ProcedureInstRefSym:
		{
			ClsProcedureInstRefSym * pProcedureInstRefSym ;
			pProcedureInstRefSym = new ClsProcedureInstRefSym(Symbol_ID, x_left_up, y_left_up, width, height , content, Page_ID,
				ReferencedDiagramID, ReferencedDiagramFirstPageID) ;
			m_Symbols.push_back(pProcedureInstRefSym) ;
			Info.set_Info(ClsMoAccessInfo::insert_OK) ;
			return pProcedureInstRefSym ;
			break ;
		}
		default:
			Info.set_Info(ClsMoAccessInfo::insert_unconsidered_symbol) ;
			#ifdef olSDL_DL_TWO
			{
			DebugPrinter::print("error in: const olSDL::Symbol * olSDL::Diagram::insert_Symbol(...") ;
			DebugPrinter::print("unconsidered symbol type !!!") ;
			}
			#endif
			;
	} ; // switch
} ;
//*******************************************************************************************************
const olSDL::ClsChannel * olSDL::Diagram::insert_Channel(const int Symbol_ID, const int Page_ID, ClsChannel::e_DelayType DelayType,
									      ClsChanName ChanName,
						 			      ClsChanSourceSigList ChanSourceSigList,
						 			      ClsChanSourceCon ChanSourceCon,
						 			      ClsChanDestSigList ChanDestSigList,
						 			      ClsChanDestCon ChanDestCon,
						 			      list<int> & ChanCoords,
								              bool bSourceIsEnv, const int ChanSourceModel_ID,
									      bool bDestIsEnv, const int ChanDestModel_ID,
									      ClsMoAccessInfo & Info) {

	// page exists ?
	if (! m_PageManager.Page_exists(Page_ID))
	{
		Info.set_Info(ClsMoAccessInfo::insert_page_unknown) ;
		return NULL;
	} ;

	ClsChannel * pChannel ;
	pChannel = new ClsChannel(Symbol_ID, Page_ID, DelayType,
		ChanName, ChanSourceSigList, ChanSourceCon, ChanDestSigList, ChanDestCon, ChanCoords,
		bSourceIsEnv, ChanSourceModel_ID, bDestIsEnv, ChanDestModel_ID) ;


	m_Channels.push_back(pChannel) ;
	Info.set_Info(ClsMoAccessInfo::insert_OK) ;
	return pChannel ;
} ;
//*******************************************************************************************************
const olSDL::ClsChannel * olSDL::Diagram::insert_Channel(const int Symbol_ID, const int Page_ID, ClsChannel::e_DelayType DelayType,
										ClsChannel::e_DirectionType DirectionType,
						 				ClsChanName ChanName,
						 				ClsChanSourceSigList ChanSourceSigList,
						 				ClsChanSourceCon ChanSourceCon,
						 				ClsChanDestSigList ChanDestSigList,
						 				ClsChanDestCon ChanDestCon,
						 				list<int> & ChanCoords,
				    	  					bool bSourceIsEnv, const int ChanSourceModel_ID,
						 				bool bDestIsEnv, const int ChanDestModel_ID,
						 				ClsMoAccessInfo & Info) {

	// page exists ?
	if (! m_PageManager.Page_exists(Page_ID))
	{
		Info.set_Info(ClsMoAccessInfo::insert_page_unknown) ;
		return NULL;
	} ;

	ClsChannel * pChannel ;
	pChannel = new ClsChannel(Symbol_ID, Page_ID, DelayType, DirectionType,
		ChanName, ChanSourceSigList, ChanSourceCon, ChanDestSigList, ChanDestCon, ChanCoords,
		bSourceIsEnv, ChanSourceModel_ID, bDestIsEnv, ChanDestModel_ID) ;


	m_Channels.push_back(pChannel) ;
	Info.set_Info(ClsMoAccessInfo::insert_OK) ;
	return pChannel ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::get_AllReferencedDiagrams(type_ListOf_Poi_to_ConstDiagram & ResultList, ClsMoAccessInfo & Info) const {

	Diagram * pDiagram ;
	type_SymbolContainer::const_iterator iterSymbolContainer ;

	for (iterSymbolContainer=m_Symbols.begin();  iterSymbolContainer!= m_Symbols.end(); iterSymbolContainer++)
	{
		if ( (*iterSymbolContainer)->is_referencing_Symbol() ) {
			pDiagram = (*iterSymbolContainer)->get_referenced_Diagram() ;
			ResultList.push_back(pDiagram) ;
		} ;
	} ;

	Info.set_Info(ClsMoAccessInfo::get_OK) ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::modify_SymbolContent(const int Symbol_ID, const string Content, ClsMoAccessInfo & Info) {

	type_SymbolContainer::iterator iterSymbolContainer ;
	iterSymbolContainer = find_if(m_Symbols.begin(), m_Symbols.end(), ObjectHasID(Symbol_ID));

	if (iterSymbolContainer != m_Symbols.end()) {
		(*iterSymbolContainer)->modify_Content(Content) ;
		Info.set_Info(ClsMoAccessInfo::modify_OK) ;
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_object_unknown) ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::set_SymbolPaS(const int Symbol_ID, const ClsRecPaS & PaS, ClsMoAccessInfo & Info)  {

	type_SymbolContainer::iterator iterSymbolContainer ;
	iterSymbolContainer = find_if(m_Symbols.begin(), m_Symbols.end(), ObjectHasID(Symbol_ID));

	if (iterSymbolContainer != m_Symbols.end()) {
		(*iterSymbolContainer)->set_PaS(PaS) ;
		Info.set_Info(ClsMoAccessInfo::modify_OK) ;
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_object_unknown) ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::delete_Symbol(const int Symbol_ID, ClsMoAccessInfo & Info)  {

	type_SymbolContainer::iterator iterSymbolContainer ;
	iterSymbolContainer = find_if(m_Symbols.begin(), m_Symbols.end(), ObjectHasID(Symbol_ID));

	if (iterSymbolContainer != m_Symbols.end()) {

		switch ( (*iterSymbolContainer)->get_SymbolType() ) {
			case EnumSym::TextSym:
			{
				delete ((ClsTextSym *)(*iterSymbolContainer)) ;
				break ;
			}
			case EnumSym::ProInstRefSym:
			{
				delete ((ClsProInstRefSym *)(*iterSymbolContainer)) ;
				break ;
			}
			case EnumSym::ProTypeRefSym:
			{
				delete ((ClsProTypeRefSym *)(*iterSymbolContainer)) ;
				break ;
			}
			case EnumSym::BlkInstRefSym:
			{
				delete ((ClsBlkInstRefSym *)(*iterSymbolContainer)) ;
				break ;
			}
			case EnumSym::BlkTypeRefSym:
			{
				delete ((ClsBlkTypeRefSym *)(*iterSymbolContainer)) ;
				break ;
			}
			case EnumSym::ProcedureInstRefSym:
			{
				delete ((ClsProcedureInstRefSym *)(*iterSymbolContainer)) ;
				break ;
			}
			default:
			{
				Info.set_Info(ClsMoAccessInfo::delete_object_unknown) ;
				return ;
			}
		} ; //switch

		m_Symbols.erase(iterSymbolContainer) ;
		Info.set_Info(ClsMoAccessInfo::delete_OK) ;
		return ;

	} else {
		Info.set_Info(ClsMoAccessInfo::delete_object_unknown) ;
		return ;
	}  ; // if (iterSymbolContainer != m_Symbols.end())

} ;
//*******************************************************************************************************
void olSDL::Diagram::delete_Channel(const int Channel_ID, ClsMoAccessInfo & Info) {

	type_ChannelContainer::iterator iterChannel ;

	iterChannel = find_if(m_Channels.begin(), m_Channels.end(), ObjectHasID(Channel_ID));
	if (iterChannel == m_Channels.end()) {
		Info.set_Info(ClsMoAccessInfo::delete_object_unknown) ;
		return ;
	} ;

	delete (*iterChannel) ;
	m_Channels.erase(iterChannel) ;
	Info.set_Info(ClsMoAccessInfo::delete_OK) ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::delete_Page(const int Page_ID, ClsMoAccessInfo & Info) {

	type_SymbolContainer::iterator iterSymbolContainer =
		find_if(m_Symbols.begin(), m_Symbols.end(), ObjectOnPage(Page_ID)) ;

	while (iterSymbolContainer != m_Symbols.end()) {

		switch ( (*iterSymbolContainer)->get_SymbolType() ) {

			case EnumSym::TextSym:
			{
				delete ((ClsTextSym *) (*iterSymbolContainer)) ;
				break ;
			}
			case EnumSym::ProInstRefSym:
			{
				delete ((ClsProInstRefSym *) (*iterSymbolContainer)) ;
				break ;
			}
			case EnumSym::ProTypeRefSym:
			{
				delete ((ClsProTypeRefSym *) (*iterSymbolContainer)) ;
				break ;
			}
			case EnumSym::BlkInstRefSym:
			{
				delete ((ClsBlkInstRefSym *) (*iterSymbolContainer)) ;
				break ;
			}
			case EnumSym::BlkTypeRefSym:
			{
				delete ((ClsBlkTypeRefSym *) (*iterSymbolContainer)) ;
				break ;
			}
			case EnumSym::ProcedureInstRefSym:
			{
				delete ((ClsProcedureInstRefSym *) (*iterSymbolContainer)) ;
				break ;
			}
			default:
				Info.set_Info(ClsMoAccessInfo::delete_object_unknown) ;
				#ifdef olSDL_DL_TWO
				{
				DebugPrinter::print("error in: void olSDL::Diagram::delete_Page(const int Page_ID, ClsMoAccessInfo & Info)") ;
				DebugPrinter::print("unconsidered Symbol Type !!!") ;
				}
				#endif
				return ;

		} ; // switch ( (*iterStateMachineSymbolContainer)->get_SymbolType() )

		m_Symbols.erase(iterSymbolContainer) ;

		iterSymbolContainer = find_if(m_Symbols.begin(), m_Symbols.end(), ObjectOnPage(Page_ID));

	} ; // 	while (iterSymbolContainer != m_Symbols.end())

	//============================
	// delete channels

	type_ChannelContainer::iterator iterChannel ;

	iterChannel = find_if(m_Channels.begin(), m_Channels.end(), ObjectOnPage(Page_ID));

	while (iterChannel != m_Channels.end()) {

		delete (*iterChannel) ;
		m_Channels.erase(iterChannel) ;

		iterChannel = find_if(m_Channels.begin(), m_Channels.end(), ObjectOnPage(Page_ID));
	} ;

	//============================
	// delete StateMa Symbols

	type_StateMachineSymbolContainer::iterator iterStateMachineSymbolContainer =
		find_if(m_StateMachineSymbols.begin(), m_StateMachineSymbols.end(), ObjectOnPage(Page_ID));

	while (iterStateMachineSymbolContainer != m_StateMachineSymbols.end()) {

		switch ( (*iterStateMachineSymbolContainer)->get_SymbolType() ) {

			case ClsStateMachineSymbol::start:
			{
				delete ((ClsStateMachineStartSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::state:
			{
				delete ((ClsStateMachineStateSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::output:
			{
				delete ((ClsStateMachineOutputSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::input:
			{
				delete ((ClsStateMachineInputSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::stop:
			{
				delete ((ClsStateMachineStopSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::task:
			{
				delete ((ClsStateMachineTaskSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::decision:
			{
				delete ((ClsStateMachineDecisionSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::create_pro:
			{
				delete ((ClsStateMachineCreateProSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::call_procedure:
			{
				delete ((ClsStateMachineCallProcedureSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::procedure_start_state:
			{
				delete ((ClsStateMachineProcedureStartStateSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::procedure_return:
			{
				delete ((ClsStateMachineProcedureReturnSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			default:
				Info.set_Info(ClsMoAccessInfo::delete_object_unknown) ;
				#ifdef olSDL_DL_TWO
				{
				DebugPrinter::print("error in: void olSDL::Diagram::delete_Page(const int Page_ID, ClsMoAccessInfo & Info)") ;
				DebugPrinter::print("unconsidered Process Symbol Type !!!") ;
				}
				#endif
				return ;
		} ; // switch ( (*iterStateMachineSymbolContainer)->get_SymbolType() )

		m_StateMachineSymbols.erase(iterStateMachineSymbolContainer) ;

		iterStateMachineSymbolContainer = find_if(m_StateMachineSymbols.begin(), m_StateMachineSymbols.end(), ObjectOnPage(Page_ID));

	} ; // while (iterStateMachineSymbolContainer != m_StateMachineSymbols.end())

	//============================
	// delete FlowLines

	type_FlowLineContainer::iterator iterFlowLines = find_if(m_FlowLines.begin(), m_FlowLines.end(), ObjectOnPage(Page_ID));

	while (iterFlowLines != m_FlowLines.end()) {

		delete (*iterFlowLines) ;
		m_FlowLines.erase(iterFlowLines) ;

		iterFlowLines = find_if(m_FlowLines.begin(), m_FlowLines.end(), ObjectOnPage(Page_ID));
	} ;


	m_PageManager.delete_Page(Page_ID, Info) ;
} ;
//*******************************************************************************************************
olSDL::EnumDia::Type olSDL::Diagram::get_DiagramType( ) const {

	return m_DiagramType ;
} ;
//*******************************************************************************************************
int olSDL::Diagram::get_FirstPageID(ClsMoAccessInfo & Info) const {

	return (m_PageManager.get_FirstPageID(Info)) ;
} ;
//*******************************************************************************************************
int olSDL::Diagram::get_LastPageID(ClsMoAccessInfo & Info) const {

	return (m_PageManager.get_LastPageID(Info)) ;
} ;
//*******************************************************************************************************
int olSDL::Diagram::get_NextPageID(const int PredecessorPage_ID, ClsPageManager::e_PageIndex & PageIndex, ClsMoAccessInfo & Info) const {

	return (m_PageManager.get_NextPageID(PredecessorPage_ID, PageIndex, Info)) ;
} ;
//*******************************************************************************************************
int olSDL::Diagram::get_PreviousPageID(const int AncestorPage_ID, ClsPageManager::e_PageIndex & PageIndex, ClsMoAccessInfo & Info) const {

	return (m_PageManager.get_PreviousPageID(AncestorPage_ID, PageIndex, Info)) ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::get_IDsOfPages(list<int> & IDsOfPages) const {

	m_PageManager.get_IDsOfPages(IDsOfPages) ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::get_PageSize(const int Page_ID, int & PageWidth, int & PageHeight, ClsMoAccessInfo & Info) const  {

 	m_PageManager.get_PageSize(Page_ID, PageWidth, PageHeight, Info) ;

} ;
//*******************************************************************************************************
void olSDL::Diagram::get_PageFrameInfo(const int Page_ID, ClsRecPaS & PaS, string & HP_DiaType, string & HP_Name,
									string & HP_Index, ClsMoAccessInfo & Info) const {

	get_HeaderPart_Name(HP_Name) ;
	get_HeaderPart_DiagramType(HP_DiaType) ;

	get_HeaderPart_Index(Page_ID, HP_Index, Info)  ;
	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	m_PageManager.get_FramePaS(Page_ID, PaS, Info) ;

} ;
//*******************************************************************************************************
void olSDL::Diagram::get_HeaderPart_Name(string & str_HeaderPart_Name) const {

	str_HeaderPart_Name = m_str_HeaderPart_Name ;

} ;
//*******************************************************************************************************
void olSDL::Diagram::get_HeaderPart_Index(const int Page_ID, string & str_HeaderPart_Index, ClsMoAccessInfo & Info) const {

	string str_NumberOfPages ;
	int CurrentIndex ;
	string str_CurrentIndex ;

	str_HeaderPart_Index = "" ;

	// get number of pages
	str_NumberOfPages = olSDL::Converter::Int_to_String(m_PageManager.get_NumberOfPages()) ;

	// get current page index
	CurrentIndex = m_PageManager.get_PageIndex(Page_ID, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		str_HeaderPart_Index = "error" ;
		return ;
	} ;

	str_CurrentIndex = olSDL::Converter::Int_to_String(CurrentIndex) ;

	// compose
	str_HeaderPart_Index.append(str_CurrentIndex) ;
	str_HeaderPart_Index.append("(") ;
	str_HeaderPart_Index.append(str_NumberOfPages) ;
	str_HeaderPart_Index.append(")") ;

} ;
//*******************************************************************************************************
void olSDL::Diagram::get_HeaderPart_DiagramType(string & str_HeaderPart_DiaType) const {

	str_HeaderPart_DiaType = m_str_HeaderPart_DiaType ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::add_NewPage_after(const int NewPage_ID, const int PredecessorPage_ID, ClsMoAccessInfo & Info) {

	m_PageManager.add_NewPage_after(NewPage_ID, PredecessorPage_ID, Info) ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::modify_HeaderPart_Name(const string str_HeaderPart_Name) {

	m_str_HeaderPart_Name = str_HeaderPart_Name ;

} ;
//*******************************************************************************************************
//*******************************************************************************************************
const olSDL::Symbol * olSDL::Diagram::get_referencing_Symbol() const {

	// just for sure
	if (is_referenced_Diagram() && (m_pReferencingSymbol !=NULL)) {
		return m_pReferencingSymbol ;
	} else {
		DebugPrinter::print("error: const olSDL::Symbol * olSDL::Diagram::get_referencing_Symbol() const") ;
		return NULL ;
	} ;

} ;
//*******************************************************************************************************
int olSDL::Diagram::get_referencing_Symbol_Model_ID() const {

	// just for sure
	if (is_referenced_Diagram() && (m_pReferencingSymbol !=NULL)) {
		return (m_pReferencingSymbol->get_Model_ID()) ;
	} else {
		DebugPrinter::print("error: olSDL::Diagram::get_referencing_Symbol_Model_ID()") ;
		return -1 ;
	} ;

} ;

//*******************************************************************************************************
// pages
//*******************************************************************************************************
void olSDL::Diagram::modify_PageWidth(const int Page_ID, const int PageWidth, ClsMoAccessInfo & Info) {

	m_PageManager.modify_PageWidth(Page_ID, PageWidth, Info) ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::modify_PageHeight(const int Page_ID, const int PageHeight, ClsMoAccessInfo & Info) {

	m_PageManager.modify_PageHeight(Page_ID, PageHeight, Info) ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::modify_PageFramePaS(const int Page_ID, const int x_left_up,
				const int y_left_up, const int FrameWidth, const int FrameHeight, ClsMoAccessInfo & Info) {

	m_PageManager.modify_PageFramePaS(Page_ID, x_left_up, y_left_up, FrameWidth, FrameHeight, Info) ;
	
} ;
//*******************************************************************************************************
// state machine symbol
//*******************************************************************************************************
const olSDL::ClsStateMachineSymbol * olSDL::Diagram::get_StateMachineSymbol(const int Symbol_ID, ClsMoAccessInfo & Info) const {

	type_StateMachineSymbolContainer::const_iterator iterStateMachineSymbolContainer ;

	iterStateMachineSymbolContainer = find_if(m_StateMachineSymbols.begin(), m_StateMachineSymbols.end(), ObjectHasID(Symbol_ID));
	if (iterStateMachineSymbolContainer != m_StateMachineSymbols.end()) {
		Info.set_Info(ClsMoAccessInfo::get_OK) ;
		return (*iterStateMachineSymbolContainer) ;
	} ;

	Info.set_Info(ClsMoAccessInfo::get_object_unknown);
	#ifdef olSDL_DL_TWO
		DebugPrinter::print("error in: const ClsStateMachineSymbol * olSDL::Diagram::get_StateMachineSymbol(const int Symbol_ID, ClsMoAccessInfo & Info) const") ;
		DebugPrinter::print("Process Symbol not found!!!") ;
	#endif

	return NULL ;

} ;
//*******************************************************************************************************
void olSDL::Diagram::get_StateMachineSyms(const int Page_ID, type_ListOfPoi_to_ConstStateMachineSymbol & ResultList, ClsMoAccessInfo & Info) const {

	type_StateMachineSymbolContainer::const_iterator iterStateMachineSymbolContainer ;

	if ( ! m_PageManager.Page_exists(Page_ID) )
	{
		Info.set_Info(ClsMoAccessInfo::get_page_unknown) ;
		return ;
	} ;


	for (iterStateMachineSymbolContainer=m_StateMachineSymbols.begin(); iterStateMachineSymbolContainer != m_StateMachineSymbols.end(); iterStateMachineSymbolContainer++)
	{
			if ((*iterStateMachineSymbolContainer)->get_Page_ID() == Page_ID) {
				ResultList.push_back(*iterStateMachineSymbolContainer) ;
			} ;
	} ;

	Info.set_Info(ClsMoAccessInfo::get_OK);
} ;
//*******************************************************************************************************
void olSDL::Diagram::get_StateMachineSyms(const ClsStateMachineSymbol::e_StateMachineSymbolType StateMachineSymbolType,
				const int Page_ID, type_ListOfPoi_to_ConstStateMachineSymbol & ResultList, ClsMoAccessInfo & Info) const {

	type_StateMachineSymbolContainer::const_iterator iterStateMachineSymbolContainer ;

	if ( ! m_PageManager.Page_exists(Page_ID) )
	{
		Info.set_Info(ClsMoAccessInfo::get_page_unknown) ;
		return ;
	} ;

	for (iterStateMachineSymbolContainer=m_StateMachineSymbols.begin(); iterStateMachineSymbolContainer != m_StateMachineSymbols.end(); iterStateMachineSymbolContainer++)
	{
		if ( (*iterStateMachineSymbolContainer)->get_SymbolType() == StateMachineSymbolType ) {
			if ((*iterStateMachineSymbolContainer)->get_Page_ID() == Page_ID) {
				ResultList.push_back(*iterStateMachineSymbolContainer) ;
			} ;
		} ;
	} ;

	Info.set_Info(ClsMoAccessInfo::get_OK);

} ;
//*******************************************************************************************************
void olSDL::Diagram::modify_StateMachineSymPaS(const int Symbol_ID, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) {

	type_StateMachineSymbolContainer::iterator iterStateMachineSymbolContainer ;

	iterStateMachineSymbolContainer = find_if(m_StateMachineSymbols.begin(), m_StateMachineSymbols.end(), ObjectHasID(Symbol_ID));
	if (iterStateMachineSymbolContainer != m_StateMachineSymbols.end()) {
		(*iterStateMachineSymbolContainer)->set_PaS(PaS) ;
		Info.set_Info(ClsMoAccessInfo::modify_OK) ;
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_object_unknown) ;

} ;
//*******************************************************************************************************
void olSDL::Diagram::modify_StateMachineSymContent(const int Symbol_ID, const string Content, ClsMoAccessInfo & Info) {

	type_StateMachineSymbolContainer::iterator iterStateMachineSymbolContainer ;

	iterStateMachineSymbolContainer = find_if(m_StateMachineSymbols.begin(), m_StateMachineSymbols.end(), ObjectHasID(Symbol_ID));
	if (iterStateMachineSymbolContainer != m_StateMachineSymbols.end()) {
		(*iterStateMachineSymbolContainer)->modify_Content(Content) ;
		Info.set_Info(ClsMoAccessInfo::modify_OK) ;
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_object_unknown) ;

} ;
//*******************************************************************************************************
const olSDL::ClsStateMachineSymbol * olSDL::Diagram::insert_StateMachineSymbol(
				const ClsStateMachineSymbol::e_StateMachineSymbolType SymbolType,
				const int Symbol_ID, const int Page_ID, const int x_left_up, const int y_left_up, const int width,
				const int height , const string content, ClsMoAccessInfo & Info) {

	// page exists ?
	if (! m_PageManager.Page_exists(Page_ID))
	{
		Info.set_Info(ClsMoAccessInfo::insert_page_unknown) ;
		return NULL;
	} ;

	ClsStateMachineSymbol * pStateMachineSymbol ;

	switch (SymbolType) {
		case ClsStateMachineSymbol::start:
		{
			pStateMachineSymbol = new ClsStateMachineStartSymbol(Symbol_ID, x_left_up, y_left_up, width, height, content, Page_ID) ;
			break ;
		}
		case ClsStateMachineSymbol::state:
		{
			pStateMachineSymbol = new ClsStateMachineStateSymbol(Symbol_ID, x_left_up, y_left_up, width, height, content, Page_ID) ;
			break ;
		}
		case ClsStateMachineSymbol::output:
		{
			pStateMachineSymbol = new ClsStateMachineOutputSymbol(Symbol_ID, x_left_up, y_left_up, width, height, content, Page_ID) ;
			break ;
		}
		case ClsStateMachineSymbol::input:
		{
			pStateMachineSymbol = new ClsStateMachineInputSymbol(Symbol_ID, x_left_up, y_left_up, width, height, content, Page_ID) ;
			break ;
		}
		case ClsStateMachineSymbol::stop:
		{
			pStateMachineSymbol = new ClsStateMachineStopSymbol(Symbol_ID, x_left_up, y_left_up, width, height, content, Page_ID) ;
			break ;
		}
		case ClsStateMachineSymbol::task:
		{
			pStateMachineSymbol = new ClsStateMachineTaskSymbol(Symbol_ID, x_left_up, y_left_up, width, height, content, Page_ID) ;
			break ;
		}
		case ClsStateMachineSymbol::condition:
		{
			pStateMachineSymbol = new ClsStateMachineConditionSymbol(Symbol_ID, x_left_up, y_left_up, width, height, content, Page_ID) ;
			break ;
		}
		case ClsStateMachineSymbol::decision:
		{
			pStateMachineSymbol = new ClsStateMachineDecisionSymbol(Symbol_ID, x_left_up, y_left_up, width, height, content, Page_ID) ;
			break ;
		}
		case ClsStateMachineSymbol::create_pro:
		{
			pStateMachineSymbol = new ClsStateMachineCreateProSymbol(Symbol_ID, x_left_up, y_left_up, width, height, content, Page_ID) ;
			break ;
		}
		case ClsStateMachineSymbol::call_procedure:
		{
			pStateMachineSymbol = new ClsStateMachineCallProcedureSymbol(Symbol_ID, x_left_up, y_left_up, width, height, content, Page_ID) ;
			break ;
		}
		case ClsStateMachineSymbol::procedure_start_state:
		{
			pStateMachineSymbol = new ClsStateMachineProcedureStartStateSymbol(Symbol_ID, x_left_up, y_left_up, width, height, content, Page_ID) ;
			break ;
		}
		case ClsStateMachineSymbol::procedure_return:
		{
			pStateMachineSymbol = new ClsStateMachineProcedureReturnSymbol(Symbol_ID, x_left_up, y_left_up, width, height, content, Page_ID) ;
			break ;
		}
		default:
			Info.set_Info(ClsMoAccessInfo::insert_ProcDiaSym_unknown) ;
			#ifdef olSDL_DL_TWO
			{
			DebugPrinter::print("error in: const olSDL::ClsStateMachineSymbol * olSDL::Diagram::insert_StateMachineSymbol") ;
			DebugPrinter::print("unconsidered Process Symbol Type !!!") ;
			}
			#endif
			return NULL ;
	} ; // switch (SymbolType)


	m_StateMachineSymbols.push_back(pStateMachineSymbol) ;

	Info.set_Info(ClsMoAccessInfo::insert_OK) ;
	return pStateMachineSymbol ;

} ;
//*******************************************************************************************************
void olSDL::Diagram::delete_StateMachineSymbol(const int Symbol_ID, ClsMoAccessInfo & Info)  {

	type_StateMachineSymbolContainer::iterator iterStateMachineSymbolContainer ;

	iterStateMachineSymbolContainer = find_if(m_StateMachineSymbols.begin(), m_StateMachineSymbols.end(), ObjectHasID(Symbol_ID));
	if (iterStateMachineSymbolContainer != m_StateMachineSymbols.end()) {

		switch ( (*iterStateMachineSymbolContainer)->get_SymbolType() ) {

			case ClsStateMachineSymbol::start:
			{
				delete ((ClsStateMachineStartSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::state:
			{
				delete ((ClsStateMachineStateSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::output:
			{
				delete ((ClsStateMachineOutputSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::input:
			{
				delete ((ClsStateMachineInputSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::stop:
			{
				delete ((ClsStateMachineStopSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::task:
			{
				delete ((ClsStateMachineTaskSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::decision:
			{
				delete ((ClsStateMachineDecisionSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::create_pro:
			{
				delete ((ClsStateMachineCreateProSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::call_procedure:
			{
				delete ((ClsStateMachineCallProcedureSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::procedure_start_state:
			{
				delete ((ClsStateMachineProcedureStartStateSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			case ClsStateMachineSymbol::procedure_return:
			{
				delete ((ClsStateMachineProcedureReturnSymbol*) (*iterStateMachineSymbolContainer)) ;
				break ;
			}
			default:
				Info.set_Info(ClsMoAccessInfo::delete_object_unknown) ;
				#ifdef olSDL_DL_TWO
				{
				DebugPrinter::print("error in: void olSDL::Diagram::delete_StateMachineSymbol(const int Symbol_ID, ClsMoAccessInfo & Info)") ;
				DebugPrinter::print("unconsidered Process Symbol Type !!!") ;
				}
				#endif
				return ;
		} ; // switch ( (*iterStateMachineSymbolContainer)->get_SymbolType() )


		m_StateMachineSymbols.erase(iterStateMachineSymbolContainer) ;
		Info.set_Info(ClsMoAccessInfo::delete_OK) ;
		return ;

	} ; // (iterStateMachineSymbolContainer != m_StateMachineSymbols.end())

} ;
//*******************************************************************************************************
// flow lines
//*******************************************************************************************************
const olSDL::ClsFlowLine * olSDL::Diagram::get_FlowLine(const int FlowLine_ID, ClsMoAccessInfo & Info) const {

	type_FlowLineContainer::const_iterator iterFlowLines ;

	iterFlowLines = find_if(m_FlowLines.begin(), m_FlowLines.end(), ObjectHasID(FlowLine_ID));
	if (iterFlowLines != m_FlowLines.end()) {
		Info.set_Info(ClsMoAccessInfo::get_OK) ;
		return (*iterFlowLines) ;
	} else {
		Info.set_Info(ClsMoAccessInfo::get_flowline_unknown) ;
		return NULL ;
	} ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::get_FlowLines (const int Page_ID, type_ListOfPoi_to_ConstFlowLine & ResultList, ClsMoAccessInfo & Info) const {

	if (! m_PageManager.Page_exists(Page_ID) ) {
		Info.set_Info(ClsMoAccessInfo::get_page_unknown) ;
		return ;
	} ;

	//handle
	type_FlowLineContainer::const_iterator iterFlowLines ;

	for (iterFlowLines=m_FlowLines.begin(); iterFlowLines != m_FlowLines.end(); iterFlowLines++)
	{
		if ((*iterFlowLines)->get_Page_ID() == Page_ID) {
			ResultList.push_back(*iterFlowLines) ;
		} ;
	} ;

	Info.set_Info(ClsMoAccessInfo::get_OK);
} ;
//*******************************************************************************************************
void olSDL::Diagram::modify_FlowLineCoords(const int FlowLine_ID, list<int> & FlowLineCoords, ClsMoAccessInfo & Info) {

	type_FlowLineContainer::iterator iterFlowLines ;

	iterFlowLines = find_if(m_FlowLines.begin(), m_FlowLines.end(), ObjectHasID(FlowLine_ID));
	if (iterFlowLines == m_FlowLines.end()) {
		Info.set_Info(ClsMoAccessInfo::modify_object_unknown) ;
		return ;
	} ;

	(*iterFlowLines)->modify_FlowLineCoords(FlowLineCoords) ;
	Info.set_Info(ClsMoAccessInfo::modify_OK) ;

} ;
//*******************************************************************************************************
void olSDL::Diagram::modify_DecisionFlowLinePaS(const int DecisionFlowLine_ID, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) {

	type_FlowLineContainer::iterator iterFlowLines ;

	iterFlowLines = find_if(m_FlowLines.begin(), m_FlowLines.end(), ObjectHasID(DecisionFlowLine_ID));
	if (iterFlowLines == m_FlowLines.end()) {
		Info.set_Info(ClsMoAccessInfo::modify_object_unknown) ;
		return ;
	} ;

	if ( (*iterFlowLines)->get_FlowLineType() != ClsFlowLine::decision) {
		Info.set_Info(ClsMoAccessInfo::modify_object_unknown) ;
		return ;
	} ;

	((ClsDecisionFlowLine*)(*iterFlowLines))->modify_DecisionFlowLinePaS(PaS) ;
	Info.set_Info(ClsMoAccessInfo::modify_OK) ;

} ;
//*******************************************************************************************************
void olSDL::Diagram::modify_DecisionFlowLineContent(const int DecisionFlowLine_ID, const string Content, ClsMoAccessInfo & Info) {

	type_FlowLineContainer::iterator iterFlowLines ;

	iterFlowLines = find_if(m_FlowLines.begin(), m_FlowLines.end(), ObjectHasID(DecisionFlowLine_ID));
	if (iterFlowLines == m_FlowLines.end()) {
		Info.set_Info(ClsMoAccessInfo::modify_object_unknown) ;
		return ;
	} ;

	if ( (*iterFlowLines)->get_FlowLineType() != ClsFlowLine::decision) {
		Info.set_Info(ClsMoAccessInfo::modify_object_unknown) ;
		return ;
	} ;

	((ClsDecisionFlowLine*)(*iterFlowLines))->modify_DecisionFlowLineContent(Content) ;
	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************
const olSDL::ClsNormalFlowLine * olSDL::Diagram::insert_FlowLine(const int FlowLine_ID, const int Page_ID, list<int> & FlowLineCoords,
				    	         const int FlowLineSourceModel_ID, const int FlowLineDestModel_ID, ClsMoAccessInfo & Info) {

	ClsNormalFlowLine * pFlowLine = NULL ;

	// page exists ?
	if (! m_PageManager.Page_exists(Page_ID))
	{
		Info.set_Info(ClsMoAccessInfo::insert_page_unknown) ;
		return NULL;
	} ;

	pFlowLine = new ClsNormalFlowLine (FlowLine_ID, Page_ID, FlowLineCoords,
		FlowLineSourceModel_ID, FlowLineDestModel_ID) ;


	m_FlowLines.push_back( &(*pFlowLine)) ;
	Info.set_Info(ClsMoAccessInfo::insert_OK) ;
	return pFlowLine ;

} ;
//*******************************************************************************************************
const olSDL::ClsDecisionFlowLine * olSDL::Diagram::insert_DecisionFlowLine(const int FlowLine_ID, const int Page_ID, list<int> & FlowLineCoords,
				    	         const int FlowLineSourceModel_ID, const int FlowLineDestModel_ID, const string & content, const ClsRecPaS & PaS,
						 ClsMoAccessInfo & Info) {

	ClsDecisionFlowLine * pFlowLine = NULL ;

	// page exists ?
	if (! m_PageManager.Page_exists(Page_ID))
	{
		Info.set_Info(ClsMoAccessInfo::insert_page_unknown) ;
		return NULL;
	} ;

	pFlowLine = new ClsDecisionFlowLine(FlowLine_ID, Page_ID, FlowLineCoords, FlowLineSourceModel_ID, FlowLineDestModel_ID, content, PaS) ;

	m_FlowLines.push_back( &(*pFlowLine)) ;
	Info.set_Info(ClsMoAccessInfo::insert_OK) ;
	return pFlowLine ;
} ;
//*******************************************************************************************************
void olSDL::Diagram::delete_FlowLine(const int FlowLine_ID, ClsMoAccessInfo & Info) {

	type_FlowLineContainer::iterator iterFlowLines ;

	iterFlowLines = find_if(m_FlowLines.begin(), m_FlowLines.end(), ObjectHasID(FlowLine_ID));
	if (iterFlowLines == m_FlowLines.end()) {
		Info.set_Info(ClsMoAccessInfo::delete_object_unknown) ;
		return ;
	} ;

	switch ((*iterFlowLines)->get_FlowLineType()) {

		case ClsFlowLine::normal:
		{
			delete ( (ClsNormalFlowLine*) (*iterFlowLines) ) ;
			break ;
		}
		case ClsFlowLine::decision:
		{
			delete ( (ClsDecisionFlowLine*) (*iterFlowLines) ) ;
			break ;
		}
		default:
		{
			Info.set_Info(ClsMoAccessInfo::delete_object_unknown) ;
		}
	} ;

	m_FlowLines.erase(iterFlowLines) ;

	Info.set_Info(ClsMoAccessInfo::delete_OK) ;
} ;

//*******************************************************************************************************
// helper
//*******************************************************************************************************
void olSDL::Diagram::printChannels() const {

	type_ChannelContainer::const_iterator iterChannel ;

	iterChannel = m_Channels.begin() ;
	while (iterChannel != m_Channels.end()) {
		(*iterChannel)->print() ;
		iterChannel++ ;
	} ;
} ;
//*******************************************************************************************************
//*******************************************************************************************************


