
#include "Page.h"
#include "Helper.h"


const int olSDL::ClsPage::PAGEWIDTH = 800 ;
const int olSDL::ClsPage::PAGEHEIGHT = 600 ;
const int olSDL::ClsPage::FRAME_PERCENT = 80 ;


//*******************************************************************************************************
olSDL::ClsPage::ClsPage(const int Page_ID) : ClsObjectWithModel_ID(Page_ID), m_PageWidth(PAGEWIDTH), m_PageHeight(PAGEHEIGHT) {

	#ifdef olSDL_DL_ONE
	//DebugPrinter::print("CON: olSDL::ClsPage::ClsPage(const int Page_ID)") ;
	#endif

	// calculate frame
	int Frame_x_left_up ;
	int Frame_y_left_up ;
	int Frame_width ;
	int Frame_height ;

	Frame_x_left_up = (((100-FRAME_PERCENT)/2) * m_PageWidth)/100 ;
	Frame_y_left_up = (((100-FRAME_PERCENT)/2) * m_PageHeight)/100 ;

	Frame_width = (FRAME_PERCENT*m_PageWidth)/100 ;
	Frame_height = (FRAME_PERCENT*m_PageHeight)/100 ;

	ClsPageFrame PageFrame(Frame_x_left_up, Frame_y_left_up, Frame_width, Frame_height) ;

	m_Frame = PageFrame ;

} ;
//*******************************************************************************************************
olSDL::ClsPage::~ClsPage( ) {

	#ifdef olSDL_DL_ONE
	//DebugPrinter::print("DES: olSDL::ClsPage::~ClsPage( )") ;
	#endif

} ;
//*******************************************************************************************************
int olSDL::ClsPage::get_PageWidth( ) const {
	return m_PageWidth ;
} ;
//*******************************************************************************************************
int olSDL::ClsPage::get_PageHeight( ) const {
	return m_PageHeight ;
} ;
//*******************************************************************************************************
void olSDL::ClsPage::get_FramePaS(ClsRecPaS & PaS) const {
	PaS = m_Frame.get_PaS() ;
} ;
//*******************************************************************************************************
void olSDL::ClsPage::modify_PageWidth(const int PageWidth, ClsMoAccessInfo & Info) {

	if (! ((0 < PageWidth) && ((m_Frame.get_PaS()).get_x_right_down() < PageWidth)) ) {

		Info.set_Info(ClsMoAccessInfo::modify_wrong_page_coords) ;
		return ;
	}

	m_PageWidth = PageWidth ;
	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************
void olSDL::ClsPage::modify_PageHeight(const int PageHeight, ClsMoAccessInfo & Info) {

	if (! ((0 < PageHeight) && ((m_Frame.get_PaS()).get_y_right_down() < PageHeight)) ) {

		Info.set_Info(ClsMoAccessInfo::modify_wrong_page_coords) ;
		return ;
	}

	m_PageHeight = PageHeight ;
	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************
olSDL::ClsMoEleType::e_MoEle olSDL::ClsPage::get_MoEleType() const {
	return olSDL::ClsMoEleType::Page ;
} ;
//*******************************************************************************************************
void olSDL::ClsPage::modify_FramePaS(const int x_left_up, const int y_left_up, const int FrameWidth, const int FrameHeight, ClsMoAccessInfo & Info) {

	if (! ( (0 < x_left_up) && (0 < y_left_up) &&
		((x_left_up+FrameWidth) < m_PageWidth) && ((y_left_up+FrameHeight) < m_PageHeight) ) ) {

		Info.set_Info(ClsMoAccessInfo::modify_wrong_page_coords) ;
		return ;
	} ;

	m_Frame.modify_PaS(x_left_up, y_left_up, FrameWidth, FrameHeight) ;

	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************


//*******************************************************************************************************
// local class "ClsPageFrame"
//*******************************************************************************************************
olSDL::ClsPage::ClsPageFrame::ClsPageFrame(const int x_left_up, const int y_left_up, const int width, const int height)  {

	#ifdef olSDL_DL_ONE
	//DebugPrinter::print("CON: olSDL::ClsPage::ClsPageFrame::ClsPageFrame(const int x_left_up, const int y_left_up, const int width, const int height)") ;
	#endif

	m_pArea = new ClsRecPaS(x_left_up, y_left_up, width, height) ;

} ;
//*******************************************************************************************************
olSDL::ClsPage::ClsPageFrame::ClsPageFrame() {

	#ifdef olSDL_DL_ONE
	//DebugPrinter::print("CON: olSDL::ClsPage::ClsPageFrame::ClsPageFrame()") ;
	#endif

	m_pArea = NULL ;

} ;
//*******************************************************************************************************

olSDL::ClsPage::ClsPageFrame::~ClsPageFrame() {

	#ifdef olSDL_DL_ONE
	//DebugPrinter::print("DES: olSDL::ClsPage::ClsPageFrame::~ClsPageFrame()") ;
	#endif

	if (m_pArea!=NULL) {
		delete m_pArea ;
	} ;
} ;
//*******************************************************************************************************
olSDL::ClsPage::ClsPageFrame::ClsPageFrame(const ClsPageFrame & PageFrame) {

	#ifdef olSDL_DL_ONE
	//DebugPrinter::print("Copy: olSDL::ClsPage::ClsPageFrame::ClsPageFrame( ...") ;
	#endif

	if (PageFrame.m_pArea != NULL) {
		this->m_pArea = new ClsRecPaS(*(PageFrame.m_pArea));
	} else {
		this->m_pArea = NULL ;
	} ;
} ;
//*******************************************************************************************************
olSDL::ClsPage::ClsPageFrame& olSDL::ClsPage::ClsPageFrame::operator=(const ClsPageFrame & PageFrame) {

	#ifdef olSDL_DL_ONE
	//DebugPrinter::print("ASSIGN: olSDL::ClsPage::ClsPageFrame::operator=") ;
	#endif

	if (this != &PageFrame) {
		if (PageFrame.m_pArea != NULL) {
			this->m_pArea = new ClsRecPaS(*(PageFrame.m_pArea));
		} else {
			this->m_pArea = NULL ;
		} ;
	} ;

	return (*this) ;
} ;
//*******************************************************************************************************
olSDL::ClsRecPaS olSDL::ClsPage::ClsPageFrame::get_PaS() const {

	return (*m_pArea) ;
} ;
//*******************************************************************************************************
void olSDL::ClsPage::ClsPageFrame::modify_PaS(const int x_left_up, const int y_left_up, const int FrameWidth, const int FrameHeight) {

	(*m_pArea) = ClsRecPaS(x_left_up, y_left_up, FrameWidth, FrameHeight) ;
} ;
//*******************************************************************************************************

