
#include "PageManager.h"
#include "MoAccessInfo.h"

#include <algorithm>

//*******************************************************************************************************
olSDL::ClsPageManager::ClsPageManager(const int FirstPage_ID) {

	m_Pages.push_back(ClsPage(FirstPage_ID)) ;

} ;
//*******************************************************************************************************
int olSDL::ClsPageManager::get_NumberOfPages() const {
	return (m_Pages.size()) ;
} ;
//*******************************************************************************************************
void olSDL::ClsPageManager::get_PageSize(const int Page_ID, int & PageWidth, int & PageHeight, ClsMoAccessInfo & Info) const  {

	type_PageContainer::const_iterator iterPageContainer ;

	iterPageContainer = find_if(m_Pages.begin(), m_Pages.end(), PageHasID(Page_ID));

	if (iterPageContainer==m_Pages.end()) {
		Info.set_Info(ClsMoAccessInfo::get_page_unknown) ;
		return ;
	} ;

	PageWidth = iterPageContainer->get_PageWidth( ) ;
	PageHeight = iterPageContainer->get_PageHeight( ) ;

	Info.set_Info(ClsMoAccessInfo::get_OK) ;

} ;
//*******************************************************************************************************
void olSDL::ClsPageManager::get_FramePaS(const int Page_ID, ClsRecPaS & PaS, ClsMoAccessInfo & Info) const {

	type_PageContainer::const_iterator iterPageContainer ;

	iterPageContainer = find_if(m_Pages.begin(), m_Pages.end(), PageHasID(Page_ID));

	if (iterPageContainer==m_Pages.end()) {
		Info.set_Info(ClsMoAccessInfo::get_page_unknown) ;
		return ;
	} ;

	iterPageContainer->get_FramePaS(PaS) ;

	Info.set_Info(ClsMoAccessInfo::get_OK) ;

} ;
//*******************************************************************************************************
int olSDL::ClsPageManager::get_PageIndex(const int Page_ID, ClsMoAccessInfo & Info) const {

	bool found = false ;
	int index = 1 ;
	type_PageContainer::const_iterator iterPageContainer ;

	iterPageContainer = m_Pages.begin() ;
	while ( (!found) && (iterPageContainer!=m_Pages.end()) ) {

		if (Page_ID == (iterPageContainer->get_Model_ID()) ) {
			found = true ;
		} else {
			index++ ;
		} ;
		iterPageContainer++ ;

	} ; // while

	if (!found) {
		Info.set_Info(ClsMoAccessInfo::get_page_unknown) ;
		return index ;
	} ;


	Info.set_Info(ClsMoAccessInfo::get_OK) ;
	return index ;

} ;
//*******************************************************************************************************
int olSDL::ClsPageManager::get_LastPageID(ClsMoAccessInfo & Info) const {

	type_PageContainer::const_iterator iterPage ;

	if (m_Pages.size() == 0) {
		Info.set_Info(ClsMoAccessInfo::page_get_page_unknown) ;
		return -1 ;
	}

	iterPage = m_Pages.end() ;
	iterPage-- ;

	Info.set_Info(ClsMoAccessInfo::page_get_OK) ;

	return (iterPage->get_Model_ID()) ;
} ;
//*******************************************************************************************************
int olSDL::ClsPageManager::get_FirstPageID(ClsMoAccessInfo & Info) const {

	type_PageContainer::const_iterator iterPage ;

	if (m_Pages.size() == 0) {
		Info.set_Info(ClsMoAccessInfo::page_get_page_unknown) ;
		return -1 ;
	}

	iterPage = m_Pages.begin() ;

	Info.set_Info(ClsMoAccessInfo::page_get_OK) ;

	return (iterPage->get_Model_ID()) ;
} ;
//*******************************************************************************************************
int olSDL::ClsPageManager::get_NextPageID(const int PredecessorPage_ID, e_PageIndex & PageIndex, ClsMoAccessInfo & Info) const {

	type_PageContainer::const_iterator iterPredecessor ;
	type_PageContainer::const_iterator iterNextPage ;

	int NextPageID ;

	iterPredecessor = find_if(m_Pages.begin(), m_Pages.end(), PageHasID(PredecessorPage_ID));

	if (iterPredecessor==m_Pages.end()) {
		Info.set_Info(ClsMoAccessInfo::page_get_predecessor_unknown) ;
		return -1 ;
	} ;

	iterNextPage = iterPredecessor ;
	iterNextPage++ ;

	if (iterNextPage==m_Pages.end()) {
		PageIndex = last ;
		Info.set_Info(ClsMoAccessInfo::page_get_OK) ;

		return -1 ;
	} else {
		NextPageID = iterNextPage->get_Model_ID() ;
		PageIndex = middle ;
		Info.set_Info(ClsMoAccessInfo::page_get_OK) ;

		return NextPageID ;
	} ;

} ;
//*******************************************************************************************************
int olSDL::ClsPageManager::get_PreviousPageID(const int AncestorPage_ID, e_PageIndex & PageIndex, ClsMoAccessInfo & Info) const {

	type_PageContainer::const_iterator iterAncestor ;
	type_PageContainer::const_iterator iterPreviousPage ;

	int PreviousPageID ;

	iterAncestor = find_if(m_Pages.begin(), m_Pages.end(), PageHasID(AncestorPage_ID));

	if (iterAncestor==m_Pages.end()) {
		Info.set_Info(ClsMoAccessInfo::page_get_ancestor_unknown) ;
		return -1 ;
	} ;

	if (iterAncestor==m_Pages.begin()) {
		PageIndex = first ;
		Info.set_Info(ClsMoAccessInfo::page_get_OK) ;

		return -1 ;
	} else {

		iterPreviousPage = iterAncestor ;
		iterPreviousPage-- ;

		PreviousPageID = iterPreviousPage->get_Model_ID() ;
		PageIndex = middle ;
		Info.set_Info(ClsMoAccessInfo::page_get_OK) ;

		return PreviousPageID ;
	} ;
} ;
//*******************************************************************************************************
void olSDL::ClsPageManager::get_IDsOfPages(list<int> & IDsOfPages) const {

	type_PageContainer::const_iterator iterPages ;

	IDsOfPages.clear() ;

 	for (iterPages=m_Pages.begin(); iterPages != m_Pages.end(); iterPages++)
	{

		IDsOfPages.push_back(iterPages->get_Model_ID()) ;
	} ;
} ;
//*******************************************************************************************************
void olSDL::ClsPageManager::add_NewPage_after(const int NewPage_ID, const int PredecessorPage_ID, ClsMoAccessInfo & Info) {

	type_PageContainer::iterator iterPredecessor ;

	iterPredecessor = find_if(m_Pages.begin(), m_Pages.end(), PageHasID(PredecessorPage_ID));

	if (iterPredecessor==m_Pages.end()) {
		Info.set_Info(ClsMoAccessInfo::page_insert_predecessor_unknown) ;
		return ;
	} ;

	// insert after predecessor
	iterPredecessor++ ;
	m_Pages.insert(iterPredecessor,ClsPage(NewPage_ID)) ;

	Info.set_Info(ClsMoAccessInfo::page_insert_OK) ;
	return ;

} ;
//*******************************************************************************************************
void olSDL::ClsPageManager::delete_Page(const int Page_ID, ClsMoAccessInfo & Info) {

	type_PageContainer::iterator iterPageContainer ;

	iterPageContainer = find_if(m_Pages.begin(), m_Pages.end(), PageHasID(Page_ID));

	if (iterPageContainer==m_Pages.end()) {
		Info.set_Info(ClsMoAccessInfo::page_get_page_unknown) ;
		return ;
	} ;

	if (m_Pages.size()<=1) {
		Info.set_Info(ClsMoAccessInfo::page_delete_one_page_needed) ;
		return ;
	} ;

	m_Pages.erase(iterPageContainer);

	Info.set_Info(ClsMoAccessInfo::page_delete_OK) ;
	return ;

} ;
//*******************************************************************************************************
bool olSDL::ClsPageManager::Page_exists(const int Page_ID) const {

	type_PageContainer::const_iterator iterPageContainer ;

	iterPageContainer = find_if(m_Pages.begin(), m_Pages.end(), PageHasID(Page_ID));
	return (iterPageContainer!=m_Pages.end()) ;

} ;
//*******************************************************************************************************
void olSDL::ClsPageManager::modify_PageWidth(const int Page_ID, const int PageWidth, ClsMoAccessInfo & Info) {

	type_PageContainer::iterator iterPageContainer ;

	iterPageContainer = find_if(m_Pages.begin(), m_Pages.end(), PageHasID(Page_ID));

	if (iterPageContainer==m_Pages.end()) {
		Info.set_Info(ClsMoAccessInfo::page_modify_unknown_page) ;
		return ;
	} ;

	iterPageContainer->modify_PageWidth(PageWidth, Info) ;
} ;
//*******************************************************************************************************
void olSDL::ClsPageManager::modify_PageHeight(const int Page_ID, const int PageHeight, ClsMoAccessInfo & Info) {

	type_PageContainer::iterator iterPageContainer ;

	iterPageContainer = find_if(m_Pages.begin(), m_Pages.end(), PageHasID(Page_ID));

	if (iterPageContainer==m_Pages.end()) {
		Info.set_Info(ClsMoAccessInfo::page_modify_unknown_page) ;
		return ;
	} ;

	iterPageContainer->modify_PageHeight(PageHeight, Info) ;
} ;
//*******************************************************************************************************
void olSDL::ClsPageManager::modify_PageFramePaS(const int Page_ID, const int x_left_up,
	const int y_left_up, const int FrameWidth, const int FrameHeight, ClsMoAccessInfo & Info) {

	type_PageContainer::iterator iterPageContainer ;

	iterPageContainer = find_if(m_Pages.begin(), m_Pages.end(), PageHasID(Page_ID));

	if (iterPageContainer==m_Pages.end()) {
		Info.set_Info(ClsMoAccessInfo::page_modify_unknown_page) ;
		return ;
	} ;

	iterPageContainer->modify_FramePaS(x_left_up, y_left_up, FrameWidth, FrameHeight, Info) ;
} ;
//*******************************************************************************************************

