
#include "Symbol.h"
#include "Helper.h"

#include "BlockDia.h"
#include "ProcessDia.h"
#include "ProcedureDia.h"

//*******************************************************************************************************
olSDL::Symbol::Symbol(const int Symbol_ID, const EnumSym::Type SymbolType,
				 const int x_pos, const int y_pos, const int width, const int height,
				 const string content, const int Page_ID, const int ReferencedDiagramID, const int ReferencedDiagramFirstPageID)
				 : ClsPageObject(Page_ID), ClsRecPaS(x_pos, y_pos, width, height),
				 ClsObjectWithModel_ID(Symbol_ID), m_SymbolType(SymbolType), m_content(content) {

	#ifdef olSDL_DL_ONE
	// DebugPrinter::print("CON:    olSDL::Symbol::Symbol()") ;
	#endif

	switch (SymbolType) {
		case EnumSym::BlkInstRefSym:
		{
			m_pReferencedDiagram = new ClsBlockDia(ReferencedDiagramID, content, ReferencedDiagramFirstPageID, this) ;
			break ;
		}
		case EnumSym::ProInstRefSym:
		{
			m_pReferencedDiagram = new ClsProcessDia(ReferencedDiagramID, content, ReferencedDiagramFirstPageID, this) ;
			break ;
		}
		case EnumSym::ProcedureInstRefSym:
		{
			m_pReferencedDiagram = new ClsProcedureDia(ReferencedDiagramID, content, ReferencedDiagramFirstPageID, this) ;
			break ;
		}
		case EnumSym::TextSym:
		case EnumSym::BlkTypeRefSym:
		case EnumSym::ProTypeRefSym:
		{
			m_pReferencedDiagram = NULL ;
			break ;
		}
		default:
		{
			#ifdef olSDL_DL_TWO
			DebugPrinter::print("error: olSDL::Symbol::Symbol()") ;
			#endif
			m_pReferencedDiagram = NULL ;
		}
	} ;
} ;
//*******************************************************************************************************
olSDL::Symbol::~Symbol() {

	#ifdef olSDL_DL_ONE
	//DebugPrinter::print("DEST:    olSDL::Symbol::~Symbol()") ;
	#endif

	switch (m_SymbolType) {
		case EnumSym::BlkInstRefSym:
		{
			delete ((ClsBlockDia *) m_pReferencedDiagram) ;
			m_pReferencedDiagram = NULL ;
			break ;
		}
		case EnumSym::ProInstRefSym:
		{
			delete ((ClsProcessDia *) m_pReferencedDiagram) ;
			m_pReferencedDiagram = NULL ;
			break ;
		}
		case EnumSym::ProcedureInstRefSym:
		{
			delete ((ClsProcedureDia *) m_pReferencedDiagram) ;
			m_pReferencedDiagram = NULL ;
			break ;
		}
		default:
		{
			#ifdef olSDL_DL_TWO
			if (m_pReferencedDiagram != NULL) {
				DebugPrinter::print("error: olSDL::Symbol::~Symbol()") ;
			} ;
			#endif
		}
	} ;
} ;
//*******************************************************************************************************
//*******************************************************************************************************
olSDL::EnumSym::Type olSDL::Symbol::get_SymbolType( ) const {
	return m_SymbolType ;
} ;
//*******************************************************************************************************
string olSDL::Symbol::get_content( ) const {
	return m_content ;
} ;

//*******************************************************************************************************
int olSDL::Symbol::get_referenced_Diagram_Model_ID() const {

	// just for sure
	if (is_referencing_Symbol() && (m_pReferencedDiagram != NULL)) {
		return (m_pReferencedDiagram->get_Model_ID()) ;
	} else {
		DebugPrinter::print("error: olSDL::Symbol::get_referenced_Diagram_Model_ID()") ;
		return -1 ;
	} ;

} ;
//*******************************************************************************************************
olSDL::Diagram * olSDL::Symbol::get_referenced_Diagram() const {

	// just for sure
	if (is_referencing_Symbol() && (m_pReferencedDiagram != NULL)) {
		return m_pReferencedDiagram ;
	} else {
		DebugPrinter::print("error: olSDL::Diagram * olSDL::Symbol::get_referenced_Diagram() const") ;
		return NULL ;
	} ;

} ;

//*******************************************************************************************************
//*******************************************************************************************************
bool olSDL::Symbol::operator==(const Symbol & s) const {
	return (get_Model_ID() == s.get_Model_ID()) ;
} ;
//*******************************************************************************************************
void olSDL::Symbol::modify_Content(const string Content) {

	m_content = Content ;

	// just for sure
	if (is_referencing_Symbol() && (m_pReferencedDiagram != NULL)) {

		m_pReferencedDiagram->modify_HeaderPart_Name(Content) ;

	} else if (is_referencing_Symbol() && (m_pReferencedDiagram == NULL)) {

		DebugPrinter::print("error: void olSDL::Symbol::modify_Content(const string Content)") ;

	} ;
} ;
//*******************************************************************************************************





