
#include "Helper.h"
#include "SysSpec.h"
#include "XML_Writer.h"
#include "XML_Reader.h"
#include "ProcedureDia.h"

//*******************************************************************************************************
olSDL::SysSpec::SysSpec() {

	#ifdef olSDL_DL_ONE
	//DebugPrinter::print("olSDL::SysSpec::SysSpec()") ;
	#endif

	// initialize map
	m_pMoEleMap = new ClsMoEleMap() ;

	// generate new Diagram_ID
	int Diagram_ID = m_pMoEleMap->get_ID( );
	int First_Page_ID = m_pMoEleMap->get_ID( );

	// create empty system diagram
	m_pRoot = new ClsSysDia(Diagram_ID, "ROOT", First_Page_ID) ;

	// add to Model Element Map
	m_pMoEleMap->add_MoEle(m_pRoot, m_pRoot) ;
} ;
//*******************************************************************************************************
olSDL::SysSpec::~SysSpec() {

	#ifdef olSDL_DL_ONE
	//DebugPrinter::print("DES: olSDL::SysSpec::~SysSpec()") ;
	#endif

	if (m_pRoot != NULL) {
		delete m_pRoot ;
	} ;

	if (m_pMoEleMap != NULL) {
		delete m_pMoEleMap ;
	} ;

} ;

//*******************************************************************************************************
//*******************************************************************************************************
int olSDL::SysSpec::get_FirstPageID(const int Diagram_Model_ID, ClsMoAccessInfo & Info) const {

	const Diagram * pDiagram;

	pDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return -1 ;
	} ;

	return (pDiagram->get_FirstPageID(Info)) ;
} ;
//*******************************************************************************************************
int olSDL::SysSpec::get_LastPageID(const int Diagram_Model_ID, ClsMoAccessInfo & Info) const {

	const Diagram * pDiagram;

	pDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return -1 ;
	} ;

	return (pDiagram->get_LastPageID(Info)) ;
} ;
//*******************************************************************************************************
int olSDL::SysSpec::get_NextPageID(const int Diagram_Model_ID, const int PredecessorPage_ID, ClsPageManager::e_PageIndex & PageIndex, ClsMoAccessInfo & Info) const {

	const Diagram * pDiagram;

	pDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return -1 ;
	} ;

	return (pDiagram->get_NextPageID(PredecessorPage_ID, PageIndex, Info)) ;
} ;
//*******************************************************************************************************
int olSDL::SysSpec::get_PreviousPageID(const int Diagram_Model_ID, const int AncestorPage_ID, ClsPageManager::e_PageIndex & PageIndex, ClsMoAccessInfo & Info) const {

	const Diagram * pDiagram;

	pDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return -1 ;
	} ;

	return (pDiagram->get_PreviousPageID(AncestorPage_ID, PageIndex, Info)) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::append_NewPage(const int Diagram_Model_ID, ClsMoAccessInfo & Info) {

	int PredecessorPage_ID ;

	PredecessorPage_ID = get_LastPageID(Diagram_Model_ID, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::page_get_OK) {
		return ;
	} ;

	add_NewPage_after(Diagram_Model_ID, PredecessorPage_ID, Info) ;

} ;
//*******************************************************************************************************
void olSDL::SysSpec::add_NewPage_after(const int Diagram_Model_ID, const int PredecessorPage_ID, ClsMoAccessInfo & Info) {

	Diagram * pDiagram;
	int NewPageID = m_pMoEleMap->get_ID( ) ;

	pDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	pDiagram->add_NewPage_after(NewPageID, PredecessorPage_ID, Info) ;

} ;
//*******************************************************************************************************
void olSDL::SysSpec::modify_ProcedureDiaFormalParameters(const int Diagram_Model_ID,
	const string & str_HP_FormalParameters, ClsMoAccessInfo & Info) {

	Diagram * pDiagram ;
	ClsProcedureDia * pProcedureDiagram ;

	pDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	pProcedureDiagram = ((ClsProcedureDia *) pDiagram) ;

	pProcedureDiagram->modify_FormalParameters(str_HP_FormalParameters);

	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::get_ProcedureDiaFormalParameters(const int Diagram_Model_ID,
		string & str_HP_FormalParameters, ClsMoAccessInfo & Info) const {

	const Diagram * pDiagram ;
	const ClsProcedureDia * pProcedureDiagram ;

	pDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	pProcedureDiagram = ((ClsProcedureDia *) pDiagram) ;
	str_HP_FormalParameters = pProcedureDiagram->get_FormalParameters() ;

	Info.set_Info(ClsMoAccessInfo::get_OK) ;
} ;
//*******************************************************************************************************
int olSDL::SysSpec::get_ReferencingDiagramID(const int referenced_Diagram_Model_ID, ClsMoAccessInfo & Info) const {

	const Diagram * pReferencedDiagram ;
	const Diagram * pReferencingDiagram ;
	const Symbol * pReferencingSymbol ;
	int ReferencingSymbol_ID ;

	pReferencedDiagram = m_pMoEleMap->find_MoEle(referenced_Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return -1 ;
	} ;

	if ( ! pReferencedDiagram->is_referenced_Diagram() ) {
		#ifdef olSDL_DL_TWO
		DebugPrinter::print("error in: int olSDL::SysSpec::get_DiagramID_from_referencing_Symbol") ;
		#endif
		return -1 ;
	} ;

	pReferencingSymbol = pReferencedDiagram->get_referencing_Symbol() ;
	ReferencingSymbol_ID = pReferencingSymbol->get_Model_ID() ;

	pReferencingDiagram = m_pMoEleMap->find_MoEle(ReferencingSymbol_ID, ClsMoEleMap::get_enclosing_diagram, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::get_enclosing_diagram_OK) {
		return -1 ;
	} ;

	Info.set_Info(ClsMoAccessInfo::get_OK) ;

	return (pReferencingDiagram->get_Model_ID()) ;
} ;
//*******************************************************************************************************
int olSDL::SysSpec::get_System_Model_ID() const {

	#ifdef olSDL_DL_ONE
	//DebugPrinter::print("olSDL::SysSpec::get_System_Model_ID()") ;
	#endif

	return (m_pRoot->get_Model_ID()) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::get_PageFrameInfo(const int Diagram_Model_ID, const int Page_ID, ClsRecPaS & PaS,
								string & HP_DiaType, string & HP_Name, string & HP_Index, ClsMoAccessInfo & Info) const  {

	const Diagram * pDiagram;

	// determine the enclosing diagram
	pDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	pDiagram->get_PageFrameInfo(Page_ID, PaS, HP_DiaType, HP_Name, HP_Index, Info) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::get_PageSize(const int Diagram_Model_ID, const int Page_ID, int & PageWidth, int & PageHeight, ClsMoAccessInfo & Info) const {

	Diagram * pDiagram;

	// determine the enclosing diagram
	pDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	pDiagram->get_PageSize(Page_ID, PageWidth, PageHeight, Info) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::print_Map( ) {

	m_pMoEleMap->print_Map( ) ;
} ;

//*******************************************************************************************************
// symbol
//*******************************************************************************************************
void olSDL::SysSpec::insert_Symbol(const EnumSym::Type SymbolType, const int Location_Model_ID, const int Page_ID,
				    		   const int x_left_up, const int y_left_up, const int width, const int height , const string content,
						   ClsMoAccessInfo & Info, int & New_Model_ID) {

	#ifdef olSDL_DL_ONE
	//DebugPrinter::print("olSDL::SysSpec::insert_Symbol(...") ;
	#endif

	Diagram * pReferencedDiagram;
	Diagram * pDiagram;

	pDiagram = m_pMoEleMap->find_MoEle(Location_Model_ID, ClsMoEleMap::insert_into_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::insert_OK) {
		return ;
	} ;

	// delegate insert
	const Symbol * pInsertedSymbol = NULL ;
	int Symbol_ID = m_pMoEleMap->get_ID( ) ;
	int ReferencedDiagramID = m_pMoEleMap->get_ID( ) ;
	int ReferencedDiagramFirstPageID = m_pMoEleMap->get_ID( ) ;

	pInsertedSymbol = pDiagram->insert_Symbol(SymbolType, Symbol_ID, Page_ID, x_left_up, y_left_up,
				              				width, height , content, ReferencedDiagramID, ReferencedDiagramFirstPageID, Info) ;

	// insert successful ?
	if (Info.get_Info() != ClsMoAccessInfo::insert_OK) {
		return ;
	} ;

	New_Model_ID = Symbol_ID ;

	// add to map
	m_pMoEleMap->add_MoEle(pInsertedSymbol, pDiagram) ;

	if (pInsertedSymbol->is_referencing_Symbol()) {
		pReferencedDiagram = pInsertedSymbol->get_referenced_Diagram() ;
		m_pMoEleMap->add_MoEle(pReferencedDiagram, pReferencedDiagram) ;
	} ;

	Info.set_Info(ClsMoAccessInfo::insert_OK) ;
} ;
//*******************************************************************************************************
const olSDL::Symbol * olSDL::SysSpec::get_RecSymbol(const int Symbol_ID, ClsMoAccessInfo & Info) const {

	Diagram * pDiagram;

	// determine the enclosing diagram
	pDiagram = m_pMoEleMap->find_MoEle(Symbol_ID, ClsMoEleMap::get_enclosing_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_enclosing_diagram_OK) {
		return NULL ;
	} ;

	return (pDiagram->get_Symbol(Symbol_ID, Info)) ;
} ;
//*******************************************************************************************************
void  olSDL::SysSpec::get_Syms(const EnumSym::Type SymbolType,
			      			const int Diagram_Model_ID, const int Page_ID,
			      			type_ListOf_Poi_to_ConstSym & ResultList,
						ClsMoAccessInfo & Info) const {

	Diagram * pDiagram;

	// determine diagram
	pDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	// delegate
	pDiagram->get_Syms(SymbolType, Page_ID, ResultList, Info) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::modify_SymbolContent(const int Symbol_ID, const string Content, ClsMoAccessInfo & Info) {

	Diagram * pEnclosingDiagram;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(Symbol_ID, ClsMoEleMap::get_enclosing_diagram_for_modifying_object, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	// delegate
	pEnclosingDiagram->modify_SymbolContent(Symbol_ID, Content, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::set_SymbolPaS(const int Symbol_ID,
						      const int x_left_up, const int y_left_up, const int width, const int height,
						      ClsMoAccessInfo & Info) {

	Diagram * pEnclosingDiagram;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(Symbol_ID, ClsMoEleMap::get_enclosing_diagram_for_modifying_object, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	// delegate
	pEnclosingDiagram->set_SymbolPaS(Symbol_ID, ClsRecPaS(x_left_up,y_left_up,width,height), Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::delete_Symbol(const int Symbol_ID, ClsMoAccessInfo & Info) {

	int ReferencedDiagramID ;
	bool is_Reference_to_Diagram ;
	const Symbol * pSymbol_to_delete ;

	Diagram * pEnclosingDiagram;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(Symbol_ID, ClsMoEleMap::get_enclosing_diagram_for_deleting_object, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::delete_OK) {
		return ;
	} ;

	// determine ID from referenced diagram if present
	pSymbol_to_delete = pEnclosingDiagram->get_Symbol(Symbol_ID, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		Info.set_Info(ClsMoAccessInfo::delete_object_unknown) ;
		return ;
	} ;

	is_Reference_to_Diagram = pSymbol_to_delete->is_referencing_Symbol() ;
	if (is_Reference_to_Diagram) {
		ReferencedDiagramID = pSymbol_to_delete->get_referenced_Diagram_Model_ID() ;
	} ;
	pSymbol_to_delete = NULL ;

	// delegate
	pEnclosingDiagram->delete_Symbol(Symbol_ID, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::delete_OK) {
		return ;
	} ;

	// delete from map
	m_pMoEleMap->delete_MoEle(Symbol_ID, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::delete_OK) {
		return ;
	} ;

	if (is_Reference_to_Diagram) {
		m_pMoEleMap->delete_MoEle(ReferencedDiagramID, Info) ;
		if (Info.get_Info() != ClsMoAccessInfo::delete_OK) {
			return ;
		} ;
	} ;

	Info.set_Info(ClsMoAccessInfo::delete_OK) ;
} ;

//*******************************************************************************************************
// channel
//*******************************************************************************************************
void olSDL::SysSpec::insert_Channel(const int Location_Model_ID, const int Page_ID, ClsChannel::e_DelayType DelayType,
							const ClsChanName ChanName, const ClsChanSourceSigList ChanSourceSigList, const ClsChanSourceCon ChanSourceCon,
						 	const ClsChanDestSigList ChanDestSigList, const ClsChanDestCon ChanDestCon,
							list<int> & ChanCoords,
							bool bSourceIsEnv, const int ChanSourceModel_ID,
							bool bDestIsEnv, const int ChanDestModel_ID,
							ClsMoAccessInfo & Info, int & New_Model_ID) {


	#ifdef olSDL_DL_ONE
	//DebugPrinter::print("olSDL::SysSpec::insert_Channel") ;
	#endif

	Diagram * pDiagram;

	pDiagram = m_pMoEleMap->find_MoEle(Location_Model_ID, ClsMoEleMap::insert_into_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::insert_OK) {
		return ;
	} ;

	// check, if channel source and destination exists

	if ( !bSourceIsEnv ) {
		const Symbol * pSourceSymbol ;
		pSourceSymbol = pDiagram->get_Symbol(ChanSourceModel_ID, Info) ;
		if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
			Info.set_Info(ClsMoAccessInfo::insert_chan_source_unknown)  ;
			return ;
		} ;
	} ;

	if ( !bDestIsEnv ) {
		const Symbol * pDestSymbol ;
		pDestSymbol = pDiagram->get_Symbol(ChanDestModel_ID, Info) ;
		if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
			Info.set_Info(ClsMoAccessInfo::insert_chan_dest_unknown)  ;
			return ;
		} ;
	} ;

	// delegate insert
	const ClsChannel * pInsertedChannel = NULL ;
	int Channel_ID = m_pMoEleMap->get_ID( );

	pInsertedChannel = pDiagram->insert_Channel(Channel_ID, Page_ID, DelayType,
						 ChanName, ChanSourceSigList, ChanSourceCon,
						 ChanDestSigList, ChanDestCon, ChanCoords,
						 bSourceIsEnv, ChanSourceModel_ID,
						 bDestIsEnv, ChanDestModel_ID, Info) ;

	// insert successful ?
	if (Info.get_Info() != ClsMoAccessInfo::insert_OK) {
		return ;
	} ;

	New_Model_ID = Channel_ID ;

	m_pMoEleMap->add_MoEle(pInsertedChannel, pDiagram) ;

	Info.set_Info(ClsMoAccessInfo::insert_OK) ;
} ;
//*******************************************************************************************************
void	olSDL::SysSpec::insert_Channel(const int Location_Model_ID, const int Page_ID, ClsChannel::e_DelayType DelayType, ClsChannel::e_DirectionType DirectionType,
					const ClsChanName ChanName, const ClsChanSourceSigList ChanSourceSigList, const ClsChanSourceCon ChanSourceCon,
				 	const ClsChanDestSigList ChanDestSigList, const ClsChanDestCon ChanDestCon,
					list<int> & ChanCoords,
					bool bSourceIsEnv, const int ChanSourceModel_ID,
					bool bDestIsEnv, const int ChanDestModel_ID,
					ClsMoAccessInfo & Info, int & New_Model_ID) {

	Diagram * pDiagram;

	pDiagram = m_pMoEleMap->find_MoEle(Location_Model_ID, ClsMoEleMap::insert_into_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::insert_OK) {
		return ;
	} ;

	// check, if channel source and destination exists

	if ( !bSourceIsEnv ) {
		const Symbol * pSourceSymbol ;
		pSourceSymbol = pDiagram->get_Symbol(ChanSourceModel_ID, Info) ;
		if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
			Info.set_Info(ClsMoAccessInfo::insert_chan_source_unknown)  ;
			return ;
		} ;
	} ;

	if ( !bDestIsEnv ) {
		const Symbol * pDestSymbol ;
		pDestSymbol = pDiagram->get_Symbol(ChanDestModel_ID, Info) ;
		if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
			Info.set_Info(ClsMoAccessInfo::insert_chan_dest_unknown)  ;
			return ;
		} ;
	} ;

	// delegate insert
	const ClsChannel * pInsertedChannel = NULL ;
	int Channel_ID = m_pMoEleMap->get_ID( );

	pInsertedChannel = pDiagram->insert_Channel(Channel_ID, Page_ID, DelayType, DirectionType,
			ChanName, ChanSourceSigList, ChanSourceCon, ChanDestSigList, ChanDestCon, ChanCoords,
			bSourceIsEnv, ChanSourceModel_ID, bDestIsEnv, ChanDestModel_ID, Info) ;


	// insert successful ?
	if (Info.get_Info() != ClsMoAccessInfo::insert_OK) {
		return ;
	} ;

	New_Model_ID = Channel_ID ;

	m_pMoEleMap->add_MoEle(pInsertedChannel, pDiagram) ;

	Info.set_Info(ClsMoAccessInfo::insert_OK) ;

} ;
//*******************************************************************************************************
void  olSDL::SysSpec::get_Channels(const int Diagram_Model_ID, const int Page_ID,
			      			type_ListOf_Poi_to_ConstChannel & ResultList,
						ClsMoAccessInfo & Info) const {

	Diagram * pDiagram;

	// determine diagram
	pDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	// delegate
	pDiagram->get_Channels(Page_ID, ResultList, Info) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::modify_Chan_DirectionType (const int Channel_ID, const ClsChannel::e_DirectionType DirectionType, ClsMoAccessInfo & Info) {

	Diagram * pEnclosingDiagram;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(Channel_ID, ClsMoEleMap::get_enclosing_diagram_for_modifying_object, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	// delegate
	pEnclosingDiagram->modify_Chan_DirectionType(Channel_ID, DirectionType, Info ) ;
	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::modify_Chan_DelayType (const int Channel_ID, const ClsChannel::e_DelayType DelayType, ClsMoAccessInfo & Info) {

	Diagram * pEnclosingDiagram;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(Channel_ID, ClsMoEleMap::get_enclosing_diagram_for_modifying_object, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	// delegate
	pEnclosingDiagram->modify_Chan_DelayType(Channel_ID, DelayType, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::modify_Chan_Coords(const int Channel_ID, list<int> & ChanCoords, ClsMoAccessInfo & Info) {

	Diagram * pEnclosingDiagram;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(Channel_ID, ClsMoEleMap::get_enclosing_diagram_for_modifying_object, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	// delegate
	pEnclosingDiagram->modify_Chan_Coords(Channel_ID, ChanCoords, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::modify_ChanCompName(const int Channel_ID, const string content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) {

	Diagram * pEnclosingDiagram;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(Channel_ID, ClsMoEleMap::get_enclosing_diagram_for_modifying_object, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	// delegate
	pEnclosingDiagram->modify_Comp_Name(Channel_ID, content, PaS, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::modify_ChanSourceSig(const int Channel_ID, const string content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) {

	Diagram * pEnclosingDiagram;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(Channel_ID, ClsMoEleMap::get_enclosing_diagram_for_modifying_object, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	// delegate
	pEnclosingDiagram->modify_Comp_SourceSigList(Channel_ID, content, PaS, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::modify_ChanSourceCon(const int Channel_ID, const string content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) {

	Diagram * pEnclosingDiagram;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(Channel_ID, ClsMoEleMap::get_enclosing_diagram_for_modifying_object, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	// delegate
	pEnclosingDiagram->modify_Comp_SourceCon(Channel_ID, content, PaS, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::modify_ChanDestSig(const int Channel_ID, const string content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) {

	Diagram * pEnclosingDiagram;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(Channel_ID, ClsMoEleMap::get_enclosing_diagram_for_modifying_object, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	// delegate
	pEnclosingDiagram->modify_Comp_DestSigList(Channel_ID, content, PaS, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::modify_ChanDestCon(const int Channel_ID, const string content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) {

	Diagram * pEnclosingDiagram;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(Channel_ID, ClsMoEleMap::get_enclosing_diagram_for_modifying_object, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	// delegate
	pEnclosingDiagram->modify_Comp_DestCon(Channel_ID, content, PaS, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::delete_Channel(const int Channel_ID, ClsMoAccessInfo & Info) {

	Diagram * pEnclosingDiagram;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(Channel_ID, ClsMoEleMap::get_enclosing_diagram_for_deleting_object, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::delete_OK) {
		return ;
	} ;

	// delegate
	pEnclosingDiagram->delete_Channel(Channel_ID, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::delete_OK) {
		return ;
	} ;

	// delete in map
	m_pMoEleMap->delete_MoEle(Channel_ID, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::delete_OK) {
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::delete_OK) ;
} ;
//*******************************************************************************************************
// state machine symbol
//*******************************************************************************************************
void olSDL::SysSpec::modify_StateMachineSymPaS(const int StateMachineSymbol_ID,
		const int x_left_up, const int y_left_up, const int width, const int height, ClsMoAccessInfo & Info) {

	Diagram * pEnclosingDiagram ;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(StateMachineSymbol_ID, ClsMoEleMap::get_enclosing_diagram_for_modifying_object, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	// delegate

	pEnclosingDiagram->modify_StateMachineSymPaS(StateMachineSymbol_ID, ClsRecPaS(x_left_up,y_left_up,width,height), Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::modify_StateMachineSymContent(const int StateMachineSymbol_ID, const string Content, ClsMoAccessInfo & Info) {

	Diagram * pEnclosingDiagram;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(StateMachineSymbol_ID, ClsMoEleMap::get_enclosing_diagram_for_modifying_object, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	// delegate

	pEnclosingDiagram->modify_StateMachineSymContent(StateMachineSymbol_ID, Content, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::insert_StateMachineSymbol(const ClsStateMachineSymbol::e_StateMachineSymbolType SymbolType,
		const int Location_Model_ID, const int Page_ID, const int x_left_up, const int y_left_up, const int width, const int height , const string content,
		ClsMoAccessInfo & Info, int & New_Model_ID) {

	#ifdef olSDL_DL_ONE
	//DebugPrinter::print("void olSDL::SysSpec::insert_StateMachineSymbol(...") ;
	#endif

	Diagram * pDiagram ;
	const ClsStateMachineSymbol * pInsertedStateMachineSymbol ;

	pDiagram = m_pMoEleMap->find_MoEle(Location_Model_ID, ClsMoEleMap::insert_into_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::insert_OK) {
		return ;
	} ;

	// delegate insert
	int Symbol_ID = m_pMoEleMap->get_ID( ) ;

	pInsertedStateMachineSymbol = pDiagram->insert_StateMachineSymbol(SymbolType, Symbol_ID,
				Page_ID, x_left_up, y_left_up, width, height , content, Info) ;

	// insert successful ?
	if (Info.get_Info() != ClsMoAccessInfo::insert_OK) {
		return ;
	} ;

	New_Model_ID = Symbol_ID ;

	m_pMoEleMap->add_MoEle(pInsertedStateMachineSymbol, pDiagram) ;

	Info.set_Info(ClsMoAccessInfo::insert_OK) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::delete_StateMachineSymbol(const int StateMachineSymbol_Model_ID, ClsMoAccessInfo & Info) {

	Diagram * pEnclosingDiagram;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(StateMachineSymbol_Model_ID, ClsMoEleMap::get_enclosing_diagram_for_deleting_object, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::delete_OK) {
		return ;
	} ;

	// delegate
	pEnclosingDiagram->delete_StateMachineSymbol(StateMachineSymbol_Model_ID, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::delete_OK) {
		return ;
	} ;

	// delete from map
	m_pMoEleMap->delete_MoEle(StateMachineSymbol_Model_ID, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::delete_OK) {
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::delete_OK) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::get_InComingFlowLinesFromStateMaSym(const int Symbol_ID,
										type_ListOfPoi_to_ConstFlowLine & ResultList,
										ClsMoAccessInfo & Info) const {


	Diagram * pDiagram;

	const olSDL::ClsStateMachineSymbol * pStateMaSym ;
	int StateMaSymParentPageID ;

	type_ListOfPoi_to_ConstFlowLine temp_ResultList ;
	type_ListOfPoi_to_ConstFlowLine::const_iterator iter ;

	ResultList.clear() ;

	// ==================
	// determine the enclosing diagram
	pDiagram = m_pMoEleMap->find_MoEle(Symbol_ID, ClsMoEleMap::get_enclosing_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_enclosing_diagram_OK) {
		return ;
	} ;

	// determine the enclosing diagram
	// ==================

	// ==================
	// get StateMaSym

	pStateMaSym = pDiagram->get_StateMachineSymbol(Symbol_ID, Info);

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	StateMaSymParentPageID = pStateMaSym->get_Page_ID() ;

	// get StateMaSym
	// ==================

	// ==================
	// get all flow lines from page

	pDiagram->get_FlowLines(StateMaSymParentPageID, temp_ResultList, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	// get all flow lines from page
	// ==================

	// ==================
	// get all incoming flow lines

	for (iter = temp_ResultList.begin() ; iter != temp_ResultList.end() ; iter++) {
		if ((*iter)->get_DestID_as_Int() == Symbol_ID) {

			ResultList.push_back((*iter)) ;
		} ;
	} ;

	// get all incoming flow lines
	// ==================

	Info.set_Info(ClsMoAccessInfo::get_OK) ;

} ;
//*******************************************************************************************************
void olSDL::SysSpec::get_OutgoingFlowLinesFromStateMaSym(const int Symbol_ID,
							type_ListOfPoi_to_ConstFlowLine & ResultList,
							ClsMoAccessInfo & Info) const {

	Diagram * pDiagram;

	const olSDL::ClsStateMachineSymbol * pStateMaSym ;
	int StateMaSymParentPageID ;

	type_ListOfPoi_to_ConstFlowLine temp_ResultList ;
	type_ListOfPoi_to_ConstFlowLine::const_iterator iter ;

	ResultList.clear() ;

	// ==================
	// determine the enclosing diagram
	pDiagram = m_pMoEleMap->find_MoEle(Symbol_ID, ClsMoEleMap::get_enclosing_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_enclosing_diagram_OK) {
		return ;
	} ;

	// determine the enclosing diagram
	// ==================

	// ==================
	// get StateMaSym

	pStateMaSym = pDiagram->get_StateMachineSymbol(Symbol_ID, Info);

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	StateMaSymParentPageID = pStateMaSym->get_Page_ID() ;

	// get StateMaSym
	// ==================

	// ==================
	// get all flow lines from page

	pDiagram->get_FlowLines(StateMaSymParentPageID, temp_ResultList, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	// get all flow lines from page
	// ==================

	// ==================
	// get all outgoing  flow lines

	for (iter = temp_ResultList.begin() ; iter != temp_ResultList.end() ; iter++) {
		if ((*iter)->get_SourceID_as_Int() == Symbol_ID) {

			ResultList.push_back((*iter)) ;
		} ;
	} ;

	// get all outgoing  flow lines
	// ==================

	Info.set_Info(ClsMoAccessInfo::get_OK) ;

} ;
//*******************************************************************************************************
int olSDL::SysSpec::get_EnclosingDiagramID(const int Symbol_ID, ClsMoAccessInfo & Info) const {

	Diagram * pDiagram;

	// determine the enclosing diagram
	pDiagram = m_pMoEleMap->find_MoEle(Symbol_ID, ClsMoEleMap::get_enclosing_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_enclosing_diagram_OK) {

		return -1 ;
	} else {

		Info.set_Info(ClsMoAccessInfo::get_OK) ;
		return (pDiagram->get_Model_ID()) ;
	} ;

} ;
//*******************************************************************************************************
const olSDL::ClsStateMachineSymbol * olSDL::SysSpec::get_StateMachineSymbol(const int Symbol_ID, ClsMoAccessInfo & Info) const {

	Diagram * pDiagram;

	// determine the enclosing diagram
	pDiagram = m_pMoEleMap->find_MoEle(Symbol_ID, ClsMoEleMap::get_enclosing_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_enclosing_diagram_OK) {
		return NULL ;
	} ;

	return (pDiagram->get_StateMachineSymbol(Symbol_ID, Info)) ;
	
} ;
//*******************************************************************************************************
void olSDL::SysSpec::get_StartingStateMachineSymsFromDiagram(const ClsStateMachineSymbol::e_StateMachineSymbolType StateMachineSymbolType,
		const int Diagram_Model_ID, type_ListOfPoi_to_ConstStateMachineSymbol & ResultList, ClsMoAccessInfo & Info) const {

	type_ListOfPoi_to_ConstStateMachineSymbol temp_ResultList ;
	type_ListOfPoi_to_ConstStateMachineSymbol::iterator iter_temp_ResultList ;

	type_ListOfPoi_to_ConstFlowLine IncomingFlowLinesResultList ;


	get_StateMachineSymsFromDiagram(StateMachineSymbolType, Diagram_Model_ID, temp_ResultList, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	
	ResultList.clear() ;

	for (iter_temp_ResultList = temp_ResultList.begin(); iter_temp_ResultList != temp_ResultList.end(); iter_temp_ResultList++) {

		IncomingFlowLinesResultList.clear() ;

		get_InComingFlowLinesFromStateMaSym( (*iter_temp_ResultList)->get_Model_ID(), IncomingFlowLinesResultList, Info) ;

		if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
			return ;
		} ;

		if (IncomingFlowLinesResultList.size() == 0) {

			ResultList.push_back(*iter_temp_ResultList) ;
		} ;
	} ;

	Info.set_Info(ClsMoAccessInfo::get_OK) ;

} ;
//*******************************************************************************************************
void olSDL::SysSpec::get_StateMachineSymsFromDiagram(const ClsStateMachineSymbol::e_StateMachineSymbolType StateMachineSymbolType,
		const int Diagram_Model_ID, type_ListOfPoi_to_ConstStateMachineSymbol & ResultList, ClsMoAccessInfo & Info) const {

	Diagram * pEnclosingDiagram;

	list<int> IDsOfPages ;
	list<int>::iterator iterIDsOfPages ;

	type_ListOfPoi_to_ConstStateMachineSymbol temp_ResultList ;

	ResultList.clear() ;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;


	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	pEnclosingDiagram->get_IDsOfPages(IDsOfPages) ;

	for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)
	{
		temp_ResultList.clear() ;

		pEnclosingDiagram->get_StateMachineSyms(StateMachineSymbolType, (*iterIDsOfPages), temp_ResultList, Info) ;

		if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
			ResultList.clear() ;
			return ;
		} ;

		ResultList.insert(ResultList.end(), temp_ResultList.begin(), temp_ResultList.end()) ;
	} ;

	Info.set_Info(ClsMoAccessInfo::get_OK) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::get_StateMachineSyms(const ClsStateMachineSymbol::e_StateMachineSymbolType StateMachineSymbolType,
			const int Diagram_Model_ID, const int Page_ID,
			type_ListOfPoi_to_ConstStateMachineSymbol & ResultList, ClsMoAccessInfo & Info) const {

	Diagram * pEnclosingDiagram;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	// delegate
	pEnclosingDiagram->get_StateMachineSyms(StateMachineSymbolType, Page_ID, ResultList, Info) ;
} ;
//*******************************************************************************************************

//*******************************************************************************************************
// flow line
//*******************************************************************************************************
void olSDL::SysSpec::modify_DecisionFlowLinePaS(const int DecisionFlowLine_ID,
				const int x_left_up, const int y_left_up, const int width, const int height, ClsMoAccessInfo & Info) {

	Diagram * pEnclosingDiagram ;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(DecisionFlowLine_ID, ClsMoEleMap::get_enclosing_diagram_for_modifying_object, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	// delegate
	pEnclosingDiagram->modify_DecisionFlowLinePaS(DecisionFlowLine_ID, ClsRecPaS(x_left_up, y_left_up, width, height), Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_OK) ;

} ;
//*******************************************************************************************************
void olSDL::SysSpec::modify_DecisionFlowLineContent(const int DecisionFlowLine_ID, const string Content, ClsMoAccessInfo & Info) {

	Diagram * pEnclosingDiagram ;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(DecisionFlowLine_ID, ClsMoEleMap::get_enclosing_diagram_for_modifying_object, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	// delegate
	pEnclosingDiagram->modify_DecisionFlowLineContent(DecisionFlowLine_ID, Content, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_OK) ;

} ;
//*******************************************************************************************************
void olSDL::SysSpec::modify_FlowLineCoords(const int FlowLineID, list<int> & FlowLineCoords, ClsMoAccessInfo & Info) {

	Diagram * pEnclosingDiagram ;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(FlowLineID, ClsMoEleMap::get_enclosing_diagram_for_modifying_object, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	// delegate
	pEnclosingDiagram->modify_FlowLineCoords(FlowLineID, FlowLineCoords, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::get_FlowLines(const int Diagram_Model_ID, const int Page_ID, type_ListOfPoi_to_ConstFlowLine & ResultList,
					ClsMoAccessInfo & Info) const {

	Diagram * pEnclosingDiagram ;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	// delegate
	pEnclosingDiagram->get_FlowLines(Page_ID, ResultList, Info) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::insert_DecisionFlowLine(const int Location_Model_ID, const int Page_ID,
			list<int> & FlowLineCoords, const int FlowLineSourceModel_ID, const int FlowLineDestModel_ID,
			const string & content, const ClsRecPaS & PaS,
			ClsMoAccessInfo & Info, int & New_Model_ID) {

	Diagram * pDiagram;

	pDiagram = m_pMoEleMap->find_MoEle(Location_Model_ID, ClsMoEleMap::insert_into_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::insert_OK) {
		return ;
	} ;

	// check, if flow line source and destination exists

	pDiagram->get_StateMachineSymbol(FlowLineSourceModel_ID, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		Info.set_Info(ClsMoAccessInfo::insert_flowline_source_unknown) ;
		return ;
	} ;

	pDiagram->get_StateMachineSymbol(FlowLineDestModel_ID, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		Info.set_Info(ClsMoAccessInfo::insert_flowline_dest_unknown) ;
		return ;
	} ;

	// delegate insert
	const ClsFlowLine * pInsertedFlowLine = NULL ;
	int FlowLineID = m_pMoEleMap->get_ID( );

	pInsertedFlowLine = pDiagram->insert_DecisionFlowLine(FlowLineID, Page_ID, FlowLineCoords,
		FlowLineSourceModel_ID, FlowLineDestModel_ID, content, PaS, Info) ;


	// insert successful ?
	if (Info.get_Info() != ClsMoAccessInfo::insert_OK) {
		return ;
	} ;

	New_Model_ID = FlowLineID ;

	m_pMoEleMap->add_MoEle(pInsertedFlowLine, pDiagram) ;

	Info.set_Info(ClsMoAccessInfo::insert_OK) ;

} ;
//*******************************************************************************************************
void olSDL::SysSpec::insert_FlowLine(const int Location_Model_ID, const int Page_ID,
					list<int> & FlowLineCoords, const int FlowLineSourceModel_ID, const int FlowLineDestModel_ID,
					ClsMoAccessInfo & Info, int & New_Model_ID) {

	Diagram * pDiagram;

	pDiagram = m_pMoEleMap->find_MoEle(Location_Model_ID, ClsMoEleMap::insert_into_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::insert_OK) {
		return ;
	} ;

	// check, if flow line source and destination exists

	pDiagram->get_StateMachineSymbol(FlowLineSourceModel_ID, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		Info.set_Info(ClsMoAccessInfo::insert_flowline_source_unknown) ;
		return ;
	} ;

	pDiagram->get_StateMachineSymbol(FlowLineDestModel_ID, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		Info.set_Info(ClsMoAccessInfo::insert_flowline_dest_unknown) ;
		return ;
	} ;

	// delegate insert
	const ClsFlowLine * pInsertedFlowLine = NULL ;
	int FlowLineID = m_pMoEleMap->get_ID( );

	pInsertedFlowLine = pDiagram->insert_FlowLine(FlowLineID, Page_ID, FlowLineCoords,
				    	         FlowLineSourceModel_ID, FlowLineDestModel_ID, Info) ;

	// insert successful ?
	if (Info.get_Info() != ClsMoAccessInfo::insert_OK) {
		return ;
	} ;

	New_Model_ID = FlowLineID ;

	m_pMoEleMap->add_MoEle(pInsertedFlowLine, pDiagram) ;

	Info.set_Info(ClsMoAccessInfo::insert_OK) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::delete_Page(const int Diagram_Model_ID, const int Page_ID, ClsMoAccessInfo & Info) {

	Diagram * pDiagram ;

	pDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	switch (pDiagram->get_DiagramType()) {
		case EnumDia::SysDia:
		case EnumDia::BlkDia:
		{
			// ClsHL_InstDiagram * pHL_InstDiagram = ((ClsHL_InstDiagram *) pDiagram) ;
			pDiagram->delete_Page(Page_ID, Info) ;
			return ;
			break ;
		}
		case EnumDia::ProDia:
		{
			ClsProcessDia * pProcessDiagram = ((ClsProcessDia *) pDiagram) ;
			pProcessDiagram->delete_Page(Page_ID, Info) ;
			return ;
			break ;
		}
		case EnumDia::ProcedureDia:
		{
			ClsProcedureDia * pProcedureDiagram = ((ClsProcedureDia *) pDiagram) ;
			pProcedureDiagram->delete_Page(Page_ID, Info) ;
			return ;
			break ;
		}
		default :
		{
			Info.set_Info(ClsMoAccessInfo::delete_unconsidered_diagram_type) ;
			return ;
		} ;
	} ; // switch (pDiagram->get_DiagramType())

} ;
//*******************************************************************************************************
void olSDL::SysSpec::delete_FlowLine(const int FlowLineID, ClsMoAccessInfo & Info) {

	Diagram * pEnclosingDiagram;

	// determine diagram
	pEnclosingDiagram = m_pMoEleMap->find_MoEle(FlowLineID, ClsMoEleMap::get_enclosing_diagram_for_deleting_object, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::delete_OK) {
		return ;
	} ;

	// delegate
	pEnclosingDiagram->delete_FlowLine(FlowLineID, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::delete_OK) {
		return ;
	} ;

	// delete in map
	m_pMoEleMap->delete_MoEle(FlowLineID, Info) ;
	if (Info.get_Info() != ClsMoAccessInfo::delete_OK) {
		return ;
	} ;

	Info.set_Info(ClsMoAccessInfo::delete_OK) ;
} ;
//*******************************************************************************************************
// diagram
//*******************************************************************************************************
void olSDL::SysSpec::modify_SysDiaName(const string str_HeaderPart_Name, ClsMoAccessInfo & Info) {

	m_pRoot->modify_HeaderPart_Name(str_HeaderPart_Name) ;

	Info.set_Info(ClsMoAccessInfo::modify_OK) ;
} ;
//*******************************************************************************************************
const olSDL::ClsSysDia * olSDL::SysSpec::get_SysDia() const {

	return m_pRoot ;
} ;
//*******************************************************************************************************
const olSDL::Diagram * olSDL::SysSpec::get_Diagram(const int Diagram_Model_ID, ClsMoAccessInfo & Info) const {

	const Diagram * pDiagram ;

	pDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return NULL ;
	} ;

	return pDiagram ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::get_AllReferencedDiagrams(const int Diagram_Model_ID, type_ListOf_Poi_to_ConstDiagram & ResultList,
									ClsMoAccessInfo & Info) const {

	Diagram * pDiagram;
	EnumDia::Type DiagramType ;

	pDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	DiagramType = pDiagram->get_DiagramType() ;

	switch (DiagramType) {
		case EnumDia::SysDia:
		case EnumDia::BlkDia:
		{
			// ClsHL_InstDiagram * pHL_InstDiagram = ((ClsHL_InstDiagram *) pDiagram) ;
			pDiagram->get_AllReferencedDiagrams(ResultList, Info) ;
			return ;
			break ;
		}
		case EnumDia::ProDia:
		{
			ClsProcessDia * pProcessDiagram = ((ClsProcessDia *) pDiagram) ;
			pProcessDiagram->get_AllReferencedDiagrams(ResultList, Info) ;
			return ;
			break ;
		}
		case EnumDia::ProcedureDia:
		{
			ClsProcedureDia * pProcedureDiagram = ((ClsProcedureDia *) pDiagram) ;
			pProcedureDiagram->get_AllReferencedDiagrams(ResultList, Info) ;
			return ;
			break ;
		}
		default :
		{
			Info.set_Info(ClsMoAccessInfo::get_unconsidered_diagram_type) ;
			return ;
		} ;

	} ; // switch (DiagramType)
} ;

//*******************************************************************************************************
// pages
//*******************************************************************************************************
void olSDL::SysSpec::modify_PageWidth(const int Diagram_Model_ID, const int Page_ID, const int PageWidth, ClsMoAccessInfo & Info) {

	Diagram * pDiagram;

	pDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	pDiagram->modify_PageWidth(Page_ID, PageWidth, Info) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::modify_PageHeight(const int Diagram_Model_ID, const int Page_ID, const int PageHeight, ClsMoAccessInfo & Info) {

	Diagram * pDiagram;

	pDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	pDiagram->modify_PageHeight(Page_ID, PageHeight, Info) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::modify_PageFramePaS(const int Diagram_Model_ID, const int Page_ID, const int x_left_up, const int y_left_up,
		const int FrameWidth, const int FrameHeight, ClsMoAccessInfo & Info) {

	Diagram * pDiagram;

	pDiagram = m_pMoEleMap->find_MoEle(Diagram_Model_ID, ClsMoEleMap::get_diagram, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		return ;
	} ;

	pDiagram->modify_PageFramePaS(Page_ID, x_left_up, y_left_up, FrameWidth, FrameHeight, Info) ;
} ;
//*******************************************************************************************************
//*******************************************************************************************************
bool olSDL::SysSpec::store(const string & FileName) {

	ClsXML_Writer XML_Writer ;

	return XML_Writer.write_SysSpec(get_SysDia(), FileName) ;
} ;
//*******************************************************************************************************
void olSDL::SysSpec::load(const string & FileName) {

	ClsXML_Reader XML_Reader ;

	reset() ;

	try {

		XML_Reader.read_SysSpec(FileName, this) ;

	}
	catch ( ... ) {

		reset() ;

		throw ;
	} ;

} ;
//*******************************************************************************************************
//*******************************************************************************************************
void olSDL::SysSpec::reset() {

	if (m_pRoot != NULL) {
		delete m_pRoot ;
		m_pRoot = NULL ;
	} ;

	if (m_pMoEleMap != NULL) {
		delete m_pMoEleMap ;
		m_pMoEleMap = NULL ;
	} ;

	// initialize map
	m_pMoEleMap = new ClsMoEleMap() ;

	// generate new Diagram_ID
	int Diagram_ID = m_pMoEleMap->get_ID( );
	int First_Page_ID = m_pMoEleMap->get_ID( );

	// create empty system diagram
	m_pRoot = new ClsSysDia(Diagram_ID, "ROOT", First_Page_ID) ;

	// add to Model Element Map
	m_pMoEleMap->add_MoEle(m_pRoot, m_pRoot) ;

} ;
//*******************************************************************************************************

