#include "TclModelAccess.h"
#include <stdio.h>
#include <vector>
#include "Helper.h"
#include "AppVars.h"
#include "Converter.h"

#include "XML_Reader.h"
#include "SDL_PR_Writer.h"

extern olSDL::ClassAppVars AppVars ;

//******************************************************************************
//******************************************************************************
//**** INTERFACE
//******************************************************************************
//******************************************************************************
int olSDL::TclExt::TclModelAccess::create_EmptySysSpec (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	*/

	SysSpec * pNewSystemSpec;
	SysSpec * pOldSystemSpec;

	if (argc !=1) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR ;
	}

	// create empty SysSpec
	pNewSystemSpec = new SysSpec();

	// delete old
	pOldSystemSpec = AppVars.get_SysSpec();
	delete pOldSystemSpec;

	// set new
	AppVars.set_SysSpec(pNewSystemSpec);
	pNewSystemSpec = NULL;

	return TCL_OK;
} ;
//******************************************************************************
int olSDL::TclExt::TclModelAccess::load_Model_from_File (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: File-Name
	*/

	int CompletionCode;
	string script;

	string str_FileName;
	SysSpec * pNewSystemSpec;
	SysSpec * pOldSystemSpec;

	string ErrorMessage;

	if (argc !=2) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	// convert and analyse args
	str_FileName = argv[1];

	//  clear MAR in Tcl interp !!!!
	CompletionCode = clear_MAR_ARR(interp);
	if (CompletionCode!=TCL_OK)
	  return TCL_ERROR;

	try
	{
		pNewSystemSpec = new SysSpec();

		pNewSystemSpec->load(str_FileName);

		// delete old
		pOldSystemSpec = AppVars.get_SysSpec();
		delete pOldSystemSpec;

		// set new
		AppVars.set_SysSpec(pNewSystemSpec);
		pNewSystemSpec = NULL;
	}
    	catch (const ClsXML_Reader_ErrorReporter& e)
    	{
		delete pNewSystemSpec;

		// write to Tcl interp
		
		if (!Tcl_SetVar2(interp, pMAR_Array_Name, "LoadingState", "Error", 0))
		  return TCL_ERROR;

		if (!Tcl_SetVar2(interp, pMAR_Array_Name, "ErrorType", "ReaderError", 0))
		  return TCL_ERROR;

		CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "ErrorMessage", "{" + e.get_ErrorMessage() + "}");
		if (CompletionCode != TCL_OK)
		  return TCL_ERROR;

		return TCL_OK;
	}
	catch ( ... ) {
		delete pNewSystemSpec;

		// write to Tcl interp
		
		if (!Tcl_SetVar2(interp, pMAR_Array_Name, "LoadingState", "Error", 0))
		  return TCL_ERROR;

		
		if (!Tcl_SetVar2(interp, pMAR_Array_Name, "ErrorType", "unconsidered", 0))
		  return TCL_ERROR;

		return TCL_OK;
	}

	// write to Tcl interp
	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "LoadingState", "OK", 0))
	  return TCL_ERROR;

	return TCL_OK;
}
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::save_SDL_PR (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: File-Name
	2: Inline: true, false
	*/

	int CompletionCode ;
	string script ;

	string str_FileName ;
	string str_Inline ;
	SysSpec * pSystemSpec ;
	ClsSDL_PR_Writer SDL_PR_Writer ;

	if (argc !=3) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR ;
	}

	//  clear MAR in Tcl interp !!!!
	CompletionCode = clear_MAR_ARR(interp) ;
	if (CompletionCode!=TCL_OK) {return TCL_ERROR; } ;

	// convert and analyse args
	str_FileName = argv[1] ;
	str_Inline = argv[2] ;

	pSystemSpec = AppVars.get_SysSpec() ;

	try
	{
		if (str_Inline.compare("true") == 0) {
			SDL_PR_Writer.write_SysSpec(pSystemSpec, str_FileName, true) ;
		} else {
			SDL_PR_Writer.write_SysSpec(pSystemSpec, str_FileName, false) ;
		} ;
	}
	catch (const ClsSDL_PR_Writer_StateMaSymError & e) {
		// write to Tcl interp
		if (!Tcl_SetVar2(interp, pMAR_Array_Name, "GeneratingPR_State", "Error", 0))
		  return TCL_ERROR;

		if ( e.get_StateMaErrorType() == ClsSDL_PR_Writer_StateMaSymError::sms_transition_not_complete) {
			if (!Tcl_SetVar2(interp, pMAR_Array_Name, "ErrorType", "sms_transition_not_complete", 0))
			  return TCL_ERROR;
		} else if ( e.get_StateMaErrorType() == ClsSDL_PR_Writer_StateMaSymError::sms_illegal_start_state) {
			if (!Tcl_SetVar2(interp, pMAR_Array_Name, "ErrorType", "sms_illegal_start_state", 0))
			  return TCL_ERROR;
		} else {
			if (!Tcl_SetVar2(interp, pMAR_Array_Name, "ErrorType", "sms_none", 0))
			  return TCL_ERROR;
		} ;

		CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "Symbol_ID", e.get_Symbol_ID());
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "Page_ID", e.get_PageID());
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "Diagram_ID", e.get_DiagramID());
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		return TCL_OK ;
	}
	catch (const ClsSDL_PR_Writer_ErrorReporter& e) {
		// write to Tcl interp
		if (!Tcl_SetVar2(interp, pMAR_Array_Name, "GeneratingPR_State", "Error", 0))
		  return TCL_ERROR;

		if (!Tcl_SetVar2(interp, pMAR_Array_Name, "ErrorType", "not_specified", 0))
		  return TCL_ERROR;

		return TCL_OK ;
	}
	catch ( ... ) {
		// write to Tcl interp
		if (!Tcl_SetVar2(interp, pMAR_Array_Name, "GeneratingPR_State", "Error", 0))
		  return TCL_ERROR;

		if (!Tcl_SetVar2(interp, pMAR_Array_Name, "ErrorType", "not_specified", 0))
		  return TCL_ERROR;

		return TCL_OK ;
	}

	// write to Tcl interp
	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "GeneratingPR_State", "OK", 0))
	  return TCL_ERROR;

	return TCL_OK ;

} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::store_Model_in_File (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: File-Name
	*/

	string script ;

	string str_FileName ;
	SysSpec * pSystemSpec ;

	if (argc !=2) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	// convert and analyse args
	str_FileName = argv[1] ;

	pSystemSpec = AppVars.get_SysSpec() ;

	try
	{
		if ( !(pSystemSpec->store(str_FileName))) {

			// write to Tcl interp
			if (!Tcl_SetVar2(interp, pMAR_Array_Name, "StoringState", "Error", 0))
			  return TCL_ERROR;

			return TCL_OK ;
		} ;
	}
	catch ( ... ) {

		// write to Tcl interp
		if (!Tcl_SetVar2(interp, pMAR_Array_Name, "StoringState", "Error", 0))
		  return TCL_ERROR;

		return TCL_OK ;
	}

	// write to Tcl interp
	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "StoringState", "OK", 0))
	  return TCL_ERROR;

	return TCL_OK ;
} ;
//*******************************************************************************************************
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::modify_SysDiaName (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: SysDiaName
	*/

	string SysDiaName ;

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;

	if (argc !=2) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	// convert and analyse args
	SysDiaName = argv[1] ;

	// delegate
	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->modify_SysDiaName(SysDiaName, AccessInfo) ;


	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::modify_OK) ) {
		return TCL_ERROR ;
	} ;

	return TCL_OK ;

} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::get_System_Model_ID (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	*/

	SysSpec * pSystemSpec ;
	int System_Model_ID ;

	int CompletionCode ;
	string script ;

	if (argc !=1) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	//  clear MAR in Tcl interp !!!!
	CompletionCode = clear_MAR_ARR(interp) ;
	if (CompletionCode!=TCL_OK) {return TCL_ERROR; } ;

	pSystemSpec = AppVars.get_SysSpec() ;

	System_Model_ID = pSystemSpec->get_System_Model_ID() ;

	// write to Tcl interp
	
	if (!Tcl_SetVar2(interp,pMAR_Array_Name, "Result", pMAR_OK, 0))
	  return TCL_ERROR;

	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "System_Model_ID", System_Model_ID);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::modify_RecSym_Content (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: Symbol_ID
	2: Content
	*/

	int Symbol_ID ;
	string Content ;

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;

	if (argc !=3) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	// convert and analyse args
	Symbol_ID = atoi(argv[1]);
	Content = argv[2];

	// delegate
	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->modify_SymbolContent(Symbol_ID, Content, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::modify_OK) ) {
		return TCL_ERROR ;
	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::set_RecSym_PaS (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: Symbol_ID
	2: x_left_up
	3: y_left_up
	4: width
	5: height
	*/

	int Symbol_ID ;
	int x_left_up ;
	int y_left_up ;
	int width ;
	int height ;

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;

	if (argc !=6) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	// convert and analyse args
	Symbol_ID = atoi (argv[1]) ;
	x_left_up = atoi (argv[2]) ;
	y_left_up = atoi (argv[3]) ;
	width = atoi (argv[4]) ;
	height = atoi (argv[5]) ;

	// delegate
	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->set_SymbolPaS(Symbol_ID, x_left_up, y_left_up, width, height, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::modify_OK) ) {
		return TCL_ERROR ;
	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::get_Channels (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: Diagram_Model_ID,
	2: Page_ID
	*/

	int Diagram_Model_ID ;
	int Page_ID ;
	type_ListOf_Poi_to_ConstChannel ModelResultList ;
	ClsMoAccessInfo AccessInfo ;

	SysSpec * pSystemSpec ;
	int CompletionCode ;
	string script ;

	//  clear MAR in Tcl interp !!!!
	CompletionCode = clear_MAR_ARR(interp);
	if (CompletionCode!=TCL_OK)
	  return TCL_ERROR;

	if (argc !=3) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	// convert and analyse args; delegate

	// get (enclosing) diagram ID and page number from Tcl
	Diagram_Model_ID = atoi(argv[1]);
	Page_ID = atoi(argv[2]);

	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->get_Channels(Diagram_Model_ID, Page_ID, ModelResultList, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::get_OK) ) {
		return TCL_ERROR ;
	} ;

	CompletionCode = write_Channels_to_Tcl_Interp(interp,ModelResultList) ;
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::get_AllReferencedDiagrams(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: Diagram_Model_ID
	*/

	int Diagram_Model_ID ;

	type_ListOf_Poi_to_ConstDiagram ModelResultList ;
	SysSpec * pSystemSpec ;
	const Diagram * pDiagram ;
	int CompletionCode ;
	string script ;

	if (argc !=2) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	//  clear MAR in Tcl interp !!!!
	CompletionCode = clear_MAR_ARR(interp) ;
	if (CompletionCode!=TCL_OK) {return TCL_ERROR; } ;

	// get (enclosing) diagram ID and page number from Tcl
	Diagram_Model_ID = atoi(argv[1]);

	ClsMoAccessInfo AccessInfo ;
	pSystemSpec = AppVars.get_SysSpec() ;

	pSystemSpec->get_AllReferencedDiagrams(Diagram_Model_ID, ModelResultList, AccessInfo) ;
	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::get_OK) ) {
		return TCL_ERROR ;
	} ;

	pDiagram = pSystemSpec->get_Diagram(Diagram_Model_ID, AccessInfo) ;
	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::get_OK) ) {
		return TCL_ERROR ;
	} ;

	CompletionCode = write_ReferencedDiagrams_to_Tcl_Interp(interp, ModelResultList, pDiagram) ;
	if (CompletionCode != TCL_OK)
	  return TCL_ERROR;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::get_Symbols (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: SymbolType ,
	2: Diagram_Model_ID,
	3: Page_ID
	*/

	EnumSym::Type SymbolType ;
	int Diagram_Model_ID ;
	int Page_ID ;
	type_ListOf_Poi_to_ConstSym ModelResultList ;
	SysSpec * pSystemSpec ;
	int CompletionCode ;
	string script ;

	//  clear MAR in Tcl interp !!!!
	CompletionCode = clear_MAR_ARR(interp) ;
	if (CompletionCode!=TCL_OK) {return TCL_ERROR; } ;

	if (argc !=4) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	// convert and analyse args; delegate

	// check symbol type
	determine_SymbolType (argv[1],SymbolType) ;

	if (SymbolType == EnumSym::none) {
		if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_ERROR, 0))
		  return TCL_ERROR;
		Tcl_SetResult(interp, "unknown symbol type in proc: get_Symbols_from_Model", TCL_VOLATILE);
		return TCL_ERROR;
	}

	// get (enclosing) diagram ID and page number from Tcl
	Diagram_Model_ID = atoi (argv[2]) ;
	Page_ID = atoi (argv[3]) ;

	ClsMoAccessInfo AccessInfo ;
	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->get_Syms(SymbolType, Diagram_Model_ID, Page_ID, ModelResultList, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::get_OK) ) {
		return TCL_ERROR ;
	} ;

	CompletionCode = write_Syms_to_Tcl_Interp(interp,ModelResultList) ;
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::modify_PageWidth (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: diagram ID
	2.Argument: Page ID
	3.Argument: PageWidth
	*/
	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;

	int Diagram_Model_ID ;
	int Page_ID ;
	int PageWidth ;

   	if (argc !=4) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	} ;

	Diagram_Model_ID = atoi(argv[1]);
	Page_ID = atoi(argv[2]);
	PageWidth = atoi(argv[3]);

	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->modify_PageWidth(Diagram_Model_ID, Page_ID, PageWidth, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::modify_OK) ) {
		return TCL_ERROR ;
	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::modify_PageHeight (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: diagram ID
	2.Argument: Page ID
	3.Argument: PageHeight
	*/
	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;

	int Diagram_Model_ID ;
	int Page_ID ;
	int PageHeight ;

   	if (argc !=4) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	} ;

	Diagram_Model_ID = atoi(argv[1]);
	Page_ID = atoi(argv[2]);
	PageHeight = atoi(argv[3]);

	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->modify_PageHeight(Diagram_Model_ID, Page_ID, PageHeight, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::modify_OK) ) {
		return TCL_ERROR ;
	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::add_NewPage_after (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: diagram ID
	2.Argument: Page ID
	*/

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;

	int Diagram_Model_ID ;
	int Page_ID ;

   	if (argc !=3) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	} ;

	Diagram_Model_ID = atoi(argv[1]);
	Page_ID = atoi(argv[2]);

	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->add_NewPage_after(Diagram_Model_ID, Page_ID, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::page_insert_OK) ) {
		return TCL_ERROR ;
	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::modify_PageFramePaS (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: diagram ID
	2.Argument: Page ID
	3.Argument: Frame_x_left_up
	4.Argument: Frame_y_left_up
	5.Argument: FrameWidth
	6.Argument: FrameHeight
	*/

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;

	int Diagram_Model_ID ;
	int Page_ID ;
	int Frame_x_left_up ;
	int Frame_y_left_up ;
	int FrameWidth ;
	int FrameHeight ;

   	if (argc !=7) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	Diagram_Model_ID = atoi(argv[1]);
	Page_ID = atoi(argv[2]);

	Frame_x_left_up = atoi(argv[3]);
	Frame_y_left_up = atoi(argv[4]);
	FrameWidth = atoi(argv[5]);
	FrameHeight = atoi(argv[6]);

	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->modify_PageFramePaS(Diagram_Model_ID, Page_ID, Frame_x_left_up, Frame_y_left_up,
		FrameWidth, FrameHeight, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::modify_OK) ) {
		return TCL_ERROR ;
	}

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::delete_Page (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: diagram ID
	2.Argument: Page ID
	*/

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;

	int Diagram_Model_ID ;
	int Page_ID ;

   	if (argc !=3) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	} ;

	Diagram_Model_ID = atoi(argv[1]);
	Page_ID = atoi(argv[2]);

	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->delete_Page(Diagram_Model_ID, Page_ID, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::page_delete_OK) ) {
		return TCL_ERROR ;
	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::get_PreviousPageID (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: diagram ID
	2.Argument: ancestor ID
	*/

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;
	string script ;
	int CompletionCode ;

	int Diagram_Model_ID ;
	int AncestorPageID ;

	ClsPageManager::e_PageIndex PageIndex ;
	int PreviousPageID ;
	char *PreviousPageExists ;

	//  clear MAR in Tcl interp !!!!
	CompletionCode = clear_MAR_ARR(interp) ;
	if (CompletionCode!=TCL_OK) {return TCL_ERROR; } ;

   	if (argc !=3) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	Diagram_Model_ID = atoi(argv[1]);
	AncestorPageID = atoi(argv[2]);

	pSystemSpec = AppVars.get_SysSpec() ;
	PreviousPageID = pSystemSpec->get_PreviousPageID(Diagram_Model_ID, AncestorPageID, PageIndex, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::page_get_OK) ) {
		return TCL_ERROR ;
	}

	// write to Tcl interp
	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_OK, 0))
	  return TCL_ERROR;

	if (PageIndex == ClsPageManager::middle) {
		PreviousPageExists = "true";
	} else {
		PreviousPageExists = "false";
	}
	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "PreviousPageExists", PreviousPageExists, 0))
	  return TCL_ERROR;

	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "PreviousPageID", PreviousPageID);
	if (CompletionCode != TCL_OK)
	  return TCL_ERROR;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::get_NextPageID (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: diagram ID
	2.Argument: predecessor ID
	*/

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;
	string script ;
	int CompletionCode ;

	int Diagram_Model_ID ;
	int PredecessorPageID ;

	ClsPageManager::e_PageIndex PageIndex ;
	int NextPageID ;
	char *NextPageExists ;

	//  clear MAR in Tcl interp !!!!
	CompletionCode = clear_MAR_ARR(interp) ;
	if (CompletionCode!=TCL_OK) {return TCL_ERROR; } ;

   	if (argc !=3) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	Diagram_Model_ID = atoi (argv[1]) ;
	PredecessorPageID = atoi (argv[2]) ;

	pSystemSpec = AppVars.get_SysSpec() ;
	NextPageID = pSystemSpec->get_NextPageID(Diagram_Model_ID, PredecessorPageID, PageIndex, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::page_get_OK) ) {
		return TCL_ERROR ;
	} ;

	// write to Tcl interp
	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_OK, 0))
	  return TCL_ERROR;

	if (PageIndex == ClsPageManager::middle) {
		NextPageExists = "true" ;
	} else {
		NextPageExists = "false" ;
	} ;
	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "NextPageExists", NextPageExists, 0))
	  return TCL_ERROR;

	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "NextPageID", NextPageID);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::get_LastPageID (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: diagram ID
	*/

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;
	string script ;
	int CompletionCode ;

	int Diagram_Model_ID ;
	int LastPageID ;

	//  clear MAR in Tcl interp !!!!
	CompletionCode = clear_MAR_ARR(interp) ;
	if (CompletionCode!=TCL_OK) {return TCL_ERROR; } ;

   	if (argc !=2) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	Diagram_Model_ID = atoi(argv[1]);

	pSystemSpec = AppVars.get_SysSpec() ;
	LastPageID = pSystemSpec->get_LastPageID(Diagram_Model_ID,AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::page_get_OK) ) {
		return TCL_ERROR ;
	} ;

	// write to Tcl interp
	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_OK, 0))
	  return TCL_ERROR;

	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "LastPageID", LastPageID);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::get_FirstPageID (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: diagram ID
	*/

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;
	string script ;
	int CompletionCode ;

	int Diagram_Model_ID ;
	int FirstPageID ;

	//  clear MAR in Tcl interp !!!!
	CompletionCode = clear_MAR_ARR(interp) ;
	if (CompletionCode!=TCL_OK) {return TCL_ERROR; } ;

   	if (argc !=2) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	Diagram_Model_ID = atoi(argv[1]);

	pSystemSpec = AppVars.get_SysSpec() ;
	FirstPageID = pSystemSpec->get_FirstPageID(Diagram_Model_ID,AccessInfo);

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::page_get_OK) ) {
		return TCL_ERROR ;
	}

	// write to Tcl interp
	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_OK, 0))
	  return TCL_ERROR;

	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "FirstPageID", FirstPageID);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::get_PageSize (ClientData clientData, 
				Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: diagram ID
	2.Argument: page ID
	*/

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;
	int CompletionCode ;

	int Diagram_Model_ID ;
	int Page_ID ;
	int PageWidth ;
	int PageHeight ;

	string script ;

	//  clear MAR in Tcl interp !!!!
	CompletionCode = clear_MAR_ARR(interp) ;
	if (CompletionCode!=TCL_OK) {return TCL_ERROR; } ;

   	if (argc !=3) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	Diagram_Model_ID = atoi(argv[1]);
	Page_ID = atoi(argv[2]);

	pSystemSpec = AppVars.get_SysSpec() ;

	pSystemSpec->get_PageSize(Diagram_Model_ID, Page_ID, PageWidth, PageHeight, AccessInfo);

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::get_OK) ) {
		return TCL_ERROR ;
	} ;

	// write to Tcl interp
	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_OK, 0))
	  return TCL_ERROR;

	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "PageWidth", PageWidth);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "PageHeight", PageHeight);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::delete_RecSym (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: symbol model ID
	*/

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;
	int Symbol_ID ;

   	if (argc !=2) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	Symbol_ID = atoi(argv[1]);

	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->delete_Symbol(Symbol_ID, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::delete_OK) ) {
		return TCL_ERROR ;
	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::get_ReferencingDiagramID (ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: referenced diagram ID
	*/

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;

	int ReferencedDiagramModel_ID ;
	int ReferencingDiagramModel_ID ;

   	if (objc !=2) {
		Tcl_WrongNumArgs(interp, objc, objv, "refID");
		return TCL_ERROR;
	}

	Tcl_GetIntFromObj(interp,objv[1],&ReferencedDiagramModel_ID);

	pSystemSpec = AppVars.get_SysSpec() ;
	ReferencingDiagramModel_ID = pSystemSpec->get_ReferencingDiagramID(ReferencedDiagramModel_ID, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::get_OK) ) {
		return TCL_ERROR ;
	} ;

	Tcl_SetObjResult(interp,Tcl_NewIntObj(ReferencingDiagramModel_ID));

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::modify_ProcedureDiaFormalParameters (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: diagram ID
	2: Content
	*/

	int Diagram_ID ;
	string str_FormalParameters ;

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;

	if (argc !=3) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	// convert and analyse args
	Diagram_ID = atoi (argv[1]) ;
	str_FormalParameters = argv[2] ;

	// delegate
	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->modify_ProcedureDiaFormalParameters(Diagram_ID, str_FormalParameters, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::modify_OK) ) {
		return TCL_ERROR ;
	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::get_ProcedureDiagramFormalParameters (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: diagram ID
	*/

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;
	int CompletionCode ;

	string str_HP_FormalParameters ;

	string script ;
	string str_PartScript ;

	//  clear MAR in Tcl interp !!!!
	CompletionCode = clear_MAR_ARR(interp) ;
	if (CompletionCode!=TCL_OK) {return TCL_ERROR; } ;

   	if (argc !=2) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	int Diagram_Model_ID = atoi(argv[1]);

	pSystemSpec = AppVars.get_SysSpec() ;

	pSystemSpec->get_ProcedureDiaFormalParameters(Diagram_Model_ID, str_HP_FormalParameters, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::get_OK) ) {
		return TCL_ERROR ;
	} ;


	// write to Tcl interp
	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_OK, 0))
	  return TCL_ERROR;

	createValListScrForStrings(str_HP_FormalParameters,str_PartScript) ;
	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "HP_FormalParameters", str_PartScript);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::get_PageFrameInfo (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: diagram ID
	2.Argument: page ID
	*/

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;
	int CompletionCode ;

	ClsRecPaS PaS(0,0,10,10) ; // dummy
	list<int> IntList ; // for PaS

	string str_HP_DiaType ;
	string str_HP_Name ;
	string str_HP_Index ;

	string script ;
	string str_PartScript ;

	//  clear MAR in Tcl interp !!!!
	CompletionCode = clear_MAR_ARR(interp) ;
	if (CompletionCode!=TCL_OK) {return TCL_ERROR; } ;

   	if (argc !=3) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	int Diagram_Model_ID = atoi (argv[1]) ;
	int Page_ID = atoi (argv[2]) ;

	pSystemSpec = AppVars.get_SysSpec() ;

	pSystemSpec->get_PageFrameInfo(Diagram_Model_ID, Page_ID, PaS,
		str_HP_DiaType, str_HP_Name, str_HP_Index, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::get_OK) ) {
		return TCL_ERROR ;
	} ;

	// write to Tcl interp
	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_OK, 0))
	  return TCL_ERROR;

	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "HP_DiaType", str_HP_DiaType);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	createValListScrForStrings(str_HP_Name,str_PartScript) ;
	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "HP_Name", str_PartScript);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "HP_Index", str_HP_Index);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	str_PartScript = "" ;

	PaS.get_PaS_as_IntList(IntList) ;
	createValListScr(IntList, str_PartScript) ;
	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "FramePaS", str_PartScript);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::insert_BidirecChannel(ClientData clientData, Tcl_Interp *interp,
									      int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: Model_ID des Objektes, in das eingefgt werden soll
	2.Argument: Page_ID
	3.Argument: Delay Type
	4.Argument: ChanSourceModel_ID
	5.Argument: ChanDestModel_ID
	6.Argument: ChanCoords as space-separated Integer-List

	7.Arguement: ChanName content
	8.Argument: ChanName PaS as space-separated Integer-List

	9.Argument: ChanSourceSigList content
	10.Argument: ChanSourceSigList PaS as space-separated Integer-List

	11.Argument: ChanSourceCon content
	12.Argument: ChanSourceCon PaS as space-separated Integer-List

	13.Argument: ChanDestSigList content
	14.Argument: ChanDestSigList PaS as space-separated Integer-List

	15.Argument: ChanDestCon content
	16.Argument: ChanDestCon PaS as space-separated Integer-List
	*/

   	if (argc !=17) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	int LocationID ;
	int PageID ;

	ClsChannel::e_DelayType ChannelDelayType ;

	bool bSourceIsEnv ;
	int ChanSourceModel_ID ;
	bool bDestIsEnv ;
	int ChanDestModel_ID ;

	list<int> ChanCoords ;

	string ChanNameContent ;
	string ChanSourceSigListContent ;
	string ChanSourceConContent ;
	string ChanDestSigListContent ;
	string ChanDestConContent ;

	ClsRecPaS ChanNamePaS(0,0,10,10) ;
	ClsRecPaS ChanSourceSigListPaS(0,0,10,10) ;
	ClsRecPaS ChanSourceConPaS(0,0,10,10) ;
	ClsRecPaS ChanDestSigListPaS(0,0,10,10) ;
	ClsRecPaS ChanDestConPaS(0,0,10,10) ;

	string script ;
	int CompletionCode ;
	ClsMoAccessInfo AccessInfo ;
	SysSpec * pSystemSpec ;
	int New_Model_ID ;

	// convert args
	LocationID = atoi (argv[1]) ;
	PageID = atoi (argv[2]) ;

	if ( ! (determine_ChannelDelayType(argv[3], ChannelDelayType) ) ) {
		Tcl_SetResult(interp, "unknown channel delay type", TCL_VOLATILE) ;
		return TCL_ERROR ;

	} ;

	if ( ChannelConPoint_is_Enviroment(argv[4]) ) {
		bSourceIsEnv = true ;
		ChanSourceModel_ID = -100 ;
	} else {
		bSourceIsEnv = false ;
		ChanSourceModel_ID = atoi (argv[4]) ;
	} ;

	if ( ChannelConPoint_is_Enviroment(argv[5]) ) {
		bDestIsEnv = true ;
		ChanDestModel_ID = -100 ;
	} else {
		bDestIsEnv = false ;
		ChanDestModel_ID = atoi (argv[5]) ;
	} ;

	get_IntList_from_space_separated_IntListString(argv[6],ChanCoords) ;

	ChanNameContent = argv[7] ;
	ChanSourceSigListContent = argv[9] ;
	ChanSourceConContent = argv[11] ;
	ChanDestSigListContent = argv[13] ;
	ChanDestConContent = argv[15] ;

	get_PaS_from_space_separated_IntListString(argv[8], ChanNamePaS) ;
	get_PaS_from_space_separated_IntListString(argv[10], ChanSourceSigListPaS) ;
	get_PaS_from_space_separated_IntListString(argv[12], ChanSourceConPaS) ;
	get_PaS_from_space_separated_IntListString(argv[14], ChanDestSigListPaS) ;
	get_PaS_from_space_separated_IntListString(argv[16], ChanDestConPaS) ;

	pSystemSpec = AppVars.get_SysSpec() ;

	ClsChanName ChanName(ChanNameContent, ChanNamePaS) ;
	ClsChanSourceSigList ChanSourceSigList(ChanSourceSigListContent, ChanSourceSigListPaS) ;
	ClsChanSourceCon ChanSourceCon(ChanSourceConContent, ChanSourceConPaS) ;
	ClsChanDestSigList ChanDestSigList(ChanDestSigListContent, ChanDestSigListPaS) ;
	ClsChanDestCon ChanDestCon(ChanDestConContent, ChanDestConPaS) ;

	pSystemSpec->insert_Channel(LocationID, PageID, ChannelDelayType,
							ChanName, ChanSourceSigList, ChanSourceCon,
							ChanDestSigList, ChanDestCon,
							ChanCoords,
							bSourceIsEnv, ChanSourceModel_ID,
							bDestIsEnv, ChanDestModel_ID,
							AccessInfo, New_Model_ID) ;


	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::insert_OK) ) {
		return TCL_ERROR ;
	} ;

	// write to Tcl interp
	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_OK, 0))
	  return TCL_ERROR;

	
	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "NewObj_Model_ID", New_Model_ID);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::modify_ChanComp(ClientData clientData, Tcl_Interp *interp,
									      int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: Channel_ID
	2: channel comp type
	3: content
	4: Pas
	*/

	int Channel_ID ;
	ClsChanComp::e_CompType ChanCompType ;
	string ChanCompContent ;
	ClsRecPaS ChanCompPaS(0,0,10,10) ;

	string script ;
	ClsMoAccessInfo AccessInfo ;
	SysSpec * pSystemSpec ;

	if (argc !=5) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	Channel_ID = atoi(argv[1]);

	 if ( ! determine_ChanCompType(argv[2], ChanCompType) ) {
		Tcl_SetResult(interp, "unknown channel comp type", TCL_VOLATILE) ;
		return TCL_ERROR ;
	 }

	ChanCompContent = argv[3] ;

	get_PaS_from_space_separated_IntListString(argv[4], ChanCompPaS) ;

	pSystemSpec = AppVars.get_SysSpec() ;

	switch (ChanCompType) {
		case ClsChanComp::name:
		{
			pSystemSpec->modify_ChanCompName(Channel_ID, ChanCompContent, ChanCompPaS, AccessInfo) ;
			break ;
		}
		case ClsChanComp::source_sig:
		{
			pSystemSpec->modify_ChanSourceSig(Channel_ID, ChanCompContent, ChanCompPaS, AccessInfo) ;
			break ;
		}
		case ClsChanComp::dest_sig:
		{
			pSystemSpec->modify_ChanDestSig(Channel_ID, ChanCompContent, ChanCompPaS, AccessInfo) ;
			break ;
		}
		case ClsChanComp::source_con:
		{
			pSystemSpec->modify_ChanSourceCon(Channel_ID, ChanCompContent, ChanCompPaS, AccessInfo) ;
			break ;
		}
		case ClsChanComp::dest_con:
		{
			pSystemSpec->modify_ChanDestCon(Channel_ID, ChanCompContent, ChanCompPaS, AccessInfo) ;
			break ;
		}
		default:
		{
			Tcl_SetResult(interp, "unknown channel comp type", TCL_VOLATILE);
			return TCL_ERROR ;
		}
	} ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::modify_OK) ) {
		return TCL_ERROR ;
	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::modify_Chan_Coords(ClientData clientData, Tcl_Interp *interp,
									      int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: Channel_ID
	2: channel coords
	*/

	int Channel_ID ;
	list<int> ChanCoords ;

	string script ;
	ClsMoAccessInfo AccessInfo ;
	SysSpec * pSystemSpec ;

	if (argc !=3) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	Channel_ID = atoi(argv[1]);

	get_IntList_from_space_separated_IntListString(argv[2],ChanCoords) ;

	pSystemSpec = AppVars.get_SysSpec() ;

	pSystemSpec->modify_Chan_Coords(Channel_ID, ChanCoords, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::modify_OK) ) {
		return TCL_ERROR ;
	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::modify_Chan_Type(ClientData clientData, Tcl_Interp *interp,
									      int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: Channel_ID
	2: delay type
	3: direction type
	*/

	int Channel_ID ;
	ClsChannel::e_DelayType ChannelDelayType ;
	ClsChannel::e_DirectionType ChannelDirectionType ;

	string script ;
	ClsMoAccessInfo AccessInfo ;
	SysSpec * pSystemSpec ;

	if (argc !=4) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	Channel_ID = atoi(argv[1]);

	if ( ! (determine_ChannelDelayType(argv[2], ChannelDelayType) ) ) {
		Tcl_SetResult(interp, "unknown channel delay type", TCL_VOLATILE);
		return TCL_ERROR;
	}

	if ( ! (determine_ChannelDirectionType(argv[3], ChannelDirectionType) ) ) {
		
		Tcl_SetResult(interp, "unknown channel direction type", TCL_VOLATILE);
		return TCL_ERROR;
	}

	pSystemSpec = AppVars.get_SysSpec() ;

	pSystemSpec->modify_Chan_DirectionType(Channel_ID, ChannelDirectionType, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::modify_OK) ) {
		return TCL_ERROR ;
	} ;

	pSystemSpec->modify_Chan_DelayType(Channel_ID, ChannelDelayType, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::modify_OK) ) {
		return TCL_ERROR ;
	} ;

	// Test
	//pSystemSpec->print_Map() ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::delete_Chan(ClientData clientData,
				Tcl_Interp *interp, int argc, CONST char *argv[]) {
	/*
	0.Argument: tcl-command-name
	1.Argument: channel model ID
	*/

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;
	int Channel_ID ;

   	if (argc !=2) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	Channel_ID = atoi(argv[1]);

	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->delete_Channel(Channel_ID, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::delete_OK) ) {
		return TCL_ERROR ;
	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::insert_RecSym(ClientData clientData, Tcl_Interp *interp,
							     int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: Model_ID des Objektes, in das eingefgt werden soll
	2.Argument: Page_ID
	3.Argument: Typ des einzufgenden Objektes
	4.Argument: x_pos
	5.Argument: y_pos
	6.Argument: width
	7.Arguement: height
	8.Argument: Text
	*/

	EnumSym::Type SymbolType ;

	int Diagram_Model_ID ;
	int Page_ID ;
	int x_left_up ;
	int y_left_up ;
	int width ;
	int height ;
	string content ;

	SysSpec * pSystemSpec ;
	const Symbol * pNewSymbol ;
	int CompletionCode ;
	ClsMoAccessInfo AccessInfo ;
	string script ;
	int New_Model_ID ;

	//  clear MAR in Tcl interp !!!!
	CompletionCode = clear_MAR_ARR(interp) ;
	if (CompletionCode!=TCL_OK) {return TCL_ERROR; } ;

	if (argc !=9) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	// convert arguments
	Diagram_Model_ID = atoi (argv[1]) ;
	Page_ID = atoi (argv[2]) ;

	// check symbol type
	determine_SymbolType (argv[3],SymbolType) ;

	if (SymbolType == EnumSym::none) {
		if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_ERROR, 0))
		  return TCL_ERROR;
		Tcl_SetResult(interp, "unknown symbol type in proc: get_Symbols_from_Model", TCL_VOLATILE);
		return TCL_ERROR;
	}

	x_left_up = atoi (argv[4]) ;
	y_left_up = atoi (argv[5]) ;
	width = atoi (argv[6]) ;
	height = atoi (argv[7]) ;
	content = argv[8] ;

	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->insert_Symbol(SymbolType, Diagram_Model_ID, Page_ID, x_left_up, y_left_up, width, height , content, AccessInfo, New_Model_ID) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::insert_OK) ) {
		return TCL_ERROR ;
	} ;

	pNewSymbol = pSystemSpec->get_RecSymbol(New_Model_ID, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::get_OK) ) {
		return TCL_ERROR ;
	} ;

	// write to Tcl interp
	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_OK, 0))
	  return TCL_ERROR;

	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "NewObj_Model_ID", New_Model_ID);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	if (pNewSymbol->is_referencing_Symbol()) {
		CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "RefDia_Model_ID", pNewSymbol->get_referenced_Diagram_Model_ID());
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;
	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::write_FlowLines_to_Tcl_Interp(Tcl_Interp *interp, type_ListOfPoi_to_ConstFlowLine & ModelResultList) {

	type_ListOfPoi_to_ConstFlowLine::const_iterator iter ;

	int CompletionCode ;
	string script ;
	string PartScript ;
	int cnt ;
	int FlowLineID ;

	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_OK, 0))
	  return TCL_ERROR;

	cnt = ModelResultList.size() ;
	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "Count", cnt);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	createValListScr(ModelResultList, PartScript) ;
	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "IDs", PartScript);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	for (iter = ModelResultList.begin() ; iter != ModelResultList.end() ; iter++) {

		FlowLineID = (*iter)->get_Model_ID() ;

		PartScript = "" ;
		script = "" ;

		createValListScr((*iter), PartScript) ;
		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, FlowLineID, "Coords", PartScript);
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, FlowLineID, "Source_Model_ID", (*iter)->get_SourceID_as_String());
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, FlowLineID, "Dest_Model_ID", (*iter)->get_DestID_as_String());
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		switch ((*iter)->get_FlowLineType()) {
			case ClsFlowLine::normal:
			{
				CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, FlowLineID, "Type", "normal");
				if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

				break ;
			} ;
			case ClsFlowLine::decision:
			{
				CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, FlowLineID, "Type", "decision");
				if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

				ClsDecisionFlowLine * pDecisionFlowLine = (ClsDecisionFlowLine*)(*iter) ;

				createValListScr( pDecisionFlowLine->get_PaS_as_IntList(), PartScript) ;
				CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, FlowLineID, "PaS", PartScript);
				if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

				createValListScrForStrings(pDecisionFlowLine->get_Content(), PartScript) ;
				CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, FlowLineID, "Content", PartScript);
				if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

				break ;
			}
			default:
			{
				return TCL_ERROR ;
			}
		} ; // switch ((*iter)->get_FlowLineType())

	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::write_Channels_to_Tcl_Interp(Tcl_Interp *interp, type_ListOf_Poi_to_ConstChannel & ModelResultList) {

	type_ListOf_Poi_to_ConstChannel::const_iterator iter ;
	int CompletionCode ;
	string script ;
	string PartScript ;
	int cnt ;
	int Channel_Model_ID ;

	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_OK, 0))
	  return TCL_ERROR;

	cnt = ModelResultList.size() ;
	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "Count", cnt);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	createValListScr(ModelResultList, PartScript) ;
	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "IDs", PartScript);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	for (iter = ModelResultList.begin() ; iter != ModelResultList.end() ; iter++) {

		Channel_Model_ID = (*iter)->get_Model_ID() ;

		createValListScr((*iter), PartScript) ;
		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, Channel_Model_ID, "Coords", PartScript);
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, Channel_Model_ID, "Source_Model_ID", (*iter)->get_SourceID_as_String());
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, Channel_Model_ID, "Dest_Model_ID", (*iter)->get_DestID_as_String());
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		createValListScrForStrings((*iter)->get_NameContent(), PartScript) ;
		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, Channel_Model_ID, "NameContent", PartScript);
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		createValListScrForStrings((*iter)->get_SourceSigListContent(), PartScript) ;
		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, Channel_Model_ID, "SourceSigListContent", PartScript);
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		createValListScrForStrings((*iter)->get_SourceConContent(), PartScript) ;
		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, Channel_Model_ID, "SourceConContent", PartScript);
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		createValListScrForStrings((*iter)->get_DestSigListContent(), PartScript) ;
		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, Channel_Model_ID, "DestSigListContent", PartScript);
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		createValListScrForStrings((*iter)->get_DestConContent(), PartScript) ;
		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, Channel_Model_ID, "DestConContent", PartScript);
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, Channel_Model_ID, "DirectionType", (*iter)->get_DirectionType_as_String());
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, Channel_Model_ID, "DelayType", (*iter)->get_DelayType_as_String());
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		createValListScr((*iter)->get_NamePas(), PartScript) ;
		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, Channel_Model_ID, "NamePaS", PartScript);
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		createValListScr((*iter)->get_SourceSigListPas( ), PartScript) ;
		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, Channel_Model_ID, "SourceSigListPaS", PartScript);
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		createValListScr((*iter)->get_SourceConPas( ), PartScript) ;
		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, Channel_Model_ID, "SourceConPaS", PartScript);
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		createValListScr((*iter)->get_DestSigListPas( ), PartScript) ;
		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, Channel_Model_ID, "DestSigListPaS", PartScript);
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

		createValListScr((*iter)->get_DestConPas( ), PartScript) ;
		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, Channel_Model_ID, "DestConPaS", PartScript);
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::write_ReferencedDiagrams_to_Tcl_Interp(Tcl_Interp * interp,
				type_ListOf_Poi_to_ConstDiagram & ModelResultList, const Diagram * const pDiagram) {

	type_ListOf_Poi_to_ConstDiagram::iterator iter ;
	int CompletionCode ;
	string str_DiagramName ;
	string script ;
	string PartScript ;
	int cnt ;

	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_OK, 0))
	  return TCL_ERROR;

	cnt = ModelResultList.size() ;
	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "Count", cnt);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "DiaType", get_DiagramTypeAsString(pDiagram->get_DiagramType()));
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	pDiagram->get_HeaderPart_Name(str_DiagramName) ;
	createValListScrForStrings(str_DiagramName, PartScript) ;
	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "Text", PartScript);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	createValListScr(ModelResultList, PartScript) ;
	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "IDs", PartScript);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	for (iter = ModelResultList.begin() ; iter != ModelResultList.end() ; iter++) {

		(*iter)->get_HeaderPart_Name(str_DiagramName) ;
		createValListScrForStrings(str_DiagramName, PartScript) ;
		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, (*iter)->get_Model_ID(), "Text", PartScript);
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;
	} ;


	for (iter = ModelResultList.begin() ; iter != ModelResultList.end() ; iter++) {

		PartScript = get_DiagramTypeAsString((*iter)->get_DiagramType()) ;

		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, (*iter)->get_Model_ID(), "DiaType", PartScript);
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;
	} ;

	return TCL_OK ;

} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::write_Syms_to_Tcl_Interp(Tcl_Interp *interp, type_ListOf_Poi_to_ConstSym & ModelResultList) {

	type_ListOf_Poi_to_ConstSym::iterator iter ;
	int CompletionCode ;
	string script ;
	string PartScript ;
	int cnt ;

	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_OK, 0))
	  return TCL_ERROR;

	cnt = ModelResultList.size() ;
	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "Count", cnt);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	createValListScr(ModelResultList, PartScript) ;
	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "IDs", PartScript);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	for (iter = ModelResultList.begin() ; iter != ModelResultList.end() ; iter++) {
		createValListScr((*iter), PartScript) ;
		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, (*iter)->get_Model_ID(), "PaS", PartScript);
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;
	} ;

	for (iter = ModelResultList.begin() ; iter != ModelResultList.end() ; iter++) {
		createValListScrForStrings((*iter)->get_content(), PartScript) ;
		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, (*iter)->get_Model_ID(), "Text", PartScript);
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;
	} ;

	for (iter = ModelResultList.begin() ; iter != ModelResultList.end() ; iter++) {
		if ( (*iter)->is_referencing_Symbol() ) {
			CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, (*iter)->get_Model_ID(), "RefDia_Model_ID",
				olSDL::Converter::Int_to_String((*iter)->get_referenced_Diagram_Model_ID()));
			if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;
		} ;
	} ;

	return TCL_OK ;
} ;

//*******************************************************************************************************
//**** process diagram symbol
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::write_StateMachineSyms_to_Tcl_Interp(Tcl_Interp *interp, type_ListOfPoi_to_ConstStateMachineSymbol & ModelResultList) {

	type_ListOfPoi_to_ConstStateMachineSymbol::iterator iter ;
	int CompletionCode ;
	string script ;
	string PartScript ;
	int cnt ;

	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_OK, 0))
	  return TCL_ERROR;

	cnt = ModelResultList.size() ;
	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "Count", cnt);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	createValListScr(ModelResultList, PartScript) ;
	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "IDs", PartScript);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	for (iter = ModelResultList.begin() ; iter != ModelResultList.end() ; iter++) {
		createValListScr((*iter), PartScript) ;
		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, (*iter)->get_Model_ID(), "PaS", PartScript);
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;
	} ;

	for (iter = ModelResultList.begin() ; iter != ModelResultList.end() ; iter++) {
		createValListScrForStrings((*iter)->get_content(), PartScript) ;
		CompletionCode = TwoDimVarSet(interp, pMAR_Array_Name, (*iter)->get_Model_ID(), "Text", PartScript);
		if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;
	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::get_StateMachineSyms(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: StateMachineSymbolType ,
	2: Diagram_Model_ID,
	3: Page_ID
	*/

	ClsStateMachineSymbol::e_StateMachineSymbolType StateMachineSymbolType ;
	int Diagram_Model_ID ;
	int Page_ID ;
	type_ListOfPoi_to_ConstStateMachineSymbol ModelResultList ;
	SysSpec * pSystemSpec ;
	int CompletionCode ;
	string script ;

	//  clear MAR in Tcl interp !!!!
	CompletionCode = clear_MAR_ARR(interp) ;
	if (CompletionCode!=TCL_OK) {return TCL_ERROR; } ;

	if (argc !=4) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	// convert and analyse args; delegate

	// check symbol type
	determine_StateMachineSymbolType (argv[1], StateMachineSymbolType) ;

	if (StateMachineSymbolType == ClsStateMachineSymbol::none) {
		if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_ERROR, 0))
		  return TCL_ERROR;
		Tcl_SetResult(interp, "unknown process diagram symbol type in get_StateMachineSyms()", TCL_VOLATILE);
		return TCL_ERROR;
	}

	// get (enclosing) diagram ID and page number from Tcl
	Diagram_Model_ID = atoi (argv[2]) ;
	Page_ID = atoi (argv[3]) ;

	ClsMoAccessInfo AccessInfo ;
	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->get_StateMachineSyms(StateMachineSymbolType, Diagram_Model_ID, Page_ID, ModelResultList, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::get_OK) ) {
		return TCL_ERROR ;
	} ;

	CompletionCode = write_StateMachineSyms_to_Tcl_Interp(interp,ModelResultList) ;
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::delete_StateMachineSymbol (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: symbol model ID
	*/

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;
	int Symbol_ID ;

   	if (argc !=2) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	Symbol_ID = atoi (argv[1]) ;

	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->delete_StateMachineSymbol(Symbol_ID, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::delete_OK) ) {
		return TCL_ERROR ;
	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::modify_StateMachineSymbol_Content (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: Symbol_ID
	2: Content
	*/

	int Symbol_ID ;
	string Content ;

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;

	if (argc !=3) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	// convert and analyse args
	Symbol_ID = atoi (argv[1]) ;
	Content = argv[2] ;

	// delegate
	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->modify_StateMachineSymContent(Symbol_ID, Content, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::modify_OK) ) {
		return TCL_ERROR ;
	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::modify_StateMachineSymbol_PaS (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: Symbol_ID
	2: x_left_up
	3: y_left_up
	4: width
	5: height
	*/

	int Symbol_ID ;
	int x_left_up ;
	int y_left_up ;
	int width ;
	int height ;

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;

	if (argc !=6) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	// convert and analyse args
	Symbol_ID = atoi (argv[1]) ;
	x_left_up = atoi (argv[2]) ;
	y_left_up = atoi (argv[3]) ;
	width = atoi (argv[4]) ;
	height = atoi (argv[5]) ;

	// delegate
	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->modify_StateMachineSymPaS(Symbol_ID, x_left_up, y_left_up, width, height, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::modify_OK) ) {
		return TCL_ERROR ;
	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::insert_StateMachineSymbol(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: Model_ID des Diagrams, in das eingefgt werden soll
	2.Argument: Page_ID
	3.Argument: Typ des einzufgenden Objektes
	4.Argument: x_pos
	5.Argument: y_pos
	6.Argument: width
	7.Arguement: height
	8.Argument: Text
	*/

	int Diagram_Model_ID ;
	int Page_ID ;
	ClsStateMachineSymbol::e_StateMachineSymbolType StateMachineSymbolType ;
	int x_left_up ;
	int y_left_up ;
	int width ;
	int height ;
	string content ;

	SysSpec * pSystemSpec ;
	const ClsStateMachineSymbol * pNewSymbol ;
	int CompletionCode ;
	ClsMoAccessInfo AccessInfo ;
	string script ;
	int New_Model_ID ;

	//  clear MAR in Tcl interp !!!!
	CompletionCode = clear_MAR_ARR(interp) ;
	if (CompletionCode!=TCL_OK) {return TCL_ERROR; } ;

	if (argc !=9) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	// convert arguments
	Diagram_Model_ID = atoi (argv[1]) ;
	Page_ID = atoi (argv[2]) ;

	// check symbol type
	determine_StateMachineSymbolType (argv[3], StateMachineSymbolType) ;


	if (StateMachineSymbolType == ClsStateMachineSymbol::none) {
		if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_ERROR, 0))
		  return TCL_ERROR;
		Tcl_SetResult(interp, "unknown process diagram symbol type in insert_StateMachineSymbol()", TCL_VOLATILE);
		return TCL_ERROR;
	}

	x_left_up = atoi (argv[4]) ;
	y_left_up = atoi (argv[5]) ;
	width = atoi (argv[6]) ;
	height = atoi (argv[7]) ;
	content = argv[8] ;

	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->insert_StateMachineSymbol(StateMachineSymbolType, Diagram_Model_ID, Page_ID,
			x_left_up, y_left_up, width, height, content, AccessInfo, New_Model_ID) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::insert_OK) ) {
		return TCL_ERROR ;
	} ;

	pNewSymbol = pSystemSpec->get_StateMachineSymbol(New_Model_ID,AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::get_OK) ) {
		return TCL_ERROR ;
	} ;

	// write to Tcl interp
	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_OK, 0))
	  return TCL_ERROR;

	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "NewObj_Model_ID", New_Model_ID);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
//**** END: process diagram symbol
//*******************************************************************************************************

//*******************************************************************************************************
//**** flow line
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::insert_DecisionFlowLine(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: Model_ID des Objektes, in das eingefgt werden soll
	2.Argument: Page_ID
	3.Argument: FlowLineSourceModel_ID
	4.Argument: FlowLineDestModel_ID
	5.Argument: FlowLineCoords as space-separated Integer-List
	6.Argument: Decision Flow Line Comp Content
	7.Argument: Decision Flow Line Comp PaS as space-separated Integer-List
	*/

   	if (argc !=8) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	int LocationID ;
	int PageID ;
	int FlowLineSourceModel_ID ;
	int FlowLineDestModel_ID ;
	list<int> FlowLineCoords ;

	string Content ;
	ClsRecPaS DecisionFlowLinePas(0,0,10,10) ;

	string script ;
	int CompletionCode ;
	ClsMoAccessInfo AccessInfo ;
	SysSpec * pSystemSpec ;
	int New_Model_ID ;

	// convert args
	LocationID = atoi (argv[1]) ;
	PageID = atoi (argv[2]) ;
	FlowLineSourceModel_ID = atoi (argv[3]) ;
	FlowLineDestModel_ID = atoi (argv[4]) ;

	get_IntList_from_space_separated_IntListString(argv[5], FlowLineCoords);

	Content = argv[6] ;
	get_PaS_from_space_separated_IntListString(argv[7], DecisionFlowLinePas) ;


	pSystemSpec = AppVars.get_SysSpec() ;

	pSystemSpec->insert_DecisionFlowLine(LocationID, PageID, FlowLineCoords, FlowLineSourceModel_ID,
		FlowLineDestModel_ID, Content, DecisionFlowLinePas, AccessInfo, New_Model_ID) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::insert_OK) ) {
		return TCL_ERROR ;
	} ;

	// write to Tcl interp
	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_OK, 0))
	  return TCL_ERROR;

	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "NewObj_Model_ID", New_Model_ID);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::insert_FlowLine(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: Model_ID des Objektes, in das eingefgt werden soll
	2.Argument: Page_ID
	3.Argument: FlowLineSourceModel_ID
	4.Argument: FlowLineDestModel_ID
	5.Argument: FlowLineCoords as space-separated Integer-List
	*/

   	if (argc !=6) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	int LocationID ;
	int PageID ;
	int FlowLineSourceModel_ID ;
	int FlowLineDestModel_ID ;
	list<int> FlowLineCoords ;

	string script ;
	int CompletionCode ;
	ClsMoAccessInfo AccessInfo ;
	SysSpec * pSystemSpec ;
	int New_Model_ID ;

	// convert args
	LocationID = atoi (argv[1]) ;
	PageID = atoi (argv[2]) ;
	FlowLineSourceModel_ID = atoi (argv[3]) ;
	FlowLineDestModel_ID = atoi (argv[4]) ;

	get_IntList_from_space_separated_IntListString(argv[5], FlowLineCoords);

	pSystemSpec = AppVars.get_SysSpec() ;

	pSystemSpec->insert_FlowLine(LocationID, PageID, FlowLineCoords, 
		FlowLineSourceModel_ID, FlowLineDestModel_ID, AccessInfo, New_Model_ID) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::insert_OK) ) {
		return TCL_ERROR ;
	} ;

	// write to Tcl interp
	if (!Tcl_SetVar2(interp, pMAR_Array_Name, "Result", pMAR_OK, 0))
	  return TCL_ERROR;

	CompletionCode = OneDimVarSet(interp, pMAR_Array_Name, "NewObj_Model_ID", New_Model_ID);
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::modify_DecisionFlowLinePaS(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: FlowLineID
	2: x_left_up
	3: y_left_up
	4: width
	5: height
	*/

	int FlowLineID ;
	int x_left_up ;
	int y_left_up ;
	int width ;
	int height ;

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;

	if (argc !=6) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	// convert and analyse args
	FlowLineID = atoi (argv[1]) ;
	x_left_up = atoi (argv[2]) ;
	y_left_up = atoi (argv[3]) ;
	width = atoi (argv[4]) ;
	height = atoi (argv[5]) ;

	pSystemSpec = AppVars.get_SysSpec() ;

	pSystemSpec->modify_DecisionFlowLinePaS(FlowLineID, x_left_up, y_left_up, width, height, AccessInfo) ;


	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::modify_OK) ) {
		return TCL_ERROR ;
	} ;

	return TCL_OK ;

} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::modify_DecisionFlowLineContent(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: FlowLineID
	2: content
	*/

	int FlowLineID ;
	string Content ;

	string script ;
	ClsMoAccessInfo AccessInfo ;
	SysSpec * pSystemSpec ;

	if (argc !=3) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	FlowLineID = atoi (argv[1]) ;
	Content = argv[2] ;

	pSystemSpec = AppVars.get_SysSpec() ;

	pSystemSpec->modify_DecisionFlowLineContent(FlowLineID, Content, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::modify_OK) ) {
		return TCL_ERROR ;
	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::modify_FlowLineCoords(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: FlowLineID
	2: flow line coords
	*/

	int FlowLineID ;
	list<int> FlowLineCoords ;

	string script ;
	ClsMoAccessInfo AccessInfo ;
	SysSpec * pSystemSpec ;

	if (argc !=3) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	FlowLineID = atoi (argv[1]) ;

	get_IntList_from_space_separated_IntListString(argv[2], FlowLineCoords) ;

	pSystemSpec = AppVars.get_SysSpec() ;

	pSystemSpec->modify_FlowLineCoords(FlowLineID, FlowLineCoords, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::modify_OK) ) {
		return TCL_ERROR ;
	} ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::delete_FlowLine(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0.Argument: tcl-command-name
	1.Argument: flow line model ID
	*/

	SysSpec * pSystemSpec ;
	ClsMoAccessInfo AccessInfo ;
	int FlowLineID ;

   	if (argc !=2) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	FlowLineID = atoi (argv[1]) ;

	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->delete_FlowLine(FlowLineID, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::delete_OK) ) {
		return TCL_ERROR ;
	} ;

	// Test
	//pSystemSpec->print_Map() ;

	return TCL_OK ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::get_FlowLines(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) {

	/*
	0: tcl-command-name
	1: Diagram_Model_ID,
	2: Page_ID
	*/

	int Diagram_Model_ID ;
	int Page_ID ;
	type_ListOfPoi_to_ConstFlowLine ModelResultList ;
	ClsMoAccessInfo AccessInfo ;

	SysSpec * pSystemSpec ;
	int CompletionCode ;
	string script ;

	//  clear MAR in Tcl interp !!!!
	CompletionCode = clear_MAR_ARR(interp) ;
	if (CompletionCode!=TCL_OK) {return TCL_ERROR; } ;

	if (argc !=3) {
		Tcl_SetResult(interp, "wrong # args", TCL_VOLATILE);
		return TCL_ERROR;
	}

	// convert and analyse args; delegate

	Diagram_Model_ID = atoi (argv[1]) ;
	Page_ID = atoi (argv[2]) ;

	pSystemSpec = AppVars.get_SysSpec() ;
	pSystemSpec->get_FlowLines(Diagram_Model_ID, Page_ID, ModelResultList, AccessInfo) ;

	if ( ! AccessOK(interp, AccessInfo, ClsMoAccessInfo::get_OK) ) {
		return TCL_ERROR ;
	} ;

	CompletionCode = write_FlowLines_to_Tcl_Interp(interp,ModelResultList) ;
	if (CompletionCode != TCL_OK) { return TCL_ERROR ; } ;

	return TCL_OK ;
} ;

//*******************************************************************************************************
//**** END: flow line
//*******************************************************************************************************

//*******************************************************************************************************
//*******************************************************************************************************
//**** HELPER
//*******************************************************************************************************
//*******************************************************************************************************
bool olSDL::TclExt::TclModelAccess::ChannelConPoint_is_Enviroment(string strChannelConPoint) {

	if (strChannelConPoint.compare("ENV") == 0) {
		return true;
	} else {
		return false;
	}
}
//*******************************************************************************************************
string olSDL::TclExt::TclModelAccess::get_DiagramTypeAsString(EnumDia::Type DiagramType) {
	switch (DiagramType) {
		case EnumDia::SysDia:
		{
			return "SysDia";
			break;
		}
		case EnumDia::BlkDia:
		{
			return "BlkDia";
			break;
		}
		case EnumDia::ProDia:
		{
			return "ProDia";
			break;
		}
		case EnumDia::BlkTypeDia:
		{
			return "BlkTypeDia";
			break;
		}
		case EnumDia::ProTypeDia:
		{
			return "ProTypeDia";
			break;
		}
		case EnumDia::ProcedureDia:
		{
			return "ProcedureDia";
			break;
		}
		default:
		{
			return "none";
		}
	} // switch
}
//*******************************************************************************************************
void olSDL::TclExt::TclModelAccess::get_IntList_from_space_separated_IntListString(string strIntList, list<int> & IntList) {

	IntList.clear() ;

	int StringLength = strIntList.length( ) ;
	string PartString ;

	int StartIndex = 0 ;
	for (int i=0; i<StringLength; i++) {

		if ( strIntList.substr(i,1).compare(" ") == 0 ) {
			PartString = strIntList.substr(StartIndex, i-StartIndex) ;
			IntList.push_back(  atoi(PartString.c_str())  ) ;
			StartIndex = i + 1 ;
		} ;

	} ;

	PartString = strIntList.substr(StartIndex, (StringLength-1)) ;
	IntList.push_back(  atoi(PartString.c_str())  ) ;
}
//*******************************************************************************************************
void olSDL::TclExt::TclModelAccess::get_PaS_from_space_separated_IntListString(string strIntList, ClsRecPaS & PaS) {

	vector<int> IntVector ;

	int StringLength = strIntList.length( ) ;
	string PartString ;

	int StartIndex = 0 ;
	for (int i=0; i<StringLength; i++) {

		if ( strIntList.substr(i,1).compare(" ") == 0 ) {
			PartString = strIntList.substr(StartIndex, i-StartIndex) ;
			IntVector.push_back(  atoi(PartString.c_str())  ) ;
			StartIndex = i + 1 ;
		} ;

	} ;

	PartString = strIntList.substr(StartIndex, (StringLength-1)) ;
	IntVector.push_back(  atoi(PartString.c_str())  ) ;

	PaS.set_PaS(IntVector[0], IntVector[1], IntVector[2], IntVector[3]) ;
} ;
//*******************************************************************************************************
bool olSDL::TclExt::TclModelAccess::determine_ChannelDirectionType(const string str_ChannelDirectionType, ClsChannel::e_DirectionType & DirectionType) {
	if (str_ChannelDirectionType.compare("both") == 0) {
		DirectionType = ClsChannel::both;
	} else if (str_ChannelDirectionType.compare("from_source") == 0) {
		DirectionType = ClsChannel::from_source;
	} else if (str_ChannelDirectionType.compare("from_destination") == 0) {
		DirectionType = ClsChannel::from_destination;
	} else {
		return false;
	}
	return true;
}
//*******************************************************************************************************
bool olSDL::TclExt::TclModelAccess::determine_ChanCompType(const string str_ChanCompType, ClsChanComp::e_CompType & ChanCompType) {

	if ( str_ChanCompType.compare("ChanName") == 0 ) {
		ChanCompType = ClsChanComp::name ;
	} else if ( str_ChanCompType.compare("ChanSourceSig") == 0 ) {
		ChanCompType = ClsChanComp::source_sig ;
	} else if ( str_ChanCompType.compare("ChanDestSig") == 0 ) {
		ChanCompType = ClsChanComp::dest_sig ;
	} else if ( str_ChanCompType.compare("ChanSourceCon") == 0 ) {
		ChanCompType = ClsChanComp::source_con ;
	} else if ( str_ChanCompType.compare("ChanDestCon") == 0 ) {
		ChanCompType = ClsChanComp::dest_con ;
	} else {
		return false ;
	}
	return true ;
} ;
//*******************************************************************************************************
bool olSDL::TclExt::TclModelAccess::determine_ChannelDelayType(const string str_ChannelDelayType, ClsChannel::e_DelayType & ChannelDelayType) {
	if ( str_ChannelDelayType.compare("delay") == 0 ) {
		ChannelDelayType = ClsChannel::delay ;
	} else if ( str_ChannelDelayType.compare("no_delay") == 0 ) {
		ChannelDelayType = ClsChannel::no_delay ;
	} else {
		return false ;
	}
	return true ;
}
//*******************************************************************************************************
void olSDL::TclExt::TclModelAccess::determine_StateMachineSymbolType(const string str_StateMachineSymbolType,
	ClsStateMachineSymbol::e_StateMachineSymbolType & StateMachineSymbolType) {

	StateMachineSymbolType = ClsStateMachineSymbol::none ;

	if (str_StateMachineSymbolType.compare("StateMaStartStateSym") == 0) {
		StateMachineSymbolType = ClsStateMachineSymbol::start ;
	} else if (str_StateMachineSymbolType.compare("StateMaStateSym") == 0) {
		StateMachineSymbolType = ClsStateMachineSymbol::state ;
	} else if (str_StateMachineSymbolType.compare("StateMaOutputSym") == 0) {
		StateMachineSymbolType = ClsStateMachineSymbol::output ;
	} else if (str_StateMachineSymbolType.compare("StateMaInputSym") == 0) {
		StateMachineSymbolType = ClsStateMachineSymbol::input ;
	} else if (str_StateMachineSymbolType.compare("StateMaStopSym") == 0) {
		StateMachineSymbolType = ClsStateMachineSymbol::stop ;
	} else if (str_StateMachineSymbolType.compare("StateMaTaskSym") == 0) {
		StateMachineSymbolType = ClsStateMachineSymbol::task ;
	} else if (str_StateMachineSymbolType.compare("StateMaConditionSym") == 0) {
		StateMachineSymbolType = ClsStateMachineSymbol::condition ;
	} else if (str_StateMachineSymbolType.compare("StateMaDecisionSym") == 0) {
		StateMachineSymbolType = ClsStateMachineSymbol::decision ;
	} else if (str_StateMachineSymbolType.compare("StateMaCreateProSym") == 0) {
		StateMachineSymbolType = ClsStateMachineSymbol::create_pro ;
	} else if (str_StateMachineSymbolType.compare("StateMaCallProcedureSym") == 0) {
		StateMachineSymbolType = ClsStateMachineSymbol::call_procedure ;
	} else if (str_StateMachineSymbolType.compare("StateMaProcedureStartStateSym") == 0) {
		StateMachineSymbolType = ClsStateMachineSymbol::procedure_start_state ;
	} else if (str_StateMachineSymbolType.compare("StateMaProcedureReturnSym") == 0) {
		StateMachineSymbolType = ClsStateMachineSymbol::procedure_return ;
	} else {
		#ifdef olSDL_DL_TWO
			DebugPrinter::print("error in: void olSDL::TclExt::TclModelAccess::determine_StateMachineSymbolType(...)") ;
			DebugPrinter::print("unknown Tcl-SymbolType") ;
		#endif
	}
} ;
//*******************************************************************************************************
void olSDL::TclExt::TclModelAccess::determine_SymbolType(const string str_SymbolType, EnumSym::Type & SymbolType) {

	SymbolType = EnumSym::none ;

	if (str_SymbolType.compare("TextSym") == 0) {
		SymbolType = EnumSym::TextSym ;
	} else if (str_SymbolType.compare("ProInstRefSym") == 0) {
		SymbolType = EnumSym::ProInstRefSym ;
	} else if (str_SymbolType.compare("ProTypeRefSym") == 0) {
		SymbolType = EnumSym::ProTypeRefSym ;
	} else if (str_SymbolType.compare("BlkInstRefSym") == 0) {
		SymbolType = EnumSym::BlkInstRefSym ;
	} else if (str_SymbolType.compare("BlkTypeRefSym") == 0) {
		SymbolType = EnumSym::BlkTypeRefSym ;
	} else if (str_SymbolType.compare("ProcedureInstRefSym") == 0) {
		SymbolType = EnumSym::ProcedureInstRefSym ;
	} else {
		#ifdef olSDL_DL_TWO
			DebugPrinter::print("error in: void olSDL::TclExt::TclModelAccess::determine_SymbolType(const string str_SymbolType, e_SymbolType & SymbolType)") ;
			DebugPrinter::print("unknown Tcl-SymbolType") ;
		#endif
	} ; // else if
} ;

//*******************************************************************************************************
//*******************************************************************************************************
void olSDL::TclExt::TclModelAccess::createValListScr(const type_ListOfPoi_to_ConstFlowLine & ModelResultList, string & ScriptPart) {

	int FlowLineID ;
	type_ListOfPoi_to_ConstFlowLine::const_iterator iter ;
	list<int> IntList ;

	ScriptPart = "" ;

	for ( iter = ModelResultList.begin() ; iter != ModelResultList.end() ; iter ++) {
		FlowLineID = (*iter)->get_Model_ID() ;
		IntList.push_back(FlowLineID) ;
	} ;

	createValListScr(IntList, ScriptPart) ;
} ;
//*******************************************************************************************************
void olSDL::TclExt::TclModelAccess::createValListScr(const type_ListOf_Poi_to_ConstChannel & ModelResultList , string & ScriptPart) {

	int Channel_ID ;
	type_ListOf_Poi_to_ConstChannel::const_iterator iter ;
	list<int> IntList ;

	ScriptPart = "" ;

	for ( iter = ModelResultList.begin() ; iter != ModelResultList.end() ; iter ++) {
		Channel_ID = (*iter)->get_Model_ID() ;
		IntList.push_back(Channel_ID) ;
	} ;

	createValListScr(IntList, ScriptPart) ;
} ;
//*******************************************************************************************************
void olSDL::TclExt::TclModelAccess::createValListScr(const type_ListOf_Poi_to_ConstDiagram & ModelResultList, string & ScriptPart) {

	int DiagramID ;
	type_ListOf_Poi_to_ConstDiagram::const_iterator iter ;
	list<int> IntList ;

	ScriptPart = "" ;

	for ( iter = ModelResultList.begin() ; iter != ModelResultList.end() ; iter ++) {
		DiagramID = (*iter)->get_Model_ID() ;
		IntList.push_back(DiagramID) ;
	} ;

	createValListScr(IntList, ScriptPart) ;
} ;
//*******************************************************************************************************
void olSDL::TclExt::TclModelAccess::createValListScr(const type_ListOf_Poi_to_ConstSym & ModelResultList , string & ScriptPart) {

	int Symbol_ID ;
	type_ListOf_Poi_to_ConstSym::const_iterator iter ;
	list<int> IntList ;

	ScriptPart = "" ;

	for ( iter = ModelResultList.begin() ; iter != ModelResultList.end() ; iter ++) {
		Symbol_ID = (*iter)->get_Model_ID() ;
		IntList.push_back(Symbol_ID) ;
	} ;

	createValListScr(IntList, ScriptPart) ;
} ;
//*******************************************************************************************************
void olSDL::TclExt::TclModelAccess::createValListScr(const type_ListOfPoi_to_ConstStateMachineSymbol & ModelResultList , string & ScriptPart) {

	int Symbol_ID ;
	type_ListOfPoi_to_ConstStateMachineSymbol::const_iterator iter ;
	list<int> IntList ;

	ScriptPart = "" ;

	for ( iter = ModelResultList.begin() ; iter != ModelResultList.end() ; iter ++) {
		Symbol_ID = (*iter)->get_Model_ID() ;
		IntList.push_back(Symbol_ID) ;
	} ;

	createValListScr(IntList, ScriptPart) ;
} ;
//*******************************************************************************************************
void olSDL::TclExt::TclModelAccess::createValListScr(const list<int> & IntList, string & ScriptPart) {

	list<int>::const_iterator iter ;

	ScriptPart = "" ;

	iter = IntList.begin() ;
	while (iter != IntList.end()) {

		ScriptPart.append(olSDL::Converter::Int_to_String((*iter))) ;

		iter++ ;
		if (iter != IntList.end()) {
			ScriptPart.append(" ") ;
		} ;
	} ;
} ;
//*******************************************************************************************************
void olSDL::TclExt::TclModelAccess::createValListScr(const ClsFlowLine* pFlowLine, string & ScriptPart) {

	list<int> IntList ;
	ScriptPart = "" ;
	pFlowLine->get_Coords_as_IntList(IntList) ;
	createValListScr(IntList, ScriptPart) ;
} ;
//*******************************************************************************************************
void olSDL::TclExt::TclModelAccess::createValListScr(const ClsStateMachineSymbol* pStateMachineSymbol, string & ScriptPart) {

	list<int> IntList ;
	ScriptPart = "" ;
	pStateMachineSymbol->get_PaS_as_IntList(IntList) ;
	createValListScr(IntList, ScriptPart) ;
} ;
//*******************************************************************************************************
void olSDL::TclExt::TclModelAccess::createValListScr(const Symbol* pSymbol, string & ScriptPart) {

	list<int> IntList ;
	ScriptPart = "" ;
	pSymbol->get_PaS_as_IntList(IntList) ;
	createValListScr(IntList, ScriptPart) ;
} ;
//*******************************************************************************************************
void olSDL::TclExt::TclModelAccess::createValListScr(const ClsChannel* pChannel, string & ScriptPart) {

	list<int> IntList ;
	ScriptPart = "" ;
	pChannel->get_Coords_as_IntList(IntList) ;
	createValListScr(IntList, ScriptPart) ;
} ;
//*******************************************************************************************************
void olSDL::TclExt::TclModelAccess::createValListScr(const string & Value, string & ScriptPart) {

	ScriptPart = Value;
} ;
//*******************************************************************************************************
void olSDL::TclExt::TclModelAccess::createValListScrForStrings(const string & Value, string & ScriptPart) {

	string str_ValueWithEscapeSequences ;

	str_ValueWithEscapeSequences = Converter::StringToMAR_String(Value) ;
	ScriptPart = str_ValueWithEscapeSequences;
} ;
//*******************************************************************************************************
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::OneDimVarSet (Tcl_Interp *interp, 
	const string & NameTclVar, const string & IndexName, const string & Value) {
	int result=TCL_OK;
	char *NameString ;
	char *IndexString ;
	char *ValueString ;

	NameString = new char[NameTclVar.length()+1] ;
	NameTclVar.copy(NameString,string::npos);
	NameString[NameTclVar.length()] = 0;
	IndexString = new char[IndexName.length()+1] ;
	IndexName.copy(IndexString,string::npos);
	IndexString[IndexName.length()] = 0;
	ValueString = new char[Value.length()+1] ;
	Value.copy(ValueString,string::npos);
	ValueString[Value.length()] = 0;

	if (!Tcl_SetVar2(interp, NameString, IndexString, ValueString, 0))
		result=TCL_ERROR;
		
	delete [ ] NameString ;
	delete [ ] IndexString ;
	delete [ ] ValueString ;
	return result;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::OneDimVarSet (Tcl_Interp *interp,
	const string & NameTclVar, const string & IndexName, const int Value) {

	return OneDimVarSet(interp, NameTclVar, IndexName, olSDL::Converter::Int_to_String(Value));
};
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::TwoDimVarSet (Tcl_Interp *interp, const string & NameTclVar, const int IndexA,
									                  const string & IndexNameB, const string & Value) {

	return OneDimVarSet(interp, NameTclVar, olSDL::Converter::Int_to_String(IndexA)+","+IndexNameB, Value) ;
} ;
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::TwoDimVarSet (Tcl_Interp *interp, const string & NameTclVar, const string & IndexNameA,
									                  const string & IndexNameB, const string & Value) {

	return OneDimVarSet(interp, NameTclVar, IndexNameA+","+IndexNameB, Value) ;
} ;
//*******************************************************************************************************
//*******************************************************************************************************
int olSDL::TclExt::TclModelAccess::clear_MAR_ARR(Tcl_Interp *interp)  {

	char *pTempString ;

	pTempString = strdup(pMAR_Array_Name);

	if (Tcl_UnsetVar(interp, pTempString, 0) != TCL_OK)
	{
		Tcl_SetResult(interp, "Script error", TCL_VOLATILE) ;
		return TCL_ERROR ;
	}

	if (!Tcl_SetVar2(interp, pTempString, "", "{ }", 0))
	{
		Tcl_SetResult(interp, "Script error", TCL_VOLATILE) ;
		return TCL_ERROR ;
	}

	free(pTempString);

	return TCL_OK ;
} ;
//******************************************************************************
bool olSDL::TclExt::TclModelAccess::AccessOK(Tcl_Interp *interp, const ClsMoAccessInfo & AccessInfo, const ClsMoAccessInfo::e_ResultType ResultOK) {
	if (AccessInfo.get_Info() != ResultOK) {
		Tcl_SetResult(interp, AccessInfo.get_Info_Message(), TCL_VOLATILE) ;
		return false;
	} else {
		return true;
	}
}
//******************************************************************************

