#include <iostream>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/parsers/DOMParser.hpp>

#include "XML_Reader.h"
#include "Converter.h"
#include "AppVars.h"


const string olSDL::ClsXML_Reader_ErrorReporter::NOT_SPECIFIED = "not specified" ;
const string olSDL::ClsXML_Reader::NOT_SPECIFIED = "not specified" ;

extern olSDL::ClassAppVars AppVars ;


//*******************************************************************************************************
ostream& operator<< (ostream& target, const DOMString& s)
{
    char *p = s.transcode();
    target << p;
    delete [] p;
    return target;
}
//*******************************************************************************************************
//*******************************************************************************************************
//*******************************************************************************************************
olSDL::ClsXML_Reader_ErrorReporter::ClsXML_Reader_ErrorReporter() {

	m_ErrorType = not_specified ;
} ;
//*******************************************************************************************************
olSDL::ClsXML_Reader_ErrorReporter::ClsXML_Reader_ErrorReporter(e_ErrorType ErrorType) {

	m_ErrorType = ErrorType ;
} ;
//*******************************************************************************************************
olSDL::ClsXML_Reader_ErrorReporter::ClsXML_Reader_ErrorReporter(e_ErrorType ErrorType, const string strParserErrorMessage) {

	m_ErrorType = ErrorType ;
	m_strParserErrorMessage = strParserErrorMessage ;
} ;
//*******************************************************************************************************
olSDL::ClsXML_Reader_ErrorReporter::e_ErrorType olSDL::ClsXML_Reader_ErrorReporter::get_ErrorType() const {

	return m_ErrorType ;
} ;
//*******************************************************************************************************
string olSDL::ClsXML_Reader_ErrorReporter::get_ErrorMessage() const {

	if (m_ErrorType == DOM_Parser) {

		if (m_strParserErrorMessage == "") {
			return NOT_SPECIFIED ;
		} else {
			return m_strParserErrorMessage ;
		} ;

	} else {
		return NOT_SPECIFIED ;
	} ;
} ;
//*******************************************************************************************************
//*******************************************************************************************************
//*******************************************************************************************************
olSDL::DOMTreeErrorReporter::DOMTreeErrorReporter() :	 fSawErrors(false) {

} ;
//*******************************************************************************************************
olSDL::DOMTreeErrorReporter::~DOMTreeErrorReporter() {

} ;
//*******************************************************************************************************
void olSDL::DOMTreeErrorReporter::warning(const SAXParseException& toCatch) {

    //
    // Ignore all warnings.
    //
} ;
//*******************************************************************************************************
void olSDL::DOMTreeErrorReporter::error(const SAXParseException& toCatch) {

	fSawErrors = true ;

	m_strErrorMessage = "Error at file: \n" ;
	m_strErrorMessage.append("\"") ;
	m_strErrorMessage.append( DOMStringToStdString(DOMString(toCatch.getSystemId())) ) ;
	m_strErrorMessage.append("\"") ;
	m_strErrorMessage.append("\n" ) ;

	m_strErrorMessage.append("line: ") ;
	m_strErrorMessage.append( Converter::Int_to_String(toCatch.getLineNumber()) ) ;
	m_strErrorMessage.append("\n" ) ;

	m_strErrorMessage.append("column: ") ;
	m_strErrorMessage.append( Converter::Int_to_String(toCatch.getColumnNumber()) ) ;
	m_strErrorMessage.append("\n" ) ;

	m_strErrorMessage.append("Message: \n") ;
	m_strErrorMessage.append( DOMStringToStdString(DOMString(toCatch.getMessage())) ) ;

} ;
//*******************************************************************************************************
void olSDL::DOMTreeErrorReporter::fatalError(const SAXParseException& toCatch) {

	fSawErrors = true ;

	m_strErrorMessage = "Fatal Error at file: \n" ;
	m_strErrorMessage.append("\"") ;
	m_strErrorMessage.append( DOMStringToStdString(DOMString(toCatch.getSystemId())) ) ;
	m_strErrorMessage.append("\"") ;
	m_strErrorMessage.append("\n" ) ;

	m_strErrorMessage.append("line: ") ;
	m_strErrorMessage.append( Converter::Int_to_String(toCatch.getLineNumber()) ) ;
	m_strErrorMessage.append("\n" ) ;

	m_strErrorMessage.append("column: ") ;
	m_strErrorMessage.append( Converter::Int_to_String(toCatch.getColumnNumber()) ) ;
	m_strErrorMessage.append("\n" ) ;

	m_strErrorMessage.append("Message: \n") ;
	m_strErrorMessage.append( DOMStringToStdString(DOMString(toCatch.getMessage())) ) ;

} ;
//*******************************************************************************************************
void olSDL::DOMTreeErrorReporter::resetErrors() {

	// No-op in this case
} ;
//*******************************************************************************************************
bool olSDL::DOMTreeErrorReporter::getSawErrors() const {

	return fSawErrors;
} ;
//*******************************************************************************************************
string olSDL::DOMTreeErrorReporter::get_ErrorMessage() const {

	return m_strErrorMessage ;
} ;
//*******************************************************************************************************
string olSDL::DOMTreeErrorReporter::DOMStringToStdString(const DOMString& s) {

	char * pTempString = s.transcode() ;

	try
	{
		string Result(pTempString) ;
		delete [] pTempString ;
		return Result ;

	} catch( ... ) {

                delete [] pTempString;
                throw ;
        } ;
} ;
//*******************************************************************************************************
//*******************************************************************************************************
//*******************************************************************************************************
olSDL::ClsXML_Reader::ClsXML_Reader() {

	m_pSysSpec = NULL ;
} ;
//*******************************************************************************************************
olSDL::ClsXML_Reader::~ClsXML_Reader() {

} ;
//*******************************************************************************************************
void olSDL::ClsXML_Reader::read_SysSpec(const string & FileName, SysSpec * pSysSpec) {

	string str_ErrorInfo ;

	m_FileName = FileName ;

	DOMParser parser ;
	DOMTreeErrorReporter errReporter ;

    	parser.setValidationScheme(DOMParser::Val_Always);
    	parser.setErrorHandler(&errReporter);

    	bool errorsOccured = false ;


	try
    	{
		string str_Dummy_XML_File = AppVars.get_Dummy_XML_File() ;

        	parser.parse(str_Dummy_XML_File.c_str());
        	int errorCount = parser.getErrorCount();
        	if (errorCount > 0) {
			str_ErrorInfo = NOT_SPECIFIED ;
			errorsOccured = true ;
		} ;
    	}
    	catch (...)
    	{
		str_ErrorInfo = "An error occured during parsing init XML file\n " ;

        	errorsOccured = true ;
    	} ;


	// =================================================================
	// =================================================================

    	if (!errorsOccured && !errReporter.getSawErrors())
    	{
		// =================================================================

		try
	    	{
	        	parser.parse(m_FileName.c_str(), true);
	        	int errorCount = parser.getErrorCount();
	        	if (errorCount > 0) {
				str_ErrorInfo = NOT_SPECIFIED ;
				errorsOccured = true ;
			} ;
	    	}
	    	catch (const XMLException& e)
	    	{
			str_ErrorInfo = "An error occured during parsing\n   Message: " ;
			str_ErrorInfo = str_ErrorInfo + DOMStringToStdString(DOMString(e.getMessage())) ;
			str_ErrorInfo = str_ErrorInfo + "\n" ;

 		       	errorsOccured = true ;
	    	}
	    	catch (const DOM_DOMException& e)
	    	{
			str_ErrorInfo = "A DOM error occured during parsing\n   DOMException" ;
			str_ErrorInfo = str_ErrorInfo + "\n" ;

			errorsOccured = true ;
	    	}
	    	catch (...)
	    	{
			str_ErrorInfo = "An error occured during parsing\n " ;

	        	errorsOccured = true ;
	    	} ;

		// =================================================================

	} ; // if (!errorsOccured && !errReporter.getSawErrors())

	// =================================================================
	// =================================================================


	DOM_Node node_SysSpec ;
	DOM_Element element_SysSpec ;
	DOM_Node node_SysDia ;

	// If the parse was successful, create SysSpec
    	if (!errorsOccured && !errReporter.getSawErrors())
    	{
		m_pSysSpec = pSysSpec ;

        	DOM_Document doc = parser.getDocument();

		node_SysSpec = doc.getElementsByTagName(DOMString(TN_SYSSPEC.c_str())).item(0) ;
		element_SysSpec = (DOM_Element&) node_SysSpec ;

		node_SysDia = element_SysSpec.getElementsByTagName(DOMString(TN_SYSDIA.c_str())).item(0) ;

		read_SysDia(doc, node_SysDia) ;

	} else {

		if ( errReporter.getSawErrors() ) {

			throw ClsXML_Reader_ErrorReporter(ClsXML_Reader_ErrorReporter::DOM_Parser, errReporter.get_ErrorMessage()) ;
		} else {

			throw ClsXML_Reader_ErrorReporter(ClsXML_Reader_ErrorReporter::DOM_Parser, str_ErrorInfo) ;
		} ;

	} ; // if (!errorsOccured && !errReporter.getSawErrors())

} ;

//*******************************************************************************************************
//*******************************************************************************************************
// diagrams
//*******************************************************************************************************
//*******************************************************************************************************
void olSDL::ClsXML_Reader::read_SysDia(DOM_Document doc, DOM_Node & Node) throw (ClsXML_Reader_ErrorReporter) {

	ClsMoAccessInfo Info ;

	string DiagramName ;
	int DiagramID ;

	DOM_NamedNodeMap attributes ;
	DOM_Node attribute ;

	DOM_Element ElementNodeSysDia ;

	DOM_Node temp_NodeDiagramName ;


	if ( ! Node.getNodeName().equals(DOMString(TN_SYSDIA.c_str())) ) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	ElementNodeSysDia = (DOM_Element&) Node ;

	// ==========
	// get DiagramID

	attributes = Node.getAttributes();
        attribute = attributes.getNamedItem(DOMString(AN_ID.c_str()));

	DiagramID = get_IntValue_from_Attribute(attribute) ;

	// add to RenumberingHelper
	add_ID_Pair(DiagramID, m_pSysSpec->get_System_Model_ID()) ;

	// END: get DiagramID
	// ==========

	// ==========
	// get Diagram Name
	temp_NodeDiagramName = ElementNodeSysDia.getElementsByTagName(DOMString(TN_DIANAME.c_str())).item(0) ;
	DiagramName = get_StringContent_from_Element(temp_NodeDiagramName) ;

	m_pSysSpec->modify_SysDiaName(DiagramName, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	// END: get Diagram Name
	// ==========

	// ==========
	// pages

	read_Pages(doc, Node) ;

	// END: pages
	// ==========

	// ========================================
	// RecSyms

	read_RecSymbol(doc, ElementNodeSysDia, TN_RECSYM_TEXTSYM) ;

	read_RecSymbol(doc, ElementNodeSysDia, TN_RECSYM_BLK_INST_REF) ;

	read_RecSymbol(doc, ElementNodeSysDia, TN_RECSYM_BLK_TYPE_REF) ;

	read_RecSymbol(doc, ElementNodeSysDia, TN_RECSYM_PRO_INST_REF) ;

	read_RecSymbol(doc, ElementNodeSysDia, TN_RECSYM_PRO_TYPE_REF) ;

	// END: RecSyms
	// ========================================

	// ========================================
	// Channel

	read_Channel(doc, ElementNodeSysDia) ;

	// Channel
	// ========================================
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Reader::read_BlkDia(DOM_Document doc, DOM_Node & Node) {

	if ( ! Node.getNodeName().equals(DOMString(TN_BLKDIA.c_str())) ) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	// ==========
	// pages

	read_Pages(doc, Node) ;

	// END: pages
	// ==========

	// ========================================
	// RecSyms

	read_RecSymbol(doc, Node, TN_RECSYM_TEXTSYM) ;

	read_RecSymbol(doc, Node, TN_RECSYM_BLK_INST_REF) ;

	read_RecSymbol(doc, Node, TN_RECSYM_BLK_TYPE_REF) ;

	read_RecSymbol(doc, Node, TN_RECSYM_PRO_INST_REF) ;

	read_RecSymbol(doc, Node, TN_RECSYM_PRO_TYPE_REF) ;

	// END: RecSyms
	// ========================================

	// ========================================
	// Channel

	read_Channel(doc, Node) ;

	// Channel
	// ========================================
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Reader::read_ProDia(DOM_Document doc, DOM_Node & Node) {

	if ( ! Node.getNodeName().equals(DOMString(TN_PRODIA.c_str())) ) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	// ==========
	// pages

	read_Pages(doc, Node) ;

	// END: pages
	// ==========

	// ========================================
	// RecSyms

	read_RecSymbol(doc, Node, TN_RECSYM_TEXTSYM) ;

	read_RecSymbol(doc, Node, TN_RECSYM_PRO_INST_REF) ;

	read_RecSymbol(doc, Node, TN_RECSYM_PRO_TYPE_REF) ;

	read_RecSymbol(doc, Node, TN_RECSYM_PROCEDURE_INST_REF) ;

	// END: RecSyms
	// ========================================

	// ========================================
	// Channel

	read_Channel(doc, Node) ;

	// END: Channel
	// ========================================

	// ========================================
	// StateMaSym

	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_START) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_STATE) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_OUTPUT) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_INPUT) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_CONDITION ) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_STOP) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_TASK) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_DECISION) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_CREATE_PRO) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_CALL_PROCEDURE) ;

	// END: StateMaSym
	// ========================================

	// ========================================
	// FlowLines

	read_FlowLines(doc, Node) ;

	// END: FlowLines
	// ========================================
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Reader::read_ProcedureDia(DOM_Document doc, DOM_Node & Node) {

	if ( ! Node.getNodeName().equals(DOMString(TN_PROCEDUREDIA.c_str())) ) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	read_ProcedureFormalParameters(doc, Node) ;

	// ==========
	// pages

	read_Pages(doc, Node) ;

	// END: pages
	// ==========

	// ========================================
	// RecSyms

	read_RecSymbol(doc, Node, TN_RECSYM_TEXTSYM) ;

	// read_RecSymbol(doc, Node, TN_RECSYM_PROCEDURE_INST_REF) ;

	// END: RecSyms
	// ========================================

	// ========================================
	// StateMaSym

	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_START) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_STATE) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_OUTPUT) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_INPUT) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_CONDITION) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_STOP) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_TASK) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_DECISION) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_CREATE_PRO) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_CALL_PROCEDURE) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_PROCEDURE_START_STATE) ;
	read_StateMaSym(doc, Node, TN_STATE_MA_SYM_PROCEDURE_RETURN) ;

	// END: StateMaSym
	// ========================================

	// ========================================
	// FlowLines

	read_FlowLines(doc, Node) ;

	// END: FlowLines
	// ========================================
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Reader::read_ReferencedDiagram(DOM_Document doc, DOM_Node & Node, const Symbol * pSymbol) throw (ClsXML_Reader_ErrorReporter) {

	const Diagram * pDiagram ;

	if ( ! pSymbol->is_referencing_Symbol() ) {
		return ;
	} ;

	pDiagram = pSymbol->get_referenced_Diagram() ;

	switch (pDiagram->get_DiagramType()) {

		case EnumDia::BlkDia:
		{
			read_BlkDia(doc, Node) ;
			break ;
		} ;
		case EnumDia::ProDia:
		{
			read_ProDia(doc, Node) ;
			break ;
		}
		case EnumDia::ProcedureDia:
		{
			read_ProcedureDia(doc, Node) ;
			break ;
		}
		default:
		{
			throw ClsXML_Reader_ErrorReporter() ;
		}
	} ; // switch (pDiagram->get_DiagramType())
} ;
//*******************************************************************************************************
//*******************************************************************************************************
// FlowLines
//*******************************************************************************************************
//*******************************************************************************************************
void olSDL::ClsXML_Reader::read_FlowLines(DOM_Document doc, DOM_Node & Node) throw (ClsXML_Reader_ErrorReporter) {

	ClsMoAccessInfo Info ;

	DOM_NamedNodeMap attributes ;
	DOM_Node attribute ;

	DOM_Element DiagramElementNode ;

	int OldDiagramID ;

	int New_Model_ID ;

	int FlowLineID ;
	int ParentPageID ;
	list<int> FlowLineCoords ;
	int FlowLineSourceModel_ID ;
	int FlowLineDestModel_ID ;

	DOM_Node temp_FlowLineNode ;
	DOM_NodeList FlowLinesList ;
	int cntFlowLines ;


	if (Node.getNodeType() != DOM_Node::ELEMENT_NODE) { throw ClsXML_Reader_ErrorReporter(); } ;

	if ( ! (Node.getNodeName().equals(DOMString(TN_PRODIA.c_str())) ||
		Node.getNodeName().equals(DOMString(TN_PROCEDUREDIA.c_str())))   ) { throw ClsXML_Reader_ErrorReporter(); } ;

	// ==========
	// get DiagramID

	attributes = Node.getAttributes();
        attribute = attributes.getNamedItem(DOMString(AN_ID.c_str()));

	OldDiagramID = get_IntValue_from_Attribute(attribute) ;

	// END: get DiagramID
	// ==========

	DiagramElementNode = (DOM_Element&) Node ;

	// ==========
	// normal flow line

	FlowLinesList = DiagramElementNode.getElementsByTagName(DOMString(TN_NORMAL_FLOWLINE.c_str())) ;
	cntFlowLines = FlowLinesList.getLength() ;

	for (int k=0; k<cntFlowLines; k++) {

		temp_FlowLineNode = FlowLinesList.item(k) ;

		get_NormalFlowLine_from_ElementNode(temp_FlowLineNode, FlowLineID, ParentPageID,
			FlowLineCoords, FlowLineSourceModel_ID, FlowLineDestModel_ID) ;

		m_pSysSpec->insert_FlowLine(get_New_ID_from_Old_ID(OldDiagramID), get_New_ID_from_Old_ID(ParentPageID),
			FlowLineCoords, get_New_ID_from_Old_ID(FlowLineSourceModel_ID), get_New_ID_from_Old_ID(FlowLineDestModel_ID), Info, New_Model_ID) ;


		if (Info.get_Info() != ClsMoAccessInfo::insert_OK) {
			throw ClsXML_Reader_ErrorReporter() ;
		} ;

		// add to RenumberingHelper
		add_ID_Pair(FlowLineID, New_Model_ID) ;

	} ; // for (int k=0; k<cntFlowLines; k++)

	// END: normal flow line
	// ==========

	// ==========
	// decision flow lines

	FlowLinesList = DiagramElementNode.getElementsByTagName(DOMString(TN_DECISION_FLOWLINE.c_str())) ;
	cntFlowLines = FlowLinesList.getLength() ;

	ClsRecPaS DecisionFlowLinePaS(0,0,10,10) ;
	string DecisionFlowLineContent ;

	for (int k=0; k<cntFlowLines; k++) {

		temp_FlowLineNode = FlowLinesList.item(k) ;

		get_DecisionFlowLine_from_ElementNode(temp_FlowLineNode, FlowLineID, ParentPageID,
			FlowLineCoords, FlowLineSourceModel_ID, FlowLineDestModel_ID, DecisionFlowLineContent, DecisionFlowLinePaS) ;

		m_pSysSpec->insert_DecisionFlowLine(get_New_ID_from_Old_ID(OldDiagramID), get_New_ID_from_Old_ID(ParentPageID),
				FlowLineCoords, get_New_ID_from_Old_ID(FlowLineSourceModel_ID), get_New_ID_from_Old_ID(FlowLineDestModel_ID),
				DecisionFlowLineContent, DecisionFlowLinePaS, Info, New_Model_ID) ;

		if (Info.get_Info() != ClsMoAccessInfo::insert_OK) {
			throw ClsXML_Reader_ErrorReporter() ;
		} ;

		// add to RenumberingHelper
		add_ID_Pair(FlowLineID, New_Model_ID) ;

	} ; // for (int k=0; k<cntFlowLines; k++)


	// END: decision flow lines
	// ==========

} ;
//*******************************************************************************************************
void olSDL::ClsXML_Reader::get_DecisionFlowLine_from_ElementNode (const DOM_Node & Node, int & FlowLineID, int & ParentPageID,
		list<int> & FlowLineCoords, int & FlowLineSourceModel_ID, int & FlowLineDestModel_ID,
		string & DecisionFlowLineContent, ClsRecPaS & DecisionFlowLinePaS) throw (ClsXML_Reader_ErrorReporter) {

	DOM_NamedNodeMap attributes ;

	DOM_Node attributeID ;
	DOM_Node attributeParentPageID ;

	DOM_Node temp_Node ;
	DOM_Element ElementNode ;

	string CoordsAsString ;

	int x_left_up ;
	int y_left_up ;
	int Width ;
	int Height ;


	if ( ! Node.getNodeName().equals(DOMString(TN_DECISION_FLOWLINE.c_str())) ) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

 	attributes = Node.getAttributes();
        attributeID = attributes.getNamedItem(DOMString(AN_ID.c_str()));
	attributeParentPageID = attributes.getNamedItem(DOMString(AN_PARENT_PAGE_ID.c_str()));

	FlowLineID = get_IntValue_from_Attribute(attributeID) ;
	ParentPageID = get_IntValue_from_Attribute(attributeParentPageID) ;

	ElementNode = (DOM_Element&) Node ;

	// ==========
	// Source and Dest IDs

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_FLOWLINE_SOURCE_ID.c_str())).item(0) ;

	FlowLineSourceModel_ID = get_IntContent_from_Element(temp_Node)  ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_FLOWLINE_DESTINATION_ID.c_str())).item(0) ;

	FlowLineDestModel_ID = get_IntContent_from_Element(temp_Node)  ;

	// END: Source and Dest IDs
	// ==========

	// ==========
	// Coords

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_COORDS.c_str())).item(0) ;
	CoordsAsString = get_StringContent_from_Element(temp_Node) ;

	if ( ! Converter::IntListString_To_IntList(CoordsAsString, FlowLineCoords) ) {
		throw ClsXML_Reader_ErrorReporter(ClsXML_Reader_ErrorReporter::integer_list_expected) ;
	} ;

	// END: Coords
	// ==========

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_PAS.c_str())).item(0) ;
	get_PaS_from_ElementNode(temp_Node, x_left_up, y_left_up, Width, Height)  ;

	DecisionFlowLinePaS = ClsRecPaS(x_left_up, y_left_up, Width, Height) ;


	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_CONTENT.c_str())).item(0) ;
	DecisionFlowLineContent = get_StringContent_from_Element(temp_Node) ;

} ;
//*******************************************************************************************************
void olSDL::ClsXML_Reader::get_NormalFlowLine_from_ElementNode (const DOM_Node & Node, int & FlowLineID, int & ParentPageID,
		list<int> & FlowLineCoords, int & FlowLineSourceModel_ID, int & FlowLineDestModel_ID) throw (ClsXML_Reader_ErrorReporter) {

	DOM_NamedNodeMap attributes ;

	DOM_Node attributeID ;
	DOM_Node attributeParentPageID ;

	DOM_Node temp_Node ;
	DOM_Element ElementNode ;

	string CoordsAsString ;

	if ( ! Node.getNodeName().equals(DOMString(TN_NORMAL_FLOWLINE.c_str())) ) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

 	attributes = Node.getAttributes();
        attributeID = attributes.getNamedItem(DOMString(AN_ID.c_str()));
	attributeParentPageID = attributes.getNamedItem(DOMString(AN_PARENT_PAGE_ID.c_str()));

	FlowLineID = get_IntValue_from_Attribute(attributeID) ;
	ParentPageID = get_IntValue_from_Attribute(attributeParentPageID) ;

	ElementNode = (DOM_Element&) Node ;

	// ==========
	// Source and Dest IDs

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_FLOWLINE_SOURCE_ID.c_str())).item(0) ;

	FlowLineSourceModel_ID = get_IntContent_from_Element(temp_Node)  ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_FLOWLINE_DESTINATION_ID.c_str())).item(0) ;

	FlowLineDestModel_ID = get_IntContent_from_Element(temp_Node)  ;

	// END: Source and Dest IDs
	// ==========

	// ==========
	// Coords

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_COORDS.c_str())).item(0) ;
	CoordsAsString = get_StringContent_from_Element(temp_Node) ;

	if ( ! Converter::IntListString_To_IntList(CoordsAsString, FlowLineCoords) ) {
		throw ClsXML_Reader_ErrorReporter(ClsXML_Reader_ErrorReporter::integer_list_expected) ;
	} ;

	// END: Coords
	// ==========
} ;
//*******************************************************************************************************
//*******************************************************************************************************
// StateMaSym
//*******************************************************************************************************
//*******************************************************************************************************
void olSDL::ClsXML_Reader::read_StateMaSym(DOM_Document doc, DOM_Node & Node, const string & str_StateMaSymType)
			throw (ClsXML_Reader_ErrorReporter) {

	ClsMoAccessInfo Info ;

	DOM_NamedNodeMap attributes ;
	DOM_Node  attributeID ;

	int OldDiagramID ;

	DOM_Element DiagramElementNode ;

	int cntNode ;
	DOM_Node temp_Node ;
	DOM_NodeList NodeList ;

	int New_Model_ID ;

	int StateMaSym_ID ;
	int ParentPageID ;
	int x_left_up ;
	int y_left_up ;
	int Width ;
	int Height ;
	string SymbolContent ;


	if (Node.getNodeType() != DOM_Node::ELEMENT_NODE) { throw ClsXML_Reader_ErrorReporter(); } ;

	if ( ! ( Node.getNodeName().equals(DOMString(TN_PRODIA.c_str())) ||
		Node.getNodeName().equals(DOMString(TN_PROCEDUREDIA.c_str())))  ) { throw ClsXML_Reader_ErrorReporter(); } ;

	// ==========
	// get DiagramID

	attributes = Node.getAttributes();
        attributeID = attributes.getNamedItem(DOMString(AN_ID.c_str()));

	OldDiagramID = get_IntValue_from_Attribute(attributeID) ;

	// END: get DiagramID
	// ==========

	ClsStateMachineSymbol::e_StateMachineSymbolType StateMachineSymbolType ;

	if (str_StateMaSymType.compare(TN_STATE_MA_SYM_START) == 0) {

		StateMachineSymbolType = ClsStateMachineSymbol::start ;

	} else if (str_StateMaSymType.compare(TN_STATE_MA_SYM_STATE) == 0) {

		StateMachineSymbolType = ClsStateMachineSymbol::state ;

	} else if (str_StateMaSymType.compare(TN_STATE_MA_SYM_OUTPUT) == 0) {

		StateMachineSymbolType = ClsStateMachineSymbol::output ;

	} else if (str_StateMaSymType.compare(TN_STATE_MA_SYM_INPUT) == 0) {

		StateMachineSymbolType = ClsStateMachineSymbol::input ;

	} else if (str_StateMaSymType.compare(TN_STATE_MA_SYM_CONDITION) == 0) {

		StateMachineSymbolType = ClsStateMachineSymbol::condition ;

	} else if (str_StateMaSymType.compare(TN_STATE_MA_SYM_STOP) == 0) {

		StateMachineSymbolType = ClsStateMachineSymbol::stop ;

	} else if (str_StateMaSymType.compare(TN_STATE_MA_SYM_TASK) == 0) {

		StateMachineSymbolType = ClsStateMachineSymbol::task ;

	} else if (str_StateMaSymType.compare(TN_STATE_MA_SYM_DECISION) == 0) {

		StateMachineSymbolType = ClsStateMachineSymbol::decision ;

	} else if (str_StateMaSymType.compare(TN_STATE_MA_SYM_CREATE_PRO) == 0) {

		StateMachineSymbolType = ClsStateMachineSymbol::create_pro ;

	} else if (str_StateMaSymType.compare(TN_STATE_MA_SYM_CALL_PROCEDURE) == 0) {

		StateMachineSymbolType = ClsStateMachineSymbol::call_procedure ;

	} else if (str_StateMaSymType.compare(TN_STATE_MA_SYM_PROCEDURE_START_STATE) == 0) {

		StateMachineSymbolType = ClsStateMachineSymbol::procedure_start_state ;

	} else if (str_StateMaSymType.compare(TN_STATE_MA_SYM_PROCEDURE_RETURN) == 0) {

		StateMachineSymbolType = ClsStateMachineSymbol::procedure_return ;

	} else {

		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	DiagramElementNode = (DOM_Element&) Node ;

	NodeList = DiagramElementNode.getElementsByTagName(DOMString(str_StateMaSymType.c_str())) ;
	cntNode = NodeList.getLength() ;

	for (int k=0; k<cntNode; k++) {

		temp_Node = NodeList.item(k) ;

		get_StateMaSym_from_ElementNode (temp_Node, StateMaSym_ID, ParentPageID,
				x_left_up, y_left_up, Width, Height, SymbolContent)  ;

		m_pSysSpec->insert_StateMachineSymbol(StateMachineSymbolType, get_New_ID_from_Old_ID(OldDiagramID),
			get_New_ID_from_Old_ID(ParentPageID), x_left_up, y_left_up, Width, Height, SymbolContent, Info, New_Model_ID) ;


		if (Info.get_Info() != ClsMoAccessInfo::insert_OK) {
			throw ClsXML_Reader_ErrorReporter() ;
		} ;

		// add to RenumberingHelper
		add_ID_Pair(StateMaSym_ID, New_Model_ID) ;

	} ; // for (int k=0; k<cntNode; k++)
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Reader::get_StateMaSym_from_ElementNode (const DOM_Node & Node, int & StateMaSym_ID, int & ParentPageID,
			int & x_left_up, int & y_left_up, int & Width, int & Height, string & SymbolContent) throw (ClsXML_Reader_ErrorReporter)  {

	DOM_Node temp_Node ;
	DOM_Element ElementNode ;

	DOM_NamedNodeMap attributes ;
	DOM_Node attributeID ;
	DOM_Node attributeParentPageID ;


	if ( ! ( Node.getNodeName().equals(DOMString(TN_STATE_MA_SYM_START.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_STATE_MA_SYM_STATE.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_STATE_MA_SYM_OUTPUT.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_STATE_MA_SYM_INPUT.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_STATE_MA_SYM_CONDITION.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_STATE_MA_SYM_STOP.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_STATE_MA_SYM_DECISION.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_STATE_MA_SYM_TASK.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_STATE_MA_SYM_CREATE_PRO.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_STATE_MA_SYM_CALL_PROCEDURE.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_STATE_MA_SYM_PROCEDURE_START_STATE.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_STATE_MA_SYM_PROCEDURE_RETURN.c_str()))  ) ) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;


	ElementNode = (DOM_Element&) Node ;

 	attributes = Node.getAttributes();

        attributeID = attributes.getNamedItem(DOMString(AN_ID.c_str()));
	attributeParentPageID = attributes.getNamedItem(DOMString(AN_PARENT_PAGE_ID.c_str()));

	StateMaSym_ID = get_IntValue_from_Attribute(attributeID) ;
	ParentPageID = get_IntValue_from_Attribute(attributeParentPageID) ;


	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_PAS.c_str())).item(0) ;
	get_PaS_from_ElementNode(temp_Node, x_left_up, y_left_up, Width, Height)  ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_CONTENT.c_str())).item(0) ;
	SymbolContent = get_StringContent_from_Element(temp_Node) ;
} ;
//*******************************************************************************************************
//*******************************************************************************************************
// RecSyms
//*******************************************************************************************************
//*******************************************************************************************************
void olSDL::ClsXML_Reader::read_RecSymbol(DOM_Document doc, DOM_Node & Node, const string & str_SymbolType) throw (ClsXML_Reader_ErrorReporter) {

	ClsMoAccessInfo Info ;

	DOM_Element DiagramElementNode ;
	DOMString StringRecSymType ;

	int cntNode ;
	DOM_Node temp_Node ;
	DOM_NodeList NodeList ;

	DOM_NamedNodeMap attributes ;
	DOM_Node attribute ;

	EnumSym::Type SymbolType ;

	const Symbol * pNewSymbol ;

	int New_Model_ID ;
	int OldDiagramID ;

	int Symbol_ID ;
	int ParentPageID ;
	int ReferencedDiagramID ;
	int x_left_up ;
	int y_left_up ;
	int Width ;
	int Height ;
	string SymbolContent ;


	if (Node.getNodeType() != DOM_Node::ELEMENT_NODE) { throw ClsXML_Reader_ErrorReporter(); } ;

	if ( ! ( Node.getNodeName().equals(DOMString(TN_SYSDIA.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_BLKDIA.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_PRODIA.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_PROCEDUREDIA.c_str())) )) {

		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	// ==========
	// get DiagramID

	attributes = Node.getAttributes();
        attribute = attributes.getNamedItem(DOMString(AN_ID.c_str()));

	OldDiagramID = get_IntValue_from_Attribute(attribute) ;

	// END: get DiagramID
	// ==========


	if (str_SymbolType.compare(TN_RECSYM_TEXTSYM) == 0) {

		SymbolType = EnumSym::TextSym ;

	} else if (str_SymbolType.compare(TN_RECSYM_BLK_INST_REF) == 0) {

		SymbolType = EnumSym::BlkInstRefSym ;

	} else if (str_SymbolType.compare(TN_RECSYM_BLK_TYPE_REF) == 0) {

		SymbolType = EnumSym::BlkTypeRefSym ;

	} else if (str_SymbolType.compare(TN_RECSYM_PRO_INST_REF) == 0) {

		SymbolType = EnumSym::ProInstRefSym ;

	} else if (str_SymbolType.compare(TN_RECSYM_PRO_TYPE_REF) == 0) {

		SymbolType = EnumSym::ProTypeRefSym ;

	} else if (str_SymbolType.compare(TN_RECSYM_PROCEDURE_INST_REF) == 0) {

		SymbolType = EnumSym::ProcedureInstRefSym ;

	} else {

		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	DiagramElementNode = (DOM_Element&) Node ;


	NodeList = DiagramElementNode.getElementsByTagName(DOMString(str_SymbolType.c_str())) ;
	cntNode = NodeList.getLength() ;

	for (int k=0; k<cntNode; k++) {

		temp_Node = NodeList.item(k) ;

		get_RecSym_from_ElementNode (temp_Node, Symbol_ID, ParentPageID, ReferencedDiagramID,
			x_left_up, y_left_up, Width, Height, SymbolContent) ;

		m_pSysSpec->insert_Symbol(SymbolType, get_New_ID_from_Old_ID(OldDiagramID), get_New_ID_from_Old_ID(ParentPageID),
			x_left_up, y_left_up, Width, Height, SymbolContent, Info, New_Model_ID) ;

		if (Info.get_Info() != ClsMoAccessInfo::insert_OK) {
			throw ClsXML_Reader_ErrorReporter() ;
		} ;

		// add to RenumberingHelper
		add_ID_Pair(Symbol_ID, New_Model_ID) ;

		// =======
		// recursion

		pNewSymbol = m_pSysSpec->get_RecSymbol(get_New_ID_from_Old_ID(Symbol_ID), Info) ;
		if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
			throw ClsXML_Reader_ErrorReporter() ;
		} ;

		if ( pNewSymbol->is_referencing_Symbol() ) {

			// add to RenumberingHelper
			add_ID_Pair(ReferencedDiagramID, pNewSymbol->get_referenced_Diagram_Model_ID()) ;

			DOM_Node NodeReferencedDiagram = get_DiagramElementNodeByID(doc, ReferencedDiagramID) ;

			read_ReferencedDiagram(doc, NodeReferencedDiagram, pNewSymbol) ;
		} ;

		// END: recursion
		// =======

	} ; // for (int k=0; k<cntNode; k++)

} ;
//*******************************************************************************************************
//*******************************************************************************************************
// pages
//*******************************************************************************************************
//*******************************************************************************************************
void olSDL::ClsXML_Reader::read_Pages(DOM_Document doc, DOM_Node & Node) throw (ClsXML_Reader_ErrorReporter) {

	ClsMoAccessInfo Info ;

	int cntPages ;
	int DiagramFirstPageID ;

	int PageID ;
	int PageWidth ;
	int PageHeight ;
	int x_left_up_Frame ;
	int y_left_up_Frame ;
	int FrameWidth ;
	int FrameHeight ;

	DOM_Node temp_NodePage ;
	DOM_NodeList PageNodeList ;
	DOM_Element DiagramElementNode ;

	DOM_NamedNodeMap attributes ;
	DOM_Node attribute ;

	int NewDiagramID ;
	int OldDiagramID ;

	const Diagram * pDiagram ;

	if ( ! ( Node.getNodeName().equals(DOMString(TN_SYSDIA.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_BLKDIA.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_PRODIA.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_PROCEDUREDIA.c_str()))
		 )) {

		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	// ======
	attributes = Node.getAttributes();
        attribute = attributes.getNamedItem(DOMString(AN_ID.c_str()));

	OldDiagramID = get_IntValue_from_Attribute(attribute) ;
	NewDiagramID = get_New_ID_from_Old_ID(OldDiagramID) ;

	// ======
	pDiagram = m_pSysSpec->get_Diagram(NewDiagramID, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::get_OK) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	// ======
	DiagramElementNode = (DOM_Element&) Node ;

	PageNodeList = DiagramElementNode.getElementsByTagName(DOMString(TN_PAGE.c_str())) ;
	cntPages = PageNodeList.getLength() ;

	if (cntPages<1) { throw ClsXML_Reader_ErrorReporter(); } ;

	// ==========
	// get DiagramFirstPageID

	temp_NodePage = PageNodeList.item(0) ;

	get_Page_from_ElementNode(temp_NodePage, DiagramFirstPageID, PageWidth, PageHeight,
			x_left_up_Frame, y_left_up_Frame, FrameWidth, FrameHeight) ;

	// add to RenumberingHelper
	add_ID_Pair(DiagramFirstPageID, pDiagram->get_FirstPageID(Info)) ;

	if (Info.get_Info() != ClsMoAccessInfo::page_get_OK) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;
	// END: get DiagramFirstPageID
	// ==========


	for (int k=0; k<cntPages; k++) {

		if (k!=0) {

			temp_NodePage = PageNodeList.item(k) ;

			get_Page_from_ElementNode(temp_NodePage, PageID, PageWidth, PageHeight,
				x_left_up_Frame, y_left_up_Frame, FrameWidth, FrameHeight) ;

			m_pSysSpec->append_NewPage(NewDiagramID, Info) ;

			if (Info.get_Info() != ClsMoAccessInfo::page_insert_OK) {
				throw ClsXML_Reader_ErrorReporter() ;
			} ;

			// add to RenumberingHelper
			add_ID_Pair(PageID, m_pSysSpec->get_LastPageID(NewDiagramID, Info)) ;

			if (Info.get_Info() != ClsMoAccessInfo::page_get_OK) {
				throw ClsXML_Reader_ErrorReporter() ;
			} ;

		} else {

			temp_NodePage = PageNodeList.item(k) ;

			get_Page_from_ElementNode(temp_NodePage, PageID, PageWidth, PageHeight,
				x_left_up_Frame, y_left_up_Frame, FrameWidth, FrameHeight) ;
		} ;


		m_pSysSpec->modify_PageWidth(NewDiagramID, get_New_ID_from_Old_ID(PageID), PageWidth, Info) ;

		if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
			throw ClsXML_Reader_ErrorReporter() ;
		} ;

		m_pSysSpec->modify_PageHeight(NewDiagramID, get_New_ID_from_Old_ID(PageID), PageHeight, Info) ;

		if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
			throw ClsXML_Reader_ErrorReporter() ;
		} ;

		m_pSysSpec->modify_PageFramePaS(NewDiagramID, get_New_ID_from_Old_ID(PageID),
			x_left_up_Frame, y_left_up_Frame, FrameWidth, FrameHeight, Info) ;

		if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
			throw ClsXML_Reader_ErrorReporter() ;
		} ;

	} ; // for (int k=1; k<cntPages; k++)

} ;
//*******************************************************************************************************
void olSDL::ClsXML_Reader::get_Page_from_ElementNode(const DOM_Node & Node, int & PageID,
									int & PageWidth, int & PageHeight,
									int & x_left_up_Frame, int & y_left_up_Frame,
									int & FrameWidth, int & FrameHeight) throw (ClsXML_Reader_ErrorReporter) {

	DOM_Node temp_Node ;
	DOM_Element ElementNode ;

	DOM_NamedNodeMap attributes ;
	DOM_Node  attribute ;

	if ( ! Node.getNodeName().equals(DOMString(TN_PAGE.c_str())) ) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

 	attributes = Node.getAttributes();
        attribute = attributes.getNamedItem(DOMString(AN_ID.c_str()));

	PageID = get_IntValue_from_Attribute(attribute) ;

	ElementNode = (DOM_Element&) Node ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_WIDTH.c_str())).item(0) ;
	PageWidth = get_IntContent_from_Element(temp_Node) ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_HEIGHT.c_str())).item(0) ;
	PageHeight = get_IntContent_from_Element(temp_Node) ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_PAGEFRAME.c_str())).item(0) ;
	get_PageFrame_from_ElementNode(temp_Node, x_left_up_Frame, y_left_up_Frame, FrameWidth, FrameHeight) ;
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Reader::get_PageFrame_from_ElementNode(const DOM_Node & Node,
			int & x_left_up_Frame, int & y_left_up_Frame, int & FrameWidth, int & FrameHeight) throw (ClsXML_Reader_ErrorReporter) {

	DOM_Node temp_Node ;
	DOM_Element ElementNode ;

	if ( ! Node.getNodeName().equals(DOMString(TN_PAGEFRAME.c_str())) ) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	ElementNode = (DOM_Element&) Node ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_PAS.c_str())).item(0) ;
	get_PaS_from_ElementNode(temp_Node, x_left_up_Frame, y_left_up_Frame, FrameWidth, FrameHeight) ;
} ;
//*******************************************************************************************************
//*******************************************************************************************************
// RecSyms
//*******************************************************************************************************
//*******************************************************************************************************
void olSDL::ClsXML_Reader::get_RecSym_from_ElementNode (const DOM_Node & Node, int & Symbol_ID, int & ParentPageID, int & ReferencedDiagramID,
					int & x_left_up, int & y_left_up, int & Width, int & Height, string & SymbolContent) throw (ClsXML_Reader_ErrorReporter) {

	DOM_Node temp_Node ;
	DOM_Element ElementNode ;

	DOM_NamedNodeMap attributes ;
	DOM_Node  attributeID ;
	DOM_Node  attributeParentPageID ;
	DOM_Node  attributeReferencedDiagramID ;

	if ( ! ( Node.getNodeName().equals(DOMString(TN_RECSYM_TEXTSYM.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_RECSYM_BLK_INST_REF.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_RECSYM_BLK_TYPE_REF.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_RECSYM_PRO_INST_REF.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_RECSYM_PRO_TYPE_REF.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_RECSYM_PROCEDURE_INST_REF.c_str()))) )
	{
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	ElementNode = (DOM_Element&) Node ;

 	attributes = Node.getAttributes();

        attributeID = attributes.getNamedItem(DOMString(AN_ID.c_str()));
	attributeParentPageID = attributes.getNamedItem(DOMString(AN_PARENT_PAGE_ID.c_str()));
	attributeReferencedDiagramID = attributes.getNamedItem(DOMString(AN_ID_REF.c_str()));

	Symbol_ID = get_IntValue_from_Attribute(attributeID) ;
	ParentPageID = get_IntValue_from_Attribute(attributeParentPageID) ;

	if (attributeReferencedDiagramID != NULL) {
		ReferencedDiagramID = get_IntValue_from_Attribute(attributeReferencedDiagramID) ;
	} ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_PAS.c_str())).item(0) ;
	get_PaS_from_ElementNode(temp_Node, x_left_up, y_left_up, Width, Height)  ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_CONTENT.c_str())).item(0) ;
	SymbolContent = get_StringContent_from_Element(temp_Node) ;
} ;
//*******************************************************************************************************
//*******************************************************************************************************
// PaS
//*******************************************************************************************************
//*******************************************************************************************************
void olSDL::ClsXML_Reader::get_PaS_from_ElementNode(const DOM_Node & Node,
			int & x_left_up, int & y_left_up, int & Width, int & Height) throw (ClsXML_Reader_ErrorReporter) {

	DOM_Element ElementNode ;

	DOM_NamedNodeMap attributes ;

	DOM_Node attribute_x_left_up ;
	DOM_Node attribute_y_left_up ;
	DOM_Node attribute_width ;
	DOM_Node attribute_height ;

	if ( ! Node.getNodeName().equals(DOMString(TN_PAS.c_str())) ) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	ElementNode = (DOM_Element&) Node ;

	attributes = Node.getAttributes();

	attribute_x_left_up = attributes.getNamedItem(DOMString(AN_X_LEFT_UP.c_str())) ;
	attribute_y_left_up = attributes.getNamedItem(DOMString(AN_Y_LEFT_UP.c_str())) ;
	attribute_width = attributes.getNamedItem(DOMString(AN_WIDTH.c_str())) ;
	attribute_height = attributes.getNamedItem(DOMString(AN_HEIGHT.c_str())) ;

	x_left_up = get_IntValue_from_Attribute(attribute_x_left_up) ;
	y_left_up = get_IntValue_from_Attribute(attribute_y_left_up) ;
	Width = get_IntValue_from_Attribute(attribute_width) ;
	Height = get_IntValue_from_Attribute(attribute_height) ;

} ;
//*******************************************************************************************************
//*******************************************************************************************************
// Channel
//*******************************************************************************************************
//*******************************************************************************************************
void olSDL::ClsXML_Reader::read_Channel(DOM_Document doc, DOM_Node & Node) throw (ClsXML_Reader_ErrorReporter) {

	ClsMoAccessInfo Info ;

	DOM_NamedNodeMap attributes ;
	DOM_Node attribute ;

	DOM_Element DiagramElementNode ;

	int OldDiagramID ;

	int cntChannels ;
	DOM_Node temp_ChannelNode ;
	DOM_NodeList ChannelNodeList ;
	DOM_Element ElementNodeChannel ;

	int New_Model_ID ;

	DOM_Node temp_NodeChannelComp ;

	int Channel_ID ;
	int ParentPageID ;
	ClsChannel::e_DelayType DelayType ;
	ClsChannel::e_DirectionType DirectionType ;
	list<int> ChanCoords ;
	bool bChanSourceIsEnv ;
	int ChanSourceModel_ID ;
	bool bChanDestIsEnv ;
	int ChanDestModel_ID ;

	if (Node.getNodeType() != DOM_Node::ELEMENT_NODE) { throw ClsXML_Reader_ErrorReporter(); } ;

	if ( ! ( Node.getNodeName().equals(DOMString(TN_SYSDIA.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_BLKDIA.c_str())) ||
		 Node.getNodeName().equals(DOMString(TN_PRODIA.c_str())) )) {

		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	// ==========
	// get DiagramID

	attributes = Node.getAttributes();
        attribute = attributes.getNamedItem(DOMString(AN_ID.c_str()));

	OldDiagramID = get_IntValue_from_Attribute(attribute) ;

	// END: get DiagramID
	// ==========

	DiagramElementNode = (DOM_Element&) Node ;

	ChannelNodeList = DiagramElementNode.getElementsByTagName(DOMString(TN_CHANNEL.c_str())) ;
	cntChannels = ChannelNodeList.getLength() ;

	for (int k=0; k<cntChannels; k++) {

		temp_ChannelNode = ChannelNodeList.item(k) ;

		get_Channel_from_ElementNode (temp_ChannelNode, Channel_ID, ParentPageID, DelayType, DirectionType, ChanCoords,
			bChanSourceIsEnv, ChanSourceModel_ID, bChanDestIsEnv, ChanDestModel_ID)  ;

		ElementNodeChannel = (DOM_Element&) temp_ChannelNode ;

		temp_NodeChannelComp = ElementNodeChannel.getElementsByTagName(DOMString(TN_CHANNEL_COMP_NAME.c_str())).item(0) ;
		ClsChanName ChanName = get_ChanName(temp_NodeChannelComp) ;

		temp_NodeChannelComp = ElementNodeChannel.getElementsByTagName(DOMString(TN_CHANNEL_COMP_SOURCE_SIG.c_str())).item(0) ;
		ClsChanSourceSigList ChanSourceSigList = get_ChanSourceSigList(temp_NodeChannelComp) ;

		temp_NodeChannelComp = ElementNodeChannel.getElementsByTagName(DOMString(TN_CHANNEL_COMP_SOURCE_CON.c_str())).item(0) ;
		ClsChanSourceCon ChanSourceCon = get_ChanSourceCon(temp_NodeChannelComp) ;

		temp_NodeChannelComp = ElementNodeChannel.getElementsByTagName(DOMString(TN_CHANNEL_COMP_DEST_SIG.c_str())).item(0) ;
		ClsChanDestSigList ChanDestSigList = get_ChanDestSigList(temp_NodeChannelComp) ;

		temp_NodeChannelComp = ElementNodeChannel.getElementsByTagName(DOMString(TN_CHANNEL_COMP_DEST_CON.c_str())).item(0) ;
		ClsChanDestCon ChanDestCon = get_ChanDestCon(temp_NodeChannelComp) ;

		if (!bChanSourceIsEnv) {
			ChanSourceModel_ID = get_New_ID_from_Old_ID(ChanSourceModel_ID) ;
		} ;

		if (!bChanDestIsEnv) {
			ChanDestModel_ID = get_New_ID_from_Old_ID(ChanDestModel_ID) ;
		} ;

		m_pSysSpec->insert_Channel(get_New_ID_from_Old_ID(OldDiagramID), get_New_ID_from_Old_ID(ParentPageID), DelayType, DirectionType,
			ChanName, ChanSourceSigList, ChanSourceCon, ChanDestSigList, ChanDestCon, ChanCoords,
			bChanSourceIsEnv, ChanSourceModel_ID, bChanDestIsEnv, ChanDestModel_ID, Info, New_Model_ID) ;

		// add to RenumberingHelper
		add_ID_Pair(Channel_ID, New_Model_ID) ;

		if (Info.get_Info() != ClsMoAccessInfo::insert_OK) {
			throw ClsXML_Reader_ErrorReporter() ;
		} ;

	} ; // for (int k=0; k<cntChannels; k++)

} ;
//*******************************************************************************************************
void olSDL::ClsXML_Reader::get_Channel_from_ElementNode (const DOM_Node & Node, int & Channel_ID, int & ParentPageID,
		ClsChannel::e_DelayType & DelayType, ClsChannel::e_DirectionType & DirectionType,
		list<int> & ChanCoords, bool & bSourceIsEnv, int & ChanSourceModel_ID, bool & bDestIsEnv, int & ChanDestModel_ID) {

	DOM_NamedNodeMap attributes ;

	DOM_Node attributeID ;
	DOM_Node attributeParentPageID ;
	DOM_Node attributeDirectionType ;
	DOM_Node attributeDelayType ;

	DOM_Node temp_Node ;
	DOM_Element ElementNode ;

	string AttributeDirectionTypeValue ;
	string AttributeDelayTypeValue ;

	string CoordsAsString ;

	if ( ! Node.getNodeName().equals(DOMString(TN_CHANNEL.c_str())) ) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

 	attributes = Node.getAttributes();

        attributeID = attributes.getNamedItem(DOMString(AN_ID.c_str()));
	attributeParentPageID = attributes.getNamedItem(DOMString(AN_PARENT_PAGE_ID.c_str()));

	attributeDirectionType = attributes.getNamedItem(DOMString(AN_DIRECTION_TYPE.c_str()));
	attributeDelayType = attributes.getNamedItem(DOMString(AN_DELAY_TYPE.c_str()));

	Channel_ID = get_IntValue_from_Attribute(attributeID) ;
	ParentPageID = get_IntValue_from_Attribute(attributeParentPageID) ;

	// ==========
	// DirectionType

	AttributeDirectionTypeValue = DOMStringToStdString(attributeDirectionType.getNodeValue()) ;

	if (AttributeDirectionTypeValue.compare(AV_DIRECTION_TYPE_BOTH) == 0) {

		DirectionType = ClsChannel::both ;

	} else if (AttributeDirectionTypeValue.compare(AV_DIRECTION_TYPE_FROM_SOURCE) == 0) {

		DirectionType = ClsChannel::from_source ;

	} else if (AttributeDirectionTypeValue.compare(AV_DIRECTION_TYPE_FROM_DESTINATION) == 0) {

		DirectionType = ClsChannel::from_destination ;

	} else {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	// END: DirectionType
	// ==========

	// ==========
	// DelayType

	AttributeDelayTypeValue = DOMStringToStdString(attributeDelayType.getNodeValue()) ;

	if (AttributeDelayTypeValue.compare(AV_DELAY_TYPE_DELAY) == 0) {

		DelayType = ClsChannel::delay ;

	} else if (AttributeDelayTypeValue.compare(AV_DELAY_TYPE_NO_DELAY) == 0) {

		DelayType = ClsChannel::no_delay ;

	} else {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	// END: DelayType
	// ==========

	ElementNode = (DOM_Element&) Node ;

	// ==========
	// Source and Dest IDs

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_CHANNEL_SOURCE_ID.c_str())).item(0) ;
	get_ChanSourceID(temp_Node, bSourceIsEnv, ChanSourceModel_ID) ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_CHANNEL_DESTINATION_ID.c_str())).item(0) ;
	get_ChanDestID(temp_Node, bDestIsEnv, ChanDestModel_ID) ;

	// END: Source and Dest IDs
	// ==========

	// ==========
	// Coords

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_COORDS.c_str())).item(0) ;
	CoordsAsString = get_StringContent_from_Element(temp_Node) ;

	if ( ! Converter::IntListString_To_IntList(CoordsAsString, ChanCoords) ) {
		throw ClsXML_Reader_ErrorReporter(ClsXML_Reader_ErrorReporter::integer_list_expected) ;
	} ;

	// END: Coords
	// ==========

} ;
//*******************************************************************************************************
void olSDL::ClsXML_Reader::get_ChanSourceID(const DOM_Node & Node, bool & bSourceIsEnv, int & ChanSourceModel_ID) {

	DOM_NamedNodeMap attributes ;
	DOM_Node attribute_IsEnv ;
	string str_ChanSourceModel_ID ;

	if ( ! Node.getNodeName().equals(DOMString(TN_CHANNEL_SOURCE_ID.c_str())) ) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

 	attributes = Node.getAttributes();
        attribute_IsEnv = attributes.getNamedItem(DOMString(AN_CHANNEL_SOURCE_IS_ENV.c_str()));


	if (DOMStringToStdString(attribute_IsEnv.getNodeValue()).compare(AV_CHANNEL_SOURCE_IS_ENV_TRUE) == 0) {

		bSourceIsEnv = true ;

	} else if (DOMStringToStdString(attribute_IsEnv.getNodeValue()).compare(AV_CHANNEL_SOURCE_IS_ENV_FALSE) == 0) {

		bSourceIsEnv = false ;

		str_ChanSourceModel_ID = get_StringContent_from_Element(Node) ;

		if ( ! Converter::String_To_Int(str_ChanSourceModel_ID, ChanSourceModel_ID) ) {
			throw ClsXML_Reader_ErrorReporter(ClsXML_Reader_ErrorReporter::integer_expected) ;
		} ;

	} else { } ;
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Reader::get_ChanDestID(const DOM_Node & Node, bool & bDestIsEnv, int & ChanDestModel_ID) {

	DOM_NamedNodeMap attributes ;
	DOM_Node attribute_IsEnv ;
	string str_ChanDestModel_ID ;

	if ( ! Node.getNodeName().equals(DOMString(TN_CHANNEL_DESTINATION_ID.c_str())) ) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

 	attributes = Node.getAttributes();
        attribute_IsEnv = attributes.getNamedItem(DOMString(AN_CHANNEL_DESTINATION_IS_ENV.c_str()));


	if (DOMStringToStdString(attribute_IsEnv.getNodeValue()).compare(AV_CHANNEL_DESTINATION_IS_ENV_TRUE) == 0) {

		bDestIsEnv = true ;

	} else if (DOMStringToStdString(attribute_IsEnv.getNodeValue()).compare(AV_CHANNEL_DESTINATION_IS_ENV_FALSE) == 0) {

		bDestIsEnv = false ;

		str_ChanDestModel_ID = get_StringContent_from_Element(Node) ;

		if ( ! Converter::String_To_Int(str_ChanDestModel_ID, ChanDestModel_ID) ) {
			throw ClsXML_Reader_ErrorReporter(ClsXML_Reader_ErrorReporter::integer_expected) ;
		} ;

	} else { } ;
} ;
//*******************************************************************************************************
olSDL::ClsChanName olSDL::ClsXML_Reader::get_ChanName(const DOM_Node & Node) {

	DOM_Node temp_Node ;
	DOM_Element ElementNode ;

	string Content ;
	int x_left_up ;
	int y_left_up ;
	int Width ;
	int Height ;

	if ( ! Node.getNodeName().equals(DOMString(TN_CHANNEL_COMP_NAME.c_str())) ) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	ElementNode = (DOM_Element&) Node ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_CONTENT.c_str())).item(0) ;
	Content = get_StringContent_from_Element(temp_Node) ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_PAS.c_str())).item(0) ;
	get_PaS_from_ElementNode(temp_Node, x_left_up, y_left_up, Width, Height)  ;

	return ClsChanName(Content, ClsRecPaS(x_left_up, y_left_up, Width, Height)) ;
} ;
//*******************************************************************************************************
olSDL::ClsChanSourceSigList olSDL::ClsXML_Reader::get_ChanSourceSigList(const DOM_Node & Node) {

	DOM_Node temp_Node ;
	DOM_Element ElementNode ;

	string Content ;
	int x_left_up ;
	int y_left_up ;
	int Width ;
	int Height ;

	if ( ! Node.getNodeName().equals(DOMString(TN_CHANNEL_COMP_SOURCE_SIG.c_str())) ) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	ElementNode = (DOM_Element&) Node ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_CONTENT.c_str())).item(0) ;
	Content = get_StringContent_from_Element(temp_Node) ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_PAS.c_str())).item(0) ;
	get_PaS_from_ElementNode(temp_Node, x_left_up, y_left_up, Width, Height)  ;

	return ClsChanSourceSigList(Content, ClsRecPaS(x_left_up, y_left_up, Width, Height)) ;
} ;
//*******************************************************************************************************
olSDL::ClsChanSourceCon olSDL::ClsXML_Reader::get_ChanSourceCon(const DOM_Node & Node) {

	DOM_Node temp_Node ;
	DOM_Element ElementNode ;

	string Content ;
	int x_left_up ;
	int y_left_up ;
	int Width ;
	int Height ;

	if ( ! Node.getNodeName().equals(DOMString(TN_CHANNEL_COMP_SOURCE_CON.c_str())) ) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	ElementNode = (DOM_Element&) Node ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_CONTENT.c_str())).item(0) ;
	Content = get_StringContent_from_Element(temp_Node) ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_PAS.c_str())).item(0) ;
	get_PaS_from_ElementNode(temp_Node, x_left_up, y_left_up, Width, Height)  ;

	return ClsChanSourceCon(Content, ClsRecPaS(x_left_up, y_left_up, Width, Height)) ;
} ;
//*******************************************************************************************************
olSDL::ClsChanDestSigList olSDL::ClsXML_Reader::get_ChanDestSigList(const DOM_Node & Node) {

	DOM_Node temp_Node ;
	DOM_Element ElementNode ;

	string Content ;
	int x_left_up ;
	int y_left_up ;
	int Width ;
	int Height ;

	if ( ! Node.getNodeName().equals(DOMString(TN_CHANNEL_COMP_DEST_SIG.c_str())) ) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	ElementNode = (DOM_Element&) Node ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_CONTENT.c_str())).item(0) ;
	Content = get_StringContent_from_Element(temp_Node) ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_PAS.c_str())).item(0) ;
	get_PaS_from_ElementNode(temp_Node, x_left_up, y_left_up, Width, Height)  ;

	return ClsChanDestSigList(Content, ClsRecPaS(x_left_up, y_left_up, Width, Height)) ;
} ;
//*******************************************************************************************************
olSDL::ClsChanDestCon olSDL::ClsXML_Reader::get_ChanDestCon(const DOM_Node & Node) {

	DOM_Node temp_Node ;
	DOM_Element ElementNode ;

	string Content ;
	int x_left_up ;
	int y_left_up ;
	int Width ;
	int Height ;

	if ( ! Node.getNodeName().equals(DOMString(TN_CHANNEL_COMP_DEST_CON.c_str())) ) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	ElementNode = (DOM_Element&) Node ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_CONTENT.c_str())).item(0) ;
	Content = get_StringContent_from_Element(temp_Node) ;

	temp_Node = ElementNode.getElementsByTagName(DOMString(TN_PAS.c_str())).item(0) ;
	get_PaS_from_ElementNode(temp_Node, x_left_up, y_left_up, Width, Height)  ;

	return ClsChanDestCon(Content, ClsRecPaS(x_left_up, y_left_up, Width, Height)) ;
} ;



//*******************************************************************************************************
//*******************************************************************************************************
// Formal Parameters
//*******************************************************************************************************
//*******************************************************************************************************
void olSDL::ClsXML_Reader::read_ProcedureFormalParameters(DOM_Document doc, DOM_Node & Node) throw (ClsXML_Reader_ErrorReporter) {

	ClsMoAccessInfo Info ;

	string str_FormalParameters ;

	DOM_Node temp_NodeFormalParameters ;
	DOM_NodeList FormalParametersNodeList ;
	DOM_Element DiagramElementNode ;

	DOM_NamedNodeMap attributes ;
	DOM_Node attribute ;

	int NewDiagramID ;
	int OldDiagramID ;

	if ( ! ( Node.getNodeName().equals(DOMString(TN_PROCEDUREDIA.c_str())) )) {

		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	// ======
	attributes = Node.getAttributes();
        attribute = attributes.getNamedItem(DOMString(AN_ID.c_str()));

	OldDiagramID = get_IntValue_from_Attribute(attribute) ;
	NewDiagramID = get_New_ID_from_Old_ID(OldDiagramID) ;

	DiagramElementNode = (DOM_Element&) Node ;

	FormalParametersNodeList = DiagramElementNode.getElementsByTagName(DOMString(TN_FORMAL_PARAMETERS.c_str())) ;
	temp_NodeFormalParameters = FormalParametersNodeList.item(0) ;
	str_FormalParameters = get_StringContent_from_Element(temp_NodeFormalParameters) ;


	m_pSysSpec->modify_ProcedureDiaFormalParameters(NewDiagramID, str_FormalParameters, Info) ;

	if (Info.get_Info() != ClsMoAccessInfo::modify_OK) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

} ;
//*******************************************************************************************************
//*******************************************************************************************************
// Helper
//*******************************************************************************************************
//*******************************************************************************************************
DOM_Node olSDL::ClsXML_Reader::get_DiagramElementNodeByID(DOM_Document doc, const int & ElementID) throw (ClsXML_Reader_ErrorReporter) {

	DOM_NodeList NodeList ;
	DOM_Node temp_Node ;
	int cntNode ;

	DOM_NamedNodeMap attributes ;
	DOM_Node attribute ;

	int DiagramID ;

	// ==========
	// check SysDia
	NodeList = doc.getElementsByTagName(DOMString(TN_SYSDIA.c_str())) ;
	cntNode = NodeList.getLength() ;

	if (cntNode != 1) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	temp_Node = NodeList.item(0) ;

	attributes = temp_Node.getAttributes();
        attribute = attributes.getNamedItem(DOMString(AN_ID.c_str()));

	DiagramID = get_IntValue_from_Attribute(attribute) ;

	if ( DiagramID == ElementID ) {
		return temp_Node ;
	} ;

	// ==========
	// check BlkDia
	NodeList = doc.getElementsByTagName(DOMString(TN_BLKDIA.c_str())) ;
	cntNode = NodeList.getLength() ;

	for (int k=0; k<cntNode; k++) {

		temp_Node = NodeList.item(k) ;

		attributes = temp_Node.getAttributes();
	        attribute = attributes.getNamedItem(DOMString(AN_ID.c_str()));

		DiagramID = get_IntValue_from_Attribute(attribute) ;

		if ( DiagramID == ElementID ) {
			return temp_Node ;
		} ;

	} ; // for (int k=0; k<cntNode; k++)

	// ==========
	// check ProDia
	NodeList = doc.getElementsByTagName(DOMString(TN_PRODIA.c_str())) ;
	cntNode = NodeList.getLength() ;

	for (int k=0; k<cntNode; k++) {

		temp_Node = NodeList.item(k) ;

		attributes = temp_Node.getAttributes();
	        attribute = attributes.getNamedItem(DOMString(AN_ID.c_str()));

		DiagramID = get_IntValue_from_Attribute(attribute) ;

		if ( DiagramID == ElementID ) {
			return temp_Node ;
		} ;

	} ; // for (int k=0; k<cntNode; k++)

	// ==========
	// check ProcedureDia
	NodeList = doc.getElementsByTagName(DOMString(TN_PROCEDUREDIA.c_str())) ;
	cntNode = NodeList.getLength() ;

	for (int k=0; k<cntNode; k++) {

		temp_Node = NodeList.item(k) ;

		attributes = temp_Node.getAttributes();
	        attribute = attributes.getNamedItem(DOMString(AN_ID.c_str()));

		DiagramID = get_IntValue_from_Attribute(attribute) ;

		if ( DiagramID == ElementID ) {
			return temp_Node ;
		} ;

	} ; // for (int k=0; k<cntNode; k++)

	// not found
	throw ClsXML_Reader_ErrorReporter() ;
} ;
//*******************************************************************************************************
int olSDL::ClsXML_Reader::get_IntValue_from_Attribute(const DOM_Node & ElementNode) throw (ClsXML_Reader_ErrorReporter) {

	string str_AttributeValue ;
	int AttributeValue ;

	str_AttributeValue = DOMStringToStdString(ElementNode.getNodeValue()) ;

	if (! Converter::String_To_Int(str_AttributeValue, AttributeValue)) {
		throw ClsXML_Reader_ErrorReporter(ClsXML_Reader_ErrorReporter::integer_expected) ;
	} ;

	return AttributeValue ;
} ;
//*******************************************************************************************************
int olSDL::ClsXML_Reader::get_IntContent_from_Element(const DOM_Node & ElementNode) throw (ClsXML_Reader_ErrorReporter) {

	DOM_Node Node ;
	DOM_NodeList Children = ElementNode.getChildNodes() ;

	string str_ElementContent ;
	int ElementContent ;

	if (Children.getLength() != 1) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	Node = Children.item(0) ;

	if (Node.getNodeType() != DOM_Node::TEXT_NODE) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	str_ElementContent = DOMStringToStdString(Node.getNodeValue()) ;

	if (! Converter::String_To_Int(str_ElementContent, ElementContent)) {
		throw ClsXML_Reader_ErrorReporter(ClsXML_Reader_ErrorReporter::integer_expected) ;
	} ;

	return ElementContent ;
} ;
//*******************************************************************************************************
string olSDL::ClsXML_Reader::get_StringContent_from_Element(const DOM_Node & ElementNode,
		const bool b_deleteBeginNewline, const bool b_deleteEndNewline) throw (ClsXML_Reader_ErrorReporter) {

	DOM_Node FirstChild ;
	DOM_NodeList Children ;
	string ElementContent ;

	Children = ElementNode.getChildNodes() ;

	if (Children.getLength() != 1) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	FirstChild = Children.item(0) ;

	if (FirstChild.getNodeType() != DOM_Node::TEXT_NODE) {
		throw ClsXML_Reader_ErrorReporter() ;
	} ;

	ElementContent = DOMStringToStdString(FirstChild.getNodeValue()) ;

	if ( ElementContent.length() >= 1 ) {

		if (b_deleteBeginNewline) {
			if ( ElementContent.substr(0, 1).compare("\n") == 0 ) {
				ElementContent = ElementContent.substr(1, ElementContent.length()-1) ;
			} ;
		} ;
	} ;

	if ( ElementContent.length() >= 1 ) {

		if (b_deleteEndNewline) {
			if ( ElementContent.substr(ElementContent.length()-1, 1).compare("\n") == 0 ) {
				ElementContent = ElementContent.substr(0, ElementContent.length()-1) ;
			} ;
		} ;
	} ;

	return ElementContent ;
} ;
//*******************************************************************************************************
string olSDL::ClsXML_Reader::DOMStringToStdString(const DOMString& s) {

	char * pTempString = s.transcode() ;

	try
	{
		string Result(pTempString) ;
		delete [] pTempString ;
		return Result ;

	} catch( ... ) {

                delete [] pTempString;
                throw ;
        } ;

} ;
//*******************************************************************************************************


