
#include "XML_Writer.h"
#include "Converter.h"

const string olSDL::ClsXML_Writer::TAG_PROLOG = "<?xml version=\"1.0\"?>\n<!DOCTYPE SYSSPEC SYSTEM \"olSDL.dtd\">";


//*******************************************************************************************************
olSDL::ClsXML_Writer::ClsXML_Writer() {

	m_pOutStream = NULL ;
} ;
//*******************************************************************************************************
olSDL::ClsXML_Writer::~ClsXML_Writer() {

	if (m_pOutStream != NULL) {

		if (m_pOutStream->is_open()) {
			m_pOutStream->close() ;
		} ;

		delete m_pOutStream ;
	} ;
} ;
//*******************************************************************************************************
bool olSDL::ClsXML_Writer::write_SysSpec(const ClsSysDia * pSysDia, const string & FileName) {

	if (m_pOutStream != NULL) {

		if (m_pOutStream->is_open()) {
			m_pOutStream->close() ;
		} ;

		delete m_pOutStream ;
		m_FileName = "" ;
	} ;

	m_FileName = FileName ;

	m_pOutStream = new ofstream(m_FileName.c_str(), ios::out) ;

	if (! *m_pOutStream) {
		return false ;
	} ;

	m_pOutStream->exceptions(ios::badbit | ios::failbit | ios::eofbit) ;

	(*m_pOutStream) << TAG_PROLOG + "\n" ;

	(*m_pOutStream) << create_StartTagFromName(TN_SYSSPEC) + "\n" ;

	write_Dia(pSysDia) ;

	(*m_pOutStream) << create_EndTagFromName(TN_SYSSPEC) + "\n" ;

	m_pOutStream->close() ;

	if (! *m_pOutStream) {
		return false ;
	} else {
		return true ;
	} ;

} ;
//*******************************************************************************************************
//*******************************************************************************************************
void olSDL::ClsXML_Writer::write_Dia(const Diagram * pDiagram) const {

	switch (pDiagram->get_DiagramType()) {

		case EnumDia::SysDia:
		case EnumDia::BlkDia:
		{
			write_HL_InstDia (pDiagram) ;
			break ;
		}
		case EnumDia::ProDia:
		{
			write_ProDia((ClsProcessDia *) pDiagram) ;
			break ;
		}
		case EnumDia::ProcedureDia:
		{
			write_ProcedureDia((ClsProcedureDia *) pDiagram) ;
			break ;
		}
		default:
		{
			break ;
		}
	} ; //switch (pDiagram->get_DiagramType())

} ;
//*******************************************************************************************************
void olSDL::ClsXML_Writer::write_HL_InstDia(const Diagram * pHL_InstDiagram) const {

	string StartTag ;
	string EndTag ;
	string DiaName ;

	list<int> IDsOfPages ;
	list<int>::iterator iterIDsOfPages ;

	ClsMoAccessInfo Info ;
	type_ListOf_Poi_to_ConstSym RecSymResultList ;
	type_ListOf_Poi_to_ConstSym::const_iterator iterRecSymResultList ;

	type_ListOf_Poi_to_ConstChannel ChannelResultList ;
	type_ListOf_Poi_to_ConstChannel::const_iterator iterChannelResultList ;

	list<const Diagram *> ReferencedDiagrams ;
	list<const Diagram *>::const_iterator iterReferencedDiagrams ;

	switch (pHL_InstDiagram->get_DiagramType()) {

		case EnumDia::SysDia:
		{
			StartTag = create_StartTagFromName(TN_SYSDIA) ;
			add_AttrToTag(StartTag, AN_ID, pHL_InstDiagram->get_Model_ID()) ;
			EndTag = create_EndTagFromName(TN_SYSDIA) ;
			break ;
		}
		case EnumDia::BlkDia:
		{
			StartTag = create_StartTagFromName(TN_BLKDIA) ;
			add_AttrToTag(StartTag, AN_ID, pHL_InstDiagram->get_Model_ID()) ;
			EndTag = create_EndTagFromName(TN_BLKDIA) ;
			break ;
		}
		default:
		{
			break ;
		}
	} ; // switch (pHL_InstDiagram->get_DiagramType())

	(*m_pOutStream) << StartTag + "\n" ;


	/*==============================*/
	// Diagram Name

	pHL_InstDiagram->get_HeaderPart_Name(DiaName) ;
	write_Element(TN_DIANAME, DiaName) ;

	// END: Diagram Name
	/*==============================*/

	/*==============================*/
	// Pages
	pHL_InstDiagram->get_IDsOfPages(IDsOfPages) ;

	for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)
	{
		write_Page(pHL_InstDiagram, *iterIDsOfPages) ;
	} ;

	// END: Pages
	/*==============================*/

	/*==============================*/
	// RecSyms

	for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)
	{
		// TextSyms
		RecSymResultList.clear() ;
		pHL_InstDiagram->get_Syms(EnumSym::TextSym, *iterIDsOfPages, RecSymResultList, Info) ;
		for (iterRecSymResultList=RecSymResultList.begin(); iterRecSymResultList != RecSymResultList.end(); iterRecSymResultList++) {

			write_RecSym(*iterRecSymResultList) ;
			if ((*iterRecSymResultList)->is_referencing_Symbol()) {
				ReferencedDiagrams.push_back((*iterRecSymResultList)->get_referenced_Diagram()) ;
			} ;
		} ;

		// BlkInstRef
		RecSymResultList.clear() ;
		pHL_InstDiagram->get_Syms(EnumSym::BlkInstRefSym, *iterIDsOfPages, RecSymResultList, Info) ;
		for (iterRecSymResultList=RecSymResultList.begin(); iterRecSymResultList != RecSymResultList.end(); iterRecSymResultList++) {

			write_RecSym(*iterRecSymResultList) ;
			if ((*iterRecSymResultList)->is_referencing_Symbol()) {
				ReferencedDiagrams.push_back((*iterRecSymResultList)->get_referenced_Diagram()) ;
			} ;
		} ;

		// BlkTypeRefSym
		RecSymResultList.clear() ;
		pHL_InstDiagram->get_Syms(EnumSym::BlkTypeRefSym, *iterIDsOfPages, RecSymResultList, Info) ;
		for (iterRecSymResultList=RecSymResultList.begin(); iterRecSymResultList != RecSymResultList.end(); iterRecSymResultList++) {

			write_RecSym(*iterRecSymResultList) ;
			if ((*iterRecSymResultList)->is_referencing_Symbol()) {
				ReferencedDiagrams.push_back((*iterRecSymResultList)->get_referenced_Diagram()) ;
			} ;
		} ;

		// ProInstRefSym
		RecSymResultList.clear() ;
		pHL_InstDiagram->get_Syms(EnumSym::ProInstRefSym, *iterIDsOfPages, RecSymResultList, Info) ;
		for (iterRecSymResultList=RecSymResultList.begin(); iterRecSymResultList != RecSymResultList.end(); iterRecSymResultList++) {

			write_RecSym(*iterRecSymResultList) ;
			if ((*iterRecSymResultList)->is_referencing_Symbol()) {
				ReferencedDiagrams.push_back((*iterRecSymResultList)->get_referenced_Diagram()) ;
			} ;
		} ;

		// ProTypeRefSym
		RecSymResultList.clear() ;
		pHL_InstDiagram->get_Syms(EnumSym::ProTypeRefSym, *iterIDsOfPages, RecSymResultList, Info) ;
		for (iterRecSymResultList=RecSymResultList.begin(); iterRecSymResultList != RecSymResultList.end(); iterRecSymResultList++) {

			write_RecSym(*iterRecSymResultList) ;
			if ((*iterRecSymResultList)->is_referencing_Symbol()) {
				ReferencedDiagrams.push_back((*iterRecSymResultList)->get_referenced_Diagram()) ;
			} ;
		} ;

	} ; // for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)

	// END: RecSyms
	/*==============================*/

	/*==============================*/
	// channels

	for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)
	{
		ChannelResultList.clear() ;
		pHL_InstDiagram->get_Channels(*iterIDsOfPages, ChannelResultList, Info) ;

		for (iterChannelResultList=ChannelResultList.begin(); iterChannelResultList != ChannelResultList.end() ; iterChannelResultList++)
		{
			write_Channel(*iterChannelResultList) ;
		} ;

	} ; // for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)

	// END: channels
	/*==============================*/


	(*m_pOutStream) << EndTag + "\n" ;


	/*==============================*/
	// Referenced Diagrams

	for (iterReferencedDiagrams=ReferencedDiagrams.begin(); iterReferencedDiagrams != ReferencedDiagrams.end(); iterReferencedDiagrams++) {

		write_Dia(*iterReferencedDiagrams) ;
	} ;

	// END: Referenced Diagrams
	/*==============================*/
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Writer::write_ProDia(const ClsProcessDia * pProcessDia) const {

	string DiaName ;
	string StartTag ;
	list<int> IDsOfPages ;
	list<int>::iterator iterIDsOfPages ;

	ClsMoAccessInfo Info ;
	type_ListOf_Poi_to_ConstSym RecSymResultList ;
	type_ListOf_Poi_to_ConstSym::const_iterator iterRecSymResultList ;

	type_ListOf_Poi_to_ConstChannel ChannelResultList ;
	type_ListOf_Poi_to_ConstChannel::const_iterator iterChannelResultList ;

	type_ListOfPoi_to_ConstStateMachineSymbol StateMaSymResultList ;
	type_ListOfPoi_to_ConstStateMachineSymbol::const_iterator iterStateMaSymResultList ;

	type_ListOfPoi_to_ConstFlowLine FlowLineResultList ;
	type_ListOfPoi_to_ConstFlowLine::const_iterator iterFlowLineResultList ;

	list<const Diagram *> ReferencedDiagrams ;
	list<const Diagram *>::const_iterator iterReferencedDiagrams ;

	StartTag = create_StartTagFromName(TN_PRODIA) ;
	add_AttrToTag(StartTag, AN_ID, pProcessDia->get_Model_ID()) ;

	(*m_pOutStream) << StartTag + "\n" ;


	/*==============================*/
	// Diagram Name

	pProcessDia->get_HeaderPart_Name(DiaName) ;
	write_Element(TN_DIANAME, DiaName) ;

	// END: Diagram Name
	/*==============================*/

	/*==============================*/
	// Pages
	pProcessDia->get_IDsOfPages(IDsOfPages) ;

	for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)
	{
		write_Page(pProcessDia, *iterIDsOfPages) ;
	} ;

	// END: Pages
	/*==============================*/

	/*==============================*/
	// RecSyms

	for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)
	{
		// TextSyms
		RecSymResultList.clear() ;
		pProcessDia->get_Syms(EnumSym::TextSym, *iterIDsOfPages, RecSymResultList, Info) ;
		for (iterRecSymResultList=RecSymResultList.begin(); iterRecSymResultList != RecSymResultList.end(); iterRecSymResultList++) {

			write_RecSym(*iterRecSymResultList) ;
			if ((*iterRecSymResultList)->is_referencing_Symbol()) {
				ReferencedDiagrams.push_back((*iterRecSymResultList)->get_referenced_Diagram()) ;
			} ;
		} ;

		// ProInstRefSym
		RecSymResultList.clear() ;
		pProcessDia->get_Syms(EnumSym::ProInstRefSym, *iterIDsOfPages, RecSymResultList, Info) ;
		for (iterRecSymResultList=RecSymResultList.begin(); iterRecSymResultList != RecSymResultList.end(); iterRecSymResultList++) {

			write_RecSym(*iterRecSymResultList) ;
			if ((*iterRecSymResultList)->is_referencing_Symbol()) {
				ReferencedDiagrams.push_back((*iterRecSymResultList)->get_referenced_Diagram()) ;
			} ;
		} ;

		// ProTypeRefSym
		RecSymResultList.clear() ;
		pProcessDia->get_Syms(EnumSym::ProTypeRefSym, *iterIDsOfPages, RecSymResultList, Info) ;
		for (iterRecSymResultList=RecSymResultList.begin(); iterRecSymResultList != RecSymResultList.end(); iterRecSymResultList++) {

			write_RecSym(*iterRecSymResultList) ;
			if ((*iterRecSymResultList)->is_referencing_Symbol()) {
				ReferencedDiagrams.push_back((*iterRecSymResultList)->get_referenced_Diagram()) ;
			} ;
		} ;

		// ProcedureInstRefSym
		RecSymResultList.clear() ;
		pProcessDia->get_Syms(EnumSym::ProcedureInstRefSym, *iterIDsOfPages, RecSymResultList, Info) ;
		for (iterRecSymResultList=RecSymResultList.begin(); iterRecSymResultList != RecSymResultList.end(); iterRecSymResultList++) {

			write_RecSym(*iterRecSymResultList) ;
			if ((*iterRecSymResultList)->is_referencing_Symbol()) {
				ReferencedDiagrams.push_back((*iterRecSymResultList)->get_referenced_Diagram()) ;
			} ;
		} ;

	} ; // for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)

	// RecSyms
	/*==============================*/

	/*==============================*/
	// channels

	for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)
	{
		ChannelResultList.clear() ;
		pProcessDia->get_Channels(*iterIDsOfPages, ChannelResultList, Info) ;

		for (iterChannelResultList=ChannelResultList.begin(); iterChannelResultList != ChannelResultList.end() ; iterChannelResultList++)
		{
			write_Channel(*iterChannelResultList) ;
		} ;

	} ; // for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)

	// END: channels
	/*==============================*/

	/*==============================*/
	// StateMaSyms

	for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)
	{
		StateMaSymResultList.clear() ;
		pProcessDia->get_StateMachineSyms(*iterIDsOfPages, StateMaSymResultList, Info) ;

		for (iterStateMaSymResultList=StateMaSymResultList.begin(); iterStateMaSymResultList != StateMaSymResultList.end() ; iterStateMaSymResultList++)
		{
			write_StateMaSym(*iterStateMaSymResultList) ;
		} ;

	} ; // for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)

	// END: StateMaSyms
	/*==============================*/

	/*==============================*/
	// FlowLines

	for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)
	{
		FlowLineResultList.clear() ;
		pProcessDia->get_FlowLines(*iterIDsOfPages, FlowLineResultList, Info) ;
		for (iterFlowLineResultList=FlowLineResultList.begin(); iterFlowLineResultList != FlowLineResultList.end() ; iterFlowLineResultList++)
		{
			write_FlowLine(*iterFlowLineResultList) ;
		} ;

	} ; // for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)

	// END: FlowLines
	/*==============================*/


	(*m_pOutStream) << create_EndTagFromName(TN_PRODIA) + "\n" ;


	/*==============================*/
	// Referenced Diagrams

	for (iterReferencedDiagrams=ReferencedDiagrams.begin(); iterReferencedDiagrams != ReferencedDiagrams.end(); iterReferencedDiagrams++) {

		write_Dia(*iterReferencedDiagrams) ;
	} ;

	// END: Referenced Diagrams
	/*==============================*/
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Writer::write_ProcedureDia(const ClsProcedureDia * pProcedureDia) const {

	string DiaName ;
	string StartTag ;
	list<int> IDsOfPages ;
	list<int>::iterator iterIDsOfPages ;

	ClsMoAccessInfo Info ;
	type_ListOf_Poi_to_ConstSym RecSymResultList ;
	type_ListOf_Poi_to_ConstSym::const_iterator iterRecSymResultList ;

	type_ListOfPoi_to_ConstStateMachineSymbol StateMaSymResultList ;
	type_ListOfPoi_to_ConstStateMachineSymbol::const_iterator iterStateMaSymResultList ;

	type_ListOfPoi_to_ConstFlowLine FlowLineResultList ;
	type_ListOfPoi_to_ConstFlowLine::const_iterator iterFlowLineResultList ;

	list<const Diagram *> ReferencedDiagrams ;
	list<const Diagram *>::const_iterator iterReferencedDiagrams ;

	StartTag = create_StartTagFromName(TN_PROCEDUREDIA) ;
	add_AttrToTag(StartTag, AN_ID, pProcedureDia->get_Model_ID()) ;

	(*m_pOutStream) << StartTag + "\n" ;


	/*==============================*/
	// Diagram Name

	pProcedureDia->get_HeaderPart_Name(DiaName) ;
	write_Element(TN_DIANAME, DiaName) ;

	// END: Diagram Name
	/*==============================*/

	/*==============================*/
	// Formal Parameters

	write_Element(TN_FORMAL_PARAMETERS, pProcedureDia->get_FormalParameters()) ;

	// Formal Parameters
	/*==============================*/

	/*==============================*/
	// Pages
	pProcedureDia->get_IDsOfPages(IDsOfPages) ;

	for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)
	{
		write_Page(pProcedureDia, *iterIDsOfPages) ;
	} ;

	// END: Pages
	/*==============================*/

	/*==============================*/
	// RecSyms

	for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)
	{
		// TextSyms
		RecSymResultList.clear() ;
		pProcedureDia->get_Syms(EnumSym::TextSym, *iterIDsOfPages, RecSymResultList, Info) ;
		for (iterRecSymResultList=RecSymResultList.begin(); iterRecSymResultList != RecSymResultList.end(); iterRecSymResultList++) {

			write_RecSym(*iterRecSymResultList) ;
			if ((*iterRecSymResultList)->is_referencing_Symbol()) {
				ReferencedDiagrams.push_back((*iterRecSymResultList)->get_referenced_Diagram()) ;
			} ;
		} ;

	} ; // for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)

	// RecSyms
	/*==============================*/

	/*==============================*/
	// StateMaSyms

	for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)
	{
		StateMaSymResultList.clear() ;
		pProcedureDia->get_StateMachineSyms(*iterIDsOfPages, StateMaSymResultList, Info) ;

		for (iterStateMaSymResultList=StateMaSymResultList.begin(); iterStateMaSymResultList != StateMaSymResultList.end() ; iterStateMaSymResultList++)
		{
			write_StateMaSym(*iterStateMaSymResultList) ;
		} ;

	} ; // for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)

	// END: StateMaSyms
	/*==============================*/

	/*==============================*/
	// FlowLines

	for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)
	{
		FlowLineResultList.clear() ;
		pProcedureDia->get_FlowLines(*iterIDsOfPages, FlowLineResultList, Info) ;
		for (iterFlowLineResultList=FlowLineResultList.begin(); iterFlowLineResultList != FlowLineResultList.end() ; iterFlowLineResultList++)
		{
			write_FlowLine(*iterFlowLineResultList) ;
		} ;

	} ; // for (iterIDsOfPages=IDsOfPages.begin(); iterIDsOfPages != IDsOfPages.end(); iterIDsOfPages++)

	// END: FlowLines
	/*==============================*/

	(*m_pOutStream) << create_EndTagFromName(TN_PROCEDUREDIA) + "\n" ;


	/*==============================*/
	// Referenced Diagrams

	for (iterReferencedDiagrams=ReferencedDiagrams.begin(); iterReferencedDiagrams != ReferencedDiagrams.end(); iterReferencedDiagrams++) {

		write_Dia(*iterReferencedDiagrams) ;
	} ;

	// END: Referenced Diagrams
	/*==============================*/
} ;
//*******************************************************************************************************
//*******************************************************************************************************
void olSDL::ClsXML_Writer::write_Page(const Diagram * pDia, const int PageID) const {

	ClsMoAccessInfo Info ;

	string StartTag ;

	int PageWidth ;
	int PageHeight ;

	StartTag = create_StartTagFromName(TN_PAGE) ;
	add_AttrToTag(StartTag, AN_ID, PageID) ;
	(*m_pOutStream) << StartTag + "\n" ;

	pDia->get_PageSize(PageID, PageWidth, PageHeight, Info) ;

	write_Element(TN_WIDTH, PageWidth, false) ;
	write_Element(TN_HEIGHT, PageHeight, false) ;

	write_PageFrame(pDia, PageID) ;

	(*m_pOutStream) << create_EndTagFromName(TN_PAGE) + "\n" ;
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Writer::write_PageFrame(const Diagram * pDia, const int PageID) const {

	ClsMoAccessInfo Info ;
	string StartTag ;
	ClsRecPaS PaS(0,0,10,10) ;
	string HP_DiaType ;
	string HP_Name ;
	string HP_Index ;

	(*m_pOutStream) << create_StartTagFromName(TN_PAGEFRAME) + "\n" ;

	pDia->get_PageFrameInfo(PageID, PaS, HP_DiaType, HP_Name, HP_Index, Info) ;

	write_PaS(PaS.get_x_left_up(), PaS.get_y_left_up(), PaS.get_width(), PaS.get_height()) ;

	(*m_pOutStream) << create_EndTagFromName(TN_PAGEFRAME) + "\n" ;
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Writer::write_PaS(const int x_left_up, const int y_left_up, const int width, const int height) const {

	string StartTag ;

	StartTag = create_StartTagFromName(TN_PAS) ;

	add_AttrToTag(StartTag, AN_X_LEFT_UP, x_left_up) ;
	add_AttrToTag(StartTag, AN_Y_LEFT_UP, y_left_up) ;
	add_AttrToTag(StartTag, AN_WIDTH, width) ;
	add_AttrToTag(StartTag, AN_HEIGHT, height) ;

	(*m_pOutStream) << StartTag ;
	
	(*m_pOutStream) << create_EndTagFromName(TN_PAS) + "\n" ;
} ;
//*******************************************************************************************************
//*******************************************************************************************************
void olSDL::ClsXML_Writer::write_ChanComp(const ClsChanComp * pChanComp) const {

	string StartTag ;
	string EndTag ;

	switch (pChanComp->get_CompType()) {
		case ClsChanComp::name:
		{
			StartTag = create_StartTagFromName(TN_CHANNEL_COMP_NAME) ;
			EndTag = create_EndTagFromName(TN_CHANNEL_COMP_NAME) ;
			break ;
		}
		case ClsChanComp::source_sig:
		{
			StartTag = create_StartTagFromName(TN_CHANNEL_COMP_SOURCE_SIG) ;
			EndTag = create_EndTagFromName(TN_CHANNEL_COMP_SOURCE_SIG) ;
			break ;
		}
		case ClsChanComp::dest_sig:
		{
			StartTag = create_StartTagFromName(TN_CHANNEL_COMP_DEST_SIG) ;
			EndTag = create_EndTagFromName(TN_CHANNEL_COMP_DEST_SIG) ;
			break ;
		}
		case ClsChanComp::source_con:
		{
			StartTag = create_StartTagFromName(TN_CHANNEL_COMP_SOURCE_CON) ;
			EndTag = create_EndTagFromName(TN_CHANNEL_COMP_SOURCE_CON) ;
			break ;
		}
		case ClsChanComp::dest_con:
		{
			StartTag = create_StartTagFromName(TN_CHANNEL_COMP_DEST_CON) ;
			EndTag = create_EndTagFromName(TN_CHANNEL_COMP_DEST_CON) ;
			break ;
		}
		default:
		{
			StartTag = "STARTNONE" ;
			EndTag = "ENDNONE" ;
		}

	} ; // switch (pChanComp->get_CompType())

	(*m_pOutStream) << StartTag + "\n" ;

	write_Element(TN_CONTENT, pChanComp->get_content()) ;

	write_PaS(pChanComp->get_x_left_up(), pChanComp->get_y_left_up(), pChanComp->get_width(), pChanComp->get_height()) ;

	(*m_pOutStream) << EndTag + "\n" ;
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Writer::write_Channel(const ClsChannel * pChannel) const {

	string StartTag ;
	string MyTag ;

	list<int> CoordsIntList ;

	const ClsChanName* pChanName ;
	const ClsChanSourceSigList* pChanSourceSigList ;
	const ClsChanSourceCon* pChanSourceCon ;
	const ClsChanDestSigList* pChanDestSigList ;
	const ClsChanDestCon* pChanDestCon ;

	StartTag = create_StartTagFromName(TN_CHANNEL) ;
	add_AttrToTag(StartTag, AN_ID, pChannel->get_Model_ID()) ;
	add_AttrToTag(StartTag, AN_PARENT_PAGE_ID, pChannel->get_Page_ID()) ;

	switch (pChannel->get_DirectionType()) {
		case ClsChannel::both:
		{
			add_AttrToTag(StartTag, AN_DIRECTION_TYPE, AV_DIRECTION_TYPE_BOTH) ;
			break ;
		}
		case ClsChannel::from_source:
		{
			add_AttrToTag(StartTag, AN_DIRECTION_TYPE, AV_DIRECTION_TYPE_FROM_SOURCE) ;
			break ;
		}
		case ClsChannel::from_destination:
		{
			add_AttrToTag(StartTag, AN_DIRECTION_TYPE, AV_DIRECTION_TYPE_FROM_DESTINATION) ;
			break ;
		}
		default:
		{
			add_AttrToTag(StartTag, AN_DIRECTION_TYPE, "none") ;
		}
	} ; // switch (pChannel->get_DirectionType())

	switch (pChannel->get_DelayType()) {
		case ClsChannel::delay:
		{
			add_AttrToTag(StartTag, AN_DELAY_TYPE, AV_DELAY_TYPE_DELAY) ;
			break ;
		}
		case ClsChannel::no_delay:
		{
			add_AttrToTag(StartTag, AN_DELAY_TYPE, AV_DELAY_TYPE_NO_DELAY) ;
			break ;
		}
		default:
		{
			add_AttrToTag(StartTag, AN_DELAY_TYPE, "none") ;
		}
	} ; // switch (pChannel->get_DelayType())

	(*m_pOutStream) << StartTag + "\n" ;

	/*==============================*/
	// ChanSourceID
	MyTag = create_StartTagFromName(TN_CHANNEL_SOURCE_ID) ;

	if (pChannel->Source_is_Env()) {
		add_AttrToTag(MyTag, AN_CHANNEL_SOURCE_IS_ENV, AV_CHANNEL_SOURCE_IS_ENV_TRUE) ;
	} else {
		add_AttrToTag(MyTag, AN_CHANNEL_SOURCE_IS_ENV, AV_CHANNEL_SOURCE_IS_ENV_FALSE) ;
	} ;

	(*m_pOutStream) << MyTag + "\n" ;
	(*m_pOutStream) << pChannel->get_SourceID_as_String() + "\n" ;
	(*m_pOutStream) << create_EndTagFromName(TN_CHANNEL_SOURCE_ID) + "\n" ;

	/*==============================*/

	/*==============================*/
	// ChanDestinationID
	MyTag = create_StartTagFromName(TN_CHANNEL_DESTINATION_ID) ;

	if (pChannel->Destination_is_Env()) {
		add_AttrToTag(MyTag, AN_CHANNEL_DESTINATION_IS_ENV, AV_CHANNEL_DESTINATION_IS_ENV_TRUE) ;
	} else {
		add_AttrToTag(MyTag, AN_CHANNEL_DESTINATION_IS_ENV, AV_CHANNEL_DESTINATION_IS_ENV_FALSE) ;
	} ;

	(*m_pOutStream) << MyTag + "\n" ;
	(*m_pOutStream) << pChannel->get_DestID_as_String() + "\n" ;
	(*m_pOutStream) << create_EndTagFromName(TN_CHANNEL_DESTINATION_ID) + "\n" ;

	/*==============================*/

	/*==============================*/
	// Coords
	pChannel->get_Coords_as_IntList(CoordsIntList) ;

	write_Element(TN_COORDS, Converter::IntList_to_String(CoordsIntList)) ;
	/*==============================*/

	/*==============================*/
	// Comps

	pChanName = pChannel->get_ChanName() ;
	write_ChanComp(pChanName) ;
	pChanSourceSigList = pChannel->get_ChanSourceSigList() ;
	write_ChanComp(pChanSourceSigList) ;
	pChanSourceCon = pChannel->get_ChanSourceCon() ;
	write_ChanComp(pChanSourceCon) ;
	pChanDestSigList = pChannel->get_ChanDestSigList() ;
	write_ChanComp(pChanDestSigList) ;
	pChanDestCon = pChannel->get_ChanDestCon() ;
	write_ChanComp(pChanDestCon) ;

	// Comps
	/*==============================*/

	(*m_pOutStream) << create_EndTagFromName(TN_CHANNEL) + "\n" ;
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Writer::write_RecSym(const Symbol * pSymbol) const {

	string StartTag ;
	string EndTag ;

	list<int> IntList ;
	ClsRecPaS PaS(0,0,10,10) ;

	switch (pSymbol->get_SymbolType()) {
		case EnumSym::TextSym:
		{
			StartTag = create_StartTagFromName(TN_RECSYM_TEXTSYM) ;
			add_AttrToTag(StartTag, AN_ID, pSymbol->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_RECSYM_TEXTSYM) ;
			break ;
		}
		case EnumSym::BlkInstRefSym:
		{
			StartTag = create_StartTagFromName(TN_RECSYM_BLK_INST_REF) ;
			add_AttrToTag(StartTag, AN_ID, pSymbol->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_RECSYM_BLK_INST_REF) ;
			break ;
		}
		case EnumSym::BlkTypeRefSym:
		{
			StartTag = create_StartTagFromName(TN_RECSYM_BLK_TYPE_REF) ;
			add_AttrToTag(StartTag, AN_ID, pSymbol->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_RECSYM_BLK_TYPE_REF) ;
			break ;
		}
		case EnumSym::ProInstRefSym:
		{
			StartTag = create_StartTagFromName(TN_RECSYM_PRO_INST_REF) ;
			add_AttrToTag(StartTag, AN_ID, pSymbol->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_RECSYM_PRO_INST_REF) ;
			break ;
		}
		case EnumSym::ProTypeRefSym:
		{
			StartTag = create_StartTagFromName(TN_RECSYM_PRO_TYPE_REF) ;
			add_AttrToTag(StartTag, AN_ID, pSymbol->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_RECSYM_PRO_TYPE_REF) ;
			break ;
		}
		case EnumSym::ProcedureInstRefSym:
		{
			StartTag = create_StartTagFromName(TN_RECSYM_PROCEDURE_INST_REF) ;
			add_AttrToTag(StartTag, AN_ID, pSymbol->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_RECSYM_PROCEDURE_INST_REF) ;
			break ;
		}
		default:
		{
			return ;
		}
	} ; //switch (pSymbol->get_SymbolType())

	if (pSymbol->is_referencing_Symbol()) {
		add_AttrToTag(StartTag, AN_ID_REF, pSymbol->get_referenced_Diagram_Model_ID()) ;
	} ;

	add_AttrToTag(StartTag, AN_PARENT_PAGE_ID, pSymbol->get_Page_ID()) ;


	(*m_pOutStream) << StartTag + "\n" ;

	write_Element(TN_CONTENT, pSymbol->get_content()) ;

	PaS = pSymbol->get_PaS() ;
	write_PaS(PaS.get_x_left_up(), PaS.get_y_left_up(), PaS.get_width(), PaS.get_height()) ;

	(*m_pOutStream) << EndTag + "\n" ;
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Writer::write_StateMaSym(const ClsStateMachineSymbol * pStateMachineSymbol) const {

	string StartTag ;
	string EndTag ;

	list<int> IntList ;
	ClsRecPaS PaS(0,0,10,10) ;

	switch (pStateMachineSymbol->get_SymbolType()) {
		case ClsStateMachineSymbol::start:
		{
			StartTag = create_StartTagFromName(TN_STATE_MA_SYM_START) ;
			add_AttrToTag(StartTag, AN_ID, pStateMachineSymbol->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_STATE_MA_SYM_START) ;
			break ;
		}
		case ClsStateMachineSymbol::state:
		{
			StartTag = create_StartTagFromName(TN_STATE_MA_SYM_STATE) ;
			add_AttrToTag(StartTag, AN_ID, pStateMachineSymbol->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_STATE_MA_SYM_STATE) ;
			break ;
		}
		case ClsStateMachineSymbol::output:
		{
			StartTag = create_StartTagFromName(TN_STATE_MA_SYM_OUTPUT) ;
			add_AttrToTag(StartTag, AN_ID, pStateMachineSymbol->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_STATE_MA_SYM_OUTPUT) ;
			break ;
		}
		case ClsStateMachineSymbol::input:
		{
			StartTag = create_StartTagFromName(TN_STATE_MA_SYM_INPUT) ;
			add_AttrToTag(StartTag, AN_ID, pStateMachineSymbol->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_STATE_MA_SYM_INPUT) ;
			break ;
		}
		case ClsStateMachineSymbol::condition:
		{
			StartTag = create_StartTagFromName(TN_STATE_MA_SYM_CONDITION) ;
			add_AttrToTag(StartTag, AN_ID, pStateMachineSymbol->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_STATE_MA_SYM_CONDITION) ;
			break ;
		}
		case ClsStateMachineSymbol::stop:
		{
			StartTag = create_StartTagFromName(TN_STATE_MA_SYM_STOP) ;
			add_AttrToTag(StartTag, AN_ID, pStateMachineSymbol->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_STATE_MA_SYM_STOP) ;
			break ;
		}
		case ClsStateMachineSymbol::task:
		{
			StartTag = create_StartTagFromName(TN_STATE_MA_SYM_TASK) ;
			add_AttrToTag(StartTag, AN_ID, pStateMachineSymbol->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_STATE_MA_SYM_TASK) ;
			break ;
		}
		case ClsStateMachineSymbol::decision:
		{
			StartTag = create_StartTagFromName(TN_STATE_MA_SYM_DECISION) ;
			add_AttrToTag(StartTag, AN_ID, pStateMachineSymbol->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_STATE_MA_SYM_DECISION) ;
			break ;
		}
		case ClsStateMachineSymbol::create_pro:
		{
			StartTag = create_StartTagFromName(TN_STATE_MA_SYM_CREATE_PRO) ;
			add_AttrToTag(StartTag, AN_ID, pStateMachineSymbol->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_STATE_MA_SYM_CREATE_PRO) ;
			break ;
		}
		case ClsStateMachineSymbol::call_procedure:
		{
			StartTag = create_StartTagFromName(TN_STATE_MA_SYM_CALL_PROCEDURE) ;
			add_AttrToTag(StartTag, AN_ID, pStateMachineSymbol->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_STATE_MA_SYM_CALL_PROCEDURE) ;
			break ;
		}
		case ClsStateMachineSymbol::procedure_start_state:
		{
			StartTag = create_StartTagFromName(TN_STATE_MA_SYM_PROCEDURE_START_STATE) ;
			add_AttrToTag(StartTag, AN_ID, pStateMachineSymbol->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_STATE_MA_SYM_PROCEDURE_START_STATE) ;
			break ;
		}
		case ClsStateMachineSymbol::procedure_return:
		{
			StartTag = create_StartTagFromName(TN_STATE_MA_SYM_PROCEDURE_RETURN) ;
			add_AttrToTag(StartTag, AN_ID, pStateMachineSymbol->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_STATE_MA_SYM_PROCEDURE_RETURN) ;
			break ;
		}
		default:
		{
			return ;
		}
	} ; //switch (pSymbol->get_SymbolType())

	add_AttrToTag(StartTag, AN_PARENT_PAGE_ID, pStateMachineSymbol->get_Page_ID()) ;


	(*m_pOutStream) << StartTag + "\n" ;

	write_Element(TN_CONTENT, pStateMachineSymbol->get_content()) ;

	PaS = pStateMachineSymbol->get_PaS() ;
	write_PaS(PaS.get_x_left_up(), PaS.get_y_left_up(), PaS.get_width(), PaS.get_height()) ;

	(*m_pOutStream) << EndTag + "\n" ;
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Writer::write_FlowLine(const ClsFlowLine * pFlowLine) const {

	string StartTag ;
	string EndTag ;

	list<int> CoordsIntList ;

	ClsRecPaS PaS(0,0,10,10) ;

	switch (pFlowLine->get_FlowLineType()) {
		case ClsFlowLine::normal:
		{
			StartTag = create_StartTagFromName(TN_NORMAL_FLOWLINE) ;
			add_AttrToTag(StartTag, AN_ID, pFlowLine->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_NORMAL_FLOWLINE) ;
			break ;
		}
		case ClsFlowLine::decision:
		{
			StartTag = create_StartTagFromName(TN_DECISION_FLOWLINE) ;
			add_AttrToTag(StartTag, AN_ID, pFlowLine->get_Model_ID()) ;

			EndTag = create_EndTagFromName(TN_DECISION_FLOWLINE) ;
			break ;
		}
		default:
		{
			return ;
		}
	} ; //switch (pSymbol->get_SymbolType())

	add_AttrToTag(StartTag, AN_PARENT_PAGE_ID, pFlowLine->get_Page_ID()) ;

	(*m_pOutStream) << StartTag + "\n" ;

	write_Element(TN_FLOWLINE_SOURCE_ID, pFlowLine->get_SourceID_as_String(), false) ;
	write_Element(TN_FLOWLINE_DESTINATION_ID, pFlowLine->get_DestID_as_String(), false) ;

	/*==============================*/
	// Coords
	pFlowLine->get_Coords_as_IntList(CoordsIntList) ;

	write_Element(TN_COORDS, Converter::IntList_to_String(CoordsIntList)) ;
	/*==============================*/

	if (pFlowLine->get_FlowLineType() == ClsFlowLine::decision) {

		ClsDecisionFlowLine * pDecisionFlowLine = ((ClsDecisionFlowLine*) pFlowLine) ;

		PaS = pDecisionFlowLine->get_PaS() ;

		write_PaS(PaS.get_x_left_up(), PaS.get_y_left_up(), PaS.get_width(), PaS.get_height()) ;

		write_Element(TN_CONTENT, pDecisionFlowLine->get_Content()) ;

		pDecisionFlowLine = NULL ;
	} ;

	(*m_pOutStream) << EndTag + "\n" ;
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Writer::write_Element(const string & TagName, const string & Content, const bool b_WithNewline) const {

	size_t index=0;
	string tmp(Content);

	(*m_pOutStream) << create_StartTagFromName(TagName);
	if (b_WithNewline)
		(*m_pOutStream) <<  "\n" ;
	while((index=tmp.find('<',index))!= string::npos)
		tmp.replace(index, 1, "&lt;");
	index=0;
	while((index=tmp.find('&',index))!= string::npos)
		tmp.replace(index, 1, "&amp;");
	(*m_pOutStream) << tmp;
	if (b_WithNewline)
		(*m_pOutStream) <<  "\n" ;
	(*m_pOutStream) << create_EndTagFromName(TagName) + "\n" ;
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Writer::write_Element(const string & TagName, const int Value, const bool b_WithNewline) const {

	write_Element(TagName, Converter::Int_to_String(Value), b_WithNewline) ;
} ;
//*******************************************************************************************************
string olSDL::ClsXML_Writer::create_StartTagFromName(const string & Name) const {

	return "<" + Name + ">" ;
} ;
//*******************************************************************************************************
string olSDL::ClsXML_Writer::create_EndTagFromName(const string & Name) const {

	return "</" + Name + ">" ;
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Writer::add_AttrToTag(string & Tag, const string & AttrName, const string & AttrValue) const {

	Tag.insert(Tag.length()-1, " " + AttrName + " = \"" + AttrValue + "\"") ;
} ;
//*******************************************************************************************************
void olSDL::ClsXML_Writer::add_AttrToTag(string & Tag, const string & AttrName, const int & AttrValue) const {

	add_AttrToTag(Tag, AttrName, Converter::Int_to_String(AttrValue)) ;

} ;
//*******************************************************************************************************

