#include <stdio.h>
#include <tk.h>
#include <locale.h>
#include <string>

#include "TclModelAccess.h"
#include "Helper.h"
#include "AppVars.h"

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>
#include <iostream>


// global variable
olSDL::ClassAppVars AppVars ;

int Tcl_AppInit(Tcl_Interp* interp) {

	using namespace olSDL ;

	// init Debug-Printer
	string DebugFileName = AppVars.get_DebugDir()  + "debug" ;
	//DebugPrinter::get_Instance(DebugFileName) ;
	DebugPrinter::get_Instance("") ;

	/******************************************************************************/
	// init Xerces (Initialize the XML4C2 system)
    	try
    	{
        	XMLPlatformUtils::Initialize();
    	}
    	catch(const XMLException& toCatch)
    	{
		DebugPrinter::print("Error during Xerces-c Initialization.") ;
    	}
	/******************************************************************************/
	/******************************************************************************/
	Tk_Window main;

	main = Tk_MainWindow(interp);
	if (Tcl_Init(interp) == TCL_ERROR) {
	    return TCL_ERROR;
	}
	if (Tk_Init(interp) == TCL_ERROR) {
	    return TCL_ERROR;
	}

	if (Tcl_PkgProvide(interp, "sdl", "1.0") != TCL_OK) {
	  return TCL_ERROR;
	}

	/******************************************************************************/
	// Interpreter erweitern

	Tcl_CreateCommand(interp, "create_EmptySysSpec", TclExt::TclModelAccess::create_EmptySysSpec, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	Tcl_CreateCommand(interp, "load_Model_from_File", TclExt::TclModelAccess::load_Model_from_File, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "store_Model_in_File", TclExt::TclModelAccess::store_Model_in_File, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	Tcl_CreateCommand(interp, "save_SDL_PR_in_File", TclExt::TclModelAccess::save_SDL_PR, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	Tcl_CreateCommand(interp, "modify_SysDiaName_in_Model", TclExt::TclModelAccess::modify_SysDiaName, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	Tcl_CreateCommand(interp, "get_System_Model_ID", TclExt::TclModelAccess::get_System_Model_ID, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateObjCommand(interp, "get_ReferencingDiagramID_from_Model", TclExt::TclModelAccess::get_ReferencingDiagramID, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	Tcl_CreateCommand(interp, "get_ProcedureFormalParameters_from_Model", TclExt::TclModelAccess::get_ProcedureDiagramFormalParameters, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "modify_ProcedureDiaFormalParameters_in_Model", TclExt::TclModelAccess::modify_ProcedureDiaFormalParameters, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	// for pages
	Tcl_CreateCommand(interp, "get_PageSize_from_Model", TclExt::TclModelAccess::get_PageSize, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "get_PageFrameInfo_from_Model", TclExt::TclModelAccess::get_PageFrameInfo, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	Tcl_CreateCommand(interp, "modify_PageWidth_in_Model", TclExt::TclModelAccess::modify_PageWidth, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "modify_PageHeight_in_Model", TclExt::TclModelAccess::modify_PageHeight, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "modify_PageFramePaS_in_Model", TclExt::TclModelAccess::modify_PageFramePaS, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	Tcl_CreateCommand(interp, "add_NewPage_after_in_Model", TclExt::TclModelAccess::add_NewPage_after, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	Tcl_CreateCommand(interp, "get_FirstPageID_from_Model", TclExt::TclModelAccess::get_FirstPageID, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "get_LastPageID_from_Model", TclExt::TclModelAccess::get_LastPageID, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "get_NextPageID_from_Model", TclExt::TclModelAccess::get_NextPageID, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "get_PreviousPageID_from_Model", TclExt::TclModelAccess::get_PreviousPageID, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	Tcl_CreateCommand(interp, "delete_Page_from_Model", TclExt::TclModelAccess::delete_Page, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	// for RecSym
	Tcl_CreateCommand(interp, "insert_RecSym_into_Model", TclExt::TclModelAccess::insert_RecSym, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "get_Symbols_from_Model", TclExt::TclModelAccess::get_Symbols, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "modify_RecSym_Content_in_Model", TclExt::TclModelAccess::modify_RecSym_Content, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "set_RecSymbolPaS_in_Model", TclExt::TclModelAccess::set_RecSym_PaS, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "delete_RecSym_from_Model", TclExt::TclModelAccess::delete_RecSym, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	Tcl_CreateCommand(interp, "get_AllReferencedDiagrams_from_Model", TclExt::TclModelAccess::get_AllReferencedDiagrams, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	// for channel
	Tcl_CreateCommand(interp, "get_Channels_from_Model" , olSDL::TclExt::TclModelAccess::get_Channels, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "insert_BidirecChannel_into_Model", olSDL::TclExt::TclModelAccess::insert_BidirecChannel, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "modify_Chan_Coords_in_Model", olSDL::TclExt::TclModelAccess::modify_Chan_Coords, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "modify_ChanComp_in_Model", olSDL::TclExt::TclModelAccess::modify_ChanComp, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "modify_Chan_Type_in_Model" , olSDL::TclExt::TclModelAccess::modify_Chan_Type, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "delete_Chan_from_Model" , olSDL::TclExt::TclModelAccess::delete_Chan, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	// for state machine symbols
	Tcl_CreateCommand(interp, "insert_StateMachineSymbol_into_Model" , olSDL::TclExt::TclModelAccess::insert_StateMachineSymbol, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "modify_StateMachineSymbol_Content_in_Model" , olSDL::TclExt::TclModelAccess::modify_StateMachineSymbol_Content, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "modify_StateMachineSymbol_PaS_in_Model" , olSDL::TclExt::TclModelAccess::modify_StateMachineSymbol_PaS, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "delete_StateMachineSymbol_from_Model" , olSDL::TclExt::TclModelAccess::delete_StateMachineSymbol, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "get_StateMachineSyms_from_Model" , olSDL::TclExt::TclModelAccess::get_StateMachineSyms, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	// for flow lines
	Tcl_CreateCommand(interp, "insert_FlowLine_into_Model" , olSDL::TclExt::TclModelAccess::insert_FlowLine, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	Tcl_CreateCommand(interp, "insert_DecisionFlowLine_into_Model" , olSDL::TclExt::TclModelAccess::insert_DecisionFlowLine, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	Tcl_CreateCommand(interp, "modify_FlowLineCoords_in_Model" , olSDL::TclExt::TclModelAccess::modify_FlowLineCoords, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	Tcl_CreateCommand(interp, "modify_DecisionFlowLinePaS_in_Model" , olSDL::TclExt::TclModelAccess::modify_DecisionFlowLinePaS, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "modify_DecisionFlowLineContent_in_Model" , olSDL::TclExt::TclModelAccess::modify_DecisionFlowLineContent, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	Tcl_CreateCommand(interp, "delete_FlowLine_from_Model" , olSDL::TclExt::TclModelAccess::delete_FlowLine, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;
	Tcl_CreateCommand(interp, "get_FlowLines_from_Model" , olSDL::TclExt::TclModelAccess::get_FlowLines, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) ;

	/******************************************************************************/
	char *pTempString ;

	/******************************************************************************/
	// Kernvariablen setzen

	string str_ApplInstPath = AppVars.get_Appl_Inst_Path( ) ;

	pTempString = new char[str_ApplInstPath.length()+1] ;
	str_ApplInstPath.copy(pTempString,string::npos);
	pTempString[str_ApplInstPath.length()] = 0;
	Tcl_SetVar(interp, "ApplInstPath" , pTempString, TCL_GLOBAL_ONLY) ;
	delete [ ] pTempString ;


	string str_DefaultProjectsDir = AppVars.get_DefaultProjectsDir() ;

	pTempString = new char[str_DefaultProjectsDir.length()+1] ;
	str_DefaultProjectsDir.copy(pTempString,string::npos) ;
	pTempString[str_DefaultProjectsDir.length()] = 0 ;
	Tcl_SetVar(interp, "DefaultProjectsDir" , pTempString, TCL_GLOBAL_ONLY) ;
	delete [ ] pTempString ;

	return TCL_OK;
}

int main(int argc, char **argv)
{
	using namespace olSDL ;

	Tcl_Main(argc, argv, Tcl_AppInit);

	/******************************************************************************/

	DebugPrinter::free_Instance() ;


	/******************************************************************************/
	// terminate Xerces
 	XMLPlatformUtils::Terminate() ;

	/******************************************************************************/

	exit(0);
}

