
#ifndef olSDL_CHANNEL_H
#define olSDL_CHANNEL_H

#include "PageObject.h"
#include "ObjectWithModel_ID.h"
#include "ChanComp.h"

namespace olSDL {

	class ClsChannel : public ClsPageObject, public ClsObjectWithModel_ID {

	public:
		enum e_DirectionType {both, from_source, from_destination} ;
		enum e_DelayType {delay, no_delay} ;

	//-------------------------------------------------------
	// member variable
	//-------------------------------------------------------
	private:
		e_DirectionType m_DirectionType ;
		e_DelayType m_DelayType ;

		ClsChanName* m_pChanName ;
		ClsChanSourceSigList* m_pChanSourceSigList ;
		ClsChanSourceCon* m_pChanSourceCon ;
		ClsChanDestSigList* m_pChanDestSigList ;
		ClsChanDestCon* m_pChanDestCon ;

		list<int> m_ChanCoords ;

		bool m_bSourceIsEnv ;
		int m_ChanSourceModel_ID ;

		bool m_bDestIsEnv ;
		int m_ChanDestModel_ID ;

	//-------------------------------------------------------
	//methods
	//-------------------------------------------------------
	public:
	//============
	//Constructor, Copy-Constructor, Assign-Op, ...
	ClsChannel(const int Symbol_ID, const int Page_ID, e_DelayType DelayType,
		ClsChanName ChanName,
		ClsChanSourceSigList ChanSourceSigList,
		ClsChanSourceCon ChanSourceCon,
		ClsChanDestSigList ChanDestSigList,
		ClsChanDestCon ChanDestCon,
		const list<int> & ChanCoords,
		bool bSourceIsEnv, const int ChanSourceModel_ID,
		bool bDestIsEnv, const int ChanDestModel_ID) ;

	ClsChannel(const int Symbol_ID, const int Page_ID, e_DelayType DelayType, e_DirectionType DirectionType, 
		ClsChanName ChanName,
		ClsChanSourceSigList ChanSourceSigList,
		ClsChanSourceCon ChanSourceCon,
		ClsChanDestSigList ChanDestSigList,
		ClsChanDestCon ChanDestCon,
		const list<int> & ChanCoords,
		bool bSourceIsEnv, const int ChanSourceModel_ID,
		bool bDestIsEnv, const int ChanDestModel_ID) ;

	// Copy

	// Assign

	// Des
	~ClsChannel() ;

	//============
	// getter
	void get_Coords_as_IntList(list<int> & IntList) const ;

	olSDL::ClsMoEleType::e_MoEle get_MoEleType() const ;

	string get_NameContent( ) const ;
	string get_SourceSigListContent( ) const ;
	string get_SourceConContent( ) const ;
	string get_DestSigListContent( ) const ;
	string get_DestConContent( ) const ;

	string get_DirectionType_as_String() const ;
	string get_DelayType_as_String() const ;

	e_DirectionType get_DirectionType() const ;
	e_DelayType get_DelayType() const ;

	string get_SourceID_as_String() const ;
	string get_DestID_as_String() const ;

	int get_DestID_as_Int() const ;
	int get_SourceID_as_Int() const ;

	bool Source_is_Env() const ;
	bool Destination_is_Env() const ;

	list<int> get_NamePas( ) const ;
	list<int> get_SourceSigListPas( ) const ;
	list<int> get_SourceConPas( ) const ;
	list<int> get_DestSigListPas( ) const ;
	list<int> get_DestConPas( ) const ;

	const ClsChanName* get_ChanName() const ;
	const ClsChanSourceSigList* get_ChanSourceSigList() const ;
	const ClsChanSourceCon* get_ChanSourceCon() const ;
	const ClsChanDestSigList* get_ChanDestSigList() const ;
	const ClsChanDestCon* get_ChanDestCon() const ;

	//============
	// setter
	void modify_Chan_DirectionType (e_DirectionType DirectionType) ;
	void modify_Chan_DelayType (e_DelayType DelayType) ;
	void modify_Chan_Coords(const list<int> & ChanCoords) ;

	void modify_Comp_Name(const string content, const ClsRecPaS & PaS) ;
	void modify_Comp_SourceSigList(const string content, const ClsRecPaS & PaS) ;
	void modify_Comp_SourceCon(const string content, const ClsRecPaS & PaS) ;
	void modify_Comp_DestSigList(const string content, const ClsRecPaS & PaS) ;
	void modify_Comp_DestCon(const string content, const ClsRecPaS & PaS) ;

	// for testing
	void print() const;

	} ; // class

	typedef list<const ClsChannel *> type_ListOf_Poi_to_ConstChannel ;

} // END: namespace ol_SDL

#endif
