
#ifndef olSDL_DIAGRAM_H
#define olSDL_DIAGRAM_H

#include <list>
#include <algorithm>

#include "BasicTypes.h"
#include "MyEnums.h"
#include "MyTypedefs.h"
#include "MoAccessInfo.h"
#include "PageManager.h"

#include "StateMachineSymbol.h"
#include "FlowLine.h"
#include "Syms.h"
#include "Channel.h"

namespace olSDL{

	typedef list<const Diagram *> type_ListOf_Poi_to_ConstDiagram ;

	typedef list<Symbol *> type_SymbolContainer ;
	typedef list<const Symbol *> type_ListOfPoi_to_ConstSymbol ;

	typedef list<ClsChannel *> type_ChannelContainer ;

	typedef list<ClsStateMachineSymbol *> type_StateMachineSymbolContainer ;
	typedef list<const ClsStateMachineSymbol *> type_ListOfPoi_to_ConstStateMachineSymbol ;

	typedef list<ClsFlowLine *> type_FlowLineContainer ;
	typedef list<const ClsFlowLine *> type_ListOfPoi_to_ConstFlowLine ;


	class Diagram : public ClsObjectWithModel_ID {

	static const string STR_DIA_TYPE_SYS ;
	static const string STR_DIA_TYPE_BLK ;
	static const string STR_DIA_TYPE_BLK_TYPE ;
	static const string STR_DIA_TYPE_PRO_DIA ;
	static const string STR_DIA_TYPE_PRO_TYPE ;
	static const string STR_DIA_TYPE_PROCEDURE ;
	//-------------------------------------------------------
	// member variable
	//-------------------------------------------------------
	private:
	EnumDia::Type m_DiagramType ;

	type_SymbolContainer m_Symbols ;

	type_ChannelContainer m_Channels ;

	type_StateMachineSymbolContainer m_StateMachineSymbols ;
	type_FlowLineContainer m_FlowLines ;

	const Symbol * m_pReferencingSymbol ;

	string m_str_HeaderPart_DiaType ;
	string m_str_HeaderPart_Name ;

	protected:
	ClsPageManager m_PageManager ;

	//-------------------------------------------------------
	//methods
	//-------------------------------------------------------
	public:
	//============
	//Constructor, Copy-Constructor, Assign-Op, ...
	Diagram(const int Diagram_ID, const EnumDia::Type DiagramType,
		     const string & HeaderPart_Name, const int First_Page_ID,
		     const Symbol * const pReferencingSymbol) ;
	// Des
	~Diagram( ) ;

	//============
	// setter
	void modify_HeaderPart_Name(const string str_HeaderPart_Name) ;

	// pages
	void modify_PageWidth(const int Page_ID, const int PageWidth, ClsMoAccessInfo & Info) ;
	void modify_PageHeight(const int Page_ID, const int PageHeight, ClsMoAccessInfo & Info) ;
	void modify_PageFramePaS(const int Page_ID, const int x_left_up, const int y_left_up, const int FrameWidth, const int FrameHeight, ClsMoAccessInfo & Info) ;

	// for symbols
	virtual void set_SymbolPaS(const int Symbol_ID, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) ;
	virtual void modify_SymbolContent(const int Symbol_ID, const string Content, ClsMoAccessInfo & Info) ;

	// for channels
	virtual void modify_Chan_DirectionType (const int Channel_ID, const ClsChannel::e_DirectionType DirectionType, ClsMoAccessInfo & Info) ;
	virtual void modify_Chan_DelayType (const int Channel_ID, const ClsChannel::e_DelayType DelayType, ClsMoAccessInfo & Info) ;
	virtual void modify_Chan_Coords(const int Channel_ID, list<int> & ChanCoords, ClsMoAccessInfo & Info) ;

	virtual void modify_Comp_Name(const int Channel_ID, const string content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) ;
	virtual void modify_Comp_SourceSigList(const int Channel_ID, const string content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) ;
	virtual void modify_Comp_SourceCon(const int Channel_ID, const string content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) ;
	virtual void modify_Comp_DestSigList(const int Channel_ID, const string content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) ;
	virtual void modify_Comp_DestCon(const int Channel_ID, const string content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) ;

	// state machine symbol
	virtual void modify_StateMachineSymPaS(const int Symbol_ID, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) ;
	virtual void modify_StateMachineSymContent(const int Symbol_ID, const string Content, ClsMoAccessInfo & Info) ;

	// FlowLine
	virtual void modify_FlowLineCoords(const int FlowLine_ID, list<int> & FlowLineCoords, ClsMoAccessInfo & Info) ;

	virtual void modify_DecisionFlowLinePaS(const int DecisionFlowLine_ID, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) ;
	virtual void modify_DecisionFlowLineContent(const int DecisionFlowLine_ID, const string Content, ClsMoAccessInfo & Info) ;

	//============
	// getter
	private:
	void get_HeaderPart_Index(const int Page_ID, string & str_HeaderPart_Index, ClsMoAccessInfo & Info) const ;
	void get_HeaderPart_DiagramType(string & str_HeaderPart_DiaType) const ;

	public:
	void get_HeaderPart_Name(string & str_HeaderPart_Name) const ;
	EnumDia::Type get_DiagramType( ) const ;

	virtual bool is_referenced_Diagram() const =0 ;
	const Symbol * get_referencing_Symbol() const ;
	int get_referencing_Symbol_Model_ID() const ;

	virtual void get_AllReferencedDiagrams(type_ListOf_Poi_to_ConstDiagram & ResultList, ClsMoAccessInfo & Info) const ;

	// pages
	void get_PageSize(const int Page_ID, int & PageWidth, int & PageHeight, ClsMoAccessInfo & Info) const ;

	void get_PageFrameInfo(const int Page_ID, ClsRecPaS & PaS, string & HP_DiaType, string & HP_Name,
							string & HP_Index, ClsMoAccessInfo & Info) const ;

	int get_NextPageID(const int PredecessorPage_ID, ClsPageManager::e_PageIndex & PageIndex, ClsMoAccessInfo & Info) const ;
	int get_PreviousPageID(const int AncestorPage_ID, ClsPageManager::e_PageIndex & PageIndex, ClsMoAccessInfo & Info) const ;

	int get_FirstPageID(ClsMoAccessInfo & Info) const ;
	int get_LastPageID(ClsMoAccessInfo & Info) const ;

	void get_IDsOfPages(list<int> & IDsOfPages) const ;

	// for symbols
	virtual const Symbol * get_Symbol(const int Symbol_ID, ClsMoAccessInfo & Info) const ;

	virtual void get_Syms(const EnumSym::Type SymbolType, const int Page_ID,
					type_ListOf_Poi_to_ConstSym & ResultList,
					ClsMoAccessInfo & Info) const ;

	// for channels
	virtual const ClsChannel * get_Channel(const int Channel_ID, ClsMoAccessInfo & Info) const ;

	virtual void get_Channels(const int Page_ID, type_ListOf_Poi_to_ConstChannel & ResultList,
						ClsMoAccessInfo & Info) const ;

	// state machine symbol
	virtual const ClsStateMachineSymbol * get_StateMachineSymbol(const int Symbol_ID, ClsMoAccessInfo & Info) const ;

	virtual void get_StateMachineSyms(const ClsStateMachineSymbol::e_StateMachineSymbolType StateMachineSymbolType, const int Page_ID,
					type_ListOfPoi_to_ConstStateMachineSymbol & ResultList, ClsMoAccessInfo & Info) const ;

	virtual void get_StateMachineSyms(const int Page_ID, type_ListOfPoi_to_ConstStateMachineSymbol & ResultList, ClsMoAccessInfo & Info) const ;

	// FlowLine
	virtual const ClsFlowLine * get_FlowLine(const int FlowLine_ID, ClsMoAccessInfo & Info) const ;

	virtual void get_FlowLines (const int Page_ID, type_ListOfPoi_to_ConstFlowLine & ResultList, ClsMoAccessInfo & Info) const ;


	//============
	// insert methods
	void add_NewPage_after(const int NewPage_ID, const int PredecessorPage_ID, ClsMoAccessInfo & Info) ;


	virtual const Symbol * insert_Symbol(const EnumSym::Type SymbolType, const int Symbol_ID, const int Page_ID,
				       	            const int x_left_up, const int y_left_up, const int width, const int height ,
						    const string content, const int ReferencedDiagramID, const int ReferencedDiagramFirstPageID,
						    ClsMoAccessInfo & Info) ;

	virtual const ClsChannel * insert_Channel(const int Symbol_ID, const int Page_ID, ClsChannel::e_DelayType DelayType,
						 	    ClsChanName ChanName, ClsChanSourceSigList ChanSourceSigList, ClsChanSourceCon ChanSourceCon,
						 	    ClsChanDestSigList ChanDestSigList, ClsChanDestCon ChanDestCon,
							    list<int> & ChanCoords,
						    	    bool bSourceIsEnv, const int ChanSourceModel_ID,
							    bool bDestIsEnv, const int ChanDestModel_ID,
						 	    ClsMoAccessInfo & Info) ;

	virtual const ClsChannel * insert_Channel(const int Symbol_ID, const int Page_ID, ClsChannel::e_DelayType DelayType, ClsChannel::e_DirectionType DirectionType,
						 	    ClsChanName ChanName, ClsChanSourceSigList ChanSourceSigList, ClsChanSourceCon ChanSourceCon,
						 	    ClsChanDestSigList ChanDestSigList, ClsChanDestCon ChanDestCon,
							    list<int> & ChanCoords,
						    	    bool bSourceIsEnv, const int ChanSourceModel_ID,
							    bool bDestIsEnv, const int ChanDestModel_ID,
						 	    ClsMoAccessInfo & Info) ;

	// state machine symbol
	virtual const ClsStateMachineSymbol * insert_StateMachineSymbol(const ClsStateMachineSymbol::e_StateMachineSymbolType SymbolType, const int Symbol_ID,
				const int Page_ID, const int x_left_up, const int y_left_up, const int width, const int height ,
				const string content, ClsMoAccessInfo & Info) ;
	// FlowLine
	virtual const ClsNormalFlowLine * insert_FlowLine(const int FlowLine_ID, const int Page_ID, list<int> & FlowLineCoords,
				    	         const int FlowLineSourceModel_ID, const int FlowLineDestModel_ID,
						 ClsMoAccessInfo & Info) ;

	virtual const ClsDecisionFlowLine * insert_DecisionFlowLine(const int FlowLine_ID, const int Page_ID, list<int> & FlowLineCoords,
				    	         const int FlowLineSourceModel_ID, const int FlowLineDestModel_ID, const string & content, const ClsRecPaS & PaS,
						 ClsMoAccessInfo & Info) ;

	//============
	// delete methods

	virtual void delete_Symbol(const int Symbol_ID, ClsMoAccessInfo & Info) ;

	virtual void delete_Channel(const int Channel_ID, ClsMoAccessInfo & Info) ;

	virtual void delete_StateMachineSymbol(const int Symbol_ID, ClsMoAccessInfo & Info)  ;

	virtual void delete_FlowLine(const int FlowLine_ID, ClsMoAccessInfo & Info) ;

	virtual void delete_Page(const int Page_ID, ClsMoAccessInfo & Info) ;

	// for testing
	void printChannels() const;

	} ; // class Diagram

} // END: namespace ol_SDL


#endif
