
#ifndef olSDL_MO_ELE_MAP_H
#define olSDL_MO_ELE_MAP_H

#include <map>
#include "Diagram.h"
#include "BasicTypes.h"
#include "StateMachineSymbol.h"
#include "FlowLine.h"

namespace olSDL{

	//=============================================================================
	class ClsMoEleMap {

		//=============================================================================
		class ClsMoEleInfo {

		public:

		enum e_MoEleBasic {basic_none, Diagram, Symbol, StateMachineSymbol, Channel, FlowLine} ;

		//-------------------------------------------------------
		// member variable
		//-------------------------------------------------------
		private:
		e_MoEleBasic m_MoEleBasicType ;
		olSDL::ClsMoEleType::e_MoEle m_MoEleType ;
		olSDL::Diagram * m_pDiagram ;

		//-------------------------------------------------------
		//methods
		//-------------------------------------------------------
		public:
		//============
		//Constructor, Copy-Constructor, Assign-Op, ...
		ClsMoEleInfo(const ClsMoEleType::e_MoEle , olSDL::Diagram * const pDiagram) ;

		// for use in map !!
		ClsMoEleInfo() ;
		// Copy: for use in map !!
		ClsMoEleInfo(const ClsMoEleInfo & Element) ;
		// Assign: for use in map !!
		ClsMoEleInfo& operator=(const ClsMoEleInfo & Element) ;

		//============
		//getter
		e_MoEleBasic get_MoEleBasicType() const ;
		olSDL::ClsMoEleType::e_MoEle get_MoEleType( ) const ;

		olSDL::Diagram * get_Diagram( ) const ;
		olSDL::Diagram * get_EnclosingDiagram() const ;

		} ; // class ClsMoEleInfo


	public:
	enum EnumAccessType {insert_into_diagram, get_diagram, get_enclosing_diagram, get_enclosing_diagram_for_deleting_object,
		get_enclosing_diagram_for_modifying_object} ;

	private:
	typedef map<int, ClsMoEleInfo> typ_MoEleMap ;

	//-------------------------------------------------------
	// member variable
	//-------------------------------------------------------
	private:
	typ_MoEleMap m_InternalMoEleMap ;

	// ID-Generator attributes
	list<int> m_ID_Pool ;
	int m_max_generated_ID ;

	//-------------------------------------------------------
	//methods
	//-------------------------------------------------------

	public:
	//============
	//Constructor, Copy-Constructor, Assign-Op, ...

	ClsMoEleMap() ;

	//============
	//getter
	olSDL::ClsMoEleType::e_MoEle get_MoEleType(const int MoEle_ID) const ;

	Diagram * find_MoEle(const int MoEle_ID, const EnumAccessType AccessType, ClsMoAccessInfo & Info)  const ;

	//============
	void add_MoEle (const ClsObjectWithModel_ID * const pObjectWithModel_ID, Diagram * const pDiagram) ;

	void delete_MoEle(int Model_ID, ClsMoAccessInfo & Info) ;

	void print_Map() const ;

	// ID-Generator methods
	int get_ID() ;

	private:

	int get_max_ID_from_Map() ;
	bool ID_exists_in_Map(int ID) ;

	} ; // class ClsMoEleMap

} // END: namespace ol_SDL


#endif
