
#ifndef olSDL_SDL_PR_WRITER_H
#define olSDL_SDL_PR_WRITER_H

#include <string>
#include <fstream>

#include "SDL_PR_WriterHelper.h"
#include "ProcessDia.h"
#include "ProcedureDia.h"
#include "SysSpec.h"

namespace olSDL {

	// ===================================================================
	// ===================================================================

	class ClsSDL_PR_Writer_ErrorReporter {

	//-------------------------------------------------------
	// member variable
	//-------------------------------------------------------
	public:
	enum e_ErrorType {not_specified, StateMaSym} ;

	private:
		e_ErrorType m_ErrorType ;

	public:
	//============
	//Constructor, Copy-Constructor, Assign-Op, ...
	ClsSDL_PR_Writer_ErrorReporter(e_ErrorType ErrorType) ;

	//============
	// getter
	e_ErrorType get_ErrorType() const ;

	} ; // ClsSDL_PR_Writer_ErrorReporter

	// ===================================================================
	// ===================================================================

	class ClsSDL_PR_Writer_UnspecifiedError : public ClsSDL_PR_Writer_ErrorReporter {

	//-------------------------------------------------------
	// member variable
	//-------------------------------------------------------
	private:

	public:
	//============
	//Constructor, Copy-Constructor, Assign-Op, ...
	ClsSDL_PR_Writer_UnspecifiedError() ;

	//============
	// getter

	} ; // ClsSDL_PR_Writer_UnspecifiedError

	// ===================================================================
	// ===================================================================

	class ClsSDL_PR_Writer_StateMaSymError : public ClsSDL_PR_Writer_ErrorReporter {

	//-------------------------------------------------------
	// member variable
	//-------------------------------------------------------
	public:
	enum e_StateMaErrorType {sms_none, sms_illegal_start_state, sms_transition_not_complete} ;

	private:
	int m_StateMaSymID ;
	int m_PageID ;
	int m_DiagramID ;
	e_StateMaErrorType m_StateMaErrorType ;

	public:
	//============
	//Constructor, Copy-Constructor, Assign-Op, ...

	ClsSDL_PR_Writer_StateMaSymError(const SysSpec * pSysSpec, const ClsStateMachineSymbol * pStateMachineSymbol, e_StateMaErrorType ErrorType) ;

	//============
	// getter
	int get_Symbol_ID() const ;
	int get_PageID() const ;
	int get_DiagramID() const ;
	e_StateMaErrorType get_StateMaErrorType() const ;


	} ; // ClsSDL_PR_Writer_UnspecifiedError

	// ===================================================================
	// ===================================================================

	class ClsSDL_PR_Writer : public ClsSDL_PR_WriterHelper {

	//-------------------------------------------------------
	// member variable
	//-------------------------------------------------------
	private:

	string m_FileName ;
	ofstream * m_pOutStream ;
	const SysSpec * m_pSystemSpec ;

	protected:
	static const string KW_SYSDIA ;
	static const string KW_END_SYSDIA ;

	static const string KW_BLKDIA ;
	static const string KW_END_BLKDIA ;

	static const string KW_PRODIA ;
	static const string KW_END_PRODIA ;

	static const string KW_PROCEDUREDIA ;
	static const string KW_END_PROCEDUREDIA ;

	static const string KW_CHANNEL ;
	static const string KW_END_CHANNEL ;
	static const string KW_CHANNEL_DELAY ;
	static const string KW_CHANNEL_NODELAY ;

	static const string KW_CHANNEL_FROM ;
	static const string KW_CHANNEL_TO ;
	static const string KW_CHANNEL_WITH ;
	static const string KW_CHANNEL_VIA ;
	static const string KW_CHANNEL_ENVIROMENT ;

	static const string KW_CHANNEL_CONNECT ;
	static const string KW_CHANNEL_AND ;

	static const string KW_BLOCK_REFERENCED ;
	static const string KW_BLOCK_TYPE_REFERENCED ;
	static const string KW_PROCESS_REFERENCED ;
	static const string KW_PROCESS_TYPE_REFERENCED ;
	static const string KW_PROCEDURE_REFERENCED ;

	static const string KW_REFERENCED ;

	static const string KW_STATE_MA_START_SYM ;
	static const string KW_STATE_MA_STATE_SYM ;

	static const string KW_STATE_MA_INPUT_SYM ;
	static const string KW_STATE_MA_OUTPUT_SYM ;
	static const string KW_STATE_MA_CONDITION_SYM ;
	static const string KW_STATE_MA_TASK_SYM ;
	static const string KW_STATE_MA_STOP_SYM ;
	static const string KW_STATE_MA_DECISION_SYM ;
	static const string KW_STATE_MA_CREATE_PROC_SYM ;
	static const string KW_STATE_MA_CALL_PROCEDURE_SYM ;

	static const string KW_STATE_MA_PROCEDURE_START_STATE_SYM ;
	static const string KW_STATE_MA_PROCEDURE_RETURN_SYM ;

	static const string KW_STATE_MA_ENDDECISION ;

	static const string KW_NEXTSTATE ;

	static const string KW_JOIN ;


	//-------------------------------------------------------
	//methods
	//-------------------------------------------------------
	public:
	//============
	//Constructor, Copy-Constructor, Assign-Op, ...
	ClsSDL_PR_Writer() ;

	// Copy

	// Assign

	// Des
	~ClsSDL_PR_Writer() ;

	//============

	void write_SysSpec(const SysSpec * pSystemSpec, const string & FileName, const bool b_Inline = false) ;

	//============
	// getter

	//============
	// setter

	private:

	void write_Dia(const Diagram * pDiagram, const bool b_Inline) ;

	void write_ProDia(const ClsProcessDia * pProcessDia, const bool b_Inline) ;
	void write_ProcedureDia(const ClsProcedureDia * pProcedureDia, const bool b_Inline) ;
	void write_HL_InstDia(const Diagram * pHL_InstDiagram, const bool b_Inline) ;

	void write_RecSym(const Symbol * pSymbol, const int ParentDiagram_Model_ID) const ;
	void write_Channel(const ClsChannel * pChannel, const int ParentDiagram_Model_ID) const ;
	void write_ChannelConnection(const ClsChannel * pChannel, const int ParentDiagram_Model_ID) const ;


	void write_StateMaSymAndSuccessors(const bool b_TransitionStart,
			const ClsStateMachineSymbol * pStateMachineSymbol,
			const int ParentDiagram_Model_ID) ;
	void write_ProcedureStateMaSymAndSuccessors(const bool b_TransitionStart,
			const ClsStateMachineSymbol * pStateMachineSymbol,
			const int ParentDiagram_Model_ID) ;


	void write_StateMaSymEnclosedState(const ClsStateMachineSymbol * pStateMachineSymbol,
			const int ParentDiagram_Model_ID) ;
	void write_ProcedureStateMaSymEnclosedState(const ClsStateMachineSymbol * pStateMachineSymbol,
			const int ParentDiagram_Model_ID) ;


	//============
	// helper

	string create_StartStringFromName(const string & Name, const string & Content) const ;

	void write_StringBlockToFile	(const int Model_ID, const int ParentDiagram_Model_ID,
							const bool b_is_GR_Component, const bool b_WithEndingNewline,
							const string & s) const ;


	} ; // class


} // END: namespace ol_SDL


#endif
