#ifndef olSDL_SYS_SPEC
#define olSDL_SYS_SPEC

#include "MoEleMap.h"
#include "SysDia.h"
#include "ProcessDia.h"
#include "MoAccessInfo.h"

namespace olSDL {

	class SysSpec {

	//-------------------------------------------------------
	// member variable
	//-------------------------------------------------------
	private:
	ClsMoEleMap * m_pMoEleMap ;
	ClsSysDia * m_pRoot ;

	//-------------------------------------------------------
	//methods
	//-------------------------------------------------------
	private:
	void reset() ;

	public:
	//============
	//Constructor, Copy-Constructor, Assign-Op, ...
	SysSpec() ;
	// Des
	~SysSpec() ;

	//============
	// setter

	// pages
	void modify_PageWidth(const int Diagram_Model_ID, const int Page_ID, const int PageWidth, ClsMoAccessInfo & Info) ;
	void modify_PageHeight(const int Diagram_Model_ID, const int Page_ID, const int PageHeight, ClsMoAccessInfo & Info) ;
	void modify_PageFramePaS(const int Diagram_Model_ID, const int Page_ID, const int x_left_up, const int y_left_up,
		const int FrameWidth, const int FrameHeight, ClsMoAccessInfo & Info) ;

	void add_NewPage_after(const int Diagram_Model_ID, const int PredecessorPage_ID, ClsMoAccessInfo & Info) ;
	void append_NewPage(const int Diagram_Model_ID, ClsMoAccessInfo & Info) ;


	// symbol
	void set_SymbolPaS(const int Symbol_ID, const int x_left_up, const int y_left_up, const int width, const int height, ClsMoAccessInfo & Info) ;
	void modify_SymbolContent(const int Symbol_ID, const string Content, ClsMoAccessInfo & Info) ;

	// channel
	void modify_Chan_DirectionType (const int Channel_ID, const ClsChannel::e_DirectionType DirectionType, ClsMoAccessInfo & Info) ;
	void modify_Chan_DelayType (const int Channel_ID, const ClsChannel::e_DelayType DelayType, ClsMoAccessInfo & Info) ;
	void modify_Chan_Coords(const int Channel_ID, list<int> & ChanCoords, ClsMoAccessInfo & Info) ;

	void modify_ChanCompName(const int Channel_ID, const string Content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) ;
	void modify_ChanSourceSig(const int Channel_ID, const string Content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) ;
	void modify_ChanSourceCon(const int Channel_ID, const string Content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) ;
	void modify_ChanDestSig(const int Channel_ID, const string Content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) ;
	void modify_ChanDestCon(const int Channel_ID, const string Content, const ClsRecPaS & PaS, ClsMoAccessInfo & Info) ;

	// state machine symbol
	void modify_StateMachineSymPaS(const int StateMachineSymbol_ID, const int x_left_up, const int y_left_up, const int width, const int height, ClsMoAccessInfo & Info) ;
	void modify_StateMachineSymContent(const int StateMachineSymbol_ID, const string Content, ClsMoAccessInfo & Info) ;

	// flow line
	void modify_FlowLineCoords(const int FlowLineID, list<int> & FlowLineCoords, ClsMoAccessInfo & Info) ;

	void modify_DecisionFlowLinePaS(const int DecisionFlowLine_ID, const int x_left_up, const int y_left_up, const int width, const int height, ClsMoAccessInfo & Info) ;
	void modify_DecisionFlowLineContent(const int DecisionFlowLine_ID, const string Content, ClsMoAccessInfo & Info) ;

	//============
	// getter
	int get_System_Model_ID() const ;

	int get_ReferencingDiagramID(const int referenced_Diagram_Model_ID, ClsMoAccessInfo & Info) const ;

	void get_ProcedureDiaFormalParameters(const int Diagram_Model_ID,
		string & str_HP_FormalParameters, ClsMoAccessInfo & Info) const;

	void modify_ProcedureDiaFormalParameters(const int Diagram_Model_ID,
		const string & str_HP_FormalParameters, ClsMoAccessInfo & Info) ;

	// diagram
	const Diagram * get_Diagram(const int Diagram_Model_ID, ClsMoAccessInfo & Info) const ;
	void get_AllReferencedDiagrams(const int Diagram_Model_ID, type_ListOf_Poi_to_ConstDiagram & ResultList, ClsMoAccessInfo & Info) const ;

	const ClsSysDia * get_SysDia() const ;

	void modify_SysDiaName(const string str_HeaderPart_Name, ClsMoAccessInfo & Info) ;

	// symbols
	const Symbol * get_RecSymbol(const int Symbol_ID, ClsMoAccessInfo & Info) const ;

	void get_Syms(const EnumSym::Type SymbolType,
			      const int Diagram_Model_ID, const int Page_ID,
			      type_ListOf_Poi_to_ConstSym & ResultList,
			      ClsMoAccessInfo & Info) const ;
	// channel
	void get_Channels(const int Diagram_Model_ID, const int Page_ID,
					type_ListOf_Poi_to_ConstChannel & ResultList,
					ClsMoAccessInfo & Info) const ;

	// flow line
	void get_FlowLines(const int Diagram_Model_ID, const int Page_ID,
					type_ListOfPoi_to_ConstFlowLine & ResultList,
					ClsMoAccessInfo & Info) const ;

	// page
	void get_PageSize(const int Diagram_Model_ID, const int Page_ID,
				    int & PageWidth, int & PageHeight, ClsMoAccessInfo & Info) const ;

	void get_PageFrameInfo(const int Diagram_Model_ID, const int Page_ID, ClsRecPaS & PaS,
							string & HP_DiaType, string & HP_Name, string & HP_Index, ClsMoAccessInfo & Info) const;

	int get_FirstPageID(const int Diagram_Model_ID, ClsMoAccessInfo & Info) const ;
	int get_LastPageID(const int Diagram_Model_ID, ClsMoAccessInfo & Info) const ;

	int get_NextPageID(const int Diagram_Model_ID, const int PredecessorPage_ID, ClsPageManager::e_PageIndex & PageIndex, ClsMoAccessInfo & Info) const ;
	int get_PreviousPageID(const int Diagram_Model_ID, const int AncestorPage_ID, ClsPageManager::e_PageIndex & PageIndex, ClsMoAccessInfo & Info) const ;

	// state machine symbol
	const ClsStateMachineSymbol * get_StateMachineSymbol(const int Symbol_ID, ClsMoAccessInfo & Info) const ;

	void get_OutgoingFlowLinesFromStateMaSym(const int Symbol_ID, type_ListOfPoi_to_ConstFlowLine & ResultList, ClsMoAccessInfo & Info) const ;

	void get_InComingFlowLinesFromStateMaSym(const int Symbol_ID, type_ListOfPoi_to_ConstFlowLine & ResultList, ClsMoAccessInfo & Info) const ;

	void get_StateMachineSyms(const ClsStateMachineSymbol::e_StateMachineSymbolType StateMachineSymbolType,
					const int Diagram_Model_ID, const int Page_ID,
					type_ListOfPoi_to_ConstStateMachineSymbol & ResultList, ClsMoAccessInfo & Info) const ;

	void get_StartingStateMachineSymsFromDiagram(const ClsStateMachineSymbol::e_StateMachineSymbolType StateMachineSymbolType,
		const int Diagram_Model_ID, type_ListOfPoi_to_ConstStateMachineSymbol & ResultList, ClsMoAccessInfo & Info) const ;

	//============

	int get_EnclosingDiagramID(const int Symbol_ID, ClsMoAccessInfo & Info) const ;

	//============
	// insert methods

	// symbol
	void insert_Symbol(const EnumSym::Type SymbolType, const int Location_Model_ID, const int Page_ID,
			           const int x_left_up, const int y_left_up, const int width, const int height , const string content,
			           ClsMoAccessInfo & Info, int & New_Model_ID) ;

	// channel
	void	insert_Channel(const int Location_Model_ID, const int Page_ID, ClsChannel::e_DelayType DelayType,
					const ClsChanName ChanName, const ClsChanSourceSigList ChanSourceSigList, const ClsChanSourceCon ChanSourceCon,
				 	const ClsChanDestSigList ChanDestSigList, const ClsChanDestCon ChanDestCon,
					list<int> & ChanCoords,
					bool bSourceIsEnv, const int ChanSourceModel_ID,
					bool bDestIsEnv, const int ChanDestModel_ID,
					ClsMoAccessInfo & Info, int & New_Model_ID) ;

	void	insert_Channel(const int Location_Model_ID, const int Page_ID, ClsChannel::e_DelayType DelayType, ClsChannel::e_DirectionType DirectionType,
					const ClsChanName ChanName, const ClsChanSourceSigList ChanSourceSigList, const ClsChanSourceCon ChanSourceCon,
				 	const ClsChanDestSigList ChanDestSigList, const ClsChanDestCon ChanDestCon,
					list<int> & ChanCoords,
					bool bSourceIsEnv, const int ChanSourceModel_ID,
					bool bDestIsEnv, const int ChanDestModel_ID,
					ClsMoAccessInfo & Info, int & New_Model_ID) ;

	// state machine symbol
	void insert_StateMachineSymbol(const ClsStateMachineSymbol::e_StateMachineSymbolType SymbolType,
		const int Location_Model_ID, const int Page_ID,
		const int x_left_up, const int y_left_up, const int width, const int height , const string content,
		ClsMoAccessInfo & Info, int & New_Model_ID) ;

	// flow line
	void	insert_FlowLine(const int Location_Model_ID, const int Page_ID,
					list<int> & FlowLineCoords, const int FlowLineSourceModel_ID, const int FlowLineDestModel_ID,
					ClsMoAccessInfo & Info, int & New_Model_ID) ;

	void insert_DecisionFlowLine(const int Location_Model_ID, const int Page_ID,
					list<int> & FlowLineCoords, const int FlowLineSourceModel_ID, const int FlowLineDestModel_ID,
					const string & content, const ClsRecPaS & PaS,
					ClsMoAccessInfo & Info, int & New_Model_ID) ;

	//============
	// delete methods

	void delete_Symbol(const int Model_ID, ClsMoAccessInfo & Info) ;
	void delete_Channel(const int Channel_ID, ClsMoAccessInfo & Info) ;
	void delete_StateMachineSymbol(const int StateMachineSymbol_Model_ID, ClsMoAccessInfo & Info) ;
	void delete_FlowLine(const int FlowLineID, ClsMoAccessInfo & Info) ;
	void delete_Page(const int Diagram_Model_ID, const int Page_ID, ClsMoAccessInfo & Info) ;

	//============
	// load and store methods
	bool store(const string & FileName) ;
	void load(const string & FileName) ;

	// void load(const string & FileName, SysSpec* pSysSpec) ;


	//============
	// for testing
	void print_Map( ) ;

	private:
	void get_StateMachineSymsFromDiagram(const ClsStateMachineSymbol::e_StateMachineSymbolType StateMachineSymbolType,
					const int Diagram_Model_ID, type_ListOfPoi_to_ConstStateMachineSymbol & ResultList, ClsMoAccessInfo & Info) const ;


	} ; // class SysSpec

} // END: namespace ol_SDL

#endif
