#ifndef olSDL_TCL_MODEL_ACCESS_H
#define olSDL_TCL_MODEL_ACCESS_H

#include <tcl.h>
#include <string>
#include <list>

#include "MyEnums.h"
#include "MyTypedefs.h"
#include "Channel.h"
#include "StateMachineSymbol.h"
#include "ProcessDia.h"
#include "FlowLine.h"

		//MAR = Model Access Result
		static char * const pMAR_Array_Name = "::olSDL_MAR::arr_MAR" ;
		static char * const pMAR_OK = "MAR_OK" ;
		static char * const pMAR_ERROR = "MAR_ERROR" ;

namespace olSDL{

	namespace TclExt {

		class TclModelAccess {

		//*******************************************************************************************************
		private:
		//************************
		static void createValScr(const int Value, string & ScriptPart) ;
		static void createValListScr(const string & Value, string & ScriptPart) ;
		static void createValListScrForStrings(const string & Value, string & ScriptPart) ;

		static void createValListScr(const type_ListOf_Poi_to_ConstSym & ModelResultList , string & ScriptPart) ;
		static void createValListScr(const type_ListOf_Poi_to_ConstChannel & ModelResultList , string & ScriptPart) ;
		static void createValListScr(const type_ListOfPoi_to_ConstStateMachineSymbol & ModelResultList , string & ScriptPart) ;
		static void createValListScr(const type_ListOfPoi_to_ConstFlowLine & ModelResultList, string & ScriptPart) ;
		static void createValListScr(const type_ListOf_Poi_to_ConstDiagram & ModelResultList, string & ScriptPart) ;

		static void createValListScr(const list<int> & IntList, string & ScriptPart) ;

		static void createValListScr(const Symbol* pSymbol, string & ScriptPart) ;
		static void createValListScr(const ClsChannel* pChannel, string & ScriptPart) ;
		static void createValListScr(const ClsStateMachineSymbol* pStateMachineSymbol, string & ScriptPart) ;
		static void createValListScr(const ClsFlowLine* pFlowLine, string & ScriptPart) ;

		//************************
		static int OneDimVarSet (Tcl_Interp *interp, const string & NameTclVar, const string & IndexName, const string & Value);
		static int OneDimVarSet (Tcl_Interp *interp, const string & NameTclVar, const string & IndexName, const int Value);
		//************************
		static int TwoDimVarSet (Tcl_Interp *interp, const string & NameTclVar, const int IndexA,
									     const string & IndexNameB, const string & Value);
		static int TwoDimVarSet (Tcl_Interp *interp, const string & NameTclVar, const string & IndexNameA,
									     const string & IndexNameB, const string & Value);
		//************************
		static void create_Script_Var_set (const char * pNameTclVar,
									const char * pVarValue,
									string & script) ;
		//************************
		static void determine_SymbolType(const string str_SymbolType, EnumSym::Type & SymbolType) ;

		static bool determine_ChannelDelayType(const string str_ChannelDelayType, ClsChannel::e_DelayType & ChannelDelayType) ;

		static bool determine_ChannelDirectionType(const string str_ChannelDirectionType, ClsChannel::e_DirectionType & DirectionType) ;

		static bool determine_ChanCompType(const string str_ChanCompType, ClsChanComp::e_CompType & ChanCompType) ;

		static void determine_StateMachineSymbolType(const string str_StateMachineSymbolType,
			ClsStateMachineSymbol::e_StateMachineSymbolType & StateMachineSymbolType)  ;

		static bool ChannelConPoint_is_Enviroment(string strChannelConPoint) ;

		static	void get_PaS_from_space_separated_IntListString(string strIntList, ClsRecPaS & PaS) ;

		static void get_IntList_from_space_separated_IntListString(string strIntList, list<int> & IntList) ;

		static string get_DiagramTypeAsString(EnumDia::Type DiagramType) ;

		//************************
		static bool AccessOK(Tcl_Interp *interp, const ClsMoAccessInfo & AccessInfo, const ClsMoAccessInfo::e_ResultType ResultOK) ;

		//************************
		//************************
		static int write_Syms_to_Tcl_Interp(Tcl_Interp *interp, type_ListOf_Poi_to_ConstSym & ModelResultList) ;

		static int write_ReferencedDiagrams_to_Tcl_Interp(Tcl_Interp *interp, type_ListOf_Poi_to_ConstDiagram & ModelResultList,
			const Diagram * const pDiagram) ;

		static int write_Channels_to_Tcl_Interp(Tcl_Interp *interp, type_ListOf_Poi_to_ConstChannel & ModelResultList) ;

		static int write_StateMachineSyms_to_Tcl_Interp(Tcl_Interp *interp, type_ListOfPoi_to_ConstStateMachineSymbol & ModelResultList) ;

		static int write_FlowLines_to_Tcl_Interp(Tcl_Interp *interp, type_ListOfPoi_to_ConstFlowLine & ModelResultList) ;

		//************************
		//************************
		static int MAR_ARR_exists(Tcl_Interp *interp, bool & exists) ;

		static int clear_MAR_ARR(Tcl_Interp *interp) ;

		//*******************************************************************************************************
		//******* Interface
		//*******************************************************************************************************
		public:

		static int create_EmptySysSpec (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int load_Model_from_File (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int store_Model_in_File (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int save_SDL_PR (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int get_System_Model_ID (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int modify_SysDiaName (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int get_ProcedureDiagramFormalParameters (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int modify_ProcedureDiaFormalParameters (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		// for pages
		static int modify_PageWidth (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int modify_PageHeight (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int modify_PageFramePaS (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int add_NewPage_after (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int get_PageSize (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int get_PageFrameInfo (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int get_FirstPageID (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int get_LastPageID (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int get_NextPageID (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int get_PreviousPageID (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int delete_Page (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		// for RecSym
		static int modify_RecSym_Content (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int set_RecSym_PaS (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int insert_RecSym(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int get_Symbols (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int delete_RecSym (ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int get_AllReferencedDiagrams(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		// END: for RecSym

		// for channel
		static int insert_BidirecChannel(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int get_Channels(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int modify_ChanComp(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int modify_Chan_Coords(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int modify_Chan_Type(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int delete_Chan(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;
		// END: for channel

		static int get_ReferencingDiagramID (ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) ;

		// for state machine symbols
		static int insert_StateMachineSymbol(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int modify_StateMachineSymbol_Content(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int modify_StateMachineSymbol_PaS(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int delete_StateMachineSymbol(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int get_StateMachineSyms(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;
		// END: for state machine symbols

		// for flow lines
		static int insert_FlowLine(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int insert_DecisionFlowLine(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int modify_FlowLineCoords(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int modify_DecisionFlowLinePaS(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int modify_DecisionFlowLineContent(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int delete_FlowLine(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;

		static int get_FlowLines(ClientData clientData, Tcl_Interp *interp, int argc, CONST char *argv[]) ;
		// END: for flow lines

		} ; // class TclModelAccess

	} // END: namespace TclExt

} // END: namespace ol_SDL
#endif


