
#ifndef olSDL_XML_READER_H
#define olSDL_XML_READER_H

#include <string>
#include <fstream>

#include <xercesc/util/XercesDefs.hpp>
#include <xercesc/dom/DOM.hpp>
#include <xercesc/sax/ErrorHandler.hpp>
#include <xercesc/sax/SAXParseException.hpp>

#include "XML_Basic.h"
#include "SysSpec.h"
#include "RenumberingHelper.h"

namespace olSDL {

	class DOMTreeErrorReporter : public ErrorHandler
	{

	//-------------------------------------------------------
	// member variable
	//-------------------------------------------------------
	private:

	bool fSawErrors ;
	string m_strErrorMessage ;

	//-------------------------------------------------------
	//methods
	//-------------------------------------------------------

	public:
	//============
	//Constructor, Copy-Constructor, Assign-Op, ...
    	DOMTreeErrorReporter() ;

	// Des
    	~DOMTreeErrorReporter() ;

	//============
	void warning(const SAXParseException& toCatch) ;
    	void error(const SAXParseException& toCatch) ;
    	void fatalError(const SAXParseException& toCatch) ;
    	void resetErrors() ;

	//============
	// getter
	bool getSawErrors() const ;

	string get_ErrorMessage() const ;

	// Converter
	string DOMStringToStdString(const DOMString& s) ;

	} ; // class DOMTreeErrorReporter

	// ===================================================================
	// ===================================================================

	class ClsXML_Reader_ErrorReporter {

	//-------------------------------------------------------
	// member variable
	//-------------------------------------------------------
	public:
	enum e_ErrorType {not_specified, integer_expected, integer_list_expected, DOM_Parser} ;

	private:
	static const string NOT_SPECIFIED ;

	private:
		e_ErrorType m_ErrorType ;
		string m_strParserErrorMessage ;

	public:
	//============
	//Constructor, Copy-Constructor, Assign-Op, ...
	ClsXML_Reader_ErrorReporter() ;
	ClsXML_Reader_ErrorReporter(e_ErrorType ErrorType) ;
	ClsXML_Reader_ErrorReporter(e_ErrorType ErrorType, const string strParserErrorMessage) ;


	//============
	// getter
	e_ErrorType get_ErrorType() const ;
	string get_ErrorMessage() const ;

	} ;

	// ===================================================================
	// ===================================================================

	class ClsXML_Reader : public ClsXML_Basic, private ClsRenumberingHelper {

	//-------------------------------------------------------
	// member variable
	//-------------------------------------------------------
	private:
	static const string NOT_SPECIFIED ;

	private:
	string m_FileName ;
	SysSpec * m_pSysSpec ;

	//-------------------------------------------------------
	//methods
	//-------------------------------------------------------
	public:
	//============
	//Constructor, Copy-Constructor, Assign-Op, ...
	ClsXML_Reader() ;
	// Copy

	// Assign

	// Des
	~ClsXML_Reader() ;

	//============
	void read_SysSpec(const string & FileName, SysSpec * pSysSpec) ;

	//============
	// getter

	//============
	// setter

	private:

	//============
	// diagrams

	void read_SysDia(DOM_Document doc, DOM_Node & Node) throw (ClsXML_Reader_ErrorReporter) ;
	void read_BlkDia(DOM_Document doc, DOM_Node & Node) ;
	void read_ProDia(DOM_Document doc, DOM_Node & Node) ;
	void read_ProcedureDia(DOM_Document doc, DOM_Node & Node) ;

	void read_ReferencedDiagram(DOM_Document doc, DOM_Node & Node, const Symbol * pSymbol) throw (ClsXML_Reader_ErrorReporter) ;

	//============
	// formal parameters

	void read_ProcedureFormalParameters(DOM_Document doc, DOM_Node & Node) throw (ClsXML_Reader_ErrorReporter) ;

	// formal parameters
	//============

	//============
	//  pages

	void read_Pages(DOM_Document doc, DOM_Node & Node) throw (ClsXML_Reader_ErrorReporter) ;

	void get_Page_from_ElementNode(const DOM_Node & Node, int & PageID,
						int & PageWidth, int & PageHeight, int & x_left_up_Frame, int & y_left_up_Frame,
						int & FrameWidth, int & FrameHeight)
		throw (ClsXML_Reader_ErrorReporter) ;

	void get_PageFrame_from_ElementNode(const DOM_Node & Node, int & x_left_up_Frame,
								  int & y_left_up_Frame, int & FrameWidth, int & FrameHeight)
		throw (ClsXML_Reader_ErrorReporter) ;

	//============
	//  RecSyms

	void read_RecSymbol(DOM_Document doc, DOM_Node & Node, const string & str_SymbolType) throw (ClsXML_Reader_ErrorReporter) ;

	void get_RecSym_from_ElementNode (const DOM_Node & Node, int & Symbol_ID, int & ParentPageID, int & ReferencedDiagramID,
					int & x_left_up, int & y_left_up, int & Width, int & Height, string & SymbolContent) throw (ClsXML_Reader_ErrorReporter)  ;

	//============
	//  StateMaSym

	void read_StateMaSym(DOM_Document doc, DOM_Node & Node, const string & str_StateMaSymType) throw (ClsXML_Reader_ErrorReporter) ;

	void get_StateMaSym_from_ElementNode (const DOM_Node & Node, int & StateMaSym_ID, int & ParentPageID,
			int & x_left_up, int & y_left_up, int & Width, int & Height, string & SymbolContent) throw (ClsXML_Reader_ErrorReporter) ;

	//============
	//  FlowLines

	void read_FlowLines(DOM_Document doc, DOM_Node & Node) throw (ClsXML_Reader_ErrorReporter) ;

	void get_NormalFlowLine_from_ElementNode (const DOM_Node & Node, int & FlowLineID, int & ParentPageID,
			list<int> & FlowLineCoords, int & FlowLineSourceModel_ID, int & FlowLineDestModel_ID) throw (ClsXML_Reader_ErrorReporter) ;

	void get_DecisionFlowLine_from_ElementNode (const DOM_Node & Node, int & FlowLineID, int & ParentPageID,
			list<int> & FlowLineCoords, int & FlowLineSourceModel_ID, int & FlowLineDestModel_ID,
			string & DecisionFlowLineContent, ClsRecPaS & DecisionFlowLinePaS) throw (ClsXML_Reader_ErrorReporter) ;

	//============
	//  PaS

	void get_PaS_from_ElementNode(const DOM_Node & Node, int & x_left_up, int & y_left_up, int & Width, int & Height)
		throw (ClsXML_Reader_ErrorReporter) ;

	//============
	// Channel

	void read_Channel(DOM_Document doc, DOM_Node & Node) throw (ClsXML_Reader_ErrorReporter) ;

	void get_Channel_from_ElementNode (const DOM_Node & Node, int & Channel_ID, int & ParentPageID,
		ClsChannel::e_DelayType & DelayType, ClsChannel::e_DirectionType & DirectionType,
		list<int> & ChanCoords, bool & bSourceIsEnv, int & ChanSourceModel_ID, bool & bDestIsEnv, int & ChanDestModel_ID) ;

	ClsChanName get_ChanName(const DOM_Node & Node) ;
	ClsChanSourceSigList get_ChanSourceSigList(const DOM_Node & Node) ;
	ClsChanSourceCon get_ChanSourceCon(const DOM_Node & Node) ;
	ClsChanDestSigList get_ChanDestSigList(const DOM_Node & Node) ;
	ClsChanDestCon get_ChanDestCon(const DOM_Node & Node) ;

	void get_ChanSourceID(const DOM_Node & Node, bool & bSourceIsEnv, int & ChanSourceModel_ID) ;
	void get_ChanDestID(const DOM_Node & Node, bool & bDestIsEnv, int & ChanDestModel_ID) ;

	//============
	// Helper

	DOM_Node get_DiagramElementNodeByID(DOM_Document doc, const int & ElementID) throw (ClsXML_Reader_ErrorReporter) ;

	int get_IntValue_from_Attribute(const DOM_Node & ElementNode) throw (ClsXML_Reader_ErrorReporter) ;

	int get_IntContent_from_Element(const DOM_Node & ElementNode) throw (ClsXML_Reader_ErrorReporter) ;

	string get_StringContent_from_Element(const DOM_Node & ElementNode,
		const bool b_deleteBeginNewline=true, const bool b_deleteEndNewline=true) throw (ClsXML_Reader_ErrorReporter) ;

	// Converter
	string DOMStringToStdString(const DOMString& s) ;


	} ; // class


} // END: namespace ol_SDL


#endif
