/*
 * VrmlWindow.java
 */

import dnx.lr.*;
import dnx.lr.node.*;
import dnx.lr.app.*;
import dnx.lr.field.*;
import dnx.awtutil.*;
import dnx.geom.*;
import java.awt.Toolkit;
import java.awt.Image;
import java.net.MalformedURLException;
import java.net.URL;
import java.awt.*;
import java.applet.*;
import java.io.*;
import java.lang.*;
import java.util.Vector;
import jcom;
import AppearanceThread;
import JumpThread;
import RotateThread;
import ScaleThread;
import TouchThread;
import TranslateThread;

public class VrmlWindow extends Applet implements Runnable {

   static ApplicationDocument document;
   static View3D view;
   static BrowserEngineAdapter adapter;
   static Scene scene = null;
   static ReadAction loadScene;
   static NodeInstance resetViewpoint;
   String wrl_file;

  public VrmlWindow( String wrl_file) {
	this.wrl_file = wrl_file;
	try {
	      this.loadScene = new ReadAction (new URL ( wrl_file ));
	}
	catch (java.io.IOException e) {
	   System.err.println ("Unable to load: " + e);
	   return;
	}
        this.init();
  }

   public void init() {

	try {
	   scene     = new Scene ( loadScene );
	}
	catch (VRMLSyntaxException e) {
	   System.err.println ("Unable to load: " + e);
	   return;
	}
	catch (java.io.IOException e) {
	   System.err.println ("Unable to load: " + e);
	   return;
	}

   	// Create a document and a view.
   	document = new ApplicationDocument(scene);
   	view = new View3D(document);

   	adapter = new BrowserEngineAdapter( view );
   	document.addFrameUpdateListener( adapter );
   	document.addSceneObserverEventListener( adapter );
   	view.addView3DListener( adapter );
        resetViewpoint = scene.getBoundNodeInstance("Viewpoint");

   	document.start();

   }

  public void start () { }

  public void stop () { }

  public static void reload() {
  }

  public static void resetViewpoint () {

        adapter.bindViewpoint (resetViewpoint, resetViewpoint, true );

  }

  public static void setExamineViewer () {
      adapter.setViewer(adapter.EXAMINE_VIEWER);
  }

  public static void setWalkViewer () {
      adapter.setViewer(adapter.WALK_VIEWER);
  }

  public static void setRenderWireFrame () {
	view.setRenderQuality( view.QUALITY_WIREFRAME );
  }

  public static void setRenderFlat () {
	view.setRenderQuality( view.QUALITY_FLAT_SHADED );
  }

  public static void setRenderSmooth () {
	view.setRenderQuality( view.QUALITY_SMOOTH_SHADED );
  }

  // Translationen
  // #############

  public static void translate( String nodeName, float x, float y, float z, int add,
				int maxSteps, String id ) {

    Node rootNode    = scene.getRootNode();
    Node foundNode   = null;
    TransformNode tn = null;
    float dx, dy, dz;   

    NameSearchTraversal t = new NameSearchTraversal ( nodeName );
    rootNode.traverse (t);
    if (t.foundMatch ()) {
      foundNode = t.getFirstNode ();

      // Wenn der gefundene Knoten ein PROTOTYPE ist, so wird er auf
      // die Wurzel gesetzt.
      if (foundNode instanceof Node) {
	 foundNode = foundNode.getPrimitiveNode();
      }

      if (foundNode instanceof TransformNode) {
		tn = (TransformNode) foundNode;
	  } else {
		System.err.println("Not a Transform node!");
		jcom.send( "animationReady "+id );
		return;
      }
    } else {
       System.err.println("Node "+nodeName+" not found !");
       jcom.send( "animationReady "+id );
       return;
    }

    // Bestimmen der Translationsdifferenzen
    float[] translation = new float[3];
    tn.translation.getValue( translation );
    
    if ( add == 1 ) {
    	dx = -x / maxSteps;
    	dy = -y / maxSteps;
    	dz = -z / maxSteps;
    } else {
    	dx = (translation[0] - x) / maxSteps;
    	dy = (translation[1] - y) / maxSteps;
    	dz = (translation[2] - z) / maxSteps;
    }
    
    // start the thread 
    TranslateThread t_translation = new TranslateThread(tn, dx, dy, dz, maxSteps, id); 
  }


  // Spruenge
  // ########

  public static void jump( String nodeName, float x, float y, float z, float v0, 
				float g, float alpha, int maxSteps, String id ) {

    Node rootNode    = scene.getRootNode();
    Node foundNode   = null;
    TransformNode tn = null;

    NameSearchTraversal t = new NameSearchTraversal ( nodeName );
    rootNode.traverse (t);
    if (t.foundMatch ()) {
      foundNode = t.getFirstNode ();

      // Wenn der gefundene Knoten ein PROTOTYPE ist, so wird er auf
      // die Wurzel gesetzt.
      if (foundNode instanceof Node) {
	 foundNode = foundNode.getPrimitiveNode();
      }

      if (foundNode instanceof TransformNode) {
		tn = (TransformNode) foundNode;
	  } else {
		System.err.println("Not a Transform node!");
		jcom.send( "animationReady "+id );
		return;
      }
    } else {
       System.err.println("Node "+nodeName+" not found !");
       jcom.send( "animationReady "+id );
       return;
    }

    // start the thread 
    JumpThread t_jump = new JumpThread(tn, x, y, z, v0, g, alpha,  maxSteps, id); 
  }

  // Erscheinungen
  // #############

  public static void appearance ( String nodeName, float d1, float d2, float d3, float e1, float e2, float e3,
				float spec, float shin, float amb, float trans, int maxSteps, String id ) {

    Node rootNode    = scene.getRootNode();
    Node foundNode   = null;
    MaterialNode mn  = null;

    NameSearchTraversal t = new NameSearchTraversal ( nodeName );
    rootNode.traverse (t);
    if (t.foundMatch ()) {
      foundNode = t.getFirstNode ();

      // Wenn der gefundene Knoten ein PROTOTYPE ist, so wird er auf
      // die Wurzel gesetzt.
      if (foundNode instanceof Node) {
         foundNode = foundNode.getPrimitiveNode();
      }

	if (foundNode instanceof MaterialNode) {
	   mn = (MaterialNode) foundNode;
	} else {
	   System.err.println("Not a MaterialNode!");
	   jcom.send( "animationReady "+id );
	   return;
	}
    } else {
	System.err.println("Node "+nodeName+" not found !");
	jcom.send( "animationReady "+id );
	return;
    }

	// start the thread
	AppearanceThread t_appearance = new AppearanceThread(mn, d1, d2, d3, e1, e2, e3, spec, shin,
								 amb, trans, maxSteps, id); 
  }

   // Rotationen
   // ##########

  public static void rotate( String nodeName, float x, float y, float z, float angle, 
				int useWorld, int useCenter, float cx, float cy, 
				float cz, int maxSteps, String id ) {

    Node rootNode    = scene.getRootNode();
    Node foundNode   = null;
    TransformNode tn = null;

    NameSearchTraversal t = new NameSearchTraversal ( nodeName );
    rootNode.traverse (t);
    if (t.foundMatch ()) {
      foundNode = t.getFirstNode ();

      // Wenn der gefundene Knoten ein PROTOTYPE ist, so wird er auf
      // die Wurzel gesetzt.
      if (foundNode instanceof Node) {
         foundNode = foundNode.getPrimitiveNode();
      }

      if (foundNode instanceof TransformNode) {
		tn = (TransformNode) foundNode;
      } else {
		System.err.println("Not a Transform node!");
		jcom.send( "animationReady "+id );
      }
    } else {
	System.err.println("Node "+nodeName+" not found !");
	jcom.send( "animationReady "+id );
	return;
    }

    RotateThread t_rotate = new RotateThread(tn, x, y, z, angle, useWorld, 
					useCenter, cx,cy,cz, maxSteps, id); 
  }

  // Skalierungen
  // ############

  public static void scale( String nodeName, float x, float y, float z, int useCenter,
				float cx, float cy, float cz, int maxSteps, String id ) {

    Node rootNode    = scene.getRootNode();
    Node foundNode   = null;
    TransformNode tn = null;
    float dx, dy, dz;   

    NameSearchTraversal t = new NameSearchTraversal ( nodeName );
    rootNode.traverse (t);
    if (t.foundMatch ()) {
      foundNode = t.getFirstNode ();

      // Wenn der gefundene Knoten ein PROTOTYPE ist, so wird er auf
      // die Wurzel gesetzt.
      if (foundNode instanceof Node) {
         foundNode = foundNode.getPrimitiveNode();
      }

      if (foundNode instanceof TransformNode) {
	 tn = (TransformNode) foundNode;
      }
      else {
	  System.err.println("Not a Transform node!");
	  jcom.send( "animationReady "+id );
      }
    } else {
	System.err.println("Node "+nodeName+" not found !");
	jcom.send( "animationReady "+id );
	return;
    }
    float[] scale = new float[3];
    tn.scale.getValue( scale );

    dx = (scale[0] - x) / maxSteps;
    dy = (scale[1] - y) / maxSteps;
    dz = (scale[2] - z) / maxSteps;
    
    // start the thread 
    ScaleThread t_scale = new ScaleThread(tn, dx, dy, dz, useCenter, cx, cy, cz, maxSteps, id); 
  }

  // Aktivieren eines TouchSensorNode
  // ################################

  public static void setTouch( String tsnName, String trans, String id) {
    Node    rootNode = scene.getRootNode();
    Node    foundNode = null;
    TouchSensorNode tsn = null;
    MaterialNode mn  = null;

    NameSearchTraversal t = new NameSearchTraversal ( tsnName );
    rootNode.traverse (t);
    if (t.foundMatch ()) {
      foundNode = t.getFirstNode ();
      if (foundNode instanceof TouchSensorNode) {
        tsn = (TouchSensorNode) foundNode;
        //System.out.println( tsnName+" found...");

	// Start TouchThread	
	TouchThread touch = new TouchThread( tsn, trans, id );

      } else {
        System.err.println("Not a TouchSensorNode!");
	jcom.send( "animationReady "+id );
	return;
      }
     } else {
        System.err.println("Node "+tsnName+" not found !");
        jcom.send( "animationReady "+id );
        return;
    }
  }

  public static void getAllNodeNames() {
    Node    rootNode = scene.getRootNode();
    Node    foundNode = null;
    int	    i;
    String  transformList = "";
    String  touchList     = "+";
    String  materialList  = "+";
    String  resultList    = "";

    NodeTypeTraversal t = new NodeTypeTraversal ( );
    rootNode.traverse (t);
    if (t.foundMatch ()) {
        for (i = 0; i < t.countMatches(); i++) {
            foundNode = t.getNode(i);
	    if (foundNode instanceof TransformNode) {
		transformList = transformList + "-" + foundNode.getName();
            } else if (foundNode instanceof TouchSensorNode) {
		touchList = touchList + "-" + foundNode.getName();
            } else if (foundNode instanceof MaterialNode) {
		materialList = materialList + "-" + foundNode.getName();
	    } 
	}
        resultList = transformList+touchList+materialList;
        jcom.send( "createNodeMenu "+resultList );
    }
  }

  // Aenderung des Objektmittelpunktes
  // #################################

  public static void changeCenter( TransformNode tn, float cx, float cy, float cz) {

	float newX, newY, newZ;
	Quaternion q_akt;

	// Quaternion bzgl. Objektkoordinaten erzeugen
	float[] akt_rot = new float[4];
	tn.rotation.getValue( akt_rot );
	Vector3 vec_tmp = new Vector3 (akt_rot[0], akt_rot[1], akt_rot[2] );
	q_akt = new Quaternion( vec_tmp, akt_rot[3] );

	// Centerpoint
	Point3 centerPoint = new Point3 ();
	tn.center.getValue ( centerPoint );

	// Neuer Centerpoint
	Point3 newCenterPoint = new Point3 (cx, cy, cz);

	// Translationwerte
	Point3 transPoint = new Point3 ();
	tn.translation.getValue( transPoint );

	// Skalierungswerte
	Point3 scalePoint = new Point3 ();
	tn.scale.getValue( scalePoint );

	Vector3 scaleVec = new Vector3( 1 - scalePoint.x, 1 - scalePoint.y, 1 - scalePoint.z );
	Vector3 scaleDirectionVec = new Vector3( centerPoint, newCenterPoint );

	scalePoint.setValue( scaleVec.x*scaleDirectionVec.x,
			     scaleVec.y*scaleDirectionVec.y,
			     scaleVec.z*scaleDirectionVec.z);

	Matrix4 Gs = new Matrix4( scalePoint );

	// Centerpoint in Weltkoordinaten
	Point3 globalCenterPoint = new Point3( transPoint.x + centerPoint.x, 
				    	       transPoint.y + centerPoint.y,
					       transPoint.z + centerPoint.z );

	// Neuer Centerpoint in Weltkoordinaten
	Point3 newGlobalCenterPoint = new Point3( transPoint.x + cx, 
						  transPoint.y + cy, 
						  transPoint.z + cz );

	Vector4 centerVector     = new Vector4( globalCenterPoint );
	Vector4 newCenterVector  = new Vector4( newGlobalCenterPoint );

	// Differenz in Weltkoordinaten
	Vector4 globalDiffVector = new Vector4 ( newCenterVector );
	globalDiffVector.subtract( centerVector );

	 // Einzelwinkel aus Quaternion erzeugen 
	 // ------------------------------------
	 // roll = X-Winkel, pitch = y-Winkel, yaw = z-Winkel

	 Matrix4 Quat = new Matrix4 ( q_akt );

	 Vector3 xRow = new Vector3();
	 Vector3 yRow = new Vector3();
	 Vector3 zRow = new Vector3();

	 Quat.getRows( xRow, yRow, zRow );

	 double M11 = (double) xRow.x;  double M21 = (double) xRow.y;
	 double M31 = (double) xRow.z;
	 double M12 = (double) yRow.x;  double M22 = (double) yRow.y;
	 double M13 = (double) zRow.x;  double M23 = (double) zRow.y;

	 float yaw   = (float) Math.atan2( M21, M11 );
	 float pitch = (float) Math.atan2( -M31, M11*Math.cos(yaw) + M21*Math.sin(yaw));
	 float roll  = (float) Math.atan2( M13*Math.sin(yaw) - M23*Math.cos(yaw),
					   M22*Math.cos(yaw) - M12*Math.sin(yaw));

	 // Generatormatrizen fuer die Rotation der einzelnen Winkel erzeugen
	 Matrix4 Gx = new Matrix4 ( );
	 Gx.xrotate( roll );

	 Matrix4 Gy = new Matrix4 ( );
	 Gy.yrotate( pitch );

	 Matrix4 Gz = new Matrix4 ( );
	 Gz.zrotate( yaw );

	 // Transformation
	 Vector4 diffVector = new Vector4( centerVector );
	 diffVector.subtract( newCenterVector );

	 diffVector.transform( Gs );
	 diffVector.transform( Gx );
	 diffVector.transform( Gy );
	 diffVector.transform( Gz );
	 diffVector.add( globalDiffVector );

	 newX = tn.translation.getX() - diffVector.x;
	 newY = tn.translation.getY() - diffVector.y;
	 newZ = tn.translation.getZ() - diffVector.z;

	 tn.center.setValue ( newCenterPoint );
	 tn.translation.setValue( newX, newY, newZ );
  }

  public void run() { }
}

class NodeTypeTraversal extends NodeSearchTraversal {

   private String target;

   public NodeTypeTraversal ( ) {
      super();
      super.setInterest( FIND_ALL );
   }

   public boolean isMatch (Node node)
   {
      String name = node.getName ();
      return name == null ? false : true;
   }
}

//
// The following classes were removed from lr, so we are including 
// them here for general usage.  These are good examples on implementing
// traversals.
//

/** A traversal that searches for nodes matching a specified criterion.
  (The criterion is specified by overriding the isMatch() method.)
  This always does a COMPLETE_LEAF_TRAVERSAL, hitting all group,
  leaf, and subsidiary nodes.  The default context interest is NORMAL_NODES;
  i.e. don't look into prototypes and inlines. */

abstract class NodeSearchTraversal extends NodeReturningTraversal
{
   public static final int FIND_FIRST = 0;
   public static final int FIND_ALL = 1;

   private int searchInterest = FIND_FIRST;

   public NodeSearchTraversal ()
   {
      this (NORMAL_NODES);
   }

   public NodeSearchTraversal (int contextInterest)
   {
      super (COMPLETE_LEAF_TRAVERSAL, contextInterest);
      setNodeRecording (true);
   }

   public final void setInterest (int interest)
   {
      searchInterest = interest;
   }

   public final int getInterest ()
   {
      return searchInterest;
   }

   /** Don't call this yourself.  It's called automatically. */
   public int visit (Node node)
   {
      if (!isMatch (node))
	 return Traversal.CONTINUE;

      addFoundNode (node);

      // Terminate the search if interest is FIND_FIRST
      if (searchInterest == FIND_FIRST)
	 return Traversal.STOP;
      else
	 return Traversal.CONTINUE;
   }

   /** Don't call this yourself.  It's called automatically.
    *
    * Returns true if the node matches the criteria for this search,
    *  false otherwise.  Subclasses override this to control how what
    *  nodes they want recorded.
    */
   public abstract boolean isMatch (Node node);
}
/** A traversal that searches for nodes with a specified name.
  This always does a COMPLETE_LEAF_TRAVERSAL, hitting all group,
  leaf, and subsidiary nodes.

  @see NodeSearchTraversal
*/

class NameSearchTraversal extends NodeSearchTraversal
{
   private String target;

   /** Create a traversal, specifying the name that is looked for.
    The context interest is NORMAL_NODES. */
   public NameSearchTraversal (String name)
   {
      this (name, NORMAL_NODES);
   }

   /** Create a traversal, specifying the name that is looked for
     and the context interest. */
   public NameSearchTraversal (String name, int contextInterest)
   {
      super (contextInterest);
      setTarget (name);
   }

   /** Set the name that is looked for. */
   public void setTarget (String name)
   {
      target = name;
   }

   /** Return the name that is looked for. */
   public String getTarget ()
   {
      return target;
   }

   /** Don't call this yourself.  It's called automatically. */
   public boolean isMatch (Node node)
   {
      String name = node.getName ();
      return name == null ? false : name.equals (target);
   }
}

// class: ApplicationDocument
class ApplicationDocument extends AWTDocument
{
  private URL documentBase;
  private boolean documentBaseSet;

  public ApplicationDocument ()
  {
    super ();
  }

  public ApplicationDocument (Scene scene)
  {
    super (scene);
  }

  public static URL getCurrentDirectory ()
  {
    URL documentBase = null;
    String dir = System.getProperty ("user.dir");
    dir = dir.replace (File.separatorChar, '/');
    if (dir.charAt (dir.length() - 1) != '/')
     dir += "/";
     dir += "foo";
    try {
      documentBase = new URL ("file:" + dir);
    } catch (MalformedURLException e) {
    }
    return documentBase;
  }

  public synchronized URL getDocumentBase ()
  {
    if (!documentBaseSet)
      {
	documentBase = getCurrentDirectory ();
	documentBaseSet = true;
      }

    return documentBase;
  }

  public synchronized void setMediaFetcher (Scene newScene)
  {
    newScene.setMediaFetcher (new ApplicationMediaFetcher());
  }
}

// class: ApplicationMediaFetcher
class ApplicationMediaFetcher extends MediaFetcher
{
  public ApplicationMediaFetcher()
  {
  }

  public AudioClip getAudioClip(URL url)
  {
    return null;
  }

  public Image getImage(URL url)
  {
    return Toolkit.getDefaultToolkit().getImage(url);
  }

}

