# SimBase .tcl
#
# Globale Prozeduren fuer Simulator und Animator
#####################################################################

proc DEBUG {debType debMessage} {
#
# Ausgabe der Debugnachrichten
#
# debType = com, files, force, compute

   global debug

   if {$debug($debType)} {
	puts $debMessage
   }
}

proc createNodeMenu {nodeNames} {
#
# Erzeugt im Animator drei Menues mit den gefundenen Knotennamen

 global gvar

 if {$gvar(prg) != "animator"} {
	return
 }

 set nodeNames [split $nodeNames "+"]
 createTransformMenu [lindex $nodeNames 0]
 createTouchMenu [lindex $nodeNames 1]
 createMaterialMenu [lindex $nodeNames 2]
}

proc animationReady {id} {
#
# Mitteilung des VRML-Browsers, das eine Animation (id) fertig ist.

global anim_ok started gvar elementLookUp

  set anim_ok "$anim_ok $id"
  DEBUG com "animationReady (id): $id"

  if {$gvar(prg) == "animator"} {
	elementMarkingDelete $id
  }

  set started [minus $id $started]
  checkNextAction
}

proc checkNextAction {} {
#
# Ueberprueft, ob eine weitere Action gesendet werden muss.
# Wenn ja, so wird sie gesendet, sonst wird dem LL-Editor
# das Ende der Animation mitgeteilt

global gvar anim_ok animate no_dep started anotherAnimation animatingListElement 

   set animIds [no_deps]

   if {$gvar(prg) == "animator"} {
	foreach id $animIds {
	   elementMarkingAnimated $id
	}
   }

   set next [createCommand $animIds]

   # Keine Animation mehr
   if {$next == "" && [string trim $started] == "" } {
        catch {unset anim_ok }
        catch {unset animate}
        catch {unset no_dep}
        set started ""

	if {$anotherAnimation} {
		set animatingListElement 0
		set started ""
		return
	} else {
	   if {$gvar(prg) == "simulator"} {
		DEBUG com "Animation finished."
          	sendAnimationOkToLL
		stateSimWait
		msg ""
		updateButtons
		DEBUG com "---------------------"
	   } else {
           	waitMsg
		DEBUG com "Animation finished."
		DEBUG com "---------------------"
	   }
	}
        return
   } elseif {$next != "" && [string trim $started] == ""} {
	DEBUG com "send to VRML: $next"
        RDO "$next" $gvar(vrml.host) $gvar(vrml.port)
   } elseif {$next != "" && [string trim $started] != ""} {
	DEBUG com "send to VRML: $next"
        RDO "$next" $gvar(vrml.host) $gvar(vrml.port)
   } elseif {$next == "" && [string trim $started] != ""} {
   }
}

proc no_deps {} {
global animate anim_ok started

   set element_list ""
   set id [array startsearch animate]

   while { [set element [array nextelement animate $id]] != ""} {
        if {$animate($element) == "" || [isIn $animate($element) $anim_ok]} {
                set element_list "$element_list $element"
        }
   }

   array donesearch animate $id

   set element_list [minus $anim_ok $element_list]
   set element_list [minus $started $element_list]

   return $element_list
}


proc compute {activeAnimation} {
#
# Errechnet alle Transitionen oder Stellen fuer gegebene
# Bedingungen einer Animation

 global conditionType T P TA TD PA PD

 set resultList ""

 if {[isActionCondition $activeAnimation]} {
      set conditionType "Transition"
      computeTransitions $activeAnimation
  } elseif {[isPlaceCondition $activeAnimation]} {
      set conditionType "Place"    
      computePlaces $activeAnimation
  } else {
     if {![catch { set resultList "$resultList $TA($activeAnimation)" }] } {
         catch { set resultList [minus $TD($activeAnimation) $resultList] }
         set T($activeAnimation) $resultList
     } elseif { ![catch { set resultList "$resultList $PA($activeAnimation)" }] } {
         catch { set resultList [minus $PD($activeAnimation) $resultList] }
         set P($activeAnimation) $resultList
     }
 }
 insertAllConditions $activeAnimation
}

proc computeAllConditions {} {
# 
# Errechnet alle Transitionen und Stellen fuer gegebene
# Bedingungen aller Animationen

 global animList T P Cond conditionType

 set animList [getAnimationList]

 # Aufloesen der Animationsbedingungen
 foreach animName $animList {
    compute $animName
 }
}

proc computePlaces {animationName} {
#
# Errechnet alle Stellen, die den geg. Bedingungen entsprechen

 global Cond abs ref_tr_ll PD PA widget P gvar

 set absList [array names abs]
 set actionTransitionList ""
 set preResultList ""
 set postResultList ""
 set resultList ""
 set firstLoop 1

 set conditionList [computeConditionList $animationName]

 # Vorbereichsbedingungen fuer jede Aktion ausrechnen
 foreach condition $conditionList {

    set newResultList ""
    set preResultList ""
    set preTransitionList ""
    set postTransitionList ""
    set placeList ""

    set preConditionList [computePreConditionList  $animationName [getConditionNumber $condition] ]
    set postConditionList [computePostConditionList  $animationName [getConditionNumber $condition] ]

    set placeConditionList [getDefinitionList $condition]
    set preConditionList [getDefinitionList $preConditionList]
    set postConditionList [getDefinitionList $postConditionList]

    # Aufloesen der Bedingungen
    foreach preCondition $preConditionList {
         set preTransitionList "$preTransitionList [lsort [computeTransitionNames $preCondition ll_net] ]"
    }
    foreach postCondition $postConditionList {
          set postTransitionList "$postTransitionList [lsort [computeTransitionNames $postCondition ll_net] ]"
    }
    foreach placeCondition $placeConditionList {
          set placeList "$placeList [computePlaceNames $placeCondition ll_net]"
    }

    foreach i $placeList {
       if {$preConditionList != ""} {

         # Partieller Vergleich
         if {[partial_in $preTranstionList [getPreTransitions $i] ]} {
           lappend preResultList $i
         }
       } else {
           set preResultList $placeList
       }
    }
    foreach i $preResultList {
       if {$postConditionList != ""} {

          if {[partial_in $postTransitionList [getPostTransitions $i] ]} {
             lappend newResultList $i
          }
       } else {
          set newResultList $preResultList
       }
    }

    # Nur die Transitionen uebernehmen, die allen Bedingungen entsprechen 
    if {$firstLoop} {
        set resultList $newResultList
        set firstLoop 0
    } else {
        set resultList [getEqualElements $resultList $newResultList]
    }

 }

 # absolut definierte Stellen
 catch { set resultList "$resultList $PA($animationName)" }
 catch { set resultList [minus $PD($animationName) $resultList] }
 
 # Einsetzten in die ErgebisListbox
 if {$gvar(prg) == "animator"} {
   if {[winfo exists $widget(resultList)]} {
     if {[winfo viewable $widget(resultList)]} {
         $widget(resultList) delete 0 end
         foreach i $resultList {
             $widget(resultList) insert end $i
         }
     }
   }
 }

 set P($animationName) $resultList
 
 return $resultList
}

proc computeTransitions {animationName} {
#
# Errechnet alle Transitionen, die den geg. Bedingungen entsprechen
 
 global Cond abs ref_tr_ll TD TA widget T gvar

 set absList [array names abs]
 set actionTransitionList ""
 set preResultList ""
 set postResultList ""
 set resultList ""
 set firstLoop 1

 set conditionList [computeConditionList $animationName]
 DEBUG compute "----------------------------------------------------"
 DEBUG compute "found conditions:     $conditionList"

 # Vorbereichsbedingungen fuer jede Action ausrechnen
 foreach condition $conditionList {

    set preResultList ""
    set newResultList ""
    set prePlaceList ""
    set postPlaceList ""
    set transitionList ""

    DEBUG compute "  compute condition:    $condition"

    # Liste der abstrakten Bedingungen

    set preConditionList [computePreConditionList  $animationName [getConditionNumber $condition] ]
    set postConditionList [computePostConditionList  $animationName [getConditionNumber $condition] ]
    DEBUG compute "     found preconditions:  $preConditionList"
    DEBUG compute "     found postconditions: $preConditionList"

    # abstrakte Bedingungen durch Quelltextpositionen ersetzen
    set actionConditionList [getDefinitionList $condition]
    set preConditionList [getDefinitionList $preConditionList]
    set postConditionList [getDefinitionList $postConditionList]

    DEBUG compute "     references (condition)     : $actionConditionList"
    DEBUG compute "     references (precondition)  : $preConditionList"
    DEBUG compute "     references (postcondition) : $postConditionList"

    # Aufloesen der Bedingungen
    foreach preCondition $preConditionList {
         set prePlaceList "$prePlaceList [lsort [computePlaceNames $preCondition ll_net] ]"
    }
    foreach postCondition $postConditionList {
          set postPlaceList "$postPlaceList [lsort [computePlaceNames $postCondition ll_net] ]"
    }
    foreach actionCondition $actionConditionList {
          set transitionList "$transitionList [computeTransitionNames $actionCondition ll_net]"
    }

    DEBUG compute "     computed Names (condition)     : $transitionList"
    DEBUG compute "     computed Names (precondition)  : $prePlaceList"
    DEBUG compute "     computed Names (postcondition) : $postPlaceList"

    # Vergleich der Transitionen mit dem tatsaechlichen Vorbereich
    DEBUG compute "     comparing pre-Places"
    foreach i $transitionList {
       if {$preConditionList != ""} {
	 # Vergleich

         if {[partial_in $prePlaceList [getPrePlaces $i] ]} {
           lappend preResultList $i
           DEBUG compute "        comparing ($i): $prePlaceList -> [getPrePlaces $i] *"
         } else {
           DEBUG compute "        comparing ($i): $prePlaceList -> [getPrePlaces $i]"
	 }
       } else {
           set preResultList $transitionList
       }
    }

    DEBUG compute "     transitionlist after precondition: $preResultList"

    DEBUG compute "     comparing post-Places"
    foreach i $preResultList {
       if {$postConditionList != ""} {
	  if {[partial_in $postPlaceList [getPostPlaces $i] ]} {
             lappend newResultList $i
             DEBUG compute "        comparing ($i): $postPlaceList -> [getPostPlaces $i] *"
          } else {
             DEBUG compute "        comparing ($i): $postPlaceList -> [getPostPlaces $i]"
	  }
       } else {
          set newResultList $preResultList
       }
    }
    DEBUG compute "     transitionlist after postcondition: $newResultList"
    # Nur die Transitionen uebernehmen, die allen Bedingungen entsprechen 
    if {$firstLoop} {
        set resultList $newResultList
        set firstLoop 0
	DEBUG compute "     set resultList: $resultList"
    } else {
	DEBUG compute "     comparing: $resultList -> $newResultList"
        set resultList [getEqualElements $resultList $newResultList]
	DEBUG compute "     set resultList: $resultList"
    }
 }

 catch { set resultList "$resultList $TA($animationName)" }
 catch { set resultList [minus $TD($animationName) $resultList] }

 DEBUG compute "final: $resultList"
 
 # Einsetzten in die ErgebisListbox
 if {$gvar(prg) == "animator"} {
   if {[winfo exists $widget(resultList)]} {
     if {[winfo viewable $widget(resultList)]} {
         $widget(resultList) delete 0 end
         foreach i $resultList {
             $widget(resultList) insert end $i
         }
     }
   }
 }
 set T($animationName) $resultList
 return $resultList
}

proc getResultList {touchAnimationName} {

 global T P

 if {![catch {set resultList $T(__tmp_$touchAnimationName)} ]} {
   return [string trim $T(__tmp_$touchAnimationName)]
 }
 if {![catch {set resultList $P(__tmp_$touchAnimationName)} ]} {
   return [string trim $P(__tmp_$touchAnimationName)]
 }
 return ""
}

proc getActionType {actionName} {

 global A
 return [lindex $A($actionName) 0]
}

proc createCommand {no_dep} {
#
# Erzeugt einen Animationsbefehl, der dem VRML-Browser gesendet werden soll.
# 
   global animate H DEP A started T P

   # Menge der Aktionen, die keine Abhaengigkeiten mehr haben
   set counts [llength $no_dep]

   if {$counts < 1} {
        set command ""
   } elseif {$counts == 1} {
	# Genau eine Aktion soll gestartet werden
        set trans_id [lindex $no_dep 0]
        set anim_id [lindex $H($trans_id) 0]

	if {[getActionType $anim_id] == "touch"} {
		set resList [getResultList $anim_id]
		if {$resList == ""} {
			set return ""
		}
		regsub -all " " $resList "," resList
		set command "$A($anim_id) $resList 1 $trans_id"
	} else {
        	set command "$A($anim_id) $trans_id"
	}
        set started "$started $trans_id"
   } else {
	# Mehrere Aktionen sollen parallel gestartet werden
        set command "para ("
        for {set i 0} {$i < [llength $no_dep]} {incr i} {
           set trans_id [lindex $no_dep $i]
           set anim_id [lindex $H($trans_id) 0]

	   if {[getActionType $anim_id] == "touch"} {
	 	set resList [getResultList $anim_id]
		if {$resList != ""} {
			regsub -all " " $resList "," resList
			set command "$command$A($anim_id) $resList 1 $trans_id)("
           		set started "$started $trans_id"
		}
	   } else {
           	set command "$command$A($anim_id) $trans_id)("
           	set started "$started $trans_id"
	   }

        }
        set command [string range $command 0 [expr ([string length $command] - 2)]]
   }
   return $command
}

# Operationen auf Listen
######################################################################################

proc getEqualElements {list1 list2} {
#
# Gibt alle Elemente zurueck, die sowohl in list1 und list2 enthalten sind

 set resultList ""

 if {$list1 == "" || $list2 == ""} {
    return ""
 }

 for {set i 0} {$i < [llength $list1]} {incr i} {
        set compare_element [lindex $list1 $i]
        if { [lsearch -exact $list2 $compare_element] != -1} {
		lappend resultList $compare_element 
        }
 }

 return $resultList
}

proc isIn {list1 list2} {
#
# Diese Prozedur ueberprueft ob alle Elemente von List1
# an einer beliebigen Stelle in List2 vorkommen

 for {set i 0} {$i < [llength $list1]} {incr i} {
        set compare_element [lindex $list1 $i]
        if { [lsearch -exact $list2 $compare_element] == -1} {
                return 0
        }
        # Element ist nicht in List2
 }
 return 1
}

proc minus {list1 list2} {
#
# Zieht alle Elemente der Liste1 von Liste 2 ab und
# gibt das Ergebnis zurueck.

 for {set i 0} {$i < [llength $list1]} {incr i} {
        set compare_element [lindex $list1 $i]
        if { [set index [lsearch -exact $list2 $compare_element]] != -1} {
                set list2 [lreplace $list2 $index $index]
        }
 }
 return $list2
}

proc deleteElementFromList {element list} {
# loescht ein element aus der list und gibt diese zurueck

set returnList ""

foreach i $list {
   if {$i != $element} {
        lappend returnList $i
    }
}

return $returnList
}

######################################################################################

proc max {x y} {
# Gibt das groessere Element zurueck

 if { $x >= $y } {
    return $x
 } else {
    return $y
 }
}

proc unsetAllArrays {} {
#
# Loescht alle Arrays. (Wichtig vor dem Laden)

global H A DEP animator LPT LTP T P TA TD PA PD Cond abstr abs abs_pos 
global Animation animateWhenActive stat

 if {[array exists H]} { unset H }
 if {[array exists A]} { unset A }
 if {[array exists DEP]} { unset DEP }
 if {[array exists animator]} { unset animator }
 if {[array exists LPT]} { unset LPT }
 if {[array exists LTP]} { unset LTP }
 if {[array exists Cond]} { unset Cond }
 if {[array exists abs]} { unset abs }
 if {[array exists abstr]} { unset abstr }
 if {[array exists abs_pos]} { unset abs_pos }
 if {[array exists T]} { unset T }
 if {[array exists TA]} { unset TA }
 if {[array exists TD]} { unset TD }
 if {[array exists P]} { unset P }
 if {[array exists PA]} { unset PA }
 if {[array exists PD]} { unset PD }
 if {[array exists Animation]} { unset Animation }
 if {[array exists animateWhenActive]} { unset animateWhenActive }

 if {[array exists stat] } {
	set stat(fileMoveExists) 0
	set stat(fileWrlExists) 0
	set stat(fileMvRefExists) 0
	set stat(fileLLRefExists) 0
	set stst(fileLLNetExists) 0
	set stat(projectExists) 0
 }
}

# Berechungen der Bedingungen
###################################################################################

proc computeConditionList {actionName} {
# Sucht aus dem Cond-Array alle passenden Bedingungen

 global Cond conditionType

 set condList [array names Cond]
 set actConditionList ""

 if {$conditionType == "Transition"} {
    set indicator "T"
 } elseif {$conditionType == "Place"} {
    set indicator "P"
 } else {
    return ""
 }

 foreach i $condList {
    if {[getConditionType $i] == $indicator && [getConditionName $i] == $actionName} {
       lappend actConditionList $i
    }
 }

 return [lsort $actConditionList]
}

proc computePlaceNames {refList netType} {
# Errechnet aus einer Liste von Refenenzen die passenden Stellen-Namen

 global ref_pl_ll ref_tr_ll

 if {$refList == ""} {
   return
 }

 set refList [split $refList "-"]
 set searchType [lindex $refList 1]
 set refList [lindex $refList 0]

 set refList [split $refList "+"]
 set value [lindex $refList 1]
 set refList [lindex $refList 0]

 set refList [split $refList "*"]

 set name_list ""

 if {[array exists ref_pl_ll]} {
        set id [array startsearch ref_pl_ll]
        while {[set a_index [array nextelement ref_pl_ll $id]] != ""} {
                set ref_list $ref_pl_ll($a_index)
                set ref_value [get_value $ref_list]
                set ref_list [delete_value $ref_list]
                regsub -all {\*} $ref_list { } ref_list
                if { [partial_in $refList $ref_list] }  {
                        if {$value == "?"} {
                          if {$searchType == "p"} {
                             set name_list "$name_list $a_index"
                          } elseif {[llength $refList] == [llength $ref_list]} {
                             set name_list "$name_list $a_index"
                          }
                        } elseif {$value == $ref_value} {
                          if {$searchType == "p"} {
                             set name_list "$name_list $a_index"
                          } elseif {[llength $refList] == [llength $ref_list]} {
                             set name_list "$name_list $a_index"
                          }
                        }
                }
        }
 } 

 regsub { } $name_list {} name_list
 #regsub -all { } $name_list {,} name_list
 return $name_list
}

proc computeTransitionNames {refList netType} {
# Errechnet aus einer Liste von Referenzen (17:0*18:0) die passenden
# Transitionsnamen.

 global ref_tr_ll ref_tr_hl

 set transitionNameList ""

 if {$refList == ""} {
    return ""
 }

 # LL-Netz
 if {$netType == "ll_net"} {
   if {[array exists ref_tr_ll]} {
      set transList [array names ref_tr_ll]
      foreach i $transList {
        regsub -all {\*} $ref_tr_ll($i) " " check_ref
        if {[partial_in $refList $check_ref]} {
           lappend transitionNameList $i
        }
      }
   } else { return -1 }
 } else {
 # HL-Netz

   if {[array exists ref_tr_hl]} {
      set transList [array names ref_tr_hl]
      foreach i $transList {
         regsub -all {\*} $ref_tr_hl($i) " " check_ref
         if {[partial_in $refList $check_ref]} {
            lappend transitionNameList $i
         }
      }
   } else { return -1 }
 }

 return $transitionNameList
}

proc computePreConditionList {animationName actionNumber} {
# Sucht aus dem Cond-Array alle passenden Bedingungen fuer die Vorbedingungen
# heraus.

 global Cond

 set condList [array names Cond]
 set preConditionList ""

 foreach i $condList {
    if {[getConditionType $i] == "PRE"  &&  [getConditionName $i] == $animationName &&  [getConditionNumber $i] == $actionNumber} {

           lappend preConditionList $i
    }
 }

 return [lsort $preConditionList]
}

proc computePostConditionList {animationName actionNumber} {
# Sucht aus dem Cond-Array alle passenden Bedingungen fuer die Vorbedingungen
# heraus.

 global Cond

 set condList [array names Cond]
 set postConditionList ""

 foreach i $condList {
    if {[getConditionType $i] == "POST" && [getConditionName $i] == $animationName && [getConditionNumber $i] == $actionNumber} {
       lappend postConditionList $i
    }
 }

 return [lsort $postConditionList]
}

proc getPreTransitions {placeName} {
#
# Gibt die tatsaechlichen Vorbereichstransitionen von placeName zurueck
#
global LTP

 set names [array names LTP]
 set placeList ""

 foreach i $names {
    if {[isIn $placeName $LTP($i)]} {
       lappend placeList $i
    }
 }

 return $placeList
}

proc getPrePlaces {transName} {
#
# Gibt die tatsaechlichen Vorbereichsstellen von transName zurueck
#
global LPT

 set names [array names LPT]
 set placeList ""

 foreach i $names {
    if {[isIn $transName $LPT($i)]} {
       lappend placeList $i
    }
 }

 return $placeList
}

proc getPostTransitions {placeName} {
#
# Gibt die tatsaechlichen Nachbereichstransitionen von placeName zurueck
#
global LPT

return $LPT($placeName)
}

proc getPostPlaces {transName} {
#
# Gibt die tatsaechlichen Nachbereichsstellen von transName zurueck
#
global LTP

 set placeList ""

 if {$transName == "-1"} {
	return ""
 } else {
	return $LTP($transName)
 }
}

proc getMaxConditionNumbers {animName} {
global Cond

 set names [array names Cond $animName*]

 set preNumber 0
 set postNumber 0
 set condNumber 0

 foreach i $names {
    set type   [getConditionType $i]
    set number [getConditionNumber $i]
    if {$type == "T" || $type == "P"} {
        set condNumber [max $condNumber $number]

    } elseif {$type == "PRE"} {
        set preNumber [max $preNumber $number]

    } elseif {$type == "POST"} {
        set postNumber [max $postNumber $number]
    }
 }

 return "$preNumber $condNumber $postNumber"
}

proc getElementListboxNumber {box element} {
# Gibt die Position des Elementes element in der Listbox box
# zurueck. Dabei ist die Position des ersten Elements 1 !

set listboxSize [$box size]

for {set i 0} {$i < $listboxSize} {incr i} {
   if { $element == [$box get $i] } {
        incr i
        return $i
   }
}

return -1
}

proc getDefinitionList {abstractNameList} {
# Berechnet aus einer Liste auf Referenzen mit abstakten Namen eine
# Liste aus Definitionsnummern. (OpenDoor:0 -> 17:0)

   global Cond

   set defNumberList ""

    foreach i $abstractNameList {
	lappend defNumberList [abs_ref2def_ref $Cond($i)]
    }

    return $defNumberList
}

# Die folgenden Funktionen geben fuer einen gegebenen Bedingungsnamen
# Type, Nummer, Name oder den Index zurueck:

proc getConditionType {condName} {
   regsub -all "#" $condName " " condList
   return [lindex $condList 1]
}

proc getConditionNumber {condName} {
   regsub -all "#" $condName " " condList
   return [lindex $condList 2]
}

proc getConditionName {condName} {
   regsub -all "#" $condName " " condList
   return [lindex $condList 0]
}

proc getConditionIndex {condName} {
   regsub -all "#" $condName " " condList
   return [lindex $condList 3]
}

proc getConditionByValue {value} {
global Cond widget

set condNames [array names Cond]

foreach i $condNames {
    if {$value == $Cond($i)} {
        return $i
    }
}

return -1
}

proc getAnimationList {} {
# Gibt eine Liste aller Animationsnamen zurueck

 global H

 set names [array names H]
 set names [lsort $names]
 set retList ""
 regsub -all {\#} $names { } animList

 for {set i 0} {$i < [llength $animList]} {incr i 3} {
    set newElement [lindex $animList $i]
    if { [isIn $newElement $retList] == 0} {
       lappend retList $newElement
    }
 }

 return $retList
}

proc isPlaceCondition {actionName} {
# Prueft, ob Bedingungen fuer Stellen eingegeben wurden.

 global Cond

 set condList [array names Cond]
 set ret 0

 foreach i $condList {
    if {[getConditionType $i] == "P" && [getConditionName $i] == $actionName} {
       set ret 1
    }
 }

 return $ret
}

proc isActionCondition {actionName} {
# Prueft, ob Bedingungen fuer Transitionen eingegeben wurden.

 global Cond

 set condList [array names Cond]
 set ret 0

 foreach i $condList {
    if {[getConditionType $i] == "T" && [getConditionName $i] == $actionName} {
       set ret 1
    }
 }

 return $ret
}

proc arrayWrite {arrayName} {
#
# Ausgabe des Arrays $arrayName auf der Console
#

 global $arrayName

 if {[array exists $arrayName]} {
    set values [array get $arrayName]
    set size   [llength $values]
 } else {
    puts "$arrayName: no such array..."
    return
 }
 for {set i 0} { $i < $size} {incr i 2} {
	puts "$arrayName ([lindex $values $i]) = [lindex $values [ expr ( $i + 1) ] ]"
 }
}

# Kommunikationsprozeduren
##########################

proc hello_server {port} {
#
# port = Port des Clients, ueber den die Kommunikation laufen soll !

   global gvar widget browser_started

   set gvar(vrml.comstate) "connected"
   msg "Browser is ready"
   set gvar(vrml.port) $port

   if {$gvar(prg) == "animator"} {
	 set browser_started 1
   } else {
         updateButtons
   }
}

proc initPort {} {
#
# Einrichten des Servers

  global gvar server_port

  set gvar(my_sock) [RPC_server 0]
  set server_port [lindex [fconfigure $gvar(my_sock) -sockname] 2]
}

proc newClient {} {
#
# Oeffnen des VRML-Browsers

   global gvar widget animator

   DEBUG com "open VRML..."

   # Ueberpruefen, ob der Browser bereits gestartet ist

   if {$gvar(vrml.comstate) == "connected"} {
	msg "Browser allready started"
   	return
   } else {
   	msg "starting Browser"
   }

   # WRL-Datei suchen
   if {![file exists $animator(connect_to)]} {
	msg "Connected WRL-File not found. Please Reconnect."
	return
   }

   DEBUG files "WRL File found"

   #Erzeugen der Client Socket-Nummer
   set gvar(vrml.sock) [RPC_server 0]
   set gvar(vrml.port) [lindex [fconfigure $gvar(vrml.sock) -sockname] 2]
   set gvar(vrml.host) "localhost"

   DEBUG com "new portnumber: $gvar(vrml.port)"

   set server_port [lindex [fconfigure $gvar(my_sock) -sockname] 2]
   set file "file:$animator(connect_to)"

   DEBUG com "exec java Browse $file $server_port $gvar(vrml.port)"
   # Schliessen des Ports, damit er vom VRML Browser geoeffnet werden kann.
   close $gvar(vrml.sock)
   exec java Browse $file $server_port $gvar(vrml.port) &

   $widget(OpenVRMLButton) configure -state disabled
}

proc closeVRML {} {
global gvar

  if {$gvar(vrml.comstate) == "connected"} {
      DEBUG com "send endSessionServer to $gvar(vrml.port)"
      RDO "endSessionServer 0" localhost $gvar(vrml.port)
  }
}
