/*****************************************************************************/
/* readHL.c                                                                  */
/*****************************************************************************/
/*                                  Stefan Roemer,  06.02.1995 - 06.02.1995  */
/*****************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "readHL.h"
#include "readPEP.h"
#include "readlib.h"

/*****************************************************************************/

#define MBSIZE 4096
#define NAMES_START 2000
#define NAMES_OFFSET 1000

/*****************************************************************************/
/***************************** read_HLnet ************************************/
/*****************************************************************************/
/* Neu eingefuegt (neues Format)       19.12.1994 - 09.01.1995 Stefan Roemer */
/*****************************************************************************/

M_Net		net;
M_Place		*PlArray;
M_Transition	*TrArray,
		*PhTrArray;

Cardinal32	AnzPlNamen, MaxPlNamen,
		AnzTrNamen, MaxTrNamen,
		AnzPhTrNamen, MaxPhTrNamen;

char buffer[10];
int  swapped;
int  blockalloc, blockused;
int  textalloc, textused;
int  placecount, transcount;
bool autonumbering;

t_coords *co;
char	 *name, *corrtran, *rest, *l_symb, *links;
int	 ident;

int rp_insert_block();
int rp_insert_link_symbol();
int rp_insert_place();
int rp_insert_trans();
int rp_insert_arc();
int rp_insert_text();

M_Net read_HLnet(char *PEPfilename)
{
	t_dest null_dest[] = { {  0 ,  0 } };

	t_dest place_dest[] =
		{ { '\'',&name },	/* Bezeichner		*/
		  { '"', &name },	/*     "		*/
		  { '@', &co },		/* Koordinaten		*/
		  { '0', &ident },	/* numerischer Name	*/
		  {  0 ,  0 } };
	  
	t_dest trans_dest[] =
		{ { '\'',&name },	/* Bezeichner		*/
		  { '"', &name },	/*     "		*/
		  { '@', &co },		/* Koordinaten		*/
	          { 'l', &links },	/* Link terms		*/
		  { '0', &ident },	/* numerischer Name	*/
		  { 'P', &corrtran },	/* Liste der Ph.Trans.	*/
		  {  0 ,  0 } };
	  
	t_dest arc_dest[] =
		{ { '@', &co },		/* Quelle/Ziel		*/
		  {  0 ,  0 } };

	t_blockdest netdest[] =
		{ { "BL",  rp_insert_block, &rest, null_dest },
                  { "LS",  rp_insert_link_symbol, &l_symb, null_dest },
		  { "PL",  rp_insert_place, &rest, place_dest },
		  { "TR",  rp_insert_trans, &rest, trans_dest },
		  { "PTR", rp_insert_trans, &rest, trans_dest },
		  { "TP",  rp_insert_arc,   &rest, arc_dest },
		  { "PT",  rp_insert_arc,   &rest, arc_dest },
		  { "PTP", rp_insert_arc,   &rest, arc_dest },
		  { "PPT", rp_insert_arc,   &rest, arc_dest },
		  { "TX",  rp_insert_text,  &rest, null_dest },
		  { NULL, NULL, NULL, NULL } };

	int		count;

	PlArray = (M_Place*) MYmalloc((count = MaxPlNamen = MaxTrNamen
		= MaxPhTrNamen = NAMES_START) * sizeof(M_Place));
	TrArray = (M_Transition*) MYmalloc(count * sizeof(M_Transition));
	PhTrArray = (M_Transition*) MYmalloc(count * sizeof(M_Transition));
	AnzPlNamen = AnzTrNamen = AnzPhTrNamen = 0;
	while (--count)
	{
		PlArray[count] = (M_Place) NULL;
		TrArray[count] = (M_Transition) NULL;
		PhTrArray[count] = (M_Transition) NULL;
	} /* while */

	/* Initialize net */
	net = M_empty_net();

	/* Initialize blocks */
	net->blockinfo = MYmalloc(blockalloc = MBSIZE);
	net->textinfo = MYmalloc(textalloc = MBSIZE);
	*(net->blockinfo) = '\0';
	*(net->textinfo) = '\0';
	swapped = 0;
	blockused = textused = 0;
	placecount = transcount = 0;
	autonumbering = True;

	/* read the net */
	read_PEP_net(PEPfilename, netdest);

	net->type = filetype;

	free(PlArray);
	free(TrArray);
	free(PhTrArray);

	return net;

} /* read_HLnet() */

int rp_insert_block()
{
	if (!net->dfont && cbuf)
	{
		net->dfont = MYstrdup(cbuf);
		cc = cbuf;
		cbused = 0;
	}
	blockused += strlen(rest);
	while (++blockused > blockalloc) 
		net->blockinfo = MYrealloc(net->blockinfo, blockalloc+=MBSIZE);
	sprintf(net->blockinfo, "%s%s\n", net->blockinfo, rest);
	free(rest);
	return 0;
}

int rp_insert_link_symbol()
{
	int count; /* counter */
	int MaxLinkSymbs = net -> MaxNumLinks;
	if (!l_symb || strlen(l_symb)<1) return 0;
	/* eventually (re)alloc memory to net->Link */
	if ( net -> NumLinks >= MaxLinkSymbs )
	 {
	    if (net->Links)
	      net -> Links = (linkdef *) MYrealloc( net -> Links,
	                   ( MaxLinkSymbs += 10 ) * sizeof( linkdef *) );
	    else
	      net -> Links = (linkdef *) MYmalloc
			   ((MaxLinkSymbs += 10) * sizeof (linkdef *) );
	    for ( count = MaxLinkSymbs - 10 ; count < MaxLinkSymbs ; count++ )
	     {
		       net -> Links [ count ] = (linkdef) NULL;
	     } /* for */
	    net -> MaxNumLinks = MaxLinkSymbs;
	  } /* if (net -> NumLinks >= MaxLinkSymbs) */

	net -> Links [net->NumLinks] =
		(char *) MYmalloc ((strlen(l_symb)+1) * sizeof(char) );
	strcpy (net -> Links [net->NumLinks], l_symb);

	net -> NumLinks++;

	return 0;
} /* rp_insert_link_symbol */

int rp_insert_place()
{
	static M_Place pl_last = NULL;

	if (!net->dfont && cbuf)
	{
		net->dfont = MYstrdup(cbuf);
		cc = cbuf;
		cbused = 0;
	}
	placecount++;
	if (ident && ident != placecount) autonumbering = False;
	if (!ident && autonumbering) ident = placecount;
	if (!ident) Error((errbuf,"missing place identifier"));
		
	if (ident > AnzPlNamen)
		AnzPlNamen = ident;
	else if (PlArray[ident])
		Error((errbuf, "place identifier %d used twice", ident));

	while (AnzPlNamen >= MaxPlNamen)
	{
		int count;
		PlArray = (M_Place*) MYrealloc((char*) PlArray,
				(MaxPlNamen += NAMES_OFFSET) * sizeof(M_Place));
		for (count = MaxPlNamen - NAMES_OFFSET; count < MaxPlNamen;)
			PlArray[count++] = (M_Place) NULL;

	}

	PlArray[ident] = M_fast_insert_place(net,ident,&pl_last);
	if (!name) sprintf(buffer, "P%u", ident);
	pl_last->charname = name? name : MYstrdup(buffer);
	pl_last->co = co;
	pl_last->other = rest;
	return 0;
}

#define swap(h,x1,x2) h=x1; x1=x2; x2=h;

int rp_insert_trans()
{
	int count;
	static M_Transition tr_last = NULL;

	if (*blocktype == 'P')
	{
		M_Transition *trtmp;
		
		*blocktype = '\0';
		tr_last = NULL;
		transcount = 0;
		swapped++;
		swap(trtmp,PhTrArray,TrArray);
		swap(count,AnzPhTrNamen,AnzTrNamen);
		swap(count,MaxPhTrNamen,MaxTrNamen);
	}
	if (*blocktype == 'T')
	{
		*blocktype = 't';
		tr_last = NULL;
	}

	if (!transcount++) autonumbering = True;
	if (ident && ident != transcount) autonumbering = False;
	if (!ident && autonumbering) ident = transcount;
	if (!ident) Error((errbuf, "missing transition identifier"));

	if (ident > AnzPhTrNamen)
		AnzPhTrNamen = ident;
	else if (PhTrArray[ident])
		Error((errbuf, "%stransition identifier %d used twice", 
			    *blocktype == 't'? "" : "phantom ", ident));

	while (AnzPhTrNamen >= MaxPhTrNamen)
	{
		PhTrArray = (M_Transition *) MYrealloc((char*) PhTrArray,
			  (MaxPhTrNamen += NAMES_OFFSET) * sizeof(M_Transition));
		for (count = MaxPhTrNamen - NAMES_OFFSET; count < MaxPhTrNamen;)
			PhTrArray[count++] = (M_Transition) NULL;
	}

	PhTrArray[ident] = (*blocktype == 't'? M_fast_insert_transition
					     : M_insert_phantom_transition)
				(net, ident, &tr_last);

	if (!name) sprintf(buffer,"%s%u", *blocktype == 't'? "T" : "PT", ident);
	tr_last->charname = name? name : MYstrdup(buffer);
	tr_last->links = links;
	tr_last->corrtran = corrtran;
	tr_last->co = co;
	tr_last->other = rest;
	return 0;
}

int rp_insert_arc()
{
	static M_Edge arc_last = NULL;
	static int tp = 0, phantom = 0;
	int pl, tr, anz;
	M_Transition *arr;

	if (!swapped++)		/* oh no! ... pop! */
	{
		swap(arr,PhTrArray,TrArray);
		swap(tr,AnzPhTrNamen,AnzTrNamen);
		swap(tr,MaxPhTrNamen,MaxTrNamen);
	}

	if (isalpha(*blocktype))
	{
		arc_last = NULL;
		tp = strcmp(blocktype,"PT") && strcmp(blocktype,"PPT");
		phantom = strcmp(blocktype,"TP") && strcmp(blocktype,"PT");
		*blocktype = '\0';
	}
		
	pl = tp? co->y : co->x;
	tr = tp? co->x : co->y;
	arr = phantom? PhTrArray : TrArray;
	anz = phantom? AnzPhTrNamen : AnzTrNamen;

	if (!tr || (tr > anz) || !arr[tr])
		Error((errbuf, "arc: incorrect %stransition identifier",
			phantom? "phantom " : ""));
	if (!pl || (pl > AnzPlNamen) || !PlArray[pl] )
		Error((errbuf, "arc: incorrect place identifier"));

	(tp? (phantom? M_insert_PTP_edge : M_fast_insert_TP_edge)
	   : (phantom? M_insert_PPT_edge : M_fast_insert_PT_edge))
		(net, PlArray[pl]->numname, arr[tr]->numname,
		 &PlArray[pl], &arr[tr], &arc_last);

	arc_last->other = rest;
	return 0;
}

int rp_insert_text()
{
	textused += strlen(rest);
	textused += cbused;
	while (++textused > textalloc) 
		net->textinfo = MYrealloc(net->textinfo, textalloc+=MBSIZE);
	if (cbused)
	{
		sprintf(net->textinfo, "%s%s%s", net->textinfo, rest, cbuf);
		cc = cbuf; cbused = 0;
	}
	else
		sprintf(net->textinfo, "%s%s\n", net->textinfo, rest);
	free(rest);
	return 0;
}

/*****************************************************************************/

void ReadRefObject(FILE *infile, char ch, char **name, char **ref)
{
	static char buffer[1024];
	char *c = buffer;
	*c = ch;

	*name = buffer;
	
	while ((*++c = ReadCharComment(infile)) != '"');
	*c++ = '\0';

	*ref = c;
	while ((*c++ = ReadCharComment(infile)) != '"');
	*--c = '\0';

	if (ReadCharComment(infile) != '\n')
		PEPerror("garbage characters after reference");
		
}

void ReadBPNRefObject(FILE *infile,char ch,int *index,char **name,char **ref)
{
	static char buffer[1024];
	char *c = buffer;
	*c = ch;

	*index = 0;
	*name = buffer;
	*ref = NULL;
	
	if (isdigit(*c))
	{
		ungetc(*c,infile);
		*index=ReadNumber(infile);
		*name = NULL;
	}
	else
	{
		while (isalnum(*++c = getc(infile)));
		*c++ = '\0';
	}

	if (ReadCharComment(infile) != 'T') PEPerror("'T' expected");
	ReadCharComment(infile);

	*ref = c;
	while ((*c++ = getc(infile)) != '"' && !feof(infile));
	*--c = '\0';

	if (ReadCharComment(infile) != '\n')
		PEPerror("garbage characters after reference");
		
}

/*****************************************************************************/
/***************************** read_HLLLref **********************************/
/*****************************************************************************/

void read_HLLLref(M_Net *hnet, M_Net *lnet, char *PEPfilename)
{
	register char	ch;
	register FILE	*infile;
	M_Place		hpl, lpl, pl_last = NULL;
	M_Transition	htr, ltr, tr_last = NULL;
	int		ident;
	char		*name, *ref, *cmd;

	bool	newhlnet = !*hnet;
	bool	newllnet = !*lnet;

	hpl = lpl = (M_Place) htr = ltr = (M_Transition) 1;

	NNinput_file = PEPfilename;
	NNinput_line = 1;

	/* Oeffne PEP-Datei */
	if (!(infile = fopen(PEPfilename, "r")))
	{
		fprintf(stderr, "ERROR: read_HLLLref(): could not open file "
				"'%s' for reading.\n", PEPfilename);
		exit(2);
	} /* if */
	
	if (newllnet) *lnet = M_empty_net();
	(*lnet)->type = "PetriBox";

	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "PEP"))
		PEPerror("keyword 'PEP' expected");

	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "References_HL_LL"))
		PEPerror("format identifier 'References_HL_LL' expected");

	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strncmp(cmd, "FORMAT_N", 8))
		PEPerror("keyword 'FORMAT_N' or 'FORMAT_N2' expected");

    /* Read Link symbols :  we just ignore them ******************************/
    if (!strcmp( cmd, "LS" ))
      {
        while (!feof(infile) && strcmp(cmd=ReadCmdToken(infile), "PL"))
          ReadNewline(infile);
        /* put back the last '\n' because of the ReadNewLine after
           strcmp(cmd,"PL")*/
        ungetc ('\n',infile);
      }; /* if ( strcmp( cmd, "LS" ) ) */

	/* Read in place nodes ***********************************************/
	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "PL")) PEPerror("keyword 'PL' expected");
	ReadNewline(infile);

	ident = 0;
	if (!newllnet) lpl = (*lnet)->Places;
	if (!newhlnet) hpl = (*hnet)->Places;

	while ((ch = ReadCharComment(infile)) && (ch != 'T'))
	{
		ReadRefObject(infile,ch,&name,&ref);
		if (!newllnet)
		    if (!(lpl = lpl->nextplace) || strcmp(name,lpl->charname))
			for (lpl = (*lnet)->Places; lpl; lpl = lpl->nextplace)
				if (!strcmp(name,lpl->charname)) break;

		if (!newhlnet && strcmp(ref,hpl->charname))
		    if (!(hpl = hpl->nextplace) || strcmp(ref,hpl->charname))
			for (hpl = (*hnet)->Places; hpl; hpl = hpl->nextplace)
				if (!strcmp(ref,hpl->charname)) break;

		if (!lpl || !hpl) PEPerror("invalid place identifier");

		if (newllnet)
		{
			(void) M_fast_insert_place(*lnet,++ident,&pl_last);
			pl_last->charname = MYstrdup(name);
			pl_last->parentplace = hpl;
			pl_last->hlref = MYstrdup(ref);
		}
		else
		{
			lpl->hlref = MYstrdup(ref);
			if (!newhlnet) lpl->parentplace = hpl;
		}
		
		NNinput_line++;
	}

	/* Read in transition nodes ******************************************/
	ungetc(ch, infile);
	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "TR")) PEPerror("keyword 'TR' expected");
	ReadNewline(infile);

	ident = 0;
	if (!newllnet) ltr = (*lnet)->Transitions;
	if (!newhlnet) htr = (*hnet)->Transitions;

	while ((ch = ReadCharComment(infile)) && (ch != 'P') && !feof(infile))
	{
		ReadRefObject(infile,ch,&name,&ref);
		if (!newllnet)
		    if (!(ltr = ltr->nexttrans) || strcmp(name,ltr->charname))
		      for (ltr = (*lnet)->Transitions; ltr; ltr = ltr->nexttrans)
			if (!strcmp(name,ltr->charname)) break;

		if (!newhlnet && strcmp(ref,htr->charname))
		    if (!(htr = htr->nexttrans) || strcmp(ref,htr->charname))
		      for (htr = (*hnet)->Transitions; htr; htr = htr->nexttrans)
			if (!strcmp(ref,htr->charname)) break;

		if (!ltr || !htr) PEPerror("invalid transition identifier");

		if (newllnet)
		{
			(void) M_fast_insert_transition(*lnet,++ident,&tr_last);
			tr_last->charname = MYstrdup(name);
			tr_last->parenttrans = htr;
			tr_last->hlref = MYstrdup(ref);
		}
		else
		{
			ltr->hlref = MYstrdup(ref);
			if (!newhlnet) ltr->parenttrans = htr;
		}
		
		NNinput_line++;
	}

	/* Read in phantom transitions ***************************************/
	if (!feof(infile))
	{
	    ungetc(ch, infile);
	    cmd=ReadCmdToken(infile);
	    if (strcmp(cmd, "PTR")) PEPerror("keyword 'PTR' expected");
	    ReadNewline(infile);

	    if (!newllnet) ltr = (*lnet)->PhTransitions;
	    if (!newhlnet) htr = (*hnet)->PhTransitions;
	    ident = 0;
	    tr_last = NULL;
	    while ((ch = ReadCharComment(infile)) != '\n' && !feof(infile))
	    {
		ReadRefObject(infile,ch,&name,&ref);
		if (!newllnet)
		  if (!(ltr = ltr->nexttrans) || strcmp(name,ltr->charname))
		    for (ltr = (*lnet)->PhTransitions; ltr; ltr = ltr->nexttrans)
			if (!strcmp(name,ltr->charname)) break;

		if (!newhlnet && strcmp(ref,htr->charname))
		  if (!(htr = htr->nexttrans) || strcmp(ref,htr->charname))
		    for (htr = (*hnet)->PhTransitions; htr; htr = htr->nexttrans)
			if (!strcmp(ref,htr->charname)) break;

		if (!ltr || !htr) PEPerror("invalid transition identifier");

		if (newllnet)
		{
		    (void) M_insert_phantom_transition(*lnet,++ident,&tr_last);
		    tr_last->charname = MYstrdup(name);
		    tr_last->parenttrans = htr;
		    tr_last->hlref = MYstrdup(ref);
		}
		else
		{
		    ltr->hlref = MYstrdup(ref);
		    if (!newhlnet) ltr->parenttrans = htr;
		}
		
		NNinput_line++;
	    }

	} /* if */

	fclose(infile);

} /* read_HLLLref() */

/*****************************************************************************/
/***************************** read_BPNref ***********************************/
/*****************************************************************************/

void read_BPNref(M_Net net, char *PEPfilename)
{
	register char	ch;
	char*		c;
	register FILE	*infile;
	int		limit,ident;
	M_Place		pl;
	M_Transition	tr;
	char		*name,*ref, *cmd;

	NNinput_file = PEPfilename;
	NNinput_line = 1;

	/* Oeffne PEP-Datei */
	if (!(infile = fopen(PEPfilename, "r")))
	{
		fprintf(stderr, "ERROR: read_BPNref(): could not open file "
				"'%s' for reading.\n", PEPfilename);
		exit(2);
	} /* if */

	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "PEP"))
		PEPerror("keyword 'PEP' expected");

	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strncmp(cmd, "References_BPN2_",16))
		PEPerror("format identifier 'References_BPN2_[HL]L' expected");

	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strncmp(cmd, "FORMAT_N", 8))
		PEPerror("keyword 'FORMAT_N' or 'FORMAT_N2' expected");

	/* Read in DEF settings */
	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "DEF")) PEPerror("keyword 'DEF' expected");
	ReadNewline(infile);

	limit = MBSIZE * 3/4;
	net->definfo = MYmalloc(MBSIZE);
	c = net->definfo;
		
	while ((*c = getc(infile)) != 'P' && *c != 'B' && *c != 'L' && c++)
	{
		while ((*c++ = getc(infile)) != '\n');
		if (c-net->definfo > limit)
		{
			int offset = c-net->definfo;
			limit += MBSIZE;
			net->definfo = MYrealloc(net->definfo,limit+MBSIZE*1/4);
			c = net->definfo + offset;
		}
		NNinput_line++;
	}

	ungetc(*c,infile);
	cmd=ReadCmdToken(infile);
	*c = '\0';

	/* Read in BL settings */
	if (!strcmp(cmd, "BL"))
	{
		ReadNewline(infile);

		limit = MBSIZE * 3/4;
		net->refblockinfo = MYmalloc(MBSIZE);
		c = net->refblockinfo;
			
		while ((*c = getc(infile)) != 'P' && (*c != 'L')&& c++)
		{
			while ((*c++ = getc(infile)) != '\n');
			if (c-net->refblockinfo > limit)
			{
			    int offset = c-net->refblockinfo;
			    limit += MBSIZE;
			    net->refblockinfo = MYrealloc(net->refblockinfo,limit+MBSIZE*1/4);
			    c = net->refblockinfo + offset;
			}
			NNinput_line++;
		}

		ungetc(*c,infile);
		cmd=ReadCmdToken(infile);
		*c = '\0';
	}

	/* Read in place nodes ***********************************************/
	if (strcmp(cmd, "PL")) PEPerror("keyword 'PL' expected");
	ReadNewline(infile);
	pl = net->Places;
	while ((ch = ReadCharComment(infile)) && (ch != 'T'))
	{
		ReadBPNRefObject(infile,ch,&ident,&name,&ref);
		if ((ident && ident != pl->numname) ||
		    (name && strcmp(name,pl->charname)))
		{
			for (pl = net->Places; pl; pl = pl->nextplace)
				if ((ident && ident == pl->numname) ||
				    (name && !strcmp(name,pl->charname)))
					break;
			if (!pl)
				PEPerror("invalid place identifier");
		}

		if (ref) pl->bpnref = MYstrdup(ref);
		pl = pl->nextplace;
		if (!pl) pl = net->Places;
		NNinput_line++;
	}

	/* Read in transition nodes ******************************************/
	ungetc(ch, infile);
	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "TR")) PEPerror("keyword 'TR' expected");
	ReadNewline(infile);

	tr = net->Transitions;
	while ((ch = ReadCharComment(infile)) && (ch != 'P') && !feof(infile))
	{
		ReadBPNRefObject(infile,ch,&ident,&name,&ref);
		if ((ident && ident != tr->numname) ||
		    (name && strcmp(name,tr->charname)))
		{
			for (tr = net->Transitions; tr; tr = tr->nexttrans)
				if ((ident && ident == tr->numname) ||
				    (name && !strcmp(name,tr->charname)))
					break;
			if (!tr)
				PEPerror("invalid transition identifier");
		}

		if (ref) tr->bpnref = MYstrdup(ref);
		tr = tr->nexttrans;
		if (!tr) tr = net->Transitions;
		NNinput_line++;
	}

	/* Read in phantom transitions ***************************************/
	if (!feof(infile))
	{
		ungetc(ch, infile);
		cmd=ReadCmdToken(infile);
		if (strcmp(cmd, "PTR")) PEPerror("keyword 'PTR' expected");
		ReadNewline(infile);

		tr = net->PhTransitions;
		while ((ch = ReadCharComment(infile)) != '\n' && !feof(infile))
		{
			ReadBPNRefObject(infile,ch,&ident,&name,&ref);
			if (!tr || (ident && ident != tr->numname) ||
			    (name && strcmp(name,tr->charname)))
			{
			  for (tr = net->PhTransitions; tr; tr = tr->nexttrans)
				if ((ident && ident == tr->numname) ||
				    (name && !strcmp(name,tr->charname)))
					break;
			  if (!tr)
			      PEPerror("invalid phantom transition identifier");
			}

			if (ref) tr->bpnref = MYstrdup(ref);
			tr = tr->nexttrans;
			if (!tr) tr = net->PhTransitions;
			NNinput_line++;
		}

	} /* if */

	fclose(infile);

} /* read_BPNref() */

/*****************************************************************************/
/***************************** read_SDLref ***********************************/
/*****************************************************************************/

void read_SDLref(M_Net net, char *PEPfilename)
{
	register char	ch;
	char*		c;
	register FILE	*infile;
	int		limit,ident;
	M_Place		pl;
	M_Transition	tr;
	char		*name,*ref, *cmd;

	NNinput_file = PEPfilename;
	NNinput_line = 1;

	/* Oeffne PEP-Datei */
	if (!(infile = fopen(PEPfilename, "r")))
	{
		fprintf(stderr, "ERROR: read_SDLref(): could not open file "
				"'%s' for reading.\n", PEPfilename);
		exit(2);
	} /* if */

	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "PEP"))
		PEPerror("keyword 'PEP' expected");

	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strncmp(cmd, "References_SDL_",15))
		PEPerror("format identifier 'References_SDL_[HL]L' expected");

	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strncmp(cmd, "FORMAT_N", 8))
		PEPerror("keyword 'FORMAT_N' or 'FORMAT_N2' expected");

	/* Read in DEF settings */
	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "DEF")) PEPerror("keyword 'DEF' expected");
	ReadNewline(infile);

	limit = MBSIZE * 3/4;
	net->sdldefs = MYmalloc(MBSIZE);
	c = net->sdldefs;
		
	while ((*c = getc(infile)) != 'P' && *c != 'B' &&  *c != 'L' && c++)
	{
		while ((*c++ = getc(infile)) != '\n');
		if (c-net->sdldefs > limit)
		{
			int offset = c-net->sdldefs;
			limit += MBSIZE;
			net->sdldefs = MYrealloc(net->sdldefs,limit+MBSIZE*1/4);
			c = net->sdldefs + offset;
		}
		NNinput_line++;
	}

	ungetc(*c,infile);
	cmd=ReadCmdToken(infile);
	*c = '\0';

	/* Read in BL settings */
	if (!strcmp(cmd, "BL"))
	{
		ReadNewline(infile);

		limit = MBSIZE * 3/4;
		net->sdlblockinfo = MYmalloc(MBSIZE);
		c = net->sdlblockinfo;
			
		while ((*c = getc(infile)) != 'P' && *c!= 'L'&& c++)
		{
			while ((*c++ = getc(infile)) != '\n');
			if (c-net->sdlblockinfo > limit)
			{
			    int offset = c-net->sdlblockinfo;
			    limit += MBSIZE;
			    net->refblockinfo = MYrealloc(net->sdlblockinfo,limit+MBSIZE*1/4);
			    c = net->sdlblockinfo + offset;
			}
			NNinput_line++;
		}

		ungetc(*c,infile);
		cmd=ReadCmdToken(infile);
		*c = '\0';
	}

	/* Read in place nodes ***********************************************/
	if (strcmp(cmd, "PL")) PEPerror("keyword 'PL' expected");
	ReadNewline(infile);
	pl = net->Places;
	while ((ch = ReadCharComment(infile)) && (ch != 'T'))
	{
		ReadBPNRefObject(infile,ch,&ident,&name,&ref);
		if ((ident && ident != pl->numname) ||
		    (name && strcmp(name,pl->charname)))
		{
			for (pl = net->Places; pl; pl = pl->nextplace)
				if ((ident && ident == pl->numname) ||
				    (name && !strcmp(name,pl->charname)))
					break;
			if (!pl)
				PEPerror("invalid place identifier");
		}

		if (ref) pl->sdlref = MYstrdup(ref);
		pl = pl->nextplace;
		if (!pl) pl = net->Places;
		NNinput_line++;
	}

	/* Read in transition nodes ******************************************/
	ungetc(ch, infile);
	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "TR")) PEPerror("keyword 'TR' expected");
	ReadNewline(infile);

	tr = net->Transitions;
	while ((ch = ReadCharComment(infile)) && (ch != 'P') && !feof(infile))
	{
		ReadBPNRefObject(infile,ch,&ident,&name,&ref);
		if ((ident && ident != tr->numname) ||
		    (name && strcmp(name,tr->charname)))
		{
			for (tr = net->Transitions; tr; tr = tr->nexttrans)
				if ((ident && ident == tr->numname) ||
				    (name && !strcmp(name,tr->charname)))
					break;
			if (!tr)
				PEPerror("invalid transition identifier");
		}

		if (ref) tr->sdlref = MYstrdup(ref);
		tr = tr->nexttrans;
		if (!tr) tr = net->Transitions;
		NNinput_line++;
	}

	/* Read in phantom transitions ***************************************/
	if (!feof(infile))
	{
		ungetc(ch, infile);
		cmd=ReadCmdToken(infile);
		if (strcmp(cmd, "PTR")) PEPerror("keyword 'PTR' expected");
		ReadNewline(infile);

		tr = net->PhTransitions;
		while ((ch = ReadCharComment(infile)) != '\n' && !feof(infile))
		{
			ReadBPNRefObject(infile,ch,&ident,&name,&ref);
			if (!tr || (ident && ident != tr->numname) ||
			    (name && strcmp(name,tr->charname)))
			{
			  for (tr = net->PhTransitions; tr; tr = tr->nexttrans)
				if ((ident && ident == tr->numname) ||
				    (name && !strcmp(name,tr->charname)))
					break;
			  if (!tr)
			      PEPerror("invalid phantom transition identifier");
			}

			if (ref) tr->sdlref = MYstrdup(ref);
			tr = tr->nexttrans;
			if (!tr) tr = net->PhTransitions;
			NNinput_line++;
		}

	} /* if */

	fclose(infile);

} /* read_SDLref() */

/*****************************************************************************/
/***************************** read_deadlist *********************************/
/*****************************************************************************/

void read_deadlist(M_Net net, char *PEPfilename)
{
	M_Transition	tr, tr_last;
	char		name[512];
	FILE		*infile;

	NNinput_file = PEPfilename;
	NNinput_line = 1;

	/* Oeffne PEP-Datei */
	if (!(infile = fopen(PEPfilename, "r")))
	{
		fprintf(stderr, "ERROR: read_deadlist(): could not open file "
				"'%s' for reading.\n", PEPfilename);
		exit(2);
	} /* if */

	tr_last = net->Transitions;
	while (!feof(infile) && (ReadName(infile, name, 512)>1))
	{
	  /* As dead list is usually ordered, try to be clever */
	  for (tr=tr_last; tr && strcmp(tr->charname,name); 
		tr = tr->nexttrans)
	    skip;
	  if (!tr)
	  {
	    for (tr = net->Transitions;
		tr != tr_last && strcmp(tr->charname,name);
		tr = tr->nexttrans)
	    skip;
	  }
	  if (strcmp(tr->charname,name))
	    fprintf(stderr,"Invalid transition identifier %s ignored.\n",name);
	  else
	    tr->dead = True;
	  tr_last = tr_last->nexttrans;
	  /* skip comma after name */
	  *name=ReadWhiteSpace(infile);
	}
	fclose(infile);
} /* read_deadlist */

/*****************************************************************************/
/***************************** write_HLnet ***********************************/
/*****************************************************************************/
/*                                                  09.01.1995 Stefan Roemer */
/*****************************************************************************/

void write_HLnet(M_Net net, char *PEPfilename)
{
	register M_Place	pl;
	register M_Transition	tr;
	register M_STEdge	st;
	register M_TSEdge	ts;
	FILE			*file;
	int cnt;

	if (!(file = fopen(PEPfilename, "w")))
		PEPerror("Could not open file for writing");

	f("PEP\n");
	fx("%s\n", net->type);
	fx("FORMAT_%s\n",(*net->blockinfo || net->NumPhTransitions)? "N2" : "N");

	f("DPL s7n10@-9t2\n");
	f("DTR s7n10@-9t2\n");
	f("DPT w1t1\n");

	if (net->dfont) fx("%s",net->dfont);

	if (*net->blockinfo)
	{
		f("BL\n");
		fputs(net->blockinfo,file);
	}

    /* Link Symbols */
	if (net -> NumLinks >0)
	{
	  f("LS\n");
	  for (cnt=0;cnt<(net->NumLinks);cnt++)
	     fx("%s\n", net->Links[cnt]);
	};

	/* Liste der Stellen ausgeben */
	f("PL\n");
	for (pl = net->Places; pl; pl = pl->nextplace)
	{
		if (!pl->dead)
		{
			fxy("%u\"%s\"", pl->numname, pl->charname);
			fxy("%d@%d", pl->co->x, pl->co->y);
			fx("%s\n", pl->other);
		}
	}

	/* Liste der Transitionen ausgeben */
	f("TR\n");
	for (tr = net->Transitions; tr; tr = tr->nexttrans)
	{
		if (!tr->dead)
		{
			fxy("%u\"%s\"", tr->numname, tr->charname);
			fxy("%d@%d", tr->co->x, tr->co->y);
			if (tr->corrtran) fx("P\"%s\"",tr->corrtran);
			fx("%s\n", tr->other);
			if (tr->links) fx("l\"%s\"",tr->links);
		}
	}

	if (net->NumPhTransitions) f("PTR\n");
	for (tr = net->PhTransitions; tr; tr = tr->nexttrans)
	{
		if (!tr->dead)
		{
			fxy("%u\"%s\"", tr->numname, tr->charname);
			fxy("%d@%d", tr->co->x, tr->co->y);
			fx("P\"%s\"",tr->corrtran);
			fx("%s\n", tr->other);
		}
	}

	/* Liste der TS-Kanten ausgeben */
	f("TP\n");
	for (tr = net->Transitions; tr; tr = tr->nexttrans)
	    if (!tr->dead)
	    {
	      for (ts = tr->postset; ts; ts = ts->nextplace)
		if (!(pl = ts->place)->dead)
		{
			fxy("%u<%u", tr->numname, pl->numname);
			fx("%s\n",ts->other);
		}
	    }

	f("PT\n");
	for (tr = net->Transitions; tr; tr = tr->nexttrans)
	    if (!tr->dead)
	    {
	      for (st = tr->preset; st; st = st->nextplace)
		if (!(pl = st->place)->dead)
		{
			fxy("%u>%u", pl->numname, tr->numname);
			fx("%s\n",st->other);
		}
	    }

	/* Liste der Phantom-Kanten ausgeben */
	if (net->NumPhTransitions) f("PTP\n");
	for (tr = net->PhTransitions; tr; tr = tr->nexttrans)
	    if (!tr->dead) for (ts = tr->postset; ts; ts = ts->nextplace)
		if (!(pl = ts->place)->dead)
		{
			fxy("%u<%u", tr->numname, pl->numname);
			fx("%s\n",ts->other);
		}

	if (net->NumPhTransitions) f("PPT\n");
	for (tr = net->PhTransitions; tr; tr = tr->nexttrans)
	    if (!tr->dead) for (st = tr->preset; st; st = st->nextplace)
		if (!(pl = st->place)->dead)
		{
			fxy("%u>%u", pl->numname, tr->numname);
			fx("%s\n",st->other);
		}

	if (*net->textinfo)
	{
		f("TX\n");
		fputs(net->textinfo,file);
	}
	fclose(file);

} /* write_HLnet() */

/*****************************************************************************/
/**************************** write_HLLLref **********************************/
/*****************************************************************************/

void write_HLLLref(M_Net net, char *PEPfilename, int flag)
/* flag == 1 <=> LL-Netz wurde minimiert */
{
	register M_Place      pl;
	register M_Transition tr;
	FILE    *file;

	if (!(file = fopen(PEPfilename, "w")))
		PEPerror("Could not open file for writing");

	f("PEP\n");
	f("References_HL_LL\n");
	fx("FORMAT_%s\n", net->NumPhTransitions? "N2" : "N");

	f("PL\n");
	for (pl = net->Places; pl; pl = pl->nextplace)
	    if (pl->hlref)
		if ((flag && !pl->dead) || (!flag && !pl->parentplace->dead))
			fxy("%s\"%s\"\n", pl->charname, pl->hlref);

	f("TR\n");
	for (tr = net->Transitions; tr; tr = tr->nexttrans)
	    if (tr->hlref)
		if ((flag && !tr->dead) || (!flag && !tr->parenttrans->dead))
			fxy("%s\"%s\"\n", tr->charname, tr->hlref);

	if (net->NumPhTransitions) f("PTR\n");
	for (tr = net->PhTransitions; tr; tr = tr->nexttrans)
	    if (tr->hlref)
		if ((flag && !tr->dead) || (!flag && !tr->parenttrans->dead))
			fxy("%s\"%s\"\n", tr->charname, tr->hlref);

	fclose(file);

} /* write_HLLLref */


/*****************************************************************************/
/***************************** write_BPNref **********************************/
/*****************************************************************************/

void write_BPNref(M_Net net, char *PEPfilename)
{
	register M_Place      pl;
	register M_Transition tr;
	FILE                  *file;

	if (!(file = fopen(PEPfilename, "w")))
		PEPerror("Could not open file for writing");

	f("PEP\n");
	fx("References_BPN2_%cL\n", strcmp(net->type,"HLNet")? 'L' : 'H');
	fx("FORMAT_%s\n",
		(net->refblockinfo || net->NumPhTransitions)? "N2" : "N");

	f("DEF\n");
	if (net->definfo) fputs(net->definfo,file);

	if (net->refblockinfo)
	{
		f("BL\n");
		fputs(net->refblockinfo,file);
	}

	f("PL\n");
	for (pl = net->Places; pl; pl = pl->nextplace)
		if (pl->bpnref && !pl->dead)
			fxy("%s T\"%s\"\n", pl->charname, pl->bpnref);

	f("TR\n");
	for (tr = net->Transitions; tr; tr = tr->nexttrans)
		if (tr->bpnref && !tr->dead)
			fxy("%s T\"%s\"\n", tr->charname, tr->bpnref);

	if (net->NumPhTransitions) f("PTR\n");
	for (tr = net->PhTransitions; tr; tr = tr->nexttrans)
		if (tr->bpnref && !tr->dead)
			fxy("%s T\"%s\"\n", tr->charname, tr->bpnref);

	fclose(file);

} /* write_BPNref */

/*****************************************************************************/
/***************************** write_SDLref **********************************/
/*****************************************************************************/

void write_SDLref(M_Net net, char *PEPfilename)
{
	register M_Place      pl;
	register M_Transition tr;
	FILE                  *file;

	if (!(file = fopen(PEPfilename, "w")))
		PEPerror("Could not open file for writing");

	f("PEP\n");
	fx("References_SDL_%cL\n", strcmp(net->type,"HLNet")? 'L' : 'H');
	fx("FORMAT_%s\n",
		(net->sdlblockinfo || net->NumPhTransitions)? "N2" : "N");

	f("DEF\n");
	if (net->sdldefs) fputs(net->sdldefs,file);

	if (net->sdlblockinfo)
	{
		f("BL\n");
		fputs(net->sdlblockinfo,file);
	}

	f("PL\n");
	for (pl = net->Places; pl; pl = pl->nextplace)
		if (pl->sdlref && !pl->dead)
			fxy("%s T\"%s\"\n", pl->charname, pl->sdlref);

	f("TR\n");
	for (tr = net->Transitions; tr; tr = tr->nexttrans)
		if (tr->sdlref && !tr->dead)
			fxy("%s T\"%s\"\n", tr->charname, tr->sdlref);

	if (net->NumPhTransitions) f("PTR\n");
	for (tr = net->PhTransitions; tr; tr = tr->nexttrans)
		if (tr->sdlref && !tr->dead)
			fxy("%s T\"%s\"\n", tr->charname, tr->sdlref);

	fclose(file);

} /* write_SDLref */

/*****************************************************************************/
/* END OF FILE readHL.c                                                      */
/*****************************************************************************/

