/*****************************************************************************/
/* readHL.c                                                                  */
/*****************************************************************************/
/*                                  Stefan Roemer,  06.02.1995 - 06.02.1995  */
/*****************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "readHL.h"
#include "readPEP.h"
#include "readlib.h"
#include "ref.h"
#include "list.h"

/*****************************************************************************/

#define NAMES_START 2000
#define NAMES_OFFSET 1000

#define ReadEnclString RL_ReadEnclString
/*****************************************************************************/
/***************************** read_HLnet ************************************/
/*****************************************************************************/
/* Neu eingefuegt (neues Format)       19.12.1994 - 09.01.1995 Stefan Roemer */
/*****************************************************************************/

M_Net		net;
M_Block		*BlArray;
M_Place		*PlArray;

Cardinal32	AnzBlNamen, MaxBlNamen,
		AnzPlNamen, MaxPlNamen;

char buffer[10];
int  blockcount;
int  placecount;
bool autoblocknumbering;
bool autonumbering;

char	 *name;
char	 *blocklist;
int	 ident;

int insert_block();
int insert_place();

M_Net read_HLnet(char *PEPfilename)
{
	t_dest block_dest[] =
		{ { '\'',&name },	/* Bezeichner			*/
		  { '"', &name },	/*     "			*/
		  { '0', &ident },	/* numerischer Name		*/
		  { 'u', &blocklist },	/* Liste der Unterbloecke	*/
		  {  0 ,  0 } };
	  
	t_dest place_dest[] =
		{ { '\'',&name },	/* Bezeichner		*/
		  { '"', &name },	/*     "		*/
		  { '0', &ident },	/* numerischer Name	*/
		  { 'u', &blocklist },	/* Liste der Blocke	*/
		  {  0 ,  0 } };
	  
	t_blockdest netdest[] =
		{ { "BL",  insert_block, NULL, block_dest },
		  { "PL",  insert_place, NULL, place_dest },
		  { NULL, NULL, NULL, NULL } };

	int		count;

	BlArray = (M_Block*) MYmalloc((count = MaxBlNamen
			= NAMES_START) * sizeof(M_Block));
	PlArray = (M_Place*) MYmalloc((count = MaxPlNamen
			= NAMES_START) * sizeof(M_Place));
	AnzBlNamen = AnzPlNamen = 0;
	while (--count)
	{
		BlArray[count] = (M_Block) NULL;
		PlArray[count] = (M_Place) NULL;
	} /* while */

	/* Initialize net */
	net = M_empty_net();

	blockcount = placecount = 0;
	autoblocknumbering = autonumbering = True;

	/* read the net */
	read_PEP_net(PEPfilename, netdest);

	net->type = filetype;

	free(BlArray);
	free(PlArray);

	return net;

} /* read_HLnet() */

int insert_block()
{
	static M_Block bl_last = NULL;

	blockcount++;
	if (ident && ident != blockcount) autoblocknumbering = False;
	if (!ident && autoblocknumbering) ident = blockcount;

	if (!ident) PEPerror("missing block identifier");
		
	if (ident > AnzBlNamen)
		AnzBlNamen = ident;
	else if (BlArray[ident])
		Error((errbuf, "block identifier %d used twice", ident));

	while (AnzBlNamen >= MaxBlNamen)
	{
		int count;
		BlArray = (M_Block*) MYrealloc((char*) BlArray,
				(MaxBlNamen += NAMES_OFFSET) * sizeof(M_Block));
		for (count = MaxBlNamen - NAMES_OFFSET; count < MaxBlNamen;)
			BlArray[count++] = (M_Block) NULL;

	}

	BlArray[ident] = M_fast_insert_block(net,ident,&bl_last);
	if (!name) sprintf(buffer, "B%u", ident);
	bl_last->charname = name? name : MYstrdup(buffer);
	bl_last->childblocks = blocklist;
	return 0;
}

int insert_place()
{
	static M_Place pl_last = NULL;

	placecount++;
	if (ident && ident != placecount) autonumbering = False;
	if (!ident && autonumbering) ident = placecount;
	if (!ident) PEPerror("missing place identifier");
		
	if (ident > AnzPlNamen)
		AnzPlNamen = ident;
	else if (PlArray[ident])
		Error((errbuf, "place identifier %d used twice", ident));

	while (AnzPlNamen >= MaxPlNamen)
	{
		int count;
		PlArray = (M_Place*) MYrealloc((char*) PlArray,
				(MaxPlNamen += NAMES_OFFSET) * sizeof(M_Place));
		for (count = MaxPlNamen - NAMES_OFFSET; count < MaxPlNamen;)
			PlArray[count++] = (M_Place) NULL;

	}

	PlArray[ident] = M_fast_insert_place(net,ident,&pl_last);
	if (!name) sprintf(buffer, "P%u", ident);
	pl_last->charname = name? name : MYstrdup(buffer);
	pl_last->blocklist = blocklist;
	return 0;
}

/*****************************************************************************/

void ReadBPNRefObject(FILE *infile,char ch,int *index,char **name,char **ref)
{
	static char buffer[1024];
	char *c = buffer;
	*c = ch;

	*index = 0;
	*name = buffer;
	*ref = NULL;
	
	if (isdigit(*c))
	{
		ungetc(*c,infile);
		*index=ReadNumber(infile);
		*name = NULL;
	}
	else
	{
		while (isalnum(*++c = getc(infile)));
		*c++ = '\0';
	}

	if (ReadCharComment(infile) != 'T') PEPerror("'T' expected");
	ReadCharComment(infile);

	*ref = c;
	while ((*c++ = getc(infile)) != '"' && !feof(infile));
	*--c = '\0';

	if (ReadCharComment(infile) != '\n')
		PEPerror("garbage characters after reference");
		
}

/*****************************************************************************/
/***************************** read_BPNref ***********************************/
/*****************************************************************************/

void read_BPNref(M_Net net, char *PEPfilename)
{
	register char	ch;
	register FILE	*infile;
	int		tindex,ident;
	int		cap,type,par;
	M_Block		bl;
	M_Place		pl;
	char		*c, *cmd;
	char		*name,*ref;
	t_bi		bi;
	MTupel		parallel = MT_Create();

	NNinput_file = PEPfilename;
	NNinput_line = 1;

	/* Oeffne PEP-Datei */
	if (!(infile = fopen(PEPfilename, "r")))
	{
		fprintf(stderr, "ERROR: read_BPNref(): could not open file "
				"'%s' for reading.\n", PEPfilename);
		exit(2);
	} /* if */

	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "PEP"))
		PEPerror("keyword 'PEP' expected");

	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strncmp(cmd, "References_BPN2_LL",18))
		PEPerror("format identifier 'References_BPN2_LL' expected");

	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strncmp(cmd, "FORMAT_N", 8))
		PEPerror("keyword 'FORMAT_N' or 'FORMAT_N2' expected");

	/* Read in DEF settings */
	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "DEF")) PEPerror("keyword 'DEF' expected");
	ReadNewline(infile);

	net->blockinfo = MPT_Create();
	tindex = 0;

	while ((ch = getc(infile)) != 'P' && ch != 'B')
	{
	    ungetc(ch,infile);
	    cap = 0;
	    type = 0;
	    ident = 0;
	    par = 0;
	    while ((ch = ReadCharComment(infile)) != '\n')
	    {
		if (isdigit(ch))
		{
			ident = ch - '0';
			while (isdigit(ch = getc(infile)))
			{
				ident *= 10;
				ident += ch - '0';
			}
			ungetc(ch,infile);
		}
		else
		{
		    switch(ch)
		    {
			case 'U':
				type = PROCEDURE;
				break;
			case 'V':
				type = VARIABLE;
			case 'R':
			case 'b':
				cmd=ReadEnclString(infile);
			case 'A':
				break;
			case 'E':
				cmd=ReadEnclString(infile);
				par = (*cmd == '1');
				break;
			case 'C':
			case 'S':
				cmd=ReadEnclString(infile);
				for (c = cmd; !isdigit(*c); c++);
				for (cap = *c-'0'; isdigit(*++c); cap += *c-'0')					cap *= 10;
				type = (ch == 'S'? STACK : CHANNEL);
				break;
			default:
				PEPerror("unknown token in definition");
		    }
		}
	    }
	
	    MT_AddValue(parallel, par);
	    bi = NULL;
	    if (type)
	    {
		    bi = (t_bi)MYmalloc(sizeof(t_bi_elem));
		    bi->ident = ident;
		    bi->type = type;
		    bi->anz = ( type == VARIABLE? 1 :
		    		type == PROCEDURE? 1 :
				type == STACK? cap+1 :
					       Max(1,cap) );
		    bi->index = tindex;
		    bi->mindimen = DOT;
		    bi->maxdimen = -DOT;
		    tindex += bi->anz;
	    }
	    MPTT_AddValue(net->blockinfo,bi);
	    net->numblocks++;
	    NNinput_line++;
	}

	net->numlists = tindex;

	ungetc(ch,infile);
	cmd=ReadCmdToken(infile);

	/* Read in BL settings */
	if (!strcmp(cmd, "BL"))
	{
		ReadNewline(infile);
		bl = net->Blocks;

		while ((ch = getc(infile)) != 'P')
		{
			ReadBPNRefObject(infile,ch,&ident,&name,&ref);
			if ((ident && ident != bl->numname) ||
			    (name && strcmp(name,bl->charname)))
			{
				for (bl = net->Blocks; bl; bl = bl->nextblock)
					if ((ident && ident == bl->numname) ||
					   (name && !strcmp(name,bl->charname)))
						break;
				if (!bl)
					PEPerror("invalid block identifier");
			}

			if (ref)
			{
				c = strchr(ref,':');
				if (!c) c = ref+strlen(ref);
				*c = '\0';
				bl->defnr = atoi(ref);
				bl->parallel = MT_Place(parallel,bl->defnr-1);
			}
			bl = bl->nextblock;
			if (!bl) bl = net->Blocks;
			NNinput_line++;
		}

		ungetc(ch,infile);
		cmd=ReadCmdToken(infile);
	}

	/* Read in place nodes ***********************************************/
	if (strcmp(cmd, "PL")) PEPerror("keyword 'PL' expected");
	ReadNewline(infile);
	pl = net->Places;
	while ((ch = ReadCharComment(infile)) && (ch != 'T'))
	{
		ReadBPNRefObject(infile,ch,&ident,&name,&ref);
		if (!pl || (ident && ident != pl->numname) ||
		    (name && strcmp(name,pl->charname)))
		{
			for (pl = net->Places; pl; pl = pl->nextplace)
				if ((ident && ident == pl->numname) ||
				    (name && !strcmp(name,pl->charname)))
					break;
			if (!pl)
				PEPerror("invalid place identifier");
		}

		if (ref) pl->ref = MYstrdup(ref);
		pl = pl->nextplace;
		if (!pl) pl = net->Places;
		NNinput_line++;
	}

	/* Der Rest interessiert hier nicht. */

	fclose(infile);

} /* read_BPNref() */

/*****************************************************************************/
/***************************** read_SDLref ***********************************/
/*****************************************************************************/

void read_error()
{
	fprintf(stderr, "error while reading set in file %s, line %u\n",
			NNinput_file, NNinput_line);
	exit(2);
}

T_list read_set(char *string)
{
	T_list set = alloc_list();
	char ch, *c;
	int i, i1, i2;

	#define LBRACE '{'
	#define RBRACE '}'

	if (*string++ != LBRACE) read_error();
	for (;;)
	{
		for (c = string; isdigit(*c); c++);
		ch = *c;
		*c = 0;
		i1 = atoi(string);

		if (ch == ',')
		{
			list_add(set,i1);
			string = c+1;
			continue;
		}
		else if (ch == '.')
		{
			string = c+2;
			for (c = string; isdigit(*c); c++);
			ch = *c;
			*c = 0;
			i2 = atoi(string);
			for (i = i1; i <= i2; i++) list_add(set,i);

			if (ch == ',') 
			{
				string = c+1;
				continue;
			}
			else if (ch == RBRACE)
				return set;	
			else read_error();
		}
		else if (ch == RBRACE)
		{
			list_add(set,i1);
			return set;
		}
		else read_error();
	}
}

void alloc_array(T_defitem def)
{
	int i;

	def->array = alloc_list();
	for (i = 0; i <= def->size; i++)
		list_add(def->array,alloc_list());
}

char ref_type_chars[] = REF_TYPE_CHARS;

void read_SDLref(M_Net net, char *PEPfilename)
{
	register char	ch;
	register FILE	*infile;
	M_Place		pl;
	T_defitem	def, pdef = NULL;
	T_list		idset;
	char		*c, *cmd;
	char		*name,*ref;
	int		ident;
	int		size,type,proc,ver;
	int		i, refread;

	NNinput_file = PEPfilename;
	NNinput_line = 1;

	/* Oeffne PEP-Datei */
	if (!(infile = fopen(PEPfilename, "r")))
	{
		fprintf(stderr, "ERROR: read_SDLref(): could not open file "
				"'%s' for reading.\n", PEPfilename);
		exit(2);
	} /* if */

	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "PEP"))
		PEPerror("keyword 'PEP' expected");

	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strncmp(cmd, "References_SDL_LL",17))
		PEPerror("format identifier 'References_SDL_LL' expected");

	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strncmp(cmd, "FORMAT_N", 8))
		PEPerror("keyword 'FORMAT_N' or 'FORMAT_N2' expected");

	/* Read in DEF settings */
	ReadNewline(infile);
	cmd=ReadCmdToken(infile);
	if (strcmp(cmd, "DEF")) PEPerror("keyword 'DEF' expected");
	ReadNewline(infile);

	net->defitems = alloc_list();

	while ((ch = getc(infile)) != 'P')
	{
	    ungetc(ch,infile);
	    size = type = ident = proc = refread = ver = 0;
	    idset = NULL;
	
	    while ((ch = ReadCharComment(infile)) != '\n')
	    {
		if (isdigit(ch))
		{
			ident = ch - '0';
			while (isdigit(ch = getc(infile)))
			{
				ident *= 10;
				ident += ch - '0';
			}
			ungetc(ch,infile);
		}
		else
		{
		    if (refread) for (c = ref_type_chars; *c; c++)
			if (ch == *c) type = c-ref_type_chars;

		    switch(ch)
		    {
			case 'R':
				refread++;
			case 'V':
			case 'I':
			case 'O':
			case 'C':
			case 'X':
			case 'b':
				cmd=ReadEnclString(infile);
				break;
			case 'u':
			case 'c':
			case 'v':
				cmd=ReadEnclString(infile);
				((ch == 'u')? proc : (ch == 'c')? size : ver)
					= atoi(cmd);
				break;
			case 'P':
			case 'U':
				cmd=ReadEnclString(infile);
				idset = read_set(cmd);
				break;
			default:
			        for (c = ref_type_chars; *c; c++)
					if (ch == *c) break;
				if (ch != *c)
					PEPerror("unknown token in definition");
		    }
		}
	    }
	
	    list_add(net->defitems,(def = alloc_defitem()));
	    def->type = type;
	    def->nr = ident;
	    switch (type)
	    {
		case REF_SYSTEM:
			def->list1 = alloc_list();
			break;
		case REF_BLOCK:
			break;
		case REF_CHANNEL:
			def->size = size + ver;
			def->list1 = alloc_list();	/* Kapazitaet */
			alloc_array(def);		/* Inhalt */
			break;
		case REF_SIGNALROUTE:
			def->list1 = alloc_list();
			break;
		case REF_SIGNAL:
			break;
		case REF_CONNECT:
			def->list1 = alloc_list();
			break;
		case REF_VARIABLE:
		case REF_IN_PARM:
		case REF_IN_OUT_PARM:
			for_list (net->defitems,pdef,T_defitem)
				if (pdef->nr == proc) break;
			def->flag = (pdef->type == REF_PROCESS);
			def->idset = pdef->idset;
			def->size = pdef->idset->count;
			alloc_array(def);		/* Inhalt */
			break;
		case REF_PROCESS:
			def->list1 = alloc_list();	/* Entry, Exit etc. */
			def->size = idset->count;
			def->idset = idset;
			alloc_array(def);		/* Kontrollfluss */
			def->more = alloc_list();
			for (i = 0; i < (7+size+(ver/2)) * def->size; i++)
				list_add(def->more,alloc_list());
			break;
		case REF_PROCEDURE:
			def->list1 = alloc_list();	/* Entry, Exit etc. */
			def->list2 = alloc_list();	/* Maximum */
			def->size = idset->count;
			def->idset = idset;
			alloc_array(def);		/* Kontrollfluss */
			def->more = alloc_list();
			/* Stack, State, Dashstate */
			for (i = 0; i < 3*def->size; i++)
				list_add(def->more,alloc_list());
			break;
		case REF_ACTION:
		case REF_LABEL:
			for_list (net->defitems,pdef,T_defitem)
				if (pdef->nr == proc) break;
			def->proc = pdef;
			break;
		case REF_ANSWER:
		case REF_TRIGGER:
			break;
		case REF_SYNTYPE:
		default:
			fprintf(stderr,"unknown reference type in file %s, "
				 "line %u\n", NNinput_file, NNinput_line);
			exit(2);
	    }
	    NNinput_line++;
	}

	ungetc(ch,infile);
	cmd=ReadCmdToken(infile);

	/* Read in place nodes ***********************************************/
	if (strcmp(cmd, "PL")) PEPerror("keyword 'PL' expected");
	ReadNewline(infile);
	pl = net->Places;
	while ((ch = ReadCharComment(infile)) && (ch != 'T'))
	{
		ReadBPNRefObject(infile,ch,&ident,&name,&ref);
		if (!pl || (ident && ident != pl->numname) ||
		    (name && strcmp(name,pl->charname)))
		{
			for (pl = net->Places; pl; pl = pl->nextplace)
				if ((ident && ident == pl->numname) ||
				    (name && !strcmp(name,pl->charname)))
					break;
			if (!pl)
				PEPerror("invalid place identifier");
		}

		if (ref) pl->ref = MYstrdup(ref);
		pl = pl->nextplace;
		if (!pl) pl = net->Places;
		NNinput_line++;
	}

	/* Der Rest interessiert hier nicht. */

	fclose(infile);

} /* read_SDLref() */

/*****************************************************************************/
/* END OF FILE readHL.c                                                      */
/*****************************************************************************/

