#include <stdio.h>

#include "rw_slist.h"
#include "ref.h"
#include "readlib.h"

/*****************************************************************************/
/************************* write_sinvlist_bpn2 *******************************/
/*****************************************************************************/

void write_slist(FILE *file, MPTupel mpt, int k)
{
	M_Place pl;
	int j,c;
	
	if (mpt->count) f("\"");
	for (j = c = 0; j < mpt->count; j++)
	{	
		pl = MT_PlaceP(mpt,j);
		if (!k || k == pl->lastdimen)
		{
			if (c++) f(",");
			fx("%s",MT_PlaceP(mpt,j)->charname);
		}
	}
	if (mpt->count) f("\"\n");
}

void write_sinvlist_bpn2(M_Net net, char *PEPfilename)
{
	FILE            *file;
	int		i,j,k;
	t_bi		bi;
	M_Block		bl;

	if (!(file = fopen(PEPfilename, "w")))
		PEPerror("Could not open file for writing");

	f("PEP\n");
	f("S_List\n");
	f("FORMAT_N2\n");

	f("LIST\n");

	for (i = 0; i < net->numblocks; i++)
	    if ((bi = MT_PlaceBI(net->blockinfo,i)))
	        for (j = 0; j < bi->anz; j++)
		{
		    MPTupel mpt = MT_PlaceMPT(net->blocklists,bi->index+j);
		    if (bi->maxdimen > 0)
			for (k = bi->mindimen; k <= bi->maxdimen; k++)
			    write_slist(file,mpt,k);
		    else
			write_slist(file,mpt,0);
		}

	for (bl = net->Blocks; bl; bl = bl->nextblock)
	    if (bl->maxdimen > 0)
		for (k = bl->mindimen; k <= bl->maxdimen; k++)
		    write_slist(file,bl->places,k);
	    else
		write_slist(file,bl->places,0);
	
	fclose(file);

} /* write_sinvlist_bpn2 */

/*****************************************************************************/
/************************** write_sinvlist_sdl *******************************/
/*****************************************************************************/

void write_sdl_list(FILE *file, T_list list)
{
	M_Place pl;
	int c = 0;

	if (!list->count) return;

	f("\"");
	for_list (list,pl,M_Place)
	{	
		if (c++) f(",");
		fx("%s",pl->charname);
	}
	f("\"\n");
}

void write_sinvlist_sdl(M_Net net, char *PEPfilename)
{
	FILE            *file;
	T_defitem	def;
	T_list		list;

	#define write(list) write_sdl_list(file,list);

	if (!(file = fopen(PEPfilename, "w")))
		PEPerror("Could not open file for writing");

	f("PEP\n");
	f("S_List\n");
	f("FORMAT_N2\n");

	f("LIST\n");

	for_list (net->defitems,def,T_defitem)
		switch (def->type)
		{
		    case REF_SYSTEM:
		    case REF_SIGNALROUTE:
		    case REF_CONNECT:
			write(def->list1);
			break;
		    case REF_CHANNEL:
			write(def->list1);
			for_list (def->array,list,T_list) write(list);
			break;
		    case REF_VARIABLE:
		    case REF_IN_PARM:
		    case REF_IN_OUT_PARM:
			for_list (def->array,list,T_list) write(list);
			break;
		    case REF_PROCESS:
			write(def->list1);
			for_list (def->array,list,T_list) write(list);
			for_list (def->more,list,T_list) write(list);
			break;
		    case REF_PROCEDURE:
			write(def->list1);
			write(def->list2);
			for_list (def->array,list,T_list) write(list);
			for_list (def->more,list,T_list) write(list);
			break;
		    case REF_SIGNAL:
		    case REF_BLOCK:
		    case REF_ACTION:
		    case REF_LABEL:
		    case REF_ANSWER:
		    case REF_SYNTYPE:
		    case REF_TRIGGER:
		    default:
			break;
	    }
	
	fclose(file);

} /* write_sinvlist_sdl */

/*****************************************************************************/
/************************** write_sinvlist_pfa *******************************/
/*****************************************************************************/

void write_sinvlist_pfa(M_Net net, char *PEPfilename)
{
        FILE            *file;

        if (!(file = fopen(PEPfilename, "w")))
                PEPerror("Could not open file for writing");

        f("PEP\n");
        f("S_List\n");
        f("FORMAT_N2\n");

        f("LIST\n");

        fclose(file);

} /* write_sinvlist_pfa */

/*****************************************************************************/
/* END OF FILE rw_slist.c                                                    */
/*****************************************************************************/
