#################################################
# refInOut.tcl
#
# Einlese- und Abspeicherprozeduren fuer
#
# 1. pep_edit
# 2. simulator
# 3. animator
#################################################

########################
# HL und LL Referenzen #
########################

proc read_references { } {
 global gvar hl_ref_ext ll_ref_ext def abstr abs abs_pos stat

 if {[file exists $gvar(prjname)$hl_ref_ext]} {
      read_reference_from_hl_file $gvar(prjname)$hl_ref_ext
 }

  if {[file exists $gvar(prjname)$ll_ref_ext]} {
    read_reference_from_ll_file $gvar(prjname)$ll_ref_ext
    set stat(fileLLRefExists) 1
  }

 if {$gvar(prgtype)!="sdl"} {
   # Einlesen der abstrakten Namen
   if {[array exists abstr]} {
       unset abstr
       unset abs
       unset abs_pos
   }
   set ext ".bpn2_mv_ref"
   read_abstr_from_file $gvar(prjname)$ext
 } else {
   # da gibt's nix mehr zu tun...
 } ;# gvar(prgtype) =?=

}

###############
# Netzdateien #
###############

# Einlesen des LL-Netzes
########################

proc read_ll_net {name} {
 global LTP LPT stat

 # LTP -> Nachbereich (Trans->Place)
 # LPT -> Vorbereich  (Place->Trans)

 if {[file exists $name]} {
    set refid [open $name]
    set line [read_line $refid]
    set stat(fileLLNetExists) 1
 } else {
    return -1
 }

 # Ueberlesen bis zum Definitionsbereich...
 skip_to $refid TP

 set line [read_line $refid]
 while {![eof $refid] && $line != "PT"} {
    regsub -all {\<|v} $line " " line
    if {[llength $line] <= 3 && [string range $line 0 0] != "%"} {
	set T [lindex $line 0]
	set P [lindex $line 1]
	lappend LTP(T$T) "P$P"
    }
    set line [read_line $refid]
 }

 set line [read_line $refid]
 while {![eof $refid] && $line != "PTP"} {
    regsub -all {\>|v} $line " " line
    if {[llength $line] <= 3 && [string range $line 0 0] != "%"} {
	set P [lindex $line 0]
	set T [lindex $line 1]
	lappend LPT(P$P) "T$T"
    }
    set line [read_line $refid]
 }
 return 1
}


###################
# Abstrakte Namen #
###################

# Einlesen der abstrakten Namen (pep_edit)
##########################################
proc read_abstr_from_file {name} {
 global abstr def abs_pos abs stat

 if {[file exists $name]} {
    set refid [open $name]
    set line [read_line $refid]
    set stat(fileMvRefExists) 1
	#DEBUG files "Abstract Names found."
 } else {
    return
 }

 # Ueberlesen bis zum Definitionsbereich...
 skip_to $refid DEF

 # Einlesen der abstrakten Namen und Positionen
 set line [read_line $refid]
 while {![eof $refid] && $line != ""} {
    set abs([lindex $line 0]) [lindex $line 3]
    set abs_pos([lindex $line 0]) [lindex $line 1]
    set line [read_line $refid]
 }

 set def_size [array size def]
 #DEBUG files "-- $def_size Positions found."

 # Zuordnen der abstrakten Namen 
   for {set i 1} {$i <= $def_size} {incr i} {
      set abstr($i) [find_abs_name $def($i)]
   }
}

# Abspeichern der abstrakten Namen mit Positionen (pep_edit)
############################################################
proc save_mv_refs {file} {
   global def abstr

   if {![array exists def]} {return}

   # Ausgabe der Abstrakten Positionen
   set ext "_mv_ref"
   set refs "PEP\nReferences_BPN2_MV\nFORMAT_MV1\nDEF"

   for {set i 1} {$i <= [array size def]} {incr i} {
        set range [.tedit tag range tag$i]
        set from [lindex $range 0]
        set to   [lindex $range 1]
        if {[catch {set abstr_name $abstr($i)}]} {
           set abstr_name ""
        }
        set refs "$refs\n$i [incr_pos $range] \"[.tedit get $from $to]\" \"$abstr_name\""
   }
   echo $refs $file$ext
}

