// -------------------------------------------------------------------------
// Programm : HIPS Highlevel Petrinetz Simulator/Editor
// ----------------------------------------------------
// Datei    : Net.cc
// Funktion : Implementation der Klasse Netz (ST und PetriBox)
// Author   : Tobias Himstedt
//            Universitaet Hildesheim
//            Institut fuer Theoretische Informatik
// -------------------------------------------------------------------------
// Burkhard Bieber 1995/96
// Cecile Bui Thanh: added asynchronous links (2001)
// last-update: Dec. 2001

  //**********
  // includes
  //**********

#include <math.h>
#include <stdlib.h>
#include <strstream>
#include <time.h>
#include "Net.h"
#include "Place.h"
#include "../include/Socket.h"
#include <string.h>
#include <signal.h>

  //***************************
  // Inits fuer Statics aus Net
  // (Warning: Only once!)
  //***************************

int Net::net_counter = 0;
int Net::Title::counter = 0;

  //*********
  // externs
  //*********

extern VHMap<String, Net *> nets;

extern "C" {
  void exit(int);
}  // extern

  //*******************************************************
  // Operatoren fuer die beiden Freunde ostream und istream
  //*******************************************************

ostream &
operator <<(ostream &f, const Net &n) 
{
  n.Write(f);
  return f;
}  // operator <<

istream &
operator >>(istream &f, Net &n)
{
  n.Read(f);
  return f;
}  // operator >>

  //******************
  // Methoden fuer Net
  //******************

Net::Net() : object_size(DefaultLLSize),
  blocks((Block *) NULL),
  boxes((Box *) NULL),
  places((Place *) NULL),
  transitions((Transition *) NULL),
  ptransitions((PTransition *) NULL),
  edges((Edge *) NULL),
  pedges((Edge *) NULL),
  titles(Title(), 5),
  fileorder((NetFileOrder *) NULL),
  declared_links(EmptyLink,10),
  Blockkonvert(0),
  Placekonvert(0),
  Transkonvert(0),
  PTranskonvert(0),
  count(0),
  net_type_switch(0),
  sync_trans_switch(1), phantom_trans_switch(1),
//  rx_block_name(45), ry_block_name(-30),
//  rx_block_mean(45), ry_block_mean(40),
  rx_block_name(20), ry_block_name(-45),
  rx_block_mean(20), ry_block_mean(-20),
  block_thickness(1),
//  rx_place_name(15), ry_place_name(-20), 
//  rx_place_mean(15), ry_place_mean(10),
  rx_place_name(-12), ry_place_name(-6),
  rx_place_mean(20), ry_place_mean(-16),
  place_thickness(1),
//  rx_trans_name(20), ry_trans_name(-25), 
//  rx_trans_mean(15), ry_trans_mean(15),
  rx_trans_name(-12), ry_trans_name(-6), 
  rx_trans_mean(20), ry_trans_mean(-18),
  transition_thickness(1),
  rx_edge_weight(0), ry_edge_weight(10),
  edge_thickness(1),
  clipboard((Net *) NULL),
  with_time(0),
  defaultTrans(0)
  // Konstruktor Net
{
//  NewFileOrder("DBL", "block defaults",                 1,  0, -1,  3);
//  NewFileOrder("DPL", "place defaults",                 1,  1, -1,  3);
//  NewFileOrder("DTR", "transition defaults",            1,  2, -1,  3);
//  NewFileOrder("DPT", "edge defaults",                  1,  3, -1,  3);
//  NewFileOrder("BL",  "blocks",                         1,  4, -1,  3);
//  NewFileOrder("PL",  "places",                         0,  5, -1,  4);
//  NewFileOrder("TR",  "transitions",                    0,  6,  5,  5);
//  NewFileOrder("PTR", "phantom transitions",            1,  7,  6,  9);
//  NewFileOrder("TP",  "transition place edges",         0,  8,  6,  8);
//  NewFileOrder("PT",  "place transition edges",         0,  9,  8,  9);
//  NewFileOrder("PTP", "phantom transition place edges", 1, 10,  7,  9);
//  NewFileOrder("PPT", "place phantom transition edges", 1, 11, 10, 10);
//  NewFileOrder("TX",  "additional text",                1, 12,  8, 11);

  NewFileOrder("DBL", "block defaults",                 1,  0, -1,  3);
  NewFileOrder("DPL", "place defaults",                 1,  1, -1,  3);
  NewFileOrder("DTR", "transition defaults",            1,  2, -1,  3);
  NewFileOrder("DPT", "edge defaults",                  1,  3, -1,  3);
  NewFileOrder("BL",  "blocks",                         1,  4, -1,  3);
  NewFileOrder("LS",  "link symbols",                1, 5,  -1, 6);
  NewFileOrder("PL",  "places",                         0,  6, -1,  5);
  NewFileOrder("TR",  "transitions",                    0,  7,  6,  6);
  NewFileOrder("PTR", "phantom transitions",            1,  8,  7,  10);
  NewFileOrder("TP",  "transition place edges",         0,  9,  7,  9);
  NewFileOrder("PT",  "place transition edges",         0,  10,  9,  10);
  NewFileOrder("PTP", "phantom transition place edges", 1, 11,  8,  10);
  NewFileOrder("PPT", "place phantom transition edges", 1, 12, 11, 11);
  NewFileOrder("TX",  "additional text",                1, 13,  9, 12);

  NameP = NameT = MeaningP = MeaningT = Comments = AnnotationsA = ActionTermT = ValueTermT = TypeP = "NULL";
  for(int z = 0;z < 50;z++)
    Verbindungen[z] = (Socket *) NULL;
}  // Net::Net

Net::~Net()
  // Destruktor Net
{
  DeleteAll();
  DeleteFileOrder();  // Erst hier! Nicht in DeleteAll enthalten,
                      // da DeleteAll auch anderswo aufgerufen wird.
}  // Net::~Net

HLNet *
Net::asHLNet() const
{
  cerr << "brutal cast error! Attempt Net -> HLNet\n";
  exit(1);
  return NULL;
}  // Net::asHLNet

void
Net::DeleteAll(void)
{
  DeleteBlocks();
  DeleteBoxes();
  DeletePlaces();
  DeleteTransitions();
  DeletePTransitions();
  edges.clear();
  pedges.clear();
  titles.clear();
  Title::counter = 0;  // Gesamttitelzaehler nullen
  for(int r = 0; r < count;r++) {
    Verbindungen[r]->Write("Close");
    write(Verbindungen[r]->pipe1[1], "Close", 32);
    delete Verbindungen[r];
    Verbindungen[r] = (Socket *) NULL;
  }
  count = 0;
  summe.tv_sec = 0;
  summe.tv_usec = 0;
  protokoll = "";
}  // Net::DeleteAll

void
Net::DeleteBlocks(void)
{
  String n;
  while(blocks.length() != 0) {
    n = blocks.contents(blocks.first())->Name();
    delete blocks[n];
    blocks.del(n);
  }
}  // Net::DeleteBlocks

void
Net::DeleteBoxes(void)
{
  String n;
  while(boxes.length() != 0) {
    n = boxes.contents(boxes.first())->Name();
    delete boxes[n];
    boxes.del(n);
  }
}  // Net::DeleteBoxes

void
Net::DeleteBox(String &n)
{
  for(Pix i = transitions.first(); i; transitions.next(i)) {
    transitions.contents(i)->DeleteBox(n);
  }
  for(Pix i = places.first(); i; places.next(i)) {
    places.contents(i)->DeleteBox(n);
  }
  for(Pix i = ptransitions.first(); i; ptransitions.next(i)) {
    ptransitions.contents(i)->DeleteBox(n);
  }
  for(Pix i = blocks.first(); i; blocks.next(i)) {
    blocks.contents(i)->DeleteBox(n);
  }
  delete boxes[n];
  boxes.del(n);
}  // Net::DeleteBox

void 
Net::DeletePlaces(void)
{
  if(places.length() != 0)
    places.contents(places.first())->ResetMaxIdTo(1);
//  for (Pix i = places.first(); i; places.next(i))
//    delete places.contents(i);
  places.clear();
}  // Net::DeletePlaces

void
Net::RemoveEintrag(String &n)
{
  if(edges.contains(n))
    edges.del(n);
}  // Net::RemoveEintrag

void
Net::RemovePEintrag(String &n)
{
  if(pedges.contains(n))
    pedges.del(n);
}  // Net::RemovePEintrag

void 
Net::DeleteTransitions(void)
{
  if(transitions.length() != 0)
    transitions.contents(transitions.first())->ResetMaxIdTo(1);
//  for (Pix i = transitions.first(); i; transitions.next(i))
//    delete transitions.contents(i);
  transitions.clear();
}  // DeleteTransitions

void 
Net::DeletePTransitions(void)
{
  if(ptransitions.length() != 0)
    ptransitions.contents(ptransitions.first())->ResetMaxIdTo(1);
//  for (Pix i = ptransitions.first(); i; ptransitions.next(i)) {
//    delete ptransitions.contents(i);
//  }
  ptransitions.clear();
}  // DeletePTransitions

void 
Net::DeleteFileOrder(void)
{
  for (Pix i = fileorder.first(); i; fileorder.next(i))
    delete fileorder.contents(i);
  fileorder.clear();
}  // DeleteFileOrder

//---------------------------------------------------
//-------- functions for asynchronous links ---------
//---------------------------------------------------

boolean
Net::addLink(String l_name, String l_type)
{
  if (!declared_links.contains(l_name))
      {
        VHSet<Val> new_type = *(new VHSet<Val>(10));
	
	if (l_type != "")
           {
             stringToType(l_type,new_type);
             if (new_type.empty()) return false;
           };

        Link *new_link = new Link(l_name, l_type, new_type);
        declared_links[l_name] = *new_link;
        return true;
      };
   return false;
} // Net::addLink


// clear set of declared asynchronous links
void
Net::clearDeclaredLinks()
{
  declared_links.clear();
}


void
Net::stringToType (String s, VHSet<Val> &type)
{
	
  type.clear();
  // net->ChangeTyp(name.data(), s.data());
  String sv = s.before("X");
  if (sv != "") {
  //  type_string = s;
    VHSet<Val> s1(20), s2(20), temp(20);
    StringToSet(sv, s1);

    String av = s.after("X");
    for(Pix i = s1.first(); i; s1.next(i)) {
      String te = s;
      int help = 0;
      while(te != "") {
        te = te.after("X");
        help++;
      }
      Tupel temp(help);
      temp.add(s1(i));
      add(temp,av,type);
    }
  }  // if (sv != "")
  else {
//    if (s.contains("{")) 
//      type_string = s;
//    else
//      type_string = "{" + s + "}";
    StringToSet(s, type);
  }  // else

} // Net::stringToType


// StringToSet: copied from HLPlace.cc	
void
Net::StringToSet(String s, SET<Val> &set) const
  // Konvertiert String nach Set
{
  set.clear();
  s = s.gsub("{", "");
  s = s.gsub("}", "");
  String sv = s.before(",");
  while (sv != "") {
    StringRangeAppendSet(sv, set);
    s = s.after(",");
    sv = s.before(",");
  }  // while
  StringRangeAppendSet(s, set);
}  // Net::StringToSet

void
Net::StringRangeAppendSet(String range, SET<Val> &set) const
  // Fgt an die Menge set einen Wert an, der aus dem String range
  // geewonnen wird. range kann aber auch eine Bereich start..ende sein
  // In diesem Fall werden alle Werte von start bis end zur Menge
  // zugefgt.
{
  String lower = range.before("..");
  if (lower != "") {
    Val anfang = StringToVal(lower);
    String upper = range.after("..");
    Val ende = StringToVal(upper);
    if (anfang.Type() != INTEGER || ende.Type() != INTEGER) {
      Error("Lower and upper bounds must be Integers");
      return ;
    }  // if
    if (ende < anfang) { 
      Val tmp = anfang;
      anfang = ende;
      ende = tmp;
    }  // if
    for (int p = anfang.asInt(); p <= ende.asInt(); p++) {
      set.add(p);
    }
  }  // if
  else {
    set.add(StringToVal(range));
  }
}  // Net::StringRangeAppendSet


void
Net::add(Tupel va, String s, SET<Val> &type)
 // rekursive Methode, um alle Elemente der n-dimensionalen Menge zu bilden
{
  Tupel temp = va;
  String sv = s.before("X");
  VHSet<Val> s1(20);
  if(sv != "") {
    StringToSet(sv, s1);
    String av = s.after("X");
    for(Pix i = s1.first(); i; s1.next(i)) {
      Tupel temp2(temp);
      temp2.add(s1(i));
      add(temp2,av,type);
    }
  }
  else {
    StringToSet(s, s1);
    for(Pix j = s1.first(); j; s1.next(j)) {
      Tupel temp2(temp);
      temp2.add(s1(j));
      type.add(temp2);
    } // for j "uber alle Elemente der letzten Menge
  }  // else
}  // Net::add

// correct_link_name: check if the name contains only alpha or digit or '_'
boolean
Net::correct_link_name(String s)
{
  char t;
  
  for (unsigned int i = 0; i<s.length(); i++)
    {
      t = s[i];
      if( !(    ((t <= int('9')) && (t >= int('0')))
             || ((t <= int('Z')) && (t >= int('A')))
             || ((t <= int('z')) && (t >= int('a')))
             || (t == int('_'))))
         return false;
    };
  return true;
}

void
Net::ReadLink(istream &f)
{
//  Link *l = (Link *) NULL;
//  String *l_name = new String("");
//  String *l_type = new String("");
//  VHSet<Val> *set_of_val = new VHSet<Val>(10);

  String l_name = "";
  String l_type = "";
  
  char c;
  readline(f, l_name, '"');
  readline(f, l_type, '"');
  f>>c;	// read '\n'
  
  if (correct_link_name(l_name))
      addLink(l_name,l_type);

} //Net::ReadLink(istream &f)


void
Net::NewFileOrder(const String &sn, const String &ln,
  int of, int s, int e, int l)
  // Mit den uebergebenen Angaben wird ein neuer Abschnitt aufgenommen.
  // Der Index innerhalb von fileorder ist durch das Kuerzel 'sn' gegeben.
{
  NetFileOrder *nfo = new NetFileOrder(sn, ln, of, s, e, l);

  fileorder[nfo->ShortName()] = nfo;
}  // Net::NewFileOrder

int
Net::ExistsFileOrder(const String &sn)
  // Der uebergebene String wird als Kuerzel betrachtet und in der
  // Menge der fileorder-Elemente gesucht. Falls das Kuerzel bekannt
  // ist, wird 1 geliefert, 0 sonst.
{
  return (fileorder.contains(sn));
}  // Net::ExistsFileOrder

void
Net::Write(ostream &f) const
  // Ausgabe der Netzdatei in den Ausgabe-Stream
{
  Pix i;

     // Header schreiben
  f << "PEP" << endl;
  f << Format() << endl;
  if (net_type_switch == 0)
    f << "FORMAT_N" << endl;
  else
    f << "FORMAT_N2" << endl;
  f << "%%% Net output from Net?LEdit" << endl;
  if (NameP != "NULL")
    f << "%DFONT_name_P " << NameP << endl;
  if (NameT != "NULL")
    f << "%DFONT_name_T " << NameT << endl;
  if (MeaningP != "NULL")
    f << "%DFONT_meaning_P " << MeaningP << endl;
  if (MeaningT != "NULL")
    f << "%DFONT_meaning_T " << MeaningT << endl;
  if (Comments != "NULL")
    f << "%DFONT_comments " << Comments << endl;
  if (AnnotationsA != "NULL")
    f << "%DFONT_annotations_A " << AnnotationsA << endl;
  if (ActionTermT != "NULL")
    f << "%DFONT_actionterm_T " << ActionTermT << endl;
  if (ValueTermT != "NULL")
    f << "%DFONT_valueterm_T " << ValueTermT << endl;
  if (TypeP != "NULL")
    f << "%DFONT_type_P " << TypeP << endl;
    // verschiedenen Sektionen schreiben
  if (net_type_switch == 1) {
    f << "% --------- Blocks ---------" << endl;
    f << "BL" << endl;
    VHSet<String> Blocks;
    VHSet<String> noch, schon;
    for (i = blocks.first(); i; blocks.next(i)) {
      Blocks = blocks.contents(i)->BlockKeySet();
      if(Blocks.empty()) {
        f << *blocks.contents(i);
        schon.add(blocks.contents(i)->Name());
      } else {
        int ok = 0;
        for(Pix j = Blocks.first(); j; Blocks.next(j)) {
          if(!schon.contains(Blocks(j))) {
            ok = 1;
          }
        }
        if(ok == 0) {
          f << *blocks.contents(i);
          schon.add(blocks.contents(i)->Name());
        } else {
          noch.add(blocks.contents(i)->Name());
        }
      }
    }
    while(!noch.empty()) {
      for (i = blocks.first(); i; blocks.next(i)) {
        Blocks = blocks.contents(i)->BlockKeySet();
        if(!schon.contains(blocks.contents(i)->Name())) {
          int ok = 0;
          for(Pix j = Blocks.first(); j; Blocks.next(j)) {
            if(!schon.contains(Blocks(j))) {
              ok = 1;
            }
          }
          if(ok == 0) {
            f << *blocks.contents(i);
            schon.add(blocks.contents(i)->Name());
            if(noch.contains(blocks.contents(i)->Name()))
              noch.del(blocks.contents(i)->Name());
          }
        }
      }
    }
  }  // if neuer Netztyp 

  if (!declared_links.empty())
  {
    f << "% --------- Link symbols  ---------" << endl;
    f << "LS" << endl;
    for (i=declared_links.first(); i ;declared_links.next(i))
	{
      Link ls = declared_links.contents(i);
      if (!(ls==EmptyLink))
              f << ls;
	};
  }; // if (!declared_links.empty())

  f << "% --------- Places ---------" << endl;
  f << "PL" << endl;
  for (i = places.first(); i; places.next(i))
    f << *places.contents(i);

  f << "% --------- Transitions ---------" << endl;
  f << "TR" << endl;
  for (i = transitions.first(); i; transitions.next(i))
    f << *transitions.contents(i);

  if (net_type_switch == 1) {
    f << "% --------- Phantom Transitions ---------" << endl;
    f << "PTR" << endl;
    for (i = ptransitions.first(); i; ptransitions.next(i))
      f << *ptransitions.contents(i);
  }  // if neuer Netztyp

  f << "% --------- Transition -> Place Arcs ---------" << endl;
  f << "TP" << endl;
  for (i = transitions.first(); i; transitions.next(i)) {
    const Map<String, Edge *> &e= transitions.contents(i)->OutputEdges();
    for (Pix j = e.first(); j; e.next(j))
      f << *e.contents(j);
  }  // for i ueber die Transitionen
  f << "% --------- Place -> Transition Arcs ---------" << endl;
  f << "PT" << endl;
  for (i = places.first(); i; places.next(i)) {
    const Map<String, Edge *> &e= places.contents(i)->OutputEdges();
    for (Pix j = e.first(); j; e.next(j))
      f << *e.contents(j);
  }  // for i ueber die Stellen

  if (net_type_switch == 1) {
    f << "% --------- Transition -> Place Phantom Arcs ---------" << endl;
    f << "PTP" << endl;
    for (i = ptransitions.first(); i; ptransitions.next(i)) {
      const Map<String, Edge *> &e= ptransitions.contents(i)->OutputPEdges();
      for (Pix j = e.first(); j; e.next(j))
        f << *e.contents(j);
    }  // for i ueber die Phantomtransitionen
    f << "% --------- Place -> Transition Phantom Arcs ---------" << endl;
    f << "PPT" << endl;
    for (i = places.first(); i; places.next(i)) {
      const Map<String, Edge *> &e= places.contents(i)->OutputPEdges();
      for (Pix j = e.first(); j; e.next(j))
        f << *e.contents(j);
    }  // for i ueber die Stellen
  }  // if neuer Netztyp

  if (!titles.empty()) {
    f << "% --------- Comments  ---------" << endl;
    f << "TX" << endl;
    for (i = titles.first(); i; titles.next(i)) {
      f << 'N' << titles.contents(i).x << '@' << titles.contents(i).y;
      f << '"' << titles.contents(i).s << '"';
      if (titles.contents(i).font != "NULL")
        f << " %DFont_Comment " << titles.contents(i).font << endl;
      else
        f << endl;
    }  // for i ueber die Titel
  }  // if Titel vorhanden	
}  // Net::Write

void 
Net::EatComment(istream &f)
  // Whitespaces, evtl. vorhandene Kommentare und weiter WS ueberlesen
{
  char c;
  String line;
  String first_part_of_line;
  String last_part_of_line;

  f >> ws;
  while (f.get(c) && c == '%') {
    //while (f.get(c) && c != '\n') {
    //  cerr << c;
    //}
    readline(f, line);
    int start = 0;
    String delis = " \t";
    char found_del = ' ';
    first_part_of_line = line.StrTok(delis, start, found_del);
    delis = " \t";
    last_part_of_line = line.StrTok(delis, start, found_del);
    if(first_part_of_line == "DFONT_name_P") {
      NameP = last_part_of_line;
    }
    else if(first_part_of_line == "DFONT_name_T") {
      NameT = last_part_of_line;
    }
    else if(first_part_of_line == "DFONT_meaning_P") {
      MeaningP = last_part_of_line;
    }
    else if(first_part_of_line == "DFONT_meaning_T") {
      MeaningT = last_part_of_line;
    }
    else if(first_part_of_line == "DFONT_comments") {
      Comments = last_part_of_line;
    }
    else if(first_part_of_line == "DFONT_annotations_A") {
      AnnotationsA = last_part_of_line;
    }
    else if(first_part_of_line == "DFONT_action_term_T") {
      ActionTermT = last_part_of_line;
    }
    else if(first_part_of_line == "DFONT_valueterm_T") {
      ValueTermT = last_part_of_line;
    }
    else if(first_part_of_line == "DFONT_type_P") {
      TypeP = last_part_of_line;
    }
    f >> ws;
  }
  f.putback(c);
  f >> ws;
}  // Net::EatComment

int
Net::ReadHeader(istream &f)
{
  String header;
  int ret = 0;

  f >> header;
  if (header != "PEP")  // Fehlerfall
    return 0;
  EatComment(f);
  f >> header;
  if (!FormatIsOk(header))  // Fehlerfall
    return 0;
  EatComment(f);
  f >> header;
  if (header == "FORMAT_N") {  // Ok, altes Format
    net_type_switch = 0;
    ret = 1;
  }  // if altes Format ohne Bloecke
  if (header == "FORMAT_N2") {  // Fehlerfall
    net_type_switch = 1;
    ret = 1;
  }  // if neues Format mit Bloecken
  return ret;
}  // Net::ReadHeader

void
Net::ReadBlock(istream &f)
{
  Block *b = AllocBlock();

  f >> *b;
  blocks[b->Name()] = b;
}  // Net::ReadBlock

void
Net::ReadBox(istream &f)
{
  Box *b = AllocBox();

  f >> *b;
  boxes[b->Name()] = b;
}  // Net::ReadBlock

void
Net::ReadPlace(istream &f)
{
  Place *p = AllocPlace();

  f >> *p;
  places[p->Name()] = p;
}  // Net::ReadPlace

void
Net::ReadTransition(istream &f)
{
  Transition *t = AllocTransition();

  f >> *t;
  transitions[t->Name()] = t;
}  // Net::ReadTransition

void
Net::ReadPTransition(istream &f)
{
  PTransition *pt = AllocPTransition();

  f >> *pt;
  ptransitions[pt->Name()] = pt;
}  // Net::ReadPTransition

void
Net::ReadEdge(istream &f)
{
  Edge *e = AllocEdge();

  f >> *e;
  edges["A" + IntToString(e->Id())] = e;
}  // Net::ReadEdge

void
Net::ReadPEdge(istream &f)
{
  Edge *e = AllocPEdge();

  e->SetPhantomEdge();
  f >> *e;
  pedges["PA" + IntToString(e->Id())] = e;
}  // Net::ReadPEdge

void
Net::ReadTitle(istream &f)
{
  Title t;
  String line, first_part_of_line, last_part_of_line;
  char c;

  f >> c >> t.x >> c >> t.y >> c;
  readline(f, t.s, '"');
  readline(f, line, '\n');
  int start = 0;
  String delis = " \t";
  char found_del = ' ';
  first_part_of_line = line.StrTok(delis, start, found_del);
  delis = "";
  last_part_of_line = line.StrTok(delis, start, found_del);
  if (first_part_of_line == "%DFont_Comment") {
    t.font = last_part_of_line;
  }
  titles[Title::counter++] = t;
}  // Net::ReadTitle

void
Net::ReadBlockDefaults(istream &f)
{
  char dum;
  int size;

  int c = f.peek();
  while (c != EOF) {
    switch (c) {
      case 'n':
        f >> dum >> rx_block_name >> dum >> ry_block_name;
        break;
      case 'a':
        f >> dum >> rx_block_mean >> dum >> ry_block_mean;
        break;
      case 't':
        f >> dum >> block_thickness;
        break;
      case 's':
        f >> dum >> size;
        break;
      default:
        cerr << "Unknown block option: " << c << endl;
        f >> c;
        break;
    }  // switch ueber c
    c = f.peek();
  }  // while c nicht Dateiende
}  // Net::ReadBlockDefaults

void
Net::ReadPlaceDefaults(istream &f)
{
  char dum;
  int size;

  int c = f.peek();
  while (c != EOF) {
    switch (c) {
      case 'n':
        f >> dum >> rx_place_name >> dum >> ry_place_name;
        break;
      case 'a':
        f >> dum >> rx_place_mean >> dum >> ry_place_mean;
        break;
      case 't':
        f >> dum >> place_thickness;
        break;
      case 's':
        f >> dum >> size;
        break;
      default:
        cerr << "Unknown place option: " << c << endl;
        f >> c;
        break;
    }  // switch ueber c
    c = f.peek();
  }  // while c nicht Dateiende
  // f.get(dum);
}  // Net::ReadPlaceDefaults

void
Net::ReadTransitionDefaults(istream &f)
{
  char dum;
  int size;

  int c = f.peek();
  while (c != EOF) {
    switch (c) {
      case 'n':
        f >> dum >> rx_trans_name >> dum >> ry_trans_name;
        break;
      case 'a':
        f >> dum >> rx_trans_mean >> dum >> ry_trans_mean;
        break;
      case 't':
        f >> dum >> transition_thickness;
        break;
      case 's':
        f >> dum >> size;
        break;
      case 'E':
        f >> dum >> def_earliest;
        defaultTrans = 0;
        break;
      case 'L':
        f >> dum >> def_latest;
        defaultTrans = 0;
        break;
      default:
        cerr << "Unknown Transition option: " << c << endl;
        f >> dum;
        break;
    }  // switch ueber c
    c = f.peek();
  }  // while c nicht Dateiende
  // f.get(dum);
}  // Net::ReadTransitionDefaults

void
Net::ReadEdgeDefaults(istream &f)
{
  char dum;
  int weight;

  int c = f.peek();
  while (c!=EOF) {
    switch (c) {
      case 'n':
        f >> dum >> rx_edge_weight >> dum >> ry_edge_weight;
        break;
      case 't':
        f >> dum >> edge_thickness;
        break;
      case 'w':
        f >> dum >> weight;
        break;
      default:
        cerr << "Unknown edge option: " << c << endl;
        f >> dum;
        break;
    }  // switch ueber c
    c = f.peek();
  }  // while c nicht Dateiende
  // f.get(dum);
}  // Net::ReadEdgeDefaults

void
Net::WriteBoxes(ostream &f) const
{
  f << "PEP" << endl;
  f << "Boxes_HL" << endl;
  f << "Format_B1" << endl;
  for(Pix i = boxes.first(); i; boxes.next(i)) {
    f << *boxes.contents(i);
  }
}

void
Net::ReadBoxes(istream &f)
{
  String line;
  int the_end = 0;

  EatComment(f);
  if (!ReadBoxHeader(f)) {
    cerr << "error: wrong fileformat (boxheader)" << endl;
    return;
  }  // if ung"ultiger BoxHeader
  while (!the_end) {
    EatComment(f);
    readline(f, line);
    istrstream sf((const char*)line);
    ReadBox(sf);
    EatComment(f);  // wichtig f"ur das Erkennen des Dateiendes
    the_end = f.eof();
  }
  for(Pix i = boxes.first(); i; boxes.next(i)) {
    boxes.contents(i)->SetVisibility(0);
  }
}

int
Net::ReadBoxHeader(istream &f)
{
  String header;
  int ret = 1;

  f >> header;
  if (header != "PEP")  // Fehlerfall
    return 0;
  EatComment(f);
  f >> header;
  if (header != "Boxes_HL")  // Fehlerfall
    return 0;
  EatComment(f);
  f >> header;
  if (header != "Format_B1")
    return 0;
  EatComment(f);    
  return ret;
}

void
Net::Read(istream &f)
{
  int section_number = -1;    // keinen aktuellen Abschnitt
  int the_end = 0;            // Ende der Einleserei
  String line;                // jeweils die aktuelle Zeile
  String first_part_of_line;  // der erste Teil einer Zeile, evtl. Sektionsname
  String last_part_of_line;   // der zweite Teil einer Zeile
  int maxid;                  // Zwischenspeicher fuer die Id-Vergabe
  Pix i;                      // Laufvar. f"ur Stellen, Transitionen, ...

    // Defaultwerte setzen
  rx_block_name = 20;
  ry_block_name = -45;
  rx_block_mean = 20;
  ry_block_mean = -20;
  rx_place_name = -12;
  ry_place_name = -6;
  rx_place_mean = 20;
  ry_place_mean = -16;
  rx_trans_name = -12;
  ry_trans_name = -6;
  rx_trans_mean = 20;
  ry_trans_mean = -18;
  rx_edge_weight = 0;
  ry_edge_weight = 10;
    // Zustand der Netzreihenfolgeobjekte auf nicht verwendet setzen
  for (i = fileorder.first(); i; fileorder.next(i))
    fileorder.contents(i)->ResetUsedFlag();

    // Kopf des Netzes einlesen
  EatComment(f);
  if (!ReadHeader(f)) {
    cerr << "error: wrong fileformat (header)" << endl;
    return;
  }  // if ung"ultige Header

  while (!the_end) {
    EatComment(f);
    readline(f, line);
      // try to find the first Part, eventually it's a section name
    int start = 0;
    String delis = " \t";
    char found_del = ' ';
    first_part_of_line = line.StrTok(delis, start, found_del);
    if (ExistsFileOrder(first_part_of_line)) {
      if ((*fileorder[first_part_of_line]).EarliestNum() > section_number) {
        cerr << "error: section too early! "
          << (*fileorder[first_part_of_line]).EarliestNum()
          << " > " << section_number << endl;
      }  // if Abschnitt darf jetzt noch nicht auftauchen
      if (section_number > (*fileorder[first_part_of_line]).LatestNum()) {
        cerr << "error: section too late! "
          << (*fileorder[first_part_of_line]).LatestNum()
          << " < " << section_number << endl;
      }  // if Abschnitt darf jetzt nicht mehr auftauchen
        // einige Abschnitte m"ussen nachbearbeitet werden
      switch (section_number) {
        case 4:  // Bl"ocke
            // Suche die h"ochste Id und trage Einen mehr als MaxId ein.
          maxid = 0;
          for (i = blocks.first(); i; blocks.next(i)) {
            if (maxid < blocks.contents(i)->Id())
              maxid = blocks.contents(i)->Id();
          }  // for i ueber die Bloecke
          if (blocks.length() != 0)
            blocks.contents(blocks.first())->SetMaxId(maxid + 1);
          break;
        case 6:  // Stellen
            // Suche die h"ochste Id und trage Einen mehr als MaxId ein.
          maxid = 0;
          for (i = places.first(); i; places.next(i)) {
            if (maxid < places.contents(i)->Id())
              maxid = places.contents(i)->Id();
          }  // for i ueber die Stellen
          if (places.length() != 0)
            places.contents(places.first())->SetMaxId(maxid + 1);
          // Place::SetMaxId(maxid + 1);
          break;
        case 7:  // Transitionen
            // Suche die h"ochste Id und trage Einen mehr als MaxId ein.
          maxid = 0;
          for (i = transitions.first(); i; transitions.next(i)) {
            if (maxid < transitions.contents(i)->Id())
              maxid = transitions.contents(i)->Id();
          }  // for i ueber die Transitionen
          if (transitions.length() != 0)
            transitions.contents(transitions.first())->SetMaxId(maxid + 1);
          // Transition::SetMaxId(maxid + 1);
          break;
        case 8:  // Phantomtransitionen
            // Suche die h"ochste Id und trage Einen mehr als MaxId ein.
          maxid = 0;
          for (i = ptransitions.first(); i; ptransitions.next(i)) {
            if (maxid < ptransitions.contents(i)->Id())
              maxid = ptransitions.contents(i)->Id();
          }  // for i ueber die PTransitionen
          if (ptransitions.length() != 0)
            ptransitions.contents(ptransitions.first())->SetMaxId(maxid + 1);
          // PTransition::SetMaxId(maxid + 1);
          break;
        default:  // der Rest ist egal
          break;
      }  // switch "uber den Abschnitt
      section_number = (*fileorder[first_part_of_line]).SectionNum();
      (*fileorder[first_part_of_line]).SetUsedFlag();
        // andere Abschnitte sind in einer Zeile: DPL s7...
      delis = "";
      last_part_of_line = line.StrTok(delis, start, found_del);
      istrstream sf((const char*)last_part_of_line);
      switch (section_number) {
        case 0:  // Defaults f"ur Bl"ocke
          ReadBlockDefaults(sf);
          break;
        case 1:  // Defaults f"ur Stellen
          ReadPlaceDefaults(sf);
          break;
        case 2:  // Defaults f"ur Transitionen
          ReadTransitionDefaults(sf);
          break;
        case 3:  // Defaults f"ur Kanten
          ReadEdgeDefaults(sf);
          break;
        default:  // der Rest ist egal
          break;
      }  // switch "uber den Abschnitt
    }  // if Abschnittsk"urzel erkannt
    else {
      if (section_number == -1) {
        cerr << "error: wrong fileformat (body), unkown section_number (-1)"
          << endl;
        return;
      }  // if Status auf Suchen
      istrstream sf((const char*)line);
      switch (section_number) {
        case 4:  // Bl"ocke
          ReadBlock(sf);
          break;
        case 5:  // Link symbols
          ReadLink(sf);
          break;
        case 6:  // Stellen
          ReadPlace(sf);
          break;
        case 7:  // Transitionen
          ReadTransition(sf);
          break;
        case 8:  // Phantomtransitionen
          ReadPTransition(sf);
          break;
        case 9:  // Kanten Transition -> Stelle
          ReadEdge(sf);
          break;
        case 10:  // Kanten Stelle -> Transition
          ReadEdge(sf);
          break;
        case 11:  // Phantomkanten Phantomtransition -> Stelle
          ReadPEdge(sf);
          break;
        case 12:  // Phantomkanten Stelle -> Phantomtransition
          ReadPEdge(sf);
          break;
        case 13:  // Texte
          ReadTitle(sf);
          break;
        default:
          cerr << "internal error: unknown section number ("
            << section_number << ")" << endl;
      }  // switch Abschnitt
    }  // else, kein Abschnittsk"urzel
    EatComment(f);  // wichtig f"ur das Erkennen des Dateiendes
    the_end = f.eof();
  }  // while nicht Dateiende

    // Pr"ufen, ob alle obligaten Abschnitte aufgetaucht sind.
  for (i = fileorder.first(); i; fileorder.next(i)) {
    if (!fileorder.contents(i)->OptionFlag()
      && !fileorder.contents(i)->UsedFlag()) {
      cerr << "error: the section " << *(fileorder.contents(i))
        << " was not found in the net file" << endl;
    }  // if nicht optional und nicht aufgetaucht
  }  // for i "uber alle Abschnitte
  for (i = places.first(); i; places.next(i)) {
    if (!places.contents(i)->OutputEdges().empty() && !places.contents(i)->InputEdges().empty()) {
      places.contents(i)->setInit();
    }
  }
  Blockkonvert.clear();
  Placekonvert.clear();
  Transkonvert.clear();
  PTranskonvert.clear();
  activeTransitions.clear();
}  // Net::Read

const VHSet<String>
Net::RootBlocks()
{
  VHSet<String> ret(20);  // Menge von Blocknamen

  for (Pix i = blocks.first(); i; blocks.next(i))
    if (blocks.contents(i)->GetRootBlock())
      ret.add(blocks.key(i));
  return ret;
}  // Net::RootBlocks

VHSet<String>
Net::ActiveTransitions()
{
  if(activeTransitions.empty()) {
    for (Pix i = transitions.first(); i; transitions.next(i)) {
      if (transitions.contents(i)->CanFire())
        activeTransitions.add(transitions.key(i));
    }
  }
  return activeTransitions;
}  // Net::ActiveTransitions

void
Net::paste() {
  if(clipboard == (Net *) NULL)
    return;
  String text;
  VHSet<String> BL;
  VHSet<String> TEM;
  VHSet<String> Schnitt;
  VHSet<String> P;
  const VHMap<String, Block *> &T = clipboard->Blocks();
  for(Pix u = T.first(); u; T.next(u)) {
    TEM.add(T.key(u));
  }
  VHMap<String, Transition *> Trans((Transition *) NULL, 10);
  VHMap<String, PTransition *> PTrans((PTransition *) NULL, 10);
  VHMap<String, Place *> Stellen((Place *) NULL, 10);
  VHMap<String, Block *> Blocks((Block *) NULL, 10);
  VHMap<String, Box *>   Boxes((Box *) NULL, 10);
  Edge *e;
  const VHMap<String, Box *> &bo = clipboard->Boxes();
  for(Pix i = bo.first(); i; bo.next(i)) {
    text = NewBox();
    Boxes[text] = clipboard->GetBoxFromName(bo.key(i));
    *(boxes[text]) = *(clipboard->GetBoxFromName(bo.key(i)));
    SEL += text + ",";
  }
  clipboard->RecalcRootBlocks();
  VHSet<String> bl = clipboard->RootBlocks();
  for(Pix i = bl.first(); i; bl.next(i)) {
    text = NewBlock();
    Blocks[text] = clipboard->GetBlockFromName(bl(i));
    SEL += text + ",";
  }
  VHMap<String, Block *> TE = Blocks;
  for(Pix i = TE.first(); i; TE.next(i)) {
    *blocks[TE.key(i)] = *TE.contents(i);
    pasteBlock(&Trans, &Stellen, &Blocks, &PTrans, &Boxes, TE.contents(i)->Name(), TE.key(i));
    for(Pix k = Boxes.first(); k; Boxes.next(k)) {
      if(Boxes.contents(k)->Name() == clipboard->blocks[TE.contents(i)->Name()]->BoxName()) {
        blocks[TE.key(i)]->setBox(Boxes.key(k), GetBoxFromName(Boxes.key(k)));
      }
    }
  }
  for(Pix j = Boxes.first(); j; Boxes.next(j)) {
    VHSet<String> BL = boxes[Boxes.key(j)]->BlockKeySet();
    for(Pix m = BL.first(); m; BL.next(m)) {
      boxes[Boxes.key(j)]->RemoveBlock(BL(m));
      for(Pix v = Blocks.first(); v; Blocks.next(v)) {
        if(Blocks.contents(v)->Name() == BL(m)) {
          boxes[Boxes.key(j)]->AddBlock(Blocks.key(v), GetBlockFromName(Blocks.key(v)));
          break;
        }
      }
    }
  }
  for(Pix i = clipboard->transitions.first(); i; clipboard->transitions.next(i)) {
    BL = clipboard->transitions.contents(i)->BlockKeySet();
    Schnitt = TEM;
    Schnitt &= BL;
    if(Schnitt.empty()) {
      boolean schon = true;
      for(Pix no = Trans.first(); no; Trans.next(no)) {
        if(Trans.contents(no)->Name() == clipboard->transitions.contents(i)->Name()) {
          schon = false;
          break;
        }
      }
      if(schon) {
        text = NewTransition();
        Trans[text] = clipboard->transitions.contents(i);
        *transitions[text] = *clipboard->transitions.contents(i);
        SEL += text + ",";
        for(Pix k = Boxes.first(); k; Boxes.next(k)) {
          if(Boxes.contents(k)->Name() == clipboard->transitions.contents(i)->BoxName()) {
            transitions[text]->setBox(Boxes.key(k), GetBoxFromName(Boxes.key(k)));
          }
        }
      }
    }
  }
  for(Pix i = clipboard->ptransitions.first(); i; clipboard->ptransitions.next(i)) {
    BL = clipboard->ptransitions.contents(i)->BlockKeySet();
    Schnitt = TEM;
    Schnitt &= BL;
    if(Schnitt.empty()) {
      boolean schon = true;
      for(Pix no = PTrans.first(); no; PTrans.next(no)) {
        if(PTrans.contents(no)->Name() == clipboard->ptransitions.contents(i)->Name()) {
          schon = false;
          break;
        }
      }
      if(schon) {
        text = NewPTransition();
        PTrans[text] = clipboard->ptransitions.contents(i);
        *ptransitions[text] = *clipboard->ptransitions.contents(i);
        SEL += text + ",";
        for(Pix k = Boxes.first(); k; Boxes.next(k)) {
          if(Boxes.contents(k)->Name() == clipboard->ptransitions.contents(i)->BoxName()) {
            ptransitions[text]->setBox(Boxes.key(k), GetBoxFromName(Boxes.key(k)));
          }
        }
      }
    }
  }
  for(Pix i = clipboard->places.first(); i; clipboard->places.next(i)) {
    BL = clipboard->places.contents(i)->BlockKeySet();
    Schnitt = TEM;
    Schnitt &= BL;
    if(Schnitt.empty()) {
      boolean schon = true;
      for(Pix no = Stellen.first(); no; Stellen.next(no)) {
        if(Stellen.contents(no)->Name() == clipboard->places.contents(i)->Name()) {
          schon = false;
          break;
        }
      }
      if(schon) {
        text = NewPlace();
        Stellen[text] = clipboard->places.contents(i);
        *places[text] = *clipboard->places.contents(i);
        SEL += text + ",";
        for(Pix k = Boxes.first(); k; Boxes.next(k)) {
          if(Boxes.contents(k)->Name() == clipboard->places.contents(i)->BoxName()) {
            places[text]->setBox(Boxes.key(k), GetBoxFromName(Boxes.key(k)));
          }
        }
      }
    }
  }
  clipboard->RecalcRootBlocks();
  for(Pix i = Trans.first(); i; Trans.next(i)) {
    if(transitions[Trans.key(i)]->BlockKeySet().empty()) 
      *transitions[Trans.key(i)] = *Trans.contents(i);
    P = transitions[Trans.key(i)]->Phantoms();
    for(Pix j = P.first(); j; P.next(j)) {
      for(Pix l = PTrans.first(); l; PTrans.next(l)) {
        if(PTrans.contents(l)->Name() == P(j)) {
          transitions[Trans.key(i)]->RemovePTrans(P(j));
          transitions[Trans.key(i)]->AddPTransition(PTrans.key(l), ptransitions[PTrans.key(l)]);
        }
      }
    }
    transitions[Trans.key(i)]->RemoveInEdges();
    transitions[Trans.key(i)]->RemoveOutEdges();
  }
  for(Pix i = PTrans.first(); i; PTrans.next(i)) {
    if(ptransitions[PTrans.key(i)]->BlockKeySet().empty()) 
      *ptransitions[PTrans.key(i)] = *PTrans.contents(i);
    P = ptransitions[PTrans.key(i)]->SyncTrans();
    for(Pix j = P.first(); j; P.next(j)) {
      for(Pix l = Trans.first(); l; Trans.next(l)) {
        if(Trans.contents(l)->Name() == P(j)) {
          ptransitions[PTrans.key(i)]->RemoveSyncTrans(P(j));
          ptransitions[PTrans.key(i)]->AddSyncTransition(Trans.key(l), transitions[Trans.key(l)]);
        }
      }
    }
    ptransitions[PTrans.key(i)]->RemoveInPEdges();
    ptransitions[PTrans.key(i)]->RemoveOutPEdges();
  }
  for(Pix i = Stellen.first(); i; Stellen.next(i)) {
    if(places[Stellen.key(i)]->BlockKeySet().empty())
      *places[Stellen.key(i)] = *Stellen.contents(i);
    places[Stellen.key(i)]->RemoveInEdges();
    places[Stellen.key(i)]->RemoveOutEdges();
    places[Stellen.key(i)]->RemoveInPEdges();
    places[Stellen.key(i)]->RemoveOutPEdges();
  }
  for(Pix i = clipboard->edges.first(); i; clipboard->edges.next(i)) {
    e = clipboard->edges.contents(i);
    for (Pix j = Stellen.first(); j; Stellen.next(j)) {
      for (Pix k = Trans.first(); k; Trans.next(k)) {
        if(Stellen.contents(j)->Name() == e->Src().Name()) {
          if (Trans.contents(k)->Name() == e->Dest().Name()) {
            String text = NewEdge(Stellen.key(j), Trans.key(k));
            *edges[text] = *(clipboard->FindEdge(Stellen.contents(j)->Name(), Trans.contents(k)->Name()));
          }
        } else if (Stellen.contents(j)->Name() == e->Dest().Name()) {
          if (Trans.contents(k)->Name() == e->Src().Name()) {
            String text = NewEdge(Trans.key(k), Stellen.key(j));
            *edges[text] = *(clipboard->FindEdge(Trans.contents(k)->Name(), Stellen.contents(j)->Name()));
          }
        }
      }
    }
  }
  for(Pix i = clipboard->pedges.first(); i; clipboard->pedges.next(i)) {
    e = clipboard->pedges.contents(i);
    for (Pix j = Stellen.first(); j; Stellen.next(j)) {
      for (Pix k = PTrans.first(); k; PTrans.next(k)) {
        if(Stellen.contents(j)->Name() == e->Src().Name()) {
          if (PTrans.contents(k)->Name() == e->Dest().Name()) {
            String text = NewPEdge(Stellen.key(j), PTrans.key(k));
            *pedges[text] = *(clipboard->FindPEdge(Stellen.contents(j)->Name(), PTrans.contents(k)->Name()));
          }
        } else if (Stellen.contents(j)->Name() == e->Dest().Name()) {
          if (PTrans.contents(k)->Name() == e->Src().Name()) {
            String text = NewPEdge(PTrans.key(k), Stellen.key(j));
            *pedges[text] = *(clipboard->FindPEdge(PTrans.contents(k)->Name(), Stellen.contents(j)->Name()));
          }
        }
      }
    }
  }
}  // Net::paste

void
Net::cut(int c, String kette[]) {
  String n;
  copy(c, kette);
  for(int i = 0; i < c; i++) {
    if (transitions.contains(kette[i])) {
      n = transitions[kette[i]]->BoxName();
      DeleteBox(n);
      RemoveTransition(kette[i]);
    } else if (places.contains(kette[i])) {
      n = places[kette[i]]->BoxName();
      DeleteBox(n);
      RemovePlace(kette[i]);
    } else if (ptransitions.contains(kette[i])) {
      n = ptransitions[kette[i]]->BoxName();
      DeleteBox(n);
      RemovePTransition(kette[i]);
    } else if (blocks.contains(kette[i])) {
      RemoveBlock(kette[i]);
    } else if (boxes.contains(kette[i])) {
      for(Pix j = transitions.first(); j; transitions.next(j)) {
        if(transitions.contents(j)->BoxName() == kette[i])
          RemoveTransition(transitions.key(j));
      }
      for(Pix j = places.first(); j; places.next(j)) {
        if(places.contents(j)->BoxName() == kette[i])
          RemovePlace(places.key(j));
      }
      for(Pix j = ptransitions.first(); j; ptransitions.next(j)) {
        if(ptransitions.contents(j)->BoxName() == kette[i])
          RemovePTransition(ptransitions.key(j));
      }
      for(Pix j = blocks.first(); j; blocks.next(j)) {
        if(blocks.contents(j)->BoxName() == kette[i])
          RemoveBlock(blocks.key(j));
      }
      DeleteBox(kette[i]);
    }
  }
}  // Net::cut

void
Net::copyBlock(VHMap<String, Transition *> *Trans,
               VHMap<String, Place *> *Stellen,
               VHMap<String, Block *> *Blocks,
               VHMap<String, PTransition *> *PTrans,
               VHMap<String, Box *> *Boxes,
               String fromBlock, String toBlock) {
  VHSet<String> BL = GetBlockFromName(fromBlock)->BlockKeySet();
  for(Pix i = BL.first(); i;BL.next(i)) {
    boolean schon = true;
    for(Pix no = Blocks->first(); no; Blocks->next(no)) {
      if(Blocks->contents(no)->Name() == BL(i)) {
        schon = false;
        break;
      }
    }
    if(schon) {
      String text = clipboard->NewBlock();
      (*Blocks)[text] = blocks[BL(i)];
      *(clipboard->blocks[text]) = *(*Blocks)[text];
      String bo = blocks[BL(i)]->BoxName();
      for(Pix in = Boxes->first(); in; Boxes->next(in)) {
        if(Boxes->contents(in)->Name() == bo) {
          clipboard->blocks[text]->setBox(Boxes->key(in), clipboard->GetBoxFromName(Boxes->key(in)));
          break;
        }
      }
      clipboard->blocks[toBlock]->RemoveBlock(BL(i));
      clipboard->blocks[toBlock]->AddBlock(text, clipboard->GetBlockFromName(text));
      copyBlock(Trans, Stellen, Blocks, PTrans, Boxes, BL(i), text);
    }
  }
  for(Pix i = transitions.first(); i;transitions.next(i)) {
    VHSet<String> bl = transitions.contents(i)->BlockKeySet();
    VHSet<String> temp = GetBlockFromName(fromBlock)->BlockKeySet();
    temp &= bl;
    if (bl.contains(fromBlock) && temp.empty()) {
      boolean schon = true;
      for(Pix no = Trans->first(); no; Trans->next(no)) {
        if(Trans->contents(no)->Name() == transitions.contents(i)->Name()) {
          schon = false;
          break;
        }
      }
      if(schon) {
        String text = clipboard->NewTransition();
        (*Trans)[text] = transitions.contents(i);
        *(clipboard->transitions[text]) = *(*Trans)[text];
        String bo = transitions.contents(i)->BoxName();
        for(Pix in = Boxes->first(); in; Boxes->next(in)) {
          if(Boxes->contents(in)->Name() == bo) {
            clipboard->transitions[text]->setBox(Boxes->key(in), clipboard->GetBoxFromName(Boxes->key(in)));
            break;
          }
        }
        for(Pix t = bl.first(); t;bl.next(t)) {
          clipboard->transitions[text]->RemoveBlock(bl(t));
          for(Pix c = Blocks->first(); c; Blocks->next(c)) {
            if(Blocks->contents(c)->Name() == bl(t)) {
              clipboard->transitions[text]->AddBlock(Blocks->key(c), clipboard->blocks[Blocks->key(c)]);
            }
          }
        }
        clipboard->transitions[text]->RemoveInEdges();
        clipboard->transitions[text]->RemoveOutEdges();
      }
    }
  }
  for(Pix i = places.first(); i;places.next(i)) {
    VHSet<String> bl = places.contents(i)->BlockKeySet();
    VHSet<String> temp = GetBlockFromName(fromBlock)->BlockKeySet();
    temp &= bl;
    if (bl.contains(fromBlock) && temp.empty()) {
      boolean schon = true;
      for(Pix no = Stellen->first(); no; Stellen->next(no)) {
        if(Stellen->contents(no)->Name() == places.contents(i)->Name()) {
          schon = false;
          break;
        }
      }
      if(schon) {
        String text = clipboard->NewPlace();
        (*Stellen)[text] = places.contents(i);
        *(clipboard->places[text]) = *(*Stellen)[text];
        String bo = places.contents(i)->BoxName();
        for(Pix in = Boxes->first(); in; Boxes->next(in)) {
          if(Boxes->contents(in)->Name() == bo) {
            clipboard->places[text]->setBox(Boxes->key(in), clipboard->GetBoxFromName(Boxes->key(in)));
            break;
          }
        }
        for(Pix t = bl.first(); t;bl.next(t)) {
          clipboard->places[text]->RemoveBlock(bl(t));
          for(Pix c = Blocks->first(); c; Blocks->next(c)) {
            if(Blocks->contents(c)->Name() == bl(t)) {
              clipboard->places[text]->AddBlock(Blocks->key(c), clipboard->blocks[Blocks->key(c)]);
            }
          }
        }
        clipboard->places[text]->RemoveInEdges();
        clipboard->places[text]->RemoveOutEdges();
        clipboard->places[text]->RemoveInPEdges();
        clipboard->places[text]->RemoveOutPEdges();
      }
    }
  }
  for(Pix i = ptransitions.first(); i;ptransitions.next(i)) {
    VHSet<String> bl = ptransitions.contents(i)->BlockKeySet();
    VHSet<String> temp = GetBlockFromName(fromBlock)->BlockKeySet();
    temp &= bl;
    if (bl.contains(fromBlock) && temp.empty()) {
      boolean schon = true;
      for(Pix no = PTrans->first(); no; PTrans->next(no)) {
        if(PTrans->contents(no)->Name() == ptransitions.contents(i)->Name()) {
          schon = false;
          break;
        }
      }
      if(schon) {
        String text = clipboard->NewPTransition();
        (*PTrans)[text] = ptransitions.contents(i);
        *(clipboard->ptransitions[text]) = *(*PTrans)[text];
        String bo = ptransitions.contents(i)->BoxName();
        for(Pix in = Boxes->first(); in; Boxes->next(in)) {
          if(Boxes->contents(in)->Name() == bo) {
            clipboard->ptransitions[text]->setBox(Boxes->key(in), clipboard->GetBoxFromName(Boxes->key(in)));
            break;
          }
        }
        for(Pix t = bl.first(); t;bl.next(t)) {
          clipboard->ptransitions[text]->RemoveBlock(bl(t));
          for(Pix c = Blocks->first(); c; Blocks->next(c)) {
            if(Blocks->contents(c)->Name() == bl(t)) {
              clipboard->ptransitions[text]->AddBlock(Blocks->key(c), clipboard->blocks[Blocks->key(c)]);
            }
          }
        }
        clipboard->ptransitions[text]->RemoveInPEdges();
        clipboard->ptransitions[text]->RemoveOutPEdges();
      }
    }
  }
}  // Net::copyBlock

void
Net::pasteBlock(VHMap<String, Transition *> *Trans,
                VHMap<String, Place *> *Stellen,
                VHMap<String, Block *> *Blocks,
                VHMap<String, PTransition *> *PTrans,
                VHMap<String, Box *> *Boxes,
                String fromBlock, String toBlock) {
  VHSet<String> bl = clipboard->GetBlockFromName(fromBlock)->BlockKeySet();
  for(Pix j = bl.first(); j;bl.next(j)) {
    String text = NewBlock();
    SEL += text + ",";
    (*Blocks)[text] = clipboard->GetBlockFromName(bl(j));
    *(blocks[text]) = *(*Blocks)[text];
    pasteBlock(Trans, Stellen, Blocks, PTrans, Boxes, bl(j), text);
    blocks[toBlock]->RemoveBlock(bl(j));
    blocks[toBlock]->AddBlock(text, GetBlockFromName(text));
    String bo = clipboard->blocks[bl(j)]->BoxName();
    for(Pix in = Boxes->first(); in; Boxes->next(in)) {
      if(Boxes->contents(in)->Name() == bo) {
        blocks[text]->setBox(Boxes->key(in), GetBoxFromName(Boxes->key(in)));
        break;
      }
    }
  }
  for(Pix i = clipboard->transitions.first(); i;clipboard->transitions.next(i)) {
    VHSet<String> bl = clipboard->transitions.contents(i)->BlockKeySet();
    VHSet<String> temp = clipboard->GetBlockFromName(fromBlock)->BlockKeySet();
    VHSet<String> t = bl;
    t &= temp;
    if((t.contains(fromBlock) || t.empty()) && bl.contains(fromBlock)) {
      String text = NewTransition();
      (*Trans)[text] = clipboard->transitions.contents(i);
      SEL += text + ",";
      *(transitions[text]) = *(*Trans)[text];
      String bo = clipboard->transitions.contents(i)->BoxName();
      for(Pix in = Boxes->first(); in; Boxes->next(in)) {
        if(Boxes->contents(in)->Name() == bo) {
          transitions[text]->setBox(Boxes->key(in), GetBoxFromName(Boxes->key(in)));
          break;
        }
      }
      for(Pix l = bl.first(); l;bl.next(l)) {
        transitions[text]->RemoveBlock(bl(l));
        for(Pix c = Blocks->first(); c; Blocks->next(c)) {
          if(Blocks->contents(c)->Name() == bl(l)) {
            transitions[text]->AddBlock(Blocks->key(c), blocks[Blocks->key(c)]);
          }
        }
      }
      transitions[text]->RemoveInEdges();
      transitions[text]->RemoveOutEdges();
    }
  }
  for(Pix i = clipboard->places.first(); i;clipboard->places.next(i)) {
    VHSet<String> bl = clipboard->places.contents(i)->BlockKeySet();
    VHSet<String> temp = clipboard->GetBlockFromName(fromBlock)->BlockKeySet();
    VHSet<String> t = bl;
    t &= temp;
    if(bl.contains(fromBlock) && (t.contains(fromBlock) || t.empty())) {
      String text = NewPlace();
      SEL += text + ",";
      (*Stellen)[text] = clipboard->places.contents(i);
      *(places[text]) = *(*Stellen)[text];
      String bo = clipboard->places.contents(i)->BoxName();
      for(Pix in = Boxes->first(); in; Boxes->next(in)) {
        if(Boxes->contents(in)->Name() == bo) {
          places[text]->setBox(Boxes->key(in), GetBoxFromName(Boxes->key(in)));
          break;
        }
      }
      for(Pix l = bl.first(); l;bl.next(l)) {
        places[text]->RemoveBlock(bl(l));
        for(Pix c = Blocks->first(); c; Blocks->next(c)) {
          if(Blocks->contents(c)->Name() == bl(l)) {
            places[text]->AddBlock(Blocks->key(c), blocks[Blocks->key(c)]);
            break;
          }
        }
      }
      places[text]->RemoveInEdges();
      places[text]->RemoveOutEdges();
      places[text]->RemoveInPEdges();
      places[text]->RemoveOutPEdges();
    }
  }
  for(Pix i = clipboard->ptransitions.first(); i;clipboard->ptransitions.next(i)) {
    VHSet<String> bl = clipboard->ptransitions.contents(i)->BlockKeySet();
    VHSet<String> temp = clipboard->GetBlockFromName(fromBlock)->BlockKeySet();
    VHSet<String> t = bl;
    t &= temp;
    if((t.contains(fromBlock) || t.empty()) && bl.contains(fromBlock)) {
      String text = NewPTransition();
      SEL += text + ",";
      (*PTrans)[text] = clipboard->ptransitions.contents(i);
      *(ptransitions[text]) = *(*PTrans)[text];
      String bo = clipboard->ptransitions.contents(i)->BoxName();
      for(Pix in = Boxes->first(); in; Boxes->next(in)) {
        if(Boxes->contents(in)->Name() == bo) {
          ptransitions[text]->setBox(Boxes->key(in), GetBoxFromName(Boxes->key(in)));
          break;
        }
      }
      for(Pix l = bl.first(); l;bl.next(l)) {
        ptransitions[text]->RemoveBlock(bl(l));
        for(Pix c = Blocks->first(); c; Blocks->next(c)) {
          if(Blocks->contents(c)->Name() == bl(l)) {
            ptransitions[text]->AddBlock(Blocks->key(c), blocks[Blocks->key(c)]);
          }
        }
      }
      ptransitions[text]->RemoveInPEdges();
      ptransitions[text]->RemoveOutPEdges();
    }
  }
  RecalcBlockVis();
}  // Net::pasteBlock

void
Net::AppSuffix(String Suffix, String Box) {
  for(Pix i = places.first(); i; places.next(i)) {
    if(places.contents(i)->BoxName() == Box) {
      String NM = places.contents(i)->Name();
      ChangePlaceName(places.contents(i)->Name(),
                      NM+Suffix);
    }
  }
  for(Pix i = transitions.first(); i; transitions.next(i)) {
    if(transitions.contents(i)->BoxName() == Box) {
      String NM = transitions.contents(i)->Name();
      ChangeTransName(transitions.contents(i)->Name(),
                      NM+Suffix);
    }
  }
  for(Pix i = ptransitions.first(); i; ptransitions.next(i)) {
    if(ptransitions.contents(i)->BoxName() == Box) {
      ChangePTransName(ptransitions.contents(i)->Name(),
                       ptransitions.contents(i)->Name()+Suffix);
    }
  }
}  // Net::AppSuffix

void
Net::copy(int c, String kette[]) {
  String zw;
  if (clipboard != (Net *) NULL) {
    int blocksc = 1;
    int placesc = 1;
    int transc  = 1;
    int ptransc = 1;
    int boxesc  = 1;
    if(!blocks.empty()) {
      for(Pix i = blocks.first(); i;blocks.next(i)) {
        if(blocks.contents(i)->Id() > blocksc)
          blocksc = blocks.contents(i)->Id();
      }
    }
    if(!boxes.empty()) {
      for(Pix i = boxes.first(); i; boxes.next(i)) {
        if(boxes.contents(i)->Id() > boxesc)
          boxesc = boxes.contents(i)->Id();
      }
    }
    if(!places.empty()) {
      for(Pix i = places.first(); i;places.next(i)) {
        if(places.contents(i)->Id() > placesc)
          placesc = places.contents(i)->Id();
      }
    }
    if(!transitions.empty()) {
      for(Pix i = transitions.first(); i;transitions.next(i)) {
        if(transitions.contents(i)->Id() > transc)
          transc = transitions.contents(i)->Id();
      }
    }
    if(!ptransitions.empty()) {
      for(Pix i = ptransitions.first(); i;ptransitions.next(i)) {
        if(ptransitions.contents(i)->Id() > ptransc)
          ptransc = ptransitions.contents(i)->Id();
      }
    }
    if(!blocks.empty())
      blocks.contents(blocks.first())->ResetMaxIdTo(blocksc);
    if(!boxes.empty())
      boxes.contents(boxes.first())->ResetMaxIdTo(boxesc);
    if(!places.empty())
      places.contents(places.first())->ResetMaxIdTo(placesc);
    if(!transitions.empty())
      transitions.contents(transitions.first())->ResetMaxIdTo(transc);
    if(!ptransitions.empty())
      ptransitions.contents(ptransitions.first())->ResetMaxIdTo(ptransc);
  }
  clipboard = new Net();
  VHSet<String> BL;
  VHMap<String, Transition *> Trans((Transition *) NULL, 10);
  VHMap<String, Place *> Stellen((Place *) NULL, 10);
  VHMap<String, Block *> Blocks((Block *) NULL, 10);
  VHMap<String, Box *> Boxes((Box *) NULL, 10);
  VHMap<String, PTransition *> PTrans((PTransition *) NULL, 10);
  Transition *t;
  PTransition *pt;
  String text;
  for(int i = 0; i < c; i++) {
    if (transitions.contains(kette[i])) {
      text = clipboard->NewTransition();
      Trans[text] = transitions[kette[i]];
      *clipboard->transitions[text] = *Trans[text];
      BL = clipboard->transitions[text]->BlockKeySet();
      for(Pix j = BL.first(); j; BL.next(j)) {
        clipboard->transitions[text]->RemoveBlock(BL(j));
      }
    }
    else if (places.contains(kette[i])) {
      text = clipboard->NewPlace();
      Stellen[text] = places[kette[i]];
      *clipboard->places[text] = *Stellen[text];
      BL = clipboard->places[text]->BlockKeySet();
      for(Pix j = BL.first(); j; BL.next(j)) {
        clipboard->places[text]->RemoveBlock(BL(j));
      }
    }
    else if (ptransitions.contains(kette[i])) {
      text = clipboard->NewPTransition();
      PTrans[text] = ptransitions[kette[i]];
      *clipboard->ptransitions[text] = *PTrans[text];
      BL = clipboard->ptransitions[text]->BlockKeySet();
      for(Pix j = BL.first(); j; BL.next(j)) {
        clipboard->ptransitions[text]->RemoveBlock(BL(j));
      }
    }
  }
  for(int i = 0; i < c; i++) {
    if(boxes.contains(kette[i])) {
      String text = clipboard->NewBox();
      Boxes[text] = boxes[kette[i]];
      *(clipboard->GetBoxFromName(text)) = *boxes[kette[i]];
      VHSet<String> roots = RootBlocks();
      for(Pix j = roots.first(); j; roots.next(j)) {
        if(blocks[roots(j)]->BoxName() == kette[i]) {
          boolean schon = true;
          for(Pix no = Blocks.first(); no; Blocks.next(no)) {
            if(Blocks.contents(no)->Name() == blocks[roots(j)]->Name()) {
              schon = false;
              break;
            }
          }
          if(schon) {
            zw = clipboard->NewBlock();
            Blocks[zw] = blocks[roots(j)];
            *(clipboard->GetBlockFromName(zw)) = *(blocks[roots(j)]);
            copyBlock(&Trans, &Stellen, &Blocks, &PTrans, &Boxes, blocks[roots(j)]->Name(), zw);
            VHSet<String> BL = blocks[roots(j)]->BlockKeySet();
            for(Pix m = BL.first(); m; BL.next(m)) {
              clipboard->GetBlockFromName(zw)->RemoveBlock(BL(m));
              for(Pix v = Blocks.first(); v; Blocks.next(v)) {
                if(Blocks.contents(v)->Name() == BL(m)) {
                  clipboard->GetBlockFromName(zw)->AddBlock(Blocks.key(v), clipboard->GetBlockFromName(Blocks.key(v)));
                  break;
                }
              }
            }
            for(Pix in = Boxes.first(); in; Boxes.next(in)) {
              if(Boxes.contents(in)->Name() == kette[i]) {
                clipboard->blocks[zw]->setBox(Boxes.key(in), clipboard->GetBoxFromName(Boxes.key(in)));
                break;
              }
            }
          }
        }
      }
      for(Pix j = clipboard->boxes.first(); j; clipboard->boxes.next(j)) {
        VHSet<String> BL = clipboard->boxes.contents(j)->BlockKeySet();
        for(Pix m = BL.first(); m; BL.next(m)) {
          clipboard->boxes.contents(j)->RemoveBlock(BL(m));
          for(Pix v = Blocks.first(); v; Blocks.next(v)) {
            if(Blocks.contents(v)->Name() == BL(m)) {
              clipboard->boxes.contents(j)->AddBlock(Blocks.key(v), clipboard->GetBlockFromName(Blocks.key(v)));
              break;
            }
          }
        }
      }

      for(Pix j = transitions.first(); j; transitions.next(j)) {
        if(transitions.contents(j)->BoxName() == kette[i]) {
          boolean schon = true;
          for(Pix no = Trans.first(); no; Trans.next(no)) {
            if(Trans.contents(no)->Name() == transitions.contents(j)->Name()) {
              schon = false;
              break;
            }
          }
          if(schon) {
            zw = clipboard->NewTransition();
            Trans[zw] = transitions.contents(j);
            clipboard->transitions[zw]->setBox(text, clipboard->GetBoxFromName(text));
            BL = clipboard->transitions[zw]->BlockKeySet();
            for(Pix x = BL.first(); x; BL.next(x)) {
              clipboard->transitions[zw]->RemoveBlock(BL(x));
            }
          }
        }
      }
      for(Pix j = places.first(); j; places.next(j)) {
        if(places.contents(j)->BoxName() == kette[i]) {
          boolean schon = true;
          for(Pix no = Stellen.first(); no; Stellen.next(no)) {
            if(Stellen.contents(no)->Name() == places.contents(j)->Name()) {
              schon = false;
              break;
            }
          }
          if(schon) {
            zw = clipboard->NewPlace();
            Stellen[zw] = places.contents(j);
            clipboard->places[zw]->setBox(text, clipboard->GetBoxFromName(text));
            BL = clipboard->places[zw]->BlockKeySet();
            for(Pix x = BL.first(); x; BL.next(x)) {
              clipboard->places[zw]->RemoveBlock(BL(x));
            }
          }
        }
      }
      for(Pix j = ptransitions.first(); j; ptransitions.next(j)) {
        if(ptransitions.contents(j)->BoxName() == kette[i]) {
          boolean schon = true;
          for(Pix no = PTrans.first(); no; PTrans.next(no)) {
            if(PTrans.contents(no)->Name() == ptransitions.contents(j)->Name()) {
              schon = false;
              break;
            }
          }
          if(schon) {
            zw = clipboard->NewPTransition();
            PTrans[zw] = ptransitions.contents(j);
            clipboard->ptransitions[zw]->setBox(text, clipboard->GetBoxFromName(text));
            BL = clipboard->ptransitions[zw]->BlockKeySet();
            for(Pix x = BL.first(); x; BL.next(x)) {
              clipboard->ptransitions[zw]->RemoveBlock(BL(x));
            }
          }
        }
      }
    }
  }
  for(int i = 0; i < c; i++) {
    if(blocks.contains(kette[i])) {
      String text = clipboard->NewBlock();
      Blocks[text] = blocks[kette[i]];
      *(clipboard->GetBlockFromName(text)) = *blocks[kette[i]];
      copyBlock(&Trans, &Stellen, &Blocks, &PTrans, &Boxes, kette[i], text);
      VHSet<String> BL = blocks[kette[i]]->BlockKeySet();
      for(Pix m = BL.first(); m; BL.next(m)) {
        clipboard->GetBlockFromName(text)->RemoveBlock(BL(m));
        for(Pix v = Blocks.first(); v; Blocks.next(v)) {
          if(Blocks.contents(v)->Name() == BL(m)) {
            clipboard->GetBlockFromName(text)->AddBlock(Blocks.key(v), clipboard->GetBlockFromName(Blocks.key(v)));
            break;
          }
        }
      }
    }
  }
  VHSet<String> P;
  for(Pix i = clipboard->transitions.first(); i; clipboard->transitions.next(i)) {
//    *(clipboard->transitions.contents(i)) = *Trans[(clipboard->transitions.contents(i))->Name()];
    P = clipboard->transitions.contents(i)->Phantoms();
    for(Pix j = P.first(); j; P.next(j)) {
      for(Pix l = PTrans.first(); l; PTrans.next(l)) {
        if(PTrans.contents(l)->Name() == P(j)) {
          clipboard->transitions.contents(i)->RemovePTrans(P(j));
          clipboard->transitions.contents(i)->AddPTransition(PTrans.key(l), clipboard->ptransitions[PTrans.key(l)]);
        }
      }
    }
    clipboard->transitions.contents(i)->RemoveInEdges();
    clipboard->transitions.contents(i)->RemoveOutEdges();
  }
  for(Pix i = clipboard->places.first(); i; clipboard->places.next(i)) {
//    *(clipboard->places.contents(i)) = *Stellen[(clipboard->places.contents(i))->Name()];
    clipboard->places.contents(i)->RemoveInEdges();
    clipboard->places.contents(i)->RemoveOutEdges();
    clipboard->places.contents(i)->RemoveInPEdges();
    clipboard->places.contents(i)->RemoveOutPEdges();
  }
  for(Pix i = clipboard->ptransitions.first(); i;clipboard->ptransitions.next(i)) {
//    *(clipboard->ptransitions.contents(i)) = *PTrans[(clipboard->ptransitions.contents(i))->Name()];
    P = clipboard->ptransitions.contents(i)->SyncTrans();
    for(Pix j = P.first(); j; P.next(j)) {
      for(Pix l = Trans.first(); l; Trans.next(l)) {
        if(Trans.contents(l)->Name() == P(j)) {
          clipboard->ptransitions.contents(i)->RemoveSyncTrans(P(j));
          clipboard->ptransitions.contents(i)->AddSyncTransition(Trans.key(l), clipboard->transitions[Trans.key(l)]);
        }
      }
    }
    clipboard->ptransitions.contents(i)->RemoveInPEdges();
    clipboard->ptransitions.contents(i)->RemoveOutPEdges();
    clipboard->ptransitions.contents(i)->RemoveInEdges();
    clipboard->ptransitions.contents(i)->RemoveOutEdges();
  }
  for(Pix i = transitions.first(); i; transitions.next(i)) {
    for(Pix v = Trans.first(); v; Trans.next(v)) {
      if (Trans.contents(v)->Name() == transitions.contents(i)->Name()) {
        t = transitions.contents(i);
        const Map<String, Edge *> &out_edges = t->OutputEdges();
        const Map<String, Edge *> &in_edges = t->InputEdges();
        for(Pix j = out_edges.first(); j; out_edges.next(j)) {
          Edge *e = out_edges.contents(j);
          for (Pix b = Stellen.first(); b; Stellen.next(b)) {
            if(Stellen.contents(b)->Name() == e->Dest().Name()) {
               String text = clipboard->NewEdge(Trans.key(v), Stellen.key(b));
               *clipboard->edges[text] = *FindEdge(Trans.contents(v)->Name(), Stellen.contents(b)->Name());
             }
          }
        }
        for (Pix j = in_edges.first(); j; in_edges.next(j)) {
          Edge *e = in_edges.contents(j);
          for (Pix b = Stellen.first(); b; Stellen.next(b)) {
            if (Stellen.contents(b)->Name() == e->Src().Name()) {
              String text = clipboard->NewEdge(Stellen.key(b), Trans.key(v));
              *clipboard->edges[text] = *FindEdge(Stellen.contents(b)->Name(), Trans.contents(v)->Name());
            }
          }
        }
      }
    }
  }
  for(Pix i = ptransitions.first(); i; ptransitions.next(i)) {
    for(Pix v = PTrans.first(); v; PTrans.next(v)) {
      if (PTrans.contents(v)->Name() == ptransitions.contents(i)->Name()) {
        pt = ptransitions.contents(i);
        const Map<String, Edge *> &out_edges = pt->OutputPEdges();
        const Map<String, Edge *> &in_edges = pt->InputPEdges();
        for(Pix j = out_edges.first(); j; out_edges.next(j)) {
          Edge *e = out_edges.contents(j);
          for (Pix b = Stellen.first(); b; Stellen.next(b)) {
            if(Stellen.contents(b)->Name() == e->Dest().Name()) {
              String text = clipboard->NewPEdge(PTrans.key(v), Stellen.key(b));
              *clipboard->pedges[text] = *FindPEdge(PTrans.contents(v)->Name(), Stellen.contents(b)->Name());
            }
          }
        }
        for (Pix j = in_edges.first(); j; in_edges.next(j)) {
          Edge *e = in_edges.contents(j);
          for (Pix b = Stellen.first(); b; Stellen.next(b)) {
            if (Stellen.contents(b)->Name() == e->Src().Name()) {
              String text = clipboard->NewPEdge(Stellen.key(b), PTrans.key(v));
              *clipboard->pedges[text] = *FindPEdge(Stellen.contents(b)->Name(), PTrans.contents(v)->Name());
            }
          }
        }
      }
    }
  }
  const int Wert = int(DefaultLLSize / 2);
  for(Pix i = clipboard->transitions.first(); i; clipboard->transitions.next(i)) {
    clipboard->transitions.contents(i)->SetX(clipboard->transitions.contents(i)->GetX() - Wert);
    clipboard->transitions.contents(i)->SetY(clipboard->transitions.contents(i)->GetY() - Wert);
  }
  for(Pix i = clipboard->places.first(); i; clipboard->places.next(i)) {
    clipboard->places.contents(i)->SetX(clipboard->places.contents(i)->GetX() - Wert);
    clipboard->places.contents(i)->SetY(clipboard->places.contents(i)->GetY() - Wert);
  }
  for(Pix i = clipboard->ptransitions.first(); i; clipboard->ptransitions.next(i)) {
    clipboard->ptransitions.contents(i)->SetX(clipboard->ptransitions.contents(i)->GetX() - Wert);
    clipboard->ptransitions.contents(i)->SetY(clipboard->ptransitions.contents(i)->GetY() - Wert);
  }
}  // Net::copy

String
Net::TakeInBox(int c, String kette[])
{
  int maxid = 0;
  VHSet<String> bl;
  String z;
  for (Pix i = boxes.first(); i; boxes.next(i)) {
    if (maxid < boxes.contents(i)->Id())
      maxid = boxes.contents(i)->Id();
  }  // for i ueber die Boxen
  if (boxes.length() != 0)
    boxes.contents(boxes.first())->SetMaxId(maxid + 1);
  String b = NewBox();
  String h;
  Box *ptr = GetBoxFromName(b);
  for(int i = 0; i < c; i++) {
    if (transitions.contains(kette[i])) {
      if(transitions[kette[i]]->BoxName() != "") {
        z = transitions[kette[i]]->BoxName();
        if(z != b)
          DeleteBox(z);
      }
      transitions[kette[i]]->setBox(b, ptr);
      bl = transitions[kette[i]]->BlockKeySet();
      if(bl.length() != 0) {
        h = GetOldestFather(bl(bl.first()));
        blocks[h]->setBox(b, ptr);
        ptr->AddBlock(h, GetBlockFromName(h));
        TakeInBoxRek(b, h, ptr);
      }
    }
    else if (places.contains(kette[i])) {
      if(places[kette[i]]->BoxName() != "") {
        z = places[kette[i]]->BoxName();
        if(z != b)
          DeleteBox(z);
      }
      places[kette[i]]->setBox(b, ptr);
      bl = places[kette[i]]->BlockKeySet();
      if(bl.length() != 0) {
        h = GetOldestFather(bl(bl.first()));
        blocks[h]->setBox(b, ptr);
        ptr->AddBlock(h, GetBlockFromName(h));
        TakeInBoxRek(b, h, ptr);
      }
    }
    else if (ptransitions.contains(kette[i])) {
      if(ptransitions[kette[i]]->BoxName() != "") {
        z = ptransitions[kette[i]]->BoxName();
        if(z != b)
          DeleteBox(z);
      }
      ptransitions[kette[i]]->setBox(b, ptr);
      bl = ptransitions[kette[i]]->BlockKeySet();
      if(bl.length() != 0) {
        h = GetOldestFather(bl(bl.first()));
        blocks[h]->setBox(b, ptr);
        ptr->AddBlock(h, GetBlockFromName(h));
        TakeInBoxRek(b, h, ptr);
      }
    }
    else if (blocks.contains(kette[i])) {
      if(blocks[kette[i]]->BoxName() != "") {
        z = blocks[kette[i]]->BoxName();
        if(z != b)
          DeleteBox(z);
      }
      blocks[kette[i]]->setBox(b, ptr);
      h = GetOldestFather(kette[i]);
      ptr->AddBlock(h, GetBlockFromName(h));
      blocks[h]->setBox(b, ptr);
      TakeInBoxRek(b, h, ptr);
    }
  }
  return b;
}  // Net::TakeInBox

void
Net::AddToBox(int c, String kette[], String b)
{
  Box *ptr = GetBoxFromName(b);
  VHSet<String> bl;
  String h;
  for(int i = 0; i < c; i++) {
    if (transitions.contains(kette[i])) {
      transitions[kette[i]]->setBox(b, ptr);
      bl = transitions[kette[i]]->BlockKeySet();
      if(bl.length() != 0) {
        h = GetOldestFather(bl(bl.first()));
        blocks[h]->setBox(b, ptr);
        ptr->AddBlock(h, GetBlockFromName(h));
        TakeInBoxRek(b, h, ptr);
      }
    }
    else if (places.contains(kette[i])) {
      places[kette[i]]->setBox(b, ptr);
      bl = places[kette[i]]->BlockKeySet();
      if(bl.length() != 0) {
        h = GetOldestFather(bl(bl.first()));
        blocks[h]->setBox(b, ptr);
        ptr->AddBlock(h, GetBlockFromName(h));
        TakeInBoxRek(b, h, ptr);
      }
    }
    else if (ptransitions.contains(kette[i])) {
      ptransitions[kette[i]]->setBox(b, ptr);
      bl = ptransitions[kette[i]]->BlockKeySet();
      if(bl.length() != 0) {
        h = GetOldestFather(bl(bl.first()));
        blocks[h]->setBox(b, ptr);
        ptr->AddBlock(h, GetBlockFromName(h));
        TakeInBoxRek(b, h, ptr);
      }
    }
    else if (blocks.contains(kette[i])) {
      blocks[kette[i]]->setBox(b, ptr);
      h = GetOldestFather(kette[i]);
      ptr->AddBlock(h, GetBlockFromName(h));
      blocks[h]->setBox(b, ptr);
      TakeInBoxRek(b, h, ptr);
    }
  }
}  // Net::AddToBox

void
Net::TakeInBoxRek(String box, String block, Box *ptr)
{
  String z;
  for(Pix i = transitions.first(); i; transitions.next(i)) {
    if(transitions.contents(i)->BlockKeySet().contains(block)) {
      if(transitions.contents(i)->BoxName() != "") {
        z = transitions.contents(i)->BoxName();
        if(z != box)
          DeleteBox(z);
      }
      transitions.contents(i)->setBox(box, ptr);
    }
  }
  for(Pix i = ptransitions.first(); i; ptransitions.next(i)) {
    if(ptransitions.contents(i)->BlockKeySet().contains(block)) {
      if(ptransitions.contents(i)->BoxName() != "") {
        z = ptransitions.contents(i)->BoxName();
        if(z != box)
          DeleteBox(z);
      }
      ptransitions.contents(i)->setBox(box, ptr);
    }
  }
  for(Pix i = places.first(); i; places.next(i)) {
    if(places.contents(i)->BlockKeySet().contains(block)) {
      if(places.contents(i)->BoxName() != "") {
        z = places.contents(i)->BoxName();
        if(z != box)
          DeleteBox(z);
      }
      places.contents(i)->setBox(box, ptr);
    }
  }
  for(Pix i = blocks.first(); i; blocks.next(i)) {
    if(blocks.contents(i)->BlockKeySet().contains(block)) {
      if(blocks.contents(i)->BoxName() != "") {
        z = blocks.contents(i)->BoxName();
        if(z != box)
          DeleteBox(z);
      }
      blocks.contents(i)->setBox(box, ptr);
      boxes[box]->AddBlock(blocks.key(i), GetBlockFromName(blocks.key(i)));
      TakeInBoxRek(box, blocks.key(i), ptr);
    }
  }
}  // Net::TakeInBoxRek

String
Net::GetOldestFather(String &n)
{
  VHSet<String> roots = RootBlocks();
  if (roots.contains(n)) {
    return n;
  }
  VHSet<String> bl;
  for(Pix i = blocks.first(); i; blocks.next(i)) {
    if(blocks.contents(i)->BlockKeySet().contains(n)) {
      bl.add(blocks.key(i));
    }
  }
  roots &= bl;
  if(roots.length() != 1)
    cerr << "Error: Block hierarchy is not a tree" << endl;
  else
    return roots(roots.first());
  return "";
}  // Net::GetOldestFather


void
Net::loadmodul(CONST char *argv, boolean inbox, String b, boolean clear_links)
{
  if(clipboard != NULL) {
    int blocksc = 1;
    int placesc = 1;
    int transc  = 1;
    int ptransc = 1;
    if(!blocks.empty()) {
      for(Pix i = blocks.first(); i;blocks.next(i)) {
        if(blocks.contents(i)->Id() > blocksc)
          blocksc = blocks.contents(i)->Id();
      }
    }
    if(!places.empty()) {
      for(Pix i = places.first(); i;places.next(i)) {
        if(places.contents(i)->Id() > placesc)
          placesc = places.contents(i)->Id();
      }
    }
    if(!transitions.empty()) {
      for(Pix i = transitions.first(); i;transitions.next(i)) {
        if(transitions.contents(i)->Id() > transc)
          transc = transitions.contents(i)->Id();
      }
    }
    if(!ptransitions.empty()) {
      for(Pix i = ptransitions.first(); i;ptransitions.next(i)) {
        if(ptransitions.contents(i)->Id() > ptransc)
          ptransc = ptransitions.contents(i)->Id();
      }
    }
    clipboard->DeleteAll();
    if(!blocks.empty())
      blocks.contents(blocks.first())->ResetMaxIdTo(blocksc);
    if(!places.empty())
      places.contents(places.first())->ResetMaxIdTo(placesc);
    if(!transitions.empty())
      transitions.contents(transitions.first())->ResetMaxIdTo(transc);
    if(!ptransitions.empty())
      ptransitions.contents(ptransitions.first())->ResetMaxIdTo(ptransc);
  } else {
    clipboard = new Net();
  }
  ifstream in(argv);
  if (!in) {
    return;
  }  // if Dateifehler
  in >> *clipboard;
  if (clear_links)
       clipboard -> clearDeclaredLinks();
  paste();
  String objekte = SEL;
  int zaehler = 1;
  while(objekte != "") {
    objekte = objekte.after(",");
    zaehler++;
  }
  String kette[zaehler];
  objekte = SEL;
  for(int i = 0;i < zaehler;i++) {
    kette[i] = (i == zaehler-1) ? objekte : objekte.before(",");
    objekte = objekte.after(",");
  }
  if(inbox)
    TakeInBox(zaehler, kette);
  else
    AddToBox(zaehler, kette, b);
}  // Net::loadmodul

void
Net::moveBlock(String block, VHSet<String> &moved, int x, int y, int all)
{
  VHSet<String> bl = GetBlockFromName(block)->BlockKeySet();
  if(all == 1) {
    GetBlockFromName(block)->SetX(GetBlockFromName(block)->GetX()+x);
    GetBlockFromName(block)->SetY(GetBlockFromName(block)->GetY()+y);
  }
  for(Pix j = bl.first(); j;bl.next(j))
    moveBlock(bl(j), moved, x, y, 1);
  for(Pix i = transitions.first(); i;transitions.next(i)) {
    VHSet<String> bl = transitions.contents(i)->BlockKeySet();
    VHSet<String> temp = GetBlockFromName(block)->BlockKeySet();
    VHSet<String> t = bl;
    t &= temp;
    if((t.contains(block) || t.empty()) && bl.contains(block)) {
      moved.add(transitions.contents(i)->Name());
      transitions.contents(i)->SetX(transitions.contents(i)->GetX()+x);
      transitions.contents(i)->SetY(transitions.contents(i)->GetY()+y);
    }
  }
  for(Pix i = places.first(); i;places.next(i)) {
    VHSet<String> bl = places.contents(i)->BlockKeySet();
    VHSet<String> temp = GetBlockFromName(block)->BlockKeySet();
    VHSet<String> t = bl;
    t &= temp;
    if((t.contains(block) || t.empty()) && bl.contains(block)) {
      moved.add(places.contents(i)->Name());
      places.contents(i)->SetX(places.contents(i)->GetX()+x);
      places.contents(i)->SetY(places.contents(i)->GetY()+y);
    }
  }
  for(Pix i = ptransitions.first(); i;ptransitions.next(i)) {
    VHSet<String> bl = ptransitions.contents(i)->BlockKeySet();
    VHSet<String> temp = GetBlockFromName(block)->BlockKeySet();
    VHSet<String> t = bl;
    t &= temp;
    if((t.contains(block) || t.empty()) && bl.contains(block)) {
      moved.add(ptransitions.contents(i)->Name());
      ptransitions.contents(i)->SetX(ptransitions.contents(i)->GetX()+x);
      ptransitions.contents(i)->SetY(ptransitions.contents(i)->GetY()+y);
    }
  }
  for(Pix i = edges.first(); i; edges.next(i)) {
    if(moved.contains(edges.contents(i)->Src().Name()) && moved.contains(edges.contents(i)->Dest().Name())) {
      if(!(edges.contents(i)->getSplinex() == 0 && edges.contents(i)->getSpliney() == 0)) {
        edges.contents(i)->setSplinex(edges.contents(i)->getSplinex()+x);
        edges.contents(i)->setSpliney(edges.contents(i)->getSpliney()+y);
      }
    }
  }
  for(Pix i = pedges.first(); i; pedges.next(i)) {
    if(moved.contains(pedges.contents(i)->Src().Name()) && moved.contains(pedges.contents(i)->Dest().Name())) {
      if(!(pedges.contents(i)->getSplinex() == 0 && pedges.contents(i)->getSpliney() == 0)) {
        pedges.contents(i)->setSplinex(pedges.contents(i)->getSplinex()+x);
        pedges.contents(i)->setSpliney(pedges.contents(i)->getSpliney()+y);
      }
    }
  }
}  // Net::moveBlock

void
Net::moveBox(String box, VHSet<String> &moved, int x, int y, int all)
{
  VHSet<String> bl = GetBoxFromName(box)->BlockKeySet();
  if(all == 1) {
    GetBoxFromName(box)->SetX(GetBoxFromName(box)->GetX()+x);
    GetBoxFromName(box)->SetY(GetBoxFromName(box)->GetY()+y);
  }
  for(Pix j = bl.first(); j;bl.next(j))
    moveBlock(bl(j), moved, x, y, 1);
  for(Pix i = transitions.first(); i;transitions.next(i)) {
    if(transitions.contents(i)->BoxName() == box && !moved.contains(transitions.contents(i)->Name())) {
      moved.add(transitions.contents(i)->Name());
      transitions.contents(i)->SetX(transitions.contents(i)->GetX()+x);
      transitions.contents(i)->SetY(transitions.contents(i)->GetY()+y);
    }
  }
  for(Pix i = places.first(); i;places.next(i)) {
    if(places.contents(i)->BoxName() == box && !moved.contains(places.contents(i)->Name())) {
      moved.add(places.contents(i)->Name());
      places.contents(i)->SetX(places.contents(i)->GetX()+x);
      places.contents(i)->SetY(places.contents(i)->GetY()+y);
    }
  }
  for(Pix i = ptransitions.first(); i;ptransitions.next(i)) {
    if(ptransitions.contents(i)->BoxName() == box && !moved.contains(ptransitions.contents(i)->Name())) {
      moved.add(ptransitions.contents(i)->Name());
      ptransitions.contents(i)->SetX(ptransitions.contents(i)->GetX()+x);
      ptransitions.contents(i)->SetY(ptransitions.contents(i)->GetY()+y);
    }
  }
  for(Pix i = edges.first(); i; edges.next(i)) {
    if(moved.contains(edges.contents(i)->Src().Name()) && moved.contains(edges.contents(i)->Dest().Name())) {
      if(!(edges.contents(i)->getSplinex() == 0 && edges.contents(i)->getSpliney() == 0)) {
        edges.contents(i)->setSplinex(edges.contents(i)->getSplinex()+x);
        edges.contents(i)->setSpliney(edges.contents(i)->getSpliney()+y);
      }
    }
  }
  for(Pix i = pedges.first(); i; pedges.next(i)) {
    if(moved.contains(pedges.contents(i)->Src().Name()) && moved.contains(pedges.contents(i)->Dest().Name())) {
      if(!(pedges.contents(i)->getSplinex() == 0 && pedges.contents(i)->getSpliney() == 0)) {
        pedges.contents(i)->setSplinex(pedges.contents(i)->getSplinex()+x);
        pedges.contents(i)->setSpliney(pedges.contents(i)->getSpliney()+y);
      }
    }
  }
}  // Net::moveBox

void
Net::savemodul(CONST char *argv)
{
  if(clipboard == (Net *) NULL)
    return;
  ofstream out(argv);
  if(out) {
    out << *clipboard;
  }
} // Net::savemodul

VHSet<String>
Net::GetCurrent()
{
  VHSet<String> ret(10);
  Place* p;
  for(Pix i = places.first(); i; places.next(i)) {
    p = places.contents(i);
    if(p->NumTokens() > 0) {
      ret.add(places.key(i));
    }
  }
  return ret;
}  // Net::GetCurrent

void
Net::RecalcRootBlocks(void)
{
  VHMap<String, Block *> bc((Block *) NULL, 10);
  Pix i, j;
    // Annahme: Alle Bloecke sind Wurzeln.
  for (i = blocks.first(); i; blocks.next(i)) {
    blocks.contents(i)->SetRootBlock();
  }  // for i ueber alle Bloecke
    // Alle Kinder eines Blockes i als "Nicht-Wurzel" markieren.
  for (i = blocks.first(); i; blocks.next(i)) {
    bc = blocks.contents(i)->Blocks();  // Kinderchen holen
    for (j = bc.first(); j; bc.next(j)) {
      if (blocks.contains(bc.key(j))) {
//      if (blocks[bc.key(j)] != NULL)
        blocks[bc.key(j)]->UnsetRootBlock();
      }
    }  // for j ueber alle Kinder des Blockes i
  }  // for i ueber alle Bloecke
}  // Net::RecalcRootBlocks

  // Wird von sich selbst und RecalcBlockVis aufgerufen.
void
Net::RecalcBlockVisRec(Block *bl)
{
  BlockVisType child_block_vis;
  VHMap<String, Block *> bc((Block *) NULL, 10);
  Pix i;

  bc = bl->Blocks();  // Kinderchen holen
  switch (bl->GetBlockVis()) {
    case EXPOSED:
    case HIDDEN:
      child_block_vis = HIDDEN;  // Kinderchen sind nicht zu sehen
      break;
    case EXPOSED_CONTENTS:  // Kinderchen sind zu sehen
      child_block_vis = EXPOSED;
      break;
    default:
      cerr << "Internal error: Unknown BlockVisType in RecalcBlockVisRec()"
        << endl;
      return;
  }  // switch ueber die Sichtbarkeit der aktuellen Wurzel
    // Rekursion oder Abbruch ueber die Anzahl der Kinder.
    //   Keine Kinder -> Rekursionsabbruch
    //   Kinder -> weiter gehts
  for (i = bc.first(); i; bc.next(i)) {
    if ((bc.contents(i)->GetBlockVis() != EXPOSED_CONTENTS)
      || (child_block_vis != EXPOSED))  // nur dann aendern
      bc.contents(i)->SetBlockVis(child_block_vis);
    RecalcBlockVisRec(bc.contents(i));
  }  // for i ueber die direkten Kinder
}  // Net::RecalcBlockRec

  // Innerhalb der Blockbaeume werden die Sichtbarkeiten neu
  // bestimmt. Anscliessend werden fuer die Stellen und Transitionen
  // die Sichtbarkeiten anhand der Blocksichtbarkeit neu bestimmt.
  // Voraussetzung ist, dass die Blockstruktur stimmt, also
  // Net::RecalcRootBlocks vorher aufgerufen wurde.
void
Net::RecalcBlockVis(void)
{
  VHSet<String> rb(20);
  VHMap<String, Block *> bl((Block *) NULL, 10);
  BlockVisType new_block_vis, e_vis;
  Pix i, j, k;

    // 1. Fuer die Blockbaeume Sichtbarkeiten neu bestimmen.
  rb = RootBlocks();
  for (i = rb.first(); i; rb.next(i)) {
    if (blocks[rb(i)]->GetBlockVis() == HIDDEN)
      blocks[rb(i)]->SetBlockVis(EXPOSED);  // Safety first!
    RecalcBlockVisRec(blocks[rb(i)]);
  }  // for i ueber alle Wurzelbloecke

    // 2. Fuer die Stellen die Sichtbarkeit neu bestimmen.
  for (i = places.first(); i; places.next(i)) {
    bl = places.contents(i)->Blocks();  // Blockzugehoerigkeit holen
    new_block_vis = EXPOSED;  // Annahme: neuerdings sichtbar
    for (j = bl.first(); j; bl.next(j)) {
      if (bl.contents(j)->GetBlockVis() != EXPOSED_CONTENTS)
        new_block_vis = HIDDEN;
    }  // for j ueber alle Bloecke des Knotens
    places.contents(i)->SetBlockVis(new_block_vis);
  }  // for i ueber alle Stellen im Netz

    // 3. Fuer die Transitionen die Sichtbarkeit neu bestimmen.
  for (i = transitions.first(); i; transitions.next(i)) {
    new_block_vis = EXPOSED;  // Annahme: neuerdings sichtbar
    if ((sync_trans_switch == 0)
      && (transitions.contents(i)->GetChannel0Sync() == yes)) {
      new_block_vis = HIDDEN;
    }  // if SyncTrans, aber keine Anzeige
    else if (transitions.contents(i)->GetChannel0Sync() == no) {
      bl = transitions.contents(i)->Blocks();  // Blockzugehoerigkeit holen
      for (j = bl.first(); j; bl.next(j)) {
        if (bl.contents(j)->GetBlockVis() != EXPOSED_CONTENTS)
          new_block_vis = HIDDEN;
      }  // for j ueber alle Bloecke des Knotens
    }  // if normale Transition
    else {
        // 1. zugehoerige Phantomtransitionen holen
      const SET<String> &Phantoms = transitions.contents(i)->Phantoms();
        // 2. als Init fuer die Schnittmenge, schlicht alle bekannten Bloecke
      VHSet<String> intersect;
      Pix bl_i;
      for (bl_i = blocks.first(); bl_i; blocks.next(bl_i))
        intersect.add(blocks.key(bl_i));
        // 3. intersect mit den Bloecken der Phantomtransition schneiden
      for (Pix ptr_i = Phantoms.first(); ptr_i; Phantoms.next(ptr_i))
        intersect &= ptransitions[Phantoms(ptr_i)]->BlockKeySet();
        // 4. in intersect nun die Menge der gemeinsamen Bloecke,
        //    sprich die gemeinsamen "Wurzeln", falls diese alle
        //    EXPOSED_CONTENTS, dann ist die SyncTrans sichtbar
      for (bl_i = intersect.first(); bl_i; intersect.next(bl_i)) {
        if (blocks[intersect(bl_i)]->GetBlockVis() != EXPOSED_CONTENTS)
          new_block_vis = HIDDEN;
      }  // for bl_i ueber alle gemeinsamen Bloecke
    }  // else, Synchronisationstransition
    transitions.contents(i)->SetBlockVis(new_block_vis);
  }  // for i ueber alle Transitionen im Netz

    // 4. Fuer die Phantomtransitionen die Sichtbarkeit neu bestimmen.
  if (phantom_trans_switch == 1) {
    for (i = ptransitions.first(); i; ptransitions.next(i)) {
      bl = ptransitions.contents(i)->Blocks();  // Blockzugehoerigkeit holen
      new_block_vis = EXPOSED;  // Annahme: neuerdings sichtbar
      for (j = bl.first(); j; bl.next(j)) {
        if (bl.contents(j)->GetBlockVis() != EXPOSED_CONTENTS)
          new_block_vis = HIDDEN;
      }  // for j ueber alle Bloecke des Knotens
      ptransitions.contents(i)->SetBlockVis(new_block_vis);
    }  // for i ueber alle Phantomtransitionen im Netz
  }  // if Phantomtransitionen anzeigen
  else {  // keine Phantomtransitionen
    for (i = ptransitions.first(); i; ptransitions.next(i)) {
      ptransitions.contents(i)->SetBlockVis(HIDDEN);
    }  // for i ueber alle Phantomtransitionen im Netz
  }  // else, keine Phantomtransitionen

    // 5. Fuer die Kanten die Sichtbarkeit neu bestimmen.
    //    Durch die SyncTrans kann es spaeter Aenderungen geben.
  for (i = places.first(); i; places.next(i)) {
    const Map<String, Edge *> &ie = places.contents(i)->InputEdges();
    for (j = ie.first(); j; ie.next(j)) {
      if (places.contents(i)->GetBlockVis() == HIDDEN && ie.contents(j)->Src().GetBlockVis() == HIDDEN) {
        e_vis = HIDDEN;
        ie.contents(j)->ResetBlockSrc();
        ie.contents(j)->ResetBlockDest();
      }  // if Stelle selber nicht sichtbar
      else if (ie.contents(j)->Src().GetBlockVis() == HIDDEN) {
        e_vis = HIDDEN;
          // Blockzugehoerigkeit der Transition holen
        bl = transitions[ie.contents(j)->Src().Name()]->Blocks();
        for (k = bl.first(); k; bl.next(k)) {
          if (bl.contents(k)->GetBlockVis() == EXPOSED) {
            ie.contents(j)->AddBlockSrc(*(bl.contents(k)));
          }
        }  // for k ueber die zugehoerigen Bloecke
      }  // if Quelltransition nicht sichtbar
      else if (places.contents(i)->GetBlockVis() == HIDDEN) {
        e_vis = HIDDEN;
        bl = places.contents(i)->Blocks();
        for (k = bl.first(); k; bl.next(k)) {
          if (bl.contents(k)->GetBlockVis() == EXPOSED) {
            ie.contents(j)->AddBlockDest(*(bl.contents(k)));
          }
        }
      }
      else {
        e_vis = EXPOSED;
        ie.contents(j)->ResetBlockSrc();
        ie.contents(j)->ResetBlockDest();
      }  // else, wohl sichtbar
      ie.contents(j)->SetBlockVis(e_vis);
    }  // for j ueber alle Eingangskanten
    const Map<String, Edge *> &oe = places.contents(i)->OutputEdges();
    for (j = oe.first(); j; oe.next(j)) {
      if (places.contents(i)->GetBlockVis() == HIDDEN && oe.contents(j)->Dest().GetBlockVis() == HIDDEN) {
        e_vis = HIDDEN;
        oe.contents(j)->ResetBlockSrc();
        oe.contents(j)->ResetBlockDest();
      }  // if Stelle selber nicht sichtbar
      else if (oe.contents(j)->Dest().GetBlockVis() == HIDDEN) {
        e_vis = HIDDEN;
          // Blockzugehoerigkeit der Transition holen
        bl = transitions[oe.contents(j)->Dest().Name()]->Blocks();
        for (k = bl.first(); k; bl.next(k)) {
          if (bl.contents(k)->GetBlockVis() == EXPOSED) {
            oe.contents(j)->AddBlockDest(*(bl.contents(k)));
          }
        }  // for k ueber die zugehoerigen Bloecke
      }  // if Zieltransition nicht sichtbar
      else if (places.contents(i)->GetBlockVis() == HIDDEN) {
        e_vis = HIDDEN;
        bl = places.contents(i)->Blocks();
        for (k = bl.first(); k; bl.next(k)) {
          if (bl.contents(k)->GetBlockVis() == EXPOSED) {
            oe.contents(j)->AddBlockSrc(*(bl.contents(k)));
          }
        }
      }
      else {
        e_vis = EXPOSED;
        oe.contents(j)->ResetBlockSrc();
        oe.contents(j)->ResetBlockDest();
      }  // else, wohl sichtbar
      oe.contents(j)->SetBlockVis(e_vis);
    }  // for j ueber alle Ausgangskanten
  }  // for i ueber alle Stellen im Netz

    // 6. Bei SyncTrans die Blockkanten erstellen (optional)
  if (sync_trans_switch == 1) {
    for (i = transitions.first(); i; transitions.next(i)) {
      if ((transitions.contents(i)->GetChannel0Sync() == yes)
        && (transitions.contents(i)->GetBlockVis() == EXPOSED)) {
        const Map<String, Edge *> &ie = transitions.contents(i)->InputEdges();
        for (j = ie.first(); j; ie.next(j)) {
          if (ie.contents(j)->GetBlockVis() == HIDDEN) {
              // Blockzugehoerigkeit der Stelle holen
            bl = places[ie.contents(j)->Src().Name()]->Blocks();
            for (k = bl.first(); k; bl.next(k)) {
              if (bl.contents(k)->GetBlockVis() == EXPOSED)
                ie.contents(j)->AddBlockSrc(*(bl.contents(k)));
            }  // for k ueber die zugehoerigen Bloecke
          }  // if Kanten ist bisher hidden
        }  // for j ueber alle Eingangskanten
        const Map<String, Edge *> &oe = transitions.contents(i)->OutputEdges();
        for (j = oe.first(); j; oe.next(j)) {
          if (oe.contents(j)->GetBlockVis() == HIDDEN) {
              // Blockzugehoerigkeit der Stelle holen
            bl = places[oe.contents(j)->Dest().Name()]->Blocks();
            for (k = bl.first(); k; bl.next(k)) {
              if (bl.contents(k)->GetBlockVis() == EXPOSED)
                oe.contents(j)->AddBlockDest(*(bl.contents(k)));
            }  // for k ueber die zugehoerigen Bloecke
          }  // if Kanten ist bisher hidden
        }  // for j ueber alle Ausgangskanten
      }  // if sichtbare SyncTrans liegt vor
    }  // for i ueber alle Transitionen
  }  // if SyncTrans-Anzeigerei eingeschaltet

    // 7. Fuer die Phantomkanten die Sichtbarkeit neu bestimmen.
  if (phantom_trans_switch == 1) {
    for (i = places.first(); i; places.next(i)) {
      const Map<String, Edge *> &ie = places.contents(i)->InputPEdges();
      for (j = ie.first(); j; ie.next(j)) {
        if (places.contents(i)->GetBlockVis() == HIDDEN) {
          e_vis = HIDDEN;
        }  // if Stelle selber nicht sichtbar
        else if (&ie.contents(j)->Src() != NULL) {
          if (ie.contents(j)->Src().GetBlockVis() == HIDDEN) {
            e_vis = HIDDEN;
          }
          else {
            e_vis = EXPOSED;
          }
        }  // if Quellphantomtransition nicht sichtbar
        else {
          e_vis = HIDDEN;
        }  // else, wohl sichtbar
        ie.contents(j)->SetBlockVis(e_vis);
/*
        if (e_vis == HIDDEN) {
            // Blockzugehoerigkeit holen
          bl = ptransitions[ie.contents(j)->Src().Name()]->Blocks();
          for (k = bl.first(); k; bl.next(k)) {
            if (bl.contents(k)->GetBlockVis() == EXPOSED)
              ie.contents(j)->SetBlockSrc(*(bl.contents(k)));
          }  // for k ueber die zugehoerigen Bloecke
        }  // if eigentlich HIDDEN, also auf Blockkante pruefen
        else {
*/
          ie.contents(j)->ResetBlockSrc();
          ie.contents(j)->ResetBlockDest();
/*
        }  // else, keine Blockkante
*/
      }  // for j ueber alle Eingangsphantomkanten
      const Map<String, Edge *> &oe = places.contents(i)->OutputPEdges();
      for (j = oe.first(); j; oe.next(j)) {
        if (places.contents(i)->GetBlockVis() == HIDDEN) {
          e_vis = HIDDEN;
        }  // if Stelle selber nicht sichtbar
        else if (&oe.contents(j)->Dest() != NULL) { 
          if (oe.contents(j)->Dest().GetBlockVis() == HIDDEN) {
            e_vis = HIDDEN;
          }
          else {
            e_vis = EXPOSED;
          }
        }  // if Zielphantomtransition nicht sichtbar
        else {
          e_vis = HIDDEN;
        }  // else, wohl sichtbar
        oe.contents(j)->SetBlockVis(e_vis);
/*
        if (e_vis == HIDDEN) {
            // Blockzugehoerigkeit holen
          bl = ptransitions[oe.contents(j)->Dest().Name()]->Blocks();
          for (k = bl.first(); k; bl.next(k)) {
            if (bl.contents(k)->GetBlockVis() == EXPOSED)
              oe.contents(j)->SetBlockDest(*(bl.contents(k)));
          }  // for k ueber die zugehoerigen Bloecke
        }  // if eigentlich HIDDEN, also auf Blockkante pruefen
        else {
*/
          oe.contents(j)->ResetBlockSrc();
          oe.contents(j)->ResetBlockDest();
/*
        }  // else, keine Blockkante
*/
      }  // for j ueber alle Ausgangsphantomkanten
    }  // for i ueber alle Stellen im Netz
  }  // if Phantomtransitionen und -kanten anzeigen
  else {  // nicht anzeigen
    for (i = places.first(); i; places.next(i)) {
      const Map<String, Edge *> &ie = places.contents(i)->InputPEdges();
      for (j = ie.first(); j; ie.next(j)) {
        ie.contents(j)->SetBlockVis(HIDDEN);
        ie.contents(j)->ResetBlockSrc();
        ie.contents(j)->ResetBlockDest();
      }  // for j ueber alle Eingangsphantomkanten
      const Map<String, Edge *> &oe = places.contents(i)->OutputPEdges();
      for (j = oe.first(); j; oe.next(j)) {
        oe.contents(j)->SetBlockVis(HIDDEN);
        oe.contents(j)->ResetBlockSrc();
        oe.contents(j)->ResetBlockDest();
      }  // for j ueber alle Ausgangsphantomkanten
    }  // for i ueber alle Stellen im Netz
  }  // else, nicht anzeigen
  for(i = transitions.first(); i; transitions.next(i)) {
    if(transitions.contents(i)->BoxPointer() != NULL) {
      if(transitions.contents(i)->BoxPointer()->GetVisibility() == 0) {
        const Map<String, Edge *> &out = transitions.contents(i)->OutputEdges();
        const Map<String, Edge *> &in  = transitions.contents(i)->InputEdges();
        transitions.contents(i)->SetBlockVis(HIDDEN);
        for(j = out.first(); j; out.next(j)) {
          out.contents(j)->SetBlockVis(HIDDEN);
        }
        for(j = in.first(); j; in.next(j)) {
          in.contents(j)->SetBlockVis(HIDDEN);
        }
      }
    }
  }
  for(i = places.first(); i; places.next(i)) {
    if(places.contents(i)->BoxPointer() != NULL) {
      if(places.contents(i)->BoxPointer()->GetVisibility() == 0) {
        const Map<String, Edge *> &out = places.contents(i)->OutputEdges();
        const Map<String, Edge *> &in  = places.contents(i)->InputEdges();
        const Map<String, Edge *> &pout = places.contents(i)->OutputPEdges();
        const Map<String, Edge *> &pin  = places.contents(i)->InputPEdges();
        places.contents(i)->SetBlockVis(HIDDEN);
        for(j = out.first(); j; out.next(j)) {
          out.contents(j)->SetBlockVis(HIDDEN);
        }
        for(j = in.first(); j; in.next(j)) {
          in.contents(j)->SetBlockVis(HIDDEN);
        }
        for(j = pout.first(); j; pout.next(j)) {
          pout.contents(j)->SetBlockVis(HIDDEN);
        }
        for(j = pin.first(); j; pin.next(j)) {
          pin.contents(j)->SetBlockVis(HIDDEN);
        }
      }
    }
  }
  for(i = ptransitions.first(); i; ptransitions.next(i)) {
    if(ptransitions.contents(i)->BoxPointer() != NULL) {
      if(ptransitions.contents(i)->BoxPointer()->GetVisibility() == 0) {
        const Map<String, Edge *> &pout = ptransitions.contents(i)->OutputPEdges();
        const Map<String, Edge *> &pin  = ptransitions.contents(i)->InputPEdges();
        ptransitions.contents(i)->SetBlockVis(HIDDEN);
        for(j = pout.first(); j; pout.next(j)) {
          pout.contents(j)->SetBlockVis(HIDDEN);
        }
        for(j = pin.first(); j; pin.next(j)) {
          pin.contents(j)->SetBlockVis(HIDDEN);
        }
      }
    }
  }
  for(i = blocks.first(); i; blocks.next(i)) {
    if(blocks.contents(i)->BoxPointer() != NULL) {
      if(blocks.contents(i)->BoxPointer()->GetVisibility() == 0) {
        blocks.contents(i)->SetBlockVis(HIDDEN);
      }
    }
  }
}  // Net::RecalcBlockVis

void
Net::FireTransition(const String &tn)
{
  Transition *t = transitions[tn];
  VHSet<String> in = t->InputPlaces();
  VHSet<String> out = t->OutputPlaces();

  Pix i,j;
  for (i = in.first(); i; in.next(i)) {
    const Edge *e = t->EdgeFrom(in(i));
    places[in(i)]->RemoveTokens(e->Weight());
  }  // for i ueber die Stellen im Vorbereich
  for (i = out.first(); i; out.next(i)) {
    const Edge *e = t->EdgeTo(out(i));
    places[out(i)]->AddTokens(e->Weight());
  }  // for i ueber die Stellen im Nachbereich
  activeTransitions.del(tn);
  for (i = in.first(); i; in.next(i)) {
    const Map<String, Edge *> &OutEdges = places[in(i)]->OutputEdges();
    for (j = OutEdges.first(); j; OutEdges.next(j)) {
      Edge *e = OutEdges.contents(j);
      if(e->Dest().asTransition()->CanFire())
        activeTransitions.add(e->Dest().asTransition()->Name());
      else
        activeTransitions.del(e->Dest().asTransition()->Name());
    }  // for j "uber alle Ausgangskanten
  }  // for i "uber alle Eingangsstellen
  for (i = out.first(); i; out.next(i)) {
    const Map<String, Edge *> &OutEdges = places[out(i)]->OutputEdges();
    for (j = OutEdges.first(); j; OutEdges.next(j)) {
      Edge *e = OutEdges.contents(j);
      if(e->Dest().asTransition()->CanFire())
        activeTransitions.add(e->Dest().asTransition()->Name());
      else
        activeTransitions.del(e->Dest().asTransition()->Name());
    }  // for j "uber alle Ausgangskanten
  }  // for i "uber alle Ausgangsstellen
  if (transitions[tn]->asTransition()->CanFire())
    activeTransitions.add(tn);
}  // Net::FireTransition

Block *
Net::AllocBlock() const
{
  return new Block(*this);
}  // Net::AllocBlock

const String &
Net::NewBlock(void)
{
  Block *b = AllocBlock();
  blocks[b->Name()] = b;
  net_type_switch = 1;
  return b->Name();
}  // Net::NewBlock

Box *
Net::AllocBox() const
{
  return new Box(*this);
}  // Net::AllocBox

const String &
Net::NewBox(void)
{
  Box *b = AllocBox();
  boxes[b->Name()] = b;
  net_type_switch = 1;
  return b->Name();
}  // Net::NewBox

Edge *
Net::AllocEdge() const
{
  return new Edge(*this);
}  // Net::AllocEdge

Edge *
Net::AllocPEdge() const
{
  return new Edge(*this);
}  // Net::AllocPEdge

Edge *
Net::AllocEdgeFT(Vertex &f, Vertex &t) const
{
  return new Edge(f, t, *this);
}  // Net::AllocEdgeFT

Edge *
Net::AllocPEdgeFT(Vertex &f, Vertex &t) const
{
  return new Edge(f, t, *this);
}  // Net::AllocPEdgeFT

Edge *
Net::FindEdge(const String &from, const String &to)
{
  if (places.contains(from) && transitions.contains(to))
    return places[from]->EdgeTo(to);
  else if (transitions.contains(from) && places.contains(to)) 
    return transitions[from]->EdgeTo(to);
  return NULL;
}  // Net::FindEdge

Edge *
Net::FindPEdge(const String &from, const String &to)
{
  if (places.contains(from) && ptransitions.contains(to))
    return places[from]->PEdgeTo(to);
  else if (ptransitions.contains(from) && places.contains(to)) 
    return ptransitions[from]->EdgeTo(to);
  return NULL;
}  // Net::FindPEdge
	
String
Net::NewEdge(const String &from, const String &to)
{
  String s;
  Edge *e;

  if (places.contains(from) && transitions.contains(to)) {
    if (places[from]->IsInput(to)) 
      e = places[from]->EdgeTo(to);
    else {
      e = AllocEdgeFT(*places[from], *transitions[to]);
      places[from]->SetOutEdge(*e);
      transitions[to]->SetInEdge(*e);
    }
    s = "A" + IntToString(e->Id());
    edges[s] = e;
  }
  else if (transitions.contains(from) && places.contains(to)) {
    if (transitions[from]->IsInput(to)) 
      e = transitions[from]->EdgeTo(to);
    else {
      e = AllocEdgeFT(*transitions[from], *places[to]);
      transitions[from]->SetOutEdge(*e);
      places[to]->SetInEdge(*e);
    }
    s = "A"+IntToString(e->Id());
    edges[s] = e;
  }
  else {
    cerr << "Error: You tried to connect two\n"
      << "vertices, one of which does not exist.\n";
  }  // else, ein uebel Ding

  char buff[32];
  String t = to;
  String f = from;
  for(int n = 0; n < count; n++) {
    write(Verbindungen[n]->pipe1[1], "NewEdge", 32);
    read(Verbindungen[n]->pipe2[0], buff, 32);
    write(Verbindungen[n]->pipe1[1], f.data(), 32);
    read(Verbindungen[n]->pipe2[0], buff, 32);
    write(Verbindungen[n]->pipe1[1], t.data(), 32);
  }

  return s;
}  // Net::NewEdge

String
Net::NewPEdge(const String &from, const String &to)
{
  String s;
  Edge *e;

  if (places.contains(from) && ptransitions.contains(to)) {
    if (places[from]->PIsInput(to)) 
      e = places[from]->PEdgeTo(to);
    else {
      e = AllocPEdgeFT(*places[from], *ptransitions[to]);
      places[from]->SetOutPEdge(*e);
      ptransitions[to]->SetInPEdge(*e);
    }
    e->setPhantom(1);
    s = "PA" + IntToString(e->Id());
    pedges[s] = e;
  }
  else if (ptransitions.contains(from) && places.contains(to)) {
    if (ptransitions[from]->PIsInput(to)) 
      e = ptransitions[from]->PEdgeTo(to);
    else {
      e = AllocPEdgeFT(*ptransitions[from], *places[to]);
      ptransitions[from]->SetOutPEdge(*e);
      places[to]->SetInPEdge(*e);
    }
    e->setPhantom(1);
    s = "PA"+IntToString(e->Id());
    pedges[s] = e;
  }
  else {
    cerr << "Error: You tried to connect two\n"
      << "vertices, one of which does not exist.\n";
  }  // else, ein uebel Ding
  return s;
}  // Net::NewPEdge

Transition *
Net::AllocTransition() const
{
  return new Transition(*this);
}  // Net::AllocTransition

const String &
Net::NewTransition(void)
{
  Transition *t = AllocTransition();

  char buff[32];
  String NA = t->Name();
  for(int n = 0; n < count; n++) {
    write(Verbindungen[n]->pipe1[1], "NewTransition", 32);
    read(Verbindungen[n]->pipe2[0], buff, 32);
    write(Verbindungen[n]->pipe1[1], NA.data(), 32);
  }

  transitions[t->Name()] = t;
  return t->Name();
}  // Net::NewTransition

PTransition *
Net::AllocPTransition() const
{
  return new PTransition(*this);
}  // Net::AllocPTransition

const String &
Net::NewPTransition(void)
{
  PTransition *pt = AllocPTransition();

  ptransitions[pt->Name()] = pt;
  net_type_switch = 1;
  return pt->Name();
}  // Net::NewPTransition

Place *
Net::AllocPlace() const
{
  return new Place(*this);
}  // Net::AllocPlace

const String &
Net::NewPlace(void) 
{
  Place *p = AllocPlace();

  char buff[32];
  String NA = p->Name();
  for(int n = 0; n < count; n++) {
    write(Verbindungen[n]->pipe1[1], "NewPlace", 32);
    read(Verbindungen[n]->pipe2[0], buff, 32);
    write(Verbindungen[n]->pipe1[1], NA.data(), 32);
  }

  places[p->Name()] = p;
  return p->Name();
}  // Net::NewPlace

void
Net::NewPlace(String name)
{
  Place *p = AllocPlace();

  places[p->Name()] = p;
  p->ChangeMyName(name);
  places[p->Name()] = p;
}  // Net::NewPlace

void
Net::RemoveBlock(const String &n)
{
  delete blocks[n];
  blocks.del(n);
}  // Net::RemoveBlock

void 
Net::RemovePlace(const String &n)
{
  delete places[n];
  places.del(n);
}  // Net::RemovePlace

void 
Net::RemoveTransition(const String &n)
{
  delete transitions[n];
  transitions.del(n);
}  // Net::RemoveTransition

void 
Net::RemovePTransition(const String &n)
{
  delete ptransitions[n];
  ptransitions.del(n);
}  // Net::RemovePTransition

void
Net::RemoveEdge(Edge &e)
{
  delete &e;
}  // Net::RemoveEdge

void
Net::RemovePEdge(Edge &e)
{
  delete &e;
}  // Net::RemovePEdge

  // Einen Block anhand einer numerischen Identifikation suchen.
  // Bloecken, Stellen und Transitionen werden in der Netzdatei
  // Block-Ids mitgegeben. Mit Hilfe dieser Id wird hier der Block
  // ermittelt.
Block *
Net::GetBlockFromId(int id)
{
  for (Pix i = blocks.first(); i; blocks.next(i))
    if (blocks.contents(i)->Id() == BlockKonvert(id))
      return blocks.contents(i);
  return NULL;
}  // Net::GetBlockFromId

Block *
Net::GetBlockFromName(String name)
{
  return blocks[name];
}  // Net::GetBlockFromName

Box *
Net::GetBoxFromName(String name)
{
  return boxes[name];
}  // Net::GetBoxFromName

  // Eine Transition anhand einer numerischen Identifikation suchen.
  // Den Phantomtransitionen werden in der Netzdatei Transitions-Ids
  // mitgegeben. Mit Hilfe dieser Id wird hier die Transition ermittelt.
Transition *
Net::GetTransitionFromId(int id)
{
  for (Pix i = transitions.first(); i; transitions.next(i))
    if (transitions.contents(i)->Id() == TransKonvert(id)) 
      return transitions.contents(i);
  return NULL;
}  // Net::GetTransitionFromId

Transition *
Net::FindTransition(String name)
{
  return transitions[name];
}

PTransition *
Net::FindPTransition(String name)
{
  return ptransitions[name];
}

Place *
Net::FindPlace(String name)
{
  return places[name];
}

Vertex *
Net::GetFirstFromId(int id, char c)
{
  if (c == '<') {
    for (Pix i = transitions.first(); i; transitions.next(i))
      if (transitions.contents(i)->Id() == TransKonvert(id)) 
        return transitions.contents(i);
  }  // if c gleich "<"
  else {
    for (Pix i = places.first(); i; places.next(i))
      if (places.contents(i)->Id() == PlaceKonvert(id)) 
        return places.contents(i);
  }  // else, wahr wohl c gleich ">" gemeint
  return NULL;
}  // Net::GetFirstFromId

Vertex *
Net::GetSecondFromId(int id, char c)
{
  if (c == '>') {
    for (Pix i = transitions.first(); i; transitions.next(i))
      if (transitions.contents(i)->Id() == TransKonvert(id)) 
        return transitions.contents(i);
  }  // if c gleich ">"
  else {
    for (Pix i = places.first(); i; places.next(i))
      if (places.contents(i)->Id() == PlaceKonvert(id)) 
        return places.contents(i);
  }  // else, wahr wohl c gleich "<" gemeint
  return NULL;
}  // Net::GetSecondFromId

Vertex *
Net::GetPFirstFromId(int id, char c)
{
  if (c == '<') {
    for (Pix i = ptransitions.first(); i; ptransitions.next(i))
      if (ptransitions.contents(i)->Id() == PTransKonvert(id)) 
        return ptransitions.contents(i);
  }  // if c gleich "<"
  else {
    for (Pix i = places.first(); i; places.next(i))
      if (places.contents(i)->Id() == PlaceKonvert(id)) 
        return places.contents(i);
  }  // else, wahr wohl c gleich ">" gemeint
  return NULL;
}  // Net::GetPFirstFromId

Vertex *
Net::GetPSecondFromId(int id, char c)
{
  if (c == '>') {
    for (Pix i = ptransitions.first(); i; ptransitions.next(i))
      if (ptransitions.contents(i)->Id() == PTransKonvert(id)) 
        return ptransitions.contents(i);
  }  // if c gleich ">"
  else {
    for (Pix i = places.first(); i; places.next(i))
      if (places.contents(i)->Id() == PlaceKonvert(id)) 
        return places.contents(i);
  }  // else, wahr wohl c gleich "<" gemeint
  return NULL;
}  // Net::GetPSecondFromId


int
Net::BlockKonvert(int i)
{
  if(Blockkonvert.contains(i))
    return Blockkonvert[i];
  else 
    return -1;
} // Net::BlockKonvert

int
Net::PlaceKonvert(int i)
{
  if(Placekonvert.contains(i))
    return Placekonvert[i];
  else 
    return -1;
} // Net::PlaceKonvert

int
Net::TransKonvert(int i)
{
  if(Transkonvert.contains(i))
    return Transkonvert[i];
  else 
    return -1;
}  // Net::TransKonvert

int
Net::PTransKonvert(int i)
{
  if(PTranskonvert.contains(i))
    return PTranskonvert[i];
  else 
    return -1;
}  // Net::PTransKonvert

void
Net::BlockKonvert(int key, int value)
{
  Blockkonvert[key] = value;
}  // Net::BlockKonvert

void
Net::PlaceKonvert(int key, int value)
{
  Placekonvert[key] = value;
}  // Net::PlaceKonvert

void
Net::TransKonvert(int key, int value)
{
  Transkonvert[key] = value;
} // Net::TransKonvert

void
Net::PTransKonvert(int key, int value)
{
  PTranskonvert[key] = value;
}  // Net::PTransKonvert

void
Net::ChangeTyp(char *t1, char *t2)
{
  char buff[32];
  for(int n = 0; n < count; n++) {
    write(Verbindungen[n]->pipe1[1], "Typ", 32);
    read(Verbindungen[n]->pipe2[0], buff, 32);
    write(Verbindungen[n]->pipe1[1], t1, 32);
    read(Verbindungen[n]->pipe2[0], buff, 32);
    write(Verbindungen[n]->pipe1[1], t2, 256);
  }
}  // Net::ChangeTyp

void
Net::ChangeMarking(char *t1, char *t2)
{
  char buff[32];
  for(int n = 0; n < count; n++) {
    write(Verbindungen[n]->pipe1[1], "Marking", 32);
    read(Verbindungen[n]->pipe2[0], buff, 32);
    write(Verbindungen[n]->pipe1[1], t1, 32);
    read(Verbindungen[n]->pipe2[0], buff, 32);
    if(strcmp(t2, "") == 0)
      write(Verbindungen[n]->pipe1[1], "leer", 256);
    else
      write(Verbindungen[n]->pipe1[1], t2, 256);
  }
}  // Net::ChangeMarking

void
Net::ChangeValueTerm(char *t1, char *t2)
{
  char buff[32];
  for(int n = 0; n < count; n++) {
    write(Verbindungen[n]->pipe1[1], "ValueTerm", 32);
    read(Verbindungen[n]->pipe2[0], buff, 32);
    write(Verbindungen[n]->pipe1[1], t1, 32);
    read(Verbindungen[n]->pipe2[0], buff, 32);
    write(Verbindungen[n]->pipe1[1], t2, 256);
  }
}  // Net::ChangeValueTerm

void
Net::Parallel(String Block1, String Block2, String Name)
{
  Box *ptr = GetBoxFromName(Name);
  Box *ptr2 = GetBoxFromName(Block1);
  Box *ptr3 = GetBoxFromName(Block2);
  ptr->SetX(int((ptr2->GetX() + ptr3->GetX()) / 2));
  ptr->SetY(int((ptr2->GetY() + ptr3->GetY()) / 2));
  for (Pix i = places.first(); i;places.next(i)) {
    if (places.contents(i)->BoxName() == Block1) {
      places.contents(i)->setBox(Name, ptr);
    }
    if (places.contents(i)->BoxName() == Block2) {
      places.contents(i)->setBox(Name, ptr);
    }
  }
  for (Pix i = transitions.first(); i; transitions.next(i)) {
    if (transitions.contents(i)->BoxName() == Block1) {
      transitions.contents(i)->setBox(Name, ptr);
    }
    if (transitions.contents(i)->BoxName() == Block2) {
      transitions.contents(i)->setBox(Name, ptr);
    }
  }
  for (Pix i = ptransitions.first(); i;ptransitions.next(i)) {
    if (ptransitions.contents(i)->BoxName() == Block1) {
      ptransitions.contents(i)->setBox(Name, ptr);
    }
    if (ptransitions.contents(i)->BoxName() == Block2) {
      ptransitions.contents(i)->setBox(Name, ptr);
    }
  }
  DeleteBox(Block1);
  DeleteBox(Block2);
}  // Net::Parallel

void
Net::Sequence(String Block1, String Block2, String Name)
{
  Box *ptr = GetBoxFromName(Name);
  Box *ptr2 = GetBoxFromName(Block1);
  Box *ptr3 = GetBoxFromName(Block2);
  ptr->SetX(int((ptr2->GetX() + ptr3->GetX()) / 2));
  ptr->SetY(int((ptr2->GetY() + ptr3->GetY()) / 2));
  VHMap<String, Place *> First((Place *) NULL, 10);
  VHMap<String, Place *> Second((Place *) NULL, 10);
  for(Pix i = places.first(); i; places.next(i)) {
    if (places.contents(i)->BoxName() == Block1 &&
        places.contents(i)->State() == 1) {
      First[places.contents(i)->Name()] = places.contents(i);
    }
  }
  for(Pix i = places.first(); i; places.next(i)) {
    if (places.contents(i)->BoxName() == Block2 &&
        places.contents(i)->State() == 0) {
      Second[places.contents(i)->Name()] = places.contents(i);
    }
  }
  if (First.length() == 0 || Second.length() == 0) {
    DeleteBox(Name);
    return;
  }
  String PlaceName;
  String temp, n;
  for(Pix i = First.first(); i; First.next(i)) {
    for(Pix j = Second.first(); j; Second.next(j)) {
      temp = First.contents(i)->Name() + "@" + Second.contents(j)->Name();
      NewPlace(temp);
      places[temp]->SetX(int(First.contents(i)->GetX() +
                             Second.contents(j)->GetX()) / 2);
      places[temp]->SetY(int(First.contents(i)->GetY() +
                             Second.contents(j)->GetY()) / 2);
      places[temp]->setBox(Name, ptr);
      const Map<String, Edge *> &in = First.contents(i)->InputEdges();
      for(Pix k = in.first(); k; in.next(k)) {
        n = NewEdge(in.contents(k)->Src().Name(), temp);
      }
      const Map<String, Edge *> &out = Second.contents(j)->OutputEdges();
      for(Pix k = out.first(); k; out.next(k)) {
        n = NewEdge(temp, out.contents(k)->Dest().Name());
      }
    } 
  }
  for(Pix i = First.first(); i; First.next(i)) {
    RemovePlace(First.key(i));
  }
  for(Pix i = Second.first(); i; Second.next(i)) {
    RemovePlace(Second.key(i));
  }
  for (Pix i = places.first(); i;places.next(i)) {
    if (places.contents(i)->BoxName() == Block1) {
      places.contents(i)->setBox(Name, ptr);
    }
    if (places.contents(i)->BoxName() == Block2) {
      places.contents(i)->setBox(Name, ptr);
    }
  }
  for (Pix i = transitions.first(); i; transitions.next(i)) {
    if (transitions.contents(i)->BoxName() == Block1) {
      transitions.contents(i)->setBox(Name, ptr);
    }
    if (transitions.contents(i)->BoxName() == Block2) {
      transitions.contents(i)->setBox(Name, ptr);
    }
  }
  for (Pix i = ptransitions.first(); i;ptransitions.next(i)) {
    if (ptransitions.contents(i)->BoxName() == Block1) {
      ptransitions.contents(i)->setBox(Name, ptr);
    }
    if (ptransitions.contents(i)->BoxName() == Block2) {
      ptransitions.contents(i)->setBox(Name, ptr);
    }
  }
  DeleteBox(Block1);
  DeleteBox(Block2);
}  // Net::Sequence

void
Net::Substitution(String Box, String Transition)
{
  VHMap<String, Place *> InputBox((Place *) NULL, 10);
  const VHSet<String> &InputTrans = transitions[Transition]->InputPlaces();
  for(Pix i = places.first(); i; places.next(i)) {
    if (places.contents(i)->BoxName() == Box &&
        places.contents(i)->State() == 0) {
      InputBox[places.contents(i)->Name()] = places.contents(i);
    }
  }
  VHMap<String, Place *> OutputBox((Place *) NULL, 10);
  const VHSet<String> &OutputTrans = transitions[Transition]->OutputPlaces();
  for(Pix i = places.first(); i; places.next(i)) {
    if (places.contents(i)->BoxName() == Box &&
        places.contents(i)->State() == 1) {
      OutputBox[places.contents(i)->Name()] = places.contents(i);
    }
  }
  // Test, ob wirklich Eingangs- und Ausgangsstellen vorhanden sind.
  if (OutputBox.length() == 0 || InputBox.length() == 0 || InputTrans.length() == 0 || OutputTrans.length() == 0)
    return;

  String PlaceName;
  String temp, n;
  for(Pix i = InputBox.first(); i; InputBox.next(i)) {
    for(Pix j = InputTrans.first(); j; InputTrans.next(j)) {
      temp = InputBox.contents(i)->Name() + "@" + InputTrans(j);
      NewPlace(temp);
      places[temp]->SetX(int(InputBox.contents(i)->GetX() +
                             FindTransition(Transition)->GetX()) / 2);
      places[temp]->SetY(int(InputBox.contents(i)->GetY() +
                             FindTransition(Transition)->GetY()) / 2);
      const Map<String, Edge *> &in = InputBox.contents(i)->OutputEdges();
      for(Pix k = in.first(); k; in.next(k)) {
        n = NewEdge(temp, in.contents(k)->Dest().Name());
      }
      const Map<String, Edge *> &out = places[InputTrans(j)]->OutputEdges();
      for(Pix k = out.first(); k; out.next(k)) {
        n = NewEdge(temp, out.contents(k)->Dest().Name());
      }
      const Map<String, Edge *> &aus = places[InputTrans(j)]->InputEdges();
      for(Pix k = aus.first(); k; aus.next(k)) {
        n = NewEdge(aus.contents(k)->Src().Name(), temp);
      }
    } 
  }
  for(Pix i = InputBox.first(); i; InputBox.next(i)) {
    RemovePlace(InputBox.key(i));
  }
  for(Pix i = InputTrans.first(); i; InputTrans.next(i)) {
    RemovePlace(InputTrans(i));
  }

  for(Pix i = OutputBox.first(); i; OutputBox.next(i)) {
    for(Pix j = OutputTrans.first(); j; OutputTrans.next(j)) {
      temp = OutputBox.contents(i)->Name() + "@" + OutputTrans(j);
      NewPlace(temp);
      places[temp]->SetX(int(OutputBox.contents(i)->GetX() +
                             FindTransition(Transition)->GetX()) / 2);
      places[temp]->SetY(int(OutputBox.contents(i)->GetY() +
                             FindTransition(Transition)->GetY()) / 2);
      const Map<String, Edge *> &in = OutputBox.contents(i)->InputEdges();
      for(Pix k = in.first(); k; in.next(k)) {
        n = NewEdge(in.contents(k)->Src().Name(), temp);
      }
      const Map<String, Edge *> &out = places[OutputTrans(j)]->InputEdges();
      for(Pix k = out.first(); k; out.next(k)) {
        n = NewEdge(out.contents(k)->Src().Name(), temp);
      }
      const Map<String, Edge *> &ein = places[InputTrans(j)]->OutputEdges();
      for(Pix k = ein.first(); k; ein.next(k)) {
        n = NewEdge(temp, ein.contents(k)->Dest().Name());
      }
    }
  }
  for(Pix i = OutputBox.first(); i; OutputBox.next(i)) {
    RemovePlace(OutputBox.key(i));
  }
  for(Pix i = OutputTrans.first(); i; OutputTrans.next(i)) {
    RemovePlace(OutputTrans(i));
  }
  RemoveTransition(Transition);
}  // Net::Substitution

void
Net::Choice(String Block1, String Block2, String Name)
{
  Box *ptr = GetBoxFromName(Name);
  Box *ptr2 = GetBoxFromName(Block1);
  Box *ptr3 = GetBoxFromName(Block2);
  ptr->SetX(int((ptr2->GetX() + ptr3->GetX()) / 2));
  ptr->SetY(int((ptr2->GetY() + ptr3->GetY()) / 2));
  VHMap<String, Place *> First((Place *) NULL, 10);
  VHMap<String, Place *> Second((Place *) NULL, 10);
  VHMap<String, Place *> First2((Place *) NULL, 10);
  VHMap<String, Place *> Second2((Place *) NULL, 10);
  for(Pix i = places.first(); i; places.next(i)) {
    if (places.contents(i)->BoxName() == Block1 &&
        places.contents(i)->State() == 0) {
      First[places.contents(i)->Name()] = places.contents(i);
    }
  }
  for(Pix i = places.first(); i; places.next(i)) {
    if (places.contents(i)->BoxName() == Block2 &&
        places.contents(i)->State() == 0) {
      Second[places.contents(i)->Name()] = places.contents(i);
    }
  }
  for(Pix i = places.first(); i; places.next(i)) {
    if (places.contents(i)->BoxName() == Block1 &&
        places.contents(i)->State() == 1) {
      First2[places.contents(i)->Name()] = places.contents(i);
    }
  }
  for(Pix i = places.first(); i; places.next(i)) {
    if (places.contents(i)->BoxName() == Block2 &&
        places.contents(i)->State() == 1) {
      Second2[places.contents(i)->Name()] = places.contents(i);
    }
  }
  if (First.length() == 0 || Second.length() == 0 || First2.length() == 0 || Second2.length() == 0) {
    DeleteBox(Name);
    return;
  }
  String PlaceName;
  String temp, n;
  for(Pix i = First.first(); i; First.next(i)) {
    for(Pix j = Second.first(); j; Second.next(j)) {
      temp = First.contents(i)->Name() + "@" + Second.contents(j)->Name();
      NewPlace(temp);
      places[temp]->SetX(int(First.contents(i)->GetX() +
                             Second.contents(j)->GetX()) / 2);
      places[temp]->SetY(int(First.contents(i)->GetY() +
                             Second.contents(j)->GetY()) / 2);
      places[temp]->setBox(Name, ptr);
      places[temp]->SetEntry();
      places[temp]->DeleteAllTokens();
      const Map<String, Edge *> &in = First.contents(i)->OutputEdges();
      for(Pix k = in.first(); k; in.next(k)) {
        n = NewEdge(temp, in.contents(k)->Dest().Name());
      }
      const Map<String, Edge *> &out = Second.contents(j)->OutputEdges();
      for(Pix k = out.first(); k; out.next(k)) {
        n = NewEdge(temp, out.contents(k)->Dest().Name());
      }
    } 
  }
  for(Pix i = First.first(); i; First.next(i)) {
    RemovePlace(First.key(i));
  }
  for(Pix i = Second.first(); i; Second.next(i)) {
    RemovePlace(Second.key(i));
  }
  for(Pix i = First2.first(); i; First2.next(i)) {
    for(Pix j = Second2.first(); j; Second2.next(j)) {
      temp = First2.contents(i)->Name() + "@" + Second2.contents(j)->Name();
      NewPlace(temp);
      places[temp]->SetX(int(First2.contents(i)->GetX() +
                             Second2.contents(j)->GetX()) / 2);
      places[temp]->SetY(int(First2.contents(i)->GetY() +
                             Second2.contents(j)->GetY()) / 2);
      places[temp]->setBox(Name, ptr);
      places[temp]->SetExit();
      const Map<String, Edge *> &in = First2.contents(i)->InputEdges();
      for(Pix k = in.first(); k; in.next(k)) {
        n = NewEdge(in.contents(k)->Src().Name(), temp);
      }
      const Map<String, Edge *> &out = Second2.contents(j)->InputEdges();
      for(Pix k = out.first(); k; out.next(k)) {
        n = NewEdge(out.contents(k)->Src().Name(), temp);
      }
    } 
  }
  for(Pix i = First2.first(); i; First2.next(i)) {
    RemovePlace(First2.key(i));
  }
  for(Pix i = Second2.first(); i; Second2.next(i)) {
    RemovePlace(Second2.key(i));
  }
  for (Pix i = places.first(); i;places.next(i)) {
    if (places.contents(i)->BoxName() == Block1) {
      places.contents(i)->setBox(Name, ptr);
    }
    if (places.contents(i)->BoxName() == Block2) {
      places.contents(i)->setBox(Name, ptr);
    }
  }
  for (Pix i = transitions.first(); i; transitions.next(i)) {
    if (transitions.contents(i)->BoxName() == Block1) {
      transitions.contents(i)->setBox(Name, ptr);
    }
    if (transitions.contents(i)->BoxName() == Block2) {
      transitions.contents(i)->setBox(Name, ptr);
    }
  }
  for (Pix i = ptransitions.first(); i;ptransitions.next(i)) {
    if (ptransitions.contents(i)->BoxName() == Block1) {
      ptransitions.contents(i)->setBox(Name, ptr);
    }
    if (ptransitions.contents(i)->BoxName() == Block2) {
      ptransitions.contents(i)->setBox(Name, ptr);
    }
  }
  DeleteBox(Block1);
  DeleteBox(Block2);
} // Net::Choice

int
SubtractTimevals2(timeval *result, timeval *x, timeval *y)
{
  if(x->tv_usec < y->tv_usec) {
    int nsec = (y->tv_usec - x->tv_usec) / 1000000 + 1;
    y->tv_usec -= 1000000 * nsec;
    y->tv_sec += nsec;
  }
  if(x->tv_usec - y->tv_usec > 1000000) {
    int nsec = (y->tv_usec - x->tv_usec) / 1000000;
    y->tv_usec += 1000000 * nsec;
    y->tv_sec -= nsec;
  }
  result->tv_sec = x->tv_sec - y->tv_sec;
  result->tv_usec = x->tv_usec - y->tv_usec;

  return x->tv_sec < y->tv_sec;
}

void
AddTimevals2(timeval *result, timeval *x, timeval *y)
{
  result->tv_sec = 0;
  result->tv_usec = x->tv_usec + y->tv_usec;
  if(result->tv_usec > 1000000) {
    result->tv_sec = 1;
    result->tv_usec %= 1000000;
  }
  result->tv_sec += x->tv_sec;
  result->tv_sec += y->tv_sec;
}

void
Net::EstablishConnection(void)
  // Verbindungen zu den Schaltungsserver herstellen
{
  char *ptr;
  char *pep  = new char[255];
  char *home = new char[255];

  // Close all existing connections
  while(count--)
  {
    Verbindungen[count]->Write("Close");
    write(Verbindungen[count]->pipe1[1], "Close", 32);
    delete Verbindungen[count];
    Verbindungen[count] = (Socket *) NULL;
  }
  if((ptr = getenv("PEPHOME")) != (char *) NULL) {
    strcpy(pep, ptr);
    strcat(pep, "/");  
  }
  if((ptr = getenv("HOME")) != (char *) NULL) {
    strcpy(home, ptr);
    strcat(home, "/");
  }
  strcat(pep, ".pep_server");
  strcat(home, ".pep_server");
  ifstream server(home);
  if(!server)
    server.open(pep);
  if(server) {
    string name;
    count = 0;
    while(!server.eof()) {
      server >> name;
      if(name == "")
        continue;

      Verbindungen[count] = new Socket(name, 8123);
      int childpid;
      if(!(Verbindungen[count]->Connect())) {
        if(pipe(Verbindungen[count]->pipe1) < 0 || pipe(Verbindungen[count]->pipe2) < 0)
          cerr << "can't create pipe" << endl;

        if( (childpid = fork()) < 0) {
          cerr << "can't fork" << endl;
        } else if(childpid > 0) { // parent
          close(Verbindungen[count]->pipe1[0]);
          close(Verbindungen[count]->pipe2[1]);
        } else { // child-Serverprozess
          close(Verbindungen[count]->pipe1[1]);
          close(Verbindungen[count]->pipe2[0]);
          int n;
          char buff[32];
          char buff2[32];
          char buff3[256];
          char buff4[256];
          String command;
          String Transition;
          for(;;) {
            n = read(Verbindungen[count]->pipe1[0], buff, 32);
            command = buff;
            if(command == "Load") {
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff3, 256);
              string text;
              char c;
              ifstream datei(buff3);
              while(datei.get(c)) {
                text += c;
              }
              datei.close();
              Verbindungen[count]->Write("Load");
              if(Verbindungen[count]->Read() == "OK") {
                Verbindungen[count]->Write(text);
                if(Verbindungen[count]->Read() != "OK") {
                  cerr << "Error during transmission" << endl;
                  exit(0);
                }
              } else {
                exit(0);
              }
            } else if(command == "Close") {
              exit(0);
            } else if(command == "IsActive") {
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff2, 32);
              Verbindungen[count]->Write("IsActive");
              if(Verbindungen[count]->Read() == "OK") {
                Verbindungen[count]->Write(buff2);
                string s = Verbindungen[count]->Read();
                write(Verbindungen[count]->pipe2[1], s.c_str(), 32);
                if(s == "Yes") {
                  n = read(Verbindungen[count]->pipe1[0], buff3, 32);
                  Verbindungen[count]->Write("OK");
                  s = Verbindungen[count]->Read();
                  write(Verbindungen[count]->pipe2[1], s.c_str(), 256);
                }
              } else {
                cerr << "Server failed to answer!!" << endl;
                exit(0);
              }
              //gettimeofday(&end, NULL);
              //SubtractTimevals2(&erg, &end, &start);
              //cerr << buff2 << (double)erg.tv_sec + (double)erg.tv_usec / 1000000 << " sec" << endl;
            } else if(command == "Fire") {
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff2, 32);
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff3, 256);
              Verbindungen[count]->Write("Fire");
              if(Verbindungen[count]->Read() == "OK") {
                Verbindungen[count]->Write(buff2);
                if(Verbindungen[count]->Read() == "OK") {
                  Verbindungen[count]->Write(buff3);
                  if(Verbindungen[count]->Read() != "OK") {
                    cerr << "Error firing " << buff2 << " with binding " << buff3 << endl;
                    exit(0);
                  }
                } else
                  exit(0);
              } else
                exit(0);
            } else if(command == "RenamePlace") {
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff2, 32);
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff3, 32);
              Verbindungen[count]->Write("RenamePlace");
              if(Verbindungen[count]->Read() == "OK") {
                Verbindungen[count]->Write(buff2);
                if(Verbindungen[count]->Read() == "OK") {
                  Verbindungen[count]->Write(buff3);
                  if(Verbindungen[count]->Read() != "OK")
                    cerr << "Error renaming " << buff2 << endl;
                } else
                  exit(0);
              } else
                exit(0);
            } else if(command == "RenameTransition") {
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff2, 32);
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff3, 32);
              Verbindungen[count]->Write("RenameTransition");
              if(Verbindungen[count]->Read() == "OK") {
                Verbindungen[count]->Write(buff2);
                if(Verbindungen[count]->Read() == "OK") {
                  Verbindungen[count]->Write(buff3);
                  if(Verbindungen[count]->Read() != "OK")
                    cerr << "Error renaming " << buff2 << endl;
                } else
                  exit(0);
              } else
                exit(0);
            } else if(command == "ValueTerm") {
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff2, 32);
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff3, 256);
              Verbindungen[count]->Write("ValueTerm");
              if(Verbindungen[count]->Read() == "OK") {
                Verbindungen[count]->Write(buff2);
                if(Verbindungen[count]->Read() == "OK") {
                  Verbindungen[count]->Write(buff3);
                  if(Verbindungen[count]->Read() != "OK")
                    cerr << "Error changing the valueterm of " << buff2 << endl;
                } else
                  exit(0);
              } else
                exit(0);
            } else if(command == "Typ") {
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff2, 32);
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff3, 256);
              Verbindungen[count]->Write("Typ");
              if(Verbindungen[count]->Read() == "OK") {
                Verbindungen[count]->Write(buff2);
                if(Verbindungen[count]->Read() == "OK") {
                  Verbindungen[count]->Write(buff3);
                  if(Verbindungen[count]->Read() != "OK")
                    cerr << "Error changing the typ of " << buff2 << endl;
                } else
                  exit(0);
              } else
                exit(0);
            } else if(command == "Marking") {
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff2, 32);
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff3, 256);
              Verbindungen[count]->Write("Marking");
              if(Verbindungen[count]->Read() == "OK") {
                Verbindungen[count]->Write(buff2);
                if(Verbindungen[count]->Read() == "OK") {
                  Verbindungen[count]->Write(buff3);
                  if(Verbindungen[count]->Read() != "OK") {
                    cerr << "Error changing the marking of " << buff2 << endl;
                    exit(0);
                  }
                } else
                  exit(0);
              } else
                exit(0);
            } else if(command == "ChangeVars") {
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff2, 32);
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff3, 32);
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff4, 256);
              Verbindungen[count]->Write("ChangeVars");
              if(Verbindungen[count]->Read() == "OK") {
                Verbindungen[count]->Write(buff2);
                if(Verbindungen[count]->Read() == "OK") {
                  Verbindungen[count]->Write(buff3);
                  if(Verbindungen[count]->Read() == "OK") {
                    Verbindungen[count]->Write(buff4);
                    if(Verbindungen[count]->Read() != "OK") {
                      cerr << "Error changing the annotation from " << buff2 << " to " << buff3 << endl;
                      exit(0);
                    }
                  } else
                    exit(0);
                } else
                  exit(0);
              } else
                exit(0);
            } else if(command == "NewPlace") {
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff2, 32);
              Verbindungen[count]->Write("NewPlace");
              if(Verbindungen[count]->Read() == "OK") {
                Verbindungen[count]->Write(buff2);
                if(Verbindungen[count]->Read() != "OK") {
                  cerr << "Error creating new place " << buff2 << endl;
                  exit(0);
                }
              } else
                exit(0);
            } else if(command == "NewTransition") {
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff2, 32);
              Verbindungen[count]->Write("NewTransition");
              if(Verbindungen[count]->Read() == "OK") {
                Verbindungen[count]->Write(buff2);
                if(Verbindungen[count]->Read() != "OK") {
                  cerr << "Error creating new transition " << buff2 << endl;
                  exit(0);
                }
              } else
                exit(0);
            } else if(command == "NewEdge") {
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff2, 32);
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff3, 32);
              Verbindungen[count]->Write("NewEdge");
              if(Verbindungen[count]->Read() == "OK") {
                Verbindungen[count]->Write(buff2);
                if(Verbindungen[count]->Read() == "OK") {
                  Verbindungen[count]->Write(buff3);
                  if(Verbindungen[count]->Read() != "OK") {
                    cerr << "Error creating new edge " << buff2 << endl;
                    exit(0);
                  }
                } else
                  exit(0);
              } else
                exit(0);
            } else if(command == "DeletePlace") {
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff2, 32);
              Verbindungen[count]->Write("DeletePlace");
              if(Verbindungen[count]->Read() == "OK") {
                Verbindungen[count]->Write(buff2);
                if(Verbindungen[count]->Read() != "OK") {
                  cerr << "Error deleting place " << buff2 << endl;
                  exit(0);
                }
              } else
                exit(0);
            } else if(command == "DeleteTransition") {
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff2, 32);
              Verbindungen[count]->Write("DeleteTransition");
              if(Verbindungen[count]->Read() == "OK") {
                Verbindungen[count]->Write(buff2);
                if(Verbindungen[count]->Read() != "OK") {
                  cerr << "Error deleting transition " << buff2 << endl;
                  exit(0);
                }
              } else
                exit(0);
            } else if(command == "DeleteEdge") {
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff2, 32);
              write(Verbindungen[count]->pipe2[1], "OK", 32);
              n = read(Verbindungen[count]->pipe1[0], buff3, 32);
              Verbindungen[count]->Write("DeleteEdge");
              if(Verbindungen[count]->Read() == "OK") {
                Verbindungen[count]->Write(buff2);
                if(Verbindungen[count]->Read() == "OK") {
                  Verbindungen[count]->Write(buff3);
                  if(Verbindungen[count]->Read() != "OK") {
                    cerr << "Error deleting edge " << buff2 << endl;
                    exit(0);
                  }
                } else
                  exit(0);
              } else
                exit(0);
            }
          }
        }
        count++;
        stillsave++;
      } else {
        cerr << "Error while connecting to server: " << name << endl;
      }
    }
  }
} // Net::EstablishConnection

void
Net::ChangeBlockName(const String &old_name, const String &new_name)
  // Spaeter: weitere Aktionen im Zusammenhang mit dem Umbenennen
{
#ifdef WARN
  if (blocks.contains(new_name))
    cerr << "Warning: name " << new_name << " exists. Operation refused!\n";
  else {
#endif
    Block *b = blocks[old_name];
    blocks.del(old_name);
    blocks[new_name] = b;
#ifdef WARN
  }  // else, keine Probleme
#endif
}  // Net::ChangeBlockName

void
Net::ChangePlaceName(const String &old_name, const String &new_name)
{
#ifdef WARN
  if (places.contains(new_name))
    cerr << "Warning: name " << new_name << " exists. Operation refused!\n";
  else {
#endif
    String t1 = old_name;
    String t2 = new_name;
    char buff[32];
    for(int n = 0; n < count; n++) {
      write(Verbindungen[n]->pipe1[1], "RenamePlace", 32);
      read(Verbindungen[n]->pipe2[0], buff, 32);
      write(Verbindungen[n]->pipe1[1], t1.data(), 32);
      read(Verbindungen[n]->pipe2[0], buff, 32);
      write(Verbindungen[n]->pipe1[1], t2.data(), 32);
    }
    Place *p = places[old_name];
    places.del(old_name);
    places[new_name] = p;
    const Map<String, Edge *> &inedges = p->InputEdges();
    Pix i;
    for (i = inedges.first(); i; inedges.next(i)) {
      transitions[inedges.key(i)]->ChangeOutName(old_name, new_name);
    }
    const Map<String, Edge *> &outedges = p->OutputEdges();
    for (i = outedges.first(); i; outedges.next(i)) {
      transitions[outedges.key(i)]->ChangeInName(old_name, new_name);
    }
    p->Vertex::ChangeMyName(new_name);

#ifdef WARN
  }  // else, keine Probleme
#endif
}  // Net::ChangePlaceName

void
Net::ChangeTransName(const String &old_name, const String &new_name)
{
#ifdef WARN
  if (transitions.contains(new_name))
    cerr << "Warning: name " << new_name << " exists. Operation refused!\n";
  else {
#endif
    String t1 = old_name;
    String t2 = new_name;
    char buff[32];
    for(int n = 0; n < count; n++) {
      write(Verbindungen[n]->pipe1[1], "RenameTransition", 32);
      read(Verbindungen[n]->pipe2[0], buff, 32);
      write(Verbindungen[n]->pipe1[1], t1.data(), 32);
      read(Verbindungen[n]->pipe2[0], buff, 32);
      write(Verbindungen[n]->pipe1[1], t2.data(), 32);
    }
    Transition *t = transitions[old_name];
    transitions.del(old_name);
    transitions[new_name] = t;
    const Map<String, Edge *> &inedges = t->InputEdges();
    Pix i;
    for (i = inedges.first(); i; inedges.next(i))
      places[inedges.key(i)]->ChangeOutName(old_name, new_name);
    const Map<String, Edge *> &outedges = t->OutputEdges();
    for (i = outedges.first(); i; outedges.next(i))
      places[outedges.key(i)]->ChangeInName(old_name, new_name);
    t->Vertex::ChangeMyName(new_name);
    
#ifdef WARN
  }  // else, alles in Ordnung
#endif
}  // Net::ChangeTransName

void
Net::ChangePTransName(const String &old_name, const String &new_name)
{
#ifdef WARN
  if (ptransitions.contains(new_name))
    cerr << "Warning: name " << new_name << " exists. Operation refused!\n";
  else {
#endif
    PTransition *pt = ptransitions[old_name];
    ptransitions.del(old_name);
    ptransitions[new_name] = pt;
    const Map<String, Edge *> &inpedges = pt->InputPEdges();
    Pix i;
    for (i = inpedges.first(); i; inpedges.next(i))
      places[inpedges.key(i)]->ChangeOutPName(old_name, new_name);
    const Map<String, Edge *> &outpedges = pt->OutputPEdges();
    for (i = outpedges.first(); i; outpedges.next(i))
      places[outpedges.key(i)]->ChangeInPName(old_name, new_name);
#ifdef WARN
  }  // else, alles in Ordnung
#endif
}  // Net::ChangePTransName

int
Net::CreateLow(ClientData cd, Tcl_Interp *ip, int objc, Tcl_Obj *CONST objv[])
{
  char netname[20];

  if (objc != 1) {
    Tcl_WrongNumArgs(ip, 1, objv, NULL);
    return TCL_ERROR;
  }  // if not no argument
  Net *new_net = new Net();
  sprintf(netname, "Net%d", net_counter);
  Tcl_SetStringObj(Tcl_GetObjResult(ip), netname, -1);
  Tcl_CreateCommand(ip, netname, Net::CommandBase, (ClientData) new_net, 
    Net::Destroy);
  return TCL_OK;
}  // Net::CreateLow

void
Net::Destroy(ClientData cd)
{
  delete (Net *) cd;
}  // Net::Destroy

int
Net::CommandBase(ClientData cd, Tcl_Interp *ip, int argc, CONST char *argv[])
{
  return ((Net *) cd)->ObjectCommand(ip, argc, argv);
}  // Net::CommandBase

int 
Net::ObjectCommand(Tcl_Interp *ip, int argc, CONST char *argv[])
{ 
  if (argc < 2) {
    Tcl_AppendResult(ip, "wrong # args: should be \"",
      argv[0], " option ?arg arg ...?\"",
      NULL);
    return TCL_ERROR;
  }  // if falsche Argumentanzahl

    // Neues Objekt im Netz
  if (EQ(1, "create")) {
      // Was fuer ein Objekt?
    switch (argc) {
      case 3:  // Kann Block, Box, Transition, Phantomtransition oder Stelle sein
        if (EQ(2, "Block")) {
          Result(ip, NewBlock());
        }  // if einen neuen Block machen
        else if (EQ(2, "Place")) {
          Result(ip, NewPlace());
        }
        else if (EQ(2, "Transition")) { 
          Result(ip, NewTransition());
        }
        else if (EQ(2, "PTransition")) {
          Result(ip, NewPTransition());
        }
        else if (EQ(2, "Box")) {
          Result(ip, NewBox());
        }
        else {
          Tcl_AppendResult(ip, "wrong # args: only Block, Transition,"
            "PTransition and Place have no args",
            NULL);
          return TCL_ERROR;
        }
        break;    
      case 5:  // Muss Kante sein
        if (EQ(2, "Edge")) {
          Result(ip, NewEdge(argv[3], argv[4]));
	}
        else if (EQ(2, "PEdge")) {
          Result(ip, NewPEdge(argv[3], argv[4]));
	}
        else {
          Tcl_AppendResult(ip, "wrong #arg: only Edges/PEdges have 2 args", 
            NULL);
          return TCL_ERROR;
        }
        break;
      case 6:
        if (EQ(2, "Title")) {
          int x, y;
          if (Tcl_GetInt(ip, argv[4], &x) != TCL_OK)
            return TCL_ERROR;
          if (Tcl_GetInt(ip, argv[5], &y) != TCL_OK)
            return TCL_ERROR;
          titles[++Title::counter] = Title(argv[3], x, y);
          INTRES(Title::counter);
        }
        else {
          Tcl_AppendResult(ip, "wrong #arg: only Title has 3 args", 
            NULL);
          return TCL_ERROR;
        }
        break;
      default:  // Grosse Scheisse
        Tcl_AppendResult(ip, 
          "wrong arg: create Block\n"
          "           create Place\n"
          "           create Transition\n"
          "           create PTransition\n"
          "           create Edge placearg transitionarg\n"
          "           create Edge transitionarg placearg\n"
          "           create PEdge placearg phantomtransitionarg\n"
          "           create PEdge phantomtransitionarg placearg\n"
          "           create Title text x y\n",
          NULL);
        return TCL_ERROR;
    }  // switch
      // Ob wir wohl soweit kommen?
  }  // if create irgendwas

  else if (EQ(1, "clearActive")) {
    if (argc > 2) {
      Tcl_AppendResult(ip, "command clearActive has no args", 
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl fuer clearActive
    activeTransitions.clear();
  }

  else if (EQ(1, "dump")) {
    if (argc > 2) {
      Tcl_AppendResult(ip, "command dump has no args", 
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl fuer dump
    cout << *this;
  }  // if dump das Netz

  else if (EQ(1, "MoveBlock")) {
    if (argc != 5) {
      Tcl_AppendResult(ip, "command MoveBlock has 4 arguments", 
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl fuer dump
    int x, y;
    if (Tcl_GetInt(ip, argv[3], &x) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetInt(ip, argv[4], &y) != TCL_OK)
      return TCL_ERROR;
    VHSet<String> moved;
    moveBlock(argv[2], moved, x, y, 0);
  }  // if Bewege Knoten mit Block

  else if (EQ(1, "MoveBox")) {
    if (argc != 5) {
      Tcl_AppendResult(ip, "command MoveBox has 4 arguments", 
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl fuer dump
    int x, y;
    if (Tcl_GetInt(ip, argv[3], &x) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetInt(ip, argv[4], &y) != TCL_OK)
      return TCL_ERROR;
    VHSet<String> moved;
    moveBox(argv[2], moved, x, y, 0);
  }  // if Bewege Knoten mit Block

  else if (EQ(1, "Parallel")) {
    if (argc != 4) {
      Tcl_AppendResult(ip, "command Parallel has 2 Arguments",
        NULL);
      return TCL_ERROR;
    }
    String Box1 = argv[2];
    String Box2 = argv[3];
    if(!boxes.contains(Box1)) {
      Tcl_AppendResult(ip, "Box ", argv[2], " doesn't exist", NULL);
      return TCL_ERROR;
    }
    if(!boxes.contains(Box2)) {
      Tcl_AppendResult(ip, "Box ", argv[3], " doesn't exist", NULL);
      return TCL_ERROR;
    }
    String name = NewBox();
    Parallel(Box1, Box2, name);
    Tcl_AppendResult(ip, name.data(), NULL);
  }

  else if (EQ(1, "Sequence")) {
    if (argc != 4) {
      Tcl_AppendResult(ip, "command Sequence has 2 Arguments",
        NULL);
      return TCL_ERROR;
    }
    String Box1 = argv[2];
    String Box2 = argv[3];
    if(!boxes.contains(Box1)) {
      Tcl_AppendResult(ip, "Box ", argv[2], " doesn't exist", NULL);
      return TCL_ERROR;
    }
    if(!boxes.contains(Box2)) {
      Tcl_AppendResult(ip, "Box ", argv[3], " doesn't exist", NULL);
      return TCL_ERROR;
    }
    String name = NewBox();
    Sequence(Box1, Box2, name);
    Tcl_AppendResult(ip, name.data(), NULL);
  }

  else if (EQ(1, "Choice")) {
    if (argc != 4) {
      Tcl_AppendResult(ip, "command Choice has 2 Arguments",
        NULL);
      return TCL_ERROR;
    }
    String Box1 = argv[2];
    String Box2 = argv[3];
    if(!boxes.contains(Box1)) {
      Tcl_AppendResult(ip, "Box ", argv[2], " doesn't exist", NULL);
      return TCL_ERROR;
    }
    if(!boxes.contains(Box2)) {
      Tcl_AppendResult(ip, "Box ", argv[3], " doesn't exist", NULL);
      return TCL_ERROR;
    }
    String name = NewBox();
    Choice(Box1, Box2, name);
    Tcl_AppendResult(ip, name.data(), NULL);
  }

  else if (EQ(1, "Substitution")) {
    if (argc != 4) {
      Tcl_AppendResult(ip, "command Substitution has 2 Arguments",
        NULL);
      return TCL_ERROR;
    }
    String Box = argv[2];
    String Transition = argv[3];
    if(!boxes.contains(Box)) {
      Tcl_AppendResult(ip, "Box ", argv[2], " doesn't exist", NULL);
      return TCL_ERROR;
    }
    if(!transitions.contains(Transition)) {
      Tcl_AppendResult(ip, "Transition ", argv[3], " doesn't exist", NULL);
      return TCL_ERROR;
    }
    Substitution(Box, Transition);
  } 

  else if (EQ(1, "ToNets")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command ToNets has argument filename", 
        NULL);
      return TCL_ERROR;
    }
    if(!nets.contains(argv[2]))
      nets[argv[2]] = clipboard;
    //clipboard = (Net *) NULL;
  }

  else if (EQ(1, "InsertNet")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command InsertNet has argument filename", 
        NULL);
      return TCL_ERROR;
    }
    clipboard_temp = clipboard;
    if(nets.contains(argv[2]))
      clipboard = nets[argv[2]];
  }

  else if (EQ(1, "RestoreClipboard")) {
    if (argc != 2) {
      Tcl_AppendResult(ip, "command ToNets has no argument", 
        NULL);
      return TCL_ERROR;
    }
    clipboard = clipboard_temp;
  }

  else if (EQ(1, "DeleteNet")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command DeleteNet has argument name", 
        NULL);
      return TCL_ERROR;
    }
    if(nets.contains(argv[2])) {
      nets.del(argv[2]);
    }
  }

  else if (EQ(1, "save")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command save has argument filename", 
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl fuer save
    ofstream out(argv[2]);
    if (!out) {
      Tcl_AppendResult(ip, "can't open file ", argv[2],
        NULL);
      return TCL_ERROR;
    }  // if Dateifehler
    out << *this;
    if(boxes.length() != 0) {
      String boxname(argv[2]);
      if(boxname.contains(".hl_net"))
        boxname = boxname.before(".hl_net");
      else
        boxname = boxname.before(".ll_net");
      boxname = boxname + ".box";
      ofstream boxes(boxname.data());
      if (boxes) {
        WriteBoxes(boxes);
        boxes.close();
      }
    }
  }  // if save

  else if (EQ(1, "read")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command read has argument filename", 
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl
    ifstream in(argv[2]);
    if (!in) {
      Tcl_AppendResult(ip, "can't open file ", argv[2],
        NULL);
      return TCL_ERROR;
    }
    DeleteAll();
    in >> *this;
    in.close();
    String boxname(argv[2]);
    if(boxname.contains(".hl_net"))
      boxname = boxname.before(".hl_net");
    else
      boxname = boxname.before(".ll_net");
    boxname = boxname + ".box";
    ifstream boxes(boxname.data());
    if (boxes) {
      ReadBoxes(boxes);
      boxes.close();
    }
    // Drop out if no connection exists
    if (!count)
      return TCL_OK;
    EstablishConnection();
    char buff[32];
    for(int n = 0; n < count; n++) {
      write(Verbindungen[n]->pipe1[1], "Load", 32);
      read(Verbindungen[n]->pipe2[0], buff, 32);
      write(Verbindungen[n]->pipe1[1], argv[2], 256);
    }
  }  // if read

  else if (EQ(1, "set")) {
    if (argc < 4) {
      Tcl_AppendResult(ip, "wrong # arg for set",
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl
    if (EQ(2, "ObjectSize")) {
      if (argc != 4) {
        Tcl_AppendResult(ip, "command set ObjectSize has one argument",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl fuer ObjectSize
      if (Tcl_GetDouble(ip, argv[3], &object_size) != TCL_OK)
        return TCL_ERROR;
    }  // if ObjectSize
    else if (EQ(2, "Title")) {
      if (argc == 5) {
        int nr;
        if (Tcl_GetInt(ip, argv[3], &nr) != TCL_OK)
          return TCL_ERROR;
        if (!titles.contains(nr)) {
          Tcl_AppendResult(ip, "unknown Title", NULL);
          return TCL_ERROR;
        }
        titles[nr].font = argv[4];
        return TCL_OK;
      }
      if (argc != 7) {
        Tcl_AppendResult(ip, "command set Title has argument Title x y",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl fuer Title
      int x, y, nr;
      if (Tcl_GetInt(ip, argv[3], &nr) != TCL_OK)
        return TCL_ERROR;
      if (!titles.contains(nr)) {
        Tcl_AppendResult(ip, "unknown Title",
          NULL);
        return TCL_ERROR;
      }  // if Titel unbekannt
      if (Tcl_GetInt(ip, argv[5], &x) != TCL_OK)
        return TCL_ERROR;
      if (Tcl_GetInt(ip, argv[6], &y) != TCL_OK)
        return TCL_ERROR;
      titles[nr] = Title(argv[4], x, y);
    }  // if Title
    else if (EQ(2, "SyncTransSwitch")) {
      int new_state;
      if (Tcl_GetInt(ip, argv[3], &new_state) != TCL_OK)
        return TCL_ERROR;
      sync_trans_switch = new_state;
    }  // if dieser Schalter
    else if (EQ(2, "PhantomTransSwitch")) {
      int new_state;
      if (Tcl_GetInt(ip, argv[3], &new_state) != TCL_OK)
        return TCL_ERROR;
      phantom_trans_switch = new_state;
    }  // if jener Schalter
    else if (EQ(2, "NameP")) {
      if (argc != 4) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      NameP = argv[3];
    }
    else if (EQ(2, "NameT")) {
      if (argc != 4) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      NameT = argv[3];
    }
    else if (EQ(2, "MeaningP")) {
      if (argc != 4) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      MeaningP = argv[3];
    }
    else if (EQ(2, "MeaningT")) {
      if (argc != 4) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      MeaningT = argv[3];
    }
    else if (EQ(2, "Comments")) {
      if (argc != 4) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      Comments = argv[3];
    }
    else if (EQ(2, "AnnotationsA")) {
      if (argc != 4) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      AnnotationsA = argv[3];
    }
    else if (EQ(2, "ActionTermT")) {
      if (argc != 4) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      Tcl_AppendElement(ip, ActionTermT);
    }
    else if (EQ(2, "ValueTermT")) {
      if (argc != 4) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      ValueTermT = argv[3];
    }
    else if (EQ(2, "TypeP")) {
      if (argc != 4) {
        Tcl_AppendResult(ip, "wrong # args", NULL);
        return TCL_ERROR;
      }
      TypeP = argv[3];
    }
    else {  // unbekannt
      Tcl_AppendResult(ip, "unknown option for set",
        NULL);
      return TCL_ERROR;
    }  // else, unbekannt
  }  // if set irgendwas

  else if (EQ(1, "get")) {
    if (argc < 3) {
      Tcl_AppendResult(ip, "wrong # args",
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl fuer get
    if (EQ(2, "Blocks")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl
      for (Pix i = blocks.first(); i; blocks.next(i)) {
        Tcl_AppendElement(ip, blocks.key(i));
      }  // for i ueber alle Bloecke im Netz
    }  // if Bloecke haben wollen
    else if (EQ(2, "Boxes")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl
      for (Pix i = boxes.first(); i; boxes.next(i)) {
        Tcl_AppendElement(ip, boxes.key(i));
      }  // for i ueber alle Boxen im Netz
    }  // if Boxen haben wollen
    else if (EQ(2, "Places")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl fuer Stellen
      for (Pix i = places.first(); i; places.next(i)) {
        Tcl_AppendElement(ip, places.key(i));
      }  // for i ueber alle Stellen im Netz
    }  // if Stellen haben wollen
    else if (EQ(2, "Transitions")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl fuer Transitionen
      for (Pix i = transitions.first(); i; transitions.next(i)) {
        Tcl_AppendElement(ip, transitions.key(i));
      }  // for i ueber alle Transitionen im Netz
    }  // if Transitionen haben wollen
    else if (EQ(2, "PTransitions")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl fuer Phantomtransitionen
      for (Pix i = ptransitions.first(); i; ptransitions.next(i)) {
        Tcl_AppendElement(ip, ptransitions.key(i));
      }  // for i ueber alle Phantomtransitionen im Netz
    }  // if Phantomtransitionen haben wollen
    else if (EQ(2, "RootBlocks")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl fuer RootBlocks
      VHSet<String> rb = RootBlocks();
      for (Pix i = rb.first(); i; rb.next(i)) {
        Tcl_AppendElement(ip, rb(i));
      }  // for i ueber alle Wurzelbloecke
    }  // if Wurzelbloecke haben wollen
    else if (EQ(2, "VisBlocks")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args for get VisBlocks",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl
      for (Pix i = blocks.first(); i; blocks.next(i)) {
        if (blocks.contents(i)->GetBlockVis() == EXPOSED)
          Tcl_AppendElement(ip, blocks.key(i));
      }  // for i ueber alle Bloecke im Netz
    }  // if sichtbare Bloecke haben wollen
    else if (EQ(2, "VisBoxes")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args for get VisBlocks",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl
      for (Pix i = boxes.first(); i; boxes.next(i)) {
        if (boxes.contents(i)->GetVisibility() == 0)
          Tcl_AppendElement(ip, boxes.key(i));
      }  // for i ueber alle Bloecke im Netz
    }  // if sichtbare Bloecke haben wollen
    else if (EQ(2, "VisPlaces")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args for get VisPlaces",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl
      for (Pix i = places.first(); i; places.next(i)) {
        if (places.contents(i)->GetBlockVis() == EXPOSED)
          Tcl_AppendElement(ip, places.key(i));
      }  // for i ueber alle Stellen im Netz
    }  // if sichtbare Stellen haben wollen
    else if (EQ(2, "VisTransitions")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args for get VisTransitions",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl
      for (Pix i = transitions.first(); i; transitions.next(i)) {
        if (transitions.contents(i)->GetBlockVis() == EXPOSED)
          Tcl_AppendElement(ip, transitions.key(i));
      }  // for i ueber alle Transitionen im Netz
    }  // if sichtbare Transitionen haben wollen
    else if (EQ(2, "VisPTransitions")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args for get VisPTransitions",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl
      for (Pix i = ptransitions.first(); i; ptransitions.next(i)) {
        if (ptransitions.contents(i)->GetBlockVis() == EXPOSED)
          Tcl_AppendElement(ip, ptransitions.key(i));
      }  // for i ueber alle Phantomtransitionen im Netz
    }  // if sichtbare Phantomtransitionen haben wollen
    else if (EQ(2, "ActiveTransitions")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl fuer ActiveTransitions
      VHSet<String> ac = ActiveTransitions();
      for (Pix i = ac.first(); i; ac.next(i)) {
        Tcl_AppendElement(ip, ac(i));
      }  // for i ueber alle aktiven Transitionen
    }  // if aktivierte Transitionen haben wollen
    else if (EQ(2, "ActiveTransitionsForBlock")) {
      if (argc != 4) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl fuer ActiveTransitionsForBlock
      if (!blocks.contains(argv[3])) {
        Tcl_AppendResult(ip, "unknown Block",
          NULL);
        return TCL_ERROR;
      }  // if Block ist nicht bekannt
      VHSet<String> ac = ActiveTransitions();
      for (Pix i = ac.first(); i; ac.next(i)) {
        if (transitions[ac(i)]->BlockKeySet().contains(argv[3]))
          Tcl_AppendElement(ip, ac(i));
      }  // for i ueber alle aktiven Transitionen
    }  // if aktivierte Transitionen in dem Block haben wollen
    else if (EQ(2, "ObjectSize")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl fuer ObjectSize
      sprintf(ip->result, "%f", object_size);
    }  // if Objektgroesse haben wollen
    else if (EQ(2, "Titles")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl fuer Titel
      for (Pix i = titles.first(); i; titles.next(i)) {
        Tcl_AppendElement(ip, IntToString(titles.key(i)));
      }  // for i ueber alle Titel des Netzes
    }  // if Titel haben wollen
    else if (EQ(2, "Title")) {
      if (argc != 4) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl fuer einen Titel
      int nr;
      if (Tcl_GetInt(ip, argv[3], &nr) != TCL_OK)
        return TCL_ERROR;
      if (!titles.contains(nr)) {
        Tcl_AppendResult(ip, "unknown Title",
          NULL);
        return TCL_ERROR;
      }  // if Titel unbekannt
      Tcl_AppendElement(ip, titles[nr].s);
      Tcl_AppendElement(ip, IntToString(titles[nr].x));
      Tcl_AppendElement(ip, IntToString(titles[nr].y));
      Tcl_AppendElement(ip, titles[nr].font);
    }  // if einen Titel haben wollen
    else if (EQ(2, "SyncTransSwitch")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl fuer SyncTransSwitch
      INTRES(sync_trans_switch);
    }  // if Status haben wollen
    else if (EQ(2, "PhantomTransSwitch")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }  // if falsche Argumentanzahl fuer PhantomTransSwitch
      INTRES(phantom_trans_switch);
    }  // if Status haben wollen
    else if (EQ(2, "CurrentMarking")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      VHSet<String> ac = GetCurrent();
      for (Pix i = ac.first(); i; ac.next(i)) {
        Tcl_AppendElement(ip, ac(i));
      }  // for i ueber alle aktiven Transitionen
    } // if Aktuelle Markierung haben wollen

    else if (EQ(2, "Nets")) {
      if(argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      for(Pix i = nets.first(); i; nets.next(i)) {
        Tcl_AppendElement(ip, nets.key(i));
      }
    }

    else if (EQ(2, "NameP")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      Tcl_AppendElement(ip, NameP);
    }
  
    else if (EQ(2, "NameT")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      Tcl_AppendElement(ip, NameT);
    }

    else if (EQ(2, "MeaningP")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      Tcl_AppendElement(ip, MeaningP);
    }
      
    else if (EQ(2, "MeaningT")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      Tcl_AppendElement(ip, MeaningT);
    }

    else if (EQ(2, "Comments")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      Tcl_AppendElement(ip, Comments);
    }

    else if (EQ(2, "AnnotationsA")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      Tcl_AppendElement(ip, AnnotationsA);
    }

    else if (EQ(2, "ActionTermT")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      Tcl_AppendElement(ip, ActionTermT);
    }

    else if (EQ(2, "ValueTermT")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      Tcl_AppendElement(ip, ValueTermT);
    }

    else if (EQ(2, "TypeP")) {
      if (argc != 3) {
        Tcl_AppendResult(ip, "wrong # args",
          NULL);
        return TCL_ERROR;
      }
      Tcl_AppendElement(ip, TypeP);
    }

    else {
      Tcl_AppendResult(ip, "unknown option for get",
        NULL);
      return TCL_ERROR;
    }  // else, ich weiss nicht, was ich damit anfangen soll
  }  // if get irgendwas

    // Objekte loeschen
  else if (EQ(1, "delete")) {
    if (argc != 4) {
      Tcl_AppendResult(ip, "command delete has two arguments",
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl fuer delete
    if (EQ(2, "Block")) {
      delete blocks[argv[3]];
      blocks.del(argv[3]);
    }  // if Block loeschen
    else if (EQ(2, "Box")) {
      delete boxes[argv[3]];
      boxes.del(argv[3]);
    }
    else if (EQ(2, "Place")) {
      char buff[32];
      for(int n = 0; n < count; n++) {
        write(Verbindungen[n]->pipe1[1], "DeletePlace", 32);
        read(Verbindungen[n]->pipe2[0], buff, 32);
        write(Verbindungen[n]->pipe1[1], argv[3], 32);
      }
      delete places[argv[3]];
      places.del(argv[3]);
    }  // if Stelle
    else if (EQ(2, "Transition")) {
      char buff[32];
      for(int n = 0; n < count; n++) {
        write(Verbindungen[n]->pipe1[1], "DeleteTransition", 32);
        read(Verbindungen[n]->pipe2[0], buff, 32);
        write(Verbindungen[n]->pipe1[1], argv[3], 32);
      }
      delete transitions[argv[3]];
      transitions.del(argv[3]);
    }  // if Transition
    else if (EQ(2, "PTransition")) {
      delete ptransitions[argv[3]];
      ptransitions.del(argv[3]);
    }  // if Phantomtransition
    else if (EQ(2, "Edge")) {
      char buff[32];
      String E;
      for(int n = 0; n < count; n++) {
        write(Verbindungen[n]->pipe1[1], "DeleteEdge", 32);
        read(Verbindungen[n]->pipe2[0], buff, 32);
        E = edges[argv[3]]->Src().Name();
        write(Verbindungen[n]->pipe1[1], E.data(), 32);
        read(Verbindungen[n]->pipe2[0], buff, 32);
        E = edges[argv[3]]->Dest().Name();
        write(Verbindungen[n]->pipe1[1], E.data(), 32);
      }
      delete edges[argv[3]];
      edges.del(argv[3]);
    }  // if Kante
    else if (EQ(2, "PEdge")) {
      delete pedges[argv[3]];
      pedges.del(argv[3]);
    }  // if Kante
    else if (EQ(2, "Title"))  {
      int nr;
      if (Tcl_GetInt(ip, argv[3], &nr) != TCL_OK)
        return TCL_ERROR;
      titles.del(nr);
    }  // if Titel
    else {
      Tcl_AppendResult(ip, "hhat do you wand to delete?",
        NULL);
      return TCL_ERROR;
    }  // else, keine Ahnung, was geloescht werden soll
  }  // if delete irgenwas

  else if (EQ(1, "FireTransition")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command FireTransition has one argument",
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl fuer FireTransition
    if (!transitions.contains(argv[2])) {
      Tcl_AppendResult(ip, "Transition", argv[2], " does not exist",
        NULL);
      return TCL_ERROR;
    }  // if Transition unbekannt
    FireTransition(argv[2]);
  }  // if FireTransition

  else if (EQ(1, "RecalcRootBlocks")) {
    if (argc != 2) {
      Tcl_AppendResult(ip, "command RecalcRootBlocks has no arguments",
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl fuer RecalcRootBlocks
    RecalcRootBlocks();
  }  // if RecalcRootBlocks

  else if (EQ(1, "RecalcBlockVis")) {
    if (argc != 2) {
      Tcl_AppendResult(ip, "command RecalcBlockVis has no arguments",
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl fuer RecalcBlockVis
    RecalcBlockVis();
  }  // if RecalcBlockVis

  else if (EQ(1, "DeleteAll")) {
    if (argc != 2) {
      Tcl_AppendResult(ip, "command DeleteAll has no argument",
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl fuer DeleteAll
    DeleteAll();
  }  // if DeleteAll

  else if (EQ(1, "WithTime")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command WithTime has one argument", NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl f"ur WithTime
    with_time = (*argv[2] == '1');
  }
  
  else if (EQ(1, "Copy")) {
    if (argc > 2) {
      SEL = "";
      String para = argv[2];
      int zaehler = 0;
      while(para != "") {
        para = para.after(" ");
        zaehler++;
      }
      String kette[zaehler];
      para = argv[2];
      for(int i = 0;i < zaehler;i++) {
        kette[i] = (i == zaehler-1) ? para : para.before(" ");
        para = para.after(" ");
      }
      copy(zaehler, kette);
    }
    else {
      Tcl_AppendResult(ip, "command Copy needs some nodes",
        NULL);
      return TCL_ERROR;
    }
  }

  else if (EQ(1, "TakeInBox")) {
    if (argc > 2) {
      SEL = "";
      String para = argv[2];
      int zaehler = 0;
      while(para != "") {
        para = para.after(" ");
        zaehler++;
      }
      String kette[zaehler];
      para = argv[2];
      for(int i = 0;i < zaehler;i++) {
        kette[i] = (i == zaehler-1) ? para : para.before(" ");
        para = para.after(" ");
      }
      Tcl_AppendResult(ip, TakeInBox(zaehler, kette).data(), NULL);
    }
    else {
      Tcl_AppendResult(ip, "command TakeInBox needs some nodes",
        NULL);
      return TCL_ERROR;
    }
  }

  else if (EQ(1, "AddToBox")) {
    if (argc > 3) {
      SEL = "";
      String para = argv[2];
      String box = argv[3];
      int zaehler = 0;
      while(para != "") {
        para = para.after(" ");
        zaehler++;
      }
      String kette[zaehler];
      para = argv[2];
      for(int i = 0;i < zaehler;i++) {
        kette[i] = (i == zaehler-1) ? para : para.before(" ");
        para = para.after(" ");
      }
      AddToBox(zaehler, kette, box);
    }
    else {
      return TCL_ERROR;
    }
  }

  else if (EQ(1, "Cut")) {
   if (argc > 2) {
      SEL = "";
      String para = argv[2];
      int zaehler = 0;
      while(para != "") {
        para = para.after(" ");
        zaehler++;
      }
      String kette[zaehler];
      para = argv[2];
      for(int i = 0;i < zaehler;i++) {
        kette[i] = (i == zaehler-1) ? para : para.before(" ");
        para = para.after(" ");
      }
      cut(zaehler, kette);
    }
    else {
      Tcl_AppendResult(ip, "command Cut needs some nodes",
        NULL);
      return TCL_ERROR;
    }
  }

  else if (EQ(1, "Paste")) {
    if (argc != 2) {
      Tcl_AppendResult(ip, "command Paste has no argument",
        NULL);
      return TCL_ERROR;
    }
    paste();
  }

  else if (EQ(1, "AppendSuffix")) {
    if (argc > 3) {
      String Suffix = argv[2];
      String Box = argv[3];
      AppSuffix(Suffix, Box);
    }
    else {
      Tcl_AppendResult(ip, "command AppendSuffix needs a Suffix and a Box",
        NULL);
      return TCL_ERROR;
    }
  }

  else if (EQ(1, "Selected")) {
    const char* nodes = SEL;
    Tcl_AppendResult(ip, nodes, NULL);
    SEL = "";
  }

  else if (EQ(1, "LoadModul")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command LoadModul has one argument",
        NULL);
      return TCL_ERROR;
    }
    loadmodul(argv[2]);
  }

  else if (EQ(1, "SaveModul")) {
    if (argc != 3) {
      Tcl_AppendResult(ip, "command SaveModul has one argument",
        NULL);
      return TCL_ERROR;
    }
    savemodul(argv[2]);
  }

  else if (EQ(1, "Establish")) {
    if (argc != 2) {
      Tcl_AppendResult(ip, "command Establish has no arguments",
        NULL);
    }
    EstablishConnection();
  }

  else if (EQ(1, "Block")) {
    if (!blocks.contains(argv[2])) {
      Tcl_AppendResult(ip, "Block ", argv[2], " does not exist",
        NULL);
      return TCL_ERROR;
    }  // if Block unbekannt
    return blocks[argv[2]]->ObjectCommand(ip, argc-3, (const char**)&argv[3]);
  }  // if Block befragen

  else if (EQ(1, "Box")) {
    if (!boxes.contains(argv[2])) {
      Tcl_AppendResult(ip, "Box ", argv[2], " does not exist",
        NULL);
      return TCL_ERROR;
    }  // if Block unbekannt
    return boxes[argv[2]]->ObjectCommand(ip, argc-3, (const char**)&argv[3]);
  }  // if Block befragen

  else if (EQ(1, "Place")) {
    if (!places.contains(argv[2])) {
      Tcl_AppendResult(ip, "Place ", argv[2], " does not exist",
        NULL);
      return TCL_ERROR;
    }  // if Stelle unbekannt
    return places[argv[2]]->ObjectCommand(ip, argc-3, (const char**)&argv[3]);
  }  // if Stelle befragen

  else if (EQ(1, "Transition")) {
    if (!transitions.contains(argv[2])) {
      Tcl_AppendResult(ip, "Transition ", argv[2], " does not exist",
        NULL);
      return TCL_ERROR;
    }  // if Transition unbekannt
    return transitions[argv[2]]->ObjectCommand(ip, argc-3, (const char**)&argv[3]);
  }  // if Transition befragen

  else if (EQ(1, "PTransition")) {
    if (!ptransitions.contains(argv[2])) {
      Tcl_AppendResult(ip, "PTransition ", argv[2], " does not exist",
        NULL);
      return TCL_ERROR;
    }  // if Phantomtransition unbekannt
    return ptransitions[argv[2]]->ObjectCommand(ip, argc-3, (const char**)&argv[3]);
  }  // if Phantomtransition befragen

  else if (EQ(1, "Edge")) {
    if (!edges.contains(argv[2])) {
      Tcl_AppendResult(ip, "Edge ", argv[2], " does not exist",
        NULL);
      return TCL_ERROR;
    }  // if Kante unbekannt
    return edges[argv[2]]->ObjectCommand(ip, argc-3, (const char**)&argv[3]);
  }  // if Kante befragen

  else if (EQ(1, "PEdge")) {
    if (!pedges.contains(argv[2])) {
      Tcl_AppendResult(ip, "PEdge ", argv[2], " does not exist",
        NULL);
      return TCL_ERROR;
    }  // if Phantomkante unbekannt
    return pedges[argv[2]]->ObjectCommand(ip, argc-3, (const char**)&argv[3]);
  }  // if Phantomkante befragen

  else if(EQ(1, "CalcOffset")) {
    if ((argc != 8) && (argc != 10)) {
      Tcl_AppendResult(ip, "wrong #arg: CalcOffset Type1 Type2 x1 y1 x2 y2",
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl fuer CalcOffset
    int x1, x2, y1, y2, xm, ym;
    if (Tcl_GetInt(ip, argv[4], &x1) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetInt(ip, argv[5], &y1) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetInt(ip, argv[6], &x2) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetInt(ip, argv[7], &y2) != TCL_OK)
      return TCL_ERROR;
    if(argc == 10) {
      if (Tcl_GetInt(ip, argv[8], &xm) != TCL_OK)
        return TCL_ERROR;
      if (Tcl_GetInt(ip, argv[9], &ym) != TCL_OK)
        return TCL_ERROR;
      return CalcOffset(ip, argv[2], argv[3], x1, y1, x2, y2, xm, ym);
    }
    return CalcOffset(ip, argv[2], argv[3], x1, y1, x2, y2);
  }  // if CalcOffset

  else if(EQ(1, "TranslateNorm")) {
    if (argc != 7) {
      Tcl_AppendResult(ip, "wrong #arg: TranslateNorm x1 y1 x2 y2 up|down",
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl fuer TranslateNorm
    int x1, x2, y1, y2;
    if (Tcl_GetInt(ip, argv[3], &x1) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetInt(ip, argv[4], &y1) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetInt(ip, argv[5], &x2) != TCL_OK)
      return TCL_ERROR;
    if (Tcl_GetInt(ip, argv[6], &y2) != TCL_OK)
      return TCL_ERROR;
    return TranslateNorm(ip, x1, y1, x2, y2, EQ(7, "up"));
  }  // if TranslateNorm

  else if (EQ(1, "Exists")) {
    if (argc != 4) {
      Tcl_AppendResult(ip,
        "wrong #arg: Exists Block|Place|Transition|PTransition id",
        NULL);
      return TCL_ERROR;
    }  // if falsche Argumentanzahl fuer Exists
    if (EQ(2, "Block"))
      INTRES(blocks.contains(argv[3]));
    else if (EQ(2, "Box"))
      INTRES(boxes.contains(argv[3]));
    else if (EQ(2, "Place")) 
      INTRES(places.contains(argv[3]));
    else if (EQ(2, "Transition")) 
      INTRES(transitions.contains(argv[3]));
    else if (EQ(2, "PTransition")) 
      INTRES(ptransitions.contains(argv[3]));
    else {
      Tcl_AppendResult(ip,
        "wrong arg: Exists Block|Place|Transition|PTransition",
        NULL);
      return TCL_ERROR;
    }  // else, Typ unbekannt
  }  // if Exists irgendwas

  else {
    Tcl_AppendResult(ip, 
      "wrong arg: create arg ?arg ...?\n"
      "           dump\n"
      "           save filename\n"
      "           read filename\n"
      "           delete\n"
      "           FireTransition\n"
      "           RecalcRootBlocks\n"
      "           CalcOffset\n"
      "           TranslateNorm\n"
      "           get Blocks | Places | Transitions | PTransitions\n"
      "               | RootBlocks | VisBlocks | VisPlaces\n"
      "               | VisTransitions | VisPTransitions\n"
      "               | SyncTransSwitch | PhantomTransSwitch\n"
      "           set ObjectSize | Title | SyncTransSwitch\n"
      "               | PhantomTransSwitch",
      NULL);
    return TCL_ERROR;
  }  // else, keine Ahnung, was denn nu

  return TCL_OK;
}  // Net::ObjectCommand

int
Net::CalcOffset(Tcl_Interp *ip, const String &t1, const String &t2,
  int x1, int y1, int x2, int y2) const
  // Berechnung von Kantenkoordinaten, dabei wird der Typ der beiden
  // Knoten zur Bestimmung des Abstandes vom Knotenmittelpunkt beruecksichtigt.
  // Uebergeben wird jeweils die linke, obere Ecke des Knotens.
  // Rueckgabe sind der Start- und der Endpunkt der Kante.
{
  int xoff_1 = 0, yoff_1 = 0, xoff_2, yoff_2;
  double r, cos_a, sin_a, sin_b, cos_b, m;
  int o1_2, o2_2;

    // verschiedene Objektmasse beruecksichtigen
  if (t1 == "Block")
    o1_2 = int(object_size);
  else
    o1_2 = int(object_size / 2);
  if (t2 == "Block")
    o2_2 = int(object_size);
  else
    o2_2 = int(object_size / 2);
    // Mittelpunkt des Objektes bestimmen
  x1 += o1_2;
  y1 += o1_2;
  x2 += o2_2;
  y2 += o2_2;
  if (t1 == "Block")
    y1 -= 6;
  if (t2 == "Block")
    y2 -= 6;

  if (x1 == x2) {  // eine Vertikale
    yoff_1 = (y2>y1) ? o1_2 : -o1_2;
    yoff_2 = (y2>y1) ? -o2_2 : o2_2;
    xoff_1 = xoff_2 = 0;
  }  // if x1 gleich x2
  else if (y1 == y2) {  // eine Horizontale
    xoff_1 = x2>x1 ? o1_2 : -o1_2;
    xoff_2 = x2>x1 ? -o2_2 : o2_2;
    yoff_1 = yoff_2 = 0;
  }  // if y1 gleich y2
  else {  // eine Schraege
    r = sqrt((double)(x2-x1) * (x2-x1) + (y2-y1) * (y2-y1));
    cos_a = (x2-x1) / r;
    sin_a = (y2-y1) / r;
    cos_b = (x1-x2) / r;
    sin_b = (y1-y2) / r;
    xoff_1 = int(o1_2 * cos_a);
    yoff_1 = int(o1_2 * sin_a);
    xoff_2 = int(o2_2 * cos_b);
    yoff_2 = int(o2_2 * sin_b);
    m = double(y2-y1) / double(x2-x1);  // Steigung der Geraden
    if ((t1 == "Transition") || (t1 == "PTransition") || (t1 == "Block")) {
      if (fabs(m) <= 1) 
        xoff_1 = (xoff_1 > 0) ? o1_2 : -o1_2;
      else
        yoff_1 = (yoff_1 > 0) ? o1_2 : -o1_2;
    }  // if eine Transition
    else {
      if (fabs(m) <= 1) 
        xoff_2 = (xoff_2 > 0) ? o2_2 : -o2_2;
      else
        yoff_2 = (yoff_2 > 0) ? o2_2 : -o2_2;
    }  // else, keine Transition
  }  // else, keine Horizontale oder Vertikale

    // Aufbereitung fuer TCL
  if(t1 != "Mitte") {
    Tcl_AppendElement(ip, (char *) IntToString(xoff_1 + x1));
    Tcl_AppendElement(ip, (char *) IntToString(yoff_1 + y1));
  }
  if(t2 != "Mitte") {
    Tcl_AppendElement(ip, (char *) IntToString(xoff_2 + x2));
    Tcl_AppendElement(ip, (char *) IntToString(yoff_2 + y2));
  }
  return TCL_OK;
}  // Net::CalcOffset

int
Net::CalcOffset(Tcl_Interp *ip, const String &t1, const String &t2,
  int x1, int y1, int xm, int ym, int x2, int y2)
  // Berechnung von Kantenkoordinaten, dabei wird der Typ der beiden
  // Knoten zur Bestimmung des Abstandes vom Knotenmittelpunkt beruecksichtigt.
  // Uebergeben wird jeweils die linke, obere Ecke des Knotens.
  // Rueckgabe sind der Start- und der Endpunkt der Kante.
  // Bei dieser modifizierten Funktion wird der Splinemittelpunkt "ubergeben
  // und mit dessen Hilfe bessere Koordinaten bestimmt
{
  int xh1 = x1-xm;
  int yh1 = y1-ym;
  int xh2 = xm-x2;
  int yh2 = ym-y2;
  int euklid = int((xh1*xh1 + yh1*yh1)+(xh2*xh2 + yh2*yh2));

  if(euklid < 3*int(object_size)*int(object_size)) {
    CalcOffset(ip,t1,t2,x1,y1,x2,y2);
  }
  else {
    CalcOffset(ip,t1,"Mitte",x1,y1,xm-int(object_size/2),ym-int(object_size/2));
    CalcOffset(ip,"Mitte",t2,xm-int(object_size/2),ym-int(object_size/2),x2,y2);  }
  return TCL_OK;
}  // Net::CalcOffset


int
Net::TranslateNorm(Tcl_Interp *ip, int x1, int y1, int x2, int y2, 
  int up_or_down) const
{
  double nx, ny, sx, sy, r;

  GetNormal(x1, y1, x2, y1, nx, ny, sx, sy, r);

  int xa, ya, xb, yb;
  double o4 = object_size / 4.0;

  if (up_or_down>0) {
    xa = int(x1 - sx*o4);
    ya = int(y1 - sy*o4);
    xb = int(x2 - sx*o4);
    yb = int(y2 - sy*o4);
  }  // if up or down
  else {
    xa = int(x1 + sx*o4);
    ya = int(y1 + sy*o4);
    xb = int(x2 + sx*o4);
    yb = int(y2 + sy*o4);
  }  // else, nicht up or down
  Tcl_AppendElement(ip, (char *) IntToString(xa));
  Tcl_AppendElement(ip, (char *) IntToString(ya));
  Tcl_AppendElement(ip, (char *) IntToString(xb));
  Tcl_AppendElement(ip, (char *) IntToString(yb));
  return TCL_OK;
}  // Net::TranslateNorm

/*void Delete(PTransition *ptr, Pix i, String name) {
  ptr->synctrans.contents(i)->phantomtrans.del(name);
}*/

  //*** Dateiende Net.cc ***

